/*
	meSpeak  v.1.9.7.1
	based on speak.js, https://github.com/kripken/speak.js
	eSpeak and other code here are under the GNU GPL.
	meSpeak (Modular eSpeak) is a mod of 'speak.js' by N.Landsteiner (2011-2015), www.masswerk.at
	adding support for Webkit/Safari and external voice-modules
	v.1.1 and later default to play via the Web Audio API and uses the HTMLAudioElement as a second option.
	official project page: http://www.masswerk.at/mespeak
*/

var meSpeak = (function() {

"use strict";

function ESpeak() { // emscripten implementation

var vprint = (self.console)? console.log:function() {};

var Module = {
	noInitialRun: true
};

// WARNING: .ll contains i64 or double values. These 64-bit values are dangerous in USE_TYPED_ARRAYS == 2.
//          We store i64 as i32, and double as float. This can cause serious problems!
"use strict";

/*
// Capture the output of this into a variable, if you want
(function(Module, args) {
  Module = Module || {};
  Module.arguments = args || [];
*/

///*
// Runs much faster, for some reason
if (!Module.arguments) {
  try {
    Module.arguments = arguments;
  } catch(e) {
    Module.arguments = [];
  }
}
//*/

  
// === Auto-generated preamble library stuff ===

//========================================
// Runtime code shared with compiler
//========================================

var Runtime = {
  stackSave: function () {
    return STACKTOP;
  },
  stackRestore: function (stackTop) {
    STACKTOP = stackTop;
  },
  forceAlign: function (target, quantum) {
    quantum = quantum || 4;
    if (isNumber(target) && isNumber(quantum)) {
      return Math.ceil(target/quantum)*quantum;
    } else {
      return 'Math.ceil((' + target + ')/' + quantum + ')*' + quantum;
    }
  },
  isNumberType: function (type) {
    return type in Runtime.INT_TYPES || type in Runtime.FLOAT_TYPES;
  },
  isPointerType: function isPointerType(type) {
  return pointingLevels(type) > 0;
},
  isStructType: function isStructType(type) {
  if (isPointerType(type)) return false;
  if (new RegExp(/^\[\d+\ x\ (.*)\]/g).test(type)) return true; // [15 x ?] blocks. Like structs
  if (new RegExp(/<?{ [^}]* }>?/g).test(type)) return true; // { i32, i8 } etc. - anonymous struct types
  // See comment in isStructPointerType()
  return !Runtime.isNumberType(type) && type[0] == '%';
},
  INT_TYPES: {"i1":0,"i8":0,"i16":0,"i32":0,"i64":0},
  FLOAT_TYPES: {"float":0,"double":0},
  or64: function (x, y) {
    var l = (x | 0) | (y | 0);
    var h = (Math.round(x / 4294967296) | Math.round(y / 4294967296)) * 4294967296;
    return l + h;
  },
  and64: function (x, y) {
    var l = (x | 0) & (y | 0);
    var h = (Math.round(x / 4294967296) & Math.round(y / 4294967296)) * 4294967296;
    return l + h;
  },
  xor64: function (x, y) {
    var l = (x | 0) ^ (y | 0);
    var h = (Math.round(x / 4294967296) ^ Math.round(y / 4294967296)) * 4294967296;
    return l + h;
  },
  getNativeFieldSize: function getNativeFieldSize(type) {
  return Math.max(Runtime.getNativeTypeSize(type), 4);
},
  getNativeTypeSize: function getNativeTypeSize(type) {
  if (4 == 1) return 1;
  var size = {
    '_i1': 1,
    '_i8': 1,
    '_i16': 2,
    '_i32': 4,
    '_i64': 8,
    "_float": 4,
    "_double": 8
  }['_'+type]; // add '_' since float&double confuse Closure compiler as keys.
  if (!size && type[type.length-1] == '*') {
    size = 4; // A pointer
  }
  return size;
},
  dedup: function dedup(items, ident) {
  var seen = {};
  if (ident) {
    return items.filter(function(item) {
      if (seen[item[ident]]) return false;
      seen[item[ident]] = true;
      return true;
    });
  } else {
    return items.filter(function(item) {
      if (seen[item]) return false;
      seen[item] = true;
      return true;
    });
  }
},
  set: function set() {
  var args = typeof arguments[0] === 'object' ? arguments[0] : arguments;
  var ret = {};
  for (var i = 0; i < args.length; i++) {
    ret[args[i]] = 0;
  }
  return ret;
},
  calculateStructAlignment: function calculateStructAlignment(type) {
    type.flatSize = 0;
    type.alignSize = 0;
    var diffs = [];
    var prev = -1;
    type.flatIndexes = type.fields.map(function(field) {
      var size, alignSize;
      if (Runtime.isNumberType(field) || Runtime.isPointerType(field)) {
        size = Runtime.getNativeTypeSize(field); // pack char; char; in structs, also char[X]s.
        alignSize = size;
      } else if (Runtime.isStructType(field)) {
        size = Types.types[field].flatSize;
        alignSize = Types.types[field].alignSize;
      } else {
        dprint('Unclear type in struct: ' + field + ', in ' + type.name_ + ' :: ' + dump(Types.types[type.name_]));
        assert(0);
      }
      alignSize = type.packed ? 1 : Math.min(alignSize, 4);
      type.alignSize = Math.max(type.alignSize, alignSize);
      var curr = Runtime.alignMemory(type.flatSize, alignSize); // if necessary, place this on aligned memory
      type.flatSize = curr + size;
      if (prev >= 0) {
        diffs.push(curr-prev);
      }
      prev = curr;
      return curr;
    });
    type.flatSize = Runtime.alignMemory(type.flatSize, type.alignSize);
    if (diffs.length == 0) {
      type.flatFactor = type.flatSize;
    } else if (Runtime.dedup(diffs).length == 1) {
      type.flatFactor = diffs[0];
    }
    type.needsFlattening = (type.flatFactor != 1);
    return type.flatIndexes;
  },
  generateStructInfo: function (struct, typeName, offset) {
    var type, alignment;
    if (typeName) {
      offset = offset || 0;
      type = (typeof Types === 'undefined' ? Runtime.typeInfo : Types.types)[typeName];
      if (!type) return null;
      if (!struct) struct = (typeof Types === 'undefined' ? Runtime : Types).structMetadata[typeName.replace(/.*\./, '')];
      if (!struct) return null;
      assert(type.fields.length === struct.length, 'Number of named fields must match the type for ' + typeName + '. Perhaps due to inheritance, which is not supported yet?');
      alignment = type.flatIndexes;
    } else {
      var type = { fields: struct.map(function(item) { return item[0] }) };
      alignment = Runtime.calculateStructAlignment(type);
    }
    var ret = {
      __size__: type.flatSize
    };
    if (typeName) {
      struct.forEach(function(item, i) {
        if (typeof item === 'string') {
          ret[item] = alignment[i] + offset;
        } else {
          // embedded struct
          var key;
          for (var k in item) key = k;
          ret[key] = Runtime.generateStructInfo(item[key], type.fields[i], alignment[i]);
        }
      });
    } else {
      struct.forEach(function(item, i) {
        ret[item[1]] = alignment[i];
      });
    }
    return ret;
  },
  stackAlloc: function stackAlloc(size) { var ret = STACKTOP; _memset(STACKTOP, 0, size); STACKTOP += size;STACKTOP = Math.ceil((STACKTOP)/4)*4;; return ret; },
  staticAlloc: function staticAlloc(size) { var ret = STATICTOP; STATICTOP += size;STATICTOP = Math.ceil((STATICTOP)/4)*4;; return ret; },
  alignMemory: function alignMemory(size,quantum) { var ret = size = Math.ceil((size)/(quantum ? quantum : 4))*(quantum ? quantum : 4);; return ret; },
  __dummy__: 0
}



var CorrectionsMonitor = {
  MAX_ALLOWED: 0, // XXX
  corrections: 0,
  sigs: {},

  note: function(type, succeed, sig) {
    if (!succeed) {
      this.corrections++;
      if (this.corrections >= this.MAX_ALLOWED) abort('\n\nToo many corrections!');
    }
  },

  vprint: function() {
    var items = [];
    for (var sig in this.sigs) {
      items.push({
        sig: sig,
        fails: this.sigs[sig][0],
        succeeds: this.sigs[sig][1],
        total: this.sigs[sig][0] + this.sigs[sig][1]
      });
    }
    items.sort(function(x, y) { return y.total - x.total; });
    for (var i = 0; i < items.length; i++) {
      var item = items[i];
      vprint(item.sig + ' : ' + item.total + ' hits, %' + (Math.ceil(100*item.fails/item.total)) + ' failures');
    }
  }
};

function cRound(x) {
  return x >= 0 ? Math.floor(x) : Math.ceil(x);
}


//========================================
// Runtime essentials
//========================================

var __globalConstructor__ = function globalConstructor() {
};

var __THREW__ = false; // Used in checking for thrown exceptions.

var __ATEXIT__ = [];

var ABORT = false;

var undef = 0;
var tempValue, tempInt, tempBigInt;

function abort(text) {
  vprint(text + ':\n' + (new Error).stack);
  ABORT = true;
  throw "Assertion: " + text;
}

function assert(condition, text) {
  if (!condition) {
    abort('Assertion failed: ' + text);
  }
}

// Sets a value in memory in a dynamic way at run-time. Uses the
// type data. This is the same as makeSetValue, except that
// makeSetValue is done at compile-time and generates the needed
// code then, whereas this function picks the right code at
// run-time.

function setValue(ptr, value, type) {
  if (type[type.length-1] === '*') type = 'i32'; // pointers are 32-bit
  switch(type) {
    case 'i1': HEAP8[(ptr)]=value;; break;
    case 'i8': HEAP8[(ptr)]=value;; break;
    case 'i16': HEAP16[((ptr)>>1)]=value;; break;
    case 'i32': HEAP32[((ptr)>>2)]=value;; break;
    case 'i64': HEAP32[((ptr)>>2)]=value;; break;
    case 'float': HEAPF32[((ptr)>>2)]=value;; break;
    case 'double': HEAPF32[((ptr)>>2)]=value;; break;
    default: abort('invalid type for setValue: ' + type);
  }
}
Module.setValue = setValue;

// Parallel to setValue.

function getValue(ptr, type) {
  if (type[type.length-1] === '*') type = 'i32'; // pointers are 32-bit
  switch(type) {
    case 'i1': return HEAP8[(ptr)];
    case 'i8': return HEAP8[(ptr)];
    case 'i16': return HEAP16[((ptr)>>1)];
    case 'i32': return HEAP32[((ptr)>>2)];
    case 'i64': return HEAP32[((ptr)>>2)];
    case 'float': return HEAPF32[((ptr)>>2)];
    case 'double': return HEAPF32[((ptr)>>2)];
    default: abort('invalid type for setValue: ' + type);
  }
  return null;
}
Module.getValue = getValue;

// Allocates memory for some data and initializes it properly.

var ALLOC_NORMAL = 0; // Tries to use _malloc()
var ALLOC_STACK = 1; // Lives for the duration of the current function call
var ALLOC_STATIC = 2; // Cannot be freed
Module.ALLOC_NORMAL = ALLOC_NORMAL;
Module.ALLOC_STACK = ALLOC_STACK;
Module.ALLOC_STATIC = ALLOC_STATIC;

function allocate(slab, types, allocator) {
  var zeroinit, size;
  if (typeof slab === 'number') {
    zeroinit = true;
    size = slab;
  } else {
    zeroinit = false;
    size = slab.length;
  }

  var ret = [_malloc, Runtime.stackAlloc, Runtime.staticAlloc][typeof allocator === 'undefined' ? ALLOC_STATIC : allocator](Math.max(size, 1));

  var singleType = typeof types === 'string' ? types : null;

  var i = 0, type;
  while (i < size) {
    var curr = zeroinit ? 0 : slab[i];

    if (typeof curr === 'function') {
      curr = Runtime.getFunctionIndex(curr);
    }

    type = singleType || types[i];
    if (type === 0) {
      i++;
      continue;
    }

    setValue(ret+i, curr, type);
    i += Runtime.getNativeTypeSize(type);
  }

  return ret;
}
Module.allocate = allocate;

function Pointer_stringify(ptr) {
  var ret = "";
  var i = 0;
  var t;
  var nullByte = String.fromCharCode(0);
  while (1) {
    t = String.fromCharCode(HEAPU8[(ptr+i)]);
    if (t == nullByte) { break; } else {}
    ret += t;
    i += 1;
  }
  return ret;
}
Module.Pointer_stringify = Pointer_stringify;

function Array_stringify(array) {
  var ret = "";
  for (var i = 0; i < array.length; i++) {
    ret += String.fromCharCode(array[i]);
  }
  return ret;
}
Module.Array_stringify = Array_stringify;

// Memory management

var FUNCTION_TABLE; // XXX: In theory the indexes here can be equal to pointers to stacked or malloced memory. Such comparisons should
                    //      be false, but can turn out true. We should probably set the top bit to prevent such issues.

var PAGE_SIZE = 4096;
function alignMemoryPage(x) {
  return Math.ceil(x/PAGE_SIZE)*PAGE_SIZE;
}

var HEAP;
var HEAP8, HEAPU8, HEAP16, HEAPU16, HEAP32, HEAPU32, HEAPF32;

var STACK_ROOT, STACKTOP, STACK_MAX;
var STATICTOP;

var HAS_TYPED_ARRAYS = false;
var TOTAL_MEMORY = Module.TOTAL_MEMORY || 52428800;
var FAST_MEMORY = Module.FAST_MEMORY || 2097152;

// Initialize the runtime's memory
HAS_TYPED_ARRAYS = false;
try {
  HAS_TYPED_ARRAYS = !!Int32Array && !!Float32Array && !!(new Int32Array(1).subarray); // check for full engine support (use string 'subarray' to avoid closure compiler confusion)
} catch(e) {}

if (HAS_TYPED_ARRAYS) {
  var buffer = new ArrayBuffer(TOTAL_MEMORY);
  HEAP8 = new Int8Array(buffer);
  HEAP16 = new Int16Array(buffer);
  HEAP32 = new Int32Array(buffer);
  HEAPU8 = new Uint8Array(buffer);
  HEAPU16 = new Uint16Array(buffer);
  HEAPU32 = new Uint32Array(buffer);
  HEAPF32 = new Float32Array(buffer);

  // Endianness check (note: assumes compiler arch was little-endian)
  HEAP32[0] = 255;
  assert(HEAPU8[0] === 255 && HEAPU8[3] === 0, 'Typed arrays 2 must be run on a little-endian system');
} else
{
  // meSpeak: deleted obsolete lines
  abort('Cannot fallback to non-typed array case in USE_TYPED_ARRAYS == 2: Code is too specialized');
}

var base = intArrayFromString('(null)'); // So printing %s of NULL gives '(null)'
                                         // Also this ensures we leave 0 as an invalid address, 'NULL'
for (var i = 0; i < base.length; i++) {
  HEAP8[(i)]=base[i];
}

Module.HEAP = HEAP;
Module.HEAP8 = HEAP8;
Module.HEAP16 = HEAP16;
Module.HEAP32 = HEAP32;
Module.HEAPU8 = HEAPU8;
Module.HEAPU16 = HEAPU16;
Module.HEAPU32 = HEAPU32;
Module.HEAPF32 = HEAPF32;

STACK_ROOT = STACKTOP = alignMemoryPage(10);
var TOTAL_STACK = 1024*1024; // XXX: Changing this value can lead to bad perf on v8!
STACK_MAX = STACK_ROOT + TOTAL_STACK;

STATICTOP = alignMemoryPage(STACK_MAX);

function __shutdownRuntime__() {
  while(__ATEXIT__.length > 0) {
    var atexit = __ATEXIT__.pop();
    var func = atexit.func;
    if (typeof func === 'number') {
      func = FUNCTION_TABLE[func];
    }
    func(typeof atexit.arg === 'undefined' ? null : atexit.arg);
  }

  // allow browser to GC, set heaps to null?

  // Print summary of correction activity
  CorrectionsMonitor.vprint();
}


// Copies a list of num items on the HEAP into a
// a normal JavaScript array of numbers
function Array_copy(ptr, num) {
  // TODO: In the SAFE_HEAP case, do some reading here, for debugging purposes - currently this is an 'unnoticed read'.
  if (HAS_TYPED_ARRAYS) {
    return Array.prototype.slice.call(HEAP8.subarray(ptr, ptr+num)); // Make a normal array out of the typed 'view'
                                                                     // Consider making a typed array here, for speed?
  } else {
    return HEAP8.slice(ptr, ptr+num);
  }
  return HEAP.slice(ptr, ptr+num);
}
Module.Array_copy = Array_copy;

function String_len(ptr) {
  var i = 0;
  while (HEAP8[(ptr+i)]) i++; // Note: should be |!= 0|, technically. But this helps catch bugs with undefineds
  return i;
}
Module.String_len = String_len;

// Copies a C-style string, terminated by a zero, from the HEAP into
// a normal JavaScript array of numbers
function String_copy(ptr, addZero) {
  var len = String_len(ptr);
  if (addZero) len++;
  var ret = Array_copy(ptr, len);
  if (addZero) ret[len-1] = 0;
  return ret;
}
Module.String_copy = String_copy;

// Tools

if (typeof self.console === 'object' && typeof console.log === 'function') {
  this.vprint = function(x) { console.log(x) }; // web console
} else if (typeof vprint === 'undefined') {
  this.vprint = function(){}; // harmless no-op
}

// This processes a JS string into a C-line array of numbers, 0-terminated.
// For LLVM-originating strings, see parser.js:parseLLVMString function
function intArrayFromString(stringy, dontAddNull) {
  var ret = [];
  var t;
  var i = 0;
  while (i < stringy.length) {
    var chr = stringy.charCodeAt(i);
    if (chr > 0xFF) {
      chr &= 0xFF;
    }
    ret.push(chr);
    i = i + 1;
  }
  if (!dontAddNull) {
    ret.push(0);
  }
  return ret;
}
Module.intArrayFromString = intArrayFromString;

function intArrayToString(array) {
  var ret = [];
  for (var i = 0; i < array.length; i++) {
    var chr = array[i];
    if (chr > 0xFF) {
      chr &= 0xFF;
    }
    ret.push(String.fromCharCode(chr));
  }
  return ret.join('');
}
Module.intArrayToString = intArrayToString;

function unSign(value, bits, ignore, sig) {
  if (value >= 0) {
    return value;
  }
  return bits <= 32 ? 2*Math.abs(1 << (bits-1)) + value // Need some trickery, since if bits == 32, we are right at the limit of the bits JS uses in bitshifts
                    : Math.pow(2, bits)         + value;
  // TODO: clean up previous line
}
function reSign(value, bits, ignore, sig) {
  if (value <= 0) {
    return value;
  }
  var half = bits <= 32 ? Math.abs(1 << (bits-1)) // abs is needed if bits == 32
                        : Math.pow(2, bits-1);
  if (value >= half && (bits <= 32 || value > half)) { // for huge values, we can hit the precision limit and always get true here. so don't do that
    value = -2*half + value; // Cannot bitshift half, as it may be at the limit of the bits JS uses in bitshifts
  }
  return value;
}

// === Body ===



Runtime.QUANTUM_SIZE = 4
var $struct___sFILE___SIZE = 104; // %struct.__sFILE
  var $struct___sFILE___FLATTENER = [0,4,8,12,14,16,24,28,32,36,40,44,48,56,60,64,67,68,76,80,84,88,92,100];
var $struct___sbuf___SIZE = 8; // %struct.__sbuf
  
var $struct__reent___SIZE = 1060; // %struct._reent
  var $struct__reent___FLATTENER = [0,4,8,12,16,20,48,52,56,60,64,68,72,76,80,84,88,328,332,732,736,748];
var $struct__Bigint___SIZE = 24; // %struct._Bigint
  
var $union_anon_0___SIZE = 240; // %union.anon.0
  
var $struct_anon_1___SIZE = 240; // %struct.anon.1
  
var $struct__atexit___SIZE = 400; // %struct._atexit
  var $struct__atexit___FLATTENER = [0,4,8,136];
var $struct__on_exit_args___SIZE = 264; // %struct._on_exit_args
  var $struct__on_exit_args___FLATTENER = [0,128,256,260];
var $struct__glue___SIZE = 12; // %struct._glue
  
var $struct__mbstate_t___SIZE = 8; // %struct._mbstate_t
  
var $union_anon___SIZE = 4; // %union.anon
  
var $struct_option___SIZE = 16; // %struct.option
  
var $struct_MNEM_TAB___SIZE = 8; // %struct.MNEM_TAB
  
var $struct_MatchRecord___SIZE = 16; // %struct.MatchRecord
  
var $struct_TUNE___SIZE = 68; // %struct.TUNE
  var $struct_TUNE___FLATTENER = [0,12,16,24,25,26,27,28,29,30,31,32,33,34,35,36,39,42,43,44,45,46,47,48,49,50,51,52,53,54,55,56,64];
var $struct_SYLLABLE___SIZE = 6; // %struct.SYLLABLE
  
var $struct_TONE_HEAD___SIZE = 16; // %struct.TONE_HEAD
  var $struct_TONE_HEAD___FLATTENER = [0,1,2,3,4,8,9,10,12];
var $struct_SSML_STACK___SIZE = 76; // %struct.SSML_STACK
  var $struct_SSML_STACK___FLATTENER = [0,4,8,12,16,56];
var $struct_PARAM_STACK_45___SIZE = 64; // %struct.PARAM_STACK.45
  
var $struct_ACCENTS___SIZE = 8; // %struct.ACCENTS
  
var $struct_MBROLA_TAB___SIZE = 24; // %struct.MBROLA_TAB
  
var $struct_PHONEME_TAB___SIZE = 16; // %struct.PHONEME_TAB
  var $struct_PHONEME_TAB___FLATTENER = [0,4,8,10,11,12,13,14,15];
var $struct_PHONEME_TAB_LIST_106___SIZE = 48; // %struct.PHONEME_TAB_LIST.106
  var $struct_PHONEME_TAB_LIST_106___FLATTENER = [0,32,36,40,44];
var $struct_frameref_t___SIZE = 8; // %struct.frameref_t
  
var $struct_frame_t___SIZE = 64; // %struct.frame_t
  var $struct_frame_t___FLATTENER = [0,2,16,17,18,26,32,35,39,44,49,56,63];
var $struct_PHONEME_DATA___SIZE = 152; // %struct.PHONEME_DATA
  var $struct_PHONEME_DATA___FLATTENER = [0,4,68,88,108,124,128,132];
var $struct_PHONEME_LIST___SIZE = 24; // %struct.PHONEME_LIST
  var $struct_PHONEME_LIST___FLATTENER = [0,1,2,3,4,6,8,12,14,15,16,17,18,19,20,21];
var $struct_SPEED_FACTORS___SIZE = 64; // %struct.SPEED_FACTORS
  
var $struct_voice_t___SIZE = 1344; // %struct.voice_t
  var $struct_voice_t___FLATTENER = [0,40,44,48,52,56,60,64,68,72,76,80,84,88,92,96,100,104,108,112,144,162,180,198,216,234,252,272,308,344];
var $struct_SOUND_ICON___SIZE = 16; // %struct.SOUND_ICON
  
var $struct_Translator___SIZE = 8328; // %struct.Translator
  var $struct_Translator___FLATTENER = [0,280,284,288,292,332,632,652,656,660,668,676,692,696,700,704,708,964,968,1000,1048,1052,1056,5152,5256,6280,6792,7272,7752,7756,8012,8268,8272,8276,8280,8284,8288,8292,8296,8300,8304,8308,8312,8316,8320,8324];
var $struct_LANGUAGE_OPTIONS___SIZE = 280; // %struct.LANGUAGE_OPTIONS
  var $struct_LANGUAGE_OPTIONS___FLATTENER = [0,4,8,12,16,20,24,100,176,180,184,188,192,196,200,204,208,212,216,220,224,228,232,240,244,248,249,250,251,252,256,260,264,268,272,276];
var $struct_PHONEME_LIST2___SIZE = 8; // %struct.PHONEME_LIST2
  var $struct_PHONEME_LIST2___FLATTENER = [0,1,2,3,4,6];
var $struct_REPLACE_PHONEMES___SIZE = 3; // %struct.REPLACE_PHONEMES
  
var $struct_datablock___SIZE = 16; // %struct.datablock
  
var $struct_espeak_VOICE___SIZE = 24; // %struct.espeak_VOICE
  var $struct_espeak_VOICE___FLATTENER = [0,4,8,12,13,14,15,16,20];
var $struct_wavegen_peaks_t___SIZE = 80; // %struct.wavegen_peaks_t
  var $struct_wavegen_peaks_t___FLATTENER = [0,4,8,12,16,24,32,40,48,56,64,72];
var $struct_RESONATOR___SIZE = 40; // %struct.RESONATOR
  
var $struct_WGEN_DATA___SIZE = 64; // %struct.WGEN_DATA
  
var $struct_sonicStreamStruct___SIZE = 88; // %struct.sonicStreamStruct
  
var $struct_klatt_frame_t___SIZE = 288; // %struct.klatt_frame_t
  var $struct_klatt_frame_t___FLATTENER = [0,4,8,48,88,128,168,172,176,180,184,188,192,196,200,204,208,248];
var $struct_klatt_global_t___SIZE = 2748; // %struct.klatt_global_t
  var $struct_klatt_global_t___FLATTENER = [0,4,8,12,16,20,24,28,32,36,40,44,48,52,56,64,72,80,88,96,104,112,120,128,136,144,152,160,164,172,176,180,184,188,1468];
var $struct_resonator_t___SIZE = 64; // %struct.resonator_t
  
var $struct_klatt_peaks_t___SIZE = 80; // %struct.klatt_peaks_t
  var $struct_klatt_peaks_t___FLATTENER = [0,4,8,12,16,24,32,40,48,56,64,72];
var $struct_frame_t_199___SIZE = 64; // %struct.frame_t.199
  var $struct_frame_t_199___FLATTENER = [0,2,16,17,18,26,32,35,39,44,49,56,63];
var $struct_stat___SIZE = 80; // %struct.stat
  var $struct_stat___FLATTENER = [0,8,12,16,20,24,28,36,40,44,48,52,56,60,64,68,72];
var $struct_PARAM_STACK___SIZE = 64; // %struct.PARAM_STACK
  
var $struct_PHONEME_TAB_LIST___SIZE = 48; // %struct.PHONEME_TAB_LIST
  var $struct_PHONEME_TAB_LIST___FLATTENER = [0,32,36,40,44];
var $struct_PHONEME_TAB_11___SIZE = 16; // %struct.PHONEME_TAB.11
  var $struct_PHONEME_TAB_11___FLATTENER = [0,4,8,10,11,12,13,14,15];
var $struct_RGROUP___SIZE = 28; // %struct.RGROUP
  var $struct_RGROUP___FLATTENER = [0,16,20,24];
var $struct_WORD_TAB___SIZE = 12; // %struct.WORD_TAB
  var $struct_WORD_TAB___FLATTENER = [0,4,6,8,9,10];
var $struct_PHONEME_TAB_LIST_25___SIZE = 48; // %struct.PHONEME_TAB_LIST.25
  var $struct_PHONEME_TAB_LIST_25___FLATTENER = [0,32,36,40,44];
var $struct_WORD_TAB_29___SIZE = 12; // %struct.WORD_TAB.29
  var $struct_WORD_TAB_29___FLATTENER = [0,4,6,8,9,10];
var $struct_Translator_33___SIZE = 8328; // %struct.Translator.33
  var $struct_Translator_33___FLATTENER = [0,280,284,288,292,332,632,652,656,660,668,676,692,696,700,704,708,964,968,1000,1048,1052,1056,5152,5256,6280,6792,7272,7752,7756,8012,8268,8272,8276,8280,8284,8288,8292,8296,8300,8304,8308,8312,8316,8320,8324];
var $struct_LANGUAGE_OPTIONS_32___SIZE = 280; // %struct.LANGUAGE_OPTIONS.32
  var $struct_LANGUAGE_OPTIONS_32___FLATTENER = [0,4,8,12,16,20,24,100,176,180,184,188,192,196,200,204,208,212,216,220,224,228,232,240,244,248,249,250,251,252,256,260,264,268,272,276];
var $struct_TONE_NUCLEUS___SIZE = 16; // %struct.TONE_NUCLEUS
  var $struct_TONE_NUCLEUS___FLATTENER = [0,1,2,3,4,5,8,12,13,14];
var $struct_PARAM_STACK_54___SIZE = 64; // %struct.PARAM_STACK.54
  
var $struct_FMT_PARAMS___SIZE = 48; // %struct.FMT_PARAMS
  
var $struct_FMT_PARAMS_111___SIZE = 48; // %struct.FMT_PARAMS.111
  
var $struct_SPECT_SEQ___SIZE = 1104; // %struct.SPECT_SEQ
  var $struct_SPECT_SEQ___FLATTENER = [0,2,3,4];
var $struct_frame_t2___SIZE = 44; // %struct.frame_t2
  var $struct_frame_t2___FLATTENER = [0,2,16,17,18,26,32,35,39];
var $struct_SPECT_SEQK___SIZE = 1604; // %struct.SPECT_SEQK
  var $struct_SPECT_SEQK___FLATTENER = [0,2,3,4];
var $struct_pollfd___SIZE = 8; // %struct.pollfd
  var $struct_pollfd___FLATTENER = [0,4,6];
var $struct_dirent___SIZE = 1040; // %struct.dirent
  var $struct_dirent___FLATTENER = [0,4,1028,1032,1036];
var $struct_getopt_data___SIZE = 20; // %struct.getopt_data
  
var _path_home;
var _filetype;
var _wavefile;
var _uri_callback;
var _phoneme_callback;
var _f_wave;
var _quiet;
var _samples_total;
var _samples_split;
var _wavefile_count;
var _end_of_sentence;
var _voice_pcnt;
var __impure_ptr;
var __ZZ13DisplayVoicesP7__sFILEPcE7genders;
var __str1;
var __str3;
var __str4;
var __str5;
var __ZZ4mainE12long_options;
var __str6;
var __str7;
var __str8;
var __str9;
var __str10;
var __str11;
var __str12;
var __str13;
var __str14;
var __str15;
var __str16;
var __str17;
var __str18;
var __str19;
var __str21;
var __str23;
var __str25;
var __str27;
var __str28;
var __str29;
var __str30;
var __ZZL12OpenWaveFilePKciE8wave_hdr;
var __str32;
var __str33;
var __str35;
var __str36;
var __str37;
var __str38;
var __str39;
var __str40;
var __str41;
var __ZL5f_log;
var __str42;
var __str143;
var __str244;
var __str345;
var __str446;
var __str547;
var __str648;
var _mnem_rules;
var __str749;
var __str850;
var __str951;
var __str1052;
var __str1153;
var __str1254;
var __str1355;
var __str1456;
var __str1557;
var __str1658;
var __str1759;
var __str1860;
var __str1961;
var __str2062;
var __str2163;
var __str2264;
var __str2365;
var __str2466;
var __str2567;
var __str2668;
var __str2769;
var __str2870;
var __str2971;
var __str3072;
var __str3173;
var __str3274;
var __str3375;
var __str3476;
var __str3577;
var __str3678;
var __str3779;
var __str3880;
var __str3981;
var __str4082;
var __str4183;
var __str4284;
var __str43;
var __str44;
var __str45;
var __str46;
var __str47;
var __str48;
var __str49;
var __str50;
var __str51;
var __str52;
var __str53;
var __str54;
var __str55;
var __str56;
var __str57;
var _mnem_flags;
var __ZZ22print_dictionary_flagsPjE3buf;
var __str59;
var __ZZ10DecodeRulePKciPciE6output;
var __ZZ10DecodeRulePKciPciE7symbols;
var __ZZ10DecodeRulePKciPciE10symbols_lg;
var __str60;
var __str61;
var __str62;
var __str64;
var __str65;
var __ZL11error_count;
var __ZL21error_need_dictionary;
var __ZL19letterGroupsDefined;
var __ZL10debug_flag;
var __str66;
var __str67;
var __str69;
var __str71;
var __str72;
var __str73;
var __str74;
var __str75;
var __str76;
var __str77;
var __str78;
var __ZL7linenum;
var __ZL10group_name;
var __str79;
var __ZL9group3_ix;
var __str81;
var __str82;
var __str83;
var __str84;
var __str85;
var __str89;
var __ZL9rule_cond;
var __ZL8rule_pre;
var __ZL9rule_post;
var __ZL10rule_match;
var __ZL13rule_phonemes;
var __str90;
var __str91;
var __str92;
var __str93;
var __str94;
var __str95;
var __ZZL16copy_rule_stringPcRiE6outbuf;
var __ZZL16copy_rule_stringPcRiE10next_state;
var __ZZL16copy_rule_stringPcRiE16lettergp_letters;
var __str97;
var __str98;
var __str99;
var __str100;
var __str101;
var __str102;
var __str103;
var __ZL11hash_chains;
var __ZL11hash_counts;
var __ZL9text_mode_b;
var __str104;
var __str106;
var __str107;
var __ZZL12compile_linePcS_PiE10nullstring;
var __str108;
var __str109;
var __str110;
var __str111;
var __str112;
var __str113;
var __str114;
var __str115;
var _dictionary_skipwords;
var _dictionary_name;
var __ZL13remove_accent;
var __str116;
var __str2118;
var __str3119;
var __str4120;
var __str5121;
var __str7123;
var _ipa1;
var __str8124;
var __str9125;
var __ZL15stress_phonemes;
var __ZZ13SetWordStressP10TranslatorPcPjiiE15consonant_types;
var __ZZ13SetWordStressP10TranslatorPcPjiiE8guess_ru;
var __ZZ13SetWordStressP10TranslatorPcPjiiE10guess_ru_v;
var __ZZ13SetWordStressP10TranslatorPcPjiiE10guess_ru_t;
var __ZZ14TranslateRulesP10TranslatorPcS1_iS1_iPjE9str_pause;
var __str10126;
var __str11127;
var __ZL13diereses_list;
var __ZZ14LookupDictListP10TranslatorPPcS1_PjiP8WORD_TABE16word_replacement;
var __str15131;
var __str16132;
var __ZZ11LookupFlagsP10TranslatorPKcE5flags;
var __ZZ12RemoveEndingP10TranslatorPciS1_E6ending;
var __ZZ12RemoveEndingP10TranslatorPciS1_E16add_e_exceptions;
var __str17133;
var __ZZ12RemoveEndingP10TranslatorPciS1_E15add_e_additions;
var __str18134;
var __str19135;
var __str20136;
var __str21137;
var __str22138;
var __str23139;
var __str24140;
var __str25141;
var __str27143;
var __str28144;
var __str29145;
var __str30146;
var __str31147;
var __ZZL9MatchRuleP10TranslatorPPcS1_iS1_P11MatchRecordiiE4best;
var __str32148;
var __str34150;
var _env_fall;
var _env_rise;
var _env_frise;
var __ZL11env_r_frise;
var __ZL10env_frise2;
var __ZL12env_r_frise2;
var __ZL12env_risefall;
var __ZL13env_fallrise3;
var __ZL13env_fallrise4;
var __ZL9env_fall2;
var __ZL9env_rise2;
var __ZL16env_risefallrise;
var _envelope_data;
var __ZL18tone_nucleus_table;
var _punctuation_to_tone;
var _n_tunes;
var _tunes;
var __ZL12syllable_tab;
var __ZL8no_tonic_b;
var __ZL15tone_head_table;
var __ZL10number_pre;
var __ZL10tone_posn2;
var __ZL9tone_posn;
var __ZL11number_tail;
var __ZL14tone_pitch_env;
var __ZZL18calc_pitch_segmentiiP9TONE_HEADP12TONE_NUCLEUSiiE12continue_tab;
var __ZL8min_drop;
var __ZL7drops_0;
var __ZL5oflow;
var __ZL9oflow_emf;
var __ZL10oflow_less;
var __ZL12last_primary;
var _namedata;
var __ZL7f_input;
var _p_textinput;
var _p_wchar_input;
var __ZL10ungot_word;
var _count_characters;
var __ZL10ssml_stack;
var __ZL16current_voice_id;
var _param_stack;
var _param_defaults;
var _walpha_tab;
var __ZL10ungot_char;
var __ZL12end_of_input_b;
var __str167;
var __str1168;
var __str2169;
var __str3170;
var __str4171;
var __str5172;
var __str6173;
var __str7174;
var __str8175;
var __str9176;
var __str10177;
var __str11178;
var __str12179;
var __str13180;
var __str14181;
var __str15182;
var __str16183;
var __str17184;
var __str18185;
var __str19186;
var __str20187;
var __str21188;
var __str22189;
var __str23190;
var __str24191;
var __str25192;
var __str26193;
var __str27194;
var __str28195;
var __str29196;
var __str30197;
var __str31198;
var __ZL8ssmltags;
var __ZL10n_namedata;
var __ZL11namedata_ix;
var __str32199;
var __str33200;
var __str34201;
var __str35202;
var __str36203;
var __str37204;
var __ZL18xml_char_mnemonics;
var __ZZ10ReadClauseP10TranslatorP7__sFILEPcPsPiiS5_S3_E12ungot_string;
var __ZZ10ReadClauseP10TranslatorP7__sFILEPcPsPiiS5_S3_E15ungot_string_ix;
var __ZL19clear_skipping_text_b;
var __ZL11ungot_char2;
var __ZL19ssml_ignore_l_angle;
var __str38205;
var __ZL10sayas_mode;
var __ZL11ignore_text_b;
var __ZL11punct_chars;
var __ZL16punct_attributes;
var __str41208;
var __ZL12chars_ignore;
var __str42209;
var __ZL10audio_text_b;
var __ZL17speech_parameters;
var __ZL12n_ssml_stack;
var __ZL13n_param_stack;
var __ZL7xmlbase;
var __ZZL14LookupCharNameP10TranslatoriiE3buf;
var __str47214;
var __str48215;
var __str49216;
var __ZZL13WordToString2jE3buf;
var __str50217;
var __str51218;
var __str52219;
var __str53220;
var __str54221;
var __str55222;
var __str57224;
var __str58225;
var __str59226;
var __str60227;
var __ZZL14ProcessSsmlTagPwPcRiiiE10mnem_punct;
var __str61228;
var __str62229;
var __str63230;
var __ZZL14ProcessSsmlTagPwPcRiiiE13mnem_capitals;
var __str64231;
var __str65232;
var __str66233;
var __ZZL14ProcessSsmlTagPwPcRiiiE17mnem_interpret_as;
var __str68235;
var __str69236;
var __str70237;
var __str71238;
var __str72239;
var __ZZL14ProcessSsmlTagPwPcRiiiE17mnem_sayas_format;
var __str73240;
var __ZZL14ProcessSsmlTagPwPcRiiiE10mnem_break;
var __str74241;
var __str75242;
var __str76243;
var __str77244;
var __ZZL14ProcessSsmlTagPwPcRiiiE13mnem_emphasis;
var __str78245;
var __str79246;
var __ZZL14ProcessSsmlTagPwPcRiiiE12prosody_attr;
var __str80247;
var __str81248;
var __str82249;
var __ZL22ignore_if_self_closing;
var __str83250;
var __str84251;
var __str85252;
var __str86253;
var __str87254;
var __ZZL14ProcessSsmlTagPwPcRiiiE23emphasis_to_pitch_range;
var __ZZL14ProcessSsmlTagPwPcRiiiE18emphasis_to_volume;
var __str88255;
var __str89256;
var __str90257;
var __str91258;
var __ZL11sayas_start;
var __str92259;
var __str94261;
var __str95262;
var __str96263;
var __str97264;
var __str98265;
var __str99266;
var __ZZL14ProcessSsmlTagPwPcRiiiE11break_value;
var __str100267;
var __str101268;
var __str102269;
var __ZZL18GetVoiceAttributesPwiE11mnem_gender;
var __str105272;
var __str106273;
var __str108275;
var __ZZL14LoadSoundFile2PKcE4slot;
var __str111278;
var __str112279;
var __str113280;
var __str114281;
var __ZZL14ReplaceKeyNamePciRiE8keynames;
var __ZZL19SetProsodyParameteriPwP11PARAM_STACKE11mnem_volume;
var __str115282;
var __str116283;
var __str117;
var __str118;
var __str119;
var __ZZL19SetProsodyParameteriPwP11PARAM_STACKE9mnem_rate;
var __str120;
var __str121;
var __str122;
var __str123;
var __ZZL19SetProsodyParameteriPwP11PARAM_STACKE10mnem_pitch;
var __str124;
var __str125;
var __str126;
var __str127;
var __ZZL19SetProsodyParameteriPwP11PARAM_STACKE10mnem_range;
var __ZZL19SetProsodyParameteriPwP11PARAM_STACKE9mnem_tabs;
var __ZZL17ProcessParamStackPcRiE10cmd_letter;
var __str128;
var __ZZL16GetSsmlAttributePwPKcE5empty;
var __ZZL4GetCvE6ungot2;
var __ZZL4GetCvE5mask2;
var __ZL6speed1;
var __ZL6speed2;
var __ZL6speed3;
var __ZL12speed_lookup;
var __ZL14wav_factor_350;
var __ZL16pause_factor_350;
var __ZZ11CalcLengthsP10TranslatorE14more_syllables;
var __str294;
var __ZL18letter_accents_0e0;
var __ZL18letter_accents_250;
var __ZL13non_ascii_tab;
var __ZL11accents_tab;
var __str297;
var __str1298;
var __str2299;
var __ZZ12LookupLetterP10TranslatorjiPciE13single_letter;
var __str5302;
var __str6303;
var __str7304;
var __str8305;
var __str9306;
var __str10307;
var __str11308;
var __str12309;
var __ZZ14TranslateRomanP10TranslatorPcS1_P8WORD_TABE12roman_values;
var __str13310;
var __str14311;
var __ZZL17TranslateNumber_1P10TranslatorPcS1_PjP8WORD_TABiE9str_pause;
var __ZL14n_digit_lookup;
var __ZL12digit_lookup;
var __ZL14number_control;
var __ZL23speak_missing_thousands;
var __ZL11ph_ordinal2;
var __str15312;
var __str16313;
var __str17314;
var __str18315;
var __str19316;
var __str20317;
var __str21318;
var __str22319;
var __str23320;
var __str24321;
var __str25322;
var __str26323;
var __str27324;
var __str28325;
var __str29326;
var __str30327;
var __str31328;
var __str32329;
var __str33330;
var __str35332;
var __str36333;
var __str37334;
var __str39336;
var __str40337;
var __str41338;
var __str42339;
var __str43340;
var __str44341;
var __str45342;
var __str46343;
var __str47344;
var __str48345;
var __str49346;
var __str50347;
var __str51348;
var __str52349;
var __str53350;
var __str54351;
var __str55352;
var __str56353;
var __str57354;
var __str58355;
var __str59356;
var __str60357;
var __str61358;
var __str62359;
var __str63360;
var __str64361;
var __str65362;
var __str66363;
var __str67364;
var __str68365;
var __str69366;
var __str70367;
var __str71368;
var __str72369;
var __str73370;
var __str74371;
var __str75372;
var __str76373;
var __str77374;
var __str78375;
var _option_mbrola_phonemes;
var __ZL10mbrola_tab;
var __ZL15mbr_name_prefix;
var __str382;
var __str1383;
var __str2384;
var __str3385;
var __str4386;
var __ZL14mbrola_control;
var __ZZ15MbrolaTranslateP12PHONEME_LISTiiP7__sFILEE4phix;
var __ZZ15MbrolaTranslateP12PHONEME_LISTiiP7__sFILEE11embedded_ix;
var __ZZ15MbrolaTranslateP12PHONEME_LISTiiP7__sFILEE10word_count;
var __str6388;
var __str8390;
var __str9391;
var __str10392;
var __str11393;
var __str12394;
var __ZZ10MbrolaFilliiE9n_samples;
var __ZZL10WritePitchiiiiiE6output;
var __str13395;
var __str14396;
var __str15397;
var __str16398;
var __ZZL10GetMbrNameP12PHONEME_LISTP11PHONEME_TABS2_S2_PiS3_S3_E4mnem;
var __str399;
var _version_string;
var _version_phdata;
var _option_device_number;
var _f_logespeak;
var _logging_type;
var _n_phoneme_tab;
var _current_phoneme_table;
var _phoneme_tab;
var _phoneme_tab_flags;
var _phoneme_index;
var _phondata_ptr;
var _wavefile_data;
var __ZL16phoneme_tab_data;
var _n_phoneme_tables;
var _phoneme_tab_list;
var _phoneme_tab_number;
var _wavefile_ix;
var _wavefile_amp;
var _wavefile_ix2;
var _wavefile_amp2;
var _seq_len_adjust;
var _vowel_transition;
var _vowel_transition0;
var _vowel_transition1;
var __str1412;
var __str2413;
var __str3414;
var __str4415;
var __ZZ11LookupSpectP11PHONEME_TABiP10FMT_PARAMSPiP12PHONEME_LISTE10frames_buf;
var __str5416;
var __str7418;
var __str9420;
var __str10421;
var __str11422;
var __str13424;
var __str15426;
var __str16427;
var _this_ph_data;
var __ZZL13NumInstnWordsPtE7n_words;
var __ZZL18InterpretConditionP10TranslatoriP12PHONEME_LISTiE11ph_position;
var __ZZL15StressConditionP10TranslatorP12PHONEME_LISTiiE15condition_level;
var __str17428;
var __str18429;
var __str19430;
var _n_phoneme_list;
var _phoneme_list;
var _mbrola_delay;
var _mbrola_name;
var _speed;
var __ZL9new_voice;
var _n_soundicon_tab;
var _soundicon_tab;
var __ZZ12WordToStringjE3buf;
var __ZL14last_pitch_cmd;
var __ZL12last_amp_cmd;
var __ZL10last_frame;
var __ZL15syllable_centre;
var __ZZ18FormantTransition2P10frameref_tRijjP11PHONEME_TABiE10vcolouring;
var __ZL10modn_flags;
var __ZZ8DoSpect2P11PHONEME_TABiP10FMT_PARAMSP12PHONEME_LISTiE9wave_flag_b;
var __ZL13fmt_amplitude;
var __ZL10last_wcmdq;
var __ZL12pitch_length;
var __ZL10amp_length;
var __ZZ8GenerateP12PHONEME_LISTPiiE2ix;
var __ZZ8GenerateP12PHONEME_LISTPiiE11embedded_ix;
var __ZZ8GenerateP12PHONEME_LISTPiiE10word_count;
var __ZZ8GenerateP12PHONEME_LISTPiiE8sourceix;
var __ZL14syllable_start;
var __ZL12syllable_end;
var __ZL8timer_on_b;
var __ZL6paused_b;
var __ZZ15SpeakNextClauseP7__sFILEPKviE6f_text;
var __ZZ15SpeakNextClauseP7__sFILEPKviE6p_text;
var __str459;
var __str1460;
var __ZZL13set_frame_rmsP7frame_tiE8sqrt_tab;
var __ZZL10AllocFramevE2ix;
var __ZZL10AllocFramevE10frame_pool;
var _translator;
var _translator2;
var __ZL20translator2_language;
var _f_trans;
var _option_tone2;
var _option_tone_flags;
var _option_phonemes;
var _option_phoneme_events;
var _option_quiet;
var _option_endpause;
var _option_capitals;
var _option_punctuation;
var _option_sayas;
var _option_ssml;
var _option_phoneme_input;
var _option_phoneme_variants;
var _option_wordgap;
var _skip_sentences;
var _skip_words;
var _skip_characters;
var _skip_marker;
var _skipping_text;
var _end_character_position;
var _count_sentences;
var _count_words;
var _clause_start_char;
var _clause_start_word;
var _new_sentence;
var _pre_pause;
var _word_phonemes;
var _n_ph_list2;
var _ph_list2;
var _option_punctlist;
var _ctrl_embedded;
var _option_multibyte;
var _option_linelength;
var _embedded_list;
var _n_replace_phonemes;
var _replace_phonemes;
var __ZL10ISO_8859_1;
var __ZL10ISO_8859_2;
var __ZL10ISO_8859_3;
var __ZL10ISO_8859_4;
var __ZL10ISO_8859_5;
var __ZL10ISO_8859_7;
var __ZL10ISO_8859_9;
var __ZL11ISO_8859_14;
var __ZL6KOI8_R;
var __ZL5ISCII;
var _charsets;
var __ZL15length_mod_tabs;
var __ZZ7IsAlphajE18extra_indic_alphas;
var __ZL8brackets;
var __ZZ8utf8_outjPcE4code;
var __ZZ8utf8_in2PiPKciE4mask;
var __str543;
var __str1544;
var __ZZ13TranslateWordP10TranslatorPciP8WORD_TABE7word_zz;
var __ZZ13TranslateWordP10TranslatorPciP8WORD_TABE7word_iz;
var __ZZ13TranslateWordP10TranslatorPciP8WORD_TABE7word_ss;
var __str2545;
var __str4547;
var __str5548;
var __str6549;
var __ZL18any_stressed_words_b;
var __ZZ15TranslateClauseP10TranslatorP7__sFILEPKvPiPPcE17voice_change_name;
var __ZL11embedded_ix;
var __ZL13embedded_read;
var __ZL6source;
var __str7550;
var __str8551;
var __ZL16max_clause_pause;
var __ZL13option_sayas2;
var __ZL18count_sayas_digits;
var __ZL6breaks;
var __ZL13word_emphasis_b;
var __ZL15option_emphasis;
var __ZL13embedded_flag_b;
var __str9552;
var __str10553;
var __str11554;
var __ZZL13TranslateCharP10TranslatorPcijjPiE20hangul_compatibility;
var __ZZL14SubstituteCharP10TranslatorjjPiE11ignore_next_b;
var __str12555;
var __ZL14length_mods_en;
var __ZL15length_mods_en0;
var __ZL17length_mods_equal;
var __str580;
var __str1581;
var __str2582;
var __str3583;
var __str4584;
var __ZL14mbr_samplerate;
var __ZL14mbr_voice_path;
var __ZL10mbr_volume;
var __ZL9mbr_state;
var __ZL7mbr_pid;
var __ZL10mbr_cmd_fd;
var __str5585;
var __ZL12mbr_audio_fd;
var __ZL12mbr_errorbuf;
var __ZL12mbr_error_fd;
var __str7587;
var __str8588;
var __str9589;
var __str10590;
var __str11591;
var __str12592;
var __str13593;
var __str14594;
var __str15595;
var __str16596;
var __str17597;
var __ZL21mbr_pending_data_head;
var __ZL21mbr_pending_data_tail;
var __str18598;
var __str19599;
var __str20600;
var __str21601;
var __ZL13mbr_proc_stat;
var __str22602;
var __str23603;
var __str24604;
var __str25605;
var __str27607;
var __str28608;
var __str30610;
var __str31611;
var __str32612;
var __str33613;
var __str34614;
var __str35615;
var __ZZ15SetIndicLettersP10TranslatorE15dev_consonants2;
var __ZZ16SelectTranslatorPKcE17stress_lengths_fr;
var __ZZ16SelectTranslatorPKcE14stress_amps_fr;
var __ZZ16SelectTranslatorPKcE14stress_amps_sk;
var __ZZ16SelectTranslatorPKcE17stress_lengths_sk;
var __ZZ16SelectTranslatorPKcE17stress_lengths_ta;
var __ZZ16SelectTranslatorPKcE14stress_amps_ta;
var __ZZ16SelectTranslatorPKcE17stress_lengths_af;
var __ZZ16SelectTranslatorPKcE17stress_lengths_bn;
var __ZZ16SelectTranslatorPKcE14stress_amps_bn;
var __ZZ16SelectTranslatorPKcE17stress_lengths_cy;
var __ZZ16SelectTranslatorPKcE14stress_amps_cy;
var __ZZ16SelectTranslatorPKcE17stress_lengths_da;
var __ZZ16SelectTranslatorPKcE17stress_lengths_de;
var __ZZ16SelectTranslatorPKcE14stress_amps_de;
var __ZZ16SelectTranslatorPKcE17stress_lengths_en;
var __str630;
var __ZZ16SelectTranslatorPKcE17stress_lengths_el;
var __ZZ16SelectTranslatorPKcE14stress_amps_el;
var __ZZ16SelectTranslatorPKcE9el_vowels;
var __ZZ16SelectTranslatorPKcE10el_fvowels;
var __ZZ16SelectTranslatorPKcE12el_voiceless;
var __ZZ16SelectTranslatorPKcE13el_consonants;
var __ZZ16SelectTranslatorPKcE18el_char_apostrophe;
var __ZZ16SelectTranslatorPKcE17stress_lengths_eo;
var __ZZ16SelectTranslatorPKcE14stress_amps_eo;
var __ZZ16SelectTranslatorPKcE18eo_char_apostrophe;
var __ZZ16SelectTranslatorPKcE17stress_lengths_es;
var __ZZ16SelectTranslatorPKcE14stress_amps_es;
var __ZZ16SelectTranslatorPKcE20ca_punct_within_word;
var __ZZ16SelectTranslatorPKcE17stress_lengths_eu;
var __ZZ16SelectTranslatorPKcE14stress_amps_eu;
var __ZZ16SelectTranslatorPKcE14stress_amps_fi;
var __ZZ16SelectTranslatorPKcE17stress_lengths_fi;
var __ZZ16SelectTranslatorPKcE17stress_lengths_hi;
var __ZZ16SelectTranslatorPKcE14stress_amps_hi;
var __ZZ16SelectTranslatorPKcE14stress_amps_hr;
var __ZZ16SelectTranslatorPKcE17stress_lengths_hr;
var __ZZ16SelectTranslatorPKcE17stress_lengths_sr;
var __ZL22replace_cyrillic_latin;
var __ZZ16SelectTranslatorPKcE14stress_amps_hu;
var __ZZ16SelectTranslatorPKcE17stress_lengths_hu;
var __ZZ16SelectTranslatorPKcE17stress_lengths_hy;
var __ZZ16SelectTranslatorPKcE9hy_vowels;
var __ZZ16SelectTranslatorPKcE13hy_consonants;
var __ZZ16SelectTranslatorPKcE17stress_lengths_id;
var __ZZ16SelectTranslatorPKcE14stress_amps_id;
var __ZZ16SelectTranslatorPKcE17stress_lengths_is;
var __ZZ16SelectTranslatorPKcE16is_lettergroup_B;
var __str1631;
var __str2632;
var __ZZ16SelectTranslatorPKcE17stress_lengths_it;
var __ZZ16SelectTranslatorPKcE14stress_amps_it;
var __ZZ16SelectTranslatorPKcE18stress_lengths_jbo;
var __ZZ16SelectTranslatorPKcE21jbo_punct_within_word;
var __ZZ16SelectTranslatorPKcE9ka_vowels;
var __ZZ16SelectTranslatorPKcE13ka_consonants;
var __ZZ16SelectTranslatorPKcE10ko_ivowels;
var __ZZ16SelectTranslatorPKcE9ko_voiced;
var __ZZ16SelectTranslatorPKcE14stress_amps_ku;
var __ZZ16SelectTranslatorPKcE17stress_lengths_ku;
var __ZZ16SelectTranslatorPKcE14stress_amps_lv;
var __ZZ16SelectTranslatorPKcE17stress_lengths_lv;
var __ZZ16SelectTranslatorPKcE15vowels_cyrillic;
var __ZZ16SelectTranslatorPKcE14stress_amps_mk;
var __ZZ16SelectTranslatorPKcE17stress_lengths_mk;
var __ZZ16SelectTranslatorPKcE17stress_lengths_nl;
var __str3633;
var __ZZ16SelectTranslatorPKcE17stress_lengths_no;
var __ZZ16SelectTranslatorPKcE14stress_amps_om;
var __ZZ16SelectTranslatorPKcE17stress_lengths_om;
var __ZZ16SelectTranslatorPKcE17stress_lengths_pl;
var __ZZ16SelectTranslatorPKcE14stress_amps_pl;
var __ZZ16SelectTranslatorPKcE17stress_lengths_pt;
var __ZZ16SelectTranslatorPKcE14stress_amps_pt;
var __str4634;
var __ZZ16SelectTranslatorPKcE17stress_lengths_ro;
var __ZZ16SelectTranslatorPKcE14stress_amps_ro;
var __str5635;
var __ZZ16SelectTranslatorPKcE17stress_lengths_sq;
var __ZZ16SelectTranslatorPKcE14stress_amps_sq;
var __ZZ16SelectTranslatorPKcE14stress_amps_sv;
var __ZZ16SelectTranslatorPKcE17stress_lengths_sv;
var __ZZ16SelectTranslatorPKcE17stress_lengths_sw;
var __ZZ16SelectTranslatorPKcE14stress_amps_sw;
var __ZZ16SelectTranslatorPKcE14stress_amps_tr;
var __ZZ16SelectTranslatorPKcE17stress_lengths_tr;
var __ZZ16SelectTranslatorPKcE17stress_lengths_vi;
var __ZZ16SelectTranslatorPKcE14stress_amps_vi;
var __ZZ16SelectTranslatorPKcE9vowels_vi;
var __ZZ16SelectTranslatorPKcE17stress_lengths_zh;
var __ZZ16SelectTranslatorPKcE14stress_amps_zh;
var __ZZL18Translator_RussianP10TranslatorE14stress_amps_ru;
var __ZZL18Translator_RussianP10TranslatorE17stress_lengths_ru;
var __ZZL18Translator_RussianP10TranslatorE11ru_ivowels2;
var __ZZL18SetCyrillicLettersP10TranslatorE9ru_vowels;
var __ZZL18SetCyrillicLettersP10TranslatorE13ru_consonants;
var __ZZL18SetCyrillicLettersP10TranslatorE7ru_soft;
var __ZZL18SetCyrillicLettersP10TranslatorE7ru_hard;
var __ZZL18SetCyrillicLettersP10TranslatorE10ru_nothard;
var __ZZL18SetCyrillicLettersP10TranslatorE9ru_voiced;
var __ZZL18SetCyrillicLettersP10TranslatorE10ru_ivowels;
var __ZL8pairs_ru;
var __ZZL13NewTranslatorvE12stress_amps2;
var __ZZL13NewTranslatorvE15stress_lengths2;
var __ZZL13NewTranslatorvE13empty_wstring;
var __ZZL13NewTranslatorvE13punct_in_word;
var __ZZL13NewTranslatorvE13default_tunes;
var __str6636;
var __str7637;
var __str8638;
var __str9639;
var __str10640;
var __str11641;
var __str12642;
var __str13643;
var __str646;
var __str1647;
var __str2648;
var _genders;
var _tone_points;
var _formant_rate;
var _voice_selected;
var __str3653;
var __str4654;
var __ZL11options_tab;
var __str5655;
var __str6656;
var __str7657;
var __str8658;
var __str9659;
var __str10660;
var __str11661;
var __str12662;
var __str13663;
var __str14664;
var __str15665;
var __str16666;
var __str17667;
var __str18668;
var __str19669;
var __str20670;
var __str21671;
var __str22672;
var __str23673;
var __str24674;
var __str25675;
var __str26676;
var __str27677;
var __str28678;
var __str29679;
var __str30680;
var __str31681;
var __str32682;
var __str33683;
var __str34684;
var __str35685;
var __str36686;
var __str37687;
var __str38688;
var __str39689;
var __str40690;
var __str41691;
var __str42692;
var __str43693;
var __ZL11keyword_tab;
var __ZL15variants_either;
var __ZL13variants_male;
var __ZL15variants_female;
var _variant_lists;
var __ZL9voicedata;
var _voice;
var __str44696;
var __ZZ10VoiceResetiE15default_heights;
var __ZZ10VoiceResetiE14default_widths;
var __ZZ10VoiceResetiE13breath_widths;
var __ZL18formant_rate_22050;
var __ZZ9LoadVoicePKciE16voice_identifier;
var __ZZ9LoadVoicePKciE10voice_name;
var __ZZ9LoadVoicePKciE15voice_languages;
var __str45697;
var __str46698;
var __str47699;
var __str48700;
var __str49701;
var __str50702;
var __str51703;
var __str52704;
var __str53705;
var __str54706;
var __str55707;
var __str56708;
var __str57709;
var __str58710;
var __str59711;
var __str60712;
var __str61713;
var __str62714;
var __str63715;
var __str64716;
var __str65717;
var __str66718;
var __str67719;
var __str68720;
var __str69721;
var __ZL13n_voices_list;
var __ZL11voices_list;
var __str70722;
var __ZZ11SelectVoiceP12espeak_VOICEPiE14voice_variants;
var __ZZ11SelectVoiceP12espeak_VOICEPiE8voice_id;
var __ZZ11SelectVoiceP12espeak_VOICEPiE3buf;
var __str71723;
var __ZZ14SetVoiceByNamePKcE3buf;
var __ZZ17espeak_ListVoicesE6voices;
var __str72724;
var __ZL15len_path_voices;
var __str73725;
var __str74726;
var __str75727;
var __ZZL23ExtractVoiceVariantNamePciE12variant_name;
var __str76728;
var __str77729;
var __str78730;
var __str79731;
var __str80732;
var __str81733;
var __str82734;
var _sin_tab;
var _wvoice;
var _f_log;
var _option_waveout;
var _option_log_frames;
var _embedded_value;
var _samplerate;
var _samplerate_native;
var __ZL5peaks;
var _echo_head;
var _echo_tail;
var _echo_amp;
var _echo_buf;
var __ZL7rbreath;
var _wdata;
var __ZL13amplitude_env;
var _out_ptr;
var _out_start;
var _out_end;
var _outbuf_size;
var _wcmdq;
var _wcmdq_head;
var _wcmdq_tail;
var _embedded_default;
var _current_source_index;
var __ZL18sonicSpeedupStream;
var _sonicSpeed;
var _pitch_adjust_tab;
var _pk_shape1;
var __ZL16PHASE_INC_FACTOR;
var __ZL11Flutter_inc;
var __ZL11samplecount;
var __ZL8nsamples;
var __ZL9wavephase;
var __ZL12wavemult_max;
var __ZL15wavemult_offset;
var __ZL8wavemult;
var __ZL9pk_shape2;
var __ZL8pk_shape;
var __ZZ12GetAmplitudevE12amp_emphasis;
var __ZL17general_amplitude;
var __ZL11peak_height;
var __ZL13peak_harmonic;
var __ZL16option_harmonic1_b;
var __ZL9harmspect;
var __ZL8harm_inc;
var __ZL10minus_pi_t;
var __ZL8two_pi_t;
var __ZZ7WavegenvE4maxh;
var __ZZ7WavegenvE5maxh2;
var __ZZ7WavegenvE3agc;
var __ZZ7WavegenvE13h_switch_sign;
var __ZZ7WavegenvE11cycle_count;
var __ZZ7WavegenvE10amplitude2;
var __ZL8end_wave_b;
var __ZL7hswitch;
var __ZL6hspect;
var __ZL8phaseinc;
var __ZL13cycle_samples;
var __ZL9hf_factor;
var __ZL6cbytes;
var __ZL12glottal_flag;
var __ZL14glottal_reduce;
var __ZL6amp_ix;
var __ZL15modulation_type;
var __ZL14modulation_tab;
var __ZL7voicing;
var __ZL12embedded_max;
var __ZZ15WavegenSetVoiceP7voice_tE2v2;
var __ZL13consonant_amp;
var __ZL11flutter_amp;
var __ZZ8SetSynthiiP7frame_tS0_P7voice_tE19glottal_reduce_tab1;
var __ZZ8SetSynthiiP7frame_tS0_P7voice_tE19glottal_reduce_tab2;
var __ZL17samplecount_start;
var __ZZ12WavegenFill2iE6resume;
var __ZZ12WavegenFill2iE13echo_complete;
var __ZL11echo_length;
var __ZL7amp_inc;
var __ZZL8PlayWaveiiPhiiE9n_samples;
var __ZZL8PlayWaveiiPhiiE2ix;
var __ZZL11PlaySilenceiiE9n_samples;
var __ZZL17AdvanceParametersvE10Flutter_ix;
var __ZL11Flutter_tab;
var _pause_phonemes;
var __ZZ15MakePhonemeListP10TranslatoriiE12types_double;
var __ZL8kt_frame;
var __ZL10kt_globals;
var __ZL5peaks801;
var __ZL12sample_count;
var __ZL8nsamples802;
var __ZL6klattp;
var __ZL10klattp_inc;
var __ZL7klattp1;
var __ZL8end_wave803;
var __ZZ14SetSynth_KlattiiP7frame_tS0_P7voice_tiE7prev_fr;
var __ZL13scale_wav_tab;
var __ZZ9KlattInitvE15natural_samples;
var __ZZ9KlattInitvE10formant_hz;
var __ZZ9KlattInitvE9bandwidth;
var __ZZ9KlattInitvE12parallel_amp;
var __ZZ9KlattInitvE11parallel_bw;
var __ZZL7parwaveP13klatt_frame_tE5noise;
var __ZZL7parwaveP13klatt_frame_tE5voice;
var __ZZL7parwaveP13klatt_frame_tE5vlast;
var __ZZL7parwaveP13klatt_frame_tE8glotlast;
var __ZZL7parwaveP13klatt_frame_tE5sourc;
var __ZZL21pitch_synch_par_resetP13klatt_frame_tE4skew;
var __ZZL21pitch_synch_par_resetP13klatt_frame_tE2B0;
var __ZZL7DBtoLINlE8amptable;
var __ZZL14natural_sourcevE5vwave;
var __ZZL16impulsive_sourcevE7doublet;
var __ZZL16impulsive_sourcevE5vwave;
var __ZZL9gen_noisedE5nlast;
var __ZZL7flutterP13klatt_frame_tE10time_count;
var __ZZL10frame_initP13klatt_frame_tE14amp_par_factor;
var __str810;
var __str1811;
var _optarg;
var _optind;
var _opterr;
var _optopt;
var __ZL8optwhere;
var __str831;
var __str1832;
var __str2833;
var __str3834;
var __str4835;
var __str5836;
var __str6837;
var _llvm_dbg_declare; // stub for _llvm_dbg_declare

  
  
  var ERRNO_CODES={E2BIG: 7, EACCES: 13, EADDRINUSE: 98, EADDRNOTAVAIL: 99, EAFNOSUPPORT: 97, EAGAIN: 11, EALREADY: 114, EBADF: 9, EBADMSG: 74, EBUSY: 16, ECANCELED: 125, ECHILD: 10, ECONNABORTED: 103, ECONNREFUSED: 111, ECONNRESET: 104, EDEADLK: 35, EDESTADDRREQ: 89, EDOM: 33, EDQUOT: 122, EEXIST: 17, EFAULT: 14, EFBIG: 27, EHOSTUNREACH: 113, EIDRM: 43, EILSEQ: 84, EINPROGRESS: 115, EINTR: 4, EINVAL: 22, EIO: 5, EISCONN: 106, EISDIR: 21, ELOOP: 40, EMFILE: 24, EMLINK: 31, EMSGSIZE: 90, EMULTIHOP: 72, ENAMETOOLONG: 36, ENETDOWN: 100, ENETRESET: 102, ENETUNREACH: 101, ENFILE: 23, ENOBUFS: 105, ENODATA: 61, ENODEV: 19, ENOENT: 2, ENOEXEC: 8, ENOLCK: 37, ENOLINK: 67, ENOMEM: 12, ENOMSG: 42, ENOPROTOOPT: 92, ENOSPC: 28, ENOSR: 63, ENOSTR: 60, ENOSYS: 38, ENOTCONN: 107, ENOTDIR: 20, ENOTEMPTY: 39, ENOTRECOVERABLE: 131, ENOTSOCK: 88, ENOTSUP: 95, ENOTTY: 25, ENXIO: 6, EOVERFLOW: 75, EOWNERDEAD: 130, EPERM: 1, EPIPE: 32, EPROTO: 71, EPROTONOSUPPORT: 93, EPROTOTYPE: 91, ERANGE: 34, EROFS: 30, ESPIPE: 29, ESRCH: 3, ESTALE: 116, ETIME: 62, ETIMEDOUT: 110, ETXTBSY: 26, EWOULDBLOCK: 11, EXDEV: 18 };
  
  function ___setErrNo(value) {
      // For convenient setting and returning of errno.
      if (!___setErrNo.ret) ___setErrNo.ret = allocate([0], 'i32', ALLOC_STATIC);
      HEAP32[((___setErrNo.ret)>>2)]=value;
      return value;
    }
  
  var _stdin=0;
  
  var _stdout=0;
  
  var _stderr=0;
  
  var __impure_ptr=0;
  var FS={currentPath: "/", nextInode: 2, streams: [null], ignorePermissions: true, absolutePath: function (relative, base) {
        if (typeof relative !== 'string') return null;
        if (typeof base === 'undefined') base = FS.currentPath;
        if (relative && relative[0] == '/') base = '';
        var full = base + '/' + relative;
        var parts = full.split('/').reverse();
        var absolute = [''];
        while (parts.length) {
          var part = parts.pop();
          if (part == '' || part == '.') {
            // Nothing.
          } else if (part == '..') {
            if (absolute.length > 1) absolute.pop();
          } else {
            absolute.push(part);
          }
        }
        return absolute.length == 1 ? '/' : absolute.join('/');
      }, analyzePath: function (path, dontResolveLastLink, linksVisited) {
        var ret = {
          isRoot: false,
          exists: false,
          error: 0,
          name: null,
          path: null,
          object: null,
          parentExists: false,
          parentPath: null,
          parentObject: null
        };
        path = FS.absolutePath(path);
        if (path == '/') {
          ret.isRoot = true;
          ret.exists = ret.parentExists = true;
          ret.name = '/';
          ret.path = ret.parentPath = '/';
          ret.object = ret.parentObject = FS.root;
        } else if (path !== null) {
          linksVisited = linksVisited || 0;
          path = path.slice(1).split('/');
          var current = FS.root;
          var traversed = [''];
          while (path.length) {
            if (path.length == 1 && current.isFolder) {
              ret.parentExists = true;
              ret.parentPath = traversed.length == 1 ? '/' : traversed.join('/');
              ret.parentObject = current;
              ret.name = path[0];
            }
            var target = path.shift();
            if (!current.isFolder) {
              ret.error = ERRNO_CODES.ENOTDIR;
              break;
            } else if (!current.read) {
              ret.error = ERRNO_CODES.EACCES;
              break;
            } else if (!current.contents.hasOwnProperty(target)) {
              ret.error = ERRNO_CODES.ENOENT;
              break;
            }
            current = current.contents[target];
            if (current.link && !(dontResolveLastLink && path.length == 0)) {
              if (linksVisited > 40) { // Usual Linux SYMLOOP_MAX.
                ret.error = ERRNO_CODES.ELOOP;
                break;
              }
              var link = FS.absolutePath(current.link, traversed.join('/'));
              return FS.analyzePath([link].concat(path).join('/'),
                                    dontResolveLastLink, linksVisited + 1);
            }
            traversed.push(target);
            if (path.length == 0) {
              ret.exists = true;
              ret.path = traversed.join('/');
              ret.object = current;
            }
          }
          return ret;
        }
        return ret;
      }, findObject: function (path, dontResolveLastLink) {
        FS.ensureRoot();
        var ret = FS.analyzePath(path, dontResolveLastLink);
        if (ret.exists) {
          return ret.object;
        } else {
          ___setErrNo(ret.error);
          return null;
        }
      }, createObject: function (parent, name, properties, canRead, canWrite) {
        if (!parent) parent = '/';
        if (typeof parent === 'string') parent = FS.findObject(parent);
  
        if (!parent) {
          ___setErrNo(ERRNO_CODES.EACCES);
          throw new Error('Parent path must exist.');
        }
        if (!parent.isFolder) {
          ___setErrNo(ERRNO_CODES.ENOTDIR);
          throw new Error('Parent must be a folder.');
        }
        if (!parent.write && !FS.ignorePermissions) {
          ___setErrNo(ERRNO_CODES.EACCES);
          throw new Error('Parent folder must be writeable.');
        }
        if (!name || name == '.' || name == '..') {
          ___setErrNo(ERRNO_CODES.ENOENT);
          throw new Error('Name must not be empty.');
        }
        if (parent.contents.hasOwnProperty(name)) {
          ___setErrNo(ERRNO_CODES.EEXIST);
          throw new Error("Can't overwrite object.");
        }
  
        parent.contents[name] = {
          read: (typeof canRead === 'undefined')? true : canRead,
          write: (typeof canWrite === 'undefined')? false : canWrite,
          timestamp: Date.now(),
          inodeNumber: FS.nextInode++
        };
        for (var key in properties) {
          if (properties.hasOwnProperty(key)) {
            parent.contents[name][key] = properties[key];
          }
        }
  
        return parent.contents[name];
      }, createFolder: function (parent, name, canRead, canWrite) {
        var properties = {isFolder: true, isDevice: false, contents: {}};
        return FS.createObject(parent, name, properties, canRead, canWrite);
      }, createPath: function (parent, path, canRead, canWrite) {
        var current = FS.findObject(parent);
        if (current === null) throw new Error('Invalid parent.');
        path = path.split('/').reverse();
        while (path.length) {
          var part = path.pop();
          if (!part) continue;
          if (!current.contents.hasOwnProperty(part)) {
            FS.createFolder(current, part, canRead, canWrite);
          }
          current = current.contents[part];
        }
        return current;
      }, createFile: function (parent, name, properties, canRead, canWrite) {
        properties.isFolder = false;
        return FS.createObject(parent, name, properties, canRead, canWrite);
      }, createDataFile: function (parent, name, data, canRead, canWrite) {
        if (typeof data === 'string') {
          var dataArray = [];
          for (var i = 0; i < data.length; i++) dataArray.push(data.charCodeAt(i));
          data = dataArray;
        }
        var properties = {isDevice: false, contents: data};
        return FS.createFile(parent, name, properties, canRead, canWrite);
      }, createLazyFile: function (parent, name, url, canRead, canWrite) {
        var properties = {isDevice: false, url: url};
        return FS.createFile(parent, name, properties, canRead, canWrite);
      }, createLink: function (parent, name, target, canRead, canWrite) {
        var properties = {isDevice: false, link: target};
        return FS.createFile(parent, name, properties, canRead, canWrite);
      }, createDevice: function (parent, name, input, output) {
        if (!(input || output)) {
          throw new Error('A device must have at least one callback defined.');
        }
        var ops = {isDevice: true, input: input, output: output};
        return FS.createFile(parent, name, ops, Boolean(input), Boolean(output));
      }, forceLoadFile: function (obj) {
        if (obj.isDevice || obj.isFolder || obj.link || obj.contents) return true;
        // meSpeak: deleted lines for external loading (see frontend)
        ___setErrNo(ERRNO_CODES.EIO);
        return false;
      }, ensureRoot: function () {
        if (FS.root) return;
        // The main file system tree. All the contents are inside this.
        FS.root = {
          read: true,
          write: false,
          isFolder: true,
          isDevice: false,
          timestamp: Date.now(),
          inodeNumber: 1,
          contents: {}
        };
      }, init: function (input, output, error) {
        // Make sure we initialize only once.
        if (FS.init.initialized) return;
        FS.init.initialized = true;
  
        FS.ensureRoot();
  
        // Default handlers.
        if (!input) input = function() {
          if (!input.cache || !input.cache.length) {
            var result;
            // meSpeak: skip code for input
            /*
            if (typeof window != 'undefined' &&
                typeof window.prompt == 'function') {
              // Browser.
              result = window.prompt('Input: ');
            } else 
            */
            if (typeof readline == 'function') {
              // Command line.
              result = readline();
            }
            if (!result) result = '';
            input.cache = intArrayFromString(result + '\n', true);
            input.cache = intArrayFromString('\n', true);
          }
          return input.cache.shift();
        };
        if (!output) output = function(val) {
          if (val === null || val === '\n'.charCodeAt(0)) {
            output.printer(output.buffer.join(''));
            output.buffer = [];
          } else {
            output.buffer.push(String.fromCharCode(val));
          }
        };
        if (!output.printer) output.printer = vprint;
        if (!output.buffer) output.buffer = [];
        if (!error) error = output;
  
        // Create the temporary folder.
        FS.createFolder('/', 'tmp', true, true);
  
        // Create the I/O devices.
        var devFolder = FS.createFolder('/', 'dev', true, false);
        var stdin = FS.createDevice(devFolder, 'stdin', input);
        var stdout = FS.createDevice(devFolder, 'stdout', null, output);
        var stderr = FS.createDevice(devFolder, 'stderr', null, error);
        FS.createDevice(devFolder, 'tty', input, output);
  
        // Create default streams.
        FS.streams[1] = {
          path: '/dev/stdin',
          object: stdin,
          position: 0,
          isRead: true,
          isWrite: false,
          isAppend: false,
          error: false,
          eof: false,
          ungotten: []
        };
        FS.streams[2] = {
          path: '/dev/stdout',
          object: stdout,
          position: 0,
          isRead: false,
          isWrite: true,
          isAppend: false,
          error: false,
          eof: false,
          ungotten: []
        };
        FS.streams[3] = {
          path: '/dev/stderr',
          object: stderr,
          position: 0,
          isRead: false,
          isWrite: true,
          isAppend: false,
          error: false,
          eof: false,
          ungotten: []
        };
        _stdin = allocate([1], 'void*', ALLOC_STATIC);
        _stdout = allocate([2], 'void*', ALLOC_STATIC);
        _stderr = allocate([3], 'void*', ALLOC_STATIC);
  
        // Newlib initialization
        FS.streams[_stdin] = FS.streams[1];
        FS.streams[_stdout] = FS.streams[2];
        FS.streams[_stderr] = FS.streams[3];
        __impure_ptr = allocate([ allocate(
          [0, 0, 0, 0, _stdin, 0, 0, 0, _stdout, 0, 0, 0, _stderr, 0, 0, 0],
          'void*', ALLOC_STATIC) ], 'void*', ALLOC_STATIC);
  
        // Once initialized, permissions start having effect.
        FS.ignorePermissions = false;
      }, quit: function () {
        // Flush any partially-printed lines in stdout and stderr
        try {
          if (FS.streams[2].object.output.buffer.length > 0) FS.streams[2].object.output('\n'.charCodeAt(0));
          if (FS.streams[3].object.output.buffer.length > 0) FS.streams[3].object.output('\n'.charCodeAt(0));
        }
        catch(e) {
          recoverFromFSError(e);
        }
      } };
  
  var ___stat_struct_layout={__size__: 80, st_dev: 0, st_ino: 8, st_mode: 12, st_nlink: 16, st_uid: 20, st_gid: 24, st_rdev: 28, st_size: 36, st_atime: 40, st_spare1: 44, st_mtime: 48, st_spare2: 52, st_ctime: 56, st_spare3: 60, st_blksize: 64, st_blocks: 68, st_spare4: 72 };function _stat(path, buf, dontResolveLastLink) {
      // http://pubs.opengroup.org/onlinepubs/7908799/xsh/stat.html
      // int stat(const char *path, struct stat *buf);
      // NOTE: dontResolveLastLink is a shortcut for lstat(). It should never be
      //       used in client code.
      var obj = FS.findObject(Pointer_stringify(path), dontResolveLastLink);
      if (obj === null || !FS.forceLoadFile(obj)) return -1;
  
      var offsets = ___stat_struct_layout;
  
      // Constants.
      HEAP32[((buf+offsets.st_nlink)>>2)]=1;
      HEAP32[((buf+offsets.st_uid)>>2)]=0;
      HEAP32[((buf+offsets.st_gid)>>2)]=0;
      HEAP32[((buf+offsets.st_blksize)>>2)]=4096;
  
      // Variables.
      HEAP32[((buf+offsets.st_ino)>>2)]=obj.inodeNumber;
      var time = Math.floor(obj.timestamp / 1000);
      if (typeof offsets.st_atime === 'undefined') {
        offsets.st_atime = offsets.st_atim.tv_sec;
        offsets.st_mtime = offsets.st_mtim.tv_sec;
        offsets.st_ctime = offsets.st_ctim.tv_sec;
        var nanosec = (obj.timestamp % 1000) * 1000;
        HEAP32[((buf+offsets.st_atim.tv_nsec)>>2)]=nanosec;
        HEAP32[((buf+offsets.st_mtim.tv_nsec)>>2)]=nanosec;
        HEAP32[((buf+offsets.st_ctim.tv_nsec)>>2)]=nanosec;
      }
      HEAP32[((buf+offsets.st_atime)>>2)]=time;
      HEAP32[((buf+offsets.st_mtime)>>2)]=time;
      HEAP32[((buf+offsets.st_ctime)>>2)]=time;
      var mode = 0;
      var size = 0;
      var blocks = 0;
      var dev = 0;
      var rdev = 0;
      if (obj.isDevice) {
        //  Device numbers reuse inode numbers.
        dev = rdev = obj.inodeNumber;
        size = blocks = 0;
        mode = 0x2000;  // S_IFCHR.
      } else {
        dev = 1;
        rdev = 0;
        // NOTE: In our implementation, st_blocks = Math.ceil(st_size/st_blksize),
        //       but this is not required by the standard.
        if (obj.isFolder) {
          size = 4096;
          blocks = 1;
          mode = 0x4000;  // S_IFDIR.
        } else {
          var data = obj.contents || obj.link;
          size = data.length;
          blocks = Math.ceil(data.length / 4096);
          mode = (typeof obj.link === 'undefined')? 0x8000 : 0xA000;  // S_IFREG, S_IFLNK.
        }
      }
      HEAP32[((buf+offsets.st_dev)>>2)]=dev;
      HEAP32[((buf+offsets.st_rdev)>>2)]=rdev;
      // NOTE: These two may be i64, depending on compilation options.
      HEAP32[((buf+offsets.st_size)>>2)]=size;
      HEAP32[((buf+offsets.st_blocks)>>2)]=blocks;
      if (obj.read) mode |= 0x16D;  // S_IRUSR | S_IXUSR | S_IRGRP | S_IXGRP | S_IROTH | S_IXOTH.
      if (obj.write) mode |= 0x92;  // S_IWUSR | S_IWGRP | S_IWOTH.
      HEAP32[((buf+offsets.st_mode)>>2)]=mode;
  
      return 0;
    }

  function _malloc(size) { var ret = STATICTOP; STATICTOP += size;STATICTOP = Math.ceil((STATICTOP)/4)*4;; return ret; }

  
  
  function _pwrite(fildes, buf, nbyte, offset) {
      // ssize_t pwrite(int fildes, const void *buf, size_t nbyte, off_t offset);
      // http://pubs.opengroup.org/onlinepubs/000095399/functions/write.html
      var stream = FS.streams[fildes];
      if (!stream || stream.object.isDevice) {
        ___setErrNo(ERRNO_CODES.EBADF);
        return -1;
      } else if (!stream.isWrite) {
        ___setErrNo(ERRNO_CODES.EACCES);
        return -1;
      } else if (stream.object.isFolder) {
        ___setErrNo(ERRNO_CODES.EISDIR);
        return -1;
      } else if (nbyte < 0 || offset < 0) {
        ___setErrNo(ERRNO_CODES.EINVAL);
        return -1;
      } else {
        var contents = stream.object.contents;
        while (contents.length < offset) contents.push(0);
        for (var i = 0; i < nbyte; i++) {
          contents[offset + i] = HEAP8[(buf+i)];
        }
        stream.object.timestamp = Date.now();
        return i;
      }
    }function _write(fildes, buf, nbyte) {
      // ssize_t write(int fildes, const void *buf, size_t nbyte);
      // http://pubs.opengroup.org/onlinepubs/000095399/functions/write.html
      var stream = FS.streams[fildes];
      if (!stream) {
        ___setErrNo(ERRNO_CODES.EBADF);
        return -1;
      } else if (!stream.isWrite) {
        ___setErrNo(ERRNO_CODES.EACCES);
        return -1;
      } else if (nbyte < 0) {
        ___setErrNo(ERRNO_CODES.EINVAL);
        return -1;
      } else {
        if (stream.object.isDevice) {
          if (stream.object.output) {
            for (var i = 0; i < nbyte; i++) {
              try {
                stream.object.output(HEAP8[(buf+i)]);
              } catch (e) {
                ___setErrNo(ERRNO_CODES.EIO);
                return -1;
              }
            }
            stream.object.timestamp = Date.now();
            return i;
          } else {
            ___setErrNo(ERRNO_CODES.ENXIO);
            return -1;
          }
        } else {
          var bytesWritten = _pwrite(fildes, buf, nbyte, stream.position);
          if (bytesWritten != -1) stream.position += bytesWritten;
          return bytesWritten;
        }
      }
    }function _fwrite(ptr, size, nitems, stream) {
      // size_t fwrite(const void *restrict ptr, size_t size, size_t nitems, FILE *restrict stream);
      // http://pubs.opengroup.org/onlinepubs/000095399/functions/fwrite.html
      var bytesToWrite = nitems * size;
      if (bytesToWrite == 0) return 0;
      var bytesWritten = _write(stream, ptr, bytesToWrite);
      if (bytesWritten == -1) {
        if (FS.streams[stream]) FS.streams[stream].error = true;
        return -1;
      } else {
        return Math.floor(bytesWritten / size);
      }
    }
  
  function __formatString(format, varargs) {
      var textIndex = format;
      var argIndex = 0;
      var getNextArg = function(type) {
        // NOTE: Explicitly ignoring type safety. Otherwise this fails:
        //       int x = 4; printf("%c\n", (char)x);
        var ret;
        if (type === 'float' || type === 'double') {
          ret = HEAPF32[((varargs+argIndex)>>2)];
        } else {
          ret = HEAP32[((varargs+argIndex)>>2)];
        }
        argIndex += Runtime.getNativeFieldSize(type);
        return Number(ret);
      };
  
      var ret = [];
      var curr, next, currArg;
      while(1) {
        var startTextIndex = textIndex;
        curr = HEAP8[(textIndex)];
        if (curr === 0) break;
        next = HEAP8[(textIndex+1)];
        if (curr == '%'.charCodeAt(0)) {
          // Handle flags.
          var flagAlwaysSigned = false;
          var flagLeftAlign = false;
          var flagAlternative = false;
          var flagZeroPad = false;
          flagsLoop: while (1) {
            switch (next) {
              case '+'.charCodeAt(0):
                flagAlwaysSigned = true;
                break;
              case '-'.charCodeAt(0):
                flagLeftAlign = true;
                break;
              case '#'.charCodeAt(0):
                flagAlternative = true;
                break;
              case '0'.charCodeAt(0):
                if (flagZeroPad) {
                  break flagsLoop;
                } else {
                  flagZeroPad = true;
                  break;
                }
              default:
                break flagsLoop;
            }
            textIndex++;
            next = HEAP8[(textIndex+1)];
          }
  
          // Handle width.
          var width = 0;
          if (next == '*'.charCodeAt(0)) {
            width = getNextArg('i32');
            textIndex++;
            next = HEAP8[(textIndex+1)];
          } else {
            while (next >= '0'.charCodeAt(0) && next <= '9'.charCodeAt(0)) {
              width = width * 10 + (next - '0'.charCodeAt(0));
              textIndex++;
              next = HEAP8[(textIndex+1)];
            }
          }
  
          // Handle precision.
          var precisionSet = false;
          if (next == '.'.charCodeAt(0)) {
            var precision = 0;
            precisionSet = true;
            textIndex++;
            next = HEAP8[(textIndex+1)];
            if (next == '*'.charCodeAt(0)) {
              precision = getNextArg('i32');
              textIndex++;
            } else {
              while(1) {
                var precisionChr = HEAP8[(textIndex+1)];
                if (precisionChr < '0'.charCodeAt(0) ||
                    precisionChr > '9'.charCodeAt(0)) break;
                precision = precision * 10 + (precisionChr - '0'.charCodeAt(0));
                textIndex++;
              }
            }
            next = HEAP8[(textIndex+1)];
          } else {
            var precision = 6; // Standard default.
          }
  
          // Handle integer sizes. WARNING: These assume a 32-bit architecture!
          var argSize;
          switch (String.fromCharCode(next)) {
            case 'h':
              var nextNext = HEAP8[(textIndex+2)];
              if (nextNext == 'h'.charCodeAt(0)) {
                textIndex++;
                argSize = 1; // char
              } else {
                argSize = 2; // short
              }
              break;
            case 'l':
              var nextNext = HEAP8[(textIndex+2)];
              if (nextNext == 'l'.charCodeAt(0)) {
                textIndex++;
                argSize = 8; // long long
              } else {
                argSize = 4; // long
              }
              break;
            case 'L': // long long
            case 'q': // int64_t
            case 'j': // intmax_t
              argSize = 8;
              break;
            case 'z': // size_t
            case 't': // ptrdiff_t
            case 'I': // signed ptrdiff_t or unsigned size_t
              argSize = 4;
              break;
            default:
              argSize = undefined;
          }
          if (typeof argSize !== 'undefined') textIndex++;
          next = HEAP8[(textIndex+1)];
  
          // Handle type specifier.
          if (['d', 'i', 'u', 'o', 'x', 'X', 'p'].indexOf(String.fromCharCode(next)) != -1) {
            // Integer.
            var signed = next == 'd'.charCodeAt(0) || next == 'i'.charCodeAt(0);
            argSize = argSize || 4;
            var currArg = getNextArg('i' + (argSize * 8));
            // Truncate to requested size.
            if (argSize <= 4) {
              var limit = Math.pow(256, argSize) - 1;
              currArg = (signed ? reSign : unSign)(currArg & limit, argSize * 8);
            }
            // Format the number.
            var currAbsArg = Math.abs(currArg);
            var argText;
            var prefix = '';
            if (next == 'd'.charCodeAt(0) || next == 'i'.charCodeAt(0)) {
              argText = reSign(currArg, 8 * argSize, 1).toString(10);
            } else if (next == 'u'.charCodeAt(0)) {
              argText = unSign(currArg, 8 * argSize, 1).toString(10);
              currArg = Math.abs(currArg);
            } else if (next == 'o'.charCodeAt(0)) {
              argText = (flagAlternative ? '0' : '') + currAbsArg.toString(8);
            } else if (next == 'x'.charCodeAt(0) || next == 'X'.charCodeAt(0)) {
              prefix = flagAlternative ? '0x' : '';
              if (currArg < 0) {
                // Represent negative numbers in hex as 2's complement.
                currArg = -currArg;
                argText = (currAbsArg - 1).toString(16);
                var buffer = [];
                for (var i = 0; i < argText.length; i++) {
                  buffer.push((0xF - parseInt(argText[i], 16)).toString(16));
                }
                argText = buffer.join('');
                while (argText.length < argSize * 2) argText = 'f' + argText;
              } else {
                argText = currAbsArg.toString(16);
              }
              if (next == 'X'.charCodeAt(0)) {
                prefix = prefix.toUpperCase();
                argText = argText.toUpperCase();
              }
            } else if (next == 'p'.charCodeAt(0)) {
              if (currAbsArg === 0) {
                argText = '(nil)';
              } else {
                prefix = '0x';
                argText = currAbsArg.toString(16);
              }
            }
            if (precisionSet) {
              while (argText.length < precision) {
                argText = '0' + argText;
              }
            }
  
            // Add sign if needed
            if (flagAlwaysSigned) {
              if (currArg < 0) {
                prefix = '-' + prefix;
              } else {
                prefix = '+' + prefix;
              }
            }
  
            // Add padding.
            while (prefix.length + argText.length < width) {
              if (flagLeftAlign) {
                argText += ' ';
              } else {
                if (flagZeroPad) {
                  argText = '0' + argText;
                } else {
                  prefix = ' ' + prefix;
                }
              }
            }
  
            // Insert the result into the buffer.
            argText = prefix + argText;
            argText.split('').forEach(function(chr) {
              ret.push(chr.charCodeAt(0));
            });
          } else if (['f', 'F', 'e', 'E', 'g', 'G'].indexOf(String.fromCharCode(next)) != -1) {
            // Float.
            var currArg = getNextArg(argSize === 4 ? 'float' : 'double');
            var argText;
  
            if (isNaN(currArg)) {
              argText = 'nan';
              flagZeroPad = false;
            } else if (!isFinite(currArg)) {
              argText = (currArg < 0 ? '-' : '') + 'inf';
              flagZeroPad = false;
            } else {
              var isGeneral = false;
              var effectivePrecision = Math.min(precision, 20);
  
              // Convert g/G to f/F or e/E, as per:
              // http://pubs.opengroup.org/onlinepubs/9699919799/functions/printf.html
              if (next == 'g'.charCodeAt(0) || next == 'G'.charCodeAt(0)) {
                isGeneral = true;
                precision = precision || 1;
                var exponent = parseInt(currArg.toExponential(effectivePrecision).split('e')[1], 10);
                if (precision > exponent && exponent >= -4) {
                  next = ((next == 'g'.charCodeAt(0)) ? 'f' : 'F').charCodeAt(0);
                  precision -= exponent + 1;
                } else {
                  next = ((next == 'g'.charCodeAt(0)) ? 'e' : 'E').charCodeAt(0);
                  precision--;
                }
                effectivePrecision = Math.min(precision, 20);
              }
  
              if (next == 'e'.charCodeAt(0) || next == 'E'.charCodeAt(0)) {
                argText = currArg.toExponential(effectivePrecision);
                // Make sure the exponent has at least 2 digits.
                if (/[eE][-+]\d$/.test(argText)) {
                  argText = argText.slice(0, -1) + '0' + argText.slice(-1);
                }
              } else if (next == 'f'.charCodeAt(0) || next == 'F'.charCodeAt(0)) {
                argText = currArg.toFixed(effectivePrecision);
              }
  
              var parts = argText.split('e');
              if (isGeneral && !flagAlternative) {
                // Discard trailing zeros and periods.
                while (parts[0].length > 1 && parts[0].indexOf('.') != -1 &&
                       (parts[0].slice(-1) == '0' || parts[0].slice(-1) == '.')) {
                  parts[0] = parts[0].slice(0, -1);
                }
              } else {
                // Make sure we have a period in alternative mode.
                if (flagAlternative && argText.indexOf('.') == -1) parts[0] += '.';
                // Zero pad until required precision.
                while (precision > effectivePrecision++) parts[0] += '0';
              }
              argText = parts[0] + (parts.length > 1 ? 'e' + parts[1] : '');
  
              // Capitalize 'E' if needed.
              if (next == 'E'.charCodeAt(0)) argText = argText.toUpperCase();
  
              // Add sign.
              if (flagAlwaysSigned && currArg >= 0) {
                argText = '+' + argText;
              }
            }
  
            // Add padding.
            while (argText.length < width) {
              if (flagLeftAlign) {
                argText += ' ';
              } else {
                if (flagZeroPad && (argText[0] == '-' || argText[0] == '+')) {
                  argText = argText[0] + '0' + argText.slice(1);
                } else {
                  argText = (flagZeroPad ? '0' : ' ') + argText;
                }
              }
            }
  
            // Adjust case.
            if (next < 'a'.charCodeAt(0)) argText = argText.toUpperCase();
  
            // Insert the result into the buffer.
            argText.split('').forEach(function(chr) {
              ret.push(chr.charCodeAt(0));
            });
          } else if (next == 's'.charCodeAt(0)) {
            // String.
            var arg = getNextArg('i8*');
            var copiedString;
            if (arg) {
              copiedString = String_copy(arg);
              if (precisionSet && copiedString.length > precision) {
                copiedString = copiedString.slice(0, precision);
              }
            } else {
              copiedString = intArrayFromString('(null)', true);
            }
            if (!flagLeftAlign) {
              while (copiedString.length < width--) {
                ret.push(' '.charCodeAt(0));
              }
            }
            ret = ret.concat(copiedString);
            if (flagLeftAlign) {
              while (copiedString.length < width--) {
                ret.push(' '.charCodeAt(0));
              }
            }
          } else if (next == 'c'.charCodeAt(0)) {
            // Character.
            if (flagLeftAlign) ret.push(getNextArg('i8'));
            while (--width > 0) {
              ret.push(' '.charCodeAt(0));
            }
            if (!flagLeftAlign) ret.push(getNextArg('i8'));
          } else if (next == 'n'.charCodeAt(0)) {
            // Write the length written so far to the next parameter.
            var ptr = getNextArg('i32*');
            HEAP32[((ptr)>>2)]=ret.length;
          } else if (next == '%'.charCodeAt(0)) {
            // Literal percent sign.
            ret.push(curr);
          } else {
            // Unknown specifiers remain untouched.
            for (var i = startTextIndex; i < textIndex + 2; i++) {
              ret.push(HEAP8[(i)]);
            }
          }
          textIndex += 2;
          // TODO: Support a/A (hex float) and m (last error) specifiers.
          // TODO: Support %1${specifier} for arg selection.
        } else {
          ret.push(curr);
          textIndex += 1;
        }
      }
      return ret;
    }function _fprintf(stream, format, varargs) {
      // int fprintf(FILE *restrict stream, const char *restrict format, ...);
      // http://pubs.opengroup.org/onlinepubs/000095399/functions/printf.html
      var result = __formatString(format, varargs);
      var stack = Runtime.stackSave();
      var ret = _fwrite(allocate(result, 'i8', ALLOC_STACK), 1, result.length, stream);
      Runtime.stackRestore(stack);
      return ret;
    }

  function _free(){}

  function _strlen(ptr) {
      return String_len(ptr);
    }

  function _strcpy(pdest, psrc) {
      var i = 0;
      do {
        var $src$, $dest$, $stop$, $stop4$, $fast$;
  $src$ = psrc+i;
  $dest$ = pdest+i;
  $stop$ = $src$ + 1;
  if (($dest$%4) == ($src$%4) && 1 > 8) {
    while ($src$%4 !== 0 && $src$ < $stop$) {
      ; HEAP8[$dest$++] = HEAP8[$src$++];
    }
    $src$ >>= 2;
    $dest$ >>= 2;
    $stop4$ = $stop$ >> 2;
    while ($src$ < $stop4$) {
  ;;;    HEAP32[$dest$++] = HEAP32[$src$++];
    }
    $src$ <<= 2;
    $dest$ <<= 2;
  }while ($src$ < $stop$) {
    ; HEAP8[$dest$++] = HEAP8[$src$++];
  }
        i ++;
      } while (HEAP8[(psrc+i-1)] != 0);
      return pdest;
    }

  
  function _snprintf(s, n, format, varargs) {
      // int snprintf(char *restrict s, size_t n, const char *restrict format, ...);
      // http://pubs.opengroup.org/onlinepubs/000095399/functions/printf.html
      var result = __formatString(format, varargs);
      var limit = (typeof n === 'undefined') ? result.length
                                    : Math.min(result.length, n - 1);
      for (var i = 0; i < limit; i++) {
        HEAP8[(s+i)]=result[i];;
      }
      HEAP8[(s+i)]=0;;
      return result.length;
    }function _sprintf(s, format, varargs) {
      // int sprintf(char *restrict s, const char *restrict format, ...);
      // http://pubs.opengroup.org/onlinepubs/000095399/functions/printf.html
      return _snprintf(s, undefined, format, varargs);
    }

  
  function _fputc(c, stream) {
      // int fputc(int c, FILE *stream);
      // http://pubs.opengroup.org/onlinepubs/000095399/functions/fputc.html
      var chr = unSign(c & 0xFF);
      HEAP8[(_fputc.ret)]=chr;
      var ret = _write(stream, _fputc.ret, 1);
      if (ret == -1) {
        if (stream in FS.streams) FS.streams[stream].error = true;
        return -1;
      } else {
        return chr;
      }
    }

  
  function __scanString(format, get, unget, varargs) {
      // Supports %x, %4x, %d.%d, %s.
      // TODO: Support all format specifiers.
      format = Pointer_stringify(format);
      var formatIndex = 0;
      var argsi = 0;
      var fields = 0;
      var argIndex = 0;
      for (var formatIndex = 0; formatIndex < format.length; formatIndex++) {
        if (next <= 0) return fields;
        var next = get();
        if (next <= 0) return fields;  // End of input.
        if (format[formatIndex] === '%') {
          formatIndex++;
          var maxSpecifierStart = formatIndex;
          while (format[formatIndex].charCodeAt(0) >= '0'.charCodeAt(0) &&
                 format[formatIndex].charCodeAt(0) <= '9'.charCodeAt(0)) {
            formatIndex++;
          }
          var max_;
          if (formatIndex != maxSpecifierStart) {
            max_ = parseInt(format.slice(maxSpecifierStart, formatIndex), 10);
          }
          // TODO: Handle type size modifier.
          var type = format[formatIndex];
          formatIndex++;
          var curr = 0;
          var buffer = [];
          while ((curr < max_ || isNaN(max_)) && next > 0) {
            if ((type === 'd' && next >= '0'.charCodeAt(0) && next <= '9'.charCodeAt(0)) ||
                (type === 'x' && (next >= '0'.charCodeAt(0) && next <= '9'.charCodeAt(0) ||
                                  next >= 'a'.charCodeAt(0) && next <= 'f'.charCodeAt(0) ||
                                  next >= 'A'.charCodeAt(0) && next <= 'F'.charCodeAt(0))) ||
                (type === 's') &&
                (formatIndex >= format.length || next !== format[formatIndex].charCodeAt(0))) { // Stop when we read something that is coming up
              buffer.push(String.fromCharCode(next));
              next = get();
              curr++;
            } else {
              break;
            }
          }
          if (buffer.length === 0) return 0;  // Failure.
          var text = buffer.join('');
          var argPtr = HEAP32[((varargs+argIndex)>>2)];
          argIndex += Runtime.getNativeFieldSize('void*');
          switch (type) {
            case 'd':
              HEAP32[((argPtr)>>2)]=parseInt(text, 10);
              break;
            case 'x':
              HEAP32[((argPtr)>>2)]=parseInt(text, 16);
              break;
            case 's':
              var array = intArrayFromString(text);
              for (var j = 0; j < array.length; j++) {
                HEAP8[(argPtr+j)]=array[j];
              }
              break;
          }
          fields++;
        } else {
          // Not a specifier.
          if (format[formatIndex].charCodeAt(0) !== next) {
            unget(next);
            return fields;
          }
        }
      }
      return fields;
    }function _sscanf(s, format, varargs) {
      // int sscanf(const char *restrict s, const char *restrict format, ... );
      // http://pubs.opengroup.org/onlinepubs/000095399/functions/scanf.html
      var index = 0;
      var get = function() { return HEAP8[(s+index++)]; };
      var unget = function() { index--; };
      return __scanString(format, get, unget, varargs);
    }

  function _printf(format, varargs) {
      // int printf(const char *restrict format, ...);
      // http://pubs.opengroup.org/onlinepubs/000095399/functions/printf.html
      var stdout = HEAP32[((_stdout)>>2)];
      return _fprintf(stdout, format, varargs);
    }

  
  function __exit(status) {
      // void _exit(int status);
      // http://pubs.opengroup.org/onlinepubs/000095399/functions/exit.html
      __shutdownRuntime__();
      ABORT = true;
      throw 'exit(' + status + ') called, at ' + new Error().stack;
    }function _exit(status) {
      __exit(status);
    }

  
  function _isspace(chr) {
      return chr in { 32: 0, 9: 0, 10: 0, 11: 0, 12: 0, 13: 0 };
    }
  
  function _isdigit(chr) {
      return chr >= '0'.charCodeAt(0) && chr <= '9'.charCodeAt(0);
    }function _atoi(s) {
      var c;
      while ((c = HEAP8[(s)]) && _isspace(c)) s++;
      if (!c || !_isdigit(c)) return 0;
      var e = s;
      while ((c = HEAP8[(e)]) && _isdigit(c)) e++;
      return Math.floor(Number(Pointer_stringify(s).substr(0, e-s)));
    }

  
  
  
  
  
  
  
  
  var ___dirent_struct_layout={__size__: 1040, d_ino: 0, d_name: 4, d_off: 1028, d_reclen: 1032, d_type: 1036 };function _open(path, oflag, varargs) {
      // int open(const char *path, int oflag, ...);
      // http://pubs.opengroup.org/onlinepubs/009695399/functions/open.html
      // NOTE: This implementation tries to mimic glibc rather that strictly
      // following the POSIX standard.
  
      var mode = HEAP32[((varargs)>>2)];
  
      // Simplify flags.
      var accessMode = oflag & 3;
      var isWrite = accessMode != 0;
      var isRead = accessMode != 1;
      var isCreate = Boolean(oflag & 512);
      var isExistCheck = Boolean(oflag & 2048);
      var isTruncate = Boolean(oflag & 1024);
      var isAppend = Boolean(oflag & 8);
  
      // Verify path.
      var origPath = path;
      path = FS.analyzePath(Pointer_stringify(path));
      if (!path.parentExists) {
        ___setErrNo(path.error);
        return -1;
      }
      var target = path.object || null;
      var finalPath;
  
      // Verify the file exists, create if needed and allowed.
      if (target) {
        if (isCreate && isExistCheck) {
          ___setErrNo(ERRNO_CODES.EEXIST);
          return -1;
        }
        if ((isWrite || isCreate || isTruncate) && target.isFolder) {
          ___setErrNo(ERRNO_CODES.EISDIR);
          return -1;
        }
        if (isRead && !target.read || isWrite && !target.write) {
          ___setErrNo(ERRNO_CODES.EACCES);
          return -1;
        }
        if (isTruncate && !target.isDevice) {
          target.contents = [];
        } else {
          if (!FS.forceLoadFile(target)) {
            ___setErrNo(ERRNO_CODES.EIO);
            return -1;
          }
        }
        finalPath = path.path;
      } else {
        if (!isCreate) {
          ___setErrNo(ERRNO_CODES.ENOENT);
          return -1;
        }
        if (!path.parentObject.write) {
          ___setErrNo(ERRNO_CODES.EACCES);
          return -1;
        }
        target = FS.createDataFile(path.parentObject, path.name, [],
                                   mode & 0x100, mode & 0x80);  // S_IRUSR, S_IWUSR.
        finalPath = path.parentPath + '/' + path.name;
      }
      // Actually create an open stream.
      var id = FS.streams.length;
      if (target.isFolder) {
        var entryBuffer = 0;
        if (___dirent_struct_layout) {
          entryBuffer = _malloc(___dirent_struct_layout.__size__);
        }
        var contents = [];
        for (var key in target.contents) contents.push(key);
        FS.streams[id] = {
          path: finalPath,
          object: target,
          // An index into contents. Special values: -2 is ".", -1 is "..".
          position: -2,
          isRead: true,
          isWrite: false,
          isAppend: false,
          error: false,
          eof: false,
          ungotten: [],
          // Folder-specific properties:
          // Remember the contents at the time of opening in an array, so we can
          // seek between them relying on a single order.
          contents: contents,
          // Each stream has its own area for readdir() returns.
          currentEntry: entryBuffer
        };
      } else {
        FS.streams[id] = {
          path: finalPath,
          object: target,
          position: 0,
          isRead: isRead,
          isWrite: isWrite,
          isAppend: isAppend,
          error: false,
          eof: false,
          ungotten: []
        };
      }
      return id;
    }function _fopen(filename, mode) {
      // FILE *fopen(const char *restrict filename, const char *restrict mode);
      // http://pubs.opengroup.org/onlinepubs/000095399/functions/fopen.html
      var flags;
      mode = Pointer_stringify(mode);
      if (mode[0] == 'r') {
        if (mode.indexOf('+') != -1) {
          flags = 2;
        } else {
          flags = 0;
        }
      } else if (mode[0] == 'w') {
        if (mode.indexOf('+') != -1) {
          flags = 2;
        } else {
          flags = 1;
        }
        flags |= 512;
        flags |= 1024;
      } else if (mode[0] == 'a') {
        if (mode.indexOf('+') != -1) {
          flags = 2;
        } else {
          flags = 1;
        }
        flags |= 512;
        flags |= 8;
      } else {
        ___setErrNo(ERRNO_CODES.EINVAL);
        return 0;
      }
      var ret = _open(filename, flags, allocate([0x1FF, 0, 0, 0], 'i32', ALLOC_STACK));  // All creation permissions.
      return (ret == -1) ? 0 : ret;
    }

  
  function _memset(ptr, value, num) {
      var $dest$, $stop$, $stop4$, $fast$, $value4$;
  $dest$ = ptr;
  $stop$ = $dest$ + num;
  $value4$ = value;
  if ($value4$ < 0) $value4$ += 256;
  $value4$ = $value4$ + ($value4$<<8) + ($value4$<<16) + ($value4$*16777216);
  while ($dest$%4 !== 0 && $dest$ < $stop$) {
    ;; HEAP8[$dest$++] = value;
  }
  $dest$ >>= 2;
  $stop4$ = $stop$ >> 2;
  while ($dest$ < $stop4$) {
  ;;;;;;;  HEAP32[$dest$++] = $value4$;
  }
  $dest$ <<= 2;
  while ($dest$ < $stop$) {
    ;; HEAP8[$dest$++] = value;
  }
    }var _llvm_memset_p0i8_i32=_memset;

  function _strrchr(ptr, chr) {
      var ptr2 = ptr + _strlen(ptr);
      do {
        if (HEAP8[(ptr2)] == chr) return ptr2;
        ptr2--;
      } while (ptr2 >= ptr);
      return 0;
    }

  
  
  
  function _close(fildes) {
      // int close(int fildes);
      // http://pubs.opengroup.org/onlinepubs/000095399/functions/close.html
      if (FS.streams[fildes]) {
        if (FS.streams[fildes].currentEntry) {
          _free(FS.streams[fildes].currentEntry);
        }
        delete FS.streams[fildes];
        return 0;
      } else {
        ___setErrNo(ERRNO_CODES.EBADF);
        return -1;
      }
    }
  
  
  
  function _fsync(fildes) {
      // int fsync(int fildes);
      // http://pubs.opengroup.org/onlinepubs/000095399/functions/fsync.html
      if (FS.streams[fildes]) {
        // We write directly to the file system, so there's nothing to do here.
        return 0;
      } else {
        ___setErrNo(ERRNO_CODES.EBADF);
        return -1;
      }
    }function _fclose(stream) {
      // int fclose(FILE *stream);
      // http://pubs.opengroup.org/onlinepubs/000095399/functions/fclose.html
      _fsync(stream);
      return _close(stream);
    }

  
  
  function _fflush(stream) {
      // int fflush(FILE *stream);
      // http://pubs.opengroup.org/onlinepubs/000095399/functions/fflush.html
      var flush = function(filedes) {
        // Right now we write all data directly, except for output devices.
        if (filedes in FS.streams && FS.streams[filedes].object.output) {
          FS.streams[filedes].object.output(null);
        }
      };
      try {
        if (stream === 0) {
          for (var i in FS.streams) flush(i);
        } else {
          flush(stream);
        }
        return 0;
      } catch (e) {
        ___setErrNo(ERRNO_CODES.EIO);
        return -1;
      }
    }

  
  
  function _ftell(stream) {
      // long ftell(FILE *stream);
      // http://pubs.opengroup.org/onlinepubs/000095399/functions/ftell.html
      if (stream in FS.streams) {
        stream = FS.streams[stream];
        if (stream.object.isDevice) {
          ___setErrNo(ERRNO_CODES.ESPIPE);
          return -1;
        } else {
          return stream.position;
        }
      } else {
        ___setErrNo(ERRNO_CODES.EBADF);
        return -1;
      }
    }

  
  
  
  
  function _lseek(fildes, offset, whence) {
      // off_t lseek(int fildes, off_t offset, int whence);
      // http://pubs.opengroup.org/onlinepubs/000095399/functions/lseek.html
      if (FS.streams[fildes] && !FS.streams[fildes].isDevice) {
        var stream = FS.streams[fildes];
        var position = offset;
        if (whence === 1) {  // SEEK_CUR.
          position += stream.position;
        } else if (whence === 2) {  // SEEK_END.
          position += stream.object.contents.length;
        }
        if (position < 0) {
          ___setErrNo(ERRNO_CODES.EINVAL);
          return -1;
        } else {
          stream.ungotten = [];
          stream.position = position;
          return position;
        }
      } else {
        ___setErrNo(ERRNO_CODES.EBADF);
        return -1;
      }
    }function _fseek(stream, offset, whence) {
      // int fseek(FILE *stream, long offset, int whence);
      // http://pubs.opengroup.org/onlinepubs/000095399/functions/fseek.html
      var ret = _lseek(stream, offset, whence);
      if (ret == -1) {
        return -1;
      } else {
        FS.streams[stream].eof = false;
        return 0;
      }
    }


  
  function _strncmp(px, py, n) {
      var i = 0;
      while (i < n) {
        var x = HEAP8[(px+i)];
        var y = HEAP8[(py+i)];
        if (x == y && x == 0) return 0;
        if (x == 0) return -1;
        if (y == 0) return 1;
        if (x == y) {
          i ++;
          continue;
        } else {
          return x > y ? 1 : -1;
        }
      }
      return 0;
    }function _strcmp(px, py) {
      return _strncmp(px, py, TOTAL_MEMORY);
    }

  function _setlocale(category, locale) {
      if (!_setlocale.ret) _setlocale.ret = allocate([0], 'i8', ALLOC_NORMAL);
      return _setlocale.ret;
    }

  
  
  
  var _environ=null;
  
  var ___environ=null;function ___buildEnvironment(env) {
      // WARNING: Arbitrary limit!
      var MAX_ENV_VALUES = 64;
      var TOTAL_ENV_SIZE = 1024;
  
      // Statically allocate memory for the environment.
      var poolPtr;
      var envPtr;
      if (_environ === null) {
        // Set default values. Use string keys for Closure Compiler compatibility.
        ENV.USER = 'root';
        ENV.PATH = '/';
        ENV.PWD = '/';
        ENV.HOME = '/';
        ENV.LANG = 'en_US.UTF-8';
        ENV._ = './this.program';
        // Allocate memory.
        poolPtr = allocate(TOTAL_ENV_SIZE, 'i8', ALLOC_STATIC);
        envPtr = allocate(MAX_ENV_VALUES * 4,
                          'i8*', ALLOC_STATIC);
        HEAP32[((envPtr)>>2)]=poolPtr;
        _environ = allocate([envPtr], 'i8**', ALLOC_STATIC);
        // Set up global variable alias.
        ___environ = _environ;
      } else {
        envPtr = HEAP32[((_environ)>>2)];
        poolPtr = HEAP32[((envPtr)>>2)];
      }
  
      // Collect key=value lines.
      var strings = [];
      var totalSize = 0;
      for (var key in env) {
        if (typeof env[key] === 'string') {
          var line = key + '=' + env[key];
          strings.push(line);
          totalSize += line.length;
        }
      }
      if (totalSize > TOTAL_ENV_SIZE) {
        throw new Error('Environment size exceeded TOTAL_ENV_SIZE!');
      }
  
      // Make new.
      var ptrSize = 4;
      for (var i = 0; i < strings.length; i++) {
        var line = strings[i];
        for (var j = 0; j < line.length; j++) {
          HEAP8[(poolPtr+j)]=line.charCodeAt(j);
        }
        HEAP8[(poolPtr+j)]=0;
        HEAP32[((envPtr+i * ptrSize)>>2)]=poolPtr;
        poolPtr += line.length + 1;
      }
      HEAP32[((envPtr+strings.length * ptrSize)>>2)]=0;
    }var ENV={ };function _getenv(name) {
      // char *getenv(const char *name);
      // http://pubs.opengroup.org/onlinepubs/009695399/functions/getenv.html
      if (name === 0) return 0;
      name = Pointer_stringify(name);
      if (!ENV.hasOwnProperty(name)) return 0;
  
      if (_getenv.ret) _free(_getenv.ret);
      _getenv.ret = allocate(intArrayFromString(ENV[name]), 'i8', ALLOC_NORMAL);
      return _getenv.ret;
    }


  
  
  function _access(path, amode) {
      // int access(const char *path, int amode);
      // http://pubs.opengroup.org/onlinepubs/000095399/functions/access.html
      path = Pointer_stringify(path);
      var target = FS.findObject(path);
      if (target === null) return -1;
      if ((amode & 2 && !target.write) ||  // W_OK.
          ((amode & 1 || amode & 4) && !target.read)) {  // X_OK, R_OK.
        ___setErrNo(ERRNO_CODES.EACCES);
        return -1;
      } else {
        return 0;
      }
    }

  function _strcat(pdest, psrc) {
      var len = _strlen(pdest);
      var i = 0;
      do {
        var $src$, $dest$, $stop$, $stop4$, $fast$;
  $src$ = psrc+i;
  $dest$ = pdest+len+i;
  $stop$ = $src$ + 1;
  if (($dest$%4) == ($src$%4) && 1 > 8) {
    while ($src$%4 !== 0 && $src$ < $stop$) {
      ; HEAP8[$dest$++] = HEAP8[$src$++];
    }
    $src$ >>= 2;
    $dest$ >>= 2;
    $stop4$ = $stop$ >> 2;
    while ($src$ < $stop4$) {
  ;;;    HEAP32[$dest$++] = HEAP32[$src$++];
    }
    $src$ <<= 2;
    $dest$ <<= 2;
  }while ($src$ < $stop$) {
    ; HEAP8[$dest$++] = HEAP8[$src$++];
  }
        i ++;
      } while (HEAP8[(psrc+i-1)] != 0);
      return pdest;
    }

  
  
  
  function _pread(fildes, buf, nbyte, offset) {
      // ssize_t pread(int fildes, void *buf, size_t nbyte, off_t offset);
      // http://pubs.opengroup.org/onlinepubs/000095399/functions/read.html
      var stream = FS.streams[fildes];
      if (!stream || stream.object.isDevice) {
        ___setErrNo(ERRNO_CODES.EBADF);
        return -1;
      } else if (!stream.isRead) {
        ___setErrNo(ERRNO_CODES.EACCES);
        return -1;
      } else if (stream.object.isFolder) {
        ___setErrNo(ERRNO_CODES.EISDIR);
        return -1;
      } else if (nbyte < 0 || offset < 0) {
        ___setErrNo(ERRNO_CODES.EINVAL);
        return -1;
      } else {
        var bytesRead = 0;
        while (stream.ungotten.length && nbyte > 0) {
          HEAP8[(buf++)]=stream.ungotten.pop();
          nbyte--;
          bytesRead++;
        }
        var contents = stream.object.contents;
        var size = Math.min(contents.length - offset, nbyte);
        for (var i = 0; i < size; i++) {
          HEAP8[(buf+i)]=contents[offset + i];
          bytesRead++;
        }
        return bytesRead;
      }
    }function _read(fildes, buf, nbyte) {
      // ssize_t read(int fildes, void *buf, size_t nbyte);
      // http://pubs.opengroup.org/onlinepubs/000095399/functions/read.html
      var stream = FS.streams[fildes];
      if (!stream) {
        ___setErrNo(ERRNO_CODES.EBADF);
        return -1;
      } else if (!stream.isRead) {
        ___setErrNo(ERRNO_CODES.EACCES);
        return -1;
      } else if (nbyte < 0) {
        ___setErrNo(ERRNO_CODES.EINVAL);
        return -1;
      } else {
        var bytesRead;
        if (stream.object.isDevice) {
          if (stream.object.input) {
            bytesRead = 0;
            while (stream.ungotten.length && nbyte > 0) {
              HEAP8[(buf++)]=stream.ungotten.pop();
              nbyte--;
              bytesRead++;
            }
            for (var i = 0; i < nbyte; i++) {
              try {
                var result = stream.object.input();
              } catch (e) {
                ___setErrNo(ERRNO_CODES.EIO);
                return -1;
              }
              if (result === null || typeof result === 'undefined') break;
              bytesRead++;
              HEAP8[(buf+i)]=result;
            }
            return bytesRead;
          } else {
            ___setErrNo(ERRNO_CODES.ENXIO);
            return -1;
          }
        } else {
          var ungotSize = stream.ungotten.length;
          bytesRead = _pread(fildes, buf, nbyte, stream.position);
          if (bytesRead != -1) {
            stream.position += (stream.ungotten.length - ungotSize) + bytesRead;
          }
          return bytesRead;
        }
      }
    }function _fgetc(stream) {
      // int fgetc(FILE *stream);
      // http://pubs.opengroup.org/onlinepubs/000095399/functions/fgetc.html
      if (!(stream in FS.streams)) return -1;
      var streamObj = FS.streams[stream];
      if (streamObj.eof || streamObj.error) return -1;
      var ret = _read(stream, _fgetc.ret, 1);
      if (ret == 0) {
        streamObj.eof = true;
        return -1;
      } else if (ret == -1) {
        streamObj.error = true;
        return -1;
      } else {
        return HEAP8[(_fgetc.ret)];
      }
    }function _fgets(s, n, stream) {
      // char *fgets(char *restrict s, int n, FILE *restrict stream);
      // http://pubs.opengroup.org/onlinepubs/000095399/functions/fgets.html
      if (!(stream in FS.streams)) return 0;
      var streamObj = FS.streams[stream];
      if (streamObj.error || streamObj.eof) return 0;
      var byte_;
      for (var i = 0; i < n - 1 && byte_ != '\n'.charCodeAt(0); i++) {
        byte_ = _fgetc(stream);
        if (byte_ == -1) {
          if (streamObj.error) return 0;
          else if (streamObj.eof) break;
        }
        HEAP8[(s+i)]=byte_;
      }
      HEAP8[(s+i)]=0;
      return s;
    }

  function _strstr(ptr1, ptr2) {
      var str1 = Pointer_stringify(ptr1);
      var str2 = Pointer_stringify(ptr2);
      var ret = str1.search(str2);
      return ret >= 0 ? ptr1 + ret : 0;
    }

  function _memcmp(p1, p2, num) {
      for (var i = 0; i < num; i++) {
        var v1 = HEAP8[(p1+i)];
        var v2 = HEAP8[(p2+i)];
        if (v1 != v2) return v1 > v2 ? 1 : -1;
      }
      return 0;
    }

  
  function _memcpy(dest, src, num, idunno) {
      var $src$, $dest$, $stop$, $stop4$, $fast$;
  $src$ = src;
  $dest$ = dest;
  $stop$ = $src$ + num;
  if (($dest$%4) == ($src$%4) && num > 8) {
    while ($src$%4 !== 0 && $src$ < $stop$) {
      ; HEAP8[$dest$++] = HEAP8[$src$++];
    }
    $src$ >>= 2;
    $dest$ >>= 2;
    $stop4$ = $stop$ >> 2;
    while ($src$ < $stop4$) {
  ;;;    HEAP32[$dest$++] = HEAP32[$src$++];
    }
    $src$ <<= 2;
    $dest$ <<= 2;
  }while ($src$ < $stop$) {
    ; HEAP8[$dest$++] = HEAP8[$src$++];
  };
    }function _qsort(base, num, size, comparator) {
      if (num == 0 || size == 0) return;
      // forward calls to the JavaScript sort method
      // first, sort the items logically
      comparator = FUNCTION_TABLE[comparator];
      var keys = [];
      for (var i = 0; i < num; i++) keys.push(i);
      keys.sort(function(a, b) {
        return comparator(base+a*size, base+b*size);
      });
      // apply the sort
      var temp = _malloc(num*size);
      _memcpy(temp, base, num*size);
      for (var i = 0; i < num; i++) {
        if (keys[i] == i) continue; // already in place
        _memcpy(base+i*size, temp+keys[i]*size, size);
      }
      _free(temp);
    }


  
  function _unlink(path) {
      // int unlink(const char *path);
      // http://pubs.opengroup.org/onlinepubs/000095399/functions/unlink.html
      path = FS.analyzePath(Pointer_stringify(path));
      if (!path.parentExists || !path.exists) {
        ___setErrNo(path.error);
        return -1;
      } else if (path.object.isFolder) {
        ___setErrNo(ERRNO_CODES.EISDIR);
        return -1;
      } else if (!path.object.write) {
        ___setErrNo(ERRNO_CODES.EACCES);
        return -1;
      } else {
        delete path.parentObject.contents[path.name];
        return 0;
      }
    }
  
  
  
  function _rmdir(path) {
      // int rmdir(const char *path);
      // http://pubs.opengroup.org/onlinepubs/000095399/functions/rmdir.html
      path = FS.analyzePath(Pointer_stringify(path));
      if (!path.parentExists || !path.exists) {
        ___setErrNo(path.error);
        return -1;
      } else if (!path.object.write || path.isRoot) {
        ___setErrNo(ERRNO_CODES.EACCES);
        return -1;
      } else if (!path.object.isFolder) {
        ___setErrNo(ERRNO_CODES.ENOTDIR);
        return -1;
      } else {
        for (var i in path.object.contents) {
          ___setErrNo(ERRNO_CODES.ENOTEMPTY);
          return -1;
        }
        if (path.path == FS.currentPath) {
          ___setErrNo(ERRNO_CODES.EBUSY);
          return -1;
        } else {
          delete path.parentObject.contents[path.name];
          return 0;
        }
      }
    }function _remove(path) {
      // int remove(const char *path);
      // http://pubs.opengroup.org/onlinepubs/000095399/functions/remove.html
      var ret = _unlink(path);
      if (ret == -1) ret = _rmdir(path);
      return ret;
    }

  var _llvm_memcpy_p0i8_p0i8_i32=_memcpy;

// original emscripten ctype implementation (disabled for unicode basic latin support)
/*
  function _isalnum(chr) {
      return (chr >= '0'.charCodeAt(0) && chr <= '9'.charCodeAt(0)) ||
             (chr >= 'a'.charCodeAt(0) && chr <= 'z'.charCodeAt(0)) ||
             (chr >= 'A'.charCodeAt(0) && chr <= 'Z'.charCodeAt(0));
    }
*/

// added support for unicode basic latin + latin-1 supplement; mass:werk, N.L. 2014
  function _isalnum(chr) {
      return (chr >= 48 && chr <= 57) ||       // 0 .. 9
             (chr >= 97 && chr <= 122) ||      // a .. z
             (chr >= 65 && chr <= 90) ||       // A .. Z
             (chr >= 0xC0 && chr <= 0xD6) ||   // À .. Ö
             (chr >= 0xDF && chr <= 0xF6) ||   // ß .. ö
             (chr >= 0xD8 && chr <= 0xDE) ||   // Ø .. Þ
             (chr >= 0xF8 && chr <= 0xFF);     // ø .. ÿ
    }
// end of batch

  function _strchr(ptr, chr) {
      ptr--;
      do {
        ptr++;
        var val = HEAP8[(ptr)];
        if (val == chr) return ptr;
      } while (val);
      return 0;
    }

  function _strncpy(pdest, psrc, num) {
      var padding = false, curr;
      for (var i = 0; i < num; i++) {
        curr = padding ? 0 : HEAP8[(psrc+i)];
        HEAP8[(pdest+i)]=curr;
        padding = padding || HEAP8[(psrc+i)] == 0;
      }
      return pdest;
    }

  
  function _fread(ptr, size, nitems, stream) {
      // size_t fread(void *restrict ptr, size_t size, size_t nitems, FILE *restrict stream);
      // http://pubs.opengroup.org/onlinepubs/000095399/functions/fread.html
      var bytesToRead = nitems * size;
      if (bytesToRead == 0) return 0;
      var bytesRead = _read(stream, ptr, bytesToRead);
      var streamObj = FS.streams[stream];
      if (bytesRead == -1) {
        if (streamObj) streamObj.error = true;
        return -1;
      } else {
        if (bytesRead < bytesToRead) streamObj.eof = true;
        return Math.floor(bytesRead / size);
      }
    }


// original emscripten ctype implementation (disabled for unicode basic latin support)
/*
  function _tolower(chr) {
      if (chr >= 'A'.charCodeAt(0) && chr <= 'Z'.charCodeAt(0)) {
        return chr - 'A'.charCodeAt(0) + 'a'.charCodeAt(0);
      } else {
        return chr;
      }
    }

  function _isalpha(chr) {
      return (chr >= 'a'.charCodeAt(0) && chr <= 'z'.charCodeAt(0)) ||
             (chr >= 'A'.charCodeAt(0) && chr <= 'Z'.charCodeAt(0));
    }

  function _isupper(chr) {
      return chr >= 'A'.charCodeAt(0) && chr <= 'Z'.charCodeAt(0);
    }

  function _islower(chr) {
      return chr >= 'a'.charCodeAt(0) && chr <= 'z'.charCodeAt(0);
    }

  function _ispunct(chr) {
      return (chr >= '!'.charCodeAt(0) && chr <= '/'.charCodeAt(0)) ||
             (chr >= ':'.charCodeAt(0) && chr <= '@'.charCodeAt(0)) ||
             (chr >= '['.charCodeAt(0) && chr <= '`'.charCodeAt(0)) ||
             (chr >= '{'.charCodeAt(0) && chr <= '~'.charCodeAt(0));
    }
*/

// added support for unicode basic latin + latin-1 supplement; mass:werk, N.L. 2014
  function _tolower(chr) {
      if ((chr >= 65 && chr <90) ||
          (chr >= 0xC0 && chr <= 0xD6) ||
          (chr >= 0xD8 && chr <= 0xDE)) return chr - 65 + 97;
      return chr;
    }

  function _isalpha(chr) {
      return (chr >= 97 && chr <= 122) ||      // a .. z
             (chr >= 65 && chr <= 90) ||       // A .. Z
             (chr >= 0xC0 && chr <= 0xD6) ||   // À .. Ö
             (chr >= 0xDF && chr <= 0xF6) ||   // ß .. ö
             (chr >= 0xD8 && chr <= 0xDE) ||   // Ø .. Þ
             (chr >= 0xF8 && chr <= 0xFF);     // ø .. ÿ
    }

  function _isupper(chr) {
      return (chr >= 65 && chr <= 90) ||       // A .. Z
             (chr >= 0xC0 && chr <= 0xD6) ||   // À .. Ö
             (chr >= 0xD8 && chr <= 0xDE);     // Ø .. Þ
    }

  function _islower(chr) {
      return (chr >= 97 && chr <= 122) ||      // a .. z
             (chr >= 0xDF && chr <= 0xF6) ||   // ß .. ö
             (chr >= 0xF8 && chr <= 0xFF);     // ø .. ÿ
    }

  function _ispunct(chr) {
      return (chr >= 33 && chr <= 47) ||       // ! .. /
             (chr >= 58 && chr <= 64) ||       // : .. @
             (chr >= 91 && chr <= 96) ||       // [ .. `
             (chr >= 123 && chr <= 126);       // { .. ~
    }
// end of batch

var _atof; // stub for _atof

  function _realloc(ptr, size) {
      // Very simple, inefficient implementation - if you use a real malloc, best to use
      // a real realloc with it
      if (!size) {
        if (ptr) _free(ptr);
        return 0;
      }
      var ret = _malloc(size);
      if (ptr) {
        _memcpy(ret, ptr, size); // might be some invalid reads
        _free(ptr);
      }
      return ret;
    }
var _mkstemp; // stub for _mkstemp


  
  function _system(command) {
      // int system(const char *command);
      // http://pubs.opengroup.org/onlinepubs/000095399/functions/system.html
      // Can't call external programs.
      ___setErrNo(ERRNO_CODES.EAGAIN);
      return -1;
    }

  function _ungetc(c, stream) {
      // int ungetc(int c, FILE *stream);
      // http://pubs.opengroup.org/onlinepubs/000095399/functions/ungetc.html
      if (stream in FS.streams) {
        c = unSign(c & 0xFF);
        FS.streams[stream].ungotten.push(c);
        return c;
      } else {
        return -1;
      }
    }

  function _strdup(ptr) {
      var len = String_len(ptr);
      var newStr = _malloc(len + 1);
      var $src$, $dest$, $stop$, $stop4$, $fast$;
  $src$ = ptr;
  $dest$ = newStr;
  $stop$ = $src$ + len;
  if (($dest$%4) == ($src$%4) && len > 8) {
    while ($src$%4 !== 0 && $src$ < $stop$) {
      ; HEAP8[$dest$++] = HEAP8[$src$++];
    }
    $src$ >>= 2;
    $dest$ >>= 2;
    $stop4$ = $stop$ >> 2;
    while ($src$ < $stop4$) {
  ;;;    HEAP32[$dest$++] = HEAP32[$src$++];
    }
    $src$ <<= 2;
    $dest$ <<= 2;
  }while ($src$ < $stop$) {
    ; HEAP8[$dest$++] = HEAP8[$src$++];
  };
      HEAP8[(newStr+len)]=0;;
      return newStr;
    }

  
  function _kill(pid, sig) {
      // int kill(pid_t pid, int sig);
      // http://pubs.opengroup.org/onlinepubs/000095399/functions/kill.html
      // Makes no sense in a single-process environment.
      ___setErrNo(ERRNO_CODES.EPERM);
      return -1;
    }



  function ___errno_location() {
      return ___setErrNo.ret;
    }
  var ___errno=___errno_location;

  
  
  var ERRNO_MESSAGES={1: "Operation not permitted", 2: "No such file or directory", 3: "No such process", 4: "Interrupted system call", 5: "Input/output error", 6: "No such device or address", 8: "Exec format error", 9: "Bad file descriptor", 10: "No child processes", 11: "Resource temporarily unavailable", 12: "Cannot allocate memory", 13: "Permission denied", 14: "Bad address", 16: "Device or resource busy", 17: "File exists", 18: "Invalid cross-device link", 19: "No such device", 20: "Not a directory", 21: "Is a directory", 22: "Invalid argument", 23: "Too many open files in system", 24: "Too many open files", 25: "Inappropriate ioctl for device", 26: "Text file busy", 27: "File too large", 28: "No space left on device", 29: "Illegal seek", 30: "Read-only file system", 31: "Too many links", 32: "Broken pipe", 33: "Numerical argument out of domain", 34: "Numerical result out of range", 35: "Resource deadlock avoided", 36: "File name too long", 37: "No locks available", 38: "Function not implemented", 39: "Directory not empty", 40: "Too many levels of symbolic links", 42: "No message of desired type", 43: "Identifier removed", 60: "Device not a stream", 61: "No data available", 62: "Timer expired", 63: "Out of streams resources", 67: "Link has been severed", 71: "Protocol error", 72: "Multihop attempted", 74: "Bad message", 75: "Value too large for defined data type", 84: "Invalid or incomplete multibyte or wide character", 88: "Socket operation on non-socket", 89: "Destination address required", 90: "Message too long", 91: "Protocol wrong type for socket", 92: "Protocol not available", 93: "Protocol not supported", 95: "Operation not supported", 97: "Address family not supported by protocol", 98: "Address already in use", 99: "Cannot assign requested address", 100: "Network is down", 101: "Network is unreachable", 102: "Network dropped connection on reset", 103: "Software caused connection abort", 104: "Connection reset by peer", 105: "No buffer space available", 106: "Transport endpoint is already connected", 107: "Transport endpoint is not connected", 110: "Connection timed out", 111: "Connection refused", 113: "No route to host", 114: "Operation already in progress", 115: "Operation now in progress", 116: "Stale NFS file handle", 122: "Disk quota exceeded", 125: "Operation canceled", 130: "Owner died", 131: "State not recoverable" };
  function _strerror_r(errnum, strerrbuf, buflen) {
      if (errnum in ERRNO_MESSAGES) {
        if (ERRNO_MESSAGES[errnum].length > buflen - 1) {
          return ___setErrNo(ERRNO_CODES.ERANGE);
        } else {
          var msg = ERRNO_MESSAGES[errnum];
          for (var i = 0; i < msg.length; i++) {
            HEAP8[(strerrbuf+i)]=msg.charCodeAt(i);
          }
          HEAP8[(strerrbuf+i)]=0;
          return 0;
        }
      } else {
        return ___setErrNo(ERRNO_CODES.EINVAL);
      }
    }function _strerror(errnum) {
      if (!_strerror.buffer) _strerror.buffer = _malloc(256);
      _strerror_r(errnum, _strerror.buffer, 256);
      return _strerror.buffer;
    }


  
  function _memmove(dest, src, num, idunno) {
      // not optimized!
      if (num === 0) return; // will confuse malloc if 0
      var tmp = _malloc(num);
      _memcpy(tmp, src, num);
      _memcpy(dest, tmp, num);
      _free(tmp);
    }
  var _llvm_memmove_p0i8_p0i8_i32=_memmove;
var _llvm_va_start; // stub for _llvm_va_start

  
  
  var _vfprintf=_fprintf;

  function _llvm_va_end() {}

  
  
  function _wait(stat_loc) {
      // pid_t wait(int *stat_loc);
      // http://pubs.opengroup.org/onlinepubs/009695399/functions/wait.html
      // Makes no sense in a single-process environment.
      ___setErrNo(ERRNO_CODES.ECHILD);
      return -1;
    }
  
  var _waitpid=_wait;

  
  var _vsnprintf=_snprintf;

  
  
  var ___pollfd_struct_layout=null;function _poll(fds, nfds, timeout) {
      // int poll(struct pollfd fds[], nfds_t nfds, int timeout);
      // http://pubs.opengroup.org/onlinepubs/009695399/functions/poll.html
      // NOTE: This is pretty much a no-op mimicing glibc.
      var offsets = ___pollfd_struct_layout;
      var nonzero = 0;
      for (var i = 0; i < nfds; i++) {
        var pollfd = fds + ___pollfd_struct_layout.__size__ * i;
        var fd = HEAP32[((pollfd+offsets.fd)>>2)];
        var events = HEAP16[((pollfd+offsets.events)>>1)];
        var revents = 0;
        if (fd in FS.streams) {
          var stream = FS.streams[fd];
          if (events & 1) revents |= 1;
          if (events & 2) revents |= 2;
        } else {
          if (events & 4) revents |= 4;
        }
        if (revents) nonzero++;
        HEAP16[((pollfd+offsets.revents)>>1)]=revents;
      }
      return nonzero;
    }


  function _memchr(ptr, chr, num) {
      chr = unSign(chr);
      for (var i = 0; i < num; i++) {
        if (HEAP8[(ptr)] == chr) return ptr;
        ptr++;
      }
      return 0;
    }

  
  function _fork() {
      // pid_t fork(void);
      // http://pubs.opengroup.org/onlinepubs/000095399/functions/fork.html
      // We don't support multiple processes.
      ___setErrNo(ERRNO_CODES.EAGAIN);
      return -1;
    }

  
  
  var ___flock_struct_layout=null;function _fcntl(fildes, cmd, varargs) {
      // int fcntl(int fildes, int cmd, ...);
      // http://pubs.opengroup.org/onlinepubs/009695399/functions/fcntl.html
      if (!(fildes in FS.streams)) {
        ___setErrNo(ERRNO_CODES.EBADF);
        return -1;
      }
      var stream = FS.streams[fildes];
      switch (cmd) {
        case 0:
          var arg = HEAP32[((varargs)>>2)];
          if (arg < 0) {
            ___setErrNo(ERRNO_CODES.EINVAL);
            return -1;
          }
          var newStream = {};
          for (var member in stream) {
            newStream[member] = stream[member];
          }
          if (arg in FS.streams) arg = FS.streams.length;
          FS.streams[arg] = newStream;
          return arg;
        case 1:
        case 2:
          return 0;  // FD_CLOEXEC makes no sense for a single process.
        case 3:
          var flags = 0;
          if (stream.isRead && stream.isWrite) flags = 2;
          else if (!stream.isRead && stream.isWrite) flags = 1;
          else if (stream.isRead && !stream.isWrite) flags = 0;
          if (stream.isAppend) flags |= 8;
          // Synchronization and blocking flags are irrelevant to us.
          return flags;
        case 4:
          var arg = HEAP32[((varargs)>>2)];
          stream.isAppend = Boolean(arg | 8);
          // Synchronization and blocking flags are irrelevant to us.
          return 0;
        case 7:
        case 20:
          var arg = HEAP32[((varargs)>>2)];
          var offset = ___flock_struct_layout.l_type;
          // We're always unlocked.
          HEAP16[((arg+offset)>>1)]=3;
          return 0;
        case 8:
        case 9:
        case 21:
        case 22:
          // Pretend that the locking is successful.
          return 0;
        case 6:
        case 5:
          // These are for sockets. We don't have them implemented (yet?).
          ___setErrNo(ERRNO_CODES.EINVAL);
          return -1;
        default:
          ___setErrNo(ERRNO_CODES.EINVAL);
          return -1;
      }
      // Should never be reached. Only to silence strict warnings.
      return -1;
    }
  function _dup2(fildes, fildes2) {
      // int dup2(int fildes, int fildes2);
      // http://pubs.opengroup.org/onlinepubs/000095399/functions/dup.html
      if (fildes2 < 0) {
        ___setErrNo(ERRNO_CODES.EBADF);
        return -1;
      } else if (fildes === fildes2 && FS.streams[fildes]) {
        return fildes;
      } else {
        _close(fildes2);
        return _fcntl(fildes, 0, allocate([fildes2, 0, 0, 0], 'i32', ALLOC_STACK));  // F_DUPFD.
      }
    }


  function _signal(sig, func) {
      // TODO
      return 0;
    }

  
  
  function _execl(/* ... */) {
      // int execl(const char *path, const char *arg0, ... /*, (char *)0 */);
      // http://pubs.opengroup.org/onlinepubs/009695399/functions/exec.html
      // We don't support executing external code.
      ___setErrNo(ERRNO_CODES.ENOEXEC);
      return -1;
    }
  
  var _execlp=_execl;



  
  function _pipe(fildes) {
      // int pipe(int fildes[2]);
      // http://pubs.opengroup.org/onlinepubs/000095399/functions/pipe.html
      // It is possible to implement this using two device streams, but pipes make
      // little sense in a single-threaded environment, so we do not support them.
      ___setErrNo(ERRNO_CODES.ENOSYS);
      return -1;
    }

  
  
  function _opendir(dirname) {
      // DIR *opendir(const char *dirname);
      // http://pubs.opengroup.org/onlinepubs/007908799/xsh/opendir.html
      // NOTE: Calculating absolute path redundantly since we need to associate it
      //       with the opened stream.
      var path = FS.absolutePath(Pointer_stringify(dirname));
      if (path === null) {
        ___setErrNo(ERRNO_CODES.ENOENT);
        return 0;
      }
      var target = FS.findObject(path);
      if (target === null) return 0;
      if (!target.isFolder) {
        ___setErrNo(ERRNO_CODES.ENOTDIR);
        return 0;
      } else if (!target.read) {
        ___setErrNo(ERRNO_CODES.EACCES);
        return 0;
      }
      var id = FS.streams.length;
      var contents = [];
      for (var key in target.contents) contents.push(key);
      FS.streams[id] = {
        path: path,
        object: target,
        // An index into contents. Special values: -2 is ".", -1 is "..".
        position: -2,
        isRead: true,
        isWrite: false,
        isAppend: false,
        error: false,
        eof: false,
        ungotten: [],
        // Folder-specific properties:
        // Remember the contents at the time of opening in an array, so we can
        // seek between them relying on a single order.
        contents: contents,
        // Each stream has its own area for readdir() returns.
        currentEntry: _malloc(___dirent_struct_layout.__size__)
      };
      return id;
    }

  
  
  
  
  function _readdir_r(dirp, entry, result) {
      // int readdir_r(DIR *dirp, struct dirent *entry, struct dirent **result);
      // http://pubs.opengroup.org/onlinepubs/007908799/xsh/readdir_r.html
      if (!FS.streams[dirp] || !FS.streams[dirp].object.isFolder) {
        return ___setErrNo(ERRNO_CODES.EBADF);
      }
      var stream = FS.streams[dirp];
      var loc = stream.position;
      var entries = 0;
      for (var key in stream.contents) entries++;
      if (loc < -2 || loc >= entries) {
        HEAP32[((result)>>2)]=0;
      } else {
        var name, inode;
        if (loc === -2) {
          name = '.';
          inode = 1;  // Really undefined.
        } else if (loc === -1) {
          name = '..';
          inode = 1;  // Really undefined.
        } else {
          name = stream.contents[loc];
          inode = stream.object.contents[name].inodeNumber;
        }
        stream.position++;
        var offsets = ___dirent_struct_layout;
        HEAP32[((entry+offsets.d_ino)>>2)]=inode;
        HEAP32[((entry+offsets.d_off)>>2)]=stream.position;
        HEAP32[((entry+offsets.d_reclen)>>2)]=name.length + 1;
        for (var i = 0; i < name.length; i++) {
          HEAP8[(entry + offsets.d_name+i)]=name.charCodeAt(i);
        }
        HEAP8[(entry + offsets.d_name+i)]=0;
        var type = stream.object.isDevice ? 2 // DT_CHR, character device.
                 : stream.object.isFolder ? 4 // DT_DIR, directory.
                 : (typeof stream.object.link !== 'undefined')? 10 // DT_LNK, symbolic link.
                 : 8; // DT_REG, regular file.
        HEAP8[(entry+offsets.d_type)]=type;
        HEAP32[((result)>>2)]=entry;
      }
      return 0;
    }
  
  function _readdir(dirp) {
      // struct dirent *readdir(DIR *dirp);
      // http://pubs.opengroup.org/onlinepubs/007908799/xsh/readdir_r.html
      if (!FS.streams[dirp] || !FS.streams[dirp].object.isFolder) {
        ___setErrNo(ERRNO_CODES.EBADF);
        return 0;
      } else {
        if (!_readdir.result) _readdir.result = _malloc(4);
        _readdir_r(dirp, FS.streams[dirp].currentEntry, _readdir.result);
        if (HEAP32[((_readdir.result)>>2)] === 0) {
          return 0;
        } else {
          return FS.streams[dirp].currentEntry;
        }
      }
    }

  
  
  function _closedir(dirp) {
      // int closedir(DIR *dirp);
      // http://pubs.opengroup.org/onlinepubs/007908799/xsh/closedir.html
      if (!FS.streams[dirp] || !FS.streams[dirp].object.isFolder) {
        return ___setErrNo(ERRNO_CODES.EBADF);
      } else {
        _free(FS.streams[dirp].currentEntry);
        delete FS.streams[dirp];
        return 0;
      }
    }

  function _calloc(n, s) {
      var ret = _malloc(n*s);
      _memset(ret, 0, n*s);
      return ret;
    }

  var _cos=Math.cos;

  
  var ___rand_state=42;function _rand() {
      // int rand(void);
      ___rand_state = (1103515245 * ___rand_state + 12345) % 0x100000000;
      return ___rand_state & 0x7FFFFFFF;
    }

  var _exp=Math.exp;

  var _sin=Math.sin;

  
  var _vsprintf=_sprintf;

  
  function _fputs(s, stream) {
      // int fputs(const char *restrict s, FILE *restrict stream);
      // http://pubs.opengroup.org/onlinepubs/000095399/functions/fputs.html
      return _write(stream, s, _strlen(s));
    }

  function _exp2(x) {
      return Math.pow(2, x);
    }




  function __Z13GetFileLengthPKc($filename) {
    var __stackBase__  = STACKTOP; STACKTOP += 80; _memset(__stackBase__, 0, 80);
    var __label__;
  
    var $retval;
    var $filename_addr;
    var $statbuf=__stackBase__;
    $filename_addr=$filename;
    var $0=$filename_addr; //@line 130 "speak.cpp"
    var $call=_stat($0, $statbuf); //@line 130 "speak.cpp"
    var $cmp=((($call))|0)!=0; //@line 130 "speak.cpp"
    if ($cmp) { __label__ = 1;; } else { __label__ = 2;; } //@line 130 "speak.cpp"
    if (__label__ == 1) {
  
      $retval=0; //@line 131 "speak.cpp"
      ; //@line 131 "speak.cpp"
    }
    else if (__label__ == 2) {
  
      var $st_mode=(($statbuf+12)&4294967295); //@line 133 "speak.cpp"
      var $1=HEAP32[(($st_mode)>>2)]; //@line 133 "speak.cpp"
      var $and=($1) & 61440; //@line 133 "speak.cpp"
      var $cmp1=((($and))|0)==16384; //@line 133 "speak.cpp"
      if ($cmp1) { __label__ = 3;; } else { __label__ = 4;; } //@line 133 "speak.cpp"
      if (__label__ == 3) {
  
        $retval=-2; //@line 135 "speak.cpp"
        ; //@line 135 "speak.cpp"
      }
      else if (__label__ == 4) {
  
        var $st_size=(($statbuf+36)&4294967295); //@line 137 "speak.cpp"
        var $2=HEAP32[(($st_size)>>2)]; //@line 137 "speak.cpp"
        $retval=$2; //@line 137 "speak.cpp"
        ; //@line 137 "speak.cpp"
      }
    }
  
    var $3=$retval; //@line 138 "speak.cpp"
    STACKTOP = __stackBase__;
    return $3; //@line 138 "speak.cpp"
    return null;
  }
  

  function __Z5Alloci($size) {
    ;
    var __label__;
  
    var $size_addr;
    var $p;
    $size_addr=$size;
    var $0=$size_addr; //@line 144 "speak.cpp"
    var $call=_malloc($0); //@line 144 "speak.cpp"
    $p=$call; //@line 144 "speak.cpp"
    var $cmp=((($call))|0)==0; //@line 144 "speak.cpp"
    if ($cmp) { __label__ = 1;; } else { __label__ = 2;; } //@line 144 "speak.cpp"
    if (__label__ == 1) {
  
      var $1=HEAP32[((__impure_ptr)>>2)]; //@line 145 "speak.cpp"
      var $_stderr=(($1+12)&4294967295); //@line 145 "speak.cpp"
      var $2=HEAP32[(($_stderr)>>2)]; //@line 145 "speak.cpp"
      var $3=_fwrite(((__str106)&4294967295), 22, 1, $2); //@line 145 "speak.cpp"
      ; //@line 145 "speak.cpp"
    }
  
    var $4=$p; //@line 146 "speak.cpp"
    ;
    return $4; //@line 146 "speak.cpp"
    return null;
  }
  

  function __Z4FreePv($ptr) {
    ;
    var __label__;
  
    var $ptr_addr;
    $ptr_addr=$ptr;
    var $0=$ptr_addr; //@line 151 "speak.cpp"
    var $cmp=((($0))|0)!=0; //@line 151 "speak.cpp"
    if ($cmp) { __label__ = 1;; } else { __label__ = 2;; } //@line 151 "speak.cpp"
    if (__label__ == 1) {
  
      var $1=$ptr_addr; //@line 152 "speak.cpp"
      ; //@line 152 "speak.cpp"
      ; //@line 152 "speak.cpp"
    }
  
    ;
    return; //@line 153 "speak.cpp"
    return;
  }
  

  function __Z13DisplayVoicesP7__sFILEPc($f_out, $language) {
    var __stackBase__  = STACKTOP; STACKTOP += 36; _memset(__stackBase__, 0, 36);
    var __label__;
  
    var $f_out_addr;
    var $language_addr;
    var $ix;
    var $p;
    var $len;
    var $count;
    var $scores;
    var $v;
    var $lang_name;
    var $age_buf=__stackBase__;
    var $voices;
    var $voice_select=__stackBase__+12;
    $f_out_addr=$f_out;
    $language_addr=$language;
    $scores=0; //@line 162 "speak.cpp"
    var $0=$language_addr; //@line 171 "speak.cpp"
    var $cmp=((($0))|0)!=0; //@line 171 "speak.cpp"
    if ($cmp) { __label__ = 1;; } else { __label__ = 3;; } //@line 171 "speak.cpp"
    $land_lhs_true$$if_else$2: do { 
      if (__label__ == 1) {
  
        var $1=$language_addr; //@line 171 "speak.cpp"
        var $arrayidx=(($1)&4294967295); //@line 171 "speak.cpp"
        var $2=HEAP8[($arrayidx)]; //@line 171 "speak.cpp"
        var $conv=(tempInt=(($2)),(tempInt>=128?tempInt-256:tempInt)); //@line 171 "speak.cpp"
        var $cmp1=((($conv))|0)!=0; //@line 171 "speak.cpp"
        if (!($cmp1)) { __label__ = 3;break $land_lhs_true$$if_else$2; } //@line 171 "speak.cpp"
  
        var $3=$language_addr; //@line 174 "speak.cpp"
        var $languages=(($voice_select+4)&4294967295); //@line 174 "speak.cpp"
        HEAP32[(($languages)>>2)]=$3; //@line 174 "speak.cpp"
        var $age=(($voice_select+13)&4294967295); //@line 175 "speak.cpp"
        HEAP8[($age)]=0; //@line 175 "speak.cpp"
        var $gender=(($voice_select+12)&4294967295); //@line 176 "speak.cpp"
        HEAP8[($gender)]=0; //@line 176 "speak.cpp"
        var $name=(($voice_select)&4294967295); //@line 177 "speak.cpp"
        HEAP32[(($name)>>2)]=0; //@line 177 "speak.cpp"
        var $call=_espeak_ListVoices($voice_select); //@line 178 "speak.cpp"
        $voices=((__ZZ17espeak_ListVoicesE6voices)&4294967295); //@line 178 "speak.cpp"
        $scores=1; //@line 179 "speak.cpp"
        __label__ = 4;break $land_lhs_true$$if_else$2; //@line 180 "speak.cpp"
      }
    } while(0);
    if (__label__ == 3) {
  
      var $call2=_espeak_ListVoices(0); //@line 183 "speak.cpp"
      $voices=((__ZZ17espeak_ListVoicesE6voices)&4294967295); //@line 183 "speak.cpp"
      ;
    }
  
    var $4=$f_out_addr; //@line 186 "speak.cpp"
    var $5=_fwrite(((__str1)&4294967295), 64, 1, $4); //@line 186 "speak.cpp"
    $ix=0; //@line 188 "speak.cpp"
    var $6=$ix; //@line 188 "speak.cpp"
    var $7=$voices; //@line 188 "speak.cpp"
    var $arrayidx43=(($7+4*$6)&4294967295); //@line 188 "speak.cpp"
    var $8=HEAP32[(($arrayidx43)>>2)]; //@line 188 "speak.cpp"
    $v=$8; //@line 188 "speak.cpp"
    var $cmp54=((($8))|0)!=0; //@line 188 "speak.cpp"
    if ($cmp54) { __label__ = 5;; } else { __label__ = 15;; } //@line 188 "speak.cpp"
    $for_body_lr_ph$$for_end$8: do { 
      if (__label__ == 5) {
  
        var $arraydecay=(($age_buf)&4294967295); //@line 198 "speak.cpp"
        var $arraydecay26=(($age_buf)&4294967295); //@line 204 "speak.cpp"
        ; //@line 188 "speak.cpp"
        $for_body$10: while(1) { 
  
          $count=0; //@line 190 "speak.cpp"
          var $9=$v; //@line 191 "speak.cpp"
          var $languages6=(($9+4)&4294967295); //@line 191 "speak.cpp"
          var $10=HEAP32[(($languages6)>>2)]; //@line 191 "speak.cpp"
          $p=$10; //@line 191 "speak.cpp"
          var $11=$p; //@line 192 "speak.cpp"
          var $12=HEAP8[($11)]; //@line 192 "speak.cpp"
          var $conv71=(tempInt=(($12)),(tempInt>=128?tempInt-256:tempInt)); //@line 192 "speak.cpp"
          var $cmp82=((($conv71))|0)!=0; //@line 192 "speak.cpp"
          if ($cmp82) { __label__ = 7;; } else { __label__ = 14;; } //@line 192 "speak.cpp"
          $while_body$$while_end$12: do { 
            if (__label__ == 7) {
              while(1) { 
  
                var $13=$p; //@line 194 "speak.cpp"
                var $add_ptr=(($13+1)&4294967295); //@line 194 "speak.cpp"
                var $call9=_strlen($add_ptr); //@line 194 "speak.cpp"
                $len=$call9; //@line 194 "speak.cpp"
                var $14=$p; //@line 195 "speak.cpp"
                var $add_ptr10=(($14+1)&4294967295); //@line 195 "speak.cpp"
                $lang_name=$add_ptr10; //@line 195 "speak.cpp"
                var $15=$v; //@line 197 "speak.cpp"
                var $age11=(($15+13)&4294967295); //@line 197 "speak.cpp"
                var $16=HEAPU8[($age11)]; //@line 197 "speak.cpp"
                var $conv12=((($16))&255); //@line 197 "speak.cpp"
                var $cmp13=((($conv12))|0)==0; //@line 197 "speak.cpp"
                if ($cmp13) { __label__ = 8;; } else { __label__ = 9;; } //@line 197 "speak.cpp"
                if (__label__ == 8) {
  
                  var $src$, $dest$, $stop$, $stop4$, $fast$;
                  $src$ = ((__str41208)&4294967295);
                  $dest$ = $arraydecay;
                  $stop$ = $src$ + 4;
                  if (($dest$%4) == ($src$%4) && 4 > 8) {
                    while ($src$%4 !== 0 && $src$ < $stop$) {
                      ; HEAP8[$dest$++] = HEAP8[$src$++];
                    }
                    $src$ >>= 2;
                    $dest$ >>= 2;
                    $stop4$ = $stop$ >> 2;
                    while ($src$ < $stop4$) {
                  ;;;    HEAP32[$dest$++] = HEAP32[$src$++];
                    }
                    $src$ <<= 2;
                    $dest$ <<= 2;
                  }while ($src$ < $stop$) {
                    ; HEAP8[$dest$++] = HEAP8[$src$++];
                  }; //@line 198 "speak.cpp"
                  ; //@line 198 "speak.cpp"
                }
                else if (__label__ == 9) {
  
                  var $17=$v; //@line 200 "speak.cpp"
                  var $age18=(($17+13)&4294967295); //@line 200 "speak.cpp"
                  var $18=HEAPU8[($age18)]; //@line 200 "speak.cpp"
                  var $conv19=((($18))&255); //@line 200 "speak.cpp"
                  var $call20=_sprintf($arraydecay, ((__str3)&4294967295), allocate([$conv19,0,0,0], ["i32",0,0,0], ALLOC_STACK)); //@line 200 "speak.cpp"
                  ;
                }
  
                var $19=$count; //@line 202 "speak.cpp"
                var $cmp22=((($19))|0)==0; //@line 202 "speak.cpp"
                var $20=$f_out_addr; //@line 204 "speak.cpp"
                if ($cmp22) { __label__ = 11;; } else { __label__ = 12;; } //@line 202 "speak.cpp"
                if (__label__ == 11) {
  
                  var $21=$p; //@line 204 "speak.cpp"
                  var $arrayidx24=(($21)&4294967295); //@line 204 "speak.cpp"
                  var $22=HEAP8[($arrayidx24)]; //@line 204 "speak.cpp"
                  var $conv25=(tempInt=(($22)),(tempInt>=128?tempInt-256:tempInt)); //@line 204 "speak.cpp"
                  var $23=$lang_name; //@line 204 "speak.cpp"
                  var $24=$v; //@line 204 "speak.cpp"
                  var $gender27=(($24+12)&4294967295); //@line 204 "speak.cpp"
                  var $25=HEAPU8[($gender27)]; //@line 204 "speak.cpp"
                  var $idxprom=((($25))&255); //@line 204 "speak.cpp"
                  var $arrayidx28=((__ZZ13DisplayVoicesP7__sFILEPcE7genders+$idxprom)&4294967295); //@line 204 "speak.cpp"
                  var $26=HEAP8[($arrayidx28)]; //@line 204 "speak.cpp"
                  var $conv29=(tempInt=(($26)),(tempInt>=128?tempInt-256:tempInt)); //@line 204 "speak.cpp"
                  var $27=$v; //@line 204 "speak.cpp"
                  var $name30=(($27)&4294967295); //@line 204 "speak.cpp"
                  var $28=HEAP32[(($name30)>>2)]; //@line 204 "speak.cpp"
                  var $29=$v; //@line 204 "speak.cpp"
                  var $identifier=(($29+8)&4294967295); //@line 204 "speak.cpp"
                  var $30=HEAP32[(($identifier)>>2)]; //@line 204 "speak.cpp"
                  var $call31=_fprintf($20, ((__str4)&4294967295), allocate([$conv25,0,0,0,$23,0,0,0,$arraydecay26,0,0,0,$conv29,0,0,0,$28,0,0,0,$30,0,0,0], ["i32",0,0,0,"i8*",0,0,0,"i8*",0,0,0,"i32",0,0,0,"i8*",0,0,0,"i8*",0,0,0], ALLOC_STACK)); //@line 204 "speak.cpp"
                  ; //@line 206 "speak.cpp"
                }
                else if (__label__ == 12) {
  
                  var $31=$lang_name; //@line 209 "speak.cpp"
                  var $32=$p; //@line 209 "speak.cpp"
                  var $arrayidx33=(($32)&4294967295); //@line 209 "speak.cpp"
                  var $33=HEAP8[($arrayidx33)]; //@line 209 "speak.cpp"
                  var $conv34=(tempInt=(($33)),(tempInt>=128?tempInt-256:tempInt)); //@line 209 "speak.cpp"
                  var $call35=_fprintf($20, ((__str5)&4294967295), allocate([$31,0,0,0,$conv34,0,0,0], ["i8*",0,0,0,"i32",0,0,0], ALLOC_STACK)); //@line 209 "speak.cpp"
                  ;
                }
  
                var $34=$count; //@line 211 "speak.cpp"
                var $inc=((($34)+1)&4294967295); //@line 211 "speak.cpp"
                $count=$inc; //@line 211 "speak.cpp"
                var $35=$len; //@line 212 "speak.cpp"
                var $add=((($35)+2)&4294967295); //@line 212 "speak.cpp"
                var $36=$p; //@line 212 "speak.cpp"
                var $add_ptr37=(($36+$add)&4294967295); //@line 212 "speak.cpp"
                $p=$add_ptr37; //@line 212 "speak.cpp"
                var $37=$p; //@line 192 "speak.cpp"
                var $38=HEAP8[($37)]; //@line 192 "speak.cpp"
                var $conv7=(tempInt=(($38)),(tempInt>=128?tempInt-256:tempInt)); //@line 192 "speak.cpp"
                var $cmp8=((($conv7))|0)!=0; //@line 192 "speak.cpp"
                if ($cmp8) { __label__ = 7;continue ; } else { __label__ = 14;break $while_body$$while_end$12; } //@line 192 "speak.cpp"
              }
            }
          } while(0);
  
          var $39=$f_out_addr; //@line 216 "speak.cpp"
          var $call38=_fputc(10, $39); //@line 216 "speak.cpp"
          var $40=$ix; //@line 188 "speak.cpp"
          var $inc39=((($40)+1)&4294967295); //@line 188 "speak.cpp"
          $ix=$inc39; //@line 188 "speak.cpp"
          var $41=$ix; //@line 188 "speak.cpp"
          var $42=$voices; //@line 188 "speak.cpp"
          var $arrayidx4=(($42+4*$41)&4294967295); //@line 188 "speak.cpp"
          var $43=HEAP32[(($arrayidx4)>>2)]; //@line 188 "speak.cpp"
          $v=$43; //@line 188 "speak.cpp"
          var $cmp5=((($43))|0)!=0; //@line 188 "speak.cpp"
          if ($cmp5) { __label__ = 6;continue $for_body$10; } else { __label__ = 15;break $for_body_lr_ph$$for_end$8; } //@line 188 "speak.cpp"
        }
      }
    } while(0);
  
    STACKTOP = __stackBase__;
    return; //@line 218 "speak.cpp"
    return;
  }
  

  function __Z13WVoiceChangedP7voice_t($wvoice) {
    ;
    var __label__;
  
    ;
    return; //@line 223 "speak.cpp"
    return;
  }
  

  function __Z11MarkerEventijiPh($type, $char_position, $value, $out_ptr) {
    ;
    var __label__;
  
    var $type_addr;
    var $char_position_addr;
    var $value_addr;
    var $out_ptr_addr;
    $type_addr=$type;
    $char_position_addr=$char_position;
    $value_addr=$value;
    $out_ptr_addr=$out_ptr;
    var $0=$type_addr; //@line 292 "speak.cpp"
    var $cmp=((($0))|0)==2; //@line 292 "speak.cpp"
    if ($cmp) { __label__ = 1;; } else { __label__ = 2;; } //@line 292 "speak.cpp"
    if (__label__ == 1) {
  
      HEAP32[((_end_of_sentence)>>2)]=1; //@line 293 "speak.cpp"
      ; //@line 293 "speak.cpp"
    }
  
    ;
    return; //@line 294 "speak.cpp"
    return;
  }
  

  function _main($argc, $argv) {
    ;
    var __label__;
    var __lastLabel__ = null;
  
    var $_ZL9help_text;
    $_ZL9help_text=((__str37)&4294967295);
    var $_ZZ4mainE8err_load;
    $_ZZ4mainE8err_load=((__str18)&4294967295);
    var $retval;
    var $argc_addr;
    var $argv_addr;
    var $f_text;
    var $p_text;
    var $data_path;
    var $option_index=STACKTOP; _memset(STACKTOP, 0, 4); STACKTOP += 4;STACKTOP = Math.ceil((STACKTOP)/4)*4;;
    var $c;
    var $value=STACKTOP; _memset(STACKTOP, 0, 4); STACKTOP += 4;STACKTOP = Math.ceil((STACKTOP)/4)*4;;
    var $speed;
    var $ix;
    var $optarg2;
    var $amp;
    var $wordgap;
    var $speaking;
    var $flag_stdin;
    var $flag_compile;
    var $pitch_adjustment;
    var $voice_select=STACKTOP; _memset(STACKTOP, 0, 24); STACKTOP += 24;STACKTOP = Math.ceil((STACKTOP)/4)*4;;
    var $filename=STACKTOP; _memset(STACKTOP, 0, 200); STACKTOP += 200;STACKTOP = Math.ceil((STACKTOP)/4)*4;;
    var $voicename=STACKTOP; _memset(STACKTOP, 0, 40); STACKTOP += 40;STACKTOP = Math.ceil((STACKTOP)/4)*4;;
    var $dictname=STACKTOP; _memset(STACKTOP, 0, 40); STACKTOP += 40;STACKTOP = Math.ceil((STACKTOP)/4)*4;;
    var $extn;
    $retval=0;
    $argc_addr=$argc;
    $argv_addr=$argv;
    $f_text=0; //@line 495 "speak.cpp"
    $p_text=0; //@line 496 "speak.cpp"
    $data_path=0; //@line 497 "speak.cpp"
    HEAP32[(($option_index)>>2)]=0; //@line 499 "speak.cpp"
    $speed=175; //@line 502 "speak.cpp"
    $amp=100; //@line 505 "speak.cpp"
    $wordgap=0; //@line 506 "speak.cpp"
    $speaking=0; //@line 507 "speak.cpp"
    $flag_stdin=0; //@line 508 "speak.cpp"
    $flag_compile=0; //@line 509 "speak.cpp"
    $pitch_adjustment=50; //@line 510 "speak.cpp"
    var $arrayidx=(($voicename)&4294967295); //@line 516 "speak.cpp"
    HEAP8[($arrayidx)]=0; //@line 516 "speak.cpp"
    HEAP8[(((_mbrola_name)&4294967295))]=0; //@line 517 "speak.cpp"
    var $arrayidx1=(($dictname)&4294967295); //@line 518 "speak.cpp"
    HEAP8[($arrayidx1)]=0; //@line 518 "speak.cpp"
    HEAP8[(((_wavefile)&4294967295))]=0; //@line 519 "speak.cpp"
    var $arrayidx2=(($filename)&4294967295); //@line 520 "speak.cpp"
    HEAP8[($arrayidx2)]=0; //@line 520 "speak.cpp"
    HEAP32[((_option_linelength)>>2)]=0; //@line 521 "speak.cpp"
    HEAP32[((_option_phonemes)>>2)]=0; //@line 522 "speak.cpp"
    HEAP32[((_option_waveout)>>2)]=0; //@line 523 "speak.cpp"
    HEAP32[((_option_wordgap)>>2)]=0; //@line 524 "speak.cpp"
    HEAP32[((_option_endpause)>>2)]=1; //@line 525 "speak.cpp"
    HEAP32[((_option_phoneme_input)>>2)]=1; //@line 526 "speak.cpp"
    HEAP32[((_option_multibyte)>>2)]=0; //@line 527 "speak.cpp"
    var $0=HEAP32[((__impure_ptr)>>2)]; //@line 528 "speak.cpp"
    var $_stdout=(($0+8)&4294967295); //@line 528 "speak.cpp"
    var $1=HEAP32[(($_stdout)>>2)]; //@line 528 "speak.cpp"
    HEAP32[((_f_trans)>>2)]=$1; //@line 528 "speak.cpp"
    var $2=$argc_addr; //@line 588 "speak.cpp"
    var $3=$argv_addr; //@line 588 "speak.cpp"
    var $call6=_getopt_long($2, $3, ((__str19)&4294967295), ((__ZZ4mainE12long_options)&4294967295), $option_index); //@line 588 "speak.cpp"
    $c=$call6; //@line 588 "speak.cpp"
    var $cmp7=((($call6))|0)==-1; //@line 592 "speak.cpp"
    if ($cmp7) { __label__ = 43;; } else { __label__ = 1;; } //@line 592 "speak.cpp"
    $while_end71$$if_end_lr_ph$2: do { 
      if (__label__ == 1) {
  
        var $arraydecay=(($filename)&4294967295); //@line 638 "speak.cpp"
        var $arraydecay32=(($voicename)&4294967295); //@line 660 "speak.cpp"
        var $arraydecay41=(($voicename)&4294967295); //@line 684 "speak.cpp"
        var $_pr8=$c; //@line 597 "speak.cpp"
        __lastLabel__ = 1; ; //@line 592 "speak.cpp"
        $if_end$4: while(1) { 
  
          var $4=__lastLabel__ == 1 ? $_pr8 : ($call); //@line 597 "speak.cpp"
          var $5=HEAP32[((_optarg)>>2)]; //@line 594 "speak.cpp"
          $optarg2=$5; //@line 594 "speak.cpp"
          if ($4 == 98) {
            __lastLabel__ = 2; __label__ = 3;;
          }
          else if ($4 == 104) {
            __lastLabel__ = 2; __label__ = 5;;
          }
          else if ($4 == 107) {
            __lastLabel__ = 2; __label__ = 6;;
          }
          else if ($4 == 120) {
            __lastLabel__ = 2; __label__ = 7;;
          }
          else if ($4 == 88) {
            __lastLabel__ = 2; __label__ = 8;;
          }
          else if ($4 == 109) {
            __lastLabel__ = 2; __label__ = 9;;
          }
          else if ($4 == 112) {
            __lastLabel__ = 2; __label__ = 10;;
          }
          else if ($4 == 113) {
            __lastLabel__ = 2; __label__ = 12;;
          }
          else if ($4 == 102) {
            __lastLabel__ = 2; __label__ = 13;;
          }
          else if ($4 == 108) {
            __lastLabel__ = 2; __label__ = 14;;
          }
          else if ($4 == 97) {
            __lastLabel__ = 2; __label__ = 15;;
          }
          else if ($4 == 115) {
            __lastLabel__ = 2; __label__ = 16;;
          }
          else if ($4 == 103) {
            __lastLabel__ = 2; __label__ = 17;;
          }
          else if ($4 == 118) {
            __lastLabel__ = 2; __label__ = 18;;
          }
          else if ($4 == 119) {
            __lastLabel__ = 2; __label__ = 19;;
          }
          else if ($4 == 122) {
            __lastLabel__ = 2; __label__ = 20;;
          }
          else if ($4 == 256) {
            __lastLabel__ = 2; __label__ = 21;;
          }
          else if ($4 == 261) {
            __lastLabel__ = 2; __label__ = 22;;
          }
          else if ($4 == 257) {
            __lastLabel__ = 2; __label__ = 23;;
          }
          else if ($4 == 258) {
            __lastLabel__ = 2; __label__ = 23;;
          }
          else if ($4 == 259) {
            __lastLabel__ = 2; __label__ = 26;;
          }
          else if ($4 == 260) {
            __lastLabel__ = 2; __label__ = 32;;
          }
          else if ($4 == 262) {
            __lastLabel__ = 2; __label__ = 33;;
          }
          else if ($4 == 263) {
            __lastLabel__ = 2; __label__ = 36;;
          }
          else if ($4 == 264) {
            __lastLabel__ = 2; __label__ = 37;;
          }
          else if ($4 == 265) {
            __lastLabel__ = 2; __label__ = 40;;
          }
          else if ($4 == 266) {
            __lastLabel__ = 2; __label__ = 41;;
          }
          else {
          __lastLabel__ = 2; __label__ = 42;;
          }
          
          $sw_default$$sw_bb$$sw_bb8$$sw_bb11$$sw_bb13$$sw_bb14$$sw_bb15$$sw_bb16$$sw_bb21$$sw_bb22$$sw_bb23$$sw_bb25$$sw_bb27$$sw_bb29$$sw_bb31$$sw_bb33$$sw_bb34$$sw_bb35$$sw_bb36$$sw_bb38$$sw_bb43$$sw_bb53$$sw_bb56$$sw_bb61$$sw_bb62$$sw_bb69$$sw_bb70$6: do { 
            if (__label__ == 42) {
  
              _exit(0); //@line 732 "speak.cpp"
              __label__ = 38;break $sw_default$$sw_bb$$sw_bb8$$sw_bb11$$sw_bb13$$sw_bb14$$sw_bb15$$sw_bb16$$sw_bb21$$sw_bb22$$sw_bb23$$sw_bb25$$sw_bb27$$sw_bb29$$sw_bb31$$sw_bb33$$sw_bb34$$sw_bb35$$sw_bb36$$sw_bb38$$sw_bb43$$sw_bb53$$sw_bb56$$sw_bb61$$sw_bb62$$sw_bb69$$sw_bb70$6; //@line 733 "speak.cpp"
            }
            else if (__label__ == 3) {
  
              HEAP32[((_option_multibyte)>>2)]=2; //@line 601 "speak.cpp"
              var $6=$optarg2; //@line 602 "speak.cpp"
              var $call3=_sscanf($6, ((__str63715)&4294967295), allocate([$value,0,0,0], ["i32*",0,0,0], ALLOC_STACK)); //@line 602 "speak.cpp"
              var $cmp4=((($call3))|0)==1; //@line 602 "speak.cpp"
              var $7=HEAP32[(($value)>>2)]; //@line 602 "speak.cpp"
              var $cmp5=((($7))|0) <= 4; //@line 602 "speak.cpp"
              var $or_cond=($cmp4) & ($cmp5); //@line 602 "speak.cpp"
              if (!($or_cond)) { __label__ = 38;break $sw_default$$sw_bb$$sw_bb8$$sw_bb11$$sw_bb13$$sw_bb14$$sw_bb15$$sw_bb16$$sw_bb21$$sw_bb22$$sw_bb23$$sw_bb25$$sw_bb27$$sw_bb29$$sw_bb31$$sw_bb33$$sw_bb34$$sw_bb35$$sw_bb36$$sw_bb38$$sw_bb43$$sw_bb53$$sw_bb56$$sw_bb61$$sw_bb62$$sw_bb69$$sw_bb70$6; } //@line 602 "speak.cpp"
  
              var $8=HEAP32[(($value)>>2)]; //@line 603 "speak.cpp"
              HEAP32[((_option_multibyte)>>2)]=$8; //@line 603 "speak.cpp"
              __label__ = 38;break $sw_default$$sw_bb$$sw_bb8$$sw_bb11$$sw_bb13$$sw_bb14$$sw_bb15$$sw_bb16$$sw_bb21$$sw_bb22$$sw_bb23$$sw_bb25$$sw_bb27$$sw_bb29$$sw_bb31$$sw_bb33$$sw_bb34$$sw_bb35$$sw_bb36$$sw_bb38$$sw_bb43$$sw_bb53$$sw_bb56$$sw_bb61$$sw_bb62$$sw_bb69$$sw_bb70$6; //@line 603 "speak.cpp"
            }
            else if (__label__ == 5) {
  
              var $9=$argv_addr; //@line 607 "speak.cpp"
              var $arrayidx9=(($9)&4294967295); //@line 607 "speak.cpp"
              var $10=HEAP32[(($arrayidx9)>>2)]; //@line 607 "speak.cpp"
              var $11=$data_path; //@line 607 "speak.cpp"
              __ZL9init_pathPcS_($10, $11); //@line 607 "speak.cpp"
              var $12=HEAP32[((_version_string)>>2)]; //@line 608 "speak.cpp"
              var $13=$_ZL9help_text; //@line 608 "speak.cpp"
              var $call10=_printf(((__str21)&4294967295), allocate([$12,0,0,0,((_path_home)&4294967295),0,0,0,$13,0,0,0], ["i8*",0,0,0,"i8*",0,0,0,"i8*",0,0,0], ALLOC_STACK)); //@line 608 "speak.cpp"
              _exit(0); //@line 609 "speak.cpp"
              __label__ = 38;break $sw_default$$sw_bb$$sw_bb8$$sw_bb11$$sw_bb13$$sw_bb14$$sw_bb15$$sw_bb16$$sw_bb21$$sw_bb22$$sw_bb23$$sw_bb25$$sw_bb27$$sw_bb29$$sw_bb31$$sw_bb33$$sw_bb34$$sw_bb35$$sw_bb36$$sw_bb38$$sw_bb43$$sw_bb53$$sw_bb56$$sw_bb61$$sw_bb62$$sw_bb69$$sw_bb70$6; //@line 610 "speak.cpp"
            }
            else if (__label__ == 6) {
  
              var $14=$optarg2; //@line 613 "speak.cpp"
              var $call12=_atoi($14); //@line 613 "speak.cpp"
              HEAP32[((_option_capitals)>>2)]=$call12; //@line 613 "speak.cpp"
              __label__ = 38;break $sw_default$$sw_bb$$sw_bb8$$sw_bb11$$sw_bb13$$sw_bb14$$sw_bb15$$sw_bb16$$sw_bb21$$sw_bb22$$sw_bb23$$sw_bb25$$sw_bb27$$sw_bb29$$sw_bb31$$sw_bb33$$sw_bb34$$sw_bb35$$sw_bb36$$sw_bb38$$sw_bb43$$sw_bb53$$sw_bb56$$sw_bb61$$sw_bb62$$sw_bb69$$sw_bb70$6; //@line 614 "speak.cpp"
            }
            else if (__label__ == 7) {
  
              HEAP32[((_option_phonemes)>>2)]=1; //@line 617 "speak.cpp"
              __label__ = 38;break $sw_default$$sw_bb$$sw_bb8$$sw_bb11$$sw_bb13$$sw_bb14$$sw_bb15$$sw_bb16$$sw_bb21$$sw_bb22$$sw_bb23$$sw_bb25$$sw_bb27$$sw_bb29$$sw_bb31$$sw_bb33$$sw_bb34$$sw_bb35$$sw_bb36$$sw_bb38$$sw_bb43$$sw_bb53$$sw_bb56$$sw_bb61$$sw_bb62$$sw_bb69$$sw_bb70$6; //@line 618 "speak.cpp"
            }
            else if (__label__ == 8) {
  
              HEAP32[((_option_phonemes)>>2)]=2; //@line 621 "speak.cpp"
              __label__ = 38;break $sw_default$$sw_bb$$sw_bb8$$sw_bb11$$sw_bb13$$sw_bb14$$sw_bb15$$sw_bb16$$sw_bb21$$sw_bb22$$sw_bb23$$sw_bb25$$sw_bb27$$sw_bb29$$sw_bb31$$sw_bb33$$sw_bb34$$sw_bb35$$sw_bb36$$sw_bb38$$sw_bb43$$sw_bb53$$sw_bb56$$sw_bb61$$sw_bb62$$sw_bb69$$sw_bb70$6; //@line 622 "speak.cpp"
            }
            else if (__label__ == 9) {
  
              HEAP32[((_option_ssml)>>2)]=1; //@line 625 "speak.cpp"
              __label__ = 38;break $sw_default$$sw_bb$$sw_bb8$$sw_bb11$$sw_bb13$$sw_bb14$$sw_bb15$$sw_bb16$$sw_bb21$$sw_bb22$$sw_bb23$$sw_bb25$$sw_bb27$$sw_bb29$$sw_bb31$$sw_bb33$$sw_bb34$$sw_bb35$$sw_bb36$$sw_bb38$$sw_bb43$$sw_bb53$$sw_bb56$$sw_bb61$$sw_bb62$$sw_bb69$$sw_bb70$6; //@line 626 "speak.cpp"
            }
            else if (__label__ == 10) {
  
              var $15=$optarg2; //@line 629 "speak.cpp"
              var $call17=_atoi($15); //@line 629 "speak.cpp"
              $pitch_adjustment=$call17; //@line 629 "speak.cpp"
              var $16=$pitch_adjustment; //@line 630 "speak.cpp"
              var $cmp18=((($16))|0) > 99; //@line 630 "speak.cpp"
              if (!($cmp18)) { __label__ = 38;break $sw_default$$sw_bb$$sw_bb8$$sw_bb11$$sw_bb13$$sw_bb14$$sw_bb15$$sw_bb16$$sw_bb21$$sw_bb22$$sw_bb23$$sw_bb25$$sw_bb27$$sw_bb29$$sw_bb31$$sw_bb33$$sw_bb34$$sw_bb35$$sw_bb36$$sw_bb38$$sw_bb43$$sw_bb53$$sw_bb56$$sw_bb61$$sw_bb62$$sw_bb69$$sw_bb70$6; } //@line 630 "speak.cpp"
  
              $pitch_adjustment=99; //@line 630 "speak.cpp"
              __label__ = 38;break $sw_default$$sw_bb$$sw_bb8$$sw_bb11$$sw_bb13$$sw_bb14$$sw_bb15$$sw_bb16$$sw_bb21$$sw_bb22$$sw_bb23$$sw_bb25$$sw_bb27$$sw_bb29$$sw_bb31$$sw_bb33$$sw_bb34$$sw_bb35$$sw_bb36$$sw_bb38$$sw_bb43$$sw_bb53$$sw_bb56$$sw_bb61$$sw_bb62$$sw_bb69$$sw_bb70$6; //@line 630 "speak.cpp"
            }
            else if (__label__ == 12) {
  
              HEAP32[((_quiet)>>2)]=1; //@line 634 "speak.cpp"
              __label__ = 38;break $sw_default$$sw_bb$$sw_bb8$$sw_bb11$$sw_bb13$$sw_bb14$$sw_bb15$$sw_bb16$$sw_bb21$$sw_bb22$$sw_bb23$$sw_bb25$$sw_bb27$$sw_bb29$$sw_bb31$$sw_bb33$$sw_bb34$$sw_bb35$$sw_bb36$$sw_bb38$$sw_bb43$$sw_bb53$$sw_bb56$$sw_bb61$$sw_bb62$$sw_bb69$$sw_bb70$6; //@line 635 "speak.cpp"
            }
            else if (__label__ == 13) {
  
              var $17=$optarg2; //@line 638 "speak.cpp"
              __Z8strncpy0PcPKci($arraydecay, $17, 200); //@line 638 "speak.cpp"
              __label__ = 38;break $sw_default$$sw_bb$$sw_bb8$$sw_bb11$$sw_bb13$$sw_bb14$$sw_bb15$$sw_bb16$$sw_bb21$$sw_bb22$$sw_bb23$$sw_bb25$$sw_bb27$$sw_bb29$$sw_bb31$$sw_bb33$$sw_bb34$$sw_bb35$$sw_bb36$$sw_bb38$$sw_bb43$$sw_bb53$$sw_bb56$$sw_bb61$$sw_bb62$$sw_bb69$$sw_bb70$6; //@line 639 "speak.cpp"
            }
            else if (__label__ == 14) {
  
              HEAP32[(($value)>>2)]=0; //@line 642 "speak.cpp"
              var $18=$optarg2; //@line 643 "speak.cpp"
              var $call24=_atoi($18); //@line 643 "speak.cpp"
              HEAP32[(($value)>>2)]=$call24; //@line 643 "speak.cpp"
              var $19=HEAP32[(($value)>>2)]; //@line 644 "speak.cpp"
              HEAP32[((_option_linelength)>>2)]=$19; //@line 644 "speak.cpp"
              __label__ = 38;break $sw_default$$sw_bb$$sw_bb8$$sw_bb11$$sw_bb13$$sw_bb14$$sw_bb15$$sw_bb16$$sw_bb21$$sw_bb22$$sw_bb23$$sw_bb25$$sw_bb27$$sw_bb29$$sw_bb31$$sw_bb33$$sw_bb34$$sw_bb35$$sw_bb36$$sw_bb38$$sw_bb43$$sw_bb53$$sw_bb56$$sw_bb61$$sw_bb62$$sw_bb69$$sw_bb70$6; //@line 645 "speak.cpp"
            }
            else if (__label__ == 15) {
  
              var $20=$optarg2; //@line 648 "speak.cpp"
              var $call26=_atoi($20); //@line 648 "speak.cpp"
              $amp=$call26; //@line 648 "speak.cpp"
              __label__ = 38;break $sw_default$$sw_bb$$sw_bb8$$sw_bb11$$sw_bb13$$sw_bb14$$sw_bb15$$sw_bb16$$sw_bb21$$sw_bb22$$sw_bb23$$sw_bb25$$sw_bb27$$sw_bb29$$sw_bb31$$sw_bb33$$sw_bb34$$sw_bb35$$sw_bb36$$sw_bb38$$sw_bb43$$sw_bb53$$sw_bb56$$sw_bb61$$sw_bb62$$sw_bb69$$sw_bb70$6; //@line 649 "speak.cpp"
            }
            else if (__label__ == 16) {
  
              var $21=$optarg2; //@line 652 "speak.cpp"
              var $call28=_atoi($21); //@line 652 "speak.cpp"
              $speed=$call28; //@line 652 "speak.cpp"
              __label__ = 38;break $sw_default$$sw_bb$$sw_bb8$$sw_bb11$$sw_bb13$$sw_bb14$$sw_bb15$$sw_bb16$$sw_bb21$$sw_bb22$$sw_bb23$$sw_bb25$$sw_bb27$$sw_bb29$$sw_bb31$$sw_bb33$$sw_bb34$$sw_bb35$$sw_bb36$$sw_bb38$$sw_bb43$$sw_bb53$$sw_bb56$$sw_bb61$$sw_bb62$$sw_bb69$$sw_bb70$6; //@line 653 "speak.cpp"
            }
            else if (__label__ == 17) {
  
              var $22=$optarg2; //@line 656 "speak.cpp"
              var $call30=_atoi($22); //@line 656 "speak.cpp"
              $wordgap=$call30; //@line 656 "speak.cpp"
              __label__ = 38;break $sw_default$$sw_bb$$sw_bb8$$sw_bb11$$sw_bb13$$sw_bb14$$sw_bb15$$sw_bb16$$sw_bb21$$sw_bb22$$sw_bb23$$sw_bb25$$sw_bb27$$sw_bb29$$sw_bb31$$sw_bb33$$sw_bb34$$sw_bb35$$sw_bb36$$sw_bb38$$sw_bb43$$sw_bb53$$sw_bb56$$sw_bb61$$sw_bb62$$sw_bb69$$sw_bb70$6; //@line 657 "speak.cpp"
            }
            else if (__label__ == 18) {
  
              var $23=$optarg2; //@line 660 "speak.cpp"
              __Z8strncpy0PcPKci($arraydecay32, $23, 40); //@line 660 "speak.cpp"
              __label__ = 38;break $sw_default$$sw_bb$$sw_bb8$$sw_bb11$$sw_bb13$$sw_bb14$$sw_bb15$$sw_bb16$$sw_bb21$$sw_bb22$$sw_bb23$$sw_bb25$$sw_bb27$$sw_bb29$$sw_bb31$$sw_bb33$$sw_bb34$$sw_bb35$$sw_bb36$$sw_bb38$$sw_bb43$$sw_bb53$$sw_bb56$$sw_bb61$$sw_bb62$$sw_bb69$$sw_bb70$6; //@line 661 "speak.cpp"
            }
            else if (__label__ == 19) {
  
              HEAP32[((_option_waveout)>>2)]=1; //@line 664 "speak.cpp"
              var $24=$optarg2; //@line 665 "speak.cpp"
              __Z8strncpy0PcPKci(((_wavefile)&4294967295), $24, 200); //@line 665 "speak.cpp"
              __label__ = 38;break $sw_default$$sw_bb$$sw_bb8$$sw_bb11$$sw_bb13$$sw_bb14$$sw_bb15$$sw_bb16$$sw_bb21$$sw_bb22$$sw_bb23$$sw_bb25$$sw_bb27$$sw_bb29$$sw_bb31$$sw_bb33$$sw_bb34$$sw_bb35$$sw_bb36$$sw_bb38$$sw_bb43$$sw_bb53$$sw_bb56$$sw_bb61$$sw_bb62$$sw_bb69$$sw_bb70$6; //@line 666 "speak.cpp"
            }
            else if (__label__ == 20) {
  
              HEAP32[((_option_endpause)>>2)]=0; //@line 669 "speak.cpp"
              __label__ = 38;break $sw_default$$sw_bb$$sw_bb8$$sw_bb11$$sw_bb13$$sw_bb14$$sw_bb15$$sw_bb16$$sw_bb21$$sw_bb22$$sw_bb23$$sw_bb25$$sw_bb27$$sw_bb29$$sw_bb31$$sw_bb33$$sw_bb34$$sw_bb35$$sw_bb36$$sw_bb38$$sw_bb43$$sw_bb53$$sw_bb56$$sw_bb61$$sw_bb62$$sw_bb69$$sw_bb70$6; //@line 670 "speak.cpp"
            }
            else if (__label__ == 21) {
  
              $flag_stdin=1; //@line 673 "speak.cpp"
              __label__ = 38;break $sw_default$$sw_bb$$sw_bb8$$sw_bb11$$sw_bb13$$sw_bb14$$sw_bb15$$sw_bb16$$sw_bb21$$sw_bb22$$sw_bb23$$sw_bb25$$sw_bb27$$sw_bb29$$sw_bb31$$sw_bb33$$sw_bb34$$sw_bb35$$sw_bb36$$sw_bb38$$sw_bb43$$sw_bb53$$sw_bb56$$sw_bb61$$sw_bb62$$sw_bb69$$sw_bb70$6; //@line 674 "speak.cpp"
            }
            else if (__label__ == 22) {
  
              HEAP32[((_option_waveout)>>2)]=1; //@line 677 "speak.cpp"
              var $src$, $dest$, $stop$, $stop4$, $fast$;
              $src$ = ((__str12)&4294967295);
              $dest$ = ((_wavefile)&4294967295);
              $stop$ = $src$ + 7;
              if (($dest$%4) == ($src$%4) && 7 > 8) {
                while ($src$%4 !== 0 && $src$ < $stop$) {
                  ; HEAP8[$dest$++] = HEAP8[$src$++];
                }
                $src$ >>= 2;
                $dest$ >>= 2;
                $stop4$ = $stop$ >> 2;
                while ($src$ < $stop4$) {
              ;;;    HEAP32[$dest$++] = HEAP32[$src$++];
                }
                $src$ <<= 2;
                $dest$ <<= 2;
              }while ($src$ < $stop$) {
                ; HEAP8[$dest$++] = HEAP8[$src$++];
              }; //@line 678 "speak.cpp"
              __label__ = 38;break $sw_default$$sw_bb$$sw_bb8$$sw_bb11$$sw_bb13$$sw_bb14$$sw_bb15$$sw_bb16$$sw_bb21$$sw_bb22$$sw_bb23$$sw_bb25$$sw_bb27$$sw_bb29$$sw_bb31$$sw_bb33$$sw_bb34$$sw_bb35$$sw_bb36$$sw_bb38$$sw_bb43$$sw_bb53$$sw_bb56$$sw_bb61$$sw_bb62$$sw_bb69$$sw_bb70$6; //@line 679 "speak.cpp"
            }
            else if (__label__ == 23) {
  
              var $cmp39=((($5))|0)!=0; //@line 683 "speak.cpp"
              if ($cmp39) { __label__ = 24;; } else { __label__ = 25;; } //@line 683 "speak.cpp"
              if (__label__ == 24) {
  
                var $25=$optarg2; //@line 684 "speak.cpp"
                __Z8strncpy0PcPKci($arraydecay41, $25, 40); //@line 684 "speak.cpp"
                ; //@line 684 "speak.cpp"
              }
  
              var $26=$c; //@line 685 "speak.cpp"
              $flag_compile=$26; //@line 685 "speak.cpp"
              __label__ = 38;break $sw_default$$sw_bb$$sw_bb8$$sw_bb11$$sw_bb13$$sw_bb14$$sw_bb15$$sw_bb16$$sw_bb21$$sw_bb22$$sw_bb23$$sw_bb25$$sw_bb27$$sw_bb29$$sw_bb31$$sw_bb33$$sw_bb34$$sw_bb35$$sw_bb36$$sw_bb38$$sw_bb43$$sw_bb53$$sw_bb56$$sw_bb61$$sw_bb62$$sw_bb69$$sw_bb70$6; //@line 686 "speak.cpp"
            }
            else if (__label__ == 26) {
  
              HEAP32[((_option_punctuation)>>2)]=1; //@line 689 "speak.cpp"
              var $27=$optarg2; //@line 690 "speak.cpp"
              var $cmp44=((($27))|0)!=0; //@line 690 "speak.cpp"
              if (!($cmp44)) { __label__ = 38;break $sw_default$$sw_bb$$sw_bb8$$sw_bb11$$sw_bb13$$sw_bb14$$sw_bb15$$sw_bb16$$sw_bb21$$sw_bb22$$sw_bb23$$sw_bb25$$sw_bb27$$sw_bb29$$sw_bb31$$sw_bb33$$sw_bb34$$sw_bb35$$sw_bb36$$sw_bb38$$sw_bb43$$sw_bb53$$sw_bb56$$sw_bb61$$sw_bb62$$sw_bb69$$sw_bb70$6; } //@line 690 "speak.cpp"
  
              $ix=0; //@line 692 "speak.cpp"
              __lastLabel__ = 27; ; //@line 693 "speak.cpp"
              while(1) { 
  
                var $28=__lastLabel__ == 30 ? $inc : (0); //@line 693 "speak.cpp"
                var $cmp47=((($28))|0) < 60; //@line 693 "speak.cpp"
                if (!($cmp47)) { __label__ = 31;break ; } //@line 693 "speak.cpp"
  
                var $29=$ix; //@line 693 "speak.cpp"
                var $30=$optarg2; //@line 693 "speak.cpp"
                var $arrayidx48=(($30+$29)&4294967295); //@line 693 "speak.cpp"
                var $31=HEAP8[($arrayidx48)]; //@line 693 "speak.cpp"
                var $conv=(tempInt=(($31)),(tempInt>=128?tempInt-256:tempInt)); //@line 693 "speak.cpp"
                var $32=$ix; //@line 693 "speak.cpp"
                var $arrayidx49=((_option_punctlist+$32*4)&4294967295); //@line 693 "speak.cpp"
                HEAP32[(($arrayidx49)>>2)]=$conv; //@line 693 "speak.cpp"
                var $cmp50=((($conv))|0)!=0; //@line 693 "speak.cpp"
                if (!($cmp50)) { __label__ = 31;break ; }
  
                var $33=$ix; //@line 693 "speak.cpp"
                var $inc=((($33)+1)&4294967295); //@line 693 "speak.cpp"
                $ix=$inc; //@line 693 "speak.cpp"
                __lastLabel__ = 30; __label__ = 28;continue ; //@line 693 "speak.cpp"
              }
  
              HEAP32[((((_option_punctlist+236)&4294967295))>>2)]=0; //@line 694 "speak.cpp"
              HEAP32[((_option_punctuation)>>2)]=2; //@line 695 "speak.cpp"
              __label__ = 38;break $sw_default$$sw_bb$$sw_bb8$$sw_bb11$$sw_bb13$$sw_bb14$$sw_bb15$$sw_bb16$$sw_bb21$$sw_bb22$$sw_bb23$$sw_bb25$$sw_bb27$$sw_bb29$$sw_bb31$$sw_bb33$$sw_bb34$$sw_bb35$$sw_bb36$$sw_bb38$$sw_bb43$$sw_bb53$$sw_bb56$$sw_bb61$$sw_bb62$$sw_bb69$$sw_bb70$6; //@line 696 "speak.cpp"
            }
            else if (__label__ == 32) {
  
              var $34=$argv_addr; //@line 700 "speak.cpp"
              var $arrayidx54=(($34)&4294967295); //@line 700 "speak.cpp"
              var $35=HEAP32[(($arrayidx54)>>2)]; //@line 700 "speak.cpp"
              var $36=$data_path; //@line 700 "speak.cpp"
              __ZL9init_pathPcS_($35, $36); //@line 700 "speak.cpp"
              var $37=HEAP32[((__impure_ptr)>>2)]; //@line 701 "speak.cpp"
              var $_stdout55=(($37+8)&4294967295); //@line 701 "speak.cpp"
              var $38=HEAP32[(($_stdout55)>>2)]; //@line 701 "speak.cpp"
              var $39=$optarg2; //@line 701 "speak.cpp"
              __Z13DisplayVoicesP7__sFILEPc($38, $39); //@line 701 "speak.cpp"
              _exit(0); //@line 702 "speak.cpp"
              var $_pr=$optarg2; //@line 705 "speak.cpp"
              __lastLabel__ = 32; __label__ = 33;break $sw_default$$sw_bb$$sw_bb8$$sw_bb11$$sw_bb13$$sw_bb14$$sw_bb15$$sw_bb16$$sw_bb21$$sw_bb22$$sw_bb23$$sw_bb25$$sw_bb27$$sw_bb29$$sw_bb31$$sw_bb33$$sw_bb34$$sw_bb35$$sw_bb36$$sw_bb38$$sw_bb43$$sw_bb53$$sw_bb56$$sw_bb61$$sw_bb62$$sw_bb69$$sw_bb70$6; //@line 702 "speak.cpp"
            }
            else if (__label__ == 36) {
  
              var $42=$optarg2; //@line 712 "speak.cpp"
              $data_path=$42; //@line 712 "speak.cpp"
              __label__ = 38;break $sw_default$$sw_bb$$sw_bb8$$sw_bb11$$sw_bb13$$sw_bb14$$sw_bb15$$sw_bb16$$sw_bb21$$sw_bb22$$sw_bb23$$sw_bb25$$sw_bb27$$sw_bb29$$sw_bb31$$sw_bb33$$sw_bb34$$sw_bb35$$sw_bb36$$sw_bb38$$sw_bb43$$sw_bb53$$sw_bb56$$sw_bb61$$sw_bb62$$sw_bb69$$sw_bb70$6; //@line 713 "speak.cpp"
            }
            else if (__label__ == 37) {
  
              var $43=$optarg2; //@line 716 "speak.cpp"
              var $call63=_fopen($43, ((__str11422)&4294967295)); //@line 716 "speak.cpp"
              HEAP32[((_f_trans)>>2)]=$call63; //@line 716 "speak.cpp"
              var $cmp64=((($call63))|0)==0; //@line 716 "speak.cpp"
              if (!($cmp64)) { __label__ = 38;break $sw_default$$sw_bb$$sw_bb8$$sw_bb11$$sw_bb13$$sw_bb14$$sw_bb15$$sw_bb16$$sw_bb21$$sw_bb22$$sw_bb23$$sw_bb25$$sw_bb27$$sw_bb29$$sw_bb31$$sw_bb33$$sw_bb34$$sw_bb35$$sw_bb36$$sw_bb38$$sw_bb43$$sw_bb53$$sw_bb56$$sw_bb61$$sw_bb62$$sw_bb69$$sw_bb70$6; } //@line 716 "speak.cpp"
  
              var $46=HEAP32[((__impure_ptr)>>2)]; //@line 718 "speak.cpp"
              var $_stderr=(($46+12)&4294967295); //@line 718 "speak.cpp"
              var $47=HEAP32[(($_stderr)>>2)]; //@line 718 "speak.cpp"
              var $48=$optarg2; //@line 718 "speak.cpp"
              var $call66=_fprintf($47, ((__str23)&4294967295), allocate([$48,0,0,0], ["i8*",0,0,0], ALLOC_STACK)); //@line 718 "speak.cpp"
              var $49=HEAP32[((__impure_ptr)>>2)]; //@line 719 "speak.cpp"
              var $_stderr67=(($49+12)&4294967295); //@line 719 "speak.cpp"
              var $50=HEAP32[(($_stderr67)>>2)]; //@line 719 "speak.cpp"
              HEAP32[((_f_trans)>>2)]=$50; //@line 719 "speak.cpp"
              __label__ = 38;break $sw_default$$sw_bb$$sw_bb8$$sw_bb11$$sw_bb13$$sw_bb14$$sw_bb15$$sw_bb16$$sw_bb21$$sw_bb22$$sw_bb23$$sw_bb25$$sw_bb27$$sw_bb29$$sw_bb31$$sw_bb33$$sw_bb34$$sw_bb35$$sw_bb36$$sw_bb38$$sw_bb43$$sw_bb53$$sw_bb56$$sw_bb61$$sw_bb62$$sw_bb69$$sw_bb70$6; //@line 720 "speak.cpp"
            }
            else if (__label__ == 40) {
  
              HEAP32[((_option_mbrola_phonemes)>>2)]=16; //@line 724 "speak.cpp"
              __label__ = 38;break $sw_default$$sw_bb$$sw_bb8$$sw_bb11$$sw_bb13$$sw_bb14$$sw_bb15$$sw_bb16$$sw_bb21$$sw_bb22$$sw_bb23$$sw_bb25$$sw_bb27$$sw_bb29$$sw_bb31$$sw_bb33$$sw_bb34$$sw_bb35$$sw_bb36$$sw_bb38$$sw_bb43$$sw_bb53$$sw_bb56$$sw_bb61$$sw_bb62$$sw_bb69$$sw_bb70$6; //@line 725 "speak.cpp"
            }
            else if (__label__ == 41) {
  
              HEAP32[((_option_phonemes)>>2)]=3; //@line 728 "speak.cpp"
              __label__ = 38;break $sw_default$$sw_bb$$sw_bb8$$sw_bb11$$sw_bb13$$sw_bb14$$sw_bb15$$sw_bb16$$sw_bb21$$sw_bb22$$sw_bb23$$sw_bb25$$sw_bb27$$sw_bb29$$sw_bb31$$sw_bb33$$sw_bb34$$sw_bb35$$sw_bb36$$sw_bb38$$sw_bb43$$sw_bb53$$sw_bb56$$sw_bb61$$sw_bb62$$sw_bb69$$sw_bb70$6; //@line 729 "speak.cpp"
            }
          } while(0);
          if (__label__ == 33) {
  
            var $40=__lastLabel__ == 32 ? $_pr : ($5); //@line 705 "speak.cpp"
            var $cmp57=((($40))|0)==0; //@line 705 "speak.cpp"
            if ($cmp57) { __label__ = 34;; } else { __label__ = 35;; } //@line 705 "speak.cpp"
            if (__label__ == 34) {
  
              HEAP32[((_samples_split)>>2)]=30; //@line 706 "speak.cpp"
              ; //@line 706 "speak.cpp"
            }
            else if (__label__ == 35) {
  
              var $41=$optarg2; //@line 708 "speak.cpp"
              var $call59=_atoi($41); //@line 708 "speak.cpp"
              HEAP32[((_samples_split)>>2)]=$call59; //@line 708 "speak.cpp"
              ;
            }
          }
  
          var $44=$argc_addr; //@line 588 "speak.cpp"
          var $45=$argv_addr; //@line 588 "speak.cpp"
          var $call=_getopt_long($44, $45, ((__str19)&4294967295), ((__ZZ4mainE12long_options)&4294967295), $option_index); //@line 588 "speak.cpp"
          $c=$call; //@line 588 "speak.cpp"
          var $cmp=((($call))|0)==-1; //@line 592 "speak.cpp"
          if ($cmp) { __lastLabel__ = 38; __label__ = 43;break $while_end71$$if_end_lr_ph$2; } else { __lastLabel__ = 38; __label__ = 2;continue $if_end$4; } //@line 592 "speak.cpp"
        }
      }
    } while(0);
  
    var $51=$argv_addr; //@line 736 "speak.cpp"
    var $arrayidx72=(($51)&4294967295); //@line 736 "speak.cpp"
    var $52=HEAP32[(($arrayidx72)>>2)]; //@line 736 "speak.cpp"
    var $53=$data_path; //@line 736 "speak.cpp"
    __ZL9init_pathPcS_($52, $53); //@line 736 "speak.cpp"
    __ZL10initialisev(); //@line 737 "speak.cpp"
    var $54=$flag_compile; //@line 740 "speak.cpp"
    var $tobool=((($54))|0)!=0; //@line 740 "speak.cpp"
    if ($tobool) { __label__ = 44;; } else { __label__ = 45;; } //@line 740 "speak.cpp"
    if (__label__ == 44) {
  
      var $arraydecay75=(($voicename)&4294967295); //@line 742 "speak.cpp"
      var $call76=__Z9LoadVoicePKci($arraydecay75, 5); //@line 742 "speak.cpp"
      var $55=$flag_compile; //@line 758 "speak.cpp"
      var $and=($55) & 1; //@line 758 "speak.cpp"
      var $call77=__Z17CompileDictionaryPKcS0_P7__sFILEPci(0, ((_dictionary_name)&4294967295), 0, 0, $and); //@line 758 "speak.cpp"
      _exit(0); //@line 761 "speak.cpp"
      ; //@line 762 "speak.cpp"
    }
  
    var $arrayidx79=(($voicename)&4294967295); //@line 765 "speak.cpp"
    var $56=HEAP8[($arrayidx79)]; //@line 765 "speak.cpp"
    var $conv80=(tempInt=(($56)),(tempInt>=128?tempInt-256:tempInt)); //@line 765 "speak.cpp"
    var $cmp81=((($conv80))|0)==0; //@line 765 "speak.cpp"
    if ($cmp81) { __label__ = 46;; } else { __label__ = 47;; } //@line 765 "speak.cpp"
    if (__label__ == 46) {
  
      var $arraydecay83=(($voicename)&4294967295); //@line 766 "speak.cpp"
      var $src$, $dest$, $stop$, $stop4$, $fast$;
      $src$ = ((__str45697)&4294967295);
      $dest$ = $arraydecay83;
      $stop$ = $src$ + 8;
      if (($dest$%4) == ($src$%4) && 8 > 8) {
        while ($src$%4 !== 0 && $src$ < $stop$) {
          ; HEAP8[$dest$++] = HEAP8[$src$++];
        }
        $src$ >>= 2;
        $dest$ >>= 2;
        $stop4$ = $stop$ >> 2;
        while ($src$ < $stop4$) {
      ;;;    HEAP32[$dest$++] = HEAP32[$src$++];
        }
        $src$ <<= 2;
        $dest$ <<= 2;
      }while ($src$ < $stop$) {
        ; HEAP8[$dest$++] = HEAP8[$src$++];
      }; //@line 766 "speak.cpp"
      ; //@line 766 "speak.cpp"
    }
  
    var $arraydecay86=(($voicename)&4294967295); //@line 768 "speak.cpp"
    var $call87=__Z14SetVoiceByNamePKc($arraydecay86); //@line 768 "speak.cpp"
    var $cmp88=((($call87))|0)!=0; //@line 768 "speak.cpp"
    if ($cmp88) { __label__ = 48;; } else { __label__ = 50;; } //@line 768 "speak.cpp"
    $if_then89$$if_end98$58: do { 
      if (__label__ == 48) {
  
        var $57=$voice_select; //@line 770 "speak.cpp"
        var $dest$, $stop$, $stop4$, $fast$, $value4$;
        $dest$ = $57;
        $stop$ = $dest$ + 24;
        $value4$ = 0;
        if ($value4$ < 0) $value4$ += 256;
        $value4$ = $value4$ + ($value4$<<8) + ($value4$<<16) + ($value4$*16777216);
        while ($dest$%4 !== 0 && $dest$ < $stop$) {
          ;; HEAP8[$dest$++] = 0;
        }
        $dest$ >>= 2;
        $stop4$ = $stop$ >> 2;
        while ($dest$ < $stop4$) {
        ;;;;;;;  HEAP32[$dest$++] = $value4$;
        }
        $dest$ <<= 2;
        while ($dest$ < $stop$) {
          ;; HEAP8[$dest$++] = 0;
        }; //@line 770 "speak.cpp"
        var $arraydecay90=(($voicename)&4294967295); //@line 771 "speak.cpp"
        var $languages=(($voice_select+4)&4294967295); //@line 771 "speak.cpp"
        HEAP32[(($languages)>>2)]=$arraydecay90; //@line 771 "speak.cpp"
        var $call91=__Z20SetVoiceByPropertiesP12espeak_VOICE($voice_select); //@line 772 "speak.cpp"
        var $cmp92=((($call91))|0)!=0; //@line 772 "speak.cpp"
        if (!($cmp92)) { __label__ = 50;break $if_then89$$if_end98$58; } //@line 772 "speak.cpp"
  
        var $58=HEAP32[((__impure_ptr)>>2)]; //@line 774 "speak.cpp"
        var $_stderr94=(($58+12)&4294967295); //@line 774 "speak.cpp"
        var $59=HEAP32[(($_stderr94)>>2)]; //@line 774 "speak.cpp"
        var $60=$_ZZ4mainE8err_load; //@line 774 "speak.cpp"
        var $arraydecay95=(($voicename)&4294967295); //@line 774 "speak.cpp"
        var $call96=_fprintf($59, ((__str25)&4294967295), allocate([$60,0,0,0,$arraydecay95,0,0,0], ["i8*",0,0,0,"i8*",0,0,0], ALLOC_STACK)); //@line 774 "speak.cpp"
        _exit(2); //@line 775 "speak.cpp"
        ; //@line 776 "speak.cpp"
      }
    } while(0);
  
    var $61=$speed; //@line 779 "speak.cpp"
    __Z12SetParameteriii(1, $61, 0); //@line 779 "speak.cpp"
    var $62=$amp; //@line 780 "speak.cpp"
    __Z12SetParameteriii(2, $62, 0); //@line 780 "speak.cpp"
    var $63=HEAP32[((_option_capitals)>>2)]; //@line 781 "speak.cpp"
    __Z12SetParameteriii(6, $63, 0); //@line 781 "speak.cpp"
    var $64=HEAP32[((_option_punctuation)>>2)]; //@line 782 "speak.cpp"
    __Z12SetParameteriii(5, $64, 0); //@line 782 "speak.cpp"
    var $65=$wordgap; //@line 783 "speak.cpp"
    __Z12SetParameteriii(7, $65, 0); //@line 783 "speak.cpp"
    var $66=$pitch_adjustment; //@line 785 "speak.cpp"
    var $cmp99=((($66))|0)!=50; //@line 785 "speak.cpp"
    if ($cmp99) { __label__ = 51;; } else { __label__ = 52;; } //@line 785 "speak.cpp"
    if (__label__ == 51) {
  
      var $67=$pitch_adjustment; //@line 787 "speak.cpp"
      __Z12SetParameteriii(3, $67, 0); //@line 787 "speak.cpp"
      ; //@line 788 "speak.cpp"
    }
  
    var $68=HEAP32[((_voice)>>2)]; //@line 789 "speak.cpp"
    __Z13DoVoiceChangeP7voice_t($68); //@line 789 "speak.cpp"
    var $arrayidx102=(($filename)&4294967295); //@line 791 "speak.cpp"
    var $69=HEAP8[($arrayidx102)]; //@line 791 "speak.cpp"
    var $conv103=(tempInt=(($69)),(tempInt>=128?tempInt-256:tempInt)); //@line 791 "speak.cpp"
    var $cmp104=((($conv103))|0)==0; //@line 791 "speak.cpp"
    if ($cmp104) { __label__ = 53;; } else { __label__ = 59;; } //@line 791 "speak.cpp"
    $if_then105$$if_else116$65: do { 
      if (__label__ == 53) {
  
        var $70=HEAP32[((_optind)>>2)]; //@line 793 "speak.cpp"
        var $71=$argc_addr; //@line 793 "speak.cpp"
        var $cmp106=((($70))|0) < ((($71))|0); //@line 793 "speak.cpp"
        var $72=$flag_stdin; //@line 793 "speak.cpp"
        if ($cmp106) { __label__ = 54;; } else { __label__ = 57;; } //@line 793 "speak.cpp"
        if (__label__ == 54) {
  
          var $cmp108=((($72))|0)==0; //@line 793 "speak.cpp"
          if ($cmp108) { __label__ = 56;; } else { __label__ = 55;; } //@line 793 "speak.cpp"
          if (__label__ == 56) {
  
            var $75=HEAP32[((_optind)>>2)]; //@line 797 "speak.cpp"
            var $76=$argv_addr; //@line 797 "speak.cpp"
            var $arrayidx110=(($76+4*$75)&4294967295); //@line 797 "speak.cpp"
            var $77=HEAP32[(($arrayidx110)>>2)]; //@line 797 "speak.cpp"
            $p_text=$77; //@line 797 "speak.cpp"
            ; //@line 798 "speak.cpp"
          }
          else if (__label__ == 55) {
  
            var $73=HEAP32[((__impure_ptr)>>2)]; //@line 801 "speak.cpp"
            var $_stdin3=(($73+4)&4294967295); //@line 801 "speak.cpp"
            var $74=HEAP32[(($_stdin3)>>2)]; //@line 801 "speak.cpp"
            $f_text=$74; //@line 801 "speak.cpp"
            __lastLabel__ = 55; __label__ = 61;break $if_then105$$if_else116$65; //@line 802 "speak.cpp"
          }
        }
        else if (__label__ == 57) {
  
          var $78=HEAP32[((__impure_ptr)>>2)]; //@line 801 "speak.cpp"
          var $_stdin=(($78+4)&4294967295); //@line 801 "speak.cpp"
          var $79=HEAP32[(($_stdin)>>2)]; //@line 801 "speak.cpp"
          $f_text=$79; //@line 801 "speak.cpp"
          var $cmp112=((($72))|0)==0; //@line 802 "speak.cpp"
          if ($cmp112) { __lastLabel__ = 57; ; } else { __lastLabel__ = 57; __label__ = 61;break $if_then105$$if_else116$65; } //@line 802 "speak.cpp"
  
          HEAP32[((_option_linelength)>>2)]=-1; //@line 803 "speak.cpp"
          ; //@line 803 "speak.cpp"
        }
  
        var $_pr5=$f_text; //@line 811 "speak.cpp"
        __lastLabel__ = 60; ;
      }
      else if (__label__ == 59) {
  
        var $arraydecay117=(($filename)&4294967295); //@line 808 "speak.cpp"
        var $call118=_fopen($arraydecay117, ((__str51703)&4294967295)); //@line 808 "speak.cpp"
        $f_text=$call118; //@line 808 "speak.cpp"
        __lastLabel__ = 59; ;
      }
    } while(0);
  
    var $80=__lastLabel__ == 60 ? $_pr5 : (__lastLabel__ == 55 ? $74 : (__lastLabel__ == 57 ? $79 : ($call118))); //@line 811 "speak.cpp"
    var $cmp120=((($80))|0)==0; //@line 811 "speak.cpp"
    if ($cmp120) { __label__ = 62;; } else { __label__ = 64;; } //@line 811 "speak.cpp"
    $land_lhs_true121$$if_end127$77: do { 
      if (__label__ == 62) {
  
        var $81=$p_text; //@line 811 "speak.cpp"
        var $cmp122=((($81))|0)==0; //@line 811 "speak.cpp"
        if (!($cmp122)) { __label__ = 64;break $land_lhs_true121$$if_end127$77; } //@line 811 "speak.cpp"
  
        var $82=HEAP32[((__impure_ptr)>>2)]; //@line 813 "speak.cpp"
        var $_stderr124=(($82+12)&4294967295); //@line 813 "speak.cpp"
        var $83=HEAP32[(($_stderr124)>>2)]; //@line 813 "speak.cpp"
        var $84=$_ZZ4mainE8err_load; //@line 813 "speak.cpp"
        var $arraydecay125=(($filename)&4294967295); //@line 813 "speak.cpp"
        var $call126=_fprintf($83, ((__str27)&4294967295), allocate([$84,0,0,0,$arraydecay125,0,0,0], ["i8*",0,0,0,"i8*",0,0,0], ALLOC_STACK)); //@line 813 "speak.cpp"
        _exit(1); //@line 814 "speak.cpp"
        ; //@line 815 "speak.cpp"
      }
    } while(0);
  
    var $85=HEAP32[((_option_waveout)>>2)]; //@line 817 "speak.cpp"
    var $tobool128=((($85))|0)!=0; //@line 817 "speak.cpp"
    var $86=HEAP32[((_quiet)>>2)]; //@line 817 "speak.cpp"
    var $tobool129=((($86))|0)!=0; //@line 817 "speak.cpp"
    var $or_cond1=($tobool128) | ($tobool129); //@line 817 "speak.cpp"
    if ($or_cond1) { __label__ = 65;; } else { __label__ = 79;; } //@line 817 "speak.cpp"
    $if_then130$$if_else168$81: do { 
      if (__label__ == 65) {
  
        var $87=HEAP32[((_quiet)>>2)]; //@line 819 "speak.cpp"
        var $tobool131=((($87))|0)!=0; //@line 819 "speak.cpp"
        var $88=HEAP32[((_samplerate)>>2)]; //@line 822 "speak.cpp"
        if ($tobool131) { __label__ = 66;; } else { __label__ = 67;; } //@line 819 "speak.cpp"
        $if_then132$$if_else134$83: do { 
          if (__label__ == 66) {
  
            var $call133=__ZL12OpenWaveFilePKci(0, $88); //@line 822 "speak.cpp"
            HEAP32[((_option_waveout)>>2)]=1; //@line 823 "speak.cpp"
            ; //@line 824 "speak.cpp"
          }
          else if (__label__ == 67) {
  
            var $89=HEAP32[((_samples_split)>>2)]; //@line 828 "speak.cpp"
            var $mul=((($88)*60)&4294967295); //@line 828 "speak.cpp"
            var $mul135=((($mul)*($89))&4294967295); //@line 828 "speak.cpp"
            HEAP32[((_samples_split)>>2)]=$mul135; //@line 828 "speak.cpp"
            var $90=HEAP32[((_samples_split)>>2)]; //@line 830 "speak.cpp"
            var $tobool136=((($90))|0)!=0; //@line 830 "speak.cpp"
            if ($tobool136) { __label__ = 68;; } else { __label__ = 71;; } //@line 830 "speak.cpp"
            if (__label__ == 68) {
  
              var $call138=_strrchr(((_wavefile)&4294967295), 46); //@line 834 "speak.cpp"
              $extn=$call138; //@line 834 "speak.cpp"
              var $91=$extn; //@line 835 "speak.cpp"
              var $cmp139=((($91))|0)!=0; //@line 835 "speak.cpp"
              if (!($cmp139)) { __label__ = 73;break $if_then132$$if_else134$83; } //@line 835 "speak.cpp"
  
              var $call141=_strlen(((_wavefile)&4294967295)); //@line 835 "speak.cpp"
              var $add_ptr=((((_wavefile)&4294967295)+$call141)&4294967295); //@line 835 "speak.cpp"
              var $92=$extn; //@line 835 "speak.cpp"
              var $sub_ptr_lhs_cast=($add_ptr); //@line 835 "speak.cpp"
              var $sub_ptr_rhs_cast=($92); //@line 835 "speak.cpp"
              var $sub_ptr_sub=((($sub_ptr_lhs_cast)-($sub_ptr_rhs_cast))&4294967295); //@line 835 "speak.cpp"
              var $cmp142=((($sub_ptr_sub))|0) <= 4; //@line 835 "speak.cpp"
              if (!($cmp142)) { __label__ = 73;break $if_then132$$if_else134$83; } //@line 835 "speak.cpp"
  
              var $93=$extn; //@line 837 "speak.cpp"
              var $call144=_strcpy(((_filetype)&4294967295), $93); //@line 837 "speak.cpp"
              var $94=$extn; //@line 838 "speak.cpp"
              HEAP8[($94)]=0; //@line 838 "speak.cpp"
              ; //@line 839 "speak.cpp"
            }
            else if (__label__ == 71) {
  
              var $95=HEAP32[((_samplerate)>>2)]; //@line 842 "speak.cpp"
              var $call147=__ZL12OpenWaveFilePKci(((_wavefile)&4294967295), $95); //@line 842 "speak.cpp"
              var $cmp148=((($call147))|0)!=0; //@line 842 "speak.cpp"
              if (!($cmp148)) { __label__ = 73;break $if_then132$$if_else134$83; } //@line 842 "speak.cpp"
  
              var $96=HEAP32[((__impure_ptr)>>2)]; //@line 844 "speak.cpp"
              var $_stderr150=(($96+12)&4294967295); //@line 844 "speak.cpp"
              var $97=HEAP32[(($_stderr150)>>2)]; //@line 844 "speak.cpp"
              var $call151=_fprintf($97, ((__str28)&4294967295), allocate([((_wavefile)&4294967295),0,0,0], ["i8*",0,0,0], ALLOC_STACK)); //@line 844 "speak.cpp"
              _exit(3); //@line 845 "speak.cpp"
              ; //@line 846 "speak.cpp"
            }
          }
        } while(0);
  
        __Z8InitTexti(0); //@line 849 "speak.cpp"
        var $98=$f_text; //@line 850 "speak.cpp"
        var $99=$p_text; //@line 850 "speak.cpp"
        var $call155=__Z15SpeakNextClauseP7__sFILEPKvi($98, $99, 0); //@line 850 "speak.cpp"
        $ix=1; //@line 852 "speak.cpp"
        ; //@line 853 "speak.cpp"
        while(1) { 
  
          var $call156=__ZL11WavegenFilev(); //@line 855 "speak.cpp"
          var $cmp157=((($call156))|0)!=0; //@line 855 "speak.cpp"
          if ($cmp157) { __label__ = 75;; } else { __label__ = 76;; } //@line 855 "speak.cpp"
          if (__label__ == 75) {
  
            var $100=$ix; //@line 857 "speak.cpp"
            var $cmp159=((($100))|0)==0; //@line 857 "speak.cpp"
            if ($cmp159) { __label__ = 78;break ; } //@line 857 "speak.cpp"
          }
  
          var $call163=__Z8GenerateP12PHONEME_LISTPii(((_phoneme_list)&4294967295), _n_phoneme_list, 1); //@line 861 "speak.cpp"
          var $cmp164=((($call163))|0)==0; //@line 861 "speak.cpp"
          if (!($cmp164)) { __label__ = 74;continue ; } //@line 861 "speak.cpp"
  
          var $call166=__Z15SpeakNextClauseP7__sFILEPKvi(0, 0, 1); //@line 863 "speak.cpp"
          $ix=$call166; //@line 863 "speak.cpp"
          __label__ = 74;continue ; //@line 864 "speak.cpp"
        }
  
        __ZL13CloseWaveFilev(); //@line 867 "speak.cpp"
        __label__ = 83;break $if_then130$$if_else168$81; //@line 868 "speak.cpp"
      }
      else if (__label__ == 79) {
  
        __Z8InitTexti(0); //@line 877 "speak.cpp"
        var $101=$f_text; //@line 878 "speak.cpp"
        var $102=$p_text; //@line 878 "speak.cpp"
        var $call170=__Z15SpeakNextClauseP7__sFILEPKvi($101, $102, 0); //@line 878 "speak.cpp"
        var $103=HEAP32[((_option_quiet)>>2)]; //@line 880 "speak.cpp"
        var $tobool171=((($103))|0)!=0; //@line 880 "speak.cpp"
        if ($tobool171) { __label__ = 80;; } else { __label__ = 82;; } //@line 880 "speak.cpp"
        if (__label__ == 80) {
          while(1) { 
  
            var $call173=__Z15SpeakNextClauseP7__sFILEPKvi(0, 0, 1); //@line 882 "speak.cpp"
            var $cmp174=((($call173))|0)!=0; //@line 882 "speak.cpp"
            if ($cmp174) { __label__ = 80;continue ; } else { __label__ = 81;break ; } //@line 882 "speak.cpp"
          }
  
          $retval=0; //@line 883 "speak.cpp"
          __label__ = 87;break $if_then130$$if_else168$81; //@line 883 "speak.cpp"
        }
        else if (__label__ == 82) {
  
          var $104=HEAP32[((__impure_ptr)>>2)]; //@line 909 "speak.cpp"
          var $_stderr178=(($104+12)&4294967295); //@line 909 "speak.cpp"
          var $105=HEAP32[(($_stderr178)>>2)]; //@line 909 "speak.cpp"
          var $106=_fwrite(((__str29)&4294967295), 79, 1, $105); //@line 909 "speak.cpp"
          __label__ = 83;break $if_then130$$if_else168$81;
        }
      }
    } while(0);
    if (__label__ == 83) {
  
      var $107=HEAP32[((_f_trans)>>2)]; //@line 913 "speak.cpp"
      var $108=HEAP32[((__impure_ptr)>>2)]; //@line 913 "speak.cpp"
      var $_stdout181=(($108+8)&4294967295); //@line 913 "speak.cpp"
      var $109=HEAP32[(($_stdout181)>>2)]; //@line 913 "speak.cpp"
      var $cmp182=((($107))|0)!=((($109))|0); //@line 913 "speak.cpp"
      if ($cmp182) { __label__ = 84;; } else { __label__ = 86;; } //@line 913 "speak.cpp"
      $land_lhs_true183$$if_end188$108: do { 
        if (__label__ == 84) {
  
          var $110=HEAP32[((_f_trans)>>2)]; //@line 913 "speak.cpp"
          var $111=HEAP32[((__impure_ptr)>>2)]; //@line 913 "speak.cpp"
          var $_stderr184=(($111+12)&4294967295); //@line 913 "speak.cpp"
          var $112=HEAP32[(($_stderr184)>>2)]; //@line 913 "speak.cpp"
          var $cmp185=((($110))|0)!=((($112))|0); //@line 913 "speak.cpp"
          if (!($cmp185)) { __label__ = 86;break $land_lhs_true183$$if_end188$108; } //@line 913 "speak.cpp"
  
          var $113=HEAP32[((_f_trans)>>2)]; //@line 914 "speak.cpp"
          var $call187=_fclose($113); //@line 914 "speak.cpp"
          ; //@line 914 "speak.cpp"
        }
      } while(0);
  
      $retval=0; //@line 915 "speak.cpp"
      ; //@line 915 "speak.cpp"
    }
  
    var $114=$retval; //@line 916 "speak.cpp"
    ;
    return $114; //@line 916 "speak.cpp"
    return null;
  }
  Module["_main"] = _main;

  function __ZL9init_pathPcS_($argv0, $path_specified) {
    ;
    var __label__;
  
    var $argv0_addr;
    var $path_specified_addr;
    var $env;
    $argv0_addr=$argv0;
    $path_specified_addr=$path_specified;
    var $0=$path_specified_addr; //@line 341 "speak.cpp"
    var $tobool=((($0))|0)!=0; //@line 341 "speak.cpp"
    if ($tobool) { __label__ = 1;; } else { __label__ = 2;; } //@line 341 "speak.cpp"
    $if_then$$if_end$2: do { 
      if (__label__ == 1) {
  
        var $1=$path_specified_addr; //@line 343 "speak.cpp"
        var $call=_sprintf(((_path_home)&4294967295), ((__str38)&4294967295), allocate([$1,0,0,0], ["i8*",0,0,0], ALLOC_STACK)); //@line 343 "speak.cpp"
        ; //@line 344 "speak.cpp"
      }
      else if (__label__ == 2) {
  
        var $call1=_getenv(((__str39)&4294967295)); //@line 383 "speak.cpp"
        $env=$call1; //@line 383 "speak.cpp"
        var $cmp=((($call1))|0)!=0; //@line 383 "speak.cpp"
        if ($cmp) { __label__ = 3;; } else { __label__ = 4;; } //@line 383 "speak.cpp"
        if (__label__ == 3) {
  
          var $2=$env; //@line 385 "speak.cpp"
          var $call3=_snprintf(((_path_home)&4294967295), 150, ((__str38)&4294967295), allocate([$2,0,0,0], ["i8*",0,0,0], ALLOC_STACK)); //@line 385 "speak.cpp"
          var $call4=__Z13GetFileLengthPKc(((_path_home)&4294967295)); //@line 386 "speak.cpp"
          var $cmp5=((($call4))|0)==-2; //@line 386 "speak.cpp"
          if ($cmp5) { __label__ = 6;break $if_then$$if_end$2; } //@line 386 "speak.cpp"
        }
  
        var $call9=_getenv(((__str40)&4294967295)); //@line 390 "speak.cpp"
        var $call10=_snprintf(((_path_home)&4294967295), 150, ((__str38)&4294967295), allocate([$call9,0,0,0], ["i8*",0,0,0], ALLOC_STACK)); //@line 390 "speak.cpp"
        var $call11=_access(((_path_home)&4294967295), 4); //@line 391 "speak.cpp"
        var $cmp12=((($call11))|0)!=0; //@line 391 "speak.cpp"
        if (!($cmp12)) { __label__ = 6;break $if_then$$if_end$2; } //@line 391 "speak.cpp"
  
        var $src$, $dest$, $stop$, $stop4$, $fast$;
        $src$ = ((__str41)&4294967295);
        $dest$ = ((_path_home)&4294967295);
        $stop$ = $src$ + 23;
        if (($dest$%4) == ($src$%4) && 23 > 8) {
          while ($src$%4 !== 0 && $src$ < $stop$) {
            ; HEAP8[$dest$++] = HEAP8[$src$++];
          }
          $src$ >>= 2;
          $dest$ >>= 2;
          $stop4$ = $stop$ >> 2;
          while ($src$ < $stop4$) {
        ;;;    HEAP32[$dest$++] = HEAP32[$src$++];
          }
          $src$ <<= 2;
          $dest$ <<= 2;
        }while ($src$ < $stop$) {
          ; HEAP8[$dest$++] = HEAP8[$src$++];
        }; //@line 393 "speak.cpp"
        ; //@line 394 "speak.cpp"
      }
    } while(0);
  
    ;
    return; //@line 397 "speak.cpp"
    return;
  }
  

  function __ZL10initialisev() {
    ;
    var __label__;
  
    var $param;
    var $result;
    var $call=_setlocale(2, ((__str32)&4294967295)); //@line 411 "speak.cpp"
    var $cmp=((($call))|0)==0; //@line 411 "speak.cpp"
    if ($cmp) { __label__ = 1;; } else { __label__ = 3;; } //@line 411 "speak.cpp"
    $if_then$$if_end5$2: do { 
      if (__label__ == 1) {
  
        var $call1=_setlocale(2, ((__str33)&4294967295)); //@line 413 "speak.cpp"
        var $cmp2=((($call1))|0)==0; //@line 413 "speak.cpp"
        if (!($cmp2)) { __label__ = 3;break $if_then$$if_end5$2; } //@line 413 "speak.cpp"
  
        var $call4=_setlocale(2, ((__str13643)&4294967295)); //@line 414 "speak.cpp"
        ; //@line 414 "speak.cpp"
      }
    } while(0);
  
    __Z11WavegenInitii(22050, 0); //@line 419 "speak.cpp"
    var $call6=__Z10LoadPhDatav(); //@line 420 "speak.cpp"
    $result=$call6; //@line 420 "speak.cpp"
    var $cmp7=((($call6))|0)!=1; //@line 420 "speak.cpp"
    if ($cmp7) { __label__ = 4;; } else { __label__ = 7;; } //@line 420 "speak.cpp"
    if (__label__ == 4) {
  
      var $0=$result; //@line 422 "speak.cpp"
      var $cmp9=((($0))|0)==-1; //@line 422 "speak.cpp"
      var $1=HEAP32[((__impure_ptr)>>2)]; //@line 424 "speak.cpp"
      var $_stderr=(($1+12)&4294967295); //@line 424 "speak.cpp"
      var $2=HEAP32[(($_stderr)>>2)]; //@line 424 "speak.cpp"
      if ($cmp9) { __label__ = 5;; } else { __label__ = 6;; } //@line 422 "speak.cpp"
      if (__label__ == 5) {
  
        var $3=_fwrite(((__str35)&4294967295), 27, 1, $2); //@line 424 "speak.cpp"
        _exit(1); //@line 425 "speak.cpp"
        ; //@line 426 "speak.cpp"
      }
      else if (__label__ == 6) {
  
        var $4=$result; //@line 428 "speak.cpp"
        var $call13=_fprintf($2, ((__str36)&4294967295), allocate([$4,0,0,0,83200,0,0,0,((_path_home)&4294967295),0,0,0], ["i32",0,0,0,"i32",0,0,0,"i8*",0,0,0], ALLOC_STACK)); //@line 428 "speak.cpp"
        ;
      }
    }
  
    __Z10LoadConfigv(); //@line 430 "speak.cpp"
    __Z13SetVoiceStackP12espeak_VOICE(0); //@line 431 "speak.cpp"
    __Z14SynthesizeInitv(); //@line 432 "speak.cpp"
    $param=0; //@line 434 "speak.cpp"
    ; //@line 434 "speak.cpp"
    while(1) { 
  
      var $5=$param; //@line 435 "speak.cpp"
      var $arrayidx=((_param_defaults+$5*4)&4294967295); //@line 435 "speak.cpp"
      var $6=HEAP32[(($arrayidx)>>2)]; //@line 435 "speak.cpp"
      var $7=$param; //@line 435 "speak.cpp"
      var $arrayidx17=((((_param_stack+4)&4294967295)+$7*4)&4294967295); //@line 435 "speak.cpp"
      HEAP32[(($arrayidx17)>>2)]=$6; //@line 435 "speak.cpp"
      var $8=$param; //@line 434 "speak.cpp"
      var $inc=((($8)+1)&4294967295); //@line 434 "speak.cpp"
      $param=$inc; //@line 434 "speak.cpp"
      var $cmp16=((($inc))|0) < 15; //@line 434 "speak.cpp"
      if ($cmp16) { __label__ = 8;continue ; } else { __label__ = 9;break ; } //@line 434 "speak.cpp"
    }
  
    ;
    return;
    return;
  }
  

  function __ZL12OpenWaveFilePKci($path, $rate) {
    ;
    var __label__;
    var __lastLabel__ = null;
  
    var $retval;
    var $path_addr;
    var $rate_addr;
    $path_addr=$path;
    $rate_addr=$rate;
    var $0=$path_addr; //@line 235 "speak.cpp"
    var $cmp=((($0))|0)==0; //@line 235 "speak.cpp"
    if ($cmp) { __label__ = 1;; } else { __label__ = 2;; } //@line 235 "speak.cpp"
    if (__label__ == 1) {
  
      $retval=2; //@line 236 "speak.cpp"
      ; //@line 236 "speak.cpp"
    }
    else if (__label__ == 2) {
  
      var $1=$path_addr; //@line 238 "speak.cpp"
      var $call=_strcmp($1, ((__str12)&4294967295)); //@line 238 "speak.cpp"
      var $cmp1=((($call))|0)==0; //@line 238 "speak.cpp"
      if ($cmp1) { __label__ = 3;; } else { __label__ = 4;; } //@line 238 "speak.cpp"
      if (__label__ == 3) {
  
        var $2=HEAP32[((__impure_ptr)>>2)]; //@line 244 "speak.cpp"
        var $_stdout=(($2+8)&4294967295); //@line 244 "speak.cpp"
        var $3=HEAP32[(($_stdout)>>2)]; //@line 244 "speak.cpp"
        HEAP32[((_f_wave)>>2)]=$3; //@line 244 "speak.cpp"
        __lastLabel__ = 3; ; //@line 245 "speak.cpp"
      }
      else if (__label__ == 4) {
  
        var $4=$path_addr; //@line 247 "speak.cpp"
        var $call3=_fopen($4, ((__str79)&4294967295)); //@line 247 "speak.cpp"
        HEAP32[((_f_wave)>>2)]=$call3; //@line 247 "speak.cpp"
        __lastLabel__ = 4; ;
      }
  
      var $5=__lastLabel__ == 4 ? $call3 : ($3); //@line 249 "speak.cpp"
      var $cmp5=((($5))|0)!=0; //@line 249 "speak.cpp"
      if ($cmp5) { __label__ = 6;; } else { __label__ = 7;; } //@line 249 "speak.cpp"
      if (__label__ == 6) {
  
        var $6=HEAP32[((_f_wave)>>2)]; //@line 251 "speak.cpp"
        var $call7=_fwrite(((__ZZL12OpenWaveFilePKciE8wave_hdr)&4294967295), 1, 24, $6); //@line 251 "speak.cpp"
        var $7=HEAP32[((_f_wave)>>2)]; //@line 252 "speak.cpp"
        var $8=$rate_addr; //@line 252 "speak.cpp"
        __Z11Write4BytesP7__sFILEi($7, $8); //@line 252 "speak.cpp"
        var $9=HEAP32[((_f_wave)>>2)]; //@line 253 "speak.cpp"
        var $10=$rate_addr; //@line 253 "speak.cpp"
        var $mul=((($10)*2)&4294967295); //@line 253 "speak.cpp"
        __Z11Write4BytesP7__sFILEi($9, $mul); //@line 253 "speak.cpp"
        var $11=HEAP32[((_f_wave)>>2)]; //@line 254 "speak.cpp"
        var $call8=_fwrite(((__ZZL12OpenWaveFilePKciE8wave_hdr+32)&4294967295), 1, 12, $11); //@line 254 "speak.cpp"
        $retval=0; //@line 255 "speak.cpp"
        ; //@line 255 "speak.cpp"
      }
      else if (__label__ == 7) {
  
        $retval=1; //@line 257 "speak.cpp"
        ; //@line 257 "speak.cpp"
      }
    }
  
    var $12=$retval; //@line 258 "speak.cpp"
    ;
    return $12; //@line 258 "speak.cpp"
    return null;
  }
  

  function __ZL11WavegenFilev() {
    var __stackBase__  = STACKTOP; STACKTOP += 1236; _memset(__stackBase__, 0, 1236);
    var __label__;
  
    var $retval;
    var $finished;
    var $wav_outbuf=__stackBase__;
    var $fname=__stackBase__+1024;
    var $arraydecay=(($wav_outbuf)&4294967295); //@line 303 "speak.cpp"
    HEAP32[((_out_start)>>2)]=$arraydecay; //@line 303 "speak.cpp"
    HEAP32[((_out_ptr)>>2)]=$arraydecay; //@line 303 "speak.cpp"
    var $arraydecay1=(($wav_outbuf)&4294967295); //@line 304 "speak.cpp"
    var $add_ptr=(($arraydecay1+1024)&4294967295); //@line 304 "speak.cpp"
    HEAP32[((_out_end)>>2)]=$add_ptr; //@line 304 "speak.cpp"
    var $call=__Z11WavegenFilli(0); //@line 306 "speak.cpp"
    $finished=$call; //@line 306 "speak.cpp"
    var $0=HEAP32[((_quiet)>>2)]; //@line 308 "speak.cpp"
    var $tobool=((($0))|0)!=0; //@line 308 "speak.cpp"
    if ($tobool) { __label__ = 1;; } else { __label__ = 2;; } //@line 308 "speak.cpp"
    $if_then$$if_end$2: do { 
      if (__label__ == 1) {
  
        var $1=$finished; //@line 309 "speak.cpp"
        $retval=$1; //@line 309 "speak.cpp"
        ; //@line 309 "speak.cpp"
      }
      else if (__label__ == 2) {
  
        var $2=HEAP32[((_f_wave)>>2)]; //@line 311 "speak.cpp"
        var $cmp=((($2))|0)==0; //@line 311 "speak.cpp"
        if ($cmp) { __label__ = 3;; } else { __label__ = 5;; } //@line 311 "speak.cpp"
        $if_then2$$if_end10$5: do { 
          if (__label__ == 3) {
  
            var $arraydecay3=(($fname)&4294967295); //@line 313 "speak.cpp"
            var $3=HEAP32[((_wavefile_count)>>2)]; //@line 313 "speak.cpp"
            var $inc=((($3)+1)&4294967295); //@line 313 "speak.cpp"
            HEAP32[((_wavefile_count)>>2)]=$inc; //@line 313 "speak.cpp"
            var $call4=_sprintf($arraydecay3, ((__str30)&4294967295), allocate([((_wavefile)&4294967295),0,0,0,$inc,0,0,0,((_filetype)&4294967295),0,0,0], ["i8*",0,0,0,"i32",0,0,0,"i8*",0,0,0], ALLOC_STACK)); //@line 313 "speak.cpp"
            var $arraydecay5=(($fname)&4294967295); //@line 314 "speak.cpp"
            var $4=HEAP32[((_samplerate)>>2)]; //@line 314 "speak.cpp"
            var $call6=__ZL12OpenWaveFilePKci($arraydecay5, $4); //@line 314 "speak.cpp"
            var $cmp7=((($call6))|0)!=0; //@line 314 "speak.cpp"
            if (!($cmp7)) { __label__ = 5;break $if_then2$$if_end10$5; } //@line 314 "speak.cpp"
  
            $retval=1; //@line 315 "speak.cpp"
            __label__ = 12;break $if_then$$if_end$2; //@line 315 "speak.cpp"
          }
        } while(0);
  
        var $5=HEAP32[((_end_of_sentence)>>2)]; //@line 318 "speak.cpp"
        var $tobool11=((($5))|0)!=0; //@line 318 "speak.cpp"
        if ($tobool11) { __label__ = 6;; } else { __label__ = 9;; } //@line 318 "speak.cpp"
        $if_then12$$if_end17$9: do { 
          if (__label__ == 6) {
  
            HEAP32[((_end_of_sentence)>>2)]=0; //@line 320 "speak.cpp"
            var $6=HEAPU32[((_samples_split)>>2)]; //@line 321 "speak.cpp"
            var $cmp13=((($6))>>>0) > 0; //@line 321 "speak.cpp"
            if (!($cmp13)) { __label__ = 9;break $if_then12$$if_end17$9; } //@line 321 "speak.cpp"
  
            var $7=HEAPU32[((_samples_total)>>2)]; //@line 321 "speak.cpp"
            var $8=HEAPU32[((_samples_split)>>2)]; //@line 321 "speak.cpp"
            var $cmp14=((($7))>>>0) > ((($8))>>>0); //@line 321 "speak.cpp"
            if (!($cmp14)) { __label__ = 9;break $if_then12$$if_end17$9; } //@line 321 "speak.cpp"
  
            __ZL13CloseWaveFilev(); //@line 323 "speak.cpp"
            HEAP32[((_samples_total)>>2)]=0; //@line 324 "speak.cpp"
            ; //@line 325 "speak.cpp"
          }
        } while(0);
  
        var $9=HEAP32[((_f_wave)>>2)]; //@line 328 "speak.cpp"
        var $cmp18=((($9))|0)!=0; //@line 328 "speak.cpp"
        if ($cmp18) { __label__ = 10;; } else { __label__ = 11;; } //@line 328 "speak.cpp"
        if (__label__ == 10) {
  
          var $10=HEAP32[((_out_ptr)>>2)]; //@line 330 "speak.cpp"
          var $arraydecay20=(($wav_outbuf)&4294967295); //@line 330 "speak.cpp"
          var $sub_ptr_lhs_cast=($10); //@line 330 "speak.cpp"
          var $sub_ptr_rhs_cast=($arraydecay20); //@line 330 "speak.cpp"
          var $sub_ptr_sub=((($sub_ptr_lhs_cast)-($sub_ptr_rhs_cast))&4294967295); //@line 330 "speak.cpp"
          var $div=((((($sub_ptr_sub))|0)/2)|0); //@line 330 "speak.cpp"
          var $11=HEAP32[((_samples_total)>>2)]; //@line 330 "speak.cpp"
          var $add=((($11)+($div))&4294967295); //@line 330 "speak.cpp"
          HEAP32[((_samples_total)>>2)]=$add; //@line 330 "speak.cpp"
          var $arraydecay21=(($wav_outbuf)&4294967295); //@line 331 "speak.cpp"
          var $12=HEAP32[((_out_ptr)>>2)]; //@line 331 "speak.cpp"
          var $arraydecay22=(($wav_outbuf)&4294967295); //@line 331 "speak.cpp"
          var $sub_ptr_lhs_cast23=($12); //@line 331 "speak.cpp"
          var $sub_ptr_rhs_cast24=($arraydecay22); //@line 331 "speak.cpp"
          var $sub_ptr_sub25=((($sub_ptr_lhs_cast23)-($sub_ptr_rhs_cast24))&4294967295); //@line 331 "speak.cpp"
          var $13=HEAP32[((_f_wave)>>2)]; //@line 331 "speak.cpp"
          var $call26=_fwrite($arraydecay21, 1, $sub_ptr_sub25, $13); //@line 331 "speak.cpp"
          ; //@line 332 "speak.cpp"
        }
  
        var $14=$finished; //@line 333 "speak.cpp"
        $retval=$14; //@line 333 "speak.cpp"
        ; //@line 333 "speak.cpp"
      }
    } while(0);
  
    var $15=$retval; //@line 334 "speak.cpp"
    STACKTOP = __stackBase__;
    return $15; //@line 334 "speak.cpp"
    return null;
  }
  

  function __ZL13CloseWaveFilev() {
    ;
    var __label__;
  
    var $pos;
    var $0=HEAP32[((_f_wave)>>2)]; //@line 268 "speak.cpp"
    var $cmp=((($0))|0)==0; //@line 268 "speak.cpp"
    if ($cmp) { __label__ = 3;; } else { __label__ = 1;; } //@line 268 "speak.cpp"
    $return$$lor_lhs_false$2: do { 
      if (__label__ == 1) {
  
        var $1=HEAP32[((_f_wave)>>2)]; //@line 268 "speak.cpp"
        var $2=HEAP32[((__impure_ptr)>>2)]; //@line 268 "speak.cpp"
        var $_stdout=(($2+8)&4294967295); //@line 268 "speak.cpp"
        var $3=HEAP32[(($_stdout)>>2)]; //@line 268 "speak.cpp"
        var $cmp1=((($1))|0)==((($3))|0); //@line 268 "speak.cpp"
        if ($cmp1) { __label__ = 3;break $return$$lor_lhs_false$2; } //@line 268 "speak.cpp"
  
        var $4=HEAP32[((_f_wave)>>2)]; //@line 271 "speak.cpp"
        var $call=_fflush($4); //@line 271 "speak.cpp"
        var $5=HEAP32[((_f_wave)>>2)]; //@line 272 "speak.cpp"
        var $call2=_ftell($5); //@line 272 "speak.cpp"
        $pos=$call2; //@line 272 "speak.cpp"
        var $6=HEAP32[((_f_wave)>>2)]; //@line 274 "speak.cpp"
        var $call3=_fseek($6, 4, 0); //@line 274 "speak.cpp"
        var $7=HEAP32[((_f_wave)>>2)]; //@line 275 "speak.cpp"
        var $8=$pos; //@line 275 "speak.cpp"
        var $sub=((($8)-8)&4294967295); //@line 275 "speak.cpp"
        __Z11Write4BytesP7__sFILEi($7, $sub); //@line 275 "speak.cpp"
        var $9=HEAP32[((_f_wave)>>2)]; //@line 277 "speak.cpp"
        var $call4=_fseek($9, 40, 0); //@line 277 "speak.cpp"
        var $10=HEAP32[((_f_wave)>>2)]; //@line 278 "speak.cpp"
        var $11=$pos; //@line 278 "speak.cpp"
        var $sub5=((($11)-44)&4294967295); //@line 278 "speak.cpp"
        __Z11Write4BytesP7__sFILEi($10, $sub5); //@line 278 "speak.cpp"
        var $12=HEAP32[((_f_wave)>>2)]; //@line 281 "speak.cpp"
        var $call6=_fclose($12); //@line 281 "speak.cpp"
        HEAP32[((_f_wave)>>2)]=0; //@line 282 "speak.cpp"
        ; //@line 284 "speak.cpp"
      }
    } while(0);
  
    ;
    return; //@line 284 "speak.cpp"
    return;
  }
  

  function __Z8isspace2j($c) {
    ;
    var __label__;
  
    var $retval;
    var $c_addr;
    var $c2;
    $c_addr=$c;
    var $0=$c_addr; //@line 150 "compiledict.cpp"
    var $and=($0) & 255; //@line 150 "compiledict.cpp"
    $c2=$and; //@line 150 "compiledict.cpp"
    var $cmp=((($and))|0)==0; //@line 150 "compiledict.cpp"
    var $1=$c_addr; //@line 150 "compiledict.cpp"
    var $cmp1=((($1))>>>0) > 32; //@line 150 "compiledict.cpp"
    var $or_cond=($cmp) | ($cmp1); //@line 150 "compiledict.cpp"
    if ($or_cond) { __label__ = 1;; } else { __label__ = 2;; } //@line 150 "compiledict.cpp"
    if (__label__ == 1) {
  
      $retval=0; //@line 151 "compiledict.cpp"
      ; //@line 151 "compiledict.cpp"
    }
    else if (__label__ == 2) {
  
      $retval=1; //@line 152 "compiledict.cpp"
      ; //@line 152 "compiledict.cpp"
    }
  
    var $2=$retval; //@line 153 "compiledict.cpp"
    ;
    return $2; //@line 153 "compiledict.cpp"
    return null;
  }
  

  function __Z14LookupMnemNameP8MNEM_TABi($table, $value) {
    ;
    var __label__;
  
    var $retval;
    var $table_addr;
    var $value_addr;
    $table_addr=$table;
    $value_addr=$value;
    ; //@line 175 "compiledict.cpp"
    while(1) { 
  
      var $0=$table_addr; //@line 175 "compiledict.cpp"
      var $mnem=(($0)&4294967295); //@line 175 "compiledict.cpp"
      var $1=HEAP32[(($mnem)>>2)]; //@line 175 "compiledict.cpp"
      var $cmp=((($1))|0)!=0; //@line 175 "compiledict.cpp"
      if (!($cmp)) { __label__ = 5;break ; } //@line 175 "compiledict.cpp"
  
      var $2=$table_addr; //@line 177 "compiledict.cpp"
      var $value1=(($2+4)&4294967295); //@line 177 "compiledict.cpp"
      var $3=HEAP32[(($value1)>>2)]; //@line 177 "compiledict.cpp"
      var $4=$value_addr; //@line 177 "compiledict.cpp"
      var $cmp2=((($3))|0)==((($4))|0); //@line 177 "compiledict.cpp"
      var $5=$table_addr; //@line 178 "compiledict.cpp"
      if ($cmp2) { __label__ = 3;break ; } //@line 177 "compiledict.cpp"
  
      var $incdec_ptr=(($5+8)&4294967295); //@line 179 "compiledict.cpp"
      $table_addr=$incdec_ptr; //@line 179 "compiledict.cpp"
      __label__ = 1;continue ; //@line 180 "compiledict.cpp"
    }
    if (__label__ == 5) {
  
      $retval=((__str13643)&4294967295); //@line 181 "compiledict.cpp"
      ; //@line 181 "compiledict.cpp"
    }
    else if (__label__ == 3) {
  
      var $mnem3=(($5)&4294967295); //@line 178 "compiledict.cpp"
      var $6=HEAP32[(($mnem3)>>2)]; //@line 178 "compiledict.cpp"
      $retval=$6; //@line 178 "compiledict.cpp"
      ; //@line 178 "compiledict.cpp"
    }
  
    var $7=$retval; //@line 182 "compiledict.cpp"
    ;
    return $7; //@line 182 "compiledict.cpp"
    return null;
  }
  

  function __Z22print_dictionary_flagsPj($flags) {
    ;
    var __label__;
  
    var $flags_addr;
    $flags_addr=$flags;
    var $0=$flags_addr; //@line 189 "compiledict.cpp"
    var $arrayidx=(($0)&4294967295); //@line 189 "compiledict.cpp"
    var $1=HEAP32[(($arrayidx)>>2)]; //@line 189 "compiledict.cpp"
    var $and=($1) & 15; //@line 189 "compiledict.cpp"
    var $add=((($and)+64)&4294967295); //@line 189 "compiledict.cpp"
    var $call=__Z14LookupMnemNameP8MNEM_TABi(((_mnem_flags)&4294967295), $add); //@line 189 "compiledict.cpp"
    var $2=$flags_addr; //@line 189 "compiledict.cpp"
    var $arrayidx1=(($2)&4294967295); //@line 189 "compiledict.cpp"
    var $3=HEAP32[(($arrayidx1)>>2)]; //@line 189 "compiledict.cpp"
    var $4=$flags_addr; //@line 189 "compiledict.cpp"
    var $arrayidx2=(($4+4)&4294967295); //@line 189 "compiledict.cpp"
    var $5=HEAP32[(($arrayidx2)>>2)]; //@line 189 "compiledict.cpp"
    var $call3=_sprintf(((__ZZ22print_dictionary_flagsPjE3buf)&4294967295), ((__str59)&4294967295), allocate([$call,0,0,0,$3,0,0,0,$5,0,0,0], ["i8*",0,0,0,"i32",0,0,0,"i32",0,0,0], ALLOC_STACK)); //@line 189 "compiledict.cpp"
    ;
    return ((__ZZ22print_dictionary_flagsPjE3buf)&4294967295); //@line 190 "compiledict.cpp"
    return null;
  }
  

  function __Z10DecodeRulePKciPci($group_chars, $group_length, $rule, $control) {
    var __stackBase__  = STACKTOP; STACKTOP += 140; _memset(__stackBase__, 0, 140);
    var __label__;
    var __lastLabel__ = null;
  
    var $group_chars_addr;
    var $group_length_addr;
    var $rule_addr;
    var $control_addr;
    var $rb;
    var $c;
    var $p;
    var $ix;
    var $match_type;
    var $finished;
    var $value;
    var $linenum;
    var $flags;
    var $suffix_char;
    var $condition_num;
    var $at_start;
    var $name;
    var $buf=__stackBase__;
    var $buf_pre=__stackBase__+60;
    var $suffix=__stackBase__+120;
    $group_chars_addr=$group_chars;
    $group_length_addr=$group_length;
    $rule_addr=$rule;
    $control_addr=$control;
    $finished=0; //@line 205 "compiledict.cpp"
    $linenum=0; //@line 207 "compiledict.cpp"
    $condition_num=0; //@line 210 "compiledict.cpp"
    $at_start=0; //@line 211 "compiledict.cpp"
    $match_type=0; //@line 223 "compiledict.cpp"
    var $arrayidx=(($buf_pre)&4294967295); //@line 224 "compiledict.cpp"
    HEAP8[($arrayidx)]=0; //@line 224 "compiledict.cpp"
    $ix=0; //@line 226 "compiledict.cpp"
    var $0=$ix; //@line 226 "compiledict.cpp"
    var $1=$group_length_addr; //@line 226 "compiledict.cpp"
    var $cmp9=((($0))|0) < ((($1))|0); //@line 226 "compiledict.cpp"
    var $2=$ix; //@line 228 "compiledict.cpp"
    if ($cmp9) { __lastLabel__ = 0; __label__ = 1;; } else { __lastLabel__ = 0; __label__ = 2;; } //@line 226 "compiledict.cpp"
    $for_body$$for_end$2: do { 
      if (__label__ == 1) {
        while(1) { 
  
          var $3=__lastLabel__ == 1 ? $10 : ($2);
          var $4=$group_chars_addr; //@line 228 "compiledict.cpp"
          var $arrayidx1=(($4+$3)&4294967295); //@line 228 "compiledict.cpp"
          var $5=HEAP8[($arrayidx1)]; //@line 228 "compiledict.cpp"
          var $6=$ix; //@line 228 "compiledict.cpp"
          var $arrayidx2=(($buf+$6)&4294967295); //@line 228 "compiledict.cpp"
          HEAP8[($arrayidx2)]=$5; //@line 228 "compiledict.cpp"
          var $7=$ix; //@line 226 "compiledict.cpp"
          var $inc=((($7)+1)&4294967295); //@line 226 "compiledict.cpp"
          $ix=$inc; //@line 226 "compiledict.cpp"
          var $8=$ix; //@line 226 "compiledict.cpp"
          var $9=$group_length_addr; //@line 226 "compiledict.cpp"
          var $cmp=((($8))|0) < ((($9))|0); //@line 226 "compiledict.cpp"
          var $10=$ix; //@line 228 "compiledict.cpp"
          if ($cmp) { __lastLabel__ = 1; __label__ = 1;continue ; } else { __lastLabel__ = 1; __label__ = 2;break $for_body$$for_end$2; } //@line 226 "compiledict.cpp"
        }
      }
    } while(0);
  
    var $_lcssa8=__lastLabel__ == 0 ? $2 : ($10);
    var $arrayidx3=(($buf+$_lcssa8)&4294967295); //@line 230 "compiledict.cpp"
    HEAP8[($arrayidx3)]=0; //@line 230 "compiledict.cpp"
    var $arraydecay=(($buf)&4294967295); //@line 232 "compiledict.cpp"
    var $call=_strlen($arraydecay); //@line 232 "compiledict.cpp"
    var $arrayidx4=(($buf+$call)&4294967295); //@line 232 "compiledict.cpp"
    $p=$arrayidx4; //@line 232 "compiledict.cpp"
    var $11=$finished; //@line 233 "compiledict.cpp"
    var $tobool6=((($11))|0)!=0; //@line 233 "compiledict.cpp"
    var $lnot7=($tobool6) ^ 1; //@line 233 "compiledict.cpp"
    if ($lnot7) { __label__ = 3;; } else { __label__ = 38;; } //@line 233 "compiledict.cpp"
    $while_body_lr_ph$$while_end$6: do { 
      if (__label__ == 3) {
  
        var $arraydecay9=(($buf_pre)&4294967295); //@line 250 "compiledict.cpp"
        var $arraydecay11=(($buf)&4294967295); //@line 255 "compiledict.cpp"
        var $arraydecay13=(($buf)&4294967295); //@line 256 "compiledict.cpp"
        var $arraydecay58=(($suffix)&4294967295); //@line 293 "compiledict.cpp"
        var $arraydecay70=(($suffix)&4294967295); //@line 298 "compiledict.cpp"
        var $arraydecay80=(($suffix)&4294967295); //@line 301 "compiledict.cpp"
        var $arraydecay82=(($suffix)&4294967295); //@line 302 "compiledict.cpp"
        ; //@line 233 "compiledict.cpp"
        $while_body$8: while(1) { 
  
          var $12=$rule_addr; //@line 235 "compiledict.cpp"
          var $incdec_ptr=(($12+1)&4294967295); //@line 235 "compiledict.cpp"
          $rule_addr=$incdec_ptr; //@line 235 "compiledict.cpp"
          var $13=HEAP8[($12)]; //@line 235 "compiledict.cpp"
          $rb=$13; //@line 235 "compiledict.cpp"
          var $14=$rb; //@line 237 "compiledict.cpp"
          var $conv=((($14))&255); //@line 237 "compiledict.cpp"
          var $cmp5=((($conv))|0) <= 9; //@line 237 "compiledict.cpp"
          var $15=$rb; //@line 239 "compiledict.cpp"
          var $conv6=((($15))&255); //@line 239 "compiledict.cpp"
          if ($cmp5) { __label__ = 5;; } else { __label__ = 12;; } //@line 237 "compiledict.cpp"
          $if_then$$if_end$10: do { 
            if (__label__ == 5) {
  
              if ($conv6 == 0) {
                __label__ = 6;;
              }
              else if ($conv6 == 3) {
                __label__ = 6;;
              }
              else if ($conv6 == 8) {
                __label__ = 7;;
              }
              else if ($conv6 == 1) {
                __label__ = 8;;
              }
              else if ($conv6 == 2) {
                __label__ = 9;;
              }
              else if ($conv6 == 9) {
                __label__ = 11;;
              }
              else if ($conv6 == 5) {
                __label__ = 10;;
              }
              else {
              __label__ = 37;break $if_then$$if_end$10;
              }
              
              if (__label__ == 6) {
  
                $finished=1; //@line 243 "compiledict.cpp"
                __label__ = 37;break $if_then$$if_end$10; //@line 244 "compiledict.cpp"
              }
              else if (__label__ == 7) {
  
                $at_start=1; //@line 246 "compiledict.cpp"
                ; //@line 246 "compiledict.cpp"
              }
              else if (__label__ == 9) {
  
                $match_type=2; //@line 253 "compiledict.cpp"
                var $17=$p; //@line 254 "compiledict.cpp"
                HEAP8[($17)]=0; //@line 254 "compiledict.cpp"
                var $strlen=_strlen($arraydecay11); //@line 255 "compiledict.cpp"
                var $endptr=(($arraydecay11+$strlen)&4294967295); //@line 255 "compiledict.cpp"
                var $src$, $dest$, $stop$, $stop4$, $fast$;
                $src$ = ((__str60)&4294967295);
                $dest$ = $endptr;
                $stop$ = $src$ + 3;
                if (($dest$%4) == ($src$%4) && 3 > 8) {
                  while ($src$%4 !== 0 && $src$ < $stop$) {
                    ; HEAP8[$dest$++] = HEAP8[$src$++];
                  }
                  $src$ >>= 2;
                  $dest$ >>= 2;
                  $stop4$ = $stop$ >> 2;
                  while ($src$ < $stop4$) {
                ;;;    HEAP32[$dest$++] = HEAP32[$src$++];
                  }
                  $src$ <<= 2;
                  $dest$ <<= 2;
                }while ($src$ < $stop$) {
                  ; HEAP8[$dest$++] = HEAP8[$src$++];
                }; //@line 255 "compiledict.cpp"
                var $call14=_strlen($arraydecay13); //@line 256 "compiledict.cpp"
                var $arrayidx15=(($buf+$call14)&4294967295); //@line 256 "compiledict.cpp"
                $p=$arrayidx15; //@line 256 "compiledict.cpp"
                __label__ = 37;break $if_then$$if_end$10; //@line 257 "compiledict.cpp"
              }
              else if (__label__ == 11) {
  
                var $20=$rule_addr; //@line 265 "compiledict.cpp"
                var $arrayidx20=(($20+1)&4294967295); //@line 265 "compiledict.cpp"
                var $21=HEAP8[($arrayidx20)]; //@line 265 "compiledict.cpp"
                var $conv21=(tempInt=(($21)),(tempInt>=128?tempInt-256:tempInt)); //@line 265 "compiledict.cpp"
                var $and=($conv21) & 255; //@line 265 "compiledict.cpp"
                var $sub=((($and)-1)&4294967295); //@line 265 "compiledict.cpp"
                $value=$sub; //@line 265 "compiledict.cpp"
                var $22=$rule_addr; //@line 266 "compiledict.cpp"
                var $arrayidx22=(($22)&4294967295); //@line 266 "compiledict.cpp"
                var $23=HEAP8[($arrayidx22)]; //@line 266 "compiledict.cpp"
                var $conv23=(tempInt=(($23)),(tempInt>=128?tempInt-256:tempInt)); //@line 266 "compiledict.cpp"
                var $and24=($conv23) & 255; //@line 266 "compiledict.cpp"
                var $24=$value; //@line 266 "compiledict.cpp"
                var $mul=((($24)*255)&4294967295); //@line 266 "compiledict.cpp"
                var $sub25=((($mul)-1)&4294967295); //@line 266 "compiledict.cpp"
                var $add=((($sub25)+($and24))&4294967295); //@line 266 "compiledict.cpp"
                $linenum=$add; //@line 266 "compiledict.cpp"
                var $25=$rule_addr; //@line 267 "compiledict.cpp"
                var $add_ptr=(($25+2)&4294967295); //@line 267 "compiledict.cpp"
                $rule_addr=$add_ptr; //@line 267 "compiledict.cpp"
                __label__ = 37;break $if_then$$if_end$10; //@line 268 "compiledict.cpp"
              }
              else if (__label__ == 10) {
  
                var $18=$rule_addr; //@line 262 "compiledict.cpp"
                var $incdec_ptr17=(($18+1)&4294967295); //@line 262 "compiledict.cpp"
                $rule_addr=$incdec_ptr17; //@line 262 "compiledict.cpp"
                var $19=HEAP8[($18)]; //@line 262 "compiledict.cpp"
                var $conv18=(tempInt=(($19)),(tempInt>=128?tempInt-256:tempInt)); //@line 262 "compiledict.cpp"
                $condition_num=$conv18; //@line 262 "compiledict.cpp"
                __label__ = 37;break $if_then$$if_end$10; //@line 263 "compiledict.cpp"
              }
  
              $match_type=1; //@line 248 "compiledict.cpp"
              var $16=$p; //@line 249 "compiledict.cpp"
              HEAP8[($16)]=0; //@line 249 "compiledict.cpp"
              $p=$arraydecay9; //@line 250 "compiledict.cpp"
              ; //@line 251 "compiledict.cpp"
            }
            else if (__label__ == 12) {
  
              var $cmp27=((($conv6))|0)==28; //@line 273 "compiledict.cpp"
              if ($cmp27) { __label__ = 13;; } else { __label__ = 17;; } //@line 273 "compiledict.cpp"
              if (__label__ == 13) {
  
                var $26=$rule_addr; //@line 275 "compiledict.cpp"
                var $incdec_ptr29=(($26+1)&4294967295); //@line 275 "compiledict.cpp"
                $rule_addr=$incdec_ptr29; //@line 275 "compiledict.cpp"
                var $27=HEAP8[($26)]; //@line 275 "compiledict.cpp"
                var $conv30=(tempInt=(($27)),(tempInt>=128?tempInt-256:tempInt)); //@line 275 "compiledict.cpp"
                var $and31=($conv30) & 255; //@line 275 "compiledict.cpp"
                $value=$and31; //@line 275 "compiledict.cpp"
                var $28=$value; //@line 276 "compiledict.cpp"
                var $cmp32=((($28))|0)!=1; //@line 276 "compiledict.cpp"
                if ($cmp32) { __label__ = 15;; } else { __label__ = 14;; } //@line 276 "compiledict.cpp"
                $if_then35$$lor_lhs_false$22: do { 
                  if (__label__ == 14) {
  
                    var $29=$control_addr; //@line 276 "compiledict.cpp"
                    var $and33=($29) & -2147483648; //@line 276 "compiledict.cpp"
                    var $tobool34=((($and33))|0)!=0; //@line 276 "compiledict.cpp"
                    if ($tobool34) { __label__ = 15;break $if_then35$$lor_lhs_false$22; } else { __label__ = 16;break $if_then35$$lor_lhs_false$22; } //@line 276 "compiledict.cpp"
                  }
                } while(0);
                if (__label__ == 15) {
  
                  var $30=$p; //@line 278 "compiledict.cpp"
                  var $arrayidx36=(($30)&4294967295); //@line 278 "compiledict.cpp"
                  HEAP8[($arrayidx36)]=36; //@line 278 "compiledict.cpp"
                  var $31=$value; //@line 279 "compiledict.cpp"
                  var $call37=__Z14LookupMnemNameP8MNEM_TABi(((_mnem_rules)&4294967295), $31); //@line 279 "compiledict.cpp"
                  $name=$call37; //@line 279 "compiledict.cpp"
                  var $32=$p; //@line 280 "compiledict.cpp"
                  var $arrayidx38=(($32+1)&4294967295); //@line 280 "compiledict.cpp"
                  var $33=$name; //@line 280 "compiledict.cpp"
                  var $call39=_strcpy($arrayidx38, $33); //@line 280 "compiledict.cpp"
                  var $34=$name; //@line 281 "compiledict.cpp"
                  var $call40=_strlen($34); //@line 281 "compiledict.cpp"
                  var $add41=((($call40)+1)&4294967295); //@line 281 "compiledict.cpp"
                  var $35=$p; //@line 281 "compiledict.cpp"
                  var $add_ptr42=(($35+$add41)&4294967295); //@line 281 "compiledict.cpp"
                  $p=$add_ptr42; //@line 281 "compiledict.cpp"
                  ; //@line 282 "compiledict.cpp"
                }
  
                $c=32; //@line 283 "compiledict.cpp"
                ; //@line 284 "compiledict.cpp"
              }
              else if (__label__ == 17) {
  
                var $36=$rb; //@line 286 "compiledict.cpp"
                var $conv44=((($36))&255); //@line 286 "compiledict.cpp"
                var $cmp45=((($conv44))|0)==14; //@line 286 "compiledict.cpp"
                if ($cmp45) { __label__ = 18;; } else { __label__ = 25;; } //@line 286 "compiledict.cpp"
                if (__label__ == 18) {
  
                  var $37=$rule_addr; //@line 289 "compiledict.cpp"
                  var $arrayidx47=(($37)&4294967295); //@line 289 "compiledict.cpp"
                  var $38=HEAP8[($arrayidx47)]; //@line 289 "compiledict.cpp"
                  var $conv48=(tempInt=(($38)),(tempInt>=128?tempInt-256:tempInt)); //@line 289 "compiledict.cpp"
                  var $and49=($conv48) & 127; //@line 289 "compiledict.cpp"
                  var $shl=((($and49)*256)&4294967295); //@line 289 "compiledict.cpp"
                  var $39=$rule_addr; //@line 289 "compiledict.cpp"
                  var $arrayidx50=(($39+1)&4294967295); //@line 289 "compiledict.cpp"
                  var $40=HEAP8[($arrayidx50)]; //@line 289 "compiledict.cpp"
                  var $conv51=(tempInt=(($40)),(tempInt>=128?tempInt-256:tempInt)); //@line 289 "compiledict.cpp"
                  var $and52=($conv51) & 127; //@line 289 "compiledict.cpp"
                  var $add53=((($and52)+($shl))&4294967295); //@line 289 "compiledict.cpp"
                  $flags=$add53; //@line 289 "compiledict.cpp"
                  $suffix_char=83; //@line 290 "compiledict.cpp"
                  var $41=$flags; //@line 291 "compiledict.cpp"
                  var $and54=($41) & 4; //@line 291 "compiledict.cpp"
                  var $tobool55=((($and54))|0)!=0; //@line 291 "compiledict.cpp"
                  if ($tobool55) { __label__ = 19;; } else { __label__ = 20;; } //@line 291 "compiledict.cpp"
                  if (__label__ == 19) {
  
                    $suffix_char=80; //@line 292 "compiledict.cpp"
                    ; //@line 292 "compiledict.cpp"
                  }
  
                  var $42=$suffix_char; //@line 293 "compiledict.cpp"
                  var $43=$rule_addr; //@line 293 "compiledict.cpp"
                  var $arrayidx59=(($43+2)&4294967295); //@line 293 "compiledict.cpp"
                  var $44=HEAP8[($arrayidx59)]; //@line 293 "compiledict.cpp"
                  var $conv60=(tempInt=(($44)),(tempInt>=128?tempInt-256:tempInt)); //@line 293 "compiledict.cpp"
                  var $and61=($conv60) & 127; //@line 293 "compiledict.cpp"
                  var $call62=_sprintf($arraydecay58, ((__str62)&4294967295), allocate([$42,0,0,0,$and61,0,0,0], ["i32",0,0,0,"i32",0,0,0], ALLOC_STACK)); //@line 293 "compiledict.cpp"
                  var $45=$rule_addr; //@line 294 "compiledict.cpp"
                  var $add_ptr63=(($45+3)&4294967295); //@line 294 "compiledict.cpp"
                  $rule_addr=$add_ptr63; //@line 294 "compiledict.cpp"
                  $ix=0; //@line 295 "compiledict.cpp"
                  ; //@line 295 "compiledict.cpp"
                  while(1) { 
  
                    var $46=$flags; //@line 297 "compiledict.cpp"
                    var $and67=($46) & 1; //@line 297 "compiledict.cpp"
                    var $tobool68=((($and67))|0)!=0; //@line 297 "compiledict.cpp"
                    if ($tobool68) { __label__ = 22;; } else { __label__ = 23;; } //@line 297 "compiledict.cpp"
                    if (__label__ == 22) {
  
                      var $call71=_strlen($arraydecay70); //@line 298 "compiledict.cpp"
                      var $arrayidx72=(($suffix+$call71)&4294967295); //@line 298 "compiledict.cpp"
                      var $47=$ix; //@line 298 "compiledict.cpp"
                      var $arrayidx73=((((__str61)&4294967295)+$47)&4294967295); //@line 298 "compiledict.cpp"
                      var $48=HEAP8[($arrayidx73)]; //@line 298 "compiledict.cpp"
                      var $conv74=(tempInt=(($48)),(tempInt>=128?tempInt-256:tempInt)); //@line 298 "compiledict.cpp"
                      var $char=((($conv74)) & 255); //@line 298 "compiledict.cpp"
                      HEAP8[($arrayidx72)]=$char; //@line 298 "compiledict.cpp"
                      var $nul=(($arrayidx72+1)&4294967295); //@line 298 "compiledict.cpp"
                      HEAP8[($nul)]=0; //@line 298 "compiledict.cpp"
                      ; //@line 298 "compiledict.cpp"
                    }
  
                    var $49=$flags; //@line 299 "compiledict.cpp"
                    var $shr=($49) >> 1; //@line 299 "compiledict.cpp"
                    $flags=$shr; //@line 299 "compiledict.cpp"
                    var $50=$ix; //@line 295 "compiledict.cpp"
                    var $inc78=((($50)+1)&4294967295); //@line 295 "compiledict.cpp"
                    $ix=$inc78; //@line 295 "compiledict.cpp"
                    var $cmp65=((($inc78))|0) < 9; //@line 295 "compiledict.cpp"
                    if ($cmp65) { __label__ = 21;continue ; } else { __label__ = 24;break ; } //@line 295 "compiledict.cpp"
                  }
  
                  var $51=$p; //@line 301 "compiledict.cpp"
                  var $call81=_strcpy($51, $arraydecay80); //@line 301 "compiledict.cpp"
                  var $call83=_strlen($arraydecay82); //@line 302 "compiledict.cpp"
                  var $52=$p; //@line 302 "compiledict.cpp"
                  var $add_ptr84=(($52+$call83)&4294967295); //@line 302 "compiledict.cpp"
                  $p=$add_ptr84; //@line 302 "compiledict.cpp"
                  $c=32; //@line 303 "compiledict.cpp"
                  ; //@line 304 "compiledict.cpp"
                }
                else if (__label__ == 25) {
  
                  var $53=$rb; //@line 306 "compiledict.cpp"
                  var $conv86=((($53))&255); //@line 306 "compiledict.cpp"
                  var $cmp87=((($conv86))|0)==17; //@line 306 "compiledict.cpp"
                  if ($cmp87) { __label__ = 26;; } else { __label__ = 27;; } //@line 306 "compiledict.cpp"
                  if (__label__ == 26) {
  
                    var $54=$rule_addr; //@line 308 "compiledict.cpp"
                    var $incdec_ptr89=(($54+1)&4294967295); //@line 308 "compiledict.cpp"
                    $rule_addr=$incdec_ptr89; //@line 308 "compiledict.cpp"
                    var $55=HEAP8[($54)]; //@line 308 "compiledict.cpp"
                    var $conv90=(tempInt=(($55)),(tempInt>=128?tempInt-256:tempInt)); //@line 308 "compiledict.cpp"
                    var $sub91=((($conv90)-65)&4294967295); //@line 308 "compiledict.cpp"
                    var $arrayidx92=((__ZZ10DecodeRulePKciPciE10symbols_lg+$sub91)&4294967295); //@line 308 "compiledict.cpp"
                    var $56=HEAP8[($arrayidx92)]; //@line 308 "compiledict.cpp"
                    $c=$56; //@line 308 "compiledict.cpp"
                    ; //@line 309 "compiledict.cpp"
                  }
                  else if (__label__ == 27) {
  
                    var $57=$rb; //@line 311 "compiledict.cpp"
                    var $conv94=((($57))&255); //@line 311 "compiledict.cpp"
                    var $cmp95=((($conv94))|0)==18; //@line 311 "compiledict.cpp"
                    if ($cmp95) { __label__ = 28;; } else { __label__ = 31;; } //@line 311 "compiledict.cpp"
                    if (__label__ == 28) {
  
                      var $58=$rule_addr; //@line 313 "compiledict.cpp"
                      var $incdec_ptr97=(($58+1)&4294967295); //@line 313 "compiledict.cpp"
                      $rule_addr=$incdec_ptr97; //@line 313 "compiledict.cpp"
                      var $59=HEAP8[($58)]; //@line 313 "compiledict.cpp"
                      var $conv98=(tempInt=(($59)),(tempInt>=128?tempInt-256:tempInt)); //@line 313 "compiledict.cpp"
                      var $sub99=((($conv98)-65)&4294967295); //@line 313 "compiledict.cpp"
                      $value=$sub99; //@line 313 "compiledict.cpp"
                      var $60=$p; //@line 314 "compiledict.cpp"
                      var $arrayidx100=(($60)&4294967295); //@line 314 "compiledict.cpp"
                      HEAP8[($arrayidx100)]=76; //@line 314 "compiledict.cpp"
                      var $61=$value; //@line 315 "compiledict.cpp"
                      var $div=((((($61))|0)/10)|0); //@line 315 "compiledict.cpp"
                      var $add101=((($div)+48)&4294967295); //@line 315 "compiledict.cpp"
                      var $conv102=((($add101)) & 255); //@line 315 "compiledict.cpp"
                      var $62=$p; //@line 315 "compiledict.cpp"
                      var $arrayidx103=(($62+1)&4294967295); //@line 315 "compiledict.cpp"
                      HEAP8[($arrayidx103)]=$conv102; //@line 315 "compiledict.cpp"
                      var $63=$value; //@line 316 "compiledict.cpp"
                      var $rem=((($63))|0)%10; //@line 316 "compiledict.cpp"
                      var $add104=((($rem)+48)&4294967295); //@line 316 "compiledict.cpp"
                      var $conv105=((($add104)) & 255); //@line 316 "compiledict.cpp"
                      $c=$conv105; //@line 316 "compiledict.cpp"
                      var $64=$match_type; //@line 318 "compiledict.cpp"
                      var $cmp106=((($64))|0)==1; //@line 318 "compiledict.cpp"
                      if ($cmp106) { __label__ = 29;; } else { __label__ = 30;; } //@line 318 "compiledict.cpp"
                      if (__label__ == 29) {
  
                        var $65=$c; //@line 320 "compiledict.cpp"
                        var $66=$p; //@line 320 "compiledict.cpp"
                        var $arrayidx108=(($66)&4294967295); //@line 320 "compiledict.cpp"
                        HEAP8[($arrayidx108)]=$65; //@line 320 "compiledict.cpp"
                        $c=76; //@line 321 "compiledict.cpp"
                        ; //@line 322 "compiledict.cpp"
                      }
  
                      var $67=$p; //@line 323 "compiledict.cpp"
                      var $add_ptr110=(($67+2)&4294967295); //@line 323 "compiledict.cpp"
                      $p=$add_ptr110; //@line 323 "compiledict.cpp"
                      ; //@line 324 "compiledict.cpp"
                    }
                    else if (__label__ == 31) {
  
                      var $68=$rb; //@line 326 "compiledict.cpp"
                      var $conv112=((($68))&255); //@line 326 "compiledict.cpp"
                      var $cmp113=((($conv112))|0) <= 31; //@line 326 "compiledict.cpp"
                      var $69=$rb; //@line 327 "compiledict.cpp"
                      var $idxprom=((($69))&255); //@line 327 "compiledict.cpp"
                      if ($cmp113) { __label__ = 32;; } else { __label__ = 33;; } //@line 326 "compiledict.cpp"
                      if (__label__ == 32) {
  
                        var $arrayidx115=((__ZZ10DecodeRulePKciPciE7symbols+$idxprom)&4294967295); //@line 327 "compiledict.cpp"
                        var $70=HEAP8[($arrayidx115)]; //@line 327 "compiledict.cpp"
                        $c=$70; //@line 327 "compiledict.cpp"
                        ; //@line 327 "compiledict.cpp"
                      }
                      else if (__label__ == 33) {
  
                        var $cmp118=((($idxprom))|0)==32; //@line 329 "compiledict.cpp"
                        if ($cmp118) { __label__ = 34;; } else { __label__ = 35;; } //@line 329 "compiledict.cpp"
                        if (__label__ == 34) {
  
                          $c=95; //@line 330 "compiledict.cpp"
                          ; //@line 330 "compiledict.cpp"
                        }
                        else if (__label__ == 35) {
  
                          var $71=$rb; //@line 332 "compiledict.cpp"
                          $c=$71; //@line 332 "compiledict.cpp"
                          ;
                        }
                      }
                    }
                  }
                }
              }
  
              var $72=$c; //@line 333 "compiledict.cpp"
              var $73=$p; //@line 333 "compiledict.cpp"
              var $incdec_ptr127=(($73+1)&4294967295); //@line 333 "compiledict.cpp"
              $p=$incdec_ptr127; //@line 333 "compiledict.cpp"
              HEAP8[($73)]=$72; //@line 333 "compiledict.cpp"
              ; //@line 334 "compiledict.cpp"
            }
          } while(0);
  
          var $74=$finished; //@line 233 "compiledict.cpp"
          var $tobool=((($74))|0)!=0; //@line 233 "compiledict.cpp"
          var $lnot=($tobool) ^ 1; //@line 233 "compiledict.cpp"
          if ($lnot) { __label__ = 4;continue $while_body$8; } else { __label__ = 38;break $while_body_lr_ph$$while_end$6; } //@line 233 "compiledict.cpp"
        }
      }
    } while(0);
  
    var $75=$p; //@line 335 "compiledict.cpp"
    HEAP8[($75)]=0; //@line 335 "compiledict.cpp"
    $p=((__ZZ10DecodeRulePKciPciE6output)&4294967295); //@line 337 "compiledict.cpp"
    var $76=$linenum; //@line 338 "compiledict.cpp"
    var $cmp128=((($76))|0) > 0; //@line 338 "compiledict.cpp"
    if ($cmp128) { __label__ = 39;; } else { __label__ = 40;; } //@line 338 "compiledict.cpp"
    if (__label__ == 39) {
  
      var $77=$p; //@line 340 "compiledict.cpp"
      var $78=$linenum; //@line 340 "compiledict.cpp"
      var $call130=_sprintf($77, ((__str64)&4294967295), allocate([$78,0,0,0], ["i32",0,0,0], ALLOC_STACK)); //@line 340 "compiledict.cpp"
      var $79=$p; //@line 341 "compiledict.cpp"
      var $add_ptr131=(($79+7)&4294967295); //@line 341 "compiledict.cpp"
      $p=$add_ptr131; //@line 341 "compiledict.cpp"
      ; //@line 342 "compiledict.cpp"
    }
  
    var $80=$condition_num; //@line 343 "compiledict.cpp"
    var $cmp133=((($80))|0) > 0; //@line 343 "compiledict.cpp"
    if ($cmp133) { __label__ = 41;; } else { __label__ = 42;; } //@line 343 "compiledict.cpp"
    if (__label__ == 41) {
  
      var $81=$p; //@line 345 "compiledict.cpp"
      var $82=$condition_num; //@line 345 "compiledict.cpp"
      var $call135=_sprintf($81, ((__str65)&4294967295), allocate([$82,0,0,0], ["i32",0,0,0], ALLOC_STACK)); //@line 345 "compiledict.cpp"
      var $83=$p; //@line 346 "compiledict.cpp"
      var $call136=_strlen($83); //@line 346 "compiledict.cpp"
      var $84=$p; //@line 346 "compiledict.cpp"
      var $arrayidx137=(($84+$call136)&4294967295); //@line 346 "compiledict.cpp"
      $p=$arrayidx137; //@line 346 "compiledict.cpp"
      ; //@line 347 "compiledict.cpp"
    }
  
    var $arraydecay139=(($buf_pre)&4294967295); //@line 348 "compiledict.cpp"
    var $call140=_strlen($arraydecay139); //@line 348 "compiledict.cpp"
    $ix=$call140; //@line 348 "compiledict.cpp"
    var $cmp141=((($call140))|0) > 0; //@line 348 "compiledict.cpp"
    var $_pr=$at_start; //@line 350 "compiledict.cpp"
    var $tobool145=((($_pr))|0)!=0; //@line 350 "compiledict.cpp"
    if ($cmp141) { __label__ = 44;; } else { __label__ = 43;; } //@line 348 "compiledict.cpp"
    $if_then144$$lor_lhs_false142$64: do { 
      if (__label__ == 44) {
  
        if ($tobool145) { __label__ = 45;break $if_then144$$lor_lhs_false142$64; } else { __label__ = 46;break $if_then144$$lor_lhs_false142$64; } //@line 350 "compiledict.cpp"
      }
      else if (__label__ == 43) {
  
        if ($tobool145) { __label__ = 45;break $if_then144$$lor_lhs_false142$64; } else { __label__ = 49;break $if_then144$$lor_lhs_false142$64; } //@line 348 "compiledict.cpp"
      }
    } while(0);
    $if_then146$$while_cond149_preheader$$if_end157$67: do { 
      if (__label__ == 45) {
  
        var $85=$p; //@line 351 "compiledict.cpp"
        var $incdec_ptr147=(($85+1)&4294967295); //@line 351 "compiledict.cpp"
        $p=$incdec_ptr147; //@line 351 "compiledict.cpp"
        HEAP8[($85)]=95; //@line 351 "compiledict.cpp"
        __label__ = 46;break $if_then146$$while_cond149_preheader$$if_end157$67; //@line 351 "compiledict.cpp"
      }
    } while(0);
    if (__label__ == 46) {
  
      var $86=$ix; //@line 352 "compiledict.cpp"
      var $dec4=((($86)-1)&4294967295); //@line 352 "compiledict.cpp"
      $ix=$dec4; //@line 352 "compiledict.cpp"
      var $cmp1505=((($dec4))|0) >= 0; //@line 352 "compiledict.cpp"
      if ($cmp1505) { __label__ = 47;; } else { __label__ = 48;; } //@line 352 "compiledict.cpp"
      $while_body151$$while_end154$71: do { 
        if (__label__ == 47) {
          while(1) { 
  
            var $87=$ix; //@line 353 "compiledict.cpp"
            var $arrayidx152=(($buf_pre+$87)&4294967295); //@line 353 "compiledict.cpp"
            var $88=HEAP8[($arrayidx152)]; //@line 353 "compiledict.cpp"
            var $89=$p; //@line 353 "compiledict.cpp"
            var $incdec_ptr153=(($89+1)&4294967295); //@line 353 "compiledict.cpp"
            $p=$incdec_ptr153; //@line 353 "compiledict.cpp"
            HEAP8[($89)]=$88; //@line 353 "compiledict.cpp"
            var $90=$ix; //@line 352 "compiledict.cpp"
            var $dec=((($90)-1)&4294967295); //@line 352 "compiledict.cpp"
            $ix=$dec; //@line 352 "compiledict.cpp"
            var $cmp150=((($dec))|0) >= 0; //@line 352 "compiledict.cpp"
            if ($cmp150) { __label__ = 47;continue ; } else { __label__ = 48;break $while_body151$$while_end154$71; } //@line 352 "compiledict.cpp"
          }
        }
      } while(0);
  
      var $91=$p; //@line 354 "compiledict.cpp"
      var $incdec_ptr155=(($91+1)&4294967295); //@line 354 "compiledict.cpp"
      $p=$incdec_ptr155; //@line 354 "compiledict.cpp"
      HEAP8[($91)]=41; //@line 354 "compiledict.cpp"
      var $92=$p; //@line 355 "compiledict.cpp"
      var $incdec_ptr156=(($92+1)&4294967295); //@line 355 "compiledict.cpp"
      $p=$incdec_ptr156; //@line 355 "compiledict.cpp"
      HEAP8[($92)]=32; //@line 355 "compiledict.cpp"
      ; //@line 356 "compiledict.cpp"
    }
  
    var $93=$p; //@line 357 "compiledict.cpp"
    HEAP8[($93)]=0; //@line 357 "compiledict.cpp"
    var $94=$p; //@line 358 "compiledict.cpp"
    var $arraydecay158=(($buf)&4294967295); //@line 358 "compiledict.cpp"
    var $call159=_strcat($94, $arraydecay158); //@line 358 "compiledict.cpp"
    var $call160=_strlen(((__ZZ10DecodeRulePKciPciE6output)&4294967295)); //@line 359 "compiledict.cpp"
    $ix=$call160; //@line 359 "compiledict.cpp"
    var $cmp1623=((($call160))|0) < 8; //@line 360 "compiledict.cpp"
    var $95=$ix; //@line 361 "compiledict.cpp"
    if ($cmp1623) { __lastLabel__ = 49; __label__ = 50;; } else { __lastLabel__ = 49; __label__ = 51;; } //@line 360 "compiledict.cpp"
    $while_body163$$while_end166$76: do { 
      if (__label__ == 50) {
        while(1) { 
  
          var $96=__lastLabel__ == 50 ? $97 : ($95);
          var $inc164=((($96)+1)&4294967295); //@line 361 "compiledict.cpp"
          $ix=$inc164; //@line 361 "compiledict.cpp"
          var $arrayidx165=((__ZZ10DecodeRulePKciPciE6output+$96)&4294967295); //@line 361 "compiledict.cpp"
          HEAP8[($arrayidx165)]=32; //@line 361 "compiledict.cpp"
          var $_pr2=$ix; //@line 360 "compiledict.cpp"
          var $cmp162=((($_pr2))|0) < 8; //@line 360 "compiledict.cpp"
          var $97=$ix; //@line 361 "compiledict.cpp"
          if ($cmp162) { __lastLabel__ = 50; __label__ = 50;continue ; } else { __lastLabel__ = 50; __label__ = 51;break $while_body163$$while_end166$76; } //@line 360 "compiledict.cpp"
        }
      }
    } while(0);
  
    var $_lcssa=__lastLabel__ == 49 ? $95 : ($97);
    var $arrayidx167=((__ZZ10DecodeRulePKciPciE6output+$_lcssa)&4294967295); //@line 362 "compiledict.cpp"
    HEAP8[($arrayidx167)]=0; //@line 362 "compiledict.cpp"
    STACKTOP = __stackBase__;
    return ((__ZZ10DecodeRulePKciPciE6output)&4294967295); //@line 363 "compiledict.cpp"
    return null;
  }
  

  function __Z13string_sorterPPcS0_($a, $b) {
    ;
    var __label__;
  
    var $retval;
    var $a_addr;
    var $b_addr;
    var $pa;
    var $pb;
    var $ix;
    $a_addr=$a;
    $b_addr=$b;
    var $0=$a_addr; //@line 1341 "compiledict.cpp"
    var $1=HEAP32[(($0)>>2)]; //@line 1341 "compiledict.cpp"
    $pa=$1; //@line 1341 "compiledict.cpp"
    var $2=$b_addr; //@line 1341 "compiledict.cpp"
    var $3=HEAP32[(($2)>>2)]; //@line 1341 "compiledict.cpp"
    $pb=$3; //@line 1341 "compiledict.cpp"
    var $call=_strcmp($1, $3); //@line 1341 "compiledict.cpp"
    $ix=$call; //@line 1341 "compiledict.cpp"
    var $cmp=((($call))|0)!=0; //@line 1341 "compiledict.cpp"
    if ($cmp) { __label__ = 1;; } else { __label__ = 2;; } //@line 1341 "compiledict.cpp"
    if (__label__ == 1) {
  
      var $4=$ix; //@line 1342 "compiledict.cpp"
      $retval=$4; //@line 1342 "compiledict.cpp"
      ; //@line 1342 "compiledict.cpp"
    }
    else if (__label__ == 2) {
  
      var $5=$pa; //@line 1343 "compiledict.cpp"
      var $call1=_strlen($5); //@line 1343 "compiledict.cpp"
      var $add=((($call1)+1)&4294967295); //@line 1343 "compiledict.cpp"
      var $6=$pa; //@line 1343 "compiledict.cpp"
      var $add_ptr=(($6+$add)&4294967295); //@line 1343 "compiledict.cpp"
      $pa=$add_ptr; //@line 1343 "compiledict.cpp"
      var $7=$pb; //@line 1344 "compiledict.cpp"
      var $call2=_strlen($7); //@line 1344 "compiledict.cpp"
      var $add3=((($call2)+1)&4294967295); //@line 1344 "compiledict.cpp"
      var $8=$pb; //@line 1344 "compiledict.cpp"
      var $add_ptr4=(($8+$add3)&4294967295); //@line 1344 "compiledict.cpp"
      $pb=$add_ptr4; //@line 1344 "compiledict.cpp"
      var $9=$pa; //@line 1345 "compiledict.cpp"
      var $10=$pb; //@line 1345 "compiledict.cpp"
      var $call5=_strcmp($9, $10); //@line 1345 "compiledict.cpp"
      $retval=$call5; //@line 1345 "compiledict.cpp"
      ; //@line 1345 "compiledict.cpp"
    }
  
    var $11=$retval; //@line 1346 "compiledict.cpp"
    ;
    return $11; //@line 1346 "compiledict.cpp"
    return null;
  }
  

  function __Z17CompileDictionaryPKcS0_P7__sFILEPci($dsource, $dict_name, $log, $fname_err, $flags) {
    var __stackBase__  = STACKTOP; STACKTOP += 724; _memset(__stackBase__, 0, 724);
    var __label__;
  
    var $retval;
    var $dsource_addr;
    var $dict_name_addr;
    var $log_addr;
    var $fname_err_addr;
    var $flags_addr;
    var $f_in;
    var $f_out;
    var $offset_rules;
    var $value;
    var $fname_in=__stackBase__;
    var $fname_out=__stackBase__+196;
    var $fname_temp=__stackBase__+364;
    var $path=__stackBase__+532;
    $dsource_addr=$dsource;
    $dict_name_addr=$dict_name;
    $log_addr=$log;
    $fname_err_addr=$fname_err;
    $flags_addr=$flags;
    $offset_rules=0; //@line 1856 "compiledict.cpp"
    HEAP32[((__ZL11error_count)>>2)]=0; //@line 1863 "compiledict.cpp"
    HEAP32[((__ZL21error_need_dictionary)>>2)]=0; //@line 1864 "compiledict.cpp"
    var $dest$, $stop$, $stop4$, $fast$, $value4$;
    $dest$ = ((__ZL19letterGroupsDefined)&4294967295);
    $stop$ = $dest$ + 26;
    $value4$ = 0;
    if ($value4$ < 0) $value4$ += 256;
    $value4$ = $value4$ + ($value4$<<8) + ($value4$<<16) + ($value4$*16777216);
    while ($dest$%4 !== 0 && $dest$ < $stop$) {
      ;; HEAP8[$dest$++] = 0;
    }
    $dest$ >>= 2;
    $stop4$ = $stop$ >> 2;
    while ($dest$ < $stop4$) {
    ;;;;;;;  HEAP32[$dest$++] = $value4$;
    }
    $dest$ <<= 2;
    while ($dest$ < $stop$) {
      ;; HEAP8[$dest$++] = 0;
    }; //@line 1865 "compiledict.cpp"
    var $0=$flags_addr; //@line 1867 "compiledict.cpp"
    var $and=($0) & 1; //@line 1867 "compiledict.cpp"
    HEAP32[((__ZL10debug_flag)>>2)]=$and; //@line 1867 "compiledict.cpp"
    var $1=$dsource_addr; //@line 1869 "compiledict.cpp"
    var $cmp=((($1))|0)==0; //@line 1869 "compiledict.cpp"
    if ($cmp) { __label__ = 1;; } else { __label__ = 2;; } //@line 1869 "compiledict.cpp"
    if (__label__ == 1) {
  
      $dsource_addr=((__str13643)&4294967295); //@line 1870 "compiledict.cpp"
      ; //@line 1870 "compiledict.cpp"
    }
  
    var $2=$log_addr; //@line 1872 "compiledict.cpp"
    HEAP32[((__ZL5f_log)>>2)]=$2; //@line 1872 "compiledict.cpp"
    var $cmp1=((($2))|0)==0; //@line 1874 "compiledict.cpp"
    if ($cmp1) { __label__ = 3;; } else { __label__ = 4;; } //@line 1874 "compiledict.cpp"
    if (__label__ == 3) {
  
      var $3=HEAP32[((__impure_ptr)>>2)]; //@line 1875 "compiledict.cpp"
      var $_stderr=(($3+12)&4294967295); //@line 1875 "compiledict.cpp"
      var $4=HEAP32[(($_stderr)>>2)]; //@line 1875 "compiledict.cpp"
      HEAP32[((__ZL5f_log)>>2)]=$4; //@line 1875 "compiledict.cpp"
      ; //@line 1875 "compiledict.cpp"
    }
  
    var $arraydecay=(($path)&4294967295); //@line 1878 "compiledict.cpp"
    var $5=$dsource_addr; //@line 1878 "compiledict.cpp"
    var $6=$dict_name_addr; //@line 1878 "compiledict.cpp"
    var $call=_sprintf($arraydecay, ((__str66)&4294967295), allocate([$5,0,0,0,$6,0,0,0], ["i8*",0,0,0,"i8*",0,0,0], ALLOC_STACK)); //@line 1878 "compiledict.cpp"
    var $arraydecay4=(($fname_in)&4294967295); //@line 1879 "compiledict.cpp"
    var $arraydecay5=(($path)&4294967295); //@line 1879 "compiledict.cpp"
    var $call6=_sprintf($arraydecay4, ((__str67)&4294967295), allocate([$arraydecay5,0,0,0], ["i8*",0,0,0], ALLOC_STACK)); //@line 1879 "compiledict.cpp"
    var $arraydecay7=(($fname_in)&4294967295); //@line 1880 "compiledict.cpp"
    var $call8=_fopen($arraydecay7, ((__str51703)&4294967295)); //@line 1880 "compiledict.cpp"
    $f_in=$call8; //@line 1880 "compiledict.cpp"
    var $cmp9=((($call8))|0)==0; //@line 1880 "compiledict.cpp"
    if ($cmp9) { __label__ = 5;; } else { __label__ = 9;; } //@line 1880 "compiledict.cpp"
    $if_then10$$if_end23$8: do { 
      if (__label__ == 5) {
  
        var $arraydecay11=(($fname_in)&4294967295); //@line 1882 "compiledict.cpp"
        var $arraydecay12=(($path)&4294967295); //@line 1882 "compiledict.cpp"
        var $call13=_sprintf($arraydecay11, ((__str69)&4294967295), allocate([$arraydecay12,0,0,0], ["i8*",0,0,0], ALLOC_STACK)); //@line 1882 "compiledict.cpp"
        var $arraydecay14=(($fname_in)&4294967295); //@line 1883 "compiledict.cpp"
        var $call15=__ZL9fopen_logPKcS0_($arraydecay14, ((__str51703)&4294967295)); //@line 1883 "compiledict.cpp"
        $f_in=$call15; //@line 1883 "compiledict.cpp"
        var $cmp16=((($call15))|0)==0; //@line 1883 "compiledict.cpp"
        if (!($cmp16)) { __label__ = 9;break $if_then10$$if_end23$8; } //@line 1883 "compiledict.cpp"
  
        var $7=$fname_err_addr; //@line 1885 "compiledict.cpp"
        var $tobool=((($7))|0)!=0; //@line 1885 "compiledict.cpp"
        if ($tobool) { __label__ = 7;; } else { __label__ = 8;; } //@line 1885 "compiledict.cpp"
        if (__label__ == 7) {
  
          var $8=$fname_err_addr; //@line 1886 "compiledict.cpp"
          var $arraydecay19=(($fname_in)&4294967295); //@line 1886 "compiledict.cpp"
          var $call20=_strcpy($8, $arraydecay19); //@line 1886 "compiledict.cpp"
          ; //@line 1886 "compiledict.cpp"
        }
  
        $retval=-1; //@line 1887 "compiledict.cpp"
        __label__ = 17;break $if_then10$$if_end23$8; //@line 1887 "compiledict.cpp"
      }
    } while(0);
    if (__label__ == 9) {
  
      var $arraydecay24=(($fname_out)&4294967295); //@line 1891 "compiledict.cpp"
      var $9=$dict_name_addr; //@line 1891 "compiledict.cpp"
      var $call25=_sprintf($arraydecay24, ((__str116)&4294967295), allocate([((_path_home)&4294967295),0,0,0,47,0,0,0,$9,0,0,0], ["i8*",0,0,0,"i32",0,0,0,"i8*",0,0,0], ALLOC_STACK)); //@line 1891 "compiledict.cpp"
      var $arraydecay26=(($fname_out)&4294967295); //@line 1892 "compiledict.cpp"
      var $call27=__ZL9fopen_logPKcS0_($arraydecay26, ((__str71)&4294967295)); //@line 1892 "compiledict.cpp"
      $f_out=$call27; //@line 1892 "compiledict.cpp"
      var $cmp28=((($call27))|0)==0; //@line 1892 "compiledict.cpp"
      if ($cmp28) { __label__ = 10;; } else { __label__ = 13;; } //@line 1892 "compiledict.cpp"
      if (__label__ == 10) {
  
        var $10=$fname_err_addr; //@line 1894 "compiledict.cpp"
        var $tobool30=((($10))|0)!=0; //@line 1894 "compiledict.cpp"
        if ($tobool30) { __label__ = 11;; } else { __label__ = 12;; } //@line 1894 "compiledict.cpp"
        if (__label__ == 11) {
  
          var $11=$fname_err_addr; //@line 1895 "compiledict.cpp"
          var $arraydecay32=(($fname_in)&4294967295); //@line 1895 "compiledict.cpp"
          var $call33=_strcpy($11, $arraydecay32); //@line 1895 "compiledict.cpp"
          ; //@line 1895 "compiledict.cpp"
        }
  
        $retval=-1; //@line 1896 "compiledict.cpp"
        ; //@line 1896 "compiledict.cpp"
      }
      else if (__label__ == 13) {
  
        var $arraydecay36=(($fname_temp)&4294967295); //@line 1898 "compiledict.cpp"
        var $call37=_sprintf($arraydecay36, ((__str72)&4294967295), allocate([((_path_home)&4294967295),0,0,0,47,0,0,0], ["i8*",0,0,0,"i32",0,0,0], ALLOC_STACK)); //@line 1898 "compiledict.cpp"
        $value=1024; //@line 1900 "compiledict.cpp"
        var $12=$f_out; //@line 1901 "compiledict.cpp"
        var $13=$value; //@line 1901 "compiledict.cpp"
        __Z11Write4BytesP7__sFILEi($12, $13); //@line 1901 "compiledict.cpp"
        var $14=$f_out; //@line 1902 "compiledict.cpp"
        var $15=$offset_rules; //@line 1902 "compiledict.cpp"
        __Z11Write4BytesP7__sFILEi($14, $15); //@line 1902 "compiledict.cpp"
        __ZL22compile_dictlist_startv(); //@line 1904 "compiledict.cpp"
        var $16=HEAP32[((__ZL5f_log)>>2)]; //@line 1906 "compiledict.cpp"
        var $17=HEAP32[((_phoneme_tab_number)>>2)]; //@line 1906 "compiledict.cpp"
        var $arrayidx=((_phoneme_tab_list+$17*48)&4294967295); //@line 1906 "compiledict.cpp"
        var $name=(($arrayidx)&4294967295); //@line 1906 "compiledict.cpp"
        var $arraydecay38=(($name)&4294967295); //@line 1906 "compiledict.cpp"
        var $call39=_fprintf($16, ((__str73)&4294967295), allocate([$arraydecay38,0,0,0], ["i8*",0,0,0], ALLOC_STACK)); //@line 1906 "compiledict.cpp"
        var $arraydecay40=(($path)&4294967295); //@line 1907 "compiledict.cpp"
        __ZL21compile_dictlist_filePKcS0_($arraydecay40, ((__str74)&4294967295)); //@line 1907 "compiledict.cpp"
        var $18=HEAP32[((_translator)>>2)]; //@line 1908 "compiledict.cpp"
        var $langopts=(($18)&4294967295); //@line 1908 "compiledict.cpp"
        var $listx=(($langopts+260)&4294967295); //@line 1908 "compiledict.cpp"
        var $19=HEAP32[(($listx)>>2)]; //@line 1908 "compiledict.cpp"
        var $tobool42=((($19))|0)!=0; //@line 1908 "compiledict.cpp"
        var $arraydecay44=(($path)&4294967295); //@line 1910 "compiledict.cpp"
        if ($tobool42) { __label__ = 14;; } else { __label__ = 15;; } //@line 1908 "compiledict.cpp"
        if (__label__ == 14) {
  
          __ZL21compile_dictlist_filePKcS0_($arraydecay44, ((__str75)&4294967295)); //@line 1910 "compiledict.cpp"
          var $arraydecay46=(($path)&4294967295); //@line 1911 "compiledict.cpp"
          __ZL21compile_dictlist_filePKcS0_($arraydecay46, ((__str76)&4294967295)); //@line 1911 "compiledict.cpp"
          ; //@line 1912 "compiledict.cpp"
        }
        else if (__label__ == 15) {
  
          __ZL21compile_dictlist_filePKcS0_($arraydecay44, ((__str76)&4294967295)); //@line 1915 "compiledict.cpp"
          var $arraydecay50=(($path)&4294967295); //@line 1916 "compiledict.cpp"
          __ZL21compile_dictlist_filePKcS0_($arraydecay50, ((__str75)&4294967295)); //@line 1916 "compiledict.cpp"
          ;
        }
  
        var $arraydecay53=(($path)&4294967295); //@line 1918 "compiledict.cpp"
        __ZL21compile_dictlist_filePKcS0_($arraydecay53, ((__str77)&4294967295)); //@line 1918 "compiledict.cpp"
        var $20=$f_out; //@line 1920 "compiledict.cpp"
        __ZL20compile_dictlist_endP7__sFILE($20); //@line 1920 "compiledict.cpp"
        var $21=$f_out; //@line 1921 "compiledict.cpp"
        var $call55=_ftell($21); //@line 1921 "compiledict.cpp"
        $offset_rules=$call55; //@line 1921 "compiledict.cpp"
        var $22=HEAP32[((__ZL5f_log)>>2)]; //@line 1923 "compiledict.cpp"
        var $arraydecay56=(($fname_in)&4294967295); //@line 1923 "compiledict.cpp"
        var $call57=_fprintf($22, ((__str78)&4294967295), allocate([$arraydecay56,0,0,0], ["i8*",0,0,0], ALLOC_STACK)); //@line 1923 "compiledict.cpp"
        var $23=$f_in; //@line 1925 "compiledict.cpp"
        var $24=$f_out; //@line 1925 "compiledict.cpp"
        var $arraydecay58=(($fname_temp)&4294967295); //@line 1925 "compiledict.cpp"
        __ZL17compile_dictrulesP7__sFILES0_Pc($23, $24, $arraydecay58); //@line 1925 "compiledict.cpp"
        var $25=$f_in; //@line 1926 "compiledict.cpp"
        var $call60=_fclose($25); //@line 1926 "compiledict.cpp"
        var $26=$f_out; //@line 1928 "compiledict.cpp"
        var $call61=_fseek($26, 4, 0); //@line 1928 "compiledict.cpp"
        var $27=$f_out; //@line 1929 "compiledict.cpp"
        var $28=$offset_rules; //@line 1929 "compiledict.cpp"
        __Z11Write4BytesP7__sFILEi($27, $28); //@line 1929 "compiledict.cpp"
        var $29=$f_out; //@line 1930 "compiledict.cpp"
        var $call62=_fclose($29); //@line 1930 "compiledict.cpp"
        var $30=HEAP32[((_translator)>>2)]; //@line 1932 "compiledict.cpp"
        var $31=$dict_name_addr; //@line 1932 "compiledict.cpp"
        var $call63=__Z14LoadDictionaryP10TranslatorPKci($30, $31, 0); //@line 1932 "compiledict.cpp"
        var $32=HEAP32[((__ZL11error_count)>>2)]; //@line 1934 "compiledict.cpp"
        $retval=$32; //@line 1934 "compiledict.cpp"
        ; //@line 1934 "compiledict.cpp"
      }
    }
  
    var $33=$retval; //@line 1935 "compiledict.cpp"
    STACKTOP = __stackBase__;
    return $33; //@line 1935 "compiledict.cpp"
    return null;
  }
  

  function __ZL9fopen_logPKcS0_($fname, $access) {
    ;
    var __label__;
  
    var $fname_addr;
    var $access_addr;
    var $f;
    $fname_addr=$fname;
    $access_addr=$access;
    var $0=$fname_addr; //@line 162 "compiledict.cpp"
    var $1=$access_addr; //@line 162 "compiledict.cpp"
    var $call=_fopen($0, $1); //@line 162 "compiledict.cpp"
    $f=$call; //@line 162 "compiledict.cpp"
    var $cmp=((($call))|0)==0; //@line 162 "compiledict.cpp"
    var $2=HEAP32[((__ZL5f_log)>>2)]; //@line 164 "compiledict.cpp"
    var $cmp1=((($2))|0)!=0; //@line 164 "compiledict.cpp"
    var $or_cond=($cmp) & ($cmp1); //@line 162 "compiledict.cpp"
    if ($or_cond) { __label__ = 1;; } else { __label__ = 2;; } //@line 162 "compiledict.cpp"
    if (__label__ == 1) {
  
      var $3=HEAP32[((__ZL5f_log)>>2)]; //@line 165 "compiledict.cpp"
      var $4=$access_addr; //@line 165 "compiledict.cpp"
      var $5=$fname_addr; //@line 165 "compiledict.cpp"
      var $call3=_fprintf($3, ((__str115)&4294967295), allocate([$4,0,0,0,$5,0,0,0], ["i8*",0,0,0,"i8*",0,0,0], ALLOC_STACK)); //@line 165 "compiledict.cpp"
      ; //@line 165 "compiledict.cpp"
    }
  
    var $6=$f; //@line 167 "compiledict.cpp"
    ;
    return $6; //@line 167 "compiledict.cpp"
    return null;
  }
  

  function __ZL22compile_dictlist_startv() {
    var __stackBase__  = STACKTOP; STACKTOP += 4; _memset(__stackBase__, 0, 4);
    var __label__;
  
    var $ix;
    var $p;
    var $p2=__stackBase__;
    $ix=0; //@line 792 "compiledict.cpp"
    var $0=$p2; //@line 797 "compiledict.cpp"
    ; //@line 792 "compiledict.cpp"
    $for_body$2: while(1) { 
  
      var $1=$ix; //@line 794 "compiledict.cpp"
      var $arrayidx=((__ZL11hash_chains+$1*4)&4294967295); //@line 794 "compiledict.cpp"
      var $2=HEAP32[(($arrayidx)>>2)]; //@line 794 "compiledict.cpp"
      $p=$2; //@line 794 "compiledict.cpp"
      var $cmp11=((($2))|0)!=0; //@line 795 "compiledict.cpp"
      if ($cmp11) { __label__ = 2;; } else { __label__ = 3;; } //@line 795 "compiledict.cpp"
      $while_body$$while_end$4: do { 
        if (__label__ == 2) {
          while(1) { 
  
            var $3=$p; //@line 797 "compiledict.cpp"
            var $src$, $dest$, $stop$, $stop4$, $fast$;
            $src$ = $3;
            $dest$ = $0;
            $stop$ = $src$ + 4;
            if (($dest$%4) == ($src$%4) && 4 > 8) {
              while ($src$%4 !== 0 && $src$ < $stop$) {
                ; HEAP8[$dest$++] = HEAP8[$src$++];
              }
              $src$ >>= 2;
              $dest$ >>= 2;
              $stop4$ = $stop$ >> 2;
              while ($src$ < $stop4$) {
            ;;;    HEAP32[$dest$++] = HEAP32[$src$++];
              }
              $src$ <<= 2;
              $dest$ <<= 2;
            }while ($src$ < $stop$) {
              ; HEAP8[$dest$++] = HEAP8[$src$++];
            }; //@line 797 "compiledict.cpp"
            var $4=$p; //@line 798 "compiledict.cpp"
            ; //@line 798 "compiledict.cpp"
            var $5=HEAP32[(($p2)>>2)]; //@line 799 "compiledict.cpp"
            $p=$5; //@line 799 "compiledict.cpp"
            var $cmp1=((($5))|0)!=0; //@line 795 "compiledict.cpp"
            if ($cmp1) { __label__ = 2;continue ; } else { __label__ = 3;break $while_body$$while_end$4; } //@line 795 "compiledict.cpp"
          }
        }
      } while(0);
  
      var $6=$ix; //@line 801 "compiledict.cpp"
      var $arrayidx2=((__ZL11hash_chains+$6*4)&4294967295); //@line 801 "compiledict.cpp"
      HEAP32[(($arrayidx2)>>2)]=0; //@line 801 "compiledict.cpp"
      var $7=$ix; //@line 802 "compiledict.cpp"
      var $arrayidx3=((__ZL11hash_counts+$7*4)&4294967295); //@line 802 "compiledict.cpp"
      HEAP32[(($arrayidx3)>>2)]=0; //@line 802 "compiledict.cpp"
      var $8=$ix; //@line 792 "compiledict.cpp"
      var $inc=((($8)+1)&4294967295); //@line 792 "compiledict.cpp"
      $ix=$inc; //@line 792 "compiledict.cpp"
      var $cmp=((($inc))|0) < 1024; //@line 792 "compiledict.cpp"
      if ($cmp) { __label__ = 1;continue $for_body$2; } else { __label__ = 4;break $for_body$2; } //@line 792 "compiledict.cpp"
    }
  
    STACKTOP = __stackBase__;
    return; //@line 804 "compiledict.cpp"
    return;
  }
  

  function __ZL21compile_dictlist_filePKcS0_($path, $filename) {
    var __stackBase__  = STACKTOP; STACKTOP += 528; _memset(__stackBase__, 0, 528);
    var __label__;
  
    var $retval;
    var $path_addr;
    var $filename_addr;
    var $length;
    var $hash=__stackBase__;
    var $p;
    var $count;
    var $f_in;
    var $buf=__stackBase__+4;
    var $fname=__stackBase__+204;
    var $dict_line=__stackBase__+400;
    $path_addr=$path;
    $filename_addr=$filename;
    $count=0; //@line 849 "compiledict.cpp"
    HEAP8[(__ZL9text_mode_b)]=0;
    var $arraydecay=(($fname)&4294967295); //@line 858 "compiledict.cpp"
    var $0=$path_addr; //@line 858 "compiledict.cpp"
    var $1=$filename_addr; //@line 858 "compiledict.cpp"
    var $call=_sprintf($arraydecay, ((__str104)&4294967295), allocate([$0,0,0,0,$1,0,0,0], ["i8*",0,0,0,"i8*",0,0,0], ALLOC_STACK)); //@line 858 "compiledict.cpp"
    var $arraydecay1=(($fname)&4294967295); //@line 859 "compiledict.cpp"
    var $call2=_fopen($arraydecay1, ((__str51703)&4294967295)); //@line 859 "compiledict.cpp"
    $f_in=$call2; //@line 859 "compiledict.cpp"
    var $cmp=((($call2))|0)==0; //@line 859 "compiledict.cpp"
    if ($cmp) { __label__ = 1;; } else { __label__ = 3;; } //@line 859 "compiledict.cpp"
    $if_then$$if_end9$2: do { 
      if (__label__ == 1) {
  
        var $arraydecay3=(($fname)&4294967295); //@line 861 "compiledict.cpp"
        var $2=$path_addr; //@line 861 "compiledict.cpp"
        var $3=$filename_addr; //@line 861 "compiledict.cpp"
        var $call4=_sprintf($arraydecay3, ((__str47699)&4294967295), allocate([$2,0,0,0,$3,0,0,0], ["i8*",0,0,0,"i8*",0,0,0], ALLOC_STACK)); //@line 861 "compiledict.cpp"
        var $arraydecay5=(($fname)&4294967295); //@line 862 "compiledict.cpp"
        var $call6=_fopen($arraydecay5, ((__str51703)&4294967295)); //@line 862 "compiledict.cpp"
        $f_in=$call6; //@line 862 "compiledict.cpp"
        var $cmp7=((($call6))|0)==0; //@line 862 "compiledict.cpp"
        if (!($cmp7)) { __label__ = 3;break $if_then$$if_end9$2; } //@line 862 "compiledict.cpp"
  
        $retval=-1; //@line 863 "compiledict.cpp"
        __label__ = 11;break $if_then$$if_end9$2; //@line 863 "compiledict.cpp"
      }
    } while(0);
    if (__label__ == 3) {
  
      var $4=HEAP32[((__ZL5f_log)>>2)]; //@line 866 "compiledict.cpp"
      var $arraydecay10=(($fname)&4294967295); //@line 866 "compiledict.cpp"
      var $call11=_fprintf($4, ((__str78)&4294967295), allocate([$arraydecay10,0,0,0], ["i8*",0,0,0], ALLOC_STACK)); //@line 866 "compiledict.cpp"
      HEAP32[((__ZL7linenum)>>2)]=0; //@line 868 "compiledict.cpp"
      var $arraydecay12=(($buf)&4294967295); //@line 870 "compiledict.cpp"
      var $arraydecay15=(($buf)&4294967295); //@line 874 "compiledict.cpp"
      var $arraydecay16=(($dict_line)&4294967295); //@line 874 "compiledict.cpp"
      var $arraydecay33=(($dict_line)&4294967295); //@line 892 "compiledict.cpp"
      ; //@line 870 "compiledict.cpp"
      $while_cond$7: while(1) { 
  
        var $5=$f_in; //@line 870 "compiledict.cpp"
        var $call13=_fgets($arraydecay12, 200, $5); //@line 870 "compiledict.cpp"
        var $cmp14=((($call13))|0)!=0; //@line 870 "compiledict.cpp"
        if (!($cmp14)) { __label__ = 10;break $while_cond$7; } //@line 870 "compiledict.cpp"
  
        var $6=HEAP32[((__ZL7linenum)>>2)]; //@line 872 "compiledict.cpp"
        var $inc=((($6)+1)&4294967295); //@line 872 "compiledict.cpp"
        HEAP32[((__ZL7linenum)>>2)]=$inc; //@line 872 "compiledict.cpp"
        var $call17=__ZL12compile_linePcS_Pi($arraydecay15, $arraydecay16, $hash); //@line 874 "compiledict.cpp"
        $length=$call17; //@line 874 "compiledict.cpp"
        var $7=$length; //@line 875 "compiledict.cpp"
        var $cmp18=((($7))|0)==0; //@line 875 "compiledict.cpp"
        if ($cmp18) { __label__ = 4;continue $while_cond$7; } //@line 875 "compiledict.cpp"
  
        var $8=HEAP32[(($hash)>>2)]; //@line 877 "compiledict.cpp"
        var $arrayidx=((__ZL11hash_counts+$8*4)&4294967295); //@line 877 "compiledict.cpp"
        var $9=HEAP32[(($arrayidx)>>2)]; //@line 877 "compiledict.cpp"
        var $inc21=((($9)+1)&4294967295); //@line 877 "compiledict.cpp"
        HEAP32[(($arrayidx)>>2)]=$inc21; //@line 877 "compiledict.cpp"
        var $10=$length; //@line 879 "compiledict.cpp"
        var $add=((($10)+4)&4294967295); //@line 879 "compiledict.cpp"
        var $call22=_malloc($add); //@line 879 "compiledict.cpp"
        $p=$call22; //@line 879 "compiledict.cpp"
        var $11=$p; //@line 880 "compiledict.cpp"
        var $cmp23=((($11))|0)==0; //@line 880 "compiledict.cpp"
        if ($cmp23) { __label__ = 7;break $while_cond$7; } //@line 880 "compiledict.cpp"
  
        var $16=$p; //@line 890 "compiledict.cpp"
        var $17=HEAP32[(($hash)>>2)]; //@line 890 "compiledict.cpp"
        var $arrayidx31=((__ZL11hash_chains+$17*4)&4294967295); //@line 890 "compiledict.cpp"
        var $18=$arrayidx31; //@line 890 "compiledict.cpp"
        var $src$, $dest$, $stop$, $stop4$, $fast$;
        $src$ = $18;
        $dest$ = $16;
        $stop$ = $src$ + 4;
        if (($dest$%4) == ($src$%4) && 4 > 8) {
          while ($src$%4 !== 0 && $src$ < $stop$) {
            ; HEAP8[$dest$++] = HEAP8[$src$++];
          }
          $src$ >>= 2;
          $dest$ >>= 2;
          $stop4$ = $stop$ >> 2;
          while ($src$ < $stop4$) {
        ;;;    HEAP32[$dest$++] = HEAP32[$src$++];
          }
          $src$ <<= 2;
          $dest$ <<= 2;
        }while ($src$ < $stop$) {
          ; HEAP8[$dest$++] = HEAP8[$src$++];
        }; //@line 890 "compiledict.cpp"
        var $19=$p; //@line 891 "compiledict.cpp"
        var $20=HEAP32[(($hash)>>2)]; //@line 891 "compiledict.cpp"
        var $arrayidx32=((__ZL11hash_chains+$20*4)&4294967295); //@line 891 "compiledict.cpp"
        HEAP32[(($arrayidx32)>>2)]=$19; //@line 891 "compiledict.cpp"
        var $21=$p; //@line 892 "compiledict.cpp"
        var $add_ptr=(($21+4)&4294967295); //@line 892 "compiledict.cpp"
        var $22=$length; //@line 892 "compiledict.cpp"
        var $src$, $dest$, $stop$, $stop4$, $fast$;
        $src$ = $arraydecay33;
        $dest$ = $add_ptr;
        $stop$ = $src$ + $22;
        if (($dest$%4) == ($src$%4) && $22 > 8) {
          while ($src$%4 !== 0 && $src$ < $stop$) {
            ; HEAP8[$dest$++] = HEAP8[$src$++];
          }
          $src$ >>= 2;
          $dest$ >>= 2;
          $stop4$ = $stop$ >> 2;
          while ($src$ < $stop4$) {
        ;;;    HEAP32[$dest$++] = HEAP32[$src$++];
          }
          $src$ <<= 2;
          $dest$ <<= 2;
        }while ($src$ < $stop$) {
          ; HEAP8[$dest$++] = HEAP8[$src$++];
        }; //@line 892 "compiledict.cpp"
        var $23=$count; //@line 893 "compiledict.cpp"
        var $inc34=((($23)+1)&4294967295); //@line 893 "compiledict.cpp"
        $count=$inc34; //@line 893 "compiledict.cpp"
        __label__ = 4;continue $while_cond$7; //@line 894 "compiledict.cpp"
      }
      $while_end$$if_then24$12: do { 
        if (__label__ == 7) {
  
          var $12=HEAP32[((__ZL5f_log)>>2)]; //@line 882 "compiledict.cpp"
          var $cmp25=((($12))|0)!=0; //@line 882 "compiledict.cpp"
          if (!($cmp25)) { __label__ = 10;break $while_end$$if_then24$12; } //@line 882 "compiledict.cpp"
  
          var $13=HEAP32[((__ZL5f_log)>>2)]; //@line 884 "compiledict.cpp"
          var $14=_fwrite(((__str106)&4294967295), 22, 1, $13); //@line 884 "compiledict.cpp"
          var $15=HEAP32[((__ZL11error_count)>>2)]; //@line 885 "compiledict.cpp"
          var $inc28=((($15)+1)&4294967295); //@line 885 "compiledict.cpp"
          HEAP32[((__ZL11error_count)>>2)]=$inc28; //@line 885 "compiledict.cpp"
          ; //@line 886 "compiledict.cpp"
        }
      } while(0);
  
      var $24=HEAP32[((__ZL5f_log)>>2)]; //@line 896 "compiledict.cpp"
      var $25=$count; //@line 896 "compiledict.cpp"
      var $call35=_fprintf($24, ((__str107)&4294967295), allocate([$25,0,0,0], ["i32",0,0,0], ALLOC_STACK)); //@line 896 "compiledict.cpp"
      var $26=$f_in; //@line 897 "compiledict.cpp"
      var $call36=_fclose($26); //@line 897 "compiledict.cpp"
      $retval=0; //@line 898 "compiledict.cpp"
      ; //@line 898 "compiledict.cpp"
    }
  
    STACKTOP = __stackBase__;
    return;
    return;
  }
  

  function __ZL20compile_dictlist_endP7__sFILE($f_out) {
    var __stackBase__  = STACKTOP; STACKTOP += 4; _memset(__stackBase__, 0, 4);
    var __label__;
  
    var $f_out_addr;
    var $hash;
    var $length;
    var $p=__stackBase__;
    $f_out_addr=$f_out;
    $hash=0; //@line 827 "compiledict.cpp"
    var $0=$p; //@line 836 "compiledict.cpp"
    ; //@line 827 "compiledict.cpp"
    $for_body$2: while(1) { 
  
      var $1=$hash; //@line 829 "compiledict.cpp"
      var $arrayidx=((__ZL11hash_chains+$1*4)&4294967295); //@line 829 "compiledict.cpp"
      var $2=HEAP32[(($arrayidx)>>2)]; //@line 829 "compiledict.cpp"
      HEAP32[(($p)>>2)]=$2; //@line 829 "compiledict.cpp"
      var $3=$f_out_addr; //@line 830 "compiledict.cpp"
      var $call=_ftell($3); //@line 830 "compiledict.cpp"
      var $4=$hash; //@line 830 "compiledict.cpp"
      var $arrayidx2=((__ZL11hash_counts+$4*4)&4294967295); //@line 830 "compiledict.cpp"
      HEAP32[(($arrayidx2)>>2)]=$call; //@line 830 "compiledict.cpp"
      var $5=HEAP32[(($p)>>2)]; //@line 832 "compiledict.cpp"
      var $cmp31=((($5))|0)!=0; //@line 832 "compiledict.cpp"
      if ($cmp31) { __label__ = 2;; } else { __label__ = 3;; } //@line 832 "compiledict.cpp"
      $while_body$$while_end$4: do { 
        if (__label__ == 2) {
          while(1) { 
  
            var $6=HEAP32[(($p)>>2)]; //@line 834 "compiledict.cpp"
            var $add_ptr=(($6+4)&4294967295); //@line 834 "compiledict.cpp"
            var $7=HEAP8[($add_ptr)]; //@line 834 "compiledict.cpp"
            var $conv=(tempInt=(($7)),(tempInt>=128?tempInt-256:tempInt)); //@line 834 "compiledict.cpp"
            $length=$conv; //@line 834 "compiledict.cpp"
            var $8=HEAP32[(($p)>>2)]; //@line 835 "compiledict.cpp"
            var $add_ptr4=(($8+4)&4294967295); //@line 835 "compiledict.cpp"
            var $9=$length; //@line 835 "compiledict.cpp"
            var $10=$f_out_addr; //@line 835 "compiledict.cpp"
            var $call5=_fwrite($add_ptr4, $9, 1, $10); //@line 835 "compiledict.cpp"
            var $11=HEAP32[(($p)>>2)]; //@line 836 "compiledict.cpp"
            var $src$, $dest$, $stop$, $stop4$, $fast$;
            $src$ = $11;
            $dest$ = $0;
            $stop$ = $src$ + 4;
            if (($dest$%4) == ($src$%4) && 4 > 8) {
              while ($src$%4 !== 0 && $src$ < $stop$) {
                ; HEAP8[$dest$++] = HEAP8[$src$++];
              }
              $src$ >>= 2;
              $dest$ >>= 2;
              $stop4$ = $stop$ >> 2;
              while ($src$ < $stop4$) {
            ;;;    HEAP32[$dest$++] = HEAP32[$src$++];
              }
              $src$ <<= 2;
              $dest$ <<= 2;
            }while ($src$ < $stop$) {
              ; HEAP8[$dest$++] = HEAP8[$src$++];
            }; //@line 836 "compiledict.cpp"
            var $12=HEAP32[(($p)>>2)]; //@line 832 "compiledict.cpp"
            var $cmp3=((($12))|0)!=0; //@line 832 "compiledict.cpp"
            if ($cmp3) { __label__ = 2;continue ; } else { __label__ = 3;break $while_body$$while_end$4; } //@line 832 "compiledict.cpp"
          }
        }
      } while(0);
  
      var $13=$f_out_addr; //@line 838 "compiledict.cpp"
      var $call6=_fputc(0, $13); //@line 838 "compiledict.cpp"
      var $14=$hash; //@line 827 "compiledict.cpp"
      var $inc=((($14)+1)&4294967295); //@line 827 "compiledict.cpp"
      $hash=$inc; //@line 827 "compiledict.cpp"
      var $cmp1=((($inc))|0) < 1024; //@line 827 "compiledict.cpp"
      if ($cmp1) { __label__ = 1;continue $for_body$2; } else { __label__ = 4;break $for_body$2; } //@line 827 "compiledict.cpp"
    }
  
    STACKTOP = __stackBase__;
    return; //@line 840 "compiledict.cpp"
    return;
  }
  

  function __ZL17compile_dictrulesP7__sFILES0_Pc($f_in, $f_out, $fname_temp) {
    var __stackBase__  = STACKTOP; STACKTOP += 11876; _memset(__stackBase__, 0, 11876);
    var __label__;
    var __lastLabel__ = null;
  
    var $retval;
    var $f_in_addr;
    var $f_out_addr;
    var $fname_temp_addr;
    var $prule;
    var $p;
    var $ix;
    var $c=__stackBase__;
    var $gp;
    var $f_temp;
    var $n_rules=__stackBase__+4;
    var $count;
    var $different;
    var $wc=__stackBase__+8;
    var $prev_rgroup_name;
    var $char_code=__stackBase__+12;
    var $compile_mode;
    var $buf;
    var $buf1=__stackBase__+16;
    var $rules=__stackBase__+516;
    var $n_rgroups;
    var $n_groups3;
    var $rgroup=__stackBase__+8516;
    var $replace1;
    var $replace2;
    var $p123;
    $f_in_addr=$f_in;
    $f_out_addr=$f_out;
    $fname_temp_addr=$fname_temp;
    HEAP32[(($n_rules)>>2)]=0; //@line 1623 "compiledict.cpp"
    $count=0; //@line 1624 "compiledict.cpp"
    $compile_mode=0; //@line 1629 "compiledict.cpp"
    $n_rgroups=0; //@line 1634 "compiledict.cpp"
    $n_groups3=0; //@line 1635 "compiledict.cpp"
    HEAP32[((__ZL7linenum)>>2)]=0; //@line 1638 "compiledict.cpp"
    HEAP8[(((__ZL10group_name)&4294967295))]=0; //@line 1639 "compiledict.cpp"
    var $0=$fname_temp_addr; //@line 1641 "compiledict.cpp"
    var $call=__ZL9fopen_logPKcS0_($0, ((__str79)&4294967295)); //@line 1641 "compiledict.cpp"
    $f_temp=$call; //@line 1641 "compiledict.cpp"
    var $cmp=((($call))|0)==0; //@line 1641 "compiledict.cpp"
    if ($cmp) { __label__ = 2;; } else { __label__ = 1;; } //@line 1641 "compiledict.cpp"
    if (__label__ == 2) {
  
      $retval=1; //@line 1642 "compiledict.cpp"
      ; //@line 1642 "compiledict.cpp"
    }
    else if (__label__ == 1) {
  
      var $arraydecay=(($buf1)&4294967295); //@line 1647 "compiledict.cpp"
      var $arraydecay25=(($rules)&4294967295); //@line 1665 "compiledict.cpp"
      var $1=$n_rules; //@line 1676 "compiledict.cpp"
      ; //@line 1646 "compiledict.cpp"
      $for_cond$5: while(1) { 
  
        var $2=HEAP32[((__ZL7linenum)>>2)]; //@line 1646 "compiledict.cpp"
        var $inc=((($2)+1)&4294967295); //@line 1646 "compiledict.cpp"
        HEAP32[((__ZL7linenum)>>2)]=$inc; //@line 1646 "compiledict.cpp"
        var $3=$f_in_addr; //@line 1647 "compiledict.cpp"
        var $call1=_fgets($arraydecay, 500, $3); //@line 1647 "compiledict.cpp"
        $buf=$call1; //@line 1647 "compiledict.cpp"
        var $cmp2=((($call1))|0)!=0; //@line 1648 "compiledict.cpp"
        if ($cmp2) { __label__ = 4;; } else { __label__ = 10;; } //@line 1648 "compiledict.cpp"
        $if_then3$$if_then16$7: do { 
          if (__label__ == 4) {
  
            var $4=$buf; //@line 1650 "compiledict.cpp"
            var $call4=_strstr($4, ((__str82734)&4294967295)); //@line 1650 "compiledict.cpp"
            $p=$call4; //@line 1650 "compiledict.cpp"
            var $cmp5=((($call4))|0)!=0; //@line 1650 "compiledict.cpp"
            if ($cmp5) { __label__ = 5;; } else { __label__ = 6;; } //@line 1650 "compiledict.cpp"
            if (__label__ == 5) {
  
              var $5=$p; //@line 1651 "compiledict.cpp"
              HEAP8[($5)]=0; //@line 1651 "compiledict.cpp"
              ; //@line 1651 "compiledict.cpp"
            }
  
            var $6=$buf; //@line 1653 "compiledict.cpp"
            var $arrayidx=(($6)&4294967295); //@line 1653 "compiledict.cpp"
            var $7=HEAP8[($arrayidx)]; //@line 1653 "compiledict.cpp"
            var $conv=(tempInt=(($7)),(tempInt>=128?tempInt-256:tempInt)); //@line 1653 "compiledict.cpp"
            var $cmp8=((($conv))|0)==13; //@line 1653 "compiledict.cpp"
            if ($cmp8) { __lastLabel__ = 6; __label__ = 7;; } else { __lastLabel__ = 6; __label__ = 8;; } //@line 1653 "compiledict.cpp"
            if (__label__ == 7) {
  
              var $8=$buf; //@line 1653 "compiledict.cpp"
              var $incdec_ptr=(($8+1)&4294967295); //@line 1653 "compiledict.cpp"
              $buf=$incdec_ptr; //@line 1653 "compiledict.cpp"
              __lastLabel__ = 7; ; //@line 1653 "compiledict.cpp"
            }
  
            var $9=__lastLabel__ == 6 ? $6 : ($incdec_ptr); //@line 1656 "compiledict.cpp"
            var $cmp12=((($9))|0)==0; //@line 1656 "compiledict.cpp"
            if ($cmp12) { __label__ = 10;break $if_then3$$if_then16$7; } //@line 1656 "compiledict.cpp"
  
            var $10=$buf; //@line 1656 "compiledict.cpp"
            var $arrayidx13=(($10)&4294967295); //@line 1656 "compiledict.cpp"
            var $11=HEAP8[($arrayidx13)]; //@line 1656 "compiledict.cpp"
            var $conv14=(tempInt=(($11)),(tempInt>=128?tempInt-256:tempInt)); //@line 1656 "compiledict.cpp"
            var $cmp15=((($conv14))|0)==46; //@line 1656 "compiledict.cpp"
            if ($cmp15) { __label__ = 10;break $if_then3$$if_then16$7; } //@line 1656 "compiledict.cpp"
  
            var $69=$compile_mode; //@line 1750 "compiledict.cpp"
            if ($69 == 1) {
              __label__ = 44;;
            }
            else if ($69 == 2) {
              __label__ = 47;;
            }
            else {
            __label__ = 3;continue $for_cond$5;
            }
            
            if (__label__ == 44) {
  
              var $70=$buf; //@line 1753 "compiledict.cpp"
              var $call114=__ZL12compile_rulePc($70); //@line 1753 "compiledict.cpp"
              $prule=$call114; //@line 1753 "compiledict.cpp"
              var $71=$prule; //@line 1754 "compiledict.cpp"
              var $cmp115=((($71))|0)!=0; //@line 1754 "compiledict.cpp"
              if (!($cmp115)) { __label__ = 3;continue $for_cond$5; } //@line 1754 "compiledict.cpp"
  
              var $72=HEAP32[(($n_rules)>>2)]; //@line 1754 "compiledict.cpp"
              var $cmp117=((($72))|0) < 2000; //@line 1754 "compiledict.cpp"
              if (!($cmp117)) { __label__ = 3;continue $for_cond$5; } //@line 1754 "compiledict.cpp"
  
              var $73=$prule; //@line 1756 "compiledict.cpp"
              var $74=HEAP32[(($n_rules)>>2)]; //@line 1756 "compiledict.cpp"
              var $inc119=((($74)+1)&4294967295); //@line 1756 "compiledict.cpp"
              HEAP32[(($n_rules)>>2)]=$inc119; //@line 1756 "compiledict.cpp"
              var $arrayidx120=(($rules+$74*4)&4294967295); //@line 1756 "compiledict.cpp"
              HEAP32[(($arrayidx120)>>2)]=$73; //@line 1756 "compiledict.cpp"
              __label__ = 3;continue $for_cond$5; //@line 1757 "compiledict.cpp"
            }
            else if (__label__ == 47) {
  
              var $75=$buf; //@line 1766 "compiledict.cpp"
              $p123=$75; //@line 1766 "compiledict.cpp"
              $replace1=0; //@line 1767 "compiledict.cpp"
              $replace2=0; //@line 1768 "compiledict.cpp"
              var $76=$p123; //@line 1769 "compiledict.cpp"
              var $77=HEAP8[($76)]; //@line 1769 "compiledict.cpp"
              var $conv1254=(tempInt=(($77)),(tempInt>=128?tempInt-256:tempInt)); //@line 1769 "compiledict.cpp"
              var $call1265=__Z8isspace2j($conv1254); //@line 1769 "compiledict.cpp"
              var $tobool6=((($call1265))|0)!=0; //@line 1769 "compiledict.cpp"
              if ($tobool6) { __label__ = 48;; } else { __label__ = 49;; } //@line 1769 "compiledict.cpp"
              $while_body127$$while_end129$22: do { 
                if (__label__ == 48) {
                  while(1) { 
  
                    var $78=$p123; //@line 1769 "compiledict.cpp"
                    var $incdec_ptr128=(($78+1)&4294967295); //@line 1769 "compiledict.cpp"
                    $p123=$incdec_ptr128; //@line 1769 "compiledict.cpp"
                    var $79=$p123; //@line 1769 "compiledict.cpp"
                    var $80=HEAP8[($79)]; //@line 1769 "compiledict.cpp"
                    var $conv125=(tempInt=(($80)),(tempInt>=128?tempInt-256:tempInt)); //@line 1769 "compiledict.cpp"
                    var $call126=__Z8isspace2j($conv125); //@line 1769 "compiledict.cpp"
                    var $tobool=((($call126))|0)!=0; //@line 1769 "compiledict.cpp"
                    if ($tobool) { __label__ = 48;continue ; } else { __label__ = 49;break $while_body127$$while_end129$22; } //@line 1769 "compiledict.cpp"
                  }
                }
              } while(0);
  
              $ix=0; //@line 1770 "compiledict.cpp"
              var $81=$p123; //@line 1771 "compiledict.cpp"
              var $82=HEAPU8[($81)]; //@line 1771 "compiledict.cpp"
              var $conv1317=((($82))&255); //@line 1771 "compiledict.cpp"
              var $cmp1328=((($conv1317))|0) > 32; //@line 1771 "compiledict.cpp"
              if ($cmp1328) { __label__ = 50;; } else { __label__ = 51;; } //@line 1771 "compiledict.cpp"
              $while_body133$$while_cond138_loopexit$26: do { 
                if (__label__ == 50) {
                  while(1) { 
  
                    var $83=$p123; //@line 1773 "compiledict.cpp"
                    var $call134=__Z7utf8_inPiPKc($c, $83); //@line 1773 "compiledict.cpp"
                    var $84=$p123; //@line 1773 "compiledict.cpp"
                    var $add_ptr=(($84+$call134)&4294967295); //@line 1773 "compiledict.cpp"
                    $p123=$add_ptr; //@line 1773 "compiledict.cpp"
                    var $85=HEAP32[(($c)>>2)]; //@line 1774 "compiledict.cpp"
                    var $86=$ix; //@line 1774 "compiledict.cpp"
                    var $shl=($85) << ($86); //@line 1774 "compiledict.cpp"
                    var $87=$replace1; //@line 1774 "compiledict.cpp"
                    var $add135=((($87)+($shl))&4294967295); //@line 1774 "compiledict.cpp"
                    $replace1=$add135; //@line 1774 "compiledict.cpp"
                    var $88=$ix; //@line 1775 "compiledict.cpp"
                    var $add136=((($88)+16)&4294967295); //@line 1775 "compiledict.cpp"
                    $ix=$add136; //@line 1775 "compiledict.cpp"
                    var $89=$p123; //@line 1771 "compiledict.cpp"
                    var $90=HEAPU8[($89)]; //@line 1771 "compiledict.cpp"
                    var $conv131=((($90))&255); //@line 1771 "compiledict.cpp"
                    var $cmp132=((($conv131))|0) > 32; //@line 1771 "compiledict.cpp"
                    if ($cmp132) { __label__ = 50;continue ; } else { __label__ = 51;break $while_body133$$while_cond138_loopexit$26; } //@line 1771 "compiledict.cpp"
                  }
                }
              } while(0);
  
              var $91=$p123; //@line 1777 "compiledict.cpp"
              var $92=HEAP8[($91)]; //@line 1777 "compiledict.cpp"
              var $conv1399=(tempInt=(($92)),(tempInt>=128?tempInt-256:tempInt)); //@line 1777 "compiledict.cpp"
              var $call14010=__Z8isspace2j($conv1399); //@line 1777 "compiledict.cpp"
              var $tobool14111=((($call14010))|0)!=0; //@line 1777 "compiledict.cpp"
              if ($tobool14111) { __label__ = 52;; } else { __label__ = 53;; } //@line 1777 "compiledict.cpp"
              $while_body142$$while_end144$30: do { 
                if (__label__ == 52) {
                  while(1) { 
  
                    var $93=$p123; //@line 1777 "compiledict.cpp"
                    var $incdec_ptr143=(($93+1)&4294967295); //@line 1777 "compiledict.cpp"
                    $p123=$incdec_ptr143; //@line 1777 "compiledict.cpp"
                    var $94=$p123; //@line 1777 "compiledict.cpp"
                    var $95=HEAP8[($94)]; //@line 1777 "compiledict.cpp"
                    var $conv139=(tempInt=(($95)),(tempInt>=128?tempInt-256:tempInt)); //@line 1777 "compiledict.cpp"
                    var $call140=__Z8isspace2j($conv139); //@line 1777 "compiledict.cpp"
                    var $tobool141=((($call140))|0)!=0; //@line 1777 "compiledict.cpp"
                    if ($tobool141) { __label__ = 52;continue ; } else { __label__ = 53;break $while_body142$$while_end144$30; } //@line 1777 "compiledict.cpp"
                  }
                }
              } while(0);
  
              $ix=0; //@line 1778 "compiledict.cpp"
              var $96=$p123; //@line 1779 "compiledict.cpp"
              var $97=HEAPU8[($96)]; //@line 1779 "compiledict.cpp"
              var $conv14612=((($97))&255); //@line 1779 "compiledict.cpp"
              var $cmp14713=((($conv14612))|0) > 32; //@line 1779 "compiledict.cpp"
              if ($cmp14713) { __label__ = 54;; } else { __label__ = 55;; } //@line 1779 "compiledict.cpp"
              $while_body148$$while_end154$34: do { 
                if (__label__ == 54) {
                  while(1) { 
  
                    var $98=$p123; //@line 1781 "compiledict.cpp"
                    var $call149=__Z7utf8_inPiPKc($c, $98); //@line 1781 "compiledict.cpp"
                    var $99=$p123; //@line 1781 "compiledict.cpp"
                    var $add_ptr150=(($99+$call149)&4294967295); //@line 1781 "compiledict.cpp"
                    $p123=$add_ptr150; //@line 1781 "compiledict.cpp"
                    var $100=HEAP32[(($c)>>2)]; //@line 1782 "compiledict.cpp"
                    var $101=$ix; //@line 1782 "compiledict.cpp"
                    var $shl151=($100) << ($101); //@line 1782 "compiledict.cpp"
                    var $102=$replace2; //@line 1782 "compiledict.cpp"
                    var $add152=((($102)+($shl151))&4294967295); //@line 1782 "compiledict.cpp"
                    $replace2=$add152; //@line 1782 "compiledict.cpp"
                    var $103=$ix; //@line 1783 "compiledict.cpp"
                    var $add153=((($103)+16)&4294967295); //@line 1783 "compiledict.cpp"
                    $ix=$add153; //@line 1783 "compiledict.cpp"
                    var $104=$p123; //@line 1779 "compiledict.cpp"
                    var $105=HEAPU8[($104)]; //@line 1779 "compiledict.cpp"
                    var $conv146=((($105))&255); //@line 1779 "compiledict.cpp"
                    var $cmp147=((($conv146))|0) > 32; //@line 1779 "compiledict.cpp"
                    if ($cmp147) { __label__ = 54;continue ; } else { __label__ = 55;break $while_body148$$while_end154$34; } //@line 1779 "compiledict.cpp"
                  }
                }
              } while(0);
  
              var $106=$replace1; //@line 1785 "compiledict.cpp"
              var $cmp155=((($106))|0)!=0; //@line 1785 "compiledict.cpp"
              if (!($cmp155)) { __label__ = 3;continue $for_cond$5; } //@line 1785 "compiledict.cpp"
  
              var $107=$f_out_addr; //@line 1787 "compiledict.cpp"
              var $108=$replace1; //@line 1787 "compiledict.cpp"
              __Z11Write4BytesP7__sFILEi($107, $108); //@line 1787 "compiledict.cpp"
              var $109=$f_out_addr; //@line 1788 "compiledict.cpp"
              var $110=$replace2; //@line 1788 "compiledict.cpp"
              __Z11Write4BytesP7__sFILEi($109, $110); //@line 1788 "compiledict.cpp"
              __label__ = 3;continue $for_cond$5; //@line 1789 "compiledict.cpp"
            }
          }
        } while(0);
  
        var $12=HEAP32[(($n_rules)>>2)]; //@line 1660 "compiledict.cpp"
        var $cmp17=((($12))|0) > 0; //@line 1660 "compiledict.cpp"
        if ($cmp17) { __label__ = 11;; } else { __label__ = 12;; } //@line 1660 "compiledict.cpp"
        if (__label__ == 11) {
  
          var $13=$n_rgroups; //@line 1662 "compiledict.cpp"
          var $arrayidx19=(($rgroup+$13*28)&4294967295); //@line 1662 "compiledict.cpp"
          var $name=(($arrayidx19)&4294967295); //@line 1662 "compiledict.cpp"
          var $arraydecay20=(($name)&4294967295); //@line 1662 "compiledict.cpp"
          var $call21=_strcpy($arraydecay20, ((__ZL10group_name)&4294967295)); //@line 1662 "compiledict.cpp"
          var $14=HEAP32[((__ZL9group3_ix)>>2)]; //@line 1663 "compiledict.cpp"
          var $15=$n_rgroups; //@line 1663 "compiledict.cpp"
          var $arrayidx22=(($rgroup+$15*28)&4294967295); //@line 1663 "compiledict.cpp"
          var $group3_ix=(($arrayidx22+24)&4294967295); //@line 1663 "compiledict.cpp"
          HEAP32[(($group3_ix)>>2)]=$14; //@line 1663 "compiledict.cpp"
          var $16=$f_temp; //@line 1664 "compiledict.cpp"
          var $call23=_ftell($16); //@line 1664 "compiledict.cpp"
          var $17=$n_rgroups; //@line 1664 "compiledict.cpp"
          var $arrayidx24=(($rgroup+$17*28)&4294967295); //@line 1664 "compiledict.cpp"
          var $start=(($arrayidx24+16)&4294967295); //@line 1664 "compiledict.cpp"
          HEAP32[(($start)>>2)]=$call23; //@line 1664 "compiledict.cpp"
          var $18=$f_temp; //@line 1665 "compiledict.cpp"
          var $19=HEAP32[(($n_rules)>>2)]; //@line 1665 "compiledict.cpp"
          __ZL17output_rule_groupP7__sFILEiPPcS1_($18, $19, $arraydecay25); //@line 1665 "compiledict.cpp"
          var $20=$f_temp; //@line 1666 "compiledict.cpp"
          var $call26=_ftell($20); //@line 1666 "compiledict.cpp"
          var $21=$n_rgroups; //@line 1666 "compiledict.cpp"
          var $arrayidx27=(($rgroup+$21*28)&4294967295); //@line 1666 "compiledict.cpp"
          var $start28=(($arrayidx27+16)&4294967295); //@line 1666 "compiledict.cpp"
          var $22=HEAP32[(($start28)>>2)]; //@line 1666 "compiledict.cpp"
          var $sub=((($call26)-($22))&4294967295); //@line 1666 "compiledict.cpp"
          var $23=$n_rgroups; //@line 1666 "compiledict.cpp"
          var $arrayidx29=(($rgroup+$23*28)&4294967295); //@line 1666 "compiledict.cpp"
          var $length=(($arrayidx29+20)&4294967295); //@line 1666 "compiledict.cpp"
          HEAP32[(($length)>>2)]=$sub; //@line 1666 "compiledict.cpp"
          var $24=$n_rgroups; //@line 1667 "compiledict.cpp"
          var $inc30=((($24)+1)&4294967295); //@line 1667 "compiledict.cpp"
          $n_rgroups=$inc30; //@line 1667 "compiledict.cpp"
          var $25=HEAP32[(($n_rules)>>2)]; //@line 1669 "compiledict.cpp"
          var $26=$count; //@line 1669 "compiledict.cpp"
          var $add=((($26)+($25))&4294967295); //@line 1669 "compiledict.cpp"
          $count=$add; //@line 1669 "compiledict.cpp"
          ; //@line 1670 "compiledict.cpp"
        }
  
        HEAP32[(($n_rules)>>2)]=0; //@line 1671 "compiledict.cpp"
        var $27=$compile_mode; //@line 1673 "compiledict.cpp"
        var $cmp32=((($27))|0)==2; //@line 1673 "compiledict.cpp"
        if ($cmp32) { __label__ = 13;; } else { __label__ = 14;; } //@line 1673 "compiledict.cpp"
        if (__label__ == 13) {
  
          var $28=$f_out_addr; //@line 1676 "compiledict.cpp"
          var $call34=_fwrite($1, 1, 4, $28); //@line 1676 "compiledict.cpp"
          $compile_mode=0; //@line 1677 "compiledict.cpp"
          ; //@line 1678 "compiledict.cpp"
        }
  
        var $29=$buf; //@line 1680 "compiledict.cpp"
        var $cmp36=((($29))|0)==0; //@line 1680 "compiledict.cpp"
        if ($cmp36) { __label__ = 57;break $for_cond$5; } //@line 1680 "compiledict.cpp"
  
        var $30=$buf; //@line 1682 "compiledict.cpp"
        var $call39=_memcmp($30, ((__str81)&4294967295), 2); //@line 1682 "compiledict.cpp"
        var $cmp40=((($call39))|0)==0; //@line 1682 "compiledict.cpp"
        var $31=$buf; //@line 1684 "compiledict.cpp"
        if ($cmp40) { __label__ = 16;; } else { __label__ = 17;; } //@line 1682 "compiledict.cpp"
        if (__label__ == 16) {
  
          var $arrayidx42=(($31+2)&4294967295); //@line 1684 "compiledict.cpp"
          var $32=$f_out_addr; //@line 1684 "compiledict.cpp"
          __ZL19compile_lettergroupPcP7__sFILE($arrayidx42, $32); //@line 1684 "compiledict.cpp"
          __label__ = 3;continue $for_cond$5; //@line 1685 "compiledict.cpp"
        }
        else if (__label__ == 17) {
  
          var $call45=_memcmp($31, ((__str82)&4294967295), 8); //@line 1688 "compiledict.cpp"
          var $cmp46=((($call45))|0)==0; //@line 1688 "compiledict.cpp"
          if ($cmp46) { __label__ = 18;; } else { __label__ = 20;; } //@line 1688 "compiledict.cpp"
          $if_then47$$if_end53$50: do { 
            if (__label__ == 18) {
  
              $compile_mode=2; //@line 1690 "compiledict.cpp"
              var $33=$f_out_addr; //@line 1691 "compiledict.cpp"
              var $call48=_fputc(6, $33); //@line 1691 "compiledict.cpp"
              var $34=$f_out_addr; //@line 1692 "compiledict.cpp"
              var $call49=_fputc(20, $34); //@line 1692 "compiledict.cpp"
              var $35=$f_out_addr; //@line 1695 "compiledict.cpp"
              var $call5014=_ftell($35); //@line 1695 "compiledict.cpp"
              var $and15=($call5014) & 3; //@line 1695 "compiledict.cpp"
              var $cmp5116=((($and15))|0)!=0; //@line 1695 "compiledict.cpp"
              if (!($cmp5116)) { __label__ = 20;break $if_then47$$if_end53$50; } //@line 1695 "compiledict.cpp"
              while(1) { 
  
                var $36=$f_out_addr; //@line 1696 "compiledict.cpp"
                var $call52=_fputc(0, $36); //@line 1696 "compiledict.cpp"
                var $37=$f_out_addr; //@line 1695 "compiledict.cpp"
                var $call50=_ftell($37); //@line 1695 "compiledict.cpp"
                var $and=($call50) & 3; //@line 1695 "compiledict.cpp"
                var $cmp51=((($and))|0)!=0; //@line 1695 "compiledict.cpp"
                if ($cmp51) { __label__ = 19;continue ; } else { __label__ = 20;break $if_then47$$if_end53$50; } //@line 1695 "compiledict.cpp"
              }
            }
          } while(0);
  
          var $38=$buf; //@line 1699 "compiledict.cpp"
          var $call54=_memcmp($38, ((__str83)&4294967295), 6); //@line 1699 "compiledict.cpp"
          var $cmp55=((($call54))|0)==0; //@line 1699 "compiledict.cpp"
          if (!($cmp55)) { __label__ = 3;continue $for_cond$5; } //@line 1699 "compiledict.cpp"
  
          $compile_mode=1; //@line 1701 "compiledict.cpp"
          var $39=$buf; //@line 1703 "compiledict.cpp"
          var $arrayidx57=(($39+6)&4294967295); //@line 1703 "compiledict.cpp"
          $p=$arrayidx57; //@line 1703 "compiledict.cpp"
          ; //@line 1704 "compiledict.cpp"
          $while_cond58$56: while(1) { 
  
            var $40=$p; //@line 1704 "compiledict.cpp"
            var $arrayidx59=(($40)&4294967295); //@line 1704 "compiledict.cpp"
            var $41=HEAPU8[($arrayidx59)]; //@line 1704 "compiledict.cpp"
            var $conv60=((($41))&255); //@line 1704 "compiledict.cpp"
            var $cmp61=((($conv60))|0)==32; //@line 1704 "compiledict.cpp"
            if ($cmp61) { __label__ = 24;; } else { __label__ = 23;; } //@line 1704 "compiledict.cpp"
            if (__label__ == 23) {
  
              var $42=$p; //@line 1704 "compiledict.cpp"
              var $arrayidx62=(($42)&4294967295); //@line 1704 "compiledict.cpp"
              var $43=HEAPU8[($arrayidx62)]; //@line 1704 "compiledict.cpp"
              var $conv63=((($43))&255); //@line 1704 "compiledict.cpp"
              var $cmp64=((($conv63))|0)==9; //@line 1704 "compiledict.cpp"
              if (!($cmp64)) { __label__ = 25;break $while_cond58$56; } //@line 1704 "compiledict.cpp"
            }
  
            var $44=$p; //@line 1704 "compiledict.cpp"
            var $incdec_ptr66=(($44+1)&4294967295); //@line 1704 "compiledict.cpp"
            $p=$incdec_ptr66; //@line 1704 "compiledict.cpp"
            __label__ = 22;continue $while_cond58$56; //@line 1704 "compiledict.cpp"
          }
  
          $ix=0; //@line 1705 "compiledict.cpp"
          ; //@line 1706 "compiledict.cpp"
          $while_cond68$62: while(1) { 
  
            var $45=$p; //@line 1706 "compiledict.cpp"
            var $46=HEAPU8[($45)]; //@line 1706 "compiledict.cpp"
            var $conv69=((($46))&255); //@line 1706 "compiledict.cpp"
            var $cmp70=((($conv69))|0) > 32; //@line 1706 "compiledict.cpp"
            if (!($cmp70)) { __label__ = 29;break $while_cond68$62; } //@line 1706 "compiledict.cpp"
  
            var $47=$ix; //@line 1706 "compiledict.cpp"
            var $cmp71=((($47))|0) < 12; //@line 1706 "compiledict.cpp"
            if (!($cmp71)) { __label__ = 29;break $while_cond68$62; }
  
            var $48=$p; //@line 1707 "compiledict.cpp"
            var $incdec_ptr73=(($48+1)&4294967295); //@line 1707 "compiledict.cpp"
            $p=$incdec_ptr73; //@line 1707 "compiledict.cpp"
            var $49=HEAP8[($48)]; //@line 1707 "compiledict.cpp"
            var $50=$ix; //@line 1707 "compiledict.cpp"
            var $inc74=((($50)+1)&4294967295); //@line 1707 "compiledict.cpp"
            $ix=$inc74; //@line 1707 "compiledict.cpp"
            var $arrayidx75=((__ZL10group_name+$50)&4294967295); //@line 1707 "compiledict.cpp"
            HEAP8[($arrayidx75)]=$49; //@line 1707 "compiledict.cpp"
            __label__ = 26;continue $while_cond68$62; //@line 1707 "compiledict.cpp"
          }
  
          var $51=$ix; //@line 1708 "compiledict.cpp"
          var $arrayidx77=((__ZL10group_name+$51)&4294967295); //@line 1708 "compiledict.cpp"
          HEAP8[($arrayidx77)]=0; //@line 1708 "compiledict.cpp"
          HEAP32[((__ZL9group3_ix)>>2)]=0; //@line 1709 "compiledict.cpp"
          var $call78=_sscanf(((__ZL10group_name)&4294967295), ((__str84)&4294967295), allocate([$char_code,0,0,0], ["i32*",0,0,0], ALLOC_STACK)); //@line 1711 "compiledict.cpp"
          var $cmp79=((($call78))|0)==1; //@line 1711 "compiledict.cpp"
          if ($cmp79) { __label__ = 30;; } else { __label__ = 33;; } //@line 1711 "compiledict.cpp"
          $if_then80$$if_else$67: do { 
            if (__label__ == 30) {
  
              $p=((__ZL10group_name)&4294967295); //@line 1714 "compiledict.cpp"
              var $52=HEAPU32[(($char_code)>>2)]; //@line 1716 "compiledict.cpp"
              var $cmp81=((($52))>>>0) > 256; //@line 1716 "compiledict.cpp"
              if ($cmp81) { __label__ = 31;; } else { __label__ = 32;; } //@line 1716 "compiledict.cpp"
              if (__label__ == 31) {
  
                var $53=HEAPU32[(($char_code)>>2)]; //@line 1718 "compiledict.cpp"
                var $shr=($53) >>> 8; //@line 1718 "compiledict.cpp"
                var $conv83=((($shr)) & 255); //@line 1718 "compiledict.cpp"
                var $54=$p; //@line 1718 "compiledict.cpp"
                var $incdec_ptr84=(($54+1)&4294967295); //@line 1718 "compiledict.cpp"
                $p=$incdec_ptr84; //@line 1718 "compiledict.cpp"
                HEAP8[($54)]=$conv83; //@line 1718 "compiledict.cpp"
                ; //@line 1719 "compiledict.cpp"
              }
  
              var $55=HEAP32[(($char_code)>>2)]; //@line 1720 "compiledict.cpp"
              var $conv86=((($55)) & 255); //@line 1720 "compiledict.cpp"
              var $56=$p; //@line 1720 "compiledict.cpp"
              var $incdec_ptr87=(($56+1)&4294967295); //@line 1720 "compiledict.cpp"
              $p=$incdec_ptr87; //@line 1720 "compiledict.cpp"
              HEAP8[($56)]=$conv86; //@line 1720 "compiledict.cpp"
              var $57=$p; //@line 1721 "compiledict.cpp"
              HEAP8[($57)]=0; //@line 1721 "compiledict.cpp"
              __label__ = 37;break $if_then80$$if_else$67; //@line 1722 "compiledict.cpp"
            }
            else if (__label__ == 33) {
  
              var $58=HEAP32[((_translator)>>2)]; //@line 1725 "compiledict.cpp"
              var $letter_bits_offset=(($58+964)&4294967295); //@line 1725 "compiledict.cpp"
              var $59=HEAP32[(($letter_bits_offset)>>2)]; //@line 1725 "compiledict.cpp"
              var $cmp88=((($59))|0) > 0; //@line 1725 "compiledict.cpp"
              if (!($cmp88)) { __label__ = 37;break $if_then80$$if_else$67; } //@line 1725 "compiledict.cpp"
  
              var $call90=__Z7utf8_inPiPKc($wc, ((__ZL10group_name)&4294967295)); //@line 1727 "compiledict.cpp"
              var $60=HEAP32[(($wc)>>2)]; //@line 1728 "compiledict.cpp"
              var $61=HEAP32[((_translator)>>2)]; //@line 1728 "compiledict.cpp"
              var $letter_bits_offset91=(($61+964)&4294967295); //@line 1728 "compiledict.cpp"
              var $62=HEAP32[(($letter_bits_offset91)>>2)]; //@line 1728 "compiledict.cpp"
              var $sub92=((($60)-($62))&4294967295); //@line 1728 "compiledict.cpp"
              $ix=$sub92; //@line 1728 "compiledict.cpp"
              var $cmp93=((($sub92))|0) >= 0; //@line 1728 "compiledict.cpp"
              if (!($cmp93)) { __label__ = 37;break $if_then80$$if_else$67; } //@line 1728 "compiledict.cpp"
  
              var $63=$ix; //@line 1728 "compiledict.cpp"
              var $cmp94=((($63))|0) < 128; //@line 1728 "compiledict.cpp"
              if (!($cmp94)) { __label__ = 37;break $if_then80$$if_else$67; } //@line 1728 "compiledict.cpp"
  
              var $64=$ix; //@line 1730 "compiledict.cpp"
              var $add96=((($64)+1)&4294967295); //@line 1730 "compiledict.cpp"
              HEAP32[((__ZL9group3_ix)>>2)]=$add96; //@line 1730 "compiledict.cpp"
              __lastLabel__ = 36; __label__ = 38;break $if_then80$$if_else$67; //@line 1731 "compiledict.cpp"
            }
          } while(0);
          if (__label__ == 37) {
  
            var $_pr=HEAP32[((__ZL9group3_ix)>>2)]; //@line 1735 "compiledict.cpp"
            __lastLabel__ = 37; ;
          }
  
          var $65=__lastLabel__ == 37 ? $_pr : ($add96); //@line 1735 "compiledict.cpp"
          var $cmp100=((($65))|0)==0; //@line 1735 "compiledict.cpp"
          if (!($cmp100)) { __label__ = 3;continue $for_cond$5; } //@line 1735 "compiledict.cpp"
  
          var $call102=_strlen(((__ZL10group_name)&4294967295)); //@line 1735 "compiledict.cpp"
          var $cmp103=((($call102))>>>0) > 2; //@line 1735 "compiledict.cpp"
          if (!($cmp103)) { __label__ = 3;continue $for_cond$5; } //@line 1735 "compiledict.cpp"
  
          var $call105=__Z7utf8_inPiPKc($c, ((__ZL10group_name)&4294967295)); //@line 1737 "compiledict.cpp"
          var $cmp106=((($call105))|0) < 2; //@line 1737 "compiledict.cpp"
          if ($cmp106) { __label__ = 41;; } else { __label__ = 42;; } //@line 1737 "compiledict.cpp"
          if (__label__ == 41) {
  
            var $66=HEAP32[((__ZL5f_log)>>2)]; //@line 1739 "compiledict.cpp"
            var $67=HEAP32[((__ZL7linenum)>>2)]; //@line 1739 "compiledict.cpp"
            var $call108=_fprintf($66, ((__str85)&4294967295), allocate([$67,0,0,0], ["i32",0,0,0], ALLOC_STACK)); //@line 1739 "compiledict.cpp"
            var $68=HEAP32[((__ZL11error_count)>>2)]; //@line 1740 "compiledict.cpp"
            var $inc109=((($68)+1)&4294967295); //@line 1740 "compiledict.cpp"
            HEAP32[((__ZL11error_count)>>2)]=$inc109; //@line 1740 "compiledict.cpp"
            ; //@line 1741 "compiledict.cpp"
          }
  
          HEAP8[(((__ZL10group_name+2)&4294967295))]=0; //@line 1743 "compiledict.cpp"
          __label__ = 3;continue $for_cond$5; //@line 1744 "compiledict.cpp"
        }
      }
  
      var $111=$f_temp; //@line 1794 "compiledict.cpp"
      var $call158=_fclose($111); //@line 1794 "compiledict.cpp"
      var $arraydecay159=(($rgroup)&4294967295); //@line 1796 "compiledict.cpp"
      var $112=$arraydecay159; //@line 1796 "compiledict.cpp"
      var $113=$n_rgroups; //@line 1796 "compiledict.cpp"
      _qsort($112, $113, 28, 2); //@line 1796 "compiledict.cpp"
      var $114=$fname_temp_addr; //@line 1798 "compiledict.cpp"
      var $call160=_fopen($114, ((__str18429)&4294967295)); //@line 1798 "compiledict.cpp"
      $f_temp=$call160; //@line 1798 "compiledict.cpp"
      var $cmp161=((($call160))|0)==0; //@line 1798 "compiledict.cpp"
      if ($cmp161) { __label__ = 58;; } else { __label__ = 59;; } //@line 1798 "compiledict.cpp"
      if (__label__ == 58) {
  
        $retval=2; //@line 1799 "compiledict.cpp"
        ; //@line 1799 "compiledict.cpp"
      }
      else if (__label__ == 59) {
  
        $prev_rgroup_name=((__str15397)&4294967295); //@line 1801 "compiledict.cpp"
        $gp=0; //@line 1803 "compiledict.cpp"
        var $115=$gp; //@line 1803 "compiledict.cpp"
        var $116=$n_rgroups; //@line 1803 "compiledict.cpp"
        var $cmp1653=((($115))|0) < ((($116))|0); //@line 1803 "compiledict.cpp"
        if ($cmp1653) { __label__ = 60;; } else { __label__ = 70;; } //@line 1803 "compiledict.cpp"
        $for_body$$for_end210$88: do { 
          if (__label__ == 60) {
            $for_body$89: while(1) { 
  
              var $117=$f_temp; //@line 1805 "compiledict.cpp"
              var $118=$gp; //@line 1805 "compiledict.cpp"
              var $arrayidx166=(($rgroup+$118*28)&4294967295); //@line 1805 "compiledict.cpp"
              var $start167=(($arrayidx166+16)&4294967295); //@line 1805 "compiledict.cpp"
              var $119=HEAP32[(($start167)>>2)]; //@line 1805 "compiledict.cpp"
              var $call168=_fseek($117, $119, 0); //@line 1805 "compiledict.cpp"
              var $120=$gp; //@line 1807 "compiledict.cpp"
              var $arrayidx169=(($rgroup+$120*28)&4294967295); //@line 1807 "compiledict.cpp"
              var $name170=(($arrayidx169)&4294967295); //@line 1807 "compiledict.cpp"
              var $arraydecay171=(($name170)&4294967295); //@line 1807 "compiledict.cpp"
              var $121=$prev_rgroup_name; //@line 1807 "compiledict.cpp"
              var $call172=_strcmp($arraydecay171, $121); //@line 1807 "compiledict.cpp"
              $different=$call172; //@line 1807 "compiledict.cpp"
              var $cmp173=((($call172))|0)!=0; //@line 1807 "compiledict.cpp"
              if ($cmp173) { __label__ = 61;; } else { __label__ = 67;; } //@line 1807 "compiledict.cpp"
              if (__label__ == 61) {
  
                var $122=$gp; //@line 1810 "compiledict.cpp"
                var $cmp175=((($122))|0) > 0; //@line 1810 "compiledict.cpp"
                if ($cmp175) { __label__ = 62;; } else { __label__ = 63;; } //@line 1810 "compiledict.cpp"
                if (__label__ == 62) {
  
                  var $123=$f_out_addr; //@line 1811 "compiledict.cpp"
                  var $call177=_fputc(7, $123); //@line 1811 "compiledict.cpp"
                  ; //@line 1811 "compiledict.cpp"
                }
  
                var $124=$f_out_addr; //@line 1812 "compiledict.cpp"
                var $call179=_fputc(6, $124); //@line 1812 "compiledict.cpp"
                var $125=$gp; //@line 1814 "compiledict.cpp"
                var $arrayidx180=(($rgroup+$125*28)&4294967295); //@line 1814 "compiledict.cpp"
                var $group3_ix181=(($arrayidx180+24)&4294967295); //@line 1814 "compiledict.cpp"
                var $126=HEAP32[(($group3_ix181)>>2)]; //@line 1814 "compiledict.cpp"
                var $cmp182=((($126))|0)!=0; //@line 1814 "compiledict.cpp"
                if ($cmp182) { __label__ = 64;; } else { __label__ = 65;; } //@line 1814 "compiledict.cpp"
                if (__label__ == 64) {
  
                  var $127=$n_groups3; //@line 1816 "compiledict.cpp"
                  var $inc184=((($127)+1)&4294967295); //@line 1816 "compiledict.cpp"
                  $n_groups3=$inc184; //@line 1816 "compiledict.cpp"
                  var $128=$f_out_addr; //@line 1817 "compiledict.cpp"
                  var $call185=_fputc(1, $128); //@line 1817 "compiledict.cpp"
                  var $129=$gp; //@line 1818 "compiledict.cpp"
                  var $arrayidx186=(($rgroup+$129*28)&4294967295); //@line 1818 "compiledict.cpp"
                  var $group3_ix187=(($arrayidx186+24)&4294967295); //@line 1818 "compiledict.cpp"
                  var $130=HEAP32[(($group3_ix187)>>2)]; //@line 1818 "compiledict.cpp"
                  var $131=$f_out_addr; //@line 1818 "compiledict.cpp"
                  var $call188=_fputc($130, $131); //@line 1818 "compiledict.cpp"
                  ; //@line 1819 "compiledict.cpp"
                }
                else if (__label__ == 65) {
  
                  var $132=$f_out_addr; //@line 1822 "compiledict.cpp"
                  var $133=$gp; //@line 1822 "compiledict.cpp"
                  var $arrayidx190=(($rgroup+$133*28)&4294967295); //@line 1822 "compiledict.cpp"
                  var $name191=(($arrayidx190)&4294967295); //@line 1822 "compiledict.cpp"
                  var $arraydecay192=(($name191)&4294967295); //@line 1822 "compiledict.cpp"
                  $prev_rgroup_name=$arraydecay192; //@line 1822 "compiledict.cpp"
                  var $fputs=_fputs($arraydecay192, $132); //@line 1822 "compiledict.cpp"
                  ;
                }
  
                var $134=$f_out_addr; //@line 1824 "compiledict.cpp"
                var $call195=_fputc(0, $134); //@line 1824 "compiledict.cpp"
                ; //@line 1825 "compiledict.cpp"
              }
  
              var $135=$gp; //@line 1827 "compiledict.cpp"
              var $arrayidx197=(($rgroup+$135*28)&4294967295); //@line 1827 "compiledict.cpp"
              var $length198=(($arrayidx197+20)&4294967295); //@line 1827 "compiledict.cpp"
              var $136=HEAP32[(($length198)>>2)]; //@line 1827 "compiledict.cpp"
              $ix=$136; //@line 1827 "compiledict.cpp"
              var $cmp2002=((($136))|0) > 0; //@line 1827 "compiledict.cpp"
              if ($cmp2002) { __label__ = 68;; } else { __label__ = 69;; } //@line 1827 "compiledict.cpp"
              $for_body201$$for_end204$101: do { 
                if (__label__ == 68) {
                  while(1) { 
  
                    var $137=$f_temp; //@line 1829 "compiledict.cpp"
                    var $call202=_fgetc($137); //@line 1829 "compiledict.cpp"
                    HEAP32[(($c)>>2)]=$call202; //@line 1829 "compiledict.cpp"
                    var $138=HEAP32[(($c)>>2)]; //@line 1830 "compiledict.cpp"
                    var $139=$f_out_addr; //@line 1830 "compiledict.cpp"
                    var $call203=_fputc($138, $139); //@line 1830 "compiledict.cpp"
                    var $140=$ix; //@line 1827 "compiledict.cpp"
                    var $dec=((($140)-1)&4294967295); //@line 1827 "compiledict.cpp"
                    $ix=$dec; //@line 1827 "compiledict.cpp"
                    var $cmp200=((($dec))|0) > 0; //@line 1827 "compiledict.cpp"
                    if ($cmp200) { __label__ = 68;continue ; } else { __label__ = 69;break $for_body201$$for_end204$101; } //@line 1827 "compiledict.cpp"
                  }
                }
              } while(0);
  
              var $141=$gp; //@line 1803 "compiledict.cpp"
              var $inc209=((($141)+1)&4294967295); //@line 1803 "compiledict.cpp"
              $gp=$inc209; //@line 1803 "compiledict.cpp"
              var $142=$gp; //@line 1803 "compiledict.cpp"
              var $143=$n_rgroups; //@line 1803 "compiledict.cpp"
              var $cmp165=((($142))|0) < ((($143))|0); //@line 1803 "compiledict.cpp"
              if ($cmp165) { __label__ = 60;continue $for_body$89; } else { __label__ = 70;break $for_body$$for_end210$88; } //@line 1803 "compiledict.cpp"
            }
          }
        } while(0);
  
        var $144=$f_out_addr; //@line 1837 "compiledict.cpp"
        var $call211=_fputc(7, $144); //@line 1837 "compiledict.cpp"
        var $145=$f_out_addr; //@line 1838 "compiledict.cpp"
        var $call212=_fputc(0, $145); //@line 1838 "compiledict.cpp"
        var $146=$f_temp; //@line 1840 "compiledict.cpp"
        var $call213=_fclose($146); //@line 1840 "compiledict.cpp"
        var $147=$fname_temp_addr; //@line 1841 "compiledict.cpp"
        var $call214=_remove($147); //@line 1841 "compiledict.cpp"
        var $148=HEAP32[((__ZL5f_log)>>2)]; //@line 1843 "compiledict.cpp"
        var $149=$count; //@line 1843 "compiledict.cpp"
        var $150=$n_rgroups; //@line 1843 "compiledict.cpp"
        var $151=$n_groups3; //@line 1843 "compiledict.cpp"
        var $call215=_fprintf($148, ((__str89)&4294967295), allocate([$149,0,0,0,$150,0,0,0,$151,0,0,0], ["i32",0,0,0,"i32",0,0,0,"i32",0,0,0], ALLOC_STACK)); //@line 1843 "compiledict.cpp"
        $retval=0; //@line 1844 "compiledict.cpp"
        ; //@line 1844 "compiledict.cpp"
      }
    }
  
    STACKTOP = __stackBase__;
    return;
    return;
  }
  

  function __ZL17output_rule_groupP7__sFILEiPPcS1_($f_out, $n_rules, $rules) {
    var __stackBase__  = STACKTOP; STACKTOP += 512; _memset(__stackBase__, 0, 512);
    var __label__;
  
    var $f_out_addr;
    var $n_rules_addr;
    var $rules_addr;
    var $name_addr;
    var $ix;
    var $len1;
    var $len2;
    var $len_name;
    var $p;
    var $p2;
    var $p3;
    var $common;
    var $nextchar_count=__stackBase__;
    $f_out_addr=$f_out;
    $n_rules_addr=$n_rules;
    $rules_addr=$rules;
    $name_addr=((__ZL10group_name)&4294967295);
    var $arraydecay=(($nextchar_count)&4294967295); //@line 1480 "compiledict.cpp"
    var $0=$arraydecay; //@line 1480 "compiledict.cpp"
    var $dest$, $stop$, $stop4$, $fast$, $value4$;
    $dest$ = $0;
    $stop$ = $dest$ + 512;
    $value4$ = 0;
    if ($value4$ < 0) $value4$ += 256;
    $value4$ = $value4$ + ($value4$<<8) + ($value4$<<16) + ($value4$*16777216);
    while ($dest$%4 !== 0 && $dest$ < $stop$) {
      ;; HEAP8[$dest$++] = 0;
    }
    $dest$ >>= 2;
    $stop4$ = $stop$ >> 2;
    while ($dest$ < $stop4$) {
    ;;;;;;;  HEAP32[$dest$++] = $value4$;
    }
    $dest$ <<= 2;
    while ($dest$ < $stop$) {
      ;; HEAP8[$dest$++] = 0;
    }; //@line 1480 "compiledict.cpp"
    var $1=$name_addr; //@line 1482 "compiledict.cpp"
    var $call=_strlen($1); //@line 1482 "compiledict.cpp"
    $len_name=$call; //@line 1482 "compiledict.cpp"
    $common=((__str13643)&4294967295); //@line 1489 "compiledict.cpp"
    var $2=$rules_addr; //@line 1490 "compiledict.cpp"
    var $3=$2; //@line 1490 "compiledict.cpp"
    var $4=$n_rules_addr; //@line 1490 "compiledict.cpp"
    _qsort($3, $4, 4, 4); //@line 1490 "compiledict.cpp"
    var $5=$name_addr; //@line 1492 "compiledict.cpp"
    var $call1=_strcmp($5, ((__str103)&4294967295)); //@line 1492 "compiledict.cpp"
    var $cmp=((($call1))|0)==0; //@line 1492 "compiledict.cpp"
    if ($cmp) { __label__ = 1;; } else { __label__ = 2;; } //@line 1492 "compiledict.cpp"
    if (__label__ == 1) {
  
      $len_name=0; //@line 1493 "compiledict.cpp"
      ; //@line 1493 "compiledict.cpp"
    }
  
    $ix=0; //@line 1495 "compiledict.cpp"
    var $6=$ix; //@line 1495 "compiledict.cpp"
    var $7=$n_rules_addr; //@line 1495 "compiledict.cpp"
    var $cmp21=((($6))|0) < ((($7))|0); //@line 1495 "compiledict.cpp"
    if ($cmp21) { __label__ = 3;; } else { __label__ = 11;; } //@line 1495 "compiledict.cpp"
    $for_body$$for_end$5: do { 
      if (__label__ == 3) {
        $for_body$6: while(1) { 
  
          var $8=$ix; //@line 1497 "compiledict.cpp"
          var $9=$rules_addr; //@line 1497 "compiledict.cpp"
          var $arrayidx=(($9+4*$8)&4294967295); //@line 1497 "compiledict.cpp"
          var $10=HEAP32[(($arrayidx)>>2)]; //@line 1497 "compiledict.cpp"
          $p=$10; //@line 1497 "compiledict.cpp"
          var $11=$p; //@line 1498 "compiledict.cpp"
          var $call3=_strlen($11); //@line 1498 "compiledict.cpp"
          var $add=((($call3)+1)&4294967295); //@line 1498 "compiledict.cpp"
          $len1=$add; //@line 1498 "compiledict.cpp"
          var $12=$len1; //@line 1499 "compiledict.cpp"
          var $13=$p; //@line 1499 "compiledict.cpp"
          var $arrayidx4=(($13+$12)&4294967295); //@line 1499 "compiledict.cpp"
          $p3=$arrayidx4; //@line 1499 "compiledict.cpp"
          var $14=$p3; //@line 1500 "compiledict.cpp"
          var $15=$len_name; //@line 1500 "compiledict.cpp"
          var $add_ptr=(($14+$15)&4294967295); //@line 1500 "compiledict.cpp"
          $p2=$add_ptr; //@line 1500 "compiledict.cpp"
          var $16=$p2; //@line 1501 "compiledict.cpp"
          var $call5=_strlen($16); //@line 1501 "compiledict.cpp"
          $len2=$call5; //@line 1501 "compiledict.cpp"
          var $17=$p2; //@line 1503 "compiledict.cpp"
          var $arrayidx6=(($17)&4294967295); //@line 1503 "compiledict.cpp"
          var $18=HEAPU8[($arrayidx6)]; //@line 1503 "compiledict.cpp"
          var $idxprom=((($18))&255); //@line 1503 "compiledict.cpp"
          var $arrayidx7=(($nextchar_count+$idxprom*2)&4294967295); //@line 1503 "compiledict.cpp"
          var $19=HEAP16[(($arrayidx7)>>1)]; //@line 1503 "compiledict.cpp"
          var $inc=((($19)+1)&65535); //@line 1503 "compiledict.cpp"
          HEAP16[(($arrayidx7)>>1)]=$inc; //@line 1503 "compiledict.cpp"
          var $20=$common; //@line 1505 "compiledict.cpp"
          var $arrayidx8=(($20)&4294967295); //@line 1505 "compiledict.cpp"
          var $21=HEAP8[($arrayidx8)]; //@line 1505 "compiledict.cpp"
          var $conv=(tempInt=(($21)),(tempInt>=128?tempInt-256:tempInt)); //@line 1505 "compiledict.cpp"
          var $cmp9=((($conv))|0)!=0; //@line 1505 "compiledict.cpp"
          if ($cmp9) { __label__ = 4;; } else { __label__ = 6;; } //@line 1505 "compiledict.cpp"
          $land_lhs_true$$if_else$8: do { 
            if (__label__ == 4) {
  
              var $22=$p; //@line 1505 "compiledict.cpp"
              var $23=$common; //@line 1505 "compiledict.cpp"
              var $call10=_strcmp($22, $23); //@line 1505 "compiledict.cpp"
              var $cmp11=((($call10))|0)==0; //@line 1505 "compiledict.cpp"
              if (!($cmp11)) { __label__ = 6;break $land_lhs_true$$if_else$8; } //@line 1505 "compiledict.cpp"
  
              var $24=$p2; //@line 1507 "compiledict.cpp"
              var $25=$len2; //@line 1507 "compiledict.cpp"
              var $26=$f_out_addr; //@line 1507 "compiledict.cpp"
              var $call13=_fwrite($24, $25, 1, $26); //@line 1507 "compiledict.cpp"
              var $27=$f_out_addr; //@line 1508 "compiledict.cpp"
              var $call14=_fputc(0, $27); //@line 1508 "compiledict.cpp"
              __label__ = 10;break $land_lhs_true$$if_else$8; //@line 1509 "compiledict.cpp"
            }
          } while(0);
          if (__label__ == 6) {
  
            var $28=$ix; //@line 1512 "compiledict.cpp"
            var $29=$n_rules_addr; //@line 1512 "compiledict.cpp"
            var $sub=((($29)-1)&4294967295); //@line 1512 "compiledict.cpp"
            var $cmp15=((($28))|0) < ((($sub))|0); //@line 1512 "compiledict.cpp"
            if ($cmp15) { __label__ = 7;; } else { __label__ = 9;; } //@line 1512 "compiledict.cpp"
            $land_lhs_true16$$if_end24$13: do { 
              if (__label__ == 7) {
  
                var $30=$p; //@line 1512 "compiledict.cpp"
                var $31=$ix; //@line 1512 "compiledict.cpp"
                var $add17=((($31)+1)&4294967295); //@line 1512 "compiledict.cpp"
                var $32=$rules_addr; //@line 1512 "compiledict.cpp"
                var $arrayidx18=(($32+4*$add17)&4294967295); //@line 1512 "compiledict.cpp"
                var $33=HEAP32[(($arrayidx18)>>2)]; //@line 1512 "compiledict.cpp"
                var $call19=_strcmp($30, $33); //@line 1512 "compiledict.cpp"
                var $cmp20=((($call19))|0)==0; //@line 1512 "compiledict.cpp"
                if (!($cmp20)) { __label__ = 9;break $land_lhs_true16$$if_end24$13; } //@line 1512 "compiledict.cpp"
  
                var $34=$ix; //@line 1514 "compiledict.cpp"
                var $35=$rules_addr; //@line 1514 "compiledict.cpp"
                var $arrayidx22=(($35+4*$34)&4294967295); //@line 1514 "compiledict.cpp"
                var $36=HEAP32[(($arrayidx22)>>2)]; //@line 1514 "compiledict.cpp"
                $common=$36; //@line 1514 "compiledict.cpp"
                var $37=$f_out_addr; //@line 1515 "compiledict.cpp"
                var $call23=_fputc(4, $37); //@line 1515 "compiledict.cpp"
                ; //@line 1516 "compiledict.cpp"
              }
            } while(0);
  
            var $38=$p2; //@line 1518 "compiledict.cpp"
            var $39=$len2; //@line 1518 "compiledict.cpp"
            var $40=$f_out_addr; //@line 1518 "compiledict.cpp"
            var $call25=_fwrite($38, $39, 1, $40); //@line 1518 "compiledict.cpp"
            var $41=$f_out_addr; //@line 1519 "compiledict.cpp"
            var $call26=_fputc(3, $41); //@line 1519 "compiledict.cpp"
            var $42=$p; //@line 1520 "compiledict.cpp"
            var $43=$len1; //@line 1520 "compiledict.cpp"
            var $44=$f_out_addr; //@line 1520 "compiledict.cpp"
            var $call27=_fwrite($42, $43, 1, $44); //@line 1520 "compiledict.cpp"
            ;
          }
  
          var $45=$ix; //@line 1495 "compiledict.cpp"
          var $inc29=((($45)+1)&4294967295); //@line 1495 "compiledict.cpp"
          $ix=$inc29; //@line 1495 "compiledict.cpp"
          var $46=$ix; //@line 1495 "compiledict.cpp"
          var $47=$n_rules_addr; //@line 1495 "compiledict.cpp"
          var $cmp2=((($46))|0) < ((($47))|0); //@line 1495 "compiledict.cpp"
          if ($cmp2) { __label__ = 3;continue $for_body$6; } else { __label__ = 11;break $for_body$$for_end$5; } //@line 1495 "compiledict.cpp"
        }
      }
    } while(0);
  
    STACKTOP = __stackBase__;
    return; //@line 1531 "compiledict.cpp"
    return;
  }
  

  function __ZL19compile_lettergroupPcP7__sFILE($input, $f_out) {
    var __stackBase__  = STACKTOP; STACKTOP += 1000; _memset(__stackBase__, 0, 1000);
    var __label__;
    var __lastLabel__ = null;
  
    var $retval;
    var $input_addr;
    var $f_out_addr;
    var $p;
    var $p_start;
    var $group;
    var $ix;
    var $n_items;
    var $length;
    var $max_length;
    var $items=__stackBase__;
    var $item_length=__stackBase__+800;
    $input_addr=$input;
    $f_out_addr=$f_out;
    $max_length=0; //@line 1543 "compiledict.cpp"
    var $0=$input_addr; //@line 1549 "compiledict.cpp"
    $p=$0; //@line 1549 "compiledict.cpp"
    var $1=$p; //@line 1550 "compiledict.cpp"
    var $arrayidx=(($1)&4294967295); //@line 1550 "compiledict.cpp"
    var $2=HEAP8[($arrayidx)]; //@line 1550 "compiledict.cpp"
    var $conv=(tempInt=(($2)),(tempInt>=128?tempInt-256:tempInt)); //@line 1550 "compiledict.cpp"
    var $isdigittmp=((($conv)-48)&4294967295); //@line 1550 "compiledict.cpp"
    var $isdigit=((($isdigittmp))>>>0) < 10; //@line 1550 "compiledict.cpp"
    var $call=((($isdigit))&1); //@line 1550 "compiledict.cpp"
    var $tobool=((($call))|0)!=0; //@line 1550 "compiledict.cpp"
    if ($tobool) { __label__ = 1;; } else { __label__ = 2;; } //@line 1550 "compiledict.cpp"
    $lor_lhs_false$$if_then$2: do { 
      if (__label__ == 1) {
  
        var $3=$p; //@line 1550 "compiledict.cpp"
        var $arrayidx1=(($3+1)&4294967295); //@line 1550 "compiledict.cpp"
        var $4=HEAP8[($arrayidx1)]; //@line 1550 "compiledict.cpp"
        var $conv2=(tempInt=(($4)),(tempInt>=128?tempInt-256:tempInt)); //@line 1550 "compiledict.cpp"
        var $isdigittmp1=((($conv2)-48)&4294967295); //@line 1550 "compiledict.cpp"
        var $isdigit2=((($isdigittmp1))>>>0) < 10; //@line 1550 "compiledict.cpp"
        var $call3=((($isdigit2))&1); //@line 1550 "compiledict.cpp"
        var $tobool4=((($call3))|0)!=0; //@line 1550 "compiledict.cpp"
        if (!($tobool4)) { __label__ = 2;break $lor_lhs_false$$if_then$2; } //@line 1550 "compiledict.cpp"
  
        var $8=$p; //@line 1557 "compiledict.cpp"
        var $arrayidx6=(($8)&4294967295); //@line 1557 "compiledict.cpp"
        var $call7=_atoi($arrayidx6); //@line 1557 "compiledict.cpp"
        $group=$call7; //@line 1557 "compiledict.cpp"
        var $9=$group; //@line 1558 "compiledict.cpp"
        var $cmp=((($9))|0) >= 26; //@line 1558 "compiledict.cpp"
        if ($cmp) { __label__ = 5;; } else { __label__ = 4;; } //@line 1558 "compiledict.cpp"
        if (__label__ == 5) {
  
          var $12=HEAP32[((__ZL5f_log)>>2)]; //@line 1560 "compiledict.cpp"
          var $13=HEAP32[((__ZL7linenum)>>2)]; //@line 1560 "compiledict.cpp"
          var $call9=_fprintf($12, ((__str101)&4294967295), allocate([$13,0,0,0,25,0,0,0], ["i32",0,0,0,"i32",0,0,0], ALLOC_STACK)); //@line 1560 "compiledict.cpp"
          var $14=HEAP32[((__ZL11error_count)>>2)]; //@line 1561 "compiledict.cpp"
          var $inc10=((($14)+1)&4294967295); //@line 1561 "compiledict.cpp"
          HEAP32[((__ZL11error_count)>>2)]=$inc10; //@line 1561 "compiledict.cpp"
          $retval=1; //@line 1562 "compiledict.cpp"
          __label__ = 26;break $lor_lhs_false$$if_then$2; //@line 1562 "compiledict.cpp"
        }
        else if (__label__ == 4) {
  
          var $10=$p; //@line 1565 "compiledict.cpp"
          var $11=HEAP8[($10)]; //@line 1565 "compiledict.cpp"
          var $conv1210=(tempInt=(($11)),(tempInt>=128?tempInt-256:tempInt)); //@line 1565 "compiledict.cpp"
          var $call1311=__Z8isspace2j($conv1210); //@line 1565 "compiledict.cpp"
          var $tobool1412=((($call1311))|0)!=0; //@line 1565 "compiledict.cpp"
          var $lnot13=($tobool1412) ^ 1; //@line 1565 "compiledict.cpp"
          if ($lnot13) { __label__ = 6;; } else { __label__ = 7;; } //@line 1565 "compiledict.cpp"
          $while_body$$while_end$8: do { 
            if (__label__ == 6) {
              while(1) { 
  
                var $15=$p; //@line 1565 "compiledict.cpp"
                var $incdec_ptr=(($15+1)&4294967295); //@line 1565 "compiledict.cpp"
                $p=$incdec_ptr; //@line 1565 "compiledict.cpp"
                var $16=$p; //@line 1565 "compiledict.cpp"
                var $17=HEAP8[($16)]; //@line 1565 "compiledict.cpp"
                var $conv12=(tempInt=(($17)),(tempInt>=128?tempInt-256:tempInt)); //@line 1565 "compiledict.cpp"
                var $call13=__Z8isspace2j($conv12); //@line 1565 "compiledict.cpp"
                var $tobool14=((($call13))|0)!=0; //@line 1565 "compiledict.cpp"
                var $lnot=($tobool14) ^ 1; //@line 1565 "compiledict.cpp"
                if ($lnot) { __label__ = 6;continue ; } else { __label__ = 7;break $while_body$$while_end$8; } //@line 1565 "compiledict.cpp"
              }
            }
          } while(0);
  
          var $18=$f_out_addr; //@line 1567 "compiledict.cpp"
          var $call15=_fputc(6, $18); //@line 1567 "compiledict.cpp"
          var $19=$f_out_addr; //@line 1568 "compiledict.cpp"
          var $call16=_fputc(18, $19); //@line 1568 "compiledict.cpp"
          var $20=$group; //@line 1569 "compiledict.cpp"
          var $add=((($20)+65)&4294967295); //@line 1569 "compiledict.cpp"
          var $21=$f_out_addr; //@line 1569 "compiledict.cpp"
          var $call17=_fputc($add, $21); //@line 1569 "compiledict.cpp"
          var $22=$group; //@line 1570 "compiledict.cpp"
          var $arrayidx18=((__ZL19letterGroupsDefined+$22)&4294967295); //@line 1570 "compiledict.cpp"
          var $23=HEAP8[($arrayidx18)]; //@line 1570 "compiledict.cpp"
          var $conv19=(tempInt=(($23)),(tempInt>=128?tempInt-256:tempInt)); //@line 1570 "compiledict.cpp"
          var $cmp20=((($conv19))|0)!=0; //@line 1570 "compiledict.cpp"
          if ($cmp20) { __label__ = 8;; } else { __label__ = 9;; } //@line 1570 "compiledict.cpp"
          if (__label__ == 8) {
  
            var $24=HEAP32[((__ZL5f_log)>>2)]; //@line 1572 "compiledict.cpp"
            var $25=HEAP32[((__ZL7linenum)>>2)]; //@line 1572 "compiledict.cpp"
            var $26=$group; //@line 1572 "compiledict.cpp"
            var $call22=_fprintf($24, ((__str102)&4294967295), allocate([$25,0,0,0,$26,0,0,0], ["i32",0,0,0,"i32",0,0,0], ALLOC_STACK)); //@line 1572 "compiledict.cpp"
            var $27=HEAP32[((__ZL11error_count)>>2)]; //@line 1573 "compiledict.cpp"
            var $inc23=((($27)+1)&4294967295); //@line 1573 "compiledict.cpp"
            HEAP32[((__ZL11error_count)>>2)]=$inc23; //@line 1573 "compiledict.cpp"
            ; //@line 1574 "compiledict.cpp"
          }
  
          var $28=$group; //@line 1575 "compiledict.cpp"
          var $arrayidx25=((__ZL19letterGroupsDefined+$28)&4294967295); //@line 1575 "compiledict.cpp"
          HEAP8[($arrayidx25)]=1; //@line 1575 "compiledict.cpp"
          $n_items=0; //@line 1577 "compiledict.cpp"
          __lastLabel__ = 9; ; //@line 1578 "compiledict.cpp"
          $while_cond26$15: while(1) { 
  
            var $29=__lastLabel__ == 18 ? $_pr : (0); //@line 1578 "compiledict.cpp"
            var $cmp27=((($29))|0) < 200; //@line 1578 "compiledict.cpp"
            if (!($cmp27)) { __label__ = 19;break $while_cond26$15; } //@line 1578 "compiledict.cpp"
  
            var $30=$p; //@line 1580 "compiledict.cpp"
            var $31=HEAP8[($30)]; //@line 1580 "compiledict.cpp"
            var $conv306=(tempInt=(($31)),(tempInt>=128?tempInt-256:tempInt)); //@line 1580 "compiledict.cpp"
            var $call317=__Z8isspace2j($conv306); //@line 1580 "compiledict.cpp"
            var $tobool328=((($call317))|0)!=0; //@line 1580 "compiledict.cpp"
            var $32=$p; //@line 1580 "compiledict.cpp"
            if ($tobool328) { __lastLabel__ = 11; __label__ = 12;; } else { __lastLabel__ = 11; __label__ = 13;; } //@line 1580 "compiledict.cpp"
            $while_body33$$while_end35$18: do { 
              if (__label__ == 12) {
                while(1) { 
  
                  var $33=__lastLabel__ == 12 ? $36 : ($32);
                  var $incdec_ptr34=(($33+1)&4294967295); //@line 1580 "compiledict.cpp"
                  $p=$incdec_ptr34; //@line 1580 "compiledict.cpp"
                  var $34=$p; //@line 1580 "compiledict.cpp"
                  var $35=HEAP8[($34)]; //@line 1580 "compiledict.cpp"
                  var $conv30=(tempInt=(($35)),(tempInt>=128?tempInt-256:tempInt)); //@line 1580 "compiledict.cpp"
                  var $call31=__Z8isspace2j($conv30); //@line 1580 "compiledict.cpp"
                  var $tobool32=((($call31))|0)!=0; //@line 1580 "compiledict.cpp"
                  var $36=$p; //@line 1580 "compiledict.cpp"
                  if ($tobool32) { __lastLabel__ = 12; __label__ = 12;continue ; } else { __lastLabel__ = 12; __label__ = 13;break $while_body33$$while_end35$18; } //@line 1580 "compiledict.cpp"
                }
              }
            } while(0);
  
            var $_lcssa=__lastLabel__ == 11 ? $32 : ($36);
            var $37=HEAP8[($_lcssa)]; //@line 1581 "compiledict.cpp"
            var $conv36=(tempInt=(($37)),(tempInt>=128?tempInt-256:tempInt)); //@line 1581 "compiledict.cpp"
            var $cmp37=((($conv36))|0)==0; //@line 1581 "compiledict.cpp"
            if ($cmp37) { __label__ = 19;break $while_cond26$15; } //@line 1581 "compiledict.cpp"
  
            var $38=$p; //@line 1584 "compiledict.cpp"
            $p_start=$38; //@line 1584 "compiledict.cpp"
            var $39=$n_items; //@line 1584 "compiledict.cpp"
            var $arrayidx40=(($items+$39*4)&4294967295); //@line 1584 "compiledict.cpp"
            HEAP32[(($arrayidx40)>>2)]=$38; //@line 1584 "compiledict.cpp"
            ; //@line 1585 "compiledict.cpp"
            while(1) { 
  
              var $40=$p; //@line 1585 "compiledict.cpp"
              var $41=HEAP8[($40)]; //@line 1585 "compiledict.cpp"
              var $conv42=(tempInt=(($41)),(tempInt>=128?tempInt-256:tempInt)); //@line 1585 "compiledict.cpp"
              var $and=($conv42) & 255; //@line 1585 "compiledict.cpp"
              var $cmp43=((($and))|0) > 32; //@line 1585 "compiledict.cpp"
              var $42=$p; //@line 1587 "compiledict.cpp"
              var $incdec_ptr45=(($42+1)&4294967295); //@line 1587 "compiledict.cpp"
              $p=$incdec_ptr45; //@line 1587 "compiledict.cpp"
              if ($cmp43) { __label__ = 15;continue ; } else { __label__ = 16;break ; } //@line 1585 "compiledict.cpp"
            }
  
            HEAP8[($42)]=0; //@line 1589 "compiledict.cpp"
            var $43=$p; //@line 1590 "compiledict.cpp"
            var $44=$p_start; //@line 1590 "compiledict.cpp"
            var $sub_ptr_lhs_cast=($43); //@line 1590 "compiledict.cpp"
            var $sub_ptr_rhs_cast=($44); //@line 1590 "compiledict.cpp"
            var $sub_ptr_sub=((($sub_ptr_lhs_cast)-($sub_ptr_rhs_cast))&4294967295); //@line 1590 "compiledict.cpp"
            $length=$sub_ptr_sub; //@line 1590 "compiledict.cpp"
            var $45=$length; //@line 1591 "compiledict.cpp"
            var $46=$max_length; //@line 1591 "compiledict.cpp"
            var $cmp48=((($45))|0) > ((($46))|0); //@line 1591 "compiledict.cpp"
            if ($cmp48) { __label__ = 17;; } else { __label__ = 18;; } //@line 1591 "compiledict.cpp"
            if (__label__ == 17) {
  
              var $47=$length; //@line 1592 "compiledict.cpp"
              $max_length=$47; //@line 1592 "compiledict.cpp"
              ; //@line 1592 "compiledict.cpp"
            }
  
            var $48=$length; //@line 1593 "compiledict.cpp"
            var $conv51=((($48)) & 255); //@line 1593 "compiledict.cpp"
            var $49=$n_items; //@line 1593 "compiledict.cpp"
            var $inc52=((($49)+1)&4294967295); //@line 1593 "compiledict.cpp"
            $n_items=$inc52; //@line 1593 "compiledict.cpp"
            var $arrayidx53=(($item_length+$49)&4294967295); //@line 1593 "compiledict.cpp"
            HEAP8[($arrayidx53)]=$conv51; //@line 1593 "compiledict.cpp"
            var $_pr=$n_items; //@line 1578 "compiledict.cpp"
            __lastLabel__ = 18; __label__ = 10;continue $while_cond26$15; //@line 1594 "compiledict.cpp"
          }
  
          var $_pr3=$max_length; //@line 1597 "compiledict.cpp"
          var $cmp565=((($_pr3))|0) > 1; //@line 1597 "compiledict.cpp"
          if ($cmp565) { __label__ = 20;; } else { __label__ = 25;; } //@line 1597 "compiledict.cpp"
          $while_body57$$while_end67$30: do { 
            if (__label__ == 20) {
              $while_body57$31: while(1) { 
  
                $ix=0; //@line 1599 "compiledict.cpp"
                var $50=$ix; //@line 1599 "compiledict.cpp"
                var $51=$n_items; //@line 1599 "compiledict.cpp"
                var $cmp584=((($50))|0) < ((($51))|0); //@line 1599 "compiledict.cpp"
                if ($cmp584) { __label__ = 21;; } else { __label__ = 24;; } //@line 1599 "compiledict.cpp"
                $for_body$$for_end$33: do { 
                  if (__label__ == 21) {
                    while(1) { 
  
                      var $52=$ix; //@line 1601 "compiledict.cpp"
                      var $arrayidx59=(($item_length+$52)&4294967295); //@line 1601 "compiledict.cpp"
                      var $53=HEAP8[($arrayidx59)]; //@line 1601 "compiledict.cpp"
                      var $conv60=(tempInt=(($53)),(tempInt>=128?tempInt-256:tempInt)); //@line 1601 "compiledict.cpp"
                      var $54=$max_length; //@line 1601 "compiledict.cpp"
                      var $cmp61=((($conv60))|0)==((($54))|0); //@line 1601 "compiledict.cpp"
                      if ($cmp61) { __label__ = 22;; } else { __label__ = 23;; } //@line 1601 "compiledict.cpp"
                      if (__label__ == 22) {
  
                        var $55=$ix; //@line 1603 "compiledict.cpp"
                        var $arrayidx63=(($items+$55*4)&4294967295); //@line 1603 "compiledict.cpp"
                        var $56=HEAP32[(($arrayidx63)>>2)]; //@line 1603 "compiledict.cpp"
                        var $57=$max_length; //@line 1603 "compiledict.cpp"
                        var $58=$f_out_addr; //@line 1603 "compiledict.cpp"
                        var $call64=_fwrite($56, 1, $57, $58); //@line 1603 "compiledict.cpp"
                        ; //@line 1604 "compiledict.cpp"
                      }
  
                      var $59=$ix; //@line 1599 "compiledict.cpp"
                      var $inc66=((($59)+1)&4294967295); //@line 1599 "compiledict.cpp"
                      $ix=$inc66; //@line 1599 "compiledict.cpp"
                      var $60=$ix; //@line 1599 "compiledict.cpp"
                      var $61=$n_items; //@line 1599 "compiledict.cpp"
                      var $cmp58=((($60))|0) < ((($61))|0); //@line 1599 "compiledict.cpp"
                      if ($cmp58) { __label__ = 21;continue ; } else { __label__ = 24;break $for_body$$for_end$33; } //@line 1599 "compiledict.cpp"
                    }
                  }
                } while(0);
  
                var $62=$max_length; //@line 1606 "compiledict.cpp"
                var $dec=((($62)-1)&4294967295); //@line 1606 "compiledict.cpp"
                $max_length=$dec; //@line 1606 "compiledict.cpp"
                var $cmp56=((($dec))|0) > 1; //@line 1597 "compiledict.cpp"
                if ($cmp56) { __label__ = 20;continue $while_body57$31; } else { __label__ = 25;break $while_body57$$while_end67$30; } //@line 1597 "compiledict.cpp"
              }
            }
          } while(0);
  
          var $63=$f_out_addr; //@line 1609 "compiledict.cpp"
          var $call68=_fputc(7, $63); //@line 1609 "compiledict.cpp"
          $retval=0; //@line 1611 "compiledict.cpp"
          __label__ = 26;break $lor_lhs_false$$if_then$2; //@line 1611 "compiledict.cpp"
        }
      }
    } while(0);
    if (__label__ == 2) {
  
      var $5=HEAP32[((__ZL5f_log)>>2)]; //@line 1552 "compiledict.cpp"
      var $6=HEAP32[((__ZL7linenum)>>2)]; //@line 1552 "compiledict.cpp"
      var $call5=_fprintf($5, ((__str100)&4294967295), allocate([$6,0,0,0], ["i32",0,0,0], ALLOC_STACK)); //@line 1552 "compiledict.cpp"
      var $7=HEAP32[((__ZL11error_count)>>2)]; //@line 1553 "compiledict.cpp"
      var $inc=((($7)+1)&4294967295); //@line 1553 "compiledict.cpp"
      HEAP32[((__ZL11error_count)>>2)]=$inc; //@line 1553 "compiledict.cpp"
      $retval=1; //@line 1554 "compiledict.cpp"
      ; //@line 1554 "compiledict.cpp"
    }
  
    STACKTOP = __stackBase__;
    return;
    return;
  }
  

  function __ZL12compile_rulePc($input) {
    var __stackBase__  = STACKTOP; STACKTOP += 244; _memset(__stackBase__, 0, 244);
    var __label__;
  
    var $retval;
    var $input_addr;
    var $ix;
    var $c;
    var $wc=__stackBase__;
    var $p;
    var $prule;
    var $len;
    var $len_name;
    var $start;
    var $state=__stackBase__+4;
    var $finish;
    var $pre_bracket;
    var $buf=__stackBase__+8;
    var $output=__stackBase__+88;
    var $bad_phoneme=__stackBase__+240;
    $input_addr=$input;
    HEAP32[(($state)>>2)]=2; //@line 1161 "compiledict.cpp"
    $finish=0; //@line 1162 "compiledict.cpp"
    $pre_bracket=0; //@line 1163 "compiledict.cpp"
    var $arrayidx=(($buf)&4294967295); //@line 1168 "compiledict.cpp"
    HEAP8[($arrayidx)]=0; //@line 1168 "compiledict.cpp"
    HEAP8[(((__ZL9rule_cond)&4294967295))]=0; //@line 1169 "compiledict.cpp"
    HEAP8[(((__ZL8rule_pre)&4294967295))]=0; //@line 1170 "compiledict.cpp"
    HEAP8[(((__ZL9rule_post)&4294967295))]=0; //@line 1171 "compiledict.cpp"
    HEAP8[(((__ZL10rule_match)&4294967295))]=0; //@line 1172 "compiledict.cpp"
    HEAP8[(((__ZL13rule_phonemes)&4294967295))]=0; //@line 1173 "compiledict.cpp"
    var $arraydecay=(($buf)&4294967295); //@line 1175 "compiledict.cpp"
    $p=$arraydecay; //@line 1175 "compiledict.cpp"
    $ix=0; //@line 1177 "compiledict.cpp"
    var $0=$finish; //@line 1177 "compiledict.cpp"
    var $cmp2=((($0))|0)==0; //@line 1177 "compiledict.cpp"
    if ($cmp2) { __label__ = 1;; } else { __label__ = 13;; } //@line 1177 "compiledict.cpp"
    $for_body_lr_ph$$for_end$2: do { 
      if (__label__ == 1) {
  
        var $arraydecay3=(($buf)&4294967295); //@line 1187 "compiledict.cpp"
        var $arraydecay4=(($buf)&4294967295); //@line 1188 "compiledict.cpp"
        var $arraydecay6=(($buf)&4294967295); //@line 1194 "compiledict.cpp"
        var $arraydecay7=(($buf)&4294967295); //@line 1196 "compiledict.cpp"
        var $arraydecay12=(($buf)&4294967295); //@line 1208 "compiledict.cpp"
        var $arraydecay14=(($buf)&4294967295); //@line 1215 "compiledict.cpp"
        var $arraydecay15=(($buf)&4294967295); //@line 1216 "compiledict.cpp"
        ; //@line 1177 "compiledict.cpp"
        $for_body$4: while(1) { 
  
          var $1=$ix; //@line 1179 "compiledict.cpp"
          var $2=$input_addr; //@line 1179 "compiledict.cpp"
          var $arrayidx1=(($2+$1)&4294967295); //@line 1179 "compiledict.cpp"
          var $3=HEAP8[($arrayidx1)]; //@line 1179 "compiledict.cpp"
          $c=$3; //@line 1179 "compiledict.cpp"
          var $4=$ix; //@line 1181 "compiledict.cpp"
          var $5=$input_addr; //@line 1181 "compiledict.cpp"
          var $arrayidx2=(($5+$4)&4294967295); //@line 1181 "compiledict.cpp"
          var $6=HEAPU8[($arrayidx2)]; //@line 1181 "compiledict.cpp"
          $c=$6; //@line 1181 "compiledict.cpp"
          var $conv=((($6))&255); //@line 1181 "compiledict.cpp"
          if ($conv == 41) {
            __label__ = 3;;
          }
          else if ($conv == 40) {
            __label__ = 4;;
          }
          else if ($conv == 10) {
            __label__ = 6;break $for_body$4;
          }
          else if ($conv == 13) {
            __label__ = 6;break $for_body$4;
          }
          else if ($conv == 0) {
            __label__ = 6;break $for_body$4;
          }
          else if ($conv == 9) {
            __label__ = 7;;
          }
          else if ($conv == 32) {
            __label__ = 7;;
          }
          else if ($conv == 63) {
            __label__ = 8;;
          }
          else {
          __label__ = 11;;
          }
          
          $sw_default$$sw_bb$$sw_bb5$$sw_bb13$$sw_bb16$6: do { 
            if (__label__ == 11) {
  
              var $21=$c; //@line 1227 "compiledict.cpp"
              var $22=$p; //@line 1227 "compiledict.cpp"
              var $incdec_ptr20=(($22+1)&4294967295); //@line 1227 "compiledict.cpp"
              $p=$incdec_ptr20; //@line 1227 "compiledict.cpp"
              HEAP8[($22)]=$21; //@line 1227 "compiledict.cpp"
              ; //@line 1228 "compiledict.cpp"
            }
            else if (__label__ == 3) {
  
              var $7=$p; //@line 1184 "compiledict.cpp"
              HEAP8[($7)]=0; //@line 1184 "compiledict.cpp"
              HEAP32[(($state)>>2)]=1; //@line 1185 "compiledict.cpp"
              $pre_bracket=1; //@line 1186 "compiledict.cpp"
              __ZL16copy_rule_stringPcRi($arraydecay3, $state); //@line 1187 "compiledict.cpp"
              $p=$arraydecay4; //@line 1188 "compiledict.cpp"
              ; //@line 1189 "compiledict.cpp"
            }
            else if (__label__ == 4) {
  
              var $8=$p; //@line 1192 "compiledict.cpp"
              HEAP8[($8)]=0; //@line 1192 "compiledict.cpp"
              HEAP32[(($state)>>2)]=2; //@line 1193 "compiledict.cpp"
              __ZL16copy_rule_stringPcRi($arraydecay6, $state); //@line 1194 "compiledict.cpp"
              HEAP32[(($state)>>2)]=3; //@line 1195 "compiledict.cpp"
              $p=$arraydecay7; //@line 1196 "compiledict.cpp"
              var $9=$ix; //@line 1197 "compiledict.cpp"
              var $add=((($9)+1)&4294967295); //@line 1197 "compiledict.cpp"
              var $10=$input_addr; //@line 1197 "compiledict.cpp"
              var $arrayidx8=(($10+$add)&4294967295); //@line 1197 "compiledict.cpp"
              var $11=HEAP8[($arrayidx8)]; //@line 1197 "compiledict.cpp"
              var $conv9=(tempInt=(($11)),(tempInt>=128?tempInt-256:tempInt)); //@line 1197 "compiledict.cpp"
              var $cmp10=((($conv9))|0)==32; //@line 1197 "compiledict.cpp"
              if (!($cmp10)) { __label__ = 12;break $sw_default$$sw_bb$$sw_bb5$$sw_bb13$$sw_bb16$6; } //@line 1197 "compiledict.cpp"
  
              var $12=HEAP32[((__ZL5f_log)>>2)]; //@line 1199 "compiledict.cpp"
              var $13=HEAP32[((__ZL7linenum)>>2)]; //@line 1199 "compiledict.cpp"
              var $call=_fprintf($12, ((__str90)&4294967295), allocate([$13,0,0,0], ["i32",0,0,0], ALLOC_STACK)); //@line 1199 "compiledict.cpp"
              var $14=HEAP32[((__ZL11error_count)>>2)]; //@line 1200 "compiledict.cpp"
              var $inc=((($14)+1)&4294967295); //@line 1200 "compiledict.cpp"
              HEAP32[((__ZL11error_count)>>2)]=$inc; //@line 1200 "compiledict.cpp"
              ; //@line 1201 "compiledict.cpp"
            }
            else if (__label__ == 7) {
  
              var $17=$p; //@line 1214 "compiledict.cpp"
              HEAP8[($17)]=0; //@line 1214 "compiledict.cpp"
              __ZL16copy_rule_stringPcRi($arraydecay14, $state); //@line 1215 "compiledict.cpp"
              $p=$arraydecay15; //@line 1216 "compiledict.cpp"
              ; //@line 1217 "compiledict.cpp"
            }
            else if (__label__ == 8) {
  
              var $18=HEAP32[(($state)>>2)]; //@line 1220 "compiledict.cpp"
              var $cmp17=((($18))|0)==2; //@line 1220 "compiledict.cpp"
              if ($cmp17) { __label__ = 9;; } else { __label__ = 10;; } //@line 1220 "compiledict.cpp"
              if (__label__ == 9) {
  
                HEAP32[(($state)>>2)]=0; //@line 1221 "compiledict.cpp"
                ; //@line 1221 "compiledict.cpp"
              }
              else if (__label__ == 10) {
  
                var $19=$c; //@line 1223 "compiledict.cpp"
                var $20=$p; //@line 1223 "compiledict.cpp"
                var $incdec_ptr=(($20+1)&4294967295); //@line 1223 "compiledict.cpp"
                $p=$incdec_ptr; //@line 1223 "compiledict.cpp"
                HEAP8[($20)]=$19; //@line 1223 "compiledict.cpp"
                ;
              }
            }
          } while(0);
  
          var $_pr=$finish; //@line 1177 "compiledict.cpp"
          var $23=$ix; //@line 1177 "compiledict.cpp"
          var $inc21=((($23)+1)&4294967295); //@line 1177 "compiledict.cpp"
          $ix=$inc21; //@line 1177 "compiledict.cpp"
          var $cmp=((($_pr))|0)==0; //@line 1177 "compiledict.cpp"
          if ($cmp) { __label__ = 2;continue $for_body$4; } else { __label__ = 13;break $for_body_lr_ph$$for_end$2; } //@line 1177 "compiledict.cpp"
        }
  
        var $15=$p; //@line 1207 "compiledict.cpp"
        HEAP8[($15)]=0; //@line 1207 "compiledict.cpp"
        __ZL16copy_rule_stringPcRi($arraydecay12, $state); //@line 1208 "compiledict.cpp"
        $finish=1; //@line 1209 "compiledict.cpp"
        var $16=$ix; //@line 1177 "compiledict.cpp"
        var $inc213=((($16)+1)&4294967295); //@line 1177 "compiledict.cpp"
        $ix=$inc213; //@line 1177 "compiledict.cpp"
        ; //@line 1177 "compiledict.cpp"
      }
    } while(0);
  
    var $call22=_strcmp(((__ZL10rule_match)&4294967295), ((__str91)&4294967295)); //@line 1232 "compiledict.cpp"
    var $cmp23=((($call22))|0)==0; //@line 1232 "compiledict.cpp"
    if ($cmp23) { __label__ = 14;; } else { __label__ = 15;; } //@line 1232 "compiledict.cpp"
    if (__label__ == 14) {
  
      var $call25=_strcpy(((__ZL10rule_match)&4294967295), ((__ZL10group_name)&4294967295)); //@line 1233 "compiledict.cpp"
      ; //@line 1233 "compiledict.cpp"
    }
  
    var $24=HEAP8[(((__ZL10rule_match)&4294967295))]; //@line 1235 "compiledict.cpp"
    var $conv27=(tempInt=(($24)),(tempInt>=128?tempInt-256:tempInt)); //@line 1235 "compiledict.cpp"
    var $cmp28=((($conv27))|0)==0; //@line 1235 "compiledict.cpp"
    if ($cmp28) { __label__ = 16;; } else { __label__ = 19;; } //@line 1235 "compiledict.cpp"
    if (__label__ == 16) {
  
      var $25=HEAP8[(((__ZL9rule_post)&4294967295))]; //@line 1237 "compiledict.cpp"
      var $conv30=(tempInt=(($25)),(tempInt>=128?tempInt-256:tempInt)); //@line 1237 "compiledict.cpp"
      var $cmp31=((($conv30))|0)!=0; //@line 1237 "compiledict.cpp"
      if ($cmp31) { __label__ = 17;; } else { __label__ = 18;; } //@line 1237 "compiledict.cpp"
      if (__label__ == 17) {
  
        var $26=HEAP32[((__ZL5f_log)>>2)]; //@line 1239 "compiledict.cpp"
        var $27=HEAP32[((__ZL7linenum)>>2)]; //@line 1239 "compiledict.cpp"
        var $call33=_fprintf($26, ((__str92)&4294967295), allocate([$27,0,0,0], ["i32",0,0,0], ALLOC_STACK)); //@line 1239 "compiledict.cpp"
        var $28=HEAP32[((__ZL11error_count)>>2)]; //@line 1240 "compiledict.cpp"
        var $inc34=((($28)+1)&4294967295); //@line 1240 "compiledict.cpp"
        HEAP32[((__ZL11error_count)>>2)]=$inc34; //@line 1240 "compiledict.cpp"
        ; //@line 1241 "compiledict.cpp"
      }
  
      $retval=0; //@line 1242 "compiledict.cpp"
      ; //@line 1242 "compiledict.cpp"
    }
    else if (__label__ == 19) {
  
      var $arraydecay37=(($buf)&4294967295); //@line 1245 "compiledict.cpp"
      var $arraydecay38=(($bad_phoneme)&4294967295); //@line 1245 "compiledict.cpp"
      var $call39=__Z14EncodePhonemesPcS_Ph(((__ZL13rule_phonemes)&4294967295), $arraydecay37, $arraydecay38); //@line 1245 "compiledict.cpp"
      var $arrayidx40=(($bad_phoneme)&4294967295); //@line 1246 "compiledict.cpp"
      var $29=HEAPU8[($arrayidx40)]; //@line 1246 "compiledict.cpp"
      var $conv41=((($29))&255); //@line 1246 "compiledict.cpp"
      var $cmp42=((($conv41))|0)!=0; //@line 1246 "compiledict.cpp"
      if ($cmp42) { __label__ = 20;; } else { __label__ = 21;; } //@line 1246 "compiledict.cpp"
      if (__label__ == 20) {
  
        var $30=HEAP32[((__ZL5f_log)>>2)]; //@line 1248 "compiledict.cpp"
        var $31=HEAP32[((__ZL7linenum)>>2)]; //@line 1248 "compiledict.cpp"
        var $arrayidx44=(($bad_phoneme)&4294967295); //@line 1248 "compiledict.cpp"
        var $32=HEAPU8[($arrayidx44)]; //@line 1248 "compiledict.cpp"
        var $conv45=((($32))&255); //@line 1248 "compiledict.cpp"
        var $33=$input_addr; //@line 1248 "compiledict.cpp"
        var $call46=_fprintf($30, ((__str93)&4294967295), allocate([$31,0,0,0,$conv45,0,0,0,$33,0,0,0], ["i32",0,0,0,"i32",0,0,0,"i8*",0,0,0], ALLOC_STACK)); //@line 1248 "compiledict.cpp"
        var $34=HEAP32[((__ZL11error_count)>>2)]; //@line 1249 "compiledict.cpp"
        var $inc47=((($34)+1)&4294967295); //@line 1249 "compiledict.cpp"
        HEAP32[((__ZL11error_count)>>2)]=$inc47; //@line 1249 "compiledict.cpp"
        ; //@line 1250 "compiledict.cpp"
      }
  
      var $arraydecay49=(($output)&4294967295); //@line 1251 "compiledict.cpp"
      var $arraydecay50=(($buf)&4294967295); //@line 1251 "compiledict.cpp"
      var $call51=_strcpy($arraydecay49, $arraydecay50); //@line 1251 "compiledict.cpp"
      var $arraydecay52=(($buf)&4294967295); //@line 1252 "compiledict.cpp"
      var $call53=_strlen($arraydecay52); //@line 1252 "compiledict.cpp"
      var $add54=((($call53)+1)&4294967295); //@line 1252 "compiledict.cpp"
      $len=$add54; //@line 1252 "compiledict.cpp"
      var $call55=_strlen(((__ZL10group_name)&4294967295)); //@line 1254 "compiledict.cpp"
      $len_name=$call55; //@line 1254 "compiledict.cpp"
      var $cmp56=((($call55))|0) > 0; //@line 1255 "compiledict.cpp"
      if ($cmp56) { __label__ = 22;; } else { __label__ = 26;; } //@line 1255 "compiledict.cpp"
      $land_lhs_true$$if_end70$31: do { 
        if (__label__ == 22) {
  
          var $35=$len_name; //@line 1255 "compiledict.cpp"
          var $call57=_memcmp(((__ZL10rule_match)&4294967295), ((__ZL10group_name)&4294967295), $35); //@line 1255 "compiledict.cpp"
          var $cmp58=((($call57))|0)!=0; //@line 1255 "compiledict.cpp"
          if (!($cmp58)) { __label__ = 26;break $land_lhs_true$$if_end70$31; } //@line 1255 "compiledict.cpp"
  
          var $call60=__Z7utf8_inPiPKc($wc, ((__ZL10rule_match)&4294967295)); //@line 1257 "compiledict.cpp"
          var $36=HEAP8[(((__ZL10group_name)&4294967295))]; //@line 1258 "compiledict.cpp"
          var $conv61=(tempInt=(($36)),(tempInt>=128?tempInt-256:tempInt)); //@line 1258 "compiledict.cpp"
          var $cmp62=((($conv61))|0)==57; //@line 1258 "compiledict.cpp"
          if ($cmp62) { __label__ = 24;; } else { __label__ = 25;; } //@line 1258 "compiledict.cpp"
          if (__label__ == 24) {
  
            var $37=HEAP32[(($wc)>>2)]; //@line 1258 "compiledict.cpp"
            var $call64=__Z7IsDigitj($37); //@line 1258 "compiledict.cpp"
            var $tobool=((($call64))|0)!=0; //@line 1258 "compiledict.cpp"
            if ($tobool) { __label__ = 26;break $land_lhs_true$$if_end70$31; } //@line 1258 "compiledict.cpp"
          }
  
          var $38=HEAP32[((__ZL5f_log)>>2)]; //@line 1264 "compiledict.cpp"
          var $39=HEAP32[((__ZL7linenum)>>2)]; //@line 1264 "compiledict.cpp"
          var $call67=_fprintf($38, ((__str94)&4294967295), allocate([$39,0,0,0,((__ZL10rule_match)&4294967295),0,0,0,((__ZL10group_name)&4294967295),0,0,0], ["i32",0,0,0,"i8*",0,0,0,"i8*",0,0,0], ALLOC_STACK)); //@line 1264 "compiledict.cpp"
          var $40=HEAP32[((__ZL11error_count)>>2)]; //@line 1265 "compiledict.cpp"
          var $inc68=((($40)+1)&4294967295); //@line 1265 "compiledict.cpp"
          HEAP32[((__ZL11error_count)>>2)]=$inc68; //@line 1265 "compiledict.cpp"
          ;
        }
      } while(0);
  
      var $41=$len; //@line 1268 "compiledict.cpp"
      var $arrayidx71=(($output+$41)&4294967295); //@line 1268 "compiledict.cpp"
      var $call72=_strcpy($arrayidx71, ((__ZL10rule_match)&4294967295)); //@line 1268 "compiledict.cpp"
      var $call73=_strlen(((__ZL10rule_match)&4294967295)); //@line 1269 "compiledict.cpp"
      var $42=$len; //@line 1269 "compiledict.cpp"
      var $add74=((($42)+($call73))&4294967295); //@line 1269 "compiledict.cpp"
      $len=$add74; //@line 1269 "compiledict.cpp"
      var $43=HEAP32[((__ZL10debug_flag)>>2)]; //@line 1271 "compiledict.cpp"
      var $tobool75=((($43))|0)!=0; //@line 1271 "compiledict.cpp"
      if ($tobool75) { __label__ = 27;; } else { __label__ = 28;; } //@line 1271 "compiledict.cpp"
      if (__label__ == 27) {
  
        var $44=$len; //@line 1273 "compiledict.cpp"
        var $arrayidx77=(($output+$44)&4294967295); //@line 1273 "compiledict.cpp"
        HEAP8[($arrayidx77)]=9; //@line 1273 "compiledict.cpp"
        var $45=HEAP32[((__ZL7linenum)>>2)]; //@line 1274 "compiledict.cpp"
        var $rem=((($45))|0)%255; //@line 1274 "compiledict.cpp"
        var $add78=((($rem)+1)&4294967295); //@line 1274 "compiledict.cpp"
        var $conv79=((($add78)) & 255); //@line 1274 "compiledict.cpp"
        var $46=$len; //@line 1274 "compiledict.cpp"
        var $add80=((($46)+1)&4294967295); //@line 1274 "compiledict.cpp"
        var $arrayidx81=(($output+$add80)&4294967295); //@line 1274 "compiledict.cpp"
        HEAP8[($arrayidx81)]=$conv79; //@line 1274 "compiledict.cpp"
        var $47=HEAP32[((__ZL7linenum)>>2)]; //@line 1275 "compiledict.cpp"
        var $div=((((($47))|0)/255)|0); //@line 1275 "compiledict.cpp"
        var $add82=((($div)+1)&4294967295); //@line 1275 "compiledict.cpp"
        var $conv83=((($add82)) & 255); //@line 1275 "compiledict.cpp"
        var $48=$len; //@line 1275 "compiledict.cpp"
        var $add84=((($48)+2)&4294967295); //@line 1275 "compiledict.cpp"
        var $arrayidx85=(($output+$add84)&4294967295); //@line 1275 "compiledict.cpp"
        HEAP8[($arrayidx85)]=$conv83; //@line 1275 "compiledict.cpp"
        var $49=$len; //@line 1276 "compiledict.cpp"
        var $add86=((($49)+3)&4294967295); //@line 1276 "compiledict.cpp"
        $len=$add86; //@line 1276 "compiledict.cpp"
        ; //@line 1277 "compiledict.cpp"
      }
  
      var $50=HEAP8[(((__ZL9rule_cond)&4294967295))]; //@line 1279 "compiledict.cpp"
      var $conv88=(tempInt=(($50)),(tempInt>=128?tempInt-256:tempInt)); //@line 1279 "compiledict.cpp"
      var $cmp89=((($conv88))|0)!=0; //@line 1279 "compiledict.cpp"
      if ($cmp89) { __label__ = 29;; } else { __label__ = 35;; } //@line 1279 "compiledict.cpp"
      if (__label__ == 29) {
  
        $ix=-1; //@line 1281 "compiledict.cpp"
        var $51=HEAP8[(((__ZL9rule_cond)&4294967295))]; //@line 1282 "compiledict.cpp"
        var $conv91=(tempInt=(($51)),(tempInt>=128?tempInt-256:tempInt)); //@line 1282 "compiledict.cpp"
        var $cmp92=((($conv91))|0)==33; //@line 1282 "compiledict.cpp"
        if ($cmp92) { __label__ = 30;; } else { __label__ = 31;; } //@line 1282 "compiledict.cpp"
        if (__label__ == 30) {
  
          var $call94=_atoi(((__ZL9rule_cond+1)&4294967295)); //@line 1285 "compiledict.cpp"
          var $add95=((($call94)+32)&4294967295); //@line 1285 "compiledict.cpp"
          $ix=$add95; //@line 1285 "compiledict.cpp"
          ; //@line 1286 "compiledict.cpp"
        }
        else if (__label__ == 31) {
  
          var $call97=_atoi(((__ZL9rule_cond)&4294967295)); //@line 1290 "compiledict.cpp"
          $ix=$call97; //@line 1290 "compiledict.cpp"
          ;
        }
  
        var $52=$ix; //@line 1293 "compiledict.cpp"
        var $cmp99=((($52))|0) > 0; //@line 1293 "compiledict.cpp"
        var $53=$ix; //@line 1293 "compiledict.cpp"
        var $cmp101=((($53))|0) < 255; //@line 1293 "compiledict.cpp"
        var $or_cond=($cmp99) & ($cmp101); //@line 1293 "compiledict.cpp"
        if ($or_cond) { __label__ = 33;; } else { __label__ = 34;; } //@line 1293 "compiledict.cpp"
        if (__label__ == 33) {
  
          var $54=$len; //@line 1295 "compiledict.cpp"
          var $inc103=((($54)+1)&4294967295); //@line 1295 "compiledict.cpp"
          $len=$inc103; //@line 1295 "compiledict.cpp"
          var $arrayidx104=(($output+$54)&4294967295); //@line 1295 "compiledict.cpp"
          HEAP8[($arrayidx104)]=5; //@line 1295 "compiledict.cpp"
          var $55=$ix; //@line 1296 "compiledict.cpp"
          var $conv105=((($55)) & 255); //@line 1296 "compiledict.cpp"
          var $56=$len; //@line 1296 "compiledict.cpp"
          var $inc106=((($56)+1)&4294967295); //@line 1296 "compiledict.cpp"
          $len=$inc106; //@line 1296 "compiledict.cpp"
          var $arrayidx107=(($output+$56)&4294967295); //@line 1296 "compiledict.cpp"
          HEAP8[($arrayidx107)]=$conv105; //@line 1296 "compiledict.cpp"
          ; //@line 1297 "compiledict.cpp"
        }
        else if (__label__ == 34) {
  
          var $57=HEAP32[((__ZL5f_log)>>2)]; //@line 1300 "compiledict.cpp"
          var $58=HEAP32[((__ZL7linenum)>>2)]; //@line 1300 "compiledict.cpp"
          var $59=$ix; //@line 1300 "compiledict.cpp"
          var $call109=_fprintf($57, ((__str95)&4294967295), allocate([$58,0,0,0,$59,0,0,0], ["i32",0,0,0,"i32",0,0,0], ALLOC_STACK)); //@line 1300 "compiledict.cpp"
          var $60=HEAP32[((__ZL11error_count)>>2)]; //@line 1301 "compiledict.cpp"
          var $inc110=((($60)+1)&4294967295); //@line 1301 "compiledict.cpp"
          HEAP32[((__ZL11error_count)>>2)]=$inc110; //@line 1301 "compiledict.cpp"
          ;
        }
      }
  
      var $61=HEAP8[(((__ZL8rule_pre)&4294967295))]; //@line 1304 "compiledict.cpp"
      var $conv113=(tempInt=(($61)),(tempInt>=128?tempInt-256:tempInt)); //@line 1304 "compiledict.cpp"
      var $cmp114=((($conv113))|0)!=0; //@line 1304 "compiledict.cpp"
      if ($cmp114) { __label__ = 36;; } else { __label__ = 41;; } //@line 1304 "compiledict.cpp"
      $if_then115$$if_end132$51: do { 
        if (__label__ == 36) {
  
          $start=0; //@line 1306 "compiledict.cpp"
          var $62=HEAP8[(((__ZL8rule_pre)&4294967295))]; //@line 1307 "compiledict.cpp"
          var $conv116=(tempInt=(($62)),(tempInt>=128?tempInt-256:tempInt)); //@line 1307 "compiledict.cpp"
          var $cmp117=((($conv116))|0)==32; //@line 1307 "compiledict.cpp"
          if ($cmp117) { __label__ = 37;; } else { __label__ = 38;; } //@line 1307 "compiledict.cpp"
          if (__label__ == 37) {
  
            $c=8; //@line 1310 "compiledict.cpp"
            $start=1; //@line 1311 "compiledict.cpp"
            ; //@line 1312 "compiledict.cpp"
          }
          else if (__label__ == 38) {
  
            $c=1; //@line 1315 "compiledict.cpp"
            ;
          }
  
          var $63=$c; //@line 1317 "compiledict.cpp"
          var $64=$len; //@line 1317 "compiledict.cpp"
          var $inc121=((($64)+1)&4294967295); //@line 1317 "compiledict.cpp"
          $len=$inc121; //@line 1317 "compiledict.cpp"
          var $arrayidx122=(($output+$64)&4294967295); //@line 1317 "compiledict.cpp"
          HEAP8[($arrayidx122)]=$63; //@line 1317 "compiledict.cpp"
          var $call123=_strlen(((__ZL8rule_pre)&4294967295)); //@line 1320 "compiledict.cpp"
          var $sub=((($call123)-1)&4294967295); //@line 1320 "compiledict.cpp"
          $ix=$sub; //@line 1320 "compiledict.cpp"
          var $65=$ix; //@line 1320 "compiledict.cpp"
          var $66=$start; //@line 1320 "compiledict.cpp"
          var $cmp1251=((($65))|0) >= ((($66))|0); //@line 1320 "compiledict.cpp"
          if (!($cmp1251)) { __label__ = 41;break $if_then115$$if_end132$51; } //@line 1320 "compiledict.cpp"
          while(1) { 
  
            var $67=$ix; //@line 1321 "compiledict.cpp"
            var $arrayidx127=((__ZL8rule_pre+$67)&4294967295); //@line 1321 "compiledict.cpp"
            var $68=HEAP8[($arrayidx127)]; //@line 1321 "compiledict.cpp"
            var $69=$len; //@line 1321 "compiledict.cpp"
            var $inc128=((($69)+1)&4294967295); //@line 1321 "compiledict.cpp"
            $len=$inc128; //@line 1321 "compiledict.cpp"
            var $arrayidx129=(($output+$69)&4294967295); //@line 1321 "compiledict.cpp"
            HEAP8[($arrayidx129)]=$68; //@line 1321 "compiledict.cpp"
            var $70=$ix; //@line 1320 "compiledict.cpp"
            var $dec=((($70)-1)&4294967295); //@line 1320 "compiledict.cpp"
            $ix=$dec; //@line 1320 "compiledict.cpp"
            var $71=$ix; //@line 1320 "compiledict.cpp"
            var $72=$start; //@line 1320 "compiledict.cpp"
            var $cmp125=((($71))|0) >= ((($72))|0); //@line 1320 "compiledict.cpp"
            if ($cmp125) { __label__ = 40;continue ; } else { __label__ = 41;break $if_then115$$if_end132$51; } //@line 1320 "compiledict.cpp"
          }
        }
      } while(0);
  
      var $73=HEAP8[(((__ZL9rule_post)&4294967295))]; //@line 1324 "compiledict.cpp"
      var $conv133=(tempInt=(($73)),(tempInt>=128?tempInt-256:tempInt)); //@line 1324 "compiledict.cpp"
      var $cmp134=((($conv133))|0)!=0; //@line 1324 "compiledict.cpp"
      if ($cmp134) { __label__ = 42;; } else { __label__ = 43;; } //@line 1324 "compiledict.cpp"
      if (__label__ == 42) {
  
        var $74=$len; //@line 1326 "compiledict.cpp"
        var $arrayidx136=(($output+$74)&4294967295); //@line 1326 "compiledict.cpp"
        var $call137=_sprintf($arrayidx136, ((__str70722)&4294967295), allocate([2,0,0,0,((__ZL9rule_post)&4294967295),0,0,0], ["i32",0,0,0,"i8*",0,0,0], ALLOC_STACK)); //@line 1326 "compiledict.cpp"
        var $call138=_strlen(((__ZL9rule_post)&4294967295)); //@line 1327 "compiledict.cpp"
        var $75=$len; //@line 1327 "compiledict.cpp"
        var $add139=((($call138)+1)&4294967295); //@line 1327 "compiledict.cpp"
        var $add140=((($add139)+($75))&4294967295); //@line 1327 "compiledict.cpp"
        $len=$add140; //@line 1327 "compiledict.cpp"
        ; //@line 1328 "compiledict.cpp"
      }
  
      var $76=$len; //@line 1329 "compiledict.cpp"
      var $inc142=((($76)+1)&4294967295); //@line 1329 "compiledict.cpp"
      $len=$inc142; //@line 1329 "compiledict.cpp"
      var $arrayidx143=(($output+$76)&4294967295); //@line 1329 "compiledict.cpp"
      HEAP8[($arrayidx143)]=0; //@line 1329 "compiledict.cpp"
      var $77=$len; //@line 1330 "compiledict.cpp"
      var $call144=_malloc($77); //@line 1330 "compiledict.cpp"
      $prule=$call144; //@line 1330 "compiledict.cpp"
      var $78=$prule; //@line 1331 "compiledict.cpp"
      var $arraydecay145=(($output)&4294967295); //@line 1331 "compiledict.cpp"
      var $79=$len; //@line 1331 "compiledict.cpp"
      var $src$, $dest$, $stop$, $stop4$, $fast$;
      $src$ = $arraydecay145;
      $dest$ = $78;
      $stop$ = $src$ + $79;
      if (($dest$%4) == ($src$%4) && $79 > 8) {
        while ($src$%4 !== 0 && $src$ < $stop$) {
          ; HEAP8[$dest$++] = HEAP8[$src$++];
        }
        $src$ >>= 2;
        $dest$ >>= 2;
        $stop4$ = $stop$ >> 2;
        while ($src$ < $stop4$) {
      ;;;    HEAP32[$dest$++] = HEAP32[$src$++];
        }
        $src$ <<= 2;
        $dest$ <<= 2;
      }while ($src$ < $stop$) {
        ; HEAP8[$dest$++] = HEAP8[$src$++];
      }; //@line 1331 "compiledict.cpp"
      var $80=$prule; //@line 1332 "compiledict.cpp"
      $retval=$80; //@line 1332 "compiledict.cpp"
      ; //@line 1332 "compiledict.cpp"
    }
  
    var $81=$retval; //@line 1333 "compiledict.cpp"
    STACKTOP = __stackBase__;
    return $81; //@line 1333 "compiledict.cpp"
    return null;
  }
  

  function __ZL13rgroup_sorterP6RGROUPS0_($a, $b) {
    ;
    var __label__;
  
    var $retval;
    var $a_addr;
    var $b_addr;
    var $ix;
    $a_addr=$a;
    $b_addr=$b;
    var $0=$b_addr; //@line 1353 "compiledict.cpp"
    var $name=(($0)&4294967295); //@line 1353 "compiledict.cpp"
    var $arraydecay=(($name)&4294967295); //@line 1353 "compiledict.cpp"
    var $call=_strlen($arraydecay); //@line 1353 "compiledict.cpp"
    var $1=$a_addr; //@line 1353 "compiledict.cpp"
    var $name1=(($1)&4294967295); //@line 1353 "compiledict.cpp"
    var $arraydecay2=(($name1)&4294967295); //@line 1353 "compiledict.cpp"
    var $call3=_strlen($arraydecay2); //@line 1353 "compiledict.cpp"
    var $sub=((($call)-($call3))&4294967295); //@line 1353 "compiledict.cpp"
    $ix=$sub; //@line 1353 "compiledict.cpp"
    var $2=$ix; //@line 1354 "compiledict.cpp"
    var $cmp=((($2))|0)!=0; //@line 1354 "compiledict.cpp"
    if ($cmp) { __label__ = 1;; } else { __label__ = 2;; } //@line 1354 "compiledict.cpp"
    if (__label__ == 1) {
  
      var $3=$ix; //@line 1354 "compiledict.cpp"
      $retval=$3; //@line 1354 "compiledict.cpp"
      ; //@line 1354 "compiledict.cpp"
    }
    else if (__label__ == 2) {
  
      var $4=$a_addr; //@line 1355 "compiledict.cpp"
      var $name4=(($4)&4294967295); //@line 1355 "compiledict.cpp"
      var $arraydecay5=(($name4)&4294967295); //@line 1355 "compiledict.cpp"
      var $5=$b_addr; //@line 1355 "compiledict.cpp"
      var $name6=(($5)&4294967295); //@line 1355 "compiledict.cpp"
      var $arraydecay7=(($name6)&4294967295); //@line 1355 "compiledict.cpp"
      var $call8=_strcmp($arraydecay5, $arraydecay7); //@line 1355 "compiledict.cpp"
      $ix=$call8; //@line 1355 "compiledict.cpp"
      var $6=$ix; //@line 1356 "compiledict.cpp"
      var $cmp9=((($6))|0)!=0; //@line 1356 "compiledict.cpp"
      if ($cmp9) { __label__ = 3;; } else { __label__ = 4;; } //@line 1356 "compiledict.cpp"
      if (__label__ == 3) {
  
        var $7=$ix; //@line 1356 "compiledict.cpp"
        $retval=$7; //@line 1356 "compiledict.cpp"
        ; //@line 1356 "compiledict.cpp"
      }
      else if (__label__ == 4) {
  
        var $8=$a_addr; //@line 1357 "compiledict.cpp"
        var $start=(($8+16)&4294967295); //@line 1357 "compiledict.cpp"
        var $9=HEAP32[(($start)>>2)]; //@line 1357 "compiledict.cpp"
        var $10=$b_addr; //@line 1357 "compiledict.cpp"
        var $start12=(($10+16)&4294967295); //@line 1357 "compiledict.cpp"
        var $11=HEAP32[(($start12)>>2)]; //@line 1357 "compiledict.cpp"
        var $sub13=((($9)-($11))&4294967295); //@line 1357 "compiledict.cpp"
        $retval=$sub13; //@line 1357 "compiledict.cpp"
        ; //@line 1357 "compiledict.cpp"
      }
    }
  
    var $12=$retval; //@line 1358 "compiledict.cpp"
    ;
    return $12; //@line 1358 "compiledict.cpp"
    return null;
  }
  

  function __ZL16copy_rule_stringPcRi($string, $state) {
    ;
    var __label__;
  
    var $string_addr;
    var $state_addr;
    var $output;
    var $p;
    var $ix;
    var $len;
    var $c;
    var $sxflags;
    var $value;
    var $literal;
    var $mr;
    $string_addr=$string;
    $state_addr=$state;
    var $0=$string_addr; //@line 930 "compiledict.cpp"
    var $arrayidx=(($0)&4294967295); //@line 930 "compiledict.cpp"
    var $1=HEAP8[($arrayidx)]; //@line 930 "compiledict.cpp"
    var $conv=(tempInt=(($1)),(tempInt>=128?tempInt-256:tempInt)); //@line 930 "compiledict.cpp"
    var $cmp=((($conv))|0)==0; //@line 930 "compiledict.cpp"
    if ($cmp) { __label__ = 75;; } else { __label__ = 1;; } //@line 930 "compiledict.cpp"
    if (__label__ == 1) {
  
      var $2=$state_addr; //@line 932 "compiledict.cpp"
      var $3=HEAP32[(($2)>>2)]; //@line 932 "compiledict.cpp"
      var $arrayidx1=((__ZZL16copy_rule_stringPcRiE6outbuf+$3*4)&4294967295); //@line 932 "compiledict.cpp"
      var $4=HEAP32[(($arrayidx1)>>2)]; //@line 932 "compiledict.cpp"
      $output=$4; //@line 932 "compiledict.cpp"
      var $5=$state_addr; //@line 933 "compiledict.cpp"
      var $6=HEAP32[(($5)>>2)]; //@line 933 "compiledict.cpp"
      var $cmp2=((($6))|0)==4; //@line 933 "compiledict.cpp"
      if ($cmp2) { __label__ = 2;; } else { __label__ = 5;; } //@line 933 "compiledict.cpp"
      if (__label__ == 2) {
  
        var $call=_strlen(((__ZL13rule_phonemes)&4294967295)); //@line 936 "compiledict.cpp"
        $len=$call; //@line 936 "compiledict.cpp"
        var $7=$len; //@line 937 "compiledict.cpp"
        var $cmp4=((($7))|0) > 0; //@line 937 "compiledict.cpp"
        if ($cmp4) { __label__ = 3;; } else { __label__ = 4;; } //@line 937 "compiledict.cpp"
        if (__label__ == 3) {
  
          var $8=$len; //@line 938 "compiledict.cpp"
          var $inc=((($8)+1)&4294967295); //@line 938 "compiledict.cpp"
          $len=$inc; //@line 938 "compiledict.cpp"
          var $arrayidx6=((__ZL13rule_phonemes+$8)&4294967295); //@line 938 "compiledict.cpp"
          HEAP8[($arrayidx6)]=32; //@line 938 "compiledict.cpp"
          ; //@line 938 "compiledict.cpp"
        }
  
        var $9=$len; //@line 939 "compiledict.cpp"
        var $arrayidx8=((__ZL13rule_phonemes+$9)&4294967295); //@line 939 "compiledict.cpp"
        $output=$arrayidx8; //@line 939 "compiledict.cpp"
        ; //@line 940 "compiledict.cpp"
      }
  
      $sxflags=8421376; //@line 941 "compiledict.cpp"
      var $10=$string_addr; //@line 943 "compiledict.cpp"
      $p=$10; //@line 943 "compiledict.cpp"
      $ix=0; //@line 943 "compiledict.cpp"
      ; //@line 943 "compiledict.cpp"
      $for_cond$10: while(1) { 
  
        $literal=0; //@line 945 "compiledict.cpp"
        var $11=$p; //@line 946 "compiledict.cpp"
        var $incdec_ptr=(($11+1)&4294967295); //@line 946 "compiledict.cpp"
        $p=$incdec_ptr; //@line 946 "compiledict.cpp"
        var $12=HEAP8[($11)]; //@line 946 "compiledict.cpp"
        $c=$12; //@line 946 "compiledict.cpp"
        var $13=$c; //@line 947 "compiledict.cpp"
        var $conv10=(tempInt=(($13)),(tempInt>=128?tempInt-256:tempInt)); //@line 947 "compiledict.cpp"
        var $cmp11=((($conv10))|0)==92; //@line 947 "compiledict.cpp"
        if ($cmp11) { __label__ = 7;; } else { __label__ = 15;; } //@line 947 "compiledict.cpp"
        if (__label__ == 7) {
  
          var $14=$p; //@line 949 "compiledict.cpp"
          var $incdec_ptr13=(($14+1)&4294967295); //@line 949 "compiledict.cpp"
          $p=$incdec_ptr13; //@line 949 "compiledict.cpp"
          var $15=HEAP8[($14)]; //@line 949 "compiledict.cpp"
          $c=$15; //@line 949 "compiledict.cpp"
          var $16=$c; //@line 950 "compiledict.cpp"
          var $conv14=(tempInt=(($16)),(tempInt>=128?tempInt-256:tempInt)); //@line 950 "compiledict.cpp"
          var $cmp15=((($conv14))|0) >= 48; //@line 950 "compiledict.cpp"
          if ($cmp15) { __label__ = 8;; } else { __label__ = 14;; } //@line 950 "compiledict.cpp"
          $land_lhs_true$$if_end45$14: do { 
            if (__label__ == 8) {
  
              var $17=$c; //@line 950 "compiledict.cpp"
              var $conv16=(tempInt=(($17)),(tempInt>=128?tempInt-256:tempInt)); //@line 950 "compiledict.cpp"
              var $cmp17=((($conv16))|0) <= 51; //@line 950 "compiledict.cpp"
              if (!($cmp17)) { __label__ = 14;break $land_lhs_true$$if_end45$14; } //@line 950 "compiledict.cpp"
  
              var $18=$p; //@line 950 "compiledict.cpp"
              var $arrayidx19=(($18)&4294967295); //@line 950 "compiledict.cpp"
              var $19=HEAP8[($arrayidx19)]; //@line 950 "compiledict.cpp"
              var $conv20=(tempInt=(($19)),(tempInt>=128?tempInt-256:tempInt)); //@line 950 "compiledict.cpp"
              var $cmp21=((($conv20))|0) >= 48; //@line 950 "compiledict.cpp"
              if (!($cmp21)) { __label__ = 14;break $land_lhs_true$$if_end45$14; } //@line 950 "compiledict.cpp"
  
              var $20=$p; //@line 950 "compiledict.cpp"
              var $arrayidx23=(($20)&4294967295); //@line 950 "compiledict.cpp"
              var $21=HEAP8[($arrayidx23)]; //@line 950 "compiledict.cpp"
              var $conv24=(tempInt=(($21)),(tempInt>=128?tempInt-256:tempInt)); //@line 950 "compiledict.cpp"
              var $cmp25=((($conv24))|0) <= 55; //@line 950 "compiledict.cpp"
              if (!($cmp25)) { __label__ = 14;break $land_lhs_true$$if_end45$14; } //@line 950 "compiledict.cpp"
  
              var $22=$p; //@line 950 "compiledict.cpp"
              var $arrayidx27=(($22+1)&4294967295); //@line 950 "compiledict.cpp"
              var $23=HEAP8[($arrayidx27)]; //@line 950 "compiledict.cpp"
              var $conv28=(tempInt=(($23)),(tempInt>=128?tempInt-256:tempInt)); //@line 950 "compiledict.cpp"
              var $cmp29=((($conv28))|0) >= 48; //@line 950 "compiledict.cpp"
              if (!($cmp29)) { __label__ = 14;break $land_lhs_true$$if_end45$14; } //@line 950 "compiledict.cpp"
  
              var $24=$p; //@line 950 "compiledict.cpp"
              var $arrayidx31=(($24+1)&4294967295); //@line 950 "compiledict.cpp"
              var $25=HEAP8[($arrayidx31)]; //@line 950 "compiledict.cpp"
              var $conv32=(tempInt=(($25)),(tempInt>=128?tempInt-256:tempInt)); //@line 950 "compiledict.cpp"
              var $cmp33=((($conv32))|0) <= 55; //@line 950 "compiledict.cpp"
              if (!($cmp33)) { __label__ = 14;break $land_lhs_true$$if_end45$14; } //@line 950 "compiledict.cpp"
  
              var $26=$c; //@line 953 "compiledict.cpp"
              var $conv35=(tempInt=(($26)),(tempInt>=128?tempInt-256:tempInt)); //@line 953 "compiledict.cpp"
              var $sub=((($conv35)-48)&4294967295); //@line 953 "compiledict.cpp"
              var $mul=((($sub)*64)&4294967295); //@line 953 "compiledict.cpp"
              var $27=$p; //@line 953 "compiledict.cpp"
              var $arrayidx36=(($27)&4294967295); //@line 953 "compiledict.cpp"
              var $28=HEAP8[($arrayidx36)]; //@line 953 "compiledict.cpp"
              var $conv37=(tempInt=(($28)),(tempInt>=128?tempInt-256:tempInt)); //@line 953 "compiledict.cpp"
              var $sub38=((($conv37)-48)&4294967295); //@line 953 "compiledict.cpp"
              var $mul39=((($sub38)*8)&4294967295); //@line 953 "compiledict.cpp"
              var $29=$p; //@line 953 "compiledict.cpp"
              var $arrayidx40=(($29+1)&4294967295); //@line 953 "compiledict.cpp"
              var $30=HEAP8[($arrayidx40)]; //@line 953 "compiledict.cpp"
              var $conv41=(tempInt=(($30)),(tempInt>=128?tempInt-256:tempInt)); //@line 953 "compiledict.cpp"
              var $add=((($mul)-48)&4294967295); //@line 953 "compiledict.cpp"
              var $sub42=((($add)+($conv41))&4294967295); //@line 953 "compiledict.cpp"
              var $add43=((($sub42)+($mul39))&4294967295); //@line 953 "compiledict.cpp"
              var $conv44=((($add43)) & 255); //@line 953 "compiledict.cpp"
              $c=$conv44; //@line 953 "compiledict.cpp"
              var $31=$p; //@line 954 "compiledict.cpp"
              var $add_ptr=(($31+2)&4294967295); //@line 954 "compiledict.cpp"
              $p=$add_ptr; //@line 954 "compiledict.cpp"
              ; //@line 955 "compiledict.cpp"
            }
          } while(0);
  
          $literal=1; //@line 956 "compiledict.cpp"
          ; //@line 957 "compiledict.cpp"
        }
  
        var $32=$state_addr; //@line 959 "compiledict.cpp"
        var $33=HEAP32[(($32)>>2)]; //@line 959 "compiledict.cpp"
        var $cmp47=((($33))|0)==1; //@line 959 "compiledict.cpp"
        if ($cmp47) { __label__ = 17;; } else { __label__ = 16;; } //@line 959 "compiledict.cpp"
        $if_then49$$lor_lhs_false$23: do { 
          if (__label__ == 16) {
  
            var $34=$state_addr; //@line 959 "compiledict.cpp"
            var $35=HEAP32[(($34)>>2)]; //@line 959 "compiledict.cpp"
            var $cmp48=((($35))|0)==3; //@line 959 "compiledict.cpp"
            if ($cmp48) { __label__ = 17;break $if_then49$$lor_lhs_false$23; } else { __label__ = 73;break $if_then49$$lor_lhs_false$23; } //@line 959 "compiledict.cpp"
          }
        } while(0);
        $if_then49$$if_end212$25: do { 
          if (__label__ == 17) {
  
            var $36=$literal; //@line 962 "compiledict.cpp"
            var $cmp50=((($36))|0)==0; //@line 962 "compiledict.cpp"
            if (!($cmp50)) { __label__ = 73;break $if_then49$$if_end212$25; } //@line 962 "compiledict.cpp"
  
            var $37=$c; //@line 965 "compiledict.cpp"
            var $conv52=(tempInt=(($37)),(tempInt>=128?tempInt-256:tempInt)); //@line 965 "compiledict.cpp"
            if ($conv52 == 95) {
              __label__ = 19;;
            }
            else if ($conv52 == 89) {
              __label__ = 20;;
            }
            else if ($conv52 == 65) {
              __label__ = 21;;
            }
            else if ($conv52 == 66) {
              __label__ = 21;;
            }
            else if ($conv52 == 67) {
              __label__ = 21;;
            }
            else if ($conv52 == 72) {
              __label__ = 21;;
            }
            else if ($conv52 == 70) {
              __label__ = 21;;
            }
            else if ($conv52 == 71) {
              __label__ = 21;;
            }
            else if ($conv52 == 68) {
              __label__ = 24;;
            }
            else if ($conv52 == 75) {
              __label__ = 25;;
            }
            else if ($conv52 == 78) {
              __label__ = 26;;
            }
            else if ($conv52 == 86) {
              __label__ = 27;;
            }
            else if ($conv52 == 90) {
              __label__ = 28;;
            }
            else if ($conv52 == 43) {
              __label__ = 29;;
            }
            else if ($conv52 == 64) {
              __label__ = 30;;
            }
            else if ($conv52 == 38) {
              __label__ = 31;;
            }
            else if ($conv52 == 37) {
              __label__ = 32;;
            }
            else if ($conv52 == 35) {
              __label__ = 33;;
            }
            else if ($conv52 == 33) {
              __label__ = 34;;
            }
            else if ($conv52 == 84) {
              __label__ = 35;;
            }
            else if ($conv52 == 87) {
              __label__ = 36;;
            }
            else if ($conv52 == 88) {
              __label__ = 37;;
            }
            else if ($conv52 == 74) {
              __label__ = 38;;
            }
            else if ($conv52 == 76) {
              __label__ = 39;;
            }
            else if ($conv52 == 36) {
              __label__ = 48;;
            }
            else if ($conv52 == 80) {
              __label__ = 55;;
            }
            else if ($conv52 == 83) {
              __label__ = 56;;
            }
            else {
            __label__ = 73;break $if_then49$$if_end212$25;
            }
            
            $sw_bb$$sw_bb53$$sw_bb54$$sw_bb74$$sw_bb75$$sw_bb76$$sw_bb77$$sw_bb78$$sw_bb79$$sw_bb80$$sw_bb81$$sw_bb82$$sw_bb83$$sw_bb84$$sw_bb85$$sw_bb88$$sw_bb89$$sw_bb90$$sw_bb91$$sw_bb137$$sw_bb158$$sw_bb159$28: do { 
              if (__label__ == 19) {
  
                $c=32; //@line 968 "compiledict.cpp"
                __label__ = 73;break $if_then49$$if_end212$25; //@line 969 "compiledict.cpp"
              }
              else if (__label__ == 20) {
  
                $c=73; //@line 972 "compiledict.cpp"
                __label__ = 21;break $sw_bb$$sw_bb53$$sw_bb54$$sw_bb74$$sw_bb75$$sw_bb76$$sw_bb77$$sw_bb78$$sw_bb79$$sw_bb80$$sw_bb81$$sw_bb82$$sw_bb83$$sw_bb84$$sw_bb85$$sw_bb88$$sw_bb89$$sw_bb90$$sw_bb91$$sw_bb137$$sw_bb158$$sw_bb159$28; //@line 972 "compiledict.cpp"
              }
              else if (__label__ == 24) {
  
                $c=15; //@line 992 "compiledict.cpp"
                __label__ = 73;break $if_then49$$if_end212$25; //@line 993 "compiledict.cpp"
              }
              else if (__label__ == 25) {
  
                $c=25; //@line 995 "compiledict.cpp"
                __label__ = 73;break $if_then49$$if_end212$25; //@line 996 "compiledict.cpp"
              }
              else if (__label__ == 26) {
  
                $c=24; //@line 998 "compiledict.cpp"
                __label__ = 73;break $if_then49$$if_end212$25; //@line 999 "compiledict.cpp"
              }
              else if (__label__ == 27) {
  
                $c=26; //@line 1001 "compiledict.cpp"
                __label__ = 73;break $if_then49$$if_end212$25; //@line 1002 "compiledict.cpp"
              }
              else if (__label__ == 28) {
  
                $c=16; //@line 1004 "compiledict.cpp"
                __label__ = 73;break $if_then49$$if_end212$25; //@line 1005 "compiledict.cpp"
              }
              else if (__label__ == 29) {
  
                $c=12; //@line 1007 "compiledict.cpp"
                __label__ = 73;break $if_then49$$if_end212$25; //@line 1008 "compiledict.cpp"
              }
              else if (__label__ == 30) {
  
                $c=21; //@line 1010 "compiledict.cpp"
                __label__ = 73;break $if_then49$$if_end212$25; //@line 1011 "compiledict.cpp"
              }
              else if (__label__ == 31) {
  
                $c=10; //@line 1013 "compiledict.cpp"
                __label__ = 73;break $if_then49$$if_end212$25; //@line 1014 "compiledict.cpp"
              }
              else if (__label__ == 32) {
  
                $c=11; //@line 1016 "compiledict.cpp"
                __label__ = 73;break $if_then49$$if_end212$25; //@line 1017 "compiledict.cpp"
              }
              else if (__label__ == 33) {
  
                $c=13; //@line 1019 "compiledict.cpp"
                __label__ = 73;break $if_then49$$if_end212$25; //@line 1020 "compiledict.cpp"
              }
              else if (__label__ == 34) {
  
                $c=19; //@line 1022 "compiledict.cpp"
                __label__ = 73;break $if_then49$$if_end212$25; //@line 1023 "compiledict.cpp"
              }
              else if (__label__ == 35) {
  
                var $48=$ix; //@line 1025 "compiledict.cpp"
                var $inc86=((($48)+1)&4294967295); //@line 1025 "compiledict.cpp"
                $ix=$inc86; //@line 1025 "compiledict.cpp"
                var $49=$output; //@line 1025 "compiledict.cpp"
                var $arrayidx87=(($49+$48)&4294967295); //@line 1025 "compiledict.cpp"
                HEAP8[($arrayidx87)]=28; //@line 1025 "compiledict.cpp"
                $c=17; //@line 1026 "compiledict.cpp"
                __label__ = 73;break $if_then49$$if_end212$25; //@line 1027 "compiledict.cpp"
              }
              else if (__label__ == 36) {
  
                $c=31; //@line 1029 "compiledict.cpp"
                __label__ = 73;break $if_then49$$if_end212$25; //@line 1030 "compiledict.cpp"
              }
              else if (__label__ == 37) {
  
                $c=29; //@line 1032 "compiledict.cpp"
                __label__ = 73;break $if_then49$$if_end212$25; //@line 1033 "compiledict.cpp"
              }
              else if (__label__ == 38) {
  
                $c=23; //@line 1035 "compiledict.cpp"
                __label__ = 73;break $if_then49$$if_end212$25; //@line 1036 "compiledict.cpp"
              }
              else if (__label__ == 39) {
  
                var $50=$p; //@line 1039 "compiledict.cpp"
                var $incdec_ptr92=(($50+1)&4294967295); //@line 1039 "compiledict.cpp"
                $p=$incdec_ptr92; //@line 1039 "compiledict.cpp"
                var $51=HEAP8[($50)]; //@line 1039 "compiledict.cpp"
                var $conv93=(tempInt=(($51)),(tempInt>=128?tempInt-256:tempInt)); //@line 1039 "compiledict.cpp"
                var $sub94=((($conv93)-48)&4294967295); //@line 1039 "compiledict.cpp"
                var $conv95=((($sub94)) & 255); //@line 1039 "compiledict.cpp"
                $c=$conv95; //@line 1039 "compiledict.cpp"
                var $52=$p; //@line 1040 "compiledict.cpp"
                var $incdec_ptr96=(($52+1)&4294967295); //@line 1040 "compiledict.cpp"
                $p=$incdec_ptr96; //@line 1040 "compiledict.cpp"
                var $53=HEAP8[($52)]; //@line 1040 "compiledict.cpp"
                var $conv97=(tempInt=(($53)),(tempInt>=128?tempInt-256:tempInt)); //@line 1040 "compiledict.cpp"
                var $sub98=((($conv97)-48)&4294967295); //@line 1040 "compiledict.cpp"
                $value=$sub98; //@line 1040 "compiledict.cpp"
                var $54=$c; //@line 1041 "compiledict.cpp"
                var $conv99=(tempInt=(($54)),(tempInt>=128?tempInt-256:tempInt)); //@line 1041 "compiledict.cpp"
                var $mul100=((($conv99)*10)&4294967295); //@line 1041 "compiledict.cpp"
                var $55=$value; //@line 1041 "compiledict.cpp"
                var $add101=((($mul100)+($55))&4294967295); //@line 1041 "compiledict.cpp"
                var $conv102=((($add101)) & 255); //@line 1041 "compiledict.cpp"
                $c=$conv102; //@line 1041 "compiledict.cpp"
                var $56=$value; //@line 1042 "compiledict.cpp"
                var $cmp103=((($56))|0) < 0; //@line 1042 "compiledict.cpp"
                var $57=$value; //@line 1042 "compiledict.cpp"
                var $cmp105=((($57))|0) > 9; //@line 1042 "compiledict.cpp"
                var $or_cond=($cmp103) | ($cmp105); //@line 1042 "compiledict.cpp"
                if ($or_cond) { __label__ = 40;; } else { __label__ = 41;; } //@line 1042 "compiledict.cpp"
                $if_then106$$if_else109$47: do { 
                  if (__label__ == 40) {
  
                    $c=0; //@line 1044 "compiledict.cpp"
                    var $58=HEAP32[((__ZL5f_log)>>2)]; //@line 1045 "compiledict.cpp"
                    var $59=HEAP32[((__ZL7linenum)>>2)]; //@line 1045 "compiledict.cpp"
                    var $call107=_fprintf($58, ((__str97)&4294967295), allocate([$59,0,0,0], ["i32",0,0,0], ALLOC_STACK)); //@line 1045 "compiledict.cpp"
                    var $60=HEAP32[((__ZL11error_count)>>2)]; //@line 1046 "compiledict.cpp"
                    var $inc108=((($60)+1)&4294967295); //@line 1046 "compiledict.cpp"
                    HEAP32[((__ZL11error_count)>>2)]=$inc108; //@line 1046 "compiledict.cpp"
                    ; //@line 1047 "compiledict.cpp"
                  }
                  else if (__label__ == 41) {
  
                    var $61=$c; //@line 1049 "compiledict.cpp"
                    var $conv110=(tempInt=(($61)),(tempInt>=128?tempInt-256:tempInt)); //@line 1049 "compiledict.cpp"
                    var $cmp111=((($conv110))|0) <= 0; //@line 1049 "compiledict.cpp"
                    if ($cmp111) { __label__ = 44;; } else { __label__ = 42;; } //@line 1049 "compiledict.cpp"
                    $if_then120$$lor_lhs_false112$50: do { 
                      if (__label__ == 42) {
  
                        var $62=$c; //@line 1049 "compiledict.cpp"
                        var $conv113=(tempInt=(($62)),(tempInt>=128?tempInt-256:tempInt)); //@line 1049 "compiledict.cpp"
                        var $cmp114=((($conv113))|0) >= 26; //@line 1049 "compiledict.cpp"
                        if ($cmp114) { __label__ = 44;break $if_then120$$lor_lhs_false112$50; } //@line 1049 "compiledict.cpp"
  
                        var $63=$c; //@line 1049 "compiledict.cpp"
                        var $conv116=(tempInt=(($63)),(tempInt>=128?tempInt-256:tempInt)); //@line 1049 "compiledict.cpp"
                        var $arrayidx117=((__ZL19letterGroupsDefined+$conv116)&4294967295); //@line 1049 "compiledict.cpp"
                        var $64=HEAP8[($arrayidx117)]; //@line 1049 "compiledict.cpp"
                        var $conv118=(tempInt=(($64)),(tempInt>=128?tempInt-256:tempInt)); //@line 1049 "compiledict.cpp"
                        var $cmp119=((($conv118))|0)==0; //@line 1049 "compiledict.cpp"
                        if (!($cmp119)) { __label__ = 45;break $if_then106$$if_else109$47; } //@line 1049 "compiledict.cpp"
                      }
                    } while(0);
  
                    var $65=HEAP32[((__ZL5f_log)>>2)]; //@line 1051 "compiledict.cpp"
                    var $66=HEAP32[((__ZL7linenum)>>2)]; //@line 1051 "compiledict.cpp"
                    var $67=$c; //@line 1051 "compiledict.cpp"
                    var $conv121=(tempInt=(($67)),(tempInt>=128?tempInt-256:tempInt)); //@line 1051 "compiledict.cpp"
                    var $call122=_fprintf($65, ((__str98)&4294967295), allocate([$66,0,0,0,$conv121,0,0,0], ["i32",0,0,0,"i32",0,0,0], ALLOC_STACK)); //@line 1051 "compiledict.cpp"
                    var $68=HEAP32[((__ZL11error_count)>>2)]; //@line 1052 "compiledict.cpp"
                    var $inc123=((($68)+1)&4294967295); //@line 1052 "compiledict.cpp"
                    HEAP32[((__ZL11error_count)>>2)]=$inc123; //@line 1052 "compiledict.cpp"
                    ; //@line 1053 "compiledict.cpp"
                  }
                } while(0);
  
                var $69=$c; //@line 1054 "compiledict.cpp"
                var $conv126=(tempInt=(($69)),(tempInt>=128?tempInt-256:tempInt)); //@line 1054 "compiledict.cpp"
                var $add127=((($conv126)+65)&4294967295); //@line 1054 "compiledict.cpp"
                var $conv128=((($add127)) & 255); //@line 1054 "compiledict.cpp"
                $c=$conv128; //@line 1054 "compiledict.cpp"
                var $70=$state_addr; //@line 1055 "compiledict.cpp"
                var $71=HEAP32[(($70)>>2)]; //@line 1055 "compiledict.cpp"
                var $cmp129=((($71))|0)==1; //@line 1055 "compiledict.cpp"
                if ($cmp129) { __label__ = 46;; } else { __label__ = 47;; } //@line 1055 "compiledict.cpp"
                if (__label__ == 46) {
  
                  var $72=$c; //@line 1058 "compiledict.cpp"
                  var $73=$ix; //@line 1058 "compiledict.cpp"
                  var $inc131=((($73)+1)&4294967295); //@line 1058 "compiledict.cpp"
                  $ix=$inc131; //@line 1058 "compiledict.cpp"
                  var $74=$output; //@line 1058 "compiledict.cpp"
                  var $arrayidx132=(($74+$73)&4294967295); //@line 1058 "compiledict.cpp"
                  HEAP8[($arrayidx132)]=$72; //@line 1058 "compiledict.cpp"
                  $c=18; //@line 1059 "compiledict.cpp"
                  __label__ = 73;break $if_then49$$if_end212$25; //@line 1060 "compiledict.cpp"
                }
                else if (__label__ == 47) {
  
                  var $75=$ix; //@line 1063 "compiledict.cpp"
                  var $inc134=((($75)+1)&4294967295); //@line 1063 "compiledict.cpp"
                  $ix=$inc134; //@line 1063 "compiledict.cpp"
                  var $76=$output; //@line 1063 "compiledict.cpp"
                  var $arrayidx135=(($76+$75)&4294967295); //@line 1063 "compiledict.cpp"
                  HEAP8[($arrayidx135)]=18; //@line 1063 "compiledict.cpp"
                  __label__ = 73;break $if_then49$$if_end212$25;
                }
              }
              else if (__label__ == 48) {
  
                var $77=$ix; //@line 1068 "compiledict.cpp"
                var $inc138=((($77)+1)&4294967295); //@line 1068 "compiledict.cpp"
                $ix=$inc138; //@line 1068 "compiledict.cpp"
                var $78=$output; //@line 1068 "compiledict.cpp"
                var $arrayidx139=(($78+$77)&4294967295); //@line 1068 "compiledict.cpp"
                HEAP8[($arrayidx139)]=28; //@line 1068 "compiledict.cpp"
                $c=0; //@line 1069 "compiledict.cpp"
                $mr=((_mnem_rules)&4294967295); //@line 1070 "compiledict.cpp"
                ; //@line 1071 "compiledict.cpp"
                while(1) { 
  
                  var $79=$mr; //@line 1071 "compiledict.cpp"
                  var $mnem=(($79)&4294967295); //@line 1071 "compiledict.cpp"
                  var $80=HEAP32[(($mnem)>>2)]; //@line 1071 "compiledict.cpp"
                  var $cmp140=((($80))|0)!=0; //@line 1071 "compiledict.cpp"
                  if (!($cmp140)) { __label__ = 53;break ; } //@line 1071 "compiledict.cpp"
  
                  var $81=$mr; //@line 1073 "compiledict.cpp"
                  var $mnem141=(($81)&4294967295); //@line 1073 "compiledict.cpp"
                  var $82=HEAP32[(($mnem141)>>2)]; //@line 1073 "compiledict.cpp"
                  var $call142=_strlen($82); //@line 1073 "compiledict.cpp"
                  $len=$call142; //@line 1073 "compiledict.cpp"
                  var $83=$p; //@line 1074 "compiledict.cpp"
                  var $84=$mr; //@line 1074 "compiledict.cpp"
                  var $mnem143=(($84)&4294967295); //@line 1074 "compiledict.cpp"
                  var $85=HEAP32[(($mnem143)>>2)]; //@line 1074 "compiledict.cpp"
                  var $86=$len; //@line 1074 "compiledict.cpp"
                  var $call144=_memcmp($83, $85, $86); //@line 1074 "compiledict.cpp"
                  var $cmp145=((($call144))|0)==0; //@line 1074 "compiledict.cpp"
                  var $87=$mr; //@line 1076 "compiledict.cpp"
                  if ($cmp145) { __label__ = 51;break ; } //@line 1074 "compiledict.cpp"
  
                  var $incdec_ptr151=(($87+8)&4294967295); //@line 1080 "compiledict.cpp"
                  $mr=$incdec_ptr151; //@line 1080 "compiledict.cpp"
                  __label__ = 49;continue ; //@line 1081 "compiledict.cpp"
                }
                if (__label__ == 51) {
  
                  var $value147=(($87+4)&4294967295); //@line 1076 "compiledict.cpp"
                  var $88=HEAP32[(($value147)>>2)]; //@line 1076 "compiledict.cpp"
                  var $conv148=((($88)) & 255); //@line 1076 "compiledict.cpp"
                  $c=$conv148; //@line 1076 "compiledict.cpp"
                  var $89=$len; //@line 1077 "compiledict.cpp"
                  var $90=$p; //@line 1077 "compiledict.cpp"
                  var $add_ptr149=(($90+$89)&4294967295); //@line 1077 "compiledict.cpp"
                  $p=$add_ptr149; //@line 1077 "compiledict.cpp"
                  ; //@line 1078 "compiledict.cpp"
                }
  
                var $91=$c; //@line 1082 "compiledict.cpp"
                var $conv152=(tempInt=(($91)),(tempInt>=128?tempInt-256:tempInt)); //@line 1082 "compiledict.cpp"
                var $cmp153=((($conv152))|0)==0; //@line 1082 "compiledict.cpp"
                if (!($cmp153)) { __label__ = 73;break $if_then49$$if_end212$25; } //@line 1082 "compiledict.cpp"
  
                var $92=HEAP32[((__ZL5f_log)>>2)]; //@line 1084 "compiledict.cpp"
                var $93=HEAP32[((__ZL7linenum)>>2)]; //@line 1084 "compiledict.cpp"
                var $call155=_fprintf($92, ((__str99)&4294967295), allocate([$93,0,0,0], ["i32",0,0,0], ALLOC_STACK)); //@line 1084 "compiledict.cpp"
                var $94=HEAP32[((__ZL11error_count)>>2)]; //@line 1085 "compiledict.cpp"
                var $inc156=((($94)+1)&4294967295); //@line 1085 "compiledict.cpp"
                HEAP32[((__ZL11error_count)>>2)]=$inc156; //@line 1085 "compiledict.cpp"
                __label__ = 73;break $if_then49$$if_end212$25; //@line 1086 "compiledict.cpp"
              }
              else if (__label__ == 55) {
  
                var $95=$sxflags; //@line 1090 "compiledict.cpp"
                var $or=($95) | 1024; //@line 1090 "compiledict.cpp"
                $sxflags=$or; //@line 1090 "compiledict.cpp"
                __label__ = 56;break $sw_bb$$sw_bb53$$sw_bb54$$sw_bb74$$sw_bb75$$sw_bb76$$sw_bb77$$sw_bb78$$sw_bb79$$sw_bb80$$sw_bb81$$sw_bb82$$sw_bb83$$sw_bb84$$sw_bb85$$sw_bb88$$sw_bb89$$sw_bb90$$sw_bb91$$sw_bb137$$sw_bb158$$sw_bb159$28; //@line 1090 "compiledict.cpp"
              }
            } while(0);
            if (__label__ == 21) {
  
              var $38=$state_addr; //@line 979 "compiledict.cpp"
              var $39=HEAP32[(($38)>>2)]; //@line 979 "compiledict.cpp"
              var $cmp55=((($39))|0)==1; //@line 979 "compiledict.cpp"
              if ($cmp55) { __label__ = 22;; } else { __label__ = 23;; } //@line 979 "compiledict.cpp"
              if (__label__ == 22) {
  
                var $40=$c; //@line 982 "compiledict.cpp"
                var $conv57=(tempInt=(($40)),(tempInt>=128?tempInt-256:tempInt)); //@line 982 "compiledict.cpp"
                var $sub58=((($conv57)-65)&4294967295); //@line 982 "compiledict.cpp"
                var $arrayidx59=((__ZZL16copy_rule_stringPcRiE16lettergp_letters+$sub58)&4294967295); //@line 982 "compiledict.cpp"
                var $41=HEAP8[($arrayidx59)]; //@line 982 "compiledict.cpp"
                var $conv60=(tempInt=(($41)),(tempInt>=128?tempInt-256:tempInt)); //@line 982 "compiledict.cpp"
                var $add61=((($conv60)+65)&4294967295); //@line 982 "compiledict.cpp"
                var $conv62=((($add61)) & 255); //@line 982 "compiledict.cpp"
                var $42=$ix; //@line 982 "compiledict.cpp"
                var $inc63=((($42)+1)&4294967295); //@line 982 "compiledict.cpp"
                $ix=$inc63; //@line 982 "compiledict.cpp"
                var $43=$output; //@line 982 "compiledict.cpp"
                var $arrayidx64=(($43+$42)&4294967295); //@line 982 "compiledict.cpp"
                HEAP8[($arrayidx64)]=$conv62; //@line 982 "compiledict.cpp"
                $c=17; //@line 983 "compiledict.cpp"
                ; //@line 984 "compiledict.cpp"
              }
              else if (__label__ == 23) {
  
                var $44=$ix; //@line 987 "compiledict.cpp"
                var $inc65=((($44)+1)&4294967295); //@line 987 "compiledict.cpp"
                $ix=$inc65; //@line 987 "compiledict.cpp"
                var $45=$output; //@line 987 "compiledict.cpp"
                var $arrayidx66=(($45+$44)&4294967295); //@line 987 "compiledict.cpp"
                HEAP8[($arrayidx66)]=17; //@line 987 "compiledict.cpp"
                var $46=$c; //@line 988 "compiledict.cpp"
                var $conv67=(tempInt=(($46)),(tempInt>=128?tempInt-256:tempInt)); //@line 988 "compiledict.cpp"
                var $sub68=((($conv67)-65)&4294967295); //@line 988 "compiledict.cpp"
                var $arrayidx69=((__ZZL16copy_rule_stringPcRiE16lettergp_letters+$sub68)&4294967295); //@line 988 "compiledict.cpp"
                var $47=HEAP8[($arrayidx69)]; //@line 988 "compiledict.cpp"
                var $conv70=(tempInt=(($47)),(tempInt>=128?tempInt-256:tempInt)); //@line 988 "compiledict.cpp"
                var $add71=((($conv70)+65)&4294967295); //@line 988 "compiledict.cpp"
                var $conv72=((($add71)) & 255); //@line 988 "compiledict.cpp"
                $c=$conv72; //@line 988 "compiledict.cpp"
                ;
              }
            }
            else if (__label__ == 56) {
  
              var $96=$ix; //@line 1092 "compiledict.cpp"
              var $inc160=((($96)+1)&4294967295); //@line 1092 "compiledict.cpp"
              $ix=$inc160; //@line 1092 "compiledict.cpp"
              var $97=$output; //@line 1092 "compiledict.cpp"
              var $arrayidx161=(($97+$96)&4294967295); //@line 1092 "compiledict.cpp"
              HEAP8[($arrayidx161)]=14; //@line 1092 "compiledict.cpp"
              $value=0; //@line 1093 "compiledict.cpp"
              ; //@line 1094 "compiledict.cpp"
              while(1) { 
  
                var $98=$p; //@line 1094 "compiledict.cpp"
                var $incdec_ptr163=(($98+1)&4294967295); //@line 1094 "compiledict.cpp"
                $p=$incdec_ptr163; //@line 1094 "compiledict.cpp"
                var $99=HEAP8[($98)]; //@line 1094 "compiledict.cpp"
                $c=$99; //@line 1094 "compiledict.cpp"
                var $conv164=(tempInt=(($99)),(tempInt>=128?tempInt-256:tempInt)); //@line 1094 "compiledict.cpp"
                var $call165=__Z8isspace2j($conv164); //@line 1094 "compiledict.cpp"
                var $tobool=((($call165))|0)!=0; //@line 1094 "compiledict.cpp"
                if ($tobool) { __label__ = 72;break ; } //@line 1094 "compiledict.cpp"
  
                var $100=$c; //@line 1094 "compiledict.cpp"
                var $conv166=(tempInt=(($100)),(tempInt>=128?tempInt-256:tempInt)); //@line 1094 "compiledict.cpp"
                var $cmp167=((($conv166))|0)!=0; //@line 1094 "compiledict.cpp"
                if (!($cmp167)) { __label__ = 72;break ; }
  
                var $101=$c; //@line 1096 "compiledict.cpp"
                var $conv169=(tempInt=(($101)),(tempInt>=128?tempInt-256:tempInt)); //@line 1096 "compiledict.cpp"
                if ($conv169 == 101) {
                  __label__ = 60;;
                }
                else if ($conv169 == 105) {
                  __label__ = 61;;
                }
                else if ($conv169 == 112) {
                  __label__ = 62;;
                }
                else if ($conv169 == 118) {
                  __label__ = 63;;
                }
                else if ($conv169 == 100) {
                  __label__ = 64;;
                }
                else if ($conv169 == 102) {
                  __label__ = 65;;
                }
                else if ($conv169 == 113) {
                  __label__ = 66;;
                }
                else if ($conv169 == 116) {
                  __label__ = 67;;
                }
                else if ($conv169 == 98) {
                  __label__ = 68;;
                }
                else if ($conv169 == 97) {
                  __label__ = 69;;
                }
                else {
                __label__ = 70;;
                }
                
                if (__label__ == 70) {
  
                  var $112=$c; //@line 1129 "compiledict.cpp"
                  var $conv190=(tempInt=(($112)),(tempInt>=128?tempInt-256:tempInt)); //@line 1129 "compiledict.cpp"
                  var $isdigittmp=((($conv190)-48)&4294967295); //@line 1129 "compiledict.cpp"
                  var $isdigit=((($isdigittmp))>>>0) < 10; //@line 1129 "compiledict.cpp"
                  var $call191=((($isdigit))&1); //@line 1129 "compiledict.cpp"
                  var $tobool192=((($call191))|0)!=0; //@line 1129 "compiledict.cpp"
                  if (!($tobool192)) { __label__ = 57;continue ; } //@line 1129 "compiledict.cpp"
  
                  var $113=$value; //@line 1130 "compiledict.cpp"
                  var $mul194=((($113)*10)&4294967295); //@line 1130 "compiledict.cpp"
                  var $114=$c; //@line 1130 "compiledict.cpp"
                  var $conv195=(tempInt=(($114)),(tempInt>=128?tempInt-256:tempInt)); //@line 1130 "compiledict.cpp"
                  var $sub196=((($mul194)-48)&4294967295); //@line 1130 "compiledict.cpp"
                  var $add197=((($sub196)+($conv195))&4294967295); //@line 1130 "compiledict.cpp"
                  $value=$add197; //@line 1130 "compiledict.cpp"
                  __label__ = 57;continue ; //@line 1130 "compiledict.cpp"
                }
                else if (__label__ == 60) {
  
                  var $102=$sxflags; //@line 1099 "compiledict.cpp"
                  var $or171=($102) | 256; //@line 1099 "compiledict.cpp"
                  $sxflags=$or171; //@line 1099 "compiledict.cpp"
                  __label__ = 57;continue ; //@line 1100 "compiledict.cpp"
                }
                else if (__label__ == 61) {
  
                  var $103=$sxflags; //@line 1102 "compiledict.cpp"
                  var $or173=($103) | 512; //@line 1102 "compiledict.cpp"
                  $sxflags=$or173; //@line 1102 "compiledict.cpp"
                  __label__ = 57;continue ; //@line 1103 "compiledict.cpp"
                }
                else if (__label__ == 62) {
  
                  var $104=$sxflags; //@line 1105 "compiledict.cpp"
                  var $or175=($104) | 1024; //@line 1105 "compiledict.cpp"
                  $sxflags=$or175; //@line 1105 "compiledict.cpp"
                  __label__ = 57;continue ; //@line 1106 "compiledict.cpp"
                }
                else if (__label__ == 63) {
  
                  var $105=$sxflags; //@line 1108 "compiledict.cpp"
                  var $or177=($105) | 2048; //@line 1108 "compiledict.cpp"
                  $sxflags=$or177; //@line 1108 "compiledict.cpp"
                  __label__ = 57;continue ; //@line 1109 "compiledict.cpp"
                }
                else if (__label__ == 64) {
  
                  var $106=$sxflags; //@line 1111 "compiledict.cpp"
                  var $or179=($106) | 4096; //@line 1111 "compiledict.cpp"
                  $sxflags=$or179; //@line 1111 "compiledict.cpp"
                  __label__ = 57;continue ; //@line 1112 "compiledict.cpp"
                }
                else if (__label__ == 65) {
  
                  var $107=$sxflags; //@line 1114 "compiledict.cpp"
                  var $or181=($107) | 8192; //@line 1114 "compiledict.cpp"
                  $sxflags=$or181; //@line 1114 "compiledict.cpp"
                  __label__ = 57;continue ; //@line 1115 "compiledict.cpp"
                }
                else if (__label__ == 66) {
  
                  var $108=$sxflags; //@line 1117 "compiledict.cpp"
                  var $or183=($108) | 16384; //@line 1117 "compiledict.cpp"
                  $sxflags=$or183; //@line 1117 "compiledict.cpp"
                  __label__ = 57;continue ; //@line 1118 "compiledict.cpp"
                }
                else if (__label__ == 67) {
  
                  var $109=$sxflags; //@line 1120 "compiledict.cpp"
                  var $or185=($109) | 65536; //@line 1120 "compiledict.cpp"
                  $sxflags=$or185; //@line 1120 "compiledict.cpp"
                  __label__ = 57;continue ; //@line 1121 "compiledict.cpp"
                }
                else if (__label__ == 68) {
  
                  var $110=$sxflags; //@line 1123 "compiledict.cpp"
                  var $or187=($110) | 131072; //@line 1123 "compiledict.cpp"
                  $sxflags=$or187; //@line 1123 "compiledict.cpp"
                  __label__ = 57;continue ; //@line 1124 "compiledict.cpp"
                }
                else if (__label__ == 69) {
  
                  var $111=$sxflags; //@line 1126 "compiledict.cpp"
                  var $or189=($111) | 262144; //@line 1126 "compiledict.cpp"
                  $sxflags=$or189; //@line 1126 "compiledict.cpp"
                  __label__ = 57;continue ; //@line 1127 "compiledict.cpp"
                }
              }
  
              var $115=$p; //@line 1134 "compiledict.cpp"
              var $incdec_ptr200=(($115+-1)&4294967295); //@line 1134 "compiledict.cpp"
              $p=$incdec_ptr200; //@line 1134 "compiledict.cpp"
              var $116=$sxflags; //@line 1135 "compiledict.cpp"
              var $shr=($116) >> 16; //@line 1135 "compiledict.cpp"
              var $conv201=((($shr)) & 255); //@line 1135 "compiledict.cpp"
              var $117=$ix; //@line 1135 "compiledict.cpp"
              var $inc202=((($117)+1)&4294967295); //@line 1135 "compiledict.cpp"
              $ix=$inc202; //@line 1135 "compiledict.cpp"
              var $118=$output; //@line 1135 "compiledict.cpp"
              var $arrayidx203=(($118+$117)&4294967295); //@line 1135 "compiledict.cpp"
              HEAP8[($arrayidx203)]=$conv201; //@line 1135 "compiledict.cpp"
              var $119=$sxflags; //@line 1136 "compiledict.cpp"
              var $shr204=($119) >> 8; //@line 1136 "compiledict.cpp"
              var $conv205=((($shr204)) & 255); //@line 1136 "compiledict.cpp"
              var $120=$ix; //@line 1136 "compiledict.cpp"
              var $inc206=((($120)+1)&4294967295); //@line 1136 "compiledict.cpp"
              $ix=$inc206; //@line 1136 "compiledict.cpp"
              var $121=$output; //@line 1136 "compiledict.cpp"
              var $arrayidx207=(($121+$120)&4294967295); //@line 1136 "compiledict.cpp"
              HEAP8[($arrayidx207)]=$conv205; //@line 1136 "compiledict.cpp"
              var $122=$value; //@line 1137 "compiledict.cpp"
              var $or208=($122) | 128; //@line 1137 "compiledict.cpp"
              var $conv209=((($or208)) & 255); //@line 1137 "compiledict.cpp"
              $c=$conv209; //@line 1137 "compiledict.cpp"
              ; //@line 1138 "compiledict.cpp"
            }
          }
        } while(0);
  
        var $123=$c; //@line 1142 "compiledict.cpp"
        var $124=$ix; //@line 1142 "compiledict.cpp"
        var $inc213=((($124)+1)&4294967295); //@line 1142 "compiledict.cpp"
        $ix=$inc213; //@line 1142 "compiledict.cpp"
        var $125=$output; //@line 1142 "compiledict.cpp"
        var $arrayidx214=(($125+$124)&4294967295); //@line 1142 "compiledict.cpp"
        HEAP8[($arrayidx214)]=$123; //@line 1142 "compiledict.cpp"
        var $126=$c; //@line 1143 "compiledict.cpp"
        var $conv215=(tempInt=(($126)),(tempInt>=128?tempInt-256:tempInt)); //@line 1143 "compiledict.cpp"
        var $cmp216=((($conv215))|0)==0; //@line 1143 "compiledict.cpp"
        if ($cmp216) { __label__ = 74;break $for_cond$10; } else { __label__ = 6;continue $for_cond$10; } //@line 1143 "compiledict.cpp"
      }
  
      var $127=$state_addr; //@line 1146 "compiledict.cpp"
      var $128=HEAP32[(($127)>>2)]; //@line 1146 "compiledict.cpp"
      var $arrayidx219=((__ZZL16copy_rule_stringPcRiE10next_state+$128*4)&4294967295); //@line 1146 "compiledict.cpp"
      var $129=HEAP32[(($arrayidx219)>>2)]; //@line 1146 "compiledict.cpp"
      var $130=$state_addr; //@line 1146 "compiledict.cpp"
      HEAP32[(($130)>>2)]=$129; //@line 1146 "compiledict.cpp"
      ; //@line 1147 "compiledict.cpp"
    }
  
    ;
    return; //@line 1147 "compiledict.cpp"
    return;
  }
  

  function __ZL12compile_linePcS_Pi($linebuf, $dict_line, $hash) {
    var __stackBase__  = STACKTOP; STACKTOP += 312; _memset(__stackBase__, 0, 312);
    var __label__;
    var __lastLabel__ = null;
  
    var $retval;
    var $linebuf_addr;
    var $dict_line_addr;
    var $hash_addr;
    var $c;
    var $p;
    var $word;
    var $phonetic;
    var $ix;
    var $step;
    var $n_flag_codes;
    var $flag_offset;
    var $length;
    var $multiple_words;
    var $multiple_numeric_hyphen;
    var $multiple_string;
    var $multiple_string_end;
    var $len_word;
    var $len_phonetic;
    var $text_not_phonemes;
    var $wc=__stackBase__;
    var $all_upper_case;
    var $mnemptr;
    var $comment;
    var $flag_codes=__stackBase__+4;
    var $encoded_ph=__stackBase__+104;
    var $bad_phoneme=__stackBase__+304;
    var $c2=__stackBase__+308;
    $linebuf_addr=$linebuf;
    $dict_line_addr=$dict_line;
    $hash_addr=$hash;
    $n_flag_codes=0; //@line 378 "compiledict.cpp"
    $multiple_words=0; //@line 381 "compiledict.cpp"
    $multiple_numeric_hyphen=0; //@line 382 "compiledict.cpp"
    $multiple_string=0; //@line 383 "compiledict.cpp"
    $multiple_string_end=0; //@line 384 "compiledict.cpp"
    $comment=0; //@line 399 "compiledict.cpp"
    $text_not_phonemes=0; //@line 400 "compiledict.cpp"
    $word=((__ZZL12compile_linePcS_PiE10nullstring)&4294967295); //@line 401 "compiledict.cpp"
    $phonetic=((__ZZL12compile_linePcS_PiE10nullstring)&4294967295); //@line 401 "compiledict.cpp"
    var $0=$linebuf_addr; //@line 403 "compiledict.cpp"
    var $call=_memcmp($0, ((__str108)&4294967295), 2); //@line 403 "compiledict.cpp"
    var $cmp=((($call))|0)==0; //@line 403 "compiledict.cpp"
    if ($cmp) { __label__ = 1;; } else { __label__ = 2;; } //@line 403 "compiledict.cpp"
    if (__label__ == 1) {
  
      $step=1; //@line 405 "compiledict.cpp"
      ; //@line 406 "compiledict.cpp"
    }
  
    var $1=$linebuf_addr; //@line 407 "compiledict.cpp"
    $p=$1; //@line 407 "compiledict.cpp"
    $step=0; //@line 426 "compiledict.cpp"
    $c=0; //@line 428 "compiledict.cpp"
    var $2=$c; //@line 429 "compiledict.cpp"
    var $conv13=((($2))&255); //@line 429 "compiledict.cpp"
    var $cmp114=((($conv13))|0)!=10; //@line 429 "compiledict.cpp"
    if ($cmp114) { __label__ = 3;; } else { __label__ = 55;; } //@line 429 "compiledict.cpp"
    $while_body$$while_end147$5: do { 
      if (__label__ == 3) {
        $while_body$6: while(1) { 
  
          var $3=$p; //@line 431 "compiledict.cpp"
          var $4=HEAP8[($3)]; //@line 431 "compiledict.cpp"
          $c=$4; //@line 431 "compiledict.cpp"
          var $5=$c; //@line 433 "compiledict.cpp"
          var $conv2=((($5))&255); //@line 433 "compiledict.cpp"
          var $cmp3=((($conv2))|0)==63; //@line 433 "compiledict.cpp"
          if ($cmp3) { __label__ = 4;; } else { __label__ = 12;; } //@line 433 "compiledict.cpp"
          $land_lhs_true$$if_end28$8: do { 
            if (__label__ == 4) {
  
              var $6=$step; //@line 433 "compiledict.cpp"
              var $cmp4=((($6))|0)==0; //@line 433 "compiledict.cpp"
              if (!($cmp4)) { __label__ = 12;break $land_lhs_true$$if_end28$8; } //@line 433 "compiledict.cpp"
  
              $flag_offset=100; //@line 436 "compiledict.cpp"
              var $7=$p; //@line 438 "compiledict.cpp"
              var $incdec_ptr=(($7+1)&4294967295); //@line 438 "compiledict.cpp"
              $p=$incdec_ptr; //@line 438 "compiledict.cpp"
              var $8=$p; //@line 439 "compiledict.cpp"
              var $9=HEAP8[($8)]; //@line 439 "compiledict.cpp"
              var $conv6=(tempInt=(($9)),(tempInt>=128?tempInt-256:tempInt)); //@line 439 "compiledict.cpp"
              var $cmp7=((($conv6))|0)==33; //@line 439 "compiledict.cpp"
              if ($cmp7) { __label__ = 6;; } else { __label__ = 7;; } //@line 439 "compiledict.cpp"
              if (__label__ == 6) {
  
                $flag_offset=132; //@line 442 "compiledict.cpp"
                var $10=$p; //@line 443 "compiledict.cpp"
                var $incdec_ptr9=(($10+1)&4294967295); //@line 443 "compiledict.cpp"
                $p=$incdec_ptr9; //@line 443 "compiledict.cpp"
                ; //@line 444 "compiledict.cpp"
              }
  
              $ix=0; //@line 446 "compiledict.cpp"
              var $11=$p; //@line 447 "compiledict.cpp"
              var $12=HEAP8[($11)]; //@line 447 "compiledict.cpp"
              var $conv11=(tempInt=(($12)),(tempInt>=128?tempInt-256:tempInt)); //@line 447 "compiledict.cpp"
              var $isdigittmp=((($conv11)-48)&4294967295); //@line 447 "compiledict.cpp"
              var $isdigit=((($isdigittmp))>>>0) < 10; //@line 447 "compiledict.cpp"
              var $call12=((($isdigit))&1); //@line 447 "compiledict.cpp"
              var $tobool=((($call12))|0)!=0; //@line 447 "compiledict.cpp"
              if ($tobool) { __label__ = 8;; } else { __label__ = 9;; } //@line 447 "compiledict.cpp"
              if (__label__ == 8) {
  
                var $13=$p; //@line 449 "compiledict.cpp"
                var $14=HEAP8[($13)]; //@line 449 "compiledict.cpp"
                var $conv14=(tempInt=(($14)),(tempInt>=128?tempInt-256:tempInt)); //@line 449 "compiledict.cpp"
                var $15=$ix; //@line 449 "compiledict.cpp"
                var $sub=((($15)-48)&4294967295); //@line 449 "compiledict.cpp"
                var $add=((($sub)+($conv14))&4294967295); //@line 449 "compiledict.cpp"
                $ix=$add; //@line 449 "compiledict.cpp"
                var $16=$p; //@line 450 "compiledict.cpp"
                var $incdec_ptr15=(($16+1)&4294967295); //@line 450 "compiledict.cpp"
                $p=$incdec_ptr15; //@line 450 "compiledict.cpp"
                ; //@line 451 "compiledict.cpp"
              }
  
              var $17=$p; //@line 452 "compiledict.cpp"
              var $18=HEAP8[($17)]; //@line 452 "compiledict.cpp"
              var $conv17=(tempInt=(($18)),(tempInt>=128?tempInt-256:tempInt)); //@line 452 "compiledict.cpp"
              var $isdigittmp1=((($conv17)-48)&4294967295); //@line 452 "compiledict.cpp"
              var $isdigit2=((($isdigittmp1))>>>0) < 10; //@line 452 "compiledict.cpp"
              var $call18=((($isdigit2))&1); //@line 452 "compiledict.cpp"
              var $tobool19=((($call18))|0)!=0; //@line 452 "compiledict.cpp"
              if ($tobool19) { __label__ = 10;; } else { __label__ = 11;; } //@line 452 "compiledict.cpp"
              if (__label__ == 10) {
  
                var $19=$ix; //@line 454 "compiledict.cpp"
                var $mul=((($19)*10)&4294967295); //@line 454 "compiledict.cpp"
                var $20=$p; //@line 454 "compiledict.cpp"
                var $21=HEAP8[($20)]; //@line 454 "compiledict.cpp"
                var $conv21=(tempInt=(($21)),(tempInt>=128?tempInt-256:tempInt)); //@line 454 "compiledict.cpp"
                var $sub22=((($mul)-48)&4294967295); //@line 454 "compiledict.cpp"
                var $add23=((($sub22)+($conv21))&4294967295); //@line 454 "compiledict.cpp"
                $ix=$add23; //@line 454 "compiledict.cpp"
                var $22=$p; //@line 455 "compiledict.cpp"
                var $incdec_ptr24=(($22+1)&4294967295); //@line 455 "compiledict.cpp"
                $p=$incdec_ptr24; //@line 455 "compiledict.cpp"
                ; //@line 456 "compiledict.cpp"
              }
  
              var $23=$ix; //@line 457 "compiledict.cpp"
              var $24=$flag_offset; //@line 457 "compiledict.cpp"
              var $add26=((($24)+($23))&4294967295); //@line 457 "compiledict.cpp"
              var $conv27=((($add26)) & 255); //@line 457 "compiledict.cpp"
              var $25=$n_flag_codes; //@line 457 "compiledict.cpp"
              var $inc=((($25)+1)&4294967295); //@line 457 "compiledict.cpp"
              $n_flag_codes=$inc; //@line 457 "compiledict.cpp"
              var $arrayidx=(($flag_codes+$25)&4294967295); //@line 457 "compiledict.cpp"
              HEAP8[($arrayidx)]=$conv27; //@line 457 "compiledict.cpp"
              var $26=$p; //@line 458 "compiledict.cpp"
              var $27=HEAP8[($26)]; //@line 458 "compiledict.cpp"
              $c=$27; //@line 458 "compiledict.cpp"
              ; //@line 459 "compiledict.cpp"
            }
          } while(0);
  
          var $28=$c; //@line 461 "compiledict.cpp"
          var $conv29=((($28))&255); //@line 461 "compiledict.cpp"
          var $cmp30=((($conv29))|0)==36; //@line 461 "compiledict.cpp"
          if ($cmp30) { __label__ = 13;; } else { __label__ = 25;; } //@line 461 "compiledict.cpp"
          $land_lhs_true31$$if_end64$21: do { 
            if (__label__ == 13) {
  
              var $29=$p; //@line 461 "compiledict.cpp"
              var $arrayidx32=(($29+1)&4294967295); //@line 461 "compiledict.cpp"
              var $30=HEAP8[($arrayidx32)]; //@line 461 "compiledict.cpp"
              var $conv33=(tempInt=(($30)),(tempInt>=128?tempInt-256:tempInt)); //@line 461 "compiledict.cpp"
              var $call34=_isalnum($conv33); //@line 461 "compiledict.cpp"
              var $tobool35=((($call34))|0)!=0; //@line 461 "compiledict.cpp"
              if (!($tobool35)) { __label__ = 25;break $land_lhs_true31$$if_end64$21; } //@line 461 "compiledict.cpp"
  
              var $31=$p; //@line 464 "compiledict.cpp"
              $mnemptr=$31; //@line 464 "compiledict.cpp"
              var $32=$p; //@line 465 "compiledict.cpp"
              var $33=HEAPU8[($32)]; //@line 465 "compiledict.cpp"
              $c=$33; //@line 465 "compiledict.cpp"
              var $conv389=((($33))&255); //@line 465 "compiledict.cpp"
              var $call3910=__Z8isspace2j($conv389); //@line 465 "compiledict.cpp"
              var $tobool4011=((($call3910))|0)!=0; //@line 465 "compiledict.cpp"
              var $lnot12=($tobool4011) ^ 1; //@line 465 "compiledict.cpp"
              var $34=$p; //@line 465 "compiledict.cpp"
              if ($lnot12) { __lastLabel__ = 14; __label__ = 15;; } else { __lastLabel__ = 14; __label__ = 16;; } //@line 465 "compiledict.cpp"
              $while_body41$$while_end$24: do { 
                if (__label__ == 15) {
                  while(1) { 
  
                    var $35=__lastLabel__ == 15 ? $38 : ($34);
                    var $incdec_ptr42=(($35+1)&4294967295); //@line 465 "compiledict.cpp"
                    $p=$incdec_ptr42; //@line 465 "compiledict.cpp"
                    var $36=$p; //@line 465 "compiledict.cpp"
                    var $37=HEAPU8[($36)]; //@line 465 "compiledict.cpp"
                    $c=$37; //@line 465 "compiledict.cpp"
                    var $conv38=((($37))&255); //@line 465 "compiledict.cpp"
                    var $call39=__Z8isspace2j($conv38); //@line 465 "compiledict.cpp"
                    var $tobool40=((($call39))|0)!=0; //@line 465 "compiledict.cpp"
                    var $lnot=($tobool40) ^ 1; //@line 465 "compiledict.cpp"
                    var $38=$p; //@line 465 "compiledict.cpp"
                    if ($lnot) { __lastLabel__ = 15; __label__ = 15;continue ; } else { __lastLabel__ = 15; __label__ = 16;break $while_body41$$while_end$24; } //@line 465 "compiledict.cpp"
                  }
                }
              } while(0);
  
              var $_lcssa=__lastLabel__ == 14 ? $34 : ($38);
              HEAP8[($_lcssa)]=0; //@line 466 "compiledict.cpp"
              var $39=$mnemptr; //@line 468 "compiledict.cpp"
              var $call43=__Z10LookupMnemP8MNEM_TABPKc(((_mnem_flags)&4294967295), $39); //@line 468 "compiledict.cpp"
              $ix=$call43; //@line 468 "compiledict.cpp"
              var $cmp44=((($call43))>>>0) > 0; //@line 469 "compiledict.cpp"
              if ($cmp44) { __label__ = 17;; } else { __label__ = 24;; } //@line 469 "compiledict.cpp"
              if (__label__ == 17) {
  
                var $40=$ix; //@line 471 "compiledict.cpp"
                var $cmp46=((($40))|0)==200; //@line 471 "compiledict.cpp"
                if ($cmp46) { __label__ = 18;; } else { __label__ = 19;; } //@line 471 "compiledict.cpp"
                if (__label__ == 18) {
  
                  HEAP8[(__ZL9text_mode_b)]=1;
                  ; //@line 474 "compiledict.cpp"
                }
                else if (__label__ == 19) {
  
                  var $41=$ix; //@line 476 "compiledict.cpp"
                  var $cmp48=((($41))|0)==201; //@line 476 "compiledict.cpp"
                  if ($cmp48) { __label__ = 20;; } else { __label__ = 21;; } //@line 476 "compiledict.cpp"
                  if (__label__ == 20) {
  
                    HEAP8[(__ZL9text_mode_b)]=0;
                    ; //@line 479 "compiledict.cpp"
                  }
                  else if (__label__ == 21) {
  
                    var $42=$ix; //@line 481 "compiledict.cpp"
                    var $cmp51=((($42))|0)==29; //@line 481 "compiledict.cpp"
                    if ($cmp51) { __label__ = 22;; } else { __label__ = 23;; } //@line 481 "compiledict.cpp"
                    if (__label__ == 22) {
  
                      $text_not_phonemes=1; //@line 483 "compiledict.cpp"
                      ; //@line 484 "compiledict.cpp"
                    }
                    else if (__label__ == 23) {
  
                      var $43=$ix; //@line 487 "compiledict.cpp"
                      var $conv54=((($43)) & 255); //@line 487 "compiledict.cpp"
                      var $44=$n_flag_codes; //@line 487 "compiledict.cpp"
                      var $inc55=((($44)+1)&4294967295); //@line 487 "compiledict.cpp"
                      $n_flag_codes=$inc55; //@line 487 "compiledict.cpp"
                      var $arrayidx56=(($flag_codes+$44)&4294967295); //@line 487 "compiledict.cpp"
                      HEAP8[($arrayidx56)]=$conv54; //@line 487 "compiledict.cpp"
                      ;
                    }
                  }
                }
              }
              else if (__label__ == 24) {
  
                var $45=HEAP32[((__ZL5f_log)>>2)]; //@line 492 "compiledict.cpp"
                var $46=HEAP32[((__ZL7linenum)>>2)]; //@line 492 "compiledict.cpp"
                var $47=$mnemptr; //@line 492 "compiledict.cpp"
                var $call61=_fprintf($45, ((__str109)&4294967295), allocate([$46,0,0,0,$47,0,0,0], ["i32",0,0,0,"i8*",0,0,0], ALLOC_STACK)); //@line 492 "compiledict.cpp"
                var $48=HEAP32[((__ZL11error_count)>>2)]; //@line 493 "compiledict.cpp"
                var $inc62=((($48)+1)&4294967295); //@line 493 "compiledict.cpp"
                HEAP32[((__ZL11error_count)>>2)]=$inc62; //@line 493 "compiledict.cpp"
                ;
              }
            }
          } while(0);
  
          var $49=$c; //@line 497 "compiledict.cpp"
          var $conv65=((($49))&255); //@line 497 "compiledict.cpp"
          var $cmp66=((($conv65))|0)==47; //@line 497 "compiledict.cpp"
          if ($cmp66) { __label__ = 26;; } else { __label__ = 29;; } //@line 497 "compiledict.cpp"
          $land_lhs_true67$$if_end74$41: do { 
            if (__label__ == 26) {
  
              var $50=$p; //@line 497 "compiledict.cpp"
              var $arrayidx68=(($50+1)&4294967295); //@line 497 "compiledict.cpp"
              var $51=HEAP8[($arrayidx68)]; //@line 497 "compiledict.cpp"
              var $conv69=(tempInt=(($51)),(tempInt>=128?tempInt-256:tempInt)); //@line 497 "compiledict.cpp"
              var $cmp70=((($conv69))|0)==47; //@line 497 "compiledict.cpp"
              if (!($cmp70)) { __label__ = 29;break $land_lhs_true67$$if_end74$41; } //@line 497 "compiledict.cpp"
  
              var $52=$multiple_words; //@line 497 "compiledict.cpp"
              var $cmp72=((($52))|0)==0; //@line 497 "compiledict.cpp"
              if (!($cmp72)) { __label__ = 29;break $land_lhs_true67$$if_end74$41; } //@line 497 "compiledict.cpp"
  
              $c=10; //@line 499 "compiledict.cpp"
              var $53=$p; //@line 500 "compiledict.cpp"
              $comment=$53; //@line 500 "compiledict.cpp"
              ; //@line 501 "compiledict.cpp"
            }
          } while(0);
  
          var $54=$step; //@line 503 "compiledict.cpp"
          if ($54 == 0) {
            __label__ = 30;;
          }
          else if ($54 == 1) {
            __label__ = 34;;
          }
          else if ($54 == 2) {
            __label__ = 46;;
          }
          else if ($54 == 3) {
            __label__ = 50;;
          }
          else if ($54 == 4) {
            __label__ = 52;;
          }
          else {
          __label__ = 54;;
          }
          
          $sw_epilog$$sw_bb$$sw_bb85$$sw_bb119$$sw_bb133$$sw_bb139$46: do { 
            if (__label__ == 30) {
  
              var $55=$c; //@line 506 "compiledict.cpp"
              var $conv75=((($55))&255); //@line 506 "compiledict.cpp"
              var $cmp76=((($conv75))|0)==40; //@line 506 "compiledict.cpp"
              if ($cmp76) { __label__ = 31;; } else { __label__ = 32;; } //@line 506 "compiledict.cpp"
              if (__label__ == 31) {
  
                $multiple_words=1; //@line 508 "compiledict.cpp"
                var $56=$p; //@line 509 "compiledict.cpp"
                var $add_ptr=(($56+1)&4294967295); //@line 509 "compiledict.cpp"
                $word=$add_ptr; //@line 509 "compiledict.cpp"
                $step=1; //@line 510 "compiledict.cpp"
                ; //@line 511 "compiledict.cpp"
              }
              else if (__label__ == 32) {
  
                var $57=$c; //@line 513 "compiledict.cpp"
                var $conv79=((($57))&255); //@line 513 "compiledict.cpp"
                var $call80=__Z8isspace2j($conv79); //@line 513 "compiledict.cpp"
                var $tobool81=((($call80))|0)!=0; //@line 513 "compiledict.cpp"
                if ($tobool81) { __label__ = 54;break $sw_epilog$$sw_bb$$sw_bb85$$sw_bb119$$sw_bb133$$sw_bb139$46; } //@line 513 "compiledict.cpp"
  
                var $58=$p; //@line 515 "compiledict.cpp"
                $word=$58; //@line 515 "compiledict.cpp"
                $step=1; //@line 516 "compiledict.cpp"
                ; //@line 517 "compiledict.cpp"
              }
            }
            else if (__label__ == 34) {
  
              var $59=$c; //@line 521 "compiledict.cpp"
              var $conv86=((($59))&255); //@line 521 "compiledict.cpp"
              var $cmp87=((($conv86))|0)==45; //@line 521 "compiledict.cpp"
              if ($cmp87) { __label__ = 35;; } else { __label__ = 39;; } //@line 521 "compiledict.cpp"
              $land_lhs_true88$$if_end99$53: do { 
                if (__label__ == 35) {
  
                  var $60=$multiple_words; //@line 521 "compiledict.cpp"
                  var $tobool89=((($60))|0)!=0; //@line 521 "compiledict.cpp"
                  if (!($tobool89)) { __label__ = 39;break $land_lhs_true88$$if_end99$53; } //@line 521 "compiledict.cpp"
  
                  var $61=$word; //@line 523 "compiledict.cpp"
                  var $arrayidx91=(($61)&4294967295); //@line 523 "compiledict.cpp"
                  var $62=HEAP8[($arrayidx91)]; //@line 523 "compiledict.cpp"
                  var $conv92=(tempInt=(($62)),(tempInt>=128?tempInt-256:tempInt)); //@line 523 "compiledict.cpp"
                  var $isdigittmp3=((($conv92)-48)&4294967295); //@line 523 "compiledict.cpp"
                  var $isdigit4=((($isdigittmp3))>>>0) < 10; //@line 523 "compiledict.cpp"
                  var $call93=((($isdigit4))&1); //@line 523 "compiledict.cpp"
                  var $tobool94=((($call93))|0)!=0; //@line 523 "compiledict.cpp"
                  if ($tobool94) { __label__ = 37;; } else { __label__ = 38;; } //@line 523 "compiledict.cpp"
                  if (__label__ == 37) {
  
                    $multiple_numeric_hyphen=1; //@line 525 "compiledict.cpp"
                    ; //@line 526 "compiledict.cpp"
                  }
  
                  var $63=$n_flag_codes; //@line 529 "compiledict.cpp"
                  var $inc97=((($63)+1)&4294967295); //@line 529 "compiledict.cpp"
                  $n_flag_codes=$inc97; //@line 529 "compiledict.cpp"
                  var $arrayidx98=(($flag_codes+$63)&4294967295); //@line 529 "compiledict.cpp"
                  HEAP8[($arrayidx98)]=44; //@line 529 "compiledict.cpp"
                  $c=32; //@line 531 "compiledict.cpp"
                  ; //@line 532 "compiledict.cpp"
                }
              } while(0);
  
              var $64=$c; //@line 533 "compiledict.cpp"
              var $conv100=((($64))&255); //@line 533 "compiledict.cpp"
              var $call101=__Z8isspace2j($conv100); //@line 533 "compiledict.cpp"
              var $tobool102=((($call101))|0)!=0; //@line 533 "compiledict.cpp"
              if ($tobool102) { __label__ = 40;; } else { __label__ = 43;; } //@line 533 "compiledict.cpp"
              if (__label__ == 40) {
  
                var $65=$p; //@line 535 "compiledict.cpp"
                var $arrayidx104=(($65)&4294967295); //@line 535 "compiledict.cpp"
                HEAP8[($arrayidx104)]=0; //@line 535 "compiledict.cpp"
                var $66=$multiple_words; //@line 537 "compiledict.cpp"
                var $tobool105=((($66))|0)!=0; //@line 537 "compiledict.cpp"
                if ($tobool105) { __label__ = 41;; } else { __label__ = 42;; } //@line 537 "compiledict.cpp"
                if (__label__ == 41) {
  
                  var $67=$p; //@line 539 "compiledict.cpp"
                  var $add_ptr107=(($67+1)&4294967295); //@line 539 "compiledict.cpp"
                  $multiple_string_end=$add_ptr107; //@line 539 "compiledict.cpp"
                  $multiple_string=$add_ptr107; //@line 539 "compiledict.cpp"
                  $step=2; //@line 540 "compiledict.cpp"
                  ; //@line 541 "compiledict.cpp"
                }
                else if (__label__ == 42) {
  
                  $step=3; //@line 544 "compiledict.cpp"
                  ;
                }
              }
              else if (__label__ == 43) {
  
                var $68=$c; //@line 548 "compiledict.cpp"
                var $conv111=((($68))&255); //@line 548 "compiledict.cpp"
                var $cmp112=((($conv111))|0)==41; //@line 548 "compiledict.cpp"
                if (!($cmp112)) { __label__ = 54;break $sw_epilog$$sw_bb$$sw_bb85$$sw_bb119$$sw_bb133$$sw_bb139$46; } //@line 548 "compiledict.cpp"
  
                var $69=$multiple_words; //@line 548 "compiledict.cpp"
                var $tobool114=((($69))|0)!=0; //@line 548 "compiledict.cpp"
                if (!($tobool114)) { __label__ = 54;break $sw_epilog$$sw_bb$$sw_bb85$$sw_bb119$$sw_bb133$$sw_bb139$46; } //@line 548 "compiledict.cpp"
  
                var $70=$p; //@line 550 "compiledict.cpp"
                var $arrayidx116=(($70)&4294967295); //@line 550 "compiledict.cpp"
                HEAP8[($arrayidx116)]=0; //@line 550 "compiledict.cpp"
                $step=3; //@line 551 "compiledict.cpp"
                $multiple_words=0; //@line 552 "compiledict.cpp"
                ; //@line 553 "compiledict.cpp"
              }
            }
            else if (__label__ == 46) {
  
              var $71=$c; //@line 557 "compiledict.cpp"
              var $conv120=((($71))&255); //@line 557 "compiledict.cpp"
              var $call121=__Z8isspace2j($conv120); //@line 557 "compiledict.cpp"
              var $tobool122=((($call121))|0)!=0; //@line 557 "compiledict.cpp"
              if ($tobool122) { __label__ = 47;; } else { __label__ = 48;; } //@line 557 "compiledict.cpp"
              if (__label__ == 47) {
  
                var $72=$multiple_words; //@line 559 "compiledict.cpp"
                var $inc124=((($72)+1)&4294967295); //@line 559 "compiledict.cpp"
                $multiple_words=$inc124; //@line 559 "compiledict.cpp"
                ; //@line 560 "compiledict.cpp"
              }
              else if (__label__ == 48) {
  
                var $73=$c; //@line 562 "compiledict.cpp"
                var $conv126=((($73))&255); //@line 562 "compiledict.cpp"
                var $cmp127=((($conv126))|0)==41; //@line 562 "compiledict.cpp"
                if (!($cmp127)) { __label__ = 54;break $sw_epilog$$sw_bb$$sw_bb85$$sw_bb119$$sw_bb133$$sw_bb139$46; } //@line 562 "compiledict.cpp"
  
                var $74=$p; //@line 564 "compiledict.cpp"
                var $arrayidx129=(($74)&4294967295); //@line 564 "compiledict.cpp"
                HEAP8[($arrayidx129)]=32; //@line 564 "compiledict.cpp"
                var $75=$p; //@line 565 "compiledict.cpp"
                var $add_ptr130=(($75+1)&4294967295); //@line 565 "compiledict.cpp"
                $multiple_string_end=$add_ptr130; //@line 565 "compiledict.cpp"
                $step=3; //@line 566 "compiledict.cpp"
                ; //@line 567 "compiledict.cpp"
              }
            }
            else if (__label__ == 50) {
  
              var $76=$c; //@line 571 "compiledict.cpp"
              var $conv134=((($76))&255); //@line 571 "compiledict.cpp"
              var $call135=__Z8isspace2j($conv134); //@line 571 "compiledict.cpp"
              var $tobool136=((($call135))|0)!=0; //@line 571 "compiledict.cpp"
              if ($tobool136) { __label__ = 54;break $sw_epilog$$sw_bb$$sw_bb85$$sw_bb119$$sw_bb133$$sw_bb139$46; } //@line 571 "compiledict.cpp"
  
              var $77=$p; //@line 573 "compiledict.cpp"
              $phonetic=$77; //@line 573 "compiledict.cpp"
              $step=4; //@line 574 "compiledict.cpp"
              ; //@line 575 "compiledict.cpp"
            }
            else if (__label__ == 52) {
  
              var $78=$c; //@line 579 "compiledict.cpp"
              var $conv140=((($78))&255); //@line 579 "compiledict.cpp"
              var $call141=__Z8isspace2j($conv140); //@line 579 "compiledict.cpp"
              var $tobool142=((($call141))|0)!=0; //@line 579 "compiledict.cpp"
              if (!($tobool142)) { __label__ = 54;break $sw_epilog$$sw_bb$$sw_bb85$$sw_bb119$$sw_bb133$$sw_bb139$46; } //@line 579 "compiledict.cpp"
  
              var $79=$p; //@line 581 "compiledict.cpp"
              var $arrayidx144=(($79)&4294967295); //@line 581 "compiledict.cpp"
              HEAP8[($arrayidx144)]=0; //@line 581 "compiledict.cpp"
              $step=5; //@line 582 "compiledict.cpp"
              ; //@line 583 "compiledict.cpp"
            }
          } while(0);
  
          var $80=$p; //@line 589 "compiledict.cpp"
          var $incdec_ptr146=(($80+1)&4294967295); //@line 589 "compiledict.cpp"
          $p=$incdec_ptr146; //@line 589 "compiledict.cpp"
          var $81=$c; //@line 429 "compiledict.cpp"
          var $conv=((($81))&255); //@line 429 "compiledict.cpp"
          var $cmp1=((($conv))|0)!=10; //@line 429 "compiledict.cpp"
          if ($cmp1) { __label__ = 3;continue $while_body$6; } else { __label__ = 55;break $while_body$$while_end147$5; } //@line 429 "compiledict.cpp"
        }
      }
    } while(0);
  
    var $82=$word; //@line 592 "compiledict.cpp"
    var $arrayidx148=(($82)&4294967295); //@line 592 "compiledict.cpp"
    var $83=HEAP8[($arrayidx148)]; //@line 592 "compiledict.cpp"
    var $conv149=(tempInt=(($83)),(tempInt>=128?tempInt-256:tempInt)); //@line 592 "compiledict.cpp"
    var $cmp150=((($conv149))|0)==0; //@line 592 "compiledict.cpp"
    if ($cmp150) { __label__ = 56;; } else { __label__ = 57;; } //@line 592 "compiledict.cpp"
    if (__label__ == 56) {
  
      $retval=0; //@line 600 "compiledict.cpp"
      ; //@line 600 "compiledict.cpp"
    }
    else if (__label__ == 57) {
  
      var $_b=HEAPU8[(__ZL9text_mode_b)];
      var $84=((($_b))&1);
      var $tobool153=((($84))|0)!=0; //@line 603 "compiledict.cpp"
      if ($tobool153) { __label__ = 58;; } else { __label__ = 59;; } //@line 603 "compiledict.cpp"
      $if_end155_thread$$if_end155$82: do { 
        if (__label__ == 58) {
  
          $text_not_phonemes=1; //@line 604 "compiledict.cpp"
          __label__ = 60;break $if_end155_thread$$if_end155$82; //@line 606 "compiledict.cpp"
        }
        else if (__label__ == 59) {
  
          var $_pr=$text_not_phonemes; //@line 606 "compiledict.cpp"
          var $tobool156=((($_pr))|0)!=0; //@line 606 "compiledict.cpp"
          if ($tobool156) { __label__ = 60;break $if_end155_thread$$if_end155$82; } //@line 606 "compiledict.cpp"
  
          var $96=$phonetic; //@line 639 "compiledict.cpp"
          var $arraydecay176=(($encoded_ph)&4294967295); //@line 639 "compiledict.cpp"
          var $arraydecay177=(($bad_phoneme)&4294967295); //@line 639 "compiledict.cpp"
          var $call178=__Z14EncodePhonemesPcS_Ph($96, $arraydecay176, $arraydecay177); //@line 639 "compiledict.cpp"
          var $arraydecay179=(($encoded_ph)&4294967295); //@line 640 "compiledict.cpp"
          var $call180=_strchr($arraydecay179, 21); //@line 640 "compiledict.cpp"
          var $cmp181=((($call180))|0)!=0; //@line 640 "compiledict.cpp"
          if ($cmp181) { __label__ = 65;; } else { __label__ = 66;; } //@line 640 "compiledict.cpp"
          if (__label__ == 65) {
  
            var $97=$n_flag_codes; //@line 642 "compiledict.cpp"
            var $inc183=((($97)+1)&4294967295); //@line 642 "compiledict.cpp"
            $n_flag_codes=$inc183; //@line 642 "compiledict.cpp"
            var $arrayidx184=(($flag_codes+$97)&4294967295); //@line 642 "compiledict.cpp"
            HEAP8[($arrayidx184)]=10; //@line 642 "compiledict.cpp"
            ; //@line 643 "compiledict.cpp"
          }
  
          var $arrayidx186=(($bad_phoneme)&4294967295); //@line 646 "compiledict.cpp"
          var $98=HEAPU8[($arrayidx186)]; //@line 646 "compiledict.cpp"
          var $conv187=((($98))&255); //@line 646 "compiledict.cpp"
          var $cmp188=((($conv187))|0)!=0; //@line 646 "compiledict.cpp"
          if (!($cmp188)) { __label__ = 68;break $if_end155_thread$$if_end155$82; } //@line 646 "compiledict.cpp"
  
          var $99=HEAP32[((__ZL5f_log)>>2)]; //@line 649 "compiledict.cpp"
          var $100=HEAP32[((__ZL7linenum)>>2)]; //@line 649 "compiledict.cpp"
          var $arrayidx190=(($bad_phoneme)&4294967295); //@line 649 "compiledict.cpp"
          var $101=HEAPU8[($arrayidx190)]; //@line 649 "compiledict.cpp"
          var $conv191=((($101))&255); //@line 649 "compiledict.cpp"
          var $arrayidx192=(($bad_phoneme)&4294967295); //@line 649 "compiledict.cpp"
          var $102=HEAPU8[($arrayidx192)]; //@line 649 "compiledict.cpp"
          var $conv193=((($102))&255); //@line 649 "compiledict.cpp"
          var $103=$word; //@line 649 "compiledict.cpp"
          var $104=$phonetic; //@line 649 "compiledict.cpp"
          var $call194=_fprintf($99, ((__str112)&4294967295), allocate([$100,0,0,0,$conv191,0,0,0,$conv193,0,0,0,$103,0,0,0,$104,0,0,0], ["i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i8*",0,0,0,"i8*",0,0,0], ALLOC_STACK)); //@line 649 "compiledict.cpp"
          var $105=HEAP32[((__ZL11error_count)>>2)]; //@line 650 "compiledict.cpp"
          var $inc195=((($105)+1)&4294967295); //@line 650 "compiledict.cpp"
          HEAP32[((__ZL11error_count)>>2)]=$inc195; //@line 650 "compiledict.cpp"
          __label__ = 68;break $if_end155_thread$$if_end155$82; //@line 651 "compiledict.cpp"
        }
      } while(0);
      $if_then157$$if_end197$90: do { 
        if (__label__ == 60) {
  
          var $85=$word; //@line 608 "compiledict.cpp"
          var $arrayidx158=(($85)&4294967295); //@line 608 "compiledict.cpp"
          var $86=HEAP8[($arrayidx158)]; //@line 608 "compiledict.cpp"
          var $conv159=(tempInt=(($86)),(tempInt>=128?tempInt-256:tempInt)); //@line 608 "compiledict.cpp"
          var $cmp160=((($conv159))|0)==95; //@line 608 "compiledict.cpp"
          if ($cmp160) { __label__ = 61;; } else { __label__ = 63;; } //@line 608 "compiledict.cpp"
          if (__label__ == 61) {
  
            var $87=$phonetic; //@line 611 "compiledict.cpp"
            var $strlen=_strlen($87); //@line 611 "compiledict.cpp"
            var $endptr=(($87+$strlen)&4294967295); //@line 611 "compiledict.cpp"
            var $src$, $dest$, $stop$, $stop4$, $fast$;
            $src$ = ((__str110)&4294967295);
            $dest$ = $endptr;
            $stop$ = $src$ + 2;
            if (($dest$%4) == ($src$%4) && 2 > 8) {
              while ($src$%4 !== 0 && $src$ < $stop$) {
                ; HEAP8[$dest$++] = HEAP8[$src$++];
              }
              $src$ >>= 2;
              $dest$ >>= 2;
              $stop4$ = $stop$ >> 2;
              while ($src$ < $stop4$) {
            ;;;    HEAP32[$dest$++] = HEAP32[$src$++];
              }
              $src$ <<= 2;
              $dest$ <<= 2;
            }while ($src$ < $stop$) {
              ; HEAP8[$dest$++] = HEAP8[$src$++];
            }; //@line 611 "compiledict.cpp"
            var $88=HEAP32[((_translator)>>2)]; //@line 615 "compiledict.cpp"
            var $89=$phonetic; //@line 615 "compiledict.cpp"
            var $call163=__Z13TranslateWordP10TranslatorPciP8WORD_TAB($88, $89, 0, 0); //@line 615 "compiledict.cpp"
            $text_not_phonemes=0; //@line 616 "compiledict.cpp"
            var $arraydecay=(($encoded_ph)&4294967295); //@line 617 "compiledict.cpp"
            __Z8strncpy0PcPKci($arraydecay, ((_word_phonemes)&4294967295), 156); //@line 617 "compiledict.cpp"
            var $90=HEAP8[(((_word_phonemes)&4294967295))]; //@line 619 "compiledict.cpp"
            var $conv164=(tempInt=(($90)),(tempInt>=128?tempInt-256:tempInt)); //@line 619 "compiledict.cpp"
            var $cmp165=((($conv164))|0)==0; //@line 619 "compiledict.cpp"
            var $91=HEAP32[((__ZL21error_need_dictionary)>>2)]; //@line 619 "compiledict.cpp"
            var $cmp167=((($91))|0) < 3; //@line 619 "compiledict.cpp"
            var $or_cond=($cmp165) & ($cmp167); //@line 619 "compiledict.cpp"
            if (!($or_cond)) { __label__ = 68;break $if_then157$$if_end197$90; } //@line 619 "compiledict.cpp"
  
            var $92=HEAP32[((__ZL21error_need_dictionary)>>2)]; //@line 622 "compiledict.cpp"
            var $inc169=((($92)+1)&4294967295); //@line 622 "compiledict.cpp"
            HEAP32[((__ZL21error_need_dictionary)>>2)]=$inc169; //@line 622 "compiledict.cpp"
            var $93=HEAP32[((__ZL5f_log)>>2)]; //@line 623 "compiledict.cpp"
            var $94=HEAP32[((__ZL7linenum)>>2)]; //@line 623 "compiledict.cpp"
            var $call170=_fprintf($93, ((__str111)&4294967295), allocate([$94,0,0,0], ["i32",0,0,0], ALLOC_STACK)); //@line 623 "compiledict.cpp"
            ; //@line 624 "compiledict.cpp"
          }
          else if (__label__ == 63) {
  
            var $arraydecay173=(($encoded_ph)&4294967295); //@line 634 "compiledict.cpp"
            var $95=$phonetic; //@line 634 "compiledict.cpp"
            __Z8strncpy0PcPKci($arraydecay173, $95, 156); //@line 634 "compiledict.cpp"
            ;
          }
        }
      } while(0);
  
      var $106=$text_not_phonemes; //@line 654 "compiledict.cpp"
      var $107=HEAP32[((_translator)>>2)]; //@line 654 "compiledict.cpp"
      var $langopts=(($107)&4294967295); //@line 654 "compiledict.cpp"
      var $textmode=(($langopts+252)&4294967295); //@line 654 "compiledict.cpp"
      var $108=HEAP8[($textmode)]; //@line 654 "compiledict.cpp"
      var $conv198=(tempInt=(($108)),(tempInt>=128?tempInt-256:tempInt)); //@line 654 "compiledict.cpp"
      var $cmp199=((($106))|0)!=((($conv198))|0); //@line 654 "compiledict.cpp"
      if ($cmp199) { __label__ = 69;; } else { __label__ = 70;; } //@line 654 "compiledict.cpp"
      if (__label__ == 69) {
  
        var $109=$n_flag_codes; //@line 656 "compiledict.cpp"
        var $inc201=((($109)+1)&4294967295); //@line 656 "compiledict.cpp"
        $n_flag_codes=$inc201; //@line 656 "compiledict.cpp"
        var $arrayidx202=(($flag_codes+$109)&4294967295); //@line 656 "compiledict.cpp"
        HEAP8[($arrayidx202)]=29; //@line 656 "compiledict.cpp"
        ; //@line 657 "compiledict.cpp"
      }
  
      var $110=$word; //@line 660 "compiledict.cpp"
      var $call204=_sscanf($110, ((__str113)&4294967295), allocate([$wc,0,0,0], ["i32*",0,0,0], ALLOC_STACK)); //@line 660 "compiledict.cpp"
      var $cmp205=((($call204))|0)==1; //@line 660 "compiledict.cpp"
      if ($cmp205) { __label__ = 71;; } else { __label__ = 72;; } //@line 660 "compiledict.cpp"
      $if_then206$$if_else209$100: do { 
        if (__label__ == 71) {
  
          var $111=HEAP32[(($wc)>>2)]; //@line 663 "compiledict.cpp"
          var $112=$word; //@line 663 "compiledict.cpp"
          var $call207=__Z8utf8_outjPc($111, $112); //@line 663 "compiledict.cpp"
          $ix=$call207; //@line 663 "compiledict.cpp"
          var $113=$ix; //@line 664 "compiledict.cpp"
          var $114=$word; //@line 664 "compiledict.cpp"
          var $arrayidx208=(($114+$113)&4294967295); //@line 664 "compiledict.cpp"
          HEAP8[($arrayidx208)]=0; //@line 664 "compiledict.cpp"
          ; //@line 665 "compiledict.cpp"
        }
        else if (__label__ == 72) {
  
          var $115=$word; //@line 667 "compiledict.cpp"
          var $arrayidx210=(($115)&4294967295); //@line 667 "compiledict.cpp"
          var $116=HEAP8[($arrayidx210)]; //@line 667 "compiledict.cpp"
          var $conv211=(tempInt=(($116)),(tempInt>=128?tempInt-256:tempInt)); //@line 667 "compiledict.cpp"
          var $cmp212=((($conv211))|0)!=95; //@line 667 "compiledict.cpp"
          if (!($cmp212)) { __label__ = 80;break $if_then206$$if_else209$100; } //@line 667 "compiledict.cpp"
  
          $all_upper_case=1; //@line 672 "compiledict.cpp"
          var $117=$word; //@line 673 "compiledict.cpp"
          $p=$117; //@line 673 "compiledict.cpp"
          var $118=$word; //@line 674 "compiledict.cpp"
          $p=$118; //@line 674 "compiledict.cpp"
          var $119=$p; //@line 678 "compiledict.cpp"
          var $call2146=__Z7utf8_inPiPKc($c2, $119); //@line 678 "compiledict.cpp"
          $ix=$call2146; //@line 678 "compiledict.cpp"
          var $120=HEAP32[(($c2)>>2)]; //@line 679 "compiledict.cpp"
          var $cmp2157=((($120))|0)==0; //@line 679 "compiledict.cpp"
          if ($cmp2157) { __label__ = 78;; } else { __label__ = 74;; } //@line 679 "compiledict.cpp"
          $for_end$$if_end217$104: do { 
            if (__label__ == 74) {
              while(1) { 
  
                var $121=HEAP32[(($c2)>>2)]; //@line 681 "compiledict.cpp"
                var $call218=_iswupper($121); //@line 681 "compiledict.cpp"
                var $tobool219=((($call218))|0)!=0; //@line 681 "compiledict.cpp"
                if ($tobool219) { __label__ = 75;; } else { __label__ = 76;; } //@line 681 "compiledict.cpp"
                if (__label__ == 75) {
  
                  var $122=HEAP32[(($c2)>>2)]; //@line 683 "compiledict.cpp"
                  var $call221=_towlower($122); //@line 683 "compiledict.cpp"
                  var $123=$p; //@line 683 "compiledict.cpp"
                  var $call222=__Z8utf8_outjPc($call221, $123); //@line 683 "compiledict.cpp"
                  ; //@line 684 "compiledict.cpp"
                }
                else if (__label__ == 76) {
  
                  $all_upper_case=0; //@line 687 "compiledict.cpp"
                  ;
                }
  
                var $124=$ix; //@line 689 "compiledict.cpp"
                var $125=$p; //@line 689 "compiledict.cpp"
                var $add_ptr225=(($125+$124)&4294967295); //@line 689 "compiledict.cpp"
                $p=$add_ptr225; //@line 689 "compiledict.cpp"
                var $126=$p; //@line 678 "compiledict.cpp"
                var $call214=__Z7utf8_inPiPKc($c2, $126); //@line 678 "compiledict.cpp"
                $ix=$call214; //@line 678 "compiledict.cpp"
                var $127=HEAP32[(($c2)>>2)]; //@line 679 "compiledict.cpp"
                var $cmp215=((($127))|0)==0; //@line 679 "compiledict.cpp"
                if ($cmp215) { __label__ = 78;break $for_end$$if_end217$104; } else { __label__ = 74;continue ; } //@line 679 "compiledict.cpp"
              }
            }
          } while(0);
  
          var $128=$all_upper_case; //@line 691 "compiledict.cpp"
          var $tobool226=((($128))|0)!=0; //@line 691 "compiledict.cpp"
          if (!($tobool226)) { __label__ = 80;break $if_then206$$if_else209$100; } //@line 691 "compiledict.cpp"
  
          var $129=$n_flag_codes; //@line 693 "compiledict.cpp"
          var $inc228=((($129)+1)&4294967295); //@line 693 "compiledict.cpp"
          $n_flag_codes=$inc228; //@line 693 "compiledict.cpp"
          var $arrayidx229=(($flag_codes+$129)&4294967295); //@line 693 "compiledict.cpp"
          HEAP8[($arrayidx229)]=42; //@line 693 "compiledict.cpp"
          ; //@line 694 "compiledict.cpp"
        }
      } while(0);
  
      var $130=$word; //@line 697 "compiledict.cpp"
      var $call233=_strlen($130); //@line 697 "compiledict.cpp"
      $len_word=$call233; //@line 697 "compiledict.cpp"
      var $131=HEAP32[((_translator)>>2)]; //@line 699 "compiledict.cpp"
      var $transpose_min=(($131+288)&4294967295); //@line 699 "compiledict.cpp"
      var $132=HEAP32[(($transpose_min)>>2)]; //@line 699 "compiledict.cpp"
      var $cmp234=((($132))|0) > 0; //@line 699 "compiledict.cpp"
      if ($cmp234) { __label__ = 81;; } else { __label__ = 82;; } //@line 699 "compiledict.cpp"
      if (__label__ == 81) {
  
        var $133=HEAP32[((_translator)>>2)]; //@line 701 "compiledict.cpp"
        var $134=$word; //@line 701 "compiledict.cpp"
        var $call236=__Z17TransposeAlphabetP10TranslatorPc($133, $134); //@line 701 "compiledict.cpp"
        $len_word=$call236; //@line 701 "compiledict.cpp"
        ; //@line 702 "compiledict.cpp"
      }
  
      var $135=$word; //@line 704 "compiledict.cpp"
      var $call238=__Z14HashDictionaryPKc($135); //@line 704 "compiledict.cpp"
      var $136=$hash_addr; //@line 704 "compiledict.cpp"
      HEAP32[(($136)>>2)]=$call238; //@line 704 "compiledict.cpp"
      var $arraydecay239=(($encoded_ph)&4294967295); //@line 705 "compiledict.cpp"
      var $call240=_strlen($arraydecay239); //@line 705 "compiledict.cpp"
      $len_phonetic=$call240; //@line 705 "compiledict.cpp"
      var $137=$len_word; //@line 707 "compiledict.cpp"
      var $conv241=((($137)) & 255); //@line 707 "compiledict.cpp"
      var $138=$dict_line_addr; //@line 707 "compiledict.cpp"
      var $arrayidx242=(($138+1)&4294967295); //@line 707 "compiledict.cpp"
      HEAP8[($arrayidx242)]=$conv241; //@line 707 "compiledict.cpp"
      var $139=$len_word; //@line 708 "compiledict.cpp"
      var $and=($139) & 63; //@line 708 "compiledict.cpp"
      $len_word=$and; //@line 708 "compiledict.cpp"
      var $140=$dict_line_addr; //@line 710 "compiledict.cpp"
      var $arrayidx243=(($140+2)&4294967295); //@line 710 "compiledict.cpp"
      var $141=$word; //@line 710 "compiledict.cpp"
      var $142=$len_word; //@line 710 "compiledict.cpp"
      var $src$, $dest$, $stop$, $stop4$, $fast$;
      $src$ = $141;
      $dest$ = $arrayidx243;
      $stop$ = $src$ + $142;
      if (($dest$%4) == ($src$%4) && $142 > 8) {
        while ($src$%4 !== 0 && $src$ < $stop$) {
          ; HEAP8[$dest$++] = HEAP8[$src$++];
        }
        $src$ >>= 2;
        $dest$ >>= 2;
        $stop4$ = $stop$ >> 2;
        while ($src$ < $stop4$) {
      ;;;    HEAP32[$dest$++] = HEAP32[$src$++];
        }
        $src$ <<= 2;
        $dest$ <<= 2;
      }while ($src$ < $stop$) {
        ; HEAP8[$dest$++] = HEAP8[$src$++];
      }; //@line 710 "compiledict.cpp"
      var $143=$len_phonetic; //@line 712 "compiledict.cpp"
      var $cmp244=((($143))|0)==0; //@line 712 "compiledict.cpp"
      if ($cmp244) { __label__ = 83;; } else { __label__ = 84;; } //@line 712 "compiledict.cpp"
      if (__label__ == 83) {
  
        var $144=$dict_line_addr; //@line 715 "compiledict.cpp"
        var $arrayidx246=(($144+1)&4294967295); //@line 715 "compiledict.cpp"
        var $145=HEAP8[($arrayidx246)]; //@line 715 "compiledict.cpp"
        var $conv247=(tempInt=(($145)),(tempInt>=128?tempInt-256:tempInt)); //@line 715 "compiledict.cpp"
        var $or=($conv247) | 128; //@line 715 "compiledict.cpp"
        var $conv248=((($or)) & 255); //@line 715 "compiledict.cpp"
        HEAP8[($arrayidx246)]=$conv248; //@line 715 "compiledict.cpp"
        var $146=$len_word; //@line 716 "compiledict.cpp"
        var $add249=((($146)+2)&4294967295); //@line 716 "compiledict.cpp"
        $length=$add249; //@line 716 "compiledict.cpp"
        ; //@line 717 "compiledict.cpp"
      }
      else if (__label__ == 84) {
  
        var $147=$len_word; //@line 720 "compiledict.cpp"
        var $148=$len_phonetic; //@line 720 "compiledict.cpp"
        var $add251=((($147)+3)&4294967295); //@line 720 "compiledict.cpp"
        var $add252=((($add251)+($148))&4294967295); //@line 720 "compiledict.cpp"
        $length=$add252; //@line 720 "compiledict.cpp"
        var $149=$len_word; //@line 721 "compiledict.cpp"
        var $add253=((($149)+2)&4294967295); //@line 721 "compiledict.cpp"
        var $150=$dict_line_addr; //@line 721 "compiledict.cpp"
        var $arrayidx254=(($150+$add253)&4294967295); //@line 721 "compiledict.cpp"
        var $arraydecay255=(($encoded_ph)&4294967295); //@line 721 "compiledict.cpp"
        var $call256=_strcpy($arrayidx254, $arraydecay255); //@line 721 "compiledict.cpp"
        ;
      }
  
      $ix=0; //@line 724 "compiledict.cpp"
      var $151=$ix; //@line 724 "compiledict.cpp"
      var $152=$n_flag_codes; //@line 724 "compiledict.cpp"
      var $cmp2598=((($151))>>>0) < ((($152))>>>0); //@line 724 "compiledict.cpp"
      if ($cmp2598) { __label__ = 86;; } else { __label__ = 87;; } //@line 724 "compiledict.cpp"
      $for_body$$for_end264$121: do { 
        if (__label__ == 86) {
          while(1) { 
  
            var $153=$ix; //@line 726 "compiledict.cpp"
            var $arrayidx260=(($flag_codes+$153)&4294967295); //@line 726 "compiledict.cpp"
            var $154=HEAP8[($arrayidx260)]; //@line 726 "compiledict.cpp"
            var $155=$ix; //@line 726 "compiledict.cpp"
            var $156=$length; //@line 726 "compiledict.cpp"
            var $add261=((($156)+($155))&4294967295); //@line 726 "compiledict.cpp"
            var $157=$dict_line_addr; //@line 726 "compiledict.cpp"
            var $arrayidx262=(($157+$add261)&4294967295); //@line 726 "compiledict.cpp"
            HEAP8[($arrayidx262)]=$154; //@line 726 "compiledict.cpp"
            var $158=$ix; //@line 724 "compiledict.cpp"
            var $inc263=((($158)+1)&4294967295); //@line 724 "compiledict.cpp"
            $ix=$inc263; //@line 724 "compiledict.cpp"
            var $159=$ix; //@line 724 "compiledict.cpp"
            var $160=$n_flag_codes; //@line 724 "compiledict.cpp"
            var $cmp259=((($159))>>>0) < ((($160))>>>0); //@line 724 "compiledict.cpp"
            if ($cmp259) { __label__ = 86;continue ; } else { __label__ = 87;break $for_body$$for_end264$121; } //@line 724 "compiledict.cpp"
          }
        }
      } while(0);
  
      var $161=$n_flag_codes; //@line 728 "compiledict.cpp"
      var $162=$length; //@line 728 "compiledict.cpp"
      var $add265=((($162)+($161))&4294967295); //@line 728 "compiledict.cpp"
      $length=$add265; //@line 728 "compiledict.cpp"
      var $163=$multiple_string; //@line 730 "compiledict.cpp"
      var $cmp266=((($163))|0)!=0; //@line 730 "compiledict.cpp"
      if ($cmp266) { __label__ = 88;; } else { __label__ = 94;; } //@line 730 "compiledict.cpp"
      $land_lhs_true267$$if_end286$125: do { 
        if (__label__ == 88) {
  
          var $164=$multiple_words; //@line 730 "compiledict.cpp"
          var $cmp268=((($164))|0) > 0; //@line 730 "compiledict.cpp"
          if (!($cmp268)) { __label__ = 94;break $land_lhs_true267$$if_end286$125; } //@line 730 "compiledict.cpp"
  
          var $165=$multiple_words; //@line 732 "compiledict.cpp"
          var $cmp270=((($165))|0) > 10; //@line 732 "compiledict.cpp"
          if ($cmp270) { __label__ = 90;; } else { __label__ = 91;; } //@line 732 "compiledict.cpp"
          if (__label__ == 90) {
  
            var $166=HEAP32[((__ZL5f_log)>>2)]; //@line 734 "compiledict.cpp"
            var $167=HEAP32[((__ZL7linenum)>>2)]; //@line 734 "compiledict.cpp"
            var $168=$multiple_words; //@line 734 "compiledict.cpp"
            var $call272=_fprintf($166, ((__str114)&4294967295), allocate([$167,0,0,0,$168,0,0,0], ["i32",0,0,0,"i32",0,0,0], ALLOC_STACK)); //@line 734 "compiledict.cpp"
            ; //@line 735 "compiledict.cpp"
          }
          else if (__label__ == 91) {
  
            var $169=$multiple_words; //@line 738 "compiledict.cpp"
            var $add274=((($169)+80)&4294967295); //@line 738 "compiledict.cpp"
            var $conv275=((($add274)) & 255); //@line 738 "compiledict.cpp"
            var $170=$length; //@line 738 "compiledict.cpp"
            var $inc276=((($170)+1)&4294967295); //@line 738 "compiledict.cpp"
            $length=$inc276; //@line 738 "compiledict.cpp"
            var $171=$dict_line_addr; //@line 738 "compiledict.cpp"
            var $arrayidx277=(($171+$170)&4294967295); //@line 738 "compiledict.cpp"
            HEAP8[($arrayidx277)]=$conv275; //@line 738 "compiledict.cpp"
            var $172=$multiple_string_end; //@line 739 "compiledict.cpp"
            var $173=$multiple_string; //@line 739 "compiledict.cpp"
            var $sub_ptr_lhs_cast=($172); //@line 739 "compiledict.cpp"
            var $sub_ptr_rhs_cast=($173); //@line 739 "compiledict.cpp"
            var $sub_ptr_sub=((($sub_ptr_lhs_cast)-($sub_ptr_rhs_cast))&4294967295); //@line 739 "compiledict.cpp"
            $ix=$sub_ptr_sub; //@line 739 "compiledict.cpp"
            var $174=$multiple_numeric_hyphen; //@line 740 "compiledict.cpp"
            var $tobool278=((($174))|0)!=0; //@line 740 "compiledict.cpp"
            if ($tobool278) { __label__ = 92;; } else { __label__ = 93;; } //@line 740 "compiledict.cpp"
            if (__label__ == 92) {
  
              var $175=$length; //@line 742 "compiledict.cpp"
              var $inc280=((($175)+1)&4294967295); //@line 742 "compiledict.cpp"
              $length=$inc280; //@line 742 "compiledict.cpp"
              var $176=$dict_line_addr; //@line 742 "compiledict.cpp"
              var $arrayidx281=(($176+$175)&4294967295); //@line 742 "compiledict.cpp"
              HEAP8[($arrayidx281)]=32; //@line 742 "compiledict.cpp"
              ; //@line 743 "compiledict.cpp"
            }
  
            var $177=$length; //@line 744 "compiledict.cpp"
            var $178=$dict_line_addr; //@line 744 "compiledict.cpp"
            var $arrayidx283=(($178+$177)&4294967295); //@line 744 "compiledict.cpp"
            var $179=$multiple_string; //@line 744 "compiledict.cpp"
            var $180=$ix; //@line 744 "compiledict.cpp"
            var $src$, $dest$, $stop$, $stop4$, $fast$;
            $src$ = $179;
            $dest$ = $arrayidx283;
            $stop$ = $src$ + $180;
            if (($dest$%4) == ($src$%4) && $180 > 8) {
              while ($src$%4 !== 0 && $src$ < $stop$) {
                ; HEAP8[$dest$++] = HEAP8[$src$++];
              }
              $src$ >>= 2;
              $dest$ >>= 2;
              $stop4$ = $stop$ >> 2;
              while ($src$ < $stop4$) {
            ;;;    HEAP32[$dest$++] = HEAP32[$src$++];
              }
              $src$ <<= 2;
              $dest$ <<= 2;
            }while ($src$ < $stop$) {
              ; HEAP8[$dest$++] = HEAP8[$src$++];
            }; //@line 744 "compiledict.cpp"
            var $181=$ix; //@line 745 "compiledict.cpp"
            var $182=$length; //@line 745 "compiledict.cpp"
            var $add284=((($182)+($181))&4294967295); //@line 745 "compiledict.cpp"
            $length=$add284; //@line 745 "compiledict.cpp"
            ;
          }
        }
      } while(0);
  
      var $183=$length; //@line 748 "compiledict.cpp"
      var $conv287=((($183)) & 255); //@line 748 "compiledict.cpp"
      var $184=$dict_line_addr; //@line 748 "compiledict.cpp"
      var $arrayidx288=(($184)&4294967295); //@line 748 "compiledict.cpp"
      HEAP8[($arrayidx288)]=$conv287; //@line 748 "compiledict.cpp"
      var $185=$length; //@line 780 "compiledict.cpp"
      $retval=$185; //@line 780 "compiledict.cpp"
      ; //@line 780 "compiledict.cpp"
    }
  
    var $186=$retval; //@line 781 "compiledict.cpp"
    STACKTOP = __stackBase__;
    return $186; //@line 781 "compiledict.cpp"
    return null;
  }
  

  function __Z8strncpy0PcPKci($to, $from, $size) {
    ;
    var __label__;
  
    var $to_addr;
    var $from_addr;
    var $size_addr;
    $to_addr=$to;
    $from_addr=$from;
    $size_addr=$size;
    var $0=$to_addr; //@line 87 "dictionary.cpp"
    var $1=$from_addr; //@line 87 "dictionary.cpp"
    var $2=$size_addr; //@line 87 "dictionary.cpp"
    var $call=_strncpy($0, $1, $2); //@line 87 "dictionary.cpp"
    var $3=$size_addr; //@line 88 "dictionary.cpp"
    var $sub=((($3)-1)&4294967295); //@line 88 "dictionary.cpp"
    var $4=$to_addr; //@line 88 "dictionary.cpp"
    var $arrayidx=(($4+$sub)&4294967295); //@line 88 "dictionary.cpp"
    HEAP8[($arrayidx)]=0; //@line 88 "dictionary.cpp"
    ;
    return; //@line 89 "dictionary.cpp"
    return;
  }
  

  function __Z13Reverse4Bytesi($word) {
    ;
    var __label__;
  
    var $word_addr;
    $word_addr=$word;
    var $0=$word_addr; //@line 106 "dictionary.cpp"
    ;
    return $0; //@line 106 "dictionary.cpp"
    return null;
  }
  

  function __Z10LookupMnemP8MNEM_TABPKc($table, $string) {
    ;
    var __label__;
  
    var $retval;
    var $table_addr;
    var $string_addr;
    $table_addr=$table;
    $string_addr=$string;
    ; //@line 113 "dictionary.cpp"
    while(1) { 
  
      var $0=$table_addr; //@line 113 "dictionary.cpp"
      var $mnem=(($0)&4294967295); //@line 113 "dictionary.cpp"
      var $1=HEAP32[(($mnem)>>2)]; //@line 113 "dictionary.cpp"
      var $cmp=((($1))|0)!=0; //@line 113 "dictionary.cpp"
      if (!($cmp)) { __label__ = 5;break ; } //@line 113 "dictionary.cpp"
  
      var $2=$string_addr; //@line 115 "dictionary.cpp"
      var $3=$table_addr; //@line 115 "dictionary.cpp"
      var $mnem1=(($3)&4294967295); //@line 115 "dictionary.cpp"
      var $4=HEAP32[(($mnem1)>>2)]; //@line 115 "dictionary.cpp"
      var $call=_strcmp($2, $4); //@line 115 "dictionary.cpp"
      var $cmp2=((($call))|0)==0; //@line 115 "dictionary.cpp"
      var $5=$table_addr; //@line 116 "dictionary.cpp"
      if ($cmp2) { __label__ = 3;break ; } //@line 115 "dictionary.cpp"
  
      var $incdec_ptr=(($5+8)&4294967295); //@line 117 "dictionary.cpp"
      $table_addr=$incdec_ptr; //@line 117 "dictionary.cpp"
      __label__ = 1;continue ; //@line 118 "dictionary.cpp"
    }
    if (__label__ == 5) {
  
      var $7=$table_addr; //@line 119 "dictionary.cpp"
      var $value3=(($7+4)&4294967295); //@line 119 "dictionary.cpp"
      var $8=HEAP32[(($value3)>>2)]; //@line 119 "dictionary.cpp"
      $retval=$8; //@line 119 "dictionary.cpp"
      ; //@line 119 "dictionary.cpp"
    }
    else if (__label__ == 3) {
  
      var $value=(($5+4)&4294967295); //@line 116 "dictionary.cpp"
      var $6=HEAP32[(($value)>>2)]; //@line 116 "dictionary.cpp"
      $retval=$6; //@line 116 "dictionary.cpp"
      ; //@line 116 "dictionary.cpp"
    }
  
    var $9=$retval; //@line 120 "dictionary.cpp"
    ;
    return $9; //@line 120 "dictionary.cpp"
    return null;
  }
  

  function __Z14LoadDictionaryP10TranslatorPKci($tr, $name, $no_error) {
    var __stackBase__  = STACKTOP; STACKTOP += 172; _memset(__stackBase__, 0, 172);
    var __label__;
  
    var $retval;
    var $tr_addr;
    var $name_addr;
    var $no_error_addr;
    var $hash;
    var $p;
    var $pw;
    var $length;
    var $f;
    var $size;
    var $fname=__stackBase__;
    $tr_addr=$tr;
    $name_addr=$name;
    $no_error_addr=$no_error;
    var $0=$name_addr; //@line 250 "dictionary.cpp"
    var $call=_strcpy(((_dictionary_name)&4294967295), $0); //@line 250 "dictionary.cpp"
    var $1=$tr_addr; //@line 251 "dictionary.cpp"
    var $dictionary_name=(($1+292)&4294967295); //@line 251 "dictionary.cpp"
    var $arraydecay=(($dictionary_name)&4294967295); //@line 251 "dictionary.cpp"
    var $2=$name_addr; //@line 251 "dictionary.cpp"
    var $call1=_strcpy($arraydecay, $2); //@line 251 "dictionary.cpp"
    var $arraydecay2=(($fname)&4294967295); //@line 256 "dictionary.cpp"
    var $3=$name_addr; //@line 256 "dictionary.cpp"
    var $call3=_sprintf($arraydecay2, ((__str116)&4294967295), allocate([((_path_home)&4294967295),0,0,0,47,0,0,0,$3,0,0,0], ["i8*",0,0,0,"i32",0,0,0,"i8*",0,0,0], ALLOC_STACK)); //@line 256 "dictionary.cpp"
    var $arraydecay4=(($fname)&4294967295); //@line 257 "dictionary.cpp"
    var $call5=__Z13GetFileLengthPKc($arraydecay4); //@line 257 "dictionary.cpp"
    $size=$call5; //@line 257 "dictionary.cpp"
    var $4=$tr_addr; //@line 259 "dictionary.cpp"
    var $data_dictlist=(($4+1052)&4294967295); //@line 259 "dictionary.cpp"
    var $5=HEAP32[(($data_dictlist)>>2)]; //@line 259 "dictionary.cpp"
    var $cmp=((($5))|0)!=0; //@line 259 "dictionary.cpp"
    if ($cmp) { __label__ = 1;; } else { __label__ = 2;; } //@line 259 "dictionary.cpp"
    if (__label__ == 1) {
  
      var $6=$tr_addr; //@line 261 "dictionary.cpp"
      var $data_dictlist6=(($6+1052)&4294967295); //@line 261 "dictionary.cpp"
      var $7=HEAP32[(($data_dictlist6)>>2)]; //@line 261 "dictionary.cpp"
      __Z4FreePv($7); //@line 261 "dictionary.cpp"
      var $8=$tr_addr; //@line 262 "dictionary.cpp"
      var $data_dictlist7=(($8+1052)&4294967295); //@line 262 "dictionary.cpp"
      HEAP32[(($data_dictlist7)>>2)]=0; //@line 262 "dictionary.cpp"
      ; //@line 263 "dictionary.cpp"
    }
  
    var $arraydecay8=(($fname)&4294967295); //@line 265 "dictionary.cpp"
    var $call9=_fopen($arraydecay8, ((__str18429)&4294967295)); //@line 265 "dictionary.cpp"
    $f=$call9; //@line 265 "dictionary.cpp"
    var $cmp10=((($call9))|0)==0; //@line 266 "dictionary.cpp"
    if ($cmp10) { __label__ = 4;; } else { __label__ = 3;; } //@line 266 "dictionary.cpp"
    $if_then12$$lor_lhs_false$5: do { 
      if (__label__ == 3) {
  
        var $9=$size; //@line 266 "dictionary.cpp"
        var $cmp11=((($9))>>>0) <= 0; //@line 266 "dictionary.cpp"
        if ($cmp11) { __label__ = 4;break $if_then12$$lor_lhs_false$5; } //@line 266 "dictionary.cpp"
  
        var $13=$size; //@line 275 "dictionary.cpp"
        var $call19=__Z5Alloci($13); //@line 275 "dictionary.cpp"
        var $14=$tr_addr; //@line 275 "dictionary.cpp"
        var $data_dictlist20=(($14+1052)&4294967295); //@line 275 "dictionary.cpp"
        HEAP32[(($data_dictlist20)>>2)]=$call19; //@line 275 "dictionary.cpp"
        var $15=$tr_addr; //@line 276 "dictionary.cpp"
        var $data_dictlist21=(($15+1052)&4294967295); //@line 276 "dictionary.cpp"
        var $16=HEAP32[(($data_dictlist21)>>2)]; //@line 276 "dictionary.cpp"
        var $17=$size; //@line 276 "dictionary.cpp"
        var $18=$f; //@line 276 "dictionary.cpp"
        var $call22=_fread($16, 1, $17, $18); //@line 276 "dictionary.cpp"
        $size=$call22; //@line 276 "dictionary.cpp"
        var $19=$f; //@line 277 "dictionary.cpp"
        var $call23=_fclose($19); //@line 277 "dictionary.cpp"
        var $20=$tr_addr; //@line 280 "dictionary.cpp"
        var $data_dictlist24=(($20+1052)&4294967295); //@line 280 "dictionary.cpp"
        var $21=HEAP32[(($data_dictlist24)>>2)]; //@line 280 "dictionary.cpp"
        var $22=$21; //@line 280 "dictionary.cpp"
        $pw=$22; //@line 280 "dictionary.cpp"
        var $23=$pw; //@line 281 "dictionary.cpp"
        var $arrayidx=(($23+4)&4294967295); //@line 281 "dictionary.cpp"
        var $24=HEAP32[(($arrayidx)>>2)]; //@line 281 "dictionary.cpp"
        var $call25=__Z13Reverse4Bytesi($24); //@line 281 "dictionary.cpp"
        $length=$call25; //@line 281 "dictionary.cpp"
        var $25=$size; //@line 283 "dictionary.cpp"
        var $cmp26=((($25))>>>0) <= 1032; //@line 283 "dictionary.cpp"
        if ($cmp26) { __label__ = 8;; } else { __label__ = 9;; } //@line 283 "dictionary.cpp"
        if (__label__ == 8) {
  
          var $26=HEAP32[((__impure_ptr)>>2)]; //@line 285 "dictionary.cpp"
          var $_stderr28=(($26+12)&4294967295); //@line 285 "dictionary.cpp"
          var $27=HEAP32[(($_stderr28)>>2)]; //@line 285 "dictionary.cpp"
          var $arraydecay29=(($fname)&4294967295); //@line 285 "dictionary.cpp"
          var $call30=_fprintf($27, ((__str3119)&4294967295), allocate([$arraydecay29,0,0,0], ["i8*",0,0,0], ALLOC_STACK)); //@line 285 "dictionary.cpp"
          $retval=2; //@line 286 "dictionary.cpp"
          __label__ = 19;break $if_then12$$lor_lhs_false$5; //@line 286 "dictionary.cpp"
        }
        else if (__label__ == 9) {
  
          var $28=$pw; //@line 289 "dictionary.cpp"
          var $arrayidx32=(($28)&4294967295); //@line 289 "dictionary.cpp"
          var $29=HEAP32[(($arrayidx32)>>2)]; //@line 289 "dictionary.cpp"
          var $call33=__Z13Reverse4Bytesi($29); //@line 289 "dictionary.cpp"
          var $cmp34=((($call33))|0)!=1024; //@line 289 "dictionary.cpp"
          if ($cmp34) { __label__ = 11;; } else { __label__ = 10;; } //@line 289 "dictionary.cpp"
          $if_then39$$lor_lhs_false35$11: do { 
            if (__label__ == 10) {
  
              var $30=$length; //@line 289 "dictionary.cpp"
              var $cmp36=((($30))|0) <= 0; //@line 289 "dictionary.cpp"
              var $31=$length; //@line 289 "dictionary.cpp"
              var $cmp38=((($31))|0) > 134217728; //@line 289 "dictionary.cpp"
              var $or_cond=($cmp36) | ($cmp38); //@line 289 "dictionary.cpp"
              if ($or_cond) { __label__ = 11;break $if_then39$$lor_lhs_false35$11; } //@line 289 "dictionary.cpp"
  
              var $37=$length; //@line 295 "dictionary.cpp"
              var $38=$tr_addr; //@line 295 "dictionary.cpp"
              var $data_dictlist46=(($38+1052)&4294967295); //@line 295 "dictionary.cpp"
              var $39=HEAP32[(($data_dictlist46)>>2)]; //@line 295 "dictionary.cpp"
              var $arrayidx47=(($39+$37)&4294967295); //@line 295 "dictionary.cpp"
              var $40=$tr_addr; //@line 295 "dictionary.cpp"
              var $data_dictrules=(($40+1048)&4294967295); //@line 295 "dictionary.cpp"
              HEAP32[(($data_dictrules)>>2)]=$arrayidx47; //@line 295 "dictionary.cpp"
              var $41=$tr_addr; //@line 298 "dictionary.cpp"
              __ZL10InitGroupsP10Translator($41); //@line 298 "dictionary.cpp"
              var $42=$tr_addr; //@line 299 "dictionary.cpp"
              var $groups1=(($42+5256)&4294967295); //@line 299 "dictionary.cpp"
              var $arrayidx48=(($groups1)&4294967295); //@line 299 "dictionary.cpp"
              var $43=HEAP32[(($arrayidx48)>>2)]; //@line 299 "dictionary.cpp"
              var $cmp49=((($43))|0)==0; //@line 299 "dictionary.cpp"
              if ($cmp49) { __label__ = 13;; } else { __label__ = 14;; } //@line 299 "dictionary.cpp"
              if (__label__ == 13) {
  
                var $44=HEAP32[((__impure_ptr)>>2)]; //@line 301 "dictionary.cpp"
                var $_stderr51=(($44+12)&4294967295); //@line 301 "dictionary.cpp"
                var $45=HEAP32[(($_stderr51)>>2)]; //@line 301 "dictionary.cpp"
                var $46=$name_addr; //@line 301 "dictionary.cpp"
                var $call52=_fprintf($45, ((__str5121)&4294967295), allocate([$46,0,0,0], ["i8*",0,0,0], ALLOC_STACK)); //@line 301 "dictionary.cpp"
                ; //@line 302 "dictionary.cpp"
              }
  
              var $47=$tr_addr; //@line 305 "dictionary.cpp"
              var $data_dictlist54=(($47+1052)&4294967295); //@line 305 "dictionary.cpp"
              var $48=HEAP32[(($data_dictlist54)>>2)]; //@line 305 "dictionary.cpp"
              var $arrayidx55=(($48+8)&4294967295); //@line 305 "dictionary.cpp"
              $p=$arrayidx55; //@line 305 "dictionary.cpp"
              $hash=0; //@line 307 "dictionary.cpp"
              ; //@line 307 "dictionary.cpp"
              $for_body$17: while(1) { 
  
                var $49=$p; //@line 309 "dictionary.cpp"
                var $50=$hash; //@line 309 "dictionary.cpp"
                var $51=$tr_addr; //@line 309 "dictionary.cpp"
                var $dict_hashtab=(($51+1056)&4294967295); //@line 309 "dictionary.cpp"
                var $arrayidx57=(($dict_hashtab+$50*4)&4294967295); //@line 309 "dictionary.cpp"
                HEAP32[(($arrayidx57)>>2)]=$49; //@line 309 "dictionary.cpp"
                var $52=$p; //@line 310 "dictionary.cpp"
                var $53=HEAP8[($52)]; //@line 310 "dictionary.cpp"
                var $conv1=(tempInt=(($53)),(tempInt>=128?tempInt-256:tempInt)); //@line 310 "dictionary.cpp"
                $length=$conv1; //@line 310 "dictionary.cpp"
                var $cmp582=((($conv1))|0)!=0; //@line 310 "dictionary.cpp"
                if ($cmp582) { __label__ = 16;; } else { __label__ = 17;; } //@line 310 "dictionary.cpp"
                $while_body$$while_end$19: do { 
                  if (__label__ == 16) {
                    while(1) { 
  
                      var $54=$length; //@line 312 "dictionary.cpp"
                      var $55=$p; //@line 312 "dictionary.cpp"
                      var $add_ptr=(($55+$54)&4294967295); //@line 312 "dictionary.cpp"
                      $p=$add_ptr; //@line 312 "dictionary.cpp"
                      var $56=$p; //@line 310 "dictionary.cpp"
                      var $57=HEAP8[($56)]; //@line 310 "dictionary.cpp"
                      var $conv=(tempInt=(($57)),(tempInt>=128?tempInt-256:tempInt)); //@line 310 "dictionary.cpp"
                      $length=$conv; //@line 310 "dictionary.cpp"
                      var $cmp58=((($conv))|0)!=0; //@line 310 "dictionary.cpp"
                      if ($cmp58) { __label__ = 16;continue ; } else { __label__ = 17;break $while_body$$while_end$19; } //@line 310 "dictionary.cpp"
                    }
                  }
                } while(0);
  
                var $58=$p; //@line 314 "dictionary.cpp"
                var $incdec_ptr=(($58+1)&4294967295); //@line 314 "dictionary.cpp"
                $p=$incdec_ptr; //@line 314 "dictionary.cpp"
                var $59=$hash; //@line 307 "dictionary.cpp"
                var $inc=((($59)+1)&4294967295); //@line 307 "dictionary.cpp"
                $hash=$inc; //@line 307 "dictionary.cpp"
                var $cmp56=((($inc))|0) < 1024; //@line 307 "dictionary.cpp"
                if ($cmp56) { __label__ = 15;continue $for_body$17; } else { __label__ = 18;break $for_body$17; } //@line 307 "dictionary.cpp"
              }
  
              $retval=0; //@line 317 "dictionary.cpp"
              __label__ = 19;break $if_then12$$lor_lhs_false$5; //@line 317 "dictionary.cpp"
            }
          } while(0);
  
          var $32=HEAP32[((__impure_ptr)>>2)]; //@line 292 "dictionary.cpp"
          var $_stderr40=(($32+12)&4294967295); //@line 292 "dictionary.cpp"
          var $33=HEAP32[(($_stderr40)>>2)]; //@line 292 "dictionary.cpp"
          var $arraydecay41=(($fname)&4294967295); //@line 292 "dictionary.cpp"
          var $34=$pw; //@line 292 "dictionary.cpp"
          var $arrayidx42=(($34)&4294967295); //@line 292 "dictionary.cpp"
          var $35=HEAP32[(($arrayidx42)>>2)]; //@line 292 "dictionary.cpp"
          var $call43=__Z13Reverse4Bytesi($35); //@line 292 "dictionary.cpp"
          var $36=$length; //@line 292 "dictionary.cpp"
          var $call44=_fprintf($33, ((__str4120)&4294967295), allocate([$arraydecay41,0,0,0,$call43,0,0,0,$36,0,0,0], ["i8*",0,0,0,"i32",0,0,0,"i32",0,0,0], ALLOC_STACK)); //@line 292 "dictionary.cpp"
          $retval=2; //@line 293 "dictionary.cpp"
          __label__ = 19;break $if_then12$$lor_lhs_false$5; //@line 293 "dictionary.cpp"
        }
      }
    } while(0);
    if (__label__ == 4) {
  
      var $10=$no_error_addr; //@line 268 "dictionary.cpp"
      var $cmp13=((($10))|0)==0; //@line 268 "dictionary.cpp"
      if ($cmp13) { __label__ = 5;; } else { __label__ = 6;; } //@line 268 "dictionary.cpp"
      if (__label__ == 5) {
  
        var $11=HEAP32[((__impure_ptr)>>2)]; //@line 270 "dictionary.cpp"
        var $_stderr=(($11+12)&4294967295); //@line 270 "dictionary.cpp"
        var $12=HEAP32[(($_stderr)>>2)]; //@line 270 "dictionary.cpp"
        var $arraydecay15=(($fname)&4294967295); //@line 270 "dictionary.cpp"
        var $call16=_fprintf($12, ((__str2118)&4294967295), allocate([$arraydecay15,0,0,0], ["i8*",0,0,0], ALLOC_STACK)); //@line 270 "dictionary.cpp"
        ; //@line 271 "dictionary.cpp"
      }
  
      $retval=1; //@line 272 "dictionary.cpp"
      ; //@line 272 "dictionary.cpp"
    }
  
    var $60=$retval; //@line 318 "dictionary.cpp"
    STACKTOP = __stackBase__;
    return $60; //@line 318 "dictionary.cpp"
    return null;
  }
  

  function __ZL10InitGroupsP10Translator($tr) {
    ;
    var __label__;
    var __lastLabel__ = null;
  
    var $tr_addr;
    var $ix;
    var $p;
    var $p_name;
    var $pw;
    var $c;
    var $c2;
    var $len;
    $tr_addr=$tr;
    var $0=$tr_addr; //@line 142 "dictionary.cpp"
    var $n_groups2=(($0+7752)&4294967295); //@line 142 "dictionary.cpp"
    HEAP32[(($n_groups2)>>2)]=0; //@line 142 "dictionary.cpp"
    $ix=0; //@line 143 "dictionary.cpp"
    ; //@line 143 "dictionary.cpp"
    $for_body$2: while(1) { 
  
      var $1=$ix; //@line 145 "dictionary.cpp"
      var $2=$tr_addr; //@line 145 "dictionary.cpp"
      var $groups1=(($2+5256)&4294967295); //@line 145 "dictionary.cpp"
      var $arrayidx=(($groups1+$1*4)&4294967295); //@line 145 "dictionary.cpp"
      HEAP32[(($arrayidx)>>2)]=0; //@line 145 "dictionary.cpp"
      var $3=$ix; //@line 146 "dictionary.cpp"
      var $4=$tr_addr; //@line 146 "dictionary.cpp"
      var $groups2_count=(($4+7756)&4294967295); //@line 146 "dictionary.cpp"
      var $arrayidx1=(($groups2_count+$3)&4294967295); //@line 146 "dictionary.cpp"
      HEAP8[($arrayidx1)]=0; //@line 146 "dictionary.cpp"
      var $5=$ix; //@line 147 "dictionary.cpp"
      var $6=$tr_addr; //@line 147 "dictionary.cpp"
      var $groups2_start=(($6+8012)&4294967295); //@line 147 "dictionary.cpp"
      var $arrayidx2=(($groups2_start+$5)&4294967295); //@line 147 "dictionary.cpp"
      HEAP8[($arrayidx2)]=-1; //@line 147 "dictionary.cpp"
      var $7=$ix; //@line 143 "dictionary.cpp"
      var $inc=((($7)+1)&4294967295); //@line 143 "dictionary.cpp"
      $ix=$inc; //@line 143 "dictionary.cpp"
      var $cmp=((($inc))|0) < 256; //@line 143 "dictionary.cpp"
      if ($cmp) { __label__ = 1;continue $for_body$2; } else { __label__ = 2;break $for_body$2; } //@line 143 "dictionary.cpp"
    }
  
    var $8=$tr_addr; //@line 149 "dictionary.cpp"
    var $letterGroups=(($8+5152)&4294967295); //@line 149 "dictionary.cpp"
    var $arraydecay=(($letterGroups)&4294967295); //@line 149 "dictionary.cpp"
    var $9=$arraydecay; //@line 149 "dictionary.cpp"
    var $dest$, $stop$, $stop4$, $fast$, $value4$;
    $dest$ = $9;
    $stop$ = $dest$ + 104;
    $value4$ = 0;
    if ($value4$ < 0) $value4$ += 256;
    $value4$ = $value4$ + ($value4$<<8) + ($value4$<<16) + ($value4$*16777216);
    while ($dest$%4 !== 0 && $dest$ < $stop$) {
      ;; HEAP8[$dest$++] = 0;
    }
    $dest$ >>= 2;
    $stop4$ = $stop$ >> 2;
    while ($dest$ < $stop4$) {
    ;;;;;;;  HEAP32[$dest$++] = $value4$;
    }
    $dest$ <<= 2;
    while ($dest$ < $stop$) {
      ;; HEAP8[$dest$++] = 0;
    }; //@line 149 "dictionary.cpp"
    var $10=$tr_addr; //@line 150 "dictionary.cpp"
    var $groups3=(($10+6280)&4294967295); //@line 150 "dictionary.cpp"
    var $arraydecay3=(($groups3)&4294967295); //@line 150 "dictionary.cpp"
    var $11=$arraydecay3; //@line 150 "dictionary.cpp"
    var $dest$, $stop$, $stop4$, $fast$, $value4$;
    $dest$ = $11;
    $stop$ = $dest$ + 512;
    $value4$ = 0;
    if ($value4$ < 0) $value4$ += 256;
    $value4$ = $value4$ + ($value4$<<8) + ($value4$<<16) + ($value4$*16777216);
    while ($dest$%4 !== 0 && $dest$ < $stop$) {
      ;; HEAP8[$dest$++] = 0;
    }
    $dest$ >>= 2;
    $stop4$ = $stop$ >> 2;
    while ($dest$ < $stop4$) {
    ;;;;;;;  HEAP32[$dest$++] = $value4$;
    }
    $dest$ <<= 2;
    while ($dest$ < $stop$) {
      ;; HEAP8[$dest$++] = 0;
    }; //@line 150 "dictionary.cpp"
    var $12=$tr_addr; //@line 152 "dictionary.cpp"
    var $data_dictrules=(($12+1048)&4294967295); //@line 152 "dictionary.cpp"
    var $13=HEAP32[(($data_dictrules)>>2)]; //@line 152 "dictionary.cpp"
    $p=$13; //@line 152 "dictionary.cpp"
    ; //@line 153 "dictionary.cpp"
    $while_cond$5: while(1) { 
  
      var $14=$p; //@line 153 "dictionary.cpp"
      var $15=HEAP8[($14)]; //@line 153 "dictionary.cpp"
      var $conv=(tempInt=(($15)),(tempInt>=128?tempInt-256:tempInt)); //@line 153 "dictionary.cpp"
      var $cmp4=((($conv))|0)!=0; //@line 153 "dictionary.cpp"
      if (!($cmp4)) { __label__ = 25;break $while_cond$5; } //@line 153 "dictionary.cpp"
  
      var $16=$p; //@line 155 "dictionary.cpp"
      var $17=HEAP8[($16)]; //@line 155 "dictionary.cpp"
      var $conv5=(tempInt=(($17)),(tempInt>=128?tempInt-256:tempInt)); //@line 155 "dictionary.cpp"
      var $cmp6=((($conv5))|0)!=6; //@line 155 "dictionary.cpp"
      if ($cmp6) { __label__ = 5;break $while_cond$5; } //@line 155 "dictionary.cpp"
  
      var $23=$p; //@line 160 "dictionary.cpp"
      var $incdec_ptr=(($23+1)&4294967295); //@line 160 "dictionary.cpp"
      $p=$incdec_ptr; //@line 160 "dictionary.cpp"
      var $24=$p; //@line 162 "dictionary.cpp"
      var $arrayidx8=(($24)&4294967295); //@line 162 "dictionary.cpp"
      var $25=HEAP8[($arrayidx8)]; //@line 162 "dictionary.cpp"
      var $conv9=(tempInt=(($25)),(tempInt>=128?tempInt-256:tempInt)); //@line 162 "dictionary.cpp"
      var $cmp10=((($conv9))|0)==20; //@line 162 "dictionary.cpp"
      var $26=$p; //@line 164 "dictionary.cpp"
      if ($cmp10) { __label__ = 7;; } else { __label__ = 10;; } //@line 162 "dictionary.cpp"
      if (__label__ == 7) {
  
        var $27=($26); //@line 164 "dictionary.cpp"
        var $add=((($27)+4)&4294967295); //@line 164 "dictionary.cpp"
        var $and=($add) & -4; //@line 164 "dictionary.cpp"
        var $28=($and); //@line 164 "dictionary.cpp"
        $pw=$28; //@line 164 "dictionary.cpp"
        var $29=$pw; //@line 165 "dictionary.cpp"
        var $30=$tr_addr; //@line 165 "dictionary.cpp"
        var $langopts=(($30)&4294967295); //@line 165 "dictionary.cpp"
        var $replace_chars=(($langopts+264)&4294967295); //@line 165 "dictionary.cpp"
        HEAP32[(($replace_chars)>>2)]=$29; //@line 165 "dictionary.cpp"
        var $31=$pw; //@line 166 "dictionary.cpp"
        var $arrayidx134=(($31)&4294967295); //@line 166 "dictionary.cpp"
        var $32=HEAP32[(($arrayidx134)>>2)]; //@line 166 "dictionary.cpp"
        var $cmp145=((($32))|0)!=0; //@line 166 "dictionary.cpp"
        var $33=$pw; //@line 168 "dictionary.cpp"
        if ($cmp145) { __lastLabel__ = 7; __label__ = 8;; } else { __lastLabel__ = 7; __label__ = 9;; } //@line 166 "dictionary.cpp"
        $while_body15$$while_end$11: do { 
          if (__label__ == 8) {
            while(1) { 
  
              var $34=__lastLabel__ == 8 ? $37 : ($33);
              var $add_ptr=(($34+8)&4294967295); //@line 168 "dictionary.cpp"
              $pw=$add_ptr; //@line 168 "dictionary.cpp"
              var $35=$pw; //@line 166 "dictionary.cpp"
              var $arrayidx13=(($35)&4294967295); //@line 166 "dictionary.cpp"
              var $36=HEAP32[(($arrayidx13)>>2)]; //@line 166 "dictionary.cpp"
              var $cmp14=((($36))|0)!=0; //@line 166 "dictionary.cpp"
              var $37=$pw; //@line 168 "dictionary.cpp"
              if ($cmp14) { __lastLabel__ = 8; __label__ = 8;continue ; } else { __lastLabel__ = 8; __label__ = 9;break $while_body15$$while_end$11; } //@line 166 "dictionary.cpp"
            }
          }
        } while(0);
  
        var $_lcssa3=__lastLabel__ == 7 ? $33 : ($37);
        var $add_ptr16=(($_lcssa3+4)&4294967295); //@line 170 "dictionary.cpp"
        var $38=$add_ptr16; //@line 170 "dictionary.cpp"
        $p=$38; //@line 170 "dictionary.cpp"
        __label__ = 3;continue $while_cond$5; //@line 182 "dictionary.cpp"
      }
      else if (__label__ == 10) {
  
        var $arrayidx18=(($26)&4294967295); //@line 185 "dictionary.cpp"
        var $39=HEAP8[($arrayidx18)]; //@line 185 "dictionary.cpp"
        var $conv19=(tempInt=(($39)),(tempInt>=128?tempInt-256:tempInt)); //@line 185 "dictionary.cpp"
        var $cmp20=((($conv19))|0)==18; //@line 185 "dictionary.cpp"
        var $40=$p; //@line 187 "dictionary.cpp"
        if ($cmp20) { __label__ = 11;; } else { __label__ = 13;; } //@line 185 "dictionary.cpp"
        $if_then21$$if_else$16: do { 
          if (__label__ == 11) {
  
            var $arrayidx22=(($40+1)&4294967295); //@line 187 "dictionary.cpp"
            var $41=HEAP8[($arrayidx22)]; //@line 187 "dictionary.cpp"
            var $conv23=(tempInt=(($41)),(tempInt>=128?tempInt-256:tempInt)); //@line 187 "dictionary.cpp"
            var $sub=((($conv23)-65)&4294967295); //@line 187 "dictionary.cpp"
            $ix=$sub; //@line 187 "dictionary.cpp"
            var $42=$p; //@line 188 "dictionary.cpp"
            var $add_ptr24=(($42+2)&4294967295); //@line 188 "dictionary.cpp"
            $p=$add_ptr24; //@line 188 "dictionary.cpp"
            var $43=$ix; //@line 189 "dictionary.cpp"
            var $cmp25=((($43))|0) >= 0; //@line 189 "dictionary.cpp"
            var $44=$ix; //@line 189 "dictionary.cpp"
            var $cmp26=((($44))|0) < 26; //@line 189 "dictionary.cpp"
            var $or_cond=($cmp25) & ($cmp26); //@line 189 "dictionary.cpp"
            if (!($or_cond)) { __label__ = 22;break $if_then21$$if_else$16; } //@line 189 "dictionary.cpp"
  
            var $45=$p; //@line 191 "dictionary.cpp"
            var $46=$ix; //@line 191 "dictionary.cpp"
            var $47=$tr_addr; //@line 191 "dictionary.cpp"
            var $letterGroups28=(($47+5152)&4294967295); //@line 191 "dictionary.cpp"
            var $arrayidx29=(($letterGroups28+$46*4)&4294967295); //@line 191 "dictionary.cpp"
            HEAP32[(($arrayidx29)>>2)]=$45; //@line 191 "dictionary.cpp"
            ; //@line 192 "dictionary.cpp"
          }
          else if (__label__ == 13) {
  
            var $call31=_strlen($40); //@line 196 "dictionary.cpp"
            $len=$call31; //@line 196 "dictionary.cpp"
            var $48=$p; //@line 197 "dictionary.cpp"
            $p_name=$48; //@line 197 "dictionary.cpp"
            var $49=$p_name; //@line 198 "dictionary.cpp"
            var $arrayidx32=(($49)&4294967295); //@line 198 "dictionary.cpp"
            var $50=HEAP8[($arrayidx32)]; //@line 198 "dictionary.cpp"
            $c=$50; //@line 198 "dictionary.cpp"
            var $51=$p_name; //@line 199 "dictionary.cpp"
            var $arrayidx33=(($51+1)&4294967295); //@line 199 "dictionary.cpp"
            var $52=HEAP8[($arrayidx33)]; //@line 199 "dictionary.cpp"
            $c2=$52; //@line 199 "dictionary.cpp"
            var $53=$len; //@line 201 "dictionary.cpp"
            var $add34=((($53)+1)&4294967295); //@line 201 "dictionary.cpp"
            var $54=$p; //@line 201 "dictionary.cpp"
            var $add_ptr35=(($54+$add34)&4294967295); //@line 201 "dictionary.cpp"
            $p=$add_ptr35; //@line 201 "dictionary.cpp"
            var $55=$len; //@line 202 "dictionary.cpp"
            var $cmp36=((($55))|0)==1; //@line 202 "dictionary.cpp"
            if ($cmp36) { __label__ = 14;; } else { __label__ = 15;; } //@line 202 "dictionary.cpp"
            if (__label__ == 14) {
  
              var $56=$p; //@line 204 "dictionary.cpp"
              var $57=$c; //@line 204 "dictionary.cpp"
              var $idxprom=((($57))&255); //@line 204 "dictionary.cpp"
              var $58=$tr_addr; //@line 204 "dictionary.cpp"
              var $groups138=(($58+5256)&4294967295); //@line 204 "dictionary.cpp"
              var $arrayidx39=(($groups138+$idxprom*4)&4294967295); //@line 204 "dictionary.cpp"
              HEAP32[(($arrayidx39)>>2)]=$56; //@line 204 "dictionary.cpp"
              ; //@line 205 "dictionary.cpp"
            }
            else if (__label__ == 15) {
  
              var $59=$len; //@line 207 "dictionary.cpp"
              var $cmp41=((($59))|0)==0; //@line 207 "dictionary.cpp"
              if ($cmp41) { __label__ = 16;; } else { __label__ = 17;; } //@line 207 "dictionary.cpp"
              if (__label__ == 16) {
  
                var $60=$p; //@line 209 "dictionary.cpp"
                var $61=$tr_addr; //@line 209 "dictionary.cpp"
                var $groups143=(($61+5256)&4294967295); //@line 209 "dictionary.cpp"
                var $arrayidx44=(($groups143)&4294967295); //@line 209 "dictionary.cpp"
                HEAP32[(($arrayidx44)>>2)]=$60; //@line 209 "dictionary.cpp"
                ; //@line 210 "dictionary.cpp"
              }
              else if (__label__ == 17) {
  
                var $62=$c; //@line 212 "dictionary.cpp"
                var $conv46=((($62))&255); //@line 212 "dictionary.cpp"
                var $cmp47=((($conv46))|0)==1; //@line 212 "dictionary.cpp"
                if ($cmp47) { __label__ = 18;; } else { __label__ = 19;; } //@line 212 "dictionary.cpp"
                if (__label__ == 18) {
  
                  var $63=$p; //@line 215 "dictionary.cpp"
                  var $64=$c2; //@line 215 "dictionary.cpp"
                  var $conv49=((($64))&255); //@line 215 "dictionary.cpp"
                  var $sub50=((($conv49)-1)&4294967295); //@line 215 "dictionary.cpp"
                  var $65=$tr_addr; //@line 215 "dictionary.cpp"
                  var $groups351=(($65+6280)&4294967295); //@line 215 "dictionary.cpp"
                  var $arrayidx52=(($groups351+$sub50*4)&4294967295); //@line 215 "dictionary.cpp"
                  HEAP32[(($arrayidx52)>>2)]=$63; //@line 215 "dictionary.cpp"
                  ; //@line 216 "dictionary.cpp"
                }
                else if (__label__ == 19) {
  
                  var $66=$c; //@line 219 "dictionary.cpp"
                  var $idxprom54=((($66))&255); //@line 219 "dictionary.cpp"
                  var $67=$tr_addr; //@line 219 "dictionary.cpp"
                  var $groups2_start55=(($67+8012)&4294967295); //@line 219 "dictionary.cpp"
                  var $arrayidx56=(($groups2_start55+$idxprom54)&4294967295); //@line 219 "dictionary.cpp"
                  var $68=HEAPU8[($arrayidx56)]; //@line 219 "dictionary.cpp"
                  var $conv57=((($68))&255); //@line 219 "dictionary.cpp"
                  var $cmp58=((($conv57))|0)==255; //@line 219 "dictionary.cpp"
                  if ($cmp58) { __label__ = 20;; } else { __label__ = 21;; } //@line 219 "dictionary.cpp"
                  if (__label__ == 20) {
  
                    var $69=$tr_addr; //@line 220 "dictionary.cpp"
                    var $n_groups260=(($69+7752)&4294967295); //@line 220 "dictionary.cpp"
                    var $70=HEAP32[(($n_groups260)>>2)]; //@line 220 "dictionary.cpp"
                    var $conv61=((($70)) & 255); //@line 220 "dictionary.cpp"
                    var $71=$c; //@line 220 "dictionary.cpp"
                    var $idxprom62=((($71))&255); //@line 220 "dictionary.cpp"
                    var $72=$tr_addr; //@line 220 "dictionary.cpp"
                    var $groups2_start63=(($72+8012)&4294967295); //@line 220 "dictionary.cpp"
                    var $arrayidx64=(($groups2_start63+$idxprom62)&4294967295); //@line 220 "dictionary.cpp"
                    HEAP8[($arrayidx64)]=$conv61; //@line 220 "dictionary.cpp"
                    ; //@line 220 "dictionary.cpp"
                  }
  
                  var $73=$c; //@line 222 "dictionary.cpp"
                  var $idxprom66=((($73))&255); //@line 222 "dictionary.cpp"
                  var $74=$tr_addr; //@line 222 "dictionary.cpp"
                  var $groups2_count67=(($74+7756)&4294967295); //@line 222 "dictionary.cpp"
                  var $arrayidx68=(($groups2_count67+$idxprom66)&4294967295); //@line 222 "dictionary.cpp"
                  var $75=HEAP8[($arrayidx68)]; //@line 222 "dictionary.cpp"
                  var $inc69=((($75)+1)&255); //@line 222 "dictionary.cpp"
                  HEAP8[($arrayidx68)]=$inc69; //@line 222 "dictionary.cpp"
                  var $76=$p; //@line 223 "dictionary.cpp"
                  var $77=$tr_addr; //@line 223 "dictionary.cpp"
                  var $n_groups270=(($77+7752)&4294967295); //@line 223 "dictionary.cpp"
                  var $78=HEAP32[(($n_groups270)>>2)]; //@line 223 "dictionary.cpp"
                  var $79=$tr_addr; //@line 223 "dictionary.cpp"
                  var $groups2=(($79+6792)&4294967295); //@line 223 "dictionary.cpp"
                  var $arrayidx71=(($groups2+$78*4)&4294967295); //@line 223 "dictionary.cpp"
                  HEAP32[(($arrayidx71)>>2)]=$76; //@line 223 "dictionary.cpp"
                  var $80=$c; //@line 224 "dictionary.cpp"
                  var $conv72=((($80))&255); //@line 224 "dictionary.cpp"
                  var $81=$c2; //@line 224 "dictionary.cpp"
                  var $conv73=((($81))&255); //@line 224 "dictionary.cpp"
                  var $shl=((($conv73)*256)&4294967295); //@line 224 "dictionary.cpp"
                  var $add74=((($shl)+($conv72))&4294967295); //@line 224 "dictionary.cpp"
                  var $82=$tr_addr; //@line 224 "dictionary.cpp"
                  var $n_groups275=(($82+7752)&4294967295); //@line 224 "dictionary.cpp"
                  var $83=HEAPU32[(($n_groups275)>>2)]; //@line 224 "dictionary.cpp"
                  var $inc76=((($83)+1)&4294967295); //@line 224 "dictionary.cpp"
                  HEAP32[(($n_groups275)>>2)]=$inc76; //@line 224 "dictionary.cpp"
                  var $84=$tr_addr; //@line 224 "dictionary.cpp"
                  var $groups2_name=(($84+7272)&4294967295); //@line 224 "dictionary.cpp"
                  var $arrayidx77=(($groups2_name+$83*4)&4294967295); //@line 224 "dictionary.cpp"
                  HEAP32[(($arrayidx77)>>2)]=$add74; //@line 224 "dictionary.cpp"
                  ;
                }
              }
            }
          }
        } while(0);
  
        var $85=$p; //@line 229 "dictionary.cpp"
        var $86=HEAP8[($85)]; //@line 229 "dictionary.cpp"
        var $conv831=(tempInt=(($86)),(tempInt>=128?tempInt-256:tempInt)); //@line 229 "dictionary.cpp"
        var $cmp842=((($conv831))|0)!=7; //@line 229 "dictionary.cpp"
        var $87=$p; //@line 231 "dictionary.cpp"
        if ($cmp842) { __lastLabel__ = 22; __label__ = 23;; } else { __lastLabel__ = 22; __label__ = 24;; } //@line 229 "dictionary.cpp"
        $while_body85$$while_end89$33: do { 
          if (__label__ == 23) {
            while(1) { 
  
              var $88=__lastLabel__ == 23 ? $92 : ($87);
              var $call86=_strlen($88); //@line 231 "dictionary.cpp"
              var $add87=((($call86)+1)&4294967295); //@line 231 "dictionary.cpp"
              var $89=$p; //@line 231 "dictionary.cpp"
              var $add_ptr88=(($89+$add87)&4294967295); //@line 231 "dictionary.cpp"
              $p=$add_ptr88; //@line 231 "dictionary.cpp"
              var $90=$p; //@line 229 "dictionary.cpp"
              var $91=HEAP8[($90)]; //@line 229 "dictionary.cpp"
              var $conv83=(tempInt=(($91)),(tempInt>=128?tempInt-256:tempInt)); //@line 229 "dictionary.cpp"
              var $cmp84=((($conv83))|0)!=7; //@line 229 "dictionary.cpp"
              var $92=$p; //@line 231 "dictionary.cpp"
              if ($cmp84) { __lastLabel__ = 23; __label__ = 23;continue ; } else { __lastLabel__ = 23; __label__ = 24;break $while_body85$$while_end89$33; } //@line 229 "dictionary.cpp"
            }
          }
        } while(0);
  
        var $_lcssa=__lastLabel__ == 22 ? $87 : ($92);
        var $incdec_ptr90=(($_lcssa+1)&4294967295); //@line 233 "dictionary.cpp"
        $p=$incdec_ptr90; //@line 233 "dictionary.cpp"
        __label__ = 3;continue $while_cond$5; //@line 234 "dictionary.cpp"
      }
    }
    if (__label__ == 5) {
  
      var $18=HEAP32[((__impure_ptr)>>2)]; //@line 157 "dictionary.cpp"
      var $_stderr=(($18+12)&4294967295); //@line 157 "dictionary.cpp"
      var $19=HEAP32[(($_stderr)>>2)]; //@line 157 "dictionary.cpp"
      var $20=$p; //@line 157 "dictionary.cpp"
      var $21=$tr_addr; //@line 157 "dictionary.cpp"
      var $data_dictrules7=(($21+1048)&4294967295); //@line 157 "dictionary.cpp"
      var $22=HEAP32[(($data_dictrules7)>>2)]; //@line 157 "dictionary.cpp"
      var $sub_ptr_lhs_cast=($20); //@line 157 "dictionary.cpp"
      var $sub_ptr_rhs_cast=($22); //@line 157 "dictionary.cpp"
      var $sub_ptr_sub=((($sub_ptr_lhs_cast)-($sub_ptr_rhs_cast))&4294967295); //@line 157 "dictionary.cpp"
      var $call=_fprintf($19, ((__str34150)&4294967295), allocate([((_dictionary_name)&4294967295),0,0,0,$sub_ptr_sub,0,0,0], ["i8*",0,0,0,"i32",0,0,0], ALLOC_STACK)); //@line 157 "dictionary.cpp"
      ; //@line 158 "dictionary.cpp"
    }
  
    ;
    return; //@line 236 "dictionary.cpp"
    return;
  }
  

  function __Z14HashDictionaryPKc($string) {
    ;
    var __label__;
    var __lastLabel__ = null;
  
    var $string_addr;
    var $c;
    var $chars;
    var $hash;
    $string_addr=$string;
    $chars=0; //@line 328 "dictionary.cpp"
    $hash=0; //@line 329 "dictionary.cpp"
    var $0=$string_addr; //@line 331 "dictionary.cpp"
    var $incdec_ptr1=(($0+1)&4294967295); //@line 331 "dictionary.cpp"
    $string_addr=$incdec_ptr1; //@line 331 "dictionary.cpp"
    var $1=HEAP8[($0)]; //@line 331 "dictionary.cpp"
    var $conv2=(tempInt=(($1)),(tempInt>=128?tempInt-256:tempInt)); //@line 331 "dictionary.cpp"
    var $and4=($conv2) & 255; //@line 331 "dictionary.cpp"
    $c=$and4; //@line 331 "dictionary.cpp"
    var $cmp5=((($and4))|0)!=0; //@line 331 "dictionary.cpp"
    var $2=$hash; //@line 333 "dictionary.cpp"
    if ($cmp5) { __lastLabel__ = 0; __label__ = 1;; } else { __lastLabel__ = 0; __label__ = 2;; } //@line 331 "dictionary.cpp"
    $while_body$$while_end$2: do { 
      if (__label__ == 1) {
        while(1) { 
  
          var $3=__lastLabel__ == 1 ? $10 : ($2);
          var $mul=((($3)*8)&4294967295); //@line 333 "dictionary.cpp"
          var $4=$c; //@line 333 "dictionary.cpp"
          var $add=((($4)+($mul))&4294967295); //@line 333 "dictionary.cpp"
          $hash=$add; //@line 333 "dictionary.cpp"
          var $5=$hash; //@line 334 "dictionary.cpp"
          var $and1=($5) & 1023; //@line 334 "dictionary.cpp"
          var $6=$hash; //@line 334 "dictionary.cpp"
          var $shr=($6) >> 8; //@line 334 "dictionary.cpp"
          var $xor=($shr) ^ ($and1); //@line 334 "dictionary.cpp"
          $hash=$xor; //@line 334 "dictionary.cpp"
          var $7=$chars; //@line 335 "dictionary.cpp"
          var $inc=((($7)+1)&4294967295); //@line 335 "dictionary.cpp"
          $chars=$inc; //@line 335 "dictionary.cpp"
          var $8=$string_addr; //@line 331 "dictionary.cpp"
          var $incdec_ptr=(($8+1)&4294967295); //@line 331 "dictionary.cpp"
          $string_addr=$incdec_ptr; //@line 331 "dictionary.cpp"
          var $9=HEAP8[($8)]; //@line 331 "dictionary.cpp"
          var $conv=(tempInt=(($9)),(tempInt>=128?tempInt-256:tempInt)); //@line 331 "dictionary.cpp"
          var $and=($conv) & 255; //@line 331 "dictionary.cpp"
          $c=$and; //@line 331 "dictionary.cpp"
          var $cmp=((($and))|0)!=0; //@line 331 "dictionary.cpp"
          var $10=$hash; //@line 333 "dictionary.cpp"
          if ($cmp) { __lastLabel__ = 1; __label__ = 1;continue ; } else { __lastLabel__ = 1; __label__ = 2;break $while_body$$while_end$2; } //@line 331 "dictionary.cpp"
        }
      }
    } while(0);
  
    var $_lcssa=__lastLabel__ == 0 ? $2 : ($10);
    var $11=$chars; //@line 338 "dictionary.cpp"
    var $add2=((($11)+($_lcssa))&4294967295); //@line 338 "dictionary.cpp"
    var $and3=($add2) & 1023; //@line 338 "dictionary.cpp"
    ;
    return $and3; //@line 338 "dictionary.cpp"
    return null;
  }
  

  function __Z14EncodePhonemesPcS_Ph($p, $outptr, $bad_phoneme) {
    ;
    var __label__;
    var __lastLabel__ = null;
  
    var $retval;
    var $p_addr;
    var $outptr_addr;
    var $bad_phoneme_addr;
    var $ix;
    var $c;
    var $count;
    var $max;
    var $max_ph;
    var $consumed;
    var $mnemonic_word;
    var $p_lang;
    $p_addr=$p;
    $outptr_addr=$outptr;
    $bad_phoneme_addr=$bad_phoneme;
    var $0=$bad_phoneme_addr; //@line 367 "dictionary.cpp"
    var $arrayidx=(($0)&4294967295); //@line 367 "dictionary.cpp"
    HEAP8[($arrayidx)]=0; //@line 367 "dictionary.cpp"
    var $1=$p_addr; //@line 370 "dictionary.cpp"
    var $2=HEAP8[($1)]; //@line 370 "dictionary.cpp"
    var $conv4=(tempInt=(($2)),(tempInt>=128?tempInt-256:tempInt)); //@line 370 "dictionary.cpp"
    var $call5=_isspace($conv4); //@line 370 "dictionary.cpp"
    var $tobool6=((($call5))|0)!=0; //@line 370 "dictionary.cpp"
    if ($tobool6) { __label__ = 1;; } else { __label__ = 2;; } //@line 370 "dictionary.cpp"
    $while_body$$while_cond1$2: do { 
      if (__label__ == 1) {
        while(1) { 
  
          var $3=$p_addr; //@line 372 "dictionary.cpp"
          var $incdec_ptr=(($3+1)&4294967295); //@line 372 "dictionary.cpp"
          $p_addr=$incdec_ptr; //@line 372 "dictionary.cpp"
          var $4=$p_addr; //@line 370 "dictionary.cpp"
          var $5=HEAP8[($4)]; //@line 370 "dictionary.cpp"
          var $conv=(tempInt=(($5)),(tempInt>=128?tempInt-256:tempInt)); //@line 370 "dictionary.cpp"
          var $call=_isspace($conv); //@line 370 "dictionary.cpp"
          var $tobool=((($call))|0)!=0; //@line 370 "dictionary.cpp"
          if ($tobool) { __label__ = 1;continue ; } else { __label__ = 2;break $while_body$$while_cond1$2; } //@line 370 "dictionary.cpp"
        }
      }
    } while(0);
    $while_cond1$5: while(1) { 
  
      var $6=$p_addr; //@line 375 "dictionary.cpp"
      var $7=HEAPU8[($6)]; //@line 375 "dictionary.cpp"
      $c=$7; //@line 375 "dictionary.cpp"
      var $conv2=((($7))&255); //@line 375 "dictionary.cpp"
      var $cmp=((($conv2))|0)!=0; //@line 375 "dictionary.cpp"
      if (!($cmp)) { __label__ = 34;break $while_cond1$5; } //@line 375 "dictionary.cpp"
  
      var $8=$c; //@line 375 "dictionary.cpp"
      var $conv3=((($8))&255); //@line 375 "dictionary.cpp"
      var $call4=_isspace($conv3); //@line 375 "dictionary.cpp"
      var $tobool5=((($call4))|0)!=0; //@line 375 "dictionary.cpp"
      var $lnot=($tobool5) ^ 1; //@line 375 "dictionary.cpp"
      if (!($lnot)) { __label__ = 34;break $while_cond1$5; }
  
      $consumed=0; //@line 377 "dictionary.cpp"
      var $9=$c; //@line 379 "dictionary.cpp"
      var $conv7=((($9))&255); //@line 379 "dictionary.cpp"
      var $cond=((($conv7))|0)==124; //@line 379 "dictionary.cpp"
      if ($cond) { __label__ = 5;; } else { __label__ = 7;; } //@line 379 "dictionary.cpp"
      $sw_bb$$sw_default$9: do { 
        if (__label__ == 5) {
  
          var $10=$p_addr; //@line 385 "dictionary.cpp"
          var $arrayidx8=(($10+1)&4294967295); //@line 385 "dictionary.cpp"
          var $11=HEAPU8[($arrayidx8)]; //@line 385 "dictionary.cpp"
          $c=$11; //@line 385 "dictionary.cpp"
          var $conv9=((($11))&255); //@line 385 "dictionary.cpp"
          var $cmp10=((($conv9))|0)==124; //@line 385 "dictionary.cpp"
          if ($cmp10) { __label__ = 7;break $sw_bb$$sw_default$9; } //@line 385 "dictionary.cpp"
  
          var $12=$p_addr; //@line 392 "dictionary.cpp"
          var $incdec_ptr11=(($12+1)&4294967295); //@line 392 "dictionary.cpp"
          $p_addr=$incdec_ptr11; //@line 392 "dictionary.cpp"
          __label__ = 2;continue $while_cond1$5; //@line 393 "dictionary.cpp"
        }
      } while(0);
  
      $max=-1; //@line 399 "dictionary.cpp"
      $max_ph=0; //@line 400 "dictionary.cpp"
      $ix=1; //@line 402 "dictionary.cpp"
      var $13=$ix; //@line 402 "dictionary.cpp"
      var $14=HEAP32[((_n_phoneme_tab)>>2)]; //@line 402 "dictionary.cpp"
      var $cmp121=((($13))|0) < ((($14))|0); //@line 402 "dictionary.cpp"
      if (!($cmp121)) { __label__ = 22;break $while_cond1$5; } //@line 402 "dictionary.cpp"
      $for_body$13: while(1) { 
  
        var $15=$ix; //@line 404 "dictionary.cpp"
        var $arrayidx13=((_phoneme_tab+$15*4)&4294967295); //@line 404 "dictionary.cpp"
        var $16=HEAP32[(($arrayidx13)>>2)]; //@line 404 "dictionary.cpp"
        var $cmp14=((($16))|0)==0; //@line 404 "dictionary.cpp"
        if ($cmp14) { __label__ = 20;; } else { __label__ = 9;; } //@line 404 "dictionary.cpp"
        $for_inc$$if_end16$15: do { 
          if (__label__ == 9) {
  
            var $17=$ix; //@line 406 "dictionary.cpp"
            var $arrayidx17=((_phoneme_tab+$17*4)&4294967295); //@line 406 "dictionary.cpp"
            var $18=HEAP32[(($arrayidx17)>>2)]; //@line 406 "dictionary.cpp"
            var $type=(($18+11)&4294967295); //@line 406 "dictionary.cpp"
            var $19=HEAPU8[($type)]; //@line 406 "dictionary.cpp"
            var $conv18=((($19))&255); //@line 406 "dictionary.cpp"
            var $cmp19=((($conv18))|0)==15; //@line 406 "dictionary.cpp"
            if ($cmp19) { __label__ = 20;break $for_inc$$if_end16$15; } //@line 406 "dictionary.cpp"
  
            $count=0; //@line 409 "dictionary.cpp"
            var $20=$ix; //@line 410 "dictionary.cpp"
            var $arrayidx22=((_phoneme_tab+$20*4)&4294967295); //@line 410 "dictionary.cpp"
            var $21=HEAP32[(($arrayidx22)>>2)]; //@line 410 "dictionary.cpp"
            var $mnemonic=(($21)&4294967295); //@line 410 "dictionary.cpp"
            var $22=HEAP32[(($mnemonic)>>2)]; //@line 410 "dictionary.cpp"
            $mnemonic_word=$22; //@line 410 "dictionary.cpp"
            ; //@line 412 "dictionary.cpp"
            while(1) { 
  
              var $23=$count; //@line 412 "dictionary.cpp"
              var $24=$p_addr; //@line 412 "dictionary.cpp"
              var $arrayidx24=(($24+$23)&4294967295); //@line 412 "dictionary.cpp"
              var $25=HEAPU8[($arrayidx24)]; //@line 412 "dictionary.cpp"
              $c=$25; //@line 412 "dictionary.cpp"
              var $conv25=((($25))&255); //@line 412 "dictionary.cpp"
              var $cmp26=((($conv25))|0) > 32; //@line 412 "dictionary.cpp"
              if (!($cmp26)) { __label__ = 13;break ; } //@line 412 "dictionary.cpp"
  
              var $26=$count; //@line 412 "dictionary.cpp"
              var $cmp27=((($26))|0) < 4; //@line 412 "dictionary.cpp"
              if (!($cmp27)) { __label__ = 13;break ; } //@line 412 "dictionary.cpp"
  
              var $28=$c; //@line 412 "dictionary.cpp"
              var $conv29=((($28))&255); //@line 412 "dictionary.cpp"
              var $29=$mnemonic_word; //@line 412 "dictionary.cpp"
              var $30=$count; //@line 412 "dictionary.cpp"
              var $mul=((($30)*8)&4294967295); //@line 412 "dictionary.cpp"
              var $shr=($29) >>> ((($mul))>>>0); //@line 412 "dictionary.cpp"
              var $and=($shr) & 255; //@line 412 "dictionary.cpp"
              var $cmp30=((($conv29))|0)==((($and))|0); //@line 412 "dictionary.cpp"
              var $31=$count; //@line 414 "dictionary.cpp"
              if ($cmp30) { __lastLabel__ = 14; ; } else { __lastLabel__ = 14; __label__ = 16;break ; }
  
              var $inc=((($31)+1)&4294967295); //@line 414 "dictionary.cpp"
              $count=$inc; //@line 414 "dictionary.cpp"
              __label__ = 11;continue ; //@line 414 "dictionary.cpp"
            }
            if (__label__ == 13) {
  
              var $27=$count; //@line 414 "dictionary.cpp"
              __lastLabel__ = 13; ;
            }
  
            var $32=__lastLabel__ == 13 ? $27 : ($31);
            var $33=$max; //@line 416 "dictionary.cpp"
            var $cmp34=((($32))|0) > ((($33))|0); //@line 416 "dictionary.cpp"
            if (!($cmp34)) { __label__ = 20;break $for_inc$$if_end16$15; } //@line 416 "dictionary.cpp"
  
            var $34=$count; //@line 416 "dictionary.cpp"
            var $cmp36=((($34))|0)==4; //@line 416 "dictionary.cpp"
            if ($cmp36) { __label__ = 19;; } else { __label__ = 18;; } //@line 416 "dictionary.cpp"
            if (__label__ == 18) {
  
              var $35=$mnemonic_word; //@line 416 "dictionary.cpp"
              var $36=$count; //@line 416 "dictionary.cpp"
              var $mul37=((($36)*8)&4294967295); //@line 416 "dictionary.cpp"
              var $shr38=($35) >>> ((($mul37))>>>0); //@line 416 "dictionary.cpp"
              var $and39=($shr38) & 255; //@line 416 "dictionary.cpp"
              var $cmp40=((($and39))|0)==0; //@line 416 "dictionary.cpp"
              if (!($cmp40)) { __label__ = 20;break $for_inc$$if_end16$15; } //@line 416 "dictionary.cpp"
            }
  
            var $37=$count; //@line 419 "dictionary.cpp"
            $max=$37; //@line 419 "dictionary.cpp"
            var $38=$ix; //@line 420 "dictionary.cpp"
            var $arrayidx42=((_phoneme_tab+$38*4)&4294967295); //@line 420 "dictionary.cpp"
            var $39=HEAP32[(($arrayidx42)>>2)]; //@line 420 "dictionary.cpp"
            var $code=(($39+10)&4294967295); //@line 420 "dictionary.cpp"
            var $40=HEAPU8[($code)]; //@line 420 "dictionary.cpp"
            var $conv43=((($40))&255); //@line 420 "dictionary.cpp"
            $max_ph=$conv43; //@line 420 "dictionary.cpp"
            ; //@line 421 "dictionary.cpp"
          }
        } while(0);
  
        var $41=$ix; //@line 402 "dictionary.cpp"
        var $inc45=((($41)+1)&4294967295); //@line 402 "dictionary.cpp"
        $ix=$inc45; //@line 402 "dictionary.cpp"
        var $42=$ix; //@line 402 "dictionary.cpp"
        var $43=HEAP32[((_n_phoneme_tab)>>2)]; //@line 402 "dictionary.cpp"
        var $cmp12=((($42))|0) < ((($43))|0); //@line 402 "dictionary.cpp"
        if ($cmp12) { __label__ = 8;continue $for_body$13; } else { __label__ = 21;break $for_body$13; } //@line 402 "dictionary.cpp"
      }
  
      var $_pr=$max_ph; //@line 424 "dictionary.cpp"
      var $cmp46=((($_pr))|0)==0; //@line 424 "dictionary.cpp"
      if ($cmp46) { __label__ = 22;break $while_cond1$5; } //@line 424 "dictionary.cpp"
  
      var $50=$max; //@line 433 "dictionary.cpp"
      var $cmp52=((($50))|0) <= 0; //@line 433 "dictionary.cpp"
      if ($cmp52) { __label__ = 24;; } else { __label__ = 25;; } //@line 433 "dictionary.cpp"
      if (__label__ == 24) {
  
        $max=1; //@line 434 "dictionary.cpp"
        ; //@line 434 "dictionary.cpp"
      }
  
      var $51=$consumed; //@line 435 "dictionary.cpp"
      var $52=$max; //@line 435 "dictionary.cpp"
      var $add=((($52)+($51))&4294967295); //@line 435 "dictionary.cpp"
      var $53=$p_addr; //@line 435 "dictionary.cpp"
      var $add_ptr55=(($53+$add)&4294967295); //@line 435 "dictionary.cpp"
      $p_addr=$add_ptr55; //@line 435 "dictionary.cpp"
      var $54=$max_ph; //@line 436 "dictionary.cpp"
      var $conv56=((($54)) & 255); //@line 436 "dictionary.cpp"
      var $55=$outptr_addr; //@line 436 "dictionary.cpp"
      var $incdec_ptr57=(($55+1)&4294967295); //@line 436 "dictionary.cpp"
      $outptr_addr=$incdec_ptr57; //@line 436 "dictionary.cpp"
      HEAP8[($55)]=$conv56; //@line 436 "dictionary.cpp"
      var $56=$max_ph; //@line 438 "dictionary.cpp"
      var $cmp58=((($56))|0)==21; //@line 438 "dictionary.cpp"
      if (!($cmp58)) { __label__ = 2;continue $while_cond1$5; } //@line 438 "dictionary.cpp"
  
      var $57=$outptr_addr; //@line 441 "dictionary.cpp"
      $p_lang=$57; //@line 441 "dictionary.cpp"
      ; //@line 442 "dictionary.cpp"
      while(1) { 
  
        var $58=$p_addr; //@line 442 "dictionary.cpp"
        var $59=HEAPU8[($58)]; //@line 442 "dictionary.cpp"
        $c=$59; //@line 442 "dictionary.cpp"
        var $conv61=((($59))&255); //@line 442 "dictionary.cpp"
        var $call62=_isspace($conv61); //@line 442 "dictionary.cpp"
        var $tobool63=((($call62))|0)!=0; //@line 442 "dictionary.cpp"
        if ($tobool63) { __label__ = 30;break ; } //@line 442 "dictionary.cpp"
  
        var $60=$c; //@line 442 "dictionary.cpp"
        var $conv65=((($60))&255); //@line 442 "dictionary.cpp"
        var $cmp66=((($conv65))|0)!=0; //@line 442 "dictionary.cpp"
        if (!($cmp66)) { __label__ = 30;break ; }
  
        var $61=$p_addr; //@line 444 "dictionary.cpp"
        var $incdec_ptr69=(($61+1)&4294967295); //@line 444 "dictionary.cpp"
        $p_addr=$incdec_ptr69; //@line 444 "dictionary.cpp"
        var $62=$c; //@line 445 "dictionary.cpp"
        var $conv70=((($62))&255); //@line 445 "dictionary.cpp"
        var $call71=_tolower($conv70); //@line 445 "dictionary.cpp"
        var $conv72=((($call71)) & 255); //@line 445 "dictionary.cpp"
        var $63=$outptr_addr; //@line 445 "dictionary.cpp"
        var $incdec_ptr73=(($63+1)&4294967295); //@line 445 "dictionary.cpp"
        $outptr_addr=$incdec_ptr73; //@line 445 "dictionary.cpp"
        HEAP8[($63)]=$conv72; //@line 445 "dictionary.cpp"
        __label__ = 27;continue ; //@line 446 "dictionary.cpp"
      }
  
      var $64=$outptr_addr; //@line 447 "dictionary.cpp"
      HEAP8[($64)]=0; //@line 447 "dictionary.cpp"
      var $65=$c; //@line 448 "dictionary.cpp"
      var $conv75=((($65))&255); //@line 448 "dictionary.cpp"
      var $cmp76=((($conv75))|0)==0; //@line 448 "dictionary.cpp"
      if ($cmp76) { __label__ = 31;; } else { __label__ = 33;; } //@line 448 "dictionary.cpp"
      if (__label__ == 31) {
  
        var $66=$p_lang; //@line 450 "dictionary.cpp"
        var $call78=_strcmp($66, ((__str52704)&4294967295)); //@line 450 "dictionary.cpp"
        var $cmp79=((($call78))|0)==0; //@line 450 "dictionary.cpp"
        if ($cmp79) { __label__ = 32;break $while_cond1$5; } else { __label__ = 2;continue $while_cond1$5; } //@line 450 "dictionary.cpp"
      }
      else if (__label__ == 33) {
  
        var $69=$outptr_addr; //@line 458 "dictionary.cpp"
        var $incdec_ptr83=(($69+1)&4294967295); //@line 458 "dictionary.cpp"
        $outptr_addr=$incdec_ptr83; //@line 458 "dictionary.cpp"
        HEAP8[($69)]=124; //@line 458 "dictionary.cpp"
        __label__ = 2;continue $while_cond1$5;
      }
    }
    if (__label__ == 34) {
  
      var $70=$outptr_addr; //@line 465 "dictionary.cpp"
      HEAP8[($70)]=0; //@line 465 "dictionary.cpp"
      var $71=$p_addr; //@line 466 "dictionary.cpp"
      $retval=$71; //@line 466 "dictionary.cpp"
      ; //@line 466 "dictionary.cpp"
    }
    else if (__label__ == 22) {
  
      var $44=$p_addr; //@line 427 "dictionary.cpp"
      var $45=HEAP8[($44)]; //@line 427 "dictionary.cpp"
      var $46=$bad_phoneme_addr; //@line 427 "dictionary.cpp"
      var $arrayidx48=(($46)&4294967295); //@line 427 "dictionary.cpp"
      HEAP8[($arrayidx48)]=$45; //@line 427 "dictionary.cpp"
      var $47=$bad_phoneme_addr; //@line 428 "dictionary.cpp"
      var $arrayidx49=(($47+1)&4294967295); //@line 428 "dictionary.cpp"
      HEAP8[($arrayidx49)]=0; //@line 428 "dictionary.cpp"
      var $48=$outptr_addr; //@line 429 "dictionary.cpp"
      var $incdec_ptr50=(($48+1)&4294967295); //@line 429 "dictionary.cpp"
      $outptr_addr=$incdec_ptr50; //@line 429 "dictionary.cpp"
      HEAP8[($48)]=0; //@line 429 "dictionary.cpp"
      var $49=$p_addr; //@line 430 "dictionary.cpp"
      var $add_ptr=(($49+1)&4294967295); //@line 430 "dictionary.cpp"
      $retval=$add_ptr; //@line 430 "dictionary.cpp"
      ; //@line 430 "dictionary.cpp"
    }
    else if (__label__ == 32) {
  
      var $67=$p_lang; //@line 452 "dictionary.cpp"
      HEAP8[($67)]=0; //@line 452 "dictionary.cpp"
      var $68=$p_addr; //@line 453 "dictionary.cpp"
      $retval=$68; //@line 453 "dictionary.cpp"
      ; //@line 453 "dictionary.cpp"
    }
  
    var $72=$retval; //@line 467 "dictionary.cpp"
    ;
    return $72; //@line 467 "dictionary.cpp"
    return null;
  }
  

  function __Z14DecodePhonemesPKcPc($inptr, $outptr) {
    ;
    var __label__;
  
    var $inptr_addr;
    var $outptr_addr;
    var $phcode;
    var $c;
    var $mnem;
    var $ph;
    $inptr_addr=$inptr;
    $outptr_addr=$outptr;
    var $0=$inptr_addr; //@line 481 "dictionary.cpp"
    var $incdec_ptr8=(($0+1)&4294967295); //@line 481 "dictionary.cpp"
    $inptr_addr=$incdec_ptr8; //@line 481 "dictionary.cpp"
    var $1=HEAPU8[($0)]; //@line 481 "dictionary.cpp"
    $phcode=$1; //@line 481 "dictionary.cpp"
    var $conv9=((($1))&255); //@line 481 "dictionary.cpp"
    var $cmp10=((($conv9))|0) > 0; //@line 481 "dictionary.cpp"
    if ($cmp10) { __label__ = 1;; } else { __label__ = 14;; } //@line 481 "dictionary.cpp"
    $while_body$$while_end40$2: do { 
      if (__label__ == 1) {
        $while_body$3: while(1) { 
  
          var $2=$phcode; //@line 483 "dictionary.cpp"
          var $conv1=((($2))&255); //@line 483 "dictionary.cpp"
          var $cmp2=((($conv1))|0)==255; //@line 483 "dictionary.cpp"
          if ($cmp2) { __label__ = 2;; } else { __label__ = 3;; } //@line 483 "dictionary.cpp"
          $while_cond_backedge$$if_end$5: do { 
            if (__label__ == 3) {
  
              var $5=$phcode; //@line 485 "dictionary.cpp"
              var $idxprom=((($5))&255); //@line 485 "dictionary.cpp"
              var $arrayidx=((_phoneme_tab+$idxprom*4)&4294967295); //@line 485 "dictionary.cpp"
              var $6=HEAPU32[(($arrayidx)>>2)]; //@line 485 "dictionary.cpp"
              $ph=$6; //@line 485 "dictionary.cpp"
              var $cmp3=((($6))|0)==0; //@line 485 "dictionary.cpp"
              if ($cmp3) { __label__ = 2;break $while_cond_backedge$$if_end$5; } //@line 485 "dictionary.cpp"
  
              var $7=$ph; //@line 488 "dictionary.cpp"
              var $type=(($7+11)&4294967295); //@line 488 "dictionary.cpp"
              var $8=HEAPU8[($type)]; //@line 488 "dictionary.cpp"
              var $conv6=((($8))&255); //@line 488 "dictionary.cpp"
              var $cmp7=((($conv6))|0)==1; //@line 488 "dictionary.cpp"
              if ($cmp7) { __label__ = 5;; } else { __label__ = 9;; } //@line 488 "dictionary.cpp"
              $land_lhs_true$$if_else$8: do { 
                if (__label__ == 5) {
  
                  var $9=$ph; //@line 488 "dictionary.cpp"
                  var $std_length=(($9+14)&4294967295); //@line 488 "dictionary.cpp"
                  var $10=HEAPU8[($std_length)]; //@line 488 "dictionary.cpp"
                  var $conv8=((($10))&255); //@line 488 "dictionary.cpp"
                  var $cmp9=((($conv8))|0) <= 4; //@line 488 "dictionary.cpp"
                  if (!($cmp9)) { __label__ = 9;break $land_lhs_true$$if_else$8; } //@line 488 "dictionary.cpp"
  
                  var $11=$ph; //@line 488 "dictionary.cpp"
                  var $program=(($11+8)&4294967295); //@line 488 "dictionary.cpp"
                  var $12=HEAPU16[(($program)>>1)]; //@line 488 "dictionary.cpp"
                  var $conv11=((($12))&65535); //@line 488 "dictionary.cpp"
                  var $cmp12=((($conv11))|0)==0; //@line 488 "dictionary.cpp"
                  if (!($cmp12)) { __label__ = 9;break $land_lhs_true$$if_else$8; } //@line 488 "dictionary.cpp"
  
                  var $13=$ph; //@line 490 "dictionary.cpp"
                  var $std_length14=(($13+14)&4294967295); //@line 490 "dictionary.cpp"
                  var $14=HEAPU8[($std_length14)]; //@line 490 "dictionary.cpp"
                  var $conv15=((($14))&255); //@line 490 "dictionary.cpp"
                  var $cmp16=((($conv15))|0) > 1; //@line 490 "dictionary.cpp"
                  if (!($cmp16)) { __label__ = 2;break $while_cond_backedge$$if_end$5; } //@line 490 "dictionary.cpp"
  
                  var $15=$ph; //@line 491 "dictionary.cpp"
                  var $std_length18=(($15+14)&4294967295); //@line 491 "dictionary.cpp"
                  var $16=HEAPU8[($std_length18)]; //@line 491 "dictionary.cpp"
                  var $idxprom19=((($16))&255); //@line 491 "dictionary.cpp"
                  var $arrayidx20=((((__str7123)&4294967295)+$idxprom19)&4294967295); //@line 491 "dictionary.cpp"
                  var $17=HEAP8[($arrayidx20)]; //@line 491 "dictionary.cpp"
                  var $18=$outptr_addr; //@line 491 "dictionary.cpp"
                  var $incdec_ptr21=(($18+1)&4294967295); //@line 491 "dictionary.cpp"
                  $outptr_addr=$incdec_ptr21; //@line 491 "dictionary.cpp"
                  HEAP8[($18)]=$17; //@line 491 "dictionary.cpp"
                  __label__ = 2;break $while_cond_backedge$$if_end$5; //@line 491 "dictionary.cpp"
                }
              } while(0);
  
              var $19=$ph; //@line 495 "dictionary.cpp"
              var $mnemonic=(($19)&4294967295); //@line 495 "dictionary.cpp"
              var $20=HEAP32[(($mnemonic)>>2)]; //@line 495 "dictionary.cpp"
              $mnem=$20; //@line 495 "dictionary.cpp"
              var $21=$mnem; //@line 497 "dictionary.cpp"
              var $and4=($21) & 255; //@line 497 "dictionary.cpp"
              var $conv245=((($and4)) & 255); //@line 497 "dictionary.cpp"
              $c=$conv245; //@line 497 "dictionary.cpp"
              var $conv256=((($conv245))&255); //@line 497 "dictionary.cpp"
              var $cmp267=((($conv256))|0)!=0; //@line 497 "dictionary.cpp"
              if ($cmp267) { __label__ = 10;; } else { __label__ = 11;; } //@line 497 "dictionary.cpp"
              $while_body27$$while_end$14: do { 
                if (__label__ == 10) {
                  while(1) { 
  
                    var $22=$c; //@line 499 "dictionary.cpp"
                    var $23=$outptr_addr; //@line 499 "dictionary.cpp"
                    var $incdec_ptr28=(($23+1)&4294967295); //@line 499 "dictionary.cpp"
                    $outptr_addr=$incdec_ptr28; //@line 499 "dictionary.cpp"
                    HEAP8[($23)]=$22; //@line 499 "dictionary.cpp"
                    var $24=$mnem; //@line 500 "dictionary.cpp"
                    var $shr=($24) >>> 8; //@line 500 "dictionary.cpp"
                    $mnem=$shr; //@line 500 "dictionary.cpp"
                    var $25=$mnem; //@line 497 "dictionary.cpp"
                    var $and=($25) & 255; //@line 497 "dictionary.cpp"
                    var $conv24=((($and)) & 255); //@line 497 "dictionary.cpp"
                    $c=$conv24; //@line 497 "dictionary.cpp"
                    var $conv25=((($conv24))&255); //@line 497 "dictionary.cpp"
                    var $cmp26=((($conv25))|0)!=0; //@line 497 "dictionary.cpp"
                    if ($cmp26) { __label__ = 10;continue ; } else { __label__ = 11;break $while_body27$$while_end$14; } //@line 497 "dictionary.cpp"
                  }
                }
              } while(0);
  
              var $26=$phcode; //@line 502 "dictionary.cpp"
              var $conv29=((($26))&255); //@line 502 "dictionary.cpp"
              var $cmp30=((($conv29))|0)==21; //@line 502 "dictionary.cpp"
              if (!($cmp30)) { __label__ = 2;break $while_cond_backedge$$if_end$5; } //@line 502 "dictionary.cpp"
  
              var $27=$inptr_addr; //@line 504 "dictionary.cpp"
              var $28=HEAP8[($27)]; //@line 504 "dictionary.cpp"
              var $conv331=(tempInt=(($28)),(tempInt>=128?tempInt-256:tempInt)); //@line 504 "dictionary.cpp"
              var $call2=_isalpha($conv331); //@line 504 "dictionary.cpp"
              var $tobool3=((($call2))|0)!=0; //@line 504 "dictionary.cpp"
              if (!($tobool3)) { __label__ = 2;break $while_cond_backedge$$if_end$5; } //@line 504 "dictionary.cpp"
              while(1) { 
  
                var $29=$inptr_addr; //@line 506 "dictionary.cpp"
                var $incdec_ptr35=(($29+1)&4294967295); //@line 506 "dictionary.cpp"
                $inptr_addr=$incdec_ptr35; //@line 506 "dictionary.cpp"
                var $30=HEAP8[($29)]; //@line 506 "dictionary.cpp"
                var $31=$outptr_addr; //@line 506 "dictionary.cpp"
                var $incdec_ptr36=(($31+1)&4294967295); //@line 506 "dictionary.cpp"
                $outptr_addr=$incdec_ptr36; //@line 506 "dictionary.cpp"
                HEAP8[($31)]=$30; //@line 506 "dictionary.cpp"
                var $32=$inptr_addr; //@line 504 "dictionary.cpp"
                var $33=HEAP8[($32)]; //@line 504 "dictionary.cpp"
                var $conv33=(tempInt=(($33)),(tempInt>=128?tempInt-256:tempInt)); //@line 504 "dictionary.cpp"
                var $call=_isalpha($conv33); //@line 504 "dictionary.cpp"
                var $tobool=((($call))|0)!=0; //@line 504 "dictionary.cpp"
                if ($tobool) { __label__ = 13;continue ; } else { __label__ = 2;break $while_cond_backedge$$if_end$5; } //@line 504 "dictionary.cpp"
              }
            }
          } while(0);
  
          var $3=$inptr_addr; //@line 481 "dictionary.cpp"
          var $incdec_ptr=(($3+1)&4294967295); //@line 481 "dictionary.cpp"
          $inptr_addr=$incdec_ptr; //@line 481 "dictionary.cpp"
          var $4=HEAPU8[($3)]; //@line 481 "dictionary.cpp"
          $phcode=$4; //@line 481 "dictionary.cpp"
          var $conv=((($4))&255); //@line 481 "dictionary.cpp"
          var $cmp=((($conv))|0) > 0; //@line 481 "dictionary.cpp"
          if ($cmp) { __label__ = 1;continue $while_body$3; } else { __label__ = 14;break $while_body$$while_end40$2; } //@line 481 "dictionary.cpp"
        }
      }
    } while(0);
  
    var $34=$outptr_addr; //@line 511 "dictionary.cpp"
    HEAP8[($34)]=0; //@line 511 "dictionary.cpp"
    ;
    return; //@line 512 "dictionary.cpp"
    return;
  }
  

  function __Z26GetTranslatedPhonemeStringPci($phon_out, $n_phon_out) {
    var __stackBase__  = STACKTOP; STACKTOP += 4; _memset(__stackBase__, 0, 4);
    var __label__;
  
    var $phon_out_addr;
    var $n_phon_out_addr;
    var $ix;
    var $phon_out_ix=__stackBase__;
    var $stress;
    var $c;
    var $p;
    var $plist;
    $phon_out_addr=$phon_out;
    $n_phon_out_addr=$n_phon_out;
    HEAP32[(($phon_out_ix)>>2)]=0; //@line 603 "dictionary.cpp"
    var $0=$phon_out_addr; //@line 611 "dictionary.cpp"
    var $cmp=((($0))|0)!=0; //@line 611 "dictionary.cpp"
    if ($cmp) { __label__ = 1;; } else { __label__ = 28;; } //@line 611 "dictionary.cpp"
    if (__label__ == 1) {
  
      $ix=1; //@line 613 "dictionary.cpp"
      ; //@line 613 "dictionary.cpp"
      $for_cond$4: while(1) { 
  
        var $1=$ix; //@line 613 "dictionary.cpp"
        var $2=HEAP32[((_n_phoneme_list)>>2)]; //@line 613 "dictionary.cpp"
        var $sub=((($2)-2)&4294967295); //@line 613 "dictionary.cpp"
        var $cmp1=((($1))|0) < ((($sub))|0); //@line 613 "dictionary.cpp"
        if (!($cmp1)) { __label__ = 25;break $for_cond$4; } //@line 613 "dictionary.cpp"
  
        var $3=HEAP32[(($phon_out_ix)>>2)]; //@line 613 "dictionary.cpp"
        var $4=$n_phon_out_addr; //@line 613 "dictionary.cpp"
        var $sub2=((($4)-6)&4294967295); //@line 613 "dictionary.cpp"
        var $cmp3=((($3))|0) < ((($sub2))|0); //@line 613 "dictionary.cpp"
        if (!($cmp3)) { __label__ = 25;break $for_cond$4; }
  
        var $5=$ix; //@line 615 "dictionary.cpp"
        var $arrayidx=((_phoneme_list+$5*24)&4294967295); //@line 615 "dictionary.cpp"
        $plist=$arrayidx; //@line 615 "dictionary.cpp"
        var $6=$plist; //@line 616 "dictionary.cpp"
        var $newword=(($6+19)&4294967295); //@line 616 "dictionary.cpp"
        var $7=HEAP8[($newword)]; //@line 616 "dictionary.cpp"
        var $tobool=(tempInt=(($7)),(tempInt>=128?tempInt-256:tempInt))!=0; //@line 616 "dictionary.cpp"
        if ($tobool) { __label__ = 5;; } else { __label__ = 6;; } //@line 616 "dictionary.cpp"
        if (__label__ == 5) {
  
          var $8=HEAP32[(($phon_out_ix)>>2)]; //@line 617 "dictionary.cpp"
          var $inc=((($8)+1)&4294967295); //@line 617 "dictionary.cpp"
          HEAP32[(($phon_out_ix)>>2)]=$inc; //@line 617 "dictionary.cpp"
          var $9=$phon_out_addr; //@line 617 "dictionary.cpp"
          var $arrayidx5=(($9+$8)&4294967295); //@line 617 "dictionary.cpp"
          HEAP8[($arrayidx5)]=32; //@line 617 "dictionary.cpp"
          ; //@line 617 "dictionary.cpp"
        }
  
        var $10=$plist; //@line 619 "dictionary.cpp"
        var $synthflags=(($10+4)&4294967295); //@line 619 "dictionary.cpp"
        var $11=HEAPU16[(($synthflags)>>1)]; //@line 619 "dictionary.cpp"
        var $conv=((($11))&65535); //@line 619 "dictionary.cpp"
        var $and=($conv) & 4; //@line 619 "dictionary.cpp"
        var $tobool6=((($and))|0)!=0; //@line 619 "dictionary.cpp"
        if ($tobool6) { __label__ = 7;; } else { __label__ = 15;; } //@line 619 "dictionary.cpp"
        $if_then7$$if_end27$11: do { 
          if (__label__ == 7) {
  
            var $12=$plist; //@line 621 "dictionary.cpp"
            var $stresslevel=(($12+1)&4294967295); //@line 621 "dictionary.cpp"
            var $13=HEAPU8[($stresslevel)]; //@line 621 "dictionary.cpp"
            var $conv8=((($13))&255); //@line 621 "dictionary.cpp"
            $stress=$conv8; //@line 621 "dictionary.cpp"
            var $cmp9=((($conv8))|0) > 1; //@line 621 "dictionary.cpp"
            if (!($cmp9)) { __label__ = 15;break $if_then7$$if_end27$11; } //@line 621 "dictionary.cpp"
  
            $c=0; //@line 623 "dictionary.cpp"
            var $14=$stress; //@line 624 "dictionary.cpp"
            var $cmp11=((($14))|0) > 5; //@line 624 "dictionary.cpp"
            if ($cmp11) { __label__ = 9;; } else { __label__ = 10;; } //@line 624 "dictionary.cpp"
            if (__label__ == 9) {
  
              $stress=5; //@line 624 "dictionary.cpp"
              ; //@line 624 "dictionary.cpp"
            }
  
            var $15=HEAP32[((_option_phonemes)>>2)]; //@line 626 "dictionary.cpp"
            var $cmp14=((($15))|0)==3; //@line 626 "dictionary.cpp"
            if ($cmp14) { __label__ = 11;; } else { __label__ = 13;; } //@line 626 "dictionary.cpp"
            $if_then15$$if_end21$17: do { 
              if (__label__ == 11) {
  
                $c=716; //@line 628 "dictionary.cpp"
                var $16=$stress; //@line 629 "dictionary.cpp"
                var $cmp16=((($16))|0) > 3; //@line 629 "dictionary.cpp"
                if (!($cmp16)) { __label__ = 14;break $if_then15$$if_end21$17; } //@line 629 "dictionary.cpp"
  
                $c=712; //@line 630 "dictionary.cpp"
                ; //@line 630 "dictionary.cpp"
              }
              else if (__label__ == 13) {
  
                var $17=$stress; //@line 634 "dictionary.cpp"
                var $arrayidx19=((((__str8124)&4294967295)+$17)&4294967295); //@line 634 "dictionary.cpp"
                var $18=HEAP8[($arrayidx19)]; //@line 634 "dictionary.cpp"
                var $conv20=(tempInt=(($18)),(tempInt>=128?tempInt-256:tempInt)); //@line 634 "dictionary.cpp"
                $c=$conv20; //@line 634 "dictionary.cpp"
                var $cmp22=((($conv20))|0)!=0; //@line 637 "dictionary.cpp"
                if (!($cmp22)) { __label__ = 15;break $if_then7$$if_end27$11; } //@line 637 "dictionary.cpp"
              }
            } while(0);
  
            var $19=$c; //@line 639 "dictionary.cpp"
            var $20=HEAP32[(($phon_out_ix)>>2)]; //@line 639 "dictionary.cpp"
            var $21=$phon_out_addr; //@line 639 "dictionary.cpp"
            var $arrayidx24=(($21+$20)&4294967295); //@line 639 "dictionary.cpp"
            var $call=__Z8utf8_outjPc($19, $arrayidx24); //@line 639 "dictionary.cpp"
            var $22=HEAP32[(($phon_out_ix)>>2)]; //@line 639 "dictionary.cpp"
            var $add=((($22)+($call))&4294967295); //@line 639 "dictionary.cpp"
            HEAP32[(($phon_out_ix)>>2)]=$add; //@line 639 "dictionary.cpp"
            ; //@line 640 "dictionary.cpp"
          }
        } while(0);
  
        var $23=$plist; //@line 644 "dictionary.cpp"
        var $ph=(($23+8)&4294967295); //@line 644 "dictionary.cpp"
        var $24=HEAP32[(($ph)>>2)]; //@line 644 "dictionary.cpp"
        var $code=(($24+10)&4294967295); //@line 644 "dictionary.cpp"
        var $25=HEAPU8[($code)]; //@line 644 "dictionary.cpp"
        var $conv28=((($25))&255); //@line 644 "dictionary.cpp"
        var $cmp29=((($conv28))|0)==21; //@line 644 "dictionary.cpp"
        if ($cmp29) { __label__ = 16;; } else { __label__ = 17;; } //@line 644 "dictionary.cpp"
        $if_then30$$if_else37$23: do { 
          if (__label__ == 16) {
  
            var $26=$plist; //@line 647 "dictionary.cpp"
            var $tone_ph=(($26+3)&4294967295); //@line 647 "dictionary.cpp"
            var $27=HEAPU8[($tone_ph)]; //@line 647 "dictionary.cpp"
            var $idxprom=((($27))&255); //@line 647 "dictionary.cpp"
            var $arrayidx31=((_phoneme_tab_list+$idxprom*48)&4294967295); //@line 647 "dictionary.cpp"
            var $name=(($arrayidx31)&4294967295); //@line 647 "dictionary.cpp"
            var $arraydecay=(($name)&4294967295); //@line 647 "dictionary.cpp"
            $p=$arraydecay; //@line 647 "dictionary.cpp"
            var $28=HEAP32[(($phon_out_ix)>>2)]; //@line 649 "dictionary.cpp"
            var $29=$phon_out_addr; //@line 649 "dictionary.cpp"
            var $arrayidx32=(($29+$28)&4294967295); //@line 649 "dictionary.cpp"
            var $30=$p; //@line 649 "dictionary.cpp"
            var $call33=_sprintf($arrayidx32, ((__str9125)&4294967295), allocate([$30,0,0,0], ["i8*",0,0,0], ALLOC_STACK)); //@line 649 "dictionary.cpp"
            var $31=$p; //@line 650 "dictionary.cpp"
            var $call34=_strlen($31); //@line 650 "dictionary.cpp"
            var $32=HEAP32[(($phon_out_ix)>>2)]; //@line 650 "dictionary.cpp"
            var $add35=((($call34)+2)&4294967295); //@line 650 "dictionary.cpp"
            var $add36=((($add35)+($32))&4294967295); //@line 650 "dictionary.cpp"
            HEAP32[(($phon_out_ix)>>2)]=$add36; //@line 650 "dictionary.cpp"
            ; //@line 651 "dictionary.cpp"
          }
          else if (__label__ == 17) {
  
            var $33=$phon_out_addr; //@line 654 "dictionary.cpp"
            var $34=$plist; //@line 654 "dictionary.cpp"
            var $ph38=(($34+8)&4294967295); //@line 654 "dictionary.cpp"
            var $35=HEAP32[(($ph38)>>2)]; //@line 654 "dictionary.cpp"
            var $36=$plist; //@line 654 "dictionary.cpp"
            __ZL15WritePhMnemonicPcPiP11PHONEME_TABP12PHONEME_LIST($33, $phon_out_ix, $35, $36); //@line 654 "dictionary.cpp"
            var $37=$plist; //@line 656 "dictionary.cpp"
            var $synthflags39=(($37+4)&4294967295); //@line 656 "dictionary.cpp"
            var $38=HEAPU16[(($synthflags39)>>1)]; //@line 656 "dictionary.cpp"
            var $conv40=((($38))&65535); //@line 656 "dictionary.cpp"
            var $and41=($conv40) & 8; //@line 656 "dictionary.cpp"
            var $tobool42=((($and41))|0)!=0; //@line 656 "dictionary.cpp"
            if ($tobool42) { __label__ = 18;; } else { __label__ = 19;; } //@line 656 "dictionary.cpp"
            if (__label__ == 18) {
  
              var $39=$phon_out_addr; //@line 658 "dictionary.cpp"
              var $40=HEAP32[((((_phoneme_tab+48)&4294967295))>>2)]; //@line 658 "dictionary.cpp"
              __ZL15WritePhMnemonicPcPiP11PHONEME_TABP12PHONEME_LIST($39, $phon_out_ix, $40, 0); //@line 658 "dictionary.cpp"
              ; //@line 659 "dictionary.cpp"
            }
  
            var $41=$plist; //@line 660 "dictionary.cpp"
            var $synthflags45=(($41+4)&4294967295); //@line 660 "dictionary.cpp"
            var $42=HEAPU16[(($synthflags45)>>1)]; //@line 660 "dictionary.cpp"
            var $conv46=((($42))&65535); //@line 660 "dictionary.cpp"
            var $and47=($conv46) & 4; //@line 660 "dictionary.cpp"
            var $tobool48=((($and47))|0)!=0; //@line 660 "dictionary.cpp"
            if ($tobool48) { __label__ = 20;; } else { __label__ = 22;; } //@line 660 "dictionary.cpp"
            $land_lhs_true$$if_end52$29: do { 
              if (__label__ == 20) {
  
                var $43=$plist; //@line 660 "dictionary.cpp"
                var $type=(($43+15)&4294967295); //@line 660 "dictionary.cpp"
                var $44=HEAPU8[($type)]; //@line 660 "dictionary.cpp"
                var $conv49=((($44))&255); //@line 660 "dictionary.cpp"
                var $cmp50=((($conv49))|0)!=2; //@line 660 "dictionary.cpp"
                if (!($cmp50)) { __label__ = 22;break $land_lhs_true$$if_end52$29; } //@line 660 "dictionary.cpp"
  
                var $45=$phon_out_addr; //@line 663 "dictionary.cpp"
                var $46=HEAP32[((((_phoneme_tab+80)&4294967295))>>2)]; //@line 663 "dictionary.cpp"
                __ZL15WritePhMnemonicPcPiP11PHONEME_TABP12PHONEME_LIST($45, $phon_out_ix, $46, 0); //@line 663 "dictionary.cpp"
                ; //@line 664 "dictionary.cpp"
              }
            } while(0);
  
            var $47=$plist; //@line 665 "dictionary.cpp"
            var $tone_ph53=(($47+3)&4294967295); //@line 665 "dictionary.cpp"
            var $48=HEAPU8[($tone_ph53)]; //@line 665 "dictionary.cpp"
            var $conv54=((($48))&255); //@line 665 "dictionary.cpp"
            var $cmp55=((($conv54))|0) > 0; //@line 665 "dictionary.cpp"
            if (!($cmp55)) { __label__ = 24;break $if_then30$$if_else37$23; } //@line 665 "dictionary.cpp"
  
            var $49=$phon_out_addr; //@line 667 "dictionary.cpp"
            var $50=$plist; //@line 667 "dictionary.cpp"
            var $tone_ph57=(($50+3)&4294967295); //@line 667 "dictionary.cpp"
            var $51=HEAPU8[($tone_ph57)]; //@line 667 "dictionary.cpp"
            var $idxprom58=((($51))&255); //@line 667 "dictionary.cpp"
            var $arrayidx59=((_phoneme_tab+$idxprom58*4)&4294967295); //@line 667 "dictionary.cpp"
            var $52=HEAP32[(($arrayidx59)>>2)]; //@line 667 "dictionary.cpp"
            __ZL15WritePhMnemonicPcPiP11PHONEME_TABP12PHONEME_LIST($49, $phon_out_ix, $52, 0); //@line 667 "dictionary.cpp"
            ; //@line 668 "dictionary.cpp"
          }
        } while(0);
  
        var $53=$ix; //@line 613 "dictionary.cpp"
        var $inc62=((($53)+1)&4294967295); //@line 613 "dictionary.cpp"
        $ix=$inc62; //@line 613 "dictionary.cpp"
        __label__ = 2;continue $for_cond$4; //@line 613 "dictionary.cpp"
      }
  
      var $54=HEAP32[(($phon_out_ix)>>2)]; //@line 672 "dictionary.cpp"
      var $55=$n_phon_out_addr; //@line 672 "dictionary.cpp"
      var $cmp63=((($54))|0) >= ((($55))|0); //@line 672 "dictionary.cpp"
      if ($cmp63) { __label__ = 26;; } else { __label__ = 27;; } //@line 672 "dictionary.cpp"
      if (__label__ == 26) {
  
        var $56=$n_phon_out_addr; //@line 673 "dictionary.cpp"
        var $sub65=((($56)-1)&4294967295); //@line 673 "dictionary.cpp"
        HEAP32[(($phon_out_ix)>>2)]=$sub65; //@line 673 "dictionary.cpp"
        ; //@line 673 "dictionary.cpp"
      }
  
      var $57=HEAP32[(($phon_out_ix)>>2)]; //@line 674 "dictionary.cpp"
      var $58=$phon_out_addr; //@line 674 "dictionary.cpp"
      var $arrayidx67=(($58+$57)&4294967295); //@line 674 "dictionary.cpp"
      HEAP8[($arrayidx67)]=0; //@line 674 "dictionary.cpp"
      ; //@line 675 "dictionary.cpp"
    }
  
    STACKTOP = __stackBase__;
    return; //@line 676 "dictionary.cpp"
    return;
  }
  

  function __ZL15WritePhMnemonicPcPiP11PHONEME_TABP12PHONEME_LIST($phon_out, $ix, $ph, $plist) {
    var __stackBase__  = STACKTOP; STACKTOP += 152; _memset(__stackBase__, 0, 152);
    var __label__;
    var __lastLabel__ = null;
  
    var $phon_out_addr;
    var $ix_addr;
    var $ph_addr;
    var $plist_addr;
    var $c;
    var $mnem;
    var $len;
    var $first;
    var $ipa_control;
    var $phdata=__stackBase__;
    $phon_out_addr=$phon_out;
    $ix_addr=$ix;
    $ph_addr=$ph;
    $plist_addr=$plist;
    $ipa_control=0; //@line 532 "dictionary.cpp"
    var $0=HEAP32[((_option_phonemes)>>2)]; //@line 535 "dictionary.cpp"
    var $cmp=((($0))|0)==3; //@line 535 "dictionary.cpp"
    if ($cmp) { __label__ = 1;; } else { __label__ = 8;; } //@line 535 "dictionary.cpp"
    $if_then$$if_end20$2: do { 
      if (__label__ == 1) {
  
        var $ipa_string=(($phdata+132)&4294967295); //@line 538 "dictionary.cpp"
        var $arrayidx=(($ipa_string)&4294967295); //@line 538 "dictionary.cpp"
        HEAP8[($arrayidx)]=0; //@line 538 "dictionary.cpp"
        var $1=$plist_addr; //@line 540 "dictionary.cpp"
        var $cmp1=((($1))|0)==0; //@line 540 "dictionary.cpp"
        if ($cmp1) { __label__ = 2;; } else { __label__ = 3;; } //@line 540 "dictionary.cpp"
        if (__label__ == 2) {
  
          var $2=$ph_addr; //@line 542 "dictionary.cpp"
          var $code=(($2+10)&4294967295); //@line 542 "dictionary.cpp"
          var $3=HEAPU8[($code)]; //@line 542 "dictionary.cpp"
          var $conv=((($3))&255); //@line 542 "dictionary.cpp"
          __Z17InterpretPhoneme2iP12PHONEME_DATA($conv, $phdata); //@line 542 "dictionary.cpp"
          ; //@line 543 "dictionary.cpp"
        }
        else if (__label__ == 3) {
  
          var $4=$plist_addr; //@line 546 "dictionary.cpp"
          __Z16InterpretPhonemeP10TranslatoriP12PHONEME_LISTP12PHONEME_DATA(0, 0, $4, $phdata); //@line 546 "dictionary.cpp"
          ;
        }
  
        var $ipa_string3=(($phdata+132)&4294967295); //@line 549 "dictionary.cpp"
        var $arraydecay=(($ipa_string3)&4294967295); //@line 549 "dictionary.cpp"
        var $call=_strlen($arraydecay); //@line 549 "dictionary.cpp"
        $len=$call; //@line 549 "dictionary.cpp"
        var $cmp4=((($call))|0) > 0; //@line 550 "dictionary.cpp"
        if (!($cmp4)) { __label__ = 8;break $if_then$$if_end20$2; } //@line 550 "dictionary.cpp"
  
        var $ipa_string6=(($phdata+132)&4294967295); //@line 552 "dictionary.cpp"
        var $arrayidx7=(($ipa_string6)&4294967295); //@line 552 "dictionary.cpp"
        var $5=HEAP8[($arrayidx7)]; //@line 552 "dictionary.cpp"
        var $conv8=(tempInt=(($5)),(tempInt>=128?tempInt-256:tempInt)); //@line 552 "dictionary.cpp"
        $ipa_control=$conv8; //@line 552 "dictionary.cpp"
        var $cmp9=((($conv8))>>>0) > 32; //@line 552 "dictionary.cpp"
        if ($cmp9) { __lastLabel__ = 5; __label__ = 6;; } else { __lastLabel__ = 5; __label__ = 7;; } //@line 552 "dictionary.cpp"
        if (__label__ == 6) {
  
          var $6=$ix_addr; //@line 554 "dictionary.cpp"
          var $7=HEAP32[(($6)>>2)]; //@line 554 "dictionary.cpp"
          var $8=$phon_out_addr; //@line 554 "dictionary.cpp"
          var $arrayidx11=(($8+$7)&4294967295); //@line 554 "dictionary.cpp"
          var $ipa_string12=(($phdata+132)&4294967295); //@line 554 "dictionary.cpp"
          var $arraydecay13=(($ipa_string12)&4294967295); //@line 554 "dictionary.cpp"
          var $call14=_strcpy($arrayidx11, $arraydecay13); //@line 554 "dictionary.cpp"
          var $9=$len; //@line 555 "dictionary.cpp"
          var $10=$ix_addr; //@line 555 "dictionary.cpp"
          var $11=HEAP32[(($10)>>2)]; //@line 555 "dictionary.cpp"
          var $add=((($11)+($9))&4294967295); //@line 555 "dictionary.cpp"
          HEAP32[(($10)>>2)]=$add; //@line 555 "dictionary.cpp"
          var $_pr=$ipa_control; //@line 557 "dictionary.cpp"
          __lastLabel__ = 6; ; //@line 556 "dictionary.cpp"
        }
  
        var $12=__lastLabel__ == 6 ? $_pr : ($conv8); //@line 557 "dictionary.cpp"
        var $cmp16=((($12))>>>0) >= 32; //@line 557 "dictionary.cpp"
        if ($cmp16) { __label__ = 23;break $if_then$$if_end20$2; } else { __label__ = 8;break $if_then$$if_end20$2; } //@line 557 "dictionary.cpp"
      }
    } while(0);
    $for_end$$if_end20$12: do { 
      if (__label__ == 8) {
  
        $first=1; //@line 562 "dictionary.cpp"
        var $13=$ph_addr; //@line 563 "dictionary.cpp"
        var $mnemonic=(($13)&4294967295); //@line 563 "dictionary.cpp"
        var $14=HEAP32[(($mnemonic)>>2)]; //@line 563 "dictionary.cpp"
        $mnem=$14; //@line 563 "dictionary.cpp"
        ; //@line 563 "dictionary.cpp"
        $for_cond$14: while(1) { 
  
          var $15=$mnem; //@line 563 "dictionary.cpp"
          var $and=($15) & 255; //@line 563 "dictionary.cpp"
          $c=$and; //@line 563 "dictionary.cpp"
          var $cmp21=((($and))|0)!=0; //@line 563 "dictionary.cpp"
          if (!($cmp21)) { __label__ = 23;break $for_end$$if_end20$12; } //@line 563 "dictionary.cpp"
  
          var $16=$c; //@line 565 "dictionary.cpp"
          var $cmp22=((($16))|0)==47; //@line 565 "dictionary.cpp"
          var $17=HEAP32[((_option_phoneme_variants)>>2)]; //@line 565 "dictionary.cpp"
          var $cmp23=((($17))|0)==0; //@line 565 "dictionary.cpp"
          var $or_cond=($cmp22) & ($cmp23); //@line 565 "dictionary.cpp"
          if ($or_cond) { __label__ = 23;break $for_end$$if_end20$12; } //@line 565 "dictionary.cpp"
  
          var $18=HEAP32[((_option_phonemes)>>2)]; //@line 568 "dictionary.cpp"
          var $cmp26=((($18))|0)==3; //@line 568 "dictionary.cpp"
          if ($cmp26) { __label__ = 12;; } else { __label__ = 20;; } //@line 568 "dictionary.cpp"
          $if_then27$$if_else54$18: do { 
            if (__label__ == 12) {
  
              var $19=$first; //@line 571 "dictionary.cpp"
              var $tobool=((($19))|0)!=0; //@line 571 "dictionary.cpp"
              var $20=$c; //@line 571 "dictionary.cpp"
              var $cmp29=((($20))|0)==95; //@line 571 "dictionary.cpp"
              var $or_cond3=($tobool) & ($cmp29); //@line 571 "dictionary.cpp"
              if ($or_cond3) { __label__ = 23;break $for_end$$if_end20$12; } //@line 571 "dictionary.cpp"
  
              var $cmp32=((($20))|0)==35; //@line 574 "dictionary.cpp"
              if ($cmp32) { __label__ = 14;; } else { __label__ = 15;; } //@line 574 "dictionary.cpp"
              if (__label__ == 14) {
  
                var $21=$ph_addr; //@line 574 "dictionary.cpp"
                var $type=(($21+11)&4294967295); //@line 574 "dictionary.cpp"
                var $22=HEAPU8[($type)]; //@line 574 "dictionary.cpp"
                var $conv34=((($22))&255); //@line 574 "dictionary.cpp"
                var $cmp35=((($conv34))|0)==2; //@line 574 "dictionary.cpp"
                if ($cmp35) { __label__ = 23;break $for_end$$if_end20$12; } //@line 574 "dictionary.cpp"
              }
  
              var $23=$first; //@line 578 "dictionary.cpp"
              var $tobool38=((($23))|0)!=0; //@line 578 "dictionary.cpp"
              if ($tobool38) { __label__ = 17;; } else { __label__ = 16;; } //@line 578 "dictionary.cpp"
              if (__label__ == 16) {
  
                var $24=$c; //@line 578 "dictionary.cpp"
                var $isdigittmp=((($24)-48)&4294967295); //@line 578 "dictionary.cpp"
                var $isdigit=((($isdigittmp))>>>0) < 10; //@line 578 "dictionary.cpp"
                var $call40=((($isdigit))&1); //@line 578 "dictionary.cpp"
                var $tobool41=((($call40))|0)!=0; //@line 578 "dictionary.cpp"
                if ($tobool41) { __label__ = 22;break $if_then27$$if_else54$18; } //@line 578 "dictionary.cpp"
              }
  
              var $25=$c; //@line 581 "dictionary.cpp"
              var $cmp44=((($25))|0) >= 32; //@line 581 "dictionary.cpp"
              var $26=$c; //@line 581 "dictionary.cpp"
              var $cmp46=((($26))|0) < 128; //@line 581 "dictionary.cpp"
              var $or_cond1=($cmp44) & ($cmp46); //@line 581 "dictionary.cpp"
              if ($or_cond1) { __label__ = 18;; } else { __label__ = 19;; } //@line 581 "dictionary.cpp"
              if (__label__ == 18) {
  
                var $27=$c; //@line 582 "dictionary.cpp"
                var $sub=((($27)-32)&4294967295); //@line 582 "dictionary.cpp"
                var $arrayidx48=((_ipa1+$sub*2)&4294967295); //@line 582 "dictionary.cpp"
                var $28=HEAPU16[(($arrayidx48)>>1)]; //@line 582 "dictionary.cpp"
                var $conv49=((($28))&65535); //@line 582 "dictionary.cpp"
                $c=$conv49; //@line 582 "dictionary.cpp"
                ; //@line 582 "dictionary.cpp"
              }
  
              var $29=$c; //@line 584 "dictionary.cpp"
              var $30=$ix_addr; //@line 584 "dictionary.cpp"
              var $31=HEAP32[(($30)>>2)]; //@line 584 "dictionary.cpp"
              var $32=$phon_out_addr; //@line 584 "dictionary.cpp"
              var $arrayidx51=(($32+$31)&4294967295); //@line 584 "dictionary.cpp"
              var $call52=__Z8utf8_outjPc($29, $arrayidx51); //@line 584 "dictionary.cpp"
              var $33=$ix_addr; //@line 584 "dictionary.cpp"
              var $34=HEAP32[(($33)>>2)]; //@line 584 "dictionary.cpp"
              var $add53=((($34)+($call52))&4294967295); //@line 584 "dictionary.cpp"
              HEAP32[(($33)>>2)]=$add53; //@line 584 "dictionary.cpp"
              __label__ = 21;break $if_then27$$if_else54$18; //@line 585 "dictionary.cpp"
            }
            else if (__label__ == 20) {
  
              var $35=$c; //@line 588 "dictionary.cpp"
              var $conv55=((($35)) & 255); //@line 588 "dictionary.cpp"
              var $36=$ix_addr; //@line 588 "dictionary.cpp"
              var $37=HEAP32[(($36)>>2)]; //@line 588 "dictionary.cpp"
              var $inc=((($37)+1)&4294967295); //@line 588 "dictionary.cpp"
              HEAP32[(($36)>>2)]=$inc; //@line 588 "dictionary.cpp"
              var $38=$phon_out_addr; //@line 588 "dictionary.cpp"
              var $arrayidx56=(($38+$37)&4294967295); //@line 588 "dictionary.cpp"
              HEAP8[($arrayidx56)]=$conv55; //@line 588 "dictionary.cpp"
              __label__ = 21;break $if_then27$$if_else54$18;
            }
          } while(0);
          if (__label__ == 21) {
  
            $first=0; //@line 590 "dictionary.cpp"
            ; //@line 591 "dictionary.cpp"
          }
  
          var $39=$mnem; //@line 563 "dictionary.cpp"
          var $shr=($39) >> 8; //@line 563 "dictionary.cpp"
          $mnem=$shr; //@line 563 "dictionary.cpp"
          __label__ = 9;continue $for_cond$14; //@line 563 "dictionary.cpp"
        }
      }
    } while(0);
  
    STACKTOP = __stackBase__;
    return; //@line 592 "dictionary.cpp"
    return;
  }
  

  function __Z7IsVowelP10Translatori($tr, $letter) {
    ;
    var __label__;
  
    var $tr_addr;
    var $letter_addr;
    $tr_addr=$tr;
    $letter_addr=$letter;
    var $0=$tr_addr; //@line 763 "dictionary.cpp"
    var $1=$letter_addr; //@line 763 "dictionary.cpp"
    var $call=__ZL8IsLetterP10Translatorii($0, $1, 7); //@line 763 "dictionary.cpp"
    ;
    return $call; //@line 763 "dictionary.cpp"
    return null;
  }
  

  function __ZL8IsLetterP10Translatorii($tr, $letter, $group) {
    ;
    var __label__;
  
    var $retval;
    var $tr_addr;
    var $letter_addr;
    var $group_addr;
    var $letter2;
    $tr_addr=$tr;
    $letter_addr=$letter;
    $group_addr=$group;
    var $0=$group_addr; //@line 731 "dictionary.cpp"
    var $1=$tr_addr; //@line 731 "dictionary.cpp"
    var $letter_groups=(($1+968)&4294967295); //@line 731 "dictionary.cpp"
    var $arrayidx=(($letter_groups+$0*4)&4294967295); //@line 731 "dictionary.cpp"
    var $2=HEAP32[(($arrayidx)>>2)]; //@line 731 "dictionary.cpp"
    var $cmp=((($2))|0)!=0; //@line 731 "dictionary.cpp"
    var $3=$group_addr; //@line 733 "dictionary.cpp"
    if ($cmp) { __label__ = 1;; } else { __label__ = 4;; } //@line 731 "dictionary.cpp"
    $if_then$$if_end4$2: do { 
      if (__label__ == 1) {
  
        var $4=$tr_addr; //@line 733 "dictionary.cpp"
        var $letter_groups1=(($4+968)&4294967295); //@line 733 "dictionary.cpp"
        var $arrayidx2=(($letter_groups1+$3*4)&4294967295); //@line 733 "dictionary.cpp"
        var $5=HEAP32[(($arrayidx2)>>2)]; //@line 733 "dictionary.cpp"
        var $6=$letter_addr; //@line 733 "dictionary.cpp"
        var $call=_wcschr($5, $6); //@line 733 "dictionary.cpp"
        var $tobool=((($call))|0)!=0; //@line 733 "dictionary.cpp"
        if ($tobool) { __label__ = 2;; } else { __label__ = 3;; } //@line 733 "dictionary.cpp"
        if (__label__ == 2) {
  
          $retval=1; //@line 734 "dictionary.cpp"
          ; //@line 734 "dictionary.cpp"
        }
        else if (__label__ == 3) {
  
          $retval=0; //@line 735 "dictionary.cpp"
          ; //@line 735 "dictionary.cpp"
        }
      }
      else if (__label__ == 4) {
  
        var $cmp5=((($3))|0) > 7; //@line 738 "dictionary.cpp"
        if ($cmp5) { __label__ = 5;; } else { __label__ = 6;; } //@line 738 "dictionary.cpp"
        if (__label__ == 5) {
  
          $retval=0; //@line 739 "dictionary.cpp"
          ; //@line 739 "dictionary.cpp"
        }
        else if (__label__ == 6) {
  
          var $7=$tr_addr; //@line 741 "dictionary.cpp"
          var $letter_bits_offset=(($7+964)&4294967295); //@line 741 "dictionary.cpp"
          var $8=HEAP32[(($letter_bits_offset)>>2)]; //@line 741 "dictionary.cpp"
          var $cmp8=((($8))|0) > 0; //@line 741 "dictionary.cpp"
          var $9=$letter_addr; //@line 743 "dictionary.cpp"
          if ($cmp8) { __label__ = 7;; } else { __label__ = 11;; } //@line 741 "dictionary.cpp"
          $if_then9$$if_else15$11: do { 
            if (__label__ == 7) {
  
              var $10=$tr_addr; //@line 743 "dictionary.cpp"
              var $letter_bits_offset10=(($10+964)&4294967295); //@line 743 "dictionary.cpp"
              var $11=HEAP32[(($letter_bits_offset10)>>2)]; //@line 743 "dictionary.cpp"
              var $sub=((($9)-($11))&4294967295); //@line 743 "dictionary.cpp"
              $letter2=$sub; //@line 743 "dictionary.cpp"
              var $cmp11=((($sub))|0) > 0; //@line 743 "dictionary.cpp"
              if ($cmp11) { __label__ = 8;; } else { __label__ = 10;; } //@line 743 "dictionary.cpp"
              $land_lhs_true$$if_else$13: do { 
                if (__label__ == 8) {
  
                  var $12=$letter2; //@line 743 "dictionary.cpp"
                  var $cmp12=((($12))|0) < 128; //@line 743 "dictionary.cpp"
                  if (!($cmp12)) { __label__ = 10;break $land_lhs_true$$if_else$13; } //@line 743 "dictionary.cpp"
  
                  var $13=$letter2; //@line 744 "dictionary.cpp"
                  $letter_addr=$13; //@line 744 "dictionary.cpp"
                  __label__ = 13;break $if_then9$$if_else15$11; //@line 747 "dictionary.cpp"
                }
              } while(0);
  
              $retval=0; //@line 746 "dictionary.cpp"
              __label__ = 16;break $if_then$$if_end4$2; //@line 746 "dictionary.cpp"
            }
            else if (__label__ == 11) {
  
              var $cmp16=((($9))|0) >= 192; //@line 750 "dictionary.cpp"
              var $14=$letter_addr; //@line 750 "dictionary.cpp"
              var $cmp18=((($14))|0) < 606; //@line 750 "dictionary.cpp"
              var $or_cond=($cmp16) & ($cmp18); //@line 750 "dictionary.cpp"
              if (!($or_cond)) { __label__ = 13;break $if_then9$$if_else15$11; } //@line 750 "dictionary.cpp"
  
              var $15=$letter_addr; //@line 751 "dictionary.cpp"
              var $sub20=((($15)-192)&4294967295); //@line 751 "dictionary.cpp"
              var $arrayidx21=((__ZL13remove_accent+$sub20)&4294967295); //@line 751 "dictionary.cpp"
              var $16=HEAPU8[($arrayidx21)]; //@line 751 "dictionary.cpp"
              var $idxprom=((($16))&255); //@line 751 "dictionary.cpp"
              var $17=$tr_addr; //@line 751 "dictionary.cpp"
              var $letter_bits=(($17+708)&4294967295); //@line 751 "dictionary.cpp"
              var $arrayidx22=(($letter_bits+$idxprom)&4294967295); //@line 751 "dictionary.cpp"
              var $18=HEAPU8[($arrayidx22)]; //@line 751 "dictionary.cpp"
              var $conv=((($18))&255); //@line 751 "dictionary.cpp"
              var $19=$group_addr; //@line 751 "dictionary.cpp"
              var $shl=1 << ($19); //@line 751 "dictionary.cpp"
              var $and=($shl) & ($conv); //@line 751 "dictionary.cpp"
              $retval=$and; //@line 751 "dictionary.cpp"
              __label__ = 16;break $if_then$$if_end4$2; //@line 751 "dictionary.cpp"
            }
          } while(0);
  
          var $20=$letter_addr; //@line 754 "dictionary.cpp"
          var $cmp25=((($20))|0) >= 0; //@line 754 "dictionary.cpp"
          var $21=$letter_addr; //@line 754 "dictionary.cpp"
          var $cmp27=((($21))|0) < 128; //@line 754 "dictionary.cpp"
          var $or_cond1=($cmp25) & ($cmp27); //@line 754 "dictionary.cpp"
          if ($or_cond1) { __label__ = 14;; } else { __label__ = 15;; } //@line 754 "dictionary.cpp"
          if (__label__ == 14) {
  
            var $22=$letter_addr; //@line 755 "dictionary.cpp"
            var $23=$tr_addr; //@line 755 "dictionary.cpp"
            var $letter_bits29=(($23+708)&4294967295); //@line 755 "dictionary.cpp"
            var $arrayidx30=(($letter_bits29+$22)&4294967295); //@line 755 "dictionary.cpp"
            var $24=HEAPU8[($arrayidx30)]; //@line 755 "dictionary.cpp"
            var $conv31=((($24))&255); //@line 755 "dictionary.cpp"
            var $25=$group_addr; //@line 755 "dictionary.cpp"
            var $shl32=1 << ($25); //@line 755 "dictionary.cpp"
            var $and33=($shl32) & ($conv31); //@line 755 "dictionary.cpp"
            $retval=$and33; //@line 755 "dictionary.cpp"
            ; //@line 755 "dictionary.cpp"
          }
          else if (__label__ == 15) {
  
            $retval=0; //@line 757 "dictionary.cpp"
            ; //@line 757 "dictionary.cpp"
          }
        }
      }
    } while(0);
  
    var $26=$retval; //@line 758 "dictionary.cpp"
    ;
    return $26; //@line 758 "dictionary.cpp"
    return null;
  }
  

  function __Z14UnpronouncableP10TranslatorPci($tr, $word, $posn) {
    var __stackBase__  = STACKTOP; STACKTOP += 4; _memset(__stackBase__, 0, 4);
    var __label__;
    var __lastLabel__ = null;
  
    var $retval;
    var $tr_addr;
    var $word_addr;
    var $posn_addr;
    var $c=__stackBase__;
    var $c1;
    var $vowel_posn;
    var $index;
    var $count;
    var $apostrophe;
    $tr_addr=$tr;
    $word_addr=$word;
    $posn_addr=$posn;
    $c1=0; //@line 795 "dictionary.cpp"
    $vowel_posn=9; //@line 796 "dictionary.cpp"
    $apostrophe=0; //@line 799 "dictionary.cpp"
    var $0=$word_addr; //@line 801 "dictionary.cpp"
    var $call=__Z7utf8_inPiPKc($c, $0); //@line 801 "dictionary.cpp"
    var $1=$tr_addr; //@line 802 "dictionary.cpp"
    var $letter_bits_offset=(($1+964)&4294967295); //@line 802 "dictionary.cpp"
    var $2=HEAP32[(($letter_bits_offset)>>2)]; //@line 802 "dictionary.cpp"
    var $cmp=((($2))|0) > 0; //@line 802 "dictionary.cpp"
    if ($cmp) { __label__ = 1;; } else { __label__ = 3;; } //@line 802 "dictionary.cpp"
    $land_lhs_true$$if_end$2: do { 
      if (__label__ == 1) {
  
        var $3=HEAP32[(($c)>>2)]; //@line 802 "dictionary.cpp"
        var $cmp1=((($3))|0) < 577; //@line 802 "dictionary.cpp"
        if (!($cmp1)) { __label__ = 3;break $land_lhs_true$$if_end$2; } //@line 802 "dictionary.cpp"
  
        $retval=0; //@line 805 "dictionary.cpp"
        __label__ = 30;break $land_lhs_true$$if_end$2; //@line 805 "dictionary.cpp"
      }
    } while(0);
    $return$$if_end$5: do { 
      if (__label__ == 3) {
  
        var $4=$tr_addr; //@line 808 "dictionary.cpp"
        var $langopts=(($4)&4294967295); //@line 808 "dictionary.cpp"
        var $param=(($langopts+24)&4294967295); //@line 808 "dictionary.cpp"
        var $arrayidx=(($param+20)&4294967295); //@line 808 "dictionary.cpp"
        var $5=HEAP32[(($arrayidx)>>2)]; //@line 808 "dictionary.cpp"
        var $cmp2=((($5))|0)==1; //@line 808 "dictionary.cpp"
        if ($cmp2) { __label__ = 4;; } else { __label__ = 5;; } //@line 808 "dictionary.cpp"
        if (__label__ == 4) {
  
          $retval=0; //@line 809 "dictionary.cpp"
          ; //@line 809 "dictionary.cpp"
        }
        else if (__label__ == 5) {
  
          var $6=$word_addr; //@line 811 "dictionary.cpp"
          var $7=HEAP8[($6)]; //@line 811 "dictionary.cpp"
          var $conv=(tempInt=(($7)),(tempInt>=128?tempInt-256:tempInt)); //@line 811 "dictionary.cpp"
          HEAP32[(($c)>>2)]=$conv; //@line 811 "dictionary.cpp"
          var $cmp5=((($conv))|0)==32; //@line 811 "dictionary.cpp"
          if ($cmp5) { __label__ = 7;; } else { __label__ = 6;; } //@line 811 "dictionary.cpp"
          $if_then9$$lor_lhs_false$10: do { 
            if (__label__ == 6) {
  
              var $8=HEAP32[(($c)>>2)]; //@line 811 "dictionary.cpp"
              var $cmp6=((($8))|0)==0; //@line 811 "dictionary.cpp"
              var $9=HEAP32[(($c)>>2)]; //@line 811 "dictionary.cpp"
              var $cmp8=((($9))|0)==39; //@line 811 "dictionary.cpp"
              var $or_cond=($cmp6) | ($cmp8); //@line 811 "dictionary.cpp"
              if ($or_cond) { __label__ = 7;break $if_then9$$lor_lhs_false$10; } //@line 811 "dictionary.cpp"
  
              $index=0; //@line 814 "dictionary.cpp"
              $count=0; //@line 815 "dictionary.cpp"
              ; //@line 816 "dictionary.cpp"
              $for_cond$13: while(1) { 
  
                var $10=$index; //@line 818 "dictionary.cpp"
                var $11=$word_addr; //@line 818 "dictionary.cpp"
                var $arrayidx11=(($11+$10)&4294967295); //@line 818 "dictionary.cpp"
                var $call12=__Z7utf8_inPiPKc($c, $arrayidx11); //@line 818 "dictionary.cpp"
                var $12=$index; //@line 818 "dictionary.cpp"
                var $add=((($12)+($call12))&4294967295); //@line 818 "dictionary.cpp"
                $index=$add; //@line 818 "dictionary.cpp"
                var $13=HEAP32[(($c)>>2)]; //@line 819 "dictionary.cpp"
                var $cmp13=((($13))|0)==0; //@line 819 "dictionary.cpp"
                var $14=HEAP32[(($c)>>2)]; //@line 819 "dictionary.cpp"
                var $cmp15=((($14))|0)==32; //@line 819 "dictionary.cpp"
                var $or_cond1=($cmp13) | ($cmp15); //@line 819 "dictionary.cpp"
                if ($or_cond1) { __label__ = 21;break $for_cond$13; } //@line 819 "dictionary.cpp"
  
                var $15=HEAP32[(($c)>>2)]; //@line 822 "dictionary.cpp"
                var $cmp18=((($15))|0)==39; //@line 822 "dictionary.cpp"
                if ($cmp18) { __label__ = 11;; } else { __label__ = 13;; } //@line 822 "dictionary.cpp"
                if (__label__ == 11) {
  
                  var $16=$count; //@line 822 "dictionary.cpp"
                  var $cmp20=((($16))|0) > 1; //@line 822 "dictionary.cpp"
                  if ($cmp20) { __label__ = 21;break $for_cond$13; } //@line 822 "dictionary.cpp"
  
                  var $17=$posn_addr; //@line 822 "dictionary.cpp"
                  var $cmp22=((($17))|0) > 0; //@line 822 "dictionary.cpp"
                  if ($cmp22) { __label__ = 21;break $for_cond$13; } //@line 822 "dictionary.cpp"
                }
  
                var $18=$count; //@line 825 "dictionary.cpp"
                var $cmp25=((($18))|0)==0; //@line 825 "dictionary.cpp"
                if ($cmp25) { __label__ = 14;; } else { __label__ = 15;; } //@line 825 "dictionary.cpp"
                if (__label__ == 14) {
  
                  var $19=HEAP32[(($c)>>2)]; //@line 826 "dictionary.cpp"
                  $c1=$19; //@line 826 "dictionary.cpp"
                  ; //@line 826 "dictionary.cpp"
                }
  
                var $20=$count; //@line 827 "dictionary.cpp"
                var $inc=((($20)+1)&4294967295); //@line 827 "dictionary.cpp"
                $count=$inc; //@line 827 "dictionary.cpp"
                var $21=$tr_addr; //@line 829 "dictionary.cpp"
                var $22=HEAP32[(($c)>>2)]; //@line 829 "dictionary.cpp"
                var $call28=__Z7IsVowelP10Translatori($21, $22); //@line 829 "dictionary.cpp"
                var $tobool=((($call28))|0)!=0; //@line 829 "dictionary.cpp"
                if ($tobool) { __label__ = 16;break $for_cond$13; } //@line 829 "dictionary.cpp"
  
                var $24=HEAP32[(($c)>>2)]; //@line 835 "dictionary.cpp"
                var $cmp31=((($24))|0)==39; //@line 835 "dictionary.cpp"
                if ($cmp31) { __label__ = 18;; } else { __label__ = 19;; } //@line 835 "dictionary.cpp"
                if (__label__ == 18) {
  
                  $apostrophe=1; //@line 836 "dictionary.cpp"
                  __label__ = 9;continue $for_cond$13; //@line 836 "dictionary.cpp"
                }
                else if (__label__ == 19) {
  
                  var $25=HEAP32[(($c)>>2)]; //@line 838 "dictionary.cpp"
                  var $call33=_iswalpha($25); //@line 838 "dictionary.cpp"
                  var $tobool34=((($call33))|0)!=0; //@line 838 "dictionary.cpp"
                  if ($tobool34) { __label__ = 9;continue $for_cond$13; } else { __label__ = 20;break $for_cond$13; } //@line 838 "dictionary.cpp"
                }
              }
              if (__label__ == 21) {
  
                var $_pr=$vowel_posn; //@line 842 "dictionary.cpp"
                __lastLabel__ = 21; ;
              }
              else if (__label__ == 16) {
  
                var $23=$count; //@line 831 "dictionary.cpp"
                $vowel_posn=$23; //@line 831 "dictionary.cpp"
                __lastLabel__ = 16; ; //@line 832 "dictionary.cpp"
              }
              else if (__label__ == 20) {
  
                $retval=0; //@line 839 "dictionary.cpp"
                __label__ = 30;break $return$$if_end$5; //@line 839 "dictionary.cpp"
              }
  
              var $26=__lastLabel__ == 21 ? $_pr : ($23); //@line 842 "dictionary.cpp"
              var $cmp38=((($26))|0) > 2; //@line 842 "dictionary.cpp"
              if ($cmp38) { __label__ = 23;; } else { __label__ = 25;; } //@line 842 "dictionary.cpp"
              $land_lhs_true39$$if_end46$32: do { 
                if (__label__ == 23) {
  
                  var $27=$tr_addr; //@line 842 "dictionary.cpp"
                  var $langopts40=(($27)&4294967295); //@line 842 "dictionary.cpp"
                  var $param41=(($langopts40+24)&4294967295); //@line 842 "dictionary.cpp"
                  var $arrayidx42=(($param41+20)&4294967295); //@line 842 "dictionary.cpp"
                  var $28=HEAP32[(($arrayidx42)>>2)]; //@line 842 "dictionary.cpp"
                  var $cmp43=((($28))|0)==2; //@line 842 "dictionary.cpp"
                  if (!($cmp43)) { __label__ = 25;break $land_lhs_true39$$if_end46$32; } //@line 842 "dictionary.cpp"
  
                  var $29=$tr_addr; //@line 845 "dictionary.cpp"
                  var $30=$word_addr; //@line 845 "dictionary.cpp"
                  var $call45=__ZL15Unpronouncable2P10TranslatorPc($29, $30); //@line 845 "dictionary.cpp"
                  $retval=$call45; //@line 845 "dictionary.cpp"
                  __label__ = 30;break $return$$if_end$5; //@line 845 "dictionary.cpp"
                }
              } while(0);
  
              var $31=$c1; //@line 848 "dictionary.cpp"
              var $32=$tr_addr; //@line 848 "dictionary.cpp"
              var $langopts47=(($32)&4294967295); //@line 848 "dictionary.cpp"
              var $param48=(($langopts47+24)&4294967295); //@line 848 "dictionary.cpp"
              var $arrayidx49=(($param48+20)&4294967295); //@line 848 "dictionary.cpp"
              var $33=HEAP32[(($arrayidx49)>>2)]; //@line 848 "dictionary.cpp"
              var $cmp50=((($31))|0)==((($33))|0); //@line 848 "dictionary.cpp"
              if ($cmp50) { __label__ = 26;; } else { __label__ = 27;; } //@line 848 "dictionary.cpp"
              if (__label__ == 26) {
  
                var $34=$vowel_posn; //@line 849 "dictionary.cpp"
                var $dec=((($34)-1)&4294967295); //@line 849 "dictionary.cpp"
                $vowel_posn=$dec; //@line 849 "dictionary.cpp"
                ; //@line 849 "dictionary.cpp"
              }
  
              var $35=$vowel_posn; //@line 851 "dictionary.cpp"
              var $36=$tr_addr; //@line 851 "dictionary.cpp"
              var $langopts53=(($36)&4294967295); //@line 851 "dictionary.cpp"
              var $max_initial_consonants=(($langopts53+248)&4294967295); //@line 851 "dictionary.cpp"
              var $37=HEAP8[($max_initial_consonants)]; //@line 851 "dictionary.cpp"
              var $conv54=(tempInt=(($37)),(tempInt>=128?tempInt-256:tempInt)); //@line 851 "dictionary.cpp"
              var $add55=((($conv54)+1)&4294967295); //@line 851 "dictionary.cpp"
              var $cmp56=((($35))|0) > ((($add55))|0); //@line 851 "dictionary.cpp"
              if ($cmp56) { __label__ = 28;; } else { __label__ = 29;; } //@line 851 "dictionary.cpp"
              if (__label__ == 28) {
  
                $retval=1; //@line 852 "dictionary.cpp"
                __label__ = 30;break $return$$if_end$5; //@line 852 "dictionary.cpp"
              }
              else if (__label__ == 29) {
  
                $retval=0; //@line 854 "dictionary.cpp"
                __label__ = 30;break $return$$if_end$5; //@line 854 "dictionary.cpp"
              }
            }
          } while(0);
  
          $retval=0; //@line 812 "dictionary.cpp"
          ; //@line 812 "dictionary.cpp"
        }
      }
    } while(0);
  
    var $38=$retval; //@line 856 "dictionary.cpp"
    STACKTOP = __stackBase__;
    return $38; //@line 856 "dictionary.cpp"
    return null;
  }
  

  function __ZL15Unpronouncable2P10TranslatorPc($tr, $word) {
    var __stackBase__  = STACKTOP; STACKTOP += 160; _memset(__stackBase__, 0, 160);
    var __label__;
  
    var $retval;
    var $tr_addr;
    var $word_addr;
    var $c;
    var $end_flags;
    var $ph_buf=__stackBase__;
    $tr_addr=$tr;
    $word_addr=$word;
    var $arrayidx=(($ph_buf)&4294967295); //@line 775 "dictionary.cpp"
    HEAP8[($arrayidx)]=0; //@line 775 "dictionary.cpp"
    var $0=$word_addr; //@line 776 "dictionary.cpp"
    var $arrayidx1=(($0+-1)&4294967295); //@line 776 "dictionary.cpp"
    var $1=HEAP8[($arrayidx1)]; //@line 776 "dictionary.cpp"
    var $conv=(tempInt=(($1)),(tempInt>=128?tempInt-256:tempInt)); //@line 776 "dictionary.cpp"
    $c=$conv; //@line 776 "dictionary.cpp"
    var $2=$word_addr; //@line 777 "dictionary.cpp"
    var $arrayidx2=(($2+-1)&4294967295); //@line 777 "dictionary.cpp"
    HEAP8[($arrayidx2)]=32; //@line 777 "dictionary.cpp"
    var $3=$tr_addr; //@line 778 "dictionary.cpp"
    var $4=$word_addr; //@line 778 "dictionary.cpp"
    var $arraydecay=(($ph_buf)&4294967295); //@line 778 "dictionary.cpp"
    var $call=__Z14TranslateRulesP10TranslatorPcS1_iS1_iPj($3, $4, $arraydecay, 160, 0, -2147483648, 0); //@line 778 "dictionary.cpp"
    $end_flags=$call; //@line 778 "dictionary.cpp"
    var $5=$c; //@line 779 "dictionary.cpp"
    var $conv3=((($5)) & 255); //@line 779 "dictionary.cpp"
    var $6=$word_addr; //@line 779 "dictionary.cpp"
    var $arrayidx4=(($6+-1)&4294967295); //@line 779 "dictionary.cpp"
    HEAP8[($arrayidx4)]=$conv3; //@line 779 "dictionary.cpp"
    var $7=$end_flags; //@line 780 "dictionary.cpp"
    var $cmp=((($7))|0)==0; //@line 780 "dictionary.cpp"
    if ($cmp) { __label__ = 2;; } else { __label__ = 1;; } //@line 780 "dictionary.cpp"
    $if_then$$lor_lhs_false$2: do { 
      if (__label__ == 1) {
  
        var $8=$end_flags; //@line 780 "dictionary.cpp"
        var $and=($8) & 32768; //@line 780 "dictionary.cpp"
        var $tobool=((($and))|0)!=0; //@line 780 "dictionary.cpp"
        if ($tobool) { __label__ = 2;break $if_then$$lor_lhs_false$2; } //@line 780 "dictionary.cpp"
  
        $retval=0; //@line 782 "dictionary.cpp"
        __label__ = 4;break $if_then$$lor_lhs_false$2; //@line 782 "dictionary.cpp"
      }
    } while(0);
    if (__label__ == 2) {
  
      $retval=1; //@line 781 "dictionary.cpp"
      ; //@line 781 "dictionary.cpp"
    }
  
    var $9=$retval; //@line 783 "dictionary.cpp"
    STACKTOP = __stackBase__;
    return $9; //@line 783 "dictionary.cpp"
    return null;
  }
  

  function __Z16ChangeWordStressP10TranslatorPci($tr, $word, $new_stress) {
    var __stackBase__  = STACKTOP; STACKTOP += 248; _memset(__stackBase__, 0, 248);
    var __label__;
  
    var $tr_addr;
    var $word_addr;
    var $new_stress_addr;
    var $ix;
    var $p;
    var $max_stress;
    var $vowel_count=__stackBase__;
    var $stressed_syllable=__stackBase__+4;
    var $phonetic=__stackBase__+8;
    var $vowel_stress=__stackBase__+168;
    $tr_addr=$tr;
    $word_addr=$word;
    $new_stress_addr=$new_stress;
    HEAP32[(($stressed_syllable)>>2)]=0; //@line 1010 "dictionary.cpp"
    var $arraydecay=(($phonetic)&4294967295); //@line 1014 "dictionary.cpp"
    var $0=$word_addr; //@line 1014 "dictionary.cpp"
    var $call=_strcpy($arraydecay, $0); //@line 1014 "dictionary.cpp"
    var $1=$tr_addr; //@line 1015 "dictionary.cpp"
    var $arraydecay1=(($phonetic)&4294967295); //@line 1015 "dictionary.cpp"
    var $arraydecay2=(($vowel_stress)&4294967295); //@line 1015 "dictionary.cpp"
    var $call3=__ZL14GetVowelStressP10TranslatorPhPaRiS3_i($1, $arraydecay1, $arraydecay2, $vowel_count, $stressed_syllable, 0); //@line 1015 "dictionary.cpp"
    $max_stress=$call3; //@line 1015 "dictionary.cpp"
    var $2=$new_stress_addr; //@line 1017 "dictionary.cpp"
    var $cmp=((($2))|0) >= 4; //@line 1017 "dictionary.cpp"
    $ix=1; //@line 1020 "dictionary.cpp"
    if ($cmp) { __label__ = 2;; } else { __label__ = 1;; } //@line 1017 "dictionary.cpp"
    $for_cond$$for_cond9_preheader$2: do { 
      if (__label__ == 2) {
        while(1) { 
  
          var $5=$ix; //@line 1020 "dictionary.cpp"
          var $6=HEAP32[(($vowel_count)>>2)]; //@line 1020 "dictionary.cpp"
          var $cmp4=((($5))|0) < ((($6))|0); //@line 1020 "dictionary.cpp"
          if (!($cmp4)) { __label__ = 9;break $for_cond$$for_cond9_preheader$2; } //@line 1020 "dictionary.cpp"
  
          var $7=$ix; //@line 1022 "dictionary.cpp"
          var $arrayidx=(($vowel_stress+$7)&4294967295); //@line 1022 "dictionary.cpp"
          var $8=HEAP8[($arrayidx)]; //@line 1022 "dictionary.cpp"
          var $conv=(tempInt=(($8)),(tempInt>=128?tempInt-256:tempInt)); //@line 1022 "dictionary.cpp"
          var $9=$max_stress; //@line 1022 "dictionary.cpp"
          var $cmp5=((($conv))|0) >= ((($9))|0); //@line 1022 "dictionary.cpp"
          if ($cmp5) { __label__ = 4;break ; } //@line 1022 "dictionary.cpp"
  
          var $12=$ix; //@line 1020 "dictionary.cpp"
          var $inc=((($12)+1)&4294967295); //@line 1020 "dictionary.cpp"
          $ix=$inc; //@line 1020 "dictionary.cpp"
          __label__ = 2;continue ; //@line 1020 "dictionary.cpp"
        }
  
        var $10=$new_stress_addr; //@line 1024 "dictionary.cpp"
        var $conv7=((($10)) & 255); //@line 1024 "dictionary.cpp"
        var $11=$ix; //@line 1024 "dictionary.cpp"
        var $arrayidx8=(($vowel_stress+$11)&4294967295); //@line 1024 "dictionary.cpp"
        HEAP8[($arrayidx8)]=$conv7; //@line 1024 "dictionary.cpp"
        ; //@line 1025 "dictionary.cpp"
      }
      else if (__label__ == 1) {
  
        var $3=$ix; //@line 1032 "dictionary.cpp"
        var $4=HEAP32[(($vowel_count)>>2)]; //@line 1032 "dictionary.cpp"
        var $cmp101=((($3))|0) < ((($4))|0); //@line 1032 "dictionary.cpp"
        if (!($cmp101)) { __label__ = 9;break $for_cond$$for_cond9_preheader$2; } //@line 1032 "dictionary.cpp"
        while(1) { 
  
          var $13=$ix; //@line 1034 "dictionary.cpp"
          var $arrayidx12=(($vowel_stress+$13)&4294967295); //@line 1034 "dictionary.cpp"
          var $14=HEAP8[($arrayidx12)]; //@line 1034 "dictionary.cpp"
          var $conv13=(tempInt=(($14)),(tempInt>=128?tempInt-256:tempInt)); //@line 1034 "dictionary.cpp"
          var $15=$new_stress_addr; //@line 1034 "dictionary.cpp"
          var $cmp14=((($conv13))|0) > ((($15))|0); //@line 1034 "dictionary.cpp"
          if ($cmp14) { __label__ = 7;; } else { __label__ = 8;; } //@line 1034 "dictionary.cpp"
          if (__label__ == 7) {
  
            var $16=$new_stress_addr; //@line 1035 "dictionary.cpp"
            var $conv16=((($16)) & 255); //@line 1035 "dictionary.cpp"
            var $17=$ix; //@line 1035 "dictionary.cpp"
            var $arrayidx17=(($vowel_stress+$17)&4294967295); //@line 1035 "dictionary.cpp"
            HEAP8[($arrayidx17)]=$conv16; //@line 1035 "dictionary.cpp"
            ; //@line 1035 "dictionary.cpp"
          }
  
          var $18=$ix; //@line 1032 "dictionary.cpp"
          var $inc20=((($18)+1)&4294967295); //@line 1032 "dictionary.cpp"
          $ix=$inc20; //@line 1032 "dictionary.cpp"
          var $19=$ix; //@line 1032 "dictionary.cpp"
          var $20=HEAP32[(($vowel_count)>>2)]; //@line 1032 "dictionary.cpp"
          var $cmp10=((($19))|0) < ((($20))|0); //@line 1032 "dictionary.cpp"
          if ($cmp10) { __label__ = 6;continue ; } else { __label__ = 9;break $for_cond$$for_cond9_preheader$2; } //@line 1032 "dictionary.cpp"
        }
      }
    } while(0);
  
    $ix=1; //@line 1040 "dictionary.cpp"
    var $arraydecay23=(($phonetic)&4294967295); //@line 1041 "dictionary.cpp"
    $p=$arraydecay23; //@line 1041 "dictionary.cpp"
    var $21=$p; //@line 1042 "dictionary.cpp"
    var $22=HEAPU8[($21)]; //@line 1042 "dictionary.cpp"
    var $conv242=((($22))&255); //@line 1042 "dictionary.cpp"
    var $cmp253=((($conv242))|0)!=0; //@line 1042 "dictionary.cpp"
    if ($cmp253) { __label__ = 10;; } else { __label__ = 17;; } //@line 1042 "dictionary.cpp"
    $while_body$$while_end$15: do { 
      if (__label__ == 10) {
        $while_body$16: while(1) { 
  
          var $23=$p; //@line 1044 "dictionary.cpp"
          var $24=HEAPU8[($23)]; //@line 1044 "dictionary.cpp"
          var $idxprom=((($24))&255); //@line 1044 "dictionary.cpp"
          var $arrayidx26=((_phoneme_tab+$idxprom*4)&4294967295); //@line 1044 "dictionary.cpp"
          var $25=HEAP32[(($arrayidx26)>>2)]; //@line 1044 "dictionary.cpp"
          var $type=(($25+11)&4294967295); //@line 1044 "dictionary.cpp"
          var $26=HEAPU8[($type)]; //@line 1044 "dictionary.cpp"
          var $conv27=((($26))&255); //@line 1044 "dictionary.cpp"
          var $cmp28=((($conv27))|0)==2; //@line 1044 "dictionary.cpp"
          if ($cmp28) { __label__ = 11;; } else { __label__ = 16;; } //@line 1044 "dictionary.cpp"
          $land_lhs_true$$if_end44$18: do { 
            if (__label__ == 11) {
  
              var $27=$p; //@line 1044 "dictionary.cpp"
              var $28=HEAPU8[($27)]; //@line 1044 "dictionary.cpp"
              var $idxprom29=((($28))&255); //@line 1044 "dictionary.cpp"
              var $arrayidx30=((_phoneme_tab+$idxprom29*4)&4294967295); //@line 1044 "dictionary.cpp"
              var $29=HEAP32[(($arrayidx30)>>2)]; //@line 1044 "dictionary.cpp"
              var $phflags=(($29+4)&4294967295); //@line 1044 "dictionary.cpp"
              var $30=HEAP32[(($phflags)>>2)]; //@line 1044 "dictionary.cpp"
              var $and=($30) & 1048576; //@line 1044 "dictionary.cpp"
              var $tobool=((($and))|0)!=0; //@line 1044 "dictionary.cpp"
              if ($tobool) { __label__ = 16;break $land_lhs_true$$if_end44$18; } //@line 1044 "dictionary.cpp"
  
              var $31=$ix; //@line 1046 "dictionary.cpp"
              var $arrayidx32=(($vowel_stress+$31)&4294967295); //@line 1046 "dictionary.cpp"
              var $32=HEAP8[($arrayidx32)]; //@line 1046 "dictionary.cpp"
              var $conv33=(tempInt=(($32)),(tempInt>=128?tempInt-256:tempInt)); //@line 1046 "dictionary.cpp"
              var $cmp34=((($conv33))|0)==0; //@line 1046 "dictionary.cpp"
              if ($cmp34) { __label__ = 14;; } else { __label__ = 13;; } //@line 1046 "dictionary.cpp"
              $if_then38$$lor_lhs_false$21: do { 
                if (__label__ == 13) {
  
                  var $33=$ix; //@line 1046 "dictionary.cpp"
                  var $arrayidx35=(($vowel_stress+$33)&4294967295); //@line 1046 "dictionary.cpp"
                  var $34=HEAP8[($arrayidx35)]; //@line 1046 "dictionary.cpp"
                  var $conv36=(tempInt=(($34)),(tempInt>=128?tempInt-256:tempInt)); //@line 1046 "dictionary.cpp"
                  var $cmp37=((($conv36))|0) > 1; //@line 1046 "dictionary.cpp"
                  if ($cmp37) { __label__ = 14;break $if_then38$$lor_lhs_false$21; } else { __label__ = 15;break $if_then38$$lor_lhs_false$21; } //@line 1046 "dictionary.cpp"
                }
              } while(0);
              if (__label__ == 14) {
  
                var $35=$ix; //@line 1047 "dictionary.cpp"
                var $arrayidx39=(($vowel_stress+$35)&4294967295); //@line 1047 "dictionary.cpp"
                var $36=HEAPU8[($arrayidx39)]; //@line 1047 "dictionary.cpp"
                var $idxprom40=((($36))&255); //@line 1047 "dictionary.cpp"
                var $arrayidx41=((__ZL15stress_phonemes+$idxprom40)&4294967295); //@line 1047 "dictionary.cpp"
                var $37=HEAP8[($arrayidx41)]; //@line 1047 "dictionary.cpp"
                var $38=$word_addr; //@line 1047 "dictionary.cpp"
                var $incdec_ptr=(($38+1)&4294967295); //@line 1047 "dictionary.cpp"
                $word_addr=$incdec_ptr; //@line 1047 "dictionary.cpp"
                HEAP8[($38)]=$37; //@line 1047 "dictionary.cpp"
                ; //@line 1047 "dictionary.cpp"
              }
  
              var $39=$ix; //@line 1049 "dictionary.cpp"
              var $inc43=((($39)+1)&4294967295); //@line 1049 "dictionary.cpp"
              $ix=$inc43; //@line 1049 "dictionary.cpp"
              ; //@line 1050 "dictionary.cpp"
            }
          } while(0);
  
          var $40=$p; //@line 1051 "dictionary.cpp"
          var $incdec_ptr45=(($40+1)&4294967295); //@line 1051 "dictionary.cpp"
          $p=$incdec_ptr45; //@line 1051 "dictionary.cpp"
          var $41=HEAP8[($40)]; //@line 1051 "dictionary.cpp"
          var $42=$word_addr; //@line 1051 "dictionary.cpp"
          var $incdec_ptr46=(($42+1)&4294967295); //@line 1051 "dictionary.cpp"
          $word_addr=$incdec_ptr46; //@line 1051 "dictionary.cpp"
          HEAP8[($42)]=$41; //@line 1051 "dictionary.cpp"
          var $43=$p; //@line 1042 "dictionary.cpp"
          var $44=HEAPU8[($43)]; //@line 1042 "dictionary.cpp"
          var $conv24=((($44))&255); //@line 1042 "dictionary.cpp"
          var $cmp25=((($conv24))|0)!=0; //@line 1042 "dictionary.cpp"
          if ($cmp25) { __label__ = 10;continue $while_body$16; } else { __label__ = 17;break $while_body$$while_end$15; } //@line 1042 "dictionary.cpp"
        }
      }
    } while(0);
  
    var $45=$word_addr; //@line 1053 "dictionary.cpp"
    HEAP8[($45)]=0; //@line 1053 "dictionary.cpp"
    STACKTOP = __stackBase__;
    return; //@line 1054 "dictionary.cpp"
    return;
  }
  

  function __ZL14GetVowelStressP10TranslatorPhPaRiS3_i($tr, $phonemes, $vowel_stress, $vowel_count, $stressed_syllable, $control) {
    ;
    var __label__;
    var __lastLabel__ = null;
  
    var $tr_addr;
    var $phonemes_addr;
    var $vowel_stress_addr;
    var $vowel_count_addr;
    var $stressed_syllable_addr;
    var $control_addr;
    var $phcode;
    var $ph;
    var $ph_out;
    var $count;
    var $max_stress;
    var $ix;
    var $j;
    var $stress;
    var $primary_posn;
    $tr_addr=$tr;
    $phonemes_addr=$phonemes;
    $vowel_stress_addr=$vowel_stress;
    $vowel_count_addr=$vowel_count;
    $stressed_syllable_addr=$stressed_syllable;
    $control_addr=$control;
    var $0=$phonemes_addr; //@line 872 "dictionary.cpp"
    $ph_out=$0; //@line 872 "dictionary.cpp"
    $count=1; //@line 873 "dictionary.cpp"
    $max_stress=-1; //@line 874 "dictionary.cpp"
    $stress=-1; //@line 877 "dictionary.cpp"
    $primary_posn=0; //@line 878 "dictionary.cpp"
    var $1=$vowel_stress_addr; //@line 880 "dictionary.cpp"
    var $arrayidx=(($1)&4294967295); //@line 880 "dictionary.cpp"
    HEAP8[($arrayidx)]=1; //@line 880 "dictionary.cpp"
    ; //@line 881 "dictionary.cpp"
    $while_cond$2: while(1) { 
  
      var $2=$phonemes_addr; //@line 881 "dictionary.cpp"
      var $incdec_ptr=(($2+1)&4294967295); //@line 881 "dictionary.cpp"
      $phonemes_addr=$incdec_ptr; //@line 881 "dictionary.cpp"
      var $3=HEAPU8[($2)]; //@line 881 "dictionary.cpp"
      $phcode=$3; //@line 881 "dictionary.cpp"
      var $conv=((($3))&255); //@line 881 "dictionary.cpp"
      var $cmp=((($conv))|0)!=0; //@line 881 "dictionary.cpp"
      if (!($cmp)) { __label__ = 39;break $while_cond$2; } //@line 881 "dictionary.cpp"
  
      var $4=$count; //@line 881 "dictionary.cpp"
      var $cmp1=((($4))|0) < 79; //@line 881 "dictionary.cpp"
      if (!($cmp1)) { __label__ = 39;break $while_cond$2; }
  
      var $5=$phcode; //@line 883 "dictionary.cpp"
      var $idxprom=((($5))&255); //@line 883 "dictionary.cpp"
      var $arrayidx2=((_phoneme_tab+$idxprom*4)&4294967295); //@line 883 "dictionary.cpp"
      var $6=HEAPU32[(($arrayidx2)>>2)]; //@line 883 "dictionary.cpp"
      $ph=$6; //@line 883 "dictionary.cpp"
      var $cmp3=((($6))|0)==0; //@line 883 "dictionary.cpp"
      if ($cmp3) { __label__ = 1;continue $while_cond$2; } //@line 883 "dictionary.cpp"
  
      var $7=$ph; //@line 886 "dictionary.cpp"
      var $type=(($7+11)&4294967295); //@line 886 "dictionary.cpp"
      var $8=HEAPU8[($type)]; //@line 886 "dictionary.cpp"
      var $conv4=((($8))&255); //@line 886 "dictionary.cpp"
      var $cmp5=((($conv4))|0)==1; //@line 886 "dictionary.cpp"
      if ($cmp5) { __label__ = 5;; } else { __label__ = 24;; } //@line 886 "dictionary.cpp"
      $land_lhs_true$$if_end53$7: do { 
        if (__label__ == 5) {
  
          var $9=$ph; //@line 886 "dictionary.cpp"
          var $program=(($9+8)&4294967295); //@line 886 "dictionary.cpp"
          var $10=HEAPU16[(($program)>>1)]; //@line 886 "dictionary.cpp"
          var $conv6=((($10))&65535); //@line 886 "dictionary.cpp"
          var $cmp7=((($conv6))|0)==0; //@line 886 "dictionary.cpp"
          if (!($cmp7)) { __label__ = 24;break $land_lhs_true$$if_end53$7; } //@line 886 "dictionary.cpp"
  
          var $11=$phcode; //@line 890 "dictionary.cpp"
          var $conv9=((($11))&255); //@line 890 "dictionary.cpp"
          var $cmp10=((($conv9))|0)==8; //@line 890 "dictionary.cpp"
          if ($cmp10) { __label__ = 7;; } else { __label__ = 20;; } //@line 890 "dictionary.cpp"
          if (__label__ == 7) {
  
            var $12=$count; //@line 893 "dictionary.cpp"
            var $sub=((($12)-1)&4294967295); //@line 893 "dictionary.cpp"
            $j=$sub; //@line 893 "dictionary.cpp"
            __lastLabel__ = 7; ; //@line 894 "dictionary.cpp"
            $while_cond12$12: while(1) { 
  
              var $13=__lastLabel__ == 19 ? $dec : ($sub); //@line 894 "dictionary.cpp"
              var $cmp13=((($13))|0) > 0; //@line 894 "dictionary.cpp"
              if (!($cmp13)) { __label__ = 1;continue $while_cond$2; } //@line 894 "dictionary.cpp"
  
              var $14=$stressed_syllable_addr; //@line 894 "dictionary.cpp"
              var $15=HEAP32[(($14)>>2)]; //@line 894 "dictionary.cpp"
              var $cmp15=((($15))|0)==0; //@line 894 "dictionary.cpp"
              if (!($cmp15)) { __label__ = 1;continue $while_cond$2; } //@line 894 "dictionary.cpp"
  
              var $16=$j; //@line 894 "dictionary.cpp"
              var $17=$vowel_stress_addr; //@line 894 "dictionary.cpp"
              var $arrayidx17=(($17+$16)&4294967295); //@line 894 "dictionary.cpp"
              var $18=HEAP8[($arrayidx17)]; //@line 894 "dictionary.cpp"
              var $conv18=(tempInt=(($18)),(tempInt>=128?tempInt-256:tempInt)); //@line 894 "dictionary.cpp"
              var $cmp19=((($conv18))|0) < 4; //@line 894 "dictionary.cpp"
              if (!($cmp19)) { __label__ = 1;continue $while_cond$2; }
  
              var $19=$j; //@line 896 "dictionary.cpp"
              var $20=$vowel_stress_addr; //@line 896 "dictionary.cpp"
              var $arrayidx22=(($20+$19)&4294967295); //@line 896 "dictionary.cpp"
              var $21=HEAP8[($arrayidx22)]; //@line 896 "dictionary.cpp"
              var $conv23=(tempInt=(($21)),(tempInt>=128?tempInt-256:tempInt)); //@line 896 "dictionary.cpp"
              var $cmp24=((($conv23))|0)!=0; //@line 896 "dictionary.cpp"
              if ($cmp24) { __label__ = 12;; } else { __label__ = 19;; } //@line 896 "dictionary.cpp"
              if (__label__ == 12) {
  
                var $22=$j; //@line 896 "dictionary.cpp"
                var $23=$vowel_stress_addr; //@line 896 "dictionary.cpp"
                var $arrayidx26=(($23+$22)&4294967295); //@line 896 "dictionary.cpp"
                var $24=HEAP8[($arrayidx26)]; //@line 896 "dictionary.cpp"
                var $conv27=(tempInt=(($24)),(tempInt>=128?tempInt-256:tempInt)); //@line 896 "dictionary.cpp"
                var $cmp28=((($conv27))|0)!=1; //@line 896 "dictionary.cpp"
                if ($cmp28) { __label__ = 13;break $while_cond12$12; } //@line 896 "dictionary.cpp"
              }
  
              var $39=$j; //@line 915 "dictionary.cpp"
              var $dec=((($39)-1)&4294967295); //@line 915 "dictionary.cpp"
              $j=$dec; //@line 915 "dictionary.cpp"
              __lastLabel__ = 19; __label__ = 8;continue $while_cond12$12; //@line 916 "dictionary.cpp"
            }
  
            var $25=$j; //@line 899 "dictionary.cpp"
            var $26=$vowel_stress_addr; //@line 899 "dictionary.cpp"
            var $arrayidx30=(($26+$25)&4294967295); //@line 899 "dictionary.cpp"
            HEAP8[($arrayidx30)]=4; //@line 899 "dictionary.cpp"
            var $27=$max_stress; //@line 901 "dictionary.cpp"
            var $cmp31=((($27))|0) < 4; //@line 901 "dictionary.cpp"
            if ($cmp31) { __label__ = 14;; } else { __label__ = 15;; } //@line 901 "dictionary.cpp"
            if (__label__ == 14) {
  
              $max_stress=4; //@line 903 "dictionary.cpp"
              var $28=$j; //@line 904 "dictionary.cpp"
              $primary_posn=$28; //@line 904 "dictionary.cpp"
              ; //@line 905 "dictionary.cpp"
            }
  
            $ix=1; //@line 908 "dictionary.cpp"
            var $29=$ix; //@line 908 "dictionary.cpp"
            var $30=$j; //@line 908 "dictionary.cpp"
            var $cmp344=((($29))|0) < ((($30))|0); //@line 908 "dictionary.cpp"
            if (!($cmp344)) { __label__ = 1;continue $while_cond$2; } //@line 908 "dictionary.cpp"
            while(1) { 
  
              var $31=$ix; //@line 910 "dictionary.cpp"
              var $32=$vowel_stress_addr; //@line 910 "dictionary.cpp"
              var $arrayidx35=(($32+$31)&4294967295); //@line 910 "dictionary.cpp"
              var $33=HEAP8[($arrayidx35)]; //@line 910 "dictionary.cpp"
              var $conv36=(tempInt=(($33)),(tempInt>=128?tempInt-256:tempInt)); //@line 910 "dictionary.cpp"
              var $cmp37=((($conv36))|0)==4; //@line 910 "dictionary.cpp"
              if ($cmp37) { __label__ = 17;; } else { __label__ = 18;; } //@line 910 "dictionary.cpp"
              if (__label__ == 17) {
  
                var $34=$ix; //@line 911 "dictionary.cpp"
                var $35=$vowel_stress_addr; //@line 911 "dictionary.cpp"
                var $arrayidx39=(($35+$34)&4294967295); //@line 911 "dictionary.cpp"
                HEAP8[($arrayidx39)]=3; //@line 911 "dictionary.cpp"
                ; //@line 911 "dictionary.cpp"
              }
  
              var $36=$ix; //@line 908 "dictionary.cpp"
              var $inc=((($36)+1)&4294967295); //@line 908 "dictionary.cpp"
              $ix=$inc; //@line 908 "dictionary.cpp"
              var $37=$ix; //@line 908 "dictionary.cpp"
              var $38=$j; //@line 908 "dictionary.cpp"
              var $cmp34=((($37))|0) < ((($38))|0); //@line 908 "dictionary.cpp"
              if ($cmp34) { __label__ = 16;continue ; } else { __label__ = 1;continue $while_cond$2; } //@line 908 "dictionary.cpp"
            }
          }
          else if (__label__ == 20) {
  
            var $40=$ph; //@line 920 "dictionary.cpp"
            var $std_length=(($40+14)&4294967295); //@line 920 "dictionary.cpp"
            var $41=HEAPU8[($std_length)]; //@line 920 "dictionary.cpp"
            var $conv42=((($41))&255); //@line 920 "dictionary.cpp"
            var $cmp43=((($conv42))|0) < 4; //@line 920 "dictionary.cpp"
            if ($cmp43) { __label__ = 22;; } else { __label__ = 21;; } //@line 920 "dictionary.cpp"
            if (__label__ == 21) {
  
              var $42=$stressed_syllable_addr; //@line 920 "dictionary.cpp"
              var $43=HEAP32[(($42)>>2)]; //@line 920 "dictionary.cpp"
              var $cmp44=((($43))|0)==0; //@line 920 "dictionary.cpp"
              if (!($cmp44)) { __label__ = 1;continue $while_cond$2; } //@line 920 "dictionary.cpp"
            }
  
            var $44=$ph; //@line 922 "dictionary.cpp"
            var $std_length46=(($44+14)&4294967295); //@line 922 "dictionary.cpp"
            var $45=HEAPU8[($std_length46)]; //@line 922 "dictionary.cpp"
            var $conv47=((($45))&255); //@line 922 "dictionary.cpp"
            $stress=$conv47; //@line 922 "dictionary.cpp"
            var $46=$stress; //@line 924 "dictionary.cpp"
            var $47=$max_stress; //@line 924 "dictionary.cpp"
            var $cmp48=((($46))|0) > ((($47))|0); //@line 924 "dictionary.cpp"
            if (!($cmp48)) { __label__ = 1;continue $while_cond$2; } //@line 924 "dictionary.cpp"
  
            var $48=$stress; //@line 925 "dictionary.cpp"
            $max_stress=$48; //@line 925 "dictionary.cpp"
            __label__ = 1;continue $while_cond$2; //@line 925 "dictionary.cpp"
          }
        }
      } while(0);
  
      var $49=$ph; //@line 931 "dictionary.cpp"
      var $type54=(($49+11)&4294967295); //@line 931 "dictionary.cpp"
      var $50=HEAPU8[($type54)]; //@line 931 "dictionary.cpp"
      var $conv55=((($50))&255); //@line 931 "dictionary.cpp"
      var $cmp56=((($conv55))|0)==2; //@line 931 "dictionary.cpp"
      if ($cmp56) { __label__ = 25;; } else { __label__ = 34;; } //@line 931 "dictionary.cpp"
      $land_lhs_true57$$if_else78$35: do { 
        if (__label__ == 25) {
  
          var $51=$ph; //@line 931 "dictionary.cpp"
          var $phflags=(($51+4)&4294967295); //@line 931 "dictionary.cpp"
          var $52=HEAP32[(($phflags)>>2)]; //@line 931 "dictionary.cpp"
          var $and=($52) & 1048576; //@line 931 "dictionary.cpp"
          var $tobool=((($and))|0)!=0; //@line 931 "dictionary.cpp"
          if ($tobool) { __label__ = 34;break $land_lhs_true57$$if_else78$35; } //@line 931 "dictionary.cpp"
  
          var $53=$stress; //@line 933 "dictionary.cpp"
          var $conv59=((($53)) & 255); //@line 933 "dictionary.cpp"
          var $54=$count; //@line 933 "dictionary.cpp"
          var $55=$vowel_stress_addr; //@line 933 "dictionary.cpp"
          var $arrayidx60=(($55+$54)&4294967295); //@line 933 "dictionary.cpp"
          HEAP8[($arrayidx60)]=$conv59; //@line 933 "dictionary.cpp"
          var $56=$stress; //@line 934 "dictionary.cpp"
          var $cmp61=((($56))|0) >= 4; //@line 934 "dictionary.cpp"
          if ($cmp61) { __lastLabel__ = 26; __label__ = 27;; } else { __lastLabel__ = 26; __label__ = 29;; } //@line 934 "dictionary.cpp"
          $land_lhs_true62$$if_end65$38: do { 
            if (__label__ == 27) {
  
              var $57=$stress; //@line 934 "dictionary.cpp"
              var $58=$max_stress; //@line 934 "dictionary.cpp"
              var $cmp63=((($57))|0) >= ((($58))|0); //@line 934 "dictionary.cpp"
              if ($cmp63) { __lastLabel__ = 27; ; } else { __lastLabel__ = 27; __label__ = 29;break $land_lhs_true62$$if_end65$38; } //@line 934 "dictionary.cpp"
  
              var $59=$count; //@line 936 "dictionary.cpp"
              $primary_posn=$59; //@line 936 "dictionary.cpp"
              var $60=$stress; //@line 937 "dictionary.cpp"
              $max_stress=$60; //@line 937 "dictionary.cpp"
              __lastLabel__ = 28; ; //@line 938 "dictionary.cpp"
            }
          } while(0);
  
          var $61=__lastLabel__ == 28 ? $60 : (__lastLabel__ == 27 ? $57 : ($56)); //@line 940 "dictionary.cpp"
          var $cmp66=((($61))|0) < 0; //@line 940 "dictionary.cpp"
          if ($cmp66) { __label__ = 30;; } else { __label__ = 33;; } //@line 940 "dictionary.cpp"
          $land_lhs_true67$$if_end76$42: do { 
            if (__label__ == 30) {
  
              var $62=$control_addr; //@line 940 "dictionary.cpp"
              var $and68=($62) & 1; //@line 940 "dictionary.cpp"
              var $tobool69=((($and68))|0)!=0; //@line 940 "dictionary.cpp"
              if (!($tobool69)) { __label__ = 33;break $land_lhs_true67$$if_end76$42; } //@line 940 "dictionary.cpp"
  
              var $63=$ph; //@line 940 "dictionary.cpp"
              var $phflags71=(($63+4)&4294967295); //@line 940 "dictionary.cpp"
              var $64=HEAP32[(($phflags71)>>2)]; //@line 940 "dictionary.cpp"
              var $and72=($64) & 2; //@line 940 "dictionary.cpp"
              var $tobool73=((($and72))|0)!=0; //@line 940 "dictionary.cpp"
              if (!($tobool73)) { __label__ = 33;break $land_lhs_true67$$if_end76$42; } //@line 940 "dictionary.cpp"
  
              var $65=$count; //@line 941 "dictionary.cpp"
              var $66=$vowel_stress_addr; //@line 941 "dictionary.cpp"
              var $arrayidx75=(($66+$65)&4294967295); //@line 941 "dictionary.cpp"
              HEAP8[($arrayidx75)]=1; //@line 941 "dictionary.cpp"
              ; //@line 941 "dictionary.cpp"
            }
          } while(0);
  
          var $67=$count; //@line 943 "dictionary.cpp"
          var $inc77=((($67)+1)&4294967295); //@line 943 "dictionary.cpp"
          $count=$inc77; //@line 943 "dictionary.cpp"
          $stress=-1; //@line 944 "dictionary.cpp"
          __label__ = 38;break $land_lhs_true57$$if_else78$35; //@line 945 "dictionary.cpp"
        }
      } while(0);
      $if_else78$$if_end93$47: do { 
        if (__label__ == 34) {
  
          var $68=$phcode; //@line 947 "dictionary.cpp"
          var $conv79=((($68))&255); //@line 947 "dictionary.cpp"
          var $cmp80=((($conv79))|0)==20; //@line 947 "dictionary.cpp"
          if (!($cmp80)) { __label__ = 38;break $if_else78$$if_end93$47; } //@line 947 "dictionary.cpp"
  
          var $69=$stress; //@line 950 "dictionary.cpp"
          var $conv82=((($69)) & 255); //@line 950 "dictionary.cpp"
          var $70=$count; //@line 950 "dictionary.cpp"
          var $71=$vowel_stress_addr; //@line 950 "dictionary.cpp"
          var $arrayidx83=(($71+$70)&4294967295); //@line 950 "dictionary.cpp"
          HEAP8[($arrayidx83)]=$conv82; //@line 950 "dictionary.cpp"
          var $72=$stress; //@line 951 "dictionary.cpp"
          var $cmp84=((($72))|0)==0; //@line 951 "dictionary.cpp"
          if (!($cmp84)) { __label__ = 38;break $if_else78$$if_end93$47; } //@line 951 "dictionary.cpp"
  
          var $73=$control_addr; //@line 951 "dictionary.cpp"
          var $and86=($73) & 1; //@line 951 "dictionary.cpp"
          var $tobool87=((($and86))|0)!=0; //@line 951 "dictionary.cpp"
          if (!($tobool87)) { __label__ = 38;break $if_else78$$if_end93$47; } //@line 951 "dictionary.cpp"
  
          var $74=$count; //@line 952 "dictionary.cpp"
          var $inc89=((($74)+1)&4294967295); //@line 952 "dictionary.cpp"
          $count=$inc89; //@line 952 "dictionary.cpp"
          var $75=$vowel_stress_addr; //@line 952 "dictionary.cpp"
          var $arrayidx90=(($75+$74)&4294967295); //@line 952 "dictionary.cpp"
          HEAP8[($arrayidx90)]=1; //@line 952 "dictionary.cpp"
          ; //@line 952 "dictionary.cpp"
        }
      } while(0);
  
      var $76=$phcode; //@line 955 "dictionary.cpp"
      var $77=$ph_out; //@line 955 "dictionary.cpp"
      var $incdec_ptr94=(($77+1)&4294967295); //@line 955 "dictionary.cpp"
      $ph_out=$incdec_ptr94; //@line 955 "dictionary.cpp"
      HEAP8[($77)]=$76; //@line 955 "dictionary.cpp"
      __label__ = 1;continue $while_cond$2; //@line 956 "dictionary.cpp"
    }
  
    var $78=$count; //@line 957 "dictionary.cpp"
    var $79=$vowel_stress_addr; //@line 957 "dictionary.cpp"
    var $arrayidx96=(($79+$78)&4294967295); //@line 957 "dictionary.cpp"
    HEAP8[($arrayidx96)]=1; //@line 957 "dictionary.cpp"
    var $80=$ph_out; //@line 958 "dictionary.cpp"
    HEAP8[($80)]=0; //@line 958 "dictionary.cpp"
    var $81=$stressed_syllable_addr; //@line 961 "dictionary.cpp"
    var $82=HEAP32[(($81)>>2)]; //@line 961 "dictionary.cpp"
    var $cmp97=((($82))|0) > 0; //@line 961 "dictionary.cpp"
    if ($cmp97) { __label__ = 40;; } else { __label__ = 43;; } //@line 961 "dictionary.cpp"
    $if_then98$$if_end104$54: do { 
      if (__label__ == 40) {
  
        var $83=$stressed_syllable_addr; //@line 963 "dictionary.cpp"
        var $84=HEAP32[(($83)>>2)]; //@line 963 "dictionary.cpp"
        var $85=$count; //@line 963 "dictionary.cpp"
        var $cmp99=((($84))|0) >= ((($85))|0); //@line 963 "dictionary.cpp"
        if ($cmp99) { __label__ = 41;; } else { __label__ = 42;; } //@line 963 "dictionary.cpp"
        if (__label__ == 41) {
  
          var $86=$count; //@line 964 "dictionary.cpp"
          var $sub101=((($86)-1)&4294967295); //@line 964 "dictionary.cpp"
          var $87=$stressed_syllable_addr; //@line 964 "dictionary.cpp"
          HEAP32[(($87)>>2)]=$sub101; //@line 964 "dictionary.cpp"
          ; //@line 964 "dictionary.cpp"
        }
  
        var $88=$stressed_syllable_addr; //@line 966 "dictionary.cpp"
        var $89=HEAP32[(($88)>>2)]; //@line 966 "dictionary.cpp"
        var $90=$vowel_stress_addr; //@line 966 "dictionary.cpp"
        var $arrayidx103=(($90+$89)&4294967295); //@line 966 "dictionary.cpp"
        HEAP8[($arrayidx103)]=4; //@line 966 "dictionary.cpp"
        $max_stress=4; //@line 967 "dictionary.cpp"
        var $91=$stressed_syllable_addr; //@line 968 "dictionary.cpp"
        var $92=HEAP32[(($91)>>2)]; //@line 968 "dictionary.cpp"
        $primary_posn=$92; //@line 968 "dictionary.cpp"
        ; //@line 971 "dictionary.cpp"
      }
      else if (__label__ == 43) {
  
        var $_pr=$max_stress; //@line 971 "dictionary.cpp"
        var $cmp105=((($_pr))|0)==5; //@line 971 "dictionary.cpp"
        if (!($cmp105)) { __label__ = 53;break $if_then98$$if_end104$54; } //@line 971 "dictionary.cpp"
  
        $ix=1; //@line 974 "dictionary.cpp"
        var $93=$ix; //@line 974 "dictionary.cpp"
        var $94=$count; //@line 974 "dictionary.cpp"
        var $cmp1083=((($93))|0) < ((($94))|0); //@line 974 "dictionary.cpp"
        if ($cmp1083) { __label__ = 45;; } else { __label__ = 52;; } //@line 974 "dictionary.cpp"
        $for_body109$$for_end130$61: do { 
          if (__label__ == 45) {
            while(1) { 
  
              var $95=$ix; //@line 976 "dictionary.cpp"
              var $96=$vowel_stress_addr; //@line 976 "dictionary.cpp"
              var $arrayidx110=(($96+$95)&4294967295); //@line 976 "dictionary.cpp"
              var $97=HEAP8[($arrayidx110)]; //@line 976 "dictionary.cpp"
              var $conv111=(tempInt=(($97)),(tempInt>=128?tempInt-256:tempInt)); //@line 976 "dictionary.cpp"
              var $cmp112=((($conv111))|0)==4; //@line 976 "dictionary.cpp"
              if ($cmp112) { __label__ = 46;; } else { __label__ = 49;; } //@line 976 "dictionary.cpp"
              if (__label__ == 46) {
  
                var $98=$tr_addr; //@line 978 "dictionary.cpp"
                var $langopts=(($98)&4294967295); //@line 978 "dictionary.cpp"
                var $stress_flags=(($langopts+12)&4294967295); //@line 978 "dictionary.cpp"
                var $99=HEAP32[(($stress_flags)>>2)]; //@line 978 "dictionary.cpp"
                var $and114=($99) & 131072; //@line 978 "dictionary.cpp"
                var $tobool115=((($and114))|0)!=0; //@line 978 "dictionary.cpp"
                var $100=$ix; //@line 979 "dictionary.cpp"
                var $101=$vowel_stress_addr; //@line 979 "dictionary.cpp"
                var $arrayidx117=(($101+$100)&4294967295); //@line 979 "dictionary.cpp"
                if ($tobool115) { __label__ = 47;; } else { __label__ = 48;; } //@line 978 "dictionary.cpp"
                if (__label__ == 47) {
  
                  HEAP8[($arrayidx117)]=1; //@line 979 "dictionary.cpp"
                  ; //@line 979 "dictionary.cpp"
                }
                else if (__label__ == 48) {
  
                  HEAP8[($arrayidx117)]=3; //@line 981 "dictionary.cpp"
                  ;
                }
              }
  
              var $102=$ix; //@line 984 "dictionary.cpp"
              var $103=$vowel_stress_addr; //@line 984 "dictionary.cpp"
              var $arrayidx122=(($103+$102)&4294967295); //@line 984 "dictionary.cpp"
              var $104=HEAP8[($arrayidx122)]; //@line 984 "dictionary.cpp"
              var $conv123=(tempInt=(($104)),(tempInt>=128?tempInt-256:tempInt)); //@line 984 "dictionary.cpp"
              var $cmp124=((($conv123))|0)==5; //@line 984 "dictionary.cpp"
              if ($cmp124) { __label__ = 50;; } else { __label__ = 51;; } //@line 984 "dictionary.cpp"
              if (__label__ == 50) {
  
                var $105=$ix; //@line 986 "dictionary.cpp"
                var $106=$vowel_stress_addr; //@line 986 "dictionary.cpp"
                var $arrayidx126=(($106+$105)&4294967295); //@line 986 "dictionary.cpp"
                HEAP8[($arrayidx126)]=4; //@line 986 "dictionary.cpp"
                var $107=$ix; //@line 987 "dictionary.cpp"
                $primary_posn=$107; //@line 987 "dictionary.cpp"
                ; //@line 988 "dictionary.cpp"
              }
  
              var $108=$ix; //@line 974 "dictionary.cpp"
              var $inc129=((($108)+1)&4294967295); //@line 974 "dictionary.cpp"
              $ix=$inc129; //@line 974 "dictionary.cpp"
              var $109=$ix; //@line 974 "dictionary.cpp"
              var $110=$count; //@line 974 "dictionary.cpp"
              var $cmp108=((($109))|0) < ((($110))|0); //@line 974 "dictionary.cpp"
              if ($cmp108) { __label__ = 45;continue ; } else { __label__ = 52;break $for_body109$$for_end130$61; } //@line 974 "dictionary.cpp"
            }
          }
        } while(0);
  
        $max_stress=4; //@line 990 "dictionary.cpp"
        ; //@line 991 "dictionary.cpp"
      }
    } while(0);
  
    var $111=$primary_posn; //@line 993 "dictionary.cpp"
    var $112=$stressed_syllable_addr; //@line 993 "dictionary.cpp"
    HEAP32[(($112)>>2)]=$111; //@line 993 "dictionary.cpp"
    var $113=$count; //@line 994 "dictionary.cpp"
    var $114=$vowel_count_addr; //@line 994 "dictionary.cpp"
    HEAP32[(($114)>>2)]=$113; //@line 994 "dictionary.cpp"
    var $115=$max_stress; //@line 995 "dictionary.cpp"
    ;
    return $115; //@line 995 "dictionary.cpp"
    return null;
  }
  

  function __Z13SetWordStressP10TranslatorPcPjii($tr, $output, $dictionary_flags, $tonic, $control) {
    var __stackBase__  = STACKTOP; STACKTOP += 408; _memset(__stackBase__, 0, 408);
    var __label__;
    var __lastLabel__ = null;
  
    var $tr_addr;
    var $output_addr;
    var $dictionary_flags_addr;
    var $tonic_addr;
    var $control_addr;
    var $phcode;
    var $p;
    var $ph;
    var $stress;
    var $max_stress;
    var $vowel_count=__stackBase__;
    var $ix;
    var $v;
    var $v_stress;
    var $stressed_syllable=__stackBase__+4;
    var $max_stress_posn;
    var $unstressed_word;
    var $max_output;
    var $final_ph;
    var $final_ph2;
    var $mnem;
    var $mnem2;
    var $post_tonic;
    var $opt_length;
    var $done;
    var $stressflags;
    var $dflags;
    var $first_primary;
    var $vowel_stress=__stackBase__+8;
    var $syllable_weight=__stackBase__+88;
    var $vowel_length=__stackBase__+168;
    var $phonetic=__stackBase__+248;
    var $weight;
    var $lengthened;
    var $wt;
    var $max_weight;
    var $prev_stressed;
    var $shorten;
    $tr_addr=$tr;
    $output_addr=$output;
    $dictionary_flags_addr=$dictionary_flags;
    $tonic_addr=$tonic;
    $control_addr=$control;
    $unstressed_word=0; //@line 1084 "dictionary.cpp"
    $dflags=0; //@line 1094 "dictionary.cpp"
    var $0=$tr_addr; //@line 1112 "dictionary.cpp"
    var $langopts=(($0)&4294967295); //@line 1112 "dictionary.cpp"
    var $stress_flags=(($langopts+12)&4294967295); //@line 1112 "dictionary.cpp"
    var $1=HEAP32[(($stress_flags)>>2)]; //@line 1112 "dictionary.cpp"
    $stressflags=$1; //@line 1112 "dictionary.cpp"
    var $2=$dictionary_flags_addr; //@line 1114 "dictionary.cpp"
    var $cmp=((($2))|0)!=0; //@line 1114 "dictionary.cpp"
    if ($cmp) { __label__ = 1;; } else { __label__ = 2;; } //@line 1114 "dictionary.cpp"
    if (__label__ == 1) {
  
      var $3=$dictionary_flags_addr; //@line 1115 "dictionary.cpp"
      var $arrayidx=(($3)&4294967295); //@line 1115 "dictionary.cpp"
      var $4=HEAP32[(($arrayidx)>>2)]; //@line 1115 "dictionary.cpp"
      $dflags=$4; //@line 1115 "dictionary.cpp"
      ; //@line 1115 "dictionary.cpp"
    }
  
    $ix=0; //@line 1118 "dictionary.cpp"
    __lastLabel__ = 2; ; //@line 1118 "dictionary.cpp"
    $for_cond$5: while(1) { 
  
      var $5=__lastLabel__ == 7 ? $inc : (0); //@line 1118 "dictionary.cpp"
      var $cmp1=((($5))|0) < 160; //@line 1118 "dictionary.cpp"
      var $6=$ix; //@line 1120 "dictionary.cpp"
      if ($cmp1) { __lastLabel__ = 3; ; } else { __lastLabel__ = 3; __label__ = 8;break $for_cond$5; } //@line 1118 "dictionary.cpp"
  
      var $7=$output_addr; //@line 1120 "dictionary.cpp"
      var $arrayidx2=(($7+$6)&4294967295); //@line 1120 "dictionary.cpp"
      var $8=HEAP8[($arrayidx2)]; //@line 1120 "dictionary.cpp"
      var $9=$ix; //@line 1120 "dictionary.cpp"
      var $arrayidx3=(($phonetic+$9)&4294967295); //@line 1120 "dictionary.cpp"
      HEAP8[($arrayidx3)]=$8; //@line 1120 "dictionary.cpp"
      var $10=$ix; //@line 1122 "dictionary.cpp"
      var $arrayidx4=(($phonetic+$10)&4294967295); //@line 1122 "dictionary.cpp"
      var $11=HEAPU8[($arrayidx4)]; //@line 1122 "dictionary.cpp"
      var $conv=((($11))&255); //@line 1122 "dictionary.cpp"
      var $12=HEAP32[((_n_phoneme_tab)>>2)]; //@line 1122 "dictionary.cpp"
      var $cmp5=((($conv))|0) >= ((($12))|0); //@line 1122 "dictionary.cpp"
      if ($cmp5) { __label__ = 5;; } else { __label__ = 6;; } //@line 1122 "dictionary.cpp"
      if (__label__ == 5) {
  
        var $13=$ix; //@line 1123 "dictionary.cpp"
        var $arrayidx7=(($phonetic+$13)&4294967295); //@line 1123 "dictionary.cpp"
        HEAP8[($arrayidx7)]=13; //@line 1123 "dictionary.cpp"
        ; //@line 1123 "dictionary.cpp"
      }
  
      var $14=$ix; //@line 1124 "dictionary.cpp"
      var $arrayidx9=(($phonetic+$14)&4294967295); //@line 1124 "dictionary.cpp"
      var $15=HEAPU8[($arrayidx9)]; //@line 1124 "dictionary.cpp"
      var $conv10=((($15))&255); //@line 1124 "dictionary.cpp"
      var $cmp11=((($conv10))|0)==0; //@line 1124 "dictionary.cpp"
      if ($cmp11) { __lastLabel__ = 6; __label__ = 8;break $for_cond$5; } else { __lastLabel__ = 6; ; } //@line 1124 "dictionary.cpp"
  
      var $16=$ix; //@line 1118 "dictionary.cpp"
      var $inc=((($16)+1)&4294967295); //@line 1118 "dictionary.cpp"
      $ix=$inc; //@line 1118 "dictionary.cpp"
      __lastLabel__ = 7; __label__ = 3;continue $for_cond$5; //@line 1118 "dictionary.cpp"
    }
  
    var $17=__lastLabel__ == 6 ? $14 : ($6); //@line 1127 "dictionary.cpp"
    var $cmp14=((($17))|0)==0; //@line 1127 "dictionary.cpp"
    if ($cmp14) { __label__ = 223;; } else { __label__ = 9;; } //@line 1127 "dictionary.cpp"
    if (__label__ == 9) {
  
      var $18=$ix; //@line 1128 "dictionary.cpp"
      var $sub=((($18)-1)&4294967295); //@line 1128 "dictionary.cpp"
      var $arrayidx17=(($phonetic+$sub)&4294967295); //@line 1128 "dictionary.cpp"
      var $19=HEAPU8[($arrayidx17)]; //@line 1128 "dictionary.cpp"
      var $conv18=((($19))&255); //@line 1128 "dictionary.cpp"
      $final_ph=$conv18; //@line 1128 "dictionary.cpp"
      var $20=$ix; //@line 1129 "dictionary.cpp"
      var $sub19=((($20)-2)&4294967295); //@line 1129 "dictionary.cpp"
      var $arrayidx20=(($phonetic+$sub19)&4294967295); //@line 1129 "dictionary.cpp"
      var $21=HEAPU8[($arrayidx20)]; //@line 1129 "dictionary.cpp"
      var $conv21=((($21))&255); //@line 1129 "dictionary.cpp"
      $final_ph2=$conv21; //@line 1129 "dictionary.cpp"
      var $22=$output_addr; //@line 1131 "dictionary.cpp"
      var $add_ptr=(($22+157)&4294967295); //@line 1131 "dictionary.cpp"
      $max_output=$add_ptr; //@line 1131 "dictionary.cpp"
      var $23=$dflags; //@line 1134 "dictionary.cpp"
      var $and=($23) & 7; //@line 1134 "dictionary.cpp"
      HEAP32[(($stressed_syllable)>>2)]=$and; //@line 1134 "dictionary.cpp"
      var $24=$dflags; //@line 1135 "dictionary.cpp"
      var $and22=($24) & 8; //@line 1135 "dictionary.cpp"
      var $tobool=((($and22))|0)!=0; //@line 1135 "dictionary.cpp"
      if ($tobool) { __label__ = 10;; } else { __label__ = 11;; } //@line 1135 "dictionary.cpp"
      if (__label__ == 10) {
  
        var $25=$dflags; //@line 1138 "dictionary.cpp"
        var $and24=($25) & 3; //@line 1138 "dictionary.cpp"
        HEAP32[(($stressed_syllable)>>2)]=$and24; //@line 1138 "dictionary.cpp"
        $unstressed_word=1; //@line 1139 "dictionary.cpp"
        ; //@line 1140 "dictionary.cpp"
      }
  
      var $26=$tr_addr; //@line 1142 "dictionary.cpp"
      var $arraydecay=(($phonetic)&4294967295); //@line 1142 "dictionary.cpp"
      var $arraydecay26=(($vowel_stress)&4294967295); //@line 1142 "dictionary.cpp"
      var $call=__ZL14GetVowelStressP10TranslatorPhPaRiS3_i($26, $arraydecay, $arraydecay26, $vowel_count, $stressed_syllable, 1); //@line 1142 "dictionary.cpp"
      $max_stress=$call; //@line 1142 "dictionary.cpp"
      var $cmp27=((($call))|0) < 0; //@line 1143 "dictionary.cpp"
      if ($cmp27) { __label__ = 12;; } else { __label__ = 17;; } //@line 1143 "dictionary.cpp"
      $land_lhs_true$$if_end40$18: do { 
        if (__label__ == 12) {
  
          var $27=$dictionary_flags_addr; //@line 1143 "dictionary.cpp"
          var $tobool28=((($27))|0)!=0; //@line 1143 "dictionary.cpp"
          if (!($tobool28)) { __label__ = 17;break $land_lhs_true$$if_end40$18; } //@line 1143 "dictionary.cpp"
  
          var $28=$tr_addr; //@line 1145 "dictionary.cpp"
          var $langopts30=(($28)&4294967295); //@line 1145 "dictionary.cpp"
          var $stress_flags31=(($langopts30+12)&4294967295); //@line 1145 "dictionary.cpp"
          var $29=HEAP32[(($stress_flags31)>>2)]; //@line 1145 "dictionary.cpp"
          var $and32=($29) & 1; //@line 1145 "dictionary.cpp"
          var $tobool33=((($and32))|0)!=0; //@line 1145 "dictionary.cpp"
          if ($tobool33) { __label__ = 14;; } else { __label__ = 16;; } //@line 1145 "dictionary.cpp"
          $land_lhs_true34$$if_end39$21: do { 
            if (__label__ == 14) {
  
              var $30=HEAP32[(($vowel_count)>>2)]; //@line 1145 "dictionary.cpp"
              var $cmp35=((($30))|0)==2; //@line 1145 "dictionary.cpp"
              if (!($cmp35)) { __label__ = 16;break $land_lhs_true34$$if_end39$21; } //@line 1145 "dictionary.cpp"
  
              var $arrayidx37=(($vowel_stress+1)&4294967295); //@line 1148 "dictionary.cpp"
              HEAP8[($arrayidx37)]=0; //@line 1148 "dictionary.cpp"
              var $31=$dictionary_flags_addr; //@line 1149 "dictionary.cpp"
              var $arrayidx38=(($31)&4294967295); //@line 1149 "dictionary.cpp"
              var $32=HEAP32[(($arrayidx38)>>2)]; //@line 1149 "dictionary.cpp"
              var $or=($32) | 4096; //@line 1149 "dictionary.cpp"
              HEAP32[(($arrayidx38)>>2)]=$or; //@line 1149 "dictionary.cpp"
              ; //@line 1150 "dictionary.cpp"
            }
          } while(0);
  
          $max_stress=0; //@line 1151 "dictionary.cpp"
          ; //@line 1152 "dictionary.cpp"
        }
      } while(0);
  
      $ix=1; //@line 1155 "dictionary.cpp"
      var $arraydecay41=(($phonetic)&4294967295); //@line 1156 "dictionary.cpp"
      $p=$arraydecay41; //@line 1156 "dictionary.cpp"
      var $33=$p; //@line 1156 "dictionary.cpp"
      var $34=HEAPU8[($33)]; //@line 1156 "dictionary.cpp"
      var $conv4319=((($34))&255); //@line 1156 "dictionary.cpp"
      var $cmp4420=((($conv4319))|0)!=0; //@line 1156 "dictionary.cpp"
      if ($cmp4420) { __label__ = 18;; } else { __label__ = 33;; } //@line 1156 "dictionary.cpp"
      $for_body45$$for_end109$26: do { 
        if (__label__ == 18) {
          $for_body45$27: while(1) { 
  
            var $35=$p; //@line 1158 "dictionary.cpp"
            var $arrayidx46=(($35)&4294967295); //@line 1158 "dictionary.cpp"
            var $36=HEAPU8[($arrayidx46)]; //@line 1158 "dictionary.cpp"
            var $idxprom=((($36))&255); //@line 1158 "dictionary.cpp"
            var $arrayidx47=((_phoneme_tab+$idxprom*4)&4294967295); //@line 1158 "dictionary.cpp"
            var $37=HEAP32[(($arrayidx47)>>2)]; //@line 1158 "dictionary.cpp"
            var $type=(($37+11)&4294967295); //@line 1158 "dictionary.cpp"
            var $38=HEAPU8[($type)]; //@line 1158 "dictionary.cpp"
            var $conv48=((($38))&255); //@line 1158 "dictionary.cpp"
            var $cmp49=((($conv48))|0)==2; //@line 1158 "dictionary.cpp"
            if ($cmp49) { __label__ = 19;; } else { __label__ = 32;; } //@line 1158 "dictionary.cpp"
            $land_lhs_true50$$for_inc107$29: do { 
              if (__label__ == 19) {
  
                var $39=$p; //@line 1158 "dictionary.cpp"
                var $arrayidx51=(($39)&4294967295); //@line 1158 "dictionary.cpp"
                var $40=HEAPU8[($arrayidx51)]; //@line 1158 "dictionary.cpp"
                var $idxprom52=((($40))&255); //@line 1158 "dictionary.cpp"
                var $arrayidx53=((_phoneme_tab+$idxprom52*4)&4294967295); //@line 1158 "dictionary.cpp"
                var $41=HEAP32[(($arrayidx53)>>2)]; //@line 1158 "dictionary.cpp"
                var $phflags=(($41+4)&4294967295); //@line 1158 "dictionary.cpp"
                var $42=HEAP32[(($phflags)>>2)]; //@line 1158 "dictionary.cpp"
                var $and54=($42) & 1048576; //@line 1158 "dictionary.cpp"
                var $tobool55=((($and54))|0)!=0; //@line 1158 "dictionary.cpp"
                if ($tobool55) { __label__ = 32;break $land_lhs_true50$$for_inc107$29; } //@line 1158 "dictionary.cpp"
  
                $weight=0; //@line 1160 "dictionary.cpp"
                $lengthened=0; //@line 1161 "dictionary.cpp"
                var $43=$p; //@line 1163 "dictionary.cpp"
                var $arrayidx57=(($43+1)&4294967295); //@line 1163 "dictionary.cpp"
                var $44=HEAPU8[($arrayidx57)]; //@line 1163 "dictionary.cpp"
                var $idxprom58=((($44))&255); //@line 1163 "dictionary.cpp"
                var $arrayidx59=((_phoneme_tab+$idxprom58*4)&4294967295); //@line 1163 "dictionary.cpp"
                var $45=HEAP32[(($arrayidx59)>>2)]; //@line 1163 "dictionary.cpp"
                var $code=(($45+10)&4294967295); //@line 1163 "dictionary.cpp"
                var $46=HEAPU8[($code)]; //@line 1163 "dictionary.cpp"
                var $conv60=((($46))&255); //@line 1163 "dictionary.cpp"
                var $cmp61=((($conv60))|0)==12; //@line 1163 "dictionary.cpp"
                if ($cmp61) { __label__ = 21;; } else { __label__ = 22;; } //@line 1163 "dictionary.cpp"
                $if_end63_thread$$if_end63$32: do { 
                  if (__label__ == 21) {
  
                    $lengthened=1; //@line 1164 "dictionary.cpp"
                    __label__ = 24;break $if_end63_thread$$if_end63$32; //@line 1166 "dictionary.cpp"
                  }
                  else if (__label__ == 22) {
  
                    var $_pr3=$lengthened; //@line 1166 "dictionary.cpp"
                    var $tobool64=((($_pr3))|0)!=0; //@line 1166 "dictionary.cpp"
                    if ($tobool64) { __label__ = 24;break $if_end63_thread$$if_end63$32; } //@line 1166 "dictionary.cpp"
  
                    var $47=$p; //@line 1166 "dictionary.cpp"
                    var $arrayidx65=(($47)&4294967295); //@line 1166 "dictionary.cpp"
                    var $48=HEAPU8[($arrayidx65)]; //@line 1166 "dictionary.cpp"
                    var $idxprom66=((($48))&255); //@line 1166 "dictionary.cpp"
                    var $arrayidx67=((_phoneme_tab+$idxprom66*4)&4294967295); //@line 1166 "dictionary.cpp"
                    var $49=HEAP32[(($arrayidx67)>>2)]; //@line 1166 "dictionary.cpp"
                    var $phflags68=(($49+4)&4294967295); //@line 1166 "dictionary.cpp"
                    var $50=HEAP32[(($phflags68)>>2)]; //@line 1166 "dictionary.cpp"
                    var $and69=($50) & 2097152; //@line 1166 "dictionary.cpp"
                    var $tobool70=((($and69))|0)!=0; //@line 1166 "dictionary.cpp"
                    if ($tobool70) { __label__ = 24;break $if_end63_thread$$if_end63$32; } else { __label__ = 25;break $if_end63_thread$$if_end63$32; } //@line 1166 "dictionary.cpp"
                  }
                } while(0);
                if (__label__ == 24) {
  
                  var $51=$weight; //@line 1169 "dictionary.cpp"
                  var $inc72=((($51)+1)&4294967295); //@line 1169 "dictionary.cpp"
                  $weight=$inc72; //@line 1169 "dictionary.cpp"
                  ; //@line 1170 "dictionary.cpp"
                }
  
                var $52=$weight; //@line 1171 "dictionary.cpp"
                var $conv74=((($52)) & 255); //@line 1171 "dictionary.cpp"
                var $53=$ix; //@line 1171 "dictionary.cpp"
                var $arrayidx75=(($vowel_length+$53)&4294967295); //@line 1171 "dictionary.cpp"
                HEAP8[($arrayidx75)]=$conv74; //@line 1171 "dictionary.cpp"
                var $54=$lengthened; //@line 1173 "dictionary.cpp"
                var $tobool76=((($54))|0)!=0; //@line 1173 "dictionary.cpp"
                if ($tobool76) { __label__ = 26;; } else { __label__ = 27;; } //@line 1173 "dictionary.cpp"
                if (__label__ == 26) {
  
                  var $55=$p; //@line 1173 "dictionary.cpp"
                  var $incdec_ptr=(($55+1)&4294967295); //@line 1173 "dictionary.cpp"
                  $p=$incdec_ptr; //@line 1173 "dictionary.cpp"
                  ; //@line 1173 "dictionary.cpp"
                }
  
                var $56=$p; //@line 1175 "dictionary.cpp"
                var $arrayidx79=(($56+1)&4294967295); //@line 1175 "dictionary.cpp"
                var $57=HEAPU8[($arrayidx79)]; //@line 1175 "dictionary.cpp"
                var $idxprom80=((($57))&255); //@line 1175 "dictionary.cpp"
                var $arrayidx81=((_phoneme_tab+$idxprom80*4)&4294967295); //@line 1175 "dictionary.cpp"
                var $58=HEAP32[(($arrayidx81)>>2)]; //@line 1175 "dictionary.cpp"
                var $type82=(($58+11)&4294967295); //@line 1175 "dictionary.cpp"
                var $59=HEAPU8[($type82)]; //@line 1175 "dictionary.cpp"
                var $idxprom83=((($59))&255); //@line 1175 "dictionary.cpp"
                var $arrayidx84=((__ZZ13SetWordStressP10TranslatorPcPjiiE15consonant_types+$idxprom83)&4294967295); //@line 1175 "dictionary.cpp"
                var $60=HEAP8[($arrayidx84)]; //@line 1175 "dictionary.cpp"
                var $tobool85=(tempInt=(($60)),(tempInt>=128?tempInt-256:tempInt))!=0; //@line 1175 "dictionary.cpp"
                if ($tobool85) { __label__ = 28;; } else { __label__ = 31;; } //@line 1175 "dictionary.cpp"
                $land_lhs_true86$$if_end102$42: do { 
                  if (__label__ == 28) {
  
                    var $61=$p; //@line 1175 "dictionary.cpp"
                    var $arrayidx87=(($61+2)&4294967295); //@line 1175 "dictionary.cpp"
                    var $62=HEAPU8[($arrayidx87)]; //@line 1175 "dictionary.cpp"
                    var $idxprom88=((($62))&255); //@line 1175 "dictionary.cpp"
                    var $arrayidx89=((_phoneme_tab+$idxprom88*4)&4294967295); //@line 1175 "dictionary.cpp"
                    var $63=HEAP32[(($arrayidx89)>>2)]; //@line 1175 "dictionary.cpp"
                    var $type90=(($63+11)&4294967295); //@line 1175 "dictionary.cpp"
                    var $64=HEAPU8[($type90)]; //@line 1175 "dictionary.cpp"
                    var $conv91=((($64))&255); //@line 1175 "dictionary.cpp"
                    var $cmp92=((($conv91))|0)!=2; //@line 1175 "dictionary.cpp"
                    if ($cmp92) { __label__ = 30;; } else { __label__ = 29;; } //@line 1175 "dictionary.cpp"
                    if (__label__ == 29) {
  
                      var $65=$p; //@line 1175 "dictionary.cpp"
                      var $arrayidx94=(($65+1)&4294967295); //@line 1175 "dictionary.cpp"
                      var $66=HEAPU8[($arrayidx94)]; //@line 1175 "dictionary.cpp"
                      var $idxprom95=((($66))&255); //@line 1175 "dictionary.cpp"
                      var $arrayidx96=((_phoneme_tab+$idxprom95*4)&4294967295); //@line 1175 "dictionary.cpp"
                      var $67=HEAP32[(($arrayidx96)>>2)]; //@line 1175 "dictionary.cpp"
                      var $phflags97=(($67+4)&4294967295); //@line 1175 "dictionary.cpp"
                      var $68=HEAP32[(($phflags97)>>2)]; //@line 1175 "dictionary.cpp"
                      var $and98=($68) & 2097152; //@line 1175 "dictionary.cpp"
                      var $tobool99=((($and98))|0)!=0; //@line 1175 "dictionary.cpp"
                      if (!($tobool99)) { __label__ = 31;break $land_lhs_true86$$if_end102$42; } //@line 1175 "dictionary.cpp"
                    }
  
                    var $69=$weight; //@line 1178 "dictionary.cpp"
                    var $inc101=((($69)+1)&4294967295); //@line 1178 "dictionary.cpp"
                    $weight=$inc101; //@line 1178 "dictionary.cpp"
                    ; //@line 1179 "dictionary.cpp"
                  }
                } while(0);
  
                var $70=$weight; //@line 1180 "dictionary.cpp"
                var $conv103=((($70)) & 255); //@line 1180 "dictionary.cpp"
                var $71=$ix; //@line 1180 "dictionary.cpp"
                var $arrayidx104=(($syllable_weight+$71)&4294967295); //@line 1180 "dictionary.cpp"
                HEAP8[($arrayidx104)]=$conv103; //@line 1180 "dictionary.cpp"
                var $72=$ix; //@line 1181 "dictionary.cpp"
                var $inc105=((($72)+1)&4294967295); //@line 1181 "dictionary.cpp"
                $ix=$inc105; //@line 1181 "dictionary.cpp"
                ; //@line 1182 "dictionary.cpp"
              }
            } while(0);
  
            var $73=$p; //@line 1156 "dictionary.cpp"
            var $incdec_ptr108=(($73+1)&4294967295); //@line 1156 "dictionary.cpp"
            $p=$incdec_ptr108; //@line 1156 "dictionary.cpp"
            var $74=$p; //@line 1156 "dictionary.cpp"
            var $75=HEAPU8[($74)]; //@line 1156 "dictionary.cpp"
            var $conv43=((($75))&255); //@line 1156 "dictionary.cpp"
            var $cmp44=((($conv43))|0)!=0; //@line 1156 "dictionary.cpp"
            if ($cmp44) { __label__ = 18;continue $for_body45$27; } else { __label__ = 33;break $for_body45$$for_end109$26; } //@line 1156 "dictionary.cpp"
          }
        }
      } while(0);
  
      var $76=$tr_addr; //@line 1185 "dictionary.cpp"
      var $langopts110=(($76)&4294967295); //@line 1185 "dictionary.cpp"
      var $stress_rule=(($langopts110+8)&4294967295); //@line 1185 "dictionary.cpp"
      var $77=HEAP32[(($stress_rule)>>2)]; //@line 1185 "dictionary.cpp"
      if ($77 == 8) {
        __label__ = 34;;
      }
      else if ($77 == 1) {
        __label__ = 36;;
      }
      else if ($77 == 10) {
        __label__ = 41;;
      }
      else if ($77 == 2) {
        __label__ = 44;;
      }
      else if ($77 == 3) {
        __label__ = 68;;
      }
      else if ($77 == 4) {
        __label__ = 75;;
      }
      else if ($77 == 5) {
        __label__ = 81;;
      }
      else if ($77 == 6) {
        __label__ = 89;;
      }
      else if ($77 == 7) {
        __label__ = 101;;
      }
      else if ($77 == 9) {
        __label__ = 108;;
      }
      else {
      __label__ = 112;;
      }
      
      $sw_epilog$$sw_bb$$sw_bb120$$sw_bb130$$sw_bb139$$sw_bb228$$sw_bb242$$sw_bb254$$sw_bb283$$sw_bb318$$sw_bb336$50: do { 
        if (__label__ == 34) {
  
          var $arrayidx111=(($syllable_weight+1)&4294967295); //@line 1189 "dictionary.cpp"
          var $78=HEAP8[($arrayidx111)]; //@line 1189 "dictionary.cpp"
          var $conv112=(tempInt=(($78)),(tempInt>=128?tempInt-256:tempInt)); //@line 1189 "dictionary.cpp"
          var $cmp113=((($conv112))|0) > 0; //@line 1189 "dictionary.cpp"
          if ($cmp113) { __label__ = 112;break $sw_epilog$$sw_bb$$sw_bb120$$sw_bb130$$sw_bb139$$sw_bb228$$sw_bb242$$sw_bb254$$sw_bb283$$sw_bb318$$sw_bb336$50; } //@line 1189 "dictionary.cpp"
  
          var $arrayidx115=(($syllable_weight+2)&4294967295); //@line 1189 "dictionary.cpp"
          var $79=HEAP8[($arrayidx115)]; //@line 1189 "dictionary.cpp"
          var $conv116=(tempInt=(($79)),(tempInt>=128?tempInt-256:tempInt)); //@line 1189 "dictionary.cpp"
          var $cmp117=((($conv116))|0)==0; //@line 1189 "dictionary.cpp"
          if ($cmp117) { __label__ = 112;break $sw_epilog$$sw_bb$$sw_bb120$$sw_bb130$$sw_bb139$$sw_bb228$$sw_bb242$$sw_bb254$$sw_bb283$$sw_bb318$$sw_bb336$50; } else { __label__ = 36;break $sw_epilog$$sw_bb$$sw_bb120$$sw_bb130$$sw_bb139$$sw_bb228$$sw_bb242$$sw_bb254$$sw_bb283$$sw_bb318$$sw_bb336$50; } //@line 1189 "dictionary.cpp"
        }
        else if (__label__ == 41) {
  
          var $84=HEAP32[(($stressed_syllable)>>2)]; //@line 1206 "dictionary.cpp"
          var $cmp131=((($84))|0)==0; //@line 1206 "dictionary.cpp"
          if (!($cmp131)) { __label__ = 112;break $sw_epilog$$sw_bb$$sw_bb120$$sw_bb130$$sw_bb139$$sw_bb228$$sw_bb242$$sw_bb254$$sw_bb283$$sw_bb318$$sw_bb336$50; } //@line 1206 "dictionary.cpp"
  
          var $85=HEAP32[(($vowel_count)>>2)]; //@line 1208 "dictionary.cpp"
          var $cmp133=((($85))|0) < 4; //@line 1208 "dictionary.cpp"
          if (!($cmp133)) { __label__ = 44;break $sw_epilog$$sw_bb$$sw_bb120$$sw_bb130$$sw_bb139$$sw_bb228$$sw_bb242$$sw_bb254$$sw_bb283$$sw_bb318$$sw_bb336$50; } //@line 1208 "dictionary.cpp"
  
          var $86=HEAP32[(($vowel_count)>>2)]; //@line 1210 "dictionary.cpp"
          var $sub135=((($86)-1)&4294967295); //@line 1210 "dictionary.cpp"
          var $arrayidx136=(($vowel_stress+$sub135)&4294967295); //@line 1210 "dictionary.cpp"
          HEAP8[($arrayidx136)]=4; //@line 1210 "dictionary.cpp"
          $max_stress=4; //@line 1211 "dictionary.cpp"
          __label__ = 114;break $sw_epilog$$sw_bb$$sw_bb120$$sw_bb130$$sw_bb139$$sw_bb228$$sw_bb242$$sw_bb254$$sw_bb283$$sw_bb318$$sw_bb336$50; //@line 1212 "dictionary.cpp"
        }
        else if (__label__ == 68) {
  
          var $129=HEAP32[(($stressed_syllable)>>2)]; //@line 1289 "dictionary.cpp"
          var $cmp229=((($129))|0)==0; //@line 1289 "dictionary.cpp"
          if (!($cmp229)) { __label__ = 112;break $sw_epilog$$sw_bb$$sw_bb120$$sw_bb130$$sw_bb139$$sw_bb228$$sw_bb242$$sw_bb254$$sw_bb283$$sw_bb318$$sw_bb336$50; } //@line 1289 "dictionary.cpp"
  
          var $130=HEAP32[(($vowel_count)>>2)]; //@line 1292 "dictionary.cpp"
          var $sub231=((($130)-1)&4294967295); //@line 1292 "dictionary.cpp"
          HEAP32[(($stressed_syllable)>>2)]=$sub231; //@line 1292 "dictionary.cpp"
          __lastLabel__ = 69; ; //@line 1294 "dictionary.cpp"
          while(1) { 
  
            var $131=__lastLabel__ == 73 ? $dec239 : ($sub231); //@line 1294 "dictionary.cpp"
            var $cmp232=((($131))|0) > 0; //@line 1294 "dictionary.cpp"
            if (!($cmp232)) { __label__ = 74;break ; } //@line 1294 "dictionary.cpp"
  
            var $132=HEAP32[(($stressed_syllable)>>2)]; //@line 1297 "dictionary.cpp"
            var $arrayidx233=(($vowel_stress+$132)&4294967295); //@line 1297 "dictionary.cpp"
            var $133=HEAP8[($arrayidx233)]; //@line 1297 "dictionary.cpp"
            var $conv234=(tempInt=(($133)),(tempInt>=128?tempInt-256:tempInt)); //@line 1297 "dictionary.cpp"
            var $cmp235=((($conv234))|0) < 0; //@line 1297 "dictionary.cpp"
            var $134=HEAP32[(($stressed_syllable)>>2)]; //@line 1299 "dictionary.cpp"
            if ($cmp235) { __label__ = 72;break ; } //@line 1297 "dictionary.cpp"
  
            var $dec239=((($134)-1)&4294967295); //@line 1303 "dictionary.cpp"
            HEAP32[(($stressed_syllable)>>2)]=$dec239; //@line 1303 "dictionary.cpp"
            __lastLabel__ = 73; __label__ = 70;continue ; //@line 1304 "dictionary.cpp"
          }
          if (__label__ == 72) {
  
            var $arrayidx237=(($vowel_stress+$134)&4294967295); //@line 1299 "dictionary.cpp"
            HEAP8[($arrayidx237)]=4; //@line 1299 "dictionary.cpp"
            ; //@line 1300 "dictionary.cpp"
          }
  
          $max_stress=4; //@line 1305 "dictionary.cpp"
          __label__ = 114;break $sw_epilog$$sw_bb$$sw_bb120$$sw_bb130$$sw_bb139$$sw_bb228$$sw_bb242$$sw_bb254$$sw_bb283$$sw_bb318$$sw_bb336$50; //@line 1306 "dictionary.cpp"
        }
        else if (__label__ == 75) {
  
          var $135=HEAP32[(($stressed_syllable)>>2)]; //@line 1310 "dictionary.cpp"
          var $cmp243=((($135))|0)==0; //@line 1310 "dictionary.cpp"
          if (!($cmp243)) { __label__ = 112;break $sw_epilog$$sw_bb$$sw_bb120$$sw_bb130$$sw_bb139$$sw_bb228$$sw_bb242$$sw_bb254$$sw_bb283$$sw_bb318$$sw_bb336$50; } //@line 1310 "dictionary.cpp"
  
          var $136=HEAP32[(($vowel_count)>>2)]; //@line 1312 "dictionary.cpp"
          var $sub245=((($136)-3)&4294967295); //@line 1312 "dictionary.cpp"
          HEAP32[(($stressed_syllable)>>2)]=$sub245; //@line 1312 "dictionary.cpp"
          var $137=HEAP32[(($stressed_syllable)>>2)]; //@line 1313 "dictionary.cpp"
          var $cmp246=((($137))|0) < 1; //@line 1313 "dictionary.cpp"
          if ($cmp246) { __label__ = 77;; } else { __label__ = 78;; } //@line 1313 "dictionary.cpp"
          if (__label__ == 77) {
  
            HEAP32[(($stressed_syllable)>>2)]=1; //@line 1314 "dictionary.cpp"
            ; //@line 1314 "dictionary.cpp"
          }
  
          var $138=$max_stress; //@line 1316 "dictionary.cpp"
          var $cmp249=((($138))|0)==0; //@line 1316 "dictionary.cpp"
          if ($cmp249) { __label__ = 79;; } else { __label__ = 80;; } //@line 1316 "dictionary.cpp"
          if (__label__ == 79) {
  
            var $139=HEAP32[(($stressed_syllable)>>2)]; //@line 1318 "dictionary.cpp"
            var $arrayidx251=(($vowel_stress+$139)&4294967295); //@line 1318 "dictionary.cpp"
            HEAP8[($arrayidx251)]=4; //@line 1318 "dictionary.cpp"
            ; //@line 1319 "dictionary.cpp"
          }
  
          $max_stress=4; //@line 1320 "dictionary.cpp"
          __label__ = 114;break $sw_epilog$$sw_bb$$sw_bb120$$sw_bb130$$sw_bb139$$sw_bb228$$sw_bb242$$sw_bb254$$sw_bb283$$sw_bb318$$sw_bb336$50; //@line 1321 "dictionary.cpp"
        }
        else if (__label__ == 81) {
  
          var $140=HEAP32[(($stressed_syllable)>>2)]; //@line 1326 "dictionary.cpp"
          var $cmp255=((($140))|0)==0; //@line 1326 "dictionary.cpp"
          if (!($cmp255)) { __label__ = 112;break $sw_epilog$$sw_bb$$sw_bb120$$sw_bb130$$sw_bb139$$sw_bb228$$sw_bb242$$sw_bb254$$sw_bb283$$sw_bb318$$sw_bb336$50; } //@line 1326 "dictionary.cpp"
  
          var $141=HEAP32[(($vowel_count)>>2)]; //@line 1333 "dictionary.cpp"
          var $sub257=((($141)-3)&4294967295); //@line 1333 "dictionary.cpp"
          HEAP32[(($stressed_syllable)>>2)]=$sub257; //@line 1333 "dictionary.cpp"
          var $142=HEAP32[(($vowel_count)>>2)]; //@line 1334 "dictionary.cpp"
          var $cmp258=((($142))|0) < 16; //@line 1334 "dictionary.cpp"
          if ($cmp258) { __label__ = 83;; } else { __label__ = 88;; } //@line 1334 "dictionary.cpp"
          if (__label__ == 83) {
  
            var $143=$final_ph; //@line 1336 "dictionary.cpp"
            var $arrayidx260=((_phoneme_tab+$143*4)&4294967295); //@line 1336 "dictionary.cpp"
            var $144=HEAP32[(($arrayidx260)>>2)]; //@line 1336 "dictionary.cpp"
            var $type261=(($144+11)&4294967295); //@line 1336 "dictionary.cpp"
            var $145=HEAPU8[($type261)]; //@line 1336 "dictionary.cpp"
            var $conv262=((($145))&255); //@line 1336 "dictionary.cpp"
            var $cmp263=((($conv262))|0)==2; //@line 1336 "dictionary.cpp"
            if ($cmp263) { __label__ = 84;; } else { __label__ = 85;; } //@line 1336 "dictionary.cpp"
            if (__label__ == 84) {
  
              var $146=HEAP32[(($vowel_count)>>2)]; //@line 1337 "dictionary.cpp"
              var $arrayidx265=((__ZZ13SetWordStressP10TranslatorPcPjiiE10guess_ru_v+$146)&4294967295); //@line 1337 "dictionary.cpp"
              var $147=HEAP8[($arrayidx265)]; //@line 1337 "dictionary.cpp"
              var $conv266=(tempInt=(($147)),(tempInt>=128?tempInt-256:tempInt)); //@line 1337 "dictionary.cpp"
              HEAP32[(($stressed_syllable)>>2)]=$conv266; //@line 1337 "dictionary.cpp"
              ; //@line 1337 "dictionary.cpp"
            }
            else if (__label__ == 85) {
  
              var $148=$final_ph; //@line 1339 "dictionary.cpp"
              var $arrayidx268=((_phoneme_tab+$148*4)&4294967295); //@line 1339 "dictionary.cpp"
              var $149=HEAP32[(($arrayidx268)>>2)]; //@line 1339 "dictionary.cpp"
              var $type269=(($149+11)&4294967295); //@line 1339 "dictionary.cpp"
              var $150=HEAPU8[($type269)]; //@line 1339 "dictionary.cpp"
              var $conv270=((($150))&255); //@line 1339 "dictionary.cpp"
              var $cmp271=((($conv270))|0)==4; //@line 1339 "dictionary.cpp"
              var $151=HEAP32[(($vowel_count)>>2)]; //@line 1340 "dictionary.cpp"
              if ($cmp271) { __label__ = 86;; } else { __label__ = 87;; } //@line 1339 "dictionary.cpp"
              if (__label__ == 86) {
  
                var $arrayidx273=((__ZZ13SetWordStressP10TranslatorPcPjiiE10guess_ru_t+$151)&4294967295); //@line 1340 "dictionary.cpp"
                var $152=HEAP8[($arrayidx273)]; //@line 1340 "dictionary.cpp"
                var $conv274=(tempInt=(($152)),(tempInt>=128?tempInt-256:tempInt)); //@line 1340 "dictionary.cpp"
                HEAP32[(($stressed_syllable)>>2)]=$conv274; //@line 1340 "dictionary.cpp"
                ; //@line 1340 "dictionary.cpp"
              }
              else if (__label__ == 87) {
  
                var $arrayidx276=((__ZZ13SetWordStressP10TranslatorPcPjiiE8guess_ru+$151)&4294967295); //@line 1342 "dictionary.cpp"
                var $153=HEAP8[($arrayidx276)]; //@line 1342 "dictionary.cpp"
                var $conv277=(tempInt=(($153)),(tempInt>=128?tempInt-256:tempInt)); //@line 1342 "dictionary.cpp"
                HEAP32[(($stressed_syllable)>>2)]=$conv277; //@line 1342 "dictionary.cpp"
                ;
              }
            }
          }
  
          var $154=HEAP32[(($stressed_syllable)>>2)]; //@line 1344 "dictionary.cpp"
          var $arrayidx281=(($vowel_stress+$154)&4294967295); //@line 1344 "dictionary.cpp"
          HEAP8[($arrayidx281)]=4; //@line 1344 "dictionary.cpp"
          $max_stress=4; //@line 1345 "dictionary.cpp"
          __label__ = 114;break $sw_epilog$$sw_bb$$sw_bb120$$sw_bb130$$sw_bb139$$sw_bb228$$sw_bb242$$sw_bb254$$sw_bb283$$sw_bb318$$sw_bb336$50; //@line 1346 "dictionary.cpp"
        }
        else if (__label__ == 89) {
  
          var $155=HEAP32[(($stressed_syllable)>>2)]; //@line 1350 "dictionary.cpp"
          var $cmp284=((($155))|0)==0; //@line 1350 "dictionary.cpp"
          if (!($cmp284)) { __label__ = 112;break $sw_epilog$$sw_bb$$sw_bb120$$sw_bb130$$sw_bb139$$sw_bb228$$sw_bb242$$sw_bb254$$sw_bb283$$sw_bb318$$sw_bb336$50; } //@line 1350 "dictionary.cpp"
  
          $max_weight=-1; //@line 1353 "dictionary.cpp"
          $ix=1; //@line 1357 "dictionary.cpp"
          var $156=$ix; //@line 1357 "dictionary.cpp"
          var $157=HEAP32[(($vowel_count)>>2)]; //@line 1357 "dictionary.cpp"
          var $sub28714=((($157)-1)&4294967295); //@line 1357 "dictionary.cpp"
          var $cmp28815=((($156))|0) < ((($sub28714))|0); //@line 1357 "dictionary.cpp"
          if ($cmp28815) { __label__ = 91;; } else { __label__ = 95;; } //@line 1357 "dictionary.cpp"
          $for_body289$$for_end302$86: do { 
            if (__label__ == 91) {
              $for_body289$87: while(1) { 
  
                var $158=$ix; //@line 1359 "dictionary.cpp"
                var $arrayidx290=(($vowel_stress+$158)&4294967295); //@line 1359 "dictionary.cpp"
                var $159=HEAP8[($arrayidx290)]; //@line 1359 "dictionary.cpp"
                var $conv291=(tempInt=(($159)),(tempInt>=128?tempInt-256:tempInt)); //@line 1359 "dictionary.cpp"
                var $cmp292=((($conv291))|0) < 0; //@line 1359 "dictionary.cpp"
                if ($cmp292) { __label__ = 92;; } else { __label__ = 94;; } //@line 1359 "dictionary.cpp"
                $if_then293$$for_inc300$89: do { 
                  if (__label__ == 92) {
  
                    var $160=$ix; //@line 1361 "dictionary.cpp"
                    var $arrayidx294=(($syllable_weight+$160)&4294967295); //@line 1361 "dictionary.cpp"
                    var $161=HEAP8[($arrayidx294)]; //@line 1361 "dictionary.cpp"
                    var $conv295=(tempInt=(($161)),(tempInt>=128?tempInt-256:tempInt)); //@line 1361 "dictionary.cpp"
                    $wt=$conv295; //@line 1361 "dictionary.cpp"
                    var $162=$max_weight; //@line 1361 "dictionary.cpp"
                    var $cmp296=((($conv295))|0) >= ((($162))|0); //@line 1361 "dictionary.cpp"
                    if (!($cmp296)) { __label__ = 94;break $if_then293$$for_inc300$89; } //@line 1361 "dictionary.cpp"
  
                    var $163=$wt; //@line 1363 "dictionary.cpp"
                    $max_weight=$163; //@line 1363 "dictionary.cpp"
                    var $164=HEAP32[(($stressed_syllable)>>2)]; //@line 1364 "dictionary.cpp"
                    $prev_stressed=$164; //@line 1364 "dictionary.cpp"
                    var $165=$ix; //@line 1365 "dictionary.cpp"
                    HEAP32[(($stressed_syllable)>>2)]=$165; //@line 1365 "dictionary.cpp"
                    ; //@line 1366 "dictionary.cpp"
                  }
                } while(0);
  
                var $166=$ix; //@line 1357 "dictionary.cpp"
                var $inc301=((($166)+1)&4294967295); //@line 1357 "dictionary.cpp"
                $ix=$inc301; //@line 1357 "dictionary.cpp"
                var $167=$ix; //@line 1357 "dictionary.cpp"
                var $168=HEAP32[(($vowel_count)>>2)]; //@line 1357 "dictionary.cpp"
                var $sub287=((($168)-1)&4294967295); //@line 1357 "dictionary.cpp"
                var $cmp288=((($167))|0) < ((($sub287))|0); //@line 1357 "dictionary.cpp"
                if ($cmp288) { __label__ = 91;continue $for_body289$87; } else { __label__ = 95;break $for_body289$$for_end302$86; } //@line 1357 "dictionary.cpp"
              }
            }
          } while(0);
  
          var $169=HEAP32[(($vowel_count)>>2)]; //@line 1370 "dictionary.cpp"
          var $sub303=((($169)-1)&4294967295); //@line 1370 "dictionary.cpp"
          var $arrayidx304=(($syllable_weight+$sub303)&4294967295); //@line 1370 "dictionary.cpp"
          var $170=HEAP8[($arrayidx304)]; //@line 1370 "dictionary.cpp"
          var $conv305=(tempInt=(($170)),(tempInt>=128?tempInt-256:tempInt)); //@line 1370 "dictionary.cpp"
          var $cmp306=((($conv305))|0)==2; //@line 1370 "dictionary.cpp"
          var $171=$max_weight; //@line 1370 "dictionary.cpp"
          if ($cmp306) { __label__ = 96;; } else { __label__ = 98;; } //@line 1370 "dictionary.cpp"
          $land_lhs_true307$$if_else311$94: do { 
            if (__label__ == 96) {
  
              var $cmp308=((($171))|0) < 2; //@line 1370 "dictionary.cpp"
              if (!($cmp308)) { __label__ = 100;break $land_lhs_true307$$if_else311$94; } //@line 1370 "dictionary.cpp"
  
              var $172=HEAP32[(($vowel_count)>>2)]; //@line 1373 "dictionary.cpp"
              var $sub310=((($172)-1)&4294967295); //@line 1373 "dictionary.cpp"
              HEAP32[(($stressed_syllable)>>2)]=$sub310; //@line 1373 "dictionary.cpp"
              ; //@line 1374 "dictionary.cpp"
            }
            else if (__label__ == 98) {
  
              var $cmp312=((($171))|0) <= 0; //@line 1376 "dictionary.cpp"
              if (!($cmp312)) { __label__ = 100;break $land_lhs_true307$$if_else311$94; } //@line 1376 "dictionary.cpp"
  
              HEAP32[(($stressed_syllable)>>2)]=1; //@line 1379 "dictionary.cpp"
              ; //@line 1380 "dictionary.cpp"
            }
          } while(0);
  
          var $173=HEAP32[(($stressed_syllable)>>2)]; //@line 1382 "dictionary.cpp"
          var $arrayidx316=(($vowel_stress+$173)&4294967295); //@line 1382 "dictionary.cpp"
          HEAP8[($arrayidx316)]=4; //@line 1382 "dictionary.cpp"
          $max_stress=4; //@line 1383 "dictionary.cpp"
          __label__ = 114;break $sw_epilog$$sw_bb$$sw_bb120$$sw_bb130$$sw_bb139$$sw_bb228$$sw_bb242$$sw_bb254$$sw_bb283$$sw_bb318$$sw_bb336$50; //@line 1384 "dictionary.cpp"
        }
        else if (__label__ == 101) {
  
          var $174=HEAP32[(($stressed_syllable)>>2)]; //@line 1388 "dictionary.cpp"
          var $cmp319=((($174))|0)==0; //@line 1388 "dictionary.cpp"
          if (!($cmp319)) { __label__ = 112;break $sw_epilog$$sw_bb$$sw_bb120$$sw_bb130$$sw_bb139$$sw_bb228$$sw_bb242$$sw_bb254$$sw_bb283$$sw_bb318$$sw_bb336$50; } //@line 1388 "dictionary.cpp"
  
          var $175=HEAP32[(($vowel_count)>>2)]; //@line 1390 "dictionary.cpp"
          var $sub321=((($175)-1)&4294967295); //@line 1390 "dictionary.cpp"
          HEAP32[(($stressed_syllable)>>2)]=$sub321; //@line 1390 "dictionary.cpp"
          $ix=1; //@line 1391 "dictionary.cpp"
          ; //@line 1391 "dictionary.cpp"
          while(1) { 
  
            var $176=$ix; //@line 1391 "dictionary.cpp"
            var $177=HEAP32[(($vowel_count)>>2)]; //@line 1391 "dictionary.cpp"
            var $cmp323=((($176))|0) < ((($177))|0); //@line 1391 "dictionary.cpp"
            if (!($cmp323)) { __label__ = 107;break ; } //@line 1391 "dictionary.cpp"
  
            var $178=$ix; //@line 1393 "dictionary.cpp"
            var $arrayidx325=(($vowel_stress+$178)&4294967295); //@line 1393 "dictionary.cpp"
            var $179=HEAP8[($arrayidx325)]; //@line 1393 "dictionary.cpp"
            var $conv326=(tempInt=(($179)),(tempInt>=128?tempInt-256:tempInt)); //@line 1393 "dictionary.cpp"
            var $cmp327=((($conv326))|0)==1; //@line 1393 "dictionary.cpp"
            var $180=$ix; //@line 1395 "dictionary.cpp"
            if ($cmp327) { __label__ = 105;break ; } //@line 1393 "dictionary.cpp"
  
            var $inc332=((($180)+1)&4294967295); //@line 1391 "dictionary.cpp"
            $ix=$inc332; //@line 1391 "dictionary.cpp"
            __label__ = 103;continue ; //@line 1391 "dictionary.cpp"
          }
          if (__label__ == 105) {
  
            var $sub329=((($180)-1)&4294967295); //@line 1395 "dictionary.cpp"
            HEAP32[(($stressed_syllable)>>2)]=$sub329; //@line 1395 "dictionary.cpp"
            ; //@line 1396 "dictionary.cpp"
          }
  
          var $181=HEAP32[(($stressed_syllable)>>2)]; //@line 1399 "dictionary.cpp"
          var $arrayidx334=(($vowel_stress+$181)&4294967295); //@line 1399 "dictionary.cpp"
          HEAP8[($arrayidx334)]=4; //@line 1399 "dictionary.cpp"
          $max_stress=4; //@line 1400 "dictionary.cpp"
          __label__ = 114;break $sw_epilog$$sw_bb$$sw_bb120$$sw_bb130$$sw_bb139$$sw_bb228$$sw_bb242$$sw_bb254$$sw_bb283$$sw_bb318$$sw_bb336$50; //@line 1401 "dictionary.cpp"
        }
        else if (__label__ == 108) {
  
          $ix=1; //@line 1405 "dictionary.cpp"
          var $182=$ix; //@line 1405 "dictionary.cpp"
          var $183=HEAP32[(($vowel_count)>>2)]; //@line 1405 "dictionary.cpp"
          var $cmp33813=((($182))|0) < ((($183))|0); //@line 1405 "dictionary.cpp"
          if (!($cmp33813)) { __label__ = 112;break $sw_epilog$$sw_bb$$sw_bb120$$sw_bb130$$sw_bb139$$sw_bb228$$sw_bb242$$sw_bb254$$sw_bb283$$sw_bb318$$sw_bb336$50; } //@line 1405 "dictionary.cpp"
          while(1) { 
  
            var $184=$ix; //@line 1407 "dictionary.cpp"
            var $arrayidx340=(($vowel_stress+$184)&4294967295); //@line 1407 "dictionary.cpp"
            var $185=HEAP8[($arrayidx340)]; //@line 1407 "dictionary.cpp"
            var $conv341=(tempInt=(($185)),(tempInt>=128?tempInt-256:tempInt)); //@line 1407 "dictionary.cpp"
            var $cmp342=((($conv341))|0) < 0; //@line 1407 "dictionary.cpp"
            if ($cmp342) { __label__ = 110;; } else { __label__ = 111;; } //@line 1407 "dictionary.cpp"
            if (__label__ == 110) {
  
              var $186=$ix; //@line 1408 "dictionary.cpp"
              var $arrayidx344=(($vowel_stress+$186)&4294967295); //@line 1408 "dictionary.cpp"
              HEAP8[($arrayidx344)]=4; //@line 1408 "dictionary.cpp"
              ; //@line 1408 "dictionary.cpp"
            }
  
            var $187=$ix; //@line 1405 "dictionary.cpp"
            var $inc347=((($187)+1)&4294967295); //@line 1405 "dictionary.cpp"
            $ix=$inc347; //@line 1405 "dictionary.cpp"
            var $188=$ix; //@line 1405 "dictionary.cpp"
            var $189=HEAP32[(($vowel_count)>>2)]; //@line 1405 "dictionary.cpp"
            var $cmp338=((($188))|0) < ((($189))|0); //@line 1405 "dictionary.cpp"
            if ($cmp338) { __label__ = 109;continue ; } else { __label__ = 112;break $sw_epilog$$sw_bb$$sw_bb120$$sw_bb130$$sw_bb139$$sw_bb228$$sw_bb242$$sw_bb254$$sw_bb283$$sw_bb318$$sw_bb336$50; } //@line 1405 "dictionary.cpp"
          }
        }
      } while(0);
      $sw_epilog$$sw_bb120$$if_else351$$sw_bb139$115: do { 
        if (__label__ == 36) {
  
          var $80=HEAP32[(($stressed_syllable)>>2)]; //@line 1194 "dictionary.cpp"
          var $cmp121=((($80))|0)==0; //@line 1194 "dictionary.cpp"
          if (!($cmp121)) { __label__ = 112;break $sw_epilog$$sw_bb120$$if_else351$$sw_bb139$115; } //@line 1194 "dictionary.cpp"
  
          var $81=HEAP32[(($vowel_count)>>2)]; //@line 1194 "dictionary.cpp"
          var $cmp123=((($81))|0) > 2; //@line 1194 "dictionary.cpp"
          if (!($cmp123)) { __label__ = 112;break $sw_epilog$$sw_bb120$$if_else351$$sw_bb139$115; } //@line 1194 "dictionary.cpp"
  
          HEAP32[(($stressed_syllable)>>2)]=2; //@line 1196 "dictionary.cpp"
          var $82=$max_stress; //@line 1197 "dictionary.cpp"
          var $cmp125=((($82))|0)==0; //@line 1197 "dictionary.cpp"
          if ($cmp125) { __label__ = 39;; } else { __label__ = 40;; } //@line 1197 "dictionary.cpp"
          if (__label__ == 39) {
  
            var $83=HEAP32[(($stressed_syllable)>>2)]; //@line 1199 "dictionary.cpp"
            var $arrayidx127=(($vowel_stress+$83)&4294967295); //@line 1199 "dictionary.cpp"
            HEAP8[($arrayidx127)]=4; //@line 1199 "dictionary.cpp"
            ; //@line 1200 "dictionary.cpp"
          }
  
          $max_stress=4; //@line 1201 "dictionary.cpp"
          __label__ = 114;break $sw_epilog$$sw_bb120$$if_else351$$sw_bb139$115; //@line 1202 "dictionary.cpp"
        }
        else if (__label__ == 44) {
  
          var $_pr5=HEAP32[(($stressed_syllable)>>2)]; //@line 1219 "dictionary.cpp"
          var $cmp140=((($_pr5))|0)==0; //@line 1219 "dictionary.cpp"
          if (!($cmp140)) { __label__ = 112;break $sw_epilog$$sw_bb120$$if_else351$$sw_bb139$115; } //@line 1219 "dictionary.cpp"
  
          $max_stress=4; //@line 1222 "dictionary.cpp"
          var $87=HEAP32[(($vowel_count)>>2)]; //@line 1224 "dictionary.cpp"
          var $cmp142=((($87))|0) > 2; //@line 1224 "dictionary.cpp"
          if ($cmp142) { __label__ = 46;; } else { __label__ = 63;; } //@line 1224 "dictionary.cpp"
          $if_then143$$if_else208$124: do { 
            if (__label__ == 46) {
  
              var $88=HEAP32[(($vowel_count)>>2)]; //@line 1226 "dictionary.cpp"
              var $sub144=((($88)-2)&4294967295); //@line 1226 "dictionary.cpp"
              HEAP32[(($stressed_syllable)>>2)]=$sub144; //@line 1226 "dictionary.cpp"
              var $89=$stressflags; //@line 1228 "dictionary.cpp"
              var $and145=($89) & 768; //@line 1228 "dictionary.cpp"
              var $tobool146=((($and145))|0)!=0; //@line 1228 "dictionary.cpp"
              if ($tobool146) { __label__ = 47;; } else { __label__ = 55;; } //@line 1228 "dictionary.cpp"
              $if_then147$$if_end179$126: do { 
                if (__label__ == 47) {
  
                  var $90=$final_ph; //@line 1231 "dictionary.cpp"
                  var $arrayidx148=((_phoneme_tab+$90*4)&4294967295); //@line 1231 "dictionary.cpp"
                  var $91=HEAP32[(($arrayidx148)>>2)]; //@line 1231 "dictionary.cpp"
                  var $type149=(($91+11)&4294967295); //@line 1231 "dictionary.cpp"
                  var $92=HEAPU8[($type149)]; //@line 1231 "dictionary.cpp"
                  var $conv150=((($92))&255); //@line 1231 "dictionary.cpp"
                  var $cmp151=((($conv150))|0)!=2; //@line 1231 "dictionary.cpp"
                  if (!($cmp151)) { __label__ = 55;break $if_then147$$if_end179$126; } //@line 1231 "dictionary.cpp"
  
                  var $93=$stressflags; //@line 1233 "dictionary.cpp"
                  var $and153=($93) & 256; //@line 1233 "dictionary.cpp"
                  var $tobool154=((($and153))|0)!=0; //@line 1233 "dictionary.cpp"
                  if ($tobool154) { __label__ = 49;; } else { __label__ = 50;; } //@line 1233 "dictionary.cpp"
                  if (__label__ == 49) {
  
                    var $94=HEAP32[(($vowel_count)>>2)]; //@line 1235 "dictionary.cpp"
                    var $sub156=((($94)-1)&4294967295); //@line 1235 "dictionary.cpp"
                    HEAP32[(($stressed_syllable)>>2)]=$sub156; //@line 1235 "dictionary.cpp"
                    ; //@line 1236 "dictionary.cpp"
                  }
                  else if (__label__ == 50) {
  
                    var $95=$final_ph; //@line 1239 "dictionary.cpp"
                    var $arrayidx157=((_phoneme_tab+$95*4)&4294967295); //@line 1239 "dictionary.cpp"
                    var $96=HEAP32[(($arrayidx157)>>2)]; //@line 1239 "dictionary.cpp"
                    var $mnemonic=(($96)&4294967295); //@line 1239 "dictionary.cpp"
                    var $97=HEAP32[(($mnemonic)>>2)]; //@line 1239 "dictionary.cpp"
                    $mnem=$97; //@line 1239 "dictionary.cpp"
                    var $98=$final_ph2; //@line 1240 "dictionary.cpp"
                    var $arrayidx158=((_phoneme_tab+$98*4)&4294967295); //@line 1240 "dictionary.cpp"
                    var $99=HEAP32[(($arrayidx158)>>2)]; //@line 1240 "dictionary.cpp"
                    var $mnemonic159=(($99)&4294967295); //@line 1240 "dictionary.cpp"
                    var $100=HEAP32[(($mnemonic159)>>2)]; //@line 1240 "dictionary.cpp"
                    $mnem2=$100; //@line 1240 "dictionary.cpp"
                    var $101=$mnem; //@line 1242 "dictionary.cpp"
                    var $cmp160=((($101))|0)==115; //@line 1242 "dictionary.cpp"
                    if ($cmp160) { __label__ = 51;; } else { __label__ = 52;; } //@line 1242 "dictionary.cpp"
                    if (__label__ == 51) {
  
                      var $102=$mnem2; //@line 1242 "dictionary.cpp"
                      var $cmp162=((($102))|0)==110; //@line 1242 "dictionary.cpp"
                      if ($cmp162) { __label__ = 55;break $if_then147$$if_end179$126; } //@line 1242 "dictionary.cpp"
                    }
  
                    var $103=$mnem; //@line 1247 "dictionary.cpp"
                    var $cmp165=((($103))|0)!=110; //@line 1247 "dictionary.cpp"
                    var $104=$mnem; //@line 1247 "dictionary.cpp"
                    var $cmp167=((($104))|0)!=115; //@line 1247 "dictionary.cpp"
                    var $or_cond=($cmp165) & ($cmp167); //@line 1247 "dictionary.cpp"
                    if ($or_cond) { __label__ = 54;; } else { __label__ = 53;; } //@line 1247 "dictionary.cpp"
                    if (__label__ == 53) {
  
                      var $105=$final_ph2; //@line 1247 "dictionary.cpp"
                      var $arrayidx169=((_phoneme_tab+$105*4)&4294967295); //@line 1247 "dictionary.cpp"
                      var $106=HEAP32[(($arrayidx169)>>2)]; //@line 1247 "dictionary.cpp"
                      var $type170=(($106+11)&4294967295); //@line 1247 "dictionary.cpp"
                      var $107=HEAPU8[($type170)]; //@line 1247 "dictionary.cpp"
                      var $conv171=((($107))&255); //@line 1247 "dictionary.cpp"
                      var $cmp172=((($conv171))|0)!=2; //@line 1247 "dictionary.cpp"
                      if (!($cmp172)) { __label__ = 55;break $if_then147$$if_end179$126; } //@line 1247 "dictionary.cpp"
                    }
  
                    var $108=HEAP32[(($vowel_count)>>2)]; //@line 1249 "dictionary.cpp"
                    var $sub174=((($108)-1)&4294967295); //@line 1249 "dictionary.cpp"
                    HEAP32[(($stressed_syllable)>>2)]=$sub174; //@line 1249 "dictionary.cpp"
                    ; //@line 1250 "dictionary.cpp"
                  }
                }
              } while(0);
  
              var $109=$stressflags; //@line 1254 "dictionary.cpp"
              var $and180=($109) & 524288; //@line 1254 "dictionary.cpp"
              var $tobool181=((($and180))|0)!=0; //@line 1254 "dictionary.cpp"
              if ($tobool181) { __label__ = 56;; } else { __label__ = 58;; } //@line 1254 "dictionary.cpp"
              $if_then182$$if_end193$139: do { 
                if (__label__ == 56) {
  
                  var $110=HEAP32[(($vowel_count)>>2)]; //@line 1257 "dictionary.cpp"
                  var $sub183=((($110)-1)&4294967295); //@line 1257 "dictionary.cpp"
                  var $arrayidx184=(($vowel_length+$sub183)&4294967295); //@line 1257 "dictionary.cpp"
                  var $111=HEAP8[($arrayidx184)]; //@line 1257 "dictionary.cpp"
                  var $conv185=(tempInt=(($111)),(tempInt>=128?tempInt-256:tempInt)); //@line 1257 "dictionary.cpp"
                  var $112=HEAP32[(($vowel_count)>>2)]; //@line 1257 "dictionary.cpp"
                  var $sub186=((($112)-2)&4294967295); //@line 1257 "dictionary.cpp"
                  var $arrayidx187=(($vowel_length+$sub186)&4294967295); //@line 1257 "dictionary.cpp"
                  var $113=HEAP8[($arrayidx187)]; //@line 1257 "dictionary.cpp"
                  var $conv188=(tempInt=(($113)),(tempInt>=128?tempInt-256:tempInt)); //@line 1257 "dictionary.cpp"
                  var $cmp189=((($conv185))|0) > ((($conv188))|0); //@line 1257 "dictionary.cpp"
                  if (!($cmp189)) { __label__ = 58;break $if_then182$$if_end193$139; } //@line 1257 "dictionary.cpp"
  
                  var $114=HEAP32[(($vowel_count)>>2)]; //@line 1259 "dictionary.cpp"
                  var $sub191=((($114)-1)&4294967295); //@line 1259 "dictionary.cpp"
                  HEAP32[(($stressed_syllable)>>2)]=$sub191; //@line 1259 "dictionary.cpp"
                  ; //@line 1260 "dictionary.cpp"
                }
              } while(0);
  
              var $115=HEAP32[(($stressed_syllable)>>2)]; //@line 1263 "dictionary.cpp"
              var $arrayidx194=(($vowel_stress+$115)&4294967295); //@line 1263 "dictionary.cpp"
              var $116=HEAP8[($arrayidx194)]; //@line 1263 "dictionary.cpp"
              var $conv195=(tempInt=(($116)),(tempInt>=128?tempInt-256:tempInt)); //@line 1263 "dictionary.cpp"
              var $cmp196=((($conv195))|0)==0; //@line 1263 "dictionary.cpp"
              if ($cmp196) { __lastLabel__ = 58; __label__ = 60;; } else { __lastLabel__ = 58; __label__ = 59;; } //@line 1263 "dictionary.cpp"
              if (__label__ == 59) {
  
                var $117=HEAP32[(($stressed_syllable)>>2)]; //@line 1263 "dictionary.cpp"
                var $arrayidx198=(($vowel_stress+$117)&4294967295); //@line 1263 "dictionary.cpp"
                var $118=HEAP8[($arrayidx198)]; //@line 1263 "dictionary.cpp"
                var $conv199=(tempInt=(($118)),(tempInt>=128?tempInt-256:tempInt)); //@line 1263 "dictionary.cpp"
                var $cmp200=((($conv199))|0)==1; //@line 1263 "dictionary.cpp"
                if ($cmp200) { __lastLabel__ = 59; ; } else { __lastLabel__ = 59; __label__ = 64;break $if_then143$$if_else208$124; } //@line 1263 "dictionary.cpp"
              }
  
              var $119=__lastLabel__ == 59 ? $117 : ($115); //@line 1266 "dictionary.cpp"
              var $cmp202=((($119))|0) > 1; //@line 1266 "dictionary.cpp"
              var $120=HEAP32[(($stressed_syllable)>>2)]; //@line 1267 "dictionary.cpp"
              if ($cmp202) { __label__ = 61;; } else { __label__ = 62;; } //@line 1266 "dictionary.cpp"
              if (__label__ == 61) {
  
                var $dec=((($120)-1)&4294967295); //@line 1267 "dictionary.cpp"
                HEAP32[(($stressed_syllable)>>2)]=$dec; //@line 1267 "dictionary.cpp"
                ; //@line 1267 "dictionary.cpp"
              }
              else if (__label__ == 62) {
  
                var $inc205=((($120)+1)&4294967295); //@line 1269 "dictionary.cpp"
                HEAP32[(($stressed_syllable)>>2)]=$inc205; //@line 1269 "dictionary.cpp"
                ;
              }
            }
            else if (__label__ == 63) {
  
              HEAP32[(($stressed_syllable)>>2)]=1; //@line 1274 "dictionary.cpp"
              ;
            }
          } while(0);
  
          var $121=HEAP32[(($stressed_syllable)>>2)]; //@line 1278 "dictionary.cpp"
          var $arrayidx210=(($vowel_stress+$121)&4294967295); //@line 1278 "dictionary.cpp"
          var $122=HEAP8[($arrayidx210)]; //@line 1278 "dictionary.cpp"
          var $conv211=(tempInt=(($122)),(tempInt>=128?tempInt-256:tempInt)); //@line 1278 "dictionary.cpp"
          var $cmp212=((($conv211))|0) < 0; //@line 1278 "dictionary.cpp"
          if (!($cmp212)) { __label__ = 112;break $sw_epilog$$sw_bb120$$if_else351$$sw_bb139$115; } //@line 1278 "dictionary.cpp"
  
          var $123=HEAP32[(($stressed_syllable)>>2)]; //@line 1281 "dictionary.cpp"
          var $sub214=((($123)-1)&4294967295); //@line 1281 "dictionary.cpp"
          var $arrayidx215=(($vowel_stress+$sub214)&4294967295); //@line 1281 "dictionary.cpp"
          var $124=HEAP8[($arrayidx215)]; //@line 1281 "dictionary.cpp"
          var $conv216=(tempInt=(($124)),(tempInt>=128?tempInt-256:tempInt)); //@line 1281 "dictionary.cpp"
          var $cmp217=((($conv216))|0) < 4; //@line 1281 "dictionary.cpp"
          if ($cmp217) { __label__ = 67;; } else { __label__ = 66;; } //@line 1281 "dictionary.cpp"
          if (__label__ == 66) {
  
            var $125=HEAP32[(($stressed_syllable)>>2)]; //@line 1281 "dictionary.cpp"
            var $add=((($125)+1)&4294967295); //@line 1281 "dictionary.cpp"
            var $arrayidx219=(($vowel_stress+$add)&4294967295); //@line 1281 "dictionary.cpp"
            var $126=HEAP8[($arrayidx219)]; //@line 1281 "dictionary.cpp"
            var $conv220=(tempInt=(($126)),(tempInt>=128?tempInt-256:tempInt)); //@line 1281 "dictionary.cpp"
            var $cmp221=((($conv220))|0) < 4; //@line 1281 "dictionary.cpp"
            if (!($cmp221)) { __label__ = 112;break $sw_epilog$$sw_bb120$$if_else351$$sw_bb139$115; } //@line 1281 "dictionary.cpp"
          }
  
          var $127=$max_stress; //@line 1282 "dictionary.cpp"
          var $conv223=((($127)) & 255); //@line 1282 "dictionary.cpp"
          var $128=HEAP32[(($stressed_syllable)>>2)]; //@line 1282 "dictionary.cpp"
          var $arrayidx224=(($vowel_stress+$128)&4294967295); //@line 1282 "dictionary.cpp"
          HEAP8[($arrayidx224)]=$conv223; //@line 1282 "dictionary.cpp"
          __label__ = 112;break $sw_epilog$$sw_bb120$$if_else351$$sw_bb139$115; //@line 1282 "dictionary.cpp"
        }
      } while(0);
      $sw_epilog$$if_else351$155: do { 
        if (__label__ == 112) {
  
          var $_pr9=$max_stress; //@line 1414 "dictionary.cpp"
          var $cmp349=((($_pr9))|0) < 4; //@line 1414 "dictionary.cpp"
          if (!($cmp349)) { __label__ = 114;break $sw_epilog$$if_else351$155; } //@line 1414 "dictionary.cpp"
  
          $stress=4; //@line 1415 "dictionary.cpp"
          __label__ = 115;break $sw_epilog$$if_else351$155; //@line 1415 "dictionary.cpp"
        }
      } while(0);
      if (__label__ == 114) {
  
        $stress=3; //@line 1417 "dictionary.cpp"
        ;
      }
  
      var $190=$stressflags; //@line 1420 "dictionary.cpp"
      var $and353=($190) & 4096; //@line 1420 "dictionary.cpp"
      var $tobool354=((($and353))|0)!=0; //@line 1420 "dictionary.cpp"
      if ($tobool354) { __label__ = 116;; } else { __label__ = 121;; } //@line 1420 "dictionary.cpp"
      $land_lhs_true355$$if_end370$161: do { 
        if (__label__ == 116) {
  
          var $191=HEAP32[(($vowel_count)>>2)]; //@line 1420 "dictionary.cpp"
          var $cmp356=((($191))|0)==2; //@line 1420 "dictionary.cpp"
          if (!($cmp356)) { __label__ = 121;break $land_lhs_true355$$if_end370$161; } //@line 1420 "dictionary.cpp"
  
          var $arrayidx358=(($vowel_stress+1)&4294967295); //@line 1423 "dictionary.cpp"
          var $192=HEAP8[($arrayidx358)]; //@line 1423 "dictionary.cpp"
          var $conv359=(tempInt=(($192)),(tempInt>=128?tempInt-256:tempInt)); //@line 1423 "dictionary.cpp"
          var $cmp360=((($conv359))|0)==4; //@line 1423 "dictionary.cpp"
          if ($cmp360) { __label__ = 118;; } else { __label__ = 119;; } //@line 1423 "dictionary.cpp"
          if (__label__ == 118) {
  
            var $arrayidx362=(($vowel_stress+2)&4294967295); //@line 1424 "dictionary.cpp"
            HEAP8[($arrayidx362)]=3; //@line 1424 "dictionary.cpp"
            ; //@line 1424 "dictionary.cpp"
          }
  
          var $arrayidx364=(($vowel_stress+2)&4294967295); //@line 1425 "dictionary.cpp"
          var $193=HEAP8[($arrayidx364)]; //@line 1425 "dictionary.cpp"
          var $conv365=(tempInt=(($193)),(tempInt>=128?tempInt-256:tempInt)); //@line 1425 "dictionary.cpp"
          var $cmp366=((($conv365))|0)==4; //@line 1425 "dictionary.cpp"
          if (!($cmp366)) { __label__ = 121;break $land_lhs_true355$$if_end370$161; } //@line 1425 "dictionary.cpp"
  
          var $arrayidx368=(($vowel_stress+1)&4294967295); //@line 1426 "dictionary.cpp"
          HEAP8[($arrayidx368)]=3; //@line 1426 "dictionary.cpp"
          ; //@line 1426 "dictionary.cpp"
        }
      } while(0);
  
      var $194=$stressflags; //@line 1429 "dictionary.cpp"
      var $and371=($194) & 8192; //@line 1429 "dictionary.cpp"
      var $tobool372=((($and371))|0)!=0; //@line 1429 "dictionary.cpp"
      if ($tobool372) { __label__ = 122;; } else { __label__ = 126;; } //@line 1429 "dictionary.cpp"
      $land_lhs_true373$$if_end386$169: do { 
        if (__label__ == 122) {
  
          var $arrayidx374=(($vowel_stress+1)&4294967295); //@line 1429 "dictionary.cpp"
          var $195=HEAP8[($arrayidx374)]; //@line 1429 "dictionary.cpp"
          var $conv375=(tempInt=(($195)),(tempInt>=128?tempInt-256:tempInt)); //@line 1429 "dictionary.cpp"
          var $cmp376=((($conv375))|0) < 0; //@line 1429 "dictionary.cpp"
          if (!($cmp376)) { __label__ = 126;break $land_lhs_true373$$if_end386$169; } //@line 1429 "dictionary.cpp"
  
          var $196=HEAP32[(($vowel_count)>>2)]; //@line 1432 "dictionary.cpp"
          var $cmp378=((($196))|0) > 2; //@line 1432 "dictionary.cpp"
          if (!($cmp378)) { __label__ = 126;break $land_lhs_true373$$if_end386$169; } //@line 1432 "dictionary.cpp"
  
          var $arrayidx380=(($vowel_stress+2)&4294967295); //@line 1432 "dictionary.cpp"
          var $197=HEAP8[($arrayidx380)]; //@line 1432 "dictionary.cpp"
          var $conv381=(tempInt=(($197)),(tempInt>=128?tempInt-256:tempInt)); //@line 1432 "dictionary.cpp"
          var $cmp382=((($conv381))|0) >= 4; //@line 1432 "dictionary.cpp"
          if (!($cmp382)) { __label__ = 126;break $land_lhs_true373$$if_end386$169; } //@line 1432 "dictionary.cpp"
  
          var $arrayidx384=(($vowel_stress+1)&4294967295); //@line 1434 "dictionary.cpp"
          HEAP8[($arrayidx384)]=3; //@line 1434 "dictionary.cpp"
          ; //@line 1435 "dictionary.cpp"
        }
      } while(0);
  
      $done=0; //@line 1438 "dictionary.cpp"
      $first_primary=0; //@line 1439 "dictionary.cpp"
      $v=1; //@line 1440 "dictionary.cpp"
      var $198=$v; //@line 1440 "dictionary.cpp"
      var $199=HEAP32[(($vowel_count)>>2)]; //@line 1440 "dictionary.cpp"
      var $cmp38818=((($198))|0) < ((($199))|0); //@line 1440 "dictionary.cpp"
      if ($cmp38818) { __label__ = 127;; } else { __label__ = 151;; } //@line 1440 "dictionary.cpp"
      $for_body389$$for_end471$175: do { 
        if (__label__ == 127) {
          $for_body389$176: while(1) { 
  
            var $200=$v; //@line 1442 "dictionary.cpp"
            var $arrayidx390=(($vowel_stress+$200)&4294967295); //@line 1442 "dictionary.cpp"
            var $201=HEAP8[($arrayidx390)]; //@line 1442 "dictionary.cpp"
            var $conv391=(tempInt=(($201)),(tempInt>=128?tempInt-256:tempInt)); //@line 1442 "dictionary.cpp"
            var $cmp392=((($conv391))|0) < 0; //@line 1442 "dictionary.cpp"
            if ($cmp392) { __label__ = 128;; } else { __label__ = 145;; } //@line 1442 "dictionary.cpp"
            $if_then393$$if_end454$178: do { 
              if (__label__ == 128) {
  
                var $202=$stressflags; //@line 1444 "dictionary.cpp"
                var $and394=($202) & 16; //@line 1444 "dictionary.cpp"
                var $tobool395=((($and394))|0)!=0; //@line 1444 "dictionary.cpp"
                if ($tobool395) { __label__ = 129;; } else { __label__ = 131;; } //@line 1444 "dictionary.cpp"
                $land_lhs_true396$$if_else402$180: do { 
                  if (__label__ == 129) {
  
                    var $203=$stress; //@line 1444 "dictionary.cpp"
                    var $cmp397=((($203))|0) < 4; //@line 1444 "dictionary.cpp"
                    if (!($cmp397)) { __label__ = 131;break $land_lhs_true396$$if_else402$180; } //@line 1444 "dictionary.cpp"
  
                    var $204=$v; //@line 1444 "dictionary.cpp"
                    var $205=HEAP32[(($vowel_count)>>2)]; //@line 1444 "dictionary.cpp"
                    var $sub399=((($205)-1)&4294967295); //@line 1444 "dictionary.cpp"
                    var $cmp400=((($204))|0)==((($sub399))|0); //@line 1444 "dictionary.cpp"
                    if ($cmp400) { __label__ = 145;break $if_then393$$if_end454$178; } //@line 1444 "dictionary.cpp"
                  }
                } while(0);
  
                var $206=$stressflags; //@line 1449 "dictionary.cpp"
                var $and403=($206) & 32768; //@line 1449 "dictionary.cpp"
                var $tobool404=((($and403))|0)!=0; //@line 1449 "dictionary.cpp"
                if ($tobool404) { __label__ = 132;; } else { __label__ = 134;; } //@line 1449 "dictionary.cpp"
                $land_lhs_true405$$if_else410$184: do { 
                  if (__label__ == 132) {
  
                    var $207=$done; //@line 1449 "dictionary.cpp"
                    var $cmp406=((($207))|0)==0; //@line 1449 "dictionary.cpp"
                    if (!($cmp406)) { __label__ = 134;break $land_lhs_true405$$if_else410$184; } //@line 1449 "dictionary.cpp"
  
                    var $208=$stress; //@line 1451 "dictionary.cpp"
                    var $conv408=((($208)) & 255); //@line 1451 "dictionary.cpp"
                    var $209=$v; //@line 1451 "dictionary.cpp"
                    var $arrayidx409=(($vowel_stress+$209)&4294967295); //@line 1451 "dictionary.cpp"
                    HEAP8[($arrayidx409)]=$conv408; //@line 1451 "dictionary.cpp"
                    $done=1; //@line 1452 "dictionary.cpp"
                    $stress=3; //@line 1453 "dictionary.cpp"
                    __label__ = 145;break $if_then393$$if_end454$178; //@line 1454 "dictionary.cpp"
                  }
                } while(0);
  
                var $210=$v; //@line 1456 "dictionary.cpp"
                var $sub411=((($210)-1)&4294967295); //@line 1456 "dictionary.cpp"
                var $arrayidx412=(($vowel_stress+$sub411)&4294967295); //@line 1456 "dictionary.cpp"
                var $211=HEAP8[($arrayidx412)]; //@line 1456 "dictionary.cpp"
                var $conv413=(tempInt=(($211)),(tempInt>=128?tempInt-256:tempInt)); //@line 1456 "dictionary.cpp"
                var $cmp414=((($conv413))|0) <= 1; //@line 1456 "dictionary.cpp"
                if (!($cmp414)) { __label__ = 145;break $if_then393$$if_end454$178; } //@line 1456 "dictionary.cpp"
  
                var $212=$v; //@line 1456 "dictionary.cpp"
                var $add416=((($212)+1)&4294967295); //@line 1456 "dictionary.cpp"
                var $arrayidx417=(($vowel_stress+$add416)&4294967295); //@line 1456 "dictionary.cpp"
                var $213=HEAP8[($arrayidx417)]; //@line 1456 "dictionary.cpp"
                var $conv418=(tempInt=(($213)),(tempInt>=128?tempInt-256:tempInt)); //@line 1456 "dictionary.cpp"
                var $cmp419=((($conv418))|0) <= 1; //@line 1456 "dictionary.cpp"
                if ($cmp419) { __label__ = 138;; } else { __label__ = 136;; } //@line 1456 "dictionary.cpp"
                if (__label__ == 136) {
  
                  var $214=$stress; //@line 1456 "dictionary.cpp"
                  var $cmp421=((($214))|0)==4; //@line 1456 "dictionary.cpp"
                  if (!($cmp421)) { __label__ = 145;break $if_then393$$if_end454$178; } //@line 1456 "dictionary.cpp"
  
                  var $215=$v; //@line 1456 "dictionary.cpp"
                  var $add423=((($215)+1)&4294967295); //@line 1456 "dictionary.cpp"
                  var $arrayidx424=(($vowel_stress+$add423)&4294967295); //@line 1456 "dictionary.cpp"
                  var $216=HEAP8[($arrayidx424)]; //@line 1456 "dictionary.cpp"
                  var $conv425=(tempInt=(($216)),(tempInt>=128?tempInt-256:tempInt)); //@line 1456 "dictionary.cpp"
                  var $cmp426=((($conv425))|0) <= 2; //@line 1456 "dictionary.cpp"
                  if (!($cmp426)) { __label__ = 145;break $if_then393$$if_end454$178; } //@line 1456 "dictionary.cpp"
                }
  
                var $217=$stress; //@line 1460 "dictionary.cpp"
                var $cmp428=((($217))|0)==3; //@line 1460 "dictionary.cpp"
                if ($cmp428) { __label__ = 139;; } else { __label__ = 140;; } //@line 1460 "dictionary.cpp"
                if (__label__ == 139) {
  
                  var $218=$stressflags; //@line 1460 "dictionary.cpp"
                  var $and430=($218) & 32; //@line 1460 "dictionary.cpp"
                  var $tobool431=((($and430))|0)!=0; //@line 1460 "dictionary.cpp"
                  if ($tobool431) { __label__ = 150;break $if_then393$$if_end454$178; } //@line 1460 "dictionary.cpp"
                }
  
                var $219=$v; //@line 1463 "dictionary.cpp"
                var $cmp434=((($219))|0) > 1; //@line 1463 "dictionary.cpp"
                if ($cmp434) { __label__ = 141;; } else { __label__ = 144;; } //@line 1463 "dictionary.cpp"
                $land_lhs_true435$$if_end448$196: do { 
                  if (__label__ == 141) {
  
                    var $220=$stressflags; //@line 1463 "dictionary.cpp"
                    var $and436=($220) & 64; //@line 1463 "dictionary.cpp"
                    var $tobool437=((($and436))|0)!=0; //@line 1463 "dictionary.cpp"
                    if (!($tobool437)) { __label__ = 144;break $land_lhs_true435$$if_end448$196; } //@line 1463 "dictionary.cpp"
  
                    var $221=$v; //@line 1463 "dictionary.cpp"
                    var $arrayidx439=(($syllable_weight+$221)&4294967295); //@line 1463 "dictionary.cpp"
                    var $222=HEAP8[($arrayidx439)]; //@line 1463 "dictionary.cpp"
                    var $conv440=(tempInt=(($222)),(tempInt>=128?tempInt-256:tempInt)); //@line 1463 "dictionary.cpp"
                    var $cmp441=((($conv440))|0)==0; //@line 1463 "dictionary.cpp"
                    if (!($cmp441)) { __label__ = 144;break $land_lhs_true435$$if_end448$196; } //@line 1463 "dictionary.cpp"
  
                    var $223=$v; //@line 1463 "dictionary.cpp"
                    var $add443=((($223)+1)&4294967295); //@line 1463 "dictionary.cpp"
                    var $arrayidx444=(($syllable_weight+$add443)&4294967295); //@line 1463 "dictionary.cpp"
                    var $224=HEAP8[($arrayidx444)]; //@line 1463 "dictionary.cpp"
                    var $conv445=(tempInt=(($224)),(tempInt>=128?tempInt-256:tempInt)); //@line 1463 "dictionary.cpp"
                    var $cmp446=((($conv445))|0) > 0; //@line 1463 "dictionary.cpp"
                    if ($cmp446) { __label__ = 150;break $if_then393$$if_end454$178; } //@line 1463 "dictionary.cpp"
                  }
                } while(0);
  
                var $225=$stress; //@line 1471 "dictionary.cpp"
                var $conv449=((($225)) & 255); //@line 1471 "dictionary.cpp"
                var $226=$v; //@line 1471 "dictionary.cpp"
                var $arrayidx450=(($vowel_stress+$226)&4294967295); //@line 1471 "dictionary.cpp"
                HEAP8[($arrayidx450)]=$conv449; //@line 1471 "dictionary.cpp"
                $done=1; //@line 1472 "dictionary.cpp"
                $stress=3; //@line 1473 "dictionary.cpp"
                __label__ = 145;break $if_then393$$if_end454$178; //@line 1474 "dictionary.cpp"
              }
            } while(0);
            $if_end454$$for_inc469$201: do { 
              if (__label__ == 145) {
  
                var $227=$v; //@line 1477 "dictionary.cpp"
                var $arrayidx455=(($vowel_stress+$227)&4294967295); //@line 1477 "dictionary.cpp"
                var $228=HEAP8[($arrayidx455)]; //@line 1477 "dictionary.cpp"
                var $conv456=(tempInt=(($228)),(tempInt>=128?tempInt-256:tempInt)); //@line 1477 "dictionary.cpp"
                var $cmp457=((($conv456))|0) >= 4; //@line 1477 "dictionary.cpp"
                if (!($cmp457)) { __label__ = 150;break $if_end454$$for_inc469$201; } //@line 1477 "dictionary.cpp"
  
                var $229=$first_primary; //@line 1479 "dictionary.cpp"
                var $cmp459=((($229))|0)==0; //@line 1479 "dictionary.cpp"
                if ($cmp459) { __label__ = 147;; } else { __label__ = 148;; } //@line 1479 "dictionary.cpp"
                if (__label__ == 147) {
  
                  var $230=$v; //@line 1480 "dictionary.cpp"
                  $first_primary=$230; //@line 1480 "dictionary.cpp"
                  ; //@line 1480 "dictionary.cpp"
                }
                else if (__label__ == 148) {
  
                  var $231=$stressflags; //@line 1482 "dictionary.cpp"
                  var $and462=($231) & 128; //@line 1482 "dictionary.cpp"
                  var $tobool463=((($and462))|0)!=0; //@line 1482 "dictionary.cpp"
                  if (!($tobool463)) { __label__ = 150;break $if_end454$$for_inc469$201; } //@line 1482 "dictionary.cpp"
  
                  var $232=$v; //@line 1485 "dictionary.cpp"
                  var $arrayidx465=(($vowel_stress+$232)&4294967295); //@line 1485 "dictionary.cpp"
                  HEAP8[($arrayidx465)]=3; //@line 1485 "dictionary.cpp"
                  ; //@line 1486 "dictionary.cpp"
                }
              }
            } while(0);
  
            var $233=$v; //@line 1440 "dictionary.cpp"
            var $inc470=((($233)+1)&4294967295); //@line 1440 "dictionary.cpp"
            $v=$inc470; //@line 1440 "dictionary.cpp"
            var $234=$v; //@line 1440 "dictionary.cpp"
            var $235=HEAP32[(($vowel_count)>>2)]; //@line 1440 "dictionary.cpp"
            var $cmp388=((($234))|0) < ((($235))|0); //@line 1440 "dictionary.cpp"
            if ($cmp388) { __label__ = 127;continue $for_body389$176; } else { __label__ = 151;break $for_body389$$for_end471$175; } //@line 1440 "dictionary.cpp"
          }
        }
      } while(0);
  
      var $236=$unstressed_word; //@line 1490 "dictionary.cpp"
      var $tobool472=((($236))|0)!=0; //@line 1490 "dictionary.cpp"
      if ($tobool472) { __label__ = 152;; } else { __label__ = 156;; } //@line 1490 "dictionary.cpp"
      $land_lhs_true473$$if_end482$210: do { 
        if (__label__ == 152) {
  
          var $237=$tonic_addr; //@line 1490 "dictionary.cpp"
          var $cmp474=((($237))|0) < 0; //@line 1490 "dictionary.cpp"
          if (!($cmp474)) { __label__ = 156;break $land_lhs_true473$$if_end482$210; } //@line 1490 "dictionary.cpp"
  
          var $238=HEAP32[(($vowel_count)>>2)]; //@line 1492 "dictionary.cpp"
          var $cmp476=((($238))|0) <= 2; //@line 1492 "dictionary.cpp"
          var $239=$tr_addr; //@line 1493 "dictionary.cpp"
          var $langopts478=(($239)&4294967295); //@line 1493 "dictionary.cpp"
          if ($cmp476) { __label__ = 154;; } else { __label__ = 155;; } //@line 1492 "dictionary.cpp"
          if (__label__ == 154) {
  
            var $unstressed_wd1=(($langopts478+16)&4294967295); //@line 1493 "dictionary.cpp"
            var $240=HEAP32[(($unstressed_wd1)>>2)]; //@line 1493 "dictionary.cpp"
            $tonic_addr=$240; //@line 1493 "dictionary.cpp"
            ; //@line 1493 "dictionary.cpp"
          }
          else if (__label__ == 155) {
  
            var $unstressed_wd2=(($langopts478+20)&4294967295); //@line 1495 "dictionary.cpp"
            var $241=HEAP32[(($unstressed_wd2)>>2)]; //@line 1495 "dictionary.cpp"
            $tonic_addr=$241; //@line 1495 "dictionary.cpp"
            ;
          }
        }
      } while(0);
  
      $max_stress=0; //@line 1498 "dictionary.cpp"
      $max_stress_posn=0; //@line 1499 "dictionary.cpp"
      $v=1; //@line 1500 "dictionary.cpp"
      var $242=$v; //@line 1500 "dictionary.cpp"
      var $243=HEAP32[(($vowel_count)>>2)]; //@line 1500 "dictionary.cpp"
      var $cmp48417=((($242))|0) < ((($243))|0); //@line 1500 "dictionary.cpp"
      if ($cmp48417) { __label__ = 157;; } else { __label__ = 160;; } //@line 1500 "dictionary.cpp"
      $for_body485$$for_end495$217: do { 
        if (__label__ == 157) {
          while(1) { 
  
            var $244=$v; //@line 1502 "dictionary.cpp"
            var $arrayidx486=(($vowel_stress+$244)&4294967295); //@line 1502 "dictionary.cpp"
            var $245=HEAP8[($arrayidx486)]; //@line 1502 "dictionary.cpp"
            var $conv487=(tempInt=(($245)),(tempInt>=128?tempInt-256:tempInt)); //@line 1502 "dictionary.cpp"
            var $246=$max_stress; //@line 1502 "dictionary.cpp"
            var $cmp488=((($conv487))|0) >= ((($246))|0); //@line 1502 "dictionary.cpp"
            if ($cmp488) { __label__ = 158;; } else { __label__ = 159;; } //@line 1502 "dictionary.cpp"
            if (__label__ == 158) {
  
              var $247=$v; //@line 1504 "dictionary.cpp"
              var $arrayidx490=(($vowel_stress+$247)&4294967295); //@line 1504 "dictionary.cpp"
              var $248=HEAP8[($arrayidx490)]; //@line 1504 "dictionary.cpp"
              var $conv491=(tempInt=(($248)),(tempInt>=128?tempInt-256:tempInt)); //@line 1504 "dictionary.cpp"
              $max_stress=$conv491; //@line 1504 "dictionary.cpp"
              var $249=$v; //@line 1505 "dictionary.cpp"
              $max_stress_posn=$249; //@line 1505 "dictionary.cpp"
              ; //@line 1506 "dictionary.cpp"
            }
  
            var $250=$v; //@line 1500 "dictionary.cpp"
            var $inc494=((($250)+1)&4294967295); //@line 1500 "dictionary.cpp"
            $v=$inc494; //@line 1500 "dictionary.cpp"
            var $251=$v; //@line 1500 "dictionary.cpp"
            var $252=HEAP32[(($vowel_count)>>2)]; //@line 1500 "dictionary.cpp"
            var $cmp484=((($251))|0) < ((($252))|0); //@line 1500 "dictionary.cpp"
            if ($cmp484) { __label__ = 157;continue ; } else { __label__ = 160;break $for_body485$$for_end495$217; } //@line 1500 "dictionary.cpp"
          }
        }
      } while(0);
  
      var $253=$tonic_addr; //@line 1509 "dictionary.cpp"
      var $cmp496=((($253))|0) >= 0; //@line 1509 "dictionary.cpp"
      if ($cmp496) { __label__ = 161;; } else { __label__ = 164;; } //@line 1509 "dictionary.cpp"
      if (__label__ == 161) {
  
        var $254=$tonic_addr; //@line 1514 "dictionary.cpp"
        var $255=$max_stress; //@line 1514 "dictionary.cpp"
        var $cmp498=((($254))|0) > ((($255))|0); //@line 1514 "dictionary.cpp"
        var $256=$max_stress; //@line 1514 "dictionary.cpp"
        var $cmp500=((($256))|0) <= 4; //@line 1514 "dictionary.cpp"
        var $or_cond1=($cmp498) | ($cmp500); //@line 1514 "dictionary.cpp"
        if ($or_cond1) { __label__ = 162;; } else { __label__ = 163;; } //@line 1514 "dictionary.cpp"
        if (__label__ == 162) {
  
          var $257=$tonic_addr; //@line 1515 "dictionary.cpp"
          var $conv502=((($257)) & 255); //@line 1515 "dictionary.cpp"
          var $258=$max_stress_posn; //@line 1515 "dictionary.cpp"
          var $arrayidx503=(($vowel_stress+$258)&4294967295); //@line 1515 "dictionary.cpp"
          HEAP8[($arrayidx503)]=$conv502; //@line 1515 "dictionary.cpp"
          ; //@line 1515 "dictionary.cpp"
        }
  
        var $259=$tonic_addr; //@line 1516 "dictionary.cpp"
        $max_stress=$259; //@line 1516 "dictionary.cpp"
        ; //@line 1517 "dictionary.cpp"
      }
  
      var $arraydecay506=(($phonetic)&4294967295); //@line 1521 "dictionary.cpp"
      $p=$arraydecay506; //@line 1521 "dictionary.cpp"
      $v=1; //@line 1522 "dictionary.cpp"
      var $260=$control_addr; //@line 1524 "dictionary.cpp"
      var $and507=($260) & 1; //@line 1524 "dictionary.cpp"
      var $tobool508=((($and507))|0)!=0; //@line 1524 "dictionary.cpp"
      if ($tobool508) { __label__ = 174;; } else { __label__ = 165;; } //@line 1524 "dictionary.cpp"
      $if_end544$$land_lhs_true509$230: do { 
        if (__label__ == 165) {
  
          var $261=$p; //@line 1524 "dictionary.cpp"
          var $262=HEAPU8[($261)]; //@line 1524 "dictionary.cpp"
          var $idxprom510=((($262))&255); //@line 1524 "dictionary.cpp"
          var $arrayidx511=((_phoneme_tab+$idxprom510*4)&4294967295); //@line 1524 "dictionary.cpp"
          var $263=HEAPU32[(($arrayidx511)>>2)]; //@line 1524 "dictionary.cpp"
          $ph=$263; //@line 1524 "dictionary.cpp"
          var $cmp512=((($263))|0)!=0; //@line 1524 "dictionary.cpp"
          if (!($cmp512)) { __label__ = 174;break $if_end544$$land_lhs_true509$230; } //@line 1524 "dictionary.cpp"
  
          var $264=$ph; //@line 1527 "dictionary.cpp"
          var $type514=(($264+11)&4294967295); //@line 1527 "dictionary.cpp"
          var $265=HEAPU8[($type514)]; //@line 1527 "dictionary.cpp"
          var $conv515=((($265))&255); //@line 1527 "dictionary.cpp"
          var $cmp516=((($conv515))|0)==1; //@line 1527 "dictionary.cpp"
          if ($cmp516) { __label__ = 167;; } else { __label__ = 168;; } //@line 1527 "dictionary.cpp"
          if (__label__ == 167) {
  
            var $266=$p; //@line 1528 "dictionary.cpp"
            var $arrayidx518=(($266+1)&4294967295); //@line 1528 "dictionary.cpp"
            var $267=HEAPU8[($arrayidx518)]; //@line 1528 "dictionary.cpp"
            var $idxprom519=((($267))&255); //@line 1528 "dictionary.cpp"
            var $arrayidx520=((_phoneme_tab+$idxprom519*4)&4294967295); //@line 1528 "dictionary.cpp"
            var $268=HEAP32[(($arrayidx520)>>2)]; //@line 1528 "dictionary.cpp"
            $ph=$268; //@line 1528 "dictionary.cpp"
            ; //@line 1528 "dictionary.cpp"
          }
  
          var $269=$tr_addr; //@line 1539 "dictionary.cpp"
          var $langopts522=(($269)&4294967295); //@line 1539 "dictionary.cpp"
          var $vowel_pause=(($langopts522+4)&4294967295); //@line 1539 "dictionary.cpp"
          var $270=HEAP32[(($vowel_pause)>>2)]; //@line 1539 "dictionary.cpp"
          var $and523=($270) & 48; //@line 1539 "dictionary.cpp"
          var $tobool524=((($and523))|0)!=0; //@line 1539 "dictionary.cpp"
          if (!($tobool524)) { __label__ = 174;break $if_end544$$land_lhs_true509$230; } //@line 1539 "dictionary.cpp"
  
          var $271=$ph; //@line 1539 "dictionary.cpp"
          var $type526=(($271+11)&4294967295); //@line 1539 "dictionary.cpp"
          var $272=HEAPU8[($type526)]; //@line 1539 "dictionary.cpp"
          var $conv527=((($272))&255); //@line 1539 "dictionary.cpp"
          var $cmp528=((($conv527))|0)==2; //@line 1539 "dictionary.cpp"
          if (!($cmp528)) { __label__ = 174;break $if_end544$$land_lhs_true509$230; } //@line 1539 "dictionary.cpp"
  
          var $273=$tr_addr; //@line 1543 "dictionary.cpp"
          var $langopts530=(($273)&4294967295); //@line 1543 "dictionary.cpp"
          var $vowel_pause531=(($langopts530+4)&4294967295); //@line 1543 "dictionary.cpp"
          var $274=HEAP32[(($vowel_pause531)>>2)]; //@line 1543 "dictionary.cpp"
          var $and532=($274) & 32; //@line 1543 "dictionary.cpp"
          var $tobool533=((($and532))|0)!=0; //@line 1543 "dictionary.cpp"
          if ($tobool533) { __label__ = 171;; } else { __label__ = 173;; } //@line 1543 "dictionary.cpp"
          $land_lhs_true534$$if_else540$238: do { 
            if (__label__ == 171) {
  
              var $arrayidx535=(($vowel_stress+1)&4294967295); //@line 1543 "dictionary.cpp"
              var $275=HEAP8[($arrayidx535)]; //@line 1543 "dictionary.cpp"
              var $conv536=(tempInt=(($275)),(tempInt>=128?tempInt-256:tempInt)); //@line 1543 "dictionary.cpp"
              var $cmp537=((($conv536))|0) >= 4; //@line 1543 "dictionary.cpp"
              if (!($cmp537)) { __label__ = 173;break $land_lhs_true534$$if_else540$238; } //@line 1543 "dictionary.cpp"
  
              var $276=$output_addr; //@line 1545 "dictionary.cpp"
              var $incdec_ptr539=(($276+1)&4294967295); //@line 1545 "dictionary.cpp"
              $output_addr=$incdec_ptr539; //@line 1545 "dictionary.cpp"
              HEAP8[($276)]=11; //@line 1545 "dictionary.cpp"
              __label__ = 174;break $if_end544$$land_lhs_true509$230; //@line 1546 "dictionary.cpp"
            }
          } while(0);
  
          var $277=$output_addr; //@line 1549 "dictionary.cpp"
          var $incdec_ptr541=(($277+1)&4294967295); //@line 1549 "dictionary.cpp"
          $output_addr=$incdec_ptr541; //@line 1549 "dictionary.cpp"
          HEAP8[($277)]=23; //@line 1549 "dictionary.cpp"
          ;
        }
      } while(0);
  
      var $arraydecay545=(($phonetic)&4294967295); //@line 1554 "dictionary.cpp"
      $p=$arraydecay545; //@line 1554 "dictionary.cpp"
      $post_tonic=0; //@line 1555 "dictionary.cpp"
      ; //@line 1556 "dictionary.cpp"
      $while_cond546$243: while(1) { 
  
        var $278=$p; //@line 1556 "dictionary.cpp"
        var $incdec_ptr547=(($278+1)&4294967295); //@line 1556 "dictionary.cpp"
        $p=$incdec_ptr547; //@line 1556 "dictionary.cpp"
        var $279=HEAPU8[($278)]; //@line 1556 "dictionary.cpp"
        $phcode=$279; //@line 1556 "dictionary.cpp"
        var $conv548=((($279))&255); //@line 1556 "dictionary.cpp"
        var $cmp549=((($conv548))|0)!=0; //@line 1556 "dictionary.cpp"
        if (!($cmp549)) { __label__ = 222;break $while_cond546$243; } //@line 1556 "dictionary.cpp"
  
        var $280=$output_addr; //@line 1556 "dictionary.cpp"
        var $281=$max_output; //@line 1556 "dictionary.cpp"
        var $cmp550=((($280))>>>0) < ((($281))>>>0); //@line 1556 "dictionary.cpp"
        if (!($cmp550)) { __label__ = 222;break $while_cond546$243; }
  
        var $282=$phcode; //@line 1558 "dictionary.cpp"
        var $idxprom552=((($282))&255); //@line 1558 "dictionary.cpp"
        var $arrayidx553=((_phoneme_tab+$idxprom552*4)&4294967295); //@line 1558 "dictionary.cpp"
        var $283=HEAPU32[(($arrayidx553)>>2)]; //@line 1558 "dictionary.cpp"
        $ph=$283; //@line 1558 "dictionary.cpp"
        var $cmp554=((($283))|0)==0; //@line 1558 "dictionary.cpp"
        if ($cmp554) { __label__ = 175;continue $while_cond546$243; } //@line 1558 "dictionary.cpp"
  
        var $284=$ph; //@line 1564 "dictionary.cpp"
        var $type557=(($284+11)&4294967295); //@line 1564 "dictionary.cpp"
        var $285=HEAPU8[($type557)]; //@line 1564 "dictionary.cpp"
        var $conv558=((($285))&255); //@line 1564 "dictionary.cpp"
        var $cmp559=((($conv558))|0)==0; //@line 1564 "dictionary.cpp"
        if ($cmp559) { __label__ = 179;; } else { __label__ = 180;; } //@line 1564 "dictionary.cpp"
        $if_then560$$if_else561$248: do { 
          if (__label__ == 179) {
  
            var $286=$tr_addr; //@line 1566 "dictionary.cpp"
            var $prev_last_stress=(($286+8288)&4294967295); //@line 1566 "dictionary.cpp"
            HEAP32[(($prev_last_stress)>>2)]=0; //@line 1566 "dictionary.cpp"
            ; //@line 1567 "dictionary.cpp"
          }
          else if (__label__ == 180) {
  
            var $287=$ph; //@line 1569 "dictionary.cpp"
            var $type562=(($287+11)&4294967295); //@line 1569 "dictionary.cpp"
            var $288=HEAPU8[($type562)]; //@line 1569 "dictionary.cpp"
            var $conv563=((($288))&255); //@line 1569 "dictionary.cpp"
            var $cmp564=((($conv563))|0)==2; //@line 1569 "dictionary.cpp"
            if ($cmp564) { __label__ = 181;; } else { __label__ = 182;; } //@line 1569 "dictionary.cpp"
            $land_lhs_true565$$lor_lhs_false569$251: do { 
              if (__label__ == 181) {
  
                var $289=$ph; //@line 1569 "dictionary.cpp"
                var $phflags566=(($289+4)&4294967295); //@line 1569 "dictionary.cpp"
                var $290=HEAP32[(($phflags566)>>2)]; //@line 1569 "dictionary.cpp"
                var $and567=($290) & 1048576; //@line 1569 "dictionary.cpp"
                var $tobool568=((($and567))|0)!=0; //@line 1569 "dictionary.cpp"
                if ($tobool568) { __label__ = 182;break $land_lhs_true565$$lor_lhs_false569$251; } else { __label__ = 183;break $land_lhs_true565$$lor_lhs_false569$251; } //@line 1569 "dictionary.cpp"
              }
            } while(0);
            if (__label__ == 182) {
  
              var $291=$p; //@line 1569 "dictionary.cpp"
              var $292=HEAPU8[($291)]; //@line 1569 "dictionary.cpp"
              var $conv570=((($292))&255); //@line 1569 "dictionary.cpp"
              var $cmp571=((($conv570))|0)==20; //@line 1569 "dictionary.cpp"
              if (!($cmp571)) { __label__ = 220;break $if_then560$$if_else561$248; } //@line 1569 "dictionary.cpp"
            }
  
            var $293=$v; //@line 1573 "dictionary.cpp"
            var $arrayidx573=(($vowel_stress+$293)&4294967295); //@line 1573 "dictionary.cpp"
            var $294=HEAP8[($arrayidx573)]; //@line 1573 "dictionary.cpp"
            var $conv574=(tempInt=(($294)),(tempInt>=128?tempInt-256:tempInt)); //@line 1573 "dictionary.cpp"
            $v_stress=$conv574; //@line 1573 "dictionary.cpp"
            var $295=$v_stress; //@line 1574 "dictionary.cpp"
            var $296=$tr_addr; //@line 1574 "dictionary.cpp"
            var $prev_last_stress575=(($296+8288)&4294967295); //@line 1574 "dictionary.cpp"
            HEAP32[(($prev_last_stress575)>>2)]=$295; //@line 1574 "dictionary.cpp"
            var $297=$v; //@line 1576 "dictionary.cpp"
            var $sub576=((($297)-1)&4294967295); //@line 1576 "dictionary.cpp"
            var $arrayidx577=(($vowel_stress+$sub576)&4294967295); //@line 1576 "dictionary.cpp"
            var $298=HEAP8[($arrayidx577)]; //@line 1576 "dictionary.cpp"
            var $conv578=(tempInt=(($298)),(tempInt>=128?tempInt-256:tempInt)); //@line 1576 "dictionary.cpp"
            var $299=$max_stress; //@line 1576 "dictionary.cpp"
            var $cmp579=((($conv578))|0) >= ((($299))|0); //@line 1576 "dictionary.cpp"
            if ($cmp579) { __label__ = 184;; } else { __label__ = 185;; } //@line 1576 "dictionary.cpp"
            if (__label__ == 184) {
  
              $post_tonic=1; //@line 1577 "dictionary.cpp"
              ; //@line 1577 "dictionary.cpp"
            }
  
            var $300=$v_stress; //@line 1579 "dictionary.cpp"
            var $cmp582=((($300))|0) <= 1; //@line 1579 "dictionary.cpp"
            if ($cmp582) { __label__ = 186;; } else { __label__ = 201;; } //@line 1579 "dictionary.cpp"
            $if_then583$$if_end627$259: do { 
              if (__label__ == 186) {
  
                var $301=$v; //@line 1581 "dictionary.cpp"
                var $cmp584=((($301))|0) > 1; //@line 1581 "dictionary.cpp"
                if ($cmp584) { __label__ = 187;; } else { __label__ = 191;; } //@line 1581 "dictionary.cpp"
                $land_lhs_true585$$if_else594$261: do { 
                  if (__label__ == 187) {
  
                    var $302=$max_stress; //@line 1581 "dictionary.cpp"
                    var $cmp586=((($302))|0) >= 4; //@line 1581 "dictionary.cpp"
                    if (!($cmp586)) { __label__ = 191;break $land_lhs_true585$$if_else594$261; } //@line 1581 "dictionary.cpp"
  
                    var $303=$stressflags; //@line 1581 "dictionary.cpp"
                    var $and588=($303) & 4; //@line 1581 "dictionary.cpp"
                    var $tobool589=((($and588))|0)!=0; //@line 1581 "dictionary.cpp"
                    if (!($tobool589)) { __label__ = 191;break $land_lhs_true585$$if_else594$261; } //@line 1581 "dictionary.cpp"
  
                    var $304=$v; //@line 1581 "dictionary.cpp"
                    var $305=HEAP32[(($vowel_count)>>2)]; //@line 1581 "dictionary.cpp"
                    var $sub591=((($305)-1)&4294967295); //@line 1581 "dictionary.cpp"
                    var $cmp592=((($304))|0)==((($sub591))|0); //@line 1581 "dictionary.cpp"
                    if (!($cmp592)) { __label__ = 191;break $land_lhs_true585$$if_else594$261; } //@line 1581 "dictionary.cpp"
  
                    $v_stress=0; //@line 1584 "dictionary.cpp"
                    __label__ = 201;break $if_then583$$if_end627$259; //@line 1585 "dictionary.cpp"
                  }
                } while(0);
  
                var $306=$stressflags; //@line 1587 "dictionary.cpp"
                var $and595=($306) & 2; //@line 1587 "dictionary.cpp"
                var $tobool596=((($and595))|0)!=0; //@line 1587 "dictionary.cpp"
                if ($tobool596) { __label__ = 194;; } else { __label__ = 192;; } //@line 1587 "dictionary.cpp"
                $if_then602$$lor_lhs_false597$267: do { 
                  if (__label__ == 192) {
  
                    var $307=$v; //@line 1587 "dictionary.cpp"
                    var $cmp598=((($307))|0)==1; //@line 1587 "dictionary.cpp"
                    if ($cmp598) { __label__ = 194;break $if_then602$$lor_lhs_false597$267; } //@line 1587 "dictionary.cpp"
  
                    var $308=$v; //@line 1587 "dictionary.cpp"
                    var $309=HEAP32[(($vowel_count)>>2)]; //@line 1587 "dictionary.cpp"
                    var $sub600=((($309)-1)&4294967295); //@line 1587 "dictionary.cpp"
                    var $cmp601=((($308))|0)==((($sub600))|0); //@line 1587 "dictionary.cpp"
                    if ($cmp601) { __label__ = 194;break $if_then602$$lor_lhs_false597$267; } //@line 1587 "dictionary.cpp"
  
                    var $310=$v; //@line 1593 "dictionary.cpp"
                    var $311=HEAP32[(($vowel_count)>>2)]; //@line 1593 "dictionary.cpp"
                    var $sub604=((($311)-2)&4294967295); //@line 1593 "dictionary.cpp"
                    var $cmp605=((($310))|0)==((($sub604))|0); //@line 1593 "dictionary.cpp"
                    if ($cmp605) { __label__ = 196;; } else { __label__ = 198;; } //@line 1593 "dictionary.cpp"
                    $land_lhs_true606$$if_else612$271: do { 
                      if (__label__ == 196) {
  
                        var $312=HEAP32[(($vowel_count)>>2)]; //@line 1593 "dictionary.cpp"
                        var $sub607=((($312)-1)&4294967295); //@line 1593 "dictionary.cpp"
                        var $arrayidx608=(($vowel_stress+$sub607)&4294967295); //@line 1593 "dictionary.cpp"
                        var $313=HEAP8[($arrayidx608)]; //@line 1593 "dictionary.cpp"
                        var $conv609=(tempInt=(($313)),(tempInt>=128?tempInt-256:tempInt)); //@line 1593 "dictionary.cpp"
                        var $cmp610=((($conv609))|0) <= 1; //@line 1593 "dictionary.cpp"
                        if (!($cmp610)) { __label__ = 198;break $land_lhs_true606$$if_else612$271; } //@line 1593 "dictionary.cpp"
  
                        $v_stress=1; //@line 1596 "dictionary.cpp"
                        __label__ = 201;break $if_then583$$if_end627$259; //@line 1597 "dictionary.cpp"
                      }
                    } while(0);
  
                    var $314=$v; //@line 1601 "dictionary.cpp"
                    var $sub613=((($314)-1)&4294967295); //@line 1601 "dictionary.cpp"
                    var $arrayidx614=(($vowel_stress+$sub613)&4294967295); //@line 1601 "dictionary.cpp"
                    var $315=HEAP8[($arrayidx614)]; //@line 1601 "dictionary.cpp"
                    var $conv615=(tempInt=(($315)),(tempInt>=128?tempInt-256:tempInt)); //@line 1601 "dictionary.cpp"
                    var $cmp616=((($conv615))|0) < 0; //@line 1601 "dictionary.cpp"
                    if ($cmp616) { __label__ = 200;; } else { __label__ = 199;; } //@line 1601 "dictionary.cpp"
                    if (__label__ == 199) {
  
                      var $316=$stressflags; //@line 1601 "dictionary.cpp"
                      var $and618=($316) & 65536; //@line 1601 "dictionary.cpp"
                      var $cmp619=((($and618))|0)==0; //@line 1601 "dictionary.cpp"
                      if (!($cmp619)) { __label__ = 201;break $if_then583$$if_end627$259; } //@line 1601 "dictionary.cpp"
                    }
  
                    $v_stress=0; //@line 1603 "dictionary.cpp"
                    var $317=$v_stress; //@line 1604 "dictionary.cpp"
                    var $conv621=((($317)) & 255); //@line 1604 "dictionary.cpp"
                    var $318=$v; //@line 1604 "dictionary.cpp"
                    var $arrayidx622=(($vowel_stress+$318)&4294967295); //@line 1604 "dictionary.cpp"
                    HEAP8[($arrayidx622)]=$conv621; //@line 1604 "dictionary.cpp"
                    __label__ = 201;break $if_then583$$if_end627$259; //@line 1605 "dictionary.cpp"
                  }
                } while(0);
  
                $v_stress=1; //@line 1590 "dictionary.cpp"
                ; //@line 1591 "dictionary.cpp"
              }
            } while(0);
  
            var $319=$v_stress; //@line 1609 "dictionary.cpp"
            var $cmp628=((($319))|0)==0; //@line 1609 "dictionary.cpp"
            var $320=$v_stress; //@line 1609 "dictionary.cpp"
            var $cmp630=((($320))|0) > 1; //@line 1609 "dictionary.cpp"
            var $or_cond2=($cmp628) | ($cmp630); //@line 1609 "dictionary.cpp"
            if ($or_cond2) { __label__ = 202;; } else { __label__ = 203;; } //@line 1609 "dictionary.cpp"
            if (__label__ == 202) {
  
              var $321=$v_stress; //@line 1610 "dictionary.cpp"
              var $arrayidx632=((__ZL15stress_phonemes+$321)&4294967295); //@line 1610 "dictionary.cpp"
              var $322=HEAP8[($arrayidx632)]; //@line 1610 "dictionary.cpp"
              var $323=$output_addr; //@line 1610 "dictionary.cpp"
              var $incdec_ptr633=(($323+1)&4294967295); //@line 1610 "dictionary.cpp"
              $output_addr=$incdec_ptr633; //@line 1610 "dictionary.cpp"
              HEAP8[($323)]=$322; //@line 1610 "dictionary.cpp"
              ; //@line 1610 "dictionary.cpp"
            }
  
            var $324=$v; //@line 1613 "dictionary.cpp"
            var $arrayidx635=(($vowel_stress+$324)&4294967295); //@line 1613 "dictionary.cpp"
            var $325=HEAP8[($arrayidx635)]; //@line 1613 "dictionary.cpp"
            var $conv636=(tempInt=(($325)),(tempInt>=128?tempInt-256:tempInt)); //@line 1613 "dictionary.cpp"
            var $326=$max_stress; //@line 1613 "dictionary.cpp"
            var $cmp637=((($conv636))|0) > ((($326))|0); //@line 1613 "dictionary.cpp"
            if ($cmp637) { __label__ = 204;; } else { __label__ = 205;; } //@line 1613 "dictionary.cpp"
            if (__label__ == 204) {
  
              var $327=$v; //@line 1615 "dictionary.cpp"
              var $arrayidx639=(($vowel_stress+$327)&4294967295); //@line 1615 "dictionary.cpp"
              var $328=HEAP8[($arrayidx639)]; //@line 1615 "dictionary.cpp"
              var $conv640=(tempInt=(($328)),(tempInt>=128?tempInt-256:tempInt)); //@line 1615 "dictionary.cpp"
              $max_stress=$conv640; //@line 1615 "dictionary.cpp"
              ; //@line 1616 "dictionary.cpp"
            }
  
            var $329=$p; //@line 1618 "dictionary.cpp"
            var $330=HEAPU8[($329)]; //@line 1618 "dictionary.cpp"
            var $conv642=((($330))&255); //@line 1618 "dictionary.cpp"
            var $cmp643=((($conv642))|0)==12; //@line 1618 "dictionary.cpp"
            if ($cmp643) { __label__ = 206;; } else { __label__ = 214;; } //@line 1618 "dictionary.cpp"
            $land_lhs_true644$$if_end665$286: do { 
              if (__label__ == 206) {
  
                var $331=$tr_addr; //@line 1618 "dictionary.cpp"
                var $langopts645=(($331)&4294967295); //@line 1618 "dictionary.cpp"
                var $param=(($langopts645+24)&4294967295); //@line 1618 "dictionary.cpp"
                var $arrayidx646=(($param+8)&4294967295); //@line 1618 "dictionary.cpp"
                var $332=HEAP32[(($arrayidx646)>>2)]; //@line 1618 "dictionary.cpp"
                $opt_length=$332; //@line 1618 "dictionary.cpp"
                var $and647=($332) & 1; //@line 1618 "dictionary.cpp"
                var $tobool648=((($and647))|0)!=0; //@line 1618 "dictionary.cpp"
                if (!($tobool648)) { __label__ = 214;break $land_lhs_true644$$if_end665$286; } //@line 1618 "dictionary.cpp"
  
                $shorten=0; //@line 1621 "dictionary.cpp"
                var $333=$opt_length; //@line 1623 "dictionary.cpp"
                var $and650=($333) & 16; //@line 1623 "dictionary.cpp"
                var $tobool651=((($and650))|0)!=0; //@line 1623 "dictionary.cpp"
                if ($tobool651) { __label__ = 208;; } else { __label__ = 210;; } //@line 1623 "dictionary.cpp"
                $if_then652$$if_else656$289: do { 
                  if (__label__ == 208) {
  
                    var $334=$v; //@line 1626 "dictionary.cpp"
                    var $335=$max_stress_posn; //@line 1626 "dictionary.cpp"
                    var $cmp653=((($334))|0)!=((($335))|0); //@line 1626 "dictionary.cpp"
                    if (!($cmp653)) { __label__ = 212;break $if_then652$$if_else656$289; } //@line 1626 "dictionary.cpp"
  
                    $shorten=1; //@line 1627 "dictionary.cpp"
                    __label__ = 213;break $if_then652$$if_else656$289; //@line 1627 "dictionary.cpp"
                  }
                  else if (__label__ == 210) {
  
                    var $336=$v_stress; //@line 1630 "dictionary.cpp"
                    var $cmp657=((($336))|0) < 4; //@line 1630 "dictionary.cpp"
                    if (!($cmp657)) { __label__ = 212;break $if_then652$$if_else656$289; } //@line 1630 "dictionary.cpp"
  
                    $shorten=1; //@line 1633 "dictionary.cpp"
                    __label__ = 213;break $if_then652$$if_else656$289; //@line 1634 "dictionary.cpp"
                  }
                } while(0);
                if (__label__ == 212) {
  
                  var $_pr11=$shorten; //@line 1636 "dictionary.cpp"
                  var $tobool661=((($_pr11))|0)!=0; //@line 1636 "dictionary.cpp"
                  if (!($tobool661)) { __label__ = 214;break $land_lhs_true644$$if_end665$286; } //@line 1636 "dictionary.cpp"
                }
  
                var $337=$p; //@line 1637 "dictionary.cpp"
                var $incdec_ptr663=(($337+1)&4294967295); //@line 1637 "dictionary.cpp"
                $p=$incdec_ptr663; //@line 1637 "dictionary.cpp"
                ; //@line 1637 "dictionary.cpp"
              }
            } while(0);
  
            var $338=$v_stress; //@line 1640 "dictionary.cpp"
            var $cmp666=((($338))|0) >= 4; //@line 1640 "dictionary.cpp"
            if ($cmp666) { __label__ = 215;; } else { __label__ = 219;; } //@line 1640 "dictionary.cpp"
            $land_lhs_true667$$if_end682$298: do { 
              if (__label__ == 215) {
  
                var $339=$tr_addr; //@line 1640 "dictionary.cpp"
                var $langopts668=(($339)&4294967295); //@line 1640 "dictionary.cpp"
                var $param669=(($langopts668+24)&4294967295); //@line 1640 "dictionary.cpp"
                var $arrayidx670=(($param669+8)&4294967295); //@line 1640 "dictionary.cpp"
                var $340=HEAP32[(($arrayidx670)>>2)]; //@line 1640 "dictionary.cpp"
                var $cmp671=((($340))|0)==2; //@line 1640 "dictionary.cpp"
                if (!($cmp671)) { __label__ = 219;break $land_lhs_true667$$if_end682$298; } //@line 1640 "dictionary.cpp"
  
                var $341=$v; //@line 1643 "dictionary.cpp"
                var $342=HEAP32[(($vowel_count)>>2)]; //@line 1643 "dictionary.cpp"
                var $sub673=((($342)-2)&4294967295); //@line 1643 "dictionary.cpp"
                var $cmp674=((($341))|0)==((($sub673))|0); //@line 1643 "dictionary.cpp"
                if (!($cmp674)) { __label__ = 219;break $land_lhs_true667$$if_end682$298; } //@line 1643 "dictionary.cpp"
  
                var $343=$v; //@line 1643 "dictionary.cpp"
                var $arrayidx676=(($syllable_weight+$343)&4294967295); //@line 1643 "dictionary.cpp"
                var $344=HEAP8[($arrayidx676)]; //@line 1643 "dictionary.cpp"
                var $conv677=(tempInt=(($344)),(tempInt>=128?tempInt-256:tempInt)); //@line 1643 "dictionary.cpp"
                var $cmp678=((($conv677))|0)==0; //@line 1643 "dictionary.cpp"
                if (!($cmp678)) { __label__ = 219;break $land_lhs_true667$$if_end682$298; } //@line 1643 "dictionary.cpp"
  
                var $345=$phcode; //@line 1645 "dictionary.cpp"
                var $346=$output_addr; //@line 1645 "dictionary.cpp"
                var $incdec_ptr680=(($346+1)&4294967295); //@line 1645 "dictionary.cpp"
                $output_addr=$incdec_ptr680; //@line 1645 "dictionary.cpp"
                HEAP8[($346)]=$345; //@line 1645 "dictionary.cpp"
                $phcode=12; //@line 1646 "dictionary.cpp"
                ; //@line 1647 "dictionary.cpp"
              }
            } while(0);
  
            var $347=$v; //@line 1650 "dictionary.cpp"
            var $inc683=((($347)+1)&4294967295); //@line 1650 "dictionary.cpp"
            $v=$inc683; //@line 1650 "dictionary.cpp"
            ; //@line 1651 "dictionary.cpp"
          }
        } while(0);
  
        var $348=$phcode; //@line 1653 "dictionary.cpp"
        var $conv686=((($348))&255); //@line 1653 "dictionary.cpp"
        var $cmp687=((($conv686))|0)!=1; //@line 1653 "dictionary.cpp"
        if (!($cmp687)) { __label__ = 175;continue $while_cond546$243; } //@line 1653 "dictionary.cpp"
  
        var $349=$phcode; //@line 1654 "dictionary.cpp"
        var $350=$output_addr; //@line 1654 "dictionary.cpp"
        var $incdec_ptr689=(($350+1)&4294967295); //@line 1654 "dictionary.cpp"
        $output_addr=$incdec_ptr689; //@line 1654 "dictionary.cpp"
        HEAP8[($350)]=$349; //@line 1654 "dictionary.cpp"
        __label__ = 175;continue $while_cond546$243; //@line 1654 "dictionary.cpp"
      }
  
      var $351=$output_addr; //@line 1656 "dictionary.cpp"
      var $incdec_ptr692=(($351+1)&4294967295); //@line 1656 "dictionary.cpp"
      $output_addr=$incdec_ptr692; //@line 1656 "dictionary.cpp"
      HEAP8[($351)]=0; //@line 1656 "dictionary.cpp"
      ; //@line 1658 "dictionary.cpp"
    }
  
    STACKTOP = __stackBase__;
    return; //@line 1659 "dictionary.cpp"
    return;
  }
  

  function __Z14AppendPhonemesP10TranslatorPciPKc($tr, $string, $size, $ph) {
    ;
    var __label__;
  
    var $tr_addr;
    var $string_addr;
    var $size_addr;
    var $ph_addr;
    var $p;
    var $c;
    var $unstress_mark;
    var $length;
    $tr_addr=$tr;
    $string_addr=$string;
    $size_addr=$size;
    $ph_addr=$ph;
    var $0=$ph_addr; //@line 1683 "dictionary.cpp"
    var $call=_strlen($0); //@line 1683 "dictionary.cpp"
    var $1=$string_addr; //@line 1683 "dictionary.cpp"
    var $call1=_strlen($1); //@line 1683 "dictionary.cpp"
    var $add=((($call1)+($call))&4294967295); //@line 1683 "dictionary.cpp"
    $length=$add; //@line 1683 "dictionary.cpp"
    var $2=$length; //@line 1684 "dictionary.cpp"
    var $3=$size_addr; //@line 1684 "dictionary.cpp"
    var $cmp=((($2))|0) >= ((($3))|0); //@line 1684 "dictionary.cpp"
    if ($cmp) { __label__ = 14;; } else { __label__ = 1;; } //@line 1684 "dictionary.cpp"
    $if_end34$$if_end$2: do { 
      if (__label__ == 1) {
  
        $unstress_mark=0; //@line 1690 "dictionary.cpp"
        var $4=$ph_addr; //@line 1691 "dictionary.cpp"
        $p=$4; //@line 1691 "dictionary.cpp"
        var $5=$p; //@line 1692 "dictionary.cpp"
        var $incdec_ptr1=(($5+1)&4294967295); //@line 1692 "dictionary.cpp"
        $p=$incdec_ptr1; //@line 1692 "dictionary.cpp"
        var $6=HEAPU8[($5)]; //@line 1692 "dictionary.cpp"
        $c=$6; //@line 1692 "dictionary.cpp"
        var $conv2=((($6))&255); //@line 1692 "dictionary.cpp"
        var $cmp23=((($conv2))|0)!=0; //@line 1692 "dictionary.cpp"
        if ($cmp23) { __label__ = 2;; } else { __label__ = 12;; } //@line 1692 "dictionary.cpp"
        $while_body$$while_end$4: do { 
          if (__label__ == 2) {
            $while_body$5: while(1) { 
  
              var $7=$c; //@line 1694 "dictionary.cpp"
              var $conv3=((($7))&255); //@line 1694 "dictionary.cpp"
              var $8=HEAP32[((_n_phoneme_tab)>>2)]; //@line 1694 "dictionary.cpp"
              var $cmp4=((($conv3))|0) >= ((($8))|0); //@line 1694 "dictionary.cpp"
              if ($cmp4) { __label__ = 3;; } else { __label__ = 4;; } //@line 1694 "dictionary.cpp"
              $while_cond_backedge$$if_end6$7: do { 
                if (__label__ == 4) {
  
                  var $11=$c; //@line 1696 "dictionary.cpp"
                  var $idxprom=((($11))&255); //@line 1696 "dictionary.cpp"
                  var $arrayidx=((_phoneme_tab+$idxprom*4)&4294967295); //@line 1696 "dictionary.cpp"
                  var $12=HEAP32[(($arrayidx)>>2)]; //@line 1696 "dictionary.cpp"
                  var $type=(($12+11)&4294967295); //@line 1696 "dictionary.cpp"
                  var $13=HEAPU8[($type)]; //@line 1696 "dictionary.cpp"
                  var $conv7=((($13))&255); //@line 1696 "dictionary.cpp"
                  var $cmp8=((($conv7))|0)==1; //@line 1696 "dictionary.cpp"
                  var $14=$c; //@line 1698 "dictionary.cpp"
                  var $idxprom10=((($14))&255); //@line 1698 "dictionary.cpp"
                  var $arrayidx11=((_phoneme_tab+$idxprom10*4)&4294967295); //@line 1698 "dictionary.cpp"
                  var $15=HEAP32[(($arrayidx11)>>2)]; //@line 1698 "dictionary.cpp"
                  if ($cmp8) { __label__ = 5;; } else { __label__ = 7;; } //@line 1696 "dictionary.cpp"
                  if (__label__ == 5) {
  
                    var $std_length=(($15+14)&4294967295); //@line 1698 "dictionary.cpp"
                    var $16=HEAPU8[($std_length)]; //@line 1698 "dictionary.cpp"
                    var $conv12=((($16))&255); //@line 1698 "dictionary.cpp"
                    var $cmp13=((($conv12))|0) < 4; //@line 1698 "dictionary.cpp"
                    if (!($cmp13)) { __label__ = 3;break $while_cond_backedge$$if_end6$7; } //@line 1698 "dictionary.cpp"
  
                    $unstress_mark=1; //@line 1699 "dictionary.cpp"
                    ; //@line 1699 "dictionary.cpp"
                  }
                  else if (__label__ == 7) {
  
                    var $type18=(($15+11)&4294967295); //@line 1703 "dictionary.cpp"
                    var $17=HEAPU8[($type18)]; //@line 1703 "dictionary.cpp"
                    var $conv19=((($17))&255); //@line 1703 "dictionary.cpp"
                    var $cmp20=((($conv19))|0)==2; //@line 1703 "dictionary.cpp"
                    if (!($cmp20)) { __label__ = 3;break $while_cond_backedge$$if_end6$7; } //@line 1703 "dictionary.cpp"
  
                    var $18=$c; //@line 1705 "dictionary.cpp"
                    var $idxprom22=((($18))&255); //@line 1705 "dictionary.cpp"
                    var $arrayidx23=((_phoneme_tab+$idxprom22*4)&4294967295); //@line 1705 "dictionary.cpp"
                    var $19=HEAP32[(($arrayidx23)>>2)]; //@line 1705 "dictionary.cpp"
                    var $phflags=(($19+4)&4294967295); //@line 1705 "dictionary.cpp"
                    var $20=HEAP32[(($phflags)>>2)]; //@line 1705 "dictionary.cpp"
                    var $and=($20) & 2; //@line 1705 "dictionary.cpp"
                    var $cmp24=((($and))|0)==0; //@line 1705 "dictionary.cpp"
                    if ($cmp24) { __label__ = 9;; } else { __label__ = 11;; } //@line 1705 "dictionary.cpp"
                    $land_lhs_true$$if_end27$14: do { 
                      if (__label__ == 9) {
  
                        var $21=$unstress_mark; //@line 1705 "dictionary.cpp"
                        var $cmp25=((($21))|0)==0; //@line 1705 "dictionary.cpp"
                        if (!($cmp25)) { __label__ = 11;break $land_lhs_true$$if_end27$14; } //@line 1705 "dictionary.cpp"
  
                        var $22=$tr_addr; //@line 1708 "dictionary.cpp"
                        var $word_stressed_count=(($22+8300)&4294967295); //@line 1708 "dictionary.cpp"
                        var $23=HEAP32[(($word_stressed_count)>>2)]; //@line 1708 "dictionary.cpp"
                        var $inc=((($23)+1)&4294967295); //@line 1708 "dictionary.cpp"
                        HEAP32[(($word_stressed_count)>>2)]=$inc; //@line 1708 "dictionary.cpp"
                        ; //@line 1709 "dictionary.cpp"
                      }
                    } while(0);
  
                    $unstress_mark=0; //@line 1710 "dictionary.cpp"
                    var $24=$tr_addr; //@line 1711 "dictionary.cpp"
                    var $word_vowel_count=(($24+8296)&4294967295); //@line 1711 "dictionary.cpp"
                    var $25=HEAP32[(($word_vowel_count)>>2)]; //@line 1711 "dictionary.cpp"
                    var $inc28=((($25)+1)&4294967295); //@line 1711 "dictionary.cpp"
                    HEAP32[(($word_vowel_count)>>2)]=$inc28; //@line 1711 "dictionary.cpp"
                    ; //@line 1712 "dictionary.cpp"
                  }
                }
              } while(0);
  
              var $9=$p; //@line 1692 "dictionary.cpp"
              var $incdec_ptr=(($9+1)&4294967295); //@line 1692 "dictionary.cpp"
              $p=$incdec_ptr; //@line 1692 "dictionary.cpp"
              var $10=HEAPU8[($9)]; //@line 1692 "dictionary.cpp"
              $c=$10; //@line 1692 "dictionary.cpp"
              var $conv=((($10))&255); //@line 1692 "dictionary.cpp"
              var $cmp2=((($conv))|0)!=0; //@line 1692 "dictionary.cpp"
              if ($cmp2) { __label__ = 2;continue $while_body$5; } else { __label__ = 12;break $while_body$$while_end$4; } //@line 1692 "dictionary.cpp"
            }
          }
        } while(0);
  
        var $26=$string_addr; //@line 1716 "dictionary.cpp"
        var $cmp31=((($26))|0)!=0; //@line 1716 "dictionary.cpp"
        if (!($cmp31)) { __label__ = 14;break $if_end34$$if_end$2; } //@line 1716 "dictionary.cpp"
  
        var $27=$string_addr; //@line 1717 "dictionary.cpp"
        var $28=$ph_addr; //@line 1717 "dictionary.cpp"
        var $call33=_strcat($27, $28); //@line 1717 "dictionary.cpp"
        ; //@line 1717 "dictionary.cpp"
      }
    } while(0);
  
    ;
    return; //@line 1718 "dictionary.cpp"
    return;
  }
  

  function __Z14TranslateRulesP10TranslatorPcS1_iS1_iPj($tr, $p_start, $phonemes, $ph_size, $end_phonemes, $word_flags, $dict_flags) {
    var __stackBase__  = STACKTOP; STACKTOP += 416; _memset(__stackBase__, 0, 416);
    var __label__;
    var __lastLabel__ = null;
  
    var $retval;
    var $tr_addr;
    var $p_start_addr;
    var $phonemes_addr;
    var $ph_size_addr;
    var $end_phonemes_addr;
    var $word_flags_addr;
    var $dict_flags_addr;
    var $c;
    var $c2;
    var $c12;
    var $c123;
    var $wc=__stackBase__;
    var $wc_prev;
    var $wc_bytes;
    var $p2=__stackBase__+4;
    var $found;
    var $g;
    var $g1;
    var $n;
    var $letter=__stackBase__+8;
    var $any_alpha;
    var $ix;
    var $digit_count;
    var $p=__stackBase__+12;
    var $dict_flags0;
    var $match1=__stackBase__+16;
    var $match2=__stackBase__+32;
    var $ph_buf=__stackBase__+48;
    var $word_copy=__stackBase__+88;
    var $wordbuf=__stackBase__+248;
    var $ix14;
    var $string=__stackBase__+368;
    var $buf=__stackBase__+376;
    $tr_addr=$tr;
    $p_start_addr=$p_start;
    $phonemes_addr=$phonemes;
    $ph_size_addr=$ph_size;
    $end_phonemes_addr=$end_phonemes;
    $word_flags_addr=$word_flags;
    $dict_flags_addr=$dict_flags;
    HEAP32[(($wc)>>2)]=0; //@line 2393 "dictionary.cpp"
    $any_alpha=0; //@line 2402 "dictionary.cpp"
    $digit_count=0; //@line 2404 "dictionary.cpp"
    $dict_flags0=0; //@line 2406 "dictionary.cpp"
    var $0=$tr_addr; //@line 2413 "dictionary.cpp"
    var $data_dictrules=(($0+1048)&4294967295); //@line 2413 "dictionary.cpp"
    var $1=HEAP32[(($data_dictrules)>>2)]; //@line 2413 "dictionary.cpp"
    var $cmp=((($1))|0)==0; //@line 2413 "dictionary.cpp"
    if ($cmp) { __label__ = 1;; } else { __label__ = 2;; } //@line 2413 "dictionary.cpp"
    $if_then$$if_end$2: do { 
      if (__label__ == 1) {
  
        $retval=0; //@line 2414 "dictionary.cpp"
        ; //@line 2414 "dictionary.cpp"
      }
      else if (__label__ == 2) {
  
        var $2=$dict_flags_addr; //@line 2416 "dictionary.cpp"
        var $cmp1=((($2))|0)!=0; //@line 2416 "dictionary.cpp"
        if ($cmp1) { __label__ = 3;; } else { __label__ = 4;; } //@line 2416 "dictionary.cpp"
        if (__label__ == 3) {
  
          var $3=$dict_flags_addr; //@line 2417 "dictionary.cpp"
          var $arrayidx=(($3)&4294967295); //@line 2417 "dictionary.cpp"
          var $4=HEAP32[(($arrayidx)>>2)]; //@line 2417 "dictionary.cpp"
          $dict_flags0=$4; //@line 2417 "dictionary.cpp"
          ; //@line 2417 "dictionary.cpp"
        }
  
        $ix=0; //@line 2419 "dictionary.cpp"
        __lastLabel__ = 4; ; //@line 2419 "dictionary.cpp"
        $for_cond$8: while(1) { 
  
          var $5=__lastLabel__ == 5 ? $_pr : (0); //@line 2419 "dictionary.cpp"
          var $cmp4=((($5))|0) < 159; //@line 2419 "dictionary.cpp"
          if (!($cmp4)) { __label__ = 8;break $for_cond$8; } //@line 2419 "dictionary.cpp"
  
          var $6=$ix; //@line 2421 "dictionary.cpp"
          var $7=$p_start_addr; //@line 2421 "dictionary.cpp"
          var $arrayidx5=(($7+$6)&4294967295); //@line 2421 "dictionary.cpp"
          var $8=HEAP8[($arrayidx5)]; //@line 2421 "dictionary.cpp"
          $c=$8; //@line 2421 "dictionary.cpp"
          var $9=$c; //@line 2422 "dictionary.cpp"
          var $10=$ix; //@line 2422 "dictionary.cpp"
          var $inc=((($10)+1)&4294967295); //@line 2422 "dictionary.cpp"
          $ix=$inc; //@line 2422 "dictionary.cpp"
          var $arrayidx6=(($word_copy+$10)&4294967295); //@line 2422 "dictionary.cpp"
          HEAP8[($arrayidx6)]=$9; //@line 2422 "dictionary.cpp"
          var $11=$c; //@line 2423 "dictionary.cpp"
          var $conv=((($11))&255); //@line 2423 "dictionary.cpp"
          var $cmp7=((($conv))|0)==0; //@line 2423 "dictionary.cpp"
          if ($cmp7) { __label__ = 8;break $for_cond$8; } //@line 2423 "dictionary.cpp"
  
          var $_pr=$ix; //@line 2419 "dictionary.cpp"
          __lastLabel__ = 5; __label__ = 6;continue $for_cond$8;
        }
  
        var $12=$ix; //@line 2426 "dictionary.cpp"
        var $arrayidx10=(($word_copy+$12)&4294967295); //@line 2426 "dictionary.cpp"
        HEAP8[($arrayidx10)]=0; //@line 2426 "dictionary.cpp"
        var $13=HEAP32[((_option_phonemes)>>2)]; //@line 2429 "dictionary.cpp"
        var $cmp11=((($13))|0)==2; //@line 2429 "dictionary.cpp"
        if ($cmp11) { __label__ = 9;; } else { __label__ = 17;; } //@line 2429 "dictionary.cpp"
        $land_lhs_true$$if_end31$13: do { 
          if (__label__ == 9) {
  
            var $14=$word_flags_addr; //@line 2429 "dictionary.cpp"
            var $and=($14) & 268435456; //@line 2429 "dictionary.cpp"
            var $cmp12=((($and))|0)==0; //@line 2429 "dictionary.cpp"
            if (!($cmp12)) { __label__ = 17;break $land_lhs_true$$if_end31$13; } //@line 2429 "dictionary.cpp"
  
            $ix14=0; //@line 2434 "dictionary.cpp"
            ; //@line 2434 "dictionary.cpp"
            while(1) { 
  
              var $15=$ix14; //@line 2434 "dictionary.cpp"
              var $16=$p_start_addr; //@line 2434 "dictionary.cpp"
              var $arrayidx16=(($16+$15)&4294967295); //@line 2434 "dictionary.cpp"
              var $17=HEAPU8[($arrayidx16)]; //@line 2434 "dictionary.cpp"
              $c=$17; //@line 2434 "dictionary.cpp"
              var $conv17=((($17))&255); //@line 2434 "dictionary.cpp"
              var $cmp18=((($conv17))|0)!=32; //@line 2434 "dictionary.cpp"
              if (!($cmp18)) { __label__ = 14;break ; } //@line 2434 "dictionary.cpp"
  
              var $18=$c; //@line 2434 "dictionary.cpp"
              var $conv19=((($18))&255); //@line 2434 "dictionary.cpp"
              var $cmp20=((($conv19))|0)!=0; //@line 2434 "dictionary.cpp"
              if (!($cmp20)) { __label__ = 14;break ; }
  
              var $19=$c; //@line 2436 "dictionary.cpp"
              var $20=$ix14; //@line 2436 "dictionary.cpp"
              var $arrayidx22=(($wordbuf+$20)&4294967295); //@line 2436 "dictionary.cpp"
              HEAP8[($arrayidx22)]=$19; //@line 2436 "dictionary.cpp"
              var $21=$ix14; //@line 2434 "dictionary.cpp"
              var $inc23=((($21)+1)&4294967295); //@line 2434 "dictionary.cpp"
              $ix14=$inc23; //@line 2434 "dictionary.cpp"
              __label__ = 11;continue ; //@line 2434 "dictionary.cpp"
            }
  
            var $22=$ix14; //@line 2438 "dictionary.cpp"
            var $arrayidx25=(($wordbuf+$22)&4294967295); //@line 2438 "dictionary.cpp"
            HEAP8[($arrayidx25)]=0; //@line 2438 "dictionary.cpp"
            var $23=$word_flags_addr; //@line 2439 "dictionary.cpp"
            var $and26=($23) & -2147483648; //@line 2439 "dictionary.cpp"
            var $tobool=((($and26))|0)!=0; //@line 2439 "dictionary.cpp"
            var $24=HEAP32[((_f_trans)>>2)]; //@line 2440 "dictionary.cpp"
            var $arraydecay=(($wordbuf)&4294967295); //@line 2440 "dictionary.cpp"
            if ($tobool) { __label__ = 15;; } else { __label__ = 16;; } //@line 2439 "dictionary.cpp"
            if (__label__ == 15) {
  
              var $call=_fprintf($24, ((__str10126)&4294967295), allocate([$arraydecay,0,0,0], ["i8*",0,0,0], ALLOC_STACK)); //@line 2440 "dictionary.cpp"
              ; //@line 2440 "dictionary.cpp"
            }
            else if (__label__ == 16) {
  
              var $call29=_fprintf($24, ((__str11127)&4294967295), allocate([$arraydecay,0,0,0], ["i8*",0,0,0], ALLOC_STACK)); //@line 2442 "dictionary.cpp"
              ;
            }
          }
        } while(0);
  
        var $25=$p_start_addr; //@line 2445 "dictionary.cpp"
        HEAP32[(($p)>>2)]=$25; //@line 2445 "dictionary.cpp"
        var $26=$tr_addr; //@line 2446 "dictionary.cpp"
        var $word_vowel_count=(($26+8296)&4294967295); //@line 2446 "dictionary.cpp"
        HEAP32[(($word_vowel_count)>>2)]=0; //@line 2446 "dictionary.cpp"
        var $27=$tr_addr; //@line 2447 "dictionary.cpp"
        var $word_stressed_count=(($27+8300)&4294967295); //@line 2447 "dictionary.cpp"
        HEAP32[(($word_stressed_count)>>2)]=0; //@line 2447 "dictionary.cpp"
        var $28=$end_phonemes_addr; //@line 2449 "dictionary.cpp"
        var $cmp32=((($28))|0)!=0; //@line 2449 "dictionary.cpp"
        if ($cmp32) { __label__ = 18;; } else { __label__ = 19;; } //@line 2449 "dictionary.cpp"
        if (__label__ == 18) {
  
          var $29=$end_phonemes_addr; //@line 2450 "dictionary.cpp"
          var $arrayidx34=(($29)&4294967295); //@line 2450 "dictionary.cpp"
          HEAP8[($arrayidx34)]=0; //@line 2450 "dictionary.cpp"
          ; //@line 2450 "dictionary.cpp"
        }
  
        var $arrayidx57=(($string)&4294967295); //@line 2465 "dictionary.cpp"
        var $arrayidx58=(($string+1)&4294967295); //@line 2466 "dictionary.cpp"
        var $arraydecay60=(($string)&4294967295); //@line 2468 "dictionary.cpp"
        var $arraydecay61=(($buf)&4294967295); //@line 2468 "dictionary.cpp"
        var $arraydecay66=(($buf)&4294967295); //@line 2471 "dictionary.cpp"
        var $arraydecay69=(($buf)&4294967295); //@line 2474 "dictionary.cpp"
        var $phonemes282=(($match1+4)&4294967295); //@line 2644 "dictionary.cpp"
        var $phonemes285=(($match1+4)&4294967295); //@line 2645 "dictionary.cpp"
        var $points287=(($match1)&4294967295); //@line 2647 "dictionary.cpp"
        var $phonemes295=(($match1+4)&4294967295); //@line 2652 "dictionary.cpp"
        var $end_type306=(($match1+8)&4294967295); //@line 2659 "dictionary.cpp"
        var $end_type313=(($match1+8)&4294967295); //@line 2664 "dictionary.cpp"
        var $del_fwd=(($match1+12)&4294967295); //@line 2680 "dictionary.cpp"
        var $del_fwd342=(($match1+12)&4294967295); //@line 2681 "dictionary.cpp"
        var $phonemes344=(($match1+4)&4294967295); //@line 2682 "dictionary.cpp"
        var $points=(($match2)&4294967295); //@line 2508 "dictionary.cpp"
        var $points107=(($match2)&4294967295); //@line 2509 "dictionary.cpp"
        var $points112=(($match2)&4294967295); //@line 2514 "dictionary.cpp"
        var $points113=(($match1)&4294967295); //@line 2514 "dictionary.cpp"
        var $30=$match1; //@line 2517 "dictionary.cpp"
        var $31=$match2; //@line 2517 "dictionary.cpp"
        var $points241=(($match1)&4294967295); //@line 2608 "dictionary.cpp"
        var $arraydecay266=(($ph_buf)&4294967295); //@line 2628 "dictionary.cpp"
        var $arrayidx267=(($ph_buf)&4294967295); //@line 2629 "dictionary.cpp"
        var $arraydecay270=(($ph_buf)&4294967295); //@line 2631 "dictionary.cpp"
        var $phonemes271=(($match1+4)&4294967295); //@line 2631 "dictionary.cpp"
        var $points272=(($match1)&4294967295); //@line 2632 "dictionary.cpp"
        var $points135=(($match1)&4294967295); //@line 2534 "dictionary.cpp"
        var $arraydecay230=(($ph_buf)&4294967295); //@line 2600 "dictionary.cpp"
        var $arraydecay232=(($ph_buf)&4294967295); //@line 2601 "dictionary.cpp"
        var $phonemes233=(($match1+4)&4294967295); //@line 2601 "dictionary.cpp"
        var $points234=(($match1)&4294967295); //@line 2602 "dictionary.cpp"
        ; //@line 2452 "dictionary.cpp"
        $while_cond$28: while(1) { 
  
          var $32=HEAP32[(($p)>>2)]; //@line 2452 "dictionary.cpp"
          var $33=HEAPU8[($32)]; //@line 2452 "dictionary.cpp"
          $c=$33; //@line 2452 "dictionary.cpp"
          var $conv36=((($33))&255); //@line 2452 "dictionary.cpp"
          var $cmp37=((($conv36))|0)!=32; //@line 2452 "dictionary.cpp"
          if (!($cmp37)) { __label__ = 104;break $while_cond$28; } //@line 2452 "dictionary.cpp"
  
          var $34=$c; //@line 2452 "dictionary.cpp"
          var $conv39=((($34))&255); //@line 2452 "dictionary.cpp"
          var $cmp40=((($conv39))|0)!=0; //@line 2452 "dictionary.cpp"
          if (!($cmp40)) { __label__ = 104;break $while_cond$28; }
  
          var $35=HEAP32[(($wc)>>2)]; //@line 2454 "dictionary.cpp"
          $wc_prev=$35; //@line 2454 "dictionary.cpp"
          var $36=HEAP32[(($p)>>2)]; //@line 2455 "dictionary.cpp"
          var $call42=__Z7utf8_inPiPKc($wc, $36); //@line 2455 "dictionary.cpp"
          $wc_bytes=$call42; //@line 2455 "dictionary.cpp"
          var $37=HEAP32[(($wc)>>2)]; //@line 2456 "dictionary.cpp"
          var $call43=__Z7IsAlphaj($37); //@line 2456 "dictionary.cpp"
          var $tobool44=((($call43))|0)!=0; //@line 2456 "dictionary.cpp"
          if ($tobool44) { __label__ = 23;; } else { __label__ = 24;; } //@line 2456 "dictionary.cpp"
          if (__label__ == 23) {
  
            var $38=$any_alpha; //@line 2457 "dictionary.cpp"
            var $inc46=((($38)+1)&4294967295); //@line 2457 "dictionary.cpp"
            $any_alpha=$inc46; //@line 2457 "dictionary.cpp"
            ; //@line 2457 "dictionary.cpp"
          }
  
          var $39=$c; //@line 2459 "dictionary.cpp"
          var $idxprom=((($39))&255); //@line 2459 "dictionary.cpp"
          var $40=$tr_addr; //@line 2459 "dictionary.cpp"
          var $groups2_count=(($40+7756)&4294967295); //@line 2459 "dictionary.cpp"
          var $arrayidx48=(($groups2_count+$idxprom)&4294967295); //@line 2459 "dictionary.cpp"
          var $41=HEAPU8[($arrayidx48)]; //@line 2459 "dictionary.cpp"
          var $conv49=((($41))&255); //@line 2459 "dictionary.cpp"
          $n=$conv49; //@line 2459 "dictionary.cpp"
          var $42=HEAP32[(($wc)>>2)]; //@line 2460 "dictionary.cpp"
          var $call50=__Z7IsDigitj($42); //@line 2460 "dictionary.cpp"
          var $tobool51=((($call50))|0)!=0; //@line 2460 "dictionary.cpp"
          if ($tobool51) { __label__ = 25;; } else { __label__ = 30;; } //@line 2460 "dictionary.cpp"
          $land_lhs_true52$$if_else70$35: do { 
            if (__label__ == 25) {
  
              var $43=$tr_addr; //@line 2460 "dictionary.cpp"
              var $langopts=(($43)&4294967295); //@line 2460 "dictionary.cpp"
              var $tone_numbers=(($langopts+250)&4294967295); //@line 2460 "dictionary.cpp"
              var $44=HEAP8[($tone_numbers)]; //@line 2460 "dictionary.cpp"
              var $conv53=(tempInt=(($44)),(tempInt>=128?tempInt-256:tempInt)); //@line 2460 "dictionary.cpp"
              var $cmp54=((($conv53))|0)==0; //@line 2460 "dictionary.cpp"
              if ($cmp54) { __label__ = 27;; } else { __label__ = 26;; } //@line 2460 "dictionary.cpp"
              if (__label__ == 26) {
  
                var $45=$any_alpha; //@line 2460 "dictionary.cpp"
                var $tobool55=((($45))|0)!=0; //@line 2460 "dictionary.cpp"
                if ($tobool55) { __label__ = 30;break $land_lhs_true52$$if_else70$35; } //@line 2460 "dictionary.cpp"
              }
  
              HEAP8[($arrayidx57)]=95; //@line 2465 "dictionary.cpp"
              var $46=HEAP32[(($p)>>2)]; //@line 2466 "dictionary.cpp"
              var $47=$wc_bytes; //@line 2466 "dictionary.cpp"
              var $src$, $dest$, $stop$, $stop4$, $fast$;
              $src$ = $46;
              $dest$ = $arrayidx58;
              $stop$ = $src$ + $47;
              if (($dest$%4) == ($src$%4) && $47 > 8) {
                while ($src$%4 !== 0 && $src$ < $stop$) {
                  ; HEAP8[$dest$++] = HEAP8[$src$++];
                }
                $src$ >>= 2;
                $dest$ >>= 2;
                $stop4$ = $stop$ >> 2;
                while ($src$ < $stop4$) {
              ;;;    HEAP32[$dest$++] = HEAP32[$src$++];
                }
                $src$ <<= 2;
                $dest$ <<= 2;
              }while ($src$ < $stop$) {
                ; HEAP8[$dest$++] = HEAP8[$src$++];
              }; //@line 2466 "dictionary.cpp"
              var $48=$wc_bytes; //@line 2467 "dictionary.cpp"
              var $add=((($48)+1)&4294967295); //@line 2467 "dictionary.cpp"
              var $arrayidx59=(($string+$add)&4294967295); //@line 2467 "dictionary.cpp"
              HEAP8[($arrayidx59)]=0; //@line 2467 "dictionary.cpp"
              var $49=$tr_addr; //@line 2468 "dictionary.cpp"
              var $call62=__Z6LookupP10TranslatorPKcPc($49, $arraydecay60, $arraydecay61); //@line 2468 "dictionary.cpp"
              var $50=$digit_count; //@line 2469 "dictionary.cpp"
              var $inc63=((($50)+1)&4294967295); //@line 2469 "dictionary.cpp"
              $digit_count=$inc63; //@line 2469 "dictionary.cpp"
              var $cmp64=((($inc63))>>>0) >= 2; //@line 2469 "dictionary.cpp"
              if ($cmp64) { __label__ = 28;; } else { __label__ = 29;; } //@line 2469 "dictionary.cpp"
              if (__label__ == 28) {
  
                var $strlen=_strlen($arraydecay66); //@line 2471 "dictionary.cpp"
                var $endptr=(($arraydecay66+$strlen)&4294967295); //@line 2471 "dictionary.cpp"
                var $src$, $dest$, $stop$, $stop4$, $fast$;
                $src$ = ((__ZZ14TranslateRulesP10TranslatorPcS1_iS1_iPjE9str_pause)&4294967295);
                $dest$ = $endptr;
                $stop$ = $src$ + 2;
                if (($dest$%4) == ($src$%4) && 2 > 8) {
                  while ($src$%4 !== 0 && $src$ < $stop$) {
                    ; HEAP8[$dest$++] = HEAP8[$src$++];
                  }
                  $src$ >>= 2;
                  $dest$ >>= 2;
                  $stop4$ = $stop$ >> 2;
                  while ($src$ < $stop4$) {
                ;;;    HEAP32[$dest$++] = HEAP32[$src$++];
                  }
                  $src$ <<= 2;
                  $dest$ <<= 2;
                }while ($src$ < $stop$) {
                  ; HEAP8[$dest$++] = HEAP8[$src$++];
                }; //@line 2471 "dictionary.cpp"
                $digit_count=0; //@line 2472 "dictionary.cpp"
                ; //@line 2473 "dictionary.cpp"
              }
  
              var $51=$tr_addr; //@line 2474 "dictionary.cpp"
              var $52=$phonemes_addr; //@line 2474 "dictionary.cpp"
              var $53=$ph_size_addr; //@line 2474 "dictionary.cpp"
              __Z14AppendPhonemesP10TranslatorPciPKc($51, $52, $53, $arraydecay69); //@line 2474 "dictionary.cpp"
              var $54=$wc_bytes; //@line 2475 "dictionary.cpp"
              var $55=HEAP32[(($p)>>2)]; //@line 2475 "dictionary.cpp"
              var $add_ptr=(($55+$54)&4294967295); //@line 2475 "dictionary.cpp"
              HEAP32[(($p)>>2)]=$add_ptr; //@line 2475 "dictionary.cpp"
              __label__ = 20;continue $while_cond$28; //@line 2476 "dictionary.cpp"
            }
          } while(0);
  
          $digit_count=0; //@line 2480 "dictionary.cpp"
          $found=0; //@line 2481 "dictionary.cpp"
          var $56=HEAP32[(($wc)>>2)]; //@line 2483 "dictionary.cpp"
          var $57=$tr_addr; //@line 2483 "dictionary.cpp"
          var $letter_bits_offset=(($57+964)&4294967295); //@line 2483 "dictionary.cpp"
          var $58=HEAP32[(($letter_bits_offset)>>2)]; //@line 2483 "dictionary.cpp"
          var $sub=((($56)-($58))&4294967295); //@line 2483 "dictionary.cpp"
          $ix=$sub; //@line 2483 "dictionary.cpp"
          var $cmp71=((($sub))|0) >= 0; //@line 2483 "dictionary.cpp"
          if ($cmp71) { __label__ = 31;; } else { __label__ = 34;; } //@line 2483 "dictionary.cpp"
          $land_lhs_true72$$if_end81$44: do { 
            if (__label__ == 31) {
  
              var $59=$ix; //@line 2483 "dictionary.cpp"
              var $cmp73=((($59))|0) < 128; //@line 2483 "dictionary.cpp"
              if (!($cmp73)) { __label__ = 34;break $land_lhs_true72$$if_end81$44; } //@line 2483 "dictionary.cpp"
  
              var $60=$ix; //@line 2485 "dictionary.cpp"
              var $61=$tr_addr; //@line 2485 "dictionary.cpp"
              var $groups3=(($61+6280)&4294967295); //@line 2485 "dictionary.cpp"
              var $arrayidx75=(($groups3+$60*4)&4294967295); //@line 2485 "dictionary.cpp"
              var $62=HEAP32[(($arrayidx75)>>2)]; //@line 2485 "dictionary.cpp"
              var $cmp76=((($62))|0)!=0; //@line 2485 "dictionary.cpp"
              if (!($cmp76)) { __label__ = 34;break $land_lhs_true72$$if_end81$44; } //@line 2485 "dictionary.cpp"
  
              var $63=$tr_addr; //@line 2487 "dictionary.cpp"
              var $64=$p_start_addr; //@line 2487 "dictionary.cpp"
              var $65=$wc_bytes; //@line 2487 "dictionary.cpp"
              var $66=$ix; //@line 2487 "dictionary.cpp"
              var $67=$tr_addr; //@line 2487 "dictionary.cpp"
              var $groups378=(($67+6280)&4294967295); //@line 2487 "dictionary.cpp"
              var $arrayidx79=(($groups378+$66*4)&4294967295); //@line 2487 "dictionary.cpp"
              var $68=HEAP32[(($arrayidx79)>>2)]; //@line 2487 "dictionary.cpp"
              var $69=$word_flags_addr; //@line 2487 "dictionary.cpp"
              var $70=$dict_flags0; //@line 2487 "dictionary.cpp"
              __ZL9MatchRuleP10TranslatorPPcS1_iS1_P11MatchRecordii($63, $p, $64, $65, $68, $match1, $69, $70); //@line 2487 "dictionary.cpp"
              $found=1; //@line 2488 "dictionary.cpp"
              __label__ = 85;break $land_lhs_true72$$if_end81$44; //@line 2524 "dictionary.cpp"
            }
          } while(0);
          $if_end281$$if_end81$48: do { 
            if (__label__ == 34) {
  
              var $_pr3=$found; //@line 2492 "dictionary.cpp"
              var $tobool82=((($_pr3))|0)!=0; //@line 2492 "dictionary.cpp"
              if ($tobool82) { __label__ = 85;break $if_end281$$if_end81$48; } //@line 2492 "dictionary.cpp"
  
              var $71=$n; //@line 2492 "dictionary.cpp"
              var $cmp84=((($71))|0) > 0; //@line 2492 "dictionary.cpp"
              if ($cmp84) { __label__ = 36;; } else { __label__ = 43;; } //@line 2492 "dictionary.cpp"
              $if_then85$$if_end121$51: do { 
                if (__label__ == 36) {
  
                  var $72=HEAP32[(($p)>>2)]; //@line 2495 "dictionary.cpp"
                  var $arrayidx86=(($72+1)&4294967295); //@line 2495 "dictionary.cpp"
                  var $73=HEAP8[($arrayidx86)]; //@line 2495 "dictionary.cpp"
                  $c2=$73; //@line 2495 "dictionary.cpp"
                  var $74=$c; //@line 2496 "dictionary.cpp"
                  var $conv87=((($74))&255); //@line 2496 "dictionary.cpp"
                  var $75=$c2; //@line 2496 "dictionary.cpp"
                  var $conv88=((($75))&255); //@line 2496 "dictionary.cpp"
                  var $shl=((($conv88)*256)&4294967295); //@line 2496 "dictionary.cpp"
                  var $add89=((($shl)+($conv87))&4294967295); //@line 2496 "dictionary.cpp"
                  $c12=$add89; //@line 2496 "dictionary.cpp"
                  var $76=$c12; //@line 2497 "dictionary.cpp"
                  var $77=HEAP32[(($p)>>2)]; //@line 2497 "dictionary.cpp"
                  var $arrayidx90=(($77+2)&4294967295); //@line 2497 "dictionary.cpp"
                  var $78=HEAP8[($arrayidx90)]; //@line 2497 "dictionary.cpp"
                  var $conv91=(tempInt=(($78)),(tempInt>=128?tempInt-256:tempInt)); //@line 2497 "dictionary.cpp"
                  var $shl92=((($conv91)*65536)&4294967295); //@line 2497 "dictionary.cpp"
                  var $add93=((($shl92)+($76))&4294967295); //@line 2497 "dictionary.cpp"
                  $c123=$add93; //@line 2497 "dictionary.cpp"
                  var $79=$c; //@line 2499 "dictionary.cpp"
                  var $idxprom94=((($79))&255); //@line 2499 "dictionary.cpp"
                  var $80=$tr_addr; //@line 2499 "dictionary.cpp"
                  var $groups2_start=(($80+8012)&4294967295); //@line 2499 "dictionary.cpp"
                  var $arrayidx95=(($groups2_start+$idxprom94)&4294967295); //@line 2499 "dictionary.cpp"
                  var $81=HEAPU8[($arrayidx95)]; //@line 2499 "dictionary.cpp"
                  var $conv96=((($81))&255); //@line 2499 "dictionary.cpp"
                  $g1=$conv96; //@line 2499 "dictionary.cpp"
                  var $82=$g1; //@line 2500 "dictionary.cpp"
                  $g=$82; //@line 2500 "dictionary.cpp"
                  var $83=$g; //@line 2500 "dictionary.cpp"
                  var $84=$g1; //@line 2500 "dictionary.cpp"
                  var $85=$n; //@line 2500 "dictionary.cpp"
                  var $add989=((($85)+($84))&4294967295); //@line 2500 "dictionary.cpp"
                  var $cmp9910=((($83))|0) < ((($add989))|0); //@line 2500 "dictionary.cpp"
                  if (!($cmp9910)) { __label__ = 43;break $if_then85$$if_end121$51; } //@line 2500 "dictionary.cpp"
                  $for_body100$53: while(1) { 
  
                    var $86=$g; //@line 2502 "dictionary.cpp"
                    var $87=$tr_addr; //@line 2502 "dictionary.cpp"
                    var $groups2_name=(($87+7272)&4294967295); //@line 2502 "dictionary.cpp"
                    var $arrayidx101=(($groups2_name+$86*4)&4294967295); //@line 2502 "dictionary.cpp"
                    var $88=HEAP32[(($arrayidx101)>>2)]; //@line 2502 "dictionary.cpp"
                    var $89=$c12; //@line 2502 "dictionary.cpp"
                    var $cmp102=((($88))|0)==((($89))|0); //@line 2502 "dictionary.cpp"
                    if ($cmp102) { __label__ = 38;; } else { __label__ = 42;; } //@line 2502 "dictionary.cpp"
                    $if_then103$$for_inc118$55: do { 
                      if (__label__ == 38) {
  
                        $found=1; //@line 2504 "dictionary.cpp"
                        var $90=HEAP32[(($p)>>2)]; //@line 2506 "dictionary.cpp"
                        HEAP32[(($p2)>>2)]=$90; //@line 2506 "dictionary.cpp"
                        var $91=$tr_addr; //@line 2507 "dictionary.cpp"
                        var $92=$p_start_addr; //@line 2507 "dictionary.cpp"
                        var $93=$g; //@line 2507 "dictionary.cpp"
                        var $94=$tr_addr; //@line 2507 "dictionary.cpp"
                        var $groups2=(($94+6792)&4294967295); //@line 2507 "dictionary.cpp"
                        var $arrayidx104=(($groups2+$93*4)&4294967295); //@line 2507 "dictionary.cpp"
                        var $95=HEAP32[(($arrayidx104)>>2)]; //@line 2507 "dictionary.cpp"
                        var $96=$word_flags_addr; //@line 2507 "dictionary.cpp"
                        var $97=$dict_flags0; //@line 2507 "dictionary.cpp"
                        __ZL9MatchRuleP10TranslatorPPcS1_iS1_P11MatchRecordii($91, $p2, $92, 2, $95, $match2, $96, $97); //@line 2507 "dictionary.cpp"
                        var $98=HEAP32[(($points)>>2)]; //@line 2508 "dictionary.cpp"
                        var $cmp105=((($98))|0) > 0; //@line 2508 "dictionary.cpp"
                        if ($cmp105) { __label__ = 39;; } else { __label__ = 40;; } //@line 2508 "dictionary.cpp"
                        if (__label__ == 39) {
  
                          var $99=HEAP32[(($points107)>>2)]; //@line 2509 "dictionary.cpp"
                          var $add108=((($99)+35)&4294967295); //@line 2509 "dictionary.cpp"
                          HEAP32[(($points107)>>2)]=$add108; //@line 2509 "dictionary.cpp"
                          ; //@line 2509 "dictionary.cpp"
                        }
  
                        var $100=$tr_addr; //@line 2512 "dictionary.cpp"
                        var $101=$p_start_addr; //@line 2512 "dictionary.cpp"
                        var $102=$c; //@line 2512 "dictionary.cpp"
                        var $idxprom110=((($102))&255); //@line 2512 "dictionary.cpp"
                        var $103=$tr_addr; //@line 2512 "dictionary.cpp"
                        var $groups1=(($103+5256)&4294967295); //@line 2512 "dictionary.cpp"
                        var $arrayidx111=(($groups1+$idxprom110*4)&4294967295); //@line 2512 "dictionary.cpp"
                        var $104=HEAP32[(($arrayidx111)>>2)]; //@line 2512 "dictionary.cpp"
                        var $105=$word_flags_addr; //@line 2512 "dictionary.cpp"
                        var $106=$dict_flags0; //@line 2512 "dictionary.cpp"
                        __ZL9MatchRuleP10TranslatorPPcS1_iS1_P11MatchRecordii($100, $p, $101, 1, $104, $match1, $105, $106); //@line 2512 "dictionary.cpp"
                        var $107=HEAP32[(($points112)>>2)]; //@line 2514 "dictionary.cpp"
                        var $108=HEAP32[(($points113)>>2)]; //@line 2514 "dictionary.cpp"
                        var $cmp114=((($107))|0) >= ((($108))|0); //@line 2514 "dictionary.cpp"
                        if (!($cmp114)) { __label__ = 42;break $if_then103$$for_inc118$55; } //@line 2514 "dictionary.cpp"
  
                        var $src$, $dest$, $stop$, $stop4$, $fast$;
                        $src$ = $31;
                        $dest$ = $30;
                        $stop$ = $src$ + 16;
                        if (($dest$%4) == ($src$%4) && 16 > 8) {
                          while ($src$%4 !== 0 && $src$ < $stop$) {
                            ; HEAP8[$dest$++] = HEAP8[$src$++];
                          }
                          $src$ >>= 2;
                          $dest$ >>= 2;
                          $stop4$ = $stop$ >> 2;
                          while ($src$ < $stop4$) {
                        ;;;    HEAP32[$dest$++] = HEAP32[$src$++];
                          }
                          $src$ <<= 2;
                          $dest$ <<= 2;
                        }while ($src$ < $stop$) {
                          ; HEAP8[$dest$++] = HEAP8[$src$++];
                        }; //@line 2517 "dictionary.cpp"
                        var $109=HEAP32[(($p2)>>2)]; //@line 2518 "dictionary.cpp"
                        HEAP32[(($p)>>2)]=$109; //@line 2518 "dictionary.cpp"
                        ; //@line 2519 "dictionary.cpp"
                      }
                    } while(0);
  
                    var $110=$g; //@line 2500 "dictionary.cpp"
                    var $inc119=((($110)+1)&4294967295); //@line 2500 "dictionary.cpp"
                    $g=$inc119; //@line 2500 "dictionary.cpp"
                    var $111=$g; //@line 2500 "dictionary.cpp"
                    var $112=$g1; //@line 2500 "dictionary.cpp"
                    var $113=$n; //@line 2500 "dictionary.cpp"
                    var $add98=((($113)+($112))&4294967295); //@line 2500 "dictionary.cpp"
                    var $cmp99=((($111))|0) < ((($add98))|0); //@line 2500 "dictionary.cpp"
                    if ($cmp99) { __label__ = 37;continue $for_body100$53; } else { __label__ = 43;break $if_then85$$if_end121$51; } //@line 2500 "dictionary.cpp"
                  }
                }
              } while(0);
  
              var $_pr5_pr=$found; //@line 2524 "dictionary.cpp"
              var $tobool122=((($_pr5_pr))|0)!=0; //@line 2524 "dictionary.cpp"
              if ($tobool122) { __label__ = 85;break $if_end281$$if_end81$48; } //@line 2524 "dictionary.cpp"
  
              var $114=$c; //@line 2527 "dictionary.cpp"
              var $idxprom124=((($114))&255); //@line 2527 "dictionary.cpp"
              var $115=$tr_addr; //@line 2527 "dictionary.cpp"
              var $groups1125=(($115+5256)&4294967295); //@line 2527 "dictionary.cpp"
              var $arrayidx126=(($groups1125+$idxprom124*4)&4294967295); //@line 2527 "dictionary.cpp"
              var $116=HEAP32[(($arrayidx126)>>2)]; //@line 2527 "dictionary.cpp"
              var $cmp127=((($116))|0)!=0; //@line 2527 "dictionary.cpp"
              var $117=$tr_addr; //@line 2528 "dictionary.cpp"
              var $118=$p_start_addr; //@line 2528 "dictionary.cpp"
              if ($cmp127) { __label__ = 45;; } else { __label__ = 46;; } //@line 2527 "dictionary.cpp"
              $if_then128$$if_else132$64: do { 
                if (__label__ == 45) {
  
                  var $119=$c; //@line 2528 "dictionary.cpp"
                  var $idxprom129=((($119))&255); //@line 2528 "dictionary.cpp"
                  var $120=$tr_addr; //@line 2528 "dictionary.cpp"
                  var $groups1130=(($120+5256)&4294967295); //@line 2528 "dictionary.cpp"
                  var $arrayidx131=(($groups1130+$idxprom129*4)&4294967295); //@line 2528 "dictionary.cpp"
                  var $121=HEAP32[(($arrayidx131)>>2)]; //@line 2528 "dictionary.cpp"
                  var $122=$word_flags_addr; //@line 2528 "dictionary.cpp"
                  var $123=$dict_flags0; //@line 2528 "dictionary.cpp"
                  __ZL9MatchRuleP10TranslatorPPcS1_iS1_P11MatchRecordii($117, $p, $118, 1, $121, $match1, $122, $123); //@line 2528 "dictionary.cpp"
                  __label__ = 74;break $if_then128$$if_else132$64; //@line 2528 "dictionary.cpp"
                }
                else if (__label__ == 46) {
  
                  var $124=$tr_addr; //@line 2532 "dictionary.cpp"
                  var $groups1133=(($124+5256)&4294967295); //@line 2532 "dictionary.cpp"
                  var $arrayidx134=(($groups1133)&4294967295); //@line 2532 "dictionary.cpp"
                  var $125=HEAP32[(($arrayidx134)>>2)]; //@line 2532 "dictionary.cpp"
                  var $126=$word_flags_addr; //@line 2532 "dictionary.cpp"
                  var $127=$dict_flags0; //@line 2532 "dictionary.cpp"
                  __ZL9MatchRuleP10TranslatorPPcS1_iS1_P11MatchRecordii($117, $p, $118, 0, $125, $match1, $126, $127); //@line 2532 "dictionary.cpp"
                  var $128=HEAP32[(($points135)>>2)]; //@line 2534 "dictionary.cpp"
                  var $cmp136=((($128))|0)==0; //@line 2534 "dictionary.cpp"
                  if (!($cmp136)) { __label__ = 84;break $if_then128$$if_else132$64; } //@line 2534 "dictionary.cpp"
  
                  var $129=HEAP32[((_option_sayas)>>2)]; //@line 2534 "dictionary.cpp"
                  var $and138=($129) & 16; //@line 2534 "dictionary.cpp"
                  var $cmp139=((($and138))|0)==0; //@line 2534 "dictionary.cpp"
                  if (!($cmp139)) { __label__ = 74;break $if_then128$$if_else132$64; } //@line 2534 "dictionary.cpp"
  
                  var $130=HEAP32[(($p)>>2)]; //@line 2536 "dictionary.cpp"
                  var $add_ptr141=(($130+-1)&4294967295); //@line 2536 "dictionary.cpp"
                  var $call142=__Z7utf8_inPiPKc($letter, $add_ptr141); //@line 2536 "dictionary.cpp"
                  var $sub143=((($call142)-1)&4294967295); //@line 2536 "dictionary.cpp"
                  $n=$sub143; //@line 2536 "dictionary.cpp"
                  var $131=$tr_addr; //@line 2538 "dictionary.cpp"
                  var $letter_bits_offset144=(($131+964)&4294967295); //@line 2538 "dictionary.cpp"
                  var $132=HEAP32[(($letter_bits_offset144)>>2)]; //@line 2538 "dictionary.cpp"
                  var $cmp145=((($132))|0) > 0; //@line 2538 "dictionary.cpp"
                  if ($cmp145) { __label__ = 49;; } else { __label__ = 52;; } //@line 2538 "dictionary.cpp"
                  $if_then146$$if_end155thread_pre_split$69: do { 
                    if (__label__ == 49) {
  
                      var $133=HEAP32[(($letter)>>2)]; //@line 2541 "dictionary.cpp"
                      var $cmp147=((($133))|0) <= 577; //@line 2541 "dictionary.cpp"
                      if ($cmp147) { __lastLabel__ = 49; ; } else { __lastLabel__ = 49; __label__ = 53;break $if_then146$$if_end155thread_pre_split$69; } //@line 2541 "dictionary.cpp"
  
                      var $134=HEAP32[(($letter)>>2)]; //@line 2541 "dictionary.cpp"
                      var $call149=_iswalpha($134); //@line 2541 "dictionary.cpp"
                      var $tobool150=((($call149))|0)!=0; //@line 2541 "dictionary.cpp"
                      if ($tobool150) { __label__ = 51;break $while_cond$28; } else { __label__ = 52;break $if_then146$$if_end155thread_pre_split$69; } //@line 2541 "dictionary.cpp"
                    }
                  } while(0);
                  if (__label__ == 52) {
  
                    var $_pr7=HEAP32[(($letter)>>2)]; //@line 2558 "dictionary.cpp"
                    __lastLabel__ = 52; ;
                  }
  
                  var $138=__lastLabel__ == 52 ? $_pr7 : ($133); //@line 2558 "dictionary.cpp"
                  var $cmp156=((($138))|0)==57384; //@line 2558 "dictionary.cpp"
                  if ($cmp156) { __label__ = 54;; } else { __label__ = 56;; } //@line 2558 "dictionary.cpp"
                  $if_then157$$if_end166$75: do { 
                    if (__label__ == 54) {
  
                      var $139=HEAP32[((_pre_pause)>>2)]; //@line 2560 "dictionary.cpp"
                      var $140=$tr_addr; //@line 2560 "dictionary.cpp"
                      var $langopts158=(($140)&4294967295); //@line 2560 "dictionary.cpp"
                      var $param2=(($langopts158+100)&4294967295); //@line 2560 "dictionary.cpp"
                      var $arrayidx159=(($param2+64)&4294967295); //@line 2560 "dictionary.cpp"
                      var $141=HEAP32[(($arrayidx159)>>2)]; //@line 2560 "dictionary.cpp"
                      var $cmp160=((($139))|0) < ((($141))|0); //@line 2560 "dictionary.cpp"
                      if (!($cmp160)) { __label__ = 56;break $if_then157$$if_end166$75; } //@line 2560 "dictionary.cpp"
  
                      var $142=$tr_addr; //@line 2561 "dictionary.cpp"
                      var $langopts162=(($142)&4294967295); //@line 2561 "dictionary.cpp"
                      var $param2163=(($langopts162+100)&4294967295); //@line 2561 "dictionary.cpp"
                      var $arrayidx164=(($param2163+64)&4294967295); //@line 2561 "dictionary.cpp"
                      var $143=HEAP32[(($arrayidx164)>>2)]; //@line 2561 "dictionary.cpp"
                      HEAP32[((_pre_pause)>>2)]=$143; //@line 2561 "dictionary.cpp"
                      ; //@line 2561 "dictionary.cpp"
                    }
                  } while(0);
  
                  var $144=HEAP32[(($letter)>>2)]; //@line 2563 "dictionary.cpp"
                  var $call167=__Z9IsBracketi($144); //@line 2563 "dictionary.cpp"
                  var $tobool168=((($call167))|0)!=0; //@line 2563 "dictionary.cpp"
                  if ($tobool168) { __label__ = 57;; } else { __label__ = 59;; } //@line 2563 "dictionary.cpp"
                  $if_then169$$if_end178$79: do { 
                    if (__label__ == 57) {
  
                      var $145=HEAP32[((_pre_pause)>>2)]; //@line 2565 "dictionary.cpp"
                      var $146=$tr_addr; //@line 2565 "dictionary.cpp"
                      var $langopts170=(($146)&4294967295); //@line 2565 "dictionary.cpp"
                      var $param=(($langopts170+24)&4294967295); //@line 2565 "dictionary.cpp"
                      var $arrayidx171=(($param+64)&4294967295); //@line 2565 "dictionary.cpp"
                      var $147=HEAP32[(($arrayidx171)>>2)]; //@line 2565 "dictionary.cpp"
                      var $cmp172=((($145))|0) < ((($147))|0); //@line 2565 "dictionary.cpp"
                      if (!($cmp172)) { __label__ = 59;break $if_then169$$if_end178$79; } //@line 2565 "dictionary.cpp"
  
                      var $148=$tr_addr; //@line 2566 "dictionary.cpp"
                      var $langopts174=(($148)&4294967295); //@line 2566 "dictionary.cpp"
                      var $param175=(($langopts174+24)&4294967295); //@line 2566 "dictionary.cpp"
                      var $arrayidx176=(($param175+64)&4294967295); //@line 2566 "dictionary.cpp"
                      var $149=HEAP32[(($arrayidx176)>>2)]; //@line 2566 "dictionary.cpp"
                      HEAP32[((_pre_pause)>>2)]=$149; //@line 2566 "dictionary.cpp"
                      ; //@line 2566 "dictionary.cpp"
                    }
                  } while(0);
  
                  var $150=HEAP32[(($letter)>>2)]; //@line 2570 "dictionary.cpp"
                  var $cmp179=((($150))|0) >= 192; //@line 2570 "dictionary.cpp"
                  var $151=HEAP32[(($letter)>>2)]; //@line 2570 "dictionary.cpp"
                  var $cmp181=((($151))|0) < 606; //@line 2570 "dictionary.cpp"
                  var $or_cond=($cmp179) & ($cmp181); //@line 2570 "dictionary.cpp"
                  if ($or_cond) { __label__ = 60;; } else { __label__ = 71;; } //@line 2570 "dictionary.cpp"
                  $land_lhs_true182$$if_else223$83: do { 
                    if (__label__ == 60) {
  
                      var $152=HEAP32[(($letter)>>2)]; //@line 2570 "dictionary.cpp"
                      var $sub183=((($152)-192)&4294967295); //@line 2570 "dictionary.cpp"
                      var $arrayidx184=((__ZL13remove_accent+$sub183)&4294967295); //@line 2570 "dictionary.cpp"
                      var $153=HEAPU8[($arrayidx184)]; //@line 2570 "dictionary.cpp"
                      var $conv185=((($153))&255); //@line 2570 "dictionary.cpp"
                      $ix=$conv185; //@line 2570 "dictionary.cpp"
                      var $cmp186=((($conv185))|0)!=0; //@line 2570 "dictionary.cpp"
                      if (!($cmp186)) { __label__ = 71;break $land_lhs_true182$$if_else223$83; } //@line 2570 "dictionary.cpp"
  
                      var $154=HEAP32[(($p)>>2)]; //@line 2573 "dictionary.cpp"
                      var $arrayidx188=(($154+-2)&4294967295); //@line 2573 "dictionary.cpp"
                      var $155=HEAP8[($arrayidx188)]; //@line 2573 "dictionary.cpp"
                      var $conv189=(tempInt=(($155)),(tempInt>=128?tempInt-256:tempInt)); //@line 2573 "dictionary.cpp"
                      var $cmp190=((($conv189))|0)!=32; //@line 2573 "dictionary.cpp"
                      if ($cmp190) { __label__ = 63;; } else { __label__ = 62;; } //@line 2573 "dictionary.cpp"
                      if (__label__ == 62) {
  
                        var $156=$n; //@line 2573 "dictionary.cpp"
                        var $157=HEAP32[(($p)>>2)]; //@line 2573 "dictionary.cpp"
                        var $arrayidx192=(($157+$156)&4294967295); //@line 2573 "dictionary.cpp"
                        var $158=HEAP8[($arrayidx192)]; //@line 2573 "dictionary.cpp"
                        var $conv193=(tempInt=(($158)),(tempInt>=128?tempInt-256:tempInt)); //@line 2573 "dictionary.cpp"
                        var $cmp194=((($conv193))|0)!=32; //@line 2573 "dictionary.cpp"
                        if (!($cmp194)) { __label__ = 74;break $if_then128$$if_else132$64; } //@line 2573 "dictionary.cpp"
                      }
  
                      var $159=HEAP32[(($p)>>2)]; //@line 2576 "dictionary.cpp"
                      var $add_ptr196=(($159+-1)&4294967295); //@line 2576 "dictionary.cpp"
                      HEAP32[(($p2)>>2)]=$add_ptr196; //@line 2576 "dictionary.cpp"
                      var $160=$ix; //@line 2577 "dictionary.cpp"
                      var $conv197=((($160)) & 255); //@line 2577 "dictionary.cpp"
                      var $161=HEAP32[(($p)>>2)]; //@line 2577 "dictionary.cpp"
                      var $arrayidx198=(($161+-1)&4294967295); //@line 2577 "dictionary.cpp"
                      HEAP8[($arrayidx198)]=$conv197; //@line 2577 "dictionary.cpp"
                      var $162=$n; //@line 2578 "dictionary.cpp"
                      var $163=HEAP32[(($p)>>2)]; //@line 2578 "dictionary.cpp"
                      var $arrayidx20011=(($163+$162)&4294967295); //@line 2578 "dictionary.cpp"
                      var $164=HEAP8[($arrayidx20011)]; //@line 2578 "dictionary.cpp"
                      var $165=HEAP32[(($p)>>2)]; //@line 2578 "dictionary.cpp"
                      var $arrayidx20112=(($165)&4294967295); //@line 2578 "dictionary.cpp"
                      HEAP8[($arrayidx20112)]=$164; //@line 2578 "dictionary.cpp"
                      var $conv20213=(tempInt=(($164)),(tempInt>=128?tempInt-256:tempInt)); //@line 2578 "dictionary.cpp"
                      var $cmp20314=((($conv20213))|0)!=32; //@line 2578 "dictionary.cpp"
                      if ($cmp20314) { __label__ = 64;; } else { __label__ = 65;; } //@line 2578 "dictionary.cpp"
                      $while_body204$$while_cond205_loopexit$89: do { 
                        if (__label__ == 64) {
                          while(1) { 
  
                            var $166=HEAP32[(($p)>>2)]; //@line 2578 "dictionary.cpp"
                            var $incdec_ptr=(($166+1)&4294967295); //@line 2578 "dictionary.cpp"
                            HEAP32[(($p)>>2)]=$incdec_ptr; //@line 2578 "dictionary.cpp"
                            var $167=$n; //@line 2578 "dictionary.cpp"
                            var $168=HEAP32[(($p)>>2)]; //@line 2578 "dictionary.cpp"
                            var $arrayidx200=(($168+$167)&4294967295); //@line 2578 "dictionary.cpp"
                            var $169=HEAP8[($arrayidx200)]; //@line 2578 "dictionary.cpp"
                            var $170=HEAP32[(($p)>>2)]; //@line 2578 "dictionary.cpp"
                            var $arrayidx201=(($170)&4294967295); //@line 2578 "dictionary.cpp"
                            HEAP8[($arrayidx201)]=$169; //@line 2578 "dictionary.cpp"
                            var $conv202=(tempInt=(($169)),(tempInt>=128?tempInt-256:tempInt)); //@line 2578 "dictionary.cpp"
                            var $cmp203=((($conv202))|0)!=32; //@line 2578 "dictionary.cpp"
                            if ($cmp203) { __label__ = 64;continue ; } else { __label__ = 65;break $while_body204$$while_cond205_loopexit$89; } //@line 2578 "dictionary.cpp"
                          }
                        }
                      } while(0);
  
                      var $171=$n; //@line 2579 "dictionary.cpp"
                      var $dec15=((($171)-1)&4294967295); //@line 2579 "dictionary.cpp"
                      $n=$dec15; //@line 2579 "dictionary.cpp"
                      var $cmp20616=((($171))|0) > 0; //@line 2579 "dictionary.cpp"
                      if ($cmp20616) { __label__ = 66;; } else { __label__ = 67;; } //@line 2579 "dictionary.cpp"
                      $while_body207$$while_end209$93: do { 
                        if (__label__ == 66) {
                          while(1) { 
  
                            var $172=HEAP32[(($p)>>2)]; //@line 2579 "dictionary.cpp"
                            var $incdec_ptr208=(($172+1)&4294967295); //@line 2579 "dictionary.cpp"
                            HEAP32[(($p)>>2)]=$incdec_ptr208; //@line 2579 "dictionary.cpp"
                            HEAP8[($172)]=32; //@line 2579 "dictionary.cpp"
                            var $173=$n; //@line 2579 "dictionary.cpp"
                            var $dec=((($173)-1)&4294967295); //@line 2579 "dictionary.cpp"
                            $n=$dec; //@line 2579 "dictionary.cpp"
                            var $cmp206=((($173))|0) > 0; //@line 2579 "dictionary.cpp"
                            if ($cmp206) { __label__ = 66;continue ; } else { __label__ = 67;break $while_body207$$while_end209$93; } //@line 2579 "dictionary.cpp"
                          }
                        }
                      } while(0);
  
                      var $174=$tr_addr; //@line 2581 "dictionary.cpp"
                      var $langopts210=(($174)&4294967295); //@line 2581 "dictionary.cpp"
                      var $param211=(($langopts210+24)&4294967295); //@line 2581 "dictionary.cpp"
                      var $arrayidx212=(($param211+4)&4294967295); //@line 2581 "dictionary.cpp"
                      var $175=HEAP32[(($arrayidx212)>>2)]; //@line 2581 "dictionary.cpp"
                      var $tobool213=((($175))|0)!=0; //@line 2581 "dictionary.cpp"
                      if ($tobool213) { __label__ = 68;; } else { __label__ = 70;; } //@line 2581 "dictionary.cpp"
                      $land_lhs_true214$$if_end218$97: do { 
                        if (__label__ == 68) {
  
                          var $176=HEAP32[(($letter)>>2)]; //@line 2581 "dictionary.cpp"
                          var $call215=__Z11lookupwcharPKti(((__ZL13diereses_list)&4294967295), $176); //@line 2581 "dictionary.cpp"
                          var $cmp216=((($call215))|0) > 0; //@line 2581 "dictionary.cpp"
                          if (!($cmp216)) { __label__ = 70;break $land_lhs_true214$$if_end218$97; } //@line 2581 "dictionary.cpp"
  
                          var $177=HEAP32[(($p2)>>2)]; //@line 2584 "dictionary.cpp"
                          HEAP32[(($p)>>2)]=$177; //@line 2584 "dictionary.cpp"
                          __label__ = 20;continue $while_cond$28; //@line 2585 "dictionary.cpp"
                        }
                      } while(0);
  
                      var $178=$phonemes_addr; //@line 2588 "dictionary.cpp"
                      var $arrayidx219=(($178)&4294967295); //@line 2588 "dictionary.cpp"
                      HEAP8[($arrayidx219)]=0; //@line 2588 "dictionary.cpp"
                      var $179=$p_start_addr; //@line 2589 "dictionary.cpp"
                      HEAP32[(($p)>>2)]=$179; //@line 2589 "dictionary.cpp"
                      var $180=$tr_addr; //@line 2590 "dictionary.cpp"
                      var $word_vowel_count220=(($180+8296)&4294967295); //@line 2590 "dictionary.cpp"
                      HEAP32[(($word_vowel_count220)>>2)]=0; //@line 2590 "dictionary.cpp"
                      var $181=$tr_addr; //@line 2591 "dictionary.cpp"
                      var $word_stressed_count221=(($181+8300)&4294967295); //@line 2591 "dictionary.cpp"
                      HEAP32[(($word_stressed_count221)>>2)]=0; //@line 2591 "dictionary.cpp"
                      __label__ = 20;continue $while_cond$28; //@line 2592 "dictionary.cpp"
                    }
                  } while(0);
  
                  var $182=HEAP32[(($letter)>>2)]; //@line 2596 "dictionary.cpp"
                  var $cmp224=((($182))|0) >= 12800; //@line 2596 "dictionary.cpp"
                  var $183=HEAP32[(($letter)>>2)]; //@line 2596 "dictionary.cpp"
                  var $cmp226=((($183))|0) < 42752; //@line 2596 "dictionary.cpp"
                  var $or_cond1=($cmp224) & ($cmp226); //@line 2596 "dictionary.cpp"
                  if (!($or_cond1)) { __label__ = 74;break $if_then128$$if_else132$64; } //@line 2596 "dictionary.cpp"
  
                  var $184=$end_phonemes_addr; //@line 2596 "dictionary.cpp"
                  var $cmp228=((($184))|0)!=0; //@line 2596 "dictionary.cpp"
                  if (!($cmp228)) { __label__ = 74;break $if_then128$$if_else132$64; } //@line 2596 "dictionary.cpp"
  
                  var $185=$tr_addr; //@line 2600 "dictionary.cpp"
                  var $call231=__Z6LookupP10TranslatorPKcPc($185, ((__str9306)&4294967295), $arraydecay230); //@line 2600 "dictionary.cpp"
                  HEAP32[(($phonemes233)>>2)]=$arraydecay232; //@line 2601 "dictionary.cpp"
                  HEAP32[(($points234)>>2)]=1; //@line 2602 "dictionary.cpp"
                  var $186=$wc_bytes; //@line 2603 "dictionary.cpp"
                  var $sub235=((($186)-1)&4294967295); //@line 2603 "dictionary.cpp"
                  var $187=HEAP32[(($p)>>2)]; //@line 2603 "dictionary.cpp"
                  var $add_ptr236=(($187+$sub235)&4294967295); //@line 2603 "dictionary.cpp"
                  HEAP32[(($p)>>2)]=$add_ptr236; //@line 2603 "dictionary.cpp"
                  __label__ = 74;break $if_then128$$if_else132$64; //@line 2604 "dictionary.cpp"
                }
              } while(0);
              $if_end240$$if_else278$104: do { 
                if (__label__ == 74) {
  
                  var $_pr17=HEAP32[(($points241)>>2)]; //@line 2608 "dictionary.cpp"
                  var $cmp242=((($_pr17))|0)==0; //@line 2608 "dictionary.cpp"
                  if (!($cmp242)) { __label__ = 84;break $if_end240$$if_else278$104; } //@line 2608 "dictionary.cpp"
  
                  var $188=HEAP32[(($wc)>>2)]; //@line 2610 "dictionary.cpp"
                  var $cmp244=((($188))|0) >= 768; //@line 2610 "dictionary.cpp"
                  var $189=HEAP32[(($wc)>>2)]; //@line 2610 "dictionary.cpp"
                  var $cmp246=((($189))|0) <= 879; //@line 2610 "dictionary.cpp"
                  var $or_cond2=($cmp244) & ($cmp246); //@line 2610 "dictionary.cpp"
                  if ($or_cond2) { __label__ = 83;; } else { __label__ = 76;; } //@line 2610 "dictionary.cpp"
                  $if_end275$$if_else248$107: do { 
                    if (__label__ == 76) {
  
                      var $190=HEAP32[(($wc)>>2)]; //@line 2615 "dictionary.cpp"
                      var $call249=__Z7IsAlphaj($190); //@line 2615 "dictionary.cpp"
                      var $tobool250=((($call249))|0)!=0; //@line 2615 "dictionary.cpp"
                      if ($tobool250) { __label__ = 77;; } else { __label__ = 81;; } //@line 2615 "dictionary.cpp"
                      if (__label__ == 77) {
  
                        var $191=$any_alpha; //@line 2617 "dictionary.cpp"
                        var $cmp252=((($191))|0) > 1; //@line 2617 "dictionary.cpp"
                        if ($cmp252) { __label__ = 79;break $while_cond$28; } //@line 2617 "dictionary.cpp"
  
                        var $192=$wc_bytes; //@line 2617 "dictionary.cpp"
                        var $sub254=((($192)-1)&4294967295); //@line 2617 "dictionary.cpp"
                        var $193=HEAP32[(($p)>>2)]; //@line 2617 "dictionary.cpp"
                        var $arrayidx255=(($193+$sub254)&4294967295); //@line 2617 "dictionary.cpp"
                        var $194=HEAP8[($arrayidx255)]; //@line 2617 "dictionary.cpp"
                        var $conv256=(tempInt=(($194)),(tempInt>=128?tempInt-256:tempInt)); //@line 2617 "dictionary.cpp"
                        var $cmp257=((($conv256))|0) > 32; //@line 2617 "dictionary.cpp"
                        if ($cmp257) { __label__ = 79;break $while_cond$28; } //@line 2617 "dictionary.cpp"
                      }
                      else if (__label__ == 81) {
  
                        var $199=$tr_addr; //@line 2628 "dictionary.cpp"
                        var $200=HEAP32[(($wc)>>2)]; //@line 2628 "dictionary.cpp"
                        __Z12LookupLetterP10TranslatorjiPci($199, $200, -1, $arraydecay266, 0); //@line 2628 "dictionary.cpp"
                        var $201=HEAP8[($arrayidx267)]; //@line 2629 "dictionary.cpp"
                        var $tobool268=(tempInt=(($201)),(tempInt>=128?tempInt-256:tempInt))!=0; //@line 2629 "dictionary.cpp"
                        if (!($tobool268)) { __label__ = 83;break $if_end275$$if_else248$107; } //@line 2629 "dictionary.cpp"
  
                        HEAP32[(($phonemes271)>>2)]=$arraydecay270; //@line 2631 "dictionary.cpp"
                        HEAP32[(($points272)>>2)]=1; //@line 2632 "dictionary.cpp"
                        ; //@line 2633 "dictionary.cpp"
                      }
                    }
                  } while(0);
  
                  var $202=$wc_bytes; //@line 2635 "dictionary.cpp"
                  var $sub276=((($202)-1)&4294967295); //@line 2635 "dictionary.cpp"
                  var $203=HEAP32[(($p)>>2)]; //@line 2635 "dictionary.cpp"
                  var $add_ptr277=(($203+$sub276)&4294967295); //@line 2635 "dictionary.cpp"
                  HEAP32[(($p)>>2)]=$add_ptr277; //@line 2635 "dictionary.cpp"
                  __label__ = 85;break $if_end281$$if_end81$48; //@line 2636 "dictionary.cpp"
                }
              } while(0);
  
              var $204=$tr_addr; //@line 2639 "dictionary.cpp"
              var $phonemes_repeat_count=(($204+652)&4294967295); //@line 2639 "dictionary.cpp"
              HEAP32[(($phonemes_repeat_count)>>2)]=0; //@line 2639 "dictionary.cpp"
              ;
            }
          } while(0);
  
          var $205=HEAP32[(($phonemes282)>>2)]; //@line 2644 "dictionary.cpp"
          var $cmp283=((($205))|0)==0; //@line 2644 "dictionary.cpp"
          if ($cmp283) { __label__ = 86;; } else { __label__ = 87;; } //@line 2644 "dictionary.cpp"
          if (__label__ == 86) {
  
            HEAP32[(($phonemes285)>>2)]=((__str13643)&4294967295); //@line 2645 "dictionary.cpp"
            ; //@line 2645 "dictionary.cpp"
          }
  
          var $206=HEAP32[(($points287)>>2)]; //@line 2647 "dictionary.cpp"
          var $cmp288=((($206))|0) > 0; //@line 2647 "dictionary.cpp"
          if (!($cmp288)) { __label__ = 20;continue $while_cond$28; } //@line 2647 "dictionary.cpp"
  
          var $207=$word_flags_addr; //@line 2649 "dictionary.cpp"
          var $and290=($207) & -2147483648; //@line 2649 "dictionary.cpp"
          var $tobool291=((($and290))|0)!=0; //@line 2649 "dictionary.cpp"
          if ($tobool291) { __label__ = 89;break $while_cond$28; } //@line 2649 "dictionary.cpp"
  
          var $209=HEAP32[(($phonemes295)>>2)]; //@line 2652 "dictionary.cpp"
          var $arrayidx296=(($209)&4294967295); //@line 2652 "dictionary.cpp"
          var $210=HEAP8[($arrayidx296)]; //@line 2652 "dictionary.cpp"
          var $conv297=(tempInt=(($210)),(tempInt>=128?tempInt-256:tempInt)); //@line 2652 "dictionary.cpp"
          var $cmp298=((($conv297))|0)==21; //@line 2652 "dictionary.cpp"
          if ($cmp298) { __label__ = 91;; } else { __label__ = 93;; } //@line 2652 "dictionary.cpp"
          if (__label__ == 91) {
  
            var $211=$word_flags_addr; //@line 2652 "dictionary.cpp"
            var $and300=($211) & 4096; //@line 2652 "dictionary.cpp"
            var $cmp301=((($and300))|0)==0; //@line 2652 "dictionary.cpp"
            if ($cmp301) { __label__ = 92;break $while_cond$28; } //@line 2652 "dictionary.cpp"
          }
  
          var $214=HEAP32[(($end_type306)>>2)]; //@line 2659 "dictionary.cpp"
          var $and307=($214) & -32769; //@line 2659 "dictionary.cpp"
          HEAP32[(($end_type306)>>2)]=$and307; //@line 2659 "dictionary.cpp"
          var $cmp309=((($and307))|0)!=0; //@line 2661 "dictionary.cpp"
          if ($cmp309) { __label__ = 94;; } else { __label__ = 101;; } //@line 2661 "dictionary.cpp"
          $land_lhs_true310$$if_end339$125: do { 
            if (__label__ == 94) {
  
              var $215=$end_phonemes_addr; //@line 2661 "dictionary.cpp"
              var $cmp311=((($215))|0)!=0; //@line 2661 "dictionary.cpp"
              if (!($cmp311)) { __label__ = 101;break $land_lhs_true310$$if_end339$125; } //@line 2661 "dictionary.cpp"
  
              var $216=HEAP32[(($end_type313)>>2)]; //@line 2664 "dictionary.cpp"
              var $and314=($216) & 1024; //@line 2664 "dictionary.cpp"
              var $tobool315=((($and314))|0)!=0; //@line 2664 "dictionary.cpp"
              if (!($tobool315)) { __label__ = 97;break $while_cond$28; } //@line 2664 "dictionary.cpp"
  
              var $217=$word_flags_addr; //@line 2664 "dictionary.cpp"
              var $and317=($217) & 536870912; //@line 2664 "dictionary.cpp"
              var $tobool318=((($and317))|0)!=0; //@line 2664 "dictionary.cpp"
              if (!($tobool318)) { __label__ = 97;break $while_cond$28; } //@line 2664 "dictionary.cpp"
            }
          } while(0);
  
          var $227=HEAP32[(($del_fwd)>>2)]; //@line 2680 "dictionary.cpp"
          var $cmp340=((($227))|0)!=0; //@line 2680 "dictionary.cpp"
          if ($cmp340) { __label__ = 102;; } else { __label__ = 103;; } //@line 2680 "dictionary.cpp"
          if (__label__ == 102) {
  
            var $228=HEAP32[(($del_fwd342)>>2)]; //@line 2681 "dictionary.cpp"
            HEAP8[($228)]=69; //@line 2681 "dictionary.cpp"
            ; //@line 2681 "dictionary.cpp"
          }
  
          var $229=$tr_addr; //@line 2682 "dictionary.cpp"
          var $230=$phonemes_addr; //@line 2682 "dictionary.cpp"
          var $231=$ph_size_addr; //@line 2682 "dictionary.cpp"
          var $232=HEAP32[(($phonemes344)>>2)]; //@line 2682 "dictionary.cpp"
          __Z14AppendPhonemesP10TranslatorPciPKc($229, $230, $231, $232); //@line 2682 "dictionary.cpp"
          __label__ = 20;continue $while_cond$28; //@line 2683 "dictionary.cpp"
        }
        $while_end346$$if_then292$$if_then302$$if_else320$$if_then258$$if_then151$133: do { 
          if (__label__ == 89) {
  
            var $end_type=(($match1+8)&4294967295); //@line 2650 "dictionary.cpp"
            var $208=HEAP32[(($end_type)>>2)]; //@line 2650 "dictionary.cpp"
            var $or293=($208) | 1; //@line 2650 "dictionary.cpp"
            $retval=$or293; //@line 2650 "dictionary.cpp"
            __label__ = 105;break $if_then$$if_end$2; //@line 2650 "dictionary.cpp"
          }
          else if (__label__ == 92) {
  
            var $212=$phonemes_addr; //@line 2655 "dictionary.cpp"
            var $phonemes303=(($match1+4)&4294967295); //@line 2655 "dictionary.cpp"
            var $213=HEAP32[(($phonemes303)>>2)]; //@line 2655 "dictionary.cpp"
            var $call304=_strcpy($212, $213); //@line 2655 "dictionary.cpp"
            $retval=0; //@line 2656 "dictionary.cpp"
            __label__ = 105;break $if_then$$if_end$2; //@line 2656 "dictionary.cpp"
          }
          else if (__label__ == 97) {
  
            var $end_type321=(($match1+8)&4294967295); //@line 2670 "dictionary.cpp"
            var $218=HEAP32[(($end_type321)>>2)]; //@line 2670 "dictionary.cpp"
            var $and322=($218) & 1024; //@line 2670 "dictionary.cpp"
            var $tobool323=((($and322))|0)!=0; //@line 2670 "dictionary.cpp"
            if ($tobool323) { __label__ = 98;; } else { __label__ = 100;; } //@line 2670 "dictionary.cpp"
            $land_lhs_true324$$if_end331$137: do { 
              if (__label__ == 98) {
  
                var $end_type325=(($match1+8)&4294967295); //@line 2670 "dictionary.cpp"
                var $219=HEAP32[(($end_type325)>>2)]; //@line 2670 "dictionary.cpp"
                var $and326=($219) & 127; //@line 2670 "dictionary.cpp"
                var $cmp327=((($and326))|0)==0; //@line 2670 "dictionary.cpp"
                if (!($cmp327)) { __label__ = 100;break $land_lhs_true324$$if_end331$137; } //@line 2670 "dictionary.cpp"
  
                var $220=HEAP32[(($p)>>2)]; //@line 2673 "dictionary.cpp"
                var $221=$p_start_addr; //@line 2673 "dictionary.cpp"
                var $sub_ptr_lhs_cast=($220); //@line 2673 "dictionary.cpp"
                var $sub_ptr_rhs_cast=($221); //@line 2673 "dictionary.cpp"
                var $sub_ptr_sub=((($sub_ptr_lhs_cast)-($sub_ptr_rhs_cast))&4294967295); //@line 2673 "dictionary.cpp"
                var $end_type329=(($match1+8)&4294967295); //@line 2673 "dictionary.cpp"
                var $222=HEAP32[(($end_type329)>>2)]; //@line 2673 "dictionary.cpp"
                var $or330=($sub_ptr_sub) | ($222); //@line 2673 "dictionary.cpp"
                HEAP32[(($end_type329)>>2)]=$or330; //@line 2673 "dictionary.cpp"
                ; //@line 2674 "dictionary.cpp"
              }
            } while(0);
  
            var $223=$end_phonemes_addr; //@line 2675 "dictionary.cpp"
            var $phonemes332=(($match1+4)&4294967295); //@line 2675 "dictionary.cpp"
            var $224=HEAP32[(($phonemes332)>>2)]; //@line 2675 "dictionary.cpp"
            var $call333=_strcpy($223, $224); //@line 2675 "dictionary.cpp"
            var $225=$p_start_addr; //@line 2676 "dictionary.cpp"
            var $arraydecay334=(($word_copy)&4294967295); //@line 2676 "dictionary.cpp"
            var $arraydecay335=(($word_copy)&4294967295); //@line 2676 "dictionary.cpp"
            var $call336=_strlen($arraydecay335); //@line 2676 "dictionary.cpp"
            var $src$, $dest$, $stop$, $stop4$, $fast$;
            $src$ = $arraydecay334;
            $dest$ = $225;
            $stop$ = $src$ + $call336;
            if (($dest$%4) == ($src$%4) && $call336 > 8) {
              while ($src$%4 !== 0 && $src$ < $stop$) {
                ; HEAP8[$dest$++] = HEAP8[$src$++];
              }
              $src$ >>= 2;
              $dest$ >>= 2;
              $stop4$ = $stop$ >> 2;
              while ($src$ < $stop4$) {
            ;;;    HEAP32[$dest$++] = HEAP32[$src$++];
              }
              $src$ <<= 2;
              $dest$ <<= 2;
            }while ($src$ < $stop$) {
              ; HEAP8[$dest$++] = HEAP8[$src$++];
            }; //@line 2676 "dictionary.cpp"
            var $end_type337=(($match1+8)&4294967295); //@line 2677 "dictionary.cpp"
            var $226=HEAP32[(($end_type337)>>2)]; //@line 2677 "dictionary.cpp"
            $retval=$226; //@line 2677 "dictionary.cpp"
            __label__ = 105;break $if_then$$if_end$2; //@line 2677 "dictionary.cpp"
          }
          else if (__label__ == 79) {
  
            var $195=$phonemes_addr; //@line 2620 "dictionary.cpp"
            var $arrayidx259=(($195)&4294967295); //@line 2620 "dictionary.cpp"
            HEAP8[($arrayidx259)]=0; //@line 2620 "dictionary.cpp"
            var $196=$dict_flags_addr; //@line 2621 "dictionary.cpp"
            var $cmp260=((($196))|0)!=0; //@line 2621 "dictionary.cpp"
            if (!($cmp260)) { __label__ = 104;break $while_end346$$if_then292$$if_then302$$if_else320$$if_then258$$if_then151$133; } //@line 2621 "dictionary.cpp"
  
            var $197=$dict_flags_addr; //@line 2622 "dictionary.cpp"
            var $arrayidx262=(($197)&4294967295); //@line 2622 "dictionary.cpp"
            var $198=HEAP32[(($arrayidx262)>>2)]; //@line 2622 "dictionary.cpp"
            var $or=($198) | 65536; //@line 2622 "dictionary.cpp"
            HEAP32[(($arrayidx262)>>2)]=$or; //@line 2622 "dictionary.cpp"
            ; //@line 2622 "dictionary.cpp"
          }
          else if (__label__ == 51) {
  
            var $135=$phonemes_addr; //@line 2543 "dictionary.cpp"
            var $136=$tr_addr; //@line 2543 "dictionary.cpp"
            var $langopts152=(($136)&4294967295); //@line 2543 "dictionary.cpp"
            var $ascii_language=(($langopts152+268)&4294967295); //@line 2543 "dictionary.cpp"
            var $137=HEAP32[(($ascii_language)>>2)]; //@line 2543 "dictionary.cpp"
            var $call153=_sprintf($135, ((__str70722)&4294967295), allocate([21,0,0,0,$137,0,0,0], ["i32",0,0,0,"i8*",0,0,0], ALLOC_STACK)); //@line 2543 "dictionary.cpp"
            $retval=0; //@line 2544 "dictionary.cpp"
            __label__ = 105;break $if_then$$if_end$2; //@line 2544 "dictionary.cpp"
          }
        } while(0);
  
        var $233=$tr_addr; //@line 2687 "dictionary.cpp"
        var $234=$phonemes_addr; //@line 2687 "dictionary.cpp"
        var $235=$dict_flags0; //@line 2687 "dictionary.cpp"
        __Z21ApplySpecialAttributeP10TranslatorPci($233, $234, $235); //@line 2687 "dictionary.cpp"
        var $236=$p_start_addr; //@line 2688 "dictionary.cpp"
        var $arraydecay347=(($word_copy)&4294967295); //@line 2688 "dictionary.cpp"
        var $arraydecay348=(($word_copy)&4294967295); //@line 2688 "dictionary.cpp"
        var $call349=_strlen($arraydecay348); //@line 2688 "dictionary.cpp"
        var $src$, $dest$, $stop$, $stop4$, $fast$;
        $src$ = $arraydecay347;
        $dest$ = $236;
        $stop$ = $src$ + $call349;
        if (($dest$%4) == ($src$%4) && $call349 > 8) {
          while ($src$%4 !== 0 && $src$ < $stop$) {
            ; HEAP8[$dest$++] = HEAP8[$src$++];
          }
          $src$ >>= 2;
          $dest$ >>= 2;
          $stop4$ = $stop$ >> 2;
          while ($src$ < $stop4$) {
        ;;;    HEAP32[$dest$++] = HEAP32[$src$++];
          }
          $src$ <<= 2;
          $dest$ <<= 2;
        }while ($src$ < $stop$) {
          ; HEAP8[$dest$++] = HEAP8[$src$++];
        }; //@line 2688 "dictionary.cpp"
        $retval=0; //@line 2690 "dictionary.cpp"
        ; //@line 2690 "dictionary.cpp"
      }
    } while(0);
  
    var $237=$retval; //@line 2691 "dictionary.cpp"
    STACKTOP = __stackBase__;
    return $237; //@line 2691 "dictionary.cpp"
    return null;
  }
  

  function __Z6LookupP10TranslatorPKcPc($tr, $word, $ph_out) {
    var __stackBase__  = STACKTOP; STACKTOP += 12; _memset(__stackBase__, 0, 12);
    var __label__;
  
    var $tr_addr;
    var $word_addr;
    var $ph_out_addr;
    var $flags=__stackBase__;
    var $word1=__stackBase__+8;
    $tr_addr=$tr;
    $word_addr=$word;
    $ph_out_addr=$ph_out;
    var $arrayidx=(($flags+4)&4294967295); //@line 3369 "dictionary.cpp"
    HEAP32[(($arrayidx)>>2)]=0; //@line 3369 "dictionary.cpp"
    var $arrayidx1=(($flags)&4294967295); //@line 3369 "dictionary.cpp"
    HEAP32[(($arrayidx1)>>2)]=0; //@line 3369 "dictionary.cpp"
    var $0=$word_addr; //@line 3370 "dictionary.cpp"
    HEAP32[(($word1)>>2)]=$0; //@line 3370 "dictionary.cpp"
    var $1=$tr_addr; //@line 3371 "dictionary.cpp"
    var $2=$ph_out_addr; //@line 3371 "dictionary.cpp"
    var $arraydecay=(($flags)&4294967295); //@line 3371 "dictionary.cpp"
    var $call=__Z14LookupDictListP10TranslatorPPcS1_PjiP8WORD_TAB($1, $word1, $2, $arraydecay, 0, 0); //@line 3371 "dictionary.cpp"
    STACKTOP = __stackBase__;
    return $call; //@line 3371 "dictionary.cpp"
    return null;
  }
  

  function __ZL9MatchRuleP10TranslatorPPcS1_iS1_P11MatchRecordii($tr, $word, $word_start, $group_length, $rule, $match_out, $word_flags, $dict_flags) {
    var __stackBase__  = STACKTOP; STACKTOP += 264; _memset(__stackBase__, 0, 264);
    var __label__;
    var __lastLabel__ = null;
  
    var $tr_addr;
    var $word_addr;
    var $word_start_addr;
    var $group_length_addr;
    var $rule_addr;
    var $match_out_addr;
    var $word_flags_addr;
    var $dict_flags_addr;
    var $rb;
    var $letter;
    var $letter_w=__stackBase__;
    var $letter_xbytes;
    var $last_letter;
    var $pre_ptr;
    var $post_ptr;
    var $rule_start;
    var $p;
    var $ix;
    var $match_type;
    var $failed;
    var $unpron_ignore;
    var $consumed;
    var $syllable_count;
    var $vowel;
    var $letter_group;
    var $distance_right;
    var $distance_left;
    var $lg_pts;
    var $n_bytes;
    var $add_points;
    var $command;
    var $check_atstart;
    var $match=__stackBase__+4;
    var $total_consumed;
    var $condition_num;
    var $common_phonemes;
    var $group_chars;
    var $word_buf=__stackBase__+20;
    var $p222;
    var $vowel_count;
    var $p252;
    var $p269;
    var $p2;
    var $rule_w=__stackBase__+180;
    var $p419;
    var $pts;
    var $decoded_phonemes=__stackBase__+184;
    $tr_addr=$tr;
    $word_addr=$word;
    $word_start_addr=$word_start;
    $group_length_addr=$group_length;
    $rule_addr=$rule;
    $match_out_addr=$match_out;
    $word_flags_addr=$word_flags;
    $dict_flags_addr=$dict_flags;
    var $0=$word_addr; //@line 1777 "dictionary.cpp"
    var $1=HEAP32[(($0)>>2)]; //@line 1777 "dictionary.cpp"
    $group_chars=$1; //@line 1777 "dictionary.cpp"
    var $2=$rule_addr; //@line 1779 "dictionary.cpp"
    var $cmp=((($2))|0)==0; //@line 1779 "dictionary.cpp"
    if ($cmp) { __label__ = 1;; } else { __label__ = 2;; } //@line 1779 "dictionary.cpp"
    if (__label__ == 1) {
  
      var $3=$match_out_addr; //@line 1781 "dictionary.cpp"
      var $points=(($3)&4294967295); //@line 1781 "dictionary.cpp"
      HEAP32[(($points)>>2)]=0; //@line 1781 "dictionary.cpp"
      var $4=$word_addr; //@line 1782 "dictionary.cpp"
      var $5=HEAP32[(($4)>>2)]; //@line 1782 "dictionary.cpp"
      var $incdec_ptr=(($5+1)&4294967295); //@line 1782 "dictionary.cpp"
      HEAP32[(($4)>>2)]=$incdec_ptr; //@line 1782 "dictionary.cpp"
      ; //@line 1783 "dictionary.cpp"
    }
    else if (__label__ == 2) {
  
      $total_consumed=0; //@line 1787 "dictionary.cpp"
      $common_phonemes=0; //@line 1788 "dictionary.cpp"
      $match_type=0; //@line 1789 "dictionary.cpp"
      HEAP32[((((__ZZL9MatchRuleP10TranslatorPPcS1_iS1_P11MatchRecordiiE4best)&4294967295))>>2)]=0; //@line 1791 "dictionary.cpp"
      HEAP32[((((__ZZL9MatchRuleP10TranslatorPPcS1_iS1_P11MatchRecordiiE4best+4)&4294967295))>>2)]=((__str13643)&4294967295); //@line 1792 "dictionary.cpp"
      HEAP32[((((__ZZL9MatchRuleP10TranslatorPPcS1_iS1_P11MatchRecordiiE4best+8)&4294967295))>>2)]=0; //@line 1793 "dictionary.cpp"
      HEAP32[((((__ZZL9MatchRuleP10TranslatorPPcS1_iS1_P11MatchRecordiiE4best+12)&4294967295))>>2)]=0; //@line 1794 "dictionary.cpp"
      var $6=$rule_addr; //@line 1797 "dictionary.cpp"
      var $7=HEAP8[($6)]; //@line 1797 "dictionary.cpp"
      var $conv22=(tempInt=(($7)),(tempInt>=128?tempInt-256:tempInt)); //@line 1797 "dictionary.cpp"
      var $cmp123=((($conv22))|0)!=7; //@line 1797 "dictionary.cpp"
      if ($cmp123) { __label__ = 3;; } else { __label__ = 198;; } //@line 1797 "dictionary.cpp"
      $while_body_lr_ph$$while_end549$5: do { 
        if (__label__ == 3) {
  
          var $points2=(($match)&4294967295); //@line 1807 "dictionary.cpp"
          var $end_type=(($match+8)&4294967295); //@line 1808 "dictionary.cpp"
          var $del_fwd=(($match+12)&4294967295); //@line 1809 "dictionary.cpp"
          var $points518=(($match)&4294967295); //@line 2340 "dictionary.cpp"
          var $8=$match; //@line 2342 "dictionary.cpp"
          var $points524=(($match)&4294967295); //@line 2346 "dictionary.cpp"
          var $points530=(($match)&4294967295); //@line 2352 "dictionary.cpp"
          var $phonemes535=(($match+4)&4294967295); //@line 2355 "dictionary.cpp"
          var $arraydecay536=(($decoded_phonemes)&4294967295); //@line 2355 "dictionary.cpp"
          var $arraydecay538=(($decoded_phonemes)&4294967295); //@line 2356 "dictionary.cpp"
          var $points515=(($match)&4294967295); //@line 2337 "dictionary.cpp"
          var $phonemes=(($match+4)&4294967295); //@line 1829 "dictionary.cpp"
          var $phonemes13=(($match+4)&4294967295); //@line 1830 "dictionary.cpp"
          var $phonemes23=(($match+4)&4294967295); //@line 1833 "dictionary.cpp"
          var $phonemes29=(($match+4)&4294967295); //@line 1835 "dictionary.cpp"
          var $phonemes32=(($match+4)&4294967295); //@line 1840 "dictionary.cpp"
          var $phonemes43=(($match+4)&4294967295); //@line 1867 "dictionary.cpp"
          var $points66=(($match)&4294967295); //@line 1891 "dictionary.cpp"
          var $points499=(($match)&4294967295); //@line 2328 "dictionary.cpp"
          var $end_type171=(($match+8)&4294967295); //@line 2006 "dictionary.cpp"
          var $arraydecay=(($word_buf)&4294967295); //@line 2023 "dictionary.cpp"
          var $arrayidx195=(($word_buf+1)&4294967295); //@line 2027 "dictionary.cpp"
          var $del_fwd291=(($match+12)&4294967295); //@line 2121 "dictionary.cpp"
          var $end_type307=(($match+8)&4294967295); //@line 2129 "dictionary.cpp"
          ; //@line 1797 "dictionary.cpp"
          $while_body$7: while(1) { 
  
            var $11=$word_flags_addr; //@line 1799 "dictionary.cpp"
            var $and=($11) & -2147483648; //@line 1799 "dictionary.cpp"
            $unpron_ignore=$and; //@line 1799 "dictionary.cpp"
            $match_type=0; //@line 1800 "dictionary.cpp"
            $consumed=0; //@line 1801 "dictionary.cpp"
            $letter=0; //@line 1802 "dictionary.cpp"
            $distance_right=-6; //@line 1803 "dictionary.cpp"
            $distance_left=-2; //@line 1804 "dictionary.cpp"
            $check_atstart=0; //@line 1805 "dictionary.cpp"
            HEAP32[(($points2)>>2)]=1; //@line 1807 "dictionary.cpp"
            HEAP32[(($end_type)>>2)]=0; //@line 1808 "dictionary.cpp"
            HEAP32[(($del_fwd)>>2)]=0; //@line 1809 "dictionary.cpp"
            var $12=$word_addr; //@line 1811 "dictionary.cpp"
            var $13=HEAP32[(($12)>>2)]; //@line 1811 "dictionary.cpp"
            $pre_ptr=$13; //@line 1811 "dictionary.cpp"
            var $14=$word_addr; //@line 1812 "dictionary.cpp"
            var $15=HEAP32[(($14)>>2)]; //@line 1812 "dictionary.cpp"
            var $16=$group_length_addr; //@line 1812 "dictionary.cpp"
            var $add_ptr=(($15+$16)&4294967295); //@line 1812 "dictionary.cpp"
            $post_ptr=$add_ptr; //@line 1812 "dictionary.cpp"
            var $17=$rule_addr; //@line 1815 "dictionary.cpp"
            $rule_start=$17; //@line 1815 "dictionary.cpp"
            $failed=0; //@line 1817 "dictionary.cpp"
            var $18=$failed; //@line 1818 "dictionary.cpp"
            var $tobool20=((($18))|0)!=0; //@line 1818 "dictionary.cpp"
            var $lnot21=($tobool20) ^ 1; //@line 1818 "dictionary.cpp"
            if ($lnot21) { __lastLabel__ = 5; __label__ = 6;; } else { __lastLabel__ = 5; __label__ = 183;; } //@line 1818 "dictionary.cpp"
            $while_body4$$while_end502$9: do { 
              if (__label__ == 6) {
                $while_body4$10: while(1) { 
  
                  var $19=$rule_addr; //@line 1820 "dictionary.cpp"
                  var $incdec_ptr5=(($19+1)&4294967295); //@line 1820 "dictionary.cpp"
                  $rule_addr=$incdec_ptr5; //@line 1820 "dictionary.cpp"
                  var $20=HEAP8[($19)]; //@line 1820 "dictionary.cpp"
                  $rb=$20; //@line 1820 "dictionary.cpp"
                  var $21=$rb; //@line 1822 "dictionary.cpp"
                  var $conv6=((($21))&255); //@line 1822 "dictionary.cpp"
                  var $cmp7=((($conv6))|0) <= 9; //@line 1822 "dictionary.cpp"
                  if ($cmp7) { __label__ = 7;; } else { __label__ = 33;; } //@line 1822 "dictionary.cpp"
                  $if_then8$$if_end70$12: do { 
                    if (__label__ == 7) {
  
                      var $22=$rb; //@line 1824 "dictionary.cpp"
                      var $conv9=((($22))&255); //@line 1824 "dictionary.cpp"
                      if ($conv9 == 0) {
                        __label__ = 8;;
                      }
                      else if ($conv9 == 8) {
                        __label__ = 18;;
                      }
                      else if ($conv9 == 1) {
                        __label__ = 19;;
                      }
                      else if ($conv9 == 2) {
                        __label__ = 21;;
                      }
                      else if ($conv9 == 3) {
                        __label__ = 22;;
                      }
                      else if ($conv9 == 4) {
                        __label__ = 23;;
                      }
                      else if ($conv9 == 5) {
                        __label__ = 24;;
                      }
                      else if ($conv9 == 9) {
                        __label__ = 32;;
                      }
                      else {
                      __label__ = 30;break $if_then8$$if_end70$12;
                      }
                      
                      if (__label__ == 8) {
  
                        var $23=$common_phonemes; //@line 1827 "dictionary.cpp"
                        var $cmp10=((($23))|0)!=0; //@line 1827 "dictionary.cpp"
                        if ($cmp10) { __label__ = 9;; } else { __label__ = 16;; } //@line 1827 "dictionary.cpp"
                        $if_then11$$if_else$16: do { 
                          if (__label__ == 9) {
  
                            var $24=$common_phonemes; //@line 1829 "dictionary.cpp"
                            HEAP32[(($phonemes)>>2)]=$24; //@line 1829 "dictionary.cpp"
                            ; //@line 1830 "dictionary.cpp"
                            while(1) { 
  
                              var $25=HEAP32[(($phonemes13)>>2)]; //@line 1830 "dictionary.cpp"
                              var $incdec_ptr14=(($25+1)&4294967295); //@line 1830 "dictionary.cpp"
                              HEAP32[(($phonemes13)>>2)]=$incdec_ptr14; //@line 1830 "dictionary.cpp"
                              var $26=HEAPU8[($25)]; //@line 1830 "dictionary.cpp"
                              $rb=$26; //@line 1830 "dictionary.cpp"
                              var $conv15=((($26))&255); //@line 1830 "dictionary.cpp"
                              var $cmp16=((($conv15))|0)!=0; //@line 1830 "dictionary.cpp"
                              if (!($cmp16)) { __label__ = 17;break $if_then11$$if_else$16; } //@line 1830 "dictionary.cpp"
  
                              var $27=$rb; //@line 1830 "dictionary.cpp"
                              var $conv17=((($27))&255); //@line 1830 "dictionary.cpp"
                              var $cmp18=((($conv17))|0)!=3; //@line 1830 "dictionary.cpp"
                              if (!($cmp18)) { __label__ = 17;break $if_then11$$if_else$16; }
  
                              var $28=$rb; //@line 1832 "dictionary.cpp"
                              var $conv20=((($28))&255); //@line 1832 "dictionary.cpp"
                              var $cmp21=((($conv20))|0)==5; //@line 1832 "dictionary.cpp"
                              if ($cmp21) { __label__ = 13;; } else { __label__ = 14;; } //@line 1832 "dictionary.cpp"
                              if (__label__ == 13) {
  
                                var $29=HEAP32[(($phonemes23)>>2)]; //@line 1833 "dictionary.cpp"
                                var $incdec_ptr24=(($29+1)&4294967295); //@line 1833 "dictionary.cpp"
                                HEAP32[(($phonemes23)>>2)]=$incdec_ptr24; //@line 1833 "dictionary.cpp"
                                ; //@line 1833 "dictionary.cpp"
                              }
  
                              var $30=$rb; //@line 1834 "dictionary.cpp"
                              var $conv26=((($30))&255); //@line 1834 "dictionary.cpp"
                              var $cmp27=((($conv26))|0)==9; //@line 1834 "dictionary.cpp"
                              if (!($cmp27)) { __label__ = 10;continue ; } //@line 1834 "dictionary.cpp"
  
                              var $31=HEAP32[(($phonemes29)>>2)]; //@line 1835 "dictionary.cpp"
                              var $add_ptr30=(($31+2)&4294967295); //@line 1835 "dictionary.cpp"
                              HEAP32[(($phonemes29)>>2)]=$add_ptr30; //@line 1835 "dictionary.cpp"
                              __label__ = 10;continue ; //@line 1835 "dictionary.cpp"
                            }
                          }
                          else if (__label__ == 16) {
  
                            HEAP32[(($phonemes32)>>2)]=((__str13643)&4294967295); //@line 1840 "dictionary.cpp"
                            ;
                          }
                        } while(0);
  
                        var $32=$rule_addr; //@line 1842 "dictionary.cpp"
                        var $incdec_ptr34=(($32+-1)&4294967295); //@line 1842 "dictionary.cpp"
                        $rule_addr=$incdec_ptr34; //@line 1842 "dictionary.cpp"
                        $failed=2; //@line 1843 "dictionary.cpp"
                        ; //@line 1844 "dictionary.cpp"
                      }
                      else if (__label__ == 18) {
  
                        $check_atstart=1; //@line 1847 "dictionary.cpp"
                        $unpron_ignore=0; //@line 1848 "dictionary.cpp"
                        $match_type=1; //@line 1849 "dictionary.cpp"
                        ; //@line 1850 "dictionary.cpp"
                      }
                      else if (__label__ == 19) {
  
                        $match_type=1; //@line 1853 "dictionary.cpp"
                        var $33=$word_flags_addr; //@line 1854 "dictionary.cpp"
                        var $and37=($33) & -2147483648; //@line 1854 "dictionary.cpp"
                        var $tobool38=((($and37))|0)!=0; //@line 1854 "dictionary.cpp"
                        if (!($tobool38)) { __label__ = 30;break $if_then8$$if_end70$12; } //@line 1854 "dictionary.cpp"
  
                        $failed=1; //@line 1859 "dictionary.cpp"
                        ; //@line 1860 "dictionary.cpp"
                      }
                      else if (__label__ == 21) {
  
                        $match_type=2; //@line 1864 "dictionary.cpp"
                        ; //@line 1865 "dictionary.cpp"
                      }
                      else if (__label__ == 22) {
  
                        var $34=$rule_addr; //@line 1867 "dictionary.cpp"
                        HEAP32[(($phonemes43)>>2)]=$34; //@line 1867 "dictionary.cpp"
                        $failed=2; //@line 1868 "dictionary.cpp"
                        ; //@line 1869 "dictionary.cpp"
                      }
                      else if (__label__ == 23) {
  
                        var $35=$rule_addr; //@line 1871 "dictionary.cpp"
                        $common_phonemes=$35; //@line 1871 "dictionary.cpp"
                        ; //@line 1872 "dictionary.cpp"
                      }
                      else if (__label__ == 24) {
  
                        var $36=$rule_addr; //@line 1875 "dictionary.cpp"
                        var $incdec_ptr46=(($36+1)&4294967295); //@line 1875 "dictionary.cpp"
                        $rule_addr=$incdec_ptr46; //@line 1875 "dictionary.cpp"
                        var $37=HEAP8[($36)]; //@line 1875 "dictionary.cpp"
                        $condition_num=$37; //@line 1875 "dictionary.cpp"
                        var $38=$condition_num; //@line 1877 "dictionary.cpp"
                        var $conv47=((($38))&255); //@line 1877 "dictionary.cpp"
                        var $cmp48=((($conv47))|0) >= 32; //@line 1877 "dictionary.cpp"
                        var $39=$tr_addr; //@line 1880 "dictionary.cpp"
                        var $dict_condition=(($39+692)&4294967295); //@line 1880 "dictionary.cpp"
                        var $40=HEAP32[(($dict_condition)>>2)]; //@line 1880 "dictionary.cpp"
                        var $41=$condition_num; //@line 1880 "dictionary.cpp"
                        var $conv50=((($41))&255); //@line 1880 "dictionary.cpp"
                        if ($cmp48) { __label__ = 25;; } else { __label__ = 27;; } //@line 1877 "dictionary.cpp"
                        $if_then49$$if_else55$35: do { 
                          if (__label__ == 25) {
  
                            var $sub=((($conv50)-32)&4294967295); //@line 1880 "dictionary.cpp"
                            var $shl=1 << ($sub); //@line 1880 "dictionary.cpp"
                            var $and51=($shl) & ($40); //@line 1880 "dictionary.cpp"
                            var $cmp52=((($and51))|0)!=0; //@line 1880 "dictionary.cpp"
                            if (!($cmp52)) { __label__ = 29;break $if_then49$$if_else55$35; } //@line 1880 "dictionary.cpp"
  
                            $failed=1; //@line 1881 "dictionary.cpp"
                            __label__ = 30;break $if_then8$$if_end70$12; //@line 1881 "dictionary.cpp"
                          }
                          else if (__label__ == 27) {
  
                            var $shl58=1 << ($conv50); //@line 1886 "dictionary.cpp"
                            var $and59=($shl58) & ($40); //@line 1886 "dictionary.cpp"
                            var $cmp60=((($and59))|0)==0; //@line 1886 "dictionary.cpp"
                            if (!($cmp60)) { __label__ = 29;break $if_then49$$if_else55$35; } //@line 1886 "dictionary.cpp"
  
                            $failed=1; //@line 1887 "dictionary.cpp"
                            __label__ = 30;break $if_then8$$if_end70$12; //@line 1887 "dictionary.cpp"
                          }
                        } while(0);
  
                        var $_pr=$failed; //@line 1890 "dictionary.cpp"
                        var $tobool64=((($_pr))|0)!=0; //@line 1890 "dictionary.cpp"
                        if ($tobool64) { __label__ = 30;break $if_then8$$if_end70$12; } //@line 1890 "dictionary.cpp"
  
                        var $43=HEAP32[(($points66)>>2)]; //@line 1891 "dictionary.cpp"
                        var $inc=((($43)+1)&4294967295); //@line 1891 "dictionary.cpp"
                        HEAP32[(($points66)>>2)]=$inc; //@line 1891 "dictionary.cpp"
                        ; //@line 1891 "dictionary.cpp"
                      }
                      else if (__label__ == 32) {
  
                        var $44=$rule_addr; //@line 1894 "dictionary.cpp"
                        var $add_ptr69=(($44+2)&4294967295); //@line 1894 "dictionary.cpp"
                        $rule_addr=$add_ptr69; //@line 1894 "dictionary.cpp"
                        ; //@line 1895 "dictionary.cpp"
                      }
                    }
                    else if (__label__ == 33) {
  
                      $add_points=0; //@line 1900 "dictionary.cpp"
                      var $45=$match_type; //@line 1902 "dictionary.cpp"
                      if ($45 == 0) {
                        __label__ = 34;;
                      }
                      else if ($45 == 2) {
                        __label__ = 41;;
                      }
                      else if ($45 == 1) {
                        __label__ = 119;;
                      }
                      else {
                      __label__ = 181;;
                      }
                      
                      $sw_epilog496$$sw_bb71$$sw_bb89$$sw_bb328$44: do { 
                        if (__label__ == 34) {
  
                          var $46=$letter; //@line 1906 "dictionary.cpp"
                          $last_letter=$46; //@line 1906 "dictionary.cpp"
                          var $47=$post_ptr; //@line 1907 "dictionary.cpp"
                          var $incdec_ptr72=(($47+1)&4294967295); //@line 1907 "dictionary.cpp"
                          $post_ptr=$incdec_ptr72; //@line 1907 "dictionary.cpp"
                          var $48=HEAP8[($47)]; //@line 1907 "dictionary.cpp"
                          $letter=$48; //@line 1907 "dictionary.cpp"
                          var $49=$letter; //@line 1909 "dictionary.cpp"
                          var $conv73=((($49))&255); //@line 1909 "dictionary.cpp"
                          var $50=$rb; //@line 1909 "dictionary.cpp"
                          var $conv74=((($50))&255); //@line 1909 "dictionary.cpp"
                          var $cmp75=((($conv73))|0)==((($conv74))|0); //@line 1909 "dictionary.cpp"
                          if ($cmp75) { __label__ = 37;; } else { __label__ = 35;; } //@line 1909 "dictionary.cpp"
                          $if_then80$$lor_lhs_false$46: do { 
                            if (__label__ == 35) {
  
                              var $51=$letter; //@line 1909 "dictionary.cpp"
                              var $conv76=((($51))&255); //@line 1909 "dictionary.cpp"
                              var $cmp77=((($conv76))|0)==69; //@line 1909 "dictionary.cpp"
                              if ($cmp77) { __label__ = 36;; } else { __label__ = 40;; } //@line 1909 "dictionary.cpp"
                              if (__label__ == 36) {
  
                                var $52=$rb; //@line 1909 "dictionary.cpp"
                                var $conv78=((($52))&255); //@line 1909 "dictionary.cpp"
                                var $cmp79=((($conv78))|0)==101; //@line 1909 "dictionary.cpp"
                                if ($cmp79) { __label__ = 37;break $if_then80$$lor_lhs_false$46; } //@line 1909 "dictionary.cpp"
                              }
  
                              $failed=1; //@line 1916 "dictionary.cpp"
                              __label__ = 30;break $if_then8$$if_end70$12;
                            }
                          } while(0);
  
                          var $53=$letter; //@line 1911 "dictionary.cpp"
                          var $conv81=((($53))&255); //@line 1911 "dictionary.cpp"
                          var $and82=($conv81) & 192; //@line 1911 "dictionary.cpp"
                          var $cmp83=((($and82))|0)!=128; //@line 1911 "dictionary.cpp"
                          if ($cmp83) { __label__ = 38;; } else { __label__ = 39;; } //@line 1911 "dictionary.cpp"
                          if (__label__ == 38) {
  
                            $add_points=21; //@line 1912 "dictionary.cpp"
                            ; //@line 1912 "dictionary.cpp"
                          }
  
                          var $54=$consumed; //@line 1913 "dictionary.cpp"
                          var $inc86=((($54)+1)&4294967295); //@line 1913 "dictionary.cpp"
                          $consumed=$inc86; //@line 1913 "dictionary.cpp"
                          ; //@line 1914 "dictionary.cpp"
                        }
                        else if (__label__ == 41) {
  
                          var $55=$distance_right; //@line 1922 "dictionary.cpp"
                          var $add=((($55)+6)&4294967295); //@line 1922 "dictionary.cpp"
                          $distance_right=$add; //@line 1922 "dictionary.cpp"
                          var $56=$distance_right; //@line 1923 "dictionary.cpp"
                          var $cmp90=((($56))|0) > 18; //@line 1923 "dictionary.cpp"
                          if ($cmp90) { __label__ = 42;; } else { __label__ = 43;; } //@line 1923 "dictionary.cpp"
                          if (__label__ == 42) {
  
                            $distance_right=19; //@line 1924 "dictionary.cpp"
                            ; //@line 1924 "dictionary.cpp"
                          }
  
                          var $57=$letter; //@line 1925 "dictionary.cpp"
                          $last_letter=$57; //@line 1925 "dictionary.cpp"
                          var $58=$post_ptr; //@line 1926 "dictionary.cpp"
                          var $call=__Z7utf8_inPiPKc($letter_w, $58); //@line 1926 "dictionary.cpp"
                          var $sub93=((($call)-1)&4294967295); //@line 1926 "dictionary.cpp"
                          $letter_xbytes=$sub93; //@line 1926 "dictionary.cpp"
                          var $59=$post_ptr; //@line 1927 "dictionary.cpp"
                          var $incdec_ptr94=(($59+1)&4294967295); //@line 1927 "dictionary.cpp"
                          $post_ptr=$incdec_ptr94; //@line 1927 "dictionary.cpp"
                          var $60=HEAP8[($59)]; //@line 1927 "dictionary.cpp"
                          $letter=$60; //@line 1927 "dictionary.cpp"
                          var $61=$rb; //@line 1929 "dictionary.cpp"
                          var $conv95=((($61))&255); //@line 1929 "dictionary.cpp"
                          if ($conv95 == 17) {
                            __label__ = 44;;
                          }
                          else if ($conv95 == 18) {
                            __label__ = 49;;
                          }
                          else if ($conv95 == 25) {
                            __label__ = 52;;
                          }
                          else if ($conv95 == 15) {
                            __label__ = 57;;
                          }
                          else if ($conv95 == 16) {
                            __label__ = 62;;
                          }
                          else if ($conv95 == 11) {
                            __label__ = 65;;
                          }
                          else if ($conv95 == 28) {
                            __label__ = 68;;
                          }
                          else if ($conv95 == 45) {
                            __label__ = 78;;
                          }
                          else if ($conv95 == 21) {
                            __label__ = 83;;
                          }
                          else if ($conv95 == 29) {
                            __label__ = 94;;
                          }
                          else if ($conv95 == 23) {
                            __label__ = 101;;
                          }
                          else if ($conv95 == 12) {
                            __label__ = 105;;
                          }
                          else if ($conv95 == 13) {
                            __label__ = 106;;
                          }
                          else if ($conv95 == 14) {
                            __label__ = 111;;
                          }
                          else if ($conv95 == 24) {
                            __label__ = 112;;
                          }
                          else {
                          __label__ = 115;;
                          }
                          
                          if (__label__ == 115) {
  
                            var $186=$letter; //@line 2141 "dictionary.cpp"
                            var $conv315=((($186))&255); //@line 2141 "dictionary.cpp"
                            var $187=$rb; //@line 2141 "dictionary.cpp"
                            var $conv316=((($187))&255); //@line 2141 "dictionary.cpp"
                            var $cmp317=((($conv315))|0)==((($conv316))|0); //@line 2141 "dictionary.cpp"
                            if ($cmp317) { __label__ = 116;; } else { __label__ = 118;; } //@line 2141 "dictionary.cpp"
                            if (__label__ == 116) {
  
                              var $188=$letter; //@line 2143 "dictionary.cpp"
                              var $conv319=((($188))&255); //@line 2143 "dictionary.cpp"
                              var $and320=($conv319) & 192; //@line 2143 "dictionary.cpp"
                              var $cmp321=((($and320))|0)!=128; //@line 2143 "dictionary.cpp"
                              if (!($cmp321)) { __label__ = 181;break $sw_epilog496$$sw_bb71$$sw_bb89$$sw_bb328$44; } //@line 2143 "dictionary.cpp"
  
                              var $189=$distance_right; //@line 2146 "dictionary.cpp"
                              var $sub323=((21-($189))&4294967295); //@line 2146 "dictionary.cpp"
                              $add_points=$sub323; //@line 2146 "dictionary.cpp"
                              ; //@line 2147 "dictionary.cpp"
                            }
                            else if (__label__ == 118) {
  
                              $failed=1; //@line 2150 "dictionary.cpp"
                              __label__ = 30;break $if_then8$$if_end70$12;
                            }
                          }
                          else if (__label__ == 44) {
  
                            var $62=$rule_addr; //@line 1932 "dictionary.cpp"
                            var $incdec_ptr97=(($62+1)&4294967295); //@line 1932 "dictionary.cpp"
                            $rule_addr=$incdec_ptr97; //@line 1932 "dictionary.cpp"
                            var $63=HEAP8[($62)]; //@line 1932 "dictionary.cpp"
                            var $conv98=(tempInt=(($63)),(tempInt>=128?tempInt-256:tempInt)); //@line 1932 "dictionary.cpp"
                            var $sub99=((($conv98)-65)&4294967295); //@line 1932 "dictionary.cpp"
                            $letter_group=$sub99; //@line 1932 "dictionary.cpp"
                            var $64=$tr_addr; //@line 1933 "dictionary.cpp"
                            var $65=HEAP32[(($letter_w)>>2)]; //@line 1933 "dictionary.cpp"
                            var $66=$letter_group; //@line 1933 "dictionary.cpp"
                            var $call100=__ZL8IsLetterP10Translatorii($64, $65, $66); //@line 1933 "dictionary.cpp"
                            var $tobool101=((($call100))|0)!=0; //@line 1933 "dictionary.cpp"
                            if ($tobool101) { __label__ = 45;; } else { __label__ = 48;; } //@line 1933 "dictionary.cpp"
                            if (__label__ == 45) {
  
                              $lg_pts=20; //@line 1935 "dictionary.cpp"
                              var $67=$letter_group; //@line 1936 "dictionary.cpp"
                              var $cmp103=((($67))|0)==2; //@line 1936 "dictionary.cpp"
                              if ($cmp103) { __label__ = 46;; } else { __label__ = 47;; } //@line 1936 "dictionary.cpp"
                              if (__label__ == 46) {
  
                                $lg_pts=19; //@line 1937 "dictionary.cpp"
                                ; //@line 1937 "dictionary.cpp"
                              }
  
                              var $68=$lg_pts; //@line 1938 "dictionary.cpp"
                              var $69=$distance_right; //@line 1938 "dictionary.cpp"
                              var $sub106=((($68)-($69))&4294967295); //@line 1938 "dictionary.cpp"
                              $add_points=$sub106; //@line 1938 "dictionary.cpp"
                              var $70=$letter_xbytes; //@line 1939 "dictionary.cpp"
                              var $71=$post_ptr; //@line 1939 "dictionary.cpp"
                              var $add_ptr107=(($71+$70)&4294967295); //@line 1939 "dictionary.cpp"
                              $post_ptr=$add_ptr107; //@line 1939 "dictionary.cpp"
                              ; //@line 1940 "dictionary.cpp"
                            }
                            else if (__label__ == 48) {
  
                              $failed=1; //@line 1942 "dictionary.cpp"
                              __label__ = 30;break $if_then8$$if_end70$12;
                            }
                          }
                          else if (__label__ == 49) {
  
                            var $72=$rule_addr; //@line 1946 "dictionary.cpp"
                            var $incdec_ptr111=(($72+1)&4294967295); //@line 1946 "dictionary.cpp"
                            $rule_addr=$incdec_ptr111; //@line 1946 "dictionary.cpp"
                            var $73=HEAP8[($72)]; //@line 1946 "dictionary.cpp"
                            var $conv112=(tempInt=(($73)),(tempInt>=128?tempInt-256:tempInt)); //@line 1946 "dictionary.cpp"
                            var $sub113=((($conv112)-65)&4294967295); //@line 1946 "dictionary.cpp"
                            $letter_group=$sub113; //@line 1946 "dictionary.cpp"
                            var $74=$tr_addr; //@line 1947 "dictionary.cpp"
                            var $75=$post_ptr; //@line 1947 "dictionary.cpp"
                            var $add_ptr114=(($75+-1)&4294967295); //@line 1947 "dictionary.cpp"
                            var $76=$letter_group; //@line 1947 "dictionary.cpp"
                            var $call115=__ZL13IsLetterGroupP10TranslatorPcii($74, $add_ptr114, $76, 0); //@line 1947 "dictionary.cpp"
                            $n_bytes=$call115; //@line 1947 "dictionary.cpp"
                            var $cmp116=((($call115))|0) > 0; //@line 1947 "dictionary.cpp"
                            if ($cmp116) { __label__ = 50;; } else { __label__ = 51;; } //@line 1947 "dictionary.cpp"
                            if (__label__ == 50) {
  
                              var $77=$distance_right; //@line 1949 "dictionary.cpp"
                              var $sub118=((20-($77))&4294967295); //@line 1949 "dictionary.cpp"
                              $add_points=$sub118; //@line 1949 "dictionary.cpp"
                              var $78=$n_bytes; //@line 1950 "dictionary.cpp"
                              var $sub119=((($78)-1)&4294967295); //@line 1950 "dictionary.cpp"
                              var $79=$post_ptr; //@line 1950 "dictionary.cpp"
                              var $add_ptr120=(($79+$sub119)&4294967295); //@line 1950 "dictionary.cpp"
                              $post_ptr=$add_ptr120; //@line 1950 "dictionary.cpp"
                              ; //@line 1951 "dictionary.cpp"
                            }
                            else if (__label__ == 51) {
  
                              $failed=1; //@line 1953 "dictionary.cpp"
                              __label__ = 30;break $if_then8$$if_end70$12;
                            }
                          }
                          else if (__label__ == 52) {
  
                            var $80=$tr_addr; //@line 1957 "dictionary.cpp"
                            var $81=HEAP32[(($letter_w)>>2)]; //@line 1957 "dictionary.cpp"
                            var $call124=__ZL8IsLetterP10Translatorii($80, $81, 0); //@line 1957 "dictionary.cpp"
                            var $tobool125=((($call124))|0)!=0; //@line 1957 "dictionary.cpp"
                            if ($tobool125) { __label__ = 55;; } else { __label__ = 53;; } //@line 1957 "dictionary.cpp"
                            $if_then131$$lor_lhs_false126$77: do { 
                              if (__label__ == 53) {
  
                                var $82=HEAP32[(($letter_w)>>2)]; //@line 1957 "dictionary.cpp"
                                var $cmp127=((($82))|0)==32; //@line 1957 "dictionary.cpp"
                                if ($cmp127) { __label__ = 54;; } else { __label__ = 56;; } //@line 1957 "dictionary.cpp"
                                if (__label__ == 54) {
  
                                  var $83=$word_flags_addr; //@line 1957 "dictionary.cpp"
                                  var $and129=($83) & 134217728; //@line 1957 "dictionary.cpp"
                                  var $tobool130=((($and129))|0)!=0; //@line 1957 "dictionary.cpp"
                                  if ($tobool130) { __label__ = 55;break $if_then131$$lor_lhs_false126$77; } //@line 1957 "dictionary.cpp"
                                }
  
                                var $84=$distance_right; //@line 1963 "dictionary.cpp"
                                var $sub133=((20-($84))&4294967295); //@line 1963 "dictionary.cpp"
                                $add_points=$sub133; //@line 1963 "dictionary.cpp"
                                var $85=$letter_xbytes; //@line 1964 "dictionary.cpp"
                                var $86=$post_ptr; //@line 1964 "dictionary.cpp"
                                var $add_ptr134=(($86+$85)&4294967295); //@line 1964 "dictionary.cpp"
                                $post_ptr=$add_ptr134; //@line 1964 "dictionary.cpp"
                                __label__ = 181;break $sw_epilog496$$sw_bb71$$sw_bb89$$sw_bb328$44;
                              }
                            } while(0);
  
                            $failed=1; //@line 1959 "dictionary.cpp"
                            __label__ = 30;break $if_then8$$if_end70$12; //@line 1960 "dictionary.cpp"
                          }
                          else if (__label__ == 57) {
  
                            var $87=HEAP32[(($letter_w)>>2)]; //@line 1969 "dictionary.cpp"
                            var $call137=__Z7IsDigitj($87); //@line 1969 "dictionary.cpp"
                            var $tobool138=((($call137))|0)!=0; //@line 1969 "dictionary.cpp"
                            if ($tobool138) { __label__ = 58;; } else { __label__ = 59;; } //@line 1969 "dictionary.cpp"
                            if (__label__ == 58) {
  
                              var $88=$distance_right; //@line 1971 "dictionary.cpp"
                              var $sub140=((20-($88))&4294967295); //@line 1971 "dictionary.cpp"
                              $add_points=$sub140; //@line 1971 "dictionary.cpp"
                              var $89=$letter_xbytes; //@line 1972 "dictionary.cpp"
                              var $90=$post_ptr; //@line 1972 "dictionary.cpp"
                              var $add_ptr141=(($90+$89)&4294967295); //@line 1972 "dictionary.cpp"
                              $post_ptr=$add_ptr141; //@line 1972 "dictionary.cpp"
                              ; //@line 1973 "dictionary.cpp"
                            }
                            else if (__label__ == 59) {
  
                              var $91=$tr_addr; //@line 1975 "dictionary.cpp"
                              var $langopts=(($91)&4294967295); //@line 1975 "dictionary.cpp"
                              var $tone_numbers=(($langopts+250)&4294967295); //@line 1975 "dictionary.cpp"
                              var $92=HEAP8[($tone_numbers)]; //@line 1975 "dictionary.cpp"
                              var $tobool143=(tempInt=(($92)),(tempInt>=128?tempInt-256:tempInt))!=0; //@line 1975 "dictionary.cpp"
                              if ($tobool143) { __label__ = 60;; } else { __label__ = 61;; } //@line 1975 "dictionary.cpp"
                              if (__label__ == 60) {
  
                                var $93=$distance_right; //@line 1978 "dictionary.cpp"
                                var $sub145=((20-($93))&4294967295); //@line 1978 "dictionary.cpp"
                                $add_points=$sub145; //@line 1978 "dictionary.cpp"
                                var $94=$post_ptr; //@line 1979 "dictionary.cpp"
                                var $incdec_ptr146=(($94+-1)&4294967295); //@line 1979 "dictionary.cpp"
                                $post_ptr=$incdec_ptr146; //@line 1979 "dictionary.cpp"
                                ; //@line 1980 "dictionary.cpp"
                              }
                              else if (__label__ == 61) {
  
                                $failed=1; //@line 1982 "dictionary.cpp"
                                __label__ = 30;break $if_then8$$if_end70$12;
                              }
                            }
                          }
                          else if (__label__ == 62) {
  
                            var $95=HEAP32[(($letter_w)>>2)]; //@line 1986 "dictionary.cpp"
                            var $call151=_iswalpha($95); //@line 1986 "dictionary.cpp"
                            var $tobool152=((($call151))|0)!=0; //@line 1986 "dictionary.cpp"
                            if ($tobool152) { __label__ = 64;; } else { __label__ = 63;; } //@line 1986 "dictionary.cpp"
                            if (__label__ == 64) {
  
                              $failed=1; //@line 1992 "dictionary.cpp"
                              __label__ = 30;break $if_then8$$if_end70$12;
                            }
                            else if (__label__ == 63) {
  
                              var $96=$distance_right; //@line 1988 "dictionary.cpp"
                              var $sub154=((21-($96))&4294967295); //@line 1988 "dictionary.cpp"
                              $add_points=$sub154; //@line 1988 "dictionary.cpp"
                              var $97=$letter_xbytes; //@line 1989 "dictionary.cpp"
                              var $98=$post_ptr; //@line 1989 "dictionary.cpp"
                              var $add_ptr155=(($98+$97)&4294967295); //@line 1989 "dictionary.cpp"
                              $post_ptr=$add_ptr155; //@line 1989 "dictionary.cpp"
                              ; //@line 1990 "dictionary.cpp"
                            }
                          }
                          else if (__label__ == 65) {
  
                            var $99=$letter; //@line 1996 "dictionary.cpp"
                            var $conv159=((($99))&255); //@line 1996 "dictionary.cpp"
                            var $100=$last_letter; //@line 1996 "dictionary.cpp"
                            var $conv160=((($100))&255); //@line 1996 "dictionary.cpp"
                            var $cmp161=((($conv159))|0)==((($conv160))|0); //@line 1996 "dictionary.cpp"
                            if ($cmp161) { __label__ = 66;; } else { __label__ = 67;; } //@line 1996 "dictionary.cpp"
                            if (__label__ == 66) {
  
                              var $101=$distance_right; //@line 1997 "dictionary.cpp"
                              var $sub163=((21-($101))&4294967295); //@line 1997 "dictionary.cpp"
                              $add_points=$sub163; //@line 1997 "dictionary.cpp"
                              ; //@line 1997 "dictionary.cpp"
                            }
                            else if (__label__ == 67) {
  
                              $failed=1; //@line 1999 "dictionary.cpp"
                              __label__ = 30;break $if_then8$$if_end70$12;
                            }
                          }
                          else if (__label__ == 68) {
  
                            var $102=$rule_addr; //@line 2003 "dictionary.cpp"
                            var $incdec_ptr167=(($102+1)&4294967295); //@line 2003 "dictionary.cpp"
                            $rule_addr=$incdec_ptr167; //@line 2003 "dictionary.cpp"
                            var $103=HEAP8[($102)]; //@line 2003 "dictionary.cpp"
                            var $conv168=(tempInt=(($103)),(tempInt>=128?tempInt-256:tempInt)); //@line 2003 "dictionary.cpp"
                            $command=$conv168; //@line 2003 "dictionary.cpp"
                            var $104=$command; //@line 2004 "dictionary.cpp"
                            var $cmp169=((($104))|0)==1; //@line 2004 "dictionary.cpp"
                            if ($cmp169) { __label__ = 69;; } else { __label__ = 70;; } //@line 2004 "dictionary.cpp"
                            if (__label__ == 69) {
  
                              HEAP32[(($end_type171)>>2)]=32768; //@line 2006 "dictionary.cpp"
                              ; //@line 2007 "dictionary.cpp"
                            }
                            else if (__label__ == 70) {
  
                              var $105=$command; //@line 2009 "dictionary.cpp"
                              var $and173=($105) & 240; //@line 2009 "dictionary.cpp"
                              var $cmp174=((($and173))|0)==16; //@line 2009 "dictionary.cpp"
                              if ($cmp174) { __label__ = 71;; } else { __label__ = 74;; } //@line 2009 "dictionary.cpp"
                              if (__label__ == 71) {
  
                                var $106=$dict_flags_addr; //@line 2012 "dictionary.cpp"
                                var $107=$command; //@line 2012 "dictionary.cpp"
                                var $and176=($107) & 15; //@line 2012 "dictionary.cpp"
                                var $add177=((($and176)+19)&4294967295); //@line 2012 "dictionary.cpp"
                                var $shl178=1 << ($add177); //@line 2012 "dictionary.cpp"
                                var $and179=($shl178) & ($106); //@line 2012 "dictionary.cpp"
                                var $tobool180=((($and179))|0)!=0; //@line 2012 "dictionary.cpp"
                                if ($tobool180) { __label__ = 72;; } else { __label__ = 73;; } //@line 2012 "dictionary.cpp"
                                if (__label__ == 72) {
  
                                  $add_points=23; //@line 2013 "dictionary.cpp"
                                  ; //@line 2013 "dictionary.cpp"
                                }
                                else if (__label__ == 73) {
  
                                  $failed=1; //@line 2015 "dictionary.cpp"
                                  __label__ = 30;break $if_then8$$if_end70$12;
                                }
                              }
                              else if (__label__ == 74) {
  
                                var $108=$command; //@line 2018 "dictionary.cpp"
                                var $and185=($108) & 240; //@line 2018 "dictionary.cpp"
                                var $cmp186=((($and185))|0)==32; //@line 2018 "dictionary.cpp"
                                if (!($cmp186)) { __label__ = 181;break $sw_epilog496$$sw_bb71$$sw_bb89$$sw_bb328$44; } //@line 2018 "dictionary.cpp"
  
                                var $109=$word_addr; //@line 2022 "dictionary.cpp"
                                var $110=HEAP32[(($109)>>2)]; //@line 2022 "dictionary.cpp"
                                var $111=$word_start_addr; //@line 2022 "dictionary.cpp"
                                var $sub_ptr_lhs_cast=($110); //@line 2022 "dictionary.cpp"
                                var $sub_ptr_rhs_cast=($111); //@line 2022 "dictionary.cpp"
                                var $sub_ptr_rhs_cast_neg=(((-($sub_ptr_rhs_cast)))&4294967295);
                                var $112=$consumed; //@line 2022 "dictionary.cpp"
                                var $113=$group_length_addr; //@line 2022 "dictionary.cpp"
                                var $sub_ptr_sub=((($sub_ptr_lhs_cast)+1)&4294967295); //@line 2022 "dictionary.cpp"
                                var $add188=((($sub_ptr_sub)+($112))&4294967295); //@line 2022 "dictionary.cpp"
                                var $add189=((($add188)+($sub_ptr_rhs_cast_neg))&4294967295); //@line 2022 "dictionary.cpp"
                                var $add190=((($add189)+($113))&4294967295); //@line 2022 "dictionary.cpp"
                                $ix=$add190; //@line 2022 "dictionary.cpp"
                                var $114=$word_start_addr; //@line 2023 "dictionary.cpp"
                                var $add_ptr191=(($114+-1)&4294967295); //@line 2023 "dictionary.cpp"
                                var $115=$ix; //@line 2023 "dictionary.cpp"
                                var $src$, $dest$, $stop$, $stop4$, $fast$;
                                $src$ = $add_ptr191;
                                $dest$ = $arraydecay;
                                $stop$ = $src$ + $115;
                                if (($dest$%4) == ($src$%4) && $115 > 8) {
                                  while ($src$%4 !== 0 && $src$ < $stop$) {
                                    ; HEAP8[$dest$++] = HEAP8[$src$++];
                                  }
                                  $src$ >>= 2;
                                  $dest$ >>= 2;
                                  $stop4$ = $stop$ >> 2;
                                  while ($src$ < $stop4$) {
                                ;;;    HEAP32[$dest$++] = HEAP32[$src$++];
                                  }
                                  $src$ <<= 2;
                                  $dest$ <<= 2;
                                }while ($src$ < $stop$) {
                                  ; HEAP8[$dest$++] = HEAP8[$src$++];
                                }; //@line 2023 "dictionary.cpp"
                                var $116=$ix; //@line 2024 "dictionary.cpp"
                                var $arrayidx192=(($word_buf+$116)&4294967295); //@line 2024 "dictionary.cpp"
                                HEAP8[($arrayidx192)]=32; //@line 2024 "dictionary.cpp"
                                var $117=$ix; //@line 2025 "dictionary.cpp"
                                var $add193=((($117)+1)&4294967295); //@line 2025 "dictionary.cpp"
                                var $arrayidx194=(($word_buf+$add193)&4294967295); //@line 2025 "dictionary.cpp"
                                HEAP8[($arrayidx194)]=0; //@line 2025 "dictionary.cpp"
                                var $118=$tr_addr; //@line 2027 "dictionary.cpp"
                                var $call196=__Z11LookupFlagsP10TranslatorPKc($118, $arrayidx195); //@line 2027 "dictionary.cpp"
                                var $119=$command; //@line 2027 "dictionary.cpp"
                                var $and197=($119) & 15; //@line 2027 "dictionary.cpp"
                                var $add198=((($and197)+19)&4294967295); //@line 2027 "dictionary.cpp"
                                var $shl199=1 << ($add198); //@line 2027 "dictionary.cpp"
                                var $and200=($shl199) & ($call196); //@line 2027 "dictionary.cpp"
                                var $tobool201=((($and200))|0)!=0; //@line 2027 "dictionary.cpp"
                                if ($tobool201) { __label__ = 76;; } else { __label__ = 77;; } //@line 2027 "dictionary.cpp"
                                if (__label__ == 76) {
  
                                  $add_points=23; //@line 2028 "dictionary.cpp"
                                  ; //@line 2028 "dictionary.cpp"
                                }
                                else if (__label__ == 77) {
  
                                  $failed=1; //@line 2030 "dictionary.cpp"
                                  __label__ = 30;break $if_then8$$if_end70$12;
                                }
                              }
                            }
                          }
                          else if (__label__ == 78) {
  
                            var $120=$letter; //@line 2035 "dictionary.cpp"
                            var $conv209=((($120))&255); //@line 2035 "dictionary.cpp"
                            var $cmp210=((($conv209))|0)==45; //@line 2035 "dictionary.cpp"
                            if ($cmp210) { __label__ = 81;; } else { __label__ = 79;; } //@line 2035 "dictionary.cpp"
                            $if_then217$$lor_lhs_false211$113: do { 
                              if (__label__ == 79) {
  
                                var $121=$letter; //@line 2035 "dictionary.cpp"
                                var $conv212=((($121))&255); //@line 2035 "dictionary.cpp"
                                var $cmp213=((($conv212))|0)==32; //@line 2035 "dictionary.cpp"
                                if ($cmp213) { __label__ = 80;; } else { __label__ = 82;; } //@line 2035 "dictionary.cpp"
                                if (__label__ == 80) {
  
                                  var $122=$word_flags_addr; //@line 2035 "dictionary.cpp"
                                  var $and215=($122) & 16384; //@line 2035 "dictionary.cpp"
                                  var $tobool216=((($and215))|0)!=0; //@line 2035 "dictionary.cpp"
                                  if ($tobool216) { __label__ = 81;break $if_then217$$lor_lhs_false211$113; } //@line 2035 "dictionary.cpp"
                                }
  
                                $failed=1; //@line 2040 "dictionary.cpp"
                                __label__ = 30;break $if_then8$$if_end70$12;
                              }
                            } while(0);
  
                            var $123=$distance_right; //@line 2037 "dictionary.cpp"
                            var $sub218=((22-($123))&4294967295); //@line 2037 "dictionary.cpp"
                            $add_points=$sub218; //@line 2037 "dictionary.cpp"
                            ; //@line 2038 "dictionary.cpp"
                          }
                          else if (__label__ == 83) {
  
                            var $124=$post_ptr; //@line 2046 "dictionary.cpp"
                            var $125=$letter_xbytes; //@line 2046 "dictionary.cpp"
                            var $add_ptr223=(($124+$125)&4294967295); //@line 2046 "dictionary.cpp"
                            $p222=$add_ptr223; //@line 2046 "dictionary.cpp"
                            $vowel_count=0; //@line 2047 "dictionary.cpp"
                            $syllable_count=1; //@line 2049 "dictionary.cpp"
                            var $126=$rule_addr; //@line 2050 "dictionary.cpp"
                            var $127=HEAP8[($126)]; //@line 2050 "dictionary.cpp"
                            var $conv22514=(tempInt=(($127)),(tempInt>=128?tempInt-256:tempInt)); //@line 2050 "dictionary.cpp"
                            var $cmp22615=((($conv22514))|0)==21; //@line 2050 "dictionary.cpp"
                            if ($cmp22615) { __label__ = 84;; } else { __label__ = 85;; } //@line 2050 "dictionary.cpp"
                            $while_body227$$while_end230$120: do { 
                              if (__label__ == 84) {
                                while(1) { 
  
                                  var $128=$rule_addr; //@line 2052 "dictionary.cpp"
                                  var $incdec_ptr228=(($128+1)&4294967295); //@line 2052 "dictionary.cpp"
                                  $rule_addr=$incdec_ptr228; //@line 2052 "dictionary.cpp"
                                  var $129=$syllable_count; //@line 2053 "dictionary.cpp"
                                  var $add229=((($129)+1)&4294967295); //@line 2053 "dictionary.cpp"
                                  $syllable_count=$add229; //@line 2053 "dictionary.cpp"
                                  var $130=$rule_addr; //@line 2050 "dictionary.cpp"
                                  var $131=HEAP8[($130)]; //@line 2050 "dictionary.cpp"
                                  var $conv225=(tempInt=(($131)),(tempInt>=128?tempInt-256:tempInt)); //@line 2050 "dictionary.cpp"
                                  var $cmp226=((($conv225))|0)==21; //@line 2050 "dictionary.cpp"
                                  if ($cmp226) { __label__ = 84;continue ; } else { __label__ = 85;break $while_body227$$while_end230$120; } //@line 2050 "dictionary.cpp"
                                }
                              }
                            } while(0);
  
                            $vowel=0; //@line 2055 "dictionary.cpp"
                            var $132=HEAP32[(($letter_w)>>2)]; //@line 2056 "dictionary.cpp"
                            var $cmp23216=((($132))|0)!=32; //@line 2056 "dictionary.cpp"
                            if ($cmp23216) { __lastLabel__ = 85; __label__ = 87;; } else { __lastLabel__ = 85; __label__ = 91;; } //@line 2056 "dictionary.cpp"
                            $while_body233$$while_end244$124: do { 
                              if (__label__ == 87) {
                                $while_body233$125: while(1) { 
  
                                  var $133=__lastLabel__ == 86 ? $_pr24 : (0); //@line 2058 "dictionary.cpp"
                                  var $cmp234=((($133))|0)==0; //@line 2058 "dictionary.cpp"
                                  if ($cmp234) { __label__ = 88;; } else { __label__ = 90;; } //@line 2058 "dictionary.cpp"
                                  $land_lhs_true235$$if_end240$127: do { 
                                    if (__label__ == 88) {
  
                                      var $134=$tr_addr; //@line 2058 "dictionary.cpp"
                                      var $135=HEAP32[(($letter_w)>>2)]; //@line 2058 "dictionary.cpp"
                                      var $call236=__ZL8IsLetterP10Translatorii($134, $135, 7); //@line 2058 "dictionary.cpp"
                                      var $tobool237=((($call236))|0)!=0; //@line 2058 "dictionary.cpp"
                                      if (!($tobool237)) { __label__ = 90;break $land_lhs_true235$$if_end240$127; } //@line 2058 "dictionary.cpp"
  
                                      var $136=$vowel_count; //@line 2061 "dictionary.cpp"
                                      var $inc239=((($136)+1)&4294967295); //@line 2061 "dictionary.cpp"
                                      $vowel_count=$inc239; //@line 2061 "dictionary.cpp"
                                      ; //@line 2062 "dictionary.cpp"
                                    }
                                  } while(0);
  
                                  var $137=$tr_addr; //@line 2063 "dictionary.cpp"
                                  var $138=HEAP32[(($letter_w)>>2)]; //@line 2063 "dictionary.cpp"
                                  var $call241=__ZL8IsLetterP10Translatorii($137, $138, 7); //@line 2063 "dictionary.cpp"
                                  $vowel=$call241; //@line 2063 "dictionary.cpp"
                                  var $139=$p222; //@line 2064 "dictionary.cpp"
                                  var $call242=__Z7utf8_inPiPKc($letter_w, $139); //@line 2064 "dictionary.cpp"
                                  var $140=$p222; //@line 2064 "dictionary.cpp"
                                  var $add_ptr243=(($140+$call242)&4294967295); //@line 2064 "dictionary.cpp"
                                  $p222=$add_ptr243; //@line 2064 "dictionary.cpp"
                                  var $141=HEAP32[(($letter_w)>>2)]; //@line 2056 "dictionary.cpp"
                                  var $cmp232=((($141))|0)!=32; //@line 2056 "dictionary.cpp"
                                  if (!($cmp232)) { __label__ = 91;break $while_body233$$while_end244$124; } //@line 2056 "dictionary.cpp"
  
                                  var $_pr24=$vowel; //@line 2058 "dictionary.cpp"
                                  __lastLabel__ = 86; __label__ = 87;continue $while_body233$125;
                                }
                              }
                            } while(0);
  
                            var $142=$syllable_count; //@line 2066 "dictionary.cpp"
                            var $143=$vowel_count; //@line 2066 "dictionary.cpp"
                            var $cmp245=((($142))|0) <= ((($143))|0); //@line 2066 "dictionary.cpp"
                            if ($cmp245) { __label__ = 92;; } else { __label__ = 93;; } //@line 2066 "dictionary.cpp"
                            if (__label__ == 92) {
  
                              var $144=$syllable_count; //@line 2067 "dictionary.cpp"
                              var $145=$distance_right; //@line 2067 "dictionary.cpp"
                              var $_neg=(((-($145)))&4294967295);
                              var $add247=((($144)+18)&4294967295); //@line 2067 "dictionary.cpp"
                              var $sub248=((($add247)+($_neg))&4294967295); //@line 2067 "dictionary.cpp"
                              $add_points=$sub248; //@line 2067 "dictionary.cpp"
                              ; //@line 2067 "dictionary.cpp"
                            }
                            else if (__label__ == 93) {
  
                              $failed=1; //@line 2069 "dictionary.cpp"
                              __label__ = 30;break $if_then8$$if_end70$12;
                            }
                          }
                          else if (__label__ == 94) {
  
                            var $146=$post_ptr; //@line 2075 "dictionary.cpp"
                            var $147=$letter_xbytes; //@line 2075 "dictionary.cpp"
                            var $add_ptr253=(($146+$147)&4294967295); //@line 2075 "dictionary.cpp"
                            $p252=$add_ptr253; //@line 2075 "dictionary.cpp"
                            ; //@line 2076 "dictionary.cpp"
                            while(1) { 
  
                              var $148=HEAP32[(($letter_w)>>2)]; //@line 2076 "dictionary.cpp"
                              var $cmp255=((($148))|0)!=32; //@line 2076 "dictionary.cpp"
                              if (!($cmp255)) { __label__ = 99;break ; } //@line 2076 "dictionary.cpp"
  
                              var $149=$tr_addr; //@line 2078 "dictionary.cpp"
                              var $150=HEAP32[(($letter_w)>>2)]; //@line 2078 "dictionary.cpp"
                              var $call257=__ZL8IsLetterP10Translatorii($149, $150, 7); //@line 2078 "dictionary.cpp"
                              var $tobool258=((($call257))|0)!=0; //@line 2078 "dictionary.cpp"
                              if ($tobool258) { __label__ = 97;break ; } //@line 2078 "dictionary.cpp"
  
                              var $151=$p252; //@line 2083 "dictionary.cpp"
                              var $call261=__Z7utf8_inPiPKc($letter_w, $151); //@line 2083 "dictionary.cpp"
                              var $152=$p252; //@line 2083 "dictionary.cpp"
                              var $add_ptr262=(($152+$call261)&4294967295); //@line 2083 "dictionary.cpp"
                              $p252=$add_ptr262; //@line 2083 "dictionary.cpp"
                              __label__ = 95;continue ; //@line 2084 "dictionary.cpp"
                            }
                            if (__label__ == 99) {
  
                              var $_pr2=$failed; //@line 2085 "dictionary.cpp"
                              var $tobool264=((($_pr2))|0)!=0; //@line 2085 "dictionary.cpp"
                              if ($tobool264) { __label__ = 30;break $if_then8$$if_end70$12; } //@line 2085 "dictionary.cpp"
  
                              var $153=$distance_right; //@line 2086 "dictionary.cpp"
                              var $sub266=((19-($153))&4294967295); //@line 2086 "dictionary.cpp"
                              $add_points=$sub266; //@line 2086 "dictionary.cpp"
                              ; //@line 2086 "dictionary.cpp"
                            }
                            else if (__label__ == 97) {
  
                              $failed=1; //@line 2080 "dictionary.cpp"
                              __label__ = 30;break $if_then8$$if_end70$12; //@line 2085 "dictionary.cpp"
                            }
                          }
                          else if (__label__ == 101) {
  
                            var $154=$post_ptr; //@line 2095 "dictionary.cpp"
                            var $155=$letter_xbytes; //@line 2095 "dictionary.cpp"
                            var $add_ptr270=(($154+$155)&4294967295); //@line 2095 "dictionary.cpp"
                            $p269=$add_ptr270; //@line 2095 "dictionary.cpp"
                            var $156=$p269; //@line 2096 "dictionary.cpp"
                            $p2=$156; //@line 2096 "dictionary.cpp"
                            var $157=$rule_addr; //@line 2098 "dictionary.cpp"
                            var $call271=__Z7utf8_inPiPKc($rule_w, $157); //@line 2098 "dictionary.cpp"
                            var $158=HEAP32[(($letter_w)>>2)]; //@line 2099 "dictionary.cpp"
                            var $159=HEAP32[(($rule_w)>>2)]; //@line 2099 "dictionary.cpp"
                            var $cmp27317=((($158))|0)!=((($159))|0); //@line 2099 "dictionary.cpp"
                            var $160=HEAP32[(($letter_w)>>2)]; //@line 2099 "dictionary.cpp"
                            var $cmp27518=((($160))|0)!=32; //@line 2099 "dictionary.cpp"
                            var $or_cond19=($cmp27317) & ($cmp27518); //@line 2099 "dictionary.cpp"
                            if ($or_cond19) { __label__ = 102;; } else { __label__ = 103;; } //@line 2099 "dictionary.cpp"
                            $while_body277$$while_end280$146: do { 
                              if (__label__ == 102) {
                                while(1) { 
  
                                  var $161=$p269; //@line 2101 "dictionary.cpp"
                                  $p2=$161; //@line 2101 "dictionary.cpp"
                                  var $162=$p269; //@line 2102 "dictionary.cpp"
                                  var $call278=__Z7utf8_inPiPKc($letter_w, $162); //@line 2102 "dictionary.cpp"
                                  var $163=$p269; //@line 2102 "dictionary.cpp"
                                  var $add_ptr279=(($163+$call278)&4294967295); //@line 2102 "dictionary.cpp"
                                  $p269=$add_ptr279; //@line 2102 "dictionary.cpp"
                                  var $164=HEAP32[(($letter_w)>>2)]; //@line 2099 "dictionary.cpp"
                                  var $165=HEAP32[(($rule_w)>>2)]; //@line 2099 "dictionary.cpp"
                                  var $cmp273=((($164))|0)!=((($165))|0); //@line 2099 "dictionary.cpp"
                                  var $166=HEAP32[(($letter_w)>>2)]; //@line 2099 "dictionary.cpp"
                                  var $cmp275=((($166))|0)!=32; //@line 2099 "dictionary.cpp"
                                  var $or_cond=($cmp273) & ($cmp275); //@line 2099 "dictionary.cpp"
                                  if ($or_cond) { __label__ = 102;continue ; } else { __label__ = 103;break $while_body277$$while_end280$146; } //@line 2099 "dictionary.cpp"
                                }
                              }
                            } while(0);
  
                            var $167=HEAP32[(($letter_w)>>2)]; //@line 2104 "dictionary.cpp"
                            var $168=HEAP32[(($rule_w)>>2)]; //@line 2104 "dictionary.cpp"
                            var $cmp281=((($167))|0)==((($168))|0); //@line 2104 "dictionary.cpp"
                            if (!($cmp281)) { __label__ = 181;break $sw_epilog496$$sw_bb71$$sw_bb89$$sw_bb328$44; } //@line 2104 "dictionary.cpp"
  
                            var $169=$p2; //@line 2106 "dictionary.cpp"
                            $post_ptr=$169; //@line 2106 "dictionary.cpp"
                            ; //@line 2107 "dictionary.cpp"
                          }
                          else if (__label__ == 105) {
  
                            $add_points=20; //@line 2112 "dictionary.cpp"
                            ; //@line 2113 "dictionary.cpp"
                          }
                          else if (__label__ == 106) {
  
                            var $170=$word_addr; //@line 2117 "dictionary.cpp"
                            var $171=HEAP32[(($170)>>2)]; //@line 2117 "dictionary.cpp"
                            var $172=$group_length_addr; //@line 2117 "dictionary.cpp"
                            var $add_ptr286=(($171+$172)&4294967295); //@line 2117 "dictionary.cpp"
                            $p=$add_ptr286; //@line 2117 "dictionary.cpp"
                            ; //@line 2117 "dictionary.cpp"
                            while(1) { 
  
                              var $173=$p; //@line 2117 "dictionary.cpp"
                              var $174=$post_ptr; //@line 2117 "dictionary.cpp"
                              var $cmp287=((($173))>>>0) < ((($174))>>>0); //@line 2117 "dictionary.cpp"
                              if (!($cmp287)) { __label__ = 181;break $sw_epilog496$$sw_bb71$$sw_bb89$$sw_bb328$44; } //@line 2117 "dictionary.cpp"
  
                              var $175=$p; //@line 2119 "dictionary.cpp"
                              var $176=HEAP8[($175)]; //@line 2119 "dictionary.cpp"
                              var $conv288=(tempInt=(($176)),(tempInt>=128?tempInt-256:tempInt)); //@line 2119 "dictionary.cpp"
                              var $cmp289=((($conv288))|0)==101; //@line 2119 "dictionary.cpp"
                              var $177=$p; //@line 2121 "dictionary.cpp"
                              if ($cmp289) { __label__ = 109;break ; } //@line 2119 "dictionary.cpp"
  
                              var $incdec_ptr293=(($177+1)&4294967295); //@line 2117 "dictionary.cpp"
                              $p=$incdec_ptr293; //@line 2117 "dictionary.cpp"
                              __label__ = 107;continue ; //@line 2117 "dictionary.cpp"
                            }
  
                            HEAP32[(($del_fwd291)>>2)]=$177; //@line 2121 "dictionary.cpp"
                            ; //@line 2122 "dictionary.cpp"
                          }
                          else if (__label__ == 111) {
  
                            var $178=$rule_addr; //@line 2129 "dictionary.cpp"
                            var $arrayidx295=(($178)&4294967295); //@line 2129 "dictionary.cpp"
                            var $179=HEAP8[($arrayidx295)]; //@line 2129 "dictionary.cpp"
                            var $conv296=(tempInt=(($179)),(tempInt>=128?tempInt-256:tempInt)); //@line 2129 "dictionary.cpp"
                            var $shl297=((($conv296)*65536)&4294967295); //@line 2129 "dictionary.cpp"
                            var $180=$rule_addr; //@line 2129 "dictionary.cpp"
                            var $arrayidx298=(($180+1)&4294967295); //@line 2129 "dictionary.cpp"
                            var $181=HEAP8[($arrayidx298)]; //@line 2129 "dictionary.cpp"
                            var $conv299=(tempInt=(($181)),(tempInt>=128?tempInt-256:tempInt)); //@line 2129 "dictionary.cpp"
                            var $and300=($conv299) & 127; //@line 2129 "dictionary.cpp"
                            var $shl301=((($and300)*256)&4294967295); //@line 2129 "dictionary.cpp"
                            var $182=$rule_addr; //@line 2129 "dictionary.cpp"
                            var $arrayidx303=(($182+2)&4294967295); //@line 2129 "dictionary.cpp"
                            var $183=HEAP8[($arrayidx303)]; //@line 2129 "dictionary.cpp"
                            var $conv304=(tempInt=(($183)),(tempInt>=128?tempInt-256:tempInt)); //@line 2129 "dictionary.cpp"
                            var $and305=($conv304) & 127; //@line 2129 "dictionary.cpp"
                            var $add302=((($shl301)+($shl297))&4294967295); //@line 2129 "dictionary.cpp"
                            var $add306=((($add302)+($and305))&4294967295); //@line 2129 "dictionary.cpp"
                            HEAP32[(($end_type307)>>2)]=$add306; //@line 2129 "dictionary.cpp"
                            var $184=$rule_addr; //@line 2130 "dictionary.cpp"
                            var $add_ptr308=(($184+3)&4294967295); //@line 2130 "dictionary.cpp"
                            $rule_addr=$add_ptr308; //@line 2130 "dictionary.cpp"
                            ; //@line 2131 "dictionary.cpp"
                          }
                          else if (__label__ == 112) {
  
                            var $185=$word_flags_addr; //@line 2134 "dictionary.cpp"
                            var $and310=($185) & 8192; //@line 2134 "dictionary.cpp"
                            var $tobool311=((($and310))|0)!=0; //@line 2134 "dictionary.cpp"
                            if ($tobool311) { __label__ = 113;; } else { __label__ = 114;; } //@line 2134 "dictionary.cpp"
                            if (__label__ == 113) {
  
                              $failed=1; //@line 2135 "dictionary.cpp"
                              __label__ = 30;break $if_then8$$if_end70$12; //@line 2135 "dictionary.cpp"
                            }
                            else if (__label__ == 114) {
  
                              $add_points=1; //@line 2137 "dictionary.cpp"
                              ;
                            }
                          }
                        }
                        else if (__label__ == 119) {
  
                          var $190=$distance_left; //@line 2158 "dictionary.cpp"
                          var $add329=((($190)+2)&4294967295); //@line 2158 "dictionary.cpp"
                          $distance_left=$add329; //@line 2158 "dictionary.cpp"
                          var $191=$distance_left; //@line 2159 "dictionary.cpp"
                          var $cmp330=((($191))|0) > 18; //@line 2159 "dictionary.cpp"
                          if ($cmp330) { __label__ = 120;; } else { __label__ = 121;; } //@line 2159 "dictionary.cpp"
                          if (__label__ == 120) {
  
                            $distance_left=19; //@line 2160 "dictionary.cpp"
                            ; //@line 2160 "dictionary.cpp"
                          }
  
                          var $192=$pre_ptr; //@line 2162 "dictionary.cpp"
                          var $193=HEAP8[($192)]; //@line 2162 "dictionary.cpp"
                          $last_letter=$193; //@line 2162 "dictionary.cpp"
                          var $194=$pre_ptr; //@line 2163 "dictionary.cpp"
                          var $incdec_ptr333=(($194+-1)&4294967295); //@line 2163 "dictionary.cpp"
                          $pre_ptr=$incdec_ptr333; //@line 2163 "dictionary.cpp"
                          var $195=$pre_ptr; //@line 2164 "dictionary.cpp"
                          var $call334=__Z8utf8_in2PiPKci($letter_w, $195, 1); //@line 2164 "dictionary.cpp"
                          var $sub335=((($call334)-1)&4294967295); //@line 2164 "dictionary.cpp"
                          $letter_xbytes=$sub335; //@line 2164 "dictionary.cpp"
                          var $196=$pre_ptr; //@line 2165 "dictionary.cpp"
                          var $197=HEAP8[($196)]; //@line 2165 "dictionary.cpp"
                          $letter=$197; //@line 2165 "dictionary.cpp"
                          var $198=$rb; //@line 2167 "dictionary.cpp"
                          var $conv336=((($198))&255); //@line 2167 "dictionary.cpp"
                          if ($conv336 == 17) {
                            __label__ = 122;;
                          }
                          else if ($conv336 == 18) {
                            __label__ = 127;;
                          }
                          else if ($conv336 == 25) {
                            __label__ = 130;;
                          }
                          else if ($conv336 == 11) {
                            __label__ = 133;;
                          }
                          else if ($conv336 == 15) {
                            __label__ = 136;;
                          }
                          else if ($conv336 == 16) {
                            __label__ = 139;;
                          }
                          else if ($conv336 == 21) {
                            __label__ = 142;;
                          }
                          else if ($conv336 == 10) {
                            __label__ = 147;;
                          }
                          else if ($conv336 == 29) {
                            __label__ = 150;;
                          }
                          else if ($conv336 == 26) {
                            __label__ = 157;;
                          }
                          else if ($conv336 == 19) {
                            __label__ = 160;;
                          }
                          else if ($conv336 == 46) {
                            __label__ = 163;;
                          }
                          else if ($conv336 == 45) {
                            __label__ = 170;;
                          }
                          else {
                          __label__ = 175;;
                          }
                          
                          if (__label__ == 175) {
  
                            var $268=$letter; //@line 2307 "dictionary.cpp"
                            var $conv478=((($268))&255); //@line 2307 "dictionary.cpp"
                            var $269=$rb; //@line 2307 "dictionary.cpp"
                            var $conv479=((($269))&255); //@line 2307 "dictionary.cpp"
                            var $cmp480=((($conv478))|0)==((($conv479))|0); //@line 2307 "dictionary.cpp"
                            if ($cmp480) { __label__ = 176;; } else { __label__ = 180;; } //@line 2307 "dictionary.cpp"
                            if (__label__ == 176) {
  
                              var $270=$letter; //@line 2309 "dictionary.cpp"
                              var $conv482=((($270))&255); //@line 2309 "dictionary.cpp"
                              var $cmp483=((($conv482))|0)==32; //@line 2309 "dictionary.cpp"
                              if ($cmp483) { __label__ = 177;; } else { __label__ = 178;; } //@line 2309 "dictionary.cpp"
                              if (__label__ == 177) {
  
                                $add_points=4; //@line 2310 "dictionary.cpp"
                                ; //@line 2310 "dictionary.cpp"
                              }
                              else if (__label__ == 178) {
  
                                var $271=$letter; //@line 2313 "dictionary.cpp"
                                var $conv486=((($271))&255); //@line 2313 "dictionary.cpp"
                                var $and487=($conv486) & 192; //@line 2313 "dictionary.cpp"
                                var $cmp488=((($and487))|0)!=128; //@line 2313 "dictionary.cpp"
                                if (!($cmp488)) { __label__ = 181;break $sw_epilog496$$sw_bb71$$sw_bb89$$sw_bb328$44; } //@line 2313 "dictionary.cpp"
  
                                var $272=$distance_left; //@line 2316 "dictionary.cpp"
                                var $sub490=((21-($272))&4294967295); //@line 2316 "dictionary.cpp"
                                $add_points=$sub490; //@line 2316 "dictionary.cpp"
                                ; //@line 2317 "dictionary.cpp"
                              }
                            }
                            else if (__label__ == 180) {
  
                              $failed=1; //@line 2321 "dictionary.cpp"
                              __label__ = 30;break $if_then8$$if_end70$12;
                            }
                          }
                          else if (__label__ == 122) {
  
                            var $199=$rule_addr; //@line 2170 "dictionary.cpp"
                            var $incdec_ptr338=(($199+1)&4294967295); //@line 2170 "dictionary.cpp"
                            $rule_addr=$incdec_ptr338; //@line 2170 "dictionary.cpp"
                            var $200=HEAP8[($199)]; //@line 2170 "dictionary.cpp"
                            var $conv339=(tempInt=(($200)),(tempInt>=128?tempInt-256:tempInt)); //@line 2170 "dictionary.cpp"
                            var $sub340=((($conv339)-65)&4294967295); //@line 2170 "dictionary.cpp"
                            $letter_group=$sub340; //@line 2170 "dictionary.cpp"
                            var $201=$tr_addr; //@line 2171 "dictionary.cpp"
                            var $202=HEAP32[(($letter_w)>>2)]; //@line 2171 "dictionary.cpp"
                            var $203=$letter_group; //@line 2171 "dictionary.cpp"
                            var $call341=__ZL8IsLetterP10Translatorii($201, $202, $203); //@line 2171 "dictionary.cpp"
                            var $tobool342=((($call341))|0)!=0; //@line 2171 "dictionary.cpp"
                            if ($tobool342) { __label__ = 123;; } else { __label__ = 126;; } //@line 2171 "dictionary.cpp"
                            if (__label__ == 123) {
  
                              $lg_pts=20; //@line 2173 "dictionary.cpp"
                              var $204=$letter_group; //@line 2174 "dictionary.cpp"
                              var $cmp344=((($204))|0)==2; //@line 2174 "dictionary.cpp"
                              if ($cmp344) { __label__ = 124;; } else { __label__ = 125;; } //@line 2174 "dictionary.cpp"
                              if (__label__ == 124) {
  
                                $lg_pts=19; //@line 2175 "dictionary.cpp"
                                ; //@line 2175 "dictionary.cpp"
                              }
  
                              var $205=$lg_pts; //@line 2176 "dictionary.cpp"
                              var $206=$distance_left; //@line 2176 "dictionary.cpp"
                              var $sub347=((($205)-($206))&4294967295); //@line 2176 "dictionary.cpp"
                              $add_points=$sub347; //@line 2176 "dictionary.cpp"
                              var $207=$letter_xbytes; //@line 2177 "dictionary.cpp"
                              var $208=$pre_ptr; //@line 2177 "dictionary.cpp"
                              var $idx_neg=(((-($207)))&4294967295); //@line 2177 "dictionary.cpp"
                              var $add_ptr348=(($208+$idx_neg)&4294967295); //@line 2177 "dictionary.cpp"
                              $pre_ptr=$add_ptr348; //@line 2177 "dictionary.cpp"
                              ; //@line 2178 "dictionary.cpp"
                            }
                            else if (__label__ == 126) {
  
                              $failed=1; //@line 2180 "dictionary.cpp"
                              __label__ = 30;break $if_then8$$if_end70$12;
                            }
                          }
                          else if (__label__ == 127) {
  
                            var $209=$rule_addr; //@line 2184 "dictionary.cpp"
                            var $incdec_ptr352=(($209+1)&4294967295); //@line 2184 "dictionary.cpp"
                            $rule_addr=$incdec_ptr352; //@line 2184 "dictionary.cpp"
                            var $210=HEAP8[($209)]; //@line 2184 "dictionary.cpp"
                            var $conv353=(tempInt=(($210)),(tempInt>=128?tempInt-256:tempInt)); //@line 2184 "dictionary.cpp"
                            var $sub354=((($conv353)-65)&4294967295); //@line 2184 "dictionary.cpp"
                            $letter_group=$sub354; //@line 2184 "dictionary.cpp"
                            var $211=$tr_addr; //@line 2185 "dictionary.cpp"
                            var $212=$pre_ptr; //@line 2185 "dictionary.cpp"
                            var $213=$letter_group; //@line 2185 "dictionary.cpp"
                            var $call355=__ZL13IsLetterGroupP10TranslatorPcii($211, $212, $213, 1); //@line 2185 "dictionary.cpp"
                            $n_bytes=$call355; //@line 2185 "dictionary.cpp"
                            var $cmp356=((($call355))|0) > 0; //@line 2185 "dictionary.cpp"
                            if ($cmp356) { __label__ = 128;; } else { __label__ = 129;; } //@line 2185 "dictionary.cpp"
                            if (__label__ == 128) {
  
                              var $214=$distance_right; //@line 2187 "dictionary.cpp"
                              var $sub358=((20-($214))&4294967295); //@line 2187 "dictionary.cpp"
                              $add_points=$sub358; //@line 2187 "dictionary.cpp"
                              var $215=$n_bytes; //@line 2188 "dictionary.cpp"
                              var $sub359=((($215)-1)&4294967295); //@line 2188 "dictionary.cpp"
                              var $216=$pre_ptr; //@line 2188 "dictionary.cpp"
                              var $idx_neg360=(((-($sub359)))&4294967295); //@line 2188 "dictionary.cpp"
                              var $add_ptr361=(($216+$idx_neg360)&4294967295); //@line 2188 "dictionary.cpp"
                              $pre_ptr=$add_ptr361; //@line 2188 "dictionary.cpp"
                              ; //@line 2189 "dictionary.cpp"
                            }
                            else if (__label__ == 129) {
  
                              $failed=1; //@line 2191 "dictionary.cpp"
                              __label__ = 30;break $if_then8$$if_end70$12;
                            }
                          }
                          else if (__label__ == 130) {
  
                            var $217=$tr_addr; //@line 2195 "dictionary.cpp"
                            var $218=HEAP32[(($letter_w)>>2)]; //@line 2195 "dictionary.cpp"
                            var $call365=__ZL8IsLetterP10Translatorii($217, $218, 0); //@line 2195 "dictionary.cpp"
                            var $tobool366=((($call365))|0)!=0; //@line 2195 "dictionary.cpp"
                            if ($tobool366) { __label__ = 132;; } else { __label__ = 131;; } //@line 2195 "dictionary.cpp"
                            if (__label__ == 132) {
  
                              $failed=1; //@line 2201 "dictionary.cpp"
                              __label__ = 30;break $if_then8$$if_end70$12;
                            }
                            else if (__label__ == 131) {
  
                              var $219=$distance_left; //@line 2197 "dictionary.cpp"
                              var $sub368=((20-($219))&4294967295); //@line 2197 "dictionary.cpp"
                              $add_points=$sub368; //@line 2197 "dictionary.cpp"
                              var $220=$letter_xbytes; //@line 2198 "dictionary.cpp"
                              var $221=$pre_ptr; //@line 2198 "dictionary.cpp"
                              var $idx_neg369=(((-($220)))&4294967295); //@line 2198 "dictionary.cpp"
                              var $add_ptr370=(($221+$idx_neg369)&4294967295); //@line 2198 "dictionary.cpp"
                              $pre_ptr=$add_ptr370; //@line 2198 "dictionary.cpp"
                              ; //@line 2199 "dictionary.cpp"
                            }
                          }
                          else if (__label__ == 133) {
  
                            var $222=$letter; //@line 2205 "dictionary.cpp"
                            var $conv374=((($222))&255); //@line 2205 "dictionary.cpp"
                            var $223=$last_letter; //@line 2205 "dictionary.cpp"
                            var $conv375=((($223))&255); //@line 2205 "dictionary.cpp"
                            var $cmp376=((($conv374))|0)==((($conv375))|0); //@line 2205 "dictionary.cpp"
                            if ($cmp376) { __label__ = 134;; } else { __label__ = 135;; } //@line 2205 "dictionary.cpp"
                            if (__label__ == 134) {
  
                              var $224=$distance_left; //@line 2206 "dictionary.cpp"
                              var $sub378=((21-($224))&4294967295); //@line 2206 "dictionary.cpp"
                              $add_points=$sub378; //@line 2206 "dictionary.cpp"
                              ; //@line 2206 "dictionary.cpp"
                            }
                            else if (__label__ == 135) {
  
                              $failed=1; //@line 2208 "dictionary.cpp"
                              __label__ = 30;break $if_then8$$if_end70$12;
                            }
                          }
                          else if (__label__ == 136) {
  
                            var $225=HEAP32[(($letter_w)>>2)]; //@line 2212 "dictionary.cpp"
                            var $call382=__Z7IsDigitj($225); //@line 2212 "dictionary.cpp"
                            var $tobool383=((($call382))|0)!=0; //@line 2212 "dictionary.cpp"
                            if ($tobool383) { __label__ = 137;; } else { __label__ = 138;; } //@line 2212 "dictionary.cpp"
                            if (__label__ == 137) {
  
                              var $226=$distance_left; //@line 2214 "dictionary.cpp"
                              var $sub385=((21-($226))&4294967295); //@line 2214 "dictionary.cpp"
                              $add_points=$sub385; //@line 2214 "dictionary.cpp"
                              var $227=$letter_xbytes; //@line 2215 "dictionary.cpp"
                              var $228=$pre_ptr; //@line 2215 "dictionary.cpp"
                              var $idx_neg386=(((-($227)))&4294967295); //@line 2215 "dictionary.cpp"
                              var $add_ptr387=(($228+$idx_neg386)&4294967295); //@line 2215 "dictionary.cpp"
                              $pre_ptr=$add_ptr387; //@line 2215 "dictionary.cpp"
                              ; //@line 2216 "dictionary.cpp"
                            }
                            else if (__label__ == 138) {
  
                              $failed=1; //@line 2218 "dictionary.cpp"
                              __label__ = 30;break $if_then8$$if_end70$12;
                            }
                          }
                          else if (__label__ == 139) {
  
                            var $229=HEAP32[(($letter_w)>>2)]; //@line 2222 "dictionary.cpp"
                            var $call391=_iswalpha($229); //@line 2222 "dictionary.cpp"
                            var $tobool392=((($call391))|0)!=0; //@line 2222 "dictionary.cpp"
                            if ($tobool392) { __label__ = 141;; } else { __label__ = 140;; } //@line 2222 "dictionary.cpp"
                            if (__label__ == 141) {
  
                              $failed=1; //@line 2228 "dictionary.cpp"
                              __label__ = 30;break $if_then8$$if_end70$12;
                            }
                            else if (__label__ == 140) {
  
                              var $230=$distance_right; //@line 2224 "dictionary.cpp"
                              var $sub394=((21-($230))&4294967295); //@line 2224 "dictionary.cpp"
                              $add_points=$sub394; //@line 2224 "dictionary.cpp"
                              var $231=$letter_xbytes; //@line 2225 "dictionary.cpp"
                              var $232=$pre_ptr; //@line 2225 "dictionary.cpp"
                              var $idx_neg395=(((-($231)))&4294967295); //@line 2225 "dictionary.cpp"
                              var $add_ptr396=(($232+$idx_neg395)&4294967295); //@line 2225 "dictionary.cpp"
                              $pre_ptr=$add_ptr396; //@line 2225 "dictionary.cpp"
                              ; //@line 2226 "dictionary.cpp"
                            }
                          }
                          else if (__label__ == 142) {
  
                            $syllable_count=1; //@line 2233 "dictionary.cpp"
                            var $233=$rule_addr; //@line 2234 "dictionary.cpp"
                            var $234=HEAP8[($233)]; //@line 2234 "dictionary.cpp"
                            var $conv40112=(tempInt=(($234)),(tempInt>=128?tempInt-256:tempInt)); //@line 2234 "dictionary.cpp"
                            var $cmp40213=((($conv40112))|0)==21; //@line 2234 "dictionary.cpp"
                            if ($cmp40213) { __label__ = 143;; } else { __label__ = 144;; } //@line 2234 "dictionary.cpp"
                            $while_body403$$while_end406$204: do { 
                              if (__label__ == 143) {
                                while(1) { 
  
                                  var $235=$rule_addr; //@line 2236 "dictionary.cpp"
                                  var $incdec_ptr404=(($235+1)&4294967295); //@line 2236 "dictionary.cpp"
                                  $rule_addr=$incdec_ptr404; //@line 2236 "dictionary.cpp"
                                  var $236=$syllable_count; //@line 2237 "dictionary.cpp"
                                  var $inc405=((($236)+1)&4294967295); //@line 2237 "dictionary.cpp"
                                  $syllable_count=$inc405; //@line 2237 "dictionary.cpp"
                                  var $237=$rule_addr; //@line 2234 "dictionary.cpp"
                                  var $238=HEAP8[($237)]; //@line 2234 "dictionary.cpp"
                                  var $conv401=(tempInt=(($238)),(tempInt>=128?tempInt-256:tempInt)); //@line 2234 "dictionary.cpp"
                                  var $cmp402=((($conv401))|0)==21; //@line 2234 "dictionary.cpp"
                                  if ($cmp402) { __label__ = 143;continue ; } else { __label__ = 144;break $while_body403$$while_end406$204; } //@line 2234 "dictionary.cpp"
                                }
                              }
                            } while(0);
  
                            var $239=$syllable_count; //@line 2239 "dictionary.cpp"
                            var $240=$tr_addr; //@line 2239 "dictionary.cpp"
                            var $word_vowel_count=(($240+8296)&4294967295); //@line 2239 "dictionary.cpp"
                            var $241=HEAP32[(($word_vowel_count)>>2)]; //@line 2239 "dictionary.cpp"
                            var $cmp407=((($239))|0) <= ((($241))|0); //@line 2239 "dictionary.cpp"
                            if ($cmp407) { __label__ = 145;; } else { __label__ = 146;; } //@line 2239 "dictionary.cpp"
                            if (__label__ == 145) {
  
                              var $242=$syllable_count; //@line 2240 "dictionary.cpp"
                              var $243=$distance_left; //@line 2240 "dictionary.cpp"
                              var $_neg11=(((-($243)))&4294967295);
                              var $add409=((($242)+18)&4294967295); //@line 2240 "dictionary.cpp"
                              var $sub410=((($add409)+($_neg11))&4294967295); //@line 2240 "dictionary.cpp"
                              $add_points=$sub410; //@line 2240 "dictionary.cpp"
                              ; //@line 2240 "dictionary.cpp"
                            }
                            else if (__label__ == 146) {
  
                              $failed=1; //@line 2242 "dictionary.cpp"
                              __label__ = 30;break $if_then8$$if_end70$12;
                            }
                          }
                          else if (__label__ == 147) {
  
                            var $244=$tr_addr; //@line 2246 "dictionary.cpp"
                            var $word_stressed_count=(($244+8300)&4294967295); //@line 2246 "dictionary.cpp"
                            var $245=HEAP32[(($word_stressed_count)>>2)]; //@line 2246 "dictionary.cpp"
                            var $cmp414=((($245))|0) > 0; //@line 2246 "dictionary.cpp"
                            if ($cmp414) { __label__ = 148;; } else { __label__ = 149;; } //@line 2246 "dictionary.cpp"
                            if (__label__ == 148) {
  
                              $add_points=19; //@line 2247 "dictionary.cpp"
                              ; //@line 2247 "dictionary.cpp"
                            }
                            else if (__label__ == 149) {
  
                              $failed=1; //@line 2249 "dictionary.cpp"
                              __label__ = 30;break $if_then8$$if_end70$12;
                            }
                          }
                          else if (__label__ == 150) {
  
                            var $246=$pre_ptr; //@line 2254 "dictionary.cpp"
                            var $247=$letter_xbytes; //@line 2254 "dictionary.cpp"
                            var $idx_neg420=(((-($247)))&4294967295); //@line 2254 "dictionary.cpp"
                            var $add_ptr421=(($246+$idx_neg420)&4294967295); //@line 2254 "dictionary.cpp"
                            var $add_ptr422=(($add_ptr421+-1)&4294967295); //@line 2254 "dictionary.cpp"
                            $p419=$add_ptr422; //@line 2254 "dictionary.cpp"
                            ; //@line 2255 "dictionary.cpp"
                            while(1) { 
  
                              var $248=HEAP32[(($letter_w)>>2)]; //@line 2255 "dictionary.cpp"
                              var $cmp424=((($248))|0)!=32; //@line 2255 "dictionary.cpp"
                              if (!($cmp424)) { __label__ = 155;break ; } //@line 2255 "dictionary.cpp"
  
                              var $249=$tr_addr; //@line 2257 "dictionary.cpp"
                              var $250=HEAP32[(($letter_w)>>2)]; //@line 2257 "dictionary.cpp"
                              var $call426=__ZL8IsLetterP10Translatorii($249, $250, 7); //@line 2257 "dictionary.cpp"
                              var $tobool427=((($call426))|0)!=0; //@line 2257 "dictionary.cpp"
                              if ($tobool427) { __label__ = 153;break ; } //@line 2257 "dictionary.cpp"
  
                              var $251=$p419; //@line 2262 "dictionary.cpp"
                              var $call430=__Z8utf8_in2PiPKci($letter_w, $251, 1); //@line 2262 "dictionary.cpp"
                              var $252=$p419; //@line 2262 "dictionary.cpp"
                              var $idx_neg431=(((-($call430)))&4294967295); //@line 2262 "dictionary.cpp"
                              var $add_ptr432=(($252+$idx_neg431)&4294967295); //@line 2262 "dictionary.cpp"
                              $p419=$add_ptr432; //@line 2262 "dictionary.cpp"
                              __label__ = 151;continue ; //@line 2263 "dictionary.cpp"
                            }
                            if (__label__ == 155) {
  
                              var $_pr4=$failed; //@line 2264 "dictionary.cpp"
                              var $tobool434=((($_pr4))|0)!=0; //@line 2264 "dictionary.cpp"
                              if ($tobool434) { __label__ = 30;break $if_then8$$if_end70$12; } //@line 2264 "dictionary.cpp"
  
                              $add_points=3; //@line 2265 "dictionary.cpp"
                              ; //@line 2265 "dictionary.cpp"
                            }
                            else if (__label__ == 153) {
  
                              $failed=1; //@line 2259 "dictionary.cpp"
                              __label__ = 30;break $if_then8$$if_end70$12; //@line 2264 "dictionary.cpp"
                            }
                          }
                          else if (__label__ == 157) {
  
                            var $253=$tr_addr; //@line 2270 "dictionary.cpp"
                            var $expect_verb=(($253+8272)&4294967295); //@line 2270 "dictionary.cpp"
                            var $254=HEAP32[(($expect_verb)>>2)]; //@line 2270 "dictionary.cpp"
                            var $tobool438=((($254))|0)!=0; //@line 2270 "dictionary.cpp"
                            if ($tobool438) { __label__ = 158;; } else { __label__ = 159;; } //@line 2270 "dictionary.cpp"
                            if (__label__ == 158) {
  
                              $add_points=1; //@line 2271 "dictionary.cpp"
                              ; //@line 2271 "dictionary.cpp"
                            }
                            else if (__label__ == 159) {
  
                              $failed=1; //@line 2273 "dictionary.cpp"
                              __label__ = 30;break $if_then8$$if_end70$12;
                            }
                          }
                          else if (__label__ == 160) {
  
                            var $255=$word_flags_addr; //@line 2277 "dictionary.cpp"
                            var $and443=($255) & 2; //@line 2277 "dictionary.cpp"
                            var $tobool444=((($and443))|0)!=0; //@line 2277 "dictionary.cpp"
                            if ($tobool444) { __label__ = 161;; } else { __label__ = 162;; } //@line 2277 "dictionary.cpp"
                            if (__label__ == 161) {
  
                              $add_points=1; //@line 2278 "dictionary.cpp"
                              ; //@line 2278 "dictionary.cpp"
                            }
                            else if (__label__ == 162) {
  
                              $failed=1; //@line 2280 "dictionary.cpp"
                              __label__ = 30;break $if_then8$$if_end70$12;
                            }
                          }
                          else if (__label__ == 163) {
  
                            var $256=$pre_ptr; //@line 2285 "dictionary.cpp"
                            $p=$256; //@line 2285 "dictionary.cpp"
                            ; //@line 2285 "dictionary.cpp"
                            while(1) { 
  
                              var $257=$p; //@line 2285 "dictionary.cpp"
                              var $258=HEAP8[($257)]; //@line 2285 "dictionary.cpp"
                              var $conv450=(tempInt=(($258)),(tempInt>=128?tempInt-256:tempInt)); //@line 2285 "dictionary.cpp"
                              var $cmp451=((($conv450))|0)!=32; //@line 2285 "dictionary.cpp"
                              if (!($cmp451)) { __label__ = 168;break ; } //@line 2285 "dictionary.cpp"
  
                              var $259=$p; //@line 2287 "dictionary.cpp"
                              var $260=HEAP8[($259)]; //@line 2287 "dictionary.cpp"
                              var $conv453=(tempInt=(($260)),(tempInt>=128?tempInt-256:tempInt)); //@line 2287 "dictionary.cpp"
                              var $cmp454=((($conv453))|0)==46; //@line 2287 "dictionary.cpp"
                              if ($cmp454) { __label__ = 166;break ; } //@line 2287 "dictionary.cpp"
  
                              var $261=$p; //@line 2285 "dictionary.cpp"
                              var $incdec_ptr458=(($261+-1)&4294967295); //@line 2285 "dictionary.cpp"
                              $p=$incdec_ptr458; //@line 2285 "dictionary.cpp"
                              __label__ = 164;continue ; //@line 2285 "dictionary.cpp"
                            }
                            if (__label__ == 166) {
  
                              $add_points=50; //@line 2289 "dictionary.cpp"
                              ; //@line 2290 "dictionary.cpp"
                            }
  
                            var $262=$p; //@line 2293 "dictionary.cpp"
                            var $263=HEAP8[($262)]; //@line 2293 "dictionary.cpp"
                            var $conv460=(tempInt=(($263)),(tempInt>=128?tempInt-256:tempInt)); //@line 2293 "dictionary.cpp"
                            var $cmp461=((($conv460))|0)==32; //@line 2293 "dictionary.cpp"
                            if (!($cmp461)) { __label__ = 181;break $sw_epilog496$$sw_bb71$$sw_bb89$$sw_bb328$44; } //@line 2293 "dictionary.cpp"
  
                            $failed=1; //@line 2294 "dictionary.cpp"
                            __label__ = 30;break $if_then8$$if_end70$12; //@line 2294 "dictionary.cpp"
                          }
                          else if (__label__ == 170) {
  
                            var $264=$letter; //@line 2298 "dictionary.cpp"
                            var $conv465=((($264))&255); //@line 2298 "dictionary.cpp"
                            var $cmp466=((($conv465))|0)==45; //@line 2298 "dictionary.cpp"
                            if ($cmp466) { __label__ = 173;; } else { __label__ = 171;; } //@line 2298 "dictionary.cpp"
                            $if_then473$$lor_lhs_false467$242: do { 
                              if (__label__ == 171) {
  
                                var $265=$letter; //@line 2298 "dictionary.cpp"
                                var $conv468=((($265))&255); //@line 2298 "dictionary.cpp"
                                var $cmp469=((($conv468))|0)==32; //@line 2298 "dictionary.cpp"
                                if ($cmp469) { __label__ = 172;; } else { __label__ = 174;; } //@line 2298 "dictionary.cpp"
                                if (__label__ == 172) {
  
                                  var $266=$word_flags_addr; //@line 2298 "dictionary.cpp"
                                  var $and471=($266) & 128; //@line 2298 "dictionary.cpp"
                                  var $tobool472=((($and471))|0)!=0; //@line 2298 "dictionary.cpp"
                                  if ($tobool472) { __label__ = 173;break $if_then473$$lor_lhs_false467$242; } //@line 2298 "dictionary.cpp"
                                }
  
                                $failed=1; //@line 2303 "dictionary.cpp"
                                __label__ = 30;break $if_then8$$if_end70$12;
                              }
                            } while(0);
  
                            var $267=$distance_right; //@line 2300 "dictionary.cpp"
                            var $sub474=((22-($267))&4294967295); //@line 2300 "dictionary.cpp"
                            $add_points=$sub474; //@line 2300 "dictionary.cpp"
                            ; //@line 2301 "dictionary.cpp"
                          }
                        }
                      } while(0);
  
                      var $_pr6_pr=$failed; //@line 2327 "dictionary.cpp"
                      var $cmp497=((($_pr6_pr))|0)==0; //@line 2327 "dictionary.cpp"
                      if (!($cmp497)) { __label__ = 30;break $if_then8$$if_end70$12; } //@line 2327 "dictionary.cpp"
  
                      var $273=$add_points; //@line 2328 "dictionary.cpp"
                      var $274=HEAP32[(($points499)>>2)]; //@line 2328 "dictionary.cpp"
                      var $add500=((($274)+($273))&4294967295); //@line 2328 "dictionary.cpp"
                      HEAP32[(($points499)>>2)]=$add500; //@line 2328 "dictionary.cpp"
                      ; //@line 2328 "dictionary.cpp"
                    }
                  } while(0);
  
                  var $42=$failed; //@line 1818 "dictionary.cpp"
                  var $tobool=((($42))|0)!=0; //@line 1818 "dictionary.cpp"
                  var $lnot=($tobool) ^ 1; //@line 1818 "dictionary.cpp"
                  if ($lnot) { __lastLabel__ = 30; __label__ = 6;continue $while_body4$10; } else { __lastLabel__ = 30; __label__ = 183;break $while_body4$$while_end502$9; } //@line 1818 "dictionary.cpp"
                }
              }
            } while(0);
  
            var $275=__lastLabel__ == 30 ? $42 : ($18); //@line 2331 "dictionary.cpp"
            var $cmp503=((($275))|0)==2; //@line 2331 "dictionary.cpp"
            if ($cmp503) { __label__ = 184;; } else { __label__ = 197;; } //@line 2331 "dictionary.cpp"
            $land_lhs_true504$$while_cond543$252: do { 
              if (__label__ == 184) {
  
                var $276=$unpron_ignore; //@line 2331 "dictionary.cpp"
                var $cmp505=((($276))|0)==0; //@line 2331 "dictionary.cpp"
                if (!($cmp505)) { __label__ = 197;break $land_lhs_true504$$while_cond543$252; } //@line 2331 "dictionary.cpp"
  
                var $277=$check_atstart; //@line 2334 "dictionary.cpp"
                var $cmp507=((($277))|0)==0; //@line 2334 "dictionary.cpp"
                if ($cmp507) { __label__ = 189;; } else { __label__ = 186;; } //@line 2334 "dictionary.cpp"
                $if_end517$$lor_lhs_false508$255: do { 
                  if (__label__ == 186) {
  
                    var $278=$pre_ptr; //@line 2334 "dictionary.cpp"
                    var $arrayidx509=(($278+-1)&4294967295); //@line 2334 "dictionary.cpp"
                    var $279=HEAP8[($arrayidx509)]; //@line 2334 "dictionary.cpp"
                    var $conv510=(tempInt=(($279)),(tempInt>=128?tempInt-256:tempInt)); //@line 2334 "dictionary.cpp"
                    var $cmp511=((($conv510))|0)==32; //@line 2334 "dictionary.cpp"
                    if (!($cmp511)) { __label__ = 197;break $land_lhs_true504$$while_cond543$252; } //@line 2334 "dictionary.cpp"
  
                    var $_pr8=$check_atstart; //@line 2336 "dictionary.cpp"
                    var $tobool513=((($_pr8))|0)!=0; //@line 2336 "dictionary.cpp"
                    if (!($tobool513)) { __label__ = 189;break $if_end517$$lor_lhs_false508$255; } //@line 2336 "dictionary.cpp"
  
                    var $280=HEAP32[(($points515)>>2)]; //@line 2337 "dictionary.cpp"
                    var $add516=((($280)+4)&4294967295); //@line 2337 "dictionary.cpp"
                    HEAP32[(($points515)>>2)]=$add516; //@line 2337 "dictionary.cpp"
                    ; //@line 2337 "dictionary.cpp"
                  }
                } while(0);
  
                var $281=HEAP32[(($points518)>>2)]; //@line 2340 "dictionary.cpp"
                var $282=HEAP32[((((__ZZL9MatchRuleP10TranslatorPPcS1_iS1_P11MatchRecordiiE4best)&4294967295))>>2)]; //@line 2340 "dictionary.cpp"
                var $cmp519=((($281))|0) >= ((($282))|0); //@line 2340 "dictionary.cpp"
                if ($cmp519) { __label__ = 190;; } else { __label__ = 191;; } //@line 2340 "dictionary.cpp"
                if (__label__ == 190) {
  
                  var $src$, $dest$, $stop$, $stop4$, $fast$;
                  $src$ = $8;
                  $dest$ = __ZZL9MatchRuleP10TranslatorPPcS1_iS1_P11MatchRecordiiE4best;
                  $stop$ = $src$ + 16;
                  if (($dest$%4) == ($src$%4) && 16 > 8) {
                    while ($src$%4 !== 0 && $src$ < $stop$) {
                      ; HEAP8[$dest$++] = HEAP8[$src$++];
                    }
                    $src$ >>= 2;
                    $dest$ >>= 2;
                    $stop4$ = $stop$ >> 2;
                    while ($src$ < $stop4$) {
                  ;;;    HEAP32[$dest$++] = HEAP32[$src$++];
                    }
                    $src$ <<= 2;
                    $dest$ <<= 2;
                  }while ($src$ < $stop$) {
                    ; HEAP8[$dest$++] = HEAP8[$src$++];
                  }; //@line 2342 "dictionary.cpp"
                  var $283=$consumed; //@line 2343 "dictionary.cpp"
                  $total_consumed=$283; //@line 2343 "dictionary.cpp"
                  ; //@line 2344 "dictionary.cpp"
                }
  
                var $284=HEAP32[((_option_phonemes)>>2)]; //@line 2346 "dictionary.cpp"
                var $cmp522=((($284))|0)==2; //@line 2346 "dictionary.cpp"
                if (!($cmp522)) { __label__ = 197;break $land_lhs_true504$$while_cond543$252; } //@line 2346 "dictionary.cpp"
  
                var $285=HEAP32[(($points524)>>2)]; //@line 2346 "dictionary.cpp"
                var $cmp525=((($285))|0) > 0; //@line 2346 "dictionary.cpp"
                if (!($cmp525)) { __label__ = 197;break $land_lhs_true504$$while_cond543$252; } //@line 2346 "dictionary.cpp"
  
                var $286=$word_flags_addr; //@line 2346 "dictionary.cpp"
                var $and527=($286) & 268435456; //@line 2346 "dictionary.cpp"
                var $cmp528=((($and527))|0)==0; //@line 2346 "dictionary.cpp"
                if (!($cmp528)) { __label__ = 197;break $land_lhs_true504$$while_cond543$252; } //@line 2346 "dictionary.cpp"
  
                var $287=HEAP32[(($points530)>>2)]; //@line 2352 "dictionary.cpp"
                $pts=$287; //@line 2352 "dictionary.cpp"
                var $288=$group_length_addr; //@line 2353 "dictionary.cpp"
                var $cmp531=((($288))|0) > 1; //@line 2353 "dictionary.cpp"
                if ($cmp531) { __label__ = 195;; } else { __label__ = 196;; } //@line 2353 "dictionary.cpp"
                if (__label__ == 195) {
  
                  var $289=$pts; //@line 2354 "dictionary.cpp"
                  var $add533=((($289)+35)&4294967295); //@line 2354 "dictionary.cpp"
                  $pts=$add533; //@line 2354 "dictionary.cpp"
                  ; //@line 2354 "dictionary.cpp"
                }
  
                var $290=HEAP32[(($phonemes535)>>2)]; //@line 2355 "dictionary.cpp"
                __Z14DecodePhonemesPKcPc($290, $arraydecay536); //@line 2355 "dictionary.cpp"
                var $291=HEAP32[((_f_trans)>>2)]; //@line 2356 "dictionary.cpp"
                var $292=$pts; //@line 2356 "dictionary.cpp"
                var $293=$group_chars; //@line 2356 "dictionary.cpp"
                var $294=$group_length_addr; //@line 2356 "dictionary.cpp"
                var $295=$rule_start; //@line 2356 "dictionary.cpp"
                var $296=$word_flags_addr; //@line 2356 "dictionary.cpp"
                var $call537=__Z10DecodeRulePKciPci($293, $294, $295, $296); //@line 2356 "dictionary.cpp"
                var $call539=_fprintf($291, ((__str32148)&4294967295), allocate([$292,0,0,0,((__ZZ10DecodeRulePKciPciE6output)&4294967295),0,0,0,$arraydecay538,0,0,0], ["i32",0,0,0,"i8*",0,0,0,"i8*",0,0,0], ALLOC_STACK)); //@line 2356 "dictionary.cpp"
                ; //@line 2357 "dictionary.cpp"
              }
            } while(0);
            while(1) { 
  
              var $297=$rule_addr; //@line 2362 "dictionary.cpp"
              var $incdec_ptr544=(($297+1)&4294967295); //@line 2362 "dictionary.cpp"
              $rule_addr=$incdec_ptr544; //@line 2362 "dictionary.cpp"
              var $298=HEAP8[($297)]; //@line 2362 "dictionary.cpp"
              var $conv545=(tempInt=(($298)),(tempInt>=128?tempInt-256:tempInt)); //@line 2362 "dictionary.cpp"
              var $cmp546=((($conv545))|0)!=0; //@line 2362 "dictionary.cpp"
              if ($cmp546) { __label__ = 197;continue ; } else { __label__ = 4;break ; } //@line 2362 "dictionary.cpp"
            }
  
            var $9=$rule_addr; //@line 1797 "dictionary.cpp"
            var $arrayidx=(($9)&4294967295); //@line 1797 "dictionary.cpp"
            var $10=HEAP8[($arrayidx)]; //@line 1797 "dictionary.cpp"
            var $conv=(tempInt=(($10)),(tempInt>=128?tempInt-256:tempInt)); //@line 1797 "dictionary.cpp"
            var $cmp1=((($conv))|0)!=7; //@line 1797 "dictionary.cpp"
            if ($cmp1) { __label__ = 5;continue $while_body$7; } else { __label__ = 198;break $while_body_lr_ph$$while_end549$5; } //@line 1797 "dictionary.cpp"
          }
        }
      } while(0);
  
      var $299=HEAP32[((_option_phonemes)>>2)]; //@line 2365 "dictionary.cpp"
      var $cmp550=((($299))|0)==2; //@line 2365 "dictionary.cpp"
      if ($cmp550) { __label__ = 199;; } else { __label__ = 202;; } //@line 2365 "dictionary.cpp"
      $land_lhs_true551$$if_end559$273: do { 
        if (__label__ == 199) {
  
          var $300=$word_flags_addr; //@line 2365 "dictionary.cpp"
          var $and552=($300) & 268435456; //@line 2365 "dictionary.cpp"
          var $cmp553=((($and552))|0)==0; //@line 2365 "dictionary.cpp"
          if (!($cmp553)) { __label__ = 202;break $land_lhs_true551$$if_end559$273; } //@line 2365 "dictionary.cpp"
  
          var $301=$group_length_addr; //@line 2367 "dictionary.cpp"
          var $cmp555=((($301))|0) <= 1; //@line 2367 "dictionary.cpp"
          if (!($cmp555)) { __label__ = 202;break $land_lhs_true551$$if_end559$273; } //@line 2367 "dictionary.cpp"
  
          var $302=HEAP32[((_f_trans)>>2)]; //@line 2368 "dictionary.cpp"
          var $fputc=_fputc(10, $302); //@line 2368 "dictionary.cpp"
          ; //@line 2368 "dictionary.cpp"
        }
      } while(0);
  
      var $303=$group_length_addr; //@line 2372 "dictionary.cpp"
      var $304=$total_consumed; //@line 2372 "dictionary.cpp"
      var $add560=((($304)+($303))&4294967295); //@line 2372 "dictionary.cpp"
      $total_consumed=$add560; //@line 2372 "dictionary.cpp"
      var $cmp561=((($add560))|0)==0; //@line 2373 "dictionary.cpp"
      if ($cmp561) { __label__ = 203;; } else { __label__ = 204;; } //@line 2373 "dictionary.cpp"
      if (__label__ == 203) {
  
        $total_consumed=1; //@line 2374 "dictionary.cpp"
        ; //@line 2374 "dictionary.cpp"
      }
  
      var $305=$total_consumed; //@line 2376 "dictionary.cpp"
      var $306=$word_addr; //@line 2376 "dictionary.cpp"
      var $307=HEAP32[(($306)>>2)]; //@line 2376 "dictionary.cpp"
      var $add_ptr564=(($307+$305)&4294967295); //@line 2376 "dictionary.cpp"
      HEAP32[(($306)>>2)]=$add_ptr564; //@line 2376 "dictionary.cpp"
      var $308=HEAP32[((((__ZZL9MatchRuleP10TranslatorPPcS1_iS1_P11MatchRecordiiE4best)&4294967295))>>2)]; //@line 2378 "dictionary.cpp"
      var $cmp565=((($308))|0)==0; //@line 2378 "dictionary.cpp"
      if ($cmp565) { __label__ = 205;; } else { __label__ = 206;; } //@line 2378 "dictionary.cpp"
      if (__label__ == 205) {
  
        HEAP32[((((__ZZL9MatchRuleP10TranslatorPPcS1_iS1_P11MatchRecordiiE4best+4)&4294967295))>>2)]=((__str13643)&4294967295); //@line 2379 "dictionary.cpp"
        ; //@line 2379 "dictionary.cpp"
      }
  
      var $309=$match_out_addr; //@line 2380 "dictionary.cpp"
      var $310=$309; //@line 2380 "dictionary.cpp"
      var $src$, $dest$, $stop$, $stop4$, $fast$;
      $src$ = __ZZL9MatchRuleP10TranslatorPPcS1_iS1_P11MatchRecordiiE4best;
      $dest$ = $310;
      $stop$ = $src$ + 16;
      if (($dest$%4) == ($src$%4) && 16 > 8) {
        while ($src$%4 !== 0 && $src$ < $stop$) {
          ; HEAP8[$dest$++] = HEAP8[$src$++];
        }
        $src$ >>= 2;
        $dest$ >>= 2;
        $stop4$ = $stop$ >> 2;
        while ($src$ < $stop4$) {
      ;;;    HEAP32[$dest$++] = HEAP32[$src$++];
        }
        $src$ <<= 2;
        $dest$ <<= 2;
      }while ($src$ < $stop$) {
        ; HEAP8[$dest$++] = HEAP8[$src$++];
      }; //@line 2380 "dictionary.cpp"
      ; //@line 2381 "dictionary.cpp"
    }
  
    STACKTOP = __stackBase__;
    return; //@line 2381 "dictionary.cpp"
    return;
  }
  

  function __Z21ApplySpecialAttributeP10TranslatorPci($tr, $phonemes, $dict_flags) {
    ;
    var __label__;
  
    var $tr_addr;
    var $phonemes_addr;
    var $dict_flags_addr;
    var $len;
    var $p_end;
    $tr_addr=$tr;
    $phonemes_addr=$phonemes;
    $dict_flags_addr=$dict_flags;
    var $0=$dict_flags_addr; //@line 2737 "dictionary.cpp"
    var $and=($0) & 3145728; //@line 2737 "dictionary.cpp"
    var $cmp=((($and))|0)==0; //@line 2737 "dictionary.cpp"
    if ($cmp) { __label__ = 4;; } else { __label__ = 1;; } //@line 2737 "dictionary.cpp"
    $sw_epilog$$if_end$2: do { 
      if (__label__ == 1) {
  
        var $1=$phonemes_addr; //@line 2740 "dictionary.cpp"
        var $call=_strlen($1); //@line 2740 "dictionary.cpp"
        $len=$call; //@line 2740 "dictionary.cpp"
        var $2=$len; //@line 2741 "dictionary.cpp"
        var $sub=((($2)-1)&4294967295); //@line 2741 "dictionary.cpp"
        var $3=$phonemes_addr; //@line 2741 "dictionary.cpp"
        var $arrayidx=(($3+$sub)&4294967295); //@line 2741 "dictionary.cpp"
        $p_end=$arrayidx; //@line 2741 "dictionary.cpp"
        var $4=$tr_addr; //@line 2743 "dictionary.cpp"
        var $translator_name=(($4+280)&4294967295); //@line 2743 "dictionary.cpp"
        var $5=HEAP32[(($translator_name)>>2)]; //@line 2743 "dictionary.cpp"
        var $cond=((($5))|0)==29295; //@line 2743 "dictionary.cpp"
        if (!($cond)) { __label__ = 4;break $sw_epilog$$if_end$2; } //@line 2743 "dictionary.cpp"
  
        var $6=$p_end; //@line 2760 "dictionary.cpp"
        var $arrayidx1=(($6)&4294967295); //@line 2760 "dictionary.cpp"
        var $7=HEAP8[($arrayidx1)]; //@line 2760 "dictionary.cpp"
        var $conv=(tempInt=(($7)),(tempInt>=128?tempInt-256:tempInt)); //@line 2760 "dictionary.cpp"
        var $call2=__Z11PhonemeCodej(106); //@line 2760 "dictionary.cpp"
        var $cmp3=((($conv))|0)==((($call2))|0); //@line 2760 "dictionary.cpp"
        if (!($cmp3)) { __label__ = 4;break $sw_epilog$$if_end$2; } //@line 2760 "dictionary.cpp"
  
        var $8=$p_end; //@line 2763 "dictionary.cpp"
        var $arrayidx5=(($8)&4294967295); //@line 2763 "dictionary.cpp"
        HEAP8[($arrayidx5)]=6; //@line 2763 "dictionary.cpp"
        var $call6=__Z11PhonemeCodej(105); //@line 2764 "dictionary.cpp"
        var $conv7=((($call6)) & 255); //@line 2764 "dictionary.cpp"
        var $9=$p_end; //@line 2764 "dictionary.cpp"
        var $arrayidx8=(($9+1)&4294967295); //@line 2764 "dictionary.cpp"
        HEAP8[($arrayidx8)]=$conv7; //@line 2764 "dictionary.cpp"
        var $10=$p_end; //@line 2765 "dictionary.cpp"
        var $arrayidx9=(($10+2)&4294967295); //@line 2765 "dictionary.cpp"
        HEAP8[($arrayidx9)]=0; //@line 2765 "dictionary.cpp"
        ; //@line 2766 "dictionary.cpp"
      }
    } while(0);
  
    ;
    return; //@line 2769 "dictionary.cpp"
    return;
  }
  

  function __Z22ApplySpecialAttribute2P10TranslatorPci($tr, $phonemes, $dict_flags) {
    ;
    var __label__;
  
    var $tr_addr;
    var $phonemes_addr;
    var $dict_flags_addr;
    var $ix;
    var $len;
    var $p;
    $tr_addr=$tr;
    $phonemes_addr=$phonemes;
    $dict_flags_addr=$dict_flags;
    var $0=$phonemes_addr; //@line 2701 "dictionary.cpp"
    var $call=_strlen($0); //@line 2701 "dictionary.cpp"
    $len=$call; //@line 2701 "dictionary.cpp"
    var $1=$tr_addr; //@line 2703 "dictionary.cpp"
    var $langopts=(($1)&4294967295); //@line 2703 "dictionary.cpp"
    var $param=(($langopts+24)&4294967295); //@line 2703 "dictionary.cpp"
    var $arrayidx=(($param+60)&4294967295); //@line 2703 "dictionary.cpp"
    var $2=HEAP32[(($arrayidx)>>2)]; //@line 2703 "dictionary.cpp"
    var $and=($2) & 2; //@line 2703 "dictionary.cpp"
    var $tobool=((($and))|0)!=0; //@line 2703 "dictionary.cpp"
    if ($tobool) { __label__ = 1;; } else { __label__ = 14;; } //@line 2703 "dictionary.cpp"
    $if_then$$if_end37$2: do { 
      if (__label__ == 1) {
  
        $ix=0; //@line 2705 "dictionary.cpp"
        ; //@line 2705 "dictionary.cpp"
        while(1) { 
  
          var $3=$ix; //@line 2705 "dictionary.cpp"
          var $4=$len; //@line 2705 "dictionary.cpp"
          var $sub=((($4)-1)&4294967295); //@line 2705 "dictionary.cpp"
          var $cmp=((($3))|0) < ((($sub))|0); //@line 2705 "dictionary.cpp"
          if (!($cmp)) { __label__ = 14;break $if_then$$if_end37$2; } //@line 2705 "dictionary.cpp"
  
          var $5=$ix; //@line 2707 "dictionary.cpp"
          var $6=$phonemes_addr; //@line 2707 "dictionary.cpp"
          var $arrayidx1=(($6+$5)&4294967295); //@line 2707 "dictionary.cpp"
          var $7=HEAP8[($arrayidx1)]; //@line 2707 "dictionary.cpp"
          var $conv=(tempInt=(($7)),(tempInt>=128?tempInt-256:tempInt)); //@line 2707 "dictionary.cpp"
          var $cmp2=((($conv))|0)==6; //@line 2707 "dictionary.cpp"
          var $8=$ix; //@line 2709 "dictionary.cpp"
          var $add=((($8)+1)&4294967295); //@line 2709 "dictionary.cpp"
          if ($cmp2) { __label__ = 4;break ; } //@line 2707 "dictionary.cpp"
  
          $ix=$add; //@line 2705 "dictionary.cpp"
          __label__ = 2;continue ; //@line 2705 "dictionary.cpp"
        }
  
        var $9=$phonemes_addr; //@line 2709 "dictionary.cpp"
        var $arrayidx4=(($9+$add)&4294967295); //@line 2709 "dictionary.cpp"
        $p=$arrayidx4; //@line 2709 "dictionary.cpp"
        var $10=$dict_flags_addr; //@line 2710 "dictionary.cpp"
        var $and5=($10) & 2097152; //@line 2710 "dictionary.cpp"
        var $cmp6=((($and5))|0)!=0; //@line 2710 "dictionary.cpp"
        var $11=$p; //@line 2712 "dictionary.cpp"
        var $12=HEAP8[($11)]; //@line 2712 "dictionary.cpp"
        var $conv8=(tempInt=(($12)),(tempInt>=128?tempInt-256:tempInt)); //@line 2712 "dictionary.cpp"
        if ($cmp6) { __label__ = 5;; } else { __label__ = 9;; } //@line 2710 "dictionary.cpp"
        if (__label__ == 5) {
  
          var $call9=__Z11PhonemeCodej(69); //@line 2712 "dictionary.cpp"
          var $cmp10=((($conv8))|0)==((($call9))|0); //@line 2712 "dictionary.cpp"
          if ($cmp10) { __label__ = 6;; } else { __label__ = 7;; } //@line 2712 "dictionary.cpp"
          if (__label__ == 6) {
  
            var $call12=__Z11PhonemeCodej(101); //@line 2713 "dictionary.cpp"
            var $conv13=((($call12)) & 255); //@line 2713 "dictionary.cpp"
            var $13=$p; //@line 2713 "dictionary.cpp"
            HEAP8[($13)]=$conv13; //@line 2713 "dictionary.cpp"
            ; //@line 2713 "dictionary.cpp"
          }
  
          var $14=$p; //@line 2714 "dictionary.cpp"
          var $15=HEAP8[($14)]; //@line 2714 "dictionary.cpp"
          var $conv14=(tempInt=(($15)),(tempInt>=128?tempInt-256:tempInt)); //@line 2714 "dictionary.cpp"
          var $call15=__Z11PhonemeCodej(79); //@line 2714 "dictionary.cpp"
          var $cmp16=((($conv14))|0)==((($call15))|0); //@line 2714 "dictionary.cpp"
          if (!($cmp16)) { __label__ = 14;break $if_then$$if_end37$2; } //@line 2714 "dictionary.cpp"
  
          var $call18=__Z11PhonemeCodej(111); //@line 2715 "dictionary.cpp"
          var $conv19=((($call18)) & 255); //@line 2715 "dictionary.cpp"
          var $16=$p; //@line 2715 "dictionary.cpp"
          HEAP8[($16)]=$conv19; //@line 2715 "dictionary.cpp"
          ; //@line 2715 "dictionary.cpp"
        }
        else if (__label__ == 9) {
  
          var $call22=__Z11PhonemeCodej(101); //@line 2719 "dictionary.cpp"
          var $cmp23=((($conv8))|0)==((($call22))|0); //@line 2719 "dictionary.cpp"
          if ($cmp23) { __label__ = 10;; } else { __label__ = 11;; } //@line 2719 "dictionary.cpp"
          if (__label__ == 10) {
  
            var $call25=__Z11PhonemeCodej(69); //@line 2720 "dictionary.cpp"
            var $conv26=((($call25)) & 255); //@line 2720 "dictionary.cpp"
            var $17=$p; //@line 2720 "dictionary.cpp"
            HEAP8[($17)]=$conv26; //@line 2720 "dictionary.cpp"
            ; //@line 2720 "dictionary.cpp"
          }
  
          var $18=$p; //@line 2721 "dictionary.cpp"
          var $19=HEAP8[($18)]; //@line 2721 "dictionary.cpp"
          var $conv28=(tempInt=(($19)),(tempInt>=128?tempInt-256:tempInt)); //@line 2721 "dictionary.cpp"
          var $call29=__Z11PhonemeCodej(111); //@line 2721 "dictionary.cpp"
          var $cmp30=((($conv28))|0)==((($call29))|0); //@line 2721 "dictionary.cpp"
          if (!($cmp30)) { __label__ = 14;break $if_then$$if_end37$2; } //@line 2721 "dictionary.cpp"
  
          var $call32=__Z11PhonemeCodej(79); //@line 2722 "dictionary.cpp"
          var $conv33=((($call32)) & 255); //@line 2722 "dictionary.cpp"
          var $20=$p; //@line 2722 "dictionary.cpp"
          HEAP8[($20)]=$conv33; //@line 2722 "dictionary.cpp"
          ; //@line 2722 "dictionary.cpp"
        }
      }
    } while(0);
  
    ;
    return; //@line 2728 "dictionary.cpp"
    return;
  }
  

  function __Z17TransposeAlphabetP10TranslatorPc($tr, $text) {
    var __stackBase__  = STACKTOP; STACKTOP += 4; _memset(__stackBase__, 0, 4);
    var __label__;
    var __lastLabel__ = null;
  
    var $retval;
    var $tr_addr;
    var $text_addr;
    var $c=__stackBase__;
    var $c2;
    var $ix;
    var $offset;
    var $min;
    var $max;
    var $p;
    var $p2;
    var $all_alpha;
    var $bits;
    var $acc;
    var $pairs_start;
    var $pairs_list;
    $tr_addr=$tr;
    $text_addr=$text;
    var $0=$text_addr; //@line 2790 "dictionary.cpp"
    $p=$0; //@line 2790 "dictionary.cpp"
    var $1=$text_addr; //@line 2791 "dictionary.cpp"
    $p2=$1; //@line 2791 "dictionary.cpp"
    $all_alpha=1; //@line 2792 "dictionary.cpp"
    var $2=$tr_addr; //@line 2798 "dictionary.cpp"
    var $transpose_min=(($2+288)&4294967295); //@line 2798 "dictionary.cpp"
    var $3=HEAP32[(($transpose_min)>>2)]; //@line 2798 "dictionary.cpp"
    var $sub=((($3)-1)&4294967295); //@line 2798 "dictionary.cpp"
    $offset=$sub; //@line 2798 "dictionary.cpp"
    var $4=$tr_addr; //@line 2799 "dictionary.cpp"
    var $transpose_min1=(($4+288)&4294967295); //@line 2799 "dictionary.cpp"
    var $5=HEAP32[(($transpose_min1)>>2)]; //@line 2799 "dictionary.cpp"
    $min=$5; //@line 2799 "dictionary.cpp"
    var $6=$tr_addr; //@line 2800 "dictionary.cpp"
    var $transpose_max=(($6+284)&4294967295); //@line 2800 "dictionary.cpp"
    var $7=HEAP32[(($transpose_max)>>2)]; //@line 2800 "dictionary.cpp"
    $max=$7; //@line 2800 "dictionary.cpp"
    var $8=$max; //@line 2802 "dictionary.cpp"
    var $9=$min; //@line 2802 "dictionary.cpp"
    var $_neg=(((-($9)))&4294967295);
    var $sub2=((($8)+2)&4294967295); //@line 2802 "dictionary.cpp"
    var $add=((($sub2)+($_neg))&4294967295); //@line 2802 "dictionary.cpp"
    $pairs_start=$add; //@line 2802 "dictionary.cpp"
    ; //@line 2804 "dictionary.cpp"
    $do_body$2: while(1) { 
  
      var $10=$p; //@line 2805 "dictionary.cpp"
      var $call=__Z7utf8_inPiPKc($c, $10); //@line 2805 "dictionary.cpp"
      var $11=$p; //@line 2805 "dictionary.cpp"
      var $add_ptr=(($11+$call)&4294967295); //@line 2805 "dictionary.cpp"
      $p=$add_ptr; //@line 2805 "dictionary.cpp"
      var $12=HEAP32[(($c)>>2)]; //@line 2806 "dictionary.cpp"
      var $13=$min; //@line 2806 "dictionary.cpp"
      var $cmp=((($12))|0) >= ((($13))|0); //@line 2806 "dictionary.cpp"
      if ($cmp) { __lastLabel__ = 1; __label__ = 2;; } else { __lastLabel__ = 1; __label__ = 4;; } //@line 2806 "dictionary.cpp"
      $land_lhs_true$$if_else$4: do { 
        if (__label__ == 2) {
  
          var $14=HEAP32[(($c)>>2)]; //@line 2806 "dictionary.cpp"
          var $15=$max; //@line 2806 "dictionary.cpp"
          var $cmp3=((($14))|0) <= ((($15))|0); //@line 2806 "dictionary.cpp"
          if ($cmp3) { __lastLabel__ = 2; ; } else { __lastLabel__ = 2; __label__ = 4;break $land_lhs_true$$if_else$4; } //@line 2806 "dictionary.cpp"
  
          var $16=HEAP32[(($c)>>2)]; //@line 2808 "dictionary.cpp"
          var $17=$offset; //@line 2808 "dictionary.cpp"
          var $sub4=((($16)-($17))&4294967295); //@line 2808 "dictionary.cpp"
          var $conv=((($sub4)) & 255); //@line 2808 "dictionary.cpp"
          var $18=$p2; //@line 2808 "dictionary.cpp"
          var $incdec_ptr=(($18+1)&4294967295); //@line 2808 "dictionary.cpp"
          $p2=$incdec_ptr; //@line 2808 "dictionary.cpp"
          HEAP8[($18)]=$conv; //@line 2808 "dictionary.cpp"
          __label__ = 6;break $land_lhs_true$$if_else$4; //@line 2809 "dictionary.cpp"
        }
      } while(0);
      $do_cond$$if_else$7: do { 
        if (__label__ == 4) {
  
          var $19=__lastLabel__ == 2 ? $14 : ($12); //@line 2811 "dictionary.cpp"
          var $cmp5=((($19))|0)!=0; //@line 2811 "dictionary.cpp"
          if (!($cmp5)) { __label__ = 6;break $do_cond$$if_else$7; } //@line 2811 "dictionary.cpp"
  
          var $20=HEAP32[(($c)>>2)]; //@line 2813 "dictionary.cpp"
          var $21=$p2; //@line 2813 "dictionary.cpp"
          var $call7=__Z8utf8_outjPc($20, $21); //@line 2813 "dictionary.cpp"
          var $22=$p2; //@line 2813 "dictionary.cpp"
          var $add_ptr8=(($22+$call7)&4294967295); //@line 2813 "dictionary.cpp"
          $p2=$add_ptr8; //@line 2813 "dictionary.cpp"
          $all_alpha=0; //@line 2814 "dictionary.cpp"
          ; //@line 2815 "dictionary.cpp"
        }
      } while(0);
  
      var $23=HEAP32[(($c)>>2)]; //@line 2816 "dictionary.cpp"
      var $cmp10=((($23))|0)!=0; //@line 2816 "dictionary.cpp"
      if ($cmp10) { __label__ = 1;continue $do_body$2; } else { __label__ = 7;break $do_body$2; } //@line 2816 "dictionary.cpp"
    }
  
    var $24=$p2; //@line 2817 "dictionary.cpp"
    HEAP8[($24)]=0; //@line 2817 "dictionary.cpp"
    var $25=$all_alpha; //@line 2819 "dictionary.cpp"
    var $tobool=((($25))|0)!=0; //@line 2819 "dictionary.cpp"
    if ($tobool) { __label__ = 8;; } else { __label__ = 21;; } //@line 2819 "dictionary.cpp"
    if (__label__ == 8) {
  
      $acc=0; //@line 2822 "dictionary.cpp"
      $bits=0; //@line 2823 "dictionary.cpp"
      var $26=$text_addr; //@line 2825 "dictionary.cpp"
      $p=$26; //@line 2825 "dictionary.cpp"
      var $27=$text_addr; //@line 2826 "dictionary.cpp"
      $p2=$27; //@line 2826 "dictionary.cpp"
      var $28=$p; //@line 2827 "dictionary.cpp"
      var $incdec_ptr121=(($28+1)&4294967295); //@line 2827 "dictionary.cpp"
      $p=$incdec_ptr121; //@line 2827 "dictionary.cpp"
      var $29=HEAP8[($28)]; //@line 2827 "dictionary.cpp"
      var $conv132=(tempInt=(($29)),(tempInt>=128?tempInt-256:tempInt)); //@line 2827 "dictionary.cpp"
      HEAP32[(($c)>>2)]=$conv132; //@line 2827 "dictionary.cpp"
      var $cmp143=((($conv132))|0)!=0; //@line 2827 "dictionary.cpp"
      if ($cmp143) { __label__ = 9;; } else { __label__ = 18;; } //@line 2827 "dictionary.cpp"
      $while_body$$while_end$14: do { 
        if (__label__ == 9) {
          $while_body$15: while(1) { 
  
            var $30=$tr_addr; //@line 2829 "dictionary.cpp"
            var $frequent_pairs=(($30+8268)&4294967295); //@line 2829 "dictionary.cpp"
            var $31=HEAP32[(($frequent_pairs)>>2)]; //@line 2829 "dictionary.cpp"
            $pairs_list=$31; //@line 2829 "dictionary.cpp"
            var $cmp15=((($31))|0)!=0; //@line 2829 "dictionary.cpp"
            if ($cmp15) { __label__ = 10;; } else { __label__ = 15;; } //@line 2829 "dictionary.cpp"
            $if_then16$$if_end28$17: do { 
              if (__label__ == 10) {
  
                var $32=HEAP32[(($c)>>2)]; //@line 2831 "dictionary.cpp"
                var $33=$p; //@line 2831 "dictionary.cpp"
                var $34=HEAP8[($33)]; //@line 2831 "dictionary.cpp"
                var $conv17=(tempInt=(($34)),(tempInt>=128?tempInt-256:tempInt)); //@line 2831 "dictionary.cpp"
                var $shl=((($conv17)*256)&4294967295); //@line 2831 "dictionary.cpp"
                var $add18=((($shl)+($32))&4294967295); //@line 2831 "dictionary.cpp"
                $c2=$add18; //@line 2831 "dictionary.cpp"
                $ix=0; //@line 2832 "dictionary.cpp"
                ; //@line 2832 "dictionary.cpp"
                while(1) { 
  
                  var $35=$c2; //@line 2832 "dictionary.cpp"
                  var $36=$ix; //@line 2832 "dictionary.cpp"
                  var $37=$pairs_list; //@line 2832 "dictionary.cpp"
                  var $arrayidx=(($37+2*$36)&4294967295); //@line 2832 "dictionary.cpp"
                  var $38=HEAP16[(($arrayidx)>>1)]; //@line 2832 "dictionary.cpp"
                  var $conv19=(tempInt=(($38)),(tempInt>=32768?tempInt-65536:tempInt)); //@line 2832 "dictionary.cpp"
                  var $cmp20=((($35))|0) >= ((($conv19))|0); //@line 2832 "dictionary.cpp"
                  if (!($cmp20)) { __label__ = 15;break $if_then16$$if_end28$17; } //@line 2832 "dictionary.cpp"
  
                  var $39=$c2; //@line 2834 "dictionary.cpp"
                  var $40=$ix; //@line 2834 "dictionary.cpp"
                  var $41=$pairs_list; //@line 2834 "dictionary.cpp"
                  var $arrayidx21=(($41+2*$40)&4294967295); //@line 2834 "dictionary.cpp"
                  var $42=HEAP16[(($arrayidx21)>>1)]; //@line 2834 "dictionary.cpp"
                  var $conv22=(tempInt=(($42)),(tempInt>=32768?tempInt-65536:tempInt)); //@line 2834 "dictionary.cpp"
                  var $cmp23=((($39))|0)==((($conv22))|0); //@line 2834 "dictionary.cpp"
                  var $43=$ix; //@line 2837 "dictionary.cpp"
                  if ($cmp23) { __label__ = 13;break ; } //@line 2834 "dictionary.cpp"
  
                  var $inc=((($43)+1)&4294967295); //@line 2832 "dictionary.cpp"
                  $ix=$inc; //@line 2832 "dictionary.cpp"
                  __label__ = 11;continue ; //@line 2832 "dictionary.cpp"
                }
  
                var $44=$pairs_start; //@line 2837 "dictionary.cpp"
                var $add25=((($44)+($43))&4294967295); //@line 2837 "dictionary.cpp"
                HEAP32[(($c)>>2)]=$add25; //@line 2837 "dictionary.cpp"
                var $45=$p; //@line 2838 "dictionary.cpp"
                var $incdec_ptr26=(($45+1)&4294967295); //@line 2838 "dictionary.cpp"
                $p=$incdec_ptr26; //@line 2838 "dictionary.cpp"
                ; //@line 2839 "dictionary.cpp"
              }
            } while(0);
  
            var $46=$acc; //@line 2843 "dictionary.cpp"
            var $shl29=((($46)*64)&4294967295); //@line 2843 "dictionary.cpp"
            var $47=HEAP32[(($c)>>2)]; //@line 2843 "dictionary.cpp"
            var $and=($47) & 63; //@line 2843 "dictionary.cpp"
            var $add30=((($and)+($shl29))&4294967295); //@line 2843 "dictionary.cpp"
            $acc=$add30; //@line 2843 "dictionary.cpp"
            var $48=$bits; //@line 2844 "dictionary.cpp"
            var $add31=((($48)+6)&4294967295); //@line 2844 "dictionary.cpp"
            $bits=$add31; //@line 2844 "dictionary.cpp"
            var $cmp32=((($add31))|0) >= 8; //@line 2846 "dictionary.cpp"
            if ($cmp32) { __label__ = 16;; } else { __label__ = 17;; } //@line 2846 "dictionary.cpp"
            if (__label__ == 16) {
  
              var $49=$bits; //@line 2848 "dictionary.cpp"
              var $sub34=((($49)-8)&4294967295); //@line 2848 "dictionary.cpp"
              $bits=$sub34; //@line 2848 "dictionary.cpp"
              var $50=$acc; //@line 2849 "dictionary.cpp"
              var $51=$bits; //@line 2849 "dictionary.cpp"
              var $shr=($50) >> ((($51))|0); //@line 2849 "dictionary.cpp"
              var $conv35=((($shr)) & 255); //@line 2849 "dictionary.cpp"
              var $52=$p2; //@line 2849 "dictionary.cpp"
              var $incdec_ptr36=(($52+1)&4294967295); //@line 2849 "dictionary.cpp"
              $p2=$incdec_ptr36; //@line 2849 "dictionary.cpp"
              HEAP8[($52)]=$conv35; //@line 2849 "dictionary.cpp"
              ; //@line 2850 "dictionary.cpp"
            }
  
            var $53=$p; //@line 2827 "dictionary.cpp"
            var $incdec_ptr12=(($53+1)&4294967295); //@line 2827 "dictionary.cpp"
            $p=$incdec_ptr12; //@line 2827 "dictionary.cpp"
            var $54=HEAP8[($53)]; //@line 2827 "dictionary.cpp"
            var $conv13=(tempInt=(($54)),(tempInt>=128?tempInt-256:tempInt)); //@line 2827 "dictionary.cpp"
            HEAP32[(($c)>>2)]=$conv13; //@line 2827 "dictionary.cpp"
            var $cmp14=((($conv13))|0)!=0; //@line 2827 "dictionary.cpp"
            if ($cmp14) { __label__ = 9;continue $while_body$15; } else { __label__ = 18;break $while_body$$while_end$14; } //@line 2827 "dictionary.cpp"
          }
        }
      } while(0);
  
      var $55=$bits; //@line 2852 "dictionary.cpp"
      var $cmp38=((($55))|0) > 0; //@line 2852 "dictionary.cpp"
      if ($cmp38) { __label__ = 19;; } else { __label__ = 20;; } //@line 2852 "dictionary.cpp"
      if (__label__ == 19) {
  
        var $56=$acc; //@line 2854 "dictionary.cpp"
        var $57=$bits; //@line 2854 "dictionary.cpp"
        var $sub40=((8-($57))&4294967295); //@line 2854 "dictionary.cpp"
        var $shl41=($56) << ($sub40); //@line 2854 "dictionary.cpp"
        var $conv42=((($shl41)) & 255); //@line 2854 "dictionary.cpp"
        var $58=$p2; //@line 2854 "dictionary.cpp"
        var $incdec_ptr43=(($58+1)&4294967295); //@line 2854 "dictionary.cpp"
        $p2=$incdec_ptr43; //@line 2854 "dictionary.cpp"
        HEAP8[($58)]=$conv42; //@line 2854 "dictionary.cpp"
        ; //@line 2855 "dictionary.cpp"
      }
  
      var $59=$p2; //@line 2856 "dictionary.cpp"
      HEAP8[($59)]=0; //@line 2856 "dictionary.cpp"
      var $60=$p2; //@line 2857 "dictionary.cpp"
      var $61=$text_addr; //@line 2857 "dictionary.cpp"
      var $sub_ptr_lhs_cast=($60); //@line 2857 "dictionary.cpp"
      var $sub_ptr_rhs_cast=($61); //@line 2857 "dictionary.cpp"
      var $sub_ptr_sub=((($sub_ptr_lhs_cast)-($sub_ptr_rhs_cast))&4294967295); //@line 2857 "dictionary.cpp"
      var $or=($sub_ptr_sub) | 64; //@line 2857 "dictionary.cpp"
      $retval=$or; //@line 2857 "dictionary.cpp"
      ; //@line 2857 "dictionary.cpp"
    }
    else if (__label__ == 21) {
  
      var $62=$p2; //@line 2859 "dictionary.cpp"
      var $63=$text_addr; //@line 2859 "dictionary.cpp"
      var $sub_ptr_lhs_cast46=($62); //@line 2859 "dictionary.cpp"
      var $sub_ptr_rhs_cast47=($63); //@line 2859 "dictionary.cpp"
      var $sub_ptr_sub48=((($sub_ptr_lhs_cast46)-($sub_ptr_rhs_cast47))&4294967295); //@line 2859 "dictionary.cpp"
      $retval=$sub_ptr_sub48; //@line 2859 "dictionary.cpp"
      ; //@line 2859 "dictionary.cpp"
    }
  
    var $64=$retval; //@line 2860 "dictionary.cpp"
    STACKTOP = __stackBase__;
    return $64; //@line 2860 "dictionary.cpp"
    return null;
  }
  

  function __Z14LookupDictListP10TranslatorPPcS1_PjiP8WORD_TAB($tr, $wordptr, $ph_out, $flags, $end_flags, $wtab) {
    var __stackBase__  = STACKTOP; STACKTOP += 164; _memset(__stackBase__, 0, 164);
    var __label__;
    var __lastLabel__ = null;
  
    var $retval;
    var $tr_addr;
    var $wordptr_addr;
    var $ph_out_addr;
    var $flags_addr;
    var $end_flags_addr;
    var $wtab_addr;
    var $length;
    var $found;
    var $word1;
    var $word2;
    var $c;
    var $nbytes;
    var $len;
    var $word=__stackBase__;
    var $letter=__stackBase__+160;
    $tr_addr=$tr;
    $wordptr_addr=$wordptr;
    $ph_out_addr=$ph_out;
    $flags_addr=$flags;
    $end_flags_addr=$end_flags;
    $wtab_addr=$wtab;
    $length=0; //@line 3221 "dictionary.cpp"
    var $0=$wordptr_addr; //@line 3222 "dictionary.cpp"
    var $1=HEAP32[(($0)>>2)]; //@line 3222 "dictionary.cpp"
    $word1=$1; //@line 3222 "dictionary.cpp"
    $word2=$1; //@line 3222 "dictionary.cpp"
    ; //@line 3224 "dictionary.cpp"
    $while_cond$2: while(1) { 
  
      var $2=$word2; //@line 3224 "dictionary.cpp"
      var $call=__Z11utf8_nbytesPKc($2); //@line 3224 "dictionary.cpp"
      $nbytes=$call; //@line 3224 "dictionary.cpp"
      var $3=$word2; //@line 3224 "dictionary.cpp"
      var $arrayidx=(($3+$call)&4294967295); //@line 3224 "dictionary.cpp"
      var $4=HEAP8[($arrayidx)]; //@line 3224 "dictionary.cpp"
      var $conv=(tempInt=(($4)),(tempInt>=128?tempInt-256:tempInt)); //@line 3224 "dictionary.cpp"
      var $cmp=((($conv))|0)==32; //@line 3224 "dictionary.cpp"
      if (!($cmp)) { __label__ = 2;break $while_cond$2; } //@line 3224 "dictionary.cpp"
  
      var $6=$nbytes; //@line 3224 "dictionary.cpp"
      var $add=((($6)+1)&4294967295); //@line 3224 "dictionary.cpp"
      var $7=$word2; //@line 3224 "dictionary.cpp"
      var $arrayidx1=(($7+$add)&4294967295); //@line 3224 "dictionary.cpp"
      var $8=HEAP8[($arrayidx1)]; //@line 3224 "dictionary.cpp"
      var $conv2=(tempInt=(($8)),(tempInt>=128?tempInt-256:tempInt)); //@line 3224 "dictionary.cpp"
      var $cmp3=((($conv2))|0)==46; //@line 3224 "dictionary.cpp"
      var $9=$length; //@line 3228 "dictionary.cpp"
      if ($cmp3) { __lastLabel__ = 3; ; } else { __lastLabel__ = 3; __label__ = 5;break $while_cond$2; }
  
      var $arrayidx4=(($word+$9)&4294967295); //@line 3228 "dictionary.cpp"
      var $10=$word2; //@line 3228 "dictionary.cpp"
      var $11=$nbytes; //@line 3228 "dictionary.cpp"
      var $src$, $dest$, $stop$, $stop4$, $fast$;
      $src$ = $10;
      $dest$ = $arrayidx4;
      $stop$ = $src$ + $11;
      if (($dest$%4) == ($src$%4) && $11 > 8) {
        while ($src$%4 !== 0 && $src$ < $stop$) {
          ; HEAP8[$dest$++] = HEAP8[$src$++];
        }
        $src$ >>= 2;
        $dest$ >>= 2;
        $stop4$ = $stop$ >> 2;
        while ($src$ < $stop4$) {
      ;;;    HEAP32[$dest$++] = HEAP32[$src$++];
        }
        $src$ <<= 2;
        $dest$ <<= 2;
      }while ($src$ < $stop$) {
        ; HEAP8[$dest$++] = HEAP8[$src$++];
      }; //@line 3228 "dictionary.cpp"
      var $12=$nbytes; //@line 3229 "dictionary.cpp"
      var $13=$length; //@line 3229 "dictionary.cpp"
      var $add5=((($13)+($12))&4294967295); //@line 3229 "dictionary.cpp"
      $length=$add5; //@line 3229 "dictionary.cpp"
      var $14=$length; //@line 3230 "dictionary.cpp"
      var $inc=((($14)+1)&4294967295); //@line 3230 "dictionary.cpp"
      $length=$inc; //@line 3230 "dictionary.cpp"
      var $arrayidx6=(($word+$14)&4294967295); //@line 3230 "dictionary.cpp"
      HEAP8[($arrayidx6)]=46; //@line 3230 "dictionary.cpp"
      var $15=$nbytes; //@line 3231 "dictionary.cpp"
      var $add7=((($15)+3)&4294967295); //@line 3231 "dictionary.cpp"
      var $16=$word2; //@line 3231 "dictionary.cpp"
      var $add_ptr=(($16+$add7)&4294967295); //@line 3231 "dictionary.cpp"
      $word2=$add_ptr; //@line 3231 "dictionary.cpp"
      __label__ = 1;continue $while_cond$2; //@line 3232 "dictionary.cpp"
    }
    if (__label__ == 2) {
  
      var $5=$length; //@line 3228 "dictionary.cpp"
      __lastLabel__ = 2; ;
    }
  
    var $17=__lastLabel__ == 2 ? $5 : ($9);
    var $cmp8=((($17))|0) > 0; //@line 3233 "dictionary.cpp"
    if ($cmp8) { __label__ = 6;; } else { __label__ = 12;; } //@line 3233 "dictionary.cpp"
    $if_then$$if_end26$9: do { 
      if (__label__ == 6) {
  
        $nbytes=0; //@line 3236 "dictionary.cpp"
        ; //@line 3237 "dictionary.cpp"
        while(1) { 
  
          var $18=$nbytes; //@line 3237 "dictionary.cpp"
          var $19=$word2; //@line 3237 "dictionary.cpp"
          var $arrayidx10=(($19+$18)&4294967295); //@line 3237 "dictionary.cpp"
          var $20=HEAPU8[($arrayidx10)]; //@line 3237 "dictionary.cpp"
          $c=$20; //@line 3237 "dictionary.cpp"
          var $conv11=((($20))&255); //@line 3237 "dictionary.cpp"
          var $cmp12=((($conv11))|0)!=0; //@line 3237 "dictionary.cpp"
          if (!($cmp12)) { __label__ = 10;break ; } //@line 3237 "dictionary.cpp"
  
          var $21=$c; //@line 3237 "dictionary.cpp"
          var $conv14=((($21))&255); //@line 3237 "dictionary.cpp"
          var $cmp15=((($conv14))|0)!=32; //@line 3237 "dictionary.cpp"
          if (!($cmp15)) { __label__ = 10;break ; }
  
          var $22=$nbytes; //@line 3239 "dictionary.cpp"
          var $inc18=((($22)+1)&4294967295); //@line 3239 "dictionary.cpp"
          $nbytes=$inc18; //@line 3239 "dictionary.cpp"
          __label__ = 7;continue ; //@line 3240 "dictionary.cpp"
        }
  
        var $23=$length; //@line 3241 "dictionary.cpp"
        var $arrayidx20=(($word+$23)&4294967295); //@line 3241 "dictionary.cpp"
        var $24=$word2; //@line 3241 "dictionary.cpp"
        var $25=$nbytes; //@line 3241 "dictionary.cpp"
        var $src$, $dest$, $stop$, $stop4$, $fast$;
        $src$ = $24;
        $dest$ = $arrayidx20;
        $stop$ = $src$ + $25;
        if (($dest$%4) == ($src$%4) && $25 > 8) {
          while ($src$%4 !== 0 && $src$ < $stop$) {
            ; HEAP8[$dest$++] = HEAP8[$src$++];
          }
          $src$ >>= 2;
          $dest$ >>= 2;
          $stop4$ = $stop$ >> 2;
          while ($src$ < $stop4$) {
        ;;;    HEAP32[$dest$++] = HEAP32[$src$++];
          }
          $src$ <<= 2;
          $dest$ <<= 2;
        }while ($src$ < $stop$) {
          ; HEAP8[$dest$++] = HEAP8[$src$++];
        }; //@line 3241 "dictionary.cpp"
        var $26=$length; //@line 3242 "dictionary.cpp"
        var $27=$nbytes; //@line 3242 "dictionary.cpp"
        var $add21=((($27)+($26))&4294967295); //@line 3242 "dictionary.cpp"
        var $arrayidx22=(($word+$add21)&4294967295); //@line 3242 "dictionary.cpp"
        HEAP8[($arrayidx22)]=0; //@line 3242 "dictionary.cpp"
        var $28=$tr_addr; //@line 3243 "dictionary.cpp"
        var $arraydecay=(($word)&4294967295); //@line 3243 "dictionary.cpp"
        var $29=$word2; //@line 3243 "dictionary.cpp"
        var $30=$ph_out_addr; //@line 3243 "dictionary.cpp"
        var $31=$flags_addr; //@line 3243 "dictionary.cpp"
        var $32=$end_flags_addr; //@line 3243 "dictionary.cpp"
        var $33=$wtab_addr; //@line 3243 "dictionary.cpp"
        var $call23=__ZL11LookupDict2P10TranslatorPKcS2_PcPjiP8WORD_TAB($28, $arraydecay, $29, $30, $31, $32, $33); //@line 3243 "dictionary.cpp"
        $found=$call23; //@line 3243 "dictionary.cpp"
        var $tobool=((($call23))|0)!=0; //@line 3244 "dictionary.cpp"
        if (!($tobool)) { __label__ = 12;break $if_then$$if_end26$9; } //@line 3244 "dictionary.cpp"
  
        var $34=$flags_addr; //@line 3247 "dictionary.cpp"
        var $arrayidx25=(($34)&4294967295); //@line 3247 "dictionary.cpp"
        var $35=HEAP32[(($arrayidx25)>>2)]; //@line 3247 "dictionary.cpp"
        var $or=($35) | 128; //@line 3247 "dictionary.cpp"
        HEAP32[(($arrayidx25)>>2)]=$or; //@line 3247 "dictionary.cpp"
        var $36=$length; //@line 3248 "dictionary.cpp"
        HEAP32[((_dictionary_skipwords)>>2)]=$36; //@line 3248 "dictionary.cpp"
        $retval=1; //@line 3249 "dictionary.cpp"
        __label__ = 50;break $if_then$$if_end26$9; //@line 3249 "dictionary.cpp"
      }
    } while(0);
    if (__label__ == 12) {
  
      $length=0; //@line 3253 "dictionary.cpp"
      __lastLabel__ = 12; ; //@line 3253 "dictionary.cpp"
      $for_cond$19: while(1) { 
  
        var $37=__lastLabel__ == 19 ? $inc45 : (0); //@line 3253 "dictionary.cpp"
        var $cmp27=((($37))|0) < 159; //@line 3253 "dictionary.cpp"
        if (!($cmp27)) { __label__ = 20;break $for_cond$19; } //@line 3253 "dictionary.cpp"
  
        var $38=$word1; //@line 3255 "dictionary.cpp"
        var $incdec_ptr=(($38+1)&4294967295); //@line 3255 "dictionary.cpp"
        $word1=$incdec_ptr; //@line 3255 "dictionary.cpp"
        var $39=HEAPU8[($38)]; //@line 3255 "dictionary.cpp"
        $c=$39; //@line 3255 "dictionary.cpp"
        var $conv28=((($39))&255); //@line 3255 "dictionary.cpp"
        var $cmp29=((($conv28))|0)==0; //@line 3255 "dictionary.cpp"
        if ($cmp29) { __label__ = 20;break $for_cond$19; } //@line 3255 "dictionary.cpp"
  
        var $40=$c; //@line 3255 "dictionary.cpp"
        var $conv30=((($40))&255); //@line 3255 "dictionary.cpp"
        var $cmp31=((($conv30))|0)==32; //@line 3255 "dictionary.cpp"
        if ($cmp31) { __label__ = 20;break $for_cond$19; } //@line 3255 "dictionary.cpp"
  
        var $41=$c; //@line 3258 "dictionary.cpp"
        var $conv34=((($41))&255); //@line 3258 "dictionary.cpp"
        var $cmp35=((($conv34))|0)==46; //@line 3258 "dictionary.cpp"
        if ($cmp35) { __label__ = 17;; } else { __label__ = 19;; } //@line 3258 "dictionary.cpp"
        $land_lhs_true$$if_end43$24: do { 
          if (__label__ == 17) {
  
            var $42=$length; //@line 3258 "dictionary.cpp"
            var $cmp36=((($42))|0) > 0; //@line 3258 "dictionary.cpp"
            if (!($cmp36)) { __label__ = 19;break $land_lhs_true$$if_end43$24; } //@line 3258 "dictionary.cpp"
  
            var $43=$length; //@line 3258 "dictionary.cpp"
            var $sub=((($43)-1)&4294967295); //@line 3258 "dictionary.cpp"
            var $arrayidx38=(($word+$sub)&4294967295); //@line 3258 "dictionary.cpp"
            var $44=HEAP8[($arrayidx38)]; //@line 3258 "dictionary.cpp"
            var $conv39=(tempInt=(($44)),(tempInt>=128?tempInt-256:tempInt)); //@line 3258 "dictionary.cpp"
            var $isdigittmp=((($conv39)-48)&4294967295); //@line 3258 "dictionary.cpp"
            var $isdigit=((($isdigittmp))>>>0) < 10; //@line 3258 "dictionary.cpp"
            var $call40=((($isdigit))&1); //@line 3258 "dictionary.cpp"
            var $tobool41=((($call40))|0)!=0; //@line 3258 "dictionary.cpp"
            if ($tobool41) { __label__ = 20;break $for_cond$19; } //@line 3258 "dictionary.cpp"
          }
        } while(0);
  
        var $45=$c; //@line 3261 "dictionary.cpp"
        var $46=$length; //@line 3261 "dictionary.cpp"
        var $arrayidx44=(($word+$46)&4294967295); //@line 3261 "dictionary.cpp"
        HEAP8[($arrayidx44)]=$45; //@line 3261 "dictionary.cpp"
        var $47=$length; //@line 3253 "dictionary.cpp"
        var $inc45=((($47)+1)&4294967295); //@line 3253 "dictionary.cpp"
        $length=$inc45; //@line 3253 "dictionary.cpp"
        __lastLabel__ = 19; __label__ = 13;continue $for_cond$19; //@line 3253 "dictionary.cpp"
      }
  
      var $48=$length; //@line 3263 "dictionary.cpp"
      var $arrayidx46=(($word+$48)&4294967295); //@line 3263 "dictionary.cpp"
      HEAP8[($arrayidx46)]=0; //@line 3263 "dictionary.cpp"
      var $49=$tr_addr; //@line 3265 "dictionary.cpp"
      var $arraydecay47=(($word)&4294967295); //@line 3265 "dictionary.cpp"
      var $50=$word1; //@line 3265 "dictionary.cpp"
      var $51=$ph_out_addr; //@line 3265 "dictionary.cpp"
      var $52=$flags_addr; //@line 3265 "dictionary.cpp"
      var $53=$end_flags_addr; //@line 3265 "dictionary.cpp"
      var $54=$wtab_addr; //@line 3265 "dictionary.cpp"
      var $call48=__ZL11LookupDict2P10TranslatorPKcS2_PcPjiP8WORD_TAB($49, $arraydecay47, $50, $51, $52, $53, $54); //@line 3265 "dictionary.cpp"
      $found=$call48; //@line 3265 "dictionary.cpp"
      var $55=$flags_addr; //@line 3267 "dictionary.cpp"
      var $arrayidx49=(($55)&4294967295); //@line 3267 "dictionary.cpp"
      var $56=HEAP32[(($arrayidx49)>>2)]; //@line 3267 "dictionary.cpp"
      var $and=($56) & 134217728; //@line 3267 "dictionary.cpp"
      var $tobool50=((($and))|0)!=0; //@line 3267 "dictionary.cpp"
      if ($tobool50) { __label__ = 21;; } else { __label__ = 25;; } //@line 3267 "dictionary.cpp"
      $if_then51$$if_else66$29: do { 
        if (__label__ == 21) {
  
          var $57=$ph_out_addr; //@line 3269 "dictionary.cpp"
          var $58=$tr_addr; //@line 3269 "dictionary.cpp"
          var $phonemes_repeat=(($58+632)&4294967295); //@line 3269 "dictionary.cpp"
          var $arraydecay52=(($phonemes_repeat)&4294967295); //@line 3269 "dictionary.cpp"
          var $call53=_strcmp($57, $arraydecay52); //@line 3269 "dictionary.cpp"
          var $cmp54=((($call53))|0)==0; //@line 3269 "dictionary.cpp"
          var $59=$tr_addr; //@line 3271 "dictionary.cpp"
          if ($cmp54) { __label__ = 22;; } else { __label__ = 24;; } //@line 3269 "dictionary.cpp"
          if (__label__ == 22) {
  
            var $phonemes_repeat_count=(($59+652)&4294967295); //@line 3271 "dictionary.cpp"
            var $60=HEAP32[(($phonemes_repeat_count)>>2)]; //@line 3271 "dictionary.cpp"
            var $inc56=((($60)+1)&4294967295); //@line 3271 "dictionary.cpp"
            HEAP32[(($phonemes_repeat_count)>>2)]=$inc56; //@line 3271 "dictionary.cpp"
            var $61=$tr_addr; //@line 3272 "dictionary.cpp"
            var $phonemes_repeat_count57=(($61+652)&4294967295); //@line 3272 "dictionary.cpp"
            var $62=HEAP32[(($phonemes_repeat_count57)>>2)]; //@line 3272 "dictionary.cpp"
            var $cmp58=((($62))|0) > 3; //@line 3272 "dictionary.cpp"
            if (!($cmp58)) { __label__ = 26;break $if_then51$$if_else66$29; } //@line 3272 "dictionary.cpp"
  
            var $63=$ph_out_addr; //@line 3274 "dictionary.cpp"
            var $arrayidx60=(($63)&4294967295); //@line 3274 "dictionary.cpp"
            HEAP8[($arrayidx60)]=0; //@line 3274 "dictionary.cpp"
            ; //@line 3275 "dictionary.cpp"
          }
          else if (__label__ == 24) {
  
            var $phonemes_repeat62=(($59+632)&4294967295); //@line 3279 "dictionary.cpp"
            var $arraydecay63=(($phonemes_repeat62)&4294967295); //@line 3279 "dictionary.cpp"
            var $64=$ph_out_addr; //@line 3279 "dictionary.cpp"
            __Z8strncpy0PcPKci($arraydecay63, $64, 20); //@line 3279 "dictionary.cpp"
            var $65=$tr_addr; //@line 3280 "dictionary.cpp"
            var $phonemes_repeat_count64=(($65+652)&4294967295); //@line 3280 "dictionary.cpp"
            HEAP32[(($phonemes_repeat_count64)>>2)]=1; //@line 3280 "dictionary.cpp"
            ;
          }
        }
        else if (__label__ == 25) {
  
          var $66=$tr_addr; //@line 3285 "dictionary.cpp"
          var $phonemes_repeat_count67=(($66+652)&4294967295); //@line 3285 "dictionary.cpp"
          HEAP32[(($phonemes_repeat_count67)>>2)]=0; //@line 3285 "dictionary.cpp"
          ;
        }
      } while(0);
  
      var $67=$found; //@line 3289 "dictionary.cpp"
      var $cmp69=((($67))|0)==0; //@line 3289 "dictionary.cpp"
      if ($cmp69) { __label__ = 27;; } else { __label__ = 39;; } //@line 3289 "dictionary.cpp"
      $land_lhs_true70$$if_end117thread_pre_split$37: do { 
        if (__label__ == 27) {
  
          var $68=$flags_addr; //@line 3289 "dictionary.cpp"
          var $arrayidx71=(($68+4)&4294967295); //@line 3289 "dictionary.cpp"
          var $69=HEAP32[(($arrayidx71)>>2)]; //@line 3289 "dictionary.cpp"
          var $and72=($69) & 2048; //@line 3289 "dictionary.cpp"
          var $tobool73=((($and72))|0)!=0; //@line 3289 "dictionary.cpp"
          if ($tobool73) { __label__ = 28;; } else { __label__ = 31;; } //@line 3289 "dictionary.cpp"
          if (__label__ == 28) {
  
            var $arraydecay75=(($word)&4294967295); //@line 3292 "dictionary.cpp"
            $word2=$arraydecay75; //@line 3292 "dictionary.cpp"
            var $70=$word2; //@line 3293 "dictionary.cpp"
            var $71=HEAP8[($70)]; //@line 3293 "dictionary.cpp"
            var $conv76=(tempInt=(($71)),(tempInt>=128?tempInt-256:tempInt)); //@line 3293 "dictionary.cpp"
            var $cmp77=((($conv76))|0)==95; //@line 3293 "dictionary.cpp"
            if ($cmp77) { __label__ = 29;; } else { __label__ = 30;; } //@line 3293 "dictionary.cpp"
            if (__label__ == 29) {
  
              var $72=$word2; //@line 3293 "dictionary.cpp"
              var $incdec_ptr79=(($72+1)&4294967295); //@line 3293 "dictionary.cpp"
              $word2=$incdec_ptr79; //@line 3293 "dictionary.cpp"
              ; //@line 3293 "dictionary.cpp"
            }
  
            var $73=$word2; //@line 3294 "dictionary.cpp"
            var $call81=__Z7utf8_inPiPKc($letter, $73); //@line 3294 "dictionary.cpp"
            $len=$call81; //@line 3294 "dictionary.cpp"
            var $74=$tr_addr; //@line 3295 "dictionary.cpp"
            var $75=HEAP32[(($letter)>>2)]; //@line 3295 "dictionary.cpp"
            var $76=$ph_out_addr; //@line 3295 "dictionary.cpp"
            __Z20LookupAccentedLetterP10TranslatorjPc($74, $75, $76); //@line 3295 "dictionary.cpp"
            var $77=$word2; //@line 3296 "dictionary.cpp"
            var $78=$len; //@line 3296 "dictionary.cpp"
            var $add_ptr82=(($77+$78)&4294967295); //@line 3296 "dictionary.cpp"
            $found=$add_ptr82; //@line 3296 "dictionary.cpp"
            __lastLabel__ = 30; ; //@line 3297 "dictionary.cpp"
          }
          else if (__label__ == 31) {
  
            var $_pr=$found; //@line 3299 "dictionary.cpp"
            __lastLabel__ = 31; ;
          }
  
          var $79=__lastLabel__ == 31 ? $_pr : ($add_ptr82); //@line 3299 "dictionary.cpp"
          var $cmp84=((($79))|0)==0; //@line 3299 "dictionary.cpp"
          if (!($cmp84)) { __label__ = 39;break $land_lhs_true70$$if_end117thread_pre_split$37; } //@line 3299 "dictionary.cpp"
  
          var $80=$ph_out_addr; //@line 3301 "dictionary.cpp"
          var $arrayidx86=(($80)&4294967295); //@line 3301 "dictionary.cpp"
          HEAP8[($arrayidx86)]=0; //@line 3301 "dictionary.cpp"
          var $81=$end_flags_addr; //@line 3305 "dictionary.cpp"
          var $and87=($81) & 16; //@line 3305 "dictionary.cpp"
          var $tobool88=((($and87))|0)!=0; //@line 3305 "dictionary.cpp"
          if ($tobool88) { __label__ = 34;; } else { __label__ = 36;; } //@line 3305 "dictionary.cpp"
          $land_lhs_true89$$if_else99$47: do { 
            if (__label__ == 34) {
  
              var $82=$length; //@line 3305 "dictionary.cpp"
              var $sub90=((($82)-1)&4294967295); //@line 3305 "dictionary.cpp"
              var $arrayidx91=(($word+$sub90)&4294967295); //@line 3305 "dictionary.cpp"
              var $83=HEAP8[($arrayidx91)]; //@line 3305 "dictionary.cpp"
              var $conv92=(tempInt=(($83)),(tempInt>=128?tempInt-256:tempInt)); //@line 3305 "dictionary.cpp"
              var $cmp93=((($conv92))|0)==101; //@line 3305 "dictionary.cpp"
              if (!($cmp93)) { __label__ = 36;break $land_lhs_true89$$if_else99$47; } //@line 3305 "dictionary.cpp"
  
              var $84=$length; //@line 3308 "dictionary.cpp"
              var $sub95=((($84)-1)&4294967295); //@line 3308 "dictionary.cpp"
              var $arrayidx96=(($word+$sub95)&4294967295); //@line 3308 "dictionary.cpp"
              HEAP8[($arrayidx96)]=0; //@line 3308 "dictionary.cpp"
              var $85=$tr_addr; //@line 3309 "dictionary.cpp"
              var $arraydecay97=(($word)&4294967295); //@line 3309 "dictionary.cpp"
              var $86=$word1; //@line 3309 "dictionary.cpp"
              var $87=$ph_out_addr; //@line 3309 "dictionary.cpp"
              var $88=$flags_addr; //@line 3309 "dictionary.cpp"
              var $89=$end_flags_addr; //@line 3309 "dictionary.cpp"
              var $90=$wtab_addr; //@line 3309 "dictionary.cpp"
              var $call98=__ZL11LookupDict2P10TranslatorPKcS2_PcPjiP8WORD_TAB($85, $arraydecay97, $86, $87, $88, $89, $90); //@line 3309 "dictionary.cpp"
              $found=$call98; //@line 3309 "dictionary.cpp"
              __lastLabel__ = 35; __label__ = 40;break $land_lhs_true70$$if_end117thread_pre_split$37; //@line 3310 "dictionary.cpp"
            }
          } while(0);
  
          var $91=$end_flags_addr; //@line 3312 "dictionary.cpp"
          var $and100=($91) & 4096; //@line 3312 "dictionary.cpp"
          var $tobool101=((($and100))|0)!=0; //@line 3312 "dictionary.cpp"
          if (!($tobool101)) { __label__ = 39;break $land_lhs_true70$$if_end117thread_pre_split$37; } //@line 3312 "dictionary.cpp"
  
          var $92=$length; //@line 3312 "dictionary.cpp"
          var $sub103=((($92)-1)&4294967295); //@line 3312 "dictionary.cpp"
          var $arrayidx104=(($word+$sub103)&4294967295); //@line 3312 "dictionary.cpp"
          var $93=HEAP8[($arrayidx104)]; //@line 3312 "dictionary.cpp"
          var $conv105=(tempInt=(($93)),(tempInt>=128?tempInt-256:tempInt)); //@line 3312 "dictionary.cpp"
          var $94=$length; //@line 3312 "dictionary.cpp"
          var $sub106=((($94)-2)&4294967295); //@line 3312 "dictionary.cpp"
          var $arrayidx107=(($word+$sub106)&4294967295); //@line 3312 "dictionary.cpp"
          var $95=HEAP8[($arrayidx107)]; //@line 3312 "dictionary.cpp"
          var $conv108=(tempInt=(($95)),(tempInt>=128?tempInt-256:tempInt)); //@line 3312 "dictionary.cpp"
          var $cmp109=((($conv105))|0)==((($conv108))|0); //@line 3312 "dictionary.cpp"
          if (!($cmp109)) { __label__ = 39;break $land_lhs_true70$$if_end117thread_pre_split$37; } //@line 3312 "dictionary.cpp"
  
          var $96=$length; //@line 3315 "dictionary.cpp"
          var $sub111=((($96)-1)&4294967295); //@line 3315 "dictionary.cpp"
          var $arrayidx112=(($word+$sub111)&4294967295); //@line 3315 "dictionary.cpp"
          HEAP8[($arrayidx112)]=0; //@line 3315 "dictionary.cpp"
          var $97=$tr_addr; //@line 3316 "dictionary.cpp"
          var $arraydecay113=(($word)&4294967295); //@line 3316 "dictionary.cpp"
          var $98=$word1; //@line 3316 "dictionary.cpp"
          var $99=$ph_out_addr; //@line 3316 "dictionary.cpp"
          var $100=$flags_addr; //@line 3316 "dictionary.cpp"
          var $101=$end_flags_addr; //@line 3316 "dictionary.cpp"
          var $102=$wtab_addr; //@line 3316 "dictionary.cpp"
          var $call114=__ZL11LookupDict2P10TranslatorPKcS2_PcPjiP8WORD_TAB($97, $arraydecay113, $98, $99, $100, $101, $102); //@line 3316 "dictionary.cpp"
          $found=$call114; //@line 3316 "dictionary.cpp"
          __lastLabel__ = 38; __label__ = 40;break $land_lhs_true70$$if_end117thread_pre_split$37; //@line 3317 "dictionary.cpp"
        }
      } while(0);
      if (__label__ == 39) {
  
        var $_pr2=$found; //@line 3320 "dictionary.cpp"
        __lastLabel__ = 39; ;
      }
  
      var $103=__lastLabel__ == 39 ? $_pr2 : (__lastLabel__ == 35 ? $call98 : ($call114)); //@line 3320 "dictionary.cpp"
      var $tobool118=((($103))|0)!=0; //@line 3320 "dictionary.cpp"
      if ($tobool118) { __label__ = 41;; } else { __label__ = 49;; } //@line 3320 "dictionary.cpp"
      if (__label__ == 41) {
  
        var $104=$tr_addr; //@line 3323 "dictionary.cpp"
        var $langopts=(($104)&4294967295); //@line 3323 "dictionary.cpp"
        var $textmode=(($langopts+252)&4294967295); //@line 3323 "dictionary.cpp"
        var $105=HEAP8[($textmode)]; //@line 3323 "dictionary.cpp"
        var $tobool120=(tempInt=(($105)),(tempInt>=128?tempInt-256:tempInt))!=0; //@line 3323 "dictionary.cpp"
        if ($tobool120) { __label__ = 42;; } else { __label__ = 43;; } //@line 3323 "dictionary.cpp"
        if (__label__ == 42) {
  
          var $106=$flags_addr; //@line 3324 "dictionary.cpp"
          var $107=HEAP32[(($106)>>2)]; //@line 3324 "dictionary.cpp"
          var $xor=($107) ^ 536870912; //@line 3324 "dictionary.cpp"
          HEAP32[(($106)>>2)]=$xor; //@line 3324 "dictionary.cpp"
          ; //@line 3324 "dictionary.cpp"
        }
  
        var $108=$flags_addr; //@line 3326 "dictionary.cpp"
        var $109=HEAP32[(($108)>>2)]; //@line 3326 "dictionary.cpp"
        var $and123=($109) & 536870912; //@line 3326 "dictionary.cpp"
        var $tobool124=((($and123))|0)!=0; //@line 3326 "dictionary.cpp"
        if ($tobool124) { __label__ = 44;; } else { __label__ = 48;; } //@line 3326 "dictionary.cpp"
        if (__label__ == 44) {
  
          var $110=$end_flags_addr; //@line 3330 "dictionary.cpp"
          var $and126=($110) & 2; //@line 3330 "dictionary.cpp"
          var $tobool127=((($and126))|0)!=0; //@line 3330 "dictionary.cpp"
          if ($tobool127) { __label__ = 45;; } else { __label__ = 47;; } //@line 3330 "dictionary.cpp"
          $if_then128$$if_end138$63: do { 
            if (__label__ == 45) {
  
              HEAP8[(((__ZZ14LookupDictListP10TranslatorPPcS1_PjiP8WORD_TABE16word_replacement)&4294967295))]=0; //@line 3333 "dictionary.cpp"
              HEAP8[(((__ZZ14LookupDictListP10TranslatorPPcS1_PjiP8WORD_TABE16word_replacement+1)&4294967295))]=32; //@line 3334 "dictionary.cpp"
              var $111=$ph_out_addr; //@line 3335 "dictionary.cpp"
              var $call129=_sprintf(((__ZZ14LookupDictListP10TranslatorPPcS1_PjiP8WORD_TABE16word_replacement+2)&4294967295), ((__str15131)&4294967295), allocate([$111,0,0,0], ["i8*",0,0,0], ALLOC_STACK)); //@line 3335 "dictionary.cpp"
              var $112=$wordptr_addr; //@line 3337 "dictionary.cpp"
              var $113=HEAP32[(($112)>>2)]; //@line 3337 "dictionary.cpp"
              $word1=$113; //@line 3337 "dictionary.cpp"
              var $114=$wordptr_addr; //@line 3338 "dictionary.cpp"
              HEAP32[(($114)>>2)]=((__ZZ14LookupDictListP10TranslatorPPcS1_PjiP8WORD_TABE16word_replacement+2)&4294967295); //@line 3338 "dictionary.cpp"
              var $115=HEAP32[((_option_phonemes)>>2)]; //@line 3340 "dictionary.cpp"
              var $cmp130=((($115))|0)==2; //@line 3340 "dictionary.cpp"
              if (!($cmp130)) { __label__ = 47;break $if_then128$$if_end138$63; } //@line 3340 "dictionary.cpp"
  
              var $116=$found; //@line 3342 "dictionary.cpp"
              var $117=$word1; //@line 3342 "dictionary.cpp"
              var $sub_ptr_lhs_cast=($116); //@line 3342 "dictionary.cpp"
              var $sub_ptr_rhs_cast=($117); //@line 3342 "dictionary.cpp"
              var $sub_ptr_sub=((($sub_ptr_lhs_cast)-($sub_ptr_rhs_cast))&4294967295); //@line 3342 "dictionary.cpp"
              $len=$sub_ptr_sub; //@line 3342 "dictionary.cpp"
              var $arraydecay132=(($word)&4294967295); //@line 3343 "dictionary.cpp"
              var $118=$word1; //@line 3343 "dictionary.cpp"
              var $119=$len; //@line 3343 "dictionary.cpp"
              var $src$, $dest$, $stop$, $stop4$, $fast$;
              $src$ = $118;
              $dest$ = $arraydecay132;
              $stop$ = $src$ + $119;
              if (($dest$%4) == ($src$%4) && $119 > 8) {
                while ($src$%4 !== 0 && $src$ < $stop$) {
                  ; HEAP8[$dest$++] = HEAP8[$src$++];
                }
                $src$ >>= 2;
                $dest$ >>= 2;
                $stop4$ = $stop$ >> 2;
                while ($src$ < $stop4$) {
              ;;;    HEAP32[$dest$++] = HEAP32[$src$++];
                }
                $src$ <<= 2;
                $dest$ <<= 2;
              }while ($src$ < $stop$) {
                ; HEAP8[$dest$++] = HEAP8[$src$++];
              }; //@line 3343 "dictionary.cpp"
              var $120=$len; //@line 3344 "dictionary.cpp"
              var $arrayidx133=(($word+$120)&4294967295); //@line 3344 "dictionary.cpp"
              HEAP8[($arrayidx133)]=0; //@line 3344 "dictionary.cpp"
              var $121=HEAP32[((_f_trans)>>2)]; //@line 3345 "dictionary.cpp"
              var $arraydecay134=(($word)&4294967295); //@line 3345 "dictionary.cpp"
              var $122=$wordptr_addr; //@line 3345 "dictionary.cpp"
              var $123=HEAP32[(($122)>>2)]; //@line 3345 "dictionary.cpp"
              var $call135=_fprintf($121, ((__str16132)&4294967295), allocate([$arraydecay134,0,0,0,$123,0,0,0], ["i8*",0,0,0,"i8*",0,0,0], ALLOC_STACK)); //@line 3345 "dictionary.cpp"
              ; //@line 3346 "dictionary.cpp"
            }
          } while(0);
  
          var $124=$ph_out_addr; //@line 3353 "dictionary.cpp"
          var $arrayidx139=(($124)&4294967295); //@line 3353 "dictionary.cpp"
          HEAP8[($arrayidx139)]=0; //@line 3353 "dictionary.cpp"
          $retval=0; //@line 3354 "dictionary.cpp"
          ; //@line 3354 "dictionary.cpp"
        }
        else if (__label__ == 48) {
  
          $retval=1; //@line 3357 "dictionary.cpp"
          ; //@line 3357 "dictionary.cpp"
        }
      }
      else if (__label__ == 49) {
  
        var $125=$ph_out_addr; //@line 3360 "dictionary.cpp"
        var $arrayidx142=(($125)&4294967295); //@line 3360 "dictionary.cpp"
        HEAP8[($arrayidx142)]=0; //@line 3360 "dictionary.cpp"
        $retval=0; //@line 3361 "dictionary.cpp"
        ; //@line 3361 "dictionary.cpp"
      }
    }
  
    var $126=$retval; //@line 3362 "dictionary.cpp"
    STACKTOP = __stackBase__;
    return $126; //@line 3362 "dictionary.cpp"
    return null;
  }
  

  function __ZL11LookupDict2P10TranslatorPKcS2_PcPjiP8WORD_TAB($tr, $word, $word2, $phonetic, $flags, $end_flags, $wtab) {
    var __stackBase__  = STACKTOP; STACKTOP += 320; _memset(__stackBase__, 0, 320);
    var __label__;
  
    var $retval;
    var $tr_addr;
    var $word_addr;
    var $word2_addr;
    var $phonetic_addr;
    var $flags_addr;
    var $end_flags_addr;
    var $wtab_addr;
    var $p;
    var $next;
    var $hash;
    var $phoneme_len;
    var $wlen;
    var $flag;
    var $dictionary_flags;
    var $dictionary_flags2;
    var $condition_failed;
    var $n_chars;
    var $no_phonemes;
    var $skipwords;
    var $ix;
    var $word_end;
    var $word1;
    var $wflags;
    var $word_buf=__stackBase__;
    var $flags1259;
    var $ph_decoded=__stackBase__+160;
    var $textmode;
    $tr_addr=$tr;
    $word_addr=$word;
    $word2_addr=$word2;
    $phonetic_addr=$phonetic;
    $flags_addr=$flags;
    $end_flags_addr=$end_flags;
    $wtab_addr=$wtab;
    $condition_failed=0; //@line 2887 "dictionary.cpp"
    $wflags=0; //@line 2894 "dictionary.cpp"
    var $0=$wtab_addr; //@line 2897 "dictionary.cpp"
    var $cmp=((($0))|0)!=0; //@line 2897 "dictionary.cpp"
    if ($cmp) { __label__ = 1;; } else { __label__ = 2;; } //@line 2897 "dictionary.cpp"
    if (__label__ == 1) {
  
      var $1=$wtab_addr; //@line 2899 "dictionary.cpp"
      var $flags1=(($1)&4294967295); //@line 2899 "dictionary.cpp"
      var $2=HEAP32[(($flags1)>>2)]; //@line 2899 "dictionary.cpp"
      $wflags=$2; //@line 2899 "dictionary.cpp"
      ; //@line 2900 "dictionary.cpp"
    }
  
    var $3=$word_addr; //@line 2902 "dictionary.cpp"
    $word1=$3; //@line 2902 "dictionary.cpp"
    var $4=$tr_addr; //@line 2903 "dictionary.cpp"
    var $transpose_min=(($4+288)&4294967295); //@line 2903 "dictionary.cpp"
    var $5=HEAP32[(($transpose_min)>>2)]; //@line 2903 "dictionary.cpp"
    var $cmp2=((($5))|0) > 0; //@line 2903 "dictionary.cpp"
    if ($cmp2) { __label__ = 3;; } else { __label__ = 4;; } //@line 2903 "dictionary.cpp"
    if (__label__ == 3) {
  
      var $arraydecay=(($word_buf)&4294967295); //@line 2905 "dictionary.cpp"
      var $6=$word_addr; //@line 2905 "dictionary.cpp"
      var $call=_strcpy($arraydecay, $6); //@line 2905 "dictionary.cpp"
      var $7=$tr_addr; //@line 2906 "dictionary.cpp"
      var $arraydecay4=(($word_buf)&4294967295); //@line 2906 "dictionary.cpp"
      var $call5=__Z17TransposeAlphabetP10TranslatorPc($7, $arraydecay4); //@line 2906 "dictionary.cpp"
      $wlen=$call5; //@line 2906 "dictionary.cpp"
      var $arraydecay6=(($word_buf)&4294967295); //@line 2907 "dictionary.cpp"
      $word_addr=$arraydecay6; //@line 2907 "dictionary.cpp"
      ; //@line 2908 "dictionary.cpp"
    }
    else if (__label__ == 4) {
  
      var $8=$word_addr; //@line 2911 "dictionary.cpp"
      var $call7=_strlen($8); //@line 2911 "dictionary.cpp"
      $wlen=$call7; //@line 2911 "dictionary.cpp"
      ;
    }
  
    var $9=$word_addr; //@line 2914 "dictionary.cpp"
    var $call9=__Z14HashDictionaryPKc($9); //@line 2914 "dictionary.cpp"
    $hash=$call9; //@line 2914 "dictionary.cpp"
    var $10=$hash; //@line 2915 "dictionary.cpp"
    var $11=$tr_addr; //@line 2915 "dictionary.cpp"
    var $dict_hashtab=(($11+1056)&4294967295); //@line 2915 "dictionary.cpp"
    var $arrayidx=(($dict_hashtab+$10*4)&4294967295); //@line 2915 "dictionary.cpp"
    var $12=HEAP32[(($arrayidx)>>2)]; //@line 2915 "dictionary.cpp"
    $p=$12; //@line 2915 "dictionary.cpp"
    var $cmp10=((($12))|0)==0; //@line 2917 "dictionary.cpp"
    if ($cmp10) { __label__ = 6;; } else { __label__ = 9;; } //@line 2917 "dictionary.cpp"
    if (__label__ == 6) {
  
      var $13=$flags_addr; //@line 2919 "dictionary.cpp"
      var $cmp12=((($13))|0)!=0; //@line 2919 "dictionary.cpp"
      if ($cmp12) { __label__ = 7;; } else { __label__ = 8;; } //@line 2919 "dictionary.cpp"
      if (__label__ == 7) {
  
        var $14=$flags_addr; //@line 2920 "dictionary.cpp"
        HEAP32[(($14)>>2)]=0; //@line 2920 "dictionary.cpp"
        ; //@line 2920 "dictionary.cpp"
      }
  
      $retval=0; //@line 2921 "dictionary.cpp"
      ; //@line 2921 "dictionary.cpp"
    }
    else if (__label__ == 9) {
      $while_cond$14: while(1) { 
  
        var $15=$p; //@line 2927 "dictionary.cpp"
        var $16=HEAP8[($15)]; //@line 2927 "dictionary.cpp"
        var $conv=(tempInt=(($16)),(tempInt>=128?tempInt-256:tempInt)); //@line 2927 "dictionary.cpp"
        var $cmp16=((($conv))|0)!=0; //@line 2927 "dictionary.cpp"
        if (!($cmp16)) { __label__ = 95;break $while_cond$14; } //@line 2927 "dictionary.cpp"
  
        var $17=$p; //@line 2929 "dictionary.cpp"
        var $18=$p; //@line 2929 "dictionary.cpp"
        var $arrayidx17=(($18)&4294967295); //@line 2929 "dictionary.cpp"
        var $19=HEAP8[($arrayidx17)]; //@line 2929 "dictionary.cpp"
        var $conv18=(tempInt=(($19)),(tempInt>=128?tempInt-256:tempInt)); //@line 2929 "dictionary.cpp"
        var $add_ptr=(($17+$conv18)&4294967295); //@line 2929 "dictionary.cpp"
        $next=$add_ptr; //@line 2929 "dictionary.cpp"
        var $20=$p; //@line 2931 "dictionary.cpp"
        var $arrayidx19=(($20+1)&4294967295); //@line 2931 "dictionary.cpp"
        var $21=HEAP8[($arrayidx19)]; //@line 2931 "dictionary.cpp"
        var $conv20=(tempInt=(($21)),(tempInt>=128?tempInt-256:tempInt)); //@line 2931 "dictionary.cpp"
        var $and=($conv20) & 127; //@line 2931 "dictionary.cpp"
        var $22=$wlen; //@line 2931 "dictionary.cpp"
        var $cmp21=((($and))|0)!=((($22))|0); //@line 2931 "dictionary.cpp"
        if ($cmp21) { __label__ = 12;; } else { __label__ = 11;; } //@line 2931 "dictionary.cpp"
        $if_then26$$lor_lhs_false$17: do { 
          if (__label__ == 11) {
  
            var $23=$word_addr; //@line 2931 "dictionary.cpp"
            var $24=$p; //@line 2931 "dictionary.cpp"
            var $arrayidx22=(($24+2)&4294967295); //@line 2931 "dictionary.cpp"
            var $25=$wlen; //@line 2931 "dictionary.cpp"
            var $and23=($25) & 63; //@line 2931 "dictionary.cpp"
            var $call24=_memcmp($23, $arrayidx22, $and23); //@line 2931 "dictionary.cpp"
            var $cmp25=((($call24))|0)!=0; //@line 2931 "dictionary.cpp"
            if ($cmp25) { __label__ = 12;break $if_then26$$lor_lhs_false$17; } //@line 2931 "dictionary.cpp"
  
            var $27=$word2_addr; //@line 2939 "dictionary.cpp"
            $word_end=$27; //@line 2939 "dictionary.cpp"
            $dictionary_flags=0; //@line 2941 "dictionary.cpp"
            $dictionary_flags2=0; //@line 2942 "dictionary.cpp"
            var $28=$p; //@line 2943 "dictionary.cpp"
            var $arrayidx28=(($28+1)&4294967295); //@line 2943 "dictionary.cpp"
            var $29=HEAP8[($arrayidx28)]; //@line 2943 "dictionary.cpp"
            var $conv29=(tempInt=(($29)),(tempInt>=128?tempInt-256:tempInt)); //@line 2943 "dictionary.cpp"
            var $and30=($conv29) & 128; //@line 2943 "dictionary.cpp"
            $no_phonemes=$and30; //@line 2943 "dictionary.cpp"
            var $30=$p; //@line 2945 "dictionary.cpp"
            var $arrayidx31=(($30+1)&4294967295); //@line 2945 "dictionary.cpp"
            var $31=HEAP8[($arrayidx31)]; //@line 2945 "dictionary.cpp"
            var $conv32=(tempInt=(($31)),(tempInt>=128?tempInt-256:tempInt)); //@line 2945 "dictionary.cpp"
            var $and33=($conv32) & 63; //@line 2945 "dictionary.cpp"
            var $add=((($and33)+2)&4294967295); //@line 2945 "dictionary.cpp"
            var $32=$p; //@line 2945 "dictionary.cpp"
            var $add_ptr34=(($32+$add)&4294967295); //@line 2945 "dictionary.cpp"
            $p=$add_ptr34; //@line 2945 "dictionary.cpp"
            var $33=$no_phonemes; //@line 2947 "dictionary.cpp"
            var $tobool=((($33))|0)!=0; //@line 2947 "dictionary.cpp"
            var $34=$phonetic_addr; //@line 2949 "dictionary.cpp"
            if ($tobool) { __label__ = 14;; } else { __label__ = 15;; } //@line 2947 "dictionary.cpp"
            if (__label__ == 14) {
  
              var $arrayidx36=(($34)&4294967295); //@line 2949 "dictionary.cpp"
              HEAP8[($arrayidx36)]=0; //@line 2949 "dictionary.cpp"
              $phoneme_len=0; //@line 2950 "dictionary.cpp"
              ; //@line 2951 "dictionary.cpp"
            }
            else if (__label__ == 15) {
  
              var $35=$p; //@line 2954 "dictionary.cpp"
              var $call38=_strcpy($34, $35); //@line 2954 "dictionary.cpp"
              var $36=$p; //@line 2955 "dictionary.cpp"
              var $call39=_strlen($36); //@line 2955 "dictionary.cpp"
              $phoneme_len=$call39; //@line 2955 "dictionary.cpp"
              var $37=$phoneme_len; //@line 2956 "dictionary.cpp"
              var $add40=((($37)+1)&4294967295); //@line 2956 "dictionary.cpp"
              var $38=$p; //@line 2956 "dictionary.cpp"
              var $add_ptr41=(($38+$add40)&4294967295); //@line 2956 "dictionary.cpp"
              $p=$add_ptr41; //@line 2956 "dictionary.cpp"
              ;
            }
            $while_cond43$23: while(1) { 
  
              var $39=$p; //@line 2959 "dictionary.cpp"
              var $40=$next; //@line 2959 "dictionary.cpp"
              var $cmp44=((($39))>>>0) < ((($40))>>>0); //@line 2959 "dictionary.cpp"
              if (!($cmp44)) { __label__ = 39;break $while_cond43$23; } //@line 2959 "dictionary.cpp"
  
              var $41=$p; //@line 2963 "dictionary.cpp"
              var $incdec_ptr=(($41+1)&4294967295); //@line 2963 "dictionary.cpp"
              $p=$incdec_ptr; //@line 2963 "dictionary.cpp"
              var $42=HEAP8[($41)]; //@line 2963 "dictionary.cpp"
              $flag=$42; //@line 2963 "dictionary.cpp"
              var $43=$flag; //@line 2964 "dictionary.cpp"
              var $conv46=((($43))&255); //@line 2964 "dictionary.cpp"
              var $cmp47=((($conv46))|0) >= 100; //@line 2964 "dictionary.cpp"
              var $44=$flag; //@line 2967 "dictionary.cpp"
              var $conv49=((($44))&255); //@line 2967 "dictionary.cpp"
              if ($cmp47) { __label__ = 18;; } else { __label__ = 23;; } //@line 2964 "dictionary.cpp"
              if (__label__ == 18) {
  
                var $cmp50=((($conv49))|0) >= 132; //@line 2967 "dictionary.cpp"
                var $45=$tr_addr; //@line 2970 "dictionary.cpp"
                var $dict_condition=(($45+692)&4294967295); //@line 2970 "dictionary.cpp"
                var $46=HEAPU32[(($dict_condition)>>2)]; //@line 2970 "dictionary.cpp"
                var $47=$flag; //@line 2970 "dictionary.cpp"
                var $conv52=((($47))&255); //@line 2970 "dictionary.cpp"
                if ($cmp50) { __label__ = 19;; } else { __label__ = 21;; } //@line 2967 "dictionary.cpp"
                if (__label__ == 19) {
  
                  var $sub=((($conv52)-132)&4294967295); //@line 2970 "dictionary.cpp"
                  var $shl=1 << ($sub); //@line 2970 "dictionary.cpp"
                  var $and53=($shl) & ($46); //@line 2970 "dictionary.cpp"
                  var $cmp54=((($and53))|0)!=0; //@line 2970 "dictionary.cpp"
                  if (!($cmp54)) { __label__ = 16;continue $while_cond43$23; } //@line 2970 "dictionary.cpp"
  
                  $condition_failed=1; //@line 2971 "dictionary.cpp"
                  __label__ = 16;continue $while_cond43$23; //@line 2971 "dictionary.cpp"
                }
                else if (__label__ == 21) {
  
                  var $sub60=((($conv52)-100)&4294967295); //@line 2976 "dictionary.cpp"
                  var $shl61=1 << ($sub60); //@line 2976 "dictionary.cpp"
                  var $and62=($shl61) & ($46); //@line 2976 "dictionary.cpp"
                  var $cmp63=((($and62))|0)==0; //@line 2976 "dictionary.cpp"
                  if (!($cmp63)) { __label__ = 16;continue $while_cond43$23; } //@line 2976 "dictionary.cpp"
  
                  $condition_failed=1; //@line 2977 "dictionary.cpp"
                  __label__ = 16;continue $while_cond43$23; //@line 2977 "dictionary.cpp"
                }
              }
              else if (__label__ == 23) {
  
                var $cmp69=((($conv49))|0) > 80; //@line 2981 "dictionary.cpp"
                if ($cmp69) { __label__ = 24;; } else { __label__ = 33;; } //@line 2981 "dictionary.cpp"
                if (__label__ == 24) {
  
                  var $48=$next; //@line 2985 "dictionary.cpp"
                  var $49=$p; //@line 2985 "dictionary.cpp"
                  var $sub_ptr_lhs_cast=($48); //@line 2985 "dictionary.cpp"
                  var $sub_ptr_rhs_cast=($49); //@line 2985 "dictionary.cpp"
                  var $sub_ptr_sub=((($sub_ptr_lhs_cast)-($sub_ptr_rhs_cast))&4294967295); //@line 2985 "dictionary.cpp"
                  $n_chars=$sub_ptr_sub; //@line 2985 "dictionary.cpp"
                  var $50=$flag; //@line 2986 "dictionary.cpp"
                  var $conv71=((($50))&255); //@line 2986 "dictionary.cpp"
                  var $sub72=((($conv71)-80)&4294967295); //@line 2986 "dictionary.cpp"
                  $skipwords=$sub72; //@line 2986 "dictionary.cpp"
                  $ix=0; //@line 2989 "dictionary.cpp"
                  var $51=$ix; //@line 2989 "dictionary.cpp"
                  var $52=$skipwords; //@line 2989 "dictionary.cpp"
                  var $cmp732=((($51))|0) <= ((($52))|0); //@line 2989 "dictionary.cpp"
                  if ($cmp732) { __label__ = 25;; } else { __label__ = 28;; } //@line 2989 "dictionary.cpp"
                  $for_body$$for_end$36: do { 
                    if (__label__ == 25) {
                      while(1) { 
  
                        var $53=$wflags; //@line 2991 "dictionary.cpp"
                        var $and74=($53) & 3072; //@line 2991 "dictionary.cpp"
                        var $tobool75=((($and74))|0)!=0; //@line 2991 "dictionary.cpp"
                        if ($tobool75) { __label__ = 26;; } else { __label__ = 27;; } //@line 2991 "dictionary.cpp"
                        if (__label__ == 26) {
  
                          $condition_failed=1; //@line 2993 "dictionary.cpp"
                          ; //@line 2994 "dictionary.cpp"
                        }
  
                        var $54=$ix; //@line 2989 "dictionary.cpp"
                        var $inc=((($54)+1)&4294967295); //@line 2989 "dictionary.cpp"
                        $ix=$inc; //@line 2989 "dictionary.cpp"
                        var $55=$ix; //@line 2989 "dictionary.cpp"
                        var $56=$skipwords; //@line 2989 "dictionary.cpp"
                        var $cmp73=((($55))|0) <= ((($56))|0); //@line 2989 "dictionary.cpp"
                        if ($cmp73) { __label__ = 25;continue ; } else { __label__ = 28;break $for_body$$for_end$36; } //@line 2989 "dictionary.cpp"
                      }
                    }
                  } while(0);
  
                  var $57=$word2_addr; //@line 2997 "dictionary.cpp"
                  var $58=$p; //@line 2997 "dictionary.cpp"
                  var $59=$n_chars; //@line 2997 "dictionary.cpp"
                  var $call78=_memcmp($57, $58, $59); //@line 2997 "dictionary.cpp"
                  var $cmp79=((($call78))|0)!=0; //@line 2997 "dictionary.cpp"
                  if ($cmp79) { __label__ = 29;break $while_cond43$23; } //@line 2997 "dictionary.cpp"
  
                  var $_pr=$condition_failed; //@line 3000 "dictionary.cpp"
                  var $tobool82=((($_pr))|0)!=0; //@line 3000 "dictionary.cpp"
                  if ($tobool82) { __label__ = 31;break $while_cond43$23; } //@line 3000 "dictionary.cpp"
  
                  var $61=$dictionary_flags; //@line 3006 "dictionary.cpp"
                  var $or=($61) | 128; //@line 3006 "dictionary.cpp"
                  $dictionary_flags=$or; //@line 3006 "dictionary.cpp"
                  var $62=$skipwords; //@line 3007 "dictionary.cpp"
                  HEAP32[((_dictionary_skipwords)>>2)]=$62; //@line 3007 "dictionary.cpp"
                  var $63=$next; //@line 3008 "dictionary.cpp"
                  $p=$63; //@line 3008 "dictionary.cpp"
                  var $64=$word2_addr; //@line 3009 "dictionary.cpp"
                  var $65=$n_chars; //@line 3009 "dictionary.cpp"
                  var $add_ptr85=(($64+$65)&4294967295); //@line 3009 "dictionary.cpp"
                  $word_end=$add_ptr85; //@line 3009 "dictionary.cpp"
                  __label__ = 16;continue $while_cond43$23; //@line 3010 "dictionary.cpp"
                }
                else if (__label__ == 33) {
  
                  var $66=$flag; //@line 3012 "dictionary.cpp"
                  var $conv87=((($66))&255); //@line 3012 "dictionary.cpp"
                  var $cmp88=((($conv87))|0) > 64; //@line 3012 "dictionary.cpp"
                  if ($cmp88) { __label__ = 34;; } else { __label__ = 36;; } //@line 3012 "dictionary.cpp"
                  if (__label__ == 34) {
  
                    var $67=$dictionary_flags; //@line 3015 "dictionary.cpp"
                    var $and90=($67) & -16; //@line 3015 "dictionary.cpp"
                    var $68=$flag; //@line 3015 "dictionary.cpp"
                    var $conv91=((($68))&255); //@line 3015 "dictionary.cpp"
                    var $and92=($conv91) & 15; //@line 3015 "dictionary.cpp"
                    var $or93=($and92) | ($and90); //@line 3015 "dictionary.cpp"
                    $dictionary_flags=$or93; //@line 3015 "dictionary.cpp"
                    var $69=$flag; //@line 3016 "dictionary.cpp"
                    var $conv94=((($69))&255); //@line 3016 "dictionary.cpp"
                    var $and95=($conv94) & 12; //@line 3016 "dictionary.cpp"
                    var $cmp96=((($and95))|0)==12; //@line 3016 "dictionary.cpp"
                    if (!($cmp96)) { __label__ = 16;continue $while_cond43$23; } //@line 3016 "dictionary.cpp"
  
                    var $70=$dictionary_flags; //@line 3017 "dictionary.cpp"
                    var $or98=($70) | 2048; //@line 3017 "dictionary.cpp"
                    $dictionary_flags=$or98; //@line 3017 "dictionary.cpp"
                    __label__ = 16;continue $while_cond43$23; //@line 3017 "dictionary.cpp"
                  }
                  else if (__label__ == 36) {
  
                    var $71=$flag; //@line 3020 "dictionary.cpp"
                    var $conv101=((($71))&255); //@line 3020 "dictionary.cpp"
                    var $cmp102=((($conv101))|0) >= 32; //@line 3020 "dictionary.cpp"
                    var $72=$flag; //@line 3022 "dictionary.cpp"
                    var $conv104=((($72))&255); //@line 3022 "dictionary.cpp"
                    if ($cmp102) { __label__ = 37;; } else { __label__ = 38;; } //@line 3020 "dictionary.cpp"
                    if (__label__ == 37) {
  
                      var $sub105=((($conv104)-32)&4294967295); //@line 3022 "dictionary.cpp"
                      var $shl106=1 << ($sub105); //@line 3022 "dictionary.cpp"
                      var $73=$dictionary_flags2; //@line 3022 "dictionary.cpp"
                      var $or107=($73) | ($shl106); //@line 3022 "dictionary.cpp"
                      $dictionary_flags2=$or107; //@line 3022 "dictionary.cpp"
                      __label__ = 16;continue $while_cond43$23; //@line 3023 "dictionary.cpp"
                    }
                    else if (__label__ == 38) {
  
                      var $shl110=1 << ($conv104); //@line 3026 "dictionary.cpp"
                      var $74=$dictionary_flags; //@line 3026 "dictionary.cpp"
                      var $or111=($74) | ($shl110); //@line 3026 "dictionary.cpp"
                      $dictionary_flags=$or111; //@line 3026 "dictionary.cpp"
                      __label__ = 16;continue $while_cond43$23;
                    }
                  }
                }
              }
            }
            $while_end$$if_end81_thread$$if_then83$53: do { 
              if (__label__ == 29) {
  
                $condition_failed=1; //@line 2998 "dictionary.cpp"
                __label__ = 31;break $while_end$$if_end81_thread$$if_then83$53; //@line 3000 "dictionary.cpp"
              }
            } while(0);
            if (__label__ == 31) {
  
              var $60=$next; //@line 3002 "dictionary.cpp"
              $p=$60; //@line 3002 "dictionary.cpp"
              ; //@line 3003 "dictionary.cpp"
            }
  
            var $75=$condition_failed; //@line 3030 "dictionary.cpp"
            var $tobool116=((($75))|0)!=0; //@line 3030 "dictionary.cpp"
            if ($tobool116) { __label__ = 40;; } else { __label__ = 41;; } //@line 3030 "dictionary.cpp"
            if (__label__ == 40) {
  
              $condition_failed=0; //@line 3032 "dictionary.cpp"
              __label__ = 9;continue $while_cond$14; //@line 3033 "dictionary.cpp"
            }
            else if (__label__ == 41) {
  
              var $76=$end_flags_addr; //@line 3036 "dictionary.cpp"
              var $and119=($76) & 4; //@line 3036 "dictionary.cpp"
              var $cmp120=((($and119))|0)==0; //@line 3036 "dictionary.cpp"
              if ($cmp120) { __label__ = 42;; } else { __label__ = 43;; } //@line 3036 "dictionary.cpp"
              if (__label__ == 42) {
  
                var $77=$dictionary_flags; //@line 3039 "dictionary.cpp"
                var $and122=($77) & 262144; //@line 3039 "dictionary.cpp"
                var $tobool123=((($and122))|0)!=0; //@line 3039 "dictionary.cpp"
                if ($tobool123) { __label__ = 9;continue $while_cond$14; } //@line 3039 "dictionary.cpp"
              }
  
              var $78=$end_flags_addr; //@line 3043 "dictionary.cpp"
              var $and127=($78) & 1024; //@line 3043 "dictionary.cpp"
              var $tobool128=((($and127))|0)!=0; //@line 3043 "dictionary.cpp"
              if ($tobool128) { __label__ = 44;; } else { __label__ = 45;; } //@line 3043 "dictionary.cpp"
              if (__label__ == 44) {
  
                var $79=$dictionary_flags; //@line 3043 "dictionary.cpp"
                var $and129=($79) & 1536; //@line 3043 "dictionary.cpp"
                var $tobool130=((($and129))|0)!=0; //@line 3043 "dictionary.cpp"
                if ($tobool130) { __label__ = 9;continue $while_cond$14; } //@line 3043 "dictionary.cpp"
              }
  
              var $80=$end_flags_addr; //@line 3046 "dictionary.cpp"
              var $and133=($80) & 4; //@line 3046 "dictionary.cpp"
              var $tobool134=((($and133))|0)!=0; //@line 3046 "dictionary.cpp"
              if ($tobool134) { __label__ = 46;; } else { __label__ = 49;; } //@line 3046 "dictionary.cpp"
              $if_then135$$if_end147$67: do { 
                if (__label__ == 46) {
  
                  var $81=$dictionary_flags; //@line 3049 "dictionary.cpp"
                  var $and136=($81) & 512; //@line 3049 "dictionary.cpp"
                  var $tobool137=((($and136))|0)!=0; //@line 3049 "dictionary.cpp"
                  if ($tobool137) { __label__ = 9;continue $while_cond$14; } //@line 3049 "dictionary.cpp"
  
                  var $82=$dictionary_flags; //@line 3052 "dictionary.cpp"
                  var $and140=($82) & 1024; //@line 3052 "dictionary.cpp"
                  var $tobool141=((($and140))|0)!=0; //@line 3052 "dictionary.cpp"
                  if (!($tobool141)) { __label__ = 49;break $if_then135$$if_end147$67; } //@line 3052 "dictionary.cpp"
  
                  var $83=$end_flags_addr; //@line 3052 "dictionary.cpp"
                  var $and143=($83) & 8; //@line 3052 "dictionary.cpp"
                  var $cmp144=((($and143))|0)==0; //@line 3052 "dictionary.cpp"
                  if ($cmp144) { __label__ = 9;continue $while_cond$14; } //@line 3052 "dictionary.cpp"
                }
              } while(0);
  
              var $84=$dictionary_flags2; //@line 3059 "dictionary.cpp"
              var $and148=($84) & 4096; //@line 3059 "dictionary.cpp"
              var $tobool149=((($and148))|0)!=0; //@line 3059 "dictionary.cpp"
              if ($tobool149) { __label__ = 50;; } else { __label__ = 51;; } //@line 3059 "dictionary.cpp"
              if (__label__ == 50) {
  
                var $85=$wflags; //@line 3061 "dictionary.cpp"
                var $and151=($85) & 16384; //@line 3061 "dictionary.cpp"
                var $tobool152=((($and151))|0)!=0; //@line 3061 "dictionary.cpp"
                if (!($tobool152)) { __label__ = 9;continue $while_cond$14; } //@line 3061 "dictionary.cpp"
              }
  
              var $86=$dictionary_flags2; //@line 3066 "dictionary.cpp"
              var $and156=($86) & 512; //@line 3066 "dictionary.cpp"
              var $tobool157=((($and156))|0)!=0; //@line 3066 "dictionary.cpp"
              if ($tobool157) { __label__ = 52;; } else { __label__ = 53;; } //@line 3066 "dictionary.cpp"
              if (__label__ == 52) {
  
                var $87=$wflags; //@line 3068 "dictionary.cpp"
                var $and159=($87) & 2; //@line 3068 "dictionary.cpp"
                var $tobool160=((($and159))|0)!=0; //@line 3068 "dictionary.cpp"
                if (!($tobool160)) { __label__ = 9;continue $while_cond$14; } //@line 3068 "dictionary.cpp"
              }
  
              var $88=$dictionary_flags2; //@line 3073 "dictionary.cpp"
              var $and164=($88) & 1024; //@line 3073 "dictionary.cpp"
              var $tobool165=((($and164))|0)!=0; //@line 3073 "dictionary.cpp"
              if ($tobool165) { __label__ = 54;; } else { __label__ = 55;; } //@line 3073 "dictionary.cpp"
              if (__label__ == 54) {
  
                var $89=$wflags; //@line 3075 "dictionary.cpp"
                var $and167=($89) & 1; //@line 3075 "dictionary.cpp"
                var $tobool168=((($and167))|0)!=0; //@line 3075 "dictionary.cpp"
                if (!($tobool168)) { __label__ = 9;continue $while_cond$14; } //@line 3075 "dictionary.cpp"
              }
  
              var $90=$dictionary_flags; //@line 3080 "dictionary.cpp"
              var $and172=($90) & 33554432; //@line 3080 "dictionary.cpp"
              var $tobool173=((($and172))|0)!=0; //@line 3080 "dictionary.cpp"
              if ($tobool173) { __label__ = 56;; } else { __label__ = 57;; } //@line 3080 "dictionary.cpp"
              if (__label__ == 56) {
  
                var $91=$wflags; //@line 3082 "dictionary.cpp"
                var $and175=($91) & 65536; //@line 3082 "dictionary.cpp"
                var $tobool176=((($and175))|0)!=0; //@line 3082 "dictionary.cpp"
                if (!($tobool176)) { __label__ = 9;continue $while_cond$14; } //@line 3082 "dictionary.cpp"
              }
  
              var $92=$dictionary_flags; //@line 3086 "dictionary.cpp"
              var $and180=($92) & 16384; //@line 3086 "dictionary.cpp"
              var $tobool181=((($and180))|0)!=0; //@line 3086 "dictionary.cpp"
              if ($tobool181) { __label__ = 58;; } else { __label__ = 59;; } //@line 3086 "dictionary.cpp"
              if (__label__ == 58) {
  
                var $93=$word_end; //@line 3086 "dictionary.cpp"
                var $94=$tr_addr; //@line 3086 "dictionary.cpp"
                var $clause_end=(($94+8292)&4294967295); //@line 3086 "dictionary.cpp"
                var $95=HEAPU32[(($clause_end)>>2)]; //@line 3086 "dictionary.cpp"
                var $cmp183=((($93))>>>0) < ((($95))>>>0); //@line 3086 "dictionary.cpp"
                if ($cmp183) { __label__ = 9;continue $while_cond$14; } //@line 3086 "dictionary.cpp"
              }
  
              var $96=$dictionary_flags; //@line 3092 "dictionary.cpp"
              var $and186=($96) & 32768; //@line 3092 "dictionary.cpp"
              var $tobool187=((($and186))|0)!=0; //@line 3092 "dictionary.cpp"
              if ($tobool187) { __label__ = 60;; } else { __label__ = 61;; } //@line 3092 "dictionary.cpp"
              if (__label__ == 60) {
  
                var $97=$wtab_addr; //@line 3092 "dictionary.cpp"
                var $flags189=(($97)&4294967295); //@line 3092 "dictionary.cpp"
                var $98=HEAP32[(($flags189)>>2)]; //@line 3092 "dictionary.cpp"
                var $and190=($98) & 512; //@line 3092 "dictionary.cpp"
                var $tobool191=((($and190))|0)!=0; //@line 3092 "dictionary.cpp"
                if (!($tobool191)) { __label__ = 9;continue $while_cond$14; } //@line 3092 "dictionary.cpp"
              }
  
              var $99=$dictionary_flags2; //@line 3098 "dictionary.cpp"
              var $and194=($99) & 8192; //@line 3098 "dictionary.cpp"
              var $tobool195=((($and194))|0)!=0; //@line 3098 "dictionary.cpp"
              if ($tobool195) { __label__ = 62;; } else { __label__ = 63;; } //@line 3098 "dictionary.cpp"
              if (__label__ == 62) {
  
                var $100=$tr_addr; //@line 3098 "dictionary.cpp"
                var $clause_terminator=(($100+8324)&4294967295); //@line 3098 "dictionary.cpp"
                var $101=HEAP32[(($clause_terminator)>>2)]; //@line 3098 "dictionary.cpp"
                var $and197=($101) & 524288; //@line 3098 "dictionary.cpp"
                var $tobool198=((($and197))|0)!=0; //@line 3098 "dictionary.cpp"
                if (!($tobool198)) { __label__ = 9;continue $while_cond$14; } //@line 3098 "dictionary.cpp"
              }
  
              var $102=$dictionary_flags2; //@line 3104 "dictionary.cpp"
              var $and201=($102) & 16; //@line 3104 "dictionary.cpp"
              var $tobool202=((($and201))|0)!=0; //@line 3104 "dictionary.cpp"
              if ($tobool202) { __label__ = 64;; } else { __label__ = 67;; } //@line 3104 "dictionary.cpp"
              $if_then203$$if_end213$93: do { 
                if (__label__ == 64) {
  
                  var $103=$tr_addr; //@line 3108 "dictionary.cpp"
                  var $expect_verb=(($103+8272)&4294967295); //@line 3108 "dictionary.cpp"
                  var $104=HEAP32[(($expect_verb)>>2)]; //@line 3108 "dictionary.cpp"
                  var $tobool204=((($104))|0)!=0; //@line 3108 "dictionary.cpp"
                  if ($tobool204) { __label__ = 67;break $if_then203$$if_end213$93; } //@line 3108 "dictionary.cpp"
  
                  var $105=$tr_addr; //@line 3108 "dictionary.cpp"
                  var $expect_verb_s=(($105+8280)&4294967295); //@line 3108 "dictionary.cpp"
                  var $106=HEAP32[(($expect_verb_s)>>2)]; //@line 3108 "dictionary.cpp"
                  var $tobool206=((($106))|0)!=0; //@line 3108 "dictionary.cpp"
                  if (!($tobool206)) { __label__ = 9;continue $while_cond$14; } //@line 3108 "dictionary.cpp"
  
                  var $107=$end_flags_addr; //@line 3108 "dictionary.cpp"
                  var $and208=($107) & 8; //@line 3108 "dictionary.cpp"
                  var $tobool209=((($and208))|0)!=0; //@line 3108 "dictionary.cpp"
                  if (!($tobool209)) { __label__ = 9;continue $while_cond$14; } //@line 3108 "dictionary.cpp"
                }
              } while(0);
  
              var $108=$dictionary_flags2; //@line 3119 "dictionary.cpp"
              var $and214=($108) & 64; //@line 3119 "dictionary.cpp"
              var $tobool215=((($and214))|0)!=0; //@line 3119 "dictionary.cpp"
              if ($tobool215) { __label__ = 68;; } else { __label__ = 69;; } //@line 3119 "dictionary.cpp"
              if (__label__ == 68) {
  
                var $109=$tr_addr; //@line 3121 "dictionary.cpp"
                var $expect_past=(($109+8276)&4294967295); //@line 3121 "dictionary.cpp"
                var $110=HEAP32[(($expect_past)>>2)]; //@line 3121 "dictionary.cpp"
                var $tobool217=((($110))|0)!=0; //@line 3121 "dictionary.cpp"
                if (!($tobool217)) { __label__ = 9;continue $while_cond$14; } //@line 3121 "dictionary.cpp"
              }
  
              var $111=$dictionary_flags2; //@line 3128 "dictionary.cpp"
              var $and221=($111) & 32; //@line 3128 "dictionary.cpp"
              var $tobool222=((($and221))|0)!=0; //@line 3128 "dictionary.cpp"
              if ($tobool222) { __label__ = 70;; } else { __label__ = 71;; } //@line 3128 "dictionary.cpp"
              if (__label__ == 70) {
  
                var $112=$tr_addr; //@line 3130 "dictionary.cpp"
                var $expect_noun=(($112+8284)&4294967295); //@line 3130 "dictionary.cpp"
                var $113=HEAP32[(($expect_noun)>>2)]; //@line 3130 "dictionary.cpp"
                var $tobool224=((($113))|0)!=0; //@line 3130 "dictionary.cpp"
                if (!($tobool224)) { __label__ = 9;continue $while_cond$14; } //@line 3130 "dictionary.cpp"
              }
  
              var $114=$dictionary_flags; //@line 3137 "dictionary.cpp"
              var $and228=($114) & 2097152; //@line 3137 "dictionary.cpp"
              var $tobool229=((($and228))|0)!=0; //@line 3137 "dictionary.cpp"
              if (!($tobool229)) { __label__ = 74;break $while_cond$14; } //@line 3137 "dictionary.cpp"
  
              var $115=$tr_addr; //@line 3140 "dictionary.cpp"
              var $translator_name=(($115+280)&4294967295); //@line 3140 "dictionary.cpp"
              var $116=HEAP32[(($translator_name)>>2)]; //@line 3140 "dictionary.cpp"
              var $cmp231=((($116))|0)==26741; //@line 3140 "dictionary.cpp"
              if (!($cmp231)) { __label__ = 74;break $while_cond$14; } //@line 3140 "dictionary.cpp"
  
              var $117=$tr_addr; //@line 3140 "dictionary.cpp"
              var $prev_dict_flags=(($117+8320)&4294967295); //@line 3140 "dictionary.cpp"
              var $118=HEAP32[(($prev_dict_flags)>>2)]; //@line 3140 "dictionary.cpp"
              var $and233=($118) & 1048576; //@line 3140 "dictionary.cpp"
              var $tobool234=((($and233))|0)!=0; //@line 3140 "dictionary.cpp"
              if ($tobool234) { __label__ = 74;break $while_cond$14; } else { __label__ = 9;continue $while_cond$14; } //@line 3140 "dictionary.cpp"
            }
          }
        } while(0);
  
        var $26=$next; //@line 2934 "dictionary.cpp"
        $p=$26; //@line 2934 "dictionary.cpp"
        __label__ = 9;continue $while_cond$14; //@line 2935 "dictionary.cpp"
      }
      if (__label__ == 95) {
  
        $retval=0; //@line 3198 "dictionary.cpp"
        ; //@line 3198 "dictionary.cpp"
      }
      else if (__label__ == 74) {
  
        var $119=$flags_addr; //@line 3144 "dictionary.cpp"
        var $cmp238=((($119))|0)!=0; //@line 3144 "dictionary.cpp"
        if ($cmp238) { __label__ = 75;; } else { __label__ = 76;; } //@line 3144 "dictionary.cpp"
        if (__label__ == 75) {
  
          var $120=$dictionary_flags; //@line 3146 "dictionary.cpp"
          var $or240=($120) | 1073741824; //@line 3146 "dictionary.cpp"
          var $121=$flags_addr; //@line 3146 "dictionary.cpp"
          var $arrayidx241=(($121)&4294967295); //@line 3146 "dictionary.cpp"
          HEAP32[(($arrayidx241)>>2)]=$or240; //@line 3146 "dictionary.cpp"
          var $122=$dictionary_flags2; //@line 3147 "dictionary.cpp"
          var $123=$flags_addr; //@line 3147 "dictionary.cpp"
          var $arrayidx242=(($123+4)&4294967295); //@line 3147 "dictionary.cpp"
          HEAP32[(($arrayidx242)>>2)]=$122; //@line 3147 "dictionary.cpp"
          ; //@line 3148 "dictionary.cpp"
        }
  
        var $124=$phoneme_len; //@line 3150 "dictionary.cpp"
        var $cmp244=((($124))|0)==0; //@line 3150 "dictionary.cpp"
        if ($cmp244) { __label__ = 77;; } else { __label__ = 80;; } //@line 3150 "dictionary.cpp"
        if (__label__ == 77) {
  
          var $125=HEAP32[((_option_phonemes)>>2)]; //@line 3152 "dictionary.cpp"
          var $cmp246=((($125))|0)==2; //@line 3152 "dictionary.cpp"
          if ($cmp246) { __label__ = 78;; } else { __label__ = 79;; } //@line 3152 "dictionary.cpp"
          if (__label__ == 78) {
  
            var $126=HEAP32[((_f_trans)>>2)]; //@line 3154 "dictionary.cpp"
            var $127=$word1; //@line 3154 "dictionary.cpp"
            var $128=$flags_addr; //@line 3154 "dictionary.cpp"
            var $call248=__Z22print_dictionary_flagsPj($128); //@line 3154 "dictionary.cpp"
            var $call249=_fprintf($126, ((__str28144)&4294967295), allocate([$127,0,0,0,((__ZZ22print_dictionary_flagsPjE3buf)&4294967295),0,0,0], ["i8*",0,0,0,"i8*",0,0,0], ALLOC_STACK)); //@line 3154 "dictionary.cpp"
            ; //@line 3155 "dictionary.cpp"
          }
  
          $retval=0; //@line 3156 "dictionary.cpp"
          ; //@line 3156 "dictionary.cpp"
        }
        else if (__label__ == 80) {
  
          var $129=$flags_addr; //@line 3159 "dictionary.cpp"
          var $cmp252=((($129))|0)!=0; //@line 3159 "dictionary.cpp"
          if ($cmp252) { __label__ = 81;; } else { __label__ = 82;; } //@line 3159 "dictionary.cpp"
          if (__label__ == 81) {
  
            var $130=$flags_addr; //@line 3160 "dictionary.cpp"
            var $arrayidx254=(($130)&4294967295); //@line 3160 "dictionary.cpp"
            var $131=HEAP32[(($arrayidx254)>>2)]; //@line 3160 "dictionary.cpp"
            var $or255=($131) | -2147483648; //@line 3160 "dictionary.cpp"
            HEAP32[(($arrayidx254)>>2)]=$or255; //@line 3160 "dictionary.cpp"
            ; //@line 3160 "dictionary.cpp"
          }
  
          var $132=HEAP32[((_option_phonemes)>>2)]; //@line 3162 "dictionary.cpp"
          var $cmp257=((($132))|0)==2; //@line 3162 "dictionary.cpp"
          if ($cmp257) { __label__ = 83;; } else { __label__ = 94;; } //@line 3162 "dictionary.cpp"
          $if_then258$$if_end296$122: do { 
            if (__label__ == 83) {
  
              $flags1259=0; //@line 3164 "dictionary.cpp"
              var $133=$phonetic_addr; //@line 3168 "dictionary.cpp"
              var $arraydecay260=(($ph_decoded)&4294967295); //@line 3168 "dictionary.cpp"
              __Z14DecodePhonemesPKcPc($133, $arraydecay260); //@line 3168 "dictionary.cpp"
              var $134=$flags_addr; //@line 3169 "dictionary.cpp"
              var $cmp261=((($134))|0)!=0; //@line 3169 "dictionary.cpp"
              if ($cmp261) { __label__ = 84;; } else { __label__ = 85;; } //@line 3169 "dictionary.cpp"
              if (__label__ == 84) {
  
                var $135=$flags_addr; //@line 3170 "dictionary.cpp"
                var $arrayidx263=(($135)&4294967295); //@line 3170 "dictionary.cpp"
                var $136=HEAP32[(($arrayidx263)>>2)]; //@line 3170 "dictionary.cpp"
                $flags1259=$136; //@line 3170 "dictionary.cpp"
                ; //@line 3170 "dictionary.cpp"
              }
  
              var $137=$dictionary_flags; //@line 3172 "dictionary.cpp"
              var $and265=($137) & 536870912; //@line 3172 "dictionary.cpp"
              var $cmp266=((($and265))|0)==0; //@line 3172 "dictionary.cpp"
              if ($cmp266) { __label__ = 86;; } else { __label__ = 87;; } //@line 3172 "dictionary.cpp"
              if (__label__ == 86) {
  
                $textmode=0; //@line 3173 "dictionary.cpp"
                ; //@line 3173 "dictionary.cpp"
              }
              else if (__label__ == 87) {
  
                $textmode=1; //@line 3175 "dictionary.cpp"
                ;
              }
  
              var $138=$textmode; //@line 3177 "dictionary.cpp"
              var $139=HEAP32[((_translator)>>2)]; //@line 3177 "dictionary.cpp"
              var $langopts=(($139)&4294967295); //@line 3177 "dictionary.cpp"
              var $textmode270=(($langopts+252)&4294967295); //@line 3177 "dictionary.cpp"
              var $140=HEAP8[($textmode270)]; //@line 3177 "dictionary.cpp"
              var $conv271=(tempInt=(($140)),(tempInt>=128?tempInt-256:tempInt)); //@line 3177 "dictionary.cpp"
              var $cmp272=((($138))|0)==((($conv271))|0); //@line 3177 "dictionary.cpp"
              if (!($cmp272)) { __label__ = 94;break $if_then258$$if_end296$122; } //@line 3177 "dictionary.cpp"
  
              var $141=HEAP32[((_dictionary_skipwords)>>2)]; //@line 3180 "dictionary.cpp"
              var $tobool274=((($141))|0)!=0; //@line 3180 "dictionary.cpp"
              if ($tobool274) { __label__ = 90;; } else { __label__ = 92;; } //@line 3180 "dictionary.cpp"
              $land_lhs_true275$$if_else289$132: do { 
                if (__label__ == 90) {
  
                  var $142=$wtab_addr; //@line 3180 "dictionary.cpp"
                  var $cmp276=((($142))|0)!=0; //@line 3180 "dictionary.cpp"
                  if (!($cmp276)) { __label__ = 92;break $land_lhs_true275$$if_else289$132; } //@line 3180 "dictionary.cpp"
  
                  var $arraydecay278=(($word_buf)&4294967295); //@line 3184 "dictionary.cpp"
                  var $143=$word2_addr; //@line 3184 "dictionary.cpp"
                  var $144=$word_end; //@line 3184 "dictionary.cpp"
                  var $145=$word2_addr; //@line 3184 "dictionary.cpp"
                  var $sub_ptr_lhs_cast279=($144); //@line 3184 "dictionary.cpp"
                  var $sub_ptr_rhs_cast280=($145); //@line 3184 "dictionary.cpp"
                  var $sub_ptr_sub281=((($sub_ptr_lhs_cast279)-($sub_ptr_rhs_cast280))&4294967295); //@line 3184 "dictionary.cpp"
                  var $src$, $dest$, $stop$, $stop4$, $fast$;
                  $src$ = $143;
                  $dest$ = $arraydecay278;
                  $stop$ = $src$ + $sub_ptr_sub281;
                  if (($dest$%4) == ($src$%4) && $sub_ptr_sub281 > 8) {
                    while ($src$%4 !== 0 && $src$ < $stop$) {
                      ; HEAP8[$dest$++] = HEAP8[$src$++];
                    }
                    $src$ >>= 2;
                    $dest$ >>= 2;
                    $stop4$ = $stop$ >> 2;
                    while ($src$ < $stop4$) {
                  ;;;    HEAP32[$dest$++] = HEAP32[$src$++];
                    }
                    $src$ <<= 2;
                    $dest$ <<= 2;
                  }while ($src$ < $stop$) {
                    ; HEAP8[$dest$++] = HEAP8[$src$++];
                  }; //@line 3184 "dictionary.cpp"
                  var $146=$word_end; //@line 3185 "dictionary.cpp"
                  var $147=$word2_addr; //@line 3185 "dictionary.cpp"
                  var $sub_ptr_lhs_cast282=($146); //@line 3185 "dictionary.cpp"
                  var $sub_ptr_rhs_cast283=($147); //@line 3185 "dictionary.cpp"
                  var $sub_ptr_rhs_cast283_neg=(((-($sub_ptr_rhs_cast283)))&4294967295);
                  var $sub_ptr_sub284=((($sub_ptr_lhs_cast282)-1)&4294967295); //@line 3185 "dictionary.cpp"
                  var $sub285=((($sub_ptr_sub284)+($sub_ptr_rhs_cast283_neg))&4294967295); //@line 3185 "dictionary.cpp"
                  var $arrayidx286=(($word_buf+$sub285)&4294967295); //@line 3185 "dictionary.cpp"
                  HEAP8[($arrayidx286)]=0; //@line 3185 "dictionary.cpp"
                  var $148=HEAP32[((_f_trans)>>2)]; //@line 3186 "dictionary.cpp"
                  var $149=$word1; //@line 3186 "dictionary.cpp"
                  var $arraydecay287=(($word_buf)&4294967295); //@line 3186 "dictionary.cpp"
                  var $call288=_fprintf($148, ((__str29145)&4294967295), allocate([$149,0,0,0,$arraydecay287,0,0,0], ["i8*",0,0,0,"i8*",0,0,0], ALLOC_STACK)); //@line 3186 "dictionary.cpp"
                  __label__ = 93;break $land_lhs_true275$$if_else289$132; //@line 3187 "dictionary.cpp"
                }
              } while(0);
              if (__label__ == 92) {
  
                var $150=HEAP32[((_f_trans)>>2)]; //@line 3190 "dictionary.cpp"
                var $151=$word1; //@line 3190 "dictionary.cpp"
                var $call290=_fprintf($150, ((__str30146)&4294967295), allocate([$151,0,0,0], ["i8*",0,0,0], ALLOC_STACK)); //@line 3190 "dictionary.cpp"
                ;
              }
  
              var $152=HEAP32[((_f_trans)>>2)]; //@line 3192 "dictionary.cpp"
              var $arraydecay292=(($ph_decoded)&4294967295); //@line 3192 "dictionary.cpp"
              var $153=$flags_addr; //@line 3192 "dictionary.cpp"
              var $call293=__Z22print_dictionary_flagsPj($153); //@line 3192 "dictionary.cpp"
              var $call294=_fprintf($152, ((__str31147)&4294967295), allocate([$arraydecay292,0,0,0,((__ZZ22print_dictionary_flagsPjE3buf)&4294967295),0,0,0], ["i8*",0,0,0,"i8*",0,0,0], ALLOC_STACK)); //@line 3192 "dictionary.cpp"
              ; //@line 3193 "dictionary.cpp"
            }
          } while(0);
  
          var $154=$word_end; //@line 3195 "dictionary.cpp"
          $retval=$154; //@line 3195 "dictionary.cpp"
          ; //@line 3195 "dictionary.cpp"
        }
      }
    }
  
    var $155=$retval; //@line 3199 "dictionary.cpp"
    STACKTOP = __stackBase__;
    return $155; //@line 3199 "dictionary.cpp"
    return null;
  }
  

  function __Z11LookupFlagsP10TranslatorPKc($tr, $word) {
    var __stackBase__  = STACKTOP; STACKTOP += 104; _memset(__stackBase__, 0, 104);
    var __label__;
  
    var $tr_addr;
    var $word_addr;
    var $buf=__stackBase__;
    var $word1=__stackBase__+100;
    $tr_addr=$tr;
    $word_addr=$word;
    HEAP32[((((__ZZ11LookupFlagsP10TranslatorPKcE5flags+4)&4294967295))>>2)]=0; //@line 3379 "dictionary.cpp"
    HEAP32[((((__ZZ11LookupFlagsP10TranslatorPKcE5flags)&4294967295))>>2)]=0; //@line 3379 "dictionary.cpp"
    var $0=$word_addr; //@line 3380 "dictionary.cpp"
    HEAP32[(($word1)>>2)]=$0; //@line 3380 "dictionary.cpp"
    var $1=$tr_addr; //@line 3381 "dictionary.cpp"
    var $arraydecay=(($buf)&4294967295); //@line 3381 "dictionary.cpp"
    var $call=__Z14LookupDictListP10TranslatorPPcS1_PjiP8WORD_TAB($1, $word1, $arraydecay, ((__ZZ11LookupFlagsP10TranslatorPKcE5flags)&4294967295), 0, 0); //@line 3381 "dictionary.cpp"
    var $2=HEAP32[((((__ZZ11LookupFlagsP10TranslatorPKcE5flags)&4294967295))>>2)]; //@line 3382 "dictionary.cpp"
    STACKTOP = __stackBase__;
    return $2; //@line 3382 "dictionary.cpp"
    return null;
  }
  

  function __Z12RemoveEndingP10TranslatorPciS1_($tr, $word, $end_type, $word_copy) {
    ;
    var __label__;
    var __lastLabel__ = null;
  
    var $tr_addr;
    var $word_addr;
    var $end_type_addr;
    var $word_copy_addr;
    var $i;
    var $word_end;
    var $len_ending;
    var $end_flags;
    var $p;
    var $len;
    $tr_addr=$tr;
    $word_addr=$word;
    $end_type_addr=$end_type;
    $word_copy_addr=$word_copy;
    var $0=$word_addr; //@line 3414 "dictionary.cpp"
    $word_end=$0; //@line 3414 "dictionary.cpp"
    var $1=$word_end; //@line 3414 "dictionary.cpp"
    var $2=HEAP8[($1)]; //@line 3414 "dictionary.cpp"
    var $conv8=(tempInt=(($2)),(tempInt>=128?tempInt-256:tempInt)); //@line 3414 "dictionary.cpp"
    var $cmp10=((($conv8))|0)!=32; //@line 3414 "dictionary.cpp"
    var $3=$word_end; //@line 3417 "dictionary.cpp"
    if ($cmp10) { __lastLabel__ = 0; __label__ = 1;; } else { __lastLabel__ = 0; __label__ = 4;; } //@line 3414 "dictionary.cpp"
    $for_body$$for_end$2: do { 
      if (__label__ == 1) {
        while(1) { 
  
          var $4=__lastLabel__ == 3 ? $10 : ($3);
          var $5=HEAP8[($4)]; //@line 3417 "dictionary.cpp"
          var $conv1=(tempInt=(($5)),(tempInt>=128?tempInt-256:tempInt)); //@line 3417 "dictionary.cpp"
          var $cmp2=((($conv1))|0)==69; //@line 3417 "dictionary.cpp"
          if ($cmp2) { __label__ = 2;; } else { __label__ = 3;; } //@line 3417 "dictionary.cpp"
          if (__label__ == 2) {
  
            var $6=$word_end; //@line 3418 "dictionary.cpp"
            HEAP8[($6)]=101; //@line 3418 "dictionary.cpp"
            ; //@line 3418 "dictionary.cpp"
          }
  
          var $7=$word_end; //@line 3414 "dictionary.cpp"
          var $incdec_ptr=(($7+1)&4294967295); //@line 3414 "dictionary.cpp"
          $word_end=$incdec_ptr; //@line 3414 "dictionary.cpp"
          var $8=$word_end; //@line 3414 "dictionary.cpp"
          var $9=HEAP8[($8)]; //@line 3414 "dictionary.cpp"
          var $conv=(tempInt=(($9)),(tempInt>=128?tempInt-256:tempInt)); //@line 3414 "dictionary.cpp"
          var $cmp=((($conv))|0)!=32; //@line 3414 "dictionary.cpp"
          var $10=$word_end; //@line 3417 "dictionary.cpp"
          if ($cmp) { __lastLabel__ = 3; __label__ = 1;continue ; } else { __lastLabel__ = 3; __label__ = 4;break $for_body$$for_end$2; } //@line 3414 "dictionary.cpp"
        }
      }
    } while(0);
  
    var $_lcssa7=__lastLabel__ == 0 ? $3 : ($10);
    var $11=$word_addr; //@line 3420 "dictionary.cpp"
    var $sub_ptr_lhs_cast=($_lcssa7); //@line 3420 "dictionary.cpp"
    var $sub_ptr_rhs_cast=($11); //@line 3420 "dictionary.cpp"
    var $sub_ptr_sub=((($sub_ptr_lhs_cast)-($sub_ptr_rhs_cast))&4294967295); //@line 3420 "dictionary.cpp"
    $i=$sub_ptr_sub; //@line 3420 "dictionary.cpp"
    var $12=$word_copy_addr; //@line 3421 "dictionary.cpp"
    var $13=$word_addr; //@line 3421 "dictionary.cpp"
    var $14=$i; //@line 3421 "dictionary.cpp"
    var $src$, $dest$, $stop$, $stop4$, $fast$;
    $src$ = $13;
    $dest$ = $12;
    $stop$ = $src$ + $14;
    if (($dest$%4) == ($src$%4) && $14 > 8) {
      while ($src$%4 !== 0 && $src$ < $stop$) {
        ; HEAP8[$dest$++] = HEAP8[$src$++];
      }
      $src$ >>= 2;
      $dest$ >>= 2;
      $stop4$ = $stop$ >> 2;
      while ($src$ < $stop4$) {
    ;;;    HEAP32[$dest$++] = HEAP32[$src$++];
      }
      $src$ <<= 2;
      $dest$ <<= 2;
    }while ($src$ < $stop$) {
      ; HEAP8[$dest$++] = HEAP8[$src$++];
    }; //@line 3421 "dictionary.cpp"
    var $15=$i; //@line 3422 "dictionary.cpp"
    var $16=$word_copy_addr; //@line 3422 "dictionary.cpp"
    var $arrayidx=(($16+$15)&4294967295); //@line 3422 "dictionary.cpp"
    HEAP8[($arrayidx)]=0; //@line 3422 "dictionary.cpp"
    var $17=$end_type_addr; //@line 3425 "dictionary.cpp"
    var $and=($17) & 63; //@line 3425 "dictionary.cpp"
    $i=$and; //@line 3425 "dictionary.cpp"
    $len_ending=$and; //@line 3425 "dictionary.cpp"
    var $cmp46=((($and))|0) > 0; //@line 3425 "dictionary.cpp"
    if ($cmp46) { __label__ = 5;; } else { __label__ = 8;; } //@line 3425 "dictionary.cpp"
    $for_body5$$for_end12$9: do { 
      if (__label__ == 5) {
        $for_body5$10: while(1) { 
  
          var $18=$word_end; //@line 3427 "dictionary.cpp"
          var $incdec_ptr6=(($18+-1)&4294967295); //@line 3427 "dictionary.cpp"
          $word_end=$incdec_ptr6; //@line 3427 "dictionary.cpp"
          var $19=$word_end; //@line 3428 "dictionary.cpp"
          var $20=HEAP8[($19)]; //@line 3428 "dictionary.cpp"
          var $conv73=(tempInt=(($20)),(tempInt>=128?tempInt-256:tempInt)); //@line 3428 "dictionary.cpp"
          var $and84=($conv73) & 192; //@line 3428 "dictionary.cpp"
          var $cmp95=((($and84))|0)==128; //@line 3428 "dictionary.cpp"
          if ($cmp95) { __label__ = 6;; } else { __label__ = 7;; } //@line 3428 "dictionary.cpp"
          $while_body$$for_inc11$12: do { 
            if (__label__ == 6) {
              while(1) { 
  
                var $21=$word_end; //@line 3430 "dictionary.cpp"
                var $incdec_ptr10=(($21+-1)&4294967295); //@line 3430 "dictionary.cpp"
                $word_end=$incdec_ptr10; //@line 3430 "dictionary.cpp"
                var $22=$len_ending; //@line 3431 "dictionary.cpp"
                var $inc=((($22)+1)&4294967295); //@line 3431 "dictionary.cpp"
                $len_ending=$inc; //@line 3431 "dictionary.cpp"
                var $23=$word_end; //@line 3428 "dictionary.cpp"
                var $24=HEAP8[($23)]; //@line 3428 "dictionary.cpp"
                var $conv7=(tempInt=(($24)),(tempInt>=128?tempInt-256:tempInt)); //@line 3428 "dictionary.cpp"
                var $and8=($conv7) & 192; //@line 3428 "dictionary.cpp"
                var $cmp9=((($and8))|0)==128; //@line 3428 "dictionary.cpp"
                if ($cmp9) { __label__ = 6;continue ; } else { __label__ = 7;break $while_body$$for_inc11$12; } //@line 3428 "dictionary.cpp"
              }
            }
          } while(0);
  
          var $25=$i; //@line 3425 "dictionary.cpp"
          var $dec=((($25)-1)&4294967295); //@line 3425 "dictionary.cpp"
          $i=$dec; //@line 3425 "dictionary.cpp"
          var $cmp4=((($dec))|0) > 0; //@line 3425 "dictionary.cpp"
          if ($cmp4) { __label__ = 5;continue $for_body5$10; } else { __label__ = 8;break $for_body5$$for_end12$9; } //@line 3425 "dictionary.cpp"
        }
      }
    } while(0);
  
    $i=0; //@line 3436 "dictionary.cpp"
    var $26=$i; //@line 3436 "dictionary.cpp"
    var $27=$len_ending; //@line 3436 "dictionary.cpp"
    var $cmp142=((($26))|0) < ((($27))|0); //@line 3436 "dictionary.cpp"
    var $28=$i; //@line 3438 "dictionary.cpp"
    if ($cmp142) { __lastLabel__ = 8; __label__ = 9;; } else { __lastLabel__ = 8; __label__ = 10;; } //@line 3436 "dictionary.cpp"
    $for_body15$$for_end21$17: do { 
      if (__label__ == 9) {
        while(1) { 
  
          var $29=__lastLabel__ == 9 ? $38 : ($28);
          var $30=$word_end; //@line 3438 "dictionary.cpp"
          var $arrayidx16=(($30+$29)&4294967295); //@line 3438 "dictionary.cpp"
          var $31=HEAP8[($arrayidx16)]; //@line 3438 "dictionary.cpp"
          var $32=$i; //@line 3438 "dictionary.cpp"
          var $arrayidx17=((__ZZ12RemoveEndingP10TranslatorPciS1_E6ending+$32)&4294967295); //@line 3438 "dictionary.cpp"
          HEAP8[($arrayidx17)]=$31; //@line 3438 "dictionary.cpp"
          var $33=$i; //@line 3439 "dictionary.cpp"
          var $34=$word_end; //@line 3439 "dictionary.cpp"
          var $arrayidx18=(($34+$33)&4294967295); //@line 3439 "dictionary.cpp"
          HEAP8[($arrayidx18)]=32; //@line 3439 "dictionary.cpp"
          var $35=$i; //@line 3436 "dictionary.cpp"
          var $inc20=((($35)+1)&4294967295); //@line 3436 "dictionary.cpp"
          $i=$inc20; //@line 3436 "dictionary.cpp"
          var $36=$i; //@line 3436 "dictionary.cpp"
          var $37=$len_ending; //@line 3436 "dictionary.cpp"
          var $cmp14=((($36))|0) < ((($37))|0); //@line 3436 "dictionary.cpp"
          var $38=$i; //@line 3438 "dictionary.cpp"
          if ($cmp14) { __lastLabel__ = 9; __label__ = 9;continue ; } else { __lastLabel__ = 9; __label__ = 10;break $for_body15$$for_end21$17; } //@line 3436 "dictionary.cpp"
        }
      }
    } while(0);
  
    var $_lcssa=__lastLabel__ == 8 ? $28 : ($38);
    var $arrayidx22=((__ZZ12RemoveEndingP10TranslatorPciS1_E6ending+$_lcssa)&4294967295); //@line 3441 "dictionary.cpp"
    HEAP8[($arrayidx22)]=0; //@line 3441 "dictionary.cpp"
    var $39=$word_end; //@line 3442 "dictionary.cpp"
    var $incdec_ptr23=(($39+-1)&4294967295); //@line 3442 "dictionary.cpp"
    $word_end=$incdec_ptr23; //@line 3442 "dictionary.cpp"
    var $40=$end_type_addr; //@line 3444 "dictionary.cpp"
    var $and24=($40) & 65520; //@line 3444 "dictionary.cpp"
    var $or=($and24) | 4; //@line 3444 "dictionary.cpp"
    $end_flags=$or; //@line 3444 "dictionary.cpp"
    var $41=$end_type_addr; //@line 3450 "dictionary.cpp"
    var $and25=($41) & 512; //@line 3450 "dictionary.cpp"
    var $tobool=((($and25))|0)!=0; //@line 3450 "dictionary.cpp"
    if ($tobool) { __label__ = 11;; } else { __label__ = 13;; } //@line 3450 "dictionary.cpp"
    $if_then26$$if_end33$21: do { 
      if (__label__ == 11) {
  
        var $42=$word_end; //@line 3452 "dictionary.cpp"
        var $arrayidx27=(($42)&4294967295); //@line 3452 "dictionary.cpp"
        var $43=HEAP8[($arrayidx27)]; //@line 3452 "dictionary.cpp"
        var $conv28=(tempInt=(($43)),(tempInt>=128?tempInt-256:tempInt)); //@line 3452 "dictionary.cpp"
        var $cmp29=((($conv28))|0)==105; //@line 3452 "dictionary.cpp"
        if (!($cmp29)) { __label__ = 13;break $if_then26$$if_end33$21; } //@line 3452 "dictionary.cpp"
  
        var $44=$word_end; //@line 3453 "dictionary.cpp"
        var $arrayidx31=(($44)&4294967295); //@line 3453 "dictionary.cpp"
        HEAP8[($arrayidx31)]=121; //@line 3453 "dictionary.cpp"
        ; //@line 3453 "dictionary.cpp"
      }
    } while(0);
  
    var $45=$end_type_addr; //@line 3456 "dictionary.cpp"
    var $and34=($45) & 256; //@line 3456 "dictionary.cpp"
    var $tobool35=((($and34))|0)!=0; //@line 3456 "dictionary.cpp"
    if ($tobool35) { __label__ = 14;; } else { __label__ = 33;; } //@line 3456 "dictionary.cpp"
    $if_then36$$if_end98$25: do { 
      if (__label__ == 14) {
  
        var $46=$tr_addr; //@line 3458 "dictionary.cpp"
        var $translator_name=(($46+280)&4294967295); //@line 3458 "dictionary.cpp"
        var $47=HEAP32[(($translator_name)>>2)]; //@line 3458 "dictionary.cpp"
        var $cmp37=((($47))|0)==25966; //@line 3458 "dictionary.cpp"
        var $48=$tr_addr; //@line 3461 "dictionary.cpp"
        if ($cmp37) { __label__ = 15;; } else { __label__ = 28;; } //@line 3458 "dictionary.cpp"
        $if_then38$$if_else80$27: do { 
          if (__label__ == 15) {
  
            var $49=$word_end; //@line 3461 "dictionary.cpp"
            var $arrayidx39=(($49+-1)&4294967295); //@line 3461 "dictionary.cpp"
            var $50=HEAP8[($arrayidx39)]; //@line 3461 "dictionary.cpp"
            var $conv40=(tempInt=(($50)),(tempInt>=128?tempInt-256:tempInt)); //@line 3461 "dictionary.cpp"
            var $call=__ZL8IsLetterP10Translatorii($48, $conv40, 7); //@line 3461 "dictionary.cpp"
            var $tobool41=((($call))|0)!=0; //@line 3461 "dictionary.cpp"
            if ($tobool41) { __label__ = 16;; } else { __label__ = 23;; } //@line 3461 "dictionary.cpp"
            $land_lhs_true$$if_else$29: do { 
              if (__label__ == 16) {
  
                var $51=$tr_addr; //@line 3461 "dictionary.cpp"
                var $52=$word_end; //@line 3461 "dictionary.cpp"
                var $arrayidx42=(($52)&4294967295); //@line 3461 "dictionary.cpp"
                var $53=HEAP8[($arrayidx42)]; //@line 3461 "dictionary.cpp"
                var $conv43=(tempInt=(($53)),(tempInt>=128?tempInt-256:tempInt)); //@line 3461 "dictionary.cpp"
                var $call44=__ZL8IsLetterP10Translatorii($51, $conv43, 1); //@line 3461 "dictionary.cpp"
                var $tobool45=((($call44))|0)!=0; //@line 3461 "dictionary.cpp"
                if (!($tobool45)) { __label__ = 23;break $land_lhs_true$$if_else$29; } //@line 3461 "dictionary.cpp"
  
                $i=0; //@line 3465 "dictionary.cpp"
                ; //@line 3465 "dictionary.cpp"
                while(1) { 
  
                  var $54=$i; //@line 3465 "dictionary.cpp"
                  var $arrayidx48=((__ZZ12RemoveEndingP10TranslatorPciS1_E16add_e_exceptions+$54*4)&4294967295); //@line 3465 "dictionary.cpp"
                  var $55=HEAP32[(($arrayidx48)>>2)]; //@line 3465 "dictionary.cpp"
                  $p=$55; //@line 3465 "dictionary.cpp"
                  var $cmp49=((($55))|0)!=0; //@line 3465 "dictionary.cpp"
                  if (!($cmp49)) { __label__ = 22;break ; } //@line 3465 "dictionary.cpp"
  
                  var $56=$p; //@line 3467 "dictionary.cpp"
                  var $call51=_strlen($56); //@line 3467 "dictionary.cpp"
                  $len=$call51; //@line 3467 "dictionary.cpp"
                  var $57=$p; //@line 3468 "dictionary.cpp"
                  var $58=$len; //@line 3468 "dictionary.cpp"
                  var $sub=((1-($58))&4294967295); //@line 3468 "dictionary.cpp"
                  var $59=$word_end; //@line 3468 "dictionary.cpp"
                  var $arrayidx52=(($59+$sub)&4294967295); //@line 3468 "dictionary.cpp"
                  var $60=$len; //@line 3468 "dictionary.cpp"
                  var $call53=_memcmp($57, $arrayidx52, $60); //@line 3468 "dictionary.cpp"
                  var $cmp54=((($call53))|0)==0; //@line 3468 "dictionary.cpp"
                  if ($cmp54) { __label__ = 21;break ; } //@line 3468 "dictionary.cpp"
  
                  var $61=$i; //@line 3465 "dictionary.cpp"
                  var $inc58=((($61)+1)&4294967295); //@line 3465 "dictionary.cpp"
                  $i=$inc58; //@line 3465 "dictionary.cpp"
                  __label__ = 18;continue ; //@line 3465 "dictionary.cpp"
                }
                if (__label__ == 21) {
  
                  var $_pr=$p; //@line 3473 "dictionary.cpp"
                  var $cmp60=((($_pr))|0)==0; //@line 3473 "dictionary.cpp"
                  if (!($cmp60)) { __label__ = 30;break $if_then38$$if_else80$27; } //@line 3473 "dictionary.cpp"
                }
  
                var $62=$end_flags; //@line 3474 "dictionary.cpp"
                var $or62=($62) | 16; //@line 3474 "dictionary.cpp"
                $end_flags=$or62; //@line 3474 "dictionary.cpp"
                __label__ = 30;break $if_then38$$if_else80$27; //@line 3474 "dictionary.cpp"
              }
            } while(0);
  
            $i=0; //@line 3478 "dictionary.cpp"
            ; //@line 3478 "dictionary.cpp"
            while(1) { 
  
              var $63=$i; //@line 3478 "dictionary.cpp"
              var $arrayidx65=((__ZZ12RemoveEndingP10TranslatorPciS1_E15add_e_additions+$63*4)&4294967295); //@line 3478 "dictionary.cpp"
              var $64=HEAP32[(($arrayidx65)>>2)]; //@line 3478 "dictionary.cpp"
              $p=$64; //@line 3478 "dictionary.cpp"
              var $cmp66=((($64))|0)!=0; //@line 3478 "dictionary.cpp"
              if (!($cmp66)) { __label__ = 30;break $if_then38$$if_else80$27; } //@line 3478 "dictionary.cpp"
  
              var $65=$p; //@line 3480 "dictionary.cpp"
              var $call68=_strlen($65); //@line 3480 "dictionary.cpp"
              $len=$call68; //@line 3480 "dictionary.cpp"
              var $66=$p; //@line 3481 "dictionary.cpp"
              var $67=$len; //@line 3481 "dictionary.cpp"
              var $sub69=((1-($67))&4294967295); //@line 3481 "dictionary.cpp"
              var $68=$word_end; //@line 3481 "dictionary.cpp"
              var $arrayidx70=(($68+$sub69)&4294967295); //@line 3481 "dictionary.cpp"
              var $69=$len; //@line 3481 "dictionary.cpp"
              var $call71=_memcmp($66, $arrayidx70, $69); //@line 3481 "dictionary.cpp"
              var $cmp72=((($call71))|0)==0; //@line 3481 "dictionary.cpp"
              if ($cmp72) { __label__ = 26;break ; } //@line 3481 "dictionary.cpp"
  
              var $71=$i; //@line 3478 "dictionary.cpp"
              var $inc77=((($71)+1)&4294967295); //@line 3478 "dictionary.cpp"
              $i=$inc77; //@line 3478 "dictionary.cpp"
              __label__ = 24;continue ; //@line 3478 "dictionary.cpp"
            }
  
            var $70=$end_flags; //@line 3483 "dictionary.cpp"
            var $or74=($70) | 16; //@line 3483 "dictionary.cpp"
            $end_flags=$or74; //@line 3483 "dictionary.cpp"
            ; //@line 3484 "dictionary.cpp"
          }
          else if (__label__ == 28) {
  
            var $langopts=(($48)&4294967295); //@line 3490 "dictionary.cpp"
            var $suffix_add_e=(($langopts+276)&4294967295); //@line 3490 "dictionary.cpp"
            var $72=HEAP32[(($suffix_add_e)>>2)]; //@line 3490 "dictionary.cpp"
            var $cmp81=((($72))|0)!=0; //@line 3490 "dictionary.cpp"
            if (!($cmp81)) { __label__ = 30;break $if_then38$$if_else80$27; } //@line 3490 "dictionary.cpp"
  
            var $73=$end_flags; //@line 3492 "dictionary.cpp"
            var $or83=($73) | 16; //@line 3492 "dictionary.cpp"
            $end_flags=$or83; //@line 3492 "dictionary.cpp"
            ; //@line 3493 "dictionary.cpp"
          }
        } while(0);
  
        var $74=$end_flags; //@line 3495 "dictionary.cpp"
        var $and86=($74) & 16; //@line 3495 "dictionary.cpp"
        var $tobool87=((($and86))|0)!=0; //@line 3495 "dictionary.cpp"
        if (!($tobool87)) { __label__ = 33;break $if_then36$$if_end98$25; } //@line 3495 "dictionary.cpp"
  
        var $75=$tr_addr; //@line 3497 "dictionary.cpp"
        var $langopts89=(($75)&4294967295); //@line 3497 "dictionary.cpp"
        var $suffix_add_e90=(($langopts89+276)&4294967295); //@line 3497 "dictionary.cpp"
        var $76=HEAP32[(($suffix_add_e90)>>2)]; //@line 3497 "dictionary.cpp"
        var $77=$word_end; //@line 3497 "dictionary.cpp"
        var $arrayidx91=(($77+1)&4294967295); //@line 3497 "dictionary.cpp"
        var $call92=__Z8utf8_outjPc($76, $arrayidx91); //@line 3497 "dictionary.cpp"
        var $78=HEAP32[((_option_phonemes)>>2)]; //@line 3499 "dictionary.cpp"
        var $cmp93=((($78))|0)==2; //@line 3499 "dictionary.cpp"
        if (!($cmp93)) { __label__ = 33;break $if_then36$$if_end98$25; } //@line 3499 "dictionary.cpp"
  
        var $79=HEAP32[((_f_trans)>>2)]; //@line 3501 "dictionary.cpp"
        var $80=_fwrite(((__str25141)&4294967295), 6, 1, $79); //@line 3501 "dictionary.cpp"
        ; //@line 3502 "dictionary.cpp"
      }
    } while(0);
  
    var $81=$end_type_addr; //@line 3506 "dictionary.cpp"
    var $and99=($81) & 2048; //@line 3506 "dictionary.cpp"
    var $tobool100=((($and99))|0)!=0; //@line 3506 "dictionary.cpp"
    if ($tobool100) { __label__ = 34;; } else { __label__ = 36;; } //@line 3506 "dictionary.cpp"
    $land_lhs_true101$$if_end105$51: do { 
      if (__label__ == 34) {
  
        var $82=$tr_addr; //@line 3506 "dictionary.cpp"
        var $expect_verb=(($82+8272)&4294967295); //@line 3506 "dictionary.cpp"
        var $83=HEAP32[(($expect_verb)>>2)]; //@line 3506 "dictionary.cpp"
        var $cmp102=((($83))|0)==0; //@line 3506 "dictionary.cpp"
        if (!($cmp102)) { __label__ = 36;break $land_lhs_true101$$if_end105$51; } //@line 3506 "dictionary.cpp"
  
        var $84=$tr_addr; //@line 3507 "dictionary.cpp"
        var $expect_verb104=(($84+8272)&4294967295); //@line 3507 "dictionary.cpp"
        HEAP32[(($expect_verb104)>>2)]=1; //@line 3507 "dictionary.cpp"
        ; //@line 3507 "dictionary.cpp"
      }
    } while(0);
  
    var $call106=_strcmp(((__ZZ12RemoveEndingP10TranslatorPciS1_E6ending)&4294967295), ((__str5172)&4294967295)); //@line 3510 "dictionary.cpp"
    var $cmp107=((($call106))|0)==0; //@line 3510 "dictionary.cpp"
    if ($cmp107) { __label__ = 38;; } else { __label__ = 37;; } //@line 3510 "dictionary.cpp"
    $if_then110$$lor_lhs_false$55: do { 
      if (__label__ == 37) {
  
        var $call108=_strcmp(((__ZZ12RemoveEndingP10TranslatorPciS1_E6ending)&4294967295), ((__str27143)&4294967295)); //@line 3510 "dictionary.cpp"
        var $cmp109=((($call108))|0)==0; //@line 3510 "dictionary.cpp"
        if ($cmp109) { __label__ = 38;break $if_then110$$lor_lhs_false$55; } else { __label__ = 39;break $if_then110$$lor_lhs_false$55; } //@line 3510 "dictionary.cpp"
      }
    } while(0);
    if (__label__ == 38) {
  
      var $85=$end_flags; //@line 3511 "dictionary.cpp"
      var $or111=($85) | 8; //@line 3511 "dictionary.cpp"
      $end_flags=$or111; //@line 3511 "dictionary.cpp"
      ; //@line 3511 "dictionary.cpp"
    }
  
    var $86=HEAP8[(((__ZZ12RemoveEndingP10TranslatorPciS1_E6ending)&4294967295))]; //@line 3514 "dictionary.cpp"
    var $conv113=(tempInt=(($86)),(tempInt>=128?tempInt-256:tempInt)); //@line 3514 "dictionary.cpp"
    var $cmp114=((($conv113))|0)==39; //@line 3514 "dictionary.cpp"
    if ($cmp114) { __label__ = 40;; } else { __label__ = 41;; } //@line 3514 "dictionary.cpp"
    if (__label__ == 40) {
  
      var $87=$end_flags; //@line 3515 "dictionary.cpp"
      var $and116=($87) & -5; //@line 3515 "dictionary.cpp"
      $end_flags=$and116; //@line 3515 "dictionary.cpp"
      ; //@line 3515 "dictionary.cpp"
    }
  
    var $88=$end_flags; //@line 3517 "dictionary.cpp"
    ;
    return $88; //@line 3517 "dictionary.cpp"
    return null;
  }
  

  function __ZL13IsLetterGroupP10TranslatorPcii($tr, $word, $group, $pre) {
    ;
    var __label__;
  
    var $retval;
    var $tr_addr;
    var $word_addr;
    var $group_addr;
    var $pre_addr;
    var $p;
    var $w;
    var $len;
    $tr_addr=$tr;
    $word_addr=$word;
    $group_addr=$group;
    $pre_addr=$pre;
    $len=0; //@line 692 "dictionary.cpp"
    var $0=$group_addr; //@line 694 "dictionary.cpp"
    var $1=$tr_addr; //@line 694 "dictionary.cpp"
    var $letterGroups=(($1+5152)&4294967295); //@line 694 "dictionary.cpp"
    var $arrayidx=(($letterGroups+$0*4)&4294967295); //@line 694 "dictionary.cpp"
    var $2=HEAP32[(($arrayidx)>>2)]; //@line 694 "dictionary.cpp"
    $p=$2; //@line 694 "dictionary.cpp"
    var $3=$p; //@line 695 "dictionary.cpp"
    var $cmp=((($3))|0)==0; //@line 695 "dictionary.cpp"
    if ($cmp) { __label__ = 1;; } else { __label__ = 2;; } //@line 695 "dictionary.cpp"
    if (__label__ == 1) {
  
      $retval=0; //@line 696 "dictionary.cpp"
      ; //@line 696 "dictionary.cpp"
    }
    else if (__label__ == 2) {
      $while_cond$4: while(1) { 
  
        var $4=$p; //@line 698 "dictionary.cpp"
        var $5=HEAP8[($4)]; //@line 698 "dictionary.cpp"
        var $conv=(tempInt=(($5)),(tempInt>=128?tempInt-256:tempInt)); //@line 698 "dictionary.cpp"
        var $cmp1=((($conv))|0)!=7; //@line 698 "dictionary.cpp"
        if (!($cmp1)) { __label__ = 13;break $while_cond$4; } //@line 698 "dictionary.cpp"
  
        var $6=$pre_addr; //@line 700 "dictionary.cpp"
        var $tobool=((($6))|0)!=0; //@line 700 "dictionary.cpp"
        if ($tobool) { __label__ = 4;; } else { __label__ = 5;; } //@line 700 "dictionary.cpp"
        if (__label__ == 4) {
  
          var $7=$p; //@line 702 "dictionary.cpp"
          var $call=_strlen($7); //@line 702 "dictionary.cpp"
          $len=$call; //@line 702 "dictionary.cpp"
          var $8=$word_addr; //@line 703 "dictionary.cpp"
          var $9=$len; //@line 703 "dictionary.cpp"
          var $idx_neg=(((-($9)))&4294967295); //@line 703 "dictionary.cpp"
          var $add_ptr=(($8+$idx_neg)&4294967295); //@line 703 "dictionary.cpp"
          var $add_ptr3=(($add_ptr+1)&4294967295); //@line 703 "dictionary.cpp"
          $w=$add_ptr3; //@line 703 "dictionary.cpp"
          ; //@line 704 "dictionary.cpp"
        }
        else if (__label__ == 5) {
  
          var $10=$word_addr; //@line 707 "dictionary.cpp"
          $w=$10; //@line 707 "dictionary.cpp"
          ;
        }
  
        var $11=$p; //@line 709 "dictionary.cpp"
        var $12=HEAP8[($11)]; //@line 709 "dictionary.cpp"
        var $conv61=(tempInt=(($12)),(tempInt>=128?tempInt-256:tempInt)); //@line 709 "dictionary.cpp"
        var $13=$w; //@line 709 "dictionary.cpp"
        var $14=HEAP8[($13)]; //@line 709 "dictionary.cpp"
        var $conv72=(tempInt=(($14)),(tempInt>=128?tempInt-256:tempInt)); //@line 709 "dictionary.cpp"
        var $cmp83=((($conv61))|0)==((($conv72))|0); //@line 709 "dictionary.cpp"
        if ($cmp83) { __label__ = 7;; } else { __label__ = 8;; } //@line 709 "dictionary.cpp"
        $while_body9$$while_end$11: do { 
          if (__label__ == 7) {
            while(1) { 
  
              var $15=$w; //@line 711 "dictionary.cpp"
              var $incdec_ptr=(($15+1)&4294967295); //@line 711 "dictionary.cpp"
              $w=$incdec_ptr; //@line 711 "dictionary.cpp"
              var $16=$p; //@line 712 "dictionary.cpp"
              var $incdec_ptr10=(($16+1)&4294967295); //@line 712 "dictionary.cpp"
              $p=$incdec_ptr10; //@line 712 "dictionary.cpp"
              var $17=$p; //@line 709 "dictionary.cpp"
              var $18=HEAP8[($17)]; //@line 709 "dictionary.cpp"
              var $conv6=(tempInt=(($18)),(tempInt>=128?tempInt-256:tempInt)); //@line 709 "dictionary.cpp"
              var $19=$w; //@line 709 "dictionary.cpp"
              var $20=HEAP8[($19)]; //@line 709 "dictionary.cpp"
              var $conv7=(tempInt=(($20)),(tempInt>=128?tempInt-256:tempInt)); //@line 709 "dictionary.cpp"
              var $cmp8=((($conv6))|0)==((($conv7))|0); //@line 709 "dictionary.cpp"
              if ($cmp8) { __label__ = 7;continue ; } else { __label__ = 8;break $while_body9$$while_end$11; } //@line 709 "dictionary.cpp"
            }
          }
        } while(0);
  
        var $21=$p; //@line 714 "dictionary.cpp"
        var $22=HEAP8[($21)]; //@line 714 "dictionary.cpp"
        var $conv11=(tempInt=(($22)),(tempInt>=128?tempInt-256:tempInt)); //@line 714 "dictionary.cpp"
        var $cmp12=((($conv11))|0)==0; //@line 714 "dictionary.cpp"
        if ($cmp12) { __label__ = 9;break $while_cond$4; } //@line 714 "dictionary.cpp"
        while(1) { 
  
          var $27=$p; //@line 721 "dictionary.cpp"
          var $incdec_ptr19=(($27+1)&4294967295); //@line 721 "dictionary.cpp"
          $p=$incdec_ptr19; //@line 721 "dictionary.cpp"
          var $28=HEAP8[($27)]; //@line 721 "dictionary.cpp"
          var $conv20=(tempInt=(($28)),(tempInt>=128?tempInt-256:tempInt)); //@line 721 "dictionary.cpp"
          var $cmp21=((($conv20))|0)!=0; //@line 721 "dictionary.cpp"
          if ($cmp21) { __label__ = 12;continue ; } else { __label__ = 2;continue $while_cond$4; } //@line 721 "dictionary.cpp"
        }
      }
      if (__label__ == 13) {
  
        $retval=0; //@line 723 "dictionary.cpp"
        ; //@line 723 "dictionary.cpp"
      }
      else if (__label__ == 9) {
  
        var $23=$pre_addr; //@line 716 "dictionary.cpp"
        var $tobool14=((($23))|0)!=0; //@line 716 "dictionary.cpp"
        if ($tobool14) { __label__ = 10;; } else { __label__ = 11;; } //@line 716 "dictionary.cpp"
        if (__label__ == 10) {
  
          var $24=$len; //@line 717 "dictionary.cpp"
          $retval=$24; //@line 717 "dictionary.cpp"
          ; //@line 717 "dictionary.cpp"
        }
        else if (__label__ == 11) {
  
          var $25=$w; //@line 718 "dictionary.cpp"
          var $26=$word_addr; //@line 718 "dictionary.cpp"
          var $sub_ptr_lhs_cast=($25); //@line 718 "dictionary.cpp"
          var $sub_ptr_rhs_cast=($26); //@line 718 "dictionary.cpp"
          var $sub_ptr_sub=((($sub_ptr_lhs_cast)-($sub_ptr_rhs_cast))&4294967295); //@line 718 "dictionary.cpp"
          $retval=$sub_ptr_sub; //@line 718 "dictionary.cpp"
          ; //@line 718 "dictionary.cpp"
        }
      }
    }
  
    var $29=$retval; //@line 724 "dictionary.cpp"
    ;
    return $29; //@line 724 "dictionary.cpp"
    return null;
  }
  

  function __Z11CalcPitchesP10Translatori($tr, $clause_type) {
    var __stackBase__  = STACKTOP; STACKTOP += 6000; _memset(__stackBase__, 0, 6000);
    var __label__;
    var __lastLabel__ = null;
  
    var $tr_addr;
    var $clause_type_addr;
    var $p;
    var $syl;
    var $ix;
    var $x;
    var $st_ix;
    var $n_st;
    var $option;
    var $group_tone;
    var $group_tone_emph;
    var $group_tone_comma;
    var $ph_start;
    var $st_start;
    var $st_clause_end;
    var $count;
    var $n_primary;
    var $count_primary;
    var $ph;
    var $ph_end;
    var $syllable_tab2=__stackBase__;
    $tr_addr=$tr;
    $clause_type_addr=$clause_type;
    $ph_start=0; //@line 1126 "intonation.cpp"
    var $0=HEAP32[((_n_phoneme_list)>>2)]; //@line 1133 "intonation.cpp"
    $ph_end=$0; //@line 1133 "intonation.cpp"
    var $arraydecay=(($syllable_tab2)&4294967295); //@line 1137 "intonation.cpp"
    HEAP32[((__ZL12syllable_tab)>>2)]=$arraydecay; //@line 1137 "intonation.cpp"
    $n_st=0; //@line 1138 "intonation.cpp"
    $n_primary=0; //@line 1139 "intonation.cpp"
    $ix=0; //@line 1140 "intonation.cpp"
    var $1=$ix; //@line 1140 "intonation.cpp"
    var $2=HEAP32[((_n_phoneme_list)>>2)]; //@line 1140 "intonation.cpp"
    var $sub4=((($2)-1)&4294967295); //@line 1140 "intonation.cpp"
    var $cmp5=((($1))|0) < ((($sub4))|0); //@line 1140 "intonation.cpp"
    if ($cmp5) { __label__ = 1;; } else { __label__ = 8;; } //@line 1140 "intonation.cpp"
    $for_body$$for_end$2: do { 
      if (__label__ == 1) {
        $for_body$3: while(1) { 
  
          var $3=$ix; //@line 1142 "intonation.cpp"
          var $arrayidx=((_phoneme_list+$3*24)&4294967295); //@line 1142 "intonation.cpp"
          $p=$arrayidx; //@line 1142 "intonation.cpp"
          var $4=$p; //@line 1143 "intonation.cpp"
          var $synthflags=(($4+4)&4294967295); //@line 1143 "intonation.cpp"
          var $5=HEAPU16[(($synthflags)>>1)]; //@line 1143 "intonation.cpp"
          var $conv=((($5))&65535); //@line 1143 "intonation.cpp"
          var $and=($conv) & 4; //@line 1143 "intonation.cpp"
          var $tobool=((($and))|0)!=0; //@line 1143 "intonation.cpp"
          if ($tobool) { __label__ = 2;; } else { __label__ = 4;; } //@line 1143 "intonation.cpp"
          $if_then$$if_else$5: do { 
            if (__label__ == 2) {
  
              var $6=$n_st; //@line 1145 "intonation.cpp"
              var $7=HEAP32[((__ZL12syllable_tab)>>2)]; //@line 1145 "intonation.cpp"
              var $arrayidx1=(($7+6*$6)&4294967295); //@line 1145 "intonation.cpp"
              var $flags=(($arrayidx1+2)&4294967295); //@line 1145 "intonation.cpp"
              HEAP8[($flags)]=0; //@line 1145 "intonation.cpp"
              var $8=$n_st; //@line 1146 "intonation.cpp"
              var $9=HEAP32[((__ZL12syllable_tab)>>2)]; //@line 1146 "intonation.cpp"
              var $arrayidx2=(($9+6*$8)&4294967295); //@line 1146 "intonation.cpp"
              var $env=(($arrayidx2+1)&4294967295); //@line 1146 "intonation.cpp"
              HEAP8[($env)]=0; //@line 1146 "intonation.cpp"
              var $10=$ix; //@line 1147 "intonation.cpp"
              var $add=((($10)+1)&4294967295); //@line 1147 "intonation.cpp"
              var $arrayidx3=((_phoneme_list+$add*24)&4294967295); //@line 1147 "intonation.cpp"
              var $type=(($arrayidx3+15)&4294967295); //@line 1147 "intonation.cpp"
              var $11=HEAP8[($type)]; //@line 1147 "intonation.cpp"
              var $12=$n_st; //@line 1147 "intonation.cpp"
              var $13=HEAP32[((__ZL12syllable_tab)>>2)]; //@line 1147 "intonation.cpp"
              var $arrayidx4=(($13+6*$12)&4294967295); //@line 1147 "intonation.cpp"
              var $nextph_type=(($arrayidx4+3)&4294967295); //@line 1147 "intonation.cpp"
              HEAP8[($nextph_type)]=$11; //@line 1147 "intonation.cpp"
              var $14=$p; //@line 1148 "intonation.cpp"
              var $stresslevel=(($14+1)&4294967295); //@line 1148 "intonation.cpp"
              var $15=HEAP8[($stresslevel)]; //@line 1148 "intonation.cpp"
              var $16=$n_st; //@line 1148 "intonation.cpp"
              var $inc=((($16)+1)&4294967295); //@line 1148 "intonation.cpp"
              $n_st=$inc; //@line 1148 "intonation.cpp"
              var $17=HEAP32[((__ZL12syllable_tab)>>2)]; //@line 1148 "intonation.cpp"
              var $arrayidx5=(($17+6*$16)&4294967295); //@line 1148 "intonation.cpp"
              var $stress=(($arrayidx5)&4294967295); //@line 1148 "intonation.cpp"
              HEAP8[($stress)]=$15; //@line 1148 "intonation.cpp"
              var $18=$p; //@line 1150 "intonation.cpp"
              var $stresslevel6=(($18+1)&4294967295); //@line 1150 "intonation.cpp"
              var $19=HEAPU8[($stresslevel6)]; //@line 1150 "intonation.cpp"
              var $conv7=((($19))&255); //@line 1150 "intonation.cpp"
              var $cmp8=((($conv7))|0) >= 4; //@line 1150 "intonation.cpp"
              if (!($cmp8)) { __label__ = 7;break $if_then$$if_else$5; } //@line 1150 "intonation.cpp"
  
              var $20=$n_primary; //@line 1151 "intonation.cpp"
              var $inc10=((($20)+1)&4294967295); //@line 1151 "intonation.cpp"
              $n_primary=$inc10; //@line 1151 "intonation.cpp"
              ; //@line 1151 "intonation.cpp"
            }
            else if (__label__ == 4) {
  
              var $21=$p; //@line 1154 "intonation.cpp"
              var $ph11=(($21+8)&4294967295); //@line 1154 "intonation.cpp"
              var $22=HEAP32[(($ph11)>>2)]; //@line 1154 "intonation.cpp"
              var $code=(($22+10)&4294967295); //@line 1154 "intonation.cpp"
              var $23=HEAPU8[($code)]; //@line 1154 "intonation.cpp"
              var $conv12=((($23))&255); //@line 1154 "intonation.cpp"
              var $cmp13=((($conv12))|0)==27; //@line 1154 "intonation.cpp"
              if (!($cmp13)) { __label__ = 7;break $if_then$$if_else$5; } //@line 1154 "intonation.cpp"
  
              var $24=$n_st; //@line 1154 "intonation.cpp"
              var $cmp14=((($24))|0) > 0; //@line 1154 "intonation.cpp"
              if (!($cmp14)) { __label__ = 7;break $if_then$$if_else$5; } //@line 1154 "intonation.cpp"
  
              var $25=$n_st; //@line 1156 "intonation.cpp"
              var $sub16=((($25)-1)&4294967295); //@line 1156 "intonation.cpp"
              var $26=HEAP32[((__ZL12syllable_tab)>>2)]; //@line 1156 "intonation.cpp"
              var $arrayidx17=(($26+6*$sub16)&4294967295); //@line 1156 "intonation.cpp"
              var $flags18=(($arrayidx17+2)&4294967295); //@line 1156 "intonation.cpp"
              var $27=HEAP8[($flags18)]; //@line 1156 "intonation.cpp"
              var $conv19=(tempInt=(($27)),(tempInt>=128?tempInt-256:tempInt)); //@line 1156 "intonation.cpp"
              var $or=($conv19) | 4; //@line 1156 "intonation.cpp"
              var $conv20=((($or)) & 255); //@line 1156 "intonation.cpp"
              HEAP8[($flags18)]=$conv20; //@line 1156 "intonation.cpp"
              ; //@line 1157 "intonation.cpp"
            }
          } while(0);
  
          var $28=$ix; //@line 1140 "intonation.cpp"
          var $inc23=((($28)+1)&4294967295); //@line 1140 "intonation.cpp"
          $ix=$inc23; //@line 1140 "intonation.cpp"
          var $29=$ix; //@line 1140 "intonation.cpp"
          var $30=HEAP32[((_n_phoneme_list)>>2)]; //@line 1140 "intonation.cpp"
          var $sub=((($30)-1)&4294967295); //@line 1140 "intonation.cpp"
          var $cmp=((($29))|0) < ((($sub))|0); //@line 1140 "intonation.cpp"
          if ($cmp) { __label__ = 1;continue $for_body$3; } else { __label__ = 8;break $for_body$$for_end$2; } //@line 1140 "intonation.cpp"
        }
      }
    } while(0);
  
    var $31=$n_st; //@line 1159 "intonation.cpp"
    var $32=HEAP32[((__ZL12syllable_tab)>>2)]; //@line 1159 "intonation.cpp"
    var $arrayidx24=(($32+6*$31)&4294967295); //@line 1159 "intonation.cpp"
    var $stress25=(($arrayidx24)&4294967295); //@line 1159 "intonation.cpp"
    HEAP8[($stress25)]=0; //@line 1159 "intonation.cpp"
    var $33=$n_st; //@line 1161 "intonation.cpp"
    var $cmp26=((($33))|0)==0; //@line 1161 "intonation.cpp"
    if ($cmp26) { __label__ = 70;; } else { __label__ = 9;; } //@line 1161 "intonation.cpp"
    $for_end258$$if_end28$13: do { 
      if (__label__ == 9) {
  
        var $34=$tr_addr; //@line 1166 "intonation.cpp"
        var $langopts=(($34)&4294967295); //@line 1166 "intonation.cpp"
        var $tone_language=(($langopts+224)&4294967295); //@line 1166 "intonation.cpp"
        var $35=HEAP32[(($tone_language)>>2)]; //@line 1166 "intonation.cpp"
        var $cmp29=((($35))|0)==1; //@line 1166 "intonation.cpp"
        var $36=$tr_addr; //@line 1168 "intonation.cpp"
        if ($cmp29) { __label__ = 10;; } else { __label__ = 11;; } //@line 1166 "intonation.cpp"
        if (__label__ == 10) {
  
          var $37=$clause_type_addr; //@line 1168 "intonation.cpp"
          __ZL16CalcPitches_ToneP10Translatori($36, $37); //@line 1168 "intonation.cpp"
          ; //@line 1169 "intonation.cpp"
        }
        else if (__label__ == 11) {
  
          var $langopts32=(($36)&4294967295); //@line 1173 "intonation.cpp"
          var $intonation_group=(($langopts32+228)&4294967295); //@line 1173 "intonation.cpp"
          var $38=HEAP32[(($intonation_group)>>2)]; //@line 1173 "intonation.cpp"
          $option=$38; //@line 1173 "intonation.cpp"
          var $39=$option; //@line 1174 "intonation.cpp"
          var $cmp33=((($39))|0) >= 8; //@line 1174 "intonation.cpp"
          if ($cmp33) { __label__ = 12;; } else { __label__ = 13;; } //@line 1174 "intonation.cpp"
          $if_end35_thread$$if_end35$18: do { 
            if (__label__ == 12) {
  
              $option=1; //@line 1175 "intonation.cpp"
              var $40=$clause_type_addr; //@line 1179 "intonation.cpp"
              __lastLabel__ = 12; __label__ = 15;break $if_end35_thread$$if_end35$18; //@line 1177 "intonation.cpp"
            }
            else if (__label__ == 13) {
  
              var $cmp36=((($39))|0)==0; //@line 1177 "intonation.cpp"
              var $41=$clause_type_addr; //@line 1179 "intonation.cpp"
              if ($cmp36) { __lastLabel__ = 13; ; } else { __lastLabel__ = 13; __label__ = 15;break $if_end35_thread$$if_end35$18; } //@line 1177 "intonation.cpp"
  
              var $42=$tr_addr; //@line 1179 "intonation.cpp"
              var $langopts38=(($42)&4294967295); //@line 1179 "intonation.cpp"
              var $tunes=(($langopts38+232)&4294967295); //@line 1179 "intonation.cpp"
              var $arrayidx39=(($tunes+$41)&4294967295); //@line 1179 "intonation.cpp"
              var $43=HEAPU8[($arrayidx39)]; //@line 1179 "intonation.cpp"
              var $conv40=((($43))&255); //@line 1179 "intonation.cpp"
              $group_tone=$conv40; //@line 1179 "intonation.cpp"
              var $44=$tr_addr; //@line 1180 "intonation.cpp"
              var $langopts41=(($44)&4294967295); //@line 1180 "intonation.cpp"
              var $tunes42=(($langopts41+232)&4294967295); //@line 1180 "intonation.cpp"
              var $arrayidx43=(($tunes42+5)&4294967295); //@line 1180 "intonation.cpp"
              var $45=HEAPU8[($arrayidx43)]; //@line 1180 "intonation.cpp"
              var $conv44=((($45))&255); //@line 1180 "intonation.cpp"
              $group_tone_emph=$conv44; //@line 1180 "intonation.cpp"
              var $46=$tr_addr; //@line 1181 "intonation.cpp"
              var $langopts45=(($46)&4294967295); //@line 1181 "intonation.cpp"
              var $tunes46=(($langopts45+232)&4294967295); //@line 1181 "intonation.cpp"
              var $arrayidx47=(($tunes46+1)&4294967295); //@line 1181 "intonation.cpp"
              var $47=HEAPU8[($arrayidx47)]; //@line 1181 "intonation.cpp"
              var $conv48=((($47))&255); //@line 1181 "intonation.cpp"
              $group_tone_comma=$conv48; //@line 1181 "intonation.cpp"
              __label__ = 16;break $if_end35_thread$$if_end35$18; //@line 1182 "intonation.cpp"
            }
          } while(0);
          if (__label__ == 15) {
  
            var $48=__lastLabel__ == 12 ? $40 : ($41);
            var $49=$option; //@line 1185 "intonation.cpp"
            var $50=$tr_addr; //@line 1185 "intonation.cpp"
            var $punct_to_tone=(($50+1000)&4294967295); //@line 1185 "intonation.cpp"
            var $arrayidx50=(($punct_to_tone+$49*6)&4294967295); //@line 1185 "intonation.cpp"
            var $arrayidx51=(($arrayidx50+$48)&4294967295); //@line 1185 "intonation.cpp"
            var $51=HEAPU8[($arrayidx51)]; //@line 1185 "intonation.cpp"
            var $conv52=((($51))&255); //@line 1185 "intonation.cpp"
            $group_tone=$conv52; //@line 1185 "intonation.cpp"
            var $52=$option; //@line 1186 "intonation.cpp"
            var $53=$tr_addr; //@line 1186 "intonation.cpp"
            var $punct_to_tone53=(($53+1000)&4294967295); //@line 1186 "intonation.cpp"
            var $arrayidx54=(($punct_to_tone53+$52*6)&4294967295); //@line 1186 "intonation.cpp"
            var $arrayidx55=(($arrayidx54+5)&4294967295); //@line 1186 "intonation.cpp"
            var $54=HEAPU8[($arrayidx55)]; //@line 1186 "intonation.cpp"
            var $conv56=((($54))&255); //@line 1186 "intonation.cpp"
            $group_tone_emph=$conv56; //@line 1186 "intonation.cpp"
            var $55=$option; //@line 1187 "intonation.cpp"
            var $56=$tr_addr; //@line 1187 "intonation.cpp"
            var $punct_to_tone57=(($56+1000)&4294967295); //@line 1187 "intonation.cpp"
            var $arrayidx58=(($punct_to_tone57+$55*6)&4294967295); //@line 1187 "intonation.cpp"
            var $arrayidx59=(($arrayidx58+1)&4294967295); //@line 1187 "intonation.cpp"
            var $57=HEAPU8[($arrayidx59)]; //@line 1187 "intonation.cpp"
            var $conv60=((($57))&255); //@line 1187 "intonation.cpp"
            $group_tone_comma=$conv60; //@line 1187 "intonation.cpp"
            ;
          }
  
          var $58=$clause_type_addr; //@line 1190 "intonation.cpp"
          var $cmp62=((($58))|0)==4; //@line 1190 "intonation.cpp"
          if ($cmp62) { __label__ = 17;; } else { __label__ = 18;; } //@line 1190 "intonation.cpp"
          if (__label__ == 17) {
  
            HEAP8[(__ZL8no_tonic_b)]=1;
            ; //@line 1191 "intonation.cpp"
          }
          else if (__label__ == 18) {
  
            HEAP8[(__ZL8no_tonic_b)]=0;
            ;
          }
  
          $st_start=0; //@line 1195 "intonation.cpp"
          $count_primary=0; //@line 1196 "intonation.cpp"
          $st_ix=0; //@line 1197 "intonation.cpp"
          var $59=$st_ix; //@line 1197 "intonation.cpp"
          var $60=$n_st; //@line 1197 "intonation.cpp"
          var $cmp673=((($59))|0) < ((($60))|0); //@line 1197 "intonation.cpp"
          if ($cmp673) { __label__ = 20;; } else { __label__ = 54;; } //@line 1197 "intonation.cpp"
          $for_body68$$for_end177$29: do { 
            if (__label__ == 20) {
              $for_body68$30: while(1) { 
  
                var $61=$st_ix; //@line 1199 "intonation.cpp"
                var $62=HEAP32[((__ZL12syllable_tab)>>2)]; //@line 1199 "intonation.cpp"
                var $arrayidx69=(($62+6*$61)&4294967295); //@line 1199 "intonation.cpp"
                $syl=$arrayidx69; //@line 1199 "intonation.cpp"
                var $63=$syl; //@line 1201 "intonation.cpp"
                var $stress70=(($63)&4294967295); //@line 1201 "intonation.cpp"
                var $64=HEAP8[($stress70)]; //@line 1201 "intonation.cpp"
                var $conv71=(tempInt=(($64)),(tempInt>=128?tempInt-256:tempInt)); //@line 1201 "intonation.cpp"
                var $cmp72=((($conv71))|0) >= 4; //@line 1201 "intonation.cpp"
                if ($cmp72) { __label__ = 21;; } else { __label__ = 22;; } //@line 1201 "intonation.cpp"
                if (__label__ == 21) {
  
                  var $65=$count_primary; //@line 1202 "intonation.cpp"
                  var $inc74=((($65)+1)&4294967295); //@line 1202 "intonation.cpp"
                  $count_primary=$inc74; //@line 1202 "intonation.cpp"
                  ; //@line 1202 "intonation.cpp"
                }
  
                var $66=$syl; //@line 1204 "intonation.cpp"
                var $stress76=(($66)&4294967295); //@line 1204 "intonation.cpp"
                var $67=HEAP8[($stress76)]; //@line 1204 "intonation.cpp"
                var $conv77=(tempInt=(($67)),(tempInt>=128?tempInt-256:tempInt)); //@line 1204 "intonation.cpp"
                var $cmp78=((($conv77))|0)==6; //@line 1204 "intonation.cpp"
                if ($cmp78) { __label__ = 23;; } else { __label__ = 36;; } //@line 1204 "intonation.cpp"
                $if_then79$$if_end124$35: do { 
                  if (__label__ == 23) {
  
                    var $68=$st_ix; //@line 1207 "intonation.cpp"
                    var $sub80=((($68)-1)&4294967295); //@line 1207 "intonation.cpp"
                    $ix=$sub80; //@line 1207 "intonation.cpp"
                    ; //@line 1207 "intonation.cpp"
                    $for_cond81$37: while(1) { 
  
                      var $69=$ix; //@line 1207 "intonation.cpp"
                      var $70=$st_start; //@line 1207 "intonation.cpp"
                      var $cmp82=((($69))|0) >= ((($70))|0); //@line 1207 "intonation.cpp"
                      if (!($cmp82)) { __label__ = 30;break $for_cond81$37; } //@line 1207 "intonation.cpp"
  
                      var $71=$ix; //@line 1207 "intonation.cpp"
                      var $72=$st_ix; //@line 1207 "intonation.cpp"
                      var $sub83=((($72)-3)&4294967295); //@line 1207 "intonation.cpp"
                      var $cmp84=((($71))|0) >= ((($sub83))|0); //@line 1207 "intonation.cpp"
                      if (!($cmp84)) { __label__ = 30;break $for_cond81$37; }
  
                      var $73=$ix; //@line 1209 "intonation.cpp"
                      var $74=HEAP32[((__ZL12syllable_tab)>>2)]; //@line 1209 "intonation.cpp"
                      var $arrayidx86=(($74+6*$73)&4294967295); //@line 1209 "intonation.cpp"
                      var $stress87=(($arrayidx86)&4294967295); //@line 1209 "intonation.cpp"
                      var $75=HEAP8[($stress87)]; //@line 1209 "intonation.cpp"
                      var $conv88=(tempInt=(($75)),(tempInt>=128?tempInt-256:tempInt)); //@line 1209 "intonation.cpp"
                      var $cmp89=((($conv88))|0)==6; //@line 1209 "intonation.cpp"
                      if ($cmp89) { __label__ = 30;break $for_cond81$37; } //@line 1209 "intonation.cpp"
  
                      var $76=$ix; //@line 1211 "intonation.cpp"
                      var $77=HEAP32[((__ZL12syllable_tab)>>2)]; //@line 1211 "intonation.cpp"
                      var $arrayidx92=(($77+6*$76)&4294967295); //@line 1211 "intonation.cpp"
                      var $stress93=(($arrayidx92)&4294967295); //@line 1211 "intonation.cpp"
                      var $78=HEAP8[($stress93)]; //@line 1211 "intonation.cpp"
                      var $conv94=(tempInt=(($78)),(tempInt>=128?tempInt-256:tempInt)); //@line 1211 "intonation.cpp"
                      var $cmp95=((($conv94))|0)==4; //@line 1211 "intonation.cpp"
                      var $79=$ix; //@line 1213 "intonation.cpp"
                      if ($cmp95) { __label__ = 28;break $for_cond81$37; } //@line 1211 "intonation.cpp"
  
                      var $dec=((($79)-1)&4294967295); //@line 1207 "intonation.cpp"
                      $ix=$dec; //@line 1207 "intonation.cpp"
                      __label__ = 24;continue $for_cond81$37; //@line 1207 "intonation.cpp"
                    }
                    if (__label__ == 28) {
  
                      var $80=HEAP32[((__ZL12syllable_tab)>>2)]; //@line 1213 "intonation.cpp"
                      var $arrayidx97=(($80+6*$79)&4294967295); //@line 1213 "intonation.cpp"
                      var $stress98=(($arrayidx97)&4294967295); //@line 1213 "intonation.cpp"
                      HEAP8[($stress98)]=3; //@line 1213 "intonation.cpp"
                      ; //@line 1214 "intonation.cpp"
                    }
  
                    var $81=$st_ix; //@line 1219 "intonation.cpp"
                    var $add102=((($81)+1)&4294967295); //@line 1219 "intonation.cpp"
                    $ix=$add102; //@line 1219 "intonation.cpp"
                    ; //@line 1219 "intonation.cpp"
                    while(1) { 
  
                      var $82=$ix; //@line 1219 "intonation.cpp"
                      var $83=$n_st; //@line 1219 "intonation.cpp"
                      var $cmp104=((($82))|0) < ((($83))|0); //@line 1219 "intonation.cpp"
                      if (!($cmp104)) { __label__ = 36;break $if_then79$$if_end124$35; } //@line 1219 "intonation.cpp"
  
                      var $84=$ix; //@line 1221 "intonation.cpp"
                      var $85=HEAP32[((__ZL12syllable_tab)>>2)]; //@line 1221 "intonation.cpp"
                      var $arrayidx106=(($85+6*$84)&4294967295); //@line 1221 "intonation.cpp"
                      var $stress107=(($arrayidx106)&4294967295); //@line 1221 "intonation.cpp"
                      var $86=HEAP8[($stress107)]; //@line 1221 "intonation.cpp"
                      var $conv108=(tempInt=(($86)),(tempInt>=128?tempInt-256:tempInt)); //@line 1221 "intonation.cpp"
                      var $cmp109=((($conv108))|0)==4; //@line 1221 "intonation.cpp"
                      if ($cmp109) { __label__ = 36;break $if_then79$$if_end124$35; } //@line 1221 "intonation.cpp"
  
                      var $87=$ix; //@line 1223 "intonation.cpp"
                      var $88=HEAP32[((__ZL12syllable_tab)>>2)]; //@line 1223 "intonation.cpp"
                      var $arrayidx112=(($88+6*$87)&4294967295); //@line 1223 "intonation.cpp"
                      var $stress113=(($arrayidx112)&4294967295); //@line 1223 "intonation.cpp"
                      var $89=HEAP8[($stress113)]; //@line 1223 "intonation.cpp"
                      var $conv114=(tempInt=(($89)),(tempInt>=128?tempInt-256:tempInt)); //@line 1223 "intonation.cpp"
                      var $cmp115=((($conv114))|0)==6; //@line 1223 "intonation.cpp"
                      if ($cmp115) { __label__ = 34;break ; } //@line 1223 "intonation.cpp"
  
                      var $93=$ix; //@line 1219 "intonation.cpp"
                      var $inc122=((($93)+1)&4294967295); //@line 1219 "intonation.cpp"
                      $ix=$inc122; //@line 1219 "intonation.cpp"
                      __label__ = 31;continue ; //@line 1219 "intonation.cpp"
                    }
  
                    var $90=$st_ix; //@line 1226 "intonation.cpp"
                    var $91=HEAP32[((__ZL12syllable_tab)>>2)]; //@line 1226 "intonation.cpp"
                    var $arrayidx117=(($91+6*$90)&4294967295); //@line 1226 "intonation.cpp"
                    var $flags118=(($arrayidx117+2)&4294967295); //@line 1226 "intonation.cpp"
                    HEAP8[($flags118)]=2; //@line 1226 "intonation.cpp"
                    var $92=$syl; //@line 1227 "intonation.cpp"
                    var $stress119=(($92)&4294967295); //@line 1227 "intonation.cpp"
                    HEAP8[($stress119)]=5; //@line 1227 "intonation.cpp"
                    ; //@line 1228 "intonation.cpp"
                  }
                } while(0);
  
                var $94=$syl; //@line 1233 "intonation.cpp"
                var $stress125=(($94)&4294967295); //@line 1233 "intonation.cpp"
                var $95=HEAP8[($stress125)]; //@line 1233 "intonation.cpp"
                var $conv126=(tempInt=(($95)),(tempInt>=128?tempInt-256:tempInt)); //@line 1233 "intonation.cpp"
                var $cmp127=((($conv126))|0)==6; //@line 1233 "intonation.cpp"
                if ($cmp127) { __label__ = 37;; } else { __label__ = 50;; } //@line 1233 "intonation.cpp"
                if (__label__ == 37) {
  
                  var $96=$st_ix; //@line 1236 "intonation.cpp"
                  var $97=HEAP32[((__ZL12syllable_tab)>>2)]; //@line 1236 "intonation.cpp"
                  var $arrayidx129=(($97+6*$96)&4294967295); //@line 1236 "intonation.cpp"
                  var $flags130=(($arrayidx129+2)&4294967295); //@line 1236 "intonation.cpp"
                  HEAP8[($flags130)]=2; //@line 1236 "intonation.cpp"
                  $count=0; //@line 1238 "intonation.cpp"
                  var $98=$n_primary; //@line 1239 "intonation.cpp"
                  var $99=$count_primary; //@line 1239 "intonation.cpp"
                  var $sub131=((($98)-($99))&4294967295); //@line 1239 "intonation.cpp"
                  var $cmp132=((($sub131))|0) > 1; //@line 1239 "intonation.cpp"
                  if ($cmp132) { __label__ = 38;; } else { __label__ = 39;; } //@line 1239 "intonation.cpp"
                  if (__label__ == 38) {
  
                    $count=1; //@line 1240 "intonation.cpp"
                    ; //@line 1240 "intonation.cpp"
                  }
  
                  var $100=$st_ix; //@line 1242 "intonation.cpp"
                  var $add135=((($100)+1)&4294967295); //@line 1242 "intonation.cpp"
                  $ix=$add135; //@line 1242 "intonation.cpp"
                  ; //@line 1242 "intonation.cpp"
                  $for_cond136$58: while(1) { 
  
                    var $101=$ix; //@line 1242 "intonation.cpp"
                    var $102=$n_st; //@line 1242 "intonation.cpp"
                    var $cmp137=((($101))|0) < ((($102))|0); //@line 1242 "intonation.cpp"
                    if (!($cmp137)) { __label__ = 45;break $for_cond136$58; } //@line 1242 "intonation.cpp"
  
                    var $103=$ix; //@line 1244 "intonation.cpp"
                    var $104=HEAP32[((__ZL12syllable_tab)>>2)]; //@line 1244 "intonation.cpp"
                    var $arrayidx139=(($104+6*$103)&4294967295); //@line 1244 "intonation.cpp"
                    var $stress140=(($arrayidx139)&4294967295); //@line 1244 "intonation.cpp"
                    var $105=HEAP8[($stress140)]; //@line 1244 "intonation.cpp"
                    var $conv141=(tempInt=(($105)),(tempInt>=128?tempInt-256:tempInt)); //@line 1244 "intonation.cpp"
                    var $cmp142=((($conv141))|0) > 4; //@line 1244 "intonation.cpp"
                    if ($cmp142) { __label__ = 45;break $for_cond136$58; } //@line 1244 "intonation.cpp"
  
                    var $106=$ix; //@line 1246 "intonation.cpp"
                    var $107=HEAP32[((__ZL12syllable_tab)>>2)]; //@line 1246 "intonation.cpp"
                    var $arrayidx145=(($107+6*$106)&4294967295); //@line 1246 "intonation.cpp"
                    var $stress146=(($arrayidx145)&4294967295); //@line 1246 "intonation.cpp"
                    var $108=HEAP8[($stress146)]; //@line 1246 "intonation.cpp"
                    var $conv147=(tempInt=(($108)),(tempInt>=128?tempInt-256:tempInt)); //@line 1246 "intonation.cpp"
                    var $cmp148=((($conv147))|0)==4; //@line 1246 "intonation.cpp"
                    if ($cmp148) { __label__ = 43;; } else { __label__ = 44;; } //@line 1246 "intonation.cpp"
                    if (__label__ == 43) {
  
                      var $109=$count; //@line 1248 "intonation.cpp"
                      var $inc150=((($109)+1)&4294967295); //@line 1248 "intonation.cpp"
                      $count=$inc150; //@line 1248 "intonation.cpp"
                      var $110=$count; //@line 1249 "intonation.cpp"
                      var $cmp151=((($110))|0) > 1; //@line 1249 "intonation.cpp"
                      if ($cmp151) { __label__ = 45;break $for_cond136$58; } //@line 1249 "intonation.cpp"
                    }
  
                    var $111=$ix; //@line 1242 "intonation.cpp"
                    var $inc156=((($111)+1)&4294967295); //@line 1242 "intonation.cpp"
                    $ix=$inc156; //@line 1242 "intonation.cpp"
                    __label__ = 40;continue $for_cond136$58; //@line 1242 "intonation.cpp"
                  }
  
                  var $112=$st_start; //@line 1254 "intonation.cpp"
                  var $113=$ix; //@line 1254 "intonation.cpp"
                  var $114=$n_st; //@line 1254 "intonation.cpp"
                  __ZL18count_pitch_vowelsiii($112, $113, $114); //@line 1254 "intonation.cpp"
                  var $115=$ix; //@line 1255 "intonation.cpp"
                  var $116=$n_st; //@line 1255 "intonation.cpp"
                  var $cmp158=((($115))|0) < ((($116))|0); //@line 1255 "intonation.cpp"
                  if ($cmp158) { __label__ = 47;; } else { __label__ = 46;; } //@line 1255 "intonation.cpp"
                  $if_then160$$lor_lhs_false$66: do { 
                    if (__label__ == 46) {
  
                      var $117=$clause_type_addr; //@line 1255 "intonation.cpp"
                      var $cmp159=((($117))|0)==0; //@line 1255 "intonation.cpp"
                      if ($cmp159) { __label__ = 47;break $if_then160$$lor_lhs_false$66; } //@line 1255 "intonation.cpp"
  
                      var $122=$option; //@line 1258 "intonation.cpp"
                      var $123=$st_start; //@line 1258 "intonation.cpp"
                      var $124=$ix; //@line 1258 "intonation.cpp"
                      var $125=$group_tone; //@line 1258 "intonation.cpp"
                      __ZL12calc_pitchesiiii($122, $123, $124, $125); //@line 1258 "intonation.cpp"
                      __label__ = 49;break $if_then160$$lor_lhs_false$66;
                    }
                  } while(0);
                  if (__label__ == 47) {
  
                    var $118=$option; //@line 1256 "intonation.cpp"
                    var $119=$st_start; //@line 1256 "intonation.cpp"
                    var $120=$ix; //@line 1256 "intonation.cpp"
                    var $121=$group_tone_emph; //@line 1256 "intonation.cpp"
                    __ZL12calc_pitchesiiii($118, $119, $120, $121); //@line 1256 "intonation.cpp"
                    ; //@line 1256 "intonation.cpp"
                  }
  
                  var $126=$ix; //@line 1260 "intonation.cpp"
                  $st_start=$126; //@line 1260 "intonation.cpp"
                  ; //@line 1261 "intonation.cpp"
                }
  
                var $127=$st_start; //@line 1262 "intonation.cpp"
                var $128=$st_ix; //@line 1262 "intonation.cpp"
                var $cmp165=((($127))|0) < ((($128))|0); //@line 1262 "intonation.cpp"
                if ($cmp165) { __label__ = 51;; } else { __label__ = 53;; } //@line 1262 "intonation.cpp"
                $land_lhs_true166$$for_inc175$73: do { 
                  if (__label__ == 51) {
  
                    var $129=$syl; //@line 1262 "intonation.cpp"
                    var $flags167=(($129+2)&4294967295); //@line 1262 "intonation.cpp"
                    var $130=HEAP8[($flags167)]; //@line 1262 "intonation.cpp"
                    var $conv168=(tempInt=(($130)),(tempInt>=128?tempInt-256:tempInt)); //@line 1262 "intonation.cpp"
                    var $and169=($conv168) & 4; //@line 1262 "intonation.cpp"
                    var $tobool170=((($and169))|0)!=0; //@line 1262 "intonation.cpp"
                    if (!($tobool170)) { __label__ = 53;break $land_lhs_true166$$for_inc175$73; } //@line 1262 "intonation.cpp"
  
                    var $131=$st_ix; //@line 1265 "intonation.cpp"
                    var $add172=((($131)+1)&4294967295); //@line 1265 "intonation.cpp"
                    $st_clause_end=$add172; //@line 1265 "intonation.cpp"
                    var $132=$st_start; //@line 1266 "intonation.cpp"
                    var $133=$st_clause_end; //@line 1266 "intonation.cpp"
                    var $134=$st_clause_end; //@line 1266 "intonation.cpp"
                    __ZL18count_pitch_vowelsiii($132, $133, $134); //@line 1266 "intonation.cpp"
                    var $135=$option; //@line 1267 "intonation.cpp"
                    var $136=$st_start; //@line 1267 "intonation.cpp"
                    var $137=$st_clause_end; //@line 1267 "intonation.cpp"
                    var $138=$group_tone_comma; //@line 1267 "intonation.cpp"
                    __ZL12calc_pitchesiiii($135, $136, $137, $138); //@line 1267 "intonation.cpp"
                    var $139=$st_clause_end; //@line 1268 "intonation.cpp"
                    $st_start=$139; //@line 1268 "intonation.cpp"
                    ; //@line 1269 "intonation.cpp"
                  }
                } while(0);
  
                var $140=$st_ix; //@line 1197 "intonation.cpp"
                var $inc176=((($140)+1)&4294967295); //@line 1197 "intonation.cpp"
                $st_ix=$inc176; //@line 1197 "intonation.cpp"
                var $141=$st_ix; //@line 1197 "intonation.cpp"
                var $142=$n_st; //@line 1197 "intonation.cpp"
                var $cmp67=((($141))|0) < ((($142))|0); //@line 1197 "intonation.cpp"
                if ($cmp67) { __label__ = 20;continue $for_body68$30; } else { __label__ = 54;break $for_body68$$for_end177$29; } //@line 1197 "intonation.cpp"
              }
            }
          } while(0);
  
          var $143=$st_start; //@line 1272 "intonation.cpp"
          var $144=$st_ix; //@line 1272 "intonation.cpp"
          var $cmp178=((($143))|0) < ((($144))|0); //@line 1272 "intonation.cpp"
          if ($cmp178) { __label__ = 55;; } else { __label__ = 56;; } //@line 1272 "intonation.cpp"
          if (__label__ == 55) {
  
            var $145=$st_start; //@line 1274 "intonation.cpp"
            var $146=$st_ix; //@line 1274 "intonation.cpp"
            var $147=$n_st; //@line 1274 "intonation.cpp"
            __ZL18count_pitch_vowelsiii($145, $146, $147); //@line 1274 "intonation.cpp"
            var $148=$option; //@line 1275 "intonation.cpp"
            var $149=$st_start; //@line 1275 "intonation.cpp"
            var $150=$st_ix; //@line 1275 "intonation.cpp"
            var $151=$group_tone; //@line 1275 "intonation.cpp"
            __ZL12calc_pitchesiiii($148, $149, $150, $151); //@line 1275 "intonation.cpp"
            ; //@line 1276 "intonation.cpp"
          }
  
          $st_ix=0; //@line 1280 "intonation.cpp"
          var $152=$ph_start; //@line 1281 "intonation.cpp"
          $ix=$152; //@line 1281 "intonation.cpp"
          var $153=$ix; //@line 1281 "intonation.cpp"
          var $154=$ph_end; //@line 1281 "intonation.cpp"
          var $cmp1832=((($153))|0) < ((($154))|0); //@line 1281 "intonation.cpp"
          if (!($cmp1832)) { __label__ = 70;break $for_end258$$if_end28$13; } //@line 1281 "intonation.cpp"
          $for_body184$81: while(1) { 
  
            var $155=$ix; //@line 1283 "intonation.cpp"
            var $arrayidx185=((_phoneme_list+$155*24)&4294967295); //@line 1283 "intonation.cpp"
            $p=$arrayidx185; //@line 1283 "intonation.cpp"
            var $156=$st_ix; //@line 1284 "intonation.cpp"
            var $157=HEAP32[((__ZL12syllable_tab)>>2)]; //@line 1284 "intonation.cpp"
            var $arrayidx186=(($157+6*$156)&4294967295); //@line 1284 "intonation.cpp"
            var $stress187=(($arrayidx186)&4294967295); //@line 1284 "intonation.cpp"
            var $158=HEAP8[($stress187)]; //@line 1284 "intonation.cpp"
            var $159=$p; //@line 1284 "intonation.cpp"
            var $stresslevel188=(($159+1)&4294967295); //@line 1284 "intonation.cpp"
            HEAP8[($stresslevel188)]=$158; //@line 1284 "intonation.cpp"
            var $160=$p; //@line 1286 "intonation.cpp"
            var $synthflags189=(($160+4)&4294967295); //@line 1286 "intonation.cpp"
            var $161=HEAPU16[(($synthflags189)>>1)]; //@line 1286 "intonation.cpp"
            var $conv190=((($161))&65535); //@line 1286 "intonation.cpp"
            var $and191=($conv190) & 4; //@line 1286 "intonation.cpp"
            var $tobool192=((($and191))|0)!=0; //@line 1286 "intonation.cpp"
            if ($tobool192) { __label__ = 58;; } else { __label__ = 69;; } //@line 1286 "intonation.cpp"
            if (__label__ == 58) {
  
              var $162=$st_ix; //@line 1288 "intonation.cpp"
              var $163=HEAP32[((__ZL12syllable_tab)>>2)]; //@line 1288 "intonation.cpp"
              var $arrayidx194=(($163+6*$162)&4294967295); //@line 1288 "intonation.cpp"
              $syl=$arrayidx194; //@line 1288 "intonation.cpp"
              var $164=$syl; //@line 1290 "intonation.cpp"
              var $pitch1=(($164+4)&4294967295); //@line 1290 "intonation.cpp"
              var $165=HEAP8[($pitch1)]; //@line 1290 "intonation.cpp"
              var $166=$p; //@line 1290 "intonation.cpp"
              var $pitch1195=(($166+20)&4294967295); //@line 1290 "intonation.cpp"
              HEAP8[($pitch1195)]=$165; //@line 1290 "intonation.cpp"
              var $167=$syl; //@line 1291 "intonation.cpp"
              var $pitch2=(($167+5)&4294967295); //@line 1291 "intonation.cpp"
              var $168=HEAP8[($pitch2)]; //@line 1291 "intonation.cpp"
              var $169=$p; //@line 1291 "intonation.cpp"
              var $pitch2196=(($169+21)&4294967295); //@line 1291 "intonation.cpp"
              HEAP8[($pitch2196)]=$168; //@line 1291 "intonation.cpp"
              var $170=$p; //@line 1293 "intonation.cpp"
              var $env197=(($170+14)&4294967295); //@line 1293 "intonation.cpp"
              HEAP8[($env197)]=0; //@line 1293 "intonation.cpp"
              var $171=$syl; //@line 1294 "intonation.cpp"
              var $flags198=(($171+2)&4294967295); //@line 1294 "intonation.cpp"
              var $172=HEAP8[($flags198)]; //@line 1294 "intonation.cpp"
              var $conv199=(tempInt=(($172)),(tempInt>=128?tempInt-256:tempInt)); //@line 1294 "intonation.cpp"
              var $and200=($conv199) & 1; //@line 1294 "intonation.cpp"
              var $tobool201=((($and200))|0)!=0; //@line 1294 "intonation.cpp"
              var $173=$p; //@line 1296 "intonation.cpp"
              if ($tobool201) { __label__ = 59;; } else { __label__ = 60;; } //@line 1294 "intonation.cpp"
              $if_then202$$if_else204$85: do { 
                if (__label__ == 59) {
  
                  var $env203=(($173+14)&4294967295); //@line 1296 "intonation.cpp"
                  HEAP8[($env203)]=2; //@line 1296 "intonation.cpp"
                  ; //@line 1297 "intonation.cpp"
                }
                else if (__label__ == 60) {
  
                  var $stresslevel205=(($173+1)&4294967295); //@line 1299 "intonation.cpp"
                  var $174=HEAPU8[($stresslevel205)]; //@line 1299 "intonation.cpp"
                  var $conv206=((($174))&255); //@line 1299 "intonation.cpp"
                  var $cmp207=((($conv206))|0) > 5; //@line 1299 "intonation.cpp"
                  if (!($cmp207)) { __label__ = 62;break $if_then202$$if_else204$85; } //@line 1299 "intonation.cpp"
  
                  var $175=$syl; //@line 1300 "intonation.cpp"
                  var $env209=(($175+1)&4294967295); //@line 1300 "intonation.cpp"
                  var $176=HEAP8[($env209)]; //@line 1300 "intonation.cpp"
                  var $177=$p; //@line 1300 "intonation.cpp"
                  var $env210=(($177+14)&4294967295); //@line 1300 "intonation.cpp"
                  HEAP8[($env210)]=$176; //@line 1300 "intonation.cpp"
                  ; //@line 1300 "intonation.cpp"
                }
              } while(0);
  
              var $178=$p; //@line 1302 "intonation.cpp"
              var $pitch1213=(($178+20)&4294967295); //@line 1302 "intonation.cpp"
              var $179=HEAPU8[($pitch1213)]; //@line 1302 "intonation.cpp"
              var $conv214=((($179))&255); //@line 1302 "intonation.cpp"
              var $180=$p; //@line 1302 "intonation.cpp"
              var $pitch2215=(($180+21)&4294967295); //@line 1302 "intonation.cpp"
              var $181=HEAPU8[($pitch2215)]; //@line 1302 "intonation.cpp"
              var $conv216=((($181))&255); //@line 1302 "intonation.cpp"
              var $cmp217=((($conv214))|0) > ((($conv216))|0); //@line 1302 "intonation.cpp"
              if ($cmp217) { __label__ = 63;; } else { __label__ = 64;; } //@line 1302 "intonation.cpp"
              if (__label__ == 63) {
  
                var $182=$p; //@line 1305 "intonation.cpp"
                var $pitch1219=(($182+20)&4294967295); //@line 1305 "intonation.cpp"
                var $183=HEAPU8[($pitch1219)]; //@line 1305 "intonation.cpp"
                var $conv220=((($183))&255); //@line 1305 "intonation.cpp"
                $x=$conv220; //@line 1305 "intonation.cpp"
                var $184=$p; //@line 1306 "intonation.cpp"
                var $pitch2221=(($184+21)&4294967295); //@line 1306 "intonation.cpp"
                var $185=HEAP8[($pitch2221)]; //@line 1306 "intonation.cpp"
                var $186=$p; //@line 1306 "intonation.cpp"
                var $pitch1222=(($186+20)&4294967295); //@line 1306 "intonation.cpp"
                HEAP8[($pitch1222)]=$185; //@line 1306 "intonation.cpp"
                var $187=$x; //@line 1307 "intonation.cpp"
                var $conv223=((($187)) & 255); //@line 1307 "intonation.cpp"
                var $188=$p; //@line 1307 "intonation.cpp"
                var $pitch2224=(($188+21)&4294967295); //@line 1307 "intonation.cpp"
                HEAP8[($pitch2224)]=$conv223; //@line 1307 "intonation.cpp"
                ; //@line 1308 "intonation.cpp"
              }
  
              var $189=$p; //@line 1310 "intonation.cpp"
              var $tone_ph=(($189+3)&4294967295); //@line 1310 "intonation.cpp"
              var $190=HEAP8[($tone_ph)]; //@line 1310 "intonation.cpp"
              var $tobool226=(tempInt=(($190)),(tempInt>=128?tempInt-256:tempInt))!=0; //@line 1310 "intonation.cpp"
              if ($tobool226) { __label__ = 65;; } else { __label__ = 66;; } //@line 1310 "intonation.cpp"
              if (__label__ == 65) {
  
                var $191=$p; //@line 1312 "intonation.cpp"
                var $tone_ph228=(($191+3)&4294967295); //@line 1312 "intonation.cpp"
                var $192=HEAPU8[($tone_ph228)]; //@line 1312 "intonation.cpp"
                var $idxprom=((($192))&255); //@line 1312 "intonation.cpp"
                var $arrayidx229=((_phoneme_tab+$idxprom*4)&4294967295); //@line 1312 "intonation.cpp"
                var $193=HEAP32[(($arrayidx229)>>2)]; //@line 1312 "intonation.cpp"
                $ph=$193; //@line 1312 "intonation.cpp"
                var $194=$p; //@line 1313 "intonation.cpp"
                var $pitch1230=(($194+20)&4294967295); //@line 1313 "intonation.cpp"
                var $195=HEAPU8[($pitch1230)]; //@line 1313 "intonation.cpp"
                var $conv231=((($195))&255); //@line 1313 "intonation.cpp"
                var $196=$p; //@line 1313 "intonation.cpp"
                var $pitch2232=(($196+21)&4294967295); //@line 1313 "intonation.cpp"
                var $197=HEAPU8[($pitch2232)]; //@line 1313 "intonation.cpp"
                var $conv233=((($197))&255); //@line 1313 "intonation.cpp"
                var $add234=((($conv233)+($conv231))&4294967295); //@line 1313 "intonation.cpp"
                var $div=((((($add234))|0)/2)|0); //@line 1313 "intonation.cpp"
                $x=$div; //@line 1313 "intonation.cpp"
                var $198=$x; //@line 1314 "intonation.cpp"
                var $199=$ph; //@line 1314 "intonation.cpp"
                var $end_type=(($199+13)&4294967295); //@line 1314 "intonation.cpp"
                var $200=HEAPU8[($end_type)]; //@line 1314 "intonation.cpp"
                var $conv235=((($200))&255); //@line 1314 "intonation.cpp"
                var $add236=((($conv235)+($198))&4294967295); //@line 1314 "intonation.cpp"
                var $conv237=((($add236)) & 255); //@line 1314 "intonation.cpp"
                var $201=$p; //@line 1314 "intonation.cpp"
                var $pitch2238=(($201+21)&4294967295); //@line 1314 "intonation.cpp"
                HEAP8[($pitch2238)]=$conv237; //@line 1314 "intonation.cpp"
                var $202=$x; //@line 1315 "intonation.cpp"
                var $203=$ph; //@line 1315 "intonation.cpp"
                var $start_type=(($203+12)&4294967295); //@line 1315 "intonation.cpp"
                var $204=HEAPU8[($start_type)]; //@line 1315 "intonation.cpp"
                var $conv239=((($204))&255); //@line 1315 "intonation.cpp"
                var $add240=((($conv239)+($202))&4294967295); //@line 1315 "intonation.cpp"
                var $conv241=((($add240)) & 255); //@line 1315 "intonation.cpp"
                var $205=$p; //@line 1315 "intonation.cpp"
                var $pitch1242=(($205+20)&4294967295); //@line 1315 "intonation.cpp"
                HEAP8[($pitch1242)]=$conv241; //@line 1315 "intonation.cpp"
                ; //@line 1316 "intonation.cpp"
              }
  
              var $206=$syl; //@line 1318 "intonation.cpp"
              var $flags244=(($206+2)&4294967295); //@line 1318 "intonation.cpp"
              var $207=HEAP8[($flags244)]; //@line 1318 "intonation.cpp"
              var $conv245=(tempInt=(($207)),(tempInt>=128?tempInt-256:tempInt)); //@line 1318 "intonation.cpp"
              var $and246=($conv245) & 2; //@line 1318 "intonation.cpp"
              var $tobool247=((($and246))|0)!=0; //@line 1318 "intonation.cpp"
              if ($tobool247) { __label__ = 67;; } else { __label__ = 68;; } //@line 1318 "intonation.cpp"
              if (__label__ == 67) {
  
                var $208=$p; //@line 1320 "intonation.cpp"
                var $stresslevel249=(($208+1)&4294967295); //@line 1320 "intonation.cpp"
                var $209=HEAPU8[($stresslevel249)]; //@line 1320 "intonation.cpp"
                var $conv250=((($209))&255); //@line 1320 "intonation.cpp"
                var $or251=($conv250) | 8; //@line 1320 "intonation.cpp"
                var $conv252=((($or251)) & 255); //@line 1320 "intonation.cpp"
                HEAP8[($stresslevel249)]=$conv252; //@line 1320 "intonation.cpp"
                ; //@line 1321 "intonation.cpp"
              }
  
              var $210=$st_ix; //@line 1323 "intonation.cpp"
              var $inc254=((($210)+1)&4294967295); //@line 1323 "intonation.cpp"
              $st_ix=$inc254; //@line 1323 "intonation.cpp"
              ; //@line 1324 "intonation.cpp"
            }
  
            var $211=$ix; //@line 1281 "intonation.cpp"
            var $inc257=((($211)+1)&4294967295); //@line 1281 "intonation.cpp"
            $ix=$inc257; //@line 1281 "intonation.cpp"
            var $212=$ix; //@line 1281 "intonation.cpp"
            var $213=$ph_end; //@line 1281 "intonation.cpp"
            var $cmp183=((($212))|0) < ((($213))|0); //@line 1281 "intonation.cpp"
            if ($cmp183) { __label__ = 57;continue $for_body184$81; } else { __label__ = 70;break $for_end258$$if_end28$13; } //@line 1281 "intonation.cpp"
          }
        }
      }
    } while(0);
  
    STACKTOP = __stackBase__;
    return; //@line 1327 "intonation.cpp"
    return;
  }
  

  function __ZL16CalcPitches_ToneP10Translatori($tr, $clause_tone) {
    ;
    var __label__;
  
    var $tr_addr;
    var $clause_tone_addr;
    var $p;
    var $ix;
    var $count_stressed;
    var $final_stressed;
    var $tone_ph;
    var $pause;
    var $tone_promoted;
    var $tph;
    var $prev_tph;
    var $prevw_tph;
    var $prev2_tph;
    var $prev_p;
    var $pitch_adjust;
    var $pitch_decrement;
    var $pitch_low;
    var $pitch_high;
    $tr_addr=$tr;
    $clause_tone_addr=$clause_tone;
    $count_stressed=0; //@line 935 "intonation.cpp"
    $final_stressed=0; //@line 936 "intonation.cpp"
    $pitch_adjust=0; //@line 947 "intonation.cpp"
    $pitch_decrement=0; //@line 948 "intonation.cpp"
    $pitch_low=0; //@line 949 "intonation.cpp"
    $pitch_high=0; //@line 950 "intonation.cpp"
    $p=((_phoneme_list)&4294967295); //@line 955 "intonation.cpp"
    $ix=0; //@line 956 "intonation.cpp"
    var $0=$ix; //@line 956 "intonation.cpp"
    var $1=HEAP32[((_n_phoneme_list)>>2)]; //@line 956 "intonation.cpp"
    var $cmp5=((($0))|0) < ((($1))|0); //@line 956 "intonation.cpp"
    if ($cmp5) { __label__ = 1;; } else { __label__ = 8;; } //@line 956 "intonation.cpp"
    $for_body$$for_end$2: do { 
      if (__label__ == 1) {
        $for_body$3: while(1) { 
  
          var $2=$p; //@line 958 "intonation.cpp"
          var $type=(($2+15)&4294967295); //@line 958 "intonation.cpp"
          var $3=HEAPU8[($type)]; //@line 958 "intonation.cpp"
          var $conv=((($3))&255); //@line 958 "intonation.cpp"
          var $cmp1=((($conv))|0)==2; //@line 958 "intonation.cpp"
          if ($cmp1) { __label__ = 2;; } else { __label__ = 7;; } //@line 958 "intonation.cpp"
          $land_lhs_true$$for_inc$5: do { 
            if (__label__ == 2) {
  
              var $4=$p; //@line 958 "intonation.cpp"
              var $stresslevel=(($4+1)&4294967295); //@line 958 "intonation.cpp"
              var $5=HEAPU8[($stresslevel)]; //@line 958 "intonation.cpp"
              var $conv2=((($5))&255); //@line 958 "intonation.cpp"
              var $cmp3=((($conv2))|0) >= 4; //@line 958 "intonation.cpp"
              if (!($cmp3)) { __label__ = 7;break $land_lhs_true$$for_inc$5; } //@line 958 "intonation.cpp"
  
              var $6=$count_stressed; //@line 960 "intonation.cpp"
              var $cmp4=((($6))|0)==0; //@line 960 "intonation.cpp"
              if ($cmp4) { __label__ = 4;; } else { __label__ = 5;; } //@line 960 "intonation.cpp"
              if (__label__ == 4) {
  
                var $7=$ix; //@line 961 "intonation.cpp"
                $final_stressed=$7; //@line 961 "intonation.cpp"
                ; //@line 961 "intonation.cpp"
              }
  
              var $8=$p; //@line 963 "intonation.cpp"
              var $stresslevel6=(($8+1)&4294967295); //@line 963 "intonation.cpp"
              var $9=HEAPU8[($stresslevel6)]; //@line 963 "intonation.cpp"
              var $conv7=((($9))&255); //@line 963 "intonation.cpp"
              var $cmp8=((($conv7))|0) >= 4; //@line 963 "intonation.cpp"
              if (!($cmp8)) { __label__ = 7;break $land_lhs_true$$for_inc$5; } //@line 963 "intonation.cpp"
  
              var $10=$ix; //@line 965 "intonation.cpp"
              $final_stressed=$10; //@line 965 "intonation.cpp"
              var $11=$count_stressed; //@line 966 "intonation.cpp"
              var $inc=((($11)+1)&4294967295); //@line 966 "intonation.cpp"
              $count_stressed=$inc; //@line 966 "intonation.cpp"
              ; //@line 967 "intonation.cpp"
            }
          } while(0);
  
          var $12=$ix; //@line 956 "intonation.cpp"
          var $inc12=((($12)+1)&4294967295); //@line 956 "intonation.cpp"
          $ix=$inc12; //@line 956 "intonation.cpp"
          var $13=$p; //@line 956 "intonation.cpp"
          var $incdec_ptr=(($13+24)&4294967295); //@line 956 "intonation.cpp"
          $p=$incdec_ptr; //@line 956 "intonation.cpp"
          var $14=$ix; //@line 956 "intonation.cpp"
          var $15=HEAP32[((_n_phoneme_list)>>2)]; //@line 956 "intonation.cpp"
          var $cmp=((($14))|0) < ((($15))|0); //@line 956 "intonation.cpp"
          if ($cmp) { __label__ = 1;continue $for_body$3; } else { __label__ = 8;break $for_body$$for_end$2; } //@line 956 "intonation.cpp"
        }
      }
    } while(0);
  
    var $16=$final_stressed; //@line 971 "intonation.cpp"
    var $arrayidx=((_phoneme_list+$16*24)&4294967295); //@line 971 "intonation.cpp"
    var $stresslevel13=(($arrayidx+1)&4294967295); //@line 971 "intonation.cpp"
    HEAP8[($stresslevel13)]=7; //@line 971 "intonation.cpp"
    var $17=$tr_addr; //@line 974 "intonation.cpp"
    var $translator_name=(($17+280)&4294967295); //@line 974 "intonation.cpp"
    var $18=HEAP32[(($translator_name)>>2)]; //@line 974 "intonation.cpp"
    var $cmp14=((($18))|0)==30313; //@line 974 "intonation.cpp"
    if ($cmp14) { __label__ = 9;; } else { __label__ = 11;; } //@line 974 "intonation.cpp"
    $if_then15$$if_end24$14: do { 
      if (__label__ == 9) {
  
        var $19=$final_stressed; //@line 977 "intonation.cpp"
        var $arrayidx16=((_phoneme_list+$19*24)&4294967295); //@line 977 "intonation.cpp"
        $p=$arrayidx16; //@line 977 "intonation.cpp"
        var $20=$p; //@line 978 "intonation.cpp"
        var $tone_ph17=(($20+3)&4294967295); //@line 978 "intonation.cpp"
        var $21=HEAPU8[($tone_ph17)]; //@line 978 "intonation.cpp"
        var $conv18=((($21))&255); //@line 978 "intonation.cpp"
        var $cmp19=((($conv18))|0)==0; //@line 978 "intonation.cpp"
        if (!($cmp19)) { __label__ = 11;break $if_then15$$if_end24$14; } //@line 978 "intonation.cpp"
  
        var $call=__Z11PhonemeCodej(55); //@line 979 "intonation.cpp"
        var $conv21=((($call)) & 255); //@line 979 "intonation.cpp"
        var $22=$p; //@line 979 "intonation.cpp"
        var $tone_ph22=(($22+3)&4294967295); //@line 979 "intonation.cpp"
        HEAP8[($tone_ph22)]=$conv21; //@line 979 "intonation.cpp"
        ; //@line 979 "intonation.cpp"
      }
    } while(0);
  
    $pause=1; //@line 983 "intonation.cpp"
    $tone_promoted=0; //@line 984 "intonation.cpp"
    $p=((_phoneme_list)&4294967295); //@line 986 "intonation.cpp"
    $prev_p=((_phoneme_list)&4294967295); //@line 986 "intonation.cpp"
    var $23=HEAP32[((((_phoneme_tab+36)&4294967295))>>2)]; //@line 987 "intonation.cpp"
    $prevw_tph=$23; //@line 987 "intonation.cpp"
    $prev_tph=$23; //@line 987 "intonation.cpp"
    $ix=0; //@line 990 "intonation.cpp"
    var $24=$ix; //@line 990 "intonation.cpp"
    var $25=HEAP32[((_n_phoneme_list)>>2)]; //@line 990 "intonation.cpp"
    var $cmp262=((($24))|0) < ((($25))|0); //@line 990 "intonation.cpp"
    if ($cmp262) { __label__ = 12;; } else { __label__ = 47;; } //@line 990 "intonation.cpp"
    $for_body27$$for_end127$18: do { 
      if (__label__ == 12) {
        $for_body27$19: while(1) { 
  
          var $26=$p; //@line 992 "intonation.cpp"
          var $type28=(($26+15)&4294967295); //@line 992 "intonation.cpp"
          var $27=HEAPU8[($type28)]; //@line 992 "intonation.cpp"
          var $conv29=((($27))&255); //@line 992 "intonation.cpp"
          var $cmp30=((($conv29))|0)==0; //@line 992 "intonation.cpp"
          if ($cmp30) { __label__ = 13;; } else { __label__ = 15;; } //@line 992 "intonation.cpp"
          $land_lhs_true31$$if_end35$21: do { 
            if (__label__ == 13) {
  
              var $28=$p; //@line 992 "intonation.cpp"
              var $ph=(($28+8)&4294967295); //@line 992 "intonation.cpp"
              var $29=HEAP32[(($ph)>>2)]; //@line 992 "intonation.cpp"
              var $std_length=(($29+14)&4294967295); //@line 992 "intonation.cpp"
              var $30=HEAPU8[($std_length)]; //@line 992 "intonation.cpp"
              var $conv32=((($30))&255); //@line 992 "intonation.cpp"
              var $cmp33=((($conv32))|0) > 50; //@line 992 "intonation.cpp"
              if (!($cmp33)) { __label__ = 15;break $land_lhs_true31$$if_end35$21; } //@line 992 "intonation.cpp"
  
              $pause=1; //@line 994 "intonation.cpp"
              var $31=HEAP32[((((_phoneme_tab+36)&4294967295))>>2)]; //@line 995 "intonation.cpp"
              $prevw_tph=$31; //@line 995 "intonation.cpp"
              ; //@line 996 "intonation.cpp"
            }
          } while(0);
  
          var $32=$p; //@line 998 "intonation.cpp"
          var $newword=(($32+19)&4294967295); //@line 998 "intonation.cpp"
          var $33=HEAP8[($newword)]; //@line 998 "intonation.cpp"
          var $tobool=(tempInt=(($33)),(tempInt>=128?tempInt-256:tempInt))!=0; //@line 998 "intonation.cpp"
          if ($tobool) { __label__ = 16;; } else { __label__ = 17;; } //@line 998 "intonation.cpp"
          if (__label__ == 16) {
  
            var $34=HEAP32[((((_phoneme_tab+36)&4294967295))>>2)]; //@line 1000 "intonation.cpp"
            $prev_tph=$34; //@line 1000 "intonation.cpp"
            ; //@line 1001 "intonation.cpp"
          }
  
          var $35=$p; //@line 1003 "intonation.cpp"
          var $synthflags=(($35+4)&4294967295); //@line 1003 "intonation.cpp"
          var $36=HEAPU16[(($synthflags)>>1)]; //@line 1003 "intonation.cpp"
          var $conv38=((($36))&65535); //@line 1003 "intonation.cpp"
          var $and=($conv38) & 4; //@line 1003 "intonation.cpp"
          var $tobool39=((($and))|0)!=0; //@line 1003 "intonation.cpp"
          if ($tobool39) { __label__ = 18;; } else { __label__ = 46;; } //@line 1003 "intonation.cpp"
          if (__label__ == 18) {
  
            var $37=$p; //@line 1005 "intonation.cpp"
            var $tone_ph41=(($37+3)&4294967295); //@line 1005 "intonation.cpp"
            var $38=HEAPU8[($tone_ph41)]; //@line 1005 "intonation.cpp"
            var $conv42=((($38))&255); //@line 1005 "intonation.cpp"
            $tone_ph=$conv42; //@line 1005 "intonation.cpp"
            var $39=$tone_ph; //@line 1006 "intonation.cpp"
            var $arrayidx43=((_phoneme_tab+$39*4)&4294967295); //@line 1006 "intonation.cpp"
            var $40=HEAP32[(($arrayidx43)>>2)]; //@line 1006 "intonation.cpp"
            $tph=$40; //@line 1006 "intonation.cpp"
            var $41=$tr_addr; //@line 1009 "intonation.cpp"
            var $translator_name44=(($41+280)&4294967295); //@line 1009 "intonation.cpp"
            var $42=HEAP32[(($translator_name44)>>2)]; //@line 1009 "intonation.cpp"
            var $cmp45=((($42))|0)==31336; //@line 1009 "intonation.cpp"
            if ($cmp45) { __label__ = 19;; } else { __label__ = 45;; } //@line 1009 "intonation.cpp"
            $if_then46$$if_end122$30: do { 
              if (__label__ == 19) {
  
                var $43=$tone_ph; //@line 1011 "intonation.cpp"
                var $cmp47=((($43))|0)==0; //@line 1011 "intonation.cpp"
                if ($cmp47) { __label__ = 20;; } else { __label__ = 25;; } //@line 1011 "intonation.cpp"
                if (__label__ == 20) {
  
                  var $44=$pause; //@line 1013 "intonation.cpp"
                  var $tobool49=((($44))|0)!=0; //@line 1013 "intonation.cpp"
                  if ($tobool49) { __label__ = 22;; } else { __label__ = 21;; } //@line 1013 "intonation.cpp"
                  $if_then51$$lor_lhs_false$34: do { 
                    if (__label__ == 21) {
  
                      var $45=$tone_promoted; //@line 1013 "intonation.cpp"
                      var $tobool50=((($45))|0)!=0; //@line 1013 "intonation.cpp"
                      if ($tobool50) { __label__ = 22;break $if_then51$$lor_lhs_false$34; } //@line 1013 "intonation.cpp"
  
                      var $call53=__Z11PhonemeCodej(12593); //@line 1020 "intonation.cpp"
                      $tone_ph=$call53; //@line 1020 "intonation.cpp"
                      __label__ = 24;break $if_then51$$lor_lhs_false$34;
                    }
                  } while(0);
                  if (__label__ == 22) {
  
                    var $call52=__Z11PhonemeCodej(13621); //@line 1015 "intonation.cpp"
                    $tone_ph=$call52; //@line 1015 "intonation.cpp"
                    $tone_promoted=1; //@line 1016 "intonation.cpp"
                    ; //@line 1017 "intonation.cpp"
                  }
  
                  var $46=$tone_ph; //@line 1023 "intonation.cpp"
                  var $conv55=((($46)) & 255); //@line 1023 "intonation.cpp"
                  var $47=$p; //@line 1023 "intonation.cpp"
                  var $tone_ph56=(($47+3)&4294967295); //@line 1023 "intonation.cpp"
                  HEAP8[($tone_ph56)]=$conv55; //@line 1023 "intonation.cpp"
                  var $48=$tone_ph; //@line 1024 "intonation.cpp"
                  var $arrayidx57=((_phoneme_tab+$48*4)&4294967295); //@line 1024 "intonation.cpp"
                  var $49=HEAP32[(($arrayidx57)>>2)]; //@line 1024 "intonation.cpp"
                  $tph=$49; //@line 1024 "intonation.cpp"
                  ; //@line 1026 "intonation.cpp"
                }
                else if (__label__ == 25) {
  
                  $tone_promoted=0; //@line 1029 "intonation.cpp"
                  ;
                }
  
                var $50=$ix; //@line 1032 "intonation.cpp"
                var $51=$final_stressed; //@line 1032 "intonation.cpp"
                var $cmp60=((($50))|0)==((($51))|0); //@line 1032 "intonation.cpp"
                if ($cmp60) { __label__ = 27;; } else { __label__ = 30;; } //@line 1032 "intonation.cpp"
                $if_then61$$if_end70$42: do { 
                  if (__label__ == 27) {
  
                    var $52=$tph; //@line 1034 "intonation.cpp"
                    var $mnemonic=(($52)&4294967295); //@line 1034 "intonation.cpp"
                    var $53=HEAP32[(($mnemonic)>>2)]; //@line 1034 "intonation.cpp"
                    var $cmp62=((($53))|0)==13621; //@line 1034 "intonation.cpp"
                    if ($cmp62) { __label__ = 29;; } else { __label__ = 28;; } //@line 1034 "intonation.cpp"
                    if (__label__ == 28) {
  
                      var $54=$tph; //@line 1034 "intonation.cpp"
                      var $mnemonic64=(($54)&4294967295); //@line 1034 "intonation.cpp"
                      var $55=HEAP32[(($mnemonic64)>>2)]; //@line 1034 "intonation.cpp"
                      var $cmp65=((($55))|0)==12597; //@line 1034 "intonation.cpp"
                      if (!($cmp65)) { __label__ = 30;break $if_then61$$if_end70$42; } //@line 1034 "intonation.cpp"
                    }
  
                    var $56=$final_stressed; //@line 1037 "intonation.cpp"
                    var $arrayidx67=((_phoneme_list+$56*24)&4294967295); //@line 1037 "intonation.cpp"
                    var $stresslevel68=(($arrayidx67+1)&4294967295); //@line 1037 "intonation.cpp"
                    HEAP8[($stresslevel68)]=6; //@line 1037 "intonation.cpp"
                    ; //@line 1038 "intonation.cpp"
                  }
                } while(0);
  
                var $57=$prevw_tph; //@line 1041 "intonation.cpp"
                var $mnemonic71=(($57)&4294967295); //@line 1041 "intonation.cpp"
                var $58=HEAP32[(($mnemonic71)>>2)]; //@line 1041 "intonation.cpp"
                var $cmp72=((($58))|0)==3420466; //@line 1041 "intonation.cpp"
                if ($cmp72) { __label__ = 31;; } else { __label__ = 34;; } //@line 1041 "intonation.cpp"
                if (__label__ == 31) {
  
                  var $59=$tph; //@line 1043 "intonation.cpp"
                  var $mnemonic74=(($59)&4294967295); //@line 1043 "intonation.cpp"
                  var $60=HEAP32[(($mnemonic74)>>2)]; //@line 1043 "intonation.cpp"
                  var $cmp75=((($60))|0)==3420466; //@line 1043 "intonation.cpp"
                  if ($cmp75) { __label__ = 32;; } else { __label__ = 33;; } //@line 1043 "intonation.cpp"
                  if (__label__ == 32) {
  
                    var $call77=__Z11PhonemeCodej(13619); //@line 1044 "intonation.cpp"
                    var $conv78=((($call77)) & 255); //@line 1044 "intonation.cpp"
                    var $61=$prev_p; //@line 1044 "intonation.cpp"
                    var $tone_ph79=(($61+3)&4294967295); //@line 1044 "intonation.cpp"
                    HEAP8[($tone_ph79)]=$conv78; //@line 1044 "intonation.cpp"
                    ; //@line 1044 "intonation.cpp"
                  }
                  else if (__label__ == 33) {
  
                    var $call81=__Z11PhonemeCodej(12594); //@line 1046 "intonation.cpp"
                    var $conv82=((($call81)) & 255); //@line 1046 "intonation.cpp"
                    var $62=$prev_p; //@line 1046 "intonation.cpp"
                    var $tone_ph83=(($62+3)&4294967295); //@line 1046 "intonation.cpp"
                    HEAP8[($tone_ph83)]=$conv82; //@line 1046 "intonation.cpp"
                    ;
                  }
                }
  
                var $63=$prev_tph; //@line 1048 "intonation.cpp"
                var $mnemonic86=(($63)&4294967295); //@line 1048 "intonation.cpp"
                var $64=HEAP32[(($mnemonic86)>>2)]; //@line 1048 "intonation.cpp"
                var $cmp87=((($64))|0)==12597; //@line 1048 "intonation.cpp"
                if ($cmp87) { __label__ = 35;; } else { __label__ = 37;; } //@line 1048 "intonation.cpp"
                $land_lhs_true88$$if_end95$54: do { 
                  if (__label__ == 35) {
  
                    var $65=$tph; //@line 1048 "intonation.cpp"
                    var $mnemonic89=(($65)&4294967295); //@line 1048 "intonation.cpp"
                    var $66=HEAP32[(($mnemonic89)>>2)]; //@line 1048 "intonation.cpp"
                    var $cmp90=((($66))|0)==12597; //@line 1048 "intonation.cpp"
                    if (!($cmp90)) { __label__ = 37;break $land_lhs_true88$$if_end95$54; } //@line 1048 "intonation.cpp"
  
                    var $call92=__Z11PhonemeCodej(13109); //@line 1050 "intonation.cpp"
                    var $conv93=((($call92)) & 255); //@line 1050 "intonation.cpp"
                    var $67=$prev_p; //@line 1050 "intonation.cpp"
                    var $tone_ph94=(($67+3)&4294967295); //@line 1050 "intonation.cpp"
                    HEAP8[($tone_ph94)]=$conv93; //@line 1050 "intonation.cpp"
                    ; //@line 1051 "intonation.cpp"
                  }
                } while(0);
  
                var $68=$tph; //@line 1053 "intonation.cpp"
                var $mnemonic96=(($68)&4294967295); //@line 1053 "intonation.cpp"
                var $69=HEAP32[(($mnemonic96)>>2)]; //@line 1053 "intonation.cpp"
                var $cmp97=((($69))|0)==12593; //@line 1053 "intonation.cpp"
                if (!($cmp97)) { __label__ = 45;break $if_then46$$if_end122$30; } //@line 1053 "intonation.cpp"
  
                var $70=$prevw_tph; //@line 1056 "intonation.cpp"
                var $mnemonic99=(($70)&4294967295); //@line 1056 "intonation.cpp"
                var $71=HEAP32[(($mnemonic99)>>2)]; //@line 1056 "intonation.cpp"
                var $cmp100=((($71))|0)==13621; //@line 1056 "intonation.cpp"
                if ($cmp100) { __label__ = 39;; } else { __label__ = 40;; } //@line 1056 "intonation.cpp"
                if (__label__ == 39) {
  
                  var $call102=__Z11PhonemeCodej(12850); //@line 1057 "intonation.cpp"
                  var $conv103=((($call102)) & 255); //@line 1057 "intonation.cpp"
                  var $72=$p; //@line 1057 "intonation.cpp"
                  var $tone_ph104=(($72+3)&4294967295); //@line 1057 "intonation.cpp"
                  HEAP8[($tone_ph104)]=$conv103; //@line 1057 "intonation.cpp"
                  ; //@line 1057 "intonation.cpp"
                }
  
                var $73=$prevw_tph; //@line 1058 "intonation.cpp"
                var $mnemonic106=(($73)&4294967295); //@line 1058 "intonation.cpp"
                var $74=HEAP32[(($mnemonic106)>>2)]; //@line 1058 "intonation.cpp"
                var $cmp107=((($74))|0)==13619; //@line 1058 "intonation.cpp"
                if ($cmp107) { __label__ = 41;; } else { __label__ = 42;; } //@line 1058 "intonation.cpp"
                if (__label__ == 41) {
  
                  var $call109=__Z11PhonemeCodej(13107); //@line 1059 "intonation.cpp"
                  var $conv110=((($call109)) & 255); //@line 1059 "intonation.cpp"
                  var $75=$p; //@line 1059 "intonation.cpp"
                  var $tone_ph111=(($75+3)&4294967295); //@line 1059 "intonation.cpp"
                  HEAP8[($tone_ph111)]=$conv110; //@line 1059 "intonation.cpp"
                  ; //@line 1059 "intonation.cpp"
                }
  
                var $76=$prevw_tph; //@line 1060 "intonation.cpp"
                var $mnemonic113=(($76)&4294967295); //@line 1060 "intonation.cpp"
                var $77=HEAP32[(($mnemonic113)>>2)]; //@line 1060 "intonation.cpp"
                var $cmp114=((($77))|0)==3420466; //@line 1060 "intonation.cpp"
                if ($cmp114) { __label__ = 43;; } else { __label__ = 44;; } //@line 1060 "intonation.cpp"
                if (__label__ == 43) {
  
                  var $call116=__Z11PhonemeCodej(13364); //@line 1061 "intonation.cpp"
                  var $conv117=((($call116)) & 255); //@line 1061 "intonation.cpp"
                  var $78=$p; //@line 1061 "intonation.cpp"
                  var $tone_ph118=(($78+3)&4294967295); //@line 1061 "intonation.cpp"
                  HEAP8[($tone_ph118)]=$conv117; //@line 1061 "intonation.cpp"
                  ; //@line 1061 "intonation.cpp"
                }
  
                var $79=$p; //@line 1064 "intonation.cpp"
                var $stresslevel120=(($79+1)&4294967295); //@line 1064 "intonation.cpp"
                HEAP8[($stresslevel120)]=0; //@line 1064 "intonation.cpp"
                ; //@line 1065 "intonation.cpp"
              }
            } while(0);
  
            var $80=$p; //@line 1068 "intonation.cpp"
            $prev_p=$80; //@line 1068 "intonation.cpp"
            var $81=$prevw_tph; //@line 1069 "intonation.cpp"
            $prev2_tph=$81; //@line 1069 "intonation.cpp"
            var $82=$tph; //@line 1070 "intonation.cpp"
            $prev_tph=$82; //@line 1070 "intonation.cpp"
            $prevw_tph=$82; //@line 1070 "intonation.cpp"
            $pause=0; //@line 1071 "intonation.cpp"
            ; //@line 1072 "intonation.cpp"
          }
  
          var $83=$ix; //@line 990 "intonation.cpp"
          var $inc125=((($83)+1)&4294967295); //@line 990 "intonation.cpp"
          $ix=$inc125; //@line 990 "intonation.cpp"
          var $84=$p; //@line 990 "intonation.cpp"
          var $incdec_ptr126=(($84+24)&4294967295); //@line 990 "intonation.cpp"
          $p=$incdec_ptr126; //@line 990 "intonation.cpp"
          var $85=$ix; //@line 990 "intonation.cpp"
          var $86=HEAP32[((_n_phoneme_list)>>2)]; //@line 990 "intonation.cpp"
          var $cmp26=((($85))|0) < ((($86))|0); //@line 990 "intonation.cpp"
          if ($cmp26) { __label__ = 12;continue $for_body27$19; } else { __label__ = 47;break $for_body27$$for_end127$18; } //@line 990 "intonation.cpp"
        }
      }
    } while(0);
  
    $p=((_phoneme_list)&4294967295); //@line 1076 "intonation.cpp"
    $ix=0; //@line 1077 "intonation.cpp"
    var $87=$ix; //@line 1077 "intonation.cpp"
    var $88=HEAP32[((_n_phoneme_list)>>2)]; //@line 1077 "intonation.cpp"
    var $cmp1291=((($87))|0) < ((($88))|0); //@line 1077 "intonation.cpp"
    if ($cmp1291) { __label__ = 48;; } else { __label__ = 58;; } //@line 1077 "intonation.cpp"
    $for_body130$$for_end166$71: do { 
      if (__label__ == 48) {
        $for_body130$72: while(1) { 
  
          var $89=$p; //@line 1079 "intonation.cpp"
          var $synthflags131=(($89+4)&4294967295); //@line 1079 "intonation.cpp"
          var $90=HEAPU16[(($synthflags131)>>1)]; //@line 1079 "intonation.cpp"
          var $conv132=((($90))&65535); //@line 1079 "intonation.cpp"
          var $and133=($conv132) & 4; //@line 1079 "intonation.cpp"
          var $tobool134=((($and133))|0)!=0; //@line 1079 "intonation.cpp"
          if ($tobool134) { __label__ = 49;; } else { __label__ = 57;; } //@line 1079 "intonation.cpp"
          if (__label__ == 49) {
  
            var $91=$p; //@line 1081 "intonation.cpp"
            var $tone_ph136=(($91+3)&4294967295); //@line 1081 "intonation.cpp"
            var $92=HEAPU8[($tone_ph136)]; //@line 1081 "intonation.cpp"
            var $conv137=((($92))&255); //@line 1081 "intonation.cpp"
            $tone_ph=$conv137; //@line 1081 "intonation.cpp"
            var $93=$p; //@line 1083 "intonation.cpp"
            var $stresslevel138=(($93+1)&4294967295); //@line 1083 "intonation.cpp"
            var $94=HEAPU8[($stresslevel138)]; //@line 1083 "intonation.cpp"
            var $conv139=((($94))&255); //@line 1083 "intonation.cpp"
            var $cmp140=((($conv139))|0)!=0; //@line 1083 "intonation.cpp"
            if ($cmp140) { __label__ = 50;; } else { __label__ = 54;; } //@line 1083 "intonation.cpp"
            $if_then141$$if_end149$76: do { 
              if (__label__ == 50) {
  
                var $95=$ix; //@line 1085 "intonation.cpp"
                var $96=$final_stressed; //@line 1085 "intonation.cpp"
                var $cmp142=((($95))|0)==((($96))|0); //@line 1085 "intonation.cpp"
                if ($cmp142) { __label__ = 51;; } else { __label__ = 52;; } //@line 1085 "intonation.cpp"
                if (__label__ == 51) {
  
                  var $97=$pitch_low; //@line 1088 "intonation.cpp"
                  $pitch_adjust=$97; //@line 1088 "intonation.cpp"
                  ; //@line 1089 "intonation.cpp"
                }
                else if (__label__ == 52) {
  
                  var $98=$pitch_decrement; //@line 1092 "intonation.cpp"
                  var $99=$pitch_adjust; //@line 1092 "intonation.cpp"
                  var $sub=((($99)-($98))&4294967295); //@line 1092 "intonation.cpp"
                  $pitch_adjust=$sub; //@line 1092 "intonation.cpp"
                  var $100=$pitch_adjust; //@line 1093 "intonation.cpp"
                  var $101=$pitch_low; //@line 1093 "intonation.cpp"
                  var $cmp145=((($100))|0) <= ((($101))|0); //@line 1093 "intonation.cpp"
                  if (!($cmp145)) { __label__ = 54;break $if_then141$$if_end149$76; } //@line 1093 "intonation.cpp"
  
                  var $102=$pitch_high; //@line 1094 "intonation.cpp"
                  $pitch_adjust=$102; //@line 1094 "intonation.cpp"
                  ; //@line 1094 "intonation.cpp"
                }
              }
            } while(0);
  
            var $103=$tone_ph; //@line 1098 "intonation.cpp"
            var $cmp150=((($103))|0)==0; //@line 1098 "intonation.cpp"
            if ($cmp150) { __label__ = 55;; } else { __label__ = 56;; } //@line 1098 "intonation.cpp"
            if (__label__ == 55) {
  
              $tone_ph=17; //@line 1100 "intonation.cpp"
              var $104=$tone_ph; //@line 1101 "intonation.cpp"
              var $conv152=((($104)) & 255); //@line 1101 "intonation.cpp"
              var $105=$p; //@line 1101 "intonation.cpp"
              var $tone_ph153=(($105+3)&4294967295); //@line 1101 "intonation.cpp"
              HEAP8[($tone_ph153)]=$conv152; //@line 1101 "intonation.cpp"
              ; //@line 1102 "intonation.cpp"
            }
  
            var $106=$pitch_adjust; //@line 1103 "intonation.cpp"
            var $107=$tone_ph; //@line 1103 "intonation.cpp"
            var $arrayidx155=((_phoneme_tab+$107*4)&4294967295); //@line 1103 "intonation.cpp"
            var $108=HEAP32[(($arrayidx155)>>2)]; //@line 1103 "intonation.cpp"
            var $start_type=(($108+12)&4294967295); //@line 1103 "intonation.cpp"
            var $109=HEAPU8[($start_type)]; //@line 1103 "intonation.cpp"
            var $conv156=((($109))&255); //@line 1103 "intonation.cpp"
            var $add=((($conv156)+($106))&4294967295); //@line 1103 "intonation.cpp"
            var $conv157=((($add)) & 255); //@line 1103 "intonation.cpp"
            var $110=$p; //@line 1103 "intonation.cpp"
            var $pitch1=(($110+20)&4294967295); //@line 1103 "intonation.cpp"
            HEAP8[($pitch1)]=$conv157; //@line 1103 "intonation.cpp"
            var $111=$pitch_adjust; //@line 1104 "intonation.cpp"
            var $112=$tone_ph; //@line 1104 "intonation.cpp"
            var $arrayidx158=((_phoneme_tab+$112*4)&4294967295); //@line 1104 "intonation.cpp"
            var $113=HEAP32[(($arrayidx158)>>2)]; //@line 1104 "intonation.cpp"
            var $end_type=(($113+13)&4294967295); //@line 1104 "intonation.cpp"
            var $114=HEAPU8[($end_type)]; //@line 1104 "intonation.cpp"
            var $conv159=((($114))&255); //@line 1104 "intonation.cpp"
            var $add160=((($conv159)+($111))&4294967295); //@line 1104 "intonation.cpp"
            var $conv161=((($add160)) & 255); //@line 1104 "intonation.cpp"
            var $115=$p; //@line 1104 "intonation.cpp"
            var $pitch2=(($115+21)&4294967295); //@line 1104 "intonation.cpp"
            HEAP8[($pitch2)]=$conv161; //@line 1104 "intonation.cpp"
            ; //@line 1105 "intonation.cpp"
          }
  
          var $116=$ix; //@line 1077 "intonation.cpp"
          var $inc164=((($116)+1)&4294967295); //@line 1077 "intonation.cpp"
          $ix=$inc164; //@line 1077 "intonation.cpp"
          var $117=$p; //@line 1077 "intonation.cpp"
          var $incdec_ptr165=(($117+24)&4294967295); //@line 1077 "intonation.cpp"
          $p=$incdec_ptr165; //@line 1077 "intonation.cpp"
          var $118=$ix; //@line 1077 "intonation.cpp"
          var $119=HEAP32[((_n_phoneme_list)>>2)]; //@line 1077 "intonation.cpp"
          var $cmp129=((($118))|0) < ((($119))|0); //@line 1077 "intonation.cpp"
          if ($cmp129) { __label__ = 48;continue $for_body130$72; } else { __label__ = 58;break $for_body130$$for_end166$71; } //@line 1077 "intonation.cpp"
        }
      }
    } while(0);
  
    ;
    return; //@line 1109 "intonation.cpp"
    return;
  }
  

  function __ZL18count_pitch_vowelsiii($start, $end, $clause_end) {
    ;
    var __label__;
    var __lastLabel__ = null;
  
    var $start_addr;
    var $end_addr;
    var $clause_end_addr;
    var $ix;
    var $stress;
    var $max_stress;
    var $max_stress_posn;
    var $max_stress_posn2;
    $start_addr=$start;
    $end_addr=$end;
    $clause_end_addr=$clause_end;
    $max_stress=0; //@line 333 "intonation.cpp"
    $max_stress_posn=0; //@line 334 "intonation.cpp"
    $max_stress_posn2=0; //@line 335 "intonation.cpp"
    HEAP32[((__ZL10number_pre)>>2)]=-1; //@line 337 "intonation.cpp"
    HEAP32[((__ZL11number_tail)>>2)]=0; //@line 339 "intonation.cpp"
    HEAP32[((__ZL12last_primary)>>2)]=-1; //@line 340 "intonation.cpp"
    var $0=$start_addr; //@line 342 "intonation.cpp"
    $ix=$0; //@line 342 "intonation.cpp"
    var $1=$ix; //@line 342 "intonation.cpp"
    var $2=$end_addr; //@line 342 "intonation.cpp"
    var $cmp1=((($1))|0) < ((($2))|0); //@line 342 "intonation.cpp"
    if ($cmp1) { __label__ = 1;; } else { __label__ = 11;; } //@line 342 "intonation.cpp"
    $for_body$$for_end$2: do { 
      if (__label__ == 1) {
        while(1) { 
  
          var $3=$ix; //@line 344 "intonation.cpp"
          var $4=HEAP32[((__ZL12syllable_tab)>>2)]; //@line 344 "intonation.cpp"
          var $arrayidx=(($4+6*$3)&4294967295); //@line 344 "intonation.cpp"
          var $stress1=(($arrayidx)&4294967295); //@line 344 "intonation.cpp"
          var $5=HEAP8[($stress1)]; //@line 344 "intonation.cpp"
          var $conv=(tempInt=(($5)),(tempInt>=128?tempInt-256:tempInt)); //@line 344 "intonation.cpp"
          $stress=$conv; //@line 344 "intonation.cpp"
          var $6=$stress; //@line 346 "intonation.cpp"
          var $7=$max_stress; //@line 346 "intonation.cpp"
          var $cmp2=((($6))|0) >= ((($7))|0); //@line 346 "intonation.cpp"
          if ($cmp2) { __lastLabel__ = 1; __label__ = 2;; } else { __lastLabel__ = 1; __label__ = 6;; } //@line 346 "intonation.cpp"
          if (__label__ == 2) {
  
            var $8=$stress; //@line 348 "intonation.cpp"
            var $9=$max_stress; //@line 348 "intonation.cpp"
            var $cmp3=((($8))|0) > ((($9))|0); //@line 348 "intonation.cpp"
            if ($cmp3) { __label__ = 3;; } else { __label__ = 4;; } //@line 348 "intonation.cpp"
            if (__label__ == 3) {
  
              var $10=$ix; //@line 350 "intonation.cpp"
              $max_stress_posn2=$10; //@line 350 "intonation.cpp"
              ; //@line 351 "intonation.cpp"
            }
            else if (__label__ == 4) {
  
              var $11=$max_stress_posn; //@line 354 "intonation.cpp"
              $max_stress_posn2=$11; //@line 354 "intonation.cpp"
              ;
            }
  
            var $12=$ix; //@line 356 "intonation.cpp"
            $max_stress_posn=$12; //@line 356 "intonation.cpp"
            var $13=$stress; //@line 357 "intonation.cpp"
            $max_stress=$13; //@line 357 "intonation.cpp"
            __lastLabel__ = 5; ; //@line 358 "intonation.cpp"
          }
  
          var $14=__lastLabel__ == 5 ? $13 : ($6); //@line 359 "intonation.cpp"
          var $cmp6=((($14))|0) >= 4; //@line 359 "intonation.cpp"
          if ($cmp6) { __label__ = 7;; } else { __label__ = 10;; } //@line 359 "intonation.cpp"
          if (__label__ == 7) {
  
            var $15=HEAP32[((__ZL10number_pre)>>2)]; //@line 361 "intonation.cpp"
            var $cmp8=((($15))|0) < 0; //@line 361 "intonation.cpp"
            if ($cmp8) { __label__ = 8;; } else { __label__ = 9;; } //@line 361 "intonation.cpp"
            if (__label__ == 8) {
  
              var $16=$ix; //@line 362 "intonation.cpp"
              var $17=$start_addr; //@line 362 "intonation.cpp"
              var $sub=((($16)-($17))&4294967295); //@line 362 "intonation.cpp"
              HEAP32[((__ZL10number_pre)>>2)]=$sub; //@line 362 "intonation.cpp"
              ; //@line 362 "intonation.cpp"
            }
  
            var $18=$ix; //@line 364 "intonation.cpp"
            HEAP32[((__ZL12last_primary)>>2)]=$18; //@line 364 "intonation.cpp"
            ; //@line 365 "intonation.cpp"
          }
  
          var $19=$ix; //@line 342 "intonation.cpp"
          var $inc=((($19)+1)&4294967295); //@line 342 "intonation.cpp"
          $ix=$inc; //@line 342 "intonation.cpp"
          var $20=$ix; //@line 342 "intonation.cpp"
          var $21=$end_addr; //@line 342 "intonation.cpp"
          var $cmp=((($20))|0) < ((($21))|0); //@line 342 "intonation.cpp"
          if ($cmp) { __label__ = 1;continue ; } else { __label__ = 11;break $for_body$$for_end$2; } //@line 342 "intonation.cpp"
        }
      }
    } while(0);
  
    var $22=HEAP32[((__ZL10number_pre)>>2)]; //@line 369 "intonation.cpp"
    var $cmp12=((($22))|0) < 0; //@line 369 "intonation.cpp"
    if ($cmp12) { __label__ = 12;; } else { __label__ = 13;; } //@line 369 "intonation.cpp"
    if (__label__ == 12) {
  
      var $23=$end_addr; //@line 370 "intonation.cpp"
      HEAP32[((__ZL10number_pre)>>2)]=$23; //@line 370 "intonation.cpp"
      ; //@line 370 "intonation.cpp"
    }
  
    var $24=$end_addr; //@line 372 "intonation.cpp"
    var $25=$max_stress_posn; //@line 372 "intonation.cpp"
    var $_neg=(((-($25)))&4294967295);
    var $sub15=((($24)-1)&4294967295); //@line 372 "intonation.cpp"
    var $sub16=((($sub15)+($_neg))&4294967295); //@line 372 "intonation.cpp"
    HEAP32[((__ZL11number_tail)>>2)]=$sub16; //@line 372 "intonation.cpp"
    var $26=$max_stress_posn; //@line 373 "intonation.cpp"
    HEAP32[((__ZL9tone_posn)>>2)]=$26; //@line 373 "intonation.cpp"
    var $27=$max_stress_posn2; //@line 374 "intonation.cpp"
    HEAP32[((__ZL10tone_posn2)>>2)]=$27; //@line 374 "intonation.cpp"
    var $_b=HEAPU8[(__ZL8no_tonic_b)];
    var $28=((($_b))&1);
    var $tobool=((($28))|0)!=0; //@line 376 "intonation.cpp"
    if ($tobool) { __label__ = 14;; } else { __label__ = 15;; } //@line 376 "intonation.cpp"
    $if_then17$$if_else18$22: do { 
      if (__label__ == 14) {
  
        var $29=$end_addr; //@line 378 "intonation.cpp"
        HEAP32[((__ZL10tone_posn2)>>2)]=$29; //@line 378 "intonation.cpp"
        HEAP32[((__ZL9tone_posn)>>2)]=$29; //@line 378 "intonation.cpp"
        ; //@line 379 "intonation.cpp"
      }
      else if (__label__ == 15) {
  
        var $30=HEAP32[((__ZL12last_primary)>>2)]; //@line 381 "intonation.cpp"
        var $cmp19=((($30))|0) >= 0; //@line 381 "intonation.cpp"
        if ($cmp19) { __label__ = 16;; } else { __label__ = 18;; } //@line 381 "intonation.cpp"
        if (__label__ == 16) {
  
          var $31=$end_addr; //@line 383 "intonation.cpp"
          var $32=$clause_end_addr; //@line 383 "intonation.cpp"
          var $cmp21=((($31))|0)==((($32))|0); //@line 383 "intonation.cpp"
          if (!($cmp21)) { __label__ = 19;break $if_then17$$if_else18$22; } //@line 383 "intonation.cpp"
  
          var $33=HEAP32[((__ZL12last_primary)>>2)]; //@line 385 "intonation.cpp"
          var $34=HEAP32[((__ZL12syllable_tab)>>2)]; //@line 385 "intonation.cpp"
          var $arrayidx23=(($34+6*$33)&4294967295); //@line 385 "intonation.cpp"
          var $stress24=(($arrayidx23)&4294967295); //@line 385 "intonation.cpp"
          HEAP8[($stress24)]=7; //@line 385 "intonation.cpp"
          ; //@line 386 "intonation.cpp"
        }
        else if (__label__ == 18) {
  
          var $35=HEAP32[((__ZL9tone_posn)>>2)]; //@line 391 "intonation.cpp"
          var $36=HEAP32[((__ZL12syllable_tab)>>2)]; //@line 391 "intonation.cpp"
          var $arrayidx27=(($36+6*$35)&4294967295); //@line 391 "intonation.cpp"
          var $stress28=(($arrayidx27)&4294967295); //@line 391 "intonation.cpp"
          HEAP8[($stress28)]=7; //@line 391 "intonation.cpp"
          ;
        }
      }
    } while(0);
  
    ;
    return; //@line 393 "intonation.cpp"
    return;
  }
  

  function __ZL12calc_pitchesiiii($control, $start, $end, $tune_number) {
    ;
    var __label__;
  
    var $retval;
    var $control_addr;
    var $start_addr;
    var $end_addr;
    var $tune_number_addr;
    var $ix;
    var $th;
    var $tn;
    var $drop;
    var $continuing;
    $control_addr=$control;
    $start_addr=$start;
    $end_addr=$end;
    $tune_number_addr=$tune_number;
    $continuing=0; //@line 859 "intonation.cpp"
    var $0=$control_addr; //@line 861 "intonation.cpp"
    var $cmp=((($0))|0)==0; //@line 861 "intonation.cpp"
    var $1=$start_addr; //@line 863 "intonation.cpp"
    if ($cmp) { __label__ = 1;; } else { __label__ = 2;; } //@line 861 "intonation.cpp"
    if (__label__ == 1) {
  
      var $2=$end_addr; //@line 863 "intonation.cpp"
      var $3=$tune_number_addr; //@line 863 "intonation.cpp"
      var $call=__ZL13calc_pitches2iii($1, $2, $3); //@line 863 "intonation.cpp"
      $retval=$call; //@line 863 "intonation.cpp"
      ; //@line 863 "intonation.cpp"
    }
    else if (__label__ == 2) {
  
      var $cmp1=((($1))|0) > 0; //@line 866 "intonation.cpp"
      if ($cmp1) { __label__ = 3;; } else { __label__ = 4;; } //@line 866 "intonation.cpp"
      if (__label__ == 3) {
  
        $continuing=1; //@line 867 "intonation.cpp"
        ; //@line 867 "intonation.cpp"
      }
  
      var $4=$tune_number_addr; //@line 869 "intonation.cpp"
      var $arrayidx=((__ZL15tone_head_table+$4*16)&4294967295); //@line 869 "intonation.cpp"
      $th=$arrayidx; //@line 869 "intonation.cpp"
      var $5=$tune_number_addr; //@line 870 "intonation.cpp"
      var $arrayidx4=((__ZL18tone_nucleus_table+$5*16)&4294967295); //@line 870 "intonation.cpp"
      $tn=$arrayidx4; //@line 870 "intonation.cpp"
      var $6=$start_addr; //@line 871 "intonation.cpp"
      $ix=$6; //@line 871 "intonation.cpp"
      var $7=$ix; //@line 876 "intonation.cpp"
      var $8=$ix; //@line 876 "intonation.cpp"
      var $9=HEAP32[((__ZL10number_pre)>>2)]; //@line 876 "intonation.cpp"
      var $add=((($9)+($8))&4294967295); //@line 876 "intonation.cpp"
      var $10=$th; //@line 876 "intonation.cpp"
      var $pre_start=(($10)&4294967295); //@line 876 "intonation.cpp"
      var $11=HEAPU8[($pre_start)]; //@line 876 "intonation.cpp"
      var $conv=((($11))&255); //@line 876 "intonation.cpp"
      var $12=$th; //@line 876 "intonation.cpp"
      var $pre_end=(($12+1)&4294967295); //@line 876 "intonation.cpp"
      var $13=HEAPU8[($pre_end)]; //@line 876 "intonation.cpp"
      var $conv5=((($13))&255); //@line 876 "intonation.cpp"
      __ZL16SetPitchGradientiiii($7, $add, $conv, $conv5); //@line 876 "intonation.cpp"
      var $14=HEAP32[((__ZL10number_pre)>>2)]; //@line 877 "intonation.cpp"
      var $15=$ix; //@line 877 "intonation.cpp"
      var $add6=((($15)+($14))&4294967295); //@line 877 "intonation.cpp"
      $ix=$add6; //@line 877 "intonation.cpp"
      var $16=HEAP32[((_option_tone_flags)>>2)]; //@line 882 "intonation.cpp"
      var $and=($16) & 512; //@line 882 "intonation.cpp"
      var $tobool=((($and))|0)!=0; //@line 882 "intonation.cpp"
      if ($tobool) { __label__ = 5;; } else { __label__ = 6;; } //@line 882 "intonation.cpp"
      if (__label__ == 5) {
  
        var $17=HEAP32[((__ZL10tone_posn2)>>2)]; //@line 884 "intonation.cpp"
        HEAP32[((__ZL9tone_posn)>>2)]=$17; //@line 884 "intonation.cpp"
        ; //@line 885 "intonation.cpp"
      }
  
      var $18=$ix; //@line 886 "intonation.cpp"
      var $19=HEAP32[((__ZL9tone_posn)>>2)]; //@line 886 "intonation.cpp"
      var $20=$th; //@line 886 "intonation.cpp"
      var $21=$tn; //@line 886 "intonation.cpp"
      var $22=$continuing; //@line 886 "intonation.cpp"
      var $call9=__ZL18calc_pitch_segmentiiP9TONE_HEADP12TONE_NUCLEUSii($18, $19, $20, $21, $22); //@line 886 "intonation.cpp"
      $ix=$call9; //@line 886 "intonation.cpp"
      var $_b=HEAPU8[(__ZL8no_tonic_b)];
      var $23=((($_b))&1);
      var $tobool10=((($23))|0)!=0; //@line 889 "intonation.cpp"
      if ($tobool10) { __label__ = 7;; } else { __label__ = 8;; } //@line 889 "intonation.cpp"
      if (__label__ == 7) {
  
        $retval=0; //@line 890 "intonation.cpp"
        ; //@line 890 "intonation.cpp"
      }
      else if (__label__ == 8) {
  
        var $24=$tn; //@line 895 "intonation.cpp"
        var $flags=(($24+14)&4294967295); //@line 895 "intonation.cpp"
        var $25=HEAPU8[($flags)]; //@line 895 "intonation.cpp"
        var $conv13=((($25))&255); //@line 895 "intonation.cpp"
        var $and14=($conv13) & 1; //@line 895 "intonation.cpp"
        var $tobool15=((($and14))|0)!=0; //@line 895 "intonation.cpp"
        if ($tobool15) { __label__ = 9;; } else { __label__ = 10;; } //@line 895 "intonation.cpp"
        if (__label__ == 9) {
  
          var $26=$ix; //@line 897 "intonation.cpp"
          var $27=HEAP32[((__ZL12syllable_tab)>>2)]; //@line 897 "intonation.cpp"
          var $arrayidx17=(($27+6*$26)&4294967295); //@line 897 "intonation.cpp"
          var $flags18=(($arrayidx17+2)&4294967295); //@line 897 "intonation.cpp"
          var $28=HEAP8[($flags18)]; //@line 897 "intonation.cpp"
          var $conv19=(tempInt=(($28)),(tempInt>=128?tempInt-256:tempInt)); //@line 897 "intonation.cpp"
          var $or=($conv19) | 2; //@line 897 "intonation.cpp"
          var $conv20=((($or)) & 255); //@line 897 "intonation.cpp"
          HEAP8[($flags18)]=$conv20; //@line 897 "intonation.cpp"
          ; //@line 898 "intonation.cpp"
        }
  
        var $29=HEAP32[((__ZL11number_tail)>>2)]; //@line 900 "intonation.cpp"
        var $cmp22=((($29))|0)==0; //@line 900 "intonation.cpp"
        var $30=$tn; //@line 902 "intonation.cpp"
        if ($cmp22) { __label__ = 11;; } else { __label__ = 12;; } //@line 900 "intonation.cpp"
        if (__label__ == 11) {
  
          var $pitch_env0=(($30)&4294967295); //@line 902 "intonation.cpp"
          var $31=HEAPU8[($pitch_env0)]; //@line 902 "intonation.cpp"
          var $conv24=((($31))&255); //@line 902 "intonation.cpp"
          HEAP32[((__ZL14tone_pitch_env)>>2)]=$conv24; //@line 902 "intonation.cpp"
          var $32=$tn; //@line 903 "intonation.cpp"
          var $tonic_max0=(($32+1)&4294967295); //@line 903 "intonation.cpp"
          var $33=HEAPU8[($tonic_max0)]; //@line 903 "intonation.cpp"
          var $conv25=((($33))&255); //@line 903 "intonation.cpp"
          var $34=$tn; //@line 903 "intonation.cpp"
          var $tonic_min0=(($34+2)&4294967295); //@line 903 "intonation.cpp"
          var $35=HEAPU8[($tonic_min0)]; //@line 903 "intonation.cpp"
          var $conv26=((($35))&255); //@line 903 "intonation.cpp"
          var $sub=((($conv25)-($conv26))&4294967295); //@line 903 "intonation.cpp"
          $drop=$sub; //@line 903 "intonation.cpp"
          var $36=$ix; //@line 904 "intonation.cpp"
          var $inc=((($36)+1)&4294967295); //@line 904 "intonation.cpp"
          $ix=$inc; //@line 904 "intonation.cpp"
          var $37=HEAP32[((__ZL12syllable_tab)>>2)]; //@line 904 "intonation.cpp"
          var $arrayidx27=(($37+6*$36)&4294967295); //@line 904 "intonation.cpp"
          var $38=$tn; //@line 904 "intonation.cpp"
          var $tonic_min028=(($38+2)&4294967295); //@line 904 "intonation.cpp"
          var $39=HEAPU8[($tonic_min028)]; //@line 904 "intonation.cpp"
          var $conv29=((($39))&255); //@line 904 "intonation.cpp"
          var $40=$drop; //@line 904 "intonation.cpp"
          __ZL9set_pitchP8SYLLABLEii($arrayidx27, $conv29, $40); //@line 904 "intonation.cpp"
          ; //@line 905 "intonation.cpp"
        }
        else if (__label__ == 12) {
  
          var $pitch_env1=(($30+3)&4294967295); //@line 908 "intonation.cpp"
          var $41=HEAPU8[($pitch_env1)]; //@line 908 "intonation.cpp"
          var $conv30=((($41))&255); //@line 908 "intonation.cpp"
          HEAP32[((__ZL14tone_pitch_env)>>2)]=$conv30; //@line 908 "intonation.cpp"
          var $42=$tn; //@line 909 "intonation.cpp"
          var $tonic_max1=(($42+4)&4294967295); //@line 909 "intonation.cpp"
          var $43=HEAPU8[($tonic_max1)]; //@line 909 "intonation.cpp"
          var $conv31=((($43))&255); //@line 909 "intonation.cpp"
          var $44=$tn; //@line 909 "intonation.cpp"
          var $tonic_min1=(($44+5)&4294967295); //@line 909 "intonation.cpp"
          var $45=HEAPU8[($tonic_min1)]; //@line 909 "intonation.cpp"
          var $conv32=((($45))&255); //@line 909 "intonation.cpp"
          var $sub33=((($conv31)-($conv32))&4294967295); //@line 909 "intonation.cpp"
          $drop=$sub33; //@line 909 "intonation.cpp"
          var $46=$ix; //@line 910 "intonation.cpp"
          var $inc34=((($46)+1)&4294967295); //@line 910 "intonation.cpp"
          $ix=$inc34; //@line 910 "intonation.cpp"
          var $47=HEAP32[((__ZL12syllable_tab)>>2)]; //@line 910 "intonation.cpp"
          var $arrayidx35=(($47+6*$46)&4294967295); //@line 910 "intonation.cpp"
          var $48=$tn; //@line 910 "intonation.cpp"
          var $tonic_min136=(($48+5)&4294967295); //@line 910 "intonation.cpp"
          var $49=HEAPU8[($tonic_min136)]; //@line 910 "intonation.cpp"
          var $conv37=((($49))&255); //@line 910 "intonation.cpp"
          var $50=$drop; //@line 910 "intonation.cpp"
          __ZL9set_pitchP8SYLLABLEii($arrayidx35, $conv37, $50); //@line 910 "intonation.cpp"
          ;
        }
  
        var $51=HEAP32[((__ZL14tone_pitch_env)>>2)]; //@line 913 "intonation.cpp"
        var $conv39=((($51)) & 255); //@line 913 "intonation.cpp"
        var $52=HEAP32[((__ZL9tone_posn)>>2)]; //@line 913 "intonation.cpp"
        var $53=HEAP32[((__ZL12syllable_tab)>>2)]; //@line 913 "intonation.cpp"
        var $arrayidx40=(($53+6*$52)&4294967295); //@line 913 "intonation.cpp"
        var $env=(($arrayidx40+1)&4294967295); //@line 913 "intonation.cpp"
        HEAP8[($env)]=$conv39; //@line 913 "intonation.cpp"
        var $54=HEAP32[((__ZL9tone_posn)>>2)]; //@line 914 "intonation.cpp"
        var $55=HEAP32[((__ZL12syllable_tab)>>2)]; //@line 914 "intonation.cpp"
        var $arrayidx41=(($55+6*$54)&4294967295); //@line 914 "intonation.cpp"
        var $stress=(($arrayidx41)&4294967295); //@line 914 "intonation.cpp"
        var $56=HEAP8[($stress)]; //@line 914 "intonation.cpp"
        var $conv42=(tempInt=(($56)),(tempInt>=128?tempInt-256:tempInt)); //@line 914 "intonation.cpp"
        var $cmp43=((($conv42))|0)==4; //@line 914 "intonation.cpp"
        if ($cmp43) { __label__ = 14;; } else { __label__ = 15;; } //@line 914 "intonation.cpp"
        if (__label__ == 14) {
  
          var $57=HEAP32[((__ZL9tone_posn)>>2)]; //@line 915 "intonation.cpp"
          var $58=HEAP32[((__ZL12syllable_tab)>>2)]; //@line 915 "intonation.cpp"
          var $arrayidx45=(($58+6*$57)&4294967295); //@line 915 "intonation.cpp"
          var $stress46=(($arrayidx45)&4294967295); //@line 915 "intonation.cpp"
          HEAP8[($stress46)]=6; //@line 915 "intonation.cpp"
          ; //@line 915 "intonation.cpp"
        }
  
        var $59=$ix; //@line 920 "intonation.cpp"
        var $60=$end_addr; //@line 920 "intonation.cpp"
        var $61=$tn; //@line 920 "intonation.cpp"
        var $tail_start=(($61+12)&4294967295); //@line 920 "intonation.cpp"
        var $62=HEAPU8[($tail_start)]; //@line 920 "intonation.cpp"
        var $conv48=((($62))&255); //@line 920 "intonation.cpp"
        var $63=$tn; //@line 920 "intonation.cpp"
        var $tail_end=(($63+13)&4294967295); //@line 920 "intonation.cpp"
        var $64=HEAPU8[($tail_end)]; //@line 920 "intonation.cpp"
        var $conv49=((($64))&255); //@line 920 "intonation.cpp"
        __ZL16SetPitchGradientiiii($59, $60, $conv48, $conv49); //@line 920 "intonation.cpp"
        var $65=HEAP32[((__ZL14tone_pitch_env)>>2)]; //@line 922 "intonation.cpp"
        $retval=$65; //@line 922 "intonation.cpp"
        ; //@line 922 "intonation.cpp"
      }
    }
  
    ;
    return;
    return;
  }
  

  function __ZL13calc_pitches2iii($start, $end, $tune_number) {
    ;
    var __label__;
  
    var $retval;
    var $start_addr;
    var $end_addr;
    var $tune_number_addr;
    var $ix;
    var $tune;
    var $drop;
    var $continuing;
    $start_addr=$start;
    $end_addr=$end;
    $tune_number_addr=$tune_number;
    $continuing=0; //@line 790 "intonation.cpp"
    var $0=$start_addr; //@line 792 "intonation.cpp"
    var $cmp=((($0))|0) > 0; //@line 792 "intonation.cpp"
    if ($cmp) { __label__ = 1;; } else { __label__ = 2;; } //@line 792 "intonation.cpp"
    if (__label__ == 1) {
  
      $continuing=1; //@line 793 "intonation.cpp"
      ; //@line 793 "intonation.cpp"
    }
  
    var $1=$tune_number_addr; //@line 795 "intonation.cpp"
    var $2=HEAP32[((_tunes)>>2)]; //@line 795 "intonation.cpp"
    var $arrayidx=(($2+68*$1)&4294967295); //@line 795 "intonation.cpp"
    $tune=$arrayidx; //@line 795 "intonation.cpp"
    var $3=$start_addr; //@line 796 "intonation.cpp"
    $ix=$3; //@line 796 "intonation.cpp"
    var $4=$ix; //@line 801 "intonation.cpp"
    var $5=$ix; //@line 801 "intonation.cpp"
    var $6=HEAP32[((__ZL10number_pre)>>2)]; //@line 801 "intonation.cpp"
    var $add=((($6)+($5))&4294967295); //@line 801 "intonation.cpp"
    var $7=$tune; //@line 801 "intonation.cpp"
    var $prehead_start=(($7+24)&4294967295); //@line 801 "intonation.cpp"
    var $8=HEAPU8[($prehead_start)]; //@line 801 "intonation.cpp"
    var $conv=((($8))&255); //@line 801 "intonation.cpp"
    var $9=$tune; //@line 801 "intonation.cpp"
    var $prehead_end=(($9+25)&4294967295); //@line 801 "intonation.cpp"
    var $10=HEAPU8[($prehead_end)]; //@line 801 "intonation.cpp"
    var $conv1=((($10))&255); //@line 801 "intonation.cpp"
    __ZL16SetPitchGradientiiii($4, $add, $conv, $conv1); //@line 801 "intonation.cpp"
    var $11=HEAP32[((__ZL10number_pre)>>2)]; //@line 802 "intonation.cpp"
    var $12=$ix; //@line 802 "intonation.cpp"
    var $add2=((($12)+($11))&4294967295); //@line 802 "intonation.cpp"
    $ix=$add2; //@line 802 "intonation.cpp"
    var $13=HEAP32[((_option_tone_flags)>>2)]; //@line 807 "intonation.cpp"
    var $and=($13) & 512; //@line 807 "intonation.cpp"
    var $tobool=((($and))|0)!=0; //@line 807 "intonation.cpp"
    if ($tobool) { __label__ = 3;; } else { __label__ = 4;; } //@line 807 "intonation.cpp"
    if (__label__ == 3) {
  
      var $14=HEAP32[((__ZL10tone_posn2)>>2)]; //@line 809 "intonation.cpp"
      HEAP32[((__ZL9tone_posn)>>2)]=$14; //@line 809 "intonation.cpp"
      ; //@line 810 "intonation.cpp"
    }
  
    var $15=$tune; //@line 811 "intonation.cpp"
    var $16=$ix; //@line 811 "intonation.cpp"
    var $17=HEAP32[((__ZL9tone_posn)>>2)]; //@line 811 "intonation.cpp"
    var $call=__ZL17SetHeadIntonationP4TUNEiii($15, $16, $17); //@line 811 "intonation.cpp"
    $ix=$call; //@line 811 "intonation.cpp"
    var $_b=HEAPU8[(__ZL8no_tonic_b)];
    var $18=((($_b))&1);
    var $tobool5=((($18))|0)!=0; //@line 813 "intonation.cpp"
    if ($tobool5) { __label__ = 5;; } else { __label__ = 6;; } //@line 813 "intonation.cpp"
    if (__label__ == 5) {
  
      $retval=0; //@line 814 "intonation.cpp"
      ; //@line 814 "intonation.cpp"
    }
    else if (__label__ == 6) {
  
      var $19=HEAP32[((__ZL11number_tail)>>2)]; //@line 824 "intonation.cpp"
      var $cmp8=((($19))|0)==0; //@line 824 "intonation.cpp"
      var $20=$tune; //@line 826 "intonation.cpp"
      if ($cmp8) { __label__ = 7;; } else { __label__ = 8;; } //@line 824 "intonation.cpp"
      if (__label__ == 7) {
  
        var $nucleus0_env=(($20+42)&4294967295); //@line 826 "intonation.cpp"
        var $21=HEAPU8[($nucleus0_env)]; //@line 826 "intonation.cpp"
        var $conv10=((($21))&255); //@line 826 "intonation.cpp"
        HEAP32[((__ZL14tone_pitch_env)>>2)]=$conv10; //@line 826 "intonation.cpp"
        var $22=$tune; //@line 827 "intonation.cpp"
        var $nucleus0_max=(($22+43)&4294967295); //@line 827 "intonation.cpp"
        var $23=HEAPU8[($nucleus0_max)]; //@line 827 "intonation.cpp"
        var $conv11=((($23))&255); //@line 827 "intonation.cpp"
        var $24=$tune; //@line 827 "intonation.cpp"
        var $nucleus0_min=(($24+44)&4294967295); //@line 827 "intonation.cpp"
        var $25=HEAPU8[($nucleus0_min)]; //@line 827 "intonation.cpp"
        var $conv12=((($25))&255); //@line 827 "intonation.cpp"
        var $sub=((($conv11)-($conv12))&4294967295); //@line 827 "intonation.cpp"
        $drop=$sub; //@line 827 "intonation.cpp"
        var $26=$ix; //@line 828 "intonation.cpp"
        var $inc=((($26)+1)&4294967295); //@line 828 "intonation.cpp"
        $ix=$inc; //@line 828 "intonation.cpp"
        var $27=HEAP32[((__ZL12syllable_tab)>>2)]; //@line 828 "intonation.cpp"
        var $arrayidx13=(($27+6*$26)&4294967295); //@line 828 "intonation.cpp"
        var $28=$tune; //@line 828 "intonation.cpp"
        var $nucleus0_min14=(($28+44)&4294967295); //@line 828 "intonation.cpp"
        var $29=HEAPU8[($nucleus0_min14)]; //@line 828 "intonation.cpp"
        var $conv15=((($29))&255); //@line 828 "intonation.cpp"
        var $30=$drop; //@line 828 "intonation.cpp"
        __ZL9set_pitchP8SYLLABLEii($arrayidx13, $conv15, $30); //@line 828 "intonation.cpp"
        ; //@line 829 "intonation.cpp"
      }
      else if (__label__ == 8) {
  
        var $nucleus1_env=(($20+45)&4294967295); //@line 832 "intonation.cpp"
        var $31=HEAPU8[($nucleus1_env)]; //@line 832 "intonation.cpp"
        var $conv16=((($31))&255); //@line 832 "intonation.cpp"
        HEAP32[((__ZL14tone_pitch_env)>>2)]=$conv16; //@line 832 "intonation.cpp"
        var $32=$tune; //@line 833 "intonation.cpp"
        var $nucleus1_max=(($32+46)&4294967295); //@line 833 "intonation.cpp"
        var $33=HEAPU8[($nucleus1_max)]; //@line 833 "intonation.cpp"
        var $conv17=((($33))&255); //@line 833 "intonation.cpp"
        var $34=$tune; //@line 833 "intonation.cpp"
        var $nucleus1_min=(($34+47)&4294967295); //@line 833 "intonation.cpp"
        var $35=HEAPU8[($nucleus1_min)]; //@line 833 "intonation.cpp"
        var $conv18=((($35))&255); //@line 833 "intonation.cpp"
        var $sub19=((($conv17)-($conv18))&4294967295); //@line 833 "intonation.cpp"
        $drop=$sub19; //@line 833 "intonation.cpp"
        var $36=$ix; //@line 834 "intonation.cpp"
        var $inc20=((($36)+1)&4294967295); //@line 834 "intonation.cpp"
        $ix=$inc20; //@line 834 "intonation.cpp"
        var $37=HEAP32[((__ZL12syllable_tab)>>2)]; //@line 834 "intonation.cpp"
        var $arrayidx21=(($37+6*$36)&4294967295); //@line 834 "intonation.cpp"
        var $38=$tune; //@line 834 "intonation.cpp"
        var $nucleus1_min22=(($38+47)&4294967295); //@line 834 "intonation.cpp"
        var $39=HEAPU8[($nucleus1_min22)]; //@line 834 "intonation.cpp"
        var $conv23=((($39))&255); //@line 834 "intonation.cpp"
        var $40=$drop; //@line 834 "intonation.cpp"
        __ZL9set_pitchP8SYLLABLEii($arrayidx21, $conv23, $40); //@line 834 "intonation.cpp"
        ;
      }
  
      var $41=HEAP32[((__ZL14tone_pitch_env)>>2)]; //@line 837 "intonation.cpp"
      var $conv25=((($41)) & 255); //@line 837 "intonation.cpp"
      var $42=HEAP32[((__ZL9tone_posn)>>2)]; //@line 837 "intonation.cpp"
      var $43=HEAP32[((__ZL12syllable_tab)>>2)]; //@line 837 "intonation.cpp"
      var $arrayidx26=(($43+6*$42)&4294967295); //@line 837 "intonation.cpp"
      var $env=(($arrayidx26+1)&4294967295); //@line 837 "intonation.cpp"
      HEAP8[($env)]=$conv25; //@line 837 "intonation.cpp"
      var $44=HEAP32[((__ZL9tone_posn)>>2)]; //@line 838 "intonation.cpp"
      var $45=HEAP32[((__ZL12syllable_tab)>>2)]; //@line 838 "intonation.cpp"
      var $arrayidx27=(($45+6*$44)&4294967295); //@line 838 "intonation.cpp"
      var $stress=(($arrayidx27)&4294967295); //@line 838 "intonation.cpp"
      var $46=HEAP8[($stress)]; //@line 838 "intonation.cpp"
      var $conv28=(tempInt=(($46)),(tempInt>=128?tempInt-256:tempInt)); //@line 838 "intonation.cpp"
      var $cmp29=((($conv28))|0)==4; //@line 838 "intonation.cpp"
      if ($cmp29) { __label__ = 10;; } else { __label__ = 11;; } //@line 838 "intonation.cpp"
      if (__label__ == 10) {
  
        var $47=HEAP32[((__ZL9tone_posn)>>2)]; //@line 839 "intonation.cpp"
        var $48=HEAP32[((__ZL12syllable_tab)>>2)]; //@line 839 "intonation.cpp"
        var $arrayidx31=(($48+6*$47)&4294967295); //@line 839 "intonation.cpp"
        var $stress32=(($arrayidx31)&4294967295); //@line 839 "intonation.cpp"
        HEAP8[($stress32)]=6; //@line 839 "intonation.cpp"
        ; //@line 839 "intonation.cpp"
      }
  
      var $49=$ix; //@line 844 "intonation.cpp"
      var $50=$end_addr; //@line 844 "intonation.cpp"
      var $51=$tune; //@line 844 "intonation.cpp"
      var $tail_start=(($51+48)&4294967295); //@line 844 "intonation.cpp"
      var $52=HEAPU8[($tail_start)]; //@line 844 "intonation.cpp"
      var $conv34=((($52))&255); //@line 844 "intonation.cpp"
      var $53=$tune; //@line 844 "intonation.cpp"
      var $tail_end=(($53+49)&4294967295); //@line 844 "intonation.cpp"
      var $54=HEAPU8[($tail_end)]; //@line 844 "intonation.cpp"
      var $conv35=((($54))&255); //@line 844 "intonation.cpp"
      __ZL16SetPitchGradientiiii($49, $50, $conv34, $conv35); //@line 844 "intonation.cpp"
      var $55=HEAP32[((__ZL14tone_pitch_env)>>2)]; //@line 846 "intonation.cpp"
      $retval=$55; //@line 846 "intonation.cpp"
      ; //@line 846 "intonation.cpp"
    }
  
    var $56=$retval; //@line 847 "intonation.cpp"
    ;
    return $56; //@line 847 "intonation.cpp"
    return null;
  }
  

  function __ZL16SetPitchGradientiiii($start_ix, $end_ix, $start_pitch, $end_pitch) {
    ;
    var __label__;
    var __lastLabel__ = null;
  
    var $start_ix_addr;
    var $end_ix_addr;
    var $start_pitch_addr;
    var $end_pitch_addr;
    var $ix;
    var $stress;
    var $pitch;
    var $increment;
    var $n_increments;
    var $drop;
    var $syl;
    $start_ix_addr=$start_ix;
    $end_ix_addr=$end_ix;
    $start_pitch_addr=$start_pitch;
    $end_pitch_addr=$end_pitch;
    var $0=$end_pitch_addr; //@line 743 "intonation.cpp"
    var $1=$start_pitch_addr; //@line 743 "intonation.cpp"
    var $sub=((($0)-($1))&4294967295); //@line 743 "intonation.cpp"
    var $shl=($sub) << 8; //@line 743 "intonation.cpp"
    $increment=$shl; //@line 743 "intonation.cpp"
    var $2=$end_ix_addr; //@line 744 "intonation.cpp"
    var $3=$start_ix_addr; //@line 744 "intonation.cpp"
    var $sub1=((($2)-($3))&4294967295); //@line 744 "intonation.cpp"
    $n_increments=$sub1; //@line 744 "intonation.cpp"
    var $4=$n_increments; //@line 746 "intonation.cpp"
    var $cmp=((($4))|0) <= 0; //@line 746 "intonation.cpp"
    if ($cmp) { __label__ = 12;; } else { __label__ = 1;; } //@line 746 "intonation.cpp"
    $for_end$$if_end$2: do { 
      if (__label__ == 1) {
  
        var $5=$n_increments; //@line 749 "intonation.cpp"
        var $cmp2=((($5))|0) > 1; //@line 749 "intonation.cpp"
        if ($cmp2) { __label__ = 2;; } else { __label__ = 3;; } //@line 749 "intonation.cpp"
        if (__label__ == 2) {
  
          var $6=$increment; //@line 751 "intonation.cpp"
          var $7=$n_increments; //@line 751 "intonation.cpp"
          var $div=((((($6))|0)/((($7))|0))|0); //@line 751 "intonation.cpp"
          $increment=$div; //@line 751 "intonation.cpp"
          ; //@line 752 "intonation.cpp"
        }
  
        var $8=$start_pitch_addr; //@line 754 "intonation.cpp"
        var $shl5=($8) << 8; //@line 754 "intonation.cpp"
        $pitch=$shl5; //@line 754 "intonation.cpp"
        var $9=$start_ix_addr; //@line 756 "intonation.cpp"
        $ix=$9; //@line 756 "intonation.cpp"
        var $10=$ix; //@line 756 "intonation.cpp"
        var $11=$end_ix_addr; //@line 756 "intonation.cpp"
        var $cmp61=((($10))|0) < ((($11))|0); //@line 756 "intonation.cpp"
        if (!($cmp61)) { __label__ = 12;break $for_end$$if_end$2; } //@line 756 "intonation.cpp"
        while(1) { 
  
          var $12=$ix; //@line 758 "intonation.cpp"
          var $13=HEAP32[((__ZL12syllable_tab)>>2)]; //@line 758 "intonation.cpp"
          var $arrayidx=(($13+6*$12)&4294967295); //@line 758 "intonation.cpp"
          $syl=$arrayidx; //@line 758 "intonation.cpp"
          var $14=$syl; //@line 759 "intonation.cpp"
          var $stress7=(($14)&4294967295); //@line 759 "intonation.cpp"
          var $15=HEAP8[($stress7)]; //@line 759 "intonation.cpp"
          var $conv=(tempInt=(($15)),(tempInt>=128?tempInt-256:tempInt)); //@line 759 "intonation.cpp"
          $stress=$conv; //@line 759 "intonation.cpp"
          var $16=$increment; //@line 761 "intonation.cpp"
          var $cmp8=((($16))|0) > 0; //@line 761 "intonation.cpp"
          if ($cmp8) { __label__ = 5;; } else { __label__ = 6;; } //@line 761 "intonation.cpp"
          if (__label__ == 5) {
  
            var $17=$syl; //@line 763 "intonation.cpp"
            var $18=$pitch; //@line 763 "intonation.cpp"
            var $shr=($18) >> 8; //@line 763 "intonation.cpp"
            var $19=$increment; //@line 763 "intonation.cpp"
            var $shr10=($19) >> 8; //@line 763 "intonation.cpp"
            var $sub11=(((-($shr10)))&4294967295); //@line 763 "intonation.cpp"
            __ZL9set_pitchP8SYLLABLEii($17, $shr, $sub11); //@line 763 "intonation.cpp"
            var $20=$increment; //@line 764 "intonation.cpp"
            var $21=$pitch; //@line 764 "intonation.cpp"
            var $add=((($21)+($20))&4294967295); //@line 764 "intonation.cpp"
            $pitch=$add; //@line 764 "intonation.cpp"
            ; //@line 765 "intonation.cpp"
          }
          else if (__label__ == 6) {
  
            var $22=$increment; //@line 768 "intonation.cpp"
            var $shr12=($22) >> 8; //@line 768 "intonation.cpp"
            var $sub13=(((-($shr12)))&4294967295); //@line 768 "intonation.cpp"
            $drop=$sub13; //@line 768 "intonation.cpp"
            var $23=$drop; //@line 769 "intonation.cpp"
            var $24=$stress; //@line 769 "intonation.cpp"
            var $arrayidx14=((__ZL8min_drop+$24*4)&4294967295); //@line 769 "intonation.cpp"
            var $25=HEAP32[(($arrayidx14)>>2)]; //@line 769 "intonation.cpp"
            var $cmp15=((($23))|0) < ((($25))|0); //@line 769 "intonation.cpp"
            if ($cmp15) { __lastLabel__ = 6; __label__ = 7;; } else { __lastLabel__ = 6; __label__ = 8;; } //@line 769 "intonation.cpp"
            if (__label__ == 7) {
  
              var $26=$stress; //@line 770 "intonation.cpp"
              var $arrayidx17=((__ZL8min_drop+$26*4)&4294967295); //@line 770 "intonation.cpp"
              var $27=HEAP32[(($arrayidx17)>>2)]; //@line 770 "intonation.cpp"
              $drop=$27; //@line 770 "intonation.cpp"
              __lastLabel__ = 7; ; //@line 770 "intonation.cpp"
            }
  
            var $28=__lastLabel__ == 7 ? $27 : ($23); //@line 774 "intonation.cpp"
            var $29=$increment; //@line 772 "intonation.cpp"
            var $30=$pitch; //@line 772 "intonation.cpp"
            var $add19=((($30)+($29))&4294967295); //@line 772 "intonation.cpp"
            $pitch=$add19; //@line 772 "intonation.cpp"
            var $cmp20=((($28))|0) > 18; //@line 774 "intonation.cpp"
            if ($cmp20) { __label__ = 9;; } else { __label__ = 10;; } //@line 774 "intonation.cpp"
            if (__label__ == 9) {
  
              $drop=18; //@line 775 "intonation.cpp"
              ; //@line 775 "intonation.cpp"
            }
  
            var $31=$syl; //@line 776 "intonation.cpp"
            var $32=$pitch; //@line 776 "intonation.cpp"
            var $shr23=($32) >> 8; //@line 776 "intonation.cpp"
            var $33=$drop; //@line 776 "intonation.cpp"
            __ZL9set_pitchP8SYLLABLEii($31, $shr23, $33); //@line 776 "intonation.cpp"
            ;
          }
  
          var $34=$ix; //@line 756 "intonation.cpp"
          var $inc=((($34)+1)&4294967295); //@line 756 "intonation.cpp"
          $ix=$inc; //@line 756 "intonation.cpp"
          var $35=$ix; //@line 756 "intonation.cpp"
          var $36=$end_ix_addr; //@line 756 "intonation.cpp"
          var $cmp6=((($35))|0) < ((($36))|0); //@line 756 "intonation.cpp"
          if ($cmp6) { __label__ = 4;continue ; } else { __label__ = 12;break $for_end$$if_end$2; } //@line 756 "intonation.cpp"
        }
      }
    } while(0);
  
    ;
    return; //@line 779 "intonation.cpp"
    return;
  }
  

  function __ZL18calc_pitch_segmentiiP9TONE_HEADP12TONE_NUCLEUSii($ix, $end_ix, $th, $tn, $continuing) {
    ;
    var __label__;
    var __lastLabel__ = null;
  
    var $ix_addr;
    var $end_ix_addr;
    var $th_addr;
    var $tn_addr;
    var $min_stress_addr;
    var $continuing_addr;
    var $stress;
    var $pitch;
    var $increment;
    var $n_primary;
    var $n_steps;
    var $initial;
    var $overflow;
    var $n_overflow;
    var $pitch_range;
    var $pitch_range_abs;
    var $drops;
    var $overflow_tab;
    var $syl;
    $ix_addr=$ix;
    $end_ix_addr=$end_ix;
    $th_addr=$th;
    $tn_addr=$tn;
    $min_stress_addr=4;
    $continuing_addr=$continuing;
    $pitch=0; //@line 616 "intonation.cpp"
    $increment=0; //@line 617 "intonation.cpp"
    $n_primary=0; //@line 618 "intonation.cpp"
    $n_steps=0; //@line 619 "intonation.cpp"
    $overflow=0; //@line 621 "intonation.cpp"
    var $0=$th_addr; //@line 631 "intonation.cpp"
    var $body_drops=(($0+4)&4294967295); //@line 631 "intonation.cpp"
    var $1=HEAP32[(($body_drops)>>2)]; //@line 631 "intonation.cpp"
    $drops=$1; //@line 631 "intonation.cpp"
    var $2=$th_addr; //@line 632 "intonation.cpp"
    var $body_end=(($2+3)&4294967295); //@line 632 "intonation.cpp"
    var $3=HEAPU8[($body_end)]; //@line 632 "intonation.cpp"
    var $conv=((($3))&255); //@line 632 "intonation.cpp"
    var $4=$th_addr; //@line 632 "intonation.cpp"
    var $body_start=(($4+2)&4294967295); //@line 632 "intonation.cpp"
    var $5=HEAPU8[($body_start)]; //@line 632 "intonation.cpp"
    var $conv1=((($5))&255); //@line 632 "intonation.cpp"
    var $sub=((($conv)-($conv1))&4294967295); //@line 632 "intonation.cpp"
    var $shl=($sub) << 8; //@line 632 "intonation.cpp"
    $pitch_range=$shl; //@line 632 "intonation.cpp"
    var $6=$pitch_range; //@line 633 "intonation.cpp"
    var $ispos=((($6))|0) > -1; //@line 633 "intonation.cpp"
    var $neg=(((-($6)))&4294967295); //@line 633 "intonation.cpp"
    var $call=($ispos) ? ($6) : ($neg); //@line 633 "intonation.cpp"
    $pitch_range_abs=$call; //@line 633 "intonation.cpp"
    var $7=$continuing_addr; //@line 635 "intonation.cpp"
    var $tobool=((($7))|0)!=0; //@line 635 "intonation.cpp"
    if ($tobool) { __label__ = 1;; } else { __label__ = 2;; } //@line 635 "intonation.cpp"
    if (__label__ == 1) {
  
      $initial=0; //@line 637 "intonation.cpp"
      $overflow=0; //@line 638 "intonation.cpp"
      $n_overflow=5; //@line 639 "intonation.cpp"
      $overflow_tab=((__ZZL18calc_pitch_segmentiiP9TONE_HEADP12TONE_NUCLEUSiiE12continue_tab)&4294967295); //@line 640 "intonation.cpp"
      var $8=$pitch_range; //@line 641 "intonation.cpp"
      var $9=$th_addr; //@line 641 "intonation.cpp"
      var $body_max_steps=(($9+8)&4294967295); //@line 641 "intonation.cpp"
      var $10=HEAPU8[($body_max_steps)]; //@line 641 "intonation.cpp"
      var $conv2=((($10))&255); //@line 641 "intonation.cpp"
      var $sub3=((($conv2)-1)&4294967295); //@line 641 "intonation.cpp"
      var $div=((((($8))|0)/((($sub3))|0))|0); //@line 641 "intonation.cpp"
      $increment=$div; //@line 641 "intonation.cpp"
      ; //@line 642 "intonation.cpp"
    }
    else if (__label__ == 2) {
  
      var $11=$th_addr; //@line 645 "intonation.cpp"
      var $n_overflow4=(($11+10)&4294967295); //@line 645 "intonation.cpp"
      var $12=HEAPU8[($n_overflow4)]; //@line 645 "intonation.cpp"
      var $conv5=((($12))&255); //@line 645 "intonation.cpp"
      $n_overflow=$conv5; //@line 645 "intonation.cpp"
      var $13=$th_addr; //@line 646 "intonation.cpp"
      var $overflow6=(($13+12)&4294967295); //@line 646 "intonation.cpp"
      var $14=HEAP32[(($overflow6)>>2)]; //@line 646 "intonation.cpp"
      $overflow_tab=$14; //@line 646 "intonation.cpp"
      $initial=1; //@line 647 "intonation.cpp"
      ;
    }
  
    var $15=$ix_addr; //@line 650 "intonation.cpp"
    var $16=$end_ix_addr; //@line 650 "intonation.cpp"
    var $cmp4=((($15))|0) < ((($16))|0); //@line 650 "intonation.cpp"
    var $17=$ix_addr; //@line 652 "intonation.cpp"
    if ($cmp4) { __lastLabel__ = 3; __label__ = 4;; } else { __lastLabel__ = 3; __label__ = 31;; } //@line 650 "intonation.cpp"
    $while_body$$while_end$6: do { 
      if (__label__ == 4) {
        $while_body$7: while(1) { 
  
          var $18=__lastLabel__ == 30 ? $90 : ($17);
          var $19=HEAP32[((__ZL12syllable_tab)>>2)]; //@line 652 "intonation.cpp"
          var $arrayidx=(($19+6*$18)&4294967295); //@line 652 "intonation.cpp"
          $syl=$arrayidx; //@line 652 "intonation.cpp"
          var $20=$syl; //@line 653 "intonation.cpp"
          var $stress7=(($20)&4294967295); //@line 653 "intonation.cpp"
          var $21=HEAP8[($stress7)]; //@line 653 "intonation.cpp"
          var $conv8=(tempInt=(($21)),(tempInt>=128?tempInt-256:tempInt)); //@line 653 "intonation.cpp"
          $stress=$conv8; //@line 653 "intonation.cpp"
          var $22=$initial; //@line 658 "intonation.cpp"
          var $tobool9=((($22))|0)!=0; //@line 658 "intonation.cpp"
          if ($tobool9) { __label__ = 8;; } else { __label__ = 5;; } //@line 658 "intonation.cpp"
          $if_then15$$lor_lhs_false$9: do { 
            if (__label__ == 5) {
  
              var $23=$stress; //@line 658 "intonation.cpp"
              var $24=$min_stress_addr; //@line 658 "intonation.cpp"
              var $cmp10=((($23))|0) >= ((($24))|0); //@line 658 "intonation.cpp"
              if ($cmp10) { __lastLabel__ = 5; ; } else { __lastLabel__ = 5; __label__ = 23;break $if_then15$$lor_lhs_false$9; } //@line 658 "intonation.cpp"
  
              var $_pr=$initial; //@line 662 "intonation.cpp"
              var $tobool12=((($_pr))|0)!=0; //@line 662 "intonation.cpp"
              if ($tobool12) { __label__ = 8;break $if_then15$$lor_lhs_false$9; } //@line 662 "intonation.cpp"
  
              var $25=$stress; //@line 662 "intonation.cpp"
              var $cmp14=((($25))|0)==5; //@line 662 "intonation.cpp"
              if ($cmp14) { __label__ = 8;break $if_then15$$lor_lhs_false$9; } //@line 662 "intonation.cpp"
  
              var $39=$n_steps; //@line 682 "intonation.cpp"
              var $cmp34=((($39))|0) > 0; //@line 682 "intonation.cpp"
              if ($cmp34) { __label__ = 16;; } else { __label__ = 17;; } //@line 682 "intonation.cpp"
              if (__label__ == 16) {
  
                var $40=$increment; //@line 683 "intonation.cpp"
                var $41=$pitch; //@line 683 "intonation.cpp"
                var $add=((($41)+($40))&4294967295); //@line 683 "intonation.cpp"
                $pitch=$add; //@line 683 "intonation.cpp"
                __label__ = 19;break $if_then15$$lor_lhs_false$9; //@line 683 "intonation.cpp"
              }
              else if (__label__ == 17) {
  
                var $42=$th_addr; //@line 686 "intonation.cpp"
                var $body_end37=(($42+3)&4294967295); //@line 686 "intonation.cpp"
                var $43=HEAPU8[($body_end37)]; //@line 686 "intonation.cpp"
                var $conv38=((($43))&255); //@line 686 "intonation.cpp"
                var $shl39=((($conv38)*256)&4294967295); //@line 686 "intonation.cpp"
                var $44=$pitch_range_abs; //@line 686 "intonation.cpp"
                var $45=$overflow; //@line 686 "intonation.cpp"
                var $inc=((($45)+1)&4294967295); //@line 686 "intonation.cpp"
                $overflow=$inc; //@line 686 "intonation.cpp"
                var $46=$overflow_tab; //@line 686 "intonation.cpp"
                var $arrayidx40=(($46+$45)&4294967295); //@line 686 "intonation.cpp"
                var $47=HEAP8[($arrayidx40)]; //@line 686 "intonation.cpp"
                var $conv41=(tempInt=(($47)),(tempInt>=128?tempInt-256:tempInt)); //@line 686 "intonation.cpp"
                var $mul=((($conv41)*($44))&4294967295); //@line 686 "intonation.cpp"
                var $div42=((((($mul))|0)/64)|0); //@line 686 "intonation.cpp"
                var $add43=((($div42)+($shl39))&4294967295); //@line 686 "intonation.cpp"
                $pitch=$add43; //@line 686 "intonation.cpp"
                var $48=$overflow; //@line 687 "intonation.cpp"
                var $49=$n_overflow; //@line 687 "intonation.cpp"
                var $cmp44=((($48))|0) >= ((($49))|0); //@line 687 "intonation.cpp"
                if (!($cmp44)) { __label__ = 19;break $if_then15$$lor_lhs_false$9; } //@line 687 "intonation.cpp"
  
                $overflow=0; //@line 689 "intonation.cpp"
                var $50=$th_addr; //@line 690 "intonation.cpp"
                var $overflow46=(($50+12)&4294967295); //@line 690 "intonation.cpp"
                var $51=HEAP32[(($overflow46)>>2)]; //@line 690 "intonation.cpp"
                $overflow_tab=$51; //@line 690 "intonation.cpp"
                __label__ = 19;break $if_then15$$lor_lhs_false$9; //@line 691 "intonation.cpp"
              }
            }
          } while(0);
          $if_then15$$if_end49$$if_end59$18: do { 
            if (__label__ == 8) {
  
              $initial=0; //@line 664 "intonation.cpp"
              $overflow=0; //@line 665 "intonation.cpp"
              var $26=$ix_addr; //@line 666 "intonation.cpp"
              var $27=$end_ix_addr; //@line 666 "intonation.cpp"
              var $28=$min_stress_addr; //@line 666 "intonation.cpp"
              var $call16=__ZL16count_incrementsiii($26, $27, $28); //@line 666 "intonation.cpp"
              $n_primary=$call16; //@line 666 "intonation.cpp"
              $n_steps=$call16; //@line 666 "intonation.cpp"
              var $29=$n_steps; //@line 668 "intonation.cpp"
              var $30=$th_addr; //@line 668 "intonation.cpp"
              var $body_max_steps17=(($30+8)&4294967295); //@line 668 "intonation.cpp"
              var $31=HEAPU8[($body_max_steps17)]; //@line 668 "intonation.cpp"
              var $conv18=((($31))&255); //@line 668 "intonation.cpp"
              var $cmp19=((($29))|0) > ((($conv18))|0); //@line 668 "intonation.cpp"
              if ($cmp19) { __label__ = 9;; } else { __label__ = 10;; } //@line 668 "intonation.cpp"
              if (__label__ == 9) {
  
                var $32=$th_addr; //@line 669 "intonation.cpp"
                var $body_max_steps21=(($32+8)&4294967295); //@line 669 "intonation.cpp"
                var $33=HEAPU8[($body_max_steps21)]; //@line 669 "intonation.cpp"
                var $conv22=((($33))&255); //@line 669 "intonation.cpp"
                $n_steps=$conv22; //@line 669 "intonation.cpp"
                __lastLabel__ = 9; ; //@line 669 "intonation.cpp"
              }
              else if (__label__ == 10) {
  
                var $_pr2=$n_steps; //@line 671 "intonation.cpp"
                __lastLabel__ = 10; ;
              }
  
              var $34=__lastLabel__ == 10 ? $_pr2 : ($conv22); //@line 671 "intonation.cpp"
              var $cmp24=((($34))|0) > 1; //@line 671 "intonation.cpp"
              if ($cmp24) { __label__ = 12;; } else { __label__ = 13;; } //@line 671 "intonation.cpp"
              if (__label__ == 12) {
  
                var $35=$pitch_range; //@line 673 "intonation.cpp"
                var $36=$n_steps; //@line 673 "intonation.cpp"
                var $sub26=((($36)-1)&4294967295); //@line 673 "intonation.cpp"
                var $div27=((((($35))|0)/((($sub26))|0))|0); //@line 673 "intonation.cpp"
                $increment=$div27; //@line 673 "intonation.cpp"
                ; //@line 674 "intonation.cpp"
              }
              else if (__label__ == 13) {
  
                $increment=0; //@line 676 "intonation.cpp"
                ;
              }
  
              var $37=$th_addr; //@line 678 "intonation.cpp"
              var $body_start30=(($37+2)&4294967295); //@line 678 "intonation.cpp"
              var $38=HEAPU8[($body_start30)]; //@line 678 "intonation.cpp"
              var $conv31=((($38))&255); //@line 678 "intonation.cpp"
              var $shl32=($conv31) << 8; //@line 678 "intonation.cpp"
              $pitch=$shl32; //@line 678 "intonation.cpp"
              __label__ = 19;break $if_then15$$if_end49$$if_end59$18; //@line 679 "intonation.cpp"
            }
          } while(0);
          if (__label__ == 19) {
  
            var $52=$n_steps; //@line 695 "intonation.cpp"
            var $dec=((($52)-1)&4294967295); //@line 695 "intonation.cpp"
            $n_steps=$dec; //@line 695 "intonation.cpp"
            var $53=$n_primary; //@line 697 "intonation.cpp"
            var $dec50=((($53)-1)&4294967295); //@line 697 "intonation.cpp"
            $n_primary=$dec50; //@line 697 "intonation.cpp"
            var $54=$tn_addr; //@line 698 "intonation.cpp"
            var $backwards=(($54+8)&4294967295); //@line 698 "intonation.cpp"
            var $55=HEAP32[(($backwards)>>2)]; //@line 698 "intonation.cpp"
            var $tobool51=((($55))|0)!=0; //@line 698 "intonation.cpp"
            if ($tobool51) { __label__ = 20;; } else { __label__ = 22;; } //@line 698 "intonation.cpp"
            $land_lhs_true$$if_end59thread_pre_split$30: do { 
              if (__label__ == 20) {
  
                var $56=$n_primary; //@line 698 "intonation.cpp"
                var $cmp52=((($56))|0) < 2; //@line 698 "intonation.cpp"
                if (!($cmp52)) { __label__ = 22;break $land_lhs_true$$if_end59thread_pre_split$30; } //@line 698 "intonation.cpp"
  
                var $57=$n_primary; //@line 700 "intonation.cpp"
                var $58=$tn_addr; //@line 700 "intonation.cpp"
                var $backwards54=(($58+8)&4294967295); //@line 700 "intonation.cpp"
                var $59=HEAP32[(($backwards54)>>2)]; //@line 700 "intonation.cpp"
                var $arrayidx55=(($59+2*$57)&4294967295); //@line 700 "intonation.cpp"
                var $60=HEAP16[(($arrayidx55)>>1)]; //@line 700 "intonation.cpp"
                var $conv56=(tempInt=(($60)),(tempInt>=32768?tempInt-65536:tempInt)); //@line 700 "intonation.cpp"
                var $shl57=($conv56) << 8; //@line 700 "intonation.cpp"
                $pitch=$shl57; //@line 700 "intonation.cpp"
                ; //@line 701 "intonation.cpp"
              }
            } while(0);
  
            var $_pr3=$stress; //@line 704 "intonation.cpp"
            __lastLabel__ = 22; ;
          }
  
          var $61=__lastLabel__ == 22 ? $_pr3 : ($23); //@line 704 "intonation.cpp"
          var $cmp60=((($61))|0) >= 4; //@line 704 "intonation.cpp"
          if ($cmp60) { __label__ = 24;; } else { __label__ = 25;; } //@line 704 "intonation.cpp"
          if (__label__ == 24) {
  
            var $62=$syl; //@line 706 "intonation.cpp"
            var $stress62=(($62)&4294967295); //@line 706 "intonation.cpp"
            HEAP8[($stress62)]=6; //@line 706 "intonation.cpp"
            var $63=$syl; //@line 707 "intonation.cpp"
            var $64=$pitch; //@line 707 "intonation.cpp"
            var $shr=($64) >> 8; //@line 707 "intonation.cpp"
            var $65=$stress; //@line 707 "intonation.cpp"
            var $66=$drops; //@line 707 "intonation.cpp"
            var $arrayidx63=(($66+4*$65)&4294967295); //@line 707 "intonation.cpp"
            var $67=HEAP32[(($arrayidx63)>>2)]; //@line 707 "intonation.cpp"
            __ZL9set_pitchP8SYLLABLEii($63, $shr, $67); //@line 707 "intonation.cpp"
            ; //@line 708 "intonation.cpp"
          }
          else if (__label__ == 25) {
  
            var $68=$stress; //@line 710 "intonation.cpp"
            var $cmp65=((($68))|0) >= 3; //@line 710 "intonation.cpp"
            if ($cmp65) { __label__ = 26;; } else { __label__ = 27;; } //@line 710 "intonation.cpp"
            if (__label__ == 26) {
  
              var $69=$syl; //@line 712 "intonation.cpp"
              var $70=$pitch; //@line 712 "intonation.cpp"
              var $shr67=($70) >> 8; //@line 712 "intonation.cpp"
              var $71=$stress; //@line 712 "intonation.cpp"
              var $72=$drops; //@line 712 "intonation.cpp"
              var $arrayidx68=(($72+4*$71)&4294967295); //@line 712 "intonation.cpp"
              var $73=HEAP32[(($arrayidx68)>>2)]; //@line 712 "intonation.cpp"
              __ZL9set_pitchP8SYLLABLEii($69, $shr67, $73); //@line 712 "intonation.cpp"
              ; //@line 713 "intonation.cpp"
            }
            else if (__label__ == 27) {
  
              var $74=$ix_addr; //@line 717 "intonation.cpp"
              var $sub70=((($74)-1)&4294967295); //@line 717 "intonation.cpp"
              var $75=HEAP32[((__ZL12syllable_tab)>>2)]; //@line 717 "intonation.cpp"
              var $arrayidx71=(($75+6*$sub70)&4294967295); //@line 717 "intonation.cpp"
              var $stress72=(($arrayidx71)&4294967295); //@line 717 "intonation.cpp"
              var $76=HEAP8[($stress72)]; //@line 717 "intonation.cpp"
              var $conv73=(tempInt=(($76)),(tempInt>=128?tempInt-256:tempInt)); //@line 717 "intonation.cpp"
              var $and=($conv73) & 63; //@line 717 "intonation.cpp"
              var $cmp74=((($and))|0) >= 3; //@line 717 "intonation.cpp"
              var $77=$syl; //@line 718 "intonation.cpp"
              var $78=$pitch; //@line 718 "intonation.cpp"
              var $shr76=($78) >> 8; //@line 718 "intonation.cpp"
              if ($cmp74) { __label__ = 28;; } else { __label__ = 29;; } //@line 717 "intonation.cpp"
              if (__label__ == 28) {
  
                var $79=$th_addr; //@line 718 "intonation.cpp"
                var $body_lower_u=(($79+9)&4294967295); //@line 718 "intonation.cpp"
                var $80=HEAP8[($body_lower_u)]; //@line 718 "intonation.cpp"
                var $conv77=(tempInt=(($80)),(tempInt>=128?tempInt-256:tempInt)); //@line 718 "intonation.cpp"
                var $sub78=((($shr76)-($conv77))&4294967295); //@line 718 "intonation.cpp"
                var $81=$stress; //@line 718 "intonation.cpp"
                var $82=$drops; //@line 718 "intonation.cpp"
                var $arrayidx79=(($82+4*$81)&4294967295); //@line 718 "intonation.cpp"
                var $83=HEAP32[(($arrayidx79)>>2)]; //@line 718 "intonation.cpp"
                __ZL9set_pitchP8SYLLABLEii($77, $sub78, $83); //@line 718 "intonation.cpp"
                ; //@line 718 "intonation.cpp"
              }
              else if (__label__ == 29) {
  
                var $84=$stress; //@line 720 "intonation.cpp"
                var $85=$drops; //@line 720 "intonation.cpp"
                var $arrayidx82=(($85+4*$84)&4294967295); //@line 720 "intonation.cpp"
                var $86=HEAP32[(($arrayidx82)>>2)]; //@line 720 "intonation.cpp"
                __ZL9set_pitchP8SYLLABLEii($77, $shr76, $86); //@line 720 "intonation.cpp"
                ;
              }
            }
          }
  
          var $87=$ix_addr; //@line 723 "intonation.cpp"
          var $inc86=((($87)+1)&4294967295); //@line 723 "intonation.cpp"
          $ix_addr=$inc86; //@line 723 "intonation.cpp"
          var $88=$ix_addr; //@line 650 "intonation.cpp"
          var $89=$end_ix_addr; //@line 650 "intonation.cpp"
          var $cmp=((($88))|0) < ((($89))|0); //@line 650 "intonation.cpp"
          var $90=$ix_addr; //@line 652 "intonation.cpp"
          if ($cmp) { __lastLabel__ = 30; __label__ = 4;continue $while_body$7; } else { __lastLabel__ = 30; __label__ = 31;break $while_body$$while_end$6; } //@line 650 "intonation.cpp"
        }
      }
    } while(0);
  
    var $_lcssa=__lastLabel__ == 3 ? $17 : ($90);
    ;
    return $_lcssa; //@line 725 "intonation.cpp"
    return null;
  }
  

  function __ZL9set_pitchP8SYLLABLEii($syl, $base, $drop) {
    ;
    var __label__;
    var __lastLabel__ = null;
  
    var $syl_addr;
    var $base_addr;
    var $drop_addr;
    var $pitch1;
    var $pitch2;
    var $flags;
    $syl_addr=$syl;
    $base_addr=$base;
    $drop_addr=$drop;
    $flags=0; //@line 425 "intonation.cpp"
    var $0=$base_addr; //@line 427 "intonation.cpp"
    var $cmp=((($0))|0) < 0; //@line 427 "intonation.cpp"
    if ($cmp) { __lastLabel__ = 0; __label__ = 1;; } else { __lastLabel__ = 0; __label__ = 2;; } //@line 427 "intonation.cpp"
    if (__label__ == 1) {
  
      $base_addr=0; //@line 427 "intonation.cpp"
      var $_pr=$drop_addr; //@line 431 "intonation.cpp"
      __lastLabel__ = 1; ; //@line 427 "intonation.cpp"
    }
  
    var $1=__lastLabel__ == 1 ? $_pr : ($drop); //@line 431 "intonation.cpp"
    var $2=$base_addr; //@line 429 "intonation.cpp"
    $pitch2=$2; //@line 429 "intonation.cpp"
    var $cmp1=((($1))|0) < 0; //@line 431 "intonation.cpp"
    if ($cmp1) { __label__ = 3;; } else { __label__ = 4;; } //@line 431 "intonation.cpp"
    if (__label__ == 3) {
  
      $flags=1; //@line 433 "intonation.cpp"
      var $3=$drop_addr; //@line 434 "intonation.cpp"
      var $sub=(((-($3)))&4294967295); //@line 434 "intonation.cpp"
      $drop_addr=$sub; //@line 434 "intonation.cpp"
      ; //@line 435 "intonation.cpp"
    }
  
    var $4=$pitch2; //@line 437 "intonation.cpp"
    var $5=$drop_addr; //@line 437 "intonation.cpp"
    var $add=((($5)+($4))&4294967295); //@line 437 "intonation.cpp"
    $pitch1=$add; //@line 437 "intonation.cpp"
    var $cmp4=((($add))|0) < 0; //@line 438 "intonation.cpp"
    if ($cmp4) { __label__ = 5;; } else { __label__ = 6;; } //@line 438 "intonation.cpp"
    $if_end6_thread$$if_end6$8: do { 
      if (__label__ == 5) {
  
        $pitch1=0; //@line 439 "intonation.cpp"
        ; //@line 441 "intonation.cpp"
      }
      else if (__label__ == 6) {
  
        var $cmp7=((($add))|0) > 254; //@line 441 "intonation.cpp"
        if (!($cmp7)) { __label__ = 8;break $if_end6_thread$$if_end6$8; } //@line 441 "intonation.cpp"
  
        $pitch1=254; //@line 441 "intonation.cpp"
        ; //@line 441 "intonation.cpp"
      }
    } while(0);
  
    var $6=$pitch2; //@line 442 "intonation.cpp"
    var $cmp10=((($6))|0) > 254; //@line 442 "intonation.cpp"
    if ($cmp10) { __label__ = 9;; } else { __label__ = 10;; } //@line 442 "intonation.cpp"
    if (__label__ == 9) {
  
      $pitch2=254; //@line 442 "intonation.cpp"
      ; //@line 442 "intonation.cpp"
    }
  
    var $7=$pitch1; //@line 444 "intonation.cpp"
    var $conv=((($7)) & 255); //@line 444 "intonation.cpp"
    var $8=$syl_addr; //@line 444 "intonation.cpp"
    var $pitch113=(($8+4)&4294967295); //@line 444 "intonation.cpp"
    HEAP8[($pitch113)]=$conv; //@line 444 "intonation.cpp"
    var $9=$pitch2; //@line 445 "intonation.cpp"
    var $conv14=((($9)) & 255); //@line 445 "intonation.cpp"
    var $10=$syl_addr; //@line 445 "intonation.cpp"
    var $pitch215=(($10+5)&4294967295); //@line 445 "intonation.cpp"
    HEAP8[($pitch215)]=$conv14; //@line 445 "intonation.cpp"
    var $11=$flags; //@line 446 "intonation.cpp"
    var $12=$syl_addr; //@line 446 "intonation.cpp"
    var $flags16=(($12+2)&4294967295); //@line 446 "intonation.cpp"
    var $13=HEAP8[($flags16)]; //@line 446 "intonation.cpp"
    var $conv17=(tempInt=(($13)),(tempInt>=128?tempInt-256:tempInt)); //@line 446 "intonation.cpp"
    var $or=($conv17) | ($11); //@line 446 "intonation.cpp"
    var $conv18=((($or)) & 255); //@line 446 "intonation.cpp"
    HEAP8[($flags16)]=$conv18; //@line 446 "intonation.cpp"
    ;
    return; //@line 447 "intonation.cpp"
    return;
  }
  

  function __ZL16count_incrementsiii($ix, $end_ix, $min_stress) {
    ;
    var __label__;
  
    var $ix_addr;
    var $end_ix_addr;
    var $min_stress_addr;
    var $count;
    var $stress;
    $ix_addr=$ix;
    $end_ix_addr=$end_ix;
    $min_stress_addr=$min_stress;
    $count=0; //@line 402 "intonation.cpp"
    ; //@line 405 "intonation.cpp"
    while(1) { 
  
      var $0=$ix_addr; //@line 405 "intonation.cpp"
      var $1=$end_ix_addr; //@line 405 "intonation.cpp"
      var $cmp=((($0))|0) < ((($1))|0); //@line 405 "intonation.cpp"
      if (!($cmp)) { __label__ = 5;break ; } //@line 405 "intonation.cpp"
  
      var $2=$ix_addr; //@line 407 "intonation.cpp"
      var $inc=((($2)+1)&4294967295); //@line 407 "intonation.cpp"
      $ix_addr=$inc; //@line 407 "intonation.cpp"
      var $3=HEAP32[((__ZL12syllable_tab)>>2)]; //@line 407 "intonation.cpp"
      var $arrayidx=(($3+6*$2)&4294967295); //@line 407 "intonation.cpp"
      var $stress1=(($arrayidx)&4294967295); //@line 407 "intonation.cpp"
      var $4=HEAP8[($stress1)]; //@line 407 "intonation.cpp"
      var $conv=(tempInt=(($4)),(tempInt>=128?tempInt-256:tempInt)); //@line 407 "intonation.cpp"
      $stress=$conv; //@line 407 "intonation.cpp"
      var $5=$stress; //@line 408 "intonation.cpp"
      var $cmp2=((($5))|0) >= 7; //@line 408 "intonation.cpp"
      if ($cmp2) { __label__ = 5;break ; } //@line 408 "intonation.cpp"
  
      var $6=$stress; //@line 411 "intonation.cpp"
      var $7=$min_stress_addr; //@line 411 "intonation.cpp"
      var $cmp3=((($6))|0) >= ((($7))|0); //@line 411 "intonation.cpp"
      if (!($cmp3)) { __label__ = 1;continue ; } //@line 411 "intonation.cpp"
  
      var $8=$count; //@line 412 "intonation.cpp"
      var $inc5=((($8)+1)&4294967295); //@line 412 "intonation.cpp"
      $count=$inc5; //@line 412 "intonation.cpp"
      __label__ = 1;continue ; //@line 412 "intonation.cpp"
    }
  
    var $9=$count; //@line 414 "intonation.cpp"
    ;
    return $9; //@line 414 "intonation.cpp"
    return null;
  }
  

  function __ZL17SetHeadIntonationP4TUNEiii($tune, $syl_ix, $end_ix) {
    ;
    var __label__;
    var __lastLabel__ = null;
  
    var $tune_addr;
    var $syl_ix_addr;
    var $end_ix_addr;
    var $control_addr;
    var $stress;
    var $syl;
    var $ix;
    var $pitch;
    var $increment;
    var $n_steps;
    var $stage;
    var $initial;
    var $overflow_ix;
    var $pitch_range;
    var $pitch_range_abs;
    var $drops;
    var $n_unstressed;
    var $unstressed_ix;
    var $unstressed_inc;
    var $used_onset;
    var $head_final;
    var $secondary;
    $tune_addr=$tune;
    $syl_ix_addr=$syl_ix;
    $end_ix_addr=$end_ix;
    $control_addr=0;
    $pitch=0; //@line 467 "intonation.cpp"
    $increment=0; //@line 468 "intonation.cpp"
    $n_steps=0; //@line 469 "intonation.cpp"
    $overflow_ix=0; //@line 472 "intonation.cpp"
    $n_unstressed=0; //@line 476 "intonation.cpp"
    $unstressed_ix=0; //@line 477 "intonation.cpp"
    $used_onset=0; //@line 479 "intonation.cpp"
    var $0=$end_ix_addr; //@line 480 "intonation.cpp"
    $head_final=$0; //@line 480 "intonation.cpp"
    $secondary=2; //@line 481 "intonation.cpp"
    var $1=$tune_addr; //@line 483 "intonation.cpp"
    var $head_end=(($1+32)&4294967295); //@line 483 "intonation.cpp"
    var $2=HEAPU8[($head_end)]; //@line 483 "intonation.cpp"
    var $conv=((($2))&255); //@line 483 "intonation.cpp"
    var $3=$tune_addr; //@line 483 "intonation.cpp"
    var $head_start=(($3+31)&4294967295); //@line 483 "intonation.cpp"
    var $4=HEAPU8[($head_start)]; //@line 483 "intonation.cpp"
    var $conv1=((($4))&255); //@line 483 "intonation.cpp"
    var $sub=((($conv)-($conv1))&4294967295); //@line 483 "intonation.cpp"
    var $shl=($sub) << 8; //@line 483 "intonation.cpp"
    $pitch_range=$shl; //@line 483 "intonation.cpp"
    var $5=$pitch_range; //@line 484 "intonation.cpp"
    var $ispos=((($5))|0) > -1; //@line 484 "intonation.cpp"
    var $neg=(((-($5)))&4294967295); //@line 484 "intonation.cpp"
    var $call=($ispos) ? ($5) : ($neg); //@line 484 "intonation.cpp"
    $pitch_range_abs=$call; //@line 484 "intonation.cpp"
    $drops=((__ZL7drops_0)&4294967295); //@line 485 "intonation.cpp"
    $initial=1; //@line 486 "intonation.cpp"
    $stage=0; //@line 488 "intonation.cpp"
    var $6=$tune_addr; //@line 489 "intonation.cpp"
    var $onset=(($6+30)&4294967295); //@line 489 "intonation.cpp"
    var $7=HEAPU8[($onset)]; //@line 489 "intonation.cpp"
    var $conv2=((($7))&255); //@line 489 "intonation.cpp"
    var $cmp=((($conv2))|0)==255; //@line 489 "intonation.cpp"
    if ($cmp) { __label__ = 1;; } else { __label__ = 2;; } //@line 489 "intonation.cpp"
    if (__label__ == 1) {
  
      $stage=1; //@line 490 "intonation.cpp"
      ; //@line 490 "intonation.cpp"
    }
  
    var $8=$tune_addr; //@line 492 "intonation.cpp"
    var $head_last=(($8+33)&4294967295); //@line 492 "intonation.cpp"
    var $9=HEAPU8[($head_last)]; //@line 492 "intonation.cpp"
    var $conv3=((($9))&255); //@line 492 "intonation.cpp"
    var $cmp4=((($conv3))|0)!=255; //@line 492 "intonation.cpp"
    if ($cmp4) { __label__ = 4;; } else { __label__ = 3;; } //@line 492 "intonation.cpp"
    $if_then5$$while_cond_preheader$5: do { 
      if (__label__ == 4) {
  
        var $13=$end_ix_addr; //@line 495 "intonation.cpp"
        var $sub6=((($13)-1)&4294967295); //@line 495 "intonation.cpp"
        $ix=$sub6; //@line 495 "intonation.cpp"
        ; //@line 495 "intonation.cpp"
        while(1) { 
  
          var $14=$ix; //@line 495 "intonation.cpp"
          var $15=$syl_ix_addr; //@line 495 "intonation.cpp"
          var $cmp7=((($14))|0) >= ((($15))|0); //@line 495 "intonation.cpp"
          if (!($cmp7)) { __label__ = 3;break $if_then5$$while_cond_preheader$5; } //@line 495 "intonation.cpp"
  
          var $16=$ix; //@line 497 "intonation.cpp"
          var $17=HEAP32[((__ZL12syllable_tab)>>2)]; //@line 497 "intonation.cpp"
          var $arrayidx=(($17+6*$16)&4294967295); //@line 497 "intonation.cpp"
          var $stress8=(($arrayidx)&4294967295); //@line 497 "intonation.cpp"
          var $18=HEAP8[($stress8)]; //@line 497 "intonation.cpp"
          var $conv9=(tempInt=(($18)),(tempInt>=128?tempInt-256:tempInt)); //@line 497 "intonation.cpp"
          var $cmp10=((($conv9))|0) >= 4; //@line 497 "intonation.cpp"
          var $19=$ix; //@line 499 "intonation.cpp"
          if ($cmp10) { __label__ = 7;break ; } //@line 497 "intonation.cpp"
  
          var $dec=((($19)-1)&4294967295); //@line 495 "intonation.cpp"
          $ix=$dec; //@line 495 "intonation.cpp"
          __label__ = 5;continue ; //@line 495 "intonation.cpp"
        }
  
        $head_final=$19; //@line 499 "intonation.cpp"
        ; //@line 500 "intonation.cpp"
      }
    } while(0);
  
    var $10=$syl_ix_addr; //@line 505 "intonation.cpp"
    var $11=$end_ix_addr; //@line 505 "intonation.cpp"
    var $cmp145=((($10))|0) < ((($11))|0); //@line 505 "intonation.cpp"
    var $12=$syl_ix_addr; //@line 507 "intonation.cpp"
    if ($cmp145) { __lastLabel__ = 3; __label__ = 9;; } else { __lastLabel__ = 3; __label__ = 39;; } //@line 505 "intonation.cpp"
    $while_body$$while_end$13: do { 
      if (__label__ == 9) {
        $while_body$14: while(1) { 
  
          var $20=__lastLabel__ == 38 ? $108 : ($12);
          var $21=HEAP32[((__ZL12syllable_tab)>>2)]; //@line 507 "intonation.cpp"
          var $arrayidx15=(($21+6*$20)&4294967295); //@line 507 "intonation.cpp"
          $syl=$arrayidx15; //@line 507 "intonation.cpp"
          var $22=$syl; //@line 508 "intonation.cpp"
          var $stress16=(($22)&4294967295); //@line 508 "intonation.cpp"
          var $23=HEAP8[($stress16)]; //@line 508 "intonation.cpp"
          var $conv17=(tempInt=(($23)),(tempInt>=128?tempInt-256:tempInt)); //@line 508 "intonation.cpp"
          $stress=$conv17; //@line 508 "intonation.cpp"
          var $24=$initial; //@line 510 "intonation.cpp"
          var $tobool=((($24))|0)!=0; //@line 510 "intonation.cpp"
          if ($tobool) { __label__ = 13;; } else { __label__ = 10;; } //@line 510 "intonation.cpp"
          $if_then23$$lor_lhs_false$16: do { 
            if (__label__ == 10) {
  
              var $25=$stress; //@line 510 "intonation.cpp"
              var $cmp18=((($25))|0) >= 4; //@line 510 "intonation.cpp"
              if (!($cmp18)) { __label__ = 32;break $if_then23$$lor_lhs_false$16; } //@line 510 "intonation.cpp"
  
              var $_pr=$initial; //@line 514 "intonation.cpp"
              var $tobool20=((($_pr))|0)!=0; //@line 514 "intonation.cpp"
              if ($tobool20) { __label__ = 13;break $if_then23$$lor_lhs_false$16; } //@line 514 "intonation.cpp"
  
              var $26=$stress; //@line 514 "intonation.cpp"
              var $cmp22=((($26))|0)==5; //@line 514 "intonation.cpp"
              if ($cmp22) { __label__ = 13;break $if_then23$$lor_lhs_false$16; } //@line 514 "intonation.cpp"
  
              var $44=$syl_ix_addr; //@line 544 "intonation.cpp"
              var $45=$head_final; //@line 544 "intonation.cpp"
              var $cmp49=((($44))|0)==((($45))|0); //@line 544 "intonation.cpp"
              if ($cmp49) { __label__ = 23;; } else { __label__ = 24;; } //@line 544 "intonation.cpp"
              if (__label__ == 23) {
  
                var $46=$tune_addr; //@line 547 "intonation.cpp"
                var $head_last51=(($46+33)&4294967295); //@line 547 "intonation.cpp"
                var $47=HEAPU8[($head_last51)]; //@line 547 "intonation.cpp"
                var $conv52=((($47))&255); //@line 547 "intonation.cpp"
                var $shl53=($conv52) << 8; //@line 547 "intonation.cpp"
                $pitch=$shl53; //@line 547 "intonation.cpp"
                $stage=2; //@line 548 "intonation.cpp"
                __label__ = 30;break $if_then23$$lor_lhs_false$16; //@line 549 "intonation.cpp"
              }
              else if (__label__ == 24) {
  
                var $48=$used_onset; //@line 552 "intonation.cpp"
                var $tobool55=((($48))|0)!=0; //@line 552 "intonation.cpp"
                if ($tobool55) { __label__ = 25;; } else { __label__ = 26;; } //@line 552 "intonation.cpp"
                if (__label__ == 25) {
  
                  $stage=1; //@line 554 "intonation.cpp"
                  $used_onset=0; //@line 555 "intonation.cpp"
                  var $49=$tune_addr; //@line 556 "intonation.cpp"
                  var $head_start57=(($49+31)&4294967295); //@line 556 "intonation.cpp"
                  var $50=HEAPU8[($head_start57)]; //@line 556 "intonation.cpp"
                  var $conv58=((($50))&255); //@line 556 "intonation.cpp"
                  var $shl59=($conv58) << 8; //@line 556 "intonation.cpp"
                  $pitch=$shl59; //@line 556 "intonation.cpp"
                  var $51=$n_steps; //@line 557 "intonation.cpp"
                  var $inc=((($51)+1)&4294967295); //@line 557 "intonation.cpp"
                  $n_steps=$inc; //@line 557 "intonation.cpp"
                  __label__ = 30;break $if_then23$$lor_lhs_false$16; //@line 558 "intonation.cpp"
                }
                else if (__label__ == 26) {
  
                  var $52=$n_steps; //@line 560 "intonation.cpp"
                  var $cmp61=((($52))|0) > 0; //@line 560 "intonation.cpp"
                  if ($cmp61) { __label__ = 27;; } else { __label__ = 28;; } //@line 560 "intonation.cpp"
                  if (__label__ == 27) {
  
                    var $53=$increment; //@line 561 "intonation.cpp"
                    var $54=$pitch; //@line 561 "intonation.cpp"
                    var $add63=((($54)+($53))&4294967295); //@line 561 "intonation.cpp"
                    $pitch=$add63; //@line 561 "intonation.cpp"
                    __label__ = 30;break $if_then23$$lor_lhs_false$16; //@line 561 "intonation.cpp"
                  }
                  else if (__label__ == 28) {
  
                    var $55=$tune_addr; //@line 564 "intonation.cpp"
                    var $head_end65=(($55+32)&4294967295); //@line 564 "intonation.cpp"
                    var $56=HEAPU8[($head_end65)]; //@line 564 "intonation.cpp"
                    var $conv66=((($56))&255); //@line 564 "intonation.cpp"
                    var $shl67=((($conv66)*256)&4294967295); //@line 564 "intonation.cpp"
                    var $57=$pitch_range_abs; //@line 564 "intonation.cpp"
                    var $58=$overflow_ix; //@line 564 "intonation.cpp"
                    var $inc68=((($58)+1)&4294967295); //@line 564 "intonation.cpp"
                    $overflow_ix=$inc68; //@line 564 "intonation.cpp"
                    var $59=$tune_addr; //@line 564 "intonation.cpp"
                    var $head_extend=(($59+16)&4294967295); //@line 564 "intonation.cpp"
                    var $arrayidx69=(($head_extend+$58)&4294967295); //@line 564 "intonation.cpp"
                    var $60=HEAP8[($arrayidx69)]; //@line 564 "intonation.cpp"
                    var $conv70=(tempInt=(($60)),(tempInt>=128?tempInt-256:tempInt)); //@line 564 "intonation.cpp"
                    var $mul=((($conv70)*($57))&4294967295); //@line 564 "intonation.cpp"
                    var $div71=((((($mul))|0)/64)|0); //@line 564 "intonation.cpp"
                    var $add72=((($div71)+($shl67))&4294967295); //@line 564 "intonation.cpp"
                    $pitch=$add72; //@line 564 "intonation.cpp"
                    var $61=$overflow_ix; //@line 565 "intonation.cpp"
                    var $62=$tune_addr; //@line 565 "intonation.cpp"
                    var $n_head_extend=(($62+35)&4294967295); //@line 565 "intonation.cpp"
                    var $63=HEAPU8[($n_head_extend)]; //@line 565 "intonation.cpp"
                    var $conv73=((($63))&255); //@line 565 "intonation.cpp"
                    var $cmp74=((($61))|0) >= ((($conv73))|0); //@line 565 "intonation.cpp"
                    if (!($cmp74)) { __label__ = 30;break $if_then23$$lor_lhs_false$16; } //@line 565 "intonation.cpp"
  
                    $overflow_ix=0; //@line 567 "intonation.cpp"
                    __label__ = 30;break $if_then23$$lor_lhs_false$16; //@line 568 "intonation.cpp"
                  }
                }
              }
            }
          } while(0);
          $if_then23$$if_end82$$if_else89$31: do { 
            if (__label__ == 13) {
  
              $initial=0; //@line 516 "intonation.cpp"
              $overflow_ix=0; //@line 517 "intonation.cpp"
              var $27=$tune_addr; //@line 519 "intonation.cpp"
              var $onset24=(($27+30)&4294967295); //@line 519 "intonation.cpp"
              var $28=HEAPU8[($onset24)]; //@line 519 "intonation.cpp"
              var $conv25=((($28))&255); //@line 519 "intonation.cpp"
              var $cmp26=((($conv25))|0)==255; //@line 519 "intonation.cpp"
              var $29=$syl_ix_addr; //@line 521 "intonation.cpp"
              if ($cmp26) { __label__ = 14;; } else { __label__ = 15;; } //@line 519 "intonation.cpp"
              if (__label__ == 14) {
  
                var $30=$head_final; //@line 521 "intonation.cpp"
                var $call28=__ZL16count_incrementsiii($29, $30, 4); //@line 521 "intonation.cpp"
                $n_steps=$call28; //@line 521 "intonation.cpp"
                var $31=$tune_addr; //@line 522 "intonation.cpp"
                var $head_start29=(($31+31)&4294967295); //@line 522 "intonation.cpp"
                var $32=HEAPU8[($head_start29)]; //@line 522 "intonation.cpp"
                var $conv30=((($32))&255); //@line 522 "intonation.cpp"
                var $shl31=($conv30) << 8; //@line 522 "intonation.cpp"
                $pitch=$shl31; //@line 522 "intonation.cpp"
                ; //@line 523 "intonation.cpp"
              }
              else if (__label__ == 15) {
  
                var $add=((($29)+1)&4294967295); //@line 527 "intonation.cpp"
                var $33=$head_final; //@line 527 "intonation.cpp"
                var $call32=__ZL16count_incrementsiii($add, $33, 4); //@line 527 "intonation.cpp"
                $n_steps=$call32; //@line 527 "intonation.cpp"
                var $34=$tune_addr; //@line 528 "intonation.cpp"
                var $onset33=(($34+30)&4294967295); //@line 528 "intonation.cpp"
                var $35=HEAPU8[($onset33)]; //@line 528 "intonation.cpp"
                var $conv34=((($35))&255); //@line 528 "intonation.cpp"
                var $shl35=($conv34) << 8; //@line 528 "intonation.cpp"
                $pitch=$shl35; //@line 528 "intonation.cpp"
                $used_onset=1; //@line 529 "intonation.cpp"
                ;
              }
  
              var $36=$n_steps; //@line 532 "intonation.cpp"
              var $37=$tune_addr; //@line 532 "intonation.cpp"
              var $head_max_steps=(($37+34)&4294967295); //@line 532 "intonation.cpp"
              var $38=HEAPU8[($head_max_steps)]; //@line 532 "intonation.cpp"
              var $conv37=((($38))&255); //@line 532 "intonation.cpp"
              var $cmp38=((($36))|0) > ((($conv37))|0); //@line 532 "intonation.cpp"
              if ($cmp38) { __label__ = 17;; } else { __label__ = 18;; } //@line 532 "intonation.cpp"
              if (__label__ == 17) {
  
                var $39=$tune_addr; //@line 533 "intonation.cpp"
                var $head_max_steps40=(($39+34)&4294967295); //@line 533 "intonation.cpp"
                var $40=HEAPU8[($head_max_steps40)]; //@line 533 "intonation.cpp"
                var $conv41=((($40))&255); //@line 533 "intonation.cpp"
                $n_steps=$conv41; //@line 533 "intonation.cpp"
                __lastLabel__ = 17; ; //@line 533 "intonation.cpp"
              }
              else if (__label__ == 18) {
  
                var $_pr2=$n_steps; //@line 535 "intonation.cpp"
                __lastLabel__ = 18; ;
              }
  
              var $41=__lastLabel__ == 18 ? $_pr2 : ($conv41); //@line 535 "intonation.cpp"
              var $cmp43=((($41))|0) > 1; //@line 535 "intonation.cpp"
              if ($cmp43) { __label__ = 20;; } else { __label__ = 21;; } //@line 535 "intonation.cpp"
              if (__label__ == 20) {
  
                var $42=$pitch_range; //@line 537 "intonation.cpp"
                var $43=$n_steps; //@line 537 "intonation.cpp"
                var $sub45=((($43)-1)&4294967295); //@line 537 "intonation.cpp"
                var $div=((((($42))|0)/((($sub45))|0))|0); //@line 537 "intonation.cpp"
                $increment=$div; //@line 537 "intonation.cpp"
                __label__ = 30;break $if_then23$$if_end82$$if_else89$31; //@line 538 "intonation.cpp"
              }
              else if (__label__ == 21) {
  
                $increment=0; //@line 540 "intonation.cpp"
                __label__ = 30;break $if_then23$$if_end82$$if_else89$31;
              }
            }
          } while(0);
          $if_end82$$if_else89$44: do { 
            if (__label__ == 30) {
  
              var $64=$n_steps; //@line 572 "intonation.cpp"
              var $dec81=((($64)-1)&4294967295); //@line 572 "intonation.cpp"
              $n_steps=$dec81; //@line 572 "intonation.cpp"
              var $_pr3=$stress; //@line 575 "intonation.cpp"
              var $cmp83=((($_pr3))|0) >= 4; //@line 575 "intonation.cpp"
              if (!($cmp83)) { __label__ = 32;break $if_end82$$if_else89$44; } //@line 575 "intonation.cpp"
  
              var $65=$syl_ix_addr; //@line 577 "intonation.cpp"
              var $add85=((($65)+1)&4294967295); //@line 577 "intonation.cpp"
              var $66=$end_ix_addr; //@line 577 "intonation.cpp"
              var $67=$secondary; //@line 577 "intonation.cpp"
              var $call86=__ZL15CountUnstressediii($add85, $66, $67); //@line 577 "intonation.cpp"
              $n_unstressed=$call86; //@line 577 "intonation.cpp"
              $unstressed_ix=0; //@line 578 "intonation.cpp"
              var $68=$syl; //@line 579 "intonation.cpp"
              var $stress87=(($68)&4294967295); //@line 579 "intonation.cpp"
              HEAP8[($stress87)]=6; //@line 579 "intonation.cpp"
              var $69=$tune_addr; //@line 580 "intonation.cpp"
              var $stressed_env=(($69+26)&4294967295); //@line 580 "intonation.cpp"
              var $70=HEAP8[($stressed_env)]; //@line 580 "intonation.cpp"
              var $71=$syl; //@line 580 "intonation.cpp"
              var $env=(($71+1)&4294967295); //@line 580 "intonation.cpp"
              HEAP8[($env)]=$70; //@line 580 "intonation.cpp"
              var $72=$syl; //@line 581 "intonation.cpp"
              var $73=$pitch; //@line 581 "intonation.cpp"
              var $shr=($73) >> 8; //@line 581 "intonation.cpp"
              var $74=$tune_addr; //@line 581 "intonation.cpp"
              var $stressed_drop=(($74+27)&4294967295); //@line 581 "intonation.cpp"
              var $75=HEAPU8[($stressed_drop)]; //@line 581 "intonation.cpp"
              var $conv88=((($75))&255); //@line 581 "intonation.cpp"
              __ZL9set_pitchP8SYLLABLEii($72, $shr, $conv88); //@line 581 "intonation.cpp"
              __label__ = 38;break $if_end82$$if_else89$44; //@line 582 "intonation.cpp"
            }
          } while(0);
          if (__label__ == 32) {
  
            var $76=$stress; //@line 584 "intonation.cpp"
            var $77=$secondary; //@line 584 "intonation.cpp"
            var $cmp90=((($76))|0) >= ((($77))|0); //@line 584 "intonation.cpp"
            if ($cmp90) { __label__ = 33;; } else { __label__ = 34;; } //@line 584 "intonation.cpp"
            if (__label__ == 33) {
  
              var $78=$syl_ix_addr; //@line 586 "intonation.cpp"
              var $add92=((($78)+1)&4294967295); //@line 586 "intonation.cpp"
              var $79=$end_ix_addr; //@line 586 "intonation.cpp"
              var $80=$secondary; //@line 586 "intonation.cpp"
              var $call93=__ZL15CountUnstressediii($add92, $79, $80); //@line 586 "intonation.cpp"
              $n_unstressed=$call93; //@line 586 "intonation.cpp"
              $unstressed_ix=0; //@line 587 "intonation.cpp"
              var $81=$syl; //@line 588 "intonation.cpp"
              var $82=$pitch; //@line 588 "intonation.cpp"
              var $shr94=($82) >> 8; //@line 588 "intonation.cpp"
              var $83=$stress; //@line 588 "intonation.cpp"
              var $84=$drops; //@line 588 "intonation.cpp"
              var $arrayidx95=(($84+4*$83)&4294967295); //@line 588 "intonation.cpp"
              var $85=HEAP32[(($arrayidx95)>>2)]; //@line 588 "intonation.cpp"
              __ZL9set_pitchP8SYLLABLEii($81, $shr94, $85); //@line 588 "intonation.cpp"
              ; //@line 589 "intonation.cpp"
            }
            else if (__label__ == 34) {
  
              var $86=$n_unstressed; //@line 592 "intonation.cpp"
              var $cmp97=((($86))|0) > 1; //@line 592 "intonation.cpp"
              if ($cmp97) { __label__ = 35;; } else { __label__ = 36;; } //@line 592 "intonation.cpp"
              if (__label__ == 35) {
  
                var $87=$stage; //@line 593 "intonation.cpp"
                var $88=$tune_addr; //@line 593 "intonation.cpp"
                var $unstr_end=(($88+39)&4294967295); //@line 593 "intonation.cpp"
                var $arrayidx99=(($unstr_end+$87)&4294967295); //@line 593 "intonation.cpp"
                var $89=HEAP8[($arrayidx99)]; //@line 593 "intonation.cpp"
                var $conv100=(tempInt=(($89)),(tempInt>=128?tempInt-256:tempInt)); //@line 593 "intonation.cpp"
                var $90=$stage; //@line 593 "intonation.cpp"
                var $91=$tune_addr; //@line 593 "intonation.cpp"
                var $unstr_start=(($91+36)&4294967295); //@line 593 "intonation.cpp"
                var $arrayidx101=(($unstr_start+$90)&4294967295); //@line 593 "intonation.cpp"
                var $92=HEAP8[($arrayidx101)]; //@line 593 "intonation.cpp"
                var $conv102=(tempInt=(($92)),(tempInt>=128?tempInt-256:tempInt)); //@line 593 "intonation.cpp"
                var $sub103=((($conv100)-($conv102))&4294967295); //@line 593 "intonation.cpp"
                var $93=$n_unstressed; //@line 593 "intonation.cpp"
                var $sub104=((($93)-1)&4294967295); //@line 593 "intonation.cpp"
                var $div105=((((($sub103))|0)/((($sub104))|0))|0); //@line 593 "intonation.cpp"
                $unstressed_inc=$div105; //@line 593 "intonation.cpp"
                ; //@line 593 "intonation.cpp"
              }
              else if (__label__ == 36) {
  
                $unstressed_inc=0; //@line 595 "intonation.cpp"
                ;
              }
  
              var $94=$syl; //@line 597 "intonation.cpp"
              var $95=$pitch; //@line 597 "intonation.cpp"
              var $shr108=($95) >> 8; //@line 597 "intonation.cpp"
              var $96=$stage; //@line 597 "intonation.cpp"
              var $97=$tune_addr; //@line 597 "intonation.cpp"
              var $unstr_start109=(($97+36)&4294967295); //@line 597 "intonation.cpp"
              var $arrayidx110=(($unstr_start109+$96)&4294967295); //@line 597 "intonation.cpp"
              var $98=HEAP8[($arrayidx110)]; //@line 597 "intonation.cpp"
              var $conv111=(tempInt=(($98)),(tempInt>=128?tempInt-256:tempInt)); //@line 597 "intonation.cpp"
              var $99=$unstressed_inc; //@line 597 "intonation.cpp"
              var $100=$unstressed_ix; //@line 597 "intonation.cpp"
              var $mul113=((($100)*($99))&4294967295); //@line 597 "intonation.cpp"
              var $add112=((($conv111)+($shr108))&4294967295); //@line 597 "intonation.cpp"
              var $add114=((($add112)+($mul113))&4294967295); //@line 597 "intonation.cpp"
              var $101=$stress; //@line 597 "intonation.cpp"
              var $102=$drops; //@line 597 "intonation.cpp"
              var $arrayidx115=(($102+4*$101)&4294967295); //@line 597 "intonation.cpp"
              var $103=HEAP32[(($arrayidx115)>>2)]; //@line 597 "intonation.cpp"
              __ZL9set_pitchP8SYLLABLEii($94, $add114, $103); //@line 597 "intonation.cpp"
              var $104=$unstressed_ix; //@line 598 "intonation.cpp"
              var $inc116=((($104)+1)&4294967295); //@line 598 "intonation.cpp"
              $unstressed_ix=$inc116; //@line 598 "intonation.cpp"
              ;
            }
          }
  
          var $105=$syl_ix_addr; //@line 601 "intonation.cpp"
          var $inc119=((($105)+1)&4294967295); //@line 601 "intonation.cpp"
          $syl_ix_addr=$inc119; //@line 601 "intonation.cpp"
          var $106=$syl_ix_addr; //@line 505 "intonation.cpp"
          var $107=$end_ix_addr; //@line 505 "intonation.cpp"
          var $cmp14=((($106))|0) < ((($107))|0); //@line 505 "intonation.cpp"
          var $108=$syl_ix_addr; //@line 507 "intonation.cpp"
          if ($cmp14) { __lastLabel__ = 38; __label__ = 9;continue $while_body$14; } else { __lastLabel__ = 38; __label__ = 39;break $while_body$$while_end$13; } //@line 505 "intonation.cpp"
        }
      }
    } while(0);
  
    var $_lcssa=__lastLabel__ == 3 ? $12 : ($108);
    ;
    return $_lcssa; //@line 603 "intonation.cpp"
    return null;
  }
  

  function __ZL15CountUnstressediii($start, $end, $limit) {
    ;
    var __label__;
  
    var $start_addr;
    var $end_addr;
    var $limit_addr;
    var $ix;
    $start_addr=$start;
    $end_addr=$end;
    $limit_addr=$limit;
    var $0=$start_addr; //@line 454 "intonation.cpp"
    $ix=$0; //@line 454 "intonation.cpp"
    ; //@line 454 "intonation.cpp"
    while(1) { 
  
      var $1=$ix; //@line 454 "intonation.cpp"
      var $2=$end_addr; //@line 454 "intonation.cpp"
      var $cmp=((($1))|0) <= ((($2))|0); //@line 454 "intonation.cpp"
      if (!($cmp)) { __label__ = 4;break ; } //@line 454 "intonation.cpp"
  
      var $3=$ix; //@line 456 "intonation.cpp"
      var $4=HEAP32[((__ZL12syllable_tab)>>2)]; //@line 456 "intonation.cpp"
      var $arrayidx=(($4+6*$3)&4294967295); //@line 456 "intonation.cpp"
      var $stress=(($arrayidx)&4294967295); //@line 456 "intonation.cpp"
      var $5=HEAP8[($stress)]; //@line 456 "intonation.cpp"
      var $conv=(tempInt=(($5)),(tempInt>=128?tempInt-256:tempInt)); //@line 456 "intonation.cpp"
      var $6=$limit_addr; //@line 456 "intonation.cpp"
      var $cmp1=((($conv))|0) >= ((($6))|0); //@line 456 "intonation.cpp"
      if ($cmp1) { __label__ = 4;break ; } //@line 456 "intonation.cpp"
  
      var $7=$ix; //@line 454 "intonation.cpp"
      var $inc=((($7)+1)&4294967295); //@line 454 "intonation.cpp"
      $ix=$inc; //@line 454 "intonation.cpp"
      __label__ = 1;continue ; //@line 454 "intonation.cpp"
    }
  
    var $8=$ix; //@line 459 "intonation.cpp"
    var $9=$start_addr; //@line 459 "intonation.cpp"
    var $sub=((($8)-($9))&4294967295); //@line 459 "intonation.cpp"
    ;
    return $sub; //@line 459 "intonation.cpp"
    return null;
  }
  

  function _iswalpha($c) {
    ;
    var __label__;
  
    var $retval;
    var $c_addr;
    $c_addr=$c;
    var $0=$c_addr; //@line 239 "readclause.cpp"
    var $cmp=((($0))>>>0) < 256; //@line 239 "readclause.cpp"
    var $1=$c_addr; //@line 240 "readclause.cpp"
    if ($cmp) { __label__ = 1;; } else { __label__ = 2;; } //@line 239 "readclause.cpp"
    if (__label__ == 1) {
  
      var $call=_isalpha($1); //@line 240 "readclause.cpp"
      $retval=$call; //@line 240 "readclause.cpp"
      ; //@line 240 "readclause.cpp"
    }
    else if (__label__ == 2) {
  
      var $cmp1=((($1))>>>0) > 12352; //@line 241 "readclause.cpp"
      var $2=$c_addr; //@line 241 "readclause.cpp"
      var $cmp2=((($2))>>>0) <= 42752; //@line 241 "readclause.cpp"
      var $or_cond=($cmp1) & ($cmp2); //@line 241 "readclause.cpp"
      if ($or_cond) { __label__ = 3;; } else { __label__ = 4;; } //@line 241 "readclause.cpp"
      if (__label__ == 3) {
  
        $retval=1; //@line 242 "readclause.cpp"
        ; //@line 242 "readclause.cpp"
      }
      else if (__label__ == 4) {
  
        var $3=$c_addr; //@line 243 "readclause.cpp"
        var $cmp5=((($3))>>>0) > 563; //@line 243 "readclause.cpp"
        if ($cmp5) { __label__ = 5;; } else { __label__ = 6;; } //@line 243 "readclause.cpp"
        if (__label__ == 5) {
  
          $retval=0; //@line 244 "readclause.cpp"
          ; //@line 244 "readclause.cpp"
        }
        else if (__label__ == 6) {
  
          var $4=$c_addr; //@line 245 "readclause.cpp"
          var $sub=((($4)-256)&4294967295); //@line 245 "readclause.cpp"
          var $arrayidx=((_walpha_tab+$sub)&4294967295); //@line 245 "readclause.cpp"
          var $5=HEAPU8[($arrayidx)]; //@line 245 "readclause.cpp"
          var $conv=((($5))&255); //@line 245 "readclause.cpp"
          $retval=$conv; //@line 245 "readclause.cpp"
          ; //@line 245 "readclause.cpp"
        }
      }
    }
  
    var $6=$retval; //@line 246 "readclause.cpp"
    ;
    return $6; //@line 246 "readclause.cpp"
    return null;
  }
  

  function _iswdigit($c) {
    ;
    var __label__;
  
    var $retval;
    var $c_addr;
    $c_addr=$c;
    var $0=$c_addr; //@line 250 "readclause.cpp"
    var $cmp=((($0))>>>0) < 256; //@line 250 "readclause.cpp"
    if ($cmp) { __label__ = 1;; } else { __label__ = 2;; } //@line 250 "readclause.cpp"
    if (__label__ == 1) {
  
      var $1=$c_addr; //@line 251 "readclause.cpp"
      var $isdigittmp=((($1)-48)&4294967295); //@line 251 "readclause.cpp"
      var $isdigit=((($isdigittmp))>>>0) < 10; //@line 251 "readclause.cpp"
      var $call=((($isdigit))&1); //@line 251 "readclause.cpp"
      $retval=$call; //@line 251 "readclause.cpp"
      ; //@line 251 "readclause.cpp"
    }
    else if (__label__ == 2) {
  
      $retval=0; //@line 252 "readclause.cpp"
      ; //@line 252 "readclause.cpp"
    }
  
    var $2=$retval; //@line 253 "readclause.cpp"
    ;
    return $2; //@line 253 "readclause.cpp"
    return null;
  }
  

  function _iswalnum($c) {
    ;
    var __label__;
  
    var $retval;
    var $c_addr;
    $c_addr=$c;
    var $0=$c_addr; //@line 257 "readclause.cpp"
    var $call=_iswdigit($0); //@line 257 "readclause.cpp"
    var $tobool=((($call))|0)!=0; //@line 257 "readclause.cpp"
    if ($tobool) { __label__ = 1;; } else { __label__ = 2;; } //@line 257 "readclause.cpp"
    if (__label__ == 1) {
  
      $retval=1; //@line 258 "readclause.cpp"
      ; //@line 258 "readclause.cpp"
    }
    else if (__label__ == 2) {
  
      var $1=$c_addr; //@line 259 "readclause.cpp"
      var $call1=_iswalpha($1); //@line 259 "readclause.cpp"
      $retval=$call1; //@line 259 "readclause.cpp"
      ; //@line 259 "readclause.cpp"
    }
  
    var $2=$retval; //@line 260 "readclause.cpp"
    ;
    return $2; //@line 260 "readclause.cpp"
    return null;
  }
  

  function _towlower($c) {
    ;
    var __label__;
  
    var $retval;
    var $c_addr;
    var $x;
    $c_addr=$c;
    var $0=$c_addr; //@line 265 "readclause.cpp"
    var $cmp=((($0))>>>0) < 256; //@line 265 "readclause.cpp"
    var $1=$c_addr; //@line 266 "readclause.cpp"
    if ($cmp) { __label__ = 1;; } else { __label__ = 2;; } //@line 265 "readclause.cpp"
    $if_then$$if_end$2: do { 
      if (__label__ == 1) {
  
        var $call=_tolower($1); //@line 266 "readclause.cpp"
        $retval=$call; //@line 266 "readclause.cpp"
        ; //@line 266 "readclause.cpp"
      }
      else if (__label__ == 2) {
  
        var $cmp1=((($1))>>>0) > 563; //@line 267 "readclause.cpp"
        if ($cmp1) { __label__ = 4;; } else { __label__ = 3;; } //@line 267 "readclause.cpp"
        $if_then3$$lor_lhs_false$5: do { 
          if (__label__ == 3) {
  
            var $2=$c_addr; //@line 267 "readclause.cpp"
            var $sub=((($2)-256)&4294967295); //@line 267 "readclause.cpp"
            var $arrayidx=((_walpha_tab+$sub)&4294967295); //@line 267 "readclause.cpp"
            var $3=HEAPU8[($arrayidx)]; //@line 267 "readclause.cpp"
            var $conv=((($3))&255); //@line 267 "readclause.cpp"
            $x=$conv; //@line 267 "readclause.cpp"
            var $cmp2=((($conv))|0)==255; //@line 267 "readclause.cpp"
            if ($cmp2) { __label__ = 4;break $if_then3$$lor_lhs_false$5; } //@line 267 "readclause.cpp"
  
            var $5=$x; //@line 269 "readclause.cpp"
            var $cmp5=((($5))|0)==254; //@line 269 "readclause.cpp"
            if ($cmp5) { __label__ = 6;; } else { __label__ = 8;; } //@line 269 "readclause.cpp"
            $if_then6$$if_end10$8: do { 
              if (__label__ == 6) {
  
                var $6=$c_addr; //@line 272 "readclause.cpp"
                var $cmp7=((($6))|0)==304; //@line 272 "readclause.cpp"
                if (!($cmp7)) { __label__ = 8;break $if_then6$$if_end10$8; } //@line 272 "readclause.cpp"
  
                $retval=105; //@line 273 "readclause.cpp"
                __label__ = 9;break $if_then$$if_end$2; //@line 273 "readclause.cpp"
              }
            } while(0);
  
            var $7=$c_addr; //@line 275 "readclause.cpp"
            var $8=$x; //@line 275 "readclause.cpp"
            var $add=((($8)+($7))&4294967295); //@line 275 "readclause.cpp"
            $retval=$add; //@line 275 "readclause.cpp"
            __label__ = 9;break $if_then$$if_end$2; //@line 275 "readclause.cpp"
          }
        } while(0);
  
        var $4=$c_addr; //@line 268 "readclause.cpp"
        $retval=$4; //@line 268 "readclause.cpp"
        ; //@line 268 "readclause.cpp"
      }
    } while(0);
  
    var $9=$retval; //@line 276 "readclause.cpp"
    ;
    return $9; //@line 276 "readclause.cpp"
    return null;
  }
  

  function _towupper($c) {
    ;
    var __label__;
  
    var $retval;
    var $c_addr;
    $c_addr=$c;
    var $0=$c_addr; //@line 281 "readclause.cpp"
    var $sub=((($0)-1)&4294967295); //@line 281 "readclause.cpp"
    var $call=_tolower($sub); //@line 281 "readclause.cpp"
    var $1=$c_addr; //@line 281 "readclause.cpp"
    var $cmp=((($call))|0)==((($1))|0); //@line 281 "readclause.cpp"
    var $2=$c_addr; //@line 282 "readclause.cpp"
    if ($cmp) { __label__ = 1;; } else { __label__ = 2;; } //@line 281 "readclause.cpp"
    if (__label__ == 1) {
  
      var $sub1=((($2)-1)&4294967295); //@line 282 "readclause.cpp"
      $retval=$sub1; //@line 282 "readclause.cpp"
      ; //@line 282 "readclause.cpp"
    }
    else if (__label__ == 2) {
  
      $retval=$2; //@line 283 "readclause.cpp"
      ; //@line 283 "readclause.cpp"
    }
  
    var $3=$retval; //@line 284 "readclause.cpp"
    ;
    return $3; //@line 284 "readclause.cpp"
    return null;
  }
  

  function _iswupper($c) {
    ;
    var __label__;
  
    var $retval;
    var $c_addr;
    var $x;
    $c_addr=$c;
    var $0=$c_addr; //@line 289 "readclause.cpp"
    var $cmp=((($0))>>>0) < 256; //@line 289 "readclause.cpp"
    var $1=$c_addr; //@line 290 "readclause.cpp"
    if ($cmp) { __label__ = 1;; } else { __label__ = 2;; } //@line 289 "readclause.cpp"
    $if_then$$if_end$2: do { 
      if (__label__ == 1) {
  
        var $call=_isupper($1); //@line 290 "readclause.cpp"
        $retval=$call; //@line 290 "readclause.cpp"
        ; //@line 290 "readclause.cpp"
      }
      else if (__label__ == 2) {
  
        var $cmp1=((($1))>>>0) > 563; //@line 291 "readclause.cpp"
        if ($cmp1) { __label__ = 5;; } else { __label__ = 3;; } //@line 291 "readclause.cpp"
        $if_then5$$lor_lhs_false$5: do { 
          if (__label__ == 3) {
  
            var $2=$c_addr; //@line 291 "readclause.cpp"
            var $sub=((($2)-256)&4294967295); //@line 291 "readclause.cpp"
            var $arrayidx=((_walpha_tab+$sub)&4294967295); //@line 291 "readclause.cpp"
            var $3=HEAPU8[($arrayidx)]; //@line 291 "readclause.cpp"
            var $conv=((($3))&255); //@line 291 "readclause.cpp"
            $x=$conv; //@line 291 "readclause.cpp"
            var $cmp2=((($conv))|0)==0; //@line 291 "readclause.cpp"
            if ($cmp2) { __label__ = 5;break $if_then5$$lor_lhs_false$5; } //@line 291 "readclause.cpp"
  
            var $4=$x; //@line 291 "readclause.cpp"
            var $cmp4=((($4))|0)==255; //@line 291 "readclause.cpp"
            if ($cmp4) { __label__ = 5;break $if_then5$$lor_lhs_false$5; } //@line 291 "readclause.cpp"
  
            $retval=1; //@line 293 "readclause.cpp"
            __label__ = 7;break $if_then$$if_end$2; //@line 293 "readclause.cpp"
          }
        } while(0);
  
        $retval=0; //@line 292 "readclause.cpp"
        ; //@line 292 "readclause.cpp"
      }
    } while(0);
  
    var $5=$retval; //@line 294 "readclause.cpp"
    ;
    return $5; //@line 294 "readclause.cpp"
    return null;
  }
  

  function _iswlower($c) {
    ;
    var __label__;
  
    var $retval;
    var $c_addr;
    $c_addr=$c;
    var $0=$c_addr; //@line 298 "readclause.cpp"
    var $cmp=((($0))>>>0) < 256; //@line 298 "readclause.cpp"
    var $1=$c_addr; //@line 299 "readclause.cpp"
    if ($cmp) { __label__ = 1;; } else { __label__ = 2;; } //@line 298 "readclause.cpp"
    $if_then$$if_end$2: do { 
      if (__label__ == 1) {
  
        var $call=_islower($1); //@line 299 "readclause.cpp"
        $retval=$call; //@line 299 "readclause.cpp"
        ; //@line 299 "readclause.cpp"
      }
      else if (__label__ == 2) {
  
        var $cmp1=((($1))>>>0) > 563; //@line 300 "readclause.cpp"
        if ($cmp1) { __label__ = 4;; } else { __label__ = 3;; } //@line 300 "readclause.cpp"
        $if_then3$$lor_lhs_false$5: do { 
          if (__label__ == 3) {
  
            var $2=$c_addr; //@line 300 "readclause.cpp"
            var $sub=((($2)-256)&4294967295); //@line 300 "readclause.cpp"
            var $arrayidx=((_walpha_tab+$sub)&4294967295); //@line 300 "readclause.cpp"
            var $3=HEAPU8[($arrayidx)]; //@line 300 "readclause.cpp"
            var $conv=((($3))&255); //@line 300 "readclause.cpp"
            var $cmp2=((($conv))|0)!=255; //@line 300 "readclause.cpp"
            if ($cmp2) { __label__ = 4;break $if_then3$$lor_lhs_false$5; } //@line 300 "readclause.cpp"
  
            $retval=1; //@line 302 "readclause.cpp"
            __label__ = 6;break $if_then$$if_end$2; //@line 302 "readclause.cpp"
          }
        } while(0);
  
        $retval=0; //@line 301 "readclause.cpp"
        ; //@line 301 "readclause.cpp"
      }
    } while(0);
  
    var $4=$retval; //@line 303 "readclause.cpp"
    ;
    return $4; //@line 303 "readclause.cpp"
    return null;
  }
  

  function _iswspace($c) {
    ;
    var __label__;
  
    var $retval;
    var $c_addr;
    $c_addr=$c;
    var $0=$c_addr; //@line 307 "readclause.cpp"
    var $cmp=((($0))>>>0) < 256; //@line 307 "readclause.cpp"
    if ($cmp) { __label__ = 1;; } else { __label__ = 2;; } //@line 307 "readclause.cpp"
    if (__label__ == 1) {
  
      var $1=$c_addr; //@line 308 "readclause.cpp"
      var $call=_isspace($1); //@line 308 "readclause.cpp"
      $retval=$call; //@line 308 "readclause.cpp"
      ; //@line 308 "readclause.cpp"
    }
    else if (__label__ == 2) {
  
      $retval=0; //@line 309 "readclause.cpp"
      ; //@line 309 "readclause.cpp"
    }
  
    var $2=$retval; //@line 310 "readclause.cpp"
    ;
    return $2; //@line 310 "readclause.cpp"
    return null;
  }
  

  function _iswpunct($c) {
    ;
    var __label__;
  
    var $retval;
    var $c_addr;
    $c_addr=$c;
    var $0=$c_addr; //@line 314 "readclause.cpp"
    var $cmp=((($0))>>>0) < 256; //@line 314 "readclause.cpp"
    if ($cmp) { __label__ = 1;; } else { __label__ = 2;; } //@line 314 "readclause.cpp"
    if (__label__ == 1) {
  
      var $1=$c_addr; //@line 315 "readclause.cpp"
      var $call=_ispunct($1); //@line 315 "readclause.cpp"
      $retval=$call; //@line 315 "readclause.cpp"
      ; //@line 315 "readclause.cpp"
    }
    else if (__label__ == 2) {
  
      $retval=0; //@line 316 "readclause.cpp"
      ; //@line 316 "readclause.cpp"
    }
  
    var $2=$retval; //@line 317 "readclause.cpp"
    ;
    return $2; //@line 317 "readclause.cpp"
    return null;
  }
  

  function _wcschr($str, $c) {
    ;
    var __label__;
  
    var $retval;
    var $str_addr;
    var $c_addr;
    $str_addr=$str;
    $c_addr=$c;
    ; //@line 321 "readclause.cpp"
    while(1) { 
  
      var $0=$str_addr; //@line 321 "readclause.cpp"
      var $1=HEAP32[(($0)>>2)]; //@line 321 "readclause.cpp"
      var $cmp=((($1))|0)!=0; //@line 321 "readclause.cpp"
      if (!($cmp)) { __label__ = 5;break ; } //@line 321 "readclause.cpp"
  
      var $2=$str_addr; //@line 323 "readclause.cpp"
      var $3=HEAP32[(($2)>>2)]; //@line 323 "readclause.cpp"
      var $4=$c_addr; //@line 323 "readclause.cpp"
      var $cmp1=((($3))|0)==((($4))|0); //@line 323 "readclause.cpp"
      var $5=$str_addr; //@line 324 "readclause.cpp"
      if ($cmp1) { __label__ = 3;break ; } //@line 323 "readclause.cpp"
  
      var $incdec_ptr=(($5+4)&4294967295); //@line 325 "readclause.cpp"
      $str_addr=$incdec_ptr; //@line 325 "readclause.cpp"
      __label__ = 1;continue ; //@line 326 "readclause.cpp"
    }
    if (__label__ == 5) {
  
      $retval=0; //@line 327 "readclause.cpp"
      ; //@line 327 "readclause.cpp"
    }
    else if (__label__ == 3) {
  
      $retval=$5; //@line 324 "readclause.cpp"
      ; //@line 324 "readclause.cpp"
    }
  
    var $6=$retval; //@line 328 "readclause.cpp"
    ;
    return $6; //@line 328 "readclause.cpp"
    return null;
  }
  

  function _wcslen($str) {
    ;
    var __label__;
    var __lastLabel__ = null;
  
    var $str_addr;
    var $ix;
    $str_addr=$str;
    $ix=0; //@line 334 "readclause.cpp"
    var $0=$str_addr; //@line 336 "readclause.cpp"
    var $1=HEAP32[(($0)>>2)]; //@line 336 "readclause.cpp"
    var $cmp1=((($1))|0)!=0; //@line 336 "readclause.cpp"
    var $2=$ix; //@line 338 "readclause.cpp"
    if ($cmp1) { __lastLabel__ = 0; __label__ = 1;; } else { __lastLabel__ = 0; __label__ = 2;; } //@line 336 "readclause.cpp"
    $while_body$$while_end$2: do { 
      if (__label__ == 1) {
        while(1) { 
  
          var $3=__lastLabel__ == 1 ? $6 : ($2);
          var $inc=((($3)+1)&4294967295); //@line 338 "readclause.cpp"
          $ix=$inc; //@line 338 "readclause.cpp"
          var $4=$str_addr; //@line 336 "readclause.cpp"
          var $5=HEAP32[(($4)>>2)]; //@line 336 "readclause.cpp"
          var $cmp=((($5))|0)!=0; //@line 336 "readclause.cpp"
          var $6=$ix; //@line 338 "readclause.cpp"
          if ($cmp) { __lastLabel__ = 1; __label__ = 1;continue ; } else { __lastLabel__ = 1; __label__ = 2;break $while_body$$while_end$2; } //@line 336 "readclause.cpp"
        }
      }
    } while(0);
  
    var $_lcssa=__lastLabel__ == 0 ? $2 : ($6);
    ;
    return $_lcssa; //@line 340 "readclause.cpp"
    return null;
  }
  

  function _wcstod($str, $tailptr) {
    var __stackBase__  = STACKTOP; STACKTOP += 80; _memset(__stackBase__, 0, 80);
    var __label__;
    var __lastLabel__ = null;
  
    var $str_addr;
    var $tailptr_addr;
    var $ix;
    var $buf=__stackBase__;
    $str_addr=$str;
    $tailptr_addr=$tailptr;
    var $0=$str_addr; //@line 348 "readclause.cpp"
    var $1=HEAP32[(($0)>>2)]; //@line 348 "readclause.cpp"
    var $call1=_isspace($1); //@line 348 "readclause.cpp"
    var $tobool2=((($call1))|0)!=0; //@line 348 "readclause.cpp"
    if ($tobool2) { __label__ = 1;; } else { __label__ = 2;; } //@line 348 "readclause.cpp"
    $while_body$$while_end$2: do { 
      if (__label__ == 1) {
        while(1) { 
  
          var $2=$str_addr; //@line 348 "readclause.cpp"
          var $incdec_ptr=(($2+4)&4294967295); //@line 348 "readclause.cpp"
          $str_addr=$incdec_ptr; //@line 348 "readclause.cpp"
          var $3=$str_addr; //@line 348 "readclause.cpp"
          var $4=HEAP32[(($3)>>2)]; //@line 348 "readclause.cpp"
          var $call=_isspace($4); //@line 348 "readclause.cpp"
          var $tobool=((($call))|0)!=0; //@line 348 "readclause.cpp"
          if ($tobool) { __label__ = 1;continue ; } else { __label__ = 2;break $while_body$$while_end$2; } //@line 348 "readclause.cpp"
        }
      }
    } while(0);
  
    $ix=0; //@line 349 "readclause.cpp"
    __lastLabel__ = 2; ; //@line 349 "readclause.cpp"
    while(1) { 
  
      var $5=__lastLabel__ == 5 ? $inc : (0); //@line 349 "readclause.cpp"
      var $cmp=((($5))|0) < 80; //@line 349 "readclause.cpp"
      if (!($cmp)) { __label__ = 6;break ; } //@line 349 "readclause.cpp"
  
      var $6=$ix; //@line 351 "readclause.cpp"
      var $7=$str_addr; //@line 351 "readclause.cpp"
      var $arrayidx=(($7+4*$6)&4294967295); //@line 351 "readclause.cpp"
      var $8=HEAP32[(($arrayidx)>>2)]; //@line 351 "readclause.cpp"
      var $conv=((($8)) & 255); //@line 351 "readclause.cpp"
      var $9=$ix; //@line 351 "readclause.cpp"
      var $arrayidx1=(($buf+$9)&4294967295); //@line 351 "readclause.cpp"
      HEAP8[($arrayidx1)]=$conv; //@line 351 "readclause.cpp"
      var $10=$ix; //@line 352 "readclause.cpp"
      var $arrayidx2=(($buf+$10)&4294967295); //@line 352 "readclause.cpp"
      var $11=HEAP8[($arrayidx2)]; //@line 352 "readclause.cpp"
      var $conv3=(tempInt=(($11)),(tempInt>=128?tempInt-256:tempInt)); //@line 352 "readclause.cpp"
      var $call4=_isspace($conv3); //@line 352 "readclause.cpp"
      var $tobool5=((($call4))|0)!=0; //@line 352 "readclause.cpp"
      if ($tobool5) { __label__ = 6;break ; } //@line 352 "readclause.cpp"
  
      var $12=$ix; //@line 349 "readclause.cpp"
      var $inc=((($12)+1)&4294967295); //@line 349 "readclause.cpp"
      $ix=$inc; //@line 349 "readclause.cpp"
      __lastLabel__ = 5; __label__ = 3;continue ; //@line 349 "readclause.cpp"
    }
  
    var $13=$ix; //@line 355 "readclause.cpp"
    var $14=$str_addr; //@line 355 "readclause.cpp"
    var $arrayidx6=(($14+4*$13)&4294967295); //@line 355 "readclause.cpp"
    var $15=$tailptr_addr; //@line 355 "readclause.cpp"
    HEAP32[(($15)>>2)]=$arrayidx6; //@line 355 "readclause.cpp"
    var $arraydecay=(($buf)&4294967295); //@line 356 "readclause.cpp"
    var $call7=_atof($arraydecay); //@line 356 "readclause.cpp"
    var $conv8=($call7); //@line 356 "readclause.cpp"
    STACKTOP = __stackBase__;
    return $conv8; //@line 356 "readclause.cpp"
    return null;
  }
  

  function __Z9towlower2j($c) {
    ;
    var __label__;
  
    var $c_addr;
    $c_addr=$c;
    var $0=$c_addr; //@line 366 "readclause.cpp"
    var $cmp=((($0))|0)==73; //@line 366 "readclause.cpp"
    if ($cmp) { __label__ = 1;; } else { __label__ = 3;; } //@line 366 "readclause.cpp"
    $if_then$$if_end3$2: do { 
      if (__label__ == 1) {
  
        var $1=HEAP32[((_translator)>>2)]; //@line 368 "readclause.cpp"
        var $translator_name=(($1+280)&4294967295); //@line 368 "readclause.cpp"
        var $2=HEAP32[(($translator_name)>>2)]; //@line 368 "readclause.cpp"
        var $cmp1=((($2))|0)==29810; //@line 368 "readclause.cpp"
        if (!($cmp1)) { __label__ = 3;break $if_then$$if_end3$2; } //@line 368 "readclause.cpp"
  
        $c_addr=305; //@line 370 "readclause.cpp"
        ; //@line 371 "readclause.cpp"
      }
    } while(0);
  
    var $3=$c_addr; //@line 377 "readclause.cpp"
    var $call=_towlower($3); //@line 377 "readclause.cpp"
    ;
    return $call; //@line 377 "readclause.cpp"
    return null;
  }
  

  function __Z3Eofv() {
    ;
    var __label__;
  
    var $retval;
    var $0=HEAP32[((__ZL10ungot_char)>>2)]; //@line 404 "readclause.cpp"
    var $cmp=((($0))|0)!=0; //@line 404 "readclause.cpp"
    if ($cmp) { __label__ = 1;; } else { __label__ = 2;; } //@line 404 "readclause.cpp"
    if (__label__ == 1) {
  
      $retval=0; //@line 405 "readclause.cpp"
      ; //@line 405 "readclause.cpp"
    }
    else if (__label__ == 2) {
  
      var $1=HEAP32[((__ZL7f_input)>>2)]; //@line 407 "readclause.cpp"
      var $cmp1=((($1))|0)!=0; //@line 407 "readclause.cpp"
      if ($cmp1) { __label__ = 3;; } else { __label__ = 4;; } //@line 407 "readclause.cpp"
      if (__label__ == 3) {
  
        var $2=HEAP32[((__ZL7f_input)>>2)]; //@line 408 "readclause.cpp"
        var $_flags=(($2+12)&4294967295); //@line 408 "readclause.cpp"
        var $3=HEAP16[(($_flags)>>1)]; //@line 408 "readclause.cpp"
        var $conv=(tempInt=(($3)),(tempInt>=32768?tempInt-65536:tempInt)); //@line 408 "readclause.cpp"
        var $and=($conv) & 32; //@line 408 "readclause.cpp"
        var $cmp3=((($and))|0)!=0; //@line 408 "readclause.cpp"
        var $conv4=((($cmp3))&1); //@line 408 "readclause.cpp"
        $retval=$conv4; //@line 408 "readclause.cpp"
        ; //@line 408 "readclause.cpp"
      }
      else if (__label__ == 4) {
  
        var $_b=HEAPU8[(__ZL12end_of_input_b)];
        var $4=((($_b))&1);
        $retval=$4; //@line 410 "readclause.cpp"
        ; //@line 410 "readclause.cpp"
      }
    }
  
    var $5=$retval; //@line 411 "readclause.cpp"
    ;
    return $5; //@line 411 "readclause.cpp"
    return null;
  }
  

  function __Z10Read4BytesP7__sFILE($f) {
    ;
    var __label__;
  
    var $f_addr;
    var $ix;
    var $c;
    var $acc;
    $f_addr=$f;
    $acc=0; //@line 701 "readclause.cpp"
    $ix=0; //@line 703 "readclause.cpp"
    ; //@line 703 "readclause.cpp"
    while(1) { 
  
      var $0=$f_addr; //@line 705 "readclause.cpp"
      var $call=_fgetc($0); //@line 705 "readclause.cpp"
      var $and=($call) & 255; //@line 705 "readclause.cpp"
      var $conv=((($and)) & 255); //@line 705 "readclause.cpp"
      $c=$conv; //@line 705 "readclause.cpp"
      var $1=$c; //@line 706 "readclause.cpp"
      var $conv1=((($1))&255); //@line 706 "readclause.cpp"
      var $2=$ix; //@line 706 "readclause.cpp"
      var $mul=((($2)*8)&4294967295); //@line 706 "readclause.cpp"
      var $shl=($conv1) << ($mul); //@line 706 "readclause.cpp"
      var $3=$acc; //@line 706 "readclause.cpp"
      var $add=((($shl)+($3))&4294967295); //@line 706 "readclause.cpp"
      $acc=$add; //@line 706 "readclause.cpp"
      var $4=$ix; //@line 703 "readclause.cpp"
      var $inc=((($4)+1)&4294967295); //@line 703 "readclause.cpp"
      $ix=$inc; //@line 703 "readclause.cpp"
      var $cmp=((($inc))|0) < 4; //@line 703 "readclause.cpp"
      if ($cmp) { __label__ = 1;continue ; } else { __label__ = 2;break ; } //@line 703 "readclause.cpp"
    }
  
    var $5=$acc; //@line 708 "readclause.cpp"
    ;
    return $5; //@line 708 "readclause.cpp"
    return null;
  }
  

  function __Z11AddNameDataPKci($name, $wide) {
    ;
    var __label__;
  
    var $retval;
    var $name_addr;
    var $wide_addr;
    var $ix;
    var $len;
    var $vp;
    $name_addr=$name;
    $wide_addr=$wide;
    var $0=$wide_addr; //@line 1371 "readclause.cpp"
    var $tobool=((($0))|0)!=0; //@line 1371 "readclause.cpp"
    var $1=$name_addr; //@line 1373 "readclause.cpp"
    if ($tobool) { __label__ = 1;; } else { __label__ = 2;; } //@line 1371 "readclause.cpp"
    if (__label__ == 1) {
  
      var $2=$1; //@line 1373 "readclause.cpp"
      var $call=_wcslen($2); //@line 1373 "readclause.cpp"
      var $add=((($call)+1)&4294967295); //@line 1373 "readclause.cpp"
      var $mul=((($add)*4)&4294967295); //@line 1373 "readclause.cpp"
      $len=$mul; //@line 1373 "readclause.cpp"
      var $3=HEAP32[((__ZL10n_namedata)>>2)]; //@line 1374 "readclause.cpp"
      var $sub=((($3)+3)&4294967295); //@line 1374 "readclause.cpp"
      var $rem=((($sub))>>>0)%4; //@line 1374 "readclause.cpp"
      HEAP32[((__ZL10n_namedata)>>2)]=$rem; //@line 1374 "readclause.cpp"
      ; //@line 1375 "readclause.cpp"
    }
    else if (__label__ == 2) {
  
      var $call2=_strlen($1); //@line 1378 "readclause.cpp"
      var $add3=((($call2)+1)&4294967295); //@line 1378 "readclause.cpp"
      $len=$add3; //@line 1378 "readclause.cpp"
      ;
    }
  
    var $4=HEAP32[((__ZL11namedata_ix)>>2)]; //@line 1381 "readclause.cpp"
    var $5=$len; //@line 1381 "readclause.cpp"
    var $add4=((($5)+($4))&4294967295); //@line 1381 "readclause.cpp"
    var $6=HEAP32[((__ZL10n_namedata)>>2)]; //@line 1381 "readclause.cpp"
    var $cmp=((($add4))|0) >= ((($6))|0); //@line 1381 "readclause.cpp"
    if ($cmp) { __label__ = 4;; } else { __label__ = 7;; } //@line 1381 "readclause.cpp"
    $if_then5$$if_end14$6: do { 
      if (__label__ == 4) {
  
        var $7=HEAP32[((_namedata)>>2)]; //@line 1384 "readclause.cpp"
        var $8=HEAP32[((__ZL11namedata_ix)>>2)]; //@line 1384 "readclause.cpp"
        var $9=$len; //@line 1384 "readclause.cpp"
        var $add6=((($8)+300)&4294967295); //@line 1384 "readclause.cpp"
        var $add7=((($add6)+($9))&4294967295); //@line 1384 "readclause.cpp"
        var $call8=_realloc($7, $add7); //@line 1384 "readclause.cpp"
        $vp=$call8; //@line 1384 "readclause.cpp"
        var $cmp9=((($call8))|0)==0; //@line 1384 "readclause.cpp"
        if ($cmp9) { __label__ = 5;; } else { __label__ = 6;; } //@line 1384 "readclause.cpp"
        if (__label__ == 5) {
  
          $retval=-1; //@line 1385 "readclause.cpp"
          __label__ = 8;break $if_then5$$if_end14$6; //@line 1385 "readclause.cpp"
        }
        else if (__label__ == 6) {
  
          var $10=$vp; //@line 1387 "readclause.cpp"
          HEAP32[((_namedata)>>2)]=$10; //@line 1387 "readclause.cpp"
          var $11=HEAP32[((__ZL11namedata_ix)>>2)]; //@line 1388 "readclause.cpp"
          var $12=$len; //@line 1388 "readclause.cpp"
          var $add12=((($11)+300)&4294967295); //@line 1388 "readclause.cpp"
          var $add13=((($add12)+($12))&4294967295); //@line 1388 "readclause.cpp"
          HEAP32[((__ZL10n_namedata)>>2)]=$add13; //@line 1388 "readclause.cpp"
          __label__ = 7;break $if_then5$$if_end14$6; //@line 1389 "readclause.cpp"
        }
      }
    } while(0);
    if (__label__ == 7) {
  
      var $13=HEAP32[((__ZL11namedata_ix)>>2)]; //@line 1390 "readclause.cpp"
      $ix=$13; //@line 1390 "readclause.cpp"
      var $14=HEAP32[((_namedata)>>2)]; //@line 1390 "readclause.cpp"
      var $arrayidx=(($14+$13)&4294967295); //@line 1390 "readclause.cpp"
      var $15=$name_addr; //@line 1390 "readclause.cpp"
      var $16=$len; //@line 1390 "readclause.cpp"
      var $src$, $dest$, $stop$, $stop4$, $fast$;
      $src$ = $15;
      $dest$ = $arrayidx;
      $stop$ = $src$ + $16;
      if (($dest$%4) == ($src$%4) && $16 > 8) {
        while ($src$%4 !== 0 && $src$ < $stop$) {
          ; HEAP8[$dest$++] = HEAP8[$src$++];
        }
        $src$ >>= 2;
        $dest$ >>= 2;
        $stop4$ = $stop$ >> 2;
        while ($src$ < $stop4$) {
      ;;;    HEAP32[$dest$++] = HEAP32[$src$++];
        }
        $src$ <<= 2;
        $dest$ <<= 2;
      }while ($src$ < $stop$) {
        ; HEAP8[$dest$++] = HEAP8[$src$++];
      }; //@line 1390 "readclause.cpp"
      var $17=$len; //@line 1391 "readclause.cpp"
      var $18=HEAP32[((__ZL11namedata_ix)>>2)]; //@line 1391 "readclause.cpp"
      var $add15=((($18)+($17))&4294967295); //@line 1391 "readclause.cpp"
      HEAP32[((__ZL11namedata_ix)>>2)]=$add15; //@line 1391 "readclause.cpp"
      var $19=$ix; //@line 1392 "readclause.cpp"
      $retval=$19; //@line 1392 "readclause.cpp"
      ; //@line 1392 "readclause.cpp"
    }
  
    var $20=$retval; //@line 1393 "readclause.cpp"
    ;
    return $20; //@line 1393 "readclause.cpp"
    return null;
  }
  

  function __Z13SetVoiceStackP12espeak_VOICE($v) {
    ;
    var __label__;
  
    var $v_addr;
    var $sp;
    $v_addr=$v;
    $sp=((__ZL10ssml_stack)&4294967295); //@line 1399 "readclause.cpp"
    var $0=$v_addr; //@line 1401 "readclause.cpp"
    var $cmp=((($0))|0)==0; //@line 1401 "readclause.cpp"
    if ($cmp) { __label__ = 1;; } else { __label__ = 2;; } //@line 1401 "readclause.cpp"
    if (__label__ == 1) {
  
      var $1=$sp; //@line 1403 "readclause.cpp"
      var $2=$1; //@line 1403 "readclause.cpp"
      var $dest$, $stop$, $stop4$, $fast$, $value4$;
      $dest$ = $2;
      $stop$ = $dest$ + 76;
      $value4$ = 0;
      if ($value4$ < 0) $value4$ += 256;
      $value4$ = $value4$ + ($value4$<<8) + ($value4$<<16) + ($value4$*16777216);
      while ($dest$%4 !== 0 && $dest$ < $stop$) {
        ;; HEAP8[$dest$++] = 0;
      }
      $dest$ >>= 2;
      $stop4$ = $stop$ >> 2;
      while ($dest$ < $stop4$) {
      ;;;;;;;  HEAP32[$dest$++] = $value4$;
      }
      $dest$ <<= 2;
      while ($dest$ < $stop$) {
        ;; HEAP8[$dest$++] = 0;
      }; //@line 1403 "readclause.cpp"
      ; //@line 1404 "readclause.cpp"
    }
    else if (__label__ == 2) {
  
      var $3=$v_addr; //@line 1406 "readclause.cpp"
      var $languages=(($3+4)&4294967295); //@line 1406 "readclause.cpp"
      var $4=HEAP32[(($languages)>>2)]; //@line 1406 "readclause.cpp"
      var $cmp1=((($4))|0)!=0; //@line 1406 "readclause.cpp"
      if ($cmp1) { __label__ = 3;; } else { __label__ = 4;; } //@line 1406 "readclause.cpp"
      if (__label__ == 3) {
  
        var $5=$sp; //@line 1407 "readclause.cpp"
        var $language=(($5+56)&4294967295); //@line 1407 "readclause.cpp"
        var $arraydecay=(($language)&4294967295); //@line 1407 "readclause.cpp"
        var $6=$v_addr; //@line 1407 "readclause.cpp"
        var $languages3=(($6+4)&4294967295); //@line 1407 "readclause.cpp"
        var $7=HEAP32[(($languages3)>>2)]; //@line 1407 "readclause.cpp"
        var $call=_strcpy($arraydecay, $7); //@line 1407 "readclause.cpp"
        ; //@line 1407 "readclause.cpp"
      }
  
      var $8=$v_addr; //@line 1408 "readclause.cpp"
      var $name=(($8)&4294967295); //@line 1408 "readclause.cpp"
      var $9=HEAP32[(($name)>>2)]; //@line 1408 "readclause.cpp"
      var $cmp5=((($9))|0)!=0; //@line 1408 "readclause.cpp"
      if ($cmp5) { __label__ = 5;; } else { __label__ = 6;; } //@line 1408 "readclause.cpp"
      if (__label__ == 5) {
  
        var $10=$sp; //@line 1409 "readclause.cpp"
        var $voice_name=(($10+16)&4294967295); //@line 1409 "readclause.cpp"
        var $arraydecay7=(($voice_name)&4294967295); //@line 1409 "readclause.cpp"
        var $11=$v_addr; //@line 1409 "readclause.cpp"
        var $name8=(($11)&4294967295); //@line 1409 "readclause.cpp"
        var $12=HEAP32[(($name8)>>2)]; //@line 1409 "readclause.cpp"
        var $call9=_strcpy($arraydecay7, $12); //@line 1409 "readclause.cpp"
        ; //@line 1409 "readclause.cpp"
      }
  
      var $13=$v_addr; //@line 1410 "readclause.cpp"
      var $variant=(($13+14)&4294967295); //@line 1410 "readclause.cpp"
      var $14=HEAPU8[($variant)]; //@line 1410 "readclause.cpp"
      var $conv=((($14))&255); //@line 1410 "readclause.cpp"
      var $15=$sp; //@line 1410 "readclause.cpp"
      var $voice_variant=(($15+4)&4294967295); //@line 1410 "readclause.cpp"
      HEAP32[(($voice_variant)>>2)]=$conv; //@line 1410 "readclause.cpp"
      var $16=$v_addr; //@line 1411 "readclause.cpp"
      var $age=(($16+13)&4294967295); //@line 1411 "readclause.cpp"
      var $17=HEAPU8[($age)]; //@line 1411 "readclause.cpp"
      var $conv11=((($17))&255); //@line 1411 "readclause.cpp"
      var $18=$sp; //@line 1411 "readclause.cpp"
      var $voice_age=(($18+12)&4294967295); //@line 1411 "readclause.cpp"
      HEAP32[(($voice_age)>>2)]=$conv11; //@line 1411 "readclause.cpp"
      var $19=$v_addr; //@line 1412 "readclause.cpp"
      var $gender=(($19+12)&4294967295); //@line 1412 "readclause.cpp"
      var $20=HEAPU8[($gender)]; //@line 1412 "readclause.cpp"
      var $conv12=((($20))&255); //@line 1412 "readclause.cpp"
      var $21=$sp; //@line 1412 "readclause.cpp"
      var $voice_gender=(($21+8)&4294967295); //@line 1412 "readclause.cpp"
      HEAP32[(($voice_gender)>>2)]=$conv12; //@line 1412 "readclause.cpp"
      ; //@line 1413 "readclause.cpp"
    }
  
    ;
    return; //@line 1413 "readclause.cpp"
    return;
  }
  

  function __Z10ReadClauseP10TranslatorP7__sFILEPcPsPiiS5_S3_($tr, $f_in, $buf, $charix, $charix_top, $n_buf, $tone_type, $voice_change) {
    var __stackBase__  = STACKTOP; STACKTOP += 1136; _memset(__stackBase__, 0, 1136);
    var __label__;
    var __lastLabel__ = null;
  
    var $retval;
    var $tr_addr;
    var $f_in_addr;
    var $buf_addr;
    var $charix_addr;
    var $charix_top_addr;
    var $n_buf_addr;
    var $tone_type_addr;
    var $voice_change_addr;
    var $c1=__stackBase__;
    var $c2=__stackBase__+4;
    var $cprev;
    var $cprev2;
    var $c_next;
    var $parag;
    var $ix=__stackBase__+8;
    var $j;
    var $nl_count;
    var $linelength;
    var $phoneme_mode;
    var $n_xml_buf;
    var $terminator;
    var $punct;
    var $found;
    var $any_alnum;
    var $self_closing;
    var $punct_data;
    var $is_end_clause;
    var $announced_punctuation;
    var $prev_announced_punctuation;
    var $stressed_word;
    var $end_clause_after_tag;
    var $end_clause_index;
    var $xml_buf=__stackBase__+12;
    var $xml_buf2=__stackBase__+1040;
    var $p_word;
    var $text_buf=__stackBase__+1064;
    var $text_buf2=__stackBase__+1104;
    var $p2;
    $tr_addr=$tr;
    $f_in_addr=$f_in;
    $buf_addr=$buf;
    $charix_addr=$charix;
    $charix_top_addr=$charix_top;
    $n_buf_addr=$n_buf;
    $tone_type_addr=$tone_type;
    $voice_change_addr=$voice_change;
    HEAP32[(($c1)>>2)]=32; //@line 2047 "readclause.cpp"
    $cprev=32; //@line 2049 "readclause.cpp"
    $cprev2=32; //@line 2050 "readclause.cpp"
    HEAP32[(($ix)>>2)]=0; //@line 2053 "readclause.cpp"
    $linelength=0; //@line 2056 "readclause.cpp"
    $phoneme_mode=0; //@line 2057 "readclause.cpp"
    $any_alnum=0; //@line 2062 "readclause.cpp"
    $punct_data=0; //@line 2064 "readclause.cpp"
    $announced_punctuation=0; //@line 2066 "readclause.cpp"
    $stressed_word=0; //@line 2068 "readclause.cpp"
    $end_clause_after_tag=0; //@line 2069 "readclause.cpp"
    $end_clause_index=0; //@line 2070 "readclause.cpp"
    var $_b=HEAPU8[(__ZL19clear_skipping_text_b)];
    var $0=((($_b))&1);
    var $tobool=((($0))|0)!=0; //@line 2078 "readclause.cpp"
    if ($tobool) { __label__ = 1;; } else { __label__ = 2;; } //@line 2078 "readclause.cpp"
    if (__label__ == 1) {
  
      HEAP32[((_skipping_text)>>2)]=0; //@line 2080 "readclause.cpp"
      HEAP8[(__ZL19clear_skipping_text_b)]=0;
      ; //@line 2082 "readclause.cpp"
    }
  
    var $1=$tr_addr; //@line 2084 "readclause.cpp"
    var $phonemes_repeat_count=(($1+652)&4294967295); //@line 2084 "readclause.cpp"
    HEAP32[(($phonemes_repeat_count)>>2)]=0; //@line 2084 "readclause.cpp"
    var $2=$tr_addr; //@line 2085 "readclause.cpp"
    var $clause_upper_count=(($2+8304)&4294967295); //@line 2085 "readclause.cpp"
    HEAP32[(($clause_upper_count)>>2)]=0; //@line 2085 "readclause.cpp"
    var $3=$tr_addr; //@line 2086 "readclause.cpp"
    var $clause_lower_count=(($3+8308)&4294967295); //@line 2086 "readclause.cpp"
    HEAP32[(($clause_lower_count)>>2)]=0; //@line 2086 "readclause.cpp"
    HEAP8[(__ZL12end_of_input_b)]=0;
    var $4=$tone_type_addr; //@line 2088 "readclause.cpp"
    HEAP32[(($4)>>2)]=0; //@line 2088 "readclause.cpp"
    var $5=$voice_change_addr; //@line 2089 "readclause.cpp"
    HEAP8[($5)]=0; //@line 2089 "readclause.cpp"
    var $6=$f_in_addr; //@line 2091 "readclause.cpp"
    HEAP32[((__ZL7f_input)>>2)]=$6; //@line 2091 "readclause.cpp"
    var $7=HEAP32[((__ZL10ungot_word)>>2)]; //@line 2093 "readclause.cpp"
    var $cmp=((($7))|0)!=0; //@line 2093 "readclause.cpp"
    if ($cmp) { __label__ = 3;; } else { __label__ = 4;; } //@line 2093 "readclause.cpp"
    if (__label__ == 3) {
  
      var $8=$buf_addr; //@line 2095 "readclause.cpp"
      var $9=HEAP32[((__ZL10ungot_word)>>2)]; //@line 2095 "readclause.cpp"
      var $call=_strcpy($8, $9); //@line 2095 "readclause.cpp"
      var $10=HEAP32[((__ZL10ungot_word)>>2)]; //@line 2096 "readclause.cpp"
      var $call2=_strlen($10); //@line 2096 "readclause.cpp"
      var $11=HEAP32[(($ix)>>2)]; //@line 2096 "readclause.cpp"
      var $add=((($11)+($call2))&4294967295); //@line 2096 "readclause.cpp"
      HEAP32[(($ix)>>2)]=$add; //@line 2096 "readclause.cpp"
      HEAP32[((__ZL10ungot_word)>>2)]=0; //@line 2097 "readclause.cpp"
      ; //@line 2098 "readclause.cpp"
    }
  
    var $12=HEAP32[((__ZL11ungot_char2)>>2)]; //@line 2100 "readclause.cpp"
    var $cmp4=((($12))|0)!=0; //@line 2100 "readclause.cpp"
    if ($cmp4) { __label__ = 5;; } else { __label__ = 6;; } //@line 2100 "readclause.cpp"
    if (__label__ == 5) {
  
      var $13=HEAP32[((__ZL11ungot_char2)>>2)]; //@line 2102 "readclause.cpp"
      HEAP32[(($c2)>>2)]=$13; //@line 2102 "readclause.cpp"
      ; //@line 2103 "readclause.cpp"
    }
    else if (__label__ == 6) {
  
      var $call6=__ZL4GetCv(); //@line 2106 "readclause.cpp"
      HEAP32[(($c2)>>2)]=$call6; //@line 2106 "readclause.cpp"
      ;
    }
  
    var $arrayidx82=(($xml_buf2)&4294967295); //@line 2172 "readclause.cpp"
    var $arrayidx86=(($xml_buf2)&4294967295); //@line 2176 "readclause.cpp"
    var $arrayidx90=(($xml_buf2+1)&4294967295); //@line 2179 "readclause.cpp"
    var $arrayidx94=(($xml_buf2+2)&4294967295); //@line 2180 "readclause.cpp"
    var $arrayidx97=(($xml_buf2+1)&4294967295); //@line 2182 "readclause.cpp"
    var $arraydecay=(($xml_buf2)&4294967295); //@line 2186 "readclause.cpp"
    var $arraydecay164=(($xml_buf)&4294967295); //@line 2245 "readclause.cpp"
    var $arraydecay324=(($text_buf2)&4294967295); //@line 2396 "readclause.cpp"
    var $arraydecay328=(($text_buf)&4294967295); //@line 2398 "readclause.cpp"
    var $arraydecay329=(($text_buf2)&4294967295); //@line 2398 "readclause.cpp"
    var $arraydecay331=(($text_buf)&4294967295); //@line 2399 "readclause.cpp"
    var $arraydecay337=(($text_buf)&4294967295); //@line 2402 "readclause.cpp"
    ; //@line 2109 "readclause.cpp"
    $while_cond$12: while(1) { 
  
      var $call8=__Z3Eofv(); //@line 2109 "readclause.cpp"
      var $tobool9=((($call8))|0)==0; //@line 2109 "readclause.cpp"
      var $14=HEAP32[((__ZL10ungot_char)>>2)]; //@line 2109 "readclause.cpp"
      var $cmp10=((($14))|0)!=0; //@line 2109 "readclause.cpp"
      var $or_cond=($tobool9) | ($cmp10); //@line 2109 "readclause.cpp"
      var $15=HEAP32[((__ZL11ungot_char2)>>2)]; //@line 2109 "readclause.cpp"
      var $cmp12=((($15))|0)!=0; //@line 2109 "readclause.cpp"
      var $or_cond3=($or_cond) | ($cmp12); //@line 2109 "readclause.cpp"
      var $16=HEAP32[((__ZZ10ReadClauseP10TranslatorP7__sFILEPcPsPiiS5_S3_E15ungot_string_ix)>>2)]; //@line 2109 "readclause.cpp"
      var $cmp13=((($16))|0) >= 0; //@line 2109 "readclause.cpp"
      var $or_cond19=($or_cond3) | ($cmp13); //@line 2109 "readclause.cpp"
      if (!($or_cond19)) { __label__ = 256;break $while_cond$12; } //@line 2109 "readclause.cpp"
  
      var $17=HEAP32[(($c1)>>2)]; //@line 2111 "readclause.cpp"
      var $call14=_iswalnum($17); //@line 2111 "readclause.cpp"
      var $tobool15=((($call14))|0)!=0; //@line 2111 "readclause.cpp"
      if ($tobool15) { __label__ = 16;; } else { __label__ = 10;; } //@line 2111 "readclause.cpp"
      $if_end26$$if_then16$15: do { 
        if (__label__ == 10) {
  
          var $18=HEAP32[((_end_character_position)>>2)]; //@line 2113 "readclause.cpp"
          var $cmp17=((($18))|0) > 0; //@line 2113 "readclause.cpp"
          if ($cmp17) { __label__ = 11;; } else { __label__ = 13;; } //@line 2113 "readclause.cpp"
          if (__label__ == 11) {
  
            var $19=HEAP32[((_count_characters)>>2)]; //@line 2113 "readclause.cpp"
            var $20=HEAP32[((_end_character_position)>>2)]; //@line 2113 "readclause.cpp"
            var $cmp18=((($19))|0) > ((($20))|0); //@line 2113 "readclause.cpp"
            if ($cmp18) { __label__ = 12;break $while_cond$12; } //@line 2113 "readclause.cpp"
          }
  
          var $21=HEAP32[((_skip_characters)>>2)]; //@line 2119 "readclause.cpp"
          var $cmp21=((($21))|0) > 0; //@line 2119 "readclause.cpp"
          if (!($cmp21)) { __label__ = 16;break $if_end26$$if_then16$15; } //@line 2119 "readclause.cpp"
  
          var $22=HEAP32[((_count_characters)>>2)]; //@line 2119 "readclause.cpp"
          var $23=HEAP32[((_skip_characters)>>2)]; //@line 2119 "readclause.cpp"
          var $cmp23=((($22))|0) > ((($23))|0); //@line 2119 "readclause.cpp"
          if ($cmp23) { __label__ = 15;break $while_cond$12; } //@line 2119 "readclause.cpp"
        }
      } while(0);
  
      var $25=$cprev; //@line 2130 "readclause.cpp"
      $cprev2=$25; //@line 2130 "readclause.cpp"
      var $26=HEAP32[(($c1)>>2)]; //@line 2131 "readclause.cpp"
      $cprev=$26; //@line 2131 "readclause.cpp"
      var $27=HEAP32[(($c2)>>2)]; //@line 2132 "readclause.cpp"
      HEAP32[(($c1)>>2)]=$27; //@line 2132 "readclause.cpp"
      var $28=HEAP32[((__ZZ10ReadClauseP10TranslatorP7__sFILEPcPsPiiS5_S3_E15ungot_string_ix)>>2)]; //@line 2134 "readclause.cpp"
      var $cmp27=((($28))|0) >= 0; //@line 2134 "readclause.cpp"
      if ($cmp27) { __label__ = 17;; } else { __label__ = 19;; } //@line 2134 "readclause.cpp"
      $if_then28$$if_end32$22: do { 
        if (__label__ == 17) {
  
          var $29=HEAP32[((__ZZ10ReadClauseP10TranslatorP7__sFILEPcPsPiiS5_S3_E15ungot_string_ix)>>2)]; //@line 2136 "readclause.cpp"
          var $arrayidx=((__ZZ10ReadClauseP10TranslatorP7__sFILEPcPsPiiS5_S3_E12ungot_string+$29)&4294967295); //@line 2136 "readclause.cpp"
          var $30=HEAP8[($arrayidx)]; //@line 2136 "readclause.cpp"
          var $conv=(tempInt=(($30)),(tempInt>=128?tempInt-256:tempInt)); //@line 2136 "readclause.cpp"
          var $cmp29=((($conv))|0)==0; //@line 2136 "readclause.cpp"
          if (!($cmp29)) { __label__ = 19;break $if_then28$$if_end32$22; } //@line 2136 "readclause.cpp"
  
          HEAP32[((__ZZ10ReadClauseP10TranslatorP7__sFILEPcPsPiiS5_S3_E15ungot_string_ix)>>2)]=-1; //@line 2137 "readclause.cpp"
          ; //@line 2137 "readclause.cpp"
        }
      } while(0);
  
      var $31=HEAP32[((__ZZ10ReadClauseP10TranslatorP7__sFILEPcPsPiiS5_S3_E15ungot_string_ix)>>2)]; //@line 2140 "readclause.cpp"
      var $cmp33=((($31))|0)==0; //@line 2140 "readclause.cpp"
      var $32=HEAP32[((__ZL11ungot_char2)>>2)]; //@line 2140 "readclause.cpp"
      var $cmp35=((($32))|0)==0; //@line 2140 "readclause.cpp"
      var $or_cond4=($cmp33) & ($cmp35); //@line 2140 "readclause.cpp"
      if ($or_cond4) { __lastLabel__ = 19; __label__ = 20;; } else { __lastLabel__ = 19; __label__ = 21;; } //@line 2140 "readclause.cpp"
      if (__label__ == 20) {
  
        var $33=HEAP32[((__ZZ10ReadClauseP10TranslatorP7__sFILEPcPsPiiS5_S3_E15ungot_string_ix)>>2)]; //@line 2142 "readclause.cpp"
        var $inc=((($33)+1)&4294967295); //@line 2142 "readclause.cpp"
        HEAP32[((__ZZ10ReadClauseP10TranslatorP7__sFILEPcPsPiiS5_S3_E15ungot_string_ix)>>2)]=$inc; //@line 2142 "readclause.cpp"
        var $arrayidx37=((__ZZ10ReadClauseP10TranslatorP7__sFILEPcPsPiiS5_S3_E12ungot_string+$33)&4294967295); //@line 2142 "readclause.cpp"
        var $34=HEAP8[($arrayidx37)]; //@line 2142 "readclause.cpp"
        var $conv38=(tempInt=(($34)),(tempInt>=128?tempInt-256:tempInt)); //@line 2142 "readclause.cpp"
        HEAP32[(($c1)>>2)]=$conv38; //@line 2142 "readclause.cpp"
        __lastLabel__ = 20; ; //@line 2143 "readclause.cpp"
      }
  
      var $35=__lastLabel__ == 20 ? $inc : ($31); //@line 2144 "readclause.cpp"
      var $cmp40=((($35))|0) >= 0; //@line 2144 "readclause.cpp"
      if ($cmp40) { __label__ = 22;; } else { __label__ = 23;; } //@line 2144 "readclause.cpp"
      $if_then41$$if_else45$29: do { 
        if (__label__ == 22) {
  
          var $36=HEAP32[((__ZZ10ReadClauseP10TranslatorP7__sFILEPcPsPiiS5_S3_E15ungot_string_ix)>>2)]; //@line 2146 "readclause.cpp"
          var $inc42=((($36)+1)&4294967295); //@line 2146 "readclause.cpp"
          HEAP32[((__ZZ10ReadClauseP10TranslatorP7__sFILEPcPsPiiS5_S3_E15ungot_string_ix)>>2)]=$inc42; //@line 2146 "readclause.cpp"
          var $arrayidx43=((__ZZ10ReadClauseP10TranslatorP7__sFILEPcPsPiiS5_S3_E12ungot_string+$36)&4294967295); //@line 2146 "readclause.cpp"
          var $37=HEAP8[($arrayidx43)]; //@line 2146 "readclause.cpp"
          var $conv44=(tempInt=(($37)),(tempInt>=128?tempInt-256:tempInt)); //@line 2146 "readclause.cpp"
          HEAP32[(($c2)>>2)]=$conv44; //@line 2146 "readclause.cpp"
          ; //@line 2147 "readclause.cpp"
        }
        else if (__label__ == 23) {
  
          var $call46=__ZL4GetCv(); //@line 2150 "readclause.cpp"
          HEAP32[(($c2)>>2)]=$call46; //@line 2150 "readclause.cpp"
          var $call47=__Z3Eofv(); //@line 2152 "readclause.cpp"
          var $tobool48=((($call47))|0)!=0; //@line 2152 "readclause.cpp"
          if (!($tobool48)) { __label__ = 25;break $if_then41$$if_else45$29; } //@line 2152 "readclause.cpp"
  
          HEAP32[(($c2)>>2)]=32; //@line 2154 "readclause.cpp"
          ; //@line 2155 "readclause.cpp"
        }
      } while(0);
  
      HEAP32[((__ZL11ungot_char2)>>2)]=0; //@line 2157 "readclause.cpp"
      var $38=HEAP32[((_option_ssml)>>2)]; //@line 2159 "readclause.cpp"
      var $tobool52=((($38))|0)!=0; //@line 2159 "readclause.cpp"
      if ($tobool52) { __label__ = 26;; } else { __label__ = 72;; } //@line 2159 "readclause.cpp"
      $land_lhs_true53$$if_end184$34: do { 
        if (__label__ == 26) {
  
          var $39=$phoneme_mode; //@line 2159 "readclause.cpp"
          var $cmp54=((($39))|0)==0; //@line 2159 "readclause.cpp"
          if (!($cmp54)) { __label__ = 72;break $land_lhs_true53$$if_end184$34; } //@line 2159 "readclause.cpp"
  
          var $40=HEAP32[((__ZL19ssml_ignore_l_angle)>>2)]; //@line 2161 "readclause.cpp"
          var $cmp56=((($40))|0)!=38; //@line 2161 "readclause.cpp"
          if ($cmp56) { __label__ = 28;; } else { __label__ = 50;; } //@line 2161 "readclause.cpp"
          $land_lhs_true57$$if_else122$37: do { 
            if (__label__ == 28) {
  
              var $41=HEAP32[(($c1)>>2)]; //@line 2161 "readclause.cpp"
              var $cmp58=((($41))|0)==38; //@line 2161 "readclause.cpp"
              if (!($cmp58)) { __label__ = 50;break $land_lhs_true57$$if_else122$37; } //@line 2161 "readclause.cpp"
  
              var $42=HEAP32[(($c2)>>2)]; //@line 2161 "readclause.cpp"
              var $cmp60=((($42))|0)==35; //@line 2161 "readclause.cpp"
              if ($cmp60) { __label__ = 31;; } else { __label__ = 30;; } //@line 2161 "readclause.cpp"
              if (__label__ == 30) {
  
                var $43=HEAP32[(($c2)>>2)]; //@line 2161 "readclause.cpp"
                var $cmp62=((($43))|0) >= 97; //@line 2161 "readclause.cpp"
                var $44=HEAP32[(($c2)>>2)]; //@line 2161 "readclause.cpp"
                var $cmp64=((($44))|0) <= 122; //@line 2161 "readclause.cpp"
                var $or_cond5=($cmp62) & ($cmp64); //@line 2161 "readclause.cpp"
                if (!($or_cond5)) { __label__ = 50;break $land_lhs_true57$$if_else122$37; } //@line 2161 "readclause.cpp"
              }
  
              $n_xml_buf=0; //@line 2163 "readclause.cpp"
              var $45=HEAP32[(($c2)>>2)]; //@line 2164 "readclause.cpp"
              HEAP32[(($c1)>>2)]=$45; //@line 2164 "readclause.cpp"
              ; //@line 2165 "readclause.cpp"
              $while_cond66$43: while(1) { 
  
                var $call67=__Z3Eofv(); //@line 2165 "readclause.cpp"
                var $tobool68=((($call67))|0)!=0; //@line 2165 "readclause.cpp"
                if ($tobool68) { __label__ = 36;break $while_cond66$43; } //@line 2165 "readclause.cpp"
  
                var $46=HEAP32[(($c1)>>2)]; //@line 2165 "readclause.cpp"
                var $call70=_iswalnum($46); //@line 2165 "readclause.cpp"
                var $tobool71=((($call70))|0)!=0; //@line 2165 "readclause.cpp"
                var $47=HEAP32[(($c1)>>2)]; //@line 2165 "readclause.cpp"
                var $cmp73=((($47))|0)==35; //@line 2165 "readclause.cpp"
                var $or_cond6=($tobool71) | ($cmp73); //@line 2165 "readclause.cpp"
                if (!($or_cond6)) { __label__ = 36;break $while_cond66$43; } //@line 2165 "readclause.cpp"
  
                var $48=$n_xml_buf; //@line 2165 "readclause.cpp"
                var $cmp74=((($48))|0) < 20; //@line 2165 "readclause.cpp"
                if (!($cmp74)) { __label__ = 36;break $while_cond66$43; }
  
                var $49=HEAP32[(($c1)>>2)]; //@line 2167 "readclause.cpp"
                var $conv76=((($49)) & 255); //@line 2167 "readclause.cpp"
                var $50=$n_xml_buf; //@line 2167 "readclause.cpp"
                var $inc77=((($50)+1)&4294967295); //@line 2167 "readclause.cpp"
                $n_xml_buf=$inc77; //@line 2167 "readclause.cpp"
                var $arrayidx78=(($xml_buf2+$50)&4294967295); //@line 2167 "readclause.cpp"
                HEAP8[($arrayidx78)]=$conv76; //@line 2167 "readclause.cpp"
                var $call79=__ZL4GetCv(); //@line 2168 "readclause.cpp"
                HEAP32[(($c1)>>2)]=$call79; //@line 2168 "readclause.cpp"
                __label__ = 32;continue $while_cond66$43; //@line 2169 "readclause.cpp"
              }
  
              var $51=$n_xml_buf; //@line 2170 "readclause.cpp"
              var $arrayidx80=(($xml_buf2+$51)&4294967295); //@line 2170 "readclause.cpp"
              HEAP8[($arrayidx80)]=0; //@line 2170 "readclause.cpp"
              var $call81=__ZL4GetCv(); //@line 2171 "readclause.cpp"
              HEAP32[(($c2)>>2)]=$call81; //@line 2171 "readclause.cpp"
              var $52=HEAP32[(($c1)>>2)]; //@line 2172 "readclause.cpp"
              var $53=HEAP32[(($c2)>>2)]; //@line 2172 "readclause.cpp"
              var $call83=_sprintf(((__ZZ10ReadClauseP10TranslatorP7__sFILEPcPsPiiS5_S3_E12ungot_string)&4294967295), ((__str38205)&4294967295), allocate([$arrayidx82,0,0,0,$52,0,0,0,$53,0,0,0], ["i8*",0,0,0,"i32",0,0,0,"i32",0,0,0], ALLOC_STACK)); //@line 2172 "readclause.cpp"
              var $54=HEAP32[(($c1)>>2)]; //@line 2174 "readclause.cpp"
              var $cmp84=((($54))|0)==59; //@line 2174 "readclause.cpp"
              if ($cmp84) { __label__ = 37;; } else { __label__ = 44;; } //@line 2174 "readclause.cpp"
              $if_then85$$if_else109$49: do { 
                if (__label__ == 37) {
  
                  var $55=HEAP8[($arrayidx86)]; //@line 2176 "readclause.cpp"
                  var $conv87=(tempInt=(($55)),(tempInt>=128?tempInt-256:tempInt)); //@line 2176 "readclause.cpp"
                  var $cmp88=((($conv87))|0)==35; //@line 2176 "readclause.cpp"
                  if ($cmp88) { __label__ = 38;; } else { __label__ = 41;; } //@line 2176 "readclause.cpp"
                  $if_then89$$if_else100$51: do { 
                    if (__label__ == 38) {
  
                      var $56=HEAP8[($arrayidx90)]; //@line 2179 "readclause.cpp"
                      var $conv91=(tempInt=(($56)),(tempInt>=128?tempInt-256:tempInt)); //@line 2179 "readclause.cpp"
                      var $cmp92=((($conv91))|0)==120; //@line 2179 "readclause.cpp"
                      if ($cmp92) { __label__ = 39;; } else { __label__ = 40;; } //@line 2179 "readclause.cpp"
                      if (__label__ == 39) {
  
                        var $call95=_sscanf($arrayidx94, ((__str10307)&4294967295), allocate([$c1,0,0,0], ["i32*",0,0,0], ALLOC_STACK)); //@line 2180 "readclause.cpp"
                        $found=$call95; //@line 2180 "readclause.cpp"
                        __lastLabel__ = 39; ; //@line 2180 "readclause.cpp"
                      }
                      else if (__label__ == 40) {
  
                        var $call98=_sscanf($arrayidx97, ((__str63715)&4294967295), allocate([$c1,0,0,0], ["i32*",0,0,0], ALLOC_STACK)); //@line 2182 "readclause.cpp"
                        $found=$call98; //@line 2182 "readclause.cpp"
                        __lastLabel__ = 40; ;
                      }
                    }
                    else if (__label__ == 41) {
  
                      var $call101=__Z10LookupMnemP8MNEM_TABPKc(((__ZL18xml_char_mnemonics)&4294967295), $arraydecay); //@line 2186 "readclause.cpp"
                      $found=$call101; //@line 2186 "readclause.cpp"
                      var $cmp102=((($call101))|0)!=-1; //@line 2186 "readclause.cpp"
                      if (!($cmp102)) { __label__ = 46;break $if_then85$$if_else109$49; } //@line 2186 "readclause.cpp"
  
                      var $57=$found; //@line 2188 "readclause.cpp"
                      HEAP32[(($c1)>>2)]=$57; //@line 2188 "readclause.cpp"
                      var $58=HEAP32[(($c2)>>2)]; //@line 2189 "readclause.cpp"
                      var $cmp104=((($58))|0)==0; //@line 2189 "readclause.cpp"
                      if ($cmp104) { __lastLabel__ = 42; ; } else { __lastLabel__ = 42; __label__ = 45;break $if_then89$$if_else100$51; } //@line 2189 "readclause.cpp"
  
                      HEAP32[(($c2)>>2)]=32; //@line 2190 "readclause.cpp"
                      var $_pr=$found; //@line 2199 "readclause.cpp"
                      __lastLabel__ = 43; ; //@line 2190 "readclause.cpp"
                    }
                  } while(0);
  
                  var $59=__lastLabel__ == 40 ? $call98 : (__lastLabel__ == 39 ? $call95 : (__lastLabel__ == 42 ? $57 : ($_pr))); //@line 2199 "readclause.cpp"
                  var $cmp111=((($59))|0) <= 0; //@line 2199 "readclause.cpp"
                  if ($cmp111) { __label__ = 46;break $if_then85$$if_else109$49; } //@line 2199 "readclause.cpp"
  
                  var $_pr23=HEAP32[(($c1)>>2)]; //@line 2206 "readclause.cpp"
                  var $cmp114=((($_pr23))|0) <= 32; //@line 2206 "readclause.cpp"
                  if (!($cmp114)) { __label__ = 72;break $land_lhs_true53$$if_end184$34; } //@line 2206 "readclause.cpp"
  
                  var $60=HEAP32[((__ZL10sayas_mode)>>2)]; //@line 2206 "readclause.cpp"
                  var $cmp116=((($60))|0)==20; //@line 2206 "readclause.cpp"
                  var $61=HEAP32[((__ZL10sayas_mode)>>2)]; //@line 2206 "readclause.cpp"
                  var $cmp118=((($61))|0)==36; //@line 2206 "readclause.cpp"
                  var $or_cond8=($cmp116) | ($cmp118); //@line 2206 "readclause.cpp"
                  if (!($or_cond8)) { __label__ = 72;break $land_lhs_true53$$if_end184$34; } //@line 2206 "readclause.cpp"
  
                  var $62=HEAP32[(($c1)>>2)]; //@line 2208 "readclause.cpp"
                  var $add120=((($62)+57344)&4294967295); //@line 2208 "readclause.cpp"
                  HEAP32[(($c1)>>2)]=$add120; //@line 2208 "readclause.cpp"
                  __label__ = 72;break $land_lhs_true53$$if_end184$34; //@line 2209 "readclause.cpp"
                }
                else if (__label__ == 44) {
  
                  $found=-1; //@line 2196 "readclause.cpp"
                  ;
                }
              } while(0);
  
              HEAP32[((__ZZ10ReadClauseP10TranslatorP7__sFILEPcPsPiiS5_S3_E15ungot_string_ix)>>2)]=0; //@line 2201 "readclause.cpp"
              HEAP32[(($c1)>>2)]=38; //@line 2202 "readclause.cpp"
              HEAP32[(($c2)>>2)]=32; //@line 2203 "readclause.cpp"
              __label__ = 72;break $land_lhs_true53$$if_end184$34; //@line 2206 "readclause.cpp"
            }
          } while(0);
  
          var $63=HEAP32[(($c1)>>2)]; //@line 2212 "readclause.cpp"
          var $cmp123=((($63))|0)==60; //@line 2212 "readclause.cpp"
          var $64=HEAP32[((__ZL19ssml_ignore_l_angle)>>2)]; //@line 2212 "readclause.cpp"
          var $cmp125=((($64))|0)!=60; //@line 2212 "readclause.cpp"
          var $or_cond9=($cmp123) & ($cmp125); //@line 2212 "readclause.cpp"
          if (!($or_cond9)) { __label__ = 72;break $land_lhs_true53$$if_end184$34; } //@line 2212 "readclause.cpp"
  
          var $65=HEAP32[(($c2)>>2)]; //@line 2214 "readclause.cpp"
          var $cmp127=((($65))|0)==33; //@line 2214 "readclause.cpp"
          if ($cmp127) { __label__ = 52;; } else { __label__ = 56;; } //@line 2214 "readclause.cpp"
          if (__label__ == 52) {
            while(1) { 
  
              var $call130=__Z3Eofv(); //@line 2217 "readclause.cpp"
              var $tobool131=((($call130))|0)!=0; //@line 2217 "readclause.cpp"
              if ($tobool131) { __label__ = 55;break ; } //@line 2217 "readclause.cpp"
  
              var $66=HEAP32[(($c1)>>2)]; //@line 2217 "readclause.cpp"
              var $cmp133=((($66))|0)!=62; //@line 2217 "readclause.cpp"
              if (!($cmp133)) { __label__ = 55;break ; }
  
              var $call136=__ZL4GetCv(); //@line 2219 "readclause.cpp"
              HEAP32[(($c1)>>2)]=$call136; //@line 2219 "readclause.cpp"
              __label__ = 52;continue ; //@line 2220 "readclause.cpp"
            }
  
            HEAP32[(($c2)>>2)]=32; //@line 2221 "readclause.cpp"
            ; //@line 2222 "readclause.cpp"
          }
          else if (__label__ == 56) {
  
            var $67=HEAP32[(($c2)>>2)]; //@line 2224 "readclause.cpp"
            var $cmp139=((($67))|0)==47; //@line 2224 "readclause.cpp"
            if ($cmp139) { __label__ = 58;; } else { __label__ = 57;; } //@line 2224 "readclause.cpp"
            if (__label__ == 57) {
  
              var $68=HEAP32[(($c2)>>2)]; //@line 2224 "readclause.cpp"
              var $call141=_iswalpha($68); //@line 2224 "readclause.cpp"
              var $tobool142=((($call141))|0)!=0; //@line 2224 "readclause.cpp"
              if (!($tobool142)) { __label__ = 72;break $land_lhs_true53$$if_end184$34; } //@line 2224 "readclause.cpp"
            }
  
            $n_xml_buf=0; //@line 2227 "readclause.cpp"
            var $69=HEAP32[(($c2)>>2)]; //@line 2228 "readclause.cpp"
            HEAP32[(($c1)>>2)]=$69; //@line 2228 "readclause.cpp"
            ; //@line 2229 "readclause.cpp"
            while(1) { 
  
              var $call145=__Z3Eofv(); //@line 2229 "readclause.cpp"
              var $tobool146=((($call145))|0)!=0; //@line 2229 "readclause.cpp"
              if ($tobool146) { __label__ = 63;break ; } //@line 2229 "readclause.cpp"
  
              var $70=HEAP32[(($c1)>>2)]; //@line 2229 "readclause.cpp"
              var $cmp148=((($70))|0)!=62; //@line 2229 "readclause.cpp"
              if (!($cmp148)) { __label__ = 63;break ; } //@line 2229 "readclause.cpp"
  
              var $71=$n_xml_buf; //@line 2229 "readclause.cpp"
              var $cmp150=((($71))|0) < 256; //@line 2229 "readclause.cpp"
              if (!($cmp150)) { __label__ = 63;break ; }
  
              var $72=HEAP32[(($c1)>>2)]; //@line 2231 "readclause.cpp"
              var $73=$n_xml_buf; //@line 2231 "readclause.cpp"
              var $inc153=((($73)+1)&4294967295); //@line 2231 "readclause.cpp"
              $n_xml_buf=$inc153; //@line 2231 "readclause.cpp"
              var $arrayidx154=(($xml_buf+$73*4)&4294967295); //@line 2231 "readclause.cpp"
              HEAP32[(($arrayidx154)>>2)]=$72; //@line 2231 "readclause.cpp"
              var $call155=__ZL4GetCv(); //@line 2232 "readclause.cpp"
              HEAP32[(($c1)>>2)]=$call155; //@line 2232 "readclause.cpp"
              __label__ = 59;continue ; //@line 2233 "readclause.cpp"
            }
  
            var $74=$n_xml_buf; //@line 2234 "readclause.cpp"
            var $arrayidx157=(($xml_buf+$74*4)&4294967295); //@line 2234 "readclause.cpp"
            HEAP32[(($arrayidx157)>>2)]=0; //@line 2234 "readclause.cpp"
            HEAP32[(($c2)>>2)]=32; //@line 2235 "readclause.cpp"
            $self_closing=0; //@line 2237 "readclause.cpp"
            var $75=$n_xml_buf; //@line 2238 "readclause.cpp"
            var $sub=((($75)-1)&4294967295); //@line 2238 "readclause.cpp"
            var $arrayidx158=(($xml_buf+$sub*4)&4294967295); //@line 2238 "readclause.cpp"
            var $76=HEAP32[(($arrayidx158)>>2)]; //@line 2238 "readclause.cpp"
            var $cmp159=((($76))|0)==47; //@line 2238 "readclause.cpp"
            if ($cmp159) { __label__ = 64;; } else { __label__ = 65;; } //@line 2238 "readclause.cpp"
            if (__label__ == 64) {
  
              var $77=$n_xml_buf; //@line 2241 "readclause.cpp"
              var $sub161=((($77)-1)&4294967295); //@line 2241 "readclause.cpp"
              var $arrayidx162=(($xml_buf+$sub161*4)&4294967295); //@line 2241 "readclause.cpp"
              HEAP32[(($arrayidx162)>>2)]=32; //@line 2241 "readclause.cpp"
              $self_closing=1; //@line 2242 "readclause.cpp"
              ; //@line 2243 "readclause.cpp"
            }
  
            var $78=$buf_addr; //@line 2245 "readclause.cpp"
            var $79=$n_buf_addr; //@line 2245 "readclause.cpp"
            var $80=$self_closing; //@line 2245 "readclause.cpp"
            var $call165=__ZL14ProcessSsmlTagPwPcRiii($arraydecay164, $78, $ix, $79, $80); //@line 2245 "readclause.cpp"
            $terminator=$call165; //@line 2245 "readclause.cpp"
            var $cmp166=((($call165))|0)!=0; //@line 2247 "readclause.cpp"
            if ($cmp166) { __label__ = 66;break $while_cond$12; } //@line 2247 "readclause.cpp"
  
            HEAP32[(($c1)>>2)]=32; //@line 2261 "readclause.cpp"
            var $call179=__ZL4GetCv(); //@line 2262 "readclause.cpp"
            HEAP32[(($c2)>>2)]=$call179; //@line 2262 "readclause.cpp"
            __label__ = 8;continue $while_cond$12; //@line 2263 "readclause.cpp"
          }
        }
      } while(0);
  
      HEAP32[((__ZL19ssml_ignore_l_angle)>>2)]=0; //@line 2267 "readclause.cpp"
      var $_b1=HEAPU8[(__ZL11ignore_text_b)];
      var $90=((($_b1))&1);
      var $tobool185=((($90))|0)!=0; //@line 2269 "readclause.cpp"
      if ($tobool185) { __label__ = 8;continue $while_cond$12; } //@line 2269 "readclause.cpp"
  
      var $91=HEAP32[(($c2)>>2)]; //@line 2272 "readclause.cpp"
      var $cmp188=((($91))|0)==10; //@line 2272 "readclause.cpp"
      var $92=HEAP32[((_option_linelength)>>2)]; //@line 2272 "readclause.cpp"
      var $cmp190=((($92))|0)==-1; //@line 2272 "readclause.cpp"
      var $or_cond11=($cmp188) & ($cmp190); //@line 2272 "readclause.cpp"
      var $93=HEAPU32[(($c1)>>2)]; //@line 2275 "readclause.cpp"
      if ($or_cond11) { __label__ = 74;break $while_cond$12; } //@line 2272 "readclause.cpp"
  
      var $cmp208=((($93))|0)==1; //@line 2291 "readclause.cpp"
      if ($cmp208) { __label__ = 80;; } else { __label__ = 79;; } //@line 2291 "readclause.cpp"
      $if_then212$$lor_lhs_false209$90: do { 
        if (__label__ == 79) {
  
          var $111=HEAP32[(($c1)>>2)]; //@line 2291 "readclause.cpp"
          var $112=HEAP8[(_ctrl_embedded)]; //@line 2291 "readclause.cpp"
          var $conv210=(tempInt=(($112)),(tempInt>=128?tempInt-256:tempInt)); //@line 2291 "readclause.cpp"
          var $cmp211=((($111))|0)==((($conv210))|0); //@line 2291 "readclause.cpp"
          if ($cmp211) { __label__ = 80;break $if_then212$$lor_lhs_false209$90; } else { __label__ = 97;break $if_then212$$lor_lhs_false209$90; } //@line 2291 "readclause.cpp"
        }
      } while(0);
      $if_then212$$if_end267$92: do { 
        if (__label__ == 80) {
  
          var $113=HEAP32[(($c2)>>2)]; //@line 2294 "readclause.cpp"
          var $cmp213=((($113))|0)==86; //@line 2294 "readclause.cpp"
          if ($cmp213) { __label__ = 81;break $while_cond$12; } //@line 2294 "readclause.cpp"
  
          var $123=HEAP32[(($c2)>>2)]; //@line 2303 "readclause.cpp"
          var $cmp236=((($123))|0)==66; //@line 2303 "readclause.cpp"
          if (!($cmp236)) { __label__ = 97;break $if_then212$$if_end267$92; } //@line 2303 "readclause.cpp"
  
          var $124=HEAP32[(($ix)>>2)]; //@line 2307 "readclause.cpp"
          var $125=$buf_addr; //@line 2307 "readclause.cpp"
          var $arrayidx238=(($125+$124)&4294967295); //@line 2307 "readclause.cpp"
          var $src$, $dest$, $stop$, $stop4$, $fast$;
          $src$ = ((__str41208)&4294967295);
          $dest$ = $arrayidx238;
          $stop$ = $src$ + 4;
          if (($dest$%4) == ($src$%4) && 4 > 8) {
            while ($src$%4 !== 0 && $src$ < $stop$) {
              ; HEAP8[$dest$++] = HEAP8[$src$++];
            }
            $src$ >>= 2;
            $dest$ >>= 2;
            $stop4$ = $stop$ >> 2;
            while ($src$ < $stop4$) {
          ;;;    HEAP32[$dest$++] = HEAP32[$src$++];
            }
            $src$ <<= 2;
            $dest$ <<= 2;
          }while ($src$ < $stop$) {
            ; HEAP8[$dest$++] = HEAP8[$src$++];
          }; //@line 2307 "readclause.cpp"
          var $126=HEAP32[(($ix)>>2)]; //@line 2308 "readclause.cpp"
          var $add240=((($126)+3)&4294967295); //@line 2308 "readclause.cpp"
          HEAP32[(($ix)>>2)]=$add240; //@line 2308 "readclause.cpp"
          var $call241=__ZL4GetCv(); //@line 2310 "readclause.cpp"
          HEAP32[(($c2)>>2)]=$call241; //@line 2310 "readclause.cpp"
          var $cmp242=((($call241))|0)==48; //@line 2310 "readclause.cpp"
          if ($cmp242) { __label__ = 89;; } else { __label__ = 90;; } //@line 2310 "readclause.cpp"
          $if_then243$$if_else244$96: do { 
            if (__label__ == 89) {
  
              HEAP32[((_option_punctuation)>>2)]=0; //@line 2311 "readclause.cpp"
              ; //@line 2311 "readclause.cpp"
            }
            else if (__label__ == 90) {
  
              HEAP32[((_option_punctuation)>>2)]=1; //@line 2314 "readclause.cpp"
              HEAP32[((((_option_punctlist)&4294967295))>>2)]=0; //@line 2315 "readclause.cpp"
              var $127=HEAP32[(($c2)>>2)]; //@line 2316 "readclause.cpp"
              var $cmp245=((($127))|0)!=49; //@line 2316 "readclause.cpp"
              if (!($cmp245)) { __label__ = 96;break $if_then243$$if_else244$96; } //@line 2316 "readclause.cpp"
  
              $j=0; //@line 2319 "readclause.cpp"
              ; //@line 2320 "readclause.cpp"
              while(1) { 
  
                var $128=HEAP32[(($c2)>>2)]; //@line 2320 "readclause.cpp"
                var $call248=_iswspace($128); //@line 2320 "readclause.cpp"
                var $tobool249=((($call248))|0)!=0; //@line 2320 "readclause.cpp"
                if ($tobool249) { __label__ = 95;break ; } //@line 2320 "readclause.cpp"
  
                var $call251=__Z3Eofv(); //@line 2320 "readclause.cpp"
                var $tobool252=((($call251))|0)!=0; //@line 2320 "readclause.cpp"
                var $lnot=($tobool252) ^ 1; //@line 2320 "readclause.cpp"
                if (!($lnot)) { __label__ = 95;break ; }
  
                var $129=HEAP32[(($c2)>>2)]; //@line 2322 "readclause.cpp"
                var $130=$j; //@line 2322 "readclause.cpp"
                var $inc255=((($130)+1)&4294967295); //@line 2322 "readclause.cpp"
                $j=$inc255; //@line 2322 "readclause.cpp"
                var $arrayidx256=((_option_punctlist+$130*4)&4294967295); //@line 2322 "readclause.cpp"
                HEAP32[(($arrayidx256)>>2)]=$129; //@line 2322 "readclause.cpp"
                var $call257=__ZL4GetCv(); //@line 2323 "readclause.cpp"
                HEAP32[(($c2)>>2)]=$call257; //@line 2323 "readclause.cpp"
                var $131=HEAP32[(($ix)>>2)]; //@line 2324 "readclause.cpp"
                var $inc258=((($131)+1)&4294967295); //@line 2324 "readclause.cpp"
                HEAP32[(($ix)>>2)]=$inc258; //@line 2324 "readclause.cpp"
                var $132=$buf_addr; //@line 2324 "readclause.cpp"
                var $arrayidx259=(($132+$131)&4294967295); //@line 2324 "readclause.cpp"
                HEAP8[($arrayidx259)]=32; //@line 2324 "readclause.cpp"
                __label__ = 92;continue ; //@line 2325 "readclause.cpp"
              }
  
              var $133=$j; //@line 2326 "readclause.cpp"
              var $arrayidx261=((_option_punctlist+$133*4)&4294967295); //@line 2326 "readclause.cpp"
              HEAP32[(($arrayidx261)>>2)]=0; //@line 2326 "readclause.cpp"
              HEAP32[((_option_punctuation)>>2)]=2; //@line 2327 "readclause.cpp"
              ; //@line 2328 "readclause.cpp"
            }
          } while(0);
  
          var $call264=__ZL4GetCv(); //@line 2330 "readclause.cpp"
          HEAP32[(($c2)>>2)]=$call264; //@line 2330 "readclause.cpp"
          __label__ = 8;continue $while_cond$12; //@line 2331 "readclause.cpp"
        }
      } while(0);
  
      var $134=$linelength; //@line 2335 "readclause.cpp"
      var $inc268=((($134)+1)&4294967295); //@line 2335 "readclause.cpp"
      $linelength=$inc268; //@line 2335 "readclause.cpp"
      var $135=HEAP32[(($c1)>>2)]; //@line 2337 "readclause.cpp"
      var $call269=_iswalnum($135); //@line 2337 "readclause.cpp"
      var $tobool270=((($call269))|0)!=0; //@line 2337 "readclause.cpp"
      if ($tobool270) { __label__ = 98;; } else { __label__ = 99;; } //@line 2337 "readclause.cpp"
      $if_then271$$if_else272$107: do { 
        if (__label__ == 98) {
  
          $any_alnum=1; //@line 2338 "readclause.cpp"
          ; //@line 2338 "readclause.cpp"
        }
        else if (__label__ == 99) {
  
          var $136=$stressed_word; //@line 2341 "readclause.cpp"
          var $tobool273=((($136))|0)!=0; //@line 2341 "readclause.cpp"
          if ($tobool273) { __label__ = 100;; } else { __label__ = 101;; } //@line 2341 "readclause.cpp"
          if (__label__ == 100) {
  
            $stressed_word=0; //@line 2343 "readclause.cpp"
            HEAP32[(($c1)>>2)]=1328; //@line 2344 "readclause.cpp"
            var $137=HEAP32[(($c2)>>2)]; //@line 2345 "readclause.cpp"
            __ZL6UngetCi($137); //@line 2345 "readclause.cpp"
            HEAP32[(($c2)>>2)]=32; //@line 2346 "readclause.cpp"
            ; //@line 2347 "readclause.cpp"
          }
  
          var $138=HEAP32[(($c1)>>2)]; //@line 2349 "readclause.cpp"
          var $call276=__Z11lookupwcharPKti(((__ZL12chars_ignore)&4294967295), $138); //@line 2349 "readclause.cpp"
          var $tobool277=((($call276))|0)!=0; //@line 2349 "readclause.cpp"
          if ($tobool277) { __label__ = 8;continue $while_cond$12; } //@line 2349 "readclause.cpp"
  
          var $139=HEAP32[(($c1)>>2)]; //@line 2355 "readclause.cpp"
          var $call280=_iswspace($139); //@line 2355 "readclause.cpp"
          var $tobool281=((($call280))|0)!=0; //@line 2355 "readclause.cpp"
          if ($tobool281) { __label__ = 103;; } else { __label__ = 109;; } //@line 2355 "readclause.cpp"
          $if_then282$$if_end304$114: do { 
            if (__label__ == 103) {
  
              var $140=$tr_addr; //@line 2359 "readclause.cpp"
              var $translator_name=(($140+280)&4294967295); //@line 2359 "readclause.cpp"
              var $141=HEAP32[(($translator_name)>>2)]; //@line 2359 "readclause.cpp"
              var $cmp283=((($141))|0)==6972015; //@line 2359 "readclause.cpp"
              if (!($cmp283)) { __label__ = 109;break $if_then282$$if_end304$114; } //@line 2359 "readclause.cpp"
  
              var $142=HEAP32[(($ix)>>2)]; //@line 2363 "readclause.cpp"
              var $sub285=((($142)-1)&4294967295); //@line 2363 "readclause.cpp"
              var $143=$buf_addr; //@line 2363 "readclause.cpp"
              var $arrayidx286=(($143+$sub285)&4294967295); //@line 2363 "readclause.cpp"
              $p_word=$arrayidx286; //@line 2363 "readclause.cpp"
              var $144=$p_word; //@line 2364 "readclause.cpp"
              var $arrayidx287=(($144)&4294967295); //@line 2364 "readclause.cpp"
              var $145=HEAP8[($arrayidx287)]; //@line 2364 "readclause.cpp"
              var $conv288=(tempInt=(($145)),(tempInt>=128?tempInt-256:tempInt)); //@line 2364 "readclause.cpp"
              var $cmp289=((($conv288))|0)==105; //@line 2364 "readclause.cpp"
              if (!($cmp289)) { __label__ = 109;break $if_then282$$if_end304$114; } //@line 2364 "readclause.cpp"
  
              var $146=$p_word; //@line 2366 "readclause.cpp"
              var $arrayidx291=(($146+-1)&4294967295); //@line 2366 "readclause.cpp"
              var $147=HEAP8[($arrayidx291)]; //@line 2366 "readclause.cpp"
              var $conv292=(tempInt=(($147)),(tempInt>=128?tempInt-256:tempInt)); //@line 2366 "readclause.cpp"
              var $cmp293=((($conv292))|0)==46; //@line 2366 "readclause.cpp"
              if ($cmp293) { __label__ = 106;; } else { __label__ = 107;; } //@line 2366 "readclause.cpp"
              if (__label__ == 106) {
  
                var $148=$p_word; //@line 2367 "readclause.cpp"
                var $incdec_ptr=(($148+-1)&4294967295); //@line 2367 "readclause.cpp"
                $p_word=$incdec_ptr; //@line 2367 "readclause.cpp"
                ; //@line 2367 "readclause.cpp"
              }
  
              var $149=$p_word; //@line 2368 "readclause.cpp"
              var $arrayidx296=(($149+-1)&4294967295); //@line 2368 "readclause.cpp"
              var $150=HEAP8[($arrayidx296)]; //@line 2368 "readclause.cpp"
              var $conv297=(tempInt=(($150)),(tempInt>=128?tempInt-256:tempInt)); //@line 2368 "readclause.cpp"
              var $cmp298=((($conv297))|0)==32; //@line 2368 "readclause.cpp"
              if ($cmp298) { __label__ = 108;break $while_cond$12; } //@line 2368 "readclause.cpp"
            }
          } while(0);
  
          var $153=HEAP32[(($c1)>>2)]; //@line 2379 "readclause.cpp"
          var $cmp305=((($153))|0)==3405; //@line 2379 "readclause.cpp"
          if (!($cmp305)) { __label__ = 112;break $if_then271$$if_else272$107; } //@line 2379 "readclause.cpp"
  
          var $154=HEAP32[(($c2)>>2)]; //@line 2382 "readclause.cpp"
          var $cmp307=((($154))|0)==8205; //@line 2382 "readclause.cpp"
          if (!($cmp307)) { __label__ = 112;break $if_then271$$if_else272$107; } //@line 2382 "readclause.cpp"
  
          HEAP32[(($c1)>>2)]=3406; //@line 2384 "readclause.cpp"
          ; //@line 2385 "readclause.cpp"
        }
      } while(0);
  
      var $155=HEAP32[(($c1)>>2)]; //@line 2389 "readclause.cpp"
      var $call312=_iswupper($155); //@line 2389 "readclause.cpp"
      var $tobool313=((($call312))|0)!=0; //@line 2389 "readclause.cpp"
      if ($tobool313) { __label__ = 113;; } else { __label__ = 118;; } //@line 2389 "readclause.cpp"
      $if_then314$$if_else343$125: do { 
        if (__label__ == 113) {
  
          var $156=$tr_addr; //@line 2391 "readclause.cpp"
          var $clause_upper_count315=(($156+8304)&4294967295); //@line 2391 "readclause.cpp"
          var $157=HEAP32[(($clause_upper_count315)>>2)]; //@line 2391 "readclause.cpp"
          var $inc316=((($157)+1)&4294967295); //@line 2391 "readclause.cpp"
          HEAP32[(($clause_upper_count315)>>2)]=$inc316; //@line 2391 "readclause.cpp"
          var $158=HEAP32[((_option_capitals)>>2)]; //@line 2392 "readclause.cpp"
          var $cmp317=((($158))|0)==2; //@line 2392 "readclause.cpp"
          var $159=HEAP32[((__ZL10sayas_mode)>>2)]; //@line 2392 "readclause.cpp"
          var $cmp319=((($159))|0)==0; //@line 2392 "readclause.cpp"
          var $or_cond13=($cmp317) & ($cmp319); //@line 2392 "readclause.cpp"
          if (!($or_cond13)) { __label__ = 120;break $if_then314$$if_else343$125; } //@line 2392 "readclause.cpp"
  
          var $160=$cprev; //@line 2392 "readclause.cpp"
          var $call321=_iswupper($160); //@line 2392 "readclause.cpp"
          var $tobool322=((($call321))|0)!=0; //@line 2392 "readclause.cpp"
          if ($tobool322) { __label__ = 120;break $if_then314$$if_else343$125; } //@line 2392 "readclause.cpp"
  
          var $161=$tr_addr; //@line 2396 "readclause.cpp"
          var $call325=__ZL13LookupSpecialP10TranslatorPKcPc($161, $arraydecay324); //@line 2396 "readclause.cpp"
          var $cmp326=((($call325))|0)!=0; //@line 2396 "readclause.cpp"
          if (!($cmp326)) { __label__ = 120;break $if_then314$$if_else343$125; } //@line 2396 "readclause.cpp"
  
          var $strlen=_strlen($arraydecay329); //@line 2398 "readclause.cpp"
          var $leninc=((($strlen)+1)&4294967295); //@line 2398 "readclause.cpp"
          var $src$, $dest$, $stop$, $stop4$, $fast$;
          $src$ = $arraydecay329;
          $dest$ = $arraydecay328;
          $stop$ = $src$ + $leninc;
          if (($dest$%4) == ($src$%4) && $leninc > 8) {
            while ($src$%4 !== 0 && $src$ < $stop$) {
              ; HEAP8[$dest$++] = HEAP8[$src$++];
            }
            $src$ >>= 2;
            $dest$ >>= 2;
            $stop4$ = $stop$ >> 2;
            while ($src$ < $stop4$) {
          ;;;    HEAP32[$dest$++] = HEAP32[$src$++];
            }
            $src$ <<= 2;
            $dest$ <<= 2;
          }while ($src$ < $stop$) {
            ; HEAP8[$dest$++] = HEAP8[$src$++];
          }; //@line 2398 "readclause.cpp"
          var $call332=_strlen($arraydecay331); //@line 2399 "readclause.cpp"
          $j=$call332; //@line 2399 "readclause.cpp"
          var $162=HEAP32[(($ix)>>2)]; //@line 2400 "readclause.cpp"
          var $163=$j; //@line 2400 "readclause.cpp"
          var $add333=((($163)+($162))&4294967295); //@line 2400 "readclause.cpp"
          var $164=$n_buf_addr; //@line 2400 "readclause.cpp"
          var $cmp334=((($add333))|0) < ((($164))|0); //@line 2400 "readclause.cpp"
          if (!($cmp334)) { __label__ = 120;break $if_then314$$if_else343$125; } //@line 2400 "readclause.cpp"
  
          var $165=HEAP32[(($ix)>>2)]; //@line 2402 "readclause.cpp"
          var $166=$buf_addr; //@line 2402 "readclause.cpp"
          var $arrayidx336=(($166+$165)&4294967295); //@line 2402 "readclause.cpp"
          var $call338=_strcpy($arrayidx336, $arraydecay337); //@line 2402 "readclause.cpp"
          var $167=$j; //@line 2403 "readclause.cpp"
          var $168=HEAP32[(($ix)>>2)]; //@line 2403 "readclause.cpp"
          var $add339=((($168)+($167))&4294967295); //@line 2403 "readclause.cpp"
          HEAP32[(($ix)>>2)]=$add339; //@line 2403 "readclause.cpp"
          ; //@line 2404 "readclause.cpp"
        }
        else if (__label__ == 118) {
  
          var $169=HEAP32[(($c1)>>2)]; //@line 2409 "readclause.cpp"
          var $call344=_iswalpha($169); //@line 2409 "readclause.cpp"
          var $tobool345=((($call344))|0)!=0; //@line 2409 "readclause.cpp"
          if (!($tobool345)) { __label__ = 120;break $if_then314$$if_else343$125; } //@line 2409 "readclause.cpp"
  
          var $170=$tr_addr; //@line 2410 "readclause.cpp"
          var $clause_lower_count347=(($170+8308)&4294967295); //@line 2410 "readclause.cpp"
          var $171=HEAP32[(($clause_lower_count347)>>2)]; //@line 2410 "readclause.cpp"
          var $inc348=((($171)+1)&4294967295); //@line 2410 "readclause.cpp"
          HEAP32[(($clause_lower_count347)>>2)]=$inc348; //@line 2410 "readclause.cpp"
          ; //@line 2410 "readclause.cpp"
        }
      } while(0);
  
      var $172=HEAP32[((_option_phoneme_input)>>2)]; //@line 2412 "readclause.cpp"
      var $tobool351=((($172))|0)!=0; //@line 2412 "readclause.cpp"
      if ($tobool351) { __label__ = 121;; } else { __label__ = 130;; } //@line 2412 "readclause.cpp"
      $if_then352$$if_end368$134: do { 
        if (__label__ == 121) {
  
          var $173=$phoneme_mode; //@line 2414 "readclause.cpp"
          var $cmp353=((($173))|0) > 0; //@line 2414 "readclause.cpp"
          if ($cmp353) { __label__ = 122;; } else { __label__ = 123;; } //@line 2414 "readclause.cpp"
          if (__label__ == 122) {
  
            var $174=$phoneme_mode; //@line 2415 "readclause.cpp"
            var $dec=((($174)-1)&4294967295); //@line 2415 "readclause.cpp"
            $phoneme_mode=$dec; //@line 2415 "readclause.cpp"
            ; //@line 2415 "readclause.cpp"
          }
          else if (__label__ == 123) {
  
            var $175=HEAP32[(($c1)>>2)]; //@line 2417 "readclause.cpp"
            var $cmp356=((($175))|0)==91; //@line 2417 "readclause.cpp"
            if ($cmp356) { __lastLabel__ = 123; __label__ = 124;; } else { __lastLabel__ = 123; __label__ = 127;; } //@line 2417 "readclause.cpp"
            if (__label__ == 124) {
  
              var $176=HEAP32[(($c2)>>2)]; //@line 2417 "readclause.cpp"
              var $cmp358=((($176))|0)==91; //@line 2417 "readclause.cpp"
              if ($cmp358) { __label__ = 125;; } else { __label__ = 126;; } //@line 2417 "readclause.cpp"
              if (__label__ == 125) {
  
                $phoneme_mode=-1; //@line 2418 "readclause.cpp"
                __label__ = 130;break $if_then352$$if_end368$134; //@line 2418 "readclause.cpp"
              }
              else if (__label__ == 126) {
  
                var $_pr25=HEAP32[(($c1)>>2)]; //@line 2420 "readclause.cpp"
                __lastLabel__ = 126; ;
              }
            }
  
            var $177=__lastLabel__ == 126 ? $_pr25 : ($175); //@line 2420 "readclause.cpp"
            var $cmp361=((($177))|0)==93; //@line 2420 "readclause.cpp"
            if (!($cmp361)) { __label__ = 130;break $if_then352$$if_end368$134; } //@line 2420 "readclause.cpp"
  
            var $178=HEAP32[(($c2)>>2)]; //@line 2420 "readclause.cpp"
            var $cmp363=((($178))|0)==93; //@line 2420 "readclause.cpp"
            if (!($cmp363)) { __label__ = 130;break $if_then352$$if_end368$134; } //@line 2420 "readclause.cpp"
  
            $phoneme_mode=2; //@line 2421 "readclause.cpp"
            ; //@line 2421 "readclause.cpp"
          }
        }
      } while(0);
  
      var $179=HEAP32[(($c1)>>2)]; //@line 2424 "readclause.cpp"
      var $cmp369=((($179))|0)==10; //@line 2424 "readclause.cpp"
      if ($cmp369) { __label__ = 131;; } else { __label__ = 148;; } //@line 2424 "readclause.cpp"
      if (__label__ == 131) {
  
        $parag=0; //@line 2426 "readclause.cpp"
        ; //@line 2429 "readclause.cpp"
        while(1) { 
  
          var $call372=__Z3Eofv(); //@line 2429 "readclause.cpp"
          var $tobool373=((($call372))|0)!=0; //@line 2429 "readclause.cpp"
          if ($tobool373) { __label__ = 137;break ; } //@line 2429 "readclause.cpp"
  
          var $180=HEAP32[(($c2)>>2)]; //@line 2429 "readclause.cpp"
          var $call375=_iswspace($180); //@line 2429 "readclause.cpp"
          var $tobool376=((($call375))|0)!=0; //@line 2429 "readclause.cpp"
          if (!($tobool376)) { __label__ = 137;break ; }
  
          var $181=HEAP32[(($c2)>>2)]; //@line 2431 "readclause.cpp"
          var $cmp379=((($181))|0)==10; //@line 2431 "readclause.cpp"
          if ($cmp379) { __label__ = 135;; } else { __label__ = 136;; } //@line 2431 "readclause.cpp"
          if (__label__ == 135) {
  
            var $182=$parag; //@line 2432 "readclause.cpp"
            var $inc381=((($182)+1)&4294967295); //@line 2432 "readclause.cpp"
            $parag=$inc381; //@line 2432 "readclause.cpp"
            ; //@line 2432 "readclause.cpp"
          }
  
          var $call383=__ZL4GetCv(); //@line 2433 "readclause.cpp"
          HEAP32[(($c2)>>2)]=$call383; //@line 2433 "readclause.cpp"
          __label__ = 132;continue ; //@line 2434 "readclause.cpp"
        }
  
        var $183=$parag; //@line 2435 "readclause.cpp"
        var $cmp385=((($183))|0) > 0; //@line 2435 "readclause.cpp"
        if ($cmp385) { __label__ = 138;break $while_cond$12; } //@line 2435 "readclause.cpp"
  
        var $195=$linelength; //@line 2452 "readclause.cpp"
        var $196=HEAP32[((_option_linelength)>>2)]; //@line 2452 "readclause.cpp"
        var $cmp402=((($195))|0) <= ((($196))|0); //@line 2452 "readclause.cpp"
        if ($cmp402) { __label__ = 146;break $while_cond$12; } //@line 2452 "readclause.cpp"
  
        $linelength=0; //@line 2461 "readclause.cpp"
        ; //@line 2462 "readclause.cpp"
      }
  
      var $202=$announced_punctuation; //@line 2464 "readclause.cpp"
      $prev_announced_punctuation=$202; //@line 2464 "readclause.cpp"
      $announced_punctuation=0; //@line 2465 "readclause.cpp"
      var $203=$phoneme_mode; //@line 2467 "readclause.cpp"
      var $cmp409=((($203))|0)==0; //@line 2467 "readclause.cpp"
      var $204=HEAP32[((__ZL10sayas_mode)>>2)]; //@line 2467 "readclause.cpp"
      var $cmp411=((($204))|0)==0; //@line 2467 "readclause.cpp"
      var $or_cond14=($cmp409) & ($cmp411); //@line 2467 "readclause.cpp"
      if ($or_cond14) { __label__ = 149;; } else { __label__ = 238;; } //@line 2467 "readclause.cpp"
      $if_then412$$if_end643$161: do { 
        if (__label__ == 149) {
  
          $is_end_clause=0; //@line 2469 "readclause.cpp"
          var $205=$end_clause_after_tag; //@line 2471 "readclause.cpp"
          var $tobool413=((($205))|0)!=0; //@line 2471 "readclause.cpp"
          if ($tobool413) { __label__ = 150;; } else { __label__ = 155;; } //@line 2471 "readclause.cpp"
          $if_then414$$if_end429$163: do { 
            if (__label__ == 150) {
  
              var $206=HEAP32[(($c1)>>2)]; //@line 2477 "readclause.cpp"
              var $call415=_iswspace($206); //@line 2477 "readclause.cpp"
              var $tobool416=((($call415))|0)!=0; //@line 2477 "readclause.cpp"
              if ($tobool416) { __label__ = 155;break $if_then414$$if_end429$163; } //@line 2477 "readclause.cpp"
  
              var $207=HEAP32[(($c1)>>2)]; //@line 2479 "readclause.cpp"
              var $call418=__Z7IsAlphaj($207); //@line 2479 "readclause.cpp"
              var $tobool419=((($call418))|0)!=0; //@line 2479 "readclause.cpp"
              if (!($tobool419)) { __label__ = 153;break $while_cond$12; } //@line 2479 "readclause.cpp"
  
              var $208=HEAP32[(($c1)>>2)]; //@line 2479 "readclause.cpp"
              var $call421=_iswlower($208); //@line 2479 "readclause.cpp"
              var $tobool422=((($call421))|0)!=0; //@line 2479 "readclause.cpp"
              if (!($tobool422)) { __label__ = 153;break $while_cond$12; } //@line 2479 "readclause.cpp"
  
              $end_clause_after_tag=0; //@line 2488 "readclause.cpp"
              ; //@line 2489 "readclause.cpp"
            }
          } while(0);
  
          var $216=HEAP32[(($c1)>>2)]; //@line 2492 "readclause.cpp"
          var $cmp430=((($216))|0)==46; //@line 2492 "readclause.cpp"
          if ($cmp430) { __label__ = 156;; } else { __label__ = 162;; } //@line 2492 "readclause.cpp"
          $land_lhs_true431$$if_end443$169: do { 
            if (__label__ == 156) {
  
              var $217=HEAP32[(($c2)>>2)]; //@line 2492 "readclause.cpp"
              var $cmp432=((($217))|0)==46; //@line 2492 "readclause.cpp"
              if (!($cmp432)) { __label__ = 162;break $land_lhs_true431$$if_end443$169; } //@line 2492 "readclause.cpp"
  
              var $call43538=__ZL4GetCv(); //@line 2494 "readclause.cpp"
              $c_next=$call43538; //@line 2494 "readclause.cpp"
              var $cmp43639=((($call43538))|0)==46; //@line 2494 "readclause.cpp"
              if ($cmp43639) { __label__ = 158;; } else { __label__ = 159;; } //@line 2494 "readclause.cpp"
              $while_body437$$while_end438$172: do { 
                if (__label__ == 158) {
                  while(1) { 
  
                    HEAP32[(($c1)>>2)]=8230; //@line 2497 "readclause.cpp"
                    HEAP32[(($c2)>>2)]=32; //@line 2498 "readclause.cpp"
                    var $call435=__ZL4GetCv(); //@line 2494 "readclause.cpp"
                    $c_next=$call435; //@line 2494 "readclause.cpp"
                    var $cmp436=((($call435))|0)==46; //@line 2494 "readclause.cpp"
                    if ($cmp436) { __label__ = 158;continue ; } else { __label__ = 159;break $while_body437$$while_end438$172; } //@line 2494 "readclause.cpp"
                  }
                }
              } while(0);
  
              var $218=HEAP32[(($c1)>>2)]; //@line 2500 "readclause.cpp"
              var $cmp439=((($218))|0)==8230; //@line 2500 "readclause.cpp"
              var $219=$c_next; //@line 2501 "readclause.cpp"
              if ($cmp439) { __label__ = 160;; } else { __label__ = 161;; } //@line 2500 "readclause.cpp"
              if (__label__ == 160) {
  
                HEAP32[(($c2)>>2)]=$219; //@line 2501 "readclause.cpp"
                ; //@line 2501 "readclause.cpp"
              }
              else if (__label__ == 161) {
  
                __ZL6UngetCi($219); //@line 2503 "readclause.cpp"
                ;
              }
            }
          } while(0);
  
          $punct_data=0; //@line 2506 "readclause.cpp"
          var $220=HEAP32[(($c1)>>2)]; //@line 2507 "readclause.cpp"
          var $call444=__Z11lookupwcharPKti(((__ZL11punct_chars)&4294967295), $220); //@line 2507 "readclause.cpp"
          $punct=$call444; //@line 2507 "readclause.cpp"
          var $cmp445=((($call444))|0)!=0; //@line 2507 "readclause.cpp"
          if ($cmp445) { __label__ = 163;; } else { __label__ = 171;; } //@line 2507 "readclause.cpp"
          $if_then446$$if_end471$180: do { 
            if (__label__ == 163) {
  
              var $221=$punct; //@line 2509 "readclause.cpp"
              var $arrayidx447=((__ZL16punct_attributes+$221*4)&4294967295); //@line 2509 "readclause.cpp"
              var $222=HEAP32[(($arrayidx447)>>2)]; //@line 2509 "readclause.cpp"
              $punct_data=$222; //@line 2509 "readclause.cpp"
              var $223=$punct_data; //@line 2511 "readclause.cpp"
              var $and448=($223) & 1048576; //@line 2511 "readclause.cpp"
              var $tobool449=((($and448))|0)!=0; //@line 2511 "readclause.cpp"
              if ($tobool449) { __label__ = 164;; } else { __label__ = 165;; } //@line 2511 "readclause.cpp"
              if (__label__ == 164) {
  
                $stressed_word=1; //@line 2514 "readclause.cpp"
                var $224=$punct_data; //@line 2515 "readclause.cpp"
                var $shr=($224) >> 12; //@line 2515 "readclause.cpp"
                var $and451=($shr) & 15; //@line 2515 "readclause.cpp"
                var $225=$tone_type_addr; //@line 2515 "readclause.cpp"
                HEAP32[(($225)>>2)]=$and451; //@line 2515 "readclause.cpp"
                __label__ = 8;continue $while_cond$12; //@line 2516 "readclause.cpp"
              }
              else if (__label__ == 165) {
  
                var $226=HEAP32[(($c2)>>2)]; //@line 2519 "readclause.cpp"
                var $call453=_iswspace($226); //@line 2519 "readclause.cpp"
                var $tobool454=((($call453))|0)!=0; //@line 2519 "readclause.cpp"
                if ($tobool454) { __label__ = 170;; } else { __label__ = 166;; } //@line 2519 "readclause.cpp"
                $if_then469$$lor_lhs_false455$185: do { 
                  if (__label__ == 166) {
  
                    var $227=$punct_data; //@line 2519 "readclause.cpp"
                    var $and456=($227) & 32768; //@line 2519 "readclause.cpp"
                    var $tobool457=((($and456))|0)!=0; //@line 2519 "readclause.cpp"
                    if ($tobool457) { __label__ = 170;break $if_then469$$lor_lhs_false455$185; } //@line 2519 "readclause.cpp"
  
                    var $228=HEAP32[(($c2)>>2)]; //@line 2519 "readclause.cpp"
                    var $call459=__Z9IsBracketi($228); //@line 2519 "readclause.cpp"
                    var $tobool460=((($call459))|0)!=0; //@line 2519 "readclause.cpp"
                    var $229=HEAP32[(($c2)>>2)]; //@line 2519 "readclause.cpp"
                    var $cmp462=((($229))|0)==63; //@line 2519 "readclause.cpp"
                    var $or_cond15=($tobool460) | ($cmp462); //@line 2519 "readclause.cpp"
                    if ($or_cond15) { __label__ = 170;break $if_then469$$lor_lhs_false455$185; } //@line 2519 "readclause.cpp"
  
                    var $call464=__Z3Eofv(); //@line 2519 "readclause.cpp"
                    var $tobool465=((($call464))|0)!=0; //@line 2519 "readclause.cpp"
                    if ($tobool465) { __label__ = 170;break $if_then469$$lor_lhs_false455$185; } //@line 2519 "readclause.cpp"
  
                    var $230=HEAP32[(($c2)>>2)]; //@line 2519 "readclause.cpp"
                    var $231=HEAP8[(_ctrl_embedded)]; //@line 2519 "readclause.cpp"
                    var $conv467=(tempInt=(($231)),(tempInt>=128?tempInt-256:tempInt)); //@line 2519 "readclause.cpp"
                    var $cmp468=((($230))|0)==((($conv467))|0); //@line 2519 "readclause.cpp"
                    if (!($cmp468)) { __label__ = 171;break $if_then446$$if_end471$180; } //@line 2519 "readclause.cpp"
                  }
                } while(0);
  
                $is_end_clause=1; //@line 2523 "readclause.cpp"
                ; //@line 2524 "readclause.cpp"
              }
            }
          } while(0);
  
          var $232=HEAP32[(($c1)>>2)]; //@line 2528 "readclause.cpp"
          var $cmp472=((($232))|0)==57404; //@line 2528 "readclause.cpp"
          if ($cmp472) { __label__ = 172;; } else { __label__ = 173;; } //@line 2528 "readclause.cpp"
          if (__label__ == 172) {
  
            HEAP32[(($c1)>>2)]=60; //@line 2528 "readclause.cpp"
            ; //@line 2528 "readclause.cpp"
          }
  
          var $233=HEAP32[((_option_punctuation)>>2)]; //@line 2529 "readclause.cpp"
          var $tobool475=((($233))|0)!=0; //@line 2529 "readclause.cpp"
          if ($tobool475) { __label__ = 174;; } else { __label__ = 181;; } //@line 2529 "readclause.cpp"
          $land_lhs_true476$$if_end493$195: do { 
            if (__label__ == 174) {
  
              var $234=HEAP32[(($c1)>>2)]; //@line 2529 "readclause.cpp"
              var $call477=_iswpunct($234); //@line 2529 "readclause.cpp"
              var $tobool478=((($call477))|0)!=0; //@line 2529 "readclause.cpp"
              if (!($tobool478)) { __label__ = 181;break $land_lhs_true476$$if_end493$195; } //@line 2529 "readclause.cpp"
  
              var $_b2=HEAPU8[(__ZL10audio_text_b)];
              var $235=((($_b2))&1);
              var $cmp480=((($235))|0)==0; //@line 2529 "readclause.cpp"
              if (!($cmp480)) { __label__ = 181;break $land_lhs_true476$$if_end493$195; } //@line 2529 "readclause.cpp"
  
              var $236=HEAP32[((_option_punctuation)>>2)]; //@line 2533 "readclause.cpp"
              var $cmp482=((($236))|0)==1; //@line 2533 "readclause.cpp"
              if ($cmp482) { __label__ = 178;; } else { __label__ = 177;; } //@line 2533 "readclause.cpp"
              if (__label__ == 177) {
  
                var $237=HEAP32[(($c1)>>2)]; //@line 2533 "readclause.cpp"
                var $call484=_wcschr(((_option_punctlist)&4294967295), $237); //@line 2533 "readclause.cpp"
                var $cmp485=((($call484))|0)!=0; //@line 2533 "readclause.cpp"
                if (!($cmp485)) { __label__ = 181;break $land_lhs_true476$$if_end493$195; } //@line 2533 "readclause.cpp"
              }
  
              var $238=$tr_addr; //@line 2535 "readclause.cpp"
              var $phonemes_repeat_count487=(($238+652)&4294967295); //@line 2535 "readclause.cpp"
              HEAP32[(($phonemes_repeat_count487)>>2)]=0; //@line 2535 "readclause.cpp"
              var $239=$tr_addr; //@line 2536 "readclause.cpp"
              var $240=HEAP32[(($c1)>>2)]; //@line 2536 "readclause.cpp"
              var $241=$buf_addr; //@line 2536 "readclause.cpp"
              var $242=$is_end_clause; //@line 2536 "readclause.cpp"
              var $call488=__ZL19AnnouncePunctuationP10TranslatoriPiPcS1_i($239, $240, $c2, $241, $ix, $242); //@line 2536 "readclause.cpp"
              $terminator=$call488; //@line 2536 "readclause.cpp"
              var $cmp489=((($call488))|0) >= 0; //@line 2536 "readclause.cpp"
              if ($cmp489) { __label__ = 179;break $while_cond$12; } //@line 2536 "readclause.cpp"
  
              var $244=HEAP32[(($c1)>>2)]; //@line 2538 "readclause.cpp"
              $announced_punctuation=$244; //@line 2538 "readclause.cpp"
              ; //@line 2539 "readclause.cpp"
            }
          } while(0);
  
          var $245=$punct_data; //@line 2542 "readclause.cpp"
          var $and494=($245) & 2097152; //@line 2542 "readclause.cpp"
          var $tobool495=((($and494))|0)!=0; //@line 2542 "readclause.cpp"
          if ($tobool495) { __label__ = 182;; } else { __label__ = 185;; } //@line 2542 "readclause.cpp"
          $land_lhs_true496$$if_end510$204: do { 
            if (__label__ == 182) {
  
              var $246=$announced_punctuation; //@line 2542 "readclause.cpp"
              var $cmp497=((($246))|0)==0; //@line 2542 "readclause.cpp"
              if (!($cmp497)) { __label__ = 185;break $land_lhs_true496$$if_end510$204; } //@line 2542 "readclause.cpp"
  
              var $247=HEAP32[(($ix)>>2)]; //@line 2547 "readclause.cpp"
              var $248=$buf_addr; //@line 2547 "readclause.cpp"
              var $arrayidx499=(($248+$247)&4294967295); //@line 2547 "readclause.cpp"
              $p2=$arrayidx499; //@line 2547 "readclause.cpp"
              var $249=$p2; //@line 2548 "readclause.cpp"
              var $250=$tr_addr; //@line 2548 "readclause.cpp"
              var $251=HEAP32[(($c1)>>2)]; //@line 2548 "readclause.cpp"
              __ZL14LookupCharNameP10Translatorii($250, $251, 1); //@line 2548 "readclause.cpp"
              var $strlen20=_strlen(((__ZZL14LookupCharNameP10TranslatoriiE3buf)&4294967295)); //@line 2548 "readclause.cpp"
              var $leninc21=((($strlen20)+1)&4294967295); //@line 2548 "readclause.cpp"
              var $src$, $dest$, $stop$, $stop4$, $fast$;
              $src$ = ((__ZZL14LookupCharNameP10TranslatoriiE3buf)&4294967295);
              $dest$ = $249;
              $stop$ = $src$ + $leninc21;
              if (($dest$%4) == ($src$%4) && $leninc21 > 8) {
                while ($src$%4 !== 0 && $src$ < $stop$) {
                  ; HEAP8[$dest$++] = HEAP8[$src$++];
                }
                $src$ >>= 2;
                $dest$ >>= 2;
                $stop4$ = $stop$ >> 2;
                while ($src$ < $stop4$) {
              ;;;    HEAP32[$dest$++] = HEAP32[$src$++];
                }
                $src$ <<= 2;
                $dest$ <<= 2;
              }while ($src$ < $stop$) {
                ; HEAP8[$dest$++] = HEAP8[$src$++];
              }; //@line 2548 "readclause.cpp"
              var $252=$p2; //@line 2549 "readclause.cpp"
              var $arrayidx502=(($252)&4294967295); //@line 2549 "readclause.cpp"
              var $253=HEAP8[($arrayidx502)]; //@line 2549 "readclause.cpp"
              var $conv503=(tempInt=(($253)),(tempInt>=128?tempInt-256:tempInt)); //@line 2549 "readclause.cpp"
              var $cmp504=((($conv503))|0)!=0; //@line 2549 "readclause.cpp"
              if (!($cmp504)) { __label__ = 185;break $land_lhs_true496$$if_end510$204; } //@line 2549 "readclause.cpp"
  
              var $254=$p2; //@line 2551 "readclause.cpp"
              var $call506=_strlen($254); //@line 2551 "readclause.cpp"
              var $255=HEAP32[(($ix)>>2)]; //@line 2551 "readclause.cpp"
              var $add507=((($255)+($call506))&4294967295); //@line 2551 "readclause.cpp"
              HEAP32[(($ix)>>2)]=$add507; //@line 2551 "readclause.cpp"
              var $256=HEAP32[(($c1)>>2)]; //@line 2552 "readclause.cpp"
              $announced_punctuation=$256; //@line 2552 "readclause.cpp"
              var $257=$punct_data; //@line 2553 "readclause.cpp"
              var $and508=($257) & -28673; //@line 2553 "readclause.cpp"
              $punct_data=$and508; //@line 2553 "readclause.cpp"
              ; //@line 2554 "readclause.cpp"
            }
          } while(0);
  
          var $258=$is_end_clause; //@line 2557 "readclause.cpp"
          var $tobool511=((($258))|0)!=0; //@line 2557 "readclause.cpp"
          if (!($tobool511)) { __label__ = 238;break $if_then412$$if_end643$161; } //@line 2557 "readclause.cpp"
  
          $nl_count=0; //@line 2559 "readclause.cpp"
          var $259=HEAP32[(($c2)>>2)]; //@line 2560 "readclause.cpp"
          $c_next=$259; //@line 2560 "readclause.cpp"
          var $260=$c_next; //@line 2562 "readclause.cpp"
          var $call513=_iswspace($260); //@line 2562 "readclause.cpp"
          var $tobool514=((($call513))|0)!=0; //@line 2562 "readclause.cpp"
          if ($tobool514) { __label__ = 187;; } else { __label__ = 192;; } //@line 2562 "readclause.cpp"
          $while_cond516$$if_end530$210: do { 
            if (__label__ == 187) {
              while(1) { 
  
                var $call517=__Z3Eofv(); //@line 2564 "readclause.cpp"
                var $tobool518=((($call517))|0)!=0; //@line 2564 "readclause.cpp"
                if ($tobool518) { __label__ = 192;break $while_cond516$$if_end530$210; } //@line 2564 "readclause.cpp"
  
                var $261=$c_next; //@line 2564 "readclause.cpp"
                var $call520=_iswspace($261); //@line 2564 "readclause.cpp"
                var $tobool521=((($call520))|0)!=0; //@line 2564 "readclause.cpp"
                if (!($tobool521)) { __label__ = 192;break $while_cond516$$if_end530$210; }
  
                var $262=$c_next; //@line 2566 "readclause.cpp"
                var $cmp524=((($262))|0)==10; //@line 2566 "readclause.cpp"
                if ($cmp524) { __label__ = 190;; } else { __label__ = 191;; } //@line 2566 "readclause.cpp"
                if (__label__ == 190) {
  
                  var $263=$nl_count; //@line 2567 "readclause.cpp"
                  var $inc526=((($263)+1)&4294967295); //@line 2567 "readclause.cpp"
                  $nl_count=$inc526; //@line 2567 "readclause.cpp"
                  ; //@line 2567 "readclause.cpp"
                }
  
                var $call528=__ZL4GetCv(); //@line 2568 "readclause.cpp"
                $c_next=$call528; //@line 2568 "readclause.cpp"
                __label__ = 187;continue ; //@line 2569 "readclause.cpp"
              }
            }
          } while(0);
  
          var $264=HEAP32[(($c1)>>2)]; //@line 2572 "readclause.cpp"
          var $cmp531=((($264))|0)==46; //@line 2572 "readclause.cpp"
          if ($cmp531) { __label__ = 193;; } else { __label__ = 195;; } //@line 2572 "readclause.cpp"
          $land_lhs_true532$$if_end535$219: do { 
            if (__label__ == 193) {
  
              var $265=$nl_count; //@line 2572 "readclause.cpp"
              var $cmp533=((($265))|0) < 2; //@line 2572 "readclause.cpp"
              if (!($cmp533)) { __label__ = 226;break $land_lhs_true532$$if_end535$219; } //@line 2572 "readclause.cpp"
  
              var $266=$punct_data; //@line 2574 "readclause.cpp"
              var $or=($266) | 4194304; //@line 2574 "readclause.cpp"
              $punct_data=$or; //@line 2574 "readclause.cpp"
              __label__ = 195;break $land_lhs_true532$$if_end535$219; //@line 2575 "readclause.cpp"
            }
          } while(0);
          $if_end535$$if_end609$222: do { 
            if (__label__ == 195) {
  
              var $_pr26=$nl_count; //@line 2577 "readclause.cpp"
              var $cmp536=((($_pr26))|0)==0; //@line 2577 "readclause.cpp"
              if (!($cmp536)) { __label__ = 226;break $if_end535$$if_end609$222; } //@line 2577 "readclause.cpp"
  
              var $267=HEAP32[(($c1)>>2)]; //@line 2579 "readclause.cpp"
              var $cmp538=((($267))|0)==44; //@line 2579 "readclause.cpp"
              if ($cmp538) { __lastLabel__ = 196; __label__ = 197;; } else { __lastLabel__ = 196; __label__ = 204;; } //@line 2579 "readclause.cpp"
              $land_lhs_true539$$if_end554$225: do { 
                if (__label__ == 197) {
  
                  var $268=$cprev; //@line 2579 "readclause.cpp"
                  var $cmp540=((($268))|0)==46; //@line 2579 "readclause.cpp"
                  if ($cmp540) { __label__ = 198;; } else { __label__ = 203;; } //@line 2579 "readclause.cpp"
                  $land_lhs_true541$$if_end554thread_pre_split$227: do { 
                    if (__label__ == 198) {
  
                      var $269=$tr_addr; //@line 2579 "readclause.cpp"
                      var $translator_name542=(($269+280)&4294967295); //@line 2579 "readclause.cpp"
                      var $270=HEAP32[(($translator_name542)>>2)]; //@line 2579 "readclause.cpp"
                      var $cmp543=((($270))|0)==26741; //@line 2579 "readclause.cpp"
                      if (!($cmp543)) { __label__ = 203;break $land_lhs_true541$$if_end554thread_pre_split$227; } //@line 2579 "readclause.cpp"
  
                      var $271=$cprev2; //@line 2579 "readclause.cpp"
                      var $call545=_iswdigit($271); //@line 2579 "readclause.cpp"
                      var $tobool546=((($call545))|0)!=0; //@line 2579 "readclause.cpp"
                      if (!($tobool546)) { __label__ = 203;break $land_lhs_true541$$if_end554thread_pre_split$227; } //@line 2579 "readclause.cpp"
  
                      var $272=$c_next; //@line 2579 "readclause.cpp"
                      var $call548=_iswdigit($272); //@line 2579 "readclause.cpp"
                      var $tobool549=((($call548))|0)!=0; //@line 2579 "readclause.cpp"
                      if ($tobool549) { __label__ = 202;; } else { __label__ = 201;; } //@line 2579 "readclause.cpp"
                      if (__label__ == 201) {
  
                        var $273=$c_next; //@line 2579 "readclause.cpp"
                        var $call551=_iswlower($273); //@line 2579 "readclause.cpp"
                        var $tobool552=((($call551))|0)!=0; //@line 2579 "readclause.cpp"
                        if (!($tobool552)) { __label__ = 203;break $land_lhs_true541$$if_end554thread_pre_split$227; } //@line 2579 "readclause.cpp"
                      }
  
                      HEAP32[(($c1)>>2)]=1367; //@line 2582 "readclause.cpp"
                      $is_end_clause=0; //@line 2583 "readclause.cpp"
                      __label__ = 220;break $land_lhs_true539$$if_end554$225; //@line 2586 "readclause.cpp"
                    }
                  } while(0);
  
                  var $_pr28=HEAP32[(($c1)>>2)]; //@line 2586 "readclause.cpp"
                  __lastLabel__ = 203; __label__ = 204;break $land_lhs_true539$$if_end554$225;
                }
              } while(0);
              $if_else595$$if_end554$235: do { 
                if (__label__ == 204) {
  
                  var $274=__lastLabel__ == 203 ? $_pr28 : ($267); //@line 2586 "readclause.cpp"
                  var $cmp555=((($274))|0)==46; //@line 2586 "readclause.cpp"
                  if (!($cmp555)) { __label__ = 220;break $if_else595$$if_end554$235; } //@line 2586 "readclause.cpp"
  
                  var $275=$tr_addr; //@line 2588 "readclause.cpp"
                  var $langopts=(($275)&4294967295); //@line 2588 "readclause.cpp"
                  var $numbers=(($langopts+184)&4294967295); //@line 2588 "readclause.cpp"
                  var $276=HEAP32[(($numbers)>>2)]; //@line 2588 "readclause.cpp"
                  var $and557=($276) & 65536; //@line 2588 "readclause.cpp"
                  var $tobool558=((($and557))|0)!=0; //@line 2588 "readclause.cpp"
                  if ($tobool558) { __label__ = 206;; } else { __label__ = 214;; } //@line 2588 "readclause.cpp"
                  $land_lhs_true559$$if_else583$238: do { 
                    if (__label__ == 206) {
  
                      var $277=$cprev; //@line 2589 "readclause.cpp"
                      var $call560=_iswdigit($277); //@line 2589 "readclause.cpp"
                      var $tobool561=((($call560))|0)!=0; //@line 2589 "readclause.cpp"
                      if ($tobool561) { __label__ = 210;; } else { __label__ = 207;; } //@line 2589 "readclause.cpp"
                      $if_then571$$lor_lhs_false562$240: do { 
                        if (__label__ == 207) {
  
                          var $278=$cprev; //@line 2589 "readclause.cpp"
                          var $call563=__ZL8IsRomanUj($278); //@line 2589 "readclause.cpp"
                          var $tobool564=((($call563))|0)!=0; //@line 2589 "readclause.cpp"
                          if (!($tobool564)) { __label__ = 214;break $land_lhs_true559$$if_else583$238; } //@line 2589 "readclause.cpp"
  
                          var $279=$cprev2; //@line 2589 "readclause.cpp"
                          var $call566=__ZL8IsRomanUj($279); //@line 2589 "readclause.cpp"
                          var $tobool567=((($call566))|0)!=0; //@line 2589 "readclause.cpp"
                          if ($tobool567) { __label__ = 210;break $if_then571$$lor_lhs_false562$240; } //@line 2589 "readclause.cpp"
  
                          var $280=$cprev2; //@line 2589 "readclause.cpp"
                          var $call569=_iswspace($280); //@line 2589 "readclause.cpp"
                          var $tobool570=((($call569))|0)!=0; //@line 2589 "readclause.cpp"
                          if (!($tobool570)) { __label__ = 214;break $land_lhs_true559$$if_else583$238; } //@line 2589 "readclause.cpp"
                        }
                      } while(0);
  
                      var $281=$cprev; //@line 2592 "readclause.cpp"
                      var $call572=_iswdigit($281); //@line 2592 "readclause.cpp"
                      var $tobool573=((($call572))|0)!=0; //@line 2592 "readclause.cpp"
                      if ($tobool573) { __label__ = 212;; } else { __label__ = 211;; } //@line 2592 "readclause.cpp"
                      if (__label__ == 212) {
  
                        var $282=$c_next; //@line 2598 "readclause.cpp"
                        var $call576=_iswlower($282); //@line 2598 "readclause.cpp"
                        var $tobool577=((($call576))|0)!=0; //@line 2598 "readclause.cpp"
                        var $283=$c_next; //@line 2598 "readclause.cpp"
                        var $cmp579=((($283))|0)==45; //@line 2598 "readclause.cpp"
                        var $or_cond16=($tobool577) | ($cmp579); //@line 2598 "readclause.cpp"
                        if (!($or_cond16)) { __label__ = 216;break $land_lhs_true559$$if_else583$238; } //@line 2598 "readclause.cpp"
  
                        $is_end_clause=0; //@line 2599 "readclause.cpp"
                        __label__ = 216;break $land_lhs_true559$$if_else583$238; //@line 2599 "readclause.cpp"
                      }
                      else if (__label__ == 211) {
  
                        $is_end_clause=0; //@line 2594 "readclause.cpp"
                        __label__ = 216;break $land_lhs_true559$$if_else583$238; //@line 2595 "readclause.cpp"
                      }
                    }
                  } while(0);
                  $if_end587$$if_else583$249: do { 
                    if (__label__ == 214) {
  
                      var $284=$c_next; //@line 2603 "readclause.cpp"
                      var $cmp584=((($284))|0)==39; //@line 2603 "readclause.cpp"
                      if (!($cmp584)) { __label__ = 216;break $if_end587$$if_else583$249; } //@line 2603 "readclause.cpp"
  
                      $is_end_clause=0; //@line 2605 "readclause.cpp"
                      ; //@line 2606 "readclause.cpp"
                    }
                  } while(0);
  
                  var $285=$c_next; //@line 2607 "readclause.cpp"
                  var $call588=_iswlower($285); //@line 2607 "readclause.cpp"
                  var $tobool589=((($call588))|0)!=0; //@line 2607 "readclause.cpp"
                  if ($tobool589) { __label__ = 217;; } else { __label__ = 218;; } //@line 2607 "readclause.cpp"
                  if (__label__ == 217) {
  
                    $is_end_clause=0; //@line 2611 "readclause.cpp"
                    ; //@line 2612 "readclause.cpp"
                  }
  
                  var $286=$any_alnum; //@line 2613 "readclause.cpp"
                  var $cmp592=((($286))|0)==0; //@line 2613 "readclause.cpp"
                  if (!($cmp592)) { __label__ = 222;break $if_else595$$if_end554$235; } //@line 2613 "readclause.cpp"
  
                  HEAP32[(($c1)>>2)]=32; //@line 2617 "readclause.cpp"
                  $is_end_clause=0; //@line 2618 "readclause.cpp"
                  __label__ = 235;break $if_end535$$if_end609$222; //@line 2619 "readclause.cpp"
                }
              } while(0);
              $if_else595$$if_end599$257: do { 
                if (__label__ == 220) {
  
                  var $287=$any_alnum; //@line 2623 "readclause.cpp"
                  var $cmp596=((($287))|0)==0; //@line 2623 "readclause.cpp"
                  if (!($cmp596)) { __label__ = 222;break $if_else595$$if_end599$257; } //@line 2623 "readclause.cpp"
  
                  $is_end_clause=0; //@line 2626 "readclause.cpp"
                  __label__ = 235;break $if_end535$$if_end609$222; //@line 2627 "readclause.cpp"
                }
              } while(0);
  
              var $_pr30=$is_end_clause; //@line 2630 "readclause.cpp"
              var $tobool600=((($_pr30))|0)!=0; //@line 2630 "readclause.cpp"
              if (!($tobool600)) { __label__ = 235;break $if_end535$$if_end609$222; } //@line 2630 "readclause.cpp"
  
              var $288=HEAP32[(($c1)>>2)]; //@line 2630 "readclause.cpp"
              var $cmp602=((($288))|0)==46; //@line 2630 "readclause.cpp"
              if (!($cmp602)) { __label__ = 226;break $if_end535$$if_end609$222; } //@line 2630 "readclause.cpp"
  
              var $289=$c_next; //@line 2630 "readclause.cpp"
              var $cmp604=((($289))|0)==60; //@line 2630 "readclause.cpp"
              var $290=HEAP32[((_option_ssml)>>2)]; //@line 2630 "readclause.cpp"
              var $tobool606=((($290))|0)!=0; //@line 2630 "readclause.cpp"
              var $or_cond17=($cmp604) & ($tobool606); //@line 2630 "readclause.cpp"
              if (!($or_cond17)) { __label__ = 226;break $if_end535$$if_end609$222; } //@line 2630 "readclause.cpp"
  
              $is_end_clause=0; //@line 2633 "readclause.cpp"
              var $291=HEAP32[(($ix)>>2)]; //@line 2634 "readclause.cpp"
              $end_clause_index=$291; //@line 2634 "readclause.cpp"
              var $292=$punct_data; //@line 2635 "readclause.cpp"
              $end_clause_after_tag=$292; //@line 2635 "readclause.cpp"
              __label__ = 235;break $if_end535$$if_end609$222; //@line 2636 "readclause.cpp"
            }
          } while(0);
          if (__label__ == 226) {
  
            var $_pr32_pr=$is_end_clause; //@line 2639 "readclause.cpp"
            var $tobool610=((($_pr32_pr))|0)!=0; //@line 2639 "readclause.cpp"
            if ($tobool610) { __label__ = 227;break $while_cond$12; } //@line 2639 "readclause.cpp"
          }
  
          var $call633=__Z3Eofv(); //@line 2659 "readclause.cpp"
          var $tobool634=((($call633))|0)!=0; //@line 2659 "readclause.cpp"
          if ($tobool634) { __label__ = 238;break $if_then412$$if_end643$161; } //@line 2659 "readclause.cpp"
  
          var $305=HEAP32[(($c2)>>2)]; //@line 2661 "readclause.cpp"
          var $call636=_iswspace($305); //@line 2661 "readclause.cpp"
          var $tobool637=((($call636))|0)!=0; //@line 2661 "readclause.cpp"
          if (!($tobool637)) { __label__ = 238;break $if_then412$$if_end643$161; } //@line 2661 "readclause.cpp"
  
          var $306=$c_next; //@line 2662 "readclause.cpp"
          __ZL6UngetCi($306); //@line 2662 "readclause.cpp"
          ; //@line 2662 "readclause.cpp"
        }
      } while(0);
  
      var $307=HEAP32[((((__ZL17speech_parameters)&4294967295))>>2)]; //@line 2668 "readclause.cpp"
      var $cmp644=((($307))|0)==1; //@line 2668 "readclause.cpp"
      if ($cmp644) { __label__ = 8;continue $while_cond$12; } //@line 2668 "readclause.cpp"
  
      var $308=HEAP32[(($c1)>>2)]; //@line 2671 "readclause.cpp"
      var $309=$announced_punctuation; //@line 2671 "readclause.cpp"
      var $cmp647=((($308))|0)==((($309))|0); //@line 2671 "readclause.cpp"
      if ($cmp647) { __label__ = 240;; } else { __label__ = 245;; } //@line 2671 "readclause.cpp"
      $if_then648$$if_end657$271: do { 
        if (__label__ == 240) {
  
          var $310=HEAP32[(($c1)>>2)]; //@line 2675 "readclause.cpp"
          var $call649=__Z9IsBracketi($310); //@line 2675 "readclause.cpp"
          var $tobool650=((($call649))|0)!=0; //@line 2675 "readclause.cpp"
          if ($tobool650) { __label__ = 241;; } else { __label__ = 242;; } //@line 2675 "readclause.cpp"
          $if_then651$$if_else652$273: do { 
            if (__label__ == 241) {
  
              HEAP32[(($c1)>>2)]=57384; //@line 2677 "readclause.cpp"
              ; //@line 2678 "readclause.cpp"
            }
            else if (__label__ == 242) {
  
              var $311=HEAP32[(($c1)>>2)]; //@line 2680 "readclause.cpp"
              var $cmp653=((($311))|0)!=45; //@line 2680 "readclause.cpp"
              if (!($cmp653)) { __label__ = 244;break $if_then651$$if_else652$273; } //@line 2680 "readclause.cpp"
  
              HEAP32[(($c1)>>2)]=32; //@line 2682 "readclause.cpp"
              ; //@line 2683 "readclause.cpp"
            }
          } while(0);
  
          var $312=HEAP32[(($ix)>>2)]; //@line 2686 "readclause.cpp"
          var $add65834=((($312)+1)&4294967295); //@line 2686 "readclause.cpp"
          $j=$add65834; //@line 2686 "readclause.cpp"
          ; //@line 2688 "readclause.cpp"
        }
        else if (__label__ == 245) {
  
          var $313=HEAP32[(($ix)>>2)]; //@line 2686 "readclause.cpp"
          var $add658=((($313)+1)&4294967295); //@line 2686 "readclause.cpp"
          $j=$add658; //@line 2686 "readclause.cpp"
          var $cmp659=((($308))|0)==57404; //@line 2688 "readclause.cpp"
          if (!($cmp659)) { __label__ = 247;break $if_then648$$if_end657$271; } //@line 2688 "readclause.cpp"
  
          HEAP32[(($c1)>>2)]=60; //@line 2688 "readclause.cpp"
          ; //@line 2688 "readclause.cpp"
        }
      } while(0);
  
      var $314=HEAP32[(($c1)>>2)]; //@line 2690 "readclause.cpp"
      var $315=HEAP32[(($ix)>>2)]; //@line 2690 "readclause.cpp"
      var $316=$buf_addr; //@line 2690 "readclause.cpp"
      var $arrayidx662=(($316+$315)&4294967295); //@line 2690 "readclause.cpp"
      var $call663=__Z8utf8_outjPc($314, $arrayidx662); //@line 2690 "readclause.cpp"
      var $317=HEAP32[(($ix)>>2)]; //@line 2690 "readclause.cpp"
      var $add664=((($317)+($call663))&4294967295); //@line 2690 "readclause.cpp"
      HEAP32[(($ix)>>2)]=$add664; //@line 2690 "readclause.cpp"
      var $318=HEAP32[(($c1)>>2)]; //@line 2691 "readclause.cpp"
      var $call665=_iswspace($318); //@line 2691 "readclause.cpp"
      var $tobool666=((($call665))|0)!=0; //@line 2691 "readclause.cpp"
      if ($tobool666) { __label__ = 251;; } else { __label__ = 248;; } //@line 2691 "readclause.cpp"
      $if_end680$$land_lhs_true667$281: do { 
        if (__label__ == 248) {
  
          var $319=HEAP32[(($c1)>>2)]; //@line 2691 "readclause.cpp"
          var $call668=__Z9IsBracketi($319); //@line 2691 "readclause.cpp"
          var $tobool669=((($call668))|0)!=0; //@line 2691 "readclause.cpp"
          if ($tobool669) { __label__ = 251;break $if_end680$$land_lhs_true667$281; } //@line 2691 "readclause.cpp"
  
          var $320=HEAP32[((_count_characters)>>2)]; //@line 2693 "readclause.cpp"
          var $321=HEAP32[((_clause_start_char)>>2)]; //@line 2693 "readclause.cpp"
          var $sub671=((($320)-($321))&4294967295); //@line 2693 "readclause.cpp"
          var $conv672=((($sub671)) & 65535); //@line 2693 "readclause.cpp"
          var $322=HEAP32[(($ix)>>2)]; //@line 2693 "readclause.cpp"
          var $323=$charix_addr; //@line 2693 "readclause.cpp"
          var $arrayidx673=(($323+2*$322)&4294967295); //@line 2693 "readclause.cpp"
          HEAP16[(($arrayidx673)>>1)]=$conv672; //@line 2693 "readclause.cpp"
          var $324=$j; //@line 2694 "readclause.cpp"
          var $325=HEAP32[(($ix)>>2)]; //@line 2694 "readclause.cpp"
          var $cmp67537=((($324))|0) < ((($325))|0); //@line 2694 "readclause.cpp"
          if (!($cmp67537)) { __label__ = 251;break $if_end680$$land_lhs_true667$281; } //@line 2694 "readclause.cpp"
          while(1) { 
  
            var $326=$j; //@line 2695 "readclause.cpp"
            var $inc677=((($326)+1)&4294967295); //@line 2695 "readclause.cpp"
            $j=$inc677; //@line 2695 "readclause.cpp"
            var $327=$charix_addr; //@line 2695 "readclause.cpp"
            var $arrayidx678=(($327+2*$326)&4294967295); //@line 2695 "readclause.cpp"
            HEAP16[(($arrayidx678)>>1)]=-1; //@line 2695 "readclause.cpp"
            var $328=$j; //@line 2694 "readclause.cpp"
            var $329=HEAP32[(($ix)>>2)]; //@line 2694 "readclause.cpp"
            var $cmp675=((($328))|0) < ((($329))|0); //@line 2694 "readclause.cpp"
            if ($cmp675) { __label__ = 250;continue ; } else { __label__ = 251;break $if_end680$$land_lhs_true667$281; } //@line 2694 "readclause.cpp"
          }
        }
      } while(0);
  
      var $330=HEAP32[(($ix)>>2)]; //@line 2697 "readclause.cpp"
      var $331=$charix_top_addr; //@line 2697 "readclause.cpp"
      HEAP32[(($331)>>2)]=$330; //@line 2697 "readclause.cpp"
      var $332=HEAP32[(($ix)>>2)]; //@line 2699 "readclause.cpp"
      var $333=$n_buf_addr; //@line 2699 "readclause.cpp"
      var $sub681=((($333)-20)&4294967295); //@line 2699 "readclause.cpp"
      var $cmp682=((($332))|0) > ((($sub681))|0); //@line 2699 "readclause.cpp"
      if ($cmp682) { __label__ = 252;; } else { __label__ = 254;; } //@line 2699 "readclause.cpp"
      $land_lhs_true683$$lor_lhs_false689$287: do { 
        if (__label__ == 252) {
  
          var $334=HEAP32[(($c1)>>2)]; //@line 2699 "readclause.cpp"
          var $call684=__Z7IsAlphaj($334); //@line 2699 "readclause.cpp"
          var $tobool685=((($call684))|0)!=0; //@line 2699 "readclause.cpp"
          if ($tobool685) { __label__ = 254;break $land_lhs_true683$$lor_lhs_false689$287; } //@line 2699 "readclause.cpp"
  
          var $335=HEAP32[(($c1)>>2)]; //@line 2699 "readclause.cpp"
          var $call687=_iswdigit($335); //@line 2699 "readclause.cpp"
          var $tobool688=((($call687))|0)!=0; //@line 2699 "readclause.cpp"
          if (!($tobool688)) { __label__ = 255;break $while_cond$12; } //@line 2699 "readclause.cpp"
        }
      } while(0);
  
      var $336=HEAP32[(($ix)>>2)]; //@line 2699 "readclause.cpp"
      var $337=$n_buf_addr; //@line 2699 "readclause.cpp"
      var $sub690=((($337)-2)&4294967295); //@line 2699 "readclause.cpp"
      var $cmp691=((($336))|0) >= ((($sub690))|0); //@line 2699 "readclause.cpp"
      if ($cmp691) { __label__ = 255;break $while_cond$12; } else { __label__ = 8;continue $while_cond$12; } //@line 2699 "readclause.cpp"
    }
    if (__label__ == 256) {
  
      var $343=$stressed_word; //@line 2710 "readclause.cpp"
      var $tobool698=((($343))|0)!=0; //@line 2710 "readclause.cpp"
      if ($tobool698) { __label__ = 257;; } else { __label__ = 258;; } //@line 2710 "readclause.cpp"
      if (__label__ == 257) {
  
        var $344=HEAP32[(($ix)>>2)]; //@line 2712 "readclause.cpp"
        var $345=$buf_addr; //@line 2712 "readclause.cpp"
        var $arrayidx700=(($345+$344)&4294967295); //@line 2712 "readclause.cpp"
        var $call701=__Z8utf8_outjPc(1328, $arrayidx700); //@line 2712 "readclause.cpp"
        var $346=HEAP32[(($ix)>>2)]; //@line 2712 "readclause.cpp"
        var $add702=((($346)+($call701))&4294967295); //@line 2712 "readclause.cpp"
        HEAP32[(($ix)>>2)]=$add702; //@line 2712 "readclause.cpp"
        ; //@line 2713 "readclause.cpp"
      }
  
      var $347=$end_clause_after_tag; //@line 2714 "readclause.cpp"
      var $tobool704=((($347))|0)!=0; //@line 2714 "readclause.cpp"
      if ($tobool704) { __label__ = 259;; } else { __label__ = 260;; } //@line 2714 "readclause.cpp"
      if (__label__ == 259) {
  
        var $348=$end_clause_index; //@line 2716 "readclause.cpp"
        var $349=$buf_addr; //@line 2716 "readclause.cpp"
        var $arrayidx706=(($349+$348)&4294967295); //@line 2716 "readclause.cpp"
        __ZL10RemoveCharPc($arrayidx706); //@line 2716 "readclause.cpp"
        ; //@line 2717 "readclause.cpp"
      }
  
      var $350=HEAP32[(($ix)>>2)]; //@line 2718 "readclause.cpp"
      var $351=$buf_addr; //@line 2718 "readclause.cpp"
      var $arrayidx708=(($351+$350)&4294967295); //@line 2718 "readclause.cpp"
      HEAP8[($arrayidx708)]=32; //@line 2718 "readclause.cpp"
      var $352=HEAP32[(($ix)>>2)]; //@line 2719 "readclause.cpp"
      var $add709=((($352)+1)&4294967295); //@line 2719 "readclause.cpp"
      var $353=$buf_addr; //@line 2719 "readclause.cpp"
      var $arrayidx710=(($353+$add709)&4294967295); //@line 2719 "readclause.cpp"
      HEAP8[($arrayidx710)]=0; //@line 2719 "readclause.cpp"
      $retval=589864; //@line 2720 "readclause.cpp"
      ; //@line 2720 "readclause.cpp"
    }
    else if (__label__ == 12) {
  
      HEAP8[(__ZL12end_of_input_b)]=1;
      $retval=589864; //@line 2116 "readclause.cpp"
      ; //@line 2116 "readclause.cpp"
    }
    else if (__label__ == 15) {
  
      HEAP8[(__ZL19clear_skipping_text_b)]=1;
      HEAP32[((_skip_characters)>>2)]=0; //@line 2124 "readclause.cpp"
      var $24=HEAP32[(($c2)>>2)]; //@line 2125 "readclause.cpp"
      __ZL6UngetCi($24); //@line 2125 "readclause.cpp"
      $retval=16384; //@line 2126 "readclause.cpp"
      ; //@line 2126 "readclause.cpp"
    }
    else if (__label__ == 66) {
  
      var $81=$end_clause_after_tag; //@line 2249 "readclause.cpp"
      var $tobool168=((($81))|0)!=0; //@line 2249 "readclause.cpp"
      if ($tobool168) { __label__ = 67;; } else { __label__ = 68;; } //@line 2249 "readclause.cpp"
      if (__label__ == 67) {
  
        var $82=$end_clause_index; //@line 2250 "readclause.cpp"
        HEAP32[(($ix)>>2)]=$82; //@line 2250 "readclause.cpp"
        ; //@line 2250 "readclause.cpp"
      }
  
      var $83=HEAP32[(($ix)>>2)]; //@line 2252 "readclause.cpp"
      var $84=$buf_addr; //@line 2252 "readclause.cpp"
      var $arrayidx171=(($84+$83)&4294967295); //@line 2252 "readclause.cpp"
      HEAP8[($arrayidx171)]=32; //@line 2252 "readclause.cpp"
      var $85=HEAP32[(($ix)>>2)]; //@line 2253 "readclause.cpp"
      var $inc172=((($85)+1)&4294967295); //@line 2253 "readclause.cpp"
      HEAP32[(($ix)>>2)]=$inc172; //@line 2253 "readclause.cpp"
      var $86=$buf_addr; //@line 2253 "readclause.cpp"
      var $arrayidx173=(($86+$85)&4294967295); //@line 2253 "readclause.cpp"
      HEAP8[($arrayidx173)]=0; //@line 2253 "readclause.cpp"
      var $87=$terminator; //@line 2255 "readclause.cpp"
      var $and=($87) & 131072; //@line 2255 "readclause.cpp"
      var $tobool174=((($and))|0)!=0; //@line 2255 "readclause.cpp"
      if ($tobool174) { __label__ = 69;; } else { __label__ = 70;; } //@line 2255 "readclause.cpp"
      if (__label__ == 69) {
  
        var $88=$voice_change_addr; //@line 2257 "readclause.cpp"
        var $call176=_strcpy($88, ((__ZL16current_voice_id)&4294967295)); //@line 2257 "readclause.cpp"
        ; //@line 2258 "readclause.cpp"
      }
  
      var $89=$terminator; //@line 2259 "readclause.cpp"
      $retval=$89; //@line 2259 "readclause.cpp"
      ; //@line 2259 "readclause.cpp"
    }
    else if (__label__ == 74) {
  
      var $call192=__Z11lookupwcharPKti(((__ZL11punct_chars)&4294967295), $93); //@line 2275 "readclause.cpp"
      $punct=$call192; //@line 2275 "readclause.cpp"
      var $cmp193=((($call192))|0)==0; //@line 2275 "readclause.cpp"
      if ($cmp193) { __label__ = 75;; } else { __label__ = 76;; } //@line 2275 "readclause.cpp"
      if (__label__ == 75) {
  
        var $94=HEAP32[((_count_characters)>>2)]; //@line 2277 "readclause.cpp"
        var $95=HEAP32[((_clause_start_char)>>2)]; //@line 2277 "readclause.cpp"
        var $sub195=((($94)-($95))&4294967295); //@line 2277 "readclause.cpp"
        var $conv196=((($sub195)) & 65535); //@line 2277 "readclause.cpp"
        var $96=HEAP32[(($ix)>>2)]; //@line 2277 "readclause.cpp"
        var $97=$charix_addr; //@line 2277 "readclause.cpp"
        var $arrayidx197=(($97+2*$96)&4294967295); //@line 2277 "readclause.cpp"
        HEAP16[(($arrayidx197)>>1)]=$conv196; //@line 2277 "readclause.cpp"
        var $98=HEAP32[(($ix)>>2)]; //@line 2278 "readclause.cpp"
        var $99=$charix_top_addr; //@line 2278 "readclause.cpp"
        HEAP32[(($99)>>2)]=$98; //@line 2278 "readclause.cpp"
        var $100=HEAP32[(($c1)>>2)]; //@line 2279 "readclause.cpp"
        var $101=HEAP32[(($ix)>>2)]; //@line 2279 "readclause.cpp"
        var $102=$buf_addr; //@line 2279 "readclause.cpp"
        var $arrayidx198=(($102+$101)&4294967295); //@line 2279 "readclause.cpp"
        var $call199=__Z8utf8_outjPc($100, $arrayidx198); //@line 2279 "readclause.cpp"
        var $103=HEAP32[(($ix)>>2)]; //@line 2279 "readclause.cpp"
        var $add200=((($103)+($call199))&4294967295); //@line 2279 "readclause.cpp"
        HEAP32[(($ix)>>2)]=$add200; //@line 2279 "readclause.cpp"
        $terminator=524328; //@line 2280 "readclause.cpp"
        ; //@line 2281 "readclause.cpp"
      }
      else if (__label__ == 76) {
  
        var $104=$punct; //@line 2284 "readclause.cpp"
        var $arrayidx202=((__ZL16punct_attributes+$104*4)&4294967295); //@line 2284 "readclause.cpp"
        var $105=HEAP32[(($arrayidx202)>>2)]; //@line 2284 "readclause.cpp"
        $terminator=$105; //@line 2284 "readclause.cpp"
        ;
      }
  
      var $106=HEAP32[(($ix)>>2)]; //@line 2286 "readclause.cpp"
      var $107=$buf_addr; //@line 2286 "readclause.cpp"
      var $arrayidx204=(($107+$106)&4294967295); //@line 2286 "readclause.cpp"
      HEAP8[($arrayidx204)]=32; //@line 2286 "readclause.cpp"
      var $108=HEAP32[(($ix)>>2)]; //@line 2287 "readclause.cpp"
      var $add205=((($108)+1)&4294967295); //@line 2287 "readclause.cpp"
      var $109=$buf_addr; //@line 2287 "readclause.cpp"
      var $arrayidx206=(($109+$add205)&4294967295); //@line 2287 "readclause.cpp"
      HEAP8[($arrayidx206)]=0; //@line 2287 "readclause.cpp"
      var $110=$terminator; //@line 2288 "readclause.cpp"
      $retval=$110; //@line 2288 "readclause.cpp"
      ; //@line 2288 "readclause.cpp"
    }
    else if (__label__ == 81) {
  
      var $114=HEAP32[(($ix)>>2)]; //@line 2296 "readclause.cpp"
      var $inc215=((($114)+1)&4294967295); //@line 2296 "readclause.cpp"
      HEAP32[(($ix)>>2)]=$inc215; //@line 2296 "readclause.cpp"
      var $115=$buf_addr; //@line 2296 "readclause.cpp"
      var $arrayidx216=(($115+$114)&4294967295); //@line 2296 "readclause.cpp"
      HEAP8[($arrayidx216)]=0; //@line 2296 "readclause.cpp"
      ; //@line 2297 "readclause.cpp"
      while(1) { 
  
        var $call218=__ZL4GetCv(); //@line 2297 "readclause.cpp"
        HEAP32[(($c1)>>2)]=$call218; //@line 2297 "readclause.cpp"
        var $call219=_iswspace($call218); //@line 2297 "readclause.cpp"
        var $tobool220=((($call219))|0)!=0; //@line 2297 "readclause.cpp"
        if ($tobool220) { __label__ = 86;break ; } //@line 2297 "readclause.cpp"
  
        var $call222=__Z3Eofv(); //@line 2297 "readclause.cpp"
        var $tobool223=((($call222))|0)!=0; //@line 2297 "readclause.cpp"
        if ($tobool223) { __label__ = 86;break ; } //@line 2297 "readclause.cpp"
  
        var $116=HEAP32[(($ix)>>2)]; //@line 2297 "readclause.cpp"
        var $117=$n_buf_addr; //@line 2297 "readclause.cpp"
        var $sub225=((($117)-1)&4294967295); //@line 2297 "readclause.cpp"
        var $cmp226=((($116))|0) < ((($sub225))|0); //@line 2297 "readclause.cpp"
        if (!($cmp226)) { __label__ = 86;break ; }
  
        var $118=HEAP32[(($c1)>>2)]; //@line 2298 "readclause.cpp"
        var $conv229=((($118)) & 255); //@line 2298 "readclause.cpp"
        var $119=HEAP32[(($ix)>>2)]; //@line 2298 "readclause.cpp"
        var $inc230=((($119)+1)&4294967295); //@line 2298 "readclause.cpp"
        HEAP32[(($ix)>>2)]=$inc230; //@line 2298 "readclause.cpp"
        var $120=$buf_addr; //@line 2298 "readclause.cpp"
        var $arrayidx231=(($120+$119)&4294967295); //@line 2298 "readclause.cpp"
        HEAP8[($arrayidx231)]=$conv229; //@line 2298 "readclause.cpp"
        __label__ = 82;continue ; //@line 2298 "readclause.cpp"
      }
  
      var $121=HEAP32[(($ix)>>2)]; //@line 2299 "readclause.cpp"
      var $inc233=((($121)+1)&4294967295); //@line 2299 "readclause.cpp"
      HEAP32[(($ix)>>2)]=$inc233; //@line 2299 "readclause.cpp"
      var $122=$buf_addr; //@line 2299 "readclause.cpp"
      var $arrayidx234=(($122+$121)&4294967295); //@line 2299 "readclause.cpp"
      HEAP8[($arrayidx234)]=0; //@line 2299 "readclause.cpp"
      $retval=147456; //@line 2300 "readclause.cpp"
      ; //@line 2300 "readclause.cpp"
    }
    else if (__label__ == 108) {
  
      HEAP32[((__ZL10ungot_word)>>2)]=((__str42209)&4294967295); //@line 2370 "readclause.cpp"
      var $151=HEAP32[(($c2)>>2)]; //@line 2371 "readclause.cpp"
      __ZL6UngetCi($151); //@line 2371 "readclause.cpp"
      var $152=$p_word; //@line 2372 "readclause.cpp"
      var $arrayidx300=(($152)&4294967295); //@line 2372 "readclause.cpp"
      HEAP8[($arrayidx300)]=0; //@line 2372 "readclause.cpp"
      $retval=524328; //@line 2373 "readclause.cpp"
      ; //@line 2373 "readclause.cpp"
    }
    else if (__label__ == 138) {
  
      var $184=HEAP32[(($c2)>>2)]; //@line 2438 "readclause.cpp"
      __ZL6UngetCi($184); //@line 2438 "readclause.cpp"
      var $185=$end_clause_after_tag; //@line 2440 "readclause.cpp"
      var $tobool387=((($185))|0)!=0; //@line 2440 "readclause.cpp"
      if ($tobool387) { __label__ = 139;; } else { __label__ = 140;; } //@line 2440 "readclause.cpp"
      if (__label__ == 139) {
  
        var $186=$end_clause_index; //@line 2442 "readclause.cpp"
        var $187=$buf_addr; //@line 2442 "readclause.cpp"
        var $arrayidx389=(($187+$186)&4294967295); //@line 2442 "readclause.cpp"
        __ZL10RemoveCharPc($arrayidx389); //@line 2442 "readclause.cpp"
        ; //@line 2443 "readclause.cpp"
      }
  
      var $188=HEAP32[(($ix)>>2)]; //@line 2444 "readclause.cpp"
      var $189=$buf_addr; //@line 2444 "readclause.cpp"
      var $arrayidx391=(($189+$188)&4294967295); //@line 2444 "readclause.cpp"
      HEAP8[($arrayidx391)]=32; //@line 2444 "readclause.cpp"
      var $190=HEAP32[(($ix)>>2)]; //@line 2445 "readclause.cpp"
      var $add392=((($190)+1)&4294967295); //@line 2445 "readclause.cpp"
      var $191=$buf_addr; //@line 2445 "readclause.cpp"
      var $arrayidx393=(($191+$add392)&4294967295); //@line 2445 "readclause.cpp"
      HEAP8[($arrayidx393)]=0; //@line 2445 "readclause.cpp"
      var $192=$parag; //@line 2446 "readclause.cpp"
      var $cmp394=((($192))|0) > 3; //@line 2446 "readclause.cpp"
      if ($cmp394) { __label__ = 141;; } else { __label__ = 142;; } //@line 2446 "readclause.cpp"
      if (__label__ == 141) {
  
        $parag=3; //@line 2447 "readclause.cpp"
        ; //@line 2447 "readclause.cpp"
      }
  
      var $193=HEAP32[((_option_ssml)>>2)]; //@line 2448 "readclause.cpp"
      var $tobool397=((($193))|0)!=0; //@line 2448 "readclause.cpp"
      if ($tobool397) { __label__ = 143;; } else { __label__ = 144;; } //@line 2448 "readclause.cpp"
      if (__label__ == 143) {
  
        $parag=1; //@line 2448 "readclause.cpp"
        ; //@line 2448 "readclause.cpp"
      }
  
      var $194=$parag; //@line 2449 "readclause.cpp"
      var $mul=((($194)*30)&4294967295); //@line 2449 "readclause.cpp"
      var $add400=((($mul)+524328)&4294967295); //@line 2449 "readclause.cpp"
      $retval=$add400; //@line 2449 "readclause.cpp"
      ; //@line 2449 "readclause.cpp"
    }
    else if (__label__ == 146) {
  
      var $197=HEAP32[(($c2)>>2)]; //@line 2455 "readclause.cpp"
      __ZL6UngetCi($197); //@line 2455 "readclause.cpp"
      var $198=HEAP32[(($ix)>>2)]; //@line 2456 "readclause.cpp"
      var $199=$buf_addr; //@line 2456 "readclause.cpp"
      var $arrayidx404=(($199+$198)&4294967295); //@line 2456 "readclause.cpp"
      HEAP8[($arrayidx404)]=32; //@line 2456 "readclause.cpp"
      var $200=HEAP32[(($ix)>>2)]; //@line 2457 "readclause.cpp"
      var $add405=((($200)+1)&4294967295); //@line 2457 "readclause.cpp"
      var $201=$buf_addr; //@line 2457 "readclause.cpp"
      var $arrayidx406=(($201+$add405)&4294967295); //@line 2457 "readclause.cpp"
      HEAP8[($arrayidx406)]=0; //@line 2457 "readclause.cpp"
      $retval=262174; //@line 2458 "readclause.cpp"
      ; //@line 2458 "readclause.cpp"
    }
    else if (__label__ == 153) {
  
      var $209=HEAP32[(($c2)>>2)]; //@line 2482 "readclause.cpp"
      __ZL6UngetCi($209); //@line 2482 "readclause.cpp"
      var $210=HEAP32[(($c1)>>2)]; //@line 2483 "readclause.cpp"
      HEAP32[((__ZL11ungot_char2)>>2)]=$210; //@line 2483 "readclause.cpp"
      var $211=$end_clause_index; //@line 2484 "readclause.cpp"
      var $212=$buf_addr; //@line 2484 "readclause.cpp"
      var $arrayidx424=(($212+$211)&4294967295); //@line 2484 "readclause.cpp"
      HEAP8[($arrayidx424)]=32; //@line 2484 "readclause.cpp"
      var $213=$end_clause_index; //@line 2485 "readclause.cpp"
      var $add425=((($213)+1)&4294967295); //@line 2485 "readclause.cpp"
      var $214=$buf_addr; //@line 2485 "readclause.cpp"
      var $arrayidx426=(($214+$add425)&4294967295); //@line 2485 "readclause.cpp"
      HEAP8[($arrayidx426)]=0; //@line 2485 "readclause.cpp"
      var $215=$end_clause_after_tag; //@line 2486 "readclause.cpp"
      $retval=$215; //@line 2486 "readclause.cpp"
      ; //@line 2486 "readclause.cpp"
    }
    else if (__label__ == 179) {
  
      var $243=$terminator; //@line 2537 "readclause.cpp"
      $retval=$243; //@line 2537 "readclause.cpp"
      ; //@line 2537 "readclause.cpp"
    }
    else if (__label__ == 227) {
  
      var $293=$c_next; //@line 2641 "readclause.cpp"
      __ZL6UngetCi($293); //@line 2641 "readclause.cpp"
      var $294=HEAP32[(($ix)>>2)]; //@line 2642 "readclause.cpp"
      var $295=$buf_addr; //@line 2642 "readclause.cpp"
      var $arrayidx612=(($295+$294)&4294967295); //@line 2642 "readclause.cpp"
      HEAP8[($arrayidx612)]=32; //@line 2642 "readclause.cpp"
      var $296=HEAP32[(($ix)>>2)]; //@line 2643 "readclause.cpp"
      var $add613=((($296)+1)&4294967295); //@line 2643 "readclause.cpp"
      var $297=$buf_addr; //@line 2643 "readclause.cpp"
      var $arrayidx614=(($297+$add613)&4294967295); //@line 2643 "readclause.cpp"
      HEAP8[($arrayidx614)]=0; //@line 2643 "readclause.cpp"
      var $298=$cprev; //@line 2645 "readclause.cpp"
      var $call615=_iswdigit($298); //@line 2645 "readclause.cpp"
      var $tobool616=((($call615))|0)!=0; //@line 2645 "readclause.cpp"
      if ($tobool616) { __label__ = 228;; } else { __label__ = 230;; } //@line 2645 "readclause.cpp"
      $land_lhs_true617$$if_end622$335: do { 
        if (__label__ == 228) {
  
          var $299=$c_next; //@line 2645 "readclause.cpp"
          var $call618=__Z7IsAlphaj($299); //@line 2645 "readclause.cpp"
          var $tobool619=((($call618))|0)!=0; //@line 2645 "readclause.cpp"
          if ($tobool619) { __label__ = 230;break $land_lhs_true617$$if_end622$335; } //@line 2645 "readclause.cpp"
  
          var $300=$punct_data; //@line 2647 "readclause.cpp"
          var $and621=($300) & -4194305; //@line 2647 "readclause.cpp"
          $punct_data=$and621; //@line 2647 "readclause.cpp"
          ; //@line 2648 "readclause.cpp"
        }
      } while(0);
  
      var $301=$nl_count; //@line 2649 "readclause.cpp"
      var $cmp623=((($301))|0) > 1; //@line 2649 "readclause.cpp"
      var $302=$punct_data; //@line 2651 "readclause.cpp"
      if ($cmp623) { __label__ = 231;; } else { __label__ = 234;; } //@line 2649 "readclause.cpp"
      if (__label__ == 231) {
  
        var $cmp625=((($302))|0)==532520; //@line 2651 "readclause.cpp"
        var $303=$punct_data; //@line 2651 "readclause.cpp"
        var $cmp627=((($303))|0)==536621; //@line 2651 "readclause.cpp"
        var $or_cond18=($cmp625) | ($cmp627); //@line 2651 "readclause.cpp"
        if ($or_cond18) { __label__ = 232;; } else { __label__ = 233;; } //@line 2651 "readclause.cpp"
        if (__label__ == 232) {
  
          var $304=$punct_data; //@line 2652 "readclause.cpp"
          var $add629=((($304)+35)&4294967295); //@line 2652 "readclause.cpp"
          $retval=$add629; //@line 2652 "readclause.cpp"
          ; //@line 2652 "readclause.cpp"
        }
        else if (__label__ == 233) {
  
          $retval=524358; //@line 2653 "readclause.cpp"
          ; //@line 2653 "readclause.cpp"
        }
      }
      else if (__label__ == 234) {
  
        $retval=$302; //@line 2655 "readclause.cpp"
        ; //@line 2655 "readclause.cpp"
      }
    }
    else if (__label__ == 255) {
  
      var $338=HEAP32[(($ix)>>2)]; //@line 2703 "readclause.cpp"
      var $339=$buf_addr; //@line 2703 "readclause.cpp"
      var $arrayidx693=(($339+$338)&4294967295); //@line 2703 "readclause.cpp"
      HEAP8[($arrayidx693)]=32; //@line 2703 "readclause.cpp"
      var $340=HEAP32[(($ix)>>2)]; //@line 2704 "readclause.cpp"
      var $add694=((($340)+1)&4294967295); //@line 2704 "readclause.cpp"
      var $341=$buf_addr; //@line 2704 "readclause.cpp"
      var $arrayidx695=(($341+$add694)&4294967295); //@line 2704 "readclause.cpp"
      HEAP8[($arrayidx695)]=0; //@line 2704 "readclause.cpp"
      var $342=HEAP32[(($c2)>>2)]; //@line 2705 "readclause.cpp"
      __ZL6UngetCi($342); //@line 2705 "readclause.cpp"
      $retval=16384; //@line 2706 "readclause.cpp"
      ; //@line 2706 "readclause.cpp"
    }
  
    var $354=$retval; //@line 2721 "readclause.cpp"
    STACKTOP = __stackBase__;
    return $354; //@line 2721 "readclause.cpp"
    return null;
  }
  

  function __ZL4GetCv() {
    var __stackBase__  = STACKTOP; STACKTOP += 16; _memset(__stackBase__, 0, 16);
    var __label__;
    var __lastLabel__ = null;
  
    var $retval;
    var $c;
    var $c1;
    var $c2;
    var $cbuf=__stackBase__;
    var $ix;
    var $n_bytes;
    var $m;
    var $0=HEAP32[((__ZL10ungot_char)>>2)]; //@line 483 "readclause.cpp"
    $c1=$0; //@line 483 "readclause.cpp"
    var $cmp=((($0))|0)!=0; //@line 483 "readclause.cpp"
    if ($cmp) { __label__ = 1;; } else { __label__ = 2;; } //@line 483 "readclause.cpp"
    $if_then$$if_end$2: do { 
      if (__label__ == 1) {
  
        HEAP32[((__ZL10ungot_char)>>2)]=0; //@line 485 "readclause.cpp"
        var $1=$c1; //@line 486 "readclause.cpp"
        $retval=$1; //@line 486 "readclause.cpp"
        ; //@line 486 "readclause.cpp"
      }
      else if (__label__ == 2) {
  
        var $2=HEAP32[((__ZZL4GetCvE6ungot2)>>2)]; //@line 489 "readclause.cpp"
        var $cmp1=((($2))|0)!=0; //@line 489 "readclause.cpp"
        if ($cmp1) { __label__ = 3;; } else { __label__ = 4;; } //@line 489 "readclause.cpp"
        if (__label__ == 3) {
  
          var $3=HEAP32[((__ZZL4GetCvE6ungot2)>>2)]; //@line 491 "readclause.cpp"
          $c1=$3; //@line 491 "readclause.cpp"
          HEAP32[((__ZZL4GetCvE6ungot2)>>2)]=0; //@line 492 "readclause.cpp"
          ; //@line 493 "readclause.cpp"
        }
        else if (__label__ == 4) {
  
          var $call=__ZL8GetC_getv(); //@line 496 "readclause.cpp"
          $c1=$call; //@line 496 "readclause.cpp"
          ;
        }
  
        var $4=HEAP32[((_option_multibyte)>>2)]; //@line 499 "readclause.cpp"
        var $cmp4=((($4))|0)==3; //@line 499 "readclause.cpp"
        var $5=HEAP32[((_option_multibyte)>>2)]; //@line 499 "readclause.cpp"
        var $cmp5=((($5))|0)==4; //@line 499 "readclause.cpp"
        var $or_cond=($cmp4) | ($cmp5); //@line 499 "readclause.cpp"
        if ($or_cond) { __label__ = 6;; } else { __label__ = 7;; } //@line 499 "readclause.cpp"
        if (__label__ == 6) {
  
          var $6=HEAP32[((_count_characters)>>2)]; //@line 501 "readclause.cpp"
          var $inc=((($6)+1)&4294967295); //@line 501 "readclause.cpp"
          HEAP32[((_count_characters)>>2)]=$inc; //@line 501 "readclause.cpp"
          var $7=$c1; //@line 502 "readclause.cpp"
          $retval=$7; //@line 502 "readclause.cpp"
          ; //@line 502 "readclause.cpp"
        }
        else if (__label__ == 7) {
  
          var $8=HEAP32[((_option_multibyte)>>2)]; //@line 505 "readclause.cpp"
          var $cmp8=((($8))|0) < 2; //@line 505 "readclause.cpp"
          if ($cmp8) { __label__ = 8;; } else { __label__ = 36;; } //@line 505 "readclause.cpp"
          $land_lhs_true$$if_end65thread_pre_split$12: do { 
            if (__label__ == 8) {
  
              var $9=$c1; //@line 505 "readclause.cpp"
              var $and=($9) & 128; //@line 505 "readclause.cpp"
              var $tobool=((($and))|0)!=0; //@line 505 "readclause.cpp"
              if ($tobool) { __lastLabel__ = 8; ; } else { __lastLabel__ = 8; __label__ = 37;break $land_lhs_true$$if_end65thread_pre_split$12; } //@line 505 "readclause.cpp"
  
              $n_bytes=0; //@line 508 "readclause.cpp"
              var $10=$c1; //@line 510 "readclause.cpp"
              var $and10=($10) & 224; //@line 510 "readclause.cpp"
              var $cmp11=((($and10))|0)==192; //@line 510 "readclause.cpp"
              if ($cmp11) { __label__ = 10;; } else { __label__ = 12;; } //@line 510 "readclause.cpp"
              $land_lhs_true12$$if_else16$15: do { 
                if (__label__ == 10) {
  
                  var $11=$c1; //@line 510 "readclause.cpp"
                  var $and13=($11) & 30; //@line 510 "readclause.cpp"
                  var $cmp14=((($and13))|0)!=0; //@line 510 "readclause.cpp"
                  if (!($cmp14)) { __label__ = 12;break $land_lhs_true12$$if_else16$15; } //@line 510 "readclause.cpp"
  
                  $n_bytes=1; //@line 511 "readclause.cpp"
                  __lastLabel__ = 11; __label__ = 17;break $land_lhs_true12$$if_else16$15; //@line 511 "readclause.cpp"
                }
              } while(0);
              $if_end29_thread$$if_else16$18: do { 
                if (__label__ == 12) {
  
                  var $12=$c1; //@line 513 "readclause.cpp"
                  var $and17=($12) & 240; //@line 513 "readclause.cpp"
                  var $cmp18=((($and17))|0)==224; //@line 513 "readclause.cpp"
                  if ($cmp18) { __label__ = 13;; } else { __label__ = 14;; } //@line 513 "readclause.cpp"
                  if (__label__ == 13) {
  
                    $n_bytes=2; //@line 514 "readclause.cpp"
                    __lastLabel__ = 13; __label__ = 17;break $if_end29_thread$$if_else16$18; //@line 514 "readclause.cpp"
                  }
                  else if (__label__ == 14) {
  
                    var $13=$c1; //@line 516 "readclause.cpp"
                    var $and21=($13) & 248; //@line 516 "readclause.cpp"
                    var $cmp22=((($and21))|0)==240; //@line 516 "readclause.cpp"
                    if ($cmp22) { __label__ = 15;; } else { __label__ = 18;; } //@line 516 "readclause.cpp"
                    $land_lhs_true23$$if_end29$23: do { 
                      if (__label__ == 15) {
  
                        var $14=$c1; //@line 516 "readclause.cpp"
                        var $and24=($14) & 15; //@line 516 "readclause.cpp"
                        var $cmp25=((($and24))|0) <= 4; //@line 516 "readclause.cpp"
                        if (!($cmp25)) { __label__ = 18;break $land_lhs_true23$$if_end29$23; } //@line 516 "readclause.cpp"
  
                        $n_bytes=3; //@line 517 "readclause.cpp"
                        __lastLabel__ = 16; __label__ = 17;break $if_end29_thread$$if_else16$18; //@line 517 "readclause.cpp"
                      }
                    } while(0);
  
                    var $_pr=$n_bytes; //@line 519 "readclause.cpp"
                    $ix=$_pr; //@line 519 "readclause.cpp"
                    var $cmp30=((($_pr))|0) > 0; //@line 519 "readclause.cpp"
                    if ($cmp30) { __label__ = 19;break $if_end29_thread$$if_else16$18; } else { __label__ = 33;break $if_end29_thread$$if_else16$18; } //@line 519 "readclause.cpp"
                  }
                }
              } while(0);
              $if_end29_thread$$if_then31$$if_end58$27: do { 
                if (__label__ == 17) {
  
                  var $_ph=__lastLabel__ == 13 ? 2 : (__lastLabel__ == 16 ? 3 : (1));
                  $ix=$_ph; //@line 519 "readclause.cpp"
                  __label__ = 19;break $if_end29_thread$$if_then31$$if_end58$27; //@line 519 "readclause.cpp"
                }
              } while(0);
              $if_then31$$if_end58$29: do { 
                if (__label__ == 19) {
  
                  var $15=$c1; //@line 521 "readclause.cpp"
                  var $16=$ix; //@line 521 "readclause.cpp"
                  var $arrayidx=((__ZZ8utf8_in2PiPKciE4mask+$16)&4294967295); //@line 521 "readclause.cpp"
                  var $17=HEAPU8[($arrayidx)]; //@line 521 "readclause.cpp"
                  var $conv=((($17))&255); //@line 521 "readclause.cpp"
                  var $and32=($conv) & ($15); //@line 521 "readclause.cpp"
                  $c=$and32; //@line 521 "readclause.cpp"
                  var $18=$ix; //@line 522 "readclause.cpp"
                  var $arrayidx33=((__ZZL4GetCvE5mask2+$18)&4294967295); //@line 522 "readclause.cpp"
                  var $19=HEAP8[($arrayidx33)]; //@line 522 "readclause.cpp"
                  $m=$19; //@line 522 "readclause.cpp"
                  __lastLabel__ = 19; ; //@line 523 "readclause.cpp"
                  $while_cond$31: while(1) { 
  
                    var $20=__lastLabel__ == 30 ? $dec : ($18); //@line 523 "readclause.cpp"
                    var $cmp34=((($20))|0) > 0; //@line 523 "readclause.cpp"
                    if (!($cmp34)) { __label__ = 31;break $while_cond$31; } //@line 523 "readclause.cpp"
  
                    var $call35=__ZL8GetC_getv(); //@line 525 "readclause.cpp"
                    var $21=$ix; //@line 525 "readclause.cpp"
                    var $arrayidx36=(($cbuf+$21*4)&4294967295); //@line 525 "readclause.cpp"
                    HEAP32[(($arrayidx36)>>2)]=$call35; //@line 525 "readclause.cpp"
                    $c2=$call35; //@line 525 "readclause.cpp"
                    var $cmp37=((($call35))|0)==0; //@line 525 "readclause.cpp"
                    if ($cmp37) { __label__ = 22;break $while_cond$31; } //@line 525 "readclause.cpp"
  
                    var $23=$c2; //@line 533 "readclause.cpp"
                    var $and43=($23) & 192; //@line 533 "readclause.cpp"
                    var $cmp44=((($and43))|0)!=128; //@line 533 "readclause.cpp"
                    if ($cmp44) { __label__ = 26;break $while_cond$31; } //@line 533 "readclause.cpp"
  
                    $m=-128; //@line 541 "readclause.cpp"
                    var $28=$c; //@line 542 "readclause.cpp"
                    var $shl=((($28)*64)&4294967295); //@line 542 "readclause.cpp"
                    var $29=$c2; //@line 542 "readclause.cpp"
                    var $and53=($29) & 63; //@line 542 "readclause.cpp"
                    var $add=((($and53)+($shl))&4294967295); //@line 542 "readclause.cpp"
                    $c=$add; //@line 542 "readclause.cpp"
                    var $30=$ix; //@line 543 "readclause.cpp"
                    var $dec=((($30)-1)&4294967295); //@line 543 "readclause.cpp"
                    $ix=$dec; //@line 543 "readclause.cpp"
                    __lastLabel__ = 30; __label__ = 20;continue $while_cond$31; //@line 544 "readclause.cpp"
                  }
                  if (__label__ == 22) {
  
                    var $22=HEAP32[((_option_multibyte)>>2)]; //@line 527 "readclause.cpp"
                    var $cmp39=((($22))|0)==0; //@line 527 "readclause.cpp"
                    if ($cmp39) { __label__ = 23;; } else { __label__ = 24;; } //@line 527 "readclause.cpp"
                    if (__label__ == 23) {
  
                      HEAP32[((_option_multibyte)>>2)]=2; //@line 528 "readclause.cpp"
                      ; //@line 528 "readclause.cpp"
                    }
  
                    __ZL10GetC_ungeti(32); //@line 529 "readclause.cpp"
                    ; //@line 530 "readclause.cpp"
                  }
                  else if (__label__ == 26) {
  
                    var $24=$n_bytes; //@line 536 "readclause.cpp"
                    var $cmp46=((($24))|0)==2; //@line 536 "readclause.cpp"
                    if ($cmp46) { __label__ = 27;; } else { __label__ = 29;; } //@line 536 "readclause.cpp"
                    $land_lhs_true47$$if_end51$42: do { 
                      if (__label__ == 27) {
  
                        var $25=$ix; //@line 536 "readclause.cpp"
                        var $cmp48=((($25))|0)==1; //@line 536 "readclause.cpp"
                        if (!($cmp48)) { __label__ = 29;break $land_lhs_true47$$if_end51$42; } //@line 536 "readclause.cpp"
  
                        var $arrayidx50=(($cbuf+8)&4294967295); //@line 537 "readclause.cpp"
                        var $26=HEAP32[(($arrayidx50)>>2)]; //@line 537 "readclause.cpp"
                        HEAP32[((__ZZL4GetCvE6ungot2)>>2)]=$26; //@line 537 "readclause.cpp"
                        ; //@line 537 "readclause.cpp"
                      }
                    } while(0);
  
                    var $27=$c2; //@line 538 "readclause.cpp"
                    __ZL10GetC_ungeti($27); //@line 538 "readclause.cpp"
                    ; //@line 539 "readclause.cpp"
                  }
  
                  var $31=$ix; //@line 545 "readclause.cpp"
                  var $cmp54=((($31))|0)==0; //@line 545 "readclause.cpp"
                  if (!($cmp54)) { __label__ = 33;break $if_then31$$if_end58$29; } //@line 545 "readclause.cpp"
  
                  var $32=HEAP32[((_count_characters)>>2)]; //@line 547 "readclause.cpp"
                  var $inc56=((($32)+1)&4294967295); //@line 547 "readclause.cpp"
                  HEAP32[((_count_characters)>>2)]=$inc56; //@line 547 "readclause.cpp"
                  var $33=$c; //@line 548 "readclause.cpp"
                  $retval=$33; //@line 548 "readclause.cpp"
                  __label__ = 40;break $if_then$$if_end$2; //@line 548 "readclause.cpp"
                }
              } while(0);
  
              var $34=HEAP32[((_option_multibyte)>>2)]; //@line 552 "readclause.cpp"
              var $cmp59=((($34))|0)==0; //@line 552 "readclause.cpp"
              if (!($cmp59)) { __label__ = 36;break $land_lhs_true$$if_end65thread_pre_split$12; } //@line 552 "readclause.cpp"
  
              var $call61=__Z3Eofv(); //@line 552 "readclause.cpp"
              var $tobool62=((($call61))|0)!=0; //@line 552 "readclause.cpp"
              if ($tobool62) { __label__ = 36;break $land_lhs_true$$if_end65thread_pre_split$12; } //@line 552 "readclause.cpp"
  
              HEAP32[((_option_multibyte)>>2)]=2; //@line 553 "readclause.cpp"
              __label__ = 36;break $land_lhs_true$$if_end65thread_pre_split$12; //@line 553 "readclause.cpp"
            }
          } while(0);
          if (__label__ == 36) {
  
            var $_pr2=$c1; //@line 558 "readclause.cpp"
            __lastLabel__ = 36; ;
          }
  
          var $35=__lastLabel__ == 36 ? $_pr2 : ($9); //@line 558 "readclause.cpp"
          var $36=HEAP32[((_count_characters)>>2)]; //@line 557 "readclause.cpp"
          var $inc66=((($36)+1)&4294967295); //@line 557 "readclause.cpp"
          HEAP32[((_count_characters)>>2)]=$inc66; //@line 557 "readclause.cpp"
          var $cmp67=((($35))|0) >= 160; //@line 558 "readclause.cpp"
          var $37=$c1; //@line 559 "readclause.cpp"
          if ($cmp67) { __label__ = 38;; } else { __label__ = 39;; } //@line 558 "readclause.cpp"
          if (__label__ == 38) {
  
            var $sub=((($37)-160)&4294967295); //@line 559 "readclause.cpp"
            var $38=HEAP32[((_translator)>>2)]; //@line 559 "readclause.cpp"
            var $charset_a0=(($38+696)&4294967295); //@line 559 "readclause.cpp"
            var $39=HEAP32[(($charset_a0)>>2)]; //@line 559 "readclause.cpp"
            var $arrayidx69=(($39+2*$sub)&4294967295); //@line 559 "readclause.cpp"
            var $40=HEAPU16[(($arrayidx69)>>1)]; //@line 559 "readclause.cpp"
            var $conv70=((($40))&65535); //@line 559 "readclause.cpp"
            $retval=$conv70; //@line 559 "readclause.cpp"
            ; //@line 559 "readclause.cpp"
          }
          else if (__label__ == 39) {
  
            $retval=$37; //@line 560 "readclause.cpp"
            ; //@line 560 "readclause.cpp"
          }
        }
      }
    } while(0);
  
    var $41=$retval; //@line 561 "readclause.cpp"
    STACKTOP = __stackBase__;
    return $41; //@line 561 "readclause.cpp"
    return null;
  }
  

  function __ZL6UngetCi($c) {
    ;
    var __label__;
  
    var $c_addr;
    $c_addr=$c;
    var $0=$c_addr; //@line 566 "readclause.cpp"
    HEAP32[((__ZL10ungot_char)>>2)]=$0; //@line 566 "readclause.cpp"
    ;
    return; //@line 567 "readclause.cpp"
    return;
  }
  

  function __ZL14ProcessSsmlTagPwPcRiii($xml_buf, $outbuf, $outix, $n_outbuf, $self_closing) {
    var __stackBase__  = STACKTOP; STACKTOP += 376; _memset(__stackBase__, 0, 376);
    var __label__;
    var __lastLabel__ = null;
  
    var $retval;
    var $xml_buf_addr;
    var $outbuf_addr;
    var $outix_addr;
    var $n_outbuf_addr;
    var $self_closing_addr;
    var $ix;
    var $index;
    var $c;
    var $tag_type;
    var $value;
    var $value2;
    var $value3;
    var $voice_change_flag;
    var $px;
    var $attr1;
    var $attr2;
    var $attr3;
    var $terminator;
    var $uri;
    var $param_type;
    var $tag_name=__stackBase__;
    var $buf=__stackBase__+40;
    var $sp;
    var $ssml_sp;
    var $fname=__stackBase__+120;
    $xml_buf_addr=$xml_buf;
    $outbuf_addr=$outbuf;
    $outix_addr=$outix;
    $n_outbuf_addr=$n_outbuf;
    $self_closing_addr=$self_closing;
    $ix=0; //@line 1659 "readclause.cpp"
    __lastLabel__ = 0; ; //@line 1659 "readclause.cpp"
    $for_cond$2: while(1) { 
  
      var $0=__lastLabel__ == 4 ? $inc : (0); //@line 1659 "readclause.cpp"
      var $cmp=((($0))>>>0) < 39; //@line 1659 "readclause.cpp"
      if (!($cmp)) { __label__ = 5;break $for_cond$2; } //@line 1659 "readclause.cpp"
  
      var $1=$ix; //@line 1661 "readclause.cpp"
      var $2=$xml_buf_addr; //@line 1661 "readclause.cpp"
      var $arrayidx=(($2+4*$1)&4294967295); //@line 1661 "readclause.cpp"
      var $3=HEAP32[(($arrayidx)>>2)]; //@line 1661 "readclause.cpp"
      $c=$3; //@line 1661 "readclause.cpp"
      var $cmp1=((($3))|0)==0; //@line 1661 "readclause.cpp"
      if ($cmp1) { __label__ = 5;break $for_cond$2; } //@line 1661 "readclause.cpp"
  
      var $4=$c; //@line 1661 "readclause.cpp"
      var $call=_iswspace($4); //@line 1661 "readclause.cpp"
      var $tobool=((($call))|0)!=0; //@line 1661 "readclause.cpp"
      if ($tobool) { __label__ = 5;break $for_cond$2; } //@line 1661 "readclause.cpp"
  
      var $5=$c; //@line 1663 "readclause.cpp"
      var $conv=((($5)) & 255); //@line 1663 "readclause.cpp"
      var $conv2=(tempInt=(($conv)),(tempInt>=128?tempInt-256:tempInt)); //@line 1663 "readclause.cpp"
      var $call3=_tolower($conv2); //@line 1663 "readclause.cpp"
      var $conv4=((($call3)) & 255); //@line 1663 "readclause.cpp"
      var $6=$ix; //@line 1663 "readclause.cpp"
      var $arrayidx5=(($tag_name+$6)&4294967295); //@line 1663 "readclause.cpp"
      HEAP8[($arrayidx5)]=$conv4; //@line 1663 "readclause.cpp"
      var $7=$ix; //@line 1659 "readclause.cpp"
      var $inc=((($7)+1)&4294967295); //@line 1659 "readclause.cpp"
      $ix=$inc; //@line 1659 "readclause.cpp"
      __lastLabel__ = 4; __label__ = 1;continue $for_cond$2; //@line 1659 "readclause.cpp"
    }
  
    var $8=$ix; //@line 1665 "readclause.cpp"
    var $arrayidx6=(($tag_name+$8)&4294967295); //@line 1665 "readclause.cpp"
    HEAP8[($arrayidx6)]=0; //@line 1665 "readclause.cpp"
    var $9=$ix; //@line 1667 "readclause.cpp"
    var $10=$xml_buf_addr; //@line 1667 "readclause.cpp"
    var $arrayidx7=(($10+4*$9)&4294967295); //@line 1667 "readclause.cpp"
    $px=$arrayidx7; //@line 1667 "readclause.cpp"
    var $arrayidx8=(($tag_name)&4294967295); //@line 1669 "readclause.cpp"
    var $11=HEAP8[($arrayidx8)]; //@line 1669 "readclause.cpp"
    var $conv9=(tempInt=(($11)),(tempInt>=128?tempInt-256:tempInt)); //@line 1669 "readclause.cpp"
    var $cmp10=((($conv9))|0)==47; //@line 1669 "readclause.cpp"
    if ($cmp10) { __label__ = 6;; } else { __label__ = 9;; } //@line 1669 "readclause.cpp"
    $if_then11$$if_else$8: do { 
      if (__label__ == 6) {
  
        var $arrayidx12=(($tag_name+1)&4294967295); //@line 1672 "readclause.cpp"
        var $call13=__Z10LookupMnemP8MNEM_TABPKc(((__ZL8ssmltags)&4294967295), $arrayidx12); //@line 1672 "readclause.cpp"
        $tag_type=$call13; //@line 1672 "readclause.cpp"
        var $cmp14=((($call13))|0)!=16; //@line 1672 "readclause.cpp"
        if ($cmp14) { __label__ = 7;; } else { __label__ = 8;; } //@line 1672 "readclause.cpp"
        if (__label__ == 7) {
  
          var $12=$outix_addr; //@line 1674 "readclause.cpp"
          var $13=HEAP32[(($12)>>2)]; //@line 1674 "readclause.cpp"
          var $inc16=((($13)+1)&4294967295); //@line 1674 "readclause.cpp"
          HEAP32[(($12)>>2)]=$inc16; //@line 1674 "readclause.cpp"
          var $14=$outbuf_addr; //@line 1674 "readclause.cpp"
          var $arrayidx17=(($14+$13)&4294967295); //@line 1674 "readclause.cpp"
          HEAP8[($arrayidx17)]=32; //@line 1674 "readclause.cpp"
          ; //@line 1675 "readclause.cpp"
        }
  
        var $15=$tag_type; //@line 1676 "readclause.cpp"
        var $add=((($15)+32)&4294967295); //@line 1676 "readclause.cpp"
        $tag_type=$add; //@line 1676 "readclause.cpp"
        __lastLabel__ = 8; __label__ = 14;break $if_then11$$if_else$8; //@line 1677 "readclause.cpp"
      }
      else if (__label__ == 9) {
  
        var $arraydecay=(($tag_name)&4294967295); //@line 1680 "readclause.cpp"
        var $call19=__Z10LookupMnemP8MNEM_TABPKc(((__ZL8ssmltags)&4294967295), $arraydecay); //@line 1680 "readclause.cpp"
        $tag_type=$call19; //@line 1680 "readclause.cpp"
        var $cmp20=((($call19))|0)!=16; //@line 1680 "readclause.cpp"
        if ($cmp20) { __label__ = 10;; } else { __label__ = 11;; } //@line 1680 "readclause.cpp"
        if (__label__ == 10) {
  
          var $16=$outix_addr; //@line 1683 "readclause.cpp"
          var $17=HEAP32[(($16)>>2)]; //@line 1683 "readclause.cpp"
          var $inc22=((($17)+1)&4294967295); //@line 1683 "readclause.cpp"
          HEAP32[(($16)>>2)]=$inc22; //@line 1683 "readclause.cpp"
          var $18=$outbuf_addr; //@line 1683 "readclause.cpp"
          var $arrayidx23=(($18+$17)&4294967295); //@line 1683 "readclause.cpp"
          HEAP8[($arrayidx23)]=32; //@line 1683 "readclause.cpp"
          ; //@line 1684 "readclause.cpp"
        }
  
        var $19=$self_closing_addr; //@line 1686 "readclause.cpp"
        var $tobool25=((($19))|0)!=0; //@line 1686 "readclause.cpp"
        var $20=$tag_type; //@line 1686 "readclause.cpp"
        if ($tobool25) { __lastLabel__ = 11; ; } else { __lastLabel__ = 11; __label__ = 14;break $if_then11$$if_else$8; } //@line 1686 "readclause.cpp"
  
        var $arrayidx26=((__ZL22ignore_if_self_closing+$20)&4294967295); //@line 1686 "readclause.cpp"
        var $21=HEAP8[($arrayidx26)]; //@line 1686 "readclause.cpp"
        var $tobool27=(tempInt=(($21)),(tempInt>=128?tempInt-256:tempInt))!=0; //@line 1686 "readclause.cpp"
        if ($tobool27) { __lastLabel__ = 12; ; } else { __lastLabel__ = 12; __label__ = 14;break $if_then11$$if_else$8; } //@line 1686 "readclause.cpp"
  
        $retval=0; //@line 1687 "readclause.cpp"
        __label__ = 113;break $if_then11$$if_else$8; //@line 1687 "readclause.cpp"
      }
    } while(0);
    $if_end30$$return$19: do { 
      if (__label__ == 14) {
  
        var $22=__lastLabel__ == 12 ? $20 : (__lastLabel__ == 8 ? $add : ($20)); //@line 1695 "readclause.cpp"
        $voice_change_flag=0; //@line 1691 "readclause.cpp"
        $terminator=16384; //@line 1692 "readclause.cpp"
        var $23=HEAPU32[((__ZL12n_ssml_stack)>>2)]; //@line 1693 "readclause.cpp"
        var $sub=((($23)-1)&4294967295); //@line 1693 "readclause.cpp"
        var $arrayidx31=((__ZL10ssml_stack+$sub*76)&4294967295); //@line 1693 "readclause.cpp"
        $ssml_sp=$arrayidx31; //@line 1693 "readclause.cpp"
        if ($22 == 10) {
          __lastLabel__ = 14; __label__ = 15;;
        }
        else if ($22 == 3) {
          __lastLabel__ = 14; __label__ = 20;;
        }
        else if ($22 == 12) {
          __lastLabel__ = 14; __label__ = 25;;
        }
        else if ($22 == 42) {
          __lastLabel__ = 14; __label__ = 31;;
        }
        else if ($22 == 35) {
          __lastLabel__ = 14; __label__ = 31;;
        }
        else if ($22 == 44) {
          __lastLabel__ = 14; __label__ = 31;;
        }
        else if ($22 == 4) {
          __lastLabel__ = 14; __label__ = 32;;
        }
        else if ($22 == 36) {
          __lastLabel__ = 14; __label__ = 39;;
        }
        else if ($22 == 9) {
          __lastLabel__ = 14; __label__ = 42;;
        }
        else if ($22 == 14) {
          __lastLabel__ = 14; __label__ = 44;;
        }
        else if ($22 == 41) {
          __lastLabel__ = 14; __label__ = 45;;
        }
        else if ($22 == 46) {
          __lastLabel__ = 14; __label__ = 45;;
        }
        else if ($22 == 5) {
          __lastLabel__ = 14; __label__ = 46;;
        }
        else if ($22 == 11) {
          __lastLabel__ = 14; __label__ = 51;;
        }
        else if ($22 == 43) {
          __lastLabel__ = 14; __label__ = 66;;
        }
        else if ($22 == 13) {
          __lastLabel__ = 14; __label__ = 67;;
        }
        else if ($22 == 1) {
          __lastLabel__ = 14; __label__ = 79;;
        }
        else if ($22 == 2) {
          __lastLabel__ = 14; __label__ = 85;;
        }
        else if ($22 == 33) {
          __lastLabel__ = 14; __label__ = 88;;
        }
        else if ($22 == 34) {
          __lastLabel__ = 14; __label__ = 92;;
        }
        else if ($22 == 15) {
          __lastLabel__ = 14; __label__ = 96;;
        }
        else if ($22 == 47) {
          __lastLabel__ = 14; __label__ = 96;;
        }
        else if ($22 == 6) {
          __lastLabel__ = 14; __label__ = 97;;
        }
        else if ($22 == 7) {
          __lastLabel__ = 14; __label__ = 100;;
        }
        else if ($22 == 38) {
          __lastLabel__ = 14; __label__ = 105;;
        }
        else if ($22 == 39) {
          __lastLabel__ = 14; __label__ = 108;;
        }
        else {
        __lastLabel__ = 14; __label__ = 112;;
        }
        
        $sw_epilog$$sw_bb$$sw_bb49$$sw_bb62$$sw_bb83$$sw_bb84$$sw_bb110$$sw_bb120$$sw_bb129$$sw_bb130$$sw_bb131$$sw_bb156$$sw_bb220$$sw_bb221$$sw_bb249$$sw_bb266$$while_cond$$while_cond280$$sw_bb293$$sw_bb294$$sw_bb302$$sw_bb317$$sw_bb324$21: do { 
          if (__label__ == 15) {
  
            var $24=$tag_type; //@line 1698 "readclause.cpp"
            var $call32=__ZL14PushParamStacki($24); //@line 1698 "readclause.cpp"
            $sp=$call32; //@line 1698 "readclause.cpp"
            var $25=$px; //@line 1699 "readclause.cpp"
            var $call33=__ZL16GetSsmlAttributePwPKc($25, ((__str83250)&4294967295)); //@line 1699 "readclause.cpp"
            $attr1=$call33; //@line 1699 "readclause.cpp"
            var $26=$px; //@line 1700 "readclause.cpp"
            var $call34=__ZL16GetSsmlAttributePwPKc($26, ((__str84251)&4294967295)); //@line 1700 "readclause.cpp"
            $attr2=$call34; //@line 1700 "readclause.cpp"
            var $27=$attr1; //@line 1703 "readclause.cpp"
            var $call35=__ZL7attrcmpPKwPKc($27, ((__str85252)&4294967295)); //@line 1703 "readclause.cpp"
            var $cmp36=((($call35))|0)==0; //@line 1703 "readclause.cpp"
            if ($cmp36) { __label__ = 16;; } else { __label__ = 17;; } //@line 1703 "readclause.cpp"
            $if_then37$$if_else40$23: do { 
              if (__label__ == 16) {
  
                var $28=$attr2; //@line 1705 "readclause.cpp"
                var $call38=__ZL10attrlookupPKwPK8MNEM_TAB($28, ((__ZZL14ProcessSsmlTagPwPcRiiiE10mnem_punct)&4294967295)); //@line 1705 "readclause.cpp"
                $value=$call38; //@line 1705 "readclause.cpp"
                var $29=$value; //@line 1706 "readclause.cpp"
                var $30=$sp; //@line 1706 "readclause.cpp"
                var $parameter=(($30+4)&4294967295); //@line 1706 "readclause.cpp"
                var $arrayidx39=(($parameter+20)&4294967295); //@line 1706 "readclause.cpp"
                HEAP32[(($arrayidx39)>>2)]=$29; //@line 1706 "readclause.cpp"
                ; //@line 1707 "readclause.cpp"
              }
              else if (__label__ == 17) {
  
                var $31=$attr1; //@line 1709 "readclause.cpp"
                var $call41=__ZL7attrcmpPKwPKc($31, ((__str86253)&4294967295)); //@line 1709 "readclause.cpp"
                var $cmp42=((($call41))|0)==0; //@line 1709 "readclause.cpp"
                if (!($cmp42)) { __label__ = 19;break $if_then37$$if_else40$23; } //@line 1709 "readclause.cpp"
  
                var $32=$attr2; //@line 1711 "readclause.cpp"
                var $call44=__ZL10attrlookupPKwPK8MNEM_TAB($32, ((__ZZL14ProcessSsmlTagPwPcRiiiE13mnem_capitals)&4294967295)); //@line 1711 "readclause.cpp"
                $value=$call44; //@line 1711 "readclause.cpp"
                var $33=$value; //@line 1712 "readclause.cpp"
                var $34=$sp; //@line 1712 "readclause.cpp"
                var $parameter45=(($34+4)&4294967295); //@line 1712 "readclause.cpp"
                var $arrayidx46=(($parameter45+24)&4294967295); //@line 1712 "readclause.cpp"
                HEAP32[(($arrayidx46)>>2)]=$33; //@line 1712 "readclause.cpp"
                ; //@line 1713 "readclause.cpp"
              }
            } while(0);
  
            var $35=$outbuf_addr; //@line 1714 "readclause.cpp"
            var $36=$outix_addr; //@line 1714 "readclause.cpp"
            __ZL17ProcessParamStackPcRi($35, $36); //@line 1714 "readclause.cpp"
            ; //@line 1715 "readclause.cpp"
          }
          else if (__label__ == 20) {
  
            var $37=$tag_type; //@line 1718 "readclause.cpp"
            var $call50=__ZL14PushParamStacki($37); //@line 1718 "readclause.cpp"
            $sp=$call50; //@line 1718 "readclause.cpp"
            $param_type=1; //@line 1721 "readclause.cpp"
            ; //@line 1721 "readclause.cpp"
            while(1) { 
  
              var $38=$px; //@line 1723 "readclause.cpp"
              var $39=$param_type; //@line 1723 "readclause.cpp"
              var $arrayidx54=((__ZZL14ProcessSsmlTagPwPcRiiiE12prosody_attr+$39*4)&4294967295); //@line 1723 "readclause.cpp"
              var $40=HEAP32[(($arrayidx54)>>2)]; //@line 1723 "readclause.cpp"
              var $call55=__ZL16GetSsmlAttributePwPKc($38, $40); //@line 1723 "readclause.cpp"
              $attr1=$call55; //@line 1723 "readclause.cpp"
              var $cmp56=((($call55))|0)!=0; //@line 1723 "readclause.cpp"
              if ($cmp56) { __label__ = 22;; } else { __label__ = 23;; } //@line 1723 "readclause.cpp"
              if (__label__ == 22) {
  
                var $41=$param_type; //@line 1725 "readclause.cpp"
                var $42=$attr1; //@line 1725 "readclause.cpp"
                var $43=$sp; //@line 1725 "readclause.cpp"
                __ZL19SetProsodyParameteriPwP11PARAM_STACK($41, $42, $43); //@line 1725 "readclause.cpp"
                ; //@line 1726 "readclause.cpp"
              }
  
              var $44=$param_type; //@line 1721 "readclause.cpp"
              var $inc60=((($44)+1)&4294967295); //@line 1721 "readclause.cpp"
              $param_type=$inc60; //@line 1721 "readclause.cpp"
              var $cmp52=((($inc60))|0) <= 4; //@line 1721 "readclause.cpp"
              if ($cmp52) { __label__ = 21;continue ; } else { __label__ = 24;break ; } //@line 1721 "readclause.cpp"
            }
  
            var $45=$outbuf_addr; //@line 1729 "readclause.cpp"
            var $46=$outix_addr; //@line 1729 "readclause.cpp"
            __ZL17ProcessParamStackPcRi($45, $46); //@line 1729 "readclause.cpp"
            ; //@line 1730 "readclause.cpp"
          }
          else if (__label__ == 25) {
  
            var $47=$tag_type; //@line 1733 "readclause.cpp"
            var $call63=__ZL14PushParamStacki($47); //@line 1733 "readclause.cpp"
            $sp=$call63; //@line 1733 "readclause.cpp"
            $value=3; //@line 1734 "readclause.cpp"
            var $48=$px; //@line 1735 "readclause.cpp"
            var $call64=__ZL16GetSsmlAttributePwPKc($48, ((__str87254)&4294967295)); //@line 1735 "readclause.cpp"
            $attr1=$call64; //@line 1735 "readclause.cpp"
            var $cmp65=((($call64))|0)!=0; //@line 1735 "readclause.cpp"
            if ($cmp65) { __label__ = 26;; } else { __label__ = 27;; } //@line 1735 "readclause.cpp"
            if (__label__ == 26) {
  
              var $49=$attr1; //@line 1737 "readclause.cpp"
              var $call67=__ZL10attrlookupPKwPK8MNEM_TAB($49, ((__ZZL14ProcessSsmlTagPwPcRiiiE13mnem_emphasis)&4294967295)); //@line 1737 "readclause.cpp"
              $value=$call67; //@line 1737 "readclause.cpp"
              ; //@line 1738 "readclause.cpp"
            }
  
            var $50=HEAP32[((_translator)>>2)]; //@line 1740 "readclause.cpp"
            var $langopts=(($50)&4294967295); //@line 1740 "readclause.cpp"
            var $tone_language=(($langopts+224)&4294967295); //@line 1740 "readclause.cpp"
            var $51=HEAP32[(($tone_language)>>2)]; //@line 1740 "readclause.cpp"
            var $cmp69=((($51))|0)==1; //@line 1740 "readclause.cpp"
            var $52=$value; //@line 1745 "readclause.cpp"
            if ($cmp69) { __label__ = 28;; } else { __label__ = 29;; } //@line 1740 "readclause.cpp"
            if (__label__ == 28) {
  
              var $arrayidx71=((__ZZL14ProcessSsmlTagPwPcRiiiE23emphasis_to_pitch_range+$52)&4294967295); //@line 1745 "readclause.cpp"
              var $53=HEAPU8[($arrayidx71)]; //@line 1745 "readclause.cpp"
              var $conv72=((($53))&255); //@line 1745 "readclause.cpp"
              var $54=$sp; //@line 1745 "readclause.cpp"
              var $parameter73=(($54+4)&4294967295); //@line 1745 "readclause.cpp"
              var $arrayidx74=(($parameter73+16)&4294967295); //@line 1745 "readclause.cpp"
              HEAP32[(($arrayidx74)>>2)]=$conv72; //@line 1745 "readclause.cpp"
              var $55=$value; //@line 1746 "readclause.cpp"
              var $arrayidx75=((__ZZL14ProcessSsmlTagPwPcRiiiE18emphasis_to_volume+$55)&4294967295); //@line 1746 "readclause.cpp"
              var $56=HEAPU8[($arrayidx75)]; //@line 1746 "readclause.cpp"
              var $conv76=((($56))&255); //@line 1746 "readclause.cpp"
              var $57=$sp; //@line 1746 "readclause.cpp"
              var $parameter77=(($57+4)&4294967295); //@line 1746 "readclause.cpp"
              var $arrayidx78=(($parameter77+8)&4294967295); //@line 1746 "readclause.cpp"
              HEAP32[(($arrayidx78)>>2)]=$conv76; //@line 1746 "readclause.cpp"
              ; //@line 1747 "readclause.cpp"
            }
            else if (__label__ == 29) {
  
              var $58=$sp; //@line 1750 "readclause.cpp"
              var $parameter80=(($58+4)&4294967295); //@line 1750 "readclause.cpp"
              var $arrayidx81=(($parameter80+48)&4294967295); //@line 1750 "readclause.cpp"
              HEAP32[(($arrayidx81)>>2)]=$52; //@line 1750 "readclause.cpp"
              ;
            }
  
            var $59=$outbuf_addr; //@line 1752 "readclause.cpp"
            var $60=$outix_addr; //@line 1752 "readclause.cpp"
            __ZL17ProcessParamStackPcRi($59, $60); //@line 1752 "readclause.cpp"
            ; //@line 1753 "readclause.cpp"
          }
          else if (__label__ == 31) {
  
            var $61=$tag_type; //@line 1758 "readclause.cpp"
            var $62=$outbuf_addr; //@line 1758 "readclause.cpp"
            var $63=$outix_addr; //@line 1758 "readclause.cpp"
            __ZL13PopParamStackiPcRi($61, $62, $63); //@line 1758 "readclause.cpp"
            ; //@line 1759 "readclause.cpp"
          }
          else if (__label__ == 32) {
  
            var $64=$px; //@line 1762 "readclause.cpp"
            var $call85=__ZL16GetSsmlAttributePwPKc($64, ((__str88255)&4294967295)); //@line 1762 "readclause.cpp"
            $attr1=$call85; //@line 1762 "readclause.cpp"
            var $65=$px; //@line 1763 "readclause.cpp"
            var $call86=__ZL16GetSsmlAttributePwPKc($65, ((__str89256)&4294967295)); //@line 1763 "readclause.cpp"
            $attr2=$call86; //@line 1763 "readclause.cpp"
            var $66=$px; //@line 1764 "readclause.cpp"
            var $call87=__ZL16GetSsmlAttributePwPKc($66, ((__str90257)&4294967295)); //@line 1764 "readclause.cpp"
            $attr3=$call87; //@line 1764 "readclause.cpp"
            var $67=$attr1; //@line 1765 "readclause.cpp"
            var $call88=__ZL10attrlookupPKwPK8MNEM_TAB($67, ((__ZZL14ProcessSsmlTagPwPcRiiiE17mnem_interpret_as)&4294967295)); //@line 1765 "readclause.cpp"
            $value=$call88; //@line 1765 "readclause.cpp"
            var $68=$attr2; //@line 1766 "readclause.cpp"
            var $call89=__ZL10attrlookupPKwPK8MNEM_TAB($68, ((__ZZL14ProcessSsmlTagPwPcRiiiE17mnem_sayas_format)&4294967295)); //@line 1766 "readclause.cpp"
            $value2=$call89; //@line 1766 "readclause.cpp"
            var $69=$value2; //@line 1767 "readclause.cpp"
            var $cmp90=((($69))|0)==1; //@line 1767 "readclause.cpp"
            if ($cmp90) { __label__ = 33;; } else { __label__ = 34;; } //@line 1767 "readclause.cpp"
            if (__label__ == 33) {
  
              $value=19; //@line 1768 "readclause.cpp"
              ; //@line 1768 "readclause.cpp"
            }
  
            var $70=$attr3; //@line 1770 "readclause.cpp"
            var $call93=__ZL10attrnumberPKwii($70, 0, 0); //@line 1770 "readclause.cpp"
            $value3=$call93; //@line 1770 "readclause.cpp"
            var $71=$value; //@line 1772 "readclause.cpp"
            var $cmp94=((($71))|0)==64; //@line 1772 "readclause.cpp"
            if ($cmp94) { __label__ = 35;; } else { __label__ = 38;; } //@line 1772 "readclause.cpp"
            if (__label__ == 35) {
  
              var $72=$value3; //@line 1774 "readclause.cpp"
              var $cmp96=((($72))|0) <= 1; //@line 1774 "readclause.cpp"
              if ($cmp96) { __label__ = 36;; } else { __label__ = 37;; } //@line 1774 "readclause.cpp"
              if (__label__ == 36) {
  
                $value=193; //@line 1775 "readclause.cpp"
                ; //@line 1775 "readclause.cpp"
              }
              else if (__label__ == 37) {
  
                var $73=$value3; //@line 1777 "readclause.cpp"
                var $add99=((($73)+64)&4294967295); //@line 1777 "readclause.cpp"
                $value=$add99; //@line 1777 "readclause.cpp"
                ;
              }
            }
  
            var $arraydecay102=(($buf)&4294967295); //@line 1780 "readclause.cpp"
            var $74=$value; //@line 1780 "readclause.cpp"
            var $call103=_sprintf($arraydecay102, ((__str91258)&4294967295), allocate([1,0,0,0,$74,0,0,0], ["i32",0,0,0,"i32",0,0,0], ALLOC_STACK)); //@line 1780 "readclause.cpp"
            var $75=$outix_addr; //@line 1781 "readclause.cpp"
            var $76=HEAP32[(($75)>>2)]; //@line 1781 "readclause.cpp"
            var $77=$outbuf_addr; //@line 1781 "readclause.cpp"
            var $arrayidx104=(($77+$76)&4294967295); //@line 1781 "readclause.cpp"
            var $arraydecay105=(($buf)&4294967295); //@line 1781 "readclause.cpp"
            var $call106=_strcpy($arrayidx104, $arraydecay105); //@line 1781 "readclause.cpp"
            var $arraydecay107=(($buf)&4294967295); //@line 1782 "readclause.cpp"
            var $call108=_strlen($arraydecay107); //@line 1782 "readclause.cpp"
            var $78=$outix_addr; //@line 1782 "readclause.cpp"
            var $79=HEAP32[(($78)>>2)]; //@line 1782 "readclause.cpp"
            var $add109=((($79)+($call108))&4294967295); //@line 1782 "readclause.cpp"
            HEAP32[(($78)>>2)]=$add109; //@line 1782 "readclause.cpp"
            var $80=$outix_addr; //@line 1784 "readclause.cpp"
            var $81=HEAP32[(($80)>>2)]; //@line 1784 "readclause.cpp"
            HEAP32[((__ZL11sayas_start)>>2)]=$81; //@line 1784 "readclause.cpp"
            var $82=$value; //@line 1785 "readclause.cpp"
            HEAP32[((__ZL10sayas_mode)>>2)]=$82; //@line 1785 "readclause.cpp"
            ; //@line 1786 "readclause.cpp"
          }
          else if (__label__ == 39) {
  
            var $83=HEAP32[((__ZL10sayas_mode)>>2)]; //@line 1789 "readclause.cpp"
            var $cmp111=((($83))|0)==36; //@line 1789 "readclause.cpp"
            if ($cmp111) { __label__ = 40;; } else { __label__ = 41;; } //@line 1789 "readclause.cpp"
            if (__label__ == 40) {
  
              var $84=$outix_addr; //@line 1791 "readclause.cpp"
              var $85=HEAP32[(($84)>>2)]; //@line 1791 "readclause.cpp"
              var $86=$outbuf_addr; //@line 1791 "readclause.cpp"
              var $arrayidx113=(($86+$85)&4294967295); //@line 1791 "readclause.cpp"
              HEAP8[($arrayidx113)]=0; //@line 1791 "readclause.cpp"
              var $87=$outbuf_addr; //@line 1792 "readclause.cpp"
              var $88=HEAP32[((__ZL11sayas_start)>>2)]; //@line 1792 "readclause.cpp"
              var $89=$outix_addr; //@line 1792 "readclause.cpp"
              __ZL14ReplaceKeyNamePciRi($87, $88, $89); //@line 1792 "readclause.cpp"
              ; //@line 1793 "readclause.cpp"
            }
  
            var $90=$outix_addr; //@line 1795 "readclause.cpp"
            var $91=HEAP32[(($90)>>2)]; //@line 1795 "readclause.cpp"
            var $inc116=((($91)+1)&4294967295); //@line 1795 "readclause.cpp"
            HEAP32[(($90)>>2)]=$inc116; //@line 1795 "readclause.cpp"
            var $92=$outbuf_addr; //@line 1795 "readclause.cpp"
            var $arrayidx117=(($92+$91)&4294967295); //@line 1795 "readclause.cpp"
            HEAP8[($arrayidx117)]=1; //@line 1795 "readclause.cpp"
            var $93=$outix_addr; //@line 1796 "readclause.cpp"
            var $94=HEAP32[(($93)>>2)]; //@line 1796 "readclause.cpp"
            var $inc118=((($94)+1)&4294967295); //@line 1796 "readclause.cpp"
            HEAP32[(($93)>>2)]=$inc118; //@line 1796 "readclause.cpp"
            var $95=$outbuf_addr; //@line 1796 "readclause.cpp"
            var $arrayidx119=(($95+$94)&4294967295); //@line 1796 "readclause.cpp"
            HEAP8[($arrayidx119)]=89; //@line 1796 "readclause.cpp"
            HEAP32[((__ZL10sayas_mode)>>2)]=0; //@line 1797 "readclause.cpp"
            ; //@line 1798 "readclause.cpp"
          }
          else if (__label__ == 42) {
  
            var $96=$px; //@line 1801 "readclause.cpp"
            var $call121=__ZL16GetSsmlAttributePwPKc($96, ((__str92259)&4294967295)); //@line 1801 "readclause.cpp"
            $attr1=$call121; //@line 1801 "readclause.cpp"
            var $cmp122=((($call121))|0)!=0; //@line 1801 "readclause.cpp"
            if (!($cmp122)) { __label__ = 112;break $sw_epilog$$sw_bb$$sw_bb49$$sw_bb62$$sw_bb83$$sw_bb84$$sw_bb110$$sw_bb120$$sw_bb129$$sw_bb130$$sw_bb131$$sw_bb156$$sw_bb220$$sw_bb221$$sw_bb249$$sw_bb266$$while_cond$$while_cond280$$sw_bb293$$sw_bb294$$sw_bb302$$sw_bb317$$sw_bb324$21; } //@line 1801 "readclause.cpp"
  
            HEAP8[(__ZL11ignore_text_b)]=1;
            var $97=$outix_addr; //@line 1805 "readclause.cpp"
            var $98=HEAP32[(($97)>>2)]; //@line 1805 "readclause.cpp"
            var $99=$outbuf_addr; //@line 1805 "readclause.cpp"
            var $arrayidx124=(($99+$98)&4294967295); //@line 1805 "readclause.cpp"
            var $100=$attr1; //@line 1805 "readclause.cpp"
            var $101=$n_outbuf_addr; //@line 1805 "readclause.cpp"
            var $102=$outix_addr; //@line 1805 "readclause.cpp"
            var $103=HEAP32[(($102)>>2)]; //@line 1805 "readclause.cpp"
            var $sub125=((($101)-($103))&4294967295); //@line 1805 "readclause.cpp"
            var $call126=__ZL13attrcopy_utf8PcPKwi($arrayidx124, $100, $sub125); //@line 1805 "readclause.cpp"
            var $104=$outix_addr; //@line 1805 "readclause.cpp"
            var $105=HEAP32[(($104)>>2)]; //@line 1805 "readclause.cpp"
            var $add127=((($105)+($call126))&4294967295); //@line 1805 "readclause.cpp"
            HEAP32[(($104)>>2)]=$add127; //@line 1805 "readclause.cpp"
            ; //@line 1806 "readclause.cpp"
          }
          else if (__label__ == 44) {
  
            HEAP8[(__ZL11ignore_text_b)]=1;
            ; //@line 1811 "readclause.cpp"
          }
          else if (__label__ == 45) {
  
            HEAP8[(__ZL11ignore_text_b)]=0;
            ; //@line 1816 "readclause.cpp"
          }
          else if (__label__ == 46) {
  
            var $106=$px; //@line 1819 "readclause.cpp"
            var $call132=__ZL16GetSsmlAttributePwPKc($106, ((__str5655)&4294967295)); //@line 1819 "readclause.cpp"
            $attr1=$call132; //@line 1819 "readclause.cpp"
            var $cmp133=((($call132))|0)!=0; //@line 1819 "readclause.cpp"
            if (!($cmp133)) { __label__ = 112;break $sw_epilog$$sw_bb$$sw_bb49$$sw_bb62$$sw_bb83$$sw_bb84$$sw_bb110$$sw_bb120$$sw_bb129$$sw_bb130$$sw_bb131$$sw_bb156$$sw_bb220$$sw_bb221$$sw_bb249$$sw_bb266$$while_cond$$while_cond280$$sw_bb293$$sw_bb294$$sw_bb302$$sw_bb317$$sw_bb324$21; } //@line 1819 "readclause.cpp"
  
            var $arraydecay135=(($buf)&4294967295); //@line 1822 "readclause.cpp"
            var $107=$attr1; //@line 1822 "readclause.cpp"
            var $call136=__ZL13attrcopy_utf8PcPKwi($arraydecay135, $107, 80); //@line 1822 "readclause.cpp"
            var $arraydecay137=(($buf)&4294967295); //@line 1824 "readclause.cpp"
            var $call138=_strcmp(((_skip_marker)&4294967295), $arraydecay137); //@line 1824 "readclause.cpp"
            var $cmp139=((($call138))|0)==0; //@line 1824 "readclause.cpp"
            if ($cmp139) { __label__ = 48;; } else { __label__ = 49;; } //@line 1824 "readclause.cpp"
            if (__label__ == 48) {
  
              HEAP8[(__ZL19clear_skipping_text_b)]=1;
              HEAP8[(((_skip_marker)&4294967295))]=0; //@line 1828 "readclause.cpp"
              $retval=16384; //@line 1829 "readclause.cpp"
              __label__ = 113;break $if_end30$$return$19; //@line 1829 "readclause.cpp"
            }
            else if (__label__ == 49) {
  
              var $arraydecay142=(($buf)&4294967295); //@line 1832 "readclause.cpp"
              var $call143=__Z11AddNameDataPKci($arraydecay142, 0); //@line 1832 "readclause.cpp"
              $index=$call143; //@line 1832 "readclause.cpp"
              var $cmp144=((($call143))|0) >= 0; //@line 1832 "readclause.cpp"
              if (!($cmp144)) { __label__ = 112;break $sw_epilog$$sw_bb$$sw_bb49$$sw_bb62$$sw_bb83$$sw_bb84$$sw_bb110$$sw_bb120$$sw_bb129$$sw_bb130$$sw_bb131$$sw_bb156$$sw_bb220$$sw_bb221$$sw_bb249$$sw_bb266$$while_cond$$while_cond280$$sw_bb293$$sw_bb294$$sw_bb302$$sw_bb317$$sw_bb324$21; } //@line 1832 "readclause.cpp"
  
              var $arraydecay146=(($buf)&4294967295); //@line 1834 "readclause.cpp"
              var $108=$index; //@line 1834 "readclause.cpp"
              var $call147=_sprintf($arraydecay146, ((__str94261)&4294967295), allocate([1,0,0,0,$108,0,0,0], ["i32",0,0,0,"i32",0,0,0], ALLOC_STACK)); //@line 1834 "readclause.cpp"
              var $109=$outix_addr; //@line 1835 "readclause.cpp"
              var $110=HEAP32[(($109)>>2)]; //@line 1835 "readclause.cpp"
              var $111=$outbuf_addr; //@line 1835 "readclause.cpp"
              var $arrayidx148=(($111+$110)&4294967295); //@line 1835 "readclause.cpp"
              var $arraydecay149=(($buf)&4294967295); //@line 1835 "readclause.cpp"
              var $call150=_strcpy($arrayidx148, $arraydecay149); //@line 1835 "readclause.cpp"
              var $arraydecay151=(($buf)&4294967295); //@line 1836 "readclause.cpp"
              var $call152=_strlen($arraydecay151); //@line 1836 "readclause.cpp"
              var $112=$outix_addr; //@line 1836 "readclause.cpp"
              var $113=HEAP32[(($112)>>2)]; //@line 1836 "readclause.cpp"
              var $add153=((($113)+($call152))&4294967295); //@line 1836 "readclause.cpp"
              HEAP32[(($112)>>2)]=$add153; //@line 1836 "readclause.cpp"
              ; //@line 1837 "readclause.cpp"
            }
          }
          else if (__label__ == 51) {
  
            var $114=$tag_type; //@line 1842 "readclause.cpp"
            var $call157=__ZL14PushParamStacki($114); //@line 1842 "readclause.cpp"
            $sp=$call157; //@line 1842 "readclause.cpp"
            var $115=$px; //@line 1844 "readclause.cpp"
            var $call158=__ZL16GetSsmlAttributePwPKc($115, ((__str95262)&4294967295)); //@line 1844 "readclause.cpp"
            $attr1=$call158; //@line 1844 "readclause.cpp"
            var $cmp159=((($call158))|0)!=0; //@line 1844 "readclause.cpp"
            if ($cmp159) { __label__ = 52;; } else { __label__ = 62;; } //@line 1844 "readclause.cpp"
            $if_then160$$if_end215$69: do { 
              if (__label__ == 52) {
  
                var $arraydecay161=(($buf)&4294967295); //@line 1847 "readclause.cpp"
                var $116=$attr1; //@line 1847 "readclause.cpp"
                var $call162=__ZL13attrcopy_utf8PcPKwi($arraydecay161, $116, 80); //@line 1847 "readclause.cpp"
                var $117=HEAP32[((_uri_callback)>>2)]; //@line 1849 "readclause.cpp"
                var $cmp163=((($117))|0)==0; //@line 1849 "readclause.cpp"
                if ($cmp163) { __label__ = 53;; } else { __label__ = 59;; } //@line 1849 "readclause.cpp"
                if (__label__ == 53) {
  
                  var $118=HEAP32[((__ZL7xmlbase)>>2)]; //@line 1851 "readclause.cpp"
                  var $cmp165=((($118))|0)!=0; //@line 1851 "readclause.cpp"
                  if ($cmp165) { __label__ = 54;; } else { __label__ = 56;; } //@line 1851 "readclause.cpp"
                  $land_lhs_true166$$if_else176$73: do { 
                    if (__label__ == 54) {
  
                      var $arrayidx167=(($buf)&4294967295); //@line 1851 "readclause.cpp"
                      var $119=HEAP8[($arrayidx167)]; //@line 1851 "readclause.cpp"
                      var $conv168=(tempInt=(($119)),(tempInt>=128?tempInt-256:tempInt)); //@line 1851 "readclause.cpp"
                      var $cmp169=((($conv168))|0)!=47; //@line 1851 "readclause.cpp"
                      if (!($cmp169)) { __label__ = 56;break $land_lhs_true166$$if_else176$73; } //@line 1851 "readclause.cpp"
  
                      var $arraydecay171=(($fname)&4294967295); //@line 1853 "readclause.cpp"
                      var $120=HEAP32[((__ZL7xmlbase)>>2)]; //@line 1853 "readclause.cpp"
                      var $arraydecay172=(($buf)&4294967295); //@line 1853 "readclause.cpp"
                      var $call173=_sprintf($arraydecay171, ((__str96263)&4294967295), allocate([$120,0,0,0,$arraydecay172,0,0,0], ["i8*",0,0,0,"i8*",0,0,0], ALLOC_STACK)); //@line 1853 "readclause.cpp"
                      var $arraydecay174=(($fname)&4294967295); //@line 1854 "readclause.cpp"
                      var $call175=__ZL14LoadSoundFile2PKc($arraydecay174); //@line 1854 "readclause.cpp"
                      $index=$call175; //@line 1854 "readclause.cpp"
                      __lastLabel__ = 55; __label__ = 57;break $land_lhs_true166$$if_else176$73; //@line 1855 "readclause.cpp"
                    }
                  } while(0);
                  if (__label__ == 56) {
  
                    var $arraydecay177=(($buf)&4294967295); //@line 1858 "readclause.cpp"
                    var $call178=__ZL14LoadSoundFile2PKc($arraydecay177); //@line 1858 "readclause.cpp"
                    $index=$call178; //@line 1858 "readclause.cpp"
                    __lastLabel__ = 56; ;
                  }
  
                  var $121=__lastLabel__ == 56 ? $call178 : ($call175); //@line 1860 "readclause.cpp"
                  var $cmp180=((($121))|0) >= 0; //@line 1860 "readclause.cpp"
                  if (!($cmp180)) { __label__ = 62;break $if_then160$$if_end215$69; } //@line 1860 "readclause.cpp"
  
                  var $arraydecay182=(($buf)&4294967295); //@line 1862 "readclause.cpp"
                  var $122=$index; //@line 1862 "readclause.cpp"
                  var $call183=_sprintf($arraydecay182, ((__str97264)&4294967295), allocate([1,0,0,0,$122,0,0,0], ["i32",0,0,0,"i32",0,0,0], ALLOC_STACK)); //@line 1862 "readclause.cpp"
                  var $123=$outix_addr; //@line 1863 "readclause.cpp"
                  var $124=HEAP32[(($123)>>2)]; //@line 1863 "readclause.cpp"
                  var $125=$outbuf_addr; //@line 1863 "readclause.cpp"
                  var $arrayidx184=(($125+$124)&4294967295); //@line 1863 "readclause.cpp"
                  var $arraydecay185=(($buf)&4294967295); //@line 1863 "readclause.cpp"
                  var $call186=_strcpy($arrayidx184, $arraydecay185); //@line 1863 "readclause.cpp"
                  var $arraydecay187=(($buf)&4294967295); //@line 1864 "readclause.cpp"
                  var $call188=_strlen($arraydecay187); //@line 1864 "readclause.cpp"
                  var $126=$outix_addr; //@line 1864 "readclause.cpp"
                  var $127=HEAP32[(($126)>>2)]; //@line 1864 "readclause.cpp"
                  var $add189=((($127)+($call188))&4294967295); //@line 1864 "readclause.cpp"
                  HEAP32[(($126)>>2)]=$add189; //@line 1864 "readclause.cpp"
                  var $128=$sp; //@line 1865 "readclause.cpp"
                  var $parameter190=(($128+4)&4294967295); //@line 1865 "readclause.cpp"
                  var $arrayidx191=(($parameter190)&4294967295); //@line 1865 "readclause.cpp"
                  HEAP32[(($arrayidx191)>>2)]=1; //@line 1865 "readclause.cpp"
                  ; //@line 1866 "readclause.cpp"
                }
                else if (__label__ == 59) {
  
                  var $arraydecay194=(($buf)&4294967295); //@line 1870 "readclause.cpp"
                  var $call195=__Z11AddNameDataPKci($arraydecay194, 0); //@line 1870 "readclause.cpp"
                  $index=$call195; //@line 1870 "readclause.cpp"
                  var $cmp196=((($call195))|0) >= 0; //@line 1870 "readclause.cpp"
                  if (!($cmp196)) { __label__ = 62;break $if_then160$$if_end215$69; } //@line 1870 "readclause.cpp"
  
                  var $129=$index; //@line 1872 "readclause.cpp"
                  var $130=HEAP32[((_namedata)>>2)]; //@line 1872 "readclause.cpp"
                  var $arrayidx198=(($130+$129)&4294967295); //@line 1872 "readclause.cpp"
                  $uri=$arrayidx198; //@line 1872 "readclause.cpp"
                  var $131=HEAP32[((_uri_callback)>>2)]; //@line 1873 "readclause.cpp"
                  var $132=$uri; //@line 1873 "readclause.cpp"
                  var $133=HEAP32[((__ZL7xmlbase)>>2)]; //@line 1873 "readclause.cpp"
                  var $call199=FUNCTION_TABLE[$131](1, $132, $133); //@line 1873 "readclause.cpp"
                  var $cmp200=((($call199))|0)==0; //@line 1873 "readclause.cpp"
                  if (!($cmp200)) { __label__ = 62;break $if_then160$$if_end215$69; } //@line 1873 "readclause.cpp"
  
                  var $arraydecay202=(($buf)&4294967295); //@line 1875 "readclause.cpp"
                  var $134=$index; //@line 1875 "readclause.cpp"
                  var $call203=_sprintf($arraydecay202, ((__str98265)&4294967295), allocate([1,0,0,0,$134,0,0,0], ["i32",0,0,0,"i32",0,0,0], ALLOC_STACK)); //@line 1875 "readclause.cpp"
                  var $135=$outix_addr; //@line 1876 "readclause.cpp"
                  var $136=HEAP32[(($135)>>2)]; //@line 1876 "readclause.cpp"
                  var $137=$outbuf_addr; //@line 1876 "readclause.cpp"
                  var $arrayidx204=(($137+$136)&4294967295); //@line 1876 "readclause.cpp"
                  var $arraydecay205=(($buf)&4294967295); //@line 1876 "readclause.cpp"
                  var $call206=_strcpy($arrayidx204, $arraydecay205); //@line 1876 "readclause.cpp"
                  var $arraydecay207=(($buf)&4294967295); //@line 1877 "readclause.cpp"
                  var $call208=_strlen($arraydecay207); //@line 1877 "readclause.cpp"
                  var $138=$outix_addr; //@line 1877 "readclause.cpp"
                  var $139=HEAP32[(($138)>>2)]; //@line 1877 "readclause.cpp"
                  var $add209=((($139)+($call208))&4294967295); //@line 1877 "readclause.cpp"
                  HEAP32[(($138)>>2)]=$add209; //@line 1877 "readclause.cpp"
                  var $140=$sp; //@line 1878 "readclause.cpp"
                  var $parameter210=(($140+4)&4294967295); //@line 1878 "readclause.cpp"
                  var $arrayidx211=(($parameter210)&4294967295); //@line 1878 "readclause.cpp"
                  HEAP32[(($arrayidx211)>>2)]=1; //@line 1878 "readclause.cpp"
                  ; //@line 1879 "readclause.cpp"
                }
              }
            } while(0);
  
            var $141=$outbuf_addr; //@line 1883 "readclause.cpp"
            var $142=$outix_addr; //@line 1883 "readclause.cpp"
            __ZL17ProcessParamStackPcRi($141, $142); //@line 1883 "readclause.cpp"
            var $143=$self_closing_addr; //@line 1885 "readclause.cpp"
            var $tobool216=((($143))|0)!=0; //@line 1885 "readclause.cpp"
            if ($tobool216) { __label__ = 63;; } else { __label__ = 64;; } //@line 1885 "readclause.cpp"
            if (__label__ == 63) {
  
              var $144=$tag_type; //@line 1886 "readclause.cpp"
              var $145=$outbuf_addr; //@line 1886 "readclause.cpp"
              var $146=$outix_addr; //@line 1886 "readclause.cpp"
              __ZL13PopParamStackiPcRi($144, $145, $146); //@line 1886 "readclause.cpp"
              ; //@line 1886 "readclause.cpp"
            }
            else if (__label__ == 64) {
  
              HEAP8[(__ZL10audio_text_b)]=1;
              ;
            }
  
            $retval=16384; //@line 1889 "readclause.cpp"
            __label__ = 113;break $if_end30$$return$19; //@line 1889 "readclause.cpp"
          }
          else if (__label__ == 66) {
  
            var $147=$tag_type; //@line 1892 "readclause.cpp"
            var $148=$outbuf_addr; //@line 1892 "readclause.cpp"
            var $149=$outix_addr; //@line 1892 "readclause.cpp"
            __ZL13PopParamStackiPcRi($147, $148, $149); //@line 1892 "readclause.cpp"
            HEAP8[(__ZL10audio_text_b)]=0;
            $retval=16384; //@line 1894 "readclause.cpp"
            __label__ = 113;break $if_end30$$return$19; //@line 1894 "readclause.cpp"
          }
          else if (__label__ == 67) {
  
            $value=21; //@line 1897 "readclause.cpp"
            $terminator=16384; //@line 1898 "readclause.cpp"
            var $150=$px; //@line 1900 "readclause.cpp"
            var $call222=__ZL16GetSsmlAttributePwPKc($150, ((__str99266)&4294967295)); //@line 1900 "readclause.cpp"
            $attr1=$call222; //@line 1900 "readclause.cpp"
            var $cmp223=((($call222))|0)!=0; //@line 1900 "readclause.cpp"
            if ($cmp223) { __label__ = 68;; } else { __label__ = 71;; } //@line 1900 "readclause.cpp"
            if (__label__ == 68) {
  
              var $151=$attr1; //@line 1903 "readclause.cpp"
              var $call225=__ZL10attrlookupPKwPK8MNEM_TAB($151, ((__ZZL14ProcessSsmlTagPwPcRiiiE10mnem_break)&4294967295)); //@line 1903 "readclause.cpp"
              $value=$call225; //@line 1903 "readclause.cpp"
              var $152=$value; //@line 1904 "readclause.cpp"
              var $cmp226=((($152))|0) < 3; //@line 1904 "readclause.cpp"
              if ($cmp226) { __label__ = 69;; } else { __label__ = 70;; } //@line 1904 "readclause.cpp"
              if (__label__ == 69) {
  
                var $153=$outix_addr; //@line 1907 "readclause.cpp"
                var $154=HEAP32[(($153)>>2)]; //@line 1907 "readclause.cpp"
                var $155=$outbuf_addr; //@line 1907 "readclause.cpp"
                var $arrayidx228=(($155+$154)&4294967295); //@line 1907 "readclause.cpp"
                var $156=$value; //@line 1907 "readclause.cpp"
                var $call229=_sprintf($arrayidx228, ((__str100267)&4294967295), allocate([1,0,0,0,$156,0,0,0], ["i32",0,0,0,"i32",0,0,0], ALLOC_STACK)); //@line 1907 "readclause.cpp"
                var $157=$outix_addr; //@line 1908 "readclause.cpp"
                var $158=HEAP32[(($157)>>2)]; //@line 1908 "readclause.cpp"
                var $add230=((($158)+3)&4294967295); //@line 1908 "readclause.cpp"
                HEAP32[(($157)>>2)]=$add230; //@line 1908 "readclause.cpp"
                $terminator=0; //@line 1909 "readclause.cpp"
                ; //@line 1910 "readclause.cpp"
              }
  
              var $159=$value; //@line 1911 "readclause.cpp"
              var $arrayidx232=((__ZZL14ProcessSsmlTagPwPcRiiiE11break_value+$159*4)&4294967295); //@line 1911 "readclause.cpp"
              var $160=HEAP32[(($arrayidx232)>>2)]; //@line 1911 "readclause.cpp"
              $value=$160; //@line 1911 "readclause.cpp"
              ; //@line 1912 "readclause.cpp"
            }
  
            var $161=$px; //@line 1913 "readclause.cpp"
            var $call234=__ZL16GetSsmlAttributePwPKc($161, ((__str101268)&4294967295)); //@line 1913 "readclause.cpp"
            $attr2=$call234; //@line 1913 "readclause.cpp"
            var $cmp235=((($call234))|0)!=0; //@line 1913 "readclause.cpp"
            if ($cmp235) { __label__ = 72;; } else { __label__ = 74;; } //@line 1913 "readclause.cpp"
            $if_then236$$if_end241$96: do { 
              if (__label__ == 72) {
  
                var $162=$attr2; //@line 1915 "readclause.cpp"
                var $call237=__ZL10attrnumberPKwii($162, 0, 1); //@line 1915 "readclause.cpp"
                var $mul=((($call237)*25)&4294967295); //@line 1915 "readclause.cpp"
                var $163=HEAP32[((((_speed)&4294967295))>>2)]; //@line 1915 "readclause.cpp"
                var $div=((((($mul))|0)/((($163))|0))|0); //@line 1915 "readclause.cpp"
                $value=$div; //@line 1915 "readclause.cpp"
                var $164=$terminator; //@line 1917 "readclause.cpp"
                var $cmp238=((($164))|0)==0; //@line 1917 "readclause.cpp"
                if ($cmp238) { __lastLabel__ = 72; ; } else { __lastLabel__ = 72; __label__ = 76;break $if_then236$$if_end241$96; } //@line 1917 "readclause.cpp"
  
                $terminator=16384; //@line 1918 "readclause.cpp"
                __label__ = 75;break $if_then236$$if_end241$96; //@line 1918 "readclause.cpp"
              }
              else if (__label__ == 74) {
  
                var $_pr1=$terminator; //@line 1920 "readclause.cpp"
                var $tobool242=((($_pr1))|0)!=0; //@line 1920 "readclause.cpp"
                if ($tobool242) { __label__ = 75;break $if_then236$$if_end241$96; } else { __label__ = 112;break $sw_epilog$$sw_bb$$sw_bb49$$sw_bb62$$sw_bb83$$sw_bb84$$sw_bb110$$sw_bb120$$sw_bb129$$sw_bb130$$sw_bb131$$sw_bb156$$sw_bb220$$sw_bb221$$sw_bb249$$sw_bb266$$while_cond$$while_cond280$$sw_bb293$$sw_bb294$$sw_bb302$$sw_bb317$$sw_bb324$21; } //@line 1920 "readclause.cpp"
              }
            } while(0);
            if (__label__ == 75) {
  
              var $_pr3=$value; //@line 1922 "readclause.cpp"
              __lastLabel__ = 75; ;
            }
  
            var $165=__lastLabel__ == 75 ? $_pr3 : ($div); //@line 1922 "readclause.cpp"
            var $cmp244=((($165))|0) > 4095; //@line 1922 "readclause.cpp"
            if ($cmp244) { __label__ = 77;; } else { __label__ = 78;; } //@line 1922 "readclause.cpp"
            if (__label__ == 77) {
  
              $value=4095; //@line 1923 "readclause.cpp"
              ; //@line 1923 "readclause.cpp"
            }
  
            var $166=$terminator; //@line 1924 "readclause.cpp"
            var $167=$value; //@line 1924 "readclause.cpp"
            var $add247=((($167)+($166))&4294967295); //@line 1924 "readclause.cpp"
            $retval=$add247; //@line 1924 "readclause.cpp"
            __label__ = 113;break $if_end30$$return$19; //@line 1924 "readclause.cpp"
          }
          else if (__label__ == 79) {
  
            var $168=$px; //@line 1929 "readclause.cpp"
            var $call250=__ZL16GetSsmlAttributePwPKc($168, ((__str102269)&4294967295)); //@line 1929 "readclause.cpp"
            $attr1=$call250; //@line 1929 "readclause.cpp"
            var $cmp251=((($call250))|0)!=0; //@line 1929 "readclause.cpp"
            if ($cmp251) { __label__ = 80;; } else { __label__ = 82;; } //@line 1929 "readclause.cpp"
            $if_then252$$if_end261$107: do { 
              if (__label__ == 80) {
  
                var $arraydecay253=(($buf)&4294967295); //@line 1931 "readclause.cpp"
                var $169=$attr1; //@line 1931 "readclause.cpp"
                var $call254=__ZL13attrcopy_utf8PcPKwi($arraydecay253, $169, 80); //@line 1931 "readclause.cpp"
                var $arraydecay255=(($buf)&4294967295); //@line 1932 "readclause.cpp"
                var $call256=__Z11AddNameDataPKci($arraydecay255, 0); //@line 1932 "readclause.cpp"
                $index=$call256; //@line 1932 "readclause.cpp"
                var $cmp257=((($call256))|0) >= 0; //@line 1932 "readclause.cpp"
                if (!($cmp257)) { __label__ = 82;break $if_then252$$if_end261$107; } //@line 1932 "readclause.cpp"
  
                var $170=$index; //@line 1934 "readclause.cpp"
                var $171=HEAP32[((_namedata)>>2)]; //@line 1934 "readclause.cpp"
                var $arrayidx259=(($171+$170)&4294967295); //@line 1934 "readclause.cpp"
                HEAP32[((__ZL7xmlbase)>>2)]=$arrayidx259; //@line 1934 "readclause.cpp"
                ; //@line 1935 "readclause.cpp"
              }
            } while(0);
  
            var $172=$px; //@line 1937 "readclause.cpp"
            var $173=$tag_type; //@line 1937 "readclause.cpp"
            var $call262=__ZL18GetVoiceAttributesPwi($172, $173); //@line 1937 "readclause.cpp"
            var $cmp263=((($call262))|0)==0; //@line 1937 "readclause.cpp"
            if ($cmp263) { __label__ = 83;; } else { __label__ = 84;; } //@line 1937 "readclause.cpp"
            if (__label__ == 83) {
  
              $retval=0; //@line 1938 "readclause.cpp"
              __label__ = 113;break $if_end30$$return$19; //@line 1938 "readclause.cpp"
            }
            else if (__label__ == 84) {
  
              $retval=147456; //@line 1939 "readclause.cpp"
              __label__ = 113;break $if_end30$$return$19; //@line 1939 "readclause.cpp"
            }
          }
          else if (__label__ == 85) {
  
            var $174=$px; //@line 1942 "readclause.cpp"
            var $175=$tag_type; //@line 1942 "readclause.cpp"
            var $call267=__ZL18GetVoiceAttributesPwi($174, $175); //@line 1942 "readclause.cpp"
            var $cmp268=((($call267))|0)==0; //@line 1942 "readclause.cpp"
            if ($cmp268) { __label__ = 86;; } else { __label__ = 87;; } //@line 1942 "readclause.cpp"
            if (__label__ == 86) {
  
              $retval=0; //@line 1943 "readclause.cpp"
              __label__ = 113;break $if_end30$$return$19; //@line 1943 "readclause.cpp"
            }
            else if (__label__ == 87) {
  
              $retval=147456; //@line 1944 "readclause.cpp"
              __label__ = 113;break $if_end30$$return$19; //@line 1944 "readclause.cpp"
            }
          }
          else if (__label__ == 88) {
            while(1) { 
  
              var $176=__lastLabel__ == 90 ? $dec : ($23); //@line 1948 "readclause.cpp"
              var $cmp272=((($176))|0) > 1; //@line 1948 "readclause.cpp"
              if (!($cmp272)) { __label__ = 91;break ; } //@line 1948 "readclause.cpp"
  
              var $177=HEAP32[((__ZL12n_ssml_stack)>>2)]; //@line 1948 "readclause.cpp"
              var $sub273=((($177)-1)&4294967295); //@line 1948 "readclause.cpp"
              var $arrayidx274=((__ZL10ssml_stack+$sub273*76)&4294967295); //@line 1948 "readclause.cpp"
              var $tag_type275=(($arrayidx274)&4294967295); //@line 1948 "readclause.cpp"
              var $178=HEAP32[(($tag_type275)>>2)]; //@line 1948 "readclause.cpp"
              var $cmp276=((($178))|0)!=1; //@line 1948 "readclause.cpp"
              if (!($cmp276)) { __label__ = 91;break ; }
  
              var $179=HEAP32[((__ZL12n_ssml_stack)>>2)]; //@line 1950 "readclause.cpp"
              var $dec=((($179)-1)&4294967295); //@line 1950 "readclause.cpp"
              HEAP32[((__ZL12n_ssml_stack)>>2)]=$dec; //@line 1950 "readclause.cpp"
              __lastLabel__ = 90; __label__ = 88;continue ; //@line 1951 "readclause.cpp"
            }
  
            var $180=$px; //@line 1952 "readclause.cpp"
            var $181=$tag_type; //@line 1952 "readclause.cpp"
            var $call277=__ZL18GetVoiceAttributesPwi($180, $181); //@line 1952 "readclause.cpp"
            var $add278=((($call277)+524328)&4294967295); //@line 1952 "readclause.cpp"
            $retval=$add278; //@line 1952 "readclause.cpp"
            __label__ = 113;break $if_end30$$return$19; //@line 1952 "readclause.cpp"
          }
          else if (__label__ == 92) {
            while(1) { 
  
              var $182=__lastLabel__ == 94 ? $dec289 : ($23); //@line 1956 "readclause.cpp"
              var $cmp281=((($182))|0) > 1; //@line 1956 "readclause.cpp"
              if (!($cmp281)) { __label__ = 95;break ; } //@line 1956 "readclause.cpp"
  
              var $183=HEAP32[((__ZL12n_ssml_stack)>>2)]; //@line 1956 "readclause.cpp"
              var $sub283=((($183)-1)&4294967295); //@line 1956 "readclause.cpp"
              var $arrayidx284=((__ZL10ssml_stack+$sub283*76)&4294967295); //@line 1956 "readclause.cpp"
              var $tag_type285=(($arrayidx284)&4294967295); //@line 1956 "readclause.cpp"
              var $184=HEAP32[(($tag_type285)>>2)]; //@line 1956 "readclause.cpp"
              var $cmp286=((($184))|0)!=2; //@line 1956 "readclause.cpp"
              if (!($cmp286)) { __label__ = 95;break ; }
  
              var $185=HEAP32[((__ZL12n_ssml_stack)>>2)]; //@line 1958 "readclause.cpp"
              var $dec289=((($185)-1)&4294967295); //@line 1958 "readclause.cpp"
              HEAP32[((__ZL12n_ssml_stack)>>2)]=$dec289; //@line 1958 "readclause.cpp"
              __lastLabel__ = 94; __label__ = 92;continue ; //@line 1959 "readclause.cpp"
            }
  
            $terminator=0; //@line 1961 "readclause.cpp"
            var $186=$terminator; //@line 1962 "readclause.cpp"
            var $187=$px; //@line 1962 "readclause.cpp"
            var $188=$tag_type; //@line 1962 "readclause.cpp"
            var $call291=__ZL18GetVoiceAttributesPwi($187, $188); //@line 1962 "readclause.cpp"
            var $add292=((($call291)+($186))&4294967295); //@line 1962 "readclause.cpp"
            $retval=$add292; //@line 1962 "readclause.cpp"
            __label__ = 113;break $if_end30$$return$19; //@line 1962 "readclause.cpp"
          }
          else if (__label__ == 96) {
  
            $retval=262174; //@line 1966 "readclause.cpp"
            __label__ = 113;break $if_end30$$return$19; //@line 1966 "readclause.cpp"
          }
          else if (__label__ == 97) {
  
            var $189=$ssml_sp; //@line 1969 "readclause.cpp"
            var $tag_type295=(($189)&4294967295); //@line 1969 "readclause.cpp"
            var $190=HEAP32[(($tag_type295)>>2)]; //@line 1969 "readclause.cpp"
            var $cmp296=((($190))|0)==6; //@line 1969 "readclause.cpp"
            if ($cmp296) { __label__ = 98;; } else { __label__ = 99;; } //@line 1969 "readclause.cpp"
            if (__label__ == 98) {
  
              var $191=$px; //@line 1972 "readclause.cpp"
              var $call298=__ZL18GetVoiceAttributesPwi($191, 38); //@line 1972 "readclause.cpp"
              $voice_change_flag=$call298; //@line 1972 "readclause.cpp"
              ; //@line 1973 "readclause.cpp"
            }
  
            var $192=$px; //@line 1974 "readclause.cpp"
            var $193=$tag_type; //@line 1974 "readclause.cpp"
            var $call300=__ZL18GetVoiceAttributesPwi($192, $193); //@line 1974 "readclause.cpp"
            var $194=$voice_change_flag; //@line 1974 "readclause.cpp"
            var $or=($194) | ($call300); //@line 1974 "readclause.cpp"
            $voice_change_flag=$or; //@line 1974 "readclause.cpp"
            var $195=$voice_change_flag; //@line 1975 "readclause.cpp"
            var $add301=((($195)+524358)&4294967295); //@line 1975 "readclause.cpp"
            $retval=$add301; //@line 1975 "readclause.cpp"
            __label__ = 113;break $if_end30$$return$19; //@line 1975 "readclause.cpp"
          }
          else if (__label__ == 100) {
  
            var $196=$ssml_sp; //@line 1979 "readclause.cpp"
            var $tag_type303=(($196)&4294967295); //@line 1979 "readclause.cpp"
            var $197=HEAP32[(($tag_type303)>>2)]; //@line 1979 "readclause.cpp"
            var $cmp304=((($197))|0)==6; //@line 1979 "readclause.cpp"
            if ($cmp304) { __label__ = 101;; } else { __label__ = 102;; } //@line 1979 "readclause.cpp"
            if (__label__ == 101) {
  
              var $198=$px; //@line 1982 "readclause.cpp"
              var $call306=__ZL18GetVoiceAttributesPwi($198, 38); //@line 1982 "readclause.cpp"
              $voice_change_flag=$call306; //@line 1982 "readclause.cpp"
              ; //@line 1983 "readclause.cpp"
            }
  
            var $199=$ssml_sp; //@line 1984 "readclause.cpp"
            var $tag_type308=(($199)&4294967295); //@line 1984 "readclause.cpp"
            var $200=HEAP32[(($tag_type308)>>2)]; //@line 1984 "readclause.cpp"
            var $cmp309=((($200))|0)==7; //@line 1984 "readclause.cpp"
            if ($cmp309) { __label__ = 103;; } else { __label__ = 104;; } //@line 1984 "readclause.cpp"
            if (__label__ == 103) {
  
              var $201=$px; //@line 1987 "readclause.cpp"
              var $call311=__ZL18GetVoiceAttributesPwi($201, 39); //@line 1987 "readclause.cpp"
              var $202=$voice_change_flag; //@line 1987 "readclause.cpp"
              var $or312=($202) | ($call311); //@line 1987 "readclause.cpp"
              $voice_change_flag=$or312; //@line 1987 "readclause.cpp"
              ; //@line 1988 "readclause.cpp"
            }
  
            var $203=$px; //@line 1989 "readclause.cpp"
            var $204=$tag_type; //@line 1989 "readclause.cpp"
            var $call314=__ZL18GetVoiceAttributesPwi($203, $204); //@line 1989 "readclause.cpp"
            var $205=$voice_change_flag; //@line 1989 "readclause.cpp"
            var $or315=($205) | ($call314); //@line 1989 "readclause.cpp"
            $voice_change_flag=$or315; //@line 1989 "readclause.cpp"
            var $206=$voice_change_flag; //@line 1990 "readclause.cpp"
            var $add316=((($206)+524358)&4294967295); //@line 1990 "readclause.cpp"
            $retval=$add316; //@line 1990 "readclause.cpp"
            __label__ = 113;break $if_end30$$return$19; //@line 1990 "readclause.cpp"
          }
          else if (__label__ == 105) {
  
            var $207=$ssml_sp; //@line 1994 "readclause.cpp"
            var $tag_type318=(($207)&4294967295); //@line 1994 "readclause.cpp"
            var $208=HEAP32[(($tag_type318)>>2)]; //@line 1994 "readclause.cpp"
            var $cmp319=((($208))|0)==6; //@line 1994 "readclause.cpp"
            if ($cmp319) { __label__ = 106;; } else { __label__ = 107;; } //@line 1994 "readclause.cpp"
            if (__label__ == 106) {
  
              var $209=$px; //@line 1997 "readclause.cpp"
              var $210=$tag_type; //@line 1997 "readclause.cpp"
              var $call321=__ZL18GetVoiceAttributesPwi($209, $210); //@line 1997 "readclause.cpp"
              $voice_change_flag=$call321; //@line 1997 "readclause.cpp"
              ; //@line 1998 "readclause.cpp"
            }
  
            var $211=$voice_change_flag; //@line 1999 "readclause.cpp"
            var $add323=((($211)+524328)&4294967295); //@line 1999 "readclause.cpp"
            $retval=$add323; //@line 1999 "readclause.cpp"
            __label__ = 113;break $if_end30$$return$19; //@line 1999 "readclause.cpp"
          }
          else if (__label__ == 108) {
  
            var $212=$ssml_sp; //@line 2003 "readclause.cpp"
            var $tag_type325=(($212)&4294967295); //@line 2003 "readclause.cpp"
            var $213=HEAP32[(($tag_type325)>>2)]; //@line 2003 "readclause.cpp"
            var $cmp326=((($213))|0)==6; //@line 2003 "readclause.cpp"
            if ($cmp326) { __label__ = 110;; } else { __label__ = 109;; } //@line 2003 "readclause.cpp"
            $if_then330$$lor_lhs_false327$145: do { 
              if (__label__ == 109) {
  
                var $214=$ssml_sp; //@line 2003 "readclause.cpp"
                var $tag_type328=(($214)&4294967295); //@line 2003 "readclause.cpp"
                var $215=HEAP32[(($tag_type328)>>2)]; //@line 2003 "readclause.cpp"
                var $cmp329=((($215))|0)==7; //@line 2003 "readclause.cpp"
                if ($cmp329) { __label__ = 110;break $if_then330$$lor_lhs_false327$145; } //@line 2003 "readclause.cpp"
  
                $retval=524358; //@line 2009 "readclause.cpp"
                __label__ = 113;break $if_end30$$return$19; //@line 2009 "readclause.cpp"
              }
            } while(0);
  
            var $216=$px; //@line 2007 "readclause.cpp"
            var $217=$tag_type; //@line 2007 "readclause.cpp"
            var $call331=__ZL18GetVoiceAttributesPwi($216, $217); //@line 2007 "readclause.cpp"
            var $add332=((($call331)+524358)&4294967295); //@line 2007 "readclause.cpp"
            $retval=$add332; //@line 2007 "readclause.cpp"
            __label__ = 113;break $if_end30$$return$19; //@line 2007 "readclause.cpp"
          }
        } while(0);
  
        $retval=0; //@line 2011 "readclause.cpp"
        ; //@line 2011 "readclause.cpp"
      }
    } while(0);
  
    var $218=$retval; //@line 2012 "readclause.cpp"
    STACKTOP = __stackBase__;
    return $218; //@line 2012 "readclause.cpp"
    return null;
  }
  

  function __ZL13LookupSpecialP10TranslatorPKcPc($tr, $text_out) {
    var __stackBase__  = STACKTOP; STACKTOP += 124; _memset(__stackBase__, 0, 124);
    var __label__;
  
    var $retval;
    var $tr_addr;
    var $string_addr;
    var $text_out_addr;
    var $flags=__stackBase__;
    var $phonemes=__stackBase__+8;
    var $phonemes2=__stackBase__+64;
    var $string1=__stackBase__+120;
    $tr_addr=$tr;
    $string_addr=((__str6303)&4294967295);
    $text_out_addr=$text_out;
    var $0=$string_addr; //@line 593 "readclause.cpp"
    HEAP32[(($string1)>>2)]=$0; //@line 593 "readclause.cpp"
    var $arrayidx=(($flags+4)&4294967295); //@line 595 "readclause.cpp"
    HEAP32[(($arrayidx)>>2)]=0; //@line 595 "readclause.cpp"
    var $arrayidx1=(($flags)&4294967295); //@line 595 "readclause.cpp"
    HEAP32[(($arrayidx1)>>2)]=0; //@line 595 "readclause.cpp"
    var $1=$tr_addr; //@line 596 "readclause.cpp"
    var $arraydecay=(($phonemes)&4294967295); //@line 596 "readclause.cpp"
    var $arraydecay2=(($flags)&4294967295); //@line 596 "readclause.cpp"
    var $call=__Z14LookupDictListP10TranslatorPPcS1_PjiP8WORD_TAB($1, $string1, $arraydecay, $arraydecay2, 0, 0); //@line 596 "readclause.cpp"
    var $tobool=((($call))|0)!=0; //@line 596 "readclause.cpp"
    if ($tobool) { __label__ = 1;; } else { __label__ = 2;; } //@line 596 "readclause.cpp"
    if (__label__ == 1) {
  
      var $2=$tr_addr; //@line 598 "readclause.cpp"
      var $arraydecay3=(($phonemes)&4294967295); //@line 598 "readclause.cpp"
      var $arraydecay4=(($flags)&4294967295); //@line 598 "readclause.cpp"
      __Z13SetWordStressP10TranslatorPcPjii($2, $arraydecay3, $arraydecay4, -1, 0); //@line 598 "readclause.cpp"
      var $arraydecay5=(($phonemes)&4294967295); //@line 599 "readclause.cpp"
      var $arraydecay6=(($phonemes2)&4294967295); //@line 599 "readclause.cpp"
      __Z14DecodePhonemesPKcPc($arraydecay5, $arraydecay6); //@line 599 "readclause.cpp"
      var $3=$text_out_addr; //@line 600 "readclause.cpp"
      var $arraydecay7=(($phonemes2)&4294967295); //@line 600 "readclause.cpp"
      var $call8=_sprintf($3, ((__str60227)&4294967295), allocate([$arraydecay7,0,0,0], ["i8*",0,0,0], ALLOC_STACK)); //@line 600 "readclause.cpp"
      var $4=$text_out_addr; //@line 601 "readclause.cpp"
      $retval=$4; //@line 601 "readclause.cpp"
      ; //@line 601 "readclause.cpp"
    }
    else if (__label__ == 2) {
  
      $retval=0; //@line 603 "readclause.cpp"
      ; //@line 603 "readclause.cpp"
    }
  
    var $5=$retval; //@line 604 "readclause.cpp"
    STACKTOP = __stackBase__;
    return $5; //@line 604 "readclause.cpp"
    return null;
  }
  

  function __ZL10RemoveCharPc($p) {
    var __stackBase__  = STACKTOP; STACKTOP += 4; _memset(__stackBase__, 0, 4);
    var __label__;
  
    var $p_addr;
    var $c=__stackBase__;
    $p_addr=$p;
    var $0=$p_addr; //@line 2020 "readclause.cpp"
    var $1=$p_addr; //@line 2020 "readclause.cpp"
    var $call=__Z7utf8_inPiPKc($c, $1); //@line 2020 "readclause.cpp"
    var $dest$, $stop$, $stop4$, $fast$, $value4$;
    $dest$ = $0;
    $stop$ = $dest$ + $call;
    $value4$ = 32;
    if ($value4$ < 0) $value4$ += 256;
    $value4$ = $value4$ + ($value4$<<8) + ($value4$<<16) + ($value4$*16777216);
    while ($dest$%4 !== 0 && $dest$ < $stop$) {
      ;; HEAP8[$dest$++] = 32;
    }
    $dest$ >>= 2;
    $stop4$ = $stop$ >> 2;
    while ($dest$ < $stop4$) {
    ;;;;;;;  HEAP32[$dest$++] = $value4$;
    }
    $dest$ <<= 2;
    while ($dest$ < $stop$) {
      ;; HEAP8[$dest$++] = 32;
    }; //@line 2020 "readclause.cpp"
    STACKTOP = __stackBase__;
    return; //@line 2021 "readclause.cpp"
    return;
  }
  

  function __ZL19AnnouncePunctuationP10TranslatoriPiPcS1_i($tr, $c1, $c2_ptr, $output, $bufix, $end_clause) {
    var __stackBase__  = STACKTOP; STACKTOP += 280; _memset(__stackBase__, 0, 280);
    var __label__;
  
    var $retval;
    var $tr_addr;
    var $c1_addr;
    var $c2_ptr_addr;
    var $output_addr;
    var $bufix_addr;
    var $end_clause_addr;
    var $punct_count;
    var $punctname;
    var $found;
    var $soundicon;
    var $attributes;
    var $short_pause;
    var $c2;
    var $len;
    var $bufix1;
    var $buf=__stackBase__;
    var $buf2=__stackBase__+200;
    $tr_addr=$tr;
    $c1_addr=$c1;
    $c2_ptr_addr=$c2_ptr;
    $output_addr=$output;
    $bufix_addr=$bufix;
    $end_clause_addr=$end_clause;
    $found=0; //@line 863 "readclause.cpp"
    var $0=$c2_ptr_addr; //@line 873 "readclause.cpp"
    var $1=HEAP32[(($0)>>2)]; //@line 873 "readclause.cpp"
    $c2=$1; //@line 873 "readclause.cpp"
    var $arrayidx=(($buf)&4294967295); //@line 874 "readclause.cpp"
    HEAP8[($arrayidx)]=0; //@line 874 "readclause.cpp"
    var $2=$c1_addr; //@line 876 "readclause.cpp"
    var $call=__ZL15LookupSoundiconi($2); //@line 876 "readclause.cpp"
    $soundicon=$call; //@line 876 "readclause.cpp"
    var $cmp=((($call))|0) >= 0; //@line 876 "readclause.cpp"
    if ($cmp) { __label__ = 1;; } else { __label__ = 2;; } //@line 876 "readclause.cpp"
    $if_end63_thread$$if_else$2: do { 
      if (__label__ == 1) {
  
        var $arraydecay=(($buf)&4294967295); //@line 879 "readclause.cpp"
        var $3=$soundicon; //@line 879 "readclause.cpp"
        var $call1=_sprintf($arraydecay, ((__str50217)&4294967295), allocate([$3,0,0,0], ["i32",0,0,0], ALLOC_STACK)); //@line 879 "readclause.cpp"
        var $4=$c2; //@line 880 "readclause.cpp"
        __ZL6UngetCi($4); //@line 880 "readclause.cpp"
        $found=1; //@line 881 "readclause.cpp"
        __label__ = 26;break $if_end63_thread$$if_else$2; //@line 946 "readclause.cpp"
      }
      else if (__label__ == 2) {
  
        var $5=$tr_addr; //@line 884 "readclause.cpp"
        var $6=$c1_addr; //@line 884 "readclause.cpp"
        __ZL14LookupCharNameP10Translatorii($5, $6, 0); //@line 884 "readclause.cpp"
        $punctname=((__ZZL14LookupCharNameP10TranslatoriiE3buf)&4294967295); //@line 884 "readclause.cpp"
        $found=1; //@line 886 "readclause.cpp"
        var $7=$bufix_addr; //@line 887 "readclause.cpp"
        var $8=HEAP32[(($7)>>2)]; //@line 887 "readclause.cpp"
        var $cmp5=((($8))|0)==0; //@line 887 "readclause.cpp"
        if ($cmp5) { __label__ = 5;; } else { __label__ = 3;; } //@line 887 "readclause.cpp"
        $if_then9$$lor_lhs_false$5: do { 
          if (__label__ == 3) {
  
            var $9=$end_clause_addr; //@line 887 "readclause.cpp"
            var $cmp6=((($9))|0)==0; //@line 887 "readclause.cpp"
            if ($cmp6) { __label__ = 5;break $if_then9$$lor_lhs_false$5; } //@line 887 "readclause.cpp"
  
            var $10=$tr_addr; //@line 887 "readclause.cpp"
            var $langopts=(($10)&4294967295); //@line 887 "readclause.cpp"
            var $param=(($langopts+24)&4294967295); //@line 887 "readclause.cpp"
            var $arrayidx8=(($param+68)&4294967295); //@line 887 "readclause.cpp"
            var $11=HEAP32[(($arrayidx8)>>2)]; //@line 887 "readclause.cpp"
            var $and=($11) & 2; //@line 887 "readclause.cpp"
            var $tobool=((($and))|0)!=0; //@line 887 "readclause.cpp"
            if ($tobool) { __label__ = 5;break $if_then9$$lor_lhs_false$5; } //@line 887 "readclause.cpp"
  
            var $34=$c2; //@line 934 "readclause.cpp"
            __ZL6UngetCi($34); //@line 934 "readclause.cpp"
            var $35=HEAP32[((_option_ssml)>>2)]; //@line 935 "readclause.cpp"
            var $tobool51=((($35))|0)!=0; //@line 935 "readclause.cpp"
            if ($tobool51) { __label__ = 21;; } else { __label__ = 23;; } //@line 935 "readclause.cpp"
            $if_then52$$if_end58$9: do { 
              if (__label__ == 21) {
  
                var $36=$c1_addr; //@line 937 "readclause.cpp"
                var $cmp53=((($36))|0)==60; //@line 937 "readclause.cpp"
                var $37=$c1_addr; //@line 937 "readclause.cpp"
                var $cmp55=((($37))|0)==38; //@line 937 "readclause.cpp"
                var $or_cond=($cmp53) | ($cmp55); //@line 937 "readclause.cpp"
                if (!($or_cond)) { __label__ = 23;break $if_then52$$if_end58$9; } //@line 937 "readclause.cpp"
  
                var $38=$c1_addr; //@line 938 "readclause.cpp"
                HEAP32[((__ZL19ssml_ignore_l_angle)>>2)]=$38; //@line 938 "readclause.cpp"
                ; //@line 938 "readclause.cpp"
              }
            } while(0);
  
            var $39=$c1_addr; //@line 940 "readclause.cpp"
            HEAP32[((__ZL11ungot_char2)>>2)]=$39; //@line 940 "readclause.cpp"
            var $arrayidx59=(($buf)&4294967295); //@line 941 "readclause.cpp"
            HEAP8[($arrayidx59)]=32; //@line 941 "readclause.cpp"
            var $arrayidx60=(($buf+1)&4294967295); //@line 942 "readclause.cpp"
            HEAP8[($arrayidx60)]=0; //@line 942 "readclause.cpp"
            __label__ = 24;break $if_then9$$lor_lhs_false$5;
          }
        } while(0);
        $if_then9$$if_end63$13: do { 
          if (__label__ == 5) {
  
            $punct_count=1; //@line 889 "readclause.cpp"
            var $12=$c2; //@line 890 "readclause.cpp"
            var $13=$c1_addr; //@line 890 "readclause.cpp"
            var $cmp105=((($12))|0)==((($13))|0); //@line 890 "readclause.cpp"
            var $14=$c1_addr; //@line 890 "readclause.cpp"
            var $cmp116=((($14))|0)!=60; //@line 890 "readclause.cpp"
            var $or_cond17=($cmp105) & ($cmp116); //@line 890 "readclause.cpp"
            if ($or_cond17) { __label__ = 6;; } else { __label__ = 7;; } //@line 890 "readclause.cpp"
            $while_body$$while_end$15: do { 
              if (__label__ == 6) {
                while(1) { 
  
                  var $15=$punct_count; //@line 892 "readclause.cpp"
                  var $inc=((($15)+1)&4294967295); //@line 892 "readclause.cpp"
                  $punct_count=$inc; //@line 892 "readclause.cpp"
                  var $call12=__ZL4GetCv(); //@line 893 "readclause.cpp"
                  $c2=$call12; //@line 893 "readclause.cpp"
                  var $16=$c2; //@line 890 "readclause.cpp"
                  var $17=$c1_addr; //@line 890 "readclause.cpp"
                  var $cmp10=((($16))|0)==((($17))|0); //@line 890 "readclause.cpp"
                  var $18=$c1_addr; //@line 890 "readclause.cpp"
                  var $cmp11=((($18))|0)!=60; //@line 890 "readclause.cpp"
                  var $or_cond1=($cmp10) & ($cmp11); //@line 890 "readclause.cpp"
                  if ($or_cond1) { __label__ = 6;continue ; } else { __label__ = 7;break $while_body$$while_end$15; } //@line 890 "readclause.cpp"
                }
              }
            } while(0);
  
            var $19=$c2; //@line 895 "readclause.cpp"
            var $20=$c2_ptr_addr; //@line 895 "readclause.cpp"
            HEAP32[(($20)>>2)]=$19; //@line 895 "readclause.cpp"
            var $21=$end_clause_addr; //@line 896 "readclause.cpp"
            var $tobool13=((($21))|0)!=0; //@line 896 "readclause.cpp"
            if ($tobool13) { __label__ = 8;; } else { __label__ = 9;; } //@line 896 "readclause.cpp"
            if (__label__ == 8) {
  
              var $22=$c2; //@line 898 "readclause.cpp"
              __ZL6UngetCi($22); //@line 898 "readclause.cpp"
              ; //@line 899 "readclause.cpp"
            }
  
            var $23=$punct_count; //@line 901 "readclause.cpp"
            var $cmp15=((($23))|0)==1; //@line 901 "readclause.cpp"
            if ($cmp15) { __label__ = 10;; } else { __label__ = 11;; } //@line 901 "readclause.cpp"
            if (__label__ == 10) {
  
              var $arraydecay17=(($buf)&4294967295); //@line 904 "readclause.cpp"
              var $24=$punctname; //@line 904 "readclause.cpp"
              var $call18=_sprintf($arraydecay17, ((__str51218)&4294967295), allocate([$24,0,0,0], ["i8*",0,0,0], ALLOC_STACK)); //@line 904 "readclause.cpp"
              ; //@line 905 "readclause.cpp"
            }
            else if (__label__ == 11) {
  
              var $25=$punct_count; //@line 907 "readclause.cpp"
              var $cmp20=((($25))|0) < 4; //@line 907 "readclause.cpp"
              var $arrayidx22=(($buf)&4294967295); //@line 909 "readclause.cpp"
              if ($cmp20) { __label__ = 12;; } else { __label__ = 19;; } //@line 907 "readclause.cpp"
              if (__label__ == 12) {
  
                HEAP8[($arrayidx22)]=0; //@line 909 "readclause.cpp"
                var $26=HEAP32[((((_embedded_value+8)&4294967295))>>2)]; //@line 910 "readclause.cpp"
                var $cmp23=((($26))|0) < 300; //@line 910 "readclause.cpp"
                if ($cmp23) { __label__ = 13;; } else { __label__ = 14;; } //@line 910 "readclause.cpp"
                if (__label__ == 13) {
  
                  var $arraydecay25=(($buf)&4294967295); //@line 911 "readclause.cpp"
                  var $src$, $dest$, $stop$, $stop4$, $fast$;
                  $src$ = ((__str52219)&4294967295);
                  $dest$ = $arraydecay25;
                  $stop$ = $src$ + 6;
                  if (($dest$%4) == ($src$%4) && 6 > 8) {
                    while ($src$%4 !== 0 && $src$ < $stop$) {
                      ; HEAP8[$dest$++] = HEAP8[$src$++];
                    }
                    $src$ >>= 2;
                    $dest$ >>= 2;
                    $stop4$ = $stop$ >> 2;
                    while ($src$ < $stop4$) {
                  ;;;    HEAP32[$dest$++] = HEAP32[$src$++];
                    }
                    $src$ <<= 2;
                    $dest$ <<= 2;
                  }while ($src$ < $stop$) {
                    ; HEAP8[$dest$++] = HEAP8[$src$++];
                  }; //@line 911 "readclause.cpp"
                  ; //@line 911 "readclause.cpp"
                }
  
                var $27=$punct_count; //@line 913 "readclause.cpp"
                var $dec3=((($27)-1)&4294967295); //@line 913 "readclause.cpp"
                $punct_count=$dec3; //@line 913 "readclause.cpp"
                var $cmp294=((($27))|0) > 0; //@line 913 "readclause.cpp"
                if ($cmp294) { __label__ = 15;; } else { __label__ = 17;; } //@line 913 "readclause.cpp"
                $while_body30_lr_ph$$while_end36$30: do { 
                  if (__label__ == 15) {
  
                    var $arraydecay31=(($buf2)&4294967295); //@line 915 "readclause.cpp"
                    var $arraydecay33=(($buf)&4294967295); //@line 916 "readclause.cpp"
                    var $arraydecay34=(($buf2)&4294967295); //@line 916 "readclause.cpp"
                    ; //@line 913 "readclause.cpp"
                    while(1) { 
  
                      var $28=$punctname; //@line 915 "readclause.cpp"
                      var $call32=_sprintf($arraydecay31, ((__str51218)&4294967295), allocate([$28,0,0,0], ["i8*",0,0,0], ALLOC_STACK)); //@line 915 "readclause.cpp"
                      var $call35=_strcat($arraydecay33, $arraydecay34); //@line 916 "readclause.cpp"
                      var $29=$punct_count; //@line 913 "readclause.cpp"
                      var $dec=((($29)-1)&4294967295); //@line 913 "readclause.cpp"
                      $punct_count=$dec; //@line 913 "readclause.cpp"
                      var $cmp29=((($29))|0) > 0; //@line 913 "readclause.cpp"
                      if ($cmp29) { __label__ = 16;continue ; } else { __label__ = 17;break $while_body30_lr_ph$$while_end36$30; } //@line 913 "readclause.cpp"
                    }
                  }
                } while(0);
  
                var $30=HEAP32[((((_embedded_value+8)&4294967295))>>2)]; //@line 919 "readclause.cpp"
                var $cmp37=((($30))|0) < 300; //@line 919 "readclause.cpp"
                if (!($cmp37)) { __label__ = 24;break $if_then9$$if_end63$13; } //@line 919 "readclause.cpp"
  
                var $arraydecay39=(($buf2)&4294967295); //@line 921 "readclause.cpp"
                var $src$, $dest$, $stop$, $stop4$, $fast$;
                $src$ = ((__str53220)&4294967295);
                $dest$ = $arraydecay39;
                $stop$ = $src$ + 7;
                if (($dest$%4) == ($src$%4) && 7 > 8) {
                  while ($src$%4 !== 0 && $src$ < $stop$) {
                    ; HEAP8[$dest$++] = HEAP8[$src$++];
                  }
                  $src$ >>= 2;
                  $dest$ >>= 2;
                  $stop4$ = $stop$ >> 2;
                  while ($src$ < $stop4$) {
                ;;;    HEAP32[$dest$++] = HEAP32[$src$++];
                  }
                  $src$ <<= 2;
                  $dest$ <<= 2;
                }while ($src$ < $stop$) {
                  ; HEAP8[$dest$++] = HEAP8[$src$++];
                }; //@line 921 "readclause.cpp"
                var $arraydecay41=(($buf)&4294967295); //@line 922 "readclause.cpp"
                var $arraydecay42=(($buf2)&4294967295); //@line 922 "readclause.cpp"
                var $call43=_strcat($arraydecay41, $arraydecay42); //@line 922 "readclause.cpp"
                ; //@line 923 "readclause.cpp"
              }
              else if (__label__ == 19) {
  
                var $31=$punctname; //@line 927 "readclause.cpp"
                var $32=$punct_count; //@line 927 "readclause.cpp"
                var $33=$punctname; //@line 927 "readclause.cpp"
                var $call47=_sprintf($arrayidx22, ((__str54221)&4294967295), allocate([$31,0,0,0,$32,0,0,0,$33,0,0,0], ["i8*",0,0,0,"i32",0,0,0,"i8*",0,0,0], ALLOC_STACK)); //@line 927 "readclause.cpp"
                ;
              }
            }
          }
        } while(0);
  
        var $_pr=$found; //@line 946 "readclause.cpp"
        var $cmp64=((($_pr))|0)==0; //@line 946 "readclause.cpp"
        if (!($cmp64)) { __label__ = 26;break $if_end63_thread$$if_else$2; } //@line 946 "readclause.cpp"
  
        $retval=-1; //@line 947 "readclause.cpp"
        __label__ = 40;break $if_end63_thread$$if_else$2; //@line 947 "readclause.cpp"
      }
    } while(0);
    $if_end66$$return$39: do { 
      if (__label__ == 26) {
  
        var $40=$bufix_addr; //@line 949 "readclause.cpp"
        var $41=HEAP32[(($40)>>2)]; //@line 949 "readclause.cpp"
        $bufix1=$41; //@line 949 "readclause.cpp"
        var $arraydecay67=(($buf)&4294967295); //@line 950 "readclause.cpp"
        var $call68=_strlen($arraydecay67); //@line 950 "readclause.cpp"
        $len=$call68; //@line 950 "readclause.cpp"
        var $42=$bufix_addr; //@line 951 "readclause.cpp"
        var $43=HEAP32[(($42)>>2)]; //@line 951 "readclause.cpp"
        var $44=$output_addr; //@line 951 "readclause.cpp"
        var $arrayidx69=(($44+$43)&4294967295); //@line 951 "readclause.cpp"
        var $arraydecay70=(($buf)&4294967295); //@line 951 "readclause.cpp"
        var $call71=_strcpy($arrayidx69, $arraydecay70); //@line 951 "readclause.cpp"
        var $45=$len; //@line 952 "readclause.cpp"
        var $46=$bufix_addr; //@line 952 "readclause.cpp"
        var $47=HEAP32[(($46)>>2)]; //@line 952 "readclause.cpp"
        var $add=((($47)+($45))&4294967295); //@line 952 "readclause.cpp"
        HEAP32[(($46)>>2)]=$add; //@line 952 "readclause.cpp"
        var $48=$end_clause_addr; //@line 954 "readclause.cpp"
        var $cmp72=((($48))|0)==0; //@line 954 "readclause.cpp"
        if ($cmp72) { __label__ = 27;; } else { __label__ = 28;; } //@line 954 "readclause.cpp"
        if (__label__ == 27) {
  
          $retval=-1; //@line 955 "readclause.cpp"
          ; //@line 955 "readclause.cpp"
        }
        else if (__label__ == 28) {
  
          var $49=$c1_addr; //@line 957 "readclause.cpp"
          var $cmp75=((($49))|0)==45; //@line 957 "readclause.cpp"
          if ($cmp75) { __label__ = 29;; } else { __label__ = 30;; } //@line 957 "readclause.cpp"
          if (__label__ == 29) {
  
            $retval=16384; //@line 958 "readclause.cpp"
            ; //@line 958 "readclause.cpp"
          }
          else if (__label__ == 30) {
  
            var $50=$c1_addr; //@line 960 "readclause.cpp"
            var $call78=__Z11lookupwcharPKti(((__ZL11punct_chars)&4294967295), $50); //@line 960 "readclause.cpp"
            var $arrayidx79=((__ZL16punct_attributes+$call78*4)&4294967295); //@line 960 "readclause.cpp"
            var $51=HEAP32[(($arrayidx79)>>2)]; //@line 960 "readclause.cpp"
            $attributes=$51; //@line 960 "readclause.cpp"
            $short_pause=262148; //@line 962 "readclause.cpp"
            var $52=$attributes; //@line 963 "readclause.cpp"
            var $and80=($52) & 28672; //@line 963 "readclause.cpp"
            var $cmp81=((($and80))|0)==4096; //@line 963 "readclause.cpp"
            if ($cmp81) { __label__ = 31;; } else { __label__ = 32;; } //@line 963 "readclause.cpp"
            if (__label__ == 31) {
  
              $short_pause=266244; //@line 964 "readclause.cpp"
              ; //@line 964 "readclause.cpp"
            }
  
            var $53=$bufix1; //@line 966 "readclause.cpp"
            var $cmp84=((($53))|0) > 0; //@line 966 "readclause.cpp"
            if ($cmp84) { __label__ = 33;; } else { __label__ = 37;; } //@line 966 "readclause.cpp"
            $land_lhs_true$$if_end95$50: do { 
              if (__label__ == 33) {
  
                var $54=$tr_addr; //@line 966 "readclause.cpp"
                var $langopts85=(($54)&4294967295); //@line 966 "readclause.cpp"
                var $param86=(($langopts85+24)&4294967295); //@line 966 "readclause.cpp"
                var $arrayidx87=(($param86+68)&4294967295); //@line 966 "readclause.cpp"
                var $55=HEAP32[(($arrayidx87)>>2)]; //@line 966 "readclause.cpp"
                var $and88=($55) & 2; //@line 966 "readclause.cpp"
                var $tobool89=((($and88))|0)!=0; //@line 966 "readclause.cpp"
                if ($tobool89) { __label__ = 37;break $land_lhs_true$$if_end95$50; } //@line 966 "readclause.cpp"
  
                var $56=$attributes; //@line 968 "readclause.cpp"
                var $and91=($56) & -32769; //@line 968 "readclause.cpp"
                var $cmp92=((($and91))|0)==266270; //@line 968 "readclause.cpp"
                if ($cmp92) { __label__ = 35;; } else { __label__ = 36;; } //@line 968 "readclause.cpp"
                if (__label__ == 35) {
  
                  $retval=262148; //@line 969 "readclause.cpp"
                  __label__ = 40;break $if_end66$$return$39; //@line 969 "readclause.cpp"
                }
                else if (__label__ == 36) {
  
                  var $57=$short_pause; //@line 970 "readclause.cpp"
                  $retval=$57; //@line 970 "readclause.cpp"
                  __label__ = 40;break $if_end66$$return$39; //@line 970 "readclause.cpp"
                }
              }
            } while(0);
  
            var $58=$attributes; //@line 973 "readclause.cpp"
            var $and96=($58) & 524288; //@line 973 "readclause.cpp"
            var $tobool97=((($and96))|0)!=0; //@line 973 "readclause.cpp"
            if ($tobool97) { __label__ = 38;; } else { __label__ = 39;; } //@line 973 "readclause.cpp"
            if (__label__ == 38) {
  
              var $59=$attributes; //@line 974 "readclause.cpp"
              $retval=$59; //@line 974 "readclause.cpp"
              ; //@line 974 "readclause.cpp"
            }
            else if (__label__ == 39) {
  
              var $60=$short_pause; //@line 976 "readclause.cpp"
              $retval=$60; //@line 976 "readclause.cpp"
              ; //@line 976 "readclause.cpp"
            }
          }
        }
      }
    } while(0);
  
    var $61=$retval; //@line 977 "readclause.cpp"
    STACKTOP = __stackBase__;
    return $61; //@line 977 "readclause.cpp"
    return null;
  }
  

  function __ZL14LookupCharNameP10Translatorii($tr, $c, $only) {
    var __stackBase__  = STACKTOP; STACKTOP += 156; _memset(__stackBase__, 0, 156);
    var __label__;
  
    var $tr_addr;
    var $c_addr;
    var $only_addr;
    var $ix;
    var $flags=__stackBase__;
    var $single_letter=__stackBase__+8;
    var $phonemes=__stackBase__+32;
    var $phonemes2=__stackBase__+92;
    var $lang_name;
    var $string=__stackBase__+152;
    $tr_addr=$tr;
    $c_addr=$c;
    $only_addr=$only;
    $lang_name=0; //@line 617 "readclause.cpp"
    HEAP8[(((__ZZL14LookupCharNameP10TranslatoriiE3buf)&4294967295))]=0; //@line 621 "readclause.cpp"
    var $arrayidx=(($flags)&4294967295); //@line 622 "readclause.cpp"
    HEAP32[(($arrayidx)>>2)]=0; //@line 622 "readclause.cpp"
    var $arrayidx1=(($flags+4)&4294967295); //@line 623 "readclause.cpp"
    HEAP32[(($arrayidx1)>>2)]=0; //@line 623 "readclause.cpp"
    var $arrayidx2=(($single_letter)&4294967295); //@line 624 "readclause.cpp"
    HEAP8[($arrayidx2)]=0; //@line 624 "readclause.cpp"
    var $arrayidx3=(($single_letter+1)&4294967295); //@line 625 "readclause.cpp"
    HEAP8[($arrayidx3)]=95; //@line 625 "readclause.cpp"
    var $0=$c_addr; //@line 626 "readclause.cpp"
    var $arrayidx4=(($single_letter+2)&4294967295); //@line 626 "readclause.cpp"
    var $call=__Z8utf8_outjPc($0, $arrayidx4); //@line 626 "readclause.cpp"
    $ix=$call; //@line 626 "readclause.cpp"
    var $1=$ix; //@line 627 "readclause.cpp"
    var $add=((($1)+2)&4294967295); //@line 627 "readclause.cpp"
    var $arrayidx5=(($single_letter+$add)&4294967295); //@line 627 "readclause.cpp"
    HEAP8[($arrayidx5)]=0; //@line 627 "readclause.cpp"
    var $2=$only_addr; //@line 629 "readclause.cpp"
    var $tobool=((($2))|0)!=0; //@line 629 "readclause.cpp"
    if ($tobool) { __label__ = 1;; } else { __label__ = 2;; } //@line 629 "readclause.cpp"
    $if_then$$if_else$2: do { 
      if (__label__ == 1) {
  
        var $arrayidx6=(($single_letter+2)&4294967295); //@line 631 "readclause.cpp"
        HEAP32[(($string)>>2)]=$arrayidx6; //@line 631 "readclause.cpp"
        var $3=$tr_addr; //@line 632 "readclause.cpp"
        var $arraydecay=(($phonemes)&4294967295); //@line 632 "readclause.cpp"
        var $arraydecay7=(($flags)&4294967295); //@line 632 "readclause.cpp"
        var $call8=__Z14LookupDictListP10TranslatorPPcS1_PjiP8WORD_TAB($3, $string, $arraydecay, $arraydecay7, 0, 0); //@line 632 "readclause.cpp"
        ; //@line 633 "readclause.cpp"
      }
      else if (__label__ == 2) {
  
        var $arrayidx9=(($single_letter+1)&4294967295); //@line 636 "readclause.cpp"
        HEAP32[(($string)>>2)]=$arrayidx9; //@line 636 "readclause.cpp"
        var $4=$tr_addr; //@line 637 "readclause.cpp"
        var $arraydecay10=(($phonemes)&4294967295); //@line 637 "readclause.cpp"
        var $arraydecay11=(($flags)&4294967295); //@line 637 "readclause.cpp"
        var $call12=__Z14LookupDictListP10TranslatorPPcS1_PjiP8WORD_TAB($4, $string, $arraydecay10, $arraydecay11, 0, 0); //@line 637 "readclause.cpp"
        var $cmp=((($call12))|0)==0; //@line 637 "readclause.cpp"
        if (!($cmp)) { __label__ = 5;break $if_then$$if_else$2; } //@line 637 "readclause.cpp"
  
        var $arrayidx14=(($single_letter+2)&4294967295); //@line 640 "readclause.cpp"
        HEAP32[(($string)>>2)]=$arrayidx14; //@line 640 "readclause.cpp"
        var $5=$tr_addr; //@line 641 "readclause.cpp"
        var $arraydecay15=(($phonemes)&4294967295); //@line 641 "readclause.cpp"
        var $arraydecay16=(($flags)&4294967295); //@line 641 "readclause.cpp"
        var $call17=__Z14LookupDictListP10TranslatorPPcS1_PjiP8WORD_TAB($5, $string, $arraydecay15, $arraydecay16, 0, 0); //@line 641 "readclause.cpp"
        var $cmp18=((($call17))|0)==0; //@line 641 "readclause.cpp"
        if (!($cmp18)) { __label__ = 5;break $if_then$$if_else$2; } //@line 641 "readclause.cpp"
  
        var $arrayidx20=(($single_letter+1)&4294967295); //@line 644 "readclause.cpp"
        HEAP8[($arrayidx20)]=32; //@line 644 "readclause.cpp"
        var $6=$tr_addr; //@line 645 "readclause.cpp"
        var $arrayidx21=(($single_letter+2)&4294967295); //@line 645 "readclause.cpp"
        var $arraydecay22=(($phonemes)&4294967295); //@line 645 "readclause.cpp"
        var $call23=__Z14TranslateRulesP10TranslatorPcS1_iS1_iPj($6, $arrayidx21, $arraydecay22, 60, 0, 0, 0); //@line 645 "readclause.cpp"
        ; //@line 646 "readclause.cpp"
      }
    } while(0);
  
    var $7=$only_addr; //@line 650 "readclause.cpp"
    var $cmp26=((($7))|0)==0; //@line 650 "readclause.cpp"
    if ($cmp26) { __label__ = 6;; } else { __label__ = 13;; } //@line 650 "readclause.cpp"
    $land_lhs_true$$if_end50$8: do { 
      if (__label__ == 6) {
  
        var $arrayidx27=(($phonemes)&4294967295); //@line 650 "readclause.cpp"
        var $8=HEAP8[($arrayidx27)]; //@line 650 "readclause.cpp"
        var $conv=(tempInt=(($8)),(tempInt>=128?tempInt-256:tempInt)); //@line 650 "readclause.cpp"
        var $cmp28=((($conv))|0)==0; //@line 650 "readclause.cpp"
        if (!($cmp28)) { __label__ = 13;break $land_lhs_true$$if_end50$8; } //@line 650 "readclause.cpp"
  
        var $9=$tr_addr; //@line 650 "readclause.cpp"
        var $translator_name=(($9+280)&4294967295); //@line 650 "readclause.cpp"
        var $10=HEAP32[(($translator_name)>>2)]; //@line 650 "readclause.cpp"
        var $cmp30=((($10))|0)!=25966; //@line 650 "readclause.cpp"
        if (!($cmp30)) { __label__ = 13;break $land_lhs_true$$if_end50$8; } //@line 650 "readclause.cpp"
  
        var $call32=__Z14SetTranslator2PKc(((__str52704)&4294967295)); //@line 653 "readclause.cpp"
        var $arrayidx33=(($single_letter+1)&4294967295); //@line 654 "readclause.cpp"
        HEAP32[(($string)>>2)]=$arrayidx33; //@line 654 "readclause.cpp"
        var $arrayidx34=(($single_letter+1)&4294967295); //@line 655 "readclause.cpp"
        HEAP8[($arrayidx34)]=95; //@line 655 "readclause.cpp"
        var $11=HEAP32[((_translator2)>>2)]; //@line 656 "readclause.cpp"
        var $arraydecay35=(($phonemes)&4294967295); //@line 656 "readclause.cpp"
        var $arraydecay36=(($flags)&4294967295); //@line 656 "readclause.cpp"
        var $call37=__Z14LookupDictListP10TranslatorPPcS1_PjiP8WORD_TAB($11, $string, $arraydecay35, $arraydecay36, 0, 0); //@line 656 "readclause.cpp"
        var $cmp38=((($call37))|0)==0; //@line 656 "readclause.cpp"
        if ($cmp38) { __label__ = 9;; } else { __label__ = 10;; } //@line 656 "readclause.cpp"
        if (__label__ == 9) {
  
          var $arrayidx40=(($single_letter+2)&4294967295); //@line 658 "readclause.cpp"
          HEAP32[(($string)>>2)]=$arrayidx40; //@line 658 "readclause.cpp"
          var $12=HEAP32[((_translator2)>>2)]; //@line 659 "readclause.cpp"
          var $arraydecay41=(($phonemes)&4294967295); //@line 659 "readclause.cpp"
          var $arraydecay42=(($flags)&4294967295); //@line 659 "readclause.cpp"
          var $call43=__Z14LookupDictListP10TranslatorPPcS1_PjiP8WORD_TAB($12, $string, $arraydecay41, $arraydecay42, 0, 0); //@line 659 "readclause.cpp"
          ; //@line 660 "readclause.cpp"
        }
  
        var $arrayidx45=(($phonemes)&4294967295); //@line 661 "readclause.cpp"
        var $13=HEAP8[($arrayidx45)]; //@line 661 "readclause.cpp"
        var $tobool46=(tempInt=(($13)),(tempInt>=128?tempInt-256:tempInt))!=0; //@line 661 "readclause.cpp"
        if ($tobool46) { __label__ = 11;; } else { __label__ = 12;; } //@line 661 "readclause.cpp"
        if (__label__ == 11) {
  
          $lang_name=((__str52704)&4294967295); //@line 663 "readclause.cpp"
          ; //@line 664 "readclause.cpp"
        }
        else if (__label__ == 12) {
  
          var $14=HEAP32[((_voice)>>2)]; //@line 667 "readclause.cpp"
          var $phoneme_tab_ix=(($14+40)&4294967295); //@line 667 "readclause.cpp"
          var $15=HEAP32[(($phoneme_tab_ix)>>2)]; //@line 667 "readclause.cpp"
          __Z18SelectPhonemeTablei($15); //@line 667 "readclause.cpp"
          ;
        }
      }
    } while(0);
  
    var $arrayidx51=(($phonemes)&4294967295); //@line 671 "readclause.cpp"
    var $16=HEAP8[($arrayidx51)]; //@line 671 "readclause.cpp"
    var $tobool52=(tempInt=(($16)),(tempInt>=128?tempInt-256:tempInt))!=0; //@line 671 "readclause.cpp"
    if ($tobool52) { __label__ = 14;; } else { __label__ = 17;; } //@line 671 "readclause.cpp"
    $if_then53$$if_else73$19: do { 
      if (__label__ == 14) {
  
        var $17=$lang_name; //@line 673 "readclause.cpp"
        var $tobool54=((($17))|0)!=0; //@line 673 "readclause.cpp"
        if ($tobool54) { __label__ = 15;; } else { __label__ = 16;; } //@line 673 "readclause.cpp"
        if (__label__ == 15) {
  
          var $18=HEAP32[((_translator2)>>2)]; //@line 675 "readclause.cpp"
          var $arraydecay56=(($phonemes)&4294967295); //@line 675 "readclause.cpp"
          var $arraydecay57=(($flags)&4294967295); //@line 675 "readclause.cpp"
          __Z13SetWordStressP10TranslatorPcPjii($18, $arraydecay56, $arraydecay57, -1, 0); //@line 675 "readclause.cpp"
          var $arraydecay58=(($phonemes)&4294967295); //@line 676 "readclause.cpp"
          var $arraydecay59=(($phonemes2)&4294967295); //@line 676 "readclause.cpp"
          __Z14DecodePhonemesPKcPc($arraydecay58, $arraydecay59); //@line 676 "readclause.cpp"
          var $arraydecay60=(($phonemes2)&4294967295); //@line 677 "readclause.cpp"
          var $19=$tr_addr; //@line 677 "readclause.cpp"
          var $translator_name61=(($19+280)&4294967295); //@line 677 "readclause.cpp"
          var $20=HEAP32[(($translator_name61)>>2)]; //@line 677 "readclause.cpp"
          __ZL13WordToString2j($20); //@line 677 "readclause.cpp"
          var $call63=_sprintf(((__ZZL14LookupCharNameP10TranslatoriiE3buf)&4294967295), ((__str47214)&4294967295), allocate([((__str52704)&4294967295),0,0,0,$arraydecay60,0,0,0,((__ZZL13WordToString2jE3buf)&4294967295),0,0,0], ["i8*",0,0,0,"i8*",0,0,0,"i8*",0,0,0], ALLOC_STACK)); //@line 677 "readclause.cpp"
          var $21=HEAP32[((_voice)>>2)]; //@line 678 "readclause.cpp"
          var $phoneme_tab_ix64=(($21+40)&4294967295); //@line 678 "readclause.cpp"
          var $22=HEAP32[(($phoneme_tab_ix64)>>2)]; //@line 678 "readclause.cpp"
          __Z18SelectPhonemeTablei($22); //@line 678 "readclause.cpp"
          ; //@line 679 "readclause.cpp"
        }
        else if (__label__ == 16) {
  
          var $23=$tr_addr; //@line 682 "readclause.cpp"
          var $arraydecay66=(($phonemes)&4294967295); //@line 682 "readclause.cpp"
          var $arraydecay67=(($flags)&4294967295); //@line 682 "readclause.cpp"
          __Z13SetWordStressP10TranslatorPcPjii($23, $arraydecay66, $arraydecay67, -1, 0); //@line 682 "readclause.cpp"
          var $arraydecay68=(($phonemes)&4294967295); //@line 683 "readclause.cpp"
          var $arraydecay69=(($phonemes2)&4294967295); //@line 683 "readclause.cpp"
          __Z14DecodePhonemesPKcPc($arraydecay68, $arraydecay69); //@line 683 "readclause.cpp"
          var $arraydecay70=(($phonemes2)&4294967295); //@line 684 "readclause.cpp"
          var $call71=_sprintf(((__ZZL14LookupCharNameP10TranslatoriiE3buf)&4294967295), ((__str48215)&4294967295), allocate([$arraydecay70,0,0,0], ["i8*",0,0,0], ALLOC_STACK)); //@line 684 "readclause.cpp"
          ;
        }
      }
      else if (__label__ == 17) {
  
        var $24=$only_addr; //@line 688 "readclause.cpp"
        var $cmp74=((($24))|0)==0; //@line 688 "readclause.cpp"
        if (!($cmp74)) { __label__ = 19;break $if_then53$$if_else73$19; } //@line 688 "readclause.cpp"
  
        var $src$, $dest$, $stop$, $stop4$, $fast$;
        $src$ = ((__str49216)&4294967295);
        $dest$ = ((__ZZL14LookupCharNameP10TranslatoriiE3buf)&4294967295);
        $stop$ = $src$ + 17;
        if (($dest$%4) == ($src$%4) && 17 > 8) {
          while ($src$%4 !== 0 && $src$ < $stop$) {
            ; HEAP8[$dest$++] = HEAP8[$src$++];
          }
          $src$ >>= 2;
          $dest$ >>= 2;
          $stop4$ = $stop$ >> 2;
          while ($src$ < $stop4$) {
        ;;;    HEAP32[$dest$++] = HEAP32[$src$++];
          }
          $src$ <<= 2;
          $dest$ <<= 2;
        }while ($src$ < $stop$) {
          ; HEAP8[$dest$++] = HEAP8[$src$++];
        }; //@line 690 "readclause.cpp"
        ; //@line 691 "readclause.cpp"
      }
    } while(0);
  
    STACKTOP = __stackBase__;
    return;
    return;
  }
  

  function __ZL8IsRomanUj($c) {
    ;
    var __label__;
  
    var $retval;
    var $c_addr;
    $c_addr=$c;
    var $0=$c_addr; //@line 383 "readclause.cpp"
    var $cmp=((($0))|0)==73; //@line 383 "readclause.cpp"
    var $1=$c_addr; //@line 383 "readclause.cpp"
    var $cmp1=((($1))|0)==86; //@line 383 "readclause.cpp"
    var $or_cond=($cmp) | ($cmp1); //@line 383 "readclause.cpp"
    var $2=$c_addr; //@line 383 "readclause.cpp"
    var $cmp3=((($2))|0)==88; //@line 383 "readclause.cpp"
    var $or_cond1=($or_cond) | ($cmp3); //@line 383 "readclause.cpp"
    var $3=$c_addr; //@line 383 "readclause.cpp"
    var $cmp5=((($3))|0)==76; //@line 383 "readclause.cpp"
    var $or_cond2=($or_cond1) | ($cmp5); //@line 383 "readclause.cpp"
    if ($or_cond2) { __label__ = 1;; } else { __label__ = 2;; } //@line 383 "readclause.cpp"
    if (__label__ == 1) {
  
      $retval=1; //@line 384 "readclause.cpp"
      ; //@line 384 "readclause.cpp"
    }
    else if (__label__ == 2) {
  
      $retval=0; //@line 385 "readclause.cpp"
      ; //@line 385 "readclause.cpp"
    }
  
    var $4=$retval; //@line 386 "readclause.cpp"
    ;
    return $4; //@line 386 "readclause.cpp"
    return null;
  }
  

  function __Z12InitNamedatav() {
    ;
    var __label__;
  
    HEAP32[((__ZL11namedata_ix)>>2)]=0; //@line 2726 "readclause.cpp"
    var $0=HEAP32[((_namedata)>>2)]; //@line 2727 "readclause.cpp"
    var $cmp=((($0))|0)!=0; //@line 2727 "readclause.cpp"
    if ($cmp) { __label__ = 1;; } else { __label__ = 2;; } //@line 2727 "readclause.cpp"
    if (__label__ == 1) {
  
      var $1=HEAP32[((_namedata)>>2)]; //@line 2729 "readclause.cpp"
      ; //@line 2729 "readclause.cpp"
      HEAP32[((_namedata)>>2)]=0; //@line 2730 "readclause.cpp"
      HEAP32[((__ZL10n_namedata)>>2)]=0; //@line 2731 "readclause.cpp"
      ; //@line 2732 "readclause.cpp"
    }
  
    ;
    return; //@line 2733 "readclause.cpp"
    return;
  }
  

  function __Z9InitText2v() {
    ;
    var __label__;
  
    var $param;
    HEAP32[((__ZL10ungot_char)>>2)]=0; //@line 2740 "readclause.cpp"
    HEAP32[((__ZL11ungot_char2)>>2)]=0; //@line 2741 "readclause.cpp"
    HEAP32[((__ZL12n_ssml_stack)>>2)]=1; //@line 2743 "readclause.cpp"
    HEAP32[((__ZL13n_param_stack)>>2)]=1; //@line 2744 "readclause.cpp"
    HEAP32[((((__ZL10ssml_stack)&4294967295))>>2)]=0; //@line 2745 "readclause.cpp"
    $param=0; //@line 2747 "readclause.cpp"
    ; //@line 2747 "readclause.cpp"
    while(1) { 
  
      var $0=$param; //@line 2748 "readclause.cpp"
      var $arrayidx=((((_param_stack+4)&4294967295)+$0*4)&4294967295); //@line 2748 "readclause.cpp"
      var $1=HEAP32[(($arrayidx)>>2)]; //@line 2748 "readclause.cpp"
      var $2=$param; //@line 2748 "readclause.cpp"
      var $arrayidx1=((__ZL17speech_parameters+$2*4)&4294967295); //@line 2748 "readclause.cpp"
      HEAP32[(($arrayidx1)>>2)]=$1; //@line 2748 "readclause.cpp"
      var $3=$param; //@line 2747 "readclause.cpp"
      var $inc=((($3)+1)&4294967295); //@line 2747 "readclause.cpp"
      $param=$inc; //@line 2747 "readclause.cpp"
      var $cmp=((($inc))|0) < 15; //@line 2747 "readclause.cpp"
      if ($cmp) { __label__ = 1;continue ; } else { __label__ = 2;break ; } //@line 2747 "readclause.cpp"
    }
  
    var $4=HEAP32[((((__ZL17speech_parameters+20)&4294967295))>>2)]; //@line 2750 "readclause.cpp"
    HEAP32[((_option_punctuation)>>2)]=$4; //@line 2750 "readclause.cpp"
    var $5=HEAP32[((((__ZL17speech_parameters+24)&4294967295))>>2)]; //@line 2751 "readclause.cpp"
    HEAP32[((_option_capitals)>>2)]=$5; //@line 2751 "readclause.cpp"
    HEAP8[(((__ZL16current_voice_id)&4294967295))]=0; //@line 2753 "readclause.cpp"
    HEAP8[(__ZL11ignore_text_b)]=0;
    HEAP8[(__ZL10audio_text_b)]=0;
    HEAP8[(__ZL19clear_skipping_text_b)]=0;
    HEAP32[((_count_characters)>>2)]=-1; //@line 2758 "readclause.cpp"
    HEAP32[((__ZL10sayas_mode)>>2)]=0; //@line 2759 "readclause.cpp"
    HEAP32[((__ZL7xmlbase)>>2)]=0; //@line 2761 "readclause.cpp"
    ;
    return; //@line 2762 "readclause.cpp"
    return;
  }
  

  function __ZL13WordToString2j($word) {
    ;
    var __label__;
  
    var $word_addr;
    var $ix;
    var $p;
    $word_addr=$word;
    $p=((__ZZL13WordToString2jE3buf)&4294967295); //@line 577 "readclause.cpp"
    $ix=3; //@line 578 "readclause.cpp"
    ; //@line 578 "readclause.cpp"
    while(1) { 
  
      var $0=$word_addr; //@line 580 "readclause.cpp"
      var $1=$ix; //@line 580 "readclause.cpp"
      var $mul=((($1)*8)&4294967295); //@line 580 "readclause.cpp"
      var $shr=($0) >>> ((($mul))>>>0); //@line 580 "readclause.cpp"
      var $conv=((($shr)) & 255); //@line 580 "readclause.cpp"
      var $2=$p; //@line 580 "readclause.cpp"
      HEAP8[($2)]=$conv; //@line 580 "readclause.cpp"
      var $conv1=(tempInt=(($conv)),(tempInt>=128?tempInt-256:tempInt)); //@line 580 "readclause.cpp"
      var $cmp2=((($conv1))|0)!=0; //@line 580 "readclause.cpp"
      if ($cmp2) { __label__ = 2;; } else { __label__ = 3;; } //@line 580 "readclause.cpp"
      if (__label__ == 2) {
  
        var $3=$p; //@line 581 "readclause.cpp"
        var $incdec_ptr=(($3+1)&4294967295); //@line 581 "readclause.cpp"
        $p=$incdec_ptr; //@line 581 "readclause.cpp"
        ; //@line 581 "readclause.cpp"
      }
  
      var $4=$ix; //@line 578 "readclause.cpp"
      var $dec=((($4)-1)&4294967295); //@line 578 "readclause.cpp"
      $ix=$dec; //@line 578 "readclause.cpp"
      var $cmp=((($dec))|0) >= 0; //@line 578 "readclause.cpp"
      if ($cmp) { __label__ = 1;continue ; } else { __label__ = 4;break ; } //@line 578 "readclause.cpp"
    }
  
    var $5=$p; //@line 583 "readclause.cpp"
    HEAP8[($5)]=0; //@line 583 "readclause.cpp"
    ;
    return;
    return;
  }
  

  function __ZL15LookupSoundiconi($c) {
    ;
    var __label__;
  
    var $retval;
    var $c_addr;
    var $ix;
    $c_addr=$c;
    $ix=4; //@line 810 "readclause.cpp"
    ; //@line 810 "readclause.cpp"
    $for_cond$2: while(1) { 
  
      var $0=$ix; //@line 810 "readclause.cpp"
      var $1=HEAP32[((_n_soundicon_tab)>>2)]; //@line 810 "readclause.cpp"
      var $cmp=((($0))|0) < ((($1))|0); //@line 810 "readclause.cpp"
      if (!($cmp)) { __label__ = 8;break $for_cond$2; } //@line 810 "readclause.cpp"
  
      var $2=$ix; //@line 812 "readclause.cpp"
      var $arrayidx=((_soundicon_tab+$2*16)&4294967295); //@line 812 "readclause.cpp"
      var $name=(($arrayidx)&4294967295); //@line 812 "readclause.cpp"
      var $3=HEAP32[(($name)>>2)]; //@line 812 "readclause.cpp"
      var $4=$c_addr; //@line 812 "readclause.cpp"
      var $cmp1=((($3))|0)==((($4))|0); //@line 812 "readclause.cpp"
      var $5=$ix; //@line 814 "readclause.cpp"
      if ($cmp1) { __label__ = 3;break $for_cond$2; } //@line 812 "readclause.cpp"
  
      var $inc=((($5)+1)&4294967295); //@line 810 "readclause.cpp"
      $ix=$inc; //@line 810 "readclause.cpp"
      __label__ = 1;continue $for_cond$2; //@line 810 "readclause.cpp"
    }
    $for_end$$if_then$6: do { 
      if (__label__ == 8) {
  
        $retval=-1; //@line 822 "readclause.cpp"
        ; //@line 822 "readclause.cpp"
      }
      else if (__label__ == 3) {
  
        var $arrayidx2=((_soundicon_tab+$5*16)&4294967295); //@line 814 "readclause.cpp"
        var $length=(($arrayidx2+4)&4294967295); //@line 814 "readclause.cpp"
        var $6=HEAP32[(($length)>>2)]; //@line 814 "readclause.cpp"
        var $cmp3=((($6))|0)==0; //@line 814 "readclause.cpp"
        if ($cmp3) { __label__ = 4;; } else { __label__ = 6;; } //@line 814 "readclause.cpp"
        $if_then4$$if_end7$9: do { 
          if (__label__ == 4) {
  
            var $7=$ix; //@line 816 "readclause.cpp"
            var $call=__ZL13LoadSoundFilePKci(0, $7); //@line 816 "readclause.cpp"
            var $cmp5=((($call))|0)!=0; //@line 816 "readclause.cpp"
            if (!($cmp5)) { __label__ = 6;break $if_then4$$if_end7$9; } //@line 816 "readclause.cpp"
  
            $retval=-1; //@line 817 "readclause.cpp"
            __label__ = 9;break $for_end$$if_then$6; //@line 817 "readclause.cpp"
          }
        } while(0);
  
        var $8=$ix; //@line 819 "readclause.cpp"
        $retval=$8; //@line 819 "readclause.cpp"
        ; //@line 819 "readclause.cpp"
      }
    } while(0);
  
    var $9=$retval; //@line 823 "readclause.cpp"
    ;
    return $9; //@line 823 "readclause.cpp"
    return null;
  }
  

  function __ZL13LoadSoundFilePKci($fname, $index) {
    var __stackBase__  = STACKTOP; STACKTOP += 764; _memset(__stackBase__, 0, 764);
    var __label__;
  
    var $retval;
    var $fname_addr;
    var $index_addr;
    var $f;
    var $p;
    var $ip;
    var $length;
    var $fname_temp=__stackBase__;
    var $fname2=__stackBase__+100;
    var $ix;
    var $fd_temp;
    var $resample;
    var $header=__stackBase__+304;
    var $command=__stackBase__+316;
    $fname_addr=$fname;
    $index_addr=$index;
    var $0=$fname_addr; //@line 721 "readclause.cpp"
    var $cmp=((($0))|0)==0; //@line 721 "readclause.cpp"
    if ($cmp) { __label__ = 1;; } else { __label__ = 3;; } //@line 721 "readclause.cpp"
    $if_end$$if_end3$2: do { 
      if (__label__ == 1) {
  
        var $1=$index_addr; //@line 724 "readclause.cpp"
        var $arrayidx=((_soundicon_tab+$1*16)&4294967295); //@line 724 "readclause.cpp"
        var $filename=(($arrayidx+12)&4294967295); //@line 724 "readclause.cpp"
        var $2=HEAP32[(($filename)>>2)]; //@line 724 "readclause.cpp"
        $fname_addr=$2; //@line 724 "readclause.cpp"
        var $cmp1=((($2))|0)==0; //@line 727 "readclause.cpp"
        if (!($cmp1)) { __label__ = 3;break $if_end$$if_end3$2; } //@line 727 "readclause.cpp"
  
        $retval=1; //@line 728 "readclause.cpp"
        __label__ = 23;break $if_end$$if_end3$2; //@line 728 "readclause.cpp"
      }
    } while(0);
    $return$$if_end3$5: do { 
      if (__label__ == 3) {
  
        var $3=$fname_addr; //@line 730 "readclause.cpp"
        var $arrayidx4=(($3)&4294967295); //@line 730 "readclause.cpp"
        var $4=HEAP8[($arrayidx4)]; //@line 730 "readclause.cpp"
        var $conv=(tempInt=(($4)),(tempInt>=128?tempInt-256:tempInt)); //@line 730 "readclause.cpp"
        var $cmp5=((($conv))|0)!=47; //@line 730 "readclause.cpp"
        if ($cmp5) { __label__ = 4;; } else { __label__ = 5;; } //@line 730 "readclause.cpp"
        if (__label__ == 4) {
  
          var $arraydecay=(($fname2)&4294967295); //@line 733 "readclause.cpp"
          var $5=$fname_addr; //@line 733 "readclause.cpp"
          var $call=_sprintf($arraydecay, ((__str55222)&4294967295), allocate([((_path_home)&4294967295),0,0,0,47,0,0,0,47,0,0,0,$5,0,0,0], ["i8*",0,0,0,"i32",0,0,0,"i32",0,0,0,"i8*",0,0,0], ALLOC_STACK)); //@line 733 "readclause.cpp"
          var $arraydecay7=(($fname2)&4294967295); //@line 734 "readclause.cpp"
          $fname_addr=$arraydecay7; //@line 734 "readclause.cpp"
          ; //@line 735 "readclause.cpp"
        }
  
        $f=0; //@line 737 "readclause.cpp"
        var $6=$fname_addr; //@line 739 "readclause.cpp"
        var $call9=_fopen($6, ((__str18429)&4294967295)); //@line 739 "readclause.cpp"
        $f=$call9; //@line 739 "readclause.cpp"
        var $cmp10=((($call9))|0)!=0; //@line 739 "readclause.cpp"
        if ($cmp10) { __label__ = 6;; } else { __label__ = 18;; } //@line 739 "readclause.cpp"
        $if_then11$$if_then49$10: do { 
          if (__label__ == 6) {
  
            var $7=$f; //@line 747 "readclause.cpp"
            var $call12=_fseek($7, 20, 0); //@line 747 "readclause.cpp"
            $ix=0; //@line 748 "readclause.cpp"
            ; //@line 748 "readclause.cpp"
            $for_body$12: while(1) { 
  
              var $8=$f; //@line 749 "readclause.cpp"
              var $call14=__Z10Read4BytesP7__sFILE($8); //@line 749 "readclause.cpp"
              var $9=$ix; //@line 749 "readclause.cpp"
              var $arrayidx15=(($header+$9*4)&4294967295); //@line 749 "readclause.cpp"
              HEAP32[(($arrayidx15)>>2)]=$call14; //@line 749 "readclause.cpp"
              var $10=$ix; //@line 748 "readclause.cpp"
              var $inc=((($10)+1)&4294967295); //@line 748 "readclause.cpp"
              $ix=$inc; //@line 748 "readclause.cpp"
              var $cmp13=((($inc))|0) < 3; //@line 748 "readclause.cpp"
              if ($cmp13) { __label__ = 7;continue $for_body$12; } else { __label__ = 8;break $for_body$12; } //@line 748 "readclause.cpp"
            }
  
            var $arrayidx16=(($header)&4294967295); //@line 752 "readclause.cpp"
            var $11=HEAP32[(($arrayidx16)>>2)]; //@line 752 "readclause.cpp"
            var $cmp17=((($11))|0)!=65537; //@line 752 "readclause.cpp"
            if ($cmp17) { __label__ = 11;; } else { __label__ = 9;; } //@line 752 "readclause.cpp"
            $if_then23$$lor_lhs_false$15: do { 
              if (__label__ == 9) {
  
                var $arrayidx18=(($header+4)&4294967295); //@line 752 "readclause.cpp"
                var $12=HEAP32[(($arrayidx18)>>2)]; //@line 752 "readclause.cpp"
                var $13=HEAP32[((_samplerate)>>2)]; //@line 752 "readclause.cpp"
                var $cmp19=((($12))|0)!=((($13))|0); //@line 752 "readclause.cpp"
                if ($cmp19) { __label__ = 11;break $if_then23$$lor_lhs_false$15; } //@line 752 "readclause.cpp"
  
                var $arrayidx21=(($header+8)&4294967295); //@line 752 "readclause.cpp"
                var $14=HEAP32[(($arrayidx21)>>2)]; //@line 752 "readclause.cpp"
                var $15=HEAP32[((_samplerate)>>2)]; //@line 752 "readclause.cpp"
                var $mul=((($15)*2)&4294967295); //@line 752 "readclause.cpp"
                var $cmp22=((($14))|0)!=((($mul))|0); //@line 752 "readclause.cpp"
                if ($cmp22) { __label__ = 11;break $if_then23$$lor_lhs_false$15; } else { __label__ = 17;break $if_then23$$lor_lhs_false$15; } //@line 752 "readclause.cpp"
              }
            } while(0);
            $if_then23$$if_end47$18: do { 
              if (__label__ == 11) {
  
                var $16=$f; //@line 754 "readclause.cpp"
                var $call24=_fclose($16); //@line 754 "readclause.cpp"
                $f=0; //@line 755 "readclause.cpp"
                var $arrayidx25=(($header+8)&4294967295); //@line 757 "readclause.cpp"
                var $17=HEAP32[(($arrayidx25)>>2)]; //@line 757 "readclause.cpp"
                var $18=HEAP32[((_samplerate)>>2)]; //@line 757 "readclause.cpp"
                var $cmp26=((($17))|0)==((($18))|0); //@line 757 "readclause.cpp"
                if ($cmp26) { __label__ = 12;; } else { __label__ = 13;; } //@line 757 "readclause.cpp"
                if (__label__ == 12) {
  
                  $resample=((__str13643)&4294967295); //@line 758 "readclause.cpp"
                  ; //@line 758 "readclause.cpp"
                }
                else if (__label__ == 13) {
  
                  $resample=((__str57224)&4294967295); //@line 760 "readclause.cpp"
                  ;
                }
  
                var $arraydecay29=(($fname_temp)&4294967295); //@line 762 "readclause.cpp"
                var $src$, $dest$, $stop$, $stop4$, $fast$;
                $src$ = ((__str58225)&4294967295);
                $dest$ = $arraydecay29;
                $stop$ = $src$ + 18;
                if (($dest$%4) == ($src$%4) && 18 > 8) {
                  while ($src$%4 !== 0 && $src$ < $stop$) {
                    ; HEAP8[$dest$++] = HEAP8[$src$++];
                  }
                  $src$ >>= 2;
                  $dest$ >>= 2;
                  $stop4$ = $stop$ >> 2;
                  while ($src$ < $stop4$) {
                ;;;    HEAP32[$dest$++] = HEAP32[$src$++];
                  }
                  $src$ <<= 2;
                  $dest$ <<= 2;
                }while ($src$ < $stop$) {
                  ; HEAP8[$dest$++] = HEAP8[$src$++];
                }; //@line 762 "readclause.cpp"
                var $arraydecay31=(($fname_temp)&4294967295); //@line 763 "readclause.cpp"
                var $call32=_mkstemp($arraydecay31); //@line 763 "readclause.cpp"
                $fd_temp=$call32; //@line 763 "readclause.cpp"
                var $cmp33=((($call32))|0) >= 0; //@line 763 "readclause.cpp"
                if (!($cmp33)) { __label__ = 17;break $if_then23$$if_end47$18; } //@line 763 "readclause.cpp"
  
                var $19=$fd_temp; //@line 765 "readclause.cpp"
                var $call35=_close($19); //@line 765 "readclause.cpp"
                var $arraydecay36=(($command)&4294967295); //@line 767 "readclause.cpp"
                var $20=$fname_addr; //@line 767 "readclause.cpp"
                var $21=HEAP32[((_samplerate)>>2)]; //@line 767 "readclause.cpp"
                var $arraydecay37=(($fname_temp)&4294967295); //@line 767 "readclause.cpp"
                var $22=$resample; //@line 767 "readclause.cpp"
                var $call38=_sprintf($arraydecay36, ((__str59226)&4294967295), allocate([$20,0,0,0,$21,0,0,0,$arraydecay37,0,0,0,$22,0,0,0], ["i8*",0,0,0,"i32",0,0,0,"i8*",0,0,0,"i8*",0,0,0], ALLOC_STACK)); //@line 767 "readclause.cpp"
                var $arraydecay39=(($command)&4294967295); //@line 768 "readclause.cpp"
                var $call40=_system($arraydecay39); //@line 768 "readclause.cpp"
                var $cmp41=((($call40))|0)==0; //@line 768 "readclause.cpp"
                if (!($cmp41)) { __label__ = 17;break $if_then23$$if_end47$18; } //@line 768 "readclause.cpp"
  
                var $arraydecay43=(($fname_temp)&4294967295); //@line 770 "readclause.cpp"
                $fname_addr=$arraydecay43; //@line 770 "readclause.cpp"
                ; //@line 771 "readclause.cpp"
              }
            } while(0);
  
            var $_pr=$f; //@line 777 "readclause.cpp"
            var $cmp48=((($_pr))|0)==0; //@line 777 "readclause.cpp"
            if ($cmp48) { __label__ = 18;break $if_then11$$if_then49$10; } else { __label__ = 20;break $if_then11$$if_then49$10; } //@line 777 "readclause.cpp"
          }
        } while(0);
        $if_then49$$if_end54$27: do { 
          if (__label__ == 18) {
  
            var $23=$fname_addr; //@line 779 "readclause.cpp"
            var $call50=_fopen($23, ((__str18429)&4294967295)); //@line 779 "readclause.cpp"
            $f=$call50; //@line 779 "readclause.cpp"
            var $cmp51=((($call50))|0)==0; //@line 780 "readclause.cpp"
            if (!($cmp51)) { __label__ = 20;break $if_then49$$if_end54$27; } //@line 780 "readclause.cpp"
  
            $retval=3; //@line 783 "readclause.cpp"
            __label__ = 23;break $return$$if_end3$5; //@line 783 "readclause.cpp"
          }
        } while(0);
  
        var $24=$fname_addr; //@line 787 "readclause.cpp"
        var $call55=__Z13GetFileLengthPKc($24); //@line 787 "readclause.cpp"
        $length=$call55; //@line 787 "readclause.cpp"
        var $25=$f; //@line 788 "readclause.cpp"
        var $call56=_fseek($25, 0, 0); //@line 788 "readclause.cpp"
        var $26=$index_addr; //@line 789 "readclause.cpp"
        var $arrayidx57=((_soundicon_tab+$26*16)&4294967295); //@line 789 "readclause.cpp"
        var $data=(($arrayidx57+8)&4294967295); //@line 789 "readclause.cpp"
        var $27=HEAP32[(($data)>>2)]; //@line 789 "readclause.cpp"
        var $28=$length; //@line 789 "readclause.cpp"
        var $call58=_realloc($27, $28); //@line 789 "readclause.cpp"
        $p=$call58; //@line 789 "readclause.cpp"
        var $cmp59=((($call58))|0)==0; //@line 789 "readclause.cpp"
        if ($cmp59) { __label__ = 21;; } else { __label__ = 22;; } //@line 789 "readclause.cpp"
        if (__label__ == 21) {
  
          var $29=$f; //@line 791 "readclause.cpp"
          var $call61=_fclose($29); //@line 791 "readclause.cpp"
          $retval=4; //@line 792 "readclause.cpp"
          ; //@line 792 "readclause.cpp"
        }
        else if (__label__ == 22) {
  
          var $30=$p; //@line 794 "readclause.cpp"
          var $31=$length; //@line 794 "readclause.cpp"
          var $32=$f; //@line 794 "readclause.cpp"
          var $call63=_fread($30, 1, $31, $32); //@line 794 "readclause.cpp"
          $length=$call63; //@line 794 "readclause.cpp"
          var $33=$f; //@line 795 "readclause.cpp"
          var $call64=_fclose($33); //@line 795 "readclause.cpp"
          var $arraydecay65=(($fname_temp)&4294967295); //@line 796 "readclause.cpp"
          var $call66=_remove($arraydecay65); //@line 796 "readclause.cpp"
          var $34=$p; //@line 798 "readclause.cpp"
          var $arrayidx67=(($34+40)&4294967295); //@line 798 "readclause.cpp"
          var $35=$arrayidx67; //@line 798 "readclause.cpp"
          $ip=$35; //@line 798 "readclause.cpp"
          var $36=$ip; //@line 799 "readclause.cpp"
          var $37=HEAP32[(($36)>>2)]; //@line 799 "readclause.cpp"
          var $div=((((($37))|0)/2)|0); //@line 799 "readclause.cpp"
          var $38=$index_addr; //@line 799 "readclause.cpp"
          var $arrayidx68=((_soundicon_tab+$38*16)&4294967295); //@line 799 "readclause.cpp"
          var $length69=(($arrayidx68+4)&4294967295); //@line 799 "readclause.cpp"
          HEAP32[(($length69)>>2)]=$div; //@line 799 "readclause.cpp"
          var $39=$p; //@line 800 "readclause.cpp"
          var $40=$index_addr; //@line 800 "readclause.cpp"
          var $arrayidx70=((_soundicon_tab+$40*16)&4294967295); //@line 800 "readclause.cpp"
          var $data71=(($arrayidx70+8)&4294967295); //@line 800 "readclause.cpp"
          HEAP32[(($data71)>>2)]=$39; //@line 800 "readclause.cpp"
          $retval=0; //@line 801 "readclause.cpp"
          ; //@line 801 "readclause.cpp"
        }
      }
    } while(0);
  
    var $41=$retval; //@line 802 "readclause.cpp"
    STACKTOP = __stackBase__;
    return $41; //@line 802 "readclause.cpp"
    return null;
  }
  

  function __ZL14PushParamStacki($tag_type) {
    ;
    var __label__;
  
    var $tag_type_addr;
    var $ix;
    var $sp;
    $tag_type_addr=$tag_type;
    var $0=HEAP32[((__ZL13n_param_stack)>>2)]; //@line 1159 "readclause.cpp"
    var $arrayidx=((_param_stack+$0*64)&4294967295); //@line 1159 "readclause.cpp"
    $sp=$arrayidx; //@line 1159 "readclause.cpp"
    var $cmp=((($0))|0) < 19; //@line 1160 "readclause.cpp"
    if ($cmp) { __label__ = 1;; } else { __label__ = 2;; } //@line 1160 "readclause.cpp"
    if (__label__ == 1) {
  
      var $1=HEAP32[((__ZL13n_param_stack)>>2)]; //@line 1161 "readclause.cpp"
      var $inc=((($1)+1)&4294967295); //@line 1161 "readclause.cpp"
      HEAP32[((__ZL13n_param_stack)>>2)]=$inc; //@line 1161 "readclause.cpp"
      ; //@line 1161 "readclause.cpp"
    }
  
    var $2=$tag_type_addr; //@line 1163 "readclause.cpp"
    var $3=$sp; //@line 1163 "readclause.cpp"
    var $type=(($3)&4294967295); //@line 1163 "readclause.cpp"
    HEAP32[(($type)>>2)]=$2; //@line 1163 "readclause.cpp"
    $ix=0; //@line 1164 "readclause.cpp"
    ; //@line 1164 "readclause.cpp"
    while(1) { 
  
      var $4=$ix; //@line 1166 "readclause.cpp"
      var $5=$sp; //@line 1166 "readclause.cpp"
      var $parameter=(($5+4)&4294967295); //@line 1166 "readclause.cpp"
      var $arrayidx2=(($parameter+$4*4)&4294967295); //@line 1166 "readclause.cpp"
      HEAP32[(($arrayidx2)>>2)]=-1; //@line 1166 "readclause.cpp"
      var $6=$ix; //@line 1164 "readclause.cpp"
      var $inc3=((($6)+1)&4294967295); //@line 1164 "readclause.cpp"
      $ix=$inc3; //@line 1164 "readclause.cpp"
      var $cmp1=((($inc3))|0) < 15; //@line 1164 "readclause.cpp"
      if ($cmp1) { __label__ = 3;continue ; } else { __label__ = 4;break ; } //@line 1164 "readclause.cpp"
    }
  
    var $7=$sp; //@line 1168 "readclause.cpp"
    ;
    return $7; //@line 1168 "readclause.cpp"
    return null;
  }
  

  function __ZL16GetSsmlAttributePwPKc($pw, $name) {
    ;
    var __label__;
    var __lastLabel__ = null;
  
    var $retval;
    var $pw_addr;
    var $name_addr;
    var $ix;
    $pw_addr=$pw;
    $name_addr=$name;
    ; //@line 1204 "readclause.cpp"
    $while_cond$2: while(1) { 
  
      var $0=$pw_addr; //@line 1204 "readclause.cpp"
      var $1=HEAP32[(($0)>>2)]; //@line 1204 "readclause.cpp"
      var $cmp=((($1))|0)!=0; //@line 1204 "readclause.cpp"
      if (!($cmp)) { __label__ = 17;break $while_cond$2; } //@line 1204 "readclause.cpp"
  
      var $2=$pw_addr; //@line 1206 "readclause.cpp"
      var $arrayidx=(($2+-4)&4294967295); //@line 1206 "readclause.cpp"
      var $3=HEAP32[(($arrayidx)>>2)]; //@line 1206 "readclause.cpp"
      var $call=_iswspace($3); //@line 1206 "readclause.cpp"
      var $tobool=((($call))|0)!=0; //@line 1206 "readclause.cpp"
      if ($tobool) { __label__ = 3;; } else { __label__ = 16;; } //@line 1206 "readclause.cpp"
      if (__label__ == 3) {
  
        $ix=0; //@line 1208 "readclause.cpp"
        var $4=$pw_addr; //@line 1209 "readclause.cpp"
        var $5=HEAP32[(($4)>>2)]; //@line 1209 "readclause.cpp"
        var $6=$ix; //@line 1209 "readclause.cpp"
        var $7=$name_addr; //@line 1209 "readclause.cpp"
        var $arrayidx27=(($7+$6)&4294967295); //@line 1209 "readclause.cpp"
        var $8=HEAP8[($arrayidx27)]; //@line 1209 "readclause.cpp"
        var $conv8=(tempInt=(($8)),(tempInt>=128?tempInt-256:tempInt)); //@line 1209 "readclause.cpp"
        var $cmp39=((($5))|0)==((($conv8))|0); //@line 1209 "readclause.cpp"
        if ($cmp39) { __label__ = 4;; } else { __label__ = 5;; } //@line 1209 "readclause.cpp"
        $while_body4$$while_end$7: do { 
          if (__label__ == 4) {
            while(1) { 
  
              var $9=$pw_addr; //@line 1211 "readclause.cpp"
              var $incdec_ptr=(($9+4)&4294967295); //@line 1211 "readclause.cpp"
              $pw_addr=$incdec_ptr; //@line 1211 "readclause.cpp"
              var $10=$ix; //@line 1212 "readclause.cpp"
              var $inc=((($10)+1)&4294967295); //@line 1212 "readclause.cpp"
              $ix=$inc; //@line 1212 "readclause.cpp"
              var $11=$pw_addr; //@line 1209 "readclause.cpp"
              var $12=HEAP32[(($11)>>2)]; //@line 1209 "readclause.cpp"
              var $13=$ix; //@line 1209 "readclause.cpp"
              var $14=$name_addr; //@line 1209 "readclause.cpp"
              var $arrayidx2=(($14+$13)&4294967295); //@line 1209 "readclause.cpp"
              var $15=HEAP8[($arrayidx2)]; //@line 1209 "readclause.cpp"
              var $conv=(tempInt=(($15)),(tempInt>=128?tempInt-256:tempInt)); //@line 1209 "readclause.cpp"
              var $cmp3=((($12))|0)==((($conv))|0); //@line 1209 "readclause.cpp"
              if ($cmp3) { __label__ = 4;continue ; } else { __label__ = 5;break $while_body4$$while_end$7; } //@line 1209 "readclause.cpp"
            }
          }
        } while(0);
  
        var $16=$ix; //@line 1214 "readclause.cpp"
        var $17=$name_addr; //@line 1214 "readclause.cpp"
        var $arrayidx5=(($17+$16)&4294967295); //@line 1214 "readclause.cpp"
        var $18=HEAP8[($arrayidx5)]; //@line 1214 "readclause.cpp"
        var $conv6=(tempInt=(($18)),(tempInt>=128?tempInt-256:tempInt)); //@line 1214 "readclause.cpp"
        var $cmp7=((($conv6))|0)==0; //@line 1214 "readclause.cpp"
        if ($cmp7) { __label__ = 6;break $while_cond$2; } //@line 1214 "readclause.cpp"
      }
  
      var $39=$pw_addr; //@line 1226 "readclause.cpp"
      var $incdec_ptr29=(($39+4)&4294967295); //@line 1226 "readclause.cpp"
      $pw_addr=$incdec_ptr29; //@line 1226 "readclause.cpp"
      __label__ = 1;continue $while_cond$2; //@line 1227 "readclause.cpp"
    }
    $while_end30$$while_cond9_preheader$12: do { 
      if (__label__ == 17) {
  
        $retval=0; //@line 1228 "readclause.cpp"
        ; //@line 1228 "readclause.cpp"
      }
      else if (__label__ == 6) {
  
        var $19=$pw_addr; //@line 1217 "readclause.cpp"
        var $20=HEAP32[(($19)>>2)]; //@line 1217 "readclause.cpp"
        var $call104=_iswspace($20); //@line 1217 "readclause.cpp"
        var $tobool115=((($call104))|0)!=0; //@line 1217 "readclause.cpp"
        var $21=$pw_addr; //@line 1217 "readclause.cpp"
        if ($tobool115) { __lastLabel__ = 6; __label__ = 7;; } else { __lastLabel__ = 6; __label__ = 8;; } //@line 1217 "readclause.cpp"
        $while_body12$$while_end14$15: do { 
          if (__label__ == 7) {
            while(1) { 
  
              var $22=__lastLabel__ == 7 ? $25 : ($21);
              var $incdec_ptr13=(($22+4)&4294967295); //@line 1217 "readclause.cpp"
              $pw_addr=$incdec_ptr13; //@line 1217 "readclause.cpp"
              var $23=$pw_addr; //@line 1217 "readclause.cpp"
              var $24=HEAP32[(($23)>>2)]; //@line 1217 "readclause.cpp"
              var $call10=_iswspace($24); //@line 1217 "readclause.cpp"
              var $tobool11=((($call10))|0)!=0; //@line 1217 "readclause.cpp"
              var $25=$pw_addr; //@line 1217 "readclause.cpp"
              if ($tobool11) { __lastLabel__ = 7; __label__ = 7;continue ; } else { __lastLabel__ = 7; __label__ = 8;break $while_body12$$while_end14$15; } //@line 1217 "readclause.cpp"
            }
          }
        } while(0);
  
        var $_lcssa3=__lastLabel__ == 6 ? $21 : ($25);
        var $26=HEAP32[(($_lcssa3)>>2)]; //@line 1218 "readclause.cpp"
        var $cmp15=((($26))|0)==61; //@line 1218 "readclause.cpp"
        if ($cmp15) { __label__ = 9;; } else { __label__ = 10;; } //@line 1218 "readclause.cpp"
        if (__label__ == 9) {
  
          var $27=$pw_addr; //@line 1218 "readclause.cpp"
          var $incdec_ptr17=(($27+4)&4294967295); //@line 1218 "readclause.cpp"
          $pw_addr=$incdec_ptr17; //@line 1218 "readclause.cpp"
          ; //@line 1218 "readclause.cpp"
        }
  
        var $28=$pw_addr; //@line 1219 "readclause.cpp"
        var $29=HEAP32[(($28)>>2)]; //@line 1219 "readclause.cpp"
        var $call191=_iswspace($29); //@line 1219 "readclause.cpp"
        var $tobool202=((($call191))|0)!=0; //@line 1219 "readclause.cpp"
        var $30=$pw_addr; //@line 1219 "readclause.cpp"
        if ($tobool202) { __lastLabel__ = 10; __label__ = 11;; } else { __lastLabel__ = 10; __label__ = 12;; } //@line 1219 "readclause.cpp"
        $while_body21$$while_end23$22: do { 
          if (__label__ == 11) {
            while(1) { 
  
              var $31=__lastLabel__ == 11 ? $34 : ($30);
              var $incdec_ptr22=(($31+4)&4294967295); //@line 1219 "readclause.cpp"
              $pw_addr=$incdec_ptr22; //@line 1219 "readclause.cpp"
              var $32=$pw_addr; //@line 1219 "readclause.cpp"
              var $33=HEAP32[(($32)>>2)]; //@line 1219 "readclause.cpp"
              var $call19=_iswspace($33); //@line 1219 "readclause.cpp"
              var $tobool20=((($call19))|0)!=0; //@line 1219 "readclause.cpp"
              var $34=$pw_addr; //@line 1219 "readclause.cpp"
              if ($tobool20) { __lastLabel__ = 11; __label__ = 11;continue ; } else { __lastLabel__ = 11; __label__ = 12;break $while_body21$$while_end23$22; } //@line 1219 "readclause.cpp"
            }
          }
        } while(0);
  
        var $_lcssa=__lastLabel__ == 10 ? $30 : ($34);
        var $35=HEAP32[(($_lcssa)>>2)]; //@line 1220 "readclause.cpp"
        var $cmp24=((($35))|0)==34; //@line 1220 "readclause.cpp"
        if ($cmp24) { __label__ = 14;; } else { __label__ = 13;; } //@line 1220 "readclause.cpp"
        $if_then26$$lor_lhs_false$26: do { 
          if (__label__ == 13) {
  
            var $36=$pw_addr; //@line 1220 "readclause.cpp"
            var $37=HEAP32[(($36)>>2)]; //@line 1220 "readclause.cpp"
            var $cmp25=((($37))|0)==39; //@line 1220 "readclause.cpp"
            if ($cmp25) { __label__ = 14;break $if_then26$$lor_lhs_false$26; } //@line 1220 "readclause.cpp"
  
            $retval=((__ZZL16GetSsmlAttributePwPKcE5empty)&4294967295); //@line 1223 "readclause.cpp"
            __label__ = 18;break $while_end30$$while_cond9_preheader$12; //@line 1223 "readclause.cpp"
          }
        } while(0);
  
        var $38=$pw_addr; //@line 1221 "readclause.cpp"
        var $add_ptr=(($38+4)&4294967295); //@line 1221 "readclause.cpp"
        $retval=$add_ptr; //@line 1221 "readclause.cpp"
        ; //@line 1221 "readclause.cpp"
      }
    } while(0);
  
    var $40=$retval; //@line 1229 "readclause.cpp"
    ;
    return $40; //@line 1229 "readclause.cpp"
    return null;
  }
  

  function __ZL7attrcmpPKwPKc($string1, $string2) {
    ;
    var __label__;
  
    var $retval;
    var $string1_addr;
    var $string2_addr;
    var $ix;
    $string1_addr=$string1;
    $string2_addr=$string2;
    var $0=$string1_addr; //@line 1236 "readclause.cpp"
    var $cmp=((($0))|0)==0; //@line 1236 "readclause.cpp"
    if ($cmp) { __label__ = 1;; } else { __label__ = 2;; } //@line 1236 "readclause.cpp"
    $if_then$$if_end$2: do { 
      if (__label__ == 1) {
  
        $retval=1; //@line 1237 "readclause.cpp"
        ; //@line 1237 "readclause.cpp"
      }
      else if (__label__ == 2) {
  
        $ix=0; //@line 1239 "readclause.cpp"
        ; //@line 1239 "readclause.cpp"
        $for_cond$5: while(1) { 
  
          var $1=$ix; //@line 1239 "readclause.cpp"
          var $2=$string1_addr; //@line 1239 "readclause.cpp"
          var $arrayidx=(($2+4*$1)&4294967295); //@line 1239 "readclause.cpp"
          var $3=HEAP32[(($arrayidx)>>2)]; //@line 1239 "readclause.cpp"
          var $4=$ix; //@line 1239 "readclause.cpp"
          var $5=$string2_addr; //@line 1239 "readclause.cpp"
          var $arrayidx1=(($5+$4)&4294967295); //@line 1239 "readclause.cpp"
          var $6=HEAP8[($arrayidx1)]; //@line 1239 "readclause.cpp"
          var $conv=(tempInt=(($6)),(tempInt>=128?tempInt-256:tempInt)); //@line 1239 "readclause.cpp"
          var $cmp2=((($3))|0)==((($conv))|0); //@line 1239 "readclause.cpp"
          if (!($cmp2)) { __label__ = 6;break $for_cond$5; } //@line 1239 "readclause.cpp"
  
          var $7=$ix; //@line 1239 "readclause.cpp"
          var $8=$string1_addr; //@line 1239 "readclause.cpp"
          var $arrayidx3=(($8+4*$7)&4294967295); //@line 1239 "readclause.cpp"
          var $9=HEAP32[(($arrayidx3)>>2)]; //@line 1239 "readclause.cpp"
          var $cmp4=((($9))|0)!=0; //@line 1239 "readclause.cpp"
          if (!($cmp4)) { __label__ = 6;break $for_cond$5; }
  
          var $10=$ix; //@line 1239 "readclause.cpp"
          var $inc=((($10)+1)&4294967295); //@line 1239 "readclause.cpp"
          $ix=$inc; //@line 1239 "readclause.cpp"
          __label__ = 3;continue $for_cond$5; //@line 1239 "readclause.cpp"
        }
  
        var $11=$ix; //@line 1242 "readclause.cpp"
        var $12=$string1_addr; //@line 1242 "readclause.cpp"
        var $arrayidx5=(($12+4*$11)&4294967295); //@line 1242 "readclause.cpp"
        var $13=HEAP32[(($arrayidx5)>>2)]; //@line 1242 "readclause.cpp"
        var $cmp6=((($13))|0)==34; //@line 1242 "readclause.cpp"
        if ($cmp6) { __label__ = 8;; } else { __label__ = 7;; } //@line 1242 "readclause.cpp"
        $land_lhs_true$$lor_lhs_false$10: do { 
          if (__label__ == 7) {
  
            var $14=$ix; //@line 1242 "readclause.cpp"
            var $15=$string1_addr; //@line 1242 "readclause.cpp"
            var $arrayidx7=(($15+4*$14)&4294967295); //@line 1242 "readclause.cpp"
            var $16=HEAP32[(($arrayidx7)>>2)]; //@line 1242 "readclause.cpp"
            var $cmp8=((($16))|0)==39; //@line 1242 "readclause.cpp"
            if ($cmp8) { __label__ = 8;break $land_lhs_true$$lor_lhs_false$10; } else { __label__ = 10;break $land_lhs_true$$lor_lhs_false$10; } //@line 1242 "readclause.cpp"
          }
        } while(0);
        $land_lhs_true$$if_end13$12: do { 
          if (__label__ == 8) {
  
            var $17=$ix; //@line 1242 "readclause.cpp"
            var $18=$string2_addr; //@line 1242 "readclause.cpp"
            var $arrayidx9=(($18+$17)&4294967295); //@line 1242 "readclause.cpp"
            var $19=HEAP8[($arrayidx9)]; //@line 1242 "readclause.cpp"
            var $conv10=(tempInt=(($19)),(tempInt>=128?tempInt-256:tempInt)); //@line 1242 "readclause.cpp"
            var $cmp11=((($conv10))|0)==0; //@line 1242 "readclause.cpp"
            if (!($cmp11)) { __label__ = 10;break $land_lhs_true$$if_end13$12; } //@line 1242 "readclause.cpp"
  
            $retval=0; //@line 1243 "readclause.cpp"
            __label__ = 11;break $if_then$$if_end$2; //@line 1243 "readclause.cpp"
          }
        } while(0);
  
        $retval=1; //@line 1244 "readclause.cpp"
        ; //@line 1244 "readclause.cpp"
      }
    } while(0);
  
    var $20=$retval; //@line 1245 "readclause.cpp"
    ;
    return $20; //@line 1245 "readclause.cpp"
    return null;
  }
  

  function __ZL10attrlookupPKwPK8MNEM_TAB($string1, $mtab) {
    ;
    var __label__;
  
    var $retval;
    var $string1_addr;
    var $mtab_addr;
    var $ix;
    $string1_addr=$string1;
    $mtab_addr=$mtab;
    $ix=0; //@line 1252 "readclause.cpp"
    ; //@line 1252 "readclause.cpp"
    while(1) { 
  
      var $0=$ix; //@line 1252 "readclause.cpp"
      var $1=$mtab_addr; //@line 1252 "readclause.cpp"
      var $arrayidx=(($1+8*$0)&4294967295); //@line 1252 "readclause.cpp"
      var $mnem=(($arrayidx)&4294967295); //@line 1252 "readclause.cpp"
      var $2=HEAP32[(($mnem)>>2)]; //@line 1252 "readclause.cpp"
      var $cmp=((($2))|0)!=0; //@line 1252 "readclause.cpp"
      if (!($cmp)) { __label__ = 5;break ; } //@line 1252 "readclause.cpp"
  
      var $3=$string1_addr; //@line 1254 "readclause.cpp"
      var $4=$ix; //@line 1254 "readclause.cpp"
      var $5=$mtab_addr; //@line 1254 "readclause.cpp"
      var $arrayidx1=(($5+8*$4)&4294967295); //@line 1254 "readclause.cpp"
      var $mnem2=(($arrayidx1)&4294967295); //@line 1254 "readclause.cpp"
      var $6=HEAP32[(($mnem2)>>2)]; //@line 1254 "readclause.cpp"
      var $call=__ZL7attrcmpPKwPKc($3, $6); //@line 1254 "readclause.cpp"
      var $cmp3=((($call))|0)==0; //@line 1254 "readclause.cpp"
      var $7=$ix; //@line 1255 "readclause.cpp"
      if ($cmp3) { __label__ = 3;break ; } //@line 1254 "readclause.cpp"
  
      var $inc=((($7)+1)&4294967295); //@line 1252 "readclause.cpp"
      $ix=$inc; //@line 1252 "readclause.cpp"
      __label__ = 1;continue ; //@line 1252 "readclause.cpp"
    }
    if (__label__ == 5) {
  
      var $10=$ix; //@line 1257 "readclause.cpp"
      var $11=$mtab_addr; //@line 1257 "readclause.cpp"
      var $arrayidx5=(($11+8*$10)&4294967295); //@line 1257 "readclause.cpp"
      var $value6=(($arrayidx5+4)&4294967295); //@line 1257 "readclause.cpp"
      var $12=HEAP32[(($value6)>>2)]; //@line 1257 "readclause.cpp"
      $retval=$12; //@line 1257 "readclause.cpp"
      ; //@line 1257 "readclause.cpp"
    }
    else if (__label__ == 3) {
  
      var $8=$mtab_addr; //@line 1255 "readclause.cpp"
      var $arrayidx4=(($8+8*$7)&4294967295); //@line 1255 "readclause.cpp"
      var $value=(($arrayidx4+4)&4294967295); //@line 1255 "readclause.cpp"
      var $9=HEAP32[(($value)>>2)]; //@line 1255 "readclause.cpp"
      $retval=$9; //@line 1255 "readclause.cpp"
      ; //@line 1255 "readclause.cpp"
    }
  
    var $13=$retval; //@line 1258 "readclause.cpp"
    ;
    return $13; //@line 1258 "readclause.cpp"
    return null;
  }
  

  function __ZL17ProcessParamStackPcRi($outbuf, $outix) {
    var __stackBase__  = STACKTOP; STACKTOP += 80; _memset(__stackBase__, 0, 80);
    var __label__;
  
    var $outbuf_addr;
    var $outix_addr;
    var $param;
    var $ix;
    var $value;
    var $buf=__stackBase__;
    var $new_parameters=__stackBase__+20;
    $outbuf_addr=$outbuf;
    $outix_addr=$outix;
    $param=0; //@line 1109 "readclause.cpp"
    ; //@line 1109 "readclause.cpp"
    $for_body$2: while(1) { 
  
      var $0=$param; //@line 1110 "readclause.cpp"
      var $arrayidx=(($new_parameters+$0*4)&4294967295); //@line 1110 "readclause.cpp"
      HEAP32[(($arrayidx)>>2)]=-1; //@line 1110 "readclause.cpp"
      var $1=$param; //@line 1109 "readclause.cpp"
      var $inc=((($1)+1)&4294967295); //@line 1109 "readclause.cpp"
      $param=$inc; //@line 1109 "readclause.cpp"
      var $cmp=((($inc))|0) < 15; //@line 1109 "readclause.cpp"
      if ($cmp) { __label__ = 1;continue $for_body$2; } else { __label__ = 2;break $for_body$2; } //@line 1109 "readclause.cpp"
    }
  
    $ix=0; //@line 1112 "readclause.cpp"
    var $2=$ix; //@line 1112 "readclause.cpp"
    var $3=HEAP32[((__ZL13n_param_stack)>>2)]; //@line 1112 "readclause.cpp"
    var $cmp22=((($2))|0) < ((($3))|0); //@line 1112 "readclause.cpp"
    $param=0; //@line 1114 "readclause.cpp"
    if ($cmp22) { __label__ = 4;; } else { __label__ = 3;; } //@line 1112 "readclause.cpp"
    $for_body6$$for_cond20_preheader$5: do { 
      if (__label__ == 4) {
        while(1) { 
  
          var $4=$param; //@line 1116 "readclause.cpp"
          var $5=$ix; //@line 1116 "readclause.cpp"
          var $arrayidx7=((_param_stack+$5*64)&4294967295); //@line 1116 "readclause.cpp"
          var $parameter=(($arrayidx7+4)&4294967295); //@line 1116 "readclause.cpp"
          var $arrayidx8=(($parameter+$4*4)&4294967295); //@line 1116 "readclause.cpp"
          var $6=HEAP32[(($arrayidx8)>>2)]; //@line 1116 "readclause.cpp"
          var $cmp9=((($6))|0) >= 0; //@line 1116 "readclause.cpp"
          if ($cmp9) { __label__ = 5;; } else { __label__ = 6;; } //@line 1116 "readclause.cpp"
          if (__label__ == 5) {
  
            var $7=$param; //@line 1117 "readclause.cpp"
            var $8=$ix; //@line 1117 "readclause.cpp"
            var $arrayidx10=((_param_stack+$8*64)&4294967295); //@line 1117 "readclause.cpp"
            var $parameter11=(($arrayidx10+4)&4294967295); //@line 1117 "readclause.cpp"
            var $arrayidx12=(($parameter11+$7*4)&4294967295); //@line 1117 "readclause.cpp"
            var $9=HEAP32[(($arrayidx12)>>2)]; //@line 1117 "readclause.cpp"
            var $10=$param; //@line 1117 "readclause.cpp"
            var $arrayidx13=(($new_parameters+$10*4)&4294967295); //@line 1117 "readclause.cpp"
            HEAP32[(($arrayidx13)>>2)]=$9; //@line 1117 "readclause.cpp"
            ; //@line 1117 "readclause.cpp"
          }
  
          var $11=$param; //@line 1114 "readclause.cpp"
          var $inc15=((($11)+1)&4294967295); //@line 1114 "readclause.cpp"
          $param=$inc15; //@line 1114 "readclause.cpp"
          var $cmp5=((($inc15))|0) < 15; //@line 1114 "readclause.cpp"
          if ($cmp5) { __label__ = 4;continue ; } //@line 1114 "readclause.cpp"
  
          var $12=$ix; //@line 1112 "readclause.cpp"
          var $inc18=((($12)+1)&4294967295); //@line 1112 "readclause.cpp"
          $ix=$inc18; //@line 1112 "readclause.cpp"
          var $13=$ix; //@line 1112 "readclause.cpp"
          var $14=HEAP32[((__ZL13n_param_stack)>>2)]; //@line 1112 "readclause.cpp"
          var $cmp2=((($13))|0) < ((($14))|0); //@line 1112 "readclause.cpp"
          $param=0; //@line 1114 "readclause.cpp"
          if ($cmp2) { __label__ = 4;continue ; } else { __label__ = 3;break $for_body6$$for_cond20_preheader$5; } //@line 1112 "readclause.cpp"
        }
      }
    } while(0);
  
    var $arrayidx27=(($buf)&4294967295); //@line 1125 "readclause.cpp"
    var $arraydecay34=(($buf)&4294967295); //@line 1147 "readclause.cpp"
    var $arraydecay36=(($buf)&4294967295); //@line 1148 "readclause.cpp"
    var $arraydecay=(($buf)&4294967295); //@line 1142 "readclause.cpp"
    ; //@line 1121 "readclause.cpp"
    while(1) { 
  
      var $15=$param; //@line 1123 "readclause.cpp"
      var $arrayidx23=(($new_parameters+$15*4)&4294967295); //@line 1123 "readclause.cpp"
      var $16=HEAP32[(($arrayidx23)>>2)]; //@line 1123 "readclause.cpp"
      $value=$16; //@line 1123 "readclause.cpp"
      var $17=$param; //@line 1123 "readclause.cpp"
      var $arrayidx24=((__ZL17speech_parameters+$17*4)&4294967295); //@line 1123 "readclause.cpp"
      var $18=HEAP32[(($arrayidx24)>>2)]; //@line 1123 "readclause.cpp"
      var $cmp25=((($16))|0)!=((($18))|0); //@line 1123 "readclause.cpp"
      if ($cmp25) { __label__ = 9;; } else { __label__ = 14;; } //@line 1123 "readclause.cpp"
      if (__label__ == 9) {
  
        HEAP8[($arrayidx27)]=0; //@line 1125 "readclause.cpp"
        var $19=$param; //@line 1127 "readclause.cpp"
        if ($19 == 5) {
          __label__ = 10;;
        }
        else if ($19 == 6) {
          __label__ = 11;;
        }
        else if ($19 == 1) {
          __label__ = 12;;
        }
        else if ($19 == 2) {
          __label__ = 12;;
        }
        else if ($19 == 3) {
          __label__ = 12;;
        }
        else if ($19 == 4) {
          __label__ = 12;;
        }
        else if ($19 == 12) {
          __label__ = 12;;
        }
        else {
        __label__ = 13;;
        }
        
        if (__label__ == 10) {
  
          var $20=$value; //@line 1130 "readclause.cpp"
          var $sub=((($20)-1)&4294967295); //@line 1130 "readclause.cpp"
          HEAP32[((_option_punctuation)>>2)]=$sub; //@line 1130 "readclause.cpp"
          ; //@line 1131 "readclause.cpp"
        }
        else if (__label__ == 11) {
  
          var $21=$value; //@line 1134 "readclause.cpp"
          HEAP32[((_option_capitals)>>2)]=$21; //@line 1134 "readclause.cpp"
          ; //@line 1135 "readclause.cpp"
        }
        else if (__label__ == 12) {
  
          var $22=$value; //@line 1142 "readclause.cpp"
          var $23=$param; //@line 1142 "readclause.cpp"
          var $arrayidx30=((__ZZL17ProcessParamStackPcRiE10cmd_letter+$23)&4294967295); //@line 1142 "readclause.cpp"
          var $24=HEAP8[($arrayidx30)]; //@line 1142 "readclause.cpp"
          var $conv=(tempInt=(($24)),(tempInt>=128?tempInt-256:tempInt)); //@line 1142 "readclause.cpp"
          var $call=_sprintf($arraydecay, ((__str128)&4294967295), allocate([1,0,0,0,$22,0,0,0,$conv,0,0,0], ["i32",0,0,0,"i32",0,0,0,"i32",0,0,0], ALLOC_STACK)); //@line 1142 "readclause.cpp"
          ; //@line 1143 "readclause.cpp"
        }
  
        var $25=$param; //@line 1146 "readclause.cpp"
        var $arrayidx31=(($new_parameters+$25*4)&4294967295); //@line 1146 "readclause.cpp"
        var $26=HEAP32[(($arrayidx31)>>2)]; //@line 1146 "readclause.cpp"
        var $27=$param; //@line 1146 "readclause.cpp"
        var $arrayidx32=((__ZL17speech_parameters+$27*4)&4294967295); //@line 1146 "readclause.cpp"
        HEAP32[(($arrayidx32)>>2)]=$26; //@line 1146 "readclause.cpp"
        var $28=$outix_addr; //@line 1147 "readclause.cpp"
        var $29=HEAP32[(($28)>>2)]; //@line 1147 "readclause.cpp"
        var $30=$outbuf_addr; //@line 1147 "readclause.cpp"
        var $arrayidx33=(($30+$29)&4294967295); //@line 1147 "readclause.cpp"
        var $call35=_strcpy($arrayidx33, $arraydecay34); //@line 1147 "readclause.cpp"
        var $call37=_strlen($arraydecay36); //@line 1148 "readclause.cpp"
        var $31=$outix_addr; //@line 1148 "readclause.cpp"
        var $32=HEAP32[(($31)>>2)]; //@line 1148 "readclause.cpp"
        var $add=((($32)+($call37))&4294967295); //@line 1148 "readclause.cpp"
        HEAP32[(($31)>>2)]=$add; //@line 1148 "readclause.cpp"
        ; //@line 1149 "readclause.cpp"
      }
  
      var $33=$param; //@line 1121 "readclause.cpp"
      var $inc40=((($33)+1)&4294967295); //@line 1121 "readclause.cpp"
      $param=$inc40; //@line 1121 "readclause.cpp"
      var $cmp21=((($inc40))|0) < 15; //@line 1121 "readclause.cpp"
      if ($cmp21) { __label__ = 8;continue ; } else { __label__ = 15;break ; } //@line 1121 "readclause.cpp"
    }
  
    STACKTOP = __stackBase__;
    return; //@line 1151 "readclause.cpp"
    return;
  }
  

  function __ZL19SetProsodyParameteriPwP11PARAM_STACK($param_type, $attr1, $sp) {
    var __stackBase__  = STACKTOP; STACKTOP += 4; _memset(__stackBase__, 0, 4);
    var __label__;
  
    var $param_type_addr;
    var $attr1_addr;
    var $sp_addr;
    var $value=__stackBase__;
    var $sign;
    $param_type_addr=$param_type;
    $attr1_addr=$attr1;
    $sp_addr=$sp;
    var $0=$attr1_addr; //@line 1538 "readclause.cpp"
    var $1=$param_type_addr; //@line 1538 "readclause.cpp"
    var $arrayidx=((__ZZL19SetProsodyParameteriPwP11PARAM_STACKE9mnem_tabs+$1*4)&4294967295); //@line 1538 "readclause.cpp"
    var $2=HEAP32[(($arrayidx)>>2)]; //@line 1538 "readclause.cpp"
    var $call=__ZL10attrlookupPKwPK8MNEM_TAB($0, $2); //@line 1538 "readclause.cpp"
    HEAP32[(($value)>>2)]=$call; //@line 1538 "readclause.cpp"
    var $cmp=((($call))|0) >= 0; //@line 1538 "readclause.cpp"
    var $3=$param_type_addr; //@line 1541 "readclause.cpp"
    if ($cmp) { __label__ = 1;; } else { __label__ = 2;; } //@line 1538 "readclause.cpp"
    if (__label__ == 1) {
  
      var $arrayidx1=((((_param_stack+4)&4294967295)+$3*4)&4294967295); //@line 1541 "readclause.cpp"
      var $4=HEAP32[(($arrayidx1)>>2)]; //@line 1541 "readclause.cpp"
      var $5=HEAP32[(($value)>>2)]; //@line 1541 "readclause.cpp"
      var $mul=((($5)*($4))&4294967295); //@line 1541 "readclause.cpp"
      var $div=((((($mul))|0)/100)|0); //@line 1541 "readclause.cpp"
      var $6=$param_type_addr; //@line 1541 "readclause.cpp"
      var $7=$sp_addr; //@line 1541 "readclause.cpp"
      var $parameter=(($7+4)&4294967295); //@line 1541 "readclause.cpp"
      var $arrayidx2=(($parameter+$6*4)&4294967295); //@line 1541 "readclause.cpp"
      HEAP32[(($arrayidx2)>>2)]=$div; //@line 1541 "readclause.cpp"
      ; //@line 1542 "readclause.cpp"
    }
    else if (__label__ == 2) {
  
      var $8=$attr1_addr; //@line 1545 "readclause.cpp"
      var $call3=__ZL18attr_prosody_valueiPKwPi($3, $8, $value); //@line 1545 "readclause.cpp"
      $sign=$call3; //@line 1545 "readclause.cpp"
      var $9=$sign; //@line 1547 "readclause.cpp"
      var $cmp4=((($9))|0)==0; //@line 1547 "readclause.cpp"
      if ($cmp4) { __label__ = 3;; } else { __label__ = 4;; } //@line 1547 "readclause.cpp"
      if (__label__ == 3) {
  
        var $10=HEAP32[(($value)>>2)]; //@line 1548 "readclause.cpp"
        var $11=$param_type_addr; //@line 1548 "readclause.cpp"
        var $12=$sp_addr; //@line 1548 "readclause.cpp"
        var $parameter6=(($12+4)&4294967295); //@line 1548 "readclause.cpp"
        var $arrayidx7=(($parameter6+$11*4)&4294967295); //@line 1548 "readclause.cpp"
        HEAP32[(($arrayidx7)>>2)]=$10; //@line 1548 "readclause.cpp"
        ; //@line 1548 "readclause.cpp"
      }
      else if (__label__ == 4) {
  
        var $13=$sign; //@line 1550 "readclause.cpp"
        var $cmp9=((($13))|0)==2; //@line 1550 "readclause.cpp"
        var $14=$param_type_addr; //@line 1553 "readclause.cpp"
        var $arrayidx11=((__ZL17speech_parameters+$14*4)&4294967295); //@line 1553 "readclause.cpp"
        var $15=HEAP32[(($arrayidx11)>>2)]; //@line 1553 "readclause.cpp"
        var $16=HEAP32[(($value)>>2)]; //@line 1553 "readclause.cpp"
        if ($cmp9) { __label__ = 5;; } else { __label__ = 6;; } //@line 1550 "readclause.cpp"
        if (__label__ == 5) {
  
          var $mul12=((($16)*($15))&4294967295); //@line 1553 "readclause.cpp"
          var $div13=((((($mul12))|0)/100)|0); //@line 1553 "readclause.cpp"
          var $17=$param_type_addr; //@line 1553 "readclause.cpp"
          var $18=$sp_addr; //@line 1553 "readclause.cpp"
          var $parameter14=(($18+4)&4294967295); //@line 1553 "readclause.cpp"
          var $arrayidx15=(($parameter14+$17*4)&4294967295); //@line 1553 "readclause.cpp"
          HEAP32[(($arrayidx15)>>2)]=$div13; //@line 1553 "readclause.cpp"
          ; //@line 1554 "readclause.cpp"
        }
        else if (__label__ == 6) {
  
          var $19=$sign; //@line 1558 "readclause.cpp"
          var $mul18=((($19)*($16))&4294967295); //@line 1558 "readclause.cpp"
          var $add=((($mul18)+($15))&4294967295); //@line 1558 "readclause.cpp"
          var $20=$param_type_addr; //@line 1558 "readclause.cpp"
          var $21=$sp_addr; //@line 1558 "readclause.cpp"
          var $parameter19=(($21+4)&4294967295); //@line 1558 "readclause.cpp"
          var $arrayidx20=(($parameter19+$20*4)&4294967295); //@line 1558 "readclause.cpp"
          HEAP32[(($arrayidx20)>>2)]=$add; //@line 1558 "readclause.cpp"
          ;
        }
      }
    }
  
    STACKTOP = __stackBase__;
    return; //@line 1561 "readclause.cpp"
    return;
  }
  

  function __ZL13PopParamStackiPcRi($tag_type, $outbuf, $outix) {
    ;
    var __label__;
  
    var $tag_type_addr;
    var $outbuf_addr;
    var $outix_addr;
    var $ix;
    var $top;
    $tag_type_addr=$tag_type;
    $outbuf_addr=$outbuf;
    $outix_addr=$outix;
    $top=0; //@line 1176 "readclause.cpp"
    var $0=$tag_type_addr; //@line 1178 "readclause.cpp"
    var $cmp=((($0))|0) >= 32; //@line 1178 "readclause.cpp"
    if ($cmp) { __label__ = 1;; } else { __label__ = 2;; } //@line 1178 "readclause.cpp"
    if (__label__ == 1) {
  
      var $1=$tag_type_addr; //@line 1179 "readclause.cpp"
      var $sub=((($1)-32)&4294967295); //@line 1179 "readclause.cpp"
      $tag_type_addr=$sub; //@line 1179 "readclause.cpp"
      ; //@line 1179 "readclause.cpp"
    }
  
    $ix=0; //@line 1181 "readclause.cpp"
    var $2=$ix; //@line 1181 "readclause.cpp"
    var $3=HEAP32[((__ZL13n_param_stack)>>2)]; //@line 1181 "readclause.cpp"
    var $cmp11=((($2))|0) < ((($3))|0); //@line 1181 "readclause.cpp"
    if ($cmp11) { __label__ = 3;; } else { __label__ = 6;; } //@line 1181 "readclause.cpp"
    $for_body$$for_end$5: do { 
      if (__label__ == 3) {
        while(1) { 
  
          var $4=$ix; //@line 1183 "readclause.cpp"
          var $arrayidx=((_param_stack+$4*64)&4294967295); //@line 1183 "readclause.cpp"
          var $type=(($arrayidx)&4294967295); //@line 1183 "readclause.cpp"
          var $5=HEAP32[(($type)>>2)]; //@line 1183 "readclause.cpp"
          var $6=$tag_type_addr; //@line 1183 "readclause.cpp"
          var $cmp2=((($5))|0)==((($6))|0); //@line 1183 "readclause.cpp"
          if ($cmp2) { __label__ = 4;; } else { __label__ = 5;; } //@line 1183 "readclause.cpp"
          if (__label__ == 4) {
  
            var $7=$ix; //@line 1185 "readclause.cpp"
            $top=$7; //@line 1185 "readclause.cpp"
            ; //@line 1186 "readclause.cpp"
          }
  
          var $8=$ix; //@line 1181 "readclause.cpp"
          var $inc=((($8)+1)&4294967295); //@line 1181 "readclause.cpp"
          $ix=$inc; //@line 1181 "readclause.cpp"
          var $9=$ix; //@line 1181 "readclause.cpp"
          var $10=HEAP32[((__ZL13n_param_stack)>>2)]; //@line 1181 "readclause.cpp"
          var $cmp1=((($9))|0) < ((($10))|0); //@line 1181 "readclause.cpp"
          if ($cmp1) { __label__ = 3;continue ; } else { __label__ = 6;break $for_body$$for_end$5; } //@line 1181 "readclause.cpp"
        }
      }
    } while(0);
  
    var $11=$top; //@line 1188 "readclause.cpp"
    var $cmp5=((($11))|0) > 0; //@line 1188 "readclause.cpp"
    if ($cmp5) { __label__ = 7;; } else { __label__ = 8;; } //@line 1188 "readclause.cpp"
    if (__label__ == 7) {
  
      var $12=$top; //@line 1190 "readclause.cpp"
      HEAP32[((__ZL13n_param_stack)>>2)]=$12; //@line 1190 "readclause.cpp"
      ; //@line 1191 "readclause.cpp"
    }
  
    var $13=$outbuf_addr; //@line 1192 "readclause.cpp"
    var $14=$outix_addr; //@line 1192 "readclause.cpp"
    __ZL17ProcessParamStackPcRi($13, $14); //@line 1192 "readclause.cpp"
    ;
    return; //@line 1193 "readclause.cpp"
    return;
  }
  

  function __ZL10attrnumberPKwii($pw, $default_value, $type) {
    ;
    var __label__;
  
    var $retval;
    var $pw_addr;
    var $default_value_addr;
    var $type_addr;
    var $value;
    $pw_addr=$pw;
    $default_value_addr=$default_value;
    $type_addr=$type;
    $value=0; //@line 1263 "readclause.cpp"
    var $0=$pw_addr; //@line 1265 "readclause.cpp"
    var $cmp=((($0))|0)==0; //@line 1265 "readclause.cpp"
    if ($cmp) { __label__ = 3;; } else { __label__ = 1;; } //@line 1265 "readclause.cpp"
    $if_then$$lor_lhs_false$2: do { 
      if (__label__ == 1) {
  
        var $1=$pw_addr; //@line 1265 "readclause.cpp"
        var $2=HEAP32[(($1)>>2)]; //@line 1265 "readclause.cpp"
        var $isdigittmp=((($2)-48)&4294967295); //@line 1265 "readclause.cpp"
        var $isdigit=((($isdigittmp))>>>0) < 10; //@line 1265 "readclause.cpp"
        var $call=((($isdigit))&1); //@line 1265 "readclause.cpp"
        var $tobool=((($call))|0)!=0; //@line 1265 "readclause.cpp"
        if (!($tobool)) { __label__ = 3;break $if_then$$lor_lhs_false$2; } //@line 1265 "readclause.cpp"
  
        var $3=$pw_addr; //@line 1268 "readclause.cpp"
        var $4=HEAP32[(($3)>>2)]; //@line 1268 "readclause.cpp"
        var $isdigittmp13=((($4)-48)&4294967295); //@line 1268 "readclause.cpp"
        var $isdigit24=((($isdigittmp13))>>>0) < 10; //@line 1268 "readclause.cpp"
        if ($isdigit24) { __label__ = 4;; } else { __label__ = 5;; } //@line 1268 "readclause.cpp"
        $while_body$$while_end$5: do { 
          if (__label__ == 4) {
            while(1) { 
  
              var $6=$value; //@line 1270 "readclause.cpp"
              var $mul=((($6)*10)&4294967295); //@line 1270 "readclause.cpp"
              var $7=$pw_addr; //@line 1270 "readclause.cpp"
              var $incdec_ptr=(($7+4)&4294967295); //@line 1270 "readclause.cpp"
              $pw_addr=$incdec_ptr; //@line 1270 "readclause.cpp"
              var $8=HEAP32[(($7)>>2)]; //@line 1270 "readclause.cpp"
              var $add=((($mul)-48)&4294967295); //@line 1270 "readclause.cpp"
              var $sub=((($add)+($8))&4294967295); //@line 1270 "readclause.cpp"
              $value=$sub; //@line 1270 "readclause.cpp"
              var $9=$pw_addr; //@line 1268 "readclause.cpp"
              var $10=HEAP32[(($9)>>2)]; //@line 1268 "readclause.cpp"
              var $isdigittmp1=((($10)-48)&4294967295); //@line 1268 "readclause.cpp"
              var $isdigit2=((($isdigittmp1))>>>0) < 10; //@line 1268 "readclause.cpp"
              var $call1=((($isdigit2))&1); //@line 1268 "readclause.cpp"
              var $tobool2=((($call1))|0)!=0; //@line 1268 "readclause.cpp"
              if ($tobool2) { __label__ = 4;continue ; } else { __label__ = 5;break $while_body$$while_end$5; } //@line 1268 "readclause.cpp"
            }
          }
        } while(0);
  
        var $11=$type_addr; //@line 1272 "readclause.cpp"
        var $cmp3=((($11))|0)==1; //@line 1272 "readclause.cpp"
        if ($cmp3) { __label__ = 6;; } else { __label__ = 8;; } //@line 1272 "readclause.cpp"
        $land_lhs_true$$if_end8$9: do { 
          if (__label__ == 6) {
  
            var $12=$pw_addr; //@line 1272 "readclause.cpp"
            var $13=HEAP32[(($12)>>2)]; //@line 1272 "readclause.cpp"
            var $call4=_towlower($13); //@line 1272 "readclause.cpp"
            var $cmp5=((($call4))|0)==115; //@line 1272 "readclause.cpp"
            if (!($cmp5)) { __label__ = 8;break $land_lhs_true$$if_end8$9; } //@line 1272 "readclause.cpp"
  
            var $14=$value; //@line 1275 "readclause.cpp"
            var $mul7=((($14)*1000)&4294967295); //@line 1275 "readclause.cpp"
            $value=$mul7; //@line 1275 "readclause.cpp"
            ; //@line 1276 "readclause.cpp"
          }
        } while(0);
  
        var $15=$value; //@line 1277 "readclause.cpp"
        $retval=$15; //@line 1277 "readclause.cpp"
        __label__ = 9;break $if_then$$lor_lhs_false$2; //@line 1277 "readclause.cpp"
      }
    } while(0);
    if (__label__ == 3) {
  
      var $5=$default_value_addr; //@line 1266 "readclause.cpp"
      $retval=$5; //@line 1266 "readclause.cpp"
      ; //@line 1266 "readclause.cpp"
    }
  
    var $16=$retval; //@line 1278 "readclause.cpp"
    ;
    return $16; //@line 1278 "readclause.cpp"
    return null;
  }
  

  function __ZL14ReplaceKeyNamePciRi($outbuf, $index, $outix) {
    var __stackBase__  = STACKTOP; STACKTOP += 40; _memset(__stackBase__, 0, 40);
    var __label__;
  
    var $retval;
    var $outbuf_addr;
    var $index_addr;
    var $outix_addr;
    var $keynames=__stackBase__;
    var $ix;
    var $letter;
    var $p;
    $outbuf_addr=$outbuf;
    $index_addr=$index;
    $outix_addr=$outix;
    var $0=$keynames; //@line 1572 "readclause.cpp"
    var $src$, $dest$, $stop$, $stop4$, $fast$;
    $src$ = __ZZL14ReplaceKeyNamePciRiE8keynames;
    $dest$ = $0;
    $stop$ = $src$ + 40;
    if (($dest$%4) == ($src$%4) && 40 > 8) {
      while ($src$%4 !== 0 && $src$ < $stop$) {
        ; HEAP8[$dest$++] = HEAP8[$src$++];
      }
      $src$ >>= 2;
      $dest$ >>= 2;
      $stop4$ = $stop$ >> 2;
      while ($src$ < $stop4$) {
    ;;;    HEAP32[$dest$++] = HEAP32[$src$++];
      }
      $src$ <<= 2;
      $dest$ <<= 2;
    }while ($src$ < $stop$) {
      ; HEAP8[$dest$++] = HEAP8[$src$++];
    }; //@line 1572 "readclause.cpp"
    var $1=$index_addr; //@line 1578 "readclause.cpp"
    var $2=$outbuf_addr; //@line 1578 "readclause.cpp"
    var $arrayidx=(($2+$1)&4294967295); //@line 1578 "readclause.cpp"
    $p=$arrayidx; //@line 1578 "readclause.cpp"
    var $arraydecay=(($keynames)&4294967295); //@line 1580 "readclause.cpp"
    var $3=$p; //@line 1580 "readclause.cpp"
    var $call=__Z10LookupMnemP8MNEM_TABPKc($arraydecay, $3); //@line 1580 "readclause.cpp"
    $letter=$call; //@line 1580 "readclause.cpp"
    var $cmp=((($call))|0)!=0; //@line 1580 "readclause.cpp"
    if ($cmp) { __label__ = 1;; } else { __label__ = 2;; } //@line 1580 "readclause.cpp"
    if (__label__ == 1) {
  
      var $4=$letter; //@line 1582 "readclause.cpp"
      var $5=$p; //@line 1582 "readclause.cpp"
      var $call1=__Z8utf8_outjPc($4, $5); //@line 1582 "readclause.cpp"
      $ix=$call1; //@line 1582 "readclause.cpp"
      var $6=$index_addr; //@line 1583 "readclause.cpp"
      var $7=$ix; //@line 1583 "readclause.cpp"
      var $add=((($7)+($6))&4294967295); //@line 1583 "readclause.cpp"
      var $8=$outix_addr; //@line 1583 "readclause.cpp"
      HEAP32[(($8)>>2)]=$add; //@line 1583 "readclause.cpp"
      var $9=$letter; //@line 1584 "readclause.cpp"
      $retval=$9; //@line 1584 "readclause.cpp"
      ; //@line 1584 "readclause.cpp"
    }
    else if (__label__ == 2) {
  
      $retval=0; //@line 1586 "readclause.cpp"
      ; //@line 1586 "readclause.cpp"
    }
  
    STACKTOP = __stackBase__;
    return;
    return;
  }
  

  function __ZL13attrcopy_utf8PcPKwi($buf, $pw, $len) {
    ;
    var __label__;
  
    var $buf_addr;
    var $pw_addr;
    var $len_addr;
    var $c;
    var $ix;
    var $n;
    var $prev_c;
    $buf_addr=$buf;
    $pw_addr=$pw;
    $len_addr=$len;
    $ix=0; //@line 1286 "readclause.cpp"
    $prev_c=0; //@line 1288 "readclause.cpp"
    var $0=$pw_addr; //@line 1290 "readclause.cpp"
    var $cmp=((($0))|0)!=0; //@line 1290 "readclause.cpp"
    if ($cmp) { __label__ = 1;; } else { __label__ = 6;; } //@line 1290 "readclause.cpp"
    $while_cond$$if_end6$2: do { 
      if (__label__ == 1) {
        while(1) { 
  
          var $1=$ix; //@line 1292 "readclause.cpp"
          var $2=$len_addr; //@line 1292 "readclause.cpp"
          var $sub=((($2)-4)&4294967295); //@line 1292 "readclause.cpp"
          var $cmp1=((($1))|0) < ((($sub))|0); //@line 1292 "readclause.cpp"
          if (!($cmp1)) { __label__ = 6;break $while_cond$$if_end6$2; } //@line 1292 "readclause.cpp"
  
          var $3=$pw_addr; //@line 1292 "readclause.cpp"
          var $incdec_ptr=(($3+4)&4294967295); //@line 1292 "readclause.cpp"
          $pw_addr=$incdec_ptr; //@line 1292 "readclause.cpp"
          var $4=HEAP32[(($3)>>2)]; //@line 1292 "readclause.cpp"
          $c=$4; //@line 1292 "readclause.cpp"
          var $cmp2=((($4))|0)!=0; //@line 1292 "readclause.cpp"
          if (!($cmp2)) { __label__ = 6;break $while_cond$$if_end6$2; }
  
          var $5=$c; //@line 1294 "readclause.cpp"
          var $cmp3=((($5))|0)==34; //@line 1294 "readclause.cpp"
          if ($cmp3) { __label__ = 4;; } else { __label__ = 5;; } //@line 1294 "readclause.cpp"
          if (__label__ == 4) {
  
            var $6=$prev_c; //@line 1294 "readclause.cpp"
            var $cmp4=((($6))|0)!=92; //@line 1294 "readclause.cpp"
            if ($cmp4) { __label__ = 6;break $while_cond$$if_end6$2; } //@line 1294 "readclause.cpp"
          }
  
          var $7=$c; //@line 1296 "readclause.cpp"
          var $8=$ix; //@line 1296 "readclause.cpp"
          var $9=$buf_addr; //@line 1296 "readclause.cpp"
          var $arrayidx=(($9+$8)&4294967295); //@line 1296 "readclause.cpp"
          var $call=__Z8utf8_outjPc($7, $arrayidx); //@line 1296 "readclause.cpp"
          $n=$call; //@line 1296 "readclause.cpp"
          var $10=$n; //@line 1297 "readclause.cpp"
          var $11=$ix; //@line 1297 "readclause.cpp"
          var $add=((($11)+($10))&4294967295); //@line 1297 "readclause.cpp"
          $ix=$add; //@line 1297 "readclause.cpp"
          var $12=$c; //@line 1298 "readclause.cpp"
          $prev_c=$12; //@line 1298 "readclause.cpp"
          __label__ = 1;continue ; //@line 1299 "readclause.cpp"
        }
      }
    } while(0);
  
    var $13=$ix; //@line 1301 "readclause.cpp"
    var $14=$buf_addr; //@line 1301 "readclause.cpp"
    var $arrayidx7=(($14+$13)&4294967295); //@line 1301 "readclause.cpp"
    HEAP8[($arrayidx7)]=0; //@line 1301 "readclause.cpp"
    var $15=$ix; //@line 1302 "readclause.cpp"
    ;
    return $15; //@line 1302 "readclause.cpp"
    return null;
  }
  

  function __ZL14LoadSoundFile2PKc($fname) {
    ;
    var __label__;
  
    var $retval;
    var $fname_addr;
    var $ix;
    $fname_addr=$fname;
    $ix=0; //@line 834 "readclause.cpp"
    ; //@line 834 "readclause.cpp"
    while(1) { 
  
      var $0=$ix; //@line 834 "readclause.cpp"
      var $1=HEAP32[((_n_soundicon_tab)>>2)]; //@line 834 "readclause.cpp"
      var $cmp=((($0))|0) < ((($1))|0); //@line 834 "readclause.cpp"
      if (!($cmp)) { __label__ = 6;break ; } //@line 834 "readclause.cpp"
  
      var $2=$ix; //@line 836 "readclause.cpp"
      var $arrayidx=((_soundicon_tab+$2*16)&4294967295); //@line 836 "readclause.cpp"
      var $filename=(($arrayidx+12)&4294967295); //@line 836 "readclause.cpp"
      var $3=HEAP32[(($filename)>>2)]; //@line 836 "readclause.cpp"
      var $cmp1=((($3))|0)!=0; //@line 836 "readclause.cpp"
      if ($cmp1) { __label__ = 3;; } else { __label__ = 5;; } //@line 836 "readclause.cpp"
      if (__label__ == 3) {
  
        var $4=$fname_addr; //@line 836 "readclause.cpp"
        var $5=$ix; //@line 836 "readclause.cpp"
        var $arrayidx2=((_soundicon_tab+$5*16)&4294967295); //@line 836 "readclause.cpp"
        var $filename3=(($arrayidx2+12)&4294967295); //@line 836 "readclause.cpp"
        var $6=HEAP32[(($filename3)>>2)]; //@line 836 "readclause.cpp"
        var $call=_strcmp($4, $6); //@line 836 "readclause.cpp"
        var $cmp4=((($call))|0)==0; //@line 836 "readclause.cpp"
        if ($cmp4) { __label__ = 4;break ; } //@line 836 "readclause.cpp"
      }
  
      var $8=$ix; //@line 834 "readclause.cpp"
      var $inc=((($8)+1)&4294967295); //@line 834 "readclause.cpp"
      $ix=$inc; //@line 834 "readclause.cpp"
      __label__ = 1;continue ; //@line 834 "readclause.cpp"
    }
    if (__label__ == 6) {
  
      var $9=HEAP32[((__ZZL14LoadSoundFile2PKcE4slot)>>2)]; //@line 841 "readclause.cpp"
      var $inc5=((($9)+1)&4294967295); //@line 841 "readclause.cpp"
      HEAP32[((__ZZL14LoadSoundFile2PKcE4slot)>>2)]=$inc5; //@line 841 "readclause.cpp"
      var $10=HEAP32[((__ZZL14LoadSoundFile2PKcE4slot)>>2)]; //@line 842 "readclause.cpp"
      var $cmp6=((($10))|0) >= 4; //@line 842 "readclause.cpp"
      if ($cmp6) { __label__ = 7;; } else { __label__ = 8;; } //@line 842 "readclause.cpp"
      if (__label__ == 7) {
  
        HEAP32[((__ZZL14LoadSoundFile2PKcE4slot)>>2)]=0; //@line 843 "readclause.cpp"
        ; //@line 843 "readclause.cpp"
      }
  
      var $11=$fname_addr; //@line 845 "readclause.cpp"
      var $12=HEAP32[((__ZZL14LoadSoundFile2PKcE4slot)>>2)]; //@line 845 "readclause.cpp"
      var $call9=__ZL13LoadSoundFilePKci($11, $12); //@line 845 "readclause.cpp"
      var $cmp10=((($call9))|0)!=0; //@line 845 "readclause.cpp"
      if ($cmp10) { __label__ = 9;; } else { __label__ = 10;; } //@line 845 "readclause.cpp"
      if (__label__ == 9) {
  
        $retval=-1; //@line 846 "readclause.cpp"
        ; //@line 846 "readclause.cpp"
      }
      else if (__label__ == 10) {
  
        var $13=$ix; //@line 848 "readclause.cpp"
        var $arrayidx13=((_soundicon_tab+$13*16)&4294967295); //@line 848 "readclause.cpp"
        var $filename14=(($arrayidx13+12)&4294967295); //@line 848 "readclause.cpp"
        var $14=HEAP32[(($filename14)>>2)]; //@line 848 "readclause.cpp"
        var $15=$fname_addr; //@line 848 "readclause.cpp"
        var $call15=_strlen($15); //@line 848 "readclause.cpp"
        var $add=((($call15)+1)&4294967295); //@line 848 "readclause.cpp"
        var $call16=_realloc($14, $add); //@line 848 "readclause.cpp"
        var $16=HEAP32[((__ZZL14LoadSoundFile2PKcE4slot)>>2)]; //@line 848 "readclause.cpp"
        var $arrayidx17=((_soundicon_tab+$16*16)&4294967295); //@line 848 "readclause.cpp"
        var $filename18=(($arrayidx17+12)&4294967295); //@line 848 "readclause.cpp"
        HEAP32[(($filename18)>>2)]=$call16; //@line 848 "readclause.cpp"
        var $17=HEAP32[((__ZZL14LoadSoundFile2PKcE4slot)>>2)]; //@line 849 "readclause.cpp"
        var $arrayidx19=((_soundicon_tab+$17*16)&4294967295); //@line 849 "readclause.cpp"
        var $filename20=(($arrayidx19+12)&4294967295); //@line 849 "readclause.cpp"
        var $18=HEAP32[(($filename20)>>2)]; //@line 849 "readclause.cpp"
        var $19=$fname_addr; //@line 849 "readclause.cpp"
        var $call21=_strcpy($18, $19); //@line 849 "readclause.cpp"
        var $20=HEAP32[((__ZZL14LoadSoundFile2PKcE4slot)>>2)]; //@line 850 "readclause.cpp"
        $retval=$20; //@line 850 "readclause.cpp"
        ; //@line 850 "readclause.cpp"
      }
    }
    else if (__label__ == 4) {
  
      var $7=$ix; //@line 837 "readclause.cpp"
      $retval=$7; //@line 837 "readclause.cpp"
      ; //@line 837 "readclause.cpp"
    }
  
    var $21=$retval; //@line 851 "readclause.cpp"
    ;
    return $21; //@line 851 "readclause.cpp"
    return null;
  }
  

  function __ZL18GetVoiceAttributesPwi($pw, $tag_type) {
    ;
    var __label__;
  
    var $retval;
    var $pw_addr;
    var $tag_type_addr;
    var $lang;
    var $gender;
    var $name;
    var $age;
    var $variant;
    var $new_voice_id;
    var $ssml_sp;
    $pw_addr=$pw;
    $tag_type_addr=$tag_type;
    var $0=$tag_type_addr; //@line 1438 "readclause.cpp"
    var $and=($0) & 32; //@line 1438 "readclause.cpp"
    var $tobool=((($and))|0)!=0; //@line 1438 "readclause.cpp"
    if ($tobool) { __label__ = 1;; } else { __label__ = 3;; } //@line 1438 "readclause.cpp"
    $if_then$$if_else$2: do { 
      if (__label__ == 1) {
  
        var $1=HEAP32[((__ZL12n_ssml_stack)>>2)]; //@line 1441 "readclause.cpp"
        var $cmp=((($1))|0) > 1; //@line 1441 "readclause.cpp"
        if (!($cmp)) { __label__ = 10;break $if_then$$if_else$2; } //@line 1441 "readclause.cpp"
  
        var $2=HEAP32[((__ZL12n_ssml_stack)>>2)]; //@line 1443 "readclause.cpp"
        var $dec=((($2)-1)&4294967295); //@line 1443 "readclause.cpp"
        HEAP32[((__ZL12n_ssml_stack)>>2)]=$dec; //@line 1443 "readclause.cpp"
        __label__ = 10;break $if_then$$if_else$2; //@line 1444 "readclause.cpp"
      }
      else if (__label__ == 3) {
  
        var $3=$pw_addr; //@line 1449 "readclause.cpp"
        var $call=__ZL16GetSsmlAttributePwPKc($3, ((__str106273)&4294967295)); //@line 1449 "readclause.cpp"
        $lang=$call; //@line 1449 "readclause.cpp"
        var $4=$tag_type_addr; //@line 1451 "readclause.cpp"
        var $cmp2=((($4))|0)!=2; //@line 1451 "readclause.cpp"
        if ($cmp2) { __label__ = 4;; } else { __label__ = 5;; } //@line 1451 "readclause.cpp"
        if (__label__ == 4) {
  
          $name=0; //@line 1454 "readclause.cpp"
          $variant=0; //@line 1455 "readclause.cpp"
          $age=0; //@line 1456 "readclause.cpp"
          $gender=0; //@line 1457 "readclause.cpp"
          ; //@line 1458 "readclause.cpp"
        }
        else if (__label__ == 5) {
  
          var $5=$pw_addr; //@line 1461 "readclause.cpp"
          var $call5=__ZL16GetSsmlAttributePwPKc($5, ((__str5655)&4294967295)); //@line 1461 "readclause.cpp"
          $name=$call5; //@line 1461 "readclause.cpp"
          var $6=$pw_addr; //@line 1462 "readclause.cpp"
          var $call6=__ZL16GetSsmlAttributePwPKc($6, ((__str55707)&4294967295)); //@line 1462 "readclause.cpp"
          $variant=$call6; //@line 1462 "readclause.cpp"
          var $7=$pw_addr; //@line 1463 "readclause.cpp"
          var $call7=__ZL16GetSsmlAttributePwPKc($7, ((__str108275)&4294967295)); //@line 1463 "readclause.cpp"
          $age=$call7; //@line 1463 "readclause.cpp"
          var $8=$pw_addr; //@line 1464 "readclause.cpp"
          var $call8=__ZL16GetSsmlAttributePwPKc($8, ((__str7657)&4294967295)); //@line 1464 "readclause.cpp"
          $gender=$call8; //@line 1464 "readclause.cpp"
          ;
        }
  
        var $9=$tag_type_addr; //@line 1467 "readclause.cpp"
        var $cmp10=((($9))|0)!=2; //@line 1467 "readclause.cpp"
        if ($cmp10) { __label__ = 7;; } else { __label__ = 9;; } //@line 1467 "readclause.cpp"
        $land_lhs_true$$if_end13$10: do { 
          if (__label__ == 7) {
  
            var $10=$lang; //@line 1467 "readclause.cpp"
            var $cmp11=((($10))|0)==0; //@line 1467 "readclause.cpp"
            if (!($cmp11)) { __label__ = 9;break $land_lhs_true$$if_end13$10; } //@line 1467 "readclause.cpp"
  
            $retval=0; //@line 1468 "readclause.cpp"
            __label__ = 13;break $if_then$$if_else$2; //@line 1468 "readclause.cpp"
          }
        } while(0);
  
        var $11=HEAP32[((__ZL12n_ssml_stack)>>2)]; //@line 1470 "readclause.cpp"
        var $inc=((($11)+1)&4294967295); //@line 1470 "readclause.cpp"
        HEAP32[((__ZL12n_ssml_stack)>>2)]=$inc; //@line 1470 "readclause.cpp"
        var $arrayidx=((__ZL10ssml_stack+$11*76)&4294967295); //@line 1470 "readclause.cpp"
        $ssml_sp=$arrayidx; //@line 1470 "readclause.cpp"
        var $12=$ssml_sp; //@line 1472 "readclause.cpp"
        var $language=(($12+56)&4294967295); //@line 1472 "readclause.cpp"
        var $arraydecay=(($language)&4294967295); //@line 1472 "readclause.cpp"
        var $13=$lang; //@line 1472 "readclause.cpp"
        var $call14=__ZL13attrcopy_utf8PcPKwi($arraydecay, $13, 20); //@line 1472 "readclause.cpp"
        var $14=$ssml_sp; //@line 1473 "readclause.cpp"
        var $voice_name=(($14+16)&4294967295); //@line 1473 "readclause.cpp"
        var $arraydecay15=(($voice_name)&4294967295); //@line 1473 "readclause.cpp"
        var $15=$name; //@line 1473 "readclause.cpp"
        var $call16=__ZL13attrcopy_utf8PcPKwi($arraydecay15, $15, 40); //@line 1473 "readclause.cpp"
        var $16=$variant; //@line 1474 "readclause.cpp"
        var $call17=__ZL10attrnumberPKwii($16, 1, 0); //@line 1474 "readclause.cpp"
        var $sub=((($call17)-1)&4294967295); //@line 1474 "readclause.cpp"
        var $17=$ssml_sp; //@line 1474 "readclause.cpp"
        var $voice_variant=(($17+4)&4294967295); //@line 1474 "readclause.cpp"
        HEAP32[(($voice_variant)>>2)]=$sub; //@line 1474 "readclause.cpp"
        var $18=$age; //@line 1475 "readclause.cpp"
        var $call18=__ZL10attrnumberPKwii($18, 0, 0); //@line 1475 "readclause.cpp"
        var $19=$ssml_sp; //@line 1475 "readclause.cpp"
        var $voice_age=(($19+12)&4294967295); //@line 1475 "readclause.cpp"
        HEAP32[(($voice_age)>>2)]=$call18; //@line 1475 "readclause.cpp"
        var $20=$gender; //@line 1476 "readclause.cpp"
        var $call19=__ZL10attrlookupPKwPK8MNEM_TAB($20, ((__ZZL18GetVoiceAttributesPwiE11mnem_gender)&4294967295)); //@line 1476 "readclause.cpp"
        var $21=$ssml_sp; //@line 1476 "readclause.cpp"
        var $voice_gender=(($21+8)&4294967295); //@line 1476 "readclause.cpp"
        HEAP32[(($voice_gender)>>2)]=$call19; //@line 1476 "readclause.cpp"
        var $22=$tag_type_addr; //@line 1477 "readclause.cpp"
        var $23=$ssml_sp; //@line 1477 "readclause.cpp"
        var $tag_type20=(($23)&4294967295); //@line 1477 "readclause.cpp"
        HEAP32[(($tag_type20)>>2)]=$22; //@line 1477 "readclause.cpp"
        __label__ = 10;break $if_then$$if_else$2;
      }
    } while(0);
    if (__label__ == 10) {
  
      var $call22=__ZL14VoiceFromStackv(); //@line 1480 "readclause.cpp"
      $new_voice_id=$call22; //@line 1480 "readclause.cpp"
      var $24=$new_voice_id; //@line 1481 "readclause.cpp"
      var $call23=_strcmp($24, ((__ZL16current_voice_id)&4294967295)); //@line 1481 "readclause.cpp"
      var $cmp24=((($call23))|0)!=0; //@line 1481 "readclause.cpp"
      if ($cmp24) { __label__ = 11;; } else { __label__ = 12;; } //@line 1481 "readclause.cpp"
      if (__label__ == 11) {
  
        var $25=$new_voice_id; //@line 1484 "readclause.cpp"
        var $call26=_strcpy(((__ZL16current_voice_id)&4294967295), $25); //@line 1484 "readclause.cpp"
        $retval=131072; //@line 1485 "readclause.cpp"
        ; //@line 1485 "readclause.cpp"
      }
      else if (__label__ == 12) {
  
        $retval=0; //@line 1488 "readclause.cpp"
        ; //@line 1488 "readclause.cpp"
      }
    }
  
    var $26=$retval; //@line 1489 "readclause.cpp"
    ;
    return $26; //@line 1489 "readclause.cpp"
    return null;
  }
  

  function __ZL14VoiceFromStackv() {
    var __stackBase__  = STACKTOP; STACKTOP += 108; _memset(__stackBase__, 0, 108);
    var __label__;
  
    var $retval;
    var $ix;
    var $sp;
    var $v_id;
    var $voice_name_specified;
    var $voice_found=__stackBase__;
    var $voice_select=__stackBase__+4;
    var $voice_name=__stackBase__+28;
    var $language=__stackBase__+68;
    var $arraydecay=(($voice_name)&4294967295); //@line 1053 "readclause.cpp"
    var $call=_strcpy($arraydecay, ((__ZL10ssml_stack+16)&4294967295)); //@line 1053 "readclause.cpp"
    var $arraydecay1=(($language)&4294967295); //@line 1054 "readclause.cpp"
    var $call2=_strcpy($arraydecay1, ((__ZL10ssml_stack+56)&4294967295)); //@line 1054 "readclause.cpp"
    var $0=HEAP32[((((__ZL10ssml_stack+12)&4294967295))>>2)]; //@line 1055 "readclause.cpp"
    var $conv=((($0)) & 255); //@line 1055 "readclause.cpp"
    var $age=(($voice_select+13)&4294967295); //@line 1055 "readclause.cpp"
    HEAP8[($age)]=$conv; //@line 1055 "readclause.cpp"
    var $1=HEAP32[((((__ZL10ssml_stack+8)&4294967295))>>2)]; //@line 1056 "readclause.cpp"
    var $conv3=((($1)) & 255); //@line 1056 "readclause.cpp"
    var $gender=(($voice_select+12)&4294967295); //@line 1056 "readclause.cpp"
    HEAP8[($gender)]=$conv3; //@line 1056 "readclause.cpp"
    var $2=HEAP32[((((__ZL10ssml_stack+4)&4294967295))>>2)]; //@line 1057 "readclause.cpp"
    var $conv4=((($2)) & 255); //@line 1057 "readclause.cpp"
    var $variant=(($voice_select+14)&4294967295); //@line 1057 "readclause.cpp"
    HEAP8[($variant)]=$conv4; //@line 1057 "readclause.cpp"
    var $identifier=(($voice_select+8)&4294967295); //@line 1058 "readclause.cpp"
    HEAP32[(($identifier)>>2)]=0; //@line 1058 "readclause.cpp"
    $ix=0; //@line 1060 "readclause.cpp"
    var $3=$ix; //@line 1060 "readclause.cpp"
    var $4=HEAP32[((__ZL12n_ssml_stack)>>2)]; //@line 1060 "readclause.cpp"
    var $cmp1=((($3))|0) < ((($4))|0); //@line 1060 "readclause.cpp"
    if ($cmp1) { __label__ = 1;; } else { __label__ = 15;; } //@line 1060 "readclause.cpp"
    $for_body_lr_ph$$for_end$2: do { 
      if (__label__ == 1) {
  
        var $arraydecay13=(($voice_name)&4294967295); //@line 1068 "readclause.cpp"
        var $arrayidx17=(($language)&4294967295); //@line 1069 "readclause.cpp"
        var $gender18=(($voice_select+12)&4294967295); //@line 1070 "readclause.cpp"
        var $age19=(($voice_select+13)&4294967295); //@line 1071 "readclause.cpp"
        var $variant20=(($voice_select+14)&4294967295); //@line 1072 "readclause.cpp"
        var $arraydecay26=(($language)&4294967295); //@line 1076 "readclause.cpp"
        var $arrayidx32=(($voice_name)&4294967295); //@line 1078 "readclause.cpp"
        var $gender39=(($voice_select+12)&4294967295); //@line 1081 "readclause.cpp"
        var $age45=(($voice_select+13)&4294967295); //@line 1083 "readclause.cpp"
        var $variant51=(($voice_select+14)&4294967295); //@line 1085 "readclause.cpp"
        ; //@line 1060 "readclause.cpp"
        $for_body$4: while(1) { 
  
          var $5=$ix; //@line 1062 "readclause.cpp"
          var $arrayidx=((__ZL10ssml_stack+$5*76)&4294967295); //@line 1062 "readclause.cpp"
          $sp=$arrayidx; //@line 1062 "readclause.cpp"
          $voice_name_specified=0; //@line 1063 "readclause.cpp"
          var $6=$sp; //@line 1065 "readclause.cpp"
          var $voice_name5=(($6+16)&4294967295); //@line 1065 "readclause.cpp"
          var $arrayidx6=(($voice_name5)&4294967295); //@line 1065 "readclause.cpp"
          var $7=HEAP8[($arrayidx6)]; //@line 1065 "readclause.cpp"
          var $conv7=(tempInt=(($7)),(tempInt>=128?tempInt-256:tempInt)); //@line 1065 "readclause.cpp"
          var $cmp8=((($conv7))|0)!=0; //@line 1065 "readclause.cpp"
          if ($cmp8) { __label__ = 3;; } else { __label__ = 5;; } //@line 1065 "readclause.cpp"
          $land_lhs_true$$if_end$6: do { 
            if (__label__ == 3) {
  
              var $8=$sp; //@line 1065 "readclause.cpp"
              var $voice_name9=(($8+16)&4294967295); //@line 1065 "readclause.cpp"
              var $arraydecay10=(($voice_name9)&4294967295); //@line 1065 "readclause.cpp"
              var $call11=__Z17SelectVoiceByNamePP12espeak_VOICEPKc(0, $arraydecay10); //@line 1065 "readclause.cpp"
              var $cmp12=((($call11))|0)!=0; //@line 1065 "readclause.cpp"
              if (!($cmp12)) { __label__ = 5;break $land_lhs_true$$if_end$6; } //@line 1065 "readclause.cpp"
  
              $voice_name_specified=1; //@line 1067 "readclause.cpp"
              var $9=$sp; //@line 1068 "readclause.cpp"
              var $voice_name14=(($9+16)&4294967295); //@line 1068 "readclause.cpp"
              var $arraydecay15=(($voice_name14)&4294967295); //@line 1068 "readclause.cpp"
              var $call16=_strcpy($arraydecay13, $arraydecay15); //@line 1068 "readclause.cpp"
              HEAP8[($arrayidx17)]=0; //@line 1069 "readclause.cpp"
              HEAP8[($gender18)]=0; //@line 1070 "readclause.cpp"
              HEAP8[($age19)]=0; //@line 1071 "readclause.cpp"
              HEAP8[($variant20)]=0; //@line 1072 "readclause.cpp"
              ; //@line 1073 "readclause.cpp"
            }
          } while(0);
  
          var $10=$sp; //@line 1074 "readclause.cpp"
          var $language21=(($10+56)&4294967295); //@line 1074 "readclause.cpp"
          var $arrayidx22=(($language21)&4294967295); //@line 1074 "readclause.cpp"
          var $11=HEAP8[($arrayidx22)]; //@line 1074 "readclause.cpp"
          var $conv23=(tempInt=(($11)),(tempInt>=128?tempInt-256:tempInt)); //@line 1074 "readclause.cpp"
          var $cmp24=((($conv23))|0)!=0; //@line 1074 "readclause.cpp"
          if ($cmp24) { __label__ = 6;; } else { __label__ = 8;; } //@line 1074 "readclause.cpp"
          $if_then25$$if_end34$10: do { 
            if (__label__ == 6) {
  
              var $12=$sp; //@line 1076 "readclause.cpp"
              var $language27=(($12+56)&4294967295); //@line 1076 "readclause.cpp"
              var $arraydecay28=(($language27)&4294967295); //@line 1076 "readclause.cpp"
              var $call29=_strcpy($arraydecay26, $arraydecay28); //@line 1076 "readclause.cpp"
              var $13=$voice_name_specified; //@line 1077 "readclause.cpp"
              var $cmp30=((($13))|0)==0; //@line 1077 "readclause.cpp"
              if (!($cmp30)) { __label__ = 8;break $if_then25$$if_end34$10; } //@line 1077 "readclause.cpp"
  
              HEAP8[($arrayidx32)]=0; //@line 1078 "readclause.cpp"
              ; //@line 1078 "readclause.cpp"
            }
          } while(0);
  
          var $14=$sp; //@line 1080 "readclause.cpp"
          var $voice_gender=(($14+8)&4294967295); //@line 1080 "readclause.cpp"
          var $15=HEAP32[(($voice_gender)>>2)]; //@line 1080 "readclause.cpp"
          var $cmp35=((($15))|0)!=0; //@line 1080 "readclause.cpp"
          if ($cmp35) { __label__ = 9;; } else { __label__ = 10;; } //@line 1080 "readclause.cpp"
          if (__label__ == 9) {
  
            var $16=$sp; //@line 1081 "readclause.cpp"
            var $voice_gender37=(($16+8)&4294967295); //@line 1081 "readclause.cpp"
            var $17=HEAP32[(($voice_gender37)>>2)]; //@line 1081 "readclause.cpp"
            var $conv38=((($17)) & 255); //@line 1081 "readclause.cpp"
            HEAP8[($gender39)]=$conv38; //@line 1081 "readclause.cpp"
            ; //@line 1081 "readclause.cpp"
          }
  
          var $18=$sp; //@line 1082 "readclause.cpp"
          var $voice_age=(($18+12)&4294967295); //@line 1082 "readclause.cpp"
          var $19=HEAP32[(($voice_age)>>2)]; //@line 1082 "readclause.cpp"
          var $cmp41=((($19))|0)!=0; //@line 1082 "readclause.cpp"
          if ($cmp41) { __label__ = 11;; } else { __label__ = 12;; } //@line 1082 "readclause.cpp"
          if (__label__ == 11) {
  
            var $20=$sp; //@line 1083 "readclause.cpp"
            var $voice_age43=(($20+12)&4294967295); //@line 1083 "readclause.cpp"
            var $21=HEAP32[(($voice_age43)>>2)]; //@line 1083 "readclause.cpp"
            var $conv44=((($21)) & 255); //@line 1083 "readclause.cpp"
            HEAP8[($age45)]=$conv44; //@line 1083 "readclause.cpp"
            ; //@line 1083 "readclause.cpp"
          }
  
          var $22=$sp; //@line 1084 "readclause.cpp"
          var $voice_variant=(($22+4)&4294967295); //@line 1084 "readclause.cpp"
          var $23=HEAP32[(($voice_variant)>>2)]; //@line 1084 "readclause.cpp"
          var $cmp47=((($23))|0)!=0; //@line 1084 "readclause.cpp"
          if ($cmp47) { __label__ = 13;; } else { __label__ = 14;; } //@line 1084 "readclause.cpp"
          if (__label__ == 13) {
  
            var $24=$sp; //@line 1085 "readclause.cpp"
            var $voice_variant49=(($24+4)&4294967295); //@line 1085 "readclause.cpp"
            var $25=HEAP32[(($voice_variant49)>>2)]; //@line 1085 "readclause.cpp"
            var $conv50=((($25)) & 255); //@line 1085 "readclause.cpp"
            HEAP8[($variant51)]=$conv50; //@line 1085 "readclause.cpp"
            ; //@line 1085 "readclause.cpp"
          }
  
          var $26=$ix; //@line 1060 "readclause.cpp"
          var $inc=((($26)+1)&4294967295); //@line 1060 "readclause.cpp"
          $ix=$inc; //@line 1060 "readclause.cpp"
          var $27=$ix; //@line 1060 "readclause.cpp"
          var $28=HEAP32[((__ZL12n_ssml_stack)>>2)]; //@line 1060 "readclause.cpp"
          var $cmp=((($27))|0) < ((($28))|0); //@line 1060 "readclause.cpp"
          if ($cmp) { __label__ = 2;continue $for_body$4; } else { __label__ = 15;break $for_body_lr_ph$$for_end$2; } //@line 1060 "readclause.cpp"
        }
      }
    } while(0);
  
    var $arraydecay53=(($voice_name)&4294967295); //@line 1088 "readclause.cpp"
    var $name=(($voice_select)&4294967295); //@line 1088 "readclause.cpp"
    HEAP32[(($name)>>2)]=$arraydecay53; //@line 1088 "readclause.cpp"
    var $arraydecay54=(($language)&4294967295); //@line 1089 "readclause.cpp"
    var $languages=(($voice_select+4)&4294967295); //@line 1089 "readclause.cpp"
    HEAP32[(($languages)>>2)]=$arraydecay54; //@line 1089 "readclause.cpp"
    var $call55=__Z11SelectVoiceP12espeak_VOICEPi($voice_select, $voice_found); //@line 1090 "readclause.cpp"
    $v_id=$call55; //@line 1090 "readclause.cpp"
    var $cmp56=((($call55))|0)==0; //@line 1091 "readclause.cpp"
    if ($cmp56) { __label__ = 16;; } else { __label__ = 17;; } //@line 1091 "readclause.cpp"
    if (__label__ == 16) {
  
      $retval=((__str45697)&4294967295); //@line 1092 "readclause.cpp"
      ; //@line 1092 "readclause.cpp"
    }
    else if (__label__ == 17) {
  
      var $29=$v_id; //@line 1093 "readclause.cpp"
      $retval=$29; //@line 1093 "readclause.cpp"
      ; //@line 1093 "readclause.cpp"
    }
  
    var $30=$retval; //@line 1094 "readclause.cpp"
    STACKTOP = __stackBase__;
    return $30; //@line 1094 "readclause.cpp"
    return null;
  }
  

  function __ZL18attr_prosody_valueiPKwPi($param_type, $pw, $value_out) {
    var __stackBase__  = STACKTOP; STACKTOP += 4; _memset(__stackBase__, 0, 4);
    var __label__;
    var __lastLabel__ = null;
  
    var $retval;
    var $param_type_addr;
    var $pw_addr;
    var $value_out_addr;
    var $sign;
    var $tail=__stackBase__;
    var $value;
    var $x;
    $param_type_addr=$param_type;
    $pw_addr=$pw;
    $value_out_addr=$value_out;
    $sign=0; //@line 1309 "readclause.cpp"
    var $0=$pw_addr; //@line 1313 "readclause.cpp"
    var $1=HEAP32[(($0)>>2)]; //@line 1313 "readclause.cpp"
    var $call1=_iswspace($1); //@line 1313 "readclause.cpp"
    var $tobool2=((($call1))|0)!=0; //@line 1313 "readclause.cpp"
    var $2=$pw_addr; //@line 1313 "readclause.cpp"
    if ($tobool2) { __lastLabel__ = 0; __label__ = 1;; } else { __lastLabel__ = 0; __label__ = 2;; } //@line 1313 "readclause.cpp"
    $while_body$$while_end$2: do { 
      if (__label__ == 1) {
        while(1) { 
  
          var $3=__lastLabel__ == 1 ? $6 : ($2);
          var $incdec_ptr=(($3+4)&4294967295); //@line 1313 "readclause.cpp"
          $pw_addr=$incdec_ptr; //@line 1313 "readclause.cpp"
          var $4=$pw_addr; //@line 1313 "readclause.cpp"
          var $5=HEAP32[(($4)>>2)]; //@line 1313 "readclause.cpp"
          var $call=_iswspace($5); //@line 1313 "readclause.cpp"
          var $tobool=((($call))|0)!=0; //@line 1313 "readclause.cpp"
          var $6=$pw_addr; //@line 1313 "readclause.cpp"
          if ($tobool) { __lastLabel__ = 1; __label__ = 1;continue ; } else { __lastLabel__ = 1; __label__ = 2;break $while_body$$while_end$2; } //@line 1313 "readclause.cpp"
        }
      }
    } while(0);
  
    var $_lcssa=__lastLabel__ == 0 ? $2 : ($6);
    var $7=HEAP32[(($_lcssa)>>2)]; //@line 1314 "readclause.cpp"
    var $cmp=((($7))|0)==43; //@line 1314 "readclause.cpp"
    if ($cmp) { __label__ = 3;; } else { __label__ = 4;; } //@line 1314 "readclause.cpp"
    if (__label__ == 3) {
  
      var $8=$pw_addr; //@line 1316 "readclause.cpp"
      var $incdec_ptr1=(($8+4)&4294967295); //@line 1316 "readclause.cpp"
      $pw_addr=$incdec_ptr1; //@line 1316 "readclause.cpp"
      $sign=1; //@line 1317 "readclause.cpp"
      ; //@line 1318 "readclause.cpp"
    }
  
    var $9=$pw_addr; //@line 1319 "readclause.cpp"
    var $10=HEAP32[(($9)>>2)]; //@line 1319 "readclause.cpp"
    var $cmp2=((($10))|0)==45; //@line 1319 "readclause.cpp"
    if ($cmp2) { __label__ = 5;; } else { __label__ = 6;; } //@line 1319 "readclause.cpp"
    if (__label__ == 5) {
  
      var $11=$pw_addr; //@line 1321 "readclause.cpp"
      var $incdec_ptr4=(($11+4)&4294967295); //@line 1321 "readclause.cpp"
      $pw_addr=$incdec_ptr4; //@line 1321 "readclause.cpp"
      $sign=-1; //@line 1322 "readclause.cpp"
      ; //@line 1323 "readclause.cpp"
    }
  
    var $12=$pw_addr; //@line 1324 "readclause.cpp"
    var $call6=_wcstod($12, $tail); //@line 1324 "readclause.cpp"
    var $conv=($call6); //@line 1324 "readclause.cpp"
    $value=$conv; //@line 1324 "readclause.cpp"
    var $13=HEAP32[(($tail)>>2)]; //@line 1325 "readclause.cpp"
    var $14=$pw_addr; //@line 1325 "readclause.cpp"
    var $cmp7=((($13))|0)==((($14))|0); //@line 1325 "readclause.cpp"
    if ($cmp7) { __label__ = 7;; } else { __label__ = 8;; } //@line 1325 "readclause.cpp"
    $if_then8$$if_end9$12: do { 
      if (__label__ == 7) {
  
        var $15=$value_out_addr; //@line 1328 "readclause.cpp"
        HEAP32[(($15)>>2)]=100; //@line 1328 "readclause.cpp"
        $retval=2; //@line 1329 "readclause.cpp"
        ; //@line 1329 "readclause.cpp"
      }
      else if (__label__ == 8) {
  
        var $16=HEAP32[(($tail)>>2)]; //@line 1332 "readclause.cpp"
        var $17=HEAP32[(($16)>>2)]; //@line 1332 "readclause.cpp"
        var $cmp10=((($17))|0)==37; //@line 1332 "readclause.cpp"
        if ($cmp10) { __label__ = 9;; } else { __label__ = 12;; } //@line 1332 "readclause.cpp"
        if (__label__ == 9) {
  
          var $18=$sign; //@line 1334 "readclause.cpp"
          var $cmp12=((($18))|0)!=0; //@line 1334 "readclause.cpp"
          if ($cmp12) { __label__ = 10;; } else { __label__ = 11;; } //@line 1334 "readclause.cpp"
          if (__label__ == 10) {
  
            var $19=$sign; //@line 1335 "readclause.cpp"
            var $conv14=((($19))|0); //@line 1335 "readclause.cpp"
            var $20=$value; //@line 1335 "readclause.cpp"
            var $mul=($conv14)*($20); //@line 1335 "readclause.cpp"
            var $add=100+($mul); //@line 1335 "readclause.cpp"
            $value=$add; //@line 1335 "readclause.cpp"
            ; //@line 1335 "readclause.cpp"
          }
  
          var $21=$value; //@line 1336 "readclause.cpp"
          var $conv16=((($21))|0); //@line 1336 "readclause.cpp"
          var $22=$value_out_addr; //@line 1336 "readclause.cpp"
          HEAP32[(($22)>>2)]=$conv16; //@line 1336 "readclause.cpp"
          $retval=2; //@line 1337 "readclause.cpp"
          ; //@line 1337 "readclause.cpp"
        }
        else if (__label__ == 12) {
  
          var $23=HEAP32[(($tail)>>2)]; //@line 1340 "readclause.cpp"
          var $arrayidx=(($23)&4294967295); //@line 1340 "readclause.cpp"
          var $24=HEAP32[(($arrayidx)>>2)]; //@line 1340 "readclause.cpp"
          var $cmp18=((($24))|0)==115; //@line 1340 "readclause.cpp"
          if ($cmp18) { __label__ = 13;; } else { __label__ = 15;; } //@line 1340 "readclause.cpp"
          $land_lhs_true$$if_end26$21: do { 
            if (__label__ == 13) {
  
              var $25=HEAP32[(($tail)>>2)]; //@line 1340 "readclause.cpp"
              var $arrayidx19=(($25+4)&4294967295); //@line 1340 "readclause.cpp"
              var $26=HEAP32[(($arrayidx19)>>2)]; //@line 1340 "readclause.cpp"
              var $cmp20=((($26))|0)==116; //@line 1340 "readclause.cpp"
              if (!($cmp20)) { __label__ = 15;break $land_lhs_true$$if_end26$21; } //@line 1340 "readclause.cpp"
  
              var $27=$value; //@line 1344 "readclause.cpp"
              var $28=$sign; //@line 1344 "readclause.cpp"
              var $conv22=((($28))|0); //@line 1344 "readclause.cpp"
              var $mul23=($27)*($conv22); //@line 1344 "readclause.cpp"
              var $div=($mul23)/12; //@line 1344 "readclause.cpp"
              var $exp2=_exp2($div); //@line 1344 "readclause.cpp"
              var $mul24=($exp2)*100; //@line 1344 "readclause.cpp"
              $x=$mul24; //@line 1344 "readclause.cpp"
              var $29=$x; //@line 1345 "readclause.cpp"
              var $conv25=((($29))|0); //@line 1345 "readclause.cpp"
              var $30=$value_out_addr; //@line 1345 "readclause.cpp"
              HEAP32[(($30)>>2)]=$conv25; //@line 1345 "readclause.cpp"
              $retval=2; //@line 1346 "readclause.cpp"
              __label__ = 21;break $if_then8$$if_end9$12; //@line 1346 "readclause.cpp"
            }
          } while(0);
  
          var $31=$param_type_addr; //@line 1349 "readclause.cpp"
          var $cmp27=((($31))|0)==1; //@line 1349 "readclause.cpp"
          if ($cmp27) { __label__ = 16;; } else { __label__ = 20;; } //@line 1349 "readclause.cpp"
          if (__label__ == 16) {
  
            var $32=$sign; //@line 1351 "readclause.cpp"
            var $cmp29=((($32))|0)==0; //@line 1351 "readclause.cpp"
            if ($cmp29) { __label__ = 17;; } else { __label__ = 18;; } //@line 1351 "readclause.cpp"
            if (__label__ == 17) {
  
              var $33=$value; //@line 1352 "readclause.cpp"
              var $mul31=($33)*100; //@line 1352 "readclause.cpp"
              var $conv32=((($mul31))|0); //@line 1352 "readclause.cpp"
              var $34=$value_out_addr; //@line 1352 "readclause.cpp"
              HEAP32[(($34)>>2)]=$conv32; //@line 1352 "readclause.cpp"
              ; //@line 1352 "readclause.cpp"
            }
            else if (__label__ == 18) {
  
              var $35=$sign; //@line 1354 "readclause.cpp"
              var $conv33=((($35))|0); //@line 1354 "readclause.cpp"
              var $36=$value; //@line 1354 "readclause.cpp"
              var $mul34=($conv33)*($36); //@line 1354 "readclause.cpp"
              var $mul35=($mul34)*100; //@line 1354 "readclause.cpp"
              var $conv36=((($mul35))|0); //@line 1354 "readclause.cpp"
              var $add37=((($conv36)+100)&4294967295); //@line 1354 "readclause.cpp"
              var $37=$value_out_addr; //@line 1354 "readclause.cpp"
              HEAP32[(($37)>>2)]=$add37; //@line 1354 "readclause.cpp"
              ;
            }
  
            $retval=2; //@line 1355 "readclause.cpp"
            ; //@line 1355 "readclause.cpp"
          }
          else if (__label__ == 20) {
  
            var $38=$value; //@line 1358 "readclause.cpp"
            var $conv40=((($38))|0); //@line 1358 "readclause.cpp"
            var $39=$value_out_addr; //@line 1358 "readclause.cpp"
            HEAP32[(($39)>>2)]=$conv40; //@line 1358 "readclause.cpp"
            var $40=$sign; //@line 1359 "readclause.cpp"
            $retval=$40; //@line 1359 "readclause.cpp"
            ; //@line 1359 "readclause.cpp"
          }
        }
      }
    } while(0);
  
    var $41=$retval; //@line 1360 "readclause.cpp"
    STACKTOP = __stackBase__;
    return $41; //@line 1360 "readclause.cpp"
    return null;
  }
  

  function __ZL8GetC_getv() {
    ;
    var __label__;
  
    var $retval;
    var $c;
    var $c2;
    var $0=HEAP32[((__ZL7f_input)>>2)]; //@line 419 "readclause.cpp"
    var $cmp=((($0))|0)!=0; //@line 419 "readclause.cpp"
    if ($cmp) { __label__ = 1;; } else { __label__ = 8;; } //@line 419 "readclause.cpp"
    $if_then$$if_end13$2: do { 
      if (__label__ == 1) {
  
        var $1=HEAP32[((__ZL7f_input)>>2)]; //@line 421 "readclause.cpp"
        var $call=_fgetc($1); //@line 421 "readclause.cpp"
        $c=$call; //@line 421 "readclause.cpp"
        var $2=HEAP32[((__ZL7f_input)>>2)]; //@line 422 "readclause.cpp"
        var $_flags=(($2+12)&4294967295); //@line 422 "readclause.cpp"
        var $3=HEAP16[(($_flags)>>1)]; //@line 422 "readclause.cpp"
        var $conv=(tempInt=(($3)),(tempInt>=32768?tempInt-65536:tempInt)); //@line 422 "readclause.cpp"
        var $and=($conv) & 32; //@line 422 "readclause.cpp"
        var $cmp1=((($and))|0)!=0; //@line 422 "readclause.cpp"
        if ($cmp1) { __label__ = 2;; } else { __label__ = 3;; } //@line 422 "readclause.cpp"
        if (__label__ == 2) {
  
          $c=32; //@line 422 "readclause.cpp"
          ; //@line 422 "readclause.cpp"
        }
  
        var $4=HEAP32[((_option_multibyte)>>2)]; //@line 424 "readclause.cpp"
        var $cmp3=((($4))|0)==4; //@line 424 "readclause.cpp"
        if ($cmp3) { __label__ = 4;; } else { __label__ = 7;; } //@line 424 "readclause.cpp"
        if (__label__ == 4) {
  
          var $5=HEAP32[((__ZL7f_input)>>2)]; //@line 426 "readclause.cpp"
          var $call5=_fgetc($5); //@line 426 "readclause.cpp"
          $c2=$call5; //@line 426 "readclause.cpp"
          var $6=HEAP32[((__ZL7f_input)>>2)]; //@line 427 "readclause.cpp"
          var $_flags6=(($6+12)&4294967295); //@line 427 "readclause.cpp"
          var $7=HEAP16[(($_flags6)>>1)]; //@line 427 "readclause.cpp"
          var $conv7=(tempInt=(($7)),(tempInt>=32768?tempInt-65536:tempInt)); //@line 427 "readclause.cpp"
          var $and8=($conv7) & 32; //@line 427 "readclause.cpp"
          var $cmp9=((($and8))|0)!=0; //@line 427 "readclause.cpp"
          if ($cmp9) { __label__ = 5;; } else { __label__ = 6;; } //@line 427 "readclause.cpp"
          if (__label__ == 5) {
  
            $c2=0; //@line 427 "readclause.cpp"
            ; //@line 427 "readclause.cpp"
          }
  
          var $8=$c; //@line 428 "readclause.cpp"
          var $9=$c2; //@line 428 "readclause.cpp"
          var $shl=((($9)*256)&4294967295); //@line 428 "readclause.cpp"
          var $add=((($shl)+($8))&4294967295); //@line 428 "readclause.cpp"
          $c=$add; //@line 428 "readclause.cpp"
          ; //@line 429 "readclause.cpp"
        }
  
        var $10=$c; //@line 430 "readclause.cpp"
        $retval=$10; //@line 430 "readclause.cpp"
        ; //@line 430 "readclause.cpp"
      }
      else if (__label__ == 8) {
  
        var $11=HEAP32[((_option_multibyte)>>2)]; //@line 433 "readclause.cpp"
        var $cmp14=((($11))|0)==3; //@line 433 "readclause.cpp"
        if ($cmp14) { __label__ = 9;; } else { __label__ = 13;; } //@line 433 "readclause.cpp"
        $if_then15$$if_else$14: do { 
          if (__label__ == 9) {
  
            var $12=HEAP32[((_p_wchar_input)>>2)]; //@line 435 "readclause.cpp"
            var $13=HEAP32[(($12)>>2)]; //@line 435 "readclause.cpp"
            var $cmp16=((($13))|0)==0; //@line 435 "readclause.cpp"
            if ($cmp16) { __label__ = 10;; } else { __label__ = 11;; } //@line 435 "readclause.cpp"
            if (__label__ == 10) {
  
              HEAP8[(__ZL12end_of_input_b)]=1;
              $retval=0; //@line 438 "readclause.cpp"
              __label__ = 20;break $if_then$$if_end13$2; //@line 438 "readclause.cpp"
            }
            else if (__label__ == 11) {
  
              var $_b1=HEAPU8[(__ZL12end_of_input_b)];
              var $14=((($_b1))&1);
              var $tobool=((($14))|0)!=0; //@line 441 "readclause.cpp"
              if ($tobool) { __label__ = 19;break $if_then15$$if_else$14; } //@line 441 "readclause.cpp"
  
              var $15=HEAP32[((_p_wchar_input)>>2)]; //@line 442 "readclause.cpp"
              var $incdec_ptr=(($15+4)&4294967295); //@line 442 "readclause.cpp"
              HEAP32[((_p_wchar_input)>>2)]=$incdec_ptr; //@line 442 "readclause.cpp"
              var $16=HEAP32[(($15)>>2)]; //@line 442 "readclause.cpp"
              $retval=$16; //@line 442 "readclause.cpp"
              __label__ = 20;break $if_then$$if_end13$2; //@line 442 "readclause.cpp"
            }
          }
          else if (__label__ == 13) {
  
            var $17=HEAP32[((_p_textinput)>>2)]; //@line 446 "readclause.cpp"
            var $18=HEAPU8[($17)]; //@line 446 "readclause.cpp"
            var $conv21=((($18))&255); //@line 446 "readclause.cpp"
            var $cmp22=((($conv21))|0)==0; //@line 446 "readclause.cpp"
            if ($cmp22) { __label__ = 14;; } else { __label__ = 15;; } //@line 446 "readclause.cpp"
            if (__label__ == 14) {
  
              HEAP8[(__ZL12end_of_input_b)]=1;
              $retval=0; //@line 449 "readclause.cpp"
              __label__ = 20;break $if_then$$if_end13$2; //@line 449 "readclause.cpp"
            }
            else if (__label__ == 15) {
  
              var $_b=HEAPU8[(__ZL12end_of_input_b)];
              var $19=((($_b))&1);
              var $tobool25=((($19))|0)!=0; //@line 452 "readclause.cpp"
              if ($tobool25) { __label__ = 19;break $if_then15$$if_else$14; } //@line 452 "readclause.cpp"
  
              var $20=HEAP32[((_option_multibyte)>>2)]; //@line 454 "readclause.cpp"
              var $cmp27=((($20))|0)==4; //@line 454 "readclause.cpp"
              var $21=HEAPU32[((_p_textinput)>>2)]; //@line 456 "readclause.cpp"
              if ($cmp27) { __label__ = 17;; } else { __label__ = 18;; } //@line 454 "readclause.cpp"
              if (__label__ == 17) {
  
                var $arrayidx=(($21)&4294967295); //@line 456 "readclause.cpp"
                var $22=HEAPU8[($arrayidx)]; //@line 456 "readclause.cpp"
                var $conv29=((($22))&255); //@line 456 "readclause.cpp"
                var $23=HEAP32[((_p_textinput)>>2)]; //@line 456 "readclause.cpp"
                var $arrayidx30=(($23+1)&4294967295); //@line 456 "readclause.cpp"
                var $24=HEAPU8[($arrayidx30)]; //@line 456 "readclause.cpp"
                var $conv31=((($24))&255); //@line 456 "readclause.cpp"
                var $shl32=((($conv31)*256)&4294967295); //@line 456 "readclause.cpp"
                var $add33=((($shl32)+($conv29))&4294967295); //@line 456 "readclause.cpp"
                $c=$add33; //@line 456 "readclause.cpp"
                var $25=HEAP32[((_p_textinput)>>2)]; //@line 457 "readclause.cpp"
                var $add_ptr=(($25+2)&4294967295); //@line 457 "readclause.cpp"
                HEAP32[((_p_textinput)>>2)]=$add_ptr; //@line 457 "readclause.cpp"
                var $26=$c; //@line 458 "readclause.cpp"
                $retval=$26; //@line 458 "readclause.cpp"
                __label__ = 20;break $if_then$$if_end13$2; //@line 458 "readclause.cpp"
              }
              else if (__label__ == 18) {
  
                var $incdec_ptr35=(($21+1)&4294967295); //@line 460 "readclause.cpp"
                HEAP32[((_p_textinput)>>2)]=$incdec_ptr35; //@line 460 "readclause.cpp"
                var $27=HEAPU8[($21)]; //@line 460 "readclause.cpp"
                var $conv36=((($27))&255); //@line 460 "readclause.cpp"
                var $and37=($conv36) & 255; //@line 460 "readclause.cpp"
                $retval=$and37; //@line 460 "readclause.cpp"
                __label__ = 20;break $if_then$$if_end13$2; //@line 460 "readclause.cpp"
              }
            }
          }
        } while(0);
  
        $retval=0; //@line 463 "readclause.cpp"
        ; //@line 463 "readclause.cpp"
      }
    } while(0);
  
    var $28=$retval; //@line 464 "readclause.cpp"
    ;
    return $28; //@line 464 "readclause.cpp"
    return null;
  }
  

  function __ZL10GetC_ungeti($c) {
    ;
    var __label__;
  
    var $c_addr;
    $c_addr=$c;
    var $0=HEAP32[((__ZL7f_input)>>2)]; //@line 392 "readclause.cpp"
    var $cmp=((($0))|0)!=0; //@line 392 "readclause.cpp"
    if ($cmp) { __label__ = 1;; } else { __label__ = 2;; } //@line 392 "readclause.cpp"
    if (__label__ == 1) {
  
      var $1=$c_addr; //@line 393 "readclause.cpp"
      var $2=HEAP32[((__ZL7f_input)>>2)]; //@line 393 "readclause.cpp"
      var $call=_ungetc($1, $2); //@line 393 "readclause.cpp"
      ; //@line 393 "readclause.cpp"
    }
    else if (__label__ == 2) {
  
      var $3=HEAP32[((_p_textinput)>>2)]; //@line 396 "readclause.cpp"
      var $incdec_ptr=(($3+-1)&4294967295); //@line 396 "readclause.cpp"
      HEAP32[((_p_textinput)>>2)]=$incdec_ptr; //@line 396 "readclause.cpp"
      var $4=$c_addr; //@line 397 "readclause.cpp"
      var $conv=((($4)) & 255); //@line 397 "readclause.cpp"
      var $5=HEAP32[((_p_textinput)>>2)]; //@line 397 "readclause.cpp"
      HEAP8[($5)]=$conv; //@line 397 "readclause.cpp"
      HEAP8[(__ZL12end_of_input_b)]=0;
      ;
    }
  
    ;
    return; //@line 400 "readclause.cpp"
    return;
  }
  

  function __Z8SetSpeedi($control) {
    ;
    var __label__;
    var __lastLabel__ = null;
  
    var $control_addr;
    var $x;
    var $s1;
    var $wpm;
    var $wpm2;
    var $wpm_value;
    var $sonic;
    $control_addr=$control;
    HEAP32[((((_speed+28)&4294967295))>>2)]=0; //@line 147 "setlengths.cpp"
    HEAP32[((((_speed+24)&4294967295))>>2)]=450; //@line 148 "setlengths.cpp"
    HEAP32[((((_speed+16)&4294967295))>>2)]=110; //@line 149 "setlengths.cpp"
    HEAP32[((((_speed+20)&4294967295))>>2)]=100; //@line 150 "setlengths.cpp"
    HEAP32[((((_speed+8)&4294967295))>>2)]=5; //@line 151 "setlengths.cpp"
    var $0=HEAP32[((((_embedded_value+8)&4294967295))>>2)]; //@line 153 "setlengths.cpp"
    $wpm=$0; //@line 153 "setlengths.cpp"
    var $1=$control_addr; //@line 154 "setlengths.cpp"
    var $cmp=((($1))|0)==2; //@line 154 "setlengths.cpp"
    if ($cmp) { __label__ = 1;; } else { __label__ = 2;; } //@line 154 "setlengths.cpp"
    if (__label__ == 1) {
  
      var $2=HEAP32[((((_embedded_value+32)&4294967295))>>2)]; //@line 155 "setlengths.cpp"
      $wpm=$2; //@line 155 "setlengths.cpp"
      ; //@line 155 "setlengths.cpp"
    }
  
    var $3=$wpm; //@line 157 "setlengths.cpp"
    $wpm_value=$3; //@line 157 "setlengths.cpp"
    var $4=HEAP32[((_voice)>>2)]; //@line 159 "setlengths.cpp"
    var $speed_percent=(($4+64)&4294967295); //@line 159 "setlengths.cpp"
    var $5=HEAP32[(($speed_percent)>>2)]; //@line 159 "setlengths.cpp"
    var $cmp1=((($5))|0) > 0; //@line 159 "setlengths.cpp"
    if ($cmp1) { __label__ = 3;; } else { __label__ = 4;; } //@line 159 "setlengths.cpp"
    if (__label__ == 3) {
  
      var $6=$wpm; //@line 161 "setlengths.cpp"
      var $7=HEAP32[((_voice)>>2)]; //@line 161 "setlengths.cpp"
      var $speed_percent3=(($7+64)&4294967295); //@line 161 "setlengths.cpp"
      var $8=HEAP32[(($speed_percent3)>>2)]; //@line 161 "setlengths.cpp"
      var $mul=((($8)*($6))&4294967295); //@line 161 "setlengths.cpp"
      var $div=((((($mul))|0)/100)|0); //@line 161 "setlengths.cpp"
      $wpm=$div; //@line 161 "setlengths.cpp"
      ; //@line 162 "setlengths.cpp"
    }
  
    var $9=$control_addr; //@line 164 "setlengths.cpp"
    var $and=($9) & 2; //@line 164 "setlengths.cpp"
    var $tobool=((($and))|0)!=0; //@line 164 "setlengths.cpp"
    if ($tobool) { __label__ = 5;; } else { __label__ = 6;; } //@line 164 "setlengths.cpp"
    if (__label__ == 5) {
  
      __Z12DoSonicSpeedi(1024); //@line 166 "setlengths.cpp"
      ; //@line 167 "setlengths.cpp"
    }
  
    var $10=$wpm_value; //@line 168 "setlengths.cpp"
    var $cmp7=((($10))|0) > 450; //@line 168 "setlengths.cpp"
    if ($cmp7) { __label__ = 10;; } else { __label__ = 7;; } //@line 168 "setlengths.cpp"
    $if_then10$$lor_lhs_false$11: do { 
      if (__label__ == 7) {
  
        var $11=$wpm_value; //@line 168 "setlengths.cpp"
        var $12=HEAP32[((((_speed+32)&4294967295))>>2)]; //@line 168 "setlengths.cpp"
        var $cmp8=((($11))|0) > ((($12))|0); //@line 168 "setlengths.cpp"
        var $13=$wpm; //@line 168 "setlengths.cpp"
        if ($cmp8) { __label__ = 8;; } else { __label__ = 14;; } //@line 168 "setlengths.cpp"
        $land_lhs_true$$if_end29$13: do { 
          if (__label__ == 8) {
  
            var $cmp9=((($13))|0) > 350; //@line 168 "setlengths.cpp"
            if ($cmp9) { __label__ = 10;break $if_then10$$lor_lhs_false$11; } //@line 168 "setlengths.cpp"
  
            var $_pr2=$wpm; //@line 210 "setlengths.cpp"
            __lastLabel__ = 9; __label__ = 16;break $land_lhs_true$$if_end29$13; //@line 207 "setlengths.cpp"
          }
          else if (__label__ == 14) {
  
            var $cmp30=((($13))|0) > 450; //@line 207 "setlengths.cpp"
            if ($cmp30) { __lastLabel__ = 14; ; } else { __lastLabel__ = 14; __label__ = 16;break $land_lhs_true$$if_end29$13; } //@line 207 "setlengths.cpp"
  
            $wpm=450; //@line 208 "setlengths.cpp"
            __label__ = 17;break $land_lhs_true$$if_end29$13; //@line 210 "setlengths.cpp"
          }
        } while(0);
        $if_end32$$if_then34$18: do { 
          if (__label__ == 16) {
  
            var $29=__lastLabel__ == 9 ? $_pr2 : ($13); //@line 210 "setlengths.cpp"
            var $cmp33=((($29))|0) > 360; //@line 210 "setlengths.cpp"
            if ($cmp33) { __label__ = 17;break $if_end32$$if_then34$18; } else { __label__ = 18;break $if_end32$$if_then34$18; } //@line 210 "setlengths.cpp"
          }
        } while(0);
        if (__label__ == 17) {
  
          var $30=$wpm; //@line 212 "setlengths.cpp"
          var $sub=((($30)-360)&4294967295); //@line 212 "setlengths.cpp"
          var $div35=((((($sub))|0)/8)|0); //@line 212 "setlengths.cpp"
          HEAP32[((((_speed+28)&4294967295))>>2)]=$div35; //@line 212 "setlengths.cpp"
          ; //@line 213 "setlengths.cpp"
        }
  
        var $31=$wpm; //@line 215 "setlengths.cpp"
        $wpm2=$31; //@line 215 "setlengths.cpp"
        var $cmp37=((($31))|0) > 359; //@line 216 "setlengths.cpp"
        if ($cmp37) { __lastLabel__ = 18; __label__ = 19;; } else { __lastLabel__ = 18; __label__ = 20;; } //@line 216 "setlengths.cpp"
        if (__label__ == 19) {
  
          $wpm2=359; //@line 216 "setlengths.cpp"
          var $_pr4=$wpm; //@line 217 "setlengths.cpp"
          __lastLabel__ = 19; ; //@line 216 "setlengths.cpp"
        }
  
        var $32=__lastLabel__ == 19 ? $_pr4 : ($31); //@line 217 "setlengths.cpp"
        var $cmp40=((($32))|0) < 80; //@line 217 "setlengths.cpp"
        if ($cmp40) { __label__ = 21;; } else { __label__ = 22;; } //@line 217 "setlengths.cpp"
        if (__label__ == 21) {
  
          $wpm2=80; //@line 217 "setlengths.cpp"
          ; //@line 217 "setlengths.cpp"
        }
  
        var $33=$wpm2; //@line 218 "setlengths.cpp"
        var $sub43=((($33)-80)&4294967295); //@line 218 "setlengths.cpp"
        var $arrayidx=((__ZL12speed_lookup+$sub43)&4294967295); //@line 218 "setlengths.cpp"
        var $34=HEAPU8[($arrayidx)]; //@line 218 "setlengths.cpp"
        var $conv44=((($34))&255); //@line 218 "setlengths.cpp"
        $x=$conv44; //@line 218 "setlengths.cpp"
        var $35=$wpm; //@line 220 "setlengths.cpp"
        var $cmp45=((($35))|0) >= 380; //@line 220 "setlengths.cpp"
        if ($cmp45) { __label__ = 23;; } else { __label__ = 25;; } //@line 220 "setlengths.cpp"
        $if_end47$$if_end50$29: do { 
          if (__label__ == 23) {
  
            $x=7; //@line 221 "setlengths.cpp"
            var $_pr5=$wpm; //@line 222 "setlengths.cpp"
            var $cmp48=((($_pr5))|0) >= 400; //@line 222 "setlengths.cpp"
            if (!($cmp48)) { __label__ = 25;break $if_end47$$if_end50$29; } //@line 222 "setlengths.cpp"
  
            $x=6; //@line 223 "setlengths.cpp"
            ; //@line 223 "setlengths.cpp"
          }
        } while(0);
  
        var $36=$control_addr; //@line 225 "setlengths.cpp"
        var $and51=($36) & 1; //@line 225 "setlengths.cpp"
        var $tobool52=((($and51))|0)!=0; //@line 225 "setlengths.cpp"
        if ($tobool52) { __label__ = 26;; } else { __label__ = 28;; } //@line 225 "setlengths.cpp"
        $if_then53$$if_end67$33: do { 
          if (__label__ == 26) {
  
            var $37=$x; //@line 229 "setlengths.cpp"
            var $38=HEAP32[((_voice)>>2)]; //@line 229 "setlengths.cpp"
            var $speedf154=(($38+52)&4294967295); //@line 229 "setlengths.cpp"
            var $39=HEAP32[(($speedf154)>>2)]; //@line 229 "setlengths.cpp"
            var $mul55=((($39)*($37))&4294967295); //@line 229 "setlengths.cpp"
            var $div56=((((($mul55))|0)/256)|0); //@line 229 "setlengths.cpp"
            HEAP32[((__ZL6speed1)>>2)]=$div56; //@line 229 "setlengths.cpp"
            var $40=$x; //@line 230 "setlengths.cpp"
            var $41=HEAP32[((_voice)>>2)]; //@line 230 "setlengths.cpp"
            var $speedf257=(($41+56)&4294967295); //@line 230 "setlengths.cpp"
            var $42=HEAP32[(($speedf257)>>2)]; //@line 230 "setlengths.cpp"
            var $mul58=((($42)*($40))&4294967295); //@line 230 "setlengths.cpp"
            var $div59=((((($mul58))|0)/256)|0); //@line 230 "setlengths.cpp"
            HEAP32[((__ZL6speed2)>>2)]=$div59; //@line 230 "setlengths.cpp"
            var $43=$x; //@line 231 "setlengths.cpp"
            var $44=HEAP32[((_voice)>>2)]; //@line 231 "setlengths.cpp"
            var $speedf360=(($44+60)&4294967295); //@line 231 "setlengths.cpp"
            var $45=HEAP32[(($speedf360)>>2)]; //@line 231 "setlengths.cpp"
            var $mul61=((($45)*($43))&4294967295); //@line 231 "setlengths.cpp"
            var $div62=((((($mul61))|0)/256)|0); //@line 231 "setlengths.cpp"
            HEAP32[((__ZL6speed3)>>2)]=$div62; //@line 231 "setlengths.cpp"
            var $46=$x; //@line 233 "setlengths.cpp"
            var $cmp63=((($46))|0) <= 7; //@line 233 "setlengths.cpp"
            if (!($cmp63)) { __label__ = 28;break $if_then53$$if_end67$33; } //@line 233 "setlengths.cpp"
  
            var $47=$x; //@line 235 "setlengths.cpp"
            HEAP32[((__ZL6speed1)>>2)]=$47; //@line 235 "setlengths.cpp"
            var $48=$x; //@line 236 "setlengths.cpp"
            var $sub65=((($48)-1)&4294967295); //@line 236 "setlengths.cpp"
            HEAP32[((__ZL6speed3)>>2)]=$sub65; //@line 236 "setlengths.cpp"
            HEAP32[((__ZL6speed2)>>2)]=$sub65; //@line 236 "setlengths.cpp"
            ; //@line 237 "setlengths.cpp"
          }
        } while(0);
  
        var $49=$control_addr; //@line 240 "setlengths.cpp"
        var $and68=($49) & 2; //@line 240 "setlengths.cpp"
        var $tobool69=((($and68))|0)!=0; //@line 240 "setlengths.cpp"
        if (!($tobool69)) { __label__ = 51;break $if_then10$$lor_lhs_false$11; } //@line 240 "setlengths.cpp"
  
        var $50=$wpm; //@line 244 "setlengths.cpp"
        var $cmp71=((($50))|0) > 350; //@line 244 "setlengths.cpp"
        var $51=$wpm; //@line 246 "setlengths.cpp"
        if ($cmp71) { __label__ = 30;; } else { __label__ = 31;; } //@line 244 "setlengths.cpp"
        $if_then72$$if_else$38: do { 
          if (__label__ == 30) {
  
            var $sub73=((($51)-350)&4294967295); //@line 246 "setlengths.cpp"
            var $div74=((((($sub73))|0)/3)|0); //@line 246 "setlengths.cpp"
            var $sub75=((85-($div74))&4294967295); //@line 246 "setlengths.cpp"
            HEAP32[((((_speed+16)&4294967295))>>2)]=$sub75; //@line 246 "setlengths.cpp"
            var $52=$wpm; //@line 247 "setlengths.cpp"
            var $sub76=((($52)-350)&4294967295); //@line 247 "setlengths.cpp"
            var $div77=((((($sub76))|0)/8)|0); //@line 247 "setlengths.cpp"
            var $sub78=((60-($div77))&4294967295); //@line 247 "setlengths.cpp"
            HEAP32[((((_speed+20)&4294967295))>>2)]=$sub78; //@line 247 "setlengths.cpp"
            ; //@line 248 "setlengths.cpp"
          }
          else if (__label__ == 31) {
  
            var $cmp79=((($51))|0) > 250; //@line 250 "setlengths.cpp"
            if (!($cmp79)) { __label__ = 33;break $if_then72$$if_else$38; } //@line 250 "setlengths.cpp"
  
            var $53=$wpm; //@line 252 "setlengths.cpp"
            var $sub81=((($53)-250)&4294967295); //@line 252 "setlengths.cpp"
            var $div82=((((($sub81))|0)/4)|0); //@line 252 "setlengths.cpp"
            var $sub83=((110-($div82))&4294967295); //@line 252 "setlengths.cpp"
            HEAP32[((((_speed+16)&4294967295))>>2)]=$sub83; //@line 252 "setlengths.cpp"
            var $54=$wpm; //@line 253 "setlengths.cpp"
            var $sub84=((($54)-250)&4294967295); //@line 253 "setlengths.cpp"
            var $div85=((((($sub84))|0)/2)|0); //@line 253 "setlengths.cpp"
            var $sub86=((110-($div85))&4294967295); //@line 253 "setlengths.cpp"
            HEAP32[((((_speed+20)&4294967295))>>2)]=$sub86; //@line 253 "setlengths.cpp"
            ; //@line 254 "setlengths.cpp"
          }
        } while(0);
  
        var $55=$x; //@line 256 "setlengths.cpp"
        var $56=HEAP32[((_voice)>>2)]; //@line 256 "setlengths.cpp"
        var $speedf189=(($56+52)&4294967295); //@line 256 "setlengths.cpp"
        var $57=HEAP32[(($speedf189)>>2)]; //@line 256 "setlengths.cpp"
        var $mul90=((($57)*($55))&4294967295); //@line 256 "setlengths.cpp"
        var $div91=((((($mul90))|0)/256)|0); //@line 256 "setlengths.cpp"
        $s1=$div91; //@line 256 "setlengths.cpp"
        var $58=$wpm; //@line 258 "setlengths.cpp"
        var $cmp92=((($58))|0) >= 170; //@line 258 "setlengths.cpp"
        var $59=$s1; //@line 259 "setlengths.cpp"
        if ($cmp92) { __label__ = 34;; } else { __label__ = 35;; } //@line 258 "setlengths.cpp"
        if (__label__ == 34) {
  
          var $mul94=((($59)*150)&4294967295); //@line 259 "setlengths.cpp"
          var $div95=((((($mul94))|0)/128)|0); //@line 259 "setlengths.cpp"
          var $add=((($div95)+110)&4294967295); //@line 259 "setlengths.cpp"
          HEAP32[((((_speed+12)&4294967295))>>2)]=$add; //@line 259 "setlengths.cpp"
          ; //@line 259 "setlengths.cpp"
        }
        else if (__label__ == 35) {
  
          var $mul97=((($59)*128)&4294967295); //@line 261 "setlengths.cpp"
          var $div98=((((($mul97))|0)/130)|0); //@line 261 "setlengths.cpp"
          var $add99=((($div98)+128)&4294967295); //@line 261 "setlengths.cpp"
          HEAP32[((((_speed+12)&4294967295))>>2)]=$add99; //@line 261 "setlengths.cpp"
          ;
        }
  
        var $60=$wpm; //@line 263 "setlengths.cpp"
        var $cmp101=((($60))|0) >= 350; //@line 263 "setlengths.cpp"
        if ($cmp101) { __label__ = 37;; } else { __label__ = 40;; } //@line 263 "setlengths.cpp"
        $if_end106$$if_end117$47: do { 
          if (__label__ == 37) {
  
            var $61=$wpm; //@line 265 "setlengths.cpp"
            var $sub103=((($61)-350)&4294967295); //@line 265 "setlengths.cpp"
            var $arrayidx104=((__ZL14wav_factor_350+$sub103)&4294967295); //@line 265 "setlengths.cpp"
            var $62=HEAPU8[($arrayidx104)]; //@line 265 "setlengths.cpp"
            var $conv105=((($62))&255); //@line 265 "setlengths.cpp"
            HEAP32[((((_speed+12)&4294967295))>>2)]=$conv105; //@line 265 "setlengths.cpp"
            var $_pr7=$wpm; //@line 268 "setlengths.cpp"
            var $cmp107=((($_pr7))|0) >= 390; //@line 268 "setlengths.cpp"
            if (!($cmp107)) { __label__ = 40;break $if_end106$$if_end117$47; } //@line 268 "setlengths.cpp"
  
            var $63=$wpm; //@line 270 "setlengths.cpp"
            var $sub109=((($63)-400)&4294967295); //@line 270 "setlengths.cpp"
            var $div110=((((($sub109))|0)/2)|0); //@line 270 "setlengths.cpp"
            var $sub111=((450-($div110))&4294967295); //@line 270 "setlengths.cpp"
            HEAP32[((((_speed+24)&4294967295))>>2)]=$sub111; //@line 270 "setlengths.cpp"
            var $64=$wpm; //@line 271 "setlengths.cpp"
            var $cmp112=((($64))|0) > 440; //@line 271 "setlengths.cpp"
            if (!($cmp112)) { __label__ = 40;break $if_end106$$if_end117$47; } //@line 271 "setlengths.cpp"
  
            var $65=$wpm; //@line 272 "setlengths.cpp"
            var $_neg=(((-($65)))&4294967295);
            var $sub115=((($_neg)+860)&4294967295); //@line 272 "setlengths.cpp"
            HEAP32[((((_speed+24)&4294967295))>>2)]=$sub115; //@line 272 "setlengths.cpp"
            ; //@line 272 "setlengths.cpp"
          }
        } while(0);
  
        var $66=$s1; //@line 275 "setlengths.cpp"
        var $mul118=((($66)*256)&4294967295); //@line 275 "setlengths.cpp"
        var $div119=((((($mul118))|0)/115)|0); //@line 275 "setlengths.cpp"
        HEAP32[((((_speed)&4294967295))>>2)]=$div119; //@line 275 "setlengths.cpp"
        HEAP32[((((_speed+4)&4294967295))>>2)]=0; //@line 276 "setlengths.cpp"
        var $67=$wpm; //@line 278 "setlengths.cpp"
        var $cmp120=((($67))|0) > 430; //@line 278 "setlengths.cpp"
        if ($cmp120) { __label__ = 41;; } else { __label__ = 42;; } //@line 278 "setlengths.cpp"
        $if_then121$$if_else122$52: do { 
          if (__label__ == 41) {
  
            HEAP32[((((_speed)&4294967295))>>2)]=12; //@line 280 "setlengths.cpp"
            ; //@line 282 "setlengths.cpp"
          }
          else if (__label__ == 42) {
  
            var $68=$wpm; //@line 284 "setlengths.cpp"
            var $cmp123=((($68))|0) > 400; //@line 284 "setlengths.cpp"
            if ($cmp123) { __label__ = 43;; } else { __label__ = 44;; } //@line 284 "setlengths.cpp"
            if (__label__ == 43) {
  
              HEAP32[((((_speed)&4294967295))>>2)]=13; //@line 286 "setlengths.cpp"
              ; //@line 288 "setlengths.cpp"
            }
            else if (__label__ == 44) {
  
              var $69=$wpm; //@line 290 "setlengths.cpp"
              var $cmp126=((($69))|0) > 374; //@line 290 "setlengths.cpp"
              if ($cmp126) { __label__ = 45;; } else { __label__ = 46;; } //@line 290 "setlengths.cpp"
              if (__label__ == 45) {
  
                HEAP32[((((_speed)&4294967295))>>2)]=14; //@line 292 "setlengths.cpp"
                ; //@line 293 "setlengths.cpp"
              }
              else if (__label__ == 46) {
  
                var $70=$wpm; //@line 295 "setlengths.cpp"
                var $cmp129=((($70))|0) > 350; //@line 295 "setlengths.cpp"
                if (!($cmp129)) { __label__ = 48;break $if_then121$$if_else122$52; } //@line 295 "setlengths.cpp"
  
                var $71=$wpm; //@line 297 "setlengths.cpp"
                var $sub131=((($71)-350)&4294967295); //@line 297 "setlengths.cpp"
                var $arrayidx132=((__ZL16pause_factor_350+$sub131)&4294967295); //@line 297 "setlengths.cpp"
                var $72=HEAPU8[($arrayidx132)]; //@line 297 "setlengths.cpp"
                var $conv133=((($72))&255); //@line 297 "setlengths.cpp"
                HEAP32[((((_speed)&4294967295))>>2)]=$conv133; //@line 297 "setlengths.cpp"
                ; //@line 298 "setlengths.cpp"
              }
            }
          }
        } while(0);
  
        var $73=HEAP32[((((_speed+4)&4294967295))>>2)]; //@line 300 "setlengths.cpp"
        var $cmp138=((($73))|0)==0; //@line 300 "setlengths.cpp"
        if (!($cmp138)) { __label__ = 51;break $if_then10$$lor_lhs_false$11; } //@line 300 "setlengths.cpp"
  
        var $74=HEAP32[((((_speed)&4294967295))>>2)]; //@line 303 "setlengths.cpp"
        HEAP32[((((_speed+4)&4294967295))>>2)]=$74; //@line 303 "setlengths.cpp"
        var $cmp140=((($74))|0) < 16; //@line 303 "setlengths.cpp"
        if (!($cmp140)) { __label__ = 51;break $if_then10$$lor_lhs_false$11; } //@line 303 "setlengths.cpp"
  
        HEAP32[((((_speed+4)&4294967295))>>2)]=16; //@line 304 "setlengths.cpp"
        __label__ = 51;break $if_then10$$lor_lhs_false$11; //@line 304 "setlengths.cpp"
      }
    } while(0);
    $if_then10$$if_end144$65: do { 
      if (__label__ == 10) {
  
        var $14=$wpm; //@line 170 "setlengths.cpp"
        $wpm2=$14; //@line 170 "setlengths.cpp"
        $wpm=175; //@line 171 "setlengths.cpp"
        $x=73; //@line 175 "setlengths.cpp"
        var $15=$control_addr; //@line 176 "setlengths.cpp"
        var $and11=($15) & 1; //@line 176 "setlengths.cpp"
        var $tobool12=((($and11))|0)!=0; //@line 176 "setlengths.cpp"
        if ($tobool12) { __label__ = 11;; } else { __label__ = 12;; } //@line 176 "setlengths.cpp"
        if (__label__ == 11) {
  
          var $16=$x; //@line 178 "setlengths.cpp"
          var $17=HEAP32[((_voice)>>2)]; //@line 178 "setlengths.cpp"
          var $speedf1=(($17+52)&4294967295); //@line 178 "setlengths.cpp"
          var $18=HEAP32[(($speedf1)>>2)]; //@line 178 "setlengths.cpp"
          var $mul14=((($18)*($16))&4294967295); //@line 178 "setlengths.cpp"
          var $div15=((((($mul14))|0)/256)|0); //@line 178 "setlengths.cpp"
          HEAP32[((__ZL6speed1)>>2)]=$div15; //@line 178 "setlengths.cpp"
          var $19=$x; //@line 179 "setlengths.cpp"
          var $20=HEAP32[((_voice)>>2)]; //@line 179 "setlengths.cpp"
          var $speedf2=(($20+56)&4294967295); //@line 179 "setlengths.cpp"
          var $21=HEAP32[(($speedf2)>>2)]; //@line 179 "setlengths.cpp"
          var $mul16=((($21)*($19))&4294967295); //@line 179 "setlengths.cpp"
          var $div17=((((($mul16))|0)/256)|0); //@line 179 "setlengths.cpp"
          HEAP32[((__ZL6speed2)>>2)]=$div17; //@line 179 "setlengths.cpp"
          var $22=$x; //@line 180 "setlengths.cpp"
          var $23=HEAP32[((_voice)>>2)]; //@line 180 "setlengths.cpp"
          var $speedf3=(($23+60)&4294967295); //@line 180 "setlengths.cpp"
          var $24=HEAP32[(($speedf3)>>2)]; //@line 180 "setlengths.cpp"
          var $mul18=((($24)*($22))&4294967295); //@line 180 "setlengths.cpp"
          var $div19=((((($mul18))|0)/256)|0); //@line 180 "setlengths.cpp"
          HEAP32[((__ZL6speed3)>>2)]=$div19; //@line 180 "setlengths.cpp"
          ; //@line 181 "setlengths.cpp"
        }
  
        var $25=$control_addr; //@line 182 "setlengths.cpp"
        var $and21=($25) & 2; //@line 182 "setlengths.cpp"
        var $tobool22=((($and21))|0)!=0; //@line 182 "setlengths.cpp"
        if (!($tobool22)) { __label__ = 51;break $if_then10$$if_end144$65; } //@line 182 "setlengths.cpp"
  
        var $26=$wpm2; //@line 184 "setlengths.cpp"
        var $conv=((($26))|0); //@line 184 "setlengths.cpp"
        var $27=$wpm; //@line 184 "setlengths.cpp"
        var $conv24=((($27))|0); //@line 184 "setlengths.cpp"
        var $div25=($conv)/($conv24); //@line 184 "setlengths.cpp"
        $sonic=$div25; //@line 184 "setlengths.cpp"
        var $28=$sonic; //@line 185 "setlengths.cpp"
        var $mul26=($28)*1024; //@line 185 "setlengths.cpp"
        var $conv27=((($mul26))|0); //@line 185 "setlengths.cpp"
        __Z12DoSonicSpeedi($conv27); //@line 185 "setlengths.cpp"
        HEAP32[((((_speed)&4294967295))>>2)]=85; //@line 186 "setlengths.cpp"
        HEAP32[((((_speed+4)&4294967295))>>2)]=80; //@line 187 "setlengths.cpp"
        HEAP32[((((_speed+8)&4294967295))>>2)]=22; //@line 188 "setlengths.cpp"
        HEAP32[((((_speed+24)&4294967295))>>2)]=900; //@line 189 "setlengths.cpp"
        HEAP32[((((_speed+12)&4294967295))>>2)]=211; //@line 190 "setlengths.cpp"
        HEAP32[((((_speed+16)&4294967295))>>2)]=210; //@line 191 "setlengths.cpp"
        HEAP32[((((_speed+20)&4294967295))>>2)]=170; //@line 192 "setlengths.cpp"
        ; //@line 193 "setlengths.cpp"
      }
    } while(0);
  
    ;
    return; //@line 312 "setlengths.cpp"
    return;
  }
  

  function __Z12SetParameteriii($parameter, $value, $relative) {
    ;
    var __label__;
  
    var $parameter_addr;
    var $value_addr;
    var $relative_addr;
    var $new_value;
    var $default_value;
    $parameter_addr=$parameter;
    $value_addr=$value;
    $relative_addr=$relative;
    var $0=$value_addr; //@line 334 "setlengths.cpp"
    $new_value=$0; //@line 334 "setlengths.cpp"
    var $1=$relative_addr; //@line 337 "setlengths.cpp"
    var $tobool=((($1))|0)!=0; //@line 337 "setlengths.cpp"
    if ($tobool) { __label__ = 1;; } else { __label__ = 3;; } //@line 337 "setlengths.cpp"
    $if_then$$if_end2$2: do { 
      if (__label__ == 1) {
  
        var $2=$parameter_addr; //@line 339 "setlengths.cpp"
        var $cmp=((($2))|0) < 5; //@line 339 "setlengths.cpp"
        if (!($cmp)) { __label__ = 3;break $if_then$$if_end2$2; } //@line 339 "setlengths.cpp"
  
        var $3=$parameter_addr; //@line 341 "setlengths.cpp"
        var $arrayidx=((_param_defaults+$3*4)&4294967295); //@line 341 "setlengths.cpp"
        var $4=HEAP32[(($arrayidx)>>2)]; //@line 341 "setlengths.cpp"
        $default_value=$4; //@line 341 "setlengths.cpp"
        var $5=$default_value; //@line 342 "setlengths.cpp"
        var $6=$default_value; //@line 342 "setlengths.cpp"
        var $7=$value_addr; //@line 342 "setlengths.cpp"
        var $mul=((($7)*($6))&4294967295); //@line 342 "setlengths.cpp"
        var $div=((((($mul))|0)/100)|0); //@line 342 "setlengths.cpp"
        var $add=((($div)+($5))&4294967295); //@line 342 "setlengths.cpp"
        $new_value=$add; //@line 342 "setlengths.cpp"
        ; //@line 343 "setlengths.cpp"
      }
    } while(0);
  
    var $8=$new_value; //@line 345 "setlengths.cpp"
    var $9=$parameter_addr; //@line 345 "setlengths.cpp"
    var $arrayidx3=((((_param_stack+4)&4294967295)+$9*4)&4294967295); //@line 345 "setlengths.cpp"
    HEAP32[(($arrayidx3)>>2)]=$8; //@line 345 "setlengths.cpp"
    var $10=$parameter_addr; //@line 347 "setlengths.cpp"
    if ($10 == 1) {
      __label__ = 4;;
    }
    else if ($10 == 2) {
      __label__ = 5;;
    }
    else if ($10 == 3) {
      __label__ = 6;;
    }
    else if ($10 == 4) {
      __label__ = 11;;
    }
    else if ($10 == 13) {
      __label__ = 14;;
    }
    else if ($10 == 7) {
      __label__ = 15;;
    }
    else if ($10 == 9) {
      __label__ = 16;;
    }
    else {
    __label__ = 19;;
    }
    
    if (__label__ == 4) {
  
      var $11=$new_value; //@line 350 "setlengths.cpp"
      HEAP32[((((_embedded_value+8)&4294967295))>>2)]=$11; //@line 350 "setlengths.cpp"
      var $12=$new_value; //@line 351 "setlengths.cpp"
      HEAP32[((((_embedded_value+32)&4294967295))>>2)]=$12; //@line 351 "setlengths.cpp"
      __Z8SetSpeedi(3); //@line 352 "setlengths.cpp"
      ; //@line 353 "setlengths.cpp"
    }
    else if (__label__ == 5) {
  
      var $13=$new_value; //@line 356 "setlengths.cpp"
      HEAP32[((((_embedded_value+12)&4294967295))>>2)]=$13; //@line 356 "setlengths.cpp"
      var $call=__Z12GetAmplitudev(); //@line 357 "setlengths.cpp"
      ; //@line 358 "setlengths.cpp"
    }
    else if (__label__ == 6) {
  
      var $14=$new_value; //@line 361 "setlengths.cpp"
      var $cmp6=((($14))|0) > 99; //@line 361 "setlengths.cpp"
      if ($cmp6) { __label__ = 7;; } else { __label__ = 8;; } //@line 361 "setlengths.cpp"
      $if_end8_thread$$if_end8$10: do { 
        if (__label__ == 7) {
  
          $new_value=99; //@line 361 "setlengths.cpp"
          ; //@line 362 "setlengths.cpp"
        }
        else if (__label__ == 8) {
  
          var $cmp9=((($14))|0) < 0; //@line 362 "setlengths.cpp"
          if (!($cmp9)) { __label__ = 10;break $if_end8_thread$$if_end8$10; } //@line 362 "setlengths.cpp"
  
          $new_value=0; //@line 362 "setlengths.cpp"
          ; //@line 362 "setlengths.cpp"
        }
      } while(0);
  
      var $15=$new_value; //@line 363 "setlengths.cpp"
      HEAP32[((((_embedded_value+4)&4294967295))>>2)]=$15; //@line 363 "setlengths.cpp"
      ; //@line 364 "setlengths.cpp"
    }
    else if (__label__ == 11) {
  
      var $16=$new_value; //@line 367 "setlengths.cpp"
      var $cmp13=((($16))|0) > 99; //@line 367 "setlengths.cpp"
      if ($cmp13) { __label__ = 12;; } else { __label__ = 13;; } //@line 367 "setlengths.cpp"
      if (__label__ == 12) {
  
        $new_value=99; //@line 367 "setlengths.cpp"
        ; //@line 367 "setlengths.cpp"
      }
  
      var $17=$new_value; //@line 368 "setlengths.cpp"
      HEAP32[((((_embedded_value+16)&4294967295))>>2)]=$17; //@line 368 "setlengths.cpp"
      ; //@line 369 "setlengths.cpp"
    }
    else if (__label__ == 14) {
  
      var $18=$new_value; //@line 372 "setlengths.cpp"
      HEAP32[((_option_linelength)>>2)]=$18; //@line 372 "setlengths.cpp"
      ; //@line 373 "setlengths.cpp"
    }
    else if (__label__ == 15) {
  
      var $19=$new_value; //@line 376 "setlengths.cpp"
      HEAP32[((_option_wordgap)>>2)]=$19; //@line 376 "setlengths.cpp"
      ; //@line 377 "setlengths.cpp"
    }
    else if (__label__ == 16) {
  
      var $20=$new_value; //@line 380 "setlengths.cpp"
      var $and=($20) & 255; //@line 380 "setlengths.cpp"
      var $cmp19=((($and))|0)!=0; //@line 380 "setlengths.cpp"
      if ($cmp19) { __label__ = 17;; } else { __label__ = 18;; } //@line 380 "setlengths.cpp"
      if (__label__ == 17) {
  
        var $21=$new_value; //@line 381 "setlengths.cpp"
        var $and21=($21) & 255; //@line 381 "setlengths.cpp"
        var $22=HEAP32[((_translator)>>2)]; //@line 381 "setlengths.cpp"
        var $langopts=(($22)&4294967295); //@line 381 "setlengths.cpp"
        var $intonation_group=(($langopts+228)&4294967295); //@line 381 "setlengths.cpp"
        HEAP32[(($intonation_group)>>2)]=$and21; //@line 381 "setlengths.cpp"
        ; //@line 381 "setlengths.cpp"
      }
  
      var $23=$new_value; //@line 382 "setlengths.cpp"
      HEAP32[((_option_tone_flags)>>2)]=$23; //@line 382 "setlengths.cpp"
      ; //@line 383 "setlengths.cpp"
    }
  
    ;
    return; //@line 388 "setlengths.cpp"
    return;
  }
  

  function __Z11CalcLengthsP10Translator($tr) {
    var __stackBase__  = STACKTOP; STACKTOP += 156; _memset(__stackBase__, 0, 156);
    var __label__;
    var __lastLabel__ = null;
  
    var $tr_addr;
    var $ix;
    var $ix2;
    var $prev;
    var $next;
    var $next2;
    var $next3;
    var $p;
    var $p2;
    var $stress;
    var $type;
    var $pre_sonorant;
    var $pre_voiced;
    var $last_pitch;
    var $pitch_start;
    var $length_mod;
    var $len;
    var $env2;
    var $end_of_clause;
    var $embedded_ix=__stackBase__;
    var $min_drop;
    var $pitch1;
    var $emphasized;
    var $tone_mod;
    var $pitch_env;
    var $phdata_tone=__stackBase__+4;
    $tr_addr=$tr;
    $pre_sonorant=0; //@line 425 "setlengths.cpp"
    $pre_voiced=0; //@line 426 "setlengths.cpp"
    $last_pitch=0; //@line 427 "setlengths.cpp"
    HEAP32[(($embedded_ix)>>2)]=0; //@line 433 "setlengths.cpp"
    $pitch_env=0; //@line 438 "setlengths.cpp"
    $ix=1; //@line 441 "setlengths.cpp"
    var $0=$ix; //@line 441 "setlengths.cpp"
    var $1=HEAP32[((_n_phoneme_list)>>2)]; //@line 441 "setlengths.cpp"
    var $cmp7=((($0))|0) < ((($1))|0); //@line 441 "setlengths.cpp"
    if ($cmp7) { __label__ = 1;; } else { __label__ = 193;; } //@line 441 "setlengths.cpp"
    $for_body_lr_ph$$for_end733$2: do { 
      if (__label__ == 1) {
  
        var $pitch_env595=(($phdata_tone+124)&4294967295); //@line 779 "setlengths.cpp"
        ; //@line 441 "setlengths.cpp"
        $for_body$4: while(1) { 
  
          var $2=$ix; //@line 443 "setlengths.cpp"
          var $sub=((($2)-1)&4294967295); //@line 443 "setlengths.cpp"
          var $arrayidx=((_phoneme_list+$sub*24)&4294967295); //@line 443 "setlengths.cpp"
          $prev=$arrayidx; //@line 443 "setlengths.cpp"
          var $3=$ix; //@line 444 "setlengths.cpp"
          var $arrayidx1=((_phoneme_list+$3*24)&4294967295); //@line 444 "setlengths.cpp"
          $p=$arrayidx1; //@line 444 "setlengths.cpp"
          var $4=$p; //@line 445 "setlengths.cpp"
          var $stresslevel=(($4+1)&4294967295); //@line 445 "setlengths.cpp"
          var $5=HEAPU8[($stresslevel)]; //@line 445 "setlengths.cpp"
          var $conv=((($5))&255); //@line 445 "setlengths.cpp"
          var $and=($conv) & 7; //@line 445 "setlengths.cpp"
          $stress=$and; //@line 445 "setlengths.cpp"
          var $6=$p; //@line 446 "setlengths.cpp"
          var $stresslevel2=(($6+1)&4294967295); //@line 446 "setlengths.cpp"
          var $7=HEAPU8[($stresslevel2)]; //@line 446 "setlengths.cpp"
          var $conv3=((($7))&255); //@line 446 "setlengths.cpp"
          var $and4=($conv3) & 8; //@line 446 "setlengths.cpp"
          $emphasized=$and4; //@line 446 "setlengths.cpp"
          var $8=$ix; //@line 448 "setlengths.cpp"
          var $add=((($8)+1)&4294967295); //@line 448 "setlengths.cpp"
          var $arrayidx5=((_phoneme_list+$add*24)&4294967295); //@line 448 "setlengths.cpp"
          $next=$arrayidx5; //@line 448 "setlengths.cpp"
          var $9=$p; //@line 450 "setlengths.cpp"
          var $synthflags=(($9+4)&4294967295); //@line 450 "setlengths.cpp"
          var $10=HEAPU16[(($synthflags)>>1)]; //@line 450 "setlengths.cpp"
          var $conv6=((($10))&65535); //@line 450 "setlengths.cpp"
          var $and7=($conv6) & 2; //@line 450 "setlengths.cpp"
          var $tobool=((($and7))|0)!=0; //@line 450 "setlengths.cpp"
          if ($tobool) { __label__ = 3;; } else { __label__ = 4;; } //@line 450 "setlengths.cpp"
          if (__label__ == 3) {
  
            __ZL11DoEmbedded2Pi($embedded_ix); //@line 452 "setlengths.cpp"
            ; //@line 453 "setlengths.cpp"
          }
  
          var $11=$p; //@line 455 "setlengths.cpp"
          var $type8=(($11+15)&4294967295); //@line 455 "setlengths.cpp"
          var $12=HEAPU8[($type8)]; //@line 455 "setlengths.cpp"
          var $conv9=((($12))&255); //@line 455 "setlengths.cpp"
          $type=$conv9; //@line 455 "setlengths.cpp"
          var $13=$p; //@line 456 "setlengths.cpp"
          var $synthflags10=(($13+4)&4294967295); //@line 456 "setlengths.cpp"
          var $14=HEAPU16[(($synthflags10)>>1)]; //@line 456 "setlengths.cpp"
          var $conv11=((($14))&65535); //@line 456 "setlengths.cpp"
          var $and12=($conv11) & 4; //@line 456 "setlengths.cpp"
          var $tobool13=((($and12))|0)!=0; //@line 456 "setlengths.cpp"
          if ($tobool13) { __label__ = 5;; } else { __label__ = 6;; } //@line 456 "setlengths.cpp"
          $if_end15_thread$$if_end15$9: do { 
            if (__label__ == 5) {
  
              $type=2; //@line 457 "setlengths.cpp"
              __label__ = 98;break $if_end15_thread$$if_end15$9; //@line 459 "setlengths.cpp"
            }
            else if (__label__ == 6) {
  
              var $_pr=$type; //@line 459 "setlengths.cpp"
              if ($_pr == 0) {
                __label__ = 7;;
              }
              else if ($_pr == 4) {
                __label__ = 8;;
              }
              else if ($_pr == 7) {
                __label__ = 23;;
              }
              else if ($_pr == 6) {
                __label__ = 23;;
              }
              else if ($_pr == 5) {
                __label__ = 49;;
              }
              else if ($_pr == 3) {
                __label__ = 74;;
              }
              else if ($_pr == 8) {
                __label__ = 74;;
              }
              else if ($_pr == 2) {
                __label__ = 98;break $if_end15_thread$$if_end15$9;
              }
              else {
              __label__ = 192;break $if_end15_thread$$if_end15$9;
              }
              
              if (__label__ == 7) {
  
                $last_pitch=0; //@line 462 "setlengths.cpp"
                __label__ = 192;break $if_end15_thread$$if_end15$9; //@line 463 "setlengths.cpp"
              }
              else if (__label__ == 8) {
  
                $last_pitch=0; //@line 466 "setlengths.cpp"
                var $15=$prev; //@line 467 "setlengths.cpp"
                var $type17=(($15+15)&4294967295); //@line 467 "setlengths.cpp"
                var $16=HEAPU8[($type17)]; //@line 467 "setlengths.cpp"
                var $conv18=((($16))&255); //@line 467 "setlengths.cpp"
                var $cmp19=((($conv18))|0)==6; //@line 467 "setlengths.cpp"
                if ($cmp19) { __label__ = 9;; } else { __label__ = 10;; } //@line 467 "setlengths.cpp"
                $if_then20$$if_else$15: do { 
                  if (__label__ == 9) {
  
                    var $17=$p; //@line 468 "setlengths.cpp"
                    var $prepause=(($17+16)&4294967295); //@line 468 "setlengths.cpp"
                    HEAP8[($prepause)]=25; //@line 468 "setlengths.cpp"
                    ; //@line 468 "setlengths.cpp"
                  }
                  else if (__label__ == 10) {
  
                    var $18=HEAP32[((__ZZ11CalcLengthsP10TranslatorE14more_syllables)>>2)]; //@line 470 "setlengths.cpp"
                    var $cmp21=((($18))|0) > 0; //@line 470 "setlengths.cpp"
                    if ($cmp21) { __label__ = 12;; } else { __label__ = 11;; } //@line 470 "setlengths.cpp"
                    $if_then23$$lor_lhs_false$18: do { 
                      if (__label__ == 11) {
  
                        var $19=$stress; //@line 470 "setlengths.cpp"
                        var $cmp22=((($19))|0) < 4; //@line 470 "setlengths.cpp"
                        if ($cmp22) { __label__ = 12;break $if_then23$$lor_lhs_false$18; } //@line 470 "setlengths.cpp"
  
                        var $21=$p; //@line 473 "setlengths.cpp"
                        var $prepause26=(($21+16)&4294967295); //@line 473 "setlengths.cpp"
                        HEAP8[($prepause26)]=60; //@line 473 "setlengths.cpp"
                        __label__ = 14;break $if_then20$$if_else$15;
                      }
                    } while(0);
  
                    var $20=$p; //@line 471 "setlengths.cpp"
                    var $prepause24=(($20+16)&4294967295); //@line 471 "setlengths.cpp"
                    HEAP8[($prepause24)]=48; //@line 471 "setlengths.cpp"
                    ; //@line 471 "setlengths.cpp"
                  }
                } while(0);
  
                var $22=$prev; //@line 475 "setlengths.cpp"
                var $type29=(($22+15)&4294967295); //@line 475 "setlengths.cpp"
                var $23=HEAPU8[($type29)]; //@line 475 "setlengths.cpp"
                var $conv30=((($23))&255); //@line 475 "setlengths.cpp"
                var $cmp31=((($conv30))|0)==4; //@line 475 "setlengths.cpp"
                if ($cmp31) { __label__ = 15;; } else { __label__ = 16;; } //@line 475 "setlengths.cpp"
                if (__label__ == 15) {
  
                  var $24=$p; //@line 476 "setlengths.cpp"
                  var $prepause33=(($24+16)&4294967295); //@line 476 "setlengths.cpp"
                  HEAP8[($prepause33)]=60; //@line 476 "setlengths.cpp"
                  ; //@line 476 "setlengths.cpp"
                }
  
                var $25=$tr_addr; //@line 478 "setlengths.cpp"
                var $langopts=(($25)&4294967295); //@line 478 "setlengths.cpp"
                var $word_gap=(($langopts)&4294967295); //@line 478 "setlengths.cpp"
                var $26=HEAP32[(($word_gap)>>2)]; //@line 478 "setlengths.cpp"
                var $and35=($26) & 16; //@line 478 "setlengths.cpp"
                var $tobool36=((($and35))|0)!=0; //@line 478 "setlengths.cpp"
                if ($tobool36) { __label__ = 17;; } else { __label__ = 19;; } //@line 478 "setlengths.cpp"
                $land_lhs_true$$if_end40$26: do { 
                  if (__label__ == 17) {
  
                    var $27=$p; //@line 478 "setlengths.cpp"
                    var $newword=(($27+19)&4294967295); //@line 478 "setlengths.cpp"
                    var $28=HEAP8[($newword)]; //@line 478 "setlengths.cpp"
                    var $tobool37=(tempInt=(($28)),(tempInt>=128?tempInt-256:tempInt))!=0; //@line 478 "setlengths.cpp"
                    if (!($tobool37)) { __label__ = 19;break $land_lhs_true$$if_end40$26; } //@line 478 "setlengths.cpp"
  
                    var $29=$p; //@line 479 "setlengths.cpp"
                    var $prepause39=(($29+16)&4294967295); //@line 479 "setlengths.cpp"
                    HEAP8[($prepause39)]=60; //@line 479 "setlengths.cpp"
                    ; //@line 479 "setlengths.cpp"
                  }
                } while(0);
  
                var $30=$p; //@line 481 "setlengths.cpp"
                var $ph=(($30+8)&4294967295); //@line 481 "setlengths.cpp"
                var $31=HEAP32[(($ph)>>2)]; //@line 481 "setlengths.cpp"
                var $phflags=(($31+4)&4294967295); //@line 481 "setlengths.cpp"
                var $32=HEAP32[(($phflags)>>2)]; //@line 481 "setlengths.cpp"
                var $and41=($32) & 4194304; //@line 481 "setlengths.cpp"
                var $tobool42=((($and41))|0)!=0; //@line 481 "setlengths.cpp"
                if ($tobool42) { __label__ = 20;; } else { __label__ = 21;; } //@line 481 "setlengths.cpp"
                if (__label__ == 20) {
  
                  var $33=$p; //@line 482 "setlengths.cpp"
                  var $prepause44=(($33+16)&4294967295); //@line 482 "setlengths.cpp"
                  var $34=HEAPU8[($prepause44)]; //@line 482 "setlengths.cpp"
                  var $conv45=((($34))&255); //@line 482 "setlengths.cpp"
                  var $add46=((($conv45)+30)&4294967295); //@line 482 "setlengths.cpp"
                  var $conv47=((($add46)) & 255); //@line 482 "setlengths.cpp"
                  HEAP8[($prepause44)]=$conv47; //@line 482 "setlengths.cpp"
                  ; //@line 482 "setlengths.cpp"
                }
  
                var $35=$p; //@line 484 "setlengths.cpp"
                var $synthflags49=(($35+4)&4294967295); //@line 484 "setlengths.cpp"
                var $36=HEAPU16[(($synthflags49)>>1)]; //@line 484 "setlengths.cpp"
                var $conv50=((($36))&65535); //@line 484 "setlengths.cpp"
                var $and51=($conv50) & 8; //@line 484 "setlengths.cpp"
                var $tobool52=((($and51))|0)!=0; //@line 484 "setlengths.cpp"
                if (!($tobool52)) { __label__ = 192;break $if_end15_thread$$if_end15$9; } //@line 484 "setlengths.cpp"
  
                var $37=$tr_addr; //@line 485 "setlengths.cpp"
                var $langopts54=(($37)&4294967295); //@line 485 "setlengths.cpp"
                var $long_stop=(($langopts54+240)&4294967295); //@line 485 "setlengths.cpp"
                var $38=HEAP32[(($long_stop)>>2)]; //@line 485 "setlengths.cpp"
                var $39=$p; //@line 485 "setlengths.cpp"
                var $prepause55=(($39+16)&4294967295); //@line 485 "setlengths.cpp"
                var $40=HEAPU8[($prepause55)]; //@line 485 "setlengths.cpp"
                var $conv56=((($40))&255); //@line 485 "setlengths.cpp"
                var $add57=((($conv56)+($38))&4294967295); //@line 485 "setlengths.cpp"
                var $conv58=((($add57)) & 255); //@line 485 "setlengths.cpp"
                HEAP8[($prepause55)]=$conv58; //@line 485 "setlengths.cpp"
                __label__ = 192;break $if_end15_thread$$if_end15$9; //@line 485 "setlengths.cpp"
              }
              else if (__label__ == 23) {
  
                var $41=$p; //@line 490 "setlengths.cpp"
                var $newword61=(($41+19)&4294967295); //@line 490 "setlengths.cpp"
                var $42=HEAP8[($newword61)]; //@line 490 "setlengths.cpp"
                var $tobool62=(tempInt=(($42)),(tempInt>=128?tempInt-256:tempInt))!=0; //@line 490 "setlengths.cpp"
                if ($tobool62) { __label__ = 24;; } else { __label__ = 27;; } //@line 490 "setlengths.cpp"
                $if_then63$$if_end76$35: do { 
                  if (__label__ == 24) {
  
                    var $43=$prev; //@line 492 "setlengths.cpp"
                    var $type64=(($43+15)&4294967295); //@line 492 "setlengths.cpp"
                    var $44=HEAPU8[($type64)]; //@line 492 "setlengths.cpp"
                    var $conv65=((($44))&255); //@line 492 "setlengths.cpp"
                    var $cmp66=((($conv65))|0)==2; //@line 492 "setlengths.cpp"
                    if ($cmp66) { __label__ = 25;; } else { __label__ = 26;; } //@line 492 "setlengths.cpp"
                    if (__label__ == 25) {
  
                      var $45=$p; //@line 492 "setlengths.cpp"
                      var $ph68=(($45+8)&4294967295); //@line 492 "setlengths.cpp"
                      var $46=HEAP32[(($ph68)>>2)]; //@line 492 "setlengths.cpp"
                      var $phflags69=(($46+4)&4294967295); //@line 492 "setlengths.cpp"
                      var $47=HEAP32[(($phflags69)>>2)]; //@line 492 "setlengths.cpp"
                      var $and70=($47) & 16777216; //@line 492 "setlengths.cpp"
                      var $tobool71=((($and70))|0)!=0; //@line 492 "setlengths.cpp"
                      if ($tobool71) { __label__ = 27;break $if_then63$$if_end76$35; } //@line 492 "setlengths.cpp"
                    }
  
                    var $48=$p; //@line 497 "setlengths.cpp"
                    var $prepause74=(($48+16)&4294967295); //@line 497 "setlengths.cpp"
                    HEAP8[($prepause74)]=15; //@line 497 "setlengths.cpp"
                    ;
                  }
                } while(0);
  
                var $49=$next; //@line 501 "setlengths.cpp"
                var $type77=(($49+15)&4294967295); //@line 501 "setlengths.cpp"
                var $50=HEAPU8[($type77)]; //@line 501 "setlengths.cpp"
                var $conv78=((($50))&255); //@line 501 "setlengths.cpp"
                var $cmp79=((($conv78))|0)==0; //@line 501 "setlengths.cpp"
                if ($cmp79) { __label__ = 28;; } else { __label__ = 31;; } //@line 501 "setlengths.cpp"
                $land_lhs_true80$$if_end91$41: do { 
                  if (__label__ == 28) {
  
                    var $51=$prev; //@line 501 "setlengths.cpp"
                    var $type81=(($51+15)&4294967295); //@line 501 "setlengths.cpp"
                    var $52=HEAPU8[($type81)]; //@line 501 "setlengths.cpp"
                    var $conv82=((($52))&255); //@line 501 "setlengths.cpp"
                    var $cmp83=((($conv82))|0)==8; //@line 501 "setlengths.cpp"
                    if (!($cmp83)) { __label__ = 31;break $land_lhs_true80$$if_end91$41; } //@line 501 "setlengths.cpp"
  
                    var $53=$p; //@line 501 "setlengths.cpp"
                    var $ph85=(($53+8)&4294967295); //@line 501 "setlengths.cpp"
                    var $54=HEAP32[(($ph85)>>2)]; //@line 501 "setlengths.cpp"
                    var $phflags86=(($54+4)&4294967295); //@line 501 "setlengths.cpp"
                    var $55=HEAP32[(($phflags86)>>2)]; //@line 501 "setlengths.cpp"
                    var $and87=($55) & 8; //@line 501 "setlengths.cpp"
                    var $tobool88=((($and87))|0)!=0; //@line 501 "setlengths.cpp"
                    if ($tobool88) { __label__ = 31;break $land_lhs_true80$$if_end91$41; } //@line 501 "setlengths.cpp"
  
                    var $56=$p; //@line 502 "setlengths.cpp"
                    var $prepause90=(($56+16)&4294967295); //@line 502 "setlengths.cpp"
                    HEAP8[($prepause90)]=25; //@line 502 "setlengths.cpp"
                    ; //@line 502 "setlengths.cpp"
                  }
                } while(0);
  
                var $57=$prev; //@line 504 "setlengths.cpp"
                var $ph92=(($57+8)&4294967295); //@line 504 "setlengths.cpp"
                var $58=HEAP32[(($ph92)>>2)]; //@line 504 "setlengths.cpp"
                var $phflags93=(($58+4)&4294967295); //@line 504 "setlengths.cpp"
                var $59=HEAP32[(($phflags93)>>2)]; //@line 504 "setlengths.cpp"
                var $and94=($59) & 16384; //@line 504 "setlengths.cpp"
                var $tobool95=((($and94))|0)!=0; //@line 504 "setlengths.cpp"
                if ($tobool95) { __label__ = 32;; } else { __label__ = 33;; } //@line 504 "setlengths.cpp"
                if (__label__ == 32) {
  
                  var $60=$p; //@line 505 "setlengths.cpp"
                  var $prepause97=(($60+16)&4294967295); //@line 505 "setlengths.cpp"
                  HEAP8[($prepause97)]=30; //@line 505 "setlengths.cpp"
                  ; //@line 505 "setlengths.cpp"
                }
  
                var $61=$tr_addr; //@line 507 "setlengths.cpp"
                var $langopts99=(($61)&4294967295); //@line 507 "setlengths.cpp"
                var $word_gap100=(($langopts99)&4294967295); //@line 507 "setlengths.cpp"
                var $62=HEAP32[(($word_gap100)>>2)]; //@line 507 "setlengths.cpp"
                var $and101=($62) & 16; //@line 507 "setlengths.cpp"
                var $tobool102=((($and101))|0)!=0; //@line 507 "setlengths.cpp"
                if ($tobool102) { __label__ = 34;; } else { __label__ = 36;; } //@line 507 "setlengths.cpp"
                $land_lhs_true103$$if_end108$49: do { 
                  if (__label__ == 34) {
  
                    var $63=$p; //@line 507 "setlengths.cpp"
                    var $newword104=(($63+19)&4294967295); //@line 507 "setlengths.cpp"
                    var $64=HEAP8[($newword104)]; //@line 507 "setlengths.cpp"
                    var $tobool105=(tempInt=(($64)),(tempInt>=128?tempInt-256:tempInt))!=0; //@line 507 "setlengths.cpp"
                    if (!($tobool105)) { __label__ = 36;break $land_lhs_true103$$if_end108$49; } //@line 507 "setlengths.cpp"
  
                    var $65=$p; //@line 508 "setlengths.cpp"
                    var $prepause107=(($65+16)&4294967295); //@line 508 "setlengths.cpp"
                    HEAP8[($prepause107)]=30; //@line 508 "setlengths.cpp"
                    ; //@line 508 "setlengths.cpp"
                  }
                } while(0);
  
                var $66=$p; //@line 510 "setlengths.cpp"
                var $ph109=(($66+8)&4294967295); //@line 510 "setlengths.cpp"
                var $67=HEAP32[(($ph109)>>2)]; //@line 510 "setlengths.cpp"
                var $phflags110=(($67+4)&4294967295); //@line 510 "setlengths.cpp"
                var $68=HEAP32[(($phflags110)>>2)]; //@line 510 "setlengths.cpp"
                var $and111=($68) & 32; //@line 510 "setlengths.cpp"
                var $tobool112=((($and111))|0)!=0; //@line 510 "setlengths.cpp"
                if ($tobool112) { __label__ = 37;; } else { __label__ = 42;; } //@line 510 "setlengths.cpp"
                $land_lhs_true113$$if_else128$53: do { 
                  if (__label__ == 37) {
  
                    var $69=$next; //@line 510 "setlengths.cpp"
                    var $type114=(($69+15)&4294967295); //@line 510 "setlengths.cpp"
                    var $70=HEAPU8[($type114)]; //@line 510 "setlengths.cpp"
                    var $conv115=((($70))&255); //@line 510 "setlengths.cpp"
                    var $cmp116=((($conv115))|0)==4; //@line 510 "setlengths.cpp"
                    if (!($cmp116)) { __label__ = 42;break $land_lhs_true113$$if_else128$53; } //@line 510 "setlengths.cpp"
  
                    var $71=$next; //@line 510 "setlengths.cpp"
                    var $newword118=(($71+19)&4294967295); //@line 510 "setlengths.cpp"
                    var $72=HEAP8[($newword118)]; //@line 510 "setlengths.cpp"
                    var $tobool119=(tempInt=(($72)),(tempInt>=128?tempInt-256:tempInt))!=0; //@line 510 "setlengths.cpp"
                    if ($tobool119) { __label__ = 42;break $land_lhs_true113$$if_else128$53; } //@line 510 "setlengths.cpp"
  
                    var $73=$prev; //@line 512 "setlengths.cpp"
                    var $type121=(($73+15)&4294967295); //@line 512 "setlengths.cpp"
                    var $74=HEAPU8[($type121)]; //@line 512 "setlengths.cpp"
                    var $conv122=((($74))&255); //@line 512 "setlengths.cpp"
                    var $cmp123=((($conv122))|0)==2; //@line 512 "setlengths.cpp"
                    var $75=$p; //@line 513 "setlengths.cpp"
                    var $length=(($75+12)&4294967295); //@line 513 "setlengths.cpp"
                    if ($cmp123) { __label__ = 40;; } else { __label__ = 41;; } //@line 512 "setlengths.cpp"
                    if (__label__ == 40) {
  
                      HEAP16[(($length)>>1)]=200; //@line 513 "setlengths.cpp"
                      __label__ = 43;break $land_lhs_true113$$if_else128$53; //@line 513 "setlengths.cpp"
                    }
                    else if (__label__ == 41) {
  
                      HEAP16[(($length)>>1)]=150; //@line 515 "setlengths.cpp"
                      __label__ = 43;break $land_lhs_true113$$if_else128$53;
                    }
                  }
                } while(0);
                if (__label__ == 42) {
  
                  var $76=$p; //@line 518 "setlengths.cpp"
                  var $length129=(($76+12)&4294967295); //@line 518 "setlengths.cpp"
                  HEAP16[(($length129)>>1)]=256; //@line 518 "setlengths.cpp"
                  ;
                }
  
                var $77=$type; //@line 520 "setlengths.cpp"
                var $cmp131=((($77))|0)==7; //@line 520 "setlengths.cpp"
                if (!($cmp131)) { __label__ = 192;break $if_end15_thread$$if_end15$9; } //@line 520 "setlengths.cpp"
  
                var $78=$next; //@line 522 "setlengths.cpp"
                var $type133=(($78+15)&4294967295); //@line 522 "setlengths.cpp"
                var $79=HEAPU8[($type133)]; //@line 522 "setlengths.cpp"
                var $conv134=((($79))&255); //@line 522 "setlengths.cpp"
                var $cmp135=((($conv134))|0)==2; //@line 522 "setlengths.cpp"
                if ($cmp135) { __label__ = 45;; } else { __label__ = 46;; } //@line 522 "setlengths.cpp"
                if (__label__ == 45) {
  
                  $pre_voiced=1; //@line 524 "setlengths.cpp"
                  ; //@line 525 "setlengths.cpp"
                }
  
                var $80=$prev; //@line 526 "setlengths.cpp"
                var $type138=(($80+15)&4294967295); //@line 526 "setlengths.cpp"
                var $81=HEAPU8[($type138)]; //@line 526 "setlengths.cpp"
                var $conv139=((($81))&255); //@line 526 "setlengths.cpp"
                var $cmp140=((($conv139))|0)==2; //@line 526 "setlengths.cpp"
                if ($cmp140) { __label__ = 48;; } else { __label__ = 47;; } //@line 526 "setlengths.cpp"
                if (__label__ == 47) {
  
                  var $82=$prev; //@line 526 "setlengths.cpp"
                  var $type142=(($82+15)&4294967295); //@line 526 "setlengths.cpp"
                  var $83=HEAPU8[($type142)]; //@line 526 "setlengths.cpp"
                  var $conv143=((($83))&255); //@line 526 "setlengths.cpp"
                  var $cmp144=((($conv143))|0)==3; //@line 526 "setlengths.cpp"
                  if (!($cmp144)) { __label__ = 192;break $if_end15_thread$$if_end15$9; } //@line 526 "setlengths.cpp"
                }
  
                var $84=$prev; //@line 528 "setlengths.cpp"
                var $length146=(($84+12)&4294967295); //@line 528 "setlengths.cpp"
                var $85=HEAP16[(($length146)>>1)]; //@line 528 "setlengths.cpp"
                var $conv147=(tempInt=(($85)),(tempInt>=32768?tempInt-65536:tempInt)); //@line 528 "setlengths.cpp"
                var $add148=((($conv147)+255)&4294967295); //@line 528 "setlengths.cpp"
                var $div=((((($add148))|0)/2)|0); //@line 528 "setlengths.cpp"
                var $conv149=((($div)) & 65535); //@line 528 "setlengths.cpp"
                var $86=$p; //@line 528 "setlengths.cpp"
                var $length150=(($86+12)&4294967295); //@line 528 "setlengths.cpp"
                HEAP16[(($length150)>>1)]=$conv149; //@line 528 "setlengths.cpp"
                __label__ = 192;break $if_end15_thread$$if_end15$9; //@line 529 "setlengths.cpp"
              }
              else if (__label__ == 49) {
  
                var $87=$prev; //@line 534 "setlengths.cpp"
                var $type154=(($87+15)&4294967295); //@line 534 "setlengths.cpp"
                var $88=HEAPU8[($type154)]; //@line 534 "setlengths.cpp"
                var $conv155=((($88))&255); //@line 534 "setlengths.cpp"
                var $cmp156=((($conv155))|0)==7; //@line 534 "setlengths.cpp"
                if ($cmp156) { __label__ = 53;; } else { __label__ = 50;; } //@line 534 "setlengths.cpp"
                $if_then170$$lor_lhs_false157$71: do { 
                  if (__label__ == 50) {
  
                    var $89=$prev; //@line 534 "setlengths.cpp"
                    var $type158=(($89+15)&4294967295); //@line 534 "setlengths.cpp"
                    var $90=HEAPU8[($type158)]; //@line 534 "setlengths.cpp"
                    var $conv159=((($90))&255); //@line 534 "setlengths.cpp"
                    var $cmp160=((($conv159))|0)==6; //@line 534 "setlengths.cpp"
                    if ($cmp160) { __label__ = 53;break $if_then170$$lor_lhs_false157$71; } //@line 534 "setlengths.cpp"
  
                    var $91=$prev; //@line 534 "setlengths.cpp"
                    var $ph162=(($91+8)&4294967295); //@line 534 "setlengths.cpp"
                    var $92=HEAP32[(($ph162)>>2)]; //@line 534 "setlengths.cpp"
                    var $phflags163=(($92+4)&4294967295); //@line 534 "setlengths.cpp"
                    var $93=HEAP32[(($phflags163)>>2)]; //@line 534 "setlengths.cpp"
                    var $and164=($93) & 32; //@line 534 "setlengths.cpp"
                    var $tobool165=((($and164))|0)!=0; //@line 534 "setlengths.cpp"
                    if ($tobool165) { __label__ = 53;break $if_then170$$lor_lhs_false157$71; } //@line 534 "setlengths.cpp"
  
                    var $94=$prev; //@line 534 "setlengths.cpp"
                    var $type167=(($94+15)&4294967295); //@line 534 "setlengths.cpp"
                    var $95=HEAPU8[($type167)]; //@line 534 "setlengths.cpp"
                    var $conv168=((($95))&255); //@line 534 "setlengths.cpp"
                    var $cmp169=((($conv168))|0)==3; //@line 534 "setlengths.cpp"
                    if ($cmp169) { __label__ = 53;break $if_then170$$lor_lhs_false157$71; } else { __label__ = 54;break $if_then170$$lor_lhs_false157$71; } //@line 534 "setlengths.cpp"
                  }
                } while(0);
                if (__label__ == 53) {
  
                  var $96=$p; //@line 535 "setlengths.cpp"
                  var $prepause171=(($96+16)&4294967295); //@line 535 "setlengths.cpp"
                  HEAP8[($prepause171)]=30; //@line 535 "setlengths.cpp"
                  ; //@line 535 "setlengths.cpp"
                }
  
                var $97=$next; //@line 537 "setlengths.cpp"
                var $type173=(($97+15)&4294967295); //@line 537 "setlengths.cpp"
                var $98=HEAPU8[($type173)]; //@line 537 "setlengths.cpp"
                var $conv174=((($98))&255); //@line 537 "setlengths.cpp"
                var $cmp175=((($conv174))|0)==2; //@line 537 "setlengths.cpp"
                if ($cmp175) { __label__ = 56;; } else { __label__ = 55;; } //@line 537 "setlengths.cpp"
                $if_then180$$lor_lhs_false176$78: do { 
                  if (__label__ == 55) {
  
                    var $99=$next; //@line 537 "setlengths.cpp"
                    var $type177=(($99+15)&4294967295); //@line 537 "setlengths.cpp"
                    var $100=HEAPU8[($type177)]; //@line 537 "setlengths.cpp"
                    var $conv178=((($100))&255); //@line 537 "setlengths.cpp"
                    var $cmp179=((($conv178))|0)==3; //@line 537 "setlengths.cpp"
                    if ($cmp179) { __label__ = 56;break $if_then180$$lor_lhs_false176$78; } else { __label__ = 70;break $if_then180$$lor_lhs_false176$78; } //@line 537 "setlengths.cpp"
                  }
                } while(0);
                $if_then180$$if_end229$80: do { 
                  if (__label__ == 56) {
  
                    var $101=$next; //@line 539 "setlengths.cpp"
                    var $type181=(($101+15)&4294967295); //@line 539 "setlengths.cpp"
                    var $102=HEAPU8[($type181)]; //@line 539 "setlengths.cpp"
                    var $conv182=((($102))&255); //@line 539 "setlengths.cpp"
                    var $cmp183=((($conv182))|0)==2; //@line 539 "setlengths.cpp"
                    if ($cmp183) { __label__ = 58;; } else { __label__ = 57;; } //@line 539 "setlengths.cpp"
                    $if_then187$$lor_lhs_false184$82: do { 
                      if (__label__ == 57) {
  
                        var $103=$next; //@line 539 "setlengths.cpp"
                        var $newword185=(($103+19)&4294967295); //@line 539 "setlengths.cpp"
                        var $104=HEAP8[($newword185)]; //@line 539 "setlengths.cpp"
                        var $tobool186=(tempInt=(($104)),(tempInt>=128?tempInt-256:tempInt))!=0; //@line 539 "setlengths.cpp"
                        if ($tobool186) { __label__ = 59;break $if_then187$$lor_lhs_false184$82; } else { __label__ = 58;break $if_then187$$lor_lhs_false184$82; } //@line 539 "setlengths.cpp"
                      }
                    } while(0);
                    if (__label__ == 58) {
  
                      $pre_voiced=1; //@line 540 "setlengths.cpp"
                      ; //@line 540 "setlengths.cpp"
                    }
  
                    var $105=$p; //@line 542 "setlengths.cpp"
                    var $prepause189=(($105+16)&4294967295); //@line 542 "setlengths.cpp"
                    HEAP8[($prepause189)]=40; //@line 542 "setlengths.cpp"
                    var $106=$prev; //@line 544 "setlengths.cpp"
                    var $type190=(($106+15)&4294967295); //@line 544 "setlengths.cpp"
                    var $107=HEAPU8[($type190)]; //@line 544 "setlengths.cpp"
                    var $conv191=((($107))&255); //@line 544 "setlengths.cpp"
                    var $cmp192=((($conv191))|0)==0; //@line 544 "setlengths.cpp"
                    if ($cmp192) { __label__ = 61;; } else { __label__ = 60;; } //@line 544 "setlengths.cpp"
                    $if_then197$$lor_lhs_false193$87: do { 
                      if (__label__ == 60) {
  
                        var $108=$prev; //@line 544 "setlengths.cpp"
                        var $type194=(($108+15)&4294967295); //@line 544 "setlengths.cpp"
                        var $109=HEAPU8[($type194)]; //@line 544 "setlengths.cpp"
                        var $conv195=((($109))&255); //@line 544 "setlengths.cpp"
                        var $cmp196=((($conv195))|0)==2; //@line 544 "setlengths.cpp"
                        if ($cmp196) { __label__ = 61;break $if_then197$$lor_lhs_false193$87; } //@line 544 "setlengths.cpp"
  
                        var $111=$p; //@line 547 "setlengths.cpp"
                        var $newword200=(($111+19)&4294967295); //@line 547 "setlengths.cpp"
                        var $112=HEAPU8[($newword200)]; //@line 547 "setlengths.cpp"
                        var $conv201=((($112))&255); //@line 547 "setlengths.cpp"
                        var $cmp202=((($conv201))|0)==0; //@line 547 "setlengths.cpp"
                        if (!($cmp202)) { __label__ = 70;break $if_then180$$if_end229$80; } //@line 547 "setlengths.cpp"
  
                        var $113=$prev; //@line 549 "setlengths.cpp"
                        var $type204=(($113+15)&4294967295); //@line 549 "setlengths.cpp"
                        var $114=HEAPU8[($type204)]; //@line 549 "setlengths.cpp"
                        var $conv205=((($114))&255); //@line 549 "setlengths.cpp"
                        var $cmp206=((($conv205))|0)==3; //@line 549 "setlengths.cpp"
                        if ($cmp206) { __label__ = 64;; } else { __label__ = 65;; } //@line 549 "setlengths.cpp"
                        if (__label__ == 64) {
  
                          var $115=$p; //@line 550 "setlengths.cpp"
                          var $prepause208=(($115+16)&4294967295); //@line 550 "setlengths.cpp"
                          HEAP8[($prepause208)]=20; //@line 550 "setlengths.cpp"
                          ; //@line 550 "setlengths.cpp"
                        }
  
                        var $116=$prev; //@line 551 "setlengths.cpp"
                        var $type210=(($116+15)&4294967295); //@line 551 "setlengths.cpp"
                        var $117=HEAPU8[($type210)]; //@line 551 "setlengths.cpp"
                        var $conv211=((($117))&255); //@line 551 "setlengths.cpp"
                        var $cmp212=((($conv211))|0)==8; //@line 551 "setlengths.cpp"
                        if ($cmp212) { __label__ = 66;; } else { __label__ = 67;; } //@line 551 "setlengths.cpp"
                        if (__label__ == 66) {
  
                          var $118=$p; //@line 552 "setlengths.cpp"
                          var $prepause214=(($118+16)&4294967295); //@line 552 "setlengths.cpp"
                          HEAP8[($prepause214)]=12; //@line 552 "setlengths.cpp"
                          ; //@line 552 "setlengths.cpp"
                        }
  
                        var $119=$prev; //@line 554 "setlengths.cpp"
                        var $type216=(($119+15)&4294967295); //@line 554 "setlengths.cpp"
                        var $120=HEAPU8[($type216)]; //@line 554 "setlengths.cpp"
                        var $conv217=((($120))&255); //@line 554 "setlengths.cpp"
                        var $cmp218=((($conv217))|0)==4; //@line 554 "setlengths.cpp"
                        if (!($cmp218)) { __label__ = 70;break $if_then180$$if_end229$80; } //@line 554 "setlengths.cpp"
  
                        var $121=$prev; //@line 554 "setlengths.cpp"
                        var $ph220=(($121+8)&4294967295); //@line 554 "setlengths.cpp"
                        var $122=HEAP32[(($ph220)>>2)]; //@line 554 "setlengths.cpp"
                        var $phflags221=(($122+4)&4294967295); //@line 554 "setlengths.cpp"
                        var $123=HEAP32[(($phflags221)>>2)]; //@line 554 "setlengths.cpp"
                        var $and222=($123) & 8; //@line 554 "setlengths.cpp"
                        var $tobool223=((($and222))|0)!=0; //@line 554 "setlengths.cpp"
                        if ($tobool223) { __label__ = 70;break $if_then180$$if_end229$80; } //@line 554 "setlengths.cpp"
  
                        var $124=$p; //@line 555 "setlengths.cpp"
                        var $prepause225=(($124+16)&4294967295); //@line 555 "setlengths.cpp"
                        HEAP8[($prepause225)]=0; //@line 555 "setlengths.cpp"
                        __label__ = 70;break $if_then180$$if_end229$80; //@line 555 "setlengths.cpp"
                      }
                    } while(0);
  
                    var $110=$p; //@line 545 "setlengths.cpp"
                    var $prepause198=(($110+16)&4294967295); //@line 545 "setlengths.cpp"
                    HEAP8[($prepause198)]=0; //@line 545 "setlengths.cpp"
                    ; //@line 545 "setlengths.cpp"
                  }
                } while(0);
  
                var $125=$tr_addr; //@line 558 "setlengths.cpp"
                var $langopts230=(($125)&4294967295); //@line 558 "setlengths.cpp"
                var $word_gap231=(($langopts230)&4294967295); //@line 558 "setlengths.cpp"
                var $126=HEAP32[(($word_gap231)>>2)]; //@line 558 "setlengths.cpp"
                var $and232=($126) & 16; //@line 558 "setlengths.cpp"
                var $tobool233=((($and232))|0)!=0; //@line 558 "setlengths.cpp"
                if (!($tobool233)) { __label__ = 192;break $if_end15_thread$$if_end15$9; } //@line 558 "setlengths.cpp"
  
                var $127=$p; //@line 558 "setlengths.cpp"
                var $newword235=(($127+19)&4294967295); //@line 558 "setlengths.cpp"
                var $128=HEAP8[($newword235)]; //@line 558 "setlengths.cpp"
                var $tobool236=(tempInt=(($128)),(tempInt>=128?tempInt-256:tempInt))!=0; //@line 558 "setlengths.cpp"
                if (!($tobool236)) { __label__ = 192;break $if_end15_thread$$if_end15$9; } //@line 558 "setlengths.cpp"
  
                var $129=$p; //@line 558 "setlengths.cpp"
                var $prepause238=(($129+16)&4294967295); //@line 558 "setlengths.cpp"
                var $130=HEAPU8[($prepause238)]; //@line 558 "setlengths.cpp"
                var $conv239=((($130))&255); //@line 558 "setlengths.cpp"
                var $cmp240=((($conv239))|0) < 20; //@line 558 "setlengths.cpp"
                if (!($cmp240)) { __label__ = 192;break $if_end15_thread$$if_end15$9; } //@line 558 "setlengths.cpp"
  
                var $131=$p; //@line 559 "setlengths.cpp"
                var $prepause242=(($131+16)&4294967295); //@line 559 "setlengths.cpp"
                HEAP8[($prepause242)]=20; //@line 559 "setlengths.cpp"
                __label__ = 192;break $if_end15_thread$$if_end15$9; //@line 559 "setlengths.cpp"
              }
              else if (__label__ == 74) {
  
                var $132=$tr_addr; //@line 565 "setlengths.cpp"
                var $stress_amps=(($132+660)&4294967295); //@line 565 "setlengths.cpp"
                var $arrayidx245=(($stress_amps)&4294967295); //@line 565 "setlengths.cpp"
                var $133=HEAP8[($arrayidx245)]; //@line 565 "setlengths.cpp"
                var $134=$p; //@line 565 "setlengths.cpp"
                var $amp=(($134+18)&4294967295); //@line 565 "setlengths.cpp"
                HEAP8[($amp)]=$133; //@line 565 "setlengths.cpp"
                var $135=$p; //@line 566 "setlengths.cpp"
                var $length246=(($135+12)&4294967295); //@line 566 "setlengths.cpp"
                HEAP16[(($length246)>>1)]=256; //@line 566 "setlengths.cpp"
                $min_drop=0; //@line 567 "setlengths.cpp"
                var $136=$p; //@line 569 "setlengths.cpp"
                var $newword247=(($136+19)&4294967295); //@line 569 "setlengths.cpp"
                var $137=HEAP8[($newword247)]; //@line 569 "setlengths.cpp"
                var $tobool248=(tempInt=(($137)),(tempInt>=128?tempInt-256:tempInt))!=0; //@line 569 "setlengths.cpp"
                if ($tobool248) { __label__ = 75;; } else { __label__ = 80;; } //@line 569 "setlengths.cpp"
                $if_then249$$if_end268$105: do { 
                  if (__label__ == 75) {
  
                    var $138=$prev; //@line 571 "setlengths.cpp"
                    var $type250=(($138+15)&4294967295); //@line 571 "setlengths.cpp"
                    var $139=HEAPU8[($type250)]; //@line 571 "setlengths.cpp"
                    var $conv251=((($139))&255); //@line 571 "setlengths.cpp"
                    var $cmp252=((($conv251))|0)==3; //@line 571 "setlengths.cpp"
                    if ($cmp252) { __label__ = 76;; } else { __label__ = 77;; } //@line 571 "setlengths.cpp"
                    if (__label__ == 76) {
  
                      var $140=$p; //@line 572 "setlengths.cpp"
                      var $prepause254=(($140+16)&4294967295); //@line 572 "setlengths.cpp"
                      HEAP8[($prepause254)]=25; //@line 572 "setlengths.cpp"
                      ; //@line 572 "setlengths.cpp"
                    }
  
                    var $141=$prev; //@line 573 "setlengths.cpp"
                    var $type256=(($141+15)&4294967295); //@line 573 "setlengths.cpp"
                    var $142=HEAPU8[($type256)]; //@line 573 "setlengths.cpp"
                    var $conv257=((($142))&255); //@line 573 "setlengths.cpp"
                    var $cmp258=((($conv257))|0)==2; //@line 573 "setlengths.cpp"
                    if (!($cmp258)) { __label__ = 80;break $if_then249$$if_end268$105; } //@line 573 "setlengths.cpp"
  
                    var $143=$p; //@line 575 "setlengths.cpp"
                    var $ph260=(($143+8)&4294967295); //@line 575 "setlengths.cpp"
                    var $144=HEAP32[(($ph260)>>2)]; //@line 575 "setlengths.cpp"
                    var $phflags261=(($144+4)&4294967295); //@line 575 "setlengths.cpp"
                    var $145=HEAP32[(($phflags261)>>2)]; //@line 575 "setlengths.cpp"
                    var $and262=($145) & 16777216; //@line 575 "setlengths.cpp"
                    var $tobool263=((($and262))|0)!=0; //@line 575 "setlengths.cpp"
                    if ($tobool263) { __label__ = 80;break $if_then249$$if_end268$105; } //@line 575 "setlengths.cpp"
  
                    var $146=$p; //@line 576 "setlengths.cpp"
                    var $prepause265=(($146+16)&4294967295); //@line 576 "setlengths.cpp"
                    HEAP8[($prepause265)]=12; //@line 576 "setlengths.cpp"
                    ; //@line 576 "setlengths.cpp"
                  }
                } while(0);
  
                var $147=$next; //@line 580 "setlengths.cpp"
                var $type269=(($147+15)&4294967295); //@line 580 "setlengths.cpp"
                var $148=HEAPU8[($type269)]; //@line 580 "setlengths.cpp"
                var $conv270=((($148))&255); //@line 580 "setlengths.cpp"
                var $cmp271=((($conv270))|0)==2; //@line 580 "setlengths.cpp"
                if ($cmp271) { __label__ = 81;; } else { __label__ = 82;; } //@line 580 "setlengths.cpp"
                if (__label__ == 81) {
  
                  $pre_sonorant=1; //@line 582 "setlengths.cpp"
                  __label__ = 192;break $if_end15_thread$$if_end15$9; //@line 583 "setlengths.cpp"
                }
                else if (__label__ == 82) {
  
                  var $149=$last_pitch; //@line 586 "setlengths.cpp"
                  var $conv274=((($149)) & 255); //@line 586 "setlengths.cpp"
                  var $150=$p; //@line 586 "setlengths.cpp"
                  var $pitch2=(($150+21)&4294967295); //@line 586 "setlengths.cpp"
                  HEAP8[($pitch2)]=$conv274; //@line 586 "setlengths.cpp"
                  var $151=$prev; //@line 588 "setlengths.cpp"
                  var $type275=(($151+15)&4294967295); //@line 588 "setlengths.cpp"
                  var $152=HEAPU8[($type275)]; //@line 588 "setlengths.cpp"
                  var $conv276=((($152))&255); //@line 588 "setlengths.cpp"
                  var $cmp277=((($conv276))|0)==2; //@line 588 "setlengths.cpp"
                  if ($cmp277) { __label__ = 84;; } else { __label__ = 83;; } //@line 588 "setlengths.cpp"
                  $if_then282$$lor_lhs_false278$116: do { 
                    if (__label__ == 83) {
  
                      var $153=$prev; //@line 588 "setlengths.cpp"
                      var $type279=(($153+15)&4294967295); //@line 588 "setlengths.cpp"
                      var $154=HEAPU8[($type279)]; //@line 588 "setlengths.cpp"
                      var $conv280=((($154))&255); //@line 588 "setlengths.cpp"
                      var $cmp281=((($conv280))|0)==3; //@line 588 "setlengths.cpp"
                      if ($cmp281) { __label__ = 84;break $if_then282$$lor_lhs_false278$116; } //@line 588 "setlengths.cpp"
  
                      var $172=$ix; //@line 608 "setlengths.cpp"
                      $ix2=$172; //@line 608 "setlengths.cpp"
                      ; //@line 608 "setlengths.cpp"
                      while(1) { 
  
                        var $173=$ix2; //@line 608 "setlengths.cpp"
                        var $174=HEAP32[((_n_phoneme_list)>>2)]; //@line 608 "setlengths.cpp"
                        var $cmp315=((($173))|0) < ((($174))|0); //@line 608 "setlengths.cpp"
                        if (!($cmp315)) { __label__ = 95;break $if_then282$$lor_lhs_false278$116; } //@line 608 "setlengths.cpp"
  
                        var $175=$ix2; //@line 610 "setlengths.cpp"
                        var $arrayidx317=((_phoneme_list+$175*24)&4294967295); //@line 610 "setlengths.cpp"
                        var $type318=(($arrayidx317+15)&4294967295); //@line 610 "setlengths.cpp"
                        var $176=HEAPU8[($type318)]; //@line 610 "setlengths.cpp"
                        var $conv319=((($176))&255); //@line 610 "setlengths.cpp"
                        var $cmp320=((($conv319))|0)==2; //@line 610 "setlengths.cpp"
                        var $177=$ix2; //@line 612 "setlengths.cpp"
                        if ($cmp320) { __label__ = 93;break ; } //@line 610 "setlengths.cpp"
  
                        var $inc=((($177)+1)&4294967295); //@line 608 "setlengths.cpp"
                        $ix2=$inc; //@line 608 "setlengths.cpp"
                        __label__ = 91;continue ; //@line 608 "setlengths.cpp"
                      }
  
                      var $arrayidx322=((_phoneme_list+$177*24)&4294967295); //@line 612 "setlengths.cpp"
                      var $pitch2323=(($arrayidx322+21)&4294967295); //@line 612 "setlengths.cpp"
                      var $178=HEAP8[($pitch2323)]; //@line 612 "setlengths.cpp"
                      var $179=$p; //@line 612 "setlengths.cpp"
                      var $pitch2324=(($179+21)&4294967295); //@line 612 "setlengths.cpp"
                      HEAP8[($pitch2324)]=$178; //@line 612 "setlengths.cpp"
                      __label__ = 95;break $if_then282$$lor_lhs_false278$116; //@line 613 "setlengths.cpp"
                    }
                  } while(0);
                  $if_then282$$if_end326$124: do { 
                    if (__label__ == 84) {
  
                      var $155=$prev; //@line 590 "setlengths.cpp"
                      var $length283=(($155+12)&4294967295); //@line 590 "setlengths.cpp"
                      var $156=HEAP16[(($length283)>>1)]; //@line 590 "setlengths.cpp"
                      var $157=$p; //@line 590 "setlengths.cpp"
                      var $length284=(($157+12)&4294967295); //@line 590 "setlengths.cpp"
                      HEAP16[(($length284)>>1)]=$156; //@line 590 "setlengths.cpp"
                      var $158=$p; //@line 592 "setlengths.cpp"
                      var $type285=(($158+15)&4294967295); //@line 592 "setlengths.cpp"
                      var $159=HEAPU8[($type285)]; //@line 592 "setlengths.cpp"
                      var $conv286=((($159))&255); //@line 592 "setlengths.cpp"
                      var $cmp287=((($conv286))|0)==3; //@line 592 "setlengths.cpp"
                      if ($cmp287) { __label__ = 85;; } else { __label__ = 86;; } //@line 592 "setlengths.cpp"
                      if (__label__ == 85) {
  
                        var $160=HEAP32[((__ZL6speed1)>>2)]; //@line 594 "setlengths.cpp"
                        var $conv289=((($160)) & 65535); //@line 594 "setlengths.cpp"
                        var $161=$p; //@line 594 "setlengths.cpp"
                        var $length290=(($161+12)&4294967295); //@line 594 "setlengths.cpp"
                        HEAP16[(($length290)>>1)]=$conv289; //@line 594 "setlengths.cpp"
                        ; //@line 595 "setlengths.cpp"
                      }
  
                      var $162=$next; //@line 597 "setlengths.cpp"
                      var $type292=(($162+15)&4294967295); //@line 597 "setlengths.cpp"
                      var $163=HEAPU8[($type292)]; //@line 597 "setlengths.cpp"
                      var $conv293=((($163))&255); //@line 597 "setlengths.cpp"
                      var $cmp294=((($conv293))|0)==5; //@line 597 "setlengths.cpp"
                      if ($cmp294) { __label__ = 87;; } else { __label__ = 88;; } //@line 597 "setlengths.cpp"
                      if (__label__ == 87) {
  
                        var $164=$p; //@line 599 "setlengths.cpp"
                        var $length296=(($164+12)&4294967295); //@line 599 "setlengths.cpp"
                        var $165=HEAP16[(($length296)>>1)]; //@line 599 "setlengths.cpp"
                        var $conv297=(tempInt=(($165)),(tempInt>=32768?tempInt-65536:tempInt)); //@line 599 "setlengths.cpp"
                        var $mul=((($conv297)*160)&4294967295); //@line 599 "setlengths.cpp"
                        var $div298=((((($mul))|0)/100)|0); //@line 599 "setlengths.cpp"
                        var $conv299=((($div298)) & 65535); //@line 599 "setlengths.cpp"
                        var $166=$p; //@line 599 "setlengths.cpp"
                        var $length300=(($166+12)&4294967295); //@line 599 "setlengths.cpp"
                        HEAP16[(($length300)>>1)]=$conv299; //@line 599 "setlengths.cpp"
                        ; //@line 600 "setlengths.cpp"
                      }
  
                      var $167=$next; //@line 601 "setlengths.cpp"
                      var $type302=(($167+15)&4294967295); //@line 601 "setlengths.cpp"
                      var $168=HEAPU8[($type302)]; //@line 601 "setlengths.cpp"
                      var $conv303=((($168))&255); //@line 601 "setlengths.cpp"
                      var $cmp304=((($conv303))|0)==7; //@line 601 "setlengths.cpp"
                      if (!($cmp304)) { __label__ = 95;break $if_then282$$if_end326$124; } //@line 601 "setlengths.cpp"
  
                      var $169=$p; //@line 603 "setlengths.cpp"
                      var $length306=(($169+12)&4294967295); //@line 603 "setlengths.cpp"
                      var $170=HEAP16[(($length306)>>1)]; //@line 603 "setlengths.cpp"
                      var $conv307=(tempInt=(($170)),(tempInt>=32768?tempInt-65536:tempInt)); //@line 603 "setlengths.cpp"
                      var $mul308=((($conv307)*120)&4294967295); //@line 603 "setlengths.cpp"
                      var $div309=((((($mul308))|0)/100)|0); //@line 603 "setlengths.cpp"
                      var $conv310=((($div309)) & 65535); //@line 603 "setlengths.cpp"
                      var $171=$p; //@line 603 "setlengths.cpp"
                      var $length311=(($171+12)&4294967295); //@line 603 "setlengths.cpp"
                      HEAP16[(($length311)>>1)]=$conv310; //@line 603 "setlengths.cpp"
                      ; //@line 604 "setlengths.cpp"
                    }
                  } while(0);
  
                  var $180=$p; //@line 618 "setlengths.cpp"
                  var $pitch2327=(($180+21)&4294967295); //@line 618 "setlengths.cpp"
                  var $181=HEAPU8[($pitch2327)]; //@line 618 "setlengths.cpp"
                  var $conv328=((($181))&255); //@line 618 "setlengths.cpp"
                  var $sub329=((($conv328)-16)&4294967295); //@line 618 "setlengths.cpp"
                  var $conv330=((($sub329)) & 255); //@line 618 "setlengths.cpp"
                  var $182=$p; //@line 618 "setlengths.cpp"
                  var $pitch1331=(($182+20)&4294967295); //@line 618 "setlengths.cpp"
                  HEAP8[($pitch1331)]=$conv330; //@line 618 "setlengths.cpp"
                  var $183=$p; //@line 619 "setlengths.cpp"
                  var $pitch2332=(($183+21)&4294967295); //@line 619 "setlengths.cpp"
                  var $184=HEAPU8[($pitch2332)]; //@line 619 "setlengths.cpp"
                  var $conv333=((($184))&255); //@line 619 "setlengths.cpp"
                  var $cmp334=((($conv333))|0) < 16; //@line 619 "setlengths.cpp"
                  if ($cmp334) { __label__ = 96;; } else { __label__ = 97;; } //@line 619 "setlengths.cpp"
                  if (__label__ == 96) {
  
                    var $185=$p; //@line 621 "setlengths.cpp"
                    var $pitch1336=(($185+20)&4294967295); //@line 621 "setlengths.cpp"
                    HEAP8[($pitch1336)]=0; //@line 621 "setlengths.cpp"
                    ; //@line 622 "setlengths.cpp"
                  }
  
                  var $186=$p; //@line 623 "setlengths.cpp"
                  var $env=(($186+14)&4294967295); //@line 623 "setlengths.cpp"
                  HEAP8[($env)]=0; //@line 623 "setlengths.cpp"
                  $pre_voiced=0; //@line 624 "setlengths.cpp"
                  __label__ = 192;break $if_end15_thread$$if_end15$9;
                }
              }
            }
          } while(0);
          if (__label__ == 98) {
  
            $min_drop=0; //@line 629 "setlengths.cpp"
            var $187=$ix; //@line 630 "setlengths.cpp"
            var $add340=((($187)+2)&4294967295); //@line 630 "setlengths.cpp"
            var $arrayidx341=((_phoneme_list+$add340*24)&4294967295); //@line 630 "setlengths.cpp"
            $next2=$arrayidx341; //@line 630 "setlengths.cpp"
            var $188=$ix; //@line 631 "setlengths.cpp"
            var $add342=((($188)+3)&4294967295); //@line 631 "setlengths.cpp"
            var $arrayidx343=((_phoneme_list+$add342*24)&4294967295); //@line 631 "setlengths.cpp"
            $next3=$arrayidx343; //@line 631 "setlengths.cpp"
            var $189=$stress; //@line 633 "setlengths.cpp"
            var $cmp344=((($189))|0) > 7; //@line 633 "setlengths.cpp"
            if ($cmp344) { __label__ = 99;; } else { __label__ = 100;; } //@line 633 "setlengths.cpp"
            $if_end346_thread$$if_end346$139: do { 
              if (__label__ == 99) {
  
                $stress=7; //@line 633 "setlengths.cpp"
                ; //@line 635 "setlengths.cpp"
              }
              else if (__label__ == 100) {
  
                var $cmp347=((($189))|0) <= 1; //@line 635 "setlengths.cpp"
                if (!($cmp347)) { __label__ = 102;break $if_end346_thread$$if_end346$139; } //@line 635 "setlengths.cpp"
  
                var $190=$stress; //@line 637 "setlengths.cpp"
                var $xor=($190) ^ 1; //@line 637 "setlengths.cpp"
                $stress=$xor; //@line 637 "setlengths.cpp"
                ; //@line 638 "setlengths.cpp"
              }
            } while(0);
  
            var $191=$pre_sonorant; //@line 639 "setlengths.cpp"
            var $tobool350=((($191))|0)!=0; //@line 639 "setlengths.cpp"
            var $192=$stress; //@line 640 "setlengths.cpp"
            var $193=$tr_addr; //@line 640 "setlengths.cpp"
            var $stress_amps352=(($193+660)&4294967295); //@line 640 "setlengths.cpp"
            var $arrayidx353=(($stress_amps352+$192)&4294967295); //@line 640 "setlengths.cpp"
            var $194=HEAPU8[($arrayidx353)]; //@line 640 "setlengths.cpp"
            if ($tobool350) { __label__ = 103;; } else { __label__ = 104;; } //@line 639 "setlengths.cpp"
            if (__label__ == 103) {
  
              var $conv354=((($194))&255); //@line 640 "setlengths.cpp"
              var $sub355=((($conv354)-1)&4294967295); //@line 640 "setlengths.cpp"
              var $conv356=((($sub355)) & 255); //@line 640 "setlengths.cpp"
              var $195=$p; //@line 640 "setlengths.cpp"
              var $amp357=(($195+18)&4294967295); //@line 640 "setlengths.cpp"
              HEAP8[($amp357)]=$conv356; //@line 640 "setlengths.cpp"
              ; //@line 640 "setlengths.cpp"
            }
            else if (__label__ == 104) {
  
              var $196=$p; //@line 642 "setlengths.cpp"
              var $amp361=(($196+18)&4294967295); //@line 642 "setlengths.cpp"
              HEAP8[($amp361)]=$194; //@line 642 "setlengths.cpp"
              ;
            }
  
            var $197=$emphasized; //@line 644 "setlengths.cpp"
            var $tobool363=((($197))|0)!=0; //@line 644 "setlengths.cpp"
            if ($tobool363) { __label__ = 106;; } else { __label__ = 107;; } //@line 644 "setlengths.cpp"
            if (__label__ == 106) {
  
              var $198=$p; //@line 645 "setlengths.cpp"
              var $amp365=(($198+18)&4294967295); //@line 645 "setlengths.cpp"
              HEAP8[($amp365)]=25; //@line 645 "setlengths.cpp"
              ; //@line 645 "setlengths.cpp"
            }
  
            var $199=$ix; //@line 647 "setlengths.cpp"
            var $200=HEAP32[((_n_phoneme_list)>>2)]; //@line 647 "setlengths.cpp"
            var $sub367=((($200)-3)&4294967295); //@line 647 "setlengths.cpp"
            var $cmp368=((($199))|0) >= ((($sub367))|0); //@line 647 "setlengths.cpp"
            if ($cmp368) { __label__ = 108;; } else { __label__ = 110;; } //@line 647 "setlengths.cpp"
            $if_then369$$if_end382$151: do { 
              if (__label__ == 108) {
  
                var $201=$p; //@line 650 "setlengths.cpp"
                var $amp370=(($201+18)&4294967295); //@line 650 "setlengths.cpp"
                var $202=HEAPU8[($amp370)]; //@line 650 "setlengths.cpp"
                var $conv371=((($202))&255); //@line 650 "setlengths.cpp"
                var $203=$tr_addr; //@line 650 "setlengths.cpp"
                var $langopts372=(($203)&4294967295); //@line 650 "setlengths.cpp"
                var $param=(($langopts372+24)&4294967295); //@line 650 "setlengths.cpp"
                var $arrayidx373=(($param+36)&4294967295); //@line 650 "setlengths.cpp"
                var $204=HEAP32[(($arrayidx373)>>2)]; //@line 650 "setlengths.cpp"
                var $cmp374=((($conv371))|0) > ((($204))|0); //@line 650 "setlengths.cpp"
                if (!($cmp374)) { __label__ = 110;break $if_then369$$if_end382$151; } //@line 650 "setlengths.cpp"
  
                var $205=$tr_addr; //@line 651 "setlengths.cpp"
                var $langopts376=(($205)&4294967295); //@line 651 "setlengths.cpp"
                var $param377=(($langopts376+24)&4294967295); //@line 651 "setlengths.cpp"
                var $arrayidx378=(($param377+36)&4294967295); //@line 651 "setlengths.cpp"
                var $206=HEAP32[(($arrayidx378)>>2)]; //@line 651 "setlengths.cpp"
                var $conv379=((($206)) & 255); //@line 651 "setlengths.cpp"
                var $207=$p; //@line 651 "setlengths.cpp"
                var $amp380=(($207+18)&4294967295); //@line 651 "setlengths.cpp"
                HEAP8[($amp380)]=$conv379; //@line 651 "setlengths.cpp"
                ; //@line 651 "setlengths.cpp"
              }
            } while(0);
  
            HEAP32[((__ZZ11CalcLengthsP10TranslatorE14more_syllables)>>2)]=0; //@line 655 "setlengths.cpp"
            $end_of_clause=0; //@line 656 "setlengths.cpp"
            var $208=$p; //@line 657 "setlengths.cpp"
            var $add_ptr=(($208+24)&4294967295); //@line 657 "setlengths.cpp"
            $p2=$add_ptr; //@line 657 "setlengths.cpp"
            var $209=$p2; //@line 657 "setlengths.cpp"
            var $newword3843=(($209+19)&4294967295); //@line 657 "setlengths.cpp"
            var $210=HEAPU8[($newword3843)]; //@line 657 "setlengths.cpp"
            var $conv3854=((($210))&255); //@line 657 "setlengths.cpp"
            var $cmp3865=((($conv3854))|0)==0; //@line 657 "setlengths.cpp"
            var $211=$p2; //@line 659 "setlengths.cpp"
            if ($cmp3865) { __lastLabel__ = 110; __label__ = 111;; } else { __lastLabel__ = 110; __label__ = 117;; } //@line 657 "setlengths.cpp"
            $for_body387$$for_end405$155: do { 
              if (__label__ == 111) {
                $for_body387$156: while(1) { 
  
                  var $212=__lastLabel__ == 116 ? $224 : ($211);
                  var $type388=(($212+15)&4294967295); //@line 659 "setlengths.cpp"
                  var $213=HEAPU8[($type388)]; //@line 659 "setlengths.cpp"
                  var $conv389=((($213))&255); //@line 659 "setlengths.cpp"
                  var $cmp390=((($conv389))|0)==2; //@line 659 "setlengths.cpp"
                  if ($cmp390) { __label__ = 112;; } else { __label__ = 114;; } //@line 659 "setlengths.cpp"
                  $land_lhs_true391$$if_end398$158: do { 
                    if (__label__ == 112) {
  
                      var $214=$p2; //@line 659 "setlengths.cpp"
                      var $ph392=(($214+8)&4294967295); //@line 659 "setlengths.cpp"
                      var $215=HEAP32[(($ph392)>>2)]; //@line 659 "setlengths.cpp"
                      var $phflags393=(($215+4)&4294967295); //@line 659 "setlengths.cpp"
                      var $216=HEAP32[(($phflags393)>>2)]; //@line 659 "setlengths.cpp"
                      var $and394=($216) & 1048576; //@line 659 "setlengths.cpp"
                      var $tobool395=((($and394))|0)!=0; //@line 659 "setlengths.cpp"
                      if ($tobool395) { __label__ = 114;break $land_lhs_true391$$if_end398$158; } //@line 659 "setlengths.cpp"
  
                      var $217=HEAP32[((__ZZ11CalcLengthsP10TranslatorE14more_syllables)>>2)]; //@line 660 "setlengths.cpp"
                      var $inc397=((($217)+1)&4294967295); //@line 660 "setlengths.cpp"
                      HEAP32[((__ZZ11CalcLengthsP10TranslatorE14more_syllables)>>2)]=$inc397; //@line 660 "setlengths.cpp"
                      ; //@line 660 "setlengths.cpp"
                    }
                  } while(0);
  
                  var $218=$p2; //@line 662 "setlengths.cpp"
                  var $ph399=(($218+8)&4294967295); //@line 662 "setlengths.cpp"
                  var $219=HEAP32[(($ph399)>>2)]; //@line 662 "setlengths.cpp"
                  var $code=(($219+10)&4294967295); //@line 662 "setlengths.cpp"
                  var $220=HEAPU8[($code)]; //@line 662 "setlengths.cpp"
                  var $conv400=((($220))&255); //@line 662 "setlengths.cpp"
                  var $cmp401=((($conv400))|0)==27; //@line 662 "setlengths.cpp"
                  if ($cmp401) { __label__ = 115;; } else { __label__ = 116;; } //@line 662 "setlengths.cpp"
                  if (__label__ == 115) {
  
                    $end_of_clause=2; //@line 663 "setlengths.cpp"
                    ; //@line 663 "setlengths.cpp"
                  }
  
                  var $221=$p2; //@line 657 "setlengths.cpp"
                  var $incdec_ptr=(($221+24)&4294967295); //@line 657 "setlengths.cpp"
                  $p2=$incdec_ptr; //@line 657 "setlengths.cpp"
                  var $222=$p2; //@line 657 "setlengths.cpp"
                  var $newword384=(($222+19)&4294967295); //@line 657 "setlengths.cpp"
                  var $223=HEAPU8[($newword384)]; //@line 657 "setlengths.cpp"
                  var $conv385=((($223))&255); //@line 657 "setlengths.cpp"
                  var $cmp386=((($conv385))|0)==0; //@line 657 "setlengths.cpp"
                  var $224=$p2; //@line 659 "setlengths.cpp"
                  if ($cmp386) { __lastLabel__ = 116; __label__ = 111;continue $for_body387$156; } else { __lastLabel__ = 116; __label__ = 117;break $for_body387$$for_end405$155; } //@line 657 "setlengths.cpp"
                }
              }
            } while(0);
  
            var $_lcssa=__lastLabel__ == 110 ? $211 : ($224);
            var $ph406=(($_lcssa+8)&4294967295); //@line 665 "setlengths.cpp"
            var $225=HEAP32[(($ph406)>>2)]; //@line 665 "setlengths.cpp"
            var $code407=(($225+10)&4294967295); //@line 665 "setlengths.cpp"
            var $226=HEAPU8[($code407)]; //@line 665 "setlengths.cpp"
            var $conv408=((($226))&255); //@line 665 "setlengths.cpp"
            var $cmp409=((($conv408))|0)==27; //@line 665 "setlengths.cpp"
            if ($cmp409) { __label__ = 118;; } else { __label__ = 119;; } //@line 665 "setlengths.cpp"
            if (__label__ == 118) {
  
              $end_of_clause=2; //@line 666 "setlengths.cpp"
              ; //@line 666 "setlengths.cpp"
            }
  
            var $227=$p2; //@line 668 "setlengths.cpp"
            var $newword412=(($227+19)&4294967295); //@line 668 "setlengths.cpp"
            var $228=HEAPU8[($newword412)]; //@line 668 "setlengths.cpp"
            var $conv413=((($228))&255); //@line 668 "setlengths.cpp"
            var $and414=($conv413) & 2; //@line 668 "setlengths.cpp"
            var $tobool415=((($and414))|0)!=0; //@line 668 "setlengths.cpp"
            var $229=HEAP32[((__ZZ11CalcLengthsP10TranslatorE14more_syllables)>>2)]; //@line 668 "setlengths.cpp"
            var $cmp417=((($229))|0)==0; //@line 668 "setlengths.cpp"
            var $or_cond=($tobool415) & ($cmp417); //@line 668 "setlengths.cpp"
            if ($or_cond) { __label__ = 120;; } else { __label__ = 121;; } //@line 668 "setlengths.cpp"
            if (__label__ == 120) {
  
              $end_of_clause=2; //@line 670 "setlengths.cpp"
              ; //@line 671 "setlengths.cpp"
            }
  
            var $230=$next; //@line 674 "setlengths.cpp"
            var $ph420=(($230+8)&4294967295); //@line 674 "setlengths.cpp"
            var $231=HEAP32[(($ph420)>>2)]; //@line 674 "setlengths.cpp"
            var $code421=(($231+10)&4294967295); //@line 674 "setlengths.cpp"
            var $232=HEAPU8[($code421)]; //@line 674 "setlengths.cpp"
            var $conv422=((($232))&255); //@line 674 "setlengths.cpp"
            var $cmp423=((($conv422))|0)==23; //@line 674 "setlengths.cpp"
            if ($cmp423) { __label__ = 122;; } else { __label__ = 124;; } //@line 674 "setlengths.cpp"
            $land_lhs_true424$$if_end431$172: do { 
              if (__label__ == 122) {
  
                var $233=$next2; //@line 674 "setlengths.cpp"
                var $type425=(($233+15)&4294967295); //@line 674 "setlengths.cpp"
                var $234=HEAPU8[($type425)]; //@line 674 "setlengths.cpp"
                var $conv426=((($234))&255); //@line 674 "setlengths.cpp"
                var $cmp427=((($conv426))|0)==0; //@line 674 "setlengths.cpp"
                if (!($cmp427)) { __label__ = 124;break $land_lhs_true424$$if_end431$172; } //@line 674 "setlengths.cpp"
  
                var $235=$next2; //@line 677 "setlengths.cpp"
                $next=$235; //@line 677 "setlengths.cpp"
                var $236=$next3; //@line 678 "setlengths.cpp"
                $next2=$236; //@line 678 "setlengths.cpp"
                var $237=$ix; //@line 679 "setlengths.cpp"
                var $add429=((($237)+4)&4294967295); //@line 679 "setlengths.cpp"
                var $arrayidx430=((_phoneme_list+$add429*24)&4294967295); //@line 679 "setlengths.cpp"
                $next3=$arrayidx430; //@line 679 "setlengths.cpp"
                ; //@line 680 "setlengths.cpp"
              }
            } while(0);
  
            var $238=HEAP32[((__ZZ11CalcLengthsP10TranslatorE14more_syllables)>>2)]; //@line 682 "setlengths.cpp"
            var $cmp432=((($238))|0)==0; //@line 682 "setlengths.cpp"
            var $239=$next2; //@line 684 "setlengths.cpp"
            var $ph434=(($239+8)&4294967295); //@line 684 "setlengths.cpp"
            var $240=HEAP32[(($ph434)>>2)]; //@line 684 "setlengths.cpp"
            var $length_mod435=(($240+15)&4294967295); //@line 684 "setlengths.cpp"
            var $241=HEAPU8[($length_mod435)]; //@line 684 "setlengths.cpp"
            var $conv436=((($241))&255); //@line 684 "setlengths.cpp"
            var $mul437=((($conv436)*10)&4294967295); //@line 684 "setlengths.cpp"
            var $242=$next; //@line 684 "setlengths.cpp"
            var $ph438=(($242+8)&4294967295); //@line 684 "setlengths.cpp"
            var $243=HEAP32[(($ph438)>>2)]; //@line 684 "setlengths.cpp"
            var $length_mod439=(($243+15)&4294967295); //@line 684 "setlengths.cpp"
            var $244=HEAPU8[($length_mod439)]; //@line 684 "setlengths.cpp"
            var $conv440=((($244))&255); //@line 684 "setlengths.cpp"
            var $add441=((($conv440)+($mul437))&4294967295); //@line 684 "setlengths.cpp"
            var $245=$tr_addr; //@line 684 "setlengths.cpp"
            var $langopts442=(($245)&4294967295); //@line 684 "setlengths.cpp"
            if ($cmp432) { __label__ = 125;; } else { __label__ = 129;; } //@line 682 "setlengths.cpp"
            $if_then433$$if_else466$176: do { 
              if (__label__ == 125) {
  
                var $length_mods0=(($langopts442+180)&4294967295); //@line 684 "setlengths.cpp"
                var $246=HEAP32[(($length_mods0)>>2)]; //@line 684 "setlengths.cpp"
                var $arrayidx443=(($246+$add441)&4294967295); //@line 684 "setlengths.cpp"
                var $247=HEAPU8[($arrayidx443)]; //@line 684 "setlengths.cpp"
                var $conv444=((($247))&255); //@line 684 "setlengths.cpp"
                $len=$conv444; //@line 684 "setlengths.cpp"
                var $248=$next; //@line 686 "setlengths.cpp"
                var $newword445=(($248+19)&4294967295); //@line 686 "setlengths.cpp"
                var $249=HEAP8[($newword445)]; //@line 686 "setlengths.cpp"
                var $tobool446=(tempInt=(($249)),(tempInt>=128?tempInt-256:tempInt))!=0; //@line 686 "setlengths.cpp"
                if ($tobool446) { __label__ = 126;; } else { __label__ = 128;; } //@line 686 "setlengths.cpp"
                $land_lhs_true447$$if_else464$178: do { 
                  if (__label__ == 126) {
  
                    var $250=$tr_addr; //@line 686 "setlengths.cpp"
                    var $langopts448=(($250)&4294967295); //@line 686 "setlengths.cpp"
                    var $word_gap449=(($langopts448)&4294967295); //@line 686 "setlengths.cpp"
                    var $251=HEAP32[(($word_gap449)>>2)]; //@line 686 "setlengths.cpp"
                    var $and450=($251) & 32; //@line 686 "setlengths.cpp"
                    var $tobool451=((($and450))|0)!=0; //@line 686 "setlengths.cpp"
                    if (!($tobool451)) { __label__ = 128;break $land_lhs_true447$$if_else464$178; } //@line 686 "setlengths.cpp"
  
                    var $252=$len; //@line 689 "setlengths.cpp"
                    var $253=$next; //@line 689 "setlengths.cpp"
                    var $ph453=(($253+8)&4294967295); //@line 689 "setlengths.cpp"
                    var $254=HEAP32[(($ph453)>>2)]; //@line 689 "setlengths.cpp"
                    var $length_mod454=(($254+15)&4294967295); //@line 689 "setlengths.cpp"
                    var $255=HEAPU8[($length_mod454)]; //@line 689 "setlengths.cpp"
                    var $conv455=((($255))&255); //@line 689 "setlengths.cpp"
                    var $mul456=((($conv455)*10)&4294967295); //@line 689 "setlengths.cpp"
                    var $add457=((($mul456)+1)&4294967295); //@line 689 "setlengths.cpp"
                    var $256=$tr_addr; //@line 689 "setlengths.cpp"
                    var $langopts458=(($256)&4294967295); //@line 689 "setlengths.cpp"
                    var $length_mods0459=(($langopts458+180)&4294967295); //@line 689 "setlengths.cpp"
                    var $257=HEAP32[(($length_mods0459)>>2)]; //@line 689 "setlengths.cpp"
                    var $arrayidx460=(($257+$add457)&4294967295); //@line 689 "setlengths.cpp"
                    var $258=HEAPU8[($arrayidx460)]; //@line 689 "setlengths.cpp"
                    var $conv461=((($258))&255); //@line 689 "setlengths.cpp"
                    var $add462=((($conv461)+($252))&4294967295); //@line 689 "setlengths.cpp"
                    var $div463=((((($add462))|0)/2)|0); //@line 689 "setlengths.cpp"
                    $length_mod=$div463; //@line 689 "setlengths.cpp"
                    __label__ = 134;break $if_then433$$if_else466$176; //@line 690 "setlengths.cpp"
                  }
                } while(0);
  
                var $259=$len; //@line 692 "setlengths.cpp"
                $length_mod=$259; //@line 692 "setlengths.cpp"
                ;
              }
              else if (__label__ == 129) {
  
                var $length_mods=(($langopts442+176)&4294967295); //@line 696 "setlengths.cpp"
                var $260=HEAP32[(($length_mods)>>2)]; //@line 696 "setlengths.cpp"
                var $arrayidx476=(($260+$add441)&4294967295); //@line 696 "setlengths.cpp"
                var $261=HEAPU8[($arrayidx476)]; //@line 696 "setlengths.cpp"
                var $conv477=((($261))&255); //@line 696 "setlengths.cpp"
                $length_mod=$conv477; //@line 696 "setlengths.cpp"
                var $262=$next; //@line 698 "setlengths.cpp"
                var $type478=(($262+15)&4294967295); //@line 698 "setlengths.cpp"
                var $263=HEAPU8[($type478)]; //@line 698 "setlengths.cpp"
                var $conv479=((($263))&255); //@line 698 "setlengths.cpp"
                var $cmp480=((($conv479))|0)==8; //@line 698 "setlengths.cpp"
                if (!($cmp480)) { __label__ = 134;break $if_then433$$if_else466$176; } //@line 698 "setlengths.cpp"
  
                var $264=$next2; //@line 698 "setlengths.cpp"
                var $type482=(($264+15)&4294967295); //@line 698 "setlengths.cpp"
                var $265=HEAPU8[($type482)]; //@line 698 "setlengths.cpp"
                var $conv483=((($265))&255); //@line 698 "setlengths.cpp"
                var $cmp484=((($conv483))|0)==4; //@line 698 "setlengths.cpp"
                if ($cmp484) { __label__ = 132;; } else { __label__ = 131;; } //@line 698 "setlengths.cpp"
                if (__label__ == 131) {
  
                  var $266=$next2; //@line 698 "setlengths.cpp"
                  var $type486=(($266+15)&4294967295); //@line 698 "setlengths.cpp"
                  var $267=HEAPU8[($type486)]; //@line 698 "setlengths.cpp"
                  var $conv487=((($267))&255); //@line 698 "setlengths.cpp"
                  var $cmp488=((($conv487))|0)==5; //@line 698 "setlengths.cpp"
                  if (!($cmp488)) { __label__ = 134;break $if_then433$$if_else466$176; } //@line 698 "setlengths.cpp"
                }
  
                var $268=$next3; //@line 698 "setlengths.cpp"
                var $ph490=(($268+8)&4294967295); //@line 698 "setlengths.cpp"
                var $269=HEAP32[(($ph490)>>2)]; //@line 698 "setlengths.cpp"
                var $phflags491=(($269+4)&4294967295); //@line 698 "setlengths.cpp"
                var $270=HEAP32[(($phflags491)>>2)]; //@line 698 "setlengths.cpp"
                var $and492=($270) & 8; //@line 698 "setlengths.cpp"
                var $tobool493=((($and492))|0)!=0; //@line 698 "setlengths.cpp"
                if (!($tobool493)) { __label__ = 134;break $if_then433$$if_else466$176; } //@line 698 "setlengths.cpp"
  
                var $271=$length_mod; //@line 699 "setlengths.cpp"
                var $sub495=((($271)-15)&4294967295); //@line 699 "setlengths.cpp"
                $length_mod=$sub495; //@line 699 "setlengths.cpp"
                ; //@line 699 "setlengths.cpp"
              }
            } while(0);
  
            var $272=HEAP32[((__ZZ11CalcLengthsP10TranslatorE14more_syllables)>>2)]; //@line 702 "setlengths.cpp"
            var $cmp498=((($272))|0)==0; //@line 702 "setlengths.cpp"
            if ($cmp498) { __label__ = 135;; } else { __label__ = 136;; } //@line 702 "setlengths.cpp"
            if (__label__ == 135) {
  
              var $273=HEAP32[((__ZL6speed1)>>2)]; //@line 703 "setlengths.cpp"
              var $274=$length_mod; //@line 703 "setlengths.cpp"
              var $mul500=((($274)*($273))&4294967295); //@line 703 "setlengths.cpp"
              $length_mod=$mul500; //@line 703 "setlengths.cpp"
              ; //@line 703 "setlengths.cpp"
            }
            else if (__label__ == 136) {
  
              var $275=HEAP32[((__ZZ11CalcLengthsP10TranslatorE14more_syllables)>>2)]; //@line 705 "setlengths.cpp"
              var $cmp502=((($275))|0)==1; //@line 705 "setlengths.cpp"
              if ($cmp502) { __label__ = 137;; } else { __label__ = 138;; } //@line 705 "setlengths.cpp"
              if (__label__ == 137) {
  
                var $276=HEAP32[((__ZL6speed2)>>2)]; //@line 706 "setlengths.cpp"
                var $277=$length_mod; //@line 706 "setlengths.cpp"
                var $mul504=((($277)*($276))&4294967295); //@line 706 "setlengths.cpp"
                $length_mod=$mul504; //@line 706 "setlengths.cpp"
                ; //@line 706 "setlengths.cpp"
              }
              else if (__label__ == 138) {
  
                var $278=HEAP32[((__ZL6speed3)>>2)]; //@line 708 "setlengths.cpp"
                var $279=$length_mod; //@line 708 "setlengths.cpp"
                var $mul506=((($279)*($278))&4294967295); //@line 708 "setlengths.cpp"
                $length_mod=$mul506; //@line 708 "setlengths.cpp"
                ;
              }
            }
  
            var $280=$length_mod; //@line 710 "setlengths.cpp"
            var $div509=((((($280))|0)/128)|0); //@line 710 "setlengths.cpp"
            $length_mod=$div509; //@line 710 "setlengths.cpp"
            var $cmp510=((($div509))|0) < 8; //@line 712 "setlengths.cpp"
            if ($cmp510) { __label__ = 140;; } else { __label__ = 141;; } //@line 712 "setlengths.cpp"
            if (__label__ == 140) {
  
              $length_mod=8; //@line 713 "setlengths.cpp"
              ; //@line 713 "setlengths.cpp"
            }
  
            var $281=$stress; //@line 715 "setlengths.cpp"
            var $cmp513=((($281))|0) >= 7; //@line 715 "setlengths.cpp"
            if ($cmp513) { __label__ = 142;; } else { __label__ = 144;; } //@line 715 "setlengths.cpp"
            $if_then514$$if_else520$199: do { 
              if (__label__ == 142) {
  
                var $282=$length_mod; //@line 718 "setlengths.cpp"
                var $add515=((($282)+20)&4294967295); //@line 718 "setlengths.cpp"
                $length_mod=$add515; //@line 718 "setlengths.cpp"
                var $283=$emphasized; //@line 719 "setlengths.cpp"
                var $tobool516=((($283))|0)!=0; //@line 719 "setlengths.cpp"
                if (!($tobool516)) { __label__ = 146;break $if_then514$$if_else520$199; } //@line 719 "setlengths.cpp"
  
                var $284=$length_mod; //@line 720 "setlengths.cpp"
                var $add518=((($284)+10)&4294967295); //@line 720 "setlengths.cpp"
                $length_mod=$add518; //@line 720 "setlengths.cpp"
                ; //@line 720 "setlengths.cpp"
              }
              else if (__label__ == 144) {
  
                var $285=$emphasized; //@line 723 "setlengths.cpp"
                var $tobool521=((($285))|0)!=0; //@line 723 "setlengths.cpp"
                if (!($tobool521)) { __label__ = 146;break $if_then514$$if_else520$199; } //@line 723 "setlengths.cpp"
  
                var $286=$length_mod; //@line 725 "setlengths.cpp"
                var $add523=((($286)+20)&4294967295); //@line 725 "setlengths.cpp"
                $length_mod=$add523; //@line 725 "setlengths.cpp"
                ; //@line 726 "setlengths.cpp"
              }
            } while(0);
  
            var $287=$stress; //@line 728 "setlengths.cpp"
            var $288=$tr_addr; //@line 728 "setlengths.cpp"
            var $stress_lengths=(($288+676)&4294967295); //@line 728 "setlengths.cpp"
            var $arrayidx526=(($stress_lengths+$287*2)&4294967295); //@line 728 "setlengths.cpp"
            var $289=HEAP16[(($arrayidx526)>>1)]; //@line 728 "setlengths.cpp"
            var $conv527=(tempInt=(($289)),(tempInt>=32768?tempInt-65536:tempInt)); //@line 728 "setlengths.cpp"
            $len=$conv527; //@line 728 "setlengths.cpp"
            var $cmp528=((($conv527))|0)==0; //@line 728 "setlengths.cpp"
            if ($cmp528) { __label__ = 147;; } else { __label__ = 148;; } //@line 728 "setlengths.cpp"
            if (__label__ == 147) {
  
              var $290=$tr_addr; //@line 729 "setlengths.cpp"
              var $stress_lengths530=(($290+676)&4294967295); //@line 729 "setlengths.cpp"
              var $arrayidx531=(($stress_lengths530+12)&4294967295); //@line 729 "setlengths.cpp"
              var $291=HEAP16[(($arrayidx531)>>1)]; //@line 729 "setlengths.cpp"
              var $conv532=(tempInt=(($291)),(tempInt>=32768?tempInt-65536:tempInt)); //@line 729 "setlengths.cpp"
              $len=$conv532; //@line 729 "setlengths.cpp"
              ; //@line 729 "setlengths.cpp"
            }
  
            var $292=$length_mod; //@line 731 "setlengths.cpp"
            var $293=$len; //@line 731 "setlengths.cpp"
            var $mul534=((($293)*($292))&4294967295); //@line 731 "setlengths.cpp"
            $length_mod=$mul534; //@line 731 "setlengths.cpp"
            var $294=$p; //@line 733 "setlengths.cpp"
            var $tone_ph=(($294+3)&4294967295); //@line 733 "setlengths.cpp"
            var $295=HEAPU8[($tone_ph)]; //@line 733 "setlengths.cpp"
            var $conv535=((($295))&255); //@line 733 "setlengths.cpp"
            var $cmp536=((($conv535))|0)!=0; //@line 733 "setlengths.cpp"
            if ($cmp536) { __label__ = 149;; } else { __label__ = 151;; } //@line 733 "setlengths.cpp"
            $if_then537$$if_end546$208: do { 
              if (__label__ == 149) {
  
                var $296=$p; //@line 735 "setlengths.cpp"
                var $tone_ph538=(($296+3)&4294967295); //@line 735 "setlengths.cpp"
                var $297=HEAPU8[($tone_ph538)]; //@line 735 "setlengths.cpp"
                var $idxprom=((($297))&255); //@line 735 "setlengths.cpp"
                var $arrayidx539=((_phoneme_tab+$idxprom*4)&4294967295); //@line 735 "setlengths.cpp"
                var $298=HEAP32[(($arrayidx539)>>2)]; //@line 735 "setlengths.cpp"
                var $std_length=(($298+14)&4294967295); //@line 735 "setlengths.cpp"
                var $299=HEAPU8[($std_length)]; //@line 735 "setlengths.cpp"
                var $conv540=((($299))&255); //@line 735 "setlengths.cpp"
                $tone_mod=$conv540; //@line 735 "setlengths.cpp"
                var $cmp541=((($conv540))|0) > 0; //@line 735 "setlengths.cpp"
                if (!($cmp541)) { __label__ = 151;break $if_then537$$if_end546$208; } //@line 735 "setlengths.cpp"
  
                var $300=$length_mod; //@line 738 "setlengths.cpp"
                var $301=$tone_mod; //@line 738 "setlengths.cpp"
                var $mul543=((($301)*($300))&4294967295); //@line 738 "setlengths.cpp"
                var $div544=((((($mul543))|0)/100)|0); //@line 738 "setlengths.cpp"
                $length_mod=$div544; //@line 738 "setlengths.cpp"
                ; //@line 739 "setlengths.cpp"
              }
            } while(0);
  
            var $302=$end_of_clause; //@line 742 "setlengths.cpp"
            var $cmp547=((($302))|0)==2; //@line 742 "setlengths.cpp"
            if ($cmp547) { __label__ = 152;; } else { __label__ = 155;; } //@line 742 "setlengths.cpp"
            if (__label__ == 152) {
  
              var $303=$p; //@line 745 "setlengths.cpp"
              var $ph549=(($303+8)&4294967295); //@line 745 "setlengths.cpp"
              var $304=HEAP32[(($ph549)>>2)]; //@line 745 "setlengths.cpp"
              var $std_length550=(($304+14)&4294967295); //@line 745 "setlengths.cpp"
              var $305=HEAPU8[($std_length550)]; //@line 745 "setlengths.cpp"
              var $conv551=((($305))&255); //@line 745 "setlengths.cpp"
              var $mul552=((($conv551)*2)&4294967295); //@line 745 "setlengths.cpp"
              $len=$mul552; //@line 745 "setlengths.cpp"
              var $306=$tr_addr; //@line 746 "setlengths.cpp"
              var $langopts553=(($306)&4294967295); //@line 746 "setlengths.cpp"
              var $stress_flags=(($langopts553+12)&4294967295); //@line 746 "setlengths.cpp"
              var $307=HEAP32[(($stress_flags)>>2)]; //@line 746 "setlengths.cpp"
              var $and554=($307) & 262144; //@line 746 "setlengths.cpp"
              var $tobool555=((($and554))|0)!=0; //@line 746 "setlengths.cpp"
              if ($tobool555) { __label__ = 153;; } else { __label__ = 154;; } //@line 746 "setlengths.cpp"
              if (__label__ == 153) {
  
                $len=200; //@line 747 "setlengths.cpp"
                ; //@line 747 "setlengths.cpp"
              }
  
              var $308=$length_mod; //@line 748 "setlengths.cpp"
              var $309=$len; //@line 748 "setlengths.cpp"
              var $sub558=((280-($309))&4294967295); //@line 748 "setlengths.cpp"
              var $div559=((((($sub558))|0)/3)|0); //@line 748 "setlengths.cpp"
              var $add560=((($div559)+256)&4294967295); //@line 748 "setlengths.cpp"
              var $mul561=((($add560)*($308))&4294967295); //@line 748 "setlengths.cpp"
              var $div562=((((($mul561))|0)/256)|0); //@line 748 "setlengths.cpp"
              $length_mod=$div562; //@line 748 "setlengths.cpp"
              ; //@line 749 "setlengths.cpp"
            }
  
            var $310=$length_mod; //@line 751 "setlengths.cpp"
            var $311=$tr_addr; //@line 751 "setlengths.cpp"
            var $langopts564=(($311)&4294967295); //@line 751 "setlengths.cpp"
            var $max_lengthmod=(($langopts564+272)&4294967295); //@line 751 "setlengths.cpp"
            var $312=HEAP32[(($max_lengthmod)>>2)]; //@line 751 "setlengths.cpp"
            var $313=HEAP32[((__ZL6speed1)>>2)]; //@line 751 "setlengths.cpp"
            var $mul565=((($313)*($312))&4294967295); //@line 751 "setlengths.cpp"
            var $cmp566=((($310))|0) > ((($mul565))|0); //@line 751 "setlengths.cpp"
            if ($cmp566) { __label__ = 156;; } else { __label__ = 157;; } //@line 751 "setlengths.cpp"
            if (__label__ == 156) {
  
              var $314=$tr_addr; //@line 754 "setlengths.cpp"
              var $langopts568=(($314)&4294967295); //@line 754 "setlengths.cpp"
              var $max_lengthmod569=(($langopts568+272)&4294967295); //@line 754 "setlengths.cpp"
              var $315=HEAP32[(($max_lengthmod569)>>2)]; //@line 754 "setlengths.cpp"
              var $316=HEAP32[((__ZL6speed1)>>2)]; //@line 754 "setlengths.cpp"
              var $mul570=((($316)*($315))&4294967295); //@line 754 "setlengths.cpp"
              $length_mod=$mul570; //@line 754 "setlengths.cpp"
              ; //@line 755 "setlengths.cpp"
            }
  
            var $317=$length_mod; //@line 757 "setlengths.cpp"
            var $div572=((((($317))|0)/128)|0); //@line 757 "setlengths.cpp"
            $length_mod=$div572; //@line 757 "setlengths.cpp"
            var $318=$p; //@line 759 "setlengths.cpp"
            var $type573=(($318+15)&4294967295); //@line 759 "setlengths.cpp"
            var $319=HEAPU8[($type573)]; //@line 759 "setlengths.cpp"
            var $conv574=((($319))&255); //@line 759 "setlengths.cpp"
            var $cmp575=((($conv574))|0)!=2; //@line 759 "setlengths.cpp"
            if ($cmp575) { __label__ = 158;; } else { __label__ = 159;; } //@line 759 "setlengths.cpp"
            if (__label__ == 158) {
  
              $length_mod=256; //@line 761 "setlengths.cpp"
              $min_drop=16; //@line 762 "setlengths.cpp"
              ; //@line 763 "setlengths.cpp"
            }
  
            var $320=$length_mod; //@line 764 "setlengths.cpp"
            var $conv578=((($320)) & 65535); //@line 764 "setlengths.cpp"
            var $321=$p; //@line 764 "setlengths.cpp"
            var $length579=(($321+12)&4294967295); //@line 764 "setlengths.cpp"
            HEAP16[(($length579)>>1)]=$conv578; //@line 764 "setlengths.cpp"
            var $322=$p; //@line 766 "setlengths.cpp"
            var $env580=(($322+14)&4294967295); //@line 766 "setlengths.cpp"
            var $323=HEAPU8[($env580)]; //@line 766 "setlengths.cpp"
            var $conv581=((($323))&255); //@line 766 "setlengths.cpp"
            var $cmp582=((($conv581))|0) >= 19; //@line 766 "setlengths.cpp"
            if ($cmp582) { __label__ = 160;; } else { __label__ = 161;; } //@line 766 "setlengths.cpp"
            if (__label__ == 160) {
  
              var $324=HEAP32[((__impure_ptr)>>2)]; //@line 768 "setlengths.cpp"
              var $_stderr=(($324+12)&4294967295); //@line 768 "setlengths.cpp"
              var $325=HEAP32[(($_stderr)>>2)]; //@line 768 "setlengths.cpp"
              var $326=_fwrite(((__str294)&4294967295), 28, 1, $325); //@line 768 "setlengths.cpp"
              var $327=$p; //@line 769 "setlengths.cpp"
              var $env584=(($327+14)&4294967295); //@line 769 "setlengths.cpp"
              HEAP8[($env584)]=0; //@line 769 "setlengths.cpp"
              ; //@line 770 "setlengths.cpp"
            }
  
            var $328=$p; //@line 774 "setlengths.cpp"
            var $env586=(($328+14)&4294967295); //@line 774 "setlengths.cpp"
            var $329=HEAPU8[($env586)]; //@line 774 "setlengths.cpp"
            var $conv587=((($329))&255); //@line 774 "setlengths.cpp"
            var $add588=((($conv587)+1)&4294967295); //@line 774 "setlengths.cpp"
            $env2=$add588; //@line 774 "setlengths.cpp"
            var $330=$p; //@line 776 "setlengths.cpp"
            var $tone_ph589=(($330+3)&4294967295); //@line 776 "setlengths.cpp"
            var $331=HEAPU8[($tone_ph589)]; //@line 776 "setlengths.cpp"
            var $conv590=((($331))&255); //@line 776 "setlengths.cpp"
            var $cmp591=((($conv590))|0)!=0; //@line 776 "setlengths.cpp"
            if ($cmp591) { __label__ = 162;; } else { __label__ = 163;; } //@line 776 "setlengths.cpp"
            if (__label__ == 162) {
  
              var $332=$p; //@line 778 "setlengths.cpp"
              var $tone_ph593=(($332+3)&4294967295); //@line 778 "setlengths.cpp"
              var $333=HEAPU8[($tone_ph593)]; //@line 778 "setlengths.cpp"
              var $conv594=((($333))&255); //@line 778 "setlengths.cpp"
              __Z17InterpretPhoneme2iP12PHONEME_DATA($conv594, $phdata_tone); //@line 778 "setlengths.cpp"
              var $334=HEAP32[(($pitch_env595)>>2)]; //@line 779 "setlengths.cpp"
              var $call596=__Z11GetEnvelopei($334); //@line 779 "setlengths.cpp"
              $pitch_env=$call596; //@line 779 "setlengths.cpp"
              ; //@line 780 "setlengths.cpp"
            }
            else if (__label__ == 163) {
  
              var $335=$env2; //@line 783 "setlengths.cpp"
              var $arrayidx598=((_envelope_data+$335*4)&4294967295); //@line 783 "setlengths.cpp"
              var $336=HEAP32[(($arrayidx598)>>2)]; //@line 783 "setlengths.cpp"
              $pitch_env=$336; //@line 783 "setlengths.cpp"
              ;
            }
  
            var $337=$p; //@line 786 "setlengths.cpp"
            var $pitch1600=(($337+20)&4294967295); //@line 786 "setlengths.cpp"
            var $338=HEAPU8[($pitch1600)]; //@line 786 "setlengths.cpp"
            var $conv601=((($338))&255); //@line 786 "setlengths.cpp"
            var $339=$p; //@line 786 "setlengths.cpp"
            var $pitch2602=(($339+21)&4294967295); //@line 786 "setlengths.cpp"
            var $340=HEAPU8[($pitch2602)]; //@line 786 "setlengths.cpp"
            var $conv603=((($340))&255); //@line 786 "setlengths.cpp"
            var $341=$p; //@line 786 "setlengths.cpp"
            var $pitch1604=(($341+20)&4294967295); //@line 786 "setlengths.cpp"
            var $342=HEAPU8[($pitch1604)]; //@line 786 "setlengths.cpp"
            var $conv605=((($342))&255); //@line 786 "setlengths.cpp"
            var $sub606=((($conv603)-($conv605))&4294967295); //@line 786 "setlengths.cpp"
            var $343=$pitch_env; //@line 786 "setlengths.cpp"
            var $arrayidx607=(($343)&4294967295); //@line 786 "setlengths.cpp"
            var $344=HEAPU8[($arrayidx607)]; //@line 786 "setlengths.cpp"
            var $conv608=((($344))&255); //@line 786 "setlengths.cpp"
            var $mul609=((($conv608)*($sub606))&4294967295); //@line 786 "setlengths.cpp"
            var $div610=((((($mul609))|0)/256)|0); //@line 786 "setlengths.cpp"
            var $add611=((($div610)+($conv601))&4294967295); //@line 786 "setlengths.cpp"
            $pitch_start=$add611; //@line 786 "setlengths.cpp"
            var $345=$pre_sonorant; //@line 788 "setlengths.cpp"
            var $tobool612=((($345))|0)!=0; //@line 788 "setlengths.cpp"
            if ($tobool612) { __lastLabel__ = 164; __label__ = 167;; } else { __lastLabel__ = 164; __label__ = 165;; } //@line 788 "setlengths.cpp"
            $if_then615$$lor_lhs_false613$231: do { 
              if (__label__ == 165) {
  
                var $346=$pre_voiced; //@line 788 "setlengths.cpp"
                var $tobool614=((($346))|0)!=0; //@line 788 "setlengths.cpp"
                if (!($tobool614)) { __label__ = 177;break $if_then615$$lor_lhs_false613$231; } //@line 788 "setlengths.cpp"
  
                var $_pr2=$pitch_start; //@line 791 "setlengths.cpp"
                __lastLabel__ = 166; __label__ = 167;break $if_then615$$lor_lhs_false613$231;
              }
            } while(0);
            $if_then615$$if_end650$234: do { 
              if (__label__ == 167) {
  
                var $347=__lastLabel__ == 166 ? $_pr2 : ($add611); //@line 791 "setlengths.cpp"
                var $cmp616=((($347))|0)==255; //@line 791 "setlengths.cpp"
                if ($cmp616) { __label__ = 168;; } else { __label__ = 169;; } //@line 791 "setlengths.cpp"
                if (__label__ == 168) {
  
                  var $348=$pitch_start; //@line 792 "setlengths.cpp"
                  $last_pitch=$348; //@line 792 "setlengths.cpp"
                  ; //@line 792 "setlengths.cpp"
                }
  
                var $349=$pitch_start; //@line 794 "setlengths.cpp"
                var $350=$last_pitch; //@line 794 "setlengths.cpp"
                var $sub619=((($349)-($350))&4294967295); //@line 794 "setlengths.cpp"
                var $cmp620=((($sub619))|0) > 16; //@line 794 "setlengths.cpp"
                if ($cmp620) { __label__ = 170;; } else { __label__ = 171;; } //@line 794 "setlengths.cpp"
                if (__label__ == 170) {
  
                  var $351=$pitch_start; //@line 795 "setlengths.cpp"
                  var $sub622=((($351)-16)&4294967295); //@line 795 "setlengths.cpp"
                  $last_pitch=$sub622; //@line 795 "setlengths.cpp"
                  ; //@line 795 "setlengths.cpp"
                }
  
                var $352=$last_pitch; //@line 797 "setlengths.cpp"
                var $conv624=((($352)) & 255); //@line 797 "setlengths.cpp"
                var $353=$prev; //@line 797 "setlengths.cpp"
                var $pitch1625=(($353+20)&4294967295); //@line 797 "setlengths.cpp"
                HEAP8[($pitch1625)]=$conv624; //@line 797 "setlengths.cpp"
                var $354=$pitch_start; //@line 798 "setlengths.cpp"
                var $conv626=((($354)) & 255); //@line 798 "setlengths.cpp"
                var $355=$prev; //@line 798 "setlengths.cpp"
                var $pitch2627=(($355+21)&4294967295); //@line 798 "setlengths.cpp"
                HEAP8[($pitch2627)]=$conv626; //@line 798 "setlengths.cpp"
                var $356=$last_pitch; //@line 799 "setlengths.cpp"
                var $357=$pitch_start; //@line 799 "setlengths.cpp"
                var $cmp628=((($356))|0) < ((($357))|0); //@line 799 "setlengths.cpp"
                var $358=$prev; //@line 801 "setlengths.cpp"
                var $env630=(($358+14)&4294967295); //@line 801 "setlengths.cpp"
                if ($cmp628) { __label__ = 172;; } else { __label__ = 173;; } //@line 799 "setlengths.cpp"
                if (__label__ == 172) {
  
                  HEAP8[($env630)]=2; //@line 801 "setlengths.cpp"
                  var $359=$env2; //@line 802 "setlengths.cpp"
                  var $conv631=((($359)) & 255); //@line 802 "setlengths.cpp"
                  var $360=$p; //@line 802 "setlengths.cpp"
                  var $env632=(($360+14)&4294967295); //@line 802 "setlengths.cpp"
                  HEAP8[($env632)]=$conv631; //@line 802 "setlengths.cpp"
                  ; //@line 803 "setlengths.cpp"
                }
                else if (__label__ == 173) {
  
                  HEAP8[($env630)]=0; //@line 806 "setlengths.cpp"
                  ;
                }
  
                var $361=$length_mod; //@line 809 "setlengths.cpp"
                var $conv636=((($361)) & 65535); //@line 809 "setlengths.cpp"
                var $362=$prev; //@line 809 "setlengths.cpp"
                var $length637=(($362+12)&4294967295); //@line 809 "setlengths.cpp"
                HEAP16[(($length637)>>1)]=$conv636; //@line 809 "setlengths.cpp"
                var $363=$p; //@line 811 "setlengths.cpp"
                var $amp638=(($363+18)&4294967295); //@line 811 "setlengths.cpp"
                var $364=HEAP8[($amp638)]; //@line 811 "setlengths.cpp"
                var $365=$prev; //@line 811 "setlengths.cpp"
                var $amp639=(($365+18)&4294967295); //@line 811 "setlengths.cpp"
                HEAP8[($amp639)]=$364; //@line 811 "setlengths.cpp"
                var $366=$prev; //@line 812 "setlengths.cpp"
                var $type640=(($366+15)&4294967295); //@line 812 "setlengths.cpp"
                var $367=HEAPU8[($type640)]; //@line 812 "setlengths.cpp"
                var $conv641=((($367))&255); //@line 812 "setlengths.cpp"
                var $cmp642=((($conv641))|0)!=3; //@line 812 "setlengths.cpp"
                if (!($cmp642)) { __label__ = 177;break $if_then615$$if_end650$234; } //@line 812 "setlengths.cpp"
  
                var $368=$prev; //@line 812 "setlengths.cpp"
                var $amp644=(($368+18)&4294967295); //@line 812 "setlengths.cpp"
                var $369=HEAPU8[($amp644)]; //@line 812 "setlengths.cpp"
                var $conv645=((($369))&255); //@line 812 "setlengths.cpp"
                var $cmp646=((($conv645))|0) > 18; //@line 812 "setlengths.cpp"
                if (!($cmp646)) { __label__ = 177;break $if_then615$$if_end650$234; } //@line 812 "setlengths.cpp"
  
                var $370=$prev; //@line 813 "setlengths.cpp"
                var $amp648=(($370+18)&4294967295); //@line 813 "setlengths.cpp"
                HEAP8[($amp648)]=18; //@line 813 "setlengths.cpp"
                ; //@line 813 "setlengths.cpp"
              }
            } while(0);
  
            var $371=$next; //@line 817 "setlengths.cpp"
            var $synthflags651=(($371+4)&4294967295); //@line 817 "setlengths.cpp"
            var $372=HEAPU16[(($synthflags651)>>1)]; //@line 817 "setlengths.cpp"
            var $conv652=((($372))&65535); //@line 817 "setlengths.cpp"
            var $and653=($conv652) & -2; //@line 817 "setlengths.cpp"
            var $conv654=((($and653)) & 65535); //@line 817 "setlengths.cpp"
            HEAP16[(($synthflags651)>>1)]=$conv654; //@line 817 "setlengths.cpp"
            var $373=$next; //@line 818 "setlengths.cpp"
            var $type655=(($373+15)&4294967295); //@line 818 "setlengths.cpp"
            var $374=HEAPU8[($type655)]; //@line 818 "setlengths.cpp"
            var $conv656=((($374))&255); //@line 818 "setlengths.cpp"
            var $cmp657=((($conv656))|0)==8; //@line 818 "setlengths.cpp"
            if ($cmp657) { __label__ = 178;; } else { __label__ = 180;; } //@line 818 "setlengths.cpp"
            $land_lhs_true658$$if_end666$249: do { 
              if (__label__ == 178) {
  
                var $375=$next2; //@line 818 "setlengths.cpp"
                var $type659=(($375+15)&4294967295); //@line 818 "setlengths.cpp"
                var $376=HEAPU8[($type659)]; //@line 818 "setlengths.cpp"
                var $conv660=((($376))&255); //@line 818 "setlengths.cpp"
                var $cmp661=((($conv660))|0)!=2; //@line 818 "setlengths.cpp"
                if (!($cmp661)) { __label__ = 180;break $land_lhs_true658$$if_end666$249; } //@line 818 "setlengths.cpp"
  
                var $377=$next; //@line 819 "setlengths.cpp"
                var $synthflags663=(($377+4)&4294967295); //@line 819 "setlengths.cpp"
                var $378=HEAPU16[(($synthflags663)>>1)]; //@line 819 "setlengths.cpp"
                var $conv664=((($378))&65535); //@line 819 "setlengths.cpp"
                var $or=($conv664) | 1; //@line 819 "setlengths.cpp"
                var $conv665=((($or)) & 65535); //@line 819 "setlengths.cpp"
                HEAP16[(($synthflags663)>>1)]=$conv665; //@line 819 "setlengths.cpp"
                ; //@line 819 "setlengths.cpp"
              }
            } while(0);
  
            var $379=$next; //@line 821 "setlengths.cpp"
            var $type667=(($379+15)&4294967295); //@line 821 "setlengths.cpp"
            var $380=HEAPU8[($type667)]; //@line 821 "setlengths.cpp"
            var $conv668=((($380))&255); //@line 821 "setlengths.cpp"
            var $cmp669=((($conv668))|0)==3; //@line 821 "setlengths.cpp"
            if ($cmp669) { __label__ = 181;; } else { __label__ = 186;; } //@line 821 "setlengths.cpp"
            $if_then670$$if_end697$253: do { 
              if (__label__ == 181) {
  
                var $381=$next; //@line 823 "setlengths.cpp"
                var $synthflags671=(($381+4)&4294967295); //@line 823 "setlengths.cpp"
                var $382=HEAPU16[(($synthflags671)>>1)]; //@line 823 "setlengths.cpp"
                var $conv672=((($382))&65535); //@line 823 "setlengths.cpp"
                var $or673=($conv672) | 1; //@line 823 "setlengths.cpp"
                var $conv674=((($or673)) & 65535); //@line 823 "setlengths.cpp"
                HEAP16[(($synthflags671)>>1)]=$conv674; //@line 823 "setlengths.cpp"
                var $383=$next2; //@line 825 "setlengths.cpp"
                var $type675=(($383+15)&4294967295); //@line 825 "setlengths.cpp"
                var $384=HEAPU8[($type675)]; //@line 825 "setlengths.cpp"
                var $conv676=((($384))&255); //@line 825 "setlengths.cpp"
                var $cmp677=((($conv676))|0)==2; //@line 825 "setlengths.cpp"
                if ($cmp677) { __label__ = 182;; } else { __label__ = 183;; } //@line 825 "setlengths.cpp"
                if (__label__ == 182) {
  
                  var $385=$next; //@line 827 "setlengths.cpp"
                  var $synthflags679=(($385+4)&4294967295); //@line 827 "setlengths.cpp"
                  var $386=HEAPU16[(($synthflags679)>>1)]; //@line 827 "setlengths.cpp"
                  var $conv680=((($386))&65535); //@line 827 "setlengths.cpp"
                  var $and681=($conv680) & -2; //@line 827 "setlengths.cpp"
                  var $conv682=((($and681)) & 65535); //@line 827 "setlengths.cpp"
                  HEAP16[(($synthflags679)>>1)]=$conv682; //@line 827 "setlengths.cpp"
                  ; //@line 828 "setlengths.cpp"
                }
  
                var $387=$next2; //@line 830 "setlengths.cpp"
                var $type684=(($387+15)&4294967295); //@line 830 "setlengths.cpp"
                var $388=HEAPU8[($type684)]; //@line 830 "setlengths.cpp"
                var $conv685=((($388))&255); //@line 830 "setlengths.cpp"
                var $cmp686=((($conv685))|0)!=2; //@line 830 "setlengths.cpp"
                if (!($cmp686)) { __label__ = 186;break $if_then670$$if_end697$253; } //@line 830 "setlengths.cpp"
  
                var $389=$next; //@line 832 "setlengths.cpp"
                var $ph688=(($389+8)&4294967295); //@line 832 "setlengths.cpp"
                var $390=HEAP32[(($ph688)>>2)]; //@line 832 "setlengths.cpp"
                var $mnemonic=(($390)&4294967295); //@line 832 "setlengths.cpp"
                var $391=HEAP32[(($mnemonic)>>2)]; //@line 832 "setlengths.cpp"
                var $cmp689=((($391))|0)==12146; //@line 832 "setlengths.cpp"
                if (!($cmp689)) { __label__ = 186;break $if_then670$$if_end697$253; } //@line 832 "setlengths.cpp"
  
                var $392=$next; //@line 834 "setlengths.cpp"
                var $synthflags691=(($392+4)&4294967295); //@line 834 "setlengths.cpp"
                var $393=HEAPU16[(($synthflags691)>>1)]; //@line 834 "setlengths.cpp"
                var $conv692=((($393))&65535); //@line 834 "setlengths.cpp"
                var $and693=($conv692) & -2; //@line 834 "setlengths.cpp"
                var $conv694=((($and693)) & 65535); //@line 834 "setlengths.cpp"
                HEAP16[(($synthflags691)>>1)]=$conv694; //@line 834 "setlengths.cpp"
                ; //@line 836 "setlengths.cpp"
              }
            } while(0);
  
            var $394=$min_drop; //@line 840 "setlengths.cpp"
            var $cmp698=((($394))|0) > 0; //@line 840 "setlengths.cpp"
            if ($cmp698) { __label__ = 187;; } else { __label__ = 191;; } //@line 840 "setlengths.cpp"
            $land_lhs_true699$$if_end715$261: do { 
              if (__label__ == 187) {
  
                var $395=$p; //@line 840 "setlengths.cpp"
                var $pitch2700=(($395+21)&4294967295); //@line 840 "setlengths.cpp"
                var $396=HEAPU8[($pitch2700)]; //@line 840 "setlengths.cpp"
                var $conv701=((($396))&255); //@line 840 "setlengths.cpp"
                var $397=$p; //@line 840 "setlengths.cpp"
                var $pitch1702=(($397+20)&4294967295); //@line 840 "setlengths.cpp"
                var $398=HEAPU8[($pitch1702)]; //@line 840 "setlengths.cpp"
                var $conv703=((($398))&255); //@line 840 "setlengths.cpp"
                var $sub704=((($conv701)-($conv703))&4294967295); //@line 840 "setlengths.cpp"
                var $399=$min_drop; //@line 840 "setlengths.cpp"
                var $cmp705=((($sub704))|0) < ((($399))|0); //@line 840 "setlengths.cpp"
                if (!($cmp705)) { __label__ = 191;break $land_lhs_true699$$if_end715$261; } //@line 840 "setlengths.cpp"
  
                var $400=$p; //@line 842 "setlengths.cpp"
                var $pitch2707=(($400+21)&4294967295); //@line 842 "setlengths.cpp"
                var $401=HEAPU8[($pitch2707)]; //@line 842 "setlengths.cpp"
                var $conv708=((($401))&255); //@line 842 "setlengths.cpp"
                var $402=$min_drop; //@line 842 "setlengths.cpp"
                var $sub709=((($conv708)-($402))&4294967295); //@line 842 "setlengths.cpp"
                $pitch1=$sub709; //@line 842 "setlengths.cpp"
                var $403=$pitch1; //@line 843 "setlengths.cpp"
                var $cmp710=((($403))|0) < 0; //@line 843 "setlengths.cpp"
                if ($cmp710) { __label__ = 189;; } else { __label__ = 190;; } //@line 843 "setlengths.cpp"
                if (__label__ == 189) {
  
                  $pitch1=0; //@line 844 "setlengths.cpp"
                  ; //@line 844 "setlengths.cpp"
                }
  
                var $404=$pitch1; //@line 845 "setlengths.cpp"
                var $conv713=((($404)) & 255); //@line 845 "setlengths.cpp"
                var $405=$p; //@line 845 "setlengths.cpp"
                var $pitch1714=(($405+20)&4294967295); //@line 845 "setlengths.cpp"
                HEAP8[($pitch1714)]=$conv713; //@line 845 "setlengths.cpp"
                ; //@line 846 "setlengths.cpp"
              }
            } while(0);
  
            var $406=$p; //@line 848 "setlengths.cpp"
            var $pitch1716=(($406+20)&4294967295); //@line 848 "setlengths.cpp"
            var $407=HEAPU8[($pitch1716)]; //@line 848 "setlengths.cpp"
            var $conv717=((($407))&255); //@line 848 "setlengths.cpp"
            var $408=$p; //@line 848 "setlengths.cpp"
            var $pitch2718=(($408+21)&4294967295); //@line 848 "setlengths.cpp"
            var $409=HEAPU8[($pitch2718)]; //@line 848 "setlengths.cpp"
            var $conv719=((($409))&255); //@line 848 "setlengths.cpp"
            var $410=$p; //@line 848 "setlengths.cpp"
            var $pitch1720=(($410+20)&4294967295); //@line 848 "setlengths.cpp"
            var $411=HEAPU8[($pitch1720)]; //@line 848 "setlengths.cpp"
            var $conv721=((($411))&255); //@line 848 "setlengths.cpp"
            var $sub722=((($conv719)-($conv721))&4294967295); //@line 848 "setlengths.cpp"
            var $412=$p; //@line 848 "setlengths.cpp"
            var $env723=(($412+14)&4294967295); //@line 848 "setlengths.cpp"
            var $413=HEAPU8[($env723)]; //@line 848 "setlengths.cpp"
            var $idxprom724=((($413))&255); //@line 848 "setlengths.cpp"
            var $arrayidx725=((_envelope_data+$idxprom724*4)&4294967295); //@line 848 "setlengths.cpp"
            var $414=HEAP32[(($arrayidx725)>>2)]; //@line 848 "setlengths.cpp"
            var $arrayidx726=(($414+127)&4294967295); //@line 848 "setlengths.cpp"
            var $415=HEAPU8[($arrayidx726)]; //@line 848 "setlengths.cpp"
            var $conv727=((($415))&255); //@line 848 "setlengths.cpp"
            var $mul728=((($conv727)*($sub722))&4294967295); //@line 848 "setlengths.cpp"
            var $div729=((((($mul728))|0)/256)|0); //@line 848 "setlengths.cpp"
            var $add730=((($div729)+($conv717))&4294967295); //@line 848 "setlengths.cpp"
            $last_pitch=$add730; //@line 848 "setlengths.cpp"
            $pre_sonorant=0; //@line 849 "setlengths.cpp"
            $pre_voiced=0; //@line 850 "setlengths.cpp"
            ; //@line 851 "setlengths.cpp"
          }
  
          var $416=$ix; //@line 441 "setlengths.cpp"
          var $inc732=((($416)+1)&4294967295); //@line 441 "setlengths.cpp"
          $ix=$inc732; //@line 441 "setlengths.cpp"
          var $417=$ix; //@line 441 "setlengths.cpp"
          var $418=HEAP32[((_n_phoneme_list)>>2)]; //@line 441 "setlengths.cpp"
          var $cmp=((($417))|0) < ((($418))|0); //@line 441 "setlengths.cpp"
          if ($cmp) { __label__ = 2;continue $for_body$4; } else { __label__ = 193;break $for_body_lr_ph$$for_end733$2; } //@line 441 "setlengths.cpp"
        }
      }
    } while(0);
  
    STACKTOP = __stackBase__;
    return; //@line 854 "setlengths.cpp"
    return;
  }
  

  function __ZL11DoEmbedded2Pi($embix) {
    ;
    var __label__;
  
    var $embix_addr;
    var $word;
    $embix_addr=$embix;
    ; //@line 398 "setlengths.cpp"
    while(1) { 
  
      var $0=$embix_addr; //@line 399 "setlengths.cpp"
      var $1=HEAP32[(($0)>>2)]; //@line 399 "setlengths.cpp"
      var $inc=((($1)+1)&4294967295); //@line 399 "setlengths.cpp"
      HEAP32[(($0)>>2)]=$inc; //@line 399 "setlengths.cpp"
      var $arrayidx=((_embedded_list+$1*4)&4294967295); //@line 399 "setlengths.cpp"
      var $2=HEAP32[(($arrayidx)>>2)]; //@line 399 "setlengths.cpp"
      $word=$2; //@line 399 "setlengths.cpp"
      var $3=$word; //@line 401 "setlengths.cpp"
      var $and=($3) & 31; //@line 401 "setlengths.cpp"
      var $cmp=((($and))|0)==2; //@line 401 "setlengths.cpp"
      if ($cmp) { __label__ = 2;; } else { __label__ = 3;; } //@line 401 "setlengths.cpp"
      if (__label__ == 2) {
  
        var $4=$word; //@line 404 "setlengths.cpp"
        var $and1=($4) & 127; //@line 404 "setlengths.cpp"
        var $5=$word; //@line 404 "setlengths.cpp"
        var $shr=($5) >>> 8; //@line 404 "setlengths.cpp"
        __Z11SetEmbeddedii($and1, $shr); //@line 404 "setlengths.cpp"
        __Z8SetSpeedi(1); //@line 405 "setlengths.cpp"
        ; //@line 406 "setlengths.cpp"
      }
  
      var $6=$word; //@line 407 "setlengths.cpp"
      var $and2=($6) & 128; //@line 407 "setlengths.cpp"
      var $cmp3=((($and2))|0)==0; //@line 407 "setlengths.cpp"
      if ($cmp3) { __label__ = 1;continue ; } else { __label__ = 4;break ; } //@line 407 "setlengths.cpp"
    }
  
    ;
    return; //@line 408 "setlengths.cpp"
    return;
  }
  

  function __Z20LookupAccentedLetterP10TranslatorjPc($tr, $letter, $ph_buf) {
    var __stackBase__  = STACKTOP; STACKTOP += 128; _memset(__stackBase__, 0, 128);
    var __label__;
    var __lastLabel__ = null;
  
    var $tr_addr;
    var $letter_addr;
    var $ph_buf_addr;
    var $accent_data;
    var $accent1;
    var $accent2;
    var $basic_letter;
    var $letter2;
    var $ph_letter1=__stackBase__;
    var $ph_letter2=__stackBase__+32;
    var $ph_accent1=__stackBase__+64;
    var $ph_accent2=__stackBase__+96;
    $tr_addr=$tr;
    $letter_addr=$letter;
    $ph_buf_addr=$ph_buf;
    $accent_data=0; //@line 418 "numbers.cpp"
    $accent1=0; //@line 419 "numbers.cpp"
    $accent2=0; //@line 420 "numbers.cpp"
    $letter2=0; //@line 422 "numbers.cpp"
    var $arrayidx=(($ph_accent2)&4294967295); //@line 428 "numbers.cpp"
    HEAP8[($arrayidx)]=0; //@line 428 "numbers.cpp"
    var $0=$letter_addr; //@line 430 "numbers.cpp"
    var $cmp=((($0))>>>0) >= 224; //@line 430 "numbers.cpp"
    var $1=$letter_addr; //@line 430 "numbers.cpp"
    var $cmp1=((($1))>>>0) < 383; //@line 430 "numbers.cpp"
    var $or_cond=($cmp) & ($cmp1); //@line 430 "numbers.cpp"
    var $2=$letter_addr; //@line 432 "numbers.cpp"
    if ($or_cond) { __label__ = 1;; } else { __label__ = 2;; } //@line 430 "numbers.cpp"
    if (__label__ == 1) {
  
      var $sub=((($2)-224)&4294967295); //@line 432 "numbers.cpp"
      var $arrayidx2=((__ZL18letter_accents_0e0+$sub*2)&4294967295); //@line 432 "numbers.cpp"
      var $3=HEAPU16[(($arrayidx2)>>1)]; //@line 432 "numbers.cpp"
      var $conv=((($3))&65535); //@line 432 "numbers.cpp"
      $accent_data=$conv; //@line 432 "numbers.cpp"
      __lastLabel__ = 1; ; //@line 433 "numbers.cpp"
    }
    else if (__label__ == 2) {
  
      var $cmp3=((($2))>>>0) >= 592; //@line 435 "numbers.cpp"
      var $4=$letter_addr; //@line 435 "numbers.cpp"
      var $cmp5=((($4))>>>0) <= 680; //@line 435 "numbers.cpp"
      var $or_cond1=($cmp3) & ($cmp5); //@line 435 "numbers.cpp"
      if ($or_cond1) { __label__ = 3;; } else { __label__ = 4;; } //@line 435 "numbers.cpp"
      if (__label__ == 3) {
  
        var $5=$letter_addr; //@line 437 "numbers.cpp"
        var $sub7=((($5)-592)&4294967295); //@line 437 "numbers.cpp"
        var $arrayidx8=((__ZL18letter_accents_250+$sub7*2)&4294967295); //@line 437 "numbers.cpp"
        var $6=HEAPU16[(($arrayidx8)>>1)]; //@line 437 "numbers.cpp"
        var $conv9=((($6))&65535); //@line 437 "numbers.cpp"
        $accent_data=$conv9; //@line 437 "numbers.cpp"
        __lastLabel__ = 3; ; //@line 438 "numbers.cpp"
      }
      else if (__label__ == 4) {
  
        var $_pr=$accent_data; //@line 440 "numbers.cpp"
        __lastLabel__ = 4; ;
      }
    }
  
    var $7=__lastLabel__ == 4 ? $_pr : (__lastLabel__ == 3 ? $conv9 : ($conv)); //@line 440 "numbers.cpp"
    var $cmp11=((($7))|0)!=0; //@line 440 "numbers.cpp"
    if ($cmp11) { __label__ = 6;; } else { __label__ = 24;; } //@line 440 "numbers.cpp"
    $if_then12$$if_end91$9: do { 
      if (__label__ == 6) {
  
        var $8=$accent_data; //@line 442 "numbers.cpp"
        var $and=($8) & 63; //@line 442 "numbers.cpp"
        var $add=((($and)+59)&4294967295); //@line 442 "numbers.cpp"
        $basic_letter=$add; //@line 442 "numbers.cpp"
        var $9=$basic_letter; //@line 443 "numbers.cpp"
        var $cmp13=((($9))|0) < 97; //@line 443 "numbers.cpp"
        if ($cmp13) { __label__ = 7;; } else { __label__ = 8;; } //@line 443 "numbers.cpp"
        if (__label__ == 7) {
  
          var $10=$basic_letter; //@line 444 "numbers.cpp"
          var $sub15=((($10)-59)&4294967295); //@line 444 "numbers.cpp"
          var $arrayidx16=((__ZL13non_ascii_tab+$sub15*2)&4294967295); //@line 444 "numbers.cpp"
          var $11=HEAP16[(($arrayidx16)>>1)]; //@line 444 "numbers.cpp"
          var $conv17=(tempInt=(($11)),(tempInt>=32768?tempInt-65536:tempInt)); //@line 444 "numbers.cpp"
          $basic_letter=$conv17; //@line 444 "numbers.cpp"
          ; //@line 444 "numbers.cpp"
        }
  
        var $12=$accent_data; //@line 446 "numbers.cpp"
        var $and19=($12) & 32768; //@line 446 "numbers.cpp"
        var $tobool=((($and19))|0)!=0; //@line 446 "numbers.cpp"
        var $13=$accent_data; //@line 448 "numbers.cpp"
        var $shr=($13) >> 6; //@line 448 "numbers.cpp"
        if ($tobool) { __label__ = 9;; } else { __label__ = 10;; } //@line 446 "numbers.cpp"
        if (__label__ == 9) {
  
          var $and21=($shr) & 63; //@line 448 "numbers.cpp"
          $letter2=$and21; //@line 448 "numbers.cpp"
          var $14=$letter2; //@line 449 "numbers.cpp"
          var $add22=((($14)+59)&4294967295); //@line 449 "numbers.cpp"
          $letter2=$add22; //@line 449 "numbers.cpp"
          var $15=$accent_data; //@line 450 "numbers.cpp"
          var $shr23=($15) >> 12; //@line 450 "numbers.cpp"
          var $and24=($shr23) & 7; //@line 450 "numbers.cpp"
          $accent2=$and24; //@line 450 "numbers.cpp"
          ; //@line 451 "numbers.cpp"
        }
        else if (__label__ == 10) {
  
          var $and27=($shr) & 31; //@line 454 "numbers.cpp"
          $accent1=$and27; //@line 454 "numbers.cpp"
          var $16=$accent_data; //@line 455 "numbers.cpp"
          var $shr28=($16) >> 11; //@line 455 "numbers.cpp"
          var $and29=($shr28) & 15; //@line 455 "numbers.cpp"
          $accent2=$and29; //@line 455 "numbers.cpp"
          ;
        }
  
        var $17=$tr_addr; //@line 459 "numbers.cpp"
        var $18=$accent1; //@line 459 "numbers.cpp"
        var $arrayidx31=((__ZL11accents_tab+$18*8)&4294967295); //@line 459 "numbers.cpp"
        var $name=(($arrayidx31)&4294967295); //@line 459 "numbers.cpp"
        var $19=HEAP32[(($name)>>2)]; //@line 459 "numbers.cpp"
        var $arraydecay=(($ph_accent1)&4294967295); //@line 459 "numbers.cpp"
        var $call=__Z6LookupP10TranslatorPKcPc($17, $19, $arraydecay); //@line 459 "numbers.cpp"
        var $cmp32=((($call))|0)!=0; //@line 459 "numbers.cpp"
        if (!($cmp32)) { __label__ = 24;break $if_then12$$if_end91$9; } //@line 459 "numbers.cpp"
  
        var $20=$tr_addr; //@line 462 "numbers.cpp"
        var $21=$basic_letter; //@line 462 "numbers.cpp"
        var $arraydecay34=(($ph_letter1)&4294967295); //@line 462 "numbers.cpp"
        var $call35=__ZL13LookupLetter2P10TranslatorjPc($20, $21, $arraydecay34); //@line 462 "numbers.cpp"
        var $cmp36=((($call35))|0)!=0; //@line 462 "numbers.cpp"
        if (!($cmp36)) { __label__ = 24;break $if_then12$$if_end91$9; } //@line 462 "numbers.cpp"
  
        var $22=$accent2; //@line 464 "numbers.cpp"
        var $cmp38=((($22))|0)!=0; //@line 464 "numbers.cpp"
        if ($cmp38) { __label__ = 14;; } else { __label__ = 16;; } //@line 464 "numbers.cpp"
        $if_then39$$if_end56$20: do { 
          if (__label__ == 14) {
  
            var $23=$tr_addr; //@line 466 "numbers.cpp"
            var $24=$accent2; //@line 466 "numbers.cpp"
            var $arrayidx40=((__ZL11accents_tab+$24*8)&4294967295); //@line 466 "numbers.cpp"
            var $name41=(($arrayidx40)&4294967295); //@line 466 "numbers.cpp"
            var $25=HEAP32[(($name41)>>2)]; //@line 466 "numbers.cpp"
            var $arraydecay42=(($ph_accent2)&4294967295); //@line 466 "numbers.cpp"
            var $call43=__Z6LookupP10TranslatorPKcPc($23, $25, $arraydecay42); //@line 466 "numbers.cpp"
            var $26=$accent2; //@line 471 "numbers.cpp"
            var $arrayidx47=((__ZL11accents_tab+$26*8)&4294967295); //@line 471 "numbers.cpp"
            var $flags=(($arrayidx47+4)&4294967295); //@line 471 "numbers.cpp"
            var $27=HEAP32[(($flags)>>2)]; //@line 471 "numbers.cpp"
            var $and48=($27) & 1; //@line 471 "numbers.cpp"
            var $tobool49=((($and48))|0)!=0; //@line 471 "numbers.cpp"
            if (!($tobool49)) { __label__ = 16;break $if_then39$$if_end56$20; } //@line 471 "numbers.cpp"
  
            var $28=$ph_buf_addr; //@line 473 "numbers.cpp"
            var $arraydecay51=(($ph_accent2)&4294967295); //@line 473 "numbers.cpp"
            var $call52=_strcpy($28, $arraydecay51); //@line 473 "numbers.cpp"
            var $29=$ph_buf_addr; //@line 474 "numbers.cpp"
            var $call53=_strlen($29); //@line 474 "numbers.cpp"
            var $30=$ph_buf_addr; //@line 474 "numbers.cpp"
            var $add_ptr=(($30+$call53)&4294967295); //@line 474 "numbers.cpp"
            $ph_buf_addr=$add_ptr; //@line 474 "numbers.cpp"
            var $arrayidx54=(($ph_accent2)&4294967295); //@line 475 "numbers.cpp"
            HEAP8[($arrayidx54)]=0; //@line 475 "numbers.cpp"
            ; //@line 476 "numbers.cpp"
          }
        } while(0);
  
        var $31=$letter2; //@line 478 "numbers.cpp"
        var $cmp57=((($31))|0)!=0; //@line 478 "numbers.cpp"
        if ($cmp57) { __label__ = 17;; } else { __label__ = 18;; } //@line 478 "numbers.cpp"
        if (__label__ == 17) {
  
          var $32=$tr_addr; //@line 481 "numbers.cpp"
          var $33=$letter2; //@line 481 "numbers.cpp"
          var $arraydecay59=(($ph_letter2)&4294967295); //@line 481 "numbers.cpp"
          var $call60=__ZL13LookupLetter2P10TranslatorjPc($32, $33, $arraydecay59); //@line 481 "numbers.cpp"
          var $34=$ph_buf_addr; //@line 482 "numbers.cpp"
          var $arraydecay61=(($ph_accent1)&4294967295); //@line 482 "numbers.cpp"
          var $arraydecay62=(($ph_letter1)&4294967295); //@line 482 "numbers.cpp"
          var $arraydecay63=(($ph_letter2)&4294967295); //@line 482 "numbers.cpp"
          var $arraydecay64=(($ph_accent2)&4294967295); //@line 482 "numbers.cpp"
          var $call65=_sprintf($34, ((__str297)&4294967295), allocate([$arraydecay61,0,0,0,23,0,0,0,$arraydecay62,0,0,0,6,0,0,0,$arraydecay63,0,0,0,$arraydecay64,0,0,0], ["i8*",0,0,0,"i32",0,0,0,"i8*",0,0,0,"i32",0,0,0,"i8*",0,0,0,"i8*",0,0,0], ALLOC_STACK)); //@line 482 "numbers.cpp"
          ; //@line 483 "numbers.cpp"
        }
        else if (__label__ == 18) {
  
          var $35=$accent1; //@line 486 "numbers.cpp"
          var $cmp67=((($35))|0)==0; //@line 486 "numbers.cpp"
          if ($cmp67) { __label__ = 19;; } else { __label__ = 20;; } //@line 486 "numbers.cpp"
          if (__label__ == 19) {
  
            var $36=$ph_buf_addr; //@line 487 "numbers.cpp"
            var $arraydecay69=(($ph_letter1)&4294967295); //@line 487 "numbers.cpp"
            var $call70=_strcpy($36, $arraydecay69); //@line 487 "numbers.cpp"
            ; //@line 487 "numbers.cpp"
          }
          else if (__label__ == 20) {
  
            var $37=$tr_addr; //@line 489 "numbers.cpp"
            var $langopts=(($37)&4294967295); //@line 489 "numbers.cpp"
            var $accents=(($langopts+220)&4294967295); //@line 489 "numbers.cpp"
            var $38=HEAP32[(($accents)>>2)]; //@line 489 "numbers.cpp"
            var $and72=($38) & 1; //@line 489 "numbers.cpp"
            var $tobool73=((($and72))|0)!=0; //@line 489 "numbers.cpp"
            if ($tobool73) { __label__ = 22;; } else { __label__ = 21;; } //@line 489 "numbers.cpp"
            $if_then78$$lor_lhs_false$30: do { 
              if (__label__ == 21) {
  
                var $39=$accent1; //@line 489 "numbers.cpp"
                var $arrayidx74=((__ZL11accents_tab+$39*8)&4294967295); //@line 489 "numbers.cpp"
                var $flags75=(($arrayidx74+4)&4294967295); //@line 489 "numbers.cpp"
                var $40=HEAP32[(($flags75)>>2)]; //@line 489 "numbers.cpp"
                var $and76=($40) & 1; //@line 489 "numbers.cpp"
                var $tobool77=((($and76))|0)!=0; //@line 489 "numbers.cpp"
                if ($tobool77) { __label__ = 22;break $if_then78$$lor_lhs_false$30; } //@line 489 "numbers.cpp"
  
                var $42=$ph_buf_addr; //@line 492 "numbers.cpp"
                var $arraydecay83=(($ph_letter1)&4294967295); //@line 492 "numbers.cpp"
                var $arraydecay84=(($ph_accent1)&4294967295); //@line 492 "numbers.cpp"
                var $call85=_sprintf($42, ((__str2299)&4294967295), allocate([4,0,0,0,$arraydecay83,0,0,0,23,0,0,0,$arraydecay84,0,0,0,23,0,0,0], ["i32",0,0,0,"i8*",0,0,0,"i32",0,0,0,"i8*",0,0,0,"i32",0,0,0], ALLOC_STACK)); //@line 492 "numbers.cpp"
                __label__ = 24;break $if_then12$$if_end91$9;
              }
            } while(0);
  
            var $41=$ph_buf_addr; //@line 490 "numbers.cpp"
            var $arraydecay79=(($ph_accent1)&4294967295); //@line 490 "numbers.cpp"
            var $arraydecay80=(($ph_letter1)&4294967295); //@line 490 "numbers.cpp"
            var $call81=_sprintf($41, ((__str1298)&4294967295), allocate([$arraydecay79,0,0,0,23,0,0,0,6,0,0,0,$arraydecay80,0,0,0], ["i8*",0,0,0,"i32",0,0,0,"i32",0,0,0,"i8*",0,0,0], ALLOC_STACK)); //@line 490 "numbers.cpp"
            ; //@line 490 "numbers.cpp"
          }
        }
      }
    } while(0);
  
    STACKTOP = __stackBase__;
    return; //@line 497 "numbers.cpp"
    return;
  }
  

  function __ZL13LookupLetter2P10TranslatorjPc($tr, $letter, $ph_buf) {
    var __stackBase__  = STACKTOP; STACKTOP += 12; _memset(__stackBase__, 0, 12);
    var __label__;
  
    var $tr_addr;
    var $letter_addr;
    var $ph_buf_addr;
    var $len;
    var $single_letter=__stackBase__;
    $tr_addr=$tr;
    $letter_addr=$letter;
    $ph_buf_addr=$ph_buf;
    var $arrayidx=(($single_letter)&4294967295); //@line 397 "numbers.cpp"
    HEAP8[($arrayidx)]=0; //@line 397 "numbers.cpp"
    var $arrayidx1=(($single_letter+1)&4294967295); //@line 398 "numbers.cpp"
    HEAP8[($arrayidx1)]=95; //@line 398 "numbers.cpp"
    var $0=$letter_addr; //@line 399 "numbers.cpp"
    var $arrayidx2=(($single_letter+2)&4294967295); //@line 399 "numbers.cpp"
    var $call=__Z8utf8_outjPc($0, $arrayidx2); //@line 399 "numbers.cpp"
    $len=$call; //@line 399 "numbers.cpp"
    var $1=$len; //@line 400 "numbers.cpp"
    var $add=((($1)+2)&4294967295); //@line 400 "numbers.cpp"
    var $arrayidx3=(($single_letter+$add)&4294967295); //@line 400 "numbers.cpp"
    HEAP8[($arrayidx3)]=32; //@line 400 "numbers.cpp"
    var $2=$len; //@line 401 "numbers.cpp"
    var $add4=((($2)+3)&4294967295); //@line 401 "numbers.cpp"
    var $arrayidx5=(($single_letter+$add4)&4294967295); //@line 401 "numbers.cpp"
    HEAP8[($arrayidx5)]=0; //@line 401 "numbers.cpp"
    var $3=$tr_addr; //@line 403 "numbers.cpp"
    var $arrayidx6=(($single_letter+1)&4294967295); //@line 403 "numbers.cpp"
    var $4=$ph_buf_addr; //@line 403 "numbers.cpp"
    var $call7=__Z6LookupP10TranslatorPKcPc($3, $arrayidx6, $4); //@line 403 "numbers.cpp"
    var $cmp=((($call7))|0)==0; //@line 403 "numbers.cpp"
    if ($cmp) { __label__ = 1;; } else { __label__ = 3;; } //@line 403 "numbers.cpp"
    $if_then$$if_end15$2: do { 
      if (__label__ == 1) {
  
        var $arrayidx8=(($single_letter+1)&4294967295); //@line 405 "numbers.cpp"
        HEAP8[($arrayidx8)]=32; //@line 405 "numbers.cpp"
        var $5=$tr_addr; //@line 406 "numbers.cpp"
        var $arrayidx9=(($single_letter+2)&4294967295); //@line 406 "numbers.cpp"
        var $6=$ph_buf_addr; //@line 406 "numbers.cpp"
        var $call10=__Z6LookupP10TranslatorPKcPc($5, $arrayidx9, $6); //@line 406 "numbers.cpp"
        var $cmp11=((($call10))|0)==0; //@line 406 "numbers.cpp"
        if (!($cmp11)) { __label__ = 3;break $if_then$$if_end15$2; } //@line 406 "numbers.cpp"
  
        var $7=$tr_addr; //@line 408 "numbers.cpp"
        var $arrayidx13=(($single_letter+2)&4294967295); //@line 408 "numbers.cpp"
        var $8=$ph_buf_addr; //@line 408 "numbers.cpp"
        var $call14=__Z14TranslateRulesP10TranslatorPcS1_iS1_iPj($7, $arrayidx13, $8, 20, 0, 0, 0); //@line 408 "numbers.cpp"
        ; //@line 409 "numbers.cpp"
      }
    } while(0);
  
    var $9=$ph_buf_addr; //@line 411 "numbers.cpp"
    var $arrayidx16=(($9)&4294967295); //@line 411 "numbers.cpp"
    var $10=HEAP8[($arrayidx16)]; //@line 411 "numbers.cpp"
    var $conv=(tempInt=(($10)),(tempInt>=128?tempInt-256:tempInt)); //@line 411 "numbers.cpp"
    STACKTOP = __stackBase__;
    return $conv; //@line 411 "numbers.cpp"
    return null;
  }
  

  function __Z12LookupLetterP10TranslatorjiPci($tr, $letter, $next_byte, $ph_buf1, $control) {
    var __stackBase__  = STACKTOP; STACKTOP += 48; _memset(__stackBase__, 0, 48);
    var __label__;
  
    var $tr_addr;
    var $letter_addr;
    var $next_byte_addr;
    var $ph_buf1_addr;
    var $control_addr;
    var $len;
    var $dict_flags=__stackBase__;
    var $ph_buf3=__stackBase__+8;
    var $ptr;
    $tr_addr=$tr;
    $letter_addr=$letter;
    $next_byte_addr=$next_byte;
    $ph_buf1_addr=$ph_buf1;
    $control_addr=$control;
    var $0=$ph_buf1_addr; //@line 511 "numbers.cpp"
    var $arrayidx=(($0)&4294967295); //@line 511 "numbers.cpp"
    HEAP8[($arrayidx)]=0; //@line 511 "numbers.cpp"
    var $1=$letter_addr; //@line 512 "numbers.cpp"
    var $call=__Z8utf8_outjPc($1, ((__ZZ12LookupLetterP10TranslatorjiPciE13single_letter+2)&4294967295)); //@line 512 "numbers.cpp"
    $len=$call; //@line 512 "numbers.cpp"
    var $2=$len; //@line 513 "numbers.cpp"
    var $add=((($2)+2)&4294967295); //@line 513 "numbers.cpp"
    var $arrayidx1=((__ZZ12LookupLetterP10TranslatorjiPciE13single_letter+$add)&4294967295); //@line 513 "numbers.cpp"
    HEAP8[($arrayidx1)]=32; //@line 513 "numbers.cpp"
    var $3=$next_byte_addr; //@line 515 "numbers.cpp"
    var $cmp=((($3))|0)==-1; //@line 515 "numbers.cpp"
    if ($cmp) { __label__ = 1;; } else { __label__ = 7;; } //@line 515 "numbers.cpp"
    $if_then$$if_end19$2: do { 
      if (__label__ == 1) {
  
        var $4=$tr_addr; //@line 518 "numbers.cpp"
        var $5=$ph_buf1_addr; //@line 518 "numbers.cpp"
        var $call2=__Z6LookupP10TranslatorPKcPc($4, ((__ZZ12LookupLetterP10TranslatorjiPciE13single_letter+2)&4294967295), $5); //@line 518 "numbers.cpp"
        var $cmp3=((($call2))|0)!=0; //@line 518 "numbers.cpp"
        if ($cmp3) { __label__ = 20;break $if_then$$if_end19$2; } //@line 518 "numbers.cpp"
  
        HEAP8[(((__ZZ12LookupLetterP10TranslatorjiPciE13single_letter+1)&4294967295))]=95; //@line 521 "numbers.cpp"
        var $6=$tr_addr; //@line 522 "numbers.cpp"
        var $arraydecay=(($ph_buf3)&4294967295); //@line 522 "numbers.cpp"
        var $call5=__Z6LookupP10TranslatorPKcPc($6, ((__ZZ12LookupLetterP10TranslatorjiPciE13single_letter+1)&4294967295), $arraydecay); //@line 522 "numbers.cpp"
        var $cmp6=((($call5))|0)!=0; //@line 522 "numbers.cpp"
        if ($cmp6) { __label__ = 20;break $if_then$$if_end19$2; } //@line 522 "numbers.cpp"
  
        var $7=$tr_addr; //@line 526 "numbers.cpp"
        var $translator_name=(($7+280)&4294967295); //@line 526 "numbers.cpp"
        var $8=HEAP32[(($translator_name)>>2)]; //@line 526 "numbers.cpp"
        var $cmp9=((($8))|0)==25966; //@line 526 "numbers.cpp"
        if ($cmp9) { __label__ = 20;break $if_then$$if_end19$2; } //@line 526 "numbers.cpp"
  
        var $call12=__Z14SetTranslator2PKc(((__str52704)&4294967295)); //@line 529 "numbers.cpp"
        var $9=HEAP32[((_translator2)>>2)]; //@line 530 "numbers.cpp"
        var $arraydecay13=(($ph_buf3)&4294967295); //@line 530 "numbers.cpp"
        var $call14=__Z6LookupP10TranslatorPKcPc($9, ((__ZZ12LookupLetterP10TranslatorjiPciE13single_letter+2)&4294967295), $arraydecay13); //@line 530 "numbers.cpp"
        var $cmp15=((($call14))|0)!=0; //@line 530 "numbers.cpp"
        if ($cmp15) { __label__ = 5;; } else { __label__ = 6;; } //@line 530 "numbers.cpp"
        if (__label__ == 5) {
  
          var $10=$ph_buf1_addr; //@line 533 "numbers.cpp"
          HEAP8[($10)]=21; //@line 533 "numbers.cpp"
          var $nul=(($10+1)&4294967295); //@line 533 "numbers.cpp"
          HEAP8[($nul)]=0; //@line 533 "numbers.cpp"
          ; //@line 534 "numbers.cpp"
        }
  
        var $11=HEAP32[((_voice)>>2)]; //@line 535 "numbers.cpp"
        var $phoneme_tab_ix=(($11+40)&4294967295); //@line 535 "numbers.cpp"
        var $12=HEAP32[(($phoneme_tab_ix)>>2)]; //@line 535 "numbers.cpp"
        __Z18SelectPhonemeTablei($12); //@line 535 "numbers.cpp"
        ; //@line 536 "numbers.cpp"
      }
      else if (__label__ == 7) {
  
        var $13=$letter_addr; //@line 539 "numbers.cpp"
        var $cmp20=((($13))>>>0) <= 32; //@line 539 "numbers.cpp"
        if ($cmp20) { __label__ = 9;; } else { __label__ = 8;; } //@line 539 "numbers.cpp"
        $if_then22$$lor_lhs_false$11: do { 
          if (__label__ == 8) {
  
            var $14=$letter_addr; //@line 539 "numbers.cpp"
            var $call21=_iswspace($14); //@line 539 "numbers.cpp"
            var $tobool=((($call21))|0)!=0; //@line 539 "numbers.cpp"
            if ($tobool) { __label__ = 9;break $if_then22$$lor_lhs_false$11; } //@line 539 "numbers.cpp"
  
            var $18=$next_byte_addr; //@line 547 "numbers.cpp"
            var $cmp26=((($18))|0)!=32; //@line 547 "numbers.cpp"
            if ($cmp26) { __label__ = 11;; } else { __label__ = 12;; } //@line 547 "numbers.cpp"
            if (__label__ == 11) {
  
              $next_byte_addr=31; //@line 548 "numbers.cpp"
              ; //@line 548 "numbers.cpp"
            }
  
            var $19=$next_byte_addr; //@line 549 "numbers.cpp"
            var $conv=((($19)) & 255); //@line 549 "numbers.cpp"
            var $20=$len; //@line 549 "numbers.cpp"
            var $add29=((($20)+3)&4294967295); //@line 549 "numbers.cpp"
            var $arrayidx30=((__ZZ12LookupLetterP10TranslatorjiPciE13single_letter+$add29)&4294967295); //@line 549 "numbers.cpp"
            HEAP8[($arrayidx30)]=$conv; //@line 549 "numbers.cpp"
            HEAP8[(((__ZZ12LookupLetterP10TranslatorjiPciE13single_letter+1)&4294967295))]=95; //@line 551 "numbers.cpp"
            var $arrayidx31=(($dict_flags+4)&4294967295); //@line 554 "numbers.cpp"
            HEAP32[(($arrayidx31)>>2)]=0; //@line 554 "numbers.cpp"
            $ptr=((__ZZ12LookupLetterP10TranslatorjiPciE13single_letter+1)&4294967295); //@line 555 "numbers.cpp"
            var $21=$tr_addr; //@line 557 "numbers.cpp"
            var $arraydecay32=(($ph_buf3)&4294967295); //@line 557 "numbers.cpp"
            var $call33=__Z6LookupP10TranslatorPKcPc($21, ((__ZZ12LookupLetterP10TranslatorjiPciE13single_letter+1)&4294967295), $arraydecay32); //@line 557 "numbers.cpp"
            var $cmp34=((($call33))|0)==0; //@line 557 "numbers.cpp"
            if ($cmp34) { __label__ = 13;; } else { __label__ = 15;; } //@line 557 "numbers.cpp"
            $if_then35$$if_end43$17: do { 
              if (__label__ == 13) {
  
                HEAP8[(((__ZZ12LookupLetterP10TranslatorjiPciE13single_letter+1)&4294967295))]=32; //@line 559 "numbers.cpp"
                var $22=$tr_addr; //@line 560 "numbers.cpp"
                var $arraydecay36=(($ph_buf3)&4294967295); //@line 560 "numbers.cpp"
                var $call37=__Z6LookupP10TranslatorPKcPc($22, ((__ZZ12LookupLetterP10TranslatorjiPciE13single_letter+2)&4294967295), $arraydecay36); //@line 560 "numbers.cpp"
                var $cmp38=((($call37))|0)==0; //@line 560 "numbers.cpp"
                if (!($cmp38)) { __label__ = 15;break $if_then35$$if_end43$17; } //@line 560 "numbers.cpp"
  
                var $23=$tr_addr; //@line 562 "numbers.cpp"
                var $arraydecay40=(($ph_buf3)&4294967295); //@line 562 "numbers.cpp"
                var $call41=__Z14TranslateRulesP10TranslatorPcS1_iS1_iPj($23, ((__ZZ12LookupLetterP10TranslatorjiPciE13single_letter+2)&4294967295), $arraydecay40, 40, 0, 268435456, 0); //@line 562 "numbers.cpp"
                ; //@line 563 "numbers.cpp"
              }
            } while(0);
  
            var $arrayidx44=(($ph_buf3)&4294967295); //@line 566 "numbers.cpp"
            var $24=HEAP8[($arrayidx44)]; //@line 566 "numbers.cpp"
            var $conv45=(tempInt=(($24)),(tempInt>=128?tempInt-256:tempInt)); //@line 566 "numbers.cpp"
            var $cmp46=((($conv45))|0)==0; //@line 566 "numbers.cpp"
            if ($cmp46) { __label__ = 16;; } else { __label__ = 17;; } //@line 566 "numbers.cpp"
            if (__label__ == 16) {
  
              var $25=$tr_addr; //@line 568 "numbers.cpp"
              var $26=$letter_addr; //@line 568 "numbers.cpp"
              var $arraydecay48=(($ph_buf3)&4294967295); //@line 568 "numbers.cpp"
              __Z20LookupAccentedLetterP10TranslatorjPc($25, $26, $arraydecay48); //@line 568 "numbers.cpp"
              ; //@line 569 "numbers.cpp"
            }
  
            var $27=$ph_buf1_addr; //@line 571 "numbers.cpp"
            var $arraydecay50=(($ph_buf3)&4294967295); //@line 571 "numbers.cpp"
            var $call51=_strcpy($27, $arraydecay50); //@line 571 "numbers.cpp"
            var $28=$ph_buf1_addr; //@line 572 "numbers.cpp"
            var $arrayidx52=(($28)&4294967295); //@line 572 "numbers.cpp"
            var $29=HEAP8[($arrayidx52)]; //@line 572 "numbers.cpp"
            var $conv53=(tempInt=(($29)),(tempInt>=128?tempInt-256:tempInt)); //@line 572 "numbers.cpp"
            var $cmp54=((($conv53))|0)==0; //@line 572 "numbers.cpp"
            if ($cmp54) { __label__ = 20;break $if_then$$if_end19$2; } //@line 572 "numbers.cpp"
  
            var $30=$ph_buf1_addr; //@line 572 "numbers.cpp"
            var $arrayidx56=(($30)&4294967295); //@line 572 "numbers.cpp"
            var $31=HEAP8[($arrayidx56)]; //@line 572 "numbers.cpp"
            var $conv57=(tempInt=(($31)),(tempInt>=128?tempInt-256:tempInt)); //@line 572 "numbers.cpp"
            var $cmp58=((($conv57))|0)==21; //@line 572 "numbers.cpp"
            if ($cmp58) { __label__ = 20;break $if_then$$if_end19$2; } //@line 572 "numbers.cpp"
  
            var $arrayidx61=(($dict_flags)&4294967295); //@line 577 "numbers.cpp"
            HEAP32[(($arrayidx61)>>2)]=0; //@line 577 "numbers.cpp"
            var $arrayidx62=(($dict_flags+4)&4294967295); //@line 578 "numbers.cpp"
            HEAP32[(($arrayidx62)>>2)]=0; //@line 578 "numbers.cpp"
            var $32=$tr_addr; //@line 579 "numbers.cpp"
            var $33=$ph_buf1_addr; //@line 579 "numbers.cpp"
            var $arraydecay63=(($dict_flags)&4294967295); //@line 579 "numbers.cpp"
            var $34=$control_addr; //@line 579 "numbers.cpp"
            var $and=($34) & 1; //@line 579 "numbers.cpp"
            __Z13SetWordStressP10TranslatorPcPjii($32, $33, $arraydecay63, -1, $and); //@line 579 "numbers.cpp"
            __label__ = 20;break $if_then$$if_end19$2; //@line 581 "numbers.cpp"
          }
        } while(0);
  
        var $15=$letter_addr; //@line 542 "numbers.cpp"
        var $call23=_sprintf(((__ZZ12LookupLetterP10TranslatorjiPciE13single_letter+1)&4294967295), ((__str5302)&4294967295), allocate([$15,0,0,0], ["i32",0,0,0], ALLOC_STACK)); //@line 542 "numbers.cpp"
        var $16=$tr_addr; //@line 543 "numbers.cpp"
        var $17=$ph_buf1_addr; //@line 543 "numbers.cpp"
        var $call24=__Z6LookupP10TranslatorPKcPc($16, ((__ZZ12LookupLetterP10TranslatorjiPciE13single_letter+1)&4294967295), $17); //@line 543 "numbers.cpp"
        ; //@line 544 "numbers.cpp"
      }
    } while(0);
  
    STACKTOP = __stackBase__;
    return; //@line 581 "numbers.cpp"
    return;
  }
  

  function __Z15TranslateLetterP10TranslatorPcS1_i($tr, $word, $phonemes, $control) {
    var __stackBase__  = STACKTOP; STACKTOP += 152; _memset(__stackBase__, 0, 152);
    var __label__;
  
    var $retval;
    var $tr_addr;
    var $word_addr;
    var $phonemes_addr;
    var $control_addr;
    var $n_bytes;
    var $letter=__stackBase__;
    var $len;
    var $save_option_phonemes;
    var $p2;
    var $pbuf;
    var $capital=__stackBase__+4;
    var $ph_buf=__stackBase__+24;
    var $ph_buf2=__stackBase__+84;
    var $hexbuf=__stackBase__+144;
    $tr_addr=$tr;
    $word_addr=$word;
    $phonemes_addr=$phonemes;
    $control_addr=$control;
    var $arrayidx=(($ph_buf)&4294967295); //@line 603 "numbers.cpp"
    HEAP8[($arrayidx)]=0; //@line 603 "numbers.cpp"
    var $arrayidx1=(($capital)&4294967295); //@line 604 "numbers.cpp"
    HEAP8[($arrayidx1)]=0; //@line 604 "numbers.cpp"
    var $0=$word_addr; //@line 606 "numbers.cpp"
    var $call=__Z7utf8_inPiPKc($letter, $0); //@line 606 "numbers.cpp"
    $n_bytes=$call; //@line 606 "numbers.cpp"
    var $1=HEAP32[(($letter)>>2)]; //@line 608 "numbers.cpp"
    var $and=($1) & 1048320; //@line 608 "numbers.cpp"
    var $cmp=((($and))|0)==57344; //@line 608 "numbers.cpp"
    if ($cmp) { __label__ = 1;; } else { __label__ = 2;; } //@line 608 "numbers.cpp"
    if (__label__ == 1) {
  
      var $2=HEAP32[(($letter)>>2)]; //@line 610 "numbers.cpp"
      var $and2=($2) & 255; //@line 610 "numbers.cpp"
      HEAP32[(($letter)>>2)]=$and2; //@line 610 "numbers.cpp"
      ; //@line 611 "numbers.cpp"
    }
  
    var $3=$control_addr; //@line 613 "numbers.cpp"
    var $and3=($3) & 2; //@line 613 "numbers.cpp"
    var $tobool=((($and3))|0)!=0; //@line 613 "numbers.cpp"
    if ($tobool) { __label__ = 3;; } else { __label__ = 5;; } //@line 613 "numbers.cpp"
    $if_then4$$if_end10$5: do { 
      if (__label__ == 3) {
  
        var $4=HEAP32[(($letter)>>2)]; //@line 616 "numbers.cpp"
        var $call5=_iswupper($4); //@line 616 "numbers.cpp"
        var $tobool6=((($call5))|0)!=0; //@line 616 "numbers.cpp"
        if (!($tobool6)) { __label__ = 5;break $if_then4$$if_end10$5; } //@line 616 "numbers.cpp"
  
        var $5=$tr_addr; //@line 618 "numbers.cpp"
        var $arraydecay=(($capital)&4294967295); //@line 618 "numbers.cpp"
        var $call8=__Z6LookupP10TranslatorPKcPc($5, ((__str6303)&4294967295), $arraydecay); //@line 618 "numbers.cpp"
        ; //@line 619 "numbers.cpp"
      }
    } while(0);
  
    var $6=HEAP32[(($letter)>>2)]; //@line 621 "numbers.cpp"
    var $call11=__Z9towlower2j($6); //@line 621 "numbers.cpp"
    HEAP32[(($letter)>>2)]=$call11; //@line 621 "numbers.cpp"
    var $7=$tr_addr; //@line 623 "numbers.cpp"
    var $8=HEAP32[(($letter)>>2)]; //@line 623 "numbers.cpp"
    var $9=$n_bytes; //@line 623 "numbers.cpp"
    var $10=$word_addr; //@line 623 "numbers.cpp"
    var $arrayidx12=(($10+$9)&4294967295); //@line 623 "numbers.cpp"
    var $11=HEAP8[($arrayidx12)]; //@line 623 "numbers.cpp"
    var $conv=(tempInt=(($11)),(tempInt>=128?tempInt-256:tempInt)); //@line 623 "numbers.cpp"
    var $arraydecay13=(($ph_buf)&4294967295); //@line 623 "numbers.cpp"
    var $12=$control_addr; //@line 623 "numbers.cpp"
    var $and14=($12) & 1; //@line 623 "numbers.cpp"
    __Z12LookupLetterP10TranslatorjiPci($7, $8, $conv, $arraydecay13, $and14); //@line 623 "numbers.cpp"
    var $arrayidx15=(($ph_buf)&4294967295); //@line 625 "numbers.cpp"
    var $13=HEAP8[($arrayidx15)]; //@line 625 "numbers.cpp"
    var $conv16=(tempInt=(($13)),(tempInt>=128?tempInt-256:tempInt)); //@line 625 "numbers.cpp"
    var $cmp17=((($conv16))|0)==21; //@line 625 "numbers.cpp"
    if ($cmp17) { __label__ = 6;; } else { __label__ = 7;; } //@line 625 "numbers.cpp"
    $if_then18$$if_end21$9: do { 
      if (__label__ == 6) {
  
        var $14=$phonemes_addr; //@line 627 "numbers.cpp"
        var $arraydecay19=(($ph_buf)&4294967295); //@line 627 "numbers.cpp"
        var $call20=_strcpy($14, $arraydecay19); //@line 627 "numbers.cpp"
        $retval=0; //@line 628 "numbers.cpp"
        ; //@line 628 "numbers.cpp"
      }
      else if (__label__ == 7) {
  
        var $arrayidx22=(($ph_buf)&4294967295); //@line 631 "numbers.cpp"
        var $15=HEAP8[($arrayidx22)]; //@line 631 "numbers.cpp"
        var $conv23=(tempInt=(($15)),(tempInt>=128?tempInt-256:tempInt)); //@line 631 "numbers.cpp"
        var $cmp24=((($conv23))|0)==0; //@line 631 "numbers.cpp"
        if ($cmp24) { __label__ = 8;; } else { __label__ = 11;; } //@line 631 "numbers.cpp"
        $land_lhs_true$$if_end38$12: do { 
          if (__label__ == 8) {
  
            var $16=$tr_addr; //@line 631 "numbers.cpp"
            var $translator_name=(($16+280)&4294967295); //@line 631 "numbers.cpp"
            var $17=HEAP32[(($translator_name)>>2)]; //@line 631 "numbers.cpp"
            var $cmp25=((($17))|0)!=25966; //@line 631 "numbers.cpp"
            if (!($cmp25)) { __label__ = 11;break $land_lhs_true$$if_end38$12; } //@line 631 "numbers.cpp"
  
            var $call27=__Z14SetTranslator2PKc(((__str52704)&4294967295)); //@line 634 "numbers.cpp"
            var $18=HEAP32[((_option_phonemes)>>2)]; //@line 635 "numbers.cpp"
            $save_option_phonemes=$18; //@line 635 "numbers.cpp"
            HEAP32[((_option_phonemes)>>2)]=0; //@line 636 "numbers.cpp"
            var $19=HEAP32[((_translator2)>>2)]; //@line 637 "numbers.cpp"
            var $20=HEAP32[(($letter)>>2)]; //@line 637 "numbers.cpp"
            var $21=$n_bytes; //@line 637 "numbers.cpp"
            var $22=$word_addr; //@line 637 "numbers.cpp"
            var $arrayidx28=(($22+$21)&4294967295); //@line 637 "numbers.cpp"
            var $23=HEAP8[($arrayidx28)]; //@line 637 "numbers.cpp"
            var $conv29=(tempInt=(($23)),(tempInt>=128?tempInt-256:tempInt)); //@line 637 "numbers.cpp"
            var $arraydecay30=(($ph_buf)&4294967295); //@line 637 "numbers.cpp"
            var $24=$control_addr; //@line 637 "numbers.cpp"
            var $and31=($24) & 1; //@line 637 "numbers.cpp"
            __Z12LookupLetterP10TranslatorjiPci($19, $20, $conv29, $arraydecay30, $and31); //@line 637 "numbers.cpp"
            var $25=HEAP32[((_voice)>>2)]; //@line 638 "numbers.cpp"
            var $phoneme_tab_ix=(($25+40)&4294967295); //@line 638 "numbers.cpp"
            var $26=HEAP32[(($phoneme_tab_ix)>>2)]; //@line 638 "numbers.cpp"
            __Z18SelectPhonemeTablei($26); //@line 638 "numbers.cpp"
            var $27=$save_option_phonemes; //@line 639 "numbers.cpp"
            HEAP32[((_option_phonemes)>>2)]=$27; //@line 639 "numbers.cpp"
            var $arrayidx32=(($ph_buf)&4294967295); //@line 641 "numbers.cpp"
            var $28=HEAP8[($arrayidx32)]; //@line 641 "numbers.cpp"
            var $conv33=(tempInt=(($28)),(tempInt>=128?tempInt-256:tempInt)); //@line 641 "numbers.cpp"
            var $cmp34=((($conv33))|0)!=0; //@line 641 "numbers.cpp"
            if (!($cmp34)) { __label__ = 11;break $land_lhs_true$$if_end38$12; } //@line 641 "numbers.cpp"
  
            var $29=$phonemes_addr; //@line 643 "numbers.cpp"
            var $call36=_sprintf($29, ((__str7304)&4294967295), allocate([21,0,0,0], ["i32",0,0,0], ALLOC_STACK)); //@line 643 "numbers.cpp"
            $retval=0; //@line 644 "numbers.cpp"
            __label__ = 26;break $if_then18$$if_end21$9; //@line 644 "numbers.cpp"
          }
        } while(0);
  
        var $arrayidx39=(($ph_buf)&4294967295); //@line 648 "numbers.cpp"
        var $30=HEAP8[($arrayidx39)]; //@line 648 "numbers.cpp"
        var $conv40=(tempInt=(($30)),(tempInt>=128?tempInt-256:tempInt)); //@line 648 "numbers.cpp"
        var $cmp41=((($conv40))|0)==0; //@line 648 "numbers.cpp"
        if ($cmp41) { __label__ = 12;; } else { __label__ = 20;; } //@line 648 "numbers.cpp"
        $if_then42$$if_end73$17: do { 
          if (__label__ == 12) {
  
            var $31=HEAP32[(($letter)>>2)]; //@line 651 "numbers.cpp"
            var $call43=_iswalpha($31); //@line 651 "numbers.cpp"
            var $tobool44=((($call43))|0)!=0; //@line 651 "numbers.cpp"
            if ($tobool44) { __label__ = 13;; } else { __label__ = 14;; } //@line 651 "numbers.cpp"
            if (__label__ == 13) {
  
              var $32=$tr_addr; //@line 652 "numbers.cpp"
              var $arraydecay46=(($ph_buf)&4294967295); //@line 652 "numbers.cpp"
              var $call47=__Z6LookupP10TranslatorPKcPc($32, ((__str8305)&4294967295), $arraydecay46); //@line 652 "numbers.cpp"
              ; //@line 652 "numbers.cpp"
            }
  
            var $arrayidx49=(($ph_buf)&4294967295); //@line 654 "numbers.cpp"
            var $33=HEAP8[($arrayidx49)]; //@line 654 "numbers.cpp"
            var $conv50=(tempInt=(($33)),(tempInt>=128?tempInt-256:tempInt)); //@line 654 "numbers.cpp"
            var $cmp51=((($conv50))|0)==0; //@line 654 "numbers.cpp"
            if ($cmp51) { __label__ = 15;; } else { __label__ = 17;; } //@line 654 "numbers.cpp"
            $land_lhs_true52$$if_end58$22: do { 
              if (__label__ == 15) {
  
                var $34=HEAP32[(($letter)>>2)]; //@line 654 "numbers.cpp"
                var $call53=_iswspace($34); //@line 654 "numbers.cpp"
                var $tobool54=((($call53))|0)!=0; //@line 654 "numbers.cpp"
                if ($tobool54) { __label__ = 17;break $land_lhs_true52$$if_end58$22; } //@line 654 "numbers.cpp"
  
                var $35=$tr_addr; //@line 655 "numbers.cpp"
                var $arraydecay56=(($ph_buf)&4294967295); //@line 655 "numbers.cpp"
                var $call57=__Z6LookupP10TranslatorPKcPc($35, ((__str9306)&4294967295), $arraydecay56); //@line 655 "numbers.cpp"
                ; //@line 655 "numbers.cpp"
              }
            } while(0);
  
            var $arrayidx59=(($ph_buf)&4294967295); //@line 657 "numbers.cpp"
            var $36=HEAP8[($arrayidx59)]; //@line 657 "numbers.cpp"
            var $conv60=(tempInt=(($36)),(tempInt>=128?tempInt-256:tempInt)); //@line 657 "numbers.cpp"
            var $cmp61=((($conv60))|0)!=0; //@line 657 "numbers.cpp"
            if (!($cmp61)) { __label__ = 20;break $if_then42$$if_end73$17; } //@line 657 "numbers.cpp"
  
            var $arraydecay63=(($hexbuf)&4294967295); //@line 660 "numbers.cpp"
            var $37=HEAP32[(($letter)>>2)]; //@line 660 "numbers.cpp"
            var $call64=_sprintf($arraydecay63, ((__str10307)&4294967295), allocate([$37,0,0,0], ["i32",0,0,0], ALLOC_STACK)); //@line 660 "numbers.cpp"
            var $arraydecay65=(($ph_buf)&4294967295); //@line 661 "numbers.cpp"
            $pbuf=$arraydecay65; //@line 661 "numbers.cpp"
            var $arraydecay66=(($hexbuf)&4294967295); //@line 662 "numbers.cpp"
            $p2=$arraydecay66; //@line 662 "numbers.cpp"
            var $38=$p2; //@line 662 "numbers.cpp"
            var $39=HEAP8[($38)]; //@line 662 "numbers.cpp"
            var $conv671=(tempInt=(($39)),(tempInt>=128?tempInt-256:tempInt)); //@line 662 "numbers.cpp"
            var $cmp682=((($conv671))|0)!=0; //@line 662 "numbers.cpp"
            if (!($cmp682)) { __label__ = 20;break $if_then42$$if_end73$17; } //@line 662 "numbers.cpp"
            while(1) { 
  
              var $40=$pbuf; //@line 664 "numbers.cpp"
              var $call69=_strlen($40); //@line 664 "numbers.cpp"
              var $41=$pbuf; //@line 664 "numbers.cpp"
              var $add_ptr=(($41+$call69)&4294967295); //@line 664 "numbers.cpp"
              $pbuf=$add_ptr; //@line 664 "numbers.cpp"
              var $42=$pbuf; //@line 665 "numbers.cpp"
              var $incdec_ptr=(($42+1)&4294967295); //@line 665 "numbers.cpp"
              $pbuf=$incdec_ptr; //@line 665 "numbers.cpp"
              HEAP8[($42)]=23; //@line 665 "numbers.cpp"
              var $43=$tr_addr; //@line 666 "numbers.cpp"
              var $44=$p2; //@line 666 "numbers.cpp"
              var $45=HEAP8[($44)]; //@line 666 "numbers.cpp"
              var $conv70=(tempInt=(($45)),(tempInt>=128?tempInt-256:tempInt)); //@line 666 "numbers.cpp"
              var $46=$pbuf; //@line 666 "numbers.cpp"
              __Z12LookupLetterP10TranslatorjiPci($43, $conv70, 0, $46, 1); //@line 666 "numbers.cpp"
              var $47=$p2; //@line 662 "numbers.cpp"
              var $incdec_ptr71=(($47+1)&4294967295); //@line 662 "numbers.cpp"
              $p2=$incdec_ptr71; //@line 662 "numbers.cpp"
              var $48=$p2; //@line 662 "numbers.cpp"
              var $49=HEAP8[($48)]; //@line 662 "numbers.cpp"
              var $conv67=(tempInt=(($49)),(tempInt>=128?tempInt-256:tempInt)); //@line 662 "numbers.cpp"
              var $cmp68=((($conv67))|0)!=0; //@line 662 "numbers.cpp"
              if ($cmp68) { __label__ = 19;continue ; } else { __label__ = 20;break $if_then42$$if_end73$17; } //@line 662 "numbers.cpp"
            }
          }
        } while(0);
  
        var $50=$phonemes_addr; //@line 671 "numbers.cpp"
        var $call74=_strlen($50); //@line 671 "numbers.cpp"
        $len=$call74; //@line 671 "numbers.cpp"
        var $51=$tr_addr; //@line 672 "numbers.cpp"
        var $langopts=(($51)&4294967295); //@line 672 "numbers.cpp"
        var $accents=(($langopts+220)&4294967295); //@line 672 "numbers.cpp"
        var $52=HEAP32[(($accents)>>2)]; //@line 672 "numbers.cpp"
        var $and75=($52) & 2; //@line 672 "numbers.cpp"
        var $tobool76=((($and75))|0)!=0; //@line 672 "numbers.cpp"
        var $arraydecay78=(($ph_buf2)&4294967295); //@line 673 "numbers.cpp"
        if ($tobool76) { __label__ = 21;; } else { __label__ = 22;; } //@line 672 "numbers.cpp"
        if (__label__ == 21) {
  
          var $arraydecay79=(($ph_buf)&4294967295); //@line 673 "numbers.cpp"
          var $arraydecay80=(($capital)&4294967295); //@line 673 "numbers.cpp"
          var $call81=_sprintf($arraydecay78, ((__str11308)&4294967295), allocate([255,0,0,0,$arraydecay79,0,0,0,$arraydecay80,0,0,0], ["i32",0,0,0,"i8*",0,0,0,"i8*",0,0,0], ALLOC_STACK)); //@line 673 "numbers.cpp"
          ; //@line 673 "numbers.cpp"
        }
        else if (__label__ == 22) {
  
          var $arraydecay83=(($capital)&4294967295); //@line 675 "numbers.cpp"
          var $arraydecay84=(($ph_buf)&4294967295); //@line 675 "numbers.cpp"
          var $call85=_sprintf($arraydecay78, ((__str11308)&4294967295), allocate([255,0,0,0,$arraydecay83,0,0,0,$arraydecay84,0,0,0], ["i32",0,0,0,"i8*",0,0,0,"i8*",0,0,0], ALLOC_STACK)); //@line 675 "numbers.cpp"
          ;
        }
  
        var $53=$len; //@line 676 "numbers.cpp"
        var $arraydecay87=(($ph_buf2)&4294967295); //@line 676 "numbers.cpp"
        var $call88=_strlen($arraydecay87); //@line 676 "numbers.cpp"
        var $add=((($call88)+($53))&4294967295); //@line 676 "numbers.cpp"
        var $cmp89=((($add))>>>0) < 160; //@line 676 "numbers.cpp"
        if ($cmp89) { __label__ = 24;; } else { __label__ = 25;; } //@line 676 "numbers.cpp"
        if (__label__ == 24) {
  
          var $54=$len; //@line 678 "numbers.cpp"
          var $55=$phonemes_addr; //@line 678 "numbers.cpp"
          var $arrayidx91=(($55+$54)&4294967295); //@line 678 "numbers.cpp"
          var $arraydecay92=(($ph_buf2)&4294967295); //@line 678 "numbers.cpp"
          var $call93=_strcpy($arrayidx91, $arraydecay92); //@line 678 "numbers.cpp"
          ; //@line 679 "numbers.cpp"
        }
  
        var $56=$n_bytes; //@line 680 "numbers.cpp"
        $retval=$56; //@line 680 "numbers.cpp"
        ; //@line 680 "numbers.cpp"
      }
    } while(0);
  
    var $57=$retval; //@line 681 "numbers.cpp"
    STACKTOP = __stackBase__;
    return $57; //@line 681 "numbers.cpp"
    return null;
  }
  

  function __Z17SetSpellingStressP10TranslatorPcii($tr, $phonemes, $control, $n_chars) {
    var __stackBase__  = STACKTOP; STACKTOP += 160; _memset(__stackBase__, 0, 160);
    var __label__;
    var __lastLabel__ = null;
  
    var $tr_addr;
    var $phonemes_addr;
    var $control_addr;
    var $n_chars_addr;
    var $ix;
    var $c;
    var $n_stress;
    var $count;
    var $buf=__stackBase__;
    $tr_addr=$tr;
    $phonemes_addr=$phonemes;
    $control_addr=$control;
    $n_chars_addr=$n_chars;
    $n_stress=0; //@line 690 "numbers.cpp"
    $ix=0; //@line 694 "numbers.cpp"
    var $0=$ix; //@line 694 "numbers.cpp"
    var $1=$phonemes_addr; //@line 694 "numbers.cpp"
    var $arrayidx4=(($1+$0)&4294967295); //@line 694 "numbers.cpp"
    var $2=HEAP8[($arrayidx4)]; //@line 694 "numbers.cpp"
    var $conv5=(tempInt=(($2)),(tempInt>=128?tempInt-256:tempInt)); //@line 694 "numbers.cpp"
    $c=$conv5; //@line 694 "numbers.cpp"
    var $cmp6=((($conv5))|0)!=0; //@line 694 "numbers.cpp"
    if ($cmp6) { __lastLabel__ = 0; __label__ = 1;; } else { __lastLabel__ = 0; __label__ = 4;; } //@line 694 "numbers.cpp"
    $for_body$$for_end$2: do { 
      if (__label__ == 1) {
        while(1) { 
  
          var $3=__lastLabel__ == 0 ? $conv5 : ($conv); //@line 696 "numbers.cpp"
          var $cmp1=((($3))|0)==6; //@line 696 "numbers.cpp"
          if ($cmp1) { __label__ = 2;; } else { __label__ = 3;; } //@line 696 "numbers.cpp"
          if (__label__ == 2) {
  
            var $4=$n_stress; //@line 698 "numbers.cpp"
            var $inc=((($4)+1)&4294967295); //@line 698 "numbers.cpp"
            $n_stress=$inc; //@line 698 "numbers.cpp"
            ; //@line 699 "numbers.cpp"
          }
  
          var $5=$c; //@line 700 "numbers.cpp"
          var $conv2=((($5)) & 255); //@line 700 "numbers.cpp"
          var $6=$ix; //@line 700 "numbers.cpp"
          var $arrayidx3=(($buf+$6)&4294967295); //@line 700 "numbers.cpp"
          HEAP8[($arrayidx3)]=$conv2; //@line 700 "numbers.cpp"
          var $7=$ix; //@line 694 "numbers.cpp"
          var $inc4=((($7)+1)&4294967295); //@line 694 "numbers.cpp"
          $ix=$inc4; //@line 694 "numbers.cpp"
          var $8=$ix; //@line 694 "numbers.cpp"
          var $9=$phonemes_addr; //@line 694 "numbers.cpp"
          var $arrayidx=(($9+$8)&4294967295); //@line 694 "numbers.cpp"
          var $10=HEAP8[($arrayidx)]; //@line 694 "numbers.cpp"
          var $conv=(tempInt=(($10)),(tempInt>=128?tempInt-256:tempInt)); //@line 694 "numbers.cpp"
          $c=$conv; //@line 694 "numbers.cpp"
          var $cmp=((($conv))|0)!=0; //@line 694 "numbers.cpp"
          if ($cmp) { __lastLabel__ = 3; __label__ = 1;continue ; } else { __lastLabel__ = 3; __label__ = 4;break $for_body$$for_end$2; } //@line 694 "numbers.cpp"
        }
      }
    } while(0);
  
    var $11=$ix; //@line 702 "numbers.cpp"
    var $arrayidx5=(($buf+$11)&4294967295); //@line 702 "numbers.cpp"
    HEAP8[($arrayidx5)]=0; //@line 702 "numbers.cpp"
    $count=0; //@line 704 "numbers.cpp"
    $ix=0; //@line 705 "numbers.cpp"
    var $12=$ix; //@line 705 "numbers.cpp"
    var $arrayidx71=(($buf+$12)&4294967295); //@line 705 "numbers.cpp"
    var $13=HEAPU8[($arrayidx71)]; //@line 705 "numbers.cpp"
    var $conv82=((($13))&255); //@line 705 "numbers.cpp"
    $c=$conv82; //@line 705 "numbers.cpp"
    var $cmp93=((($conv82))|0)!=0; //@line 705 "numbers.cpp"
    if ($cmp93) { __lastLabel__ = 4; __label__ = 5;; } else { __lastLabel__ = 4; __label__ = 26;; } //@line 705 "numbers.cpp"
    $for_body10$$for_end52$9: do { 
      if (__label__ == 5) {
        $for_body10$10: while(1) { 
  
          var $14=__lastLabel__ == 4 ? $conv82 : ($conv8); //@line 707 "numbers.cpp"
          var $cmp11=((($14))|0)==6; //@line 707 "numbers.cpp"
          if ($cmp11) { __lastLabel__ = 5; __label__ = 6;; } else { __lastLabel__ = 5; __label__ = 15;; } //@line 707 "numbers.cpp"
          $land_lhs_true$$if_else29$12: do { 
            if (__label__ == 6) {
  
              var $15=$n_chars_addr; //@line 707 "numbers.cpp"
              var $cmp12=((($15))|0) > 1; //@line 707 "numbers.cpp"
              if ($cmp12) { __label__ = 7;; } else { __label__ = 14;; } //@line 707 "numbers.cpp"
              if (__label__ == 7) {
  
                var $16=$count; //@line 709 "numbers.cpp"
                var $inc14=((($16)+1)&4294967295); //@line 709 "numbers.cpp"
                $count=$inc14; //@line 709 "numbers.cpp"
                var $17=$tr_addr; //@line 711 "numbers.cpp"
                var $langopts=(($17)&4294967295); //@line 711 "numbers.cpp"
                var $spelling_stress=(($langopts+249)&4294967295); //@line 711 "numbers.cpp"
                var $18=HEAP8[($spelling_stress)]; //@line 711 "numbers.cpp"
                var $conv15=(tempInt=(($18)),(tempInt>=128?tempInt-256:tempInt)); //@line 711 "numbers.cpp"
                var $cmp16=((($conv15))|0)==1; //@line 711 "numbers.cpp"
                var $19=$count; //@line 714 "numbers.cpp"
                if ($cmp16) { __label__ = 8;; } else { __label__ = 10;; } //@line 711 "numbers.cpp"
                if (__label__ == 8) {
  
                  var $cmp18=((($19))|0) > 1; //@line 714 "numbers.cpp"
                  if (!($cmp18)) { __label__ = 24;break $land_lhs_true$$if_else29$12; } //@line 714 "numbers.cpp"
  
                  $c=5; //@line 715 "numbers.cpp"
                  __label__ = 24;break $land_lhs_true$$if_else29$12; //@line 715 "numbers.cpp"
                }
                else if (__label__ == 10) {
  
                  var $20=$n_stress; //@line 719 "numbers.cpp"
                  var $cmp21=((($19))|0)!=((($20))|0); //@line 719 "numbers.cpp"
                  if (!($cmp21)) { __label__ = 24;break $land_lhs_true$$if_else29$12; } //@line 719 "numbers.cpp"
  
                  var $21=$count; //@line 721 "numbers.cpp"
                  var $rem=((($21))|0)%3; //@line 721 "numbers.cpp"
                  var $cmp23=((($rem))|0)!=0; //@line 721 "numbers.cpp"
                  if ($cmp23) { __label__ = 13;; } else { __label__ = 12;; } //@line 721 "numbers.cpp"
                  if (__label__ == 12) {
  
                    var $22=$count; //@line 721 "numbers.cpp"
                    var $23=$n_stress; //@line 721 "numbers.cpp"
                    var $sub=((($23)-1)&4294967295); //@line 721 "numbers.cpp"
                    var $cmp24=((($22))|0)==((($sub))|0); //@line 721 "numbers.cpp"
                    if (!($cmp24)) { __label__ = 24;break $land_lhs_true$$if_else29$12; } //@line 721 "numbers.cpp"
                  }
  
                  $c=5; //@line 722 "numbers.cpp"
                  __label__ = 24;break $land_lhs_true$$if_else29$12; //@line 722 "numbers.cpp"
                }
              }
              else if (__label__ == 14) {
  
                var $_pr=$c; //@line 727 "numbers.cpp"
                __lastLabel__ = 14; __label__ = 15;break $land_lhs_true$$if_else29$12;
              }
            }
          } while(0);
          $if_end48$$if_else29$25: do { 
            if (__label__ == 15) {
  
              var $24=__lastLabel__ == 14 ? $_pr : ($14); //@line 727 "numbers.cpp"
              var $cmp30=((($24))|0)==255; //@line 727 "numbers.cpp"
              if (!($cmp30)) { __label__ = 24;break $if_end48$$if_else29$25; } //@line 727 "numbers.cpp"
  
              var $25=$control_addr; //@line 729 "numbers.cpp"
              var $cmp32=((($25))|0) < 2; //@line 729 "numbers.cpp"
              if ($cmp32) { __label__ = 25;break $if_end48$$if_else29$25; } //@line 729 "numbers.cpp"
  
              var $26=$ix; //@line 729 "numbers.cpp"
              var $cmp34=((($26))|0)==0; //@line 729 "numbers.cpp"
              if ($cmp34) { __label__ = 25;break $if_end48$$if_else29$25; } //@line 729 "numbers.cpp"
  
              var $27=$control_addr; //@line 732 "numbers.cpp"
              var $cmp37=((($27))|0)==4; //@line 732 "numbers.cpp"
              if ($cmp37) { __label__ = 19;; } else { __label__ = 20;; } //@line 732 "numbers.cpp"
              if (__label__ == 19) {
  
                $c=9; //@line 733 "numbers.cpp"
                ; //@line 733 "numbers.cpp"
              }
  
              var $28=$count; //@line 734 "numbers.cpp"
              var $rem40=((($28))|0)%3; //@line 734 "numbers.cpp"
              var $cmp41=((($rem40))|0)==0; //@line 734 "numbers.cpp"
              if ($cmp41) { __label__ = 22;; } else { __label__ = 21;; } //@line 734 "numbers.cpp"
              $if_then44$$lor_lhs_false42$33: do { 
                if (__label__ == 21) {
  
                  var $29=$control_addr; //@line 734 "numbers.cpp"
                  var $cmp43=((($29))|0) > 2; //@line 734 "numbers.cpp"
                  if ($cmp43) { __label__ = 22;break $if_then44$$lor_lhs_false42$33; } //@line 734 "numbers.cpp"
  
                  $c=23; //@line 737 "numbers.cpp"
                  __label__ = 24;break $if_end48$$if_else29$25;
                }
              } while(0);
  
              $c=11; //@line 735 "numbers.cpp"
              __label__ = 24;break $if_end48$$if_else29$25; //@line 735 "numbers.cpp"
            }
          } while(0);
          if (__label__ == 24) {
  
            var $30=$c; //@line 741 "numbers.cpp"
            var $conv49=((($30)) & 255); //@line 741 "numbers.cpp"
            var $31=$phonemes_addr; //@line 741 "numbers.cpp"
            var $incdec_ptr=(($31+1)&4294967295); //@line 741 "numbers.cpp"
            $phonemes_addr=$incdec_ptr; //@line 741 "numbers.cpp"
            HEAP8[($31)]=$conv49; //@line 741 "numbers.cpp"
            ; //@line 742 "numbers.cpp"
          }
  
          var $32=$ix; //@line 705 "numbers.cpp"
          var $inc51=((($32)+1)&4294967295); //@line 705 "numbers.cpp"
          $ix=$inc51; //@line 705 "numbers.cpp"
          var $33=$ix; //@line 705 "numbers.cpp"
          var $arrayidx7=(($buf+$33)&4294967295); //@line 705 "numbers.cpp"
          var $34=HEAPU8[($arrayidx7)]; //@line 705 "numbers.cpp"
          var $conv8=((($34))&255); //@line 705 "numbers.cpp"
          $c=$conv8; //@line 705 "numbers.cpp"
          var $cmp9=((($conv8))|0)!=0; //@line 705 "numbers.cpp"
          if ($cmp9) { __lastLabel__ = 25; __label__ = 5;continue $for_body10$10; } else { __lastLabel__ = 25; __label__ = 26;break $for_body10$$for_end52$9; } //@line 705 "numbers.cpp"
        }
      }
    } while(0);
  
    var $35=$control_addr; //@line 743 "numbers.cpp"
    var $cmp53=((($35))|0) >= 2; //@line 743 "numbers.cpp"
    if ($cmp53) { __label__ = 27;; } else { __label__ = 28;; } //@line 743 "numbers.cpp"
    if (__label__ == 27) {
  
      var $36=$phonemes_addr; //@line 744 "numbers.cpp"
      var $incdec_ptr55=(($36+1)&4294967295); //@line 744 "numbers.cpp"
      $phonemes_addr=$incdec_ptr55; //@line 744 "numbers.cpp"
      HEAP8[($36)]=11; //@line 744 "numbers.cpp"
      ; //@line 744 "numbers.cpp"
    }
  
    var $37=$phonemes_addr; //@line 745 "numbers.cpp"
    HEAP8[($37)]=0; //@line 745 "numbers.cpp"
    STACKTOP = __stackBase__;
    return; //@line 746 "numbers.cpp"
    return;
  }
  

  function __Z14TranslateRomanP10TranslatorPcS1_P8WORD_TAB($tr, $word, $ph_out, $wtab) {
    var __stackBase__  = STACKTOP; STACKTOP += 200; _memset(__stackBase__, 0, 200);
    var __label__;
    var __lastLabel__ = null;
  
    var $retval;
    var $tr_addr;
    var $word_addr;
    var $ph_out_addr;
    var $wtab_addr;
    var $c;
    var $p;
    var $p2;
    var $acc;
    var $prev;
    var $value;
    var $subtract;
    var $repeat;
    var $n_digits;
    var $word_start;
    var $num_control;
    var $flags=__stackBase__;
    var $ph_roman=__stackBase__+8;
    var $number_chars=__stackBase__+40;
    $tr_addr=$tr;
    $word_addr=$word;
    $ph_out_addr=$ph_out;
    $wtab_addr=$wtab;
    $repeat=0; //@line 839 "numbers.cpp"
    $n_digits=0; //@line 840 "numbers.cpp"
    $num_control=0; //@line 842 "numbers.cpp"
    $acc=0; //@line 850 "numbers.cpp"
    $prev=0; //@line 851 "numbers.cpp"
    $subtract=32767; //@line 852 "numbers.cpp"
    var $0=$ph_out_addr; //@line 853 "numbers.cpp"
    var $arrayidx=(($0)&4294967295); //@line 853 "numbers.cpp"
    HEAP8[($arrayidx)]=0; //@line 853 "numbers.cpp"
    var $arrayidx1=(($flags)&4294967295); //@line 854 "numbers.cpp"
    HEAP32[(($arrayidx1)>>2)]=0; //@line 854 "numbers.cpp"
    var $arrayidx2=(($flags+4)&4294967295); //@line 855 "numbers.cpp"
    HEAP32[(($arrayidx2)>>2)]=0; //@line 855 "numbers.cpp"
    var $1=$tr_addr; //@line 857 "numbers.cpp"
    var $langopts=(($1)&4294967295); //@line 857 "numbers.cpp"
    var $numbers=(($langopts+184)&4294967295); //@line 857 "numbers.cpp"
    var $2=HEAP32[(($numbers)>>2)]; //@line 857 "numbers.cpp"
    var $and=($2) & 33554432; //@line 857 "numbers.cpp"
    var $tobool=((($and))|0)!=0; //@line 857 "numbers.cpp"
    if ($tobool) { __label__ = 1;; } else { __label__ = 2;; } //@line 857 "numbers.cpp"
    $land_lhs_true$$lor_lhs_false$2: do { 
      if (__label__ == 1) {
  
        var $3=$wtab_addr; //@line 857 "numbers.cpp"
        var $arrayidx3=(($3)&4294967295); //@line 857 "numbers.cpp"
        var $flags4=(($arrayidx3)&4294967295); //@line 857 "numbers.cpp"
        var $4=HEAP32[(($flags4)>>2)]; //@line 857 "numbers.cpp"
        var $and5=($4) & 1; //@line 857 "numbers.cpp"
        var $tobool6=((($and5))|0)!=0; //@line 857 "numbers.cpp"
        if ($tobool6) { __label__ = 2;break $land_lhs_true$$lor_lhs_false$2; } else { __label__ = 3;break $land_lhs_true$$lor_lhs_false$2; } //@line 857 "numbers.cpp"
      }
    } while(0);
    $lor_lhs_false$$if_then$4: do { 
      if (__label__ == 2) {
  
        var $5=$word_addr; //@line 857 "numbers.cpp"
        var $arrayidx7=(($5+-2)&4294967295); //@line 857 "numbers.cpp"
        var $6=HEAP8[($arrayidx7)]; //@line 857 "numbers.cpp"
        var $conv=(tempInt=(($6)),(tempInt>=128?tempInt-256:tempInt)); //@line 857 "numbers.cpp"
        var $isdigittmp=((($conv)-48)&4294967295); //@line 857 "numbers.cpp"
        var $isdigit=((($isdigittmp))>>>0) < 10; //@line 857 "numbers.cpp"
        var $call=((($isdigit))&1); //@line 857 "numbers.cpp"
        var $tobool8=((($call))|0)!=0; //@line 857 "numbers.cpp"
        if ($tobool8) { __label__ = 3;break $lor_lhs_false$$if_then$4; } //@line 857 "numbers.cpp"
  
        var $7=$word_addr; //@line 860 "numbers.cpp"
        $word_start=$7; //@line 860 "numbers.cpp"
        ; //@line 861 "numbers.cpp"
        $while_cond$7: while(1) { 
  
          var $8=$word_addr; //@line 861 "numbers.cpp"
          var $incdec_ptr=(($8+1)&4294967295); //@line 861 "numbers.cpp"
          $word_addr=$incdec_ptr; //@line 861 "numbers.cpp"
          var $9=HEAP8[($8)]; //@line 861 "numbers.cpp"
          var $conv9=(tempInt=(($9)),(tempInt>=128?tempInt-256:tempInt)); //@line 861 "numbers.cpp"
          $c=$conv9; //@line 861 "numbers.cpp"
          var $cmp=((($conv9))|0)!=32; //@line 861 "numbers.cpp"
          if (!($cmp)) { __label__ = 25;break $while_cond$7; } //@line 861 "numbers.cpp"
  
          var $10=$c; //@line 863 "numbers.cpp"
          var $memchr=_memchr(((__str12309)&4294967295), $10, 8); //@line 863 "numbers.cpp"
          $p2=$memchr; //@line 863 "numbers.cpp"
          var $cmp11=((($memchr))|0)==0; //@line 863 "numbers.cpp"
          if ($cmp11) { __label__ = 7;break $while_cond$7; } //@line 863 "numbers.cpp"
  
          var $11=$p2; //@line 866 "numbers.cpp"
          var $sub_ptr_lhs_cast=($11); //@line 866 "numbers.cpp"
          var $sub_ptr_sub=((($sub_ptr_lhs_cast)-(__str12309))&4294967295); //@line 866 "numbers.cpp"
          var $arrayidx14=((__ZZ14TranslateRomanP10TranslatorPcS1_P8WORD_TABE12roman_values+$sub_ptr_sub*4)&4294967295); //@line 866 "numbers.cpp"
          var $12=HEAP32[(($arrayidx14)>>2)]; //@line 866 "numbers.cpp"
          $value=$12; //@line 866 "numbers.cpp"
          var $13=$value; //@line 867 "numbers.cpp"
          var $14=$prev; //@line 867 "numbers.cpp"
          var $cmp15=((($13))|0)==((($14))|0); //@line 867 "numbers.cpp"
          if ($cmp15) { __label__ = 9;; } else { __label__ = 11;; } //@line 867 "numbers.cpp"
          if (__label__ == 9) {
  
            var $15=$repeat; //@line 869 "numbers.cpp"
            var $inc=((($15)+1)&4294967295); //@line 869 "numbers.cpp"
            $repeat=$inc; //@line 869 "numbers.cpp"
            var $16=$repeat; //@line 870 "numbers.cpp"
            var $cmp17=((($16))|0) >= 3; //@line 870 "numbers.cpp"
            if ($cmp17) { __label__ = 10;break $while_cond$7; } //@line 870 "numbers.cpp"
          }
          else if (__label__ == 11) {
  
            $repeat=0; //@line 874 "numbers.cpp"
            ;
          }
  
          var $17=$prev; //@line 876 "numbers.cpp"
          var $cmp21=((($17))|0) > 1; //@line 876 "numbers.cpp"
          var $18=$prev; //@line 876 "numbers.cpp"
          var $cmp23=((($18))|0)!=10; //@line 876 "numbers.cpp"
          var $or_cond=($cmp21) & ($cmp23); //@line 876 "numbers.cpp"
          var $19=$prev; //@line 876 "numbers.cpp"
          var $cmp25=((($19))|0)!=100; //@line 876 "numbers.cpp"
          var $or_cond1=($or_cond) & ($cmp25); //@line 876 "numbers.cpp"
          if ($or_cond1) { __lastLabel__ = 12; __label__ = 13;; } else { __lastLabel__ = 12; __label__ = 15;; } //@line 876 "numbers.cpp"
          if (__label__ == 13) {
  
            var $20=$value; //@line 878 "numbers.cpp"
            var $21=$prev; //@line 878 "numbers.cpp"
            var $cmp27=((($20))|0) >= ((($21))|0); //@line 878 "numbers.cpp"
            if ($cmp27) { __lastLabel__ = 13; __label__ = 14;break $while_cond$7; } else { __lastLabel__ = 13; ; } //@line 878 "numbers.cpp"
          }
  
          var $22=__lastLabel__ == 13 ? $21 : ($19); //@line 881 "numbers.cpp"
          var $cmp31=((($22))|0)!=0; //@line 881 "numbers.cpp"
          if ($cmp31) { __label__ = 16;; } else { __label__ = 21;; } //@line 881 "numbers.cpp"
          $land_lhs_true32$$if_else40$18: do { 
            if (__label__ == 16) {
  
              var $23=$prev; //@line 881 "numbers.cpp"
              var $24=$value; //@line 881 "numbers.cpp"
              var $cmp33=((($23))|0) < ((($24))|0); //@line 881 "numbers.cpp"
              if (!($cmp33)) { __label__ = 21;break $land_lhs_true32$$if_else40$18; } //@line 881 "numbers.cpp"
  
              var $25=$acc; //@line 883 "numbers.cpp"
              var $rem=((($25))|0)%10; //@line 883 "numbers.cpp"
              var $cmp35=((($rem))|0)!=0; //@line 883 "numbers.cpp"
              if ($cmp35) { __label__ = 19;break $while_cond$7; } //@line 883 "numbers.cpp"
  
              var $26=$prev; //@line 883 "numbers.cpp"
              var $mul=((($26)*10)&4294967295); //@line 883 "numbers.cpp"
              var $27=$value; //@line 883 "numbers.cpp"
              var $cmp37=((($mul))|0) < ((($27))|0); //@line 883 "numbers.cpp"
              if ($cmp37) { __label__ = 19;break $while_cond$7; } //@line 883 "numbers.cpp"
  
              var $28=$prev; //@line 885 "numbers.cpp"
              $subtract=$28; //@line 885 "numbers.cpp"
              var $29=$subtract; //@line 886 "numbers.cpp"
              var $30=$value; //@line 886 "numbers.cpp"
              var $sub=((($30)-($29))&4294967295); //@line 886 "numbers.cpp"
              $value=$sub; //@line 886 "numbers.cpp"
              __label__ = 24;break $land_lhs_true32$$if_else40$18; //@line 887 "numbers.cpp"
            }
          } while(0);
          if (__label__ == 21) {
  
            var $31=$value; //@line 889 "numbers.cpp"
            var $32=$subtract; //@line 889 "numbers.cpp"
            var $cmp41=((($31))|0) >= ((($32))|0); //@line 889 "numbers.cpp"
            if ($cmp41) { __label__ = 22;break $while_cond$7; } //@line 889 "numbers.cpp"
  
            var $33=$prev; //@line 892 "numbers.cpp"
            var $34=$acc; //@line 892 "numbers.cpp"
            var $add=((($34)+($33))&4294967295); //@line 892 "numbers.cpp"
            $acc=$add; //@line 892 "numbers.cpp"
            ;
          }
  
          var $35=$value; //@line 893 "numbers.cpp"
          $prev=$35; //@line 893 "numbers.cpp"
          var $36=$n_digits; //@line 894 "numbers.cpp"
          var $inc46=((($36)+1)&4294967295); //@line 894 "numbers.cpp"
          $n_digits=$inc46; //@line 894 "numbers.cpp"
          __label__ = 5;continue $while_cond$7; //@line 895 "numbers.cpp"
        }
        if (__label__ == 25) {
  
          var $37=$word_addr; //@line 897 "numbers.cpp"
          var $arrayidx47=(($37)&4294967295); //@line 897 "numbers.cpp"
          var $38=HEAP8[($arrayidx47)]; //@line 897 "numbers.cpp"
          var $conv48=(tempInt=(($38)),(tempInt>=128?tempInt-256:tempInt)); //@line 897 "numbers.cpp"
          var $isdigittmp2=((($conv48)-48)&4294967295); //@line 897 "numbers.cpp"
          var $isdigit3=((($isdigittmp2))>>>0) < 10; //@line 897 "numbers.cpp"
          var $call49=((($isdigit3))&1); //@line 897 "numbers.cpp"
          var $tobool50=((($call49))|0)!=0; //@line 897 "numbers.cpp"
          if ($tobool50) { __label__ = 26;; } else { __label__ = 27;; } //@line 897 "numbers.cpp"
          if (__label__ == 26) {
  
            $retval=0; //@line 898 "numbers.cpp"
            __label__ = 48;break $lor_lhs_false$$if_then$4; //@line 898 "numbers.cpp"
          }
          else if (__label__ == 27) {
  
            var $39=$prev; //@line 900 "numbers.cpp"
            var $40=$acc; //@line 900 "numbers.cpp"
            var $add53=((($40)+($39))&4294967295); //@line 900 "numbers.cpp"
            $acc=$add53; //@line 900 "numbers.cpp"
            var $41=$acc; //@line 901 "numbers.cpp"
            var $42=$tr_addr; //@line 901 "numbers.cpp"
            var $langopts54=(($42)&4294967295); //@line 901 "numbers.cpp"
            var $min_roman=(($langopts54+200)&4294967295); //@line 901 "numbers.cpp"
            var $43=HEAP32[(($min_roman)>>2)]; //@line 901 "numbers.cpp"
            var $cmp55=((($41))|0) < ((($43))|0); //@line 901 "numbers.cpp"
            if ($cmp55) { __label__ = 28;; } else { __label__ = 29;; } //@line 901 "numbers.cpp"
            if (__label__ == 28) {
  
              $retval=0; //@line 902 "numbers.cpp"
              __label__ = 48;break $lor_lhs_false$$if_then$4; //@line 902 "numbers.cpp"
            }
            else if (__label__ == 29) {
  
              var $44=$acc; //@line 904 "numbers.cpp"
              var $45=$tr_addr; //@line 904 "numbers.cpp"
              var $langopts58=(($45)&4294967295); //@line 904 "numbers.cpp"
              var $max_roman=(($langopts58+196)&4294967295); //@line 904 "numbers.cpp"
              var $46=HEAP32[(($max_roman)>>2)]; //@line 904 "numbers.cpp"
              var $cmp59=((($44))|0) > ((($46))|0); //@line 904 "numbers.cpp"
              if ($cmp59) { __label__ = 30;; } else { __label__ = 31;; } //@line 904 "numbers.cpp"
              if (__label__ == 30) {
  
                $retval=0; //@line 905 "numbers.cpp"
                __label__ = 48;break $lor_lhs_false$$if_then$4; //@line 905 "numbers.cpp"
              }
              else if (__label__ == 31) {
  
                var $47=$tr_addr; //@line 908 "numbers.cpp"
                var $arraydecay=(($ph_roman)&4294967295); //@line 908 "numbers.cpp"
                var $call62=__Z6LookupP10TranslatorPKcPc($47, ((__str13310)&4294967295), $arraydecay); //@line 908 "numbers.cpp"
                var $48=$ph_out_addr; //@line 909 "numbers.cpp"
                var $arrayidx63=(($48)&4294967295); //@line 909 "numbers.cpp"
                $p=$arrayidx63; //@line 909 "numbers.cpp"
                var $49=$tr_addr; //@line 911 "numbers.cpp"
                var $langopts64=(($49)&4294967295); //@line 911 "numbers.cpp"
                var $numbers65=(($langopts64+184)&4294967295); //@line 911 "numbers.cpp"
                var $50=HEAP32[(($numbers65)>>2)]; //@line 911 "numbers.cpp"
                var $and66=($50) & 67108864; //@line 911 "numbers.cpp"
                var $cmp67=((($and66))|0)==0; //@line 911 "numbers.cpp"
                if ($cmp67) { __label__ = 32;; } else { __label__ = 33;; } //@line 911 "numbers.cpp"
                if (__label__ == 32) {
  
                  var $51=$ph_out_addr; //@line 913 "numbers.cpp"
                  var $arraydecay69=(($ph_roman)&4294967295); //@line 913 "numbers.cpp"
                  var $call70=_strcpy($51, $arraydecay69); //@line 913 "numbers.cpp"
                  var $arraydecay71=(($ph_roman)&4294967295); //@line 914 "numbers.cpp"
                  var $call72=_strlen($arraydecay71); //@line 914 "numbers.cpp"
                  var $52=$ph_out_addr; //@line 914 "numbers.cpp"
                  var $arrayidx73=(($52+$call72)&4294967295); //@line 914 "numbers.cpp"
                  $p=$arrayidx73; //@line 914 "numbers.cpp"
                  ; //@line 915 "numbers.cpp"
                }
  
                var $arraydecay75=(($number_chars)&4294967295); //@line 917 "numbers.cpp"
                var $53=$acc; //@line 917 "numbers.cpp"
                var $call76=_sprintf($arraydecay75, ((__str14311)&4294967295), allocate([$53,0,0,0], ["i32",0,0,0], ALLOC_STACK)); //@line 917 "numbers.cpp"
                var $54=$word_addr; //@line 919 "numbers.cpp"
                var $arrayidx77=(($54)&4294967295); //@line 919 "numbers.cpp"
                var $55=HEAP8[($arrayidx77)]; //@line 919 "numbers.cpp"
                var $conv78=(tempInt=(($55)),(tempInt>=128?tempInt-256:tempInt)); //@line 919 "numbers.cpp"
                var $cmp79=((($conv78))|0)==46; //@line 919 "numbers.cpp"
                if ($cmp79) { __label__ = 34;; } else { __label__ = 35;; } //@line 919 "numbers.cpp"
                if (__label__ == 34) {
  
                  $retval=0; //@line 922 "numbers.cpp"
                  __label__ = 48;break $lor_lhs_false$$if_then$4; //@line 922 "numbers.cpp"
                }
                else if (__label__ == 35) {
  
                  var $56=$tr_addr; //@line 925 "numbers.cpp"
                  var $57=$word_start; //@line 925 "numbers.cpp"
                  var $58=$word_addr; //@line 925 "numbers.cpp"
                  var $59=$wtab_addr; //@line 925 "numbers.cpp"
                  var $call82=__ZL15CheckDotOrdinalP10TranslatorPcS1_P8WORD_TABi($56, $57, $58, $59, 1); //@line 925 "numbers.cpp"
                  var $tobool83=((($call82))|0)!=0; //@line 925 "numbers.cpp"
                  if ($tobool83) { __label__ = 36;; } else { __label__ = 37;; } //@line 925 "numbers.cpp"
                  if (__label__ == 36) {
  
                    var $60=$wtab_addr; //@line 926 "numbers.cpp"
                    var $arrayidx85=(($60)&4294967295); //@line 926 "numbers.cpp"
                    var $flags86=(($arrayidx85)&4294967295); //@line 926 "numbers.cpp"
                    var $61=HEAP32[(($flags86)>>2)]; //@line 926 "numbers.cpp"
                    var $or=($61) | 32768; //@line 926 "numbers.cpp"
                    HEAP32[(($flags86)>>2)]=$or; //@line 926 "numbers.cpp"
                    ; //@line 926 "numbers.cpp"
                  }
  
                  var $62=$tr_addr; //@line 928 "numbers.cpp"
                  var $langopts88=(($62)&4294967295); //@line 928 "numbers.cpp"
                  var $numbers89=(($langopts88+184)&4294967295); //@line 928 "numbers.cpp"
                  var $63=HEAP32[(($numbers89)>>2)]; //@line 928 "numbers.cpp"
                  var $and90=($63) & 134217728; //@line 928 "numbers.cpp"
                  var $tobool91=((($and90))|0)!=0; //@line 928 "numbers.cpp"
                  if ($tobool91) { __label__ = 38;; } else { __label__ = 45;; } //@line 928 "numbers.cpp"
                  $if_then92$$if_end117$47: do { 
                    if (__label__ == 38) {
  
                      var $64=$tr_addr; //@line 930 "numbers.cpp"
                      var $translator_name=(($64+280)&4294967295); //@line 930 "numbers.cpp"
                      var $65=HEAP32[(($translator_name)>>2)]; //@line 930 "numbers.cpp"
                      var $cmp93=((($65))|0)==26741; //@line 930 "numbers.cpp"
                      var $66=$wtab_addr; //@line 932 "numbers.cpp"
                      var $arrayidx95=(($66)&4294967295); //@line 932 "numbers.cpp"
                      var $flags96=(($arrayidx95)&4294967295); //@line 932 "numbers.cpp"
                      var $67=HEAP32[(($flags96)>>2)]; //@line 932 "numbers.cpp"
                      if ($cmp93) { __label__ = 39;; } else { __label__ = 44;; } //@line 930 "numbers.cpp"
                      if (__label__ == 39) {
  
                        var $and97=($67) & 32768; //@line 932 "numbers.cpp"
                        var $tobool98=((($and97))|0)!=0; //@line 932 "numbers.cpp"
                        if ($tobool98) { __label__ = 45;break $if_then92$$if_end117$47; } //@line 932 "numbers.cpp"
  
                        var $68=$wtab_addr; //@line 934 "numbers.cpp"
                        var $arrayidx100=(($68)&4294967295); //@line 934 "numbers.cpp"
                        var $flags101=(($arrayidx100)&4294967295); //@line 934 "numbers.cpp"
                        var $69=HEAP32[(($flags101)>>2)]; //@line 934 "numbers.cpp"
                        var $and102=($69) & 16384; //@line 934 "numbers.cpp"
                        var $tobool103=((($and102))|0)!=0; //@line 934 "numbers.cpp"
                        if ($tobool103) { __label__ = 41;; } else { __label__ = 43;; } //@line 934 "numbers.cpp"
                        $land_lhs_true104$$if_else109$52: do { 
                          if (__label__ == 41) {
  
                            var $70=$word_addr; //@line 934 "numbers.cpp"
                            var $71=$acc; //@line 934 "numbers.cpp"
                            var $call105=__ZL11hu_number_ePKcii($70, 0, $71); //@line 934 "numbers.cpp"
                            var $tobool106=((($call105))|0)!=0; //@line 934 "numbers.cpp"
                            if (!($tobool106)) { __label__ = 43;break $land_lhs_true104$$if_else109$52; } //@line 934 "numbers.cpp"
  
                            var $72=$num_control; //@line 937 "numbers.cpp"
                            var $or108=($72) | 1; //@line 937 "numbers.cpp"
                            $num_control=$or108; //@line 937 "numbers.cpp"
                            __label__ = 45;break $if_then92$$if_end117$47; //@line 941 "numbers.cpp"
                          }
                        } while(0);
  
                        $retval=0; //@line 940 "numbers.cpp"
                        __label__ = 48;break $lor_lhs_false$$if_then$4; //@line 940 "numbers.cpp"
                      }
                      else if (__label__ == 44) {
  
                        var $or115=($67) | 32768; //@line 945 "numbers.cpp"
                        HEAP32[(($flags96)>>2)]=$or115; //@line 945 "numbers.cpp"
                        ;
                      }
                    }
                  } while(0);
  
                  var $73=$tr_addr; //@line 949 "numbers.cpp"
                  var $prev_dict_flags=(($73+8320)&4294967295); //@line 949 "numbers.cpp"
                  HEAP32[(($prev_dict_flags)>>2)]=0; //@line 949 "numbers.cpp"
                  var $74=$tr_addr; //@line 950 "numbers.cpp"
                  var $arrayidx118=(($number_chars+2)&4294967295); //@line 950 "numbers.cpp"
                  var $75=$p; //@line 950 "numbers.cpp"
                  var $arraydecay119=(($flags)&4294967295); //@line 950 "numbers.cpp"
                  var $76=$wtab_addr; //@line 950 "numbers.cpp"
                  var $77=$num_control; //@line 950 "numbers.cpp"
                  var $call120=__Z15TranslateNumberP10TranslatorPcS1_PjP8WORD_TABi($74, $arrayidx118, $75, $arraydecay119, $76, $77); //@line 950 "numbers.cpp"
                  var $78=$tr_addr; //@line 952 "numbers.cpp"
                  var $langopts121=(($78)&4294967295); //@line 952 "numbers.cpp"
                  var $numbers122=(($langopts121+184)&4294967295); //@line 952 "numbers.cpp"
                  var $79=HEAP32[(($numbers122)>>2)]; //@line 952 "numbers.cpp"
                  var $and123=($79) & 67108864; //@line 952 "numbers.cpp"
                  var $tobool124=((($and123))|0)!=0; //@line 952 "numbers.cpp"
                  if ($tobool124) { __label__ = 46;; } else { __label__ = 47;; } //@line 952 "numbers.cpp"
                  if (__label__ == 46) {
  
                    var $80=$ph_out_addr; //@line 953 "numbers.cpp"
                    var $arraydecay126=(($ph_roman)&4294967295); //@line 953 "numbers.cpp"
                    var $call127=_strcat($80, $arraydecay126); //@line 953 "numbers.cpp"
                    ; //@line 953 "numbers.cpp"
                  }
  
                  $retval=1; //@line 955 "numbers.cpp"
                  __label__ = 48;break $lor_lhs_false$$if_then$4; //@line 955 "numbers.cpp"
                }
              }
            }
          }
        }
        else if (__label__ == 7) {
  
          $retval=0; //@line 864 "numbers.cpp"
          __label__ = 48;break $lor_lhs_false$$if_then$4; //@line 864 "numbers.cpp"
        }
        else if (__label__ == 10) {
  
          $retval=0; //@line 871 "numbers.cpp"
          __label__ = 48;break $lor_lhs_false$$if_then$4; //@line 871 "numbers.cpp"
        }
        else if (__label__ == 14) {
  
          $retval=0; //@line 879 "numbers.cpp"
          __label__ = 48;break $lor_lhs_false$$if_then$4; //@line 879 "numbers.cpp"
        }
        else if (__label__ == 19) {
  
          $retval=0; //@line 884 "numbers.cpp"
          __label__ = 48;break $lor_lhs_false$$if_then$4; //@line 884 "numbers.cpp"
        }
        else if (__label__ == 22) {
  
          $retval=0; //@line 890 "numbers.cpp"
          __label__ = 48;break $lor_lhs_false$$if_then$4; //@line 890 "numbers.cpp"
        }
      }
    } while(0);
    if (__label__ == 3) {
  
      $retval=0; //@line 858 "numbers.cpp"
      ; //@line 858 "numbers.cpp"
    }
  
    var $81=$retval; //@line 956 "numbers.cpp"
    STACKTOP = __stackBase__;
    return $81; //@line 956 "numbers.cpp"
    return null;
  }
  

  function __ZL15CheckDotOrdinalP10TranslatorPcS1_P8WORD_TABi($tr, $word, $word_end, $wtab, $roman) {
    var __stackBase__  = STACKTOP; STACKTOP += 4; _memset(__stackBase__, 0, 4);
    var __label__;
  
    var $tr_addr;
    var $word_addr;
    var $word_end_addr;
    var $wtab_addr;
    var $roman_addr;
    var $ordinal;
    var $c2=__stackBase__;
    var $nextflags;
    $tr_addr=$tr;
    $word_addr=$word;
    $word_end_addr=$word_end;
    $wtab_addr=$wtab;
    $roman_addr=$roman;
    $ordinal=0; //@line 758 "numbers.cpp"
    var $0=$tr_addr; //@line 762 "numbers.cpp"
    var $langopts=(($0)&4294967295); //@line 762 "numbers.cpp"
    var $numbers=(($langopts+184)&4294967295); //@line 762 "numbers.cpp"
    var $1=HEAP32[(($numbers)>>2)]; //@line 762 "numbers.cpp"
    var $and=($1) & 65536; //@line 762 "numbers.cpp"
    var $tobool=((($and))|0)!=0; //@line 762 "numbers.cpp"
    if ($tobool) { __label__ = 1;; } else { __label__ = 32;; } //@line 762 "numbers.cpp"
    $land_lhs_true$$if_end92$2: do { 
      if (__label__ == 1) {
  
        var $2=$word_end_addr; //@line 762 "numbers.cpp"
        var $arrayidx=(($2)&4294967295); //@line 762 "numbers.cpp"
        var $3=HEAP8[($arrayidx)]; //@line 762 "numbers.cpp"
        var $conv=(tempInt=(($3)),(tempInt>=128?tempInt-256:tempInt)); //@line 762 "numbers.cpp"
        var $cmp=((($conv))|0)==46; //@line 762 "numbers.cpp"
        if ($cmp) { __label__ = 3;; } else { __label__ = 2;; } //@line 762 "numbers.cpp"
        if (__label__ == 2) {
  
          var $4=$wtab_addr; //@line 762 "numbers.cpp"
          var $arrayidx1=(($4)&4294967295); //@line 762 "numbers.cpp"
          var $flags=(($arrayidx1)&4294967295); //@line 762 "numbers.cpp"
          var $5=HEAP32[(($flags)>>2)]; //@line 762 "numbers.cpp"
          var $and2=($5) & 65536; //@line 762 "numbers.cpp"
          var $tobool3=((($and2))|0)!=0; //@line 762 "numbers.cpp"
          if (!($tobool3)) { __label__ = 32;break $land_lhs_true$$if_end92$2; } //@line 762 "numbers.cpp"
        }
  
        var $6=$wtab_addr; //@line 762 "numbers.cpp"
        var $arrayidx5=(($6+12)&4294967295); //@line 762 "numbers.cpp"
        var $flags6=(($arrayidx5)&4294967295); //@line 762 "numbers.cpp"
        var $7=HEAP32[(($flags6)>>2)]; //@line 762 "numbers.cpp"
        var $and7=($7) & 256; //@line 762 "numbers.cpp"
        var $tobool8=((($and7))|0)!=0; //@line 762 "numbers.cpp"
        if ($tobool8) { __label__ = 32;break $land_lhs_true$$if_end92$2; } //@line 762 "numbers.cpp"
  
        var $8=$roman_addr; //@line 764 "numbers.cpp"
        var $tobool9=((($8))|0)!=0; //@line 764 "numbers.cpp"
        if ($tobool9) { __label__ = 6;; } else { __label__ = 5;; } //@line 764 "numbers.cpp"
        if (__label__ == 5) {
  
          var $9=$wtab_addr; //@line 764 "numbers.cpp"
          var $arrayidx11=(($9+12)&4294967295); //@line 764 "numbers.cpp"
          var $flags12=(($arrayidx11)&4294967295); //@line 764 "numbers.cpp"
          var $10=HEAP32[(($flags12)>>2)]; //@line 764 "numbers.cpp"
          var $and13=($10) & 2; //@line 764 "numbers.cpp"
          var $tobool14=((($and13))|0)!=0; //@line 764 "numbers.cpp"
          if ($tobool14) { __label__ = 32;break $land_lhs_true$$if_end92$2; } //@line 764 "numbers.cpp"
        }
  
        var $11=$word_end_addr; //@line 766 "numbers.cpp"
        var $arrayidx16=(($11)&4294967295); //@line 766 "numbers.cpp"
        var $12=HEAP8[($arrayidx16)]; //@line 766 "numbers.cpp"
        var $conv17=(tempInt=(($12)),(tempInt>=128?tempInt-256:tempInt)); //@line 766 "numbers.cpp"
        var $cmp18=((($conv17))|0)==46; //@line 766 "numbers.cpp"
        var $13=$word_end_addr; //@line 767 "numbers.cpp"
        if ($cmp18) { __label__ = 7;; } else { __label__ = 8;; } //@line 766 "numbers.cpp"
        if (__label__ == 7) {
  
          var $arrayidx20=(($13+2)&4294967295); //@line 767 "numbers.cpp"
          var $call=__Z7utf8_inPiPKc($c2, $arrayidx20); //@line 767 "numbers.cpp"
          ; //@line 767 "numbers.cpp"
        }
        else if (__label__ == 8) {
  
          var $arrayidx21=(($13)&4294967295); //@line 769 "numbers.cpp"
          var $call22=__Z7utf8_inPiPKc($c2, $arrayidx21); //@line 769 "numbers.cpp"
          ;
        }
  
        var $14=$word_end_addr; //@line 771 "numbers.cpp"
        var $arrayidx23=(($14)&4294967295); //@line 771 "numbers.cpp"
        var $15=HEAP8[($arrayidx23)]; //@line 771 "numbers.cpp"
        var $conv24=(tempInt=(($15)),(tempInt>=128?tempInt-256:tempInt)); //@line 771 "numbers.cpp"
        var $cmp25=((($conv24))|0)!=0; //@line 771 "numbers.cpp"
        if (!($cmp25)) { __label__ = 32;break $land_lhs_true$$if_end92$2; } //@line 771 "numbers.cpp"
  
        var $16=$word_end_addr; //@line 771 "numbers.cpp"
        var $arrayidx27=(($16+1)&4294967295); //@line 771 "numbers.cpp"
        var $17=HEAP8[($arrayidx27)]; //@line 771 "numbers.cpp"
        var $conv28=(tempInt=(($17)),(tempInt>=128?tempInt-256:tempInt)); //@line 771 "numbers.cpp"
        var $cmp29=((($conv28))|0)!=0; //@line 771 "numbers.cpp"
        if (!($cmp29)) { __label__ = 32;break $land_lhs_true$$if_end92$2; } //@line 771 "numbers.cpp"
  
        var $18=HEAP32[(($c2)>>2)]; //@line 771 "numbers.cpp"
        var $cmp31=((($18))|0)==0; //@line 771 "numbers.cpp"
        if ($cmp31) { __label__ = 14;; } else { __label__ = 12;; } //@line 771 "numbers.cpp"
        $if_then40$$lor_lhs_false32$17: do { 
          if (__label__ == 12) {
  
            var $19=$wtab_addr; //@line 771 "numbers.cpp"
            var $arrayidx33=(($19)&4294967295); //@line 771 "numbers.cpp"
            var $flags34=(($arrayidx33)&4294967295); //@line 771 "numbers.cpp"
            var $20=HEAP32[(($flags34)>>2)]; //@line 771 "numbers.cpp"
            var $and35=($20) & 131072; //@line 771 "numbers.cpp"
            var $tobool36=((($and35))|0)!=0; //@line 771 "numbers.cpp"
            if ($tobool36) { __label__ = 14;break $if_then40$$lor_lhs_false32$17; } //@line 771 "numbers.cpp"
  
            var $21=HEAP32[(($c2)>>2)]; //@line 771 "numbers.cpp"
            var $call38=__Z7IsAlphaj($21); //@line 771 "numbers.cpp"
            var $tobool39=((($call38))|0)!=0; //@line 771 "numbers.cpp"
            if (!($tobool39)) { __label__ = 32;break $land_lhs_true$$if_end92$2; } //@line 771 "numbers.cpp"
          }
        } while(0);
  
        $ordinal=2; //@line 776 "numbers.cpp"
        var $22=$word_end_addr; //@line 777 "numbers.cpp"
        var $arrayidx41=(($22)&4294967295); //@line 777 "numbers.cpp"
        var $23=HEAP8[($arrayidx41)]; //@line 777 "numbers.cpp"
        var $conv42=(tempInt=(($23)),(tempInt>=128?tempInt-256:tempInt)); //@line 777 "numbers.cpp"
        var $cmp43=((($conv42))|0)==46; //@line 777 "numbers.cpp"
        if ($cmp43) { __label__ = 15;; } else { __label__ = 16;; } //@line 777 "numbers.cpp"
        if (__label__ == 15) {
  
          var $24=$word_end_addr; //@line 778 "numbers.cpp"
          var $arrayidx45=(($24)&4294967295); //@line 778 "numbers.cpp"
          HEAP8[($arrayidx45)]=32; //@line 778 "numbers.cpp"
          ; //@line 778 "numbers.cpp"
        }
  
        var $25=$roman_addr; //@line 780 "numbers.cpp"
        var $cmp47=((($25))|0)==0; //@line 780 "numbers.cpp"
        if (!($cmp47)) { __label__ = 32;break $land_lhs_true$$if_end92$2; } //@line 780 "numbers.cpp"
  
        var $26=$tr_addr; //@line 780 "numbers.cpp"
        var $translator_name=(($26+280)&4294967295); //@line 780 "numbers.cpp"
        var $27=HEAP32[(($translator_name)>>2)]; //@line 780 "numbers.cpp"
        var $cmp49=((($27))|0)==26741; //@line 780 "numbers.cpp"
        if (!($cmp49)) { __label__ = 32;break $land_lhs_true$$if_end92$2; } //@line 780 "numbers.cpp"
  
        $nextflags=0; //@line 783 "numbers.cpp"
        var $28=HEAP32[(($c2)>>2)]; //@line 784 "numbers.cpp"
        var $call51=__Z7IsAlphaj($28); //@line 784 "numbers.cpp"
        var $tobool52=((($call51))|0)!=0; //@line 784 "numbers.cpp"
        if ($tobool52) { __label__ = 19;; } else { __label__ = 20;; } //@line 784 "numbers.cpp"
        if (__label__ == 19) {
  
          var $29=$tr_addr; //@line 786 "numbers.cpp"
          var $30=$word_end_addr; //@line 786 "numbers.cpp"
          var $arrayidx54=(($30+2)&4294967295); //@line 786 "numbers.cpp"
          var $call55=__Z13TranslateWordP10TranslatorPciP8WORD_TAB($29, $arrayidx54, 0, 0); //@line 786 "numbers.cpp"
          $nextflags=$call55; //@line 786 "numbers.cpp"
          ; //@line 787 "numbers.cpp"
        }
  
        var $31=$tr_addr; //@line 789 "numbers.cpp"
        var $prev_dict_flags=(($31+8320)&4294967295); //@line 789 "numbers.cpp"
        var $32=HEAP32[(($prev_dict_flags)>>2)]; //@line 789 "numbers.cpp"
        var $and57=($32) & 1048576; //@line 789 "numbers.cpp"
        var $tobool58=((($and57))|0)!=0; //@line 789 "numbers.cpp"
        if ($tobool58) { __label__ = 21;; } else { __label__ = 25;; } //@line 789 "numbers.cpp"
        $land_lhs_true59$$if_end70$29: do { 
          if (__label__ == 21) {
  
            var $33=HEAP32[(($c2)>>2)]; //@line 789 "numbers.cpp"
            var $cmp60=((($33))|0)==0; //@line 789 "numbers.cpp"
            if ($cmp60) { __label__ = 24;; } else { __label__ = 22;; } //@line 789 "numbers.cpp"
            $if_then69$$lor_lhs_false61$31: do { 
              if (__label__ == 22) {
  
                var $34=$wtab_addr; //@line 789 "numbers.cpp"
                var $arrayidx62=(($34)&4294967295); //@line 789 "numbers.cpp"
                var $flags63=(($arrayidx62)&4294967295); //@line 789 "numbers.cpp"
                var $35=HEAP32[(($flags63)>>2)]; //@line 789 "numbers.cpp"
                var $and64=($35) & 131072; //@line 789 "numbers.cpp"
                var $tobool65=((($and64))|0)!=0; //@line 789 "numbers.cpp"
                if ($tobool65) { __label__ = 24;break $if_then69$$lor_lhs_false61$31; } //@line 789 "numbers.cpp"
  
                var $36=HEAP32[(($c2)>>2)]; //@line 789 "numbers.cpp"
                var $call67=_iswdigit($36); //@line 789 "numbers.cpp"
                var $tobool68=((($call67))|0)!=0; //@line 789 "numbers.cpp"
                if (!($tobool68)) { __label__ = 25;break $land_lhs_true59$$if_end70$29; } //@line 789 "numbers.cpp"
              }
            } while(0);
  
            $ordinal=0; //@line 790 "numbers.cpp"
            ; //@line 790 "numbers.cpp"
          }
        } while(0);
  
        var $37=$nextflags; //@line 792 "numbers.cpp"
        var $and71=($37) & 1048576; //@line 792 "numbers.cpp"
        var $tobool72=((($and71))|0)!=0; //@line 792 "numbers.cpp"
        if ($tobool72) { __label__ = 26;; } else { __label__ = 27;; } //@line 792 "numbers.cpp"
        if (__label__ == 26) {
  
          $ordinal=0; //@line 793 "numbers.cpp"
          ; //@line 793 "numbers.cpp"
        }
  
        var $38=$nextflags; //@line 795 "numbers.cpp"
        var $and75=($38) & 4194304; //@line 795 "numbers.cpp"
        var $tobool76=((($and75))|0)!=0; //@line 795 "numbers.cpp"
        if (!($tobool76)) { __label__ = 32;break $land_lhs_true$$if_end92$2; } //@line 795 "numbers.cpp"
  
        var $39=$word_addr; //@line 797 "numbers.cpp"
        var $arrayidx78=(($39+-2)&4294967295); //@line 797 "numbers.cpp"
        var $40=HEAP8[($arrayidx78)]; //@line 797 "numbers.cpp"
        var $conv79=(tempInt=(($40)),(tempInt>=128?tempInt-256:tempInt)); //@line 797 "numbers.cpp"
        var $cmp80=((($conv79))|0)==45; //@line 797 "numbers.cpp"
        if ($cmp80) { __label__ = 29;; } else { __label__ = 30;; } //@line 797 "numbers.cpp"
        if (__label__ == 29) {
  
          $ordinal=0; //@line 798 "numbers.cpp"
          ; //@line 798 "numbers.cpp"
        }
  
        var $41=$tr_addr; //@line 800 "numbers.cpp"
        var $prev_dict_flags83=(($41+8320)&4294967295); //@line 800 "numbers.cpp"
        var $42=HEAP32[(($prev_dict_flags83)>>2)]; //@line 800 "numbers.cpp"
        var $and84=($42) & 5242880; //@line 800 "numbers.cpp"
        var $tobool85=((($and84))|0)!=0; //@line 800 "numbers.cpp"
        if (!($tobool85)) { __label__ = 32;break $land_lhs_true$$if_end92$2; } //@line 800 "numbers.cpp"
  
        $ordinal=34; //@line 801 "numbers.cpp"
        ; //@line 801 "numbers.cpp"
      }
    } while(0);
  
    var $43=$ordinal; //@line 807 "numbers.cpp"
    STACKTOP = __stackBase__;
    return $43; //@line 807 "numbers.cpp"
    return null;
  }
  

  function __ZL11hu_number_ePKcii($word, $thousandplex, $value) {
    ;
    var __label__;
  
    var $retval;
    var $word_addr;
    var $thousandplex_addr;
    var $value_addr;
    $word_addr=$word;
    $thousandplex_addr=$thousandplex;
    $value_addr=$value;
    var $0=$word_addr; //@line 815 "numbers.cpp"
    var $arrayidx=(($0)&4294967295); //@line 815 "numbers.cpp"
    var $1=HEAP8[($arrayidx)]; //@line 815 "numbers.cpp"
    var $conv=(tempInt=(($1)),(tempInt>=128?tempInt-256:tempInt)); //@line 815 "numbers.cpp"
    var $cmp=((($conv))|0)==97; //@line 815 "numbers.cpp"
    if ($cmp) { __label__ = 2;; } else { __label__ = 1;; } //@line 815 "numbers.cpp"
    $if_then$$lor_lhs_false$2: do { 
      if (__label__ == 1) {
  
        var $2=$word_addr; //@line 815 "numbers.cpp"
        var $arrayidx1=(($2)&4294967295); //@line 815 "numbers.cpp"
        var $3=HEAP8[($arrayidx1)]; //@line 815 "numbers.cpp"
        var $conv2=(tempInt=(($3)),(tempInt>=128?tempInt-256:tempInt)); //@line 815 "numbers.cpp"
        var $cmp3=((($conv2))|0)==101; //@line 815 "numbers.cpp"
        if ($cmp3) { __label__ = 2;break $if_then$$lor_lhs_false$2; } //@line 815 "numbers.cpp"
  
        $retval=0; //@line 825 "numbers.cpp"
        __label__ = 13;break $if_then$$lor_lhs_false$2; //@line 825 "numbers.cpp"
      }
    } while(0);
    $if_then$$return$5: do { 
      if (__label__ == 2) {
  
        var $4=$word_addr; //@line 817 "numbers.cpp"
        var $arrayidx4=(($4+1)&4294967295); //@line 817 "numbers.cpp"
        var $5=HEAP8[($arrayidx4)]; //@line 817 "numbers.cpp"
        var $conv5=(tempInt=(($5)),(tempInt>=128?tempInt-256:tempInt)); //@line 817 "numbers.cpp"
        var $cmp6=((($conv5))|0)==32; //@line 817 "numbers.cpp"
        if ($cmp6) { __label__ = 6;; } else { __label__ = 3;; } //@line 817 "numbers.cpp"
        $if_then18$$lor_lhs_false7$7: do { 
          if (__label__ == 3) {
  
            var $6=$word_addr; //@line 817 "numbers.cpp"
            var $arrayidx8=(($6+1)&4294967295); //@line 817 "numbers.cpp"
            var $7=HEAP8[($arrayidx8)]; //@line 817 "numbers.cpp"
            var $conv9=(tempInt=(($7)),(tempInt>=128?tempInt-256:tempInt)); //@line 817 "numbers.cpp"
            var $cmp10=((($conv9))|0)==122; //@line 817 "numbers.cpp"
            if ($cmp10) { __label__ = 6;break $if_then18$$lor_lhs_false7$7; } //@line 817 "numbers.cpp"
  
            var $8=$word_addr; //@line 817 "numbers.cpp"
            var $arrayidx12=(($8+1)&4294967295); //@line 817 "numbers.cpp"
            var $9=HEAP8[($arrayidx12)]; //@line 817 "numbers.cpp"
            var $conv13=(tempInt=(($9)),(tempInt>=128?tempInt-256:tempInt)); //@line 817 "numbers.cpp"
            var $cmp14=((($conv13))|0)==116; //@line 817 "numbers.cpp"
            if ($cmp14) { __label__ = 5;; } else { __label__ = 7;; } //@line 817 "numbers.cpp"
            if (__label__ == 5) {
  
              var $10=$word_addr; //@line 817 "numbers.cpp"
              var $arrayidx15=(($10+2)&4294967295); //@line 817 "numbers.cpp"
              var $11=HEAP8[($arrayidx15)]; //@line 817 "numbers.cpp"
              var $conv16=(tempInt=(($11)),(tempInt>=128?tempInt-256:tempInt)); //@line 817 "numbers.cpp"
              var $cmp17=((($conv16))|0)==116; //@line 817 "numbers.cpp"
              if ($cmp17) { __label__ = 6;break $if_then18$$lor_lhs_false7$7; } //@line 817 "numbers.cpp"
            }
  
            var $12=$thousandplex_addr; //@line 820 "numbers.cpp"
            var $cmp19=((($12))|0)==1; //@line 820 "numbers.cpp"
            if ($cmp19) { __label__ = 9;; } else { __label__ = 8;; } //@line 820 "numbers.cpp"
            $land_lhs_true22$$lor_lhs_false20$13: do { 
              if (__label__ == 8) {
  
                var $13=$value_addr; //@line 820 "numbers.cpp"
                var $rem=((($13))|0)%1000; //@line 820 "numbers.cpp"
                var $cmp21=((($rem))|0)==0; //@line 820 "numbers.cpp"
                if ($cmp21) { __label__ = 9;break $land_lhs_true22$$lor_lhs_false20$13; } else { __label__ = 11;break $land_lhs_true22$$lor_lhs_false20$13; } //@line 820 "numbers.cpp"
              }
            } while(0);
            $land_lhs_true22$$if_end27$15: do { 
              if (__label__ == 9) {
  
                var $14=$word_addr; //@line 820 "numbers.cpp"
                var $arrayidx23=(($14+1)&4294967295); //@line 820 "numbers.cpp"
                var $15=HEAP8[($arrayidx23)]; //@line 820 "numbers.cpp"
                var $conv24=(tempInt=(($15)),(tempInt>=128?tempInt-256:tempInt)); //@line 820 "numbers.cpp"
                var $cmp25=((($conv24))|0)==108; //@line 820 "numbers.cpp"
                if (!($cmp25)) { __label__ = 11;break $land_lhs_true22$$if_end27$15; } //@line 820 "numbers.cpp"
  
                $retval=0; //@line 821 "numbers.cpp"
                __label__ = 13;break $if_then$$return$5; //@line 821 "numbers.cpp"
              }
            } while(0);
  
            $retval=1; //@line 823 "numbers.cpp"
            __label__ = 13;break $if_then$$return$5; //@line 823 "numbers.cpp"
          }
        } while(0);
  
        $retval=0; //@line 818 "numbers.cpp"
        ; //@line 818 "numbers.cpp"
      }
    } while(0);
  
    var $16=$retval; //@line 826 "numbers.cpp"
    ;
    return $16; //@line 826 "numbers.cpp"
    return null;
  }
  

  function __Z15TranslateNumberP10TranslatorPcS1_PjP8WORD_TABi($tr, $word1, $ph_out, $flags, $wtab, $control) {
    ;
    var __label__;
  
    var $retval;
    var $tr_addr;
    var $word1_addr;
    var $ph_out_addr;
    var $flags_addr;
    var $wtab_addr;
    var $control_addr;
    $tr_addr=$tr;
    $word1_addr=$word1;
    $ph_out_addr=$ph_out;
    $flags_addr=$flags;
    $wtab_addr=$wtab;
    $control_addr=$control;
    var $0=HEAP32[((_option_sayas)>>2)]; //@line 1944 "numbers.cpp"
    var $cmp=((($0))|0)==193; //@line 1944 "numbers.cpp"
    if ($cmp) { __label__ = 2;; } else { __label__ = 1;; } //@line 1944 "numbers.cpp"
    $if_then$$lor_lhs_false$2: do { 
      if (__label__ == 1) {
  
        var $1=$wtab_addr; //@line 1944 "numbers.cpp"
        var $arrayidx=(($1)&4294967295); //@line 1944 "numbers.cpp"
        var $flags1=(($arrayidx)&4294967295); //@line 1944 "numbers.cpp"
        var $2=HEAP32[(($flags1)>>2)]; //@line 1944 "numbers.cpp"
        var $and=($2) & 524288; //@line 1944 "numbers.cpp"
        var $tobool=((($and))|0)!=0; //@line 1944 "numbers.cpp"
        if ($tobool) { __label__ = 2;break $if_then$$lor_lhs_false$2; } //@line 1944 "numbers.cpp"
  
        var $3=$tr_addr; //@line 1947 "numbers.cpp"
        var $langopts=(($3)&4294967295); //@line 1947 "numbers.cpp"
        var $numbers=(($langopts+184)&4294967295); //@line 1947 "numbers.cpp"
        var $4=HEAP32[(($numbers)>>2)]; //@line 1947 "numbers.cpp"
        var $cmp2=((($4))|0)!=0; //@line 1947 "numbers.cpp"
        if ($cmp2) { __label__ = 4;; } else { __label__ = 5;; } //@line 1947 "numbers.cpp"
        if (__label__ == 4) {
  
          var $5=$tr_addr; //@line 1948 "numbers.cpp"
          var $6=$word1_addr; //@line 1948 "numbers.cpp"
          var $7=$ph_out_addr; //@line 1948 "numbers.cpp"
          var $8=$flags_addr; //@line 1948 "numbers.cpp"
          var $9=$wtab_addr; //@line 1948 "numbers.cpp"
          var $10=$control_addr; //@line 1948 "numbers.cpp"
          var $call=__ZL17TranslateNumber_1P10TranslatorPcS1_PjP8WORD_TABi($5, $6, $7, $8, $9, $10); //@line 1948 "numbers.cpp"
          $retval=$call; //@line 1948 "numbers.cpp"
          __label__ = 6;break $if_then$$lor_lhs_false$2; //@line 1948 "numbers.cpp"
        }
        else if (__label__ == 5) {
  
          $retval=0; //@line 1950 "numbers.cpp"
          __label__ = 6;break $if_then$$lor_lhs_false$2; //@line 1950 "numbers.cpp"
        }
      }
    } while(0);
    if (__label__ == 2) {
  
      $retval=0; //@line 1945 "numbers.cpp"
      ; //@line 1945 "numbers.cpp"
    }
  
    var $11=$retval; //@line 1951 "numbers.cpp"
    ;
    return $11; //@line 1951 "numbers.cpp"
    return null;
  }
  

  function __ZL17TranslateNumber_1P10TranslatorPcS1_PjP8WORD_TABi($tr, $word, $ph_out, $flags, $wtab, $control) {
    var __stackBase__  = STACKTOP; STACKTOP += 560; _memset(__stackBase__, 0, 560);
    var __label__;
    var __lastLabel__ = null;
  
    var $retval;
    var $tr_addr;
    var $word_addr;
    var $ph_out_addr;
    var $flags_addr;
    var $wtab_addr;
    var $control_addr;
    var $n_digits;
    var $value;
    var $ix;
    var $digix;
    var $c;
    var $suppress_null;
    var $decimal_point;
    var $thousandplex;
    var $thousands_exact;
    var $thousands_inc;
    var $prev_thousands;
    var $ordinal;
    var $dot_ordinal;
    var $this_value;
    var $decimal_count;
    var $max_decimal_count;
    var $decimal_mode;
    var $hyphen;
    var $suffix_ix;
    var $skipwords;
    var $p=__stackBase__;
    var $string=__stackBase__+4;
    var $buf1=__stackBase__+24;
    var $ph_append=__stackBase__+124;
    var $ph_buf=__stackBase__+176;
    var $ph_buf2=__stackBase__+376;
    var $ph_zeros=__stackBase__+428;
    var $suffix=__stackBase__+480;
    var $buf_digit_lookup=__stackBase__+500;
    var $p2=__stackBase__+552;
    var $next_char=__stackBase__+556;
    var $p562;
    $tr_addr=$tr;
    $word_addr=$word;
    $ph_out_addr=$ph_out;
    $flags_addr=$flags;
    $wtab_addr=$wtab;
    $control_addr=$control;
    $suppress_null=0; //@line 1577 "numbers.cpp"
    $decimal_point=0; //@line 1578 "numbers.cpp"
    $thousandplex=0; //@line 1579 "numbers.cpp"
    $thousands_exact=1; //@line 1580 "numbers.cpp"
    $thousands_inc=0; //@line 1581 "numbers.cpp"
    $prev_thousands=0; //@line 1582 "numbers.cpp"
    $ordinal=0; //@line 1583 "numbers.cpp"
    $skipwords=0; //@line 1591 "numbers.cpp"
    var $0=$flags_addr; //@line 1604 "numbers.cpp"
    HEAP32[(($0)>>2)]=0; //@line 1604 "numbers.cpp"
    HEAP32[((__ZL14n_digit_lookup)>>2)]=0; //@line 1605 "numbers.cpp"
    var $arrayidx=(($buf_digit_lookup)&4294967295); //@line 1606 "numbers.cpp"
    HEAP8[($arrayidx)]=0; //@line 1606 "numbers.cpp"
    var $arraydecay=(($buf_digit_lookup)&4294967295); //@line 1607 "numbers.cpp"
    HEAP32[((__ZL12digit_lookup)>>2)]=$arraydecay; //@line 1607 "numbers.cpp"
    var $1=$control_addr; //@line 1608 "numbers.cpp"
    HEAP32[((__ZL14number_control)>>2)]=$1; //@line 1608 "numbers.cpp"
    $ix=0; //@line 1610 "numbers.cpp"
    var $2=$ix; //@line 1610 "numbers.cpp"
    var $3=$word_addr; //@line 1610 "numbers.cpp"
    var $arrayidx167=(($3+$2)&4294967295); //@line 1610 "numbers.cpp"
    var $4=HEAP8[($arrayidx167)]; //@line 1610 "numbers.cpp"
    var $conv68=(tempInt=(($4)),(tempInt>=128?tempInt-256:tempInt)); //@line 1610 "numbers.cpp"
    var $isdigittmp69=((($conv68)-48)&4294967295); //@line 1610 "numbers.cpp"
    var $isdigit70=((($isdigittmp69))>>>0) < 10; //@line 1610 "numbers.cpp"
    var $5=$ix; //@line 1610 "numbers.cpp"
    if ($isdigit70) { __lastLabel__ = 0; __label__ = 1;; } else { __lastLabel__ = 0; __label__ = 2;; } //@line 1610 "numbers.cpp"
    $for_inc$$for_end$2: do { 
      if (__label__ == 1) {
        while(1) { 
  
          var $6=__lastLabel__ == 1 ? $10 : ($5);
          var $inc=((($6)+1)&4294967295); //@line 1610 "numbers.cpp"
          $ix=$inc; //@line 1610 "numbers.cpp"
          var $7=$ix; //@line 1610 "numbers.cpp"
          var $8=$word_addr; //@line 1610 "numbers.cpp"
          var $arrayidx1=(($8+$7)&4294967295); //@line 1610 "numbers.cpp"
          var $9=HEAP8[($arrayidx1)]; //@line 1610 "numbers.cpp"
          var $conv=(tempInt=(($9)),(tempInt>=128?tempInt-256:tempInt)); //@line 1610 "numbers.cpp"
          var $isdigittmp=((($conv)-48)&4294967295); //@line 1610 "numbers.cpp"
          var $isdigit=((($isdigittmp))>>>0) < 10; //@line 1610 "numbers.cpp"
          var $call=((($isdigit))&1); //@line 1610 "numbers.cpp"
          var $tobool=((($call))|0)!=0; //@line 1610 "numbers.cpp"
          var $10=$ix; //@line 1610 "numbers.cpp"
          if ($tobool) { __lastLabel__ = 1; __label__ = 1;continue ; } else { __lastLabel__ = 1; __label__ = 2;break $for_inc$$for_end$2; } //@line 1610 "numbers.cpp"
        }
      }
    } while(0);
  
    var $_lcssa65=__lastLabel__ == 0 ? $5 : ($10);
    $n_digits=$_lcssa65; //@line 1611 "numbers.cpp"
    var $11=$word_addr; //@line 1612 "numbers.cpp"
    var $call2=_atoi($11); //@line 1612 "numbers.cpp"
    $this_value=$call2; //@line 1612 "numbers.cpp"
    $value=$call2; //@line 1612 "numbers.cpp"
    var $cmp=((($_lcssa65))|0)==3; //@line 1615 "numbers.cpp"
    if ($cmp) { __label__ = 3;; } else { __label__ = 6;; } //@line 1615 "numbers.cpp"
    $land_lhs_true$$if_else$6: do { 
      if (__label__ == 3) {
  
        var $12=$word_addr; //@line 1615 "numbers.cpp"
        var $arrayidx3=(($12+-2)&4294967295); //@line 1615 "numbers.cpp"
        var $13=HEAP8[($arrayidx3)]; //@line 1615 "numbers.cpp"
        var $conv4=(tempInt=(($13)),(tempInt>=128?tempInt-256:tempInt)); //@line 1615 "numbers.cpp"
        var $14=$tr_addr; //@line 1615 "numbers.cpp"
        var $langopts=(($14)&4294967295); //@line 1615 "numbers.cpp"
        var $thousands_sep=(($langopts+204)&4294967295); //@line 1615 "numbers.cpp"
        var $15=HEAP32[(($thousands_sep)>>2)]; //@line 1615 "numbers.cpp"
        var $cmp5=((($conv4))|0)==((($15))|0); //@line 1615 "numbers.cpp"
        if (!($cmp5)) { __label__ = 6;break $land_lhs_true$$if_else$6; } //@line 1615 "numbers.cpp"
  
        var $16=$word_addr; //@line 1615 "numbers.cpp"
        var $arrayidx7=(($16+-3)&4294967295); //@line 1615 "numbers.cpp"
        var $17=HEAP8[($arrayidx7)]; //@line 1615 "numbers.cpp"
        var $conv8=(tempInt=(($17)),(tempInt>=128?tempInt-256:tempInt)); //@line 1615 "numbers.cpp"
        var $isdigittmp6=((($conv8)-48)&4294967295); //@line 1615 "numbers.cpp"
        var $isdigit7=((($isdigittmp6))>>>0) < 10; //@line 1615 "numbers.cpp"
        var $call9=((($isdigit7))&1); //@line 1615 "numbers.cpp"
        var $tobool10=((($call9))|0)!=0; //@line 1615 "numbers.cpp"
        if (!($tobool10)) { __label__ = 6;break $land_lhs_true$$if_else$6; } //@line 1615 "numbers.cpp"
  
        $prev_thousands=1; //@line 1617 "numbers.cpp"
        __label__ = 14;break $land_lhs_true$$if_else$6; //@line 1618 "numbers.cpp"
      }
    } while(0);
    $if_end32$$if_else$10: do { 
      if (__label__ == 6) {
  
        var $18=$tr_addr; //@line 1620 "numbers.cpp"
        var $langopts11=(($18)&4294967295); //@line 1620 "numbers.cpp"
        var $thousands_sep12=(($langopts11+204)&4294967295); //@line 1620 "numbers.cpp"
        var $19=HEAP32[(($thousands_sep12)>>2)]; //@line 1620 "numbers.cpp"
        var $cmp13=((($19))|0)==32; //@line 1620 "numbers.cpp"
        if ($cmp13) { __label__ = 8;; } else { __label__ = 7;; } //@line 1620 "numbers.cpp"
        $if_then16$$lor_lhs_false$12: do { 
          if (__label__ == 7) {
  
            var $20=$tr_addr; //@line 1620 "numbers.cpp"
            var $langopts14=(($20)&4294967295); //@line 1620 "numbers.cpp"
            var $numbers=(($langopts14+184)&4294967295); //@line 1620 "numbers.cpp"
            var $21=HEAP32[(($numbers)>>2)]; //@line 1620 "numbers.cpp"
            var $and=($21) & 4096; //@line 1620 "numbers.cpp"
            var $tobool15=((($and))|0)!=0; //@line 1620 "numbers.cpp"
            if ($tobool15) { __label__ = 8;break $if_then16$$lor_lhs_false$12; } else { __label__ = 12;break $if_then16$$lor_lhs_false$12; } //@line 1620 "numbers.cpp"
          }
        } while(0);
        $if_then16$$if_end29$14: do { 
          if (__label__ == 8) {
  
            var $22=$n_digits; //@line 1623 "numbers.cpp"
            var $cmp17=((($22))|0)==3; //@line 1623 "numbers.cpp"
            if (!($cmp17)) { __label__ = 12;break $if_then16$$if_end29$14; } //@line 1623 "numbers.cpp"
  
            var $23=$wtab_addr; //@line 1623 "numbers.cpp"
            var $flags19=(($23)&4294967295); //@line 1623 "numbers.cpp"
            var $24=HEAP32[(($flags19)>>2)]; //@line 1623 "numbers.cpp"
            var $and20=($24) & 262144; //@line 1623 "numbers.cpp"
            var $tobool21=((($and20))|0)!=0; //@line 1623 "numbers.cpp"
            if ($tobool21) { __label__ = 12;break $if_then16$$if_end29$14; } //@line 1623 "numbers.cpp"
  
            var $25=$word_addr; //@line 1623 "numbers.cpp"
            var $arrayidx23=(($25+-2)&4294967295); //@line 1623 "numbers.cpp"
            var $26=HEAP8[($arrayidx23)]; //@line 1623 "numbers.cpp"
            var $conv24=(tempInt=(($26)),(tempInt>=128?tempInt-256:tempInt)); //@line 1623 "numbers.cpp"
            var $isdigittmp8=((($conv24)-48)&4294967295); //@line 1623 "numbers.cpp"
            var $isdigit9=((($isdigittmp8))>>>0) < 10; //@line 1623 "numbers.cpp"
            var $call25=((($isdigit9))&1); //@line 1623 "numbers.cpp"
            var $tobool26=((($call25))|0)!=0; //@line 1623 "numbers.cpp"
            if (!($tobool26)) { __label__ = 12;break $if_then16$$if_end29$14; } //@line 1623 "numbers.cpp"
  
            $prev_thousands=1; //@line 1625 "numbers.cpp"
            __label__ = 14;break $if_end32$$if_else$10; //@line 1626 "numbers.cpp"
          }
        } while(0);
  
        var $_pr=$prev_thousands; //@line 1628 "numbers.cpp"
        var $cmp30=((($_pr))|0)==0; //@line 1628 "numbers.cpp"
        if (!($cmp30)) { __label__ = 14;break $if_end32$$if_else$10; } //@line 1628 "numbers.cpp"
  
        HEAP32[((__ZL23speak_missing_thousands)>>2)]=0; //@line 1630 "numbers.cpp"
        ; //@line 1631 "numbers.cpp"
      }
    } while(0);
  
    HEAP8[(((__ZL11ph_ordinal2)&4294967295))]=0; //@line 1633 "numbers.cpp"
    var $arrayidx33=(($ph_zeros)&4294967295); //@line 1634 "numbers.cpp"
    HEAP8[($arrayidx33)]=0; //@line 1634 "numbers.cpp"
    var $27=$prev_thousands; //@line 1636 "numbers.cpp"
    var $tobool34=((($27))|0)!=0; //@line 1636 "numbers.cpp"
    if ($tobool34) { __label__ = 16;; } else { __label__ = 15;; } //@line 1636 "numbers.cpp"
    $if_then39$$lor_lhs_false35$22: do { 
      if (__label__ == 15) {
  
        var $28=$word_addr; //@line 1636 "numbers.cpp"
        var $arrayidx36=(($28)&4294967295); //@line 1636 "numbers.cpp"
        var $29=HEAP8[($arrayidx36)]; //@line 1636 "numbers.cpp"
        var $conv37=(tempInt=(($29)),(tempInt>=128?tempInt-256:tempInt)); //@line 1636 "numbers.cpp"
        var $cmp38=((($conv37))|0)!=48; //@line 1636 "numbers.cpp"
        if ($cmp38) { __label__ = 16;break $if_then39$$lor_lhs_false35$22; } else { __label__ = 18;break $if_then39$$lor_lhs_false35$22; } //@line 1636 "numbers.cpp"
      }
    } while(0);
    $if_then39$$if_end45$24: do { 
      if (__label__ == 16) {
  
        var $30=$tr_addr; //@line 1639 "numbers.cpp"
        var $31=$word_addr; //@line 1639 "numbers.cpp"
        var $32=$ix; //@line 1639 "numbers.cpp"
        var $33=$word_addr; //@line 1639 "numbers.cpp"
        var $arrayidx40=(($33+$32)&4294967295); //@line 1639 "numbers.cpp"
        var $34=$wtab_addr; //@line 1639 "numbers.cpp"
        var $call41=__ZL15CheckDotOrdinalP10TranslatorPcS1_P8WORD_TABi($30, $31, $arrayidx40, $34, 0); //@line 1639 "numbers.cpp"
        $ordinal=$call41; //@line 1639 "numbers.cpp"
        var $cmp42=((($call41))|0)!=0; //@line 1639 "numbers.cpp"
        if (!($cmp42)) { __label__ = 18;break $if_then39$$if_end45$24; } //@line 1639 "numbers.cpp"
  
        $dot_ordinal=1; //@line 1640 "numbers.cpp"
        ; //@line 1640 "numbers.cpp"
      }
    } while(0);
  
    var $35=$ix; //@line 1643 "numbers.cpp"
    var $36=$word_addr; //@line 1643 "numbers.cpp"
    var $arrayidx46=(($36+$35)&4294967295); //@line 1643 "numbers.cpp"
    var $37=HEAP8[($arrayidx46)]; //@line 1643 "numbers.cpp"
    var $conv47=(tempInt=(($37)),(tempInt>=128?tempInt-256:tempInt)); //@line 1643 "numbers.cpp"
    var $cmp48=((($conv47))|0)==46; //@line 1643 "numbers.cpp"
    if ($cmp48) { __label__ = 19;; } else { __label__ = 23;; } //@line 1643 "numbers.cpp"
    $land_lhs_true49$$if_end67$28: do { 
      if (__label__ == 19) {
  
        var $38=$ix; //@line 1643 "numbers.cpp"
        var $add=((($38)+1)&4294967295); //@line 1643 "numbers.cpp"
        var $39=$word_addr; //@line 1643 "numbers.cpp"
        var $arrayidx50=(($39+$add)&4294967295); //@line 1643 "numbers.cpp"
        var $40=HEAP8[($arrayidx50)]; //@line 1643 "numbers.cpp"
        var $conv51=(tempInt=(($40)),(tempInt>=128?tempInt-256:tempInt)); //@line 1643 "numbers.cpp"
        var $isdigittmp10=((($conv51)-48)&4294967295); //@line 1643 "numbers.cpp"
        var $isdigit11=((($isdigittmp10))>>>0) < 10; //@line 1643 "numbers.cpp"
        var $call52=((($isdigit11))&1); //@line 1643 "numbers.cpp"
        var $tobool53=((($call52))|0)!=0; //@line 1643 "numbers.cpp"
        if ($tobool53) { __label__ = 23;break $land_lhs_true49$$if_end67$28; } //@line 1643 "numbers.cpp"
  
        var $41=$ix; //@line 1643 "numbers.cpp"
        var $add55=((($41)+2)&4294967295); //@line 1643 "numbers.cpp"
        var $42=$word_addr; //@line 1643 "numbers.cpp"
        var $arrayidx56=(($42+$add55)&4294967295); //@line 1643 "numbers.cpp"
        var $43=HEAP8[($arrayidx56)]; //@line 1643 "numbers.cpp"
        var $conv57=(tempInt=(($43)),(tempInt>=128?tempInt-256:tempInt)); //@line 1643 "numbers.cpp"
        var $isdigittmp12=((($conv57)-48)&4294967295); //@line 1643 "numbers.cpp"
        var $isdigit13=((($isdigittmp12))>>>0) < 10; //@line 1643 "numbers.cpp"
        var $call58=((($isdigit13))&1); //@line 1643 "numbers.cpp"
        var $tobool59=((($call58))|0)!=0; //@line 1643 "numbers.cpp"
        if ($tobool59) { __label__ = 23;break $land_lhs_true49$$if_end67$28; } //@line 1643 "numbers.cpp"
  
        var $44=$wtab_addr; //@line 1643 "numbers.cpp"
        var $arrayidx61=(($44+12)&4294967295); //@line 1643 "numbers.cpp"
        var $flags62=(($arrayidx61)&4294967295); //@line 1643 "numbers.cpp"
        var $45=HEAP32[(($flags62)>>2)]; //@line 1643 "numbers.cpp"
        var $and63=($45) & 256; //@line 1643 "numbers.cpp"
        var $tobool64=((($and63))|0)!=0; //@line 1643 "numbers.cpp"
        if ($tobool64) { __label__ = 23;break $land_lhs_true49$$if_end67$28; } //@line 1643 "numbers.cpp"
  
        var $46=$ix; //@line 1646 "numbers.cpp"
        var $47=$word_addr; //@line 1646 "numbers.cpp"
        var $arrayidx66=(($47+$46)&4294967295); //@line 1646 "numbers.cpp"
        HEAP8[($arrayidx66)]=0; //@line 1646 "numbers.cpp"
        ; //@line 1647 "numbers.cpp"
      }
    } while(0);
  
    var $48=$ordinal; //@line 1649 "numbers.cpp"
    var $cmp68=((($48))|0)==0; //@line 1649 "numbers.cpp"
    if ($cmp68) { __label__ = 25;; } else { __label__ = 24;; } //@line 1649 "numbers.cpp"
    $if_then71$$lor_lhs_false69$34: do { 
      if (__label__ == 24) {
  
        var $49=$tr_addr; //@line 1649 "numbers.cpp"
        var $translator_name=(($49+280)&4294967295); //@line 1649 "numbers.cpp"
        var $50=HEAP32[(($translator_name)>>2)]; //@line 1649 "numbers.cpp"
        var $cmp70=((($50))|0)==26741; //@line 1649 "numbers.cpp"
        if ($cmp70) { __label__ = 25;break $if_then71$$lor_lhs_false69$34; } else { __label__ = 38;break $if_then71$$lor_lhs_false69$34; } //@line 1649 "numbers.cpp"
      }
    } while(0);
    $if_then71$$if_end123$36: do { 
      if (__label__ == 25) {
  
        var $51=$ix; //@line 1653 "numbers.cpp"
        var $inc72=((($51)+1)&4294967295); //@line 1653 "numbers.cpp"
        $ix=$inc72; //@line 1653 "numbers.cpp"
        $hyphen=0; //@line 1654 "numbers.cpp"
        var $arraydecay73=(($suffix)&4294967295); //@line 1655 "numbers.cpp"
        HEAP32[(($p)>>2)]=$arraydecay73; //@line 1655 "numbers.cpp"
        var $52=$wtab_addr; //@line 1656 "numbers.cpp"
        var $arrayidx74=(($52)&4294967295); //@line 1656 "numbers.cpp"
        var $flags75=(($arrayidx74)&4294967295); //@line 1656 "numbers.cpp"
        var $53=HEAP32[(($flags75)>>2)]; //@line 1656 "numbers.cpp"
        var $and76=($53) & 16384; //@line 1656 "numbers.cpp"
        var $tobool77=((($and76))|0)!=0; //@line 1656 "numbers.cpp"
        if ($tobool77) { __label__ = 26;; } else { __label__ = 27;; } //@line 1656 "numbers.cpp"
        if (__label__ == 26) {
  
          var $54=HEAP32[(($p)>>2)]; //@line 1658 "numbers.cpp"
          var $incdec_ptr=(($54+1)&4294967295); //@line 1658 "numbers.cpp"
          HEAP32[(($p)>>2)]=$incdec_ptr; //@line 1658 "numbers.cpp"
          HEAP8[($54)]=45; //@line 1658 "numbers.cpp"
          $hyphen=1; //@line 1659 "numbers.cpp"
          var $55=$ix; //@line 1660 "numbers.cpp"
          var $inc79=((($55)+1)&4294967295); //@line 1660 "numbers.cpp"
          $ix=$inc79; //@line 1660 "numbers.cpp"
          ; //@line 1661 "numbers.cpp"
        }
        $while_cond$40: while(1) { 
  
          var $56=$ix; //@line 1662 "numbers.cpp"
          var $57=$word_addr; //@line 1662 "numbers.cpp"
          var $arrayidx81=(($57+$56)&4294967295); //@line 1662 "numbers.cpp"
          var $58=HEAP8[($arrayidx81)]; //@line 1662 "numbers.cpp"
          var $conv82=(tempInt=(($58)),(tempInt>=128?tempInt-256:tempInt)); //@line 1662 "numbers.cpp"
          var $cmp83=((($conv82))|0)!=0; //@line 1662 "numbers.cpp"
          if (!($cmp83)) { __label__ = 31;break $while_cond$40; } //@line 1662 "numbers.cpp"
  
          var $59=$ix; //@line 1662 "numbers.cpp"
          var $60=$word_addr; //@line 1662 "numbers.cpp"
          var $arrayidx85=(($60+$59)&4294967295); //@line 1662 "numbers.cpp"
          var $61=HEAP8[($arrayidx85)]; //@line 1662 "numbers.cpp"
          var $conv86=(tempInt=(($61)),(tempInt>=128?tempInt-256:tempInt)); //@line 1662 "numbers.cpp"
          var $cmp87=((($conv86))|0)!=32; //@line 1662 "numbers.cpp"
          if (!($cmp87)) { __label__ = 31;break $while_cond$40; } //@line 1662 "numbers.cpp"
  
          var $62=$ix; //@line 1662 "numbers.cpp"
          var $cmp88=((($62))|0) < 19; //@line 1662 "numbers.cpp"
          if (!($cmp88)) { __label__ = 31;break $while_cond$40; }
  
          var $63=$ix; //@line 1664 "numbers.cpp"
          var $inc89=((($63)+1)&4294967295); //@line 1664 "numbers.cpp"
          $ix=$inc89; //@line 1664 "numbers.cpp"
          var $64=$word_addr; //@line 1664 "numbers.cpp"
          var $arrayidx90=(($64+$63)&4294967295); //@line 1664 "numbers.cpp"
          var $65=HEAP8[($arrayidx90)]; //@line 1664 "numbers.cpp"
          var $66=HEAP32[(($p)>>2)]; //@line 1664 "numbers.cpp"
          var $incdec_ptr91=(($66+1)&4294967295); //@line 1664 "numbers.cpp"
          HEAP32[(($p)>>2)]=$incdec_ptr91; //@line 1664 "numbers.cpp"
          HEAP8[($66)]=$65; //@line 1664 "numbers.cpp"
          __label__ = 27;continue $while_cond$40; //@line 1665 "numbers.cpp"
        }
  
        var $67=HEAP32[(($p)>>2)]; //@line 1666 "numbers.cpp"
        HEAP8[($67)]=0; //@line 1666 "numbers.cpp"
        var $arrayidx92=(($suffix)&4294967295); //@line 1668 "numbers.cpp"
        var $68=HEAP8[($arrayidx92)]; //@line 1668 "numbers.cpp"
        var $conv93=(tempInt=(($68)),(tempInt>=128?tempInt-256:tempInt)); //@line 1668 "numbers.cpp"
        var $cmp94=((($conv93))|0)!=0; //@line 1668 "numbers.cpp"
        if (!($cmp94)) { __label__ = 38;break $if_then71$$if_end123$36; } //@line 1668 "numbers.cpp"
  
        var $69=$tr_addr; //@line 1670 "numbers.cpp"
        var $langopts96=(($69)&4294967295); //@line 1670 "numbers.cpp"
        var $ordinal_indicator=(($langopts96+216)&4294967295); //@line 1670 "numbers.cpp"
        var $70=HEAP32[(($ordinal_indicator)>>2)]; //@line 1670 "numbers.cpp"
        var $cmp97=((($70))|0)!=0; //@line 1670 "numbers.cpp"
        if ($cmp97) { __label__ = 33;; } else { __label__ = 35;; } //@line 1670 "numbers.cpp"
        $land_lhs_true98$$if_else105$47: do { 
          if (__label__ == 33) {
  
            var $arraydecay99=(($suffix)&4294967295); //@line 1670 "numbers.cpp"
            var $71=$tr_addr; //@line 1670 "numbers.cpp"
            var $langopts100=(($71)&4294967295); //@line 1670 "numbers.cpp"
            var $ordinal_indicator101=(($langopts100+216)&4294967295); //@line 1670 "numbers.cpp"
            var $72=HEAP32[(($ordinal_indicator101)>>2)]; //@line 1670 "numbers.cpp"
            var $call102=_strcmp($arraydecay99, $72); //@line 1670 "numbers.cpp"
            var $cmp103=((($call102))|0)==0; //@line 1670 "numbers.cpp"
            if (!($cmp103)) { __label__ = 35;break $land_lhs_true98$$if_else105$47; } //@line 1670 "numbers.cpp"
  
            $ordinal=2; //@line 1672 "numbers.cpp"
            __label__ = 38;break $if_then71$$if_end123$36; //@line 1673 "numbers.cpp"
          }
        } while(0);
  
        var $arrayidx106=(($suffix)&4294967295); //@line 1675 "numbers.cpp"
        var $73=HEAP8[($arrayidx106)]; //@line 1675 "numbers.cpp"
        var $conv107=(tempInt=(($73)),(tempInt>=128?tempInt-256:tempInt)); //@line 1675 "numbers.cpp"
        var $isdigittmp14=((($conv107)-48)&4294967295); //@line 1675 "numbers.cpp"
        var $isdigit15=((($isdigittmp14))>>>0) < 10; //@line 1675 "numbers.cpp"
        var $call108=((($isdigit15))&1); //@line 1675 "numbers.cpp"
        var $tobool109=((($call108))|0)!=0; //@line 1675 "numbers.cpp"
        if ($tobool109) { __label__ = 38;break $if_then71$$if_end123$36; } //@line 1675 "numbers.cpp"
  
        var $arraydecay111=(($string)&4294967295); //@line 1677 "numbers.cpp"
        var $arraydecay112=(($suffix)&4294967295); //@line 1677 "numbers.cpp"
        var $call113=_sprintf($arraydecay111, ((__str15312)&4294967295), allocate([$arraydecay112,0,0,0], ["i8*",0,0,0], ALLOC_STACK)); //@line 1677 "numbers.cpp"
        var $74=$tr_addr; //@line 1678 "numbers.cpp"
        var $arraydecay114=(($string)&4294967295); //@line 1678 "numbers.cpp"
        var $call115=__Z6LookupP10TranslatorPKcPc($74, $arraydecay114, ((__ZL11ph_ordinal2)&4294967295)); //@line 1678 "numbers.cpp"
        var $tobool116=((($call115))|0)!=0; //@line 1678 "numbers.cpp"
        if (!($tobool116)) { __label__ = 38;break $if_then71$$if_end123$36; } //@line 1678 "numbers.cpp"
  
        $ordinal=2; //@line 1681 "numbers.cpp"
        var $75=$flags_addr; //@line 1682 "numbers.cpp"
        var $arrayidx118=(($75)&4294967295); //@line 1682 "numbers.cpp"
        var $76=HEAP32[(($arrayidx118)>>2)]; //@line 1682 "numbers.cpp"
        var $or=($76) | 128; //@line 1682 "numbers.cpp"
        HEAP32[(($arrayidx118)>>2)]=$or; //@line 1682 "numbers.cpp"
        $skipwords=1; //@line 1683 "numbers.cpp"
        ; //@line 1684 "numbers.cpp"
      }
    } while(0);
  
    var $77=$wtab_addr; //@line 1689 "numbers.cpp"
    var $arrayidx124=(($77)&4294967295); //@line 1689 "numbers.cpp"
    var $flags125=(($arrayidx124)&4294967295); //@line 1689 "numbers.cpp"
    var $78=HEAP32[(($flags125)>>2)]; //@line 1689 "numbers.cpp"
    var $and126=($78) & 32768; //@line 1689 "numbers.cpp"
    var $tobool127=((($and126))|0)!=0; //@line 1689 "numbers.cpp"
    if ($tobool127) { __label__ = 39;; } else { __label__ = 40;; } //@line 1689 "numbers.cpp"
    if (__label__ == 39) {
  
      $ordinal=2; //@line 1690 "numbers.cpp"
      ; //@line 1690 "numbers.cpp"
    }
  
    var $arrayidx130=(($ph_append)&4294967295); //@line 1692 "numbers.cpp"
    HEAP8[($arrayidx130)]=0; //@line 1692 "numbers.cpp"
    var $arrayidx131=(($ph_buf2)&4294967295); //@line 1693 "numbers.cpp"
    HEAP8[($arrayidx131)]=0; //@line 1693 "numbers.cpp"
    var $79=$word_addr; //@line 1696 "numbers.cpp"
    var $arrayidx132=(($79)&4294967295); //@line 1696 "numbers.cpp"
    var $80=HEAP8[($arrayidx132)]; //@line 1696 "numbers.cpp"
    var $conv133=(tempInt=(($80)),(tempInt>=128?tempInt-256:tempInt)); //@line 1696 "numbers.cpp"
    var $cmp134=((($conv133))|0)==48; //@line 1696 "numbers.cpp"
    if ($cmp134) { __label__ = 41;; } else { __label__ = 55;; } //@line 1696 "numbers.cpp"
    $land_lhs_true135$$if_end185$57: do { 
      if (__label__ == 41) {
  
        var $81=$prev_thousands; //@line 1696 "numbers.cpp"
        var $cmp136=((($81))|0)==0; //@line 1696 "numbers.cpp"
        if (!($cmp136)) { __label__ = 55;break $land_lhs_true135$$if_end185$57; } //@line 1696 "numbers.cpp"
  
        var $82=$word_addr; //@line 1696 "numbers.cpp"
        var $arrayidx138=(($82+1)&4294967295); //@line 1696 "numbers.cpp"
        var $83=HEAP8[($arrayidx138)]; //@line 1696 "numbers.cpp"
        var $conv139=(tempInt=(($83)),(tempInt>=128?tempInt-256:tempInt)); //@line 1696 "numbers.cpp"
        var $cmp140=((($conv139))|0)!=32; //@line 1696 "numbers.cpp"
        if (!($cmp140)) { __label__ = 55;break $land_lhs_true135$$if_end185$57; } //@line 1696 "numbers.cpp"
  
        var $84=$word_addr; //@line 1696 "numbers.cpp"
        var $arrayidx142=(($84+1)&4294967295); //@line 1696 "numbers.cpp"
        var $85=HEAP8[($arrayidx142)]; //@line 1696 "numbers.cpp"
        var $conv143=(tempInt=(($85)),(tempInt>=128?tempInt-256:tempInt)); //@line 1696 "numbers.cpp"
        var $86=$tr_addr; //@line 1696 "numbers.cpp"
        var $langopts144=(($86)&4294967295); //@line 1696 "numbers.cpp"
        var $decimal_sep=(($langopts144+208)&4294967295); //@line 1696 "numbers.cpp"
        var $87=HEAP32[(($decimal_sep)>>2)]; //@line 1696 "numbers.cpp"
        var $cmp145=((($conv143))|0)!=((($87))|0); //@line 1696 "numbers.cpp"
        if (!($cmp145)) { __label__ = 55;break $land_lhs_true135$$if_end185$57; } //@line 1696 "numbers.cpp"
  
        var $88=$n_digits; //@line 1698 "numbers.cpp"
        var $cmp147=((($88))|0)==2; //@line 1698 "numbers.cpp"
        if ($cmp147) { __lastLabel__ = 44; __label__ = 45;; } else { __lastLabel__ = 44; __label__ = 49;; } //@line 1698 "numbers.cpp"
        if (__label__ == 45) {
  
          var $89=$word_addr; //@line 1698 "numbers.cpp"
          var $arrayidx149=(($89+3)&4294967295); //@line 1698 "numbers.cpp"
          var $90=HEAP8[($arrayidx149)]; //@line 1698 "numbers.cpp"
          var $conv150=(tempInt=(($90)),(tempInt>=128?tempInt-256:tempInt)); //@line 1698 "numbers.cpp"
          var $cmp151=((($conv150))|0)==58; //@line 1698 "numbers.cpp"
          if ($cmp151) { __label__ = 46;; } else { __label__ = 48;; } //@line 1698 "numbers.cpp"
          $land_lhs_true152$$if_else163thread_pre_split$64: do { 
            if (__label__ == 46) {
  
              var $91=$word_addr; //@line 1698 "numbers.cpp"
              var $arrayidx153=(($91+5)&4294967295); //@line 1698 "numbers.cpp"
              var $92=HEAP8[($arrayidx153)]; //@line 1698 "numbers.cpp"
              var $conv154=(tempInt=(($92)),(tempInt>=128?tempInt-256:tempInt)); //@line 1698 "numbers.cpp"
              var $isdigittmp16=((($conv154)-48)&4294967295); //@line 1698 "numbers.cpp"
              var $isdigit17=((($isdigittmp16))>>>0) < 10; //@line 1698 "numbers.cpp"
              var $call155=((($isdigit17))&1); //@line 1698 "numbers.cpp"
              var $tobool156=((($call155))|0)!=0; //@line 1698 "numbers.cpp"
              if (!($tobool156)) { __label__ = 48;break $land_lhs_true152$$if_else163thread_pre_split$64; } //@line 1698 "numbers.cpp"
  
              var $93=$word_addr; //@line 1698 "numbers.cpp"
              var $arrayidx158=(($93+7)&4294967295); //@line 1698 "numbers.cpp"
              var $94=HEAP8[($arrayidx158)]; //@line 1698 "numbers.cpp"
              var $conv159=(tempInt=(($94)),(tempInt>=128?tempInt-256:tempInt)); //@line 1698 "numbers.cpp"
              var $call160=_isspace($conv159); //@line 1698 "numbers.cpp"
              var $tobool161=((($call160))|0)!=0; //@line 1698 "numbers.cpp"
              if ($tobool161) { __label__ = 55;break $land_lhs_true135$$if_end185$57; } //@line 1698 "numbers.cpp"
            }
          } while(0);
  
          var $_pr43=$n_digits; //@line 1704 "numbers.cpp"
          __lastLabel__ = 48; ;
        }
  
        var $95=__lastLabel__ == 48 ? $_pr43 : ($88); //@line 1704 "numbers.cpp"
        var $cmp164=((($95))|0) > 3; //@line 1704 "numbers.cpp"
        if ($cmp164) { __label__ = 50;; } else { __label__ = 51;; } //@line 1704 "numbers.cpp"
        if (__label__ == 50) {
  
          var $96=$flags_addr; //@line 1706 "numbers.cpp"
          var $arrayidx166=(($96)&4294967295); //@line 1706 "numbers.cpp"
          var $97=HEAP32[(($arrayidx166)>>2)]; //@line 1706 "numbers.cpp"
          var $and167=($97) & -129; //@line 1706 "numbers.cpp"
          HEAP32[(($arrayidx166)>>2)]=$and167; //@line 1706 "numbers.cpp"
          $retval=0; //@line 1707 "numbers.cpp"
          __label__ = 157;break $land_lhs_true135$$if_end185$57; //@line 1707 "numbers.cpp"
        }
        else if (__label__ == 51) {
  
          $ix=0; //@line 1711 "numbers.cpp"
          var $arraydecay177=(($ph_zeros)&4294967295); //@line 1713 "numbers.cpp"
          ; //@line 1711 "numbers.cpp"
          while(1) { 
  
            var $98=$ix; //@line 1711 "numbers.cpp"
            var $99=$word_addr; //@line 1711 "numbers.cpp"
            var $arrayidx170=(($99+$98)&4294967295); //@line 1711 "numbers.cpp"
            var $100=HEAP8[($arrayidx170)]; //@line 1711 "numbers.cpp"
            var $conv171=(tempInt=(($100)),(tempInt>=128?tempInt-256:tempInt)); //@line 1711 "numbers.cpp"
            var $cmp172=((($conv171))|0)==48; //@line 1711 "numbers.cpp"
            if (!($cmp172)) { __label__ = 55;break $land_lhs_true135$$if_end185$57; } //@line 1711 "numbers.cpp"
  
            var $101=$ix; //@line 1711 "numbers.cpp"
            var $102=$n_digits; //@line 1711 "numbers.cpp"
            var $sub=((($102)-1)&4294967295); //@line 1711 "numbers.cpp"
            var $cmp174=((($101))|0) < ((($sub))|0); //@line 1711 "numbers.cpp"
            if (!($cmp174)) { __label__ = 55;break $land_lhs_true135$$if_end185$57; }
  
            var $103=$tr_addr; //@line 1713 "numbers.cpp"
            var $call178=_strlen($arraydecay177); //@line 1713 "numbers.cpp"
            var $arrayidx179=(($ph_zeros+$call178)&4294967295); //@line 1713 "numbers.cpp"
            var $call180=__Z6LookupP10TranslatorPKcPc($103, ((__str16313)&4294967295), $arrayidx179); //@line 1713 "numbers.cpp"
            var $104=$ix; //@line 1711 "numbers.cpp"
            var $inc182=((($104)+1)&4294967295); //@line 1711 "numbers.cpp"
            $ix=$inc182; //@line 1711 "numbers.cpp"
            __label__ = 52;continue ; //@line 1711 "numbers.cpp"
          }
        }
      }
    } while(0);
    if (__label__ == 55) {
  
      var $105=$tr_addr; //@line 1718 "numbers.cpp"
      var $langopts186=(($105)&4294967295); //@line 1718 "numbers.cpp"
      var $numbers187=(($langopts186+184)&4294967295); //@line 1718 "numbers.cpp"
      var $106=HEAP32[(($numbers187)>>2)]; //@line 1718 "numbers.cpp"
      var $and188=($106) & 4096; //@line 1718 "numbers.cpp"
      var $tobool189=((($and188))|0)!=0; //@line 1718 "numbers.cpp"
      if ($tobool189) { __label__ = 56;; } else { __label__ = 58;; } //@line 1718 "numbers.cpp"
      $land_lhs_true190$$if_else195$78: do { 
        if (__label__ == 56) {
  
          var $107=$n_digits; //@line 1718 "numbers.cpp"
          var $108=$word_addr; //@line 1718 "numbers.cpp"
          var $arrayidx191=(($108+$107)&4294967295); //@line 1718 "numbers.cpp"
          var $109=HEAP8[($arrayidx191)]; //@line 1718 "numbers.cpp"
          var $conv192=(tempInt=(($109)),(tempInt>=128?tempInt-256:tempInt)); //@line 1718 "numbers.cpp"
          var $cmp193=((($conv192))|0)==32; //@line 1718 "numbers.cpp"
          if (!($cmp193)) { __label__ = 58;break $land_lhs_true190$$if_else195$78; } //@line 1718 "numbers.cpp"
  
          $thousands_inc=1; //@line 1719 "numbers.cpp"
          __label__ = 60;break $land_lhs_true190$$if_else195$78; //@line 1719 "numbers.cpp"
        }
      } while(0);
      $if_end203_thread$$if_else195$81: do { 
        if (__label__ == 58) {
  
          var $110=$n_digits; //@line 1721 "numbers.cpp"
          var $111=$word_addr; //@line 1721 "numbers.cpp"
          var $arrayidx196=(($111+$110)&4294967295); //@line 1721 "numbers.cpp"
          var $112=HEAP8[($arrayidx196)]; //@line 1721 "numbers.cpp"
          var $conv197=(tempInt=(($112)),(tempInt>=128?tempInt-256:tempInt)); //@line 1721 "numbers.cpp"
          var $113=$tr_addr; //@line 1721 "numbers.cpp"
          var $langopts198=(($113)&4294967295); //@line 1721 "numbers.cpp"
          var $thousands_sep199=(($langopts198+204)&4294967295); //@line 1721 "numbers.cpp"
          var $114=HEAP32[(($thousands_sep199)>>2)]; //@line 1721 "numbers.cpp"
          var $cmp200=((($conv197))|0)==((($114))|0); //@line 1721 "numbers.cpp"
          if ($cmp200) { __label__ = 59;; } else { __label__ = 61;; } //@line 1721 "numbers.cpp"
          if (__label__ == 59) {
  
            $thousands_inc=2; //@line 1722 "numbers.cpp"
            __label__ = 60;break $if_end203_thread$$if_else195$81; //@line 1722 "numbers.cpp"
          }
          else if (__label__ == 61) {
  
            var $_pr44=$thousands_inc; //@line 1725 "numbers.cpp"
            var $116=$n_digits; //@line 1724 "numbers.cpp"
            var $add204=((($116)+2)&4294967295); //@line 1724 "numbers.cpp"
            $suffix_ix=$add204; //@line 1724 "numbers.cpp"
            var $cmp205=((($_pr44))|0) > 0; //@line 1725 "numbers.cpp"
            if ($cmp205) { __label__ = 62;break $if_end203_thread$$if_else195$81; } else { __label__ = 77;break $if_end203_thread$$if_else195$81; } //@line 1725 "numbers.cpp"
          }
        }
      } while(0);
      $if_end203_thread$$if_then206$$if_end282$86: do { 
        if (__label__ == 60) {
  
          var $115=$n_digits; //@line 1724 "numbers.cpp"
          var $add20445=((($115)+2)&4294967295); //@line 1724 "numbers.cpp"
          $suffix_ix=$add20445; //@line 1724 "numbers.cpp"
          __label__ = 62;break $if_end203_thread$$if_then206$$if_end282$86; //@line 1725 "numbers.cpp"
        }
      } while(0);
      $if_then206$$if_end282$88: do { 
        if (__label__ == 62) {
  
          var $117=$n_digits; //@line 1729 "numbers.cpp"
          var $118=$thousands_inc; //@line 1729 "numbers.cpp"
          var $add207=((($118)+($117))&4294967295); //@line 1729 "numbers.cpp"
          $digix=$add207; //@line 1729 "numbers.cpp"
          ; //@line 1731 "numbers.cpp"
          $while_cond208$90: while(1) { 
  
            var $119=$thousandplex; //@line 1731 "numbers.cpp"
            var $add209=((($119)+1)&4294967295); //@line 1731 "numbers.cpp"
            var $120=$wtab_addr; //@line 1731 "numbers.cpp"
            var $arrayidx210=(($120+12*$add209)&4294967295); //@line 1731 "numbers.cpp"
            var $flags211=(($arrayidx210)&4294967295); //@line 1731 "numbers.cpp"
            var $121=HEAP32[(($flags211)>>2)]; //@line 1731 "numbers.cpp"
            var $and212=($121) & 262144; //@line 1731 "numbers.cpp"
            var $cmp213=((($and212))|0)==0; //@line 1731 "numbers.cpp"
            if (!($cmp213)) { __label__ = 77;break $if_then206$$if_end282$88; } //@line 1731 "numbers.cpp"
  
            var $122=$digix; //@line 1732 "numbers.cpp"
            var $123=$word_addr; //@line 1732 "numbers.cpp"
            var $arrayidx215=(($123+$122)&4294967295); //@line 1732 "numbers.cpp"
            var $124=HEAP8[($arrayidx215)]; //@line 1732 "numbers.cpp"
            var $conv216=(tempInt=(($124)),(tempInt>=128?tempInt-256:tempInt)); //@line 1732 "numbers.cpp"
            var $isdigittmp18=((($conv216)-48)&4294967295); //@line 1732 "numbers.cpp"
            var $isdigit19=((($isdigittmp18))>>>0) < 10; //@line 1732 "numbers.cpp"
            var $call217=((($isdigit19))&1); //@line 1732 "numbers.cpp"
            var $tobool218=((($call217))|0)!=0; //@line 1732 "numbers.cpp"
            if (!($tobool218)) { __label__ = 77;break $if_then206$$if_end282$88; } //@line 1732 "numbers.cpp"
  
            var $125=$digix; //@line 1732 "numbers.cpp"
            var $add220=((($125)+1)&4294967295); //@line 1732 "numbers.cpp"
            var $126=$word_addr; //@line 1732 "numbers.cpp"
            var $arrayidx221=(($126+$add220)&4294967295); //@line 1732 "numbers.cpp"
            var $127=HEAP8[($arrayidx221)]; //@line 1732 "numbers.cpp"
            var $conv222=(tempInt=(($127)),(tempInt>=128?tempInt-256:tempInt)); //@line 1732 "numbers.cpp"
            var $isdigittmp20=((($conv222)-48)&4294967295); //@line 1732 "numbers.cpp"
            var $isdigit21=((($isdigittmp20))>>>0) < 10; //@line 1732 "numbers.cpp"
            var $call223=((($isdigit21))&1); //@line 1732 "numbers.cpp"
            var $tobool224=((($call223))|0)!=0; //@line 1732 "numbers.cpp"
            if (!($tobool224)) { __label__ = 77;break $if_then206$$if_end282$88; } //@line 1732 "numbers.cpp"
  
            var $128=$digix; //@line 1732 "numbers.cpp"
            var $add226=((($128)+2)&4294967295); //@line 1732 "numbers.cpp"
            var $129=$word_addr; //@line 1732 "numbers.cpp"
            var $arrayidx227=(($129+$add226)&4294967295); //@line 1732 "numbers.cpp"
            var $130=HEAP8[($arrayidx227)]; //@line 1732 "numbers.cpp"
            var $conv228=(tempInt=(($130)),(tempInt>=128?tempInt-256:tempInt)); //@line 1732 "numbers.cpp"
            var $isdigittmp22=((($conv228)-48)&4294967295); //@line 1732 "numbers.cpp"
            var $isdigit23=((($isdigittmp22))>>>0) < 10; //@line 1732 "numbers.cpp"
            var $call229=((($isdigit23))&1); //@line 1732 "numbers.cpp"
            var $tobool230=((($call229))|0)!=0; //@line 1732 "numbers.cpp"
            if (!($tobool230)) { __label__ = 77;break $if_then206$$if_end282$88; } //@line 1732 "numbers.cpp"
  
            var $131=$digix; //@line 1732 "numbers.cpp"
            var $add232=((($131)+3)&4294967295); //@line 1732 "numbers.cpp"
            var $132=$word_addr; //@line 1732 "numbers.cpp"
            var $arrayidx233=(($132+$add232)&4294967295); //@line 1732 "numbers.cpp"
            var $133=HEAP8[($arrayidx233)]; //@line 1732 "numbers.cpp"
            var $conv234=(tempInt=(($133)),(tempInt>=128?tempInt-256:tempInt)); //@line 1732 "numbers.cpp"
            var $isdigittmp24=((($conv234)-48)&4294967295); //@line 1732 "numbers.cpp"
            var $isdigit25=((($isdigittmp24))>>>0) < 10; //@line 1732 "numbers.cpp"
            var $call235=((($isdigit25))&1); //@line 1732 "numbers.cpp"
            var $tobool236=((($call235))|0)!=0; //@line 1732 "numbers.cpp"
            if ($tobool236) { __label__ = 77;break $if_then206$$if_end282$88; } //@line 1732 "numbers.cpp"
  
            var $134=$digix; //@line 1732 "numbers.cpp"
            var $sub238=((($134)-1)&4294967295); //@line 1732 "numbers.cpp"
            var $135=$word_addr; //@line 1732 "numbers.cpp"
            var $arrayidx239=(($135+$sub238)&4294967295); //@line 1732 "numbers.cpp"
            var $136=HEAP8[($arrayidx239)]; //@line 1732 "numbers.cpp"
            var $conv240=(tempInt=(($136)),(tempInt>=128?tempInt-256:tempInt)); //@line 1732 "numbers.cpp"
            var $isdigittmp26=((($conv240)-48)&4294967295); //@line 1732 "numbers.cpp"
            var $isdigit27=((($isdigittmp26))>>>0) < 10; //@line 1732 "numbers.cpp"
            var $call241=((($isdigit27))&1); //@line 1732 "numbers.cpp"
            var $tobool242=((($call241))|0)!=0; //@line 1732 "numbers.cpp"
            var $lnot=($tobool242) ^ 1; //@line 1732 "numbers.cpp"
            if (!($lnot)) { __label__ = 77;break $if_then206$$if_end282$88; }
  
            var $137=$digix; //@line 1734 "numbers.cpp"
            var $138=$word_addr; //@line 1734 "numbers.cpp"
            var $arrayidx245=(($138+$137)&4294967295); //@line 1734 "numbers.cpp"
            var $139=HEAP8[($arrayidx245)]; //@line 1734 "numbers.cpp"
            var $conv246=(tempInt=(($139)),(tempInt>=128?tempInt-256:tempInt)); //@line 1734 "numbers.cpp"
            var $cmp247=((($conv246))|0)!=48; //@line 1734 "numbers.cpp"
            if ($cmp247) { __label__ = 72;; } else { __label__ = 70;; } //@line 1734 "numbers.cpp"
            $if_then258$$lor_lhs_false248$98: do { 
              if (__label__ == 70) {
  
                var $140=$digix; //@line 1734 "numbers.cpp"
                var $add249=((($140)+1)&4294967295); //@line 1734 "numbers.cpp"
                var $141=$word_addr; //@line 1734 "numbers.cpp"
                var $arrayidx250=(($141+$add249)&4294967295); //@line 1734 "numbers.cpp"
                var $142=HEAP8[($arrayidx250)]; //@line 1734 "numbers.cpp"
                var $conv251=(tempInt=(($142)),(tempInt>=128?tempInt-256:tempInt)); //@line 1734 "numbers.cpp"
                var $cmp252=((($conv251))|0)!=48; //@line 1734 "numbers.cpp"
                if ($cmp252) { __label__ = 72;break $if_then258$$lor_lhs_false248$98; } //@line 1734 "numbers.cpp"
  
                var $143=$digix; //@line 1734 "numbers.cpp"
                var $add254=((($143)+2)&4294967295); //@line 1734 "numbers.cpp"
                var $144=$word_addr; //@line 1734 "numbers.cpp"
                var $arrayidx255=(($144+$add254)&4294967295); //@line 1734 "numbers.cpp"
                var $145=HEAP8[($arrayidx255)]; //@line 1734 "numbers.cpp"
                var $conv256=(tempInt=(($145)),(tempInt>=128?tempInt-256:tempInt)); //@line 1734 "numbers.cpp"
                var $cmp257=((($conv256))|0)!=48; //@line 1734 "numbers.cpp"
                if ($cmp257) { __label__ = 72;break $if_then258$$lor_lhs_false248$98; } else { __label__ = 73;break $if_then258$$lor_lhs_false248$98; } //@line 1734 "numbers.cpp"
              }
            } while(0);
            if (__label__ == 72) {
  
              $thousands_exact=0; //@line 1735 "numbers.cpp"
              ; //@line 1735 "numbers.cpp"
            }
  
            var $146=$thousandplex; //@line 1737 "numbers.cpp"
            var $inc260=((($146)+1)&4294967295); //@line 1737 "numbers.cpp"
            $thousandplex=$inc260; //@line 1737 "numbers.cpp"
            var $147=$digix; //@line 1738 "numbers.cpp"
            var $add261=((($147)+3)&4294967295); //@line 1738 "numbers.cpp"
            $digix=$add261; //@line 1738 "numbers.cpp"
            var $148=$digix; //@line 1739 "numbers.cpp"
            var $149=$word_addr; //@line 1739 "numbers.cpp"
            var $arrayidx262=(($149+$148)&4294967295); //@line 1739 "numbers.cpp"
            var $150=HEAP8[($arrayidx262)]; //@line 1739 "numbers.cpp"
            var $conv263=(tempInt=(($150)),(tempInt>=128?tempInt-256:tempInt)); //@line 1739 "numbers.cpp"
            var $151=$tr_addr; //@line 1739 "numbers.cpp"
            var $langopts264=(($151)&4294967295); //@line 1739 "numbers.cpp"
            var $thousands_sep265=(($langopts264+204)&4294967295); //@line 1739 "numbers.cpp"
            var $152=HEAP32[(($thousands_sep265)>>2)]; //@line 1739 "numbers.cpp"
            var $cmp266=((($conv263))|0)==((($152))|0); //@line 1739 "numbers.cpp"
            if ($cmp266) { __label__ = 76;; } else { __label__ = 74;; } //@line 1739 "numbers.cpp"
            if (__label__ == 74) {
  
              var $153=$tr_addr; //@line 1739 "numbers.cpp"
              var $langopts268=(($153)&4294967295); //@line 1739 "numbers.cpp"
              var $numbers269=(($langopts268+184)&4294967295); //@line 1739 "numbers.cpp"
              var $154=HEAP32[(($numbers269)>>2)]; //@line 1739 "numbers.cpp"
              var $and270=($154) & 4096; //@line 1739 "numbers.cpp"
              var $tobool271=((($and270))|0)!=0; //@line 1739 "numbers.cpp"
              if (!($tobool271)) { __label__ = 77;break $if_then206$$if_end282$88; } //@line 1739 "numbers.cpp"
  
              var $155=$digix; //@line 1739 "numbers.cpp"
              var $156=$word_addr; //@line 1739 "numbers.cpp"
              var $arrayidx273=(($156+$155)&4294967295); //@line 1739 "numbers.cpp"
              var $157=HEAP8[($arrayidx273)]; //@line 1739 "numbers.cpp"
              var $conv274=(tempInt=(($157)),(tempInt>=128?tempInt-256:tempInt)); //@line 1739 "numbers.cpp"
              var $cmp275=((($conv274))|0)==32; //@line 1739 "numbers.cpp"
              if (!($cmp275)) { __label__ = 77;break $if_then206$$if_end282$88; } //@line 1739 "numbers.cpp"
            }
  
            var $158=$digix; //@line 1741 "numbers.cpp"
            var $add277=((($158)+2)&4294967295); //@line 1741 "numbers.cpp"
            $suffix_ix=$add277; //@line 1741 "numbers.cpp"
            var $159=$thousands_inc; //@line 1742 "numbers.cpp"
            var $160=$digix; //@line 1742 "numbers.cpp"
            var $add278=((($160)+($159))&4294967295); //@line 1742 "numbers.cpp"
            $digix=$add278; //@line 1742 "numbers.cpp"
            __label__ = 63;continue $while_cond208$90; //@line 1746 "numbers.cpp"
          }
        }
      } while(0);
  
      var $161=$value; //@line 1749 "numbers.cpp"
      var $cmp283=((($161))|0)==0; //@line 1749 "numbers.cpp"
      if ($cmp283) { __label__ = 78;; } else { __label__ = 80;; } //@line 1749 "numbers.cpp"
      $land_lhs_true284$$if_end287$109: do { 
        if (__label__ == 78) {
  
          var $162=$prev_thousands; //@line 1749 "numbers.cpp"
          var $tobool285=((($162))|0)!=0; //@line 1749 "numbers.cpp"
          if (!($tobool285)) { __label__ = 80;break $land_lhs_true284$$if_end287$109; } //@line 1749 "numbers.cpp"
  
          $suppress_null=1; //@line 1751 "numbers.cpp"
          ; //@line 1752 "numbers.cpp"
        }
      } while(0);
  
      var $163=$tr_addr; //@line 1754 "numbers.cpp"
      var $translator_name288=(($163+280)&4294967295); //@line 1754 "numbers.cpp"
      var $164=HEAP32[(($translator_name288)>>2)]; //@line 1754 "numbers.cpp"
      var $cmp289=((($164))|0)==26741; //@line 1754 "numbers.cpp"
      if ($cmp289) { __label__ = 81;; } else { __label__ = 85;; } //@line 1754 "numbers.cpp"
      $if_then290$$if_end304$113: do { 
        if (__label__ == 81) {
  
          var $165=$thousandplex; //@line 1757 "numbers.cpp"
          var $166=$wtab_addr; //@line 1757 "numbers.cpp"
          var $arrayidx291=(($166+12*$165)&4294967295); //@line 1757 "numbers.cpp"
          var $flags292=(($arrayidx291)&4294967295); //@line 1757 "numbers.cpp"
          var $167=HEAP32[(($flags292)>>2)]; //@line 1757 "numbers.cpp"
          var $and293=($167) & 16384; //@line 1757 "numbers.cpp"
          var $tobool294=((($and293))|0)!=0; //@line 1757 "numbers.cpp"
          if (!($tobool294)) { __label__ = 85;break $if_then290$$if_end304$113; } //@line 1757 "numbers.cpp"
  
          var $168=$thousands_exact; //@line 1757 "numbers.cpp"
          var $cmp296=((($168))|0)==1; //@line 1757 "numbers.cpp"
          if (!($cmp296)) { __label__ = 85;break $if_then290$$if_end304$113; } //@line 1757 "numbers.cpp"
  
          var $169=$suffix_ix; //@line 1757 "numbers.cpp"
          var $170=$word_addr; //@line 1757 "numbers.cpp"
          var $arrayidx298=(($170+$169)&4294967295); //@line 1757 "numbers.cpp"
          var $171=$thousandplex; //@line 1757 "numbers.cpp"
          var $172=$value; //@line 1757 "numbers.cpp"
          var $call299=__ZL11hu_number_ePKcii($arrayidx298, $171, $172); //@line 1757 "numbers.cpp"
          var $tobool300=((($call299))|0)!=0; //@line 1757 "numbers.cpp"
          if (!($tobool300)) { __label__ = 85;break $if_then290$$if_end304$113; } //@line 1757 "numbers.cpp"
  
          var $173=HEAP32[((__ZL14number_control)>>2)]; //@line 1759 "numbers.cpp"
          var $or302=($173) | 1; //@line 1759 "numbers.cpp"
          HEAP32[((__ZL14number_control)>>2)]=$or302; //@line 1759 "numbers.cpp"
          ; //@line 1760 "numbers.cpp"
        }
      } while(0);
  
      var $174=$n_digits; //@line 1763 "numbers.cpp"
      var $175=$word_addr; //@line 1763 "numbers.cpp"
      var $arrayidx305=(($175+$174)&4294967295); //@line 1763 "numbers.cpp"
      var $176=HEAP8[($arrayidx305)]; //@line 1763 "numbers.cpp"
      var $conv306=(tempInt=(($176)),(tempInt>=128?tempInt-256:tempInt)); //@line 1763 "numbers.cpp"
      var $177=$tr_addr; //@line 1763 "numbers.cpp"
      var $langopts307=(($177)&4294967295); //@line 1763 "numbers.cpp"
      var $decimal_sep308=(($langopts307+208)&4294967295); //@line 1763 "numbers.cpp"
      var $178=HEAP32[(($decimal_sep308)>>2)]; //@line 1763 "numbers.cpp"
      var $cmp309=((($conv306))|0)==((($178))|0); //@line 1763 "numbers.cpp"
      if ($cmp309) { __label__ = 86;; } else { __label__ = 88;; } //@line 1763 "numbers.cpp"
      $land_lhs_true310$$if_else319$119: do { 
        if (__label__ == 86) {
  
          var $179=$n_digits; //@line 1763 "numbers.cpp"
          var $add311=((($179)+1)&4294967295); //@line 1763 "numbers.cpp"
          var $180=$word_addr; //@line 1763 "numbers.cpp"
          var $arrayidx312=(($180+$add311)&4294967295); //@line 1763 "numbers.cpp"
          var $181=HEAP8[($arrayidx312)]; //@line 1763 "numbers.cpp"
          var $conv313=(tempInt=(($181)),(tempInt>=128?tempInt-256:tempInt)); //@line 1763 "numbers.cpp"
          var $isdigittmp28=((($conv313)-48)&4294967295); //@line 1763 "numbers.cpp"
          var $isdigit29=((($isdigittmp28))>>>0) < 10; //@line 1763 "numbers.cpp"
          var $call314=((($isdigit29))&1); //@line 1763 "numbers.cpp"
          var $tobool315=((($call314))|0)!=0; //@line 1763 "numbers.cpp"
          if (!($tobool315)) { __label__ = 88;break $land_lhs_true310$$if_else319$119; } //@line 1763 "numbers.cpp"
  
          var $182=$tr_addr; //@line 1766 "numbers.cpp"
          var $arraydecay317=(($ph_append)&4294967295); //@line 1766 "numbers.cpp"
          var $call318=__Z6LookupP10TranslatorPKcPc($182, ((__str17314)&4294967295), $arraydecay317); //@line 1766 "numbers.cpp"
          $decimal_point=1; //@line 1767 "numbers.cpp"
          __label__ = 98;break $land_lhs_true310$$if_else319$119; //@line 1768 "numbers.cpp"
        }
      } while(0);
      $if_end356$$if_else319$122: do { 
        if (__label__ == 88) {
  
          var $183=$suppress_null; //@line 1770 "numbers.cpp"
          var $cmp320=((($183))|0)==0; //@line 1770 "numbers.cpp"
          if ($cmp320) { __label__ = 89;; } else { __label__ = 95;; } //@line 1770 "numbers.cpp"
          if (__label__ == 89) {
  
            var $184=$thousands_inc; //@line 1772 "numbers.cpp"
            var $cmp322=((($184))|0) > 0; //@line 1772 "numbers.cpp"
            if (!($cmp322)) { __label__ = 98;break $if_end356$$if_else319$122; } //@line 1772 "numbers.cpp"
  
            var $185=$thousandplex; //@line 1774 "numbers.cpp"
            var $cmp324=((($185))|0) > 0; //@line 1774 "numbers.cpp"
            if (!($cmp324)) { __label__ = 98;break $if_end356$$if_else319$122; } //@line 1774 "numbers.cpp"
  
            var $186=$value; //@line 1774 "numbers.cpp"
            var $cmp326=((($186))|0) < 1000; //@line 1774 "numbers.cpp"
            if (!($cmp326)) { __label__ = 98;break $if_end356$$if_else319$122; } //@line 1774 "numbers.cpp"
  
            var $187=$suppress_null; //@line 1776 "numbers.cpp"
            var $cmp328=((($187))|0)==0; //@line 1776 "numbers.cpp"
            if (!($cmp328)) { __label__ = 98;break $if_end356$$if_else319$122; } //@line 1776 "numbers.cpp"
  
            var $188=$tr_addr; //@line 1776 "numbers.cpp"
            var $189=$value; //@line 1776 "numbers.cpp"
            var $190=$thousandplex; //@line 1776 "numbers.cpp"
            var $191=$thousands_exact; //@line 1776 "numbers.cpp"
            var $arraydecay330=(($ph_append)&4294967295); //@line 1776 "numbers.cpp"
            var $call331=__ZL15LookupThousandsP10TranslatoriiiPc($188, $189, $190, $191, $arraydecay330); //@line 1776 "numbers.cpp"
            var $tobool332=((($call331))|0)!=0; //@line 1776 "numbers.cpp"
            if (!($tobool332)) { __label__ = 98;break $if_end356$$if_else319$122; } //@line 1776 "numbers.cpp"
  
            $value=0; //@line 1779 "numbers.cpp"
            $suppress_null=1; //@line 1780 "numbers.cpp"
            ; //@line 1781 "numbers.cpp"
          }
          else if (__label__ == 95) {
  
            var $192=HEAP32[((__ZL23speak_missing_thousands)>>2)]; //@line 1787 "numbers.cpp"
            var $cmp338=((($192))|0)==1; //@line 1787 "numbers.cpp"
            if (!($cmp338)) { __label__ = 98;break $if_end356$$if_else319$122; } //@line 1787 "numbers.cpp"
  
            var $arraydecay340=(($string)&4294967295); //@line 1790 "numbers.cpp"
            var $193=$thousandplex; //@line 1790 "numbers.cpp"
            var $add341=((($193)+1)&4294967295); //@line 1790 "numbers.cpp"
            var $call342=_sprintf($arraydecay340, ((__str18315)&4294967295), allocate([$add341,0,0,0], ["i32",0,0,0], ALLOC_STACK)); //@line 1790 "numbers.cpp"
            var $194=$tr_addr; //@line 1791 "numbers.cpp"
            var $arraydecay343=(($string)&4294967295); //@line 1791 "numbers.cpp"
            var $arraydecay344=(($buf1)&4294967295); //@line 1791 "numbers.cpp"
            var $call345=__Z6LookupP10TranslatorPKcPc($194, $arraydecay343, $arraydecay344); //@line 1791 "numbers.cpp"
            var $cmp346=((($call345))|0)==0; //@line 1791 "numbers.cpp"
            if (!($cmp346)) { __label__ = 98;break $if_end356$$if_else319$122; } //@line 1791 "numbers.cpp"
  
            var $arraydecay348=(($string)&4294967295); //@line 1793 "numbers.cpp"
            var $195=$thousandplex; //@line 1793 "numbers.cpp"
            var $call349=_sprintf($arraydecay348, ((__str18315)&4294967295), allocate([$195,0,0,0], ["i32",0,0,0], ALLOC_STACK)); //@line 1793 "numbers.cpp"
            var $196=$tr_addr; //@line 1794 "numbers.cpp"
            var $arraydecay350=(($string)&4294967295); //@line 1794 "numbers.cpp"
            var $arraydecay351=(($ph_append)&4294967295); //@line 1794 "numbers.cpp"
            var $call352=__Z6LookupP10TranslatorPKcPc($196, $arraydecay350, $arraydecay351); //@line 1794 "numbers.cpp"
            ; //@line 1795 "numbers.cpp"
          }
        }
      } while(0);
  
      var $arrayidx357=(($ph_append)&4294967295); //@line 1798 "numbers.cpp"
      var $197=HEAP8[($arrayidx357)]; //@line 1798 "numbers.cpp"
      var $conv358=(tempInt=(($197)),(tempInt>=128?tempInt-256:tempInt)); //@line 1798 "numbers.cpp"
      var $cmp359=((($conv358))|0)==0; //@line 1798 "numbers.cpp"
      if ($cmp359) { __label__ = 99;; } else { __label__ = 102;; } //@line 1798 "numbers.cpp"
      $land_lhs_true360$$if_end369$135: do { 
        if (__label__ == 99) {
  
          var $198=$n_digits; //@line 1798 "numbers.cpp"
          var $199=$word_addr; //@line 1798 "numbers.cpp"
          var $arrayidx361=(($199+$198)&4294967295); //@line 1798 "numbers.cpp"
          var $200=HEAP8[($arrayidx361)]; //@line 1798 "numbers.cpp"
          var $conv362=(tempInt=(($200)),(tempInt>=128?tempInt-256:tempInt)); //@line 1798 "numbers.cpp"
          var $cmp363=((($conv362))|0)==46; //@line 1798 "numbers.cpp"
          if (!($cmp363)) { __label__ = 102;break $land_lhs_true360$$if_end369$135; } //@line 1798 "numbers.cpp"
  
          var $201=$thousandplex; //@line 1798 "numbers.cpp"
          var $cmp365=((($201))|0)==0; //@line 1798 "numbers.cpp"
          if (!($cmp365)) { __label__ = 112;break $land_lhs_true360$$if_end369$135; } //@line 1798 "numbers.cpp"
  
          var $202=$tr_addr; //@line 1800 "numbers.cpp"
          var $arraydecay367=(($ph_append)&4294967295); //@line 1800 "numbers.cpp"
          var $call368=__Z6LookupP10TranslatorPKcPc($202, ((__str19316)&4294967295), $arraydecay367); //@line 1800 "numbers.cpp"
          __label__ = 102;break $land_lhs_true360$$if_end369$135; //@line 1801 "numbers.cpp"
        }
      } while(0);
      $if_end369$$if_end404$139: do { 
        if (__label__ == 102) {
  
          var $_pr47=$thousandplex; //@line 1803 "numbers.cpp"
          var $cmp370=((($_pr47))|0)==0; //@line 1803 "numbers.cpp"
          if (!($cmp370)) { __label__ = 112;break $if_end369$$if_end404$139; } //@line 1803 "numbers.cpp"
  
          var $203=$word_addr; //@line 1807 "numbers.cpp"
          HEAP32[(($p)>>2)]=$203; //@line 1807 "numbers.cpp"
          var $204=HEAP32[(($p)>>2)]; //@line 1808 "numbers.cpp"
          var $arrayidx37360=(($204+1)&4294967295); //@line 1808 "numbers.cpp"
          var $205=HEAP8[($arrayidx37360)]; //@line 1808 "numbers.cpp"
          var $conv37461=(tempInt=(($205)),(tempInt>=128?tempInt-256:tempInt)); //@line 1808 "numbers.cpp"
          var $isdigittmp3062=((($conv37461)-48)&4294967295); //@line 1808 "numbers.cpp"
          var $isdigit3163=((($isdigittmp3062))>>>0) < 10; //@line 1808 "numbers.cpp"
          var $206=HEAPU32[(($p)>>2)]; //@line 1808 "numbers.cpp"
          if ($isdigit3163) { __lastLabel__ = 103; __label__ = 104;; } else { __lastLabel__ = 103; __label__ = 105;; } //@line 1808 "numbers.cpp"
          $while_body377$$while_end379$142: do { 
            if (__label__ == 104) {
              while(1) { 
  
                var $207=__lastLabel__ == 104 ? $210 : ($206);
                var $incdec_ptr378=(($207+1)&4294967295); //@line 1808 "numbers.cpp"
                HEAP32[(($p)>>2)]=$incdec_ptr378; //@line 1808 "numbers.cpp"
                var $208=HEAP32[(($p)>>2)]; //@line 1808 "numbers.cpp"
                var $arrayidx373=(($208+1)&4294967295); //@line 1808 "numbers.cpp"
                var $209=HEAP8[($arrayidx373)]; //@line 1808 "numbers.cpp"
                var $conv374=(tempInt=(($209)),(tempInt>=128?tempInt-256:tempInt)); //@line 1808 "numbers.cpp"
                var $isdigittmp30=((($conv374)-48)&4294967295); //@line 1808 "numbers.cpp"
                var $isdigit31=((($isdigittmp30))>>>0) < 10; //@line 1808 "numbers.cpp"
                var $call375=((($isdigit31))&1); //@line 1808 "numbers.cpp"
                var $tobool376=((($call375))|0)!=0; //@line 1808 "numbers.cpp"
                var $210=HEAPU32[(($p)>>2)]; //@line 1808 "numbers.cpp"
                if ($tobool376) { __lastLabel__ = 104; __label__ = 104;continue ; } else { __lastLabel__ = 104; __label__ = 105;break $while_body377$$while_end379$142; } //@line 1808 "numbers.cpp"
              }
            }
          } while(0);
  
          var $_lcssa=__lastLabel__ == 103 ? $206 : ($210);
          var $arrayidx380=(($_lcssa+-1)&4294967295); //@line 1809 "numbers.cpp"
          var $211=HEAP8[($arrayidx380)]; //@line 1809 "numbers.cpp"
          var $conv381=(tempInt=(($211)),(tempInt>=128?tempInt-256:tempInt)); //@line 1809 "numbers.cpp"
          var $isdigittmp32=((($conv381)-48)&4294967295); //@line 1809 "numbers.cpp"
          var $isdigit33=((($isdigittmp32))>>>0) < 10; //@line 1809 "numbers.cpp"
          var $call382=((($isdigit33))&1); //@line 1809 "numbers.cpp"
          var $tobool383=((($call382))|0)!=0; //@line 1809 "numbers.cpp"
          if ($tobool383) { __label__ = 106;; } else { __label__ = 108;; } //@line 1809 "numbers.cpp"
          $if_then384$$if_end390$146: do { 
            if (__label__ == 106) {
  
              var $212=HEAP32[(($p)>>2)]; //@line 1811 "numbers.cpp"
              var $add_ptr=(($212+-1)&4294967295); //@line 1811 "numbers.cpp"
              HEAP32[(($p2)>>2)]=$add_ptr; //@line 1811 "numbers.cpp"
              var $213=$tr_addr; //@line 1812 "numbers.cpp"
              var $arraydecay385=(($buf_digit_lookup)&4294967295); //@line 1812 "numbers.cpp"
              var $214=$flags_addr; //@line 1812 "numbers.cpp"
              var $215=$wtab_addr; //@line 1812 "numbers.cpp"
              var $call386=__Z14LookupDictListP10TranslatorPPcS1_PjiP8WORD_TAB($213, $p2, $arraydecay385, $214, 4, $215); //@line 1812 "numbers.cpp"
              var $tobool387=((($call386))|0)!=0; //@line 1812 "numbers.cpp"
              if (!($tobool387)) { __label__ = 108;break $if_then384$$if_end390$146; } //@line 1812 "numbers.cpp"
  
              HEAP32[((__ZL14n_digit_lookup)>>2)]=2; //@line 1814 "numbers.cpp"
              ; //@line 1815 "numbers.cpp"
            }
          } while(0);
  
          var $arrayidx391=(($buf_digit_lookup)&4294967295); //@line 1819 "numbers.cpp"
          var $216=HEAP8[($arrayidx391)]; //@line 1819 "numbers.cpp"
          var $conv392=(tempInt=(($216)),(tempInt>=128?tempInt-256:tempInt)); //@line 1819 "numbers.cpp"
          var $cmp393=((($conv392))|0)==0; //@line 1819 "numbers.cpp"
          if (!($cmp393)) { __label__ = 112;break $if_end369$$if_end404$139; } //@line 1819 "numbers.cpp"
  
          var $217=HEAP32[(($p)>>2)]; //@line 1819 "numbers.cpp"
          var $218=HEAP8[($217)]; //@line 1819 "numbers.cpp"
          var $conv395=(tempInt=(($218)),(tempInt>=128?tempInt-256:tempInt)); //@line 1819 "numbers.cpp"
          var $cmp396=((($conv395))|0)!=48; //@line 1819 "numbers.cpp"
          if (!($cmp396)) { __label__ = 112;break $if_end369$$if_end404$139; } //@line 1819 "numbers.cpp"
  
          var $219=$tr_addr; //@line 1822 "numbers.cpp"
          var $arraydecay398=(($buf_digit_lookup)&4294967295); //@line 1822 "numbers.cpp"
          var $220=$flags_addr; //@line 1822 "numbers.cpp"
          var $221=$wtab_addr; //@line 1822 "numbers.cpp"
          var $call399=__Z14LookupDictListP10TranslatorPPcS1_PjiP8WORD_TAB($219, $p, $arraydecay398, $220, 4, $221); //@line 1822 "numbers.cpp"
          var $tobool400=((($call399))|0)!=0; //@line 1822 "numbers.cpp"
          if (!($tobool400)) { __label__ = 112;break $if_end369$$if_end404$139; } //@line 1822 "numbers.cpp"
  
          HEAP32[((__ZL14n_digit_lookup)>>2)]=1; //@line 1824 "numbers.cpp"
          ; //@line 1825 "numbers.cpp"
        }
      } while(0);
  
      var $222=$tr_addr; //@line 1829 "numbers.cpp"
      var $223=$value; //@line 1829 "numbers.cpp"
      var $arraydecay405=(($ph_buf)&4294967295); //@line 1829 "numbers.cpp"
      var $224=$suppress_null; //@line 1829 "numbers.cpp"
      var $225=$thousandplex; //@line 1829 "numbers.cpp"
      var $226=$prev_thousands; //@line 1829 "numbers.cpp"
      var $227=$ordinal; //@line 1829 "numbers.cpp"
      var $or406=($227) | ($226); //@line 1829 "numbers.cpp"
      __ZL10LookupNum3P10TranslatoriPciii($222, $223, $arraydecay405, $224, $225, $or406); //@line 1829 "numbers.cpp"
      var $228=$thousandplex; //@line 1830 "numbers.cpp"
      var $cmp408=((($228))|0) > 0; //@line 1830 "numbers.cpp"
      if ($cmp408) { __label__ = 113;; } else { __label__ = 115;; } //@line 1830 "numbers.cpp"
      $land_lhs_true409$$if_else419$154: do { 
        if (__label__ == 113) {
  
          var $229=$tr_addr; //@line 1830 "numbers.cpp"
          var $langopts410=(($229)&4294967295); //@line 1830 "numbers.cpp"
          var $numbers2=(($langopts410+188)&4294967295); //@line 1830 "numbers.cpp"
          var $230=HEAP32[(($numbers2)>>2)]; //@line 1830 "numbers.cpp"
          var $and411=($230) & 512; //@line 1830 "numbers.cpp"
          var $tobool412=((($and411))|0)!=0; //@line 1830 "numbers.cpp"
          if (!($tobool412)) { __label__ = 115;break $land_lhs_true409$$if_else419$154; } //@line 1830 "numbers.cpp"
  
          var $231=$ph_out_addr; //@line 1831 "numbers.cpp"
          var $arraydecay414=(($ph_zeros)&4294967295); //@line 1831 "numbers.cpp"
          var $arraydecay415=(($ph_append)&4294967295); //@line 1831 "numbers.cpp"
          var $arraydecay416=(($ph_buf2)&4294967295); //@line 1831 "numbers.cpp"
          var $arraydecay417=(($ph_buf)&4294967295); //@line 1831 "numbers.cpp"
          var $call418=_sprintf($231, ((__str20317)&4294967295), allocate([$arraydecay414,0,0,0,$arraydecay415,0,0,0,$arraydecay416,0,0,0,$arraydecay417,0,0,0], ["i8*",0,0,0,"i8*",0,0,0,"i8*",0,0,0,"i8*",0,0,0], ALLOC_STACK)); //@line 1831 "numbers.cpp"
          __label__ = 116;break $land_lhs_true409$$if_else419$154; //@line 1831 "numbers.cpp"
        }
      } while(0);
      if (__label__ == 115) {
  
        var $232=$ph_out_addr; //@line 1833 "numbers.cpp"
        var $arraydecay420=(($ph_zeros)&4294967295); //@line 1833 "numbers.cpp"
        var $arraydecay421=(($ph_buf2)&4294967295); //@line 1833 "numbers.cpp"
        var $arraydecay422=(($ph_buf)&4294967295); //@line 1833 "numbers.cpp"
        var $arraydecay423=(($ph_append)&4294967295); //@line 1833 "numbers.cpp"
        var $call424=_sprintf($232, ((__str20317)&4294967295), allocate([$arraydecay420,0,0,0,$arraydecay421,0,0,0,$arraydecay422,0,0,0,$arraydecay423,0,0,0], ["i8*",0,0,0,"i8*",0,0,0,"i8*",0,0,0,"i8*",0,0,0], ALLOC_STACK)); //@line 1833 "numbers.cpp"
        ;
      }
  
      var $arraydecay472=(($ph_buf)&4294967295); //@line 1870 "numbers.cpp"
      var $arraydecay480=(($string)&4294967295); //@line 1874 "numbers.cpp"
      var $arraydecay482=(($string)&4294967295); //@line 1875 "numbers.cpp"
      var $arraydecay483=(($buf1)&4294967295); //@line 1875 "numbers.cpp"
      var $arraydecay488=(($ph_buf)&4294967295); //@line 1878 "numbers.cpp"
      var $arraydecay489=(($buf1)&4294967295); //@line 1878 "numbers.cpp"
      var $arraydecay492=(($ph_buf)&4294967295); //@line 1880 "numbers.cpp"
      var $arraydecay504=(($buf1)&4294967295); //@line 1888 "numbers.cpp"
      var $arraydecay506=(($buf1)&4294967295); //@line 1889 "numbers.cpp"
      var $arraydecay463=(($buf1)&4294967295); //@line 1862 "numbers.cpp"
      var $arraydecay465=(($buf1)&4294967295); //@line 1863 "numbers.cpp"
      var $arraydecay448=(($buf1)&4294967295); //@line 1855 "numbers.cpp"
      var $arraydecay450=(($buf1)&4294967295); //@line 1856 "numbers.cpp"
      var $arraydecay524=(($buf1)&4294967295); //@line 1900 "numbers.cpp"
      var $arraydecay526=(($buf1)&4294967295); //@line 1901 "numbers.cpp"
      var $arraydecay529=(($buf1)&4294967295); //@line 1905 "numbers.cpp"
      var $arraydecay533=(($buf1)&4294967295); //@line 1906 "numbers.cpp"
      var $arraydecay547=(($buf1)&4294967295); //@line 1910 "numbers.cpp"
      var $arraydecay549=(($buf1)&4294967295); //@line 1911 "numbers.cpp"
      ; //@line 1836 "numbers.cpp"
      $while_cond426thread_pre_split$160: while(1) { 
  
        var $_pr49=$decimal_point; //@line 1836 "numbers.cpp"
        __lastLabel__ = 117; ;
        $while_cond426$162: while(1) { 
  
          var $233=__lastLabel__ == 117 ? $_pr49 : (0); //@line 1836 "numbers.cpp"
          var $tobool427=((($233))|0)!=0; //@line 1836 "numbers.cpp"
          if (!($tobool427)) { __label__ = 145;break $while_cond426thread_pre_split$160; } //@line 1836 "numbers.cpp"
  
          var $234=$n_digits; //@line 1838 "numbers.cpp"
          var $inc429=((($234)+1)&4294967295); //@line 1838 "numbers.cpp"
          $n_digits=$inc429; //@line 1838 "numbers.cpp"
          $decimal_count=0; //@line 1840 "numbers.cpp"
          var $235=$n_digits; //@line 1841 "numbers.cpp"
          var $236=$decimal_count; //@line 1841 "numbers.cpp"
          var $add43153=((($236)+($235))&4294967295); //@line 1841 "numbers.cpp"
          var $237=$word_addr; //@line 1841 "numbers.cpp"
          var $arrayidx43254=(($237+$add43153)&4294967295); //@line 1841 "numbers.cpp"
          var $238=HEAP8[($arrayidx43254)]; //@line 1841 "numbers.cpp"
          var $conv43355=(tempInt=(($238)),(tempInt>=128?tempInt-256:tempInt)); //@line 1841 "numbers.cpp"
          var $isdigittmp3456=((($conv43355)-48)&4294967295); //@line 1841 "numbers.cpp"
          var $isdigit3557=((($isdigittmp3456))>>>0) < 10; //@line 1841 "numbers.cpp"
          var $call43458=((($isdigit3557))&1); //@line 1841 "numbers.cpp"
          var $tobool43559=((($call43458))|0)!=0; //@line 1841 "numbers.cpp"
          if ($tobool43559) { __label__ = 120;; } else { __label__ = 121;; } //@line 1841 "numbers.cpp"
          $while_body436$$while_end438$165: do { 
            if (__label__ == 120) {
              while(1) { 
  
                var $239=$decimal_count; //@line 1842 "numbers.cpp"
                var $inc437=((($239)+1)&4294967295); //@line 1842 "numbers.cpp"
                $decimal_count=$inc437; //@line 1842 "numbers.cpp"
                var $240=$n_digits; //@line 1841 "numbers.cpp"
                var $241=$decimal_count; //@line 1841 "numbers.cpp"
                var $add431=((($241)+($240))&4294967295); //@line 1841 "numbers.cpp"
                var $242=$word_addr; //@line 1841 "numbers.cpp"
                var $arrayidx432=(($242+$add431)&4294967295); //@line 1841 "numbers.cpp"
                var $243=HEAP8[($arrayidx432)]; //@line 1841 "numbers.cpp"
                var $conv433=(tempInt=(($243)),(tempInt>=128?tempInt-256:tempInt)); //@line 1841 "numbers.cpp"
                var $isdigittmp34=((($conv433)-48)&4294967295); //@line 1841 "numbers.cpp"
                var $isdigit35=((($isdigittmp34))>>>0) < 10; //@line 1841 "numbers.cpp"
                var $call434=((($isdigit35))&1); //@line 1841 "numbers.cpp"
                var $tobool435=((($call434))|0)!=0; //@line 1841 "numbers.cpp"
                if ($tobool435) { __label__ = 120;continue ; } else { __label__ = 121;break $while_body436$$while_end438$165; } //@line 1841 "numbers.cpp"
              }
            }
          } while(0);
  
          $max_decimal_count=2; //@line 1846 "numbers.cpp"
          var $244=$tr_addr; //@line 1847 "numbers.cpp"
          var $langopts439=(($244)&4294967295); //@line 1847 "numbers.cpp"
          var $numbers440=(($langopts439+184)&4294967295); //@line 1847 "numbers.cpp"
          var $245=HEAP32[(($numbers440)>>2)]; //@line 1847 "numbers.cpp"
          var $and441=($245) & 57344; //@line 1847 "numbers.cpp"
          $decimal_mode=$and441; //@line 1847 "numbers.cpp"
          if ($and441 == 32768) {
            __label__ = 122;;
          }
          else if ($and441 == 16384) {
            __label__ = 123;;
          }
          else if ($and441 == 8192) {
            __label__ = 128;;
          }
          else if ($and441 == 40960) {
            __label__ = 128;;
          }
          else if ($and441 == 24576) {
            __label__ = 133;;
          }
          else {
          __label__ = 136;;
          }
          
          $while_cond510$$sw_bb$$while_cond443_preheader$$sw_bb469$$sw_bb495$169: do { 
            if (__label__ == 122) {
  
              $max_decimal_count=5; //@line 1850 "numbers.cpp"
              __label__ = 123;break $while_cond510$$sw_bb$$while_cond443_preheader$$sw_bb469$$sw_bb495$169; //@line 1850 "numbers.cpp"
            }
            else if (__label__ == 128) {
  
              var $267=$tr_addr; //@line 1870 "numbers.cpp"
              var $268=$n_digits; //@line 1870 "numbers.cpp"
              var $269=$word_addr; //@line 1870 "numbers.cpp"
              var $arrayidx470=(($269+$268)&4294967295); //@line 1870 "numbers.cpp"
              var $call471=_atoi($arrayidx470); //@line 1870 "numbers.cpp"
              __ZL10LookupNum3P10TranslatoriPciii($267, $call471, $arraydecay472, 0, 0, 0); //@line 1870 "numbers.cpp"
              var $270=$n_digits; //@line 1871 "numbers.cpp"
              var $271=$word_addr; //@line 1871 "numbers.cpp"
              var $arrayidx474=(($271+$270)&4294967295); //@line 1871 "numbers.cpp"
              var $272=HEAP8[($arrayidx474)]; //@line 1871 "numbers.cpp"
              var $conv475=(tempInt=(($272)),(tempInt>=128?tempInt-256:tempInt)); //@line 1871 "numbers.cpp"
              var $cmp476=((($conv475))|0)==48; //@line 1871 "numbers.cpp"
              if ($cmp476) { __label__ = 130;; } else { __label__ = 129;; } //@line 1871 "numbers.cpp"
              $if_then479$$lor_lhs_false477$172: do { 
                if (__label__ == 129) {
  
                  var $273=$decimal_mode; //@line 1871 "numbers.cpp"
                  var $cmp478=((($273))|0)==40960; //@line 1871 "numbers.cpp"
                  if ($cmp478) { __label__ = 130;break $if_then479$$lor_lhs_false477$172; } else { __label__ = 132;break $if_then479$$lor_lhs_false477$172; } //@line 1871 "numbers.cpp"
                }
              } while(0);
              if (__label__ == 130) {
  
                var $274=$decimal_count; //@line 1874 "numbers.cpp"
                var $call481=_sprintf($arraydecay480, ((__str21318)&4294967295), allocate([$274,0,0,0], ["i32",0,0,0], ALLOC_STACK)); //@line 1874 "numbers.cpp"
                var $275=$tr_addr; //@line 1875 "numbers.cpp"
                var $call484=__Z6LookupP10TranslatorPKcPc($275, $arraydecay482, $arraydecay483); //@line 1875 "numbers.cpp"
                var $cmp485=((($call484))|0)==0; //@line 1875 "numbers.cpp"
                if ($cmp485) { __label__ = 136;break $while_cond510$$sw_bb$$while_cond443_preheader$$sw_bb469$$sw_bb495$169; } //@line 1875 "numbers.cpp"
  
                var $call490=_strcat($arraydecay488, $arraydecay489); //@line 1878 "numbers.cpp"
                ; //@line 1879 "numbers.cpp"
              }
  
              var $276=$ph_out_addr; //@line 1880 "numbers.cpp"
              var $call493=_strcat($276, $arraydecay492); //@line 1880 "numbers.cpp"
              var $277=$decimal_count; //@line 1881 "numbers.cpp"
              var $278=$n_digits; //@line 1881 "numbers.cpp"
              var $add494=((($278)+($277))&4294967295); //@line 1881 "numbers.cpp"
              $n_digits=$add494; //@line 1881 "numbers.cpp"
              __label__ = 136;break $while_cond510$$sw_bb$$while_cond443_preheader$$sw_bb469$$sw_bb495$169; //@line 1882 "numbers.cpp"
            }
            else if (__label__ == 133) {
  
              var $279=$decimal_count; //@line 1886 "numbers.cpp"
              var $cmp496=((($279))|0) <= 4; //@line 1886 "numbers.cpp"
              if (!($cmp496)) { __label__ = 136;break $while_cond510$$sw_bb$$while_cond443_preheader$$sw_bb469$$sw_bb495$169; } //@line 1886 "numbers.cpp"
  
              var $280=$n_digits; //@line 1886 "numbers.cpp"
              var $281=$word_addr; //@line 1886 "numbers.cpp"
              var $arrayidx498=(($281+$280)&4294967295); //@line 1886 "numbers.cpp"
              var $282=HEAP8[($arrayidx498)]; //@line 1886 "numbers.cpp"
              var $conv499=(tempInt=(($282)),(tempInt>=128?tempInt-256:tempInt)); //@line 1886 "numbers.cpp"
              var $cmp500=((($conv499))|0)!=48; //@line 1886 "numbers.cpp"
              if (!($cmp500)) { __label__ = 136;break $while_cond510$$sw_bb$$while_cond443_preheader$$sw_bb469$$sw_bb495$169; } //@line 1886 "numbers.cpp"
  
              var $283=$tr_addr; //@line 1888 "numbers.cpp"
              var $284=$n_digits; //@line 1888 "numbers.cpp"
              var $285=$word_addr; //@line 1888 "numbers.cpp"
              var $arrayidx502=(($285+$284)&4294967295); //@line 1888 "numbers.cpp"
              var $call503=_atoi($arrayidx502); //@line 1888 "numbers.cpp"
              __ZL10LookupNum3P10TranslatoriPciii($283, $call503, $arraydecay504, 0, 0, 0); //@line 1888 "numbers.cpp"
              var $286=$ph_out_addr; //@line 1889 "numbers.cpp"
              var $call507=_strcat($286, $arraydecay506); //@line 1889 "numbers.cpp"
              var $287=$decimal_count; //@line 1890 "numbers.cpp"
              var $288=$n_digits; //@line 1890 "numbers.cpp"
              var $add508=((($288)+($287))&4294967295); //@line 1890 "numbers.cpp"
              $n_digits=$add508; //@line 1890 "numbers.cpp"
              __label__ = 136;break $while_cond510$$sw_bb$$while_cond443_preheader$$sw_bb469$$sw_bb495$169; //@line 1891 "numbers.cpp"
            }
          } while(0);
          $while_cond510$$while_cond443_preheader$181: do { 
            if (__label__ == 123) {
  
              var $246=$n_digits; //@line 1853 "numbers.cpp"
              var $247=$word_addr; //@line 1853 "numbers.cpp"
              var $arrayidx44450=(($247+$246)&4294967295); //@line 1853 "numbers.cpp"
              var $248=HEAP8[($arrayidx44450)]; //@line 1853 "numbers.cpp"
              var $conv44551=(tempInt=(($248)),(tempInt>=128?tempInt-256:tempInt)); //@line 1853 "numbers.cpp"
              var $cmp44652=((($conv44551))|0)==48; //@line 1853 "numbers.cpp"
              if ($cmp44652) { __label__ = 124;; } else { __label__ = 125;; } //@line 1853 "numbers.cpp"
              $while_body447$$while_end453$183: do { 
                if (__label__ == 124) {
                  while(1) { 
  
                    var $249=$tr_addr; //@line 1855 "numbers.cpp"
                    var $call449=__Z6LookupP10TranslatorPKcPc($249, ((__str16313)&4294967295), $arraydecay448); //@line 1855 "numbers.cpp"
                    var $250=$ph_out_addr; //@line 1856 "numbers.cpp"
                    var $call451=_strcat($250, $arraydecay450); //@line 1856 "numbers.cpp"
                    var $251=$decimal_count; //@line 1857 "numbers.cpp"
                    var $dec=((($251)-1)&4294967295); //@line 1857 "numbers.cpp"
                    $decimal_count=$dec; //@line 1857 "numbers.cpp"
                    var $252=$n_digits; //@line 1858 "numbers.cpp"
                    var $inc452=((($252)+1)&4294967295); //@line 1858 "numbers.cpp"
                    $n_digits=$inc452; //@line 1858 "numbers.cpp"
                    var $253=$n_digits; //@line 1853 "numbers.cpp"
                    var $254=$word_addr; //@line 1853 "numbers.cpp"
                    var $arrayidx444=(($254+$253)&4294967295); //@line 1853 "numbers.cpp"
                    var $255=HEAP8[($arrayidx444)]; //@line 1853 "numbers.cpp"
                    var $conv445=(tempInt=(($255)),(tempInt>=128?tempInt-256:tempInt)); //@line 1853 "numbers.cpp"
                    var $cmp446=((($conv445))|0)==48; //@line 1853 "numbers.cpp"
                    if ($cmp446) { __label__ = 124;continue ; } else { __label__ = 125;break $while_body447$$while_end453$183; } //@line 1853 "numbers.cpp"
                  }
                }
              } while(0);
  
              var $256=$decimal_count; //@line 1860 "numbers.cpp"
              var $257=$max_decimal_count; //@line 1860 "numbers.cpp"
              var $cmp454=((($256))|0) <= ((($257))|0); //@line 1860 "numbers.cpp"
              if (!($cmp454)) { __label__ = 136;break $while_cond510$$while_cond443_preheader$181; } //@line 1860 "numbers.cpp"
  
              var $258=$n_digits; //@line 1860 "numbers.cpp"
              var $259=$word_addr; //@line 1860 "numbers.cpp"
              var $arrayidx456=(($259+$258)&4294967295); //@line 1860 "numbers.cpp"
              var $260=HEAP8[($arrayidx456)]; //@line 1860 "numbers.cpp"
              var $conv457=(tempInt=(($260)),(tempInt>=128?tempInt-256:tempInt)); //@line 1860 "numbers.cpp"
              var $isdigittmp36=((($conv457)-48)&4294967295); //@line 1860 "numbers.cpp"
              var $isdigit37=((($isdigittmp36))>>>0) < 10; //@line 1860 "numbers.cpp"
              var $call458=((($isdigit37))&1); //@line 1860 "numbers.cpp"
              var $tobool459=((($call458))|0)!=0; //@line 1860 "numbers.cpp"
              if (!($tobool459)) { __label__ = 136;break $while_cond510$$while_cond443_preheader$181; } //@line 1860 "numbers.cpp"
  
              var $261=$tr_addr; //@line 1862 "numbers.cpp"
              var $262=$n_digits; //@line 1862 "numbers.cpp"
              var $263=$word_addr; //@line 1862 "numbers.cpp"
              var $arrayidx461=(($263+$262)&4294967295); //@line 1862 "numbers.cpp"
              var $call462=_atoi($arrayidx461); //@line 1862 "numbers.cpp"
              __ZL10LookupNum3P10TranslatoriPciii($261, $call462, $arraydecay463, 0, 0, 0); //@line 1862 "numbers.cpp"
              var $264=$ph_out_addr; //@line 1863 "numbers.cpp"
              var $call466=_strcat($264, $arraydecay465); //@line 1863 "numbers.cpp"
              var $265=$decimal_count; //@line 1864 "numbers.cpp"
              var $266=$n_digits; //@line 1864 "numbers.cpp"
              var $add467=((($266)+($265))&4294967295); //@line 1864 "numbers.cpp"
              $n_digits=$add467; //@line 1864 "numbers.cpp"
              ; //@line 1865 "numbers.cpp"
            }
          } while(0);
          while(1) { 
  
            var $289=$n_digits; //@line 1896 "numbers.cpp"
            var $290=$word_addr; //@line 1896 "numbers.cpp"
            var $arrayidx511=(($290+$289)&4294967295); //@line 1896 "numbers.cpp"
            var $291=HEAPU8[($arrayidx511)]; //@line 1896 "numbers.cpp"
            $c=$291; //@line 1896 "numbers.cpp"
            var $conv512=((($291))&255); //@line 1896 "numbers.cpp"
            var $isdigittmp38=((($conv512)-48)&4294967295); //@line 1896 "numbers.cpp"
            var $isdigit39=((($isdigittmp38))>>>0) < 10; //@line 1896 "numbers.cpp"
            var $call513=((($isdigit39))&1); //@line 1896 "numbers.cpp"
            var $tobool514=((($call513))|0)!=0; //@line 1896 "numbers.cpp"
            if (!($tobool514)) { __label__ = 139;break ; } //@line 1896 "numbers.cpp"
  
            var $292=$ph_out_addr; //@line 1896 "numbers.cpp"
            var $call516=_strlen($292); //@line 1896 "numbers.cpp"
            var $cmp517=((($call516))>>>0) < 150; //@line 1896 "numbers.cpp"
            if (!($cmp517)) { __label__ = 139;break ; }
  
            var $293=$n_digits; //@line 1899 "numbers.cpp"
            var $inc520=((($293)+1)&4294967295); //@line 1899 "numbers.cpp"
            $n_digits=$inc520; //@line 1899 "numbers.cpp"
            var $294=$word_addr; //@line 1899 "numbers.cpp"
            var $arrayidx521=(($294+$293)&4294967295); //@line 1899 "numbers.cpp"
            var $295=HEAP8[($arrayidx521)]; //@line 1899 "numbers.cpp"
            var $conv522=(tempInt=(($295)),(tempInt>=128?tempInt-256:tempInt)); //@line 1899 "numbers.cpp"
            var $sub523=((($conv522)-48)&4294967295); //@line 1899 "numbers.cpp"
            $value=$sub523; //@line 1899 "numbers.cpp"
            var $296=$tr_addr; //@line 1900 "numbers.cpp"
            var $297=$value; //@line 1900 "numbers.cpp"
            var $call525=__ZL10LookupNum2P10TranslatoriiPc($296, $297, 2, $arraydecay524); //@line 1900 "numbers.cpp"
            var $298=$ph_out_addr; //@line 1901 "numbers.cpp"
            var $call527=_strcat($298, $arraydecay526); //@line 1901 "numbers.cpp"
            __label__ = 136;continue ; //@line 1902 "numbers.cpp"
          }
  
          var $299=$tr_addr; //@line 1905 "numbers.cpp"
          var $call530=__Z6LookupP10TranslatorPKcPc($299, ((__str22319)&4294967295), $arraydecay529); //@line 1905 "numbers.cpp"
          var $tobool531=((($call530))|0)!=0; //@line 1905 "numbers.cpp"
          if ($tobool531) { __label__ = 140;; } else { __label__ = 141;; } //@line 1905 "numbers.cpp"
          if (__label__ == 140) {
  
            var $300=$ph_out_addr; //@line 1906 "numbers.cpp"
            var $call534=_strcat($300, $arraydecay533); //@line 1906 "numbers.cpp"
            ; //@line 1906 "numbers.cpp"
          }
  
          var $301=$c; //@line 1908 "numbers.cpp"
          var $conv536=((($301))&255); //@line 1908 "numbers.cpp"
          var $302=$tr_addr; //@line 1908 "numbers.cpp"
          var $langopts537=(($302)&4294967295); //@line 1908 "numbers.cpp"
          var $decimal_sep538=(($langopts537+208)&4294967295); //@line 1908 "numbers.cpp"
          var $303=HEAP32[(($decimal_sep538)>>2)]; //@line 1908 "numbers.cpp"
          var $cmp539=((($conv536))|0)==((($303))|0); //@line 1908 "numbers.cpp"
          if ($cmp539) { __label__ = 142;; } else { __label__ = 144;; } //@line 1908 "numbers.cpp"
          if (__label__ == 142) {
  
            var $304=$n_digits; //@line 1908 "numbers.cpp"
            var $add541=((($304)+1)&4294967295); //@line 1908 "numbers.cpp"
            var $305=$word_addr; //@line 1908 "numbers.cpp"
            var $arrayidx542=(($305+$add541)&4294967295); //@line 1908 "numbers.cpp"
            var $306=HEAP8[($arrayidx542)]; //@line 1908 "numbers.cpp"
            var $conv543=(tempInt=(($306)),(tempInt>=128?tempInt-256:tempInt)); //@line 1908 "numbers.cpp"
            var $isdigittmp40=((($conv543)-48)&4294967295); //@line 1908 "numbers.cpp"
            var $isdigit41=((($isdigittmp40))>>>0) < 10; //@line 1908 "numbers.cpp"
            var $call544=((($isdigit41))&1); //@line 1908 "numbers.cpp"
            var $tobool545=((($call544))|0)!=0; //@line 1908 "numbers.cpp"
            if ($tobool545) { __label__ = 143;break $while_cond426$162; } //@line 1908 "numbers.cpp"
          }
  
          $decimal_point=0; //@line 1915 "numbers.cpp"
          __lastLabel__ = 144; __label__ = 118;continue $while_cond426$162;
        }
  
        var $307=$tr_addr; //@line 1910 "numbers.cpp"
        var $call548=__Z6LookupP10TranslatorPKcPc($307, ((__str17314)&4294967295), $arraydecay547); //@line 1910 "numbers.cpp"
        var $308=$ph_out_addr; //@line 1911 "numbers.cpp"
        var $call550=_strcat($308, $arraydecay549); //@line 1911 "numbers.cpp"
        __label__ = 117;continue $while_cond426thread_pre_split$160; //@line 1912 "numbers.cpp"
      }
  
      var $309=$ph_out_addr; //@line 1918 "numbers.cpp"
      var $arrayidx554=(($309)&4294967295); //@line 1918 "numbers.cpp"
      var $310=HEAP8[($arrayidx554)]; //@line 1918 "numbers.cpp"
      var $conv555=(tempInt=(($310)),(tempInt>=128?tempInt-256:tempInt)); //@line 1918 "numbers.cpp"
      var $cmp556=((($conv555))|0)!=0; //@line 1918 "numbers.cpp"
      if ($cmp556) { __label__ = 146;; } else { __label__ = 154;; } //@line 1918 "numbers.cpp"
      $land_lhs_true557$$if_end588$202: do { 
        if (__label__ == 146) {
  
          var $311=$ph_out_addr; //@line 1918 "numbers.cpp"
          var $arrayidx558=(($311)&4294967295); //@line 1918 "numbers.cpp"
          var $312=HEAP8[($arrayidx558)]; //@line 1918 "numbers.cpp"
          var $conv559=(tempInt=(($312)),(tempInt>=128?tempInt-256:tempInt)); //@line 1918 "numbers.cpp"
          var $cmp560=((($conv559))|0)!=21; //@line 1918 "numbers.cpp"
          if (!($cmp560)) { __label__ = 154;break $land_lhs_true557$$if_end588$202; } //@line 1918 "numbers.cpp"
  
          var $313=$n_digits; //@line 1922 "numbers.cpp"
          var $add563=((($313)+1)&4294967295); //@line 1922 "numbers.cpp"
          var $314=$word_addr; //@line 1922 "numbers.cpp"
          var $arrayidx564=(($314+$add563)&4294967295); //@line 1922 "numbers.cpp"
          $p562=$arrayidx564; //@line 1922 "numbers.cpp"
          var $315=$p562; //@line 1924 "numbers.cpp"
          var $call565=__Z7utf8_inPiPKc($next_char, $315); //@line 1924 "numbers.cpp"
          var $316=$p562; //@line 1924 "numbers.cpp"
          var $add_ptr566=(($316+$call565)&4294967295); //@line 1924 "numbers.cpp"
          $p562=$add_ptr566; //@line 1924 "numbers.cpp"
          var $317=$tr_addr; //@line 1925 "numbers.cpp"
          var $langopts567=(($317)&4294967295); //@line 1925 "numbers.cpp"
          var $numbers568=(($langopts567+184)&4294967295); //@line 1925 "numbers.cpp"
          var $318=HEAP32[(($numbers568)>>2)]; //@line 1925 "numbers.cpp"
          var $and569=($318) & 131072; //@line 1925 "numbers.cpp"
          var $tobool570=((($and569))|0)!=0; //@line 1925 "numbers.cpp"
          if ($tobool570) { __label__ = 148;; } else { __label__ = 150;; } //@line 1925 "numbers.cpp"
          $land_lhs_true571$$if_end575$205: do { 
            if (__label__ == 148) {
  
              var $319=HEAP32[(($next_char)>>2)]; //@line 1925 "numbers.cpp"
              var $cmp572=((($319))|0)==32; //@line 1925 "numbers.cpp"
              if (!($cmp572)) { __label__ = 150;break $land_lhs_true571$$if_end575$205; } //@line 1925 "numbers.cpp"
  
              var $320=$p562; //@line 1926 "numbers.cpp"
              var $call574=__Z7utf8_inPiPKc($next_char, $320); //@line 1926 "numbers.cpp"
              ; //@line 1926 "numbers.cpp"
            }
          } while(0);
  
          var $321=HEAP32[(($next_char)>>2)]; //@line 1928 "numbers.cpp"
          var $call576=_iswalpha($321); //@line 1928 "numbers.cpp"
          var $tobool577=((($call576))|0)!=0; //@line 1928 "numbers.cpp"
          if ($tobool577) { __label__ = 154;break $land_lhs_true557$$if_end588$202; } //@line 1928 "numbers.cpp"
  
          var $322=$thousandplex; //@line 1928 "numbers.cpp"
          var $323=$wtab_addr; //@line 1928 "numbers.cpp"
          var $arrayidx579=(($323+12*$322)&4294967295); //@line 1928 "numbers.cpp"
          var $flags580=(($arrayidx579)&4294967295); //@line 1928 "numbers.cpp"
          var $324=HEAP32[(($flags580)>>2)]; //@line 1928 "numbers.cpp"
          var $and581=($324) & 16384; //@line 1928 "numbers.cpp"
          var $tobool582=((($and581))|0)!=0; //@line 1928 "numbers.cpp"
          if ($tobool582) { __label__ = 152;; } else { __label__ = 153;; } //@line 1928 "numbers.cpp"
          if (__label__ == 152) {
  
            var $325=$thousands_exact; //@line 1928 "numbers.cpp"
            var $cmp584=((($325))|0)!=0; //@line 1928 "numbers.cpp"
            if ($cmp584) { __label__ = 154;break $land_lhs_true557$$if_end588$202; } //@line 1928 "numbers.cpp"
          }
  
          var $326=$ph_out_addr; //@line 1929 "numbers.cpp"
          var $strlen=_strlen($326); //@line 1929 "numbers.cpp"
          var $endptr=(($326+$strlen)&4294967295); //@line 1929 "numbers.cpp"
          var $src$, $dest$, $stop$, $stop4$, $fast$;
          $src$ = ((__ZZL17TranslateNumber_1P10TranslatorPcS1_PjP8WORD_TABiE9str_pause)&4294967295);
          $dest$ = $endptr;
          $stop$ = $src$ + 2;
          if (($dest$%4) == ($src$%4) && 2 > 8) {
            while ($src$%4 !== 0 && $src$ < $stop$) {
              ; HEAP8[$dest$++] = HEAP8[$src$++];
            }
            $src$ >>= 2;
            $dest$ >>= 2;
            $stop4$ = $stop$ >> 2;
            while ($src$ < $stop4$) {
          ;;;    HEAP32[$dest$++] = HEAP32[$src$++];
            }
            $src$ <<= 2;
            $dest$ <<= 2;
          }while ($src$ < $stop$) {
            ; HEAP8[$dest$++] = HEAP8[$src$++];
          }; //@line 1929 "numbers.cpp"
          ; //@line 1929 "numbers.cpp"
        }
      } while(0);
  
      var $327=$flags_addr; //@line 1932 "numbers.cpp"
      var $328=HEAP32[(($327)>>2)]; //@line 1932 "numbers.cpp"
      var $or589=($328) | -2147483648; //@line 1932 "numbers.cpp"
      HEAP32[(($327)>>2)]=$or589; //@line 1932 "numbers.cpp"
      var $329=HEAP32[((__ZL23speak_missing_thousands)>>2)]; //@line 1933 "numbers.cpp"
      var $dec590=((($329)-1)&4294967295); //@line 1933 "numbers.cpp"
      HEAP32[((__ZL23speak_missing_thousands)>>2)]=$dec590; //@line 1933 "numbers.cpp"
      var $330=$skipwords; //@line 1935 "numbers.cpp"
      var $tobool591=((($330))|0)!=0; //@line 1935 "numbers.cpp"
      if ($tobool591) { __label__ = 155;; } else { __label__ = 156;; } //@line 1935 "numbers.cpp"
      if (__label__ == 155) {
  
        var $331=$skipwords; //@line 1936 "numbers.cpp"
        HEAP32[((_dictionary_skipwords)>>2)]=$331; //@line 1936 "numbers.cpp"
        ; //@line 1936 "numbers.cpp"
      }
  
      $retval=1; //@line 1937 "numbers.cpp"
      ; //@line 1937 "numbers.cpp"
    }
  
    var $332=$retval; //@line 1938 "numbers.cpp"
    STACKTOP = __stackBase__;
    return $332; //@line 1938 "numbers.cpp"
    return null;
  }
  

  function __ZL15LookupThousandsP10TranslatoriiiPc($tr, $value, $thousandplex, $thousands_exact, $ph_out) {
    var __stackBase__  = STACKTOP; STACKTOP += 104; _memset(__stackBase__, 0, 104);
    var __label__;
    var __lastLabel__ = null;
  
    var $retval;
    var $tr_addr;
    var $value_addr;
    var $thousandplex_addr;
    var $thousands_exact_addr;
    var $ph_out_addr;
    var $found;
    var $found_value;
    var $string=__stackBase__;
    var $ph_of=__stackBase__+12;
    var $ph_thousands=__stackBase__+24;
    var $ph_buf=__stackBase__+64;
    $tr_addr=$tr;
    $value_addr=$value;
    $thousandplex_addr=$thousandplex;
    $thousands_exact_addr=$thousands_exact;
    $ph_out_addr=$ph_out;
    $found_value=0; //@line 991 "numbers.cpp"
    var $arrayidx=(($ph_of)&4294967295); //@line 997 "numbers.cpp"
    HEAP8[($arrayidx)]=0; //@line 997 "numbers.cpp"
    var $0=$value_addr; //@line 1000 "numbers.cpp"
    var $cmp=((($0))|0) > 0; //@line 1000 "numbers.cpp"
    if ($cmp) { __label__ = 1;; } else { __label__ = 12;; } //@line 1000 "numbers.cpp"
    $if_then$$if_end36thread_pre_split$2: do { 
      if (__label__ == 1) {
  
        var $1=$thousands_exact_addr; //@line 1002 "numbers.cpp"
        var $and=($1) & 1; //@line 1002 "numbers.cpp"
        var $tobool=((($and))|0)!=0; //@line 1002 "numbers.cpp"
        if ($tobool) { __label__ = 2;; } else { __label__ = 9;; } //@line 1002 "numbers.cpp"
        $if_then1$$if_end27thread_pre_split$4: do { 
          if (__label__ == 2) {
  
            var $2=$thousands_exact_addr; //@line 1004 "numbers.cpp"
            var $and2=($2) & 2; //@line 1004 "numbers.cpp"
            var $tobool3=((($and2))|0)!=0; //@line 1004 "numbers.cpp"
            if ($tobool3) { __label__ = 3;; } else { __label__ = 4;; } //@line 1004 "numbers.cpp"
            if (__label__ == 3) {
  
              var $arraydecay=(($string)&4294967295); //@line 1007 "numbers.cpp"
              var $3=$value_addr; //@line 1007 "numbers.cpp"
              var $4=$thousandplex_addr; //@line 1007 "numbers.cpp"
              var $call=_sprintf($arraydecay, ((__str42339)&4294967295), allocate([$3,0,0,0,$4,0,0,0], ["i32",0,0,0,"i32",0,0,0], ALLOC_STACK)); //@line 1007 "numbers.cpp"
              var $5=$tr_addr; //@line 1008 "numbers.cpp"
              var $arraydecay5=(($string)&4294967295); //@line 1008 "numbers.cpp"
              var $arraydecay6=(($ph_thousands)&4294967295); //@line 1008 "numbers.cpp"
              var $call7=__Z6LookupP10TranslatorPKcPc($5, $arraydecay5, $arraydecay6); //@line 1008 "numbers.cpp"
              $found_value=$call7; //@line 1008 "numbers.cpp"
              ; //@line 1009 "numbers.cpp"
            }
  
            var $6=$found_value; //@line 1010 "numbers.cpp"
            var $tobool8=((($6))|0)!=0; //@line 1010 "numbers.cpp"
            var $lnot=($tobool8) ^ 1; //@line 1010 "numbers.cpp"
            var $conv=((($lnot))&1); //@line 1010 "numbers.cpp"
            var $7=HEAP32[((__ZL14number_control)>>2)]; //@line 1010 "numbers.cpp"
            var $and9=($7) & 1; //@line 1010 "numbers.cpp"
            var $and10=($and9) & ($conv); //@line 1010 "numbers.cpp"
            var $tobool11=((($and10))|0)!=0; //@line 1010 "numbers.cpp"
            if ($tobool11) { __label__ = 5;; } else { __label__ = 6;; } //@line 1010 "numbers.cpp"
            if (__label__ == 5) {
  
              var $arraydecay13=(($string)&4294967295); //@line 1013 "numbers.cpp"
              var $8=$value_addr; //@line 1013 "numbers.cpp"
              var $9=$thousandplex_addr; //@line 1013 "numbers.cpp"
              var $call14=_sprintf($arraydecay13, ((__str43340)&4294967295), allocate([$8,0,0,0,$9,0,0,0], ["i32",0,0,0,"i32",0,0,0], ALLOC_STACK)); //@line 1013 "numbers.cpp"
              var $10=$tr_addr; //@line 1014 "numbers.cpp"
              var $arraydecay15=(($string)&4294967295); //@line 1014 "numbers.cpp"
              var $arraydecay16=(($ph_thousands)&4294967295); //@line 1014 "numbers.cpp"
              var $call17=__Z6LookupP10TranslatorPKcPc($10, $arraydecay15, $arraydecay16); //@line 1014 "numbers.cpp"
              $found_value=$call17; //@line 1014 "numbers.cpp"
              __lastLabel__ = 5; ; //@line 1015 "numbers.cpp"
            }
            else if (__label__ == 6) {
  
              var $_pr=$found_value; //@line 1016 "numbers.cpp"
              __lastLabel__ = 6; ;
            }
  
            var $11=__lastLabel__ == 6 ? $_pr : ($call17); //@line 1016 "numbers.cpp"
            var $tobool19=((($11))|0)!=0; //@line 1016 "numbers.cpp"
            if ($tobool19) { __label__ = 9;break $if_then1$$if_end27thread_pre_split$4; } //@line 1016 "numbers.cpp"
  
            var $arraydecay21=(($string)&4294967295); //@line 1019 "numbers.cpp"
            var $12=$value_addr; //@line 1019 "numbers.cpp"
            var $13=$thousandplex_addr; //@line 1019 "numbers.cpp"
            var $call22=_sprintf($arraydecay21, ((__str44341)&4294967295), allocate([$12,0,0,0,$13,0,0,0], ["i32",0,0,0,"i32",0,0,0], ALLOC_STACK)); //@line 1019 "numbers.cpp"
            var $14=$tr_addr; //@line 1020 "numbers.cpp"
            var $arraydecay23=(($string)&4294967295); //@line 1020 "numbers.cpp"
            var $arraydecay24=(($ph_thousands)&4294967295); //@line 1020 "numbers.cpp"
            var $call25=__Z6LookupP10TranslatorPKcPc($14, $arraydecay23, $arraydecay24); //@line 1020 "numbers.cpp"
            $found_value=$call25; //@line 1020 "numbers.cpp"
            __lastLabel__ = 8; __label__ = 10;break $if_then1$$if_end27thread_pre_split$4; //@line 1021 "numbers.cpp"
          }
        } while(0);
        if (__label__ == 9) {
  
          var $_pr1=$found_value; //@line 1023 "numbers.cpp"
          __lastLabel__ = 9; ;
        }
  
        var $15=__lastLabel__ == 9 ? $_pr1 : ($call25); //@line 1023 "numbers.cpp"
        var $cmp28=((($15))|0)==0; //@line 1023 "numbers.cpp"
        if (!($cmp28)) { __label__ = 12;break $if_then$$if_end36thread_pre_split$2; } //@line 1023 "numbers.cpp"
  
        var $arraydecay30=(($string)&4294967295); //@line 1025 "numbers.cpp"
        var $16=$value_addr; //@line 1025 "numbers.cpp"
        var $17=$thousandplex_addr; //@line 1025 "numbers.cpp"
        var $call31=_sprintf($arraydecay30, ((__str45342)&4294967295), allocate([$16,0,0,0,$17,0,0,0], ["i32",0,0,0,"i32",0,0,0], ALLOC_STACK)); //@line 1025 "numbers.cpp"
        var $18=$tr_addr; //@line 1026 "numbers.cpp"
        var $arraydecay32=(($string)&4294967295); //@line 1026 "numbers.cpp"
        var $arraydecay33=(($ph_thousands)&4294967295); //@line 1026 "numbers.cpp"
        var $call34=__Z6LookupP10TranslatorPKcPc($18, $arraydecay32, $arraydecay33); //@line 1026 "numbers.cpp"
        $found_value=$call34; //@line 1026 "numbers.cpp"
        __lastLabel__ = 11; __label__ = 13;break $if_then$$if_end36thread_pre_split$2; //@line 1027 "numbers.cpp"
      }
    } while(0);
    if (__label__ == 12) {
  
      var $_pr2=$found_value; //@line 1030 "numbers.cpp"
      __lastLabel__ = 12; ;
    }
  
    var $19=__lastLabel__ == 12 ? $_pr2 : ($call34); //@line 1030 "numbers.cpp"
    var $cmp37=((($19))|0)==0; //@line 1030 "numbers.cpp"
    if ($cmp37) { __label__ = 14;; } else { __label__ = 36;; } //@line 1030 "numbers.cpp"
    $if_then38$$if_end118$21: do { 
      if (__label__ == 14) {
  
        var $20=$value_addr; //@line 1032 "numbers.cpp"
        var $rem=((($20))|0)%100; //@line 1032 "numbers.cpp"
        var $cmp39=((($rem))|0) >= 20; //@line 1032 "numbers.cpp"
        if ($cmp39) { __label__ = 15;; } else { __label__ = 16;; } //@line 1032 "numbers.cpp"
        if (__label__ == 15) {
  
          var $21=$tr_addr; //@line 1034 "numbers.cpp"
          var $arraydecay41=(($ph_of)&4294967295); //@line 1034 "numbers.cpp"
          var $call42=__Z6LookupP10TranslatorPKcPc($21, ((__str46343)&4294967295), $arraydecay41); //@line 1034 "numbers.cpp"
          ; //@line 1035 "numbers.cpp"
        }
  
        $found=0; //@line 1037 "numbers.cpp"
        var $22=$thousands_exact_addr; //@line 1038 "numbers.cpp"
        var $and44=($22) & 1; //@line 1038 "numbers.cpp"
        var $tobool45=((($and44))|0)!=0; //@line 1038 "numbers.cpp"
        if ($tobool45) { __label__ = 17;; } else { __label__ = 28;; } //@line 1038 "numbers.cpp"
        if (__label__ == 17) {
  
          var $23=$thousands_exact_addr; //@line 1040 "numbers.cpp"
          var $and47=($23) & 2; //@line 1040 "numbers.cpp"
          var $tobool48=((($and47))|0)!=0; //@line 1040 "numbers.cpp"
          if ($tobool48) { __label__ = 18;; } else { __label__ = 19;; } //@line 1040 "numbers.cpp"
          if (__label__ == 18) {
  
            var $arraydecay50=(($string)&4294967295); //@line 1043 "numbers.cpp"
            var $24=$value_addr; //@line 1043 "numbers.cpp"
            var $call51=__ZL9M_Varianti($24); //@line 1043 "numbers.cpp"
            var $25=$thousandplex_addr; //@line 1043 "numbers.cpp"
            var $call52=_sprintf($arraydecay50, ((__str47344)&4294967295), allocate([$call51,0,0,0,$25,0,0,0], ["i8*",0,0,0,"i32",0,0,0], ALLOC_STACK)); //@line 1043 "numbers.cpp"
            var $26=$tr_addr; //@line 1044 "numbers.cpp"
            var $arraydecay53=(($string)&4294967295); //@line 1044 "numbers.cpp"
            var $arraydecay54=(($ph_thousands)&4294967295); //@line 1044 "numbers.cpp"
            var $call55=__Z6LookupP10TranslatorPKcPc($26, $arraydecay53, $arraydecay54); //@line 1044 "numbers.cpp"
            $found=$call55; //@line 1044 "numbers.cpp"
            __lastLabel__ = 18; ; //@line 1045 "numbers.cpp"
          }
          else if (__label__ == 19) {
  
            var $_pr3=$found; //@line 1046 "numbers.cpp"
            __lastLabel__ = 19; ;
          }
  
          var $27=__lastLabel__ == 19 ? $_pr3 : ($call55); //@line 1046 "numbers.cpp"
          var $tobool57=((($27))|0)!=0; //@line 1046 "numbers.cpp"
          if ($tobool57) { __label__ = 23;; } else { __label__ = 21;; } //@line 1046 "numbers.cpp"
          $if_end67thread_pre_split$$land_lhs_true$32: do { 
            if (__label__ == 21) {
  
              var $28=HEAP32[((__ZL14number_control)>>2)]; //@line 1046 "numbers.cpp"
              var $and58=($28) & 1; //@line 1046 "numbers.cpp"
              var $tobool59=((($and58))|0)!=0; //@line 1046 "numbers.cpp"
              if (!($tobool59)) { __label__ = 23;break $if_end67thread_pre_split$$land_lhs_true$32; } //@line 1046 "numbers.cpp"
  
              var $arraydecay61=(($string)&4294967295); //@line 1049 "numbers.cpp"
              var $29=$value_addr; //@line 1049 "numbers.cpp"
              var $call62=__ZL9M_Varianti($29); //@line 1049 "numbers.cpp"
              var $30=$thousandplex_addr; //@line 1049 "numbers.cpp"
              var $call63=_sprintf($arraydecay61, ((__str48345)&4294967295), allocate([$call62,0,0,0,$30,0,0,0], ["i8*",0,0,0,"i32",0,0,0], ALLOC_STACK)); //@line 1049 "numbers.cpp"
              var $31=$tr_addr; //@line 1050 "numbers.cpp"
              var $arraydecay64=(($string)&4294967295); //@line 1050 "numbers.cpp"
              var $arraydecay65=(($ph_thousands)&4294967295); //@line 1050 "numbers.cpp"
              var $call66=__Z6LookupP10TranslatorPKcPc($31, $arraydecay64, $arraydecay65); //@line 1050 "numbers.cpp"
              $found=$call66; //@line 1050 "numbers.cpp"
              __lastLabel__ = 22; __label__ = 24;break $if_end67thread_pre_split$$land_lhs_true$32; //@line 1051 "numbers.cpp"
            }
          } while(0);
          if (__label__ == 23) {
  
            var $_pr4=$found; //@line 1052 "numbers.cpp"
            __lastLabel__ = 23; ;
          }
  
          var $32=__lastLabel__ == 23 ? $_pr4 : ($call66); //@line 1052 "numbers.cpp"
          var $tobool68=((($32))|0)!=0; //@line 1052 "numbers.cpp"
          if ($tobool68) { __label__ = 26;; } else { __label__ = 25;; } //@line 1052 "numbers.cpp"
          if (__label__ == 26) {
  
            var $_pr5=$found; //@line 1059 "numbers.cpp"
            __lastLabel__ = 26; ;
          }
          else if (__label__ == 25) {
  
            var $arraydecay70=(($string)&4294967295); //@line 1055 "numbers.cpp"
            var $33=$value_addr; //@line 1055 "numbers.cpp"
            var $call71=__ZL9M_Varianti($33); //@line 1055 "numbers.cpp"
            var $34=$thousandplex_addr; //@line 1055 "numbers.cpp"
            var $call72=_sprintf($arraydecay70, ((__str49346)&4294967295), allocate([$call71,0,0,0,$34,0,0,0], ["i8*",0,0,0,"i32",0,0,0], ALLOC_STACK)); //@line 1055 "numbers.cpp"
            var $35=$tr_addr; //@line 1056 "numbers.cpp"
            var $arraydecay73=(($string)&4294967295); //@line 1056 "numbers.cpp"
            var $arraydecay74=(($ph_thousands)&4294967295); //@line 1056 "numbers.cpp"
            var $call75=__Z6LookupP10TranslatorPKcPc($35, $arraydecay73, $arraydecay74); //@line 1056 "numbers.cpp"
            $found=$call75; //@line 1056 "numbers.cpp"
            __lastLabel__ = 25; ; //@line 1057 "numbers.cpp"
          }
  
          var $36=__lastLabel__ == 26 ? $_pr5 : ($call75); //@line 1059 "numbers.cpp"
          var $cmp78=((($36))|0)==0; //@line 1059 "numbers.cpp"
          if (!($cmp78)) { __label__ = 36;break $if_then38$$if_end118$21; } //@line 1059 "numbers.cpp"
        }
  
        var $arraydecay80=(($string)&4294967295); //@line 1061 "numbers.cpp"
        var $37=$value_addr; //@line 1061 "numbers.cpp"
        var $call81=__ZL9M_Varianti($37); //@line 1061 "numbers.cpp"
        var $38=$thousandplex_addr; //@line 1061 "numbers.cpp"
        var $call82=_sprintf($arraydecay80, ((__str50347)&4294967295), allocate([$call81,0,0,0,$38,0,0,0], ["i8*",0,0,0,"i32",0,0,0], ALLOC_STACK)); //@line 1061 "numbers.cpp"
        var $39=$tr_addr; //@line 1063 "numbers.cpp"
        var $arraydecay83=(($string)&4294967295); //@line 1063 "numbers.cpp"
        var $arraydecay84=(($ph_thousands)&4294967295); //@line 1063 "numbers.cpp"
        var $call85=__Z6LookupP10TranslatorPKcPc($39, $arraydecay83, $arraydecay84); //@line 1063 "numbers.cpp"
        var $cmp86=((($call85))|0)==0; //@line 1063 "numbers.cpp"
        if (!($cmp86)) { __label__ = 36;break $if_then38$$if_end118$21; } //@line 1063 "numbers.cpp"
  
        var $40=$thousandplex_addr; //@line 1065 "numbers.cpp"
        var $cmp88=((($40))|0) > 3; //@line 1065 "numbers.cpp"
        if ($cmp88) { __label__ = 30;; } else { __label__ = 32;; } //@line 1065 "numbers.cpp"
        $if_then89$$if_end100$44: do { 
          if (__label__ == 30) {
  
            var $arraydecay90=(($string)&4294967295); //@line 1067 "numbers.cpp"
            var $41=$thousandplex_addr; //@line 1067 "numbers.cpp"
            var $sub=((($41)-1)&4294967295); //@line 1067 "numbers.cpp"
            var $call91=_sprintf($arraydecay90, ((__str18315)&4294967295), allocate([$sub,0,0,0], ["i32",0,0,0], ALLOC_STACK)); //@line 1067 "numbers.cpp"
            var $42=$tr_addr; //@line 1068 "numbers.cpp"
            var $arraydecay92=(($string)&4294967295); //@line 1068 "numbers.cpp"
            var $arraydecay93=(($ph_buf)&4294967295); //@line 1068 "numbers.cpp"
            var $call94=__Z6LookupP10TranslatorPKcPc($42, $arraydecay92, $arraydecay93); //@line 1068 "numbers.cpp"
            var $cmp95=((($call94))|0)==0; //@line 1068 "numbers.cpp"
            if (!($cmp95)) { __label__ = 32;break $if_then89$$if_end100$44; } //@line 1068 "numbers.cpp"
  
            var $43=$tr_addr; //@line 1071 "numbers.cpp"
            var $arraydecay97=(($ph_thousands)&4294967295); //@line 1071 "numbers.cpp"
            var $call98=__Z6LookupP10TranslatorPKcPc($43, ((__str51348)&4294967295), $arraydecay97); //@line 1071 "numbers.cpp"
            HEAP32[((__ZL23speak_missing_thousands)>>2)]=3; //@line 1072 "numbers.cpp"
            ; //@line 1073 "numbers.cpp"
          }
        } while(0);
  
        var $arrayidx101=(($ph_thousands)&4294967295); //@line 1075 "numbers.cpp"
        var $44=HEAP8[($arrayidx101)]; //@line 1075 "numbers.cpp"
        var $conv102=(tempInt=(($44)),(tempInt>=128?tempInt-256:tempInt)); //@line 1075 "numbers.cpp"
        var $cmp103=((($conv102))|0)==0; //@line 1075 "numbers.cpp"
        if (!($cmp103)) { __label__ = 36;break $if_then38$$if_end118$21; } //@line 1075 "numbers.cpp"
  
        var $arraydecay105=(($string)&4294967295); //@line 1078 "numbers.cpp"
        var $45=$value_addr; //@line 1078 "numbers.cpp"
        var $call106=_sprintf($arraydecay105, ((__str52349)&4294967295), allocate([$45,0,0,0], ["i32",0,0,0], ALLOC_STACK)); //@line 1078 "numbers.cpp"
        var $46=$tr_addr; //@line 1079 "numbers.cpp"
        var $arraydecay107=(($string)&4294967295); //@line 1079 "numbers.cpp"
        var $arraydecay108=(($ph_thousands)&4294967295); //@line 1079 "numbers.cpp"
        var $call109=__Z6LookupP10TranslatorPKcPc($46, $arraydecay107, $arraydecay108); //@line 1079 "numbers.cpp"
        $found_value=$call109; //@line 1079 "numbers.cpp"
        var $cmp110=((($call109))|0)==0; //@line 1079 "numbers.cpp"
        if ($cmp110) { __label__ = 34;; } else { __label__ = 35;; } //@line 1079 "numbers.cpp"
        if (__label__ == 34) {
  
          var $47=$tr_addr; //@line 1080 "numbers.cpp"
          var $arraydecay112=(($ph_thousands)&4294967295); //@line 1080 "numbers.cpp"
          var $call113=__Z6LookupP10TranslatorPKcPc($47, ((__str53350)&4294967295), $arraydecay112); //@line 1080 "numbers.cpp"
          ; //@line 1080 "numbers.cpp"
        }
  
        HEAP32[((__ZL23speak_missing_thousands)>>2)]=2; //@line 1081 "numbers.cpp"
        ; //@line 1082 "numbers.cpp"
      }
    } while(0);
  
    var $48=$ph_out_addr; //@line 1086 "numbers.cpp"
    var $arraydecay119=(($ph_of)&4294967295); //@line 1086 "numbers.cpp"
    var $arraydecay120=(($ph_thousands)&4294967295); //@line 1086 "numbers.cpp"
    var $call121=_sprintf($48, ((__str47699)&4294967295), allocate([$arraydecay119,0,0,0,$arraydecay120,0,0,0], ["i8*",0,0,0,"i8*",0,0,0], ALLOC_STACK)); //@line 1086 "numbers.cpp"
    var $49=$value_addr; //@line 1088 "numbers.cpp"
    var $cmp122=((($49))|0)==1; //@line 1088 "numbers.cpp"
    if ($cmp122) { __label__ = 37;; } else { __label__ = 40;; } //@line 1088 "numbers.cpp"
    $land_lhs_true123$$if_end129$53: do { 
      if (__label__ == 37) {
  
        var $50=$thousandplex_addr; //@line 1088 "numbers.cpp"
        var $cmp124=((($50))|0)==1; //@line 1088 "numbers.cpp"
        if (!($cmp124)) { __label__ = 40;break $land_lhs_true123$$if_end129$53; } //@line 1088 "numbers.cpp"
  
        var $51=$tr_addr; //@line 1088 "numbers.cpp"
        var $langopts=(($51)&4294967295); //@line 1088 "numbers.cpp"
        var $numbers=(($langopts+184)&4294967295); //@line 1088 "numbers.cpp"
        var $52=HEAP32[(($numbers)>>2)]; //@line 1088 "numbers.cpp"
        var $and126=($52) & 2097152; //@line 1088 "numbers.cpp"
        var $tobool127=((($and126))|0)!=0; //@line 1088 "numbers.cpp"
        if (!($tobool127)) { __label__ = 40;break $land_lhs_true123$$if_end129$53; } //@line 1088 "numbers.cpp"
  
        $retval=1; //@line 1089 "numbers.cpp"
        __label__ = 41;break $land_lhs_true123$$if_end129$53; //@line 1089 "numbers.cpp"
      }
    } while(0);
    if (__label__ == 40) {
  
      var $53=$found_value; //@line 1091 "numbers.cpp"
      $retval=$53; //@line 1091 "numbers.cpp"
      ; //@line 1091 "numbers.cpp"
    }
  
    var $54=$retval; //@line 1092 "numbers.cpp"
    STACKTOP = __stackBase__;
    return $54; //@line 1092 "numbers.cpp"
    return null;
  }
  

  function __ZL10LookupNum3P10TranslatoriPciii($tr, $value, $ph_out, $suppress_null, $thousandplex, $control) {
    var __stackBase__  = STACKTOP; STACKTOP += 380; _memset(__stackBase__, 0, 380);
    var __label__;
    var __lastLabel__ = null;
  
    var $tr_addr;
    var $value_addr;
    var $ph_out_addr;
    var $suppress_null_addr;
    var $thousandplex_addr;
    var $control_addr;
    var $found;
    var $hundreds;
    var $tensunits;
    var $x;
    var $ix;
    var $exact;
    var $ordinal;
    var $say_zero_hundred;
    var $string=__stackBase__;
    var $buf1=__stackBase__+12;
    var $buf2=__stackBase__+112;
    var $ph_100=__stackBase__+212;
    var $ph_10T=__stackBase__+232;
    var $ph_digits=__stackBase__+252;
    var $ph_thousands=__stackBase__+304;
    var $ph_hundred_and=__stackBase__+356;
    var $ph_thousand_and=__stackBase__+368;
    $tr_addr=$tr;
    $value_addr=$value;
    $ph_out_addr=$ph_out;
    $suppress_null_addr=$suppress_null;
    $thousandplex_addr=$thousandplex;
    $control_addr=$control;
    $say_zero_hundred=0; //@line 1364 "numbers.cpp"
    var $0=$control_addr; //@line 1375 "numbers.cpp"
    var $and=($0) & 34; //@line 1375 "numbers.cpp"
    $ordinal=$and; //@line 1375 "numbers.cpp"
    var $1=$value_addr; //@line 1376 "numbers.cpp"
    var $div=((((($1))|0)/100)|0); //@line 1376 "numbers.cpp"
    $hundreds=$div; //@line 1376 "numbers.cpp"
    var $2=$value_addr; //@line 1377 "numbers.cpp"
    var $rem=((($2))|0)%100; //@line 1377 "numbers.cpp"
    $tensunits=$rem; //@line 1377 "numbers.cpp"
    var $arrayidx=(($buf1)&4294967295); //@line 1378 "numbers.cpp"
    HEAP8[($arrayidx)]=0; //@line 1378 "numbers.cpp"
    var $arrayidx1=(($ph_thousands)&4294967295); //@line 1380 "numbers.cpp"
    HEAP8[($arrayidx1)]=0; //@line 1380 "numbers.cpp"
    var $arrayidx2=(($ph_thousand_and)&4294967295); //@line 1381 "numbers.cpp"
    HEAP8[($arrayidx2)]=0; //@line 1381 "numbers.cpp"
    var $3=$tr_addr; //@line 1383 "numbers.cpp"
    var $langopts=(($3)&4294967295); //@line 1383 "numbers.cpp"
    var $numbers=(($langopts+184)&4294967295); //@line 1383 "numbers.cpp"
    var $4=HEAP32[(($numbers)>>2)]; //@line 1383 "numbers.cpp"
    var $and3=($4) & 4194304; //@line 1383 "numbers.cpp"
    var $tobool=((($and3))|0)!=0; //@line 1383 "numbers.cpp"
    if ($tobool) { __label__ = 1;; } else { __label__ = 4;; } //@line 1383 "numbers.cpp"
    $land_lhs_true$$if_endthread_pre_split$2: do { 
      if (__label__ == 1) {
  
        var $5=$control_addr; //@line 1383 "numbers.cpp"
        var $and4=($5) & 1; //@line 1383 "numbers.cpp"
        var $tobool5=((($and4))|0)!=0; //@line 1383 "numbers.cpp"
        if ($tobool5) { __label__ = 3;; } else { __label__ = 2;; } //@line 1383 "numbers.cpp"
        if (__label__ == 2) {
  
          var $6=$hundreds; //@line 1383 "numbers.cpp"
          var $cmp=((($6))|0) >= 10; //@line 1383 "numbers.cpp"
          if ($cmp) { __lastLabel__ = 2; ; } else { __lastLabel__ = 2; __label__ = 5;break $land_lhs_true$$if_endthread_pre_split$2; } //@line 1383 "numbers.cpp"
        }
  
        $say_zero_hundred=1; //@line 1385 "numbers.cpp"
        __label__ = 4;break $land_lhs_true$$if_endthread_pre_split$2; //@line 1386 "numbers.cpp"
      }
    } while(0);
    if (__label__ == 4) {
  
      var $_pr=$hundreds; //@line 1388 "numbers.cpp"
      __lastLabel__ = 4; ;
    }
  
    var $7=__lastLabel__ == 4 ? $_pr : ($6); //@line 1388 "numbers.cpp"
    var $cmp6=((($7))|0) > 0; //@line 1388 "numbers.cpp"
    if ($cmp6) { __label__ = 7;; } else { __label__ = 6;; } //@line 1388 "numbers.cpp"
    $if_then9$$lor_lhs_false7$10: do { 
      if (__label__ == 6) {
  
        var $8=$say_zero_hundred; //@line 1388 "numbers.cpp"
        var $tobool8=((($8))|0)!=0; //@line 1388 "numbers.cpp"
        if ($tobool8) { __label__ = 7;break $if_then9$$lor_lhs_false7$10; } else { __label__ = 61;break $if_then9$$lor_lhs_false7$10; } //@line 1388 "numbers.cpp"
      }
    } while(0);
    if (__label__ == 7) {
  
      $found=0; //@line 1390 "numbers.cpp"
      var $9=$ordinal; //@line 1391 "numbers.cpp"
      var $tobool10=((($9))|0)!=0; //@line 1391 "numbers.cpp"
      if ($tobool10) { __label__ = 8;; } else { __label__ = 12;; } //@line 1391 "numbers.cpp"
      $land_lhs_true11$$if_then16$14: do { 
        if (__label__ == 8) {
  
          var $10=$tensunits; //@line 1391 "numbers.cpp"
          var $cmp12=((($10))|0)==0; //@line 1391 "numbers.cpp"
          if ($cmp12) { __label__ = 9;; } else { __label__ = 10;; } //@line 1391 "numbers.cpp"
          if (__label__ == 9) {
  
            var $11=$tr_addr; //@line 1394 "numbers.cpp"
            var $arraydecay=(($ph_100)&4294967295); //@line 1394 "numbers.cpp"
            var $call=__Z6LookupP10TranslatorPKcPc($11, ((__str35332)&4294967295), $arraydecay); //@line 1394 "numbers.cpp"
            $found=$call; //@line 1394 "numbers.cpp"
            __lastLabel__ = 9; ; //@line 1395 "numbers.cpp"
          }
          else if (__label__ == 10) {
  
            var $_pr1=$found; //@line 1396 "numbers.cpp"
            __lastLabel__ = 10; ;
          }
  
          var $12=__lastLabel__ == 10 ? $_pr1 : ($call); //@line 1396 "numbers.cpp"
          var $cmp15=((($12))|0)==0; //@line 1396 "numbers.cpp"
          if ($cmp15) { __label__ = 12;break $land_lhs_true11$$if_then16$14; } else { __label__ = 17;break $land_lhs_true11$$if_then16$14; } //@line 1396 "numbers.cpp"
        }
      } while(0);
      $if_then16$$if_end27$20: do { 
        if (__label__ == 12) {
  
          var $13=$tensunits; //@line 1398 "numbers.cpp"
          var $cmp17=((($13))|0)==0; //@line 1398 "numbers.cpp"
          if ($cmp17) { __label__ = 13;; } else { __label__ = 14;; } //@line 1398 "numbers.cpp"
          if (__label__ == 13) {
  
            var $14=$tr_addr; //@line 1401 "numbers.cpp"
            var $arraydecay19=(($ph_100)&4294967295); //@line 1401 "numbers.cpp"
            var $call20=__Z6LookupP10TranslatorPKcPc($14, ((__str36333)&4294967295), $arraydecay19); //@line 1401 "numbers.cpp"
            $found=$call20; //@line 1401 "numbers.cpp"
            __lastLabel__ = 13; ; //@line 1402 "numbers.cpp"
          }
          else if (__label__ == 14) {
  
            var $_pr3=$found; //@line 1403 "numbers.cpp"
            __lastLabel__ = 14; ;
          }
  
          var $15=__lastLabel__ == 14 ? $_pr3 : ($call20); //@line 1403 "numbers.cpp"
          var $tobool22=((($15))|0)!=0; //@line 1403 "numbers.cpp"
          if ($tobool22) { __label__ = 17;break $if_then16$$if_end27$20; } //@line 1403 "numbers.cpp"
  
          var $16=$tr_addr; //@line 1405 "numbers.cpp"
          var $arraydecay24=(($ph_100)&4294967295); //@line 1405 "numbers.cpp"
          var $call25=__Z6LookupP10TranslatorPKcPc($16, ((__str37334)&4294967295), $arraydecay24); //@line 1405 "numbers.cpp"
          ; //@line 1406 "numbers.cpp"
        }
      } while(0);
  
      var $17=$tr_addr; //@line 1409 "numbers.cpp"
      var $langopts28=(($17)&4294967295); //@line 1409 "numbers.cpp"
      var $numbers29=(($langopts28+184)&4294967295); //@line 1409 "numbers.cpp"
      var $18=HEAP32[(($numbers29)>>2)]; //@line 1409 "numbers.cpp"
      var $and30=($18) & 2048; //@line 1409 "numbers.cpp"
      var $cmp31=((($and30))|0)!=0; //@line 1409 "numbers.cpp"
      var $19=$hundreds; //@line 1409 "numbers.cpp"
      if ($cmp31) { __label__ = 18;; } else { __label__ = 19;; } //@line 1409 "numbers.cpp"
      $land_lhs_true32$$if_else$28: do { 
        if (__label__ == 18) {
  
          var $cmp33=((($19))|0)!=19; //@line 1409 "numbers.cpp"
          var $20=$hundreds; //@line 1415 "numbers.cpp"
          var $cmp35=((($20))|0) >= 10; //@line 1415 "numbers.cpp"
          var $or_cond=($cmp33) & ($cmp35); //@line 1409 "numbers.cpp"
          if ($or_cond) { __label__ = 20;break $land_lhs_true32$$if_else$28; } else { __label__ = 33;break $land_lhs_true32$$if_else$28; } //@line 1409 "numbers.cpp"
        }
        else if (__label__ == 19) {
  
          var $cmp35_old=((($19))|0) >= 10; //@line 1415 "numbers.cpp"
          if ($cmp35_old) { __label__ = 20;break $land_lhs_true32$$if_else$28; } else { __label__ = 33;break $land_lhs_true32$$if_else$28; } //@line 1415 "numbers.cpp"
        }
      } while(0);
      if (__label__ == 20) {
  
        var $arrayidx37=(($ph_digits)&4294967295); //@line 1417 "numbers.cpp"
        HEAP8[($arrayidx37)]=0; //@line 1417 "numbers.cpp"
        $exact=0; //@line 1419 "numbers.cpp"
        var $21=$value_addr; //@line 1420 "numbers.cpp"
        var $rem38=((($21))|0)%1000; //@line 1420 "numbers.cpp"
        var $cmp39=((($rem38))|0)==0; //@line 1420 "numbers.cpp"
        if ($cmp39) { __label__ = 21;; } else { __label__ = 22;; } //@line 1420 "numbers.cpp"
        if (__label__ == 21) {
  
          $exact=1; //@line 1421 "numbers.cpp"
          ; //@line 1421 "numbers.cpp"
        }
  
        var $22=$tr_addr; //@line 1423 "numbers.cpp"
        var $23=$hundreds; //@line 1423 "numbers.cpp"
        var $div42=((((($23))|0)/10)|0); //@line 1423 "numbers.cpp"
        var $24=$thousandplex_addr; //@line 1423 "numbers.cpp"
        var $add=((($24)+1)&4294967295); //@line 1423 "numbers.cpp"
        var $25=$exact; //@line 1423 "numbers.cpp"
        var $26=$ordinal; //@line 1423 "numbers.cpp"
        var $or=($26) | ($25); //@line 1423 "numbers.cpp"
        var $arraydecay43=(($ph_10T)&4294967295); //@line 1423 "numbers.cpp"
        var $call44=__ZL15LookupThousandsP10TranslatoriiiPc($22, $div42, $add, $or, $arraydecay43); //@line 1423 "numbers.cpp"
        var $cmp45=((($call44))|0)==0; //@line 1423 "numbers.cpp"
        if ($cmp45) { __label__ = 23;; } else { __label__ = 26;; } //@line 1423 "numbers.cpp"
        if (__label__ == 23) {
  
          $x=0; //@line 1425 "numbers.cpp"
          var $27=$tr_addr; //@line 1426 "numbers.cpp"
          var $langopts47=(($27)&4294967295); //@line 1426 "numbers.cpp"
          var $numbers2=(($langopts47+188)&4294967295); //@line 1426 "numbers.cpp"
          var $28=HEAP32[(($numbers2)>>2)]; //@line 1426 "numbers.cpp"
          var $29=$thousandplex_addr; //@line 1426 "numbers.cpp"
          var $add48=((($29)+1)&4294967295); //@line 1426 "numbers.cpp"
          var $shl=1 << ($add48); //@line 1426 "numbers.cpp"
          var $and49=($shl) & ($28); //@line 1426 "numbers.cpp"
          var $tobool50=((($and49))|0)!=0; //@line 1426 "numbers.cpp"
          if ($tobool50) { __label__ = 24;; } else { __label__ = 25;; } //@line 1426 "numbers.cpp"
          if (__label__ == 24) {
  
            $x=8; //@line 1427 "numbers.cpp"
            ; //@line 1427 "numbers.cpp"
          }
  
          var $30=$tr_addr; //@line 1428 "numbers.cpp"
          var $31=$hundreds; //@line 1428 "numbers.cpp"
          var $div53=((((($31))|0)/10)|0); //@line 1428 "numbers.cpp"
          var $32=$x; //@line 1428 "numbers.cpp"
          var $arraydecay54=(($ph_digits)&4294967295); //@line 1428 "numbers.cpp"
          var $call55=__ZL10LookupNum2P10TranslatoriiPc($30, $div53, $32, $arraydecay54); //@line 1428 "numbers.cpp"
          ; //@line 1429 "numbers.cpp"
        }
  
        var $33=$tr_addr; //@line 1431 "numbers.cpp"
        var $langopts57=(($33)&4294967295); //@line 1431 "numbers.cpp"
        var $numbers258=(($langopts57+188)&4294967295); //@line 1431 "numbers.cpp"
        var $34=HEAP32[(($numbers258)>>2)]; //@line 1431 "numbers.cpp"
        var $and59=($34) & 512; //@line 1431 "numbers.cpp"
        var $tobool60=((($and59))|0)!=0; //@line 1431 "numbers.cpp"
        var $arraydecay62=(($ph_thousands)&4294967295); //@line 1432 "numbers.cpp"
        if ($tobool60) { __label__ = 27;; } else { __label__ = 28;; } //@line 1431 "numbers.cpp"
        if (__label__ == 27) {
  
          var $arraydecay63=(($ph_10T)&4294967295); //@line 1432 "numbers.cpp"
          var $arraydecay64=(($ph_digits)&4294967295); //@line 1432 "numbers.cpp"
          var $call65=_sprintf($arraydecay62, ((__str47699)&4294967295), allocate([$arraydecay63,0,0,0,$arraydecay64,0,0,0], ["i8*",0,0,0,"i8*",0,0,0], ALLOC_STACK)); //@line 1432 "numbers.cpp"
          ; //@line 1432 "numbers.cpp"
        }
        else if (__label__ == 28) {
  
          var $arraydecay68=(($ph_digits)&4294967295); //@line 1434 "numbers.cpp"
          var $arraydecay69=(($ph_10T)&4294967295); //@line 1434 "numbers.cpp"
          var $call70=_sprintf($arraydecay62, ((__str47699)&4294967295), allocate([$arraydecay68,0,0,0,$arraydecay69,0,0,0], ["i8*",0,0,0,"i8*",0,0,0], ALLOC_STACK)); //@line 1434 "numbers.cpp"
          ;
        }
  
        var $35=$hundreds; //@line 1436 "numbers.cpp"
        var $rem72=((($35))|0)%10; //@line 1436 "numbers.cpp"
        $hundreds=$rem72; //@line 1436 "numbers.cpp"
        var $cmp73=((($rem72))|0)==0; //@line 1437 "numbers.cpp"
        if ($cmp73) { __label__ = 30;; } else { __label__ = 32;; } //@line 1437 "numbers.cpp"
        $land_lhs_true74$$if_end78$46: do { 
          if (__label__ == 30) {
  
            var $36=$say_zero_hundred; //@line 1437 "numbers.cpp"
            var $cmp75=((($36))|0)==0; //@line 1437 "numbers.cpp"
            if (!($cmp75)) { __label__ = 32;break $land_lhs_true74$$if_end78$46; } //@line 1437 "numbers.cpp"
  
            var $arrayidx77=(($ph_100)&4294967295); //@line 1438 "numbers.cpp"
            HEAP8[($arrayidx77)]=0; //@line 1438 "numbers.cpp"
            ; //@line 1438 "numbers.cpp"
          }
        } while(0);
  
        $suppress_null_addr=1; //@line 1439 "numbers.cpp"
        ; //@line 1440 "numbers.cpp"
      }
  
      var $arrayidx81=(($ph_digits)&4294967295); //@line 1442 "numbers.cpp"
      HEAP8[($arrayidx81)]=0; //@line 1442 "numbers.cpp"
      var $37=$hundreds; //@line 1444 "numbers.cpp"
      var $cmp82=((($37))|0) > 0; //@line 1444 "numbers.cpp"
      if ($cmp82) { __label__ = 35;; } else { __label__ = 34;; } //@line 1444 "numbers.cpp"
      $if_then85$$lor_lhs_false83$51: do { 
        if (__label__ == 34) {
  
          var $38=$say_zero_hundred; //@line 1444 "numbers.cpp"
          var $tobool84=((($38))|0)!=0; //@line 1444 "numbers.cpp"
          if ($tobool84) { __label__ = 35;break $if_then85$$lor_lhs_false83$51; } else { __label__ = 60;break $if_then85$$lor_lhs_false83$51; } //@line 1444 "numbers.cpp"
        }
      } while(0);
      $if_then85$$if_end166$53: do { 
        if (__label__ == 35) {
  
          var $39=$tr_addr; //@line 1446 "numbers.cpp"
          var $langopts86=(($39)&4294967295); //@line 1446 "numbers.cpp"
          var $numbers87=(($langopts86+184)&4294967295); //@line 1446 "numbers.cpp"
          var $40=HEAP32[(($numbers87)>>2)]; //@line 1446 "numbers.cpp"
          var $and88=($40) & 262144; //@line 1446 "numbers.cpp"
          var $tobool89=((($and88))|0)!=0; //@line 1446 "numbers.cpp"
          if ($tobool89) { __label__ = 36;; } else { __label__ = 39;; } //@line 1446 "numbers.cpp"
          $land_lhs_true90$$if_end99$55: do { 
            if (__label__ == 36) {
  
              var $41=$control_addr; //@line 1446 "numbers.cpp"
              var $and91=($41) & 1; //@line 1446 "numbers.cpp"
              var $tobool92=((($and91))|0)!=0; //@line 1446 "numbers.cpp"
              if ($tobool92) { __label__ = 38;; } else { __label__ = 37;; } //@line 1446 "numbers.cpp"
              if (__label__ == 37) {
  
                var $arrayidx94=(($ph_thousands)&4294967295); //@line 1446 "numbers.cpp"
                var $42=HEAP8[($arrayidx94)]; //@line 1446 "numbers.cpp"
                var $conv=(tempInt=(($42)),(tempInt>=128?tempInt-256:tempInt)); //@line 1446 "numbers.cpp"
                var $cmp95=((($conv))|0)!=0; //@line 1446 "numbers.cpp"
                if (!($cmp95)) { __label__ = 39;break $land_lhs_true90$$if_end99$55; } //@line 1446 "numbers.cpp"
              }
  
              var $43=$tr_addr; //@line 1448 "numbers.cpp"
              var $arraydecay97=(($ph_thousand_and)&4294967295); //@line 1448 "numbers.cpp"
              var $call98=__Z6LookupP10TranslatorPKcPc($43, ((__str33330)&4294967295), $arraydecay97); //@line 1448 "numbers.cpp"
              ; //@line 1449 "numbers.cpp"
            }
          } while(0);
  
          $suppress_null_addr=1; //@line 1451 "numbers.cpp"
          $found=0; //@line 1453 "numbers.cpp"
          var $44=$ordinal; //@line 1454 "numbers.cpp"
          var $tobool100=((($44))|0)!=0; //@line 1454 "numbers.cpp"
          if ($tobool100) { __label__ = 40;; } else { __label__ = 45;; } //@line 1454 "numbers.cpp"
          $land_lhs_true101$$if_end124$61: do { 
            if (__label__ == 40) {
  
              var $45=$tensunits; //@line 1454 "numbers.cpp"
              var $cmp102=((($45))|0)==0; //@line 1454 "numbers.cpp"
              if ($cmp102) { __label__ = 42;; } else { __label__ = 41;; } //@line 1454 "numbers.cpp"
              if (__label__ == 41) {
  
                var $46=$tr_addr; //@line 1454 "numbers.cpp"
                var $langopts104=(($46)&4294967295); //@line 1454 "numbers.cpp"
                var $numbers2105=(($langopts104+188)&4294967295); //@line 1454 "numbers.cpp"
                var $47=HEAP32[(($numbers2105)>>2)]; //@line 1454 "numbers.cpp"
                var $and106=($47) & 4096; //@line 1454 "numbers.cpp"
                var $tobool107=((($and106))|0)!=0; //@line 1454 "numbers.cpp"
                if (!($tobool107)) { __label__ = 45;break $land_lhs_true101$$if_end124$61; } //@line 1454 "numbers.cpp"
              }
  
              var $arraydecay109=(($string)&4294967295); //@line 1458 "numbers.cpp"
              var $48=$hundreds; //@line 1458 "numbers.cpp"
              var $call110=_sprintf($arraydecay109, ((__str39336)&4294967295), allocate([$48,0,0,0], ["i32",0,0,0], ALLOC_STACK)); //@line 1458 "numbers.cpp"
              var $49=$tr_addr; //@line 1459 "numbers.cpp"
              var $arraydecay111=(($string)&4294967295); //@line 1459 "numbers.cpp"
              var $arraydecay112=(($ph_digits)&4294967295); //@line 1459 "numbers.cpp"
              var $call113=__Z6LookupP10TranslatorPKcPc($49, $arraydecay111, $arraydecay112); //@line 1459 "numbers.cpp"
              $found=$call113; //@line 1459 "numbers.cpp"
              var $50=$tr_addr; //@line 1461 "numbers.cpp"
              var $langopts114=(($50)&4294967295); //@line 1461 "numbers.cpp"
              var $numbers2115=(($langopts114+188)&4294967295); //@line 1461 "numbers.cpp"
              var $51=HEAP32[(($numbers2115)>>2)]; //@line 1461 "numbers.cpp"
              var $and116=($51) & 4096; //@line 1461 "numbers.cpp"
              var $tobool117=((($and116))|0)!=0; //@line 1461 "numbers.cpp"
              if (!($tobool117)) { __label__ = 45;break $land_lhs_true101$$if_end124$61; } //@line 1461 "numbers.cpp"
  
              var $52=$tensunits; //@line 1461 "numbers.cpp"
              var $cmp119=((($52))|0) > 0; //@line 1461 "numbers.cpp"
              if (!($cmp119)) { __label__ = 45;break $land_lhs_true101$$if_end124$61; } //@line 1461 "numbers.cpp"
  
              var $arraydecay121=(($ph_digits)&4294967295); //@line 1465 "numbers.cpp"
              var $call122=_strcat($arraydecay121, ((__ZL11ph_ordinal2)&4294967295)); //@line 1465 "numbers.cpp"
              ; //@line 1466 "numbers.cpp"
            }
          } while(0);
  
          var $53=$hundreds; //@line 1469 "numbers.cpp"
          var $cmp125=((($53))|0)==0; //@line 1469 "numbers.cpp"
          if ($cmp125) { __label__ = 46;; } else { __label__ = 48;; } //@line 1469 "numbers.cpp"
          $land_lhs_true126$$if_else131$69: do { 
            if (__label__ == 46) {
  
              var $54=$say_zero_hundred; //@line 1469 "numbers.cpp"
              var $tobool127=((($54))|0)!=0; //@line 1469 "numbers.cpp"
              if (!($tobool127)) { __label__ = 48;break $land_lhs_true126$$if_else131$69; } //@line 1469 "numbers.cpp"
  
              var $55=$tr_addr; //@line 1471 "numbers.cpp"
              var $arraydecay129=(($ph_digits)&4294967295); //@line 1471 "numbers.cpp"
              var $call130=__Z6LookupP10TranslatorPKcPc($55, ((__str16313)&4294967295), $arraydecay129); //@line 1471 "numbers.cpp"
              __label__ = 60;break $if_then85$$if_end166$53; //@line 1472 "numbers.cpp"
            }
          } while(0);
  
          var $56=$found; //@line 1475 "numbers.cpp"
          var $tobool132=((($56))|0)!=0; //@line 1475 "numbers.cpp"
          if ($tobool132) { __label__ = 54;; } else { __label__ = 49;; } //@line 1475 "numbers.cpp"
          $if_end149thread_pre_split$$land_lhs_true133$73: do { 
            if (__label__ == 49) {
  
              var $57=$tensunits; //@line 1475 "numbers.cpp"
              var $cmp134=((($57))|0)==0; //@line 1475 "numbers.cpp"
              if ($cmp134) { __label__ = 50;; } else { __label__ = 51;; } //@line 1475 "numbers.cpp"
              if (__label__ == 50) {
  
                var $arraydecay136=(($string)&4294967295); //@line 1478 "numbers.cpp"
                var $58=$hundreds; //@line 1478 "numbers.cpp"
                var $call137=_sprintf($arraydecay136, ((__str40337)&4294967295), allocate([$58,0,0,0], ["i32",0,0,0], ALLOC_STACK)); //@line 1478 "numbers.cpp"
                var $59=$tr_addr; //@line 1479 "numbers.cpp"
                var $arraydecay138=(($string)&4294967295); //@line 1479 "numbers.cpp"
                var $arraydecay139=(($ph_digits)&4294967295); //@line 1479 "numbers.cpp"
                var $call140=__Z6LookupP10TranslatorPKcPc($59, $arraydecay138, $arraydecay139); //@line 1479 "numbers.cpp"
                $found=$call140; //@line 1479 "numbers.cpp"
                __lastLabel__ = 50; ; //@line 1480 "numbers.cpp"
              }
              else if (__label__ == 51) {
  
                var $_pr4=$found; //@line 1482 "numbers.cpp"
                __lastLabel__ = 51; ;
              }
  
              var $60=__lastLabel__ == 51 ? $_pr4 : ($call140); //@line 1482 "numbers.cpp"
              var $tobool142=((($60))|0)!=0; //@line 1482 "numbers.cpp"
              if ($tobool142) { __label__ = 54;break $if_end149thread_pre_split$$land_lhs_true133$73; } //@line 1482 "numbers.cpp"
  
              var $arraydecay144=(($string)&4294967295); //@line 1484 "numbers.cpp"
              var $61=$hundreds; //@line 1484 "numbers.cpp"
              var $call145=_sprintf($arraydecay144, ((__str41338)&4294967295), allocate([$61,0,0,0], ["i32",0,0,0], ALLOC_STACK)); //@line 1484 "numbers.cpp"
              var $62=$tr_addr; //@line 1485 "numbers.cpp"
              var $arraydecay146=(($string)&4294967295); //@line 1485 "numbers.cpp"
              var $arraydecay147=(($ph_digits)&4294967295); //@line 1485 "numbers.cpp"
              var $call148=__Z6LookupP10TranslatorPKcPc($62, $arraydecay146, $arraydecay147); //@line 1485 "numbers.cpp"
              $found=$call148; //@line 1485 "numbers.cpp"
              __lastLabel__ = 53; __label__ = 55;break $if_end149thread_pre_split$$land_lhs_true133$73; //@line 1486 "numbers.cpp"
            }
          } while(0);
          if (__label__ == 54) {
  
            var $_pr6=$found; //@line 1488 "numbers.cpp"
            __lastLabel__ = 54; ;
          }
  
          var $63=__lastLabel__ == 54 ? $_pr6 : ($call148); //@line 1488 "numbers.cpp"
          var $tobool150=((($63))|0)!=0; //@line 1488 "numbers.cpp"
          if ($tobool150) { __label__ = 56;; } else { __label__ = 57;; } //@line 1488 "numbers.cpp"
          if (__label__ == 56) {
  
            var $arrayidx152=(($ph_100)&4294967295); //@line 1490 "numbers.cpp"
            HEAP8[($arrayidx152)]=0; //@line 1490 "numbers.cpp"
            ; //@line 1491 "numbers.cpp"
          }
          else if (__label__ == 57) {
  
            var $64=$hundreds; //@line 1494 "numbers.cpp"
            var $cmp154=((($64))|0) > 1; //@line 1494 "numbers.cpp"
            if ($cmp154) { __label__ = 59;; } else { __label__ = 58;; } //@line 1494 "numbers.cpp"
            if (__label__ == 58) {
  
              var $65=$tr_addr; //@line 1494 "numbers.cpp"
              var $langopts156=(($65)&4294967295); //@line 1494 "numbers.cpp"
              var $numbers157=(($langopts156+184)&4294967295); //@line 1494 "numbers.cpp"
              var $66=HEAP32[(($numbers157)>>2)]; //@line 1494 "numbers.cpp"
              var $and158=($66) & 1024; //@line 1494 "numbers.cpp"
              var $cmp159=((($and158))|0)==0; //@line 1494 "numbers.cpp"
              if (!($cmp159)) { __label__ = 60;break $if_then85$$if_end166$53; } //@line 1494 "numbers.cpp"
            }
  
            var $67=$tr_addr; //@line 1496 "numbers.cpp"
            var $68=$hundreds; //@line 1496 "numbers.cpp"
            var $arraydecay161=(($ph_digits)&4294967295); //@line 1496 "numbers.cpp"
            var $call162=__ZL10LookupNum2P10TranslatoriiPc($67, $68, 0, $arraydecay161); //@line 1496 "numbers.cpp"
            ; //@line 1497 "numbers.cpp"
          }
        }
      } while(0);
  
      var $arraydecay167=(($buf1)&4294967295); //@line 1502 "numbers.cpp"
      var $arraydecay168=(($ph_thousands)&4294967295); //@line 1502 "numbers.cpp"
      var $arraydecay169=(($ph_thousand_and)&4294967295); //@line 1502 "numbers.cpp"
      var $arraydecay170=(($ph_digits)&4294967295); //@line 1502 "numbers.cpp"
      var $arraydecay171=(($ph_100)&4294967295); //@line 1502 "numbers.cpp"
      var $call172=_sprintf($arraydecay167, ((__str20317)&4294967295), allocate([$arraydecay168,0,0,0,$arraydecay169,0,0,0,$arraydecay170,0,0,0,$arraydecay171,0,0,0], ["i8*",0,0,0,"i8*",0,0,0,"i8*",0,0,0,"i8*",0,0,0], ALLOC_STACK)); //@line 1502 "numbers.cpp"
      ; //@line 1503 "numbers.cpp"
    }
  
    var $arrayidx174=(($ph_hundred_and)&4294967295); //@line 1505 "numbers.cpp"
    HEAP8[($arrayidx174)]=0; //@line 1505 "numbers.cpp"
    var $69=$tensunits; //@line 1506 "numbers.cpp"
    var $cmp175=((($69))|0) > 0; //@line 1506 "numbers.cpp"
    if ($cmp175) { __label__ = 62;; } else { __label__ = 74;; } //@line 1506 "numbers.cpp"
    $if_then176$$if_end217$91: do { 
      if (__label__ == 62) {
  
        var $70=$value_addr; //@line 1508 "numbers.cpp"
        var $cmp177=((($70))|0) > 100; //@line 1508 "numbers.cpp"
        if ($cmp177) { __label__ = 65;; } else { __label__ = 63;; } //@line 1508 "numbers.cpp"
        $if_then183$$lor_lhs_false178$93: do { 
          if (__label__ == 63) {
  
            var $71=$control_addr; //@line 1508 "numbers.cpp"
            var $and179=($71) & 1; //@line 1508 "numbers.cpp"
            var $tobool180=((($and179))|0)!=0; //@line 1508 "numbers.cpp"
            if (!($tobool180)) { __label__ = 69;break $if_then183$$lor_lhs_false178$93; } //@line 1508 "numbers.cpp"
  
            var $72=$thousandplex_addr; //@line 1508 "numbers.cpp"
            var $cmp182=((($72))|0)==0; //@line 1508 "numbers.cpp"
            if ($cmp182) { __label__ = 65;break $if_then183$$lor_lhs_false178$93; } else { __label__ = 69;break $if_then183$$lor_lhs_false178$93; } //@line 1508 "numbers.cpp"
          }
        } while(0);
        $if_then183$$if_end199$96: do { 
          if (__label__ == 65) {
  
            var $73=$tr_addr; //@line 1510 "numbers.cpp"
            var $langopts184=(($73)&4294967295); //@line 1510 "numbers.cpp"
            var $numbers185=(($langopts184+184)&4294967295); //@line 1510 "numbers.cpp"
            var $74=HEAP32[(($numbers185)>>2)]; //@line 1510 "numbers.cpp"
            var $and186=($74) & 64; //@line 1510 "numbers.cpp"
            var $tobool187=((($and186))|0)!=0; //@line 1510 "numbers.cpp"
            if ($tobool187) { __label__ = 68;; } else { __label__ = 66;; } //@line 1510 "numbers.cpp"
            if (__label__ == 66) {
  
              var $75=$tr_addr; //@line 1510 "numbers.cpp"
              var $langopts189=(($75)&4294967295); //@line 1510 "numbers.cpp"
              var $numbers190=(($langopts189+184)&4294967295); //@line 1510 "numbers.cpp"
              var $76=HEAP32[(($numbers190)>>2)]; //@line 1510 "numbers.cpp"
              var $and191=($76) & 8388608; //@line 1510 "numbers.cpp"
              var $tobool192=((($and191))|0)!=0; //@line 1510 "numbers.cpp"
              if (!($tobool192)) { __label__ = 69;break $if_then183$$if_end199$96; } //@line 1510 "numbers.cpp"
  
              var $77=$tensunits; //@line 1510 "numbers.cpp"
              var $cmp194=((($77))|0) < 10; //@line 1510 "numbers.cpp"
              if (!($cmp194)) { __label__ = 69;break $if_then183$$if_end199$96; } //@line 1510 "numbers.cpp"
            }
  
            var $78=$tr_addr; //@line 1512 "numbers.cpp"
            var $arraydecay196=(($ph_hundred_and)&4294967295); //@line 1512 "numbers.cpp"
            var $call197=__Z6LookupP10TranslatorPKcPc($78, ((__str33330)&4294967295), $arraydecay196); //@line 1512 "numbers.cpp"
            ; //@line 1513 "numbers.cpp"
          }
        } while(0);
  
        var $79=$tr_addr; //@line 1515 "numbers.cpp"
        var $langopts200=(($79)&4294967295); //@line 1515 "numbers.cpp"
        var $numbers201=(($langopts200+184)&4294967295); //@line 1515 "numbers.cpp"
        var $80=HEAP32[(($numbers201)>>2)]; //@line 1515 "numbers.cpp"
        var $and202=($80) & 524288; //@line 1515 "numbers.cpp"
        var $tobool203=((($and202))|0)!=0; //@line 1515 "numbers.cpp"
        if (!($tobool203)) { __label__ = 74;break $if_then176$$if_end217$91; } //@line 1515 "numbers.cpp"
  
        var $81=$hundreds; //@line 1515 "numbers.cpp"
        var $cmp205=((($81))|0)==0; //@line 1515 "numbers.cpp"
        if (!($cmp205)) { __label__ = 74;break $if_then176$$if_end217$91; } //@line 1515 "numbers.cpp"
  
        var $82=$control_addr; //@line 1515 "numbers.cpp"
        var $and207=($82) & 1; //@line 1515 "numbers.cpp"
        var $tobool208=((($and207))|0)!=0; //@line 1515 "numbers.cpp"
        if ($tobool208) { __label__ = 73;; } else { __label__ = 72;; } //@line 1515 "numbers.cpp"
        if (__label__ == 72) {
  
          var $arrayidx210=(($ph_thousands)&4294967295); //@line 1515 "numbers.cpp"
          var $83=HEAP8[($arrayidx210)]; //@line 1515 "numbers.cpp"
          var $conv211=(tempInt=(($83)),(tempInt>=128?tempInt-256:tempInt)); //@line 1515 "numbers.cpp"
          var $cmp212=((($conv211))|0)!=0; //@line 1515 "numbers.cpp"
          if (!($cmp212)) { __label__ = 74;break $if_then176$$if_end217$91; } //@line 1515 "numbers.cpp"
        }
  
        var $84=$tr_addr; //@line 1517 "numbers.cpp"
        var $arraydecay214=(($ph_hundred_and)&4294967295); //@line 1517 "numbers.cpp"
        var $call215=__Z6LookupP10TranslatorPKcPc($84, ((__str33330)&4294967295), $arraydecay214); //@line 1517 "numbers.cpp"
        ; //@line 1518 "numbers.cpp"
      }
    } while(0);
  
    var $arrayidx218=(($buf2)&4294967295); //@line 1522 "numbers.cpp"
    HEAP8[($arrayidx218)]=0; //@line 1522 "numbers.cpp"
    var $85=$tensunits; //@line 1524 "numbers.cpp"
    var $cmp219=((($85))|0)!=0; //@line 1524 "numbers.cpp"
    if ($cmp219) { __label__ = 76;; } else { __label__ = 75;; } //@line 1524 "numbers.cpp"
    $if_then222$$lor_lhs_false220$109: do { 
      if (__label__ == 75) {
  
        var $86=$suppress_null_addr; //@line 1524 "numbers.cpp"
        var $cmp221=((($86))|0)==0; //@line 1524 "numbers.cpp"
        if ($cmp221) { __label__ = 76;break $if_then222$$lor_lhs_false220$109; } //@line 1524 "numbers.cpp"
  
        var $102=HEAP8[(((__ZL11ph_ordinal2)&4294967295))]; //@line 1551 "numbers.cpp"
        var $conv262=(tempInt=(($102)),(tempInt>=128?tempInt-256:tempInt)); //@line 1551 "numbers.cpp"
        var $cmp263=((($conv262))|0)!=0; //@line 1551 "numbers.cpp"
        if (!($cmp263)) { __label__ = 94;break $if_then222$$lor_lhs_false220$109; } //@line 1551 "numbers.cpp"
  
        var $arraydecay265=(($buf1)&4294967295); //@line 1553 "numbers.cpp"
        var $call266=_strlen($arraydecay265); //@line 1553 "numbers.cpp"
        $ix=$call266; //@line 1553 "numbers.cpp"
        var $103=$ix; //@line 1554 "numbers.cpp"
        var $cmp267=((($103))|0) > 0; //@line 1554 "numbers.cpp"
        if ($cmp267) { __label__ = 91;; } else { __label__ = 93;; } //@line 1554 "numbers.cpp"
        $land_lhs_true268$$if_end275$113: do { 
          if (__label__ == 91) {
  
            var $104=$ix; //@line 1554 "numbers.cpp"
            var $sub=((($104)-1)&4294967295); //@line 1554 "numbers.cpp"
            var $arrayidx269=(($buf1+$sub)&4294967295); //@line 1554 "numbers.cpp"
            var $105=HEAP8[($arrayidx269)]; //@line 1554 "numbers.cpp"
            var $conv270=(tempInt=(($105)),(tempInt>=128?tempInt-256:tempInt)); //@line 1554 "numbers.cpp"
            var $cmp271=((($conv270))|0)==10; //@line 1554 "numbers.cpp"
            if (!($cmp271)) { __label__ = 93;break $land_lhs_true268$$if_end275$113; } //@line 1554 "numbers.cpp"
  
            var $106=$ix; //@line 1555 "numbers.cpp"
            var $sub273=((($106)-1)&4294967295); //@line 1555 "numbers.cpp"
            var $arrayidx274=(($buf1+$sub273)&4294967295); //@line 1555 "numbers.cpp"
            HEAP8[($arrayidx274)]=0; //@line 1555 "numbers.cpp"
            ; //@line 1555 "numbers.cpp"
          }
        } while(0);
  
        var $arraydecay276=(($buf2)&4294967295); //@line 1556 "numbers.cpp"
        var $call277=_strcpy($arraydecay276, ((__ZL11ph_ordinal2)&4294967295)); //@line 1556 "numbers.cpp"
        __label__ = 94;break $if_then222$$lor_lhs_false220$109; //@line 1557 "numbers.cpp"
      }
    } while(0);
    $if_then222$$if_end279$117: do { 
      if (__label__ == 76) {
  
        $x=0; //@line 1526 "numbers.cpp"
        var $87=$thousandplex_addr; //@line 1527 "numbers.cpp"
        var $cmp223=((($87))|0)==0; //@line 1527 "numbers.cpp"
        if ($cmp223) { __label__ = 77;; } else { __label__ = 84;; } //@line 1527 "numbers.cpp"
        $if_then224$$if_else240$119: do { 
          if (__label__ == 77) {
  
            $x=2; //@line 1529 "numbers.cpp"
            var $88=$ordinal; //@line 1530 "numbers.cpp"
            var $tobool225=((($88))|0)!=0; //@line 1530 "numbers.cpp"
            if ($tobool225) { __label__ = 78;; } else { __label__ = 79;; } //@line 1530 "numbers.cpp"
            if (__label__ == 78) {
  
              $x=3; //@line 1531 "numbers.cpp"
              ; //@line 1531 "numbers.cpp"
            }
  
            var $89=$value_addr; //@line 1532 "numbers.cpp"
            var $cmp228=((($89))|0) < 100; //@line 1532 "numbers.cpp"
            if ($cmp228) { __label__ = 80;; } else { __label__ = 82;; } //@line 1532 "numbers.cpp"
            $land_lhs_true229$$if_end234$124: do { 
              if (__label__ == 80) {
  
                var $90=$control_addr; //@line 1532 "numbers.cpp"
                var $and230=($90) & 1; //@line 1532 "numbers.cpp"
                var $tobool231=((($and230))|0)!=0; //@line 1532 "numbers.cpp"
                if ($tobool231) { __label__ = 82;break $land_lhs_true229$$if_end234$124; } //@line 1532 "numbers.cpp"
  
                var $91=$x; //@line 1533 "numbers.cpp"
                var $or233=($91) | 4; //@line 1533 "numbers.cpp"
                $x=$or233; //@line 1533 "numbers.cpp"
                ; //@line 1533 "numbers.cpp"
              }
            } while(0);
  
            var $92=$ordinal; //@line 1534 "numbers.cpp"
            var $and235=($92) & 32; //@line 1534 "numbers.cpp"
            var $tobool236=((($and235))|0)!=0; //@line 1534 "numbers.cpp"
            if (!($tobool236)) { __label__ = 86;break $if_then224$$if_else240$119; } //@line 1534 "numbers.cpp"
  
            var $93=$x; //@line 1535 "numbers.cpp"
            var $or238=($93) | 32; //@line 1535 "numbers.cpp"
            $x=$or238; //@line 1535 "numbers.cpp"
            ; //@line 1535 "numbers.cpp"
          }
          else if (__label__ == 84) {
  
            var $94=$tr_addr; //@line 1539 "numbers.cpp"
            var $langopts241=(($94)&4294967295); //@line 1539 "numbers.cpp"
            var $numbers2242=(($langopts241+188)&4294967295); //@line 1539 "numbers.cpp"
            var $95=HEAP32[(($numbers2242)>>2)]; //@line 1539 "numbers.cpp"
            var $96=$thousandplex_addr; //@line 1539 "numbers.cpp"
            var $shl243=1 << ($96); //@line 1539 "numbers.cpp"
            var $and244=($shl243) & ($95); //@line 1539 "numbers.cpp"
            var $tobool245=((($and244))|0)!=0; //@line 1539 "numbers.cpp"
            if (!($tobool245)) { __label__ = 86;break $if_then224$$if_else240$119; } //@line 1539 "numbers.cpp"
  
            $x=8; //@line 1540 "numbers.cpp"
            ; //@line 1540 "numbers.cpp"
          }
        } while(0);
  
        var $97=$tr_addr; //@line 1543 "numbers.cpp"
        var $98=$tensunits; //@line 1543 "numbers.cpp"
        var $99=$x; //@line 1543 "numbers.cpp"
        var $arraydecay249=(($buf2)&4294967295); //@line 1543 "numbers.cpp"
        var $call250=__ZL10LookupNum2P10TranslatoriiPc($97, $98, $99, $arraydecay249); //@line 1543 "numbers.cpp"
        var $cmp251=((($call250))|0)!=0; //@line 1543 "numbers.cpp"
        if (!($cmp251)) { __label__ = 94;break $if_then222$$if_end279$117; } //@line 1543 "numbers.cpp"
  
        var $100=$tr_addr; //@line 1545 "numbers.cpp"
        var $langopts253=(($100)&4294967295); //@line 1545 "numbers.cpp"
        var $numbers254=(($langopts253+184)&4294967295); //@line 1545 "numbers.cpp"
        var $101=HEAP32[(($numbers254)>>2)]; //@line 1545 "numbers.cpp"
        var $and255=($101) & 128; //@line 1545 "numbers.cpp"
        var $tobool256=((($and255))|0)!=0; //@line 1545 "numbers.cpp"
        if (!($tobool256)) { __label__ = 94;break $if_then222$$if_end279$117; } //@line 1545 "numbers.cpp"
  
        var $arrayidx258=(($ph_hundred_and)&4294967295); //@line 1546 "numbers.cpp"
        HEAP8[($arrayidx258)]=0; //@line 1546 "numbers.cpp"
        ; //@line 1546 "numbers.cpp"
      }
    } while(0);
  
    var $107=$ph_out_addr; //@line 1560 "numbers.cpp"
    var $arraydecay280=(($buf1)&4294967295); //@line 1560 "numbers.cpp"
    var $arraydecay281=(($ph_hundred_and)&4294967295); //@line 1560 "numbers.cpp"
    var $arraydecay282=(($buf2)&4294967295); //@line 1560 "numbers.cpp"
    var $call283=_sprintf($107, ((__str6549)&4294967295), allocate([$arraydecay280,0,0,0,$arraydecay281,0,0,0,$arraydecay282,0,0,0], ["i8*",0,0,0,"i8*",0,0,0,"i8*",0,0,0], ALLOC_STACK)); //@line 1560 "numbers.cpp"
    STACKTOP = __stackBase__;
    return;
    return;
  }
  

  function __ZL10LookupNum2P10TranslatoriiPc($tr, $value, $control, $ph_out) {
    var __stackBase__  = STACKTOP; STACKTOP += 148; _memset(__stackBase__, 0, 148);
    var __label__;
    var __lastLabel__ = null;
  
    var $tr_addr;
    var $value_addr;
    var $control_addr;
    var $ph_out_addr;
    var $found;
    var $ix;
    var $units;
    var $tens;
    var $used_and;
    var $found_ordinal;
    var $next_phtype;
    var $ord_type;
    var $string=__stackBase__;
    var $ph_ordinal=__stackBase__+12;
    var $ph_tens=__stackBase__+32;
    var $ph_digits=__stackBase__+84;
    var $ph_and=__stackBase__+136;
    $tr_addr=$tr;
    $value_addr=$value;
    $control_addr=$control;
    $ph_out_addr=$ph_out;
    $used_and=0; //@line 1109 "numbers.cpp"
    $found_ordinal=0; //@line 1110 "numbers.cpp"
    $ord_type=111; //@line 1112 "numbers.cpp"
    var $0=$value_addr; //@line 1119 "numbers.cpp"
    var $rem=((($0))|0)%10; //@line 1119 "numbers.cpp"
    $units=$rem; //@line 1119 "numbers.cpp"
    var $1=$value_addr; //@line 1120 "numbers.cpp"
    var $div=((((($1))|0)/10)|0); //@line 1120 "numbers.cpp"
    $tens=$div; //@line 1120 "numbers.cpp"
    $found=0; //@line 1122 "numbers.cpp"
    var $arrayidx=(($ph_ordinal)&4294967295); //@line 1123 "numbers.cpp"
    HEAP8[($arrayidx)]=0; //@line 1123 "numbers.cpp"
    var $arrayidx1=(($ph_tens)&4294967295); //@line 1124 "numbers.cpp"
    HEAP8[($arrayidx1)]=0; //@line 1124 "numbers.cpp"
    var $arrayidx2=(($ph_digits)&4294967295); //@line 1125 "numbers.cpp"
    HEAP8[($arrayidx2)]=0; //@line 1125 "numbers.cpp"
    var $arrayidx3=(($ph_and)&4294967295); //@line 1126 "numbers.cpp"
    HEAP8[($arrayidx3)]=0; //@line 1126 "numbers.cpp"
    var $2=$control_addr; //@line 1128 "numbers.cpp"
    var $and=($2) & 32; //@line 1128 "numbers.cpp"
    var $tobool=((($and))|0)!=0; //@line 1128 "numbers.cpp"
    if ($tobool) { __label__ = 1;; } else { __label__ = 2;; } //@line 1128 "numbers.cpp"
    if (__label__ == 1) {
  
      $ord_type=113; //@line 1130 "numbers.cpp"
      ; //@line 1131 "numbers.cpp"
    }
  
    var $3=$control_addr; //@line 1133 "numbers.cpp"
    var $and4=($3) & 2; //@line 1133 "numbers.cpp"
    var $tobool5=((($and4))|0)!=0; //@line 1133 "numbers.cpp"
    var $4=HEAP32[((__ZL14n_digit_lookup)>>2)]; //@line 1133 "numbers.cpp"
    var $cmp=((($4))|0)==2; //@line 1133 "numbers.cpp"
    var $or_cond=($tobool5) & ($cmp); //@line 1133 "numbers.cpp"
    if ($or_cond) { __label__ = 3;; } else { __label__ = 4;; } //@line 1133 "numbers.cpp"
    $if_then6$$if_else$5: do { 
      if (__label__ == 3) {
  
        var $5=$ph_out_addr; //@line 1136 "numbers.cpp"
        var $6=HEAP32[((__ZL12digit_lookup)>>2)]; //@line 1136 "numbers.cpp"
        var $call=_strcpy($5, $6); //@line 1136 "numbers.cpp"
        ; //@line 1137 "numbers.cpp"
      }
      else if (__label__ == 4) {
  
        var $7=HEAP32[((__ZL12digit_lookup)>>2)]; //@line 1140 "numbers.cpp"
        var $arrayidx7=(($7)&4294967295); //@line 1140 "numbers.cpp"
        var $8=HEAP8[($arrayidx7)]; //@line 1140 "numbers.cpp"
        var $conv=(tempInt=(($8)),(tempInt>=128?tempInt-256:tempInt)); //@line 1140 "numbers.cpp"
        var $cmp8=((($conv))|0)==0; //@line 1140 "numbers.cpp"
        if ($cmp8) { __label__ = 5;; } else { __label__ = 23;; } //@line 1140 "numbers.cpp"
        $if_then9$$if_end72$8: do { 
          if (__label__ == 5) {
  
            var $9=$control_addr; //@line 1143 "numbers.cpp"
            var $and10=($9) & 8; //@line 1143 "numbers.cpp"
            var $tobool11=((($and10))|0)!=0; //@line 1143 "numbers.cpp"
            if ($tobool11) { __label__ = 6;; } else { __label__ = 7;; } //@line 1143 "numbers.cpp"
            if (__label__ == 6) {
  
              var $arraydecay=(($string)&4294967295); //@line 1145 "numbers.cpp"
              var $10=$value_addr; //@line 1145 "numbers.cpp"
              var $call13=_sprintf($arraydecay, ((__str23320)&4294967295), allocate([$10,0,0,0], ["i32",0,0,0], ALLOC_STACK)); //@line 1145 "numbers.cpp"
              var $11=$tr_addr; //@line 1146 "numbers.cpp"
              var $arraydecay14=(($string)&4294967295); //@line 1146 "numbers.cpp"
              var $arraydecay15=(($ph_digits)&4294967295); //@line 1146 "numbers.cpp"
              var $call16=__Z6LookupP10TranslatorPKcPc($11, $arraydecay14, $arraydecay15); //@line 1146 "numbers.cpp"
              $found=$call16; //@line 1146 "numbers.cpp"
              __lastLabel__ = 6; ; //@line 1147 "numbers.cpp"
            }
            else if (__label__ == 7) {
  
              var $12=$control_addr; //@line 1149 "numbers.cpp"
              var $and18=($12) & 1; //@line 1149 "numbers.cpp"
              var $tobool19=((($and18))|0)!=0; //@line 1149 "numbers.cpp"
              if ($tobool19) { __label__ = 8;; } else { __label__ = 14;; } //@line 1149 "numbers.cpp"
              if (__label__ == 8) {
  
                var $arraydecay21=(($ph_ordinal)&4294967295); //@line 1151 "numbers.cpp"
                var $call22=_strcpy($arraydecay21, ((__ZL11ph_ordinal2)&4294967295)); //@line 1151 "numbers.cpp"
                var $13=$control_addr; //@line 1153 "numbers.cpp"
                var $and23=($13) & 4; //@line 1153 "numbers.cpp"
                var $tobool24=((($and23))|0)!=0; //@line 1153 "numbers.cpp"
                if ($tobool24) { __label__ = 9;; } else { __label__ = 10;; } //@line 1153 "numbers.cpp"
                if (__label__ == 9) {
  
                  var $arraydecay26=(($string)&4294967295); //@line 1155 "numbers.cpp"
                  var $14=$value_addr; //@line 1155 "numbers.cpp"
                  var $15=$ord_type; //@line 1155 "numbers.cpp"
                  var $call27=_sprintf($arraydecay26, ((__str24321)&4294967295), allocate([$14,0,0,0,$15,0,0,0], ["i32",0,0,0,"i32",0,0,0], ALLOC_STACK)); //@line 1155 "numbers.cpp"
                  var $16=$tr_addr; //@line 1156 "numbers.cpp"
                  var $arraydecay28=(($string)&4294967295); //@line 1156 "numbers.cpp"
                  var $arraydecay29=(($ph_digits)&4294967295); //@line 1156 "numbers.cpp"
                  var $call30=__Z6LookupP10TranslatorPKcPc($16, $arraydecay28, $arraydecay29); //@line 1156 "numbers.cpp"
                  $found=$call30; //@line 1156 "numbers.cpp"
                  __lastLabel__ = 9; ; //@line 1157 "numbers.cpp"
                }
                else if (__label__ == 10) {
  
                  var $_pr=$found; //@line 1158 "numbers.cpp"
                  __lastLabel__ = 10; ;
                }
  
                var $17=__lastLabel__ == 10 ? $_pr : ($call30); //@line 1158 "numbers.cpp"
                var $cmp32=((($17))|0)==0; //@line 1158 "numbers.cpp"
                if ($cmp32) { __label__ = 12;; } else { __label__ = 13;; } //@line 1158 "numbers.cpp"
                if (__label__ == 12) {
  
                  var $arraydecay34=(($string)&4294967295); //@line 1160 "numbers.cpp"
                  var $18=$value_addr; //@line 1160 "numbers.cpp"
                  var $19=$ord_type; //@line 1160 "numbers.cpp"
                  var $call35=_sprintf($arraydecay34, ((__str25322)&4294967295), allocate([$18,0,0,0,$19,0,0,0], ["i32",0,0,0,"i32",0,0,0], ALLOC_STACK)); //@line 1160 "numbers.cpp"
                  var $20=$tr_addr; //@line 1161 "numbers.cpp"
                  var $arraydecay36=(($string)&4294967295); //@line 1161 "numbers.cpp"
                  var $arraydecay37=(($ph_digits)&4294967295); //@line 1161 "numbers.cpp"
                  var $call38=__Z6LookupP10TranslatorPKcPc($20, $arraydecay36, $arraydecay37); //@line 1161 "numbers.cpp"
                  $found=$call38; //@line 1161 "numbers.cpp"
                  ; //@line 1162 "numbers.cpp"
                }
  
                var $21=$found; //@line 1163 "numbers.cpp"
                $found_ordinal=$21; //@line 1163 "numbers.cpp"
                __lastLabel__ = 13; ; //@line 1164 "numbers.cpp"
              }
              else if (__label__ == 14) {
  
                var $_pr1=$found; //@line 1166 "numbers.cpp"
                __lastLabel__ = 14; ;
              }
            }
  
            var $22=__lastLabel__ == 14 ? $_pr1 : (__lastLabel__ == 13 ? $21 : ($call16)); //@line 1166 "numbers.cpp"
            var $cmp42=((($22))|0)==0; //@line 1166 "numbers.cpp"
            if (!($cmp42)) { __label__ = 23;break $if_then9$$if_end72$8; } //@line 1166 "numbers.cpp"
  
            var $23=$control_addr; //@line 1168 "numbers.cpp"
            var $and44=($23) & 2; //@line 1168 "numbers.cpp"
            var $tobool45=((($and44))|0)!=0; //@line 1168 "numbers.cpp"
            if ($tobool45) { __label__ = 17;; } else { __label__ = 19;; } //@line 1168 "numbers.cpp"
            if (__label__ == 17) {
  
              var $24=HEAP32[((__ZL14number_control)>>2)]; //@line 1171 "numbers.cpp"
              var $and47=($24) & 1; //@line 1171 "numbers.cpp"
              var $tobool48=((($and47))|0)!=0; //@line 1171 "numbers.cpp"
              if ($tobool48) { __label__ = 18;; } else { __label__ = 20;; } //@line 1171 "numbers.cpp"
              if (__label__ == 18) {
  
                var $arraydecay50=(($string)&4294967295); //@line 1174 "numbers.cpp"
                var $25=$value_addr; //@line 1174 "numbers.cpp"
                var $call51=_sprintf($arraydecay50, ((__str26323)&4294967295), allocate([$25,0,0,0], ["i32",0,0,0], ALLOC_STACK)); //@line 1174 "numbers.cpp"
                var $26=$tr_addr; //@line 1175 "numbers.cpp"
                var $arraydecay52=(($string)&4294967295); //@line 1175 "numbers.cpp"
                var $arraydecay53=(($ph_digits)&4294967295); //@line 1175 "numbers.cpp"
                var $call54=__Z6LookupP10TranslatorPKcPc($26, $arraydecay52, $arraydecay53); //@line 1175 "numbers.cpp"
                $found=$call54; //@line 1175 "numbers.cpp"
                __lastLabel__ = 18; ; //@line 1176 "numbers.cpp"
              }
              else if (__label__ == 20) {
  
                var $_pr2=$found; //@line 1185 "numbers.cpp"
                __lastLabel__ = 20; ;
              }
            }
            else if (__label__ == 19) {
  
              var $arraydecay57=(($string)&4294967295); //@line 1181 "numbers.cpp"
              var $27=$value_addr; //@line 1181 "numbers.cpp"
              var $call58=_sprintf($arraydecay57, ((__str27324)&4294967295), allocate([$27,0,0,0], ["i32",0,0,0], ALLOC_STACK)); //@line 1181 "numbers.cpp"
              var $28=$tr_addr; //@line 1182 "numbers.cpp"
              var $arraydecay59=(($string)&4294967295); //@line 1182 "numbers.cpp"
              var $arraydecay60=(($ph_digits)&4294967295); //@line 1182 "numbers.cpp"
              var $call61=__Z6LookupP10TranslatorPKcPc($28, $arraydecay59, $arraydecay60); //@line 1182 "numbers.cpp"
              $found=$call61; //@line 1182 "numbers.cpp"
              __lastLabel__ = 19; ;
            }
  
            var $29=__lastLabel__ == 20 ? $_pr2 : (__lastLabel__ == 18 ? $call54 : ($call61)); //@line 1185 "numbers.cpp"
            var $tobool63=((($29))|0)!=0; //@line 1185 "numbers.cpp"
            if ($tobool63) { __label__ = 23;break $if_then9$$if_end72$8; } //@line 1185 "numbers.cpp"
  
            var $arraydecay65=(($string)&4294967295); //@line 1187 "numbers.cpp"
            var $30=$value_addr; //@line 1187 "numbers.cpp"
            var $call66=_sprintf($arraydecay65, ((__str28325)&4294967295), allocate([$30,0,0,0], ["i32",0,0,0], ALLOC_STACK)); //@line 1187 "numbers.cpp"
            var $31=$tr_addr; //@line 1188 "numbers.cpp"
            var $arraydecay67=(($string)&4294967295); //@line 1188 "numbers.cpp"
            var $arraydecay68=(($ph_digits)&4294967295); //@line 1188 "numbers.cpp"
            var $call69=__Z6LookupP10TranslatorPKcPc($31, $arraydecay67, $arraydecay68); //@line 1188 "numbers.cpp"
            $found=$call69; //@line 1188 "numbers.cpp"
            ; //@line 1189 "numbers.cpp"
          }
        } while(0);
  
        var $32=$control_addr; //@line 1195 "numbers.cpp"
        var $and73=($32) & 16; //@line 1195 "numbers.cpp"
        var $tobool74=((($and73))|0)!=0; //@line 1195 "numbers.cpp"
        if ($tobool74) { __label__ = 24;; } else { __label__ = 26;; } //@line 1195 "numbers.cpp"
        $land_lhs_true75$$if_else80$34: do { 
          if (__label__ == 24) {
  
            var $33=$value_addr; //@line 1195 "numbers.cpp"
            var $cmp76=((($33))|0) < 10; //@line 1195 "numbers.cpp"
            if (!($cmp76)) { __label__ = 26;break $land_lhs_true75$$if_else80$34; } //@line 1195 "numbers.cpp"
  
            var $34=$tr_addr; //@line 1198 "numbers.cpp"
            var $arraydecay78=(($ph_tens)&4294967295); //@line 1198 "numbers.cpp"
            var $call79=__Z6LookupP10TranslatorPKcPc($34, ((__str16313)&4294967295), $arraydecay78); //@line 1198 "numbers.cpp"
            __label__ = 61;break $land_lhs_true75$$if_else80$34; //@line 1199 "numbers.cpp"
          }
        } while(0);
        $if_end221$$if_else80$37: do { 
          if (__label__ == 26) {
  
            var $35=$found; //@line 1202 "numbers.cpp"
            var $tobool81=((($35))|0)!=0; //@line 1202 "numbers.cpp"
            if ($tobool81) { __label__ = 27;; } else { __label__ = 28;; } //@line 1202 "numbers.cpp"
            if (__label__ == 27) {
  
              var $arrayidx83=(($ph_tens)&4294967295); //@line 1204 "numbers.cpp"
              HEAP8[($arrayidx83)]=0; //@line 1204 "numbers.cpp"
              ; //@line 1205 "numbers.cpp"
            }
            else if (__label__ == 28) {
  
              var $36=$control_addr; //@line 1209 "numbers.cpp"
              var $and85=($36) & 1; //@line 1209 "numbers.cpp"
              var $tobool86=((($and85))|0)!=0; //@line 1209 "numbers.cpp"
              if ($tobool86) { __label__ = 29;; } else { __label__ = 36;; } //@line 1209 "numbers.cpp"
              $land_lhs_true87$$if_end114$42: do { 
                if (__label__ == 29) {
  
                  var $37=$units; //@line 1209 "numbers.cpp"
                  var $cmp88=((($37))|0)==0; //@line 1209 "numbers.cpp"
                  if ($cmp88) { __label__ = 32;; } else { __label__ = 30;; } //@line 1209 "numbers.cpp"
                  $if_then95$$lor_lhs_false$44: do { 
                    if (__label__ == 30) {
  
                      var $38=$tr_addr; //@line 1209 "numbers.cpp"
                      var $langopts=(($38)&4294967295); //@line 1209 "numbers.cpp"
                      var $numbers=(($langopts+184)&4294967295); //@line 1209 "numbers.cpp"
                      var $39=HEAP32[(($numbers)>>2)]; //@line 1209 "numbers.cpp"
                      var $and89=($39) & 16; //@line 1209 "numbers.cpp"
                      var $tobool90=((($and89))|0)!=0; //@line 1209 "numbers.cpp"
                      if ($tobool90) { __label__ = 32;break $if_then95$$lor_lhs_false$44; } //@line 1209 "numbers.cpp"
  
                      var $40=$tr_addr; //@line 1209 "numbers.cpp"
                      var $langopts92=(($40)&4294967295); //@line 1209 "numbers.cpp"
                      var $numbers2=(($langopts92+188)&4294967295); //@line 1209 "numbers.cpp"
                      var $41=HEAP32[(($numbers2)>>2)]; //@line 1209 "numbers.cpp"
                      var $and93=($41) & 4096; //@line 1209 "numbers.cpp"
                      var $tobool94=((($and93))|0)!=0; //@line 1209 "numbers.cpp"
                      if (!($tobool94)) { __label__ = 36;break $land_lhs_true87$$if_end114$42; } //@line 1209 "numbers.cpp"
                    }
                  } while(0);
  
                  var $arraydecay96=(($string)&4294967295); //@line 1212 "numbers.cpp"
                  var $42=$tens; //@line 1212 "numbers.cpp"
                  var $43=$ord_type; //@line 1212 "numbers.cpp"
                  var $call97=_sprintf($arraydecay96, ((__str29326)&4294967295), allocate([$42,0,0,0,$43,0,0,0], ["i32",0,0,0,"i32",0,0,0], ALLOC_STACK)); //@line 1212 "numbers.cpp"
                  var $44=$tr_addr; //@line 1213 "numbers.cpp"
                  var $arraydecay98=(($string)&4294967295); //@line 1213 "numbers.cpp"
                  var $arraydecay99=(($ph_tens)&4294967295); //@line 1213 "numbers.cpp"
                  var $call100=__Z6LookupP10TranslatorPKcPc($44, $arraydecay98, $arraydecay99); //@line 1213 "numbers.cpp"
                  var $cmp101=((($call100))|0)!=0; //@line 1213 "numbers.cpp"
                  if (!($cmp101)) { __label__ = 36;break $land_lhs_true87$$if_end114$42; } //@line 1213 "numbers.cpp"
  
                  $found_ordinal=1; //@line 1215 "numbers.cpp"
                  var $45=$units; //@line 1217 "numbers.cpp"
                  var $cmp103=((($45))|0)!=0; //@line 1217 "numbers.cpp"
                  if (!($cmp103)) { __label__ = 38;break $land_lhs_true87$$if_end114$42; } //@line 1217 "numbers.cpp"
  
                  var $46=$tr_addr; //@line 1217 "numbers.cpp"
                  var $langopts105=(($46)&4294967295); //@line 1217 "numbers.cpp"
                  var $numbers2106=(($langopts105+188)&4294967295); //@line 1217 "numbers.cpp"
                  var $47=HEAP32[(($numbers2106)>>2)]; //@line 1217 "numbers.cpp"
                  var $and107=($47) & 4096; //@line 1217 "numbers.cpp"
                  var $tobool108=((($and107))|0)!=0; //@line 1217 "numbers.cpp"
                  if (!($tobool108)) { __label__ = 36;break $land_lhs_true87$$if_end114$42; } //@line 1217 "numbers.cpp"
  
                  var $arraydecay110=(($ph_tens)&4294967295); //@line 1220 "numbers.cpp"
                  var $call111=_strcat($arraydecay110, ((__ZL11ph_ordinal2)&4294967295)); //@line 1220 "numbers.cpp"
                  __label__ = 36;break $land_lhs_true87$$if_end114$42; //@line 1221 "numbers.cpp"
                }
              } while(0);
              $if_end114$$if_end122$51: do { 
                if (__label__ == 36) {
  
                  var $_pr3=$found_ordinal; //@line 1224 "numbers.cpp"
                  var $cmp115=((($_pr3))|0)==0; //@line 1224 "numbers.cpp"
                  if (!($cmp115)) { __label__ = 38;break $if_end114$$if_end122$51; } //@line 1224 "numbers.cpp"
  
                  var $arraydecay117=(($string)&4294967295); //@line 1226 "numbers.cpp"
                  var $48=$tens; //@line 1226 "numbers.cpp"
                  var $call118=_sprintf($arraydecay117, ((__str30327)&4294967295), allocate([$48,0,0,0], ["i32",0,0,0], ALLOC_STACK)); //@line 1226 "numbers.cpp"
                  var $49=$tr_addr; //@line 1227 "numbers.cpp"
                  var $arraydecay119=(($string)&4294967295); //@line 1227 "numbers.cpp"
                  var $arraydecay120=(($ph_tens)&4294967295); //@line 1227 "numbers.cpp"
                  var $call121=__Z6LookupP10TranslatorPKcPc($49, $arraydecay119, $arraydecay120); //@line 1227 "numbers.cpp"
                  ; //@line 1228 "numbers.cpp"
                }
              } while(0);
  
              var $arrayidx123=(($ph_tens)&4294967295); //@line 1230 "numbers.cpp"
              var $50=HEAP8[($arrayidx123)]; //@line 1230 "numbers.cpp"
              var $conv124=(tempInt=(($50)),(tempInt>=128?tempInt-256:tempInt)); //@line 1230 "numbers.cpp"
              var $cmp125=((($conv124))|0)==0; //@line 1230 "numbers.cpp"
              if ($cmp125) { __label__ = 39;; } else { __label__ = 41;; } //@line 1230 "numbers.cpp"
              $land_lhs_true126$$if_end139$55: do { 
                if (__label__ == 39) {
  
                  var $51=$tr_addr; //@line 1230 "numbers.cpp"
                  var $langopts127=(($51)&4294967295); //@line 1230 "numbers.cpp"
                  var $numbers128=(($langopts127+184)&4294967295); //@line 1230 "numbers.cpp"
                  var $52=HEAP32[(($numbers128)>>2)]; //@line 1230 "numbers.cpp"
                  var $and129=($52) & 1048576; //@line 1230 "numbers.cpp"
                  var $tobool130=((($and129))|0)!=0; //@line 1230 "numbers.cpp"
                  if (!($tobool130)) { __label__ = 41;break $land_lhs_true126$$if_end139$55; } //@line 1230 "numbers.cpp"
  
                  var $53=$value_addr; //@line 1233 "numbers.cpp"
                  var $rem132=((($53))|0)%20; //@line 1233 "numbers.cpp"
                  $units=$rem132; //@line 1233 "numbers.cpp"
                  var $arraydecay133=(($string)&4294967295); //@line 1234 "numbers.cpp"
                  var $54=$tens; //@line 1234 "numbers.cpp"
                  var $and134=($54) & 254; //@line 1234 "numbers.cpp"
                  var $call135=_sprintf($arraydecay133, ((__str30327)&4294967295), allocate([$and134,0,0,0], ["i32",0,0,0], ALLOC_STACK)); //@line 1234 "numbers.cpp"
                  var $55=$tr_addr; //@line 1235 "numbers.cpp"
                  var $arraydecay136=(($string)&4294967295); //@line 1235 "numbers.cpp"
                  var $arraydecay137=(($ph_tens)&4294967295); //@line 1235 "numbers.cpp"
                  var $call138=__Z6LookupP10TranslatorPKcPc($55, $arraydecay136, $arraydecay137); //@line 1235 "numbers.cpp"
                  ; //@line 1236 "numbers.cpp"
                }
              } while(0);
  
              var $arrayidx140=(($ph_digits)&4294967295); //@line 1238 "numbers.cpp"
              HEAP8[($arrayidx140)]=0; //@line 1238 "numbers.cpp"
              var $56=$units; //@line 1239 "numbers.cpp"
              var $cmp141=((($56))|0) > 0; //@line 1239 "numbers.cpp"
              if (!($cmp141)) { __label__ = 61;break $if_end221$$if_else80$37; } //@line 1239 "numbers.cpp"
  
              $found=0; //@line 1241 "numbers.cpp"
              var $57=$control_addr; //@line 1243 "numbers.cpp"
              var $and143=($57) & 2; //@line 1243 "numbers.cpp"
              var $tobool144=((($and143))|0)!=0; //@line 1243 "numbers.cpp"
              if ($tobool144) { __label__ = 43;; } else { __label__ = 45;; } //@line 1243 "numbers.cpp"
              $land_lhs_true145$$if_else153$60: do { 
                if (__label__ == 43) {
  
                  var $58=HEAP32[((__ZL12digit_lookup)>>2)]; //@line 1243 "numbers.cpp"
                  var $arrayidx146=(($58)&4294967295); //@line 1243 "numbers.cpp"
                  var $59=HEAP8[($arrayidx146)]; //@line 1243 "numbers.cpp"
                  var $conv147=(tempInt=(($59)),(tempInt>=128?tempInt-256:tempInt)); //@line 1243 "numbers.cpp"
                  var $cmp148=((($conv147))|0)!=0; //@line 1243 "numbers.cpp"
                  if (!($cmp148)) { __label__ = 45;break $land_lhs_true145$$if_else153$60; } //@line 1243 "numbers.cpp"
  
                  var $arraydecay150=(($ph_digits)&4294967295); //@line 1246 "numbers.cpp"
                  var $60=HEAP32[((__ZL12digit_lookup)>>2)]; //@line 1246 "numbers.cpp"
                  var $call151=_strcpy($arraydecay150, $60); //@line 1246 "numbers.cpp"
                  $found_ordinal=1; //@line 1247 "numbers.cpp"
                  var $arrayidx152=(($ph_ordinal)&4294967295); //@line 1248 "numbers.cpp"
                  HEAP8[($arrayidx152)]=0; //@line 1248 "numbers.cpp"
                  __label__ = 61;break $if_end221$$if_else80$37; //@line 1249 "numbers.cpp"
                }
              } while(0);
  
              var $61=$control_addr; //@line 1252 "numbers.cpp"
              var $and154=($61) & 8; //@line 1252 "numbers.cpp"
              var $tobool155=((($and154))|0)!=0; //@line 1252 "numbers.cpp"
              if ($tobool155) { __label__ = 46;; } else { __label__ = 47;; } //@line 1252 "numbers.cpp"
              if (__label__ == 46) {
  
                var $arraydecay157=(($string)&4294967295); //@line 1255 "numbers.cpp"
                var $62=$units; //@line 1255 "numbers.cpp"
                var $call158=_sprintf($arraydecay157, ((__str23320)&4294967295), allocate([$62,0,0,0], ["i32",0,0,0], ALLOC_STACK)); //@line 1255 "numbers.cpp"
                var $63=$tr_addr; //@line 1256 "numbers.cpp"
                var $arraydecay159=(($string)&4294967295); //@line 1256 "numbers.cpp"
                var $arraydecay160=(($ph_digits)&4294967295); //@line 1256 "numbers.cpp"
                var $call161=__Z6LookupP10TranslatorPKcPc($63, $arraydecay159, $arraydecay160); //@line 1256 "numbers.cpp"
                $found=$call161; //@line 1256 "numbers.cpp"
                ; //@line 1257 "numbers.cpp"
              }
  
              var $64=$control_addr; //@line 1258 "numbers.cpp"
              var $and163=($64) & 1; //@line 1258 "numbers.cpp"
              var $tobool164=((($and163))|0)!=0; //@line 1258 "numbers.cpp"
              if ($tobool164) { __label__ = 48;; } else { __label__ = 51;; } //@line 1258 "numbers.cpp"
              $land_lhs_true165$$if_end179$67: do { 
                if (__label__ == 48) {
  
                  var $65=$tr_addr; //@line 1258 "numbers.cpp"
                  var $langopts166=(($65)&4294967295); //@line 1258 "numbers.cpp"
                  var $numbers167=(($langopts166+184)&4294967295); //@line 1258 "numbers.cpp"
                  var $66=HEAP32[(($numbers167)>>2)]; //@line 1258 "numbers.cpp"
                  var $and168=($66) & 16; //@line 1258 "numbers.cpp"
                  var $cmp169=((($and168))|0)==0; //@line 1258 "numbers.cpp"
                  if (!($cmp169)) { __label__ = 51;break $land_lhs_true165$$if_end179$67; } //@line 1258 "numbers.cpp"
  
                  var $arraydecay171=(($string)&4294967295); //@line 1261 "numbers.cpp"
                  var $67=$units; //@line 1261 "numbers.cpp"
                  var $68=$ord_type; //@line 1261 "numbers.cpp"
                  var $call172=_sprintf($arraydecay171, ((__str25322)&4294967295), allocate([$67,0,0,0,$68,0,0,0], ["i32",0,0,0,"i32",0,0,0], ALLOC_STACK)); //@line 1261 "numbers.cpp"
                  var $69=$tr_addr; //@line 1262 "numbers.cpp"
                  var $arraydecay173=(($string)&4294967295); //@line 1262 "numbers.cpp"
                  var $arraydecay174=(($ph_digits)&4294967295); //@line 1262 "numbers.cpp"
                  var $call175=__Z6LookupP10TranslatorPKcPc($69, $arraydecay173, $arraydecay174); //@line 1262 "numbers.cpp"
                  $found=$call175; //@line 1262 "numbers.cpp"
                  var $cmp176=((($call175))|0)!=0; //@line 1262 "numbers.cpp"
                  if (!($cmp176)) { __label__ = 52;break $land_lhs_true165$$if_end179$67; } //@line 1262 "numbers.cpp"
  
                  $found_ordinal=1; //@line 1264 "numbers.cpp"
                  __label__ = 51;break $land_lhs_true165$$if_end179$67; //@line 1265 "numbers.cpp"
                }
              } while(0);
              if (__label__ == 51) {
  
                var $_pr5=$found; //@line 1267 "numbers.cpp"
                var $cmp180=((($_pr5))|0)==0; //@line 1267 "numbers.cpp"
                if (!($cmp180)) { __label__ = 61;break $if_end221$$if_else80$37; } //@line 1267 "numbers.cpp"
              }
  
              var $70=HEAP32[((__ZL14number_control)>>2)]; //@line 1269 "numbers.cpp"
              var $and182=($70) & 1; //@line 1269 "numbers.cpp"
              var $tobool183=((($and182))|0)!=0; //@line 1269 "numbers.cpp"
              if ($tobool183) { __label__ = 53;; } else { __label__ = 55;; } //@line 1269 "numbers.cpp"
              $land_lhs_true184$$if_else193$74: do { 
                if (__label__ == 53) {
  
                  var $71=$control_addr; //@line 1269 "numbers.cpp"
                  var $and185=($71) & 2; //@line 1269 "numbers.cpp"
                  var $tobool186=((($and185))|0)!=0; //@line 1269 "numbers.cpp"
                  if (!($tobool186)) { __label__ = 55;break $land_lhs_true184$$if_else193$74; } //@line 1269 "numbers.cpp"
  
                  var $arraydecay188=(($string)&4294967295); //@line 1272 "numbers.cpp"
                  var $72=$units; //@line 1272 "numbers.cpp"
                  var $call189=_sprintf($arraydecay188, ((__str26323)&4294967295), allocate([$72,0,0,0], ["i32",0,0,0], ALLOC_STACK)); //@line 1272 "numbers.cpp"
                  var $73=$tr_addr; //@line 1273 "numbers.cpp"
                  var $arraydecay190=(($string)&4294967295); //@line 1273 "numbers.cpp"
                  var $arraydecay191=(($ph_digits)&4294967295); //@line 1273 "numbers.cpp"
                  var $call192=__Z6LookupP10TranslatorPKcPc($73, $arraydecay190, $arraydecay191); //@line 1273 "numbers.cpp"
                  $found=$call192; //@line 1273 "numbers.cpp"
                  __lastLabel__ = 54; __label__ = 59;break $land_lhs_true184$$if_else193$74; //@line 1274 "numbers.cpp"
                }
              } while(0);
              $if_end209$$if_else193$77: do { 
                if (__label__ == 55) {
  
                  var $74=$control_addr; //@line 1276 "numbers.cpp"
                  var $and194=($74) & 2; //@line 1276 "numbers.cpp"
                  var $cmp195=((($and194))|0)==0; //@line 1276 "numbers.cpp"
                  if ($cmp195) { __label__ = 57;; } else { __label__ = 56;; } //@line 1276 "numbers.cpp"
                  $if_then201$$lor_lhs_false196$79: do { 
                    if (__label__ == 56) {
  
                      var $75=$tr_addr; //@line 1276 "numbers.cpp"
                      var $langopts197=(($75)&4294967295); //@line 1276 "numbers.cpp"
                      var $numbers198=(($langopts197+184)&4294967295); //@line 1276 "numbers.cpp"
                      var $76=HEAP32[(($numbers198)>>2)]; //@line 1276 "numbers.cpp"
                      var $and199=($76) & 16; //@line 1276 "numbers.cpp"
                      var $cmp200=((($and199))|0)!=0; //@line 1276 "numbers.cpp"
                      if ($cmp200) { __label__ = 57;break $if_then201$$lor_lhs_false196$79; } //@line 1276 "numbers.cpp"
  
                      var $_pr7=$found; //@line 1283 "numbers.cpp"
                      __lastLabel__ = 58; __label__ = 59;break $if_end209$$if_else193$77;
                    }
                  } while(0);
  
                  var $arraydecay202=(($string)&4294967295); //@line 1279 "numbers.cpp"
                  var $77=$units; //@line 1279 "numbers.cpp"
                  var $call203=_sprintf($arraydecay202, ((__str27324)&4294967295), allocate([$77,0,0,0], ["i32",0,0,0], ALLOC_STACK)); //@line 1279 "numbers.cpp"
                  var $78=$tr_addr; //@line 1280 "numbers.cpp"
                  var $arraydecay204=(($string)&4294967295); //@line 1280 "numbers.cpp"
                  var $arraydecay205=(($ph_digits)&4294967295); //@line 1280 "numbers.cpp"
                  var $call206=__Z6LookupP10TranslatorPKcPc($78, $arraydecay204, $arraydecay205); //@line 1280 "numbers.cpp"
                  $found=$call206; //@line 1280 "numbers.cpp"
                  __lastLabel__ = 57; ; //@line 1281 "numbers.cpp"
                }
              } while(0);
  
              var $79=__lastLabel__ == 58 ? $_pr7 : (__lastLabel__ == 54 ? $call192 : ($call206)); //@line 1283 "numbers.cpp"
              var $cmp210=((($79))|0)==0; //@line 1283 "numbers.cpp"
              if (!($cmp210)) { __label__ = 61;break $if_end221$$if_else80$37; } //@line 1283 "numbers.cpp"
  
              var $arraydecay212=(($string)&4294967295); //@line 1285 "numbers.cpp"
              var $80=$units; //@line 1285 "numbers.cpp"
              var $call213=_sprintf($arraydecay212, ((__str28325)&4294967295), allocate([$80,0,0,0], ["i32",0,0,0], ALLOC_STACK)); //@line 1285 "numbers.cpp"
              var $81=$tr_addr; //@line 1286 "numbers.cpp"
              var $arraydecay214=(($string)&4294967295); //@line 1286 "numbers.cpp"
              var $arraydecay215=(($ph_digits)&4294967295); //@line 1286 "numbers.cpp"
              var $call216=__Z6LookupP10TranslatorPKcPc($81, $arraydecay214, $arraydecay215); //@line 1286 "numbers.cpp"
              ; //@line 1287 "numbers.cpp"
            }
          }
        } while(0);
  
        var $82=$control_addr; //@line 1293 "numbers.cpp"
        var $and222=($82) & 1; //@line 1293 "numbers.cpp"
        var $tobool223=((($and222))|0)!=0; //@line 1293 "numbers.cpp"
        if ($tobool223) { __label__ = 62;; } else { __label__ = 70;; } //@line 1293 "numbers.cpp"
        $land_lhs_true224$$if_end251$86: do { 
          if (__label__ == 62) {
  
            var $83=$found_ordinal; //@line 1293 "numbers.cpp"
            var $cmp225=((($83))|0)==0; //@line 1293 "numbers.cpp"
            if (!($cmp225)) { __label__ = 70;break $land_lhs_true224$$if_end251$86; } //@line 1293 "numbers.cpp"
  
            var $arrayidx227=(($ph_ordinal)&4294967295); //@line 1293 "numbers.cpp"
            var $84=HEAP8[($arrayidx227)]; //@line 1293 "numbers.cpp"
            var $conv228=(tempInt=(($84)),(tempInt>=128?tempInt-256:tempInt)); //@line 1293 "numbers.cpp"
            var $cmp229=((($conv228))|0)==0; //@line 1293 "numbers.cpp"
            if (!($cmp229)) { __label__ = 70;break $land_lhs_true224$$if_end251$86; } //@line 1293 "numbers.cpp"
  
            var $85=$value_addr; //@line 1295 "numbers.cpp"
            var $cmp231=((($85))|0) >= 20; //@line 1295 "numbers.cpp"
            if ($cmp231) { __label__ = 65;; } else { __label__ = 68;; } //@line 1295 "numbers.cpp"
            $land_lhs_true232$$if_end243$90: do { 
              if (__label__ == 65) {
  
                var $86=$value_addr; //@line 1295 "numbers.cpp"
                var $rem233=((($86))|0)%10; //@line 1295 "numbers.cpp"
                var $cmp234=((($rem233))|0)==0; //@line 1295 "numbers.cpp"
                if ($cmp234) { __label__ = 67;; } else { __label__ = 66;; } //@line 1295 "numbers.cpp"
                if (__label__ == 66) {
  
                  var $87=$tr_addr; //@line 1295 "numbers.cpp"
                  var $langopts236=(($87)&4294967295); //@line 1295 "numbers.cpp"
                  var $numbers237=(($langopts236+184)&4294967295); //@line 1295 "numbers.cpp"
                  var $88=HEAP32[(($numbers237)>>2)]; //@line 1295 "numbers.cpp"
                  var $and238=($88) & 16; //@line 1295 "numbers.cpp"
                  var $tobool239=((($and238))|0)!=0; //@line 1295 "numbers.cpp"
                  if (!($tobool239)) { __label__ = 68;break $land_lhs_true232$$if_end243$90; } //@line 1295 "numbers.cpp"
                }
  
                var $89=$tr_addr; //@line 1296 "numbers.cpp"
                var $arraydecay241=(($ph_ordinal)&4294967295); //@line 1296 "numbers.cpp"
                var $call242=__Z6LookupP10TranslatorPKcPc($89, ((__str31328)&4294967295), $arraydecay241); //@line 1296 "numbers.cpp"
                ; //@line 1296 "numbers.cpp"
              }
            } while(0);
  
            var $arrayidx244=(($ph_ordinal)&4294967295); //@line 1297 "numbers.cpp"
            var $90=HEAP8[($arrayidx244)]; //@line 1297 "numbers.cpp"
            var $conv245=(tempInt=(($90)),(tempInt>=128?tempInt-256:tempInt)); //@line 1297 "numbers.cpp"
            var $cmp246=((($conv245))|0)==0; //@line 1297 "numbers.cpp"
            if (!($cmp246)) { __label__ = 70;break $land_lhs_true224$$if_end251$86; } //@line 1297 "numbers.cpp"
  
            var $91=$tr_addr; //@line 1298 "numbers.cpp"
            var $arraydecay248=(($ph_ordinal)&4294967295); //@line 1298 "numbers.cpp"
            var $call249=__Z6LookupP10TranslatorPKcPc($91, ((__str32329)&4294967295), $arraydecay248); //@line 1298 "numbers.cpp"
            ; //@line 1298 "numbers.cpp"
          }
        } while(0);
  
        var $92=$tr_addr; //@line 1301 "numbers.cpp"
        var $langopts252=(($92)&4294967295); //@line 1301 "numbers.cpp"
        var $numbers253=(($langopts252+184)&4294967295); //@line 1301 "numbers.cpp"
        var $93=HEAP32[(($numbers253)>>2)]; //@line 1301 "numbers.cpp"
        var $and254=($93) & 48; //@line 1301 "numbers.cpp"
        var $tobool255=((($and254))|0)!=0; //@line 1301 "numbers.cpp"
        if ($tobool255) { __label__ = 71;; } else { __label__ = 79;; } //@line 1301 "numbers.cpp"
        $land_lhs_true256$$if_else291$98: do { 
          if (__label__ == 71) {
  
            var $arrayidx257=(($ph_tens)&4294967295); //@line 1301 "numbers.cpp"
            var $94=HEAP8[($arrayidx257)]; //@line 1301 "numbers.cpp"
            var $conv258=(tempInt=(($94)),(tempInt>=128?tempInt-256:tempInt)); //@line 1301 "numbers.cpp"
            var $cmp259=((($conv258))|0)!=0; //@line 1301 "numbers.cpp"
            if (!($cmp259)) { __label__ = 79;break $land_lhs_true256$$if_else291$98; } //@line 1301 "numbers.cpp"
  
            var $arrayidx261=(($ph_digits)&4294967295); //@line 1301 "numbers.cpp"
            var $95=HEAP8[($arrayidx261)]; //@line 1301 "numbers.cpp"
            var $conv262=(tempInt=(($95)),(tempInt>=128?tempInt-256:tempInt)); //@line 1301 "numbers.cpp"
            var $cmp263=((($conv262))|0)!=0; //@line 1301 "numbers.cpp"
            if (!($cmp263)) { __label__ = 79;break $land_lhs_true256$$if_else291$98; } //@line 1301 "numbers.cpp"
  
            var $96=$tr_addr; //@line 1303 "numbers.cpp"
            var $arraydecay265=(($ph_and)&4294967295); //@line 1303 "numbers.cpp"
            var $call266=__Z6LookupP10TranslatorPKcPc($96, ((__str33330)&4294967295), $arraydecay265); //@line 1303 "numbers.cpp"
            var $97=$tr_addr; //@line 1305 "numbers.cpp"
            var $langopts267=(($97)&4294967295); //@line 1305 "numbers.cpp"
            var $numbers2268=(($langopts267+188)&4294967295); //@line 1305 "numbers.cpp"
            var $98=HEAP32[(($numbers2268)>>2)]; //@line 1305 "numbers.cpp"
            var $and269=($98) & 4096; //@line 1305 "numbers.cpp"
            var $tobool270=((($and269))|0)!=0; //@line 1305 "numbers.cpp"
            if ($tobool270) { __label__ = 74;; } else { __label__ = 75;; } //@line 1305 "numbers.cpp"
            if (__label__ == 74) {
  
              var $arrayidx272=(($ph_and)&4294967295); //@line 1306 "numbers.cpp"
              HEAP8[($arrayidx272)]=0; //@line 1306 "numbers.cpp"
              ; //@line 1306 "numbers.cpp"
            }
  
            var $99=$tr_addr; //@line 1308 "numbers.cpp"
            var $langopts274=(($99)&4294967295); //@line 1308 "numbers.cpp"
            var $numbers275=(($langopts274+184)&4294967295); //@line 1308 "numbers.cpp"
            var $100=HEAP32[(($numbers275)>>2)]; //@line 1308 "numbers.cpp"
            var $and276=($100) & 16; //@line 1308 "numbers.cpp"
            var $tobool277=((($and276))|0)!=0; //@line 1308 "numbers.cpp"
            var $101=$ph_out_addr; //@line 1309 "numbers.cpp"
            if ($tobool277) { __label__ = 76;; } else { __label__ = 77;; } //@line 1308 "numbers.cpp"
            if (__label__ == 76) {
  
              var $arraydecay279=(($ph_digits)&4294967295); //@line 1309 "numbers.cpp"
              var $arraydecay280=(($ph_and)&4294967295); //@line 1309 "numbers.cpp"
              var $arraydecay281=(($ph_tens)&4294967295); //@line 1309 "numbers.cpp"
              var $arraydecay282=(($ph_ordinal)&4294967295); //@line 1309 "numbers.cpp"
              var $call283=_sprintf($101, ((__str20317)&4294967295), allocate([$arraydecay279,0,0,0,$arraydecay280,0,0,0,$arraydecay281,0,0,0,$arraydecay282,0,0,0], ["i8*",0,0,0,"i8*",0,0,0,"i8*",0,0,0,"i8*",0,0,0], ALLOC_STACK)); //@line 1309 "numbers.cpp"
              ; //@line 1309 "numbers.cpp"
            }
            else if (__label__ == 77) {
  
              var $arraydecay285=(($ph_tens)&4294967295); //@line 1311 "numbers.cpp"
              var $arraydecay286=(($ph_and)&4294967295); //@line 1311 "numbers.cpp"
              var $arraydecay287=(($ph_digits)&4294967295); //@line 1311 "numbers.cpp"
              var $arraydecay288=(($ph_ordinal)&4294967295); //@line 1311 "numbers.cpp"
              var $call289=_sprintf($101, ((__str20317)&4294967295), allocate([$arraydecay285,0,0,0,$arraydecay286,0,0,0,$arraydecay287,0,0,0,$arraydecay288,0,0,0], ["i8*",0,0,0,"i8*",0,0,0,"i8*",0,0,0,"i8*",0,0,0], ALLOC_STACK)); //@line 1311 "numbers.cpp"
              ;
            }
  
            $used_and=1; //@line 1312 "numbers.cpp"
            __label__ = 88;break $if_then6$$if_else$5; //@line 1313 "numbers.cpp"
          }
        } while(0);
  
        var $102=$tr_addr; //@line 1316 "numbers.cpp"
        var $langopts292=(($102)&4294967295); //@line 1316 "numbers.cpp"
        var $numbers293=(($langopts292+184)&4294967295); //@line 1316 "numbers.cpp"
        var $103=HEAP32[(($numbers293)>>2)]; //@line 1316 "numbers.cpp"
        var $and294=($103) & 512; //@line 1316 "numbers.cpp"
        var $tobool295=((($and294))|0)!=0; //@line 1316 "numbers.cpp"
        if ($tobool295) { __label__ = 80;; } else { __label__ = 87;; } //@line 1316 "numbers.cpp"
        $if_then296$$if_end329$110: do { 
          if (__label__ == 80) {
  
            var $arraydecay297=(($ph_tens)&4294967295); //@line 1319 "numbers.cpp"
            var $call298=_strlen($arraydecay297); //@line 1319 "numbers.cpp"
            var $sub=((($call298)-1)&4294967295); //@line 1319 "numbers.cpp"
            $ix=$sub; //@line 1319 "numbers.cpp"
            var $cmp299=((($sub))|0) >= 0; //@line 1319 "numbers.cpp"
            if (!($cmp299)) { __label__ = 87;break $if_then296$$if_end329$110; } //@line 1319 "numbers.cpp"
  
            var $arrayidx301=(($ph_digits)&4294967295); //@line 1319 "numbers.cpp"
            var $104=HEAP8[($arrayidx301)]; //@line 1319 "numbers.cpp"
            var $conv302=(tempInt=(($104)),(tempInt>=128?tempInt-256:tempInt)); //@line 1319 "numbers.cpp"
            var $cmp303=((($conv302))|0)!=0; //@line 1319 "numbers.cpp"
            if (!($cmp303)) { __label__ = 87;break $if_then296$$if_end329$110; } //@line 1319 "numbers.cpp"
  
            var $arrayidx305=(($ph_digits)&4294967295); //@line 1321 "numbers.cpp"
            var $105=HEAP8[($arrayidx305)]; //@line 1321 "numbers.cpp"
            var $conv306=(tempInt=(($105)),(tempInt>=128?tempInt-256:tempInt)); //@line 1321 "numbers.cpp"
            var $arrayidx307=((_phoneme_tab+$conv306*4)&4294967295); //@line 1321 "numbers.cpp"
            var $106=HEAP32[(($arrayidx307)>>2)]; //@line 1321 "numbers.cpp"
            var $type=(($106+11)&4294967295); //@line 1321 "numbers.cpp"
            var $107=HEAPU8[($type)]; //@line 1321 "numbers.cpp"
            var $conv308=((($107))&255); //@line 1321 "numbers.cpp"
            $next_phtype=$conv308; //@line 1321 "numbers.cpp"
            var $cmp309=((($conv308))|0)==1; //@line 1321 "numbers.cpp"
            if ($cmp309) { __label__ = 83;; } else { __label__ = 84;; } //@line 1321 "numbers.cpp"
            if (__label__ == 83) {
  
              var $arrayidx311=(($ph_digits+1)&4294967295); //@line 1322 "numbers.cpp"
              var $108=HEAP8[($arrayidx311)]; //@line 1322 "numbers.cpp"
              var $conv312=(tempInt=(($108)),(tempInt>=128?tempInt-256:tempInt)); //@line 1322 "numbers.cpp"
              var $arrayidx313=((_phoneme_tab+$conv312*4)&4294967295); //@line 1322 "numbers.cpp"
              var $109=HEAP32[(($arrayidx313)>>2)]; //@line 1322 "numbers.cpp"
              var $type314=(($109+11)&4294967295); //@line 1322 "numbers.cpp"
              var $110=HEAPU8[($type314)]; //@line 1322 "numbers.cpp"
              var $conv315=((($110))&255); //@line 1322 "numbers.cpp"
              $next_phtype=$conv315; //@line 1322 "numbers.cpp"
              ; //@line 1322 "numbers.cpp"
            }
  
            var $111=$ix; //@line 1324 "numbers.cpp"
            var $arrayidx317=(($ph_tens+$111)&4294967295); //@line 1324 "numbers.cpp"
            var $112=HEAP8[($arrayidx317)]; //@line 1324 "numbers.cpp"
            var $conv318=(tempInt=(($112)),(tempInt>=128?tempInt-256:tempInt)); //@line 1324 "numbers.cpp"
            var $arrayidx319=((_phoneme_tab+$conv318*4)&4294967295); //@line 1324 "numbers.cpp"
            var $113=HEAP32[(($arrayidx319)>>2)]; //@line 1324 "numbers.cpp"
            var $type320=(($113+11)&4294967295); //@line 1324 "numbers.cpp"
            var $114=HEAPU8[($type320)]; //@line 1324 "numbers.cpp"
            var $conv321=((($114))&255); //@line 1324 "numbers.cpp"
            var $cmp322=((($conv321))|0)==2; //@line 1324 "numbers.cpp"
            if (!($cmp322)) { __label__ = 87;break $if_then296$$if_end329$110; } //@line 1324 "numbers.cpp"
  
            var $115=$next_phtype; //@line 1324 "numbers.cpp"
            var $cmp324=((($115))|0)==2; //@line 1324 "numbers.cpp"
            if (!($cmp324)) { __label__ = 87;break $if_then296$$if_end329$110; } //@line 1324 "numbers.cpp"
  
            var $116=$ix; //@line 1325 "numbers.cpp"
            var $arrayidx326=(($ph_tens+$116)&4294967295); //@line 1325 "numbers.cpp"
            HEAP8[($arrayidx326)]=0; //@line 1325 "numbers.cpp"
            ; //@line 1325 "numbers.cpp"
          }
        } while(0);
  
        var $117=$ph_out_addr; //@line 1328 "numbers.cpp"
        var $arraydecay330=(($ph_tens)&4294967295); //@line 1328 "numbers.cpp"
        var $arraydecay331=(($ph_digits)&4294967295); //@line 1328 "numbers.cpp"
        var $arraydecay332=(($ph_ordinal)&4294967295); //@line 1328 "numbers.cpp"
        var $call333=_sprintf($117, ((__str6549)&4294967295), allocate([$arraydecay330,0,0,0,$arraydecay331,0,0,0,$arraydecay332,0,0,0], ["i8*",0,0,0,"i8*",0,0,0,"i8*",0,0,0], ALLOC_STACK)); //@line 1328 "numbers.cpp"
        ;
      }
    } while(0);
  
    var $118=$tr_addr; //@line 1332 "numbers.cpp"
    var $langopts336=(($118)&4294967295); //@line 1332 "numbers.cpp"
    var $numbers337=(($langopts336+184)&4294967295); //@line 1332 "numbers.cpp"
    var $119=HEAP32[(($numbers337)>>2)]; //@line 1332 "numbers.cpp"
    var $and338=($119) & 256; //@line 1332 "numbers.cpp"
    var $tobool339=((($and338))|0)!=0; //@line 1332 "numbers.cpp"
    if ($tobool339) { __label__ = 89;; } else { __label__ = 95;; } //@line 1332 "numbers.cpp"
    $if_then340$$if_end354$121: do { 
      if (__label__ == 89) {
  
        $found=0; //@line 1335 "numbers.cpp"
        var $120=$ph_out_addr; //@line 1336 "numbers.cpp"
        var $call341=_strlen($120); //@line 1336 "numbers.cpp"
        var $sub342=((($call341)-1)&4294967295); //@line 1336 "numbers.cpp"
        $ix=$sub342; //@line 1336 "numbers.cpp"
        var $cmp3439=((($sub342))|0) >= 0; //@line 1336 "numbers.cpp"
        if (!($cmp3439)) { __label__ = 95;break $if_then340$$if_end354$121; } //@line 1336 "numbers.cpp"
        while(1) { 
  
          var $121=$ix; //@line 1338 "numbers.cpp"
          var $122=$ph_out_addr; //@line 1338 "numbers.cpp"
          var $arrayidx344=(($122+$121)&4294967295); //@line 1338 "numbers.cpp"
          var $123=HEAP8[($arrayidx344)]; //@line 1338 "numbers.cpp"
          var $conv345=(tempInt=(($123)),(tempInt>=128?tempInt-256:tempInt)); //@line 1338 "numbers.cpp"
          var $cmp346=((($conv345))|0)==6; //@line 1338 "numbers.cpp"
          if ($cmp346) { __label__ = 91;; } else { __label__ = 94;; } //@line 1338 "numbers.cpp"
          if (__label__ == 91) {
  
            var $124=$found; //@line 1340 "numbers.cpp"
            var $tobool348=((($124))|0)!=0; //@line 1340 "numbers.cpp"
            if ($tobool348) { __label__ = 92;; } else { __label__ = 93;; } //@line 1340 "numbers.cpp"
            if (__label__ == 92) {
  
              var $125=$ix; //@line 1341 "numbers.cpp"
              var $126=$ph_out_addr; //@line 1341 "numbers.cpp"
              var $arrayidx350=(($126+$125)&4294967295); //@line 1341 "numbers.cpp"
              HEAP8[($arrayidx350)]=5; //@line 1341 "numbers.cpp"
              ; //@line 1341 "numbers.cpp"
            }
            else if (__label__ == 93) {
  
              $found=1; //@line 1343 "numbers.cpp"
              ;
            }
          }
  
          var $127=$ix; //@line 1336 "numbers.cpp"
          var $dec=((($127)-1)&4294967295); //@line 1336 "numbers.cpp"
          $ix=$dec; //@line 1336 "numbers.cpp"
          var $cmp343=((($dec))|0) >= 0; //@line 1336 "numbers.cpp"
          if ($cmp343) { __label__ = 90;continue ; } else { __label__ = 95;break $if_then340$$if_end354$121; } //@line 1336 "numbers.cpp"
        }
      }
    } while(0);
  
    var $128=$used_and; //@line 1347 "numbers.cpp"
    STACKTOP = __stackBase__;
    return $128; //@line 1347 "numbers.cpp"
    return null;
  }
  

  function __ZL9M_Varianti($value) {
    ;
    var __label__;
  
    var $retval;
    var $value_addr;
    $value_addr=$value;
    var $0=HEAP32[((_translator)>>2)]; //@line 963 "numbers.cpp"
    var $langopts=(($0)&4294967295); //@line 963 "numbers.cpp"
    var $numbers2=(($langopts+188)&4294967295); //@line 963 "numbers.cpp"
    var $1=HEAP32[(($numbers2)>>2)]; //@line 963 "numbers.cpp"
    var $and=($1) & 256; //@line 963 "numbers.cpp"
    var $tobool=((($and))|0)!=0; //@line 963 "numbers.cpp"
    if ($tobool) { __label__ = 1;; } else { __label__ = 3;; } //@line 963 "numbers.cpp"
    $land_lhs_true$$if_else$2: do { 
      if (__label__ == 1) {
  
        var $2=$value_addr; //@line 963 "numbers.cpp"
        var $cmp=((($2))|0) >= 2; //@line 963 "numbers.cpp"
        var $3=$value_addr; //@line 963 "numbers.cpp"
        var $cmp2=((($3))|0) <= 4; //@line 963 "numbers.cpp"
        var $or_cond=($cmp) & ($cmp2); //@line 963 "numbers.cpp"
        if (!($or_cond)) { __label__ = 3;break $land_lhs_true$$if_else$2; } //@line 963 "numbers.cpp"
  
        $retval=((__str54351)&4294967295); //@line 964 "numbers.cpp"
        __label__ = 13;break $land_lhs_true$$if_else$2; //@line 964 "numbers.cpp"
      }
    } while(0);
    $return$$if_else$5: do { 
      if (__label__ == 3) {
  
        var $4=$value_addr; //@line 966 "numbers.cpp"
        var $rem=((($4))|0)%100; //@line 966 "numbers.cpp"
        var $cmp3=((($rem))|0) < 10; //@line 966 "numbers.cpp"
        if ($cmp3) { __label__ = 5;; } else { __label__ = 4;; } //@line 966 "numbers.cpp"
        $if_then6$$lor_lhs_false$7: do { 
          if (__label__ == 4) {
  
            var $5=$value_addr; //@line 966 "numbers.cpp"
            var $rem4=((($5))|0)%100; //@line 966 "numbers.cpp"
            var $cmp5=((($rem4))|0) > 20; //@line 966 "numbers.cpp"
            if ($cmp5) { __label__ = 5;break $if_then6$$lor_lhs_false$7; } else { __label__ = 12;break $if_then6$$lor_lhs_false$7; } //@line 966 "numbers.cpp"
          }
        } while(0);
        $if_then6$$if_end28$9: do { 
          if (__label__ == 5) {
  
            var $6=HEAP32[((_translator)>>2)]; //@line 968 "numbers.cpp"
            var $langopts7=(($6)&4294967295); //@line 968 "numbers.cpp"
            var $numbers28=(($langopts7+188)&4294967295); //@line 968 "numbers.cpp"
            var $7=HEAP32[(($numbers28)>>2)]; //@line 968 "numbers.cpp"
            var $and9=($7) & 64; //@line 968 "numbers.cpp"
            var $tobool10=((($and9))|0)!=0; //@line 968 "numbers.cpp"
            if ($tobool10) { __label__ = 6;; } else { __label__ = 9;; } //@line 968 "numbers.cpp"
            $land_lhs_true11$$if_end$11: do { 
              if (__label__ == 6) {
  
                var $8=$value_addr; //@line 968 "numbers.cpp"
                var $rem12=((($8))|0)%10; //@line 968 "numbers.cpp"
                var $cmp13=((($rem12))|0) >= 2; //@line 968 "numbers.cpp"
                if (!($cmp13)) { __label__ = 9;break $land_lhs_true11$$if_end$11; } //@line 968 "numbers.cpp"
  
                var $9=$value_addr; //@line 968 "numbers.cpp"
                var $rem15=((($9))|0)%10; //@line 968 "numbers.cpp"
                var $cmp16=((($rem15))|0) <= 4; //@line 968 "numbers.cpp"
                if (!($cmp16)) { __label__ = 9;break $land_lhs_true11$$if_end$11; } //@line 968 "numbers.cpp"
  
                $retval=((__str54351)&4294967295); //@line 973 "numbers.cpp"
                __label__ = 13;break $return$$if_else$5; //@line 973 "numbers.cpp"
              }
            } while(0);
  
            var $10=HEAP32[((_translator)>>2)]; //@line 976 "numbers.cpp"
            var $langopts18=(($10)&4294967295); //@line 976 "numbers.cpp"
            var $numbers219=(($langopts18+188)&4294967295); //@line 976 "numbers.cpp"
            var $11=HEAP32[(($numbers219)>>2)]; //@line 976 "numbers.cpp"
            var $and20=($11) & 128; //@line 976 "numbers.cpp"
            var $tobool21=((($and20))|0)!=0; //@line 976 "numbers.cpp"
            if (!($tobool21)) { __label__ = 12;break $if_then6$$if_end28$9; } //@line 976 "numbers.cpp"
  
            var $12=$value_addr; //@line 976 "numbers.cpp"
            var $rem23=((($12))|0)%10; //@line 976 "numbers.cpp"
            var $cmp24=((($rem23))|0)==1; //@line 976 "numbers.cpp"
            if (!($cmp24)) { __label__ = 12;break $if_then6$$if_end28$9; } //@line 976 "numbers.cpp"
  
            $retval=((__str55352)&4294967295); //@line 979 "numbers.cpp"
            __label__ = 13;break $return$$if_else$5; //@line 979 "numbers.cpp"
          }
        } while(0);
  
        $retval=((__str56353)&4294967295); //@line 983 "numbers.cpp"
        ; //@line 983 "numbers.cpp"
      }
    } while(0);
  
    var $13=$retval; //@line 984 "numbers.cpp"
    ;
    return $13; //@line 984 "numbers.cpp"
    return null;
  }
  

  function __Z15LoadMbrolaTablePKcS0_i($mbrola_voice, $phtrans, $srate) {
    var __stackBase__  = STACKTOP; STACKTOP += 168; _memset(__stackBase__, 0, 168);
    var __label__;
  
    var $retval;
    var $mbrola_voice_addr;
    var $phtrans_addr;
    var $srate_addr;
    var $size;
    var $ix;
    var $pw;
    var $f_in;
    var $path=__stackBase__;
    $mbrola_voice_addr=$mbrola_voice;
    $phtrans_addr=$phtrans;
    $srate_addr=$srate;
    HEAP8[(((_mbrola_name)&4294967295))]=0; //@line 124 "synth_mbrola.cpp"
    HEAP32[((_mbrola_delay)>>2)]=0; //@line 125 "synth_mbrola.cpp"
    HEAP32[((__ZL15mbr_name_prefix)>>2)]=0; //@line 126 "synth_mbrola.cpp"
    var $0=$mbrola_voice_addr; //@line 128 "synth_mbrola.cpp"
    var $cmp=((($0))|0)==0; //@line 128 "synth_mbrola.cpp"
    if ($cmp) { __label__ = 1;; } else { __label__ = 2;; } //@line 128 "synth_mbrola.cpp"
    if (__label__ == 1) {
  
      var $1=HEAP32[((_samplerate_native)>>2)]; //@line 130 "synth_mbrola.cpp"
      HEAP32[((_samplerate)>>2)]=$1; //@line 130 "synth_mbrola.cpp"
      __Z12SetParameteriii(14, 0, 0); //@line 131 "synth_mbrola.cpp"
      $retval=0; //@line 132 "synth_mbrola.cpp"
      ; //@line 132 "synth_mbrola.cpp"
    }
    else if (__label__ == 2) {
  
      var $arraydecay=(($path)&4294967295); //@line 135 "synth_mbrola.cpp"
      var $2=$mbrola_voice_addr; //@line 135 "synth_mbrola.cpp"
      var $call=_sprintf($arraydecay, ((__str382)&4294967295), allocate([((_path_home)&4294967295),0,0,0,$2,0,0,0], ["i8*",0,0,0,"i8*",0,0,0], ALLOC_STACK)); //@line 135 "synth_mbrola.cpp"
      var $arraydecay1=(($path)&4294967295); //@line 139 "synth_mbrola.cpp"
      var $call2=__Z13GetFileLengthPKc($arraydecay1); //@line 139 "synth_mbrola.cpp"
      var $cmp3=((($call2))|0) <= 0; //@line 139 "synth_mbrola.cpp"
      if ($cmp3) { __label__ = 3;; } else { __label__ = 4;; } //@line 139 "synth_mbrola.cpp"
      if (__label__ == 3) {
  
        var $arraydecay5=(($path)&4294967295); //@line 141 "synth_mbrola.cpp"
        var $3=$mbrola_voice_addr; //@line 141 "synth_mbrola.cpp"
        var $call6=_sprintf($arraydecay5, ((__str1383)&4294967295), allocate([$3,0,0,0], ["i8*",0,0,0], ALLOC_STACK)); //@line 141 "synth_mbrola.cpp"
        ; //@line 142 "synth_mbrola.cpp"
      }
  
      var $arraydecay8=(($path)&4294967295); //@line 143 "synth_mbrola.cpp"
      var $call9=__Z13GetFileLengthPKc($arraydecay8); //@line 143 "synth_mbrola.cpp"
      var $cmp10=((($call9))|0) <= 0; //@line 143 "synth_mbrola.cpp"
      if ($cmp10) { __label__ = 5;; } else { __label__ = 6;; } //@line 143 "synth_mbrola.cpp"
      if (__label__ == 5) {
  
        var $arraydecay12=(($path)&4294967295); //@line 145 "synth_mbrola.cpp"
        var $4=$mbrola_voice_addr; //@line 145 "synth_mbrola.cpp"
        var $5=$mbrola_voice_addr; //@line 145 "synth_mbrola.cpp"
        var $call13=_sprintf($arraydecay12, ((__str2384)&4294967295), allocate([$4,0,0,0,$5,0,0,0], ["i8*",0,0,0,"i8*",0,0,0], ALLOC_STACK)); //@line 145 "synth_mbrola.cpp"
        ; //@line 146 "synth_mbrola.cpp"
      }
  
      var $arraydecay15=(($path)&4294967295); //@line 147 "synth_mbrola.cpp"
      var $call16=__Z13GetFileLengthPKc($arraydecay15); //@line 147 "synth_mbrola.cpp"
      var $cmp17=((($call16))|0) <= 0; //@line 147 "synth_mbrola.cpp"
      if ($cmp17) { __label__ = 7;; } else { __label__ = 8;; } //@line 147 "synth_mbrola.cpp"
      if (__label__ == 7) {
  
        var $arraydecay19=(($path)&4294967295); //@line 149 "synth_mbrola.cpp"
        var $6=$mbrola_voice_addr; //@line 149 "synth_mbrola.cpp"
        var $call20=_sprintf($arraydecay19, ((__str3385)&4294967295), allocate([$6,0,0,0], ["i8*",0,0,0], ALLOC_STACK)); //@line 149 "synth_mbrola.cpp"
        ; //@line 150 "synth_mbrola.cpp"
      }
  
      var $arraydecay22=(($path)&4294967295); //@line 160 "synth_mbrola.cpp"
      var $call23=_init_MBR($arraydecay22); //@line 160 "synth_mbrola.cpp"
      var $cmp24=((($call23))|0)!=0; //@line 160 "synth_mbrola.cpp"
      if ($cmp24) { __label__ = 9;; } else { __label__ = 10;; } //@line 160 "synth_mbrola.cpp"
      if (__label__ == 9) {
  
        $retval=2; //@line 161 "synth_mbrola.cpp"
        ; //@line 161 "synth_mbrola.cpp"
      }
      else if (__label__ == 10) {
  
        _setNoError_MBR(); //@line 163 "synth_mbrola.cpp"
        var $arraydecay27=(($path)&4294967295); //@line 166 "synth_mbrola.cpp"
        var $7=$phtrans_addr; //@line 166 "synth_mbrola.cpp"
        var $call28=_sprintf($arraydecay27, ((__str4386)&4294967295), allocate([((_path_home)&4294967295),0,0,0,$7,0,0,0], ["i8*",0,0,0,"i8*",0,0,0], ALLOC_STACK)); //@line 166 "synth_mbrola.cpp"
        var $arraydecay29=(($path)&4294967295); //@line 167 "synth_mbrola.cpp"
        var $call30=__Z13GetFileLengthPKc($arraydecay29); //@line 167 "synth_mbrola.cpp"
        $size=$call30; //@line 167 "synth_mbrola.cpp"
        var $arraydecay31=(($path)&4294967295); //@line 168 "synth_mbrola.cpp"
        var $call32=_fopen($arraydecay31, ((__str18429)&4294967295)); //@line 168 "synth_mbrola.cpp"
        $f_in=$call32; //@line 168 "synth_mbrola.cpp"
        var $cmp33=((($call32))|0)==0; //@line 168 "synth_mbrola.cpp"
        if ($cmp33) { __label__ = 11;; } else { __label__ = 12;; } //@line 168 "synth_mbrola.cpp"
        if (__label__ == 11) {
  
          _close_MBR(); //@line 169 "synth_mbrola.cpp"
          $retval=2; //@line 170 "synth_mbrola.cpp"
          ; //@line 170 "synth_mbrola.cpp"
        }
        else if (__label__ == 12) {
  
          var $8=HEAP32[((__ZL10mbrola_tab)>>2)]; //@line 173 "synth_mbrola.cpp"
          var $9=$8; //@line 173 "synth_mbrola.cpp"
          var $10=$size; //@line 173 "synth_mbrola.cpp"
          var $call36=_realloc($9, $10); //@line 173 "synth_mbrola.cpp"
          var $11=$call36; //@line 173 "synth_mbrola.cpp"
          HEAP32[((__ZL10mbrola_tab)>>2)]=$11; //@line 173 "synth_mbrola.cpp"
          var $cmp37=((($11))|0)==0; //@line 173 "synth_mbrola.cpp"
          var $12=$f_in; //@line 175 "synth_mbrola.cpp"
          if ($cmp37) { __label__ = 13;; } else { __label__ = 14;; } //@line 173 "synth_mbrola.cpp"
          if (__label__ == 13) {
  
            var $call39=_fclose($12); //@line 175 "synth_mbrola.cpp"
            _close_MBR(); //@line 176 "synth_mbrola.cpp"
            $retval=-1; //@line 177 "synth_mbrola.cpp"
            ; //@line 177 "synth_mbrola.cpp"
          }
          else if (__label__ == 14) {
  
            var $call41=__Z10Read4BytesP7__sFILE($12); //@line 180 "synth_mbrola.cpp"
            HEAP32[((__ZL14mbrola_control)>>2)]=$call41; //@line 180 "synth_mbrola.cpp"
            var $13=HEAP32[((__ZL10mbrola_tab)>>2)]; //@line 181 "synth_mbrola.cpp"
            var $14=$13; //@line 181 "synth_mbrola.cpp"
            $pw=$14; //@line 181 "synth_mbrola.cpp"
            $ix=4; //@line 182 "synth_mbrola.cpp"
            var $15=$ix; //@line 182 "synth_mbrola.cpp"
            var $16=$size; //@line 182 "synth_mbrola.cpp"
            var $cmp421=((($15))|0) < ((($16))|0); //@line 182 "synth_mbrola.cpp"
            if ($cmp421) { __label__ = 15;; } else { __label__ = 16;; } //@line 182 "synth_mbrola.cpp"
            $for_body$$for_end$23: do { 
              if (__label__ == 15) {
                while(1) { 
  
                  var $17=$f_in; //@line 184 "synth_mbrola.cpp"
                  var $call43=__Z10Read4BytesP7__sFILE($17); //@line 184 "synth_mbrola.cpp"
                  var $18=$pw; //@line 184 "synth_mbrola.cpp"
                  var $incdec_ptr=(($18+4)&4294967295); //@line 184 "synth_mbrola.cpp"
                  $pw=$incdec_ptr; //@line 184 "synth_mbrola.cpp"
                  HEAP32[(($18)>>2)]=$call43; //@line 184 "synth_mbrola.cpp"
                  var $19=$ix; //@line 182 "synth_mbrola.cpp"
                  var $add=((($19)+4)&4294967295); //@line 182 "synth_mbrola.cpp"
                  $ix=$add; //@line 182 "synth_mbrola.cpp"
                  var $20=$ix; //@line 182 "synth_mbrola.cpp"
                  var $21=$size; //@line 182 "synth_mbrola.cpp"
                  var $cmp42=((($20))|0) < ((($21))|0); //@line 182 "synth_mbrola.cpp"
                  if ($cmp42) { __label__ = 15;continue ; } else { __label__ = 16;break $for_body$$for_end$23; } //@line 182 "synth_mbrola.cpp"
                }
              }
            } while(0);
  
            var $22=HEAP32[((__ZL10mbrola_tab)>>2)]; //@line 186 "synth_mbrola.cpp"
            var $23=$22; //@line 186 "synth_mbrola.cpp"
            var $24=$size; //@line 186 "synth_mbrola.cpp"
            var $25=$f_in; //@line 186 "synth_mbrola.cpp"
            var $call44=_fread($23, 1, $24, $25); //@line 186 "synth_mbrola.cpp"
            $size=$call44; //@line 186 "synth_mbrola.cpp"
            var $26=$f_in; //@line 187 "synth_mbrola.cpp"
            var $call45=_fclose($26); //@line 187 "synth_mbrola.cpp"
            var $27=HEAP32[((__ZL14mbrola_control)>>2)]; //@line 189 "synth_mbrola.cpp"
            var $and=($27) & 255; //@line 189 "synth_mbrola.cpp"
            var $conv=((($and))|0); //@line 189 "synth_mbrola.cpp"
            var $div=($conv)/16; //@line 189 "synth_mbrola.cpp"
            _setVolumeRatio_MBR($div); //@line 189 "synth_mbrola.cpp"
            var $28=$srate_addr; //@line 191 "synth_mbrola.cpp"
            HEAP32[((_samplerate)>>2)]=$28; //@line 191 "synth_mbrola.cpp"
            var $cmp46=((($28))|0)==22050; //@line 192 "synth_mbrola.cpp"
            if ($cmp46) { __label__ = 17;; } else { __label__ = 18;; } //@line 192 "synth_mbrola.cpp"
            if (__label__ == 17) {
  
              __Z12SetParameteriii(14, 0, 0); //@line 193 "synth_mbrola.cpp"
              ; //@line 193 "synth_mbrola.cpp"
            }
            else if (__label__ == 18) {
  
              __Z12SetParameteriii(14, 1, 0); //@line 195 "synth_mbrola.cpp"
              ;
            }
  
            var $29=$mbrola_voice_addr; //@line 196 "synth_mbrola.cpp"
            var $call49=_strcpy(((_mbrola_name)&4294967295), $29); //@line 196 "synth_mbrola.cpp"
            HEAP32[((_mbrola_delay)>>2)]=1000; //@line 198 "synth_mbrola.cpp"
            $retval=0; //@line 199 "synth_mbrola.cpp"
            ; //@line 199 "synth_mbrola.cpp"
          }
        }
      }
    }
  
    var $30=$retval; //@line 200 "synth_mbrola.cpp"
    STACKTOP = __stackBase__;
    return $30; //@line 200 "synth_mbrola.cpp"
    return null;
  }
  

  function _setNoError_MBR() {
    ;
    var __label__;
  
    ;
    return; //@line 102 "./mbrowrap.h"
    return;
  }
  

  function __Z15MbrolaTranslateP12PHONEME_LISTiiP7__sFILE($plist, $n_phonemes, $resume, $f_mbrola) {
    var __stackBase__  = STACKTOP; STACKTOP += 332; _memset(__stackBase__, 0, 332);
    var __label__;
    var __lastLabel__ = null;
  
    var $retval;
    var $plist_addr;
    var $n_phonemes_addr;
    var $resume_addr;
    var $f_mbrola_addr;
    var $name;
    var $len;
    var $len1;
    var $ph;
    var $ph_next;
    var $ph_prev;
    var $p;
    var $next;
    var $prev;
    var $phdata=__stackBase__;
    var $fmtp=__stackBase__+152;
    var $pause;
    var $released;
    var $name2=__stackBase__+200;
    var $control=__stackBase__+204;
    var $done;
    var $len_percent=__stackBase__+208;
    var $final_pitch;
    var $ptr;
    var $mbr_buf=__stackBase__+212;
    var $pitch;
    var $pitch101;
    var $res;
    $plist_addr=$plist;
    $n_phonemes_addr=$n_phonemes;
    $resume_addr=$resume;
    $f_mbrola_addr=$f_mbrola;
    $pause=0; //@line 416 "synth_mbrola.cpp"
    var $0=$resume_addr; //@line 430 "synth_mbrola.cpp"
    var $tobool=((($0))|0)!=0; //@line 430 "synth_mbrola.cpp"
    if ($tobool) { __label__ = 2;; } else { __label__ = 1;; } //@line 430 "synth_mbrola.cpp"
    if (__label__ == 1) {
  
      HEAP32[((__ZZ15MbrolaTranslateP12PHONEME_LISTiiP7__sFILEE4phix)>>2)]=1; //@line 431 "synth_mbrola.cpp"
      HEAP32[((__ZZ15MbrolaTranslateP12PHONEME_LISTiiP7__sFILEE11embedded_ix)>>2)]=0; //@line 432 "synth_mbrola.cpp"
      HEAP32[((__ZZ15MbrolaTranslateP12PHONEME_LISTiiP7__sFILEE10word_count)>>2)]=0; //@line 433 "synth_mbrola.cpp"
      ; //@line 434 "synth_mbrola.cpp"
    }
  
    var $arraydecay=(($mbr_buf)&4294967295); //@line 441 "synth_mbrola.cpp"
    var $arraydecay233=(($mbr_buf)&4294967295); //@line 597 "synth_mbrola.cpp"
    var $arraydecay234=(($mbr_buf)&4294967295); //@line 597 "synth_mbrola.cpp"
    var $sub_ptr_rhs_cast=($arraydecay234); //@line 597 "synth_mbrola.cpp"
    var $1=$fmtp; //@line 556 "synth_mbrola.cpp"
    var $sound_addr=(($phdata+68)&4294967295); //@line 558 "synth_mbrola.cpp"
    var $arrayidx176=(($sound_addr)&4294967295); //@line 558 "synth_mbrola.cpp"
    var $fmt_addr=(($fmtp+8)&4294967295); //@line 558 "synth_mbrola.cpp"
    ; //@line 436 "synth_mbrola.cpp"
    $while_cond$5: while(1) { 
  
      var $2=HEAP32[((__ZZ15MbrolaTranslateP12PHONEME_LISTiiP7__sFILEE4phix)>>2)]; //@line 436 "synth_mbrola.cpp"
      var $3=$n_phonemes_addr; //@line 436 "synth_mbrola.cpp"
      var $cmp=((($2))|0) < ((($3))|0); //@line 436 "synth_mbrola.cpp"
      if (!($cmp)) { __label__ = 64;break $while_cond$5; } //@line 436 "synth_mbrola.cpp"
  
      var $call=__Z9WcmdqFreev(); //@line 438 "synth_mbrola.cpp"
      var $cmp1=((($call))|0) < 22; //@line 438 "synth_mbrola.cpp"
      if ($cmp1) { __label__ = 5;break $while_cond$5; } //@line 438 "synth_mbrola.cpp"
  
      $ptr=$arraydecay; //@line 441 "synth_mbrola.cpp"
      var $4=HEAP32[((__ZZ15MbrolaTranslateP12PHONEME_LISTiiP7__sFILEE4phix)>>2)]; //@line 443 "synth_mbrola.cpp"
      var $5=$plist_addr; //@line 443 "synth_mbrola.cpp"
      var $arrayidx=(($5+24*$4)&4294967295); //@line 443 "synth_mbrola.cpp"
      $p=$arrayidx; //@line 443 "synth_mbrola.cpp"
      var $6=HEAP32[((__ZZ15MbrolaTranslateP12PHONEME_LISTiiP7__sFILEE4phix)>>2)]; //@line 444 "synth_mbrola.cpp"
      var $add=((($6)+1)&4294967295); //@line 444 "synth_mbrola.cpp"
      var $7=$plist_addr; //@line 444 "synth_mbrola.cpp"
      var $arrayidx4=(($7+24*$add)&4294967295); //@line 444 "synth_mbrola.cpp"
      $next=$arrayidx4; //@line 444 "synth_mbrola.cpp"
      var $8=HEAP32[((__ZZ15MbrolaTranslateP12PHONEME_LISTiiP7__sFILEE4phix)>>2)]; //@line 445 "synth_mbrola.cpp"
      var $sub=((($8)-1)&4294967295); //@line 445 "synth_mbrola.cpp"
      var $9=$plist_addr; //@line 445 "synth_mbrola.cpp"
      var $arrayidx5=(($9+24*$sub)&4294967295); //@line 445 "synth_mbrola.cpp"
      $prev=$arrayidx5; //@line 445 "synth_mbrola.cpp"
      var $10=$p; //@line 446 "synth_mbrola.cpp"
      var $ph6=(($10+8)&4294967295); //@line 446 "synth_mbrola.cpp"
      var $11=HEAP32[(($ph6)>>2)]; //@line 446 "synth_mbrola.cpp"
      $ph=$11; //@line 446 "synth_mbrola.cpp"
      var $12=HEAP32[((__ZZ15MbrolaTranslateP12PHONEME_LISTiiP7__sFILEE4phix)>>2)]; //@line 447 "synth_mbrola.cpp"
      var $sub7=((($12)-1)&4294967295); //@line 447 "synth_mbrola.cpp"
      var $13=$plist_addr; //@line 447 "synth_mbrola.cpp"
      var $arrayidx8=(($13+24*$sub7)&4294967295); //@line 447 "synth_mbrola.cpp"
      var $ph9=(($arrayidx8+8)&4294967295); //@line 447 "synth_mbrola.cpp"
      var $14=HEAP32[(($ph9)>>2)]; //@line 447 "synth_mbrola.cpp"
      $ph_prev=$14; //@line 447 "synth_mbrola.cpp"
      var $15=HEAP32[((__ZZ15MbrolaTranslateP12PHONEME_LISTiiP7__sFILEE4phix)>>2)]; //@line 448 "synth_mbrola.cpp"
      var $add10=((($15)+1)&4294967295); //@line 448 "synth_mbrola.cpp"
      var $16=$plist_addr; //@line 448 "synth_mbrola.cpp"
      var $arrayidx11=(($16+24*$add10)&4294967295); //@line 448 "synth_mbrola.cpp"
      var $ph12=(($arrayidx11+8)&4294967295); //@line 448 "synth_mbrola.cpp"
      var $17=HEAP32[(($ph12)>>2)]; //@line 448 "synth_mbrola.cpp"
      $ph_next=$17; //@line 448 "synth_mbrola.cpp"
      var $18=$p; //@line 450 "synth_mbrola.cpp"
      var $synthflags=(($18+4)&4294967295); //@line 450 "synth_mbrola.cpp"
      var $19=HEAPU16[(($synthflags)>>1)]; //@line 450 "synth_mbrola.cpp"
      var $conv=((($19))&65535); //@line 450 "synth_mbrola.cpp"
      var $and=($conv) & 2; //@line 450 "synth_mbrola.cpp"
      var $tobool13=((($and))|0)!=0; //@line 450 "synth_mbrola.cpp"
      if ($tobool13) { __label__ = 7;; } else { __label__ = 8;; } //@line 450 "synth_mbrola.cpp"
      if (__label__ == 7) {
  
        var $20=$p; //@line 452 "synth_mbrola.cpp"
        var $sourceix=(($20+6)&4294967295); //@line 452 "synth_mbrola.cpp"
        var $21=HEAPU16[(($sourceix)>>1)]; //@line 452 "synth_mbrola.cpp"
        var $conv15=((($21))&65535); //@line 452 "synth_mbrola.cpp"
        __Z10DoEmbeddedPii(__ZZ15MbrolaTranslateP12PHONEME_LISTiiP7__sFILEE11embedded_ix, $conv15); //@line 452 "synth_mbrola.cpp"
        ; //@line 453 "synth_mbrola.cpp"
      }
  
      var $22=$p; //@line 455 "synth_mbrola.cpp"
      var $newword=(($22+19)&4294967295); //@line 455 "synth_mbrola.cpp"
      var $23=HEAPU8[($newword)]; //@line 455 "synth_mbrola.cpp"
      var $conv17=((($23))&255); //@line 455 "synth_mbrola.cpp"
      var $and18=($conv17) & 4; //@line 455 "synth_mbrola.cpp"
      var $tobool19=((($and18))|0)!=0; //@line 455 "synth_mbrola.cpp"
      if ($tobool19) { __label__ = 9;; } else { __label__ = 10;; } //@line 455 "synth_mbrola.cpp"
      if (__label__ == 9) {
  
        var $24=$p; //@line 456 "synth_mbrola.cpp"
        var $sourceix21=(($24+6)&4294967295); //@line 456 "synth_mbrola.cpp"
        var $25=HEAPU16[(($sourceix21)>>1)]; //@line 456 "synth_mbrola.cpp"
        var $conv22=((($25))&65535); //@line 456 "synth_mbrola.cpp"
        var $and23=($conv22) & 2047; //@line 456 "synth_mbrola.cpp"
        var $26=HEAP32[((_clause_start_char)>>2)]; //@line 456 "synth_mbrola.cpp"
        var $add24=((($and23)+($26))&4294967295); //@line 456 "synth_mbrola.cpp"
        var $27=HEAP32[((_count_sentences)>>2)]; //@line 456 "synth_mbrola.cpp"
        __Z8DoMarkeriiii(2, $add24, 0, $27); //@line 456 "synth_mbrola.cpp"
        ; //@line 456 "synth_mbrola.cpp"
      }
  
      var $28=$p; //@line 457 "synth_mbrola.cpp"
      var $newword26=(($28+19)&4294967295); //@line 457 "synth_mbrola.cpp"
      var $29=HEAPU8[($newword26)]; //@line 457 "synth_mbrola.cpp"
      var $conv27=((($29))&255); //@line 457 "synth_mbrola.cpp"
      var $and28=($conv27) & 1; //@line 457 "synth_mbrola.cpp"
      var $tobool29=((($and28))|0)!=0; //@line 457 "synth_mbrola.cpp"
      if ($tobool29) { __label__ = 11;; } else { __label__ = 12;; } //@line 457 "synth_mbrola.cpp"
      if (__label__ == 11) {
  
        var $30=$p; //@line 458 "synth_mbrola.cpp"
        var $sourceix31=(($30+6)&4294967295); //@line 458 "synth_mbrola.cpp"
        var $31=HEAPU16[(($sourceix31)>>1)]; //@line 458 "synth_mbrola.cpp"
        var $conv32=((($31))&65535); //@line 458 "synth_mbrola.cpp"
        var $and33=($conv32) & 2047; //@line 458 "synth_mbrola.cpp"
        var $32=HEAP32[((_clause_start_char)>>2)]; //@line 458 "synth_mbrola.cpp"
        var $add34=((($and33)+($32))&4294967295); //@line 458 "synth_mbrola.cpp"
        var $33=$p; //@line 458 "synth_mbrola.cpp"
        var $sourceix35=(($33+6)&4294967295); //@line 458 "synth_mbrola.cpp"
        var $34=HEAPU16[(($sourceix35)>>1)]; //@line 458 "synth_mbrola.cpp"
        var $conv36=((($34))&65535); //@line 458 "synth_mbrola.cpp"
        var $shr=($conv36) >> 11; //@line 458 "synth_mbrola.cpp"
        var $35=HEAP32[((_clause_start_word)>>2)]; //@line 458 "synth_mbrola.cpp"
        var $36=HEAPU32[((__ZZ15MbrolaTranslateP12PHONEME_LISTiiP7__sFILEE10word_count)>>2)]; //@line 458 "synth_mbrola.cpp"
        var $inc=((($36)+1)&4294967295); //@line 458 "synth_mbrola.cpp"
        HEAP32[((__ZZ15MbrolaTranslateP12PHONEME_LISTiiP7__sFILEE10word_count)>>2)]=$inc; //@line 458 "synth_mbrola.cpp"
        var $add37=((($36)+($35))&4294967295); //@line 458 "synth_mbrola.cpp"
        __Z8DoMarkeriiii(1, $add34, $shr, $add37); //@line 458 "synth_mbrola.cpp"
        ; //@line 458 "synth_mbrola.cpp"
      }
  
      var $37=$p; //@line 460 "synth_mbrola.cpp"
      var $38=$ph; //@line 460 "synth_mbrola.cpp"
      var $39=$ph_prev; //@line 460 "synth_mbrola.cpp"
      var $40=$ph_next; //@line 460 "synth_mbrola.cpp"
      var $call39=__ZL10GetMbrNameP12PHONEME_LISTP11PHONEME_TABS2_S2_PiS3_S3_($37, $38, $39, $40, $name2, $len_percent, $control); //@line 460 "synth_mbrola.cpp"
      $name=$call39; //@line 460 "synth_mbrola.cpp"
      var $41=HEAP32[(($control)>>2)]; //@line 461 "synth_mbrola.cpp"
      var $and40=($41) & 1; //@line 461 "synth_mbrola.cpp"
      var $tobool41=((($and40))|0)!=0; //@line 461 "synth_mbrola.cpp"
      if ($tobool41) { __lastLabel__ = 12; __label__ = 13;; } else { __lastLabel__ = 12; __label__ = 14;; } //@line 461 "synth_mbrola.cpp"
      if (__label__ == 13) {
  
        var $42=HEAP32[((__ZZ15MbrolaTranslateP12PHONEME_LISTiiP7__sFILEE4phix)>>2)]; //@line 462 "synth_mbrola.cpp"
        var $inc43=((($42)+1)&4294967295); //@line 462 "synth_mbrola.cpp"
        HEAP32[((__ZZ15MbrolaTranslateP12PHONEME_LISTiiP7__sFILEE4phix)>>2)]=$inc43; //@line 462 "synth_mbrola.cpp"
        var $_pr=$name; //@line 464 "synth_mbrola.cpp"
        __lastLabel__ = 13; ; //@line 462 "synth_mbrola.cpp"
      }
  
      var $43=__lastLabel__ == 13 ? $_pr : ($call39); //@line 464 "synth_mbrola.cpp"
      var $cmp45=((($43))|0)==0; //@line 464 "synth_mbrola.cpp"
      if ($cmp45) { __label__ = 15;; } else { __label__ = 16;; } //@line 464 "synth_mbrola.cpp"
      if (__label__ == 15) {
  
        var $44=HEAP32[((__ZZ15MbrolaTranslateP12PHONEME_LISTiiP7__sFILEE4phix)>>2)]; //@line 465 "synth_mbrola.cpp"
        var $inc47=((($44)+1)&4294967295); //@line 465 "synth_mbrola.cpp"
        HEAP32[((__ZZ15MbrolaTranslateP12PHONEME_LISTiiP7__sFILEE4phix)>>2)]=$inc47; //@line 465 "synth_mbrola.cpp"
        __label__ = 3;continue $while_cond$5; //@line 466 "synth_mbrola.cpp"
      }
      else if (__label__ == 16) {
  
        var $45=$ph; //@line 469 "synth_mbrola.cpp"
        var $type=(($45+11)&4294967295); //@line 469 "synth_mbrola.cpp"
        var $46=HEAPU8[($type)]; //@line 469 "synth_mbrola.cpp"
        var $conv49=((($46))&255); //@line 469 "synth_mbrola.cpp"
        var $cmp50=((($conv49))|0)==0; //@line 469 "synth_mbrola.cpp"
        if ($cmp50) { __label__ = 17;; } else { __label__ = 20;; } //@line 469 "synth_mbrola.cpp"
        $land_lhs_true$$if_else$24: do { 
          if (__label__ == 17) {
  
            var $47=$name; //@line 469 "synth_mbrola.cpp"
            var $48=$ph; //@line 469 "synth_mbrola.cpp"
            var $mnemonic=(($48)&4294967295); //@line 469 "synth_mbrola.cpp"
            var $49=HEAP32[(($mnemonic)>>2)]; //@line 469 "synth_mbrola.cpp"
            var $cmp51=((($47))|0)==((($49))|0); //@line 469 "synth_mbrola.cpp"
            if (!($cmp51)) { __label__ = 20;break $land_lhs_true$$if_else$24; } //@line 469 "synth_mbrola.cpp"
  
            $name=95; //@line 472 "synth_mbrola.cpp"
            var $50=$p; //@line 473 "synth_mbrola.cpp"
            var $length=(($50+12)&4294967295); //@line 473 "synth_mbrola.cpp"
            var $51=HEAP16[(($length)>>1)]; //@line 473 "synth_mbrola.cpp"
            var $conv53=(tempInt=(($51)),(tempInt>=32768?tempInt-65536:tempInt)); //@line 473 "synth_mbrola.cpp"
            var $52=HEAP32[((((_speed)&4294967295))>>2)]; //@line 473 "synth_mbrola.cpp"
            var $mul=((($conv53)*($52))&4294967295); //@line 473 "synth_mbrola.cpp"
            var $div=((((($mul))|0)/256)|0); //@line 473 "synth_mbrola.cpp"
            $len=$div; //@line 473 "synth_mbrola.cpp"
            var $53=$len; //@line 475 "synth_mbrola.cpp"
            var $cmp54=((($53))|0)==0; //@line 475 "synth_mbrola.cpp"
            if (!($cmp54)) { __label__ = 21;break $land_lhs_true$$if_else$24; } //@line 475 "synth_mbrola.cpp"
  
            $len=1; //@line 476 "synth_mbrola.cpp"
            __label__ = 21;break $land_lhs_true$$if_else$24; //@line 476 "synth_mbrola.cpp"
          }
        } while(0);
        if (__label__ == 20) {
  
          var $54=HEAP32[((((_speed+12)&4294967295))>>2)]; //@line 479 "synth_mbrola.cpp"
          var $mul57=((($54)*80)&4294967295); //@line 479 "synth_mbrola.cpp"
          var $div58=((((($mul57))|0)/256)|0); //@line 479 "synth_mbrola.cpp"
          $len=$div58; //@line 479 "synth_mbrola.cpp"
          ;
        }
  
        var $55=$p; //@line 481 "synth_mbrola.cpp"
        var $sourceix60=(($55+6)&4294967295); //@line 481 "synth_mbrola.cpp"
        var $56=HEAPU16[(($sourceix60)>>1)]; //@line 481 "synth_mbrola.cpp"
        var $conv61=((($56))&65535); //@line 481 "synth_mbrola.cpp"
        var $and62=($conv61) & 2047; //@line 481 "synth_mbrola.cpp"
        var $57=HEAP32[((_clause_start_char)>>2)]; //@line 481 "synth_mbrola.cpp"
        var $add63=((($and62)+($57))&4294967295); //@line 481 "synth_mbrola.cpp"
        var $58=$ph; //@line 481 "synth_mbrola.cpp"
        var $mnemonic64=(($58)&4294967295); //@line 481 "synth_mbrola.cpp"
        var $59=HEAP32[(($mnemonic64)>>2)]; //@line 481 "synth_mbrola.cpp"
        __Z8DoMarkeriiii(7, $add63, 0, $59); //@line 481 "synth_mbrola.cpp"
        var $60=$ptr; //@line 483 "synth_mbrola.cpp"
        var $61=$name; //@line 483 "synth_mbrola.cpp"
        var $call65=__Z12WordToStringj($61); //@line 483 "synth_mbrola.cpp"
        var $call66=_sprintf($60, ((__str6388)&4294967295), allocate([((__ZZ12WordToStringjE3buf)&4294967295),0,0,0], ["i8*",0,0,0], ALLOC_STACK)); //@line 483 "synth_mbrola.cpp"
        var $62=$ptr; //@line 483 "synth_mbrola.cpp"
        var $add_ptr=(($62+$call66)&4294967295); //@line 483 "synth_mbrola.cpp"
        $ptr=$add_ptr; //@line 483 "synth_mbrola.cpp"
        var $63=HEAP32[(($name2)>>2)]; //@line 485 "synth_mbrola.cpp"
        var $cmp67=((($63))|0)==95; //@line 485 "synth_mbrola.cpp"
        if ($cmp67) { __label__ = 22;; } else { __label__ = 23;; } //@line 485 "synth_mbrola.cpp"
        if (__label__ == 22) {
  
          var $64=HEAP32[(($len_percent)>>2)]; //@line 488 "synth_mbrola.cpp"
          $pause=$64; //@line 488 "synth_mbrola.cpp"
          HEAP32[(($name2)>>2)]=0; //@line 489 "synth_mbrola.cpp"
          ; //@line 490 "synth_mbrola.cpp"
        }
  
        $done=0; //@line 492 "synth_mbrola.cpp"
        $final_pitch=((__str13643)&4294967295); //@line 493 "synth_mbrola.cpp"
        var $65=$ph; //@line 495 "synth_mbrola.cpp"
        var $type70=(($65+11)&4294967295); //@line 495 "synth_mbrola.cpp"
        var $66=HEAPU8[($type70)]; //@line 495 "synth_mbrola.cpp"
        var $conv71=((($66))&255); //@line 495 "synth_mbrola.cpp"
        if ($conv71 == 2) {
          __label__ = 24;;
        }
        else if ($conv71 == 4) {
          __label__ = 32;;
        }
        else if ($conv71 == 5) {
          __label__ = 40;;
        }
        else if ($conv71 == 6) {
          __label__ = 41;;
        }
        else if ($conv71 == 8) {
          __label__ = 44;;
        }
        else if ($conv71 == 3) {
          __label__ = 48;;
        }
        else {
        __label__ = 50;;
        }
        
        $sw_epilog$$sw_bb$$sw_bb126$$sw_bb152$$sw_bb155$$sw_bb171$$sw_bb195$34: do { 
          if (__label__ == 24) {
  
            var $67=$ph; //@line 498 "synth_mbrola.cpp"
            var $std_length=(($67+14)&4294967295); //@line 498 "synth_mbrola.cpp"
            var $68=HEAPU8[($std_length)]; //@line 498 "synth_mbrola.cpp"
            var $conv72=((($68))&255); //@line 498 "synth_mbrola.cpp"
            $len=$conv72; //@line 498 "synth_mbrola.cpp"
            var $69=$p; //@line 499 "synth_mbrola.cpp"
            var $synthflags73=(($69+4)&4294967295); //@line 499 "synth_mbrola.cpp"
            var $70=HEAPU16[(($synthflags73)>>1)]; //@line 499 "synth_mbrola.cpp"
            var $conv74=((($70))&65535); //@line 499 "synth_mbrola.cpp"
            var $and75=($conv74) & 8; //@line 499 "synth_mbrola.cpp"
            var $tobool76=((($and75))|0)!=0; //@line 499 "synth_mbrola.cpp"
            if ($tobool76) { __label__ = 25;; } else { __label__ = 26;; } //@line 499 "synth_mbrola.cpp"
            if (__label__ == 25) {
  
              var $71=HEAP32[((((_phoneme_tab+48)&4294967295))>>2)]; //@line 500 "synth_mbrola.cpp"
              var $std_length78=(($71+14)&4294967295); //@line 500 "synth_mbrola.cpp"
              var $72=HEAPU8[($std_length78)]; //@line 500 "synth_mbrola.cpp"
              var $conv79=((($72))&255); //@line 500 "synth_mbrola.cpp"
              var $73=$len; //@line 500 "synth_mbrola.cpp"
              var $add80=((($73)+($conv79))&4294967295); //@line 500 "synth_mbrola.cpp"
              $len=$add80; //@line 500 "synth_mbrola.cpp"
              ; //@line 500 "synth_mbrola.cpp"
            }
  
            var $74=$ph_next; //@line 502 "synth_mbrola.cpp"
            var $type82=(($74+11)&4294967295); //@line 502 "synth_mbrola.cpp"
            var $75=HEAPU8[($type82)]; //@line 502 "synth_mbrola.cpp"
            var $conv83=((($75))&255); //@line 502 "synth_mbrola.cpp"
            var $cmp84=((($conv83))|0)==0; //@line 502 "synth_mbrola.cpp"
            if ($cmp84) { __label__ = 27;; } else { __label__ = 28;; } //@line 502 "synth_mbrola.cpp"
            if (__label__ == 27) {
  
              var $76=$len; //@line 503 "synth_mbrola.cpp"
              var $add86=((($76)+50)&4294967295); //@line 503 "synth_mbrola.cpp"
              $len=$add86; //@line 503 "synth_mbrola.cpp"
              ; //@line 503 "synth_mbrola.cpp"
            }
  
            var $77=$len; //@line 504 "synth_mbrola.cpp"
            var $78=$p; //@line 504 "synth_mbrola.cpp"
            var $length88=(($78+12)&4294967295); //@line 504 "synth_mbrola.cpp"
            var $79=HEAP16[(($length88)>>1)]; //@line 504 "synth_mbrola.cpp"
            var $conv89=(tempInt=(($79)),(tempInt>=32768?tempInt-65536:tempInt)); //@line 504 "synth_mbrola.cpp"
            var $mul90=((($conv89)*($77))&4294967295); //@line 504 "synth_mbrola.cpp"
            var $div91=((((($mul90))|0)/256)|0); //@line 504 "synth_mbrola.cpp"
            $len=$div91; //@line 504 "synth_mbrola.cpp"
            var $80=HEAP32[(($name2)>>2)]; //@line 506 "synth_mbrola.cpp"
            var $cmp92=((($80))|0)==0; //@line 506 "synth_mbrola.cpp"
            var $81=$p; //@line 508 "synth_mbrola.cpp"
            var $env=(($81+14)&4294967295); //@line 508 "synth_mbrola.cpp"
            var $82=HEAPU8[($env)]; //@line 508 "synth_mbrola.cpp"
            var $conv94=((($82))&255); //@line 508 "synth_mbrola.cpp"
            var $83=$p; //@line 508 "synth_mbrola.cpp"
            var $pitch1=(($83+20)&4294967295); //@line 508 "synth_mbrola.cpp"
            var $84=HEAPU8[($pitch1)]; //@line 508 "synth_mbrola.cpp"
            var $conv95=((($84))&255); //@line 508 "synth_mbrola.cpp"
            var $85=$p; //@line 508 "synth_mbrola.cpp"
            var $pitch2=(($85+21)&4294967295); //@line 508 "synth_mbrola.cpp"
            var $86=HEAPU8[($pitch2)]; //@line 508 "synth_mbrola.cpp"
            var $conv96=((($86))&255); //@line 508 "synth_mbrola.cpp"
            if ($cmp92) { __label__ = 29;; } else { __label__ = 30;; } //@line 506 "synth_mbrola.cpp"
            if (__label__ == 29) {
  
              __ZL10WritePitchiiiii($conv94, $conv95, $conv96, 0, 0); //@line 508 "synth_mbrola.cpp"
              $pitch=((__ZZL10WritePitchiiiiiE6output)&4294967295); //@line 508 "synth_mbrola.cpp"
              var $87=$ptr; //@line 509 "synth_mbrola.cpp"
              var $88=$len; //@line 509 "synth_mbrola.cpp"
              var $89=$pitch; //@line 509 "synth_mbrola.cpp"
              var $call98=_sprintf($87, ((__str8390)&4294967295), allocate([$88,0,0,0,$89,0,0,0], ["i32",0,0,0,"i8*",0,0,0], ALLOC_STACK)); //@line 509 "synth_mbrola.cpp"
              var $90=$ptr; //@line 509 "synth_mbrola.cpp"
              var $add_ptr99=(($90+$call98)&4294967295); //@line 509 "synth_mbrola.cpp"
              $ptr=$add_ptr99; //@line 509 "synth_mbrola.cpp"
              ; //@line 510 "synth_mbrola.cpp"
            }
            else if (__label__ == 30) {
  
              var $91=HEAP32[(($len_percent)>>2)]; //@line 515 "synth_mbrola.cpp"
              __ZL10WritePitchiiiii($conv94, $conv95, $conv96, $91, 0); //@line 515 "synth_mbrola.cpp"
              $pitch101=((__ZZL10WritePitchiiiiiE6output)&4294967295); //@line 515 "synth_mbrola.cpp"
              var $92=$len; //@line 516 "synth_mbrola.cpp"
              var $93=HEAP32[(($len_percent)>>2)]; //@line 516 "synth_mbrola.cpp"
              var $mul109=((($93)*($92))&4294967295); //@line 516 "synth_mbrola.cpp"
              var $div110=((((($mul109))|0)/100)|0); //@line 516 "synth_mbrola.cpp"
              $len1=$div110; //@line 516 "synth_mbrola.cpp"
              var $94=$ptr; //@line 517 "synth_mbrola.cpp"
              var $95=$len1; //@line 517 "synth_mbrola.cpp"
              var $96=$pitch101; //@line 517 "synth_mbrola.cpp"
              var $call111=_sprintf($94, ((__str8390)&4294967295), allocate([$95,0,0,0,$96,0,0,0], ["i32",0,0,0,"i8*",0,0,0], ALLOC_STACK)); //@line 517 "synth_mbrola.cpp"
              var $97=$ptr; //@line 517 "synth_mbrola.cpp"
              var $add_ptr112=(($97+$call111)&4294967295); //@line 517 "synth_mbrola.cpp"
              $ptr=$add_ptr112; //@line 517 "synth_mbrola.cpp"
              var $98=$p; //@line 519 "synth_mbrola.cpp"
              var $env113=(($98+14)&4294967295); //@line 519 "synth_mbrola.cpp"
              var $99=HEAPU8[($env113)]; //@line 519 "synth_mbrola.cpp"
              var $conv114=((($99))&255); //@line 519 "synth_mbrola.cpp"
              var $100=$p; //@line 519 "synth_mbrola.cpp"
              var $pitch1115=(($100+20)&4294967295); //@line 519 "synth_mbrola.cpp"
              var $101=HEAPU8[($pitch1115)]; //@line 519 "synth_mbrola.cpp"
              var $conv116=((($101))&255); //@line 519 "synth_mbrola.cpp"
              var $102=$p; //@line 519 "synth_mbrola.cpp"
              var $pitch2117=(($102+21)&4294967295); //@line 519 "synth_mbrola.cpp"
              var $103=HEAPU8[($pitch2117)]; //@line 519 "synth_mbrola.cpp"
              var $conv118=((($103))&255); //@line 519 "synth_mbrola.cpp"
              var $104=HEAP32[(($len_percent)>>2)]; //@line 519 "synth_mbrola.cpp"
              var $sub119=(((-($104)))&4294967295); //@line 519 "synth_mbrola.cpp"
              __ZL10WritePitchiiiii($conv114, $conv116, $conv118, $sub119, 0); //@line 519 "synth_mbrola.cpp"
              $pitch101=((__ZZL10WritePitchiiiiiE6output)&4294967295); //@line 519 "synth_mbrola.cpp"
              var $105=$ptr; //@line 520 "synth_mbrola.cpp"
              var $106=HEAP32[(($name2)>>2)]; //@line 520 "synth_mbrola.cpp"
              var $call121=__Z12WordToStringj($106); //@line 520 "synth_mbrola.cpp"
              var $107=$len; //@line 520 "synth_mbrola.cpp"
              var $108=$len1; //@line 520 "synth_mbrola.cpp"
              var $sub122=((($107)-($108))&4294967295); //@line 520 "synth_mbrola.cpp"
              var $109=$pitch101; //@line 520 "synth_mbrola.cpp"
              var $call123=_sprintf($105, ((__str9391)&4294967295), allocate([((__ZZ12WordToStringjE3buf)&4294967295),0,0,0,$sub122,0,0,0,$109,0,0,0], ["i8*",0,0,0,"i32",0,0,0,"i8*",0,0,0], ALLOC_STACK)); //@line 520 "synth_mbrola.cpp"
              var $110=$ptr; //@line 520 "synth_mbrola.cpp"
              var $add_ptr124=(($110+$call123)&4294967295); //@line 520 "synth_mbrola.cpp"
              $ptr=$add_ptr124; //@line 520 "synth_mbrola.cpp"
              ;
            }
  
            $done=1; //@line 522 "synth_mbrola.cpp"
            __label__ = 54;break $sw_epilog$$sw_bb$$sw_bb126$$sw_bb152$$sw_bb155$$sw_bb171$$sw_bb195$34; //@line 577 "synth_mbrola.cpp"
          }
          else if (__label__ == 32) {
  
            $released=0; //@line 526 "synth_mbrola.cpp"
            var $111=$next; //@line 527 "synth_mbrola.cpp"
            var $type127=(($111+15)&4294967295); //@line 527 "synth_mbrola.cpp"
            var $112=HEAPU8[($type127)]; //@line 527 "synth_mbrola.cpp"
            var $conv128=((($112))&255); //@line 527 "synth_mbrola.cpp"
            var $cmp129=((($conv128))|0)==2; //@line 527 "synth_mbrola.cpp"
            if ($cmp129) { __label__ = 33;; } else { __label__ = 34;; } //@line 527 "synth_mbrola.cpp"
            if (__label__ == 33) {
  
              $released=1; //@line 527 "synth_mbrola.cpp"
              ; //@line 527 "synth_mbrola.cpp"
            }
  
            var $113=$next; //@line 528 "synth_mbrola.cpp"
            var $type132=(($113+15)&4294967295); //@line 528 "synth_mbrola.cpp"
            var $114=HEAPU8[($type132)]; //@line 528 "synth_mbrola.cpp"
            var $conv133=((($114))&255); //@line 528 "synth_mbrola.cpp"
            var $cmp134=((($conv133))|0)==3; //@line 528 "synth_mbrola.cpp"
            if ($cmp134) { __label__ = 35;; } else { __label__ = 37;; } //@line 528 "synth_mbrola.cpp"
            $land_lhs_true135$$if_end139$50: do { 
              if (__label__ == 35) {
  
                var $115=$next; //@line 528 "synth_mbrola.cpp"
                var $newword136=(($115+19)&4294967295); //@line 528 "synth_mbrola.cpp"
                var $116=HEAP8[($newword136)]; //@line 528 "synth_mbrola.cpp"
                var $tobool137=(tempInt=(($116)),(tempInt>=128?tempInt-256:tempInt))!=0; //@line 528 "synth_mbrola.cpp"
                if ($tobool137) { __label__ = 37;break $land_lhs_true135$$if_end139$50; } //@line 528 "synth_mbrola.cpp"
  
                $released=1; //@line 528 "synth_mbrola.cpp"
                __label__ = 39;break $land_lhs_true135$$if_end139$50; //@line 530 "synth_mbrola.cpp"
              }
            } while(0);
            $if_end139$$if_end145$53: do { 
              if (__label__ == 37) {
  
                var $_pr1=$released; //@line 530 "synth_mbrola.cpp"
                var $cmp140=((($_pr1))|0)==0; //@line 530 "synth_mbrola.cpp"
                if (!($cmp140)) { __label__ = 39;break $if_end139$$if_end145$53; } //@line 530 "synth_mbrola.cpp"
  
                var $117=$p; //@line 531 "synth_mbrola.cpp"
                var $synthflags142=(($117+4)&4294967295); //@line 531 "synth_mbrola.cpp"
                var $118=HEAPU16[(($synthflags142)>>1)]; //@line 531 "synth_mbrola.cpp"
                var $conv143=((($118))&65535); //@line 531 "synth_mbrola.cpp"
                var $or=($conv143) | 8192; //@line 531 "synth_mbrola.cpp"
                var $conv144=((($or)) & 65535); //@line 531 "synth_mbrola.cpp"
                HEAP16[(($synthflags142)>>1)]=$conv144; //@line 531 "synth_mbrola.cpp"
                ; //@line 531 "synth_mbrola.cpp"
              }
            } while(0);
  
            var $119=$p; //@line 532 "synth_mbrola.cpp"
            __Z16InterpretPhonemeP10TranslatoriP12PHONEME_LISTP12PHONEME_DATA(0, 0, $119, $phdata); //@line 532 "synth_mbrola.cpp"
            var $call146=__Z9DoSample3P12PHONEME_DATAii($phdata, 0, -1); //@line 533 "synth_mbrola.cpp"
            $len=$call146; //@line 533 "synth_mbrola.cpp"
            var $120=$len; //@line 535 "synth_mbrola.cpp"
            var $mul147=((($120)*1000)&4294967295); //@line 535 "synth_mbrola.cpp"
            var $121=HEAP32[((_samplerate)>>2)]; //@line 535 "synth_mbrola.cpp"
            var $div148=((((($mul147))|0)/((($121))|0))|0); //@line 535 "synth_mbrola.cpp"
            $len=$div148; //@line 535 "synth_mbrola.cpp"
            var $122=$p; //@line 536 "synth_mbrola.cpp"
            var $prepause=(($122+16)&4294967295); //@line 536 "synth_mbrola.cpp"
            var $123=HEAPU8[($prepause)]; //@line 536 "synth_mbrola.cpp"
            var $conv149=((($123))&255); //@line 536 "synth_mbrola.cpp"
            var $call150=__Z11PauseLengthii($conv149, 1); //@line 536 "synth_mbrola.cpp"
            var $124=$len; //@line 536 "synth_mbrola.cpp"
            var $add151=((($124)+($call150))&4294967295); //@line 536 "synth_mbrola.cpp"
            $len=$add151; //@line 536 "synth_mbrola.cpp"
            __label__ = 50;break $sw_epilog$$sw_bb$$sw_bb126$$sw_bb152$$sw_bb155$$sw_bb171$$sw_bb195$34; //@line 537 "synth_mbrola.cpp"
          }
          else if (__label__ == 40) {
  
            var $125=HEAP32[((((_speed+12)&4294967295))>>2)]; //@line 540 "synth_mbrola.cpp"
            var $mul153=((($125)*80)&4294967295); //@line 540 "synth_mbrola.cpp"
            var $div154=((((($mul153))|0)/256)|0); //@line 540 "synth_mbrola.cpp"
            $len=$div154; //@line 540 "synth_mbrola.cpp"
            __label__ = 50;break $sw_epilog$$sw_bb$$sw_bb126$$sw_bb152$$sw_bb155$$sw_bb171$$sw_bb195$34; //@line 541 "synth_mbrola.cpp"
          }
          else if (__label__ == 41) {
  
            $len=0; //@line 544 "synth_mbrola.cpp"
            var $126=$p; //@line 545 "synth_mbrola.cpp"
            __Z16InterpretPhonemeP10TranslatoriP12PHONEME_LISTP12PHONEME_DATA(0, 0, $126, $phdata); //@line 545 "synth_mbrola.cpp"
            var $127=$p; //@line 546 "synth_mbrola.cpp"
            var $synthflags156=(($127+4)&4294967295); //@line 546 "synth_mbrola.cpp"
            var $128=HEAPU16[(($synthflags156)>>1)]; //@line 546 "synth_mbrola.cpp"
            var $conv157=((($128))&65535); //@line 546 "synth_mbrola.cpp"
            var $and158=($conv157) & 8; //@line 546 "synth_mbrola.cpp"
            var $tobool159=((($and158))|0)!=0; //@line 546 "synth_mbrola.cpp"
            if ($tobool159) { __label__ = 42;; } else { __label__ = 43;; } //@line 546 "synth_mbrola.cpp"
            if (__label__ == 42) {
  
              var $129=$p; //@line 547 "synth_mbrola.cpp"
              var $length161=(($129+12)&4294967295); //@line 547 "synth_mbrola.cpp"
              var $130=HEAP16[(($length161)>>1)]; //@line 547 "synth_mbrola.cpp"
              var $conv162=(tempInt=(($130)),(tempInt>=32768?tempInt-65536:tempInt)); //@line 547 "synth_mbrola.cpp"
              var $call163=__Z9DoSample3P12PHONEME_DATAii($phdata, $conv162, -1); //@line 547 "synth_mbrola.cpp"
              $len=$call163; //@line 547 "synth_mbrola.cpp"
              ; //@line 547 "synth_mbrola.cpp"
            }
  
            var $131=$p; //@line 548 "synth_mbrola.cpp"
            var $length165=(($131+12)&4294967295); //@line 548 "synth_mbrola.cpp"
            var $132=HEAP16[(($length165)>>1)]; //@line 548 "synth_mbrola.cpp"
            var $conv166=(tempInt=(($132)),(tempInt>=32768?tempInt-65536:tempInt)); //@line 548 "synth_mbrola.cpp"
            var $call167=__Z9DoSample3P12PHONEME_DATAii($phdata, $conv166, -1); //@line 548 "synth_mbrola.cpp"
            var $133=$len; //@line 548 "synth_mbrola.cpp"
            var $add168=((($133)+($call167))&4294967295); //@line 548 "synth_mbrola.cpp"
            $len=$add168; //@line 548 "synth_mbrola.cpp"
            var $134=$len; //@line 550 "synth_mbrola.cpp"
            var $mul169=((($134)*1000)&4294967295); //@line 550 "synth_mbrola.cpp"
            var $135=HEAP32[((_samplerate)>>2)]; //@line 550 "synth_mbrola.cpp"
            var $div170=((((($mul169))|0)/((($135))|0))|0); //@line 550 "synth_mbrola.cpp"
            $len=$div170; //@line 550 "synth_mbrola.cpp"
            __label__ = 50;break $sw_epilog$$sw_bb$$sw_bb126$$sw_bb152$$sw_bb155$$sw_bb171$$sw_bb195$34; //@line 551 "synth_mbrola.cpp"
          }
          else if (__label__ == 44) {
  
            var $136=$next; //@line 554 "synth_mbrola.cpp"
            var $type172=(($136+15)&4294967295); //@line 554 "synth_mbrola.cpp"
            var $137=HEAPU8[($type172)]; //@line 554 "synth_mbrola.cpp"
            var $conv173=((($137))&255); //@line 554 "synth_mbrola.cpp"
            var $cmp174=((($conv173))|0)!=2; //@line 554 "synth_mbrola.cpp"
            if (!($cmp174)) { __label__ = 50;break $sw_epilog$$sw_bb$$sw_bb126$$sw_bb152$$sw_bb155$$sw_bb171$$sw_bb195$34; } //@line 554 "synth_mbrola.cpp"
  
            var $dest$, $stop$, $stop4$, $fast$, $value4$;
            $dest$ = $1;
            $stop$ = $dest$ + 48;
            $value4$ = 0;
            if ($value4$ < 0) $value4$ += 256;
            $value4$ = $value4$ + ($value4$<<8) + ($value4$<<16) + ($value4$*16777216);
            while ($dest$%4 !== 0 && $dest$ < $stop$) {
              ;; HEAP8[$dest$++] = 0;
            }
            $dest$ >>= 2;
            $stop4$ = $stop$ >> 2;
            while ($dest$ < $stop4$) {
            ;;;;;;;  HEAP32[$dest$++] = $value4$;
            }
            $dest$ <<= 2;
            while ($dest$ < $stop$) {
              ;; HEAP8[$dest$++] = 0;
            }; //@line 556 "synth_mbrola.cpp"
            var $138=$p; //@line 557 "synth_mbrola.cpp"
            __Z16InterpretPhonemeP10TranslatoriP12PHONEME_LISTP12PHONEME_DATA(0, 0, $138, $phdata); //@line 557 "synth_mbrola.cpp"
            var $139=HEAP32[(($arrayidx176)>>2)]; //@line 558 "synth_mbrola.cpp"
            HEAP32[(($fmt_addr)>>2)]=$139; //@line 558 "synth_mbrola.cpp"
            var $140=$p; //@line 559 "synth_mbrola.cpp"
            var $ph177=(($140+8)&4294967295); //@line 559 "synth_mbrola.cpp"
            var $141=HEAP32[(($ph177)>>2)]; //@line 559 "synth_mbrola.cpp"
            var $142=$p; //@line 559 "synth_mbrola.cpp"
            var $call178=__Z8DoSpect2P11PHONEME_TABiP10FMT_PARAMSP12PHONEME_LISTi($141, 0, $fmtp, $142, -1); //@line 559 "synth_mbrola.cpp"
            $len=$call178; //@line 559 "synth_mbrola.cpp"
            var $143=$len; //@line 561 "synth_mbrola.cpp"
            var $mul179=((($143)*1000)&4294967295); //@line 561 "synth_mbrola.cpp"
            var $144=HEAP32[((_samplerate)>>2)]; //@line 561 "synth_mbrola.cpp"
            var $div180=((((($mul179))|0)/((($144))|0))|0); //@line 561 "synth_mbrola.cpp"
            $len=$div180; //@line 561 "synth_mbrola.cpp"
            var $145=$next; //@line 562 "synth_mbrola.cpp"
            var $type181=(($145+15)&4294967295); //@line 562 "synth_mbrola.cpp"
            var $146=HEAPU8[($type181)]; //@line 562 "synth_mbrola.cpp"
            var $conv182=((($146))&255); //@line 562 "synth_mbrola.cpp"
            var $cmp183=((($conv182))|0)==0; //@line 562 "synth_mbrola.cpp"
            if ($cmp183) { __label__ = 46;; } else { __label__ = 47;; } //@line 562 "synth_mbrola.cpp"
            if (__label__ == 46) {
  
              var $147=$len; //@line 563 "synth_mbrola.cpp"
              var $add185=((($147)+50)&4294967295); //@line 563 "synth_mbrola.cpp"
              $len=$add185; //@line 563 "synth_mbrola.cpp"
              ; //@line 563 "synth_mbrola.cpp"
            }
  
            var $148=$p; //@line 564 "synth_mbrola.cpp"
            var $env187=(($148+14)&4294967295); //@line 564 "synth_mbrola.cpp"
            var $149=HEAPU8[($env187)]; //@line 564 "synth_mbrola.cpp"
            var $conv188=((($149))&255); //@line 564 "synth_mbrola.cpp"
            var $150=$p; //@line 564 "synth_mbrola.cpp"
            var $pitch1189=(($150+20)&4294967295); //@line 564 "synth_mbrola.cpp"
            var $151=HEAPU8[($pitch1189)]; //@line 564 "synth_mbrola.cpp"
            var $conv190=((($151))&255); //@line 564 "synth_mbrola.cpp"
            var $152=$p; //@line 564 "synth_mbrola.cpp"
            var $pitch2191=(($152+21)&4294967295); //@line 564 "synth_mbrola.cpp"
            var $153=HEAPU8[($pitch2191)]; //@line 564 "synth_mbrola.cpp"
            var $conv192=((($153))&255); //@line 564 "synth_mbrola.cpp"
            __ZL10WritePitchiiiii($conv188, $conv190, $conv192, 0, 1); //@line 564 "synth_mbrola.cpp"
            $final_pitch=((__ZZL10WritePitchiiiiiE6output)&4294967295); //@line 564 "synth_mbrola.cpp"
            __label__ = 50;break $sw_epilog$$sw_bb$$sw_bb126$$sw_bb152$$sw_bb155$$sw_bb171$$sw_bb195$34; //@line 565 "synth_mbrola.cpp"
          }
          else if (__label__ == 48) {
  
            var $154=$next; //@line 569 "synth_mbrola.cpp"
            var $type196=(($154+15)&4294967295); //@line 569 "synth_mbrola.cpp"
            var $155=HEAPU8[($type196)]; //@line 569 "synth_mbrola.cpp"
            var $conv197=((($155))&255); //@line 569 "synth_mbrola.cpp"
            var $cmp198=((($conv197))|0)==0; //@line 569 "synth_mbrola.cpp"
            if (!($cmp198)) { __label__ = 50;break $sw_epilog$$sw_bb$$sw_bb126$$sw_bb152$$sw_bb155$$sw_bb171$$sw_bb195$34; } //@line 569 "synth_mbrola.cpp"
  
            var $156=$len; //@line 571 "synth_mbrola.cpp"
            var $add200=((($156)+50)&4294967295); //@line 571 "synth_mbrola.cpp"
            $len=$add200; //@line 571 "synth_mbrola.cpp"
            var $157=$p; //@line 572 "synth_mbrola.cpp"
            var $env201=(($157+14)&4294967295); //@line 572 "synth_mbrola.cpp"
            var $158=HEAPU8[($env201)]; //@line 572 "synth_mbrola.cpp"
            var $conv202=((($158))&255); //@line 572 "synth_mbrola.cpp"
            var $159=$p; //@line 572 "synth_mbrola.cpp"
            var $pitch1203=(($159+20)&4294967295); //@line 572 "synth_mbrola.cpp"
            var $160=HEAPU8[($pitch1203)]; //@line 572 "synth_mbrola.cpp"
            var $conv204=((($160))&255); //@line 572 "synth_mbrola.cpp"
            var $161=$p; //@line 572 "synth_mbrola.cpp"
            var $pitch2205=(($161+21)&4294967295); //@line 572 "synth_mbrola.cpp"
            var $162=HEAPU8[($pitch2205)]; //@line 572 "synth_mbrola.cpp"
            var $conv206=((($162))&255); //@line 572 "synth_mbrola.cpp"
            __ZL10WritePitchiiiii($conv202, $conv204, $conv206, 0, 1); //@line 572 "synth_mbrola.cpp"
            $final_pitch=((__ZZL10WritePitchiiiiiE6output)&4294967295); //@line 572 "synth_mbrola.cpp"
            __label__ = 50;break $sw_epilog$$sw_bb$$sw_bb126$$sw_bb152$$sw_bb155$$sw_bb171$$sw_bb195$34; //@line 573 "synth_mbrola.cpp"
          }
        } while(0);
        $sw_epilog$$if_end222$69: do { 
          if (__label__ == 50) {
  
            var $_pr3=$done; //@line 577 "synth_mbrola.cpp"
            var $tobool209=((($_pr3))|0)!=0; //@line 577 "synth_mbrola.cpp"
            if ($tobool209) { __label__ = 54;break $sw_epilog$$if_end222$69; } //@line 577 "synth_mbrola.cpp"
  
            var $163=HEAP32[(($name2)>>2)]; //@line 579 "synth_mbrola.cpp"
            var $cmp211=((($163))|0)!=0; //@line 579 "synth_mbrola.cpp"
            if ($cmp211) { __label__ = 52;; } else { __label__ = 53;; } //@line 579 "synth_mbrola.cpp"
            if (__label__ == 52) {
  
              var $164=$len; //@line 581 "synth_mbrola.cpp"
              var $165=HEAP32[(($len_percent)>>2)]; //@line 581 "synth_mbrola.cpp"
              var $mul213=((($165)*($164))&4294967295); //@line 581 "synth_mbrola.cpp"
              var $div214=((((($mul213))|0)/100)|0); //@line 581 "synth_mbrola.cpp"
              $len1=$div214; //@line 581 "synth_mbrola.cpp"
              var $166=$ptr; //@line 582 "synth_mbrola.cpp"
              var $167=$len1; //@line 582 "synth_mbrola.cpp"
              var $168=HEAP32[(($name2)>>2)]; //@line 582 "synth_mbrola.cpp"
              var $call215=__Z12WordToStringj($168); //@line 582 "synth_mbrola.cpp"
              var $call216=_sprintf($166, ((__str10392)&4294967295), allocate([$167,0,0,0,((__ZZ12WordToStringjE3buf)&4294967295),0,0,0], ["i32",0,0,0,"i8*",0,0,0], ALLOC_STACK)); //@line 582 "synth_mbrola.cpp"
              var $169=$ptr; //@line 582 "synth_mbrola.cpp"
              var $add_ptr217=(($169+$call216)&4294967295); //@line 582 "synth_mbrola.cpp"
              $ptr=$add_ptr217; //@line 582 "synth_mbrola.cpp"
              var $170=$len1; //@line 583 "synth_mbrola.cpp"
              var $171=$len; //@line 583 "synth_mbrola.cpp"
              var $sub218=((($171)-($170))&4294967295); //@line 583 "synth_mbrola.cpp"
              $len=$sub218; //@line 583 "synth_mbrola.cpp"
              ; //@line 584 "synth_mbrola.cpp"
            }
  
            var $172=$ptr; //@line 585 "synth_mbrola.cpp"
            var $173=$len; //@line 585 "synth_mbrola.cpp"
            var $174=$final_pitch; //@line 585 "synth_mbrola.cpp"
            var $call220=_sprintf($172, ((__str11393)&4294967295), allocate([$173,0,0,0,$174,0,0,0], ["i32",0,0,0,"i8*",0,0,0], ALLOC_STACK)); //@line 585 "synth_mbrola.cpp"
            var $175=$ptr; //@line 585 "synth_mbrola.cpp"
            var $add_ptr221=(($175+$call220)&4294967295); //@line 585 "synth_mbrola.cpp"
            $ptr=$add_ptr221; //@line 585 "synth_mbrola.cpp"
            ; //@line 586 "synth_mbrola.cpp"
          }
        } while(0);
  
        var $176=$pause; //@line 588 "synth_mbrola.cpp"
        var $tobool223=((($176))|0)!=0; //@line 588 "synth_mbrola.cpp"
        if ($tobool223) { __label__ = 55;; } else { __label__ = 56;; } //@line 588 "synth_mbrola.cpp"
        if (__label__ == 55) {
  
          var $177=$pause; //@line 590 "synth_mbrola.cpp"
          var $call225=__Z11PauseLengthii($177, 0); //@line 590 "synth_mbrola.cpp"
          var $178=$len; //@line 590 "synth_mbrola.cpp"
          var $add226=((($178)+($call225))&4294967295); //@line 590 "synth_mbrola.cpp"
          $len=$add226; //@line 590 "synth_mbrola.cpp"
          var $179=$ptr; //@line 591 "synth_mbrola.cpp"
          var $180=$pause; //@line 591 "synth_mbrola.cpp"
          var $call227=__Z11PauseLengthii($180, 0); //@line 591 "synth_mbrola.cpp"
          var $call228=_sprintf($179, ((__str12394)&4294967295), allocate([$call227,0,0,0], ["i32",0,0,0], ALLOC_STACK)); //@line 591 "synth_mbrola.cpp"
          var $181=$ptr; //@line 591 "synth_mbrola.cpp"
          var $add_ptr229=(($181+$call228)&4294967295); //@line 591 "synth_mbrola.cpp"
          $ptr=$add_ptr229; //@line 591 "synth_mbrola.cpp"
          $pause=0; //@line 592 "synth_mbrola.cpp"
          ; //@line 593 "synth_mbrola.cpp"
        }
  
        var $182=$f_mbrola_addr; //@line 595 "synth_mbrola.cpp"
        var $tobool231=((($182))|0)!=0; //@line 595 "synth_mbrola.cpp"
        if ($tobool231) { __label__ = 57;; } else { __label__ = 58;; } //@line 595 "synth_mbrola.cpp"
        if (__label__ == 57) {
  
          var $183=$ptr; //@line 597 "synth_mbrola.cpp"
          var $sub_ptr_lhs_cast=($183); //@line 597 "synth_mbrola.cpp"
          var $sub_ptr_sub=((($sub_ptr_lhs_cast)-($sub_ptr_rhs_cast))&4294967295); //@line 597 "synth_mbrola.cpp"
          var $184=$f_mbrola_addr; //@line 597 "synth_mbrola.cpp"
          var $call235=_fwrite($arraydecay233, 1, $sub_ptr_sub, $184); //@line 597 "synth_mbrola.cpp"
          ; //@line 598 "synth_mbrola.cpp"
        }
        else if (__label__ == 58) {
  
          var $call238=_write_MBR($arraydecay233); //@line 601 "synth_mbrola.cpp"
          $res=$call238; //@line 601 "synth_mbrola.cpp"
          var $185=$res; //@line 602 "synth_mbrola.cpp"
          var $cmp239=((($185))|0) < 0; //@line 602 "synth_mbrola.cpp"
          if ($cmp239) { __label__ = 59;break $while_cond$5; } //@line 602 "synth_mbrola.cpp"
  
          var $186=$res; //@line 604 "synth_mbrola.cpp"
          var $cmp242=((($186))|0)==0; //@line 604 "synth_mbrola.cpp"
          if ($cmp242) { __label__ = 61;break $while_cond$5; } //@line 604 "synth_mbrola.cpp"
  
          var $187=HEAP32[((_wcmdq_tail)>>2)]; //@line 606 "synth_mbrola.cpp"
          var $arrayidx245=((_wcmdq+$187*16)&4294967295); //@line 606 "synth_mbrola.cpp"
          var $arrayidx246=(($arrayidx245)&4294967295); //@line 606 "synth_mbrola.cpp"
          HEAP32[(($arrayidx246)>>2)]=13; //@line 606 "synth_mbrola.cpp"
          var $188=$len; //@line 607 "synth_mbrola.cpp"
          var $189=HEAP32[((_wcmdq_tail)>>2)]; //@line 607 "synth_mbrola.cpp"
          var $arrayidx247=((_wcmdq+$189*16)&4294967295); //@line 607 "synth_mbrola.cpp"
          var $arrayidx248=(($arrayidx247+4)&4294967295); //@line 607 "synth_mbrola.cpp"
          HEAP32[(($arrayidx248)>>2)]=$188; //@line 607 "synth_mbrola.cpp"
          __Z8WcmdqIncv(); //@line 608 "synth_mbrola.cpp"
          ;
        }
  
        var $190=HEAP32[((__ZZ15MbrolaTranslateP12PHONEME_LISTiiP7__sFILEE4phix)>>2)]; //@line 611 "synth_mbrola.cpp"
        var $inc250=((($190)+1)&4294967295); //@line 611 "synth_mbrola.cpp"
        HEAP32[((__ZZ15MbrolaTranslateP12PHONEME_LISTiiP7__sFILEE4phix)>>2)]=$inc250; //@line 611 "synth_mbrola.cpp"
        __label__ = 3;continue $while_cond$5; //@line 612 "synth_mbrola.cpp"
      }
    }
    if (__label__ == 64) {
  
      var $191=$f_mbrola_addr; //@line 614 "synth_mbrola.cpp"
      var $tobool251=((($191))|0)!=0; //@line 614 "synth_mbrola.cpp"
      if ($tobool251) { __label__ = 66;; } else { __label__ = 65;; } //@line 614 "synth_mbrola.cpp"
      if (__label__ == 65) {
  
        var $call253=_flush_MBR(); //@line 616 "synth_mbrola.cpp"
        var $192=HEAP32[((_wcmdq_tail)>>2)]; //@line 619 "synth_mbrola.cpp"
        var $arrayidx254=((_wcmdq+$192*16)&4294967295); //@line 619 "synth_mbrola.cpp"
        var $arrayidx255=(($arrayidx254)&4294967295); //@line 619 "synth_mbrola.cpp"
        HEAP32[(($arrayidx255)>>2)]=13; //@line 619 "synth_mbrola.cpp"
        var $193=HEAP32[((_wcmdq_tail)>>2)]; //@line 620 "synth_mbrola.cpp"
        var $arrayidx256=((_wcmdq+$193*16)&4294967295); //@line 620 "synth_mbrola.cpp"
        var $arrayidx257=(($arrayidx256+4)&4294967295); //@line 620 "synth_mbrola.cpp"
        HEAP32[(($arrayidx257)>>2)]=500; //@line 620 "synth_mbrola.cpp"
        __Z8WcmdqIncv(); //@line 621 "synth_mbrola.cpp"
        ; //@line 622 "synth_mbrola.cpp"
      }
  
      $retval=0; //@line 624 "synth_mbrola.cpp"
      ; //@line 624 "synth_mbrola.cpp"
    }
    else if (__label__ == 5) {
  
      $retval=1; //@line 439 "synth_mbrola.cpp"
      ; //@line 439 "synth_mbrola.cpp"
    }
    else if (__label__ == 59) {
  
      $retval=0; //@line 603 "synth_mbrola.cpp"
      ; //@line 603 "synth_mbrola.cpp"
    }
    else if (__label__ == 61) {
  
      $retval=1; //@line 605 "synth_mbrola.cpp"
      ; //@line 605 "synth_mbrola.cpp"
    }
  
    var $194=$retval; //@line 625 "synth_mbrola.cpp"
    STACKTOP = __stackBase__;
    return $194; //@line 625 "synth_mbrola.cpp"
    return null;
  }
  

  function __ZL10GetMbrNameP12PHONEME_LISTP11PHONEME_TABS2_S2_PiS3_S3_($plist, $ph, $ph_prev, $ph_next, $name2, $split, $control) {
    ;
    var __label__;
  
    var $retval;
    var $plist_addr;
    var $ph_addr;
    var $ph_prev_addr;
    var $ph_next_addr;
    var $name2_addr;
    var $split_addr;
    var $control_addr;
    var $pr;
    var $other_ph;
    var $found;
    $plist_addr=$plist;
    $ph_addr=$ph;
    $ph_prev_addr=$ph_prev;
    $ph_next_addr=$ph_next;
    $name2_addr=$name2;
    $split_addr=$split;
    $control_addr=$control;
    $found=0; //@line 209 "synth_mbrola.cpp"
    var $0=$name2_addr; //@line 219 "synth_mbrola.cpp"
    HEAP32[(($0)>>2)]=0; //@line 219 "synth_mbrola.cpp"
    var $1=$split_addr; //@line 220 "synth_mbrola.cpp"
    HEAP32[(($1)>>2)]=0; //@line 220 "synth_mbrola.cpp"
    var $2=$control_addr; //@line 221 "synth_mbrola.cpp"
    HEAP32[(($2)>>2)]=0; //@line 221 "synth_mbrola.cpp"
    var $3=$ph_addr; //@line 222 "synth_mbrola.cpp"
    var $mnemonic=(($3)&4294967295); //@line 222 "synth_mbrola.cpp"
    var $4=HEAP32[(($mnemonic)>>2)]; //@line 222 "synth_mbrola.cpp"
    HEAP32[((__ZZL10GetMbrNameP12PHONEME_LISTP11PHONEME_TABS2_S2_PiS3_S3_E4mnem)>>2)]=$4; //@line 222 "synth_mbrola.cpp"
    var $5=HEAP32[((__ZL10mbrola_tab)>>2)]; //@line 224 "synth_mbrola.cpp"
    $pr=$5; //@line 224 "synth_mbrola.cpp"
    ; //@line 225 "synth_mbrola.cpp"
    $while_cond$2: while(1) { 
  
      var $6=$pr; //@line 225 "synth_mbrola.cpp"
      var $name=(($6)&4294967295); //@line 225 "synth_mbrola.cpp"
      var $7=HEAP32[(($name)>>2)]; //@line 225 "synth_mbrola.cpp"
      var $cmp=((($7))|0)!=0; //@line 225 "synth_mbrola.cpp"
      if (!($cmp)) { __label__ = 28;break $while_cond$2; } //@line 225 "synth_mbrola.cpp"
  
      var $8=HEAP32[((__ZZL10GetMbrNameP12PHONEME_LISTP11PHONEME_TABS2_S2_PiS3_S3_E4mnem)>>2)]; //@line 227 "synth_mbrola.cpp"
      var $9=$pr; //@line 227 "synth_mbrola.cpp"
      var $name1=(($9)&4294967295); //@line 227 "synth_mbrola.cpp"
      var $10=HEAP32[(($name1)>>2)]; //@line 227 "synth_mbrola.cpp"
      var $cmp2=((($8))|0)==((($10))|0); //@line 227 "synth_mbrola.cpp"
      if ($cmp2) { __label__ = 3;; } else { __label__ = 27;; } //@line 227 "synth_mbrola.cpp"
      $if_then$$if_end60$5: do { 
        if (__label__ == 3) {
  
          var $11=$pr; //@line 229 "synth_mbrola.cpp"
          var $next_phoneme=(($11+4)&4294967295); //@line 229 "synth_mbrola.cpp"
          var $12=HEAP32[(($next_phoneme)>>2)]; //@line 229 "synth_mbrola.cpp"
          var $cmp3=((($12))|0)==0; //@line 229 "synth_mbrola.cpp"
          if ($cmp3) { __label__ = 4;; } else { __label__ = 5;; } //@line 229 "synth_mbrola.cpp"
          $if_then4$$if_else$7: do { 
            if (__label__ == 4) {
  
              $found=1; //@line 230 "synth_mbrola.cpp"
              ; //@line 230 "synth_mbrola.cpp"
            }
            else if (__label__ == 5) {
  
              var $13=$pr; //@line 232 "synth_mbrola.cpp"
              var $next_phoneme5=(($13+4)&4294967295); //@line 232 "synth_mbrola.cpp"
              var $14=HEAP32[(($next_phoneme5)>>2)]; //@line 232 "synth_mbrola.cpp"
              var $cmp6=((($14))|0)==58; //@line 232 "synth_mbrola.cpp"
              if ($cmp6) { __label__ = 6;; } else { __label__ = 8;; } //@line 232 "synth_mbrola.cpp"
              $land_lhs_true$$if_else8$10: do { 
                if (__label__ == 6) {
  
                  var $15=$plist_addr; //@line 232 "synth_mbrola.cpp"
                  var $synthflags=(($15+4)&4294967295); //@line 232 "synth_mbrola.cpp"
                  var $16=HEAPU16[(($synthflags)>>1)]; //@line 232 "synth_mbrola.cpp"
                  var $conv=((($16))&65535); //@line 232 "synth_mbrola.cpp"
                  var $and=($conv) & 8; //@line 232 "synth_mbrola.cpp"
                  var $tobool=((($and))|0)!=0; //@line 232 "synth_mbrola.cpp"
                  if (!($tobool)) { __label__ = 8;break $land_lhs_true$$if_else8$10; } //@line 232 "synth_mbrola.cpp"
  
                  $found=1; //@line 234 "synth_mbrola.cpp"
                  __label__ = 20;break $if_then4$$if_else$7; //@line 235 "synth_mbrola.cpp"
                }
              } while(0);
  
              var $17=$pr; //@line 238 "synth_mbrola.cpp"
              var $control9=(($17+20)&4294967295); //@line 238 "synth_mbrola.cpp"
              var $18=HEAP32[(($control9)>>2)]; //@line 238 "synth_mbrola.cpp"
              var $and10=($18) & 2; //@line 238 "synth_mbrola.cpp"
              var $tobool11=((($and10))|0)!=0; //@line 238 "synth_mbrola.cpp"
              if ($tobool11) { __label__ = 9;; } else { __label__ = 10;; } //@line 238 "synth_mbrola.cpp"
              $if_then12$$if_else13$14: do { 
                if (__label__ == 9) {
  
                  var $19=$ph_prev_addr; //@line 239 "synth_mbrola.cpp"
                  $other_ph=$19; //@line 239 "synth_mbrola.cpp"
                  ; //@line 239 "synth_mbrola.cpp"
                }
                else if (__label__ == 10) {
  
                  var $20=$pr; //@line 241 "synth_mbrola.cpp"
                  var $control14=(($20+20)&4294967295); //@line 241 "synth_mbrola.cpp"
                  var $21=HEAP32[(($control14)>>2)]; //@line 241 "synth_mbrola.cpp"
                  var $and15=($21) & 8; //@line 241 "synth_mbrola.cpp"
                  var $tobool16=((($and15))|0)!=0; //@line 241 "synth_mbrola.cpp"
                  if ($tobool16) { __label__ = 11;; } else { __label__ = 13;; } //@line 241 "synth_mbrola.cpp"
                  $land_lhs_true17$$if_else20$17: do { 
                    if (__label__ == 11) {
  
                      var $22=$plist_addr; //@line 241 "synth_mbrola.cpp"
                      var $add_ptr=(($22+24)&4294967295); //@line 241 "synth_mbrola.cpp"
                      var $newword=(($add_ptr+19)&4294967295); //@line 241 "synth_mbrola.cpp"
                      var $23=HEAP8[($newword)]; //@line 241 "synth_mbrola.cpp"
                      var $tobool18=(tempInt=(($23)),(tempInt>=128?tempInt-256:tempInt))!=0; //@line 241 "synth_mbrola.cpp"
                      if (!($tobool18)) { __label__ = 13;break $land_lhs_true17$$if_else20$17; } //@line 241 "synth_mbrola.cpp"
  
                      var $24=HEAP32[((((_phoneme_tab)&4294967295))>>2)]; //@line 242 "synth_mbrola.cpp"
                      $other_ph=$24; //@line 242 "synth_mbrola.cpp"
                      __label__ = 14;break $if_then12$$if_else13$14; //@line 242 "synth_mbrola.cpp"
                    }
                  } while(0);
  
                  var $25=$ph_next_addr; //@line 244 "synth_mbrola.cpp"
                  $other_ph=$25; //@line 244 "synth_mbrola.cpp"
                  ;
                }
              } while(0);
  
              var $26=$pr; //@line 246 "synth_mbrola.cpp"
              var $next_phoneme22=(($26+4)&4294967295); //@line 246 "synth_mbrola.cpp"
              var $27=HEAP32[(($next_phoneme22)>>2)]; //@line 246 "synth_mbrola.cpp"
              var $28=$other_ph; //@line 246 "synth_mbrola.cpp"
              var $mnemonic23=(($28)&4294967295); //@line 246 "synth_mbrola.cpp"
              var $29=HEAP32[(($mnemonic23)>>2)]; //@line 246 "synth_mbrola.cpp"
              var $cmp24=((($27))|0)==((($29))|0); //@line 246 "synth_mbrola.cpp"
              if ($cmp24) { __label__ = 19;; } else { __label__ = 15;; } //@line 246 "synth_mbrola.cpp"
              $if_then37$$lor_lhs_false$22: do { 
                if (__label__ == 15) {
  
                  var $30=$pr; //@line 246 "synth_mbrola.cpp"
                  var $next_phoneme25=(($30+4)&4294967295); //@line 246 "synth_mbrola.cpp"
                  var $31=HEAP32[(($next_phoneme25)>>2)]; //@line 246 "synth_mbrola.cpp"
                  var $cmp26=((($31))|0)==2; //@line 246 "synth_mbrola.cpp"
                  if ($cmp26) { __label__ = 16;; } else { __label__ = 17;; } //@line 246 "synth_mbrola.cpp"
                  if (__label__ == 16) {
  
                    var $32=$other_ph; //@line 246 "synth_mbrola.cpp"
                    var $type=(($32+11)&4294967295); //@line 246 "synth_mbrola.cpp"
                    var $33=HEAPU8[($type)]; //@line 246 "synth_mbrola.cpp"
                    var $conv28=((($33))&255); //@line 246 "synth_mbrola.cpp"
                    var $cmp29=((($conv28))|0)==2; //@line 246 "synth_mbrola.cpp"
                    if ($cmp29) { __label__ = 19;break $if_then37$$lor_lhs_false$22; } //@line 246 "synth_mbrola.cpp"
                  }
  
                  var $34=$pr; //@line 246 "synth_mbrola.cpp"
                  var $next_phoneme31=(($34+4)&4294967295); //@line 246 "synth_mbrola.cpp"
                  var $35=HEAP32[(($next_phoneme31)>>2)]; //@line 246 "synth_mbrola.cpp"
                  var $cmp32=((($35))|0)==95; //@line 246 "synth_mbrola.cpp"
                  if (!($cmp32)) { __label__ = 20;break $if_then4$$if_else$7; } //@line 246 "synth_mbrola.cpp"
  
                  var $36=$other_ph; //@line 246 "synth_mbrola.cpp"
                  var $type34=(($36+11)&4294967295); //@line 246 "synth_mbrola.cpp"
                  var $37=HEAPU8[($type34)]; //@line 246 "synth_mbrola.cpp"
                  var $conv35=((($37))&255); //@line 246 "synth_mbrola.cpp"
                  var $cmp36=((($conv35))|0)==0; //@line 246 "synth_mbrola.cpp"
                  if (!($cmp36)) { __label__ = 20;break $if_then4$$if_else$7; } //@line 246 "synth_mbrola.cpp"
                }
              } while(0);
  
              $found=1; //@line 250 "synth_mbrola.cpp"
              ; //@line 251 "synth_mbrola.cpp"
            }
          } while(0);
  
          var $38=$pr; //@line 254 "synth_mbrola.cpp"
          var $control41=(($38+20)&4294967295); //@line 254 "synth_mbrola.cpp"
          var $39=HEAP32[(($control41)>>2)]; //@line 254 "synth_mbrola.cpp"
          var $and42=($39) & 4; //@line 254 "synth_mbrola.cpp"
          var $tobool43=((($and42))|0)!=0; //@line 254 "synth_mbrola.cpp"
          if ($tobool43) { __label__ = 21;; } else { __label__ = 23;; } //@line 254 "synth_mbrola.cpp"
          $land_lhs_true44$$if_end49$30: do { 
            if (__label__ == 21) {
  
              var $40=$plist_addr; //@line 254 "synth_mbrola.cpp"
              var $newword45=(($40+19)&4294967295); //@line 254 "synth_mbrola.cpp"
              var $41=HEAPU8[($newword45)]; //@line 254 "synth_mbrola.cpp"
              var $conv46=((($41))&255); //@line 254 "synth_mbrola.cpp"
              var $cmp47=((($conv46))|0)==0; //@line 254 "synth_mbrola.cpp"
              if (!($cmp47)) { __label__ = 23;break $land_lhs_true44$$if_end49$30; } //@line 254 "synth_mbrola.cpp"
  
              $found=0; //@line 255 "synth_mbrola.cpp"
              __label__ = 27;break $if_then$$if_end60$5; //@line 257 "synth_mbrola.cpp"
            }
          } while(0);
  
          var $_pr=$found; //@line 257 "synth_mbrola.cpp"
          var $tobool50=((($_pr))|0)!=0; //@line 257 "synth_mbrola.cpp"
          if ($tobool50) { __label__ = 24;break $while_cond$2; } //@line 257 "synth_mbrola.cpp"
        }
      } while(0);
  
      var $55=$pr; //@line 273 "synth_mbrola.cpp"
      var $incdec_ptr=(($55+24)&4294967295); //@line 273 "synth_mbrola.cpp"
      $pr=$incdec_ptr; //@line 273 "synth_mbrola.cpp"
      __label__ = 1;continue $while_cond$2; //@line 274 "synth_mbrola.cpp"
    }
    $while_end$$if_then51$35: do { 
      if (__label__ == 24) {
  
        var $42=$pr; //@line 259 "synth_mbrola.cpp"
        var $mbr_name2=(($42+12)&4294967295); //@line 259 "synth_mbrola.cpp"
        var $43=HEAP32[(($mbr_name2)>>2)]; //@line 259 "synth_mbrola.cpp"
        var $44=$name2_addr; //@line 259 "synth_mbrola.cpp"
        HEAP32[(($44)>>2)]=$43; //@line 259 "synth_mbrola.cpp"
        var $45=$pr; //@line 260 "synth_mbrola.cpp"
        var $percent=(($45+16)&4294967295); //@line 260 "synth_mbrola.cpp"
        var $46=HEAP32[(($percent)>>2)]; //@line 260 "synth_mbrola.cpp"
        var $47=$split_addr; //@line 260 "synth_mbrola.cpp"
        HEAP32[(($47)>>2)]=$46; //@line 260 "synth_mbrola.cpp"
        var $48=$pr; //@line 261 "synth_mbrola.cpp"
        var $control52=(($48+20)&4294967295); //@line 261 "synth_mbrola.cpp"
        var $49=HEAP32[(($control52)>>2)]; //@line 261 "synth_mbrola.cpp"
        var $50=$control_addr; //@line 261 "synth_mbrola.cpp"
        HEAP32[(($50)>>2)]=$49; //@line 261 "synth_mbrola.cpp"
        var $51=$pr; //@line 263 "synth_mbrola.cpp"
        var $control53=(($51+20)&4294967295); //@line 263 "synth_mbrola.cpp"
        var $52=HEAP32[(($control53)>>2)]; //@line 263 "synth_mbrola.cpp"
        var $and54=($52) & 16; //@line 263 "synth_mbrola.cpp"
        var $tobool55=((($and54))|0)!=0; //@line 263 "synth_mbrola.cpp"
        var $53=$pr; //@line 265 "synth_mbrola.cpp"
        var $mbr_name=(($53+8)&4294967295); //@line 265 "synth_mbrola.cpp"
        var $54=HEAP32[(($mbr_name)>>2)]; //@line 265 "synth_mbrola.cpp"
        if ($tobool55) { __label__ = 25;; } else { __label__ = 26;; } //@line 263 "synth_mbrola.cpp"
        if (__label__ == 25) {
  
          HEAP32[((__ZL15mbr_name_prefix)>>2)]=$54; //@line 265 "synth_mbrola.cpp"
          $retval=0; //@line 266 "synth_mbrola.cpp"
          __label__ = 31;break $while_end$$if_then51$35; //@line 266 "synth_mbrola.cpp"
        }
        else if (__label__ == 26) {
  
          HEAP32[((__ZZL10GetMbrNameP12PHONEME_LISTP11PHONEME_TABS2_S2_PiS3_S3_E4mnem)>>2)]=$54; //@line 268 "synth_mbrola.cpp"
          __label__ = 28;break $while_end$$if_then51$35; //@line 269 "synth_mbrola.cpp"
        }
      }
    } while(0);
    if (__label__ == 28) {
  
      var $56=HEAP32[((__ZL15mbr_name_prefix)>>2)]; //@line 276 "synth_mbrola.cpp"
      var $cmp61=((($56))|0)!=0; //@line 276 "synth_mbrola.cpp"
      if ($cmp61) { __label__ = 29;; } else { __label__ = 30;; } //@line 276 "synth_mbrola.cpp"
      if (__label__ == 29) {
  
        var $57=HEAP32[((__ZZL10GetMbrNameP12PHONEME_LISTP11PHONEME_TABS2_S2_PiS3_S3_E4mnem)>>2)]; //@line 278 "synth_mbrola.cpp"
        var $shl=($57) << 8; //@line 278 "synth_mbrola.cpp"
        var $58=HEAP32[((__ZL15mbr_name_prefix)>>2)]; //@line 278 "synth_mbrola.cpp"
        var $and63=($58) & 255; //@line 278 "synth_mbrola.cpp"
        var $or=($and63) | ($shl); //@line 278 "synth_mbrola.cpp"
        HEAP32[((__ZZL10GetMbrNameP12PHONEME_LISTP11PHONEME_TABS2_S2_PiS3_S3_E4mnem)>>2)]=$or; //@line 278 "synth_mbrola.cpp"
        ; //@line 279 "synth_mbrola.cpp"
      }
  
      HEAP32[((__ZL15mbr_name_prefix)>>2)]=0; //@line 280 "synth_mbrola.cpp"
      var $59=HEAP32[((__ZZL10GetMbrNameP12PHONEME_LISTP11PHONEME_TABS2_S2_PiS3_S3_E4mnem)>>2)]; //@line 281 "synth_mbrola.cpp"
      $retval=$59; //@line 281 "synth_mbrola.cpp"
      ; //@line 281 "synth_mbrola.cpp"
    }
  
    var $60=$retval; //@line 282 "synth_mbrola.cpp"
    ;
    return $60; //@line 282 "synth_mbrola.cpp"
    return null;
  }
  

  function __ZL10WritePitchiiiii($env, $pitch1, $pitch2, $split, $final) {
    var __stackBase__  = STACKTOP; STACKTOP += 76; _memset(__stackBase__, 0, 76);
    var __label__;
    var __lastLabel__ = null;
  
    var $env_addr;
    var $pitch1_addr;
    var $pitch2_addr;
    var $split_addr;
    var $final_addr;
    var $x;
    var $ix;
    var $pitch_base=__stackBase__;
    var $pitch_range=__stackBase__+4;
    var $p1;
    var $p2;
    var $p_end;
    var $pitch_env;
    var $max;
    var $min;
    var $y_max;
    var $y_min;
    var $env100;
    var $y2;
    var $y=__stackBase__+8;
    var $env_split;
    var $buf=__stackBase__+24;
    $env_addr=$env;
    $pitch1_addr=$pitch1;
    $pitch2_addr=$pitch2;
    $split_addr=$split;
    $final_addr=$final;
    $max=-1; //@line 294 "synth_mbrola.cpp"
    $min=999; //@line 295 "synth_mbrola.cpp"
    $y_max=0; //@line 296 "synth_mbrola.cpp"
    $y_min=0; //@line 297 "synth_mbrola.cpp"
    $env100=80; //@line 298 "synth_mbrola.cpp"
    HEAP8[(((__ZZL10WritePitchiiiiiE6output)&4294967295))]=0; //@line 305 "synth_mbrola.cpp"
    var $0=$env_addr; //@line 306 "synth_mbrola.cpp"
    var $arrayidx=((_envelope_data+$0*4)&4294967295); //@line 306 "synth_mbrola.cpp"
    var $1=HEAP32[(($arrayidx)>>2)]; //@line 306 "synth_mbrola.cpp"
    $pitch_env=$1; //@line 306 "synth_mbrola.cpp"
    var $2=HEAP32[((_voice)>>2)]; //@line 309 "synth_mbrola.cpp"
    var $3=$pitch1_addr; //@line 309 "synth_mbrola.cpp"
    var $4=$pitch2_addr; //@line 309 "synth_mbrola.cpp"
    __Z9SetPitch2P7voice_tiiPiS1_($2, $3, $4, $pitch_base, $pitch_range); //@line 309 "synth_mbrola.cpp"
    var $5=$split_addr; //@line 312 "synth_mbrola.cpp"
    var $mul=((($5)*128)&4294967295); //@line 312 "synth_mbrola.cpp"
    var $div=((((($mul))|0)/100)|0); //@line 312 "synth_mbrola.cpp"
    $env_split=$div; //@line 312 "synth_mbrola.cpp"
    var $6=$env_split; //@line 313 "synth_mbrola.cpp"
    var $cmp=((($6))|0) < 0; //@line 313 "synth_mbrola.cpp"
    if ($cmp) { __label__ = 1;; } else { __label__ = 2;; } //@line 313 "synth_mbrola.cpp"
    if (__label__ == 1) {
  
      var $7=$env_split; //@line 314 "synth_mbrola.cpp"
      var $sub=(((-($7)))&4294967295); //@line 314 "synth_mbrola.cpp"
      $env_split=$sub; //@line 314 "synth_mbrola.cpp"
      ; //@line 314 "synth_mbrola.cpp"
    }
  
    $x=0; //@line 317 "synth_mbrola.cpp"
    ; //@line 317 "synth_mbrola.cpp"
    $for_body$5: while(1) { 
  
      var $8=$x; //@line 319 "synth_mbrola.cpp"
      var $9=$pitch_env; //@line 319 "synth_mbrola.cpp"
      var $arrayidx2=(($9+$8)&4294967295); //@line 319 "synth_mbrola.cpp"
      var $10=HEAPU8[($arrayidx2)]; //@line 319 "synth_mbrola.cpp"
      var $conv=((($10))&255); //@line 319 "synth_mbrola.cpp"
      var $11=$max; //@line 319 "synth_mbrola.cpp"
      var $cmp3=((($conv))|0) > ((($11))|0); //@line 319 "synth_mbrola.cpp"
      if ($cmp3) { __label__ = 4;; } else { __label__ = 5;; } //@line 319 "synth_mbrola.cpp"
      if (__label__ == 4) {
  
        var $12=$x; //@line 321 "synth_mbrola.cpp"
        var $13=$pitch_env; //@line 321 "synth_mbrola.cpp"
        var $arrayidx5=(($13+$12)&4294967295); //@line 321 "synth_mbrola.cpp"
        var $14=HEAPU8[($arrayidx5)]; //@line 321 "synth_mbrola.cpp"
        var $conv6=((($14))&255); //@line 321 "synth_mbrola.cpp"
        $max=$conv6; //@line 321 "synth_mbrola.cpp"
        var $15=$x; //@line 322 "synth_mbrola.cpp"
        $y_max=$15; //@line 322 "synth_mbrola.cpp"
        ; //@line 323 "synth_mbrola.cpp"
      }
  
      var $16=$x; //@line 324 "synth_mbrola.cpp"
      var $17=$pitch_env; //@line 324 "synth_mbrola.cpp"
      var $arrayidx8=(($17+$16)&4294967295); //@line 324 "synth_mbrola.cpp"
      var $18=HEAPU8[($arrayidx8)]; //@line 324 "synth_mbrola.cpp"
      var $conv9=((($18))&255); //@line 324 "synth_mbrola.cpp"
      var $19=$min; //@line 324 "synth_mbrola.cpp"
      var $cmp10=((($conv9))|0) < ((($19))|0); //@line 324 "synth_mbrola.cpp"
      if ($cmp10) { __label__ = 6;; } else { __label__ = 7;; } //@line 324 "synth_mbrola.cpp"
      if (__label__ == 6) {
  
        var $20=$x; //@line 326 "synth_mbrola.cpp"
        var $21=$pitch_env; //@line 326 "synth_mbrola.cpp"
        var $arrayidx12=(($21+$20)&4294967295); //@line 326 "synth_mbrola.cpp"
        var $22=HEAPU8[($arrayidx12)]; //@line 326 "synth_mbrola.cpp"
        var $conv13=((($22))&255); //@line 326 "synth_mbrola.cpp"
        $min=$conv13; //@line 326 "synth_mbrola.cpp"
        var $23=$x; //@line 327 "synth_mbrola.cpp"
        $y_min=$23; //@line 327 "synth_mbrola.cpp"
        ; //@line 328 "synth_mbrola.cpp"
      }
  
      var $24=$x; //@line 317 "synth_mbrola.cpp"
      var $inc=((($24)+1)&4294967295); //@line 317 "synth_mbrola.cpp"
      $x=$inc; //@line 317 "synth_mbrola.cpp"
      var $cmp1=((($inc))|0) < 128; //@line 317 "synth_mbrola.cpp"
      if ($cmp1) { __label__ = 3;continue $for_body$5; } else { __label__ = 8;break $for_body$5; } //@line 317 "synth_mbrola.cpp"
    }
  
    var $arrayidx15=(($y+8)&4294967295); //@line 332 "synth_mbrola.cpp"
    HEAP32[(($arrayidx15)>>2)]=64; //@line 332 "synth_mbrola.cpp"
    var $25=$y_max; //@line 333 "synth_mbrola.cpp"
    var $cmp16=((($25))|0) > 0; //@line 333 "synth_mbrola.cpp"
    var $26=$y_max; //@line 333 "synth_mbrola.cpp"
    var $cmp17=((($26))|0) < 127; //@line 333 "synth_mbrola.cpp"
    var $or_cond=($cmp16) & ($cmp17); //@line 333 "synth_mbrola.cpp"
    if ($or_cond) { __label__ = 9;; } else { __label__ = 10;; } //@line 333 "synth_mbrola.cpp"
    if (__label__ == 9) {
  
      var $27=$y_max; //@line 335 "synth_mbrola.cpp"
      var $arrayidx19=(($y+8)&4294967295); //@line 335 "synth_mbrola.cpp"
      HEAP32[(($arrayidx19)>>2)]=$27; //@line 335 "synth_mbrola.cpp"
      ; //@line 336 "synth_mbrola.cpp"
    }
  
    var $28=$y_min; //@line 337 "synth_mbrola.cpp"
    var $cmp21=((($28))|0) > 0; //@line 337 "synth_mbrola.cpp"
    var $29=$y_min; //@line 337 "synth_mbrola.cpp"
    var $cmp23=((($29))|0) < 127; //@line 337 "synth_mbrola.cpp"
    var $or_cond1=($cmp21) & ($cmp23); //@line 337 "synth_mbrola.cpp"
    if ($or_cond1) { __label__ = 11;; } else { __label__ = 12;; } //@line 337 "synth_mbrola.cpp"
    if (__label__ == 11) {
  
      var $30=$y_min; //@line 339 "synth_mbrola.cpp"
      var $arrayidx25=(($y+8)&4294967295); //@line 339 "synth_mbrola.cpp"
      HEAP32[(($arrayidx25)>>2)]=$30; //@line 339 "synth_mbrola.cpp"
      ; //@line 340 "synth_mbrola.cpp"
    }
  
    var $arrayidx27=(($y+8)&4294967295); //@line 341 "synth_mbrola.cpp"
    var $31=HEAP32[(($arrayidx27)>>2)]; //@line 341 "synth_mbrola.cpp"
    var $div28=((((($31))|0)/2)|0); //@line 341 "synth_mbrola.cpp"
    var $arrayidx29=(($y+4)&4294967295); //@line 341 "synth_mbrola.cpp"
    HEAP32[(($arrayidx29)>>2)]=$div28; //@line 341 "synth_mbrola.cpp"
    var $arrayidx30=(($y+8)&4294967295); //@line 342 "synth_mbrola.cpp"
    var $32=HEAP32[(($arrayidx30)>>2)]; //@line 342 "synth_mbrola.cpp"
    var $arrayidx31=(($y+8)&4294967295); //@line 342 "synth_mbrola.cpp"
    var $33=HEAP32[(($arrayidx31)>>2)]; //@line 342 "synth_mbrola.cpp"
    var $sub32=((127-($33))&4294967295); //@line 342 "synth_mbrola.cpp"
    var $div33=((((($sub32))|0)/2)|0); //@line 342 "synth_mbrola.cpp"
    var $add=((($div33)+($32))&4294967295); //@line 342 "synth_mbrola.cpp"
    var $arrayidx34=(($y+12)&4294967295); //@line 342 "synth_mbrola.cpp"
    HEAP32[(($arrayidx34)>>2)]=$add; //@line 342 "synth_mbrola.cpp"
    var $34=$pitch_env; //@line 345 "synth_mbrola.cpp"
    var $arrayidx35=(($34)&4294967295); //@line 345 "synth_mbrola.cpp"
    var $35=HEAPU8[($arrayidx35)]; //@line 345 "synth_mbrola.cpp"
    var $conv36=((($35))&255); //@line 345 "synth_mbrola.cpp"
    var $36=HEAP32[(($pitch_range)>>2)]; //@line 345 "synth_mbrola.cpp"
    var $mul37=((($conv36)*($36))&4294967295); //@line 345 "synth_mbrola.cpp"
    var $shr=($mul37) >> 8; //@line 345 "synth_mbrola.cpp"
    var $37=HEAP32[(($pitch_base)>>2)]; //@line 345 "synth_mbrola.cpp"
    var $add38=((($shr)+($37))&4294967295); //@line 345 "synth_mbrola.cpp"
    $p1=$add38; //@line 345 "synth_mbrola.cpp"
    var $38=$pitch_env; //@line 346 "synth_mbrola.cpp"
    var $arrayidx39=(($38+127)&4294967295); //@line 346 "synth_mbrola.cpp"
    var $39=HEAPU8[($arrayidx39)]; //@line 346 "synth_mbrola.cpp"
    var $conv40=((($39))&255); //@line 346 "synth_mbrola.cpp"
    var $40=HEAP32[(($pitch_range)>>2)]; //@line 346 "synth_mbrola.cpp"
    var $mul41=((($conv40)*($40))&4294967295); //@line 346 "synth_mbrola.cpp"
    var $shr42=($mul41) >> 8; //@line 346 "synth_mbrola.cpp"
    var $41=HEAP32[(($pitch_base)>>2)]; //@line 346 "synth_mbrola.cpp"
    var $add43=((($shr42)+($41))&4294967295); //@line 346 "synth_mbrola.cpp"
    $p_end=$add43; //@line 346 "synth_mbrola.cpp"
    var $42=$split_addr; //@line 349 "synth_mbrola.cpp"
    var $cmp44=((($42))|0) >= 0; //@line 349 "synth_mbrola.cpp"
    if ($cmp44) { __label__ = 13;; } else { __label__ = 14;; } //@line 349 "synth_mbrola.cpp"
    if (__label__ == 13) {
  
      var $arraydecay=(($buf)&4294967295); //@line 351 "synth_mbrola.cpp"
      var $43=$p1; //@line 351 "synth_mbrola.cpp"
      var $div46=((((($43))|0)/4096)|0); //@line 351 "synth_mbrola.cpp"
      var $call=_sprintf($arraydecay, ((__str13395)&4294967295), allocate([$div46,0,0,0], ["i32",0,0,0], ALLOC_STACK)); //@line 351 "synth_mbrola.cpp"
      var $arraydecay47=(($buf)&4294967295); //@line 352 "synth_mbrola.cpp"
      var $call48=_strcat(((__ZZL10WritePitchiiiiiE6output)&4294967295), $arraydecay47); //@line 352 "synth_mbrola.cpp"
      ; //@line 353 "synth_mbrola.cpp"
    }
  
    var $44=$env_addr; //@line 356 "synth_mbrola.cpp"
    var $cmp50=((($44))|0) > 1; //@line 356 "synth_mbrola.cpp"
    if ($cmp50) { __label__ = 15;; } else { __label__ = 25;; } //@line 356 "synth_mbrola.cpp"
    $if_then51$$if_end91$23: do { 
      if (__label__ == 15) {
  
        $ix=1; //@line 358 "synth_mbrola.cpp"
        var $arraydecay82=(($buf)&4294967295); //@line 377 "synth_mbrola.cpp"
        var $arraydecay85=(($buf)&4294967295); //@line 378 "synth_mbrola.cpp"
        ; //@line 358 "synth_mbrola.cpp"
        $for_body54$25: while(1) { 
  
          var $45=$ix; //@line 360 "synth_mbrola.cpp"
          var $arrayidx55=(($y+$45*4)&4294967295); //@line 360 "synth_mbrola.cpp"
          var $46=HEAP32[(($arrayidx55)>>2)]; //@line 360 "synth_mbrola.cpp"
          var $47=$pitch_env; //@line 360 "synth_mbrola.cpp"
          var $arrayidx56=(($47+$46)&4294967295); //@line 360 "synth_mbrola.cpp"
          var $48=HEAPU8[($arrayidx56)]; //@line 360 "synth_mbrola.cpp"
          var $conv57=((($48))&255); //@line 360 "synth_mbrola.cpp"
          var $49=HEAP32[(($pitch_range)>>2)]; //@line 360 "synth_mbrola.cpp"
          var $mul58=((($conv57)*($49))&4294967295); //@line 360 "synth_mbrola.cpp"
          var $shr59=($mul58) >> 8; //@line 360 "synth_mbrola.cpp"
          var $50=HEAP32[(($pitch_base)>>2)]; //@line 360 "synth_mbrola.cpp"
          var $add60=((($shr59)+($50))&4294967295); //@line 360 "synth_mbrola.cpp"
          $p2=$add60; //@line 360 "synth_mbrola.cpp"
          var $51=$split_addr; //@line 362 "synth_mbrola.cpp"
          var $cmp61=((($51))|0) > 0; //@line 362 "synth_mbrola.cpp"
          if ($cmp61) { __label__ = 17;; } else { __label__ = 18;; } //@line 362 "synth_mbrola.cpp"
          if (__label__ == 17) {
  
            var $52=$ix; //@line 364 "synth_mbrola.cpp"
            var $arrayidx63=(($y+$52*4)&4294967295); //@line 364 "synth_mbrola.cpp"
            var $53=HEAP32[(($arrayidx63)>>2)]; //@line 364 "synth_mbrola.cpp"
            var $54=$env100; //@line 364 "synth_mbrola.cpp"
            var $mul64=((($54)*($53))&4294967295); //@line 364 "synth_mbrola.cpp"
            var $55=$env_split; //@line 364 "synth_mbrola.cpp"
            var $div65=((((($mul64))|0)/((($55))|0))|0); //@line 364 "synth_mbrola.cpp"
            $y2=$div65; //@line 364 "synth_mbrola.cpp"
            __lastLabel__ = 17; ; //@line 365 "synth_mbrola.cpp"
          }
          else if (__label__ == 18) {
  
            var $56=$split_addr; //@line 367 "synth_mbrola.cpp"
            var $cmp66=((($56))|0) < 0; //@line 367 "synth_mbrola.cpp"
            var $57=$ix; //@line 369 "synth_mbrola.cpp"
            var $arrayidx68=(($y+$57*4)&4294967295); //@line 369 "synth_mbrola.cpp"
            var $58=HEAP32[(($arrayidx68)>>2)]; //@line 369 "synth_mbrola.cpp"
            if ($cmp66) { __label__ = 19;; } else { __label__ = 20;; } //@line 367 "synth_mbrola.cpp"
            if (__label__ == 19) {
  
              var $59=$env_split; //@line 369 "synth_mbrola.cpp"
              var $sub69=((($58)-($59))&4294967295); //@line 369 "synth_mbrola.cpp"
              var $60=$env100; //@line 369 "synth_mbrola.cpp"
              var $mul70=((($sub69)*($60))&4294967295); //@line 369 "synth_mbrola.cpp"
              var $61=$env_split; //@line 369 "synth_mbrola.cpp"
              var $div71=((((($mul70))|0)/((($61))|0))|0); //@line 369 "synth_mbrola.cpp"
              $y2=$div71; //@line 369 "synth_mbrola.cpp"
              __lastLabel__ = 19; ; //@line 370 "synth_mbrola.cpp"
            }
            else if (__label__ == 20) {
  
              var $62=$env100; //@line 373 "synth_mbrola.cpp"
              var $mul74=((($62)*($58))&4294967295); //@line 373 "synth_mbrola.cpp"
              var $div75=((((($mul74))|0)/128)|0); //@line 373 "synth_mbrola.cpp"
              $y2=$div75; //@line 373 "synth_mbrola.cpp"
              __lastLabel__ = 20; ;
            }
          }
  
          var $63=__lastLabel__ == 19 ? $div71 : (__lastLabel__ == 20 ? $div75 : ($div65)); //@line 375 "synth_mbrola.cpp"
          var $cmp78=((($63))|0) > 0; //@line 375 "synth_mbrola.cpp"
          if ($cmp78) { __label__ = 22;; } else { __label__ = 24;; } //@line 375 "synth_mbrola.cpp"
          $land_lhs_true79$$for_inc88$34: do { 
            if (__label__ == 22) {
  
              var $64=$y2; //@line 375 "synth_mbrola.cpp"
              var $65=$env100; //@line 375 "synth_mbrola.cpp"
              var $cmp80=((($64))|0) <= ((($65))|0); //@line 375 "synth_mbrola.cpp"
              if (!($cmp80)) { __label__ = 24;break $land_lhs_true79$$for_inc88$34; } //@line 375 "synth_mbrola.cpp"
  
              var $66=$y2; //@line 377 "synth_mbrola.cpp"
              var $67=$p2; //@line 377 "synth_mbrola.cpp"
              var $div83=((((($67))|0)/4096)|0); //@line 377 "synth_mbrola.cpp"
              var $call84=_sprintf($arraydecay82, ((__str14396)&4294967295), allocate([$66,0,0,0,$div83,0,0,0], ["i32",0,0,0,"i32",0,0,0], ALLOC_STACK)); //@line 377 "synth_mbrola.cpp"
              var $call86=_strcat(((__ZZL10WritePitchiiiiiE6output)&4294967295), $arraydecay85); //@line 378 "synth_mbrola.cpp"
              ; //@line 379 "synth_mbrola.cpp"
            }
          } while(0);
  
          var $68=$ix; //@line 358 "synth_mbrola.cpp"
          var $inc89=((($68)+1)&4294967295); //@line 358 "synth_mbrola.cpp"
          $ix=$inc89; //@line 358 "synth_mbrola.cpp"
          var $cmp53=((($inc89))|0) < 4; //@line 358 "synth_mbrola.cpp"
          if ($cmp53) { __label__ = 16;continue $for_body54$25; } else { __label__ = 25;break $if_then51$$if_end91$23; } //@line 358 "synth_mbrola.cpp"
        }
      }
    } while(0);
  
    var $69=$p_end; //@line 383 "synth_mbrola.cpp"
    var $div92=((((($69))|0)/4096)|0); //@line 383 "synth_mbrola.cpp"
    $p_end=$div92; //@line 383 "synth_mbrola.cpp"
    var $70=$split_addr; //@line 384 "synth_mbrola.cpp"
    var $cmp93=((($70))|0) <= 0; //@line 384 "synth_mbrola.cpp"
    if ($cmp93) { __label__ = 26;; } else { __label__ = 27;; } //@line 384 "synth_mbrola.cpp"
    if (__label__ == 26) {
  
      var $arraydecay95=(($buf)&4294967295); //@line 386 "synth_mbrola.cpp"
      var $71=$env100; //@line 386 "synth_mbrola.cpp"
      var $72=$p_end; //@line 386 "synth_mbrola.cpp"
      var $call96=_sprintf($arraydecay95, ((__str14396)&4294967295), allocate([$71,0,0,0,$72,0,0,0], ["i32",0,0,0,"i32",0,0,0], ALLOC_STACK)); //@line 386 "synth_mbrola.cpp"
      var $arraydecay97=(($buf)&4294967295); //@line 387 "synth_mbrola.cpp"
      var $call98=_strcat(((__ZZL10WritePitchiiiiiE6output)&4294967295), $arraydecay97); //@line 387 "synth_mbrola.cpp"
      ; //@line 388 "synth_mbrola.cpp"
    }
  
    var $73=$env100; //@line 389 "synth_mbrola.cpp"
    var $cmp100=((($73))|0) < 100; //@line 389 "synth_mbrola.cpp"
    if ($cmp100) { __label__ = 28;; } else { __label__ = 29;; } //@line 389 "synth_mbrola.cpp"
    if (__label__ == 28) {
  
      var $arraydecay102=(($buf)&4294967295); //@line 391 "synth_mbrola.cpp"
      var $74=$p_end; //@line 391 "synth_mbrola.cpp"
      var $call103=_sprintf($arraydecay102, ((__str14396)&4294967295), allocate([100,0,0,0,$74,0,0,0], ["i32",0,0,0,"i32",0,0,0], ALLOC_STACK)); //@line 391 "synth_mbrola.cpp"
      var $arraydecay104=(($buf)&4294967295); //@line 392 "synth_mbrola.cpp"
      var $call105=_strcat(((__ZZL10WritePitchiiiiiE6output)&4294967295), $arraydecay104); //@line 392 "synth_mbrola.cpp"
      ; //@line 393 "synth_mbrola.cpp"
    }
  
    var $strlen=_strlen(((__ZZL10WritePitchiiiiiE6output)&4294967295)); //@line 394 "synth_mbrola.cpp"
    var $endptr=((((__ZZL10WritePitchiiiiiE6output)&4294967295)+$strlen)&4294967295); //@line 394 "synth_mbrola.cpp"
    var $src$, $dest$, $stop$, $stop4$, $fast$;
    $src$ = ((__str15397)&4294967295);
    $dest$ = $endptr;
    $stop$ = $src$ + 2;
    if (($dest$%4) == ($src$%4) && 2 > 8) {
      while ($src$%4 !== 0 && $src$ < $stop$) {
        ; HEAP8[$dest$++] = HEAP8[$src$++];
      }
      $src$ >>= 2;
      $dest$ >>= 2;
      $stop4$ = $stop$ >> 2;
      while ($src$ < $stop4$) {
    ;;;    HEAP32[$dest$++] = HEAP32[$src$++];
      }
      $src$ <<= 2;
      $dest$ <<= 2;
    }while ($src$ < $stop$) {
      ; HEAP8[$dest$++] = HEAP8[$src$++];
    }; //@line 394 "synth_mbrola.cpp"
    var $75=$final_addr; //@line 396 "synth_mbrola.cpp"
    var $tobool=((($75))|0)!=0; //@line 396 "synth_mbrola.cpp"
    if ($tobool) { __label__ = 30;; } else { __label__ = 31;; } //@line 396 "synth_mbrola.cpp"
    if (__label__ == 30) {
  
      var $76=$p_end; //@line 397 "synth_mbrola.cpp"
      var $call109=_sprintf(((__ZZL10WritePitchiiiiiE6output)&4294967295), ((__str16398)&4294967295), allocate([$76,0,0,0], ["i32",0,0,0], ALLOC_STACK)); //@line 397 "synth_mbrola.cpp"
      ; //@line 397 "synth_mbrola.cpp"
    }
  
    STACKTOP = __stackBase__;
    return;
    return;
  }
  

  function __Z14MbrolaGenerateP12PHONEME_LISTPii($phoneme_list, $n_ph, $resume) {
    ;
    var __label__;
  
    var $retval;
    var $phoneme_list_addr;
    var $n_ph_addr;
    var $resume_addr;
    var $f_mbrola;
    var $again;
    $phoneme_list_addr=$phoneme_list;
    $n_ph_addr=$n_ph;
    $resume_addr=$resume;
    $f_mbrola=0; //@line 630 "synth_mbrola.cpp"
    var $0=$n_ph_addr; //@line 632 "synth_mbrola.cpp"
    var $1=HEAP32[(($0)>>2)]; //@line 632 "synth_mbrola.cpp"
    var $cmp=((($1))|0)==0; //@line 632 "synth_mbrola.cpp"
    if ($cmp) { __label__ = 1;; } else { __label__ = 2;; } //@line 632 "synth_mbrola.cpp"
    if (__label__ == 1) {
  
      $retval=0; //@line 633 "synth_mbrola.cpp"
      ; //@line 633 "synth_mbrola.cpp"
    }
    else if (__label__ == 2) {
  
      var $2=HEAP32[((_option_mbrola_phonemes)>>2)]; //@line 635 "synth_mbrola.cpp"
      var $tobool=((($2))|0)!=0; //@line 635 "synth_mbrola.cpp"
      if ($tobool) { __label__ = 3;; } else { __label__ = 4;; } //@line 635 "synth_mbrola.cpp"
      if (__label__ == 3) {
  
        var $3=HEAP32[((_f_trans)>>2)]; //@line 638 "synth_mbrola.cpp"
        $f_mbrola=$3; //@line 638 "synth_mbrola.cpp"
        ; //@line 639 "synth_mbrola.cpp"
      }
  
      var $4=$phoneme_list_addr; //@line 641 "synth_mbrola.cpp"
      var $5=$n_ph_addr; //@line 641 "synth_mbrola.cpp"
      var $6=HEAP32[(($5)>>2)]; //@line 641 "synth_mbrola.cpp"
      var $7=$resume_addr; //@line 641 "synth_mbrola.cpp"
      var $8=$f_mbrola; //@line 641 "synth_mbrola.cpp"
      var $call=__Z15MbrolaTranslateP12PHONEME_LISTiiP7__sFILE($4, $6, $7, $8); //@line 641 "synth_mbrola.cpp"
      $again=$call; //@line 641 "synth_mbrola.cpp"
      var $tobool3=((($call))|0)!=0; //@line 642 "synth_mbrola.cpp"
      if ($tobool3) { __label__ = 6;; } else { __label__ = 5;; } //@line 642 "synth_mbrola.cpp"
      if (__label__ == 5) {
  
        var $9=$n_ph_addr; //@line 643 "synth_mbrola.cpp"
        HEAP32[(($9)>>2)]=0; //@line 643 "synth_mbrola.cpp"
        ; //@line 643 "synth_mbrola.cpp"
      }
  
      var $10=$again; //@line 644 "synth_mbrola.cpp"
      $retval=$10; //@line 644 "synth_mbrola.cpp"
      ; //@line 644 "synth_mbrola.cpp"
    }
  
    var $11=$retval; //@line 645 "synth_mbrola.cpp"
    ;
    return $11; //@line 645 "synth_mbrola.cpp"
    return null;
  }
  

  function __Z10MbrolaFillii($length, $resume) {
    ;
    var __label__;
  
    var $retval;
    var $length_addr;
    var $resume_addr;
    var $req_samples;
    var $result;
    $length_addr=$length;
    $resume_addr=$resume;
    var $0=$resume_addr; //@line 655 "synth_mbrola.cpp"
    var $tobool=((($0))|0)!=0; //@line 655 "synth_mbrola.cpp"
    if ($tobool) { __label__ = 2;; } else { __label__ = 1;; } //@line 655 "synth_mbrola.cpp"
    if (__label__ == 1) {
  
      var $1=HEAP32[((_samplerate)>>2)]; //@line 656 "synth_mbrola.cpp"
      var $2=$length_addr; //@line 656 "synth_mbrola.cpp"
      var $mul=((($2)*($1))&4294967295); //@line 656 "synth_mbrola.cpp"
      var $div=((((($mul))|0)/1000)|0); //@line 656 "synth_mbrola.cpp"
      HEAP32[((__ZZ10MbrolaFilliiE9n_samples)>>2)]=$div; //@line 656 "synth_mbrola.cpp"
      ; //@line 656 "synth_mbrola.cpp"
    }
  
    var $3=HEAP32[((_out_end)>>2)]; //@line 658 "synth_mbrola.cpp"
    var $4=HEAP32[((_out_ptr)>>2)]; //@line 658 "synth_mbrola.cpp"
    var $sub_ptr_lhs_cast=($3); //@line 658 "synth_mbrola.cpp"
    var $sub_ptr_rhs_cast=($4); //@line 658 "synth_mbrola.cpp"
    var $sub_ptr_sub=((($sub_ptr_lhs_cast)-($sub_ptr_rhs_cast))&4294967295); //@line 658 "synth_mbrola.cpp"
    var $div1=((((($sub_ptr_sub))|0)/2)|0); //@line 658 "synth_mbrola.cpp"
    $req_samples=$div1; //@line 658 "synth_mbrola.cpp"
    var $5=$req_samples; //@line 659 "synth_mbrola.cpp"
    var $6=HEAP32[((__ZZ10MbrolaFilliiE9n_samples)>>2)]; //@line 659 "synth_mbrola.cpp"
    var $cmp=((($5))|0) > ((($6))|0); //@line 659 "synth_mbrola.cpp"
    if ($cmp) { __label__ = 3;; } else { __label__ = 4;; } //@line 659 "synth_mbrola.cpp"
    if (__label__ == 3) {
  
      var $7=HEAP32[((__ZZ10MbrolaFilliiE9n_samples)>>2)]; //@line 660 "synth_mbrola.cpp"
      $req_samples=$7; //@line 660 "synth_mbrola.cpp"
      ; //@line 660 "synth_mbrola.cpp"
    }
  
    var $8=HEAP32[((_out_ptr)>>2)]; //@line 661 "synth_mbrola.cpp"
    var $9=$8; //@line 661 "synth_mbrola.cpp"
    var $10=$9; //@line 661 "synth_mbrola.cpp"
    var $11=$req_samples; //@line 661 "synth_mbrola.cpp"
    var $call=_read_MBR($10, $11); //@line 661 "synth_mbrola.cpp"
    $result=$call; //@line 661 "synth_mbrola.cpp"
    var $cmp4=((($call))|0) <= 0; //@line 662 "synth_mbrola.cpp"
    if ($cmp4) { __label__ = 5;; } else { __label__ = 6;; } //@line 662 "synth_mbrola.cpp"
    if (__label__ == 5) {
  
      $retval=0; //@line 663 "synth_mbrola.cpp"
      ; //@line 663 "synth_mbrola.cpp"
    }
    else if (__label__ == 6) {
  
      var $12=$result; //@line 664 "synth_mbrola.cpp"
      var $mul7=((($12)*2)&4294967295); //@line 664 "synth_mbrola.cpp"
      var $13=HEAP32[((_out_ptr)>>2)]; //@line 664 "synth_mbrola.cpp"
      var $add_ptr=(($13+$mul7)&4294967295); //@line 664 "synth_mbrola.cpp"
      HEAP32[((_out_ptr)>>2)]=$add_ptr; //@line 664 "synth_mbrola.cpp"
      var $14=$result; //@line 665 "synth_mbrola.cpp"
      var $15=HEAP32[((__ZZ10MbrolaFilliiE9n_samples)>>2)]; //@line 665 "synth_mbrola.cpp"
      var $sub=((($15)-($14))&4294967295); //@line 665 "synth_mbrola.cpp"
      HEAP32[((__ZZ10MbrolaFilliiE9n_samples)>>2)]=$sub; //@line 665 "synth_mbrola.cpp"
      var $16=HEAP32[((__ZZ10MbrolaFilliiE9n_samples)>>2)]; //@line 666 "synth_mbrola.cpp"
      var $tobool8=((($16))|0)!=0; //@line 666 "synth_mbrola.cpp"
      var $cond=($tobool8) ? 1 : 0; //@line 666 "synth_mbrola.cpp"
      $retval=$cond; //@line 666 "synth_mbrola.cpp"
      ; //@line 666 "synth_mbrola.cpp"
    }
  
    var $17=$retval; //@line 667 "synth_mbrola.cpp"
    ;
    return $17; //@line 667 "synth_mbrola.cpp"
    return null;
  }
  

  function __Z11MbrolaResetv() {
    ;
    var __label__;
  
    var $call=_reset_MBR(); //@line 674 "synth_mbrola.cpp"
    ;
    return; //@line 675 "synth_mbrola.cpp"
    return;
  }
  

  function __Z10LoadPhDatav() {
    var __stackBase__  = STACKTOP; STACKTOP += 4; _memset(__stackBase__, 0, 4);
    var __label__;
  
    var $retval;
    var $ix;
    var $n_phonemes;
    var $version;
    var $result;
    var $length=__stackBase__;
    var $p;
    var $pw;
    $result=1; //@line 116 "synthdata.cpp"
    var $0=HEAP32[((__ZL16phoneme_tab_data)>>2)]; //@line 121 "synthdata.cpp"
    var $call=__ZL10ReadPhFilePvPKcPi($0, ((__str1412)&4294967295), 0); //@line 121 "synthdata.cpp"
    HEAP32[((__ZL16phoneme_tab_data)>>2)]=$call; //@line 121 "synthdata.cpp"
    var $cmp=((($call))|0)==0; //@line 121 "synthdata.cpp"
    if ($cmp) { __label__ = 1;; } else { __label__ = 2;; } //@line 121 "synthdata.cpp"
    if (__label__ == 1) {
  
      $retval=-1; //@line 122 "synthdata.cpp"
      ; //@line 122 "synthdata.cpp"
    }
    else if (__label__ == 2) {
  
      var $1=HEAP32[((_phoneme_index)>>2)]; //@line 123 "synthdata.cpp"
      var $2=$1; //@line 123 "synthdata.cpp"
      var $call1=__ZL10ReadPhFilePvPKcPi($2, ((__str2413)&4294967295), 0); //@line 123 "synthdata.cpp"
      var $3=$call1; //@line 123 "synthdata.cpp"
      HEAP32[((_phoneme_index)>>2)]=$3; //@line 123 "synthdata.cpp"
      var $cmp2=((($3))|0)==0; //@line 123 "synthdata.cpp"
      if ($cmp2) { __label__ = 3;; } else { __label__ = 4;; } //@line 123 "synthdata.cpp"
      if (__label__ == 3) {
  
        $retval=-1; //@line 124 "synthdata.cpp"
        ; //@line 124 "synthdata.cpp"
      }
      else if (__label__ == 4) {
  
        var $4=HEAP32[((_phondata_ptr)>>2)]; //@line 125 "synthdata.cpp"
        var $call5=__ZL10ReadPhFilePvPKcPi($4, ((__str3414)&4294967295), 0); //@line 125 "synthdata.cpp"
        HEAP32[((_phondata_ptr)>>2)]=$call5; //@line 125 "synthdata.cpp"
        var $cmp6=((($call5))|0)==0; //@line 125 "synthdata.cpp"
        if ($cmp6) { __label__ = 5;; } else { __label__ = 6;; } //@line 125 "synthdata.cpp"
        if (__label__ == 5) {
  
          $retval=-1; //@line 126 "synthdata.cpp"
          ; //@line 126 "synthdata.cpp"
        }
        else if (__label__ == 6) {
  
          var $5=HEAP32[((_tunes)>>2)]; //@line 127 "synthdata.cpp"
          var $6=$5; //@line 127 "synthdata.cpp"
          var $call9=__ZL10ReadPhFilePvPKcPi($6, ((__str4415)&4294967295), $length); //@line 127 "synthdata.cpp"
          var $7=$call9; //@line 127 "synthdata.cpp"
          HEAP32[((_tunes)>>2)]=$7; //@line 127 "synthdata.cpp"
          var $cmp10=((($7))|0)==0; //@line 127 "synthdata.cpp"
          if ($cmp10) { __label__ = 7;; } else { __label__ = 8;; } //@line 127 "synthdata.cpp"
          if (__label__ == 7) {
  
            $retval=-1; //@line 128 "synthdata.cpp"
            ; //@line 128 "synthdata.cpp"
          }
          else if (__label__ == 8) {
  
            var $8=HEAP32[((_phondata_ptr)>>2)]; //@line 129 "synthdata.cpp"
            HEAP32[((_wavefile_data)>>2)]=$8; //@line 129 "synthdata.cpp"
            var $9=HEAPU32[(($length)>>2)]; //@line 130 "synthdata.cpp"
            var $div=Math.floor(((($9))>>>0)/68); //@line 130 "synthdata.cpp"
            HEAP32[((_n_tunes)>>2)]=$div; //@line 130 "synthdata.cpp"
            $version=0; //@line 133 "synthdata.cpp"
            $ix=0; //@line 134 "synthdata.cpp"
            ; //@line 134 "synthdata.cpp"
            $for_body$14: while(1) { 
  
              var $10=$ix; //@line 136 "synthdata.cpp"
              var $11=HEAP32[((_wavefile_data)>>2)]; //@line 136 "synthdata.cpp"
              var $arrayidx=(($11+$10)&4294967295); //@line 136 "synthdata.cpp"
              var $12=HEAPU8[($arrayidx)]; //@line 136 "synthdata.cpp"
              var $conv=((($12))&255); //@line 136 "synthdata.cpp"
              var $13=$ix; //@line 136 "synthdata.cpp"
              var $mul=((($13)*8)&4294967295); //@line 136 "synthdata.cpp"
              var $shl=($conv) << ($mul); //@line 136 "synthdata.cpp"
              var $14=$version; //@line 136 "synthdata.cpp"
              var $add=((($shl)+($14))&4294967295); //@line 136 "synthdata.cpp"
              $version=$add; //@line 136 "synthdata.cpp"
              var $15=$ix; //@line 134 "synthdata.cpp"
              var $inc=((($15)+1)&4294967295); //@line 134 "synthdata.cpp"
              $ix=$inc; //@line 134 "synthdata.cpp"
              var $cmp13=((($inc))|0) < 4; //@line 134 "synthdata.cpp"
              if ($cmp13) { __label__ = 9;continue $for_body$14; } else { __label__ = 10;break $for_body$14; } //@line 134 "synthdata.cpp"
            }
  
            var $16=$version; //@line 139 "synthdata.cpp"
            var $cmp14=((($16))|0)!=83200; //@line 139 "synthdata.cpp"
            if ($cmp14) { __label__ = 11;; } else { __label__ = 12;; } //@line 139 "synthdata.cpp"
            if (__label__ == 11) {
  
              var $17=$version; //@line 141 "synthdata.cpp"
              $result=$17; //@line 141 "synthdata.cpp"
              ; //@line 142 "synthdata.cpp"
            }
  
            var $18=HEAP32[((__ZL16phoneme_tab_data)>>2)]; //@line 145 "synthdata.cpp"
            $p=$18; //@line 145 "synthdata.cpp"
            var $19=$p; //@line 146 "synthdata.cpp"
            var $arrayidx17=(($19)&4294967295); //@line 146 "synthdata.cpp"
            var $20=HEAPU8[($arrayidx17)]; //@line 146 "synthdata.cpp"
            var $conv18=((($20))&255); //@line 146 "synthdata.cpp"
            HEAP32[((_n_phoneme_tables)>>2)]=$conv18; //@line 146 "synthdata.cpp"
            var $21=$p; //@line 147 "synthdata.cpp"
            var $add_ptr=(($21+4)&4294967295); //@line 147 "synthdata.cpp"
            $p=$add_ptr; //@line 147 "synthdata.cpp"
            $ix=0; //@line 149 "synthdata.cpp"
            var $22=$ix; //@line 149 "synthdata.cpp"
            var $23=HEAP32[((_n_phoneme_tables)>>2)]; //@line 149 "synthdata.cpp"
            var $cmp201=((($22))|0) < ((($23))|0); //@line 149 "synthdata.cpp"
            if ($cmp201) { __label__ = 13;; } else { __label__ = 14;; } //@line 149 "synthdata.cpp"
            $for_body21$$for_end42$20: do { 
              if (__label__ == 13) {
                while(1) { 
  
                  var $24=$p; //@line 151 "synthdata.cpp"
                  var $arrayidx22=(($24)&4294967295); //@line 151 "synthdata.cpp"
                  var $25=HEAPU8[($arrayidx22)]; //@line 151 "synthdata.cpp"
                  var $conv23=((($25))&255); //@line 151 "synthdata.cpp"
                  $n_phonemes=$conv23; //@line 151 "synthdata.cpp"
                  var $26=$p; //@line 152 "synthdata.cpp"
                  var $arrayidx24=(($26)&4294967295); //@line 152 "synthdata.cpp"
                  var $27=HEAPU8[($arrayidx24)]; //@line 152 "synthdata.cpp"
                  var $conv25=((($27))&255); //@line 152 "synthdata.cpp"
                  var $28=$ix; //@line 152 "synthdata.cpp"
                  var $arrayidx26=((_phoneme_tab_list+$28*48)&4294967295); //@line 152 "synthdata.cpp"
                  var $n_phonemes27=(($arrayidx26+36)&4294967295); //@line 152 "synthdata.cpp"
                  HEAP32[(($n_phonemes27)>>2)]=$conv25; //@line 152 "synthdata.cpp"
                  var $29=$p; //@line 153 "synthdata.cpp"
                  var $arrayidx28=(($29+1)&4294967295); //@line 153 "synthdata.cpp"
                  var $30=HEAPU8[($arrayidx28)]; //@line 153 "synthdata.cpp"
                  var $conv29=((($30))&255); //@line 153 "synthdata.cpp"
                  var $31=$ix; //@line 153 "synthdata.cpp"
                  var $arrayidx30=((_phoneme_tab_list+$31*48)&4294967295); //@line 153 "synthdata.cpp"
                  var $includes=(($arrayidx30+40)&4294967295); //@line 153 "synthdata.cpp"
                  HEAP32[(($includes)>>2)]=$conv29; //@line 153 "synthdata.cpp"
                  var $32=$p; //@line 154 "synthdata.cpp"
                  var $33=$32; //@line 154 "synthdata.cpp"
                  $pw=$33; //@line 154 "synthdata.cpp"
                  var $34=$pw; //@line 155 "synthdata.cpp"
                  var $arrayidx31=(($34+4)&4294967295); //@line 155 "synthdata.cpp"
                  var $35=HEAP32[(($arrayidx31)>>2)]; //@line 155 "synthdata.cpp"
                  var $call32=__Z13Reverse4Bytesi($35); //@line 155 "synthdata.cpp"
                  var $36=$ix; //@line 155 "synthdata.cpp"
                  var $arrayidx33=((_phoneme_tab_list+$36*48)&4294967295); //@line 155 "synthdata.cpp"
                  var $equivalence_tables=(($arrayidx33+44)&4294967295); //@line 155 "synthdata.cpp"
                  HEAP32[(($equivalence_tables)>>2)]=$call32; //@line 155 "synthdata.cpp"
                  var $37=$p; //@line 156 "synthdata.cpp"
                  var $add_ptr34=(($37+8)&4294967295); //@line 156 "synthdata.cpp"
                  $p=$add_ptr34; //@line 156 "synthdata.cpp"
                  var $38=$ix; //@line 157 "synthdata.cpp"
                  var $arrayidx35=((_phoneme_tab_list+$38*48)&4294967295); //@line 157 "synthdata.cpp"
                  var $name=(($arrayidx35)&4294967295); //@line 157 "synthdata.cpp"
                  var $arraydecay=(($name)&4294967295); //@line 157 "synthdata.cpp"
                  var $39=$p; //@line 157 "synthdata.cpp"
                  var $src$, $dest$, $stop$, $stop4$, $fast$;
                  $src$ = $39;
                  $dest$ = $arraydecay;
                  $stop$ = $src$ + 32;
                  if (($dest$%4) == ($src$%4) && 32 > 8) {
                    while ($src$%4 !== 0 && $src$ < $stop$) {
                      ; HEAP8[$dest$++] = HEAP8[$src$++];
                    }
                    $src$ >>= 2;
                    $dest$ >>= 2;
                    $stop4$ = $stop$ >> 2;
                    while ($src$ < $stop4$) {
                  ;;;    HEAP32[$dest$++] = HEAP32[$src$++];
                    }
                    $src$ <<= 2;
                    $dest$ <<= 2;
                  }while ($src$ < $stop$) {
                    ; HEAP8[$dest$++] = HEAP8[$src$++];
                  }; //@line 157 "synthdata.cpp"
                  var $40=$p; //@line 158 "synthdata.cpp"
                  var $add_ptr36=(($40+32)&4294967295); //@line 158 "synthdata.cpp"
                  $p=$add_ptr36; //@line 158 "synthdata.cpp"
                  var $41=$p; //@line 159 "synthdata.cpp"
                  var $42=$41; //@line 159 "synthdata.cpp"
                  var $43=$ix; //@line 159 "synthdata.cpp"
                  var $arrayidx37=((_phoneme_tab_list+$43*48)&4294967295); //@line 159 "synthdata.cpp"
                  var $phoneme_tab_ptr=(($arrayidx37+32)&4294967295); //@line 159 "synthdata.cpp"
                  HEAP32[(($phoneme_tab_ptr)>>2)]=$42; //@line 159 "synthdata.cpp"
                  var $44=$n_phonemes; //@line 160 "synthdata.cpp"
                  var $mul38=((($44)*16)&4294967295); //@line 160 "synthdata.cpp"
                  var $45=$p; //@line 160 "synthdata.cpp"
                  var $add_ptr39=(($45+$mul38)&4294967295); //@line 160 "synthdata.cpp"
                  $p=$add_ptr39; //@line 160 "synthdata.cpp"
                  var $46=$ix; //@line 149 "synthdata.cpp"
                  var $inc41=((($46)+1)&4294967295); //@line 149 "synthdata.cpp"
                  $ix=$inc41; //@line 149 "synthdata.cpp"
                  var $47=$ix; //@line 149 "synthdata.cpp"
                  var $48=HEAP32[((_n_phoneme_tables)>>2)]; //@line 149 "synthdata.cpp"
                  var $cmp20=((($47))|0) < ((($48))|0); //@line 149 "synthdata.cpp"
                  if ($cmp20) { __label__ = 13;continue ; } else { __label__ = 14;break $for_body21$$for_end42$20; } //@line 149 "synthdata.cpp"
                }
              }
            } while(0);
  
            var $49=HEAP32[((_phoneme_tab_number)>>2)]; //@line 163 "synthdata.cpp"
            var $50=HEAP32[((_n_phoneme_tables)>>2)]; //@line 163 "synthdata.cpp"
            var $cmp43=((($49))|0) >= ((($50))|0); //@line 163 "synthdata.cpp"
            if ($cmp43) { __label__ = 15;; } else { __label__ = 16;; } //@line 163 "synthdata.cpp"
            if (__label__ == 15) {
  
              HEAP32[((_phoneme_tab_number)>>2)]=0; //@line 164 "synthdata.cpp"
              ; //@line 164 "synthdata.cpp"
            }
  
            var $51=$result; //@line 166 "synthdata.cpp"
            $retval=$51; //@line 166 "synthdata.cpp"
            ; //@line 166 "synthdata.cpp"
          }
        }
      }
    }
  
    var $52=$retval; //@line 167 "synthdata.cpp"
    STACKTOP = __stackBase__;
    return $52; //@line 167 "synthdata.cpp"
    return null;
  }
  

  function __ZL10ReadPhFilePvPKcPi($ptr, $fname, $size) {
    var __stackBase__  = STACKTOP; STACKTOP += 192; _memset(__stackBase__, 0, 192);
    var __label__;
  
    var $retval;
    var $ptr_addr;
    var $fname_addr;
    var $size_addr;
    var $f_in;
    var $p;
    var $length;
    var $buf=__stackBase__;
    $ptr_addr=$ptr;
    $fname_addr=$fname;
    $size_addr=$size;
    var $arraydecay=(($buf)&4294967295); //@line 81 "synthdata.cpp"
    var $0=$fname_addr; //@line 81 "synthdata.cpp"
    var $call=_sprintf($arraydecay, ((__str74726)&4294967295), allocate([((_path_home)&4294967295),0,0,0,47,0,0,0,$0,0,0,0], ["i8*",0,0,0,"i32",0,0,0,"i8*",0,0,0], ALLOC_STACK)); //@line 81 "synthdata.cpp"
    var $arraydecay1=(($buf)&4294967295); //@line 82 "synthdata.cpp"
    var $call2=__Z13GetFileLengthPKc($arraydecay1); //@line 82 "synthdata.cpp"
    $length=$call2; //@line 82 "synthdata.cpp"
    var $arraydecay3=(($buf)&4294967295); //@line 84 "synthdata.cpp"
    var $call4=_fopen($arraydecay3, ((__str18429)&4294967295)); //@line 84 "synthdata.cpp"
    $f_in=$call4; //@line 84 "synthdata.cpp"
    var $cmp=((($call4))|0)==0; //@line 84 "synthdata.cpp"
    if ($cmp) { __label__ = 1;; } else { __label__ = 2;; } //@line 84 "synthdata.cpp"
    if (__label__ == 1) {
  
      var $1=HEAP32[((__impure_ptr)>>2)]; //@line 86 "synthdata.cpp"
      var $_stderr=(($1+12)&4294967295); //@line 86 "synthdata.cpp"
      var $2=HEAP32[(($_stderr)>>2)]; //@line 86 "synthdata.cpp"
      var $arraydecay5=(($buf)&4294967295); //@line 86 "synthdata.cpp"
      var $call6=_fprintf($2, ((__str19430)&4294967295), allocate([$arraydecay5,0,0,0], ["i8*",0,0,0], ALLOC_STACK)); //@line 86 "synthdata.cpp"
      $retval=0; //@line 87 "synthdata.cpp"
      ; //@line 87 "synthdata.cpp"
    }
    else if (__label__ == 2) {
  
      var $3=$ptr_addr; //@line 90 "synthdata.cpp"
      var $cmp7=((($3))|0)!=0; //@line 90 "synthdata.cpp"
      if ($cmp7) { __label__ = 3;; } else { __label__ = 4;; } //@line 90 "synthdata.cpp"
      if (__label__ == 3) {
  
        var $4=$ptr_addr; //@line 91 "synthdata.cpp"
        __Z4FreePv($4); //@line 91 "synthdata.cpp"
        ; //@line 91 "synthdata.cpp"
      }
  
      var $5=$length; //@line 93 "synthdata.cpp"
      var $call10=__Z5Alloci($5); //@line 93 "synthdata.cpp"
      $p=$call10; //@line 93 "synthdata.cpp"
      var $cmp11=((($call10))|0)==0; //@line 93 "synthdata.cpp"
      if ($cmp11) { __label__ = 5;; } else { __label__ = 6;; } //@line 93 "synthdata.cpp"
      if (__label__ == 5) {
  
        var $6=$f_in; //@line 95 "synthdata.cpp"
        var $call13=_fclose($6); //@line 95 "synthdata.cpp"
        $retval=0; //@line 96 "synthdata.cpp"
        ; //@line 96 "synthdata.cpp"
      }
      else if (__label__ == 6) {
  
        var $7=$p; //@line 98 "synthdata.cpp"
        var $8=$length; //@line 98 "synthdata.cpp"
        var $9=$f_in; //@line 98 "synthdata.cpp"
        var $call15=_fread($7, 1, $8, $9); //@line 98 "synthdata.cpp"
        var $10=$length; //@line 98 "synthdata.cpp"
        var $cmp16=((($call15))|0)!=((($10))|0); //@line 98 "synthdata.cpp"
        var $11=$f_in; //@line 100 "synthdata.cpp"
        var $call18=_fclose($11); //@line 100 "synthdata.cpp"
        if ($cmp16) { __label__ = 7;; } else { __label__ = 8;; } //@line 98 "synthdata.cpp"
        if (__label__ == 7) {
  
          $retval=0; //@line 101 "synthdata.cpp"
          ; //@line 101 "synthdata.cpp"
        }
        else if (__label__ == 8) {
  
          var $12=$size_addr; //@line 105 "synthdata.cpp"
          var $cmp21=((($12))|0)!=0; //@line 105 "synthdata.cpp"
          if ($cmp21) { __label__ = 9;; } else { __label__ = 10;; } //@line 105 "synthdata.cpp"
          if (__label__ == 9) {
  
            var $13=$length; //@line 106 "synthdata.cpp"
            var $14=$size_addr; //@line 106 "synthdata.cpp"
            HEAP32[(($14)>>2)]=$13; //@line 106 "synthdata.cpp"
            ; //@line 106 "synthdata.cpp"
          }
  
          var $15=$p; //@line 107 "synthdata.cpp"
          $retval=$15; //@line 107 "synthdata.cpp"
          ; //@line 107 "synthdata.cpp"
        }
      }
    }
  
    var $16=$retval; //@line 108 "synthdata.cpp"
    STACKTOP = __stackBase__;
    return $16; //@line 108 "synthdata.cpp"
    return null;
  }
  

  function __Z10FreePhDatav() {
    ;
    var __label__;
  
    var $0=HEAP32[((__ZL16phoneme_tab_data)>>2)]; //@line 172 "synthdata.cpp"
    __Z4FreePv($0); //@line 172 "synthdata.cpp"
    var $1=HEAP32[((_phoneme_index)>>2)]; //@line 173 "synthdata.cpp"
    var $2=$1; //@line 173 "synthdata.cpp"
    __Z4FreePv($2); //@line 173 "synthdata.cpp"
    var $3=HEAP32[((_phondata_ptr)>>2)]; //@line 174 "synthdata.cpp"
    __Z4FreePv($3); //@line 174 "synthdata.cpp"
    HEAP32[((__ZL16phoneme_tab_data)>>2)]=0; //@line 175 "synthdata.cpp"
    HEAP32[((_phoneme_index)>>2)]=0; //@line 176 "synthdata.cpp"
    HEAP32[((_phondata_ptr)>>2)]=0; //@line 177 "synthdata.cpp"
    ;
    return; //@line 178 "synthdata.cpp"
    return;
  }
  

  function __Z11PhonemeCodej($mnem) {
    ;
    var __label__;
  
    var $retval;
    var $mnem_addr;
    var $ix;
    $mnem_addr=$mnem;
    $ix=0; //@line 185 "synthdata.cpp"
    ; //@line 185 "synthdata.cpp"
    while(1) { 
  
      var $0=$ix; //@line 185 "synthdata.cpp"
      var $1=HEAP32[((_n_phoneme_tab)>>2)]; //@line 185 "synthdata.cpp"
      var $cmp=((($0))|0) < ((($1))|0); //@line 185 "synthdata.cpp"
      if (!($cmp)) { __label__ = 6;break ; } //@line 185 "synthdata.cpp"
  
      var $2=$ix; //@line 187 "synthdata.cpp"
      var $arrayidx=((_phoneme_tab+$2*4)&4294967295); //@line 187 "synthdata.cpp"
      var $3=HEAP32[(($arrayidx)>>2)]; //@line 187 "synthdata.cpp"
      var $cmp1=((($3))|0)==0; //@line 187 "synthdata.cpp"
      if ($cmp1) { __label__ = 5;; } else { __label__ = 3;; } //@line 187 "synthdata.cpp"
      if (__label__ == 3) {
  
        var $4=$ix; //@line 189 "synthdata.cpp"
        var $arrayidx2=((_phoneme_tab+$4*4)&4294967295); //@line 189 "synthdata.cpp"
        var $5=HEAP32[(($arrayidx2)>>2)]; //@line 189 "synthdata.cpp"
        var $mnemonic=(($5)&4294967295); //@line 189 "synthdata.cpp"
        var $6=HEAP32[(($mnemonic)>>2)]; //@line 189 "synthdata.cpp"
        var $7=$mnem_addr; //@line 189 "synthdata.cpp"
        var $cmp3=((($6))|0)==((($7))|0); //@line 189 "synthdata.cpp"
        if ($cmp3) { __label__ = 4;break ; } //@line 189 "synthdata.cpp"
      }
  
      var $11=$ix; //@line 185 "synthdata.cpp"
      var $inc=((($11)+1)&4294967295); //@line 185 "synthdata.cpp"
      $ix=$inc; //@line 185 "synthdata.cpp"
      __label__ = 1;continue ; //@line 185 "synthdata.cpp"
    }
    if (__label__ == 6) {
  
      $retval=0; //@line 192 "synthdata.cpp"
      ; //@line 192 "synthdata.cpp"
    }
    else if (__label__ == 4) {
  
      var $8=$ix; //@line 190 "synthdata.cpp"
      var $arrayidx5=((_phoneme_tab+$8*4)&4294967295); //@line 190 "synthdata.cpp"
      var $9=HEAP32[(($arrayidx5)>>2)]; //@line 190 "synthdata.cpp"
      var $code=(($9+10)&4294967295); //@line 190 "synthdata.cpp"
      var $10=HEAPU8[($code)]; //@line 190 "synthdata.cpp"
      var $conv=((($10))&255); //@line 190 "synthdata.cpp"
      $retval=$conv; //@line 190 "synthdata.cpp"
      ; //@line 190 "synthdata.cpp"
    }
  
    var $12=$retval; //@line 193 "synthdata.cpp"
    ;
    return $12; //@line 193 "synthdata.cpp"
    return null;
  }
  

  function __Z19LookupPhonemeStringPKc($string) {
    ;
    var __label__;
    var __lastLabel__ = null;
  
    var $string_addr;
    var $ix;
    var $c;
    var $mnem;
    $string_addr=$string;
    $mnem=0; //@line 203 "synthdata.cpp"
    $ix=0; //@line 204 "synthdata.cpp"
    __lastLabel__ = 0; ; //@line 204 "synthdata.cpp"
    while(1) { 
  
      var $0=__lastLabel__ == 3 ? $inc : (0); //@line 204 "synthdata.cpp"
      var $cmp=((($0))|0) < 4; //@line 204 "synthdata.cpp"
      if (!($cmp)) { __label__ = 4;break ; } //@line 204 "synthdata.cpp"
  
      var $1=$ix; //@line 206 "synthdata.cpp"
      var $2=$string_addr; //@line 206 "synthdata.cpp"
      var $arrayidx=(($2+$1)&4294967295); //@line 206 "synthdata.cpp"
      var $3=HEAP8[($arrayidx)]; //@line 206 "synthdata.cpp"
      var $conv=(tempInt=(($3)),(tempInt>=128?tempInt-256:tempInt)); //@line 206 "synthdata.cpp"
      var $cmp1=((($conv))|0)==0; //@line 206 "synthdata.cpp"
      if ($cmp1) { __label__ = 4;break ; } //@line 206 "synthdata.cpp"
  
      var $4=$ix; //@line 207 "synthdata.cpp"
      var $5=$string_addr; //@line 207 "synthdata.cpp"
      var $arrayidx2=(($5+$4)&4294967295); //@line 207 "synthdata.cpp"
      var $6=HEAP8[($arrayidx2)]; //@line 207 "synthdata.cpp"
      $c=$6; //@line 207 "synthdata.cpp"
      var $7=$c; //@line 208 "synthdata.cpp"
      var $conv3=((($7))&255); //@line 208 "synthdata.cpp"
      var $8=$ix; //@line 208 "synthdata.cpp"
      var $mul=((($8)*8)&4294967295); //@line 208 "synthdata.cpp"
      var $shl=($conv3) << ($mul); //@line 208 "synthdata.cpp"
      var $9=$mnem; //@line 208 "synthdata.cpp"
      var $or=($shl) | ($9); //@line 208 "synthdata.cpp"
      $mnem=$or; //@line 208 "synthdata.cpp"
      var $10=$ix; //@line 204 "synthdata.cpp"
      var $inc=((($10)+1)&4294967295); //@line 204 "synthdata.cpp"
      $ix=$inc; //@line 204 "synthdata.cpp"
      __lastLabel__ = 3; __label__ = 1;continue ; //@line 204 "synthdata.cpp"
    }
  
    var $11=$mnem; //@line 211 "synthdata.cpp"
    var $call=__Z11PhonemeCodej($11); //@line 211 "synthdata.cpp"
    ;
    return $call; //@line 211 "synthdata.cpp"
    return null;
  }
  

  function __Z11LookupSpectP11PHONEME_TABiP10FMT_PARAMSPiP12PHONEME_LIST($this_ph, $which, $fmt_params, $n_frames, $plist) {
    var __stackBase__  = STACKTOP; STACKTOP += 4; _memset(__stackBase__, 0, 4);
    var __label__;
    var __lastLabel__ = null;
  
    var $this_ph_addr;
    var $which_addr;
    var $fmt_params_addr;
    var $n_frames_addr;
    var $plist_addr;
    var $ix;
    var $nf=__stackBase__;
    var $nf1;
    var $seq_break;
    var $frames;
    var $length1;
    var $length_std;
    var $length_factor;
    var $seq;
    var $seq2;
    var $seqk;
    var $seqk2;
    var $frame;
    $this_ph_addr=$this_ph;
    $which_addr=$which;
    $fmt_params_addr=$fmt_params;
    $n_frames_addr=$n_frames;
    $plist_addr=$plist;
    var $0=$fmt_params_addr; //@line 232 "synthdata.cpp"
    var $fmt_addr=(($0+8)&4294967295); //@line 232 "synthdata.cpp"
    var $1=HEAP32[(($fmt_addr)>>2)]; //@line 232 "synthdata.cpp"
    var $2=HEAP32[((_phondata_ptr)>>2)]; //@line 232 "synthdata.cpp"
    var $arrayidx=(($2+$1)&4294967295); //@line 232 "synthdata.cpp"
    var $3=$arrayidx; //@line 232 "synthdata.cpp"
    $seq=$3; //@line 232 "synthdata.cpp"
    var $4=$seq; //@line 233 "synthdata.cpp"
    var $5=$4; //@line 233 "synthdata.cpp"
    $seqk=$5; //@line 233 "synthdata.cpp"
    var $6=$seq; //@line 234 "synthdata.cpp"
    var $n_frames1=(($6+2)&4294967295); //@line 234 "synthdata.cpp"
    var $7=HEAPU8[($n_frames1)]; //@line 234 "synthdata.cpp"
    var $conv=((($7))&255); //@line 234 "synthdata.cpp"
    HEAP32[(($nf)>>2)]=$conv; //@line 234 "synthdata.cpp"
    var $8=HEAP32[(($nf)>>2)]; //@line 237 "synthdata.cpp"
    var $cmp=((($8))|0) >= 25; //@line 237 "synthdata.cpp"
    if ($cmp) { __label__ = 1;; } else { __label__ = 2;; } //@line 237 "synthdata.cpp"
    if (__label__ == 1) {
  
      HEAP32[(($nf)>>2)]=24; //@line 238 "synthdata.cpp"
      ; //@line 238 "synthdata.cpp"
    }
  
    var $9=$fmt_params_addr; //@line 240 "synthdata.cpp"
    var $fmt2_lenadj=(($9+24)&4294967295); //@line 240 "synthdata.cpp"
    var $10=HEAP32[(($fmt2_lenadj)>>2)]; //@line 240 "synthdata.cpp"
    var $11=$fmt_params_addr; //@line 240 "synthdata.cpp"
    var $fmt_length=(($11+12)&4294967295); //@line 240 "synthdata.cpp"
    var $12=HEAP32[(($fmt_length)>>2)]; //@line 240 "synthdata.cpp"
    var $add=((($12)+($10))&4294967295); //@line 240 "synthdata.cpp"
    HEAP32[((_seq_len_adjust)>>2)]=$add; //@line 240 "synthdata.cpp"
    $seq_break=0; //@line 241 "synthdata.cpp"
    $ix=0; //@line 243 "synthdata.cpp"
    var $13=$ix; //@line 243 "synthdata.cpp"
    var $14=HEAP32[(($nf)>>2)]; //@line 243 "synthdata.cpp"
    var $cmp29=((($13))|0) < ((($14))|0); //@line 243 "synthdata.cpp"
    if ($cmp29) { __label__ = 4;; } else { __label__ = 3;; } //@line 243 "synthdata.cpp"
    $for_body$$for_end_thread$5: do { 
      if (__label__ == 4) {
        while(1) { 
  
          var $15=$seq; //@line 245 "synthdata.cpp"
          var $frame3=(($15+4)&4294967295); //@line 245 "synthdata.cpp"
          var $arrayidx4=(($frame3)&4294967295); //@line 245 "synthdata.cpp"
          var $frflags=(($arrayidx4)&4294967295); //@line 245 "synthdata.cpp"
          var $16=HEAP16[(($frflags)>>1)]; //@line 245 "synthdata.cpp"
          var $conv5=(tempInt=(($16)),(tempInt>=32768?tempInt-65536:tempInt)); //@line 245 "synthdata.cpp"
          var $and=($conv5) & 1; //@line 245 "synthdata.cpp"
          var $tobool=((($and))|0)!=0; //@line 245 "synthdata.cpp"
          var $17=$ix; //@line 246 "synthdata.cpp"
          if ($tobool) { __label__ = 5;; } else { __label__ = 6;; } //@line 245 "synthdata.cpp"
          if (__label__ == 5) {
  
            var $18=$seqk; //@line 246 "synthdata.cpp"
            var $frame7=(($18+4)&4294967295); //@line 246 "synthdata.cpp"
            var $arrayidx8=(($frame7+$17*64)&4294967295); //@line 246 "synthdata.cpp"
            $frame=$arrayidx8; //@line 246 "synthdata.cpp"
            ; //@line 246 "synthdata.cpp"
          }
          else if (__label__ == 6) {
  
            var $19=$seq; //@line 248 "synthdata.cpp"
            var $frame9=(($19+4)&4294967295); //@line 248 "synthdata.cpp"
            var $arrayidx10=(($frame9+$17*44)&4294967295); //@line 248 "synthdata.cpp"
            var $20=$arrayidx10; //@line 248 "synthdata.cpp"
            $frame=$20; //@line 248 "synthdata.cpp"
            ;
          }
  
          var $21=$frame; //@line 249 "synthdata.cpp"
          var $22=$ix; //@line 249 "synthdata.cpp"
          var $arrayidx12=((__ZZ11LookupSpectP11PHONEME_TABiP10FMT_PARAMSPiP12PHONEME_LISTE10frames_buf+$22*8)&4294967295); //@line 249 "synthdata.cpp"
          var $frame13=(($arrayidx12+4)&4294967295); //@line 249 "synthdata.cpp"
          HEAP32[(($frame13)>>2)]=$21; //@line 249 "synthdata.cpp"
          var $23=$frame; //@line 250 "synthdata.cpp"
          var $frflags14=(($23)&4294967295); //@line 250 "synthdata.cpp"
          var $24=HEAP16[(($frflags14)>>1)]; //@line 250 "synthdata.cpp"
          var $25=$ix; //@line 250 "synthdata.cpp"
          var $arrayidx15=((__ZZ11LookupSpectP11PHONEME_TABiP10FMT_PARAMSPiP12PHONEME_LISTE10frames_buf+$25*8)&4294967295); //@line 250 "synthdata.cpp"
          var $frflags16=(($arrayidx15+2)&4294967295); //@line 250 "synthdata.cpp"
          HEAP16[(($frflags16)>>1)]=$24; //@line 250 "synthdata.cpp"
          var $26=$frame; //@line 251 "synthdata.cpp"
          var $length=(($26+16)&4294967295); //@line 251 "synthdata.cpp"
          var $27=HEAPU8[($length)]; //@line 251 "synthdata.cpp"
          var $conv17=((($27))&255); //@line 251 "synthdata.cpp"
          var $28=$ix; //@line 251 "synthdata.cpp"
          var $arrayidx18=((__ZZ11LookupSpectP11PHONEME_TABiP10FMT_PARAMSPiP12PHONEME_LISTE10frames_buf+$28*8)&4294967295); //@line 251 "synthdata.cpp"
          var $length19=(($arrayidx18)&4294967295); //@line 251 "synthdata.cpp"
          HEAP16[(($length19)>>1)]=$conv17; //@line 251 "synthdata.cpp"
          var $29=$frame; //@line 252 "synthdata.cpp"
          var $frflags20=(($29)&4294967295); //@line 252 "synthdata.cpp"
          var $30=HEAP16[(($frflags20)>>1)]; //@line 252 "synthdata.cpp"
          var $conv21=(tempInt=(($30)),(tempInt>=32768?tempInt-65536:tempInt)); //@line 252 "synthdata.cpp"
          var $and22=($conv21) & 2; //@line 252 "synthdata.cpp"
          var $tobool23=((($and22))|0)!=0; //@line 252 "synthdata.cpp"
          if ($tobool23) { __label__ = 8;; } else { __label__ = 9;; } //@line 252 "synthdata.cpp"
          if (__label__ == 8) {
  
            var $31=$ix; //@line 253 "synthdata.cpp"
            $seq_break=$31; //@line 253 "synthdata.cpp"
            ; //@line 253 "synthdata.cpp"
          }
  
          var $32=$ix; //@line 243 "synthdata.cpp"
          var $inc=((($32)+1)&4294967295); //@line 243 "synthdata.cpp"
          $ix=$inc; //@line 243 "synthdata.cpp"
          var $33=$ix; //@line 243 "synthdata.cpp"
          var $34=HEAP32[(($nf)>>2)]; //@line 243 "synthdata.cpp"
          var $cmp2=((($33))|0) < ((($34))|0); //@line 243 "synthdata.cpp"
          if ($cmp2) { __label__ = 4;continue ; } else { __label__ = 10;break ; } //@line 243 "synthdata.cpp"
        }
  
        var $_pr10=$seq_break; //@line 257 "synthdata.cpp"
        $frames=((__ZZ11LookupSpectP11PHONEME_TABiP10FMT_PARAMSPiP12PHONEME_LISTE10frames_buf)&4294967295); //@line 256 "synthdata.cpp"
        var $cmp26=((($_pr10))|0) > 0; //@line 257 "synthdata.cpp"
        if (!($cmp26)) { __label__ = 14;break $for_body$$for_end_thread$5; } //@line 257 "synthdata.cpp"
  
        var $35=$which_addr; //@line 259 "synthdata.cpp"
        var $cmp28=((($35))|0)==1; //@line 259 "synthdata.cpp"
        var $36=$seq_break; //@line 261 "synthdata.cpp"
        if ($cmp28) { __label__ = 12;; } else { __label__ = 13;; } //@line 259 "synthdata.cpp"
        if (__label__ == 12) {
  
          var $add30=((($36)+1)&4294967295); //@line 261 "synthdata.cpp"
          HEAP32[(($nf)>>2)]=$add30; //@line 261 "synthdata.cpp"
          ; //@line 262 "synthdata.cpp"
        }
        else if (__label__ == 13) {
  
          var $arrayidx32=((__ZZ11LookupSpectP11PHONEME_TABiP10FMT_PARAMSPiP12PHONEME_LISTE10frames_buf+$36*8)&4294967295); //@line 265 "synthdata.cpp"
          $frames=$arrayidx32; //@line 265 "synthdata.cpp"
          var $37=$seq_break; //@line 266 "synthdata.cpp"
          var $38=HEAP32[(($nf)>>2)]; //@line 266 "synthdata.cpp"
          var $sub=((($38)-($37))&4294967295); //@line 266 "synthdata.cpp"
          HEAP32[(($nf)>>2)]=$sub; //@line 266 "synthdata.cpp"
          ;
        }
      }
      else if (__label__ == 3) {
  
        $frames=((__ZZ11LookupSpectP11PHONEME_TABiP10FMT_PARAMSPiP12PHONEME_LISTE10frames_buf)&4294967295); //@line 256 "synthdata.cpp"
        ; //@line 257 "synthdata.cpp"
      }
    } while(0);
  
    var $39=$this_ph_addr; //@line 271 "synthdata.cpp"
    var $type=(($39+11)&4294967295); //@line 271 "synthdata.cpp"
    var $40=HEAPU8[($type)]; //@line 271 "synthdata.cpp"
    var $conv35=((($40))&255); //@line 271 "synthdata.cpp"
    var $cmp36=((($conv35))|0)==2; //@line 271 "synthdata.cpp"
    if ($cmp36) { __label__ = 15;; } else { __label__ = 18;; } //@line 271 "synthdata.cpp"
    $land_lhs_true$$if_end42$22: do { 
      if (__label__ == 15) {
  
        var $41=$fmt_params_addr; //@line 271 "synthdata.cpp"
        var $fmt2_addr=(($41+20)&4294967295); //@line 271 "synthdata.cpp"
        var $42=HEAP32[(($fmt2_addr)>>2)]; //@line 271 "synthdata.cpp"
        var $cmp37=((($42))|0)==0; //@line 271 "synthdata.cpp"
        if (!($cmp37)) { __label__ = 18;break $land_lhs_true$$if_end42$22; } //@line 271 "synthdata.cpp"
  
        var $43=$fmt_params_addr; //@line 271 "synthdata.cpp"
        var $use_vowelin=(($43+4)&4294967295); //@line 271 "synthdata.cpp"
        var $44=HEAP32[(($use_vowelin)>>2)]; //@line 271 "synthdata.cpp"
        var $tobool39=((($44))|0)!=0; //@line 271 "synthdata.cpp"
        if (!($tobool39)) { __label__ = 18;break $land_lhs_true$$if_end42$22; } //@line 271 "synthdata.cpp"
  
        var $45=$frames; //@line 273 "synthdata.cpp"
        var $46=$fmt_params_addr; //@line 273 "synthdata.cpp"
        var $transition0=(($46+36)&4294967295); //@line 273 "synthdata.cpp"
        var $47=HEAP32[(($transition0)>>2)]; //@line 273 "synthdata.cpp"
        var $48=$fmt_params_addr; //@line 273 "synthdata.cpp"
        var $transition1=(($48+40)&4294967295); //@line 273 "synthdata.cpp"
        var $49=HEAP32[(($transition1)>>2)]; //@line 273 "synthdata.cpp"
        var $50=$which_addr; //@line 273 "synthdata.cpp"
        var $call=__Z18FormantTransition2P10frameref_tRijjP11PHONEME_TABi($45, $nf, $47, $49, 0, $50); //@line 273 "synthdata.cpp"
        var $51=HEAP32[((_seq_len_adjust)>>2)]; //@line 273 "synthdata.cpp"
        var $add41=((($51)+($call))&4294967295); //@line 273 "synthdata.cpp"
        HEAP32[((_seq_len_adjust)>>2)]=$add41; //@line 273 "synthdata.cpp"
        ; //@line 274 "synthdata.cpp"
      }
    } while(0);
  
    $length1=0; //@line 276 "synthdata.cpp"
    var $52=HEAP32[(($nf)>>2)]; //@line 277 "synthdata.cpp"
    var $sub43=((($52)-1)&4294967295); //@line 277 "synthdata.cpp"
    $nf1=$sub43; //@line 277 "synthdata.cpp"
    $ix=0; //@line 278 "synthdata.cpp"
    var $53=$ix; //@line 278 "synthdata.cpp"
    var $54=$nf1; //@line 278 "synthdata.cpp"
    var $cmp457=((($53))|0) < ((($54))|0); //@line 278 "synthdata.cpp"
    if ($cmp457) { __label__ = 19;; } else { __label__ = 20;; } //@line 278 "synthdata.cpp"
    $for_body46$$for_end53$27: do { 
      if (__label__ == 19) {
        while(1) { 
  
          var $55=$ix; //@line 279 "synthdata.cpp"
          var $56=$frames; //@line 279 "synthdata.cpp"
          var $arrayidx47=(($56+8*$55)&4294967295); //@line 279 "synthdata.cpp"
          var $length48=(($arrayidx47)&4294967295); //@line 279 "synthdata.cpp"
          var $57=HEAP16[(($length48)>>1)]; //@line 279 "synthdata.cpp"
          var $conv49=(tempInt=(($57)),(tempInt>=32768?tempInt-65536:tempInt)); //@line 279 "synthdata.cpp"
          var $58=$length1; //@line 279 "synthdata.cpp"
          var $add50=((($58)+($conv49))&4294967295); //@line 279 "synthdata.cpp"
          $length1=$add50; //@line 279 "synthdata.cpp"
          var $59=$ix; //@line 278 "synthdata.cpp"
          var $inc52=((($59)+1)&4294967295); //@line 278 "synthdata.cpp"
          $ix=$inc52; //@line 278 "synthdata.cpp"
          var $60=$ix; //@line 278 "synthdata.cpp"
          var $61=$nf1; //@line 278 "synthdata.cpp"
          var $cmp45=((($60))|0) < ((($61))|0); //@line 278 "synthdata.cpp"
          if ($cmp45) { __label__ = 19;continue ; } else { __label__ = 20;break $for_body46$$for_end53$27; } //@line 278 "synthdata.cpp"
        }
      }
    } while(0);
  
    var $62=$fmt_params_addr; //@line 281 "synthdata.cpp"
    var $fmt2_addr54=(($62+20)&4294967295); //@line 281 "synthdata.cpp"
    var $63=HEAP32[(($fmt2_addr54)>>2)]; //@line 281 "synthdata.cpp"
    var $cmp55=((($63))|0)!=0; //@line 281 "synthdata.cpp"
    if ($cmp55) { __label__ = 21;; } else { __label__ = 29;; } //@line 281 "synthdata.cpp"
    if (__label__ == 21) {
  
      var $64=$fmt_params_addr; //@line 285 "synthdata.cpp"
      var $fmt2_addr57=(($64+20)&4294967295); //@line 285 "synthdata.cpp"
      var $65=HEAP32[(($fmt2_addr57)>>2)]; //@line 285 "synthdata.cpp"
      var $66=HEAP32[((_phondata_ptr)>>2)]; //@line 285 "synthdata.cpp"
      var $arrayidx58=(($66+$65)&4294967295); //@line 285 "synthdata.cpp"
      var $67=$arrayidx58; //@line 285 "synthdata.cpp"
      $seq2=$67; //@line 285 "synthdata.cpp"
      var $68=$seq2; //@line 286 "synthdata.cpp"
      var $69=$68; //@line 286 "synthdata.cpp"
      $seqk2=$69; //@line 286 "synthdata.cpp"
      var $70=HEAP32[(($nf)>>2)]; //@line 289 "synthdata.cpp"
      var $dec=((($70)-1)&4294967295); //@line 289 "synthdata.cpp"
      HEAP32[(($nf)>>2)]=$dec; //@line 289 "synthdata.cpp"
      $ix=0; //@line 290 "synthdata.cpp"
      var $71=$ix; //@line 290 "synthdata.cpp"
      var $72=$seq2; //@line 290 "synthdata.cpp"
      var $n_frames604=(($72+2)&4294967295); //@line 290 "synthdata.cpp"
      var $73=HEAPU8[($n_frames604)]; //@line 290 "synthdata.cpp"
      var $conv615=((($73))&255); //@line 290 "synthdata.cpp"
      var $cmp626=((($71))|0) < ((($conv615))|0); //@line 290 "synthdata.cpp"
      if ($cmp626) { __label__ = 22;; } else { __label__ = 28;; } //@line 290 "synthdata.cpp"
      $for_body63$$for_end92$33: do { 
        if (__label__ == 22) {
          while(1) { 
  
            var $74=$seq2; //@line 292 "synthdata.cpp"
            var $frame64=(($74+4)&4294967295); //@line 292 "synthdata.cpp"
            var $arrayidx65=(($frame64)&4294967295); //@line 292 "synthdata.cpp"
            var $frflags66=(($arrayidx65)&4294967295); //@line 292 "synthdata.cpp"
            var $75=HEAP16[(($frflags66)>>1)]; //@line 292 "synthdata.cpp"
            var $conv67=(tempInt=(($75)),(tempInt>=32768?tempInt-65536:tempInt)); //@line 292 "synthdata.cpp"
            var $and68=($conv67) & 1; //@line 292 "synthdata.cpp"
            var $tobool69=((($and68))|0)!=0; //@line 292 "synthdata.cpp"
            var $76=$ix; //@line 293 "synthdata.cpp"
            if ($tobool69) { __label__ = 23;; } else { __label__ = 24;; } //@line 292 "synthdata.cpp"
            if (__label__ == 23) {
  
              var $77=$seqk2; //@line 293 "synthdata.cpp"
              var $frame71=(($77+4)&4294967295); //@line 293 "synthdata.cpp"
              var $arrayidx72=(($frame71+$76*64)&4294967295); //@line 293 "synthdata.cpp"
              $frame=$arrayidx72; //@line 293 "synthdata.cpp"
              ; //@line 293 "synthdata.cpp"
            }
            else if (__label__ == 24) {
  
              var $78=$seq2; //@line 295 "synthdata.cpp"
              var $frame74=(($78+4)&4294967295); //@line 295 "synthdata.cpp"
              var $arrayidx75=(($frame74+$76*44)&4294967295); //@line 295 "synthdata.cpp"
              var $79=$arrayidx75; //@line 295 "synthdata.cpp"
              $frame=$79; //@line 295 "synthdata.cpp"
              ;
            }
  
            var $80=$frame; //@line 297 "synthdata.cpp"
            var $length77=(($80+16)&4294967295); //@line 297 "synthdata.cpp"
            var $81=HEAPU8[($length77)]; //@line 297 "synthdata.cpp"
            var $conv78=((($81))&255); //@line 297 "synthdata.cpp"
            var $82=HEAP32[(($nf)>>2)]; //@line 297 "synthdata.cpp"
            var $83=$frames; //@line 297 "synthdata.cpp"
            var $arrayidx79=(($83+8*$82)&4294967295); //@line 297 "synthdata.cpp"
            var $length80=(($arrayidx79)&4294967295); //@line 297 "synthdata.cpp"
            HEAP16[(($length80)>>1)]=$conv78; //@line 297 "synthdata.cpp"
            var $84=$ix; //@line 298 "synthdata.cpp"
            var $cmp81=((($84))|0) > 0; //@line 298 "synthdata.cpp"
            if ($cmp81) { __label__ = 26;; } else { __label__ = 27;; } //@line 298 "synthdata.cpp"
            if (__label__ == 26) {
  
              var $85=$frame; //@line 300 "synthdata.cpp"
              var $86=HEAP32[(($nf)>>2)]; //@line 300 "synthdata.cpp"
              var $87=$frames; //@line 300 "synthdata.cpp"
              var $arrayidx83=(($87+8*$86)&4294967295); //@line 300 "synthdata.cpp"
              var $frame84=(($arrayidx83+4)&4294967295); //@line 300 "synthdata.cpp"
              HEAP32[(($frame84)>>2)]=$85; //@line 300 "synthdata.cpp"
              var $88=$frame; //@line 301 "synthdata.cpp"
              var $frflags85=(($88)&4294967295); //@line 301 "synthdata.cpp"
              var $89=HEAP16[(($frflags85)>>1)]; //@line 301 "synthdata.cpp"
              var $90=HEAP32[(($nf)>>2)]; //@line 301 "synthdata.cpp"
              var $91=$frames; //@line 301 "synthdata.cpp"
              var $arrayidx86=(($91+8*$90)&4294967295); //@line 301 "synthdata.cpp"
              var $frflags87=(($arrayidx86+2)&4294967295); //@line 301 "synthdata.cpp"
              HEAP16[(($frflags87)>>1)]=$89; //@line 301 "synthdata.cpp"
              ; //@line 302 "synthdata.cpp"
            }
  
            var $92=HEAP32[(($nf)>>2)]; //@line 303 "synthdata.cpp"
            var $inc89=((($92)+1)&4294967295); //@line 303 "synthdata.cpp"
            HEAP32[(($nf)>>2)]=$inc89; //@line 303 "synthdata.cpp"
            var $93=$ix; //@line 290 "synthdata.cpp"
            var $inc91=((($93)+1)&4294967295); //@line 290 "synthdata.cpp"
            $ix=$inc91; //@line 290 "synthdata.cpp"
            var $94=$ix; //@line 290 "synthdata.cpp"
            var $95=$seq2; //@line 290 "synthdata.cpp"
            var $n_frames60=(($95+2)&4294967295); //@line 290 "synthdata.cpp"
            var $96=HEAPU8[($n_frames60)]; //@line 290 "synthdata.cpp"
            var $conv61=((($96))&255); //@line 290 "synthdata.cpp"
            var $cmp62=((($94))|0) < ((($conv61))|0); //@line 290 "synthdata.cpp"
            if ($cmp62) { __label__ = 22;continue ; } else { __label__ = 28;break $for_body63$$for_end92$33; } //@line 290 "synthdata.cpp"
          }
        }
      } while(0);
  
      HEAP32[((_wavefile_ix)>>2)]=0; //@line 305 "synthdata.cpp"
      ; //@line 306 "synthdata.cpp"
    }
  
    var $97=$length1; //@line 308 "synthdata.cpp"
    var $cmp94=((($97))|0) > 0; //@line 308 "synthdata.cpp"
    if ($cmp94) { __label__ = 30;; } else { __label__ = 47;; } //@line 308 "synthdata.cpp"
    $if_then95$$if_end175$45: do { 
      if (__label__ == 30) {
  
        var $98=$which_addr; //@line 310 "synthdata.cpp"
        var $cmp96=((($98))|0)==2; //@line 310 "synthdata.cpp"
        if ($cmp96) { __label__ = 31;; } else { __label__ = 37;; } //@line 310 "synthdata.cpp"
        if (__label__ == 31) {
  
          var $99=$fmt_params_addr; //@line 315 "synthdata.cpp"
          var $std_length=(($99+44)&4294967295); //@line 315 "synthdata.cpp"
          var $100=HEAP32[(($std_length)>>2)]; //@line 315 "synthdata.cpp"
          var $101=HEAP32[((_seq_len_adjust)>>2)]; //@line 315 "synthdata.cpp"
          var $add98=((($100)-45)&4294967295); //@line 315 "synthdata.cpp"
          var $sub99=((($add98)+($101))&4294967295); //@line 315 "synthdata.cpp"
          $length_std=$sub99; //@line 315 "synthdata.cpp"
          var $102=$length_std; //@line 316 "synthdata.cpp"
          var $cmp100=((($102))|0) < 10; //@line 316 "synthdata.cpp"
          if ($cmp100) { __label__ = 32;; } else { __label__ = 33;; } //@line 316 "synthdata.cpp"
          if (__label__ == 32) {
  
            $length_std=10; //@line 317 "synthdata.cpp"
            ; //@line 317 "synthdata.cpp"
          }
  
          var $103=$plist_addr; //@line 318 "synthdata.cpp"
          var $synthflags=(($103+4)&4294967295); //@line 318 "synthdata.cpp"
          var $104=HEAPU16[(($synthflags)>>1)]; //@line 318 "synthdata.cpp"
          var $conv103=((($104))&65535); //@line 318 "synthdata.cpp"
          var $and104=($conv103) & 8; //@line 318 "synthdata.cpp"
          var $tobool105=((($and104))|0)!=0; //@line 318 "synthdata.cpp"
          if ($tobool105) { __label__ = 34;; } else { __label__ = 35;; } //@line 318 "synthdata.cpp"
          if (__label__ == 34) {
  
            var $105=HEAP32[((((_phoneme_tab+48)&4294967295))>>2)]; //@line 319 "synthdata.cpp"
            var $std_length107=(($105+14)&4294967295); //@line 319 "synthdata.cpp"
            var $106=HEAPU8[($std_length107)]; //@line 319 "synthdata.cpp"
            var $conv108=((($106))&255); //@line 319 "synthdata.cpp"
            var $mul=((($conv108)*2)&4294967295); //@line 319 "synthdata.cpp"
            var $107=$length_std; //@line 319 "synthdata.cpp"
            var $add109=((($mul)+($107))&4294967295); //@line 319 "synthdata.cpp"
            $length_std=$add109; //@line 319 "synthdata.cpp"
            ; //@line 319 "synthdata.cpp"
          }
  
          var $108=$length_std; //@line 324 "synthdata.cpp"
          var $mul111=((($108)*256)&4294967295); //@line 324 "synthdata.cpp"
          var $109=$length1; //@line 324 "synthdata.cpp"
          var $div=((((($mul111))|0)/((($109))|0))|0); //@line 324 "synthdata.cpp"
          $length_factor=$div; //@line 324 "synthdata.cpp"
          $ix=0; //@line 326 "synthdata.cpp"
          var $110=$ix; //@line 326 "synthdata.cpp"
          var $111=$nf1; //@line 326 "synthdata.cpp"
          var $cmp1133=((($110))|0) < ((($111))|0); //@line 326 "synthdata.cpp"
          if (!($cmp1133)) { __label__ = 47;break $if_then95$$if_end175$45; } //@line 326 "synthdata.cpp"
          while(1) { 
  
            var $112=$ix; //@line 328 "synthdata.cpp"
            var $113=$frames; //@line 328 "synthdata.cpp"
            var $arrayidx115=(($113+8*$112)&4294967295); //@line 328 "synthdata.cpp"
            var $length116=(($arrayidx115)&4294967295); //@line 328 "synthdata.cpp"
            var $114=HEAP16[(($length116)>>1)]; //@line 328 "synthdata.cpp"
            var $conv117=(tempInt=(($114)),(tempInt>=32768?tempInt-65536:tempInt)); //@line 328 "synthdata.cpp"
            var $115=$length_factor; //@line 328 "synthdata.cpp"
            var $mul118=((($conv117)*($115))&4294967295); //@line 328 "synthdata.cpp"
            var $div119=((((($mul118))|0)/256)|0); //@line 328 "synthdata.cpp"
            var $conv120=((($div119)) & 65535); //@line 328 "synthdata.cpp"
            var $116=$ix; //@line 328 "synthdata.cpp"
            var $117=$frames; //@line 328 "synthdata.cpp"
            var $arrayidx121=(($117+8*$116)&4294967295); //@line 328 "synthdata.cpp"
            var $length122=(($arrayidx121)&4294967295); //@line 328 "synthdata.cpp"
            HEAP16[(($length122)>>1)]=$conv120; //@line 328 "synthdata.cpp"
            var $118=$ix; //@line 326 "synthdata.cpp"
            var $inc124=((($118)+1)&4294967295); //@line 326 "synthdata.cpp"
            $ix=$inc124; //@line 326 "synthdata.cpp"
            var $119=$ix; //@line 326 "synthdata.cpp"
            var $120=$nf1; //@line 326 "synthdata.cpp"
            var $cmp113=((($119))|0) < ((($120))|0); //@line 326 "synthdata.cpp"
            if ($cmp113) { __label__ = 36;continue ; } else { __label__ = 47;break $if_then95$$if_end175$45; } //@line 326 "synthdata.cpp"
          }
        }
        else if (__label__ == 37) {
  
          var $121=$which_addr; //@line 333 "synthdata.cpp"
          var $cmp127=((($121))|0)==1; //@line 333 "synthdata.cpp"
          var $122=$fmt_params_addr; //@line 336 "synthdata.cpp"
          if ($cmp127) { __label__ = 38;; } else { __label__ = 41;; } //@line 333 "synthdata.cpp"
          $if_then128$$if_else145$58: do { 
            if (__label__ == 38) {
  
              var $fmt_control=(($122)&4294967295); //@line 336 "synthdata.cpp"
              var $123=HEAP32[(($fmt_control)>>2)]; //@line 336 "synthdata.cpp"
              var $cmp129=((($123))|0)==1; //@line 336 "synthdata.cpp"
              if (!($cmp129)) { __label__ = 43;break $if_then128$$if_else145$58; } //@line 336 "synthdata.cpp"
  
              var $124=$fmt_params_addr; //@line 340 "synthdata.cpp"
              var $std_length131=(($124+44)&4294967295); //@line 340 "synthdata.cpp"
              var $125=HEAP32[(($std_length131)>>2)]; //@line 340 "synthdata.cpp"
              var $cmp132=((($125))|0) < 130; //@line 340 "synthdata.cpp"
              if (!($cmp132)) { __label__ = 43;break $if_then128$$if_else145$58; } //@line 340 "synthdata.cpp"
  
              var $126=$frames; //@line 341 "synthdata.cpp"
              var $arrayidx134=(($126)&4294967295); //@line 341 "synthdata.cpp"
              var $length135=(($arrayidx134)&4294967295); //@line 341 "synthdata.cpp"
              var $127=HEAP16[(($length135)>>1)]; //@line 341 "synthdata.cpp"
              var $conv136=(tempInt=(($127)),(tempInt>=32768?tempInt-65536:tempInt)); //@line 341 "synthdata.cpp"
              var $128=$fmt_params_addr; //@line 341 "synthdata.cpp"
              var $std_length137=(($128+44)&4294967295); //@line 341 "synthdata.cpp"
              var $129=HEAP32[(($std_length137)>>2)]; //@line 341 "synthdata.cpp"
              var $mul138=((($129)*($conv136))&4294967295); //@line 341 "synthdata.cpp"
              var $div139=((((($mul138))|0)/130)|0); //@line 341 "synthdata.cpp"
              var $conv140=((($div139)) & 65535); //@line 341 "synthdata.cpp"
              var $130=$frames; //@line 341 "synthdata.cpp"
              var $arrayidx141=(($130)&4294967295); //@line 341 "synthdata.cpp"
              var $length142=(($arrayidx141)&4294967295); //@line 341 "synthdata.cpp"
              HEAP16[(($length142)>>1)]=$conv140; //@line 341 "synthdata.cpp"
              __label__ = 43;break $if_then128$$if_else145$58; //@line 341 "synthdata.cpp"
            }
            else if (__label__ == 41) {
  
              var $std_length146=(($122+44)&4294967295); //@line 347 "synthdata.cpp"
              var $131=HEAP32[(($std_length146)>>2)]; //@line 347 "synthdata.cpp"
              var $cmp147=((($131))|0) > 0; //@line 347 "synthdata.cpp"
              if (!($cmp147)) { __label__ = 43;break $if_then128$$if_else145$58; } //@line 347 "synthdata.cpp"
  
              var $132=$fmt_params_addr; //@line 349 "synthdata.cpp"
              var $std_length149=(($132+44)&4294967295); //@line 349 "synthdata.cpp"
              var $133=HEAP32[(($std_length149)>>2)]; //@line 349 "synthdata.cpp"
              var $134=$length1; //@line 349 "synthdata.cpp"
              var $sub150=((($133)-($134))&4294967295); //@line 349 "synthdata.cpp"
              var $135=HEAP32[((_seq_len_adjust)>>2)]; //@line 349 "synthdata.cpp"
              var $add151=((($135)+($sub150))&4294967295); //@line 349 "synthdata.cpp"
              HEAP32[((_seq_len_adjust)>>2)]=$add151; //@line 349 "synthdata.cpp"
              __lastLabel__ = 42; __label__ = 44;break $if_then128$$if_else145$58; //@line 350 "synthdata.cpp"
            }
          } while(0);
          if (__label__ == 43) {
  
            var $_pr=HEAP32[((_seq_len_adjust)>>2)]; //@line 353 "synthdata.cpp"
            __lastLabel__ = 43; ;
          }
  
          var $136=__lastLabel__ == 43 ? $_pr : ($add151); //@line 353 "synthdata.cpp"
          var $cmp154=((($136))|0)!=0; //@line 353 "synthdata.cpp"
          if (!($cmp154)) { __label__ = 47;break $if_then95$$if_end175$45; } //@line 353 "synthdata.cpp"
  
          var $137=$length1; //@line 355 "synthdata.cpp"
          var $138=HEAP32[((_seq_len_adjust)>>2)]; //@line 355 "synthdata.cpp"
          var $add156=((($138)+($137))&4294967295); //@line 355 "synthdata.cpp"
          var $mul157=((($add156)*256)&4294967295); //@line 355 "synthdata.cpp"
          var $139=$length1; //@line 355 "synthdata.cpp"
          var $div158=((((($mul157))|0)/((($139))|0))|0); //@line 355 "synthdata.cpp"
          $length_factor=$div158; //@line 355 "synthdata.cpp"
          $ix=0; //@line 356 "synthdata.cpp"
          var $140=$ix; //@line 356 "synthdata.cpp"
          var $141=$nf1; //@line 356 "synthdata.cpp"
          var $cmp1601=((($140))|0) < ((($141))|0); //@line 356 "synthdata.cpp"
          if (!($cmp1601)) { __label__ = 47;break $if_then95$$if_end175$45; } //@line 356 "synthdata.cpp"
          while(1) { 
  
            var $142=$ix; //@line 358 "synthdata.cpp"
            var $143=$frames; //@line 358 "synthdata.cpp"
            var $arrayidx162=(($143+8*$142)&4294967295); //@line 358 "synthdata.cpp"
            var $length163=(($arrayidx162)&4294967295); //@line 358 "synthdata.cpp"
            var $144=HEAP16[(($length163)>>1)]; //@line 358 "synthdata.cpp"
            var $conv164=(tempInt=(($144)),(tempInt>=32768?tempInt-65536:tempInt)); //@line 358 "synthdata.cpp"
            var $145=$length_factor; //@line 358 "synthdata.cpp"
            var $mul165=((($conv164)*($145))&4294967295); //@line 358 "synthdata.cpp"
            var $div166=((((($mul165))|0)/256)|0); //@line 358 "synthdata.cpp"
            var $conv167=((($div166)) & 65535); //@line 358 "synthdata.cpp"
            var $146=$ix; //@line 358 "synthdata.cpp"
            var $147=$frames; //@line 358 "synthdata.cpp"
            var $arrayidx168=(($147+8*$146)&4294967295); //@line 358 "synthdata.cpp"
            var $length169=(($arrayidx168)&4294967295); //@line 358 "synthdata.cpp"
            HEAP16[(($length169)>>1)]=$conv167; //@line 358 "synthdata.cpp"
            var $148=$ix; //@line 356 "synthdata.cpp"
            var $inc171=((($148)+1)&4294967295); //@line 356 "synthdata.cpp"
            $ix=$inc171; //@line 356 "synthdata.cpp"
            var $149=$ix; //@line 356 "synthdata.cpp"
            var $150=$nf1; //@line 356 "synthdata.cpp"
            var $cmp160=((($149))|0) < ((($150))|0); //@line 356 "synthdata.cpp"
            if ($cmp160) { __label__ = 46;continue ; } else { __label__ = 47;break $if_then95$$if_end175$45; } //@line 356 "synthdata.cpp"
          }
        }
      }
    } while(0);
  
    var $151=HEAP32[(($nf)>>2)]; //@line 364 "synthdata.cpp"
    var $152=$n_frames_addr; //@line 364 "synthdata.cpp"
    HEAP32[(($152)>>2)]=$151; //@line 364 "synthdata.cpp"
    var $153=$frames; //@line 365 "synthdata.cpp"
    STACKTOP = __stackBase__;
    return $153; //@line 365 "synthdata.cpp"
    return null;
  }
  

  function __Z11GetEnvelopei($index) {
    ;
    var __label__;
  
    var $retval;
    var $index_addr;
    $index_addr=$index;
    var $0=$index_addr; //@line 372 "synthdata.cpp"
    var $cmp=((($0))|0)==0; //@line 372 "synthdata.cpp"
    if ($cmp) { __label__ = 1;; } else { __label__ = 2;; } //@line 372 "synthdata.cpp"
    if (__label__ == 1) {
  
      var $1=HEAP32[((__impure_ptr)>>2)]; //@line 374 "synthdata.cpp"
      var $_stderr=(($1+12)&4294967295); //@line 374 "synthdata.cpp"
      var $2=HEAP32[(($_stderr)>>2)]; //@line 374 "synthdata.cpp"
      var $3=_fwrite(((__str5416)&4294967295), 20, 1, $2); //@line 374 "synthdata.cpp"
      var $4=HEAP32[((((_envelope_data)&4294967295))>>2)]; //@line 375 "synthdata.cpp"
      $retval=$4; //@line 375 "synthdata.cpp"
      ; //@line 375 "synthdata.cpp"
    }
    else if (__label__ == 2) {
  
      var $5=$index_addr; //@line 377 "synthdata.cpp"
      var $6=HEAP32[((_phondata_ptr)>>2)]; //@line 377 "synthdata.cpp"
      var $arrayidx=(($6+$5)&4294967295); //@line 377 "synthdata.cpp"
      $retval=$arrayidx; //@line 377 "synthdata.cpp"
      ; //@line 377 "synthdata.cpp"
    }
  
    var $7=$retval; //@line 378 "synthdata.cpp"
    ;
    return $7; //@line 378 "synthdata.cpp"
    return null;
  }
  

  function __Z18SelectPhonemeTablei($number) {
    ;
    var __label__;
  
    var $number_addr;
    $number_addr=$number;
    HEAP32[((_n_phoneme_tab)>>2)]=0; //@line 416 "synthdata.cpp"
    var $0=$number_addr; //@line 417 "synthdata.cpp"
    __ZL17SetUpPhonemeTableii($0, 0); //@line 417 "synthdata.cpp"
    var $1=HEAP32[((_n_phoneme_tab)>>2)]; //@line 418 "synthdata.cpp"
    var $inc=((($1)+1)&4294967295); //@line 418 "synthdata.cpp"
    HEAP32[((_n_phoneme_tab)>>2)]=$inc; //@line 418 "synthdata.cpp"
    var $2=$number_addr; //@line 419 "synthdata.cpp"
    HEAP32[((_current_phoneme_table)>>2)]=$2; //@line 419 "synthdata.cpp"
    ;
    return; //@line 420 "synthdata.cpp"
    return;
  }
  

  function __ZL17SetUpPhonemeTableii($number, $recursing) {
    ;
    var __label__;
  
    var $number_addr;
    var $recursing_addr;
    var $ix;
    var $includes;
    var $ph_code;
    var $phtab;
    $number_addr=$number;
    $recursing_addr=$recursing;
    var $0=$recursing_addr; //@line 388 "synthdata.cpp"
    var $cmp=((($0))|0)==0; //@line 388 "synthdata.cpp"
    if ($cmp) { __label__ = 1;; } else { __label__ = 2;; } //@line 388 "synthdata.cpp"
    if (__label__ == 1) {
  
      var $dest$, $stop$, $stop4$, $fast$, $value4$;
      $dest$ = ((_phoneme_tab_flags)&4294967295);
      $stop$ = $dest$ + 256;
      $value4$ = 0;
      if ($value4$ < 0) $value4$ += 256;
      $value4$ = $value4$ + ($value4$<<8) + ($value4$<<16) + ($value4$*16777216);
      while ($dest$%4 !== 0 && $dest$ < $stop$) {
        ;; HEAP8[$dest$++] = 0;
      }
      $dest$ >>= 2;
      $stop4$ = $stop$ >> 2;
      while ($dest$ < $stop4$) {
      ;;;;;;;  HEAP32[$dest$++] = $value4$;
      }
      $dest$ <<= 2;
      while ($dest$ < $stop$) {
        ;; HEAP8[$dest$++] = 0;
      }; //@line 390 "synthdata.cpp"
      ; //@line 391 "synthdata.cpp"
    }
  
    var $1=$number_addr; //@line 393 "synthdata.cpp"
    var $arrayidx=((_phoneme_tab_list+$1*48)&4294967295); //@line 393 "synthdata.cpp"
    var $includes1=(($arrayidx+40)&4294967295); //@line 393 "synthdata.cpp"
    var $2=HEAP32[(($includes1)>>2)]; //@line 393 "synthdata.cpp"
    $includes=$2; //@line 393 "synthdata.cpp"
    var $cmp2=((($2))|0) > 0; //@line 393 "synthdata.cpp"
    if ($cmp2) { __label__ = 3;; } else { __label__ = 4;; } //@line 393 "synthdata.cpp"
    if (__label__ == 3) {
  
      var $3=$includes; //@line 396 "synthdata.cpp"
      var $sub=((($3)-1)&4294967295); //@line 396 "synthdata.cpp"
      __ZL17SetUpPhonemeTableii($sub, 1); //@line 396 "synthdata.cpp"
      ; //@line 397 "synthdata.cpp"
    }
  
    var $4=$number_addr; //@line 400 "synthdata.cpp"
    var $arrayidx5=((_phoneme_tab_list+$4*48)&4294967295); //@line 400 "synthdata.cpp"
    var $phoneme_tab_ptr=(($arrayidx5+32)&4294967295); //@line 400 "synthdata.cpp"
    var $5=HEAP32[(($phoneme_tab_ptr)>>2)]; //@line 400 "synthdata.cpp"
    $phtab=$5; //@line 400 "synthdata.cpp"
    $ix=0; //@line 401 "synthdata.cpp"
    var $6=$ix; //@line 401 "synthdata.cpp"
    var $7=$number_addr; //@line 401 "synthdata.cpp"
    var $arrayidx61=((_phoneme_tab_list+$7*48)&4294967295); //@line 401 "synthdata.cpp"
    var $n_phonemes2=(($arrayidx61+36)&4294967295); //@line 401 "synthdata.cpp"
    var $8=HEAP32[(($n_phonemes2)>>2)]; //@line 401 "synthdata.cpp"
    var $cmp73=((($6))|0) < ((($8))|0); //@line 401 "synthdata.cpp"
    if ($cmp73) { __label__ = 5;; } else { __label__ = 10;; } //@line 401 "synthdata.cpp"
    $for_body$$for_end$8: do { 
      if (__label__ == 5) {
        while(1) { 
  
          var $9=$ix; //@line 403 "synthdata.cpp"
          var $10=$phtab; //@line 403 "synthdata.cpp"
          var $arrayidx8=(($10+16*$9)&4294967295); //@line 403 "synthdata.cpp"
          var $code=(($arrayidx8+10)&4294967295); //@line 403 "synthdata.cpp"
          var $11=HEAPU8[($code)]; //@line 403 "synthdata.cpp"
          var $conv=((($11))&255); //@line 403 "synthdata.cpp"
          $ph_code=$conv; //@line 403 "synthdata.cpp"
          var $12=$ix; //@line 404 "synthdata.cpp"
          var $13=$phtab; //@line 404 "synthdata.cpp"
          var $arrayidx9=(($13+16*$12)&4294967295); //@line 404 "synthdata.cpp"
          var $14=$ph_code; //@line 404 "synthdata.cpp"
          var $arrayidx10=((_phoneme_tab+$14*4)&4294967295); //@line 404 "synthdata.cpp"
          HEAP32[(($arrayidx10)>>2)]=$arrayidx9; //@line 404 "synthdata.cpp"
          var $15=$ph_code; //@line 405 "synthdata.cpp"
          var $16=HEAP32[((_n_phoneme_tab)>>2)]; //@line 405 "synthdata.cpp"
          var $cmp11=((($15))|0) > ((($16))|0); //@line 405 "synthdata.cpp"
          if ($cmp11) { __label__ = 6;; } else { __label__ = 7;; } //@line 405 "synthdata.cpp"
          if (__label__ == 6) {
  
            var $17=$ph_code; //@line 406 "synthdata.cpp"
            HEAP32[((_n_phoneme_tab)>>2)]=$17; //@line 406 "synthdata.cpp"
            ; //@line 406 "synthdata.cpp"
          }
  
          var $18=$recursing_addr; //@line 408 "synthdata.cpp"
          var $cmp14=((($18))|0)==0; //@line 408 "synthdata.cpp"
          if ($cmp14) { __label__ = 8;; } else { __label__ = 9;; } //@line 408 "synthdata.cpp"
          if (__label__ == 8) {
  
            var $19=$ph_code; //@line 409 "synthdata.cpp"
            var $arrayidx16=((_phoneme_tab_flags+$19)&4294967295); //@line 409 "synthdata.cpp"
            var $20=HEAPU8[($arrayidx16)]; //@line 409 "synthdata.cpp"
            var $conv17=((($20))&255); //@line 409 "synthdata.cpp"
            var $or=($conv17) | 1; //@line 409 "synthdata.cpp"
            var $conv18=((($or)) & 255); //@line 409 "synthdata.cpp"
            HEAP8[($arrayidx16)]=$conv18; //@line 409 "synthdata.cpp"
            ; //@line 409 "synthdata.cpp"
          }
  
          var $21=$ix; //@line 401 "synthdata.cpp"
          var $inc=((($21)+1)&4294967295); //@line 401 "synthdata.cpp"
          $ix=$inc; //@line 401 "synthdata.cpp"
          var $22=$ix; //@line 401 "synthdata.cpp"
          var $23=$number_addr; //@line 401 "synthdata.cpp"
          var $arrayidx6=((_phoneme_tab_list+$23*48)&4294967295); //@line 401 "synthdata.cpp"
          var $n_phonemes=(($arrayidx6+36)&4294967295); //@line 401 "synthdata.cpp"
          var $24=HEAP32[(($n_phonemes)>>2)]; //@line 401 "synthdata.cpp"
          var $cmp7=((($22))|0) < ((($24))|0); //@line 401 "synthdata.cpp"
          if ($cmp7) { __label__ = 5;continue ; } else { __label__ = 10;break $for_body$$for_end$8; } //@line 401 "synthdata.cpp"
        }
      }
    } while(0);
  
    ;
    return; //@line 411 "synthdata.cpp"
    return;
  }
  

  function __Z18LookupPhonemeTablePKc($name) {
    ;
    var __label__;
  
    var $retval;
    var $name_addr;
    var $ix;
    $name_addr=$name;
    $ix=0; //@line 427 "synthdata.cpp"
    ; //@line 427 "synthdata.cpp"
    while(1) { 
  
      var $0=$ix; //@line 427 "synthdata.cpp"
      var $1=HEAP32[((_n_phoneme_tables)>>2)]; //@line 427 "synthdata.cpp"
      var $cmp=((($0))|0) < ((($1))|0); //@line 427 "synthdata.cpp"
      if (!($cmp)) { __label__ = 5;break ; } //@line 427 "synthdata.cpp"
  
      var $2=$name_addr; //@line 429 "synthdata.cpp"
      var $3=$ix; //@line 429 "synthdata.cpp"
      var $arrayidx=((_phoneme_tab_list+$3*48)&4294967295); //@line 429 "synthdata.cpp"
      var $name1=(($arrayidx)&4294967295); //@line 429 "synthdata.cpp"
      var $arraydecay=(($name1)&4294967295); //@line 429 "synthdata.cpp"
      var $call=_strcmp($2, $arraydecay); //@line 429 "synthdata.cpp"
      var $cmp2=((($call))|0)==0; //@line 429 "synthdata.cpp"
      var $4=$ix; //@line 431 "synthdata.cpp"
      if ($cmp2) { __label__ = 3;break ; } //@line 429 "synthdata.cpp"
  
      var $inc=((($4)+1)&4294967295); //@line 427 "synthdata.cpp"
      $ix=$inc; //@line 427 "synthdata.cpp"
      __label__ = 1;continue ; //@line 427 "synthdata.cpp"
    }
    if (__label__ == 3) {
  
      HEAP32[((_phoneme_tab_number)>>2)]=$4; //@line 431 "synthdata.cpp"
      ; //@line 432 "synthdata.cpp"
    }
  
    var $5=$ix; //@line 435 "synthdata.cpp"
    var $6=HEAP32[((_n_phoneme_tables)>>2)]; //@line 435 "synthdata.cpp"
    var $cmp3=((($5))|0)==((($6))|0); //@line 435 "synthdata.cpp"
    if ($cmp3) { __label__ = 6;; } else { __label__ = 7;; } //@line 435 "synthdata.cpp"
    if (__label__ == 6) {
  
      $retval=-1; //@line 436 "synthdata.cpp"
      ; //@line 436 "synthdata.cpp"
    }
    else if (__label__ == 7) {
  
      var $7=$ix; //@line 438 "synthdata.cpp"
      $retval=$7; //@line 438 "synthdata.cpp"
      ; //@line 438 "synthdata.cpp"
    }
  
    var $8=$retval; //@line 439 "synthdata.cpp"
    ;
    return $8; //@line 439 "synthdata.cpp"
    return null;
  }
  

  function __Z22SelectPhonemeTableNamePKc($name) {
    ;
    var __label__;
  
    var $retval;
    var $name_addr;
    var $ix;
    $name_addr=$name;
    var $0=$name_addr; //@line 448 "synthdata.cpp"
    var $call=__Z18LookupPhonemeTablePKc($0); //@line 448 "synthdata.cpp"
    $ix=$call; //@line 448 "synthdata.cpp"
    var $cmp=((($call))|0)==-1; //@line 448 "synthdata.cpp"
    if ($cmp) { __label__ = 1;; } else { __label__ = 2;; } //@line 448 "synthdata.cpp"
    if (__label__ == 1) {
  
      $retval=-1; //@line 449 "synthdata.cpp"
      ; //@line 449 "synthdata.cpp"
    }
    else if (__label__ == 2) {
  
      var $1=$ix; //@line 451 "synthdata.cpp"
      __Z18SelectPhonemeTablei($1); //@line 451 "synthdata.cpp"
      var $2=$ix; //@line 452 "synthdata.cpp"
      $retval=$2; //@line 452 "synthdata.cpp"
      ; //@line 452 "synthdata.cpp"
    }
  
    var $3=$retval; //@line 453 "synthdata.cpp"
    ;
    return $3; //@line 453 "synthdata.cpp"
    return null;
  }
  

  function __Z10LoadConfigv() {
    var __stackBase__  = STACKTOP; STACKTOP += 364; _memset(__stackBase__, 0, 364);
    var __label__;
  
    var $buf=__stackBase__;
    var $f;
    var $ix;
    var $c1=__stackBase__+160;
    var $p;
    var $string=__stackBase__+164;
    HEAP32[((_logging_type)>>2)]=0; //@line 468 "synthdata.cpp"
    $ix=0; //@line 470 "synthdata.cpp"
    ; //@line 470 "synthdata.cpp"
    $for_body$2: while(1) { 
  
      var $0=$ix; //@line 472 "synthdata.cpp"
      var $arrayidx=((_soundicon_tab+$0*16)&4294967295); //@line 472 "synthdata.cpp"
      var $filename=(($arrayidx+12)&4294967295); //@line 472 "synthdata.cpp"
      HEAP32[(($filename)>>2)]=0; //@line 472 "synthdata.cpp"
      var $1=$ix; //@line 473 "synthdata.cpp"
      var $arrayidx1=((_soundicon_tab+$1*16)&4294967295); //@line 473 "synthdata.cpp"
      var $data=(($arrayidx1+8)&4294967295); //@line 473 "synthdata.cpp"
      HEAP32[(($data)>>2)]=0; //@line 473 "synthdata.cpp"
      var $2=$ix; //@line 470 "synthdata.cpp"
      var $inc=((($2)+1)&4294967295); //@line 470 "synthdata.cpp"
      $ix=$inc; //@line 470 "synthdata.cpp"
      var $cmp=((($inc))|0) < 4; //@line 470 "synthdata.cpp"
      if ($cmp) { __label__ = 1;continue $for_body$2; } else { __label__ = 2;break $for_body$2; } //@line 470 "synthdata.cpp"
    }
  
    var $arraydecay=(($buf)&4294967295); //@line 476 "synthdata.cpp"
    var $call=_sprintf($arraydecay, ((__str74726)&4294967295), allocate([((_path_home)&4294967295),0,0,0,47,0,0,0,((__str7418)&4294967295),0,0,0], ["i8*",0,0,0,"i32",0,0,0,"i8*",0,0,0], ALLOC_STACK)); //@line 476 "synthdata.cpp"
    var $arraydecay2=(($buf)&4294967295); //@line 477 "synthdata.cpp"
    var $call3=_fopen($arraydecay2, ((__str51703)&4294967295)); //@line 477 "synthdata.cpp"
    $f=$call3; //@line 477 "synthdata.cpp"
    var $cmp4=((($call3))|0)==0; //@line 477 "synthdata.cpp"
    if ($cmp4) { __label__ = 17;; } else { __label__ = 3;; } //@line 477 "synthdata.cpp"
    $while_end$$while_cond_preheader$5: do { 
      if (__label__ == 3) {
  
        var $arraydecay5=(($buf)&4294967295); //@line 482 "synthdata.cpp"
        var $3=$f; //@line 482 "synthdata.cpp"
        var $call61=_fgets($arraydecay5, 160, $3); //@line 482 "synthdata.cpp"
        var $cmp72=((($call61))|0)!=0; //@line 482 "synthdata.cpp"
        if (!($cmp72)) { __label__ = 17;break $while_end$$while_cond_preheader$5; } //@line 482 "synthdata.cpp"
  
        var $arrayidx8=(($buf)&4294967295); //@line 484 "synthdata.cpp"
        var $arraydecay12=(($buf)&4294967295); //@line 486 "synthdata.cpp"
        var $arrayidx16=(($buf+4)&4294967295); //@line 488 "synthdata.cpp"
        var $arraydecay17=(($string)&4294967295); //@line 488 "synthdata.cpp"
        var $arraydecay21=(($string)&4294967295); //@line 489 "synthdata.cpp"
        var $arraydecay24=(($buf)&4294967295); //@line 492 "synthdata.cpp"
        var $arrayidx28=(($buf+5)&4294967295); //@line 494 "synthdata.cpp"
        var $arraydecay30=(($buf)&4294967295); //@line 497 "synthdata.cpp"
        var $arrayidx34=(($buf+10)&4294967295); //@line 499 "synthdata.cpp"
        var $arraydecay37=(($buf)&4294967295); //@line 502 "synthdata.cpp"
        var $arrayidx41=(($buf+10)&4294967295); //@line 504 "synthdata.cpp"
        var $arraydecay42=(($string)&4294967295); //@line 504 "synthdata.cpp"
        var $arraydecay48=(($string)&4294967295); //@line 508 "synthdata.cpp"
        var $arraydecay51=(($string)&4294967295); //@line 509 "synthdata.cpp"
        ; //@line 482 "synthdata.cpp"
        $while_body$8: while(1) { 
  
          var $4=HEAP8[($arrayidx8)]; //@line 484 "synthdata.cpp"
          var $conv=(tempInt=(($4)),(tempInt>=128?tempInt-256:tempInt)); //@line 484 "synthdata.cpp"
          var $cmp9=((($conv))|0)==47; //@line 484 "synthdata.cpp"
          if ($cmp9) { __label__ = 6;; } else { __label__ = 7;; } //@line 484 "synthdata.cpp"
          $while_cond_backedge$$if_end11$10: do { 
            if (__label__ == 7) {
  
              var $call13=_memcmp($arraydecay12, ((__str9420)&4294967295), 3); //@line 486 "synthdata.cpp"
              var $cmp14=((($call13))|0)==0; //@line 486 "synthdata.cpp"
              if ($cmp14) { __label__ = 8;; } else { __label__ = 10;; } //@line 486 "synthdata.cpp"
              if (__label__ == 8) {
  
                var $call18=_sscanf($arrayidx16, ((__str10421)&4294967295), allocate([_logging_type,0,0,0,$arraydecay17,0,0,0], ["i32*",0,0,0,"i8*",0,0,0], ALLOC_STACK)); //@line 488 "synthdata.cpp"
                var $cmp19=((($call18))|0)==2; //@line 488 "synthdata.cpp"
                if (!($cmp19)) { __label__ = 6;break $while_cond_backedge$$if_end11$10; } //@line 488 "synthdata.cpp"
  
                var $call22=_fopen($arraydecay21, ((__str11422)&4294967295)); //@line 489 "synthdata.cpp"
                HEAP32[((_f_logespeak)>>2)]=$call22; //@line 489 "synthdata.cpp"
                ; //@line 489 "synthdata.cpp"
              }
              else if (__label__ == 10) {
  
                var $call25=_memcmp($arraydecay24, ((__str27677)&4294967295), 4); //@line 492 "synthdata.cpp"
                var $cmp26=((($call25))|0)==0; //@line 492 "synthdata.cpp"
                if ($cmp26) { __label__ = 11;; } else { __label__ = 12;; } //@line 492 "synthdata.cpp"
                if (__label__ == 11) {
  
                  __Z14ReadTonePointsPcPi($arrayidx28, ((_tone_points)&4294967295)); //@line 494 "synthdata.cpp"
                  ; //@line 495 "synthdata.cpp"
                }
                else if (__label__ == 12) {
  
                  var $call31=_memcmp($arraydecay30, ((__str13424)&4294967295), 9); //@line 497 "synthdata.cpp"
                  var $cmp32=((($call31))|0)==0; //@line 497 "synthdata.cpp"
                  if ($cmp32) { __label__ = 13;; } else { __label__ = 14;; } //@line 497 "synthdata.cpp"
                  if (__label__ == 13) {
  
                    var $call35=_sscanf($arrayidx34, ((__str63715)&4294967295), allocate([_option_device_number,0,0,0], ["i32*",0,0,0], ALLOC_STACK)); //@line 499 "synthdata.cpp"
                    ; //@line 500 "synthdata.cpp"
                  }
                  else if (__label__ == 14) {
  
                    var $call38=_memcmp($arraydecay37, ((__str15426)&4294967295), 9); //@line 502 "synthdata.cpp"
                    var $cmp39=((($call38))|0)==0; //@line 502 "synthdata.cpp"
                    if (!($cmp39)) { __label__ = 6;break $while_cond_backedge$$if_end11$10; } //@line 502 "synthdata.cpp"
  
                    var $call43=_sscanf($arrayidx41, ((__str16427)&4294967295), allocate([$c1,0,0,0,$arraydecay42,0,0,0], ["i8*",0,0,0,"i8*",0,0,0], ALLOC_STACK)); //@line 504 "synthdata.cpp"
                    $ix=$call43; //@line 504 "synthdata.cpp"
                    var $6=$ix; //@line 505 "synthdata.cpp"
                    var $cmp44=((($6))|0)==2; //@line 505 "synthdata.cpp"
                    if (!($cmp44)) { __label__ = 6;break $while_cond_backedge$$if_end11$10; } //@line 505 "synthdata.cpp"
  
                    var $7=HEAP8[($c1)]; //@line 507 "synthdata.cpp"
                    var $conv46=(tempInt=(($7)),(tempInt>=128?tempInt-256:tempInt)); //@line 507 "synthdata.cpp"
                    var $8=HEAP32[((_n_soundicon_tab)>>2)]; //@line 507 "synthdata.cpp"
                    var $arrayidx47=((_soundicon_tab+$8*16)&4294967295); //@line 507 "synthdata.cpp"
                    var $name=(($arrayidx47)&4294967295); //@line 507 "synthdata.cpp"
                    HEAP32[(($name)>>2)]=$conv46; //@line 507 "synthdata.cpp"
                    var $call49=_strlen($arraydecay48); //@line 508 "synthdata.cpp"
                    var $add=((($call49)+1)&4294967295); //@line 508 "synthdata.cpp"
                    var $call50=__Z5Alloci($add); //@line 508 "synthdata.cpp"
                    $p=$call50; //@line 508 "synthdata.cpp"
                    var $9=$p; //@line 509 "synthdata.cpp"
                    var $call52=_strcpy($9, $arraydecay51); //@line 509 "synthdata.cpp"
                    var $10=$p; //@line 510 "synthdata.cpp"
                    var $11=HEAP32[((_n_soundicon_tab)>>2)]; //@line 510 "synthdata.cpp"
                    var $arrayidx53=((_soundicon_tab+$11*16)&4294967295); //@line 510 "synthdata.cpp"
                    var $filename54=(($arrayidx53+12)&4294967295); //@line 510 "synthdata.cpp"
                    HEAP32[(($filename54)>>2)]=$10; //@line 510 "synthdata.cpp"
                    var $12=HEAP32[((_n_soundicon_tab)>>2)]; //@line 511 "synthdata.cpp"
                    var $inc55=((($12)+1)&4294967295); //@line 511 "synthdata.cpp"
                    HEAP32[((_n_soundicon_tab)>>2)]=$inc55; //@line 511 "synthdata.cpp"
                    var $arrayidx56=((_soundicon_tab+$12*16)&4294967295); //@line 511 "synthdata.cpp"
                    var $length=(($arrayidx56+4)&4294967295); //@line 511 "synthdata.cpp"
                    HEAP32[(($length)>>2)]=0; //@line 511 "synthdata.cpp"
                    ; //@line 512 "synthdata.cpp"
                  }
                }
              }
            }
          } while(0);
  
          var $5=$f; //@line 482 "synthdata.cpp"
          var $call6=_fgets($arraydecay5, 160, $5); //@line 482 "synthdata.cpp"
          var $cmp7=((($call6))|0)!=0; //@line 482 "synthdata.cpp"
          if ($cmp7) { __label__ = 5;continue $while_body$8; } else { __label__ = 17;break $while_end$$while_cond_preheader$5; } //@line 482 "synthdata.cpp"
        }
      }
    } while(0);
  
    STACKTOP = __stackBase__;
    return; //@line 515 "synthdata.cpp"
    return;
  }
  

  function __Z16InterpretPhonemeP10TranslatoriP12PHONEME_LISTP12PHONEME_DATA($tr, $control, $plist, $phdata) {
    var __stackBase__  = STACKTOP; STACKTOP += 44; _memset(__stackBase__, 0, 44);
    var __label__;
    var __lastLabel__ = null;
  
    var $tr_addr;
    var $control_addr;
    var $plist_addr;
    var $phdata_addr;
    var $ph;
    var $prog=__stackBase__;
    var $instn;
    var $instn2;
    var $or_flag;
    var $truth;
    var $data;
    var $end_flag;
    var $ix;
    var $param_sc;
    var $n_return;
    var $return_addr=__stackBase__+4;
    $tr_addr=$tr;
    $control_addr=$control;
    $plist_addr=$plist;
    $phdata_addr=$phdata;
    $n_return=0; //@line 852 "synthdata.cpp"
    var $0=$plist_addr; //@line 855 "synthdata.cpp"
    var $ph1=(($0+8)&4294967295); //@line 855 "synthdata.cpp"
    var $1=HEAP32[(($ph1)>>2)]; //@line 855 "synthdata.cpp"
    $ph=$1; //@line 855 "synthdata.cpp"
    var $2=$phdata_addr; //@line 857 "synthdata.cpp"
    var $3=$2; //@line 857 "synthdata.cpp"
    var $dest$, $stop$, $stop4$, $fast$, $value4$;
    $dest$ = $3;
    $stop$ = $dest$ + 152;
    $value4$ = 0;
    if ($value4$ < 0) $value4$ += 256;
    $value4$ = $value4$ + ($value4$<<8) + ($value4$<<16) + ($value4$*16777216);
    while ($dest$%4 !== 0 && $dest$ < $stop$) {
      ;; HEAP8[$dest$++] = 0;
    }
    $dest$ >>= 2;
    $stop4$ = $stop$ >> 2;
    while ($dest$ < $stop4$) {
    ;;;;;;;  HEAP32[$dest$++] = $value4$;
    }
    $dest$ <<= 2;
    while ($dest$ < $stop$) {
      ;; HEAP8[$dest$++] = 0;
    }; //@line 857 "synthdata.cpp"
    var $4=$ph; //@line 858 "synthdata.cpp"
    var $std_length=(($4+14)&4294967295); //@line 858 "synthdata.cpp"
    var $5=HEAPU8[($std_length)]; //@line 858 "synthdata.cpp"
    var $conv=((($5))&255); //@line 858 "synthdata.cpp"
    var $6=$phdata_addr; //@line 858 "synthdata.cpp"
    var $pd_param=(($6+4)&4294967295); //@line 858 "synthdata.cpp"
    var $arrayidx=(($pd_param+40)&4294967295); //@line 858 "synthdata.cpp"
    HEAP32[(($arrayidx)>>2)]=$conv; //@line 858 "synthdata.cpp"
    var $7=$ph; //@line 859 "synthdata.cpp"
    var $length_mod=(($7+15)&4294967295); //@line 859 "synthdata.cpp"
    var $8=HEAPU8[($length_mod)]; //@line 859 "synthdata.cpp"
    var $conv2=((($8))&255); //@line 859 "synthdata.cpp"
    var $9=$phdata_addr; //@line 859 "synthdata.cpp"
    var $pd_param3=(($9+4)&4294967295); //@line 859 "synthdata.cpp"
    var $arrayidx4=(($pd_param3+36)&4294967295); //@line 859 "synthdata.cpp"
    HEAP32[(($arrayidx4)>>2)]=$conv2; //@line 859 "synthdata.cpp"
    var $10=$ph; //@line 861 "synthdata.cpp"
    var $program=(($10+8)&4294967295); //@line 861 "synthdata.cpp"
    var $11=HEAPU16[(($program)>>1)]; //@line 861 "synthdata.cpp"
    var $conv5=((($11))&65535); //@line 861 "synthdata.cpp"
    var $cmp=((($conv5))|0)==0; //@line 861 "synthdata.cpp"
    if ($cmp) { __label__ = 69;; } else { __label__ = 1;; } //@line 861 "synthdata.cpp"
    $for_end237$$if_end$2: do { 
      if (__label__ == 1) {
  
        $end_flag=0; //@line 864 "synthdata.cpp"
        var $12=$ph; //@line 866 "synthdata.cpp"
        var $program6=(($12+8)&4294967295); //@line 866 "synthdata.cpp"
        var $13=HEAPU16[(($program6)>>1)]; //@line 866 "synthdata.cpp"
        var $idxprom=((($13))&65535); //@line 866 "synthdata.cpp"
        var $14=HEAP32[((_phoneme_index)>>2)]; //@line 866 "synthdata.cpp"
        var $arrayidx7=(($14+2*$idxprom)&4294967295); //@line 866 "synthdata.cpp"
        HEAP32[(($prog)>>2)]=$arrayidx7; //@line 866 "synthdata.cpp"
        var $15=$end_flag; //@line 866 "synthdata.cpp"
        var $cmp89=((($15))|0)!=1; //@line 866 "synthdata.cpp"
        if (!($cmp89)) { __label__ = 69;break $for_end237$$if_end$2; } //@line 866 "synthdata.cpp"
        $for_body$4: while(1) { 
  
          var $16=HEAP32[(($prog)>>2)]; //@line 868 "synthdata.cpp"
          var $17=HEAP16[(($16)>>1)]; //@line 868 "synthdata.cpp"
          $instn=$17; //@line 868 "synthdata.cpp"
          var $18=$instn; //@line 869 "synthdata.cpp"
          var $conv9=((($18))&65535); //@line 869 "synthdata.cpp"
          var $shr=($conv9) >> 8; //@line 869 "synthdata.cpp"
          var $and=($shr) & 15; //@line 869 "synthdata.cpp"
          $instn2=$and; //@line 869 "synthdata.cpp"
          $or_flag=0; //@line 870 "synthdata.cpp"
          var $19=$instn; //@line 872 "synthdata.cpp"
          var $conv10=((($19))&65535); //@line 872 "synthdata.cpp"
          var $shr11=($conv10) >> 12; //@line 872 "synthdata.cpp"
          if ($shr11 == 0) {
            __label__ = 3;;
          }
          else if ($shr11 == 1) {
            __label__ = 21;;
          }
          else if ($shr11 == 2) {
            __label__ = 25;;
          }
          else if ($shr11 == 3) {
            __label__ = 25;;
          }
          else if ($shr11 == 6) {
            __label__ = 40;;
          }
          else if ($shr11 == 9) {
            __label__ = 44;;
          }
          else if ($shr11 == 10) {
            __label__ = 49;;
          }
          else if ($shr11 == 11) {
            __label__ = 53;;
          }
          else if ($shr11 == 12) {
            __label__ = 53;;
          }
          else if ($shr11 == 13) {
            __label__ = 53;;
          }
          else if ($shr11 == 14) {
            __label__ = 53;;
          }
          else if ($shr11 == 15) {
            __label__ = 53;;
          }
          else {
          __label__ = 61;;
          }
          
          $sw_default221$$sw_bb$$sw_bb70$$sw_bb84$$sw_bb129$$sw_bb138$$sw_bb155$$sw_bb178$6: do { 
            if (__label__ == 61) {
  
              var $153=$ph; //@line 1077 "synthdata.cpp"
              var $154=$instn; //@line 1077 "synthdata.cpp"
              var $conv222=((($154))&65535); //@line 1077 "synthdata.cpp"
              __ZL12InvalidInstnP11PHONEME_TABi($153, $conv222); //@line 1077 "synthdata.cpp"
              ; //@line 1078 "synthdata.cpp"
            }
            else if (__label__ == 3) {
  
              var $20=$instn; //@line 875 "synthdata.cpp"
              var $conv12=((($20))&65535); //@line 875 "synthdata.cpp"
              var $and13=($conv12) & 255; //@line 875 "synthdata.cpp"
              $data=$and13; //@line 875 "synthdata.cpp"
              var $21=$instn2; //@line 877 "synthdata.cpp"
              var $cmp14=((($21))|0)==0; //@line 877 "synthdata.cpp"
              if ($cmp14) { __label__ = 4;; } else { __label__ = 7;; } //@line 877 "synthdata.cpp"
              if (__label__ == 4) {
  
                var $22=$data; //@line 880 "synthdata.cpp"
                if ($22 == 1) {
                  __label__ = 5;;
                }
                else if ($22 == 2) {
                  __label__ = 62;break $sw_default221$$sw_bb$$sw_bb70$$sw_bb84$$sw_bb129$$sw_bb138$$sw_bb155$$sw_bb178$6;
                }
                else {
                __label__ = 6;;
                }
                
                if (__label__ == 6) {
  
                  var $23=$ph; //@line 890 "synthdata.cpp"
                  var $24=$instn; //@line 890 "synthdata.cpp"
                  var $conv17=((($24))&65535); //@line 890 "synthdata.cpp"
                  __ZL12InvalidInstnP11PHONEME_TABi($23, $conv17); //@line 890 "synthdata.cpp"
                  ; //@line 891 "synthdata.cpp"
                }
                else if (__label__ == 5) {
  
                  $end_flag=1; //@line 883 "synthdata.cpp"
                  ; //@line 884 "synthdata.cpp"
                }
              }
              else if (__label__ == 7) {
  
                var $25=$instn2; //@line 895 "synthdata.cpp"
                var $cmp18=((($25))|0)==5; //@line 895 "synthdata.cpp"
                if ($cmp18) { __label__ = 8;; } else { __label__ = 10;; } //@line 895 "synthdata.cpp"
                if (__label__ == 8) {
  
                  var $26=$plist_addr; //@line 897 "synthdata.cpp"
                  var $arrayidx20=(($26+24)&4294967295); //@line 897 "synthdata.cpp"
                  var $phcode=(($arrayidx20)&4294967295); //@line 897 "synthdata.cpp"
                  var $27=HEAPU8[($phcode)]; //@line 897 "synthdata.cpp"
                  var $idxprom21=((($27))&255); //@line 897 "synthdata.cpp"
                  var $arrayidx22=((_phoneme_tab+$idxprom21*4)&4294967295); //@line 897 "synthdata.cpp"
                  var $28=HEAP32[(($arrayidx22)>>2)]; //@line 897 "synthdata.cpp"
                  var $type=(($28+11)&4294967295); //@line 897 "synthdata.cpp"
                  var $29=HEAPU8[($type)]; //@line 897 "synthdata.cpp"
                  var $conv23=((($29))&255); //@line 897 "synthdata.cpp"
                  var $cmp24=((($conv23))|0)==2; //@line 897 "synthdata.cpp"
                  if (!($cmp24)) { __label__ = 62;break $sw_default221$$sw_bb$$sw_bb70$$sw_bb84$$sw_bb129$$sw_bb138$$sw_bb155$$sw_bb178$6; } //@line 897 "synthdata.cpp"
  
                  var $30=$data; //@line 898 "synthdata.cpp"
                  var $31=$phdata_addr; //@line 898 "synthdata.cpp"
                  var $pd_param26=(($31+4)&4294967295); //@line 898 "synthdata.cpp"
                  var $arrayidx27=(($pd_param26+16)&4294967295); //@line 898 "synthdata.cpp"
                  HEAP32[(($arrayidx27)>>2)]=$30; //@line 898 "synthdata.cpp"
                  ; //@line 898 "synthdata.cpp"
                }
                else if (__label__ == 10) {
  
                  var $32=$instn2; //@line 901 "synthdata.cpp"
                  var $cmp30=((($32))|0)==13; //@line 901 "synthdata.cpp"
                  if ($cmp30) { __label__ = 11;; } else { __label__ = 14;; } //@line 901 "synthdata.cpp"
                  if (__label__ == 11) {
  
                    $ix=0; //@line 904 "synthdata.cpp"
                    var $33=$ix; //@line 904 "synthdata.cpp"
                    var $34=$data; //@line 904 "synthdata.cpp"
                    var $cmp333=((($33))|0) < ((($34))|0); //@line 904 "synthdata.cpp"
                    var $35=$ix; //@line 904 "synthdata.cpp"
                    var $cmp344=((($35))|0) < 16; //@line 904 "synthdata.cpp"
                    var $or_cond15=($cmp333) & ($cmp344); //@line 904 "synthdata.cpp"
                    if ($or_cond15) { __label__ = 12;; } else { __label__ = 13;; } //@line 904 "synthdata.cpp"
                    $for_body35$$for_end$21: do { 
                      if (__label__ == 12) {
                        while(1) { 
  
                          var $36=HEAP32[(($prog)>>2)]; //@line 906 "synthdata.cpp"
                          var $incdec_ptr=(($36+2)&4294967295); //@line 906 "synthdata.cpp"
                          HEAP32[(($prog)>>2)]=$incdec_ptr; //@line 906 "synthdata.cpp"
                          var $37=HEAP32[(($prog)>>2)]; //@line 907 "synthdata.cpp"
                          var $arrayidx36=(($37)&4294967295); //@line 907 "synthdata.cpp"
                          var $38=HEAPU16[(($arrayidx36)>>1)]; //@line 907 "synthdata.cpp"
                          var $conv37=((($38))&65535); //@line 907 "synthdata.cpp"
                          var $shr38=($conv37) >> 8; //@line 907 "synthdata.cpp"
                          var $conv39=((($shr38)) & 255); //@line 907 "synthdata.cpp"
                          var $39=$ix; //@line 907 "synthdata.cpp"
                          var $40=$phdata_addr; //@line 907 "synthdata.cpp"
                          var $ipa_string=(($40+132)&4294967295); //@line 907 "synthdata.cpp"
                          var $arrayidx40=(($ipa_string+$39)&4294967295); //@line 907 "synthdata.cpp"
                          HEAP8[($arrayidx40)]=$conv39; //@line 907 "synthdata.cpp"
                          var $41=HEAP32[(($prog)>>2)]; //@line 908 "synthdata.cpp"
                          var $arrayidx41=(($41)&4294967295); //@line 908 "synthdata.cpp"
                          var $42=HEAPU16[(($arrayidx41)>>1)]; //@line 908 "synthdata.cpp"
                          var $conv42=((($42))&65535); //@line 908 "synthdata.cpp"
                          var $and43=($conv42) & 255; //@line 908 "synthdata.cpp"
                          var $conv44=((($and43)) & 255); //@line 908 "synthdata.cpp"
                          var $43=$ix; //@line 908 "synthdata.cpp"
                          var $add=((($43)+1)&4294967295); //@line 908 "synthdata.cpp"
                          var $44=$phdata_addr; //@line 908 "synthdata.cpp"
                          var $ipa_string45=(($44+132)&4294967295); //@line 908 "synthdata.cpp"
                          var $arrayidx46=(($ipa_string45+$add)&4294967295); //@line 908 "synthdata.cpp"
                          HEAP8[($arrayidx46)]=$conv44; //@line 908 "synthdata.cpp"
                          var $45=$ix; //@line 904 "synthdata.cpp"
                          var $add47=((($45)+2)&4294967295); //@line 904 "synthdata.cpp"
                          $ix=$add47; //@line 904 "synthdata.cpp"
                          var $46=$ix; //@line 904 "synthdata.cpp"
                          var $47=$data; //@line 904 "synthdata.cpp"
                          var $cmp33=((($46))|0) < ((($47))|0); //@line 904 "synthdata.cpp"
                          var $48=$ix; //@line 904 "synthdata.cpp"
                          var $cmp34=((($48))|0) < 16; //@line 904 "synthdata.cpp"
                          var $or_cond1=($cmp33) & ($cmp34); //@line 904 "synthdata.cpp"
                          if ($or_cond1) { __label__ = 12;continue ; } else { __label__ = 13;break $for_body35$$for_end$21; } //@line 904 "synthdata.cpp"
                        }
                      }
                    } while(0);
  
                    var $49=$ix; //@line 910 "synthdata.cpp"
                    var $50=$phdata_addr; //@line 910 "synthdata.cpp"
                    var $ipa_string48=(($50+132)&4294967295); //@line 910 "synthdata.cpp"
                    var $arrayidx49=(($ipa_string48+$49)&4294967295); //@line 910 "synthdata.cpp"
                    HEAP8[($arrayidx49)]=0; //@line 910 "synthdata.cpp"
                    ; //@line 911 "synthdata.cpp"
                  }
                  else if (__label__ == 14) {
  
                    var $51=$instn2; //@line 913 "synthdata.cpp"
                    var $cmp51=((($51))|0) < 16; //@line 913 "synthdata.cpp"
                    if ($cmp51) { __label__ = 15;; } else { __label__ = 20;; } //@line 913 "synthdata.cpp"
                    if (__label__ == 15) {
  
                      var $52=$instn2; //@line 915 "synthdata.cpp"
                      var $cmp53=((($52))|0)==12; //@line 915 "synthdata.cpp"
                      if ($cmp53) { __label__ = 16;; } else { __label__ = 17;; } //@line 915 "synthdata.cpp"
                      if (__label__ == 16) {
  
                        var $53=$data; //@line 917 "synthdata.cpp"
                        var $54=$phdata_addr; //@line 917 "synthdata.cpp"
                        var $pd_param55=(($54+4)&4294967295); //@line 917 "synthdata.cpp"
                        var $arrayidx56=(($pd_param55+4)&4294967295); //@line 917 "synthdata.cpp"
                        HEAP32[(($arrayidx56)>>2)]=$53; //@line 917 "synthdata.cpp"
                        ; //@line 918 "synthdata.cpp"
                      }
  
                      var $55=$data; //@line 919 "synthdata.cpp"
                      var $56=$instn2; //@line 919 "synthdata.cpp"
                      var $57=$phdata_addr; //@line 919 "synthdata.cpp"
                      var $pd_param58=(($57+4)&4294967295); //@line 919 "synthdata.cpp"
                      var $arrayidx59=(($pd_param58+$56*4)&4294967295); //@line 919 "synthdata.cpp"
                      HEAP32[(($arrayidx59)>>2)]=$55; //@line 919 "synthdata.cpp"
                      var $58=$instn2; //@line 920 "synthdata.cpp"
                      var $cmp60=((($58))|0)==1; //@line 920 "synthdata.cpp"
                      if (!($cmp60)) { __label__ = 62;break $sw_default221$$sw_bb$$sw_bb70$$sw_bb84$$sw_bb129$$sw_bb138$$sw_bb155$$sw_bb178$6; } //@line 920 "synthdata.cpp"
  
                      var $59=$control_addr; //@line 920 "synthdata.cpp"
                      var $and61=($59) & 256; //@line 920 "synthdata.cpp"
                      var $tobool=((($and61))|0)!=0; //@line 920 "synthdata.cpp"
                      if (!($tobool)) { __label__ = 62;break $sw_default221$$sw_bb$$sw_bb70$$sw_bb84$$sw_bb129$$sw_bb138$$sw_bb155$$sw_bb178$6; } //@line 920 "synthdata.cpp"
  
                      $end_flag=1; //@line 923 "synthdata.cpp"
                      ; //@line 924 "synthdata.cpp"
                    }
                    else if (__label__ == 20) {
  
                      var $60=$ph; //@line 928 "synthdata.cpp"
                      var $61=$instn; //@line 928 "synthdata.cpp"
                      var $conv65=((($61))&65535); //@line 928 "synthdata.cpp"
                      __ZL12InvalidInstnP11PHONEME_TABi($60, $conv65); //@line 928 "synthdata.cpp"
                      ;
                    }
                  }
                }
              }
            }
            else if (__label__ == 21) {
  
              var $62=$tr_addr; //@line 933 "synthdata.cpp"
              var $cmp71=((($62))|0)==0; //@line 933 "synthdata.cpp"
              if ($cmp71) { __label__ = 62;break $sw_default221$$sw_bb$$sw_bb70$$sw_bb84$$sw_bb129$$sw_bb138$$sw_bb155$$sw_bb178$6; } //@line 933 "synthdata.cpp"
  
              var $63=$instn2; //@line 936 "synthdata.cpp"
              var $cmp74=((($63))|0) < 8; //@line 936 "synthdata.cpp"
              if (!($cmp74)) { __label__ = 62;break $sw_default221$$sw_bb$$sw_bb70$$sw_bb84$$sw_bb129$$sw_bb138$$sw_bb155$$sw_bb178$6; } //@line 936 "synthdata.cpp"
  
              var $64=$tr_addr; //@line 939 "synthdata.cpp"
              var $65=$plist_addr; //@line 939 "synthdata.cpp"
              var $66=$instn2; //@line 939 "synthdata.cpp"
              var $and76=($66) & 7; //@line 939 "synthdata.cpp"
              var $call=__ZL15StressConditionP10TranslatorP12PHONEME_LISTii($64, $65, $and76, 1); //@line 939 "synthdata.cpp"
              if (!($call)) { __label__ = 62;break $sw_default221$$sw_bb$$sw_bb70$$sw_bb84$$sw_bb129$$sw_bb138$$sw_bb155$$sw_bb178$6; } //@line 939 "synthdata.cpp"
  
              var $67=$instn; //@line 941 "synthdata.cpp"
              var $conv78=((($67))&65535); //@line 941 "synthdata.cpp"
              var $and79=($conv78) & 255; //@line 941 "synthdata.cpp"
              var $68=$phdata_addr; //@line 941 "synthdata.cpp"
              var $pd_param80=(($68+4)&4294967295); //@line 941 "synthdata.cpp"
              var $arrayidx81=(($pd_param80+4)&4294967295); //@line 941 "synthdata.cpp"
              HEAP32[(($arrayidx81)>>2)]=$and79; //@line 941 "synthdata.cpp"
              $end_flag=1; //@line 942 "synthdata.cpp"
              ; //@line 943 "synthdata.cpp"
            }
            else if (__label__ == 25) {
  
              $or_flag=0; //@line 950 "synthdata.cpp"
              $truth=1; //@line 951 "synthdata.cpp"
              var $69=$instn; //@line 952 "synthdata.cpp"
              var $conv856=((($69))&65535); //@line 952 "synthdata.cpp"
              var $and867=($conv856) & 57344; //@line 952 "synthdata.cpp"
              var $cmp878=((($and867))|0)==8192; //@line 952 "synthdata.cpp"
              if ($cmp878) { __label__ = 26;; } else { __label__ = 34;; } //@line 952 "synthdata.cpp"
              $while_body$$while_end$39: do { 
                if (__label__ == 26) {
                  while(1) { 
  
                    var $70=$or_flag; //@line 955 "synthdata.cpp"
                    var $tobool88=((($70))|0)!=0; //@line 955 "synthdata.cpp"
                    var $71=$truth; //@line 956 "synthdata.cpp"
                    var $tobool90=((($71)) & 1); //@line 956 "synthdata.cpp"
                    if ($tobool88) { __label__ = 27;; } else { __label__ = 30;; } //@line 955 "synthdata.cpp"
                    if (__label__ == 27) {
  
                      if ($tobool90) { __lastLabel__ = 27; __label__ = 29;; } else { __lastLabel__ = 27; __label__ = 28;; } //@line 956 "synthdata.cpp"
                      if (__label__ == 28) {
  
                        var $72=$tr_addr; //@line 956 "synthdata.cpp"
                        var $73=$control_addr; //@line 956 "synthdata.cpp"
                        var $74=$plist_addr; //@line 956 "synthdata.cpp"
                        var $75=$instn; //@line 956 "synthdata.cpp"
                        var $conv91=((($75))&65535); //@line 956 "synthdata.cpp"
                        var $and92=($conv91) & 4095; //@line 956 "synthdata.cpp"
                        var $call93=__ZL18InterpretConditionP10TranslatoriP12PHONEME_LISTi($72, $73, $74, $and92); //@line 956 "synthdata.cpp"
                        __lastLabel__ = 28; ; //@line 956 "synthdata.cpp"
                      }
  
                      var $76=__lastLabel__ == 27 ? 1 : ($call93);
                      var $frombool=((($76))&1); //@line 956 "synthdata.cpp"
                      $truth=$frombool; //@line 956 "synthdata.cpp"
                      ; //@line 956 "synthdata.cpp"
                    }
                    else if (__label__ == 30) {
  
                      if ($tobool90) { __lastLabel__ = 30; __label__ = 31;; } else { __lastLabel__ = 30; __label__ = 32;; } //@line 958 "synthdata.cpp"
                      if (__label__ == 31) {
  
                        var $77=$tr_addr; //@line 958 "synthdata.cpp"
                        var $78=$control_addr; //@line 958 "synthdata.cpp"
                        var $79=$plist_addr; //@line 958 "synthdata.cpp"
                        var $80=$instn; //@line 958 "synthdata.cpp"
                        var $conv97=((($80))&65535); //@line 958 "synthdata.cpp"
                        var $and98=($conv97) & 4095; //@line 958 "synthdata.cpp"
                        var $call99=__ZL18InterpretConditionP10TranslatoriP12PHONEME_LISTi($77, $78, $79, $and98); //@line 958 "synthdata.cpp"
                        __lastLabel__ = 31; ;
                      }
  
                      var $81=__lastLabel__ == 30 ? 0 : ($call99);
                      var $frombool101=((($81))&1);
                      $truth=$frombool101;
                      ;
                    }
  
                    var $82=$instn; //@line 959 "synthdata.cpp"
                    var $conv103=((($82))&65535); //@line 959 "synthdata.cpp"
                    var $and104=($conv103) & 4096; //@line 959 "synthdata.cpp"
                    $or_flag=$and104; //@line 959 "synthdata.cpp"
                    var $83=HEAP32[(($prog)>>2)]; //@line 960 "synthdata.cpp"
                    var $incdec_ptr105=(($83+2)&4294967295); //@line 960 "synthdata.cpp"
                    HEAP32[(($prog)>>2)]=$incdec_ptr105; //@line 960 "synthdata.cpp"
                    var $84=HEAP16[(($incdec_ptr105)>>1)]; //@line 960 "synthdata.cpp"
                    $instn=$84; //@line 960 "synthdata.cpp"
                    var $85=$instn; //@line 952 "synthdata.cpp"
                    var $conv85=((($85))&65535); //@line 952 "synthdata.cpp"
                    var $and86=($conv85) & 57344; //@line 952 "synthdata.cpp"
                    var $cmp87=((($and86))|0)==8192; //@line 952 "synthdata.cpp"
                    if ($cmp87) { __label__ = 26;continue ; } else { __label__ = 34;break $while_body$$while_end$39; } //@line 952 "synthdata.cpp"
                  }
                }
              } while(0);
  
              var $86=$truth; //@line 963 "synthdata.cpp"
              var $tobool106=((($86)) & 1); //@line 963 "synthdata.cpp"
              var $conv107=((($tobool106))&1); //@line 963 "synthdata.cpp"
              var $cmp108=((($conv107))|0)==0; //@line 963 "synthdata.cpp"
              if ($cmp108) { __label__ = 35;; } else { __label__ = 39;; } //@line 963 "synthdata.cpp"
              $if_then109$$if_end127$53: do { 
                if (__label__ == 35) {
  
                  var $87=$instn; //@line 965 "synthdata.cpp"
                  var $conv110=((($87))&65535); //@line 965 "synthdata.cpp"
                  var $and111=($conv110) & 63488; //@line 965 "synthdata.cpp"
                  var $cmp112=((($and111))|0)==26624; //@line 965 "synthdata.cpp"
                  if ($cmp112) { __label__ = 36;; } else { __label__ = 37;; } //@line 965 "synthdata.cpp"
                  if (__label__ == 36) {
  
                    var $88=$instn; //@line 967 "synthdata.cpp"
                    var $conv114=((($88))&65535); //@line 967 "synthdata.cpp"
                    var $and115=($conv114) & 255; //@line 967 "synthdata.cpp"
                    var $89=HEAP32[(($prog)>>2)]; //@line 967 "synthdata.cpp"
                    var $add_ptr=(($89+2*$and115)&4294967295); //@line 967 "synthdata.cpp"
                    HEAP32[(($prog)>>2)]=$add_ptr; //@line 967 "synthdata.cpp"
                    ; //@line 968 "synthdata.cpp"
                  }
                  else if (__label__ == 37) {
  
                    var $90=HEAP32[(($prog)>>2)]; //@line 972 "synthdata.cpp"
                    var $call117=__ZL13NumInstnWordsPt($90); //@line 972 "synthdata.cpp"
                    var $91=HEAP32[(($prog)>>2)]; //@line 972 "synthdata.cpp"
                    var $add_ptr118=(($91+2*$call117)&4294967295); //@line 972 "synthdata.cpp"
                    HEAP32[(($prog)>>2)]=$add_ptr118; //@line 972 "synthdata.cpp"
                    var $92=HEAP32[(($prog)>>2)]; //@line 973 "synthdata.cpp"
                    var $arrayidx119=(($92)&4294967295); //@line 973 "synthdata.cpp"
                    var $93=HEAPU16[(($arrayidx119)>>1)]; //@line 973 "synthdata.cpp"
                    var $conv120=((($93))&65535); //@line 973 "synthdata.cpp"
                    var $and121=($conv120) & 65024; //@line 973 "synthdata.cpp"
                    var $cmp122=((($and121))|0)==24576; //@line 973 "synthdata.cpp"
                    if (!($cmp122)) { __label__ = 39;break $if_then109$$if_end127$53; } //@line 973 "synthdata.cpp"
  
                    var $94=HEAP32[(($prog)>>2)]; //@line 974 "synthdata.cpp"
                    var $incdec_ptr124=(($94+2)&4294967295); //@line 974 "synthdata.cpp"
                    HEAP32[(($prog)>>2)]=$incdec_ptr124; //@line 974 "synthdata.cpp"
                    ; //@line 974 "synthdata.cpp"
                  }
                }
              } while(0);
  
              var $95=HEAP32[(($prog)>>2)]; //@line 977 "synthdata.cpp"
              var $incdec_ptr128=(($95+-2)&4294967295); //@line 977 "synthdata.cpp"
              HEAP32[(($prog)>>2)]=$incdec_ptr128; //@line 977 "synthdata.cpp"
              ; //@line 978 "synthdata.cpp"
            }
            else if (__label__ == 40) {
  
              var $96=$instn2; //@line 982 "synthdata.cpp"
              var $shr130=($96) >> 1; //@line 982 "synthdata.cpp"
              if ($shr130 == 0) {
                __label__ = 41;;
              }
              else if ($shr130 == 6) {
                __label__ = 43;;
              }
              else if ($shr130 == 5) {
                __label__ = 42;;
              }
              else {
              __label__ = 62;break $sw_default221$$sw_bb$$sw_bb70$$sw_bb84$$sw_bb129$$sw_bb138$$sw_bb155$$sw_bb178$6;
              }
              
              if (__label__ == 41) {
  
                var $97=$instn; //@line 985 "synthdata.cpp"
                var $conv132=((($97))&65535); //@line 985 "synthdata.cpp"
                var $and133=($conv132) & 255; //@line 985 "synthdata.cpp"
                var $sub=((($and133)-1)&4294967295); //@line 985 "synthdata.cpp"
                var $98=HEAP32[(($prog)>>2)]; //@line 985 "synthdata.cpp"
                var $add_ptr134=(($98+2*$sub)&4294967295); //@line 985 "synthdata.cpp"
                HEAP32[(($prog)>>2)]=$add_ptr134; //@line 985 "synthdata.cpp"
                ; //@line 986 "synthdata.cpp"
              }
              else if (__label__ == 43) {
  
                var $101=$plist_addr; //@line 997 "synthdata.cpp"
                var $102=$phdata_addr; //@line 997 "synthdata.cpp"
                __ZL15SwitchVowelTypeP12PHONEME_LISTP12PHONEME_DATAPPti($101, $102, $prog, 3); //@line 997 "synthdata.cpp"
                ; //@line 998 "synthdata.cpp"
              }
              else if (__label__ == 42) {
  
                var $99=$plist_addr; //@line 993 "synthdata.cpp"
                var $100=$phdata_addr; //@line 993 "synthdata.cpp"
                __ZL15SwitchVowelTypeP12PHONEME_LISTP12PHONEME_DATAPPti($99, $100, $prog, 2); //@line 993 "synthdata.cpp"
                ; //@line 994 "synthdata.cpp"
              }
            }
            else if (__label__ == 44) {
  
              var $103=$instn; //@line 1003 "synthdata.cpp"
              var $conv139=((($103))&65535); //@line 1003 "synthdata.cpp"
              var $and140=($conv139) & 15; //@line 1003 "synthdata.cpp"
              var $shl=((($and140)*65536)&4294967295); //@line 1003 "synthdata.cpp"
              var $104=HEAP32[(($prog)>>2)]; //@line 1003 "synthdata.cpp"
              var $arrayidx141=(($104+2)&4294967295); //@line 1003 "synthdata.cpp"
              var $105=HEAPU16[(($arrayidx141)>>1)]; //@line 1003 "synthdata.cpp"
              var $conv142=((($105))&65535); //@line 1003 "synthdata.cpp"
              var $add143=((($shl)+($conv142))&4294967295); //@line 1003 "synthdata.cpp"
              $data=$add143; //@line 1003 "synthdata.cpp"
              var $106=HEAP32[(($prog)>>2)]; //@line 1004 "synthdata.cpp"
              var $incdec_ptr144=(($106+2)&4294967295); //@line 1004 "synthdata.cpp"
              HEAP32[(($prog)>>2)]=$incdec_ptr144; //@line 1004 "synthdata.cpp"
              var $107=$instn2; //@line 1005 "synthdata.cpp"
              if ($107 == 1) {
                __label__ = 45;;
              }
              else if ($107 == 2) {
                __label__ = 47;;
              }
              else if ($107 == 3) {
                __label__ = 48;;
              }
              else {
              __label__ = 62;break $sw_default221$$sw_bb$$sw_bb70$$sw_bb84$$sw_bb129$$sw_bb138$$sw_bb155$$sw_bb178$6;
              }
              
              if (__label__ == 45) {
  
                var $108=$n_return; //@line 1009 "synthdata.cpp"
                var $cmp146=((($108))|0) < 10; //@line 1009 "synthdata.cpp"
                if (!($cmp146)) { __label__ = 62;break $sw_default221$$sw_bb$$sw_bb70$$sw_bb84$$sw_bb129$$sw_bb138$$sw_bb155$$sw_bb178$6; } //@line 1009 "synthdata.cpp"
  
                var $109=HEAP32[(($prog)>>2)]; //@line 1011 "synthdata.cpp"
                var $110=$n_return; //@line 1011 "synthdata.cpp"
                var $inc=((($110)+1)&4294967295); //@line 1011 "synthdata.cpp"
                $n_return=$inc; //@line 1011 "synthdata.cpp"
                var $arrayidx148=(($return_addr+$110*4)&4294967295); //@line 1011 "synthdata.cpp"
                HEAP32[(($arrayidx148)>>2)]=$109; //@line 1011 "synthdata.cpp"
                var $111=$data; //@line 1012 "synthdata.cpp"
                var $112=HEAP32[((_phoneme_index)>>2)]; //@line 1012 "synthdata.cpp"
                var $arrayidx149=(($112+2*$111)&4294967295); //@line 1012 "synthdata.cpp"
                var $add_ptr150=(($arrayidx149+-2)&4294967295); //@line 1012 "synthdata.cpp"
                HEAP32[(($prog)>>2)]=$add_ptr150; //@line 1012 "synthdata.cpp"
                ; //@line 1013 "synthdata.cpp"
              }
              else if (__label__ == 47) {
  
                var $113=$data; //@line 1018 "synthdata.cpp"
                var $114=$phdata_addr; //@line 1018 "synthdata.cpp"
                var $pitch_env=(($114+124)&4294967295); //@line 1018 "synthdata.cpp"
                HEAP32[(($pitch_env)>>2)]=$113; //@line 1018 "synthdata.cpp"
                ; //@line 1019 "synthdata.cpp"
              }
              else if (__label__ == 48) {
  
                var $115=$data; //@line 1023 "synthdata.cpp"
                var $116=$phdata_addr; //@line 1023 "synthdata.cpp"
                var $amp_env=(($116+128)&4294967295); //@line 1023 "synthdata.cpp"
                HEAP32[(($amp_env)>>2)]=$115; //@line 1023 "synthdata.cpp"
                ; //@line 1024 "synthdata.cpp"
              }
            }
            else if (__label__ == 49) {
  
              var $117=$instn2; //@line 1029 "synthdata.cpp"
              var $cmp156=((($117))|0)==1; //@line 1029 "synthdata.cpp"
              if ($cmp156) { __label__ = 50;; } else { __label__ = 51;; } //@line 1029 "synthdata.cpp"
              if (__label__ == 50) {
  
                $ix=0; //@line 1030 "synthdata.cpp"
                ; //@line 1030 "synthdata.cpp"
              }
              else if (__label__ == 51) {
  
                $ix=2; //@line 1032 "synthdata.cpp"
                ;
              }
  
              var $118=HEAP32[(($prog)>>2)]; //@line 1034 "synthdata.cpp"
              var $arrayidx160=(($118)&4294967295); //@line 1034 "synthdata.cpp"
              var $119=HEAPU16[(($arrayidx160)>>1)]; //@line 1034 "synthdata.cpp"
              var $conv161=((($119))&65535); //@line 1034 "synthdata.cpp"
              var $and162=($conv161) & 255; //@line 1034 "synthdata.cpp"
              var $shl163=((($and162)*65536)&4294967295); //@line 1034 "synthdata.cpp"
              var $120=HEAP32[(($prog)>>2)]; //@line 1034 "synthdata.cpp"
              var $arrayidx164=(($120+2)&4294967295); //@line 1034 "synthdata.cpp"
              var $121=HEAPU16[(($arrayidx164)>>1)]; //@line 1034 "synthdata.cpp"
              var $conv165=((($121))&65535); //@line 1034 "synthdata.cpp"
              var $add166=((($shl163)+($conv165))&4294967295); //@line 1034 "synthdata.cpp"
              var $122=$ix; //@line 1034 "synthdata.cpp"
              var $123=$phdata_addr; //@line 1034 "synthdata.cpp"
              var $vowel_transition=(($123+108)&4294967295); //@line 1034 "synthdata.cpp"
              var $arrayidx167=(($vowel_transition+$122*4)&4294967295); //@line 1034 "synthdata.cpp"
              HEAP32[(($arrayidx167)>>2)]=$add166; //@line 1034 "synthdata.cpp"
              var $124=HEAP32[(($prog)>>2)]; //@line 1035 "synthdata.cpp"
              var $arrayidx168=(($124+4)&4294967295); //@line 1035 "synthdata.cpp"
              var $125=HEAPU16[(($arrayidx168)>>1)]; //@line 1035 "synthdata.cpp"
              var $conv169=((($125))&65535); //@line 1035 "synthdata.cpp"
              var $shl170=((($conv169)*65536)&4294967295); //@line 1035 "synthdata.cpp"
              var $126=HEAP32[(($prog)>>2)]; //@line 1035 "synthdata.cpp"
              var $arrayidx171=(($126+6)&4294967295); //@line 1035 "synthdata.cpp"
              var $127=HEAPU16[(($arrayidx171)>>1)]; //@line 1035 "synthdata.cpp"
              var $conv172=((($127))&65535); //@line 1035 "synthdata.cpp"
              var $add173=((($conv172)+($shl170))&4294967295); //@line 1035 "synthdata.cpp"
              var $128=$ix; //@line 1035 "synthdata.cpp"
              var $add174=((($128)+1)&4294967295); //@line 1035 "synthdata.cpp"
              var $129=$phdata_addr; //@line 1035 "synthdata.cpp"
              var $vowel_transition175=(($129+108)&4294967295); //@line 1035 "synthdata.cpp"
              var $arrayidx176=(($vowel_transition175+$add174*4)&4294967295); //@line 1035 "synthdata.cpp"
              HEAP32[(($arrayidx176)>>2)]=$add173; //@line 1035 "synthdata.cpp"
              var $130=HEAP32[(($prog)>>2)]; //@line 1036 "synthdata.cpp"
              var $add_ptr177=(($130+6)&4294967295); //@line 1036 "synthdata.cpp"
              HEAP32[(($prog)>>2)]=$add_ptr177; //@line 1036 "synthdata.cpp"
              ; //@line 1037 "synthdata.cpp"
            }
            else if (__label__ == 53) {
  
              var $131=$instn; //@line 1044 "synthdata.cpp"
              var $conv179=((($131))&65535); //@line 1044 "synthdata.cpp"
              var $shr180=($conv179) >> 12; //@line 1044 "synthdata.cpp"
              var $sub181=((($shr180)-11)&4294967295); //@line 1044 "synthdata.cpp"
              $instn2=$sub181; //@line 1044 "synthdata.cpp"
              var $132=$instn; //@line 1045 "synthdata.cpp"
              var $conv182=((($132))&65535); //@line 1045 "synthdata.cpp"
              var $and183=($conv182) & 15; //@line 1045 "synthdata.cpp"
              var $shl184=((($and183)*262144)&4294967295); //@line 1045 "synthdata.cpp"
              var $133=HEAP32[(($prog)>>2)]; //@line 1045 "synthdata.cpp"
              var $arrayidx185=(($133+2)&4294967295); //@line 1045 "synthdata.cpp"
              var $134=HEAPU16[(($arrayidx185)>>1)]; //@line 1045 "synthdata.cpp"
              var $conv186=((($134))&65535); //@line 1045 "synthdata.cpp"
              var $shl187=((($conv186)*4)&4294967295); //@line 1045 "synthdata.cpp"
              var $add188=((($shl187)+($shl184))&4294967295); //@line 1045 "synthdata.cpp"
              var $135=$instn2; //@line 1045 "synthdata.cpp"
              var $136=$phdata_addr; //@line 1045 "synthdata.cpp"
              var $sound_addr=(($136+68)&4294967295); //@line 1045 "synthdata.cpp"
              var $arrayidx189=(($sound_addr+$135*4)&4294967295); //@line 1045 "synthdata.cpp"
              HEAP32[(($arrayidx189)>>2)]=$add188; //@line 1045 "synthdata.cpp"
              var $137=$instn; //@line 1046 "synthdata.cpp"
              var $conv190=((($137))&65535); //@line 1046 "synthdata.cpp"
              var $shr191=($conv190) >> 4; //@line 1046 "synthdata.cpp"
              var $and192=($shr191) & 255; //@line 1046 "synthdata.cpp"
              var $138=$instn2; //@line 1046 "synthdata.cpp"
              var $139=$phdata_addr; //@line 1046 "synthdata.cpp"
              var $sound_param=(($139+88)&4294967295); //@line 1046 "synthdata.cpp"
              var $arrayidx193=(($sound_param+$138*4)&4294967295); //@line 1046 "synthdata.cpp"
              HEAP32[(($arrayidx193)>>2)]=$and192; //@line 1046 "synthdata.cpp"
              var $conv194=((($and192)) & 255); //@line 1046 "synthdata.cpp"
              $param_sc=$conv194; //@line 1046 "synthdata.cpp"
              var $140=HEAP32[(($prog)>>2)]; //@line 1047 "synthdata.cpp"
              var $incdec_ptr195=(($140+2)&4294967295); //@line 1047 "synthdata.cpp"
              HEAP32[(($prog)>>2)]=$incdec_ptr195; //@line 1047 "synthdata.cpp"
              var $141=HEAP32[(($prog)>>2)]; //@line 1049 "synthdata.cpp"
              var $arrayidx196=(($141+2)&4294967295); //@line 1049 "synthdata.cpp"
              var $142=HEAPU16[(($arrayidx196)>>1)]; //@line 1049 "synthdata.cpp"
              var $conv197=((($142))&65535); //@line 1049 "synthdata.cpp"
              var $cmp198=((($conv197))|0)!=2; //@line 1049 "synthdata.cpp"
              if (!($cmp198)) { __label__ = 62;break $sw_default221$$sw_bb$$sw_bb70$$sw_bb84$$sw_bb129$$sw_bb138$$sw_bb155$$sw_bb178$6; } //@line 1049 "synthdata.cpp"
  
              var $143=$instn2; //@line 1051 "synthdata.cpp"
              var $cmp200=((($143))|0) < 2; //@line 1051 "synthdata.cpp"
              if ($cmp200) { __label__ = 55;; } else { __label__ = 57;; } //@line 1051 "synthdata.cpp"
              $if_then201$$if_else208$78: do { 
                if (__label__ == 55) {
  
                  $end_flag=1; //@line 1054 "synthdata.cpp"
                  var $144=HEAP32[(($prog)>>2)]; //@line 1055 "synthdata.cpp"
                  var $arrayidx202=(($144+2)&4294967295); //@line 1055 "synthdata.cpp"
                  var $145=HEAPU16[(($arrayidx202)>>1)]; //@line 1055 "synthdata.cpp"
                  var $conv203=((($145))&65535); //@line 1055 "synthdata.cpp"
                  var $shr204=($conv203) >> 12; //@line 1055 "synthdata.cpp"
                  var $cmp205=((($shr204))|0)==15; //@line 1055 "synthdata.cpp"
                  if (!($cmp205)) { __label__ = 59;break $if_then201$$if_else208$78; } //@line 1055 "synthdata.cpp"
  
                  $end_flag=2; //@line 1058 "synthdata.cpp"
                  ; //@line 1059 "synthdata.cpp"
                }
                else if (__label__ == 57) {
  
                  var $146=$instn2; //@line 1062 "synthdata.cpp"
                  var $cmp209=((($146))|0)==4; //@line 1062 "synthdata.cpp"
                  if (!($cmp209)) { __label__ = 59;break $if_then201$$if_else208$78; } //@line 1062 "synthdata.cpp"
  
                  var $147=$end_flag; //@line 1065 "synthdata.cpp"
                  var $dec=((($147)-1)&4294967295); //@line 1065 "synthdata.cpp"
                  $end_flag=$dec; //@line 1065 "synthdata.cpp"
                  ; //@line 1066 "synthdata.cpp"
                }
              } while(0);
  
              var $148=$instn2; //@line 1068 "synthdata.cpp"
              var $cmp213=((($148))|0)==2; //@line 1068 "synthdata.cpp"
              var $149=$instn2; //@line 1068 "synthdata.cpp"
              var $cmp214=((($149))|0)==3; //@line 1068 "synthdata.cpp"
              var $or_cond=($cmp213) | ($cmp214); //@line 1068 "synthdata.cpp"
              if (!($or_cond)) { __label__ = 62;break $sw_default221$$sw_bb$$sw_bb70$$sw_bb84$$sw_bb129$$sw_bb138$$sw_bb155$$sw_bb178$6; } //@line 1068 "synthdata.cpp"
  
              var $150=$param_sc; //@line 1071 "synthdata.cpp"
              var $conv216=(tempInt=(($150)),(tempInt>=128?tempInt-256:tempInt)); //@line 1071 "synthdata.cpp"
              var $151=$instn2; //@line 1071 "synthdata.cpp"
              var $152=$phdata_addr; //@line 1071 "synthdata.cpp"
              var $sound_param217=(($152+88)&4294967295); //@line 1071 "synthdata.cpp"
              var $arrayidx218=(($sound_param217+$151*4)&4294967295); //@line 1071 "synthdata.cpp"
              HEAP32[(($arrayidx218)>>2)]=$conv216; //@line 1071 "synthdata.cpp"
              ; //@line 1072 "synthdata.cpp"
            }
          } while(0);
  
          var $155=$ph; //@line 1081 "synthdata.cpp"
          var $phflags=(($155+4)&4294967295); //@line 1081 "synthdata.cpp"
          var $156=HEAP32[(($phflags)>>2)]; //@line 1081 "synthdata.cpp"
          var $and224=($156) & 4096; //@line 1081 "synthdata.cpp"
          var $tobool225=((($and224))|0)!=0; //@line 1081 "synthdata.cpp"
          if ($tobool225) { __label__ = 63;; } else { __label__ = 64;; } //@line 1081 "synthdata.cpp"
          $if_end227_thread$$if_end227$86: do { 
            if (__label__ == 63) {
  
              $end_flag=1; //@line 1083 "synthdata.cpp"
              ; //@line 1086 "synthdata.cpp"
            }
            else if (__label__ == 64) {
  
              var $_pr=$end_flag; //@line 1086 "synthdata.cpp"
              var $cmp228=((($_pr))|0)==1; //@line 1086 "synthdata.cpp"
              if ($cmp228) { __label__ = 66;break $if_end227_thread$$if_end227$86; } //@line 1086 "synthdata.cpp"
  
              var $157=HEAP32[(($prog)>>2)]; //@line 866 "synthdata.cpp"
              var $incdec_ptr23611=(($157+2)&4294967295); //@line 866 "synthdata.cpp"
              HEAP32[(($prog)>>2)]=$incdec_ptr23611; //@line 866 "synthdata.cpp"
              __label__ = 2;continue $for_body$4; //@line 866 "synthdata.cpp"
            }
          } while(0);
  
          var $158=$n_return; //@line 1086 "synthdata.cpp"
          var $cmp230=((($158))|0) > 0; //@line 1086 "synthdata.cpp"
          if ($cmp230) { __label__ = 67;; } else { __label__ = 68;; } //@line 1086 "synthdata.cpp"
          if (__label__ == 67) {
  
            $end_flag=0; //@line 1089 "synthdata.cpp"
            var $159=$n_return; //@line 1090 "synthdata.cpp"
            var $dec232=((($159)-1)&4294967295); //@line 1090 "synthdata.cpp"
            $n_return=$dec232; //@line 1090 "synthdata.cpp"
            var $arrayidx233=(($return_addr+$dec232*4)&4294967295); //@line 1090 "synthdata.cpp"
            var $160=HEAP32[(($arrayidx233)>>2)]; //@line 1090 "synthdata.cpp"
            HEAP32[(($prog)>>2)]=$160; //@line 1090 "synthdata.cpp"
            ; //@line 1091 "synthdata.cpp"
          }
  
          var $_pr10=$end_flag; //@line 866 "synthdata.cpp"
          var $161=HEAP32[(($prog)>>2)]; //@line 866 "synthdata.cpp"
          var $incdec_ptr236=(($161+2)&4294967295); //@line 866 "synthdata.cpp"
          HEAP32[(($prog)>>2)]=$incdec_ptr236; //@line 866 "synthdata.cpp"
          var $cmp8=((($_pr10))|0)!=1; //@line 866 "synthdata.cpp"
          if ($cmp8) { __label__ = 2;continue $for_body$4; } else { __label__ = 69;break $for_end237$$if_end$2; } //@line 866 "synthdata.cpp"
        }
      }
    } while(0);
  
    STACKTOP = __stackBase__;
    return; //@line 1094 "synthdata.cpp"
    return;
  }
  

  function __ZL12InvalidInstnP11PHONEME_TABi($ph, $instn) {
    ;
    var __label__;
  
    var $ph_addr;
    var $instn_addr;
    $ph_addr=$ph;
    $instn_addr=$instn;
    var $0=HEAP32[((__impure_ptr)>>2)]; //@line 525 "synthdata.cpp"
    var $_stderr=(($0+12)&4294967295); //@line 525 "synthdata.cpp"
    var $1=HEAP32[(($_stderr)>>2)]; //@line 525 "synthdata.cpp"
    var $2=$instn_addr; //@line 525 "synthdata.cpp"
    var $3=$ph_addr; //@line 525 "synthdata.cpp"
    var $mnemonic=(($3)&4294967295); //@line 525 "synthdata.cpp"
    var $4=HEAP32[(($mnemonic)>>2)]; //@line 525 "synthdata.cpp"
    var $call=__Z12WordToStringj($4); //@line 525 "synthdata.cpp"
    var $call1=_fprintf($1, ((__str17428)&4294967295), allocate([$2,0,0,0,((__ZZ12WordToStringjE3buf)&4294967295),0,0,0], ["i32",0,0,0,"i8*",0,0,0], ALLOC_STACK)); //@line 525 "synthdata.cpp"
    ;
    return; //@line 526 "synthdata.cpp"
    return;
  }
  

  function __ZL15StressConditionP10TranslatorP12PHONEME_LISTii($tr, $plist, $condition, $control) {
    ;
    var __label__;
  
    var $retval;
    var $tr_addr;
    var $plist_addr;
    var $condition_addr;
    var $control_addr;
    var $stress_level;
    var $pl;
    $tr_addr=$tr;
    $plist_addr=$plist;
    $condition_addr=$condition;
    $control_addr=$control;
    var $0=$plist_addr; //@line 538 "synthdata.cpp"
    var $arrayidx=(($0)&4294967295); //@line 538 "synthdata.cpp"
    var $phcode=(($arrayidx)&4294967295); //@line 538 "synthdata.cpp"
    var $1=HEAPU8[($phcode)]; //@line 538 "synthdata.cpp"
    var $idxprom=((($1))&255); //@line 538 "synthdata.cpp"
    var $arrayidx1=((_phoneme_tab+$idxprom*4)&4294967295); //@line 538 "synthdata.cpp"
    var $2=HEAP32[(($arrayidx1)>>2)]; //@line 538 "synthdata.cpp"
    var $type=(($2+11)&4294967295); //@line 538 "synthdata.cpp"
    var $3=HEAPU8[($type)]; //@line 538 "synthdata.cpp"
    var $conv=((($3))&255); //@line 538 "synthdata.cpp"
    var $cmp=((($conv))|0)==2; //@line 538 "synthdata.cpp"
    var $4=$plist_addr; //@line 540 "synthdata.cpp"
    if ($cmp) { __label__ = 1;; } else { __label__ = 2;; } //@line 538 "synthdata.cpp"
    $if_then$$if_else$2: do { 
      if (__label__ == 1) {
  
        $pl=$4; //@line 540 "synthdata.cpp"
        __label__ = 5;break $if_then$$if_else$2; //@line 541 "synthdata.cpp"
      }
      else if (__label__ == 2) {
  
        var $arrayidx2=(($4+24)&4294967295); //@line 545 "synthdata.cpp"
        var $phcode3=(($arrayidx2)&4294967295); //@line 545 "synthdata.cpp"
        var $5=HEAPU8[($phcode3)]; //@line 545 "synthdata.cpp"
        var $idxprom4=((($5))&255); //@line 545 "synthdata.cpp"
        var $arrayidx5=((_phoneme_tab+$idxprom4*4)&4294967295); //@line 545 "synthdata.cpp"
        var $6=HEAP32[(($arrayidx5)>>2)]; //@line 545 "synthdata.cpp"
        var $type6=(($6+11)&4294967295); //@line 545 "synthdata.cpp"
        var $7=HEAPU8[($type6)]; //@line 545 "synthdata.cpp"
        var $conv7=((($7))&255); //@line 545 "synthdata.cpp"
        var $cmp8=((($conv7))|0)==2; //@line 545 "synthdata.cpp"
        if ($cmp8) { __label__ = 3;; } else { __label__ = 4;; } //@line 545 "synthdata.cpp"
        if (__label__ == 3) {
  
          var $8=$plist_addr; //@line 547 "synthdata.cpp"
          var $arrayidx10=(($8+24)&4294967295); //@line 547 "synthdata.cpp"
          $pl=$arrayidx10; //@line 547 "synthdata.cpp"
          __label__ = 5;break $if_then$$if_else$2;
        }
        else if (__label__ == 4) {
  
          $retval=0; //@line 550 "synthdata.cpp"
          __label__ = 21;break $if_then$$if_else$2; //@line 550 "synthdata.cpp"
        }
      }
    } while(0);
    $if_end12$$return$8: do { 
      if (__label__ == 5) {
  
        var $9=$pl; //@line 553 "synthdata.cpp"
        var $stresslevel=(($9+1)&4294967295); //@line 553 "synthdata.cpp"
        var $10=HEAPU8[($stresslevel)]; //@line 553 "synthdata.cpp"
        var $conv13=((($10))&255); //@line 553 "synthdata.cpp"
        var $and=($conv13) & 15; //@line 553 "synthdata.cpp"
        $stress_level=$and; //@line 553 "synthdata.cpp"
        var $11=$tr_addr; //@line 555 "synthdata.cpp"
        var $cmp14=((($11))|0)!=0; //@line 555 "synthdata.cpp"
        if ($cmp14) { __label__ = 6;; } else { __label__ = 13;; } //@line 555 "synthdata.cpp"
        $if_then15$$if_end36$10: do { 
          if (__label__ == 6) {
  
            var $12=$control_addr; //@line 557 "synthdata.cpp"
            var $and16=($12) & 1; //@line 557 "synthdata.cpp"
            var $tobool=((($and16))|0)!=0; //@line 557 "synthdata.cpp"
            if ($tobool) { __label__ = 7;; } else { __label__ = 10;; } //@line 557 "synthdata.cpp"
            $land_lhs_true$$if_end25$12: do { 
              if (__label__ == 7) {
  
                var $13=$plist_addr; //@line 557 "synthdata.cpp"
                var $synthflags=(($13+4)&4294967295); //@line 557 "synthdata.cpp"
                var $14=HEAPU16[(($synthflags)>>1)]; //@line 557 "synthdata.cpp"
                var $conv17=((($14))&65535); //@line 557 "synthdata.cpp"
                var $and18=($conv17) & 16; //@line 557 "synthdata.cpp"
                var $tobool19=((($and18))|0)!=0; //@line 557 "synthdata.cpp"
                if (!($tobool19)) { __label__ = 10;break $land_lhs_true$$if_end25$12; } //@line 557 "synthdata.cpp"
  
                var $15=$tr_addr; //@line 557 "synthdata.cpp"
                var $langopts=(($15)&4294967295); //@line 557 "synthdata.cpp"
                var $param=(($langopts+24)&4294967295); //@line 557 "synthdata.cpp"
                var $arrayidx21=(($param+40)&4294967295); //@line 557 "synthdata.cpp"
                var $16=HEAP32[(($arrayidx21)>>2)]; //@line 557 "synthdata.cpp"
                var $and22=($16) & 1; //@line 557 "synthdata.cpp"
                var $cmp23=((($and22))|0)==0; //@line 557 "synthdata.cpp"
                if (!($cmp23)) { __label__ = 10;break $land_lhs_true$$if_end25$12; } //@line 557 "synthdata.cpp"
  
                $retval=0; //@line 560 "synthdata.cpp"
                __label__ = 21;break $if_end12$$return$8; //@line 560 "synthdata.cpp"
              }
            } while(0);
  
            var $17=$tr_addr; //@line 563 "synthdata.cpp"
            var $langopts26=(($17)&4294967295); //@line 563 "synthdata.cpp"
            var $param27=(($langopts26+24)&4294967295); //@line 563 "synthdata.cpp"
            var $arrayidx28=(($param27+40)&4294967295); //@line 563 "synthdata.cpp"
            var $18=HEAP32[(($arrayidx28)>>2)]; //@line 563 "synthdata.cpp"
            var $and29=($18) & 2; //@line 563 "synthdata.cpp"
            var $tobool30=((($and29))|0)!=0; //@line 563 "synthdata.cpp"
            if (!($tobool30)) { __label__ = 13;break $if_then15$$if_end36$10; } //@line 563 "synthdata.cpp"
  
            var $19=$stress_level; //@line 563 "synthdata.cpp"
            var $20=$pl; //@line 563 "synthdata.cpp"
            var $wordstress=(($20+2)&4294967295); //@line 563 "synthdata.cpp"
            var $21=HEAPU8[($wordstress)]; //@line 563 "synthdata.cpp"
            var $conv32=((($21))&255); //@line 563 "synthdata.cpp"
            var $cmp33=((($19))|0) >= ((($conv32))|0); //@line 563 "synthdata.cpp"
            if (!($cmp33)) { __label__ = 13;break $if_then15$$if_end36$10; } //@line 563 "synthdata.cpp"
  
            $stress_level=4; //@line 566 "synthdata.cpp"
            ; //@line 567 "synthdata.cpp"
          }
        } while(0);
  
        var $22=$condition_addr; //@line 570 "synthdata.cpp"
        var $cmp37=((($22))|0)==4; //@line 570 "synthdata.cpp"
        if ($cmp37) { __label__ = 14;; } else { __label__ = 15;; } //@line 570 "synthdata.cpp"
        if (__label__ == 14) {
  
          var $23=$stress_level; //@line 572 "synthdata.cpp"
          var $24=$pl; //@line 572 "synthdata.cpp"
          var $wordstress39=(($24+2)&4294967295); //@line 572 "synthdata.cpp"
          var $25=HEAPU8[($wordstress39)]; //@line 572 "synthdata.cpp"
          var $conv40=((($25))&255); //@line 572 "synthdata.cpp"
          var $cmp41=((($23))|0) >= ((($conv40))|0); //@line 572 "synthdata.cpp"
          $retval=$cmp41; //@line 572 "synthdata.cpp"
          ; //@line 572 "synthdata.cpp"
        }
        else if (__label__ == 15) {
  
          var $26=$condition_addr; //@line 575 "synthdata.cpp"
          var $cmp43=((($26))|0)==3; //@line 575 "synthdata.cpp"
          var $27=$stress_level; //@line 578 "synthdata.cpp"
          if ($cmp43) { __label__ = 16;; } else { __label__ = 18;; } //@line 575 "synthdata.cpp"
          $if_then44$$if_else48$23: do { 
            if (__label__ == 16) {
  
              var $cmp45=((($27))|0) > 3; //@line 578 "synthdata.cpp"
              if (!($cmp45)) { __label__ = 20;break $if_then44$$if_else48$23; } //@line 578 "synthdata.cpp"
  
              $retval=1; //@line 579 "synthdata.cpp"
              __label__ = 21;break $if_end12$$return$8; //@line 579 "synthdata.cpp"
            }
            else if (__label__ == 18) {
  
              var $28=$condition_addr; //@line 583 "synthdata.cpp"
              var $arrayidx49=((__ZZL15StressConditionP10TranslatorP12PHONEME_LISTiiE15condition_level+$28*4)&4294967295); //@line 583 "synthdata.cpp"
              var $29=HEAP32[(($arrayidx49)>>2)]; //@line 583 "synthdata.cpp"
              var $cmp50=((($27))|0) < ((($29))|0); //@line 583 "synthdata.cpp"
              if (!($cmp50)) { __label__ = 20;break $if_then44$$if_else48$23; } //@line 583 "synthdata.cpp"
  
              $retval=1; //@line 584 "synthdata.cpp"
              __label__ = 21;break $if_end12$$return$8; //@line 584 "synthdata.cpp"
            }
          } while(0);
  
          $retval=0; //@line 586 "synthdata.cpp"
          ; //@line 586 "synthdata.cpp"
        }
      }
    } while(0);
  
    var $30=$retval; //@line 588 "synthdata.cpp"
    ;
    return $30; //@line 588 "synthdata.cpp"
    return null;
  }
  

  function __ZL18InterpretConditionP10TranslatoriP12PHONEME_LISTi($tr, $control, $plist, $instn) {
    ;
    var __label__;
    var __lastLabel__ = null;
  
    var $retval;
    var $tr_addr;
    var $control_addr;
    var $plist_addr;
    var $instn_addr;
    var $which;
    var $data;
    var $instn2;
    var $count;
    var $ph;
    var $plist_this;
    $tr_addr=$tr;
    $control_addr=$control;
    $plist_addr=$plist;
    $instn_addr=$instn;
    var $0=$instn_addr; //@line 601 "synthdata.cpp"
    var $and=($0) & 255; //@line 601 "synthdata.cpp"
    $data=$and; //@line 601 "synthdata.cpp"
    var $1=$instn_addr; //@line 602 "synthdata.cpp"
    var $shr=($1) >> 8; //@line 602 "synthdata.cpp"
    $instn2=$shr; //@line 602 "synthdata.cpp"
    var $2=$instn2; //@line 604 "synthdata.cpp"
    var $cmp=((($2))|0) < 14; //@line 604 "synthdata.cpp"
    var $3=$instn2; //@line 606 "synthdata.cpp"
    if ($cmp) { __label__ = 1;; } else { __label__ = 67;; } //@line 604 "synthdata.cpp"
    $if_then$$if_else187$2: do { 
      if (__label__ == 1) {
  
        var $rem=((($3))|0)%7; //@line 606 "synthdata.cpp"
        $which=$rem; //@line 606 "synthdata.cpp"
        var $4=$which; //@line 608 "synthdata.cpp"
        var $cmp1=((($4))|0)==4; //@line 608 "synthdata.cpp"
        if ($cmp1) { __lastLabel__ = 1; __label__ = 2;; } else { __lastLabel__ = 1; __label__ = 5;; } //@line 608 "synthdata.cpp"
        if (__label__ == 2) {
  
          var $5=$plist_addr; //@line 611 "synthdata.cpp"
          var $arrayidx=(($5+24)&4294967295); //@line 611 "synthdata.cpp"
          var $sourceix=(($arrayidx+6)&4294967295); //@line 611 "synthdata.cpp"
          var $6=HEAP16[(($sourceix)>>1)]; //@line 611 "synthdata.cpp"
          var $tobool=(tempInt=(($6)),(tempInt>=32768?tempInt-65536:tempInt))!=0; //@line 611 "synthdata.cpp"
          if ($tobool) { __label__ = 3;; } else { __label__ = 4;; } //@line 611 "synthdata.cpp"
          if (__label__ == 3) {
  
            $retval=0; //@line 612 "synthdata.cpp"
            __label__ = 73;break $if_then$$if_else187$2; //@line 612 "synthdata.cpp"
          }
          else if (__label__ == 4) {
  
            var $_pr=$which; //@line 614 "synthdata.cpp"
            __lastLabel__ = 4; ;
          }
        }
  
        var $7=__lastLabel__ == 4 ? $_pr : ($4); //@line 614 "synthdata.cpp"
        var $cmp5=((($7))|0)==5; //@line 614 "synthdata.cpp"
        if ($cmp5) { __label__ = 6;; } else { __label__ = 8;; } //@line 614 "synthdata.cpp"
        $if_then6$$if_end12$10: do { 
          if (__label__ == 6) {
  
            var $8=$plist_addr; //@line 617 "synthdata.cpp"
            var $arrayidx7=(($8)&4294967295); //@line 617 "synthdata.cpp"
            var $sourceix8=(($arrayidx7+6)&4294967295); //@line 617 "synthdata.cpp"
            var $9=HEAP16[(($sourceix8)>>1)]; //@line 617 "synthdata.cpp"
            var $tobool9=(tempInt=(($9)),(tempInt>=32768?tempInt-65536:tempInt))!=0; //@line 617 "synthdata.cpp"
            if (!($tobool9)) { __label__ = 8;break $if_then6$$if_end12$10; } //@line 617 "synthdata.cpp"
  
            $retval=0; //@line 618 "synthdata.cpp"
            __label__ = 73;break $if_then$$if_else187$2; //@line 618 "synthdata.cpp"
          }
        } while(0);
  
        var $10=$which; //@line 620 "synthdata.cpp"
        var $cmp13=((($10))|0)==7; //@line 620 "synthdata.cpp"
        if ($cmp13) { __lastLabel__ = 8; __label__ = 9;; } else { __lastLabel__ = 8; __label__ = 13;; } //@line 620 "synthdata.cpp"
        $if_then14$$if_end23$14: do { 
          if (__label__ == 9) {
  
            var $11=$plist_addr; //@line 623 "synthdata.cpp"
            var $arrayidx15=(($11+24)&4294967295); //@line 623 "synthdata.cpp"
            var $sourceix16=(($arrayidx15+6)&4294967295); //@line 623 "synthdata.cpp"
            var $12=HEAP16[(($sourceix16)>>1)]; //@line 623 "synthdata.cpp"
            var $tobool17=(tempInt=(($12)),(tempInt>=32768?tempInt-65536:tempInt))!=0; //@line 623 "synthdata.cpp"
            if ($tobool17) { __label__ = 11;; } else { __label__ = 10;; } //@line 623 "synthdata.cpp"
            $if_then21$$lor_lhs_false$16: do { 
              if (__label__ == 10) {
  
                var $13=$plist_addr; //@line 623 "synthdata.cpp"
                var $arrayidx18=(($13+48)&4294967295); //@line 623 "synthdata.cpp"
                var $sourceix19=(($arrayidx18+6)&4294967295); //@line 623 "synthdata.cpp"
                var $14=HEAP16[(($sourceix19)>>1)]; //@line 623 "synthdata.cpp"
                var $tobool20=(tempInt=(($14)),(tempInt>=32768?tempInt-65536:tempInt))!=0; //@line 623 "synthdata.cpp"
                if ($tobool20) { __label__ = 11;break $if_then21$$lor_lhs_false$16; } //@line 623 "synthdata.cpp"
  
                var $_pr1=$which; //@line 627 "synthdata.cpp"
                __lastLabel__ = 12; __label__ = 13;break $if_then14$$if_end23$14;
              }
            } while(0);
  
            $retval=0; //@line 624 "synthdata.cpp"
            __label__ = 73;break $if_then$$if_else187$2; //@line 624 "synthdata.cpp"
          }
        } while(0);
  
        var $15=__lastLabel__ == 12 ? $_pr1 : ($10); //@line 627 "synthdata.cpp"
        var $cmp24=((($15))|0)==6; //@line 627 "synthdata.cpp"
        if ($cmp24) { __label__ = 14;; } else { __label__ = 19;; } //@line 627 "synthdata.cpp"
        $if_then25$$if_else$21: do { 
          if (__label__ == 14) {
  
            $which=2; //@line 630 "synthdata.cpp"
            ; //@line 630 "synthdata.cpp"
            while(1) { 
  
              var $16=$which; //@line 632 "synthdata.cpp"
              var $sub=((($16)-1)&4294967295); //@line 632 "synthdata.cpp"
              var $17=$plist_addr; //@line 632 "synthdata.cpp"
              var $arrayidx26=(($17+24*$sub)&4294967295); //@line 632 "synthdata.cpp"
              var $sourceix27=(($arrayidx26+6)&4294967295); //@line 632 "synthdata.cpp"
              var $18=HEAP16[(($sourceix27)>>1)]; //@line 632 "synthdata.cpp"
              var $tobool28=(tempInt=(($18)),(tempInt>=32768?tempInt-65536:tempInt))!=0; //@line 632 "synthdata.cpp"
              if ($tobool28) { __label__ = 16;break ; } //@line 632 "synthdata.cpp"
  
              var $19=$which; //@line 634 "synthdata.cpp"
              var $sub31=((($19)-1)&4294967295); //@line 634 "synthdata.cpp"
              var $20=$plist_addr; //@line 634 "synthdata.cpp"
              var $arrayidx32=(($20+24*$sub31)&4294967295); //@line 634 "synthdata.cpp"
              var $phcode=(($arrayidx32)&4294967295); //@line 634 "synthdata.cpp"
              var $21=HEAPU8[($phcode)]; //@line 634 "synthdata.cpp"
              var $idxprom=((($21))&255); //@line 634 "synthdata.cpp"
              var $arrayidx33=((_phoneme_tab+$idxprom*4)&4294967295); //@line 634 "synthdata.cpp"
              var $22=HEAP32[(($arrayidx33)>>2)]; //@line 634 "synthdata.cpp"
              var $type=(($22+11)&4294967295); //@line 634 "synthdata.cpp"
              var $23=HEAPU8[($type)]; //@line 634 "synthdata.cpp"
              var $conv=((($23))&255); //@line 634 "synthdata.cpp"
              var $cmp34=((($conv))|0)==2; //@line 634 "synthdata.cpp"
              var $_pr2=$which; //@line 644 "synthdata.cpp"
              if ($cmp34) { __lastLabel__ = 17; __label__ = 20;break $if_then25$$if_else$21; } else { __lastLabel__ = 17; ; } //@line 634 "synthdata.cpp"
  
              var $inc=((($_pr2)+1)&4294967295); //@line 630 "synthdata.cpp"
              $which=$inc; //@line 630 "synthdata.cpp"
              __label__ = 15;continue ; //@line 630 "synthdata.cpp"
            }
  
            $retval=0; //@line 633 "synthdata.cpp"
            __label__ = 73;break $if_then$$if_else187$2; //@line 633 "synthdata.cpp"
          }
          else if (__label__ == 19) {
  
            var $24=$which; //@line 640 "synthdata.cpp"
            var $arrayidx37=((__ZZL18InterpretConditionP10TranslatoriP12PHONEME_LISTiE11ph_position+$24*4)&4294967295); //@line 640 "synthdata.cpp"
            var $25=HEAP32[(($arrayidx37)>>2)]; //@line 640 "synthdata.cpp"
            $which=$25; //@line 640 "synthdata.cpp"
            __lastLabel__ = 19; ;
          }
        } while(0);
  
        var $26=__lastLabel__ == 19 ? $25 : ($_pr2); //@line 644 "synthdata.cpp"
        var $27=$plist_addr; //@line 643 "synthdata.cpp"
        $plist_this=$27; //@line 643 "synthdata.cpp"
        var $sub39=((($26)-1)&4294967295); //@line 644 "synthdata.cpp"
        var $28=$plist_addr; //@line 644 "synthdata.cpp"
        var $arrayidx40=(($28+24*$sub39)&4294967295); //@line 644 "synthdata.cpp"
        $plist_addr=$arrayidx40; //@line 644 "synthdata.cpp"
        var $cmp41=((($26))|0)==0; //@line 646 "synthdata.cpp"
        if ($cmp41) { __label__ = 21;; } else { __label__ = 23;; } //@line 646 "synthdata.cpp"
        $if_then42$$if_end49$30: do { 
          if (__label__ == 21) {
  
            var $29=$plist_addr; //@line 648 "synthdata.cpp"
            var $phcode43=(($29)&4294967295); //@line 648 "synthdata.cpp"
            var $30=HEAPU8[($phcode43)]; //@line 648 "synthdata.cpp"
            var $conv44=((($30))&255); //@line 648 "synthdata.cpp"
            var $cmp45=((($conv44))|0)==1; //@line 648 "synthdata.cpp"
            if (!($cmp45)) { __label__ = 23;break $if_then42$$if_end49$30; } //@line 648 "synthdata.cpp"
  
            var $31=$plist_addr; //@line 651 "synthdata.cpp"
            var $arrayidx47=(($31+-24)&4294967295); //@line 651 "synthdata.cpp"
            $plist_addr=$arrayidx47; //@line 651 "synthdata.cpp"
            ; //@line 652 "synthdata.cpp"
          }
        } while(0);
  
        var $32=$control_addr; //@line 655 "synthdata.cpp"
        var $and50=($32) & 256; //@line 655 "synthdata.cpp"
        var $tobool51=((($and50))|0)!=0; //@line 655 "synthdata.cpp"
        if ($tobool51) { __label__ = 24;; } else { __label__ = 25;; } //@line 655 "synthdata.cpp"
        if (__label__ == 24) {
  
          var $33=$plist_addr; //@line 658 "synthdata.cpp"
          var $phcode53=(($33)&4294967295); //@line 658 "synthdata.cpp"
          var $34=HEAPU8[($phcode53)]; //@line 658 "synthdata.cpp"
          var $idxprom54=((($34))&255); //@line 658 "synthdata.cpp"
          var $arrayidx55=((_phoneme_tab+$idxprom54*4)&4294967295); //@line 658 "synthdata.cpp"
          var $35=HEAP32[(($arrayidx55)>>2)]; //@line 658 "synthdata.cpp"
          var $36=$plist_addr; //@line 658 "synthdata.cpp"
          var $ph56=(($36+8)&4294967295); //@line 658 "synthdata.cpp"
          HEAP32[(($ph56)>>2)]=$35; //@line 658 "synthdata.cpp"
          ; //@line 659 "synthdata.cpp"
        }
  
        var $37=$plist_addr; //@line 660 "synthdata.cpp"
        var $ph58=(($37+8)&4294967295); //@line 660 "synthdata.cpp"
        var $38=HEAP32[(($ph58)>>2)]; //@line 660 "synthdata.cpp"
        $ph=$38; //@line 660 "synthdata.cpp"
        var $39=$instn2; //@line 662 "synthdata.cpp"
        var $cmp59=((($39))|0) < 7; //@line 662 "synthdata.cpp"
        if ($cmp59) { __label__ = 26;; } else { __label__ = 32;; } //@line 662 "synthdata.cpp"
        if (__label__ == 26) {
  
          var $40=$data; //@line 665 "synthdata.cpp"
          var $arrayidx61=((_phoneme_tab+$40*4)&4294967295); //@line 665 "synthdata.cpp"
          var $41=HEAP32[(($arrayidx61)>>2)]; //@line 665 "synthdata.cpp"
          var $mnemonic=(($41)&4294967295); //@line 665 "synthdata.cpp"
          var $42=HEAP32[(($mnemonic)>>2)]; //@line 665 "synthdata.cpp"
          var $43=$ph; //@line 665 "synthdata.cpp"
          var $mnemonic62=(($43)&4294967295); //@line 665 "synthdata.cpp"
          var $44=HEAP32[(($mnemonic62)>>2)]; //@line 665 "synthdata.cpp"
          var $cmp63=((($42))|0)==((($44))|0); //@line 665 "synthdata.cpp"
          var $conv64=((($cmp63))&1); //@line 665 "synthdata.cpp"
          var $cmp65=((($conv64))|0)==1; //@line 665 "synthdata.cpp"
          if ($cmp65) { __label__ = 27;; } else { __label__ = 28;; } //@line 665 "synthdata.cpp"
          if (__label__ == 27) {
  
            $retval=1; //@line 666 "synthdata.cpp"
            ; //@line 666 "synthdata.cpp"
          }
          else if (__label__ == 28) {
  
            var $45=$which; //@line 667 "synthdata.cpp"
            var $cmp68=((($45))|0)==0; //@line 667 "synthdata.cpp"
            if ($cmp68) { __label__ = 29;; } else { __label__ = 31;; } //@line 667 "synthdata.cpp"
            $land_lhs_true$$if_end75$42: do { 
              if (__label__ == 29) {
  
                var $46=$ph; //@line 667 "synthdata.cpp"
                var $type69=(($46+11)&4294967295); //@line 667 "synthdata.cpp"
                var $47=HEAPU8[($type69)]; //@line 667 "synthdata.cpp"
                var $conv70=((($47))&255); //@line 667 "synthdata.cpp"
                var $cmp71=((($conv70))|0)==2; //@line 667 "synthdata.cpp"
                if (!($cmp71)) { __label__ = 31;break $land_lhs_true$$if_end75$42; } //@line 667 "synthdata.cpp"
  
                var $48=$data; //@line 668 "synthdata.cpp"
                var $49=$ph; //@line 668 "synthdata.cpp"
                var $end_type=(($49+13)&4294967295); //@line 668 "synthdata.cpp"
                var $50=HEAPU8[($end_type)]; //@line 668 "synthdata.cpp"
                var $conv73=((($50))&255); //@line 668 "synthdata.cpp"
                var $cmp74=((($48))|0)==((($conv73))|0); //@line 668 "synthdata.cpp"
                $retval=$cmp74; //@line 668 "synthdata.cpp"
                __label__ = 73;break $if_then$$if_else187$2; //@line 668 "synthdata.cpp"
              }
            } while(0);
  
            var $51=$data; //@line 669 "synthdata.cpp"
            var $52=$ph; //@line 669 "synthdata.cpp"
            var $start_type=(($52+12)&4294967295); //@line 669 "synthdata.cpp"
            var $53=HEAPU8[($start_type)]; //@line 669 "synthdata.cpp"
            var $conv76=((($53))&255); //@line 669 "synthdata.cpp"
            var $cmp77=((($51))|0)==((($conv76))|0); //@line 669 "synthdata.cpp"
            $retval=$cmp77; //@line 669 "synthdata.cpp"
            ; //@line 669 "synthdata.cpp"
          }
        }
        else if (__label__ == 32) {
  
          var $54=$instn_addr; //@line 672 "synthdata.cpp"
          var $and79=($54) & 31; //@line 672 "synthdata.cpp"
          $data=$and79; //@line 672 "synthdata.cpp"
          var $55=$instn_addr; //@line 674 "synthdata.cpp"
          var $and80=($55) & 224; //@line 674 "synthdata.cpp"
          if ($and80 == 0) {
            __label__ = 33;;
          }
          else if ($and80 == 32) {
            __label__ = 34;;
          }
          else if ($and80 == 64) {
            __label__ = 35;;
          }
          else if ($and80 == 128) {
            __label__ = 36;;
          }
          else {
          __label__ = 66;;
          }
          
          if (__label__ == 66) {
  
            $retval=0; //@line 758 "synthdata.cpp"
            ; //@line 758 "synthdata.cpp"
          }
          else if (__label__ == 33) {
  
            var $56=$ph; //@line 678 "synthdata.cpp"
            var $type81=(($56+11)&4294967295); //@line 678 "synthdata.cpp"
            var $57=HEAPU8[($type81)]; //@line 678 "synthdata.cpp"
            var $conv82=((($57))&255); //@line 678 "synthdata.cpp"
            var $58=$data; //@line 678 "synthdata.cpp"
            var $cmp83=((($conv82))|0)==((($58))|0); //@line 678 "synthdata.cpp"
            $retval=$cmp83; //@line 678 "synthdata.cpp"
            ; //@line 678 "synthdata.cpp"
          }
          else if (__label__ == 34) {
  
            var $59=$ph; //@line 683 "synthdata.cpp"
            var $phflags=(($59+4)&4294967295); //@line 683 "synthdata.cpp"
            var $60=HEAPU32[(($phflags)>>2)]; //@line 683 "synthdata.cpp"
            var $shr85=($60) >>> 16; //@line 683 "synthdata.cpp"
            var $and86=($shr85) & 15; //@line 683 "synthdata.cpp"
            var $61=$data; //@line 683 "synthdata.cpp"
            var $cmp87=((($and86))|0)==((($61))|0); //@line 683 "synthdata.cpp"
            $retval=$cmp87; //@line 683 "synthdata.cpp"
            ; //@line 683 "synthdata.cpp"
          }
          else if (__label__ == 35) {
  
            var $62=$ph; //@line 688 "synthdata.cpp"
            var $phflags89=(($62+4)&4294967295); //@line 688 "synthdata.cpp"
            var $63=HEAP32[(($phflags89)>>2)]; //@line 688 "synthdata.cpp"
            var $64=$data; //@line 688 "synthdata.cpp"
            var $shl=1 << ($64); //@line 688 "synthdata.cpp"
            var $and90=($shl) & ($63); //@line 688 "synthdata.cpp"
            var $cmp91=((($and90))|0)!=0; //@line 688 "synthdata.cpp"
            $retval=$cmp91; //@line 688 "synthdata.cpp"
            ; //@line 688 "synthdata.cpp"
          }
          else if (__label__ == 36) {
  
            var $65=$data; //@line 692 "synthdata.cpp"
            if ($65 == 0) {
              __label__ = 37;;
            }
            else if ($65 == 1) {
              __label__ = 37;;
            }
            else if ($65 == 2) {
              __label__ = 37;;
            }
            else if ($65 == 3) {
              __label__ = 37;;
            }
            else if ($65 == 4) {
              __label__ = 37;;
            }
            else if ($65 == 5) {
              __label__ = 38;;
            }
            else if ($65 == 6) {
              __label__ = 41;;
            }
            else if ($65 == 7) {
              __label__ = 42;;
            }
            else if ($65 == 8) {
              __label__ = 43;;
            }
            else if ($65 == 9) {
              __label__ = 46;;
            }
            else if ($65 == 10) {
              __label__ = 51;;
            }
            else if ($65 == 11) {
              __label__ = 52;;
            }
            else if ($65 == 12) {
              __label__ = 56;;
            }
            else {
            __label__ = 60;;
            }
            
            $sw_bb169$$sw_bb93$$sw_bb94$$sw_bb101$$sw_bb105$$sw_bb109$$sw_bb120$$sw_bb134$$for_cond139$$sw_bb156$53: do { 
              if (__label__ == 37) {
  
                var $66=$tr_addr; //@line 699 "synthdata.cpp"
                var $67=$plist_addr; //@line 699 "synthdata.cpp"
                var $68=$data; //@line 699 "synthdata.cpp"
                var $call=__ZL15StressConditionP10TranslatorP12PHONEME_LISTii($66, $67, $68, 0); //@line 699 "synthdata.cpp"
                $retval=$call; //@line 699 "synthdata.cpp"
                __label__ = 73;break $if_then$$if_else187$2; //@line 699 "synthdata.cpp"
              }
              else if (__label__ == 38) {
  
                var $69=$ph; //@line 702 "synthdata.cpp"
                var $type95=(($69+11)&4294967295); //@line 702 "synthdata.cpp"
                var $70=HEAPU8[($type95)]; //@line 702 "synthdata.cpp"
                var $conv96=((($70))&255); //@line 702 "synthdata.cpp"
                var $cmp97=((($conv96))|0)==0; //@line 702 "synthdata.cpp"
                if ($cmp97) { __lastLabel__ = 38; __label__ = 40;; } else { __lastLabel__ = 38; __label__ = 39;; } //@line 702 "synthdata.cpp"
                if (__label__ == 39) {
  
                  var $71=$plist_this; //@line 702 "synthdata.cpp"
                  var $synthflags=(($71+4)&4294967295); //@line 702 "synthdata.cpp"
                  var $72=HEAPU16[(($synthflags)>>1)]; //@line 702 "synthdata.cpp"
                  var $conv98=((($72))&65535); //@line 702 "synthdata.cpp"
                  var $and99=($conv98) & 8192; //@line 702 "synthdata.cpp"
                  var $tobool100=((($and99))|0)!=0; //@line 702 "synthdata.cpp"
                  __lastLabel__ = 39; ; //@line 702 "synthdata.cpp"
                }
  
                var $73=__lastLabel__ == 38 ? 1 : ($tobool100);
                $retval=$73; //@line 702 "synthdata.cpp"
                __label__ = 73;break $if_then$$if_else187$2; //@line 702 "synthdata.cpp"
              }
              else if (__label__ == 41) {
  
                var $74=$plist_addr; //@line 705 "synthdata.cpp"
                var $sourceix102=(($74+6)&4294967295); //@line 705 "synthdata.cpp"
                var $75=HEAPU16[(($sourceix102)>>1)]; //@line 705 "synthdata.cpp"
                var $conv103=((($75))&65535); //@line 705 "synthdata.cpp"
                var $cmp104=((($conv103))|0)!=0; //@line 705 "synthdata.cpp"
                $retval=$cmp104; //@line 705 "synthdata.cpp"
                __label__ = 73;break $if_then$$if_else187$2; //@line 705 "synthdata.cpp"
              }
              else if (__label__ == 42) {
  
                var $76=$plist_addr; //@line 708 "synthdata.cpp"
                var $sourceix106=(($76+6)&4294967295); //@line 708 "synthdata.cpp"
                var $77=HEAPU16[(($sourceix106)>>1)]; //@line 708 "synthdata.cpp"
                var $conv107=((($77))&65535); //@line 708 "synthdata.cpp"
                var $cmp108=((($conv107))|0)==0; //@line 708 "synthdata.cpp"
                $retval=$cmp108; //@line 708 "synthdata.cpp"
                __label__ = 73;break $if_then$$if_else187$2; //@line 708 "synthdata.cpp"
              }
              else if (__label__ == 43) {
  
                var $78=$plist_addr; //@line 711 "synthdata.cpp"
                var $arrayidx110=(($78+24)&4294967295); //@line 711 "synthdata.cpp"
                var $sourceix111=(($arrayidx110+6)&4294967295); //@line 711 "synthdata.cpp"
                var $79=HEAP16[(($sourceix111)>>1)]; //@line 711 "synthdata.cpp"
                var $tobool112=(tempInt=(($79)),(tempInt>=32768?tempInt-65536:tempInt))!=0; //@line 711 "synthdata.cpp"
                if ($tobool112) { __lastLabel__ = 43; __label__ = 45;; } else { __lastLabel__ = 43; __label__ = 44;; } //@line 711 "synthdata.cpp"
                if (__label__ == 44) {
  
                  var $80=$plist_addr; //@line 711 "synthdata.cpp"
                  var $arrayidx114=(($80+24)&4294967295); //@line 711 "synthdata.cpp"
                  var $ph115=(($arrayidx114+8)&4294967295); //@line 711 "synthdata.cpp"
                  var $81=HEAP32[(($ph115)>>2)]; //@line 711 "synthdata.cpp"
                  var $type116=(($81+11)&4294967295); //@line 711 "synthdata.cpp"
                  var $82=HEAPU8[($type116)]; //@line 711 "synthdata.cpp"
                  var $conv117=((($82))&255); //@line 711 "synthdata.cpp"
                  var $cmp118=((($conv117))|0)==0; //@line 711 "synthdata.cpp"
                  __lastLabel__ = 44; ; //@line 711 "synthdata.cpp"
                }
  
                var $83=__lastLabel__ == 43 ? 1 : ($cmp118);
                $retval=$83; //@line 711 "synthdata.cpp"
                __label__ = 73;break $if_then$$if_else187$2; //@line 711 "synthdata.cpp"
              }
              else if (__label__ == 46) {
  
                var $84=$plist_addr; //@line 715 "synthdata.cpp"
                var $sourceix121=(($84+6)&4294967295); //@line 715 "synthdata.cpp"
                var $85=HEAPU16[(($sourceix121)>>1)]; //@line 715 "synthdata.cpp"
                var $conv122=((($85))&65535); //@line 715 "synthdata.cpp"
                var $cmp123=((($conv122))|0)!=0; //@line 715 "synthdata.cpp"
                if ($cmp123) { __label__ = 47;; } else { __label__ = 48;; } //@line 715 "synthdata.cpp"
                if (__label__ == 47) {
  
                  $retval=0; //@line 716 "synthdata.cpp"
                  __label__ = 73;break $if_then$$if_else187$2; //@line 716 "synthdata.cpp"
                }
                else if (__label__ == 48) {
                  while(1) { 
  
                    var $86=$plist_addr; //@line 718 "synthdata.cpp"
                    var $incdec_ptr=(($86+-24)&4294967295); //@line 718 "synthdata.cpp"
                    $plist_addr=$incdec_ptr; //@line 718 "synthdata.cpp"
                    var $87=$plist_addr; //@line 719 "synthdata.cpp"
                    var $stresslevel=(($87+1)&4294967295); //@line 719 "synthdata.cpp"
                    var $88=HEAPU8[($stresslevel)]; //@line 719 "synthdata.cpp"
                    var $conv126=((($88))&255); //@line 719 "synthdata.cpp"
                    var $and127=($conv126) & 15; //@line 719 "synthdata.cpp"
                    var $cmp128=((($and127))|0) >= 4; //@line 719 "synthdata.cpp"
                    if ($cmp128) { __label__ = 49;break ; } //@line 719 "synthdata.cpp"
  
                    var $89=$plist_addr; //@line 722 "synthdata.cpp"
                    var $sourceix131=(($89+6)&4294967295); //@line 722 "synthdata.cpp"
                    var $90=HEAPU16[(($sourceix131)>>1)]; //@line 722 "synthdata.cpp"
                    var $conv132=((($90))&65535); //@line 722 "synthdata.cpp"
                    var $cmp133=((($conv132))|0)==0; //@line 722 "synthdata.cpp"
                    if ($cmp133) { __label__ = 48;continue ; } else { __label__ = 60;break $sw_bb169$$sw_bb93$$sw_bb94$$sw_bb101$$sw_bb105$$sw_bb109$$sw_bb120$$sw_bb134$$for_cond139$$sw_bb156$53; } //@line 722 "synthdata.cpp"
                  }
  
                  $retval=1; //@line 720 "synthdata.cpp"
                  __label__ = 73;break $if_then$$if_else187$2; //@line 720 "synthdata.cpp"
                }
              }
              else if (__label__ == 51) {
  
                var $91=$ph; //@line 726 "synthdata.cpp"
                var $type135=(($91+11)&4294967295); //@line 726 "synthdata.cpp"
                var $92=HEAPU8[($type135)]; //@line 726 "synthdata.cpp"
                var $conv136=((($92))&255); //@line 726 "synthdata.cpp"
                var $cmp137=((($conv136))|0)!=2; //@line 726 "synthdata.cpp"
                $retval=$cmp137; //@line 726 "synthdata.cpp"
                __label__ = 73;break $if_then$$if_else187$2; //@line 726 "synthdata.cpp"
              }
              else if (__label__ == 52) {
                while(1) { 
  
                  var $93=$plist_addr; //@line 731 "synthdata.cpp"
                  var $incdec_ptr140=(($93+24)&4294967295); //@line 731 "synthdata.cpp"
                  $plist_addr=$incdec_ptr140; //@line 731 "synthdata.cpp"
                  var $94=$plist_addr; //@line 732 "synthdata.cpp"
                  var $phcode141=(($94)&4294967295); //@line 732 "synthdata.cpp"
                  var $95=HEAPU8[($phcode141)]; //@line 732 "synthdata.cpp"
                  var $idxprom142=((($95))&255); //@line 732 "synthdata.cpp"
                  var $arrayidx143=((_phoneme_tab+$idxprom142*4)&4294967295); //@line 732 "synthdata.cpp"
                  var $96=HEAP32[(($arrayidx143)>>2)]; //@line 732 "synthdata.cpp"
                  var $97=$plist_addr; //@line 732 "synthdata.cpp"
                  var $ph144=(($97+8)&4294967295); //@line 732 "synthdata.cpp"
                  HEAP32[(($ph144)>>2)]=$96; //@line 732 "synthdata.cpp"
                  var $98=$plist_addr; //@line 733 "synthdata.cpp"
                  var $sourceix145=(($98+6)&4294967295); //@line 733 "synthdata.cpp"
                  var $99=HEAPU16[(($sourceix145)>>1)]; //@line 733 "synthdata.cpp"
                  var $conv146=((($99))&65535); //@line 733 "synthdata.cpp"
                  var $cmp147=((($conv146))|0)!=0; //@line 733 "synthdata.cpp"
                  if ($cmp147) { __label__ = 53;break ; } //@line 733 "synthdata.cpp"
  
                  var $100=$plist_addr; //@line 735 "synthdata.cpp"
                  var $ph150=(($100+8)&4294967295); //@line 735 "synthdata.cpp"
                  var $101=HEAP32[(($ph150)>>2)]; //@line 735 "synthdata.cpp"
                  var $type151=(($101+11)&4294967295); //@line 735 "synthdata.cpp"
                  var $102=HEAPU8[($type151)]; //@line 735 "synthdata.cpp"
                  var $conv152=((($102))&255); //@line 735 "synthdata.cpp"
                  var $cmp153=((($conv152))|0)==2; //@line 735 "synthdata.cpp"
                  if ($cmp153) { __label__ = 55;break ; } else { __label__ = 52;continue ; } //@line 735 "synthdata.cpp"
                }
                if (__label__ == 53) {
  
                  $retval=1; //@line 734 "synthdata.cpp"
                  __label__ = 73;break $if_then$$if_else187$2; //@line 734 "synthdata.cpp"
                }
                else if (__label__ == 55) {
  
                  $retval=0; //@line 736 "synthdata.cpp"
                  __label__ = 73;break $if_then$$if_else187$2; //@line 736 "synthdata.cpp"
                }
              }
              else if (__label__ == 56) {
  
                var $103=$ph; //@line 741 "synthdata.cpp"
                var $type157=(($103+11)&4294967295); //@line 741 "synthdata.cpp"
                var $104=HEAPU8[($type157)]; //@line 741 "synthdata.cpp"
                var $conv158=((($104))&255); //@line 741 "synthdata.cpp"
                var $cmp159=((($conv158))|0)==2; //@line 741 "synthdata.cpp"
                if ($cmp159) { __lastLabel__ = 56; __label__ = 59;; } else { __lastLabel__ = 56; __label__ = 57;; } //@line 741 "synthdata.cpp"
                $lor_end168$$lor_lhs_false160$80: do { 
                  if (__label__ == 57) {
  
                    var $105=$ph; //@line 741 "synthdata.cpp"
                    var $type161=(($105+11)&4294967295); //@line 741 "synthdata.cpp"
                    var $106=HEAPU8[($type161)]; //@line 741 "synthdata.cpp"
                    var $conv162=((($106))&255); //@line 741 "synthdata.cpp"
                    var $cmp163=((($conv162))|0)==3; //@line 741 "synthdata.cpp"
                    if ($cmp163) { __lastLabel__ = 57; __label__ = 59;break $lor_end168$$lor_lhs_false160$80; } else { __lastLabel__ = 57; ; } //@line 741 "synthdata.cpp"
  
                    var $107=$ph; //@line 741 "synthdata.cpp"
                    var $phflags165=(($107+4)&4294967295); //@line 741 "synthdata.cpp"
                    var $108=HEAP32[(($phflags165)>>2)]; //@line 741 "synthdata.cpp"
                    var $and166=($108) & 16; //@line 741 "synthdata.cpp"
                    var $tobool167=((($and166))|0)!=0; //@line 741 "synthdata.cpp"
                    __lastLabel__ = 58; ; //@line 741 "synthdata.cpp"
                  }
                } while(0);
  
                var $109=__lastLabel__ == 57 ? 1 : (__lastLabel__ == 56 ? 1 : ($tobool167));
                $retval=$109; //@line 741 "synthdata.cpp"
                __label__ = 73;break $if_then$$if_else187$2; //@line 741 "synthdata.cpp"
              }
            } while(0);
  
            $count=0; //@line 745 "synthdata.cpp"
            ; //@line 746 "synthdata.cpp"
            while(1) { 
  
              var $110=$plist_addr; //@line 748 "synthdata.cpp"
              var $ph171=(($110+8)&4294967295); //@line 748 "synthdata.cpp"
              var $111=HEAP32[(($ph171)>>2)]; //@line 748 "synthdata.cpp"
              var $type172=(($111+11)&4294967295); //@line 748 "synthdata.cpp"
              var $112=HEAPU8[($type172)]; //@line 748 "synthdata.cpp"
              var $conv173=((($112))&255); //@line 748 "synthdata.cpp"
              var $cmp174=((($conv173))|0)==2; //@line 748 "synthdata.cpp"
              if ($cmp174) { __label__ = 62;; } else { __label__ = 63;; } //@line 748 "synthdata.cpp"
              if (__label__ == 62) {
  
                var $113=$count; //@line 749 "synthdata.cpp"
                var $inc176=((($113)+1)&4294967295); //@line 749 "synthdata.cpp"
                $count=$inc176; //@line 749 "synthdata.cpp"
                ; //@line 749 "synthdata.cpp"
              }
  
              var $114=$plist_addr; //@line 750 "synthdata.cpp"
              var $sourceix178=(($114+6)&4294967295); //@line 750 "synthdata.cpp"
              var $115=HEAPU16[(($sourceix178)>>1)]; //@line 750 "synthdata.cpp"
              var $conv179=((($115))&65535); //@line 750 "synthdata.cpp"
              var $cmp180=((($conv179))|0)!=0; //@line 750 "synthdata.cpp"
              if ($cmp180) { __label__ = 65;break ; } //@line 750 "synthdata.cpp"
  
              var $116=$plist_addr; //@line 752 "synthdata.cpp"
              var $incdec_ptr183=(($116+-24)&4294967295); //@line 752 "synthdata.cpp"
              $plist_addr=$incdec_ptr183; //@line 752 "synthdata.cpp"
              __label__ = 61;continue ; //@line 753 "synthdata.cpp"
            }
  
            var $117=$count; //@line 754 "synthdata.cpp"
            var $cmp185=((($117))|0)==1; //@line 754 "synthdata.cpp"
            $retval=$cmp185; //@line 754 "synthdata.cpp"
            ; //@line 754 "synthdata.cpp"
          }
        }
      }
      else if (__label__ == 67) {
  
        var $cmp188=((($3))|0)==15; //@line 761 "synthdata.cpp"
        if ($cmp188) { __label__ = 68;; } else { __label__ = 72;; } //@line 761 "synthdata.cpp"
        $if_then189$$if_end201$93: do { 
          if (__label__ == 68) {
  
            var $118=$data; //@line 764 "synthdata.cpp"
            if ($118 == 1) {
              __label__ = 69;;
            }
            else if ($118 == 2) {
              __label__ = 70;;
            }
            else if ($118 == 3) {
              __label__ = 71;;
            }
            else {
            __label__ = 72;break $if_then189$$if_end201$93;
            }
            
            if (__label__ == 69) {
  
              var $119=$control_addr; //@line 767 "synthdata.cpp"
              var $and191=($119) & 1; //@line 767 "synthdata.cpp"
              var $tobool192=((($and191))|0)!=0; //@line 767 "synthdata.cpp"
              $retval=$tobool192; //@line 767 "synthdata.cpp"
              __label__ = 73;break $if_then$$if_else187$2; //@line 767 "synthdata.cpp"
            }
            else if (__label__ == 70) {
  
              var $120=HEAP32[((_voice)>>2)]; //@line 769 "synthdata.cpp"
              var $klattv=(($120+112)&4294967295); //@line 769 "synthdata.cpp"
              var $arrayidx194=(($klattv)&4294967295); //@line 769 "synthdata.cpp"
              var $121=HEAP32[(($arrayidx194)>>2)]; //@line 769 "synthdata.cpp"
              var $cmp195=((($121))|0)!=0; //@line 769 "synthdata.cpp"
              $retval=$cmp195; //@line 769 "synthdata.cpp"
              __label__ = 73;break $if_then$$if_else187$2; //@line 769 "synthdata.cpp"
            }
            else if (__label__ == 71) {
  
              var $122=HEAP8[(((_mbrola_name)&4294967295))]; //@line 771 "synthdata.cpp"
              var $conv197=(tempInt=(($122)),(tempInt>=128?tempInt-256:tempInt)); //@line 771 "synthdata.cpp"
              var $cmp198=((($conv197))|0)!=0; //@line 771 "synthdata.cpp"
              $retval=$cmp198; //@line 771 "synthdata.cpp"
              __label__ = 73;break $if_then$$if_else187$2; //@line 771 "synthdata.cpp"
            }
          }
        } while(0);
  
        $retval=0; //@line 774 "synthdata.cpp"
        ; //@line 774 "synthdata.cpp"
      }
    } while(0);
  
    var $123=$retval; //@line 775 "synthdata.cpp"
    ;
    return $123; //@line 775 "synthdata.cpp"
    return null;
  }
  

  function __ZL13NumInstnWordsPt($prog) {
    ;
    var __label__;
  
    var $retval;
    var $prog_addr;
    var $instn;
    var $instn2;
    var $instn_type;
    $prog_addr=$prog;
    var $0=$prog_addr; //@line 814 "synthdata.cpp"
    var $1=HEAPU16[(($0)>>1)]; //@line 814 "synthdata.cpp"
    var $conv=((($1))&65535); //@line 814 "synthdata.cpp"
    $instn=$conv; //@line 814 "synthdata.cpp"
    var $2=$instn; //@line 815 "synthdata.cpp"
    var $shr=($2) >> 12; //@line 815 "synthdata.cpp"
    $instn_type=$shr; //@line 815 "synthdata.cpp"
    var $3=$instn_type; //@line 816 "synthdata.cpp"
    var $cmp=((($3))|0) < 11; //@line 816 "synthdata.cpp"
    if ($cmp) { __label__ = 1;; } else { __label__ = 2;; } //@line 816 "synthdata.cpp"
    if (__label__ == 1) {
  
      var $4=$instn_type; //@line 817 "synthdata.cpp"
      var $arrayidx=((__ZZL13NumInstnWordsPtE7n_words+$4)&4294967295); //@line 817 "synthdata.cpp"
      var $5=HEAP8[($arrayidx)]; //@line 817 "synthdata.cpp"
      var $conv1=(tempInt=(($5)),(tempInt>=128?tempInt-256:tempInt)); //@line 817 "synthdata.cpp"
      $retval=$conv1; //@line 817 "synthdata.cpp"
      ; //@line 817 "synthdata.cpp"
    }
    else if (__label__ == 2) {
  
      var $6=$prog_addr; //@line 820 "synthdata.cpp"
      var $arrayidx2=(($6+4)&4294967295); //@line 820 "synthdata.cpp"
      var $7=HEAPU16[(($arrayidx2)>>1)]; //@line 820 "synthdata.cpp"
      var $conv3=((($7))&65535); //@line 820 "synthdata.cpp"
      $instn2=$conv3; //@line 820 "synthdata.cpp"
      var $8=$instn2; //@line 821 "synthdata.cpp"
      var $shr4=($8) >> 12; //@line 821 "synthdata.cpp"
      var $cmp5=((($shr4))|0)==15; //@line 821 "synthdata.cpp"
      if ($cmp5) { __label__ = 3;; } else { __label__ = 4;; } //@line 821 "synthdata.cpp"
      if (__label__ == 3) {
  
        $retval=4; //@line 824 "synthdata.cpp"
        ; //@line 824 "synthdata.cpp"
      }
      else if (__label__ == 4) {
  
        var $9=$instn2; //@line 826 "synthdata.cpp"
        var $cmp8=((($9))|0)==2; //@line 826 "synthdata.cpp"
        if ($cmp8) { __label__ = 5;; } else { __label__ = 6;; } //@line 826 "synthdata.cpp"
        if (__label__ == 5) {
  
          $retval=3; //@line 828 "synthdata.cpp"
          ; //@line 828 "synthdata.cpp"
        }
        else if (__label__ == 6) {
  
          $retval=2; //@line 830 "synthdata.cpp"
          ; //@line 830 "synthdata.cpp"
        }
      }
    }
  
    var $10=$retval; //@line 831 "synthdata.cpp"
    ;
    return $10; //@line 831 "synthdata.cpp"
    return null;
  }
  

  function __ZL15SwitchVowelTypeP12PHONEME_LISTP12PHONEME_DATAPPti($plist, $phdata, $p_prog, $instn_type) {
    ;
    var __label__;
  
    var $plist_addr;
    var $phdata_addr;
    var $p_prog_addr;
    var $instn_type_addr;
    var $prog;
    var $voweltype;
    var $x;
    $plist_addr=$plist;
    $phdata_addr=$phdata;
    $p_prog_addr=$p_prog;
    $instn_type_addr=$instn_type;
    var $0=$instn_type_addr; //@line 784 "synthdata.cpp"
    var $cmp=((($0))|0)==2; //@line 784 "synthdata.cpp"
    if ($cmp) { __label__ = 1;; } else { __label__ = 2;; } //@line 784 "synthdata.cpp"
    if (__label__ == 1) {
  
      var $1=$phdata_addr; //@line 786 "synthdata.cpp"
      var $pd_control=(($1)&4294967295); //@line 786 "synthdata.cpp"
      var $2=HEAP32[(($pd_control)>>2)]; //@line 786 "synthdata.cpp"
      var $or=($2) | 2; //@line 786 "synthdata.cpp"
      HEAP32[(($pd_control)>>2)]=$or; //@line 786 "synthdata.cpp"
      var $3=$plist_addr; //@line 787 "synthdata.cpp"
      var $arrayidx=(($3+24)&4294967295); //@line 787 "synthdata.cpp"
      var $ph=(($arrayidx+8)&4294967295); //@line 787 "synthdata.cpp"
      var $4=HEAP32[(($ph)>>2)]; //@line 787 "synthdata.cpp"
      var $start_type=(($4+12)&4294967295); //@line 787 "synthdata.cpp"
      var $5=HEAPU8[($start_type)]; //@line 787 "synthdata.cpp"
      var $conv=((($5))&255); //@line 787 "synthdata.cpp"
      $voweltype=$conv; //@line 787 "synthdata.cpp"
      ; //@line 788 "synthdata.cpp"
    }
    else if (__label__ == 2) {
  
      var $6=$plist_addr; //@line 791 "synthdata.cpp"
      var $arrayidx1=(($6+-24)&4294967295); //@line 791 "synthdata.cpp"
      var $ph2=(($arrayidx1+8)&4294967295); //@line 791 "synthdata.cpp"
      var $7=HEAP32[(($ph2)>>2)]; //@line 791 "synthdata.cpp"
      var $end_type=(($7+13)&4294967295); //@line 791 "synthdata.cpp"
      var $8=HEAPU8[($end_type)]; //@line 791 "synthdata.cpp"
      var $conv3=((($8))&255); //@line 791 "synthdata.cpp"
      $voweltype=$conv3; //@line 791 "synthdata.cpp"
      ;
    }
  
    var $9=$voweltype; //@line 794 "synthdata.cpp"
    var $sub=((($9)-28)&4294967295); //@line 794 "synthdata.cpp"
    $voweltype=$sub; //@line 794 "synthdata.cpp"
    var $10=$voweltype; //@line 795 "synthdata.cpp"
    var $cmp4=((($10))|0) >= 0; //@line 795 "synthdata.cpp"
    var $11=$voweltype; //@line 795 "synthdata.cpp"
    var $cmp5=((($11))|0) < 6; //@line 795 "synthdata.cpp"
    var $or_cond=($cmp4) & ($cmp5); //@line 795 "synthdata.cpp"
    if ($or_cond) { __label__ = 4;; } else { __label__ = 5;; } //@line 795 "synthdata.cpp"
    if (__label__ == 4) {
  
      var $12=$p_prog_addr; //@line 797 "synthdata.cpp"
      var $13=HEAP32[(($12)>>2)]; //@line 797 "synthdata.cpp"
      var $14=$voweltype; //@line 797 "synthdata.cpp"
      var $mul=((($14)*2)&4294967295); //@line 797 "synthdata.cpp"
      var $add_ptr=(($13+2*$mul)&4294967295); //@line 797 "synthdata.cpp"
      $prog=$add_ptr; //@line 797 "synthdata.cpp"
      var $15=$prog; //@line 798 "synthdata.cpp"
      var $arrayidx7=(($15+2)&4294967295); //@line 798 "synthdata.cpp"
      var $16=HEAPU16[(($arrayidx7)>>1)]; //@line 798 "synthdata.cpp"
      var $conv8=((($16))&65535); //@line 798 "synthdata.cpp"
      var $and=($conv8) & 15; //@line 798 "synthdata.cpp"
      var $shl=((($and)*65536)&4294967295); //@line 798 "synthdata.cpp"
      var $17=$prog; //@line 798 "synthdata.cpp"
      var $arrayidx9=(($17+4)&4294967295); //@line 798 "synthdata.cpp"
      var $18=HEAPU16[(($arrayidx9)>>1)]; //@line 798 "synthdata.cpp"
      var $conv10=((($18))&65535); //@line 798 "synthdata.cpp"
      var $add=((($shl)+($conv10))&4294967295); //@line 798 "synthdata.cpp"
      var $mul11=((($add)*4)&4294967295); //@line 798 "synthdata.cpp"
      var $19=$instn_type_addr; //@line 798 "synthdata.cpp"
      var $20=$phdata_addr; //@line 798 "synthdata.cpp"
      var $sound_addr=(($20+68)&4294967295); //@line 798 "synthdata.cpp"
      var $arrayidx12=(($sound_addr+$19*4)&4294967295); //@line 798 "synthdata.cpp"
      HEAP32[(($arrayidx12)>>2)]=$mul11; //@line 798 "synthdata.cpp"
      var $21=$prog; //@line 799 "synthdata.cpp"
      var $arrayidx13=(($21+2)&4294967295); //@line 799 "synthdata.cpp"
      var $22=HEAPU16[(($arrayidx13)>>1)]; //@line 799 "synthdata.cpp"
      var $conv14=((($22))&65535); //@line 799 "synthdata.cpp"
      var $shr=($conv14) >> 4; //@line 799 "synthdata.cpp"
      var $and15=($shr) & 255; //@line 799 "synthdata.cpp"
      var $conv16=((($and15)) & 255); //@line 799 "synthdata.cpp"
      $x=$conv16; //@line 799 "synthdata.cpp"
      var $23=$x; //@line 800 "synthdata.cpp"
      var $conv17=(tempInt=(($23)),(tempInt>=128?tempInt-256:tempInt)); //@line 800 "synthdata.cpp"
      var $24=$instn_type_addr; //@line 800 "synthdata.cpp"
      var $25=$phdata_addr; //@line 800 "synthdata.cpp"
      var $sound_param=(($25+88)&4294967295); //@line 800 "synthdata.cpp"
      var $arrayidx18=(($sound_param+$24*4)&4294967295); //@line 800 "synthdata.cpp"
      HEAP32[(($arrayidx18)>>2)]=$conv17; //@line 800 "synthdata.cpp"
      ; //@line 801 "synthdata.cpp"
    }
  
    var $26=$p_prog_addr; //@line 803 "synthdata.cpp"
    var $27=HEAP32[(($26)>>2)]; //@line 803 "synthdata.cpp"
    var $add_ptr20=(($27+24)&4294967295); //@line 803 "synthdata.cpp"
    HEAP32[(($26)>>2)]=$add_ptr20; //@line 803 "synthdata.cpp"
    ;
    return; //@line 804 "synthdata.cpp"
    return;
  }
  

  function __Z17InterpretPhoneme2iP12PHONEME_DATA($phcode, $phdata) {
    var __stackBase__  = STACKTOP; STACKTOP += 96; _memset(__stackBase__, 0, 96);
    var __label__;
  
    var $phcode_addr;
    var $phdata_addr;
    var $ix;
    var $plist=__stackBase__;
    $phcode_addr=$phcode;
    $phdata_addr=$phdata;
    var $arraydecay=(($plist)&4294967295); //@line 1102 "synthdata.cpp"
    var $0=$arraydecay; //@line 1102 "synthdata.cpp"
    var $dest$, $stop$, $stop4$, $fast$, $value4$;
    $dest$ = $0;
    $stop$ = $dest$ + 96;
    $value4$ = 0;
    if ($value4$ < 0) $value4$ += 256;
    $value4$ = $value4$ + ($value4$<<8) + ($value4$<<16) + ($value4$*16777216);
    while ($dest$%4 !== 0 && $dest$ < $stop$) {
      ;; HEAP8[$dest$++] = 0;
    }
    $dest$ >>= 2;
    $stop4$ = $stop$ >> 2;
    while ($dest$ < $stop4$) {
    ;;;;;;;  HEAP32[$dest$++] = $value4$;
    }
    $dest$ <<= 2;
    while ($dest$ < $stop$) {
      ;; HEAP8[$dest$++] = 0;
    }; //@line 1102 "synthdata.cpp"
    $ix=0; //@line 1104 "synthdata.cpp"
    ; //@line 1104 "synthdata.cpp"
    while(1) { 
  
      var $1=$ix; //@line 1106 "synthdata.cpp"
      var $arrayidx=(($plist+$1*24)&4294967295); //@line 1106 "synthdata.cpp"
      var $phcode1=(($arrayidx)&4294967295); //@line 1106 "synthdata.cpp"
      HEAP8[($phcode1)]=9; //@line 1106 "synthdata.cpp"
      var $2=HEAP32[((((_phoneme_tab+36)&4294967295))>>2)]; //@line 1107 "synthdata.cpp"
      var $3=$ix; //@line 1107 "synthdata.cpp"
      var $arrayidx2=(($plist+$3*24)&4294967295); //@line 1107 "synthdata.cpp"
      var $ph=(($arrayidx2+8)&4294967295); //@line 1107 "synthdata.cpp"
      HEAP32[(($ph)>>2)]=$2; //@line 1107 "synthdata.cpp"
      var $4=$ix; //@line 1104 "synthdata.cpp"
      var $inc=((($4)+1)&4294967295); //@line 1104 "synthdata.cpp"
      $ix=$inc; //@line 1104 "synthdata.cpp"
      var $cmp=((($inc))|0) < 4; //@line 1104 "synthdata.cpp"
      if ($cmp) { __label__ = 1;continue ; } else { __label__ = 2;break ; } //@line 1104 "synthdata.cpp"
    }
  
    var $5=$phcode_addr; //@line 1110 "synthdata.cpp"
    var $conv=((($5)) & 255); //@line 1110 "synthdata.cpp"
    var $arrayidx3=(($plist+24)&4294967295); //@line 1110 "synthdata.cpp"
    var $phcode4=(($arrayidx3)&4294967295); //@line 1110 "synthdata.cpp"
    HEAP8[($phcode4)]=$conv; //@line 1110 "synthdata.cpp"
    var $6=$phcode_addr; //@line 1111 "synthdata.cpp"
    var $arrayidx5=((_phoneme_tab+$6*4)&4294967295); //@line 1111 "synthdata.cpp"
    var $7=HEAP32[(($arrayidx5)>>2)]; //@line 1111 "synthdata.cpp"
    var $arrayidx6=(($plist+24)&4294967295); //@line 1111 "synthdata.cpp"
    var $ph7=(($arrayidx6+8)&4294967295); //@line 1111 "synthdata.cpp"
    HEAP32[(($ph7)>>2)]=$7; //@line 1111 "synthdata.cpp"
    var $arrayidx8=(($plist+48)&4294967295); //@line 1112 "synthdata.cpp"
    var $sourceix=(($arrayidx8+6)&4294967295); //@line 1112 "synthdata.cpp"
    HEAP16[(($sourceix)>>1)]=1; //@line 1112 "synthdata.cpp"
    var $arrayidx9=(($plist+24)&4294967295); //@line 1114 "synthdata.cpp"
    var $8=$phdata_addr; //@line 1114 "synthdata.cpp"
    __Z16InterpretPhonemeP10TranslatoriP12PHONEME_LISTP12PHONEME_DATA(0, 0, $arrayidx9, $8); //@line 1114 "synthdata.cpp"
    STACKTOP = __stackBase__;
    return; //@line 1115 "synthdata.cpp"
    return;
  }
  

  function __Z12WordToStringj($word) {
    ;
    var __label__;
  
    var $word_addr;
    var $ix;
    $word_addr=$word;
    $ix=0; //@line 85 "synthesize.cpp"
    ; //@line 85 "synthesize.cpp"
    while(1) { 
  
      var $0=$word_addr; //@line 86 "synthesize.cpp"
      var $1=$ix; //@line 86 "synthesize.cpp"
      var $mul=((($1)*8)&4294967295); //@line 86 "synthesize.cpp"
      var $shr=($0) >>> ((($mul))>>>0); //@line 86 "synthesize.cpp"
      var $conv=((($shr)) & 255); //@line 86 "synthesize.cpp"
      var $2=$ix; //@line 86 "synthesize.cpp"
      var $arrayidx=((__ZZ12WordToStringjE3buf+$2)&4294967295); //@line 86 "synthesize.cpp"
      HEAP8[($arrayidx)]=$conv; //@line 86 "synthesize.cpp"
      var $3=$ix; //@line 85 "synthesize.cpp"
      var $inc=((($3)+1)&4294967295); //@line 85 "synthesize.cpp"
      $ix=$inc; //@line 85 "synthesize.cpp"
      var $cmp=((($inc))|0) < 3; //@line 85 "synthesize.cpp"
      if ($cmp) { __label__ = 1;continue ; } else { __label__ = 2;break ; } //@line 85 "synthesize.cpp"
    }
  
    HEAP8[(((__ZZ12WordToStringjE3buf+4)&4294967295))]=0; //@line 87 "synthesize.cpp"
    ;
    return ((__ZZ12WordToStringjE3buf)&4294967295); //@line 88 "synthesize.cpp"
    return null;
  }
  

  function __Z14SynthesizeInitv() {
    ;
    var __label__;
  
    HEAP32[((__ZL14last_pitch_cmd)>>2)]=0; //@line 95 "synthesize.cpp"
    HEAP32[((__ZL12last_amp_cmd)>>2)]=0; //@line 96 "synthesize.cpp"
    HEAP32[((__ZL10last_frame)>>2)]=0; //@line 97 "synthesize.cpp"
    HEAP32[((__ZL15syllable_centre)>>2)]=-1; //@line 98 "synthesize.cpp"
    ;
    return; //@line 104 "synthesize.cpp"
    return;
  }
  

  function __Z11PauseLengthii($pause, $control) {
    ;
    var __label__;
  
    var $pause_addr;
    var $control_addr;
    var $len;
    $pause_addr=$pause;
    $control_addr=$control;
    var $0=$control_addr; //@line 193 "synthesize.cpp"
    var $cmp=((($0))|0)==0; //@line 193 "synthesize.cpp"
    var $1=$pause_addr; //@line 195 "synthesize.cpp"
    if ($cmp) { __label__ = 1;; } else { __label__ = 4;; } //@line 193 "synthesize.cpp"
    if (__label__ == 1) {
  
      var $cmp1=((($1))|0) >= 200; //@line 195 "synthesize.cpp"
      var $2=$pause_addr; //@line 196 "synthesize.cpp"
      if ($cmp1) { __label__ = 2;; } else { __label__ = 3;; } //@line 195 "synthesize.cpp"
      if (__label__ == 2) {
  
        var $3=HEAP32[((((_speed+4)&4294967295))>>2)]; //@line 196 "synthesize.cpp"
        var $mul=((($3)*($2))&4294967295); //@line 196 "synthesize.cpp"
        var $div=((((($mul))|0)/256)|0); //@line 196 "synthesize.cpp"
        $len=$div; //@line 196 "synthesize.cpp"
        ; //@line 196 "synthesize.cpp"
      }
      else if (__label__ == 3) {
  
        var $4=HEAP32[((((_speed)&4294967295))>>2)]; //@line 198 "synthesize.cpp"
        var $mul3=((($4)*($2))&4294967295); //@line 198 "synthesize.cpp"
        var $div4=((((($mul3))|0)/256)|0); //@line 198 "synthesize.cpp"
        $len=$div4; //@line 198 "synthesize.cpp"
        ;
      }
    }
    else if (__label__ == 4) {
  
      var $5=HEAP32[((((_speed+12)&4294967295))>>2)]; //@line 201 "synthesize.cpp"
      var $mul6=((($5)*($1))&4294967295); //@line 201 "synthesize.cpp"
      var $div7=((((($mul6))|0)/256)|0); //@line 201 "synthesize.cpp"
      $len=$div7; //@line 201 "synthesize.cpp"
      ;
    }
  
    var $6=$len; //@line 203 "synthesize.cpp"
    var $7=HEAP32[((((_speed+8)&4294967295))>>2)]; //@line 203 "synthesize.cpp"
    var $cmp9=((($6))|0) < ((($7))|0); //@line 203 "synthesize.cpp"
    if ($cmp9) { __label__ = 6;; } else { __label__ = 7;; } //@line 203 "synthesize.cpp"
    if (__label__ == 6) {
  
      var $8=HEAP32[((((_speed+8)&4294967295))>>2)]; //@line 205 "synthesize.cpp"
      $len=$8; //@line 205 "synthesize.cpp"
      ; //@line 206 "synthesize.cpp"
    }
  
    var $9=$len; //@line 207 "synthesize.cpp"
    ;
    return $9; //@line 207 "synthesize.cpp"
    return null;
  }
  

  function __Z9DoSample3P12PHONEME_DATAii($phdata, $length_mod, $amp) {
    ;
    var __label__;
  
    var $phdata_addr;
    var $length_mod_addr;
    var $amp_addr;
    var $amp2;
    var $len;
    $phdata_addr=$phdata;
    $length_mod_addr=$length_mod;
    $amp_addr=$amp;
    __ZL8EndPitchi(1); //@line 405 "synthesize.cpp"
    var $0=$amp_addr; //@line 407 "synthesize.cpp"
    var $cmp=((($0))|0)==-1; //@line 407 "synthesize.cpp"
    if ($cmp) { __label__ = 1;; } else { __label__ = 2;; } //@line 407 "synthesize.cpp"
    if (__label__ == 1) {
  
      var $1=$amp_addr; //@line 410 "synthesize.cpp"
      $amp2=$1; //@line 410 "synthesize.cpp"
      ; //@line 411 "synthesize.cpp"
    }
    else if (__label__ == 2) {
  
      var $2=$phdata_addr; //@line 414 "synthesize.cpp"
      var $sound_param=(($2+88)&4294967295); //@line 414 "synthesize.cpp"
      var $arrayidx=(($sound_param+4)&4294967295); //@line 414 "synthesize.cpp"
      var $3=HEAP32[(($arrayidx)>>2)]; //@line 414 "synthesize.cpp"
      $amp2=$3; //@line 414 "synthesize.cpp"
      var $4=$amp2; //@line 415 "synthesize.cpp"
      var $cmp1=((($4))|0)==0; //@line 415 "synthesize.cpp"
      if ($cmp1) { __label__ = 3;; } else { __label__ = 4;; } //@line 415 "synthesize.cpp"
      if (__label__ == 3) {
  
        $amp2=100; //@line 416 "synthesize.cpp"
        ; //@line 416 "synthesize.cpp"
      }
  
      var $5=$amp2; //@line 417 "synthesize.cpp"
      var $mul=((($5)*32)&4294967295); //@line 417 "synthesize.cpp"
      var $div=((((($mul))|0)/100)|0); //@line 417 "synthesize.cpp"
      $amp2=$div; //@line 417 "synthesize.cpp"
      ;
    }
  
    HEAP32[((_seq_len_adjust)>>2)]=0; //@line 420 "synthesize.cpp"
    var $6=$phdata_addr; //@line 422 "synthesize.cpp"
    var $sound_addr=(($6+68)&4294967295); //@line 422 "synthesize.cpp"
    var $arrayidx4=(($sound_addr+4)&4294967295); //@line 422 "synthesize.cpp"
    var $7=HEAP32[(($arrayidx4)>>2)]; //@line 422 "synthesize.cpp"
    var $cmp5=((($7))|0)==0; //@line 422 "synthesize.cpp"
    if ($cmp5) { __label__ = 6;; } else { __label__ = 7;; } //@line 422 "synthesize.cpp"
    if (__label__ == 6) {
  
      $len=0; //@line 424 "synthesize.cpp"
      ; //@line 425 "synthesize.cpp"
    }
    else if (__label__ == 7) {
  
      var $8=$phdata_addr; //@line 428 "synthesize.cpp"
      var $sound_addr8=(($8+68)&4294967295); //@line 428 "synthesize.cpp"
      var $arrayidx9=(($sound_addr8+4)&4294967295); //@line 428 "synthesize.cpp"
      var $9=HEAP32[(($arrayidx9)>>2)]; //@line 428 "synthesize.cpp"
      var $10=$phdata_addr; //@line 428 "synthesize.cpp"
      var $pd_param=(($10+4)&4294967295); //@line 428 "synthesize.cpp"
      var $arrayidx10=(($pd_param+40)&4294967295); //@line 428 "synthesize.cpp"
      var $11=HEAP32[(($arrayidx10)>>2)]; //@line 428 "synthesize.cpp"
      var $mul11=((($11)*2)&4294967295); //@line 428 "synthesize.cpp"
      var $12=$phdata_addr; //@line 428 "synthesize.cpp"
      var $pd_control=(($12)&4294967295); //@line 428 "synthesize.cpp"
      var $13=HEAP32[(($pd_control)>>2)]; //@line 428 "synthesize.cpp"
      var $14=$length_mod_addr; //@line 428 "synthesize.cpp"
      var $15=$amp2; //@line 428 "synthesize.cpp"
      var $call=__ZL9DoSample2iiiiii($9, 2, $mul11, $13, $14, $15); //@line 428 "synthesize.cpp"
      $len=$call; //@line 428 "synthesize.cpp"
      ;
    }
  
    HEAP32[((__ZL10last_frame)>>2)]=0; //@line 430 "synthesize.cpp"
    var $16=$len; //@line 431 "synthesize.cpp"
    ;
    return $16; //@line 431 "synthesize.cpp"
    return null;
  }
  

  function __ZL8EndPitchi($voice_break) {
    ;
    var __label__;
  
    var $voice_break_addr;
    $voice_break_addr=$voice_break;
    var $0=HEAP32[((__ZL12pitch_length)>>2)]; //@line 123 "synthesize.cpp"
    var $cmp=((($0))|0) > 0; //@line 123 "synthesize.cpp"
    var $1=HEAP32[((__ZL14last_pitch_cmd)>>2)]; //@line 123 "synthesize.cpp"
    var $cmp1=((($1))|0) >= 0; //@line 123 "synthesize.cpp"
    var $or_cond=($cmp) & ($cmp1); //@line 123 "synthesize.cpp"
    if ($or_cond) { __label__ = 1;; } else { __label__ = 4;; } //@line 123 "synthesize.cpp"
    if (__label__ == 1) {
  
      var $2=HEAP32[((__ZL14last_pitch_cmd)>>2)]; //@line 125 "synthesize.cpp"
      var $arrayidx=((_wcmdq+$2*16)&4294967295); //@line 125 "synthesize.cpp"
      var $arrayidx2=(($arrayidx+4)&4294967295); //@line 125 "synthesize.cpp"
      var $3=HEAP32[(($arrayidx2)>>2)]; //@line 125 "synthesize.cpp"
      var $cmp3=((($3))|0)==0; //@line 125 "synthesize.cpp"
      if ($cmp3) { __label__ = 2;; } else { __label__ = 3;; } //@line 125 "synthesize.cpp"
      if (__label__ == 2) {
  
        var $4=HEAP32[((__ZL12pitch_length)>>2)]; //@line 126 "synthesize.cpp"
        var $5=HEAP32[((__ZL14last_pitch_cmd)>>2)]; //@line 126 "synthesize.cpp"
        var $arrayidx5=((_wcmdq+$5*16)&4294967295); //@line 126 "synthesize.cpp"
        var $arrayidx6=(($arrayidx5+4)&4294967295); //@line 126 "synthesize.cpp"
        HEAP32[(($arrayidx6)>>2)]=$4; //@line 126 "synthesize.cpp"
        ; //@line 126 "synthesize.cpp"
      }
  
      HEAP32[((__ZL12pitch_length)>>2)]=0; //@line 127 "synthesize.cpp"
      ; //@line 128 "synthesize.cpp"
    }
  
    var $6=$voice_break_addr; //@line 130 "synthesize.cpp"
    var $tobool=((($6))|0)!=0; //@line 130 "synthesize.cpp"
    if ($tobool) { __label__ = 5;; } else { __label__ = 6;; } //@line 130 "synthesize.cpp"
    if (__label__ == 5) {
  
      HEAP32[((__ZL10last_wcmdq)>>2)]=-1; //@line 132 "synthesize.cpp"
      HEAP32[((__ZL10last_frame)>>2)]=0; //@line 133 "synthesize.cpp"
      var $7=HEAP32[((_wcmdq_tail)>>2)]; //@line 134 "synthesize.cpp"
      HEAP32[((__ZL12syllable_end)>>2)]=$7; //@line 134 "synthesize.cpp"
      __ZL11SmoothSpectv(); //@line 135 "synthesize.cpp"
      HEAP32[((__ZL15syllable_centre)>>2)]=-1; //@line 136 "synthesize.cpp"
      var $dest$, $stop$, $stop4$, $fast$, $value4$;
      $dest$ = _vowel_transition;
      $stop$ = $dest$ + 16;
      $value4$ = 0;
      if ($value4$ < 0) $value4$ += 256;
      $value4$ = $value4$ + ($value4$<<8) + ($value4$<<16) + ($value4$*16777216);
      while ($dest$%4 !== 0 && $dest$ < $stop$) {
        ;; HEAP8[$dest$++] = 0;
      }
      $dest$ >>= 2;
      $stop4$ = $stop$ >> 2;
      while ($dest$ < $stop4$) {
      ;;;;;;;  HEAP32[$dest$++] = $value4$;
      }
      $dest$ <<= 2;
      while ($dest$ < $stop$) {
        ;; HEAP8[$dest$++] = 0;
      }; //@line 137 "synthesize.cpp"
      ; //@line 138 "synthesize.cpp"
    }
  
    ;
    return; //@line 139 "synthesize.cpp"
    return;
  }
  

  function __ZL9DoSample2iiiiii($index, $which, $std_length, $control, $length_mod, $amp) {
    ;
    var __label__;
    var __lastLabel__ = null;
  
    var $retval;
    var $index_addr;
    var $which_addr;
    var $std_length_addr;
    var $control_addr;
    var $length_mod_addr;
    var $amp_addr;
    var $length;
    var $wav_length;
    var $wav_scale;
    var $min_length;
    var $x;
    var $len4;
    var $q;
    var $p;
    $index_addr=$index;
    $which_addr=$which;
    $std_length_addr=$std_length;
    $control_addr=$control;
    $length_mod_addr=$length_mod;
    $amp_addr=$amp;
    var $0=$index_addr; //@line 254 "synthesize.cpp"
    var $and=($0) & 8388607; //@line 254 "synthesize.cpp"
    $index_addr=$and; //@line 254 "synthesize.cpp"
    var $1=$index_addr; //@line 255 "synthesize.cpp"
    var $2=HEAP32[((_wavefile_data)>>2)]; //@line 255 "synthesize.cpp"
    var $arrayidx=(($2+$1)&4294967295); //@line 255 "synthesize.cpp"
    $p=$arrayidx; //@line 255 "synthesize.cpp"
    var $3=$p; //@line 256 "synthesize.cpp"
    var $arrayidx1=(($3+2)&4294967295); //@line 256 "synthesize.cpp"
    var $4=HEAPU8[($arrayidx1)]; //@line 256 "synthesize.cpp"
    var $conv=((($4))&255); //@line 256 "synthesize.cpp"
    $wav_scale=$conv; //@line 256 "synthesize.cpp"
    var $5=$p; //@line 257 "synthesize.cpp"
    var $arrayidx2=(($5+1)&4294967295); //@line 257 "synthesize.cpp"
    var $6=HEAPU8[($arrayidx2)]; //@line 257 "synthesize.cpp"
    var $conv3=((($6))&255); //@line 257 "synthesize.cpp"
    var $mul=((($conv3)*256)&4294967295); //@line 257 "synthesize.cpp"
    $wav_length=$mul; //@line 257 "synthesize.cpp"
    var $7=$p; //@line 258 "synthesize.cpp"
    var $arrayidx4=(($7)&4294967295); //@line 258 "synthesize.cpp"
    var $8=HEAPU8[($arrayidx4)]; //@line 258 "synthesize.cpp"
    var $conv5=((($8))&255); //@line 258 "synthesize.cpp"
    var $9=$wav_length; //@line 258 "synthesize.cpp"
    var $add=((($9)+($conv5))&4294967295); //@line 258 "synthesize.cpp"
    $wav_length=$add; //@line 258 "synthesize.cpp"
    var $10=$wav_length; //@line 260 "synthesize.cpp"
    var $cmp=((($10))|0)==0; //@line 260 "synthesize.cpp"
    if ($cmp) { __label__ = 1;; } else { __label__ = 2;; } //@line 260 "synthesize.cpp"
    if (__label__ == 1) {
  
      $retval=0; //@line 261 "synthesize.cpp"
      ; //@line 261 "synthesize.cpp"
    }
    else if (__label__ == 2) {
  
      var $11=HEAP32[((((_speed+24)&4294967295))>>2)]; //@line 263 "synthesize.cpp"
      $min_length=$11; //@line 263 "synthesize.cpp"
      var $12=$wav_scale; //@line 265 "synthesize.cpp"
      var $cmp6=((($12))|0)==0; //@line 265 "synthesize.cpp"
      if ($cmp6) { __label__ = 3;; } else { __label__ = 4;; } //@line 265 "synthesize.cpp"
      if (__label__ == 3) {
  
        var $13=$min_length; //@line 266 "synthesize.cpp"
        var $mul8=((($13)*2)&4294967295); //@line 266 "synthesize.cpp"
        $min_length=$mul8; //@line 266 "synthesize.cpp"
        ; //@line 266 "synthesize.cpp"
      }
  
      var $14=$std_length_addr; //@line 275 "synthesize.cpp"
      var $cmp10=((($14))|0) > 0; //@line 275 "synthesize.cpp"
      if ($cmp10) { __label__ = 5;; } else { __label__ = 9;; } //@line 275 "synthesize.cpp"
      $if_then11$$if_else22$8: do { 
        if (__label__ == 5) {
  
          var $15=$std_length_addr; //@line 277 "synthesize.cpp"
          var $16=HEAP32[((_samplerate)>>2)]; //@line 277 "synthesize.cpp"
          var $mul12=((($16)*($15))&4294967295); //@line 277 "synthesize.cpp"
          var $div=((((($mul12))|0)/1000)|0); //@line 277 "synthesize.cpp"
          $std_length_addr=$div; //@line 277 "synthesize.cpp"
          var $17=$wav_scale; //@line 278 "synthesize.cpp"
          var $cmp13=((($17))|0)==0; //@line 278 "synthesize.cpp"
          if ($cmp13) { __label__ = 6;; } else { __label__ = 7;; } //@line 278 "synthesize.cpp"
          if (__label__ == 6) {
  
            var $18=$std_length_addr; //@line 279 "synthesize.cpp"
            var $mul15=((($18)*2)&4294967295); //@line 279 "synthesize.cpp"
            $std_length_addr=$mul15; //@line 279 "synthesize.cpp"
            ; //@line 279 "synthesize.cpp"
          }
  
          var $19=$min_length; //@line 281 "synthesize.cpp"
          var $20=$std_length_addr; //@line 281 "synthesize.cpp"
          var $mul17=((($20)*($19))&4294967295); //@line 281 "synthesize.cpp"
          var $21=$wav_length; //@line 281 "synthesize.cpp"
          var $div18=((((($mul17))|0)/((($21))|0))|0); //@line 281 "synthesize.cpp"
          $x=$div18; //@line 281 "synthesize.cpp"
          var $22=$x; //@line 282 "synthesize.cpp"
          var $23=$min_length; //@line 282 "synthesize.cpp"
          var $cmp19=((($22))|0) > ((($23))|0); //@line 282 "synthesize.cpp"
          if (!($cmp19)) { __label__ = 10;break $if_then11$$if_else22$8; } //@line 282 "synthesize.cpp"
  
          var $24=$x; //@line 283 "synthesize.cpp"
          $min_length=$24; //@line 283 "synthesize.cpp"
          ; //@line 283 "synthesize.cpp"
        }
        else if (__label__ == 9) {
  
          var $25=$wav_length; //@line 288 "synthesize.cpp"
          $std_length_addr=$25; //@line 288 "synthesize.cpp"
          ;
        }
      } while(0);
  
      var $26=$length_mod_addr; //@line 291 "synthesize.cpp"
      var $cmp24=((($26))|0) > 0; //@line 291 "synthesize.cpp"
      if ($cmp24) { __label__ = 11;; } else { __label__ = 12;; } //@line 291 "synthesize.cpp"
      if (__label__ == 11) {
  
        var $27=$std_length_addr; //@line 293 "synthesize.cpp"
        var $28=$length_mod_addr; //@line 293 "synthesize.cpp"
        var $mul26=((($28)*($27))&4294967295); //@line 293 "synthesize.cpp"
        var $div27=((((($mul26))|0)/256)|0); //@line 293 "synthesize.cpp"
        $std_length_addr=$div27; //@line 293 "synthesize.cpp"
        ; //@line 294 "synthesize.cpp"
      }
  
      var $29=$std_length_addr; //@line 296 "synthesize.cpp"
      var $30=HEAP32[((((_speed+12)&4294967295))>>2)]; //@line 296 "synthesize.cpp"
      var $mul29=((($30)*($29))&4294967295); //@line 296 "synthesize.cpp"
      var $div30=((((($mul29))|0)/256)|0); //@line 296 "synthesize.cpp"
      $length=$div30; //@line 296 "synthesize.cpp"
      var $31=$control_addr; //@line 298 "synthesize.cpp"
      var $and31=($31) & 4; //@line 298 "synthesize.cpp"
      var $tobool=((($and31))|0)!=0; //@line 298 "synthesize.cpp"
      if ($tobool) { __label__ = 13;; } else { __label__ = 15;; } //@line 298 "synthesize.cpp"
      $if_then32$$if_end37$19: do { 
        if (__label__ == 13) {
  
          var $32=$length; //@line 302 "synthesize.cpp"
          var $33=$std_length_addr; //@line 302 "synthesize.cpp"
          var $cmp33=((($32))|0) > ((($33))|0); //@line 302 "synthesize.cpp"
          if (!($cmp33)) { __label__ = 15;break $if_then32$$if_end37$19; } //@line 302 "synthesize.cpp"
  
          var $34=$std_length_addr; //@line 305 "synthesize.cpp"
          $length=$34; //@line 305 "synthesize.cpp"
          ; //@line 306 "synthesize.cpp"
        }
      } while(0);
  
      var $35=$length; //@line 314 "synthesize.cpp"
      var $36=$min_length; //@line 314 "synthesize.cpp"
      var $cmp38=((($35))|0) < ((($36))|0); //@line 314 "synthesize.cpp"
      if ($cmp38) { __label__ = 16;; } else { __label__ = 17;; } //@line 314 "synthesize.cpp"
      if (__label__ == 16) {
  
        var $37=$min_length; //@line 315 "synthesize.cpp"
        $length=$37; //@line 315 "synthesize.cpp"
        ; //@line 315 "synthesize.cpp"
      }
  
      var $38=$wav_scale; //@line 318 "synthesize.cpp"
      var $cmp41=((($38))|0)==0; //@line 318 "synthesize.cpp"
      if ($cmp41) { __label__ = 18;; } else { __label__ = 19;; } //@line 318 "synthesize.cpp"
      if (__label__ == 18) {
  
        var $39=$length; //@line 321 "synthesize.cpp"
        var $div43=((((($39))|0)/2)|0); //@line 321 "synthesize.cpp"
        $length=$div43; //@line 321 "synthesize.cpp"
        var $40=$wav_length; //@line 322 "synthesize.cpp"
        var $div44=((((($40))|0)/2)|0); //@line 322 "synthesize.cpp"
        $wav_length=$div44; //@line 322 "synthesize.cpp"
        ; //@line 323 "synthesize.cpp"
      }
  
      var $41=$amp_addr; //@line 325 "synthesize.cpp"
      var $cmp46=((($41))|0) < 0; //@line 325 "synthesize.cpp"
      if ($cmp46) { __label__ = 20;; } else { __label__ = 21;; } //@line 325 "synthesize.cpp"
      if (__label__ == 20) {
  
        var $42=$length; //@line 326 "synthesize.cpp"
        $retval=$42; //@line 326 "synthesize.cpp"
        ; //@line 326 "synthesize.cpp"
      }
      else if (__label__ == 21) {
  
        var $43=$wav_length; //@line 328 "synthesize.cpp"
        var $div49=((((($43))|0)/4)|0); //@line 328 "synthesize.cpp"
        $len4=$div49; //@line 328 "synthesize.cpp"
        var $44=$index_addr; //@line 330 "synthesize.cpp"
        var $add50=((($44)+4)&4294967295); //@line 330 "synthesize.cpp"
        $index_addr=$add50; //@line 330 "synthesize.cpp"
        var $45=$which_addr; //@line 332 "synthesize.cpp"
        var $and51=($45) & 256; //@line 332 "synthesize.cpp"
        var $tobool52=((($and51))|0)!=0; //@line 332 "synthesize.cpp"
        if ($tobool52) { __label__ = 22;; } else { __label__ = 23;; } //@line 332 "synthesize.cpp"
        if (__label__ == 22) {
  
          var $46=HEAP32[((_wcmdq_tail)>>2)]; //@line 335 "synthesize.cpp"
          HEAP32[((__ZL10last_wcmdq)>>2)]=$46; //@line 335 "synthesize.cpp"
          var $47=HEAP32[((_wcmdq_tail)>>2)]; //@line 336 "synthesize.cpp"
          var $arrayidx54=((_wcmdq+$47*16)&4294967295); //@line 336 "synthesize.cpp"
          var $arraydecay=(($arrayidx54)&4294967295); //@line 336 "synthesize.cpp"
          $q=$arraydecay; //@line 336 "synthesize.cpp"
          var $48=$q; //@line 337 "synthesize.cpp"
          var $arrayidx55=(($48)&4294967295); //@line 337 "synthesize.cpp"
          HEAP32[(($arrayidx55)>>2)]=7; //@line 337 "synthesize.cpp"
          var $49=$length; //@line 338 "synthesize.cpp"
          var $50=$wav_length; //@line 338 "synthesize.cpp"
          var $shl=($50) << 16; //@line 338 "synthesize.cpp"
          var $or=($shl) | ($49); //@line 338 "synthesize.cpp"
          var $51=$q; //@line 338 "synthesize.cpp"
          var $arrayidx56=(($51+4)&4294967295); //@line 338 "synthesize.cpp"
          HEAP32[(($arrayidx56)>>2)]=$or; //@line 338 "synthesize.cpp"
          var $52=$index_addr; //@line 339 "synthesize.cpp"
          var $53=HEAP32[((_wavefile_data)>>2)]; //@line 339 "synthesize.cpp"
          var $arrayidx57=(($53+$52)&4294967295); //@line 339 "synthesize.cpp"
          var $54=($arrayidx57); //@line 339 "synthesize.cpp"
          var $55=$q; //@line 339 "synthesize.cpp"
          var $arrayidx58=(($55+8)&4294967295); //@line 339 "synthesize.cpp"
          HEAP32[(($arrayidx58)>>2)]=$54; //@line 339 "synthesize.cpp"
          var $56=$wav_scale; //@line 340 "synthesize.cpp"
          var $57=$amp_addr; //@line 340 "synthesize.cpp"
          var $shl59=((($57)*256)&4294967295); //@line 340 "synthesize.cpp"
          var $add60=((($shl59)+($56))&4294967295); //@line 340 "synthesize.cpp"
          var $58=$q; //@line 340 "synthesize.cpp"
          var $arrayidx61=(($58+12)&4294967295); //@line 340 "synthesize.cpp"
          HEAP32[(($arrayidx61)>>2)]=$add60; //@line 340 "synthesize.cpp"
          __Z8WcmdqIncv(); //@line 341 "synthesize.cpp"
          var $59=$length; //@line 342 "synthesize.cpp"
          $retval=$59; //@line 342 "synthesize.cpp"
          ; //@line 342 "synthesize.cpp"
        }
        else if (__label__ == 23) {
  
          var $60=$length; //@line 345 "synthesize.cpp"
          var $61=$wav_length; //@line 345 "synthesize.cpp"
          var $cmp63=((($60))|0) > ((($61))|0); //@line 345 "synthesize.cpp"
          if ($cmp63) { __label__ = 24;; } else { __label__ = 25;; } //@line 345 "synthesize.cpp"
          if (__label__ == 24) {
  
            var $62=$len4; //@line 347 "synthesize.cpp"
            var $mul65=((($62)*3)&4294967295); //@line 347 "synthesize.cpp"
            $x=$mul65; //@line 347 "synthesize.cpp"
            var $63=$x; //@line 348 "synthesize.cpp"
            var $64=$length; //@line 348 "synthesize.cpp"
            var $sub=((($64)-($63))&4294967295); //@line 348 "synthesize.cpp"
            $length=$sub; //@line 348 "synthesize.cpp"
            ; //@line 349 "synthesize.cpp"
          }
          else if (__label__ == 25) {
  
            var $65=$length; //@line 352 "synthesize.cpp"
            $x=$65; //@line 352 "synthesize.cpp"
            $length=0; //@line 353 "synthesize.cpp"
            ;
          }
  
          var $66=HEAP32[((_wcmdq_tail)>>2)]; //@line 356 "synthesize.cpp"
          HEAP32[((__ZL10last_wcmdq)>>2)]=$66; //@line 356 "synthesize.cpp"
          var $67=HEAP32[((_wcmdq_tail)>>2)]; //@line 357 "synthesize.cpp"
          var $arrayidx68=((_wcmdq+$67*16)&4294967295); //@line 357 "synthesize.cpp"
          var $arraydecay69=(($arrayidx68)&4294967295); //@line 357 "synthesize.cpp"
          $q=$arraydecay69; //@line 357 "synthesize.cpp"
          var $68=$q; //@line 358 "synthesize.cpp"
          var $arrayidx70=(($68)&4294967295); //@line 358 "synthesize.cpp"
          HEAP32[(($arrayidx70)>>2)]=6; //@line 358 "synthesize.cpp"
          var $69=$x; //@line 359 "synthesize.cpp"
          var $70=$q; //@line 359 "synthesize.cpp"
          var $arrayidx71=(($70+4)&4294967295); //@line 359 "synthesize.cpp"
          HEAP32[(($arrayidx71)>>2)]=$69; //@line 359 "synthesize.cpp"
          var $71=$index_addr; //@line 360 "synthesize.cpp"
          var $72=HEAP32[((_wavefile_data)>>2)]; //@line 360 "synthesize.cpp"
          var $arrayidx72=(($72+$71)&4294967295); //@line 360 "synthesize.cpp"
          var $73=($arrayidx72); //@line 360 "synthesize.cpp"
          var $74=$q; //@line 360 "synthesize.cpp"
          var $arrayidx73=(($74+8)&4294967295); //@line 360 "synthesize.cpp"
          HEAP32[(($arrayidx73)>>2)]=$73; //@line 360 "synthesize.cpp"
          var $75=$wav_scale; //@line 361 "synthesize.cpp"
          var $76=$amp_addr; //@line 361 "synthesize.cpp"
          var $shl74=((($76)*256)&4294967295); //@line 361 "synthesize.cpp"
          var $add75=((($shl74)+($75))&4294967295); //@line 361 "synthesize.cpp"
          var $77=$q; //@line 361 "synthesize.cpp"
          var $arrayidx76=(($77+12)&4294967295); //@line 361 "synthesize.cpp"
          HEAP32[(($arrayidx76)>>2)]=$add75; //@line 361 "synthesize.cpp"
          __Z8WcmdqIncv(); //@line 362 "synthesize.cpp"
          var $78=$length; //@line 365 "synthesize.cpp"
          var $79=$len4; //@line 365 "synthesize.cpp"
          var $mul771=((($79)*3)&4294967295); //@line 365 "synthesize.cpp"
          var $cmp782=((($78))|0) > ((($mul771))|0); //@line 365 "synthesize.cpp"
          if ($cmp782) { __lastLabel__ = 26; __label__ = 27;; } else { __lastLabel__ = 26; __label__ = 30;; } //@line 365 "synthesize.cpp"
          $while_body$$while_end$39: do { 
            if (__label__ == 27) {
              while(1) { 
  
                var $80=$len4; //@line 367 "synthesize.cpp"
                $x=$80; //@line 367 "synthesize.cpp"
                var $81=$wav_scale; //@line 368 "synthesize.cpp"
                var $cmp79=((($81))|0)==0; //@line 368 "synthesize.cpp"
                if ($cmp79) { __label__ = 28;; } else { __label__ = 29;; } //@line 368 "synthesize.cpp"
                if (__label__ == 28) {
  
                  var $82=$x; //@line 369 "synthesize.cpp"
                  var $mul81=((($82)*2)&4294967295); //@line 369 "synthesize.cpp"
                  $x=$mul81; //@line 369 "synthesize.cpp"
                  ; //@line 369 "synthesize.cpp"
                }
  
                var $83=HEAP32[((_wcmdq_tail)>>2)]; //@line 371 "synthesize.cpp"
                HEAP32[((__ZL10last_wcmdq)>>2)]=$83; //@line 371 "synthesize.cpp"
                var $84=HEAP32[((_wcmdq_tail)>>2)]; //@line 372 "synthesize.cpp"
                var $arrayidx83=((_wcmdq+$84*16)&4294967295); //@line 372 "synthesize.cpp"
                var $arraydecay84=(($arrayidx83)&4294967295); //@line 372 "synthesize.cpp"
                $q=$arraydecay84; //@line 372 "synthesize.cpp"
                var $85=$q; //@line 373 "synthesize.cpp"
                var $arrayidx85=(($85)&4294967295); //@line 373 "synthesize.cpp"
                HEAP32[(($arrayidx85)>>2)]=6; //@line 373 "synthesize.cpp"
                var $86=$len4; //@line 374 "synthesize.cpp"
                var $mul86=((($86)*2)&4294967295); //@line 374 "synthesize.cpp"
                var $87=$q; //@line 374 "synthesize.cpp"
                var $arrayidx87=(($87+4)&4294967295); //@line 374 "synthesize.cpp"
                HEAP32[(($arrayidx87)>>2)]=$mul86; //@line 374 "synthesize.cpp"
                var $88=$index_addr; //@line 375 "synthesize.cpp"
                var $89=$x; //@line 375 "synthesize.cpp"
                var $add88=((($89)+($88))&4294967295); //@line 375 "synthesize.cpp"
                var $90=HEAP32[((_wavefile_data)>>2)]; //@line 375 "synthesize.cpp"
                var $arrayidx89=(($90+$add88)&4294967295); //@line 375 "synthesize.cpp"
                var $91=($arrayidx89); //@line 375 "synthesize.cpp"
                var $92=$q; //@line 375 "synthesize.cpp"
                var $arrayidx90=(($92+8)&4294967295); //@line 375 "synthesize.cpp"
                HEAP32[(($arrayidx90)>>2)]=$91; //@line 375 "synthesize.cpp"
                var $93=$wav_scale; //@line 376 "synthesize.cpp"
                var $94=$amp_addr; //@line 376 "synthesize.cpp"
                var $shl91=((($94)*256)&4294967295); //@line 376 "synthesize.cpp"
                var $add92=((($shl91)+($93))&4294967295); //@line 376 "synthesize.cpp"
                var $95=$q; //@line 376 "synthesize.cpp"
                var $arrayidx93=(($95+12)&4294967295); //@line 376 "synthesize.cpp"
                HEAP32[(($arrayidx93)>>2)]=$add92; //@line 376 "synthesize.cpp"
                __Z8WcmdqIncv(); //@line 377 "synthesize.cpp"
                var $96=$len4; //@line 379 "synthesize.cpp"
                var $mul94=((($96)*2)&4294967295); //@line 379 "synthesize.cpp"
                var $97=$length; //@line 379 "synthesize.cpp"
                var $sub95=((($97)-($mul94))&4294967295); //@line 379 "synthesize.cpp"
                $length=$sub95; //@line 379 "synthesize.cpp"
                var $98=$length; //@line 365 "synthesize.cpp"
                var $99=$len4; //@line 365 "synthesize.cpp"
                var $mul77=((($99)*3)&4294967295); //@line 365 "synthesize.cpp"
                var $cmp78=((($98))|0) > ((($mul77))|0); //@line 365 "synthesize.cpp"
                if ($cmp78) { __lastLabel__ = 29; __label__ = 27;continue ; } else { __lastLabel__ = 29; __label__ = 30;break $while_body$$while_end$39; } //@line 365 "synthesize.cpp"
              }
            }
          } while(0);
  
          var $100=__lastLabel__ == 29 ? $98 : ($78); //@line 382 "synthesize.cpp"
          var $cmp96=((($100))|0) > 0; //@line 382 "synthesize.cpp"
          if ($cmp96) { __label__ = 31;; } else { __label__ = 34;; } //@line 382 "synthesize.cpp"
          if (__label__ == 31) {
  
            var $101=$wav_length; //@line 384 "synthesize.cpp"
            var $102=$length; //@line 384 "synthesize.cpp"
            var $sub98=((($101)-($102))&4294967295); //@line 384 "synthesize.cpp"
            $x=$sub98; //@line 384 "synthesize.cpp"
            var $103=$wav_scale; //@line 385 "synthesize.cpp"
            var $cmp99=((($103))|0)==0; //@line 385 "synthesize.cpp"
            if ($cmp99) { __label__ = 32;; } else { __label__ = 33;; } //@line 385 "synthesize.cpp"
            if (__label__ == 32) {
  
              var $104=$x; //@line 386 "synthesize.cpp"
              var $mul101=((($104)*2)&4294967295); //@line 386 "synthesize.cpp"
              $x=$mul101; //@line 386 "synthesize.cpp"
              ; //@line 386 "synthesize.cpp"
            }
  
            var $105=HEAP32[((_wcmdq_tail)>>2)]; //@line 387 "synthesize.cpp"
            HEAP32[((__ZL10last_wcmdq)>>2)]=$105; //@line 387 "synthesize.cpp"
            var $106=HEAP32[((_wcmdq_tail)>>2)]; //@line 388 "synthesize.cpp"
            var $arrayidx103=((_wcmdq+$106*16)&4294967295); //@line 388 "synthesize.cpp"
            var $arraydecay104=(($arrayidx103)&4294967295); //@line 388 "synthesize.cpp"
            $q=$arraydecay104; //@line 388 "synthesize.cpp"
            var $107=$q; //@line 389 "synthesize.cpp"
            var $arrayidx105=(($107)&4294967295); //@line 389 "synthesize.cpp"
            HEAP32[(($arrayidx105)>>2)]=6; //@line 389 "synthesize.cpp"
            var $108=$length; //@line 390 "synthesize.cpp"
            var $109=$q; //@line 390 "synthesize.cpp"
            var $arrayidx106=(($109+4)&4294967295); //@line 390 "synthesize.cpp"
            HEAP32[(($arrayidx106)>>2)]=$108; //@line 390 "synthesize.cpp"
            var $110=$index_addr; //@line 391 "synthesize.cpp"
            var $111=$x; //@line 391 "synthesize.cpp"
            var $add107=((($111)+($110))&4294967295); //@line 391 "synthesize.cpp"
            var $112=HEAP32[((_wavefile_data)>>2)]; //@line 391 "synthesize.cpp"
            var $arrayidx108=(($112+$add107)&4294967295); //@line 391 "synthesize.cpp"
            var $113=($arrayidx108); //@line 391 "synthesize.cpp"
            var $114=$q; //@line 391 "synthesize.cpp"
            var $arrayidx109=(($114+8)&4294967295); //@line 391 "synthesize.cpp"
            HEAP32[(($arrayidx109)>>2)]=$113; //@line 391 "synthesize.cpp"
            var $115=$wav_scale; //@line 392 "synthesize.cpp"
            var $116=$amp_addr; //@line 392 "synthesize.cpp"
            var $shl110=((($116)*256)&4294967295); //@line 392 "synthesize.cpp"
            var $add111=((($shl110)+($115))&4294967295); //@line 392 "synthesize.cpp"
            var $117=$q; //@line 392 "synthesize.cpp"
            var $arrayidx112=(($117+12)&4294967295); //@line 392 "synthesize.cpp"
            HEAP32[(($arrayidx112)>>2)]=$add111; //@line 392 "synthesize.cpp"
            __Z8WcmdqIncv(); //@line 393 "synthesize.cpp"
            ; //@line 394 "synthesize.cpp"
          }
  
          var $118=$length; //@line 396 "synthesize.cpp"
          $retval=$118; //@line 396 "synthesize.cpp"
          ; //@line 396 "synthesize.cpp"
        }
      }
    }
  
    var $119=$retval; //@line 397 "synthesize.cpp"
    ;
    return $119; //@line 397 "synthesize.cpp"
    return null;
  }
  

  function __Z18FormantTransition2P10frameref_tRijjP11PHONEME_TABi($seq, $n_frames, $data1, $data2, $other_ph, $which) {
    ;
    var __label__;
  
    var $retval;
    var $seq_addr;
    var $n_frames_addr;
    var $data1_addr;
    var $data2_addr;
    var $other_ph_addr;
    var $which_addr;
    var $ix;
    var $formant;
    var $next_rms;
    var $len;
    var $rms;
    var $f1;
    var $f2;
    var $f2_min;
    var $f2_max;
    var $f3_adj;
    var $f3_amp;
    var $flags;
    var $vcolour;
    var $fr;
    var $x;
    $seq_addr=$seq;
    $n_frames_addr=$n_frames;
    $data1_addr=$data1;
    $data2_addr=$data2;
    $other_ph_addr=$other_ph;
    $which_addr=$which;
    $fr=0; //@line 640 "synthesize.cpp"
    var $0=$n_frames_addr; //@line 642 "synthesize.cpp"
    var $1=HEAP32[(($0)>>2)]; //@line 642 "synthesize.cpp"
    var $cmp=((($1))|0) < 2; //@line 642 "synthesize.cpp"
    if ($cmp) { __label__ = 1;; } else { __label__ = 2;; } //@line 642 "synthesize.cpp"
    if (__label__ == 1) {
  
      $retval=0; //@line 643 "synthesize.cpp"
      ; //@line 643 "synthesize.cpp"
    }
    else if (__label__ == 2) {
  
      var $2=$data1_addr; //@line 645 "synthesize.cpp"
      var $and=($2) & 63; //@line 645 "synthesize.cpp"
      var $mul=((($and)*2)&4294967295); //@line 645 "synthesize.cpp"
      $len=$mul; //@line 645 "synthesize.cpp"
      var $3=$data1_addr; //@line 646 "synthesize.cpp"
      var $shr=($3) >>> 6; //@line 646 "synthesize.cpp"
      var $and1=($shr) & 63; //@line 646 "synthesize.cpp"
      $rms=$and1; //@line 646 "synthesize.cpp"
      var $4=$data1_addr; //@line 647 "synthesize.cpp"
      var $shr2=($4) >>> 12; //@line 647 "synthesize.cpp"
      $flags=$shr2; //@line 647 "synthesize.cpp"
      var $5=$data2_addr; //@line 649 "synthesize.cpp"
      var $and3=($5) & 63; //@line 649 "synthesize.cpp"
      var $mul4=((($and3)*50)&4294967295); //@line 649 "synthesize.cpp"
      $f2=$mul4; //@line 649 "synthesize.cpp"
      var $6=$data2_addr; //@line 650 "synthesize.cpp"
      var $shr5=($6) >>> 6; //@line 650 "synthesize.cpp"
      var $and6=($shr5) & 31; //@line 650 "synthesize.cpp"
      var $sub=((($and6)-15)&4294967295); //@line 650 "synthesize.cpp"
      var $mul7=((($sub)*50)&4294967295); //@line 650 "synthesize.cpp"
      $f2_min=$mul7; //@line 650 "synthesize.cpp"
      var $7=$data2_addr; //@line 651 "synthesize.cpp"
      var $shr8=($7) >>> 11; //@line 651 "synthesize.cpp"
      var $and9=($shr8) & 31; //@line 651 "synthesize.cpp"
      var $sub10=((($and9)-15)&4294967295); //@line 651 "synthesize.cpp"
      var $mul11=((($sub10)*50)&4294967295); //@line 651 "synthesize.cpp"
      $f2_max=$mul11; //@line 651 "synthesize.cpp"
      var $8=$data2_addr; //@line 652 "synthesize.cpp"
      var $shr12=($8) >>> 16; //@line 652 "synthesize.cpp"
      var $and13=($shr12) & 31; //@line 652 "synthesize.cpp"
      var $sub14=((($and13)-15)&4294967295); //@line 652 "synthesize.cpp"
      var $mul15=((($sub14)*50)&4294967295); //@line 652 "synthesize.cpp"
      $f3_adj=$mul15; //@line 652 "synthesize.cpp"
      var $9=$data2_addr; //@line 653 "synthesize.cpp"
      var $shr16=($9) >>> 21; //@line 653 "synthesize.cpp"
      var $and17=($shr16) & 31; //@line 653 "synthesize.cpp"
      var $mul18=((($and17)*8)&4294967295); //@line 653 "synthesize.cpp"
      $f3_amp=$mul18; //@line 653 "synthesize.cpp"
      var $10=$data2_addr; //@line 654 "synthesize.cpp"
      var $shr19=($10) >>> 26; //@line 654 "synthesize.cpp"
      var $and20=($shr19) & 7; //@line 654 "synthesize.cpp"
      $f1=$and20; //@line 654 "synthesize.cpp"
      var $11=$data2_addr; //@line 655 "synthesize.cpp"
      var $shr21=($11) >>> 29; //@line 655 "synthesize.cpp"
      $vcolour=$shr21; //@line 655 "synthesize.cpp"
      var $12=$other_ph_addr; //@line 660 "synthesize.cpp"
      var $cmp22=((($12))|0)!=0; //@line 660 "synthesize.cpp"
      if ($cmp22) { __label__ = 3;; } else { __label__ = 5;; } //@line 660 "synthesize.cpp"
      $land_lhs_true$$if_end25$5: do { 
        if (__label__ == 3) {
  
          var $13=$other_ph_addr; //@line 660 "synthesize.cpp"
          var $mnemonic=(($13)&4294967295); //@line 660 "synthesize.cpp"
          var $14=HEAP32[(($mnemonic)>>2)]; //@line 660 "synthesize.cpp"
          var $cmp23=((($14))|0)==63; //@line 660 "synthesize.cpp"
          if (!($cmp23)) { __label__ = 5;break $land_lhs_true$$if_end25$5; } //@line 660 "synthesize.cpp"
  
          var $15=$flags; //@line 661 "synthesize.cpp"
          var $or=($15) | 8; //@line 661 "synthesize.cpp"
          $flags=$or; //@line 661 "synthesize.cpp"
          ; //@line 661 "synthesize.cpp"
        }
      } while(0);
  
      var $16=$which_addr; //@line 663 "synthesize.cpp"
      var $cmp26=((($16))|0)==1; //@line 663 "synthesize.cpp"
      if ($cmp26) { __label__ = 6;; } else { __label__ = 20;; } //@line 663 "synthesize.cpp"
      $if_then27$$if_else85$9: do { 
        if (__label__ == 6) {
  
          var $17=$seq_addr; //@line 666 "synthesize.cpp"
          var $arrayidx=(($17)&4294967295); //@line 666 "synthesize.cpp"
          var $frame=(($arrayidx+4)&4294967295); //@line 666 "synthesize.cpp"
          var $18=HEAP32[(($frame)>>2)]; //@line 666 "synthesize.cpp"
          var $call=__ZL9CopyFrameP7frame_ti($18, 0); //@line 666 "synthesize.cpp"
          $fr=$call; //@line 666 "synthesize.cpp"
          var $19=$fr; //@line 667 "synthesize.cpp"
          var $20=$seq_addr; //@line 667 "synthesize.cpp"
          var $arrayidx28=(($20)&4294967295); //@line 667 "synthesize.cpp"
          var $frame29=(($arrayidx28+4)&4294967295); //@line 667 "synthesize.cpp"
          HEAP32[(($frame29)>>2)]=$19; //@line 667 "synthesize.cpp"
          var $21=$seq_addr; //@line 668 "synthesize.cpp"
          var $arrayidx30=(($21)&4294967295); //@line 668 "synthesize.cpp"
          var $length=(($arrayidx30)&4294967295); //@line 668 "synthesize.cpp"
          HEAP16[(($length)>>1)]=50; //@line 668 "synthesize.cpp"
          var $22=$len; //@line 669 "synthesize.cpp"
          var $cmp31=((($22))|0) > 0; //@line 669 "synthesize.cpp"
          if ($cmp31) { __label__ = 7;; } else { __label__ = 8;; } //@line 669 "synthesize.cpp"
          if (__label__ == 7) {
  
            var $23=$len; //@line 670 "synthesize.cpp"
            var $conv=((($23)) & 65535); //@line 670 "synthesize.cpp"
            var $24=$seq_addr; //@line 670 "synthesize.cpp"
            var $arrayidx33=(($24)&4294967295); //@line 670 "synthesize.cpp"
            var $length34=(($arrayidx33)&4294967295); //@line 670 "synthesize.cpp"
            HEAP16[(($length34)>>1)]=$conv; //@line 670 "synthesize.cpp"
            ; //@line 670 "synthesize.cpp"
          }
  
          var $25=$seq_addr; //@line 671 "synthesize.cpp"
          var $arrayidx36=(($25)&4294967295); //@line 671 "synthesize.cpp"
          var $frflags=(($arrayidx36+2)&4294967295); //@line 671 "synthesize.cpp"
          var $26=HEAP16[(($frflags)>>1)]; //@line 671 "synthesize.cpp"
          var $conv37=(tempInt=(($26)),(tempInt>=32768?tempInt-65536:tempInt)); //@line 671 "synthesize.cpp"
          var $or38=($conv37) | 16384; //@line 671 "synthesize.cpp"
          var $conv39=((($or38)) & 65535); //@line 671 "synthesize.cpp"
          HEAP16[(($frflags)>>1)]=$conv39; //@line 671 "synthesize.cpp"
          var $27=$fr; //@line 672 "synthesize.cpp"
          var $frflags40=(($27)&4294967295); //@line 672 "synthesize.cpp"
          var $28=HEAP16[(($frflags40)>>1)]; //@line 672 "synthesize.cpp"
          var $conv41=(tempInt=(($28)),(tempInt>=32768?tempInt-65536:tempInt)); //@line 672 "synthesize.cpp"
          var $or42=($conv41) | 16384; //@line 672 "synthesize.cpp"
          var $conv43=((($or42)) & 65535); //@line 672 "synthesize.cpp"
          HEAP16[(($frflags40)>>1)]=$conv43; //@line 672 "synthesize.cpp"
          var $29=$seq_addr; //@line 674 "synthesize.cpp"
          var $arrayidx44=(($29+8)&4294967295); //@line 674 "synthesize.cpp"
          var $frame45=(($arrayidx44+4)&4294967295); //@line 674 "synthesize.cpp"
          var $30=HEAP32[(($frame45)>>2)]; //@line 674 "synthesize.cpp"
          var $rms46=(($30+17)&4294967295); //@line 674 "synthesize.cpp"
          var $31=HEAPU8[($rms46)]; //@line 674 "synthesize.cpp"
          var $conv47=((($31))&255); //@line 674 "synthesize.cpp"
          $next_rms=$conv47; //@line 674 "synthesize.cpp"
          var $32=HEAP32[((_voice)>>2)]; //@line 676 "synthesize.cpp"
          var $klattv=(($32+112)&4294967295); //@line 676 "synthesize.cpp"
          var $arrayidx48=(($klattv)&4294967295); //@line 676 "synthesize.cpp"
          var $33=HEAP32[(($arrayidx48)>>2)]; //@line 676 "synthesize.cpp"
          var $tobool=((($33))|0)!=0; //@line 676 "synthesize.cpp"
          if ($tobool) { __label__ = 9;; } else { __label__ = 10;; } //@line 676 "synthesize.cpp"
          if (__label__ == 9) {
  
            var $34=$seq_addr; //@line 679 "synthesize.cpp"
            var $arrayidx50=(($34+8)&4294967295); //@line 679 "synthesize.cpp"
            var $frame51=(($arrayidx50+4)&4294967295); //@line 679 "synthesize.cpp"
            var $35=HEAP32[(($frame51)>>2)]; //@line 679 "synthesize.cpp"
            var $klattp=(($35+39)&4294967295); //@line 679 "synthesize.cpp"
            var $arrayidx52=(($klattp)&4294967295); //@line 679 "synthesize.cpp"
            var $36=HEAPU8[($arrayidx52)]; //@line 679 "synthesize.cpp"
            var $conv53=((($36))&255); //@line 679 "synthesize.cpp"
            var $sub54=((($conv53)-4)&4294967295); //@line 679 "synthesize.cpp"
            var $conv55=((($sub54)) & 255); //@line 679 "synthesize.cpp"
            var $37=$fr; //@line 679 "synthesize.cpp"
            var $klattp56=(($37+39)&4294967295); //@line 679 "synthesize.cpp"
            var $arrayidx57=(($klattp56)&4294967295); //@line 679 "synthesize.cpp"
            HEAP8[($arrayidx57)]=$conv55; //@line 679 "synthesize.cpp"
            ; //@line 680 "synthesize.cpp"
          }
  
          var $38=$f2; //@line 681 "synthesize.cpp"
          var $cmp59=((($38))|0)!=0; //@line 681 "synthesize.cpp"
          if ($cmp59) { __label__ = 11;; } else { __label__ = 15;; } //@line 681 "synthesize.cpp"
          $if_then60$$if_else$17: do { 
            if (__label__ == 11) {
  
              var $39=$rms; //@line 683 "synthesize.cpp"
              var $and61=($39) & 32; //@line 683 "synthesize.cpp"
              var $tobool62=((($and61))|0)!=0; //@line 683 "synthesize.cpp"
              if ($tobool62) { __label__ = 12;; } else { __label__ = 13;; } //@line 683 "synthesize.cpp"
              if (__label__ == 12) {
  
                var $40=$fr; //@line 685 "synthesize.cpp"
                var $41=$next_rms; //@line 685 "synthesize.cpp"
                var $42=$rms; //@line 685 "synthesize.cpp"
                var $and64=($42) & 31; //@line 685 "synthesize.cpp"
                var $mul65=((($and64)*($41))&4294967295); //@line 685 "synthesize.cpp"
                var $div=((((($mul65))|0)/30)|0); //@line 685 "synthesize.cpp"
                __ZL13set_frame_rmsP7frame_ti($40, $div); //@line 685 "synthesize.cpp"
                ; //@line 686 "synthesize.cpp"
              }
  
              var $43=$fr; //@line 687 "synthesize.cpp"
              var $44=$f2; //@line 687 "synthesize.cpp"
              var $45=$f2_min; //@line 687 "synthesize.cpp"
              var $46=$f2_max; //@line 687 "synthesize.cpp"
              var $47=$f1; //@line 687 "synthesize.cpp"
              var $48=$f3_adj; //@line 687 "synthesize.cpp"
              var $49=$f3_amp; //@line 687 "synthesize.cpp"
              var $50=$flags; //@line 687 "synthesize.cpp"
              __ZL14AdjustFormantsP7frame_tiiiiiii($43, $44, $45, $46, $47, $48, $49, $50); //@line 687 "synthesize.cpp"
              var $51=$rms; //@line 689 "synthesize.cpp"
              var $and67=($51) & 32; //@line 689 "synthesize.cpp"
              var $cmp68=((($and67))|0)==0; //@line 689 "synthesize.cpp"
              if (!($cmp68)) { __label__ = 18;break $if_then60$$if_else$17; } //@line 689 "synthesize.cpp"
  
              var $52=$fr; //@line 691 "synthesize.cpp"
              var $53=$rms; //@line 691 "synthesize.cpp"
              var $mul70=((($53)*2)&4294967295); //@line 691 "synthesize.cpp"
              __ZL13set_frame_rmsP7frame_ti($52, $mul70); //@line 691 "synthesize.cpp"
              ; //@line 692 "synthesize.cpp"
            }
            else if (__label__ == 15) {
  
              var $54=$flags; //@line 696 "synthesize.cpp"
              var $and72=($54) & 8; //@line 696 "synthesize.cpp"
              var $tobool73=((($and72))|0)!=0; //@line 696 "synthesize.cpp"
              var $55=$fr; //@line 697 "synthesize.cpp"
              if ($tobool73) { __label__ = 16;; } else { __label__ = 17;; } //@line 696 "synthesize.cpp"
              if (__label__ == 16) {
  
                var $56=$next_rms; //@line 697 "synthesize.cpp"
                var $mul75=((($56)*24)&4294967295); //@line 697 "synthesize.cpp"
                var $div76=((((($mul75))|0)/32)|0); //@line 697 "synthesize.cpp"
                __ZL13set_frame_rmsP7frame_ti($55, $div76); //@line 697 "synthesize.cpp"
                ; //@line 697 "synthesize.cpp"
              }
              else if (__label__ == 17) {
  
                __ZL13set_frame_rmsP7frame_ti($55, 28); //@line 699 "synthesize.cpp"
                ;
              }
            }
          } while(0);
  
          var $57=$flags; //@line 702 "synthesize.cpp"
          var $and80=($57) & 8; //@line 702 "synthesize.cpp"
          var $tobool81=((($and80))|0)!=0; //@line 702 "synthesize.cpp"
          if (!($tobool81)) { __label__ = 33;break $if_then27$$if_else85$9; } //@line 702 "synthesize.cpp"
  
          var $58=$fr; //@line 705 "synthesize.cpp"
          var $call83=__ZL14VowelClosenessP7frame_t($58); //@line 705 "synthesize.cpp"
          var $shl=((($call83)*256)&4294967295); //@line 705 "synthesize.cpp"
          var $add=((($shl)+2048)&4294967295); //@line 705 "synthesize.cpp"
          HEAP32[((__ZL10modn_flags)>>2)]=$add; //@line 705 "synthesize.cpp"
          ; //@line 706 "synthesize.cpp"
        }
        else if (__label__ == 20) {
  
          var $59=$rms; //@line 711 "synthesize.cpp"
          var $mul86=((($59)*2)&4294967295); //@line 711 "synthesize.cpp"
          $rms=$mul86; //@line 711 "synthesize.cpp"
          var $60=$f2; //@line 712 "synthesize.cpp"
          var $cmp87=((($60))|0)!=0; //@line 712 "synthesize.cpp"
          if ($cmp87) { __label__ = 22;; } else { __label__ = 21;; } //@line 712 "synthesize.cpp"
          if (__label__ == 21) {
  
            var $61=$flags; //@line 712 "synthesize.cpp"
            var $cmp88=((($61))|0)!=0; //@line 712 "synthesize.cpp"
            if (!($cmp88)) { __label__ = 33;break $if_then27$$if_else85$9; } //@line 712 "synthesize.cpp"
          }
  
          var $62=$flags; //@line 715 "synthesize.cpp"
          var $and90=($62) & 8; //@line 715 "synthesize.cpp"
          var $tobool91=((($and90))|0)!=0; //@line 715 "synthesize.cpp"
          if ($tobool91) { __label__ = 23;; } else { __label__ = 24;; } //@line 715 "synthesize.cpp"
          $if_then92$$if_else103$33: do { 
            if (__label__ == 23) {
  
              var $63=$n_frames_addr; //@line 717 "synthesize.cpp"
              var $64=HEAP32[(($63)>>2)]; //@line 717 "synthesize.cpp"
              var $sub93=((($64)-1)&4294967295); //@line 717 "synthesize.cpp"
              var $65=$seq_addr; //@line 717 "synthesize.cpp"
              var $arrayidx94=(($65+8*$sub93)&4294967295); //@line 717 "synthesize.cpp"
              var $frame95=(($arrayidx94+4)&4294967295); //@line 717 "synthesize.cpp"
              var $66=HEAP32[(($frame95)>>2)]; //@line 717 "synthesize.cpp"
              var $call96=__ZL9CopyFrameP7frame_ti($66, 0); //@line 717 "synthesize.cpp"
              $fr=$call96; //@line 717 "synthesize.cpp"
              var $67=$fr; //@line 718 "synthesize.cpp"
              var $68=$n_frames_addr; //@line 718 "synthesize.cpp"
              var $69=HEAP32[(($68)>>2)]; //@line 718 "synthesize.cpp"
              var $sub97=((($69)-1)&4294967295); //@line 718 "synthesize.cpp"
              var $70=$seq_addr; //@line 718 "synthesize.cpp"
              var $arrayidx98=(($70+8*$sub97)&4294967295); //@line 718 "synthesize.cpp"
              var $frame99=(($arrayidx98+4)&4294967295); //@line 718 "synthesize.cpp"
              HEAP32[(($frame99)>>2)]=$67; //@line 718 "synthesize.cpp"
              $rms=35; //@line 719 "synthesize.cpp"
              var $71=$fr; //@line 722 "synthesize.cpp"
              var $call100=__ZL14VowelClosenessP7frame_t($71); //@line 722 "synthesize.cpp"
              var $shl101=((($call100)*256)&4294967295); //@line 722 "synthesize.cpp"
              var $add102=((($shl101)+1024)&4294967295); //@line 722 "synthesize.cpp"
              HEAP32[((__ZL10modn_flags)>>2)]=$add102; //@line 722 "synthesize.cpp"
              ; //@line 723 "synthesize.cpp"
            }
            else if (__label__ == 24) {
  
              var $72=$seq_addr; //@line 726 "synthesize.cpp"
              var $73=$n_frames_addr; //@line 726 "synthesize.cpp"
              var $74=HEAP32[(($73)>>2)]; //@line 726 "synthesize.cpp"
              var $inc=((($74)+1)&4294967295); //@line 726 "synthesize.cpp"
              HEAP32[(($73)>>2)]=$inc; //@line 726 "synthesize.cpp"
              var $75=$len; //@line 726 "synthesize.cpp"
              var $call104=__ZL18DuplicateLastFrameP10frameref_tii($72, $74, $75); //@line 726 "synthesize.cpp"
              $fr=$call104; //@line 726 "synthesize.cpp"
              var $76=$len; //@line 727 "synthesize.cpp"
              var $cmp105=((($76))|0) > 36; //@line 727 "synthesize.cpp"
              if ($cmp105) { __label__ = 25;; } else { __label__ = 26;; } //@line 727 "synthesize.cpp"
              if (__label__ == 25) {
  
                var $77=$len; //@line 728 "synthesize.cpp"
                var $78=HEAP32[((_seq_len_adjust)>>2)]; //@line 728 "synthesize.cpp"
                var $sub107=((($77)-36)&4294967295); //@line 728 "synthesize.cpp"
                var $add108=((($sub107)+($78))&4294967295); //@line 728 "synthesize.cpp"
                HEAP32[((_seq_len_adjust)>>2)]=$add108; //@line 728 "synthesize.cpp"
                ; //@line 728 "synthesize.cpp"
              }
  
              var $79=$f2; //@line 730 "synthesize.cpp"
              var $cmp110=((($79))|0)!=0; //@line 730 "synthesize.cpp"
              if (!($cmp110)) { __label__ = 28;break $if_then92$$if_else103$33; } //@line 730 "synthesize.cpp"
  
              var $80=$fr; //@line 732 "synthesize.cpp"
              var $81=$f2; //@line 732 "synthesize.cpp"
              var $82=$f2_min; //@line 732 "synthesize.cpp"
              var $83=$f2_max; //@line 732 "synthesize.cpp"
              var $84=$f1; //@line 732 "synthesize.cpp"
              var $85=$f3_adj; //@line 732 "synthesize.cpp"
              var $86=$f3_amp; //@line 732 "synthesize.cpp"
              var $87=$flags; //@line 732 "synthesize.cpp"
              __ZL14AdjustFormantsP7frame_tiiiiiii($80, $81, $82, $83, $84, $85, $86, $87); //@line 732 "synthesize.cpp"
              ; //@line 733 "synthesize.cpp"
            }
          } while(0);
  
          var $88=$fr; //@line 736 "synthesize.cpp"
          var $89=$rms; //@line 736 "synthesize.cpp"
          __ZL13set_frame_rmsP7frame_ti($88, $89); //@line 736 "synthesize.cpp"
          var $90=$vcolour; //@line 738 "synthesize.cpp"
          var $cmp114=((($90))|0) > 0; //@line 738 "synthesize.cpp"
          var $91=$vcolour; //@line 738 "synthesize.cpp"
          var $cmp116=((($91))|0) <= 2; //@line 738 "synthesize.cpp"
          var $or_cond=($cmp114) & ($cmp116); //@line 738 "synthesize.cpp"
          if (!($or_cond)) { __label__ = 33;break $if_then27$$if_else85$9; } //@line 738 "synthesize.cpp"
  
          $ix=0; //@line 740 "synthesize.cpp"
          var $92=$ix; //@line 740 "synthesize.cpp"
          var $93=$n_frames_addr; //@line 740 "synthesize.cpp"
          var $94=HEAP32[(($93)>>2)]; //@line 740 "synthesize.cpp"
          var $cmp1181=((($92))|0) < ((($94))|0); //@line 740 "synthesize.cpp"
          if (!($cmp1181)) { __label__ = 33;break $if_then27$$if_else85$9; } //@line 740 "synthesize.cpp"
          $for_body$42: while(1) { 
  
            var $95=$ix; //@line 742 "synthesize.cpp"
            var $96=$seq_addr; //@line 742 "synthesize.cpp"
            var $arrayidx119=(($96+8*$95)&4294967295); //@line 742 "synthesize.cpp"
            var $frame120=(($arrayidx119+4)&4294967295); //@line 742 "synthesize.cpp"
            var $97=HEAP32[(($frame120)>>2)]; //@line 742 "synthesize.cpp"
            var $call121=__ZL9CopyFrameP7frame_ti($97, 0); //@line 742 "synthesize.cpp"
            $fr=$call121; //@line 742 "synthesize.cpp"
            var $98=$fr; //@line 743 "synthesize.cpp"
            var $99=$ix; //@line 743 "synthesize.cpp"
            var $100=$seq_addr; //@line 743 "synthesize.cpp"
            var $arrayidx122=(($100+8*$99)&4294967295); //@line 743 "synthesize.cpp"
            var $frame123=(($arrayidx122+4)&4294967295); //@line 743 "synthesize.cpp"
            HEAP32[(($frame123)>>2)]=$98; //@line 743 "synthesize.cpp"
            $formant=1; //@line 745 "synthesize.cpp"
            ; //@line 745 "synthesize.cpp"
            while(1) { 
  
              var $101=$formant; //@line 748 "synthesize.cpp"
              var $102=$fr; //@line 748 "synthesize.cpp"
              var $ffreq=(($102+2)&4294967295); //@line 748 "synthesize.cpp"
              var $arrayidx127=(($ffreq+$101*2)&4294967295); //@line 748 "synthesize.cpp"
              var $103=HEAP16[(($arrayidx127)>>1)]; //@line 748 "synthesize.cpp"
              var $conv128=(tempInt=(($103)),(tempInt>=32768?tempInt-65536:tempInt)); //@line 748 "synthesize.cpp"
              var $104=$formant; //@line 748 "synthesize.cpp"
              var $sub129=((($104)-1)&4294967295); //@line 748 "synthesize.cpp"
              var $105=$vcolour; //@line 748 "synthesize.cpp"
              var $sub130=((($105)-1)&4294967295); //@line 748 "synthesize.cpp"
              var $arrayidx131=((__ZZ18FormantTransition2P10frameref_tRijjP11PHONEME_TABiE10vcolouring+$sub130*10)&4294967295); //@line 748 "synthesize.cpp"
              var $arrayidx132=(($arrayidx131+$sub129*2)&4294967295); //@line 748 "synthesize.cpp"
              var $106=HEAP16[(($arrayidx132)>>1)]; //@line 748 "synthesize.cpp"
              var $conv133=(tempInt=(($106)),(tempInt>=32768?tempInt-65536:tempInt)); //@line 748 "synthesize.cpp"
              var $mul134=((($conv133)*($conv128))&4294967295); //@line 748 "synthesize.cpp"
              $x=$mul134; //@line 748 "synthesize.cpp"
              var $107=$x; //@line 749 "synthesize.cpp"
              var $div135=((((($107))|0)/256)|0); //@line 749 "synthesize.cpp"
              var $conv136=((($div135)) & 65535); //@line 749 "synthesize.cpp"
              var $108=$formant; //@line 749 "synthesize.cpp"
              var $109=$fr; //@line 749 "synthesize.cpp"
              var $ffreq137=(($109+2)&4294967295); //@line 749 "synthesize.cpp"
              var $arrayidx138=(($ffreq137+$108*2)&4294967295); //@line 749 "synthesize.cpp"
              HEAP16[(($arrayidx138)>>1)]=$conv136; //@line 749 "synthesize.cpp"
              var $110=$formant; //@line 745 "synthesize.cpp"
              var $inc139=((($110)+1)&4294967295); //@line 745 "synthesize.cpp"
              $formant=$inc139; //@line 745 "synthesize.cpp"
              var $cmp125=((($inc139))|0) <= 5; //@line 745 "synthesize.cpp"
              if ($cmp125) { __label__ = 31;continue ; } else { __label__ = 32;break ; } //@line 745 "synthesize.cpp"
            }
  
            var $111=$ix; //@line 740 "synthesize.cpp"
            var $inc141=((($111)+1)&4294967295); //@line 740 "synthesize.cpp"
            $ix=$inc141; //@line 740 "synthesize.cpp"
            var $112=$ix; //@line 740 "synthesize.cpp"
            var $113=$n_frames_addr; //@line 740 "synthesize.cpp"
            var $114=HEAP32[(($113)>>2)]; //@line 740 "synthesize.cpp"
            var $cmp118=((($112))|0) < ((($114))|0); //@line 740 "synthesize.cpp"
            if ($cmp118) { __label__ = 30;continue $for_body$42; } else { __label__ = 33;break $if_then27$$if_else85$9; } //@line 740 "synthesize.cpp"
          }
        }
      } while(0);
  
      var $115=$fr; //@line 756 "synthesize.cpp"
      var $cmp146=((($115))|0)!=0; //@line 756 "synthesize.cpp"
      if ($cmp146) { __label__ = 34;; } else { __label__ = 38;; } //@line 756 "synthesize.cpp"
      $if_then147$$if_end164$48: do { 
        if (__label__ == 34) {
  
          var $116=$flags; //@line 758 "synthesize.cpp"
          var $and148=($116) & 4; //@line 758 "synthesize.cpp"
          var $tobool149=((($and148))|0)!=0; //@line 758 "synthesize.cpp"
          if ($tobool149) { __label__ = 35;; } else { __label__ = 36;; } //@line 758 "synthesize.cpp"
          if (__label__ == 35) {
  
            var $117=$fr; //@line 759 "synthesize.cpp"
            var $frflags151=(($117)&4294967295); //@line 759 "synthesize.cpp"
            var $118=HEAP16[(($frflags151)>>1)]; //@line 759 "synthesize.cpp"
            var $conv152=(tempInt=(($118)),(tempInt>=32768?tempInt-65536:tempInt)); //@line 759 "synthesize.cpp"
            var $or153=($conv152) | 32; //@line 759 "synthesize.cpp"
            var $conv154=((($or153)) & 65535); //@line 759 "synthesize.cpp"
            HEAP16[(($frflags151)>>1)]=$conv154; //@line 759 "synthesize.cpp"
            ; //@line 759 "synthesize.cpp"
          }
  
          var $119=$flags; //@line 760 "synthesize.cpp"
          var $and156=($119) & 2; //@line 760 "synthesize.cpp"
          var $tobool157=((($and156))|0)!=0; //@line 760 "synthesize.cpp"
          if (!($tobool157)) { __label__ = 38;break $if_then147$$if_end164$48; } //@line 760 "synthesize.cpp"
  
          var $120=$fr; //@line 761 "synthesize.cpp"
          var $frflags159=(($120)&4294967295); //@line 761 "synthesize.cpp"
          var $121=HEAP16[(($frflags159)>>1)]; //@line 761 "synthesize.cpp"
          var $conv160=(tempInt=(($121)),(tempInt>=32768?tempInt-65536:tempInt)); //@line 761 "synthesize.cpp"
          var $or161=($conv160) | 16; //@line 761 "synthesize.cpp"
          var $conv162=((($or161)) & 65535); //@line 761 "synthesize.cpp"
          HEAP16[(($frflags159)>>1)]=$conv162; //@line 761 "synthesize.cpp"
          ; //@line 761 "synthesize.cpp"
        }
      } while(0);
  
      var $122=$flags; //@line 764 "synthesize.cpp"
      var $and165=($122) & 64; //@line 764 "synthesize.cpp"
      var $tobool166=((($and165))|0)!=0; //@line 764 "synthesize.cpp"
      if ($tobool166) { __label__ = 39;; } else { __label__ = 40;; } //@line 764 "synthesize.cpp"
      if (__label__ == 39) {
  
        __ZL7DoPauseii(12, 0); //@line 765 "synthesize.cpp"
        ; //@line 765 "synthesize.cpp"
      }
  
      var $123=$flags; //@line 767 "synthesize.cpp"
      var $and169=($123) & 16; //@line 767 "synthesize.cpp"
      var $tobool170=((($and169))|0)!=0; //@line 767 "synthesize.cpp"
      if ($tobool170) { __label__ = 41;; } else { __label__ = 42;; } //@line 767 "synthesize.cpp"
      if (__label__ == 41) {
  
        var $124=$len; //@line 768 "synthesize.cpp"
        $retval=$124; //@line 768 "synthesize.cpp"
        ; //@line 768 "synthesize.cpp"
      }
      else if (__label__ == 42) {
  
        $retval=0; //@line 769 "synthesize.cpp"
        ; //@line 769 "synthesize.cpp"
      }
    }
  
    var $125=$retval; //@line 770 "synthesize.cpp"
    ;
    return $125; //@line 770 "synthesize.cpp"
    return null;
  }
  

  function __ZL9CopyFrameP7frame_ti($frame1, $copy) {
    ;
    var __label__;
  
    var $retval;
    var $frame1_addr;
    var $copy_addr;
    var $frame2;
    $frame1_addr=$frame1;
    $copy_addr=$copy;
    var $0=$copy_addr; //@line 524 "synthesize.cpp"
    var $cmp=((($0))|0)==0; //@line 524 "synthesize.cpp"
    if ($cmp) { __label__ = 1;; } else { __label__ = 3;; } //@line 524 "synthesize.cpp"
    $land_lhs_true$$if_end$2: do { 
      if (__label__ == 1) {
  
        var $1=$frame1_addr; //@line 524 "synthesize.cpp"
        var $frflags=(($1)&4294967295); //@line 524 "synthesize.cpp"
        var $2=HEAP16[(($frflags)>>1)]; //@line 524 "synthesize.cpp"
        var $conv=(tempInt=(($2)),(tempInt>=32768?tempInt-65536:tempInt)); //@line 524 "synthesize.cpp"
        var $and=($conv) & 32768; //@line 524 "synthesize.cpp"
        var $tobool=((($and))|0)!=0; //@line 524 "synthesize.cpp"
        if (!($tobool)) { __label__ = 3;break $land_lhs_true$$if_end$2; } //@line 524 "synthesize.cpp"
  
        var $3=$frame1_addr; //@line 527 "synthesize.cpp"
        $retval=$3; //@line 527 "synthesize.cpp"
        __label__ = 6;break $land_lhs_true$$if_end$2; //@line 527 "synthesize.cpp"
      }
    } while(0);
    if (__label__ == 3) {
  
      var $call=__ZL10AllocFramev(); //@line 530 "synthesize.cpp"
      $frame2=$call; //@line 530 "synthesize.cpp"
      var $cmp1=((($call))|0)!=0; //@line 531 "synthesize.cpp"
      if ($cmp1) { __label__ = 4;; } else { __label__ = 5;; } //@line 531 "synthesize.cpp"
      if (__label__ == 4) {
  
        var $4=$frame2; //@line 533 "synthesize.cpp"
        var $5=$4; //@line 533 "synthesize.cpp"
        var $6=$frame1_addr; //@line 533 "synthesize.cpp"
        var $7=$6; //@line 533 "synthesize.cpp"
        var $src$, $dest$, $stop$, $stop4$, $fast$;
        $src$ = $7;
        $dest$ = $5;
        $stop$ = $src$ + 64;
        if (($dest$%4) == ($src$%4) && 64 > 8) {
          while ($src$%4 !== 0 && $src$ < $stop$) {
            ; HEAP8[$dest$++] = HEAP8[$src$++];
          }
          $src$ >>= 2;
          $dest$ >>= 2;
          $stop4$ = $stop$ >> 2;
          while ($src$ < $stop4$) {
        ;;;    HEAP32[$dest$++] = HEAP32[$src$++];
          }
          $src$ <<= 2;
          $dest$ <<= 2;
        }while ($src$ < $stop$) {
          ; HEAP8[$dest$++] = HEAP8[$src$++];
        }; //@line 533 "synthesize.cpp"
        var $8=$frame2; //@line 534 "synthesize.cpp"
        var $length=(($8+16)&4294967295); //@line 534 "synthesize.cpp"
        HEAP8[($length)]=0; //@line 534 "synthesize.cpp"
        var $9=$frame2; //@line 535 "synthesize.cpp"
        var $frflags3=(($9)&4294967295); //@line 535 "synthesize.cpp"
        var $10=HEAP16[(($frflags3)>>1)]; //@line 535 "synthesize.cpp"
        var $conv4=(tempInt=(($10)),(tempInt>=32768?tempInt-65536:tempInt)); //@line 535 "synthesize.cpp"
        var $or=($conv4) | 32768; //@line 535 "synthesize.cpp"
        var $conv5=((($or)) & 65535); //@line 535 "synthesize.cpp"
        HEAP16[(($frflags3)>>1)]=$conv5; //@line 535 "synthesize.cpp"
        ; //@line 536 "synthesize.cpp"
      }
  
      var $11=$frame2; //@line 537 "synthesize.cpp"
      $retval=$11; //@line 537 "synthesize.cpp"
      ; //@line 537 "synthesize.cpp"
    }
  
    var $12=$retval; //@line 538 "synthesize.cpp"
    ;
    return $12; //@line 538 "synthesize.cpp"
    return null;
  }
  

  function __ZL13set_frame_rmsP7frame_ti($fr, $new_rms) {
    ;
    var __label__;
  
    var $fr_addr;
    var $new_rms_addr;
    var $x;
    var $h;
    var $ix;
    $fr_addr=$fr;
    $new_rms_addr=$new_rms;
    var $0=HEAP32[((_voice)>>2)]; //@line 478 "synthesize.cpp"
    var $klattv=(($0+112)&4294967295); //@line 478 "synthesize.cpp"
    var $arrayidx=(($klattv)&4294967295); //@line 478 "synthesize.cpp"
    var $1=HEAP32[(($arrayidx)>>2)]; //@line 478 "synthesize.cpp"
    var $tobool=((($1))|0)!=0; //@line 478 "synthesize.cpp"
    if ($tobool) { __label__ = 1;; } else { __label__ = 3;; } //@line 478 "synthesize.cpp"
    $if_then$$if_end3$2: do { 
      if (__label__ == 1) {
  
        var $2=$new_rms_addr; //@line 480 "synthesize.cpp"
        var $cmp=((($2))|0)==-1; //@line 480 "synthesize.cpp"
        if (!($cmp)) { __label__ = 8;break $if_then$$if_end3$2; } //@line 480 "synthesize.cpp"
  
        var $3=$fr_addr; //@line 482 "synthesize.cpp"
        var $klattp=(($3+39)&4294967295); //@line 482 "synthesize.cpp"
        var $arrayidx2=(($klattp)&4294967295); //@line 482 "synthesize.cpp"
        HEAP8[($arrayidx2)]=50; //@line 482 "synthesize.cpp"
        ; //@line 483 "synthesize.cpp"
      }
      else if (__label__ == 3) {
  
        var $4=$fr_addr; //@line 487 "synthesize.cpp"
        var $rms=(($4+17)&4294967295); //@line 487 "synthesize.cpp"
        var $5=HEAPU8[($rms)]; //@line 487 "synthesize.cpp"
        var $conv=((($5))&255); //@line 487 "synthesize.cpp"
        var $cmp4=((($conv))|0)==0; //@line 487 "synthesize.cpp"
        if ($cmp4) { __label__ = 8;break $if_then$$if_end3$2; } //@line 487 "synthesize.cpp"
  
        var $6=$new_rms_addr; //@line 488 "synthesize.cpp"
        var $mul=((($6)*64)&4294967295); //@line 488 "synthesize.cpp"
        var $7=$fr_addr; //@line 488 "synthesize.cpp"
        var $rms7=(($7+17)&4294967295); //@line 488 "synthesize.cpp"
        var $8=HEAPU8[($rms7)]; //@line 488 "synthesize.cpp"
        var $conv8=((($8))&255); //@line 488 "synthesize.cpp"
        var $div=((((($mul))|0)/((($conv8))|0))|0); //@line 488 "synthesize.cpp"
        $x=$div; //@line 488 "synthesize.cpp"
        var $9=$x; //@line 489 "synthesize.cpp"
        var $cmp9=((($9))|0) >= 200; //@line 489 "synthesize.cpp"
        if ($cmp9) { __label__ = 5;; } else { __label__ = 6;; } //@line 489 "synthesize.cpp"
        if (__label__ == 5) {
  
          $x=199; //@line 489 "synthesize.cpp"
          ; //@line 489 "synthesize.cpp"
        }
  
        var $10=$x; //@line 491 "synthesize.cpp"
        var $arrayidx12=((__ZZL13set_frame_rmsP7frame_tiE8sqrt_tab+$10*2)&4294967295); //@line 491 "synthesize.cpp"
        var $11=HEAP16[(($arrayidx12)>>1)]; //@line 491 "synthesize.cpp"
        var $conv13=(tempInt=(($11)),(tempInt>=32768?tempInt-65536:tempInt)); //@line 491 "synthesize.cpp"
        $x=$conv13; //@line 491 "synthesize.cpp"
        $ix=0; //@line 493 "synthesize.cpp"
        ; //@line 493 "synthesize.cpp"
        while(1) { 
  
          var $12=$ix; //@line 495 "synthesize.cpp"
          var $13=$fr_addr; //@line 495 "synthesize.cpp"
          var $fheight=(($13+18)&4294967295); //@line 495 "synthesize.cpp"
          var $arrayidx15=(($fheight+$12)&4294967295); //@line 495 "synthesize.cpp"
          var $14=HEAPU8[($arrayidx15)]; //@line 495 "synthesize.cpp"
          var $conv16=((($14))&255); //@line 495 "synthesize.cpp"
          var $15=$x; //@line 495 "synthesize.cpp"
          var $mul17=((($conv16)*($15))&4294967295); //@line 495 "synthesize.cpp"
          $h=$mul17; //@line 495 "synthesize.cpp"
          var $16=$h; //@line 496 "synthesize.cpp"
          var $div18=((((($16))|0)/512)|0); //@line 496 "synthesize.cpp"
          var $conv19=((($div18)) & 255); //@line 496 "synthesize.cpp"
          var $17=$ix; //@line 496 "synthesize.cpp"
          var $18=$fr_addr; //@line 496 "synthesize.cpp"
          var $fheight20=(($18+18)&4294967295); //@line 496 "synthesize.cpp"
          var $arrayidx21=(($fheight20+$17)&4294967295); //@line 496 "synthesize.cpp"
          HEAP8[($arrayidx21)]=$conv19; //@line 496 "synthesize.cpp"
          var $19=$ix; //@line 493 "synthesize.cpp"
          var $inc=((($19)+1)&4294967295); //@line 493 "synthesize.cpp"
          $ix=$inc; //@line 493 "synthesize.cpp"
          var $cmp14=((($inc))|0) < 8; //@line 493 "synthesize.cpp"
          if ($cmp14) { __label__ = 7;continue ; } else { __label__ = 8;break $if_then$$if_end3$2; } //@line 493 "synthesize.cpp"
        }
      }
    } while(0);
  
    ;
    return; //@line 498 "synthesize.cpp"
    return;
  }
  

  function __ZL14AdjustFormantsP7frame_tiiiiiii($fr, $target, $min, $max, $f1_adj, $f3_adj, $hf_reduce, $flags) {
    ;
    var __label__;
    var __lastLabel__ = null;
  
    var $fr_addr;
    var $target_addr;
    var $min_addr;
    var $max_addr;
    var $f1_adj_addr;
    var $f3_adj_addr;
    var $hf_reduce_addr;
    var $flags_addr;
    var $x;
    $fr_addr=$fr;
    $target_addr=$target;
    $min_addr=$min;
    $max_addr=$max;
    $f1_adj_addr=$f1_adj;
    $f3_adj_addr=$f3_adj;
    $hf_reduce_addr=$hf_reduce;
    $flags_addr=$flags;
    var $0=$target_addr; //@line 559 "synthesize.cpp"
    var $1=HEAP32[((_voice)>>2)]; //@line 559 "synthesize.cpp"
    var $formant_factor=(($1+96)&4294967295); //@line 559 "synthesize.cpp"
    var $2=HEAP32[(($formant_factor)>>2)]; //@line 559 "synthesize.cpp"
    var $mul=((($2)*($0))&4294967295); //@line 559 "synthesize.cpp"
    var $div=((((($mul))|0)/256)|0); //@line 559 "synthesize.cpp"
    $target_addr=$div; //@line 559 "synthesize.cpp"
    var $3=$target_addr; //@line 561 "synthesize.cpp"
    var $4=$fr_addr; //@line 561 "synthesize.cpp"
    var $ffreq=(($4+2)&4294967295); //@line 561 "synthesize.cpp"
    var $arrayidx=(($ffreq+4)&4294967295); //@line 561 "synthesize.cpp"
    var $5=HEAP16[(($arrayidx)>>1)]; //@line 561 "synthesize.cpp"
    var $conv=(tempInt=(($5)),(tempInt>=32768?tempInt-65536:tempInt)); //@line 561 "synthesize.cpp"
    var $sub=((($3)-($conv))&4294967295); //@line 561 "synthesize.cpp"
    var $div1=((((($sub))|0)/2)|0); //@line 561 "synthesize.cpp"
    $x=$div1; //@line 561 "synthesize.cpp"
    var $6=$x; //@line 562 "synthesize.cpp"
    var $7=$max_addr; //@line 562 "synthesize.cpp"
    var $cmp=((($6))|0) > ((($7))|0); //@line 562 "synthesize.cpp"
    if ($cmp) { __label__ = 1;; } else { __label__ = 2;; } //@line 562 "synthesize.cpp"
    if (__label__ == 1) {
  
      var $8=$max_addr; //@line 562 "synthesize.cpp"
      $x=$8; //@line 562 "synthesize.cpp"
      ; //@line 562 "synthesize.cpp"
    }
  
    var $9=$x; //@line 563 "synthesize.cpp"
    var $10=$min_addr; //@line 563 "synthesize.cpp"
    var $cmp2=((($9))|0) < ((($10))|0); //@line 563 "synthesize.cpp"
    if ($cmp2) { __label__ = 3;; } else { __label__ = 4;; } //@line 563 "synthesize.cpp"
    if (__label__ == 3) {
  
      var $11=$min_addr; //@line 563 "synthesize.cpp"
      $x=$11; //@line 563 "synthesize.cpp"
      ; //@line 563 "synthesize.cpp"
    }
  
    var $12=$x; //@line 564 "synthesize.cpp"
    var $13=$fr_addr; //@line 564 "synthesize.cpp"
    var $ffreq5=(($13+2)&4294967295); //@line 564 "synthesize.cpp"
    var $arrayidx6=(($ffreq5+4)&4294967295); //@line 564 "synthesize.cpp"
    var $14=HEAP16[(($arrayidx6)>>1)]; //@line 564 "synthesize.cpp"
    var $conv7=(tempInt=(($14)),(tempInt>=32768?tempInt-65536:tempInt)); //@line 564 "synthesize.cpp"
    var $add=((($conv7)+($12))&4294967295); //@line 564 "synthesize.cpp"
    var $conv8=((($add)) & 65535); //@line 564 "synthesize.cpp"
    HEAP16[(($arrayidx6)>>1)]=$conv8; //@line 564 "synthesize.cpp"
    var $15=$f3_adj_addr; //@line 565 "synthesize.cpp"
    var $16=$fr_addr; //@line 565 "synthesize.cpp"
    var $ffreq9=(($16+2)&4294967295); //@line 565 "synthesize.cpp"
    var $arrayidx10=(($ffreq9+6)&4294967295); //@line 565 "synthesize.cpp"
    var $17=HEAP16[(($arrayidx10)>>1)]; //@line 565 "synthesize.cpp"
    var $conv11=(tempInt=(($17)),(tempInt>=32768?tempInt-65536:tempInt)); //@line 565 "synthesize.cpp"
    var $add12=((($conv11)+($15))&4294967295); //@line 565 "synthesize.cpp"
    var $conv13=((($add12)) & 65535); //@line 565 "synthesize.cpp"
    HEAP16[(($arrayidx10)>>1)]=$conv13; //@line 565 "synthesize.cpp"
    var $18=$flags_addr; //@line 567 "synthesize.cpp"
    var $and=($18) & 32; //@line 567 "synthesize.cpp"
    var $tobool=((($and))|0)!=0; //@line 567 "synthesize.cpp"
    if ($tobool) { __label__ = 5;; } else { __label__ = 6;; } //@line 567 "synthesize.cpp"
    if (__label__ == 5) {
  
      var $19=$f3_adj_addr; //@line 569 "synthesize.cpp"
      var $sub15=(((-($19)))&4294967295); //@line 569 "synthesize.cpp"
      $f3_adj_addr=$sub15; //@line 569 "synthesize.cpp"
      ; //@line 570 "synthesize.cpp"
    }
  
    var $20=$f3_adj_addr; //@line 571 "synthesize.cpp"
    var $21=$fr_addr; //@line 571 "synthesize.cpp"
    var $ffreq17=(($21+2)&4294967295); //@line 571 "synthesize.cpp"
    var $arrayidx18=(($ffreq17+8)&4294967295); //@line 571 "synthesize.cpp"
    var $22=HEAP16[(($arrayidx18)>>1)]; //@line 571 "synthesize.cpp"
    var $conv19=(tempInt=(($22)),(tempInt>=32768?tempInt-65536:tempInt)); //@line 571 "synthesize.cpp"
    var $add20=((($conv19)+($20))&4294967295); //@line 571 "synthesize.cpp"
    var $conv21=((($add20)) & 65535); //@line 571 "synthesize.cpp"
    HEAP16[(($arrayidx18)>>1)]=$conv21; //@line 571 "synthesize.cpp"
    var $23=$f3_adj_addr; //@line 572 "synthesize.cpp"
    var $24=$fr_addr; //@line 572 "synthesize.cpp"
    var $ffreq22=(($24+2)&4294967295); //@line 572 "synthesize.cpp"
    var $arrayidx23=(($ffreq22+10)&4294967295); //@line 572 "synthesize.cpp"
    var $25=HEAP16[(($arrayidx23)>>1)]; //@line 572 "synthesize.cpp"
    var $conv24=(tempInt=(($25)),(tempInt>=32768?tempInt-65536:tempInt)); //@line 572 "synthesize.cpp"
    var $add25=((($conv24)+($23))&4294967295); //@line 572 "synthesize.cpp"
    var $conv26=((($add25)) & 65535); //@line 572 "synthesize.cpp"
    HEAP16[(($arrayidx23)>>1)]=$conv26; //@line 572 "synthesize.cpp"
    var $26=$f1_adj_addr; //@line 574 "synthesize.cpp"
    var $cmp27=((($26))|0)==1; //@line 574 "synthesize.cpp"
    if ($cmp27) { __lastLabel__ = 6; __label__ = 7;; } else { __lastLabel__ = 6; __label__ = 12;; } //@line 574 "synthesize.cpp"
    if (__label__ == 7) {
  
      var $27=$fr_addr; //@line 576 "synthesize.cpp"
      var $ffreq29=(($27+2)&4294967295); //@line 576 "synthesize.cpp"
      var $arrayidx30=(($ffreq29+2)&4294967295); //@line 576 "synthesize.cpp"
      var $28=HEAP16[(($arrayidx30)>>1)]; //@line 576 "synthesize.cpp"
      var $conv31=(tempInt=(($28)),(tempInt>=32768?tempInt-65536:tempInt)); //@line 576 "synthesize.cpp"
      var $sub32=((235-($conv31))&4294967295); //@line 576 "synthesize.cpp"
      $x=$sub32; //@line 576 "synthesize.cpp"
      var $29=$x; //@line 577 "synthesize.cpp"
      var $cmp33=((($29))|0) < -100; //@line 577 "synthesize.cpp"
      if ($cmp33) { __label__ = 8;; } else { __label__ = 9;; } //@line 577 "synthesize.cpp"
      $if_end35_thread$$if_end35$13: do { 
        if (__label__ == 8) {
  
          $x=-100; //@line 577 "synthesize.cpp"
          ; //@line 578 "synthesize.cpp"
        }
        else if (__label__ == 9) {
  
          var $cmp36=((($29))|0) > -60; //@line 578 "synthesize.cpp"
          if (!($cmp36)) { __label__ = 11;break $if_end35_thread$$if_end35$13; } //@line 578 "synthesize.cpp"
  
          $x=-60; //@line 578 "synthesize.cpp"
          ; //@line 578 "synthesize.cpp"
        }
      } while(0);
  
      var $30=$x; //@line 579 "synthesize.cpp"
      var $31=$fr_addr; //@line 579 "synthesize.cpp"
      var $ffreq39=(($31+2)&4294967295); //@line 579 "synthesize.cpp"
      var $arrayidx40=(($ffreq39+2)&4294967295); //@line 579 "synthesize.cpp"
      var $32=HEAP16[(($arrayidx40)>>1)]; //@line 579 "synthesize.cpp"
      var $conv41=(tempInt=(($32)),(tempInt>=32768?tempInt-65536:tempInt)); //@line 579 "synthesize.cpp"
      var $add42=((($conv41)+($30))&4294967295); //@line 579 "synthesize.cpp"
      var $conv43=((($add42)) & 65535); //@line 579 "synthesize.cpp"
      HEAP16[(($arrayidx40)>>1)]=$conv43; //@line 579 "synthesize.cpp"
      var $_pr=$f1_adj_addr; //@line 581 "synthesize.cpp"
      __lastLabel__ = 11; ; //@line 580 "synthesize.cpp"
    }
  
    var $33=__lastLabel__ == 11 ? $_pr : ($26); //@line 581 "synthesize.cpp"
    var $cmp45=((($33))|0)==2; //@line 581 "synthesize.cpp"
    if ($cmp45) { __label__ = 13;; } else { __label__ = 18;; } //@line 581 "synthesize.cpp"
    if (__label__ == 13) {
  
      var $34=$fr_addr; //@line 583 "synthesize.cpp"
      var $ffreq47=(($34+2)&4294967295); //@line 583 "synthesize.cpp"
      var $arrayidx48=(($ffreq47+2)&4294967295); //@line 583 "synthesize.cpp"
      var $35=HEAP16[(($arrayidx48)>>1)]; //@line 583 "synthesize.cpp"
      var $conv49=(tempInt=(($35)),(tempInt>=32768?tempInt-65536:tempInt)); //@line 583 "synthesize.cpp"
      var $sub50=((235-($conv49))&4294967295); //@line 583 "synthesize.cpp"
      $x=$sub50; //@line 583 "synthesize.cpp"
      var $36=$x; //@line 584 "synthesize.cpp"
      var $cmp51=((($36))|0) < -300; //@line 584 "synthesize.cpp"
      if ($cmp51) { __label__ = 14;; } else { __label__ = 15;; } //@line 584 "synthesize.cpp"
      $if_end53_thread$$if_end53$21: do { 
        if (__label__ == 14) {
  
          $x=-300; //@line 584 "synthesize.cpp"
          ; //@line 585 "synthesize.cpp"
        }
        else if (__label__ == 15) {
  
          var $cmp54=((($36))|0) > -150; //@line 585 "synthesize.cpp"
          if (!($cmp54)) { __label__ = 17;break $if_end53_thread$$if_end53$21; } //@line 585 "synthesize.cpp"
  
          $x=-150; //@line 585 "synthesize.cpp"
          ; //@line 585 "synthesize.cpp"
        }
      } while(0);
  
      var $37=$x; //@line 586 "synthesize.cpp"
      var $38=$fr_addr; //@line 586 "synthesize.cpp"
      var $ffreq57=(($38+2)&4294967295); //@line 586 "synthesize.cpp"
      var $arrayidx58=(($ffreq57+2)&4294967295); //@line 586 "synthesize.cpp"
      var $39=HEAP16[(($arrayidx58)>>1)]; //@line 586 "synthesize.cpp"
      var $conv59=(tempInt=(($39)),(tempInt>=32768?tempInt-65536:tempInt)); //@line 586 "synthesize.cpp"
      var $add60=((($conv59)+($37))&4294967295); //@line 586 "synthesize.cpp"
      var $conv61=((($add60)) & 65535); //@line 586 "synthesize.cpp"
      HEAP16[(($arrayidx58)>>1)]=$conv61; //@line 586 "synthesize.cpp"
      var $40=$x; //@line 587 "synthesize.cpp"
      var $41=$fr_addr; //@line 587 "synthesize.cpp"
      var $ffreq62=(($41+2)&4294967295); //@line 587 "synthesize.cpp"
      var $arrayidx63=(($ffreq62)&4294967295); //@line 587 "synthesize.cpp"
      var $42=HEAP16[(($arrayidx63)>>1)]; //@line 587 "synthesize.cpp"
      var $conv64=(tempInt=(($42)),(tempInt>=32768?tempInt-65536:tempInt)); //@line 587 "synthesize.cpp"
      var $add65=((($conv64)+($40))&4294967295); //@line 587 "synthesize.cpp"
      var $conv66=((($add65)) & 65535); //@line 587 "synthesize.cpp"
      HEAP16[(($arrayidx63)>>1)]=$conv66; //@line 587 "synthesize.cpp"
      ; //@line 588 "synthesize.cpp"
    }
  
    var $43=$f1_adj_addr; //@line 589 "synthesize.cpp"
    var $cmp68=((($43))|0)==3; //@line 589 "synthesize.cpp"
    if ($cmp68) { __label__ = 19;; } else { __label__ = 24;; } //@line 589 "synthesize.cpp"
    if (__label__ == 19) {
  
      var $44=$fr_addr; //@line 591 "synthesize.cpp"
      var $ffreq70=(($44+2)&4294967295); //@line 591 "synthesize.cpp"
      var $arrayidx71=(($ffreq70+2)&4294967295); //@line 591 "synthesize.cpp"
      var $45=HEAP16[(($arrayidx71)>>1)]; //@line 591 "synthesize.cpp"
      var $conv72=(tempInt=(($45)),(tempInt>=32768?tempInt-65536:tempInt)); //@line 591 "synthesize.cpp"
      var $sub73=((100-($conv72))&4294967295); //@line 591 "synthesize.cpp"
      $x=$sub73; //@line 591 "synthesize.cpp"
      var $46=$x; //@line 592 "synthesize.cpp"
      var $cmp74=((($46))|0) < -400; //@line 592 "synthesize.cpp"
      if ($cmp74) { __label__ = 20;; } else { __label__ = 21;; } //@line 592 "synthesize.cpp"
      $if_end76_thread$$if_end76$29: do { 
        if (__label__ == 20) {
  
          $x=-400; //@line 592 "synthesize.cpp"
          ; //@line 593 "synthesize.cpp"
        }
        else if (__label__ == 21) {
  
          var $cmp77=((($46))|0) > -300; //@line 593 "synthesize.cpp"
          if (!($cmp77)) { __label__ = 23;break $if_end76_thread$$if_end76$29; } //@line 593 "synthesize.cpp"
  
          $x=-400; //@line 593 "synthesize.cpp"
          ; //@line 593 "synthesize.cpp"
        }
      } while(0);
  
      var $47=$x; //@line 594 "synthesize.cpp"
      var $48=$fr_addr; //@line 594 "synthesize.cpp"
      var $ffreq80=(($48+2)&4294967295); //@line 594 "synthesize.cpp"
      var $arrayidx81=(($ffreq80+2)&4294967295); //@line 594 "synthesize.cpp"
      var $49=HEAP16[(($arrayidx81)>>1)]; //@line 594 "synthesize.cpp"
      var $conv82=(tempInt=(($49)),(tempInt>=32768?tempInt-65536:tempInt)); //@line 594 "synthesize.cpp"
      var $add83=((($conv82)+($47))&4294967295); //@line 594 "synthesize.cpp"
      var $conv84=((($add83)) & 65535); //@line 594 "synthesize.cpp"
      HEAP16[(($arrayidx81)>>1)]=$conv84; //@line 594 "synthesize.cpp"
      var $50=$x; //@line 595 "synthesize.cpp"
      var $51=$fr_addr; //@line 595 "synthesize.cpp"
      var $ffreq85=(($51+2)&4294967295); //@line 595 "synthesize.cpp"
      var $arrayidx86=(($ffreq85)&4294967295); //@line 595 "synthesize.cpp"
      var $52=HEAP16[(($arrayidx86)>>1)]; //@line 595 "synthesize.cpp"
      var $conv87=(tempInt=(($52)),(tempInt>=32768?tempInt-65536:tempInt)); //@line 595 "synthesize.cpp"
      var $add88=((($conv87)+($50))&4294967295); //@line 595 "synthesize.cpp"
      var $conv89=((($add88)) & 65535); //@line 595 "synthesize.cpp"
      HEAP16[(($arrayidx86)>>1)]=$conv89; //@line 595 "synthesize.cpp"
      ; //@line 596 "synthesize.cpp"
    }
  
    var $53=$fr_addr; //@line 597 "synthesize.cpp"
    var $54=$hf_reduce_addr; //@line 597 "synthesize.cpp"
    __ZL18formants_reduce_hfP7frame_ti($53, $54); //@line 597 "synthesize.cpp"
    ;
    return; //@line 598 "synthesize.cpp"
    return;
  }
  

  function __ZL14VowelClosenessP7frame_t($fr) {
    ;
    var __label__;
  
    var $retval;
    var $fr_addr;
    var $f1;
    $fr_addr=$fr;
    var $0=$fr_addr; //@line 606 "synthesize.cpp"
    var $ffreq=(($0+2)&4294967295); //@line 606 "synthesize.cpp"
    var $arrayidx=(($ffreq+2)&4294967295); //@line 606 "synthesize.cpp"
    var $1=HEAP16[(($arrayidx)>>1)]; //@line 606 "synthesize.cpp"
    var $conv=(tempInt=(($1)),(tempInt>=32768?tempInt-65536:tempInt)); //@line 606 "synthesize.cpp"
    $f1=$conv; //@line 606 "synthesize.cpp"
    var $cmp=((($conv))|0) < 300; //@line 606 "synthesize.cpp"
    if ($cmp) { __label__ = 1;; } else { __label__ = 2;; } //@line 606 "synthesize.cpp"
    if (__label__ == 1) {
  
      $retval=3; //@line 607 "synthesize.cpp"
      ; //@line 607 "synthesize.cpp"
    }
    else if (__label__ == 2) {
  
      var $2=$f1; //@line 608 "synthesize.cpp"
      var $cmp1=((($2))|0) < 400; //@line 608 "synthesize.cpp"
      if ($cmp1) { __label__ = 3;; } else { __label__ = 4;; } //@line 608 "synthesize.cpp"
      if (__label__ == 3) {
  
        $retval=2; //@line 609 "synthesize.cpp"
        ; //@line 609 "synthesize.cpp"
      }
      else if (__label__ == 4) {
  
        var $3=$f1; //@line 610 "synthesize.cpp"
        var $cmp4=((($3))|0) < 500; //@line 610 "synthesize.cpp"
        if ($cmp4) { __label__ = 5;; } else { __label__ = 6;; } //@line 610 "synthesize.cpp"
        if (__label__ == 5) {
  
          $retval=1; //@line 611 "synthesize.cpp"
          ; //@line 611 "synthesize.cpp"
        }
        else if (__label__ == 6) {
  
          $retval=0; //@line 612 "synthesize.cpp"
          ; //@line 612 "synthesize.cpp"
        }
      }
    }
  
    var $4=$retval; //@line 613 "synthesize.cpp"
    ;
    return $4; //@line 613 "synthesize.cpp"
    return null;
  }
  

  function __ZL18DuplicateLastFrameP10frameref_tii($seq, $n_frames, $length) {
    ;
    var __label__;
  
    var $seq_addr;
    var $n_frames_addr;
    var $length_addr;
    var $fr;
    $seq_addr=$seq;
    $n_frames_addr=$n_frames;
    $length_addr=$length;
    var $0=$length_addr; //@line 545 "synthesize.cpp"
    var $conv=((($0)) & 65535); //@line 545 "synthesize.cpp"
    var $1=$n_frames_addr; //@line 545 "synthesize.cpp"
    var $sub=((($1)-1)&4294967295); //@line 545 "synthesize.cpp"
    var $2=$seq_addr; //@line 545 "synthesize.cpp"
    var $arrayidx=(($2+8*$sub)&4294967295); //@line 545 "synthesize.cpp"
    var $length1=(($arrayidx)&4294967295); //@line 545 "synthesize.cpp"
    HEAP16[(($length1)>>1)]=$conv; //@line 545 "synthesize.cpp"
    var $3=$n_frames_addr; //@line 546 "synthesize.cpp"
    var $sub2=((($3)-1)&4294967295); //@line 546 "synthesize.cpp"
    var $4=$seq_addr; //@line 546 "synthesize.cpp"
    var $arrayidx3=(($4+8*$sub2)&4294967295); //@line 546 "synthesize.cpp"
    var $frame=(($arrayidx3+4)&4294967295); //@line 546 "synthesize.cpp"
    var $5=HEAP32[(($frame)>>2)]; //@line 546 "synthesize.cpp"
    var $call=__ZL9CopyFrameP7frame_ti($5, 1); //@line 546 "synthesize.cpp"
    $fr=$call; //@line 546 "synthesize.cpp"
    var $6=$fr; //@line 547 "synthesize.cpp"
    var $7=$n_frames_addr; //@line 547 "synthesize.cpp"
    var $8=$seq_addr; //@line 547 "synthesize.cpp"
    var $arrayidx4=(($8+8*$7)&4294967295); //@line 547 "synthesize.cpp"
    var $frame5=(($arrayidx4+4)&4294967295); //@line 547 "synthesize.cpp"
    HEAP32[(($frame5)>>2)]=$6; //@line 547 "synthesize.cpp"
    var $9=$n_frames_addr; //@line 548 "synthesize.cpp"
    var $10=$seq_addr; //@line 548 "synthesize.cpp"
    var $arrayidx6=(($10+8*$9)&4294967295); //@line 548 "synthesize.cpp"
    var $length7=(($arrayidx6)&4294967295); //@line 548 "synthesize.cpp"
    HEAP16[(($length7)>>1)]=0; //@line 548 "synthesize.cpp"
    var $11=$fr; //@line 549 "synthesize.cpp"
    ;
    return $11; //@line 549 "synthesize.cpp"
    return null;
  }
  

  function __ZL7DoPauseii($length, $control) {
    ;
    var __label__;
  
    var $length_addr;
    var $control_addr;
    var $len;
    $length_addr=$length;
    $control_addr=$control;
    var $0=$length_addr; //@line 216 "synthesize.cpp"
    var $cmp=((($0))|0)==0; //@line 216 "synthesize.cpp"
    if ($cmp) { __label__ = 1;; } else { __label__ = 2;; } //@line 216 "synthesize.cpp"
    if (__label__ == 1) {
  
      $len=0; //@line 217 "synthesize.cpp"
      ; //@line 217 "synthesize.cpp"
    }
    else if (__label__ == 2) {
  
      var $1=$length_addr; //@line 220 "synthesize.cpp"
      var $2=$control_addr; //@line 220 "synthesize.cpp"
      var $call=__Z11PauseLengthii($1, $2); //@line 220 "synthesize.cpp"
      $len=$call; //@line 220 "synthesize.cpp"
      var $3=$len; //@line 222 "synthesize.cpp"
      var $4=HEAP32[((_samplerate)>>2)]; //@line 222 "synthesize.cpp"
      var $mul=((($4)*($3))&4294967295); //@line 222 "synthesize.cpp"
      var $div=((((($mul))|0)/1000)|0); //@line 222 "synthesize.cpp"
      $len=$div; //@line 222 "synthesize.cpp"
      ;
    }
  
    __ZL8EndPitchi(1); //@line 225 "synthesize.cpp"
    var $5=HEAP32[((_wcmdq_tail)>>2)]; //@line 226 "synthesize.cpp"
    var $arrayidx=((_wcmdq+$5*16)&4294967295); //@line 226 "synthesize.cpp"
    var $arrayidx1=(($arrayidx)&4294967295); //@line 226 "synthesize.cpp"
    HEAP32[(($arrayidx1)>>2)]=5; //@line 226 "synthesize.cpp"
    var $6=$len; //@line 227 "synthesize.cpp"
    var $7=HEAP32[((_wcmdq_tail)>>2)]; //@line 227 "synthesize.cpp"
    var $arrayidx2=((_wcmdq+$7*16)&4294967295); //@line 227 "synthesize.cpp"
    var $arrayidx3=(($arrayidx2+4)&4294967295); //@line 227 "synthesize.cpp"
    HEAP32[(($arrayidx3)>>2)]=$6; //@line 227 "synthesize.cpp"
    __Z8WcmdqIncv(); //@line 228 "synthesize.cpp"
    HEAP32[((__ZL10last_frame)>>2)]=0; //@line 229 "synthesize.cpp"
    var $8=HEAP32[((__ZL13fmt_amplitude)>>2)]; //@line 231 "synthesize.cpp"
    var $cmp4=((($8))|0)!=0; //@line 231 "synthesize.cpp"
    if ($cmp4) { __label__ = 4;; } else { __label__ = 5;; } //@line 231 "synthesize.cpp"
    if (__label__ == 4) {
  
      var $9=HEAP32[((_wcmdq_tail)>>2)]; //@line 233 "synthesize.cpp"
      var $arrayidx6=((_wcmdq+$9*16)&4294967295); //@line 233 "synthesize.cpp"
      var $arrayidx7=(($arrayidx6)&4294967295); //@line 233 "synthesize.cpp"
      HEAP32[(($arrayidx7)>>2)]=14; //@line 233 "synthesize.cpp"
      HEAP32[((__ZL13fmt_amplitude)>>2)]=0; //@line 234 "synthesize.cpp"
      var $10=HEAP32[((_wcmdq_tail)>>2)]; //@line 234 "synthesize.cpp"
      var $arrayidx8=((_wcmdq+$10*16)&4294967295); //@line 234 "synthesize.cpp"
      var $arrayidx9=(($arrayidx8+4)&4294967295); //@line 234 "synthesize.cpp"
      HEAP32[(($arrayidx9)>>2)]=0; //@line 234 "synthesize.cpp"
      __Z8WcmdqIncv(); //@line 235 "synthesize.cpp"
      ; //@line 236 "synthesize.cpp"
    }
  
    ;
    return; //@line 237 "synthesize.cpp"
    return;
  }
  

  function __Z8DoSpect2P11PHONEME_TABiP10FMT_PARAMSP12PHONEME_LISTi($this_ph, $which, $fmt_params, $plist, $modulation) {
    var __stackBase__  = STACKTOP; STACKTOP += 104; _memset(__stackBase__, 0, 104);
    var __label__;
    var __lastLabel__ = null;
  
    var $retval;
    var $this_ph_addr;
    var $which_addr;
    var $fmt_params_addr;
    var $plist_addr;
    var $modulation_addr;
    var $n_frames=__stackBase__;
    var $frames;
    var $frameix;
    var $frame1;
    var $frame2;
    var $fr;
    var $ix;
    var $q;
    var $len;
    var $frame_length;
    var $length_factor;
    var $length_mod;
    var $length_sum;
    var $length_min;
    var $total_len;
    var $wcmd_spect;
    var $frame_lengths=__stackBase__+4;
    $this_ph_addr=$this_ph;
    $which_addr=$which;
    $fmt_params_addr=$fmt_params;
    $plist_addr=$plist;
    $modulation_addr=$modulation;
    $total_len=0; //@line 1007 "synthesize.cpp"
    $wcmd_spect=3; //@line 1009 "synthesize.cpp"
    var $0=$fmt_params_addr; //@line 1012 "synthesize.cpp"
    var $fmt_addr=(($0+8)&4294967295); //@line 1012 "synthesize.cpp"
    var $1=HEAP32[(($fmt_addr)>>2)]; //@line 1012 "synthesize.cpp"
    var $cmp=((($1))|0)==0; //@line 1012 "synthesize.cpp"
    if ($cmp) { __label__ = 1;; } else { __label__ = 2;; } //@line 1012 "synthesize.cpp"
    if (__label__ == 1) {
  
      $retval=0; //@line 1013 "synthesize.cpp"
      ; //@line 1013 "synthesize.cpp"
    }
    else if (__label__ == 2) {
  
      var $2=$plist_addr; //@line 1015 "synthesize.cpp"
      var $length=(($2+12)&4294967295); //@line 1015 "synthesize.cpp"
      var $3=HEAP16[(($length)>>1)]; //@line 1015 "synthesize.cpp"
      var $conv=(tempInt=(($3)),(tempInt>=32768?tempInt-65536:tempInt)); //@line 1015 "synthesize.cpp"
      $length_mod=$conv; //@line 1015 "synthesize.cpp"
      var $4=$length_mod; //@line 1016 "synthesize.cpp"
      var $cmp1=((($4))|0)==0; //@line 1016 "synthesize.cpp"
      if ($cmp1) { __label__ = 3;; } else { __label__ = 4;; } //@line 1016 "synthesize.cpp"
      if (__label__ == 3) {
  
        $length_mod=256; //@line 1016 "synthesize.cpp"
        ; //@line 1016 "synthesize.cpp"
      }
  
      var $5=HEAP32[((_samplerate)>>2)]; //@line 1018 "synthesize.cpp"
      var $div=((((($5))|0)/70)|0); //@line 1018 "synthesize.cpp"
      $length_min=$div; //@line 1018 "synthesize.cpp"
      var $6=$which_addr; //@line 1019 "synthesize.cpp"
      var $cmp4=((($6))|0)==2; //@line 1019 "synthesize.cpp"
      if ($cmp4) { __lastLabel__ = 4; __label__ = 5;; } else { __lastLabel__ = 4; __label__ = 11;; } //@line 1019 "synthesize.cpp"
      if (__label__ == 5) {
  
        var $7=HEAP32[((_translator)>>2)]; //@line 1021 "synthesize.cpp"
        var $langopts=(($7)&4294967295); //@line 1021 "synthesize.cpp"
        var $param=(($langopts+24)&4294967295); //@line 1021 "synthesize.cpp"
        var $arrayidx=(($param+72)&4294967295); //@line 1021 "synthesize.cpp"
        var $8=HEAP32[(($arrayidx)>>2)]; //@line 1021 "synthesize.cpp"
        var $cmp6=((($8))|0) > 0; //@line 1021 "synthesize.cpp"
        if ($cmp6) { __label__ = 6;; } else { __label__ = 10;; } //@line 1021 "synthesize.cpp"
        $land_lhs_true$$if_end18thread_pre_split$10: do { 
          if (__label__ == 6) {
  
            var $9=$this_ph_addr; //@line 1021 "synthesize.cpp"
            var $std_length=(($9+14)&4294967295); //@line 1021 "synthesize.cpp"
            var $10=HEAPU8[($std_length)]; //@line 1021 "synthesize.cpp"
            var $conv7=((($10))&255); //@line 1021 "synthesize.cpp"
            var $11=HEAP32[((_translator)>>2)]; //@line 1021 "synthesize.cpp"
            var $langopts8=(($11)&4294967295); //@line 1021 "synthesize.cpp"
            var $param9=(($langopts8+24)&4294967295); //@line 1021 "synthesize.cpp"
            var $arrayidx10=(($param9+72)&4294967295); //@line 1021 "synthesize.cpp"
            var $12=HEAP32[(($arrayidx10)>>2)]; //@line 1021 "synthesize.cpp"
            var $cmp11=((($conv7))|0) >= ((($12))|0); //@line 1021 "synthesize.cpp"
            if ($cmp11) { __label__ = 9;; } else { __label__ = 7;; } //@line 1021 "synthesize.cpp"
            $if_then16$$lor_lhs_false$12: do { 
              if (__label__ == 7) {
  
                var $13=$plist_addr; //@line 1021 "synthesize.cpp"
                var $synthflags=(($13+4)&4294967295); //@line 1021 "synthesize.cpp"
                var $14=HEAPU16[(($synthflags)>>1)]; //@line 1021 "synthesize.cpp"
                var $conv12=((($14))&65535); //@line 1021 "synthesize.cpp"
                var $and=($conv12) & 8; //@line 1021 "synthesize.cpp"
                var $tobool=((($and))|0)!=0; //@line 1021 "synthesize.cpp"
                if ($tobool) { __label__ = 9;break $if_then16$$lor_lhs_false$12; } //@line 1021 "synthesize.cpp"
  
                var $15=$this_ph_addr; //@line 1021 "synthesize.cpp"
                var $phflags=(($15+4)&4294967295); //@line 1021 "synthesize.cpp"
                var $16=HEAP32[(($phflags)>>2)]; //@line 1021 "synthesize.cpp"
                var $and14=($16) & 2097152; //@line 1021 "synthesize.cpp"
                var $tobool15=((($and14))|0)!=0; //@line 1021 "synthesize.cpp"
                if (!($tobool15)) { __label__ = 10;break $land_lhs_true$$if_end18thread_pre_split$10; } //@line 1021 "synthesize.cpp"
              }
            } while(0);
  
            var $17=$length_min; //@line 1022 "synthesize.cpp"
            var $mul=((($17)*2)&4294967295); //@line 1022 "synthesize.cpp"
            $length_min=$mul; //@line 1022 "synthesize.cpp"
            ; //@line 1022 "synthesize.cpp"
          }
        } while(0);
  
        var $_pr=$which_addr; //@line 1025 "synthesize.cpp"
        __lastLabel__ = 10; ;
      }
  
      var $18=__lastLabel__ == 10 ? $_pr : ($6); //@line 1025 "synthesize.cpp"
      var $cmp19=((($18))|0)==1; //@line 1025 "synthesize.cpp"
      if ($cmp19) { __label__ = 12;; } else { __label__ = 17;; } //@line 1025 "synthesize.cpp"
      $if_then20$$if_end41$18: do { 
        if (__label__ == 12) {
  
          var $19=$this_ph_addr; //@line 1028 "synthesize.cpp"
          var $type=(($19+11)&4294967295); //@line 1028 "synthesize.cpp"
          var $20=HEAPU8[($type)]; //@line 1028 "synthesize.cpp"
          var $conv21=((($20))&255); //@line 1028 "synthesize.cpp"
          var $cmp22=((($conv21))|0)==3; //@line 1028 "synthesize.cpp"
          if ($cmp22) { __label__ = 15;; } else { __label__ = 13;; } //@line 1028 "synthesize.cpp"
          $if_then33$$lor_lhs_false23$20: do { 
            if (__label__ == 13) {
  
              var $21=$plist_addr; //@line 1028 "synthesize.cpp"
              var $arrayidx24=(($21+-24)&4294967295); //@line 1028 "synthesize.cpp"
              var $type25=(($arrayidx24+15)&4294967295); //@line 1028 "synthesize.cpp"
              var $22=HEAPU8[($type25)]; //@line 1028 "synthesize.cpp"
              var $conv26=((($22))&255); //@line 1028 "synthesize.cpp"
              var $cmp27=((($conv26))|0)==3; //@line 1028 "synthesize.cpp"
              if ($cmp27) { __label__ = 15;break $if_then33$$lor_lhs_false23$20; } //@line 1028 "synthesize.cpp"
  
              var $23=$plist_addr; //@line 1028 "synthesize.cpp"
              var $arrayidx29=(($23+-24)&4294967295); //@line 1028 "synthesize.cpp"
              var $type30=(($arrayidx29+15)&4294967295); //@line 1028 "synthesize.cpp"
              var $24=HEAPU8[($type30)]; //@line 1028 "synthesize.cpp"
              var $conv31=((($24))&255); //@line 1028 "synthesize.cpp"
              var $cmp32=((($conv31))|0)==8; //@line 1028 "synthesize.cpp"
              if (!($cmp32)) { __label__ = 17;break $if_then20$$if_end41$18; } //@line 1028 "synthesize.cpp"
            }
          } while(0);
  
          var $25=$length_mod; //@line 1030 "synthesize.cpp"
          var $26=HEAP32[((_translator)>>2)]; //@line 1030 "synthesize.cpp"
          var $langopts34=(($26)&4294967295); //@line 1030 "synthesize.cpp"
          var $param35=(($langopts34+24)&4294967295); //@line 1030 "synthesize.cpp"
          var $arrayidx36=(($param35+28)&4294967295); //@line 1030 "synthesize.cpp"
          var $27=HEAP32[(($arrayidx36)>>2)]; //@line 1030 "synthesize.cpp"
          $len=$27; //@line 1030 "synthesize.cpp"
          var $cmp37=((($25))|0) < ((($27))|0); //@line 1030 "synthesize.cpp"
          if (!($cmp37)) { __label__ = 17;break $if_then20$$if_end41$18; } //@line 1030 "synthesize.cpp"
  
          var $28=$len; //@line 1032 "synthesize.cpp"
          $length_mod=$28; //@line 1032 "synthesize.cpp"
          ; //@line 1033 "synthesize.cpp"
        }
      } while(0);
  
      HEAP32[((__ZL10modn_flags)>>2)]=0; //@line 1037 "synthesize.cpp"
      var $29=$this_ph_addr; //@line 1038 "synthesize.cpp"
      var $30=$which_addr; //@line 1038 "synthesize.cpp"
      var $31=$fmt_params_addr; //@line 1038 "synthesize.cpp"
      var $32=$plist_addr; //@line 1038 "synthesize.cpp"
      var $call=__Z11LookupSpectP11PHONEME_TABiP10FMT_PARAMSPiP12PHONEME_LIST($29, $30, $31, $n_frames, $32); //@line 1038 "synthesize.cpp"
      $frames=$call; //@line 1038 "synthesize.cpp"
      var $cmp42=((($call))|0)==0; //@line 1039 "synthesize.cpp"
      if ($cmp42) { __label__ = 18;; } else { __label__ = 19;; } //@line 1039 "synthesize.cpp"
      if (__label__ == 18) {
  
        $retval=0; //@line 1040 "synthesize.cpp"
        ; //@line 1040 "synthesize.cpp"
      }
      else if (__label__ == 19) {
  
        var $33=$fmt_params_addr; //@line 1042 "synthesize.cpp"
        var $fmt_amp=(($33+16)&4294967295); //@line 1042 "synthesize.cpp"
        var $34=HEAP32[(($fmt_amp)>>2)]; //@line 1042 "synthesize.cpp"
        var $35=HEAP32[((__ZL13fmt_amplitude)>>2)]; //@line 1042 "synthesize.cpp"
        var $cmp45=((($34))|0)!=((($35))|0); //@line 1042 "synthesize.cpp"
        if ($cmp45) { __label__ = 20;; } else { __label__ = 21;; } //@line 1042 "synthesize.cpp"
        if (__label__ == 20) {
  
          var $36=HEAP32[((_wcmdq_tail)>>2)]; //@line 1045 "synthesize.cpp"
          var $arrayidx47=((_wcmdq+$36*16)&4294967295); //@line 1045 "synthesize.cpp"
          var $arraydecay=(($arrayidx47)&4294967295); //@line 1045 "synthesize.cpp"
          $q=$arraydecay; //@line 1045 "synthesize.cpp"
          var $37=$q; //@line 1046 "synthesize.cpp"
          var $arrayidx48=(($37)&4294967295); //@line 1046 "synthesize.cpp"
          HEAP32[(($arrayidx48)>>2)]=14; //@line 1046 "synthesize.cpp"
          var $38=$fmt_params_addr; //@line 1047 "synthesize.cpp"
          var $fmt_amp49=(($38+16)&4294967295); //@line 1047 "synthesize.cpp"
          var $39=HEAP32[(($fmt_amp49)>>2)]; //@line 1047 "synthesize.cpp"
          HEAP32[((__ZL13fmt_amplitude)>>2)]=$39; //@line 1047 "synthesize.cpp"
          var $40=$q; //@line 1047 "synthesize.cpp"
          var $arrayidx50=(($40+4)&4294967295); //@line 1047 "synthesize.cpp"
          HEAP32[(($arrayidx50)>>2)]=$39; //@line 1047 "synthesize.cpp"
          __Z8WcmdqIncv(); //@line 1048 "synthesize.cpp"
          ; //@line 1049 "synthesize.cpp"
        }
  
        var $41=$frames; //@line 1051 "synthesize.cpp"
        var $arrayidx52=(($41)&4294967295); //@line 1051 "synthesize.cpp"
        var $frame=(($arrayidx52+4)&4294967295); //@line 1051 "synthesize.cpp"
        var $42=HEAP32[(($frame)>>2)]; //@line 1051 "synthesize.cpp"
        $frame1=$42; //@line 1051 "synthesize.cpp"
        var $43=HEAP32[((_voice)>>2)]; //@line 1052 "synthesize.cpp"
        var $klattv=(($43+112)&4294967295); //@line 1052 "synthesize.cpp"
        var $arrayidx53=(($klattv)&4294967295); //@line 1052 "synthesize.cpp"
        var $44=HEAP32[(($arrayidx53)>>2)]; //@line 1052 "synthesize.cpp"
        var $tobool54=((($44))|0)!=0; //@line 1052 "synthesize.cpp"
        if ($tobool54) { __label__ = 22;; } else { __label__ = 23;; } //@line 1052 "synthesize.cpp"
        if (__label__ == 22) {
  
          $wcmd_spect=1; //@line 1053 "synthesize.cpp"
          ; //@line 1053 "synthesize.cpp"
        }
  
        var $45=$fmt_params_addr; //@line 1055 "synthesize.cpp"
        var $wav_addr=(($45+28)&4294967295); //@line 1055 "synthesize.cpp"
        var $46=HEAP32[(($wav_addr)>>2)]; //@line 1055 "synthesize.cpp"
        HEAP32[((_wavefile_ix)>>2)]=$46; //@line 1055 "synthesize.cpp"
        var $47=$fmt_params_addr; //@line 1057 "synthesize.cpp"
        var $wav_amp=(($47+32)&4294967295); //@line 1057 "synthesize.cpp"
        var $48=HEAP32[(($wav_amp)>>2)]; //@line 1057 "synthesize.cpp"
        var $cmp57=((($48))|0)==0; //@line 1057 "synthesize.cpp"
        if ($cmp57) { __label__ = 24;; } else { __label__ = 25;; } //@line 1057 "synthesize.cpp"
        if (__label__ == 24) {
  
          HEAP32[((_wavefile_amp)>>2)]=32; //@line 1058 "synthesize.cpp"
          ; //@line 1058 "synthesize.cpp"
        }
        else if (__label__ == 25) {
  
          var $49=$fmt_params_addr; //@line 1060 "synthesize.cpp"
          var $wav_amp59=(($49+32)&4294967295); //@line 1060 "synthesize.cpp"
          var $50=HEAP32[(($wav_amp59)>>2)]; //@line 1060 "synthesize.cpp"
          var $mul60=((($50)*32)&4294967295); //@line 1060 "synthesize.cpp"
          var $div61=((((($mul60))|0)/100)|0); //@line 1060 "synthesize.cpp"
          HEAP32[((_wavefile_amp)>>2)]=$div61; //@line 1060 "synthesize.cpp"
          ;
        }
  
        var $51=HEAP32[((_wavefile_ix)>>2)]; //@line 1062 "synthesize.cpp"
        var $cmp63=((($51))|0)==0; //@line 1062 "synthesize.cpp"
        if ($cmp63) { __label__ = 27;; } else { __label__ = 33;; } //@line 1062 "synthesize.cpp"
        $if_then64$$if_end79$39: do { 
          if (__label__ == 27) {
  
            var $_b=HEAPU8[(__ZZ8DoSpect2P11PHONEME_TABiP10FMT_PARAMSP12PHONEME_LISTiE9wave_flag_b)];
            var $52=((($_b))&1);
            var $tobool65=((($52))|0)!=0; //@line 1064 "synthesize.cpp"
            if ($tobool65) { __label__ = 28;; } else { __label__ = 31;; } //@line 1064 "synthesize.cpp"
            if (__label__ == 28) {
  
              $wcmd_spect=4; //@line 1067 "synthesize.cpp"
              var $53=HEAP32[((_voice)>>2)]; //@line 1068 "synthesize.cpp"
              var $klattv67=(($53+112)&4294967295); //@line 1068 "synthesize.cpp"
              var $arrayidx68=(($klattv67)&4294967295); //@line 1068 "synthesize.cpp"
              var $54=HEAP32[(($arrayidx68)>>2)]; //@line 1068 "synthesize.cpp"
              var $tobool69=((($54))|0)!=0; //@line 1068 "synthesize.cpp"
              if ($tobool69) { __label__ = 29;; } else { __label__ = 30;; } //@line 1068 "synthesize.cpp"
              if (__label__ == 29) {
  
                $wcmd_spect=2; //@line 1069 "synthesize.cpp"
                ; //@line 1069 "synthesize.cpp"
              }
  
              HEAP8[(__ZZ8DoSpect2P11PHONEME_TABiP10FMT_PARAMSP12PHONEME_LISTiE9wave_flag_b)]=0;
              ; //@line 1071 "synthesize.cpp"
            }
            else if (__label__ == 31) {
  
              $wcmd_spect=3; //@line 1074 "synthesize.cpp"
              var $55=HEAP32[((_voice)>>2)]; //@line 1075 "synthesize.cpp"
              var $klattv73=(($55+112)&4294967295); //@line 1075 "synthesize.cpp"
              var $arrayidx74=(($klattv73)&4294967295); //@line 1075 "synthesize.cpp"
              var $56=HEAP32[(($arrayidx74)>>2)]; //@line 1075 "synthesize.cpp"
              var $tobool75=((($56))|0)!=0; //@line 1075 "synthesize.cpp"
              if (!($tobool75)) { __label__ = 33;break $if_then64$$if_end79$39; } //@line 1075 "synthesize.cpp"
  
              $wcmd_spect=1; //@line 1076 "synthesize.cpp"
              ; //@line 1076 "synthesize.cpp"
            }
          }
        } while(0);
  
        var $57=HEAP32[((__ZL10last_frame)>>2)]; //@line 1080 "synthesize.cpp"
        var $cmp80=((($57))|0)!=0; //@line 1080 "synthesize.cpp"
        if ($cmp80) { __label__ = 34;; } else { __label__ = 43;; } //@line 1080 "synthesize.cpp"
        $if_then81$$if_end117$49: do { 
          if (__label__ == 34) {
  
            var $58=HEAP32[((__ZL10last_frame)>>2)]; //@line 1082 "synthesize.cpp"
            var $length82=(($58+16)&4294967295); //@line 1082 "synthesize.cpp"
            var $59=HEAPU8[($length82)]; //@line 1082 "synthesize.cpp"
            var $conv83=((($59))&255); //@line 1082 "synthesize.cpp"
            var $cmp84=((($conv83))|0) < 2; //@line 1082 "synthesize.cpp"
            if ($cmp84) { __label__ = 36;; } else { __label__ = 35;; } //@line 1082 "synthesize.cpp"
            if (__label__ == 35) {
  
              var $60=HEAP32[((__ZL10last_frame)>>2)]; //@line 1082 "synthesize.cpp"
              var $frflags=(($60)&4294967295); //@line 1082 "synthesize.cpp"
              var $61=HEAP16[(($frflags)>>1)]; //@line 1082 "synthesize.cpp"
              var $conv86=(tempInt=(($61)),(tempInt>=32768?tempInt-65536:tempInt)); //@line 1082 "synthesize.cpp"
              var $and87=($conv86) & 2; //@line 1082 "synthesize.cpp"
              var $tobool88=((($and87))|0)!=0; //@line 1082 "synthesize.cpp"
              if (!($tobool88)) { __label__ = 43;break $if_then81$$if_end117$49; } //@line 1082 "synthesize.cpp"
            }
  
            var $62=HEAP32[((__ZL10last_frame)>>2)]; //@line 1082 "synthesize.cpp"
            var $frflags90=(($62)&4294967295); //@line 1082 "synthesize.cpp"
            var $63=HEAP16[(($frflags90)>>1)]; //@line 1082 "synthesize.cpp"
            var $conv91=(tempInt=(($63)),(tempInt>=32768?tempInt-65536:tempInt)); //@line 1082 "synthesize.cpp"
            var $and92=($conv91) & 16; //@line 1082 "synthesize.cpp"
            var $tobool93=((($and92))|0)!=0; //@line 1082 "synthesize.cpp"
            if ($tobool93) { __label__ = 43;break $if_then81$$if_end117$49; } //@line 1082 "synthesize.cpp"
  
            var $64=$frame1; //@line 1086 "synthesize.cpp"
            var $65=($64); //@line 1086 "synthesize.cpp"
            var $66=HEAP32[((__ZL10last_wcmdq)>>2)]; //@line 1086 "synthesize.cpp"
            var $arrayidx95=((_wcmdq+$66*16)&4294967295); //@line 1086 "synthesize.cpp"
            var $arrayidx96=(($arrayidx95+12)&4294967295); //@line 1086 "synthesize.cpp"
            HEAP32[(($arrayidx96)>>2)]=$65; //@line 1086 "synthesize.cpp"
            var $67=HEAP32[((__ZL10last_frame)>>2)]; //@line 1088 "synthesize.cpp"
            var $frflags97=(($67)&4294967295); //@line 1088 "synthesize.cpp"
            var $68=HEAP16[(($frflags97)>>1)]; //@line 1088 "synthesize.cpp"
            var $conv98=(tempInt=(($68)),(tempInt>=32768?tempInt-65536:tempInt)); //@line 1088 "synthesize.cpp"
            var $and99=($conv98) & 8; //@line 1088 "synthesize.cpp"
            var $tobool100=((($and99))|0)!=0; //@line 1088 "synthesize.cpp"
            if (!($tobool100)) { __label__ = 43;break $if_then81$$if_end117$49; } //@line 1088 "synthesize.cpp"
  
            var $69=$frame1; //@line 1091 "synthesize.cpp"
            var $call102=__ZL9CopyFrameP7frame_ti($69, 1); //@line 1091 "synthesize.cpp"
            $fr=$call102; //@line 1091 "synthesize.cpp"
            $ix=3; //@line 1092 "synthesize.cpp"
            __lastLabel__ = 38; ; //@line 1092 "synthesize.cpp"
            while(1) { 
  
              var $70=__lastLabel__ == 38 ? 3 : ($inc); //@line 1094 "synthesize.cpp"
              var $cmp104=((($70))|0) < 7; //@line 1094 "synthesize.cpp"
              if ($cmp104) { __label__ = 40;; } else { __label__ = 41;; } //@line 1094 "synthesize.cpp"
              if (__label__ == 40) {
  
                var $71=$ix; //@line 1095 "synthesize.cpp"
                var $72=HEAP32[((__ZL10last_frame)>>2)]; //@line 1095 "synthesize.cpp"
                var $ffreq=(($72+2)&4294967295); //@line 1095 "synthesize.cpp"
                var $arrayidx106=(($ffreq+$71*2)&4294967295); //@line 1095 "synthesize.cpp"
                var $73=HEAP16[(($arrayidx106)>>1)]; //@line 1095 "synthesize.cpp"
                var $74=$ix; //@line 1095 "synthesize.cpp"
                var $75=$fr; //@line 1095 "synthesize.cpp"
                var $ffreq107=(($75+2)&4294967295); //@line 1095 "synthesize.cpp"
                var $arrayidx108=(($ffreq107+$74*2)&4294967295); //@line 1095 "synthesize.cpp"
                HEAP16[(($arrayidx108)>>1)]=$73; //@line 1095 "synthesize.cpp"
                ; //@line 1095 "synthesize.cpp"
              }
  
              var $76=$ix; //@line 1096 "synthesize.cpp"
              var $77=HEAP32[((__ZL10last_frame)>>2)]; //@line 1096 "synthesize.cpp"
              var $fheight=(($77+18)&4294967295); //@line 1096 "synthesize.cpp"
              var $arrayidx110=(($fheight+$76)&4294967295); //@line 1096 "synthesize.cpp"
              var $78=HEAP8[($arrayidx110)]; //@line 1096 "synthesize.cpp"
              var $79=$ix; //@line 1096 "synthesize.cpp"
              var $80=$fr; //@line 1096 "synthesize.cpp"
              var $fheight111=(($80+18)&4294967295); //@line 1096 "synthesize.cpp"
              var $arrayidx112=(($fheight111+$79)&4294967295); //@line 1096 "synthesize.cpp"
              HEAP8[($arrayidx112)]=$78; //@line 1096 "synthesize.cpp"
              var $81=$ix; //@line 1092 "synthesize.cpp"
              var $inc=((($81)+1)&4294967295); //@line 1092 "synthesize.cpp"
              $ix=$inc; //@line 1092 "synthesize.cpp"
              var $cmp103=((($inc))|0) < 8; //@line 1092 "synthesize.cpp"
              if ($cmp103) { __lastLabel__ = 41; __label__ = 39;continue ; } else { __lastLabel__ = 41; __label__ = 42;break ; } //@line 1092 "synthesize.cpp"
            }
  
            var $82=$fr; //@line 1098 "synthesize.cpp"
            var $83=($82); //@line 1098 "synthesize.cpp"
            var $84=HEAP32[((__ZL10last_wcmdq)>>2)]; //@line 1098 "synthesize.cpp"
            var $arrayidx113=((_wcmdq+$84*16)&4294967295); //@line 1098 "synthesize.cpp"
            var $arrayidx114=(($arrayidx113+12)&4294967295); //@line 1098 "synthesize.cpp"
            HEAP32[(($arrayidx114)>>2)]=$83; //@line 1098 "synthesize.cpp"
            ; //@line 1099 "synthesize.cpp"
          }
        } while(0);
  
        var $85=$this_ph_addr; //@line 1103 "synthesize.cpp"
        var $type118=(($85+11)&4294967295); //@line 1103 "synthesize.cpp"
        var $86=HEAPU8[($type118)]; //@line 1103 "synthesize.cpp"
        var $conv119=((($86))&255); //@line 1103 "synthesize.cpp"
        var $cmp120=((($conv119))|0)==2; //@line 1103 "synthesize.cpp"
        if ($cmp120) { __label__ = 44;; } else { __label__ = 46;; } //@line 1103 "synthesize.cpp"
        $land_lhs_true121$$if_end124$63: do { 
          if (__label__ == 44) {
  
            var $87=$which_addr; //@line 1103 "synthesize.cpp"
            var $cmp122=((($87))|0)==2; //@line 1103 "synthesize.cpp"
            if (!($cmp122)) { __label__ = 46;break $land_lhs_true121$$if_end124$63; } //@line 1103 "synthesize.cpp"
  
            __ZL11SmoothSpectv(); //@line 1105 "synthesize.cpp"
            var $88=HEAP32[((_wcmdq_tail)>>2)]; //@line 1108 "synthesize.cpp"
            HEAP32[((__ZL15syllable_centre)>>2)]=$88; //@line 1108 "synthesize.cpp"
            ; //@line 1109 "synthesize.cpp"
          }
        } while(0);
  
        $length_sum=0; //@line 1111 "synthesize.cpp"
        $frameix=1; //@line 1112 "synthesize.cpp"
        var $89=$frameix; //@line 1112 "synthesize.cpp"
        var $90=HEAP32[(($n_frames)>>2)]; //@line 1112 "synthesize.cpp"
        var $cmp1263=((($89))|0) < ((($90))|0); //@line 1112 "synthesize.cpp"
        if ($cmp1263) { __label__ = 47;; } else { __label__ = 56;; } //@line 1112 "synthesize.cpp"
        $for_body127$$if_end180$67: do { 
          if (__label__ == 47) {
            $for_body127$68: while(1) { 
  
              var $91=$length_mod; //@line 1114 "synthesize.cpp"
              $length_factor=$91; //@line 1114 "synthesize.cpp"
              var $92=$frameix; //@line 1115 "synthesize.cpp"
              var $sub=((($92)-1)&4294967295); //@line 1115 "synthesize.cpp"
              var $93=$frames; //@line 1115 "synthesize.cpp"
              var $arrayidx128=(($93+8*$sub)&4294967295); //@line 1115 "synthesize.cpp"
              var $frflags129=(($arrayidx128+2)&4294967295); //@line 1115 "synthesize.cpp"
              var $94=HEAP16[(($frflags129)>>1)]; //@line 1115 "synthesize.cpp"
              var $conv130=(tempInt=(($94)),(tempInt>=32768?tempInt-65536:tempInt)); //@line 1115 "synthesize.cpp"
              var $and131=($conv130) & 4; //@line 1115 "synthesize.cpp"
              var $tobool132=((($and131))|0)!=0; //@line 1115 "synthesize.cpp"
              if ($tobool132) { __label__ = 48;; } else { __label__ = 49;; } //@line 1115 "synthesize.cpp"
              $if_then133$$if_else138$70: do { 
                if (__label__ == 48) {
  
                  var $95=$length_mod; //@line 1117 "synthesize.cpp"
                  var $96=HEAP32[((((_speed+16)&4294967295))>>2)]; //@line 1117 "synthesize.cpp"
                  var $sub134=((256-($96))&4294967295); //@line 1117 "synthesize.cpp"
                  var $mul135=((($sub134)*($95))&4294967295); //@line 1117 "synthesize.cpp"
                  var $97=HEAP32[((((_speed+16)&4294967295))>>2)]; //@line 1117 "synthesize.cpp"
                  var $mul136=((($97)*256)&4294967295); //@line 1117 "synthesize.cpp"
                  var $add=((($mul135)+($mul136))&4294967295); //@line 1117 "synthesize.cpp"
                  var $div137=((((($add))|0)/256)|0); //@line 1117 "synthesize.cpp"
                  $length_factor=$div137; //@line 1117 "synthesize.cpp"
                  ; //@line 1118 "synthesize.cpp"
                }
                else if (__label__ == 49) {
  
                  var $98=$frameix; //@line 1120 "synthesize.cpp"
                  var $sub139=((($98)-1)&4294967295); //@line 1120 "synthesize.cpp"
                  var $99=$frames; //@line 1120 "synthesize.cpp"
                  var $arrayidx140=(($99+8*$sub139)&4294967295); //@line 1120 "synthesize.cpp"
                  var $frflags141=(($arrayidx140+2)&4294967295); //@line 1120 "synthesize.cpp"
                  var $100=HEAP16[(($frflags141)>>1)]; //@line 1120 "synthesize.cpp"
                  var $conv142=(tempInt=(($100)),(tempInt>=32768?tempInt-65536:tempInt)); //@line 1120 "synthesize.cpp"
                  var $and143=($conv142) & 16384; //@line 1120 "synthesize.cpp"
                  var $tobool144=((($and143))|0)!=0; //@line 1120 "synthesize.cpp"
                  if (!($tobool144)) { __label__ = 51;break $if_then133$$if_else138$70; } //@line 1120 "synthesize.cpp"
  
                  var $101=$length_mod; //@line 1122 "synthesize.cpp"
                  var $102=HEAP32[((((_speed+20)&4294967295))>>2)]; //@line 1122 "synthesize.cpp"
                  var $sub146=((256-($102))&4294967295); //@line 1122 "synthesize.cpp"
                  var $mul147=((($sub146)*($101))&4294967295); //@line 1122 "synthesize.cpp"
                  var $103=HEAP32[((((_speed+20)&4294967295))>>2)]; //@line 1122 "synthesize.cpp"
                  var $mul148=((($103)*256)&4294967295); //@line 1122 "synthesize.cpp"
                  var $add149=((($mul147)+($mul148))&4294967295); //@line 1122 "synthesize.cpp"
                  var $div150=((((($add149))|0)/256)|0); //@line 1122 "synthesize.cpp"
                  $length_factor=$div150; //@line 1122 "synthesize.cpp"
                  ; //@line 1123 "synthesize.cpp"
                }
              } while(0);
  
              var $104=$frameix; //@line 1125 "synthesize.cpp"
              var $sub153=((($104)-1)&4294967295); //@line 1125 "synthesize.cpp"
              var $105=$frames; //@line 1125 "synthesize.cpp"
              var $arrayidx154=(($105+8*$sub153)&4294967295); //@line 1125 "synthesize.cpp"
              var $length155=(($arrayidx154)&4294967295); //@line 1125 "synthesize.cpp"
              var $106=HEAP16[(($length155)>>1)]; //@line 1125 "synthesize.cpp"
              var $conv156=(tempInt=(($106)),(tempInt>=32768?tempInt-65536:tempInt)); //@line 1125 "synthesize.cpp"
              $frame_length=$conv156; //@line 1125 "synthesize.cpp"
              var $107=$frame_length; //@line 1126 "synthesize.cpp"
              var $108=HEAP32[((_samplerate)>>2)]; //@line 1126 "synthesize.cpp"
              var $mul157=((($108)*($107))&4294967295); //@line 1126 "synthesize.cpp"
              var $div158=((((($mul157))|0)/1000)|0); //@line 1126 "synthesize.cpp"
              $len=$div158; //@line 1126 "synthesize.cpp"
              var $109=$len; //@line 1127 "synthesize.cpp"
              var $110=$length_factor; //@line 1127 "synthesize.cpp"
              var $mul159=((($110)*($109))&4294967295); //@line 1127 "synthesize.cpp"
              var $div160=((((($mul159))|0)/256)|0); //@line 1127 "synthesize.cpp"
              $len=$div160; //@line 1127 "synthesize.cpp"
              var $111=$len; //@line 1128 "synthesize.cpp"
              var $112=$length_sum; //@line 1128 "synthesize.cpp"
              var $add161=((($112)+($111))&4294967295); //@line 1128 "synthesize.cpp"
              $length_sum=$add161; //@line 1128 "synthesize.cpp"
              var $113=$len; //@line 1129 "synthesize.cpp"
              var $114=$frameix; //@line 1129 "synthesize.cpp"
              var $arrayidx162=(($frame_lengths+$114*4)&4294967295); //@line 1129 "synthesize.cpp"
              HEAP32[(($arrayidx162)>>2)]=$113; //@line 1129 "synthesize.cpp"
              var $115=$frameix; //@line 1112 "synthesize.cpp"
              var $inc164=((($115)+1)&4294967295); //@line 1112 "synthesize.cpp"
              $frameix=$inc164; //@line 1112 "synthesize.cpp"
              var $116=$frameix; //@line 1112 "synthesize.cpp"
              var $117=HEAP32[(($n_frames)>>2)]; //@line 1112 "synthesize.cpp"
              var $cmp126=((($116))|0) < ((($117))|0); //@line 1112 "synthesize.cpp"
              if ($cmp126) { __label__ = 47;continue $for_body127$68; } else { __label__ = 52;break $for_body127$68; } //@line 1112 "synthesize.cpp"
            }
  
            var $_pr4=$length_sum; //@line 1132 "synthesize.cpp"
            var $cmp166=((($_pr4))|0) > 0; //@line 1132 "synthesize.cpp"
            if (!($cmp166)) { __label__ = 56;break $for_body127$$if_end180$67; } //@line 1132 "synthesize.cpp"
  
            var $118=$length_sum; //@line 1132 "synthesize.cpp"
            var $119=$length_min; //@line 1132 "synthesize.cpp"
            var $cmp168=((($118))|0) < ((($119))|0); //@line 1132 "synthesize.cpp"
            if (!($cmp168)) { __label__ = 56;break $for_body127$$if_end180$67; } //@line 1132 "synthesize.cpp"
  
            $frameix=1; //@line 1135 "synthesize.cpp"
            var $120=$frameix; //@line 1135 "synthesize.cpp"
            var $121=HEAP32[(($n_frames)>>2)]; //@line 1135 "synthesize.cpp"
            var $cmp1712=((($120))|0) < ((($121))|0); //@line 1135 "synthesize.cpp"
            if (!($cmp1712)) { __label__ = 56;break $for_body127$$if_end180$67; } //@line 1135 "synthesize.cpp"
            while(1) { 
  
              var $122=$frameix; //@line 1137 "synthesize.cpp"
              var $arrayidx173=(($frame_lengths+$122*4)&4294967295); //@line 1137 "synthesize.cpp"
              var $123=HEAP32[(($arrayidx173)>>2)]; //@line 1137 "synthesize.cpp"
              var $124=$length_min; //@line 1137 "synthesize.cpp"
              var $mul174=((($124)*($123))&4294967295); //@line 1137 "synthesize.cpp"
              var $125=$length_sum; //@line 1137 "synthesize.cpp"
              var $div175=((((($mul174))|0)/((($125))|0))|0); //@line 1137 "synthesize.cpp"
              var $126=$frameix; //@line 1137 "synthesize.cpp"
              var $arrayidx176=(($frame_lengths+$126*4)&4294967295); //@line 1137 "synthesize.cpp"
              HEAP32[(($arrayidx176)>>2)]=$div175; //@line 1137 "synthesize.cpp"
              var $127=$frameix; //@line 1135 "synthesize.cpp"
              var $inc178=((($127)+1)&4294967295); //@line 1135 "synthesize.cpp"
              $frameix=$inc178; //@line 1135 "synthesize.cpp"
              var $128=$frameix; //@line 1135 "synthesize.cpp"
              var $129=HEAP32[(($n_frames)>>2)]; //@line 1135 "synthesize.cpp"
              var $cmp171=((($128))|0) < ((($129))|0); //@line 1135 "synthesize.cpp"
              if ($cmp171) { __label__ = 55;continue ; } else { __label__ = 56;break $for_body127$$if_end180$67; } //@line 1135 "synthesize.cpp"
            }
          }
        } while(0);
  
        $frameix=1; //@line 1141 "synthesize.cpp"
        var $130=$frameix; //@line 1141 "synthesize.cpp"
        var $131=HEAP32[(($n_frames)>>2)]; //@line 1141 "synthesize.cpp"
        var $cmp1821=((($130))|0) < ((($131))|0); //@line 1141 "synthesize.cpp"
        if ($cmp1821) { __label__ = 57;; } else { __label__ = 72;; } //@line 1141 "synthesize.cpp"
        $for_body183$$for_end235$81: do { 
          if (__label__ == 57) {
            $for_body183$82: while(1) { 
  
              var $132=$frameix; //@line 1143 "synthesize.cpp"
              var $133=$frames; //@line 1143 "synthesize.cpp"
              var $arrayidx184=(($133+8*$132)&4294967295); //@line 1143 "synthesize.cpp"
              var $frame185=(($arrayidx184+4)&4294967295); //@line 1143 "synthesize.cpp"
              var $134=HEAP32[(($frame185)>>2)]; //@line 1143 "synthesize.cpp"
              $frame2=$134; //@line 1143 "synthesize.cpp"
              var $135=$fmt_params_addr; //@line 1145 "synthesize.cpp"
              var $wav_addr186=(($135+28)&4294967295); //@line 1145 "synthesize.cpp"
              var $136=HEAP32[(($wav_addr186)>>2)]; //@line 1145 "synthesize.cpp"
              var $cmp187=((($136))|0)!=0; //@line 1145 "synthesize.cpp"
              if ($cmp187) { __label__ = 58;; } else { __label__ = 60;; } //@line 1145 "synthesize.cpp"
              $land_lhs_true188$$if_end198$84: do { 
                if (__label__ == 58) {
  
                  var $137=$frame1; //@line 1145 "synthesize.cpp"
                  var $frflags189=(($137)&4294967295); //@line 1145 "synthesize.cpp"
                  var $138=HEAP16[(($frflags189)>>1)]; //@line 1145 "synthesize.cpp"
                  var $conv190=(tempInt=(($138)),(tempInt>=32768?tempInt-65536:tempInt)); //@line 1145 "synthesize.cpp"
                  var $and191=($conv190) & 128; //@line 1145 "synthesize.cpp"
                  var $cmp192=((($and191))|0)==0; //@line 1145 "synthesize.cpp"
                  if (!($cmp192)) { __label__ = 60;break $land_lhs_true188$$if_end198$84; } //@line 1145 "synthesize.cpp"
  
                  HEAP32[((_seq_len_adjust)>>2)]=0; //@line 1148 "synthesize.cpp"
                  var $139=$fmt_params_addr; //@line 1149 "synthesize.cpp"
                  var $wav_addr194=(($139+28)&4294967295); //@line 1149 "synthesize.cpp"
                  var $140=HEAP32[(($wav_addr194)>>2)]; //@line 1149 "synthesize.cpp"
                  var $141=$which_addr; //@line 1149 "synthesize.cpp"
                  var $add195=((($141)+256)&4294967295); //@line 1149 "synthesize.cpp"
                  var $142=$fmt_params_addr; //@line 1149 "synthesize.cpp"
                  var $fmt_control=(($142)&4294967295); //@line 1149 "synthesize.cpp"
                  var $143=HEAP32[(($fmt_control)>>2)]; //@line 1149 "synthesize.cpp"
                  var $144=HEAP32[((_wavefile_amp)>>2)]; //@line 1149 "synthesize.cpp"
                  var $call196=__ZL9DoSample2iiiiii($140, $add195, 0, $143, 0, $144); //@line 1149 "synthesize.cpp"
                  HEAP8[(__ZZ8DoSpect2P11PHONEME_TABiP10FMT_PARAMSP12PHONEME_LISTiE9wave_flag_b)]=1;
                  HEAP32[((_wavefile_ix)>>2)]=0; //@line 1151 "synthesize.cpp"
                  var $145=$fmt_params_addr; //@line 1152 "synthesize.cpp"
                  var $wav_addr197=(($145+28)&4294967295); //@line 1152 "synthesize.cpp"
                  HEAP32[(($wav_addr197)>>2)]=0; //@line 1152 "synthesize.cpp"
                  ; //@line 1153 "synthesize.cpp"
                }
              } while(0);
  
              var $146=$modulation_addr; //@line 1155 "synthesize.cpp"
              var $cmp199=((($146))|0) >= 0; //@line 1155 "synthesize.cpp"
              if ($cmp199) { __label__ = 61;; } else { __label__ = 66;; } //@line 1155 "synthesize.cpp"
              $if_then200$$if_end214$88: do { 
                if (__label__ == 61) {
  
                  var $147=$frame1; //@line 1157 "synthesize.cpp"
                  var $frflags201=(($147)&4294967295); //@line 1157 "synthesize.cpp"
                  var $148=HEAP16[(($frflags201)>>1)]; //@line 1157 "synthesize.cpp"
                  var $conv202=(tempInt=(($148)),(tempInt>=32768?tempInt-65536:tempInt)); //@line 1157 "synthesize.cpp"
                  var $and203=($conv202) & 64; //@line 1157 "synthesize.cpp"
                  var $tobool204=((($and203))|0)!=0; //@line 1157 "synthesize.cpp"
                  if ($tobool204) { __label__ = 62;; } else { __label__ = 63;; } //@line 1157 "synthesize.cpp"
                  if (__label__ == 62) {
  
                    $modulation_addr=6; //@line 1159 "synthesize.cpp"
                    ; //@line 1160 "synthesize.cpp"
                  }
  
                  var $149=$frameix; //@line 1161 "synthesize.cpp"
                  var $150=HEAP32[(($n_frames)>>2)]; //@line 1161 "synthesize.cpp"
                  var $sub207=((($150)-1)&4294967295); //@line 1161 "synthesize.cpp"
                  var $cmp208=((($149))|0)==((($sub207))|0); //@line 1161 "synthesize.cpp"
                  if (!($cmp208)) { __label__ = 66;break $if_then200$$if_end214$88; } //@line 1161 "synthesize.cpp"
  
                  var $151=HEAP32[((__ZL10modn_flags)>>2)]; //@line 1161 "synthesize.cpp"
                  var $and210=($151) & 3840; //@line 1161 "synthesize.cpp"
                  var $tobool211=((($and210))|0)!=0; //@line 1161 "synthesize.cpp"
                  if (!($tobool211)) { __label__ = 66;break $if_then200$$if_end214$88; } //@line 1161 "synthesize.cpp"
  
                  var $152=HEAP32[((__ZL10modn_flags)>>2)]; //@line 1162 "synthesize.cpp"
                  var $153=$modulation_addr; //@line 1162 "synthesize.cpp"
                  var $or=($153) | ($152); //@line 1162 "synthesize.cpp"
                  $modulation_addr=$or; //@line 1162 "synthesize.cpp"
                  ; //@line 1162 "synthesize.cpp"
                }
              } while(0);
  
              var $154=$frameix; //@line 1165 "synthesize.cpp"
              var $arrayidx215=(($frame_lengths+$154*4)&4294967295); //@line 1165 "synthesize.cpp"
              var $155=HEAP32[(($arrayidx215)>>2)]; //@line 1165 "synthesize.cpp"
              $len=$155; //@line 1165 "synthesize.cpp"
              var $156=HEAP32[((__ZL12pitch_length)>>2)]; //@line 1166 "synthesize.cpp"
              var $add216=((($156)+($155))&4294967295); //@line 1166 "synthesize.cpp"
              HEAP32[((__ZL12pitch_length)>>2)]=$add216; //@line 1166 "synthesize.cpp"
              var $157=HEAP32[((__ZL10amp_length)>>2)]; //@line 1167 "synthesize.cpp"
              var $add217=((($157)+($155))&4294967295); //@line 1167 "synthesize.cpp"
              HEAP32[((__ZL10amp_length)>>2)]=$add217; //@line 1167 "synthesize.cpp"
              var $cmp218=((($155))|0)==0; //@line 1169 "synthesize.cpp"
              if ($cmp218) { __label__ = 67;; } else { __label__ = 68;; } //@line 1169 "synthesize.cpp"
              if (__label__ == 67) {
  
                HEAP32[((__ZL10last_frame)>>2)]=0; //@line 1171 "synthesize.cpp"
                var $158=$frame2; //@line 1172 "synthesize.cpp"
                $frame1=$158; //@line 1172 "synthesize.cpp"
                ; //@line 1173 "synthesize.cpp"
              }
              else if (__label__ == 68) {
  
                var $159=HEAP32[((_wcmdq_tail)>>2)]; //@line 1176 "synthesize.cpp"
                HEAP32[((__ZL10last_wcmdq)>>2)]=$159; //@line 1176 "synthesize.cpp"
                var $160=$modulation_addr; //@line 1178 "synthesize.cpp"
                var $cmp221=((($160))|0) >= 0; //@line 1178 "synthesize.cpp"
                if ($cmp221) { __label__ = 69;; } else { __label__ = 70;; } //@line 1178 "synthesize.cpp"
                if (__label__ == 69) {
  
                  var $161=HEAP32[((_wcmdq_tail)>>2)]; //@line 1180 "synthesize.cpp"
                  var $arrayidx223=((_wcmdq+$161*16)&4294967295); //@line 1180 "synthesize.cpp"
                  var $arraydecay224=(($arrayidx223)&4294967295); //@line 1180 "synthesize.cpp"
                  $q=$arraydecay224; //@line 1180 "synthesize.cpp"
                  var $162=$wcmd_spect; //@line 1181 "synthesize.cpp"
                  var $163=$q; //@line 1181 "synthesize.cpp"
                  var $arrayidx225=(($163)&4294967295); //@line 1181 "synthesize.cpp"
                  HEAP32[(($arrayidx225)>>2)]=$162; //@line 1181 "synthesize.cpp"
                  var $164=$len; //@line 1182 "synthesize.cpp"
                  var $165=$modulation_addr; //@line 1182 "synthesize.cpp"
                  var $shl=((($165)*65536)&4294967295); //@line 1182 "synthesize.cpp"
                  var $add226=((($shl)+($164))&4294967295); //@line 1182 "synthesize.cpp"
                  var $166=$q; //@line 1182 "synthesize.cpp"
                  var $arrayidx227=(($166+4)&4294967295); //@line 1182 "synthesize.cpp"
                  HEAP32[(($arrayidx227)>>2)]=$add226; //@line 1182 "synthesize.cpp"
                  var $167=$frame1; //@line 1183 "synthesize.cpp"
                  var $168=($167); //@line 1183 "synthesize.cpp"
                  var $169=$q; //@line 1183 "synthesize.cpp"
                  var $arrayidx228=(($169+8)&4294967295); //@line 1183 "synthesize.cpp"
                  HEAP32[(($arrayidx228)>>2)]=$168; //@line 1183 "synthesize.cpp"
                  var $170=$frame2; //@line 1184 "synthesize.cpp"
                  var $171=($170); //@line 1184 "synthesize.cpp"
                  var $172=$q; //@line 1184 "synthesize.cpp"
                  var $arrayidx229=(($172+12)&4294967295); //@line 1184 "synthesize.cpp"
                  HEAP32[(($arrayidx229)>>2)]=$171; //@line 1184 "synthesize.cpp"
                  __Z8WcmdqIncv(); //@line 1186 "synthesize.cpp"
                  ; //@line 1187 "synthesize.cpp"
                }
  
                var $173=$frame2; //@line 1188 "synthesize.cpp"
                $frame1=$173; //@line 1188 "synthesize.cpp"
                HEAP32[((__ZL10last_frame)>>2)]=$173; //@line 1188 "synthesize.cpp"
                var $174=$len; //@line 1189 "synthesize.cpp"
                var $175=$total_len; //@line 1189 "synthesize.cpp"
                var $add231=((($175)+($174))&4294967295); //@line 1189 "synthesize.cpp"
                $total_len=$add231; //@line 1189 "synthesize.cpp"
                ;
              }
  
              var $176=$frameix; //@line 1141 "synthesize.cpp"
              var $inc234=((($176)+1)&4294967295); //@line 1141 "synthesize.cpp"
              $frameix=$inc234; //@line 1141 "synthesize.cpp"
              var $177=$frameix; //@line 1141 "synthesize.cpp"
              var $178=HEAP32[(($n_frames)>>2)]; //@line 1141 "synthesize.cpp"
              var $cmp182=((($177))|0) < ((($178))|0); //@line 1141 "synthesize.cpp"
              if ($cmp182) { __label__ = 57;continue $for_body183$82; } else { __label__ = 72;break $for_body183$$for_end235$81; } //@line 1141 "synthesize.cpp"
            }
          }
        } while(0);
  
        var $179=$which_addr; //@line 1193 "synthesize.cpp"
        var $cmp236=((($179))|0)!=1; //@line 1193 "synthesize.cpp"
        var $180=HEAP32[((__ZL13fmt_amplitude)>>2)]; //@line 1193 "synthesize.cpp"
        var $cmp238=((($180))|0)!=0; //@line 1193 "synthesize.cpp"
        var $or_cond=($cmp236) & ($cmp238); //@line 1193 "synthesize.cpp"
        if ($or_cond) { __label__ = 73;; } else { __label__ = 74;; } //@line 1193 "synthesize.cpp"
        if (__label__ == 73) {
  
          var $181=HEAP32[((_wcmdq_tail)>>2)]; //@line 1195 "synthesize.cpp"
          var $arrayidx240=((_wcmdq+$181*16)&4294967295); //@line 1195 "synthesize.cpp"
          var $arraydecay241=(($arrayidx240)&4294967295); //@line 1195 "synthesize.cpp"
          $q=$arraydecay241; //@line 1195 "synthesize.cpp"
          var $182=$q; //@line 1196 "synthesize.cpp"
          var $arrayidx242=(($182)&4294967295); //@line 1196 "synthesize.cpp"
          HEAP32[(($arrayidx242)>>2)]=14; //@line 1196 "synthesize.cpp"
          HEAP32[((__ZL13fmt_amplitude)>>2)]=0; //@line 1197 "synthesize.cpp"
          var $183=$q; //@line 1197 "synthesize.cpp"
          var $arrayidx243=(($183+4)&4294967295); //@line 1197 "synthesize.cpp"
          HEAP32[(($arrayidx243)>>2)]=0; //@line 1197 "synthesize.cpp"
          __Z8WcmdqIncv(); //@line 1198 "synthesize.cpp"
          ; //@line 1199 "synthesize.cpp"
        }
  
        var $184=$total_len; //@line 1202 "synthesize.cpp"
        $retval=$184; //@line 1202 "synthesize.cpp"
        ; //@line 1202 "synthesize.cpp"
      }
    }
  
    var $185=$retval; //@line 1203 "synthesize.cpp"
    STACKTOP = __stackBase__;
    return $185; //@line 1203 "synthesize.cpp"
    return null;
  }
  

  function __ZL11SmoothSpectv() {
    ;
    var __label__;
    var __lastLabel__ = null;
  
    var $q;
    var $frame;
    var $frame2;
    var $frame1;
    var $frame_centre;
    var $ix;
    var $len;
    var $pk;
    var $modified;
    var $allowed;
    var $diff;
    var $f1;
    var $f2;
    var $f1142;
    var $f2143;
    var $0=HEAP32[((__ZL14syllable_start)>>2)]; //@line 790 "synthesize.cpp"
    var $1=HEAP32[((__ZL12syllable_end)>>2)]; //@line 790 "synthesize.cpp"
    var $cmp=((($0))|0)==((($1))|0); //@line 790 "synthesize.cpp"
    if ($cmp) { __label__ = 58;; } else { __label__ = 1;; } //@line 790 "synthesize.cpp"
    $return$$if_end$2: do { 
      if (__label__ == 1) {
  
        var $2=HEAP32[((__ZL15syllable_centre)>>2)]; //@line 793 "synthesize.cpp"
        var $cmp1=((($2))|0) < 0; //@line 793 "synthesize.cpp"
        if ($cmp1) { __label__ = 3;; } else { __label__ = 2;; } //@line 793 "synthesize.cpp"
        $if_then3$$lor_lhs_false$4: do { 
          if (__label__ == 2) {
  
            var $3=HEAP32[((__ZL15syllable_centre)>>2)]; //@line 793 "synthesize.cpp"
            var $4=HEAP32[((__ZL14syllable_start)>>2)]; //@line 793 "synthesize.cpp"
            var $cmp2=((($3))|0)==((($4))|0); //@line 793 "synthesize.cpp"
            if ($cmp2) { __label__ = 3;break $if_then3$$lor_lhs_false$4; } //@line 793 "synthesize.cpp"
  
            var $6=HEAP32[((__ZL15syllable_centre)>>2)]; //@line 799 "synthesize.cpp"
            var $arrayidx=((_wcmdq+$6*16)&4294967295); //@line 799 "synthesize.cpp"
            var $arraydecay=(($arrayidx)&4294967295); //@line 799 "synthesize.cpp"
            $q=$arraydecay; //@line 799 "synthesize.cpp"
            var $7=$q; //@line 800 "synthesize.cpp"
            var $arrayidx5=(($7+8)&4294967295); //@line 800 "synthesize.cpp"
            var $8=HEAP32[(($arrayidx5)>>2)]; //@line 800 "synthesize.cpp"
            var $9=($8); //@line 800 "synthesize.cpp"
            $frame_centre=$9; //@line 800 "synthesize.cpp"
            var $10=HEAP32[((__ZL15syllable_centre)>>2)]; //@line 803 "synthesize.cpp"
            var $sub=((($10)-1)&4294967295); //@line 803 "synthesize.cpp"
            $ix=$sub; //@line 803 "synthesize.cpp"
            var $11=$frame_centre; //@line 804 "synthesize.cpp"
            $frame2=$11; //@line 804 "synthesize.cpp"
            $frame=$11; //@line 804 "synthesize.cpp"
            __lastLabel__ = 4; ; //@line 805 "synthesize.cpp"
            $for_cond$7: while(1) { 
  
              var $12=__lastLabel__ == 30 ? $dec : ($sub); //@line 807 "synthesize.cpp"
              var $cmp6=((($12))|0) < 0; //@line 807 "synthesize.cpp"
              if ($cmp6) { __label__ = 6;; } else { __label__ = 7;; } //@line 807 "synthesize.cpp"
              if (__label__ == 6) {
  
                $ix=159; //@line 807 "synthesize.cpp"
                ; //@line 807 "synthesize.cpp"
              }
  
              var $13=$ix; //@line 808 "synthesize.cpp"
              var $arrayidx9=((_wcmdq+$13*16)&4294967295); //@line 808 "synthesize.cpp"
              var $arraydecay10=(($arrayidx9)&4294967295); //@line 808 "synthesize.cpp"
              $q=$arraydecay10; //@line 808 "synthesize.cpp"
              var $14=$q; //@line 810 "synthesize.cpp"
              var $arrayidx11=(($14)&4294967295); //@line 810 "synthesize.cpp"
              var $15=HEAP32[(($arrayidx11)>>2)]; //@line 810 "synthesize.cpp"
              var $cmp12=((($15))|0)==5; //@line 810 "synthesize.cpp"
              if ($cmp12) { __label__ = 31;break $for_cond$7; } //@line 810 "synthesize.cpp"
  
              var $16=$q; //@line 810 "synthesize.cpp"
              var $arrayidx14=(($16)&4294967295); //@line 810 "synthesize.cpp"
              var $17=HEAP32[(($arrayidx14)>>2)]; //@line 810 "synthesize.cpp"
              var $cmp15=((($17))|0)==6; //@line 810 "synthesize.cpp"
              if ($cmp15) { __label__ = 31;break $for_cond$7; } //@line 810 "synthesize.cpp"
  
              var $18=$q; //@line 813 "synthesize.cpp"
              var $arrayidx18=(($18)&4294967295); //@line 813 "synthesize.cpp"
              var $19=HEAP32[(($arrayidx18)>>2)]; //@line 813 "synthesize.cpp"
              var $cmp19=((($19))|0) <= 4; //@line 813 "synthesize.cpp"
              if ($cmp19) { __label__ = 10;; } else { __label__ = 29;; } //@line 813 "synthesize.cpp"
              $if_then20$$if_end95$14: do { 
                if (__label__ == 10) {
  
                  var $20=$q; //@line 815 "synthesize.cpp"
                  var $arrayidx21=(($20+4)&4294967295); //@line 815 "synthesize.cpp"
                  var $21=HEAP32[(($arrayidx21)>>2)]; //@line 815 "synthesize.cpp"
                  var $and=($21) & 65535; //@line 815 "synthesize.cpp"
                  $len=$and; //@line 815 "synthesize.cpp"
                  var $22=$q; //@line 817 "synthesize.cpp"
                  var $arrayidx22=(($22+12)&4294967295); //@line 817 "synthesize.cpp"
                  var $23=HEAP32[(($arrayidx22)>>2)]; //@line 817 "synthesize.cpp"
                  var $24=($23); //@line 817 "synthesize.cpp"
                  $frame1=$24; //@line 817 "synthesize.cpp"
                  var $25=$frame1; //@line 818 "synthesize.cpp"
                  var $26=$frame; //@line 818 "synthesize.cpp"
                  var $cmp23=((($25))|0)==((($26))|0); //@line 818 "synthesize.cpp"
                  if (!($cmp23)) { __label__ = 31;break $for_cond$7; } //@line 818 "synthesize.cpp"
  
                  var $27=$frame2; //@line 820 "synthesize.cpp"
                  var $28=($27); //@line 820 "synthesize.cpp"
                  var $29=$q; //@line 820 "synthesize.cpp"
                  var $arrayidx25=(($29+12)&4294967295); //@line 820 "synthesize.cpp"
                  HEAP32[(($arrayidx25)>>2)]=$28; //@line 820 "synthesize.cpp"
                  var $30=$frame2; //@line 821 "synthesize.cpp"
                  $frame1=$30; //@line 821 "synthesize.cpp"
                  var $31=$q; //@line 826 "synthesize.cpp"
                  var $arrayidx27=(($31+8)&4294967295); //@line 826 "synthesize.cpp"
                  var $32=HEAP32[(($arrayidx27)>>2)]; //@line 826 "synthesize.cpp"
                  var $33=($32); //@line 826 "synthesize.cpp"
                  $frame2=$33; //@line 826 "synthesize.cpp"
                  $frame=$33; //@line 826 "synthesize.cpp"
                  $modified=0; //@line 827 "synthesize.cpp"
                  var $34=$frame; //@line 829 "synthesize.cpp"
                  var $frflags=(($34)&4294967295); //@line 829 "synthesize.cpp"
                  var $35=HEAP16[(($frflags)>>1)]; //@line 829 "synthesize.cpp"
                  var $conv=(tempInt=(($35)),(tempInt>=32768?tempInt-65536:tempInt)); //@line 829 "synthesize.cpp"
                  var $and28=($conv) & 16; //@line 829 "synthesize.cpp"
                  var $tobool=((($and28))|0)!=0; //@line 829 "synthesize.cpp"
                  if ($tobool) { __label__ = 31;break $for_cond$7; } //@line 829 "synthesize.cpp"
  
                  var $36=$frame; //@line 832 "synthesize.cpp"
                  var $frflags31=(($36)&4294967295); //@line 832 "synthesize.cpp"
                  var $37=HEAP16[(($frflags31)>>1)]; //@line 832 "synthesize.cpp"
                  var $conv32=(tempInt=(($37)),(tempInt>=32768?tempInt-65536:tempInt)); //@line 832 "synthesize.cpp"
                  var $and33=($conv32) & 32; //@line 832 "synthesize.cpp"
                  var $tobool34=((($and33))|0)!=0; //@line 832 "synthesize.cpp"
                  if ($tobool34) { __label__ = 13;; } else { __label__ = 14;; } //@line 832 "synthesize.cpp"
                  if (__label__ == 13) {
  
                    var $38=$len; //@line 833 "synthesize.cpp"
                    var $mul=((($38)*12)&4294967295); //@line 833 "synthesize.cpp"
                    var $div=((((($mul))|0)/10)|0); //@line 833 "synthesize.cpp"
                    $len=$div; //@line 833 "synthesize.cpp"
                    ; //@line 833 "synthesize.cpp"
                  }
  
                  $pk=0; //@line 835 "synthesize.cpp"
                  ; //@line 835 "synthesize.cpp"
                  $for_body$21: while(1) { 
  
                    var $39=$frame; //@line 839 "synthesize.cpp"
                    var $frflags39=(($39)&4294967295); //@line 839 "synthesize.cpp"
                    var $40=HEAP16[(($frflags39)>>1)]; //@line 839 "synthesize.cpp"
                    var $conv40=(tempInt=(($40)),(tempInt>=32768?tempInt-65536:tempInt)); //@line 839 "synthesize.cpp"
                    var $and41=($conv40) & 8; //@line 839 "synthesize.cpp"
                    var $tobool42=((($and41))|0)!=0; //@line 839 "synthesize.cpp"
                    if ($tobool42) { __label__ = 16;; } else { __label__ = 17;; } //@line 839 "synthesize.cpp"
                    $land_lhs_true$$if_end45$23: do { 
                      if (__label__ == 16) {
  
                        var $41=$pk; //@line 839 "synthesize.cpp"
                        var $cmp43=((($41))|0) < 3; //@line 839 "synthesize.cpp"
                        if ($cmp43) { __label__ = 28;break $land_lhs_true$$if_end45$23; } else { __label__ = 17;break $land_lhs_true$$if_end45$23; } //@line 839 "synthesize.cpp"
                      }
                    } while(0);
                    $for_inc$$if_end45$25: do { 
                      if (__label__ == 17) {
  
                        var $42=$pk; //@line 842 "synthesize.cpp"
                        var $43=$frame1; //@line 842 "synthesize.cpp"
                        var $ffreq=(($43+2)&4294967295); //@line 842 "synthesize.cpp"
                        var $arrayidx46=(($ffreq+$42*2)&4294967295); //@line 842 "synthesize.cpp"
                        var $44=HEAP16[(($arrayidx46)>>1)]; //@line 842 "synthesize.cpp"
                        var $conv47=(tempInt=(($44)),(tempInt>=32768?tempInt-65536:tempInt)); //@line 842 "synthesize.cpp"
                        $f1=$conv47; //@line 842 "synthesize.cpp"
                        var $45=$pk; //@line 843 "synthesize.cpp"
                        var $46=$frame; //@line 843 "synthesize.cpp"
                        var $ffreq48=(($46+2)&4294967295); //@line 843 "synthesize.cpp"
                        var $arrayidx49=(($ffreq48+$45*2)&4294967295); //@line 843 "synthesize.cpp"
                        var $47=HEAP16[(($arrayidx49)>>1)]; //@line 843 "synthesize.cpp"
                        var $conv50=(tempInt=(($47)),(tempInt>=32768?tempInt-65536:tempInt)); //@line 843 "synthesize.cpp"
                        $f2=$conv50; //@line 843 "synthesize.cpp"
                        var $48=$f2; //@line 846 "synthesize.cpp"
                        var $49=$f1; //@line 846 "synthesize.cpp"
                        var $sub51=((($48)-($49))&4294967295); //@line 846 "synthesize.cpp"
                        $diff=$sub51; //@line 846 "synthesize.cpp"
                        var $cmp52=((($sub51))|0) > 0; //@line 846 "synthesize.cpp"
                        var $50=$f1; //@line 848 "synthesize.cpp"
                        if ($cmp52) { __label__ = 18;; } else { __label__ = 19;; } //@line 846 "synthesize.cpp"
                        if (__label__ == 18) {
  
                          var $mul54=((($50)*2)&4294967295); //@line 848 "synthesize.cpp"
                          var $51=$f2; //@line 848 "synthesize.cpp"
                          var $add=((($51)+($mul54))&4294967295); //@line 848 "synthesize.cpp"
                          $allowed=$add; //@line 848 "synthesize.cpp"
                          ; //@line 849 "synthesize.cpp"
                        }
                        else if (__label__ == 19) {
  
                          var $52=$f2; //@line 852 "synthesize.cpp"
                          var $mul56=((($52)*2)&4294967295); //@line 852 "synthesize.cpp"
                          var $add57=((($mul56)+($50))&4294967295); //@line 852 "synthesize.cpp"
                          $allowed=$add57; //@line 852 "synthesize.cpp"
                          ;
                        }
  
                        var $53=$allowed; //@line 857 "synthesize.cpp"
                        var $54=$pk; //@line 857 "synthesize.cpp"
                        var $arrayidx59=((_formant_rate+$54*4)&4294967295); //@line 857 "synthesize.cpp"
                        var $55=HEAP32[(($arrayidx59)>>2)]; //@line 857 "synthesize.cpp"
                        var $mul60=((($55)*($53))&4294967295); //@line 857 "synthesize.cpp"
                        var $div61=((((($mul60))|0)/3000)|0); //@line 857 "synthesize.cpp"
                        $allowed=$div61; //@line 857 "synthesize.cpp"
                        var $56=$allowed; //@line 858 "synthesize.cpp"
                        var $57=$len; //@line 858 "synthesize.cpp"
                        var $mul62=((($57)*($56))&4294967295); //@line 858 "synthesize.cpp"
                        var $div63=((((($mul62))|0)/256)|0); //@line 858 "synthesize.cpp"
                        $allowed=$div63; //@line 858 "synthesize.cpp"
                        var $58=$diff; //@line 860 "synthesize.cpp"
                        var $59=$allowed; //@line 860 "synthesize.cpp"
                        var $cmp64=((($58))|0) > ((($59))|0); //@line 860 "synthesize.cpp"
                        if ($cmp64) { __label__ = 21;; } else { __label__ = 24;; } //@line 860 "synthesize.cpp"
                        if (__label__ == 21) {
  
                          var $60=$modified; //@line 862 "synthesize.cpp"
                          var $cmp66=((($60))|0)==0; //@line 862 "synthesize.cpp"
                          if ($cmp66) { __label__ = 22;; } else { __label__ = 23;; } //@line 862 "synthesize.cpp"
                          if (__label__ == 22) {
  
                            var $61=$frame; //@line 864 "synthesize.cpp"
                            var $call=__ZL9CopyFrameP7frame_ti($61, 0); //@line 864 "synthesize.cpp"
                            $frame2=$call; //@line 864 "synthesize.cpp"
                            $modified=1; //@line 865 "synthesize.cpp"
                            ; //@line 866 "synthesize.cpp"
                          }
  
                          var $62=$pk; //@line 867 "synthesize.cpp"
                          var $63=$frame1; //@line 867 "synthesize.cpp"
                          var $ffreq69=(($63+2)&4294967295); //@line 867 "synthesize.cpp"
                          var $arrayidx70=(($ffreq69+$62*2)&4294967295); //@line 867 "synthesize.cpp"
                          var $64=HEAP16[(($arrayidx70)>>1)]; //@line 867 "synthesize.cpp"
                          var $conv71=(tempInt=(($64)),(tempInt>=32768?tempInt-65536:tempInt)); //@line 867 "synthesize.cpp"
                          var $65=$allowed; //@line 867 "synthesize.cpp"
                          var $add72=((($conv71)+($65))&4294967295); //@line 867 "synthesize.cpp"
                          var $conv73=((($add72)) & 65535); //@line 867 "synthesize.cpp"
                          var $66=$pk; //@line 867 "synthesize.cpp"
                          var $67=$frame2; //@line 867 "synthesize.cpp"
                          var $ffreq74=(($67+2)&4294967295); //@line 867 "synthesize.cpp"
                          var $arrayidx75=(($ffreq74+$66*2)&4294967295); //@line 867 "synthesize.cpp"
                          HEAP16[(($arrayidx75)>>1)]=$conv73; //@line 867 "synthesize.cpp"
                          var $68=$frame2; //@line 868 "synthesize.cpp"
                          var $69=($68); //@line 868 "synthesize.cpp"
                          var $70=$q; //@line 868 "synthesize.cpp"
                          var $arrayidx76=(($70+8)&4294967295); //@line 868 "synthesize.cpp"
                          HEAP32[(($arrayidx76)>>2)]=$69; //@line 868 "synthesize.cpp"
                          ; //@line 869 "synthesize.cpp"
                        }
                        else if (__label__ == 24) {
  
                          var $71=$diff; //@line 871 "synthesize.cpp"
                          var $72=$allowed; //@line 871 "synthesize.cpp"
                          var $sub78=(((-($72)))&4294967295); //@line 871 "synthesize.cpp"
                          var $cmp79=((($71))|0) < ((($sub78))|0); //@line 871 "synthesize.cpp"
                          if (!($cmp79)) { __label__ = 28;break $for_inc$$if_end45$25; } //@line 871 "synthesize.cpp"
  
                          var $73=$modified; //@line 873 "synthesize.cpp"
                          var $cmp81=((($73))|0)==0; //@line 873 "synthesize.cpp"
                          if ($cmp81) { __label__ = 26;; } else { __label__ = 27;; } //@line 873 "synthesize.cpp"
                          if (__label__ == 26) {
  
                            var $74=$frame; //@line 875 "synthesize.cpp"
                            var $call83=__ZL9CopyFrameP7frame_ti($74, 0); //@line 875 "synthesize.cpp"
                            $frame2=$call83; //@line 875 "synthesize.cpp"
                            $modified=1; //@line 876 "synthesize.cpp"
                            ; //@line 877 "synthesize.cpp"
                          }
  
                          var $75=$pk; //@line 878 "synthesize.cpp"
                          var $76=$frame1; //@line 878 "synthesize.cpp"
                          var $ffreq85=(($76+2)&4294967295); //@line 878 "synthesize.cpp"
                          var $arrayidx86=(($ffreq85+$75*2)&4294967295); //@line 878 "synthesize.cpp"
                          var $77=HEAP16[(($arrayidx86)>>1)]; //@line 878 "synthesize.cpp"
                          var $conv87=(tempInt=(($77)),(tempInt>=32768?tempInt-65536:tempInt)); //@line 878 "synthesize.cpp"
                          var $78=$allowed; //@line 878 "synthesize.cpp"
                          var $sub88=((($conv87)-($78))&4294967295); //@line 878 "synthesize.cpp"
                          var $conv89=((($sub88)) & 65535); //@line 878 "synthesize.cpp"
                          var $79=$pk; //@line 878 "synthesize.cpp"
                          var $80=$frame2; //@line 878 "synthesize.cpp"
                          var $ffreq90=(($80+2)&4294967295); //@line 878 "synthesize.cpp"
                          var $arrayidx91=(($ffreq90+$79*2)&4294967295); //@line 878 "synthesize.cpp"
                          HEAP16[(($arrayidx91)>>1)]=$conv89; //@line 878 "synthesize.cpp"
                          var $81=$frame2; //@line 879 "synthesize.cpp"
                          var $82=($81); //@line 879 "synthesize.cpp"
                          var $83=$q; //@line 879 "synthesize.cpp"
                          var $arrayidx92=(($83+8)&4294967295); //@line 879 "synthesize.cpp"
                          HEAP32[(($arrayidx92)>>2)]=$82; //@line 879 "synthesize.cpp"
                          ; //@line 880 "synthesize.cpp"
                        }
                      }
                    } while(0);
  
                    var $84=$pk; //@line 835 "synthesize.cpp"
                    var $inc=((($84)+1)&4294967295); //@line 835 "synthesize.cpp"
                    $pk=$inc; //@line 835 "synthesize.cpp"
                    var $cmp38=((($inc))|0) < 6; //@line 835 "synthesize.cpp"
                    if ($cmp38) { __label__ = 15;continue $for_body$21; } else { __label__ = 29;break $if_then20$$if_end95$14; } //@line 835 "synthesize.cpp"
                  }
                }
              } while(0);
  
              var $85=$ix; //@line 884 "synthesize.cpp"
              var $86=HEAP32[((__ZL14syllable_start)>>2)]; //@line 884 "synthesize.cpp"
              var $cmp96=((($85))|0)==((($86))|0); //@line 884 "synthesize.cpp"
              if ($cmp96) { __label__ = 31;break $for_cond$7; } //@line 884 "synthesize.cpp"
  
              var $87=$ix; //@line 886 "synthesize.cpp"
              var $dec=((($87)-1)&4294967295); //@line 886 "synthesize.cpp"
              $ix=$dec; //@line 886 "synthesize.cpp"
              __lastLabel__ = 30; __label__ = 5;continue $for_cond$7; //@line 887 "synthesize.cpp"
            }
  
            var $88=HEAP32[((__ZL15syllable_centre)>>2)]; //@line 890 "synthesize.cpp"
            $ix=$88; //@line 890 "synthesize.cpp"
            $frame=0; //@line 892 "synthesize.cpp"
            ; //@line 893 "synthesize.cpp"
            $for_cond100$45: while(1) { 
  
              var $89=$ix; //@line 895 "synthesize.cpp"
              var $arrayidx101=((_wcmdq+$89*16)&4294967295); //@line 895 "synthesize.cpp"
              var $arraydecay102=(($arrayidx101)&4294967295); //@line 895 "synthesize.cpp"
              $q=$arraydecay102; //@line 895 "synthesize.cpp"
              var $90=$q; //@line 897 "synthesize.cpp"
              var $arrayidx103=(($90)&4294967295); //@line 897 "synthesize.cpp"
              var $91=HEAP32[(($arrayidx103)>>2)]; //@line 897 "synthesize.cpp"
              var $cmp104=((($91))|0)==5; //@line 897 "synthesize.cpp"
              if ($cmp104) { __label__ = 57;break $for_cond100$45; } //@line 897 "synthesize.cpp"
  
              var $92=$q; //@line 897 "synthesize.cpp"
              var $arrayidx106=(($92)&4294967295); //@line 897 "synthesize.cpp"
              var $93=HEAP32[(($arrayidx106)>>2)]; //@line 897 "synthesize.cpp"
              var $cmp107=((($93))|0)==6; //@line 897 "synthesize.cpp"
              if ($cmp107) { __label__ = 57;break $for_cond100$45; } //@line 897 "synthesize.cpp"
  
              var $94=$q; //@line 900 "synthesize.cpp"
              var $arrayidx110=(($94)&4294967295); //@line 900 "synthesize.cpp"
              var $95=HEAP32[(($arrayidx110)>>2)]; //@line 900 "synthesize.cpp"
              var $cmp111=((($95))|0) <= 4; //@line 900 "synthesize.cpp"
              if ($cmp111) { __label__ = 35;; } else { __label__ = 54;; } //@line 900 "synthesize.cpp"
              $if_then112$$if_end199$49: do { 
                if (__label__ == 35) {
  
                  var $96=$q; //@line 903 "synthesize.cpp"
                  var $arrayidx113=(($96+4)&4294967295); //@line 903 "synthesize.cpp"
                  var $97=HEAP32[(($arrayidx113)>>2)]; //@line 903 "synthesize.cpp"
                  var $and114=($97) & 65535; //@line 903 "synthesize.cpp"
                  $len=$and114; //@line 903 "synthesize.cpp"
                  var $98=$q; //@line 905 "synthesize.cpp"
                  var $arrayidx115=(($98+8)&4294967295); //@line 905 "synthesize.cpp"
                  var $99=HEAP32[(($arrayidx115)>>2)]; //@line 905 "synthesize.cpp"
                  var $100=($99); //@line 905 "synthesize.cpp"
                  $frame1=$100; //@line 905 "synthesize.cpp"
                  var $101=$frame; //@line 906 "synthesize.cpp"
                  var $cmp116=((($101))|0)!=0; //@line 906 "synthesize.cpp"
                  if ($cmp116) { __label__ = 36;; } else { __label__ = 38;; } //@line 906 "synthesize.cpp"
                  if (__label__ == 36) {
  
                    var $102=$frame1; //@line 908 "synthesize.cpp"
                    var $103=$frame; //@line 908 "synthesize.cpp"
                    var $cmp118=((($102))|0)==((($103))|0); //@line 908 "synthesize.cpp"
                    if (!($cmp118)) { __label__ = 57;break $for_cond100$45; } //@line 908 "synthesize.cpp"
  
                    var $104=$frame2; //@line 910 "synthesize.cpp"
                    var $105=($104); //@line 910 "synthesize.cpp"
                    var $106=$q; //@line 910 "synthesize.cpp"
                    var $arrayidx120=(($106+8)&4294967295); //@line 910 "synthesize.cpp"
                    HEAP32[(($arrayidx120)>>2)]=$105; //@line 910 "synthesize.cpp"
                    var $107=$frame2; //@line 911 "synthesize.cpp"
                    $frame1=$107; //@line 911 "synthesize.cpp"
                    ; //@line 915 "synthesize.cpp"
                  }
  
                  var $108=$q; //@line 917 "synthesize.cpp"
                  var $arrayidx124=(($108+12)&4294967295); //@line 917 "synthesize.cpp"
                  var $109=HEAP32[(($arrayidx124)>>2)]; //@line 917 "synthesize.cpp"
                  var $110=($109); //@line 917 "synthesize.cpp"
                  $frame2=$110; //@line 917 "synthesize.cpp"
                  $frame=$110; //@line 917 "synthesize.cpp"
                  $modified=0; //@line 918 "synthesize.cpp"
                  var $111=$frame1; //@line 920 "synthesize.cpp"
                  var $frflags125=(($111)&4294967295); //@line 920 "synthesize.cpp"
                  var $112=HEAP16[(($frflags125)>>1)]; //@line 920 "synthesize.cpp"
                  var $conv126=(tempInt=(($112)),(tempInt>=32768?tempInt-65536:tempInt)); //@line 920 "synthesize.cpp"
                  var $and127=($conv126) & 16; //@line 920 "synthesize.cpp"
                  var $tobool128=((($and127))|0)!=0; //@line 920 "synthesize.cpp"
                  if ($tobool128) { __label__ = 57;break $for_cond100$45; } //@line 920 "synthesize.cpp"
  
                  var $113=$frame1; //@line 923 "synthesize.cpp"
                  var $frflags131=(($113)&4294967295); //@line 923 "synthesize.cpp"
                  var $114=HEAP16[(($frflags131)>>1)]; //@line 923 "synthesize.cpp"
                  var $conv132=(tempInt=(($114)),(tempInt>=32768?tempInt-65536:tempInt)); //@line 923 "synthesize.cpp"
                  var $and133=($conv132) & 32; //@line 923 "synthesize.cpp"
                  var $tobool134=((($and133))|0)!=0; //@line 923 "synthesize.cpp"
                  if ($tobool134) { __label__ = 40;; } else { __label__ = 41;; } //@line 923 "synthesize.cpp"
                  if (__label__ == 40) {
  
                    var $115=$len; //@line 924 "synthesize.cpp"
                    var $mul136=((($115)*6)&4294967295); //@line 924 "synthesize.cpp"
                    var $div137=((((($mul136))|0)/5)|0); //@line 924 "synthesize.cpp"
                    $len=$div137; //@line 924 "synthesize.cpp"
                    ; //@line 924 "synthesize.cpp"
                  }
  
                  $pk=0; //@line 926 "synthesize.cpp"
                  ; //@line 926 "synthesize.cpp"
                  $for_body141$59: while(1) { 
  
                    var $116=$pk; //@line 929 "synthesize.cpp"
                    var $117=$frame1; //@line 929 "synthesize.cpp"
                    var $ffreq144=(($117+2)&4294967295); //@line 929 "synthesize.cpp"
                    var $arrayidx145=(($ffreq144+$116*2)&4294967295); //@line 929 "synthesize.cpp"
                    var $118=HEAP16[(($arrayidx145)>>1)]; //@line 929 "synthesize.cpp"
                    var $conv146=(tempInt=(($118)),(tempInt>=32768?tempInt-65536:tempInt)); //@line 929 "synthesize.cpp"
                    $f1142=$conv146; //@line 929 "synthesize.cpp"
                    var $119=$pk; //@line 930 "synthesize.cpp"
                    var $120=$frame; //@line 930 "synthesize.cpp"
                    var $ffreq147=(($120+2)&4294967295); //@line 930 "synthesize.cpp"
                    var $arrayidx148=(($ffreq147+$119*2)&4294967295); //@line 930 "synthesize.cpp"
                    var $121=HEAP16[(($arrayidx148)>>1)]; //@line 930 "synthesize.cpp"
                    var $conv149=(tempInt=(($121)),(tempInt>=32768?tempInt-65536:tempInt)); //@line 930 "synthesize.cpp"
                    $f2143=$conv149; //@line 930 "synthesize.cpp"
                    var $122=$f2143; //@line 933 "synthesize.cpp"
                    var $123=$f1142; //@line 933 "synthesize.cpp"
                    var $sub150=((($122)-($123))&4294967295); //@line 933 "synthesize.cpp"
                    $diff=$sub150; //@line 933 "synthesize.cpp"
                    var $cmp151=((($sub150))|0) > 0; //@line 933 "synthesize.cpp"
                    var $124=$f1142; //@line 935 "synthesize.cpp"
                    if ($cmp151) { __label__ = 43;; } else { __label__ = 44;; } //@line 933 "synthesize.cpp"
                    if (__label__ == 43) {
  
                      var $mul153=((($124)*2)&4294967295); //@line 935 "synthesize.cpp"
                      var $125=$f2143; //@line 935 "synthesize.cpp"
                      var $add154=((($125)+($mul153))&4294967295); //@line 935 "synthesize.cpp"
                      $allowed=$add154; //@line 935 "synthesize.cpp"
                      ; //@line 936 "synthesize.cpp"
                    }
                    else if (__label__ == 44) {
  
                      var $126=$f2143; //@line 939 "synthesize.cpp"
                      var $mul156=((($126)*2)&4294967295); //@line 939 "synthesize.cpp"
                      var $add157=((($mul156)+($124))&4294967295); //@line 939 "synthesize.cpp"
                      $allowed=$add157; //@line 939 "synthesize.cpp"
                      ;
                    }
  
                    var $127=$allowed; //@line 941 "synthesize.cpp"
                    var $128=$pk; //@line 941 "synthesize.cpp"
                    var $arrayidx159=((_formant_rate+$128*4)&4294967295); //@line 941 "synthesize.cpp"
                    var $129=HEAP32[(($arrayidx159)>>2)]; //@line 941 "synthesize.cpp"
                    var $mul160=((($129)*($127))&4294967295); //@line 941 "synthesize.cpp"
                    var $div161=((((($mul160))|0)/3000)|0); //@line 941 "synthesize.cpp"
                    $allowed=$div161; //@line 941 "synthesize.cpp"
                    var $130=$allowed; //@line 942 "synthesize.cpp"
                    var $131=$len; //@line 942 "synthesize.cpp"
                    var $mul162=((($131)*($130))&4294967295); //@line 942 "synthesize.cpp"
                    var $div163=((((($mul162))|0)/256)|0); //@line 942 "synthesize.cpp"
                    $allowed=$div163; //@line 942 "synthesize.cpp"
                    var $132=$diff; //@line 944 "synthesize.cpp"
                    var $133=$allowed; //@line 944 "synthesize.cpp"
                    var $cmp164=((($132))|0) > ((($133))|0); //@line 944 "synthesize.cpp"
                    if ($cmp164) { __label__ = 46;; } else { __label__ = 49;; } //@line 944 "synthesize.cpp"
                    $if_then165$$if_else178$65: do { 
                      if (__label__ == 46) {
  
                        var $134=$modified; //@line 946 "synthesize.cpp"
                        var $cmp166=((($134))|0)==0; //@line 946 "synthesize.cpp"
                        if ($cmp166) { __label__ = 47;; } else { __label__ = 48;; } //@line 946 "synthesize.cpp"
                        if (__label__ == 47) {
  
                          var $135=$frame; //@line 948 "synthesize.cpp"
                          var $call168=__ZL9CopyFrameP7frame_ti($135, 0); //@line 948 "synthesize.cpp"
                          $frame2=$call168; //@line 948 "synthesize.cpp"
                          $modified=1; //@line 949 "synthesize.cpp"
                          ; //@line 950 "synthesize.cpp"
                        }
  
                        var $136=$pk; //@line 951 "synthesize.cpp"
                        var $137=$frame1; //@line 951 "synthesize.cpp"
                        var $ffreq170=(($137+2)&4294967295); //@line 951 "synthesize.cpp"
                        var $arrayidx171=(($ffreq170+$136*2)&4294967295); //@line 951 "synthesize.cpp"
                        var $138=HEAP16[(($arrayidx171)>>1)]; //@line 951 "synthesize.cpp"
                        var $conv172=(tempInt=(($138)),(tempInt>=32768?tempInt-65536:tempInt)); //@line 951 "synthesize.cpp"
                        var $139=$allowed; //@line 951 "synthesize.cpp"
                        var $add173=((($conv172)+($139))&4294967295); //@line 951 "synthesize.cpp"
                        var $conv174=((($add173)) & 65535); //@line 951 "synthesize.cpp"
                        var $140=$pk; //@line 951 "synthesize.cpp"
                        var $141=$frame2; //@line 951 "synthesize.cpp"
                        var $ffreq175=(($141+2)&4294967295); //@line 951 "synthesize.cpp"
                        var $arrayidx176=(($ffreq175+$140*2)&4294967295); //@line 951 "synthesize.cpp"
                        HEAP16[(($arrayidx176)>>1)]=$conv174; //@line 951 "synthesize.cpp"
                        var $142=$frame2; //@line 952 "synthesize.cpp"
                        var $143=($142); //@line 952 "synthesize.cpp"
                        var $144=$q; //@line 952 "synthesize.cpp"
                        var $arrayidx177=(($144+12)&4294967295); //@line 952 "synthesize.cpp"
                        HEAP32[(($arrayidx177)>>2)]=$143; //@line 952 "synthesize.cpp"
                        ; //@line 953 "synthesize.cpp"
                      }
                      else if (__label__ == 49) {
  
                        var $145=$diff; //@line 955 "synthesize.cpp"
                        var $146=$allowed; //@line 955 "synthesize.cpp"
                        var $sub179=(((-($146)))&4294967295); //@line 955 "synthesize.cpp"
                        var $cmp180=((($145))|0) < ((($sub179))|0); //@line 955 "synthesize.cpp"
                        if (!($cmp180)) { __label__ = 53;break $if_then165$$if_else178$65; } //@line 955 "synthesize.cpp"
  
                        var $147=$modified; //@line 957 "synthesize.cpp"
                        var $cmp182=((($147))|0)==0; //@line 957 "synthesize.cpp"
                        if ($cmp182) { __label__ = 51;; } else { __label__ = 52;; } //@line 957 "synthesize.cpp"
                        if (__label__ == 51) {
  
                          var $148=$frame; //@line 959 "synthesize.cpp"
                          var $call184=__ZL9CopyFrameP7frame_ti($148, 0); //@line 959 "synthesize.cpp"
                          $frame2=$call184; //@line 959 "synthesize.cpp"
                          $modified=1; //@line 960 "synthesize.cpp"
                          ; //@line 961 "synthesize.cpp"
                        }
  
                        var $149=$pk; //@line 962 "synthesize.cpp"
                        var $150=$frame1; //@line 962 "synthesize.cpp"
                        var $ffreq186=(($150+2)&4294967295); //@line 962 "synthesize.cpp"
                        var $arrayidx187=(($ffreq186+$149*2)&4294967295); //@line 962 "synthesize.cpp"
                        var $151=HEAP16[(($arrayidx187)>>1)]; //@line 962 "synthesize.cpp"
                        var $conv188=(tempInt=(($151)),(tempInt>=32768?tempInt-65536:tempInt)); //@line 962 "synthesize.cpp"
                        var $152=$allowed; //@line 962 "synthesize.cpp"
                        var $sub189=((($conv188)-($152))&4294967295); //@line 962 "synthesize.cpp"
                        var $conv190=((($sub189)) & 65535); //@line 962 "synthesize.cpp"
                        var $153=$pk; //@line 962 "synthesize.cpp"
                        var $154=$frame2; //@line 962 "synthesize.cpp"
                        var $ffreq191=(($154+2)&4294967295); //@line 962 "synthesize.cpp"
                        var $arrayidx192=(($ffreq191+$153*2)&4294967295); //@line 962 "synthesize.cpp"
                        HEAP16[(($arrayidx192)>>1)]=$conv190; //@line 962 "synthesize.cpp"
                        var $155=$frame2; //@line 963 "synthesize.cpp"
                        var $156=($155); //@line 963 "synthesize.cpp"
                        var $157=$q; //@line 963 "synthesize.cpp"
                        var $arrayidx193=(($157+12)&4294967295); //@line 963 "synthesize.cpp"
                        HEAP32[(($arrayidx193)>>2)]=$156; //@line 963 "synthesize.cpp"
                        ; //@line 964 "synthesize.cpp"
                      }
                    } while(0);
  
                    var $158=$pk; //@line 926 "synthesize.cpp"
                    var $inc197=((($158)+1)&4294967295); //@line 926 "synthesize.cpp"
                    $pk=$inc197; //@line 926 "synthesize.cpp"
                    var $cmp140=((($inc197))|0) < 6; //@line 926 "synthesize.cpp"
                    if ($cmp140) { __label__ = 42;continue $for_body141$59; } else { __label__ = 54;break $if_then112$$if_end199$49; } //@line 926 "synthesize.cpp"
                  }
                }
              } while(0);
  
              var $159=$ix; //@line 968 "synthesize.cpp"
              var $inc200=((($159)+1)&4294967295); //@line 968 "synthesize.cpp"
              $ix=$inc200; //@line 968 "synthesize.cpp"
              var $cmp201=((($inc200))|0) >= 160; //@line 969 "synthesize.cpp"
              if ($cmp201) { __label__ = 55;; } else { __label__ = 56;; } //@line 969 "synthesize.cpp"
              if (__label__ == 55) {
  
                $ix=0; //@line 969 "synthesize.cpp"
                ; //@line 969 "synthesize.cpp"
              }
  
              var $160=$ix; //@line 970 "synthesize.cpp"
              var $161=HEAP32[((__ZL12syllable_end)>>2)]; //@line 970 "synthesize.cpp"
              var $cmp204=((($160))|0)==((($161))|0); //@line 970 "synthesize.cpp"
              if ($cmp204) { __label__ = 57;break $for_cond100$45; } else { __label__ = 32;continue $for_cond100$45; } //@line 970 "synthesize.cpp"
            }
  
            var $162=HEAP32[((__ZL12syllable_end)>>2)]; //@line 974 "synthesize.cpp"
            HEAP32[((__ZL14syllable_start)>>2)]=$162; //@line 974 "synthesize.cpp"
            __label__ = 58;break $return$$if_end$2; //@line 975 "synthesize.cpp"
          }
        } while(0);
  
        var $5=HEAP32[((__ZL12syllable_end)>>2)]; //@line 795 "synthesize.cpp"
        HEAP32[((__ZL14syllable_start)>>2)]=$5; //@line 795 "synthesize.cpp"
        ; //@line 796 "synthesize.cpp"
      }
    } while(0);
  
    ;
    return; //@line 975 "synthesize.cpp"
    return;
  }
  

  function __Z8DoMarkeriiii($type, $char_posn, $length, $value) {
    ;
    var __label__;
  
    var $type_addr;
    var $char_posn_addr;
    var $length_addr;
    var $value_addr;
    $type_addr=$type;
    $char_posn_addr=$char_posn;
    $length_addr=$length;
    $value_addr=$value;
    var $0=HEAP32[((_wcmdq_tail)>>2)]; //@line 1212 "synthesize.cpp"
    var $arrayidx=((_wcmdq+$0*16)&4294967295); //@line 1212 "synthesize.cpp"
    var $arrayidx1=(($arrayidx)&4294967295); //@line 1212 "synthesize.cpp"
    HEAP32[(($arrayidx1)>>2)]=10; //@line 1212 "synthesize.cpp"
    var $1=$type_addr; //@line 1213 "synthesize.cpp"
    var $2=HEAP32[((_wcmdq_tail)>>2)]; //@line 1213 "synthesize.cpp"
    var $arrayidx2=((_wcmdq+$2*16)&4294967295); //@line 1213 "synthesize.cpp"
    var $arrayidx3=(($arrayidx2+4)&4294967295); //@line 1213 "synthesize.cpp"
    HEAP32[(($arrayidx3)>>2)]=$1; //@line 1213 "synthesize.cpp"
    var $3=$char_posn_addr; //@line 1214 "synthesize.cpp"
    var $and=($3) & 16777215; //@line 1214 "synthesize.cpp"
    var $4=$length_addr; //@line 1214 "synthesize.cpp"
    var $shl=($4) << 24; //@line 1214 "synthesize.cpp"
    var $or=($shl) | ($and); //@line 1214 "synthesize.cpp"
    var $5=HEAP32[((_wcmdq_tail)>>2)]; //@line 1214 "synthesize.cpp"
    var $arrayidx4=((_wcmdq+$5*16)&4294967295); //@line 1214 "synthesize.cpp"
    var $arrayidx5=(($arrayidx4+8)&4294967295); //@line 1214 "synthesize.cpp"
    HEAP32[(($arrayidx5)>>2)]=$or; //@line 1214 "synthesize.cpp"
    var $6=$value_addr; //@line 1215 "synthesize.cpp"
    var $7=HEAP32[((_wcmdq_tail)>>2)]; //@line 1215 "synthesize.cpp"
    var $arrayidx6=((_wcmdq+$7*16)&4294967295); //@line 1215 "synthesize.cpp"
    var $arrayidx7=(($arrayidx6+12)&4294967295); //@line 1215 "synthesize.cpp"
    HEAP32[(($arrayidx7)>>2)]=$6; //@line 1215 "synthesize.cpp"
    __Z8WcmdqIncv(); //@line 1216 "synthesize.cpp"
    ;
    return; //@line 1218 "synthesize.cpp"
    return;
  }
  

  function __Z12DoSonicSpeedi($value) {
    ;
    var __label__;
  
    var $value_addr;
    $value_addr=$value;
    var $0=HEAP32[((_wcmdq_tail)>>2)]; //@line 1224 "synthesize.cpp"
    var $arrayidx=((_wcmdq+$0*16)&4294967295); //@line 1224 "synthesize.cpp"
    var $arrayidx1=(($arrayidx)&4294967295); //@line 1224 "synthesize.cpp"
    HEAP32[(($arrayidx1)>>2)]=15; //@line 1224 "synthesize.cpp"
    var $1=$value_addr; //@line 1225 "synthesize.cpp"
    var $2=HEAP32[((_wcmdq_tail)>>2)]; //@line 1225 "synthesize.cpp"
    var $arrayidx2=((_wcmdq+$2*16)&4294967295); //@line 1225 "synthesize.cpp"
    var $arrayidx3=(($arrayidx2+4)&4294967295); //@line 1225 "synthesize.cpp"
    HEAP32[(($arrayidx3)>>2)]=$1; //@line 1225 "synthesize.cpp"
    __Z8WcmdqIncv(); //@line 1226 "synthesize.cpp"
    ;
    return; //@line 1227 "synthesize.cpp"
    return;
  }
  

  function __Z13DoVoiceChangeP7voice_t($v) {
    ;
    var __label__;
  
    var $v_addr;
    var $v2;
    $v_addr=$v;
    var $call=_malloc(1344); //@line 1235 "synthesize.cpp"
    var $0=$call; //@line 1235 "synthesize.cpp"
    $v2=$0; //@line 1235 "synthesize.cpp"
    var $1=$v2; //@line 1236 "synthesize.cpp"
    var $2=$1; //@line 1236 "synthesize.cpp"
    var $3=$v_addr; //@line 1236 "synthesize.cpp"
    var $4=$3; //@line 1236 "synthesize.cpp"
    var $src$, $dest$, $stop$, $stop4$, $fast$;
    $src$ = $4;
    $dest$ = $2;
    $stop$ = $src$ + 1344;
    if (($dest$%4) == ($src$%4) && 1344 > 8) {
      while ($src$%4 !== 0 && $src$ < $stop$) {
        ; HEAP8[$dest$++] = HEAP8[$src$++];
      }
      $src$ >>= 2;
      $dest$ >>= 2;
      $stop4$ = $stop$ >> 2;
      while ($src$ < $stop4$) {
    ;;;    HEAP32[$dest$++] = HEAP32[$src$++];
      }
      $src$ <<= 2;
      $dest$ <<= 2;
    }while ($src$ < $stop$) {
      ; HEAP8[$dest$++] = HEAP8[$src$++];
    }; //@line 1236 "synthesize.cpp"
    var $5=HEAP32[((_wcmdq_tail)>>2)]; //@line 1237 "synthesize.cpp"
    var $arrayidx=((_wcmdq+$5*16)&4294967295); //@line 1237 "synthesize.cpp"
    var $arrayidx1=(($arrayidx)&4294967295); //@line 1237 "synthesize.cpp"
    HEAP32[(($arrayidx1)>>2)]=11; //@line 1237 "synthesize.cpp"
    var $6=$v2; //@line 1238 "synthesize.cpp"
    var $7=($6); //@line 1238 "synthesize.cpp"
    var $8=HEAP32[((_wcmdq_tail)>>2)]; //@line 1238 "synthesize.cpp"
    var $arrayidx2=((_wcmdq+$8*16)&4294967295); //@line 1238 "synthesize.cpp"
    var $arrayidx3=(($arrayidx2+4)&4294967295); //@line 1238 "synthesize.cpp"
    HEAP32[(($arrayidx3)>>2)]=$7; //@line 1238 "synthesize.cpp"
    __Z8WcmdqIncv(); //@line 1239 "synthesize.cpp"
    ;
    return; //@line 1240 "synthesize.cpp"
    return;
  }
  

  function __Z10DoEmbeddedPii($embix, $sourceix) {
    ;
    var __label__;
  
    var $embix_addr;
    var $sourceix_addr;
    var $word;
    var $value;
    var $command;
    $embix_addr=$embix;
    $sourceix_addr=$sourceix;
    ; //@line 1250 "synthesize.cpp"
    $do_body$2: while(1) { 
  
      var $0=$embix_addr; //@line 1251 "synthesize.cpp"
      var $1=HEAP32[(($0)>>2)]; //@line 1251 "synthesize.cpp"
      var $arrayidx=((_embedded_list+$1*4)&4294967295); //@line 1251 "synthesize.cpp"
      var $2=HEAP32[(($arrayidx)>>2)]; //@line 1251 "synthesize.cpp"
      $word=$2; //@line 1251 "synthesize.cpp"
      var $3=$word; //@line 1252 "synthesize.cpp"
      var $shr=($3) >>> 8; //@line 1252 "synthesize.cpp"
      $value=$shr; //@line 1252 "synthesize.cpp"
      var $4=$word; //@line 1253 "synthesize.cpp"
      var $and=($4) & 127; //@line 1253 "synthesize.cpp"
      $command=$and; //@line 1253 "synthesize.cpp"
      var $cmp=((($and))|0)==0; //@line 1255 "synthesize.cpp"
      if ($cmp) { __label__ = 11;break $do_body$2; } //@line 1255 "synthesize.cpp"
  
      var $5=$embix_addr; //@line 1258 "synthesize.cpp"
      var $6=HEAP32[(($5)>>2)]; //@line 1258 "synthesize.cpp"
      var $inc=((($6)+1)&4294967295); //@line 1258 "synthesize.cpp"
      HEAP32[(($5)>>2)]=$inc; //@line 1258 "synthesize.cpp"
      var $7=$command; //@line 1260 "synthesize.cpp"
      var $and1=($7) & 31; //@line 1260 "synthesize.cpp"
      if ($and1 == 2) {
        __label__ = 3;;
      }
      else if ($and1 == 7) {
        __label__ = 4;;
      }
      else if ($and1 == 10) {
        __label__ = 7;;
      }
      else if ($and1 == 11) {
        __label__ = 8;;
      }
      else {
      __label__ = 9;;
      }
      
      $sw_default$$sw_bb$$sw_bb3$$sw_bb23$$sw_bb26$5: do { 
        if (__label__ == 9) {
  
          __ZL7DoPauseii(10, 0); //@line 1291 "synthesize.cpp"
          var $28=HEAP32[((_wcmdq_tail)>>2)]; //@line 1292 "synthesize.cpp"
          var $arrayidx28=((_wcmdq+$28*16)&4294967295); //@line 1292 "synthesize.cpp"
          var $arrayidx29=(($arrayidx28)&4294967295); //@line 1292 "synthesize.cpp"
          HEAP32[(($arrayidx29)>>2)]=12; //@line 1292 "synthesize.cpp"
          var $29=$command; //@line 1293 "synthesize.cpp"
          var $30=HEAP32[((_wcmdq_tail)>>2)]; //@line 1293 "synthesize.cpp"
          var $arrayidx30=((_wcmdq+$30*16)&4294967295); //@line 1293 "synthesize.cpp"
          var $arrayidx31=(($arrayidx30+4)&4294967295); //@line 1293 "synthesize.cpp"
          HEAP32[(($arrayidx31)>>2)]=$29; //@line 1293 "synthesize.cpp"
          var $31=$value; //@line 1294 "synthesize.cpp"
          var $32=HEAP32[((_wcmdq_tail)>>2)]; //@line 1294 "synthesize.cpp"
          var $arrayidx32=((_wcmdq+$32*16)&4294967295); //@line 1294 "synthesize.cpp"
          var $arrayidx33=(($arrayidx32+8)&4294967295); //@line 1294 "synthesize.cpp"
          HEAP32[(($arrayidx33)>>2)]=$31; //@line 1294 "synthesize.cpp"
          __Z8WcmdqIncv(); //@line 1295 "synthesize.cpp"
          ; //@line 1296 "synthesize.cpp"
        }
        else if (__label__ == 3) {
  
          var $8=$command; //@line 1263 "synthesize.cpp"
          var $and2=($8) & 96; //@line 1263 "synthesize.cpp"
          var $add=((($and2)+8)&4294967295); //@line 1263 "synthesize.cpp"
          var $9=$value; //@line 1263 "synthesize.cpp"
          __Z11SetEmbeddedii($add, $9); //@line 1263 "synthesize.cpp"
          __Z8SetSpeedi(2); //@line 1264 "synthesize.cpp"
          ; //@line 1265 "synthesize.cpp"
        }
        else if (__label__ == 4) {
  
          var $10=$value; //@line 1268 "synthesize.cpp"
          var $11=HEAP32[((_n_soundicon_tab)>>2)]; //@line 1268 "synthesize.cpp"
          var $cmp4=((($10))|0) < ((($11))|0); //@line 1268 "synthesize.cpp"
          if (!($cmp4)) { __label__ = 10;break $sw_default$$sw_bb$$sw_bb3$$sw_bb23$$sw_bb26$5; } //@line 1268 "synthesize.cpp"
  
          var $12=$value; //@line 1270 "synthesize.cpp"
          var $arrayidx6=((_soundicon_tab+$12*16)&4294967295); //@line 1270 "synthesize.cpp"
          var $length=(($arrayidx6+4)&4294967295); //@line 1270 "synthesize.cpp"
          var $13=HEAP32[(($length)>>2)]; //@line 1270 "synthesize.cpp"
          var $cmp7=((($13))|0)!=0; //@line 1270 "synthesize.cpp"
          if (!($cmp7)) { __label__ = 10;break $sw_default$$sw_bb$$sw_bb3$$sw_bb23$$sw_bb26$5; } //@line 1270 "synthesize.cpp"
  
          __ZL7DoPauseii(10, 0); //@line 1272 "synthesize.cpp"
          var $14=HEAP32[((_wcmdq_tail)>>2)]; //@line 1273 "synthesize.cpp"
          var $arrayidx9=((_wcmdq+$14*16)&4294967295); //@line 1273 "synthesize.cpp"
          var $arrayidx10=(($arrayidx9)&4294967295); //@line 1273 "synthesize.cpp"
          HEAP32[(($arrayidx10)>>2)]=6; //@line 1273 "synthesize.cpp"
          var $15=$value; //@line 1274 "synthesize.cpp"
          var $arrayidx11=((_soundicon_tab+$15*16)&4294967295); //@line 1274 "synthesize.cpp"
          var $length12=(($arrayidx11+4)&4294967295); //@line 1274 "synthesize.cpp"
          var $16=HEAP32[(($length12)>>2)]; //@line 1274 "synthesize.cpp"
          var $17=HEAP32[((_wcmdq_tail)>>2)]; //@line 1274 "synthesize.cpp"
          var $arrayidx13=((_wcmdq+$17*16)&4294967295); //@line 1274 "synthesize.cpp"
          var $arrayidx14=(($arrayidx13+4)&4294967295); //@line 1274 "synthesize.cpp"
          HEAP32[(($arrayidx14)>>2)]=$16; //@line 1274 "synthesize.cpp"
          var $18=$value; //@line 1275 "synthesize.cpp"
          var $arrayidx15=((_soundicon_tab+$18*16)&4294967295); //@line 1275 "synthesize.cpp"
          var $data=(($arrayidx15+8)&4294967295); //@line 1275 "synthesize.cpp"
          var $19=HEAP32[(($data)>>2)]; //@line 1275 "synthesize.cpp"
          var $20=($19); //@line 1275 "synthesize.cpp"
          var $add16=((($20)+44)&4294967295); //@line 1275 "synthesize.cpp"
          var $21=HEAP32[((_wcmdq_tail)>>2)]; //@line 1275 "synthesize.cpp"
          var $arrayidx17=((_wcmdq+$21*16)&4294967295); //@line 1275 "synthesize.cpp"
          var $arrayidx18=(($arrayidx17+8)&4294967295); //@line 1275 "synthesize.cpp"
          HEAP32[(($arrayidx18)>>2)]=$add16; //@line 1275 "synthesize.cpp"
          var $22=HEAP32[((_wcmdq_tail)>>2)]; //@line 1276 "synthesize.cpp"
          var $arrayidx19=((_wcmdq+$22*16)&4294967295); //@line 1276 "synthesize.cpp"
          var $arrayidx20=(($arrayidx19+12)&4294967295); //@line 1276 "synthesize.cpp"
          HEAP32[(($arrayidx20)>>2)]=5376; //@line 1276 "synthesize.cpp"
          __Z8WcmdqIncv(); //@line 1277 "synthesize.cpp"
          ; //@line 1278 "synthesize.cpp"
        }
        else if (__label__ == 7) {
  
          var $23=$sourceix_addr; //@line 1283 "synthesize.cpp"
          var $and24=($23) & 2047; //@line 1283 "synthesize.cpp"
          var $24=HEAP32[((_clause_start_char)>>2)]; //@line 1283 "synthesize.cpp"
          var $add25=((($and24)+($24))&4294967295); //@line 1283 "synthesize.cpp"
          var $25=$value; //@line 1283 "synthesize.cpp"
          __Z8DoMarkeriiii(3, $add25, 0, $25); //@line 1283 "synthesize.cpp"
          ; //@line 1284 "synthesize.cpp"
        }
        else if (__label__ == 8) {
  
          var $26=HEAP32[((_count_characters)>>2)]; //@line 1287 "synthesize.cpp"
          var $add27=((($26)+1)&4294967295); //@line 1287 "synthesize.cpp"
          var $27=$value; //@line 1287 "synthesize.cpp"
          __Z8DoMarkeriiii(4, $add27, 0, $27); //@line 1287 "synthesize.cpp"
          ; //@line 1288 "synthesize.cpp"
        }
      } while(0);
  
      var $33=$word; //@line 1298 "synthesize.cpp"
      var $and34=($33) & 128; //@line 1298 "synthesize.cpp"
      var $cmp35=((($and34))|0)==0; //@line 1298 "synthesize.cpp"
      if ($cmp35) { __label__ = 1;continue $do_body$2; } else { __label__ = 11;break $do_body$2; } //@line 1298 "synthesize.cpp"
    }
  
    ;
    return; //@line 1299 "synthesize.cpp"
    return;
  }
  

  function __Z8GenerateP12PHONEME_LISTPii($phoneme_list, $n_ph, $resume) {
    var __stackBase__  = STACKTOP; STACKTOP += 656; _memset(__stackBase__, 0, 656);
    var __label__;
  
    var $retval;
    var $phoneme_list_addr;
    var $n_ph_addr;
    var $resume_addr;
    var $prev;
    var $next;
    var $next2;
    var $p;
    var $released;
    var $stress;
    var $modulation;
    var $pre_voiced;
    var $free_min;
    var $pitch_env;
    var $amp_env;
    var $ph;
    var $prev_ph;
    var $phdata=__stackBase__;
    var $phdata_prev=__stackBase__+152;
    var $phdata_next=__stackBase__+304;
    var $phdata_tone=__stackBase__+456;
    var $fmtp=__stackBase__+608;
    $phoneme_list_addr=$phoneme_list;
    $n_ph_addr=$n_ph;
    $resume_addr=$resume;
    $pitch_env=0; //@line 1317 "synthesize.cpp"
    var $0=HEAP32[((_option_quiet)>>2)]; //@line 1329 "synthesize.cpp"
    var $tobool=((($0))|0)!=0; //@line 1329 "synthesize.cpp"
    if ($tobool) { __label__ = 1;; } else { __label__ = 2;; } //@line 1329 "synthesize.cpp"
    if (__label__ == 1) {
  
      $retval=0; //@line 1330 "synthesize.cpp"
      ; //@line 1330 "synthesize.cpp"
    }
    else if (__label__ == 2) {
  
      var $1=HEAP8[(((_mbrola_name)&4294967295))]; //@line 1332 "synthesize.cpp"
      var $conv=(tempInt=(($1)),(tempInt>=128?tempInt-256:tempInt)); //@line 1332 "synthesize.cpp"
      var $cmp=((($conv))|0)!=0; //@line 1332 "synthesize.cpp"
      if ($cmp) { __label__ = 3;; } else { __label__ = 4;; } //@line 1332 "synthesize.cpp"
      if (__label__ == 3) {
  
        var $2=$phoneme_list_addr; //@line 1333 "synthesize.cpp"
        var $3=$n_ph_addr; //@line 1333 "synthesize.cpp"
        var $4=$resume_addr; //@line 1333 "synthesize.cpp"
        var $call=__Z14MbrolaGenerateP12PHONEME_LISTPii($2, $3, $4); //@line 1333 "synthesize.cpp"
        $retval=$call; //@line 1333 "synthesize.cpp"
        ; //@line 1333 "synthesize.cpp"
      }
      else if (__label__ == 4) {
  
        var $5=$resume_addr; //@line 1335 "synthesize.cpp"
        var $cmp3=((($5))|0)==0; //@line 1335 "synthesize.cpp"
        if ($cmp3) { __label__ = 5;; } else { __label__ = 6;; } //@line 1335 "synthesize.cpp"
        if (__label__ == 5) {
  
          HEAP32[((__ZZ8GenerateP12PHONEME_LISTPiiE2ix)>>2)]=1; //@line 1337 "synthesize.cpp"
          HEAP32[((__ZZ8GenerateP12PHONEME_LISTPiiE11embedded_ix)>>2)]=0; //@line 1338 "synthesize.cpp"
          HEAP32[((__ZZ8GenerateP12PHONEME_LISTPiiE10word_count)>>2)]=0; //@line 1339 "synthesize.cpp"
          HEAP32[((__ZL12pitch_length)>>2)]=0; //@line 1340 "synthesize.cpp"
          HEAP32[((__ZL10amp_length)>>2)]=0; //@line 1341 "synthesize.cpp"
          HEAP32[((__ZL10last_frame)>>2)]=0; //@line 1342 "synthesize.cpp"
          HEAP32[((__ZL10last_wcmdq)>>2)]=-1; //@line 1343 "synthesize.cpp"
          var $6=HEAP32[((_wcmdq_tail)>>2)]; //@line 1344 "synthesize.cpp"
          HEAP32[((__ZL14syllable_start)>>2)]=$6; //@line 1344 "synthesize.cpp"
          var $7=HEAP32[((_wcmdq_tail)>>2)]; //@line 1345 "synthesize.cpp"
          HEAP32[((__ZL12syllable_end)>>2)]=$7; //@line 1345 "synthesize.cpp"
          HEAP32[((__ZL15syllable_centre)>>2)]=-1; //@line 1346 "synthesize.cpp"
          HEAP32[((__ZL14last_pitch_cmd)>>2)]=-1; //@line 1347 "synthesize.cpp"
          var $dest$, $stop$, $stop4$, $fast$, $value4$;
          $dest$ = _vowel_transition;
          $stop$ = $dest$ + 16;
          $value4$ = 0;
          if ($value4$ < 0) $value4$ += 256;
          $value4$ = $value4$ + ($value4$<<8) + ($value4$<<16) + ($value4$*16777216);
          while ($dest$%4 !== 0 && $dest$ < $stop$) {
            ;; HEAP8[$dest$++] = 0;
          }
          $dest$ >>= 2;
          $stop4$ = $stop$ >> 2;
          while ($dest$ < $stop4$) {
          ;;;;;;;  HEAP32[$dest$++] = $value4$;
          }
          $dest$ <<= 2;
          while ($dest$ < $stop$) {
            ;; HEAP8[$dest$++] = 0;
          }; //@line 1348 "synthesize.cpp"
          __ZL7DoPauseii(0, 0); //@line 1349 "synthesize.cpp"
          ; //@line 1350 "synthesize.cpp"
        }
  
        var $pd_control=(($phdata)&4294967295); //@line 1432 "synthesize.cpp"
        var $8=$fmtp; //@line 1448 "synthesize.cpp"
        var $fmt_control=(($fmtp)&4294967295); //@line 1449 "synthesize.cpp"
        var $sound_addr=(($phdata+68)&4294967295); //@line 1478 "synthesize.cpp"
        var $arrayidx173=(($sound_addr)&4294967295); //@line 1478 "synthesize.cpp"
        var $fmt_addr=(($fmtp+8)&4294967295); //@line 1478 "synthesize.cpp"
        var $sound_param=(($phdata+88)&4294967295); //@line 1479 "synthesize.cpp"
        var $arrayidx174=(($sound_param)&4294967295); //@line 1479 "synthesize.cpp"
        var $fmt_amp=(($fmtp+16)&4294967295); //@line 1479 "synthesize.cpp"
        var $sound_addr199=(($phdata+68)&4294967295); //@line 1506 "synthesize.cpp"
        var $arrayidx200=(($sound_addr199)&4294967295); //@line 1506 "synthesize.cpp"
        var $fmt_addr201=(($fmtp+8)&4294967295); //@line 1506 "synthesize.cpp"
        var $sound_param202=(($phdata+88)&4294967295); //@line 1507 "synthesize.cpp"
        var $arrayidx203=(($sound_param202)&4294967295); //@line 1507 "synthesize.cpp"
        var $fmt_amp204=(($fmtp+16)&4294967295); //@line 1507 "synthesize.cpp"
        var $sound_addr205=(($phdata+68)&4294967295); //@line 1508 "synthesize.cpp"
        var $arrayidx206=(($sound_addr205+16)&4294967295); //@line 1508 "synthesize.cpp"
        var $wav_addr=(($fmtp+28)&4294967295); //@line 1508 "synthesize.cpp"
        var $sound_param207=(($phdata+88)&4294967295); //@line 1509 "synthesize.cpp"
        var $arrayidx208=(($sound_param207+16)&4294967295); //@line 1509 "synthesize.cpp"
        var $wav_amp=(($fmtp+32)&4294967295); //@line 1509 "synthesize.cpp"
        var $9=$fmtp; //@line 1551 "synthesize.cpp"
        var $pd_param=(($phdata+4)&4294967295); //@line 1552 "synthesize.cpp"
        var $arrayidx290=(($pd_param+40)&4294967295); //@line 1552 "synthesize.cpp"
        var $std_length=(($fmtp+44)&4294967295); //@line 1552 "synthesize.cpp"
        var $sound_addr291=(($phdata+68)&4294967295); //@line 1553 "synthesize.cpp"
        var $arrayidx292=(($sound_addr291)&4294967295); //@line 1553 "synthesize.cpp"
        var $fmt_addr293=(($fmtp+8)&4294967295); //@line 1553 "synthesize.cpp"
        var $sound_param294=(($phdata+88)&4294967295); //@line 1554 "synthesize.cpp"
        var $arrayidx295=(($sound_param294)&4294967295); //@line 1554 "synthesize.cpp"
        var $fmt_amp296=(($fmtp+16)&4294967295); //@line 1554 "synthesize.cpp"
        var $sound_addr297=(($phdata+68)&4294967295); //@line 1555 "synthesize.cpp"
        var $arrayidx298=(($sound_addr297+16)&4294967295); //@line 1555 "synthesize.cpp"
        var $wav_addr299=(($fmtp+28)&4294967295); //@line 1555 "synthesize.cpp"
        var $sound_param300=(($phdata+88)&4294967295); //@line 1556 "synthesize.cpp"
        var $arrayidx301=(($sound_param300+16)&4294967295); //@line 1556 "synthesize.cpp"
        var $wav_amp302=(($fmtp+32)&4294967295); //@line 1556 "synthesize.cpp"
        var $10=$fmtp; //@line 1564 "synthesize.cpp"
        var $pd_param334=(($phdata+4)&4294967295); //@line 1577 "synthesize.cpp"
        var $arrayidx335=(($pd_param334+40)&4294967295); //@line 1577 "synthesize.cpp"
        var $std_length337=(($fmtp+44)&4294967295); //@line 1577 "synthesize.cpp"
        var $sound_addr338=(($phdata+68)&4294967295); //@line 1578 "synthesize.cpp"
        var $arrayidx339=(($sound_addr338)&4294967295); //@line 1578 "synthesize.cpp"
        var $fmt_addr340=(($fmtp+8)&4294967295); //@line 1578 "synthesize.cpp"
        var $sound_param341=(($phdata+88)&4294967295); //@line 1579 "synthesize.cpp"
        var $arrayidx342=(($sound_param341)&4294967295); //@line 1579 "synthesize.cpp"
        var $fmt_amp343=(($fmtp+16)&4294967295); //@line 1579 "synthesize.cpp"
        var $11=$fmtp; //@line 1601 "synthesize.cpp"
        var $pd_param400=(($phdata+4)&4294967295); //@line 1626 "synthesize.cpp"
        var $arrayidx401=(($pd_param400+40)&4294967295); //@line 1626 "synthesize.cpp"
        var $std_length403=(($fmtp+44)&4294967295); //@line 1626 "synthesize.cpp"
        var $sound_addr404=(($phdata+68)&4294967295); //@line 1627 "synthesize.cpp"
        var $arrayidx405=(($sound_addr404)&4294967295); //@line 1627 "synthesize.cpp"
        var $fmt_addr406=(($fmtp+8)&4294967295); //@line 1627 "synthesize.cpp"
        var $sound_param407=(($phdata+88)&4294967295); //@line 1628 "synthesize.cpp"
        var $arrayidx408=(($sound_param407)&4294967295); //@line 1628 "synthesize.cpp"
        var $fmt_amp409=(($fmtp+16)&4294967295); //@line 1628 "synthesize.cpp"
        var $sound_addr410=(($phdata+68)&4294967295); //@line 1629 "synthesize.cpp"
        var $arrayidx411=(($sound_addr410+16)&4294967295); //@line 1629 "synthesize.cpp"
        var $wav_addr412=(($fmtp+28)&4294967295); //@line 1629 "synthesize.cpp"
        var $sound_param413=(($phdata+88)&4294967295); //@line 1630 "synthesize.cpp"
        var $arrayidx414=(($sound_param413+16)&4294967295); //@line 1630 "synthesize.cpp"
        var $wav_amp415=(($fmtp+32)&4294967295); //@line 1630 "synthesize.cpp"
        var $12=$fmtp; //@line 1639 "synthesize.cpp"
        var $pd_param422=(($phdata+4)&4294967295); //@line 1642 "synthesize.cpp"
        var $arrayidx423=(($pd_param422+40)&4294967295); //@line 1642 "synthesize.cpp"
        var $std_length425=(($fmtp+44)&4294967295); //@line 1642 "synthesize.cpp"
        var $sound_addr426=(($phdata+68)&4294967295); //@line 1644 "synthesize.cpp"
        var $arrayidx427=(($sound_addr426+8)&4294967295); //@line 1644 "synthesize.cpp"
        var $fmt_addr428=(($fmtp+8)&4294967295); //@line 1644 "synthesize.cpp"
        var $pd_control431=(($phdata)&4294967295); //@line 1644 "synthesize.cpp"
        var $sound_param435=(($phdata+88)&4294967295); //@line 1647 "synthesize.cpp"
        var $arrayidx436=(($sound_param435+8)&4294967295); //@line 1647 "synthesize.cpp"
        var $fmt_length=(($fmtp+12)&4294967295); //@line 1647 "synthesize.cpp"
        var $fmt_addr455=(($fmtp+8)&4294967295); //@line 1663 "synthesize.cpp"
        var $use_vowelin=(($fmtp+4)&4294967295); //@line 1666 "synthesize.cpp"
        var $fmt_control458=(($fmtp)&4294967295); //@line 1667 "synthesize.cpp"
        var $sound_addr459=(($phdata+68)&4294967295); //@line 1668 "synthesize.cpp"
        var $arrayidx460=(($sound_addr459)&4294967295); //@line 1668 "synthesize.cpp"
        var $fmt_addr461=(($fmtp+8)&4294967295); //@line 1668 "synthesize.cpp"
        var $sound_param463=(($phdata+88)&4294967295); //@line 1671 "synthesize.cpp"
        var $arrayidx464=(($sound_param463)&4294967295); //@line 1671 "synthesize.cpp"
        var $fmt_amp465=(($fmtp+16)&4294967295); //@line 1671 "synthesize.cpp"
        var $pitch_env474=(($phdata_tone+124)&4294967295); //@line 1678 "synthesize.cpp"
        var $amp_env476=(($phdata_tone+128)&4294967295); //@line 1679 "synthesize.cpp"
        var $amp_env479=(($phdata_tone+128)&4294967295); //@line 1680 "synthesize.cpp"
        var $sound_addr542=(($phdata+68)&4294967295); //@line 1721 "synthesize.cpp"
        var $arrayidx543=(($sound_addr542)&4294967295); //@line 1721 "synthesize.cpp"
        var $fmt_addr544=(($fmtp+8)&4294967295); //@line 1721 "synthesize.cpp"
        var $sound_param545=(($phdata+88)&4294967295); //@line 1722 "synthesize.cpp"
        var $arrayidx546=(($sound_param545)&4294967295); //@line 1722 "synthesize.cpp"
        var $fmt_amp547=(($fmtp+16)&4294967295); //@line 1722 "synthesize.cpp"
        var $transition0548=(($fmtp+36)&4294967295); //@line 1723 "synthesize.cpp"
        var $transition1549=(($fmtp+40)&4294967295); //@line 1724 "synthesize.cpp"
        var $sound_addr550=(($phdata+68)&4294967295); //@line 1726 "synthesize.cpp"
        var $arrayidx551=(($sound_addr550+12)&4294967295); //@line 1726 "synthesize.cpp"
        var $fmt2_addr=(($fmtp+20)&4294967295); //@line 1726 "synthesize.cpp"
        var $sound_param554=(($phdata+88)&4294967295); //@line 1728 "synthesize.cpp"
        var $arrayidx555=(($sound_param554+12)&4294967295); //@line 1728 "synthesize.cpp"
        var $fmt2_lenadj556=(($fmtp+24)&4294967295); //@line 1728 "synthesize.cpp"
        var $fmt2_lenadj562=(($fmtp+24)&4294967295); //@line 1733 "synthesize.cpp"
        var $use_vowelin563=(($fmtp+4)&4294967295); //@line 1736 "synthesize.cpp"
        var $vowel_transition564=(($phdata_next+108)&4294967295); //@line 1737 "synthesize.cpp"
        var $arrayidx565=(($vowel_transition564+8)&4294967295); //@line 1737 "synthesize.cpp"
        var $transition0566=(($fmtp+36)&4294967295); //@line 1737 "synthesize.cpp"
        var $vowel_transition567=(($phdata_next+108)&4294967295); //@line 1738 "synthesize.cpp"
        var $arrayidx568=(($vowel_transition567+12)&4294967295); //@line 1738 "synthesize.cpp"
        var $transition1569=(($fmtp+40)&4294967295); //@line 1738 "synthesize.cpp"
        var $sound_addr570=(($phdata_next+68)&4294967295); //@line 1740 "synthesize.cpp"
        var $arrayidx571=(($sound_addr570+12)&4294967295); //@line 1740 "synthesize.cpp"
        var $fmt2_addr572=(($fmtp+20)&4294967295); //@line 1740 "synthesize.cpp"
        var $sound_param575=(($phdata_next+88)&4294967295); //@line 1742 "synthesize.cpp"
        var $arrayidx576=(($sound_param575+12)&4294967295); //@line 1742 "synthesize.cpp"
        var $fmt2_lenadj577=(($fmtp+24)&4294967295); //@line 1742 "synthesize.cpp"
        var $sound_addr442=(($phdata_prev+68)&4294967295); //@line 1654 "synthesize.cpp"
        var $arrayidx443=(($sound_addr442+8)&4294967295); //@line 1654 "synthesize.cpp"
        var $fmt_addr444=(($fmtp+8)&4294967295); //@line 1654 "synthesize.cpp"
        var $sound_param447=(($phdata_prev+88)&4294967295); //@line 1657 "synthesize.cpp"
        var $arrayidx448=(($sound_param447+8)&4294967295); //@line 1657 "synthesize.cpp"
        var $fmt2_lenadj=(($fmtp+24)&4294967295); //@line 1657 "synthesize.cpp"
        var $vowel_transition=(($phdata_prev+108)&4294967295); //@line 1659 "synthesize.cpp"
        var $arrayidx450=(($vowel_transition)&4294967295); //@line 1659 "synthesize.cpp"
        var $transition0=(($fmtp+36)&4294967295); //@line 1659 "synthesize.cpp"
        var $vowel_transition451=(($phdata_prev+108)&4294967295); //@line 1660 "synthesize.cpp"
        var $arrayidx452=(($vowel_transition451+4)&4294967295); //@line 1660 "synthesize.cpp"
        var $transition1=(($fmtp+40)&4294967295); //@line 1660 "synthesize.cpp"
        ; //@line 1352 "synthesize.cpp"
        $while_cond$11: while(1) { 
  
          var $13=HEAP32[((__ZZ8GenerateP12PHONEME_LISTPiiE2ix)>>2)]; //@line 1352 "synthesize.cpp"
          var $14=$n_ph_addr; //@line 1352 "synthesize.cpp"
          var $15=HEAP32[(($14)>>2)]; //@line 1352 "synthesize.cpp"
          var $cmp6=((($13))|0) < ((($15))|0); //@line 1352 "synthesize.cpp"
          if (!($cmp6)) { __label__ = 134;break $while_cond$11; } //@line 1352 "synthesize.cpp"
  
          var $16=HEAP32[((__ZZ8GenerateP12PHONEME_LISTPiiE2ix)>>2)]; //@line 1354 "synthesize.cpp"
          var $17=$phoneme_list_addr; //@line 1354 "synthesize.cpp"
          var $arrayidx=(($17+24*$16)&4294967295); //@line 1354 "synthesize.cpp"
          $p=$arrayidx; //@line 1354 "synthesize.cpp"
          var $18=$p; //@line 1356 "synthesize.cpp"
          var $type=(($18+15)&4294967295); //@line 1356 "synthesize.cpp"
          var $19=HEAPU8[($type)]; //@line 1356 "synthesize.cpp"
          var $conv7=((($19))&255); //@line 1356 "synthesize.cpp"
          var $cmp8=((($conv7))|0)==0; //@line 1356 "synthesize.cpp"
          if ($cmp8) { __label__ = 9;; } else { __label__ = 10;; } //@line 1356 "synthesize.cpp"
          if (__label__ == 9) {
  
            $free_min=5; //@line 1357 "synthesize.cpp"
            ; //@line 1357 "synthesize.cpp"
          }
          else if (__label__ == 10) {
  
            var $20=$p; //@line 1359 "synthesize.cpp"
            var $type10=(($20+15)&4294967295); //@line 1359 "synthesize.cpp"
            var $21=HEAPU8[($type10)]; //@line 1359 "synthesize.cpp"
            var $conv11=((($21))&255); //@line 1359 "synthesize.cpp"
            var $cmp12=((($conv11))|0)!=2; //@line 1359 "synthesize.cpp"
            if ($cmp12) { __label__ = 11;; } else { __label__ = 12;; } //@line 1359 "synthesize.cpp"
            if (__label__ == 11) {
  
              $free_min=10; //@line 1360 "synthesize.cpp"
              ; //@line 1360 "synthesize.cpp"
            }
            else if (__label__ == 12) {
  
              $free_min=22; //@line 1362 "synthesize.cpp"
              ;
            }
          }
  
          var $call17=__Z9WcmdqFreev(); //@line 1364 "synthesize.cpp"
          var $22=$free_min; //@line 1364 "synthesize.cpp"
          var $cmp18=((($call17))|0) <= ((($22))|0); //@line 1364 "synthesize.cpp"
          if ($cmp18) { __label__ = 14;break $while_cond$11; } //@line 1364 "synthesize.cpp"
  
          var $23=HEAP32[((__ZZ8GenerateP12PHONEME_LISTPiiE2ix)>>2)]; //@line 1367 "synthesize.cpp"
          var $sub=((($23)-1)&4294967295); //@line 1367 "synthesize.cpp"
          var $24=$phoneme_list_addr; //@line 1367 "synthesize.cpp"
          var $arrayidx21=(($24+24*$sub)&4294967295); //@line 1367 "synthesize.cpp"
          $prev=$arrayidx21; //@line 1367 "synthesize.cpp"
          var $25=HEAP32[((__ZZ8GenerateP12PHONEME_LISTPiiE2ix)>>2)]; //@line 1368 "synthesize.cpp"
          var $add=((($25)+1)&4294967295); //@line 1368 "synthesize.cpp"
          var $26=$phoneme_list_addr; //@line 1368 "synthesize.cpp"
          var $arrayidx22=(($26+24*$add)&4294967295); //@line 1368 "synthesize.cpp"
          $next=$arrayidx22; //@line 1368 "synthesize.cpp"
          var $27=HEAP32[((__ZZ8GenerateP12PHONEME_LISTPiiE2ix)>>2)]; //@line 1369 "synthesize.cpp"
          var $add23=((($27)+2)&4294967295); //@line 1369 "synthesize.cpp"
          var $28=$phoneme_list_addr; //@line 1369 "synthesize.cpp"
          var $arrayidx24=(($28+24*$add23)&4294967295); //@line 1369 "synthesize.cpp"
          $next2=$arrayidx24; //@line 1369 "synthesize.cpp"
          var $29=$p; //@line 1371 "synthesize.cpp"
          var $synthflags=(($29+4)&4294967295); //@line 1371 "synthesize.cpp"
          var $30=HEAPU16[(($synthflags)>>1)]; //@line 1371 "synthesize.cpp"
          var $conv25=((($30))&65535); //@line 1371 "synthesize.cpp"
          var $and=($conv25) & 2; //@line 1371 "synthesize.cpp"
          var $tobool26=((($and))|0)!=0; //@line 1371 "synthesize.cpp"
          if ($tobool26) { __label__ = 16;; } else { __label__ = 17;; } //@line 1371 "synthesize.cpp"
          if (__label__ == 16) {
  
            var $31=$p; //@line 1373 "synthesize.cpp"
            var $sourceix=(($31+6)&4294967295); //@line 1373 "synthesize.cpp"
            var $32=HEAPU16[(($sourceix)>>1)]; //@line 1373 "synthesize.cpp"
            var $conv28=((($32))&65535); //@line 1373 "synthesize.cpp"
            __Z10DoEmbeddedPii(__ZZ8GenerateP12PHONEME_LISTPiiE11embedded_ix, $conv28); //@line 1373 "synthesize.cpp"
            ; //@line 1374 "synthesize.cpp"
          }
  
          var $33=$p; //@line 1376 "synthesize.cpp"
          var $newword=(($33+19)&4294967295); //@line 1376 "synthesize.cpp"
          var $34=HEAP8[($newword)]; //@line 1376 "synthesize.cpp"
          var $tobool30=(tempInt=(($34)),(tempInt>=128?tempInt-256:tempInt))!=0; //@line 1376 "synthesize.cpp"
          if ($tobool30) { __label__ = 18;; } else { __label__ = 26;; } //@line 1376 "synthesize.cpp"
          $if_then31$$if_end63$25: do { 
            if (__label__ == 18) {
  
              var $35=$p; //@line 1378 "synthesize.cpp"
              var $type32=(($35+15)&4294967295); //@line 1378 "synthesize.cpp"
              var $36=HEAPU8[($type32)]; //@line 1378 "synthesize.cpp"
              var $conv33=((($36))&255); //@line 1378 "synthesize.cpp"
              var $cmp34=((($conv33))|0)==2; //@line 1378 "synthesize.cpp"
              if ($cmp34) { __label__ = 19;; } else { __label__ = 20;; } //@line 1378 "synthesize.cpp"
              $land_lhs_true$$lor_lhs_false$27: do { 
                if (__label__ == 19) {
  
                  var $37=HEAP32[((_translator)>>2)]; //@line 1378 "synthesize.cpp"
                  var $langopts=(($37)&4294967295); //@line 1378 "synthesize.cpp"
                  var $param=(($langopts+24)&4294967295); //@line 1378 "synthesize.cpp"
                  var $arrayidx35=(($param+32)&4294967295); //@line 1378 "synthesize.cpp"
                  var $38=HEAP32[(($arrayidx35)>>2)]; //@line 1378 "synthesize.cpp"
                  var $and36=($38) & 1; //@line 1378 "synthesize.cpp"
                  var $tobool37=((($and36))|0)!=0; //@line 1378 "synthesize.cpp"
                  if ($tobool37) { __label__ = 22;break $land_lhs_true$$lor_lhs_false$27; } else { __label__ = 20;break $land_lhs_true$$lor_lhs_false$27; } //@line 1378 "synthesize.cpp"
                }
              } while(0);
              $if_end43$$lor_lhs_false$29: do { 
                if (__label__ == 20) {
  
                  var $39=$p; //@line 1378 "synthesize.cpp"
                  var $ph38=(($39+8)&4294967295); //@line 1378 "synthesize.cpp"
                  var $40=HEAP32[(($ph38)>>2)]; //@line 1378 "synthesize.cpp"
                  var $phflags=(($40+4)&4294967295); //@line 1378 "synthesize.cpp"
                  var $41=HEAP32[(($phflags)>>2)]; //@line 1378 "synthesize.cpp"
                  var $and39=($41) & 16777216; //@line 1378 "synthesize.cpp"
                  var $tobool40=((($and39))|0)!=0; //@line 1378 "synthesize.cpp"
                  if ($tobool40) { __label__ = 22;break $if_end43$$lor_lhs_false$29; } //@line 1378 "synthesize.cpp"
  
                  HEAP32[((__ZL10last_frame)>>2)]=0; //@line 1384 "synthesize.cpp"
                  ;
                }
              } while(0);
  
              var $42=$p; //@line 1387 "synthesize.cpp"
              var $sourceix44=(($42+6)&4294967295); //@line 1387 "synthesize.cpp"
              var $43=HEAPU16[(($sourceix44)>>1)]; //@line 1387 "synthesize.cpp"
              var $conv45=((($43))&65535); //@line 1387 "synthesize.cpp"
              var $and46=($conv45) & 2047; //@line 1387 "synthesize.cpp"
              var $44=HEAP32[((_clause_start_char)>>2)]; //@line 1387 "synthesize.cpp"
              var $add47=((($and46)+($44))&4294967295); //@line 1387 "synthesize.cpp"
              HEAP32[((__ZZ8GenerateP12PHONEME_LISTPiiE8sourceix)>>2)]=$add47; //@line 1387 "synthesize.cpp"
              var $45=$p; //@line 1389 "synthesize.cpp"
              var $newword48=(($45+19)&4294967295); //@line 1389 "synthesize.cpp"
              var $46=HEAPU8[($newword48)]; //@line 1389 "synthesize.cpp"
              var $conv49=((($46))&255); //@line 1389 "synthesize.cpp"
              var $and50=($conv49) & 4; //@line 1389 "synthesize.cpp"
              var $tobool51=((($and50))|0)!=0; //@line 1389 "synthesize.cpp"
              if ($tobool51) { __label__ = 23;; } else { __label__ = 24;; } //@line 1389 "synthesize.cpp"
              if (__label__ == 23) {
  
                var $47=HEAP32[((__ZZ8GenerateP12PHONEME_LISTPiiE8sourceix)>>2)]; //@line 1390 "synthesize.cpp"
                var $48=HEAP32[((_count_sentences)>>2)]; //@line 1390 "synthesize.cpp"
                __Z8DoMarkeriiii(2, $47, 0, $48); //@line 1390 "synthesize.cpp"
                ; //@line 1390 "synthesize.cpp"
              }
  
              var $49=$p; //@line 1395 "synthesize.cpp"
              var $newword54=(($49+19)&4294967295); //@line 1395 "synthesize.cpp"
              var $50=HEAPU8[($newword54)]; //@line 1395 "synthesize.cpp"
              var $conv55=((($50))&255); //@line 1395 "synthesize.cpp"
              var $and56=($conv55) & 1; //@line 1395 "synthesize.cpp"
              var $tobool57=((($and56))|0)!=0; //@line 1395 "synthesize.cpp"
              if (!($tobool57)) { __label__ = 26;break $if_then31$$if_end63$25; } //@line 1395 "synthesize.cpp"
  
              var $51=HEAP32[((__ZZ8GenerateP12PHONEME_LISTPiiE8sourceix)>>2)]; //@line 1396 "synthesize.cpp"
              var $52=$p; //@line 1396 "synthesize.cpp"
              var $sourceix59=(($52+6)&4294967295); //@line 1396 "synthesize.cpp"
              var $53=HEAPU16[(($sourceix59)>>1)]; //@line 1396 "synthesize.cpp"
              var $conv60=((($53))&65535); //@line 1396 "synthesize.cpp"
              var $shr=($conv60) >> 11; //@line 1396 "synthesize.cpp"
              var $54=HEAP32[((_clause_start_word)>>2)]; //@line 1396 "synthesize.cpp"
              var $55=HEAPU32[((__ZZ8GenerateP12PHONEME_LISTPiiE10word_count)>>2)]; //@line 1396 "synthesize.cpp"
              var $inc=((($55)+1)&4294967295); //@line 1396 "synthesize.cpp"
              HEAP32[((__ZZ8GenerateP12PHONEME_LISTPiiE10word_count)>>2)]=$inc; //@line 1396 "synthesize.cpp"
              var $add61=((($55)+($54))&4294967295); //@line 1396 "synthesize.cpp"
              __Z8DoMarkeriiii(1, $51, $shr, $add61); //@line 1396 "synthesize.cpp"
              ; //@line 1396 "synthesize.cpp"
            }
          } while(0);
  
          __ZL12EndAmplitudev(); //@line 1399 "synthesize.cpp"
          var $56=$p; //@line 1401 "synthesize.cpp"
          var $prepause=(($56+16)&4294967295); //@line 1401 "synthesize.cpp"
          var $57=HEAPU8[($prepause)]; //@line 1401 "synthesize.cpp"
          var $conv64=((($57))&255); //@line 1401 "synthesize.cpp"
          var $cmp65=((($conv64))|0) > 0; //@line 1401 "synthesize.cpp"
          if ($cmp65) { __label__ = 27;; } else { __label__ = 28;; } //@line 1401 "synthesize.cpp"
          if (__label__ == 27) {
  
            var $58=$p; //@line 1402 "synthesize.cpp"
            var $prepause67=(($58+16)&4294967295); //@line 1402 "synthesize.cpp"
            var $59=HEAPU8[($prepause67)]; //@line 1402 "synthesize.cpp"
            var $conv68=((($59))&255); //@line 1402 "synthesize.cpp"
            __ZL7DoPauseii($conv68, 1); //@line 1402 "synthesize.cpp"
            ; //@line 1402 "synthesize.cpp"
          }
  
          var $60=HEAP32[((_option_phoneme_events)>>2)]; //@line 1404 "synthesize.cpp"
          var $tobool70=((($60))|0)!=0; //@line 1404 "synthesize.cpp"
          if ($tobool70) { __label__ = 29;; } else { __label__ = 31;; } //@line 1404 "synthesize.cpp"
          $land_lhs_true71$$if_end77$41: do { 
            if (__label__ == 29) {
  
              var $61=$p; //@line 1404 "synthesize.cpp"
              var $type72=(($61+15)&4294967295); //@line 1404 "synthesize.cpp"
              var $62=HEAPU8[($type72)]; //@line 1404 "synthesize.cpp"
              var $conv73=((($62))&255); //@line 1404 "synthesize.cpp"
              var $cmp74=((($conv73))|0)!=2; //@line 1404 "synthesize.cpp"
              if (!($cmp74)) { __label__ = 31;break $land_lhs_true71$$if_end77$41; } //@line 1404 "synthesize.cpp"
  
              var $63=HEAP32[((__ZZ8GenerateP12PHONEME_LISTPiiE8sourceix)>>2)]; //@line 1407 "synthesize.cpp"
              var $64=$p; //@line 1407 "synthesize.cpp"
              var $ph76=(($64+8)&4294967295); //@line 1407 "synthesize.cpp"
              var $65=HEAP32[(($ph76)>>2)]; //@line 1407 "synthesize.cpp"
              var $mnemonic=(($65)&4294967295); //@line 1407 "synthesize.cpp"
              var $66=HEAP32[(($mnemonic)>>2)]; //@line 1407 "synthesize.cpp"
              __Z8DoMarkeriiii(7, $63, 0, $66); //@line 1407 "synthesize.cpp"
              ; //@line 1408 "synthesize.cpp"
            }
          } while(0);
  
          var $67=$p; //@line 1410 "synthesize.cpp"
          var $type78=(($67+15)&4294967295); //@line 1410 "synthesize.cpp"
          var $68=HEAPU8[($type78)]; //@line 1410 "synthesize.cpp"
          var $conv79=((($68))&255); //@line 1410 "synthesize.cpp"
          if ($conv79 == 0) {
            __label__ = 32;;
          }
          else if ($conv79 == 4) {
            __label__ = 33;;
          }
          else if ($conv79 == 6) {
            __label__ = 41;;
          }
          else if ($conv79 == 5) {
            __label__ = 44;;
          }
          else if ($conv79 == 7) {
            __label__ = 67;;
          }
          else if ($conv79 == 8) {
            __label__ = 81;;
          }
          else if ($conv79 == 3) {
            __label__ = 91;;
          }
          else if ($conv79 == 2) {
            __label__ = 100;;
          }
          else {
          __label__ = 133;;
          }
          
          $sw_epilog$$sw_bb$$sw_bb81$$sw_bb105$$sw_bb118$$sw_bb229$$sw_bb313$$sw_bb367$$sw_bb418$45: do { 
            if (__label__ == 32) {
  
              var $69=$p; //@line 1413 "synthesize.cpp"
              var $length=(($69+12)&4294967295); //@line 1413 "synthesize.cpp"
              var $70=HEAP16[(($length)>>1)]; //@line 1413 "synthesize.cpp"
              var $conv80=(tempInt=(($70)),(tempInt>=32768?tempInt-65536:tempInt)); //@line 1413 "synthesize.cpp"
              __ZL7DoPauseii($conv80, 0); //@line 1413 "synthesize.cpp"
              ; //@line 1414 "synthesize.cpp"
            }
            else if (__label__ == 33) {
  
              $released=0; //@line 1417 "synthesize.cpp"
              var $71=$next; //@line 1418 "synthesize.cpp"
              var $type82=(($71+15)&4294967295); //@line 1418 "synthesize.cpp"
              var $72=HEAPU8[($type82)]; //@line 1418 "synthesize.cpp"
              var $conv83=((($72))&255); //@line 1418 "synthesize.cpp"
              var $cmp84=((($conv83))|0)==2; //@line 1418 "synthesize.cpp"
              if ($cmp84) { __label__ = 34;; } else { __label__ = 35;; } //@line 1418 "synthesize.cpp"
              $if_then85$$if_else86$48: do { 
                if (__label__ == 34) {
  
                  $released=1; //@line 1420 "synthesize.cpp"
                  ; //@line 1421 "synthesize.cpp"
                }
                else if (__label__ == 35) {
  
                  var $73=$next; //@line 1423 "synthesize.cpp"
                  var $newword87=(($73+19)&4294967295); //@line 1423 "synthesize.cpp"
                  var $74=HEAP8[($newword87)]; //@line 1423 "synthesize.cpp"
                  var $tobool88=(tempInt=(($74)),(tempInt>=128?tempInt-256:tempInt))!=0; //@line 1423 "synthesize.cpp"
                  if ($tobool88) { __label__ = 38;; } else { __label__ = 36;; } //@line 1423 "synthesize.cpp"
                  $if_end96$$if_then89$51: do { 
                    if (__label__ == 36) {
  
                      var $75=$next; //@line 1425 "synthesize.cpp"
                      var $type90=(($75+15)&4294967295); //@line 1425 "synthesize.cpp"
                      var $76=HEAPU8[($type90)]; //@line 1425 "synthesize.cpp"
                      var $conv91=((($76))&255); //@line 1425 "synthesize.cpp"
                      var $cmp92=((($conv91))|0)==3; //@line 1425 "synthesize.cpp"
                      if (!($cmp92)) { __label__ = 38;break $if_end96$$if_then89$51; } //@line 1425 "synthesize.cpp"
  
                      $released=1; //@line 1425 "synthesize.cpp"
                      __label__ = 40;break $if_then85$$if_else86$48; //@line 1425 "synthesize.cpp"
                    }
                  } while(0);
  
                  var $_pr=$released; //@line 1428 "synthesize.cpp"
                  var $cmp97=((($_pr))|0)==0; //@line 1428 "synthesize.cpp"
                  if (!($cmp97)) { __label__ = 40;break $if_then85$$if_else86$48; } //@line 1428 "synthesize.cpp"
  
                  var $77=$p; //@line 1429 "synthesize.cpp"
                  var $synthflags99=(($77+4)&4294967295); //@line 1429 "synthesize.cpp"
                  var $78=HEAPU16[(($synthflags99)>>1)]; //@line 1429 "synthesize.cpp"
                  var $conv100=((($78))&65535); //@line 1429 "synthesize.cpp"
                  var $or=($conv100) | 8192; //@line 1429 "synthesize.cpp"
                  var $conv101=((($or)) & 65535); //@line 1429 "synthesize.cpp"
                  HEAP16[(($synthflags99)>>1)]=$conv101; //@line 1429 "synthesize.cpp"
                  ; //@line 1429 "synthesize.cpp"
                }
              } while(0);
  
              var $79=$p; //@line 1431 "synthesize.cpp"
              __Z16InterpretPhonemeP10TranslatoriP12PHONEME_LISTP12PHONEME_DATA(0, 0, $79, $phdata); //@line 1431 "synthesize.cpp"
              var $80=HEAP32[(($pd_control)>>2)]; //@line 1432 "synthesize.cpp"
              var $or103=($80) | 4; //@line 1432 "synthesize.cpp"
              HEAP32[(($pd_control)>>2)]=$or103; //@line 1432 "synthesize.cpp"
              var $call104=__Z9DoSample3P12PHONEME_DATAii($phdata, 0, 0); //@line 1433 "synthesize.cpp"
              ; //@line 1434 "synthesize.cpp"
            }
            else if (__label__ == 41) {
  
              var $81=$p; //@line 1437 "synthesize.cpp"
              __Z16InterpretPhonemeP10TranslatoriP12PHONEME_LISTP12PHONEME_DATA(0, 0, $81, $phdata); //@line 1437 "synthesize.cpp"
              var $82=$p; //@line 1439 "synthesize.cpp"
              var $synthflags106=(($82+4)&4294967295); //@line 1439 "synthesize.cpp"
              var $83=HEAPU16[(($synthflags106)>>1)]; //@line 1439 "synthesize.cpp"
              var $conv107=((($83))&65535); //@line 1439 "synthesize.cpp"
              var $and108=($conv107) & 8; //@line 1439 "synthesize.cpp"
              var $tobool109=((($and108))|0)!=0; //@line 1439 "synthesize.cpp"
              if ($tobool109) { __label__ = 42;; } else { __label__ = 43;; } //@line 1439 "synthesize.cpp"
              if (__label__ == 42) {
  
                var $84=$p; //@line 1441 "synthesize.cpp"
                var $length111=(($84+12)&4294967295); //@line 1441 "synthesize.cpp"
                var $85=HEAP16[(($length111)>>1)]; //@line 1441 "synthesize.cpp"
                var $conv112=(tempInt=(($85)),(tempInt>=32768?tempInt-65536:tempInt)); //@line 1441 "synthesize.cpp"
                var $call113=__Z9DoSample3P12PHONEME_DATAii($phdata, $conv112, 0); //@line 1441 "synthesize.cpp"
                ; //@line 1442 "synthesize.cpp"
              }
  
              var $86=$p; //@line 1443 "synthesize.cpp"
              var $length115=(($86+12)&4294967295); //@line 1443 "synthesize.cpp"
              var $87=HEAP16[(($length115)>>1)]; //@line 1443 "synthesize.cpp"
              var $conv116=(tempInt=(($87)),(tempInt>=32768?tempInt-65536:tempInt)); //@line 1443 "synthesize.cpp"
              var $call117=__Z9DoSample3P12PHONEME_DATAii($phdata, $conv116, 0); //@line 1443 "synthesize.cpp"
              ; //@line 1444 "synthesize.cpp"
            }
            else if (__label__ == 44) {
  
              var $88=$p; //@line 1447 "synthesize.cpp"
              var $ph119=(($88+8)&4294967295); //@line 1447 "synthesize.cpp"
              var $89=HEAP32[(($ph119)>>2)]; //@line 1447 "synthesize.cpp"
              $ph=$89; //@line 1447 "synthesize.cpp"
              var $dest$, $stop$, $stop4$, $fast$, $value4$;
              $dest$ = $8;
              $stop$ = $dest$ + 48;
              $value4$ = 0;
              if ($value4$ < 0) $value4$ += 256;
              $value4$ = $value4$ + ($value4$<<8) + ($value4$<<16) + ($value4$*16777216);
              while ($dest$%4 !== 0 && $dest$ < $stop$) {
                ;; HEAP8[$dest$++] = 0;
              }
              $dest$ >>= 2;
              $stop4$ = $stop$ >> 2;
              while ($dest$ < $stop4$) {
              ;;;;;;;  HEAP32[$dest$++] = $value4$;
              }
              $dest$ <<= 2;
              while ($dest$ < $stop$) {
                ;; HEAP8[$dest$++] = 0;
              }; //@line 1448 "synthesize.cpp"
              HEAP32[(($fmt_control)>>2)]=4; //@line 1449 "synthesize.cpp"
              $pre_voiced=0; //@line 1451 "synthesize.cpp"
              var $90=$next; //@line 1452 "synthesize.cpp"
              var $type120=(($90+15)&4294967295); //@line 1452 "synthesize.cpp"
              var $91=HEAPU8[($type120)]; //@line 1452 "synthesize.cpp"
              var $conv121=((($91))&255); //@line 1452 "synthesize.cpp"
              var $cmp122=((($conv121))|0)==2; //@line 1452 "synthesize.cpp"
              if ($cmp122) { __label__ = 45;; } else { __label__ = 46;; } //@line 1452 "synthesize.cpp"
              $if_then123$$if_else128$62: do { 
                if (__label__ == 45) {
  
                  var $92=$p; //@line 1454 "synthesize.cpp"
                  var $amp=(($92+18)&4294967295); //@line 1454 "synthesize.cpp"
                  var $93=HEAPU8[($amp)]; //@line 1454 "synthesize.cpp"
                  var $conv124=((($93))&255); //@line 1454 "synthesize.cpp"
                  __ZL11DoAmplitudeiPh($conv124, 0); //@line 1454 "synthesize.cpp"
                  var $94=$p; //@line 1455 "synthesize.cpp"
                  var $env=(($94+14)&4294967295); //@line 1455 "synthesize.cpp"
                  var $95=HEAPU8[($env)]; //@line 1455 "synthesize.cpp"
                  var $idxprom=((($95))&255); //@line 1455 "synthesize.cpp"
                  var $arrayidx125=((_envelope_data+$idxprom*4)&4294967295); //@line 1455 "synthesize.cpp"
                  var $96=HEAP32[(($arrayidx125)>>2)]; //@line 1455 "synthesize.cpp"
                  var $97=$p; //@line 1455 "synthesize.cpp"
                  var $pitch1=(($97+20)&4294967295); //@line 1455 "synthesize.cpp"
                  var $98=HEAPU8[($pitch1)]; //@line 1455 "synthesize.cpp"
                  var $conv126=((($98))&255); //@line 1455 "synthesize.cpp"
                  var $99=$p; //@line 1455 "synthesize.cpp"
                  var $pitch2=(($99+21)&4294967295); //@line 1455 "synthesize.cpp"
                  var $100=HEAPU8[($pitch2)]; //@line 1455 "synthesize.cpp"
                  var $conv127=((($100))&255); //@line 1455 "synthesize.cpp"
                  __ZL7DoPitchPhii($96, $conv126, $conv127); //@line 1455 "synthesize.cpp"
                  $pre_voiced=1; //@line 1456 "synthesize.cpp"
                  ; //@line 1457 "synthesize.cpp"
                }
                else if (__label__ == 46) {
  
                  var $101=$next; //@line 1459 "synthesize.cpp"
                  var $type129=(($101+15)&4294967295); //@line 1459 "synthesize.cpp"
                  var $102=HEAPU8[($type129)]; //@line 1459 "synthesize.cpp"
                  var $conv130=((($102))&255); //@line 1459 "synthesize.cpp"
                  var $cmp131=((($conv130))|0)==3; //@line 1459 "synthesize.cpp"
                  if ($cmp131) { __label__ = 47;; } else { __label__ = 49;; } //@line 1459 "synthesize.cpp"
                  $land_lhs_true132$$if_else145$65: do { 
                    if (__label__ == 47) {
  
                      var $103=$next; //@line 1459 "synthesize.cpp"
                      var $newword133=(($103+19)&4294967295); //@line 1459 "synthesize.cpp"
                      var $104=HEAP8[($newword133)]; //@line 1459 "synthesize.cpp"
                      var $tobool134=(tempInt=(($104)),(tempInt>=128?tempInt-256:tempInt))!=0; //@line 1459 "synthesize.cpp"
                      if ($tobool134) { __label__ = 49;break $land_lhs_true132$$if_else145$65; } //@line 1459 "synthesize.cpp"
  
                      var $105=$next; //@line 1461 "synthesize.cpp"
                      var $amp136=(($105+18)&4294967295); //@line 1461 "synthesize.cpp"
                      var $106=HEAPU8[($amp136)]; //@line 1461 "synthesize.cpp"
                      var $conv137=((($106))&255); //@line 1461 "synthesize.cpp"
                      __ZL11DoAmplitudeiPh($conv137, 0); //@line 1461 "synthesize.cpp"
                      var $107=$next; //@line 1462 "synthesize.cpp"
                      var $env138=(($107+14)&4294967295); //@line 1462 "synthesize.cpp"
                      var $108=HEAPU8[($env138)]; //@line 1462 "synthesize.cpp"
                      var $idxprom139=((($108))&255); //@line 1462 "synthesize.cpp"
                      var $arrayidx140=((_envelope_data+$idxprom139*4)&4294967295); //@line 1462 "synthesize.cpp"
                      var $109=HEAP32[(($arrayidx140)>>2)]; //@line 1462 "synthesize.cpp"
                      var $110=$next; //@line 1462 "synthesize.cpp"
                      var $pitch1141=(($110+20)&4294967295); //@line 1462 "synthesize.cpp"
                      var $111=HEAPU8[($pitch1141)]; //@line 1462 "synthesize.cpp"
                      var $conv142=((($111))&255); //@line 1462 "synthesize.cpp"
                      var $112=$next; //@line 1462 "synthesize.cpp"
                      var $pitch2143=(($112+21)&4294967295); //@line 1462 "synthesize.cpp"
                      var $113=HEAPU8[($pitch2143)]; //@line 1462 "synthesize.cpp"
                      var $conv144=((($113))&255); //@line 1462 "synthesize.cpp"
                      __ZL7DoPitchPhii($109, $conv142, $conv144); //@line 1462 "synthesize.cpp"
                      $pre_voiced=1; //@line 1463 "synthesize.cpp"
                      __label__ = 51;break $if_then123$$if_else128$62; //@line 1464 "synthesize.cpp"
                    }
                  } while(0);
  
                  var $114=HEAP32[((__ZL14last_pitch_cmd)>>2)]; //@line 1467 "synthesize.cpp"
                  var $cmp146=((($114))|0) < 0; //@line 1467 "synthesize.cpp"
                  if (!($cmp146)) { __label__ = 51;break $if_then123$$if_else128$62; } //@line 1467 "synthesize.cpp"
  
                  var $115=$next; //@line 1469 "synthesize.cpp"
                  var $amp148=(($115+18)&4294967295); //@line 1469 "synthesize.cpp"
                  var $116=HEAPU8[($amp148)]; //@line 1469 "synthesize.cpp"
                  var $conv149=((($116))&255); //@line 1469 "synthesize.cpp"
                  __ZL11DoAmplitudeiPh($conv149, 0); //@line 1469 "synthesize.cpp"
                  var $117=$p; //@line 1470 "synthesize.cpp"
                  var $env150=(($117+14)&4294967295); //@line 1470 "synthesize.cpp"
                  var $118=HEAPU8[($env150)]; //@line 1470 "synthesize.cpp"
                  var $idxprom151=((($118))&255); //@line 1470 "synthesize.cpp"
                  var $arrayidx152=((_envelope_data+$idxprom151*4)&4294967295); //@line 1470 "synthesize.cpp"
                  var $119=HEAP32[(($arrayidx152)>>2)]; //@line 1470 "synthesize.cpp"
                  var $120=$p; //@line 1470 "synthesize.cpp"
                  var $pitch1153=(($120+20)&4294967295); //@line 1470 "synthesize.cpp"
                  var $121=HEAPU8[($pitch1153)]; //@line 1470 "synthesize.cpp"
                  var $conv154=((($121))&255); //@line 1470 "synthesize.cpp"
                  var $122=$p; //@line 1470 "synthesize.cpp"
                  var $pitch2155=(($122+21)&4294967295); //@line 1470 "synthesize.cpp"
                  var $123=HEAPU8[($pitch2155)]; //@line 1470 "synthesize.cpp"
                  var $conv156=((($123))&255); //@line 1470 "synthesize.cpp"
                  __ZL7DoPitchPhii($119, $conv154, $conv156); //@line 1470 "synthesize.cpp"
                  ; //@line 1471 "synthesize.cpp"
                }
              } while(0);
  
              var $124=$prev; //@line 1474 "synthesize.cpp"
              var $type160=(($124+15)&4294967295); //@line 1474 "synthesize.cpp"
              var $125=HEAPU8[($type160)]; //@line 1474 "synthesize.cpp"
              var $conv161=((($125))&255); //@line 1474 "synthesize.cpp"
              var $cmp162=((($conv161))|0)==2; //@line 1474 "synthesize.cpp"
              if ($cmp162) { __label__ = 54;; } else { __label__ = 52;; } //@line 1474 "synthesize.cpp"
              $if_then172$$lor_lhs_false163$71: do { 
                if (__label__ == 52) {
  
                  var $126=$prev; //@line 1474 "synthesize.cpp"
                  var $ph164=(($126+8)&4294967295); //@line 1474 "synthesize.cpp"
                  var $127=HEAP32[(($ph164)>>2)]; //@line 1474 "synthesize.cpp"
                  var $phflags165=(($127+4)&4294967295); //@line 1474 "synthesize.cpp"
                  var $128=HEAP32[(($phflags165)>>2)]; //@line 1474 "synthesize.cpp"
                  var $and166=($128) & 256; //@line 1474 "synthesize.cpp"
                  var $tobool167=((($and166))|0)!=0; //@line 1474 "synthesize.cpp"
                  if ($tobool167) { __label__ = 54;break $if_then172$$lor_lhs_false163$71; } //@line 1474 "synthesize.cpp"
  
                  var $129=$ph; //@line 1474 "synthesize.cpp"
                  var $phflags169=(($129+4)&4294967295); //@line 1474 "synthesize.cpp"
                  var $130=HEAP32[(($phflags169)>>2)]; //@line 1474 "synthesize.cpp"
                  var $and170=($130) & 33554432; //@line 1474 "synthesize.cpp"
                  var $tobool171=((($and170))|0)!=0; //@line 1474 "synthesize.cpp"
                  if ($tobool171) { __label__ = 54;break $if_then172$$lor_lhs_false163$71; } //@line 1474 "synthesize.cpp"
  
                  var $140=$p; //@line 1490 "synthesize.cpp"
                  var $synthflags184=(($140+4)&4294967295); //@line 1490 "synthesize.cpp"
                  var $141=HEAPU16[(($synthflags184)>>1)]; //@line 1490 "synthesize.cpp"
                  var $conv185=((($141))&65535); //@line 1490 "synthesize.cpp"
                  var $and186=($conv185) & 8; //@line 1490 "synthesize.cpp"
                  var $tobool187=((($and186))|0)!=0; //@line 1490 "synthesize.cpp"
                  if (!($tobool187)) { __label__ = 58;break $if_then172$$lor_lhs_false163$71; } //@line 1490 "synthesize.cpp"
  
                  __ZL7DoPauseii(50, 0); //@line 1492 "synthesize.cpp"
                  __label__ = 58;break $if_then172$$lor_lhs_false163$71; //@line 1493 "synthesize.cpp"
                }
              } while(0);
              $if_then172$$if_end190$76: do { 
                if (__label__ == 54) {
  
                  var $131=$p; //@line 1477 "synthesize.cpp"
                  __Z16InterpretPhonemeP10TranslatoriP12PHONEME_LISTP12PHONEME_DATA(0, 1, $131, $phdata); //@line 1477 "synthesize.cpp"
                  var $132=HEAP32[(($arrayidx173)>>2)]; //@line 1478 "synthesize.cpp"
                  HEAP32[(($fmt_addr)>>2)]=$132; //@line 1478 "synthesize.cpp"
                  var $133=HEAP32[(($arrayidx174)>>2)]; //@line 1479 "synthesize.cpp"
                  HEAP32[(($fmt_amp)>>2)]=$133; //@line 1479 "synthesize.cpp"
                  var $134=$ph; //@line 1481 "synthesize.cpp"
                  var $135=$p; //@line 1481 "synthesize.cpp"
                  var $call175=__Z8DoSpect2P11PHONEME_TABiP10FMT_PARAMSP12PHONEME_LISTi($134, 0, $fmtp, $135, 0); //@line 1481 "synthesize.cpp"
                  var $136=$p; //@line 1482 "synthesize.cpp"
                  var $synthflags176=(($136+4)&4294967295); //@line 1482 "synthesize.cpp"
                  var $137=HEAPU16[(($synthflags176)>>1)]; //@line 1482 "synthesize.cpp"
                  var $conv177=((($137))&65535); //@line 1482 "synthesize.cpp"
                  var $and178=($conv177) & 8; //@line 1482 "synthesize.cpp"
                  var $tobool179=((($and178))|0)!=0; //@line 1482 "synthesize.cpp"
                  if (!($tobool179)) { __label__ = 58;break $if_then172$$if_end190$76; } //@line 1482 "synthesize.cpp"
  
                  __ZL7DoPauseii(25, 1); //@line 1484 "synthesize.cpp"
                  var $138=$ph; //@line 1485 "synthesize.cpp"
                  var $139=$p; //@line 1485 "synthesize.cpp"
                  var $call181=__Z8DoSpect2P11PHONEME_TABiP10FMT_PARAMSP12PHONEME_LISTi($138, 0, $fmtp, $139, 0); //@line 1485 "synthesize.cpp"
                  ; //@line 1486 "synthesize.cpp"
                }
              } while(0);
  
              var $142=$pre_voiced; //@line 1496 "synthesize.cpp"
              var $tobool191=((($142))|0)!=0; //@line 1496 "synthesize.cpp"
              if ($tobool191) { __label__ = 59;; } else { __label__ = 60;; } //@line 1496 "synthesize.cpp"
              if (__label__ == 59) {
  
                __ZL13StartSyllablev(); //@line 1499 "synthesize.cpp"
                ; //@line 1500 "synthesize.cpp"
              }
              else if (__label__ == 60) {
  
                var $143=$p; //@line 1503 "synthesize.cpp"
                var $synthflags194=(($143+4)&4294967295); //@line 1503 "synthesize.cpp"
                var $144=HEAPU16[(($synthflags194)>>1)]; //@line 1503 "synthesize.cpp"
                var $conv195=((($144))&65535); //@line 1503 "synthesize.cpp"
                var $or196=($conv195) | 8192; //@line 1503 "synthesize.cpp"
                var $conv197=((($or196)) & 65535); //@line 1503 "synthesize.cpp"
                HEAP16[(($synthflags194)>>1)]=$conv197; //@line 1503 "synthesize.cpp"
                ;
              }
  
              var $145=$p; //@line 1505 "synthesize.cpp"
              __Z16InterpretPhonemeP10TranslatoriP12PHONEME_LISTP12PHONEME_DATA(0, 0, $145, $phdata); //@line 1505 "synthesize.cpp"
              var $146=HEAP32[(($arrayidx200)>>2)]; //@line 1506 "synthesize.cpp"
              HEAP32[(($fmt_addr201)>>2)]=$146; //@line 1506 "synthesize.cpp"
              var $147=HEAP32[(($arrayidx203)>>2)]; //@line 1507 "synthesize.cpp"
              HEAP32[(($fmt_amp204)>>2)]=$147; //@line 1507 "synthesize.cpp"
              var $148=HEAP32[(($arrayidx206)>>2)]; //@line 1508 "synthesize.cpp"
              HEAP32[(($wav_addr)>>2)]=$148; //@line 1508 "synthesize.cpp"
              var $149=HEAP32[(($arrayidx208)>>2)]; //@line 1509 "synthesize.cpp"
              HEAP32[(($wav_amp)>>2)]=$149; //@line 1509 "synthesize.cpp"
              var $150=$ph; //@line 1510 "synthesize.cpp"
              var $151=$p; //@line 1510 "synthesize.cpp"
              var $call209=__Z8DoSpect2P11PHONEME_TABiP10FMT_PARAMSP12PHONEME_LISTi($150, 0, $fmtp, $151, 0); //@line 1510 "synthesize.cpp"
              var $152=$p; //@line 1512 "synthesize.cpp"
              var $newword210=(($152+19)&4294967295); //@line 1512 "synthesize.cpp"
              var $153=HEAPU8[($newword210)]; //@line 1512 "synthesize.cpp"
              var $conv211=((($153))&255); //@line 1512 "synthesize.cpp"
              var $cmp212=((($conv211))|0)==0; //@line 1512 "synthesize.cpp"
              if (!($cmp212)) { __label__ = 133;break $sw_epilog$$sw_bb$$sw_bb81$$sw_bb105$$sw_bb118$$sw_bb229$$sw_bb313$$sw_bb367$$sw_bb418$45; } //@line 1512 "synthesize.cpp"
  
              var $154=$next2; //@line 1512 "synthesize.cpp"
              var $newword214=(($154+19)&4294967295); //@line 1512 "synthesize.cpp"
              var $155=HEAPU8[($newword214)]; //@line 1512 "synthesize.cpp"
              var $conv215=((($155))&255); //@line 1512 "synthesize.cpp"
              var $cmp216=((($conv215))|0)==0; //@line 1512 "synthesize.cpp"
              if (!($cmp216)) { __label__ = 133;break $sw_epilog$$sw_bb$$sw_bb81$$sw_bb105$$sw_bb118$$sw_bb229$$sw_bb313$$sw_bb367$$sw_bb418$45; } //@line 1512 "synthesize.cpp"
  
              var $156=$next; //@line 1514 "synthesize.cpp"
              var $type218=(($156+15)&4294967295); //@line 1514 "synthesize.cpp"
              var $157=HEAPU8[($type218)]; //@line 1514 "synthesize.cpp"
              var $conv219=((($157))&255); //@line 1514 "synthesize.cpp"
              var $cmp220=((($conv219))|0)==7; //@line 1514 "synthesize.cpp"
              if ($cmp220) { __label__ = 64;; } else { __label__ = 65;; } //@line 1514 "synthesize.cpp"
              if (__label__ == 64) {
  
                __ZL7DoPauseii(20, 0); //@line 1515 "synthesize.cpp"
                ; //@line 1515 "synthesize.cpp"
              }
  
              var $158=$next; //@line 1516 "synthesize.cpp"
              var $type223=(($158+15)&4294967295); //@line 1516 "synthesize.cpp"
              var $159=HEAPU8[($type223)]; //@line 1516 "synthesize.cpp"
              var $conv224=((($159))&255); //@line 1516 "synthesize.cpp"
              var $cmp225=((($conv224))|0)==6; //@line 1516 "synthesize.cpp"
              if (!($cmp225)) { __label__ = 133;break $sw_epilog$$sw_bb$$sw_bb81$$sw_bb105$$sw_bb118$$sw_bb229$$sw_bb313$$sw_bb367$$sw_bb418$45; } //@line 1516 "synthesize.cpp"
  
              __ZL7DoPauseii(12, 0); //@line 1517 "synthesize.cpp"
              ; //@line 1517 "synthesize.cpp"
            }
            else if (__label__ == 67) {
  
              var $160=$next; //@line 1522 "synthesize.cpp"
              var $type230=(($160+15)&4294967295); //@line 1522 "synthesize.cpp"
              var $161=HEAPU8[($type230)]; //@line 1522 "synthesize.cpp"
              var $conv231=((($161))&255); //@line 1522 "synthesize.cpp"
              var $cmp232=((($conv231))|0)==2; //@line 1522 "synthesize.cpp"
              if ($cmp232) { __label__ = 68;; } else { __label__ = 69;; } //@line 1522 "synthesize.cpp"
              $if_then233$$if_else243$91: do { 
                if (__label__ == 68) {
  
                  var $162=$p; //@line 1524 "synthesize.cpp"
                  var $amp234=(($162+18)&4294967295); //@line 1524 "synthesize.cpp"
                  var $163=HEAPU8[($amp234)]; //@line 1524 "synthesize.cpp"
                  var $conv235=((($163))&255); //@line 1524 "synthesize.cpp"
                  __ZL11DoAmplitudeiPh($conv235, 0); //@line 1524 "synthesize.cpp"
                  var $164=$p; //@line 1525 "synthesize.cpp"
                  var $env236=(($164+14)&4294967295); //@line 1525 "synthesize.cpp"
                  var $165=HEAPU8[($env236)]; //@line 1525 "synthesize.cpp"
                  var $idxprom237=((($165))&255); //@line 1525 "synthesize.cpp"
                  var $arrayidx238=((_envelope_data+$idxprom237*4)&4294967295); //@line 1525 "synthesize.cpp"
                  var $166=HEAP32[(($arrayidx238)>>2)]; //@line 1525 "synthesize.cpp"
                  var $167=$p; //@line 1525 "synthesize.cpp"
                  var $pitch1239=(($167+20)&4294967295); //@line 1525 "synthesize.cpp"
                  var $168=HEAPU8[($pitch1239)]; //@line 1525 "synthesize.cpp"
                  var $conv240=((($168))&255); //@line 1525 "synthesize.cpp"
                  var $169=$p; //@line 1525 "synthesize.cpp"
                  var $pitch2241=(($169+21)&4294967295); //@line 1525 "synthesize.cpp"
                  var $170=HEAPU8[($pitch2241)]; //@line 1525 "synthesize.cpp"
                  var $conv242=((($170))&255); //@line 1525 "synthesize.cpp"
                  __ZL7DoPitchPhii($166, $conv240, $conv242); //@line 1525 "synthesize.cpp"
                  ; //@line 1526 "synthesize.cpp"
                }
                else if (__label__ == 69) {
  
                  var $171=$next; //@line 1528 "synthesize.cpp"
                  var $type244=(($171+15)&4294967295); //@line 1528 "synthesize.cpp"
                  var $172=HEAPU8[($type244)]; //@line 1528 "synthesize.cpp"
                  var $conv245=((($172))&255); //@line 1528 "synthesize.cpp"
                  var $cmp246=((($conv245))|0)==3; //@line 1528 "synthesize.cpp"
                  if ($cmp246) { __label__ = 70;; } else { __label__ = 71;; } //@line 1528 "synthesize.cpp"
                  if (__label__ == 70) {
  
                    var $173=$next; //@line 1530 "synthesize.cpp"
                    var $amp248=(($173+18)&4294967295); //@line 1530 "synthesize.cpp"
                    var $174=HEAPU8[($amp248)]; //@line 1530 "synthesize.cpp"
                    var $conv249=((($174))&255); //@line 1530 "synthesize.cpp"
                    __ZL11DoAmplitudeiPh($conv249, 0); //@line 1530 "synthesize.cpp"
                    var $175=$next; //@line 1531 "synthesize.cpp"
                    var $env250=(($175+14)&4294967295); //@line 1531 "synthesize.cpp"
                    var $176=HEAPU8[($env250)]; //@line 1531 "synthesize.cpp"
                    var $idxprom251=((($176))&255); //@line 1531 "synthesize.cpp"
                    var $arrayidx252=((_envelope_data+$idxprom251*4)&4294967295); //@line 1531 "synthesize.cpp"
                    var $177=HEAP32[(($arrayidx252)>>2)]; //@line 1531 "synthesize.cpp"
                    var $178=$next; //@line 1531 "synthesize.cpp"
                    var $pitch1253=(($178+20)&4294967295); //@line 1531 "synthesize.cpp"
                    var $179=HEAPU8[($pitch1253)]; //@line 1531 "synthesize.cpp"
                    var $conv254=((($179))&255); //@line 1531 "synthesize.cpp"
                    var $180=$next; //@line 1531 "synthesize.cpp"
                    var $pitch2255=(($180+21)&4294967295); //@line 1531 "synthesize.cpp"
                    var $181=HEAPU8[($pitch2255)]; //@line 1531 "synthesize.cpp"
                    var $conv256=((($181))&255); //@line 1531 "synthesize.cpp"
                    __ZL7DoPitchPhii($177, $conv254, $conv256); //@line 1531 "synthesize.cpp"
                    ; //@line 1532 "synthesize.cpp"
                  }
                  else if (__label__ == 71) {
  
                    var $182=HEAP32[((__ZL14last_pitch_cmd)>>2)]; //@line 1535 "synthesize.cpp"
                    var $cmp258=((($182))|0) < 0; //@line 1535 "synthesize.cpp"
                    if (!($cmp258)) { __label__ = 73;break $if_then233$$if_else243$91; } //@line 1535 "synthesize.cpp"
  
                    var $183=$p; //@line 1537 "synthesize.cpp"
                    var $amp260=(($183+18)&4294967295); //@line 1537 "synthesize.cpp"
                    var $184=HEAPU8[($amp260)]; //@line 1537 "synthesize.cpp"
                    var $conv261=((($184))&255); //@line 1537 "synthesize.cpp"
                    __ZL11DoAmplitudeiPh($conv261, 0); //@line 1537 "synthesize.cpp"
                    var $185=$p; //@line 1538 "synthesize.cpp"
                    var $env262=(($185+14)&4294967295); //@line 1538 "synthesize.cpp"
                    var $186=HEAPU8[($env262)]; //@line 1538 "synthesize.cpp"
                    var $idxprom263=((($186))&255); //@line 1538 "synthesize.cpp"
                    var $arrayidx264=((_envelope_data+$idxprom263*4)&4294967295); //@line 1538 "synthesize.cpp"
                    var $187=HEAP32[(($arrayidx264)>>2)]; //@line 1538 "synthesize.cpp"
                    var $188=$p; //@line 1538 "synthesize.cpp"
                    var $pitch1265=(($188+20)&4294967295); //@line 1538 "synthesize.cpp"
                    var $189=HEAPU8[($pitch1265)]; //@line 1538 "synthesize.cpp"
                    var $conv266=((($189))&255); //@line 1538 "synthesize.cpp"
                    var $190=$p; //@line 1538 "synthesize.cpp"
                    var $pitch2267=(($190+21)&4294967295); //@line 1538 "synthesize.cpp"
                    var $191=HEAPU8[($pitch2267)]; //@line 1538 "synthesize.cpp"
                    var $conv268=((($191))&255); //@line 1538 "synthesize.cpp"
                    __ZL7DoPitchPhii($187, $conv266, $conv268); //@line 1538 "synthesize.cpp"
                    ; //@line 1539 "synthesize.cpp"
                  }
                }
              } while(0);
  
              var $192=$next; //@line 1542 "synthesize.cpp"
              var $type272=(($192+15)&4294967295); //@line 1542 "synthesize.cpp"
              var $193=HEAPU8[($type272)]; //@line 1542 "synthesize.cpp"
              var $conv273=((($193))&255); //@line 1542 "synthesize.cpp"
              var $cmp274=((($conv273))|0)==2; //@line 1542 "synthesize.cpp"
              if ($cmp274) { __label__ = 76;; } else { __label__ = 74;; } //@line 1542 "synthesize.cpp"
              $if_then283$$lor_lhs_false275$99: do { 
                if (__label__ == 74) {
  
                  var $194=$next; //@line 1542 "synthesize.cpp"
                  var $type276=(($194+15)&4294967295); //@line 1542 "synthesize.cpp"
                  var $195=HEAPU8[($type276)]; //@line 1542 "synthesize.cpp"
                  var $conv277=((($195))&255); //@line 1542 "synthesize.cpp"
                  var $cmp278=((($conv277))|0)==3; //@line 1542 "synthesize.cpp"
                  if ($cmp278) { __label__ = 75;; } else { __label__ = 77;; } //@line 1542 "synthesize.cpp"
                  if (__label__ == 75) {
  
                    var $196=$next; //@line 1542 "synthesize.cpp"
                    var $newword280=(($196+19)&4294967295); //@line 1542 "synthesize.cpp"
                    var $197=HEAPU8[($newword280)]; //@line 1542 "synthesize.cpp"
                    var $conv281=((($197))&255); //@line 1542 "synthesize.cpp"
                    var $cmp282=((($conv281))|0)==0; //@line 1542 "synthesize.cpp"
                    if ($cmp282) { __label__ = 76;break $if_then283$$lor_lhs_false275$99; } //@line 1542 "synthesize.cpp"
                  }
  
                  var $198=$p; //@line 1548 "synthesize.cpp"
                  var $synthflags285=(($198+4)&4294967295); //@line 1548 "synthesize.cpp"
                  var $199=HEAPU16[(($synthflags285)>>1)]; //@line 1548 "synthesize.cpp"
                  var $conv286=((($199))&65535); //@line 1548 "synthesize.cpp"
                  var $or287=($conv286) | 8192; //@line 1548 "synthesize.cpp"
                  var $conv288=((($or287)) & 65535); //@line 1548 "synthesize.cpp"
                  HEAP16[(($synthflags285)>>1)]=$conv288; //@line 1548 "synthesize.cpp"
                  __label__ = 78;break $if_then283$$lor_lhs_false275$99;
                }
              } while(0);
              if (__label__ == 76) {
  
                __ZL13StartSyllablev(); //@line 1544 "synthesize.cpp"
                ; //@line 1545 "synthesize.cpp"
              }
  
              var $200=$p; //@line 1550 "synthesize.cpp"
              __Z16InterpretPhonemeP10TranslatoriP12PHONEME_LISTP12PHONEME_DATA(0, 0, $200, $phdata); //@line 1550 "synthesize.cpp"
              var $dest$, $stop$, $stop4$, $fast$, $value4$;
              $dest$ = $9;
              $stop$ = $dest$ + 48;
              $value4$ = 0;
              if ($value4$ < 0) $value4$ += 256;
              $value4$ = $value4$ + ($value4$<<8) + ($value4$<<16) + ($value4$*16777216);
              while ($dest$%4 !== 0 && $dest$ < $stop$) {
                ;; HEAP8[$dest$++] = 0;
              }
              $dest$ >>= 2;
              $stop4$ = $stop$ >> 2;
              while ($dest$ < $stop4$) {
              ;;;;;;;  HEAP32[$dest$++] = $value4$;
              }
              $dest$ <<= 2;
              while ($dest$ < $stop$) {
                ;; HEAP8[$dest$++] = 0;
              }; //@line 1551 "synthesize.cpp"
              var $201=HEAP32[(($arrayidx290)>>2)]; //@line 1552 "synthesize.cpp"
              var $mul=((($201)*2)&4294967295); //@line 1552 "synthesize.cpp"
              HEAP32[(($std_length)>>2)]=$mul; //@line 1552 "synthesize.cpp"
              var $202=HEAP32[(($arrayidx292)>>2)]; //@line 1553 "synthesize.cpp"
              HEAP32[(($fmt_addr293)>>2)]=$202; //@line 1553 "synthesize.cpp"
              var $203=HEAP32[(($arrayidx295)>>2)]; //@line 1554 "synthesize.cpp"
              HEAP32[(($fmt_amp296)>>2)]=$203; //@line 1554 "synthesize.cpp"
              var $204=HEAP32[(($arrayidx298)>>2)]; //@line 1555 "synthesize.cpp"
              HEAP32[(($wav_addr299)>>2)]=$204; //@line 1555 "synthesize.cpp"
              var $205=HEAP32[(($arrayidx301)>>2)]; //@line 1556 "synthesize.cpp"
              HEAP32[(($wav_amp302)>>2)]=$205; //@line 1556 "synthesize.cpp"
              var $206=$p; //@line 1558 "synthesize.cpp"
              var $synthflags303=(($206+4)&4294967295); //@line 1558 "synthesize.cpp"
              var $207=HEAPU16[(($synthflags303)>>1)]; //@line 1558 "synthesize.cpp"
              var $conv304=((($207))&65535); //@line 1558 "synthesize.cpp"
              var $and305=($conv304) & 8; //@line 1558 "synthesize.cpp"
              var $tobool306=((($and305))|0)!=0; //@line 1558 "synthesize.cpp"
              if ($tobool306) { __label__ = 79;; } else { __label__ = 80;; } //@line 1558 "synthesize.cpp"
              if (__label__ == 79) {
  
                var $208=$p; //@line 1559 "synthesize.cpp"
                var $ph308=(($208+8)&4294967295); //@line 1559 "synthesize.cpp"
                var $209=HEAP32[(($ph308)>>2)]; //@line 1559 "synthesize.cpp"
                var $210=$p; //@line 1559 "synthesize.cpp"
                var $call309=__Z8DoSpect2P11PHONEME_TABiP10FMT_PARAMSP12PHONEME_LISTi($209, 0, $fmtp, $210, 0); //@line 1559 "synthesize.cpp"
                ; //@line 1559 "synthesize.cpp"
              }
  
              var $211=$p; //@line 1560 "synthesize.cpp"
              var $ph311=(($211+8)&4294967295); //@line 1560 "synthesize.cpp"
              var $212=HEAP32[(($ph311)>>2)]; //@line 1560 "synthesize.cpp"
              var $213=$p; //@line 1560 "synthesize.cpp"
              var $call312=__Z8DoSpect2P11PHONEME_TABiP10FMT_PARAMSP12PHONEME_LISTi($212, 0, $fmtp, $213, 0); //@line 1560 "synthesize.cpp"
              ; //@line 1561 "synthesize.cpp"
            }
            else if (__label__ == 81) {
  
              var $dest$, $stop$, $stop4$, $fast$, $value4$;
              $dest$ = $10;
              $stop$ = $dest$ + 48;
              $value4$ = 0;
              if ($value4$ < 0) $value4$ += 256;
              $value4$ = $value4$ + ($value4$<<8) + ($value4$<<16) + ($value4$*16777216);
              while ($dest$%4 !== 0 && $dest$ < $stop$) {
                ;; HEAP8[$dest$++] = 0;
              }
              $dest$ >>= 2;
              $stop4$ = $stop$ >> 2;
              while ($dest$ < $stop4$) {
              ;;;;;;;  HEAP32[$dest$++] = $value4$;
              }
              $dest$ <<= 2;
              while ($dest$ < $stop$) {
                ;; HEAP8[$dest$++] = 0;
              }; //@line 1564 "synthesize.cpp"
              var $214=$p; //@line 1565 "synthesize.cpp"
              var $synthflags314=(($214+4)&4294967295); //@line 1565 "synthesize.cpp"
              var $215=HEAPU16[(($synthflags314)>>1)]; //@line 1565 "synthesize.cpp"
              var $conv315=((($215))&65535); //@line 1565 "synthesize.cpp"
              var $and316=($conv315) & 1; //@line 1565 "synthesize.cpp"
              var $tobool317=((($and316))|0)!=0; //@line 1565 "synthesize.cpp"
              if ($tobool317) { __label__ = 83;; } else { __label__ = 82;; } //@line 1565 "synthesize.cpp"
              if (__label__ == 82) {
  
                var $216=$p; //@line 1567 "synthesize.cpp"
                var $amp319=(($216+18)&4294967295); //@line 1567 "synthesize.cpp"
                var $217=HEAPU8[($amp319)]; //@line 1567 "synthesize.cpp"
                var $conv320=((($217))&255); //@line 1567 "synthesize.cpp"
                __ZL11DoAmplitudeiPh($conv320, 0); //@line 1567 "synthesize.cpp"
                var $218=$p; //@line 1568 "synthesize.cpp"
                var $env321=(($218+14)&4294967295); //@line 1568 "synthesize.cpp"
                var $219=HEAPU8[($env321)]; //@line 1568 "synthesize.cpp"
                var $idxprom322=((($219))&255); //@line 1568 "synthesize.cpp"
                var $arrayidx323=((_envelope_data+$idxprom322*4)&4294967295); //@line 1568 "synthesize.cpp"
                var $220=HEAP32[(($arrayidx323)>>2)]; //@line 1568 "synthesize.cpp"
                var $221=$p; //@line 1568 "synthesize.cpp"
                var $pitch1324=(($221+20)&4294967295); //@line 1568 "synthesize.cpp"
                var $222=HEAPU8[($pitch1324)]; //@line 1568 "synthesize.cpp"
                var $conv325=((($222))&255); //@line 1568 "synthesize.cpp"
                var $223=$p; //@line 1568 "synthesize.cpp"
                var $pitch2326=(($223+21)&4294967295); //@line 1568 "synthesize.cpp"
                var $224=HEAPU8[($pitch2326)]; //@line 1568 "synthesize.cpp"
                var $conv327=((($224))&255); //@line 1568 "synthesize.cpp"
                __ZL7DoPitchPhii($220, $conv325, $conv327); //@line 1568 "synthesize.cpp"
                ; //@line 1569 "synthesize.cpp"
              }
  
              var $225=$prev; //@line 1571 "synthesize.cpp"
              var $type329=(($225+15)&4294967295); //@line 1571 "synthesize.cpp"
              var $226=HEAPU8[($type329)]; //@line 1571 "synthesize.cpp"
              var $conv330=((($226))&255); //@line 1571 "synthesize.cpp"
              var $cmp331=((($conv330))|0)==8; //@line 1571 "synthesize.cpp"
              if ($cmp331) { __label__ = 84;; } else { __label__ = 85;; } //@line 1571 "synthesize.cpp"
              if (__label__ == 84) {
  
                HEAP32[((__ZL10last_frame)>>2)]=0; //@line 1573 "synthesize.cpp"
                ; //@line 1574 "synthesize.cpp"
              }
  
              var $227=$p; //@line 1576 "synthesize.cpp"
              __Z16InterpretPhonemeP10TranslatoriP12PHONEME_LISTP12PHONEME_DATA(0, 0, $227, $phdata); //@line 1576 "synthesize.cpp"
              var $228=HEAP32[(($arrayidx335)>>2)]; //@line 1577 "synthesize.cpp"
              var $mul336=((($228)*2)&4294967295); //@line 1577 "synthesize.cpp"
              HEAP32[(($std_length337)>>2)]=$mul336; //@line 1577 "synthesize.cpp"
              var $229=HEAP32[(($arrayidx339)>>2)]; //@line 1578 "synthesize.cpp"
              HEAP32[(($fmt_addr340)>>2)]=$229; //@line 1578 "synthesize.cpp"
              var $230=HEAP32[(($arrayidx342)>>2)]; //@line 1579 "synthesize.cpp"
              HEAP32[(($fmt_amp343)>>2)]=$230; //@line 1579 "synthesize.cpp"
              var $231=$next; //@line 1581 "synthesize.cpp"
              var $type344=(($231+15)&4294967295); //@line 1581 "synthesize.cpp"
              var $232=HEAPU8[($type344)]; //@line 1581 "synthesize.cpp"
              var $conv345=((($232))&255); //@line 1581 "synthesize.cpp"
              var $cmp346=((($conv345))|0)==2; //@line 1581 "synthesize.cpp"
              if ($cmp346) { __label__ = 86;; } else { __label__ = 87;; } //@line 1581 "synthesize.cpp"
              if (__label__ == 86) {
  
                __ZL13StartSyllablev(); //@line 1583 "synthesize.cpp"
                var $233=$p; //@line 1584 "synthesize.cpp"
                var $ph348=(($233+8)&4294967295); //@line 1584 "synthesize.cpp"
                var $234=HEAP32[(($ph348)>>2)]; //@line 1584 "synthesize.cpp"
                var $235=$p; //@line 1584 "synthesize.cpp"
                var $call349=__Z8DoSpect2P11PHONEME_TABiP10FMT_PARAMSP12PHONEME_LISTi($234, 0, $fmtp, $235, 0); //@line 1584 "synthesize.cpp"
                ; //@line 1585 "synthesize.cpp"
              }
              else if (__label__ == 87) {
  
                var $236=$prev; //@line 1587 "synthesize.cpp"
                var $type351=(($236+15)&4294967295); //@line 1587 "synthesize.cpp"
                var $237=HEAPU8[($type351)]; //@line 1587 "synthesize.cpp"
                var $conv352=((($237))&255); //@line 1587 "synthesize.cpp"
                var $cmp353=((($conv352))|0)==2; //@line 1587 "synthesize.cpp"
                if ($cmp353) { __label__ = 88;; } else { __label__ = 90;; } //@line 1587 "synthesize.cpp"
                $land_lhs_true354$$if_else362$120: do { 
                  if (__label__ == 88) {
  
                    var $238=$p; //@line 1587 "synthesize.cpp"
                    var $synthflags355=(($238+4)&4294967295); //@line 1587 "synthesize.cpp"
                    var $239=HEAPU16[(($synthflags355)>>1)]; //@line 1587 "synthesize.cpp"
                    var $conv356=((($239))&65535); //@line 1587 "synthesize.cpp"
                    var $and357=($conv356) & 1; //@line 1587 "synthesize.cpp"
                    var $tobool358=((($and357))|0)!=0; //@line 1587 "synthesize.cpp"
                    if (!($tobool358)) { __label__ = 90;break $land_lhs_true354$$if_else362$120; } //@line 1587 "synthesize.cpp"
  
                    var $240=$p; //@line 1589 "synthesize.cpp"
                    var $ph360=(($240+8)&4294967295); //@line 1589 "synthesize.cpp"
                    var $241=HEAP32[(($ph360)>>2)]; //@line 1589 "synthesize.cpp"
                    var $242=$p; //@line 1589 "synthesize.cpp"
                    var $call361=__Z8DoSpect2P11PHONEME_TABiP10FMT_PARAMSP12PHONEME_LISTi($241, 0, $fmtp, $242, 0); //@line 1589 "synthesize.cpp"
                    __label__ = 133;break $sw_epilog$$sw_bb$$sw_bb81$$sw_bb105$$sw_bb118$$sw_bb229$$sw_bb313$$sw_bb367$$sw_bb418$45; //@line 1590 "synthesize.cpp"
                  }
                } while(0);
  
                HEAP32[((__ZL10last_frame)>>2)]=0; //@line 1593 "synthesize.cpp"
                var $243=$p; //@line 1594 "synthesize.cpp"
                var $ph363=(($243+8)&4294967295); //@line 1594 "synthesize.cpp"
                var $244=HEAP32[(($ph363)>>2)]; //@line 1594 "synthesize.cpp"
                var $245=$p; //@line 1594 "synthesize.cpp"
                var $call364=__Z8DoSpect2P11PHONEME_TABiP10FMT_PARAMSP12PHONEME_LISTi($244, 0, $fmtp, $245, 0); //@line 1594 "synthesize.cpp"
                HEAP32[((__ZL10last_frame)>>2)]=0; //@line 1595 "synthesize.cpp"
                ;
              }
            }
            else if (__label__ == 91) {
  
              var $dest$, $stop$, $stop4$, $fast$, $value4$;
              $dest$ = $11;
              $stop$ = $dest$ + 48;
              $value4$ = 0;
              if ($value4$ < 0) $value4$ += 256;
              $value4$ = $value4$ + ($value4$<<8) + ($value4$<<16) + ($value4$*16777216);
              while ($dest$%4 !== 0 && $dest$ < $stop$) {
                ;; HEAP8[$dest$++] = 0;
              }
              $dest$ >>= 2;
              $stop4$ = $stop$ >> 2;
              while ($dest$ < $stop4$) {
              ;;;;;;;  HEAP32[$dest$++] = $value4$;
              }
              $dest$ <<= 2;
              while ($dest$ < $stop$) {
                ;; HEAP8[$dest$++] = 0;
              }; //@line 1601 "synthesize.cpp"
              $modulation=0; //@line 1602 "synthesize.cpp"
              var $246=$p; //@line 1603 "synthesize.cpp"
              var $ph368=(($246+8)&4294967295); //@line 1603 "synthesize.cpp"
              var $247=HEAP32[(($ph368)>>2)]; //@line 1603 "synthesize.cpp"
              var $phflags369=(($247+4)&4294967295); //@line 1603 "synthesize.cpp"
              var $248=HEAP32[(($phflags369)>>2)]; //@line 1603 "synthesize.cpp"
              var $and370=($248) & 128; //@line 1603 "synthesize.cpp"
              var $tobool371=((($and370))|0)!=0; //@line 1603 "synthesize.cpp"
              if ($tobool371) { __label__ = 92;; } else { __label__ = 93;; } //@line 1603 "synthesize.cpp"
              if (__label__ == 92) {
  
                $modulation=5; //@line 1604 "synthesize.cpp"
                ; //@line 1604 "synthesize.cpp"
              }
  
              var $249=$prev; //@line 1606 "synthesize.cpp"
              var $ph374=(($249+8)&4294967295); //@line 1606 "synthesize.cpp"
              var $250=HEAP32[(($ph374)>>2)]; //@line 1606 "synthesize.cpp"
              $prev_ph=$250; //@line 1606 "synthesize.cpp"
              var $251=$p; //@line 1610 "synthesize.cpp"
              var $synthflags375=(($251+4)&4294967295); //@line 1610 "synthesize.cpp"
              var $252=HEAPU16[(($synthflags375)>>1)]; //@line 1610 "synthesize.cpp"
              var $conv376=((($252))&65535); //@line 1610 "synthesize.cpp"
              var $and377=($conv376) & 1; //@line 1610 "synthesize.cpp"
              var $tobool378=((($and377))|0)!=0; //@line 1610 "synthesize.cpp"
              if ($tobool378) { __label__ = 95;; } else { __label__ = 94;; } //@line 1610 "synthesize.cpp"
              if (__label__ == 94) {
  
                var $253=$p; //@line 1612 "synthesize.cpp"
                var $amp380=(($253+18)&4294967295); //@line 1612 "synthesize.cpp"
                var $254=HEAPU8[($amp380)]; //@line 1612 "synthesize.cpp"
                var $conv381=((($254))&255); //@line 1612 "synthesize.cpp"
                __ZL11DoAmplitudeiPh($conv381, 0); //@line 1612 "synthesize.cpp"
                var $255=$p; //@line 1613 "synthesize.cpp"
                var $env382=(($255+14)&4294967295); //@line 1613 "synthesize.cpp"
                var $256=HEAPU8[($env382)]; //@line 1613 "synthesize.cpp"
                var $idxprom383=((($256))&255); //@line 1613 "synthesize.cpp"
                var $arrayidx384=((_envelope_data+$idxprom383*4)&4294967295); //@line 1613 "synthesize.cpp"
                var $257=HEAP32[(($arrayidx384)>>2)]; //@line 1613 "synthesize.cpp"
                var $258=$p; //@line 1613 "synthesize.cpp"
                var $pitch1385=(($258+20)&4294967295); //@line 1613 "synthesize.cpp"
                var $259=HEAPU8[($pitch1385)]; //@line 1613 "synthesize.cpp"
                var $conv386=((($259))&255); //@line 1613 "synthesize.cpp"
                var $260=$p; //@line 1613 "synthesize.cpp"
                var $pitch2387=(($260+21)&4294967295); //@line 1613 "synthesize.cpp"
                var $261=HEAPU8[($pitch2387)]; //@line 1613 "synthesize.cpp"
                var $conv388=((($261))&255); //@line 1613 "synthesize.cpp"
                __ZL7DoPitchPhii($257, $conv386, $conv388); //@line 1613 "synthesize.cpp"
                ; //@line 1614 "synthesize.cpp"
              }
  
              var $262=$prev; //@line 1616 "synthesize.cpp"
              var $type390=(($262+15)&4294967295); //@line 1616 "synthesize.cpp"
              var $263=HEAPU8[($type390)]; //@line 1616 "synthesize.cpp"
              var $conv391=((($263))&255); //@line 1616 "synthesize.cpp"
              var $cmp392=((($conv391))|0)==8; //@line 1616 "synthesize.cpp"
              if ($cmp392) { __label__ = 96;; } else { __label__ = 97;; } //@line 1616 "synthesize.cpp"
              if (__label__ == 96) {
  
                HEAP32[((__ZL10last_frame)>>2)]=0; //@line 1618 "synthesize.cpp"
                ; //@line 1619 "synthesize.cpp"
              }
  
              var $264=$next; //@line 1621 "synthesize.cpp"
              var $type395=(($264+15)&4294967295); //@line 1621 "synthesize.cpp"
              var $265=HEAPU8[($type395)]; //@line 1621 "synthesize.cpp"
              var $conv396=((($265))&255); //@line 1621 "synthesize.cpp"
              var $cmp397=((($conv396))|0)==2; //@line 1621 "synthesize.cpp"
              if ($cmp397) { __label__ = 98;; } else { __label__ = 99;; } //@line 1621 "synthesize.cpp"
              if (__label__ == 98) {
  
                __ZL13StartSyllablev(); //@line 1623 "synthesize.cpp"
                ; //@line 1624 "synthesize.cpp"
              }
  
              var $266=$p; //@line 1625 "synthesize.cpp"
              __Z16InterpretPhonemeP10TranslatoriP12PHONEME_LISTP12PHONEME_DATA(0, 0, $266, $phdata); //@line 1625 "synthesize.cpp"
              var $267=HEAP32[(($arrayidx401)>>2)]; //@line 1626 "synthesize.cpp"
              var $mul402=((($267)*2)&4294967295); //@line 1626 "synthesize.cpp"
              HEAP32[(($std_length403)>>2)]=$mul402; //@line 1626 "synthesize.cpp"
              var $268=HEAP32[(($arrayidx405)>>2)]; //@line 1627 "synthesize.cpp"
              HEAP32[(($fmt_addr406)>>2)]=$268; //@line 1627 "synthesize.cpp"
              var $269=HEAP32[(($arrayidx408)>>2)]; //@line 1628 "synthesize.cpp"
              HEAP32[(($fmt_amp409)>>2)]=$269; //@line 1628 "synthesize.cpp"
              var $270=HEAP32[(($arrayidx411)>>2)]; //@line 1629 "synthesize.cpp"
              HEAP32[(($wav_addr412)>>2)]=$270; //@line 1629 "synthesize.cpp"
              var $271=HEAP32[(($arrayidx414)>>2)]; //@line 1630 "synthesize.cpp"
              HEAP32[(($wav_amp415)>>2)]=$271; //@line 1630 "synthesize.cpp"
              var $272=$p; //@line 1631 "synthesize.cpp"
              var $ph416=(($272+8)&4294967295); //@line 1631 "synthesize.cpp"
              var $273=HEAP32[(($ph416)>>2)]; //@line 1631 "synthesize.cpp"
              var $274=$p; //@line 1631 "synthesize.cpp"
              var $275=$modulation; //@line 1631 "synthesize.cpp"
              var $call417=__Z8DoSpect2P11PHONEME_TABiP10FMT_PARAMSP12PHONEME_LISTi($273, 0, $fmtp, $274, $275); //@line 1631 "synthesize.cpp"
              ; //@line 1633 "synthesize.cpp"
            }
            else if (__label__ == 100) {
  
              var $276=$p; //@line 1636 "synthesize.cpp"
              var $ph419=(($276+8)&4294967295); //@line 1636 "synthesize.cpp"
              var $277=HEAP32[(($ph419)>>2)]; //@line 1636 "synthesize.cpp"
              $ph=$277; //@line 1636 "synthesize.cpp"
              var $278=$p; //@line 1637 "synthesize.cpp"
              var $stresslevel=(($278+1)&4294967295); //@line 1637 "synthesize.cpp"
              var $279=HEAPU8[($stresslevel)]; //@line 1637 "synthesize.cpp"
              var $conv420=((($279))&255); //@line 1637 "synthesize.cpp"
              var $and421=($conv420) & 15; //@line 1637 "synthesize.cpp"
              $stress=$and421; //@line 1637 "synthesize.cpp"
              var $dest$, $stop$, $stop4$, $fast$, $value4$;
              $dest$ = $12;
              $stop$ = $dest$ + 48;
              $value4$ = 0;
              if ($value4$ < 0) $value4$ += 256;
              $value4$ = $value4$ + ($value4$<<8) + ($value4$<<16) + ($value4$*16777216);
              while ($dest$%4 !== 0 && $dest$ < $stop$) {
                ;; HEAP8[$dest$++] = 0;
              }
              $dest$ >>= 2;
              $stop4$ = $stop$ >> 2;
              while ($dest$ < $stop4$) {
              ;;;;;;;  HEAP32[$dest$++] = $value4$;
              }
              $dest$ <<= 2;
              while ($dest$ < $stop$) {
                ;; HEAP8[$dest$++] = 0;
              }; //@line 1639 "synthesize.cpp"
              var $280=$p; //@line 1641 "synthesize.cpp"
              __Z16InterpretPhonemeP10TranslatoriP12PHONEME_LISTP12PHONEME_DATA(0, 0, $280, $phdata); //@line 1641 "synthesize.cpp"
              var $281=HEAP32[(($arrayidx423)>>2)]; //@line 1642 "synthesize.cpp"
              var $mul424=((($281)*2)&4294967295); //@line 1642 "synthesize.cpp"
              HEAP32[(($std_length425)>>2)]=$mul424; //@line 1642 "synthesize.cpp"
              var $282=HEAP32[(($arrayidx427)>>2)]; //@line 1644 "synthesize.cpp"
              HEAP32[(($fmt_addr428)>>2)]=$282; //@line 1644 "synthesize.cpp"
              var $cmp429=((($282))|0)!=0; //@line 1644 "synthesize.cpp"
              if ($cmp429) { __label__ = 101;; } else { __label__ = 103;; } //@line 1644 "synthesize.cpp"
              $land_lhs_true430$$if_else437$138: do { 
                if (__label__ == 101) {
  
                  var $283=HEAP32[(($pd_control431)>>2)]; //@line 1644 "synthesize.cpp"
                  var $and432=($283) & 2; //@line 1644 "synthesize.cpp"
                  var $cmp433=((($and432))|0)==0; //@line 1644 "synthesize.cpp"
                  if (!($cmp433)) { __label__ = 103;break $land_lhs_true430$$if_else437$138; } //@line 1644 "synthesize.cpp"
  
                  var $284=HEAP32[(($arrayidx436)>>2)]; //@line 1647 "synthesize.cpp"
                  HEAP32[(($fmt_length)>>2)]=$284; //@line 1647 "synthesize.cpp"
                  __label__ = 107;break $land_lhs_true430$$if_else437$138; //@line 1648 "synthesize.cpp"
                }
              } while(0);
              $if_end454$$if_else437$141: do { 
                if (__label__ == 103) {
  
                  var $285=$prev; //@line 1650 "synthesize.cpp"
                  var $type438=(($285+15)&4294967295); //@line 1650 "synthesize.cpp"
                  var $286=HEAPU8[($type438)]; //@line 1650 "synthesize.cpp"
                  var $conv439=((($286))&255); //@line 1650 "synthesize.cpp"
                  var $cmp440=((($conv439))|0)!=0; //@line 1650 "synthesize.cpp"
                  if (!($cmp440)) { __label__ = 107;break $if_end454$$if_else437$141; } //@line 1650 "synthesize.cpp"
  
                  var $287=$prev; //@line 1653 "synthesize.cpp"
                  __Z16InterpretPhonemeP10TranslatoriP12PHONEME_LISTP12PHONEME_DATA(0, 0, $287, $phdata_prev); //@line 1653 "synthesize.cpp"
                  var $288=HEAP32[(($arrayidx443)>>2)]; //@line 1654 "synthesize.cpp"
                  HEAP32[(($fmt_addr444)>>2)]=$288; //@line 1654 "synthesize.cpp"
                  var $cmp445=((($288))|0)!=0; //@line 1654 "synthesize.cpp"
                  if ($cmp445) { __label__ = 105;; } else { __label__ = 106;; } //@line 1654 "synthesize.cpp"
                  if (__label__ == 105) {
  
                    var $289=HEAP32[(($arrayidx448)>>2)]; //@line 1657 "synthesize.cpp"
                    HEAP32[(($fmt2_lenadj)>>2)]=$289; //@line 1657 "synthesize.cpp"
                    ; //@line 1658 "synthesize.cpp"
                  }
  
                  var $290=HEAP32[(($arrayidx450)>>2)]; //@line 1659 "synthesize.cpp"
                  HEAP32[(($transition0)>>2)]=$290; //@line 1659 "synthesize.cpp"
                  var $291=HEAP32[(($arrayidx452)>>2)]; //@line 1660 "synthesize.cpp"
                  HEAP32[(($transition1)>>2)]=$291; //@line 1660 "synthesize.cpp"
                  ; //@line 1661 "synthesize.cpp"
                }
              } while(0);
  
              var $292=HEAP32[(($fmt_addr455)>>2)]; //@line 1663 "synthesize.cpp"
              var $cmp456=((($292))|0)==0; //@line 1663 "synthesize.cpp"
              if ($cmp456) { __label__ = 108;; } else { __label__ = 109;; } //@line 1663 "synthesize.cpp"
              if (__label__ == 108) {
  
                HEAP32[(($use_vowelin)>>2)]=1; //@line 1666 "synthesize.cpp"
                HEAP32[(($fmt_control458)>>2)]=1; //@line 1667 "synthesize.cpp"
                var $293=HEAP32[(($arrayidx460)>>2)]; //@line 1668 "synthesize.cpp"
                HEAP32[(($fmt_addr461)>>2)]=$293; //@line 1668 "synthesize.cpp"
                ; //@line 1669 "synthesize.cpp"
              }
  
              var $294=HEAP32[(($arrayidx464)>>2)]; //@line 1671 "synthesize.cpp"
              HEAP32[(($fmt_amp465)>>2)]=$294; //@line 1671 "synthesize.cpp"
              var $295=$p; //@line 1673 "synthesize.cpp"
              var $env466=(($295+14)&4294967295); //@line 1673 "synthesize.cpp"
              var $296=HEAPU8[($env466)]; //@line 1673 "synthesize.cpp"
              var $idxprom467=((($296))&255); //@line 1673 "synthesize.cpp"
              var $arrayidx468=((_envelope_data+$idxprom467*4)&4294967295); //@line 1673 "synthesize.cpp"
              var $297=HEAP32[(($arrayidx468)>>2)]; //@line 1673 "synthesize.cpp"
              $pitch_env=$297; //@line 1673 "synthesize.cpp"
              $amp_env=0; //@line 1674 "synthesize.cpp"
              var $298=$p; //@line 1675 "synthesize.cpp"
              var $tone_ph=(($298+3)&4294967295); //@line 1675 "synthesize.cpp"
              var $299=HEAPU8[($tone_ph)]; //@line 1675 "synthesize.cpp"
              var $conv469=((($299))&255); //@line 1675 "synthesize.cpp"
              var $cmp470=((($conv469))|0)!=0; //@line 1675 "synthesize.cpp"
              if ($cmp470) { __label__ = 110;; } else { __label__ = 112;; } //@line 1675 "synthesize.cpp"
              $if_then471$$if_end482$151: do { 
                if (__label__ == 110) {
  
                  var $300=$p; //@line 1677 "synthesize.cpp"
                  var $tone_ph472=(($300+3)&4294967295); //@line 1677 "synthesize.cpp"
                  var $301=HEAPU8[($tone_ph472)]; //@line 1677 "synthesize.cpp"
                  var $conv473=((($301))&255); //@line 1677 "synthesize.cpp"
                  __Z17InterpretPhoneme2iP12PHONEME_DATA($conv473, $phdata_tone); //@line 1677 "synthesize.cpp"
                  var $302=HEAP32[(($pitch_env474)>>2)]; //@line 1678 "synthesize.cpp"
                  var $call475=__Z11GetEnvelopei($302); //@line 1678 "synthesize.cpp"
                  $pitch_env=$call475; //@line 1678 "synthesize.cpp"
                  var $303=HEAP32[(($amp_env476)>>2)]; //@line 1679 "synthesize.cpp"
                  var $cmp477=((($303))|0) > 0; //@line 1679 "synthesize.cpp"
                  if (!($cmp477)) { __label__ = 112;break $if_then471$$if_end482$151; } //@line 1679 "synthesize.cpp"
  
                  var $304=HEAP32[(($amp_env479)>>2)]; //@line 1680 "synthesize.cpp"
                  var $call480=__Z11GetEnvelopei($304); //@line 1680 "synthesize.cpp"
                  $amp_env=$call480; //@line 1680 "synthesize.cpp"
                  ; //@line 1680 "synthesize.cpp"
                }
              } while(0);
  
              __ZL13StartSyllablev(); //@line 1683 "synthesize.cpp"
              $modulation=2; //@line 1685 "synthesize.cpp"
              var $305=$stress; //@line 1686 "synthesize.cpp"
              var $cmp483=((($305))|0) <= 1; //@line 1686 "synthesize.cpp"
              if ($cmp483) { __label__ = 113;; } else { __label__ = 114;; } //@line 1686 "synthesize.cpp"
              $if_then484$$if_else485$155: do { 
                if (__label__ == 113) {
  
                  $modulation=1; //@line 1687 "synthesize.cpp"
                  ; //@line 1687 "synthesize.cpp"
                }
                else if (__label__ == 114) {
  
                  var $306=$stress; //@line 1689 "synthesize.cpp"
                  var $cmp486=((($306))|0) >= 7; //@line 1689 "synthesize.cpp"
                  if (!($cmp486)) { __label__ = 116;break $if_then484$$if_else485$155; } //@line 1689 "synthesize.cpp"
  
                  $modulation=3; //@line 1690 "synthesize.cpp"
                  ; //@line 1690 "synthesize.cpp"
                }
              } while(0);
  
              var $307=$prev; //@line 1692 "synthesize.cpp"
              var $type490=(($307+15)&4294967295); //@line 1692 "synthesize.cpp"
              var $308=HEAPU8[($type490)]; //@line 1692 "synthesize.cpp"
              var $conv491=((($308))&255); //@line 1692 "synthesize.cpp"
              var $cmp492=((($conv491))|0)==5; //@line 1692 "synthesize.cpp"
              if ($cmp492) { __label__ = 118;; } else { __label__ = 117;; } //@line 1692 "synthesize.cpp"
              $if_then497$$lor_lhs_false493$160: do { 
                if (__label__ == 117) {
  
                  var $309=$prev; //@line 1692 "synthesize.cpp"
                  var $type494=(($309+15)&4294967295); //@line 1692 "synthesize.cpp"
                  var $310=HEAPU8[($type494)]; //@line 1692 "synthesize.cpp"
                  var $conv495=((($310))&255); //@line 1692 "synthesize.cpp"
                  var $cmp496=((($conv495))|0)==7; //@line 1692 "synthesize.cpp"
                  if ($cmp496) { __label__ = 118;break $if_then497$$lor_lhs_false493$160; } //@line 1692 "synthesize.cpp"
  
                  var $322=$prev; //@line 1699 "synthesize.cpp"
                  var $type506=(($322+15)&4294967295); //@line 1699 "synthesize.cpp"
                  var $323=HEAPU8[($type506)]; //@line 1699 "synthesize.cpp"
                  var $conv507=((($323))&255); //@line 1699 "synthesize.cpp"
                  var $cmp508=((($conv507))|0)==3; //@line 1699 "synthesize.cpp"
                  if ($cmp508) { __label__ = 121;; } else { __label__ = 120;; } //@line 1699 "synthesize.cpp"
                  $if_then513$$lor_lhs_false509$163: do { 
                    if (__label__ == 120) {
  
                      var $324=$prev; //@line 1699 "synthesize.cpp"
                      var $type510=(($324+15)&4294967295); //@line 1699 "synthesize.cpp"
                      var $325=HEAPU8[($type510)]; //@line 1699 "synthesize.cpp"
                      var $conv511=((($325))&255); //@line 1699 "synthesize.cpp"
                      var $cmp512=((($conv511))|0)==8; //@line 1699 "synthesize.cpp"
                      if ($cmp512) { __label__ = 121;break $if_then513$$lor_lhs_false509$163; } //@line 1699 "synthesize.cpp"
  
                      var $337=$p; //@line 1707 "synthesize.cpp"
                      var $synthflags522=(($337+4)&4294967295); //@line 1707 "synthesize.cpp"
                      var $338=HEAPU16[(($synthflags522)>>1)]; //@line 1707 "synthesize.cpp"
                      var $conv523=((($338))&65535); //@line 1707 "synthesize.cpp"
                      var $and524=($conv523) & 1; //@line 1707 "synthesize.cpp"
                      var $tobool525=((($and524))|0)!=0; //@line 1707 "synthesize.cpp"
                      if ($tobool525) { __label__ = 124;; } else { __label__ = 123;; } //@line 1707 "synthesize.cpp"
                      if (__label__ == 123) {
  
                        var $339=$p; //@line 1709 "synthesize.cpp"
                        var $amp527=(($339+18)&4294967295); //@line 1709 "synthesize.cpp"
                        var $340=HEAPU8[($amp527)]; //@line 1709 "synthesize.cpp"
                        var $conv528=((($340))&255); //@line 1709 "synthesize.cpp"
                        var $341=$amp_env; //@line 1709 "synthesize.cpp"
                        __ZL11DoAmplitudeiPh($conv528, $341); //@line 1709 "synthesize.cpp"
                        var $342=$pitch_env; //@line 1710 "synthesize.cpp"
                        var $343=$p; //@line 1710 "synthesize.cpp"
                        var $pitch1529=(($343+20)&4294967295); //@line 1710 "synthesize.cpp"
                        var $344=HEAPU8[($pitch1529)]; //@line 1710 "synthesize.cpp"
                        var $conv530=((($344))&255); //@line 1710 "synthesize.cpp"
                        var $345=$p; //@line 1710 "synthesize.cpp"
                        var $pitch2531=(($345+21)&4294967295); //@line 1710 "synthesize.cpp"
                        var $346=HEAPU8[($pitch2531)]; //@line 1710 "synthesize.cpp"
                        var $conv532=((($346))&255); //@line 1710 "synthesize.cpp"
                        __ZL7DoPitchPhii($342, $conv530, $conv532); //@line 1710 "synthesize.cpp"
                        ; //@line 1711 "synthesize.cpp"
                      }
  
                      var $347=$ph; //@line 1713 "synthesize.cpp"
                      var $348=$p; //@line 1713 "synthesize.cpp"
                      var $349=$modulation; //@line 1713 "synthesize.cpp"
                      var $call534=__Z8DoSpect2P11PHONEME_TABiP10FMT_PARAMSP12PHONEME_LISTi($347, 1, $fmtp, $348, $349); //@line 1713 "synthesize.cpp"
                      __label__ = 125;break $if_then497$$lor_lhs_false493$160;
                    }
                  } while(0);
  
                  var $326=$p; //@line 1701 "synthesize.cpp"
                  var $amp514=(($326+18)&4294967295); //@line 1701 "synthesize.cpp"
                  var $327=HEAPU8[($amp514)]; //@line 1701 "synthesize.cpp"
                  var $conv515=((($327))&255); //@line 1701 "synthesize.cpp"
                  var $328=$amp_env; //@line 1701 "synthesize.cpp"
                  __ZL11DoAmplitudeiPh($conv515, $328); //@line 1701 "synthesize.cpp"
                  var $329=$ph; //@line 1702 "synthesize.cpp"
                  var $330=$p; //@line 1702 "synthesize.cpp"
                  var $331=$modulation; //@line 1702 "synthesize.cpp"
                  var $call516=__Z8DoSpect2P11PHONEME_TABiP10FMT_PARAMSP12PHONEME_LISTi($329, 1, $fmtp, $330, $331); //@line 1702 "synthesize.cpp"
                  var $332=$pitch_env; //@line 1703 "synthesize.cpp"
                  var $333=$p; //@line 1703 "synthesize.cpp"
                  var $pitch1517=(($333+20)&4294967295); //@line 1703 "synthesize.cpp"
                  var $334=HEAPU8[($pitch1517)]; //@line 1703 "synthesize.cpp"
                  var $conv518=((($334))&255); //@line 1703 "synthesize.cpp"
                  var $335=$p; //@line 1703 "synthesize.cpp"
                  var $pitch2519=(($335+21)&4294967295); //@line 1703 "synthesize.cpp"
                  var $336=HEAPU8[($pitch2519)]; //@line 1703 "synthesize.cpp"
                  var $conv520=((($336))&255); //@line 1703 "synthesize.cpp"
                  __ZL7DoPitchPhii($332, $conv518, $conv520); //@line 1703 "synthesize.cpp"
                  __label__ = 125;break $if_then497$$lor_lhs_false493$160; //@line 1704 "synthesize.cpp"
                }
              } while(0);
              if (__label__ == 118) {
  
                var $311=$p; //@line 1694 "synthesize.cpp"
                var $amp498=(($311+18)&4294967295); //@line 1694 "synthesize.cpp"
                var $312=HEAPU8[($amp498)]; //@line 1694 "synthesize.cpp"
                var $conv499=((($312))&255); //@line 1694 "synthesize.cpp"
                var $313=$amp_env; //@line 1694 "synthesize.cpp"
                __ZL11DoAmplitudeiPh($conv499, $313); //@line 1694 "synthesize.cpp"
                var $314=$pitch_env; //@line 1695 "synthesize.cpp"
                var $315=$p; //@line 1695 "synthesize.cpp"
                var $pitch1500=(($315+20)&4294967295); //@line 1695 "synthesize.cpp"
                var $316=HEAPU8[($pitch1500)]; //@line 1695 "synthesize.cpp"
                var $conv501=((($316))&255); //@line 1695 "synthesize.cpp"
                var $317=$p; //@line 1695 "synthesize.cpp"
                var $pitch2502=(($317+21)&4294967295); //@line 1695 "synthesize.cpp"
                var $318=HEAPU8[($pitch2502)]; //@line 1695 "synthesize.cpp"
                var $conv503=((($318))&255); //@line 1695 "synthesize.cpp"
                __ZL7DoPitchPhii($314, $conv501, $conv503); //@line 1695 "synthesize.cpp"
                var $319=$ph; //@line 1696 "synthesize.cpp"
                var $320=$p; //@line 1696 "synthesize.cpp"
                var $321=$modulation; //@line 1696 "synthesize.cpp"
                var $call504=__Z8DoSpect2P11PHONEME_TABiP10FMT_PARAMSP12PHONEME_LISTi($319, 1, $fmtp, $320, $321); //@line 1696 "synthesize.cpp"
                ; //@line 1697 "synthesize.cpp"
              }
  
              var $350=HEAP32[((_option_phoneme_events)>>2)]; //@line 1716 "synthesize.cpp"
              var $tobool537=((($350))|0)!=0; //@line 1716 "synthesize.cpp"
              if ($tobool537) { __label__ = 126;; } else { __label__ = 127;; } //@line 1716 "synthesize.cpp"
              if (__label__ == 126) {
  
                var $351=HEAP32[((__ZZ8GenerateP12PHONEME_LISTPiiE8sourceix)>>2)]; //@line 1718 "synthesize.cpp"
                var $352=$p; //@line 1718 "synthesize.cpp"
                var $ph539=(($352+8)&4294967295); //@line 1718 "synthesize.cpp"
                var $353=HEAP32[(($ph539)>>2)]; //@line 1718 "synthesize.cpp"
                var $mnemonic540=(($353)&4294967295); //@line 1718 "synthesize.cpp"
                var $354=HEAP32[(($mnemonic540)>>2)]; //@line 1718 "synthesize.cpp"
                __Z8DoMarkeriiii(7, $351, 0, $354); //@line 1718 "synthesize.cpp"
                ; //@line 1719 "synthesize.cpp"
              }
  
              var $355=HEAP32[(($arrayidx543)>>2)]; //@line 1721 "synthesize.cpp"
              HEAP32[(($fmt_addr544)>>2)]=$355; //@line 1721 "synthesize.cpp"
              var $356=HEAP32[(($arrayidx546)>>2)]; //@line 1722 "synthesize.cpp"
              HEAP32[(($fmt_amp547)>>2)]=$356; //@line 1722 "synthesize.cpp"
              HEAP32[(($transition0548)>>2)]=0; //@line 1723 "synthesize.cpp"
              HEAP32[(($transition1549)>>2)]=0; //@line 1724 "synthesize.cpp"
              var $357=HEAP32[(($arrayidx551)>>2)]; //@line 1726 "synthesize.cpp"
              HEAP32[(($fmt2_addr)>>2)]=$357; //@line 1726 "synthesize.cpp"
              var $cmp552=((($357))|0)!=0; //@line 1726 "synthesize.cpp"
              if ($cmp552) { __label__ = 128;; } else { __label__ = 129;; } //@line 1726 "synthesize.cpp"
              $if_then553$$if_else557$176: do { 
                if (__label__ == 128) {
  
                  var $358=HEAP32[(($arrayidx555)>>2)]; //@line 1728 "synthesize.cpp"
                  HEAP32[(($fmt2_lenadj556)>>2)]=$358; //@line 1728 "synthesize.cpp"
                  ; //@line 1729 "synthesize.cpp"
                }
                else if (__label__ == 129) {
  
                  var $359=$next; //@line 1731 "synthesize.cpp"
                  var $type558=(($359+15)&4294967295); //@line 1731 "synthesize.cpp"
                  var $360=HEAPU8[($type558)]; //@line 1731 "synthesize.cpp"
                  var $conv559=((($360))&255); //@line 1731 "synthesize.cpp"
                  var $cmp560=((($conv559))|0)!=0; //@line 1731 "synthesize.cpp"
                  if (!($cmp560)) { __label__ = 132;break $if_then553$$if_else557$176; } //@line 1731 "synthesize.cpp"
  
                  HEAP32[(($fmt2_lenadj562)>>2)]=0; //@line 1733 "synthesize.cpp"
                  var $361=$next; //@line 1734 "synthesize.cpp"
                  __Z16InterpretPhonemeP10TranslatoriP12PHONEME_LISTP12PHONEME_DATA(0, 0, $361, $phdata_next); //@line 1734 "synthesize.cpp"
                  HEAP32[(($use_vowelin563)>>2)]=1; //@line 1736 "synthesize.cpp"
                  var $362=HEAP32[(($arrayidx565)>>2)]; //@line 1737 "synthesize.cpp"
                  HEAP32[(($transition0566)>>2)]=$362; //@line 1737 "synthesize.cpp"
                  var $363=HEAP32[(($arrayidx568)>>2)]; //@line 1738 "synthesize.cpp"
                  HEAP32[(($transition1569)>>2)]=$363; //@line 1738 "synthesize.cpp"
                  var $364=HEAP32[(($arrayidx571)>>2)]; //@line 1740 "synthesize.cpp"
                  HEAP32[(($fmt2_addr572)>>2)]=$364; //@line 1740 "synthesize.cpp"
                  var $cmp573=((($364))|0)!=0; //@line 1740 "synthesize.cpp"
                  if (!($cmp573)) { __label__ = 132;break $if_then553$$if_else557$176; } //@line 1740 "synthesize.cpp"
  
                  var $365=HEAP32[(($arrayidx576)>>2)]; //@line 1742 "synthesize.cpp"
                  HEAP32[(($fmt2_lenadj577)>>2)]=$365; //@line 1742 "synthesize.cpp"
                  ; //@line 1743 "synthesize.cpp"
                }
              } while(0);
  
              var $366=$ph; //@line 1746 "synthesize.cpp"
              var $367=$p; //@line 1746 "synthesize.cpp"
              var $368=$modulation; //@line 1746 "synthesize.cpp"
              var $call581=__Z8DoSpect2P11PHONEME_TABiP10FMT_PARAMSP12PHONEME_LISTi($366, 2, $fmtp, $367, $368); //@line 1746 "synthesize.cpp"
              ; //@line 1748 "synthesize.cpp"
            }
          } while(0);
  
          var $369=HEAP32[((__ZZ8GenerateP12PHONEME_LISTPiiE2ix)>>2)]; //@line 1750 "synthesize.cpp"
          var $inc582=((($369)+1)&4294967295); //@line 1750 "synthesize.cpp"
          HEAP32[((__ZZ8GenerateP12PHONEME_LISTPiiE2ix)>>2)]=$inc582; //@line 1750 "synthesize.cpp"
          __label__ = 7;continue $while_cond$11; //@line 1751 "synthesize.cpp"
        }
        if (__label__ == 134) {
  
          __ZL8EndPitchi(1); //@line 1752 "synthesize.cpp"
          var $370=$n_ph_addr; //@line 1753 "synthesize.cpp"
          var $371=HEAP32[(($370)>>2)]; //@line 1753 "synthesize.cpp"
          var $cmp583=((($371))|0) > 0; //@line 1753 "synthesize.cpp"
          if ($cmp583) { __label__ = 135;; } else { __label__ = 136;; } //@line 1753 "synthesize.cpp"
          if (__label__ == 135) {
  
            var $372=HEAP32[((_count_characters)>>2)]; //@line 1755 "synthesize.cpp"
            var $373=HEAP32[((_count_sentences)>>2)]; //@line 1755 "synthesize.cpp"
            __Z8DoMarkeriiii(5, $372, 0, $373); //@line 1755 "synthesize.cpp"
            var $374=$n_ph_addr; //@line 1756 "synthesize.cpp"
            HEAP32[(($374)>>2)]=0; //@line 1756 "synthesize.cpp"
            ; //@line 1757 "synthesize.cpp"
          }
  
          $retval=0; //@line 1759 "synthesize.cpp"
          ; //@line 1759 "synthesize.cpp"
        }
        else if (__label__ == 14) {
  
          $retval=1; //@line 1365 "synthesize.cpp"
          ; //@line 1365 "synthesize.cpp"
        }
      }
    }
  
    var $375=$retval; //@line 1760 "synthesize.cpp"
    STACKTOP = __stackBase__;
    return $375; //@line 1760 "synthesize.cpp"
    return null;
  }
  

  function __ZL12EndAmplitudev() {
    ;
    var __label__;
  
    var $0=HEAP32[((__ZL10amp_length)>>2)]; //@line 110 "synthesize.cpp"
    var $cmp=((($0))|0) > 0; //@line 110 "synthesize.cpp"
    if ($cmp) { __label__ = 1;; } else { __label__ = 4;; } //@line 110 "synthesize.cpp"
    if (__label__ == 1) {
  
      var $1=HEAP32[((__ZL12last_amp_cmd)>>2)]; //@line 112 "synthesize.cpp"
      var $arrayidx=((_wcmdq+$1*16)&4294967295); //@line 112 "synthesize.cpp"
      var $arrayidx1=(($arrayidx+4)&4294967295); //@line 112 "synthesize.cpp"
      var $2=HEAP32[(($arrayidx1)>>2)]; //@line 112 "synthesize.cpp"
      var $cmp2=((($2))|0)==0; //@line 112 "synthesize.cpp"
      if ($cmp2) { __label__ = 2;; } else { __label__ = 3;; } //@line 112 "synthesize.cpp"
      if (__label__ == 2) {
  
        var $3=HEAP32[((__ZL10amp_length)>>2)]; //@line 113 "synthesize.cpp"
        var $4=HEAP32[((__ZL12last_amp_cmd)>>2)]; //@line 113 "synthesize.cpp"
        var $arrayidx4=((_wcmdq+$4*16)&4294967295); //@line 113 "synthesize.cpp"
        var $arrayidx5=(($arrayidx4+4)&4294967295); //@line 113 "synthesize.cpp"
        HEAP32[(($arrayidx5)>>2)]=$3; //@line 113 "synthesize.cpp"
        ; //@line 113 "synthesize.cpp"
      }
  
      HEAP32[((__ZL10amp_length)>>2)]=0; //@line 114 "synthesize.cpp"
      ; //@line 115 "synthesize.cpp"
    }
  
    ;
    return; //@line 116 "synthesize.cpp"
    return;
  }
  

  function __ZL11DoAmplitudeiPh($amp, $amp_env) {
    ;
    var __label__;
  
    var $amp_addr;
    var $amp_env_addr;
    var $q;
    $amp_addr=$amp;
    $amp_env_addr=$amp_env;
    var $0=HEAP32[((_wcmdq_tail)>>2)]; //@line 147 "synthesize.cpp"
    HEAP32[((__ZL12last_amp_cmd)>>2)]=$0; //@line 147 "synthesize.cpp"
    HEAP32[((__ZL10amp_length)>>2)]=0; //@line 148 "synthesize.cpp"
    var $1=HEAP32[((_wcmdq_tail)>>2)]; //@line 150 "synthesize.cpp"
    var $arrayidx=((_wcmdq+$1*16)&4294967295); //@line 150 "synthesize.cpp"
    var $arraydecay=(($arrayidx)&4294967295); //@line 150 "synthesize.cpp"
    $q=$arraydecay; //@line 150 "synthesize.cpp"
    var $2=$q; //@line 151 "synthesize.cpp"
    var $arrayidx1=(($2)&4294967295); //@line 151 "synthesize.cpp"
    HEAP32[(($arrayidx1)>>2)]=8; //@line 151 "synthesize.cpp"
    var $3=$q; //@line 152 "synthesize.cpp"
    var $arrayidx2=(($3+4)&4294967295); //@line 152 "synthesize.cpp"
    HEAP32[(($arrayidx2)>>2)]=0; //@line 152 "synthesize.cpp"
    var $4=$amp_env_addr; //@line 153 "synthesize.cpp"
    var $5=($4); //@line 153 "synthesize.cpp"
    var $6=$q; //@line 153 "synthesize.cpp"
    var $arrayidx3=(($6+8)&4294967295); //@line 153 "synthesize.cpp"
    HEAP32[(($arrayidx3)>>2)]=$5; //@line 153 "synthesize.cpp"
    var $7=$amp_addr; //@line 154 "synthesize.cpp"
    var $8=$q; //@line 154 "synthesize.cpp"
    var $arrayidx4=(($8+12)&4294967295); //@line 154 "synthesize.cpp"
    HEAP32[(($arrayidx4)>>2)]=$7; //@line 154 "synthesize.cpp"
    __Z8WcmdqIncv(); //@line 155 "synthesize.cpp"
    ;
    return; //@line 156 "synthesize.cpp"
    return;
  }
  

  function __ZL7DoPitchPhii($env, $pitch1, $pitch2) {
    ;
    var __label__;
  
    var $env_addr;
    var $pitch1_addr;
    var $pitch2_addr;
    var $q;
    $env_addr=$env;
    $pitch1_addr=$pitch1;
    $pitch2_addr=$pitch2;
    __ZL8EndPitchi(0); //@line 164 "synthesize.cpp"
    var $0=$pitch1_addr; //@line 166 "synthesize.cpp"
    var $cmp=((($0))|0)==255; //@line 166 "synthesize.cpp"
    if ($cmp) { __label__ = 1;; } else { __label__ = 2;; } //@line 166 "synthesize.cpp"
    $if_end_thread$$if_end$2: do { 
      if (__label__ == 1) {
  
        $pitch1_addr=55; //@line 169 "synthesize.cpp"
        $pitch2_addr=76; //@line 170 "synthesize.cpp"
        var $1=HEAP32[((((_envelope_data)&4294967295))>>2)]; //@line 171 "synthesize.cpp"
        $env_addr=$1; //@line 171 "synthesize.cpp"
        var $2=HEAP32[((_wcmdq_tail)>>2)]; //@line 173 "synthesize.cpp"
        HEAP32[((__ZL14last_pitch_cmd)>>2)]=$2; //@line 173 "synthesize.cpp"
        HEAP32[((__ZL12pitch_length)>>2)]=0; //@line 174 "synthesize.cpp"
        ; //@line 176 "synthesize.cpp"
      }
      else if (__label__ == 2) {
  
        var $_pr=$pitch2_addr; //@line 176 "synthesize.cpp"
        var $3=HEAP32[((_wcmdq_tail)>>2)]; //@line 173 "synthesize.cpp"
        HEAP32[((__ZL14last_pitch_cmd)>>2)]=$3; //@line 173 "synthesize.cpp"
        HEAP32[((__ZL12pitch_length)>>2)]=0; //@line 174 "synthesize.cpp"
        var $cmp1=((($_pr))|0) < 0; //@line 176 "synthesize.cpp"
        if (!($cmp1)) { __label__ = 4;break $if_end_thread$$if_end$2; } //@line 176 "synthesize.cpp"
  
        $pitch2_addr=0; //@line 177 "synthesize.cpp"
        ; //@line 177 "synthesize.cpp"
      }
    } while(0);
  
    var $4=HEAP32[((_wcmdq_tail)>>2)]; //@line 179 "synthesize.cpp"
    var $arrayidx=((_wcmdq+$4*16)&4294967295); //@line 179 "synthesize.cpp"
    var $arraydecay=(($arrayidx)&4294967295); //@line 179 "synthesize.cpp"
    $q=$arraydecay; //@line 179 "synthesize.cpp"
    var $5=$q; //@line 180 "synthesize.cpp"
    var $arrayidx4=(($5)&4294967295); //@line 180 "synthesize.cpp"
    HEAP32[(($arrayidx4)>>2)]=9; //@line 180 "synthesize.cpp"
    var $6=$q; //@line 181 "synthesize.cpp"
    var $arrayidx5=(($6+4)&4294967295); //@line 181 "synthesize.cpp"
    HEAP32[(($arrayidx5)>>2)]=0; //@line 181 "synthesize.cpp"
    var $7=$env_addr; //@line 182 "synthesize.cpp"
    var $8=($7); //@line 182 "synthesize.cpp"
    var $9=$q; //@line 182 "synthesize.cpp"
    var $arrayidx6=(($9+8)&4294967295); //@line 182 "synthesize.cpp"
    HEAP32[(($arrayidx6)>>2)]=$8; //@line 182 "synthesize.cpp"
    var $10=$pitch1_addr; //@line 183 "synthesize.cpp"
    var $shl=((($10)*65536)&4294967295); //@line 183 "synthesize.cpp"
    var $11=$pitch2_addr; //@line 183 "synthesize.cpp"
    var $add=((($shl)+($11))&4294967295); //@line 183 "synthesize.cpp"
    var $12=$q; //@line 183 "synthesize.cpp"
    var $arrayidx7=(($12+12)&4294967295); //@line 183 "synthesize.cpp"
    HEAP32[(($arrayidx7)>>2)]=$add; //@line 183 "synthesize.cpp"
    __Z8WcmdqIncv(); //@line 184 "synthesize.cpp"
    ;
    return; //@line 185 "synthesize.cpp"
    return;
  }
  

  function __ZL13StartSyllablev() {
    ;
    var __label__;
  
    var $0=HEAP32[((__ZL12syllable_end)>>2)]; //@line 981 "synthesize.cpp"
    var $1=HEAP32[((__ZL14syllable_start)>>2)]; //@line 981 "synthesize.cpp"
    var $cmp=((($0))|0)==((($1))|0); //@line 981 "synthesize.cpp"
    if ($cmp) { __label__ = 1;; } else { __label__ = 2;; } //@line 981 "synthesize.cpp"
    if (__label__ == 1) {
  
      var $2=HEAP32[((_wcmdq_tail)>>2)]; //@line 982 "synthesize.cpp"
      HEAP32[((__ZL12syllable_end)>>2)]=$2; //@line 982 "synthesize.cpp"
      ; //@line 982 "synthesize.cpp"
    }
  
    ;
    return; //@line 983 "synthesize.cpp"
    return;
  }
  

  function __Z12SynthOnTimerv() {
    ;
    var __label__;
  
    var $retval;
    var $_b=HEAPU8[(__ZL8timer_on_b)];
    var $0=((($_b))&1);
    var $tobool=((($0))|0)!=0; //@line 1770 "synthesize.cpp"
    if ($tobool) { __label__ = 2;; } else { __label__ = 1;; } //@line 1770 "synthesize.cpp"
    if (__label__ == 2) {
      while(1) { 
  
        var $call1=__Z9WcmdqUsedv(); //@line 1776 "synthesize.cpp"
        var $call5=__Z8GenerateP12PHONEME_LISTPii(((_phoneme_list)&4294967295), _n_phoneme_list, 1); //@line 1779 "synthesize.cpp"
        var $cmp6=((($call5))|0)==0; //@line 1779 "synthesize.cpp"
        if ($cmp6) { __label__ = 3;; } else { __label__ = 4;; } //@line 1779 "synthesize.cpp"
        if (__label__ == 3) {
  
          var $call8=__Z15SpeakNextClauseP7__sFILEPKvi(0, 0, 1); //@line 1781 "synthesize.cpp"
          ; //@line 1782 "synthesize.cpp"
        }
  
        var $1=HEAP32[((_skipping_text)>>2)]; //@line 1783 "synthesize.cpp"
        var $tobool10=((($1))|0)!=0; //@line 1783 "synthesize.cpp"
        if ($tobool10) { __label__ = 2;continue ; } else { __label__ = 5;break ; } //@line 1783 "synthesize.cpp"
      }
  
      $retval=0; //@line 1785 "synthesize.cpp"
      ; //@line 1785 "synthesize.cpp"
    }
    else if (__label__ == 1) {
  
      $retval=0; //@line 1772 "synthesize.cpp"
      ; //@line 1772 "synthesize.cpp"
    }
  
    var $2=$retval; //@line 1786 "synthesize.cpp"
    ;
    return $2; //@line 1786 "synthesize.cpp"
    return null;
  }
  

  function __Z15SpeakNextClauseP7__sFILEPKvi($f_in, $text_in, $control) {
    var __stackBase__  = STACKTOP; STACKTOP += 8; _memset(__stackBase__, 0, 8);
    var __label__;
    var __lastLabel__ = null;
  
    var $retval;
    var $f_in_addr;
    var $text_in_addr;
    var $control_addr;
    var $clause_tone=__stackBase__;
    var $voice_change=__stackBase__+4;
    $f_in_addr=$f_in;
    $text_in_addr=$text_in;
    $control_addr=$control;
    var $0=$control_addr; //@line 1814 "synthesize.cpp"
    var $cmp=((($0))|0)==4; //@line 1814 "synthesize.cpp"
    if ($cmp) { __label__ = 1;; } else { __label__ = 4;; } //@line 1814 "synthesize.cpp"
    $if_then$$if_end$2: do { 
      if (__label__ == 1) {
  
        var $1=HEAP32[((__ZZ15SpeakNextClauseP7__sFILEPKviE6f_text)>>2)]; //@line 1816 "synthesize.cpp"
        var $cmp1=((($1))|0)==0; //@line 1816 "synthesize.cpp"
        var $2=HEAP32[((__ZZ15SpeakNextClauseP7__sFILEPKviE6p_text)>>2)]; //@line 1816 "synthesize.cpp"
        var $cmp2=((($2))|0)==0; //@line 1816 "synthesize.cpp"
        var $or_cond=($cmp1) & ($cmp2); //@line 1816 "synthesize.cpp"
        if ($or_cond) { __label__ = 2;; } else { __label__ = 3;; } //@line 1816 "synthesize.cpp"
        if (__label__ == 2) {
  
          $retval=0; //@line 1817 "synthesize.cpp"
          ; //@line 1817 "synthesize.cpp"
        }
        else if (__label__ == 3) {
  
          $retval=1; //@line 1819 "synthesize.cpp"
          ; //@line 1819 "synthesize.cpp"
        }
      }
      else if (__label__ == 4) {
  
        var $3=$control_addr; //@line 1822 "synthesize.cpp"
        var $cmp4=((($3))|0)==2; //@line 1822 "synthesize.cpp"
        if ($cmp4) { __label__ = 5;; } else { __label__ = 8;; } //@line 1822 "synthesize.cpp"
        if (__label__ == 5) {
  
          HEAP8[(__ZL8timer_on_b)]=0;
          HEAP32[((__ZZ15SpeakNextClauseP7__sFILEPKviE6p_text)>>2)]=0; //@line 1826 "synthesize.cpp"
          var $4=HEAP32[((__ZZ15SpeakNextClauseP7__sFILEPKviE6f_text)>>2)]; //@line 1827 "synthesize.cpp"
          var $cmp6=((($4))|0)!=0; //@line 1827 "synthesize.cpp"
          if ($cmp6) { __label__ = 6;; } else { __label__ = 7;; } //@line 1827 "synthesize.cpp"
          if (__label__ == 6) {
  
            var $5=HEAP32[((__ZZ15SpeakNextClauseP7__sFILEPKviE6f_text)>>2)]; //@line 1829 "synthesize.cpp"
            var $call=_fclose($5); //@line 1829 "synthesize.cpp"
            HEAP32[((__ZZ15SpeakNextClauseP7__sFILEPKviE6f_text)>>2)]=0; //@line 1830 "synthesize.cpp"
            ; //@line 1831 "synthesize.cpp"
          }
  
          HEAP32[((_n_phoneme_list)>>2)]=0; //@line 1832 "synthesize.cpp"
          __Z9WcmdqStopv(); //@line 1833 "synthesize.cpp"
          HEAP32[((((_embedded_value+24)&4294967295))>>2)]=0; //@line 1835 "synthesize.cpp"
          $retval=0; //@line 1836 "synthesize.cpp"
          ; //@line 1836 "synthesize.cpp"
        }
        else if (__label__ == 8) {
  
          var $6=$control_addr; //@line 1839 "synthesize.cpp"
          var $cmp10=((($6))|0)==3; //@line 1839 "synthesize.cpp"
          if ($cmp10) { __label__ = 9;; } else { __label__ = 13;; } //@line 1839 "synthesize.cpp"
          if (__label__ == 9) {
  
            var $_b=HEAPU8[(__ZL6paused_b)];
            var $7=($_b) ? 2 : 0;
            var $cmp12=((($7))|0)==0; //@line 1842 "synthesize.cpp"
            if ($cmp12) { __label__ = 10;; } else { __label__ = 11;; } //@line 1842 "synthesize.cpp"
            if (__label__ == 10) {
  
              HEAP8[(__ZL8timer_on_b)]=0;
              HEAP8[(__ZL6paused_b)]=1;
              ; //@line 1846 "synthesize.cpp"
            }
            else if (__label__ == 11) {
  
              HEAP8[(__ZL8timer_on_b)]=1;
              HEAP8[(__ZL6paused_b)]=0;
              var $call16=__Z8GenerateP12PHONEME_LISTPii(((_phoneme_list)&4294967295), _n_phoneme_list, 0); //@line 1852 "synthesize.cpp"
              ;
            }
  
            $retval=0; //@line 1854 "synthesize.cpp"
            ; //@line 1854 "synthesize.cpp"
          }
          else if (__label__ == 13) {
  
            var $8=$control_addr; //@line 1857 "synthesize.cpp"
            var $cmp19=((($8))|0)==5; //@line 1857 "synthesize.cpp"
            if ($cmp19) { __label__ = 14;; } else { __label__ = 15;; } //@line 1857 "synthesize.cpp"
            if (__label__ == 14) {
  
              HEAP32[((_n_phoneme_list)>>2)]=0; //@line 1860 "synthesize.cpp"
              __Z9WcmdqStopv(); //@line 1861 "synthesize.cpp"
              $retval=0; //@line 1862 "synthesize.cpp"
              ; //@line 1862 "synthesize.cpp"
            }
            else if (__label__ == 15) {
  
              var $9=$f_in_addr; //@line 1865 "synthesize.cpp"
              var $cmp22=((($9))|0)!=0; //@line 1865 "synthesize.cpp"
              if ($cmp22) { __label__ = 17;; } else { __label__ = 16;; } //@line 1865 "synthesize.cpp"
              $if_then24$$lor_lhs_false$24: do { 
                if (__label__ == 16) {
  
                  var $10=$text_in_addr; //@line 1865 "synthesize.cpp"
                  var $cmp23=((($10))|0)!=0; //@line 1865 "synthesize.cpp"
                  if ($cmp23) { __label__ = 17;break $if_then24$$lor_lhs_false$24; } else { __label__ = 18;break $if_then24$$lor_lhs_false$24; } //@line 1865 "synthesize.cpp"
                }
              } while(0);
              if (__label__ == 17) {
  
                var $11=$f_in_addr; //@line 1867 "synthesize.cpp"
                HEAP32[((__ZZ15SpeakNextClauseP7__sFILEPKviE6f_text)>>2)]=$11; //@line 1867 "synthesize.cpp"
                var $12=$text_in_addr; //@line 1868 "synthesize.cpp"
                HEAP32[((__ZZ15SpeakNextClauseP7__sFILEPKviE6p_text)>>2)]=$12; //@line 1868 "synthesize.cpp"
                HEAP8[(__ZL8timer_on_b)]=1;
                HEAP8[(__ZL6paused_b)]=0;
                ; //@line 1871 "synthesize.cpp"
              }
  
              var $13=HEAP32[((__ZZ15SpeakNextClauseP7__sFILEPKviE6f_text)>>2)]; //@line 1873 "synthesize.cpp"
              var $cmp26=((($13))|0)==0; //@line 1873 "synthesize.cpp"
              var $14=HEAP32[((__ZZ15SpeakNextClauseP7__sFILEPKviE6p_text)>>2)]; //@line 1873 "synthesize.cpp"
              var $cmp28=((($14))|0)==0; //@line 1873 "synthesize.cpp"
              var $or_cond1=($cmp26) & ($cmp28); //@line 1873 "synthesize.cpp"
              if ($or_cond1) { __label__ = 19;; } else { __label__ = 20;; } //@line 1873 "synthesize.cpp"
              if (__label__ == 19) {
  
                HEAP32[((_skipping_text)>>2)]=0; //@line 1875 "synthesize.cpp"
                HEAP8[(__ZL8timer_on_b)]=0;
                $retval=0; //@line 1877 "synthesize.cpp"
                ; //@line 1877 "synthesize.cpp"
              }
              else if (__label__ == 20) {
  
                var $15=HEAP32[((__ZZ15SpeakNextClauseP7__sFILEPKviE6f_text)>>2)]; //@line 1880 "synthesize.cpp"
                var $cmp31=((($15))|0)!=0; //@line 1880 "synthesize.cpp"
                if ($cmp31) { __label__ = 21;; } else { __label__ = 23;; } //@line 1880 "synthesize.cpp"
                $land_lhs_true32$$if_end36$32: do { 
                  if (__label__ == 21) {
  
                    var $16=HEAP32[((__ZZ15SpeakNextClauseP7__sFILEPKviE6f_text)>>2)]; //@line 1880 "synthesize.cpp"
                    var $_flags=(($16+12)&4294967295); //@line 1880 "synthesize.cpp"
                    var $17=HEAP16[(($_flags)>>1)]; //@line 1880 "synthesize.cpp"
                    var $conv=(tempInt=(($17)),(tempInt>=32768?tempInt-65536:tempInt)); //@line 1880 "synthesize.cpp"
                    var $and=($conv) & 32; //@line 1880 "synthesize.cpp"
                    var $cmp33=((($and))|0)!=0; //@line 1880 "synthesize.cpp"
                    if (!($cmp33)) { __label__ = 23;break $land_lhs_true32$$if_end36$32; } //@line 1880 "synthesize.cpp"
  
                    HEAP8[(__ZL8timer_on_b)]=0;
                    var $18=HEAP32[((__ZZ15SpeakNextClauseP7__sFILEPKviE6f_text)>>2)]; //@line 1883 "synthesize.cpp"
                    var $call35=_fclose($18); //@line 1883 "synthesize.cpp"
                    HEAP32[((__ZZ15SpeakNextClauseP7__sFILEPKviE6f_text)>>2)]=0; //@line 1884 "synthesize.cpp"
                    $retval=0; //@line 1885 "synthesize.cpp"
                    __label__ = 39;break $if_then$$if_end$2; //@line 1885 "synthesize.cpp"
                  }
                } while(0);
  
                var $19=HEAP32[((_current_phoneme_table)>>2)]; //@line 1888 "synthesize.cpp"
                var $20=HEAP32[((_voice)>>2)]; //@line 1888 "synthesize.cpp"
                var $phoneme_tab_ix=(($20+40)&4294967295); //@line 1888 "synthesize.cpp"
                var $21=HEAP32[(($phoneme_tab_ix)>>2)]; //@line 1888 "synthesize.cpp"
                var $cmp37=((($19))|0)!=((($21))|0); //@line 1888 "synthesize.cpp"
                if ($cmp37) { __label__ = 24;; } else { __label__ = 25;; } //@line 1888 "synthesize.cpp"
                if (__label__ == 24) {
  
                  var $22=HEAP32[((_voice)>>2)]; //@line 1890 "synthesize.cpp"
                  var $phoneme_tab_ix39=(($22+40)&4294967295); //@line 1890 "synthesize.cpp"
                  var $23=HEAP32[(($phoneme_tab_ix39)>>2)]; //@line 1890 "synthesize.cpp"
                  __Z18SelectPhonemeTablei($23); //@line 1890 "synthesize.cpp"
                  ; //@line 1891 "synthesize.cpp"
                }
  
                var $24=HEAP32[((_translator)>>2)]; //@line 1895 "synthesize.cpp"
                var $25=HEAP32[((__ZZ15SpeakNextClauseP7__sFILEPKviE6f_text)>>2)]; //@line 1895 "synthesize.cpp"
                var $26=HEAP32[((__ZZ15SpeakNextClauseP7__sFILEPKviE6p_text)>>2)]; //@line 1895 "synthesize.cpp"
                var $call41=__Z15TranslateClauseP10TranslatorP7__sFILEPKvPiPPc($24, $25, $26, $clause_tone, $voice_change); //@line 1895 "synthesize.cpp"
                HEAP32[((__ZZ15SpeakNextClauseP7__sFILEPKviE6p_text)>>2)]=$call41; //@line 1895 "synthesize.cpp"
                var $27=HEAP32[((_translator)>>2)]; //@line 1897 "synthesize.cpp"
                var $28=HEAP32[(($clause_tone)>>2)]; //@line 1897 "synthesize.cpp"
                __Z11CalcPitchesP10Translatori($27, $28); //@line 1897 "synthesize.cpp"
                var $29=HEAP32[((_translator)>>2)]; //@line 1898 "synthesize.cpp"
                __Z11CalcLengthsP10Translator($29); //@line 1898 "synthesize.cpp"
                var $30=HEAP32[((_option_phonemes)>>2)]; //@line 1900 "synthesize.cpp"
                var $cmp42=((($30))|0) > 0; //@line 1900 "synthesize.cpp"
                var $31=HEAP32[((_phoneme_callback)>>2)]; //@line 1900 "synthesize.cpp"
                var $cmp44=((($31))|0)!=0; //@line 1900 "synthesize.cpp"
                var $or_cond2=($cmp42) | ($cmp44); //@line 1900 "synthesize.cpp"
                if ($or_cond2) { __label__ = 26;; } else { __label__ = 31;; } //@line 1900 "synthesize.cpp"
                $if_then45$$if_end62$39: do { 
                  if (__label__ == 26) {
  
                    var $32=HEAP32[((_translator)>>2)]; //@line 1902 "synthesize.cpp"
                    var $phon_out=(($32+332)&4294967295); //@line 1902 "synthesize.cpp"
                    var $arraydecay=(($phon_out)&4294967295); //@line 1902 "synthesize.cpp"
                    __Z26GetTranslatedPhonemeStringPci($arraydecay, 300); //@line 1902 "synthesize.cpp"
                    var $33=HEAP32[((_option_phonemes)>>2)]; //@line 1903 "synthesize.cpp"
                    var $cmp46=((($33))|0) > 0; //@line 1903 "synthesize.cpp"
                    if ($cmp46) { __label__ = 27;; } else { __label__ = 29;; } //@line 1903 "synthesize.cpp"
                    $if_then47$$if_end55$41: do { 
                      if (__label__ == 27) {
  
                        var $34=HEAP32[((_f_trans)>>2)]; //@line 1905 "synthesize.cpp"
                        var $35=HEAP32[((_translator)>>2)]; //@line 1905 "synthesize.cpp"
                        var $phon_out48=(($35+332)&4294967295); //@line 1905 "synthesize.cpp"
                        var $arraydecay49=(($phon_out48)&4294967295); //@line 1905 "synthesize.cpp"
                        var $call50=_fprintf($34, ((__str459)&4294967295), allocate([$arraydecay49,0,0,0], ["i8*",0,0,0], ALLOC_STACK)); //@line 1905 "synthesize.cpp"
                        var $call51=_iswalpha(269); //@line 1907 "synthesize.cpp"
                        var $tobool=((($call51))|0)!=0; //@line 1907 "synthesize.cpp"
                        if ($tobool) { __label__ = 29;break $if_then47$$if_end55$41; } //@line 1907 "synthesize.cpp"
  
                        var $36=HEAP32[((__impure_ptr)>>2)]; //@line 1910 "synthesize.cpp"
                        var $_stderr=(($36+12)&4294967295); //@line 1910 "synthesize.cpp"
                        var $37=HEAP32[(($_stderr)>>2)]; //@line 1910 "synthesize.cpp"
                        var $38=_fwrite(((__str1460)&4294967295), 88, 1, $37); //@line 1910 "synthesize.cpp"
                        ; //@line 1911 "synthesize.cpp"
                      }
                    } while(0);
  
                    var $39=HEAP32[((_phoneme_callback)>>2)]; //@line 1913 "synthesize.cpp"
                    var $cmp56=((($39))|0)!=0; //@line 1913 "synthesize.cpp"
                    if (!($cmp56)) { __label__ = 31;break $if_then45$$if_end62$39; } //@line 1913 "synthesize.cpp"
  
                    var $40=HEAP32[((_phoneme_callback)>>2)]; //@line 1915 "synthesize.cpp"
                    var $41=HEAP32[((_translator)>>2)]; //@line 1915 "synthesize.cpp"
                    var $phon_out58=(($41+332)&4294967295); //@line 1915 "synthesize.cpp"
                    var $arraydecay59=(($phon_out58)&4294967295); //@line 1915 "synthesize.cpp"
                    var $call60=FUNCTION_TABLE[$40]($arraydecay59); //@line 1915 "synthesize.cpp"
                    ; //@line 1916 "synthesize.cpp"
                  }
                } while(0);
  
                var $42=HEAP32[((_skipping_text)>>2)]; //@line 1920 "synthesize.cpp"
                var $tobool63=((($42))|0)!=0; //@line 1920 "synthesize.cpp"
                if ($tobool63) { __label__ = 32;; } else { __label__ = 33;; } //@line 1920 "synthesize.cpp"
                if (__label__ == 32) {
  
                  HEAP32[((_n_phoneme_list)>>2)]=0; //@line 1922 "synthesize.cpp"
                  $retval=1; //@line 1923 "synthesize.cpp"
                  ; //@line 1923 "synthesize.cpp"
                }
                else if (__label__ == 33) {
  
                  var $call66=__Z8GenerateP12PHONEME_LISTPii(((_phoneme_list)&4294967295), _n_phoneme_list, 0); //@line 1926 "synthesize.cpp"
                  var $43=HEAP32[(($voice_change)>>2)]; //@line 1929 "synthesize.cpp"
                  var $cmp68=((($43))|0)!=0; //@line 1929 "synthesize.cpp"
                  if ($cmp68) { __label__ = 34;; } else { __label__ = 35;; } //@line 1929 "synthesize.cpp"
                  if (__label__ == 34) {
  
                    var $44=HEAP32[(($voice_change)>>2)]; //@line 1932 "synthesize.cpp"
                    var $call70=__Z16LoadVoiceVariantPKci($44, 0); //@line 1932 "synthesize.cpp"
                    HEAP32[((__ZL9new_voice)>>2)]=$call70; //@line 1932 "synthesize.cpp"
                    __lastLabel__ = 34; ; //@line 1933 "synthesize.cpp"
                  }
                  else if (__label__ == 35) {
  
                    var $_pr=HEAP32[((__ZL9new_voice)>>2)]; //@line 1935 "synthesize.cpp"
                    __lastLabel__ = 35; ;
                  }
  
                  var $45=__lastLabel__ == 35 ? $_pr : ($call70); //@line 1935 "synthesize.cpp"
                  var $tobool72=((($45))|0)!=0; //@line 1935 "synthesize.cpp"
                  if ($tobool72) { __label__ = 37;; } else { __label__ = 38;; } //@line 1935 "synthesize.cpp"
                  if (__label__ == 37) {
  
                    var $46=HEAP32[((_voice)>>2)]; //@line 1939 "synthesize.cpp"
                    __Z13DoVoiceChangeP7voice_t($46); //@line 1939 "synthesize.cpp"
                    HEAP32[((__ZL9new_voice)>>2)]=0; //@line 1940 "synthesize.cpp"
                    ; //@line 1941 "synthesize.cpp"
                  }
  
                  $retval=1; //@line 1943 "synthesize.cpp"
                  ; //@line 1943 "synthesize.cpp"
                }
              }
            }
          }
        }
      }
    } while(0);
  
    var $47=$retval; //@line 1944 "synthesize.cpp"
    STACKTOP = __stackBase__;
    return $47; //@line 1944 "synthesize.cpp"
    return null;
  }
  

  function __Z11SynthStatusv() {
    ;
    var __label__;
  
    var $_b=HEAPU8[(__ZL8timer_on_b)];
    var $0=((($_b))&1);
    var $_b1=HEAPU8[(__ZL6paused_b)];
    var $1=($_b1) ? 2 : 0;
    var $or=($1) | ($0); //@line 1791 "synthesize.cpp"
    ;
    return $or; //@line 1791 "synthesize.cpp"
    return null;
  }
  

  function __ZL18formants_reduce_hfP7frame_ti($fr, $level) {
    ;
    var __label__;
  
    var $fr_addr;
    var $level_addr;
    var $ix;
    var $x;
    $fr_addr=$fr;
    $level_addr=$level;
    var $0=HEAP32[((_voice)>>2)]; //@line 508 "synthesize.cpp"
    var $klattv=(($0+112)&4294967295); //@line 508 "synthesize.cpp"
    var $arrayidx=(($klattv)&4294967295); //@line 508 "synthesize.cpp"
    var $1=HEAP32[(($arrayidx)>>2)]; //@line 508 "synthesize.cpp"
    var $tobool=((($1))|0)!=0; //@line 508 "synthesize.cpp"
    if ($tobool) { __label__ = 3;; } else { __label__ = 1;; } //@line 508 "synthesize.cpp"
    $for_end$$if_end$2: do { 
      if (__label__ == 1) {
  
        $ix=2; //@line 511 "synthesize.cpp"
        ; //@line 511 "synthesize.cpp"
        while(1) { 
  
          var $2=$ix; //@line 513 "synthesize.cpp"
          var $3=$fr_addr; //@line 513 "synthesize.cpp"
          var $fheight=(($3+18)&4294967295); //@line 513 "synthesize.cpp"
          var $arrayidx1=(($fheight+$2)&4294967295); //@line 513 "synthesize.cpp"
          var $4=HEAPU8[($arrayidx1)]; //@line 513 "synthesize.cpp"
          var $conv=((($4))&255); //@line 513 "synthesize.cpp"
          var $5=$level_addr; //@line 513 "synthesize.cpp"
          var $mul=((($conv)*($5))&4294967295); //@line 513 "synthesize.cpp"
          $x=$mul; //@line 513 "synthesize.cpp"
          var $6=$x; //@line 514 "synthesize.cpp"
          var $div=((((($6))|0)/100)|0); //@line 514 "synthesize.cpp"
          var $conv2=((($div)) & 255); //@line 514 "synthesize.cpp"
          var $7=$ix; //@line 514 "synthesize.cpp"
          var $8=$fr_addr; //@line 514 "synthesize.cpp"
          var $fheight3=(($8+18)&4294967295); //@line 514 "synthesize.cpp"
          var $arrayidx4=(($fheight3+$7)&4294967295); //@line 514 "synthesize.cpp"
          HEAP8[($arrayidx4)]=$conv2; //@line 514 "synthesize.cpp"
          var $9=$ix; //@line 511 "synthesize.cpp"
          var $inc=((($9)+1)&4294967295); //@line 511 "synthesize.cpp"
          $ix=$inc; //@line 511 "synthesize.cpp"
          var $cmp=((($inc))|0) < 8; //@line 511 "synthesize.cpp"
          if ($cmp) { __label__ = 2;continue ; } else { __label__ = 3;break $for_end$$if_end$2; } //@line 511 "synthesize.cpp"
        }
      }
    } while(0);
  
    ;
    return; //@line 516 "synthesize.cpp"
    return;
  }
  

  function __ZL10AllocFramev() {
    ;
    var __label__;
  
    var $0=HEAP32[((__ZZL10AllocFramevE2ix)>>2)]; //@line 447 "synthesize.cpp"
    var $inc=((($0)+1)&4294967295); //@line 447 "synthesize.cpp"
    HEAP32[((__ZZL10AllocFramevE2ix)>>2)]=$inc; //@line 447 "synthesize.cpp"
    var $cmp=((($inc))|0) >= 160; //@line 448 "synthesize.cpp"
    if ($cmp) { __label__ = 1;; } else { __label__ = 2;; } //@line 448 "synthesize.cpp"
    if (__label__ == 1) {
  
      HEAP32[((__ZZL10AllocFramevE2ix)>>2)]=0; //@line 449 "synthesize.cpp"
      ; //@line 449 "synthesize.cpp"
    }
  
    var $1=HEAP32[((__ZZL10AllocFramevE2ix)>>2)]; //@line 450 "synthesize.cpp"
    var $arrayidx=((__ZZL10AllocFramevE10frame_pool+$1*64)&4294967295); //@line 450 "synthesize.cpp"
    ;
    return $arrayidx; //@line 450 "synthesize.cpp"
    return null;
  }
  

  function __Z13SetLengthModsP10Translatori($tr, $value) {
    ;
    var __label__;
  
    var $tr_addr;
    var $value_addr;
    var $value2;
    $tr_addr=$tr;
    $value_addr=$value;
    var $0=$value_addr; //@line 363 "translate.cpp"
    var $rem=((($0))|0)%100; //@line 363 "translate.cpp"
    var $arrayidx=((__ZL15length_mod_tabs+$rem*4)&4294967295); //@line 363 "translate.cpp"
    var $1=HEAP32[(($arrayidx)>>2)]; //@line 363 "translate.cpp"
    var $2=$tr_addr; //@line 363 "translate.cpp"
    var $langopts=(($2)&4294967295); //@line 363 "translate.cpp"
    var $length_mods=(($langopts+176)&4294967295); //@line 363 "translate.cpp"
    HEAP32[(($length_mods)>>2)]=$1; //@line 363 "translate.cpp"
    var $3=$tr_addr; //@line 363 "translate.cpp"
    var $langopts1=(($3)&4294967295); //@line 363 "translate.cpp"
    var $length_mods0=(($langopts1+180)&4294967295); //@line 363 "translate.cpp"
    HEAP32[(($length_mods0)>>2)]=$1; //@line 363 "translate.cpp"
    var $4=$value_addr; //@line 364 "translate.cpp"
    var $div=((((($4))|0)/100)|0); //@line 364 "translate.cpp"
    $value2=$div; //@line 364 "translate.cpp"
    var $cmp=((($div))|0)!=0; //@line 364 "translate.cpp"
    if ($cmp) { __label__ = 1;; } else { __label__ = 2;; } //@line 364 "translate.cpp"
    if (__label__ == 1) {
  
      var $5=$value2; //@line 366 "translate.cpp"
      var $arrayidx2=((__ZL15length_mod_tabs+$5*4)&4294967295); //@line 366 "translate.cpp"
      var $6=HEAP32[(($arrayidx2)>>2)]; //@line 366 "translate.cpp"
      var $7=$tr_addr; //@line 366 "translate.cpp"
      var $langopts3=(($7)&4294967295); //@line 366 "translate.cpp"
      var $length_mods04=(($langopts3+180)&4294967295); //@line 366 "translate.cpp"
      HEAP32[(($length_mods04)>>2)]=$6; //@line 366 "translate.cpp"
      ; //@line 367 "translate.cpp"
    }
  
    ;
    return; //@line 368 "translate.cpp"
    return;
  }
  

  function __Z7IsAlphaj($c) {
    ;
    var __label__;
  
    var $retval;
    var $c_addr;
    $c_addr=$c;
    var $0=$c_addr; //@line 379 "translate.cpp"
    var $call=_iswalpha($0); //@line 379 "translate.cpp"
    var $tobool=((($call))|0)!=0; //@line 379 "translate.cpp"
    if ($tobool) { __label__ = 1;; } else { __label__ = 2;; } //@line 379 "translate.cpp"
    if (__label__ == 1) {
  
      $retval=1; //@line 380 "translate.cpp"
      ; //@line 380 "translate.cpp"
    }
    else if (__label__ == 2) {
  
      var $1=$c_addr; //@line 382 "translate.cpp"
      var $cmp=((($1))>>>0) >= 2305; //@line 382 "translate.cpp"
      var $2=$c_addr; //@line 382 "translate.cpp"
      var $cmp1=((($2))>>>0) <= 3575; //@line 382 "translate.cpp"
      var $or_cond=($cmp) & ($cmp1); //@line 382 "translate.cpp"
      var $3=$c_addr; //@line 385 "translate.cpp"
      if ($or_cond) { __label__ = 3;; } else { __label__ = 10;; } //@line 382 "translate.cpp"
      if (__label__ == 3) {
  
        var $and=($3) & 127; //@line 385 "translate.cpp"
        var $cmp3=((($and))>>>0) < 100; //@line 385 "translate.cpp"
        if ($cmp3) { __label__ = 4;; } else { __label__ = 5;; } //@line 385 "translate.cpp"
        if (__label__ == 4) {
  
          $retval=1; //@line 386 "translate.cpp"
          ; //@line 386 "translate.cpp"
        }
        else if (__label__ == 5) {
  
          var $4=$c_addr; //@line 387 "translate.cpp"
          var $call6=__Z11lookupwcharPKti(((__ZZ7IsAlphajE18extra_indic_alphas)&4294967295), $4); //@line 387 "translate.cpp"
          var $cmp7=((($call6))|0)!=0; //@line 387 "translate.cpp"
          if ($cmp7) { __label__ = 6;; } else { __label__ = 7;; } //@line 387 "translate.cpp"
          if (__label__ == 6) {
  
            $retval=1; //@line 388 "translate.cpp"
            ; //@line 388 "translate.cpp"
          }
          else if (__label__ == 7) {
  
            var $5=$c_addr; //@line 389 "translate.cpp"
            var $cmp10=((($5))>>>0) >= 3450; //@line 389 "translate.cpp"
            var $6=$c_addr; //@line 389 "translate.cpp"
            var $cmp12=((($6))>>>0) <= 3455; //@line 389 "translate.cpp"
            var $or_cond1=($cmp10) & ($cmp12); //@line 389 "translate.cpp"
            if ($or_cond1) { __label__ = 8;; } else { __label__ = 9;; } //@line 389 "translate.cpp"
            if (__label__ == 8) {
  
              $retval=1; //@line 390 "translate.cpp"
              ; //@line 390 "translate.cpp"
            }
            else if (__label__ == 9) {
  
              $retval=0; //@line 392 "translate.cpp"
              ; //@line 392 "translate.cpp"
            }
          }
        }
      }
      else if (__label__ == 10) {
  
        var $cmp16=((($3))>>>0) >= 1611; //@line 395 "translate.cpp"
        var $7=$c_addr; //@line 395 "translate.cpp"
        var $cmp18=((($7))>>>0) <= 1630; //@line 395 "translate.cpp"
        var $or_cond2=($cmp16) & ($cmp18); //@line 395 "translate.cpp"
        if ($or_cond2) { __label__ = 11;; } else { __label__ = 12;; } //@line 395 "translate.cpp"
        if (__label__ == 11) {
  
          $retval=1; //@line 396 "translate.cpp"
          ; //@line 396 "translate.cpp"
        }
        else if (__label__ == 12) {
  
          var $8=$c_addr; //@line 398 "translate.cpp"
          var $cmp21=((($8))>>>0) >= 768; //@line 398 "translate.cpp"
          var $9=$c_addr; //@line 398 "translate.cpp"
          var $cmp23=((($9))>>>0) <= 879; //@line 398 "translate.cpp"
          var $or_cond3=($cmp21) & ($cmp23); //@line 398 "translate.cpp"
          if ($or_cond3) { __label__ = 13;; } else { __label__ = 14;; } //@line 398 "translate.cpp"
          if (__label__ == 13) {
  
            $retval=1; //@line 399 "translate.cpp"
            ; //@line 399 "translate.cpp"
          }
          else if (__label__ == 14) {
  
            var $10=$c_addr; //@line 401 "translate.cpp"
            var $cmp26=((($10))>>>0) >= 1920; //@line 401 "translate.cpp"
            var $11=$c_addr; //@line 401 "translate.cpp"
            var $cmp28=((($11))>>>0) <= 1969; //@line 401 "translate.cpp"
            var $or_cond4=($cmp26) & ($cmp28); //@line 401 "translate.cpp"
            if ($or_cond4) { __label__ = 15;; } else { __label__ = 16;; } //@line 401 "translate.cpp"
            if (__label__ == 15) {
  
              $retval=1; //@line 402 "translate.cpp"
              ; //@line 402 "translate.cpp"
            }
            else if (__label__ == 16) {
  
              var $12=$c_addr; //@line 404 "translate.cpp"
              var $cmp31=((($12))>>>0) >= 4352; //@line 404 "translate.cpp"
              var $13=$c_addr; //@line 404 "translate.cpp"
              var $cmp33=((($13))>>>0) <= 4607; //@line 404 "translate.cpp"
              var $or_cond5=($cmp31) & ($cmp33); //@line 404 "translate.cpp"
              if ($or_cond5) { __label__ = 17;; } else { __label__ = 18;; } //@line 404 "translate.cpp"
              if (__label__ == 17) {
  
                $retval=1; //@line 405 "translate.cpp"
                ; //@line 405 "translate.cpp"
              }
              else if (__label__ == 18) {
  
                var $14=$c_addr; //@line 407 "translate.cpp"
                var $cmp36=((($14))>>>0) > 12352; //@line 407 "translate.cpp"
                var $15=$c_addr; //@line 407 "translate.cpp"
                var $cmp38=((($15))>>>0) <= 42752; //@line 407 "translate.cpp"
                var $or_cond6=($cmp36) & ($cmp38); //@line 407 "translate.cpp"
                if ($or_cond6) { __label__ = 19;; } else { __label__ = 20;; } //@line 407 "translate.cpp"
                if (__label__ == 19) {
  
                  $retval=1; //@line 408 "translate.cpp"
                  ; //@line 408 "translate.cpp"
                }
                else if (__label__ == 20) {
  
                  $retval=0; //@line 410 "translate.cpp"
                  ; //@line 410 "translate.cpp"
                }
              }
            }
          }
        }
      }
    }
  
    var $16=$retval; //@line 411 "translate.cpp"
    ;
    return $16; //@line 411 "translate.cpp"
    return null;
  }
  

  function __Z11lookupwcharPKti($list, $c) {
    ;
    var __label__;
  
    var $retval;
    var $list_addr;
    var $c_addr;
    var $ix;
    $list_addr=$list;
    $c_addr=$c;
    $ix=0; //@line 456 "translate.cpp"
    ; //@line 456 "translate.cpp"
    while(1) { 
  
      var $0=$ix; //@line 456 "translate.cpp"
      var $1=$list_addr; //@line 456 "translate.cpp"
      var $arrayidx=(($1+2*$0)&4294967295); //@line 456 "translate.cpp"
      var $2=HEAPU16[(($arrayidx)>>1)]; //@line 456 "translate.cpp"
      var $conv=((($2))&65535); //@line 456 "translate.cpp"
      var $cmp=((($conv))|0)!=0; //@line 456 "translate.cpp"
      if (!($cmp)) { __label__ = 5;break ; } //@line 456 "translate.cpp"
  
      var $3=$ix; //@line 458 "translate.cpp"
      var $4=$list_addr; //@line 458 "translate.cpp"
      var $arrayidx1=(($4+2*$3)&4294967295); //@line 458 "translate.cpp"
      var $5=HEAPU16[(($arrayidx1)>>1)]; //@line 458 "translate.cpp"
      var $conv2=((($5))&65535); //@line 458 "translate.cpp"
      var $6=$c_addr; //@line 458 "translate.cpp"
      var $cmp3=((($conv2))|0)==((($6))|0); //@line 458 "translate.cpp"
      var $7=$ix; //@line 459 "translate.cpp"
      var $add=((($7)+1)&4294967295); //@line 459 "translate.cpp"
      if ($cmp3) { __label__ = 3;break ; } //@line 458 "translate.cpp"
  
      $ix=$add; //@line 456 "translate.cpp"
      __label__ = 1;continue ; //@line 456 "translate.cpp"
    }
    if (__label__ == 5) {
  
      $retval=0; //@line 461 "translate.cpp"
      ; //@line 461 "translate.cpp"
    }
    else if (__label__ == 3) {
  
      $retval=$add; //@line 459 "translate.cpp"
      ; //@line 459 "translate.cpp"
    }
  
    var $8=$retval; //@line 462 "translate.cpp"
    ;
    return $8; //@line 462 "translate.cpp"
    return null;
  }
  

  function __Z7IsDigitj($c) {
    ;
    var __label__;
  
    var $retval;
    var $c_addr;
    $c_addr=$c;
    var $0=$c_addr; //@line 422 "translate.cpp"
    var $call=_iswdigit($0); //@line 422 "translate.cpp"
    var $tobool=((($call))|0)!=0; //@line 422 "translate.cpp"
    if ($tobool) { __label__ = 1;; } else { __label__ = 2;; } //@line 422 "translate.cpp"
    if (__label__ == 1) {
  
      $retval=1; //@line 423 "translate.cpp"
      ; //@line 423 "translate.cpp"
    }
    else if (__label__ == 2) {
  
      var $1=$c_addr; //@line 425 "translate.cpp"
      var $cmp=((($1))>>>0) >= 2406; //@line 425 "translate.cpp"
      var $2=$c_addr; //@line 425 "translate.cpp"
      var $cmp1=((($2))>>>0) <= 2415; //@line 425 "translate.cpp"
      var $or_cond=($cmp) & ($cmp1); //@line 425 "translate.cpp"
      if ($or_cond) { __label__ = 3;; } else { __label__ = 4;; } //@line 425 "translate.cpp"
      if (__label__ == 3) {
  
        $retval=1; //@line 426 "translate.cpp"
        ; //@line 426 "translate.cpp"
      }
      else if (__label__ == 4) {
  
        $retval=0; //@line 428 "translate.cpp"
        ; //@line 428 "translate.cpp"
      }
    }
  
    var $3=$retval; //@line 429 "translate.cpp"
    ;
    return $3; //@line 429 "translate.cpp"
    return null;
  }
  

  function __Z7IsSpacej($c) {
    ;
    var __label__;
  
    var $retval;
    var $c_addr;
    $c_addr=$c;
    var $0=$c_addr; //@line 433 "translate.cpp"
    var $cmp=((($0))|0)==0; //@line 433 "translate.cpp"
    if ($cmp) { __label__ = 1;; } else { __label__ = 2;; } //@line 433 "translate.cpp"
    if (__label__ == 1) {
  
      $retval=0; //@line 434 "translate.cpp"
      ; //@line 434 "translate.cpp"
    }
    else if (__label__ == 2) {
  
      var $1=$c_addr; //@line 435 "translate.cpp"
      var $cmp1=((($1))>>>0) >= 9472; //@line 435 "translate.cpp"
      var $2=$c_addr; //@line 435 "translate.cpp"
      var $cmp2=((($2))>>>0) < 9632; //@line 435 "translate.cpp"
      var $or_cond=($cmp1) & ($cmp2); //@line 435 "translate.cpp"
      if ($or_cond) { __label__ = 3;; } else { __label__ = 4;; } //@line 435 "translate.cpp"
      if (__label__ == 3) {
  
        $retval=1; //@line 436 "translate.cpp"
        ; //@line 436 "translate.cpp"
      }
      else if (__label__ == 4) {
  
        var $3=$c_addr; //@line 439 "translate.cpp"
        var $call=_iswspace($3); //@line 439 "translate.cpp"
        $retval=$call; //@line 439 "translate.cpp"
        ; //@line 439 "translate.cpp"
      }
    }
  
    var $4=$retval; //@line 440 "translate.cpp"
    ;
    return $4; //@line 440 "translate.cpp"
    return null;
  }
  

  function __Z16DeleteTranslatorP10Translator($tr) {
    ;
    var __label__;
  
    var $tr_addr;
    $tr_addr=$tr;
    var $0=$tr_addr; //@line 445 "translate.cpp"
    var $data_dictlist=(($0+1052)&4294967295); //@line 445 "translate.cpp"
    var $1=HEAP32[(($data_dictlist)>>2)]; //@line 445 "translate.cpp"
    var $cmp=((($1))|0)!=0; //@line 445 "translate.cpp"
    if ($cmp) { __label__ = 1;; } else { __label__ = 2;; } //@line 445 "translate.cpp"
    if (__label__ == 1) {
  
      var $2=$tr_addr; //@line 446 "translate.cpp"
      var $data_dictlist1=(($2+1052)&4294967295); //@line 446 "translate.cpp"
      var $3=HEAP32[(($data_dictlist1)>>2)]; //@line 446 "translate.cpp"
      __Z4FreePv($3); //@line 446 "translate.cpp"
      ; //@line 446 "translate.cpp"
    }
  
    var $4=$tr_addr; //@line 447 "translate.cpp"
    var $5=$4; //@line 447 "translate.cpp"
    __Z4FreePv($5); //@line 447 "translate.cpp"
    ;
    return; //@line 448 "translate.cpp"
    return;
  }
  

  function __Z9IsBracketi($c) {
    ;
    var __label__;
  
    var $retval;
    var $c_addr;
    $c_addr=$c;
    var $0=$c_addr; //@line 466 "translate.cpp"
    var $cmp=((($0))|0) >= 8212; //@line 466 "translate.cpp"
    var $1=$c_addr; //@line 466 "translate.cpp"
    var $cmp1=((($1))|0) <= 8223; //@line 466 "translate.cpp"
    var $or_cond=($cmp) & ($cmp1); //@line 466 "translate.cpp"
    if ($or_cond) { __label__ = 1;; } else { __label__ = 2;; } //@line 466 "translate.cpp"
    if (__label__ == 1) {
  
      $retval=1; //@line 467 "translate.cpp"
      ; //@line 467 "translate.cpp"
    }
    else if (__label__ == 2) {
  
      var $2=$c_addr; //@line 468 "translate.cpp"
      var $call=__Z11lookupwcharPKti(((__ZL8brackets)&4294967295), $2); //@line 468 "translate.cpp"
      $retval=$call; //@line 468 "translate.cpp"
      ; //@line 468 "translate.cpp"
    }
  
    var $3=$retval; //@line 469 "translate.cpp"
    ;
    return $3; //@line 469 "translate.cpp"
    return null;
  }
  

  function __Z8utf8_outjPc($c, $buf) {
    ;
    var __label__;
  
    var $retval;
    var $c_addr;
    var $buf_addr;
    var $n_bytes;
    var $j;
    var $shift;
    $c_addr=$c;
    $buf_addr=$buf;
    var $0=$c_addr; //@line 481 "translate.cpp"
    var $cmp=((($0))>>>0) < 128; //@line 481 "translate.cpp"
    var $1=$c_addr; //@line 483 "translate.cpp"
    if ($cmp) { __label__ = 1;; } else { __label__ = 2;; } //@line 481 "translate.cpp"
    if (__label__ == 1) {
  
      var $conv=((($1)) & 255); //@line 483 "translate.cpp"
      var $2=$buf_addr; //@line 483 "translate.cpp"
      var $arrayidx=(($2)&4294967295); //@line 483 "translate.cpp"
      HEAP8[($arrayidx)]=$conv; //@line 483 "translate.cpp"
      $retval=1; //@line 484 "translate.cpp"
      ; //@line 484 "translate.cpp"
    }
    else if (__label__ == 2) {
  
      var $cmp1=((($1))>>>0) >= 1114112; //@line 486 "translate.cpp"
      if ($cmp1) { __label__ = 3;; } else { __label__ = 4;; } //@line 486 "translate.cpp"
      if (__label__ == 3) {
  
        var $3=$buf_addr; //@line 488 "translate.cpp"
        var $arrayidx3=(($3)&4294967295); //@line 488 "translate.cpp"
        HEAP8[($arrayidx3)]=32; //@line 488 "translate.cpp"
        $retval=1; //@line 489 "translate.cpp"
        ; //@line 489 "translate.cpp"
      }
      else if (__label__ == 4) {
  
        var $4=$c_addr; //@line 491 "translate.cpp"
        var $cmp5=((($4))>>>0) < 2048; //@line 491 "translate.cpp"
        if ($cmp5) { __label__ = 5;; } else { __label__ = 6;; } //@line 491 "translate.cpp"
        if (__label__ == 5) {
  
          $n_bytes=1; //@line 492 "translate.cpp"
          ; //@line 492 "translate.cpp"
        }
        else if (__label__ == 6) {
  
          var $5=$c_addr; //@line 494 "translate.cpp"
          var $cmp7=((($5))>>>0) < 65536; //@line 494 "translate.cpp"
          if ($cmp7) { __label__ = 7;; } else { __label__ = 8;; } //@line 494 "translate.cpp"
          if (__label__ == 7) {
  
            $n_bytes=2; //@line 495 "translate.cpp"
            ; //@line 495 "translate.cpp"
          }
          else if (__label__ == 8) {
  
            $n_bytes=3; //@line 497 "translate.cpp"
            ;
          }
        }
  
        var $6=$n_bytes; //@line 499 "translate.cpp"
        var $mul=((($6)*6)&4294967295); //@line 499 "translate.cpp"
        $shift=$mul; //@line 499 "translate.cpp"
        var $7=$n_bytes; //@line 500 "translate.cpp"
        var $arrayidx12=((__ZZ8utf8_outjPcE4code+$7)&4294967295); //@line 500 "translate.cpp"
        var $8=HEAPU8[($arrayidx12)]; //@line 500 "translate.cpp"
        var $conv13=((($8))&255); //@line 500 "translate.cpp"
        var $9=$c_addr; //@line 500 "translate.cpp"
        var $10=$shift; //@line 500 "translate.cpp"
        var $shr=($9) >>> ((($10))>>>0); //@line 500 "translate.cpp"
        var $or=($shr) | ($conv13); //@line 500 "translate.cpp"
        var $conv14=((($or)) & 255); //@line 500 "translate.cpp"
        var $11=$buf_addr; //@line 500 "translate.cpp"
        var $arrayidx15=(($11)&4294967295); //@line 500 "translate.cpp"
        HEAP8[($arrayidx15)]=$conv14; //@line 500 "translate.cpp"
        $j=0; //@line 501 "translate.cpp"
        var $12=$j; //@line 501 "translate.cpp"
        var $13=$n_bytes; //@line 501 "translate.cpp"
        var $cmp161=((($12))|0) < ((($13))|0); //@line 501 "translate.cpp"
        if ($cmp161) { __label__ = 10;; } else { __label__ = 11;; } //@line 501 "translate.cpp"
        $for_body$$for_end$15: do { 
          if (__label__ == 10) {
            while(1) { 
  
              var $14=$shift; //@line 503 "translate.cpp"
              var $sub=((($14)-6)&4294967295); //@line 503 "translate.cpp"
              $shift=$sub; //@line 503 "translate.cpp"
              var $15=$c_addr; //@line 504 "translate.cpp"
              var $16=$shift; //@line 504 "translate.cpp"
              var $shr17=($15) >>> ((($16))>>>0); //@line 504 "translate.cpp"
              var $and=($shr17) & 63; //@line 504 "translate.cpp"
              var $add=((($and)+128)&4294967295); //@line 504 "translate.cpp"
              var $conv18=((($add)) & 255); //@line 504 "translate.cpp"
              var $17=$j; //@line 504 "translate.cpp"
              var $add19=((($17)+1)&4294967295); //@line 504 "translate.cpp"
              var $18=$buf_addr; //@line 504 "translate.cpp"
              var $arrayidx20=(($18+$add19)&4294967295); //@line 504 "translate.cpp"
              HEAP8[($arrayidx20)]=$conv18; //@line 504 "translate.cpp"
              var $19=$j; //@line 501 "translate.cpp"
              var $inc=((($19)+1)&4294967295); //@line 501 "translate.cpp"
              $j=$inc; //@line 501 "translate.cpp"
              var $20=$j; //@line 501 "translate.cpp"
              var $21=$n_bytes; //@line 501 "translate.cpp"
              var $cmp16=((($20))|0) < ((($21))|0); //@line 501 "translate.cpp"
              if ($cmp16) { __label__ = 10;continue ; } else { __label__ = 11;break $for_body$$for_end$15; } //@line 501 "translate.cpp"
            }
          }
        } while(0);
  
        var $22=$n_bytes; //@line 506 "translate.cpp"
        var $add21=((($22)+1)&4294967295); //@line 506 "translate.cpp"
        $retval=$add21; //@line 506 "translate.cpp"
        ; //@line 506 "translate.cpp"
      }
    }
  
    var $23=$retval; //@line 507 "translate.cpp"
    ;
    return $23; //@line 507 "translate.cpp"
    return null;
  }
  

  function __Z11utf8_nbytesPKc($buf) {
    ;
    var __label__;
  
    var $retval;
    var $buf_addr;
    var $c;
    $buf_addr=$buf;
    var $0=$buf_addr; //@line 513 "translate.cpp"
    var $arrayidx=(($0)&4294967295); //@line 513 "translate.cpp"
    var $1=HEAP8[($arrayidx)]; //@line 513 "translate.cpp"
    $c=$1; //@line 513 "translate.cpp"
    var $2=$c; //@line 514 "translate.cpp"
    var $conv=((($2))&255); //@line 514 "translate.cpp"
    var $cmp=((($conv))|0) < 128; //@line 514 "translate.cpp"
    if ($cmp) { __label__ = 1;; } else { __label__ = 2;; } //@line 514 "translate.cpp"
    if (__label__ == 1) {
  
      $retval=1; //@line 515 "translate.cpp"
      ; //@line 515 "translate.cpp"
    }
    else if (__label__ == 2) {
  
      var $3=$c; //@line 516 "translate.cpp"
      var $conv1=((($3))&255); //@line 516 "translate.cpp"
      var $cmp2=((($conv1))|0) < 224; //@line 516 "translate.cpp"
      if ($cmp2) { __label__ = 3;; } else { __label__ = 4;; } //@line 516 "translate.cpp"
      if (__label__ == 3) {
  
        $retval=2; //@line 517 "translate.cpp"
        ; //@line 517 "translate.cpp"
      }
      else if (__label__ == 4) {
  
        var $4=$c; //@line 518 "translate.cpp"
        var $conv5=((($4))&255); //@line 518 "translate.cpp"
        var $cmp6=((($conv5))|0) < 240; //@line 518 "translate.cpp"
        if ($cmp6) { __label__ = 5;; } else { __label__ = 6;; } //@line 518 "translate.cpp"
        if (__label__ == 5) {
  
          $retval=3; //@line 519 "translate.cpp"
          ; //@line 519 "translate.cpp"
        }
        else if (__label__ == 6) {
  
          $retval=4; //@line 520 "translate.cpp"
          ; //@line 520 "translate.cpp"
        }
      }
    }
  
    var $5=$retval; //@line 521 "translate.cpp"
    ;
    return $5; //@line 521 "translate.cpp"
    return null;
  }
  

  function __Z8utf8_in2PiPKci($c, $buf, $backwards) {
    ;
    var __label__;
  
    var $c_addr;
    var $buf_addr;
    var $backwards_addr;
    var $c1;
    var $n_bytes;
    var $ix;
    $c_addr=$c;
    $buf_addr=$buf;
    $backwards_addr=$backwards;
    var $0=$buf_addr; //@line 535 "translate.cpp"
    var $1=HEAP8[($0)]; //@line 535 "translate.cpp"
    var $conv2=(tempInt=(($1)),(tempInt>=128?tempInt-256:tempInt)); //@line 535 "translate.cpp"
    var $and3=($conv2) & 192; //@line 535 "translate.cpp"
    var $cmp4=((($and3))|0)==128; //@line 535 "translate.cpp"
    if ($cmp4) { __label__ = 1;; } else { __label__ = 5;; } //@line 535 "translate.cpp"
    $while_body$$while_end$2: do { 
      if (__label__ == 1) {
        while(1) { 
  
          var $2=$backwards_addr; //@line 538 "translate.cpp"
          var $tobool=((($2))|0)!=0; //@line 538 "translate.cpp"
          var $3=$buf_addr; //@line 539 "translate.cpp"
          if ($tobool) { __label__ = 2;; } else { __label__ = 3;; } //@line 538 "translate.cpp"
          if (__label__ == 2) {
  
            var $incdec_ptr=(($3+-1)&4294967295); //@line 539 "translate.cpp"
            $buf_addr=$incdec_ptr; //@line 539 "translate.cpp"
            ; //@line 539 "translate.cpp"
          }
          else if (__label__ == 3) {
  
            var $incdec_ptr1=(($3+1)&4294967295); //@line 541 "translate.cpp"
            $buf_addr=$incdec_ptr1; //@line 541 "translate.cpp"
            ;
          }
  
          var $4=$buf_addr; //@line 535 "translate.cpp"
          var $5=HEAP8[($4)]; //@line 535 "translate.cpp"
          var $conv=(tempInt=(($5)),(tempInt>=128?tempInt-256:tempInt)); //@line 535 "translate.cpp"
          var $and=($conv) & 192; //@line 535 "translate.cpp"
          var $cmp=((($and))|0)==128; //@line 535 "translate.cpp"
          if ($cmp) { __label__ = 1;continue ; } else { __label__ = 5;break $while_body$$while_end$2; } //@line 535 "translate.cpp"
        }
      }
    } while(0);
  
    $n_bytes=0; //@line 544 "translate.cpp"
    var $6=$buf_addr; //@line 546 "translate.cpp"
    var $incdec_ptr2=(($6+1)&4294967295); //@line 546 "translate.cpp"
    $buf_addr=$incdec_ptr2; //@line 546 "translate.cpp"
    var $7=HEAP8[($6)]; //@line 546 "translate.cpp"
    var $conv3=(tempInt=(($7)),(tempInt>=128?tempInt-256:tempInt)); //@line 546 "translate.cpp"
    $c1=$conv3; //@line 546 "translate.cpp"
    var $and4=($conv3) & 128; //@line 546 "translate.cpp"
    var $tobool5=((($and4))|0)!=0; //@line 546 "translate.cpp"
    if ($tobool5) { __label__ = 6;; } else { __label__ = 14;; } //@line 546 "translate.cpp"
    $if_then6$$if_end27$10: do { 
      if (__label__ == 6) {
  
        var $8=$c1; //@line 548 "translate.cpp"
        var $and7=($8) & 224; //@line 548 "translate.cpp"
        var $cmp8=((($and7))|0)==192; //@line 548 "translate.cpp"
        if ($cmp8) { __label__ = 7;; } else { __label__ = 8;; } //@line 548 "translate.cpp"
        $if_then9$$if_else10$12: do { 
          if (__label__ == 7) {
  
            $n_bytes=1; //@line 549 "translate.cpp"
            ; //@line 549 "translate.cpp"
          }
          else if (__label__ == 8) {
  
            var $9=$c1; //@line 551 "translate.cpp"
            var $and11=($9) & 240; //@line 551 "translate.cpp"
            var $cmp12=((($and11))|0)==224; //@line 551 "translate.cpp"
            if ($cmp12) { __label__ = 9;; } else { __label__ = 10;; } //@line 551 "translate.cpp"
            if (__label__ == 9) {
  
              $n_bytes=2; //@line 552 "translate.cpp"
              ; //@line 552 "translate.cpp"
            }
            else if (__label__ == 10) {
  
              var $10=$c1; //@line 554 "translate.cpp"
              var $and15=($10) & 248; //@line 554 "translate.cpp"
              var $cmp16=((($and15))|0)==240; //@line 554 "translate.cpp"
              if (!($cmp16)) { __label__ = 12;break $if_then9$$if_else10$12; } //@line 554 "translate.cpp"
  
              $n_bytes=3; //@line 555 "translate.cpp"
              ; //@line 555 "translate.cpp"
            }
          }
        } while(0);
  
        var $11=$n_bytes; //@line 557 "translate.cpp"
        var $arrayidx=((__ZZ8utf8_in2PiPKciE4mask+$11)&4294967295); //@line 557 "translate.cpp"
        var $12=HEAPU8[($arrayidx)]; //@line 557 "translate.cpp"
        var $conv21=((($12))&255); //@line 557 "translate.cpp"
        var $13=$c1; //@line 557 "translate.cpp"
        var $and22=($13) & ($conv21); //@line 557 "translate.cpp"
        $c1=$and22; //@line 557 "translate.cpp"
        $ix=0; //@line 558 "translate.cpp"
        var $14=$ix; //@line 558 "translate.cpp"
        var $15=$n_bytes; //@line 558 "translate.cpp"
        var $cmp231=((($14))|0) < ((($15))|0); //@line 558 "translate.cpp"
        if (!($cmp231)) { __label__ = 14;break $if_then6$$if_end27$10; } //@line 558 "translate.cpp"
        while(1) { 
  
          var $16=$c1; //@line 560 "translate.cpp"
          var $shl=((($16)*64)&4294967295); //@line 560 "translate.cpp"
          var $17=$buf_addr; //@line 560 "translate.cpp"
          var $incdec_ptr24=(($17+1)&4294967295); //@line 560 "translate.cpp"
          $buf_addr=$incdec_ptr24; //@line 560 "translate.cpp"
          var $18=HEAP8[($17)]; //@line 560 "translate.cpp"
          var $conv25=(tempInt=(($18)),(tempInt>=128?tempInt-256:tempInt)); //@line 560 "translate.cpp"
          var $and26=($conv25) & 63; //@line 560 "translate.cpp"
          var $add=((($and26)+($shl))&4294967295); //@line 560 "translate.cpp"
          $c1=$add; //@line 560 "translate.cpp"
          var $19=$ix; //@line 558 "translate.cpp"
          var $inc=((($19)+1)&4294967295); //@line 558 "translate.cpp"
          $ix=$inc; //@line 558 "translate.cpp"
          var $20=$ix; //@line 558 "translate.cpp"
          var $21=$n_bytes; //@line 558 "translate.cpp"
          var $cmp23=((($20))|0) < ((($21))|0); //@line 558 "translate.cpp"
          if ($cmp23) { __label__ = 13;continue ; } else { __label__ = 14;break $if_then6$$if_end27$10; } //@line 558 "translate.cpp"
        }
      }
    } while(0);
  
    var $22=$c1; //@line 563 "translate.cpp"
    var $23=$c_addr; //@line 563 "translate.cpp"
    HEAP32[(($23)>>2)]=$22; //@line 563 "translate.cpp"
    var $24=$n_bytes; //@line 564 "translate.cpp"
    var $add28=((($24)+1)&4294967295); //@line 564 "translate.cpp"
    ;
    return $add28; //@line 564 "translate.cpp"
    return null;
  }
  

  function __Z7utf8_inPiPKc($c, $buf) {
    ;
    var __label__;
  
    var $c_addr;
    var $buf_addr;
    $c_addr=$c;
    $buf_addr=$buf;
    var $0=$c_addr; //@line 572 "translate.cpp"
    var $1=$buf_addr; //@line 572 "translate.cpp"
    var $call=__Z8utf8_in2PiPKci($0, $1, 0); //@line 572 "translate.cpp"
    ;
    return $call; //@line 572 "translate.cpp"
    return null;
  }
  

  function __Z8strchr_wPKci($s, $c) {
    ;
    var __label__;
  
    var $retval;
    var $s_addr;
    var $c_addr;
    $s_addr=$s;
    $c_addr=$c;
    var $0=$c_addr; //@line 579 "translate.cpp"
    var $cmp=((($0))|0) >= 128; //@line 579 "translate.cpp"
    if ($cmp) { __label__ = 1;; } else { __label__ = 2;; } //@line 579 "translate.cpp"
    if (__label__ == 1) {
  
      $retval=0; //@line 580 "translate.cpp"
      ; //@line 580 "translate.cpp"
    }
    else if (__label__ == 2) {
  
      var $1=$s_addr; //@line 581 "translate.cpp"
      var $2=$c_addr; //@line 581 "translate.cpp"
      var $call=_strchr($1, $2); //@line 581 "translate.cpp"
      $retval=$call; //@line 581 "translate.cpp"
      ; //@line 581 "translate.cpp"
    }
  
    var $3=$retval; //@line 582 "translate.cpp"
    ;
    return $3; //@line 582 "translate.cpp"
    return null;
  }
  

  function __Z10IsAllUpperPKc($word) {
    var __stackBase__  = STACKTOP; STACKTOP += 4; _memset(__stackBase__, 0, 4);
    var __label__;
  
    var $retval;
    var $word_addr;
    var $c=__stackBase__;
    $word_addr=$word;
    ; //@line 588 "translate.cpp"
    while(1) { 
  
      var $0=$word_addr; //@line 588 "translate.cpp"
      var $1=HEAP8[($0)]; //@line 588 "translate.cpp"
      var $conv=(tempInt=(($1)),(tempInt>=128?tempInt-256:tempInt)); //@line 588 "translate.cpp"
      var $cmp=((($conv))|0)!=0; //@line 588 "translate.cpp"
      if (!($cmp)) { __label__ = 5;break ; } //@line 588 "translate.cpp"
  
      var $2=$word_addr; //@line 588 "translate.cpp"
      var $3=HEAP8[($2)]; //@line 588 "translate.cpp"
      var $conv1=(tempInt=(($3)),(tempInt>=128?tempInt-256:tempInt)); //@line 588 "translate.cpp"
      var $call=__Z8isspace2j($conv1); //@line 588 "translate.cpp"
      var $tobool=((($call))|0)!=0; //@line 588 "translate.cpp"
      var $lnot=($tobool) ^ 1; //@line 588 "translate.cpp"
      if (!($lnot)) { __label__ = 5;break ; }
  
      var $4=$word_addr; //@line 590 "translate.cpp"
      var $call2=__Z7utf8_inPiPKc($c, $4); //@line 590 "translate.cpp"
      var $5=$word_addr; //@line 590 "translate.cpp"
      var $add_ptr=(($5+$call2)&4294967295); //@line 590 "translate.cpp"
      $word_addr=$add_ptr; //@line 590 "translate.cpp"
      var $6=HEAP32[(($c)>>2)]; //@line 591 "translate.cpp"
      var $call3=_iswupper($6); //@line 591 "translate.cpp"
      var $tobool4=((($call3))|0)!=0; //@line 591 "translate.cpp"
      if ($tobool4) { __label__ = 1;continue ; } else { __label__ = 4;break ; } //@line 591 "translate.cpp"
    }
    if (__label__ == 5) {
  
      $retval=1; //@line 594 "translate.cpp"
      ; //@line 594 "translate.cpp"
    }
    else if (__label__ == 4) {
  
      $retval=0; //@line 592 "translate.cpp"
      ; //@line 592 "translate.cpp"
    }
  
    var $7=$retval; //@line 595 "translate.cpp"
    STACKTOP = __stackBase__;
    return $7; //@line 595 "translate.cpp"
    return null;
  }
  

  function __Z24ChangeEquivalentPhonemesP10TranslatoriPc($tr, $lang2, $phonemes) {
    var __stackBase__  = STACKTOP; STACKTOP += 160; _memset(__stackBase__, 0, 160);
    var __label__;
  
    var $retval;
    var $tr_addr;
    var $lang2_addr;
    var $phonemes_addr;
    var $ix;
    var $len;
    var $phon;
    var $p;
    var $pb;
    var $eqlist;
    var $p_out;
    var $p_in;
    var $remove_stress;
    var $phonbuf=__stackBase__;
    $tr_addr=$tr;
    $lang2_addr=$lang2;
    $phonemes_addr=$phonemes;
    $remove_stress=0; //@line 707 "translate.cpp"
    var $0=$tr_addr; //@line 711 "translate.cpp"
    var $phoneme_tab_ix=(($0+656)&4294967295); //@line 711 "translate.cpp"
    var $1=HEAP32[(($phoneme_tab_ix)>>2)]; //@line 711 "translate.cpp"
    var $arrayidx=((_phoneme_tab_list+$1*48)&4294967295); //@line 711 "translate.cpp"
    var $equivalence_tables=(($arrayidx+44)&4294967295); //@line 711 "translate.cpp"
    var $2=HEAP32[(($equivalence_tables)>>2)]; //@line 711 "translate.cpp"
    $ix=$2; //@line 711 "translate.cpp"
    var $cmp=((($2))|0)==0; //@line 711 "translate.cpp"
    if ($cmp) { __label__ = 1;; } else { __label__ = 2;; } //@line 711 "translate.cpp"
    if (__label__ == 1) {
  
      $retval=0; //@line 712 "translate.cpp"
      ; //@line 712 "translate.cpp"
    }
    else if (__label__ == 2) {
  
      var $3=$ix; //@line 714 "translate.cpp"
      var $4=HEAP32[((_phondata_ptr)>>2)]; //@line 714 "translate.cpp"
      var $arrayidx1=(($4+$3)&4294967295); //@line 714 "translate.cpp"
      $pb=$arrayidx1; //@line 714 "translate.cpp"
      ; //@line 716 "translate.cpp"
      $for_cond$5: while(1) { 
  
        var $5=$pb; //@line 718 "translate.cpp"
        var $arrayidx2=(($5)&4294967295); //@line 718 "translate.cpp"
        var $6=HEAPU8[($arrayidx2)]; //@line 718 "translate.cpp"
        var $conv=((($6))&255); //@line 718 "translate.cpp"
        var $cmp3=((($conv))|0)==0; //@line 718 "translate.cpp"
        if ($cmp3) { __label__ = 4;break $for_cond$5; } //@line 718 "translate.cpp"
  
        var $7=$pb; //@line 721 "translate.cpp"
        var $arrayidx6=(($7)&4294967295); //@line 721 "translate.cpp"
        var $8=HEAPU8[($arrayidx6)]; //@line 721 "translate.cpp"
        var $conv7=((($8))&255); //@line 721 "translate.cpp"
        var $9=$lang2_addr; //@line 721 "translate.cpp"
        var $cmp8=((($conv7))|0)==((($9))|0); //@line 721 "translate.cpp"
        var $10=$pb; //@line 727 "translate.cpp"
        if ($cmp8) { __label__ = 7;break $for_cond$5; } //@line 721 "translate.cpp"
  
        var $arrayidx11=(($10+2)&4294967295); //@line 724 "translate.cpp"
        var $11=HEAPU8[($arrayidx11)]; //@line 724 "translate.cpp"
        var $conv12=((($11))&255); //@line 724 "translate.cpp"
        var $shl=((($conv12)*256)&4294967295); //@line 724 "translate.cpp"
        var $12=$pb; //@line 724 "translate.cpp"
        var $arrayidx13=(($12+3)&4294967295); //@line 724 "translate.cpp"
        var $13=HEAPU8[($arrayidx13)]; //@line 724 "translate.cpp"
        var $conv14=((($13))&255); //@line 724 "translate.cpp"
        var $add=((($conv14)+($shl))&4294967295); //@line 724 "translate.cpp"
        $len=$add; //@line 724 "translate.cpp"
        var $14=$len; //@line 725 "translate.cpp"
        var $mul=((($14)*4)&4294967295); //@line 725 "translate.cpp"
        var $15=$pb; //@line 725 "translate.cpp"
        var $add_ptr=(($15+$mul)&4294967295); //@line 725 "translate.cpp"
        $pb=$add_ptr; //@line 725 "translate.cpp"
        __label__ = 3;continue $for_cond$5; //@line 726 "translate.cpp"
      }
      if (__label__ == 4) {
  
        $retval=0; //@line 719 "translate.cpp"
        ; //@line 719 "translate.cpp"
      }
      else if (__label__ == 7) {
  
        var $arrayidx15=(($10+1)&4294967295); //@line 727 "translate.cpp"
        var $16=HEAPU8[($arrayidx15)]; //@line 727 "translate.cpp"
        var $conv16=((($16))&255); //@line 727 "translate.cpp"
        $remove_stress=$conv16; //@line 727 "translate.cpp"
        var $17=HEAP32[((_option_phonemes)>>2)]; //@line 729 "translate.cpp"
        var $cmp17=((($17))|0)==2; //@line 729 "translate.cpp"
        if ($cmp17) { __label__ = 8;; } else { __label__ = 9;; } //@line 729 "translate.cpp"
        if (__label__ == 8) {
  
          var $18=$phonemes_addr; //@line 731 "translate.cpp"
          var $arraydecay=(($phonbuf)&4294967295); //@line 731 "translate.cpp"
          __Z14DecodePhonemesPKcPc($18, $arraydecay); //@line 731 "translate.cpp"
          var $19=HEAP32[((_f_trans)>>2)]; //@line 732 "translate.cpp"
          var $20=$lang2_addr; //@line 732 "translate.cpp"
          var $arrayidx19=((_phoneme_tab_list+$20*48)&4294967295); //@line 732 "translate.cpp"
          var $name=(($arrayidx19)&4294967295); //@line 732 "translate.cpp"
          var $arraydecay20=(($name)&4294967295); //@line 732 "translate.cpp"
          var $arraydecay21=(($phonbuf)&4294967295); //@line 732 "translate.cpp"
          var $21=$tr_addr; //@line 732 "translate.cpp"
          var $phoneme_tab_ix22=(($21+656)&4294967295); //@line 732 "translate.cpp"
          var $22=HEAP32[(($phoneme_tab_ix22)>>2)]; //@line 732 "translate.cpp"
          var $arrayidx23=((_phoneme_tab_list+$22*48)&4294967295); //@line 732 "translate.cpp"
          var $name24=(($arrayidx23)&4294967295); //@line 732 "translate.cpp"
          var $arraydecay25=(($name24)&4294967295); //@line 732 "translate.cpp"
          var $call=_fprintf($19, ((__str543)&4294967295), allocate([$arraydecay20,0,0,0,$arraydecay21,0,0,0,$arraydecay25,0,0,0], ["i8*",0,0,0,"i8*",0,0,0,"i8*",0,0,0], ALLOC_STACK)); //@line 732 "translate.cpp"
          ; //@line 733 "translate.cpp"
        }
  
        var $23=$phonemes_addr; //@line 735 "translate.cpp"
        $p_in=$23; //@line 735 "translate.cpp"
        var $24=$pb; //@line 736 "translate.cpp"
        var $arrayidx27=(($24+8)&4294967295); //@line 736 "translate.cpp"
        $eqlist=$arrayidx27; //@line 736 "translate.cpp"
        var $arraydecay28=(($phonbuf)&4294967295); //@line 737 "translate.cpp"
        $p_out=$arraydecay28; //@line 737 "translate.cpp"
        var $25=$p_in; //@line 739 "translate.cpp"
        var $incdec_ptr1=(($25+1)&4294967295); //@line 739 "translate.cpp"
        $p_in=$incdec_ptr1; //@line 739 "translate.cpp"
        var $26=HEAP8[($25)]; //@line 739 "translate.cpp"
        $phon=$26; //@line 739 "translate.cpp"
        var $conv292=(tempInt=(($26)),(tempInt>=128?tempInt-256:tempInt)); //@line 739 "translate.cpp"
        var $cmp303=((($conv292))|0)!=0; //@line 739 "translate.cpp"
        if ($cmp303) { __label__ = 10;; } else { __label__ = 20;; } //@line 739 "translate.cpp"
        $while_body$$while_end56$15: do { 
          if (__label__ == 10) {
            $while_body$16: while(1) { 
  
              var $27=$remove_stress; //@line 741 "translate.cpp"
              var $tobool=((($27))|0)!=0; //@line 741 "translate.cpp"
              if ($tobool) { __label__ = 11;; } else { __label__ = 13;; } //@line 741 "translate.cpp"
              $land_lhs_true$$if_end34$18: do { 
                if (__label__ == 11) {
  
                  var $28=$phon; //@line 741 "translate.cpp"
                  var $conv31=(tempInt=(($28)),(tempInt>=128?tempInt-256:tempInt)); //@line 741 "translate.cpp"
                  var $and=($conv31) & 255; //@line 741 "translate.cpp"
                  var $cmp32=((($and))|0) < 8; //@line 741 "translate.cpp"
                  if ($cmp32) { __label__ = 12;break $land_lhs_true$$if_end34$18; } else { __label__ = 13;break $land_lhs_true$$if_end34$18; } //@line 741 "translate.cpp"
                }
              } while(0);
              $while_cond_backedge$$if_end34$20: do { 
                if (__label__ == 13) {
  
                  var $31=$eqlist; //@line 745 "translate.cpp"
                  $p=$31; //@line 745 "translate.cpp"
                  ; //@line 746 "translate.cpp"
                  while(1) { 
  
                    var $32=$p; //@line 746 "translate.cpp"
                    var $33=HEAP8[($32)]; //@line 746 "translate.cpp"
                    var $conv36=(tempInt=(($33)),(tempInt>=128?tempInt-256:tempInt)); //@line 746 "translate.cpp"
                    var $cmp37=((($conv36))|0)!=0; //@line 746 "translate.cpp"
                    if (!($cmp37)) { __label__ = 18;break ; } //@line 746 "translate.cpp"
  
                    var $34=$p; //@line 748 "translate.cpp"
                    var $arrayidx39=(($34+1)&4294967295); //@line 748 "translate.cpp"
                    var $call40=_strlen($arrayidx39); //@line 748 "translate.cpp"
                    $len=$call40; //@line 748 "translate.cpp"
                    var $35=$p; //@line 749 "translate.cpp"
                    var $36=HEAP8[($35)]; //@line 749 "translate.cpp"
                    var $conv41=(tempInt=(($36)),(tempInt>=128?tempInt-256:tempInt)); //@line 749 "translate.cpp"
                    var $37=$phon; //@line 749 "translate.cpp"
                    var $conv42=(tempInt=(($37)),(tempInt>=128?tempInt-256:tempInt)); //@line 749 "translate.cpp"
                    var $cmp43=((($conv41))|0)==((($conv42))|0); //@line 749 "translate.cpp"
                    if ($cmp43) { __label__ = 16;break ; } //@line 749 "translate.cpp"
  
                    var $42=$len; //@line 755 "translate.cpp"
                    var $add49=((($42)+2)&4294967295); //@line 755 "translate.cpp"
                    var $43=$p; //@line 755 "translate.cpp"
                    var $add_ptr50=(($43+$add49)&4294967295); //@line 755 "translate.cpp"
                    $p=$add_ptr50; //@line 755 "translate.cpp"
                    __label__ = 14;continue ; //@line 756 "translate.cpp"
                  }
                  if (__label__ == 16) {
  
                    var $38=$p_out; //@line 751 "translate.cpp"
                    var $39=$p; //@line 751 "translate.cpp"
                    var $arrayidx45=(($39+1)&4294967295); //@line 751 "translate.cpp"
                    var $call46=_strcpy($38, $arrayidx45); //@line 751 "translate.cpp"
                    var $40=$len; //@line 752 "translate.cpp"
                    var $41=$p_out; //@line 752 "translate.cpp"
                    var $add_ptr47=(($41+$40)&4294967295); //@line 752 "translate.cpp"
                    $p_out=$add_ptr47; //@line 752 "translate.cpp"
                    ; //@line 753 "translate.cpp"
                  }
  
                  var $44=$p; //@line 757 "translate.cpp"
                  var $45=HEAP8[($44)]; //@line 757 "translate.cpp"
                  var $conv51=(tempInt=(($45)),(tempInt>=128?tempInt-256:tempInt)); //@line 757 "translate.cpp"
                  var $cmp52=((($conv51))|0)==0; //@line 757 "translate.cpp"
                  if (!($cmp52)) { __label__ = 12;break $while_cond_backedge$$if_end34$20; } //@line 757 "translate.cpp"
  
                  var $46=$phon; //@line 760 "translate.cpp"
                  var $47=$p_out; //@line 760 "translate.cpp"
                  var $incdec_ptr54=(($47+1)&4294967295); //@line 760 "translate.cpp"
                  $p_out=$incdec_ptr54; //@line 760 "translate.cpp"
                  HEAP8[($47)]=$46; //@line 760 "translate.cpp"
                  ; //@line 761 "translate.cpp"
                }
              } while(0);
  
              var $29=$p_in; //@line 739 "translate.cpp"
              var $incdec_ptr=(($29+1)&4294967295); //@line 739 "translate.cpp"
              $p_in=$incdec_ptr; //@line 739 "translate.cpp"
              var $30=HEAP8[($29)]; //@line 739 "translate.cpp"
              $phon=$30; //@line 739 "translate.cpp"
              var $conv29=(tempInt=(($30)),(tempInt>=128?tempInt-256:tempInt)); //@line 739 "translate.cpp"
              var $cmp30=((($conv29))|0)!=0; //@line 739 "translate.cpp"
              if ($cmp30) { __label__ = 10;continue $while_body$16; } else { __label__ = 20;break $while_body$$while_end56$15; } //@line 739 "translate.cpp"
            }
          }
        } while(0);
  
        var $48=$p_out; //@line 763 "translate.cpp"
        HEAP8[($48)]=0; //@line 763 "translate.cpp"
        var $49=$remove_stress; //@line 765 "translate.cpp"
        var $tobool57=((($49))|0)!=0; //@line 765 "translate.cpp"
        if ($tobool57) { __label__ = 21;; } else { __label__ = 22;; } //@line 765 "translate.cpp"
        if (__label__ == 21) {
  
          var $50=$tr_addr; //@line 767 "translate.cpp"
          var $arraydecay59=(($phonbuf)&4294967295); //@line 767 "translate.cpp"
          __Z13SetWordStressP10TranslatorPcPjii($50, $arraydecay59, 0, -1, 0); //@line 767 "translate.cpp"
          ; //@line 768 "translate.cpp"
        }
  
        var $51=$phonemes_addr; //@line 770 "translate.cpp"
        var $arraydecay61=(($phonbuf)&4294967295); //@line 770 "translate.cpp"
        var $call62=_strcpy($51, $arraydecay61); //@line 770 "translate.cpp"
        var $52=HEAP32[((_option_phonemes)>>2)]; //@line 772 "translate.cpp"
        var $cmp63=((($52))|0)==2; //@line 772 "translate.cpp"
        if ($cmp63) { __label__ = 23;; } else { __label__ = 24;; } //@line 772 "translate.cpp"
        if (__label__ == 23) {
  
          var $53=$tr_addr; //@line 774 "translate.cpp"
          var $phoneme_tab_ix65=(($53+656)&4294967295); //@line 774 "translate.cpp"
          var $54=HEAP32[(($phoneme_tab_ix65)>>2)]; //@line 774 "translate.cpp"
          __Z18SelectPhonemeTablei($54); //@line 774 "translate.cpp"
          var $55=$phonemes_addr; //@line 775 "translate.cpp"
          var $arraydecay66=(($phonbuf)&4294967295); //@line 775 "translate.cpp"
          __Z14DecodePhonemesPKcPc($55, $arraydecay66); //@line 775 "translate.cpp"
          var $56=HEAP32[((_f_trans)>>2)]; //@line 776 "translate.cpp"
          var $arraydecay67=(($phonbuf)&4294967295); //@line 776 "translate.cpp"
          var $call68=_fprintf($56, ((__str1544)&4294967295), allocate([$arraydecay67,0,0,0], ["i8*",0,0,0], ALLOC_STACK)); //@line 776 "translate.cpp"
          ; //@line 777 "translate.cpp"
        }
  
        $retval=1; //@line 778 "translate.cpp"
        ; //@line 778 "translate.cpp"
      }
    }
  
    var $57=$retval; //@line 779 "translate.cpp"
    STACKTOP = __stackBase__;
    return $57; //@line 779 "translate.cpp"
    return null;
  }
  

  function __Z13TranslateWordP10TranslatorPciP8WORD_TAB($tr, $word_start, $next_pause, $wtab) {
    var __stackBase__  = STACKTOP; STACKTOP += 1656; _memset(__stackBase__, 0, 1656);
    var __label__;
    var __lastLabel__ = null;
  
    var $retval;
    var $tr_addr;
    var $word_start_addr;
    var $next_pause_addr;
    var $wtab_addr;
    var $word1=__stackBase__;
    var $word_length;
    var $ix;
    var $p;
    var $pfix;
    var $n_chars;
    var $dictionary_flags=__stackBase__+4;
    var $dictionary_flags2=__stackBase__+12;
    var $end_type;
    var $prefix_type;
    var $prefix_stress;
    var $wordx=__stackBase__+20;
    var $phonemes=__stackBase__+24;
    var $ph_limit;
    var $prefix_phonemes=__stackBase__+184;
    var $unpron_phonemes=__stackBase__+344;
    var $end_phonemes=__stackBase__+504;
    var $word_copy=__stackBase__+664;
    var $word_copy2=__stackBase__+824;
    var $word_copy_length;
    var $prefix_chars=__stackBase__+984;
    var $found;
    var $end_flags;
    var $c_temp;
    var $first_char=__stackBase__+1052;
    var $last_char=__stackBase__+1056;
    var $add_plural_suffix;
    var $prefix_flags;
    var $confirm_prefix;
    var $spell_word;
    var $stress_bits;
    var $emphasize_allcaps;
    var $wflags;
    var $wmark;
    var $was_unpronouncable;
    var $wtab_null=__stackBase__+1060;
    var $posn;
    var $non_initial;
    var $length;
    var $wc=__stackBase__+1156;
    var $loopcount;
    var $end2;
    var $phonemes2=__stackBase__+1160;
    var $end_phonemes2=__stackBase__+1320;
    var $wordpf=__stackBase__+1480;
    var $prefix_phonemes2=__stackBase__+1484;
    var $phonemes2482=__stackBase__+1496;
    var $p659;
    $tr_addr=$tr;
    $word_start_addr=$word_start;
    $next_pause_addr=$next_pause;
    $wtab_addr=$wtab;
    $end_type=0; //@line 796 "translate.cpp"
    $prefix_type=0; //@line 797 "translate.cpp"
    $found=0; //@line 809 "translate.cpp"
    HEAP32[(($last_char)>>2)]=0; //@line 813 "translate.cpp"
    $add_plural_suffix=0; //@line 814 "translate.cpp"
    $prefix_flags=0; //@line 815 "translate.cpp"
    $emphasize_allcaps=0; //@line 819 "translate.cpp"
    $was_unpronouncable=0; //@line 822 "translate.cpp"
    var $0=$wtab_addr; //@line 831 "translate.cpp"
    var $cmp=((($0))|0)==0; //@line 831 "translate.cpp"
    if ($cmp) { __label__ = 1;; } else { __label__ = 2;; } //@line 831 "translate.cpp"
    if (__label__ == 1) {
  
      var $arraydecay=(($wtab_null)&4294967295); //@line 833 "translate.cpp"
      var $1=$arraydecay; //@line 833 "translate.cpp"
      var $dest$, $stop$, $stop4$, $fast$, $value4$;
      $dest$ = $1;
      $stop$ = $dest$ + 96;
      $value4$ = 0;
      if ($value4$ < 0) $value4$ += 256;
      $value4$ = $value4$ + ($value4$<<8) + ($value4$<<16) + ($value4$*16777216);
      while ($dest$%4 !== 0 && $dest$ < $stop$) {
        ;; HEAP8[$dest$++] = 0;
      }
      $dest$ >>= 2;
      $stop4$ = $stop$ >> 2;
      while ($dest$ < $stop4$) {
      ;;;;;;;  HEAP32[$dest$++] = $value4$;
      }
      $dest$ <<= 2;
      while ($dest$ < $stop$) {
        ;; HEAP8[$dest$++] = 0;
      }; //@line 833 "translate.cpp"
      var $arraydecay1=(($wtab_null)&4294967295); //@line 834 "translate.cpp"
      $wtab_addr=$arraydecay1; //@line 834 "translate.cpp"
      ; //@line 835 "translate.cpp"
    }
  
    var $2=$wtab_addr; //@line 836 "translate.cpp"
    var $flags=(($2)&4294967295); //@line 836 "translate.cpp"
    var $3=HEAP32[(($flags)>>2)]; //@line 836 "translate.cpp"
    $wflags=$3; //@line 836 "translate.cpp"
    var $4=$wtab_addr; //@line 837 "translate.cpp"
    var $wmark2=(($4+9)&4294967295); //@line 837 "translate.cpp"
    var $5=HEAPU8[($wmark2)]; //@line 837 "translate.cpp"
    var $conv=((($5))&255); //@line 837 "translate.cpp"
    $wmark=$conv; //@line 837 "translate.cpp"
    var $arrayidx=(($dictionary_flags)&4294967295); //@line 839 "translate.cpp"
    HEAP32[(($arrayidx)>>2)]=0; //@line 839 "translate.cpp"
    var $arrayidx3=(($dictionary_flags+4)&4294967295); //@line 840 "translate.cpp"
    HEAP32[(($arrayidx3)>>2)]=0; //@line 840 "translate.cpp"
    var $arrayidx4=(($dictionary_flags2)&4294967295); //@line 841 "translate.cpp"
    HEAP32[(($arrayidx4)>>2)]=0; //@line 841 "translate.cpp"
    var $arrayidx5=(($dictionary_flags2+4)&4294967295); //@line 842 "translate.cpp"
    HEAP32[(($arrayidx5)>>2)]=0; //@line 842 "translate.cpp"
    HEAP32[((_dictionary_skipwords)>>2)]=0; //@line 843 "translate.cpp"
    var $arrayidx6=(($phonemes)&4294967295); //@line 845 "translate.cpp"
    HEAP8[($arrayidx6)]=0; //@line 845 "translate.cpp"
    var $arrayidx7=(($unpron_phonemes)&4294967295); //@line 846 "translate.cpp"
    HEAP8[($arrayidx7)]=0; //@line 846 "translate.cpp"
    var $arrayidx8=(($prefix_phonemes)&4294967295); //@line 847 "translate.cpp"
    HEAP8[($arrayidx8)]=0; //@line 847 "translate.cpp"
    var $arrayidx9=(($end_phonemes)&4294967295); //@line 848 "translate.cpp"
    HEAP8[($arrayidx9)]=0; //@line 848 "translate.cpp"
    var $arrayidx10=(($phonemes+160)&4294967295); //@line 849 "translate.cpp"
    $ph_limit=$arrayidx10; //@line 849 "translate.cpp"
    var $6=$tr_addr; //@line 851 "translate.cpp"
    var $data_dictlist=(($6+1052)&4294967295); //@line 851 "translate.cpp"
    var $7=HEAP32[(($data_dictlist)>>2)]; //@line 851 "translate.cpp"
    var $cmp11=((($7))|0)==0; //@line 851 "translate.cpp"
    if ($cmp11) { __label__ = 3;; } else { __label__ = 4;; } //@line 851 "translate.cpp"
    $if_then12$$if_end13$5: do { 
      if (__label__ == 3) {
  
        HEAP8[(((_word_phonemes)&4294967295))]=0; //@line 854 "translate.cpp"
        $retval=0; //@line 855 "translate.cpp"
        ; //@line 855 "translate.cpp"
      }
      else if (__label__ == 4) {
  
        var $8=$word_start_addr; //@line 859 "translate.cpp"
        HEAP32[(($word1)>>2)]=$8; //@line 859 "translate.cpp"
        var $9=HEAP32[(($word1)>>2)]; //@line 860 "translate.cpp"
        var $10=HEAP8[($9)]; //@line 860 "translate.cpp"
        var $conv14=(tempInt=(($10)),(tempInt>=128?tempInt-256:tempInt)); //@line 860 "translate.cpp"
        var $cmp15=((($conv14))|0)==32; //@line 860 "translate.cpp"
        if ($cmp15) { __label__ = 5;; } else { __label__ = 6;; } //@line 860 "translate.cpp"
        if (__label__ == 5) {
  
          var $11=HEAP32[(($word1)>>2)]; //@line 860 "translate.cpp"
          var $incdec_ptr=(($11+1)&4294967295); //@line 860 "translate.cpp"
          HEAP32[(($word1)>>2)]=$incdec_ptr; //@line 860 "translate.cpp"
          ; //@line 860 "translate.cpp"
        }
  
        var $12=HEAP32[(($word1)>>2)]; //@line 861 "translate.cpp"
        HEAP32[(($wordx)>>2)]=$12; //@line 861 "translate.cpp"
        var $13=HEAP32[(($wordx)>>2)]; //@line 863 "translate.cpp"
        var $call=__Z7utf8_inPiPKc($first_char, $13); //@line 863 "translate.cpp"
        $word_length=0; //@line 864 "translate.cpp"
        ; //@line 865 "translate.cpp"
        $while_cond$11: while(1) { 
  
          var $14=HEAP32[(($wordx)>>2)]; //@line 865 "translate.cpp"
          var $15=HEAP8[($14)]; //@line 865 "translate.cpp"
          var $conv18=(tempInt=(($15)),(tempInt>=128?tempInt-256:tempInt)); //@line 865 "translate.cpp"
          var $cmp19=((($conv18))|0)!=0; //@line 865 "translate.cpp"
          var $16=HEAP32[(($wordx)>>2)]; //@line 865 "translate.cpp"
          if ($cmp19) { __lastLabel__ = 7; ; } else { __lastLabel__ = 7; __label__ = 10;break $while_cond$11; } //@line 865 "translate.cpp"
  
          var $17=HEAP8[($16)]; //@line 865 "translate.cpp"
          var $conv20=(tempInt=(($17)),(tempInt>=128?tempInt-256:tempInt)); //@line 865 "translate.cpp"
          var $cmp21=((($conv20))|0)!=32; //@line 865 "translate.cpp"
          var $18=HEAP32[(($wordx)>>2)]; //@line 867 "translate.cpp"
          if ($cmp21) { __lastLabel__ = 8; ; } else { __lastLabel__ = 8; __label__ = 10;break $while_cond$11; }
  
          var $call22=__Z7utf8_inPiPKc($last_char, $18); //@line 867 "translate.cpp"
          var $19=HEAP32[(($wordx)>>2)]; //@line 867 "translate.cpp"
          var $add_ptr=(($19+$call22)&4294967295); //@line 867 "translate.cpp"
          HEAP32[(($wordx)>>2)]=$add_ptr; //@line 867 "translate.cpp"
          var $20=$word_length; //@line 868 "translate.cpp"
          var $inc=((($20)+1)&4294967295); //@line 868 "translate.cpp"
          $word_length=$inc; //@line 868 "translate.cpp"
          __label__ = 7;continue $while_cond$11; //@line 869 "translate.cpp"
        }
  
        var $21=__lastLabel__ == 8 ? $18 : ($16);
        var $22=$word_start_addr; //@line 871 "translate.cpp"
        var $sub_ptr_lhs_cast=($21); //@line 871 "translate.cpp"
        var $sub_ptr_rhs_cast=($22); //@line 871 "translate.cpp"
        var $sub_ptr_sub=((($sub_ptr_lhs_cast)-($sub_ptr_rhs_cast))&4294967295); //@line 871 "translate.cpp"
        $word_copy_length=$sub_ptr_sub; //@line 871 "translate.cpp"
        var $cmp23=((($sub_ptr_sub))|0) >= 160; //@line 872 "translate.cpp"
        if ($cmp23) { __label__ = 11;; } else { __label__ = 12;; } //@line 872 "translate.cpp"
        if (__label__ == 11) {
  
          $word_copy_length=159; //@line 873 "translate.cpp"
          ; //@line 873 "translate.cpp"
        }
  
        var $arraydecay26=(($word_copy2)&4294967295); //@line 874 "translate.cpp"
        var $23=$word_start_addr; //@line 874 "translate.cpp"
        var $24=$word_copy_length; //@line 874 "translate.cpp"
        var $src$, $dest$, $stop$, $stop4$, $fast$;
        $src$ = $23;
        $dest$ = $arraydecay26;
        $stop$ = $src$ + $24;
        if (($dest$%4) == ($src$%4) && $24 > 8) {
          while ($src$%4 !== 0 && $src$ < $stop$) {
            ; HEAP8[$dest$++] = HEAP8[$src$++];
          }
          $src$ >>= 2;
          $dest$ >>= 2;
          $stop4$ = $stop$ >> 2;
          while ($src$ < $stop4$) {
        ;;;    HEAP32[$dest$++] = HEAP32[$src$++];
          }
          $src$ <<= 2;
          $dest$ <<= 2;
        }while ($src$ < $stop$) {
          ; HEAP8[$dest$++] = HEAP8[$src$++];
        }; //@line 874 "translate.cpp"
        $spell_word=0; //@line 876 "translate.cpp"
        var $25=HEAP32[((_option_sayas)>>2)]; //@line 878 "translate.cpp"
        var $cmp27=((($25))|0)==36; //@line 878 "translate.cpp"
        if ($cmp27) { __label__ = 13;; } else { __label__ = 16;; } //@line 878 "translate.cpp"
        if (__label__ == 13) {
  
          var $26=$word_length; //@line 880 "translate.cpp"
          var $cmp29=((($26))|0)==1; //@line 880 "translate.cpp"
          if ($cmp29) { __label__ = 14;; } else { __label__ = 15;; } //@line 880 "translate.cpp"
          if (__label__ == 14) {
  
            $spell_word=4; //@line 881 "translate.cpp"
            ; //@line 881 "translate.cpp"
          }
          else if (__label__ == 15) {
  
            var $27=HEAP32[(($word1)>>2)]; //@line 885 "translate.cpp"
            var $incdec_ptr31=(($27+-1)&4294967295); //@line 885 "translate.cpp"
            HEAP32[(($word1)>>2)]=$incdec_ptr31; //@line 885 "translate.cpp"
            var $28=HEAP32[(($word1)>>2)]; //@line 886 "translate.cpp"
            HEAP8[($28)]=95; //@line 886 "translate.cpp"
            var $29=$tr_addr; //@line 887 "translate.cpp"
            var $arraydecay32=(($phonemes)&4294967295); //@line 887 "translate.cpp"
            var $arraydecay33=(($dictionary_flags)&4294967295); //@line 887 "translate.cpp"
            var $30=$wtab_addr; //@line 887 "translate.cpp"
            var $call34=__Z14LookupDictListP10TranslatorPPcS1_PjiP8WORD_TAB($29, $word1, $arraydecay32, $arraydecay33, 0, $30); //@line 887 "translate.cpp"
            $found=$call34; //@line 887 "translate.cpp"
            ;
          }
        }
  
        var $31=HEAP32[((_option_sayas)>>2)]; //@line 893 "translate.cpp"
        var $and=($31) & 16; //@line 893 "translate.cpp"
        var $tobool=((($and))|0)!=0; //@line 893 "translate.cpp"
        if ($tobool) { __label__ = 17;; } else { __label__ = 18;; } //@line 893 "translate.cpp"
        $if_then37$$if_else39$25: do { 
          if (__label__ == 17) {
  
            var $32=HEAP32[((_option_sayas)>>2)]; //@line 896 "translate.cpp"
            var $and38=($32) & 15; //@line 896 "translate.cpp"
            $spell_word=$and38; //@line 896 "translate.cpp"
            __lastLabel__ = 17; __label__ = 70;break $if_then37$$if_else39$25; //@line 897 "translate.cpp"
          }
          else if (__label__ == 18) {
  
            var $33=$found; //@line 900 "translate.cpp"
            var $tobool40=((($33))|0)!=0; //@line 900 "translate.cpp"
            if ($tobool40) { __label__ = 20;; } else { __label__ = 19;; } //@line 900 "translate.cpp"
            if (__label__ == 19) {
  
              var $34=$tr_addr; //@line 901 "translate.cpp"
              var $arraydecay42=(($phonemes)&4294967295); //@line 901 "translate.cpp"
              var $arraydecay43=(($dictionary_flags)&4294967295); //@line 901 "translate.cpp"
              var $35=$wtab_addr; //@line 901 "translate.cpp"
              var $call44=__Z14LookupDictListP10TranslatorPPcS1_PjiP8WORD_TAB($34, $word1, $arraydecay42, $arraydecay43, 2, $35); //@line 901 "translate.cpp"
              $found=$call44; //@line 901 "translate.cpp"
              ; //@line 901 "translate.cpp"
            }
  
            var $arrayidx46=(($dictionary_flags)&4294967295); //@line 904 "translate.cpp"
            var $36=HEAP32[(($arrayidx46)>>2)]; //@line 904 "translate.cpp"
            var $and47=($36) & 1; //@line 904 "translate.cpp"
            var $tobool48=((($and47))|0)!=0; //@line 904 "translate.cpp"
            if ($tobool48) { __label__ = 21;; } else { __label__ = 23;; } //@line 904 "translate.cpp"
            $land_lhs_true$$if_end54$31: do { 
              if (__label__ == 21) {
  
                var $37=HEAP32[(($wordx)>>2)]; //@line 904 "translate.cpp"
                var $arrayidx49=(($37+1)&4294967295); //@line 904 "translate.cpp"
                var $38=HEAP8[($arrayidx49)]; //@line 904 "translate.cpp"
                var $conv50=(tempInt=(($38)),(tempInt>=128?tempInt-256:tempInt)); //@line 904 "translate.cpp"
                var $cmp51=((($conv50))|0)==46; //@line 904 "translate.cpp"
                if (!($cmp51)) { __label__ = 23;break $land_lhs_true$$if_end54$31; } //@line 904 "translate.cpp"
  
                var $39=HEAP32[(($wordx)>>2)]; //@line 906 "translate.cpp"
                var $arrayidx53=(($39+1)&4294967295); //@line 906 "translate.cpp"
                HEAP8[($arrayidx53)]=32; //@line 906 "translate.cpp"
                ; //@line 907 "translate.cpp"
              }
            } while(0);
  
            var $arrayidx55=(($dictionary_flags)&4294967295); //@line 909 "translate.cpp"
            var $40=HEAP32[(($arrayidx55)>>2)]; //@line 909 "translate.cpp"
            var $and56=($40) & 536870912; //@line 909 "translate.cpp"
            var $tobool57=((($and56))|0)!=0; //@line 909 "translate.cpp"
            if ($tobool57) { __label__ = 24;; } else { __label__ = 27;; } //@line 909 "translate.cpp"
            $if_then58$$if_else80$35: do { 
              if (__label__ == 24) {
  
                var $41=HEAP32[(($word1)>>2)]; //@line 911 "translate.cpp"
                var $arrayidx59=(($41)&4294967295); //@line 911 "translate.cpp"
                var $42=HEAP8[($arrayidx59)]; //@line 911 "translate.cpp"
                var $conv60=(tempInt=(($42)),(tempInt>=128?tempInt-256:tempInt)); //@line 911 "translate.cpp"
                HEAP32[(($first_char)>>2)]=$conv60; //@line 911 "translate.cpp"
                var $arrayidx61=(($dictionary_flags)&4294967295); //@line 912 "translate.cpp"
                var $43=HEAP32[(($arrayidx61)>>2)]; //@line 912 "translate.cpp"
                var $and62=($43) & 127; //@line 912 "translate.cpp"
                $stress_bits=$and62; //@line 912 "translate.cpp"
                var $44=$tr_addr; //@line 913 "translate.cpp"
                var $arraydecay63=(($phonemes)&4294967295); //@line 913 "translate.cpp"
                var $arraydecay64=(($dictionary_flags2)&4294967295); //@line 913 "translate.cpp"
                var $45=$wtab_addr; //@line 913 "translate.cpp"
                var $call65=__Z14LookupDictListP10TranslatorPPcS1_PjiP8WORD_TAB($44, $word1, $arraydecay63, $arraydecay64, 0, $45); //@line 913 "translate.cpp"
                $found=$call65; //@line 913 "translate.cpp"
                var $arrayidx66=(($dictionary_flags2)&4294967295); //@line 914 "translate.cpp"
                var $46=HEAP32[(($arrayidx66)>>2)]; //@line 914 "translate.cpp"
                var $cmp67=((($46))|0)!=0; //@line 914 "translate.cpp"
                if (!($cmp67)) { __label__ = 33;break $if_then58$$if_else80$35; } //@line 914 "translate.cpp"
  
                var $arrayidx69=(($dictionary_flags2)&4294967295); //@line 916 "translate.cpp"
                var $47=HEAP32[(($arrayidx69)>>2)]; //@line 916 "translate.cpp"
                var $arrayidx70=(($dictionary_flags)&4294967295); //@line 916 "translate.cpp"
                HEAP32[(($arrayidx70)>>2)]=$47; //@line 916 "translate.cpp"
                var $arrayidx71=(($dictionary_flags2+4)&4294967295); //@line 917 "translate.cpp"
                var $48=HEAP32[(($arrayidx71)>>2)]; //@line 917 "translate.cpp"
                var $arrayidx72=(($dictionary_flags+4)&4294967295); //@line 917 "translate.cpp"
                HEAP32[(($arrayidx72)>>2)]=$48; //@line 917 "translate.cpp"
                var $49=$stress_bits; //@line 918 "translate.cpp"
                var $cmp73=((($49))|0)!=0; //@line 918 "translate.cpp"
                if (!($cmp73)) { __label__ = 33;break $if_then58$$if_else80$35; } //@line 918 "translate.cpp"
  
                var $arrayidx75=(($dictionary_flags)&4294967295); //@line 921 "translate.cpp"
                var $50=HEAP32[(($arrayidx75)>>2)]; //@line 921 "translate.cpp"
                var $and76=($50) & -128; //@line 921 "translate.cpp"
                var $51=$stress_bits; //@line 921 "translate.cpp"
                var $or=($and76) | ($51); //@line 921 "translate.cpp"
                var $arrayidx77=(($dictionary_flags)&4294967295); //@line 921 "translate.cpp"
                HEAP32[(($arrayidx77)>>2)]=$or; //@line 921 "translate.cpp"
                ; //@line 922 "translate.cpp"
              }
              else if (__label__ == 27) {
  
                var $52=$found; //@line 926 "translate.cpp"
                var $cmp81=((($52))|0)==0; //@line 926 "translate.cpp"
                if (!($cmp81)) { __label__ = 33;break $if_then58$$if_else80$35; } //@line 926 "translate.cpp"
  
                var $arrayidx83=(($dictionary_flags)&4294967295); //@line 926 "translate.cpp"
                var $53=HEAP32[(($arrayidx83)>>2)]; //@line 926 "translate.cpp"
                var $and84=($53) & 128; //@line 926 "translate.cpp"
                var $tobool85=((($and84))|0)!=0; //@line 926 "translate.cpp"
                if (!($tobool85)) { __label__ = 33;break $if_then58$$if_else80$35; } //@line 926 "translate.cpp"
  
                var $54=HEAP32[(($word1)>>2)]; //@line 929 "translate.cpp"
                HEAP32[(($wordx)>>2)]=$54; //@line 929 "translate.cpp"
                $ix=0; //@line 930 "translate.cpp"
                var $55=$ix; //@line 931 "translate.cpp"
                var $56=HEAP32[((_dictionary_skipwords)>>2)]; //@line 931 "translate.cpp"
                var $cmp8815=((($55))|0) < ((($56))|0); //@line 931 "translate.cpp"
                if (!($cmp8815)) { __label__ = 33;break $if_then58$$if_else80$35; } //@line 931 "translate.cpp"
                while(1) { 
  
                  var $57=HEAP32[(($wordx)>>2)]; //@line 933 "translate.cpp"
                  var $58=HEAP8[($57)]; //@line 933 "translate.cpp"
                  var $conv90=(tempInt=(($58)),(tempInt>=128?tempInt-256:tempInt)); //@line 933 "translate.cpp"
                  var $cmp91=((($conv90))|0)==32; //@line 933 "translate.cpp"
                  if ($cmp91) { __label__ = 31;; } else { __label__ = 32;; } //@line 933 "translate.cpp"
                  if (__label__ == 31) {
  
                    var $59=HEAP32[(($wordx)>>2)]; //@line 935 "translate.cpp"
                    HEAP8[($59)]=45; //@line 935 "translate.cpp"
                    var $60=$ix; //@line 936 "translate.cpp"
                    var $inc93=((($60)+1)&4294967295); //@line 936 "translate.cpp"
                    $ix=$inc93; //@line 936 "translate.cpp"
                    ; //@line 937 "translate.cpp"
                  }
  
                  var $61=HEAP32[(($wordx)>>2)]; //@line 938 "translate.cpp"
                  var $incdec_ptr95=(($61+1)&4294967295); //@line 938 "translate.cpp"
                  HEAP32[(($wordx)>>2)]=$incdec_ptr95; //@line 938 "translate.cpp"
                  var $62=$ix; //@line 931 "translate.cpp"
                  var $63=HEAP32[((_dictionary_skipwords)>>2)]; //@line 931 "translate.cpp"
                  var $cmp88=((($62))|0) < ((($63))|0); //@line 931 "translate.cpp"
                  if ($cmp88) { __label__ = 30;continue ; } else { __label__ = 33;break $if_then58$$if_else80$35; } //@line 931 "translate.cpp"
                }
              }
            } while(0);
  
            var $64=$word_length; //@line 942 "translate.cpp"
            var $cmp99=((($64))|0)==1; //@line 942 "translate.cpp"
            var $65=HEAP32[((_dictionary_skipwords)>>2)]; //@line 942 "translate.cpp"
            var $cmp101=((($65))|0)==0; //@line 942 "translate.cpp"
            var $or_cond=($cmp99) & ($cmp101); //@line 942 "translate.cpp"
            if ($or_cond) { __label__ = 34;; } else { __label__ = 37;; } //@line 942 "translate.cpp"
            $if_then102$$if_end113$48: do { 
              if (__label__ == 34) {
  
                var $66=HEAP32[(($word1)>>2)]; //@line 945 "translate.cpp"
                var $67=$wtab_addr; //@line 945 "translate.cpp"
                var $call103=__ZL17CheckDottedAbbrevPcP8WORD_TAB($66, $67); //@line 945 "translate.cpp"
                var $tobool104=((($call103))|0)!=0; //@line 945 "translate.cpp"
                if (!($tobool104)) { __label__ = 37;break $if_then102$$if_end113$48; } //@line 945 "translate.cpp"
  
                var $arrayidx106=(($dictionary_flags)&4294967295); //@line 947 "translate.cpp"
                HEAP32[(($arrayidx106)>>2)]=0; //@line 947 "translate.cpp"
                var $arrayidx107=(($dictionary_flags+4)&4294967295); //@line 948 "translate.cpp"
                HEAP32[(($arrayidx107)>>2)]=0; //@line 948 "translate.cpp"
                $spell_word=1; //@line 949 "translate.cpp"
                var $68=HEAP32[((_dictionary_skipwords)>>2)]; //@line 950 "translate.cpp"
                var $tobool108=((($68))|0)!=0; //@line 950 "translate.cpp"
                if (!($tobool108)) { __label__ = 37;break $if_then102$$if_end113$48; } //@line 950 "translate.cpp"
  
                var $arrayidx110=(($dictionary_flags)&4294967295); //@line 951 "translate.cpp"
                HEAP32[(($arrayidx110)>>2)]=128; //@line 951 "translate.cpp"
                ; //@line 951 "translate.cpp"
              }
            } while(0);
  
            var $arrayidx114=(($phonemes)&4294967295); //@line 958 "translate.cpp"
            var $69=HEAP8[($arrayidx114)]; //@line 958 "translate.cpp"
            var $conv115=(tempInt=(($69)),(tempInt>=128?tempInt-256:tempInt)); //@line 958 "translate.cpp"
            var $cmp116=((($conv115))|0)==21; //@line 958 "translate.cpp"
            if ($cmp116) { __label__ = 38;; } else { __label__ = 39;; } //@line 958 "translate.cpp"
            if (__label__ == 38) {
  
              var $arraydecay118=(($phonemes)&4294967295); //@line 961 "translate.cpp"
              var $call119=_strcpy(((_word_phonemes)&4294967295), $arraydecay118); //@line 961 "translate.cpp"
              $retval=0; //@line 962 "translate.cpp"
              __label__ = 236;break $if_then12$$if_end13$5; //@line 962 "translate.cpp"
            }
            else if (__label__ == 39) {
  
              var $70=$wmark; //@line 965 "translate.cpp"
              var $cmp121=((($70))|0) > 0; //@line 965 "translate.cpp"
              var $71=$wmark; //@line 965 "translate.cpp"
              var $cmp123=((($71))|0) < 8; //@line 965 "translate.cpp"
              var $or_cond1=($cmp121) & ($cmp123); //@line 965 "translate.cpp"
              if ($or_cond1) { __label__ = 40;; } else { __label__ = 41;; } //@line 965 "translate.cpp"
              if (__label__ == 40) {
  
                var $arrayidx125=(($dictionary_flags)&4294967295); //@line 968 "translate.cpp"
                var $72=HEAP32[(($arrayidx125)>>2)]; //@line 968 "translate.cpp"
                var $and126=($72) & -16; //@line 968 "translate.cpp"
                var $73=$wmark; //@line 968 "translate.cpp"
                var $or127=($and126) | ($73); //@line 968 "translate.cpp"
                var $arrayidx128=(($dictionary_flags)&4294967295); //@line 968 "translate.cpp"
                HEAP32[(($arrayidx128)>>2)]=$or127; //@line 968 "translate.cpp"
                ; //@line 969 "translate.cpp"
              }
  
              var $74=$found; //@line 971 "translate.cpp"
              var $tobool130=((($74))|0)!=0; //@line 971 "translate.cpp"
              if ($tobool130) { __label__ = 49;; } else { __label__ = 42;; } //@line 971 "translate.cpp"
              $if_end150$$land_lhs_true131$59: do { 
                if (__label__ == 42) {
  
                  var $arrayidx132=(($dictionary_flags)&4294967295); //@line 971 "translate.cpp"
                  var $75=HEAP32[(($arrayidx132)>>2)]; //@line 971 "translate.cpp"
                  var $and133=($75) & 131072; //@line 971 "translate.cpp"
                  var $tobool134=((($and133))|0)!=0; //@line 971 "translate.cpp"
                  if ($tobool134) { __label__ = 43;; } else { __label__ = 44;; } //@line 971 "translate.cpp"
                  if (__label__ == 43) {
  
                    $spell_word=1; //@line 974 "translate.cpp"
                    ; //@line 975 "translate.cpp"
                  }
  
                  var $_pr=$found; //@line 977 "translate.cpp"
                  var $tobool137=((($_pr))|0)!=0; //@line 977 "translate.cpp"
                  if ($tobool137) { __label__ = 49;break $if_end150$$land_lhs_true131$59; } //@line 977 "translate.cpp"
  
                  var $76=HEAP32[(($first_char)>>2)]; //@line 977 "translate.cpp"
                  var $call139=_iswdigit($76); //@line 977 "translate.cpp"
                  var $tobool140=((($call139))|0)!=0; //@line 977 "translate.cpp"
                  if (!($tobool140)) { __label__ = 49;break $if_end150$$land_lhs_true131$59; } //@line 977 "translate.cpp"
  
                  var $77=$tr_addr; //@line 979 "translate.cpp"
                  var $call142=__Z6LookupP10TranslatorPKcPc($77, ((__str2545)&4294967295), ((_word_phonemes)&4294967295)); //@line 979 "translate.cpp"
                  var $78=HEAP8[(((_word_phonemes)&4294967295))]; //@line 980 "translate.cpp"
                  var $conv143=(tempInt=(($78)),(tempInt>=128?tempInt-256:tempInt)); //@line 980 "translate.cpp"
                  var $cmp144=((($conv143))|0)==21; //@line 980 "translate.cpp"
                  if ($cmp144) { __label__ = 47;; } else { __label__ = 48;; } //@line 980 "translate.cpp"
                  if (__label__ == 47) {
  
                    $retval=0; //@line 981 "translate.cpp"
                    __label__ = 236;break $if_then12$$if_end13$5; //@line 981 "translate.cpp"
                  }
                  else if (__label__ == 48) {
  
                    var $79=$tr_addr; //@line 983 "translate.cpp"
                    var $80=HEAP32[(($word1)>>2)]; //@line 983 "translate.cpp"
                    var $arraydecay147=(($phonemes)&4294967295); //@line 983 "translate.cpp"
                    var $arraydecay148=(($dictionary_flags)&4294967295); //@line 983 "translate.cpp"
                    var $81=$wtab_addr; //@line 983 "translate.cpp"
                    var $call149=__Z15TranslateNumberP10TranslatorPcS1_PjP8WORD_TABi($79, $80, $arraydecay147, $arraydecay148, $81, 0); //@line 983 "translate.cpp"
                    $found=$call149; //@line 983 "translate.cpp"
                    ; //@line 984 "translate.cpp"
                  }
                }
              } while(0);
  
              var $82=$found; //@line 986 "translate.cpp"
              var $tobool151=((($82))|0)!=0; //@line 986 "translate.cpp"
              var $lnot=($tobool151) ^ 1; //@line 986 "translate.cpp"
              var $conv152=((($lnot))&1); //@line 986 "translate.cpp"
              var $83=$wflags; //@line 986 "translate.cpp"
              var $and153=($83) & 3; //@line 986 "translate.cpp"
              var $cmp154=((($and153))|0)!=2; //@line 986 "translate.cpp"
              var $conv155=((($cmp154))&1); //@line 986 "translate.cpp"
              var $and156=($conv155) & ($conv152); //@line 986 "translate.cpp"
              var $tobool157=((($and156))|0)!=0; //@line 986 "translate.cpp"
              if ($tobool157) { __label__ = 50;; } else { __label__ = 57;; } //@line 986 "translate.cpp"
              $if_then158$$if_end186$70: do { 
                if (__label__ == 50) {
  
                  var $84=$tr_addr; //@line 990 "translate.cpp"
                  var $langopts=(($84)&4294967295); //@line 990 "translate.cpp"
                  var $numbers=(($langopts+184)&4294967295); //@line 990 "translate.cpp"
                  var $85=HEAP32[(($numbers)>>2)]; //@line 990 "translate.cpp"
                  var $and159=($85) & 16777216; //@line 990 "translate.cpp"
                  var $tobool160=((($and159))|0)!=0; //@line 990 "translate.cpp"
                  if ($tobool160) { __label__ = 53;; } else { __label__ = 51;; } //@line 990 "translate.cpp"
                  if (__label__ == 51) {
  
                    var $86=$tr_addr; //@line 990 "translate.cpp"
                    var $langopts161=(($86)&4294967295); //@line 990 "translate.cpp"
                    var $numbers162=(($langopts161+184)&4294967295); //@line 990 "translate.cpp"
                    var $87=HEAP32[(($numbers162)>>2)]; //@line 990 "translate.cpp"
                    var $and163=($87) & 33554432; //@line 990 "translate.cpp"
                    var $tobool164=((($and163))|0)!=0; //@line 990 "translate.cpp"
                    if (!($tobool164)) { __label__ = 57;break $if_then158$$if_end186$70; } //@line 990 "translate.cpp"
  
                    var $88=$wflags; //@line 990 "translate.cpp"
                    var $and166=($88) & 1; //@line 990 "translate.cpp"
                    var $tobool167=((($and166))|0)!=0; //@line 990 "translate.cpp"
                    if (!($tobool167)) { __label__ = 57;break $if_then158$$if_end186$70; } //@line 990 "translate.cpp"
                  }
  
                  var $89=$wflags; //@line 992 "translate.cpp"
                  var $and169=($89) & 16; //@line 992 "translate.cpp"
                  var $tobool170=((($and169))|0)!=0; //@line 992 "translate.cpp"
                  if ($tobool170) { __label__ = 55;; } else { __label__ = 54;; } //@line 992 "translate.cpp"
                  if (__label__ == 54) {
  
                    var $90=$wtab_addr; //@line 992 "translate.cpp"
                    var $arrayidx172=(($90+12)&4294967295); //@line 992 "translate.cpp"
                    var $flags173=(($arrayidx172)&4294967295); //@line 992 "translate.cpp"
                    var $91=HEAP32[(($flags173)>>2)]; //@line 992 "translate.cpp"
                    var $and174=($91) & 256; //@line 992 "translate.cpp"
                    var $tobool175=((($and174))|0)!=0; //@line 992 "translate.cpp"
                    if ($tobool175) { __label__ = 57;break $if_then158$$if_end186$70; } //@line 992 "translate.cpp"
                  }
  
                  var $92=$tr_addr; //@line 995 "translate.cpp"
                  var $93=HEAP32[(($word1)>>2)]; //@line 995 "translate.cpp"
                  var $arraydecay177=(($phonemes)&4294967295); //@line 995 "translate.cpp"
                  var $94=$wtab_addr; //@line 995 "translate.cpp"
                  var $call178=__Z14TranslateRomanP10TranslatorPcS1_P8WORD_TAB($92, $93, $arraydecay177, $94); //@line 995 "translate.cpp"
                  $found=$call178; //@line 995 "translate.cpp"
                  var $cmp179=((($call178))|0)!=0; //@line 995 "translate.cpp"
                  if (!($cmp179)) { __label__ = 57;break $if_then158$$if_end186$70; } //@line 995 "translate.cpp"
  
                  var $arrayidx181=(($dictionary_flags)&4294967295); //@line 996 "translate.cpp"
                  var $95=HEAP32[(($arrayidx181)>>2)]; //@line 996 "translate.cpp"
                  var $or182=($95) | 131072; //@line 996 "translate.cpp"
                  HEAP32[(($arrayidx181)>>2)]=$or182; //@line 996 "translate.cpp"
                  ; //@line 996 "translate.cpp"
                }
              } while(0);
  
              var $96=$wflags; //@line 1001 "translate.cpp"
              var $and187=($96) & 1; //@line 1001 "translate.cpp"
              var $tobool188=((($and187))|0)!=0; //@line 1001 "translate.cpp"
              if ($tobool188) { __label__ = 58;; } else { __label__ = 69;; } //@line 1001 "translate.cpp"
              $land_lhs_true189$$if_end219thread_pre_split$81: do { 
                if (__label__ == 58) {
  
                  var $97=$word_length; //@line 1001 "translate.cpp"
                  var $cmp190=((($97))|0) > 1; //@line 1001 "translate.cpp"
                  if (!($cmp190)) { __label__ = 69;break $land_lhs_true189$$if_end219thread_pre_split$81; } //@line 1001 "translate.cpp"
  
                  var $98=HEAP32[(($first_char)>>2)]; //@line 1001 "translate.cpp"
                  var $call192=_iswalpha($98); //@line 1001 "translate.cpp"
                  var $tobool193=((($call192))|0)!=0; //@line 1001 "translate.cpp"
                  if (!($tobool193)) { __label__ = 69;break $land_lhs_true189$$if_end219thread_pre_split$81; } //@line 1001 "translate.cpp"
  
                  var $99=HEAP32[((_option_tone_flags)>>2)]; //@line 1003 "translate.cpp"
                  var $and195=($99) & 256; //@line 1003 "translate.cpp"
                  var $tobool196=((($and195))|0)!=0; //@line 1003 "translate.cpp"
                  if ($tobool196) { __label__ = 61;; } else { __label__ = 63;; } //@line 1003 "translate.cpp"
                  $land_lhs_true197$$if_else202$85: do { 
                    if (__label__ == 61) {
  
                      var $arrayidx198=(($dictionary_flags)&4294967295); //@line 1003 "translate.cpp"
                      var $100=HEAP32[(($arrayidx198)>>2)]; //@line 1003 "translate.cpp"
                      var $and199=($100) & 131072; //@line 1003 "translate.cpp"
                      var $tobool200=((($and199))|0)!=0; //@line 1003 "translate.cpp"
                      if ($tobool200) { __label__ = 63;break $land_lhs_true197$$if_else202$85; } //@line 1003 "translate.cpp"
  
                      $emphasize_allcaps=2048; //@line 1006 "translate.cpp"
                      __label__ = 69;break $land_lhs_true189$$if_end219thread_pre_split$81; //@line 1007 "translate.cpp"
                    }
                  } while(0);
  
                  var $101=$found; //@line 1009 "translate.cpp"
                  var $tobool203=((($101))|0)!=0; //@line 1009 "translate.cpp"
                  if ($tobool203) { __label__ = 69;break $land_lhs_true189$$if_end219thread_pre_split$81; } //@line 1009 "translate.cpp"
  
                  var $arrayidx205=(($dictionary_flags)&4294967295); //@line 1009 "translate.cpp"
                  var $102=HEAP32[(($arrayidx205)>>2)]; //@line 1009 "translate.cpp"
                  var $and206=($102) & 128; //@line 1009 "translate.cpp"
                  var $tobool207=((($and206))|0)!=0; //@line 1009 "translate.cpp"
                  if ($tobool207) { __label__ = 69;break $land_lhs_true189$$if_end219thread_pre_split$81; } //@line 1009 "translate.cpp"
  
                  var $103=$word_length; //@line 1009 "translate.cpp"
                  var $cmp209=((($103))|0) < 4; //@line 1009 "translate.cpp"
                  if (!($cmp209)) { __label__ = 69;break $land_lhs_true189$$if_end219thread_pre_split$81; } //@line 1009 "translate.cpp"
  
                  var $104=$tr_addr; //@line 1009 "translate.cpp"
                  var $clause_lower_count=(($104+8308)&4294967295); //@line 1009 "translate.cpp"
                  var $105=HEAP32[(($clause_lower_count)>>2)]; //@line 1009 "translate.cpp"
                  var $cmp211=((($105))|0) > 3; //@line 1009 "translate.cpp"
                  if (!($cmp211)) { __label__ = 69;break $land_lhs_true189$$if_end219thread_pre_split$81; } //@line 1009 "translate.cpp"
  
                  var $106=$tr_addr; //@line 1009 "translate.cpp"
                  var $clause_upper_count=(($106+8304)&4294967295); //@line 1009 "translate.cpp"
                  var $107=HEAP32[(($clause_upper_count)>>2)]; //@line 1009 "translate.cpp"
                  var $108=$tr_addr; //@line 1009 "translate.cpp"
                  var $clause_lower_count213=(($108+8308)&4294967295); //@line 1009 "translate.cpp"
                  var $109=HEAP32[(($clause_lower_count213)>>2)]; //@line 1009 "translate.cpp"
                  var $cmp214=((($107))|0) <= ((($109))|0); //@line 1009 "translate.cpp"
                  if (!($cmp214)) { __label__ = 69;break $land_lhs_true189$$if_end219thread_pre_split$81; } //@line 1009 "translate.cpp"
  
                  $spell_word=1; //@line 1013 "translate.cpp"
                  __label__ = 71;break $if_then37$$if_else39$25; //@line 1018 "translate.cpp"
                }
              } while(0);
  
              var $_pr5=$spell_word; //@line 1018 "translate.cpp"
              __lastLabel__ = 69; __label__ = 70;break $if_then37$$if_else39$25;
            }
          }
        } while(0);
        $if_end219$$if_then221$95: do { 
          if (__label__ == 70) {
  
            var $110=__lastLabel__ == 69 ? $_pr5 : ($and38); //@line 1018 "translate.cpp"
            var $cmp220=((($110))|0) > 0; //@line 1018 "translate.cpp"
            if ($cmp220) { __label__ = 71;break $if_end219$$if_then221$95; } //@line 1018 "translate.cpp"
  
            var $115=$found; //@line 1032 "translate.cpp"
            var $cmp232=((($115))|0)==0; //@line 1032 "translate.cpp"
            if (!($cmp232)) { __label__ = 166;break $if_end219$$if_then221$95; } //@line 1032 "translate.cpp"
  
            $posn=0; //@line 1040 "translate.cpp"
            $non_initial=0; //@line 1041 "translate.cpp"
            $length=999; //@line 1042 "translate.cpp"
            var $116=HEAP32[(($word1)>>2)]; //@line 1043 "translate.cpp"
            HEAP32[(($wordx)>>2)]=$116; //@line 1043 "translate.cpp"
            var $arraydecay252=(($unpron_phonemes)&4294967295); //@line 1058 "translate.cpp"
            var $arrayidx256=(($unpron_phonemes)&4294967295); //@line 1060 "translate.cpp"
            ; //@line 1045 "translate.cpp"
            $while_cond234$99: while(1) { 
  
              var $117=$length; //@line 1045 "translate.cpp"
              var $cmp235=((($117))|0) < 3; //@line 1045 "translate.cpp"
              var $118=$length; //@line 1045 "translate.cpp"
              var $cmp237=((($118))|0) > 0; //@line 1045 "translate.cpp"
              var $or_cond2=($cmp235) & ($cmp237); //@line 1045 "translate.cpp"
              if ($or_cond2) { __label__ = 80;; } else { __label__ = 78;; } //@line 1045 "translate.cpp"
              if (__label__ == 78) {
  
                var $119=$word_length; //@line 1045 "translate.cpp"
                var $cmp238=((($119))|0) > 1; //@line 1045 "translate.cpp"
                if (!($cmp238)) { __label__ = 89;break $while_cond234$99; } //@line 1045 "translate.cpp"
  
                var $120=$tr_addr; //@line 1045 "translate.cpp"
                var $121=HEAP32[(($wordx)>>2)]; //@line 1045 "translate.cpp"
                var $122=$posn; //@line 1045 "translate.cpp"
                var $call240=__Z14UnpronouncableP10TranslatorPci($120, $121, $122); //@line 1045 "translate.cpp"
                var $tobool241=((($call240))|0)!=0; //@line 1045 "translate.cpp"
                if (!($tobool241)) { __label__ = 89;break $while_cond234$99; }
              }
  
              $was_unpronouncable=67108864; //@line 1049 "translate.cpp"
              $emphasize_allcaps=0; //@line 1050 "translate.cpp"
              var $123=HEAP32[(($wordx)>>2)]; //@line 1052 "translate.cpp"
              var $arrayidx244=(($123)&4294967295); //@line 1052 "translate.cpp"
              var $124=HEAP8[($arrayidx244)]; //@line 1052 "translate.cpp"
              var $conv245=(tempInt=(($124)),(tempInt>=128?tempInt-256:tempInt)); //@line 1052 "translate.cpp"
              var $cmp246=((($conv245))|0)==39; //@line 1052 "translate.cpp"
              if ($cmp246) { __label__ = 89;break $while_cond234$99; } //@line 1052 "translate.cpp"
  
              var $125=$posn; //@line 1055 "translate.cpp"
              var $cmp249=((($125))|0) > 0; //@line 1055 "translate.cpp"
              if ($cmp249) { __label__ = 82;; } else { __label__ = 83;; } //@line 1055 "translate.cpp"
              if (__label__ == 82) {
  
                $non_initial=1; //@line 1056 "translate.cpp"
                ; //@line 1056 "translate.cpp"
              }
  
              var $126=$tr_addr; //@line 1058 "translate.cpp"
              var $127=HEAP32[(($wordx)>>2)]; //@line 1058 "translate.cpp"
              var $128=$non_initial; //@line 1058 "translate.cpp"
              var $call253=__Z15TranslateLetterP10TranslatorPcS1_i($126, $127, $arraydecay252, $128); //@line 1058 "translate.cpp"
              var $129=HEAP32[(($wordx)>>2)]; //@line 1058 "translate.cpp"
              var $add_ptr254=(($129+$call253)&4294967295); //@line 1058 "translate.cpp"
              HEAP32[(($wordx)>>2)]=$add_ptr254; //@line 1058 "translate.cpp"
              var $130=$posn; //@line 1059 "translate.cpp"
              var $inc255=((($130)+1)&4294967295); //@line 1059 "translate.cpp"
              $posn=$inc255; //@line 1059 "translate.cpp"
              var $131=HEAP8[($arrayidx256)]; //@line 1060 "translate.cpp"
              var $conv257=(tempInt=(($131)),(tempInt>=128?tempInt-256:tempInt)); //@line 1060 "translate.cpp"
              var $cmp258=((($conv257))|0)==21; //@line 1060 "translate.cpp"
              if ($cmp258) { __label__ = 84;break $while_cond234$99; } //@line 1060 "translate.cpp"
  
              $length=0; //@line 1080 "translate.cpp"
              var $132=$length; //@line 1081 "translate.cpp"
              var $133=HEAP32[(($wordx)>>2)]; //@line 1081 "translate.cpp"
              var $arrayidx26921=(($133+$132)&4294967295); //@line 1081 "translate.cpp"
              var $134=HEAP8[($arrayidx26921)]; //@line 1081 "translate.cpp"
              var $conv27022=(tempInt=(($134)),(tempInt>=128?tempInt-256:tempInt)); //@line 1081 "translate.cpp"
              var $cmp27123=((($conv27022))|0)!=32; //@line 1081 "translate.cpp"
              if (!($cmp27123)) { __label__ = 77;continue $while_cond234$99; } //@line 1081 "translate.cpp"
              while(1) { 
  
                var $135=$length; //@line 1081 "translate.cpp"
                var $inc273=((($135)+1)&4294967295); //@line 1081 "translate.cpp"
                $length=$inc273; //@line 1081 "translate.cpp"
                var $136=$length; //@line 1081 "translate.cpp"
                var $137=HEAP32[(($wordx)>>2)]; //@line 1081 "translate.cpp"
                var $arrayidx269=(($137+$136)&4294967295); //@line 1081 "translate.cpp"
                var $138=HEAP8[($arrayidx269)]; //@line 1081 "translate.cpp"
                var $conv270=(tempInt=(($138)),(tempInt>=128?tempInt-256:tempInt)); //@line 1081 "translate.cpp"
                var $cmp271=((($conv270))|0)!=32; //@line 1081 "translate.cpp"
                if ($cmp271) { __label__ = 88;continue ; } else { __label__ = 77;continue $while_cond234$99; } //@line 1081 "translate.cpp"
              }
            }
            if (__label__ == 89) {
  
              var $139=$tr_addr; //@line 1083 "translate.cpp"
              var $arraydecay276=(($unpron_phonemes)&4294967295); //@line 1083 "translate.cpp"
              var $140=$posn; //@line 1083 "translate.cpp"
              __Z17SetSpellingStressP10TranslatorPcii($139, $arraydecay276, 0, $140); //@line 1083 "translate.cpp"
              var $141=HEAP32[(($wordx)>>2)]; //@line 1086 "translate.cpp"
              var $142=HEAP8[($141)]; //@line 1086 "translate.cpp"
              var $conv277=(tempInt=(($142)),(tempInt>=128?tempInt-256:tempInt)); //@line 1086 "translate.cpp"
              var $cmp278=((($conv277))|0)!=32; //@line 1086 "translate.cpp"
              if (!($cmp278)) { __label__ = 166;break $if_end219$$if_then221$95; } //@line 1086 "translate.cpp"
  
              var $arrayidx280=(($unpron_phonemes)&4294967295); //@line 1088 "translate.cpp"
              var $143=HEAP8[($arrayidx280)]; //@line 1088 "translate.cpp"
              var $conv281=(tempInt=(($143)),(tempInt>=128?tempInt-256:tempInt)); //@line 1088 "translate.cpp"
              var $cmp282=((($conv281))|0)!=0; //@line 1088 "translate.cpp"
              if ($cmp282) { __label__ = 91;; } else { __label__ = 93;; } //@line 1088 "translate.cpp"
              $land_lhs_true283$$if_end289$115: do { 
                if (__label__ == 91) {
  
                  var $144=HEAP32[(($wordx)>>2)]; //@line 1088 "translate.cpp"
                  var $arrayidx284=(($144)&4294967295); //@line 1088 "translate.cpp"
                  var $145=HEAP8[($arrayidx284)]; //@line 1088 "translate.cpp"
                  var $conv285=(tempInt=(($145)),(tempInt>=128?tempInt-256:tempInt)); //@line 1088 "translate.cpp"
                  var $cmp286=((($conv285))|0)!=39; //@line 1088 "translate.cpp"
                  if (!($cmp286)) { __label__ = 93;break $land_lhs_true283$$if_end289$115; } //@line 1088 "translate.cpp"
  
                  var $146=HEAP32[(($wordx)>>2)]; //@line 1091 "translate.cpp"
                  var $arrayidx288=(($146+-1)&4294967295); //@line 1091 "translate.cpp"
                  HEAP8[($arrayidx288)]=32; //@line 1091 "translate.cpp"
                  ; //@line 1092 "translate.cpp"
                }
              } while(0);
  
              var $147=$tr_addr; //@line 1095 "translate.cpp"
              var $148=HEAP32[(($wordx)>>2)]; //@line 1095 "translate.cpp"
              var $arraydecay290=(($phonemes)&4294967295); //@line 1095 "translate.cpp"
              var $arraydecay291=(($end_phonemes)&4294967295); //@line 1095 "translate.cpp"
              var $149=$wflags; //@line 1095 "translate.cpp"
              var $arraydecay292=(($dictionary_flags)&4294967295); //@line 1095 "translate.cpp"
              var $call293=__Z14TranslateRulesP10TranslatorPcS1_iS1_iPj($147, $148, $arraydecay290, 160, $arraydecay291, $149, $arraydecay292); //@line 1095 "translate.cpp"
              $end_type=$call293; //@line 1095 "translate.cpp"
              var $arrayidx294=(($phonemes)&4294967295); //@line 1097 "translate.cpp"
              var $150=HEAP8[($arrayidx294)]; //@line 1097 "translate.cpp"
              var $conv295=(tempInt=(($150)),(tempInt>=128?tempInt-256:tempInt)); //@line 1097 "translate.cpp"
              var $cmp296=((($conv295))|0)==21; //@line 1097 "translate.cpp"
              var $arraydecay298=(($phonemes)&4294967295); //@line 1100 "translate.cpp"
              if ($cmp296) { __label__ = 94;; } else { __label__ = 95;; } //@line 1097 "translate.cpp"
              if (__label__ == 94) {
  
                var $call299=_strcpy(((_word_phonemes)&4294967295), $arraydecay298); //@line 1100 "translate.cpp"
                $retval=0; //@line 1101 "translate.cpp"
                __label__ = 236;break $if_then12$$if_end13$5; //@line 1101 "translate.cpp"
              }
              else if (__label__ == 95) {
  
                var $151=HEAP8[($arraydecay298)]; //@line 1104 "translate.cpp"
                var $conv302=(tempInt=(($151)),(tempInt>=128?tempInt-256:tempInt)); //@line 1104 "translate.cpp"
                var $cmp303=((($conv302))|0)==0; //@line 1104 "translate.cpp"
                if ($cmp303) { __label__ = 96;; } else { __label__ = 101;; } //@line 1104 "translate.cpp"
                $land_lhs_true304$$if_end321$122: do { 
                  if (__label__ == 96) {
  
                    var $arrayidx305=(($end_phonemes)&4294967295); //@line 1104 "translate.cpp"
                    var $152=HEAP8[($arrayidx305)]; //@line 1104 "translate.cpp"
                    var $conv306=(tempInt=(($152)),(tempInt>=128?tempInt-256:tempInt)); //@line 1104 "translate.cpp"
                    var $cmp307=((($conv306))|0)==0; //@line 1104 "translate.cpp"
                    if (!($cmp307)) { __label__ = 101;break $land_lhs_true304$$if_end321$122; } //@line 1104 "translate.cpp"
  
                    var $153=HEAP32[(($wordx)>>2)]; //@line 1109 "translate.cpp"
                    var $call309=__Z7utf8_inPiPKc($wc, $153); //@line 1109 "translate.cpp"
                    var $154=$word_length; //@line 1110 "translate.cpp"
                    var $cmp310=((($154))|0)==1; //@line 1110 "translate.cpp"
                    if (!($cmp310)) { __label__ = 101;break $land_lhs_true304$$if_end321$122; } //@line 1110 "translate.cpp"
  
                    var $155=HEAP32[(($wc)>>2)]; //@line 1110 "translate.cpp"
                    var $call312=__Z7IsAlphaj($155); //@line 1110 "translate.cpp"
                    var $tobool313=((($call312))|0)!=0; //@line 1110 "translate.cpp"
                    if (!($tobool313)) { __label__ = 101;break $land_lhs_true304$$if_end321$122; } //@line 1110 "translate.cpp"
  
                    var $156=$tr_addr; //@line 1112 "translate.cpp"
                    var $157=HEAP32[(($wordx)>>2)]; //@line 1112 "translate.cpp"
                    var $arraydecay315=(($phonemes)&4294967295); //@line 1112 "translate.cpp"
                    var $158=$spell_word; //@line 1112 "translate.cpp"
                    var $call316=__ZL22SpeakIndividualLettersP10TranslatorPcS1_i($156, $157, $arraydecay315, $158); //@line 1112 "translate.cpp"
                    HEAP32[(($wordx)>>2)]=$call316; //@line 1112 "translate.cpp"
                    var $cmp317=((($call316))|0)==0; //@line 1112 "translate.cpp"
                    if (!($cmp317)) { __label__ = 101;break $land_lhs_true304$$if_end321$122; } //@line 1112 "translate.cpp"
  
                    $retval=0; //@line 1114 "translate.cpp"
                    __label__ = 236;break $if_then12$$if_end13$5; //@line 1114 "translate.cpp"
                  }
                } while(0);
  
                var $159=HEAP32[(($wordx)>>2)]; //@line 1119 "translate.cpp"
                var $arrayidx322=(($159+-1)&4294967295); //@line 1119 "translate.cpp"
                var $160=HEAP8[($arrayidx322)]; //@line 1119 "translate.cpp"
                $c_temp=$160; //@line 1119 "translate.cpp"
                $found=0; //@line 1121 "translate.cpp"
                $confirm_prefix=1; //@line 1122 "translate.cpp"
                $loopcount=0; //@line 1123 "translate.cpp"
                var $arrayidx333=(($phonemes2)&4294967295); //@line 1134 "translate.cpp"
                var $arraydecay334=(($phonemes2)&4294967295); //@line 1135 "translate.cpp"
                var $arraydecay335=(($end_phonemes2)&4294967295); //@line 1135 "translate.cpp"
                var $arraydecay338=(($dictionary_flags)&4294967295); //@line 1135 "translate.cpp"
                var $arraydecay342=(($word_copy)&4294967295); //@line 1138 "translate.cpp"
                var $arraydecay344=(($phonemes)&4294967295); //@line 1139 "translate.cpp"
                var $arraydecay345=(($end_phonemes)&4294967295); //@line 1139 "translate.cpp"
                var $arraydecay347=(($dictionary_flags)&4294967295); //@line 1139 "translate.cpp"
                var $arraydecay349=(($word_copy)&4294967295); //@line 1140 "translate.cpp"
                var $arraydecay350=(($word_copy)&4294967295); //@line 1140 "translate.cpp"
                var $arraydecay355=(($phonemes)&4294967295); //@line 1146 "translate.cpp"
                var $arraydecay356=(($phonemes2)&4294967295); //@line 1146 "translate.cpp"
                var $arraydecay358=(($end_phonemes)&4294967295); //@line 1147 "translate.cpp"
                var $arraydecay359=(($end_phonemes2)&4294967295); //@line 1147 "translate.cpp"
                var $arraydecay363=(($end_phonemes)&4294967295); //@line 1150 "translate.cpp"
                var $arraydecay364=(($end_phonemes2)&4294967295); //@line 1150 "translate.cpp"
                var $arraydecay365=(($end_phonemes2)&4294967295); //@line 1151 "translate.cpp"
                var $arraydecay418=(($prefix_phonemes2)&4294967295); //@line 1204 "translate.cpp"
                var $arraydecay419=(($end_phonemes)&4294967295); //@line 1204 "translate.cpp"
                var $arrayidx420=(($prefix_chars+1)&4294967295); //@line 1205 "translate.cpp"
                var $arraydecay421=(($prefix_phonemes)&4294967295); //@line 1206 "translate.cpp"
                var $arraydecay422=(($phonemes)&4294967295); //@line 1206 "translate.cpp"
                var $arraydecay424=(($phonemes)&4294967295); //@line 1209 "translate.cpp"
                var $arraydecay425=(($dictionary_flags)&4294967295); //@line 1209 "translate.cpp"
                var $arraydecay429=(($prefix_phonemes)&4294967295); //@line 1212 "translate.cpp"
                var $arraydecay430=(($phonemes)&4294967295); //@line 1212 "translate.cpp"
                var $arrayidx433=(($dictionary_flags)&4294967295); //@line 1214 "translate.cpp"
                var $arrayidx437=(($prefix_phonemes)&4294967295); //@line 1216 "translate.cpp"
                var $arraydecay438=(($prefix_phonemes)&4294967295); //@line 1217 "translate.cpp"
                var $arrayidx446=(($end_phonemes)&4294967295); //@line 1224 "translate.cpp"
                var $arraydecay447=(($phonemes)&4294967295); //@line 1227 "translate.cpp"
                var $arraydecay448=(($dictionary_flags2)&4294967295); //@line 1227 "translate.cpp"
                var $arrayidx450=(($dictionary_flags)&4294967295); //@line 1228 "translate.cpp"
                var $arrayidx453=(($dictionary_flags2)&4294967295); //@line 1230 "translate.cpp"
                var $arrayidx454=(($dictionary_flags)&4294967295); //@line 1230 "translate.cpp"
                var $arrayidx455=(($dictionary_flags2+4)&4294967295); //@line 1231 "translate.cpp"
                var $arrayidx456=(($dictionary_flags+4)&4294967295); //@line 1231 "translate.cpp"
                var $arraydecay461=(($phonemes)&4294967295); //@line 1237 "translate.cpp"
                var $arraydecay462=(($end_phonemes)&4294967295); //@line 1237 "translate.cpp"
                var $arraydecay463=(($dictionary_flags)&4294967295); //@line 1237 "translate.cpp"
                var $arrayidx465=(($phonemes)&4294967295); //@line 1239 "translate.cpp"
                var $arraydecay442=(($prefix_phonemes)&4294967295); //@line 1222 "translate.cpp"
                var $arraydecay443=(($end_phonemes)&4294967295); //@line 1222 "translate.cpp"
                var $arrayidx392=(($prefix_chars)&4294967295); //@line 1179 "translate.cpp"
                __lastLabel__ = 101; ; //@line 1123 "translate.cpp"
                $for_cond$129: while(1) { 
  
                  var $161=__lastLabel__ == 136 ? $inc475 : (0); //@line 1123 "translate.cpp"
                  var $cmp323=((($161))|0) < 50; //@line 1123 "translate.cpp"
                  var $162=$end_type; //@line 1123 "translate.cpp"
                  if (!($cmp323)) { __label__ = 137;break $for_cond$129; } //@line 1123 "translate.cpp"
  
                  var $and325=($162) & 1024; //@line 1123 "translate.cpp"
                  var $tobool326=((($and325))|0)!=0; //@line 1123 "translate.cpp"
                  if (!($tobool326)) { __label__ = 137;break $for_cond$129; }
  
                  var $163=$confirm_prefix; //@line 1127 "translate.cpp"
                  var $tobool328=((($163))|0)!=0; //@line 1127 "translate.cpp"
                  if ($tobool328) { __label__ = 105;; } else { __label__ = 111;; } //@line 1127 "translate.cpp"
                  $land_lhs_true329$$if_end370$133: do { 
                    if (__label__ == 105) {
  
                      var $164=$end_type; //@line 1127 "translate.cpp"
                      var $and330=($164) & 131072; //@line 1127 "translate.cpp"
                      var $tobool331=((($and330))|0)!=0; //@line 1127 "translate.cpp"
                      if ($tobool331) { __label__ = 111;break $land_lhs_true329$$if_end370$133; } //@line 1127 "translate.cpp"
  
                      HEAP8[($arrayidx333)]=0; //@line 1134 "translate.cpp"
                      var $165=$tr_addr; //@line 1135 "translate.cpp"
                      var $166=HEAP32[(($wordx)>>2)]; //@line 1135 "translate.cpp"
                      var $167=$wflags; //@line 1135 "translate.cpp"
                      var $or337=($167) | 805306368; //@line 1135 "translate.cpp"
                      var $call339=__Z14TranslateRulesP10TranslatorPcS1_iS1_iPj($165, $166, $arraydecay334, 160, $arraydecay335, $or337, $arraydecay338); //@line 1135 "translate.cpp"
                      $end2=$call339; //@line 1135 "translate.cpp"
                      var $168=$end2; //@line 1136 "translate.cpp"
                      var $tobool340=((($168))|0)!=0; //@line 1136 "translate.cpp"
                      if (!($tobool340)) { __label__ = 111;break $land_lhs_true329$$if_end370$133; } //@line 1136 "translate.cpp"
  
                      var $169=$tr_addr; //@line 1138 "translate.cpp"
                      var $170=HEAP32[(($wordx)>>2)]; //@line 1138 "translate.cpp"
                      var $171=$end2; //@line 1138 "translate.cpp"
                      var $call343=__Z12RemoveEndingP10TranslatorPciS1_($169, $170, $171, $arraydecay342); //@line 1138 "translate.cpp"
                      var $172=$tr_addr; //@line 1139 "translate.cpp"
                      var $173=HEAP32[(($wordx)>>2)]; //@line 1139 "translate.cpp"
                      var $174=$wflags; //@line 1139 "translate.cpp"
                      var $or346=($174) | 268435456; //@line 1139 "translate.cpp"
                      var $call348=__Z14TranslateRulesP10TranslatorPcS1_iS1_iPj($172, $173, $arraydecay344, 160, $arraydecay345, $or346, $arraydecay347); //@line 1139 "translate.cpp"
                      $end_type=$call348; //@line 1139 "translate.cpp"
                      var $175=HEAP32[(($wordx)>>2)]; //@line 1140 "translate.cpp"
                      var $call351=_strlen($arraydecay350); //@line 1140 "translate.cpp"
                      var $src$, $dest$, $stop$, $stop4$, $fast$;
                      $src$ = $arraydecay349;
                      $dest$ = $175;
                      $stop$ = $src$ + $call351;
                      if (($dest$%4) == ($src$%4) && $call351 > 8) {
                        while ($src$%4 !== 0 && $src$ < $stop$) {
                          ; HEAP8[$dest$++] = HEAP8[$src$++];
                        }
                        $src$ >>= 2;
                        $dest$ >>= 2;
                        $stop4$ = $stop$ >> 2;
                        while ($src$ < $stop4$) {
                      ;;;    HEAP32[$dest$++] = HEAP32[$src$++];
                        }
                        $src$ <<= 2;
                        $dest$ <<= 2;
                      }while ($src$ < $stop$) {
                        ; HEAP8[$dest$++] = HEAP8[$src$++];
                      }; //@line 1140 "translate.cpp"
                      var $176=$end_type; //@line 1141 "translate.cpp"
                      var $and352=($176) & 1024; //@line 1141 "translate.cpp"
                      var $cmp353=((($and352))|0)==0; //@line 1141 "translate.cpp"
                      if ($cmp353) { __label__ = 108;; } else { __label__ = 110;; } //@line 1141 "translate.cpp"
                      $if_then354$$if_end368$137: do { 
                        if (__label__ == 108) {
  
                          var $177=$end2; //@line 1145 "translate.cpp"
                          $end_type=$177; //@line 1145 "translate.cpp"
                          var $call357=_strcpy($arraydecay355, $arraydecay356); //@line 1146 "translate.cpp"
                          var $call360=_strcpy($arraydecay358, $arraydecay359); //@line 1147 "translate.cpp"
                          var $178=HEAP32[((_option_phonemes)>>2)]; //@line 1148 "translate.cpp"
                          var $cmp361=((($178))|0)==2; //@line 1148 "translate.cpp"
                          if (!($cmp361)) { __label__ = 110;break $if_then354$$if_end368$137; } //@line 1148 "translate.cpp"
  
                          __Z14DecodePhonemesPKcPc($arraydecay363, $arraydecay364); //@line 1150 "translate.cpp"
                          var $179=HEAP32[((_f_trans)>>2)]; //@line 1151 "translate.cpp"
                          var $call366=_fprintf($179, ((__str4547)&4294967295), allocate([$arraydecay365,0,0,0], ["i8*",0,0,0], ALLOC_STACK)); //@line 1151 "translate.cpp"
                          ; //@line 1152 "translate.cpp"
                        }
                      } while(0);
  
                      $confirm_prefix=0; //@line 1154 "translate.cpp"
                      __label__ = 136;break $land_lhs_true329$$if_end370$133; //@line 1155 "translate.cpp"
                    }
                  } while(0);
                  $if_end370$$for_inc474$141: do { 
                    if (__label__ == 111) {
  
                      var $180=$end_type; //@line 1159 "translate.cpp"
                      $prefix_type=$180; //@line 1159 "translate.cpp"
                      var $181=$prefix_type; //@line 1161 "translate.cpp"
                      var $and371=($181) & 2048; //@line 1161 "translate.cpp"
                      var $tobool372=((($and371))|0)!=0; //@line 1161 "translate.cpp"
                      if ($tobool372) { __label__ = 112;; } else { __label__ = 113;; } //@line 1161 "translate.cpp"
                      if (__label__ == 112) {
  
                        var $182=$tr_addr; //@line 1163 "translate.cpp"
                        var $expect_verb=(($182+8272)&4294967295); //@line 1163 "translate.cpp"
                        HEAP32[(($expect_verb)>>2)]=1; //@line 1163 "translate.cpp"
                        ; //@line 1164 "translate.cpp"
                      }
  
                      var $183=$c_temp; //@line 1166 "translate.cpp"
                      var $184=HEAP32[(($wordx)>>2)]; //@line 1166 "translate.cpp"
                      var $arrayidx375=(($184+-1)&4294967295); //@line 1166 "translate.cpp"
                      HEAP8[($arrayidx375)]=$183; //@line 1166 "translate.cpp"
                      var $185=$prefix_type; //@line 1168 "translate.cpp"
                      var $and376=($185) & 131072; //@line 1168 "translate.cpp"
                      var $cmp377=((($and376))|0)==0; //@line 1168 "translate.cpp"
                      if ($cmp377) { __label__ = 114;; } else { __label__ = 118;; } //@line 1168 "translate.cpp"
                      $if_then378$$if_else391$146: do { 
                        if (__label__ == 114) {
  
                          var $186=$prefix_type; //@line 1170 "translate.cpp"
                          var $and379=($186) & 15; //@line 1170 "translate.cpp"
                          $ix=$and379; //@line 1170 "translate.cpp"
                          var $cmp38119=((($and379))|0) > 0; //@line 1170 "translate.cpp"
                          if (!($cmp38119)) { __label__ = 124;break $if_then378$$if_else391$146; } //@line 1170 "translate.cpp"
                          $for_body382$148: while(1) { 
  
                            var $187=HEAP32[(($wordx)>>2)]; //@line 1172 "translate.cpp"
                            var $incdec_ptr383=(($187+1)&4294967295); //@line 1172 "translate.cpp"
                            HEAP32[(($wordx)>>2)]=$incdec_ptr383; //@line 1172 "translate.cpp"
                            var $188=HEAP32[(($wordx)>>2)]; //@line 1173 "translate.cpp"
                            var $189=HEAP8[($188)]; //@line 1173 "translate.cpp"
                            var $conv38516=(tempInt=(($189)),(tempInt>=128?tempInt-256:tempInt)); //@line 1173 "translate.cpp"
                            var $and38617=($conv38516) & 192; //@line 1173 "translate.cpp"
                            var $cmp38718=((($and38617))|0)==128; //@line 1173 "translate.cpp"
                            if ($cmp38718) { __label__ = 116;; } else { __label__ = 117;; } //@line 1173 "translate.cpp"
                            $while_body388$$for_inc$150: do { 
                              if (__label__ == 116) {
                                while(1) { 
  
                                  var $190=HEAP32[(($wordx)>>2)]; //@line 1173 "translate.cpp"
                                  var $incdec_ptr389=(($190+1)&4294967295); //@line 1173 "translate.cpp"
                                  HEAP32[(($wordx)>>2)]=$incdec_ptr389; //@line 1173 "translate.cpp"
                                  var $191=HEAP32[(($wordx)>>2)]; //@line 1173 "translate.cpp"
                                  var $192=HEAP8[($191)]; //@line 1173 "translate.cpp"
                                  var $conv385=(tempInt=(($192)),(tempInt>=128?tempInt-256:tempInt)); //@line 1173 "translate.cpp"
                                  var $and386=($conv385) & 192; //@line 1173 "translate.cpp"
                                  var $cmp387=((($and386))|0)==128; //@line 1173 "translate.cpp"
                                  if ($cmp387) { __label__ = 116;continue ; } else { __label__ = 117;break $while_body388$$for_inc$150; } //@line 1173 "translate.cpp"
                                }
                              }
                            } while(0);
  
                            var $193=$ix; //@line 1170 "translate.cpp"
                            var $dec=((($193)-1)&4294967295); //@line 1170 "translate.cpp"
                            $ix=$dec; //@line 1170 "translate.cpp"
                            var $cmp381=((($dec))|0) > 0; //@line 1170 "translate.cpp"
                            if ($cmp381) { __label__ = 115;continue $for_body382$148; } else { __label__ = 124;break $if_then378$$if_else391$146; } //@line 1170 "translate.cpp"
                          }
                        }
                        else if (__label__ == 118) {
  
                          $pfix=1; //@line 1178 "translate.cpp"
                          HEAP8[($arrayidx392)]=0; //@line 1179 "translate.cpp"
                          var $194=$prefix_type; //@line 1180 "translate.cpp"
                          var $and393=($194) & 63; //@line 1180 "translate.cpp"
                          $n_chars=$and393; //@line 1180 "translate.cpp"
                          $ix=0; //@line 1182 "translate.cpp"
                          var $195=$ix; //@line 1182 "translate.cpp"
                          var $196=$n_chars; //@line 1182 "translate.cpp"
                          var $cmp39520=((($195))|0) < ((($196))|0); //@line 1182 "translate.cpp"
                          if ($cmp39520) { __label__ = 119;; } else { __label__ = 123;; } //@line 1182 "translate.cpp"
                          $for_body396$$for_end410$155: do { 
                            if (__label__ == 119) {
                              $for_body396$156: while(1) { 
  
                                var $197=HEAP32[(($wordx)>>2)]; //@line 1184 "translate.cpp"
                                var $incdec_ptr397=(($197+1)&4294967295); //@line 1184 "translate.cpp"
                                HEAP32[(($wordx)>>2)]=$incdec_ptr397; //@line 1184 "translate.cpp"
                                var $198=HEAP8[($197)]; //@line 1184 "translate.cpp"
                                var $199=$pfix; //@line 1184 "translate.cpp"
                                var $inc398=((($199)+1)&4294967295); //@line 1184 "translate.cpp"
                                $pfix=$inc398; //@line 1184 "translate.cpp"
                                var $arrayidx399=(($prefix_chars+$199)&4294967295); //@line 1184 "translate.cpp"
                                HEAP8[($arrayidx399)]=$198; //@line 1184 "translate.cpp"
                                var $200=$prefix_type; //@line 1186 "translate.cpp"
                                var $and400=($200) & 131072; //@line 1186 "translate.cpp"
                                var $tobool401=((($and400))|0)!=0; //@line 1186 "translate.cpp"
                                if ($tobool401) { __label__ = 120;; } else { __label__ = 122;; } //@line 1186 "translate.cpp"
                                $land_lhs_true402$$for_inc408$158: do { 
                                  if (__label__ == 120) {
  
                                    var $201=$ix; //@line 1186 "translate.cpp"
                                    var $202=$n_chars; //@line 1186 "translate.cpp"
                                    var $sub=((($202)-1)&4294967295); //@line 1186 "translate.cpp"
                                    var $cmp403=((($201))|0)==((($sub))|0); //@line 1186 "translate.cpp"
                                    if (!($cmp403)) { __label__ = 122;break $land_lhs_true402$$for_inc408$158; } //@line 1186 "translate.cpp"
  
                                    var $203=$pfix; //@line 1188 "translate.cpp"
                                    var $sub405=((($203)-1)&4294967295); //@line 1188 "translate.cpp"
                                    var $arrayidx406=(($prefix_chars+$sub405)&4294967295); //@line 1188 "translate.cpp"
                                    HEAP8[($arrayidx406)]=0; //@line 1188 "translate.cpp"
                                    ; //@line 1189 "translate.cpp"
                                  }
                                } while(0);
  
                                var $204=$ix; //@line 1182 "translate.cpp"
                                var $inc409=((($204)+1)&4294967295); //@line 1182 "translate.cpp"
                                $ix=$inc409; //@line 1182 "translate.cpp"
                                var $205=$ix; //@line 1182 "translate.cpp"
                                var $206=$n_chars; //@line 1182 "translate.cpp"
                                var $cmp395=((($205))|0) < ((($206))|0); //@line 1182 "translate.cpp"
                                if ($cmp395) { __label__ = 119;continue $for_body396$156; } else { __label__ = 123;break $for_body396$$for_end410$155; } //@line 1182 "translate.cpp"
                              }
                            }
                          } while(0);
  
                          var $207=$pfix; //@line 1191 "translate.cpp"
                          var $arrayidx411=(($prefix_chars+$207)&4294967295); //@line 1191 "translate.cpp"
                          HEAP8[($arrayidx411)]=0; //@line 1191 "translate.cpp"
                          ;
                        }
                      } while(0);
  
                      var $208=HEAP32[(($wordx)>>2)]; //@line 1193 "translate.cpp"
                      var $arrayidx413=(($208+-1)&4294967295); //@line 1193 "translate.cpp"
                      var $209=HEAP8[($arrayidx413)]; //@line 1193 "translate.cpp"
                      $c_temp=$209; //@line 1193 "translate.cpp"
                      var $210=HEAP32[(($wordx)>>2)]; //@line 1194 "translate.cpp"
                      var $arrayidx414=(($210+-1)&4294967295); //@line 1194 "translate.cpp"
                      HEAP8[($arrayidx414)]=32; //@line 1194 "translate.cpp"
                      $confirm_prefix=1; //@line 1195 "translate.cpp"
                      var $211=$prefix_type; //@line 1197 "translate.cpp"
                      var $and415=($211) & 131072; //@line 1197 "translate.cpp"
                      var $tobool416=((($and415))|0)!=0; //@line 1197 "translate.cpp"
                      if ($tobool416) { __label__ = 125;; } else { __label__ = 129;; } //@line 1197 "translate.cpp"
                      $if_then417$$if_else441$164: do { 
                        if (__label__ == 125) {
  
                          __Z8strncpy0PcPKci($arraydecay418, $arraydecay419, 12); //@line 1204 "translate.cpp"
                          HEAP32[(($wordpf)>>2)]=$arrayidx420; //@line 1205 "translate.cpp"
                          var $call423=_strcpy($arraydecay421, $arraydecay422); //@line 1206 "translate.cpp"
                          var $212=$tr_addr; //@line 1209 "translate.cpp"
                          var $213=$wtab_addr; //@line 1209 "translate.cpp"
                          var $call426=__Z14LookupDictListP10TranslatorPPcS1_PjiP8WORD_TAB($212, $wordpf, $arraydecay424, $arraydecay425, 0, $213); //@line 1209 "translate.cpp"
                          $found=$call426; //@line 1209 "translate.cpp"
                          var $214=$found; //@line 1210 "translate.cpp"
                          var $tobool427=((($214))|0)!=0; //@line 1210 "translate.cpp"
                          if ($tobool427) { __label__ = 126;; } else { __label__ = 127;; } //@line 1210 "translate.cpp"
                          if (__label__ == 126) {
  
                            var $call431=_strcpy($arraydecay429, $arraydecay430); //@line 1212 "translate.cpp"
                            ; //@line 1213 "translate.cpp"
                          }
  
                          var $215=HEAP32[(($arrayidx433)>>2)]; //@line 1214 "translate.cpp"
                          var $and434=($215) & 131072; //@line 1214 "translate.cpp"
                          var $tobool435=((($and434))|0)!=0; //@line 1214 "translate.cpp"
                          if (!($tobool435)) { __label__ = 130;break $if_then417$$if_else441$164; } //@line 1214 "translate.cpp"
  
                          HEAP8[($arrayidx437)]=0; //@line 1216 "translate.cpp"
                          var $216=$tr_addr; //@line 1217 "translate.cpp"
                          var $217=HEAP32[(($wordpf)>>2)]; //@line 1217 "translate.cpp"
                          var $call439=__ZL22SpeakIndividualLettersP10TranslatorPcS1_i($216, $217, $arraydecay438, 1); //@line 1217 "translate.cpp"
                          ; //@line 1218 "translate.cpp"
                        }
                        else if (__label__ == 129) {
  
                          var $call444=_strcat($arraydecay442, $arraydecay443); //@line 1222 "translate.cpp"
                          ;
                        }
                      } while(0);
  
                      HEAP8[($arrayidx446)]=0; //@line 1224 "translate.cpp"
                      $end_type=0; //@line 1226 "translate.cpp"
                      var $218=$tr_addr; //@line 1227 "translate.cpp"
                      var $219=$wtab_addr; //@line 1227 "translate.cpp"
                      var $call449=__Z14LookupDictListP10TranslatorPPcS1_PjiP8WORD_TAB($218, $wordx, $arraydecay447, $arraydecay448, 1024, $219); //@line 1227 "translate.cpp"
                      $found=$call449; //@line 1227 "translate.cpp"
                      var $220=HEAP32[(($arrayidx450)>>2)]; //@line 1228 "translate.cpp"
                      var $cmp451=((($220))|0)==0; //@line 1228 "translate.cpp"
                      if ($cmp451) { __label__ = 131;; } else { __label__ = 132;; } //@line 1228 "translate.cpp"
                      if (__label__ == 131) {
  
                        var $221=HEAP32[(($arrayidx453)>>2)]; //@line 1230 "translate.cpp"
                        HEAP32[(($arrayidx454)>>2)]=$221; //@line 1230 "translate.cpp"
                        var $222=HEAP32[(($arrayidx455)>>2)]; //@line 1231 "translate.cpp"
                        HEAP32[(($arrayidx456)>>2)]=$222; //@line 1231 "translate.cpp"
                        ; //@line 1232 "translate.cpp"
                      }
                      else if (__label__ == 132) {
  
                        $prefix_flags=1; //@line 1234 "translate.cpp"
                        ;
                      }
  
                      var $223=$found; //@line 1235 "translate.cpp"
                      var $cmp459=((($223))|0)==0; //@line 1235 "translate.cpp"
                      if (!($cmp459)) { __label__ = 136;break $if_end370$$for_inc474$141; } //@line 1235 "translate.cpp"
  
                      var $224=$tr_addr; //@line 1237 "translate.cpp"
                      var $225=HEAP32[(($wordx)>>2)]; //@line 1237 "translate.cpp"
                      var $call464=__Z14TranslateRulesP10TranslatorPcS1_iS1_iPj($224, $225, $arraydecay461, 160, $arraydecay462, 0, $arraydecay463); //@line 1237 "translate.cpp"
                      $end_type=$call464; //@line 1237 "translate.cpp"
                      var $226=HEAP8[($arrayidx465)]; //@line 1239 "translate.cpp"
                      var $conv466=(tempInt=(($226)),(tempInt>=128?tempInt-256:tempInt)); //@line 1239 "translate.cpp"
                      var $cmp467=((($conv466))|0)==21; //@line 1239 "translate.cpp"
                      if ($cmp467) { __label__ = 135;break $for_cond$129; } //@line 1239 "translate.cpp"
                    }
                  } while(0);
  
                  var $229=$loopcount; //@line 1123 "translate.cpp"
                  var $inc475=((($229)+1)&4294967295); //@line 1123 "translate.cpp"
                  $loopcount=$inc475; //@line 1123 "translate.cpp"
                  __lastLabel__ = 136; __label__ = 102;continue $for_cond$129; //@line 1123 "translate.cpp"
                }
                if (__label__ == 137) {
  
                  var $cmp477=((($162))|0)!=0; //@line 1249 "translate.cpp"
                  if ($cmp477) { __label__ = 138;; } else { __label__ = 165;; } //@line 1249 "translate.cpp"
                  $land_lhs_true478$$if_end603$180: do { 
                    if (__label__ == 138) {
  
                      var $230=$end_type; //@line 1249 "translate.cpp"
                      var $and479=($230) & 1024; //@line 1249 "translate.cpp"
                      var $tobool480=((($and479))|0)!=0; //@line 1249 "translate.cpp"
                      if ($tobool480) { __label__ = 165;break $land_lhs_true478$$if_end603$180; } //@line 1249 "translate.cpp"
  
                      var $arraydecay483=(($phonemes2482)&4294967295); //@line 1252 "translate.cpp"
                      var $arraydecay484=(($phonemes)&4294967295); //@line 1252 "translate.cpp"
                      var $call485=_strcpy($arraydecay483, $arraydecay484); //@line 1252 "translate.cpp"
                      var $231=$tr_addr; //@line 1255 "translate.cpp"
                      var $232=HEAP32[(($wordx)>>2)]; //@line 1255 "translate.cpp"
                      var $233=$end_type; //@line 1255 "translate.cpp"
                      var $arraydecay486=(($word_copy)&4294967295); //@line 1255 "translate.cpp"
                      var $call487=__Z12RemoveEndingP10TranslatorPciS1_($231, $232, $233, $arraydecay486); //@line 1255 "translate.cpp"
                      $end_flags=$call487; //@line 1255 "translate.cpp"
                      var $arrayidx488=(($phonemes)&4294967295); //@line 1257 "translate.cpp"
                      HEAP8[($arrayidx488)]=0; //@line 1257 "translate.cpp"
                      var $arrayidx489=(($prefix_phonemes)&4294967295); //@line 1259 "translate.cpp"
                      var $234=HEAP8[($arrayidx489)]; //@line 1259 "translate.cpp"
                      var $conv490=(tempInt=(($234)),(tempInt>=128?tempInt-256:tempInt)); //@line 1259 "translate.cpp"
                      var $cmp491=((($conv490))|0)!=0; //@line 1259 "translate.cpp"
                      if ($cmp491) { __label__ = 140;; } else { __label__ = 148;; } //@line 1259 "translate.cpp"
                      $if_then492$$if_end526$183: do { 
                        if (__label__ == 140) {
  
                          var $235=$c_temp; //@line 1262 "translate.cpp"
                          var $236=HEAP32[(($wordx)>>2)]; //@line 1262 "translate.cpp"
                          var $arrayidx493=(($236+-1)&4294967295); //@line 1262 "translate.cpp"
                          HEAP8[($arrayidx493)]=$235; //@line 1262 "translate.cpp"
                          var $237=$tr_addr; //@line 1263 "translate.cpp"
                          var $arraydecay494=(($phonemes)&4294967295); //@line 1263 "translate.cpp"
                          var $arraydecay495=(($dictionary_flags2)&4294967295); //@line 1263 "translate.cpp"
                          var $238=$end_flags; //@line 1263 "translate.cpp"
                          var $239=$wtab_addr; //@line 1263 "translate.cpp"
                          var $call496=__Z14LookupDictListP10TranslatorPPcS1_PjiP8WORD_TAB($237, $word1, $arraydecay494, $arraydecay495, $238, $239); //@line 1263 "translate.cpp"
                          $found=$call496; //@line 1263 "translate.cpp"
                          var $240=HEAP32[(($wordx)>>2)]; //@line 1264 "translate.cpp"
                          var $arrayidx497=(($240+-1)&4294967295); //@line 1264 "translate.cpp"
                          HEAP8[($arrayidx497)]=32; //@line 1264 "translate.cpp"
                          var $arrayidx498=(($phonemes)&4294967295); //@line 1265 "translate.cpp"
                          var $241=HEAP8[($arrayidx498)]; //@line 1265 "translate.cpp"
                          var $conv499=(tempInt=(($241)),(tempInt>=128?tempInt-256:tempInt)); //@line 1265 "translate.cpp"
                          var $cmp500=((($conv499))|0)==21; //@line 1265 "translate.cpp"
                          if ($cmp500) { __label__ = 141;; } else { __label__ = 142;; } //@line 1265 "translate.cpp"
                          if (__label__ == 141) {
  
                            var $242=HEAP32[(($wordx)>>2)]; //@line 1268 "translate.cpp"
                            var $arraydecay502=(($word_copy)&4294967295); //@line 1268 "translate.cpp"
                            var $arraydecay503=(($word_copy)&4294967295); //@line 1268 "translate.cpp"
                            var $call504=_strlen($arraydecay503); //@line 1268 "translate.cpp"
                            var $src$, $dest$, $stop$, $stop4$, $fast$;
                            $src$ = $arraydecay502;
                            $dest$ = $242;
                            $stop$ = $src$ + $call504;
                            if (($dest$%4) == ($src$%4) && $call504 > 8) {
                              while ($src$%4 !== 0 && $src$ < $stop$) {
                                ; HEAP8[$dest$++] = HEAP8[$src$++];
                              }
                              $src$ >>= 2;
                              $dest$ >>= 2;
                              $stop4$ = $stop$ >> 2;
                              while ($src$ < $stop4$) {
                            ;;;    HEAP32[$dest$++] = HEAP32[$src$++];
                              }
                              $src$ <<= 2;
                              $dest$ <<= 2;
                            }while ($src$ < $stop$) {
                              ; HEAP8[$dest$++] = HEAP8[$src$++];
                            }; //@line 1268 "translate.cpp"
                            var $arraydecay505=(($phonemes)&4294967295); //@line 1269 "translate.cpp"
                            var $call506=_strcpy(((_word_phonemes)&4294967295), $arraydecay505); //@line 1269 "translate.cpp"
                            $retval=0; //@line 1270 "translate.cpp"
                            __label__ = 236;break $if_then12$$if_end13$5; //@line 1270 "translate.cpp"
                          }
                          else if (__label__ == 142) {
  
                            var $arrayidx508=(($dictionary_flags)&4294967295); //@line 1272 "translate.cpp"
                            var $243=HEAP32[(($arrayidx508)>>2)]; //@line 1272 "translate.cpp"
                            var $cmp509=((($243))|0)==0; //@line 1272 "translate.cpp"
                            if ($cmp509) { __label__ = 143;; } else { __label__ = 144;; } //@line 1272 "translate.cpp"
                            if (__label__ == 143) {
  
                              var $arrayidx511=(($dictionary_flags2)&4294967295); //@line 1274 "translate.cpp"
                              var $244=HEAP32[(($arrayidx511)>>2)]; //@line 1274 "translate.cpp"
                              var $arrayidx512=(($dictionary_flags)&4294967295); //@line 1274 "translate.cpp"
                              HEAP32[(($arrayidx512)>>2)]=$244; //@line 1274 "translate.cpp"
                              var $arrayidx513=(($dictionary_flags2+4)&4294967295); //@line 1275 "translate.cpp"
                              var $245=HEAP32[(($arrayidx513)>>2)]; //@line 1275 "translate.cpp"
                              var $arrayidx514=(($dictionary_flags+4)&4294967295); //@line 1275 "translate.cpp"
                              HEAP32[(($arrayidx514)>>2)]=$245; //@line 1275 "translate.cpp"
                              ; //@line 1276 "translate.cpp"
                            }
  
                            var $246=$found; //@line 1277 "translate.cpp"
                            var $tobool516=((($246))|0)!=0; //@line 1277 "translate.cpp"
                            if ($tobool516) { __label__ = 145;; } else { __label__ = 146;; } //@line 1277 "translate.cpp"
                            if (__label__ == 145) {
  
                              var $arrayidx518=(($prefix_phonemes)&4294967295); //@line 1278 "translate.cpp"
                              HEAP8[($arrayidx518)]=0; //@line 1278 "translate.cpp"
                              var $_pr8=$found; //@line 1280 "translate.cpp"
                              var $cmp520=((($_pr8))|0)==0; //@line 1280 "translate.cpp"
                              if (!($cmp520)) { __label__ = 162;break $if_then492$$if_end526$183; } //@line 1280 "translate.cpp"
                            }
  
                            var $arrayidx522=(($dictionary_flags2)&4294967295); //@line 1280 "translate.cpp"
                            var $247=HEAP32[(($arrayidx522)>>2)]; //@line 1280 "translate.cpp"
                            var $cmp523=((($247))|0)!=0; //@line 1280 "translate.cpp"
                            if (!($cmp523)) { __label__ = 148;break $if_then492$$if_end526$183; } //@line 1280 "translate.cpp"
  
                            $prefix_flags=1; //@line 1281 "translate.cpp"
                            __label__ = 148;break $if_then492$$if_end526$183; //@line 1281 "translate.cpp"
                          }
                        }
                      } while(0);
                      $if_end526$$if_end592$195: do { 
                        if (__label__ == 148) {
  
                          var $_pr10=$found; //@line 1283 "translate.cpp"
                          var $cmp527=((($_pr10))|0)==0; //@line 1283 "translate.cpp"
                          if (!($cmp527)) { __label__ = 162;break $if_end526$$if_end592$195; } //@line 1283 "translate.cpp"
  
                          var $248=$tr_addr; //@line 1285 "translate.cpp"
                          var $arraydecay529=(($phonemes)&4294967295); //@line 1285 "translate.cpp"
                          var $arraydecay530=(($dictionary_flags2)&4294967295); //@line 1285 "translate.cpp"
                          var $249=$end_flags; //@line 1285 "translate.cpp"
                          var $250=$wtab_addr; //@line 1285 "translate.cpp"
                          var $call531=__Z14LookupDictListP10TranslatorPPcS1_PjiP8WORD_TAB($248, $wordx, $arraydecay529, $arraydecay530, $249, $250); //@line 1285 "translate.cpp"
                          $found=$call531; //@line 1285 "translate.cpp"
                          var $arrayidx532=(($phonemes)&4294967295); //@line 1286 "translate.cpp"
                          var $251=HEAP8[($arrayidx532)]; //@line 1286 "translate.cpp"
                          var $conv533=(tempInt=(($251)),(tempInt>=128?tempInt-256:tempInt)); //@line 1286 "translate.cpp"
                          var $cmp534=((($conv533))|0)==21; //@line 1286 "translate.cpp"
                          if ($cmp534) { __label__ = 150;; } else { __label__ = 151;; } //@line 1286 "translate.cpp"
                          if (__label__ == 150) {
  
                            var $252=HEAP32[(($wordx)>>2)]; //@line 1289 "translate.cpp"
                            var $arraydecay536=(($word_copy)&4294967295); //@line 1289 "translate.cpp"
                            var $arraydecay537=(($word_copy)&4294967295); //@line 1289 "translate.cpp"
                            var $call538=_strlen($arraydecay537); //@line 1289 "translate.cpp"
                            var $src$, $dest$, $stop$, $stop4$, $fast$;
                            $src$ = $arraydecay536;
                            $dest$ = $252;
                            $stop$ = $src$ + $call538;
                            if (($dest$%4) == ($src$%4) && $call538 > 8) {
                              while ($src$%4 !== 0 && $src$ < $stop$) {
                                ; HEAP8[$dest$++] = HEAP8[$src$++];
                              }
                              $src$ >>= 2;
                              $dest$ >>= 2;
                              $stop4$ = $stop$ >> 2;
                              while ($src$ < $stop4$) {
                            ;;;    HEAP32[$dest$++] = HEAP32[$src$++];
                              }
                              $src$ <<= 2;
                              $dest$ <<= 2;
                            }while ($src$ < $stop$) {
                              ; HEAP8[$dest$++] = HEAP8[$src$++];
                            }; //@line 1289 "translate.cpp"
                            var $arraydecay539=(($phonemes)&4294967295); //@line 1290 "translate.cpp"
                            var $call540=_strcpy(((_word_phonemes)&4294967295), $arraydecay539); //@line 1290 "translate.cpp"
                            $retval=0; //@line 1291 "translate.cpp"
                            __label__ = 236;break $if_then12$$if_end13$5; //@line 1291 "translate.cpp"
                          }
                          else if (__label__ == 151) {
  
                            var $arrayidx547=(($dictionary_flags)&4294967295); //@line 1299 "translate.cpp"
                            var $253=HEAP32[(($arrayidx547)>>2)]; //@line 1299 "translate.cpp"
                            var $cmp548=((($253))|0)==0; //@line 1299 "translate.cpp"
                            if ($cmp548) { __label__ = 152;; } else { __label__ = 153;; } //@line 1299 "translate.cpp"
                            if (__label__ == 152) {
  
                              var $arrayidx550=(($dictionary_flags2)&4294967295); //@line 1301 "translate.cpp"
                              var $254=HEAP32[(($arrayidx550)>>2)]; //@line 1301 "translate.cpp"
                              var $arrayidx551=(($dictionary_flags)&4294967295); //@line 1301 "translate.cpp"
                              HEAP32[(($arrayidx551)>>2)]=$254; //@line 1301 "translate.cpp"
                              var $arrayidx552=(($dictionary_flags2+4)&4294967295); //@line 1302 "translate.cpp"
                              var $255=HEAP32[(($arrayidx552)>>2)]; //@line 1302 "translate.cpp"
                              var $arrayidx553=(($dictionary_flags+4)&4294967295); //@line 1302 "translate.cpp"
                              HEAP32[(($arrayidx553)>>2)]=$255; //@line 1302 "translate.cpp"
                              ; //@line 1303 "translate.cpp"
                            }
  
                            var $_pr12_pr=$found; //@line 1305 "translate.cpp"
                            var $cmp556=((($_pr12_pr))|0)==0; //@line 1305 "translate.cpp"
                            if (!($cmp556)) { __label__ = 162;break $if_end526$$if_end592$195; } //@line 1305 "translate.cpp"
  
                            var $256=$end_type; //@line 1307 "translate.cpp"
                            var $and558=($256) & 16384; //@line 1307 "translate.cpp"
                            var $tobool559=((($and558))|0)!=0; //@line 1307 "translate.cpp"
                            if ($tobool559) { __label__ = 155;; } else { __label__ = 156;; } //@line 1307 "translate.cpp"
                            if (__label__ == 155) {
  
                              var $arraydecay561=(($phonemes)&4294967295); //@line 1310 "translate.cpp"
                              var $arraydecay562=(($phonemes2482)&4294967295); //@line 1310 "translate.cpp"
                              var $call563=_strcpy($arraydecay561, $arraydecay562); //@line 1310 "translate.cpp"
                              var $257=$tr_addr; //@line 1313 "translate.cpp"
                              var $arraydecay564=(($phonemes)&4294967295); //@line 1313 "translate.cpp"
                              var $arrayidx565=(($dictionary_flags)&4294967295); //@line 1313 "translate.cpp"
                              var $258=HEAP32[(($arrayidx565)>>2)]; //@line 1313 "translate.cpp"
                              __Z21ApplySpecialAttributeP10TranslatorPci($257, $arraydecay564, $258); //@line 1313 "translate.cpp"
                              ; //@line 1314 "translate.cpp"
                            }
                            else if (__label__ == 156) {
  
                              var $259=$end_flags; //@line 1317 "translate.cpp"
                              var $and567=($259) & 4; //@line 1317 "translate.cpp"
                              var $tobool568=((($and567))|0)!=0; //@line 1317 "translate.cpp"
                              if ($tobool568) { __label__ = 157;; } else { __label__ = 158;; } //@line 1317 "translate.cpp"
                              if (__label__ == 157) {
  
                                var $260=$wflags; //@line 1318 "translate.cpp"
                                var $or570=($260) | 8192; //@line 1318 "translate.cpp"
                                $wflags=$or570; //@line 1318 "translate.cpp"
                                ; //@line 1318 "translate.cpp"
                              }
  
                              var $261=$end_type; //@line 1319 "translate.cpp"
                              var $and572=($261) & 262144; //@line 1319 "translate.cpp"
                              var $tobool573=((($and572))|0)!=0; //@line 1319 "translate.cpp"
                              if ($tobool573) { __label__ = 159;; } else { __label__ = 160;; } //@line 1319 "translate.cpp"
                              if (__label__ == 159) {
  
                                var $262=$wflags; //@line 1320 "translate.cpp"
                                var $or575=($262) | 134217728; //@line 1320 "translate.cpp"
                                $wflags=$or575; //@line 1320 "translate.cpp"
                                ; //@line 1320 "translate.cpp"
                              }
  
                              var $263=$tr_addr; //@line 1322 "translate.cpp"
                              var $264=HEAP32[(($wordx)>>2)]; //@line 1322 "translate.cpp"
                              var $arraydecay577=(($phonemes)&4294967295); //@line 1322 "translate.cpp"
                              var $265=$wflags; //@line 1322 "translate.cpp"
                              var $arraydecay578=(($dictionary_flags)&4294967295); //@line 1322 "translate.cpp"
                              var $call579=__Z14TranslateRulesP10TranslatorPcS1_iS1_iPj($263, $264, $arraydecay577, 160, 0, $265, $arraydecay578); //@line 1322 "translate.cpp"
                              var $arrayidx580=(($phonemes)&4294967295); //@line 1324 "translate.cpp"
                              var $266=HEAP8[($arrayidx580)]; //@line 1324 "translate.cpp"
                              var $conv581=(tempInt=(($266)),(tempInt>=128?tempInt-256:tempInt)); //@line 1324 "translate.cpp"
                              var $cmp582=((($conv581))|0)==21; //@line 1324 "translate.cpp"
                              if (!($cmp582)) { __label__ = 162;break $if_end526$$if_end592$195; } //@line 1324 "translate.cpp"
  
                              var $arraydecay584=(($phonemes)&4294967295); //@line 1327 "translate.cpp"
                              var $call585=_strcpy(((_word_phonemes)&4294967295), $arraydecay584); //@line 1327 "translate.cpp"
                              var $267=HEAP32[(($wordx)>>2)]; //@line 1328 "translate.cpp"
                              var $arraydecay586=(($word_copy)&4294967295); //@line 1328 "translate.cpp"
                              var $arraydecay587=(($word_copy)&4294967295); //@line 1328 "translate.cpp"
                              var $call588=_strlen($arraydecay587); //@line 1328 "translate.cpp"
                              var $src$, $dest$, $stop$, $stop4$, $fast$;
                              $src$ = $arraydecay586;
                              $dest$ = $267;
                              $stop$ = $src$ + $call588;
                              if (($dest$%4) == ($src$%4) && $call588 > 8) {
                                while ($src$%4 !== 0 && $src$ < $stop$) {
                                  ; HEAP8[$dest$++] = HEAP8[$src$++];
                                }
                                $src$ >>= 2;
                                $dest$ >>= 2;
                                $stop4$ = $stop$ >> 2;
                                while ($src$ < $stop4$) {
                              ;;;    HEAP32[$dest$++] = HEAP32[$src$++];
                                }
                                $src$ <<= 2;
                                $dest$ <<= 2;
                              }while ($src$ < $stop$) {
                                ; HEAP8[$dest$++] = HEAP8[$src$++];
                              }; //@line 1328 "translate.cpp"
                              var $268=$c_temp; //@line 1329 "translate.cpp"
                              var $269=HEAP32[(($wordx)>>2)]; //@line 1329 "translate.cpp"
                              var $arrayidx589=(($269+-1)&4294967295); //@line 1329 "translate.cpp"
                              HEAP8[($arrayidx589)]=$268; //@line 1329 "translate.cpp"
                              $retval=0; //@line 1330 "translate.cpp"
                              __label__ = 236;break $if_then12$$if_end13$5; //@line 1330 "translate.cpp"
                            }
                          }
                        }
                      } while(0);
  
                      var $270=$end_type; //@line 1335 "translate.cpp"
                      var $and593=($270) & 65536; //@line 1335 "translate.cpp"
                      var $cmp594=((($and593))|0)==0; //@line 1335 "translate.cpp"
                      if ($cmp594) { __label__ = 163;; } else { __label__ = 164;; } //@line 1335 "translate.cpp"
                      if (__label__ == 163) {
  
                        var $271=$tr_addr; //@line 1338 "translate.cpp"
                        var $arraydecay596=(($phonemes)&4294967295); //@line 1338 "translate.cpp"
                        var $arraydecay597=(($end_phonemes)&4294967295); //@line 1338 "translate.cpp"
                        __Z14AppendPhonemesP10TranslatorPciPKc($271, $arraydecay596, 160, $arraydecay597); //@line 1338 "translate.cpp"
                        var $arrayidx598=(($end_phonemes)&4294967295); //@line 1339 "translate.cpp"
                        HEAP8[($arrayidx598)]=0; //@line 1339 "translate.cpp"
                        ; //@line 1340 "translate.cpp"
                      }
  
                      var $272=HEAP32[(($wordx)>>2)]; //@line 1341 "translate.cpp"
                      var $arraydecay600=(($word_copy)&4294967295); //@line 1341 "translate.cpp"
                      var $arraydecay601=(($word_copy)&4294967295); //@line 1341 "translate.cpp"
                      var $call602=_strlen($arraydecay601); //@line 1341 "translate.cpp"
                      var $src$, $dest$, $stop$, $stop4$, $fast$;
                      $src$ = $arraydecay600;
                      $dest$ = $272;
                      $stop$ = $src$ + $call602;
                      if (($dest$%4) == ($src$%4) && $call602 > 8) {
                        while ($src$%4 !== 0 && $src$ < $stop$) {
                          ; HEAP8[$dest$++] = HEAP8[$src$++];
                        }
                        $src$ >>= 2;
                        $dest$ >>= 2;
                        $stop4$ = $stop$ >> 2;
                        while ($src$ < $stop4$) {
                      ;;;    HEAP32[$dest$++] = HEAP32[$src$++];
                        }
                        $src$ <<= 2;
                        $dest$ <<= 2;
                      }while ($src$ < $stop$) {
                        ; HEAP8[$dest$++] = HEAP8[$src$++];
                      }; //@line 1341 "translate.cpp"
                      ; //@line 1342 "translate.cpp"
                    }
                  } while(0);
  
                  var $273=$c_temp; //@line 1343 "translate.cpp"
                  var $274=HEAP32[(($wordx)>>2)]; //@line 1343 "translate.cpp"
                  var $arrayidx604=(($274+-1)&4294967295); //@line 1343 "translate.cpp"
                  HEAP8[($arrayidx604)]=$273; //@line 1343 "translate.cpp"
                  __label__ = 166;break $if_end219$$if_then221$95; //@line 1344 "translate.cpp"
                }
                else if (__label__ == 135) {
  
                  var $227=$c_temp; //@line 1242 "translate.cpp"
                  var $228=HEAP32[(($wordx)>>2)]; //@line 1242 "translate.cpp"
                  var $arrayidx469=(($228+-1)&4294967295); //@line 1242 "translate.cpp"
                  HEAP8[($arrayidx469)]=$227; //@line 1242 "translate.cpp"
                  var $arraydecay470=(($phonemes)&4294967295); //@line 1243 "translate.cpp"
                  var $call471=_strcpy(((_word_phonemes)&4294967295), $arraydecay470); //@line 1243 "translate.cpp"
                  $retval=0; //@line 1244 "translate.cpp"
                  __label__ = 236;break $if_then12$$if_end13$5; //@line 1244 "translate.cpp"
                }
              }
            }
            else if (__label__ == 84) {
  
              var $arraydecay260=(($unpron_phonemes)&4294967295); //@line 1063 "translate.cpp"
              var $call261=_strcpy(((_word_phonemes)&4294967295), $arraydecay260); //@line 1063 "translate.cpp"
              var $arrayidx262=(($unpron_phonemes+1)&4294967295); //@line 1064 "translate.cpp"
              var $call263=_strcmp($arrayidx262, ((__str52704)&4294967295)); //@line 1064 "translate.cpp"
              var $cmp264=((($call263))|0)==0; //@line 1064 "translate.cpp"
              if ($cmp264) { __label__ = 85;; } else { __label__ = 86;; } //@line 1064 "translate.cpp"
              if (__label__ == 85) {
  
                $retval=65536; //@line 1065 "translate.cpp"
                __label__ = 236;break $if_then12$$if_end13$5; //@line 1065 "translate.cpp"
              }
              else if (__label__ == 86) {
  
                $retval=0; //@line 1066 "translate.cpp"
                __label__ = 236;break $if_then12$$if_end13$5; //@line 1066 "translate.cpp"
              }
            }
          }
        } while(0);
        $if_then221$$if_end607$225: do { 
          if (__label__ == 71) {
  
            var $arrayidx222=(($phonemes)&4294967295); //@line 1021 "translate.cpp"
            HEAP8[($arrayidx222)]=0; //@line 1021 "translate.cpp"
            $end_type=0; //@line 1022 "translate.cpp"
            var $111=$tr_addr; //@line 1024 "translate.cpp"
            var $112=HEAP32[(($word1)>>2)]; //@line 1024 "translate.cpp"
            var $arraydecay223=(($phonemes)&4294967295); //@line 1024 "translate.cpp"
            var $113=$spell_word; //@line 1024 "translate.cpp"
            var $call224=__ZL22SpeakIndividualLettersP10TranslatorPcS1_i($111, $112, $arraydecay223, $113); //@line 1024 "translate.cpp"
            var $cmp225=((($call224))|0)==0; //@line 1024 "translate.cpp"
            if (!($cmp225)) { __label__ = 166;break $if_then221$$if_end607$225; } //@line 1024 "translate.cpp"
  
            var $114=$word_length; //@line 1026 "translate.cpp"
            var $cmp227=((($114))|0) > 1; //@line 1026 "translate.cpp"
            if ($cmp227) { __label__ = 73;; } else { __label__ = 74;; } //@line 1026 "translate.cpp"
            if (__label__ == 73) {
  
              $retval=65536; //@line 1027 "translate.cpp"
              __label__ = 236;break $if_then12$$if_end13$5; //@line 1027 "translate.cpp"
            }
            else if (__label__ == 74) {
  
              $retval=0; //@line 1028 "translate.cpp"
              __label__ = 236;break $if_then12$$if_end13$5; //@line 1028 "translate.cpp"
            }
          }
        } while(0);
  
        var $275=$add_plural_suffix; //@line 1347 "translate.cpp"
        var $tobool608=((($275))|0)!=0; //@line 1347 "translate.cpp"
        if ($tobool608) { __label__ = 168;; } else { __label__ = 167;; } //@line 1347 "translate.cpp"
        $if_then612$$lor_lhs_false609$232: do { 
          if (__label__ == 167) {
  
            var $276=$wflags; //@line 1347 "translate.cpp"
            var $and610=($276) & 4; //@line 1347 "translate.cpp"
            var $tobool611=((($and610))|0)!=0; //@line 1347 "translate.cpp"
            if ($tobool611) { __label__ = 168;break $if_then612$$lor_lhs_false609$232; } else { __label__ = 174;break $if_then612$$lor_lhs_false609$232; } //@line 1347 "translate.cpp"
          }
        } while(0);
        $if_then612$$if_end630$234: do { 
          if (__label__ == 168) {
  
            var $277=HEAP32[(($last_char)>>2)]; //@line 1350 "translate.cpp"
            var $cmp613=((($277))|0)==102; //@line 1350 "translate.cpp"
            if ($cmp613) { __label__ = 169;; } else { __label__ = 170;; } //@line 1350 "translate.cpp"
            if (__label__ == 169) {
  
              var $278=$tr_addr; //@line 1351 "translate.cpp"
              var $arraydecay615=(($phonemes)&4294967295); //@line 1351 "translate.cpp"
              var $call616=__Z14TranslateRulesP10TranslatorPcS1_iS1_iPj($278, ((__ZZ13TranslateWordP10TranslatorPciP8WORD_TABE7word_ss+1)&4294967295), $arraydecay615, 160, 0, 0, 0); //@line 1351 "translate.cpp"
              ; //@line 1351 "translate.cpp"
            }
            else if (__label__ == 170) {
  
              var $279=HEAP32[(($last_char)>>2)]; //@line 1353 "translate.cpp"
              var $cmp618=((($279))|0)==0; //@line 1353 "translate.cpp"
              if ($cmp618) { __label__ = 172;; } else { __label__ = 171;; } //@line 1353 "translate.cpp"
              $if_then622$$lor_lhs_false619$239: do { 
                if (__label__ == 171) {
  
                  var $280=HEAP32[(($last_char)>>2)]; //@line 1353 "translate.cpp"
                  var $call620=__Z8strchr_wPKci(((__str5548)&4294967295), $280); //@line 1353 "translate.cpp"
                  var $cmp621=((($call620))|0)==0; //@line 1353 "translate.cpp"
                  if ($cmp621) { __label__ = 172;break $if_then622$$lor_lhs_false619$239; } //@line 1353 "translate.cpp"
  
                  var $282=$tr_addr; //@line 1356 "translate.cpp"
                  var $arraydecay626=(($phonemes)&4294967295); //@line 1356 "translate.cpp"
                  var $call627=__Z14TranslateRulesP10TranslatorPcS1_iS1_iPj($282, ((__ZZ13TranslateWordP10TranslatorPciP8WORD_TABE7word_iz+1)&4294967295), $arraydecay626, 160, 0, 0, 0); //@line 1356 "translate.cpp"
                  __label__ = 174;break $if_then612$$if_end630$234;
                }
              } while(0);
  
              var $281=$tr_addr; //@line 1354 "translate.cpp"
              var $arraydecay623=(($phonemes)&4294967295); //@line 1354 "translate.cpp"
              var $call624=__Z14TranslateRulesP10TranslatorPcS1_iS1_iPj($281, ((__ZZ13TranslateWordP10TranslatorPciP8WORD_TABE7word_zz+1)&4294967295), $arraydecay623, 160, 0, 0, 0); //@line 1354 "translate.cpp"
              ; //@line 1354 "translate.cpp"
            }
          }
        } while(0);
  
        var $283=$emphasize_allcaps; //@line 1359 "translate.cpp"
        var $284=$wflags; //@line 1359 "translate.cpp"
        var $or631=($284) | ($283); //@line 1359 "translate.cpp"
        $wflags=$or631; //@line 1359 "translate.cpp"
        $prefix_stress=0; //@line 1364 "translate.cpp"
        var $arraydecay632=(($prefix_phonemes)&4294967295); //@line 1365 "translate.cpp"
        $p=$arraydecay632; //@line 1365 "translate.cpp"
        var $285=$p; //@line 1365 "translate.cpp"
        var $286=HEAP8[($285)]; //@line 1365 "translate.cpp"
        var $conv63426=(tempInt=(($286)),(tempInt>=128?tempInt-256:tempInt)); //@line 1365 "translate.cpp"
        var $cmp63527=((($conv63426))|0)!=0; //@line 1365 "translate.cpp"
        if ($cmp63527) { __label__ = 175;; } else { __label__ = 179;; } //@line 1365 "translate.cpp"
        $for_body636$$for_end647$244: do { 
          if (__label__ == 175) {
            $for_body636$245: while(1) { 
  
              var $287=$p; //@line 1367 "translate.cpp"
              var $288=HEAP8[($287)]; //@line 1367 "translate.cpp"
              var $conv637=(tempInt=(($288)),(tempInt>=128?tempInt-256:tempInt)); //@line 1367 "translate.cpp"
              var $cmp638=((($conv637))|0)==6; //@line 1367 "translate.cpp"
              if ($cmp638) { __label__ = 177;; } else { __label__ = 176;; } //@line 1367 "translate.cpp"
              $if_then642$$lor_lhs_false639$247: do { 
                if (__label__ == 176) {
  
                  var $289=$p; //@line 1367 "translate.cpp"
                  var $290=HEAP8[($289)]; //@line 1367 "translate.cpp"
                  var $conv640=(tempInt=(($290)),(tempInt>=128?tempInt-256:tempInt)); //@line 1367 "translate.cpp"
                  var $cmp641=((($conv640))|0)==7; //@line 1367 "translate.cpp"
                  if ($cmp641) { __label__ = 177;break $if_then642$$lor_lhs_false639$247; } else { __label__ = 178;break $if_then642$$lor_lhs_false639$247; } //@line 1367 "translate.cpp"
                }
              } while(0);
              if (__label__ == 177) {
  
                var $291=$p; //@line 1369 "translate.cpp"
                var $292=HEAP8[($291)]; //@line 1369 "translate.cpp"
                var $conv643=(tempInt=(($292)),(tempInt>=128?tempInt-256:tempInt)); //@line 1369 "translate.cpp"
                $prefix_stress=$conv643; //@line 1369 "translate.cpp"
                ; //@line 1370 "translate.cpp"
              }
  
              var $293=$p; //@line 1365 "translate.cpp"
              var $incdec_ptr646=(($293+1)&4294967295); //@line 1365 "translate.cpp"
              $p=$incdec_ptr646; //@line 1365 "translate.cpp"
              var $294=$p; //@line 1365 "translate.cpp"
              var $295=HEAP8[($294)]; //@line 1365 "translate.cpp"
              var $conv634=(tempInt=(($295)),(tempInt>=128?tempInt-256:tempInt)); //@line 1365 "translate.cpp"
              var $cmp635=((($conv634))|0)!=0; //@line 1365 "translate.cpp"
              if ($cmp635) { __label__ = 175;continue $for_body636$245; } else { __label__ = 179;break $for_body636$$for_end647$244; } //@line 1365 "translate.cpp"
            }
          }
        } while(0);
  
        var $296=$prefix_flags; //@line 1372 "translate.cpp"
        var $tobool648=((($296))|0)!=0; //@line 1372 "translate.cpp"
        if ($tobool648) { __label__ = 181;; } else { __label__ = 180;; } //@line 1372 "translate.cpp"
        $if_then651$$lor_lhs_false649$253: do { 
          if (__label__ == 180) {
  
            var $297=$prefix_stress; //@line 1372 "translate.cpp"
            var $cmp650=((($297))|0)!=0; //@line 1372 "translate.cpp"
            if ($cmp650) { __label__ = 181;break $if_then651$$lor_lhs_false649$253; } //@line 1372 "translate.cpp"
  
            var $313=$tr_addr; //@line 1415 "translate.cpp"
            var $arraydecay695=(($phonemes)&4294967295); //@line 1415 "translate.cpp"
            var $arraydecay696=(($dictionary_flags)&4294967295); //@line 1415 "translate.cpp"
            __Z13SetWordStressP10TranslatorPcPjii($313, $arraydecay695, $arraydecay696, -1, 0); //@line 1415 "translate.cpp"
            var $arraydecay701=(($unpron_phonemes)&4294967295); //@line 1421 "translate.cpp"
            var $arraydecay702=(($prefix_phonemes)&4294967295); //@line 1421 "translate.cpp"
            var $arraydecay703=(($phonemes)&4294967295); //@line 1421 "translate.cpp"
            var $call704=_snprintf(((_word_phonemes)&4294967295), 160, ((__str6549)&4294967295), allocate([$arraydecay701,0,0,0,$arraydecay702,0,0,0,$arraydecay703,0,0,0], ["i8*",0,0,0,"i8*",0,0,0,"i8*",0,0,0], ALLOC_STACK)); //@line 1421 "translate.cpp"
            HEAP8[(((_word_phonemes+159)&4294967295))]=0; //@line 1423 "translate.cpp"
            __label__ = 192;break $if_then651$$lor_lhs_false649$253;
          }
        } while(0);
        $if_then651$$if_end705$256: do { 
          if (__label__ == 181) {
  
            var $298=$tr_addr; //@line 1374 "translate.cpp"
            var $langopts652=(($298)&4294967295); //@line 1374 "translate.cpp"
            var $param=(($langopts652+24)&4294967295); //@line 1374 "translate.cpp"
            var $arrayidx653=(($param+12)&4294967295); //@line 1374 "translate.cpp"
            var $299=HEAP32[(($arrayidx653)>>2)]; //@line 1374 "translate.cpp"
            var $tobool654=((($299))|0)!=0; //@line 1374 "translate.cpp"
            if ($tobool654) { __label__ = 183;; } else { __label__ = 182;; } //@line 1374 "translate.cpp"
            $if_then658$$lor_lhs_false655$258: do { 
              if (__label__ == 182) {
  
                var $300=$prefix_type; //@line 1374 "translate.cpp"
                var $and656=($300) & 65536; //@line 1374 "translate.cpp"
                var $tobool657=((($and656))|0)!=0; //@line 1374 "translate.cpp"
                if ($tobool657) { __label__ = 183;break $if_then658$$lor_lhs_false655$258; } //@line 1374 "translate.cpp"
  
                var $arraydecay684=(($unpron_phonemes)&4294967295); //@line 1406 "translate.cpp"
                var $arraydecay685=(($prefix_phonemes)&4294967295); //@line 1406 "translate.cpp"
                var $arraydecay686=(($phonemes)&4294967295); //@line 1406 "translate.cpp"
                var $call687=_snprintf(((_word_phonemes)&4294967295), 160, ((__str6549)&4294967295), allocate([$arraydecay684,0,0,0,$arraydecay685,0,0,0,$arraydecay686,0,0,0], ["i8*",0,0,0,"i8*",0,0,0,"i8*",0,0,0], ALLOC_STACK)); //@line 1406 "translate.cpp"
                HEAP8[(((_word_phonemes+159)&4294967295))]=0; //@line 1408 "translate.cpp"
                var $312=$tr_addr; //@line 1409 "translate.cpp"
                var $arraydecay688=(($dictionary_flags)&4294967295); //@line 1409 "translate.cpp"
                __Z13SetWordStressP10TranslatorPcPjii($312, ((_word_phonemes)&4294967295), $arraydecay688, -1, 0); //@line 1409 "translate.cpp"
                __label__ = 192;break $if_then651$$if_end705$256;
              }
            } while(0);
  
            var $301=$tr_addr; //@line 1378 "translate.cpp"
            var $arraydecay660=(($phonemes)&4294967295); //@line 1378 "translate.cpp"
            var $arraydecay661=(($dictionary_flags)&4294967295); //@line 1378 "translate.cpp"
            __Z13SetWordStressP10TranslatorPcPjii($301, $arraydecay660, $arraydecay661, 3, 0); //@line 1378 "translate.cpp"
            $ix=0; //@line 1381 "translate.cpp"
            var $arraydecay662=(($prefix_phonemes)&4294967295); //@line 1382 "translate.cpp"
            $p659=$arraydecay662; //@line 1382 "translate.cpp"
            var $302=$p659; //@line 1382 "translate.cpp"
            var $303=HEAP8[($302)]; //@line 1382 "translate.cpp"
            var $conv66424=(tempInt=(($303)),(tempInt>=128?tempInt-256:tempInt)); //@line 1382 "translate.cpp"
            var $cmp66525=((($conv66424))|0)!=0; //@line 1382 "translate.cpp"
            if ($cmp66525) { __label__ = 184;; } else { __label__ = 189;; } //@line 1382 "translate.cpp"
            $for_body666$$for_end677$262: do { 
              if (__label__ == 184) {
                while(1) { 
  
                  var $304=$p659; //@line 1384 "translate.cpp"
                  var $305=HEAP8[($304)]; //@line 1384 "translate.cpp"
                  var $conv667=(tempInt=(($305)),(tempInt>=128?tempInt-256:tempInt)); //@line 1384 "translate.cpp"
                  var $cmp668=((($conv667))|0)==6; //@line 1384 "translate.cpp"
                  if ($cmp668) { __label__ = 185;; } else { __label__ = 188;; } //@line 1384 "translate.cpp"
                  if (__label__ == 185) {
  
                    var $306=$ix; //@line 1386 "translate.cpp"
                    var $cmp670=((($306))|0)==0; //@line 1386 "translate.cpp"
                    if ($cmp670) { __label__ = 186;; } else { __label__ = 187;; } //@line 1386 "translate.cpp"
                    if (__label__ == 186) {
  
                      $ix=1; //@line 1387 "translate.cpp"
                      ; //@line 1387 "translate.cpp"
                    }
                    else if (__label__ == 187) {
  
                      var $307=$p659; //@line 1389 "translate.cpp"
                      HEAP8[($307)]=5; //@line 1389 "translate.cpp"
                      ;
                    }
                  }
  
                  var $308=$p659; //@line 1382 "translate.cpp"
                  var $incdec_ptr676=(($308+1)&4294967295); //@line 1382 "translate.cpp"
                  $p659=$incdec_ptr676; //@line 1382 "translate.cpp"
                  var $309=$p659; //@line 1382 "translate.cpp"
                  var $310=HEAP8[($309)]; //@line 1382 "translate.cpp"
                  var $conv664=(tempInt=(($310)),(tempInt>=128?tempInt-256:tempInt)); //@line 1382 "translate.cpp"
                  var $cmp665=((($conv664))|0)!=0; //@line 1382 "translate.cpp"
                  if ($cmp665) { __label__ = 184;continue ; } else { __label__ = 189;break $for_body666$$for_end677$262; } //@line 1382 "translate.cpp"
                }
              }
            } while(0);
  
            var $arraydecay678=(($unpron_phonemes)&4294967295); //@line 1395 "translate.cpp"
            var $arraydecay679=(($prefix_phonemes)&4294967295); //@line 1395 "translate.cpp"
            var $arraydecay680=(($phonemes)&4294967295); //@line 1395 "translate.cpp"
            var $call681=_snprintf(((_word_phonemes)&4294967295), 160, ((__str6549)&4294967295), allocate([$arraydecay678,0,0,0,$arraydecay679,0,0,0,$arraydecay680,0,0,0], ["i8*",0,0,0,"i8*",0,0,0,"i8*",0,0,0], ALLOC_STACK)); //@line 1395 "translate.cpp"
            HEAP8[(((_word_phonemes+159)&4294967295))]=0; //@line 1397 "translate.cpp"
            var $311=$tr_addr; //@line 1398 "translate.cpp"
            var $arraydecay682=(($dictionary_flags)&4294967295); //@line 1398 "translate.cpp"
            __Z13SetWordStressP10TranslatorPcPjii($311, ((_word_phonemes)&4294967295), $arraydecay682, -1, 0); //@line 1398 "translate.cpp"
            ; //@line 1399 "translate.cpp"
          }
        } while(0);
  
        var $arrayidx706=(($end_phonemes)&4294967295); //@line 1426 "translate.cpp"
        var $314=HEAP8[($arrayidx706)]; //@line 1426 "translate.cpp"
        var $conv707=(tempInt=(($314)),(tempInt>=128?tempInt-256:tempInt)); //@line 1426 "translate.cpp"
        var $cmp708=((($conv707))|0)!=0; //@line 1426 "translate.cpp"
        if ($cmp708) { __label__ = 193;; } else { __label__ = 194;; } //@line 1426 "translate.cpp"
        if (__label__ == 193) {
  
          var $call710=_strlen(((_word_phonemes)&4294967295)); //@line 1429 "translate.cpp"
          $ix=$call710; //@line 1429 "translate.cpp"
          var $315=$ix; //@line 1430 "translate.cpp"
          var $sub711=((159-($315))&4294967295); //@line 1430 "translate.cpp"
          var $arrayidx712=(($end_phonemes+$sub711)&4294967295); //@line 1430 "translate.cpp"
          HEAP8[($arrayidx712)]=0; //@line 1430 "translate.cpp"
          var $316=$ix; //@line 1431 "translate.cpp"
          var $arrayidx713=((_word_phonemes+$316)&4294967295); //@line 1431 "translate.cpp"
          var $arraydecay714=(($end_phonemes)&4294967295); //@line 1431 "translate.cpp"
          var $call715=_strcpy($arrayidx713, $arraydecay714); //@line 1431 "translate.cpp"
          ; //@line 1432 "translate.cpp"
        }
  
        var $317=$wflags; //@line 1434 "translate.cpp"
        var $and717=($317) & 16; //@line 1434 "translate.cpp"
        var $tobool718=((($and717))|0)!=0; //@line 1434 "translate.cpp"
        if ($tobool718) { __label__ = 195;; } else { __label__ = 196;; } //@line 1434 "translate.cpp"
        if (__label__ == 195) {
  
          var $arrayidx720=(($dictionary_flags)&4294967295); //@line 1438 "translate.cpp"
          var $318=HEAP32[(($arrayidx720)>>2)]; //@line 1438 "translate.cpp"
          var $and721=($318) & -268435457; //@line 1438 "translate.cpp"
          HEAP32[(($arrayidx720)>>2)]=$and721; //@line 1438 "translate.cpp"
          ; //@line 1439 "translate.cpp"
        }
  
        var $319=$wflags; //@line 1441 "translate.cpp"
        var $and723=($319) & 128; //@line 1441 "translate.cpp"
        var $tobool724=((($and723))|0)!=0; //@line 1441 "translate.cpp"
        if ($tobool724) { __label__ = 197;; } else { __label__ = 199;; } //@line 1441 "translate.cpp"
        $land_lhs_true725$$if_else730$279: do { 
          if (__label__ == 197) {
  
            var $320=$tr_addr; //@line 1441 "translate.cpp"
            var $langopts726=(($320)&4294967295); //@line 1441 "translate.cpp"
            var $stress_flags=(($langopts726+12)&4294967295); //@line 1441 "translate.cpp"
            var $321=HEAP32[(($stress_flags)>>2)]; //@line 1441 "translate.cpp"
            var $and727=($321) & 1048576; //@line 1441 "translate.cpp"
            var $tobool728=((($and727))|0)!=0; //@line 1441 "translate.cpp"
            if (!($tobool728)) { __label__ = 199;break $land_lhs_true725$$if_else730$279; } //@line 1441 "translate.cpp"
  
            var $322=$tr_addr; //@line 1443 "translate.cpp"
            __Z16ChangeWordStressP10TranslatorPci($322, ((_word_phonemes)&4294967295), 3); //@line 1443 "translate.cpp"
            __label__ = 208;break $land_lhs_true725$$if_else730$279; //@line 1444 "translate.cpp"
          }
        } while(0);
        $if_end761$$if_else730$282: do { 
          if (__label__ == 199) {
  
            var $323=$wflags; //@line 1446 "translate.cpp"
            var $and731=($323) & 3072; //@line 1446 "translate.cpp"
            var $tobool732=((($and731))|0)!=0; //@line 1446 "translate.cpp"
            if ($tobool732) { __label__ = 200;; } else { __label__ = 202;; } //@line 1446 "translate.cpp"
            if (__label__ == 200) {
  
              var $324=$tr_addr; //@line 1450 "translate.cpp"
              __Z16ChangeWordStressP10TranslatorPci($324, ((_word_phonemes)&4294967295), 6); //@line 1450 "translate.cpp"
              var $325=$wflags; //@line 1452 "translate.cpp"
              var $and734=($325) & 2048; //@line 1452 "translate.cpp"
              var $tobool735=((($and734))|0)!=0; //@line 1452 "translate.cpp"
              if (!($tobool735)) { __label__ = 208;break $if_end761$$if_else730$282; } //@line 1452 "translate.cpp"
  
              var $arrayidx737=(($dictionary_flags)&4294967295); //@line 1453 "translate.cpp"
              var $326=HEAP32[(($arrayidx737)>>2)]; //@line 1453 "translate.cpp"
              var $or738=($326) | 268435456; //@line 1453 "translate.cpp"
              HEAP32[(($arrayidx737)>>2)]=$or738; //@line 1453 "translate.cpp"
              ; //@line 1453 "translate.cpp"
            }
            else if (__label__ == 202) {
  
              var $327=HEAP32[((_dictionary_skipwords)>>2)]; //@line 1456 "translate.cpp"
              var $328=$wtab_addr; //@line 1456 "translate.cpp"
              var $arrayidx741=(($328+12*$327)&4294967295); //@line 1456 "translate.cpp"
              var $flags742=(($arrayidx741)&4294967295); //@line 1456 "translate.cpp"
              var $329=HEAP32[(($flags742)>>2)]; //@line 1456 "translate.cpp"
              var $and743=($329) & 16; //@line 1456 "translate.cpp"
              var $tobool744=((($and743))|0)!=0; //@line 1456 "translate.cpp"
              if (!($tobool744)) { __label__ = 208;break $if_end761$$if_else730$282; } //@line 1456 "translate.cpp"
  
              var $arrayidx746=(($dictionary_flags)&4294967295); //@line 1459 "translate.cpp"
              var $330=HEAP32[(($arrayidx746)>>2)]; //@line 1459 "translate.cpp"
              var $and747=($330) & 6144; //@line 1459 "translate.cpp"
              var $tobool748=((($and747))|0)!=0; //@line 1459 "translate.cpp"
              if ($tobool748) { __label__ = 204;; } else { __label__ = 205;; } //@line 1459 "translate.cpp"
              if (__label__ == 204) {
  
                var $331=$tr_addr; //@line 1460 "translate.cpp"
                __Z16ChangeWordStressP10TranslatorPci($331, ((_word_phonemes)&4294967295), 4); //@line 1460 "translate.cpp"
                ; //@line 1460 "translate.cpp"
              }
              else if (__label__ == 205) {
  
                var $arrayidx751=(($dictionary_flags)&4294967295); //@line 1462 "translate.cpp"
                var $332=HEAP32[(($arrayidx751)>>2)]; //@line 1462 "translate.cpp"
                var $and752=($332) & 8192; //@line 1462 "translate.cpp"
                var $tobool753=((($and752))|0)!=0; //@line 1462 "translate.cpp"
                if (!($tobool753)) { __label__ = 208;break $if_end761$$if_else730$282; } //@line 1462 "translate.cpp"
  
                var $_b=HEAPU8[(__ZL18any_stressed_words_b)];
                var $333=((($_b))&1);
                var $tobool755=((($333))|0)!=0; //@line 1462 "translate.cpp"
                if (!($tobool755)) { __label__ = 208;break $if_end761$$if_else730$282; } //@line 1462 "translate.cpp"
  
                var $334=$tr_addr; //@line 1463 "translate.cpp"
                __Z16ChangeWordStressP10TranslatorPci($334, ((_word_phonemes)&4294967295), 3); //@line 1463 "translate.cpp"
                ; //@line 1463 "translate.cpp"
              }
            }
          }
        } while(0);
  
        var $335=$end_type; //@line 1468 "translate.cpp"
        var $and762=($335) & 8192; //@line 1468 "translate.cpp"
        var $tobool763=((($and762))|0)!=0; //@line 1468 "translate.cpp"
        if ($tobool763) { __label__ = 209;; } else { __label__ = 210;; } //@line 1468 "translate.cpp"
        if (__label__ == 209) {
  
          var $336=$tr_addr; //@line 1471 "translate.cpp"
          var $expect_verb765=(($336+8272)&4294967295); //@line 1471 "translate.cpp"
          HEAP32[(($expect_verb765)>>2)]=2; //@line 1471 "translate.cpp"
          var $337=$tr_addr; //@line 1472 "translate.cpp"
          var $expect_verb_s=(($337+8280)&4294967295); //@line 1472 "translate.cpp"
          HEAP32[(($expect_verb_s)>>2)]=2; //@line 1472 "translate.cpp"
          ; //@line 1473 "translate.cpp"
        }
  
        var $arrayidx767=(($dictionary_flags+4)&4294967295); //@line 1475 "translate.cpp"
        var $338=HEAP32[(($arrayidx767)>>2)]; //@line 1475 "translate.cpp"
        var $and768=($338) & 8; //@line 1475 "translate.cpp"
        var $tobool769=((($and768))|0)!=0; //@line 1475 "translate.cpp"
        if ($tobool769) { __label__ = 211;; } else { __label__ = 212;; } //@line 1475 "translate.cpp"
        $if_then770$$if_else772$298: do { 
          if (__label__ == 211) {
  
            var $339=$tr_addr; //@line 1478 "translate.cpp"
            var $expect_past=(($339+8276)&4294967295); //@line 1478 "translate.cpp"
            HEAP32[(($expect_past)>>2)]=3; //@line 1478 "translate.cpp"
            var $340=$tr_addr; //@line 1479 "translate.cpp"
            var $expect_verb771=(($340+8272)&4294967295); //@line 1479 "translate.cpp"
            HEAP32[(($expect_verb771)>>2)]=0; //@line 1479 "translate.cpp"
            var $341=$tr_addr; //@line 1480 "translate.cpp"
            var $expect_noun=(($341+8284)&4294967295); //@line 1480 "translate.cpp"
            HEAP32[(($expect_noun)>>2)]=0; //@line 1480 "translate.cpp"
            ; //@line 1481 "translate.cpp"
          }
          else if (__label__ == 212) {
  
            var $arrayidx773=(($dictionary_flags+4)&4294967295); //@line 1483 "translate.cpp"
            var $342=HEAP32[(($arrayidx773)>>2)]; //@line 1483 "translate.cpp"
            var $and774=($342) & 1; //@line 1483 "translate.cpp"
            var $tobool775=((($and774))|0)!=0; //@line 1483 "translate.cpp"
            if ($tobool775) { __label__ = 213;; } else { __label__ = 214;; } //@line 1483 "translate.cpp"
            if (__label__ == 213) {
  
              var $343=$tr_addr; //@line 1486 "translate.cpp"
              var $expect_verb777=(($343+8272)&4294967295); //@line 1486 "translate.cpp"
              HEAP32[(($expect_verb777)>>2)]=2; //@line 1486 "translate.cpp"
              var $344=$tr_addr; //@line 1487 "translate.cpp"
              var $expect_verb_s778=(($344+8280)&4294967295); //@line 1487 "translate.cpp"
              HEAP32[(($expect_verb_s778)>>2)]=0; //@line 1487 "translate.cpp"
              var $345=$tr_addr; //@line 1488 "translate.cpp"
              var $expect_noun779=(($345+8284)&4294967295); //@line 1488 "translate.cpp"
              HEAP32[(($expect_noun779)>>2)]=0; //@line 1488 "translate.cpp"
              ; //@line 1489 "translate.cpp"
            }
            else if (__label__ == 214) {
  
              var $arrayidx781=(($dictionary_flags+4)&4294967295); //@line 1491 "translate.cpp"
              var $346=HEAP32[(($arrayidx781)>>2)]; //@line 1491 "translate.cpp"
              var $and782=($346) & 2; //@line 1491 "translate.cpp"
              var $tobool783=((($and782))|0)!=0; //@line 1491 "translate.cpp"
              if ($tobool783) { __label__ = 215;; } else { __label__ = 216;; } //@line 1491 "translate.cpp"
              if (__label__ == 215) {
  
                var $347=$tr_addr; //@line 1494 "translate.cpp"
                var $expect_verb785=(($347+8272)&4294967295); //@line 1494 "translate.cpp"
                HEAP32[(($expect_verb785)>>2)]=0; //@line 1494 "translate.cpp"
                var $348=$tr_addr; //@line 1495 "translate.cpp"
                var $expect_verb_s786=(($348+8280)&4294967295); //@line 1495 "translate.cpp"
                HEAP32[(($expect_verb_s786)>>2)]=2; //@line 1495 "translate.cpp"
                var $349=$tr_addr; //@line 1496 "translate.cpp"
                var $expect_past787=(($349+8276)&4294967295); //@line 1496 "translate.cpp"
                HEAP32[(($expect_past787)>>2)]=0; //@line 1496 "translate.cpp"
                var $350=$tr_addr; //@line 1497 "translate.cpp"
                var $expect_noun788=(($350+8284)&4294967295); //@line 1497 "translate.cpp"
                HEAP32[(($expect_noun788)>>2)]=0; //@line 1497 "translate.cpp"
                ; //@line 1498 "translate.cpp"
              }
              else if (__label__ == 216) {
  
                var $arrayidx790=(($dictionary_flags+4)&4294967295); //@line 1500 "translate.cpp"
                var $351=HEAP32[(($arrayidx790)>>2)]; //@line 1500 "translate.cpp"
                var $and791=($351) & 4; //@line 1500 "translate.cpp"
                var $tobool792=((($and791))|0)!=0; //@line 1500 "translate.cpp"
                if (!($tobool792)) { __label__ = 218;break $if_then770$$if_else772$298; } //@line 1500 "translate.cpp"
  
                var $352=$tr_addr; //@line 1503 "translate.cpp"
                var $expect_noun794=(($352+8284)&4294967295); //@line 1503 "translate.cpp"
                HEAP32[(($expect_noun794)>>2)]=2; //@line 1503 "translate.cpp"
                var $353=$tr_addr; //@line 1504 "translate.cpp"
                var $expect_verb795=(($353+8272)&4294967295); //@line 1504 "translate.cpp"
                HEAP32[(($expect_verb795)>>2)]=0; //@line 1504 "translate.cpp"
                var $354=$tr_addr; //@line 1505 "translate.cpp"
                var $expect_verb_s796=(($354+8280)&4294967295); //@line 1505 "translate.cpp"
                HEAP32[(($expect_verb_s796)>>2)]=0; //@line 1505 "translate.cpp"
                var $355=$tr_addr; //@line 1506 "translate.cpp"
                var $expect_past797=(($355+8276)&4294967295); //@line 1506 "translate.cpp"
                HEAP32[(($expect_past797)>>2)]=0; //@line 1506 "translate.cpp"
                ; //@line 1507 "translate.cpp"
              }
            }
          }
        } while(0);
  
        var $356=HEAP32[(($wordx)>>2)]; //@line 1509 "translate.cpp"
        var $arrayidx802=(($356)&4294967295); //@line 1509 "translate.cpp"
        var $357=HEAP8[($arrayidx802)]; //@line 1509 "translate.cpp"
        var $conv803=(tempInt=(($357)),(tempInt>=128?tempInt-256:tempInt)); //@line 1509 "translate.cpp"
        var $cmp804=((($conv803))|0)!=0; //@line 1509 "translate.cpp"
        if ($cmp804) { __label__ = 219;; } else { __label__ = 228;; } //@line 1509 "translate.cpp"
        $land_lhs_true805$$if_end834$309: do { 
          if (__label__ == 219) {
  
            var $arrayidx806=(($dictionary_flags+4)&4294967295); //@line 1509 "translate.cpp"
            var $358=HEAP32[(($arrayidx806)>>2)]; //@line 1509 "translate.cpp"
            var $and807=($358) & 256; //@line 1509 "translate.cpp"
            var $tobool808=((($and807))|0)!=0; //@line 1509 "translate.cpp"
            if ($tobool808) { __label__ = 228;break $land_lhs_true805$$if_end834$309; } //@line 1509 "translate.cpp"
  
            var $359=$tr_addr; //@line 1511 "translate.cpp"
            var $expect_verb810=(($359+8272)&4294967295); //@line 1511 "translate.cpp"
            var $360=HEAP32[(($expect_verb810)>>2)]; //@line 1511 "translate.cpp"
            var $cmp811=((($360))|0) > 0; //@line 1511 "translate.cpp"
            if ($cmp811) { __label__ = 221;; } else { __label__ = 222;; } //@line 1511 "translate.cpp"
            if (__label__ == 221) {
  
              var $361=$tr_addr; //@line 1512 "translate.cpp"
              var $expect_verb813=(($361+8272)&4294967295); //@line 1512 "translate.cpp"
              var $362=HEAP32[(($expect_verb813)>>2)]; //@line 1512 "translate.cpp"
              var $dec814=((($362)-1)&4294967295); //@line 1512 "translate.cpp"
              HEAP32[(($expect_verb813)>>2)]=$dec814; //@line 1512 "translate.cpp"
              ; //@line 1512 "translate.cpp"
            }
  
            var $363=$tr_addr; //@line 1514 "translate.cpp"
            var $expect_verb_s816=(($363+8280)&4294967295); //@line 1514 "translate.cpp"
            var $364=HEAP32[(($expect_verb_s816)>>2)]; //@line 1514 "translate.cpp"
            var $cmp817=((($364))|0) > 0; //@line 1514 "translate.cpp"
            if ($cmp817) { __label__ = 223;; } else { __label__ = 224;; } //@line 1514 "translate.cpp"
            if (__label__ == 223) {
  
              var $365=$tr_addr; //@line 1515 "translate.cpp"
              var $expect_verb_s819=(($365+8280)&4294967295); //@line 1515 "translate.cpp"
              var $366=HEAP32[(($expect_verb_s819)>>2)]; //@line 1515 "translate.cpp"
              var $dec820=((($366)-1)&4294967295); //@line 1515 "translate.cpp"
              HEAP32[(($expect_verb_s819)>>2)]=$dec820; //@line 1515 "translate.cpp"
              ; //@line 1515 "translate.cpp"
            }
  
            var $367=$tr_addr; //@line 1517 "translate.cpp"
            var $expect_noun822=(($367+8284)&4294967295); //@line 1517 "translate.cpp"
            var $368=HEAP32[(($expect_noun822)>>2)]; //@line 1517 "translate.cpp"
            var $cmp823=((($368))|0) > 0; //@line 1517 "translate.cpp"
            if ($cmp823) { __label__ = 225;; } else { __label__ = 226;; } //@line 1517 "translate.cpp"
            if (__label__ == 225) {
  
              var $369=$tr_addr; //@line 1518 "translate.cpp"
              var $expect_noun825=(($369+8284)&4294967295); //@line 1518 "translate.cpp"
              var $370=HEAP32[(($expect_noun825)>>2)]; //@line 1518 "translate.cpp"
              var $dec826=((($370)-1)&4294967295); //@line 1518 "translate.cpp"
              HEAP32[(($expect_noun825)>>2)]=$dec826; //@line 1518 "translate.cpp"
              ; //@line 1518 "translate.cpp"
            }
  
            var $371=$tr_addr; //@line 1520 "translate.cpp"
            var $expect_past828=(($371+8276)&4294967295); //@line 1520 "translate.cpp"
            var $372=HEAP32[(($expect_past828)>>2)]; //@line 1520 "translate.cpp"
            var $cmp829=((($372))|0) > 0; //@line 1520 "translate.cpp"
            if (!($cmp829)) { __label__ = 228;break $land_lhs_true805$$if_end834$309; } //@line 1520 "translate.cpp"
  
            var $373=$tr_addr; //@line 1521 "translate.cpp"
            var $expect_past831=(($373+8276)&4294967295); //@line 1521 "translate.cpp"
            var $374=HEAP32[(($expect_past831)>>2)]; //@line 1521 "translate.cpp"
            var $dec832=((($374)-1)&4294967295); //@line 1521 "translate.cpp"
            HEAP32[(($expect_past831)>>2)]=$dec832; //@line 1521 "translate.cpp"
            ; //@line 1521 "translate.cpp"
          }
        } while(0);
  
        var $375=$word_length; //@line 1524 "translate.cpp"
        var $cmp835=((($375))|0)==1; //@line 1524 "translate.cpp"
        if ($cmp835) { __label__ = 229;; } else { __label__ = 232;; } //@line 1524 "translate.cpp"
        $land_lhs_true836$$if_end846$323: do { 
          if (__label__ == 229) {
  
            var $376=$tr_addr; //@line 1524 "translate.cpp"
            var $translator_name=(($376+280)&4294967295); //@line 1524 "translate.cpp"
            var $377=HEAP32[(($translator_name)>>2)]; //@line 1524 "translate.cpp"
            var $cmp837=((($377))|0)==25966; //@line 1524 "translate.cpp"
            if (!($cmp837)) { __label__ = 232;break $land_lhs_true836$$if_end846$323; } //@line 1524 "translate.cpp"
  
            var $378=HEAP32[(($first_char)>>2)]; //@line 1524 "translate.cpp"
            var $call839=_iswalpha($378); //@line 1524 "translate.cpp"
            var $tobool840=((($call839))|0)!=0; //@line 1524 "translate.cpp"
            var $379=HEAP32[(($first_char)>>2)]; //@line 1524 "translate.cpp"
            var $cmp842=((($379))|0)!=105; //@line 1524 "translate.cpp"
            var $or_cond3=($tobool840) & ($cmp842); //@line 1524 "translate.cpp"
            if (!($or_cond3)) { __label__ = 232;break $land_lhs_true836$$if_end846$323; } //@line 1524 "translate.cpp"
  
            var $arrayidx844=(($dictionary_flags)&4294967295); //@line 1528 "translate.cpp"
            var $380=HEAP32[(($arrayidx844)>>2)]; //@line 1528 "translate.cpp"
            var $or845=($380) | 16777216; //@line 1528 "translate.cpp"
            HEAP32[(($arrayidx844)>>2)]=$or845; //@line 1528 "translate.cpp"
            ; //@line 1529 "translate.cpp"
          }
        } while(0);
  
        var $381=$tr_addr; //@line 1531 "translate.cpp"
        var $langopts847=(($381)&4294967295); //@line 1531 "translate.cpp"
        var $param848=(($langopts847+24)&4294967295); //@line 1531 "translate.cpp"
        var $arrayidx849=(($param848+60)&4294967295); //@line 1531 "translate.cpp"
        var $382=HEAP32[(($arrayidx849)>>2)]; //@line 1531 "translate.cpp"
        var $and850=($382) & 2; //@line 1531 "translate.cpp"
        var $tobool851=((($and850))|0)!=0; //@line 1531 "translate.cpp"
        if ($tobool851) { __label__ = 233;; } else { __label__ = 235;; } //@line 1531 "translate.cpp"
        $land_lhs_true852$$if_end858$328: do { 
          if (__label__ == 233) {
  
            var $arrayidx853=(($dictionary_flags)&4294967295); //@line 1531 "translate.cpp"
            var $383=HEAP32[(($arrayidx853)>>2)]; //@line 1531 "translate.cpp"
            var $and854=($383) & 3145728; //@line 1531 "translate.cpp"
            var $cmp855=((($and854))|0)!=0; //@line 1531 "translate.cpp"
            if (!($cmp855)) { __label__ = 235;break $land_lhs_true852$$if_end858$328; } //@line 1531 "translate.cpp"
  
            var $384=$tr_addr; //@line 1533 "translate.cpp"
            var $arrayidx857=(($dictionary_flags)&4294967295); //@line 1533 "translate.cpp"
            var $385=HEAP32[(($arrayidx857)>>2)]; //@line 1533 "translate.cpp"
            __Z22ApplySpecialAttribute2P10TranslatorPci($384, ((_word_phonemes)&4294967295), $385); //@line 1533 "translate.cpp"
            ; //@line 1534 "translate.cpp"
          }
        } while(0);
  
        var $386=$was_unpronouncable; //@line 1536 "translate.cpp"
        var $arrayidx859=(($dictionary_flags)&4294967295); //@line 1536 "translate.cpp"
        var $387=HEAP32[(($arrayidx859)>>2)]; //@line 1536 "translate.cpp"
        var $or860=($387) | ($386); //@line 1536 "translate.cpp"
        HEAP32[(($arrayidx859)>>2)]=$or860; //@line 1536 "translate.cpp"
        var $388=$word_start_addr; //@line 1537 "translate.cpp"
        var $arraydecay861=(($word_copy2)&4294967295); //@line 1537 "translate.cpp"
        var $389=$word_copy_length; //@line 1537 "translate.cpp"
        var $src$, $dest$, $stop$, $stop4$, $fast$;
        $src$ = $arraydecay861;
        $dest$ = $388;
        $stop$ = $src$ + $389;
        if (($dest$%4) == ($src$%4) && $389 > 8) {
          while ($src$%4 !== 0 && $src$ < $stop$) {
            ; HEAP8[$dest$++] = HEAP8[$src$++];
          }
          $src$ >>= 2;
          $dest$ >>= 2;
          $stop4$ = $stop$ >> 2;
          while ($src$ < $stop4$) {
        ;;;    HEAP32[$dest$++] = HEAP32[$src$++];
          }
          $src$ <<= 2;
          $dest$ <<= 2;
        }while ($src$ < $stop$) {
          ; HEAP8[$dest$++] = HEAP8[$src$++];
        }; //@line 1537 "translate.cpp"
        var $arrayidx862=(($dictionary_flags)&4294967295); //@line 1538 "translate.cpp"
        var $390=HEAP32[(($arrayidx862)>>2)]; //@line 1538 "translate.cpp"
        $retval=$390; //@line 1538 "translate.cpp"
        ; //@line 1538 "translate.cpp"
      }
    } while(0);
  
    var $391=$retval; //@line 1539 "translate.cpp"
    STACKTOP = __stackBase__;
    return $391; //@line 1539 "translate.cpp"
    return null;
  }
  

  function __ZL17CheckDottedAbbrevPcP8WORD_TAB($word1, $wtab) {
    var __stackBase__  = STACKTOP; STACKTOP += 84; _memset(__stackBase__, 0, 84);
    var __label__;
  
    var $word1_addr;
    var $wtab_addr;
    var $wc=__stackBase__;
    var $count;
    var $nbytes;
    var $ok;
    var $ix;
    var $word;
    var $wbuf;
    var $word_buf=__stackBase__+4;
    $word1_addr=$word1;
    $wtab_addr=$wtab;
    $count=0; //@line 628 "translate.cpp"
    var $0=$word1_addr; //@line 636 "translate.cpp"
    $word=$0; //@line 636 "translate.cpp"
    var $arraydecay=(($word_buf)&4294967295); //@line 637 "translate.cpp"
    $wbuf=$arraydecay; //@line 637 "translate.cpp"
    $ix=0; //@line 638 "translate.cpp"
    ; //@line 640 "translate.cpp"
    $for_cond$2: while(1) { 
  
      $ok=0; //@line 642 "translate.cpp"
      var $1=$word; //@line 643 "translate.cpp"
      var $call=__Z7utf8_inPiPKc($wc, $1); //@line 643 "translate.cpp"
      $nbytes=$call; //@line 643 "translate.cpp"
      var $2=$nbytes; //@line 644 "translate.cpp"
      var $3=$word; //@line 644 "translate.cpp"
      var $arrayidx=(($3+$2)&4294967295); //@line 644 "translate.cpp"
      var $4=HEAP8[($arrayidx)]; //@line 644 "translate.cpp"
      var $conv=(tempInt=(($4)),(tempInt>=128?tempInt-256:tempInt)); //@line 644 "translate.cpp"
      var $cmp=((($conv))|0)==32; //@line 644 "translate.cpp"
      if ($cmp) { __label__ = 2;; } else { __label__ = 11;; } //@line 644 "translate.cpp"
      $land_lhs_true$$if_end27$4: do { 
        if (__label__ == 2) {
  
          var $5=HEAP32[(($wc)>>2)]; //@line 644 "translate.cpp"
          var $call1=__Z7IsAlphaj($5); //@line 644 "translate.cpp"
          var $tobool=((($call1))|0)!=0; //@line 644 "translate.cpp"
          if (!($tobool)) { __label__ = 11;break $land_lhs_true$$if_end27$4; } //@line 644 "translate.cpp"
  
          var $6=$nbytes; //@line 646 "translate.cpp"
          var $add=((($6)+1)&4294967295); //@line 646 "translate.cpp"
          var $7=$word; //@line 646 "translate.cpp"
          var $arrayidx2=(($7+$add)&4294967295); //@line 646 "translate.cpp"
          var $8=HEAP8[($arrayidx2)]; //@line 646 "translate.cpp"
          var $conv3=(tempInt=(($8)),(tempInt>=128?tempInt-256:tempInt)); //@line 646 "translate.cpp"
          var $cmp4=((($conv3))|0)==46; //@line 646 "translate.cpp"
          if ($cmp4) { __label__ = 4;; } else { __label__ = 8;; } //@line 646 "translate.cpp"
          if (__label__ == 4) {
  
            var $9=$nbytes; //@line 648 "translate.cpp"
            var $add6=((($9)+2)&4294967295); //@line 648 "translate.cpp"
            var $10=$word; //@line 648 "translate.cpp"
            var $arrayidx7=(($10+$add6)&4294967295); //@line 648 "translate.cpp"
            var $11=HEAP8[($arrayidx7)]; //@line 648 "translate.cpp"
            var $conv8=(tempInt=(($11)),(tempInt>=128?tempInt-256:tempInt)); //@line 648 "translate.cpp"
            var $cmp9=((($conv8))|0)==32; //@line 648 "translate.cpp"
            if ($cmp9) { __label__ = 5;; } else { __label__ = 6;; } //@line 648 "translate.cpp"
            if (__label__ == 5) {
  
              $ok=1; //@line 649 "translate.cpp"
              __label__ = 12;break $land_lhs_true$$if_end27$4; //@line 649 "translate.cpp"
            }
            else if (__label__ == 6) {
  
              var $12=$nbytes; //@line 651 "translate.cpp"
              var $add11=((($12)+2)&4294967295); //@line 651 "translate.cpp"
              var $13=$word; //@line 651 "translate.cpp"
              var $arrayidx12=(($13+$add11)&4294967295); //@line 651 "translate.cpp"
              var $14=HEAP8[($arrayidx12)]; //@line 651 "translate.cpp"
              var $conv13=(tempInt=(($14)),(tempInt>=128?tempInt-256:tempInt)); //@line 651 "translate.cpp"
              var $cmp14=((($conv13))|0)==39; //@line 651 "translate.cpp"
              if (!($cmp14)) { __label__ = 11;break $land_lhs_true$$if_end27$4; } //@line 651 "translate.cpp"
  
              var $15=$nbytes; //@line 653 "translate.cpp"
              var $add16=((($15)+2)&4294967295); //@line 653 "translate.cpp"
              $nbytes=$add16; //@line 653 "translate.cpp"
              $ok=2; //@line 654 "translate.cpp"
              __label__ = 12;break $land_lhs_true$$if_end27$4; //@line 655 "translate.cpp"
            }
          }
          else if (__label__ == 8) {
  
            var $16=$count; //@line 658 "translate.cpp"
            var $cmp19=((($16))|0) > 0; //@line 658 "translate.cpp"
            if (!($cmp19)) { __label__ = 11;break $land_lhs_true$$if_end27$4; } //@line 658 "translate.cpp"
  
            var $17=$nbytes; //@line 658 "translate.cpp"
            var $18=$word; //@line 658 "translate.cpp"
            var $arrayidx21=(($18+$17)&4294967295); //@line 658 "translate.cpp"
            var $19=HEAP8[($arrayidx21)]; //@line 658 "translate.cpp"
            var $conv22=(tempInt=(($19)),(tempInt>=128?tempInt-256:tempInt)); //@line 658 "translate.cpp"
            var $cmp23=((($conv22))|0)==32; //@line 658 "translate.cpp"
            if (!($cmp23)) { __label__ = 11;break $land_lhs_true$$if_end27$4; } //@line 658 "translate.cpp"
  
            $ok=2; //@line 659 "translate.cpp"
            __label__ = 12;break $land_lhs_true$$if_end27$4; //@line 659 "translate.cpp"
          }
        }
      } while(0);
      if (__label__ == 11) {
  
        var $_pr=$ok; //@line 662 "translate.cpp"
        var $cmp28=((($_pr))|0)==0; //@line 662 "translate.cpp"
        if ($cmp28) { __label__ = 17;break $for_cond$2; } //@line 662 "translate.cpp"
      }
  
      $ix=0; //@line 665 "translate.cpp"
      var $20=$ix; //@line 665 "translate.cpp"
      var $21=$nbytes; //@line 665 "translate.cpp"
      var $cmp324=((($20))|0) < ((($21))|0); //@line 665 "translate.cpp"
      if ($cmp324) { __label__ = 13;; } else { __label__ = 14;; } //@line 665 "translate.cpp"
      $for_body$$for_end$19: do { 
        if (__label__ == 13) {
          while(1) { 
  
            var $22=$ix; //@line 666 "translate.cpp"
            var $23=$word; //@line 666 "translate.cpp"
            var $arrayidx33=(($23+$22)&4294967295); //@line 666 "translate.cpp"
            var $24=HEAP8[($arrayidx33)]; //@line 666 "translate.cpp"
            var $25=$wbuf; //@line 666 "translate.cpp"
            var $incdec_ptr=(($25+1)&4294967295); //@line 666 "translate.cpp"
            $wbuf=$incdec_ptr; //@line 666 "translate.cpp"
            HEAP8[($25)]=$24; //@line 666 "translate.cpp"
            var $26=$ix; //@line 665 "translate.cpp"
            var $inc=((($26)+1)&4294967295); //@line 665 "translate.cpp"
            $ix=$inc; //@line 665 "translate.cpp"
            var $27=$ix; //@line 665 "translate.cpp"
            var $28=$nbytes; //@line 665 "translate.cpp"
            var $cmp32=((($27))|0) < ((($28))|0); //@line 665 "translate.cpp"
            if ($cmp32) { __label__ = 13;continue ; } else { __label__ = 14;break $for_body$$for_end$19; } //@line 665 "translate.cpp"
          }
        }
      } while(0);
  
      var $29=$count; //@line 668 "translate.cpp"
      var $inc34=((($29)+1)&4294967295); //@line 668 "translate.cpp"
      $count=$inc34; //@line 668 "translate.cpp"
      var $30=$ok; //@line 670 "translate.cpp"
      var $cmp35=((($30))|0)==2; //@line 670 "translate.cpp"
      var $31=$nbytes; //@line 672 "translate.cpp"
      if ($cmp35) { __label__ = 15;break $for_cond$2; } //@line 670 "translate.cpp"
  
      var $add38=((($31)+3)&4294967295); //@line 676 "translate.cpp"
      var $33=$word; //@line 676 "translate.cpp"
      var $add_ptr39=(($33+$add38)&4294967295); //@line 676 "translate.cpp"
      $word=$add_ptr39; //@line 676 "translate.cpp"
      __label__ = 1;continue $for_cond$2; //@line 677 "translate.cpp"
    }
    if (__label__ == 15) {
  
      var $32=$word; //@line 672 "translate.cpp"
      var $add_ptr=(($32+$31)&4294967295); //@line 672 "translate.cpp"
      $word=$add_ptr; //@line 672 "translate.cpp"
      ; //@line 673 "translate.cpp"
    }
  
    var $34=$count; //@line 679 "translate.cpp"
    var $cmp41=((($34))|0) > 1; //@line 679 "translate.cpp"
    if ($cmp41) { __label__ = 18;; } else { __label__ = 21;; } //@line 679 "translate.cpp"
    if (__label__ == 18) {
  
      var $35=$wbuf; //@line 681 "translate.cpp"
      var $arraydecay43=(($word_buf)&4294967295); //@line 681 "translate.cpp"
      var $sub_ptr_lhs_cast=($35); //@line 681 "translate.cpp"
      var $sub_ptr_rhs_cast=($arraydecay43); //@line 681 "translate.cpp"
      var $sub_ptr_sub=((($sub_ptr_lhs_cast)-($sub_ptr_rhs_cast))&4294967295); //@line 681 "translate.cpp"
      $ix=$sub_ptr_sub; //@line 681 "translate.cpp"
      var $36=$word1_addr; //@line 682 "translate.cpp"
      var $arraydecay44=(($word_buf)&4294967295); //@line 682 "translate.cpp"
      var $37=$ix; //@line 682 "translate.cpp"
      var $src$, $dest$, $stop$, $stop4$, $fast$;
      $src$ = $arraydecay44;
      $dest$ = $36;
      $stop$ = $src$ + $37;
      if (($dest$%4) == ($src$%4) && $37 > 8) {
        while ($src$%4 !== 0 && $src$ < $stop$) {
          ; HEAP8[$dest$++] = HEAP8[$src$++];
        }
        $src$ >>= 2;
        $dest$ >>= 2;
        $stop4$ = $stop$ >> 2;
        while ($src$ < $stop4$) {
      ;;;    HEAP32[$dest$++] = HEAP32[$src$++];
        }
        $src$ <<= 2;
        $dest$ <<= 2;
      }while ($src$ < $stop$) {
        ; HEAP8[$dest$++] = HEAP8[$src$++];
      }; //@line 682 "translate.cpp"
      var $38=$ix; //@line 683 "translate.cpp"
      var $39=$word1_addr; //@line 683 "translate.cpp"
      var $arrayidx452=(($39+$38)&4294967295); //@line 683 "translate.cpp"
      var $40=$word; //@line 683 "translate.cpp"
      var $cmp463=((($arrayidx452))>>>0) < ((($40))>>>0); //@line 683 "translate.cpp"
      if ($cmp463) { __label__ = 19;; } else { __label__ = 20;; } //@line 683 "translate.cpp"
      $while_body$$while_end$29: do { 
        if (__label__ == 19) {
          while(1) { 
  
            var $41=$ix; //@line 684 "translate.cpp"
            var $inc47=((($41)+1)&4294967295); //@line 684 "translate.cpp"
            $ix=$inc47; //@line 684 "translate.cpp"
            var $42=$word1_addr; //@line 684 "translate.cpp"
            var $arrayidx48=(($42+$41)&4294967295); //@line 684 "translate.cpp"
            HEAP8[($arrayidx48)]=32; //@line 684 "translate.cpp"
            var $43=$ix; //@line 683 "translate.cpp"
            var $44=$word1_addr; //@line 683 "translate.cpp"
            var $arrayidx45=(($44+$43)&4294967295); //@line 683 "translate.cpp"
            var $45=$word; //@line 683 "translate.cpp"
            var $cmp46=((($arrayidx45))>>>0) < ((($45))>>>0); //@line 683 "translate.cpp"
            if ($cmp46) { __label__ = 19;continue ; } else { __label__ = 20;break $while_body$$while_end$29; } //@line 683 "translate.cpp"
          }
        }
      } while(0);
  
      var $46=$count; //@line 685 "translate.cpp"
      var $sub=((($46)-1)&4294967295); //@line 685 "translate.cpp"
      var $mul=((($sub)*2)&4294967295); //@line 685 "translate.cpp"
      HEAP32[((_dictionary_skipwords)>>2)]=$mul; //@line 685 "translate.cpp"
      ; //@line 686 "translate.cpp"
    }
  
    var $47=$count; //@line 687 "translate.cpp"
    STACKTOP = __stackBase__;
    return $47; //@line 687 "translate.cpp"
    return null;
  }
  

  function __ZL22SpeakIndividualLettersP10TranslatorPcS1_i($tr, $word, $phonemes, $spell_word) {
    ;
    var __label__;
    var __lastLabel__ = null;
  
    var $retval;
    var $tr_addr;
    var $word_addr;
    var $phonemes_addr;
    var $spell_word_addr;
    var $posn;
    var $capitals;
    var $non_initial;
    $tr_addr=$tr;
    $word_addr=$word;
    $phonemes_addr=$phonemes;
    $spell_word_addr=$spell_word;
    $posn=0; //@line 600 "translate.cpp"
    $capitals=0; //@line 601 "translate.cpp"
    $non_initial=0; //@line 602 "translate.cpp"
    var $0=$spell_word_addr; //@line 604 "translate.cpp"
    var $cmp=((($0))|0) > 2; //@line 604 "translate.cpp"
    if ($cmp) { __label__ = 1;; } else { __label__ = 2;; } //@line 604 "translate.cpp"
    if (__label__ == 1) {
  
      $capitals=2; //@line 605 "translate.cpp"
      ; //@line 605 "translate.cpp"
    }
    $while_cond$4: while(1) { 
  
      var $1=$word_addr; //@line 607 "translate.cpp"
      var $2=HEAP8[($1)]; //@line 607 "translate.cpp"
      var $conv=(tempInt=(($2)),(tempInt>=128?tempInt-256:tempInt)); //@line 607 "translate.cpp"
      var $cmp1=((($conv))|0)!=32; //@line 607 "translate.cpp"
      if (!($cmp1)) { __label__ = 3;break $while_cond$4; } //@line 607 "translate.cpp"
  
      var $4=$word_addr; //@line 607 "translate.cpp"
      var $5=HEAP8[($4)]; //@line 607 "translate.cpp"
      var $conv2=(tempInt=(($5)),(tempInt>=128?tempInt-256:tempInt)); //@line 607 "translate.cpp"
      var $cmp3=((($conv2))|0)!=0; //@line 607 "translate.cpp"
      var $6=$tr_addr; //@line 609 "translate.cpp"
      if ($cmp3) { __lastLabel__ = 4; ; } else { __lastLabel__ = 4; __label__ = 7;break $while_cond$4; }
  
      var $7=$word_addr; //@line 609 "translate.cpp"
      var $8=$phonemes_addr; //@line 609 "translate.cpp"
      var $9=$capitals; //@line 609 "translate.cpp"
      var $10=$non_initial; //@line 609 "translate.cpp"
      var $or=($10) | ($9); //@line 609 "translate.cpp"
      var $call=__Z15TranslateLetterP10TranslatorPcS1_i($6, $7, $8, $or); //@line 609 "translate.cpp"
      var $11=$word_addr; //@line 609 "translate.cpp"
      var $add_ptr=(($11+$call)&4294967295); //@line 609 "translate.cpp"
      $word_addr=$add_ptr; //@line 609 "translate.cpp"
      var $12=$posn; //@line 610 "translate.cpp"
      var $inc=((($12)+1)&4294967295); //@line 610 "translate.cpp"
      $posn=$inc; //@line 610 "translate.cpp"
      $non_initial=1; //@line 611 "translate.cpp"
      var $13=$phonemes_addr; //@line 612 "translate.cpp"
      var $arrayidx=(($13)&4294967295); //@line 612 "translate.cpp"
      var $14=HEAP8[($arrayidx)]; //@line 612 "translate.cpp"
      var $conv4=(tempInt=(($14)),(tempInt>=128?tempInt-256:tempInt)); //@line 612 "translate.cpp"
      var $cmp5=((($conv4))|0)==21; //@line 612 "translate.cpp"
      if ($cmp5) { __label__ = 6;break $while_cond$4; } else { __label__ = 2;continue $while_cond$4; } //@line 612 "translate.cpp"
    }
    $land_end_thread$$while_end$$if_then6$8: do { 
      if (__label__ == 3) {
  
        var $3=$tr_addr; //@line 609 "translate.cpp"
        __lastLabel__ = 3; __label__ = 7;break $land_end_thread$$while_end$$if_then6$8;
      }
      else if (__label__ == 6) {
  
        var $15=$phonemes_addr; //@line 615 "translate.cpp"
        var $call7=_strcpy(((_word_phonemes)&4294967295), $15); //@line 615 "translate.cpp"
        $retval=0; //@line 616 "translate.cpp"
        __label__ = 8;break $land_end_thread$$while_end$$if_then6$8; //@line 616 "translate.cpp"
      }
    } while(0);
    if (__label__ == 7) {
  
      var $16=__lastLabel__ == 3 ? $3 : ($6);
      var $17=$phonemes_addr; //@line 619 "translate.cpp"
      var $18=$spell_word_addr; //@line 619 "translate.cpp"
      var $19=$posn; //@line 619 "translate.cpp"
      __Z17SetSpellingStressP10TranslatorPcii($16, $17, $18, $19); //@line 619 "translate.cpp"
      var $20=$word_addr; //@line 620 "translate.cpp"
      $retval=$20; //@line 620 "translate.cpp"
      ; //@line 620 "translate.cpp"
    }
  
    var $21=$retval; //@line 621 "translate.cpp"
    ;
    return $21; //@line 621 "translate.cpp"
    return null;
  }
  

  function __Z14SetTranslator2PKc($new_language) {
    ;
    var __label__;
  
    var $new_language_addr;
    var $new_phoneme_tab;
    $new_language_addr=$new_language;
    var $0=$new_language_addr; //@line 1571 "translate.cpp"
    var $call=__Z22SelectPhonemeTableNamePKc($0); //@line 1571 "translate.cpp"
    $new_phoneme_tab=$call; //@line 1571 "translate.cpp"
    var $cmp=((($call))|0) >= 0; //@line 1571 "translate.cpp"
    if ($cmp) { __label__ = 1;; } else { __label__ = 7;; } //@line 1571 "translate.cpp"
    $if_then$$if_end14$2: do { 
      if (__label__ == 1) {
  
        var $1=HEAP32[((_translator2)>>2)]; //@line 1573 "translate.cpp"
        var $cmp1=((($1))|0)!=0; //@line 1573 "translate.cpp"
        if ($cmp1) { __label__ = 2;; } else { __label__ = 5;; } //@line 1573 "translate.cpp"
        if (__label__ == 2) {
  
          var $2=$new_language_addr; //@line 1573 "translate.cpp"
          var $call2=_strcmp($2, ((__ZL20translator2_language)&4294967295)); //@line 1573 "translate.cpp"
          var $cmp3=((($call2))|0)!=0; //@line 1573 "translate.cpp"
          var $3=HEAP32[((_translator2)>>2)]; //@line 1576 "translate.cpp"
          if ($cmp3) { __label__ = 3;; } else { __label__ = 4;; } //@line 1573 "translate.cpp"
          if (__label__ == 3) {
  
            __Z16DeleteTranslatorP10Translator($3); //@line 1576 "translate.cpp"
            HEAP32[((_translator2)>>2)]=0; //@line 1577 "translate.cpp"
            ; //@line 1578 "translate.cpp"
          }
          else if (__label__ == 4) {
  
            var $cmp5=((($3))|0)==0; //@line 1580 "translate.cpp"
            if (!($cmp5)) { __label__ = 7;break $if_then$$if_end14$2; } //@line 1580 "translate.cpp"
          }
        }
  
        var $4=$new_language_addr; //@line 1582 "translate.cpp"
        var $call7=__Z16SelectTranslatorPKc($4); //@line 1582 "translate.cpp"
        HEAP32[((_translator2)>>2)]=$call7; //@line 1582 "translate.cpp"
        var $5=$new_language_addr; //@line 1583 "translate.cpp"
        var $call8=_strcpy(((__ZL20translator2_language)&4294967295), $5); //@line 1583 "translate.cpp"
        var $6=HEAP32[((_translator2)>>2)]; //@line 1585 "translate.cpp"
        var $7=$new_language_addr; //@line 1585 "translate.cpp"
        var $call9=__Z14LoadDictionaryP10TranslatorPKci($6, $7, 0); //@line 1585 "translate.cpp"
        var $cmp10=((($call9))|0)!=0; //@line 1585 "translate.cpp"
        if (!($cmp10)) { __label__ = 7;break $if_then$$if_end14$2; } //@line 1585 "translate.cpp"
  
        var $8=HEAP32[((_voice)>>2)]; //@line 1587 "translate.cpp"
        var $phoneme_tab_ix=(($8+40)&4294967295); //@line 1587 "translate.cpp"
        var $9=HEAP32[(($phoneme_tab_ix)>>2)]; //@line 1587 "translate.cpp"
        __Z18SelectPhonemeTablei($9); //@line 1587 "translate.cpp"
        $new_phoneme_tab=-1; //@line 1588 "translate.cpp"
        HEAP8[(((__ZL20translator2_language)&4294967295))]=0; //@line 1589 "translate.cpp"
        ; //@line 1590 "translate.cpp"
      }
    } while(0);
  
    var $10=$new_phoneme_tab; //@line 1593 "translate.cpp"
    ;
    return $10; //@line 1593 "translate.cpp"
    return null;
  }
  

  function __Z15TranslateClauseP10TranslatorP7__sFILEPKvPiPPc($tr, $f_text, $vp_input, $tone_out, $voice_change) {
    var __stackBase__  = STACKTOP; STACKTOP += 6500; _memset(__stackBase__, 0, 6500);
    var __label__;
    var __lastLabel__ = null;
  
    var $retval;
    var $tr_addr;
    var $f_text_addr;
    var $vp_input_addr;
    var $tone_out_addr;
    var $voice_change_addr;
    var $ix;
    var $c;
    var $cc=__stackBase__;
    var $source_index=__stackBase__+4;
    var $prev_source_index;
    var $source_index_word;
    var $prev_in=__stackBase__+8;
    var $prev_out=__stackBase__+12;
    var $prev_out2;
    var $prev_in2;
    var $prev_in_save;
    var $next_in=__stackBase__+16;
    var $next_in_nbytes;
    var $char_inserted=__stackBase__+20;
    var $clause_pause;
    var $pre_pause_add;
    var $word_mark;
    var $all_upper_case;
    var $finished;
    var $single_quoted;
    var $phoneme_mode;
    var $dict_flags;
    var $word_flags;
    var $next_word_flags;
    var $embedded_count;
    var $letter_count;
    var $space_inserted;
    var $syllable_marked;
    var $decimal_sep_count;
    var $word;
    var $p;
    var $j;
    var $k;
    var $n_digits;
    var $individual_digits;
    var $charix_top=__stackBase__+24;
    var $charix=__stackBase__+28;
    var $words=__stackBase__+1436;
    var $word_count;
    var $sbuf=__stackBase__+5036;
    var $terminator;
    var $tone;
    var $tone2=__stackBase__+5736;
    var $srcix;
    var $next2_in=__stackBase__+5740;
    var $nx;
    var $c_temp=__stackBase__+5744;
    var $pn;
    var $pw;
    var $nw;
    var $number_buf=__stackBase__+5748;
    var $num_wtab=__stackBase__+5900;
    var $p2;
    $tr_addr=$tr;
    $f_text_addr=$f_text;
    $vp_input_addr=$vp_input;
    $tone_out_addr=$tone_out;
    $voice_change_addr=$voice_change;
    HEAP32[(($source_index)>>2)]=0; //@line 2344 "translate.cpp"
    $prev_source_index=0; //@line 2345 "translate.cpp"
    $source_index_word=0; //@line 2346 "translate.cpp"
    HEAP32[(($prev_out)>>2)]=32; //@line 2348 "translate.cpp"
    $prev_in2=0; //@line 2350 "translate.cpp"
    $prev_in_save=0; //@line 2351 "translate.cpp"
    HEAP32[(($char_inserted)>>2)]=0; //@line 2354 "translate.cpp"
    $pre_pause_add=0; //@line 2356 "translate.cpp"
    $word_mark=0; //@line 2357 "translate.cpp"
    $all_upper_case=1; //@line 2358 "translate.cpp"
    $phoneme_mode=0; //@line 2361 "translate.cpp"
    $dict_flags=0; //@line 2362 "translate.cpp"
    $embedded_count=0; //@line 2365 "translate.cpp"
    $letter_count=0; //@line 2366 "translate.cpp"
    $space_inserted=0; //@line 2367 "translate.cpp"
    $syllable_marked=0; //@line 2368 "translate.cpp"
    $decimal_sep_count=0; //@line 2369 "translate.cpp"
    HEAP32[(($charix_top)>>2)]=0; //@line 2375 "translate.cpp"
    $word_count=0; //@line 2380 "translate.cpp"
    var $0=$tr_addr; //@line 2388 "translate.cpp"
    var $cmp=((($0))|0)==0; //@line 2388 "translate.cpp"
    if ($cmp) { __label__ = 1;; } else { __label__ = 2;; } //@line 2388 "translate.cpp"
    $if_then$$if_end$2: do { 
      if (__label__ == 1) {
  
        $retval=0; //@line 2390 "translate.cpp"
        ; //@line 2390 "translate.cpp"
      }
      else if (__label__ == 2) {
  
        var $1=$vp_input_addr; //@line 2393 "translate.cpp"
        HEAP32[((_p_textinput)>>2)]=$1; //@line 2393 "translate.cpp"
        var $2=$vp_input_addr; //@line 2394 "translate.cpp"
        var $3=$2; //@line 2394 "translate.cpp"
        HEAP32[((_p_wchar_input)>>2)]=$3; //@line 2394 "translate.cpp"
        HEAP32[((__ZL11embedded_ix)>>2)]=0; //@line 2396 "translate.cpp"
        HEAP32[((__ZL13embedded_read)>>2)]=0; //@line 2397 "translate.cpp"
        HEAP32[((_pre_pause)>>2)]=0; //@line 2398 "translate.cpp"
        HEAP8[(__ZL18any_stressed_words_b)]=0;
        var $4=HEAP32[((_count_characters)>>2)]; //@line 2401 "translate.cpp"
        HEAP32[((_clause_start_char)>>2)]=$4; //@line 2401 "translate.cpp"
        var $cmp1=((($4))|0) < 0; //@line 2401 "translate.cpp"
        if ($cmp1) { __label__ = 3;; } else { __label__ = 4;; } //@line 2401 "translate.cpp"
        if (__label__ == 3) {
  
          HEAP32[((_clause_start_char)>>2)]=0; //@line 2402 "translate.cpp"
          ; //@line 2402 "translate.cpp"
        }
  
        var $5=HEAP32[((_count_words)>>2)]; //@line 2403 "translate.cpp"
        var $add=((($5)+1)&4294967295); //@line 2403 "translate.cpp"
        HEAP32[((_clause_start_word)>>2)]=$add; //@line 2403 "translate.cpp"
        $ix=0; //@line 2405 "translate.cpp"
        ; //@line 2405 "translate.cpp"
        $for_body$8: while(1) { 
  
          var $6=$ix; //@line 2406 "translate.cpp"
          var $arrayidx=(($charix+$6*2)&4294967295); //@line 2406 "translate.cpp"
          HEAP16[(($arrayidx)>>1)]=0; //@line 2406 "translate.cpp"
          var $7=$ix; //@line 2405 "translate.cpp"
          var $inc=((($7)+1)&4294967295); //@line 2405 "translate.cpp"
          $ix=$inc; //@line 2405 "translate.cpp"
          var $cmp4=((($inc))|0) < 700; //@line 2405 "translate.cpp"
          if ($cmp4) { __label__ = 5;continue $for_body$8; } else { __label__ = 6;break $for_body$8; } //@line 2405 "translate.cpp"
        }
  
        var $8=$tr_addr; //@line 2407 "translate.cpp"
        var $9=$f_text_addr; //@line 2407 "translate.cpp"
        var $arraydecay=(($charix)&4294967295); //@line 2407 "translate.cpp"
        var $call=__Z10ReadClauseP10TranslatorP7__sFILEPcPsPiiS5_S3_($8, $9, ((__ZL6source)&4294967295), $arraydecay, $charix_top, 700, $tone2, ((__ZZ15TranslateClauseP10TranslatorP7__sFILEPKvPiPPcE17voice_change_name)&4294967295)); //@line 2407 "translate.cpp"
        $terminator=$call; //@line 2407 "translate.cpp"
        var $10=HEAP32[((_f_logespeak)>>2)]; //@line 2409 "translate.cpp"
        var $cmp5=((($10))|0)!=0; //@line 2409 "translate.cpp"
        if ($cmp5) { __label__ = 7;; } else { __label__ = 11;; } //@line 2409 "translate.cpp"
        $land_lhs_true$$if_end17$11: do { 
          if (__label__ == 7) {
  
            var $11=HEAP32[((_logging_type)>>2)]; //@line 2409 "translate.cpp"
            var $and=($11) & 4; //@line 2409 "translate.cpp"
            var $tobool=((($and))|0)!=0; //@line 2409 "translate.cpp"
            if (!($tobool)) { __label__ = 11;break $land_lhs_true$$if_end17$11; } //@line 2409 "translate.cpp"
  
            var $12=HEAP32[((_f_logespeak)>>2)]; //@line 2411 "translate.cpp"
            var $13=$terminator; //@line 2411 "translate.cpp"
            var $call7=_fprintf($12, ((__str7550)&4294967295), allocate([$13,0,0,0], ["i32",0,0,0], ALLOC_STACK)); //@line 2411 "translate.cpp"
            $p=((__ZL6source)&4294967295); //@line 2412 "translate.cpp"
            var $14=$p; //@line 2412 "translate.cpp"
            var $15=HEAP8[($14)]; //@line 2412 "translate.cpp"
            var $conv59=(tempInt=(($15)),(tempInt>=128?tempInt-256:tempInt)); //@line 2412 "translate.cpp"
            var $cmp960=((($conv59))|0)!=0; //@line 2412 "translate.cpp"
            if ($cmp960) { __label__ = 9;; } else { __label__ = 10;; } //@line 2412 "translate.cpp"
            $for_body10$$for_end14$14: do { 
              if (__label__ == 9) {
                while(1) { 
  
                  var $16=$p; //@line 2413 "translate.cpp"
                  var $17=HEAP8[($16)]; //@line 2413 "translate.cpp"
                  var $conv11=(tempInt=(($17)),(tempInt>=128?tempInt-256:tempInt)); //@line 2413 "translate.cpp"
                  var $18=HEAP32[((_f_logespeak)>>2)]; //@line 2413 "translate.cpp"
                  var $call12=_fputc($conv11, $18); //@line 2413 "translate.cpp"
                  var $19=$p; //@line 2412 "translate.cpp"
                  var $incdec_ptr=(($19+1)&4294967295); //@line 2412 "translate.cpp"
                  $p=$incdec_ptr; //@line 2412 "translate.cpp"
                  var $20=$p; //@line 2412 "translate.cpp"
                  var $21=HEAP8[($20)]; //@line 2412 "translate.cpp"
                  var $conv=(tempInt=(($21)),(tempInt>=128?tempInt-256:tempInt)); //@line 2412 "translate.cpp"
                  var $cmp9=((($conv))|0)!=0; //@line 2412 "translate.cpp"
                  if ($cmp9) { __label__ = 9;continue ; } else { __label__ = 10;break $for_body10$$for_end14$14; } //@line 2412 "translate.cpp"
                }
              }
            } while(0);
  
            var $22=HEAP32[((_f_logespeak)>>2)]; //@line 2414 "translate.cpp"
            var $23=_fwrite(((__str8551)&4294967295), 10, 1, $22); //@line 2414 "translate.cpp"
            var $24=HEAP32[((_f_logespeak)>>2)]; //@line 2415 "translate.cpp"
            var $call16=_fflush($24); //@line 2415 "translate.cpp"
            ; //@line 2416 "translate.cpp"
          }
        } while(0);
  
        $p=((__ZL6source)&4294967295); //@line 2417 "translate.cpp"
        var $25=HEAP32[(($charix_top)>>2)]; //@line 2419 "translate.cpp"
        var $add18=((($25)+1)&4294967295); //@line 2419 "translate.cpp"
        var $arrayidx19=(($charix+$add18*2)&4294967295); //@line 2419 "translate.cpp"
        HEAP16[(($arrayidx19)>>1)]=0; //@line 2419 "translate.cpp"
        var $26=HEAP32[(($charix_top)>>2)]; //@line 2420 "translate.cpp"
        var $add20=((($26)+2)&4294967295); //@line 2420 "translate.cpp"
        var $arrayidx21=(($charix+$add20*2)&4294967295); //@line 2420 "translate.cpp"
        HEAP16[(($arrayidx21)>>1)]=32767; //@line 2420 "translate.cpp"
        var $27=HEAP32[(($charix_top)>>2)]; //@line 2421 "translate.cpp"
        var $add22=((($27)+3)&4294967295); //@line 2421 "translate.cpp"
        var $arrayidx23=(($charix+$add22*2)&4294967295); //@line 2421 "translate.cpp"
        HEAP16[(($arrayidx23)>>1)]=0; //@line 2421 "translate.cpp"
        var $28=$terminator; //@line 2423 "translate.cpp"
        var $and24=($28) & 4095; //@line 2423 "translate.cpp"
        var $mul=((($and24)*10)&4294967295); //@line 2423 "translate.cpp"
        $clause_pause=$mul; //@line 2423 "translate.cpp"
        var $29=$terminator; //@line 2424 "translate.cpp"
        var $shr=($29) >> 12; //@line 2424 "translate.cpp"
        var $and25=($shr) & 7; //@line 2424 "translate.cpp"
        $tone=$and25; //@line 2424 "translate.cpp"
        var $30=HEAP32[(($tone2)>>2)]; //@line 2425 "translate.cpp"
        var $cmp26=((($30))|0)!=0; //@line 2425 "translate.cpp"
        if ($cmp26) { __label__ = 12;; } else { __label__ = 13;; } //@line 2425 "translate.cpp"
        if (__label__ == 12) {
  
          var $31=HEAP32[(($tone2)>>2)]; //@line 2428 "translate.cpp"
          $tone=$31; //@line 2428 "translate.cpp"
          ; //@line 2429 "translate.cpp"
        }
  
        $p=((__ZL6source)&4294967295); //@line 2431 "translate.cpp"
        ; //@line 2431 "translate.cpp"
        $for_cond29$22: while(1) { 
  
          var $32=$p; //@line 2431 "translate.cpp"
          var $33=HEAP8[($32)]; //@line 2431 "translate.cpp"
          var $conv30=(tempInt=(($33)),(tempInt>=128?tempInt-256:tempInt)); //@line 2431 "translate.cpp"
          var $cmp31=((($conv30))|0)!=0; //@line 2431 "translate.cpp"
          if (!($cmp31)) { __label__ = 17;break $for_cond29$22; } //@line 2431 "translate.cpp"
  
          var $34=$p; //@line 2433 "translate.cpp"
          var $35=HEAP8[($34)]; //@line 2433 "translate.cpp"
          var $conv33=(tempInt=(($35)),(tempInt>=128?tempInt-256:tempInt)); //@line 2433 "translate.cpp"
          var $call34=__Z8isspace2j($conv33); //@line 2433 "translate.cpp"
          var $tobool35=((($call34))|0)!=0; //@line 2433 "translate.cpp"
          if (!($tobool35)) { __label__ = 17;break $for_cond29$22; } //@line 2433 "translate.cpp"
  
          var $36=$p; //@line 2431 "translate.cpp"
          var $incdec_ptr39=(($36+1)&4294967295); //@line 2431 "translate.cpp"
          $p=$incdec_ptr39; //@line 2431 "translate.cpp"
          __label__ = 14;continue $for_cond29$22; //@line 2431 "translate.cpp"
        }
  
        var $37=$p; //@line 2438 "translate.cpp"
        var $38=HEAP8[($37)]; //@line 2438 "translate.cpp"
        var $conv41=(tempInt=(($38)),(tempInt>=128?tempInt-256:tempInt)); //@line 2438 "translate.cpp"
        var $cmp42=((($conv41))|0)==0; //@line 2438 "translate.cpp"
        if ($cmp42) { __label__ = 18;; } else { __label__ = 21;; } //@line 2438 "translate.cpp"
        if (__label__ == 18) {
  
          var $39=HEAP32[((__ZL16max_clause_pause)>>2)]; //@line 2442 "translate.cpp"
          var $40=$clause_pause; //@line 2442 "translate.cpp"
          var $sub=((($40)-($39))&4294967295); //@line 2442 "translate.cpp"
          $clause_pause=$sub; //@line 2442 "translate.cpp"
          var $41=$clause_pause; //@line 2443 "translate.cpp"
          var $cmp44=((($41))|0) < 0; //@line 2443 "translate.cpp"
          if ($cmp44) { __label__ = 19;; } else { __label__ = 20;; } //@line 2443 "translate.cpp"
          if (__label__ == 19) {
  
            $clause_pause=0; //@line 2444 "translate.cpp"
            ; //@line 2444 "translate.cpp"
          }
  
          var $42=$terminator; //@line 2446 "translate.cpp"
          var $and47=($42) & -524289; //@line 2446 "translate.cpp"
          $terminator=$and47; //@line 2446 "translate.cpp"
          var $43=$clause_pause; //@line 2447 "translate.cpp"
          var $44=HEAP32[((__ZL16max_clause_pause)>>2)]; //@line 2447 "translate.cpp"
          var $add48=((($44)+($43))&4294967295); //@line 2447 "translate.cpp"
          HEAP32[((__ZL16max_clause_pause)>>2)]=$add48; //@line 2447 "translate.cpp"
          ; //@line 2448 "translate.cpp"
        }
        else if (__label__ == 21) {
  
          var $45=$clause_pause; //@line 2451 "translate.cpp"
          HEAP32[((__ZL16max_clause_pause)>>2)]=$45; //@line 2451 "translate.cpp"
          ;
        }
  
        var $46=$terminator; //@line 2453 "translate.cpp"
        var $47=$tr_addr; //@line 2453 "translate.cpp"
        var $clause_terminator=(($47+8324)&4294967295); //@line 2453 "translate.cpp"
        HEAP32[(($clause_terminator)>>2)]=$46; //@line 2453 "translate.cpp"
        var $48=HEAP32[((_new_sentence)>>2)]; //@line 2455 "translate.cpp"
        var $tobool50=((($48))|0)!=0; //@line 2455 "translate.cpp"
        if ($tobool50) { __label__ = 23;; } else { __label__ = 26;; } //@line 2455 "translate.cpp"
        $if_then51$$if_end59$34: do { 
          if (__label__ == 23) {
  
            var $49=HEAP32[((_count_sentences)>>2)]; //@line 2457 "translate.cpp"
            var $inc52=((($49)+1)&4294967295); //@line 2457 "translate.cpp"
            HEAP32[((_count_sentences)>>2)]=$inc52; //@line 2457 "translate.cpp"
            var $50=HEAP32[((_skip_sentences)>>2)]; //@line 2458 "translate.cpp"
            var $cmp53=((($50))|0) > 0; //@line 2458 "translate.cpp"
            if (!($cmp53)) { __label__ = 26;break $if_then51$$if_end59$34; } //@line 2458 "translate.cpp"
  
            var $51=HEAP32[((_skip_sentences)>>2)]; //@line 2460 "translate.cpp"
            var $dec=((($51)-1)&4294967295); //@line 2460 "translate.cpp"
            HEAP32[((_skip_sentences)>>2)]=$dec; //@line 2460 "translate.cpp"
            var $52=HEAP32[((_skip_sentences)>>2)]; //@line 2461 "translate.cpp"
            var $cmp55=((($52))|0)==0; //@line 2461 "translate.cpp"
            if (!($cmp55)) { __label__ = 26;break $if_then51$$if_end59$34; } //@line 2461 "translate.cpp"
  
            HEAP32[((_skipping_text)>>2)]=0; //@line 2462 "translate.cpp"
            ; //@line 2462 "translate.cpp"
          }
        } while(0);
  
        var $dest$, $stop$, $stop4$, $fast$, $value4$;
        $dest$ = ((_ph_list2)&4294967295);
        $stop$ = $dest$ + 8;
        $value4$ = 0;
        if ($value4$ < 0) $value4$ += 256;
        $value4$ = $value4$ + ($value4$<<8) + ($value4$<<16) + ($value4$*16777216);
        while ($dest$%4 !== 0 && $dest$ < $stop$) {
          ;; HEAP8[$dest$++] = 0;
        }
        $dest$ >>= 2;
        $stop4$ = $stop$ >> 2;
        while ($dest$ < $stop4$) {
        ;;;;;;;  HEAP32[$dest$++] = $value4$;
        }
        $dest$ <<= 2;
        while ($dest$ < $stop$) {
          ;; HEAP8[$dest$++] = 0;
        }; //@line 2466 "translate.cpp"
        HEAP8[(((_ph_list2)&4294967295))]=10; //@line 2467 "translate.cpp"
        HEAP32[((_n_ph_list2)>>2)]=1; //@line 2469 "translate.cpp"
        var $53=$tr_addr; //@line 2470 "translate.cpp"
        var $prev_last_stress=(($53+8288)&4294967295); //@line 2470 "translate.cpp"
        HEAP32[(($prev_last_stress)>>2)]=0; //@line 2470 "translate.cpp"
        var $54=$tr_addr; //@line 2471 "translate.cpp"
        var $prepause_timeout=(($54+8312)&4294967295); //@line 2471 "translate.cpp"
        HEAP32[(($prepause_timeout)>>2)]=0; //@line 2471 "translate.cpp"
        var $55=$tr_addr; //@line 2472 "translate.cpp"
        var $expect_verb=(($55+8272)&4294967295); //@line 2472 "translate.cpp"
        HEAP32[(($expect_verb)>>2)]=0; //@line 2472 "translate.cpp"
        var $56=$tr_addr; //@line 2473 "translate.cpp"
        var $expect_noun=(($56+8284)&4294967295); //@line 2473 "translate.cpp"
        HEAP32[(($expect_noun)>>2)]=0; //@line 2473 "translate.cpp"
        var $57=$tr_addr; //@line 2474 "translate.cpp"
        var $expect_past=(($57+8276)&4294967295); //@line 2474 "translate.cpp"
        HEAP32[(($expect_past)>>2)]=0; //@line 2474 "translate.cpp"
        var $58=$tr_addr; //@line 2475 "translate.cpp"
        var $expect_verb_s=(($58+8280)&4294967295); //@line 2475 "translate.cpp"
        HEAP32[(($expect_verb_s)>>2)]=0; //@line 2475 "translate.cpp"
        var $59=$tr_addr; //@line 2476 "translate.cpp"
        var $phonemes_repeat_count=(($59+652)&4294967295); //@line 2476 "translate.cpp"
        HEAP32[(($phonemes_repeat_count)>>2)]=0; //@line 2476 "translate.cpp"
        var $60=$tr_addr; //@line 2477 "translate.cpp"
        var $end_stressed_vowel=(($60+8316)&4294967295); //@line 2477 "translate.cpp"
        HEAP32[(($end_stressed_vowel)>>2)]=0; //@line 2477 "translate.cpp"
        var $61=$tr_addr; //@line 2478 "translate.cpp"
        var $prev_dict_flags=(($61+8320)&4294967295); //@line 2478 "translate.cpp"
        HEAP32[(($prev_dict_flags)>>2)]=0; //@line 2478 "translate.cpp"
        $word_count=0; //@line 2480 "translate.cpp"
        $single_quoted=0; //@line 2481 "translate.cpp"
        $word_flags=0; //@line 2482 "translate.cpp"
        $next_word_flags=0; //@line 2483 "translate.cpp"
        var $arrayidx60=(($sbuf)&4294967295); //@line 2485 "translate.cpp"
        HEAP8[($arrayidx60)]=0; //@line 2485 "translate.cpp"
        var $arrayidx61=(($sbuf+1)&4294967295); //@line 2486 "translate.cpp"
        HEAP8[($arrayidx61)]=32; //@line 2486 "translate.cpp"
        var $arrayidx62=(($sbuf+2)&4294967295); //@line 2487 "translate.cpp"
        HEAP8[($arrayidx62)]=32; //@line 2487 "translate.cpp"
        $ix=3; //@line 2488 "translate.cpp"
        HEAP32[(($prev_in)>>2)]=32; //@line 2489 "translate.cpp"
        var $62=$ix; //@line 2491 "translate.cpp"
        var $conv63=((($62)) & 65535); //@line 2491 "translate.cpp"
        var $arrayidx64=(($words)&4294967295); //@line 2491 "translate.cpp"
        var $start=(($arrayidx64+4)&4294967295); //@line 2491 "translate.cpp"
        HEAP16[(($start)>>1)]=$conv63; //@line 2491 "translate.cpp"
        var $arrayidx65=(($words)&4294967295); //@line 2492 "translate.cpp"
        var $flags=(($arrayidx65)&4294967295); //@line 2492 "translate.cpp"
        HEAP32[(($flags)>>2)]=0; //@line 2492 "translate.cpp"
        $finished=0; //@line 2493 "translate.cpp"
        $j=0; //@line 2495 "translate.cpp"
        var $63=$j; //@line 2495 "translate.cpp"
        var $arrayidx6755=(($charix+$63*2)&4294967295); //@line 2495 "translate.cpp"
        var $64=HEAP16[(($arrayidx6755)>>1)]; //@line 2495 "translate.cpp"
        var $conv6856=(tempInt=(($64)),(tempInt>=32768?tempInt-65536:tempInt)); //@line 2495 "translate.cpp"
        var $cmp6957=((($conv6856))|0) <= 0; //@line 2495 "translate.cpp"
        var $65=$j; //@line 2495 "translate.cpp"
        if ($cmp6957) { __lastLabel__ = 26; __label__ = 27;; } else { __lastLabel__ = 26; __label__ = 28;; } //@line 2495 "translate.cpp"
        $for_inc71$$for_end73$39: do { 
          if (__label__ == 27) {
            while(1) { 
  
              var $66=__lastLabel__ == 27 ? $69 : ($65);
              var $inc72=((($66)+1)&4294967295); //@line 2495 "translate.cpp"
              $j=$inc72; //@line 2495 "translate.cpp"
              var $67=$j; //@line 2495 "translate.cpp"
              var $arrayidx67=(($charix+$67*2)&4294967295); //@line 2495 "translate.cpp"
              var $68=HEAP16[(($arrayidx67)>>1)]; //@line 2495 "translate.cpp"
              var $conv68=(tempInt=(($68)),(tempInt>=32768?tempInt-65536:tempInt)); //@line 2495 "translate.cpp"
              var $cmp69=((($conv68))|0) <= 0; //@line 2495 "translate.cpp"
              var $69=$j; //@line 2495 "translate.cpp"
              if ($cmp69) { __lastLabel__ = 27; __label__ = 27;continue ; } else { __lastLabel__ = 27; __label__ = 28;break $for_inc71$$for_end73$39; } //@line 2495 "translate.cpp"
            }
          }
        } while(0);
  
        var $_lcssa54=__lastLabel__ == 26 ? $65 : ($69);
        var $arrayidx74=(($charix+$_lcssa54*2)&4294967295); //@line 2496 "translate.cpp"
        var $70=HEAP16[(($arrayidx74)>>1)]; //@line 2496 "translate.cpp"
        var $arrayidx75=(($words)&4294967295); //@line 2496 "translate.cpp"
        var $sourceix=(($arrayidx75+6)&4294967295); //@line 2496 "translate.cpp"
        HEAP16[(($sourceix)>>1)]=$70; //@line 2496 "translate.cpp"
        $k=0; //@line 2497 "translate.cpp"
        var $71=$j; //@line 2498 "translate.cpp"
        var $arrayidx7651=(($charix+$71*2)&4294967295); //@line 2498 "translate.cpp"
        var $72=HEAP16[(($arrayidx7651)>>1)]; //@line 2498 "translate.cpp"
        var $conv7752=(tempInt=(($72)),(tempInt>=32768?tempInt-65536:tempInt)); //@line 2498 "translate.cpp"
        var $cmp7853=((($conv7752))|0)!=0; //@line 2498 "translate.cpp"
        if ($cmp7853) { __label__ = 29;; } else { __label__ = 32;; } //@line 2498 "translate.cpp"
        $while_body$$while_end$43: do { 
          if (__label__ == 29) {
            while(1) { 
  
              var $73=$j; //@line 2501 "translate.cpp"
              var $inc79=((($73)+1)&4294967295); //@line 2501 "translate.cpp"
              $j=$inc79; //@line 2501 "translate.cpp"
              var $arrayidx80=(($charix+$73*2)&4294967295); //@line 2501 "translate.cpp"
              var $74=HEAP16[(($arrayidx80)>>1)]; //@line 2501 "translate.cpp"
              var $conv81=(tempInt=(($74)),(tempInt>=32768?tempInt-65536:tempInt)); //@line 2501 "translate.cpp"
              var $cmp82=((($conv81))|0)!=-1; //@line 2501 "translate.cpp"
              if ($cmp82) { __label__ = 30;; } else { __label__ = 31;; } //@line 2501 "translate.cpp"
              if (__label__ == 30) {
  
                var $75=$k; //@line 2502 "translate.cpp"
                var $inc84=((($75)+1)&4294967295); //@line 2502 "translate.cpp"
                $k=$inc84; //@line 2502 "translate.cpp"
                ; //@line 2502 "translate.cpp"
              }
  
              var $76=$j; //@line 2498 "translate.cpp"
              var $arrayidx76=(($charix+$76*2)&4294967295); //@line 2498 "translate.cpp"
              var $77=HEAP16[(($arrayidx76)>>1)]; //@line 2498 "translate.cpp"
              var $conv77=(tempInt=(($77)),(tempInt>=32768?tempInt-65536:tempInt)); //@line 2498 "translate.cpp"
              var $cmp78=((($conv77))|0)!=0; //@line 2498 "translate.cpp"
              if ($cmp78) { __label__ = 29;continue ; } else { __label__ = 32;break $while_body$$while_end$43; } //@line 2498 "translate.cpp"
            }
          }
        } while(0);
  
        var $78=$k; //@line 2504 "translate.cpp"
        var $conv86=((($78)) & 255); //@line 2504 "translate.cpp"
        var $arrayidx87=(($words)&4294967295); //@line 2504 "translate.cpp"
        var $length=(($arrayidx87+10)&4294967295); //@line 2504 "translate.cpp"
        HEAP8[($length)]=$conv86; //@line 2504 "translate.cpp"
        ; //@line 2506 "translate.cpp"
        $while_cond88$50: while(1) { 
  
          var $79=$finished; //@line 2506 "translate.cpp"
          var $tobool89=((($79))|0)!=0; //@line 2506 "translate.cpp"
          if ($tobool89) { __label__ = 230;break $while_cond88$50; } //@line 2506 "translate.cpp"
  
          var $80=$ix; //@line 2506 "translate.cpp"
          var $cmp91=((($80))|0) < 700; //@line 2506 "translate.cpp"
          var $81=HEAP32[((_n_ph_list2)>>2)]; //@line 2506 "translate.cpp"
          var $cmp92=((($81))|0) < 996; //@line 2506 "translate.cpp"
          var $or_cond12=($cmp91) & ($cmp92); //@line 2506 "translate.cpp"
          if (!($or_cond12)) { __label__ = 230;break $while_cond88$50; } //@line 2506 "translate.cpp"
  
          var $82=HEAP32[(($prev_out)>>2)]; //@line 2508 "translate.cpp"
          $prev_out2=$82; //@line 2508 "translate.cpp"
          var $83=$ix; //@line 2509 "translate.cpp"
          var $sub94=((($83)-1)&4294967295); //@line 2509 "translate.cpp"
          var $arrayidx95=(($sbuf+$sub94)&4294967295); //@line 2509 "translate.cpp"
          var $call96=__Z8utf8_in2PiPKci($prev_out, $arrayidx95, 1); //@line 2509 "translate.cpp"
          var $84=$tr_addr; //@line 2511 "translate.cpp"
          var $langopts=(($84)&4294967295); //@line 2511 "translate.cpp"
          var $tone_numbers=(($langopts+250)&4294967295); //@line 2511 "translate.cpp"
          var $85=HEAP8[($tone_numbers)]; //@line 2511 "translate.cpp"
          var $tobool97=(tempInt=(($85)),(tempInt>=128?tempInt-256:tempInt))!=0; //@line 2511 "translate.cpp"
          if ($tobool97) { __label__ = 36;; } else { __label__ = 39;; } //@line 2511 "translate.cpp"
          $land_lhs_true98$$if_end105$54: do { 
            if (__label__ == 36) {
  
              var $86=HEAP32[(($prev_out)>>2)]; //@line 2511 "translate.cpp"
              var $call99=__ZL9IsDigit09j($86); //@line 2511 "translate.cpp"
              var $tobool100=((($call99))|0)!=0; //@line 2511 "translate.cpp"
              if (!($tobool100)) { __label__ = 39;break $land_lhs_true98$$if_end105$54; } //@line 2511 "translate.cpp"
  
              var $87=$prev_out2; //@line 2511 "translate.cpp"
              var $call102=__Z7IsAlphaj($87); //@line 2511 "translate.cpp"
              var $tobool103=((($call102))|0)!=0; //@line 2511 "translate.cpp"
              if (!($tobool103)) { __label__ = 39;break $land_lhs_true98$$if_end105$54; } //@line 2511 "translate.cpp"
  
              HEAP32[(($prev_out)>>2)]=97; //@line 2514 "translate.cpp"
              ; //@line 2515 "translate.cpp"
            }
          } while(0);
  
          var $88=$prev_in_save; //@line 2517 "translate.cpp"
          var $cmp106=((($88))|0)!=0; //@line 2517 "translate.cpp"
          if ($cmp106) { __label__ = 40;; } else { __label__ = 41;; } //@line 2517 "translate.cpp"
          $if_then107$$if_else108$59: do { 
            if (__label__ == 40) {
  
              var $89=$prev_in_save; //@line 2519 "translate.cpp"
              HEAP32[(($prev_in)>>2)]=$89; //@line 2519 "translate.cpp"
              $prev_in_save=0; //@line 2520 "translate.cpp"
              ; //@line 2521 "translate.cpp"
            }
            else if (__label__ == 41) {
  
              var $90=HEAPU32[(($source_index)>>2)]; //@line 2523 "translate.cpp"
              var $cmp109=((($90))>>>0) > 0; //@line 2523 "translate.cpp"
              if (!($cmp109)) { __label__ = 43;break $if_then107$$if_else108$59; } //@line 2523 "translate.cpp"
  
              var $91=HEAP32[(($prev_in)>>2)]; //@line 2525 "translate.cpp"
              $prev_in2=$91; //@line 2525 "translate.cpp"
              var $92=HEAP32[(($source_index)>>2)]; //@line 2526 "translate.cpp"
              var $sub111=((($92)-1)&4294967295); //@line 2526 "translate.cpp"
              var $arrayidx112=((__ZL6source+$sub111)&4294967295); //@line 2526 "translate.cpp"
              var $call113=__Z8utf8_in2PiPKci($prev_in, $arrayidx112, 1); //@line 2526 "translate.cpp"
              ; //@line 2527 "translate.cpp"
            }
          } while(0);
  
          var $93=HEAP32[(($source_index)>>2)]; //@line 2529 "translate.cpp"
          $prev_source_index=$93; //@line 2529 "translate.cpp"
          var $94=HEAP32[(($char_inserted)>>2)]; //@line 2531 "translate.cpp"
          var $tobool116=((($94))|0)!=0; //@line 2531 "translate.cpp"
          if ($tobool116) { __label__ = 44;; } else { __label__ = 45;; } //@line 2531 "translate.cpp"
          if (__label__ == 44) {
  
            var $95=HEAP32[(($char_inserted)>>2)]; //@line 2533 "translate.cpp"
            $c=$95; //@line 2533 "translate.cpp"
            HEAP32[(($char_inserted)>>2)]=0; //@line 2534 "translate.cpp"
            ; //@line 2535 "translate.cpp"
          }
          else if (__label__ == 45) {
  
            var $96=HEAP32[(($source_index)>>2)]; //@line 2538 "translate.cpp"
            var $arrayidx119=((__ZL6source+$96)&4294967295); //@line 2538 "translate.cpp"
            var $call120=__Z7utf8_inPiPKc($cc, $arrayidx119); //@line 2538 "translate.cpp"
            var $97=HEAP32[(($source_index)>>2)]; //@line 2538 "translate.cpp"
            var $add121=((($97)+($call120))&4294967295); //@line 2538 "translate.cpp"
            HEAP32[(($source_index)>>2)]=$add121; //@line 2538 "translate.cpp"
            var $98=HEAP32[(($cc)>>2)]; //@line 2539 "translate.cpp"
            $c=$98; //@line 2539 "translate.cpp"
            ;
          }
  
          var $99=HEAP32[(($source_index)>>2)]; //@line 2541 "translate.cpp"
          var $arrayidx123=((__ZL6source+$99)&4294967295); //@line 2541 "translate.cpp"
          var $call124=__Z7utf8_inPiPKc($next_in, $arrayidx123); //@line 2541 "translate.cpp"
          $next_in_nbytes=$call124; //@line 2541 "translate.cpp"
          var $100=$c; //@line 2543 "translate.cpp"
          if ($100 == 0) {
            __label__ = 47;;
          }
          else if ($100 == 1) {
            __label__ = 49;;
          }
          else {
          __label__ = 48;;
          }
          
          $lor_lhs_false$$if_end127_thread$$if_then131$68: do { 
            if (__label__ == 47) {
  
              $finished=1; //@line 2545 "translate.cpp"
              $c=32; //@line 2546 "translate.cpp"
              __label__ = 48;break $lor_lhs_false$$if_end127_thread$$if_then131$68; //@line 2549 "translate.cpp"
            }
          } while(0);
          $lor_lhs_false$$if_then131$70: do { 
            if (__label__ == 48) {
  
              var $101=$c; //@line 2549 "translate.cpp"
              var $102=HEAP8[(_ctrl_embedded)]; //@line 2549 "translate.cpp"
              var $conv129=(tempInt=(($102)),(tempInt>=128?tempInt-256:tempInt)); //@line 2549 "translate.cpp"
              var $cmp130=((($101))|0)==((($conv129))|0); //@line 2549 "translate.cpp"
              if ($cmp130) { __label__ = 49;break $lor_lhs_false$$if_then131$70; } else { __label__ = 52;break $lor_lhs_false$$if_then131$70; } //@line 2549 "translate.cpp"
            }
          } while(0);
          if (__label__ == 49) {
  
            var $103=HEAP32[(($source_index)>>2)]; //@line 2552 "translate.cpp"
            var $sub132=((($103)-1)&4294967295); //@line 2552 "translate.cpp"
            $srcix=$sub132; //@line 2552 "translate.cpp"
            var $104=HEAP32[(($prev_in)>>2)]; //@line 2554 "translate.cpp"
            var $cmp133=((($104))|0)!=32; //@line 2554 "translate.cpp"
            if ($cmp133) { __label__ = 50;; } else { __label__ = 51;; } //@line 2554 "translate.cpp"
            if (__label__ == 50) {
  
              $c=32; //@line 2556 "translate.cpp"
              var $105=$c; //@line 2557 "translate.cpp"
              $prev_in_save=$105; //@line 2557 "translate.cpp"
              var $106=HEAP32[(($source_index)>>2)]; //@line 2558 "translate.cpp"
              var $dec135=((($106)-1)&4294967295); //@line 2558 "translate.cpp"
              HEAP32[(($source_index)>>2)]=$dec135; //@line 2558 "translate.cpp"
              ; //@line 2569 "translate.cpp"
            }
            else if (__label__ == 51) {
  
              var $call137=__ZL15EmbeddedCommandRj($source_index); //@line 2562 "translate.cpp"
              var $107=$embedded_count; //@line 2562 "translate.cpp"
              var $add138=((($107)+($call137))&4294967295); //@line 2562 "translate.cpp"
              $embedded_count=$add138; //@line 2562 "translate.cpp"
              var $108=HEAP32[(($prev_in)>>2)]; //@line 2563 "translate.cpp"
              $prev_in_save=$108; //@line 2563 "translate.cpp"
              var $109=$srcix; //@line 2565 "translate.cpp"
              var $arrayidx139=((__ZL6source+$109)&4294967295); //@line 2565 "translate.cpp"
              var $110=HEAP32[(($source_index)>>2)]; //@line 2565 "translate.cpp"
              var $111=$srcix; //@line 2565 "translate.cpp"
              var $sub140=((($110)-($111))&4294967295); //@line 2565 "translate.cpp"
              var $dest$, $stop$, $stop4$, $fast$, $value4$;
              $dest$ = $arrayidx139;
              $stop$ = $dest$ + $sub140;
              $value4$ = 32;
              if ($value4$ < 0) $value4$ += 256;
              $value4$ = $value4$ + ($value4$<<8) + ($value4$<<16) + ($value4$*16777216);
              while ($dest$%4 !== 0 && $dest$ < $stop$) {
                ;; HEAP8[$dest$++] = 32;
              }
              $dest$ >>= 2;
              $stop4$ = $stop$ >> 2;
              while ($dest$ < $stop4$) {
              ;;;;;;;  HEAP32[$dest$++] = $value4$;
              }
              $dest$ <<= 2;
              while ($dest$ < $stop$) {
                ;; HEAP8[$dest$++] = 32;
              }; //@line 2565 "translate.cpp"
              var $112=$srcix; //@line 2566 "translate.cpp"
              HEAP32[(($source_index)>>2)]=$112; //@line 2566 "translate.cpp"
              __label__ = 33;continue $while_cond88$50; //@line 2567 "translate.cpp"
            }
          }
  
          var $113=HEAP32[((__ZL13option_sayas2)>>2)]; //@line 2571 "translate.cpp"
          var $cmp143=((($113))|0)==36; //@line 2571 "translate.cpp"
          if ($cmp143) { __label__ = 53;; } else { __label__ = 58;; } //@line 2571 "translate.cpp"
          $land_lhs_true144$$if_end153$78: do { 
            if (__label__ == 53) {
  
              var $114=$c; //@line 2571 "translate.cpp"
              var $cmp145=((($114))|0)!=32; //@line 2571 "translate.cpp"
              if (!($cmp145)) { __label__ = 58;break $land_lhs_true144$$if_end153$78; } //@line 2571 "translate.cpp"
  
              var $115=HEAP32[(($prev_in)>>2)]; //@line 2573 "translate.cpp"
              var $cmp147=((($115))|0)==32; //@line 2573 "translate.cpp"
              if ($cmp147) { __label__ = 55;; } else { __label__ = 57;; } //@line 2573 "translate.cpp"
              $land_lhs_true148$$if_end151$81: do { 
                if (__label__ == 55) {
  
                  var $116=HEAP32[(($next_in)>>2)]; //@line 2573 "translate.cpp"
                  var $cmp149=((($116))|0)==32; //@line 2573 "translate.cpp"
                  if (!($cmp149)) { __label__ = 57;break $land_lhs_true148$$if_end151$81; } //@line 2573 "translate.cpp"
  
                  HEAP32[((__ZL13option_sayas2)>>2)]=20; //@line 2574 "translate.cpp"
                  ; //@line 2574 "translate.cpp"
                }
              } while(0);
  
              var $117=$c; //@line 2575 "translate.cpp"
              var $call152=__Z9towlower2j($117); //@line 2575 "translate.cpp"
              $c=$call152; //@line 2575 "translate.cpp"
              ; //@line 2576 "translate.cpp"
            }
          } while(0);
  
          var $118=$phoneme_mode; //@line 2579 "translate.cpp"
          var $tobool154=((($118))|0)!=0; //@line 2579 "translate.cpp"
          if ($tobool154) { __label__ = 59;; } else { __label__ = 62;; } //@line 2579 "translate.cpp"
          $if_then155$$if_else162$86: do { 
            if (__label__ == 59) {
  
              $all_upper_case=8; //@line 2581 "translate.cpp"
              var $119=$c; //@line 2583 "translate.cpp"
              var $cmp156=((($119))|0)==93; //@line 2583 "translate.cpp"
              if (!($cmp156)) { __label__ = 195;break $if_then155$$if_else162$86; } //@line 2583 "translate.cpp"
  
              var $120=HEAP32[(($next_in)>>2)]; //@line 2583 "translate.cpp"
              var $cmp158=((($120))|0)==93; //@line 2583 "translate.cpp"
              if (!($cmp158)) { __label__ = 195;break $if_then155$$if_else162$86; } //@line 2583 "translate.cpp"
  
              $phoneme_mode=0; //@line 2585 "translate.cpp"
              var $121=HEAP32[(($source_index)>>2)]; //@line 2586 "translate.cpp"
              var $inc160=((($121)+1)&4294967295); //@line 2586 "translate.cpp"
              HEAP32[(($source_index)>>2)]=$inc160; //@line 2586 "translate.cpp"
              $c=32; //@line 2587 "translate.cpp"
              ; //@line 2588 "translate.cpp"
            }
            else if (__label__ == 62) {
  
              var $122=HEAP32[((__ZL13option_sayas2)>>2)]; //@line 2591 "translate.cpp"
              var $and163=($122) & 240; //@line 2591 "translate.cpp"
              var $cmp164=((($and163))|0)==64; //@line 2591 "translate.cpp"
              if ($cmp164) { __label__ = 63;; } else { __label__ = 68;; } //@line 2591 "translate.cpp"
              if (__label__ == 63) {
  
                var $123=$c; //@line 2593 "translate.cpp"
                var $call166=_iswdigit($123); //@line 2593 "translate.cpp"
                var $tobool167=((($call166))|0)!=0; //@line 2593 "translate.cpp"
                if ($tobool167) { __label__ = 64;; } else { __label__ = 66;; } //@line 2593 "translate.cpp"
                if (__label__ == 64) {
  
                  var $124=HEAP32[((__ZL18count_sayas_digits)>>2)]; //@line 2595 "translate.cpp"
                  var $inc169=((($124)+1)&4294967295); //@line 2595 "translate.cpp"
                  HEAP32[((__ZL18count_sayas_digits)>>2)]=$inc169; //@line 2595 "translate.cpp"
                  var $125=HEAP32[((__ZL18count_sayas_digits)>>2)]; //@line 2596 "translate.cpp"
                  var $126=HEAP32[((__ZL13option_sayas2)>>2)]; //@line 2596 "translate.cpp"
                  var $and170=($126) & 15; //@line 2596 "translate.cpp"
                  var $cmp171=((($125))|0) > ((($and170))|0); //@line 2596 "translate.cpp"
                  if (!($cmp171)) { __label__ = 195;break $if_then155$$if_else162$86; } //@line 2596 "translate.cpp"
  
                  $c=32; //@line 2599 "translate.cpp"
                  $space_inserted=1; //@line 2600 "translate.cpp"
                  HEAP32[((__ZL18count_sayas_digits)>>2)]=0; //@line 2601 "translate.cpp"
                  ; //@line 2602 "translate.cpp"
                }
                else if (__label__ == 66) {
  
                  HEAP32[((__ZL18count_sayas_digits)>>2)]=0; //@line 2606 "translate.cpp"
                  var $127=HEAP32[(($prev_out)>>2)]; //@line 2607 "translate.cpp"
                  var $call175=_iswdigit($127); //@line 2607 "translate.cpp"
                  var $tobool176=((($call175))|0)!=0; //@line 2607 "translate.cpp"
                  if (!($tobool176)) { __label__ = 195;break $if_then155$$if_else162$86; } //@line 2607 "translate.cpp"
  
                  $c=32; //@line 2609 "translate.cpp"
                  $space_inserted=1; //@line 2610 "translate.cpp"
                  ; //@line 2611 "translate.cpp"
                }
              }
              else if (__label__ == 68) {
  
                var $128=HEAP32[((__ZL13option_sayas2)>>2)]; //@line 2615 "translate.cpp"
                var $and181=($128) & 16; //@line 2615 "translate.cpp"
                var $cmp182=((($and181))|0)==0; //@line 2615 "translate.cpp"
                if (!($cmp182)) { __label__ = 195;break $if_then155$$if_else162$86; } //@line 2615 "translate.cpp"
  
                var $129=$c; //@line 2628 "translate.cpp"
                var $cmp184=((($129))|0)==146; //@line 2628 "translate.cpp"
                var $130=$c; //@line 2628 "translate.cpp"
                var $cmp186=((($130))|0)==180; //@line 2628 "translate.cpp"
                var $or_cond=($cmp184) | ($cmp186); //@line 2628 "translate.cpp"
                var $131=$c; //@line 2628 "translate.cpp"
                var $cmp188=((($131))|0)==8217; //@line 2628 "translate.cpp"
                var $or_cond2=($or_cond) | ($cmp188); //@line 2628 "translate.cpp"
                var $132=$c; //@line 2628 "translate.cpp"
                var $cmp190=((($132))|0)==8242; //@line 2628 "translate.cpp"
                var $or_cond3=($or_cond2) | ($cmp190); //@line 2628 "translate.cpp"
                if ($or_cond3) { __label__ = 70;; } else { __label__ = 71;; } //@line 2628 "translate.cpp"
                if (__label__ == 70) {
  
                  $c=39; //@line 2629 "translate.cpp"
                  ; //@line 2629 "translate.cpp"
                }
  
                var $133=$c; //@line 2631 "translate.cpp"
                var $cmp193=((($133))|0)==8216; //@line 2631 "translate.cpp"
                var $134=$c; //@line 2631 "translate.cpp"
                var $cmp195=((($134))|0)==63; //@line 2631 "translate.cpp"
                var $or_cond4=($cmp193) | ($cmp195); //@line 2631 "translate.cpp"
                if ($or_cond4) { __lastLabel__ = 71; __label__ = 72;; } else { __lastLabel__ = 71; __label__ = 76;; } //@line 2631 "translate.cpp"
                $land_lhs_true196$$if_end203$103: do { 
                  if (__label__ == 72) {
  
                    var $135=HEAP32[(($prev_out)>>2)]; //@line 2631 "translate.cpp"
                    var $call197=__Z7IsAlphaj($135); //@line 2631 "translate.cpp"
                    var $tobool198=((($call197))|0)!=0; //@line 2631 "translate.cpp"
                    if ($tobool198) { __label__ = 73;; } else { __label__ = 75;; } //@line 2631 "translate.cpp"
                    $land_lhs_true199$$if_end203thread_pre_split$105: do { 
                      if (__label__ == 73) {
  
                        var $136=HEAP32[(($next_in)>>2)]; //@line 2631 "translate.cpp"
                        var $call200=__Z7IsAlphaj($136); //@line 2631 "translate.cpp"
                        var $tobool201=((($call200))|0)!=0; //@line 2631 "translate.cpp"
                        if (!($tobool201)) { __label__ = 75;break $land_lhs_true199$$if_end203thread_pre_split$105; } //@line 2631 "translate.cpp"
  
                        $c=39; //@line 2634 "translate.cpp"
                        __label__ = 80;break $land_lhs_true196$$if_end203$103; //@line 2644 "translate.cpp"
                      }
                    } while(0);
  
                    var $_pr=$c; //@line 2637 "translate.cpp"
                    __lastLabel__ = 75; __label__ = 76;break $land_lhs_true196$$if_end203$103;
                  }
                } while(0);
                $if_end210$$if_end203$109: do { 
                  if (__label__ == 76) {
  
                    var $137=__lastLabel__ == 75 ? $_pr : ($134); //@line 2637 "translate.cpp"
                    var $cmp204=((($137))|0)==1328; //@line 2637 "translate.cpp"
                    if ($cmp204) { __label__ = 77;; } else { __label__ = 78;; } //@line 2637 "translate.cpp"
                    if (__label__ == 77) {
  
                      $c=32; //@line 2640 "translate.cpp"
                      var $138=$word_flags; //@line 2641 "translate.cpp"
                      var $or=($138) | 1024; //@line 2641 "translate.cpp"
                      $word_flags=$or; //@line 2641 "translate.cpp"
                      ; //@line 2644 "translate.cpp"
                    }
                    else if (__label__ == 78) {
  
                      var $_pr15_pr=$c; //@line 2644 "translate.cpp"
                      var $cmp207=((($_pr15_pr))|0)==1367; //@line 2644 "translate.cpp"
                      if (!($cmp207)) { __label__ = 80;break $if_end210$$if_end203$109; } //@line 2644 "translate.cpp"
  
                      $c=32; //@line 2646 "translate.cpp"
                      var $139=$word_flags; //@line 2647 "translate.cpp"
                      var $or209=($139) | 131072; //@line 2647 "translate.cpp"
                      $word_flags=$or209; //@line 2647 "translate.cpp"
                      ; //@line 2648 "translate.cpp"
                    }
                  }
                } while(0);
  
                var $140=$tr_addr; //@line 2650 "translate.cpp"
                var $141=HEAP32[(($source_index)>>2)]; //@line 2650 "translate.cpp"
                var $arrayidx211=((__ZL6source+$141)&4294967295); //@line 2650 "translate.cpp"
                var $142=HEAP32[(($prev_in)>>2)]; //@line 2650 "translate.cpp"
                var $143=$c; //@line 2650 "translate.cpp"
                var $144=HEAP32[(($next_in)>>2)]; //@line 2650 "translate.cpp"
                var $call212=__ZL13TranslateCharP10TranslatorPcijjPi($140, $arrayidx211, $142, $143, $144, $char_inserted); //@line 2650 "translate.cpp"
                $c=$call212; //@line 2650 "translate.cpp"
                var $cmp213=((($call212))|0)==8; //@line 2651 "translate.cpp"
                if ($cmp213) { __label__ = 33;continue $while_cond88$50; } //@line 2651 "translate.cpp"
  
                var $145=HEAP32[(($char_inserted)>>2)]; //@line 2654 "translate.cpp"
                var $tobool216=((($145))|0)!=0; //@line 2654 "translate.cpp"
                if ($tobool216) { __label__ = 82;; } else { __label__ = 83;; } //@line 2654 "translate.cpp"
                if (__label__ == 82) {
  
                  var $146=HEAP32[(($char_inserted)>>2)]; //@line 2655 "translate.cpp"
                  HEAP32[(($next_in)>>2)]=$146; //@line 2655 "translate.cpp"
                  ; //@line 2655 "translate.cpp"
                }
  
                var $147=$c; //@line 2658 "translate.cpp"
                var $call219=__Z7IsAlphaj($147); //@line 2658 "translate.cpp"
                var $tobool220=((($call219))|0)!=0; //@line 2658 "translate.cpp"
                if ($tobool220) { __label__ = 91;; } else { __label__ = 84;; } //@line 2658 "translate.cpp"
                $if_end244$$land_lhs_true221$120: do { 
                  if (__label__ == 84) {
  
                    var $148=$c; //@line 2658 "translate.cpp"
                    var $call222=__Z7IsSpacej($148); //@line 2658 "translate.cpp"
                    var $tobool223=((($call222))|0)!=0; //@line 2658 "translate.cpp"
                    if ($tobool223) { __label__ = 91;break $if_end244$$land_lhs_true221$120; } //@line 2658 "translate.cpp"
  
                    var $149=$tr_addr; //@line 2658 "translate.cpp"
                    var $punct_within_word=(($149+704)&4294967295); //@line 2658 "translate.cpp"
                    var $150=HEAP32[(($punct_within_word)>>2)]; //@line 2658 "translate.cpp"
                    var $151=$c; //@line 2658 "translate.cpp"
                    var $call225=_wcschr($150, $151); //@line 2658 "translate.cpp"
                    var $cmp226=((($call225))|0)==0; //@line 2658 "translate.cpp"
                    if (!($cmp226)) { __label__ = 91;break $if_end244$$land_lhs_true221$120; } //@line 2658 "translate.cpp"
  
                    var $152=HEAP32[(($prev_out)>>2)]; //@line 2660 "translate.cpp"
                    var $call228=__Z7IsAlphaj($152); //@line 2660 "translate.cpp"
                    var $tobool229=((($call228))|0)!=0; //@line 2660 "translate.cpp"
                    if (!($tobool229)) { __label__ = 91;break $if_end244$$land_lhs_true221$120; } //@line 2660 "translate.cpp"
  
                    var $153=$tr_addr; //@line 2662 "translate.cpp"
                    var $langopts231=(($153)&4294967295); //@line 2662 "translate.cpp"
                    var $tone_numbers232=(($langopts231+250)&4294967295); //@line 2662 "translate.cpp"
                    var $154=HEAP8[($tone_numbers232)]; //@line 2662 "translate.cpp"
                    var $tobool233=(tempInt=(($154)),(tempInt>=128?tempInt-256:tempInt))!=0; //@line 2662 "translate.cpp"
                    if ($tobool233) { __label__ = 88;; } else { __label__ = 90;; } //@line 2662 "translate.cpp"
                    $land_lhs_true234$$if_else241$125: do { 
                      if (__label__ == 88) {
  
                        var $155=$c; //@line 2662 "translate.cpp"
                        var $call235=__ZL9IsDigit09j($155); //@line 2662 "translate.cpp"
                        var $tobool236=((($call235))|0)!=0; //@line 2662 "translate.cpp"
                        if (!($tobool236)) { __label__ = 90;break $land_lhs_true234$$if_else241$125; } //@line 2662 "translate.cpp"
  
                        var $156=HEAP32[(($next_in)>>2)]; //@line 2662 "translate.cpp"
                        var $call238=__ZL9IsDigit09j($156); //@line 2662 "translate.cpp"
                        var $tobool239=((($call238))|0)!=0; //@line 2662 "translate.cpp"
                        if (!($tobool239)) { __label__ = 91;break $if_end244$$land_lhs_true221$120; } //@line 2662 "translate.cpp"
                      }
                    } while(0);
  
                    $c=32; //@line 2668 "translate.cpp"
                    $space_inserted=1; //@line 2669 "translate.cpp"
                    ;
                  }
                } while(0);
  
                var $157=HEAP32[(($prev_out)>>2)]; //@line 2674 "translate.cpp"
                var $call245=_iswdigit($157); //@line 2674 "translate.cpp"
                var $tobool246=((($call245))|0)!=0; //@line 2674 "translate.cpp"
                if ($tobool246) { __label__ = 92;; } else { __label__ = 94;; } //@line 2674 "translate.cpp"
                $if_then247$$if_else258$130: do { 
                  if (__label__ == 92) {
  
                    var $158=$c; //@line 2676 "translate.cpp"
                    var $call248=_iswdigit($158); //@line 2676 "translate.cpp"
                    var $tobool249=((($call248))|0)==0; //@line 2676 "translate.cpp"
                    var $159=$c; //@line 2676 "translate.cpp"
                    var $cmp251=((($159))|0)!=46; //@line 2676 "translate.cpp"
                    var $or_cond5=($tobool249) & ($cmp251); //@line 2676 "translate.cpp"
                    var $160=$c; //@line 2676 "translate.cpp"
                    var $cmp253=((($160))|0)!=44; //@line 2676 "translate.cpp"
                    var $or_cond6=($or_cond5) & ($cmp253); //@line 2676 "translate.cpp"
                    var $161=$c; //@line 2676 "translate.cpp"
                    var $cmp255=((($161))|0)!=32; //@line 2676 "translate.cpp"
                    var $or_cond7=($or_cond6) & ($cmp255); //@line 2676 "translate.cpp"
                    if ($or_cond7) { __lastLabel__ = 92; ; } else { __lastLabel__ = 92; __label__ = 97;break $if_then247$$if_else258$130; } //@line 2676 "translate.cpp"
  
                    $c=32; //@line 2678 "translate.cpp"
                    $space_inserted=1; //@line 2679 "translate.cpp"
                    __label__ = 101;break $if_then247$$if_else258$130; //@line 2690 "translate.cpp"
                  }
                  else if (__label__ == 94) {
  
                    var $162=HEAP32[(($prev_in)>>2)]; //@line 2684 "translate.cpp"
                    var $cmp259=((($162))|0)!=44; //@line 2684 "translate.cpp"
                    if ($cmp259) { __label__ = 95;; } else { __label__ = 96;; } //@line 2684 "translate.cpp"
                    if (__label__ == 95) {
  
                      $decimal_sep_count=0; //@line 2686 "translate.cpp"
                      ; //@line 2687 "translate.cpp"
                    }
  
                    var $_pr17=$c; //@line 2690 "translate.cpp"
                    __lastLabel__ = 96; __label__ = 97;break $if_then247$$if_else258$130;
                  }
                } while(0);
                $if_end273$$if_end262$137: do { 
                  if (__label__ == 97) {
  
                    var $163=__lastLabel__ == 96 ? $_pr17 : ($161); //@line 2690 "translate.cpp"
                    var $cmp263=((($163))|0)==91; //@line 2690 "translate.cpp"
                    if (!($cmp263)) { __label__ = 101;break $if_end273$$if_end262$137; } //@line 2690 "translate.cpp"
  
                    var $164=HEAP32[(($next_in)>>2)]; //@line 2692 "translate.cpp"
                    var $cmp265=((($164))|0)==2; //@line 2692 "translate.cpp"
                    if ($cmp265) { __label__ = 100;; } else { __label__ = 99;; } //@line 2692 "translate.cpp"
                    if (__label__ == 99) {
  
                      var $165=HEAP32[(($next_in)>>2)]; //@line 2692 "translate.cpp"
                      var $cmp267=((($165))|0)==91; //@line 2692 "translate.cpp"
                      var $166=HEAP32[((_option_phoneme_input)>>2)]; //@line 2692 "translate.cpp"
                      var $tobool269=((($166))|0)!=0; //@line 2692 "translate.cpp"
                      var $or_cond8=($cmp267) & ($tobool269); //@line 2692 "translate.cpp"
                      if (!($or_cond8)) { __label__ = 101;break $if_end273$$if_end262$137; } //@line 2692 "translate.cpp"
                    }
  
                    $phoneme_mode=8; //@line 2695 "translate.cpp"
                    var $167=HEAP32[(($source_index)>>2)]; //@line 2696 "translate.cpp"
                    var $inc271=((($167)+1)&4294967295); //@line 2696 "translate.cpp"
                    HEAP32[(($source_index)>>2)]=$inc271; //@line 2696 "translate.cpp"
                    __label__ = 33;continue $while_cond88$50; //@line 2697 "translate.cpp"
                  }
                } while(0);
  
                var $168=$c; //@line 2701 "translate.cpp"
                var $call274=__Z7IsAlphaj($168); //@line 2701 "translate.cpp"
                var $tobool275=((($call274))|0)!=0; //@line 2701 "translate.cpp"
                if ($tobool275) { __label__ = 102;; } else { __label__ = 138;; } //@line 2701 "translate.cpp"
                if (__label__ == 102) {
  
                  var $169=HEAP32[(($prev_out)>>2)]; //@line 2703 "translate.cpp"
                  var $call277=__Z7IsAlphaj($169); //@line 2703 "translate.cpp"
                  var $tobool278=((($call277))|0)!=0; //@line 2703 "translate.cpp"
                  if ($tobool278) { __label__ = 103;; } else { __label__ = 106;; } //@line 2703 "translate.cpp"
                  $lor_lhs_false279$$if_then286$146: do { 
                    if (__label__ == 103) {
  
                      var $170=$tr_addr; //@line 2703 "translate.cpp"
                      var $langopts280=(($170)&4294967295); //@line 2703 "translate.cpp"
                      var $ideographs=(($langopts280+251)&4294967295); //@line 2703 "translate.cpp"
                      var $171=HEAP8[($ideographs)]; //@line 2703 "translate.cpp"
                      var $tobool281=(tempInt=(($171)),(tempInt>=128?tempInt-256:tempInt))!=0; //@line 2703 "translate.cpp"
                      if (!($tobool281)) { __label__ = 118;break $lor_lhs_false279$$if_then286$146; } //@line 2703 "translate.cpp"
  
                      var $172=$c; //@line 2703 "translate.cpp"
                      var $cmp283=((($172))|0) > 12352; //@line 2703 "translate.cpp"
                      if ($cmp283) { __label__ = 106;break $lor_lhs_false279$$if_then286$146; } //@line 2703 "translate.cpp"
  
                      var $173=HEAP32[(($prev_out)>>2)]; //@line 2703 "translate.cpp"
                      var $cmp285=((($173))|0) > 12352; //@line 2703 "translate.cpp"
                      if ($cmp285) { __label__ = 106;break $lor_lhs_false279$$if_then286$146; } else { __label__ = 118;break $lor_lhs_false279$$if_then286$146; } //@line 2703 "translate.cpp"
                    }
                  } while(0);
                  $if_then286$$if_end327$150: do { 
                    if (__label__ == 106) {
  
                      var $174=$tr_addr; //@line 2705 "translate.cpp"
                      var $punct_within_word287=(($174+704)&4294967295); //@line 2705 "translate.cpp"
                      var $175=HEAP32[(($punct_within_word287)>>2)]; //@line 2705 "translate.cpp"
                      var $176=HEAP32[(($prev_out)>>2)]; //@line 2705 "translate.cpp"
                      var $call288=_wcschr($175, $176); //@line 2705 "translate.cpp"
                      var $cmp289=((($call288))|0)==0; //@line 2705 "translate.cpp"
                      if ($cmp289) { __label__ = 107;; } else { __label__ = 108;; } //@line 2705 "translate.cpp"
                      if (__label__ == 107) {
  
                        $letter_count=0; //@line 2706 "translate.cpp"
                        ; //@line 2706 "translate.cpp"
                      }
  
                      var $177=HEAP32[(($prev_out)>>2)]; //@line 2708 "translate.cpp"
                      var $cmp292=((($177))|0)!=32; //@line 2708 "translate.cpp"
                      if ($cmp292) { __label__ = 109;; } else { __label__ = 112;; } //@line 2708 "translate.cpp"
                      $land_lhs_true293$$if_else303$155: do { 
                        if (__label__ == 109) {
  
                          var $178=$tr_addr; //@line 2708 "translate.cpp"
                          var $punct_within_word294=(($178+704)&4294967295); //@line 2708 "translate.cpp"
                          var $179=HEAP32[(($punct_within_word294)>>2)]; //@line 2708 "translate.cpp"
                          var $180=HEAP32[(($prev_out)>>2)]; //@line 2708 "translate.cpp"
                          var $call295=_wcschr($179, $180); //@line 2708 "translate.cpp"
                          var $cmp296=((($call295))|0)==0; //@line 2708 "translate.cpp"
                          if (!($cmp296)) { __label__ = 112;break $land_lhs_true293$$if_else303$155; } //@line 2708 "translate.cpp"
  
                          $c=32; //@line 2711 "translate.cpp"
                          $space_inserted=1; //@line 2712 "translate.cpp"
                          var $181=HEAP32[(($prev_out)>>2)]; //@line 2714 "translate.cpp"
                          var $call298=__Z9IsBracketi($181); //@line 2714 "translate.cpp"
                          var $tobool299=((($call298))|0)!=0; //@line 2714 "translate.cpp"
                          if ($tobool299) { __label__ = 118;break $if_then286$$if_end327$150; } //@line 2714 "translate.cpp"
  
                          var $182=$next_word_flags; //@line 2716 "translate.cpp"
                          var $or301=($182) | 256; //@line 2716 "translate.cpp"
                          $next_word_flags=$or301; //@line 2716 "translate.cpp"
                          __label__ = 118;break $if_then286$$if_end327$150; //@line 2717 "translate.cpp"
                        }
                      } while(0);
  
                      var $183=$c; //@line 2721 "translate.cpp"
                      var $call304=_iswupper($183); //@line 2721 "translate.cpp"
                      var $tobool305=((($call304))|0)!=0; //@line 2721 "translate.cpp"
                      if ($tobool305) { __label__ = 113;; } else { __label__ = 114;; } //@line 2721 "translate.cpp"
                      if (__label__ == 113) {
  
                        var $184=$word_flags; //@line 2722 "translate.cpp"
                        var $or307=($184) | 2; //@line 2722 "translate.cpp"
                        $word_flags=$or307; //@line 2722 "translate.cpp"
                        ; //@line 2722 "translate.cpp"
                      }
  
                      var $185=HEAP32[(($prev_out)>>2)]; //@line 2724 "translate.cpp"
                      var $cmp309=((($185))|0)==32; //@line 2724 "translate.cpp"
                      if (!($cmp309)) { __label__ = 118;break $if_then286$$if_end327$150; } //@line 2724 "translate.cpp"
  
                      var $186=$ix; //@line 2724 "translate.cpp"
                      var $sub311=((($186)-2)&4294967295); //@line 2724 "translate.cpp"
                      var $arrayidx312=(($sbuf+$sub311)&4294967295); //@line 2724 "translate.cpp"
                      var $187=HEAP8[($arrayidx312)]; //@line 2724 "translate.cpp"
                      var $conv313=(tempInt=(($187)),(tempInt>=128?tempInt-256:tempInt)); //@line 2724 "translate.cpp"
                      var $call314=_iswdigit($conv313); //@line 2724 "translate.cpp"
                      var $tobool315=((($call314))|0)!=0; //@line 2724 "translate.cpp"
                      if (!($tobool315)) { __label__ = 118;break $if_then286$$if_end327$150; } //@line 2724 "translate.cpp"
  
                      var $188=HEAP32[(($prev_in)>>2)]; //@line 2724 "translate.cpp"
                      var $call317=_iswdigit($188); //@line 2724 "translate.cpp"
                      var $tobool318=((($call317))|0)!=0; //@line 2724 "translate.cpp"
                      if ($tobool318) { __label__ = 118;break $if_then286$$if_end327$150; } //@line 2724 "translate.cpp"
  
                      var $189=$ix; //@line 2728 "translate.cpp"
                      var $inc320=((($189)+1)&4294967295); //@line 2728 "translate.cpp"
                      $ix=$inc320; //@line 2728 "translate.cpp"
                      var $arrayidx321=(($sbuf+$189)&4294967295); //@line 2728 "translate.cpp"
                      HEAP8[($arrayidx321)]=32; //@line 2728 "translate.cpp"
                      var $190=$word_count; //@line 2729 "translate.cpp"
                      var $arrayidx322=(($words+$190*12)&4294967295); //@line 2729 "translate.cpp"
                      var $start323=(($arrayidx322+4)&4294967295); //@line 2729 "translate.cpp"
                      var $191=HEAP16[(($start323)>>1)]; //@line 2729 "translate.cpp"
                      var $inc324=((($191)+1)&65535); //@line 2729 "translate.cpp"
                      HEAP16[(($start323)>>1)]=$inc324; //@line 2729 "translate.cpp"
                      ; //@line 2730 "translate.cpp"
                    }
                  } while(0);
  
                  var $192=$letter_count; //@line 2734 "translate.cpp"
                  var $inc328=((($192)+1)&4294967295); //@line 2734 "translate.cpp"
                  $letter_count=$inc328; //@line 2734 "translate.cpp"
                  var $193=$c; //@line 2736 "translate.cpp"
                  var $call329=_iswupper($193); //@line 2736 "translate.cpp"
                  var $tobool330=((($call329))|0)!=0; //@line 2736 "translate.cpp"
                  if ($tobool330) { __label__ = 119;; } else { __label__ = 130;; } //@line 2736 "translate.cpp"
                  if (__label__ == 119) {
  
                    var $194=$c; //@line 2738 "translate.cpp"
                    var $call332=__Z9towlower2j($194); //@line 2738 "translate.cpp"
                    $c=$call332; //@line 2738 "translate.cpp"
                    var $195=$tr_addr; //@line 2740 "translate.cpp"
                    var $langopts333=(($195)&4294967295); //@line 2740 "translate.cpp"
                    var $param=(($langopts333+24)&4294967295); //@line 2740 "translate.cpp"
                    var $arrayidx334=(($param+52)&4294967295); //@line 2740 "translate.cpp"
                    var $196=HEAP32[(($arrayidx334)>>2)]; //@line 2740 "translate.cpp"
                    $j=$196; //@line 2740 "translate.cpp"
                    var $cmp335=((($196))|0) > 0; //@line 2740 "translate.cpp"
                    if ($cmp335) { __label__ = 120;; } else { __label__ = 123;; } //@line 2740 "translate.cpp"
                    if (__label__ == 120) {
  
                      var $197=$j; //@line 2742 "translate.cpp"
                      var $cmp337=((($197))|0)==2; //@line 2742 "translate.cpp"
                      if (!($cmp337)) { __label__ = 195;break $if_then155$$if_else162$86; } //@line 2742 "translate.cpp"
  
                      var $198=$syllable_marked; //@line 2742 "translate.cpp"
                      var $cmp339=((($198))|0)==0; //@line 2742 "translate.cpp"
                      if (!($cmp339)) { __label__ = 195;break $if_then155$$if_else162$86; } //@line 2742 "translate.cpp"
  
                      var $199=$c; //@line 2744 "translate.cpp"
                      HEAP32[(($char_inserted)>>2)]=$199; //@line 2744 "translate.cpp"
                      $c=712; //@line 2745 "translate.cpp"
                      $syllable_marked=1; //@line 2746 "translate.cpp"
                      ; //@line 2747 "translate.cpp"
                    }
                    else if (__label__ == 123) {
  
                      var $200=HEAP32[(($prev_in)>>2)]; //@line 2751 "translate.cpp"
                      var $call343=_iswlower($200); //@line 2751 "translate.cpp"
                      var $tobool344=((($call343))|0)!=0; //@line 2751 "translate.cpp"
                      if ($tobool344) { __label__ = 124;; } else { __label__ = 125;; } //@line 2751 "translate.cpp"
                      if (__label__ == 124) {
  
                        $c=32; //@line 2753 "translate.cpp"
                        $space_inserted=1; //@line 2754 "translate.cpp"
                        var $201=$c; //@line 2755 "translate.cpp"
                        $prev_in_save=$201; //@line 2755 "translate.cpp"
                        ; //@line 2757 "translate.cpp"
                      }
                      else if (__label__ == 125) {
  
                        var $202=$c; //@line 2759 "translate.cpp"
                        var $cmp347=((($202))|0)!=32; //@line 2759 "translate.cpp"
                        if (!($cmp347)) { __label__ = 195;break $if_then155$$if_else162$86; } //@line 2759 "translate.cpp"
  
                        var $203=HEAP32[(($prev_in)>>2)]; //@line 2759 "translate.cpp"
                        var $call349=_iswupper($203); //@line 2759 "translate.cpp"
                        var $tobool350=((($call349))|0)!=0; //@line 2759 "translate.cpp"
                        if (!($tobool350)) { __label__ = 195;break $if_then155$$if_else162$86; } //@line 2759 "translate.cpp"
  
                        var $204=HEAP32[(($next_in)>>2)]; //@line 2759 "translate.cpp"
                        var $call352=_iswlower($204); //@line 2759 "translate.cpp"
                        var $tobool353=((($call352))|0)!=0; //@line 2759 "translate.cpp"
                        if (!($tobool353)) { __label__ = 195;break $if_then155$$if_else162$86; } //@line 2759 "translate.cpp"
  
                        var $205=HEAP32[(($source_index)>>2)]; //@line 2762 "translate.cpp"
                        var $206=$next_in_nbytes; //@line 2762 "translate.cpp"
                        var $add355=((($206)+($205))&4294967295); //@line 2762 "translate.cpp"
                        var $arrayidx356=((__ZL6source+$add355)&4294967295); //@line 2762 "translate.cpp"
                        var $call357=__Z7utf8_inPiPKc($next2_in, $arrayidx356); //@line 2762 "translate.cpp"
                        var $207=HEAP32[(($next2_in)>>2)]; //@line 2763 "translate.cpp"
                        var $call358=__Z7IsAlphaj($207); //@line 2763 "translate.cpp"
                        var $tobool359=((($call358))|0)!=0; //@line 2763 "translate.cpp"
                        if (!($tobool359)) { __label__ = 195;break $if_then155$$if_else162$86; } //@line 2763 "translate.cpp"
  
                        $c=32; //@line 2766 "translate.cpp"
                        $space_inserted=1; //@line 2767 "translate.cpp"
                        var $208=$c; //@line 2768 "translate.cpp"
                        $prev_in_save=$208; //@line 2768 "translate.cpp"
                        var $209=$next_word_flags; //@line 2769 "translate.cpp"
                        var $or361=($209) | 256; //@line 2769 "translate.cpp"
                        $next_word_flags=$or361; //@line 2769 "translate.cpp"
                        ; //@line 2770 "translate.cpp"
                      }
                    }
                  }
                  else if (__label__ == 130) {
  
                    var $210=$all_upper_case; //@line 2776 "translate.cpp"
                    var $tobool367=((($210))|0)!=0; //@line 2776 "translate.cpp"
                    if ($tobool367) { __label__ = 131;; } else { __label__ = 137;; } //@line 2776 "translate.cpp"
                    $land_lhs_true368$$if_else386$182: do { 
                      if (__label__ == 131) {
  
                        var $211=$letter_count; //@line 2776 "translate.cpp"
                        var $cmp369=((($211))|0) > 2; //@line 2776 "translate.cpp"
                        if (!($cmp369)) { __label__ = 137;break $land_lhs_true368$$if_else386$182; } //@line 2776 "translate.cpp"
  
                        var $212=$c; //@line 2778 "translate.cpp"
                        var $cmp371=((($212))|0)==115; //@line 2778 "translate.cpp"
                        if ($cmp371) { __label__ = 133;; } else { __label__ = 136;; } //@line 2778 "translate.cpp"
                        $land_lhs_true372$$if_else384$185: do { 
                          if (__label__ == 133) {
  
                            var $213=HEAP32[(($next_in)>>2)]; //@line 2778 "translate.cpp"
                            var $cmp373=((($213))|0)==32; //@line 2778 "translate.cpp"
                            if (!($cmp373)) { __label__ = 136;break $land_lhs_true372$$if_else384$185; } //@line 2778 "translate.cpp"
  
                            $c=32; //@line 2780 "translate.cpp"
                            var $214=$all_upper_case; //@line 2781 "translate.cpp"
                            var $or375=($214) | 4; //@line 2781 "translate.cpp"
                            $all_upper_case=$or375; //@line 2781 "translate.cpp"
                            var $215=$ix; //@line 2783 "translate.cpp"
                            var $sub376=((($215)-1)&4294967295); //@line 2783 "translate.cpp"
                            var $arrayidx377=(($sbuf+$sub376)&4294967295); //@line 2783 "translate.cpp"
                            var $216=HEAP8[($arrayidx377)]; //@line 2783 "translate.cpp"
                            var $conv378=(tempInt=(($216)),(tempInt>=128?tempInt-256:tempInt)); //@line 2783 "translate.cpp"
                            var $cmp379=((($conv378))|0)==39; //@line 2783 "translate.cpp"
                            if (!($cmp379)) { __label__ = 195;break $if_then155$$if_else162$86; } //@line 2783 "translate.cpp"
  
                            var $217=$ix; //@line 2784 "translate.cpp"
                            var $sub381=((($217)-1)&4294967295); //@line 2784 "translate.cpp"
                            var $arrayidx382=(($sbuf+$sub381)&4294967295); //@line 2784 "translate.cpp"
                            HEAP8[($arrayidx382)]=32; //@line 2784 "translate.cpp"
                            __label__ = 195;break $if_then155$$if_else162$86; //@line 2784 "translate.cpp"
                          }
                        } while(0);
  
                        $all_upper_case=0; //@line 2787 "translate.cpp"
                        __label__ = 195;break $if_then155$$if_else162$86;
                      }
                    } while(0);
  
                    $all_upper_case=0; //@line 2790 "translate.cpp"
                    ;
                  }
                }
                else if (__label__ == 138) {
  
                  var $218=$c; //@line 2794 "translate.cpp"
                  var $cmp390=((($218))|0)==45; //@line 2794 "translate.cpp"
                  if ($cmp390) { __label__ = 139;; } else { __label__ = 154;; } //@line 2794 "translate.cpp"
                  if (__label__ == 139) {
  
                    var $219=HEAP32[(($prev_in)>>2)]; //@line 2796 "translate.cpp"
                    var $call392=__Z7IsSpacej($219); //@line 2796 "translate.cpp"
                    var $tobool393=((($call392))|0)!=0; //@line 2796 "translate.cpp"
                    if ($tobool393) { __label__ = 146;; } else { __label__ = 140;; } //@line 2796 "translate.cpp"
                    $if_else410$$land_lhs_true394$194: do { 
                      if (__label__ == 140) {
  
                        var $220=HEAP32[(($next_in)>>2)]; //@line 2796 "translate.cpp"
                        var $call395=__Z7IsAlphaj($220); //@line 2796 "translate.cpp"
                        var $tobool396=((($call395))|0)!=0; //@line 2796 "translate.cpp"
                        if (!($tobool396)) { __label__ = 146;break $if_else410$$land_lhs_true394$194; } //@line 2796 "translate.cpp"
  
                        var $221=HEAP32[(($prev_out)>>2)]; //@line 2798 "translate.cpp"
                        var $cmp398=((($221))|0)!=32; //@line 2798 "translate.cpp"
                        if ($cmp398) { __label__ = 142;; } else { __label__ = 143;; } //@line 2798 "translate.cpp"
                        if (__label__ == 142) {
  
                          $c=32; //@line 2801 "translate.cpp"
                          $space_inserted=1; //@line 2802 "translate.cpp"
                          __label__ = 195;break $if_then155$$if_else162$86; //@line 2803 "translate.cpp"
                        }
                        else if (__label__ == 143) {
  
                          var $222=$word_flags; //@line 2807 "translate.cpp"
                          var $or401=($222) | 128; //@line 2807 "translate.cpp"
                          $word_flags=$or401; //@line 2807 "translate.cpp"
                          var $223=$word_count; //@line 2808 "translate.cpp"
                          var $cmp402=((($223))|0) > 0; //@line 2808 "translate.cpp"
                          if ($cmp402) { __label__ = 144;; } else { __label__ = 145;; } //@line 2808 "translate.cpp"
                          if (__label__ == 144) {
  
                            var $224=$word_count; //@line 2809 "translate.cpp"
                            var $sub404=((($224)-1)&4294967295); //@line 2809 "translate.cpp"
                            var $arrayidx405=(($words+$sub404*12)&4294967295); //@line 2809 "translate.cpp"
                            var $flags406=(($arrayidx405)&4294967295); //@line 2809 "translate.cpp"
                            var $225=HEAP32[(($flags406)>>2)]; //@line 2809 "translate.cpp"
                            var $or407=($225) | 16384; //@line 2809 "translate.cpp"
                            HEAP32[(($flags406)>>2)]=$or407; //@line 2809 "translate.cpp"
                            ; //@line 2809 "translate.cpp"
                          }
  
                          $c=32; //@line 2810 "translate.cpp"
                          __label__ = 195;break $if_then155$$if_else162$86;
                        }
                      }
                    } while(0);
  
                    var $226=HEAP32[(($prev_in)>>2)]; //@line 2814 "translate.cpp"
                    var $cmp411=((($226))|0)==32; //@line 2814 "translate.cpp"
                    var $227=HEAP32[(($next_in)>>2)]; //@line 2814 "translate.cpp"
                    var $cmp413=((($227))|0)==32; //@line 2814 "translate.cpp"
                    var $or_cond26=($cmp411) & ($cmp413); //@line 2814 "translate.cpp"
                    if ($or_cond26) { __label__ = 147;; } else { __label__ = 148;; } //@line 2814 "translate.cpp"
                    if (__label__ == 147) {
  
                      $c=32; //@line 2817 "translate.cpp"
                      $pre_pause_add=4; //@line 2818 "translate.cpp"
                      ; //@line 2819 "translate.cpp"
                    }
                    else if (__label__ == 148) {
  
                      var $cmp416=((($227))|0)==45; //@line 2821 "translate.cpp"
                      if ($cmp416) { __label__ = 149;; } else { __label__ = 150;; } //@line 2821 "translate.cpp"
                      if (__label__ == 149) {
  
                        var $228=HEAP32[(($source_index)>>2)]; //@line 2824 "translate.cpp"
                        var $inc418=((($228)+1)&4294967295); //@line 2824 "translate.cpp"
                        HEAP32[(($source_index)>>2)]=$inc418; //@line 2824 "translate.cpp"
                        $c=32; //@line 2825 "translate.cpp"
                        $pre_pause_add=4; //@line 2826 "translate.cpp"
                        ; //@line 2827 "translate.cpp"
                      }
                      else if (__label__ == 150) {
  
                        var $229=HEAP32[(($prev_out)>>2)]; //@line 2829 "translate.cpp"
                        var $cmp420=((($229))|0)==32; //@line 2829 "translate.cpp"
                        if (!($cmp420)) { __label__ = 195;break $if_then155$$if_else162$86; } //@line 2829 "translate.cpp"
  
                        var $230=$ix; //@line 2829 "translate.cpp"
                        var $sub422=((($230)-2)&4294967295); //@line 2829 "translate.cpp"
                        var $arrayidx423=(($sbuf+$sub422)&4294967295); //@line 2829 "translate.cpp"
                        var $231=HEAP8[($arrayidx423)]; //@line 2829 "translate.cpp"
                        var $conv424=(tempInt=(($231)),(tempInt>=128?tempInt-256:tempInt)); //@line 2829 "translate.cpp"
                        var $call425=__Z7IsAlphaj($conv424); //@line 2829 "translate.cpp"
                        var $tobool426=((($call425))|0)!=0; //@line 2829 "translate.cpp"
                        if (!($tobool426)) { __label__ = 195;break $if_then155$$if_else162$86; } //@line 2829 "translate.cpp"
  
                        var $232=HEAP32[(($prev_in)>>2)]; //@line 2829 "translate.cpp"
                        var $call428=__Z7IsAlphaj($232); //@line 2829 "translate.cpp"
                        var $tobool429=((($call428))|0)!=0; //@line 2829 "translate.cpp"
                        if ($tobool429) { __label__ = 195;break $if_then155$$if_else162$86; } //@line 2829 "translate.cpp"
  
                        var $233=$ix; //@line 2832 "translate.cpp"
                        var $inc431=((($233)+1)&4294967295); //@line 2832 "translate.cpp"
                        $ix=$inc431; //@line 2832 "translate.cpp"
                        var $arrayidx432=(($sbuf+$233)&4294967295); //@line 2832 "translate.cpp"
                        HEAP8[($arrayidx432)]=32; //@line 2832 "translate.cpp"
                        var $234=$word_count; //@line 2833 "translate.cpp"
                        var $arrayidx433=(($words+$234*12)&4294967295); //@line 2833 "translate.cpp"
                        var $start434=(($arrayidx433+4)&4294967295); //@line 2833 "translate.cpp"
                        var $235=HEAP16[(($start434)>>1)]; //@line 2833 "translate.cpp"
                        var $inc435=((($235)+1)&65535); //@line 2833 "translate.cpp"
                        HEAP16[(($start434)>>1)]=$inc435; //@line 2833 "translate.cpp"
                        ; //@line 2834 "translate.cpp"
                      }
                    }
                  }
                  else if (__label__ == 154) {
  
                    var $236=$c; //@line 2837 "translate.cpp"
                    var $cmp441=((($236))|0)==46; //@line 2837 "translate.cpp"
                    if ($cmp441) { __label__ = 155;; } else { __label__ = 162;; } //@line 2837 "translate.cpp"
                    if (__label__ == 155) {
  
                      var $237=HEAP32[(($prev_out)>>2)]; //@line 2839 "translate.cpp"
                      var $cmp443=((($237))|0)==46; //@line 2839 "translate.cpp"
                      if ($cmp443) { __label__ = 156;; } else { __label__ = 157;; } //@line 2839 "translate.cpp"
                      if (__label__ == 156) {
  
                        $c=32; //@line 2842 "translate.cpp"
                        $space_inserted=1; //@line 2843 "translate.cpp"
                        ; //@line 2844 "translate.cpp"
                      }
                      else if (__label__ == 157) {
  
                        var $238=$word_count; //@line 2846 "translate.cpp"
                        var $cmp446=((($238))|0) > 0; //@line 2846 "translate.cpp"
                        if (!($cmp446)) { __label__ = 195;break $if_then155$$if_else162$86; } //@line 2846 "translate.cpp"
  
                        var $239=$word_count; //@line 2846 "translate.cpp"
                        var $sub448=((($239)-1)&4294967295); //@line 2846 "translate.cpp"
                        var $arrayidx449=(($words+$sub448*12)&4294967295); //@line 2846 "translate.cpp"
                        var $flags450=(($arrayidx449)&4294967295); //@line 2846 "translate.cpp"
                        var $240=HEAP32[(($flags450)>>2)]; //@line 2846 "translate.cpp"
                        var $and451=($240) & 256; //@line 2846 "translate.cpp"
                        var $tobool452=((($and451))|0)!=0; //@line 2846 "translate.cpp"
                        if ($tobool452) { __label__ = 195;break $if_then155$$if_else162$86; } //@line 2846 "translate.cpp"
  
                        var $241=HEAP32[(($prev_in)>>2)]; //@line 2846 "translate.cpp"
                        var $call454=__Z7IsAlphaj($241); //@line 2846 "translate.cpp"
                        var $tobool455=((($call454))|0)!=0; //@line 2846 "translate.cpp"
                        if (!($tobool455)) { __label__ = 195;break $if_then155$$if_else162$86; } //@line 2846 "translate.cpp"
  
                        var $242=$word_count; //@line 2849 "translate.cpp"
                        var $sub457=((($242)-1)&4294967295); //@line 2849 "translate.cpp"
                        var $arrayidx458=(($words+$sub457*12)&4294967295); //@line 2849 "translate.cpp"
                        var $flags459=(($arrayidx458)&4294967295); //@line 2849 "translate.cpp"
                        var $243=HEAP32[(($flags459)>>2)]; //@line 2849 "translate.cpp"
                        var $or460=($243) | 65536; //@line 2849 "translate.cpp"
                        HEAP32[(($flags459)>>2)]=$or460; //@line 2849 "translate.cpp"
                        var $244=HEAP32[(($next_in)>>2)]; //@line 2851 "translate.cpp"
                        var $call461=__Z7IsSpacej($244); //@line 2851 "translate.cpp"
                        var $tobool462=((($call461))|0)!=0; //@line 2851 "translate.cpp"
                        var $245=HEAP32[(($next_in)>>2)]; //@line 2851 "translate.cpp"
                        var $cmp464=((($245))|0)==45; //@line 2851 "translate.cpp"
                        var $or_cond9=($tobool462) | ($cmp464); //@line 2851 "translate.cpp"
                        if (!($or_cond9)) { __label__ = 195;break $if_then155$$if_else162$86; } //@line 2851 "translate.cpp"
  
                        $c=32; //@line 2852 "translate.cpp"
                        ; //@line 2852 "translate.cpp"
                      }
                    }
                    else if (__label__ == 162) {
  
                      var $246=$c; //@line 2856 "translate.cpp"
                      var $cmp470=((($246))|0)==39; //@line 2856 "translate.cpp"
                      if ($cmp470) { __label__ = 163;; } else { __label__ = 178;; } //@line 2856 "translate.cpp"
                      if (__label__ == 163) {
  
                        var $247=HEAP32[(($prev_in)>>2)]; //@line 2858 "translate.cpp"
                        var $cmp472=((($247))|0)==46; //@line 2858 "translate.cpp"
                        if ($cmp472) { __label__ = 165;; } else { __label__ = 164;; } //@line 2858 "translate.cpp"
                        $land_lhs_true476$$lor_lhs_false473$226: do { 
                          if (__label__ == 164) {
  
                            var $248=HEAP32[(($prev_in)>>2)]; //@line 2858 "translate.cpp"
                            var $call474=_iswalnum($248); //@line 2858 "translate.cpp"
                            var $tobool475=((($call474))|0)!=0; //@line 2858 "translate.cpp"
                            if ($tobool475) { __label__ = 165;break $land_lhs_true476$$lor_lhs_false473$226; } else { __label__ = 167;break $land_lhs_true476$$lor_lhs_false473$226; } //@line 2858 "translate.cpp"
                          }
                        } while(0);
                        $land_lhs_true476$$if_else480$228: do { 
                          if (__label__ == 165) {
  
                            var $249=HEAP32[(($next_in)>>2)]; //@line 2858 "translate.cpp"
                            var $call477=__Z7IsAlphaj($249); //@line 2858 "translate.cpp"
                            var $tobool478=((($call477))|0)!=0; //@line 2858 "translate.cpp"
                            if (!($tobool478)) { __label__ = 167;break $land_lhs_true476$$if_else480$228; } //@line 2858 "translate.cpp"
  
                            $single_quoted=0; //@line 2861 "translate.cpp"
                            __label__ = 195;break $if_then155$$if_else162$86; //@line 2862 "translate.cpp"
                          }
                        } while(0);
  
                        var $250=$tr_addr; //@line 2864 "translate.cpp"
                        var $char_plus_apostrophe=(($250+700)&4294967295); //@line 2864 "translate.cpp"
                        var $251=HEAP32[(($char_plus_apostrophe)>>2)]; //@line 2864 "translate.cpp"
                        var $252=HEAP32[(($prev_in)>>2)]; //@line 2864 "translate.cpp"
                        var $call481=_wcschr($251, $252); //@line 2864 "translate.cpp"
                        var $cmp482=((($call481))|0)!=0; //@line 2864 "translate.cpp"
                        if ($cmp482) { __label__ = 168;; } else { __label__ = 171;; } //@line 2864 "translate.cpp"
                        $land_lhs_true483$$if_else490$232: do { 
                          if (__label__ == 168) {
  
                            var $253=$prev_out2; //@line 2864 "translate.cpp"
                            var $cmp484=((($253))|0)==32; //@line 2864 "translate.cpp"
                            if (!($cmp484)) { __label__ = 171;break $land_lhs_true483$$if_else490$232; } //@line 2864 "translate.cpp"
  
                            $single_quoted=0; //@line 2867 "translate.cpp"
                            var $254=HEAP32[(($next_in)>>2)]; //@line 2868 "translate.cpp"
                            var $cmp486=((($254))|0)==32; //@line 2868 "translate.cpp"
                            if (!($cmp486)) { __label__ = 195;break $if_then155$$if_else162$86; } //@line 2868 "translate.cpp"
  
                            var $255=HEAP32[(($source_index)>>2)]; //@line 2870 "translate.cpp"
                            var $inc488=((($255)+1)&4294967295); //@line 2870 "translate.cpp"
                            HEAP32[(($source_index)>>2)]=$inc488; //@line 2870 "translate.cpp"
                            __label__ = 195;break $if_then155$$if_else162$86; //@line 2871 "translate.cpp"
                          }
                        } while(0);
  
                        var $256=HEAP32[(($prev_out)>>2)]; //@line 2875 "translate.cpp"
                        var $cmp491=((($256))|0)==115; //@line 2875 "translate.cpp"
                        if ($cmp491) { __label__ = 172;; } else { __label__ = 174;; } //@line 2875 "translate.cpp"
                        $land_lhs_true492$$if_else495$237: do { 
                          if (__label__ == 172) {
  
                            var $257=$single_quoted; //@line 2875 "translate.cpp"
                            var $cmp493=((($257))|0)==0; //@line 2875 "translate.cpp"
                            if (!($cmp493)) { __label__ = 174;break $land_lhs_true492$$if_else495$237; } //@line 2875 "translate.cpp"
  
                            $c=32; //@line 2878 "translate.cpp"
                            __label__ = 195;break $if_then155$$if_else162$86; //@line 2879 "translate.cpp"
                          }
                        } while(0);
  
                        var $258=HEAP32[(($prev_out)>>2)]; //@line 2882 "translate.cpp"
                        var $call496=__Z7IsSpacej($258); //@line 2882 "translate.cpp"
                        var $tobool497=((($call496))|0)!=0; //@line 2882 "translate.cpp"
                        if ($tobool497) { __label__ = 175;; } else { __label__ = 176;; } //@line 2882 "translate.cpp"
                        if (__label__ == 175) {
  
                          $single_quoted=1; //@line 2883 "translate.cpp"
                          ; //@line 2883 "translate.cpp"
                        }
                        else if (__label__ == 176) {
  
                          $single_quoted=0; //@line 2885 "translate.cpp"
                          ;
                        }
  
                        $pre_pause_add=4; //@line 2887 "translate.cpp"
                        $c=32; //@line 2888 "translate.cpp"
                        ;
                      }
                      else if (__label__ == 178) {
  
                        var $259=$c; //@line 2902 "translate.cpp"
                        var $call505=__Z11lookupwcharPKti(((__ZL6breaks)&4294967295), $259); //@line 2902 "translate.cpp"
                        var $cmp506=((($call505))|0)!=0; //@line 2902 "translate.cpp"
                        if ($cmp506) { __label__ = 179;; } else { __label__ = 180;; } //@line 2902 "translate.cpp"
                        if (__label__ == 179) {
  
                          $c=32; //@line 2904 "translate.cpp"
                          ; //@line 2905 "translate.cpp"
                        }
                        else if (__label__ == 180) {
  
                          var $260=$c; //@line 2907 "translate.cpp"
                          var $call509=_iswdigit($260); //@line 2907 "translate.cpp"
                          var $tobool510=((($call509))|0)!=0; //@line 2907 "translate.cpp"
                          if (!($tobool510)) { __label__ = 195;break $if_then155$$if_else162$86; } //@line 2907 "translate.cpp"
  
                          var $261=$tr_addr; //@line 2909 "translate.cpp"
                          var $langopts512=(($261)&4294967295); //@line 2909 "translate.cpp"
                          var $tone_numbers513=(($langopts512+250)&4294967295); //@line 2909 "translate.cpp"
                          var $262=HEAP8[($tone_numbers513)]; //@line 2909 "translate.cpp"
                          var $tobool514=(tempInt=(($262)),(tempInt>=128?tempInt-256:tempInt))!=0; //@line 2909 "translate.cpp"
                          if ($tobool514) { __label__ = 182;; } else { __label__ = 184;; } //@line 2909 "translate.cpp"
                          $land_lhs_true515$$if_else522$250: do { 
                            if (__label__ == 182) {
  
                              var $263=HEAP32[(($prev_out)>>2)]; //@line 2909 "translate.cpp"
                              var $call516=__Z7IsAlphaj($263); //@line 2909 "translate.cpp"
                              var $tobool517=((($call516))|0)!=0; //@line 2909 "translate.cpp"
                              if (!($tobool517)) { __label__ = 184;break $land_lhs_true515$$if_else522$250; } //@line 2909 "translate.cpp"
  
                              var $264=HEAP32[(($next_in)>>2)]; //@line 2909 "translate.cpp"
                              var $call519=__Z7IsDigitj($264); //@line 2909 "translate.cpp"
                              var $tobool520=((($call519))|0)!=0; //@line 2909 "translate.cpp"
                              if (!($tobool520)) { __label__ = 195;break $if_then155$$if_else162$86; } //@line 2909 "translate.cpp"
                            }
                          } while(0);
  
                          var $265=HEAP32[(($prev_out)>>2)]; //@line 2913 "translate.cpp"
                          var $cmp523=((($265))|0)!=32; //@line 2913 "translate.cpp"
                          if ($cmp523) { __label__ = 185;; } else { __label__ = 192;; } //@line 2913 "translate.cpp"
                          if (__label__ == 185) {
  
                            var $266=HEAP32[(($prev_out)>>2)]; //@line 2913 "translate.cpp"
                            var $call525=_iswdigit($266); //@line 2913 "translate.cpp"
                            var $tobool526=((($call525))|0)!=0; //@line 2913 "translate.cpp"
                            var $_pr20=HEAP32[(($prev_out)>>2)]; //@line 2926 "translate.cpp"
                            if ($tobool526) { __label__ = 191;; } else { __label__ = 186;; } //@line 2913 "translate.cpp"
                            if (__label__ == 191) {
  
                              var $cmp540=((($_pr20))|0)==32; //@line 2926 "translate.cpp"
                              if (!($cmp540)) { __label__ = 195;break $if_then155$$if_else162$86; } //@line 2926 "translate.cpp"
                            }
                            else if (__label__ == 186) {
  
                              var $267=$tr_addr; //@line 2915 "translate.cpp"
                              var $langopts528=(($267)&4294967295); //@line 2915 "translate.cpp"
                              var $decimal_sep=(($langopts528+208)&4294967295); //@line 2915 "translate.cpp"
                              var $268=HEAP32[(($decimal_sep)>>2)]; //@line 2915 "translate.cpp"
                              var $cmp529=((($_pr20))|0)!=((($268))|0); //@line 2915 "translate.cpp"
                              if ($cmp529) { __label__ = 189;; } else { __label__ = 187;; } //@line 2915 "translate.cpp"
                              $if_then536$$lor_lhs_false530$259: do { 
                                if (__label__ == 187) {
  
                                  var $269=$decimal_sep_count; //@line 2915 "translate.cpp"
                                  var $cmp531=((($269))|0) > 0; //@line 2915 "translate.cpp"
                                  if ($cmp531) { __label__ = 188;; } else { __label__ = 190;; } //@line 2915 "translate.cpp"
                                  if (__label__ == 188) {
  
                                    var $270=$tr_addr; //@line 2915 "translate.cpp"
                                    var $langopts533=(($270)&4294967295); //@line 2915 "translate.cpp"
                                    var $decimal_sep534=(($langopts533+208)&4294967295); //@line 2915 "translate.cpp"
                                    var $271=HEAP32[(($decimal_sep534)>>2)]; //@line 2915 "translate.cpp"
                                    var $cmp535=((($271))|0)==44; //@line 2915 "translate.cpp"
                                    if ($cmp535) { __label__ = 189;break $if_then536$$lor_lhs_false530$259; } //@line 2915 "translate.cpp"
                                  }
  
                                  $decimal_sep_count=1; //@line 2922 "translate.cpp"
                                  __label__ = 195;break $if_then155$$if_else162$86;
                                }
                              } while(0);
  
                              $c=32; //@line 2917 "translate.cpp"
                              $space_inserted=1; //@line 2918 "translate.cpp"
                              __label__ = 195;break $if_then155$$if_else162$86; //@line 2919 "translate.cpp"
                            }
                          }
  
                          var $272=$ix; //@line 2926 "translate.cpp"
                          var $sub542=((($272)-2)&4294967295); //@line 2926 "translate.cpp"
                          var $arrayidx543=(($sbuf+$sub542)&4294967295); //@line 2926 "translate.cpp"
                          var $273=HEAP8[($arrayidx543)]; //@line 2926 "translate.cpp"
                          var $conv544=(tempInt=(($273)),(tempInt>=128?tempInt-256:tempInt)); //@line 2926 "translate.cpp"
                          var $call545=__Z7IsAlphaj($conv544); //@line 2926 "translate.cpp"
                          var $tobool546=((($call545))|0)!=0; //@line 2926 "translate.cpp"
                          if (!($tobool546)) { __label__ = 195;break $if_then155$$if_else162$86; } //@line 2926 "translate.cpp"
  
                          var $274=HEAP32[(($prev_in)>>2)]; //@line 2926 "translate.cpp"
                          var $call548=__Z7IsAlphaj($274); //@line 2926 "translate.cpp"
                          var $tobool549=((($call548))|0)!=0; //@line 2926 "translate.cpp"
                          if ($tobool549) { __label__ = 195;break $if_then155$$if_else162$86; } //@line 2926 "translate.cpp"
  
                          var $275=$ix; //@line 2929 "translate.cpp"
                          var $inc551=((($275)+1)&4294967295); //@line 2929 "translate.cpp"
                          $ix=$inc551; //@line 2929 "translate.cpp"
                          var $arrayidx552=(($sbuf+$275)&4294967295); //@line 2929 "translate.cpp"
                          HEAP8[($arrayidx552)]=32; //@line 2929 "translate.cpp"
                          var $276=$word_count; //@line 2930 "translate.cpp"
                          var $arrayidx553=(($words+$276*12)&4294967295); //@line 2930 "translate.cpp"
                          var $start554=(($arrayidx553+4)&4294967295); //@line 2930 "translate.cpp"
                          var $277=HEAP16[(($start554)>>1)]; //@line 2930 "translate.cpp"
                          var $inc555=((($277)+1)&65535); //@line 2930 "translate.cpp"
                          HEAP16[(($start554)>>1)]=$inc555; //@line 2930 "translate.cpp"
                          ; //@line 2931 "translate.cpp"
                        }
                      }
                    }
                  }
                }
              }
            }
          } while(0);
  
          var $278=$c; //@line 2935 "translate.cpp"
          var $call568=__Z7IsSpacej($278); //@line 2935 "translate.cpp"
          var $tobool569=((($call568))|0)!=0; //@line 2935 "translate.cpp"
          if ($tobool569) { __label__ = 196;; } else { __label__ = 225;; } //@line 2935 "translate.cpp"
          $if_then570$$if_else688$269: do { 
            if (__label__ == 196) {
  
              var $279=HEAP32[(($prev_out)>>2)]; //@line 2937 "translate.cpp"
              var $cmp571=((($279))|0)==32; //@line 2937 "translate.cpp"
              if ($cmp571) { __label__ = 197;; } else { __label__ = 198;; } //@line 2937 "translate.cpp"
              if (__label__ == 197) {
  
                var $280=$word_flags; //@line 2939 "translate.cpp"
                var $or573=($280) | 262144; //@line 2939 "translate.cpp"
                $word_flags=$or573; //@line 2939 "translate.cpp"
                __label__ = 33;continue $while_cond88$50; //@line 2940 "translate.cpp"
              }
              else if (__label__ == 198) {
  
                var $281=HEAP32[(($cc)>>2)]; //@line 2943 "translate.cpp"
                var $cmp575=((($281))|0)==9; //@line 2943 "translate.cpp"
                var $282=HEAP32[(($cc)>>2)]; //@line 2943 "translate.cpp"
                var $cmp577=((($282))|0)==10; //@line 2943 "translate.cpp"
                var $or_cond10=($cmp575) | ($cmp577); //@line 2943 "translate.cpp"
                if ($or_cond10) { __label__ = 199;; } else { __label__ = 200;; } //@line 2943 "translate.cpp"
                if (__label__ == 199) {
  
                  var $283=$next_word_flags; //@line 2945 "translate.cpp"
                  var $or579=($283) | 262144; //@line 2945 "translate.cpp"
                  $next_word_flags=$or579; //@line 2945 "translate.cpp"
                  ; //@line 2946 "translate.cpp"
                }
  
                var $284=$space_inserted; //@line 2948 "translate.cpp"
                var $tobool581=((($284))|0)!=0; //@line 2948 "translate.cpp"
                if ($tobool581) { __label__ = 201;; } else { __label__ = 208;; } //@line 2948 "translate.cpp"
                if (__label__ == 201) {
  
                  $j=0; //@line 2951 "translate.cpp"
                  var $285=HEAP32[(($source_index)>>2)]; //@line 2952 "translate.cpp"
                  var $sub583=((($285)-1)&4294967295); //@line 2952 "translate.cpp"
                  $k=$sub583; //@line 2952 "translate.cpp"
                  ; //@line 2953 "translate.cpp"
                  while(1) { 
  
                    var $286=$k; //@line 2953 "translate.cpp"
                    var $287=$source_index_word; //@line 2953 "translate.cpp"
                    var $cmp585=((($286))|0) >= ((($287))|0); //@line 2953 "translate.cpp"
                    if (!($cmp585)) { __label__ = 207;break ; } //@line 2953 "translate.cpp"
  
                    var $288=$k; //@line 2953 "translate.cpp"
                    var $arrayidx587=(($charix+$288*2)&4294967295); //@line 2953 "translate.cpp"
                    var $289=HEAP16[(($arrayidx587)>>1)]; //@line 2953 "translate.cpp"
                    var $conv588=(tempInt=(($289)),(tempInt>=32768?tempInt-65536:tempInt)); //@line 2953 "translate.cpp"
                    var $cmp589=((($conv588))|0)!=0; //@line 2953 "translate.cpp"
                    if (!($cmp589)) { __label__ = 207;break ; }
  
                    var $290=$k; //@line 2955 "translate.cpp"
                    var $arrayidx592=(($charix+$290*2)&4294967295); //@line 2955 "translate.cpp"
                    var $291=HEAP16[(($arrayidx592)>>1)]; //@line 2955 "translate.cpp"
                    var $conv593=(tempInt=(($291)),(tempInt>=32768?tempInt-65536:tempInt)); //@line 2955 "translate.cpp"
                    var $cmp594=((($conv593))|0) > 0; //@line 2955 "translate.cpp"
                    if ($cmp594) { __label__ = 205;; } else { __label__ = 206;; } //@line 2955 "translate.cpp"
                    if (__label__ == 205) {
  
                      var $292=$j; //@line 2956 "translate.cpp"
                      var $inc596=((($292)+1)&4294967295); //@line 2956 "translate.cpp"
                      $j=$inc596; //@line 2956 "translate.cpp"
                      ; //@line 2956 "translate.cpp"
                    }
  
                    var $293=$k; //@line 2957 "translate.cpp"
                    var $dec598=((($293)-1)&4294967295); //@line 2957 "translate.cpp"
                    $k=$dec598; //@line 2957 "translate.cpp"
                    __label__ = 202;continue ; //@line 2958 "translate.cpp"
                  }
  
                  var $294=$j; //@line 2959 "translate.cpp"
                  var $conv600=((($294)) & 255); //@line 2959 "translate.cpp"
                  var $295=$word_count; //@line 2959 "translate.cpp"
                  var $arrayidx601=(($words+$295*12)&4294967295); //@line 2959 "translate.cpp"
                  var $length602=(($arrayidx601+10)&4294967295); //@line 2959 "translate.cpp"
                  HEAP8[($length602)]=$conv600; //@line 2959 "translate.cpp"
                  ; //@line 2960 "translate.cpp"
                }
  
                var $296=HEAP32[(($source_index)>>2)]; //@line 2962 "translate.cpp"
                $source_index_word=$296; //@line 2962 "translate.cpp"
                var $297=$ix; //@line 2965 "translate.cpp"
                var $inc604=((($297)+1)&4294967295); //@line 2965 "translate.cpp"
                $ix=$inc604; //@line 2965 "translate.cpp"
                var $arrayidx605=(($sbuf+$297)&4294967295); //@line 2965 "translate.cpp"
                HEAP8[($arrayidx605)]=32; //@line 2965 "translate.cpp"
                var $298=$word_count; //@line 2967 "translate.cpp"
                var $cmp606=((($298))|0) < 299; //@line 2967 "translate.cpp"
                if ($cmp606) { __label__ = 209;; } else { __label__ = 223;; } //@line 2967 "translate.cpp"
                $land_lhs_true607$$if_end684$288: do { 
                  if (__label__ == 209) {
  
                    var $299=$ix; //@line 2967 "translate.cpp"
                    var $300=$word_count; //@line 2967 "translate.cpp"
                    var $arrayidx608=(($words+$300*12)&4294967295); //@line 2967 "translate.cpp"
                    var $start609=(($arrayidx608+4)&4294967295); //@line 2967 "translate.cpp"
                    var $301=HEAPU16[(($start609)>>1)]; //@line 2967 "translate.cpp"
                    var $conv610=((($301))&65535); //@line 2967 "translate.cpp"
                    var $cmp611=((($299))|0) > ((($conv610))|0); //@line 2967 "translate.cpp"
                    if (!($cmp611)) { __label__ = 223;break $land_lhs_true607$$if_end684$288; } //@line 2967 "translate.cpp"
  
                    var $302=$embedded_count; //@line 2969 "translate.cpp"
                    var $cmp613=((($302))|0) > 0; //@line 2969 "translate.cpp"
                    if ($cmp613) { __label__ = 211;; } else { __label__ = 212;; } //@line 2969 "translate.cpp"
                    if (__label__ == 211) {
  
                      var $303=HEAP32[((__ZL11embedded_ix)>>2)]; //@line 2972 "translate.cpp"
                      var $sub615=((($303)-1)&4294967295); //@line 2972 "translate.cpp"
                      var $arrayidx616=((_embedded_list+$sub615*4)&4294967295); //@line 2972 "translate.cpp"
                      var $304=HEAP32[(($arrayidx616)>>2)]; //@line 2972 "translate.cpp"
                      var $or617=($304) | 128; //@line 2972 "translate.cpp"
                      HEAP32[(($arrayidx616)>>2)]=$or617; //@line 2972 "translate.cpp"
                      var $305=$word_count; //@line 2973 "translate.cpp"
                      var $arrayidx618=(($words+$305*12)&4294967295); //@line 2973 "translate.cpp"
                      var $flags619=(($arrayidx618)&4294967295); //@line 2973 "translate.cpp"
                      var $306=HEAP32[(($flags619)>>2)]; //@line 2973 "translate.cpp"
                      var $or620=($306) | 64; //@line 2973 "translate.cpp"
                      HEAP32[(($flags619)>>2)]=$or620; //@line 2973 "translate.cpp"
                      $embedded_count=0; //@line 2974 "translate.cpp"
                      ; //@line 2975 "translate.cpp"
                    }
  
                    var $307=HEAP32[((_pre_pause)>>2)]; //@line 2976 "translate.cpp"
                    var $conv622=((($307)) & 255); //@line 2976 "translate.cpp"
                    var $308=$word_count; //@line 2976 "translate.cpp"
                    var $arrayidx623=(($words+$308*12)&4294967295); //@line 2976 "translate.cpp"
                    var $pre_pause=(($arrayidx623+8)&4294967295); //@line 2976 "translate.cpp"
                    HEAP8[($pre_pause)]=$conv622; //@line 2976 "translate.cpp"
                    var $309=$all_upper_case; //@line 2977 "translate.cpp"
                    var $310=$word_flags; //@line 2977 "translate.cpp"
                    var $_b=HEAPU8[(__ZL13word_emphasis_b)];
                    var $311=($_b) ? 2048 : 0;
                    var $312=$word_count; //@line 2977 "translate.cpp"
                    var $arrayidx626=(($words+$312*12)&4294967295); //@line 2977 "translate.cpp"
                    var $flags627=(($arrayidx626)&4294967295); //@line 2977 "translate.cpp"
                    var $313=HEAP32[(($flags627)>>2)]; //@line 2977 "translate.cpp"
                    var $or624=($310) | ($309); //@line 2977 "translate.cpp"
                    var $or625=($or624) | ($311); //@line 2977 "translate.cpp"
                    var $or628=($or625) | ($313); //@line 2977 "translate.cpp"
                    HEAP32[(($flags627)>>2)]=$or628; //@line 2977 "translate.cpp"
                    var $314=$word_mark; //@line 2978 "translate.cpp"
                    var $conv629=((($314)) & 255); //@line 2978 "translate.cpp"
                    var $315=$word_count; //@line 2978 "translate.cpp"
                    var $arrayidx630=(($words+$315*12)&4294967295); //@line 2978 "translate.cpp"
                    var $wmark=(($arrayidx630+9)&4294967295); //@line 2978 "translate.cpp"
                    HEAP8[($wmark)]=$conv629; //@line 2978 "translate.cpp"
                    var $316=HEAP32[((_pre_pause)>>2)]; //@line 2980 "translate.cpp"
                    var $cmp631=((($316))|0) > 0; //@line 2980 "translate.cpp"
                    if ($cmp631) { __label__ = 213;; } else { __label__ = 216;; } //@line 2980 "translate.cpp"
                    if (__label__ == 213) {
  
                      var $317=$ix; //@line 2983 "translate.cpp"
                      $j=$317; //@line 2983 "translate.cpp"
                      var $318=$j; //@line 2983 "translate.cpp"
                      var $319=$word_count; //@line 2983 "translate.cpp"
                      var $arrayidx63438=(($words+$319*12)&4294967295); //@line 2983 "translate.cpp"
                      var $start63539=(($arrayidx63438+4)&4294967295); //@line 2983 "translate.cpp"
                      var $320=HEAPU16[(($start63539)>>1)]; //@line 2983 "translate.cpp"
                      var $conv63640=((($320))&65535); //@line 2983 "translate.cpp"
                      var $cmp63741=((($318))|0) > ((($conv63640))|0); //@line 2983 "translate.cpp"
                      var $321=$j; //@line 2985 "translate.cpp"
                      if ($cmp63741) { __lastLabel__ = 213; __label__ = 214;; } else { __lastLabel__ = 213; __label__ = 215;; } //@line 2983 "translate.cpp"
                      $for_body638$$for_end644$296: do { 
                        if (__label__ == 214) {
                          while(1) { 
  
                            var $322=__lastLabel__ == 214 ? $329 : ($321);
                            var $sub639=((($322)-1)&4294967295); //@line 2985 "translate.cpp"
                            var $arrayidx640=(($sbuf+$sub639)&4294967295); //@line 2985 "translate.cpp"
                            var $323=HEAP8[($arrayidx640)]; //@line 2985 "translate.cpp"
                            var $324=$j; //@line 2985 "translate.cpp"
                            var $arrayidx641=(($sbuf+$324)&4294967295); //@line 2985 "translate.cpp"
                            HEAP8[($arrayidx641)]=$323; //@line 2985 "translate.cpp"
                            var $325=$j; //@line 2983 "translate.cpp"
                            var $dec643=((($325)-1)&4294967295); //@line 2983 "translate.cpp"
                            $j=$dec643; //@line 2983 "translate.cpp"
                            var $326=$j; //@line 2983 "translate.cpp"
                            var $327=$word_count; //@line 2983 "translate.cpp"
                            var $arrayidx634=(($words+$327*12)&4294967295); //@line 2983 "translate.cpp"
                            var $start635=(($arrayidx634+4)&4294967295); //@line 2983 "translate.cpp"
                            var $328=HEAPU16[(($start635)>>1)]; //@line 2983 "translate.cpp"
                            var $conv636=((($328))&65535); //@line 2983 "translate.cpp"
                            var $cmp637=((($326))|0) > ((($conv636))|0); //@line 2983 "translate.cpp"
                            var $329=$j; //@line 2985 "translate.cpp"
                            if ($cmp637) { __lastLabel__ = 214; __label__ = 214;continue ; } else { __lastLabel__ = 214; __label__ = 215;break $for_body638$$for_end644$296; } //@line 2983 "translate.cpp"
                          }
                        }
                      } while(0);
  
                      var $_lcssa37=__lastLabel__ == 213 ? $321 : ($329);
                      var $arrayidx645=(($sbuf+$_lcssa37)&4294967295); //@line 2987 "translate.cpp"
                      HEAP8[($arrayidx645)]=32; //@line 2987 "translate.cpp"
                      var $330=$word_count; //@line 2988 "translate.cpp"
                      var $arrayidx646=(($words+$330*12)&4294967295); //@line 2988 "translate.cpp"
                      var $start647=(($arrayidx646+4)&4294967295); //@line 2988 "translate.cpp"
                      var $331=HEAP16[(($start647)>>1)]; //@line 2988 "translate.cpp"
                      var $inc648=((($331)+1)&65535); //@line 2988 "translate.cpp"
                      HEAP16[(($start647)>>1)]=$inc648; //@line 2988 "translate.cpp"
                      var $332=$ix; //@line 2989 "translate.cpp"
                      var $inc649=((($332)+1)&4294967295); //@line 2989 "translate.cpp"
                      $ix=$inc649; //@line 2989 "translate.cpp"
                      ; //@line 2990 "translate.cpp"
                    }
  
                    var $333=$word_count; //@line 2992 "translate.cpp"
                    var $inc651=((($333)+1)&4294967295); //@line 2992 "translate.cpp"
                    $word_count=$inc651; //@line 2992 "translate.cpp"
                    var $334=$ix; //@line 2993 "translate.cpp"
                    var $conv652=((($334)) & 65535); //@line 2993 "translate.cpp"
                    var $335=$word_count; //@line 2993 "translate.cpp"
                    var $arrayidx653=(($words+$335*12)&4294967295); //@line 2993 "translate.cpp"
                    var $start654=(($arrayidx653+4)&4294967295); //@line 2993 "translate.cpp"
                    HEAP16[(($start654)>>1)]=$conv652; //@line 2993 "translate.cpp"
                    var $336=$word_count; //@line 2994 "translate.cpp"
                    var $arrayidx655=(($words+$336*12)&4294967295); //@line 2994 "translate.cpp"
                    var $flags656=(($arrayidx655)&4294967295); //@line 2994 "translate.cpp"
                    HEAP32[(($flags656)>>2)]=0; //@line 2994 "translate.cpp"
                    var $337=HEAP32[(($source_index)>>2)]; //@line 2996 "translate.cpp"
                    $j=$337; //@line 2996 "translate.cpp"
                    var $338=$j; //@line 2996 "translate.cpp"
                    var $arrayidx65844=(($charix+$338*2)&4294967295); //@line 2996 "translate.cpp"
                    var $339=HEAP16[(($arrayidx65844)>>1)]; //@line 2996 "translate.cpp"
                    var $conv65945=(tempInt=(($339)),(tempInt>=32768?tempInt-65536:tempInt)); //@line 2996 "translate.cpp"
                    var $cmp66046=((($conv65945))|0) <= 0; //@line 2996 "translate.cpp"
                    var $340=$j; //@line 2996 "translate.cpp"
                    if ($cmp66046) { __lastLabel__ = 216; __label__ = 217;; } else { __lastLabel__ = 216; __label__ = 218;; } //@line 2996 "translate.cpp"
                    $for_inc662$$for_end664$301: do { 
                      if (__label__ == 217) {
                        while(1) { 
  
                          var $341=__lastLabel__ == 217 ? $344 : ($340);
                          var $inc663=((($341)+1)&4294967295); //@line 2996 "translate.cpp"
                          $j=$inc663; //@line 2996 "translate.cpp"
                          var $342=$j; //@line 2996 "translate.cpp"
                          var $arrayidx658=(($charix+$342*2)&4294967295); //@line 2996 "translate.cpp"
                          var $343=HEAP16[(($arrayidx658)>>1)]; //@line 2996 "translate.cpp"
                          var $conv659=(tempInt=(($343)),(tempInt>=32768?tempInt-65536:tempInt)); //@line 2996 "translate.cpp"
                          var $cmp660=((($conv659))|0) <= 0; //@line 2996 "translate.cpp"
                          var $344=$j; //@line 2996 "translate.cpp"
                          if ($cmp660) { __lastLabel__ = 217; __label__ = 217;continue ; } else { __lastLabel__ = 217; __label__ = 218;break $for_inc662$$for_end664$301; } //@line 2996 "translate.cpp"
                        }
                      }
                    } while(0);
  
                    var $_lcssa43=__lastLabel__ == 216 ? $340 : ($344);
                    var $arrayidx665=(($charix+$_lcssa43*2)&4294967295); //@line 2997 "translate.cpp"
                    var $345=HEAP16[(($arrayidx665)>>1)]; //@line 2997 "translate.cpp"
                    var $346=$word_count; //@line 2997 "translate.cpp"
                    var $arrayidx666=(($words+$346*12)&4294967295); //@line 2997 "translate.cpp"
                    var $sourceix667=(($arrayidx666+6)&4294967295); //@line 2997 "translate.cpp"
                    HEAP16[(($sourceix667)>>1)]=$345; //@line 2997 "translate.cpp"
                    $k=0; //@line 2998 "translate.cpp"
                    var $347=$j; //@line 2999 "translate.cpp"
                    var $arrayidx66948=(($charix+$347*2)&4294967295); //@line 2999 "translate.cpp"
                    var $348=HEAP16[(($arrayidx66948)>>1)]; //@line 2999 "translate.cpp"
                    var $conv67049=(tempInt=(($348)),(tempInt>=32768?tempInt-65536:tempInt)); //@line 2999 "translate.cpp"
                    var $cmp67150=((($conv67049))|0)!=0; //@line 2999 "translate.cpp"
                    if ($cmp67150) { __label__ = 219;; } else { __label__ = 222;; } //@line 2999 "translate.cpp"
                    $while_body672$$while_end680$305: do { 
                      if (__label__ == 219) {
                        while(1) { 
  
                          var $349=$j; //@line 3002 "translate.cpp"
                          var $inc673=((($349)+1)&4294967295); //@line 3002 "translate.cpp"
                          $j=$inc673; //@line 3002 "translate.cpp"
                          var $arrayidx674=(($charix+$349*2)&4294967295); //@line 3002 "translate.cpp"
                          var $350=HEAP16[(($arrayidx674)>>1)]; //@line 3002 "translate.cpp"
                          var $conv675=(tempInt=(($350)),(tempInt>=32768?tempInt-65536:tempInt)); //@line 3002 "translate.cpp"
                          var $cmp676=((($conv675))|0)!=-1; //@line 3002 "translate.cpp"
                          if ($cmp676) { __label__ = 220;; } else { __label__ = 221;; } //@line 3002 "translate.cpp"
                          if (__label__ == 220) {
  
                            var $351=$k; //@line 3003 "translate.cpp"
                            var $inc678=((($351)+1)&4294967295); //@line 3003 "translate.cpp"
                            $k=$inc678; //@line 3003 "translate.cpp"
                            ; //@line 3003 "translate.cpp"
                          }
  
                          var $352=$j; //@line 2999 "translate.cpp"
                          var $arrayidx669=(($charix+$352*2)&4294967295); //@line 2999 "translate.cpp"
                          var $353=HEAP16[(($arrayidx669)>>1)]; //@line 2999 "translate.cpp"
                          var $conv670=(tempInt=(($353)),(tempInt>=32768?tempInt-65536:tempInt)); //@line 2999 "translate.cpp"
                          var $cmp671=((($conv670))|0)!=0; //@line 2999 "translate.cpp"
                          if ($cmp671) { __label__ = 219;continue ; } else { __label__ = 222;break $while_body672$$while_end680$305; } //@line 2999 "translate.cpp"
                        }
                      }
                    } while(0);
  
                    var $354=$k; //@line 3005 "translate.cpp"
                    var $conv681=((($354)) & 255); //@line 3005 "translate.cpp"
                    var $355=$word_count; //@line 3005 "translate.cpp"
                    var $arrayidx682=(($words+$355*12)&4294967295); //@line 3005 "translate.cpp"
                    var $length683=(($arrayidx682+10)&4294967295); //@line 3005 "translate.cpp"
                    HEAP8[($length683)]=$conv681; //@line 3005 "translate.cpp"
                    var $356=$next_word_flags; //@line 3007 "translate.cpp"
                    $word_flags=$356; //@line 3007 "translate.cpp"
                    $next_word_flags=0; //@line 3008 "translate.cpp"
                    HEAP32[((_pre_pause)>>2)]=0; //@line 3009 "translate.cpp"
                    $word_mark=0; //@line 3010 "translate.cpp"
                    $all_upper_case=1; //@line 3011 "translate.cpp"
                    $syllable_marked=0; //@line 3012 "translate.cpp"
                    ; //@line 3013 "translate.cpp"
                  }
                } while(0);
  
                var $357=$space_inserted; //@line 3015 "translate.cpp"
                var $tobool685=((($357))|0)!=0; //@line 3015 "translate.cpp"
                if (!($tobool685)) { __label__ = 227;break $if_then570$$if_else688$269; } //@line 3015 "translate.cpp"
  
                var $358=$prev_source_index; //@line 3017 "translate.cpp"
                HEAP32[(($source_index)>>2)]=$358; //@line 3017 "translate.cpp"
                HEAP32[(($char_inserted)>>2)]=0; //@line 3018 "translate.cpp"
                $space_inserted=0; //@line 3019 "translate.cpp"
                ; //@line 3020 "translate.cpp"
              }
            }
            else if (__label__ == 225) {
  
              var $359=$ix; //@line 3024 "translate.cpp"
              var $cmp689=((($359))|0) < 696; //@line 3024 "translate.cpp"
              if (!($cmp689)) { __label__ = 227;break $if_then570$$if_else688$269; } //@line 3024 "translate.cpp"
  
              var $360=$c; //@line 3025 "translate.cpp"
              var $361=$ix; //@line 3025 "translate.cpp"
              var $arrayidx691=(($sbuf+$361)&4294967295); //@line 3025 "translate.cpp"
              var $call692=__Z8utf8_outjPc($360, $arrayidx691); //@line 3025 "translate.cpp"
              var $362=$ix; //@line 3025 "translate.cpp"
              var $add693=((($362)+($call692))&4294967295); //@line 3025 "translate.cpp"
              $ix=$add693; //@line 3025 "translate.cpp"
              ; //@line 3025 "translate.cpp"
            }
          } while(0);
  
          var $363=$pre_pause_add; //@line 3027 "translate.cpp"
          var $364=HEAP32[((_pre_pause)>>2)]; //@line 3027 "translate.cpp"
          var $cmp696=((($363))|0) > ((($364))|0); //@line 3027 "translate.cpp"
          if ($cmp696) { __label__ = 228;; } else { __label__ = 229;; } //@line 3027 "translate.cpp"
          if (__label__ == 228) {
  
            var $365=$pre_pause_add; //@line 3028 "translate.cpp"
            HEAP32[((_pre_pause)>>2)]=$365; //@line 3028 "translate.cpp"
            ; //@line 3028 "translate.cpp"
          }
  
          $pre_pause_add=0; //@line 3029 "translate.cpp"
          __label__ = 33;continue $while_cond88$50; //@line 3030 "translate.cpp"
        }
  
        var $366=$word_count; //@line 3032 "translate.cpp"
        var $cmp700=((($366))|0)==0; //@line 3032 "translate.cpp"
        if ($cmp700) { __label__ = 231;; } else { __label__ = 233;; } //@line 3032 "translate.cpp"
        $land_lhs_true701$$if_end710$321: do { 
          if (__label__ == 231) {
  
            var $367=$embedded_count; //@line 3032 "translate.cpp"
            var $cmp702=((($367))|0) > 0; //@line 3032 "translate.cpp"
            if (!($cmp702)) { __label__ = 233;break $land_lhs_true701$$if_end710$321; } //@line 3032 "translate.cpp"
  
            var $368=HEAP32[((__ZL11embedded_ix)>>2)]; //@line 3035 "translate.cpp"
            var $sub704=((($368)-1)&4294967295); //@line 3035 "translate.cpp"
            var $arrayidx705=((_embedded_list+$sub704*4)&4294967295); //@line 3035 "translate.cpp"
            var $369=HEAP32[(($arrayidx705)>>2)]; //@line 3035 "translate.cpp"
            var $or706=($369) | 128; //@line 3035 "translate.cpp"
            HEAP32[(($arrayidx705)>>2)]=$or706; //@line 3035 "translate.cpp"
            var $370=$word_count; //@line 3036 "translate.cpp"
            var $arrayidx707=(($words+$370*12)&4294967295); //@line 3036 "translate.cpp"
            var $flags708=(($arrayidx707)&4294967295); //@line 3036 "translate.cpp"
            var $371=HEAP32[(($flags708)>>2)]; //@line 3036 "translate.cpp"
            var $or709=($371) | 64; //@line 3036 "translate.cpp"
            HEAP32[(($flags708)>>2)]=$or709; //@line 3036 "translate.cpp"
            $word_count=1; //@line 3037 "translate.cpp"
            ; //@line 3038 "translate.cpp"
          }
        } while(0);
  
        var $372=$ix; //@line 3040 "translate.cpp"
        var $sub711=((($372)-1)&4294967295); //@line 3040 "translate.cpp"
        var $arrayidx712=(($sbuf+$sub711)&4294967295); //@line 3040 "translate.cpp"
        var $373=$tr_addr; //@line 3040 "translate.cpp"
        var $clause_end=(($373+8292)&4294967295); //@line 3040 "translate.cpp"
        HEAP32[(($clause_end)>>2)]=$arrayidx712; //@line 3040 "translate.cpp"
        var $374=$ix; //@line 3041 "translate.cpp"
        var $arrayidx713=(($sbuf+$374)&4294967295); //@line 3041 "translate.cpp"
        HEAP8[($arrayidx713)]=0; //@line 3041 "translate.cpp"
        var $arrayidx714=(($words)&4294967295); //@line 3042 "translate.cpp"
        var $pre_pause715=(($arrayidx714+8)&4294967295); //@line 3042 "translate.cpp"
        HEAP8[($pre_pause715)]=0; //@line 3042 "translate.cpp"
        var $375=$word_count; //@line 3043 "translate.cpp"
        var $arrayidx716=(($words+$375*12)&4294967295); //@line 3043 "translate.cpp"
        var $pre_pause717=(($arrayidx716+8)&4294967295); //@line 3043 "translate.cpp"
        HEAP8[($pre_pause717)]=8; //@line 3043 "translate.cpp"
        var $376=$word_count; //@line 3044 "translate.cpp"
        var $cmp718=((($376))|0) > 0; //@line 3044 "translate.cpp"
        if ($cmp718) { __label__ = 234;; } else { __label__ = 241;; } //@line 3044 "translate.cpp"
        $if_then719$$if_end751$325: do { 
          if (__label__ == 234) {
  
            var $377=$word_count; //@line 3046 "translate.cpp"
            var $sub720=((($377)-1)&4294967295); //@line 3046 "translate.cpp"
            $ix=$sub720; //@line 3046 "translate.cpp"
            __lastLabel__ = 234; ; //@line 3047 "translate.cpp"
            while(1) { 
  
              var $378=__lastLabel__ == 237 ? $dec732 : ($sub720); //@line 3047 "translate.cpp"
              var $cmp722=((($378))|0) > 0; //@line 3047 "translate.cpp"
              var $379=$ix; //@line 3047 "translate.cpp"
              if ($cmp722) { __lastLabel__ = 235; ; } else { __lastLabel__ = 235; __label__ = 238;break ; } //@line 3047 "translate.cpp"
  
              var $arrayidx724=(($words+$379*12)&4294967295); //@line 3047 "translate.cpp"
              var $start725=(($arrayidx724+4)&4294967295); //@line 3047 "translate.cpp"
              var $380=HEAPU16[(($start725)>>1)]; //@line 3047 "translate.cpp"
              var $idxprom=((($380))&65535); //@line 3047 "translate.cpp"
              var $arrayidx726=(($sbuf+$idxprom)&4294967295); //@line 3047 "translate.cpp"
              var $381=HEAP8[($arrayidx726)]; //@line 3047 "translate.cpp"
              var $conv727=(tempInt=(($381)),(tempInt>=128?tempInt-256:tempInt)); //@line 3047 "translate.cpp"
              var $call728=__Z9IsBracketi($conv727); //@line 3047 "translate.cpp"
              var $tobool729=((($call728))|0)!=0; //@line 3047 "translate.cpp"
              var $382=$ix; //@line 3048 "translate.cpp"
              if ($tobool729) { __lastLabel__ = 236; ; } else { __lastLabel__ = 236; __label__ = 238;break ; }
  
              var $dec732=((($382)-1)&4294967295); //@line 3048 "translate.cpp"
              $ix=$dec732; //@line 3048 "translate.cpp"
              __lastLabel__ = 237; __label__ = 235;continue ; //@line 3048 "translate.cpp"
            }
  
            var $383=__lastLabel__ == 236 ? $382 : ($379);
            var $arrayidx734=(($words+$383*12)&4294967295); //@line 3049 "translate.cpp"
            var $flags735=(($arrayidx734)&4294967295); //@line 3049 "translate.cpp"
            var $384=HEAP32[(($flags735)>>2)]; //@line 3049 "translate.cpp"
            var $or736=($384) | 16; //@line 3049 "translate.cpp"
            HEAP32[(($flags735)>>2)]=$or736; //@line 3049 "translate.cpp"
            var $385=$terminator; //@line 3052 "translate.cpp"
            var $and737=($385) & 4194304; //@line 3052 "translate.cpp"
            var $tobool738=((($and737))|0)!=0; //@line 3052 "translate.cpp"
            if (!($tobool738)) { __label__ = 241;break $if_then719$$if_end751$325; } //@line 3052 "translate.cpp"
  
            var $386=$word_count; //@line 3052 "translate.cpp"
            var $sub740=((($386)-1)&4294967295); //@line 3052 "translate.cpp"
            var $arrayidx741=(($words+$sub740*12)&4294967295); //@line 3052 "translate.cpp"
            var $flags742=(($arrayidx741)&4294967295); //@line 3052 "translate.cpp"
            var $387=HEAP32[(($flags742)>>2)]; //@line 3052 "translate.cpp"
            var $and743=($387) & 256; //@line 3052 "translate.cpp"
            var $tobool744=((($and743))|0)!=0; //@line 3052 "translate.cpp"
            if ($tobool744) { __label__ = 241;break $if_then719$$if_end751$325; } //@line 3052 "translate.cpp"
  
            var $388=$word_count; //@line 3053 "translate.cpp"
            var $sub746=((($388)-1)&4294967295); //@line 3053 "translate.cpp"
            var $arrayidx747=(($words+$sub746*12)&4294967295); //@line 3053 "translate.cpp"
            var $flags748=(($arrayidx747)&4294967295); //@line 3053 "translate.cpp"
            var $389=HEAP32[(($flags748)>>2)]; //@line 3053 "translate.cpp"
            var $or749=($389) | 65536; //@line 3053 "translate.cpp"
            HEAP32[(($flags748)>>2)]=$or749; //@line 3053 "translate.cpp"
            ; //@line 3053 "translate.cpp"
          }
        } while(0);
  
        var $arrayidx752=(($words)&4294967295); //@line 3055 "translate.cpp"
        var $flags753=(($arrayidx752)&4294967295); //@line 3055 "translate.cpp"
        var $390=HEAP32[(($flags753)>>2)]; //@line 3055 "translate.cpp"
        var $or754=($390) | 512; //@line 3055 "translate.cpp"
        HEAP32[(($flags753)>>2)]=$or754; //@line 3055 "translate.cpp"
        $ix=0; //@line 3058 "translate.cpp"
        var $391=$ix; //@line 3058 "translate.cpp"
        var $392=$word_count; //@line 3058 "translate.cpp"
        var $cmp75636=((($391))|0) < ((($392))|0); //@line 3058 "translate.cpp"
        if ($cmp75636) { __label__ = 242;; } else { __label__ = 296;; } //@line 3058 "translate.cpp"
        $for_body757_lr_ph$$for_end994$335: do { 
          if (__label__ == 242) {
  
            var $arraydecay781=(($number_buf)&4294967295); //@line 3086 "translate.cpp"
            var $arrayidx783=(($number_buf+130)&4294967295); //@line 3087 "translate.cpp"
            var $arraydecay816=(($number_buf)&4294967295); //@line 3104 "translate.cpp"
            var $sub_ptr_rhs_cast818=($arraydecay816); //@line 3104 "translate.cpp"
            var $arraydecay820=(($number_buf)&4294967295); //@line 3105 "translate.cpp"
            var $arrayidx836=(($number_buf)&4294967295); //@line 3117 "translate.cpp"
            var $arrayidx837=(($number_buf+1)&4294967295); //@line 3118 "translate.cpp"
            var $arrayidx850=(($number_buf+130)&4294967295); //@line 3126 "translate.cpp"
            var $arrayidx920=(($number_buf+1)&4294967295); //@line 3167 "translate.cpp"
            var $arraydecay967=(($number_buf)&4294967295); //@line 3191 "translate.cpp"
            var $arrayidx969=(($number_buf+2)&4294967295); //@line 3193 "translate.cpp"
            var $arrayidx970=(($number_buf+2)&4294967295); //@line 3194 "translate.cpp"
            ; //@line 3058 "translate.cpp"
            $for_body757$337: while(1) { 
  
              var $393=HEAP32[((_count_words)>>2)]; //@line 3069 "translate.cpp"
              var $inc758=((($393)+1)&4294967295); //@line 3069 "translate.cpp"
              HEAP32[((_count_words)>>2)]=$inc758; //@line 3069 "translate.cpp"
              var $394=HEAP32[((_skip_words)>>2)]; //@line 3070 "translate.cpp"
              var $cmp759=((($394))|0) > 0; //@line 3070 "translate.cpp"
              if ($cmp759) { __label__ = 244;; } else { __label__ = 246;; } //@line 3070 "translate.cpp"
              $if_then760$$if_end765$339: do { 
                if (__label__ == 244) {
  
                  var $395=HEAP32[((_skip_words)>>2)]; //@line 3072 "translate.cpp"
                  var $dec761=((($395)-1)&4294967295); //@line 3072 "translate.cpp"
                  HEAP32[((_skip_words)>>2)]=$dec761; //@line 3072 "translate.cpp"
                  var $396=HEAP32[((_skip_words)>>2)]; //@line 3073 "translate.cpp"
                  var $cmp762=((($396))|0)==0; //@line 3073 "translate.cpp"
                  if (!($cmp762)) { __label__ = 246;break $if_then760$$if_end765$339; } //@line 3073 "translate.cpp"
  
                  HEAP32[((_skipping_text)>>2)]=0; //@line 3074 "translate.cpp"
                  __label__ = 247;break $if_then760$$if_end765$339; //@line 3076 "translate.cpp"
                }
              } while(0);
              $if_end768$$if_end765$342: do { 
                if (__label__ == 246) {
  
                  var $_pr22=HEAP32[((_skipping_text)>>2)]; //@line 3076 "translate.cpp"
                  var $tobool766=((($_pr22))|0)!=0; //@line 3076 "translate.cpp"
                  if ($tobool766) { __label__ = 295;break $if_end768$$if_end765$342; } else { __label__ = 247;break $if_end768$$if_end765$342; } //@line 3076 "translate.cpp"
                }
              } while(0);
              $if_end768$$for_inc992$344: do { 
                if (__label__ == 247) {
  
                  var $397=$ix; //@line 3081 "translate.cpp"
                  var $arrayidx769=(($words+$397*12)&4294967295); //@line 3081 "translate.cpp"
                  var $start770=(($arrayidx769+4)&4294967295); //@line 3081 "translate.cpp"
                  var $398=HEAPU16[(($start770)>>1)]; //@line 3081 "translate.cpp"
                  var $idxprom771=((($398))&65535); //@line 3081 "translate.cpp"
                  var $arrayidx772=(($sbuf+$idxprom771)&4294967295); //@line 3081 "translate.cpp"
                  $pw=$arrayidx772; //@line 3081 "translate.cpp"
                  $word=$arrayidx772; //@line 3081 "translate.cpp"
                  var $399=$word; //@line 3083 "translate.cpp"
                  var $arrayidx773=(($399)&4294967295); //@line 3083 "translate.cpp"
                  var $400=HEAP8[($arrayidx773)]; //@line 3083 "translate.cpp"
                  var $conv774=(tempInt=(($400)),(tempInt>=128?tempInt-256:tempInt)); //@line 3083 "translate.cpp"
                  var $call775=_iswdigit($conv774); //@line 3083 "translate.cpp"
                  var $tobool776=((($call775))|0)!=0; //@line 3083 "translate.cpp"
                  if ($tobool776) { __label__ = 248;; } else { __label__ = 261;; } //@line 3083 "translate.cpp"
                  $land_lhs_true777$$if_end824$346: do { 
                    if (__label__ == 248) {
  
                      var $401=$tr_addr; //@line 3083 "translate.cpp"
                      var $langopts778=(($401)&4294967295); //@line 3083 "translate.cpp"
                      var $break_numbers=(($langopts778+192)&4294967295); //@line 3083 "translate.cpp"
                      var $402=HEAP32[(($break_numbers)>>2)]; //@line 3083 "translate.cpp"
                      var $cmp779=((($402))|0)!=1227133512; //@line 3083 "translate.cpp"
                      if (!($cmp779)) { __label__ = 261;break $land_lhs_true777$$if_end824$346; } //@line 3083 "translate.cpp"
  
                      $pn=$arraydecay781; //@line 3086 "translate.cpp"
                      ; //@line 3087 "translate.cpp"
                      while(1) { 
  
                        var $403=$pn; //@line 3087 "translate.cpp"
                        var $cmp784=((($403))>>>0) < ((($arrayidx783))>>>0); //@line 3087 "translate.cpp"
                        if (!($cmp784)) { __label__ = 260;break ; } //@line 3087 "translate.cpp"
  
                        var $404=$pw; //@line 3089 "translate.cpp"
                        var $405=HEAP8[($404)]; //@line 3089 "translate.cpp"
                        var $conv786=(tempInt=(($405)),(tempInt>=128?tempInt-256:tempInt)); //@line 3089 "translate.cpp"
                        var $call787=_iswdigit($conv786); //@line 3089 "translate.cpp"
                        var $tobool788=((($call787))|0)!=0; //@line 3089 "translate.cpp"
                        var $406=$pw; //@line 3091 "translate.cpp"
                        if ($tobool788) { __label__ = 252;; } else { __label__ = 253;; } //@line 3089 "translate.cpp"
                        if (__label__ == 252) {
  
                          var $incdec_ptr790=(($406+1)&4294967295); //@line 3091 "translate.cpp"
                          $pw=$incdec_ptr790; //@line 3091 "translate.cpp"
                          var $407=HEAP8[($406)]; //@line 3091 "translate.cpp"
                          var $408=$pn; //@line 3091 "translate.cpp"
                          var $incdec_ptr791=(($408+1)&4294967295); //@line 3091 "translate.cpp"
                          $pn=$incdec_ptr791; //@line 3091 "translate.cpp"
                          HEAP8[($408)]=$407; //@line 3091 "translate.cpp"
                          __label__ = 250;continue ; //@line 3092 "translate.cpp"
                        }
                        else if (__label__ == 253) {
  
                          var $409=HEAP8[($406)]; //@line 3094 "translate.cpp"
                          var $conv793=(tempInt=(($409)),(tempInt>=128?tempInt-256:tempInt)); //@line 3094 "translate.cpp"
                          var $410=$tr_addr; //@line 3094 "translate.cpp"
                          var $langopts794=(($410)&4294967295); //@line 3094 "translate.cpp"
                          var $thousands_sep=(($langopts794+204)&4294967295); //@line 3094 "translate.cpp"
                          var $411=HEAP32[(($thousands_sep)>>2)]; //@line 3094 "translate.cpp"
                          var $cmp795=((($conv793))|0)==((($411))|0); //@line 3094 "translate.cpp"
                          if (!($cmp795)) { __label__ = 259;break ; } //@line 3094 "translate.cpp"
  
                          var $412=$pw; //@line 3094 "translate.cpp"
                          var $arrayidx797=(($412+1)&4294967295); //@line 3094 "translate.cpp"
                          var $413=HEAP8[($arrayidx797)]; //@line 3094 "translate.cpp"
                          var $conv798=(tempInt=(($413)),(tempInt>=128?tempInt-256:tempInt)); //@line 3094 "translate.cpp"
                          var $cmp799=((($conv798))|0)==32; //@line 3094 "translate.cpp"
                          if (!($cmp799)) { __label__ = 259;break ; } //@line 3094 "translate.cpp"
  
                          var $414=$pw; //@line 3095 "translate.cpp"
                          var $arrayidx801=(($414+2)&4294967295); //@line 3095 "translate.cpp"
                          var $415=HEAP8[($arrayidx801)]; //@line 3095 "translate.cpp"
                          var $conv802=(tempInt=(($415)),(tempInt>=128?tempInt-256:tempInt)); //@line 3095 "translate.cpp"
                          var $call803=_iswdigit($conv802); //@line 3095 "translate.cpp"
                          var $tobool804=((($call803))|0)!=0; //@line 3095 "translate.cpp"
                          if (!($tobool804)) { __label__ = 259;break ; } //@line 3095 "translate.cpp"
  
                          var $416=$pw; //@line 3095 "translate.cpp"
                          var $arrayidx806=(($416+3)&4294967295); //@line 3095 "translate.cpp"
                          var $417=HEAP8[($arrayidx806)]; //@line 3095 "translate.cpp"
                          var $conv807=(tempInt=(($417)),(tempInt>=128?tempInt-256:tempInt)); //@line 3095 "translate.cpp"
                          var $cmp808=((($conv807))|0)!=32; //@line 3095 "translate.cpp"
                          if (!($cmp808)) { __label__ = 259;break ; } //@line 3095 "translate.cpp"
  
                          var $418=$pw; //@line 3095 "translate.cpp"
                          var $arrayidx810=(($418+4)&4294967295); //@line 3095 "translate.cpp"
                          var $419=HEAP8[($arrayidx810)]; //@line 3095 "translate.cpp"
                          var $conv811=(tempInt=(($419)),(tempInt>=128?tempInt-256:tempInt)); //@line 3095 "translate.cpp"
                          var $cmp812=((($conv811))|0)!=32; //@line 3095 "translate.cpp"
                          if (!($cmp812)) { __label__ = 259;break ; } //@line 3095 "translate.cpp"
  
                          var $420=$pw; //@line 3097 "translate.cpp"
                          var $add_ptr=(($420+2)&4294967295); //@line 3097 "translate.cpp"
                          $pw=$add_ptr; //@line 3097 "translate.cpp"
                          var $421=$ix; //@line 3098 "translate.cpp"
                          var $inc814=((($421)+1)&4294967295); //@line 3098 "translate.cpp"
                          $ix=$inc814; //@line 3098 "translate.cpp"
                          __label__ = 250;continue ;
                        }
                      }
                      if (__label__ == 259) {
  
                        var $422=$pw; //@line 3102 "translate.cpp"
                        var $423=$word; //@line 3102 "translate.cpp"
                        var $sub_ptr_lhs_cast=($422); //@line 3102 "translate.cpp"
                        var $sub_ptr_rhs_cast=($423); //@line 3102 "translate.cpp"
                        var $sub_ptr_sub=((($sub_ptr_lhs_cast)-($sub_ptr_rhs_cast))&4294967295); //@line 3102 "translate.cpp"
                        $nx=$sub_ptr_sub; //@line 3102 "translate.cpp"
                        var $424=$word; //@line 3103 "translate.cpp"
                        var $425=$nx; //@line 3103 "translate.cpp"
                        var $dest$, $stop$, $stop4$, $fast$, $value4$;
                        $dest$ = $424;
                        $stop$ = $dest$ + $425;
                        $value4$ = 32;
                        if ($value4$ < 0) $value4$ += 256;
                        $value4$ = $value4$ + ($value4$<<8) + ($value4$<<16) + ($value4$*16777216);
                        while ($dest$%4 !== 0 && $dest$ < $stop$) {
                          ;; HEAP8[$dest$++] = 32;
                        }
                        $dest$ >>= 2;
                        $stop4$ = $stop$ >> 2;
                        while ($dest$ < $stop4$) {
                        ;;;;;;;  HEAP32[$dest$++] = $value4$;
                        }
                        $dest$ <<= 2;
                        while ($dest$ < $stop$) {
                          ;; HEAP8[$dest$++] = 32;
                        }; //@line 3103 "translate.cpp"
                        var $426=$pn; //@line 3104 "translate.cpp"
                        var $sub_ptr_lhs_cast817=($426); //@line 3104 "translate.cpp"
                        var $sub_ptr_sub819=((($sub_ptr_lhs_cast817)-($sub_ptr_rhs_cast818))&4294967295); //@line 3104 "translate.cpp"
                        $nx=$sub_ptr_sub819; //@line 3104 "translate.cpp"
                        var $427=$word; //@line 3105 "translate.cpp"
                        var $428=$nx; //@line 3105 "translate.cpp"
                        var $src$, $dest$, $stop$, $stop4$, $fast$;
                        $src$ = $arraydecay820;
                        $dest$ = $427;
                        $stop$ = $src$ + $428;
                        if (($dest$%4) == ($src$%4) && $428 > 8) {
                          while ($src$%4 !== 0 && $src$ < $stop$) {
                            ; HEAP8[$dest$++] = HEAP8[$src$++];
                          }
                          $src$ >>= 2;
                          $dest$ >>= 2;
                          $stop4$ = $stop$ >> 2;
                          while ($src$ < $stop4$) {
                        ;;;    HEAP32[$dest$++] = HEAP32[$src$++];
                          }
                          $src$ <<= 2;
                          $dest$ <<= 2;
                        }while ($src$ < $stop$) {
                          ; HEAP8[$dest$++] = HEAP8[$src$++];
                        }; //@line 3105 "translate.cpp"
                        ; //@line 3106 "translate.cpp"
                      }
  
                      var $429=$word; //@line 3109 "translate.cpp"
                      $pw=$429; //@line 3109 "translate.cpp"
                      ; //@line 3110 "translate.cpp"
                    }
                  } while(0);
  
                  $n_digits=0; //@line 3112 "translate.cpp"
                  var $430=$n_digits; //@line 3112 "translate.cpp"
                  var $431=$word; //@line 3112 "translate.cpp"
                  var $arrayidx82630=(($431+$430)&4294967295); //@line 3112 "translate.cpp"
                  var $432=HEAP8[($arrayidx82630)]; //@line 3112 "translate.cpp"
                  var $conv82731=(tempInt=(($432)),(tempInt>=128?tempInt-256:tempInt)); //@line 3112 "translate.cpp"
                  var $call82832=_iswdigit($conv82731); //@line 3112 "translate.cpp"
                  var $tobool82933=((($call82832))|0)!=0; //@line 3112 "translate.cpp"
                  var $433=$n_digits; //@line 3112 "translate.cpp"
                  if ($tobool82933) { __lastLabel__ = 261; __label__ = 262;; } else { __lastLabel__ = 261; __label__ = 263;; } //@line 3112 "translate.cpp"
                  $for_inc831$$for_end833$364: do { 
                    if (__label__ == 262) {
                      while(1) { 
  
                        var $434=__lastLabel__ == 262 ? $438 : ($433);
                        var $inc832=((($434)+1)&4294967295); //@line 3112 "translate.cpp"
                        $n_digits=$inc832; //@line 3112 "translate.cpp"
                        var $435=$n_digits; //@line 3112 "translate.cpp"
                        var $436=$word; //@line 3112 "translate.cpp"
                        var $arrayidx826=(($436+$435)&4294967295); //@line 3112 "translate.cpp"
                        var $437=HEAP8[($arrayidx826)]; //@line 3112 "translate.cpp"
                        var $conv827=(tempInt=(($437)),(tempInt>=128?tempInt-256:tempInt)); //@line 3112 "translate.cpp"
                        var $call828=_iswdigit($conv827); //@line 3112 "translate.cpp"
                        var $tobool829=((($call828))|0)!=0; //@line 3112 "translate.cpp"
                        var $438=$n_digits; //@line 3112 "translate.cpp"
                        if ($tobool829) { __lastLabel__ = 262; __label__ = 262;continue ; } else { __lastLabel__ = 262; __label__ = 263;break $for_inc831$$for_end833$364; } //@line 3112 "translate.cpp"
                      }
                    }
                  } while(0);
  
                  var $_lcssa29=__lastLabel__ == 261 ? $433 : ($438);
                  var $cmp834=((($_lcssa29))|0) > 4; //@line 3114 "translate.cpp"
                  if ($cmp834) { __label__ = 264;; } else { __label__ = 284;; } //@line 3114 "translate.cpp"
                  $if_then835$$if_else939$368: do { 
                    if (__label__ == 264) {
  
                      HEAP8[($arrayidx836)]=32; //@line 3117 "translate.cpp"
                      $pn=$arrayidx837; //@line 3118 "translate.cpp"
                      var $439=$n_digits; //@line 3119 "translate.cpp"
                      $nx=$439; //@line 3119 "translate.cpp"
                      $nw=0; //@line 3120 "translate.cpp"
                      $individual_digits=0; //@line 3121 "translate.cpp"
                      var $440=$n_digits; //@line 3123 "translate.cpp"
                      var $441=$tr_addr; //@line 3123 "translate.cpp"
                      var $langopts838=(($441)&4294967295); //@line 3123 "translate.cpp"
                      var $max_digits=(($langopts838+212)&4294967295); //@line 3123 "translate.cpp"
                      var $442=HEAP32[(($max_digits)>>2)]; //@line 3123 "translate.cpp"
                      var $cmp839=((($440))|0) > ((($442))|0); //@line 3123 "translate.cpp"
                      if ($cmp839) { __label__ = 266;; } else { __label__ = 265;; } //@line 3123 "translate.cpp"
                      $if_then844$$lor_lhs_false840$370: do { 
                        if (__label__ == 265) {
  
                          var $443=$word; //@line 3123 "translate.cpp"
                          var $arrayidx841=(($443)&4294967295); //@line 3123 "translate.cpp"
                          var $444=HEAP8[($arrayidx841)]; //@line 3123 "translate.cpp"
                          var $conv842=(tempInt=(($444)),(tempInt>=128?tempInt-256:tempInt)); //@line 3123 "translate.cpp"
                          var $cmp843=((($conv842))|0)==48; //@line 3123 "translate.cpp"
                          if ($cmp843) { __label__ = 266;break $if_then844$$lor_lhs_false840$370; } else { __label__ = 267;break $if_then844$$lor_lhs_false840$370; } //@line 3123 "translate.cpp"
                        }
                      } while(0);
                      if (__label__ == 266) {
  
                        var $445=$ix; //@line 3124 "translate.cpp"
                        var $arrayidx845=(($words+$445*12)&4294967295); //@line 3124 "translate.cpp"
                        var $flags846=(($arrayidx845)&4294967295); //@line 3124 "translate.cpp"
                        var $446=HEAP32[(($flags846)>>2)]; //@line 3124 "translate.cpp"
                        var $or847=($446) | 524288; //@line 3124 "translate.cpp"
                        HEAP32[(($flags846)>>2)]=$or847; //@line 3124 "translate.cpp"
                        ; //@line 3124 "translate.cpp"
                      }
                      $while_cond849$374: while(1) { 
  
                        var $447=$pn; //@line 3126 "translate.cpp"
                        var $cmp851=((($447))>>>0) < ((($arrayidx850))>>>0); //@line 3126 "translate.cpp"
                        if (!($cmp851)) { __label__ = 278;break $while_cond849$374; } //@line 3126 "translate.cpp"
  
                        var $448=$pw; //@line 3128 "translate.cpp"
                        var $incdec_ptr853=(($448+1)&4294967295); //@line 3128 "translate.cpp"
                        $pw=$incdec_ptr853; //@line 3128 "translate.cpp"
                        var $449=HEAP8[($448)]; //@line 3128 "translate.cpp"
                        var $conv854=(tempInt=(($449)),(tempInt>=128?tempInt-256:tempInt)); //@line 3128 "translate.cpp"
                        $c=$conv854; //@line 3128 "translate.cpp"
                        var $isdigittmp=((($conv854)-48)&4294967295); //@line 3128 "translate.cpp"
                        var $isdigit=((($isdigittmp))>>>0) < 10; //@line 3128 "translate.cpp"
                        var $call855=((($isdigit))&1); //@line 3128 "translate.cpp"
                        var $tobool856=((($call855))|0)!=0; //@line 3128 "translate.cpp"
                        if ($tobool856) { __label__ = 270;; } else { __label__ = 269;; } //@line 3128 "translate.cpp"
                        if (__label__ == 269) {
  
                          var $450=$c; //@line 3128 "translate.cpp"
                          var $451=$tr_addr; //@line 3128 "translate.cpp"
                          var $langopts858=(($451)&4294967295); //@line 3128 "translate.cpp"
                          var $decimal_sep859=(($langopts858+208)&4294967295); //@line 3128 "translate.cpp"
                          var $452=HEAP32[(($decimal_sep859)>>2)]; //@line 3128 "translate.cpp"
                          var $cmp860=((($450))|0)!=((($452))|0); //@line 3128 "translate.cpp"
                          if ($cmp860) { __label__ = 278;break $while_cond849$374; } //@line 3128 "translate.cpp"
                        }
  
                        var $453=$c; //@line 3131 "translate.cpp"
                        var $conv863=((($453)) & 255); //@line 3131 "translate.cpp"
                        var $454=$pn; //@line 3131 "translate.cpp"
                        var $incdec_ptr864=(($454+1)&4294967295); //@line 3131 "translate.cpp"
                        $pn=$incdec_ptr864; //@line 3131 "translate.cpp"
                        HEAP8[($454)]=$conv863; //@line 3131 "translate.cpp"
                        var $455=$nx; //@line 3132 "translate.cpp"
                        var $dec865=((($455)-1)&4294967295); //@line 3132 "translate.cpp"
                        $nx=$dec865; //@line 3132 "translate.cpp"
                        var $cmp866=((($dec865))|0) > 0; //@line 3132 "translate.cpp"
                        if (!($cmp866)) { __label__ = 267;continue $while_cond849$374; } //@line 3132 "translate.cpp"
  
                        var $456=$tr_addr; //@line 3132 "translate.cpp"
                        var $langopts868=(($456)&4294967295); //@line 3132 "translate.cpp"
                        var $break_numbers869=(($langopts868+192)&4294967295); //@line 3132 "translate.cpp"
                        var $457=HEAP32[(($break_numbers869)>>2)]; //@line 3132 "translate.cpp"
                        var $458=$nx; //@line 3132 "translate.cpp"
                        var $shl=1 << ($458); //@line 3132 "translate.cpp"
                        var $and870=($shl) & ($457); //@line 3132 "translate.cpp"
                        var $tobool871=((($and870))|0)!=0; //@line 3132 "translate.cpp"
                        if (!($tobool871)) { __label__ = 267;continue $while_cond849$374; } //@line 3132 "translate.cpp"
  
                        var $459=$nw; //@line 3134 "translate.cpp"
                        var $inc873=((($459)+1)&4294967295); //@line 3134 "translate.cpp"
                        $nw=$inc873; //@line 3134 "translate.cpp"
                        var $arrayidx874=(($num_wtab+$459*12)&4294967295); //@line 3134 "translate.cpp"
                        var $460=$arrayidx874; //@line 3134 "translate.cpp"
                        var $461=$ix; //@line 3134 "translate.cpp"
                        var $arrayidx875=(($words+$461*12)&4294967295); //@line 3134 "translate.cpp"
                        var $462=$arrayidx875; //@line 3134 "translate.cpp"
                        var $src$, $dest$, $stop$, $stop4$, $fast$;
                        $src$ = $462;
                        $dest$ = $460;
                        $stop$ = $src$ + 12;
                        if (($dest$%4) == ($src$%4) && 12 > 8) {
                          while ($src$%4 !== 0 && $src$ < $stop$) {
                            ; HEAP8[$dest$++] = HEAP8[$src$++];
                          }
                          $src$ >>= 2;
                          $dest$ >>= 2;
                          $stop4$ = $stop$ >> 2;
                          while ($src$ < $stop4$) {
                        ;;;    HEAP32[$dest$++] = HEAP32[$src$++];
                          }
                          $src$ <<= 2;
                          $dest$ <<= 2;
                        }while ($src$ < $stop$) {
                          ; HEAP8[$dest$++] = HEAP8[$src$++];
                        }; //@line 3134 "translate.cpp"
                        var $463=$tr_addr; //@line 3136 "translate.cpp"
                        var $langopts876=(($463)&4294967295); //@line 3136 "translate.cpp"
                        var $thousands_sep877=(($langopts876+204)&4294967295); //@line 3136 "translate.cpp"
                        var $464=HEAP32[(($thousands_sep877)>>2)]; //@line 3136 "translate.cpp"
                        var $cmp878=((($464))|0)!=32; //@line 3136 "translate.cpp"
                        if ($cmp878) { __label__ = 273;; } else { __label__ = 274;; } //@line 3136 "translate.cpp"
                        if (__label__ == 273) {
  
                          var $465=$tr_addr; //@line 3138 "translate.cpp"
                          var $langopts880=(($465)&4294967295); //@line 3138 "translate.cpp"
                          var $thousands_sep881=(($langopts880+204)&4294967295); //@line 3138 "translate.cpp"
                          var $466=HEAP32[(($thousands_sep881)>>2)]; //@line 3138 "translate.cpp"
                          var $conv882=((($466)) & 255); //@line 3138 "translate.cpp"
                          var $467=$pn; //@line 3138 "translate.cpp"
                          var $incdec_ptr883=(($467+1)&4294967295); //@line 3138 "translate.cpp"
                          $pn=$incdec_ptr883; //@line 3138 "translate.cpp"
                          HEAP8[($467)]=$conv882; //@line 3138 "translate.cpp"
                          ; //@line 3139 "translate.cpp"
                        }
  
                        var $468=$pn; //@line 3140 "translate.cpp"
                        var $incdec_ptr885=(($468+1)&4294967295); //@line 3140 "translate.cpp"
                        $pn=$incdec_ptr885; //@line 3140 "translate.cpp"
                        HEAP8[($468)]=32; //@line 3140 "translate.cpp"
                        var $469=$tr_addr; //@line 3141 "translate.cpp"
                        var $langopts886=(($469)&4294967295); //@line 3141 "translate.cpp"
                        var $break_numbers887=(($langopts886+192)&4294967295); //@line 3141 "translate.cpp"
                        var $470=HEAP32[(($break_numbers887)>>2)]; //@line 3141 "translate.cpp"
                        var $471=$nx; //@line 3141 "translate.cpp"
                        var $sub888=((($471)-1)&4294967295); //@line 3141 "translate.cpp"
                        var $shl889=1 << ($sub888); //@line 3141 "translate.cpp"
                        var $and890=($shl889) & ($470); //@line 3141 "translate.cpp"
                        var $tobool891=((($and890))|0)!=0; //@line 3141 "translate.cpp"
                        if ($tobool891) { __label__ = 275;; } else { __label__ = 276;; } //@line 3141 "translate.cpp"
                        if (__label__ == 275) {
  
                          var $472=$pn; //@line 3144 "translate.cpp"
                          var $incdec_ptr893=(($472+1)&4294967295); //@line 3144 "translate.cpp"
                          $pn=$incdec_ptr893; //@line 3144 "translate.cpp"
                          HEAP8[($472)]=48; //@line 3144 "translate.cpp"
                          var $473=$pn; //@line 3145 "translate.cpp"
                          var $incdec_ptr894=(($473+1)&4294967295); //@line 3145 "translate.cpp"
                          $pn=$incdec_ptr894; //@line 3145 "translate.cpp"
                          HEAP8[($473)]=48; //@line 3145 "translate.cpp"
                          ; //@line 3146 "translate.cpp"
                        }
  
                        var $474=$tr_addr; //@line 3147 "translate.cpp"
                        var $langopts896=(($474)&4294967295); //@line 3147 "translate.cpp"
                        var $break_numbers897=(($langopts896+192)&4294967295); //@line 3147 "translate.cpp"
                        var $475=HEAP32[(($break_numbers897)>>2)]; //@line 3147 "translate.cpp"
                        var $476=$nx; //@line 3147 "translate.cpp"
                        var $sub898=((($476)-2)&4294967295); //@line 3147 "translate.cpp"
                        var $shl899=1 << ($sub898); //@line 3147 "translate.cpp"
                        var $and900=($shl899) & ($475); //@line 3147 "translate.cpp"
                        var $tobool901=((($and900))|0)!=0; //@line 3147 "translate.cpp"
                        if (!($tobool901)) { __label__ = 267;continue $while_cond849$374; } //@line 3147 "translate.cpp"
  
                        var $477=$pn; //@line 3150 "translate.cpp"
                        var $incdec_ptr903=(($477+1)&4294967295); //@line 3150 "translate.cpp"
                        $pn=$incdec_ptr903; //@line 3150 "translate.cpp"
                        HEAP8[($477)]=48; //@line 3150 "translate.cpp"
                        __label__ = 267;continue $while_cond849$374; //@line 3151 "translate.cpp"
                      }
  
                      var $478=$pw; //@line 3154 "translate.cpp"
                      var $incdec_ptr907=(($478+-1)&4294967295); //@line 3154 "translate.cpp"
                      $pw=$incdec_ptr907; //@line 3154 "translate.cpp"
                      var $479=$nw; //@line 3155 "translate.cpp"
                      var $arrayidx908=(($num_wtab+$479*12)&4294967295); //@line 3155 "translate.cpp"
                      var $480=$arrayidx908; //@line 3155 "translate.cpp"
                      var $481=$ix; //@line 3155 "translate.cpp"
                      var $arrayidx909=(($words+$481*12)&4294967295); //@line 3155 "translate.cpp"
                      var $482=$arrayidx909; //@line 3155 "translate.cpp"
                      var $src$, $dest$, $stop$, $stop4$, $fast$;
                      $src$ = $482;
                      $dest$ = $480;
                      $stop$ = $src$ + 24;
                      if (($dest$%4) == ($src$%4) && 24 > 8) {
                        while ($src$%4 !== 0 && $src$ < $stop$) {
                          ; HEAP8[$dest$++] = HEAP8[$src$++];
                        }
                        $src$ >>= 2;
                        $dest$ >>= 2;
                        $stop4$ = $stop$ >> 2;
                        while ($src$ < $stop4$) {
                      ;;;    HEAP32[$dest$++] = HEAP32[$src$++];
                        }
                        $src$ <<= 2;
                        $dest$ <<= 2;
                      }while ($src$ < $stop$) {
                        ; HEAP8[$dest$++] = HEAP8[$src$++];
                      }; //@line 3155 "translate.cpp"
                      $j=1; //@line 3157 "translate.cpp"
                      var $483=$j; //@line 3157 "translate.cpp"
                      var $484=$nw; //@line 3157 "translate.cpp"
                      var $cmp91134=((($483))|0) <= ((($484))|0); //@line 3157 "translate.cpp"
                      if ($cmp91134) { __label__ = 279;; } else { __label__ = 280;; } //@line 3157 "translate.cpp"
                      $for_body912$$for_end918$390: do { 
                        if (__label__ == 279) {
                          while(1) { 
  
                            var $485=$j; //@line 3159 "translate.cpp"
                            var $arrayidx913=(($num_wtab+$485*12)&4294967295); //@line 3159 "translate.cpp"
                            var $flags914=(($arrayidx913)&4294967295); //@line 3159 "translate.cpp"
                            var $486=HEAP32[(($flags914)>>2)]; //@line 3159 "translate.cpp"
                            var $and915=($486) & -262209; //@line 3159 "translate.cpp"
                            HEAP32[(($flags914)>>2)]=$and915; //@line 3159 "translate.cpp"
                            var $487=$j; //@line 3157 "translate.cpp"
                            var $inc917=((($487)+1)&4294967295); //@line 3157 "translate.cpp"
                            $j=$inc917; //@line 3157 "translate.cpp"
                            var $488=$j; //@line 3157 "translate.cpp"
                            var $489=$nw; //@line 3157 "translate.cpp"
                            var $cmp911=((($488))|0) <= ((($489))|0); //@line 3157 "translate.cpp"
                            if ($cmp911) { __label__ = 279;continue ; } else { __label__ = 280;break $for_body912$$for_end918$390; } //@line 3157 "translate.cpp"
                          }
                        }
                      } while(0);
  
                      var $490=$pn; //@line 3163 "translate.cpp"
                      var $491=$pw; //@line 3163 "translate.cpp"
                      var $src$, $dest$, $stop$, $stop4$, $fast$;
                      $src$ = $491;
                      $dest$ = $490;
                      $stop$ = $src$ + 16;
                      if (($dest$%4) == ($src$%4) && 16 > 8) {
                        while ($src$%4 !== 0 && $src$ < $stop$) {
                          ; HEAP8[$dest$++] = HEAP8[$src$++];
                        }
                        $src$ >>= 2;
                        $dest$ >>= 2;
                        $stop4$ = $stop$ >> 2;
                        while ($src$ < $stop4$) {
                      ;;;    HEAP32[$dest$++] = HEAP32[$src$++];
                        }
                        $src$ <<= 2;
                        $dest$ <<= 2;
                      }while ($src$ < $stop$) {
                        ; HEAP8[$dest$++] = HEAP8[$src$++];
                      }; //@line 3163 "translate.cpp"
                      var $492=$pn; //@line 3164 "translate.cpp"
                      var $arrayidx919=(($492+16)&4294967295); //@line 3164 "translate.cpp"
                      HEAP8[($arrayidx919)]=0; //@line 3164 "translate.cpp"
                      $nw=0; //@line 3165 "translate.cpp"
                      $pw=$arrayidx920; //@line 3167 "translate.cpp"
                      var $493=$pw; //@line 3167 "translate.cpp"
                      var $494=$pn; //@line 3167 "translate.cpp"
                      var $cmp92235=((($493))>>>0) < ((($494))>>>0); //@line 3167 "translate.cpp"
                      if (!($cmp92235)) { __label__ = 293;break $if_then835$$if_else939$368; } //@line 3167 "translate.cpp"
                      $for_body923$394: while(1) { 
  
                        var $495=$tr_addr; //@line 3170 "translate.cpp"
                        var $496=$pw; //@line 3170 "translate.cpp"
                        var $497=$nw; //@line 3170 "translate.cpp"
                        var $inc924=((($497)+1)&4294967295); //@line 3170 "translate.cpp"
                        $nw=$inc924; //@line 3170 "translate.cpp"
                        var $arrayidx925=(($num_wtab+$497*12)&4294967295); //@line 3170 "translate.cpp"
                        var $498=$ix; //@line 3170 "translate.cpp"
                        var $arrayidx926=(($words+$498*12)&4294967295); //@line 3170 "translate.cpp"
                        var $pre_pause927=(($arrayidx926+8)&4294967295); //@line 3170 "translate.cpp"
                        var $499=HEAPU8[($pre_pause927)]; //@line 3170 "translate.cpp"
                        var $conv928=((($499))&255); //@line 3170 "translate.cpp"
                        var $call929=__ZL14TranslateWord2P10TranslatorPcP8WORD_TABii($495, $496, $arrayidx925, $conv928, 0); //@line 3170 "translate.cpp"
                        $dict_flags=$call929; //@line 3170 "translate.cpp"
                        ; //@line 3171 "translate.cpp"
                        while(1) { 
  
                          var $500=$pw; //@line 3171 "translate.cpp"
                          var $incdec_ptr931=(($500+1)&4294967295); //@line 3171 "translate.cpp"
                          $pw=$incdec_ptr931; //@line 3171 "translate.cpp"
                          var $501=HEAP8[($500)]; //@line 3171 "translate.cpp"
                          var $conv932=(tempInt=(($501)),(tempInt>=128?tempInt-256:tempInt)); //@line 3171 "translate.cpp"
                          var $cmp933=((($conv932))|0)!=32; //@line 3171 "translate.cpp"
                          if ($cmp933) { __label__ = 282;continue ; } else { __label__ = 283;break ; } //@line 3171 "translate.cpp"
                        }
  
                        var $502=$ix; //@line 3172 "translate.cpp"
                        var $arrayidx936=(($words+$502*12)&4294967295); //@line 3172 "translate.cpp"
                        var $pre_pause937=(($arrayidx936+8)&4294967295); //@line 3172 "translate.cpp"
                        HEAP8[($pre_pause937)]=0; //@line 3172 "translate.cpp"
                        var $503=$pw; //@line 3167 "translate.cpp"
                        var $504=$pn; //@line 3167 "translate.cpp"
                        var $cmp922=((($503))>>>0) < ((($504))>>>0); //@line 3167 "translate.cpp"
                        if ($cmp922) { __label__ = 281;continue $for_body923$394; } else { __label__ = 293;break $if_then835$$if_else939$368; } //@line 3167 "translate.cpp"
                      }
                    }
                    else if (__label__ == 284) {
  
                      HEAP32[((_pre_pause)>>2)]=0; //@line 3177 "translate.cpp"
                      var $505=$tr_addr; //@line 3178 "translate.cpp"
                      var $506=$word; //@line 3178 "translate.cpp"
                      var $507=$ix; //@line 3178 "translate.cpp"
                      var $arrayidx940=(($words+$507*12)&4294967295); //@line 3178 "translate.cpp"
                      var $508=$ix; //@line 3178 "translate.cpp"
                      var $arrayidx941=(($words+$508*12)&4294967295); //@line 3178 "translate.cpp"
                      var $pre_pause942=(($arrayidx941+8)&4294967295); //@line 3178 "translate.cpp"
                      var $509=HEAPU8[($pre_pause942)]; //@line 3178 "translate.cpp"
                      var $conv943=((($509))&255); //@line 3178 "translate.cpp"
                      var $510=$ix; //@line 3178 "translate.cpp"
                      var $add944=((($510)+1)&4294967295); //@line 3178 "translate.cpp"
                      var $arrayidx945=(($words+$add944*12)&4294967295); //@line 3178 "translate.cpp"
                      var $pre_pause946=(($arrayidx945+8)&4294967295); //@line 3178 "translate.cpp"
                      var $511=HEAPU8[($pre_pause946)]; //@line 3178 "translate.cpp"
                      var $conv947=((($511))&255); //@line 3178 "translate.cpp"
                      var $call948=__ZL14TranslateWord2P10TranslatorPcP8WORD_TABii($505, $506, $arrayidx940, $conv943, $conv947); //@line 3178 "translate.cpp"
                      $dict_flags=$call948; //@line 3178 "translate.cpp"
                      var $512=HEAP32[((_pre_pause)>>2)]; //@line 3180 "translate.cpp"
                      var $513=$ix; //@line 3180 "translate.cpp"
                      var $add949=((($513)+1)&4294967295); //@line 3180 "translate.cpp"
                      var $arrayidx950=(($words+$add949*12)&4294967295); //@line 3180 "translate.cpp"
                      var $pre_pause951=(($arrayidx950+8)&4294967295); //@line 3180 "translate.cpp"
                      var $514=HEAPU8[($pre_pause951)]; //@line 3180 "translate.cpp"
                      var $conv952=((($514))&255); //@line 3180 "translate.cpp"
                      var $cmp953=((($512))|0) > ((($conv952))|0); //@line 3180 "translate.cpp"
                      if ($cmp953) { __label__ = 285;; } else { __label__ = 286;; } //@line 3180 "translate.cpp"
                      if (__label__ == 285) {
  
                        var $515=HEAP32[((_pre_pause)>>2)]; //@line 3182 "translate.cpp"
                        var $conv955=((($515)) & 255); //@line 3182 "translate.cpp"
                        var $516=$ix; //@line 3182 "translate.cpp"
                        var $add956=((($516)+1)&4294967295); //@line 3182 "translate.cpp"
                        var $arrayidx957=(($words+$add956*12)&4294967295); //@line 3182 "translate.cpp"
                        var $pre_pause958=(($arrayidx957+8)&4294967295); //@line 3182 "translate.cpp"
                        HEAP8[($pre_pause958)]=$conv955; //@line 3182 "translate.cpp"
                        HEAP32[((_pre_pause)>>2)]=0; //@line 3183 "translate.cpp"
                        ; //@line 3184 "translate.cpp"
                      }
  
                      var $517=$dict_flags; //@line 3186 "translate.cpp"
                      var $and960=($517) & 65536; //@line 3186 "translate.cpp"
                      var $tobool961=((($and960))|0)!=0; //@line 3186 "translate.cpp"
                      if ($tobool961) { __label__ = 287;; } else { __label__ = 289;; } //@line 3186 "translate.cpp"
                      $if_then962$$if_end975$403: do { 
                        if (__label__ == 287) {
  
                          var $518=$word; //@line 3189 "translate.cpp"
                          $pw=$518; //@line 3189 "translate.cpp"
                          var $519=$pw; //@line 3189 "translate.cpp"
                          var $520=HEAP8[($519)]; //@line 3189 "translate.cpp"
                          var $conv96427=(tempInt=(($520)),(tempInt>=128?tempInt-256:tempInt)); //@line 3189 "translate.cpp"
                          var $cmp96528=((($conv96427))|0)!=32; //@line 3189 "translate.cpp"
                          if (!($cmp96528)) { __label__ = 289;break $if_then962$$if_end975$403; } //@line 3189 "translate.cpp"
                          while(1) { 
  
                            var $dest$, $stop$, $stop4$, $fast$, $value4$;
                            $dest$ = $arraydecay967;
                            $stop$ = $dest$ + 9;
                            $value4$ = 32;
                            if ($value4$ < 0) $value4$ += 256;
                            $value4$ = $value4$ + ($value4$<<8) + ($value4$<<16) + ($value4$*16777216);
                            while ($dest$%4 !== 0 && $dest$ < $stop$) {
                              ;; HEAP8[$dest$++] = 32;
                            }
                            $dest$ >>= 2;
                            $stop4$ = $stop$ >> 2;
                            while ($dest$ < $stop4$) {
                            ;;;;;;;  HEAP32[$dest$++] = $value4$;
                            }
                            $dest$ <<= 2;
                            while ($dest$ < $stop$) {
                              ;; HEAP8[$dest$++] = 32;
                            }; //@line 3191 "translate.cpp"
                            var $521=$pw; //@line 3192 "translate.cpp"
                            var $call968=__Z7utf8_inPiPKc($c_temp, $521); //@line 3192 "translate.cpp"
                            $nx=$call968; //@line 3192 "translate.cpp"
                            var $522=$pw; //@line 3193 "translate.cpp"
                            var $523=$nx; //@line 3193 "translate.cpp"
                            var $src$, $dest$, $stop$, $stop4$, $fast$;
                            $src$ = $522;
                            $dest$ = $arrayidx969;
                            $stop$ = $src$ + $523;
                            if (($dest$%4) == ($src$%4) && $523 > 8) {
                              while ($src$%4 !== 0 && $src$ < $stop$) {
                                ; HEAP8[$dest$++] = HEAP8[$src$++];
                              }
                              $src$ >>= 2;
                              $dest$ >>= 2;
                              $stop4$ = $stop$ >> 2;
                              while ($src$ < $stop4$) {
                            ;;;    HEAP32[$dest$++] = HEAP32[$src$++];
                              }
                              $src$ <<= 2;
                              $dest$ <<= 2;
                            }while ($src$ < $stop$) {
                              ; HEAP8[$dest$++] = HEAP8[$src$++];
                            }; //@line 3193 "translate.cpp"
                            var $524=$tr_addr; //@line 3194 "translate.cpp"
                            var $525=$ix; //@line 3194 "translate.cpp"
                            var $arrayidx971=(($words+$525*12)&4294967295); //@line 3194 "translate.cpp"
                            var $call972=__ZL14TranslateWord2P10TranslatorPcP8WORD_TABii($524, $arrayidx970, $arrayidx971, 0, 0); //@line 3194 "translate.cpp"
                            var $526=$nx; //@line 3195 "translate.cpp"
                            var $527=$pw; //@line 3195 "translate.cpp"
                            var $add_ptr973=(($527+$526)&4294967295); //@line 3195 "translate.cpp"
                            $pw=$add_ptr973; //@line 3195 "translate.cpp"
                            var $528=$pw; //@line 3189 "translate.cpp"
                            var $529=HEAP8[($528)]; //@line 3189 "translate.cpp"
                            var $conv964=(tempInt=(($529)),(tempInt>=128?tempInt-256:tempInt)); //@line 3189 "translate.cpp"
                            var $cmp965=((($conv964))|0)!=32; //@line 3189 "translate.cpp"
                            if ($cmp965) { __label__ = 288;continue ; } else { __label__ = 289;break $if_then962$$if_end975$403; } //@line 3189 "translate.cpp"
                          }
                        }
                      } while(0);
  
                      var $530=$dict_flags; //@line 3199 "translate.cpp"
                      var $and976=($530) & 50331648; //@line 3199 "translate.cpp"
                      var $tobool977=((($and976))|0)!=0; //@line 3199 "translate.cpp"
                      if (!($tobool977)) { __label__ = 293;break $if_then835$$if_else939$368; } //@line 3199 "translate.cpp"
  
                      var $531=$ix; //@line 3199 "translate.cpp"
                      var $532=$word_count; //@line 3199 "translate.cpp"
                      var $sub979=((($532)-1)&4294967295); //@line 3199 "translate.cpp"
                      var $cmp980=((($531))|0)==((($sub979))|0); //@line 3199 "translate.cpp"
                      if (!($cmp980)) { __label__ = 293;break $if_then835$$if_else939$368; } //@line 3199 "translate.cpp"
  
                      var $533=$terminator; //@line 3199 "translate.cpp"
                      var $and982=($533) & 4194304; //@line 3199 "translate.cpp"
                      var $tobool983=((($and982))|0)!=0; //@line 3199 "translate.cpp"
                      if (!($tobool983)) { __label__ = 293;break $if_then835$$if_else939$368; } //@line 3199 "translate.cpp"
  
                      $clause_pause=10; //@line 3202 "translate.cpp"
                      $tone=4; //@line 3203 "translate.cpp"
                      ; //@line 3204 "translate.cpp"
                    }
                  } while(0);
  
                  var $534=$dict_flags; //@line 3207 "translate.cpp"
                  var $and987=($534) & 128; //@line 3207 "translate.cpp"
                  var $tobool988=((($and987))|0)!=0; //@line 3207 "translate.cpp"
                  if (!($tobool988)) { __label__ = 295;break $if_end768$$for_inc992$344; } //@line 3207 "translate.cpp"
  
                  var $535=HEAP32[((_dictionary_skipwords)>>2)]; //@line 3209 "translate.cpp"
                  var $536=$ix; //@line 3209 "translate.cpp"
                  var $add990=((($536)+($535))&4294967295); //@line 3209 "translate.cpp"
                  $ix=$add990; //@line 3209 "translate.cpp"
                  ; //@line 3210 "translate.cpp"
                }
              } while(0);
  
              var $537=$ix; //@line 3058 "translate.cpp"
              var $inc993=((($537)+1)&4294967295); //@line 3058 "translate.cpp"
              $ix=$inc993; //@line 3058 "translate.cpp"
              var $538=$ix; //@line 3058 "translate.cpp"
              var $539=$word_count; //@line 3058 "translate.cpp"
              var $cmp756=((($538))|0) < ((($539))|0); //@line 3058 "translate.cpp"
              if ($cmp756) { __label__ = 243;continue $for_body757$337; } else { __label__ = 296;break $for_body757_lr_ph$$for_end994$335; } //@line 3058 "translate.cpp"
            }
          }
        } while(0);
  
        $ix=0; //@line 3213 "translate.cpp"
        var $540=HEAP32[((_n_ph_list2)>>2)]; //@line 3217 "translate.cpp"
        __lastLabel__ = 296; ; //@line 3213 "translate.cpp"
        $for_body997$415: while(1) { 
  
          var $541=__lastLabel__ == 296 ? $540 : ($549);
          var $542=$ix; //@line 3217 "translate.cpp"
          var $add998=((($542)+($541))&4294967295); //@line 3217 "translate.cpp"
          var $arrayidx999=((_ph_list2+$add998*8)&4294967295); //@line 3217 "translate.cpp"
          $p2=$arrayidx999; //@line 3217 "translate.cpp"
          var $543=$p2; //@line 3218 "translate.cpp"
          var $phcode=(($543)&4294967295); //@line 3218 "translate.cpp"
          HEAP8[($phcode)]=9; //@line 3218 "translate.cpp"
          var $544=$p2; //@line 3219 "translate.cpp"
          var $stresslevel=(($544+1)&4294967295); //@line 3219 "translate.cpp"
          HEAP8[($stresslevel)]=0; //@line 3219 "translate.cpp"
          var $545=HEAP32[(($source_index)>>2)]; //@line 3220 "translate.cpp"
          var $conv1000=((($545)) & 65535); //@line 3220 "translate.cpp"
          var $546=$p2; //@line 3220 "translate.cpp"
          var $sourceix1001=(($546+6)&4294967295); //@line 3220 "translate.cpp"
          HEAP16[(($sourceix1001)>>1)]=$conv1000; //@line 3220 "translate.cpp"
          var $547=$p2; //@line 3221 "translate.cpp"
          var $synthflags=(($547+4)&4294967295); //@line 3221 "translate.cpp"
          HEAP16[(($synthflags)>>1)]=0; //@line 3221 "translate.cpp"
          var $548=$ix; //@line 3213 "translate.cpp"
          var $inc1003=((($548)+1)&4294967295); //@line 3213 "translate.cpp"
          $ix=$inc1003; //@line 3213 "translate.cpp"
          var $cmp996=((($inc1003))|0) < 2; //@line 3213 "translate.cpp"
          var $549=HEAP32[((_n_ph_list2)>>2)]; //@line 3217 "translate.cpp"
          if ($cmp996) { __lastLabel__ = 297; __label__ = 297;continue $for_body997$415; } else { __lastLabel__ = 297; __label__ = 298;break $for_body997$415; } //@line 3213 "translate.cpp"
        }
  
        var $add1005=((($549)+2)&4294967295); //@line 3223 "translate.cpp"
        HEAP32[((_n_ph_list2)>>2)]=$add1005; //@line 3223 "translate.cpp"
        var $550=HEAP32[((_count_words)>>2)]; //@line 3225 "translate.cpp"
        var $cmp1006=((($550))|0)==0; //@line 3225 "translate.cpp"
        if ($cmp1006) { __label__ = 299;; } else { __label__ = 300;; } //@line 3225 "translate.cpp"
        if (__label__ == 299) {
  
          $clause_pause=0; //@line 3227 "translate.cpp"
          ; //@line 3228 "translate.cpp"
        }
  
        var $call1009=__Z3Eofv(); //@line 3229 "translate.cpp"
        var $tobool1010=((($call1009))|0)!=0; //@line 3229 "translate.cpp"
        if ($tobool1010) { __label__ = 301;; } else { __label__ = 303;; } //@line 3229 "translate.cpp"
        $land_lhs_true1011$$if_end1016$421: do { 
          if (__label__ == 301) {
  
            var $551=$word_count; //@line 3229 "translate.cpp"
            var $cmp1012=((($551))|0)==0; //@line 3229 "translate.cpp"
            var $552=HEAP32[((_option_endpause)>>2)]; //@line 3229 "translate.cpp"
            var $cmp1014=((($552))|0)==0; //@line 3229 "translate.cpp"
            var $or_cond11=($cmp1012) | ($cmp1014); //@line 3229 "translate.cpp"
            if (!($or_cond11)) { __label__ = 303;break $land_lhs_true1011$$if_end1016$421; } //@line 3229 "translate.cpp"
  
            $clause_pause=10; //@line 3231 "translate.cpp"
            ; //@line 3232 "translate.cpp"
          }
        } while(0);
  
        var $553=$tr_addr; //@line 3234 "translate.cpp"
        var $554=$clause_pause; //@line 3234 "translate.cpp"
        var $555=HEAP32[((_new_sentence)>>2)]; //@line 3234 "translate.cpp"
        __Z15MakePhonemeListP10Translatorii($553, $554, $555); //@line 3234 "translate.cpp"
        var $556=$embedded_count; //@line 3236 "translate.cpp"
        var $tobool1017=((($556))|0)!=0; //@line 3236 "translate.cpp"
        if ($tobool1017) { __label__ = 304;; } else { __label__ = 305;; } //@line 3236 "translate.cpp"
        if (__label__ == 304) {
  
          var $557=HEAP32[((_n_phoneme_list)>>2)]; //@line 3238 "translate.cpp"
          var $sub1019=((($557)-2)&4294967295); //@line 3238 "translate.cpp"
          var $arrayidx1020=((_phoneme_list+$sub1019*24)&4294967295); //@line 3238 "translate.cpp"
          var $synthflags1021=(($arrayidx1020+4)&4294967295); //@line 3238 "translate.cpp"
          HEAP16[(($synthflags1021)>>1)]=2; //@line 3238 "translate.cpp"
          var $558=HEAP32[((__ZL11embedded_ix)>>2)]; //@line 3239 "translate.cpp"
          var $sub1022=((($558)-1)&4294967295); //@line 3239 "translate.cpp"
          var $arrayidx1023=((_embedded_list+$sub1022*4)&4294967295); //@line 3239 "translate.cpp"
          var $559=HEAP32[(($arrayidx1023)>>2)]; //@line 3239 "translate.cpp"
          var $or1024=($559) | 128; //@line 3239 "translate.cpp"
          HEAP32[(($arrayidx1023)>>2)]=$or1024; //@line 3239 "translate.cpp"
          ; //@line 3240 "translate.cpp"
        }
  
        var $560=$tone; //@line 3245 "translate.cpp"
        var $561=$tone_out_addr; //@line 3245 "translate.cpp"
        HEAP32[(($561)>>2)]=$560; //@line 3245 "translate.cpp"
        HEAP32[((_new_sentence)>>2)]=0; //@line 3247 "translate.cpp"
        var $562=$terminator; //@line 3248 "translate.cpp"
        var $and1026=($562) & 524288; //@line 3248 "translate.cpp"
        var $tobool1027=((($and1026))|0)!=0; //@line 3248 "translate.cpp"
        if ($tobool1027) { __label__ = 306;; } else { __label__ = 307;; } //@line 3248 "translate.cpp"
        if (__label__ == 306) {
  
          HEAP32[((_new_sentence)>>2)]=1; //@line 3250 "translate.cpp"
          ; //@line 3251 "translate.cpp"
        }
  
        var $563=$voice_change_addr; //@line 3254 "translate.cpp"
        var $cmp1030=((($563))|0)!=0; //@line 3254 "translate.cpp"
        if ($cmp1030) { __label__ = 308;; } else { __label__ = 311;; } //@line 3254 "translate.cpp"
        if (__label__ == 308) {
  
          var $564=$terminator; //@line 3257 "translate.cpp"
          var $and1032=($564) & 131072; //@line 3257 "translate.cpp"
          var $tobool1033=((($and1032))|0)!=0; //@line 3257 "translate.cpp"
          var $565=$voice_change_addr; //@line 3258 "translate.cpp"
          if ($tobool1033) { __label__ = 309;; } else { __label__ = 310;; } //@line 3257 "translate.cpp"
          if (__label__ == 309) {
  
            HEAP32[(($565)>>2)]=((__ZZ15TranslateClauseP10TranslatorP7__sFILEPKvPiPPcE17voice_change_name)&4294967295); //@line 3258 "translate.cpp"
            ; //@line 3258 "translate.cpp"
          }
          else if (__label__ == 310) {
  
            HEAP32[(($565)>>2)]=0; //@line 3260 "translate.cpp"
            ;
          }
        }
  
        var $call1038=__Z3Eofv(); //@line 3263 "translate.cpp"
        var $tobool1039=((($call1038))|0)!=0; //@line 3263 "translate.cpp"
        if ($tobool1039) { __label__ = 313;; } else { __label__ = 312;; } //@line 3263 "translate.cpp"
        $if_then1042$$lor_lhs_false1040$437: do { 
          if (__label__ == 312) {
  
            var $566=$vp_input_addr; //@line 3263 "translate.cpp"
            var $cmp1041=((($566))|0)==0; //@line 3263 "translate.cpp"
            if ($cmp1041) { __label__ = 313;break $if_then1042$$lor_lhs_false1040$437; } //@line 3263 "translate.cpp"
  
            var $567=HEAP32[((_option_multibyte)>>2)]; //@line 3266 "translate.cpp"
            var $cmp1044=((($567))|0)==3; //@line 3266 "translate.cpp"
            if ($cmp1044) { __label__ = 315;; } else { __label__ = 316;; } //@line 3266 "translate.cpp"
            if (__label__ == 315) {
  
              var $568=HEAP32[((_p_wchar_input)>>2)]; //@line 3267 "translate.cpp"
              var $569=$568; //@line 3267 "translate.cpp"
              $retval=$569; //@line 3267 "translate.cpp"
              __label__ = 317;break $if_then$$if_end$2; //@line 3267 "translate.cpp"
            }
            else if (__label__ == 316) {
  
              var $570=HEAP32[((_p_textinput)>>2)]; //@line 3269 "translate.cpp"
              $retval=$570; //@line 3269 "translate.cpp"
              __label__ = 317;break $if_then$$if_end$2; //@line 3269 "translate.cpp"
            }
          }
        } while(0);
  
        $retval=0; //@line 3264 "translate.cpp"
        ; //@line 3264 "translate.cpp"
      }
    } while(0);
  
    var $571=$retval; //@line 3270 "translate.cpp"
    STACKTOP = __stackBase__;
    return $571; //@line 3270 "translate.cpp"
    return null;
  }
  

  function __ZL9IsDigit09j($c) {
    ;
    var __label__;
  
    var $retval;
    var $c_addr;
    $c_addr=$c;
    var $0=$c_addr; //@line 415 "translate.cpp"
    var $cmp=((($0))>>>0) >= 48; //@line 415 "translate.cpp"
    var $1=$c_addr; //@line 415 "translate.cpp"
    var $cmp1=((($1))>>>0) <= 57; //@line 415 "translate.cpp"
    var $or_cond=($cmp) & ($cmp1); //@line 415 "translate.cpp"
    if ($or_cond) { __label__ = 1;; } else { __label__ = 2;; } //@line 415 "translate.cpp"
    if (__label__ == 1) {
  
      $retval=1; //@line 416 "translate.cpp"
      ; //@line 416 "translate.cpp"
    }
    else if (__label__ == 2) {
  
      $retval=0; //@line 417 "translate.cpp"
      ; //@line 417 "translate.cpp"
    }
  
    var $2=$retval; //@line 418 "translate.cpp"
    ;
    return $2; //@line 418 "translate.cpp"
    return null;
  }
  

  function __ZL15EmbeddedCommandRj($source_index) {
    ;
    var __label__;
    var __lastLabel__ = null;
  
    var $retval;
    var $source_index_addr;
    var $commands;
    var $value;
    var $sign;
    var $c;
    var $p;
    var $cmd;
    $source_index_addr=$source_index;
    $commands=((__str12555)&4294967295); //@line 2131 "translate.cpp"
    $value=-1; //@line 2132 "translate.cpp"
    $sign=0; //@line 2133 "translate.cpp"
    var $0=$source_index_addr; //@line 2138 "translate.cpp"
    var $1=HEAP32[(($0)>>2)]; //@line 2138 "translate.cpp"
    var $arrayidx=((__ZL6source+$1)&4294967295); //@line 2138 "translate.cpp"
    var $2=HEAP8[($arrayidx)]; //@line 2138 "translate.cpp"
    $c=$2; //@line 2138 "translate.cpp"
    var $3=$c; //@line 2139 "translate.cpp"
    var $conv=((($3))&255); //@line 2139 "translate.cpp"
    var $cmp=((($conv))|0)==43; //@line 2139 "translate.cpp"
    if ($cmp) { __label__ = 1;; } else { __label__ = 2;; } //@line 2139 "translate.cpp"
    $if_then$$if_else$2: do { 
      if (__label__ == 1) {
  
        $sign=64; //@line 2141 "translate.cpp"
        var $4=$source_index_addr; //@line 2142 "translate.cpp"
        var $5=HEAP32[(($4)>>2)]; //@line 2142 "translate.cpp"
        var $inc=((($5)+1)&4294967295); //@line 2142 "translate.cpp"
        HEAP32[(($4)>>2)]=$inc; //@line 2142 "translate.cpp"
        ; //@line 2143 "translate.cpp"
      }
      else if (__label__ == 2) {
  
        var $6=$c; //@line 2145 "translate.cpp"
        var $conv1=((($6))&255); //@line 2145 "translate.cpp"
        var $cmp2=((($conv1))|0)==45; //@line 2145 "translate.cpp"
        if (!($cmp2)) { __label__ = 4;break $if_then$$if_else$2; } //@line 2145 "translate.cpp"
  
        $sign=96; //@line 2147 "translate.cpp"
        var $7=$source_index_addr; //@line 2148 "translate.cpp"
        var $8=HEAP32[(($7)>>2)]; //@line 2148 "translate.cpp"
        var $inc4=((($8)+1)&4294967295); //@line 2148 "translate.cpp"
        HEAP32[(($7)>>2)]=$inc4; //@line 2148 "translate.cpp"
        ; //@line 2149 "translate.cpp"
      }
    } while(0);
  
    var $9=$source_index_addr; //@line 2151 "translate.cpp"
    var $10=HEAP32[(($9)>>2)]; //@line 2151 "translate.cpp"
    var $arrayidx6=((__ZL6source+$10)&4294967295); //@line 2151 "translate.cpp"
    var $11=HEAP8[($arrayidx6)]; //@line 2151 "translate.cpp"
    var $conv7=(tempInt=(($11)),(tempInt>=128?tempInt-256:tempInt)); //@line 2151 "translate.cpp"
    var $isdigittmp=((($conv7)-48)&4294967295); //@line 2151 "translate.cpp"
    var $isdigit=((($isdigittmp))>>>0) < 10; //@line 2151 "translate.cpp"
    var $call=((($isdigit))&1); //@line 2151 "translate.cpp"
    var $tobool=((($call))|0)!=0; //@line 2151 "translate.cpp"
    if ($tobool) { __label__ = 5;; } else { __label__ = 7;; } //@line 2151 "translate.cpp"
    $if_then8$$if_end16$7: do { 
      if (__label__ == 5) {
  
        var $12=$source_index_addr; //@line 2153 "translate.cpp"
        var $13=HEAP32[(($12)>>2)]; //@line 2153 "translate.cpp"
        var $arrayidx9=((__ZL6source+$13)&4294967295); //@line 2153 "translate.cpp"
        var $call10=_atoi($arrayidx9); //@line 2153 "translate.cpp"
        $value=$call10; //@line 2153 "translate.cpp"
        var $14=$source_index_addr; //@line 2154 "translate.cpp"
        var $15=HEAP32[(($14)>>2)]; //@line 2154 "translate.cpp"
        var $arrayidx113=((__ZL6source+$15)&4294967295); //@line 2154 "translate.cpp"
        var $16=HEAP8[($arrayidx113)]; //@line 2154 "translate.cpp"
        var $conv124=(tempInt=(($16)),(tempInt>=128?tempInt-256:tempInt)); //@line 2154 "translate.cpp"
        var $isdigittmp15=((($conv124)-48)&4294967295); //@line 2154 "translate.cpp"
        var $isdigit26=((($isdigittmp15))>>>0) < 10; //@line 2154 "translate.cpp"
        if (!($isdigit26)) { __label__ = 7;break $if_then8$$if_end16$7; } //@line 2154 "translate.cpp"
        while(1) { 
  
          var $17=$source_index_addr; //@line 2155 "translate.cpp"
          var $18=HEAP32[(($17)>>2)]; //@line 2155 "translate.cpp"
          var $inc15=((($18)+1)&4294967295); //@line 2155 "translate.cpp"
          HEAP32[(($17)>>2)]=$inc15; //@line 2155 "translate.cpp"
          var $19=$source_index_addr; //@line 2154 "translate.cpp"
          var $20=HEAP32[(($19)>>2)]; //@line 2154 "translate.cpp"
          var $arrayidx11=((__ZL6source+$20)&4294967295); //@line 2154 "translate.cpp"
          var $21=HEAP8[($arrayidx11)]; //@line 2154 "translate.cpp"
          var $conv12=(tempInt=(($21)),(tempInt>=128?tempInt-256:tempInt)); //@line 2154 "translate.cpp"
          var $isdigittmp1=((($conv12)-48)&4294967295); //@line 2154 "translate.cpp"
          var $isdigit2=((($isdigittmp1))>>>0) < 10; //@line 2154 "translate.cpp"
          var $call13=((($isdigit2))&1); //@line 2154 "translate.cpp"
          var $tobool14=((($call13))|0)!=0; //@line 2154 "translate.cpp"
          if ($tobool14) { __label__ = 6;continue ; } else { __label__ = 7;break $if_then8$$if_end16$7; } //@line 2154 "translate.cpp"
        }
      }
    } while(0);
  
    var $22=$source_index_addr; //@line 2158 "translate.cpp"
    var $23=HEAP32[(($22)>>2)]; //@line 2158 "translate.cpp"
    var $inc17=((($23)+1)&4294967295); //@line 2158 "translate.cpp"
    HEAP32[(($22)>>2)]=$inc17; //@line 2158 "translate.cpp"
    var $arrayidx18=((__ZL6source+$23)&4294967295); //@line 2158 "translate.cpp"
    var $24=HEAP8[($arrayidx18)]; //@line 2158 "translate.cpp"
    $c=$24; //@line 2158 "translate.cpp"
    var $25=HEAP32[((__ZL11embedded_ix)>>2)]; //@line 2159 "translate.cpp"
    var $cmp19=((($25))|0) >= 248; //@line 2159 "translate.cpp"
    if ($cmp19) { __label__ = 8;; } else { __label__ = 9;; } //@line 2159 "translate.cpp"
    if (__label__ == 8) {
  
      $retval=0; //@line 2160 "translate.cpp"
      ; //@line 2160 "translate.cpp"
    }
    else if (__label__ == 9) {
  
      var $26=$commands; //@line 2162 "translate.cpp"
      var $27=$c; //@line 2162 "translate.cpp"
      var $conv22=((($27))&255); //@line 2162 "translate.cpp"
      var $call23=__Z8strchr_wPKci($26, $conv22); //@line 2162 "translate.cpp"
      $p=$call23; //@line 2162 "translate.cpp"
      var $cmp24=((($call23))|0)==0; //@line 2162 "translate.cpp"
      if ($cmp24) { __label__ = 10;; } else { __label__ = 11;; } //@line 2162 "translate.cpp"
      if (__label__ == 10) {
  
        $retval=0; //@line 2163 "translate.cpp"
        ; //@line 2163 "translate.cpp"
      }
      else if (__label__ == 11) {
  
        var $28=$p; //@line 2164 "translate.cpp"
        var $29=$commands; //@line 2164 "translate.cpp"
        var $sub_ptr_lhs_cast=($28); //@line 2164 "translate.cpp"
        var $sub_ptr_rhs_cast=($29); //@line 2164 "translate.cpp"
        var $sub_ptr_sub=((($sub_ptr_lhs_cast)-($sub_ptr_rhs_cast))&4294967295); //@line 2164 "translate.cpp"
        var $add=((($sub_ptr_sub)+1)&4294967295); //@line 2164 "translate.cpp"
        $cmd=$add; //@line 2164 "translate.cpp"
        var $30=$value; //@line 2165 "translate.cpp"
        var $cmp27=((($30))|0)==-1; //@line 2165 "translate.cpp"
        if ($cmp27) { __lastLabel__ = 11; __label__ = 12;; } else { __lastLabel__ = 11; __label__ = 13;; } //@line 2165 "translate.cpp"
        if (__label__ == 12) {
  
          var $31=$cmd; //@line 2167 "translate.cpp"
          var $arrayidx29=((_embedded_default+$31*4)&4294967295); //@line 2167 "translate.cpp"
          var $32=HEAP32[(($arrayidx29)>>2)]; //@line 2167 "translate.cpp"
          $value=$32; //@line 2167 "translate.cpp"
          $sign=0; //@line 2168 "translate.cpp"
          __lastLabel__ = 12; ; //@line 2169 "translate.cpp"
        }
  
        var $33=__lastLabel__ == 12 ? $31 : ($add); //@line 2171 "translate.cpp"
        var $cmp31=((($33))|0)==9; //@line 2171 "translate.cpp"
        if ($cmp31) { __label__ = 14;; } else { __label__ = 15;; } //@line 2171 "translate.cpp"
        if (__label__ == 14) {
  
          var $34=$value; //@line 2173 "translate.cpp"
          HEAP32[((__ZL13option_sayas2)>>2)]=$34; //@line 2173 "translate.cpp"
          HEAP32[((__ZL18count_sayas_digits)>>2)]=0; //@line 2174 "translate.cpp"
          ; //@line 2175 "translate.cpp"
        }
  
        var $35=$cmd; //@line 2176 "translate.cpp"
        var $cmp34=((($35))|0)==13; //@line 2176 "translate.cpp"
        if ($cmp34) { __label__ = 16;; } else { __label__ = 19;; } //@line 2176 "translate.cpp"
        if (__label__ == 16) {
  
          var $36=$value; //@line 2178 "translate.cpp"
          var $cmp36=((($36))|0) >= 3; //@line 2178 "translate.cpp"
          if ($cmp36) { __label__ = 17;; } else { __label__ = 18;; } //@line 2178 "translate.cpp"
          if (__label__ == 17) {
  
            HEAP8[(__ZL13word_emphasis_b)]=1;
            ; //@line 2179 "translate.cpp"
          }
          else if (__label__ == 18) {
  
            HEAP8[(__ZL13word_emphasis_b)]=0;
            ;
          }
        }
  
        var $37=$cmd; //@line 2184 "translate.cpp"
        var $38=$sign; //@line 2184 "translate.cpp"
        var $39=$value; //@line 2184 "translate.cpp"
        var $shl=((($39)*256)&4294967295); //@line 2184 "translate.cpp"
        var $add41=((($38)+($37))&4294967295); //@line 2184 "translate.cpp"
        var $add42=((($add41)+($shl))&4294967295); //@line 2184 "translate.cpp"
        var $40=HEAP32[((__ZL11embedded_ix)>>2)]; //@line 2184 "translate.cpp"
        var $inc43=((($40)+1)&4294967295); //@line 2184 "translate.cpp"
        HEAP32[((__ZL11embedded_ix)>>2)]=$inc43; //@line 2184 "translate.cpp"
        var $arrayidx44=((_embedded_list+$40*4)&4294967295); //@line 2184 "translate.cpp"
        HEAP32[(($arrayidx44)>>2)]=$add42; //@line 2184 "translate.cpp"
        $retval=1; //@line 2185 "translate.cpp"
        ; //@line 2185 "translate.cpp"
      }
    }
  
    var $41=$retval; //@line 2186 "translate.cpp"
    ;
    return $41; //@line 2186 "translate.cpp"
    return null;
  }
  

  function __ZL13TranslateCharP10TranslatorPcijjPi($tr, $ptr, $prev_in, $c, $next_in, $insert) {
    var __stackBase__  = STACKTOP; STACKTOP += 4; _memset(__stackBase__, 0, 4);
    var __label__;
  
    var $retval;
    var $tr_addr;
    var $ptr_addr;
    var $prev_in_addr;
    var $c_addr;
    var $next_in_addr;
    var $insert_addr;
    var $code;
    var $initial;
    var $medial;
    var $final;
    var $next2=__stackBase__;
    $tr_addr=$tr;
    $ptr_addr=$ptr;
    $prev_in_addr=$prev_in;
    $c_addr=$c;
    $next_in_addr=$next_in;
    $insert_addr=$insert;
    var $0=$tr_addr; //@line 2276 "translate.cpp"
    var $translator_name=(($0+280)&4294967295); //@line 2276 "translate.cpp"
    var $1=HEAP32[(($translator_name)>>2)]; //@line 2276 "translate.cpp"
    if ($1 == 24934) {
      __label__ = 1;;
    }
    else if ($1 == 28268) {
      __label__ = 1;;
    }
    else if ($1 == 27503) {
      __label__ = 9;;
    }
    else {
    __label__ = 18;;
    }
    
    $sw_epilog$$sw_bb$$sw_bb17$2: do { 
      if (__label__ == 1) {
  
        var $2=$prev_in_addr; //@line 2283 "translate.cpp"
        var $call=_iswalpha($2); //@line 2283 "translate.cpp"
        var $tobool=((($call))|0)!=0; //@line 2283 "translate.cpp"
        if ($tobool) { __label__ = 18;break $sw_epilog$$sw_bb$$sw_bb17$2; } //@line 2283 "translate.cpp"
  
        var $3=$ptr_addr; //@line 2285 "translate.cpp"
        var $arrayidx=(($3+1)&4294967295); //@line 2285 "translate.cpp"
        var $call1=__Z7utf8_inPiPKc($next2, $arrayidx); //@line 2285 "translate.cpp"
        var $4=$c_addr; //@line 2287 "translate.cpp"
        var $cmp=((($4))|0)==39; //@line 2287 "translate.cpp"
        if (!($cmp)) { __label__ = 18;break $sw_epilog$$sw_bb$$sw_bb17$2; } //@line 2287 "translate.cpp"
  
        var $5=HEAP32[(($next2)>>2)]; //@line 2287 "translate.cpp"
        var $call2=__Z7IsSpacej($5); //@line 2287 "translate.cpp"
        var $tobool3=((($call2))|0)!=0; //@line 2287 "translate.cpp"
        if (!($tobool3)) { __label__ = 18;break $sw_epilog$$sw_bb$$sw_bb17$2; } //@line 2287 "translate.cpp"
  
        var $6=$next_in_addr; //@line 2289 "translate.cpp"
        var $cmp5=((($6))|0)==110; //@line 2289 "translate.cpp"
        if ($cmp5) { __label__ = 5;; } else { __label__ = 7;; } //@line 2289 "translate.cpp"
        $land_lhs_true6$$if_end$7: do { 
          if (__label__ == 5) {
  
            var $7=$tr_addr; //@line 2289 "translate.cpp"
            var $translator_name7=(($7+280)&4294967295); //@line 2289 "translate.cpp"
            var $8=HEAP32[(($translator_name7)>>2)]; //@line 2289 "translate.cpp"
            var $cmp8=((($8))|0)==24934; //@line 2289 "translate.cpp"
            if (!($cmp8)) { __label__ = 7;break $land_lhs_true6$$if_end$7; } //@line 2289 "translate.cpp"
  
            var $9=$ptr_addr; //@line 2292 "translate.cpp"
            var $arrayidx10=(($9)&4294967295); //@line 2292 "translate.cpp"
            HEAP8[($arrayidx10)]=32; //@line 2292 "translate.cpp"
            $retval=601; //@line 2293 "translate.cpp"
            __label__ = 19;break $sw_epilog$$sw_bb$$sw_bb17$2; //@line 2293 "translate.cpp"
          }
        } while(0);
  
        var $10=$next_in_addr; //@line 2295 "translate.cpp"
        var $cmp11=((($10))|0)==110; //@line 2295 "translate.cpp"
        var $11=$next_in_addr; //@line 2295 "translate.cpp"
        var $cmp12=((($11))|0)==116; //@line 2295 "translate.cpp"
        var $or_cond=($cmp11) | ($cmp12); //@line 2295 "translate.cpp"
        if (!($or_cond)) { __label__ = 18;break $sw_epilog$$sw_bb$$sw_bb17$2; } //@line 2295 "translate.cpp"
  
        $retval=601; //@line 2298 "translate.cpp"
        __label__ = 19;break $sw_epilog$$sw_bb$$sw_bb17$2; //@line 2298 "translate.cpp"
      }
      else if (__label__ == 9) {
  
        var $12=$c_addr; //@line 2305 "translate.cpp"
        var $sub=((($12)-44032)&4294967295); //@line 2305 "translate.cpp"
        $code=$sub; //@line 2305 "translate.cpp"
        var $cmp18=((($sub))|0) >= 0; //@line 2305 "translate.cpp"
        var $13=$c_addr; //@line 2305 "translate.cpp"
        var $cmp20=((($13))>>>0) <= 55215; //@line 2305 "translate.cpp"
        var $or_cond1=($cmp18) & ($cmp20); //@line 2305 "translate.cpp"
        if ($or_cond1) { __label__ = 10;; } else { __label__ = 15;; } //@line 2305 "translate.cpp"
        if (__label__ == 10) {
  
          var $14=$code; //@line 2308 "translate.cpp"
          var $div=((((($14))|0)/28)|0); //@line 2308 "translate.cpp"
          var $div22=((((($div))|0)/21)|0); //@line 2308 "translate.cpp"
          $initial=$div22; //@line 2308 "translate.cpp"
          var $15=$code; //@line 2309 "translate.cpp"
          var $div23=((((($15))|0)/28)|0); //@line 2309 "translate.cpp"
          var $rem=((($div23))|0)%21; //@line 2309 "translate.cpp"
          $medial=$rem; //@line 2309 "translate.cpp"
          var $16=$code; //@line 2310 "translate.cpp"
          var $rem24=((($16))|0)%28; //@line 2310 "translate.cpp"
          $final=$rem24; //@line 2310 "translate.cpp"
          var $17=$initial; //@line 2312 "translate.cpp"
          var $cmp25=((($17))|0)==11; //@line 2312 "translate.cpp"
          if ($cmp25) { __label__ = 11;; } else { __label__ = 13;; } //@line 2312 "translate.cpp"
          $if_then26$$if_else$15: do { 
            if (__label__ == 11) {
  
              var $18=$medial; //@line 2315 "translate.cpp"
              var $add=((($18)+4449)&4294967295); //@line 2315 "translate.cpp"
              $c_addr=$add; //@line 2315 "translate.cpp"
              var $19=$final; //@line 2316 "translate.cpp"
              var $cmp27=((($19))|0) > 0; //@line 2316 "translate.cpp"
              if (!($cmp27)) { __label__ = 14;break $if_then26$$if_else$15; } //@line 2316 "translate.cpp"
  
              var $20=$final; //@line 2317 "translate.cpp"
              var $add29=((($20)+4519)&4294967295); //@line 2317 "translate.cpp"
              var $21=$insert_addr; //@line 2317 "translate.cpp"
              HEAP32[(($21)>>2)]=$add29; //@line 2317 "translate.cpp"
              ; //@line 2317 "translate.cpp"
            }
            else if (__label__ == 13) {
  
              var $22=$initial; //@line 2322 "translate.cpp"
              var $add31=((($22)+4352)&4294967295); //@line 2322 "translate.cpp"
              $c_addr=$add31; //@line 2322 "translate.cpp"
              var $23=$medial; //@line 2323 "translate.cpp"
              var $mul=((($23)*28)&4294967295); //@line 2323 "translate.cpp"
              var $24=$final; //@line 2323 "translate.cpp"
              var $add33=((($24)+50500)&4294967295); //@line 2323 "translate.cpp"
              var $add34=((($add33)+($mul))&4294967295); //@line 2323 "translate.cpp"
              var $25=$insert_addr; //@line 2323 "translate.cpp"
              HEAP32[(($25)>>2)]=$add34; //@line 2323 "translate.cpp"
              ;
            }
          } while(0);
  
          var $26=$c_addr; //@line 2325 "translate.cpp"
          $retval=$26; //@line 2325 "translate.cpp"
          __label__ = 19;break $sw_epilog$$sw_bb$$sw_bb17$2; //@line 2325 "translate.cpp"
        }
        else if (__label__ == 15) {
  
          var $27=$c_addr; //@line 2328 "translate.cpp"
          var $sub37=((($27)-12592)&4294967295); //@line 2328 "translate.cpp"
          $code=$sub37; //@line 2328 "translate.cpp"
          var $cmp38=((($sub37))|0) >= 0; //@line 2328 "translate.cpp"
          if (!($cmp38)) { __label__ = 18;break $sw_epilog$$sw_bb$$sw_bb17$2; } //@line 2328 "translate.cpp"
  
          var $28=$code; //@line 2328 "translate.cpp"
          var $cmp40=((($28))|0) < 52; //@line 2328 "translate.cpp"
          if (!($cmp40)) { __label__ = 18;break $sw_epilog$$sw_bb$$sw_bb17$2; } //@line 2328 "translate.cpp"
  
          var $29=$code; //@line 2331 "translate.cpp"
          var $arrayidx42=((__ZZL13TranslateCharP10TranslatorPcijjPiE20hangul_compatibility+$29)&4294967295); //@line 2331 "translate.cpp"
          var $30=HEAPU8[($arrayidx42)]; //@line 2331 "translate.cpp"
          var $conv=((($30))&255); //@line 2331 "translate.cpp"
          var $add43=((($conv)+4352)&4294967295); //@line 2331 "translate.cpp"
          $retval=$add43; //@line 2331 "translate.cpp"
          __label__ = 19;break $sw_epilog$$sw_bb$$sw_bb17$2; //@line 2331 "translate.cpp"
        }
      }
    } while(0);
    if (__label__ == 18) {
  
      var $31=$tr_addr; //@line 2335 "translate.cpp"
      var $32=$c_addr; //@line 2335 "translate.cpp"
      var $33=$next_in_addr; //@line 2335 "translate.cpp"
      var $34=$insert_addr; //@line 2335 "translate.cpp"
      var $call46=__ZL14SubstituteCharP10TranslatorjjPi($31, $32, $33, $34); //@line 2335 "translate.cpp"
      $retval=$call46; //@line 2335 "translate.cpp"
      ; //@line 2335 "translate.cpp"
    }
  
    var $35=$retval; //@line 2336 "translate.cpp"
    STACKTOP = __stackBase__;
    return $35; //@line 2336 "translate.cpp"
    return null;
  }
  

  function __ZL14TranslateWord2P10TranslatorPcP8WORD_TABii($tr, $word, $wtab, $pre_pause, $next_pause) {
    var __stackBase__  = STACKTOP; STACKTOP += 392; _memset(__stackBase__, 0, 392);
    var __label__;
    var __lastLabel__ = null;
  
    var $retval;
    var $tr_addr;
    var $word_addr;
    var $wtab_addr;
    var $pre_pause_addr;
    var $next_pause_addr;
    var $flags;
    var $stress;
    var $next_stress;
    var $next_tone;
    var $p;
    var $srcix;
    var $embedded_cmd;
    var $value;
    var $found_dict_flag;
    var $ph_code;
    var $plist2;
    var $ph;
    var $max_stress;
    var $max_stress_ix;
    var $prev_vowel;
    var $pitch_raised;
    var $switch_phonemes;
    var $first_phoneme;
    var $source_ix;
    var $len;
    var $ix;
    var $sylimit;
    var $new_language;
    var $bad_phoneme=__stackBase__;
    var $word_flags;
    var $word_copy_len;
    var $word_copy=__stackBase__+4;
    var $old_dictionary_name=__stackBase__+168;
    var $c1;
    var $lang_name=__stackBase__+208;
    var $c2;
    var $p2;
    var $ok;
    var $flags2=__stackBase__+220;
    var $c_word2=__stackBase__+228;
    var $ph_buf=__stackBase__+232;
    $tr_addr=$tr;
    $word_addr=$word;
    $wtab_addr=$wtab;
    $pre_pause_addr=$pre_pause;
    $next_pause_addr=$next_pause;
    $flags=0; //@line 1600 "translate.cpp"
    $next_tone=0; //@line 1603 "translate.cpp"
    $max_stress_ix=0; //@line 1613 "translate.cpp"
    $prev_vowel=-1; //@line 1614 "translate.cpp"
    $pitch_raised=0; //@line 1615 "translate.cpp"
    $switch_phonemes=-1; //@line 1616 "translate.cpp"
    $first_phoneme=1; //@line 1617 "translate.cpp"
    var $0=HEAP32[((_f_logespeak)>>2)]; //@line 1629 "translate.cpp"
    var $cmp=((($0))|0)!=0; //@line 1629 "translate.cpp"
    if ($cmp) { __label__ = 1;; } else { __label__ = 7;; } //@line 1629 "translate.cpp"
    $land_lhs_true$$if_end10$2: do { 
      if (__label__ == 1) {
  
        var $1=HEAP32[((_logging_type)>>2)]; //@line 1629 "translate.cpp"
        var $and=($1) & 8; //@line 1629 "translate.cpp"
        var $tobool=((($and))|0)!=0; //@line 1629 "translate.cpp"
        if (!($tobool)) { __label__ = 7;break $land_lhs_true$$if_end10$2; } //@line 1629 "translate.cpp"
  
        var $2=HEAP32[((_f_logespeak)>>2)]; //@line 1631 "translate.cpp"
        var $3=$wtab_addr; //@line 1631 "translate.cpp"
        var $flags1=(($3)&4294967295); //@line 1631 "translate.cpp"
        var $4=HEAP32[(($flags1)>>2)]; //@line 1631 "translate.cpp"
        var $5=$wtab_addr; //@line 1631 "translate.cpp"
        var $length=(($5+10)&4294967295); //@line 1631 "translate.cpp"
        var $6=HEAPU8[($length)]; //@line 1631 "translate.cpp"
        var $conv=((($6))&255); //@line 1631 "translate.cpp"
        var $call=_fprintf($2, ((__str9552)&4294967295), allocate([$4,0,0,0,$conv,0,0,0], ["i32",0,0,0,"i32",0,0,0], ALLOC_STACK)); //@line 1631 "translate.cpp"
        $ix=0; //@line 1632 "translate.cpp"
        __lastLabel__ = 2; ; //@line 1632 "translate.cpp"
        while(1) { 
  
          var $7=__lastLabel__ == 5 ? $inc : (0); //@line 1632 "translate.cpp"
          var $cmp2=((($7))|0) < 40; //@line 1632 "translate.cpp"
          if (!($cmp2)) { __label__ = 6;break ; } //@line 1632 "translate.cpp"
  
          var $8=$ix; //@line 1634 "translate.cpp"
          var $9=$word_addr; //@line 1634 "translate.cpp"
          var $arrayidx=(($9+$8)&4294967295); //@line 1634 "translate.cpp"
          var $10=HEAP8[($arrayidx)]; //@line 1634 "translate.cpp"
          var $conv3=(tempInt=(($10)),(tempInt>=128?tempInt-256:tempInt)); //@line 1634 "translate.cpp"
          var $cmp4=((($conv3))|0)==0; //@line 1634 "translate.cpp"
          if ($cmp4) { __label__ = 6;break ; } //@line 1634 "translate.cpp"
  
          var $11=$ix; //@line 1635 "translate.cpp"
          var $12=$word_addr; //@line 1635 "translate.cpp"
          var $arrayidx6=(($12+$11)&4294967295); //@line 1635 "translate.cpp"
          var $13=HEAP8[($arrayidx6)]; //@line 1635 "translate.cpp"
          var $conv7=(tempInt=(($13)),(tempInt>=128?tempInt-256:tempInt)); //@line 1635 "translate.cpp"
          var $14=HEAP32[((_f_logespeak)>>2)]; //@line 1635 "translate.cpp"
          var $call8=_fputc($conv7, $14); //@line 1635 "translate.cpp"
          var $15=$ix; //@line 1632 "translate.cpp"
          var $inc=((($15)+1)&4294967295); //@line 1632 "translate.cpp"
          $ix=$inc; //@line 1632 "translate.cpp"
          __lastLabel__ = 5; __label__ = 3;continue ; //@line 1632 "translate.cpp"
        }
  
        var $16=HEAP32[((_f_logespeak)>>2)]; //@line 1637 "translate.cpp"
        var $17=_fwrite(((__str10553)&4294967295), 2, 1, $16); //@line 1637 "translate.cpp"
        ; //@line 1638 "translate.cpp"
      }
    } while(0);
  
    var $18=$wtab_addr; //@line 1640 "translate.cpp"
    var $length11=(($18+10)&4294967295); //@line 1640 "translate.cpp"
    var $19=HEAPU8[($length11)]; //@line 1640 "translate.cpp"
    var $conv12=((($19))&255); //@line 1640 "translate.cpp"
    $len=$conv12; //@line 1640 "translate.cpp"
    var $cmp13=((($conv12))|0) > 31; //@line 1641 "translate.cpp"
    if ($cmp13) { __label__ = 8;; } else { __label__ = 9;; } //@line 1641 "translate.cpp"
    if (__label__ == 8) {
  
      $len=31; //@line 1641 "translate.cpp"
      ; //@line 1641 "translate.cpp"
    }
  
    var $20=$wtab_addr; //@line 1642 "translate.cpp"
    var $sourceix=(($20+6)&4294967295); //@line 1642 "translate.cpp"
    var $21=HEAPU16[(($sourceix)>>1)]; //@line 1642 "translate.cpp"
    var $conv16=((($21))&65535); //@line 1642 "translate.cpp"
    var $and17=($conv16) & 2047; //@line 1642 "translate.cpp"
    var $22=$len; //@line 1642 "translate.cpp"
    var $shl=($22) << 11; //@line 1642 "translate.cpp"
    var $or=($and17) | ($shl); //@line 1642 "translate.cpp"
    $source_ix=$or; //@line 1642 "translate.cpp"
    var $23=$wtab_addr; //@line 1644 "translate.cpp"
    var $arrayidx18=(($23)&4294967295); //@line 1644 "translate.cpp"
    var $flags19=(($arrayidx18)&4294967295); //@line 1644 "translate.cpp"
    var $24=HEAP32[(($flags19)>>2)]; //@line 1644 "translate.cpp"
    $word_flags=$24; //@line 1644 "translate.cpp"
    var $25=$word_flags; //@line 1645 "translate.cpp"
    var $and20=($25) & 64; //@line 1645 "translate.cpp"
    var $tobool21=((($and20))|0)!=0; //@line 1645 "translate.cpp"
    if ($tobool21) { __label__ = 10;; } else { __label__ = 18;; } //@line 1645 "translate.cpp"
    $if_then22$$if_end33$14: do { 
      if (__label__ == 10) {
  
        HEAP8[(__ZL13embedded_flag_b)]=1;
        ; //@line 1649 "translate.cpp"
        while(1) { 
  
          var $26=HEAP32[((__ZL13embedded_read)>>2)]; //@line 1651 "translate.cpp"
          var $inc23=((($26)+1)&4294967295); //@line 1651 "translate.cpp"
          HEAP32[((__ZL13embedded_read)>>2)]=$inc23; //@line 1651 "translate.cpp"
          var $arrayidx24=((_embedded_list+$26*4)&4294967295); //@line 1651 "translate.cpp"
          var $27=HEAP32[(($arrayidx24)>>2)]; //@line 1651 "translate.cpp"
          $embedded_cmd=$27; //@line 1651 "translate.cpp"
          var $28=$embedded_cmd; //@line 1652 "translate.cpp"
          var $shr=($28) >> 8; //@line 1652 "translate.cpp"
          $value=$shr; //@line 1652 "translate.cpp"
          var $29=$embedded_cmd; //@line 1654 "translate.cpp"
          var $and25=($29) & 31; //@line 1654 "translate.cpp"
          if ($and25 == 9) {
            __label__ = 12;;
          }
          else if ($and25 == 13) {
            __label__ = 13;;
          }
          else if ($and25 == 12) {
            __label__ = 14;;
          }
          else {
          __label__ = 17;;
          }
          
          if (__label__ == 12) {
  
            var $30=$value; //@line 1657 "translate.cpp"
            HEAP32[((_option_sayas)>>2)]=$30; //@line 1657 "translate.cpp"
            ; //@line 1658 "translate.cpp"
          }
          else if (__label__ == 13) {
  
            var $31=$value; //@line 1661 "translate.cpp"
            HEAP32[((__ZL15option_emphasis)>>2)]=$31; //@line 1661 "translate.cpp"
            ; //@line 1662 "translate.cpp"
          }
          else if (__label__ == 14) {
  
            var $32=$value; //@line 1666 "translate.cpp"
            var $cmp28=((($32))|0)==0; //@line 1666 "translate.cpp"
            if ($cmp28) { __label__ = 15;; } else { __label__ = 16;; } //@line 1666 "translate.cpp"
            if (__label__ == 15) {
  
              $pre_pause_addr=0; //@line 1667 "translate.cpp"
              ; //@line 1667 "translate.cpp"
            }
            else if (__label__ == 16) {
  
              var $33=$value; //@line 1669 "translate.cpp"
              var $34=$pre_pause_addr; //@line 1669 "translate.cpp"
              var $add=((($34)+($33))&4294967295); //@line 1669 "translate.cpp"
              $pre_pause_addr=$add; //@line 1669 "translate.cpp"
              ;
            }
          }
  
          var $35=$embedded_cmd; //@line 1672 "translate.cpp"
          var $and31=($35) & 128; //@line 1672 "translate.cpp"
          var $cmp32=((($and31))|0)==0; //@line 1672 "translate.cpp"
          if ($cmp32) { __label__ = 11;continue ; } else { __label__ = 18;break $if_then22$$if_end33$14; } //@line 1672 "translate.cpp"
        }
      }
    } while(0);
  
    var $36=$word_addr; //@line 1675 "translate.cpp"
    var $arrayidx34=(($36)&4294967295); //@line 1675 "translate.cpp"
    var $37=HEAP8[($arrayidx34)]; //@line 1675 "translate.cpp"
    var $conv35=(tempInt=(($37)),(tempInt>=128?tempInt-256:tempInt)); //@line 1675 "translate.cpp"
    var $cmp36=((($conv35))|0)==0; //@line 1675 "translate.cpp"
    if ($cmp36) { __label__ = 19;; } else { __label__ = 20;; } //@line 1675 "translate.cpp"
    $if_then37$$if_end38$27: do { 
      if (__label__ == 19) {
  
        HEAP8[(((_word_phonemes)&4294967295))]=0; //@line 1678 "translate.cpp"
        $retval=0; //@line 1679 "translate.cpp"
        ; //@line 1679 "translate.cpp"
      }
      else if (__label__ == 20) {
  
        var $38=$tr_addr; //@line 1683 "translate.cpp"
        var $prepause_timeout=(($38+8312)&4294967295); //@line 1683 "translate.cpp"
        var $39=HEAP32[(($prepause_timeout)>>2)]; //@line 1683 "translate.cpp"
        var $cmp39=((($39))|0) > 0; //@line 1683 "translate.cpp"
        if ($cmp39) { __label__ = 21;; } else { __label__ = 22;; } //@line 1683 "translate.cpp"
        if (__label__ == 21) {
  
          var $40=$tr_addr; //@line 1684 "translate.cpp"
          var $prepause_timeout41=(($40+8312)&4294967295); //@line 1684 "translate.cpp"
          var $41=HEAP32[(($prepause_timeout41)>>2)]; //@line 1684 "translate.cpp"
          var $dec=((($41)-1)&4294967295); //@line 1684 "translate.cpp"
          HEAP32[(($prepause_timeout41)>>2)]=$dec; //@line 1684 "translate.cpp"
          ; //@line 1684 "translate.cpp"
        }
  
        var $42=HEAP32[((_option_sayas)>>2)]; //@line 1686 "translate.cpp"
        var $and43=($42) & 240; //@line 1686 "translate.cpp"
        var $cmp44=((($and43))|0)==16; //@line 1686 "translate.cpp"
        if ($cmp44) { __label__ = 23;; } else { __label__ = 25;; } //@line 1686 "translate.cpp"
        $if_then45$$if_end51$33: do { 
          if (__label__ == 23) {
  
            var $43=$word_flags; //@line 1688 "translate.cpp"
            var $and46=($43) & 512; //@line 1688 "translate.cpp"
            var $tobool47=((($and46))|0)!=0; //@line 1688 "translate.cpp"
            if ($tobool47) { __label__ = 25;break $if_then45$$if_end51$33; } //@line 1688 "translate.cpp"
  
            var $44=$pre_pause_addr; //@line 1691 "translate.cpp"
            var $add49=((($44)+4)&4294967295); //@line 1691 "translate.cpp"
            $pre_pause_addr=$add49; //@line 1691 "translate.cpp"
            ; //@line 1692 "translate.cpp"
          }
        } while(0);
  
        var $45=$word_flags; //@line 1695 "translate.cpp"
        var $and52=($45) & 2; //@line 1695 "translate.cpp"
        var $tobool53=((($and52))|0)!=0; //@line 1695 "translate.cpp"
        if ($tobool53) { __label__ = 26;; } else { __label__ = 32;; } //@line 1695 "translate.cpp"
        $if_then54$$if_end72$37: do { 
          if (__label__ == 26) {
  
            var $46=HEAP32[((_option_capitals)>>2)]; //@line 1697 "translate.cpp"
            var $cmp55=((($46))|0) > 2; //@line 1697 "translate.cpp"
            var $47=HEAP32[((__ZL11embedded_ix)>>2)]; //@line 1697 "translate.cpp"
            var $cmp57=((($47))|0) < 244; //@line 1697 "translate.cpp"
            var $or_cond=($cmp55) & ($cmp57); //@line 1697 "translate.cpp"
            if (!($or_cond)) { __label__ = 32;break $if_then54$$if_end72$37; } //@line 1697 "translate.cpp"
  
            var $_b3=HEAPU8[(__ZL13embedded_flag_b)];
            var $48=($_b3) ? 2 : 0;
            var $tobool59=((($48))|0)!=0; //@line 1700 "translate.cpp"
            if ($tobool59) { __label__ = 28;; } else { __label__ = 29;; } //@line 1700 "translate.cpp"
            if (__label__ == 28) {
  
              var $49=HEAP32[((__ZL11embedded_ix)>>2)]; //@line 1701 "translate.cpp"
              var $sub=((($49)-1)&4294967295); //@line 1701 "translate.cpp"
              var $arrayidx61=((_embedded_list+$sub*4)&4294967295); //@line 1701 "translate.cpp"
              var $50=HEAP32[(($arrayidx61)>>2)]; //@line 1701 "translate.cpp"
              var $and62=($50) & -129; //@line 1701 "translate.cpp"
              HEAP32[(($arrayidx61)>>2)]=$and62; //@line 1701 "translate.cpp"
              ; //@line 1701 "translate.cpp"
            }
  
            var $51=HEAP32[((_option_capitals)>>2)]; //@line 1702 "translate.cpp"
            $pitch_raised=$51; //@line 1702 "translate.cpp"
            var $cmp64=((($51))|0)==3; //@line 1702 "translate.cpp"
            if ($cmp64) { __label__ = 30;; } else { __label__ = 31;; } //@line 1702 "translate.cpp"
            if (__label__ == 30) {
  
              $pitch_raised=20; //@line 1703 "translate.cpp"
              ; //@line 1703 "translate.cpp"
            }
  
            var $52=$pitch_raised; //@line 1704 "translate.cpp"
            var $shl67=((($52)*256)&4294967295); //@line 1704 "translate.cpp"
            var $add68=((($shl67)+193)&4294967295); //@line 1704 "translate.cpp"
            var $53=HEAP32[((__ZL11embedded_ix)>>2)]; //@line 1704 "translate.cpp"
            var $inc69=((($53)+1)&4294967295); //@line 1704 "translate.cpp"
            HEAP32[((__ZL11embedded_ix)>>2)]=$inc69; //@line 1704 "translate.cpp"
            var $arrayidx70=((_embedded_list+$53*4)&4294967295); //@line 1704 "translate.cpp"
            HEAP32[(($arrayidx70)>>2)]=$add68; //@line 1704 "translate.cpp"
            HEAP8[(__ZL13embedded_flag_b)]=1;
            ; //@line 1706 "translate.cpp"
          }
        } while(0);
  
        $p=((_word_phonemes)&4294967295); //@line 1709 "translate.cpp"
        var $54=$word_flags; //@line 1710 "translate.cpp"
        var $and73=($54) & 8; //@line 1710 "translate.cpp"
        var $tobool74=((($and73))|0)!=0; //@line 1710 "translate.cpp"
        if ($tobool74) { __label__ = 33;; } else { __label__ = 40;; } //@line 1710 "translate.cpp"
        $if_then75$$if_else100$47: do { 
          if (__label__ == 33) {
  
            var $55=$word_addr; //@line 1716 "translate.cpp"
            var $call76=_memcmp($55, ((__str11554)&4294967295), 3); //@line 1716 "translate.cpp"
            var $cmp77=((($call76))|0)==0; //@line 1716 "translate.cpp"
            var $56=$word_addr; //@line 1719 "translate.cpp"
            if ($cmp77) { __label__ = 34;; } else { __label__ = 38;; } //@line 1716 "translate.cpp"
            $if_then78$$if_else96$49: do { 
              if (__label__ == 34) {
  
                var $add_ptr=(($56+3)&4294967295); //@line 1719 "translate.cpp"
                $word_addr=$add_ptr; //@line 1719 "translate.cpp"
                $ix=0; //@line 1720 "translate.cpp"
                var $57=$word_addr; //@line 1722 "translate.cpp"
                var $incdec_ptr32=(($57+1)&4294967295); //@line 1722 "translate.cpp"
                $word_addr=$incdec_ptr32; //@line 1722 "translate.cpp"
                var $58=HEAP8[($57)]; //@line 1722 "translate.cpp"
                var $conv8033=(tempInt=(($58)),(tempInt>=128?tempInt-256:tempInt)); //@line 1722 "translate.cpp"
                $c1=$conv8033; //@line 1722 "translate.cpp"
                var $59=$c1; //@line 1723 "translate.cpp"
                var $cmp8134=((($59))|0)==32; //@line 1723 "translate.cpp"
                var $60=$c1; //@line 1723 "translate.cpp"
                var $cmp8235=((($60))|0)==0; //@line 1723 "translate.cpp"
                var $or_cond436=($cmp8134) | ($cmp8235); //@line 1723 "translate.cpp"
                if ($or_cond436) { __label__ = 36;; } else { __label__ = 35;; } //@line 1723 "translate.cpp"
                $for_end89$$if_end84$51: do { 
                  if (__label__ == 35) {
                    while(1) { 
  
                      var $61=$c1; //@line 1725 "translate.cpp"
                      var $call85=_tolower($61); //@line 1725 "translate.cpp"
                      var $conv86=((($call85)) & 255); //@line 1725 "translate.cpp"
                      var $62=$ix; //@line 1725 "translate.cpp"
                      var $inc87=((($62)+1)&4294967295); //@line 1725 "translate.cpp"
                      $ix=$inc87; //@line 1725 "translate.cpp"
                      var $arrayidx88=(($lang_name+$62)&4294967295); //@line 1725 "translate.cpp"
                      HEAP8[($arrayidx88)]=$conv86; //@line 1725 "translate.cpp"
                      var $63=$word_addr; //@line 1722 "translate.cpp"
                      var $incdec_ptr=(($63+1)&4294967295); //@line 1722 "translate.cpp"
                      $word_addr=$incdec_ptr; //@line 1722 "translate.cpp"
                      var $64=HEAP8[($63)]; //@line 1722 "translate.cpp"
                      var $conv80=(tempInt=(($64)),(tempInt>=128?tempInt-256:tempInt)); //@line 1722 "translate.cpp"
                      $c1=$conv80; //@line 1722 "translate.cpp"
                      var $65=$c1; //@line 1723 "translate.cpp"
                      var $cmp81=((($65))|0)==32; //@line 1723 "translate.cpp"
                      var $66=$c1; //@line 1723 "translate.cpp"
                      var $cmp82=((($66))|0)==0; //@line 1723 "translate.cpp"
                      var $or_cond4=($cmp81) | ($cmp82); //@line 1723 "translate.cpp"
                      if ($or_cond4) { __label__ = 36;break $for_end89$$if_end84$51; } else { __label__ = 35;continue ; } //@line 1723 "translate.cpp"
                    }
                  }
                } while(0);
  
                var $67=$ix; //@line 1727 "translate.cpp"
                var $arrayidx90=(($lang_name+$67)&4294967295); //@line 1727 "translate.cpp"
                HEAP8[($arrayidx90)]=0; //@line 1727 "translate.cpp"
                var $arraydecay=(($lang_name)&4294967295); //@line 1729 "translate.cpp"
                var $call91=__Z18LookupPhonemeTablePKc($arraydecay); //@line 1729 "translate.cpp"
                $ix=$call91; //@line 1729 "translate.cpp"
                var $cmp92=((($call91))|0) > 0; //@line 1729 "translate.cpp"
                if (!($cmp92)) { __label__ = 39;break $if_then78$$if_else96$49; } //@line 1729 "translate.cpp"
  
                var $68=$ix; //@line 1731 "translate.cpp"
                __Z18SelectPhonemeTablei($68); //@line 1731 "translate.cpp"
                HEAP8[(((_word_phonemes)&4294967295))]=21; //@line 1732 "translate.cpp"
                var $69=$ix; //@line 1733 "translate.cpp"
                var $conv94=((($69)) & 255); //@line 1733 "translate.cpp"
                HEAP8[(((_word_phonemes+1)&4294967295))]=$conv94; //@line 1733 "translate.cpp"
                HEAP8[(((_word_phonemes+2)&4294967295))]=0; //@line 1734 "translate.cpp"
                ; //@line 1735 "translate.cpp"
              }
              else if (__label__ == 38) {
  
                var $arraydecay97=(($bad_phoneme)&4294967295); //@line 1739 "translate.cpp"
                var $call98=__Z14EncodePhonemesPcS_Ph($56, ((_word_phonemes)&4294967295), $arraydecay97); //@line 1739 "translate.cpp"
                ;
              }
            } while(0);
  
            $flags=-2147483648; //@line 1741 "translate.cpp"
            ; //@line 1742 "translate.cpp"
          }
          else if (__label__ == 40) {
  
            $ix=0; //@line 1746 "translate.cpp"
            ; //@line 1747 "translate.cpp"
            $while_cond$59: while(1) { 
  
              var $70=$ix; //@line 1747 "translate.cpp"
              var $71=$word_addr; //@line 1747 "translate.cpp"
              var $arrayidx101=(($71+$70)&4294967295); //@line 1747 "translate.cpp"
              var $72=HEAP8[($arrayidx101)]; //@line 1747 "translate.cpp"
              var $73=$ix; //@line 1747 "translate.cpp"
              var $arrayidx102=(($word_copy+$73)&4294967295); //@line 1747 "translate.cpp"
              HEAP8[($arrayidx102)]=$72; //@line 1747 "translate.cpp"
              var $conv103=(tempInt=(($72)),(tempInt>=128?tempInt-256:tempInt)); //@line 1747 "translate.cpp"
              $c2=$conv103; //@line 1747 "translate.cpp"
              var $cmp104=((($conv103))|0)!=32; //@line 1747 "translate.cpp"
              if (!($cmp104)) { __label__ = 43;break $while_cond$59; } //@line 1747 "translate.cpp"
  
              var $74=$c2; //@line 1747 "translate.cpp"
              var $cmp106=((($74))|0)!=0; //@line 1747 "translate.cpp"
              if (!($cmp106)) { __label__ = 43;break $while_cond$59; } //@line 1747 "translate.cpp"
  
              var $76=$ix; //@line 1747 "translate.cpp"
              var $cmp107=((($76))|0) < 160; //@line 1747 "translate.cpp"
              var $77=$ix; //@line 1747 "translate.cpp"
              if ($cmp107) { __lastLabel__ = 44; ; } else { __lastLabel__ = 44; __label__ = 46;break $while_cond$59; }
  
              var $inc108=((($77)+1)&4294967295); //@line 1747 "translate.cpp"
              $ix=$inc108; //@line 1747 "translate.cpp"
              __label__ = 41;continue $while_cond$59; //@line 1747 "translate.cpp"
            }
            if (__label__ == 43) {
  
              var $75=$ix; //@line 1747 "translate.cpp"
              __lastLabel__ = 43; ;
            }
  
            var $78=__lastLabel__ == 43 ? $75 : ($77);
            $word_copy_len=$78; //@line 1748 "translate.cpp"
            var $79=HEAP32[((_translator)>>2)]; //@line 1750 "translate.cpp"
            var $80=$word_addr; //@line 1750 "translate.cpp"
            var $81=$next_pause_addr; //@line 1750 "translate.cpp"
            var $82=$wtab_addr; //@line 1750 "translate.cpp"
            var $call109=__Z13TranslateWordP10TranslatorPciP8WORD_TAB($79, $80, $81, $82); //@line 1750 "translate.cpp"
            $flags=$call109; //@line 1750 "translate.cpp"
            var $83=$flags; //@line 1752 "translate.cpp"
            var $and110=($83) & 65536; //@line 1752 "translate.cpp"
            var $tobool111=((($and110))|0)!=0; //@line 1752 "translate.cpp"
            if ($tobool111) { __label__ = 47;; } else { __label__ = 48;; } //@line 1752 "translate.cpp"
            if (__label__ == 47) {
  
              var $84=$word_addr; //@line 1755 "translate.cpp"
              var $arraydecay113=(($word_copy)&4294967295); //@line 1755 "translate.cpp"
              var $85=$word_copy_len; //@line 1755 "translate.cpp"
              var $src$, $dest$, $stop$, $stop4$, $fast$;
              $src$ = $arraydecay113;
              $dest$ = $84;
              $stop$ = $src$ + $85;
              if (($dest$%4) == ($src$%4) && $85 > 8) {
                while ($src$%4 !== 0 && $src$ < $stop$) {
                  ; HEAP8[$dest$++] = HEAP8[$src$++];
                }
                $src$ >>= 2;
                $dest$ >>= 2;
                $stop4$ = $stop$ >> 2;
                while ($src$ < $stop4$) {
              ;;;    HEAP32[$dest$++] = HEAP32[$src$++];
                }
                $src$ <<= 2;
                $dest$ <<= 2;
              }while ($src$ < $stop$) {
                ; HEAP8[$dest$++] = HEAP8[$src$++];
              }; //@line 1755 "translate.cpp"
              var $86=$flags; //@line 1756 "translate.cpp"
              $retval=$86; //@line 1756 "translate.cpp"
              __label__ = 171;break $if_then37$$if_end38$27; //@line 1756 "translate.cpp"
            }
            else if (__label__ == 48) {
  
              var $87=$flags; //@line 1759 "translate.cpp"
              var $and115=($87) & 8388608; //@line 1759 "translate.cpp"
              var $tobool116=((($and115))|0)!=0; //@line 1759 "translate.cpp"
              if ($tobool116) { __label__ = 49;; } else { __label__ = 72;; } //@line 1759 "translate.cpp"
              $land_lhs_true117$$if_end192$70: do { 
                if (__label__ == 49) {
  
                  var $88=$wtab_addr; //@line 1759 "translate.cpp"
                  var $arrayidx118=(($88+12)&4294967295); //@line 1759 "translate.cpp"
                  var $flags119=(($arrayidx118)&4294967295); //@line 1759 "translate.cpp"
                  var $89=HEAP32[(($flags119)>>2)]; //@line 1759 "translate.cpp"
                  var $and120=($89) & 8; //@line 1759 "translate.cpp"
                  var $tobool121=((($and120))|0)!=0; //@line 1759 "translate.cpp"
                  if ($tobool121) { __label__ = 72;break $land_lhs_true117$$if_end192$70; } //@line 1759 "translate.cpp"
  
                  $ok=1; //@line 1762 "translate.cpp"
                  var $arrayidx123=(($flags2)&4294967295); //@line 1767 "translate.cpp"
                  HEAP32[(($arrayidx123)>>2)]=0; //@line 1767 "translate.cpp"
                  var $90=$tr_addr; //@line 1768 "translate.cpp"
                  var $langopts=(($90)&4294967295); //@line 1768 "translate.cpp"
                  var $param=(($langopts+24)&4294967295); //@line 1768 "translate.cpp"
                  var $arrayidx124=(($param+44)&4294967295); //@line 1768 "translate.cpp"
                  var $91=HEAP32[(($arrayidx124)>>2)]; //@line 1768 "translate.cpp"
                  $sylimit=$91; //@line 1768 "translate.cpp"
                  var $92=$word_addr; //@line 1772 "translate.cpp"
                  $p2=$92; //@line 1772 "translate.cpp"
                  var $93=$p2; //@line 1773 "translate.cpp"
                  var $94=HEAP8[($93)]; //@line 1773 "translate.cpp"
                  var $conv12620=(tempInt=(($94)),(tempInt>=128?tempInt-256:tempInt)); //@line 1773 "translate.cpp"
                  var $cmp12721=((($conv12620))|0)!=32; //@line 1773 "translate.cpp"
                  var $95=$p2; //@line 1773 "translate.cpp"
                  var $incdec_ptr12922=(($95+1)&4294967295); //@line 1773 "translate.cpp"
                  if ($cmp12721) { __lastLabel__ = 50; __label__ = 51;; } else { __lastLabel__ = 50; __label__ = 52;; } //@line 1773 "translate.cpp"
                  $while_body128$$while_end130$73: do { 
                    if (__label__ == 51) {
                      while(1) { 
  
                        var $incdec_ptr12923=__lastLabel__ == 51 ? $incdec_ptr129 : ($incdec_ptr12922);
                        $p2=$incdec_ptr12923; //@line 1773 "translate.cpp"
                        var $96=$p2; //@line 1773 "translate.cpp"
                        var $97=HEAP8[($96)]; //@line 1773 "translate.cpp"
                        var $conv126=(tempInt=(($97)),(tempInt>=128?tempInt-256:tempInt)); //@line 1773 "translate.cpp"
                        var $cmp127=((($conv126))|0)!=32; //@line 1773 "translate.cpp"
                        var $98=$p2; //@line 1773 "translate.cpp"
                        var $incdec_ptr129=(($98+1)&4294967295); //@line 1773 "translate.cpp"
                        if ($cmp127) { __lastLabel__ = 51; __label__ = 51;continue ; } else { __lastLabel__ = 51; __label__ = 52;break $while_body128$$while_end130$73; } //@line 1773 "translate.cpp"
                      }
                    }
                  } while(0);
  
                  var $incdec_ptr129_lcssa=__lastLabel__ == 50 ? $incdec_ptr12922 : ($incdec_ptr129);
                  var $call132=__Z7utf8_inPiPKc($c_word2, $incdec_ptr129_lcssa); //@line 1775 "translate.cpp"
                  var $99=HEAP32[(($c_word2)>>2)]; //@line 1776 "translate.cpp"
                  var $call133=_iswalpha($99); //@line 1776 "translate.cpp"
                  var $tobool134=((($call133))|0)!=0; //@line 1776 "translate.cpp"
                  if ($tobool134) { __label__ = 54;; } else { __label__ = 53;; } //@line 1776 "translate.cpp"
                  if (__label__ == 54) {
  
                    var $_pr=$ok; //@line 1781 "translate.cpp"
                    var $cmp137=((($_pr))|0)!=0; //@line 1781 "translate.cpp"
                    if (!($cmp137)) { __label__ = 72;break $land_lhs_true117$$if_end192$70; } //@line 1781 "translate.cpp"
  
                    var $arraydecay139=(($ph_buf)&4294967295); //@line 1783 "translate.cpp"
                    var $call140=_strcpy($arraydecay139, ((_word_phonemes)&4294967295)); //@line 1783 "translate.cpp"
                    var $100=HEAP32[((_translator)>>2)]; //@line 1785 "translate.cpp"
                    var $101=$p2; //@line 1785 "translate.cpp"
                    var $add_ptr141=(($101+1)&4294967295); //@line 1785 "translate.cpp"
                    var $102=$wtab_addr; //@line 1785 "translate.cpp"
                    var $add_ptr142=(($102+12)&4294967295); //@line 1785 "translate.cpp"
                    var $call143=__Z13TranslateWordP10TranslatorPciP8WORD_TAB($100, $add_ptr141, 0, $add_ptr142); //@line 1785 "translate.cpp"
                    var $arrayidx144=(($flags2)&4294967295); //@line 1785 "translate.cpp"
                    HEAP32[(($arrayidx144)>>2)]=$call143; //@line 1785 "translate.cpp"
                    var $arrayidx145=(($flags2)&4294967295); //@line 1786 "translate.cpp"
                    var $103=HEAP32[(($arrayidx145)>>2)]; //@line 1786 "translate.cpp"
                    var $and146=($103) & 67108864; //@line 1786 "translate.cpp"
                    var $tobool147=((($and146))|0)!=0; //@line 1786 "translate.cpp"
                    if ($tobool147) { __label__ = 56;; } else { __label__ = 57;; } //@line 1786 "translate.cpp"
                    if (__label__ == 56) {
  
                      $ok=0; //@line 1787 "translate.cpp"
                      ; //@line 1787 "translate.cpp"
                    }
  
                    var $104=$sylimit; //@line 1789 "translate.cpp"
                    var $and150=($104) & 256; //@line 1789 "translate.cpp"
                    var $tobool151=((($and150))|0)!=0; //@line 1789 "translate.cpp"
                    if ($tobool151) { __label__ = 58;; } else { __label__ = 60;; } //@line 1789 "translate.cpp"
                    $if_then152$$if_end158$83: do { 
                      if (__label__ == 58) {
  
                        var $arrayidx153=(($flags2)&4294967295); //@line 1792 "translate.cpp"
                        var $105=HEAP32[(($arrayidx153)>>2)]; //@line 1792 "translate.cpp"
                        var $and154=($105) & 1048576; //@line 1792 "translate.cpp"
                        var $cmp155=((($and154))|0)==0; //@line 1792 "translate.cpp"
                        if (!($cmp155)) { __label__ = 60;break $if_then152$$if_end158$83; } //@line 1792 "translate.cpp"
  
                        $ok=0; //@line 1794 "translate.cpp"
                        ; //@line 1795 "translate.cpp"
                      }
                    } while(0);
  
                    var $106=$sylimit; //@line 1798 "translate.cpp"
                    var $and159=($106) & 512; //@line 1798 "translate.cpp"
                    var $tobool160=((($and159))|0)!=0; //@line 1798 "translate.cpp"
                    if ($tobool160) { __label__ = 61;; } else { __label__ = 63;; } //@line 1798 "translate.cpp"
                    $land_lhs_true161$$if_end167$87: do { 
                      if (__label__ == 61) {
  
                        var $107=$wtab_addr; //@line 1798 "translate.cpp"
                        var $add_ptr162=(($107+12)&4294967295); //@line 1798 "translate.cpp"
                        var $flags163=(($add_ptr162)&4294967295); //@line 1798 "translate.cpp"
                        var $108=HEAP32[(($flags163)>>2)]; //@line 1798 "translate.cpp"
                        var $and164=($108) & 16; //@line 1798 "translate.cpp"
                        var $tobool165=((($and164))|0)!=0; //@line 1798 "translate.cpp"
                        if (!($tobool165)) { __label__ = 63;break $land_lhs_true161$$if_end167$87; } //@line 1798 "translate.cpp"
  
                        $ok=0; //@line 1801 "translate.cpp"
                        __label__ = 64;break $land_lhs_true161$$if_end167$87; //@line 1804 "translate.cpp"
                      }
                    } while(0);
                    $if_end173$$if_end167$90: do { 
                      if (__label__ == 63) {
  
                        var $_pr8=$ok; //@line 1804 "translate.cpp"
                        var $cmp168=((($_pr8))|0)==0; //@line 1804 "translate.cpp"
                        if ($cmp168) { __label__ = 64;break $if_end173$$if_end167$90; } else { __label__ = 65;break $if_end173$$if_end167$90; } //@line 1804 "translate.cpp"
                      }
                    } while(0);
                    if (__label__ == 64) {
  
                      var $arraydecay170=(($ph_buf)&4294967295); //@line 1806 "translate.cpp"
                      var $call171=_strcpy(((_word_phonemes)&4294967295), $arraydecay170); //@line 1806 "translate.cpp"
                      var $_pr10_pr=$ok; //@line 1810 "translate.cpp"
                      var $tobool174=((($_pr10_pr))|0)!=0; //@line 1810 "translate.cpp"
                      if (!($tobool174)) { __label__ = 72;break $land_lhs_true117$$if_end192$70; } //@line 1810 "translate.cpp"
                    }
  
                    var $109=$p2; //@line 1812 "translate.cpp"
                    HEAP8[($109)]=45; //@line 1812 "translate.cpp"
                    var $110=HEAP32[((_translator)>>2)]; //@line 1813 "translate.cpp"
                    var $111=$word_addr; //@line 1813 "translate.cpp"
                    var $112=$next_pause_addr; //@line 1813 "translate.cpp"
                    var $113=$wtab_addr; //@line 1813 "translate.cpp"
                    var $call176=__Z13TranslateWordP10TranslatorPciP8WORD_TAB($110, $111, $112, $113); //@line 1813 "translate.cpp"
                    $flags=$call176; //@line 1813 "translate.cpp"
                    var $114=$sylimit; //@line 1814 "translate.cpp"
                    var $cmp177=((($114))|0) > 0; //@line 1814 "translate.cpp"
                    if ($cmp177) { __lastLabel__ = 65; __label__ = 66;; } else { __lastLabel__ = 65; __label__ = 69;; } //@line 1814 "translate.cpp"
                    if (__label__ == 66) {
  
                      var $115=$p; //@line 1814 "translate.cpp"
                      var $call179=__ZL14CountSyllablesPh($115); //@line 1814 "translate.cpp"
                      var $116=$sylimit; //@line 1814 "translate.cpp"
                      var $and180=($116) & 31; //@line 1814 "translate.cpp"
                      var $cmp181=((($call179))|0) > ((($and180))|0); //@line 1814 "translate.cpp"
                      if ($cmp181) { __label__ = 67;; } else { __label__ = 68;; } //@line 1814 "translate.cpp"
                      if (__label__ == 67) {
  
                        var $117=$p2; //@line 1817 "translate.cpp"
                        HEAP8[($117)]=32; //@line 1817 "translate.cpp"
                        var $118=HEAP32[((_translator)>>2)]; //@line 1818 "translate.cpp"
                        var $119=$word_addr; //@line 1818 "translate.cpp"
                        var $120=$next_pause_addr; //@line 1818 "translate.cpp"
                        var $121=$wtab_addr; //@line 1818 "translate.cpp"
                        var $call183=__Z13TranslateWordP10TranslatorPciP8WORD_TAB($118, $119, $120, $121); //@line 1818 "translate.cpp"
                        $flags=$call183; //@line 1818 "translate.cpp"
                        __label__ = 72;break $land_lhs_true117$$if_end192$70; //@line 1819 "translate.cpp"
                      }
                      else if (__label__ == 68) {
  
                        var $_pr14=$flags; //@line 1822 "translate.cpp"
                        __lastLabel__ = 68; ;
                      }
                    }
  
                    var $122=__lastLabel__ == 68 ? $_pr14 : ($call176); //@line 1822 "translate.cpp"
                    var $cmp185=((($122))|0)==0; //@line 1822 "translate.cpp"
                    if ($cmp185) { __label__ = 70;; } else { __label__ = 71;; } //@line 1822 "translate.cpp"
                    if (__label__ == 70) {
  
                      var $arrayidx187=(($flags2)&4294967295); //@line 1823 "translate.cpp"
                      var $123=HEAP32[(($arrayidx187)>>2)]; //@line 1823 "translate.cpp"
                      $flags=$123; //@line 1823 "translate.cpp"
                      ; //@line 1823 "translate.cpp"
                    }
  
                    var $124=$flags; //@line 1824 "translate.cpp"
                    var $or189=($124) | 128; //@line 1824 "translate.cpp"
                    $flags=$or189; //@line 1824 "translate.cpp"
                    HEAP32[((_dictionary_skipwords)>>2)]=1; //@line 1825 "translate.cpp"
                    ;
                  }
                  else if (__label__ == 53) {
  
                    $ok=0; //@line 1778 "translate.cpp"
                    ; //@line 1810 "translate.cpp"
                  }
                }
              } while(0);
  
              var $125=$p; //@line 1830 "translate.cpp"
              var $arrayidx193=(($125)&4294967295); //@line 1830 "translate.cpp"
              var $126=HEAPU8[($arrayidx193)]; //@line 1830 "translate.cpp"
              var $conv194=((($126))&255); //@line 1830 "translate.cpp"
              var $cmp195=((($conv194))|0)==21; //@line 1830 "translate.cpp"
              if ($cmp195) { __label__ = 73;; } else { __label__ = 84;; } //@line 1830 "translate.cpp"
              $if_then196$$if_end232$106: do { 
                if (__label__ == 73) {
  
                  var $127=$word_addr; //@line 1833 "translate.cpp"
                  var $arraydecay197=(($word_copy)&4294967295); //@line 1833 "translate.cpp"
                  var $128=$word_copy_len; //@line 1833 "translate.cpp"
                  var $src$, $dest$, $stop$, $stop4$, $fast$;
                  $src$ = $arraydecay197;
                  $dest$ = $127;
                  $stop$ = $src$ + $128;
                  if (($dest$%4) == ($src$%4) && $128 > 8) {
                    while ($src$%4 !== 0 && $src$ < $stop$) {
                      ; HEAP8[$dest$++] = HEAP8[$src$++];
                    }
                    $src$ >>= 2;
                    $dest$ >>= 2;
                    $stop4$ = $stop$ >> 2;
                    while ($src$ < $stop4$) {
                  ;;;    HEAP32[$dest$++] = HEAP32[$src$++];
                    }
                    $src$ <<= 2;
                    $dest$ <<= 2;
                  }while ($src$ < $stop$) {
                    ; HEAP8[$dest$++] = HEAP8[$src$++];
                  }; //@line 1833 "translate.cpp"
                  var $129=$p; //@line 1835 "translate.cpp"
                  var $arrayidx198=(($129+1)&4294967295); //@line 1835 "translate.cpp"
                  $new_language=$arrayidx198; //@line 1835 "translate.cpp"
                  var $130=$new_language; //@line 1836 "translate.cpp"
                  var $arrayidx199=(($130)&4294967295); //@line 1836 "translate.cpp"
                  var $131=HEAP8[($arrayidx199)]; //@line 1836 "translate.cpp"
                  var $conv200=(tempInt=(($131)),(tempInt>=128?tempInt-256:tempInt)); //@line 1836 "translate.cpp"
                  var $cmp201=((($conv200))|0)==0; //@line 1836 "translate.cpp"
                  if ($cmp201) { __label__ = 74;; } else { __label__ = 75;; } //@line 1836 "translate.cpp"
                  if (__label__ == 74) {
  
                    $new_language=((__str52704)&4294967295); //@line 1837 "translate.cpp"
                    ; //@line 1837 "translate.cpp"
                  }
  
                  var $arraydecay204=(($old_dictionary_name)&4294967295); //@line 1839 "translate.cpp"
                  var $call205=_strcpy($arraydecay204, ((_dictionary_name)&4294967295)); //@line 1839 "translate.cpp"
                  var $132=$new_language; //@line 1840 "translate.cpp"
                  var $call206=__Z14SetTranslator2PKc($132); //@line 1840 "translate.cpp"
                  $switch_phonemes=$call206; //@line 1840 "translate.cpp"
                  var $cmp207=((($call206))|0) >= 0; //@line 1842 "translate.cpp"
                  if ($cmp207) { __label__ = 76;; } else { __label__ = 79;; } //@line 1842 "translate.cpp"
                  $if_then208$$if_then217$111: do { 
                    if (__label__ == 76) {
  
                      var $133=HEAP32[((_translator2)>>2)]; //@line 1845 "translate.cpp"
                      var $134=$word_addr; //@line 1845 "translate.cpp"
                      var $135=$next_pause_addr; //@line 1845 "translate.cpp"
                      var $136=$wtab_addr; //@line 1845 "translate.cpp"
                      var $call209=__Z13TranslateWordP10TranslatorPciP8WORD_TAB($133, $134, $135, $136); //@line 1845 "translate.cpp"
                      $flags=$call209; //@line 1845 "translate.cpp"
                      var $137=$p; //@line 1847 "translate.cpp"
                      var $arrayidx210=(($137)&4294967295); //@line 1847 "translate.cpp"
                      var $138=HEAPU8[($arrayidx210)]; //@line 1847 "translate.cpp"
                      var $conv211=((($138))&255); //@line 1847 "translate.cpp"
                      var $cmp212=((($conv211))|0)==21; //@line 1847 "translate.cpp"
                      if ($cmp212) { __label__ = 77;; } else { __label__ = 78;; } //@line 1847 "translate.cpp"
                      if (__label__ == 77) {
  
                        $switch_phonemes=-1; //@line 1850 "translate.cpp"
                        __label__ = 79;break $if_then208$$if_then217$111; //@line 1851 "translate.cpp"
                      }
                      else if (__label__ == 78) {
  
                        var $_pr15=$switch_phonemes; //@line 1853 "translate.cpp"
                        var $cmp216=((($_pr15))|0) < 0; //@line 1853 "translate.cpp"
                        if ($cmp216) { __label__ = 79;break $if_then208$$if_then217$111; } else { __label__ = 80;break $if_then208$$if_then217$111; } //@line 1853 "translate.cpp"
                      }
                    }
                  } while(0);
                  if (__label__ == 79) {
  
                    var $139=$p; //@line 1856 "translate.cpp"
                    var $arrayidx218=(($139)&4294967295); //@line 1856 "translate.cpp"
                    HEAP8[($arrayidx218)]=13; //@line 1856 "translate.cpp"
                    var $140=$p; //@line 1857 "translate.cpp"
                    var $arrayidx219=(($140+1)&4294967295); //@line 1857 "translate.cpp"
                    HEAP8[($arrayidx219)]=13; //@line 1857 "translate.cpp"
                    var $141=$p; //@line 1858 "translate.cpp"
                    var $arrayidx220=(($141+2)&4294967295); //@line 1858 "translate.cpp"
                    HEAP8[($arrayidx220)]=0; //@line 1858 "translate.cpp"
                    ; //@line 1859 "translate.cpp"
                  }
  
                  var $142=$tr_addr; //@line 1863 "translate.cpp"
                  var $143=$switch_phonemes; //@line 1863 "translate.cpp"
                  var $144=$p; //@line 1863 "translate.cpp"
                  var $call222=__Z24ChangeEquivalentPhonemesP10TranslatoriPc($142, $143, $144); //@line 1863 "translate.cpp"
                  var $tobool223=((($call222))|0)!=0; //@line 1863 "translate.cpp"
                  if ($tobool223) { __label__ = 81;; } else { __label__ = 82;; } //@line 1863 "translate.cpp"
                  if (__label__ == 81) {
  
                    $switch_phonemes=-1; //@line 1866 "translate.cpp"
                    ; //@line 1869 "translate.cpp"
                  }
                  else if (__label__ == 82) {
  
                    var $_pr17=$switch_phonemes; //@line 1869 "translate.cpp"
                    var $cmp226=((($_pr17))|0)==-1; //@line 1869 "translate.cpp"
                    if (!($cmp226)) { __label__ = 84;break $if_then196$$if_end232$106; } //@line 1869 "translate.cpp"
                  }
  
                  var $arraydecay228=(($old_dictionary_name)&4294967295); //@line 1871 "translate.cpp"
                  var $call229=_strcpy(((_dictionary_name)&4294967295), $arraydecay228); //@line 1871 "translate.cpp"
                  var $145=HEAP32[((_voice)>>2)]; //@line 1872 "translate.cpp"
                  var $phoneme_tab_ix=(($145+40)&4294967295); //@line 1872 "translate.cpp"
                  var $146=HEAP32[(($phoneme_tab_ix)>>2)]; //@line 1872 "translate.cpp"
                  __Z18SelectPhonemeTablei($146); //@line 1872 "translate.cpp"
                  var $147=HEAP32[((_voice)>>2)]; //@line 1876 "translate.cpp"
                  var $phoneme_tab_ix230=(($147+40)&4294967295); //@line 1876 "translate.cpp"
                  var $148=HEAP32[(($phoneme_tab_ix230)>>2)]; //@line 1876 "translate.cpp"
                  $switch_phonemes=$148; //@line 1876 "translate.cpp"
                  ; //@line 1877 "translate.cpp"
                }
              } while(0);
  
              var $149=$word_flags; //@line 1880 "translate.cpp"
              var $and233=($149) & 128; //@line 1880 "translate.cpp"
              var $tobool234=((($and233))|0)!=0; //@line 1880 "translate.cpp"
              if ($tobool234) { __label__ = 95;; } else { __label__ = 85;; } //@line 1880 "translate.cpp"
              $if_end261$$if_then235$124: do { 
                if (__label__ == 85) {
  
                  var $150=$flags; //@line 1882 "translate.cpp"
                  var $and236=($150) & 268435456; //@line 1882 "translate.cpp"
                  var $tobool237=((($and236))|0)!=0; //@line 1882 "translate.cpp"
                  if ($tobool237) { __label__ = 86;; } else { __label__ = 88;; } //@line 1882 "translate.cpp"
                  $if_then238$$if_end242$126: do { 
                    if (__label__ == 86) {
  
                      var $151=$pre_pause_addr; //@line 1884 "translate.cpp"
                      var $cmp239=((($151))|0) < 1; //@line 1884 "translate.cpp"
                      if (!($cmp239)) { __label__ = 88;break $if_then238$$if_end242$126; } //@line 1884 "translate.cpp"
  
                      $pre_pause_addr=1; //@line 1885 "translate.cpp"
                      ; //@line 1885 "translate.cpp"
                    }
                  } while(0);
  
                  var $152=$flags; //@line 1887 "translate.cpp"
                  var $and243=($152) & 256; //@line 1887 "translate.cpp"
                  var $tobool244=((($and243))|0)!=0; //@line 1887 "translate.cpp"
                  if (!($tobool244)) { __label__ = 95;break $if_end261$$if_then235$124; } //@line 1887 "translate.cpp"
  
                  var $153=$word_flags; //@line 1887 "translate.cpp"
                  var $tobool246=((($153))|0)!=0; //@line 1887 "translate.cpp"
                  if ($tobool246) { __label__ = 95;break $if_end261$$if_then235$124; } //@line 1887 "translate.cpp"
  
                  var $154=$wtab_addr; //@line 1887 "translate.cpp"
                  var $arrayidx248=(($154+-12)&4294967295); //@line 1887 "translate.cpp"
                  var $flags249=(($arrayidx248)&4294967295); //@line 1887 "translate.cpp"
                  var $155=HEAP32[(($flags249)>>2)]; //@line 1887 "translate.cpp"
                  var $and250=($155) & 512; //@line 1887 "translate.cpp"
                  var $tobool251=((($and250))|0)!=0; //@line 1887 "translate.cpp"
                  if ($tobool251) { __label__ = 95;break $if_end261$$if_then235$124; } //@line 1887 "translate.cpp"
  
                  var $156=$tr_addr; //@line 1887 "translate.cpp"
                  var $prepause_timeout253=(($156+8312)&4294967295); //@line 1887 "translate.cpp"
                  var $157=HEAP32[(($prepause_timeout253)>>2)]; //@line 1887 "translate.cpp"
                  var $cmp254=((($157))|0)==0; //@line 1887 "translate.cpp"
                  if (!($cmp254)) { __label__ = 95;break $if_end261$$if_then235$124; } //@line 1887 "translate.cpp"
  
                  var $158=$pre_pause_addr; //@line 1890 "translate.cpp"
                  var $cmp256=((($158))|0) < 4; //@line 1890 "translate.cpp"
                  if ($cmp256) { __label__ = 93;; } else { __label__ = 94;; } //@line 1890 "translate.cpp"
                  if (__label__ == 93) {
  
                    $pre_pause_addr=4; //@line 1890 "translate.cpp"
                    ; //@line 1890 "translate.cpp"
                  }
  
                  var $159=$tr_addr; //@line 1891 "translate.cpp"
                  var $prepause_timeout259=(($159+8312)&4294967295); //@line 1891 "translate.cpp"
                  HEAP32[(($prepause_timeout259)>>2)]=3; //@line 1891 "translate.cpp"
                  ; //@line 1892 "translate.cpp"
                }
              } while(0);
  
              var $160=HEAP32[((__ZL15option_emphasis)>>2)]; //@line 1895 "translate.cpp"
              var $cmp262=((($160))|0) >= 3; //@line 1895 "translate.cpp"
              if (!($cmp262)) { __label__ = 98;break $if_then75$$if_else100$47; } //@line 1895 "translate.cpp"
  
              var $161=$pre_pause_addr; //@line 1895 "translate.cpp"
              var $cmp264=((($161))|0) < 1; //@line 1895 "translate.cpp"
              if (!($cmp264)) { __label__ = 98;break $if_then75$$if_else100$47; } //@line 1895 "translate.cpp"
  
              $pre_pause_addr=1; //@line 1896 "translate.cpp"
              ; //@line 1896 "translate.cpp"
            }
          }
        } while(0);
  
        var $162=HEAP32[((_n_ph_list2)>>2)]; //@line 1899 "translate.cpp"
        var $arrayidx268=((_ph_list2+$162*8)&4294967295); //@line 1899 "translate.cpp"
        $plist2=$arrayidx268; //@line 1899 "translate.cpp"
        $stress=0; //@line 1900 "translate.cpp"
        $next_stress=1; //@line 1901 "translate.cpp"
        $srcix=0; //@line 1902 "translate.cpp"
        $max_stress=-1; //@line 1903 "translate.cpp"
        $found_dict_flag=0; //@line 1905 "translate.cpp"
        var $163=$flags; //@line 1906 "translate.cpp"
        var $and269=($163) & -2147483648; //@line 1906 "translate.cpp"
        var $tobool270=((($and269))|0)!=0; //@line 1906 "translate.cpp"
        if ($tobool270) { __label__ = 99;; } else { __label__ = 101;; } //@line 1906 "translate.cpp"
        $land_lhs_true271$$while_cond276_preheader$141: do { 
          if (__label__ == 99) {
  
            var $164=$flags; //@line 1906 "translate.cpp"
            var $and272=($164) & 536870912; //@line 1906 "translate.cpp"
            var $tobool273=((($and272))|0)!=0; //@line 1906 "translate.cpp"
            if ($tobool273) { __label__ = 101;break $land_lhs_true271$$while_cond276_preheader$141; } //@line 1906 "translate.cpp"
  
            $found_dict_flag=16; //@line 1907 "translate.cpp"
            ; //@line 1907 "translate.cpp"
          }
        } while(0);
  
        var $165=$pre_pause_addr; //@line 1909 "translate.cpp"
        var $cmp27729=((($165))|0) > 0; //@line 1909 "translate.cpp"
        var $166=HEAP32[((_n_ph_list2)>>2)]; //@line 1909 "translate.cpp"
        var $cmp27930=((($166))|0) < 996; //@line 1909 "translate.cpp"
        var $or_cond531=($cmp27729) & ($cmp27930); //@line 1909 "translate.cpp"
        if ($or_cond531) { __lastLabel__ = 101; __label__ = 102;; } else { __lastLabel__ = 101; __label__ = 106;; } //@line 1909 "translate.cpp"
        $while_body281$$while_end292$145: do { 
          if (__label__ == 102) {
            while(1) { 
  
              var $167=__lastLabel__ == 101 ? $165 : ($173); //@line 1913 "translate.cpp"
              var $cmp282=((($167))|0) > 1; //@line 1913 "translate.cpp"
              var $168=HEAP32[((_n_ph_list2)>>2)]; //@line 1915 "translate.cpp"
              var $inc284=((($168)+1)&4294967295); //@line 1915 "translate.cpp"
              HEAP32[((_n_ph_list2)>>2)]=$inc284; //@line 1915 "translate.cpp"
              var $arrayidx285=((_ph_list2+$168*8)&4294967295); //@line 1915 "translate.cpp"
              if ($cmp282) { __label__ = 103;; } else { __label__ = 104;; } //@line 1913 "translate.cpp"
              if (__label__ == 103) {
  
                __ZL9SetPlist2P13PHONEME_LIST2h($arrayidx285, 9); //@line 1915 "translate.cpp"
                var $169=$pre_pause_addr; //@line 1916 "translate.cpp"
                var $sub286=((($169)-2)&4294967295); //@line 1916 "translate.cpp"
                $pre_pause_addr=$sub286; //@line 1916 "translate.cpp"
                ; //@line 1917 "translate.cpp"
              }
              else if (__label__ == 104) {
  
                __ZL9SetPlist2P13PHONEME_LIST2h($arrayidx285, 11); //@line 1920 "translate.cpp"
                var $170=$pre_pause_addr; //@line 1921 "translate.cpp"
                var $dec290=((($170)-1)&4294967295); //@line 1921 "translate.cpp"
                $pre_pause_addr=$dec290; //@line 1921 "translate.cpp"
                ;
              }
  
              var $171=$tr_addr; //@line 1923 "translate.cpp"
              var $end_stressed_vowel=(($171+8316)&4294967295); //@line 1923 "translate.cpp"
              HEAP32[(($end_stressed_vowel)>>2)]=0; //@line 1923 "translate.cpp"
              var $172=$tr_addr; //@line 1924 "translate.cpp"
              var $prev_dict_flags=(($172+8320)&4294967295); //@line 1924 "translate.cpp"
              HEAP32[(($prev_dict_flags)>>2)]=0; //@line 1924 "translate.cpp"
              var $173=$pre_pause_addr; //@line 1909 "translate.cpp"
              var $cmp277=((($173))|0) > 0; //@line 1909 "translate.cpp"
              var $174=HEAP32[((_n_ph_list2)>>2)]; //@line 1909 "translate.cpp"
              var $cmp279=((($174))|0) < 996; //@line 1909 "translate.cpp"
              var $or_cond5=($cmp277) & ($cmp279); //@line 1909 "translate.cpp"
              if ($or_cond5) { __lastLabel__ = 105; __label__ = 102;continue ; } else { __lastLabel__ = 105; __label__ = 106;break $while_body281$$while_end292$145; } //@line 1909 "translate.cpp"
            }
          }
        } while(0);
  
        var $175=HEAP32[((_option_capitals)>>2)]; //@line 1927 "translate.cpp"
        var $cmp293=((($175))|0)==1; //@line 1927 "translate.cpp"
        if ($cmp293) { __label__ = 107;; } else { __label__ = 111;; } //@line 1927 "translate.cpp"
        $land_lhs_true294$$if_end315$153: do { 
          if (__label__ == 107) {
  
            var $176=$word_flags; //@line 1927 "translate.cpp"
            var $and295=($176) & 2; //@line 1927 "translate.cpp"
            var $tobool296=((($and295))|0)!=0; //@line 1927 "translate.cpp"
            if (!($tobool296)) { __label__ = 111;break $land_lhs_true294$$if_end315$153; } //@line 1927 "translate.cpp"
  
            var $177=HEAP32[((_n_ph_list2)>>2)]; //@line 1929 "translate.cpp"
            var $inc298=((($177)+1)&4294967295); //@line 1929 "translate.cpp"
            HEAP32[((_n_ph_list2)>>2)]=$inc298; //@line 1929 "translate.cpp"
            var $arrayidx299=((_ph_list2+$177*8)&4294967295); //@line 1929 "translate.cpp"
            __ZL9SetPlist2P13PHONEME_LIST2h($arrayidx299, 10); //@line 1929 "translate.cpp"
            var $178=HEAP32[((_n_ph_list2)>>2)]; //@line 1930 "translate.cpp"
            var $inc300=((($178)+1)&4294967295); //@line 1930 "translate.cpp"
            HEAP32[((_n_ph_list2)>>2)]=$inc300; //@line 1930 "translate.cpp"
            var $arrayidx301=((_ph_list2+$178*8)&4294967295); //@line 1930 "translate.cpp"
            __ZL9SetPlist2P13PHONEME_LIST2h($arrayidx301, 18); //@line 1930 "translate.cpp"
            var $179=$word_flags; //@line 1931 "translate.cpp"
            var $and302=($179) & 1; //@line 1931 "translate.cpp"
            var $tobool303=((($and302))|0)!=0; //@line 1931 "translate.cpp"
            if (!($tobool303)) { __label__ = 111;break $land_lhs_true294$$if_end315$153; } //@line 1931 "translate.cpp"
  
            var $180=$word_addr; //@line 1931 "translate.cpp"
            var $arrayidx305=(($180+1)&4294967295); //@line 1931 "translate.cpp"
            var $181=HEAP8[($arrayidx305)]; //@line 1931 "translate.cpp"
            var $conv306=(tempInt=(($181)),(tempInt>=128?tempInt-256:tempInt)); //@line 1931 "translate.cpp"
            var $call307=__Z7IsAlphaj($conv306); //@line 1931 "translate.cpp"
            var $tobool308=((($call307))|0)!=0; //@line 1931 "translate.cpp"
            if (!($tobool308)) { __label__ = 111;break $land_lhs_true294$$if_end315$153; } //@line 1931 "translate.cpp"
  
            var $182=HEAP32[((_n_ph_list2)>>2)]; //@line 1934 "translate.cpp"
            var $inc310=((($182)+1)&4294967295); //@line 1934 "translate.cpp"
            HEAP32[((_n_ph_list2)>>2)]=$inc310; //@line 1934 "translate.cpp"
            var $arrayidx311=((_ph_list2+$182*8)&4294967295); //@line 1934 "translate.cpp"
            __ZL9SetPlist2P13PHONEME_LIST2h($arrayidx311, 10); //@line 1934 "translate.cpp"
            var $183=HEAP32[((_n_ph_list2)>>2)]; //@line 1935 "translate.cpp"
            var $inc312=((($183)+1)&4294967295); //@line 1935 "translate.cpp"
            HEAP32[((_n_ph_list2)>>2)]=$inc312; //@line 1935 "translate.cpp"
            var $arrayidx313=((_ph_list2+$183*8)&4294967295); //@line 1935 "translate.cpp"
            __ZL9SetPlist2P13PHONEME_LIST2h($arrayidx313, 18); //@line 1935 "translate.cpp"
            ; //@line 1936 "translate.cpp"
          }
        } while(0);
  
        var $184=$switch_phonemes; //@line 1939 "translate.cpp"
        var $cmp316=((($184))|0) >= 0; //@line 1939 "translate.cpp"
        if ($cmp316) { __label__ = 112;; } else { __label__ = 113;; } //@line 1939 "translate.cpp"
        if (__label__ == 112) {
  
          var $185=HEAP32[((_n_ph_list2)>>2)]; //@line 1942 "translate.cpp"
          var $arrayidx318=((_ph_list2+$185*8)&4294967295); //@line 1942 "translate.cpp"
          __ZL9SetPlist2P13PHONEME_LIST2h($arrayidx318, 21); //@line 1942 "translate.cpp"
          var $186=$switch_phonemes; //@line 1943 "translate.cpp"
          var $conv319=((($186)) & 255); //@line 1943 "translate.cpp"
          var $187=HEAP32[((_n_ph_list2)>>2)]; //@line 1943 "translate.cpp"
          var $inc320=((($187)+1)&4294967295); //@line 1943 "translate.cpp"
          HEAP32[((_n_ph_list2)>>2)]=$inc320; //@line 1943 "translate.cpp"
          var $arrayidx321=((_ph_list2+$187*8)&4294967295); //@line 1943 "translate.cpp"
          var $tone_ph=(($arrayidx321+3)&4294967295); //@line 1943 "translate.cpp"
          HEAP8[($tone_ph)]=$conv319; //@line 1943 "translate.cpp"
          ; //@line 1944 "translate.cpp"
        }
  
        var $188=$word_flags; //@line 1947 "translate.cpp"
        var $and323=($188) & 128; //@line 1947 "translate.cpp"
        var $tobool324=((($and323))|0)!=0; //@line 1947 "translate.cpp"
        if ($tobool324) { __label__ = 114;; } else { __label__ = 116;; } //@line 1947 "translate.cpp"
        $land_lhs_true325$$if_end331$162: do { 
          if (__label__ == 114) {
  
            var $189=$p; //@line 1947 "translate.cpp"
            var $190=HEAPU8[($189)]; //@line 1947 "translate.cpp"
            var $idxprom=((($190))&255); //@line 1947 "translate.cpp"
            var $arrayidx326=((_phoneme_tab+$idxprom*4)&4294967295); //@line 1947 "translate.cpp"
            var $191=HEAP32[(($arrayidx326)>>2)]; //@line 1947 "translate.cpp"
            var $type=(($191+11)&4294967295); //@line 1947 "translate.cpp"
            var $192=HEAPU8[($type)]; //@line 1947 "translate.cpp"
            var $conv327=((($192))&255); //@line 1947 "translate.cpp"
            var $cmp328=((($conv327))|0)==0; //@line 1947 "translate.cpp"
            if (!($cmp328)) { __label__ = 116;break $land_lhs_true325$$if_end331$162; } //@line 1947 "translate.cpp"
  
            var $193=$p; //@line 1948 "translate.cpp"
            var $incdec_ptr330=(($193+1)&4294967295); //@line 1948 "translate.cpp"
            $p=$incdec_ptr330; //@line 1948 "translate.cpp"
            ; //@line 1948 "translate.cpp"
          }
        } while(0);
  
        var $194=$p; //@line 1950 "translate.cpp"
        var $arrayidx332=(($194)&4294967295); //@line 1950 "translate.cpp"
        var $195=HEAPU8[($arrayidx332)]; //@line 1950 "translate.cpp"
        var $conv333=((($195))&255); //@line 1950 "translate.cpp"
        var $cmp334=((($conv333))|0)==0; //@line 1950 "translate.cpp"
        if ($cmp334) { __label__ = 117;; } else { __label__ = 119;; } //@line 1950 "translate.cpp"
        $land_lhs_true335$$while_cond341_preheader$166: do { 
          if (__label__ == 117) {
  
            var $_b2=HEAPU8[(__ZL13embedded_flag_b)];
            var $196=($_b2) ? 2 : 0;
            var $tobool336=((($196))|0)!=0; //@line 1950 "translate.cpp"
            if (!($tobool336)) { __label__ = 119;break $land_lhs_true335$$while_cond341_preheader$166; } //@line 1950 "translate.cpp"
  
            var $197=$p; //@line 1953 "translate.cpp"
            var $arrayidx338=(($197)&4294967295); //@line 1953 "translate.cpp"
            HEAP8[($arrayidx338)]=23; //@line 1953 "translate.cpp"
            var $198=$p; //@line 1954 "translate.cpp"
            var $arrayidx339=(($198+1)&4294967295); //@line 1954 "translate.cpp"
            HEAP8[($arrayidx339)]=0; //@line 1954 "translate.cpp"
            ; //@line 1955 "translate.cpp"
          }
        } while(0);
  
        var $199=$p; //@line 1957 "translate.cpp"
        var $incdec_ptr34224=(($199+1)&4294967295); //@line 1957 "translate.cpp"
        $p=$incdec_ptr34224; //@line 1957 "translate.cpp"
        var $200=HEAPU8[($199)]; //@line 1957 "translate.cpp"
        $ph_code=$200; //@line 1957 "translate.cpp"
        var $conv34325=((($200))&255); //@line 1957 "translate.cpp"
        var $cmp34426=((($conv34325))|0)!=0; //@line 1957 "translate.cpp"
        var $201=HEAP32[((_n_ph_list2)>>2)]; //@line 1957 "translate.cpp"
        var $cmp34627=((($201))|0) < 996; //@line 1957 "translate.cpp"
        var $or_cond628=($cmp34426) & ($cmp34627); //@line 1957 "translate.cpp"
        if ($or_cond628) { __label__ = 120;; } else { __label__ = 157;; } //@line 1957 "translate.cpp"
        $while_body348$$while_end511$170: do { 
          if (__label__ == 120) {
            $while_body348$171: while(1) { 
  
              var $202=$ph_code; //@line 1959 "translate.cpp"
              var $conv349=((($202))&255); //@line 1959 "translate.cpp"
              var $cmp350=((($conv349))|0)==255; //@line 1959 "translate.cpp"
              if ($cmp350) { __label__ = 121;; } else { __label__ = 122;; } //@line 1959 "translate.cpp"
              $while_cond341_backedge$$if_end352$173: do { 
                if (__label__ == 122) {
  
                  var $206=$ph_code; //@line 1963 "translate.cpp"
                  var $idxprom353=((($206))&255); //@line 1963 "translate.cpp"
                  var $arrayidx354=((_phoneme_tab+$idxprom353*4)&4294967295); //@line 1963 "translate.cpp"
                  var $207=HEAP32[(($arrayidx354)>>2)]; //@line 1963 "translate.cpp"
                  $ph=$207; //@line 1963 "translate.cpp"
                  var $208=$ph_code; //@line 1965 "translate.cpp"
                  var $conv355=((($208))&255); //@line 1965 "translate.cpp"
                  var $cmp356=((($conv355))|0)==21; //@line 1965 "translate.cpp"
                  if ($cmp356) { __label__ = 123;; } else { __label__ = 124;; } //@line 1965 "translate.cpp"
                  if (__label__ == 123) {
  
                    var $209=$ph_code; //@line 1967 "translate.cpp"
                    var $210=HEAP32[((_n_ph_list2)>>2)]; //@line 1967 "translate.cpp"
                    var $arrayidx358=((_ph_list2+$210*8)&4294967295); //@line 1967 "translate.cpp"
                    var $phcode=(($arrayidx358)&4294967295); //@line 1967 "translate.cpp"
                    HEAP8[($phcode)]=$209; //@line 1967 "translate.cpp"
                    var $211=HEAP32[((_n_ph_list2)>>2)]; //@line 1968 "translate.cpp"
                    var $arrayidx359=((_ph_list2+$211*8)&4294967295); //@line 1968 "translate.cpp"
                    var $sourceix360=(($arrayidx359+6)&4294967295); //@line 1968 "translate.cpp"
                    HEAP16[(($sourceix360)>>1)]=0; //@line 1968 "translate.cpp"
                    var $_b1=HEAPU8[(__ZL13embedded_flag_b)];
                    var $212=($_b1) ? 2 : 0;
                    var $conv361=((($212)) & 65535); //@line 1969 "translate.cpp"
                    var $213=HEAP32[((_n_ph_list2)>>2)]; //@line 1969 "translate.cpp"
                    var $arrayidx362=((_ph_list2+$213*8)&4294967295); //@line 1969 "translate.cpp"
                    var $synthflags=(($arrayidx362+4)&4294967295); //@line 1969 "translate.cpp"
                    HEAP16[(($synthflags)>>1)]=$conv361; //@line 1969 "translate.cpp"
                    var $214=$p; //@line 1970 "translate.cpp"
                    var $incdec_ptr363=(($214+1)&4294967295); //@line 1970 "translate.cpp"
                    $p=$incdec_ptr363; //@line 1970 "translate.cpp"
                    var $215=HEAP8[($214)]; //@line 1970 "translate.cpp"
                    var $216=HEAP32[((_n_ph_list2)>>2)]; //@line 1970 "translate.cpp"
                    var $inc364=((($216)+1)&4294967295); //@line 1970 "translate.cpp"
                    HEAP32[((_n_ph_list2)>>2)]=$inc364; //@line 1970 "translate.cpp"
                    var $arrayidx365=((_ph_list2+$216*8)&4294967295); //@line 1970 "translate.cpp"
                    var $tone_ph366=(($arrayidx365+3)&4294967295); //@line 1970 "translate.cpp"
                    HEAP8[($tone_ph366)]=$215; //@line 1970 "translate.cpp"
                    ; //@line 1971 "translate.cpp"
                  }
                  else if (__label__ == 124) {
  
                    var $217=$ph; //@line 1973 "translate.cpp"
                    var $type368=(($217+11)&4294967295); //@line 1973 "translate.cpp"
                    var $218=HEAPU8[($type368)]; //@line 1973 "translate.cpp"
                    var $conv369=((($218))&255); //@line 1973 "translate.cpp"
                    var $cmp370=((($conv369))|0)==1; //@line 1973 "translate.cpp"
                    if ($cmp370) { __label__ = 125;; } else { __label__ = 130;; } //@line 1973 "translate.cpp"
                    if (__label__ == 125) {
  
                      var $219=$ph; //@line 1978 "translate.cpp"
                      var $program=(($219+8)&4294967295); //@line 1978 "translate.cpp"
                      var $220=HEAPU16[(($program)>>1)]; //@line 1978 "translate.cpp"
                      var $conv372=((($220))&65535); //@line 1978 "translate.cpp"
                      var $cmp373=((($conv372))|0)==0; //@line 1978 "translate.cpp"
                      if ($cmp373) { __label__ = 126;; } else { __label__ = 127;; } //@line 1978 "translate.cpp"
                      if (__label__ == 126) {
  
                        var $221=$ph; //@line 1979 "translate.cpp"
                        var $std_length=(($221+14)&4294967295); //@line 1979 "translate.cpp"
                        var $222=HEAPU8[($std_length)]; //@line 1979 "translate.cpp"
                        var $conv375=((($222))&255); //@line 1979 "translate.cpp"
                        $next_stress=$conv375; //@line 1979 "translate.cpp"
                        ; //@line 1979 "translate.cpp"
                      }
                      else if (__label__ == 127) {
  
                        var $223=$prev_vowel; //@line 1983 "translate.cpp"
                        var $cmp377=((($223))|0) >= 0; //@line 1983 "translate.cpp"
                        var $224=$ph_code; //@line 1985 "translate.cpp"
                        if ($cmp377) { __label__ = 128;; } else { __label__ = 129;; } //@line 1983 "translate.cpp"
                        if (__label__ == 128) {
  
                          var $225=$prev_vowel; //@line 1985 "translate.cpp"
                          var $arrayidx379=((_ph_list2+$225*8)&4294967295); //@line 1985 "translate.cpp"
                          var $tone_ph380=(($arrayidx379+3)&4294967295); //@line 1985 "translate.cpp"
                          HEAP8[($tone_ph380)]=$224; //@line 1985 "translate.cpp"
                          ; //@line 1986 "translate.cpp"
                        }
                        else if (__label__ == 129) {
  
                          var $conv382=((($224))&255); //@line 1989 "translate.cpp"
                          $next_tone=$conv382; //@line 1989 "translate.cpp"
                          ;
                        }
                      }
                    }
                    else if (__label__ == 130) {
  
                      var $226=$ph_code; //@line 1994 "translate.cpp"
                      var $conv386=((($226))&255); //@line 1994 "translate.cpp"
                      var $cmp387=((($conv386))|0)==20; //@line 1994 "translate.cpp"
                      if ($cmp387) { __label__ = 131;; } else { __label__ = 132;; } //@line 1994 "translate.cpp"
                      if (__label__ == 131) {
  
                        var $227=HEAP32[((_n_ph_list2)>>2)]; //@line 1997 "translate.cpp"
                        var $sub389=((($227)-1)&4294967295); //@line 1997 "translate.cpp"
                        $prev_vowel=$sub389; //@line 1997 "translate.cpp"
                        var $228=$prev_vowel; //@line 1998 "translate.cpp"
                        var $arrayidx390=((_ph_list2+$228*8)&4294967295); //@line 1998 "translate.cpp"
                        var $synthflags391=(($arrayidx390+4)&4294967295); //@line 1998 "translate.cpp"
                        var $229=HEAPU16[(($synthflags391)>>1)]; //@line 1998 "translate.cpp"
                        var $conv392=((($229))&65535); //@line 1998 "translate.cpp"
                        var $or393=($conv392) | 4; //@line 1998 "translate.cpp"
                        var $conv394=((($or393)) & 65535); //@line 1998 "translate.cpp"
                        HEAP16[(($synthflags391)>>1)]=$conv394; //@line 1998 "translate.cpp"
                        var $230=$next_stress; //@line 1999 "translate.cpp"
                        var $conv395=((($230)) & 255); //@line 1999 "translate.cpp"
                        var $231=$prev_vowel; //@line 1999 "translate.cpp"
                        var $arrayidx396=((_ph_list2+$231*8)&4294967295); //@line 1999 "translate.cpp"
                        var $stresslevel=(($arrayidx396+1)&4294967295); //@line 1999 "translate.cpp"
                        HEAP8[($stresslevel)]=$conv395; //@line 1999 "translate.cpp"
                        ; //@line 2000 "translate.cpp"
                      }
                      else if (__label__ == 132) {
  
                        var $232=$ph_code; //@line 2002 "translate.cpp"
                        var $conv398=((($232))&255); //@line 2002 "translate.cpp"
                        var $cmp399=((($conv398))|0)==12; //@line 2002 "translate.cpp"
                        if ($cmp399) { __label__ = 133;; } else { __label__ = 134;; } //@line 2002 "translate.cpp"
                        if (__label__ == 133) {
  
                          var $233=HEAP32[((_n_ph_list2)>>2)]; //@line 2004 "translate.cpp"
                          var $sub401=((($233)-1)&4294967295); //@line 2004 "translate.cpp"
                          var $arrayidx402=((_ph_list2+$sub401*8)&4294967295); //@line 2004 "translate.cpp"
                          var $synthflags403=(($arrayidx402+4)&4294967295); //@line 2004 "translate.cpp"
                          var $234=HEAPU16[(($synthflags403)>>1)]; //@line 2004 "translate.cpp"
                          var $conv404=((($234))&65535); //@line 2004 "translate.cpp"
                          var $or405=($conv404) | 8; //@line 2004 "translate.cpp"
                          var $conv406=((($or405)) & 65535); //@line 2004 "translate.cpp"
                          HEAP16[(($synthflags403)>>1)]=$conv406; //@line 2004 "translate.cpp"
                          ; //@line 2005 "translate.cpp"
                        }
                        else if (__label__ == 134) {
  
                          var $235=$ph_code; //@line 2007 "translate.cpp"
                          var $conv408=((($235))&255); //@line 2007 "translate.cpp"
                          var $cmp409=((($conv408))|0)==15; //@line 2007 "translate.cpp"
                          if ($cmp409) { __label__ = 135;; } else { __label__ = 136;; } //@line 2007 "translate.cpp"
                          if (__label__ == 135) {
  
                            var $236=$source_ix; //@line 2012 "translate.cpp"
                            var $add411=((($236)+1)&4294967295); //@line 2012 "translate.cpp"
                            $srcix=$add411; //@line 2012 "translate.cpp"
                            ; //@line 2013 "translate.cpp"
                          }
                          else if (__label__ == 136) {
  
                            var $237=$ph_code; //@line 2015 "translate.cpp"
                            var $conv413=((($237))&255); //@line 2015 "translate.cpp"
                            var $cmp414=((($conv413))|0)==22; //@line 2015 "translate.cpp"
                            if ($cmp414) { __label__ = 137;; } else { __label__ = 139;; } //@line 2015 "translate.cpp"
                            if (__label__ == 137) {
  
                              var $238=$tr_addr; //@line 2018 "translate.cpp"
                              var $langopts416=(($238)&4294967295); //@line 2018 "translate.cpp"
                              var $param417=(($langopts416+24)&4294967295); //@line 2018 "translate.cpp"
                              var $arrayidx418=(($param417+56)&4294967295); //@line 2018 "translate.cpp"
                              var $239=HEAP32[(($arrayidx418)>>2)]; //@line 2018 "translate.cpp"
                              var $tobool419=((($239))|0)!=0; //@line 2018 "translate.cpp"
                              if (!($tobool419)) { __label__ = 121;break $while_cond341_backedge$$if_end352$173; } //@line 2018 "translate.cpp"
  
                              var $240=$flags; //@line 2020 "translate.cpp"
                              var $or421=($240) | 524288; //@line 2020 "translate.cpp"
                              $flags=$or421; //@line 2020 "translate.cpp"
                              ; //@line 2021 "translate.cpp"
                            }
                            else if (__label__ == 139) {
  
                              var $241=$ph_code; //@line 2025 "translate.cpp"
                              var $242=HEAP32[((_n_ph_list2)>>2)]; //@line 2025 "translate.cpp"
                              var $arrayidx424=((_ph_list2+$242*8)&4294967295); //@line 2025 "translate.cpp"
                              var $phcode425=(($arrayidx424)&4294967295); //@line 2025 "translate.cpp"
                              HEAP8[($phcode425)]=$241; //@line 2025 "translate.cpp"
                              var $243=HEAP32[((_n_ph_list2)>>2)]; //@line 2026 "translate.cpp"
                              var $arrayidx426=((_ph_list2+$243*8)&4294967295); //@line 2026 "translate.cpp"
                              var $tone_ph427=(($arrayidx426+3)&4294967295); //@line 2026 "translate.cpp"
                              HEAP8[($tone_ph427)]=0; //@line 2026 "translate.cpp"
                              var $_b=HEAPU8[(__ZL13embedded_flag_b)];
                              var $244=($_b) ? 2 : 0;
                              var $245=$found_dict_flag; //@line 2027 "translate.cpp"
                              var $or428=($244) | ($245); //@line 2027 "translate.cpp"
                              var $conv429=((($or428)) & 65535); //@line 2027 "translate.cpp"
                              var $246=HEAP32[((_n_ph_list2)>>2)]; //@line 2027 "translate.cpp"
                              var $arrayidx430=((_ph_list2+$246*8)&4294967295); //@line 2027 "translate.cpp"
                              var $synthflags431=(($arrayidx430+4)&4294967295); //@line 2027 "translate.cpp"
                              HEAP16[(($synthflags431)>>1)]=$conv429; //@line 2027 "translate.cpp"
                              HEAP8[(__ZL13embedded_flag_b)]=0;
                              var $247=$srcix; //@line 2029 "translate.cpp"
                              var $conv432=((($247)) & 65535); //@line 2029 "translate.cpp"
                              var $248=HEAP32[((_n_ph_list2)>>2)]; //@line 2029 "translate.cpp"
                              var $arrayidx433=((_ph_list2+$248*8)&4294967295); //@line 2029 "translate.cpp"
                              var $sourceix434=(($arrayidx433+6)&4294967295); //@line 2029 "translate.cpp"
                              HEAP16[(($sourceix434)>>1)]=$conv432; //@line 2029 "translate.cpp"
                              $srcix=0; //@line 2030 "translate.cpp"
                              var $249=$ph; //@line 2032 "translate.cpp"
                              var $type435=(($249+11)&4294967295); //@line 2032 "translate.cpp"
                              var $250=HEAPU8[($type435)]; //@line 2032 "translate.cpp"
                              var $conv436=((($250))&255); //@line 2032 "translate.cpp"
                              var $cmp437=((($conv436))|0)==2; //@line 2032 "translate.cpp"
                              if ($cmp437) { __label__ = 140;; } else { __label__ = 149;; } //@line 2032 "translate.cpp"
                              $if_then438$$if_else466$200: do { 
                                if (__label__ == 140) {
  
                                  var $251=$next_stress; //@line 2034 "translate.cpp"
                                  $stress=$251; //@line 2034 "translate.cpp"
                                  $next_stress=1; //@line 2035 "translate.cpp"
                                  var $252=$stress; //@line 2037 "translate.cpp"
                                  var $cmp439=((($252))|0) >= 4; //@line 2037 "translate.cpp"
                                  if ($cmp439) { __label__ = 141;; } else { __label__ = 142;; } //@line 2037 "translate.cpp"
                                  if (__label__ == 141) {
  
                                    HEAP8[(__ZL18any_stressed_words_b)]=1;
                                    ; //@line 2040 "translate.cpp"
                                  }
  
                                  var $253=$prev_vowel; //@line 2042 "translate.cpp"
                                  var $cmp442=((($253))|0) >= 0; //@line 2042 "translate.cpp"
                                  if ($cmp442) { __label__ = 143;; } else { __label__ = 145;; } //@line 2042 "translate.cpp"
                                  $land_lhs_true443$$if_end451$205: do { 
                                    if (__label__ == 143) {
  
                                      var $254=HEAP32[((_n_ph_list2)>>2)]; //@line 2042 "translate.cpp"
                                      var $sub444=((($254)-1)&4294967295); //@line 2042 "translate.cpp"
                                      var $255=$prev_vowel; //@line 2042 "translate.cpp"
                                      var $cmp445=((($sub444))|0)!=((($255))|0); //@line 2042 "translate.cpp"
                                      if (!($cmp445)) { __label__ = 145;break $land_lhs_true443$$if_end451$205; } //@line 2042 "translate.cpp"
  
                                      var $256=$stress; //@line 2043 "translate.cpp"
                                      var $conv447=((($256)) & 255); //@line 2043 "translate.cpp"
                                      var $257=HEAP32[((_n_ph_list2)>>2)]; //@line 2043 "translate.cpp"
                                      var $sub448=((($257)-1)&4294967295); //@line 2043 "translate.cpp"
                                      var $arrayidx449=((_ph_list2+$sub448*8)&4294967295); //@line 2043 "translate.cpp"
                                      var $stresslevel450=(($arrayidx449+1)&4294967295); //@line 2043 "translate.cpp"
                                      HEAP8[($stresslevel450)]=$conv447; //@line 2043 "translate.cpp"
                                      ; //@line 2043 "translate.cpp"
                                    }
                                  } while(0);
  
                                  var $258=HEAP32[((_n_ph_list2)>>2)]; //@line 2045 "translate.cpp"
                                  var $arrayidx452=((_ph_list2+$258*8)&4294967295); //@line 2045 "translate.cpp"
                                  var $synthflags453=(($arrayidx452+4)&4294967295); //@line 2045 "translate.cpp"
                                  var $259=HEAPU16[(($synthflags453)>>1)]; //@line 2045 "translate.cpp"
                                  var $conv454=((($259))&65535); //@line 2045 "translate.cpp"
                                  var $or455=($conv454) | 4; //@line 2045 "translate.cpp"
                                  var $conv456=((($or455)) & 65535); //@line 2045 "translate.cpp"
                                  HEAP16[(($synthflags453)>>1)]=$conv456; //@line 2045 "translate.cpp"
                                  var $260=HEAP32[((_n_ph_list2)>>2)]; //@line 2046 "translate.cpp"
                                  $prev_vowel=$260; //@line 2046 "translate.cpp"
                                  var $261=$stress; //@line 2048 "translate.cpp"
                                  var $262=$max_stress; //@line 2048 "translate.cpp"
                                  var $cmp457=((($261))|0) > ((($262))|0); //@line 2048 "translate.cpp"
                                  if ($cmp457) { __label__ = 146;; } else { __label__ = 147;; } //@line 2048 "translate.cpp"
                                  if (__label__ == 146) {
  
                                    var $263=$stress; //@line 2050 "translate.cpp"
                                    $max_stress=$263; //@line 2050 "translate.cpp"
                                    var $264=HEAP32[((_n_ph_list2)>>2)]; //@line 2051 "translate.cpp"
                                    $max_stress_ix=$264; //@line 2051 "translate.cpp"
                                    ; //@line 2052 "translate.cpp"
                                  }
  
                                  var $265=$next_tone; //@line 2053 "translate.cpp"
                                  var $cmp460=((($265))|0)!=0; //@line 2053 "translate.cpp"
                                  if (!($cmp460)) { __label__ = 156;break $if_then438$$if_else466$200; } //@line 2053 "translate.cpp"
  
                                  var $266=$next_tone; //@line 2055 "translate.cpp"
                                  var $conv462=((($266)) & 255); //@line 2055 "translate.cpp"
                                  var $267=HEAP32[((_n_ph_list2)>>2)]; //@line 2055 "translate.cpp"
                                  var $arrayidx463=((_ph_list2+$267*8)&4294967295); //@line 2055 "translate.cpp"
                                  var $tone_ph464=(($arrayidx463+3)&4294967295); //@line 2055 "translate.cpp"
                                  HEAP8[($tone_ph464)]=$conv462; //@line 2055 "translate.cpp"
                                  $next_tone=0; //@line 2056 "translate.cpp"
                                  ; //@line 2057 "translate.cpp"
                                }
                                else if (__label__ == 149) {
  
                                  var $268=$first_phoneme; //@line 2061 "translate.cpp"
                                  var $tobool467=((($268))|0)!=0; //@line 2061 "translate.cpp"
                                  if (!($tobool467)) { __label__ = 156;break $if_then438$$if_else466$200; } //@line 2061 "translate.cpp"
  
                                  var $269=$tr_addr; //@line 2061 "translate.cpp"
                                  var $langopts469=(($269)&4294967295); //@line 2061 "translate.cpp"
                                  var $param470=(($langopts469+24)&4294967295); //@line 2061 "translate.cpp"
                                  var $arrayidx471=(($param470+56)&4294967295); //@line 2061 "translate.cpp"
                                  var $270=HEAP32[(($arrayidx471)>>2)]; //@line 2061 "translate.cpp"
                                  var $tobool472=((($270))|0)!=0; //@line 2061 "translate.cpp"
                                  if (!($tobool472)) { __label__ = 156;break $if_then438$$if_else466$200; } //@line 2061 "translate.cpp"
  
                                  var $271=$tr_addr; //@line 2063 "translate.cpp"
                                  var $prev_dict_flags474=(($271+8320)&4294967295); //@line 2063 "translate.cpp"
                                  var $272=HEAP32[(($prev_dict_flags474)>>2)]; //@line 2063 "translate.cpp"
                                  var $and475=($272) & 524288; //@line 2063 "translate.cpp"
                                  var $tobool476=((($and475))|0)!=0; //@line 2063 "translate.cpp"
                                  if ($tobool476) { __label__ = 152;; } else { __label__ = 153;; } //@line 2063 "translate.cpp"
                                  $land_lhs_true477$$lor_lhs_false483$216: do { 
                                    if (__label__ == 152) {
  
                                      var $273=$tr_addr; //@line 2063 "translate.cpp"
                                      var $langopts478=(($273)&4294967295); //@line 2063 "translate.cpp"
                                      var $param479=(($langopts478+24)&4294967295); //@line 2063 "translate.cpp"
                                      var $arrayidx480=(($param479+56)&4294967295); //@line 2063 "translate.cpp"
                                      var $274=HEAP32[(($arrayidx480)>>2)]; //@line 2063 "translate.cpp"
                                      var $and481=($274) & 1; //@line 2063 "translate.cpp"
                                      var $tobool482=((($and481))|0)!=0; //@line 2063 "translate.cpp"
                                      if ($tobool482) { __label__ = 155;break $land_lhs_true477$$lor_lhs_false483$216; } else { __label__ = 153;break $land_lhs_true477$$lor_lhs_false483$216; } //@line 2063 "translate.cpp"
                                    }
                                  } while(0);
                                  if (__label__ == 153) {
  
                                    var $275=$tr_addr; //@line 2063 "translate.cpp"
                                    var $end_stressed_vowel484=(($275+8316)&4294967295); //@line 2063 "translate.cpp"
                                    var $276=HEAP32[(($end_stressed_vowel484)>>2)]; //@line 2063 "translate.cpp"
                                    var $tobool485=((($276))|0)!=0; //@line 2063 "translate.cpp"
                                    if (!($tobool485)) { __label__ = 156;break $if_then438$$if_else466$200; } //@line 2063 "translate.cpp"
  
                                    var $277=$tr_addr; //@line 2063 "translate.cpp"
                                    var $langopts487=(($277)&4294967295); //@line 2063 "translate.cpp"
                                    var $param488=(($langopts487+24)&4294967295); //@line 2063 "translate.cpp"
                                    var $arrayidx489=(($param488+56)&4294967295); //@line 2063 "translate.cpp"
                                    var $278=HEAP32[(($arrayidx489)>>2)]; //@line 2063 "translate.cpp"
                                    var $and490=($278) & 2; //@line 2063 "translate.cpp"
                                    var $tobool491=((($and490))|0)!=0; //@line 2063 "translate.cpp"
                                    if (!($tobool491)) { __label__ = 156;break $if_then438$$if_else466$200; } //@line 2063 "translate.cpp"
                                  }
  
                                  var $279=HEAP32[((_n_ph_list2)>>2)]; //@line 2068 "translate.cpp"
                                  var $arrayidx493=((_ph_list2+$279*8)&4294967295); //@line 2068 "translate.cpp"
                                  var $synthflags494=(($arrayidx493+4)&4294967295); //@line 2068 "translate.cpp"
                                  var $280=HEAPU16[(($synthflags494)>>1)]; //@line 2068 "translate.cpp"
                                  var $conv495=((($280))&65535); //@line 2068 "translate.cpp"
                                  var $or496=($conv495) | 8; //@line 2068 "translate.cpp"
                                  var $conv497=((($or496)) & 65535); //@line 2068 "translate.cpp"
                                  HEAP16[(($synthflags494)>>1)]=$conv497; //@line 2068 "translate.cpp"
                                  ; //@line 2069 "translate.cpp"
                                }
                              } while(0);
  
                              var $281=$stress; //@line 2073 "translate.cpp"
                              var $conv501=((($281)) & 255); //@line 2073 "translate.cpp"
                              var $282=HEAP32[((_n_ph_list2)>>2)]; //@line 2073 "translate.cpp"
                              var $arrayidx502=((_ph_list2+$282*8)&4294967295); //@line 2073 "translate.cpp"
                              var $stresslevel503=(($arrayidx502+1)&4294967295); //@line 2073 "translate.cpp"
                              HEAP8[($stresslevel503)]=$conv501; //@line 2073 "translate.cpp"
                              var $283=HEAP32[((_n_ph_list2)>>2)]; //@line 2074 "translate.cpp"
                              var $inc504=((($283)+1)&4294967295); //@line 2074 "translate.cpp"
                              HEAP32[((_n_ph_list2)>>2)]=$inc504; //@line 2074 "translate.cpp"
                              $first_phoneme=0; //@line 2075 "translate.cpp"
                              ;
                            }
                          }
                        }
                      }
                    }
                  }
                }
              } while(0);
  
              var $203=$p; //@line 1957 "translate.cpp"
              var $incdec_ptr342=(($203+1)&4294967295); //@line 1957 "translate.cpp"
              $p=$incdec_ptr342; //@line 1957 "translate.cpp"
              var $204=HEAPU8[($203)]; //@line 1957 "translate.cpp"
              $ph_code=$204; //@line 1957 "translate.cpp"
              var $conv343=((($204))&255); //@line 1957 "translate.cpp"
              var $cmp344=((($conv343))|0)!=0; //@line 1957 "translate.cpp"
              var $205=HEAP32[((_n_ph_list2)>>2)]; //@line 1957 "translate.cpp"
              var $cmp346=((($205))|0) < 996; //@line 1957 "translate.cpp"
              var $or_cond6=($cmp344) & ($cmp346); //@line 1957 "translate.cpp"
              if ($or_cond6) { __label__ = 120;continue $while_body348$171; } else { __label__ = 157;break $while_body348$$while_end511$170; } //@line 1957 "translate.cpp"
            }
          }
        } while(0);
  
        var $284=$word_flags; //@line 2079 "translate.cpp"
        var $and512=($284) & 131072; //@line 2079 "translate.cpp"
        var $tobool513=((($and512))|0)!=0; //@line 2079 "translate.cpp"
        if ($tobool513) { __label__ = 158;; } else { __label__ = 159;; } //@line 2079 "translate.cpp"
        if (__label__ == 158) {
  
          var $285=HEAP32[((_n_ph_list2)>>2)]; //@line 2081 "translate.cpp"
          var $inc515=((($285)+1)&4294967295); //@line 2081 "translate.cpp"
          HEAP32[((_n_ph_list2)>>2)]=$inc515; //@line 2081 "translate.cpp"
          var $arrayidx516=((_ph_list2+$285*8)&4294967295); //@line 2081 "translate.cpp"
          __ZL9SetPlist2P13PHONEME_LIST2h($arrayidx516, 27); //@line 2081 "translate.cpp"
          ; //@line 2082 "translate.cpp"
        }
  
        var $286=$word_flags; //@line 2085 "translate.cpp"
        var $and518=($286) & 128; //@line 2085 "translate.cpp"
        var $cmp519=((($and518))|0)==0; //@line 2085 "translate.cpp"
        if ($cmp519) { __label__ = 160;; } else { __label__ = 161;; } //@line 2085 "translate.cpp"
        if (__label__ == 160) {
  
          var $287=$source_ix; //@line 2087 "translate.cpp"
          var $conv521=((($287)) & 65535); //@line 2087 "translate.cpp"
          var $288=$plist2; //@line 2087 "translate.cpp"
          var $sourceix522=(($288+6)&4294967295); //@line 2087 "translate.cpp"
          HEAP16[(($sourceix522)>>1)]=$conv521; //@line 2087 "translate.cpp"
          ; //@line 2088 "translate.cpp"
        }
  
        var $289=$tr_addr; //@line 2090 "translate.cpp"
        var $end_stressed_vowel524=(($289+8316)&4294967295); //@line 2090 "translate.cpp"
        HEAP32[(($end_stressed_vowel524)>>2)]=0; //@line 2090 "translate.cpp"
        var $290=$stress; //@line 2091 "translate.cpp"
        var $cmp525=((($290))|0) >= 4; //@line 2091 "translate.cpp"
        if ($cmp525) { __label__ = 162;; } else { __label__ = 164;; } //@line 2091 "translate.cpp"
        $land_lhs_true526$$if_end537$231: do { 
          if (__label__ == 162) {
  
            var $291=HEAP32[((_n_ph_list2)>>2)]; //@line 2091 "translate.cpp"
            var $sub527=((($291)-1)&4294967295); //@line 2091 "translate.cpp"
            var $arrayidx528=((_ph_list2+$sub527*8)&4294967295); //@line 2091 "translate.cpp"
            var $phcode529=(($arrayidx528)&4294967295); //@line 2091 "translate.cpp"
            var $292=HEAPU8[($phcode529)]; //@line 2091 "translate.cpp"
            var $idxprom530=((($292))&255); //@line 2091 "translate.cpp"
            var $arrayidx531=((_phoneme_tab+$idxprom530*4)&4294967295); //@line 2091 "translate.cpp"
            var $293=HEAP32[(($arrayidx531)>>2)]; //@line 2091 "translate.cpp"
            var $type532=(($293+11)&4294967295); //@line 2091 "translate.cpp"
            var $294=HEAPU8[($type532)]; //@line 2091 "translate.cpp"
            var $conv533=((($294))&255); //@line 2091 "translate.cpp"
            var $cmp534=((($conv533))|0)==2; //@line 2091 "translate.cpp"
            if (!($cmp534)) { __label__ = 164;break $land_lhs_true526$$if_end537$231; } //@line 2091 "translate.cpp"
  
            var $295=$tr_addr; //@line 2093 "translate.cpp"
            var $end_stressed_vowel536=(($295+8316)&4294967295); //@line 2093 "translate.cpp"
            HEAP32[(($end_stressed_vowel536)>>2)]=1; //@line 2093 "translate.cpp"
            ; //@line 2094 "translate.cpp"
          }
        } while(0);
  
        var $296=$switch_phonemes; //@line 2096 "translate.cpp"
        var $cmp538=((($296))|0) >= 0; //@line 2096 "translate.cpp"
        if ($cmp538) { __label__ = 165;; } else { __label__ = 166;; } //@line 2096 "translate.cpp"
        if (__label__ == 165) {
  
          var $arraydecay540=(($old_dictionary_name)&4294967295); //@line 2099 "translate.cpp"
          var $call541=_strcpy(((_dictionary_name)&4294967295), $arraydecay540); //@line 2099 "translate.cpp"
          var $297=HEAP32[((_voice)>>2)]; //@line 2100 "translate.cpp"
          var $phoneme_tab_ix542=(($297+40)&4294967295); //@line 2100 "translate.cpp"
          var $298=HEAP32[(($phoneme_tab_ix542)>>2)]; //@line 2100 "translate.cpp"
          __Z18SelectPhonemeTablei($298); //@line 2100 "translate.cpp"
          var $299=HEAP32[((_n_ph_list2)>>2)]; //@line 2101 "translate.cpp"
          var $arrayidx543=((_ph_list2+$299*8)&4294967295); //@line 2101 "translate.cpp"
          __ZL9SetPlist2P13PHONEME_LIST2h($arrayidx543, 21); //@line 2101 "translate.cpp"
          var $300=HEAP32[((_voice)>>2)]; //@line 2102 "translate.cpp"
          var $phoneme_tab_ix544=(($300+40)&4294967295); //@line 2102 "translate.cpp"
          var $301=HEAP32[(($phoneme_tab_ix544)>>2)]; //@line 2102 "translate.cpp"
          var $conv545=((($301)) & 255); //@line 2102 "translate.cpp"
          var $302=HEAP32[((_n_ph_list2)>>2)]; //@line 2102 "translate.cpp"
          var $inc546=((($302)+1)&4294967295); //@line 2102 "translate.cpp"
          HEAP32[((_n_ph_list2)>>2)]=$inc546; //@line 2102 "translate.cpp"
          var $arrayidx547=((_ph_list2+$302*8)&4294967295); //@line 2102 "translate.cpp"
          var $tone_ph548=(($arrayidx547+3)&4294967295); //@line 2102 "translate.cpp"
          HEAP8[($tone_ph548)]=$conv545; //@line 2102 "translate.cpp"
          ; //@line 2103 "translate.cpp"
        }
  
        var $303=$pitch_raised; //@line 2106 "translate.cpp"
        var $cmp550=((($303))|0) > 0; //@line 2106 "translate.cpp"
        if ($cmp550) { __label__ = 167;; } else { __label__ = 168;; } //@line 2106 "translate.cpp"
        if (__label__ == 167) {
  
          var $304=$pitch_raised; //@line 2108 "translate.cpp"
          var $shl552=((($304)*256)&4294967295); //@line 2108 "translate.cpp"
          var $add553=((($shl552)+225)&4294967295); //@line 2108 "translate.cpp"
          var $305=HEAP32[((__ZL11embedded_ix)>>2)]; //@line 2108 "translate.cpp"
          var $inc554=((($305)+1)&4294967295); //@line 2108 "translate.cpp"
          HEAP32[((__ZL11embedded_ix)>>2)]=$inc554; //@line 2108 "translate.cpp"
          var $arrayidx555=((_embedded_list+$305*4)&4294967295); //@line 2108 "translate.cpp"
          HEAP32[(($arrayidx555)>>2)]=$add553; //@line 2108 "translate.cpp"
          var $306=HEAP32[((_n_ph_list2)>>2)]; //@line 2109 "translate.cpp"
          var $arrayidx556=((_ph_list2+$306*8)&4294967295); //@line 2109 "translate.cpp"
          __ZL9SetPlist2P13PHONEME_LIST2h($arrayidx556, 10); //@line 2109 "translate.cpp"
          var $307=HEAP32[((_n_ph_list2)>>2)]; //@line 2110 "translate.cpp"
          var $inc557=((($307)+1)&4294967295); //@line 2110 "translate.cpp"
          HEAP32[((_n_ph_list2)>>2)]=$inc557; //@line 2110 "translate.cpp"
          var $arrayidx558=((_ph_list2+$307*8)&4294967295); //@line 2110 "translate.cpp"
          var $synthflags559=(($arrayidx558+4)&4294967295); //@line 2110 "translate.cpp"
          HEAP16[(($synthflags559)>>1)]=2; //@line 2110 "translate.cpp"
          ; //@line 2111 "translate.cpp"
        }
  
        var $308=$flags; //@line 2113 "translate.cpp"
        var $and561=($308) & 4096; //@line 2113 "translate.cpp"
        var $tobool562=((($and561))|0)!=0; //@line 2113 "translate.cpp"
        if ($tobool562) { __label__ = 169;; } else { __label__ = 170;; } //@line 2113 "translate.cpp"
        if (__label__ == 169) {
  
          var $309=$max_stress_ix; //@line 2116 "translate.cpp"
          var $arrayidx564=((_ph_list2+$309*8)&4294967295); //@line 2116 "translate.cpp"
          var $synthflags565=(($arrayidx564+4)&4294967295); //@line 2116 "translate.cpp"
          var $310=HEAPU16[(($synthflags565)>>1)]; //@line 2116 "translate.cpp"
          var $conv566=((($310))&65535); //@line 2116 "translate.cpp"
          var $or567=($conv566) | 64; //@line 2116 "translate.cpp"
          var $conv568=((($or567)) & 65535); //@line 2116 "translate.cpp"
          HEAP16[(($synthflags565)>>1)]=$conv568; //@line 2116 "translate.cpp"
          ; //@line 2117 "translate.cpp"
        }
  
        var $311=$flags; //@line 2119 "translate.cpp"
        var $312=$tr_addr; //@line 2119 "translate.cpp"
        var $prev_dict_flags570=(($312+8320)&4294967295); //@line 2119 "translate.cpp"
        HEAP32[(($prev_dict_flags570)>>2)]=$311; //@line 2119 "translate.cpp"
        var $313=$flags; //@line 2120 "translate.cpp"
        $retval=$313; //@line 2120 "translate.cpp"
        ; //@line 2120 "translate.cpp"
      }
    } while(0);
  
    var $314=$retval; //@line 2121 "translate.cpp"
    STACKTOP = __stackBase__;
    return $314; //@line 2121 "translate.cpp"
    return null;
  }
  

  function __Z8InitTexti($control) {
    ;
    var __label__;
  
    var $control_addr;
    $control_addr=$control;
    HEAP32[((_count_sentences)>>2)]=0; //@line 3278 "translate.cpp"
    HEAP32[((_count_words)>>2)]=0; //@line 3279 "translate.cpp"
    HEAP32[((_end_character_position)>>2)]=0; //@line 3280 "translate.cpp"
    HEAP32[((_skip_sentences)>>2)]=0; //@line 3281 "translate.cpp"
    HEAP8[(((_skip_marker)&4294967295))]=0; //@line 3282 "translate.cpp"
    HEAP32[((_skip_words)>>2)]=0; //@line 3283 "translate.cpp"
    HEAP32[((_skip_characters)>>2)]=0; //@line 3284 "translate.cpp"
    HEAP32[((_skipping_text)>>2)]=0; //@line 3285 "translate.cpp"
    HEAP32[((_new_sentence)>>2)]=1; //@line 3286 "translate.cpp"
    HEAP32[((_option_sayas)>>2)]=0; //@line 3290 "translate.cpp"
    HEAP32[((__ZL13option_sayas2)>>2)]=0; //@line 3291 "translate.cpp"
    HEAP32[((__ZL15option_emphasis)>>2)]=0; //@line 3292 "translate.cpp"
    HEAP8[(__ZL13word_emphasis_b)]=0;
    HEAP8[(__ZL13embedded_flag_b)]=0;
    __Z9InitText2v(); //@line 3296 "translate.cpp"
    var $0=$control_addr; //@line 3298 "translate.cpp"
    var $and=($0) & 8192; //@line 3298 "translate.cpp"
    var $cmp=((($and))|0)==0; //@line 3298 "translate.cpp"
    if ($cmp) { __label__ = 1;; } else { __label__ = 2;; } //@line 3298 "translate.cpp"
    if (__label__ == 1) {
  
      __Z12InitNamedatav(); //@line 3300 "translate.cpp"
      ; //@line 3301 "translate.cpp"
    }
  
    ;
    return; //@line 3302 "translate.cpp"
    return;
  }
  

  function __ZL14CountSyllablesPh($phonemes) {
    ;
    var __label__;
  
    var $phonemes_addr;
    var $count;
    var $phon;
    $phonemes_addr=$phonemes;
    $count=0; //@line 1555 "translate.cpp"
    var $0=$phonemes_addr; //@line 1557 "translate.cpp"
    var $incdec_ptr1=(($0+1)&4294967295); //@line 1557 "translate.cpp"
    $phonemes_addr=$incdec_ptr1; //@line 1557 "translate.cpp"
    var $1=HEAPU8[($0)]; //@line 1557 "translate.cpp"
    var $conv2=((($1))&255); //@line 1557 "translate.cpp"
    $phon=$conv2; //@line 1557 "translate.cpp"
    var $cmp3=((($conv2))|0)!=0; //@line 1557 "translate.cpp"
    if ($cmp3) { __label__ = 1;; } else { __label__ = 4;; } //@line 1557 "translate.cpp"
    $while_body$$while_end$2: do { 
      if (__label__ == 1) {
        while(1) { 
  
          var $2=$phon; //@line 1559 "translate.cpp"
          var $arrayidx=((_phoneme_tab+$2*4)&4294967295); //@line 1559 "translate.cpp"
          var $3=HEAP32[(($arrayidx)>>2)]; //@line 1559 "translate.cpp"
          var $type=(($3+11)&4294967295); //@line 1559 "translate.cpp"
          var $4=HEAPU8[($type)]; //@line 1559 "translate.cpp"
          var $conv1=((($4))&255); //@line 1559 "translate.cpp"
          var $cmp2=((($conv1))|0)==2; //@line 1559 "translate.cpp"
          if ($cmp2) { __label__ = 2;; } else { __label__ = 3;; } //@line 1559 "translate.cpp"
          if (__label__ == 2) {
  
            var $5=$count; //@line 1560 "translate.cpp"
            var $inc=((($5)+1)&4294967295); //@line 1560 "translate.cpp"
            $count=$inc; //@line 1560 "translate.cpp"
            ; //@line 1560 "translate.cpp"
          }
  
          var $6=$phonemes_addr; //@line 1557 "translate.cpp"
          var $incdec_ptr=(($6+1)&4294967295); //@line 1557 "translate.cpp"
          $phonemes_addr=$incdec_ptr; //@line 1557 "translate.cpp"
          var $7=HEAPU8[($6)]; //@line 1557 "translate.cpp"
          var $conv=((($7))&255); //@line 1557 "translate.cpp"
          $phon=$conv; //@line 1557 "translate.cpp"
          var $cmp=((($conv))|0)!=0; //@line 1557 "translate.cpp"
          if ($cmp) { __label__ = 1;continue ; } else { __label__ = 4;break $while_body$$while_end$2; } //@line 1557 "translate.cpp"
        }
      }
    } while(0);
  
    var $8=$count; //@line 1562 "translate.cpp"
    ;
    return $8; //@line 1562 "translate.cpp"
    return null;
  }
  

  function __ZL9SetPlist2P13PHONEME_LIST2h($p, $phcode) {
    ;
    var __label__;
  
    var $p_addr;
    var $phcode_addr;
    $p_addr=$p;
    $phcode_addr=$phcode;
    var $0=$phcode_addr; //@line 1545 "translate.cpp"
    var $1=$p_addr; //@line 1545 "translate.cpp"
    var $phcode1=(($1)&4294967295); //@line 1545 "translate.cpp"
    HEAP8[($phcode1)]=$0; //@line 1545 "translate.cpp"
    var $2=$p_addr; //@line 1546 "translate.cpp"
    var $stresslevel=(($2+1)&4294967295); //@line 1546 "translate.cpp"
    HEAP8[($stresslevel)]=0; //@line 1546 "translate.cpp"
    var $3=$p_addr; //@line 1547 "translate.cpp"
    var $tone_ph=(($3+3)&4294967295); //@line 1547 "translate.cpp"
    HEAP8[($tone_ph)]=0; //@line 1547 "translate.cpp"
    var $_b=HEAPU8[(__ZL13embedded_flag_b)];
    var $4=($_b) ? 2 : 0;
    var $conv=((($4)) & 65535); //@line 1548 "translate.cpp"
    var $5=$p_addr; //@line 1548 "translate.cpp"
    var $synthflags=(($5+4)&4294967295); //@line 1548 "translate.cpp"
    HEAP16[(($synthflags)>>1)]=$conv; //@line 1548 "translate.cpp"
    var $6=$p_addr; //@line 1549 "translate.cpp"
    var $sourceix=(($6+6)&4294967295); //@line 1549 "translate.cpp"
    HEAP16[(($sourceix)>>1)]=0; //@line 1549 "translate.cpp"
    HEAP8[(__ZL13embedded_flag_b)]=0;
    ;
    return; //@line 1551 "translate.cpp"
    return;
  }
  

  function __ZL14SubstituteCharP10TranslatorjjPi($tr, $c, $next_in, $insert) {
    ;
    var __label__;
    var __lastLabel__ = null;
  
    var $retval;
    var $tr_addr;
    var $c_addr;
    var $next_in_addr;
    var $insert_addr;
    var $ix;
    var $word;
    var $new_c;
    var $c2;
    var $c_lower;
    var $upper_case;
    var $replace_chars;
    $tr_addr=$tr;
    $c_addr=$c;
    $next_in_addr=$next_in;
    $insert_addr=$insert;
    $upper_case=0; //@line 2195 "translate.cpp"
    var $_b=HEAPU8[(__ZZL14SubstituteCharP10TranslatorjjPiE11ignore_next_b)];
    var $0=((($_b))&1);
    var $tobool=((($0))|0)!=0; //@line 2199 "translate.cpp"
    if ($tobool) { __label__ = 1;; } else { __label__ = 2;; } //@line 2199 "translate.cpp"
    if (__label__ == 1) {
  
      HEAP8[(__ZZL14SubstituteCharP10TranslatorjjPiE11ignore_next_b)]=0;
      $retval=8; //@line 2202 "translate.cpp"
      ; //@line 2202 "translate.cpp"
    }
    else if (__label__ == 2) {
  
      var $1=$c_addr; //@line 2204 "translate.cpp"
      var $cmp=((($1))|0)==0; //@line 2204 "translate.cpp"
      if ($cmp) { __label__ = 3;; } else { __label__ = 4;; } //@line 2204 "translate.cpp"
      if (__label__ == 3) {
  
        $retval=0; //@line 2204 "translate.cpp"
        ; //@line 2204 "translate.cpp"
      }
      else if (__label__ == 4) {
  
        var $2=$tr_addr; //@line 2206 "translate.cpp"
        var $langopts=(($2)&4294967295); //@line 2206 "translate.cpp"
        var $replace_chars3=(($langopts+264)&4294967295); //@line 2206 "translate.cpp"
        var $3=HEAP32[(($replace_chars3)>>2)]; //@line 2206 "translate.cpp"
        $replace_chars=$3; //@line 2206 "translate.cpp"
        var $cmp4=((($3))|0)==0; //@line 2206 "translate.cpp"
        var $4=$c_addr; //@line 2207 "translate.cpp"
        if ($cmp4) { __label__ = 5;; } else { __label__ = 6;; } //@line 2206 "translate.cpp"
        if (__label__ == 5) {
  
          $retval=$4; //@line 2207 "translate.cpp"
          ; //@line 2207 "translate.cpp"
        }
        else if (__label__ == 6) {
  
          $c_lower=$4; //@line 2211 "translate.cpp"
          var $call=_iswupper($4); //@line 2211 "translate.cpp"
          var $tobool7=((($call))|0)!=0; //@line 2211 "translate.cpp"
          if ($tobool7) { __label__ = 7;; } else { __label__ = 8;; } //@line 2211 "translate.cpp"
          if (__label__ == 7) {
  
            var $5=$c_addr; //@line 2213 "translate.cpp"
            var $call9=_towlower($5); //@line 2213 "translate.cpp"
            $c_lower=$call9; //@line 2213 "translate.cpp"
            $upper_case=1; //@line 2214 "translate.cpp"
            ; //@line 2215 "translate.cpp"
          }
  
          $new_c=0; //@line 2217 "translate.cpp"
          $ix=0; //@line 2218 "translate.cpp"
          ; //@line 2218 "translate.cpp"
          $for_cond$14: while(1) { 
  
            var $6=$ix; //@line 2218 "translate.cpp"
            var $7=$replace_chars; //@line 2218 "translate.cpp"
            var $arrayidx=(($7+4*$6)&4294967295); //@line 2218 "translate.cpp"
            var $8=HEAP32[(($arrayidx)>>2)]; //@line 2218 "translate.cpp"
            $word=$8; //@line 2218 "translate.cpp"
            var $cmp11=((($8))|0)!=0; //@line 2218 "translate.cpp"
            if (!($cmp11)) { __label__ = 16;break $for_cond$14; } //@line 2218 "translate.cpp"
  
            var $9=$c_lower; //@line 2220 "translate.cpp"
            var $10=$word; //@line 2220 "translate.cpp"
            var $and=($10) & 65535; //@line 2220 "translate.cpp"
            var $cmp12=((($9))|0)==((($and))|0); //@line 2220 "translate.cpp"
            if ($cmp12) { __label__ = 11;; } else { __label__ = 15;; } //@line 2220 "translate.cpp"
            if (__label__ == 11) {
  
              var $11=$word; //@line 2222 "translate.cpp"
              var $shr=($11) >>> 16; //@line 2222 "translate.cpp"
              var $cmp14=((($shr))|0)==0; //@line 2222 "translate.cpp"
              if ($cmp14) { __label__ = 12;break $for_cond$14; } //@line 2222 "translate.cpp"
  
              var $15=$word; //@line 2227 "translate.cpp"
              var $shr18=($15) >>> 16; //@line 2227 "translate.cpp"
              var $16=$next_in_addr; //@line 2227 "translate.cpp"
              var $call19=_towlower($16); //@line 2227 "translate.cpp"
              var $cmp20=((($shr18))|0)==((($call19))|0); //@line 2227 "translate.cpp"
              if ($cmp20) { __label__ = 14;break $for_cond$14; } //@line 2227 "translate.cpp"
            }
  
            var $20=$ix; //@line 2218 "translate.cpp"
            var $add26=((($20)+2)&4294967295); //@line 2218 "translate.cpp"
            $ix=$add26; //@line 2218 "translate.cpp"
            __label__ = 9;continue $for_cond$14; //@line 2218 "translate.cpp"
          }
          if (__label__ == 16) {
  
            var $_pr=$new_c; //@line 2236 "translate.cpp"
            __lastLabel__ = 16; ;
          }
          else if (__label__ == 12) {
  
            var $12=$ix; //@line 2224 "translate.cpp"
            var $add=((($12)+1)&4294967295); //@line 2224 "translate.cpp"
            var $13=$replace_chars; //@line 2224 "translate.cpp"
            var $arrayidx16=(($13+4*$add)&4294967295); //@line 2224 "translate.cpp"
            var $14=HEAPU32[(($arrayidx16)>>2)]; //@line 2224 "translate.cpp"
            $new_c=$14; //@line 2224 "translate.cpp"
            __lastLabel__ = 12; ; //@line 2225 "translate.cpp"
          }
          else if (__label__ == 14) {
  
            var $17=$ix; //@line 2229 "translate.cpp"
            var $add22=((($17)+1)&4294967295); //@line 2229 "translate.cpp"
            var $18=$replace_chars; //@line 2229 "translate.cpp"
            var $arrayidx23=(($18+4*$add22)&4294967295); //@line 2229 "translate.cpp"
            var $19=HEAPU32[(($arrayidx23)>>2)]; //@line 2229 "translate.cpp"
            $new_c=$19; //@line 2229 "translate.cpp"
            HEAP8[(__ZZL14SubstituteCharP10TranslatorjjPiE11ignore_next_b)]=1;
            __lastLabel__ = 14; ; //@line 2231 "translate.cpp"
          }
  
          var $21=__lastLabel__ == 16 ? $_pr : (__lastLabel__ == 14 ? $19 : ($14)); //@line 2236 "translate.cpp"
          var $cmp27=((($21))|0)==0; //@line 2236 "translate.cpp"
          if ($cmp27) { __label__ = 18;; } else { __label__ = 19;; } //@line 2236 "translate.cpp"
          if (__label__ == 18) {
  
            var $22=$c_addr; //@line 2237 "translate.cpp"
            $retval=$22; //@line 2237 "translate.cpp"
            ; //@line 2237 "translate.cpp"
          }
          else if (__label__ == 19) {
  
            var $23=$new_c; //@line 2239 "translate.cpp"
            var $and30=($23) & -2097152; //@line 2239 "translate.cpp"
            var $tobool31=((($and30))|0)!=0; //@line 2239 "translate.cpp"
            if ($tobool31) { __label__ = 20;; } else { __label__ = 24;; } //@line 2239 "translate.cpp"
            if (__label__ == 20) {
  
              var $24=$new_c; //@line 2243 "translate.cpp"
              var $shr33=($24) >>> 16; //@line 2243 "translate.cpp"
              $c2=$shr33; //@line 2243 "translate.cpp"
              var $25=$upper_case; //@line 2244 "translate.cpp"
              var $tobool34=((($25))|0)!=0; //@line 2244 "translate.cpp"
              if ($tobool34) { __label__ = 21;; } else { __label__ = 23;; } //@line 2244 "translate.cpp"
              $land_lhs_true$$if_end39$31: do { 
                if (__label__ == 21) {
  
                  var $26=$next_in_addr; //@line 2244 "translate.cpp"
                  var $call35=_iswupper($26); //@line 2244 "translate.cpp"
                  var $tobool36=((($call35))|0)!=0; //@line 2244 "translate.cpp"
                  if (!($tobool36)) { __label__ = 23;break $land_lhs_true$$if_end39$31; } //@line 2244 "translate.cpp"
  
                  var $27=$c2; //@line 2245 "translate.cpp"
                  var $call38=_towupper($27); //@line 2245 "translate.cpp"
                  $c2=$call38; //@line 2245 "translate.cpp"
                  ; //@line 2245 "translate.cpp"
                }
              } while(0);
  
              var $28=$c2; //@line 2246 "translate.cpp"
              var $29=$insert_addr; //@line 2246 "translate.cpp"
              HEAP32[(($29)>>2)]=$28; //@line 2246 "translate.cpp"
              var $30=$new_c; //@line 2247 "translate.cpp"
              var $and40=($30) & 65535; //@line 2247 "translate.cpp"
              $new_c=$and40; //@line 2247 "translate.cpp"
              ; //@line 2248 "translate.cpp"
            }
  
            var $31=$upper_case; //@line 2250 "translate.cpp"
            var $tobool42=((($31))|0)!=0; //@line 2250 "translate.cpp"
            if ($tobool42) { __label__ = 25;; } else { __label__ = 26;; } //@line 2250 "translate.cpp"
            if (__label__ == 25) {
  
              var $32=$new_c; //@line 2251 "translate.cpp"
              var $call44=_towupper($32); //@line 2251 "translate.cpp"
              $new_c=$call44; //@line 2251 "translate.cpp"
              ; //@line 2251 "translate.cpp"
            }
  
            var $33=$new_c; //@line 2252 "translate.cpp"
            $retval=$33; //@line 2252 "translate.cpp"
            ; //@line 2252 "translate.cpp"
          }
        }
      }
    }
  
    var $34=$retval; //@line 2254 "translate.cpp"
    ;
    return $34; //@line 2254 "translate.cpp"
    return null;
  }
  

  function _init_MBR($voice_path) {
    var __stackBase__  = STACKTOP; STACKTOP += 48; _memset(__stackBase__, 0, 48);
    var __label__;
  
    var $retval;
    var $voice_path_addr;
    var $error;
    var $result;
    var $wavhdr=__stackBase__;
    $voice_path_addr=$voice_path;
    var $0=$voice_path_addr; //@line 484 "mbrowrap.cpp"
    var $call=__ZL12start_mbrolaPKc($0); //@line 484 "mbrowrap.cpp"
    $error=$call; //@line 484 "mbrowrap.cpp"
    var $1=$error; //@line 485 "mbrowrap.cpp"
    var $tobool=((($1))|0)!=0; //@line 485 "mbrowrap.cpp"
    if ($tobool) { __label__ = 1;; } else { __label__ = 2;; } //@line 485 "mbrowrap.cpp"
    $if_then$$if_end$2: do { 
      if (__label__ == 1) {
  
        $retval=-1; //@line 486 "mbrowrap.cpp"
        ; //@line 486 "mbrowrap.cpp"
      }
      else if (__label__ == 2) {
  
        var $call1=__ZL14send_to_mbrolaPKc(((__str580)&4294967295)); //@line 488 "mbrowrap.cpp"
        $result=$call1; //@line 488 "mbrowrap.cpp"
        var $2=$result; //@line 489 "mbrowrap.cpp"
        var $cmp=((($2))|0)!=2; //@line 489 "mbrowrap.cpp"
        if ($cmp) { __label__ = 3;; } else { __label__ = 4;; } //@line 489 "mbrowrap.cpp"
        if (__label__ == 3) {
  
          __ZL11stop_mbrolav(); //@line 490 "mbrowrap.cpp"
          $retval=-1; //@line 491 "mbrowrap.cpp"
          ; //@line 491 "mbrowrap.cpp"
        }
        else if (__label__ == 4) {
  
          var $arraydecay=(($wavhdr)&4294967295); //@line 495 "mbrowrap.cpp"
          var $call4=__ZL19receive_from_mbrolaPvj($arraydecay, 45); //@line 495 "mbrowrap.cpp"
          $result=$call4; //@line 495 "mbrowrap.cpp"
          var $3=$result; //@line 496 "mbrowrap.cpp"
          var $cmp5=((($3))|0)!=44; //@line 496 "mbrowrap.cpp"
          if ($cmp5) { __label__ = 5;; } else { __label__ = 8;; } //@line 496 "mbrowrap.cpp"
          if (__label__ == 5) {
  
            var $4=$result; //@line 497 "mbrowrap.cpp"
            var $cmp7=((($4))|0) >= 0; //@line 497 "mbrowrap.cpp"
            if ($cmp7) { __label__ = 6;; } else { __label__ = 7;; } //@line 497 "mbrowrap.cpp"
            if (__label__ == 6) {
  
              __ZL3errPKcz(((__str1581)&4294967295), allocate(1, "i32", ALLOC_STACK)); //@line 498 "mbrowrap.cpp"
              ; //@line 498 "mbrowrap.cpp"
            }
  
            __ZL11stop_mbrolav(); //@line 499 "mbrowrap.cpp"
            $retval=-1; //@line 500 "mbrowrap.cpp"
            ; //@line 500 "mbrowrap.cpp"
          }
          else if (__label__ == 8) {
  
            var $arraydecay11=(($wavhdr)&4294967295); //@line 504 "mbrowrap.cpp"
            var $call12=_memcmp($arraydecay11, ((__str2582)&4294967295), 4); //@line 504 "mbrowrap.cpp"
            var $cmp13=((($call12))|0)!=0; //@line 504 "mbrowrap.cpp"
            if ($cmp13) { __label__ = 10;; } else { __label__ = 9;; } //@line 504 "mbrowrap.cpp"
            $if_then17$$lor_lhs_false$14: do { 
              if (__label__ == 9) {
  
                var $arraydecay14=(($wavhdr)&4294967295); //@line 505 "mbrowrap.cpp"
                var $add_ptr=(($arraydecay14+8)&4294967295); //@line 505 "mbrowrap.cpp"
                var $call15=_memcmp($add_ptr, ((__str3583)&4294967295), 8); //@line 505 "mbrowrap.cpp"
                var $cmp16=((($call15))|0)!=0; //@line 505 "mbrowrap.cpp"
                if ($cmp16) { __label__ = 10;break $if_then17$$lor_lhs_false$14; } //@line 505 "mbrowrap.cpp"
  
                var $arrayidx=(($wavhdr+24)&4294967295); //@line 510 "mbrowrap.cpp"
                var $5=HEAPU8[($arrayidx)]; //@line 510 "mbrowrap.cpp"
                var $conv=((($5))&255); //@line 510 "mbrowrap.cpp"
                var $arrayidx19=(($wavhdr+25)&4294967295); //@line 510 "mbrowrap.cpp"
                var $6=HEAPU8[($arrayidx19)]; //@line 510 "mbrowrap.cpp"
                var $conv20=((($6))&255); //@line 510 "mbrowrap.cpp"
                var $shl=((($conv20)*256)&4294967295); //@line 510 "mbrowrap.cpp"
                var $arrayidx21=(($wavhdr+26)&4294967295); //@line 510 "mbrowrap.cpp"
                var $7=HEAPU8[($arrayidx21)]; //@line 510 "mbrowrap.cpp"
                var $conv22=((($7))&255); //@line 510 "mbrowrap.cpp"
                var $shl23=((($conv22)*65536)&4294967295); //@line 510 "mbrowrap.cpp"
                var $arrayidx25=(($wavhdr+27)&4294967295); //@line 510 "mbrowrap.cpp"
                var $8=HEAPU8[($arrayidx25)]; //@line 510 "mbrowrap.cpp"
                var $conv26=((($8))&255); //@line 510 "mbrowrap.cpp"
                var $shl27=((($conv26)*16777216)&4294967295); //@line 510 "mbrowrap.cpp"
                var $add=((($shl)+($conv))&4294967295); //@line 510 "mbrowrap.cpp"
                var $add24=((($add)+($shl23))&4294967295); //@line 510 "mbrowrap.cpp"
                var $add28=((($add24)+($shl27))&4294967295); //@line 510 "mbrowrap.cpp"
                HEAP32[((__ZL14mbr_samplerate)>>2)]=$add28; //@line 510 "mbrowrap.cpp"
                var $9=HEAP32[((__ZL14mbr_voice_path)>>2)]; //@line 515 "mbrowrap.cpp"
                var $10=$voice_path_addr; //@line 515 "mbrowrap.cpp"
                var $cmp29=((($9))|0)!=((($10))|0); //@line 515 "mbrowrap.cpp"
                if ($cmp29) { __label__ = 12;; } else { __label__ = 13;; } //@line 515 "mbrowrap.cpp"
                if (__label__ == 12) {
  
                  var $11=HEAP32[((__ZL14mbr_voice_path)>>2)]; //@line 516 "mbrowrap.cpp"
                  ; //@line 516 "mbrowrap.cpp"
                  var $12=$voice_path_addr; //@line 517 "mbrowrap.cpp"
                  var $call31=_strdup($12); //@line 517 "mbrowrap.cpp"
                  HEAP32[((__ZL14mbr_voice_path)>>2)]=$call31; //@line 517 "mbrowrap.cpp"
                  ; //@line 518 "mbrowrap.cpp"
                }
  
                $retval=0; //@line 520 "mbrowrap.cpp"
                __label__ = 14;break $if_then$$if_end$2; //@line 520 "mbrowrap.cpp"
              }
            } while(0);
  
            __ZL3errPKcz(((__str4584)&4294967295), allocate(1, "i32", ALLOC_STACK)); //@line 506 "mbrowrap.cpp"
            __ZL11stop_mbrolav(); //@line 507 "mbrowrap.cpp"
            $retval=-1; //@line 508 "mbrowrap.cpp"
            ; //@line 508 "mbrowrap.cpp"
          }
        }
      }
    } while(0);
  
    var $13=$retval; //@line 521 "mbrowrap.cpp"
    STACKTOP = __stackBase__;
    return $13; //@line 521 "mbrowrap.cpp"
    return null;
  }
  

  function __ZL12start_mbrolaPKc($voice_path) {
    var __stackBase__  = STACKTOP; STACKTOP += 44; _memset(__stackBase__, 0, 44);
    var __label__;
  
    var $retval;
    var $voice_path_addr;
    var $error;
    var $p_stdin=__stackBase__;
    var $p_stdout=__stackBase__+8;
    var $p_stderr=__stackBase__+16;
    var $charbuf=__stackBase__+24;
    var $i;
    $voice_path_addr=$voice_path;
    var $0=HEAP32[((__ZL9mbr_state)>>2)]; //@line 128 "mbrowrap.cpp"
    var $cmp=((($0))|0)!=0; //@line 128 "mbrowrap.cpp"
    if ($cmp) { __label__ = 1;; } else { __label__ = 2;; } //@line 128 "mbrowrap.cpp"
    $if_then$$if_end$2: do { 
      if (__label__ == 1) {
  
        __ZL3errPKcz(((__str22602)&4294967295), allocate(1, "i32", ALLOC_STACK)); //@line 129 "mbrowrap.cpp"
        $retval=-1; //@line 130 "mbrowrap.cpp"
        ; //@line 130 "mbrowrap.cpp"
      }
      else if (__label__ == 2) {
  
        var $arraydecay=(($p_stdin)&4294967295); //@line 133 "mbrowrap.cpp"
        var $arraydecay1=(($p_stdout)&4294967295); //@line 133 "mbrowrap.cpp"
        var $arraydecay2=(($p_stderr)&4294967295); //@line 133 "mbrowrap.cpp"
        var $call=__ZL12create_pipesPiS_S_($arraydecay, $arraydecay1, $arraydecay2); //@line 133 "mbrowrap.cpp"
        $error=$call; //@line 133 "mbrowrap.cpp"
        var $1=$error; //@line 134 "mbrowrap.cpp"
        var $tobool=((($1))|0)!=0; //@line 134 "mbrowrap.cpp"
        if ($tobool) { __label__ = 3;; } else { __label__ = 4;; } //@line 134 "mbrowrap.cpp"
        if (__label__ == 3) {
  
          $retval=-1; //@line 135 "mbrowrap.cpp"
          ; //@line 135 "mbrowrap.cpp"
        }
        else if (__label__ == 4) {
  
          var $call5=_fork(); //@line 137 "mbrowrap.cpp"
          HEAP32[((__ZL7mbr_pid)>>2)]=$call5; //@line 137 "mbrowrap.cpp"
          var $2=HEAP32[((__ZL7mbr_pid)>>2)]; //@line 139 "mbrowrap.cpp"
          var $cmp6=((($2))|0)==-1; //@line 139 "mbrowrap.cpp"
          if ($cmp6) { __label__ = 5;; } else { __label__ = 6;; } //@line 139 "mbrowrap.cpp"
          if (__label__ == 5) {
  
            var $call8=___errno(); //@line 140 "mbrowrap.cpp"
            var $3=HEAP32[(($call8)>>2)]; //@line 140 "mbrowrap.cpp"
            $error=$3; //@line 140 "mbrowrap.cpp"
            var $arraydecay9=(($p_stdin)&4294967295); //@line 141 "mbrowrap.cpp"
            var $arraydecay10=(($p_stdout)&4294967295); //@line 141 "mbrowrap.cpp"
            var $arraydecay11=(($p_stderr)&4294967295); //@line 141 "mbrowrap.cpp"
            __ZL11close_pipesPiS_S_($arraydecay9, $arraydecay10, $arraydecay11); //@line 141 "mbrowrap.cpp"
            var $4=$error; //@line 142 "mbrowrap.cpp"
            var $call12=_strerror($4); //@line 142 "mbrowrap.cpp"
            __ZL3errPKcz(((__str23603)&4294967295), allocate([$call12,0,0,0], ["i8*",0,0,0], ALLOC_STACK)); //@line 142 "mbrowrap.cpp"
            $retval=-1; //@line 143 "mbrowrap.cpp"
            ; //@line 143 "mbrowrap.cpp"
          }
          else if (__label__ == 6) {
  
            var $5=HEAP32[((__ZL7mbr_pid)>>2)]; //@line 146 "mbrowrap.cpp"
            var $cmp14=((($5))|0)==0; //@line 146 "mbrowrap.cpp"
            if ($cmp14) { __label__ = 7;; } else { __label__ = 14;; } //@line 146 "mbrowrap.cpp"
            if (__label__ == 7) {
  
              var $arrayidx=(($p_stdin)&4294967295); //@line 149 "mbrowrap.cpp"
              var $6=HEAP32[(($arrayidx)>>2)]; //@line 149 "mbrowrap.cpp"
              var $call16=_dup2($6, 0); //@line 149 "mbrowrap.cpp"
              var $cmp17=((($call16))|0)==-1; //@line 149 "mbrowrap.cpp"
              if ($cmp17) { __label__ = 10;; } else { __label__ = 8;; } //@line 149 "mbrowrap.cpp"
              $if_then25$$lor_lhs_false$13: do { 
                if (__label__ == 8) {
  
                  var $arrayidx18=(($p_stdout+4)&4294967295); //@line 150 "mbrowrap.cpp"
                  var $7=HEAP32[(($arrayidx18)>>2)]; //@line 150 "mbrowrap.cpp"
                  var $call19=_dup2($7, 1); //@line 150 "mbrowrap.cpp"
                  var $cmp20=((($call19))|0)==-1; //@line 150 "mbrowrap.cpp"
                  if ($cmp20) { __label__ = 10;break $if_then25$$lor_lhs_false$13; } //@line 150 "mbrowrap.cpp"
  
                  var $arrayidx22=(($p_stderr+4)&4294967295); //@line 151 "mbrowrap.cpp"
                  var $8=HEAP32[(($arrayidx22)>>2)]; //@line 151 "mbrowrap.cpp"
                  var $call23=_dup2($8, 2); //@line 151 "mbrowrap.cpp"
                  var $cmp24=((($call23))|0)==-1; //@line 151 "mbrowrap.cpp"
                  if ($cmp24) { __label__ = 10;break $if_then25$$lor_lhs_false$13; } else { __label__ = 11;break $if_then25$$lor_lhs_false$13; } //@line 151 "mbrowrap.cpp"
                }
              } while(0);
              if (__label__ == 10) {
  
                var $call26=___errno(); //@line 153 "mbrowrap.cpp"
                var $9=HEAP32[(($call26)>>2)]; //@line 153 "mbrowrap.cpp"
                var $call27=_strerror($9); //@line 153 "mbrowrap.cpp"
                var $call28=_snprintf(((__ZL12mbr_errorbuf)&4294967295), 160, ((__str24604)&4294967295), allocate([$call27,0,0,0], ["i8*",0,0,0], ALLOC_STACK)); //@line 153 "mbrowrap.cpp"
                var $arrayidx29=(($p_stderr+4)&4294967295); //@line 154 "mbrowrap.cpp"
                var $10=HEAP32[(($arrayidx29)>>2)]; //@line 154 "mbrowrap.cpp"
                var $call30=_strlen(((__ZL12mbr_errorbuf)&4294967295)); //@line 154 "mbrowrap.cpp"
                var $call31=_write($10, ((__ZL12mbr_errorbuf)&4294967295), $call30); //@line 154 "mbrowrap.cpp"
                __exit(1); //@line 155 "mbrowrap.cpp"
                ; //@line 156 "mbrowrap.cpp"
              }
  
              var $arrayidx33=(($p_stderr+4)&4294967295); //@line 158 "mbrowrap.cpp"
              var $11=HEAP32[(($arrayidx33)>>2)]; //@line 158 "mbrowrap.cpp"
              $i=$11; //@line 158 "mbrowrap.cpp"
              var $cmp341=((($11))|0) > 2; //@line 158 "mbrowrap.cpp"
              if ($cmp341) { __label__ = 12;; } else { __label__ = 13;; } //@line 158 "mbrowrap.cpp"
              $for_body$$for_end$19: do { 
                if (__label__ == 12) {
                  while(1) { 
  
                    var $12=$i; //@line 159 "mbrowrap.cpp"
                    var $call35=_close($12); //@line 159 "mbrowrap.cpp"
                    var $13=$i; //@line 158 "mbrowrap.cpp"
                    var $dec=((($13)-1)&4294967295); //@line 158 "mbrowrap.cpp"
                    $i=$dec; //@line 158 "mbrowrap.cpp"
                    var $cmp34=((($dec))|0) > 2; //@line 158 "mbrowrap.cpp"
                    if ($cmp34) { __label__ = 12;continue ; } else { __label__ = 13;break $for_body$$for_end$19; } //@line 158 "mbrowrap.cpp"
                  }
                }
              } while(0);
  
              var $call36=_signal(1, 1); //@line 160 "mbrowrap.cpp"
              var $call37=_signal(2, 1); //@line 161 "mbrowrap.cpp"
              var $call38=_signal(3, 1); //@line 162 "mbrowrap.cpp"
              var $call39=_signal(15, 1); //@line 163 "mbrowrap.cpp"
              var $arraydecay40=(($charbuf)&4294967295); //@line 165 "mbrowrap.cpp"
              var $14=HEAPF32[((__ZL10mbr_volume)>>2)]; //@line 165 "mbrowrap.cpp"
              var $conv=($14); //@line 165 "mbrowrap.cpp"
              var $call41=_snprintf($arraydecay40, 20, ((__str25605)&4294967295), allocate([$conv,0,0,0,0,0,0,0], ["double",0,0,0,0,0,0,0], ALLOC_STACK)); //@line 165 "mbrowrap.cpp"
              var $arraydecay42=(($charbuf)&4294967295); //@line 166 "mbrowrap.cpp"
              var $15=$voice_path_addr; //@line 166 "mbrowrap.cpp"
              var $call43=_execlp(((__str33683)&4294967295), ((__str33683)&4294967295), allocate([((__str27607)&4294967295),0,0,0,((__str28608)&4294967295),0,0,0,$arraydecay42,0,0,0,$15,0,0,0,((__str56708)&4294967295),0,0,0,((__str30610)&4294967295),0,0,0,0,0,0,0], ["i8*",0,0,0,"i8*",0,0,0,"i8*",0,0,0,"i8*",0,0,0,"i8*",0,0,0,"i8*",0,0,0,"i8*",0,0,0], ALLOC_STACK)); //@line 166 "mbrowrap.cpp"
              var $call44=___errno(); //@line 170 "mbrowrap.cpp"
              var $16=HEAP32[(($call44)>>2)]; //@line 170 "mbrowrap.cpp"
              var $call45=_strerror($16); //@line 170 "mbrowrap.cpp"
              var $call46=_snprintf(((__ZL12mbr_errorbuf)&4294967295), 160, ((__str31611)&4294967295), allocate([$call45,0,0,0], ["i8*",0,0,0], ALLOC_STACK)); //@line 170 "mbrowrap.cpp"
              var $call47=_strlen(((__ZL12mbr_errorbuf)&4294967295)); //@line 171 "mbrowrap.cpp"
              var $call48=_write(2, ((__ZL12mbr_errorbuf)&4294967295), $call47); //@line 171 "mbrowrap.cpp"
              __exit(1); //@line 172 "mbrowrap.cpp"
              ; //@line 173 "mbrowrap.cpp"
            }
  
            var $arraydecay50=(($charbuf)&4294967295); //@line 175 "mbrowrap.cpp"
            var $17=HEAP32[((__ZL7mbr_pid)>>2)]; //@line 175 "mbrowrap.cpp"
            var $call51=_snprintf($arraydecay50, 20, ((__str32612)&4294967295), allocate([$17,0,0,0], ["i32",0,0,0], ALLOC_STACK)); //@line 175 "mbrowrap.cpp"
            var $arraydecay52=(($charbuf)&4294967295); //@line 176 "mbrowrap.cpp"
            var $call53=_open($arraydecay52, 0, allocate(1, "i32", ALLOC_STACK)); //@line 176 "mbrowrap.cpp"
            HEAP32[((__ZL13mbr_proc_stat)>>2)]=$call53; //@line 176 "mbrowrap.cpp"
            var $cmp54=((($call53))|0)==-1; //@line 177 "mbrowrap.cpp"
            if ($cmp54) { __label__ = 15;; } else { __label__ = 16;; } //@line 177 "mbrowrap.cpp"
            if (__label__ == 15) {
  
              var $call56=___errno(); //@line 178 "mbrowrap.cpp"
              var $18=HEAP32[(($call56)>>2)]; //@line 178 "mbrowrap.cpp"
              $error=$18; //@line 178 "mbrowrap.cpp"
              var $arraydecay57=(($p_stdin)&4294967295); //@line 179 "mbrowrap.cpp"
              var $arraydecay58=(($p_stdout)&4294967295); //@line 179 "mbrowrap.cpp"
              var $arraydecay59=(($p_stderr)&4294967295); //@line 179 "mbrowrap.cpp"
              __ZL11close_pipesPiS_S_($arraydecay57, $arraydecay58, $arraydecay59); //@line 179 "mbrowrap.cpp"
              var $19=HEAP32[((__ZL7mbr_pid)>>2)]; //@line 180 "mbrowrap.cpp"
              var $call60=_waitpid($19, 0, 0); //@line 180 "mbrowrap.cpp"
              HEAP32[((__ZL7mbr_pid)>>2)]=0; //@line 181 "mbrowrap.cpp"
              var $20=$error; //@line 182 "mbrowrap.cpp"
              var $call61=_strerror($20); //@line 182 "mbrowrap.cpp"
              __ZL3errPKcz(((__str33613)&4294967295), allocate([$call61,0,0,0], ["i8*",0,0,0], ALLOC_STACK)); //@line 182 "mbrowrap.cpp"
              $retval=-1; //@line 183 "mbrowrap.cpp"
              ; //@line 183 "mbrowrap.cpp"
            }
            else if (__label__ == 16) {
  
              var $call63=_signal(13, 1); //@line 186 "mbrowrap.cpp"
              var $arrayidx64=(($p_stdin+4)&4294967295); //@line 188 "mbrowrap.cpp"
              var $21=HEAP32[(($arrayidx64)>>2)]; //@line 188 "mbrowrap.cpp"
              var $call65=_fcntl($21, 4, allocate([16384,0,0,0], ["i32",0,0,0], ALLOC_STACK)); //@line 188 "mbrowrap.cpp"
              var $cmp66=((($call65))|0)==-1; //@line 188 "mbrowrap.cpp"
              if ($cmp66) { __label__ = 19;; } else { __label__ = 17;; } //@line 188 "mbrowrap.cpp"
              $if_then75$$lor_lhs_false67$27: do { 
                if (__label__ == 17) {
  
                  var $arrayidx68=(($p_stdout)&4294967295); //@line 189 "mbrowrap.cpp"
                  var $22=HEAP32[(($arrayidx68)>>2)]; //@line 189 "mbrowrap.cpp"
                  var $call69=_fcntl($22, 4, allocate([16384,0,0,0], ["i32",0,0,0], ALLOC_STACK)); //@line 189 "mbrowrap.cpp"
                  var $cmp70=((($call69))|0)==-1; //@line 189 "mbrowrap.cpp"
                  if ($cmp70) { __label__ = 19;break $if_then75$$lor_lhs_false67$27; } //@line 189 "mbrowrap.cpp"
  
                  var $arrayidx72=(($p_stderr)&4294967295); //@line 190 "mbrowrap.cpp"
                  var $23=HEAP32[(($arrayidx72)>>2)]; //@line 190 "mbrowrap.cpp"
                  var $call73=_fcntl($23, 4, allocate([16384,0,0,0], ["i32",0,0,0], ALLOC_STACK)); //@line 190 "mbrowrap.cpp"
                  var $cmp74=((($call73))|0)==-1; //@line 190 "mbrowrap.cpp"
                  if ($cmp74) { __label__ = 19;break $if_then75$$lor_lhs_false67$27; } //@line 190 "mbrowrap.cpp"
  
                  var $arrayidx83=(($p_stdin+4)&4294967295); //@line 199 "mbrowrap.cpp"
                  var $27=HEAP32[(($arrayidx83)>>2)]; //@line 199 "mbrowrap.cpp"
                  HEAP32[((__ZL10mbr_cmd_fd)>>2)]=$27; //@line 199 "mbrowrap.cpp"
                  var $arrayidx84=(($p_stdout)&4294967295); //@line 200 "mbrowrap.cpp"
                  var $28=HEAP32[(($arrayidx84)>>2)]; //@line 200 "mbrowrap.cpp"
                  HEAP32[((__ZL12mbr_audio_fd)>>2)]=$28; //@line 200 "mbrowrap.cpp"
                  var $arrayidx85=(($p_stderr)&4294967295); //@line 201 "mbrowrap.cpp"
                  var $29=HEAP32[(($arrayidx85)>>2)]; //@line 201 "mbrowrap.cpp"
                  HEAP32[((__ZL12mbr_error_fd)>>2)]=$29; //@line 201 "mbrowrap.cpp"
                  var $arrayidx86=(($p_stdin)&4294967295); //@line 202 "mbrowrap.cpp"
                  var $30=HEAP32[(($arrayidx86)>>2)]; //@line 202 "mbrowrap.cpp"
                  var $call87=_close($30); //@line 202 "mbrowrap.cpp"
                  var $arrayidx88=(($p_stdout+4)&4294967295); //@line 203 "mbrowrap.cpp"
                  var $31=HEAP32[(($arrayidx88)>>2)]; //@line 203 "mbrowrap.cpp"
                  var $call89=_close($31); //@line 203 "mbrowrap.cpp"
                  var $arrayidx90=(($p_stderr+4)&4294967295); //@line 204 "mbrowrap.cpp"
                  var $32=HEAP32[(($arrayidx90)>>2)]; //@line 204 "mbrowrap.cpp"
                  var $call91=_close($32); //@line 204 "mbrowrap.cpp"
                  HEAP32[((__ZL9mbr_state)>>2)]=1; //@line 206 "mbrowrap.cpp"
                  $retval=0; //@line 207 "mbrowrap.cpp"
                  __label__ = 21;break $if_then$$if_end$2; //@line 207 "mbrowrap.cpp"
                }
              } while(0);
  
              var $call76=___errno(); //@line 191 "mbrowrap.cpp"
              var $24=HEAP32[(($call76)>>2)]; //@line 191 "mbrowrap.cpp"
              $error=$24; //@line 191 "mbrowrap.cpp"
              var $arraydecay77=(($p_stdin)&4294967295); //@line 192 "mbrowrap.cpp"
              var $arraydecay78=(($p_stdout)&4294967295); //@line 192 "mbrowrap.cpp"
              var $arraydecay79=(($p_stderr)&4294967295); //@line 192 "mbrowrap.cpp"
              __ZL11close_pipesPiS_S_($arraydecay77, $arraydecay78, $arraydecay79); //@line 192 "mbrowrap.cpp"
              var $25=HEAP32[((__ZL7mbr_pid)>>2)]; //@line 193 "mbrowrap.cpp"
              var $call80=_waitpid($25, 0, 0); //@line 193 "mbrowrap.cpp"
              HEAP32[((__ZL7mbr_pid)>>2)]=0; //@line 194 "mbrowrap.cpp"
              var $26=$error; //@line 195 "mbrowrap.cpp"
              var $call81=_strerror($26); //@line 195 "mbrowrap.cpp"
              __ZL3errPKcz(((__str34614)&4294967295), allocate([$call81,0,0,0], ["i8*",0,0,0], ALLOC_STACK)); //@line 195 "mbrowrap.cpp"
              $retval=-1; //@line 196 "mbrowrap.cpp"
              ; //@line 196 "mbrowrap.cpp"
            }
          }
        }
      }
    } while(0);
  
    var $33=$retval; //@line 208 "mbrowrap.cpp"
    STACKTOP = __stackBase__;
    return $33; //@line 208 "mbrowrap.cpp"
    return null;
  }
  

  function __ZL14send_to_mbrolaPKc($cmd) {
    ;
    var __label__;
    var __lastLabel__ = null;
  
    var $retval;
    var $cmd_addr;
    var $result;
    var $len;
    var $error;
    var $data;
    $cmd_addr=$cmd;
    var $0=HEAP32[((__ZL7mbr_pid)>>2)]; //@line 328 "mbrowrap.cpp"
    var $tobool=((($0))|0)!=0; //@line 328 "mbrowrap.cpp"
    if ($tobool) { __label__ = 2;; } else { __label__ = 1;; } //@line 328 "mbrowrap.cpp"
    $if_end$$if_then$2: do { 
      if (__label__ == 2) {
  
        var $1=$cmd_addr; //@line 331 "mbrowrap.cpp"
        var $call=_strlen($1); //@line 331 "mbrowrap.cpp"
        $len=$call; //@line 331 "mbrowrap.cpp"
        var $2=HEAP32[((__ZL10mbr_cmd_fd)>>2)]; //@line 332 "mbrowrap.cpp"
        var $3=$cmd_addr; //@line 332 "mbrowrap.cpp"
        var $4=$len; //@line 332 "mbrowrap.cpp"
        var $call1=_write($2, $3, $4); //@line 332 "mbrowrap.cpp"
        $result=$call1; //@line 332 "mbrowrap.cpp"
        var $5=$result; //@line 334 "mbrowrap.cpp"
        var $cmp=((($5))|0)==-1; //@line 334 "mbrowrap.cpp"
        if ($cmp) { __label__ = 3;; } else { __label__ = 10;; } //@line 334 "mbrowrap.cpp"
        if (__label__ == 3) {
  
          var $call3=___errno(); //@line 335 "mbrowrap.cpp"
          var $6=HEAP32[(($call3)>>2)]; //@line 335 "mbrowrap.cpp"
          $error=$6; //@line 335 "mbrowrap.cpp"
          var $7=$error; //@line 336 "mbrowrap.cpp"
          var $cmp4=((($7))|0)==32; //@line 336 "mbrowrap.cpp"
          if ($cmp4) { __lastLabel__ = 3; __label__ = 4;; } else { __lastLabel__ = 3; __label__ = 7;; } //@line 336 "mbrowrap.cpp"
          if (__label__ == 4) {
  
            var $call5=__ZL17mbrola_has_errorsv(); //@line 336 "mbrowrap.cpp"
            var $tobool6=((($call5))|0)!=0; //@line 336 "mbrowrap.cpp"
            if ($tobool6) { __label__ = 5;; } else { __label__ = 6;; } //@line 336 "mbrowrap.cpp"
            if (__label__ == 5) {
  
              $retval=-1; //@line 337 "mbrowrap.cpp"
              __label__ = 17;break $if_end$$if_then$2; //@line 337 "mbrowrap.cpp"
            }
            else if (__label__ == 6) {
  
              var $_pr=$error; //@line 338 "mbrowrap.cpp"
              __lastLabel__ = 6; ;
            }
          }
  
          var $8=__lastLabel__ == 6 ? $_pr : ($7); //@line 338 "mbrowrap.cpp"
          var $cmp8=((($8))|0)==11; //@line 338 "mbrowrap.cpp"
          if ($cmp8) { __label__ = 8;; } else { __label__ = 9;; } //@line 338 "mbrowrap.cpp"
          if (__label__ == 8) {
  
            $result=0; //@line 339 "mbrowrap.cpp"
            ; //@line 344 "mbrowrap.cpp"
          }
          else if (__label__ == 9) {
  
            var $9=$error; //@line 341 "mbrowrap.cpp"
            var $call11=_strerror($9); //@line 341 "mbrowrap.cpp"
            __ZL3errPKcz(((__str20600)&4294967295), allocate([$call11,0,0,0], ["i8*",0,0,0], ALLOC_STACK)); //@line 341 "mbrowrap.cpp"
            $retval=-1; //@line 342 "mbrowrap.cpp"
            __label__ = 17;break $if_end$$if_then$2; //@line 342 "mbrowrap.cpp"
          }
        }
  
        var $10=$result; //@line 346 "mbrowrap.cpp"
        var $11=$len; //@line 346 "mbrowrap.cpp"
        var $cmp15=((($10))|0)!=((($11))|0); //@line 346 "mbrowrap.cpp"
        if ($cmp15) { __label__ = 11;; } else { __label__ = 16;; } //@line 346 "mbrowrap.cpp"
        $if_then16$$if_end28$16: do { 
          if (__label__ == 11) {
  
            var $12=$len; //@line 348 "mbrowrap.cpp"
            var $13=$result; //@line 348 "mbrowrap.cpp"
            var $_neg=(((-($13)))&4294967295);
            var $add=((($12)+16)&4294967295); //@line 348 "mbrowrap.cpp"
            var $sub=((($add)+($_neg))&4294967295); //@line 348 "mbrowrap.cpp"
            var $call17=_malloc($sub); //@line 348 "mbrowrap.cpp"
            var $14=$call17; //@line 348 "mbrowrap.cpp"
            $data=$14; //@line 348 "mbrowrap.cpp"
            var $15=$data; //@line 349 "mbrowrap.cpp"
            var $tobool18=((($15))|0)!=0; //@line 349 "mbrowrap.cpp"
            if (!($tobool18)) { __label__ = 16;break $if_then16$$if_end28$16; } //@line 349 "mbrowrap.cpp"
  
            var $16=$data; //@line 350 "mbrowrap.cpp"
            var $next=(($16)&4294967295); //@line 350 "mbrowrap.cpp"
            HEAP32[(($next)>>2)]=0; //@line 350 "mbrowrap.cpp"
            var $17=$data; //@line 351 "mbrowrap.cpp"
            var $done=(($17+4)&4294967295); //@line 351 "mbrowrap.cpp"
            HEAP32[(($done)>>2)]=0; //@line 351 "mbrowrap.cpp"
            var $18=$len; //@line 352 "mbrowrap.cpp"
            var $19=$result; //@line 352 "mbrowrap.cpp"
            var $sub20=((($18)-($19))&4294967295); //@line 352 "mbrowrap.cpp"
            var $20=$data; //@line 352 "mbrowrap.cpp"
            var $size=(($20+8)&4294967295); //@line 352 "mbrowrap.cpp"
            HEAP32[(($size)>>2)]=$sub20; //@line 352 "mbrowrap.cpp"
            var $21=$data; //@line 353 "mbrowrap.cpp"
            var $buffer=(($21+12)&4294967295); //@line 353 "mbrowrap.cpp"
            var $arraydecay=(($buffer)&4294967295); //@line 353 "mbrowrap.cpp"
            var $22=$cmd_addr; //@line 353 "mbrowrap.cpp"
            var $23=$result; //@line 353 "mbrowrap.cpp"
            var $add_ptr=(($22+$23)&4294967295); //@line 353 "mbrowrap.cpp"
            var $24=$len; //@line 353 "mbrowrap.cpp"
            var $25=$result; //@line 353 "mbrowrap.cpp"
            var $sub21=((($24)-($25))&4294967295); //@line 353 "mbrowrap.cpp"
            var $src$, $dest$, $stop$, $stop4$, $fast$;
            $src$ = $add_ptr;
            $dest$ = $arraydecay;
            $stop$ = $src$ + $sub21;
            if (($dest$%4) == ($src$%4) && $sub21 > 8) {
              while ($src$%4 !== 0 && $src$ < $stop$) {
                ; HEAP8[$dest$++] = HEAP8[$src$++];
              }
              $src$ >>= 2;
              $dest$ >>= 2;
              $stop4$ = $stop$ >> 2;
              while ($src$ < $stop4$) {
            ;;;    HEAP32[$dest$++] = HEAP32[$src$++];
              }
              $src$ <<= 2;
              $dest$ <<= 2;
            }while ($src$ < $stop$) {
              ; HEAP8[$dest$++] = HEAP8[$src$++];
            }; //@line 353 "mbrowrap.cpp"
            var $26=$len; //@line 354 "mbrowrap.cpp"
            $result=$26; //@line 354 "mbrowrap.cpp"
            var $27=HEAP32[((__ZL21mbr_pending_data_head)>>2)]; //@line 355 "mbrowrap.cpp"
            var $tobool22=((($27))|0)!=0; //@line 355 "mbrowrap.cpp"
            var $28=$data; //@line 358 "mbrowrap.cpp"
            if ($tobool22) { __label__ = 14;; } else { __label__ = 13;; } //@line 355 "mbrowrap.cpp"
            if (__label__ == 14) {
  
              var $29=HEAP32[((__ZL21mbr_pending_data_tail)>>2)]; //@line 358 "mbrowrap.cpp"
              var $next25=(($29)&4294967295); //@line 358 "mbrowrap.cpp"
              HEAP32[(($next25)>>2)]=$28; //@line 358 "mbrowrap.cpp"
              ;
            }
            else if (__label__ == 13) {
  
              HEAP32[((__ZL21mbr_pending_data_head)>>2)]=$28; //@line 356 "mbrowrap.cpp"
              ; //@line 356 "mbrowrap.cpp"
            }
  
            var $30=$data; //@line 359 "mbrowrap.cpp"
            HEAP32[((__ZL21mbr_pending_data_tail)>>2)]=$30; //@line 359 "mbrowrap.cpp"
            ; //@line 360 "mbrowrap.cpp"
          }
        } while(0);
  
        var $31=$result; //@line 363 "mbrowrap.cpp"
        $retval=$31; //@line 363 "mbrowrap.cpp"
        ; //@line 363 "mbrowrap.cpp"
      }
      else if (__label__ == 1) {
  
        $retval=-1; //@line 329 "mbrowrap.cpp"
        ; //@line 329 "mbrowrap.cpp"
      }
    } while(0);
  
    var $32=$retval; //@line 364 "mbrowrap.cpp"
    ;
    return $32; //@line 364 "mbrowrap.cpp"
    return null;
  }
  

  function __ZL11stop_mbrolav() {
    ;
    var __label__;
  
    var $0=HEAP32[((__ZL9mbr_state)>>2)]; //@line 212 "mbrowrap.cpp"
    var $cmp=((($0))|0)==0; //@line 212 "mbrowrap.cpp"
    if ($cmp) { __label__ = 4;; } else { __label__ = 1;; } //@line 212 "mbrowrap.cpp"
    if (__label__ == 1) {
  
      var $1=HEAP32[((__ZL13mbr_proc_stat)>>2)]; //@line 214 "mbrowrap.cpp"
      var $call=_close($1); //@line 214 "mbrowrap.cpp"
      var $2=HEAP32[((__ZL10mbr_cmd_fd)>>2)]; //@line 215 "mbrowrap.cpp"
      var $call1=_close($2); //@line 215 "mbrowrap.cpp"
      var $3=HEAP32[((__ZL12mbr_audio_fd)>>2)]; //@line 216 "mbrowrap.cpp"
      var $call2=_close($3); //@line 216 "mbrowrap.cpp"
      var $4=HEAP32[((__ZL12mbr_error_fd)>>2)]; //@line 217 "mbrowrap.cpp"
      var $call3=_close($4); //@line 217 "mbrowrap.cpp"
      var $5=HEAP32[((__ZL7mbr_pid)>>2)]; //@line 218 "mbrowrap.cpp"
      var $tobool=((($5))|0)!=0; //@line 218 "mbrowrap.cpp"
      if ($tobool) { __label__ = 2;; } else { __label__ = 3;; } //@line 218 "mbrowrap.cpp"
      if (__label__ == 2) {
  
        var $6=HEAP32[((__ZL7mbr_pid)>>2)]; //@line 219 "mbrowrap.cpp"
        var $call5=_kill($6, 15); //@line 219 "mbrowrap.cpp"
        var $7=HEAP32[((__ZL7mbr_pid)>>2)]; //@line 220 "mbrowrap.cpp"
        var $call6=_waitpid($7, 0, 0); //@line 220 "mbrowrap.cpp"
        HEAP32[((__ZL7mbr_pid)>>2)]=0; //@line 221 "mbrowrap.cpp"
        ; //@line 222 "mbrowrap.cpp"
      }
  
      HEAP32[((__ZL9mbr_state)>>2)]=0; //@line 223 "mbrowrap.cpp"
      ; //@line 224 "mbrowrap.cpp"
    }
  
    ;
    return; //@line 224 "mbrowrap.cpp"
    return;
  }
  

  function __ZL19receive_from_mbrolaPvj($buffer, $bufsize) {
    var __stackBase__  = STACKTOP; STACKTOP += 24; _memset(__stackBase__, 0, 24);
    var __label__;
  
    var $retval;
    var $buffer_addr;
    var $bufsize_addr;
    var $result;
    var $wait;
    var $cursize;
    var $pollfd=__stackBase__;
    var $nfds;
    var $idle;
    var $head;
    var $data;
    var $left;
    var $error;
    var $curpos;
    var $space;
    var $obtained;
    $buffer_addr=$buffer;
    $bufsize_addr=$bufsize;
    $wait=1; //@line 384 "mbrowrap.cpp"
    $cursize=0; //@line 385 "mbrowrap.cpp"
    var $0=HEAP32[((__ZL7mbr_pid)>>2)]; //@line 387 "mbrowrap.cpp"
    var $tobool=((($0))|0)!=0; //@line 387 "mbrowrap.cpp"
    if ($tobool) { __label__ = 1;; } else { __label__ = 2;; } //@line 387 "mbrowrap.cpp"
    $do_body_preheader$$if_then$2: do { 
      if (__label__ == 1) {
  
        var $arrayidx=(($pollfd)&4294967295); //@line 395 "mbrowrap.cpp"
        var $fd=(($arrayidx)&4294967295); //@line 395 "mbrowrap.cpp"
        var $arrayidx1=(($pollfd)&4294967295); //@line 396 "mbrowrap.cpp"
        var $events=(($arrayidx1+4)&4294967295); //@line 396 "mbrowrap.cpp"
        var $arrayidx2=(($pollfd+8)&4294967295); //@line 399 "mbrowrap.cpp"
        var $fd3=(($arrayidx2)&4294967295); //@line 399 "mbrowrap.cpp"
        var $arrayidx4=(($pollfd+8)&4294967295); //@line 400 "mbrowrap.cpp"
        var $events5=(($arrayidx4+4)&4294967295); //@line 400 "mbrowrap.cpp"
        var $arrayidx9=(($pollfd+16)&4294967295); //@line 404 "mbrowrap.cpp"
        var $fd10=(($arrayidx9)&4294967295); //@line 404 "mbrowrap.cpp"
        var $arrayidx11=(($pollfd+16)&4294967295); //@line 405 "mbrowrap.cpp"
        var $events12=(($arrayidx11+4)&4294967295); //@line 405 "mbrowrap.cpp"
        var $arraydecay=(($pollfd)&4294967295); //@line 410 "mbrowrap.cpp"
        var $arrayidx29=(($pollfd+8)&4294967295); //@line 432 "mbrowrap.cpp"
        var $revents=(($arrayidx29+6)&4294967295); //@line 432 "mbrowrap.cpp"
        var $arrayidx37=(($pollfd+16)&4294967295); //@line 435 "mbrowrap.cpp"
        var $revents38=(($arrayidx37+6)&4294967295); //@line 435 "mbrowrap.cpp"
        var $arrayidx66=(($pollfd)&4294967295); //@line 459 "mbrowrap.cpp"
        var $revents67=(($arrayidx66+6)&4294967295); //@line 459 "mbrowrap.cpp"
        ; //@line 391 "mbrowrap.cpp"
        $do_body$4: while(1) { 
  
          $nfds=0; //@line 392 "mbrowrap.cpp"
          var $1=HEAP32[((__ZL12mbr_audio_fd)>>2)]; //@line 395 "mbrowrap.cpp"
          HEAP32[(($fd)>>2)]=$1; //@line 395 "mbrowrap.cpp"
          HEAP16[(($events)>>1)]=1; //@line 396 "mbrowrap.cpp"
          var $2=$nfds; //@line 397 "mbrowrap.cpp"
          var $inc=((($2)+1)&4294967295); //@line 397 "mbrowrap.cpp"
          $nfds=$inc; //@line 397 "mbrowrap.cpp"
          var $3=HEAP32[((__ZL12mbr_error_fd)>>2)]; //@line 399 "mbrowrap.cpp"
          HEAP32[(($fd3)>>2)]=$3; //@line 399 "mbrowrap.cpp"
          HEAP16[(($events5)>>1)]=1; //@line 400 "mbrowrap.cpp"
          var $4=$nfds; //@line 401 "mbrowrap.cpp"
          var $inc6=((($4)+1)&4294967295); //@line 401 "mbrowrap.cpp"
          $nfds=$inc6; //@line 401 "mbrowrap.cpp"
          var $5=HEAP32[((__ZL21mbr_pending_data_head)>>2)]; //@line 403 "mbrowrap.cpp"
          var $tobool7=((($5))|0)!=0; //@line 403 "mbrowrap.cpp"
          if ($tobool7) { __label__ = 4;; } else { __label__ = 5;; } //@line 403 "mbrowrap.cpp"
          if (__label__ == 4) {
  
            var $6=HEAP32[((__ZL10mbr_cmd_fd)>>2)]; //@line 404 "mbrowrap.cpp"
            HEAP32[(($fd10)>>2)]=$6; //@line 404 "mbrowrap.cpp"
            HEAP16[(($events12)>>1)]=2; //@line 405 "mbrowrap.cpp"
            var $7=$nfds; //@line 406 "mbrowrap.cpp"
            var $inc13=((($7)+1)&4294967295); //@line 406 "mbrowrap.cpp"
            $nfds=$inc13; //@line 406 "mbrowrap.cpp"
            ; //@line 407 "mbrowrap.cpp"
          }
  
          var $call=__ZL14mbrola_is_idlev(); //@line 409 "mbrowrap.cpp"
          $idle=$call; //@line 409 "mbrowrap.cpp"
          var $8=$nfds; //@line 410 "mbrowrap.cpp"
          var $9=$idle; //@line 410 "mbrowrap.cpp"
          var $tobool15=((($9))|0)!=0; //@line 410 "mbrowrap.cpp"
          var $10=$wait; //@line 410 "mbrowrap.cpp"
          var $cond=($tobool15) ? 0 : ($10); //@line 410 "mbrowrap.cpp"
          var $call16=_poll($arraydecay, $8, $cond); //@line 410 "mbrowrap.cpp"
          $result=$call16; //@line 410 "mbrowrap.cpp"
          var $cmp=((($call16))|0)==-1; //@line 411 "mbrowrap.cpp"
          if ($cmp) { __label__ = 6;break $do_body$4; } //@line 411 "mbrowrap.cpp"
  
          var $12=$result; //@line 415 "mbrowrap.cpp"
          var $cmp21=((($12))|0)==0; //@line 415 "mbrowrap.cpp"
          if ($cmp21) { __label__ = 8;; } else { __label__ = 13;; } //@line 415 "mbrowrap.cpp"
          $if_then22$$if_end28$10: do { 
            if (__label__ == 8) {
  
              var $13=$idle; //@line 416 "mbrowrap.cpp"
              var $tobool23=((($13))|0)!=0; //@line 416 "mbrowrap.cpp"
              if ($tobool23) { __label__ = 9;break $do_body$4; } //@line 416 "mbrowrap.cpp"
  
              var $14=$wait; //@line 420 "mbrowrap.cpp"
              var $cmp25=((($14))|0) >= 3750; //@line 420 "mbrowrap.cpp"
              if ($cmp25) { __label__ = 11;break $do_body$4; } //@line 420 "mbrowrap.cpp"
  
              var $15=$wait; //@line 425 "mbrowrap.cpp"
              var $mul=((($15)*4)&4294967295); //@line 425 "mbrowrap.cpp"
              $wait=$mul; //@line 425 "mbrowrap.cpp"
              ; //@line 426 "mbrowrap.cpp"
            }
            else if (__label__ == 13) {
  
              $wait=1; //@line 430 "mbrowrap.cpp"
              var $16=HEAP16[(($revents)>>1)]; //@line 432 "mbrowrap.cpp"
              var $tobool30=(tempInt=(($16)),(tempInt>=32768?tempInt-65536:tempInt))!=0; //@line 432 "mbrowrap.cpp"
              if ($tobool30) { __label__ = 14;; } else { __label__ = 16;; } //@line 432 "mbrowrap.cpp"
              if (__label__ == 14) {
  
                var $call31=__ZL17mbrola_has_errorsv(); //@line 432 "mbrowrap.cpp"
                var $tobool32=((($call31))|0)!=0; //@line 432 "mbrowrap.cpp"
                if ($tobool32) { __label__ = 15;break $do_body$4; } //@line 432 "mbrowrap.cpp"
              }
  
              var $17=HEAP32[((__ZL21mbr_pending_data_head)>>2)]; //@line 435 "mbrowrap.cpp"
              var $tobool35=((($17))|0)!=0; //@line 435 "mbrowrap.cpp"
              if ($tobool35) { __label__ = 17;; } else { __label__ = 27;; } //@line 435 "mbrowrap.cpp"
              $land_lhs_true36$$if_end65$18: do { 
                if (__label__ == 17) {
  
                  var $18=HEAP16[(($revents38)>>1)]; //@line 435 "mbrowrap.cpp"
                  var $tobool39=(tempInt=(($18)),(tempInt>=32768?tempInt-65536:tempInt))!=0; //@line 435 "mbrowrap.cpp"
                  if (!($tobool39)) { __label__ = 27;break $land_lhs_true36$$if_end65$18; } //@line 435 "mbrowrap.cpp"
  
                  var $19=HEAP32[((__ZL21mbr_pending_data_head)>>2)]; //@line 436 "mbrowrap.cpp"
                  $head=$19; //@line 436 "mbrowrap.cpp"
                  var $20=$head; //@line 437 "mbrowrap.cpp"
                  var $buffer41=(($20+12)&4294967295); //@line 437 "mbrowrap.cpp"
                  var $arraydecay42=(($buffer41)&4294967295); //@line 437 "mbrowrap.cpp"
                  var $21=$head; //@line 437 "mbrowrap.cpp"
                  var $done=(($21+4)&4294967295); //@line 437 "mbrowrap.cpp"
                  var $22=HEAP32[(($done)>>2)]; //@line 437 "mbrowrap.cpp"
                  var $add_ptr=(($arraydecay42+$22)&4294967295); //@line 437 "mbrowrap.cpp"
                  $data=$add_ptr; //@line 437 "mbrowrap.cpp"
                  var $23=$head; //@line 438 "mbrowrap.cpp"
                  var $size=(($23+8)&4294967295); //@line 438 "mbrowrap.cpp"
                  var $24=HEAP32[(($size)>>2)]; //@line 438 "mbrowrap.cpp"
                  var $25=$head; //@line 438 "mbrowrap.cpp"
                  var $done43=(($25+4)&4294967295); //@line 438 "mbrowrap.cpp"
                  var $26=HEAP32[(($done43)>>2)]; //@line 438 "mbrowrap.cpp"
                  var $sub=((($24)-($26))&4294967295); //@line 438 "mbrowrap.cpp"
                  $left=$sub; //@line 438 "mbrowrap.cpp"
                  var $27=HEAP32[((__ZL10mbr_cmd_fd)>>2)]; //@line 439 "mbrowrap.cpp"
                  var $28=$data; //@line 439 "mbrowrap.cpp"
                  var $29=$left; //@line 439 "mbrowrap.cpp"
                  var $call44=_write($27, $28, $29); //@line 439 "mbrowrap.cpp"
                  $result=$call44; //@line 439 "mbrowrap.cpp"
                  var $30=$result; //@line 440 "mbrowrap.cpp"
                  var $cmp45=((($30))|0)==-1; //@line 440 "mbrowrap.cpp"
                  if ($cmp45) { __label__ = 19;break $do_body$4; } //@line 440 "mbrowrap.cpp"
  
                  var $34=$result; //@line 447 "mbrowrap.cpp"
                  var $35=$left; //@line 447 "mbrowrap.cpp"
                  var $cmp56=((($34))|0)!=((($35))|0); //@line 447 "mbrowrap.cpp"
                  if ($cmp56) { __label__ = 24;; } else { __label__ = 25;; } //@line 447 "mbrowrap.cpp"
                  if (__label__ == 24) {
  
                    var $36=$result; //@line 448 "mbrowrap.cpp"
                    var $37=$head; //@line 448 "mbrowrap.cpp"
                    var $done58=(($37+4)&4294967295); //@line 448 "mbrowrap.cpp"
                    var $38=HEAP32[(($done58)>>2)]; //@line 448 "mbrowrap.cpp"
                    var $add=((($38)+($36))&4294967295); //@line 448 "mbrowrap.cpp"
                    HEAP32[(($done58)>>2)]=$add; //@line 448 "mbrowrap.cpp"
                    ; //@line 449 "mbrowrap.cpp"
                  }
                  else if (__label__ == 25) {
  
                    var $39=$head; //@line 450 "mbrowrap.cpp"
                    var $next=(($39)&4294967295); //@line 450 "mbrowrap.cpp"
                    var $40=HEAP32[(($next)>>2)]; //@line 450 "mbrowrap.cpp"
                    HEAP32[((__ZL21mbr_pending_data_head)>>2)]=$40; //@line 450 "mbrowrap.cpp"
                    var $41=$head; //@line 451 "mbrowrap.cpp"
                    var $42=$41; //@line 451 "mbrowrap.cpp"
                    ; //@line 451 "mbrowrap.cpp"
                    var $43=HEAP32[((__ZL21mbr_pending_data_head)>>2)]; //@line 452 "mbrowrap.cpp"
                    var $tobool60=((($43))|0)!=0; //@line 452 "mbrowrap.cpp"
                    if ($tobool60) { __label__ = 31;break $if_then22$$if_end28$10; } //@line 452 "mbrowrap.cpp"
  
                    HEAP32[((__ZL21mbr_pending_data_tail)>>2)]=0; //@line 453 "mbrowrap.cpp"
                    ;
                  }
                }
              } while(0);
  
              var $44=HEAP16[(($revents67)>>1)]; //@line 459 "mbrowrap.cpp"
              var $tobool68=(tempInt=(($44)),(tempInt>=32768?tempInt-65536:tempInt))!=0; //@line 459 "mbrowrap.cpp"
              if (!($tobool68)) { __label__ = 31;break $if_then22$$if_end28$10; } //@line 459 "mbrowrap.cpp"
  
              var $45=$buffer_addr; //@line 460 "mbrowrap.cpp"
              var $46=$cursize; //@line 460 "mbrowrap.cpp"
              var $add_ptr70=(($45+$46)&4294967295); //@line 460 "mbrowrap.cpp"
              $curpos=$add_ptr70; //@line 460 "mbrowrap.cpp"
              var $47=$bufsize_addr; //@line 461 "mbrowrap.cpp"
              var $48=$cursize; //@line 461 "mbrowrap.cpp"
              var $sub71=((($47)-($48))&4294967295); //@line 461 "mbrowrap.cpp"
              $space=$sub71; //@line 461 "mbrowrap.cpp"
              var $49=HEAP32[((__ZL12mbr_audio_fd)>>2)]; //@line 462 "mbrowrap.cpp"
              var $50=$curpos; //@line 462 "mbrowrap.cpp"
              var $51=$space; //@line 462 "mbrowrap.cpp"
              var $call72=_read($49, $50, $51); //@line 462 "mbrowrap.cpp"
              $obtained=$call72; //@line 462 "mbrowrap.cpp"
              var $52=$obtained; //@line 463 "mbrowrap.cpp"
              var $cmp73=((($52))|0)==-1; //@line 463 "mbrowrap.cpp"
              if ($cmp73) { __label__ = 29;break $do_body$4; } //@line 463 "mbrowrap.cpp"
  
              var $54=$obtained; //@line 467 "mbrowrap.cpp"
              var $55=$cursize; //@line 467 "mbrowrap.cpp"
              var $add78=((($55)+($54))&4294967295); //@line 467 "mbrowrap.cpp"
              $cursize=$add78; //@line 467 "mbrowrap.cpp"
              HEAP32[((__ZL9mbr_state)>>2)]=3; //@line 468 "mbrowrap.cpp"
              ; //@line 469 "mbrowrap.cpp"
            }
          } while(0);
  
          var $56=$cursize; //@line 470 "mbrowrap.cpp"
          var $57=$bufsize_addr; //@line 470 "mbrowrap.cpp"
          var $cmp80=((($56))>>>0) < ((($57))>>>0); //@line 470 "mbrowrap.cpp"
          if ($cmp80) { __label__ = 3;continue $do_body$4; } else { __label__ = 32;break $do_body$4; } //@line 470 "mbrowrap.cpp"
        }
        if (__label__ == 6) {
  
          var $call18=___errno(); //@line 412 "mbrowrap.cpp"
          var $11=HEAP32[(($call18)>>2)]; //@line 412 "mbrowrap.cpp"
          var $call19=_strerror($11); //@line 412 "mbrowrap.cpp"
          __ZL3errPKcz(((__str18598)&4294967295), allocate([$call19,0,0,0], ["i8*",0,0,0], ALLOC_STACK)); //@line 412 "mbrowrap.cpp"
          $retval=-1; //@line 413 "mbrowrap.cpp"
          __label__ = 33;break $do_body_preheader$$if_then$2; //@line 413 "mbrowrap.cpp"
        }
        else if (__label__ == 9) {
  
          HEAP32[((__ZL9mbr_state)>>2)]=1; //@line 417 "mbrowrap.cpp"
          ; //@line 418 "mbrowrap.cpp"
        }
        else if (__label__ == 11) {
  
          HEAP32[((__ZL9mbr_state)>>2)]=4; //@line 421 "mbrowrap.cpp"
          __ZL3errPKcz(((__str19599)&4294967295), allocate(1, "i32", ALLOC_STACK)); //@line 422 "mbrowrap.cpp"
          ; //@line 423 "mbrowrap.cpp"
        }
        else if (__label__ == 15) {
  
          $retval=-1; //@line 433 "mbrowrap.cpp"
          __label__ = 33;break $do_body_preheader$$if_then$2; //@line 433 "mbrowrap.cpp"
        }
        else if (__label__ == 19) {
  
          var $call47=___errno(); //@line 441 "mbrowrap.cpp"
          var $31=HEAP32[(($call47)>>2)]; //@line 441 "mbrowrap.cpp"
          $error=$31; //@line 441 "mbrowrap.cpp"
          var $32=$error; //@line 442 "mbrowrap.cpp"
          var $cmp48=((($32))|0)==32; //@line 442 "mbrowrap.cpp"
          if ($cmp48) { __label__ = 20;; } else { __label__ = 22;; } //@line 442 "mbrowrap.cpp"
          $land_lhs_true49$$if_end53$36: do { 
            if (__label__ == 20) {
  
              var $call50=__ZL17mbrola_has_errorsv(); //@line 442 "mbrowrap.cpp"
              var $tobool51=((($call50))|0)!=0; //@line 442 "mbrowrap.cpp"
              if (!($tobool51)) { __label__ = 22;break $land_lhs_true49$$if_end53$36; } //@line 442 "mbrowrap.cpp"
  
              $retval=-1; //@line 443 "mbrowrap.cpp"
              __label__ = 33;break $do_body_preheader$$if_then$2; //@line 443 "mbrowrap.cpp"
            }
          } while(0);
  
          var $33=$error; //@line 444 "mbrowrap.cpp"
          var $call54=_strerror($33); //@line 444 "mbrowrap.cpp"
          __ZL3errPKcz(((__str20600)&4294967295), allocate([$call54,0,0,0], ["i8*",0,0,0], ALLOC_STACK)); //@line 444 "mbrowrap.cpp"
          $retval=-1; //@line 445 "mbrowrap.cpp"
          __label__ = 33;break $do_body_preheader$$if_then$2; //@line 445 "mbrowrap.cpp"
        }
        else if (__label__ == 29) {
  
          var $call75=___errno(); //@line 464 "mbrowrap.cpp"
          var $53=HEAP32[(($call75)>>2)]; //@line 464 "mbrowrap.cpp"
          var $call76=_strerror($53); //@line 464 "mbrowrap.cpp"
          __ZL3errPKcz(((__str21601)&4294967295), allocate([$call76,0,0,0], ["i8*",0,0,0], ALLOC_STACK)); //@line 464 "mbrowrap.cpp"
          $retval=-1; //@line 465 "mbrowrap.cpp"
          __label__ = 33;break $do_body_preheader$$if_then$2; //@line 465 "mbrowrap.cpp"
        }
  
        var $58=$cursize; //@line 472 "mbrowrap.cpp"
        $retval=$58; //@line 472 "mbrowrap.cpp"
        ; //@line 472 "mbrowrap.cpp"
      }
      else if (__label__ == 2) {
  
        $retval=-1; //@line 388 "mbrowrap.cpp"
        ; //@line 388 "mbrowrap.cpp"
      }
    } while(0);
  
    var $59=$retval; //@line 473 "mbrowrap.cpp"
    STACKTOP = __stackBase__;
    return $59; //@line 473 "mbrowrap.cpp"
    return null;
  }
  

  function __ZL3errPKcz($errmsg) {
    var __stackBase__  = STACKTOP; STACKTOP += 4; _memset(__stackBase__, 0, 4);
    var __label__;
  
    var $errmsg_addr;
    var $params=__stackBase__;
    $errmsg_addr=$errmsg;
    var $params1=$params; //@line 84 "mbrowrap.cpp"
    HEAP32[(($params1)>>2)]=arguments[__ZL3errPKcz.length];; //@line 84 "mbrowrap.cpp"
    var $0=$errmsg_addr; //@line 85 "mbrowrap.cpp"
    var $1=HEAP32[(($params)>>2)]; //@line 85 "mbrowrap.cpp"
    var $call=_vsnprintf(((__ZL12mbr_errorbuf)&4294967295), 160, $0, $1); //@line 85 "mbrowrap.cpp"
    var $params2=$params; //@line 86 "mbrowrap.cpp"
    ; //@line 86 "mbrowrap.cpp"
    __ZL3logPKcz(((__str16596)&4294967295), allocate([((__ZL12mbr_errorbuf)&4294967295),0,0,0], ["i8*",0,0,0], ALLOC_STACK)); //@line 87 "mbrowrap.cpp"
    STACKTOP = __stackBase__;
    return; //@line 88 "mbrowrap.cpp"
    return;
  }
  

  function _close_MBR() {
    ;
    var __label__;
  
    __ZL11stop_mbrolav(); //@line 525 "mbrowrap.cpp"
    __ZL17free_pending_datav(); //@line 526 "mbrowrap.cpp"
    var $0=HEAP32[((__ZL14mbr_voice_path)>>2)]; //@line 527 "mbrowrap.cpp"
    ; //@line 527 "mbrowrap.cpp"
    HEAP32[((__ZL14mbr_voice_path)>>2)]=0; //@line 528 "mbrowrap.cpp"
    HEAPF32[((__ZL10mbr_volume)>>2)]=1; //@line 529 "mbrowrap.cpp"
    ;
    return; //@line 530 "mbrowrap.cpp"
    return;
  }
  

  function __ZL17free_pending_datav() {
    ;
    var __label__;
  
    var $p;
    var $head;
    var $0=HEAP32[((__ZL21mbr_pending_data_head)>>2)]; //@line 228 "mbrowrap.cpp"
    $head=$0; //@line 228 "mbrowrap.cpp"
    var $tobool1=((($0))|0)!=0; //@line 229 "mbrowrap.cpp"
    if ($tobool1) { __label__ = 1;; } else { __label__ = 2;; } //@line 229 "mbrowrap.cpp"
    $while_body$$while_end$2: do { 
      if (__label__ == 1) {
        while(1) { 
  
          var $1=$head; //@line 230 "mbrowrap.cpp"
          $p=$1; //@line 230 "mbrowrap.cpp"
          var $2=$head; //@line 231 "mbrowrap.cpp"
          var $next=(($2)&4294967295); //@line 231 "mbrowrap.cpp"
          var $3=HEAP32[(($next)>>2)]; //@line 231 "mbrowrap.cpp"
          $head=$3; //@line 231 "mbrowrap.cpp"
          var $4=$p; //@line 232 "mbrowrap.cpp"
          var $5=$4; //@line 232 "mbrowrap.cpp"
          ; //@line 232 "mbrowrap.cpp"
          var $_pr=$head; //@line 229 "mbrowrap.cpp"
          var $tobool=((($_pr))|0)!=0; //@line 229 "mbrowrap.cpp"
          if ($tobool) { __label__ = 1;continue ; } else { __label__ = 2;break $while_body$$while_end$2; } //@line 229 "mbrowrap.cpp"
        }
      }
    } while(0);
  
    HEAP32[((__ZL21mbr_pending_data_head)>>2)]=0; //@line 234 "mbrowrap.cpp"
    HEAP32[((__ZL21mbr_pending_data_tail)>>2)]=0; //@line 235 "mbrowrap.cpp"
    ;
    return; //@line 236 "mbrowrap.cpp"
    return;
  }
  

  function _reset_MBR() {
    var __stackBase__  = STACKTOP; STACKTOP += 4096; _memset(__stackBase__, 0, 4096);
    var __label__;
  
    var $retval;
    var $result;
    var $success;
    var $dummybuf=__stackBase__;
    $success=1; //@line 534 "mbrowrap.cpp"
    var $0=HEAP32[((__ZL9mbr_state)>>2)]; //@line 537 "mbrowrap.cpp"
    var $cmp=((($0))|0)==1; //@line 537 "mbrowrap.cpp"
    if ($cmp) { __label__ = 1;; } else { __label__ = 2;; } //@line 537 "mbrowrap.cpp"
    if (__label__ == 1) {
  
      $retval=1; //@line 538 "mbrowrap.cpp"
      ; //@line 538 "mbrowrap.cpp"
    }
    else if (__label__ == 2) {
  
      var $1=HEAP32[((__ZL7mbr_pid)>>2)]; //@line 539 "mbrowrap.cpp"
      var $tobool=((($1))|0)!=0; //@line 539 "mbrowrap.cpp"
      if ($tobool) { __label__ = 4;; } else { __label__ = 3;; } //@line 539 "mbrowrap.cpp"
      if (__label__ == 4) {
  
        var $2=HEAP32[((__ZL7mbr_pid)>>2)]; //@line 541 "mbrowrap.cpp"
        var $call=_kill($2, 30); //@line 541 "mbrowrap.cpp"
        var $cmp3=((($call))|0)==-1; //@line 541 "mbrowrap.cpp"
        if ($cmp3) { __label__ = 5;; } else { __label__ = 6;; } //@line 541 "mbrowrap.cpp"
        if (__label__ == 5) {
  
          $success=0; //@line 542 "mbrowrap.cpp"
          ; //@line 542 "mbrowrap.cpp"
        }
  
        __ZL17free_pending_datav(); //@line 543 "mbrowrap.cpp"
        var $3=HEAP32[((__ZL10mbr_cmd_fd)>>2)]; //@line 544 "mbrowrap.cpp"
        var $call6=_write($3, ((__str5585)&4294967295), 3); //@line 544 "mbrowrap.cpp"
        $result=$call6; //@line 544 "mbrowrap.cpp"
        var $cmp7=((($call6))|0)!=3; //@line 545 "mbrowrap.cpp"
        if ($cmp7) { __label__ = 7;; } else { __label__ = 8;; } //@line 545 "mbrowrap.cpp"
        if (__label__ == 7) {
  
          $success=0; //@line 546 "mbrowrap.cpp"
          ; //@line 546 "mbrowrap.cpp"
        }
  
        var $arraydecay=(($dummybuf)&4294967295); //@line 548 "mbrowrap.cpp"
        ; //@line 548 "mbrowrap.cpp"
        $do_body$13: while(1) { 
  
          var $4=HEAP32[((__ZL12mbr_audio_fd)>>2)]; //@line 548 "mbrowrap.cpp"
          var $call10=_read($4, $arraydecay, 4096); //@line 548 "mbrowrap.cpp"
          $result=$call10; //@line 548 "mbrowrap.cpp"
          var $cmp11=((($call10))|0) > 0; //@line 549 "mbrowrap.cpp"
          if ($cmp11) { __label__ = 9;continue $do_body$13; } else { __label__ = 10;break $do_body$13; } //@line 549 "mbrowrap.cpp"
        }
  
        var $5=$result; //@line 550 "mbrowrap.cpp"
        var $cmp12=((($5))|0)!=-1; //@line 550 "mbrowrap.cpp"
        if ($cmp12) { __label__ = 12;; } else { __label__ = 11;; } //@line 550 "mbrowrap.cpp"
        $if_then15$$lor_lhs_false$16: do { 
          if (__label__ == 11) {
  
            var $call13=___errno(); //@line 550 "mbrowrap.cpp"
            var $6=HEAP32[(($call13)>>2)]; //@line 550 "mbrowrap.cpp"
            var $cmp14=((($6))|0)!=11; //@line 550 "mbrowrap.cpp"
            if ($cmp14) { __label__ = 12;break $if_then15$$lor_lhs_false$16; } else { __label__ = 13;break $if_then15$$lor_lhs_false$16; } //@line 550 "mbrowrap.cpp"
          }
        } while(0);
        if (__label__ == 12) {
  
          $success=0; //@line 551 "mbrowrap.cpp"
          ; //@line 551 "mbrowrap.cpp"
        }
  
        var $call17=__ZL17mbrola_has_errorsv(); //@line 552 "mbrowrap.cpp"
        var $tobool18=((($call17))|0)!=0; //@line 552 "mbrowrap.cpp"
        if ($tobool18) { __label__ = 16;; } else { __label__ = 14;; } //@line 552 "mbrowrap.cpp"
        $if_end21$$land_lhs_true$21: do { 
          if (__label__ == 14) {
  
            var $7=$success; //@line 552 "mbrowrap.cpp"
            var $tobool19=((($7))|0)!=0; //@line 552 "mbrowrap.cpp"
            if (!($tobool19)) { __label__ = 16;break $if_end21$$land_lhs_true$21; } //@line 552 "mbrowrap.cpp"
  
            HEAP32[((__ZL9mbr_state)>>2)]=1; //@line 553 "mbrowrap.cpp"
            ; //@line 553 "mbrowrap.cpp"
          }
        } while(0);
  
        var $8=$success; //@line 554 "mbrowrap.cpp"
        $retval=$8; //@line 554 "mbrowrap.cpp"
        ; //@line 554 "mbrowrap.cpp"
      }
      else if (__label__ == 3) {
  
        $retval=0; //@line 540 "mbrowrap.cpp"
        ; //@line 540 "mbrowrap.cpp"
      }
    }
  
    var $9=$retval; //@line 555 "mbrowrap.cpp"
    STACKTOP = __stackBase__;
    return $9; //@line 555 "mbrowrap.cpp"
    return null;
  }
  

  function __ZL17mbrola_has_errorsv() {
    var __stackBase__  = STACKTOP; STACKTOP += 256; _memset(__stackBase__, 0, 256);
    var __label__;
  
    var $retval;
    var $result;
    var $buffer=__stackBase__;
    var $buf_ptr;
    var $lf;
    var $arraydecay=(($buffer)&4294967295); //@line 284 "mbrowrap.cpp"
    $buf_ptr=$arraydecay; //@line 284 "mbrowrap.cpp"
    var $arraydecay1=(($buffer)&4294967295); //@line 286 "mbrowrap.cpp"
    var $sub_ptr_rhs_cast=($arraydecay1); //@line 286 "mbrowrap.cpp"
    var $sub_ptr_rhs_cast_neg=(((-($sub_ptr_rhs_cast)))&4294967295);
    var $sub_ptr_rhs_cast_neg_neg=(((-($sub_ptr_rhs_cast_neg)))&4294967295);
    var $sub=((($sub_ptr_rhs_cast_neg_neg)+255)&4294967295); //@line 286 "mbrowrap.cpp"
    var $arraydecay27=(($buffer)&4294967295); //@line 318 "mbrowrap.cpp"
    var $arraydecay28=(($buffer)&4294967295); //@line 319 "mbrowrap.cpp"
    ; //@line 285 "mbrowrap.cpp"
    $for_cond$2: while(1) { 
  
      var $0=HEAP32[((__ZL12mbr_error_fd)>>2)]; //@line 286 "mbrowrap.cpp"
      var $1=$buf_ptr; //@line 286 "mbrowrap.cpp"
      var $2=$buf_ptr; //@line 286 "mbrowrap.cpp"
      var $sub_ptr_lhs_cast=($2); //@line 286 "mbrowrap.cpp"
      var $sub_ptr_lhs_cast_neg=(((-($sub_ptr_lhs_cast)))&4294967295);
      var $sub2=((($sub)+($sub_ptr_lhs_cast_neg))&4294967295); //@line 286 "mbrowrap.cpp"
      var $call=_read($0, $1, $sub2); //@line 286 "mbrowrap.cpp"
      $result=$call; //@line 286 "mbrowrap.cpp"
      var $cmp=((($call))|0)==-1; //@line 288 "mbrowrap.cpp"
      if ($cmp) { __label__ = 2;break $for_cond$2; } //@line 288 "mbrowrap.cpp"
  
      var $5=$result; //@line 295 "mbrowrap.cpp"
      var $cmp9=((($5))|0)==0; //@line 295 "mbrowrap.cpp"
      if ($cmp9) { __label__ = 6;break $for_cond$2; } //@line 295 "mbrowrap.cpp"
  
      var $6=$result; //@line 300 "mbrowrap.cpp"
      var $7=$buf_ptr; //@line 300 "mbrowrap.cpp"
      var $arrayidx=(($7+$6)&4294967295); //@line 300 "mbrowrap.cpp"
      HEAP8[($arrayidx)]=0; //@line 300 "mbrowrap.cpp"
      ; //@line 302 "mbrowrap.cpp"
      $for_cond13$6: while(1) { 
  
        var $8=$buf_ptr; //@line 302 "mbrowrap.cpp"
        var $call14=_strchr($8, 10); //@line 302 "mbrowrap.cpp"
        $lf=$call14; //@line 302 "mbrowrap.cpp"
        var $tobool=((($call14))|0)!=0; //@line 302 "mbrowrap.cpp"
        var $9=$buf_ptr; //@line 304 "mbrowrap.cpp"
        if (!($tobool)) { __label__ = 14;break $for_cond13$6; } //@line 302 "mbrowrap.cpp"
  
        var $call15=_strncmp($9, ((__str8588)&4294967295), 18); //@line 304 "mbrowrap.cpp"
        var $cmp16=((($call15))|0)==0; //@line 304 "mbrowrap.cpp"
        if ($cmp16) { __label__ = 13;; } else { __label__ = 10;; } //@line 304 "mbrowrap.cpp"
        $for_inc$$lor_lhs_false$9: do { 
          if (__label__ == 10) {
  
            var $10=$buf_ptr; //@line 305 "mbrowrap.cpp"
            var $call17=_strncmp($10, ((__str9589)&4294967295), 18); //@line 305 "mbrowrap.cpp"
            var $cmp18=((($call17))|0)==0; //@line 305 "mbrowrap.cpp"
            if ($cmp18) { __label__ = 13;break $for_inc$$lor_lhs_false$9; } //@line 305 "mbrowrap.cpp"
  
            var $11=$lf; //@line 307 "mbrowrap.cpp"
            HEAP8[($11)]=0; //@line 307 "mbrowrap.cpp"
            var $12=$buf_ptr; //@line 308 "mbrowrap.cpp"
            __ZL3logPKcz(((__str10590)&4294967295), allocate([$12,0,0,0], ["i8*",0,0,0], ALLOC_STACK)); //@line 308 "mbrowrap.cpp"
            var $13=$lf; //@line 310 "mbrowrap.cpp"
            var $14=$result; //@line 310 "mbrowrap.cpp"
            var $sub21=((($14)-1)&4294967295); //@line 310 "mbrowrap.cpp"
            var $15=$buf_ptr; //@line 310 "mbrowrap.cpp"
            var $arrayidx22=(($15+$sub21)&4294967295); //@line 310 "mbrowrap.cpp"
            var $cmp23=((($13))|0)==((($arrayidx22))|0); //@line 310 "mbrowrap.cpp"
            if ($cmp23) { __label__ = 12;break $for_cond$2; } //@line 310 "mbrowrap.cpp"
          }
        } while(0);
  
        var $17=$lf; //@line 302 "mbrowrap.cpp"
        var $add_ptr=(($17+1)&4294967295); //@line 302 "mbrowrap.cpp"
        $buf_ptr=$add_ptr; //@line 302 "mbrowrap.cpp"
        __label__ = 8;continue $for_cond13$6; //@line 302 "mbrowrap.cpp"
      }
  
      var $18=$result; //@line 318 "mbrowrap.cpp"
      _llvm_memmove_p0i8_p0i8_i32($arraydecay27, $9, $18, 1, 0); //@line 318 "mbrowrap.cpp"
      var $19=$result; //@line 319 "mbrowrap.cpp"
      var $add_ptr29=(($arraydecay28+$19)&4294967295); //@line 319 "mbrowrap.cpp"
      $buf_ptr=$add_ptr29; //@line 319 "mbrowrap.cpp"
      __label__ = 1;continue $for_cond$2; //@line 320 "mbrowrap.cpp"
    }
    if (__label__ == 2) {
  
      var $call3=___errno(); //@line 289 "mbrowrap.cpp"
      var $3=HEAP32[(($call3)>>2)]; //@line 289 "mbrowrap.cpp"
      var $cmp4=((($3))|0)==11; //@line 289 "mbrowrap.cpp"
      if ($cmp4) { __label__ = 3;; } else { __label__ = 4;; } //@line 289 "mbrowrap.cpp"
      if (__label__ == 3) {
  
        $retval=0; //@line 290 "mbrowrap.cpp"
        ; //@line 290 "mbrowrap.cpp"
      }
      else if (__label__ == 4) {
  
        var $call6=___errno(); //@line 291 "mbrowrap.cpp"
        var $4=HEAP32[(($call6)>>2)]; //@line 291 "mbrowrap.cpp"
        var $call7=_strerror($4); //@line 291 "mbrowrap.cpp"
        __ZL3errPKcz(((__str7587)&4294967295), allocate([$call7,0,0,0], ["i8*",0,0,0], ALLOC_STACK)); //@line 291 "mbrowrap.cpp"
        $retval=-1; //@line 292 "mbrowrap.cpp"
        ; //@line 292 "mbrowrap.cpp"
      }
    }
    else if (__label__ == 6) {
  
      __ZL11mbrola_diedv(); //@line 297 "mbrowrap.cpp"
      $retval=-1; //@line 297 "mbrowrap.cpp"
      ; //@line 297 "mbrowrap.cpp"
    }
    else if (__label__ == 12) {
  
      var $16=$buf_ptr; //@line 311 "mbrowrap.cpp"
      var $call25=_snprintf(((__ZL12mbr_errorbuf)&4294967295), 160, ((__str57709)&4294967295), allocate([$16,0,0,0], ["i8*",0,0,0], ALLOC_STACK)); //@line 311 "mbrowrap.cpp"
      $retval=0; //@line 314 "mbrowrap.cpp"
      ; //@line 314 "mbrowrap.cpp"
    }
  
    var $20=$retval; //@line 321 "mbrowrap.cpp"
    STACKTOP = __stackBase__;
    return $20; //@line 321 "mbrowrap.cpp"
    return null;
  }
  

  function _read_MBR($buffer, $nb_samples) {
    ;
    var __label__;
  
    var $buffer_addr;
    var $nb_samples_addr;
    var $result;
    $buffer_addr=$buffer;
    $nb_samples_addr=$nb_samples;
    var $0=$buffer_addr; //@line 559 "mbrowrap.cpp"
    var $1=$nb_samples_addr; //@line 559 "mbrowrap.cpp"
    var $mul=((($1)*2)&4294967295); //@line 559 "mbrowrap.cpp"
    var $call=__ZL19receive_from_mbrolaPvj($0, $mul); //@line 559 "mbrowrap.cpp"
    $result=$call; //@line 559 "mbrowrap.cpp"
    var $2=$result; //@line 560 "mbrowrap.cpp"
    var $cmp=((($2))|0) > 0; //@line 560 "mbrowrap.cpp"
    if ($cmp) { __label__ = 1;; } else { __label__ = 2;; } //@line 560 "mbrowrap.cpp"
    if (__label__ == 1) {
  
      var $3=$result; //@line 561 "mbrowrap.cpp"
      var $div=((((($3))|0)/2)|0); //@line 561 "mbrowrap.cpp"
      $result=$div; //@line 561 "mbrowrap.cpp"
      ; //@line 561 "mbrowrap.cpp"
    }
  
    var $4=$result; //@line 562 "mbrowrap.cpp"
    ;
    return $4; //@line 562 "mbrowrap.cpp"
    return null;
  }
  

  function _write_MBR($data) {
    ;
    var __label__;
  
    var $data_addr;
    $data_addr=$data;
    HEAP32[((__ZL9mbr_state)>>2)]=2; //@line 567 "mbrowrap.cpp"
    var $0=$data_addr; //@line 568 "mbrowrap.cpp"
    var $call=__ZL14send_to_mbrolaPKc($0); //@line 568 "mbrowrap.cpp"
    ;
    return $call; //@line 568 "mbrowrap.cpp"
    return null;
  }
  

  function _flush_MBR() {
    ;
    var __label__;
  
    var $call=__ZL14send_to_mbrolaPKc(((__str5585)&4294967295)); //@line 573 "mbrowrap.cpp"
    var $cmp=((($call))|0)==3; //@line 573 "mbrowrap.cpp"
    var $conv=((($cmp))&1); //@line 573 "mbrowrap.cpp"
    ;
    return $conv; //@line 573 "mbrowrap.cpp"
    return null;
  }
  

  function _getFreq_MBR() {
    ;
    var __label__;
  
    var $0=HEAP32[((__ZL14mbr_samplerate)>>2)]; //@line 578 "mbrowrap.cpp"
    ;
    return $0; //@line 578 "mbrowrap.cpp"
    return null;
  }
  

  function _setVolumeRatio_MBR($value) {
    ;
    var __label__;
  
    var $value_addr;
    $value_addr=$value;
    var $0=$value_addr; //@line 583 "mbrowrap.cpp"
    var $1=HEAPF32[((__ZL10mbr_volume)>>2)]; //@line 583 "mbrowrap.cpp"
    var $cmp=($0) == ($1); //@line 583 "mbrowrap.cpp"
    if ($cmp) { __label__ = 3;; } else { __label__ = 1;; } //@line 583 "mbrowrap.cpp"
    $return$$if_end$2: do { 
      if (__label__ == 1) {
  
        var $2=$value_addr; //@line 585 "mbrowrap.cpp"
        HEAPF32[((__ZL10mbr_volume)>>2)]=$2; //@line 585 "mbrowrap.cpp"
        var $3=HEAP32[((__ZL9mbr_state)>>2)]; //@line 586 "mbrowrap.cpp"
        var $cmp1=((($3))|0)!=1; //@line 586 "mbrowrap.cpp"
        if ($cmp1) { __label__ = 3;break $return$$if_end$2; } //@line 586 "mbrowrap.cpp"
  
        __ZL11stop_mbrolav(); //@line 592 "mbrowrap.cpp"
        var $4=HEAP32[((__ZL14mbr_voice_path)>>2)]; //@line 593 "mbrowrap.cpp"
        var $call=_init_MBR($4); //@line 593 "mbrowrap.cpp"
        ; //@line 594 "mbrowrap.cpp"
      }
    } while(0);
  
    ;
    return; //@line 594 "mbrowrap.cpp"
    return;
  }
  

  function _lastErrorStr_MBR($buffer, $bufsize) {
    ;
    var __label__;
    var __lastLabel__ = null;
  
    var $buffer_addr;
    var $bufsize_addr;
    var $result;
    $buffer_addr=$buffer;
    $bufsize_addr=$bufsize;
    var $0=HEAP32[((__ZL7mbr_pid)>>2)]; //@line 599 "mbrowrap.cpp"
    var $tobool=((($0))|0)!=0; //@line 599 "mbrowrap.cpp"
    if ($tobool) { __label__ = 1;; } else { __label__ = 2;; } //@line 599 "mbrowrap.cpp"
    if (__label__ == 1) {
  
      var $call=__ZL17mbrola_has_errorsv(); //@line 600 "mbrowrap.cpp"
      ; //@line 600 "mbrowrap.cpp"
    }
  
    var $1=$buffer_addr; //@line 601 "mbrowrap.cpp"
    var $2=$bufsize_addr; //@line 601 "mbrowrap.cpp"
    var $call1=_snprintf($1, $2, ((__str57709)&4294967295), allocate([((__ZL12mbr_errorbuf)&4294967295),0,0,0], ["i8*",0,0,0], ALLOC_STACK)); //@line 601 "mbrowrap.cpp"
    $result=$call1; //@line 601 "mbrowrap.cpp"
    var $3=$result; //@line 602 "mbrowrap.cpp"
    var $4=$bufsize_addr; //@line 602 "mbrowrap.cpp"
    var $cmp=((($3))|0) >= ((($4))|0); //@line 602 "mbrowrap.cpp"
    if ($cmp) { __label__ = 3;; } else { __label__ = 4;; } //@line 602 "mbrowrap.cpp"
    if (__label__ == 3) {
  
      var $5=$bufsize_addr; //@line 602 "mbrowrap.cpp"
      var $sub=((($5)-1)&4294967295); //@line 602 "mbrowrap.cpp"
      __lastLabel__ = 3; ; //@line 602 "mbrowrap.cpp"
    }
    else if (__label__ == 4) {
  
      var $6=$result; //@line 602 "mbrowrap.cpp"
      __lastLabel__ = 4; ; //@line 602 "mbrowrap.cpp"
    }
  
    var $cond=__lastLabel__ == 3 ? $sub : ($6); //@line 602 "mbrowrap.cpp"
    ;
    return $cond; //@line 602 "mbrowrap.cpp"
    return null;
  }
  

  function _resetError_MBR() {
    ;
    var __label__;
  
    HEAP8[(((__ZL12mbr_errorbuf)&4294967295))]=0; //@line 607 "mbrowrap.cpp"
    ;
    return; //@line 608 "mbrowrap.cpp"
    return;
  }
  

  function __ZL11mbrola_diedv() {
    var __stackBase__  = STACKTOP; STACKTOP += 84; _memset(__stackBase__, 0, 84);
    var __label__;
  
    var $pid;
    var $status=__stackBase__;
    var $len;
    var $msg;
    var $msgbuf=__stackBase__+4;
    var $sig;
    var $exst;
    var $0=HEAP32[((__ZL7mbr_pid)>>2)]; //@line 245 "mbrowrap.cpp"
    var $call=_waitpid($0, $status, 1); //@line 245 "mbrowrap.cpp"
    $pid=$call; //@line 245 "mbrowrap.cpp"
    var $1=$pid; //@line 246 "mbrowrap.cpp"
    var $tobool=((($1))|0)!=0; //@line 246 "mbrowrap.cpp"
    if ($tobool) { __label__ = 2;; } else { __label__ = 1;; } //@line 246 "mbrowrap.cpp"
    $if_else$$if_then$2: do { 
      if (__label__ == 2) {
  
        var $2=$pid; //@line 248 "mbrowrap.cpp"
        var $3=HEAP32[((__ZL7mbr_pid)>>2)]; //@line 248 "mbrowrap.cpp"
        var $cmp=((($2))|0)!=((($3))|0); //@line 248 "mbrowrap.cpp"
        if ($cmp) { __label__ = 3;; } else { __label__ = 4;; } //@line 248 "mbrowrap.cpp"
        if (__label__ == 3) {
  
          $msg=((__str12592)&4294967295); //@line 249 "mbrowrap.cpp"
          ; //@line 250 "mbrowrap.cpp"
        }
        else if (__label__ == 4) {
  
          HEAP32[((__ZL7mbr_pid)>>2)]=0; //@line 251 "mbrowrap.cpp"
          var $4=HEAP32[(($status)>>2)]; //@line 252 "mbrowrap.cpp"
          var $and=($4) & 127; //@line 252 "mbrowrap.cpp"
          var $cmp3=((($and))|0) > 0; //@line 252 "mbrowrap.cpp"
          if ($cmp3) { __label__ = 5;; } else { __label__ = 7;; } //@line 252 "mbrowrap.cpp"
          $land_lhs_true$$if_else10$7: do { 
            if (__label__ == 5) {
  
              var $5=HEAP32[(($status)>>2)]; //@line 252 "mbrowrap.cpp"
              var $and4=($5) & 127; //@line 252 "mbrowrap.cpp"
              var $cmp5=((($and4))|0) < 127; //@line 252 "mbrowrap.cpp"
              if (!($cmp5)) { __label__ = 7;break $land_lhs_true$$if_else10$7; } //@line 252 "mbrowrap.cpp"
  
              var $6=HEAP32[(($status)>>2)]; //@line 253 "mbrowrap.cpp"
              var $and7=($6) & 127; //@line 253 "mbrowrap.cpp"
              $sig=$and7; //@line 253 "mbrowrap.cpp"
              var $arraydecay=(($msgbuf)&4294967295); //@line 254 "mbrowrap.cpp"
              var $7=$sig; //@line 254 "mbrowrap.cpp"
              var $call8=_snprintf($arraydecay, 80, ((__str13593)&4294967295), allocate([$7,0,0,0], ["i32",0,0,0], ALLOC_STACK)); //@line 254 "mbrowrap.cpp"
              var $arraydecay9=(($msgbuf)&4294967295); //@line 256 "mbrowrap.cpp"
              $msg=$arraydecay9; //@line 256 "mbrowrap.cpp"
              __label__ = 10;break $if_else$$if_then$2; //@line 257 "mbrowrap.cpp"
            }
          } while(0);
  
          var $8=HEAP32[(($status)>>2)]; //@line 257 "mbrowrap.cpp"
          var $and11=($8) & 255; //@line 257 "mbrowrap.cpp"
          var $cmp12=((($and11))|0)==0; //@line 257 "mbrowrap.cpp"
          if ($cmp12) { __label__ = 8;; } else { __label__ = 9;; } //@line 257 "mbrowrap.cpp"
          if (__label__ == 8) {
  
            var $9=HEAP32[(($status)>>2)]; //@line 258 "mbrowrap.cpp"
            var $shr=($9) >> 8; //@line 258 "mbrowrap.cpp"
            var $and14=($shr) & 255; //@line 258 "mbrowrap.cpp"
            $exst=$and14; //@line 258 "mbrowrap.cpp"
            var $arraydecay15=(($msgbuf)&4294967295); //@line 259 "mbrowrap.cpp"
            var $10=$exst; //@line 259 "mbrowrap.cpp"
            var $call16=_snprintf($arraydecay15, 80, ((__str14594)&4294967295), allocate([$10,0,0,0], ["i32",0,0,0], ALLOC_STACK)); //@line 259 "mbrowrap.cpp"
            var $arraydecay17=(($msgbuf)&4294967295); //@line 261 "mbrowrap.cpp"
            $msg=$arraydecay17; //@line 261 "mbrowrap.cpp"
            ; //@line 262 "mbrowrap.cpp"
          }
          else if (__label__ == 9) {
  
            $msg=((__str15595)&4294967295); //@line 263 "mbrowrap.cpp"
            ;
          }
        }
      }
      else if (__label__ == 1) {
  
        $msg=((__str11591)&4294967295); //@line 247 "mbrowrap.cpp"
        ; //@line 248 "mbrowrap.cpp"
      }
    } while(0);
  
    var $11=$msg; //@line 267 "mbrowrap.cpp"
    __ZL3logPKcz(((__str16596)&4294967295), allocate([$11,0,0,0], ["i8*",0,0,0], ALLOC_STACK)); //@line 267 "mbrowrap.cpp"
    var $call22=_strlen(((__ZL12mbr_errorbuf)&4294967295)); //@line 269 "mbrowrap.cpp"
    $len=$call22; //@line 269 "mbrowrap.cpp"
    var $tobool23=((($call22))|0)!=0; //@line 270 "mbrowrap.cpp"
    if ($tobool23) { __label__ = 12;; } else { __label__ = 11;; } //@line 270 "mbrowrap.cpp"
    if (__label__ == 12) {
  
      var $13=$len; //@line 273 "mbrowrap.cpp"
      var $add_ptr=((((__ZL12mbr_errorbuf)&4294967295)+$13)&4294967295); //@line 273 "mbrowrap.cpp"
      var $14=$len; //@line 273 "mbrowrap.cpp"
      var $sub=((160-($14))&4294967295); //@line 273 "mbrowrap.cpp"
      var $15=$msg; //@line 273 "mbrowrap.cpp"
      var $call27=_snprintf($add_ptr, $sub, ((__str17597)&4294967295), allocate([$15,0,0,0], ["i8*",0,0,0], ALLOC_STACK)); //@line 273 "mbrowrap.cpp"
      ;
    }
    else if (__label__ == 11) {
  
      var $12=$msg; //@line 271 "mbrowrap.cpp"
      var $call25=_snprintf(((__ZL12mbr_errorbuf)&4294967295), 160, ((__str57709)&4294967295), allocate([$12,0,0,0], ["i8*",0,0,0], ALLOC_STACK)); //@line 271 "mbrowrap.cpp"
      ; //@line 271 "mbrowrap.cpp"
    }
  
    STACKTOP = __stackBase__;
    return;
    return;
  }
  

  function __ZL3logPKcz($msg) {
    var __stackBase__  = STACKTOP; STACKTOP += 4; _memset(__stackBase__, 0, 4);
    var __label__;
  
    var $msg_addr;
    var $params=__stackBase__;
    $msg_addr=$msg;
    var $params1=$params; //@line 74 "mbrowrap.cpp"
    HEAP32[(($params1)>>2)]=arguments[__ZL3logPKcz.length];; //@line 74 "mbrowrap.cpp"
    var $0=HEAP32[((__impure_ptr)>>2)]; //@line 75 "mbrowrap.cpp"
    var $_stderr=(($0+12)&4294967295); //@line 75 "mbrowrap.cpp"
    var $1=HEAP32[(($_stderr)>>2)]; //@line 75 "mbrowrap.cpp"
    var $2=$msg_addr; //@line 75 "mbrowrap.cpp"
    var $3=HEAP32[(($params)>>2)]; //@line 75 "mbrowrap.cpp"
    var $call=_vfprintf($1, $2, $3); //@line 75 "mbrowrap.cpp"
    var $4=HEAP32[((__impure_ptr)>>2)]; //@line 76 "mbrowrap.cpp"
    var $_stderr2=(($4+12)&4294967295); //@line 76 "mbrowrap.cpp"
    var $5=HEAP32[(($_stderr2)>>2)]; //@line 76 "mbrowrap.cpp"
    var $call3=_fputc(10, $5); //@line 76 "mbrowrap.cpp"
    var $params4=$params; //@line 77 "mbrowrap.cpp"
    ; //@line 77 "mbrowrap.cpp"
    STACKTOP = __stackBase__;
    return; //@line 78 "mbrowrap.cpp"
    return;
  }
  

  function __ZL14mbrola_is_idlev() {
    var __stackBase__  = STACKTOP; STACKTOP += 20; _memset(__stackBase__, 0, 20);
    var __label__;
    var __lastLabel__ = null;
  
    var $retval;
    var $p;
    var $buffer=__stackBase__;
    var $0=HEAP32[((__ZL13mbr_proc_stat)>>2)]; //@line 372 "mbrowrap.cpp"
    var $call=_lseek($0, 0, 0); //@line 372 "mbrowrap.cpp"
    var $cmp=((($call))|0)!=0; //@line 372 "mbrowrap.cpp"
    if ($cmp) { __label__ = 1;; } else { __label__ = 2;; } //@line 372 "mbrowrap.cpp"
    $if_then$$if_end$2: do { 
      if (__label__ == 1) {
  
        $retval=0; //@line 373 "mbrowrap.cpp"
        ; //@line 373 "mbrowrap.cpp"
      }
      else if (__label__ == 2) {
  
        var $1=HEAP32[((__ZL13mbr_proc_stat)>>2)]; //@line 374 "mbrowrap.cpp"
        var $arraydecay=(($buffer)&4294967295); //@line 374 "mbrowrap.cpp"
        var $call1=_read($1, $arraydecay, 20); //@line 374 "mbrowrap.cpp"
        var $cmp2=((($call1))|0)!=20; //@line 374 "mbrowrap.cpp"
        if ($cmp2) { __label__ = 3;; } else { __label__ = 4;; } //@line 374 "mbrowrap.cpp"
        if (__label__ == 3) {
  
          $retval=0; //@line 375 "mbrowrap.cpp"
          ; //@line 375 "mbrowrap.cpp"
        }
        else if (__label__ == 4) {
  
          var $arraydecay5=(($buffer)&4294967295); //@line 376 "mbrowrap.cpp"
          var $call6=_memchr($arraydecay5, 41, 20); //@line 376 "mbrowrap.cpp"
          $p=$call6; //@line 376 "mbrowrap.cpp"
          var $2=$p; //@line 377 "mbrowrap.cpp"
          var $tobool=((($2))|0)!=0; //@line 377 "mbrowrap.cpp"
          if ($tobool) { __label__ = 5;; } else { __label__ = 6;; } //@line 377 "mbrowrap.cpp"
          $lor_lhs_false$$if_then9$8: do { 
            if (__label__ == 5) {
  
              var $3=$p; //@line 377 "mbrowrap.cpp"
              var $arraydecay7=(($buffer)&4294967295); //@line 377 "mbrowrap.cpp"
              var $sub_ptr_lhs_cast=($3); //@line 377 "mbrowrap.cpp"
              var $sub_ptr_rhs_cast=($arraydecay7); //@line 377 "mbrowrap.cpp"
              var $sub_ptr_sub=((($sub_ptr_lhs_cast)-($sub_ptr_rhs_cast))&4294967295); //@line 377 "mbrowrap.cpp"
              var $cmp8=((($sub_ptr_sub))>>>0) >= 18; //@line 377 "mbrowrap.cpp"
              if ($cmp8) { __label__ = 6;break $lor_lhs_false$$if_then9$8; } //@line 377 "mbrowrap.cpp"
  
              var $4=$p; //@line 379 "mbrowrap.cpp"
              var $arrayidx=(($4+1)&4294967295); //@line 379 "mbrowrap.cpp"
              var $5=HEAP8[($arrayidx)]; //@line 379 "mbrowrap.cpp"
              var $conv=(tempInt=(($5)),(tempInt>=128?tempInt-256:tempInt)); //@line 379 "mbrowrap.cpp"
              var $cmp11=((($conv))|0)==32; //@line 379 "mbrowrap.cpp"
              if ($cmp11) { __lastLabel__ = 7; __label__ = 8;; } else { __lastLabel__ = 7; __label__ = 9;; } //@line 379 "mbrowrap.cpp"
              if (__label__ == 8) {
  
                var $6=$p; //@line 379 "mbrowrap.cpp"
                var $arrayidx12=(($6+2)&4294967295); //@line 379 "mbrowrap.cpp"
                var $7=HEAP8[($arrayidx12)]; //@line 379 "mbrowrap.cpp"
                var $conv13=(tempInt=(($7)),(tempInt>=128?tempInt-256:tempInt)); //@line 379 "mbrowrap.cpp"
                var $cmp14=((($conv13))|0)==83; //@line 379 "mbrowrap.cpp"
                __lastLabel__ = 8; ;
              }
  
              var $8=__lastLabel__ == 7 ? 0 : ($cmp14);
              var $conv15=((($8))&1);
              $retval=$conv15;
              __label__ = 10;break $if_then$$if_end$2;
            }
          } while(0);
  
          $retval=0; //@line 378 "mbrowrap.cpp"
          ; //@line 378 "mbrowrap.cpp"
        }
      }
    } while(0);
  
    var $9=$retval; //@line 380 "mbrowrap.cpp"
    STACKTOP = __stackBase__;
    return $9; //@line 380 "mbrowrap.cpp"
    return null;
  }
  

  function __ZL12create_pipesPiS_S_($p1, $p2, $p3) {
    ;
    var __label__;
  
    var $retval;
    var $p1_addr;
    var $p2_addr;
    var $p3_addr;
    var $error;
    $p1_addr=$p1;
    $p2_addr=$p2;
    $p3_addr=$p3;
    var $0=$p1_addr; //@line 94 "mbrowrap.cpp"
    var $call=_pipe($0); //@line 94 "mbrowrap.cpp"
    var $cmp=((($call))|0)!=-1; //@line 94 "mbrowrap.cpp"
    if ($cmp) { __label__ = 1;; } else { __label__ = 7;; } //@line 94 "mbrowrap.cpp"
    $if_then$$if_else18$2: do { 
      if (__label__ == 1) {
  
        var $1=$p2_addr; //@line 95 "mbrowrap.cpp"
        var $call1=_pipe($1); //@line 95 "mbrowrap.cpp"
        var $cmp2=((($call1))|0)!=-1; //@line 95 "mbrowrap.cpp"
        if ($cmp2) { __label__ = 2;; } else { __label__ = 5;; } //@line 95 "mbrowrap.cpp"
        if (__label__ == 2) {
  
          var $2=$p3_addr; //@line 96 "mbrowrap.cpp"
          var $call4=_pipe($2); //@line 96 "mbrowrap.cpp"
          var $cmp5=((($call4))|0)!=-1; //@line 96 "mbrowrap.cpp"
          if ($cmp5) { __label__ = 3;; } else { __label__ = 4;; } //@line 96 "mbrowrap.cpp"
          if (__label__ == 3) {
  
            $retval=0; //@line 97 "mbrowrap.cpp"
            __label__ = 9;break $if_then$$if_else18$2; //@line 97 "mbrowrap.cpp"
          }
          else if (__label__ == 4) {
  
            var $call7=___errno(); //@line 99 "mbrowrap.cpp"
            var $3=HEAP32[(($call7)>>2)]; //@line 99 "mbrowrap.cpp"
            $error=$3; //@line 99 "mbrowrap.cpp"
            var $4=$p2_addr; //@line 100 "mbrowrap.cpp"
            var $arrayidx=(($4)&4294967295); //@line 100 "mbrowrap.cpp"
            var $5=HEAP32[(($arrayidx)>>2)]; //@line 100 "mbrowrap.cpp"
            var $call8=_close($5); //@line 100 "mbrowrap.cpp"
            var $6=$p2_addr; //@line 101 "mbrowrap.cpp"
            var $arrayidx9=(($6+4)&4294967295); //@line 101 "mbrowrap.cpp"
            var $7=HEAP32[(($arrayidx9)>>2)]; //@line 101 "mbrowrap.cpp"
            var $call10=_close($7); //@line 101 "mbrowrap.cpp"
            ; //@line 102 "mbrowrap.cpp"
          }
        }
        else if (__label__ == 5) {
  
          var $call12=___errno(); //@line 103 "mbrowrap.cpp"
          var $8=HEAP32[(($call12)>>2)]; //@line 103 "mbrowrap.cpp"
          $error=$8; //@line 103 "mbrowrap.cpp"
          ;
        }
  
        var $9=$p1_addr; //@line 104 "mbrowrap.cpp"
        var $arrayidx14=(($9)&4294967295); //@line 104 "mbrowrap.cpp"
        var $10=HEAP32[(($arrayidx14)>>2)]; //@line 104 "mbrowrap.cpp"
        var $call15=_close($10); //@line 104 "mbrowrap.cpp"
        var $11=$p1_addr; //@line 105 "mbrowrap.cpp"
        var $arrayidx16=(($11+4)&4294967295); //@line 105 "mbrowrap.cpp"
        var $12=HEAP32[(($arrayidx16)>>2)]; //@line 105 "mbrowrap.cpp"
        var $call17=_close($12); //@line 105 "mbrowrap.cpp"
        __label__ = 8;break $if_then$$if_else18$2; //@line 106 "mbrowrap.cpp"
      }
      else if (__label__ == 7) {
  
        var $call19=___errno(); //@line 107 "mbrowrap.cpp"
        var $13=HEAP32[(($call19)>>2)]; //@line 107 "mbrowrap.cpp"
        $error=$13; //@line 107 "mbrowrap.cpp"
        __label__ = 8;break $if_then$$if_else18$2;
      }
    } while(0);
    if (__label__ == 8) {
  
      var $14=$error; //@line 109 "mbrowrap.cpp"
      var $call21=_strerror($14); //@line 109 "mbrowrap.cpp"
      __ZL3errPKcz(((__str35615)&4294967295), allocate([$call21,0,0,0], ["i8*",0,0,0], ALLOC_STACK)); //@line 109 "mbrowrap.cpp"
      $retval=-1; //@line 110 "mbrowrap.cpp"
      ; //@line 110 "mbrowrap.cpp"
    }
  
    var $15=$retval; //@line 111 "mbrowrap.cpp"
    ;
    return $15; //@line 111 "mbrowrap.cpp"
    return null;
  }
  

  function __ZL11close_pipesPiS_S_($p1, $p2, $p3) {
    ;
    var __label__;
  
    var $p1_addr;
    var $p2_addr;
    var $p3_addr;
    $p1_addr=$p1;
    $p2_addr=$p2;
    $p3_addr=$p3;
    var $0=$p1_addr; //@line 115 "mbrowrap.cpp"
    var $arrayidx=(($0)&4294967295); //@line 115 "mbrowrap.cpp"
    var $1=HEAP32[(($arrayidx)>>2)]; //@line 115 "mbrowrap.cpp"
    var $call=_close($1); //@line 115 "mbrowrap.cpp"
    var $2=$p1_addr; //@line 116 "mbrowrap.cpp"
    var $arrayidx1=(($2+4)&4294967295); //@line 116 "mbrowrap.cpp"
    var $3=HEAP32[(($arrayidx1)>>2)]; //@line 116 "mbrowrap.cpp"
    var $call2=_close($3); //@line 116 "mbrowrap.cpp"
    var $4=$p2_addr; //@line 117 "mbrowrap.cpp"
    var $arrayidx3=(($4)&4294967295); //@line 117 "mbrowrap.cpp"
    var $5=HEAP32[(($arrayidx3)>>2)]; //@line 117 "mbrowrap.cpp"
    var $call4=_close($5); //@line 117 "mbrowrap.cpp"
    var $6=$p2_addr; //@line 118 "mbrowrap.cpp"
    var $arrayidx5=(($6+4)&4294967295); //@line 118 "mbrowrap.cpp"
    var $7=HEAP32[(($arrayidx5)>>2)]; //@line 118 "mbrowrap.cpp"
    var $call6=_close($7); //@line 118 "mbrowrap.cpp"
    var $8=$p3_addr; //@line 119 "mbrowrap.cpp"
    var $arrayidx7=(($8)&4294967295); //@line 119 "mbrowrap.cpp"
    var $9=HEAP32[(($arrayidx7)>>2)]; //@line 119 "mbrowrap.cpp"
    var $call8=_close($9); //@line 119 "mbrowrap.cpp"
    var $10=$p3_addr; //@line 120 "mbrowrap.cpp"
    var $arrayidx9=(($10+4)&4294967295); //@line 120 "mbrowrap.cpp"
    var $11=HEAP32[(($arrayidx9)>>2)]; //@line 120 "mbrowrap.cpp"
    var $call10=_close($11); //@line 120 "mbrowrap.cpp"
    ;
    return; //@line 121 "mbrowrap.cpp"
    return;
  }
  

  function __Z15SetIndicLettersP10Translator($tr) {
    ;
    var __label__;
  
    var $tr_addr;
    $tr_addr=$tr;
    var $0=$tr_addr; //@line 308 "tr_languages.cpp"
    var $letter_bits=(($0+708)&4294967295); //@line 308 "tr_languages.cpp"
    var $arraydecay=(($letter_bits)&4294967295); //@line 308 "tr_languages.cpp"
    var $dest$, $stop$, $stop4$, $fast$, $value4$;
    $dest$ = $arraydecay;
    $stop$ = $dest$ + 256;
    $value4$ = 0;
    if ($value4$ < 0) $value4$ += 256;
    $value4$ = $value4$ + ($value4$<<8) + ($value4$<<16) + ($value4$*16777216);
    while ($dest$%4 !== 0 && $dest$ < $stop$) {
      ;; HEAP8[$dest$++] = 0;
    }
    $dest$ >>= 2;
    $stop4$ = $stop$ >> 2;
    while ($dest$ < $stop4$) {
    ;;;;;;;  HEAP32[$dest$++] = $value4$;
    }
    $dest$ <<= 2;
    while ($dest$ < $stop$) {
      ;; HEAP8[$dest$++] = 0;
    }; //@line 308 "tr_languages.cpp"
    var $1=$tr_addr; //@line 309 "tr_languages.cpp"
    __ZL18SetLetterBitsRangeP10Translatoriii($1, 0, 4, 20); //@line 309 "tr_languages.cpp"
    var $2=$tr_addr; //@line 310 "tr_languages.cpp"
    __ZL18SetLetterBitsRangeP10Translatoriii($2, 0, 62, 77); //@line 310 "tr_languages.cpp"
    var $3=$tr_addr; //@line 311 "tr_languages.cpp"
    __ZL18SetLetterBitsRangeP10Translatoriii($3, 0, 85, 87); //@line 311 "tr_languages.cpp"
    var $4=$tr_addr; //@line 313 "tr_languages.cpp"
    __ZL18SetLetterBitsRangeP10Translatoriii($4, 1, 62, 77); //@line 313 "tr_languages.cpp"
    var $5=$tr_addr; //@line 314 "tr_languages.cpp"
    __ZL18SetLetterBitsRangeP10Translatoriii($5, 1, 85, 87); //@line 314 "tr_languages.cpp"
    var $6=$tr_addr; //@line 316 "tr_languages.cpp"
    __ZL18SetLetterBitsRangeP10Translatoriii($6, 2, 21, 57); //@line 316 "tr_languages.cpp"
    var $7=$tr_addr; //@line 317 "tr_languages.cpp"
    __ZL13SetLetterBitsP10TranslatoriPKc($7, 2, ((__ZZ15SetIndicLettersP10TranslatorE15dev_consonants2)&4294967295)); //@line 317 "tr_languages.cpp"
    var $8=$tr_addr; //@line 319 "tr_languages.cpp"
    __ZL18SetLetterBitsRangeP10Translatoriii($8, 6, 4, 20); //@line 319 "tr_languages.cpp"
    var $9=$tr_addr; //@line 320 "tr_languages.cpp"
    __ZL18SetLetterBitsRangeP10Translatoriii($9, 6, 62, 76); //@line 320 "tr_languages.cpp"
    var $10=$tr_addr; //@line 321 "tr_languages.cpp"
    __ZL18SetLetterBitsRangeP10Translatoriii($10, 6, 85, 87); //@line 321 "tr_languages.cpp"
    var $11=$tr_addr; //@line 323 "tr_languages.cpp"
    var $langopts=(($11)&4294967295); //@line 323 "tr_languages.cpp"
    var $param=(($langopts+24)&4294967295); //@line 323 "tr_languages.cpp"
    var $arrayidx=(($param+20)&4294967295); //@line 323 "tr_languages.cpp"
    HEAP32[(($arrayidx)>>2)]=1; //@line 323 "tr_languages.cpp"
    var $12=$tr_addr; //@line 324 "tr_languages.cpp"
    var $letter_bits_offset=(($12+964)&4294967295); //@line 324 "tr_languages.cpp"
    var $13=HEAP32[(($letter_bits_offset)>>2)]; //@line 324 "tr_languages.cpp"
    var $add=((($13)+77)&4294967295); //@line 324 "tr_languages.cpp"
    var $14=$tr_addr; //@line 324 "tr_languages.cpp"
    var $langopts1=(($14)&4294967295); //@line 324 "tr_languages.cpp"
    var $suffix_add_e=(($langopts1+276)&4294967295); //@line 324 "tr_languages.cpp"
    HEAP32[(($suffix_add_e)>>2)]=$add; //@line 324 "tr_languages.cpp"
    ;
    return; //@line 325 "tr_languages.cpp"
    return;
  }
  

  function __ZL18SetLetterBitsRangeP10Translatoriii($tr, $group, $first, $last) {
    ;
    var __label__;
  
    var $tr_addr;
    var $group_addr;
    var $first_addr;
    var $last_addr;
    var $bits;
    var $ix;
    $tr_addr=$tr;
    $group_addr=$group;
    $first_addr=$first;
    $last_addr=$last;
    var $0=$group_addr; //@line 105 "tr_languages.cpp"
    var $shl=1 << ($0); //@line 105 "tr_languages.cpp"
    $bits=$shl; //@line 105 "tr_languages.cpp"
    var $1=$first_addr; //@line 106 "tr_languages.cpp"
    $ix=$1; //@line 106 "tr_languages.cpp"
    var $2=$ix; //@line 106 "tr_languages.cpp"
    var $3=$last_addr; //@line 106 "tr_languages.cpp"
    var $cmp1=((($2))|0) <= ((($3))|0); //@line 106 "tr_languages.cpp"
    if ($cmp1) { __label__ = 1;; } else { __label__ = 2;; } //@line 106 "tr_languages.cpp"
    $for_body$$for_end$2: do { 
      if (__label__ == 1) {
        while(1) { 
  
          var $4=$bits; //@line 108 "tr_languages.cpp"
          var $5=$ix; //@line 108 "tr_languages.cpp"
          var $6=$tr_addr; //@line 108 "tr_languages.cpp"
          var $letter_bits=(($6+708)&4294967295); //@line 108 "tr_languages.cpp"
          var $arrayidx=(($letter_bits+$5)&4294967295); //@line 108 "tr_languages.cpp"
          var $7=HEAPU8[($arrayidx)]; //@line 108 "tr_languages.cpp"
          var $conv=((($7))&255); //@line 108 "tr_languages.cpp"
          var $or=($conv) | ($4); //@line 108 "tr_languages.cpp"
          var $conv1=((($or)) & 255); //@line 108 "tr_languages.cpp"
          HEAP8[($arrayidx)]=$conv1; //@line 108 "tr_languages.cpp"
          var $8=$ix; //@line 106 "tr_languages.cpp"
          var $inc=((($8)+1)&4294967295); //@line 106 "tr_languages.cpp"
          $ix=$inc; //@line 106 "tr_languages.cpp"
          var $9=$ix; //@line 106 "tr_languages.cpp"
          var $10=$last_addr; //@line 106 "tr_languages.cpp"
          var $cmp=((($9))|0) <= ((($10))|0); //@line 106 "tr_languages.cpp"
          if ($cmp) { __label__ = 1;continue ; } else { __label__ = 2;break $for_body$$for_end$2; } //@line 106 "tr_languages.cpp"
        }
      }
    } while(0);
  
    ;
    return; //@line 110 "tr_languages.cpp"
    return;
  }
  

  function __ZL13SetLetterBitsP10TranslatoriPKc($tr, $group, $string) {
    ;
    var __label__;
  
    var $tr_addr;
    var $group_addr;
    var $string_addr;
    var $bits;
    var $c;
    $tr_addr=$tr;
    $group_addr=$group;
    $string_addr=$string;
    var $0=$group_addr; //@line 95 "tr_languages.cpp"
    var $shl=1 << ($0); //@line 95 "tr_languages.cpp"
    $bits=$shl; //@line 95 "tr_languages.cpp"
    var $1=$string_addr; //@line 96 "tr_languages.cpp"
    var $incdec_ptr1=(($1+1)&4294967295); //@line 96 "tr_languages.cpp"
    $string_addr=$incdec_ptr1; //@line 96 "tr_languages.cpp"
    var $2=HEAPU8[($1)]; //@line 96 "tr_languages.cpp"
    $c=$2; //@line 96 "tr_languages.cpp"
    var $conv3=((($2))&255); //@line 96 "tr_languages.cpp"
    var $cmp4=((($conv3))|0)!=0; //@line 96 "tr_languages.cpp"
    if ($cmp4) { __label__ = 1;; } else { __label__ = 2;; } //@line 96 "tr_languages.cpp"
    $while_body$$while_end$2: do { 
      if (__label__ == 1) {
        while(1) { 
  
          var $3=$bits; //@line 97 "tr_languages.cpp"
          var $4=$c; //@line 97 "tr_languages.cpp"
          var $idxprom=((($4))&255); //@line 97 "tr_languages.cpp"
          var $5=$tr_addr; //@line 97 "tr_languages.cpp"
          var $letter_bits=(($5+708)&4294967295); //@line 97 "tr_languages.cpp"
          var $arrayidx=(($letter_bits+$idxprom)&4294967295); //@line 97 "tr_languages.cpp"
          var $6=HEAPU8[($arrayidx)]; //@line 97 "tr_languages.cpp"
          var $conv1=((($6))&255); //@line 97 "tr_languages.cpp"
          var $or=($conv1) | ($3); //@line 97 "tr_languages.cpp"
          var $conv2=((($or)) & 255); //@line 97 "tr_languages.cpp"
          HEAP8[($arrayidx)]=$conv2; //@line 97 "tr_languages.cpp"
          var $7=$string_addr; //@line 96 "tr_languages.cpp"
          var $incdec_ptr=(($7+1)&4294967295); //@line 96 "tr_languages.cpp"
          $string_addr=$incdec_ptr; //@line 96 "tr_languages.cpp"
          var $8=HEAPU8[($7)]; //@line 96 "tr_languages.cpp"
          $c=$8; //@line 96 "tr_languages.cpp"
          var $conv=((($8))&255); //@line 96 "tr_languages.cpp"
          var $cmp=((($conv))|0)!=0; //@line 96 "tr_languages.cpp"
          if ($cmp) { __label__ = 1;continue ; } else { __label__ = 2;break $while_body$$while_end$2; } //@line 96 "tr_languages.cpp"
        }
      }
    } while(0);
  
    ;
    return; //@line 98 "tr_languages.cpp"
    return;
  }
  

  function __Z15SetupTranslatorP10TranslatorPKsPKh($tr, $lengths, $amps) {
    ;
    var __label__;
    var __lastLabel__ = null;
  
    var $tr_addr;
    var $lengths_addr;
    var $amps_addr;
    $tr_addr=$tr;
    $lengths_addr=$lengths;
    $amps_addr=$amps;
    var $0=$lengths_addr; //@line 330 "tr_languages.cpp"
    var $cmp=((($0))|0)!=0; //@line 330 "tr_languages.cpp"
    if ($cmp) { __lastLabel__ = 0; __label__ = 1;; } else { __lastLabel__ = 0; __label__ = 2;; } //@line 330 "tr_languages.cpp"
    if (__label__ == 1) {
  
      var $1=$tr_addr; //@line 331 "tr_languages.cpp"
      var $stress_lengths=(($1+676)&4294967295); //@line 331 "tr_languages.cpp"
      var $arraydecay=(($stress_lengths)&4294967295); //@line 331 "tr_languages.cpp"
      var $2=$arraydecay; //@line 331 "tr_languages.cpp"
      var $3=$lengths_addr; //@line 331 "tr_languages.cpp"
      var $4=$3; //@line 331 "tr_languages.cpp"
      var $src$, $dest$, $stop$, $stop4$, $fast$;
      $src$ = $4;
      $dest$ = $2;
      $stop$ = $src$ + 16;
      if (($dest$%4) == ($src$%4) && 16 > 8) {
        while ($src$%4 !== 0 && $src$ < $stop$) {
          ; HEAP8[$dest$++] = HEAP8[$src$++];
        }
        $src$ >>= 2;
        $dest$ >>= 2;
        $stop4$ = $stop$ >> 2;
        while ($src$ < $stop4$) {
      ;;;    HEAP32[$dest$++] = HEAP32[$src$++];
        }
        $src$ <<= 2;
        $dest$ <<= 2;
      }while ($src$ < $stop$) {
        ; HEAP8[$dest$++] = HEAP8[$src$++];
      }; //@line 331 "tr_languages.cpp"
      var $_pr=$amps_addr; //@line 332 "tr_languages.cpp"
      __lastLabel__ = 1; ; //@line 331 "tr_languages.cpp"
    }
  
    var $5=__lastLabel__ == 1 ? $_pr : ($amps); //@line 332 "tr_languages.cpp"
    var $cmp1=((($5))|0)!=0; //@line 332 "tr_languages.cpp"
    if ($cmp1) { __label__ = 3;; } else { __label__ = 4;; } //@line 332 "tr_languages.cpp"
    if (__label__ == 3) {
  
      var $6=$tr_addr; //@line 333 "tr_languages.cpp"
      var $stress_amps=(($6+660)&4294967295); //@line 333 "tr_languages.cpp"
      var $arraydecay3=(($stress_amps)&4294967295); //@line 333 "tr_languages.cpp"
      var $7=$amps_addr; //@line 333 "tr_languages.cpp"
      var $src$, $dest$, $stop$, $stop4$, $fast$;
      $src$ = $7;
      $dest$ = $arraydecay3;
      $stop$ = $src$ + 8;
      if (($dest$%4) == ($src$%4) && 8 > 8) {
        while ($src$%4 !== 0 && $src$ < $stop$) {
          ; HEAP8[$dest$++] = HEAP8[$src$++];
        }
        $src$ >>= 2;
        $dest$ >>= 2;
        $stop4$ = $stop$ >> 2;
        while ($src$ < $stop4$) {
      ;;;    HEAP32[$dest$++] = HEAP32[$src$++];
        }
        $src$ <<= 2;
        $dest$ <<= 2;
      }while ($src$ < $stop$) {
        ; HEAP8[$dest$++] = HEAP8[$src$++];
      }; //@line 333 "tr_languages.cpp"
      ; //@line 333 "tr_languages.cpp"
    }
  
    ;
    return; //@line 334 "tr_languages.cpp"
    return;
  }
  

  function __Z16SelectTranslatorPKc($name) {
    ;
    var __label__;
    var __lastLabel__ = null;
  
    var $name_addr;
    var $name2;
    var $tr;
    $name_addr=$name;
    $name2=0; //@line 339 "tr_languages.cpp"
    var $0=$name_addr; //@line 352 "tr_languages.cpp"
    var $1=HEAP8[($0)]; //@line 352 "tr_languages.cpp"
    var $conv2=(tempInt=(($1)),(tempInt>=128?tempInt-256:tempInt)); //@line 352 "tr_languages.cpp"
    var $cmp3=((($conv2))|0)!=0; //@line 352 "tr_languages.cpp"
    if ($cmp3) { __label__ = 1;; } else { __label__ = 2;; } //@line 352 "tr_languages.cpp"
    $while_body$$while_end$2: do { 
      if (__label__ == 1) {
        while(1) { 
  
          var $2=$name2; //@line 353 "tr_languages.cpp"
          var $shl=((($2)*256)&4294967295); //@line 353 "tr_languages.cpp"
          var $3=$name_addr; //@line 353 "tr_languages.cpp"
          var $incdec_ptr=(($3+1)&4294967295); //@line 353 "tr_languages.cpp"
          $name_addr=$incdec_ptr; //@line 353 "tr_languages.cpp"
          var $4=HEAP8[($3)]; //@line 353 "tr_languages.cpp"
          var $conv1=(tempInt=(($4)),(tempInt>=128?tempInt-256:tempInt)); //@line 353 "tr_languages.cpp"
          var $add=((($conv1)+($shl))&4294967295); //@line 353 "tr_languages.cpp"
          $name2=$add; //@line 353 "tr_languages.cpp"
          var $5=$name_addr; //@line 352 "tr_languages.cpp"
          var $6=HEAP8[($5)]; //@line 352 "tr_languages.cpp"
          var $conv=(tempInt=(($6)),(tempInt>=128?tempInt-256:tempInt)); //@line 352 "tr_languages.cpp"
          var $cmp=((($conv))|0)!=0; //@line 352 "tr_languages.cpp"
          if ($cmp) { __label__ = 1;continue ; } else { __label__ = 2;break $while_body$$while_end$2; } //@line 352 "tr_languages.cpp"
        }
      }
    } while(0);
  
    var $call=__ZL13NewTranslatorv(); //@line 355 "tr_languages.cpp"
    $tr=$call; //@line 355 "tr_languages.cpp"
    var $7=$name2; //@line 357 "tr_languages.cpp"
    if ($7 == 24934) {
      __label__ = 3;;
    }
    else if ($7 == 24941) {
      __label__ = 4;;
    }
    else if ($7 == 24946) {
      __label__ = 5;;
    }
    else if ($7 == 25191) {
      __label__ = 6;;
    }
    else if ($7 == 25198) {
      __label__ = 7;;
    }
    else if ($7 == 25465) {
      __label__ = 8;;
    }
    else if ($7 == 25697) {
      __label__ = 9;;
    }
    else if ($7 == 25701) {
      __label__ = 10;;
    }
    else if ($7 == 25718) {
      __label__ = 11;;
    }
    else if ($7 == 25966) {
      __label__ = 12;;
    }
    else if ($7 == 25964) {
      __label__ = 13;;
    }
    else if ($7 == 6779491) {
      __label__ = 13;;
    }
    else if ($7 == 25967) {
      __label__ = 15;;
    }
    else if ($7 == 25971) {
      __label__ = 16;;
    }
    else if ($7 == 25441) {
      __label__ = 16;;
    }
    else if ($7 == 7364976) {
      __label__ = 16;;
    }
    else if ($7 == 25973) {
      __label__ = 21;;
    }
    else if ($7 == 26209) {
      __label__ = 22;;
    }
    else if ($7 == 25972) {
      __label__ = 23;;
    }
    else if ($7 == 26217) {
      __label__ = 24;;
    }
    else if ($7 == 26226) {
      __label__ = 25;;
    }
    else if ($7 == 26729) {
      __label__ = 26;;
    }
    else if ($7 == 28261) {
      __label__ = 26;;
    }
    else if ($7 == 28769) {
      __label__ = 26;;
    }
    else if ($7 == 26738) {
      __label__ = 29;;
    }
    else if ($7 == 25203) {
      __label__ = 29;;
    }
    else if ($7 == 29554) {
      __label__ = 29;;
    }
    else if ($7 == 26740) {
      __label__ = 33;;
    }
    else if ($7 == 26741) {
      __label__ = 34;;
    }
    else if ($7 == 26745) {
      __label__ = 35;;
    }
    else if ($7 == 26980) {
      __label__ = 36;;
    }
    else if ($7 == 26995) {
      __label__ = 37;;
    }
    else if ($7 == 26996) {
      __label__ = 38;;
    }
    else if ($7 == 6972015) {
      __label__ = 39;;
    }
    else if ($7 == 27489) {
      __label__ = 40;;
    }
    else if ($7 == 27503) {
      __label__ = 41;;
    }
    else if ($7 == 27509) {
      __label__ = 42;;
    }
    else if ($7 == 27745) {
      __label__ = 43;;
    }
    else if ($7 == 27766) {
      __label__ = 44;;
    }
    else if ($7 == 28011) {
      __label__ = 45;;
    }
    else if ($7 == 28268) {
      __label__ = 46;;
    }
    else if ($7 == 28271) {
      __label__ = 47;;
    }
    else if ($7 == 28525) {
      __label__ = 48;;
    }
    else if ($7 == 28780) {
      __label__ = 49;;
    }
    else if ($7 == 28788) {
      __label__ = 50;;
    }
    else if ($7 == 29295) {
      __label__ = 51;;
    }
    else if ($7 == 29301) {
      __label__ = 52;;
    }
    else if ($7 == 29303) {
      __label__ = 53;;
    }
    else if ($7 == 29547) {
      __label__ = 54;;
    }
    else if ($7 == 25459) {
      __label__ = 54;;
    }
    else if ($7 == 29545) {
      __label__ = 57;;
    }
    else if ($7 == 29548) {
      __label__ = 58;;
    }
    else if ($7 == 29553) {
      __label__ = 59;;
    }
    else if ($7 == 29558) {
      __label__ = 60;;
    }
    else if ($7 == 29559) {
      __label__ = 61;;
    }
    else if ($7 == 29806) {
      __label__ = 61;;
    }
    else if ($7 == 29793) {
      __label__ = 62;;
    }
    else if ($7 == 28012) {
      __label__ = 62;;
    }
    else if ($7 == 27502) {
      __label__ = 62;;
    }
    else if ($7 == 28018) {
      __label__ = 62;;
    }
    else if ($7 == 29797) {
      __label__ = 62;;
    }
    else if ($7 == 29810) {
      __label__ = 73;;
    }
    else if ($7 == 30066) {
      __label__ = 74;;
    }
    else if ($7 == 30313) {
      __label__ = 75;;
    }
    else if ($7 == 30575) {
      __label__ = 76;;
    }
    else if ($7 == 31336) {
      __label__ = 77;;
    }
    else if ($7 == 8022137) {
      __label__ = 77;;
    }
    else {
    __label__ = 79;;
    }
    
    $sw_default$$sw_bb$$sw_bb9$$sw_bb18$$sw_bb24$$sw_bb39$$sw_bb52$$sw_bb64$$sw_bb72$$sw_bb92$$sw_bb109$$sw_bb123$$sw_bb148$$sw_bb161$$sw_bb195$$sw_bb200$$sw_bb207$$sw_bb209$$sw_bb222$$sw_bb238$$sw_bb257$$sw_bb280$$sw_bb287$$sw_bb312$$sw_bb322$$sw_bb331$$sw_bb347$$sw_bb381$$sw_bb391$$sw_bb403$$sw_bb414$$sw_bb422$$sw_bb439$$sw_bb449$$sw_bb461$$sw_bb481$$sw_bb486$$sw_bb491$$sw_bb509$$sw_bb523$$sw_bb533$$sw_bb534$$sw_bb547$$sw_bb576$$sw_bb599$$sw_bb614$$sw_bb623$$sw_bb630$$sw_bb645$$sw_bb692$$sw_bb702$$sw_bb711$$sw_bb729$$sw_bb734$6: do { 
      if (__label__ == 79) {
  
        var $495=$tr; //@line 1330 "tr_languages.cpp"
        var $langopts756=(($495)&4294967295); //@line 1330 "tr_languages.cpp"
        var $param757=(($langopts756+24)&4294967295); //@line 1330 "tr_languages.cpp"
        var $arrayidx758=(($param757+20)&4294967295); //@line 1330 "tr_languages.cpp"
        HEAP32[(($arrayidx758)>>2)]=1; //@line 1330 "tr_languages.cpp"
        __label__ = 80;break $sw_default$$sw_bb$$sw_bb9$$sw_bb18$$sw_bb24$$sw_bb39$$sw_bb52$$sw_bb64$$sw_bb72$$sw_bb92$$sw_bb109$$sw_bb123$$sw_bb148$$sw_bb161$$sw_bb195$$sw_bb200$$sw_bb207$$sw_bb209$$sw_bb222$$sw_bb238$$sw_bb257$$sw_bb280$$sw_bb287$$sw_bb312$$sw_bb322$$sw_bb331$$sw_bb347$$sw_bb381$$sw_bb391$$sw_bb403$$sw_bb414$$sw_bb422$$sw_bb439$$sw_bb449$$sw_bb461$$sw_bb481$$sw_bb486$$sw_bb491$$sw_bb509$$sw_bb523$$sw_bb533$$sw_bb534$$sw_bb547$$sw_bb576$$sw_bb599$$sw_bb614$$sw_bb623$$sw_bb630$$sw_bb645$$sw_bb692$$sw_bb702$$sw_bb711$$sw_bb729$$sw_bb734$6; //@line 1331 "tr_languages.cpp"
      }
      else if (__label__ == 3) {
  
        var $8=$tr; //@line 362 "tr_languages.cpp"
        __Z15SetupTranslatorP10TranslatorPKsPKh($8, ((__ZZ16SelectTranslatorPKcE17stress_lengths_af)&4294967295), 0); //@line 362 "tr_languages.cpp"
        var $9=$tr; //@line 364 "tr_languages.cpp"
        var $langopts=(($9)&4294967295); //@line 364 "tr_languages.cpp"
        var $stress_rule=(($langopts+8)&4294967295); //@line 364 "tr_languages.cpp"
        HEAP32[(($stress_rule)>>2)]=0; //@line 364 "tr_languages.cpp"
        var $10=$tr; //@line 365 "tr_languages.cpp"
        var $langopts2=(($10)&4294967295); //@line 365 "tr_languages.cpp"
        var $vowel_pause=(($langopts2+4)&4294967295); //@line 365 "tr_languages.cpp"
        HEAP32[(($vowel_pause)>>2)]=48; //@line 365 "tr_languages.cpp"
        var $11=$tr; //@line 366 "tr_languages.cpp"
        var $langopts3=(($11)&4294967295); //@line 366 "tr_languages.cpp"
        var $param=(($langopts3+24)&4294967295); //@line 366 "tr_languages.cpp"
        var $arrayidx=(($param+4)&4294967295); //@line 366 "tr_languages.cpp"
        HEAP32[(($arrayidx)>>2)]=1; //@line 366 "tr_languages.cpp"
        var $12=$tr; //@line 367 "tr_languages.cpp"
        var $langopts4=(($12)&4294967295); //@line 367 "tr_languages.cpp"
        var $param5=(($langopts4+24)&4294967295); //@line 367 "tr_languages.cpp"
        var $arrayidx6=(($param5+12)&4294967295); //@line 367 "tr_languages.cpp"
        HEAP32[(($arrayidx6)>>2)]=1; //@line 367 "tr_languages.cpp"
        var $13=$tr; //@line 368 "tr_languages.cpp"
        __ZL14SetLetterVowelP10Translatori($13, 121); //@line 368 "tr_languages.cpp"
        var $14=$tr; //@line 370 "tr_languages.cpp"
        var $langopts7=(($14)&4294967295); //@line 370 "tr_languages.cpp"
        var $numbers=(($langopts7+184)&4294967295); //@line 370 "tr_languages.cpp"
        HEAP32[(($numbers)>>2)]=16779472; //@line 370 "tr_languages.cpp"
        var $15=$tr; //@line 371 "tr_languages.cpp"
        var $langopts8=(($15)&4294967295); //@line 371 "tr_languages.cpp"
        var $accents=(($langopts8+220)&4294967295); //@line 371 "tr_languages.cpp"
        HEAP32[(($accents)>>2)]=1; //@line 371 "tr_languages.cpp"
        __label__ = 80;break $sw_default$$sw_bb$$sw_bb9$$sw_bb18$$sw_bb24$$sw_bb39$$sw_bb52$$sw_bb64$$sw_bb72$$sw_bb92$$sw_bb109$$sw_bb123$$sw_bb148$$sw_bb161$$sw_bb195$$sw_bb200$$sw_bb207$$sw_bb209$$sw_bb222$$sw_bb238$$sw_bb257$$sw_bb280$$sw_bb287$$sw_bb312$$sw_bb322$$sw_bb331$$sw_bb347$$sw_bb381$$sw_bb391$$sw_bb403$$sw_bb414$$sw_bb422$$sw_bb439$$sw_bb449$$sw_bb461$$sw_bb481$$sw_bb486$$sw_bb491$$sw_bb509$$sw_bb523$$sw_bb533$$sw_bb534$$sw_bb547$$sw_bb576$$sw_bb599$$sw_bb614$$sw_bb623$$sw_bb630$$sw_bb645$$sw_bb692$$sw_bb702$$sw_bb711$$sw_bb729$$sw_bb734$6; //@line 373 "tr_languages.cpp"
      }
      else if (__label__ == 4) {
  
        var $16=$tr; //@line 377 "tr_languages.cpp"
        __Z15SetupTranslatorP10TranslatorPKsPKh($16, ((__ZZ16SelectTranslatorPKcE17stress_lengths_fr)&4294967295), ((__ZZ16SelectTranslatorPKcE14stress_amps_fr)&4294967295)); //@line 377 "tr_languages.cpp"
        var $17=$tr; //@line 378 "tr_languages.cpp"
        var $langopts10=(($17)&4294967295); //@line 378 "tr_languages.cpp"
        var $stress_rule11=(($langopts10+8)&4294967295); //@line 378 "tr_languages.cpp"
        HEAP32[(($stress_rule11)>>2)]=0; //@line 378 "tr_languages.cpp"
        var $18=$tr; //@line 379 "tr_languages.cpp"
        var $langopts12=(($18)&4294967295); //@line 379 "tr_languages.cpp"
        var $stress_flags=(($langopts12+12)&4294967295); //@line 379 "tr_languages.cpp"
        HEAP32[(($stress_flags)>>2)]=36; //@line 379 "tr_languages.cpp"
        var $19=$tr; //@line 380 "tr_languages.cpp"
        var $langopts13=(($19)&4294967295); //@line 380 "tr_languages.cpp"
        var $length_mods=(($langopts13+176)&4294967295); //@line 380 "tr_languages.cpp"
        var $20=HEAP32[(($length_mods)>>2)]; //@line 380 "tr_languages.cpp"
        var $21=$tr; //@line 380 "tr_languages.cpp"
        var $langopts14=(($21)&4294967295); //@line 380 "tr_languages.cpp"
        var $length_mods0=(($langopts14+180)&4294967295); //@line 380 "tr_languages.cpp"
        HEAP32[(($length_mods0)>>2)]=$20; //@line 380 "tr_languages.cpp"
        var $22=$tr; //@line 381 "tr_languages.cpp"
        var $langopts15=(($22)&4294967295); //@line 381 "tr_languages.cpp"
        var $param16=(($langopts15+24)&4294967295); //@line 381 "tr_languages.cpp"
        var $arrayidx17=(($param16+20)&4294967295); //@line 381 "tr_languages.cpp"
        HEAP32[(($arrayidx17)>>2)]=1; //@line 381 "tr_languages.cpp"
        __label__ = 80;break $sw_default$$sw_bb$$sw_bb9$$sw_bb18$$sw_bb24$$sw_bb39$$sw_bb52$$sw_bb64$$sw_bb72$$sw_bb92$$sw_bb109$$sw_bb123$$sw_bb148$$sw_bb161$$sw_bb195$$sw_bb200$$sw_bb207$$sw_bb209$$sw_bb222$$sw_bb238$$sw_bb257$$sw_bb280$$sw_bb287$$sw_bb312$$sw_bb322$$sw_bb331$$sw_bb347$$sw_bb381$$sw_bb391$$sw_bb403$$sw_bb414$$sw_bb422$$sw_bb439$$sw_bb449$$sw_bb461$$sw_bb481$$sw_bb486$$sw_bb491$$sw_bb509$$sw_bb523$$sw_bb533$$sw_bb534$$sw_bb547$$sw_bb576$$sw_bb599$$sw_bb614$$sw_bb623$$sw_bb630$$sw_bb645$$sw_bb692$$sw_bb702$$sw_bb711$$sw_bb729$$sw_bb734$6; //@line 383 "tr_languages.cpp"
      }
      else if (__label__ == 5) {
  
        var $23=$tr; //@line 387 "tr_languages.cpp"
        var $letter_bits_offset=(($23+964)&4294967295); //@line 387 "tr_languages.cpp"
        HEAP32[(($letter_bits_offset)>>2)]=1536; //@line 387 "tr_languages.cpp"
        var $24=$tr; //@line 388 "tr_languages.cpp"
        var $langopts19=(($24)&4294967295); //@line 388 "tr_languages.cpp"
        var $numbers20=(($langopts19+184)&4294967295); //@line 388 "tr_languages.cpp"
        HEAP32[(($numbers20)>>2)]=2884720; //@line 388 "tr_languages.cpp"
        var $25=$tr; //@line 389 "tr_languages.cpp"
        var $langopts21=(($25)&4294967295); //@line 389 "tr_languages.cpp"
        var $param22=(($langopts21+24)&4294967295); //@line 389 "tr_languages.cpp"
        var $arrayidx23=(($param22+20)&4294967295); //@line 389 "tr_languages.cpp"
        HEAP32[(($arrayidx23)>>2)]=1; //@line 389 "tr_languages.cpp"
        __label__ = 80;break $sw_default$$sw_bb$$sw_bb9$$sw_bb18$$sw_bb24$$sw_bb39$$sw_bb52$$sw_bb64$$sw_bb72$$sw_bb92$$sw_bb109$$sw_bb123$$sw_bb148$$sw_bb161$$sw_bb195$$sw_bb200$$sw_bb207$$sw_bb209$$sw_bb222$$sw_bb238$$sw_bb257$$sw_bb280$$sw_bb287$$sw_bb312$$sw_bb322$$sw_bb331$$sw_bb347$$sw_bb381$$sw_bb391$$sw_bb403$$sw_bb414$$sw_bb422$$sw_bb439$$sw_bb449$$sw_bb461$$sw_bb481$$sw_bb486$$sw_bb491$$sw_bb509$$sw_bb523$$sw_bb533$$sw_bb534$$sw_bb547$$sw_bb576$$sw_bb599$$sw_bb614$$sw_bb623$$sw_bb630$$sw_bb645$$sw_bb692$$sw_bb702$$sw_bb711$$sw_bb729$$sw_bb734$6; //@line 390 "tr_languages.cpp"
      }
      else if (__label__ == 6) {
  
        var $26=$tr; //@line 394 "tr_languages.cpp"
        __ZL18SetCyrillicLettersP10Translator($26); //@line 394 "tr_languages.cpp"
        var $27=$tr; //@line 395 "tr_languages.cpp"
        __ZL14SetLetterVowelP10Translatori($27, 42); //@line 395 "tr_languages.cpp"
        var $28=$tr; //@line 396 "tr_languages.cpp"
        var $langopts25=(($28)&4294967295); //@line 396 "tr_languages.cpp"
        var $param26=(($langopts25+24)&4294967295); //@line 396 "tr_languages.cpp"
        var $arrayidx27=(($param26+20)&4294967295); //@line 396 "tr_languages.cpp"
        HEAP32[(($arrayidx27)>>2)]=1074; //@line 396 "tr_languages.cpp"
        var $29=$tr; //@line 397 "tr_languages.cpp"
        var $langopts28=(($29)&4294967295); //@line 397 "tr_languages.cpp"
        var $param29=(($langopts28+24)&4294967295); //@line 397 "tr_languages.cpp"
        var $arrayidx30=(($param29+16)&4294967295); //@line 397 "tr_languages.cpp"
        HEAP32[(($arrayidx30)>>2)]=23; //@line 397 "tr_languages.cpp"
        var $30=$tr; //@line 398 "tr_languages.cpp"
        var $langopts31=(($30)&4294967295); //@line 398 "tr_languages.cpp"
        var $param32=(($langopts31+24)&4294967295); //@line 398 "tr_languages.cpp"
        var $arrayidx33=(($param32+40)&4294967295); //@line 398 "tr_languages.cpp"
        HEAP32[(($arrayidx33)>>2)]=2; //@line 398 "tr_languages.cpp"
        var $31=$tr; //@line 399 "tr_languages.cpp"
        var $langopts34=(($31)&4294967295); //@line 399 "tr_languages.cpp"
        var $stress_rule35=(($langopts34+8)&4294967295); //@line 399 "tr_languages.cpp"
        HEAP32[(($stress_rule35)>>2)]=2; //@line 399 "tr_languages.cpp"
        var $32=$tr; //@line 400 "tr_languages.cpp"
        var $langopts36=(($32)&4294967295); //@line 400 "tr_languages.cpp"
        var $numbers37=(($langopts36+184)&4294967295); //@line 400 "tr_languages.cpp"
        HEAP32[(($numbers37)>>2)]=184554728; //@line 400 "tr_languages.cpp"
        var $33=$tr; //@line 401 "tr_languages.cpp"
        var $langopts38=(($33)&4294967295); //@line 401 "tr_languages.cpp"
        var $thousands_sep=(($langopts38+204)&4294967295); //@line 401 "tr_languages.cpp"
        HEAP32[(($thousands_sep)>>2)]=32; //@line 401 "tr_languages.cpp"
        __label__ = 80;break $sw_default$$sw_bb$$sw_bb9$$sw_bb18$$sw_bb24$$sw_bb39$$sw_bb52$$sw_bb64$$sw_bb72$$sw_bb92$$sw_bb109$$sw_bb123$$sw_bb148$$sw_bb161$$sw_bb195$$sw_bb200$$sw_bb207$$sw_bb209$$sw_bb222$$sw_bb238$$sw_bb257$$sw_bb280$$sw_bb287$$sw_bb312$$sw_bb322$$sw_bb331$$sw_bb347$$sw_bb381$$sw_bb391$$sw_bb403$$sw_bb414$$sw_bb422$$sw_bb439$$sw_bb449$$sw_bb461$$sw_bb481$$sw_bb486$$sw_bb491$$sw_bb509$$sw_bb523$$sw_bb533$$sw_bb534$$sw_bb547$$sw_bb576$$sw_bb599$$sw_bb614$$sw_bb623$$sw_bb630$$sw_bb645$$sw_bb692$$sw_bb702$$sw_bb711$$sw_bb729$$sw_bb734$6; //@line 403 "tr_languages.cpp"
      }
      else if (__label__ == 7) {
  
        var $34=$tr; //@line 410 "tr_languages.cpp"
        __Z15SetupTranslatorP10TranslatorPKsPKh($34, ((__ZZ16SelectTranslatorPKcE17stress_lengths_bn)&4294967295), ((__ZZ16SelectTranslatorPKcE14stress_amps_bn)&4294967295)); //@line 410 "tr_languages.cpp"
        var $35=$tr; //@line 411 "tr_languages.cpp"
        var $langopts40=(($35)&4294967295); //@line 411 "tr_languages.cpp"
        var $length_mods41=(($langopts40+176)&4294967295); //@line 411 "tr_languages.cpp"
        var $36=HEAP32[(($length_mods41)>>2)]; //@line 411 "tr_languages.cpp"
        var $37=$tr; //@line 411 "tr_languages.cpp"
        var $langopts42=(($37)&4294967295); //@line 411 "tr_languages.cpp"
        var $length_mods043=(($langopts42+180)&4294967295); //@line 411 "tr_languages.cpp"
        HEAP32[(($length_mods043)>>2)]=$36; //@line 411 "tr_languages.cpp"
        var $38=$tr; //@line 413 "tr_languages.cpp"
        var $langopts44=(($38)&4294967295); //@line 413 "tr_languages.cpp"
        var $stress_rule45=(($langopts44+8)&4294967295); //@line 413 "tr_languages.cpp"
        HEAP32[(($stress_rule45)>>2)]=0; //@line 413 "tr_languages.cpp"
        var $39=$tr; //@line 414 "tr_languages.cpp"
        var $langopts46=(($39)&4294967295); //@line 414 "tr_languages.cpp"
        var $stress_flags47=(($langopts46+12)&4294967295); //@line 414 "tr_languages.cpp"
        HEAP32[(($stress_flags47)>>2)]=65540; //@line 414 "tr_languages.cpp"
        var $40=$tr; //@line 415 "tr_languages.cpp"
        var $letter_bits_offset48=(($40+964)&4294967295); //@line 415 "tr_languages.cpp"
        HEAP32[(($letter_bits_offset48)>>2)]=2432; //@line 415 "tr_languages.cpp"
        var $41=$tr; //@line 416 "tr_languages.cpp"
        __Z15SetIndicLettersP10Translator($41); //@line 416 "tr_languages.cpp"
        var $42=$tr; //@line 417 "tr_languages.cpp"
        __ZL18SetLetterBitsRangeP10Translatoriii($42, 4, 62, 76); //@line 417 "tr_languages.cpp"
        var $43=$tr; //@line 419 "tr_languages.cpp"
        var $langopts49=(($43)&4294967295); //@line 419 "tr_languages.cpp"
        var $numbers50=(($langopts49+184)&4294967295); //@line 419 "tr_languages.cpp"
        HEAP32[(($numbers50)>>2)]=1; //@line 419 "tr_languages.cpp"
        var $44=$tr; //@line 420 "tr_languages.cpp"
        var $langopts51=(($44)&4294967295); //@line 420 "tr_languages.cpp"
        var $break_numbers=(($langopts51+192)&4294967295); //@line 420 "tr_languages.cpp"
        HEAP32[(($break_numbers)>>2)]=613567144; //@line 420 "tr_languages.cpp"
        __label__ = 80;break $sw_default$$sw_bb$$sw_bb9$$sw_bb18$$sw_bb24$$sw_bb39$$sw_bb52$$sw_bb64$$sw_bb72$$sw_bb92$$sw_bb109$$sw_bb123$$sw_bb148$$sw_bb161$$sw_bb195$$sw_bb200$$sw_bb207$$sw_bb209$$sw_bb222$$sw_bb238$$sw_bb257$$sw_bb280$$sw_bb287$$sw_bb312$$sw_bb322$$sw_bb331$$sw_bb347$$sw_bb381$$sw_bb391$$sw_bb403$$sw_bb414$$sw_bb422$$sw_bb439$$sw_bb449$$sw_bb461$$sw_bb481$$sw_bb486$$sw_bb491$$sw_bb509$$sw_bb523$$sw_bb533$$sw_bb534$$sw_bb547$$sw_bb576$$sw_bb599$$sw_bb614$$sw_bb623$$sw_bb630$$sw_bb645$$sw_bb692$$sw_bb702$$sw_bb711$$sw_bb729$$sw_bb734$6; //@line 422 "tr_languages.cpp"
      }
      else if (__label__ == 8) {
  
        var $45=$tr; //@line 429 "tr_languages.cpp"
        __Z15SetupTranslatorP10TranslatorPKsPKh($45, ((__ZZ16SelectTranslatorPKcE17stress_lengths_cy)&4294967295), ((__ZZ16SelectTranslatorPKcE14stress_amps_cy)&4294967295)); //@line 429 "tr_languages.cpp"
        var $46=HEAP32[((((_charsets+56)&4294967295))>>2)]; //@line 431 "tr_languages.cpp"
        var $47=$tr; //@line 431 "tr_languages.cpp"
        var $charset_a0=(($47+696)&4294967295); //@line 431 "tr_languages.cpp"
        HEAP32[(($charset_a0)>>2)]=$46; //@line 431 "tr_languages.cpp"
        var $48=$tr; //@line 433 "tr_languages.cpp"
        var $langopts53=(($48)&4294967295); //@line 433 "tr_languages.cpp"
        var $stress_rule54=(($langopts53+8)&4294967295); //@line 433 "tr_languages.cpp"
        HEAP32[(($stress_rule54)>>2)]=2; //@line 433 "tr_languages.cpp"
        var $49=$tr; //@line 437 "tr_languages.cpp"
        var $langopts55=(($49)&4294967295); //@line 437 "tr_languages.cpp"
        var $stress_flags56=(($langopts55+12)&4294967295); //@line 437 "tr_languages.cpp"
        HEAP32[(($stress_flags56)>>2)]=22; //@line 437 "tr_languages.cpp"
        var $50=$tr; //@line 438 "tr_languages.cpp"
        var $langopts57=(($50)&4294967295); //@line 438 "tr_languages.cpp"
        var $unstressed_wd1=(($langopts57+16)&4294967295); //@line 438 "tr_languages.cpp"
        HEAP32[(($unstressed_wd1)>>2)]=0; //@line 438 "tr_languages.cpp"
        var $51=$tr; //@line 439 "tr_languages.cpp"
        var $langopts58=(($51)&4294967295); //@line 439 "tr_languages.cpp"
        var $unstressed_wd2=(($langopts58+20)&4294967295); //@line 439 "tr_languages.cpp"
        HEAP32[(($unstressed_wd2)>>2)]=2; //@line 439 "tr_languages.cpp"
        var $52=$tr; //@line 440 "tr_languages.cpp"
        var $langopts59=(($52)&4294967295); //@line 440 "tr_languages.cpp"
        var $param60=(($langopts59+24)&4294967295); //@line 440 "tr_languages.cpp"
        var $arrayidx61=(($param60+28)&4294967295); //@line 440 "tr_languages.cpp"
        HEAP32[(($arrayidx61)>>2)]=120; //@line 440 "tr_languages.cpp"
        var $53=$tr; //@line 442 "tr_languages.cpp"
        var $langopts62=(($53)&4294967295); //@line 442 "tr_languages.cpp"
        var $numbers63=(($langopts62+184)&4294967295); //@line 442 "tr_languages.cpp"
        HEAP32[(($numbers63)>>2)]=1024; //@line 442 "tr_languages.cpp"
        var $54=$tr; //@line 444 "tr_languages.cpp"
        __ZL14SetLetterVowelP10Translatori($54, 119); //@line 444 "tr_languages.cpp"
        var $55=$tr; //@line 445 "tr_languages.cpp"
        __ZL14SetLetterVowelP10Translatori($55, 121); //@line 445 "tr_languages.cpp"
        __label__ = 80;break $sw_default$$sw_bb$$sw_bb9$$sw_bb18$$sw_bb24$$sw_bb39$$sw_bb52$$sw_bb64$$sw_bb72$$sw_bb92$$sw_bb109$$sw_bb123$$sw_bb148$$sw_bb161$$sw_bb195$$sw_bb200$$sw_bb207$$sw_bb209$$sw_bb222$$sw_bb238$$sw_bb257$$sw_bb280$$sw_bb287$$sw_bb312$$sw_bb322$$sw_bb331$$sw_bb347$$sw_bb381$$sw_bb391$$sw_bb403$$sw_bb414$$sw_bb422$$sw_bb439$$sw_bb449$$sw_bb461$$sw_bb481$$sw_bb486$$sw_bb491$$sw_bb509$$sw_bb523$$sw_bb533$$sw_bb534$$sw_bb547$$sw_bb576$$sw_bb599$$sw_bb614$$sw_bb623$$sw_bb630$$sw_bb645$$sw_bb692$$sw_bb702$$sw_bb711$$sw_bb729$$sw_bb734$6; //@line 447 "tr_languages.cpp"
      }
      else if (__label__ == 9) {
  
        var $56=$tr; //@line 452 "tr_languages.cpp"
        __Z15SetupTranslatorP10TranslatorPKsPKh($56, ((__ZZ16SelectTranslatorPKcE17stress_lengths_da)&4294967295), 0); //@line 452 "tr_languages.cpp"
        var $57=$tr; //@line 454 "tr_languages.cpp"
        var $langopts65=(($57)&4294967295); //@line 454 "tr_languages.cpp"
        var $stress_rule66=(($langopts65+8)&4294967295); //@line 454 "tr_languages.cpp"
        HEAP32[(($stress_rule66)>>2)]=0; //@line 454 "tr_languages.cpp"
        var $58=$tr; //@line 455 "tr_languages.cpp"
        var $langopts67=(($58)&4294967295); //@line 455 "tr_languages.cpp"
        var $param68=(($langopts67+24)&4294967295); //@line 455 "tr_languages.cpp"
        var $arrayidx69=(($param68+12)&4294967295); //@line 455 "tr_languages.cpp"
        HEAP32[(($arrayidx69)>>2)]=1; //@line 455 "tr_languages.cpp"
        var $59=$tr; //@line 456 "tr_languages.cpp"
        __ZL14SetLetterVowelP10Translatori($59, 121); //@line 456 "tr_languages.cpp"
        var $60=$tr; //@line 457 "tr_languages.cpp"
        var $langopts70=(($60)&4294967295); //@line 457 "tr_languages.cpp"
        var $numbers71=(($langopts70+184)&4294967295); //@line 457 "tr_languages.cpp"
        HEAP32[(($numbers71)>>2)]=184618072; //@line 457 "tr_languages.cpp"
        __label__ = 80;break $sw_default$$sw_bb$$sw_bb9$$sw_bb18$$sw_bb24$$sw_bb39$$sw_bb52$$sw_bb64$$sw_bb72$$sw_bb92$$sw_bb109$$sw_bb123$$sw_bb148$$sw_bb161$$sw_bb195$$sw_bb200$$sw_bb207$$sw_bb209$$sw_bb222$$sw_bb238$$sw_bb257$$sw_bb280$$sw_bb287$$sw_bb312$$sw_bb322$$sw_bb331$$sw_bb347$$sw_bb381$$sw_bb391$$sw_bb403$$sw_bb414$$sw_bb422$$sw_bb439$$sw_bb449$$sw_bb461$$sw_bb481$$sw_bb486$$sw_bb491$$sw_bb509$$sw_bb523$$sw_bb533$$sw_bb534$$sw_bb547$$sw_bb576$$sw_bb599$$sw_bb614$$sw_bb623$$sw_bb630$$sw_bb645$$sw_bb692$$sw_bb702$$sw_bb711$$sw_bb729$$sw_bb734$6; //@line 459 "tr_languages.cpp"
      }
      else if (__label__ == 10) {
  
        var $61=$tr; //@line 466 "tr_languages.cpp"
        __Z15SetupTranslatorP10TranslatorPKsPKh($61, ((__ZZ16SelectTranslatorPKcE17stress_lengths_de)&4294967295), ((__ZZ16SelectTranslatorPKcE14stress_amps_de)&4294967295)); //@line 466 "tr_languages.cpp"
        var $62=$tr; //@line 467 "tr_languages.cpp"
        var $langopts73=(($62)&4294967295); //@line 467 "tr_languages.cpp"
        var $stress_rule74=(($langopts73+8)&4294967295); //@line 467 "tr_languages.cpp"
        HEAP32[(($stress_rule74)>>2)]=0; //@line 467 "tr_languages.cpp"
        var $63=$tr; //@line 468 "tr_languages.cpp"
        var $langopts75=(($63)&4294967295); //@line 468 "tr_languages.cpp"
        var $word_gap=(($langopts75)&4294967295); //@line 468 "tr_languages.cpp"
        HEAP32[(($word_gap)>>2)]=8; //@line 468 "tr_languages.cpp"
        var $64=$tr; //@line 469 "tr_languages.cpp"
        var $langopts76=(($64)&4294967295); //@line 469 "tr_languages.cpp"
        var $vowel_pause77=(($langopts76+4)&4294967295); //@line 469 "tr_languages.cpp"
        HEAP32[(($vowel_pause77)>>2)]=48; //@line 469 "tr_languages.cpp"
        var $65=$tr; //@line 470 "tr_languages.cpp"
        var $langopts78=(($65)&4294967295); //@line 470 "tr_languages.cpp"
        var $param79=(($langopts78+24)&4294967295); //@line 470 "tr_languages.cpp"
        var $arrayidx80=(($param79+12)&4294967295); //@line 470 "tr_languages.cpp"
        HEAP32[(($arrayidx80)>>2)]=1; //@line 470 "tr_languages.cpp"
        var $66=$tr; //@line 471 "tr_languages.cpp"
        var $langopts81=(($66)&4294967295); //@line 471 "tr_languages.cpp"
        var $param82=(($langopts81+24)&4294967295); //@line 471 "tr_languages.cpp"
        var $arrayidx83=(($param82+16)&4294967295); //@line 471 "tr_languages.cpp"
        HEAP32[(($arrayidx83)>>2)]=16; //@line 471 "tr_languages.cpp"
        var $67=$tr; //@line 472 "tr_languages.cpp"
        var $langopts84=(($67)&4294967295); //@line 472 "tr_languages.cpp"
        var $param85=(($langopts84+24)&4294967295); //@line 472 "tr_languages.cpp"
        var $arrayidx86=(($param85+72)&4294967295); //@line 472 "tr_languages.cpp"
        HEAP32[(($arrayidx86)>>2)]=87; //@line 472 "tr_languages.cpp"
        var $68=$tr; //@line 474 "tr_languages.cpp"
        var $langopts87=(($68)&4294967295); //@line 474 "tr_languages.cpp"
        var $numbers88=(($langopts87+184)&4294967295); //@line 474 "tr_languages.cpp"
        HEAP32[(($numbers88)>>2)]=16846872; //@line 474 "tr_languages.cpp"
        var $69=$tr; //@line 476 "tr_languages.cpp"
        __ZL14SetLetterVowelP10Translatori($69, 121); //@line 476 "tr_languages.cpp"
        var $70=$tr; //@line 477 "tr_languages.cpp"
        var $langopts89=(($70)&4294967295); //@line 477 "tr_languages.cpp"
        var $param90=(($langopts89+24)&4294967295); //@line 477 "tr_languages.cpp"
        var $arrayidx91=(($param90+20)&4294967295); //@line 477 "tr_languages.cpp"
        HEAP32[(($arrayidx91)>>2)]=2; //@line 477 "tr_languages.cpp"
        __label__ = 80;break $sw_default$$sw_bb$$sw_bb9$$sw_bb18$$sw_bb24$$sw_bb39$$sw_bb52$$sw_bb64$$sw_bb72$$sw_bb92$$sw_bb109$$sw_bb123$$sw_bb148$$sw_bb161$$sw_bb195$$sw_bb200$$sw_bb207$$sw_bb209$$sw_bb222$$sw_bb238$$sw_bb257$$sw_bb280$$sw_bb287$$sw_bb312$$sw_bb322$$sw_bb331$$sw_bb347$$sw_bb381$$sw_bb391$$sw_bb403$$sw_bb414$$sw_bb422$$sw_bb439$$sw_bb449$$sw_bb461$$sw_bb481$$sw_bb486$$sw_bb491$$sw_bb509$$sw_bb523$$sw_bb533$$sw_bb534$$sw_bb547$$sw_bb576$$sw_bb599$$sw_bb614$$sw_bb623$$sw_bb630$$sw_bb645$$sw_bb692$$sw_bb702$$sw_bb711$$sw_bb729$$sw_bb734$6; //@line 479 "tr_languages.cpp"
      }
      else if (__label__ == 11) {
  
        var $71=$tr; //@line 483 "tr_languages.cpp"
        __Z15SetupTranslatorP10TranslatorPKsPKh($71, ((__ZZ16SelectTranslatorPKcE17stress_lengths_ta)&4294967295), ((__ZZ16SelectTranslatorPKcE14stress_amps_ta)&4294967295)); //@line 483 "tr_languages.cpp"
        var $72=$tr; //@line 484 "tr_languages.cpp"
        var $langopts93=(($72)&4294967295); //@line 484 "tr_languages.cpp"
        var $param94=(($langopts93+24)&4294967295); //@line 484 "tr_languages.cpp"
        var $arrayidx95=(($param94+20)&4294967295); //@line 484 "tr_languages.cpp"
        HEAP32[(($arrayidx95)>>2)]=1; //@line 484 "tr_languages.cpp"
        var $73=$tr; //@line 485 "tr_languages.cpp"
        var $langopts96=(($73)&4294967295); //@line 485 "tr_languages.cpp"
        var $length_mods97=(($langopts96+176)&4294967295); //@line 485 "tr_languages.cpp"
        var $74=HEAP32[(($length_mods97)>>2)]; //@line 485 "tr_languages.cpp"
        var $75=$tr; //@line 485 "tr_languages.cpp"
        var $langopts98=(($75)&4294967295); //@line 485 "tr_languages.cpp"
        var $length_mods099=(($langopts98+180)&4294967295); //@line 485 "tr_languages.cpp"
        HEAP32[(($length_mods099)>>2)]=$74; //@line 485 "tr_languages.cpp"
        var $76=$tr; //@line 486 "tr_languages.cpp"
        var $letter_bits_offset100=(($76+964)&4294967295); //@line 486 "tr_languages.cpp"
        HEAP32[(($letter_bits_offset100)>>2)]=1920; //@line 486 "tr_languages.cpp"
        var $77=$tr; //@line 487 "tr_languages.cpp"
        var $langopts101=(($77)&4294967295); //@line 487 "tr_languages.cpp"
        var $stress_rule102=(($langopts101+8)&4294967295); //@line 487 "tr_languages.cpp"
        HEAP32[(($stress_rule102)>>2)]=0; //@line 487 "tr_languages.cpp"
        var $78=$tr; //@line 488 "tr_languages.cpp"
        var $langopts103=(($78)&4294967295); //@line 488 "tr_languages.cpp"
        var $stress_flags104=(($langopts103+12)&4294967295); //@line 488 "tr_languages.cpp"
        HEAP32[(($stress_flags104)>>2)]=65540; //@line 488 "tr_languages.cpp"
        var $79=$tr; //@line 489 "tr_languages.cpp"
        __ZL18SetLetterBitsRangeP10Translatoriii($79, 1, 38, 48); //@line 489 "tr_languages.cpp"
        var $80=$tr; //@line 490 "tr_languages.cpp"
        var $langopts105=(($80)&4294967295); //@line 490 "tr_languages.cpp"
        var $break_numbers106=(($langopts105+192)&4294967295); //@line 490 "tr_languages.cpp"
        HEAP32[(($break_numbers106)>>2)]=5288; //@line 490 "tr_languages.cpp"
        var $81=$tr; //@line 491 "tr_languages.cpp"
        var $langopts107=(($81)&4294967295); //@line 491 "tr_languages.cpp"
        var $numbers108=(($langopts107+184)&4294967295); //@line 491 "tr_languages.cpp"
        HEAP32[(($numbers108)>>2)]=1; //@line 491 "tr_languages.cpp"
        __label__ = 80;break $sw_default$$sw_bb$$sw_bb9$$sw_bb18$$sw_bb24$$sw_bb39$$sw_bb52$$sw_bb64$$sw_bb72$$sw_bb92$$sw_bb109$$sw_bb123$$sw_bb148$$sw_bb161$$sw_bb195$$sw_bb200$$sw_bb207$$sw_bb209$$sw_bb222$$sw_bb238$$sw_bb257$$sw_bb280$$sw_bb287$$sw_bb312$$sw_bb322$$sw_bb331$$sw_bb347$$sw_bb381$$sw_bb391$$sw_bb403$$sw_bb414$$sw_bb422$$sw_bb439$$sw_bb449$$sw_bb461$$sw_bb481$$sw_bb486$$sw_bb491$$sw_bb509$$sw_bb523$$sw_bb533$$sw_bb534$$sw_bb547$$sw_bb576$$sw_bb599$$sw_bb614$$sw_bb623$$sw_bb630$$sw_bb645$$sw_bb692$$sw_bb702$$sw_bb711$$sw_bb729$$sw_bb734$6; //@line 493 "tr_languages.cpp"
      }
      else if (__label__ == 12) {
  
        var $82=$tr; //@line 498 "tr_languages.cpp"
        __Z15SetupTranslatorP10TranslatorPKsPKh($82, ((__ZZ16SelectTranslatorPKcE17stress_lengths_en)&4294967295), 0); //@line 498 "tr_languages.cpp"
        var $83=$tr; //@line 500 "tr_languages.cpp"
        var $langopts110=(($83)&4294967295); //@line 500 "tr_languages.cpp"
        var $stress_rule111=(($langopts110+8)&4294967295); //@line 500 "tr_languages.cpp"
        HEAP32[(($stress_rule111)>>2)]=0; //@line 500 "tr_languages.cpp"
        var $84=$tr; //@line 501 "tr_languages.cpp"
        var $langopts112=(($84)&4294967295); //@line 501 "tr_languages.cpp"
        var $stress_flags113=(($langopts112+12)&4294967295); //@line 501 "tr_languages.cpp"
        HEAP32[(($stress_flags113)>>2)]=8; //@line 501 "tr_languages.cpp"
        var $85=$tr; //@line 502 "tr_languages.cpp"
        var $langopts114=(($85)&4294967295); //@line 502 "tr_languages.cpp"
        var $numbers115=(($langopts114+184)&4294967295); //@line 502 "tr_languages.cpp"
        HEAP32[(($numbers115)>>2)]=16779328; //@line 502 "tr_languages.cpp"
        var $86=$tr; //@line 503 "tr_languages.cpp"
        var $langopts116=(($86)&4294967295); //@line 503 "tr_languages.cpp"
        var $param117=(($langopts116+24)&4294967295); //@line 503 "tr_languages.cpp"
        var $arrayidx118=(($param117+44)&4294967295); //@line 503 "tr_languages.cpp"
        HEAP32[(($arrayidx118)>>2)]=2; //@line 503 "tr_languages.cpp"
        var $87=$tr; //@line 504 "tr_languages.cpp"
        var $langopts119=(($87)&4294967295); //@line 504 "tr_languages.cpp"
        var $suffix_add_e=(($langopts119+276)&4294967295); //@line 504 "tr_languages.cpp"
        HEAP32[(($suffix_add_e)>>2)]=101; //@line 504 "tr_languages.cpp"
        var $88=$tr; //@line 505 "tr_languages.cpp"
        var $langopts120=(($88)&4294967295); //@line 505 "tr_languages.cpp"
        var $param121=(($langopts120+24)&4294967295); //@line 505 "tr_languages.cpp"
        var $arrayidx122=(($param121+20)&4294967295); //@line 505 "tr_languages.cpp"
        HEAP32[(($arrayidx122)>>2)]=2; //@line 505 "tr_languages.cpp"
        var $89=$tr; //@line 506 "tr_languages.cpp"
        __ZL13SetLetterBitsP10TranslatoriPKc($89, 6, ((__str630)&4294967295)); //@line 506 "tr_languages.cpp"
        __label__ = 80;break $sw_default$$sw_bb$$sw_bb9$$sw_bb18$$sw_bb24$$sw_bb39$$sw_bb52$$sw_bb64$$sw_bb72$$sw_bb92$$sw_bb109$$sw_bb123$$sw_bb148$$sw_bb161$$sw_bb195$$sw_bb200$$sw_bb207$$sw_bb209$$sw_bb222$$sw_bb238$$sw_bb257$$sw_bb280$$sw_bb287$$sw_bb312$$sw_bb322$$sw_bb331$$sw_bb347$$sw_bb381$$sw_bb391$$sw_bb403$$sw_bb414$$sw_bb422$$sw_bb439$$sw_bb449$$sw_bb461$$sw_bb481$$sw_bb486$$sw_bb491$$sw_bb509$$sw_bb523$$sw_bb533$$sw_bb534$$sw_bb547$$sw_bb576$$sw_bb599$$sw_bb614$$sw_bb623$$sw_bb630$$sw_bb645$$sw_bb692$$sw_bb702$$sw_bb711$$sw_bb729$$sw_bb734$6; //@line 508 "tr_languages.cpp"
      }
      else if (__label__ == 13) {
  
        var $90=$tr; //@line 523 "tr_languages.cpp"
        __Z15SetupTranslatorP10TranslatorPKsPKh($90, ((__ZZ16SelectTranslatorPKcE17stress_lengths_el)&4294967295), ((__ZZ16SelectTranslatorPKcE14stress_amps_el)&4294967295)); //@line 523 "tr_languages.cpp"
        var $91=HEAP32[((((_charsets+28)&4294967295))>>2)]; //@line 525 "tr_languages.cpp"
        var $92=$tr; //@line 525 "tr_languages.cpp"
        var $charset_a0124=(($92+696)&4294967295); //@line 525 "tr_languages.cpp"
        HEAP32[(($charset_a0124)>>2)]=$91; //@line 525 "tr_languages.cpp"
        var $93=$tr; //@line 526 "tr_languages.cpp"
        var $char_plus_apostrophe=(($93+700)&4294967295); //@line 526 "tr_languages.cpp"
        HEAP32[(($char_plus_apostrophe)>>2)]=((__ZZ16SelectTranslatorPKcE18el_char_apostrophe)&4294967295); //@line 526 "tr_languages.cpp"
        var $94=$tr; //@line 528 "tr_languages.cpp"
        var $letter_bits_offset125=(($94+964)&4294967295); //@line 528 "tr_languages.cpp"
        HEAP32[(($letter_bits_offset125)>>2)]=896; //@line 528 "tr_languages.cpp"
        var $95=$tr; //@line 529 "tr_languages.cpp"
        var $letter_bits=(($95+708)&4294967295); //@line 529 "tr_languages.cpp"
        var $arraydecay=(($letter_bits)&4294967295); //@line 529 "tr_languages.cpp"
        var $dest$, $stop$, $stop4$, $fast$, $value4$;
        $dest$ = $arraydecay;
        $stop$ = $dest$ + 256;
        $value4$ = 0;
        if ($value4$ < 0) $value4$ += 256;
        $value4$ = $value4$ + ($value4$<<8) + ($value4$<<16) + ($value4$*16777216);
        while ($dest$%4 !== 0 && $dest$ < $stop$) {
          ;; HEAP8[$dest$++] = 0;
        }
        $dest$ >>= 2;
        $stop4$ = $stop$ >> 2;
        while ($dest$ < $stop4$) {
        ;;;;;;;  HEAP32[$dest$++] = $value4$;
        }
        $dest$ <<= 2;
        while ($dest$ < $stop$) {
          ;; HEAP8[$dest$++] = 0;
        }; //@line 529 "tr_languages.cpp"
        var $96=$tr; //@line 530 "tr_languages.cpp"
        __ZL13SetLetterBitsP10TranslatoriPKc($96, 0, ((__ZZ16SelectTranslatorPKcE9el_vowels)&4294967295)); //@line 530 "tr_languages.cpp"
        var $97=$tr; //@line 531 "tr_languages.cpp"
        __ZL13SetLetterBitsP10TranslatoriPKc($97, 7, ((__ZZ16SelectTranslatorPKcE9el_vowels)&4294967295)); //@line 531 "tr_languages.cpp"
        var $98=$tr; //@line 532 "tr_languages.cpp"
        __ZL13SetLetterBitsP10TranslatoriPKc($98, 1, ((__ZZ16SelectTranslatorPKcE12el_voiceless)&4294967295)); //@line 532 "tr_languages.cpp"
        var $99=$tr; //@line 533 "tr_languages.cpp"
        __ZL13SetLetterBitsP10TranslatoriPKc($99, 2, ((__ZZ16SelectTranslatorPKcE13el_consonants)&4294967295)); //@line 533 "tr_languages.cpp"
        var $100=$tr; //@line 534 "tr_languages.cpp"
        __ZL13SetLetterBitsP10TranslatoriPKc($100, 6, ((__ZZ16SelectTranslatorPKcE10el_fvowels)&4294967295)); //@line 534 "tr_languages.cpp"
        var $101=$tr; //@line 536 "tr_languages.cpp"
        var $langopts126=(($101)&4294967295); //@line 536 "tr_languages.cpp"
        var $length_mods127=(($langopts126+176)&4294967295); //@line 536 "tr_languages.cpp"
        var $102=HEAP32[(($length_mods127)>>2)]; //@line 536 "tr_languages.cpp"
        var $103=$tr; //@line 536 "tr_languages.cpp"
        var $langopts128=(($103)&4294967295); //@line 536 "tr_languages.cpp"
        var $length_mods0129=(($langopts128+180)&4294967295); //@line 536 "tr_languages.cpp"
        HEAP32[(($length_mods0129)>>2)]=$102; //@line 536 "tr_languages.cpp"
        var $104=$tr; //@line 537 "tr_languages.cpp"
        var $langopts130=(($104)&4294967295); //@line 537 "tr_languages.cpp"
        var $stress_rule131=(($langopts130+8)&4294967295); //@line 537 "tr_languages.cpp"
        HEAP32[(($stress_rule131)>>2)]=2; //@line 537 "tr_languages.cpp"
        var $105=$tr; //@line 538 "tr_languages.cpp"
        var $langopts132=(($105)&4294967295); //@line 538 "tr_languages.cpp"
        var $stress_flags133=(($langopts132+12)&4294967295); //@line 538 "tr_languages.cpp"
        HEAP32[(($stress_flags133)>>2)]=6; //@line 538 "tr_languages.cpp"
        var $106=$tr; //@line 539 "tr_languages.cpp"
        var $langopts134=(($106)&4294967295); //@line 539 "tr_languages.cpp"
        var $unstressed_wd1135=(($langopts134+16)&4294967295); //@line 539 "tr_languages.cpp"
        HEAP32[(($unstressed_wd1135)>>2)]=0; //@line 539 "tr_languages.cpp"
        var $107=$tr; //@line 540 "tr_languages.cpp"
        var $langopts136=(($107)&4294967295); //@line 540 "tr_languages.cpp"
        var $unstressed_wd2137=(($langopts136+20)&4294967295); //@line 540 "tr_languages.cpp"
        HEAP32[(($unstressed_wd2137)>>2)]=2; //@line 540 "tr_languages.cpp"
        var $108=$tr; //@line 541 "tr_languages.cpp"
        var $langopts138=(($108)&4294967295); //@line 541 "tr_languages.cpp"
        var $param139=(($langopts138+24)&4294967295); //@line 541 "tr_languages.cpp"
        var $arrayidx140=(($param139+28)&4294967295); //@line 541 "tr_languages.cpp"
        HEAP32[(($arrayidx140)>>2)]=130; //@line 541 "tr_languages.cpp"
        var $109=$tr; //@line 543 "tr_languages.cpp"
        var $langopts141=(($109)&4294967295); //@line 543 "tr_languages.cpp"
        var $numbers142=(($langopts141+184)&4294967295); //@line 543 "tr_languages.cpp"
        HEAP32[(($numbers142)>>2)]=264; //@line 543 "tr_languages.cpp"
        var $110=$tr; //@line 544 "tr_languages.cpp"
        var $langopts143=(($110)&4294967295); //@line 544 "tr_languages.cpp"
        var $numbers2=(($langopts143+188)&4294967295); //@line 544 "tr_languages.cpp"
        HEAP32[(($numbers2)>>2)]=4098; //@line 544 "tr_languages.cpp"
        var $111=$name2; //@line 546 "tr_languages.cpp"
        var $cmp144=((($111))|0)==6779491; //@line 546 "tr_languages.cpp"
        if (!($cmp144)) { __label__ = 80;break $sw_default$$sw_bb$$sw_bb9$$sw_bb18$$sw_bb24$$sw_bb39$$sw_bb52$$sw_bb64$$sw_bb72$$sw_bb92$$sw_bb109$$sw_bb123$$sw_bb148$$sw_bb161$$sw_bb195$$sw_bb200$$sw_bb207$$sw_bb209$$sw_bb222$$sw_bb238$$sw_bb257$$sw_bb280$$sw_bb287$$sw_bb312$$sw_bb322$$sw_bb331$$sw_bb347$$sw_bb381$$sw_bb391$$sw_bb403$$sw_bb414$$sw_bb422$$sw_bb439$$sw_bb449$$sw_bb461$$sw_bb481$$sw_bb486$$sw_bb491$$sw_bb509$$sw_bb523$$sw_bb533$$sw_bb534$$sw_bb547$$sw_bb576$$sw_bb599$$sw_bb614$$sw_bb623$$sw_bb630$$sw_bb645$$sw_bb692$$sw_bb702$$sw_bb711$$sw_bb729$$sw_bb734$6; } //@line 546 "tr_languages.cpp"
  
        var $112=$tr; //@line 549 "tr_languages.cpp"
        var $langopts145=(($112)&4294967295); //@line 549 "tr_languages.cpp"
        var $param146=(($langopts145+24)&4294967295); //@line 549 "tr_languages.cpp"
        var $arrayidx147=(($param146+20)&4294967295); //@line 549 "tr_languages.cpp"
        HEAP32[(($arrayidx147)>>2)]=1; //@line 549 "tr_languages.cpp"
        __label__ = 80;break $sw_default$$sw_bb$$sw_bb9$$sw_bb18$$sw_bb24$$sw_bb39$$sw_bb52$$sw_bb64$$sw_bb72$$sw_bb92$$sw_bb109$$sw_bb123$$sw_bb148$$sw_bb161$$sw_bb195$$sw_bb200$$sw_bb207$$sw_bb209$$sw_bb222$$sw_bb238$$sw_bb257$$sw_bb280$$sw_bb287$$sw_bb312$$sw_bb322$$sw_bb331$$sw_bb347$$sw_bb381$$sw_bb391$$sw_bb403$$sw_bb414$$sw_bb422$$sw_bb439$$sw_bb449$$sw_bb461$$sw_bb481$$sw_bb486$$sw_bb491$$sw_bb509$$sw_bb523$$sw_bb533$$sw_bb534$$sw_bb547$$sw_bb576$$sw_bb599$$sw_bb614$$sw_bb623$$sw_bb630$$sw_bb645$$sw_bb692$$sw_bb702$$sw_bb711$$sw_bb729$$sw_bb734$6; //@line 550 "tr_languages.cpp"
      }
      else if (__label__ == 15) {
  
        var $113=$tr; //@line 560 "tr_languages.cpp"
        __Z15SetupTranslatorP10TranslatorPKsPKh($113, ((__ZZ16SelectTranslatorPKcE17stress_lengths_eo)&4294967295), ((__ZZ16SelectTranslatorPKcE14stress_amps_eo)&4294967295)); //@line 560 "tr_languages.cpp"
        var $114=HEAP32[((((_charsets+12)&4294967295))>>2)]; //@line 562 "tr_languages.cpp"
        var $115=$tr; //@line 562 "tr_languages.cpp"
        var $charset_a0149=(($115+696)&4294967295); //@line 562 "tr_languages.cpp"
        HEAP32[(($charset_a0149)>>2)]=$114; //@line 562 "tr_languages.cpp"
        var $116=$tr; //@line 563 "tr_languages.cpp"
        var $char_plus_apostrophe150=(($116+700)&4294967295); //@line 563 "tr_languages.cpp"
        HEAP32[(($char_plus_apostrophe150)>>2)]=((__ZZ16SelectTranslatorPKcE18eo_char_apostrophe)&4294967295); //@line 563 "tr_languages.cpp"
        var $117=$tr; //@line 566 "tr_languages.cpp"
        var $langopts151=(($117)&4294967295); //@line 566 "tr_languages.cpp"
        var $vowel_pause152=(($langopts151+4)&4294967295); //@line 566 "tr_languages.cpp"
        HEAP32[(($vowel_pause152)>>2)]=2; //@line 566 "tr_languages.cpp"
        var $118=$tr; //@line 567 "tr_languages.cpp"
        var $langopts153=(($118)&4294967295); //@line 567 "tr_languages.cpp"
        var $stress_rule154=(($langopts153+8)&4294967295); //@line 567 "tr_languages.cpp"
        HEAP32[(($stress_rule154)>>2)]=2; //@line 567 "tr_languages.cpp"
        var $119=$tr; //@line 568 "tr_languages.cpp"
        var $langopts155=(($119)&4294967295); //@line 568 "tr_languages.cpp"
        var $stress_flags156=(($langopts155+12)&4294967295); //@line 568 "tr_languages.cpp"
        HEAP32[(($stress_flags156)>>2)]=22; //@line 568 "tr_languages.cpp"
        var $120=$tr; //@line 570 "tr_languages.cpp"
        var $langopts157=(($120)&4294967295); //@line 570 "tr_languages.cpp"
        var $unstressed_wd2158=(($langopts157+20)&4294967295); //@line 570 "tr_languages.cpp"
        HEAP32[(($unstressed_wd2158)>>2)]=2; //@line 570 "tr_languages.cpp"
        var $121=$tr; //@line 572 "tr_languages.cpp"
        var $langopts159=(($121)&4294967295); //@line 572 "tr_languages.cpp"
        var $numbers160=(($langopts159+184)&4294967295); //@line 572 "tr_languages.cpp"
        HEAP32[(($numbers160)>>2)]=16782344; //@line 572 "tr_languages.cpp"
        __label__ = 80;break $sw_default$$sw_bb$$sw_bb9$$sw_bb18$$sw_bb24$$sw_bb39$$sw_bb52$$sw_bb64$$sw_bb72$$sw_bb92$$sw_bb109$$sw_bb123$$sw_bb148$$sw_bb161$$sw_bb195$$sw_bb200$$sw_bb207$$sw_bb209$$sw_bb222$$sw_bb238$$sw_bb257$$sw_bb280$$sw_bb287$$sw_bb312$$sw_bb322$$sw_bb331$$sw_bb347$$sw_bb381$$sw_bb391$$sw_bb403$$sw_bb414$$sw_bb422$$sw_bb439$$sw_bb449$$sw_bb461$$sw_bb481$$sw_bb486$$sw_bb491$$sw_bb509$$sw_bb523$$sw_bb533$$sw_bb534$$sw_bb547$$sw_bb576$$sw_bb599$$sw_bb614$$sw_bb623$$sw_bb630$$sw_bb645$$sw_bb692$$sw_bb702$$sw_bb711$$sw_bb729$$sw_bb734$6; //@line 574 "tr_languages.cpp"
      }
      else if (__label__ == 16) {
  
        var $122=$tr; //@line 585 "tr_languages.cpp"
        __Z15SetupTranslatorP10TranslatorPKsPKh($122, ((__ZZ16SelectTranslatorPKcE17stress_lengths_es)&4294967295), ((__ZZ16SelectTranslatorPKcE14stress_amps_es)&4294967295)); //@line 585 "tr_languages.cpp"
        var $123=$tr; //@line 587 "tr_languages.cpp"
        var $langopts162=(($123)&4294967295); //@line 587 "tr_languages.cpp"
        var $length_mods163=(($langopts162+176)&4294967295); //@line 587 "tr_languages.cpp"
        var $124=HEAP32[(($length_mods163)>>2)]; //@line 587 "tr_languages.cpp"
        var $125=$tr; //@line 587 "tr_languages.cpp"
        var $langopts164=(($125)&4294967295); //@line 587 "tr_languages.cpp"
        var $length_mods0165=(($langopts164+180)&4294967295); //@line 587 "tr_languages.cpp"
        HEAP32[(($length_mods0165)>>2)]=$124; //@line 587 "tr_languages.cpp"
        var $126=$tr; //@line 588 "tr_languages.cpp"
        var $langopts166=(($126)&4294967295); //@line 588 "tr_languages.cpp"
        var $stress_rule167=(($langopts166+8)&4294967295); //@line 588 "tr_languages.cpp"
        HEAP32[(($stress_rule167)>>2)]=2; //@line 588 "tr_languages.cpp"
        var $127=$tr; //@line 592 "tr_languages.cpp"
        var $langopts168=(($127)&4294967295); //@line 592 "tr_languages.cpp"
        var $stress_flags169=(($langopts168+12)&4294967295); //@line 592 "tr_languages.cpp"
        HEAP32[(($stress_flags169)>>2)]=534; //@line 592 "tr_languages.cpp"
        var $128=$tr; //@line 593 "tr_languages.cpp"
        var $langopts170=(($128)&4294967295); //@line 593 "tr_languages.cpp"
        var $unstressed_wd1171=(($langopts170+16)&4294967295); //@line 593 "tr_languages.cpp"
        HEAP32[(($unstressed_wd1171)>>2)]=0; //@line 593 "tr_languages.cpp"
        var $129=$tr; //@line 594 "tr_languages.cpp"
        var $langopts172=(($129)&4294967295); //@line 594 "tr_languages.cpp"
        var $unstressed_wd2173=(($langopts172+20)&4294967295); //@line 594 "tr_languages.cpp"
        HEAP32[(($unstressed_wd2173)>>2)]=2; //@line 594 "tr_languages.cpp"
        var $130=$tr; //@line 595 "tr_languages.cpp"
        var $langopts174=(($130)&4294967295); //@line 595 "tr_languages.cpp"
        var $param175=(($langopts174+24)&4294967295); //@line 595 "tr_languages.cpp"
        var $arrayidx176=(($param175+28)&4294967295); //@line 595 "tr_languages.cpp"
        HEAP32[(($arrayidx176)>>2)]=120; //@line 595 "tr_languages.cpp"
        var $131=$tr; //@line 597 "tr_languages.cpp"
        var $langopts177=(($131)&4294967295); //@line 597 "tr_languages.cpp"
        var $numbers178=(($langopts177+184)&4294967295); //@line 597 "tr_languages.cpp"
        HEAP32[(($numbers178)>>2)]=85984552; //@line 597 "tr_languages.cpp"
        var $132=$tr; //@line 598 "tr_languages.cpp"
        var $langopts179=(($132)&4294967295); //@line 598 "tr_languages.cpp"
        var $numbers2180=(($langopts179+188)&4294967295); //@line 598 "tr_languages.cpp"
        HEAP32[(($numbers2180)>>2)]=4096; //@line 598 "tr_languages.cpp"
        var $133=$name2; //@line 600 "tr_languages.cpp"
        var $cmp181=((($133))|0)==25441; //@line 600 "tr_languages.cpp"
        if ($cmp181) { __label__ = 17;; } else { __label__ = 18;; } //@line 600 "tr_languages.cpp"
        if (__label__ == 17) {
  
          var $134=$tr; //@line 602 "tr_languages.cpp"
          var $punct_within_word=(($134+704)&4294967295); //@line 602 "tr_languages.cpp"
          HEAP32[(($punct_within_word)>>2)]=((__ZZ16SelectTranslatorPKcE20ca_punct_within_word)&4294967295); //@line 602 "tr_languages.cpp"
          var $135=$tr; //@line 603 "tr_languages.cpp"
          var $langopts183=(($135)&4294967295); //@line 603 "tr_languages.cpp"
          var $stress_flags184=(($langopts183+12)&4294967295); //@line 603 "tr_languages.cpp"
          HEAP32[(($stress_flags184)>>2)]=566; //@line 603 "tr_languages.cpp"
          __label__ = 80;break $sw_default$$sw_bb$$sw_bb9$$sw_bb18$$sw_bb24$$sw_bb39$$sw_bb52$$sw_bb64$$sw_bb72$$sw_bb92$$sw_bb109$$sw_bb123$$sw_bb148$$sw_bb161$$sw_bb195$$sw_bb200$$sw_bb207$$sw_bb209$$sw_bb222$$sw_bb238$$sw_bb257$$sw_bb280$$sw_bb287$$sw_bb312$$sw_bb322$$sw_bb331$$sw_bb347$$sw_bb381$$sw_bb391$$sw_bb403$$sw_bb414$$sw_bb422$$sw_bb439$$sw_bb449$$sw_bb461$$sw_bb481$$sw_bb486$$sw_bb491$$sw_bb509$$sw_bb523$$sw_bb533$$sw_bb534$$sw_bb547$$sw_bb576$$sw_bb599$$sw_bb614$$sw_bb623$$sw_bb630$$sw_bb645$$sw_bb692$$sw_bb702$$sw_bb711$$sw_bb729$$sw_bb734$6; //@line 604 "tr_languages.cpp"
        }
        else if (__label__ == 18) {
  
          var $136=$name2; //@line 606 "tr_languages.cpp"
          var $cmp185=((($136))|0)==7364976; //@line 606 "tr_languages.cpp"
          var $137=$tr; //@line 608 "tr_languages.cpp"
          var $langopts187=(($137)&4294967295); //@line 608 "tr_languages.cpp"
          if ($cmp185) { __label__ = 19;; } else { __label__ = 20;; } //@line 606 "tr_languages.cpp"
          if (__label__ == 19) {
  
            var $stress_flags188=(($langopts187+12)&4294967295); //@line 608 "tr_languages.cpp"
            HEAP32[(($stress_flags188)>>2)]=310; //@line 608 "tr_languages.cpp"
            __label__ = 80;break $sw_default$$sw_bb$$sw_bb9$$sw_bb18$$sw_bb24$$sw_bb39$$sw_bb52$$sw_bb64$$sw_bb72$$sw_bb92$$sw_bb109$$sw_bb123$$sw_bb148$$sw_bb161$$sw_bb195$$sw_bb200$$sw_bb207$$sw_bb209$$sw_bb222$$sw_bb238$$sw_bb257$$sw_bb280$$sw_bb287$$sw_bb312$$sw_bb322$$sw_bb331$$sw_bb347$$sw_bb381$$sw_bb391$$sw_bb403$$sw_bb414$$sw_bb422$$sw_bb439$$sw_bb449$$sw_bb461$$sw_bb481$$sw_bb486$$sw_bb491$$sw_bb509$$sw_bb523$$sw_bb533$$sw_bb534$$sw_bb547$$sw_bb576$$sw_bb599$$sw_bb614$$sw_bb623$$sw_bb630$$sw_bb645$$sw_bb692$$sw_bb702$$sw_bb711$$sw_bb729$$sw_bb734$6; //@line 609 "tr_languages.cpp"
          }
          else if (__label__ == 20) {
  
            var $param191=(($langopts187+24)&4294967295); //@line 612 "tr_languages.cpp"
            var $arrayidx192=(($param191+20)&4294967295); //@line 612 "tr_languages.cpp"
            HEAP32[(($arrayidx192)>>2)]=2; //@line 612 "tr_languages.cpp"
            __label__ = 80;break $sw_default$$sw_bb$$sw_bb9$$sw_bb18$$sw_bb24$$sw_bb39$$sw_bb52$$sw_bb64$$sw_bb72$$sw_bb92$$sw_bb109$$sw_bb123$$sw_bb148$$sw_bb161$$sw_bb195$$sw_bb200$$sw_bb207$$sw_bb209$$sw_bb222$$sw_bb238$$sw_bb257$$sw_bb280$$sw_bb287$$sw_bb312$$sw_bb322$$sw_bb331$$sw_bb347$$sw_bb381$$sw_bb391$$sw_bb403$$sw_bb414$$sw_bb422$$sw_bb439$$sw_bb449$$sw_bb461$$sw_bb481$$sw_bb486$$sw_bb491$$sw_bb509$$sw_bb523$$sw_bb533$$sw_bb534$$sw_bb547$$sw_bb576$$sw_bb599$$sw_bb614$$sw_bb623$$sw_bb630$$sw_bb645$$sw_bb692$$sw_bb702$$sw_bb711$$sw_bb729$$sw_bb734$6;
          }
        }
      }
      else if (__label__ == 21) {
  
        var $138=$tr; //@line 621 "tr_languages.cpp"
        __Z15SetupTranslatorP10TranslatorPKsPKh($138, ((__ZZ16SelectTranslatorPKcE17stress_lengths_eu)&4294967295), ((__ZZ16SelectTranslatorPKcE14stress_amps_eu)&4294967295)); //@line 621 "tr_languages.cpp"
        var $139=$tr; //@line 622 "tr_languages.cpp"
        var $langopts196=(($139)&4294967295); //@line 622 "tr_languages.cpp"
        var $stress_rule197=(($langopts196+8)&4294967295); //@line 622 "tr_languages.cpp"
        HEAP32[(($stress_rule197)>>2)]=1; //@line 622 "tr_languages.cpp"
        var $140=$tr; //@line 623 "tr_languages.cpp"
        var $langopts198=(($140)&4294967295); //@line 623 "tr_languages.cpp"
        var $numbers199=(($langopts198+184)&4294967295); //@line 623 "tr_languages.cpp"
        HEAP32[(($numbers199)>>2)]=1049960; //@line 623 "tr_languages.cpp"
        __label__ = 80;break $sw_default$$sw_bb$$sw_bb9$$sw_bb18$$sw_bb24$$sw_bb39$$sw_bb52$$sw_bb64$$sw_bb72$$sw_bb92$$sw_bb109$$sw_bb123$$sw_bb148$$sw_bb161$$sw_bb195$$sw_bb200$$sw_bb207$$sw_bb209$$sw_bb222$$sw_bb238$$sw_bb257$$sw_bb280$$sw_bb287$$sw_bb312$$sw_bb322$$sw_bb331$$sw_bb347$$sw_bb381$$sw_bb391$$sw_bb403$$sw_bb414$$sw_bb422$$sw_bb439$$sw_bb449$$sw_bb461$$sw_bb481$$sw_bb486$$sw_bb491$$sw_bb509$$sw_bb523$$sw_bb533$$sw_bb534$$sw_bb547$$sw_bb576$$sw_bb599$$sw_bb614$$sw_bb623$$sw_bb630$$sw_bb645$$sw_bb692$$sw_bb702$$sw_bb711$$sw_bb729$$sw_bb734$6; //@line 625 "tr_languages.cpp"
      }
      else if (__label__ == 22) {
  
        var $141=$tr; //@line 630 "tr_languages.cpp"
        var $letter_bits_offset201=(($141+964)&4294967295); //@line 630 "tr_languages.cpp"
        HEAP32[(($letter_bits_offset201)>>2)]=1536; //@line 630 "tr_languages.cpp"
        var $142=$tr; //@line 631 "tr_languages.cpp"
        var $langopts202=(($142)&4294967295); //@line 631 "tr_languages.cpp"
        var $numbers203=(($langopts202+184)&4294967295); //@line 631 "tr_languages.cpp"
        HEAP32[(($numbers203)>>2)]=96; //@line 631 "tr_languages.cpp"
        var $143=$tr; //@line 632 "tr_languages.cpp"
        var $langopts204=(($143)&4294967295); //@line 632 "tr_languages.cpp"
        var $param205=(($langopts204+24)&4294967295); //@line 632 "tr_languages.cpp"
        var $arrayidx206=(($param205+20)&4294967295); //@line 632 "tr_languages.cpp"
        HEAP32[(($arrayidx206)>>2)]=1; //@line 632 "tr_languages.cpp"
        __label__ = 80;break $sw_default$$sw_bb$$sw_bb9$$sw_bb18$$sw_bb24$$sw_bb39$$sw_bb52$$sw_bb64$$sw_bb72$$sw_bb92$$sw_bb109$$sw_bb123$$sw_bb148$$sw_bb161$$sw_bb195$$sw_bb200$$sw_bb207$$sw_bb209$$sw_bb222$$sw_bb238$$sw_bb257$$sw_bb280$$sw_bb287$$sw_bb312$$sw_bb322$$sw_bb331$$sw_bb347$$sw_bb381$$sw_bb391$$sw_bb403$$sw_bb414$$sw_bb422$$sw_bb439$$sw_bb449$$sw_bb461$$sw_bb481$$sw_bb486$$sw_bb491$$sw_bb509$$sw_bb523$$sw_bb533$$sw_bb534$$sw_bb547$$sw_bb576$$sw_bb599$$sw_bb614$$sw_bb623$$sw_bb630$$sw_bb645$$sw_bb692$$sw_bb702$$sw_bb711$$sw_bb729$$sw_bb734$6; //@line 634 "tr_languages.cpp"
      }
      else if (__label__ == 23) {
  
        var $144=HEAP32[((((_charsets+16)&4294967295))>>2)]; //@line 638 "tr_languages.cpp"
        var $145=$tr; //@line 638 "tr_languages.cpp"
        var $charset_a0208=(($145+696)&4294967295); //@line 638 "tr_languages.cpp"
        HEAP32[(($charset_a0208)>>2)]=$144; //@line 638 "tr_languages.cpp"
        __label__ = 24;break $sw_default$$sw_bb$$sw_bb9$$sw_bb18$$sw_bb24$$sw_bb39$$sw_bb52$$sw_bb64$$sw_bb72$$sw_bb92$$sw_bb109$$sw_bb123$$sw_bb148$$sw_bb161$$sw_bb195$$sw_bb200$$sw_bb207$$sw_bb209$$sw_bb222$$sw_bb238$$sw_bb257$$sw_bb280$$sw_bb287$$sw_bb312$$sw_bb322$$sw_bb331$$sw_bb347$$sw_bb381$$sw_bb391$$sw_bb403$$sw_bb414$$sw_bb422$$sw_bb439$$sw_bb449$$sw_bb461$$sw_bb481$$sw_bb486$$sw_bb491$$sw_bb509$$sw_bb523$$sw_bb533$$sw_bb534$$sw_bb547$$sw_bb576$$sw_bb599$$sw_bb614$$sw_bb623$$sw_bb630$$sw_bb645$$sw_bb692$$sw_bb702$$sw_bb711$$sw_bb729$$sw_bb734$6; //@line 638 "tr_languages.cpp"
      }
      else if (__label__ == 25) {
  
        var $155=$tr; //@line 662 "tr_languages.cpp"
        __Z15SetupTranslatorP10TranslatorPKsPKh($155, ((__ZZ16SelectTranslatorPKcE17stress_lengths_fr)&4294967295), ((__ZZ16SelectTranslatorPKcE14stress_amps_fr)&4294967295)); //@line 662 "tr_languages.cpp"
        var $156=$tr; //@line 663 "tr_languages.cpp"
        var $langopts223=(($156)&4294967295); //@line 663 "tr_languages.cpp"
        var $stress_rule224=(($langopts223+8)&4294967295); //@line 663 "tr_languages.cpp"
        HEAP32[(($stress_rule224)>>2)]=3; //@line 663 "tr_languages.cpp"
        var $157=$tr; //@line 664 "tr_languages.cpp"
        var $langopts225=(($157)&4294967295); //@line 664 "tr_languages.cpp"
        var $stress_flags226=(($langopts225+12)&4294967295); //@line 664 "tr_languages.cpp"
        HEAP32[(($stress_flags226)>>2)]=36; //@line 664 "tr_languages.cpp"
        var $158=$tr; //@line 665 "tr_languages.cpp"
        var $langopts227=(($158)&4294967295); //@line 665 "tr_languages.cpp"
        var $param228=(($langopts227+24)&4294967295); //@line 665 "tr_languages.cpp"
        var $arrayidx229=(($param228+8)&4294967295); //@line 665 "tr_languages.cpp"
        HEAP32[(($arrayidx229)>>2)]=1; //@line 665 "tr_languages.cpp"
        var $159=$tr; //@line 666 "tr_languages.cpp"
        var $langopts230=(($159)&4294967295); //@line 666 "tr_languages.cpp"
        var $length_mods231=(($langopts230+176)&4294967295); //@line 666 "tr_languages.cpp"
        var $160=HEAP32[(($length_mods231)>>2)]; //@line 666 "tr_languages.cpp"
        var $161=$tr; //@line 666 "tr_languages.cpp"
        var $langopts232=(($161)&4294967295); //@line 666 "tr_languages.cpp"
        var $length_mods0233=(($langopts232+180)&4294967295); //@line 666 "tr_languages.cpp"
        HEAP32[(($length_mods0233)>>2)]=$160; //@line 666 "tr_languages.cpp"
        var $162=$tr; //@line 667 "tr_languages.cpp"
        var $langopts234=(($162)&4294967295); //@line 667 "tr_languages.cpp"
        var $accents235=(($langopts234+220)&4294967295); //@line 667 "tr_languages.cpp"
        HEAP32[(($accents235)>>2)]=2; //@line 667 "tr_languages.cpp"
        var $163=$tr; //@line 669 "tr_languages.cpp"
        var $langopts236=(($163)&4294967295); //@line 669 "tr_languages.cpp"
        var $numbers237=(($langopts236+184)&4294967295); //@line 669 "tr_languages.cpp"
        HEAP32[(($numbers237)>>2)]=118658312; //@line 669 "tr_languages.cpp"
        var $164=$tr; //@line 670 "tr_languages.cpp"
        __ZL14SetLetterVowelP10Translatori($164, 121); //@line 670 "tr_languages.cpp"
        __label__ = 80;break $sw_default$$sw_bb$$sw_bb9$$sw_bb18$$sw_bb24$$sw_bb39$$sw_bb52$$sw_bb64$$sw_bb72$$sw_bb92$$sw_bb109$$sw_bb123$$sw_bb148$$sw_bb161$$sw_bb195$$sw_bb200$$sw_bb207$$sw_bb209$$sw_bb222$$sw_bb238$$sw_bb257$$sw_bb280$$sw_bb287$$sw_bb312$$sw_bb322$$sw_bb331$$sw_bb347$$sw_bb381$$sw_bb391$$sw_bb403$$sw_bb414$$sw_bb422$$sw_bb439$$sw_bb449$$sw_bb461$$sw_bb481$$sw_bb486$$sw_bb491$$sw_bb509$$sw_bb523$$sw_bb533$$sw_bb534$$sw_bb547$$sw_bb576$$sw_bb599$$sw_bb614$$sw_bb623$$sw_bb630$$sw_bb645$$sw_bb692$$sw_bb702$$sw_bb711$$sw_bb729$$sw_bb734$6; //@line 672 "tr_languages.cpp"
      }
      else if (__label__ == 26) {
  
        var $165=$tr; //@line 681 "tr_languages.cpp"
        __Z15SetupTranslatorP10TranslatorPKsPKh($165, ((__ZZ16SelectTranslatorPKcE17stress_lengths_hi)&4294967295), ((__ZZ16SelectTranslatorPKcE14stress_amps_hi)&4294967295)); //@line 681 "tr_languages.cpp"
        var $166=HEAP32[((((_charsets+76)&4294967295))>>2)]; //@line 682 "tr_languages.cpp"
        var $167=$tr; //@line 682 "tr_languages.cpp"
        var $charset_a0239=(($167+696)&4294967295); //@line 682 "tr_languages.cpp"
        HEAP32[(($charset_a0239)>>2)]=$166; //@line 682 "tr_languages.cpp"
        var $168=$tr; //@line 683 "tr_languages.cpp"
        var $langopts240=(($168)&4294967295); //@line 683 "tr_languages.cpp"
        var $length_mods241=(($langopts240+176)&4294967295); //@line 683 "tr_languages.cpp"
        var $169=HEAP32[(($length_mods241)>>2)]; //@line 683 "tr_languages.cpp"
        var $170=$tr; //@line 683 "tr_languages.cpp"
        var $langopts242=(($170)&4294967295); //@line 683 "tr_languages.cpp"
        var $length_mods0243=(($langopts242+180)&4294967295); //@line 683 "tr_languages.cpp"
        HEAP32[(($length_mods0243)>>2)]=$169; //@line 683 "tr_languages.cpp"
        var $171=$tr; //@line 685 "tr_languages.cpp"
        var $langopts244=(($171)&4294967295); //@line 685 "tr_languages.cpp"
        var $stress_rule245=(($langopts244+8)&4294967295); //@line 685 "tr_languages.cpp"
        HEAP32[(($stress_rule245)>>2)]=6; //@line 685 "tr_languages.cpp"
        var $172=$tr; //@line 686 "tr_languages.cpp"
        var $langopts246=(($172)&4294967295); //@line 686 "tr_languages.cpp"
        var $stress_flags247=(($langopts246+12)&4294967295); //@line 686 "tr_languages.cpp"
        HEAP32[(($stress_flags247)>>2)]=65540; //@line 686 "tr_languages.cpp"
        var $173=$tr; //@line 687 "tr_languages.cpp"
        var $langopts248=(($173)&4294967295); //@line 687 "tr_languages.cpp"
        var $numbers249=(($langopts248+184)&4294967295); //@line 687 "tr_languages.cpp"
        HEAP32[(($numbers249)>>2)]=16; //@line 687 "tr_languages.cpp"
        var $174=$tr; //@line 688 "tr_languages.cpp"
        var $langopts250=(($174)&4294967295); //@line 688 "tr_languages.cpp"
        var $break_numbers251=(($langopts250+192)&4294967295); //@line 688 "tr_languages.cpp"
        HEAP32[(($break_numbers251)>>2)]=84648; //@line 688 "tr_languages.cpp"
        var $175=$tr; //@line 689 "tr_languages.cpp"
        var $letter_bits_offset252=(($175+964)&4294967295); //@line 689 "tr_languages.cpp"
        HEAP32[(($letter_bits_offset252)>>2)]=2304; //@line 689 "tr_languages.cpp"
        var $176=$name2; //@line 691 "tr_languages.cpp"
        var $cmp253=((($176))|0)==28769; //@line 691 "tr_languages.cpp"
        if ($cmp253) { __label__ = 27;; } else { __label__ = 28;; } //@line 691 "tr_languages.cpp"
        if (__label__ == 27) {
  
          var $177=$tr; //@line 693 "tr_languages.cpp"
          var $letter_bits_offset255=(($177+964)&4294967295); //@line 693 "tr_languages.cpp"
          HEAP32[(($letter_bits_offset255)>>2)]=2560; //@line 693 "tr_languages.cpp"
          ; //@line 694 "tr_languages.cpp"
        }
  
        var $178=$tr; //@line 695 "tr_languages.cpp"
        __Z15SetIndicLettersP10Translator($178); //@line 695 "tr_languages.cpp"
        __label__ = 80;break $sw_default$$sw_bb$$sw_bb9$$sw_bb18$$sw_bb24$$sw_bb39$$sw_bb52$$sw_bb64$$sw_bb72$$sw_bb92$$sw_bb109$$sw_bb123$$sw_bb148$$sw_bb161$$sw_bb195$$sw_bb200$$sw_bb207$$sw_bb209$$sw_bb222$$sw_bb238$$sw_bb257$$sw_bb280$$sw_bb287$$sw_bb312$$sw_bb322$$sw_bb331$$sw_bb347$$sw_bb381$$sw_bb391$$sw_bb403$$sw_bb414$$sw_bb422$$sw_bb439$$sw_bb449$$sw_bb461$$sw_bb481$$sw_bb486$$sw_bb491$$sw_bb509$$sw_bb523$$sw_bb533$$sw_bb534$$sw_bb547$$sw_bb576$$sw_bb599$$sw_bb614$$sw_bb623$$sw_bb630$$sw_bb645$$sw_bb692$$sw_bb702$$sw_bb711$$sw_bb729$$sw_bb734$6; //@line 697 "tr_languages.cpp"
      }
      else if (__label__ == 29) {
  
        var $cmp258=((($7))|0)==29554; //@line 708 "tr_languages.cpp"
        var $179=$tr; //@line 709 "tr_languages.cpp"
        if ($cmp258) { __label__ = 30;; } else { __label__ = 31;; } //@line 708 "tr_languages.cpp"
        if (__label__ == 30) {
  
          __Z15SetupTranslatorP10TranslatorPKsPKh($179, ((__ZZ16SelectTranslatorPKcE17stress_lengths_sr)&4294967295), ((__ZZ16SelectTranslatorPKcE14stress_amps_hr)&4294967295)); //@line 709 "tr_languages.cpp"
          ; //@line 709 "tr_languages.cpp"
        }
        else if (__label__ == 31) {
  
          __Z15SetupTranslatorP10TranslatorPKsPKh($179, ((__ZZ16SelectTranslatorPKcE17stress_lengths_hr)&4294967295), ((__ZZ16SelectTranslatorPKcE14stress_amps_hr)&4294967295)); //@line 711 "tr_languages.cpp"
          ;
        }
  
        var $180=HEAP32[((((_charsets+8)&4294967295))>>2)]; //@line 712 "tr_languages.cpp"
        var $181=$tr; //@line 712 "tr_languages.cpp"
        var $charset_a0262=(($181+696)&4294967295); //@line 712 "tr_languages.cpp"
        HEAP32[(($charset_a0262)>>2)]=$180; //@line 712 "tr_languages.cpp"
        var $182=$tr; //@line 714 "tr_languages.cpp"
        var $langopts263=(($182)&4294967295); //@line 714 "tr_languages.cpp"
        var $stress_rule264=(($langopts263+8)&4294967295); //@line 714 "tr_languages.cpp"
        HEAP32[(($stress_rule264)>>2)]=0; //@line 714 "tr_languages.cpp"
        var $183=$tr; //@line 715 "tr_languages.cpp"
        var $langopts265=(($183)&4294967295); //@line 715 "tr_languages.cpp"
        var $stress_flags266=(($langopts265+12)&4294967295); //@line 715 "tr_languages.cpp"
        HEAP32[(($stress_flags266)>>2)]=16; //@line 715 "tr_languages.cpp"
        var $184=$tr; //@line 716 "tr_languages.cpp"
        var $langopts267=(($184)&4294967295); //@line 716 "tr_languages.cpp"
        var $param268=(($langopts267+24)&4294967295); //@line 716 "tr_languages.cpp"
        var $arrayidx269=(($param268+16)&4294967295); //@line 716 "tr_languages.cpp"
        HEAP32[(($arrayidx269)>>2)]=3; //@line 716 "tr_languages.cpp"
        var $185=$tr; //@line 717 "tr_languages.cpp"
        var $langopts270=(($185)&4294967295); //@line 717 "tr_languages.cpp"
        var $max_initial_consonants=(($langopts270+248)&4294967295); //@line 717 "tr_languages.cpp"
        HEAP8[($max_initial_consonants)]=5; //@line 717 "tr_languages.cpp"
        var $186=$tr; //@line 718 "tr_languages.cpp"
        var $langopts271=(($186)&4294967295); //@line 718 "tr_languages.cpp"
        var $spelling_stress272=(($langopts271+249)&4294967295); //@line 718 "tr_languages.cpp"
        HEAP8[($spelling_stress272)]=1; //@line 718 "tr_languages.cpp"
        var $187=$tr; //@line 719 "tr_languages.cpp"
        var $langopts273=(($187)&4294967295); //@line 719 "tr_languages.cpp"
        var $accents274=(($langopts273+220)&4294967295); //@line 719 "tr_languages.cpp"
        HEAP32[(($accents274)>>2)]=1; //@line 719 "tr_languages.cpp"
        var $188=$tr; //@line 721 "tr_languages.cpp"
        var $langopts275=(($188)&4294967295); //@line 721 "tr_languages.cpp"
        var $numbers276=(($langopts275+184)&4294967295); //@line 721 "tr_languages.cpp"
        HEAP32[(($numbers276)>>2)]=33572172; //@line 721 "tr_languages.cpp"
        var $189=$tr; //@line 722 "tr_languages.cpp"
        var $langopts277=(($189)&4294967295); //@line 722 "tr_languages.cpp"
        var $numbers2278=(($langopts277+188)&4294967295); //@line 722 "tr_languages.cpp"
        HEAP32[(($numbers2278)>>2)]=74; //@line 722 "tr_languages.cpp"
        var $190=$tr; //@line 723 "tr_languages.cpp"
        var $langopts279=(($190)&4294967295); //@line 723 "tr_languages.cpp"
        var $replace_chars=(($langopts279+264)&4294967295); //@line 723 "tr_languages.cpp"
        HEAP32[(($replace_chars)>>2)]=((__ZL22replace_cyrillic_latin)&4294967295); //@line 723 "tr_languages.cpp"
        var $191=$tr; //@line 725 "tr_languages.cpp"
        __ZL14SetLetterVowelP10Translatori($191, 121); //@line 725 "tr_languages.cpp"
        var $192=$tr; //@line 726 "tr_languages.cpp"
        __ZL14SetLetterVowelP10Translatori($192, 114); //@line 726 "tr_languages.cpp"
        __label__ = 80;break $sw_default$$sw_bb$$sw_bb9$$sw_bb18$$sw_bb24$$sw_bb39$$sw_bb52$$sw_bb64$$sw_bb72$$sw_bb92$$sw_bb109$$sw_bb123$$sw_bb148$$sw_bb161$$sw_bb195$$sw_bb200$$sw_bb207$$sw_bb209$$sw_bb222$$sw_bb238$$sw_bb257$$sw_bb280$$sw_bb287$$sw_bb312$$sw_bb322$$sw_bb331$$sw_bb347$$sw_bb381$$sw_bb391$$sw_bb403$$sw_bb414$$sw_bb422$$sw_bb439$$sw_bb449$$sw_bb461$$sw_bb481$$sw_bb486$$sw_bb491$$sw_bb509$$sw_bb523$$sw_bb533$$sw_bb534$$sw_bb547$$sw_bb576$$sw_bb599$$sw_bb614$$sw_bb623$$sw_bb630$$sw_bb645$$sw_bb692$$sw_bb702$$sw_bb711$$sw_bb729$$sw_bb734$6; //@line 728 "tr_languages.cpp"
      }
      else if (__label__ == 33) {
  
        var $193=$tr; //@line 733 "tr_languages.cpp"
        var $langopts281=(($193)&4294967295); //@line 733 "tr_languages.cpp"
        var $stress_rule282=(($langopts281+8)&4294967295); //@line 733 "tr_languages.cpp"
        HEAP32[(($stress_rule282)>>2)]=3; //@line 733 "tr_languages.cpp"
        var $194=$tr; //@line 734 "tr_languages.cpp"
        var $langopts283=(($194)&4294967295); //@line 734 "tr_languages.cpp"
        var $stress_flags284=(($langopts283+12)&4294967295); //@line 734 "tr_languages.cpp"
        HEAP32[(($stress_flags284)>>2)]=36; //@line 734 "tr_languages.cpp"
        var $195=$tr; //@line 735 "tr_languages.cpp"
        var $langopts285=(($195)&4294967295); //@line 735 "tr_languages.cpp"
        var $numbers286=(($langopts285+184)&4294967295); //@line 735 "tr_languages.cpp"
        HEAP32[(($numbers286)>>2)]=17990912; //@line 735 "tr_languages.cpp"
        __label__ = 80;break $sw_default$$sw_bb$$sw_bb9$$sw_bb18$$sw_bb24$$sw_bb39$$sw_bb52$$sw_bb64$$sw_bb72$$sw_bb92$$sw_bb109$$sw_bb123$$sw_bb148$$sw_bb161$$sw_bb195$$sw_bb200$$sw_bb207$$sw_bb209$$sw_bb222$$sw_bb238$$sw_bb257$$sw_bb280$$sw_bb287$$sw_bb312$$sw_bb322$$sw_bb331$$sw_bb347$$sw_bb381$$sw_bb391$$sw_bb403$$sw_bb414$$sw_bb422$$sw_bb439$$sw_bb449$$sw_bb461$$sw_bb481$$sw_bb486$$sw_bb491$$sw_bb509$$sw_bb523$$sw_bb533$$sw_bb534$$sw_bb547$$sw_bb576$$sw_bb599$$sw_bb614$$sw_bb623$$sw_bb630$$sw_bb645$$sw_bb692$$sw_bb702$$sw_bb711$$sw_bb729$$sw_bb734$6; //@line 736 "tr_languages.cpp"
      }
      else if (__label__ == 34) {
  
        var $196=$tr; //@line 744 "tr_languages.cpp"
        __Z15SetupTranslatorP10TranslatorPKsPKh($196, ((__ZZ16SelectTranslatorPKcE17stress_lengths_hu)&4294967295), ((__ZZ16SelectTranslatorPKcE14stress_amps_hu)&4294967295)); //@line 744 "tr_languages.cpp"
        var $197=HEAP32[((((_charsets+8)&4294967295))>>2)]; //@line 745 "tr_languages.cpp"
        var $198=$tr; //@line 745 "tr_languages.cpp"
        var $charset_a0288=(($198+696)&4294967295); //@line 745 "tr_languages.cpp"
        HEAP32[(($charset_a0288)>>2)]=$197; //@line 745 "tr_languages.cpp"
        var $199=$tr; //@line 747 "tr_languages.cpp"
        var $langopts289=(($199)&4294967295); //@line 747 "tr_languages.cpp"
        var $vowel_pause290=(($langopts289+4)&4294967295); //@line 747 "tr_languages.cpp"
        HEAP32[(($vowel_pause290)>>2)]=32; //@line 747 "tr_languages.cpp"
        var $200=$tr; //@line 748 "tr_languages.cpp"
        var $langopts291=(($200)&4294967295); //@line 748 "tr_languages.cpp"
        var $stress_rule292=(($langopts291+8)&4294967295); //@line 748 "tr_languages.cpp"
        HEAP32[(($stress_rule292)>>2)]=0; //@line 748 "tr_languages.cpp"
        var $201=$tr; //@line 749 "tr_languages.cpp"
        var $langopts293=(($201)&4294967295); //@line 749 "tr_languages.cpp"
        var $stress_flags294=(($langopts293+12)&4294967295); //@line 749 "tr_languages.cpp"
        HEAP32[(($stress_flags294)>>2)]=1081398; //@line 749 "tr_languages.cpp"
        var $202=$tr; //@line 750 "tr_languages.cpp"
        var $langopts295=(($202)&4294967295); //@line 750 "tr_languages.cpp"
        var $unstressed_wd1296=(($langopts295+16)&4294967295); //@line 750 "tr_languages.cpp"
        HEAP32[(($unstressed_wd1296)>>2)]=2; //@line 750 "tr_languages.cpp"
        var $203=$tr; //@line 751 "tr_languages.cpp"
        var $langopts297=(($203)&4294967295); //@line 751 "tr_languages.cpp"
        var $param298=(($langopts297+24)&4294967295); //@line 751 "tr_languages.cpp"
        var $arrayidx299=(($param298+56)&4294967295); //@line 751 "tr_languages.cpp"
        HEAP32[(($arrayidx299)>>2)]=1; //@line 751 "tr_languages.cpp"
        var $204=$tr; //@line 752 "tr_languages.cpp"
        var $langopts300=(($204)&4294967295); //@line 752 "tr_languages.cpp"
        var $param301=(($langopts300+24)&4294967295); //@line 752 "tr_languages.cpp"
        var $arrayidx302=(($param301+68)&4294967295); //@line 752 "tr_languages.cpp"
        HEAP32[(($arrayidx302)>>2)]=2; //@line 752 "tr_languages.cpp"
        var $205=$tr; //@line 754 "tr_languages.cpp"
        var $langopts303=(($205)&4294967295); //@line 754 "tr_languages.cpp"
        var $numbers304=(($langopts303+184)&4294967295); //@line 754 "tr_languages.cpp"
        HEAP32[(($numbers304)>>2)]=186758144; //@line 754 "tr_languages.cpp"
        var $206=$tr; //@line 755 "tr_languages.cpp"
        var $langopts305=(($206)&4294967295); //@line 755 "tr_languages.cpp"
        var $thousands_sep306=(($langopts305+204)&4294967295); //@line 755 "tr_languages.cpp"
        HEAP32[(($thousands_sep306)>>2)]=32; //@line 755 "tr_languages.cpp"
        var $207=$tr; //@line 756 "tr_languages.cpp"
        var $langopts307=(($207)&4294967295); //@line 756 "tr_languages.cpp"
        var $decimal_sep=(($langopts307+208)&4294967295); //@line 756 "tr_languages.cpp"
        HEAP32[(($decimal_sep)>>2)]=44; //@line 756 "tr_languages.cpp"
        var $208=$tr; //@line 757 "tr_languages.cpp"
        var $langopts308=(($208)&4294967295); //@line 757 "tr_languages.cpp"
        var $max_roman=(($langopts308+196)&4294967295); //@line 757 "tr_languages.cpp"
        HEAP32[(($max_roman)>>2)]=899; //@line 757 "tr_languages.cpp"
        var $209=$tr; //@line 758 "tr_languages.cpp"
        var $langopts309=(($209)&4294967295); //@line 758 "tr_languages.cpp"
        var $min_roman=(($langopts309+200)&4294967295); //@line 758 "tr_languages.cpp"
        HEAP32[(($min_roman)>>2)]=1; //@line 758 "tr_languages.cpp"
        var $210=$tr; //@line 759 "tr_languages.cpp"
        __ZL14SetLetterVowelP10Translatori($210, 121); //@line 759 "tr_languages.cpp"
        var $211=$tr; //@line 760 "tr_languages.cpp"
        var $langopts310=(($211)&4294967295); //@line 760 "tr_languages.cpp"
        var $spelling_stress311=(($langopts310+249)&4294967295); //@line 760 "tr_languages.cpp"
        HEAP8[($spelling_stress311)]=1; //@line 760 "tr_languages.cpp"
        var $212=$tr; //@line 761 "tr_languages.cpp"
        __Z13SetLengthModsP10Translatori($212, 3); //@line 761 "tr_languages.cpp"
        __label__ = 80;break $sw_default$$sw_bb$$sw_bb9$$sw_bb18$$sw_bb24$$sw_bb39$$sw_bb52$$sw_bb64$$sw_bb72$$sw_bb92$$sw_bb109$$sw_bb123$$sw_bb148$$sw_bb161$$sw_bb195$$sw_bb200$$sw_bb207$$sw_bb209$$sw_bb222$$sw_bb238$$sw_bb257$$sw_bb280$$sw_bb287$$sw_bb312$$sw_bb322$$sw_bb331$$sw_bb347$$sw_bb381$$sw_bb391$$sw_bb403$$sw_bb414$$sw_bb422$$sw_bb439$$sw_bb449$$sw_bb461$$sw_bb481$$sw_bb486$$sw_bb491$$sw_bb509$$sw_bb523$$sw_bb533$$sw_bb534$$sw_bb547$$sw_bb576$$sw_bb599$$sw_bb614$$sw_bb623$$sw_bb630$$sw_bb645$$sw_bb692$$sw_bb702$$sw_bb711$$sw_bb729$$sw_bb734$6; //@line 763 "tr_languages.cpp"
      }
      else if (__label__ == 35) {
  
        var $213=$tr; //@line 772 "tr_languages.cpp"
        __Z15SetupTranslatorP10TranslatorPKsPKh($213, ((__ZZ16SelectTranslatorPKcE17stress_lengths_hy)&4294967295), 0); //@line 772 "tr_languages.cpp"
        var $214=$tr; //@line 773 "tr_languages.cpp"
        var $langopts313=(($214)&4294967295); //@line 773 "tr_languages.cpp"
        var $stress_rule314=(($langopts313+8)&4294967295); //@line 773 "tr_languages.cpp"
        HEAP32[(($stress_rule314)>>2)]=3; //@line 773 "tr_languages.cpp"
        var $215=$tr; //@line 775 "tr_languages.cpp"
        var $letter_bits_offset315=(($215+964)&4294967295); //@line 775 "tr_languages.cpp"
        HEAP32[(($letter_bits_offset315)>>2)]=1328; //@line 775 "tr_languages.cpp"
        var $216=$tr; //@line 776 "tr_languages.cpp"
        var $letter_bits316=(($216+708)&4294967295); //@line 776 "tr_languages.cpp"
        var $arraydecay317=(($letter_bits316)&4294967295); //@line 776 "tr_languages.cpp"
        var $dest$, $stop$, $stop4$, $fast$, $value4$;
        $dest$ = $arraydecay317;
        $stop$ = $dest$ + 256;
        $value4$ = 0;
        if ($value4$ < 0) $value4$ += 256;
        $value4$ = $value4$ + ($value4$<<8) + ($value4$<<16) + ($value4$*16777216);
        while ($dest$%4 !== 0 && $dest$ < $stop$) {
          ;; HEAP8[$dest$++] = 0;
        }
        $dest$ >>= 2;
        $stop4$ = $stop$ >> 2;
        while ($dest$ < $stop4$) {
        ;;;;;;;  HEAP32[$dest$++] = $value4$;
        }
        $dest$ <<= 2;
        while ($dest$ < $stop$) {
          ;; HEAP8[$dest$++] = 0;
        }; //@line 776 "tr_languages.cpp"
        var $217=$tr; //@line 777 "tr_languages.cpp"
        __ZL13SetLetterBitsP10TranslatoriPKc($217, 0, ((__ZZ16SelectTranslatorPKcE9hy_vowels)&4294967295)); //@line 777 "tr_languages.cpp"
        var $218=$tr; //@line 778 "tr_languages.cpp"
        __ZL13SetLetterBitsP10TranslatoriPKc($218, 7, ((__ZZ16SelectTranslatorPKcE9hy_vowels)&4294967295)); //@line 778 "tr_languages.cpp"
        var $219=$tr; //@line 779 "tr_languages.cpp"
        __ZL13SetLetterBitsP10TranslatoriPKc($219, 2, ((__ZZ16SelectTranslatorPKcE13hy_consonants)&4294967295)); //@line 779 "tr_languages.cpp"
        var $220=$tr; //@line 780 "tr_languages.cpp"
        var $langopts318=(($220)&4294967295); //@line 780 "tr_languages.cpp"
        var $max_initial_consonants319=(($langopts318+248)&4294967295); //@line 780 "tr_languages.cpp"
        HEAP8[($max_initial_consonants319)]=6; //@line 780 "tr_languages.cpp"
        var $221=$tr; //@line 781 "tr_languages.cpp"
        var $langopts320=(($221)&4294967295); //@line 781 "tr_languages.cpp"
        var $numbers321=(($langopts320+184)&4294967295); //@line 781 "tr_languages.cpp"
        HEAP32[(($numbers321)>>2)]=1032; //@line 781 "tr_languages.cpp"
        __label__ = 80;break $sw_default$$sw_bb$$sw_bb9$$sw_bb18$$sw_bb24$$sw_bb39$$sw_bb52$$sw_bb64$$sw_bb72$$sw_bb92$$sw_bb109$$sw_bb123$$sw_bb148$$sw_bb161$$sw_bb195$$sw_bb200$$sw_bb207$$sw_bb209$$sw_bb222$$sw_bb238$$sw_bb257$$sw_bb280$$sw_bb287$$sw_bb312$$sw_bb322$$sw_bb331$$sw_bb347$$sw_bb381$$sw_bb391$$sw_bb403$$sw_bb414$$sw_bb422$$sw_bb439$$sw_bb449$$sw_bb461$$sw_bb481$$sw_bb486$$sw_bb491$$sw_bb509$$sw_bb523$$sw_bb533$$sw_bb534$$sw_bb547$$sw_bb576$$sw_bb599$$sw_bb614$$sw_bb623$$sw_bb630$$sw_bb645$$sw_bb692$$sw_bb702$$sw_bb711$$sw_bb729$$sw_bb734$6; //@line 784 "tr_languages.cpp"
      }
      else if (__label__ == 36) {
  
        var $222=$tr; //@line 791 "tr_languages.cpp"
        __Z15SetupTranslatorP10TranslatorPKsPKh($222, ((__ZZ16SelectTranslatorPKcE17stress_lengths_id)&4294967295), ((__ZZ16SelectTranslatorPKcE14stress_amps_id)&4294967295)); //@line 791 "tr_languages.cpp"
        var $223=$tr; //@line 792 "tr_languages.cpp"
        var $langopts323=(($223)&4294967295); //@line 792 "tr_languages.cpp"
        var $stress_rule324=(($langopts323+8)&4294967295); //@line 792 "tr_languages.cpp"
        HEAP32[(($stress_rule324)>>2)]=2; //@line 792 "tr_languages.cpp"
        var $224=$tr; //@line 793 "tr_languages.cpp"
        var $langopts325=(($224)&4294967295); //@line 793 "tr_languages.cpp"
        var $numbers326=(($langopts325+184)&4294967295); //@line 793 "tr_languages.cpp"
        HEAP32[(($numbers326)>>2)]=16781320; //@line 793 "tr_languages.cpp"
        var $225=$tr; //@line 794 "tr_languages.cpp"
        var $langopts327=(($225)&4294967295); //@line 794 "tr_languages.cpp"
        var $stress_flags328=(($langopts327+12)&4294967295); //@line 794 "tr_languages.cpp"
        HEAP32[(($stress_flags328)>>2)]=22; //@line 794 "tr_languages.cpp"
        var $226=$tr; //@line 795 "tr_languages.cpp"
        var $langopts329=(($226)&4294967295); //@line 795 "tr_languages.cpp"
        var $accents330=(($langopts329+220)&4294967295); //@line 795 "tr_languages.cpp"
        HEAP32[(($accents330)>>2)]=2; //@line 795 "tr_languages.cpp"
        __label__ = 80;break $sw_default$$sw_bb$$sw_bb9$$sw_bb18$$sw_bb24$$sw_bb39$$sw_bb52$$sw_bb64$$sw_bb72$$sw_bb92$$sw_bb109$$sw_bb123$$sw_bb148$$sw_bb161$$sw_bb195$$sw_bb200$$sw_bb207$$sw_bb209$$sw_bb222$$sw_bb238$$sw_bb257$$sw_bb280$$sw_bb287$$sw_bb312$$sw_bb322$$sw_bb331$$sw_bb347$$sw_bb381$$sw_bb391$$sw_bb403$$sw_bb414$$sw_bb422$$sw_bb439$$sw_bb449$$sw_bb461$$sw_bb481$$sw_bb486$$sw_bb491$$sw_bb509$$sw_bb523$$sw_bb533$$sw_bb534$$sw_bb547$$sw_bb576$$sw_bb599$$sw_bb614$$sw_bb623$$sw_bb630$$sw_bb645$$sw_bb692$$sw_bb702$$sw_bb711$$sw_bb729$$sw_bb734$6; //@line 797 "tr_languages.cpp"
      }
      else if (__label__ == 37) {
  
        var $227=$tr; //@line 804 "tr_languages.cpp"
        __Z15SetupTranslatorP10TranslatorPKsPKh($227, ((__ZZ16SelectTranslatorPKcE17stress_lengths_is)&4294967295), 0); //@line 804 "tr_languages.cpp"
        var $228=$tr; //@line 805 "tr_languages.cpp"
        var $langopts332=(($228)&4294967295); //@line 805 "tr_languages.cpp"
        var $stress_rule333=(($langopts332+8)&4294967295); //@line 805 "tr_languages.cpp"
        HEAP32[(($stress_rule333)>>2)]=0; //@line 805 "tr_languages.cpp"
        var $229=$tr; //@line 806 "tr_languages.cpp"
        var $langopts334=(($229)&4294967295); //@line 806 "tr_languages.cpp"
        var $stress_flags335=(($langopts334+12)&4294967295); //@line 806 "tr_languages.cpp"
        HEAP32[(($stress_flags335)>>2)]=16; //@line 806 "tr_languages.cpp"
        var $230=$tr; //@line 807 "tr_languages.cpp"
        var $langopts336=(($230)&4294967295); //@line 807 "tr_languages.cpp"
        var $param337=(($langopts336+24)&4294967295); //@line 807 "tr_languages.cpp"
        var $arrayidx338=(($param337+8)&4294967295); //@line 807 "tr_languages.cpp"
        HEAP32[(($arrayidx338)>>2)]=17; //@line 807 "tr_languages.cpp"
        var $231=$tr; //@line 808 "tr_languages.cpp"
        var $langopts339=(($231)&4294967295); //@line 808 "tr_languages.cpp"
        var $param340=(($langopts339+24)&4294967295); //@line 808 "tr_languages.cpp"
        var $arrayidx341=(($param340+40)&4294967295); //@line 808 "tr_languages.cpp"
        HEAP32[(($arrayidx341)>>2)]=2; //@line 808 "tr_languages.cpp"
        var $232=$tr; //@line 810 "tr_languages.cpp"
        __ZL15ResetLetterBitsP10Translatori($232, 24); //@line 810 "tr_languages.cpp"
        var $233=$tr; //@line 811 "tr_languages.cpp"
        __ZL13SetLetterBitsP10TranslatoriPKc($233, 4, ((__str1631)&4294967295)); //@line 811 "tr_languages.cpp"
        var $234=$tr; //@line 812 "tr_languages.cpp"
        __ZL13SetLetterBitsP10TranslatoriPKc($234, 3, ((__str2632)&4294967295)); //@line 812 "tr_languages.cpp"
        var $235=$tr; //@line 813 "tr_languages.cpp"
        var $letter_groups=(($235+968)&4294967295); //@line 813 "tr_languages.cpp"
        var $arrayidx342=(($letter_groups+4)&4294967295); //@line 813 "tr_languages.cpp"
        HEAP32[(($arrayidx342)>>2)]=((__ZZ16SelectTranslatorPKcE16is_lettergroup_B)&4294967295); //@line 813 "tr_languages.cpp"
        var $236=$tr; //@line 814 "tr_languages.cpp"
        __ZL14SetLetterVowelP10Translatori($236, 121); //@line 814 "tr_languages.cpp"
        var $237=$tr; //@line 815 "tr_languages.cpp"
        var $langopts343=(($237)&4294967295); //@line 815 "tr_languages.cpp"
        var $numbers344=(($langopts343+184)&4294967295); //@line 815 "tr_languages.cpp"
        HEAP32[(($numbers344)>>2)]=2280; //@line 815 "tr_languages.cpp"
        var $238=$tr; //@line 816 "tr_languages.cpp"
        var $langopts345=(($238)&4294967295); //@line 816 "tr_languages.cpp"
        var $numbers2346=(($langopts345+188)&4294967295); //@line 816 "tr_languages.cpp"
        HEAP32[(($numbers2346)>>2)]=2; //@line 816 "tr_languages.cpp"
        __label__ = 80;break $sw_default$$sw_bb$$sw_bb9$$sw_bb18$$sw_bb24$$sw_bb39$$sw_bb52$$sw_bb64$$sw_bb72$$sw_bb92$$sw_bb109$$sw_bb123$$sw_bb148$$sw_bb161$$sw_bb195$$sw_bb200$$sw_bb207$$sw_bb209$$sw_bb222$$sw_bb238$$sw_bb257$$sw_bb280$$sw_bb287$$sw_bb312$$sw_bb322$$sw_bb331$$sw_bb347$$sw_bb381$$sw_bb391$$sw_bb403$$sw_bb414$$sw_bb422$$sw_bb439$$sw_bb449$$sw_bb461$$sw_bb481$$sw_bb486$$sw_bb491$$sw_bb509$$sw_bb523$$sw_bb533$$sw_bb534$$sw_bb547$$sw_bb576$$sw_bb599$$sw_bb614$$sw_bb623$$sw_bb630$$sw_bb645$$sw_bb692$$sw_bb702$$sw_bb711$$sw_bb729$$sw_bb734$6; //@line 818 "tr_languages.cpp"
      }
      else if (__label__ == 38) {
  
        var $239=$tr; //@line 825 "tr_languages.cpp"
        __Z15SetupTranslatorP10TranslatorPKsPKh($239, ((__ZZ16SelectTranslatorPKcE17stress_lengths_it)&4294967295), ((__ZZ16SelectTranslatorPKcE14stress_amps_it)&4294967295)); //@line 825 "tr_languages.cpp"
        var $240=$tr; //@line 827 "tr_languages.cpp"
        var $langopts348=(($240)&4294967295); //@line 827 "tr_languages.cpp"
        var $length_mods349=(($langopts348+176)&4294967295); //@line 827 "tr_languages.cpp"
        var $241=HEAP32[(($length_mods349)>>2)]; //@line 827 "tr_languages.cpp"
        var $242=$tr; //@line 827 "tr_languages.cpp"
        var $langopts350=(($242)&4294967295); //@line 827 "tr_languages.cpp"
        var $length_mods0351=(($langopts350+180)&4294967295); //@line 827 "tr_languages.cpp"
        HEAP32[(($length_mods0351)>>2)]=$241; //@line 827 "tr_languages.cpp"
        var $243=$tr; //@line 828 "tr_languages.cpp"
        var $langopts352=(($243)&4294967295); //@line 828 "tr_languages.cpp"
        var $stress_rule353=(($langopts352+8)&4294967295); //@line 828 "tr_languages.cpp"
        HEAP32[(($stress_rule353)>>2)]=2; //@line 828 "tr_languages.cpp"
        var $244=$tr; //@line 829 "tr_languages.cpp"
        var $langopts354=(($244)&4294967295); //@line 829 "tr_languages.cpp"
        var $stress_flags355=(($langopts354+12)&4294967295); //@line 829 "tr_languages.cpp"
        HEAP32[(($stress_flags355)>>2)]=131088; //@line 829 "tr_languages.cpp"
        var $245=$tr; //@line 830 "tr_languages.cpp"
        var $langopts356=(($245)&4294967295); //@line 830 "tr_languages.cpp"
        var $vowel_pause357=(($langopts356+4)&4294967295); //@line 830 "tr_languages.cpp"
        HEAP32[(($vowel_pause357)>>2)]=1; //@line 830 "tr_languages.cpp"
        var $246=$tr; //@line 831 "tr_languages.cpp"
        var $langopts358=(($246)&4294967295); //@line 831 "tr_languages.cpp"
        var $unstressed_wd1359=(($langopts358+16)&4294967295); //@line 831 "tr_languages.cpp"
        HEAP32[(($unstressed_wd1359)>>2)]=2; //@line 831 "tr_languages.cpp"
        var $247=$tr; //@line 832 "tr_languages.cpp"
        var $langopts360=(($247)&4294967295); //@line 832 "tr_languages.cpp"
        var $unstressed_wd2361=(($langopts360+20)&4294967295); //@line 832 "tr_languages.cpp"
        HEAP32[(($unstressed_wd2361)>>2)]=2; //@line 832 "tr_languages.cpp"
        var $248=$tr; //@line 833 "tr_languages.cpp"
        var $langopts362=(($248)&4294967295); //@line 833 "tr_languages.cpp"
        var $param363=(($langopts362+24)&4294967295); //@line 833 "tr_languages.cpp"
        var $arrayidx364=(($param363+8)&4294967295); //@line 833 "tr_languages.cpp"
        HEAP32[(($arrayidx364)>>2)]=2; //@line 833 "tr_languages.cpp"
        var $249=$tr; //@line 834 "tr_languages.cpp"
        var $langopts365=(($249)&4294967295); //@line 834 "tr_languages.cpp"
        var $param366=(($langopts365+24)&4294967295); //@line 834 "tr_languages.cpp"
        var $arrayidx367=(($param366+56)&4294967295); //@line 834 "tr_languages.cpp"
        HEAP32[(($arrayidx367)>>2)]=2; //@line 834 "tr_languages.cpp"
        var $250=$tr; //@line 835 "tr_languages.cpp"
        var $langopts368=(($250)&4294967295); //@line 835 "tr_languages.cpp"
        var $param369=(($langopts368+24)&4294967295); //@line 835 "tr_languages.cpp"
        var $arrayidx370=(($param369+28)&4294967295); //@line 835 "tr_languages.cpp"
        HEAP32[(($arrayidx370)>>2)]=130; //@line 835 "tr_languages.cpp"
        var $251=$tr; //@line 836 "tr_languages.cpp"
        var $langopts371=(($251)&4294967295); //@line 836 "tr_languages.cpp"
        var $param372=(($langopts371+24)&4294967295); //@line 836 "tr_languages.cpp"
        var $arrayidx373=(($param372+40)&4294967295); //@line 836 "tr_languages.cpp"
        HEAP32[(($arrayidx373)>>2)]=1; //@line 836 "tr_languages.cpp"
        var $252=$tr; //@line 837 "tr_languages.cpp"
        var $langopts374=(($252)&4294967295); //@line 837 "tr_languages.cpp"
        var $param375=(($langopts374+24)&4294967295); //@line 837 "tr_languages.cpp"
        var $arrayidx376=(($param375+60)&4294967295); //@line 837 "tr_languages.cpp"
        HEAP32[(($arrayidx376)>>2)]=2; //@line 837 "tr_languages.cpp"
        var $253=$tr; //@line 838 "tr_languages.cpp"
        var $langopts377=(($253)&4294967295); //@line 838 "tr_languages.cpp"
        var $numbers378=(($langopts377+184)&4294967295); //@line 838 "tr_languages.cpp"
        HEAP32[(($numbers378)>>2)]=16786952; //@line 838 "tr_languages.cpp"
        var $254=$tr; //@line 839 "tr_languages.cpp"
        var $langopts379=(($254)&4294967295); //@line 839 "tr_languages.cpp"
        var $accents380=(($langopts379+220)&4294967295); //@line 839 "tr_languages.cpp"
        HEAP32[(($accents380)>>2)]=2; //@line 839 "tr_languages.cpp"
        var $255=$tr; //@line 840 "tr_languages.cpp"
        __ZL14SetLetterVowelP10Translatori($255, 121); //@line 840 "tr_languages.cpp"
        __label__ = 80;break $sw_default$$sw_bb$$sw_bb9$$sw_bb18$$sw_bb24$$sw_bb39$$sw_bb52$$sw_bb64$$sw_bb72$$sw_bb92$$sw_bb109$$sw_bb123$$sw_bb148$$sw_bb161$$sw_bb195$$sw_bb200$$sw_bb207$$sw_bb209$$sw_bb222$$sw_bb238$$sw_bb257$$sw_bb280$$sw_bb287$$sw_bb312$$sw_bb322$$sw_bb331$$sw_bb347$$sw_bb381$$sw_bb391$$sw_bb403$$sw_bb414$$sw_bb422$$sw_bb439$$sw_bb449$$sw_bb461$$sw_bb481$$sw_bb486$$sw_bb491$$sw_bb509$$sw_bb523$$sw_bb533$$sw_bb534$$sw_bb547$$sw_bb576$$sw_bb599$$sw_bb614$$sw_bb623$$sw_bb630$$sw_bb645$$sw_bb692$$sw_bb702$$sw_bb711$$sw_bb729$$sw_bb734$6; //@line 842 "tr_languages.cpp"
      }
      else if (__label__ == 39) {
  
        var $256=$tr; //@line 849 "tr_languages.cpp"
        __Z15SetupTranslatorP10TranslatorPKsPKh($256, ((__ZZ16SelectTranslatorPKcE18stress_lengths_jbo)&4294967295), 0); //@line 849 "tr_languages.cpp"
        var $257=$tr; //@line 850 "tr_languages.cpp"
        var $langopts382=(($257)&4294967295); //@line 850 "tr_languages.cpp"
        var $stress_rule383=(($langopts382+8)&4294967295); //@line 850 "tr_languages.cpp"
        HEAP32[(($stress_rule383)>>2)]=2; //@line 850 "tr_languages.cpp"
        var $258=$tr; //@line 851 "tr_languages.cpp"
        var $langopts384=(($258)&4294967295); //@line 851 "tr_languages.cpp"
        var $vowel_pause385=(($langopts384+4)&4294967295); //@line 851 "tr_languages.cpp"
        HEAP32[(($vowel_pause385)>>2)]=524; //@line 851 "tr_languages.cpp"
        var $259=$tr; //@line 853 "tr_languages.cpp"
        var $punct_within_word386=(($259+704)&4294967295); //@line 853 "tr_languages.cpp"
        HEAP32[(($punct_within_word386)>>2)]=((__ZZ16SelectTranslatorPKcE21jbo_punct_within_word)&4294967295); //@line 853 "tr_languages.cpp"
        var $260=$tr; //@line 854 "tr_languages.cpp"
        var $langopts387=(($260)&4294967295); //@line 854 "tr_languages.cpp"
        var $param388=(($langopts387+24)&4294967295); //@line 854 "tr_languages.cpp"
        var $arrayidx389=(($param388+52)&4294967295); //@line 854 "tr_languages.cpp"
        HEAP32[(($arrayidx389)>>2)]=2; //@line 854 "tr_languages.cpp"
        var $261=$tr; //@line 855 "tr_languages.cpp"
        __ZL14SetLetterVowelP10Translatori($261, 121); //@line 855 "tr_languages.cpp"
        var $262=$tr; //@line 856 "tr_languages.cpp"
        var $langopts390=(($262)&4294967295); //@line 856 "tr_languages.cpp"
        var $max_lengthmod=(($langopts390+272)&4294967295); //@line 856 "tr_languages.cpp"
        HEAP32[(($max_lengthmod)>>2)]=368; //@line 856 "tr_languages.cpp"
        __label__ = 80;break $sw_default$$sw_bb$$sw_bb9$$sw_bb18$$sw_bb24$$sw_bb39$$sw_bb52$$sw_bb64$$sw_bb72$$sw_bb92$$sw_bb109$$sw_bb123$$sw_bb148$$sw_bb161$$sw_bb195$$sw_bb200$$sw_bb207$$sw_bb209$$sw_bb222$$sw_bb238$$sw_bb257$$sw_bb280$$sw_bb287$$sw_bb312$$sw_bb322$$sw_bb331$$sw_bb347$$sw_bb381$$sw_bb391$$sw_bb403$$sw_bb414$$sw_bb422$$sw_bb439$$sw_bb449$$sw_bb461$$sw_bb481$$sw_bb486$$sw_bb491$$sw_bb509$$sw_bb523$$sw_bb533$$sw_bb534$$sw_bb547$$sw_bb576$$sw_bb599$$sw_bb614$$sw_bb623$$sw_bb630$$sw_bb645$$sw_bb692$$sw_bb702$$sw_bb711$$sw_bb729$$sw_bb734$6; //@line 858 "tr_languages.cpp"
      }
      else if (__label__ == 40) {
  
        var $263=$tr; //@line 865 "tr_languages.cpp"
        __Z15SetupTranslatorP10TranslatorPKsPKh($263, ((__ZZ16SelectTranslatorPKcE17stress_lengths_ta)&4294967295), ((__ZZ16SelectTranslatorPKcE14stress_amps_ta)&4294967295)); //@line 865 "tr_languages.cpp"
        var $264=$tr; //@line 866 "tr_languages.cpp"
        var $letter_bits392=(($264+708)&4294967295); //@line 866 "tr_languages.cpp"
        var $arraydecay393=(($letter_bits392)&4294967295); //@line 866 "tr_languages.cpp"
        var $dest$, $stop$, $stop4$, $fast$, $value4$;
        $dest$ = $arraydecay393;
        $stop$ = $dest$ + 256;
        $value4$ = 0;
        if ($value4$ < 0) $value4$ += 256;
        $value4$ = $value4$ + ($value4$<<8) + ($value4$<<16) + ($value4$*16777216);
        while ($dest$%4 !== 0 && $dest$ < $stop$) {
          ;; HEAP8[$dest$++] = 0;
        }
        $dest$ >>= 2;
        $stop4$ = $stop$ >> 2;
        while ($dest$ < $stop4$) {
        ;;;;;;;  HEAP32[$dest$++] = $value4$;
        }
        $dest$ <<= 2;
        while ($dest$ < $stop$) {
          ;; HEAP8[$dest$++] = 0;
        }; //@line 866 "tr_languages.cpp"
        var $265=$tr; //@line 867 "tr_languages.cpp"
        __ZL13SetLetterBitsP10TranslatoriPKc($265, 0, ((__ZZ16SelectTranslatorPKcE9ka_vowels)&4294967295)); //@line 867 "tr_languages.cpp"
        var $266=$tr; //@line 868 "tr_languages.cpp"
        __ZL13SetLetterBitsP10TranslatoriPKc($266, 2, ((__ZZ16SelectTranslatorPKcE13ka_consonants)&4294967295)); //@line 868 "tr_languages.cpp"
        var $267=$tr; //@line 869 "tr_languages.cpp"
        __ZL13SetLetterBitsP10TranslatoriPKc($267, 7, ((__ZZ16SelectTranslatorPKcE9ka_vowels)&4294967295)); //@line 869 "tr_languages.cpp"
        var $268=$tr; //@line 871 "tr_languages.cpp"
        var $langopts394=(($268)&4294967295); //@line 871 "tr_languages.cpp"
        var $stress_rule395=(($langopts394+8)&4294967295); //@line 871 "tr_languages.cpp"
        HEAP32[(($stress_rule395)>>2)]=0; //@line 871 "tr_languages.cpp"
        var $269=$tr; //@line 872 "tr_languages.cpp"
        var $langopts396=(($269)&4294967295); //@line 872 "tr_languages.cpp"
        var $stress_flags397=(($langopts396+12)&4294967295); //@line 872 "tr_languages.cpp"
        HEAP32[(($stress_flags397)>>2)]=16; //@line 872 "tr_languages.cpp"
        var $270=$tr; //@line 873 "tr_languages.cpp"
        var $letter_bits_offset398=(($270+964)&4294967295); //@line 873 "tr_languages.cpp"
        HEAP32[(($letter_bits_offset398)>>2)]=4224; //@line 873 "tr_languages.cpp"
        var $271=$tr; //@line 875 "tr_languages.cpp"
        var $langopts399=(($271)&4294967295); //@line 875 "tr_languages.cpp"
        var $max_initial_consonants400=(($langopts399+248)&4294967295); //@line 875 "tr_languages.cpp"
        HEAP8[($max_initial_consonants400)]=7; //@line 875 "tr_languages.cpp"
        var $272=$tr; //@line 876 "tr_languages.cpp"
        var $langopts401=(($272)&4294967295); //@line 876 "tr_languages.cpp"
        var $numbers402=(($langopts401+184)&4294967295); //@line 876 "tr_languages.cpp"
        HEAP32[(($numbers402)>>2)]=3187744; //@line 876 "tr_languages.cpp"
        __label__ = 80;break $sw_default$$sw_bb$$sw_bb9$$sw_bb18$$sw_bb24$$sw_bb39$$sw_bb52$$sw_bb64$$sw_bb72$$sw_bb92$$sw_bb109$$sw_bb123$$sw_bb148$$sw_bb161$$sw_bb195$$sw_bb200$$sw_bb207$$sw_bb209$$sw_bb222$$sw_bb238$$sw_bb257$$sw_bb280$$sw_bb287$$sw_bb312$$sw_bb322$$sw_bb331$$sw_bb347$$sw_bb381$$sw_bb391$$sw_bb403$$sw_bb414$$sw_bb422$$sw_bb439$$sw_bb449$$sw_bb461$$sw_bb481$$sw_bb486$$sw_bb491$$sw_bb509$$sw_bb523$$sw_bb533$$sw_bb534$$sw_bb547$$sw_bb576$$sw_bb599$$sw_bb614$$sw_bb623$$sw_bb630$$sw_bb645$$sw_bb692$$sw_bb702$$sw_bb711$$sw_bb729$$sw_bb734$6; //@line 878 "tr_languages.cpp"
      }
      else if (__label__ == 41) {
  
        var $273=$tr; //@line 885 "tr_languages.cpp"
        var $letter_bits_offset404=(($273+964)&4294967295); //@line 885 "tr_languages.cpp"
        HEAP32[(($letter_bits_offset404)>>2)]=4352; //@line 885 "tr_languages.cpp"
        var $274=$tr; //@line 886 "tr_languages.cpp"
        var $letter_bits405=(($274+708)&4294967295); //@line 886 "tr_languages.cpp"
        var $arraydecay406=(($letter_bits405)&4294967295); //@line 886 "tr_languages.cpp"
        var $dest$, $stop$, $stop4$, $fast$, $value4$;
        $dest$ = $arraydecay406;
        $stop$ = $dest$ + 256;
        $value4$ = 0;
        if ($value4$ < 0) $value4$ += 256;
        $value4$ = $value4$ + ($value4$<<8) + ($value4$<<16) + ($value4$*16777216);
        while ($dest$%4 !== 0 && $dest$ < $stop$) {
          ;; HEAP8[$dest$++] = 0;
        }
        $dest$ >>= 2;
        $stop4$ = $stop$ >> 2;
        while ($dest$ < $stop4$) {
        ;;;;;;;  HEAP32[$dest$++] = $value4$;
        }
        $dest$ <<= 2;
        while ($dest$ < $stop$) {
          ;; HEAP8[$dest$++] = 0;
        }; //@line 886 "tr_languages.cpp"
        var $275=$tr; //@line 887 "tr_languages.cpp"
        __ZL18SetLetterBitsRangeP10Translatoriii($275, 0, 97, 117); //@line 887 "tr_languages.cpp"
        var $276=$tr; //@line 888 "tr_languages.cpp"
        __ZL13SetLetterBitsP10TranslatoriPKc($276, 6, ((__ZZ16SelectTranslatorPKcE10ko_ivowels)&4294967295)); //@line 888 "tr_languages.cpp"
        var $277=$tr; //@line 889 "tr_languages.cpp"
        __ZL13SetLetterBitsP10TranslatoriPKc($277, 5, ((__ZZ16SelectTranslatorPKcE9ko_voiced)&4294967295)); //@line 889 "tr_languages.cpp"
        var $278=$tr; //@line 891 "tr_languages.cpp"
        var $langopts407=(($278)&4294967295); //@line 891 "tr_languages.cpp"
        var $stress_rule408=(($langopts407+8)&4294967295); //@line 891 "tr_languages.cpp"
        HEAP32[(($stress_rule408)>>2)]=8; //@line 891 "tr_languages.cpp"
        var $279=$tr; //@line 892 "tr_languages.cpp"
        var $langopts409=(($279)&4294967295); //@line 892 "tr_languages.cpp"
        var $param410=(($langopts409+24)&4294967295); //@line 892 "tr_languages.cpp"
        var $arrayidx411=(($param410+20)&4294967295); //@line 892 "tr_languages.cpp"
        HEAP32[(($arrayidx411)>>2)]=1; //@line 892 "tr_languages.cpp"
        var $280=$tr; //@line 893 "tr_languages.cpp"
        var $langopts412=(($280)&4294967295); //@line 893 "tr_languages.cpp"
        var $numbers413=(($langopts412+184)&4294967295); //@line 893 "tr_languages.cpp"
        HEAP32[(($numbers413)>>2)]=1024; //@line 893 "tr_languages.cpp"
        __label__ = 80;break $sw_default$$sw_bb$$sw_bb9$$sw_bb18$$sw_bb24$$sw_bb39$$sw_bb52$$sw_bb64$$sw_bb72$$sw_bb92$$sw_bb109$$sw_bb123$$sw_bb148$$sw_bb161$$sw_bb195$$sw_bb200$$sw_bb207$$sw_bb209$$sw_bb222$$sw_bb238$$sw_bb257$$sw_bb280$$sw_bb287$$sw_bb312$$sw_bb322$$sw_bb331$$sw_bb347$$sw_bb381$$sw_bb391$$sw_bb403$$sw_bb414$$sw_bb422$$sw_bb439$$sw_bb449$$sw_bb461$$sw_bb481$$sw_bb486$$sw_bb491$$sw_bb509$$sw_bb523$$sw_bb533$$sw_bb534$$sw_bb547$$sw_bb576$$sw_bb599$$sw_bb614$$sw_bb623$$sw_bb630$$sw_bb645$$sw_bb692$$sw_bb702$$sw_bb711$$sw_bb729$$sw_bb734$6; //@line 895 "tr_languages.cpp"
      }
      else if (__label__ == 42) {
  
        var $281=$tr; //@line 902 "tr_languages.cpp"
        __Z15SetupTranslatorP10TranslatorPKsPKh($281, ((__ZZ16SelectTranslatorPKcE17stress_lengths_ku)&4294967295), ((__ZZ16SelectTranslatorPKcE14stress_amps_ku)&4294967295)); //@line 902 "tr_languages.cpp"
        var $282=HEAP32[((((_charsets+36)&4294967295))>>2)]; //@line 903 "tr_languages.cpp"
        var $283=$tr; //@line 903 "tr_languages.cpp"
        var $charset_a0415=(($283+696)&4294967295); //@line 903 "tr_languages.cpp"
        HEAP32[(($charset_a0415)>>2)]=$282; //@line 903 "tr_languages.cpp"
        var $284=$tr; //@line 905 "tr_languages.cpp"
        var $langopts416=(($284)&4294967295); //@line 905 "tr_languages.cpp"
        var $stress_rule417=(($langopts416+8)&4294967295); //@line 905 "tr_languages.cpp"
        HEAP32[(($stress_rule417)>>2)]=7; //@line 905 "tr_languages.cpp"
        var $285=$tr; //@line 907 "tr_languages.cpp"
        var $langopts418=(($285)&4294967295); //@line 907 "tr_languages.cpp"
        var $numbers419=(($langopts418+184)&4294967295); //@line 907 "tr_languages.cpp"
        HEAP32[(($numbers419)>>2)]=263264; //@line 907 "tr_languages.cpp"
        var $286=$tr; //@line 908 "tr_languages.cpp"
        var $langopts420=(($286)&4294967295); //@line 908 "tr_languages.cpp"
        var $max_initial_consonants421=(($langopts420+248)&4294967295); //@line 908 "tr_languages.cpp"
        HEAP8[($max_initial_consonants421)]=2; //@line 908 "tr_languages.cpp"
        __label__ = 80;break $sw_default$$sw_bb$$sw_bb9$$sw_bb18$$sw_bb24$$sw_bb39$$sw_bb52$$sw_bb64$$sw_bb72$$sw_bb92$$sw_bb109$$sw_bb123$$sw_bb148$$sw_bb161$$sw_bb195$$sw_bb200$$sw_bb207$$sw_bb209$$sw_bb222$$sw_bb238$$sw_bb257$$sw_bb280$$sw_bb287$$sw_bb312$$sw_bb322$$sw_bb331$$sw_bb347$$sw_bb381$$sw_bb391$$sw_bb403$$sw_bb414$$sw_bb422$$sw_bb439$$sw_bb449$$sw_bb461$$sw_bb481$$sw_bb486$$sw_bb491$$sw_bb509$$sw_bb523$$sw_bb533$$sw_bb534$$sw_bb547$$sw_bb576$$sw_bb599$$sw_bb614$$sw_bb623$$sw_bb630$$sw_bb645$$sw_bb692$$sw_bb702$$sw_bb711$$sw_bb729$$sw_bb734$6; //@line 910 "tr_languages.cpp"
      }
      else if (__label__ == 43) {
  
        var $287=HEAP32[((((_charsets+16)&4294967295))>>2)]; //@line 914 "tr_languages.cpp"
        var $288=$tr; //@line 914 "tr_languages.cpp"
        var $charset_a0423=(($288+696)&4294967295); //@line 914 "tr_languages.cpp"
        HEAP32[(($charset_a0423)>>2)]=$287; //@line 914 "tr_languages.cpp"
        var $289=$tr; //@line 915 "tr_languages.cpp"
        var $langopts424=(($289)&4294967295); //@line 915 "tr_languages.cpp"
        var $stress_rule425=(($langopts424+8)&4294967295); //@line 915 "tr_languages.cpp"
        HEAP32[(($stress_rule425)>>2)]=2; //@line 915 "tr_languages.cpp"
        var $290=$tr; //@line 916 "tr_languages.cpp"
        var $langopts426=(($290)&4294967295); //@line 916 "tr_languages.cpp"
        var $stress_flags427=(($langopts426+12)&4294967295); //@line 916 "tr_languages.cpp"
        HEAP32[(($stress_flags427)>>2)]=32; //@line 916 "tr_languages.cpp"
        var $291=$tr; //@line 917 "tr_languages.cpp"
        var $langopts428=(($291)&4294967295); //@line 917 "tr_languages.cpp"
        var $unstressed_wd1429=(($langopts428+16)&4294967295); //@line 917 "tr_languages.cpp"
        HEAP32[(($unstressed_wd1429)>>2)]=0; //@line 917 "tr_languages.cpp"
        var $292=$tr; //@line 918 "tr_languages.cpp"
        var $langopts430=(($292)&4294967295); //@line 918 "tr_languages.cpp"
        var $unstressed_wd2431=(($langopts430+20)&4294967295); //@line 918 "tr_languages.cpp"
        HEAP32[(($unstressed_wd2431)>>2)]=2; //@line 918 "tr_languages.cpp"
        var $293=$tr; //@line 919 "tr_languages.cpp"
        var $langopts432=(($293)&4294967295); //@line 919 "tr_languages.cpp"
        var $param433=(($langopts432+24)&4294967295); //@line 919 "tr_languages.cpp"
        var $arrayidx434=(($param433+4)&4294967295); //@line 919 "tr_languages.cpp"
        HEAP32[(($arrayidx434)>>2)]=1; //@line 919 "tr_languages.cpp"
        var $294=$tr; //@line 920 "tr_languages.cpp"
        var $langopts435=(($294)&4294967295); //@line 920 "tr_languages.cpp"
        var $numbers436=(($langopts435+184)&4294967295); //@line 920 "tr_languages.cpp"
        HEAP32[(($numbers436)>>2)]=16777216; //@line 920 "tr_languages.cpp"
        var $295=$tr; //@line 921 "tr_languages.cpp"
        var $langopts437=(($295)&4294967295); //@line 921 "tr_languages.cpp"
        var $max_roman438=(($langopts437+196)&4294967295); //@line 921 "tr_languages.cpp"
        HEAP32[(($max_roman438)>>2)]=5000; //@line 921 "tr_languages.cpp"
        __label__ = 80;break $sw_default$$sw_bb$$sw_bb9$$sw_bb18$$sw_bb24$$sw_bb39$$sw_bb52$$sw_bb64$$sw_bb72$$sw_bb92$$sw_bb109$$sw_bb123$$sw_bb148$$sw_bb161$$sw_bb195$$sw_bb200$$sw_bb207$$sw_bb209$$sw_bb222$$sw_bb238$$sw_bb257$$sw_bb280$$sw_bb287$$sw_bb312$$sw_bb322$$sw_bb331$$sw_bb347$$sw_bb381$$sw_bb391$$sw_bb403$$sw_bb414$$sw_bb422$$sw_bb439$$sw_bb449$$sw_bb461$$sw_bb481$$sw_bb486$$sw_bb491$$sw_bb509$$sw_bb523$$sw_bb533$$sw_bb534$$sw_bb547$$sw_bb576$$sw_bb599$$sw_bb614$$sw_bb623$$sw_bb630$$sw_bb645$$sw_bb692$$sw_bb702$$sw_bb711$$sw_bb729$$sw_bb734$6; //@line 923 "tr_languages.cpp"
      }
      else if (__label__ == 44) {
  
        var $296=$tr; //@line 930 "tr_languages.cpp"
        __Z15SetupTranslatorP10TranslatorPKsPKh($296, ((__ZZ16SelectTranslatorPKcE17stress_lengths_lv)&4294967295), ((__ZZ16SelectTranslatorPKcE14stress_amps_lv)&4294967295)); //@line 930 "tr_languages.cpp"
        var $297=$tr; //@line 932 "tr_languages.cpp"
        var $langopts440=(($297)&4294967295); //@line 932 "tr_languages.cpp"
        var $stress_rule441=(($langopts440+8)&4294967295); //@line 932 "tr_languages.cpp"
        HEAP32[(($stress_rule441)>>2)]=0; //@line 932 "tr_languages.cpp"
        var $298=$tr; //@line 933 "tr_languages.cpp"
        var $langopts442=(($298)&4294967295); //@line 933 "tr_languages.cpp"
        var $spelling_stress443=(($langopts442+249)&4294967295); //@line 933 "tr_languages.cpp"
        HEAP8[($spelling_stress443)]=1; //@line 933 "tr_languages.cpp"
        var $299=HEAP32[((((_charsets+16)&4294967295))>>2)]; //@line 934 "tr_languages.cpp"
        var $300=$tr; //@line 934 "tr_languages.cpp"
        var $charset_a0444=(($300+696)&4294967295); //@line 934 "tr_languages.cpp"
        HEAP32[(($charset_a0444)>>2)]=$299; //@line 934 "tr_languages.cpp"
        var $301=$tr; //@line 935 "tr_languages.cpp"
        var $langopts445=(($301)&4294967295); //@line 935 "tr_languages.cpp"
        var $numbers446=(($langopts445+184)&4294967295); //@line 935 "tr_languages.cpp"
        HEAP32[(($numbers446)>>2)]=99336; //@line 935 "tr_languages.cpp"
        var $302=$tr; //@line 936 "tr_languages.cpp"
        var $langopts447=(($302)&4294967295); //@line 936 "tr_languages.cpp"
        var $stress_flags448=(($langopts447+12)&4294967295); //@line 936 "tr_languages.cpp"
        HEAP32[(($stress_flags448)>>2)]=262166; //@line 936 "tr_languages.cpp"
        __label__ = 80;break $sw_default$$sw_bb$$sw_bb9$$sw_bb18$$sw_bb24$$sw_bb39$$sw_bb52$$sw_bb64$$sw_bb72$$sw_bb92$$sw_bb109$$sw_bb123$$sw_bb148$$sw_bb161$$sw_bb195$$sw_bb200$$sw_bb207$$sw_bb209$$sw_bb222$$sw_bb238$$sw_bb257$$sw_bb280$$sw_bb287$$sw_bb312$$sw_bb322$$sw_bb331$$sw_bb347$$sw_bb381$$sw_bb391$$sw_bb403$$sw_bb414$$sw_bb422$$sw_bb439$$sw_bb449$$sw_bb461$$sw_bb481$$sw_bb486$$sw_bb491$$sw_bb509$$sw_bb523$$sw_bb533$$sw_bb534$$sw_bb547$$sw_bb576$$sw_bb599$$sw_bb614$$sw_bb623$$sw_bb630$$sw_bb645$$sw_bb692$$sw_bb702$$sw_bb711$$sw_bb729$$sw_bb734$6; //@line 938 "tr_languages.cpp"
      }
      else if (__label__ == 45) {
  
        var $303=$tr; //@line 947 "tr_languages.cpp"
        __Z15SetupTranslatorP10TranslatorPKsPKh($303, ((__ZZ16SelectTranslatorPKcE17stress_lengths_mk)&4294967295), ((__ZZ16SelectTranslatorPKcE14stress_amps_mk)&4294967295)); //@line 947 "tr_languages.cpp"
        var $304=HEAP32[((((_charsets+20)&4294967295))>>2)]; //@line 948 "tr_languages.cpp"
        var $305=$tr; //@line 948 "tr_languages.cpp"
        var $charset_a0450=(($305+696)&4294967295); //@line 948 "tr_languages.cpp"
        HEAP32[(($charset_a0450)>>2)]=$304; //@line 948 "tr_languages.cpp"
        var $306=$tr; //@line 949 "tr_languages.cpp"
        var $letter_groups451=(($306+968)&4294967295); //@line 949 "tr_languages.cpp"
        var $arrayidx452=(($letter_groups451+28)&4294967295); //@line 949 "tr_languages.cpp"
        HEAP32[(($arrayidx452)>>2)]=((__ZZ16SelectTranslatorPKcE15vowels_cyrillic)&4294967295); //@line 949 "tr_languages.cpp"
        var $307=$tr; //@line 949 "tr_languages.cpp"
        var $letter_groups453=(($307+968)&4294967295); //@line 949 "tr_languages.cpp"
        var $arrayidx454=(($letter_groups453)&4294967295); //@line 949 "tr_languages.cpp"
        HEAP32[(($arrayidx454)>>2)]=((__ZZ16SelectTranslatorPKcE15vowels_cyrillic)&4294967295); //@line 949 "tr_languages.cpp"
        var $308=$tr; //@line 951 "tr_languages.cpp"
        var $langopts455=(($308)&4294967295); //@line 951 "tr_languages.cpp"
        var $stress_rule456=(($langopts455+8)&4294967295); //@line 951 "tr_languages.cpp"
        HEAP32[(($stress_rule456)>>2)]=4; //@line 951 "tr_languages.cpp"
        var $309=$tr; //@line 952 "tr_languages.cpp"
        var $langopts457=(($309)&4294967295); //@line 952 "tr_languages.cpp"
        var $numbers458=(($langopts457+184)&4294967295); //@line 952 "tr_languages.cpp"
        HEAP32[(($numbers458)>>2)]=2114600; //@line 952 "tr_languages.cpp"
        var $310=$tr; //@line 953 "tr_languages.cpp"
        var $langopts459=(($310)&4294967295); //@line 953 "tr_languages.cpp"
        var $numbers2460=(($langopts459+188)&4294967295); //@line 953 "tr_languages.cpp"
        HEAP32[(($numbers2460)>>2)]=138; //@line 953 "tr_languages.cpp"
        __label__ = 80;break $sw_default$$sw_bb$$sw_bb9$$sw_bb18$$sw_bb24$$sw_bb39$$sw_bb52$$sw_bb64$$sw_bb72$$sw_bb92$$sw_bb109$$sw_bb123$$sw_bb148$$sw_bb161$$sw_bb195$$sw_bb200$$sw_bb207$$sw_bb209$$sw_bb222$$sw_bb238$$sw_bb257$$sw_bb280$$sw_bb287$$sw_bb312$$sw_bb322$$sw_bb331$$sw_bb347$$sw_bb381$$sw_bb391$$sw_bb403$$sw_bb414$$sw_bb422$$sw_bb439$$sw_bb449$$sw_bb461$$sw_bb481$$sw_bb486$$sw_bb491$$sw_bb509$$sw_bb523$$sw_bb533$$sw_bb534$$sw_bb547$$sw_bb576$$sw_bb599$$sw_bb614$$sw_bb623$$sw_bb630$$sw_bb645$$sw_bb692$$sw_bb702$$sw_bb711$$sw_bb729$$sw_bb734$6; //@line 955 "tr_languages.cpp"
      }
      else if (__label__ == 46) {
  
        var $311=$tr; //@line 962 "tr_languages.cpp"
        var $langopts462=(($311)&4294967295); //@line 962 "tr_languages.cpp"
        var $stress_rule463=(($langopts462+8)&4294967295); //@line 962 "tr_languages.cpp"
        HEAP32[(($stress_rule463)>>2)]=0; //@line 962 "tr_languages.cpp"
        var $312=$tr; //@line 963 "tr_languages.cpp"
        var $langopts464=(($312)&4294967295); //@line 963 "tr_languages.cpp"
        var $vowel_pause465=(($langopts464+4)&4294967295); //@line 963 "tr_languages.cpp"
        HEAP32[(($vowel_pause465)>>2)]=48; //@line 963 "tr_languages.cpp"
        var $313=$tr; //@line 964 "tr_languages.cpp"
        var $langopts466=(($313)&4294967295); //@line 964 "tr_languages.cpp"
        var $param467=(($langopts466+24)&4294967295); //@line 964 "tr_languages.cpp"
        var $arrayidx468=(($param467+4)&4294967295); //@line 964 "tr_languages.cpp"
        HEAP32[(($arrayidx468)>>2)]=1; //@line 964 "tr_languages.cpp"
        var $314=$tr; //@line 965 "tr_languages.cpp"
        var $langopts469=(($314)&4294967295); //@line 965 "tr_languages.cpp"
        var $param470=(($langopts469+24)&4294967295); //@line 965 "tr_languages.cpp"
        var $arrayidx471=(($param470+12)&4294967295); //@line 965 "tr_languages.cpp"
        HEAP32[(($arrayidx471)>>2)]=1; //@line 965 "tr_languages.cpp"
        var $315=$tr; //@line 966 "tr_languages.cpp"
        var $langopts472=(($315)&4294967295); //@line 966 "tr_languages.cpp"
        var $param473=(($langopts472+24)&4294967295); //@line 966 "tr_languages.cpp"
        var $arrayidx474=(($param473+16)&4294967295); //@line 966 "tr_languages.cpp"
        HEAP32[(($arrayidx474)>>2)]=16; //@line 966 "tr_languages.cpp"
        var $316=$tr; //@line 967 "tr_languages.cpp"
        __ZL14SetLetterVowelP10Translatori($316, 121); //@line 967 "tr_languages.cpp"
        var $317=$tr; //@line 969 "tr_languages.cpp"
        var $langopts475=(($317)&4294967295); //@line 969 "tr_languages.cpp"
        var $numbers476=(($langopts475+184)&4294967295); //@line 969 "tr_languages.cpp"
        HEAP32[(($numbers476)>>2)]=2169880; //@line 969 "tr_languages.cpp"
        var $318=$tr; //@line 970 "tr_languages.cpp"
        var $langopts477=(($318)&4294967295); //@line 970 "tr_languages.cpp"
        var $ordinal_indicator=(($langopts477+216)&4294967295); //@line 970 "tr_languages.cpp"
        HEAP32[(($ordinal_indicator)>>2)]=((__str3633)&4294967295); //@line 970 "tr_languages.cpp"
        var $319=$tr; //@line 971 "tr_languages.cpp"
        var $langopts478=(($319)&4294967295); //@line 971 "tr_languages.cpp"
        var $stress_flags479=(($langopts478+12)&4294967295); //@line 971 "tr_languages.cpp"
        HEAP32[(($stress_flags479)>>2)]=128; //@line 971 "tr_languages.cpp"
        var $320=$tr; //@line 972 "tr_languages.cpp"
        var $stress_lengths=(($320+676)&4294967295); //@line 972 "tr_languages.cpp"
        var $arraydecay480=(($stress_lengths)&4294967295); //@line 972 "tr_languages.cpp"
        var $321=$arraydecay480; //@line 972 "tr_languages.cpp"
        var $src$, $dest$, $stop$, $stop4$, $fast$;
        $src$ = __ZZ16SelectTranslatorPKcE17stress_lengths_nl;
        $dest$ = $321;
        $stop$ = $src$ + 16;
        if (($dest$%4) == ($src$%4) && 16 > 8) {
          while ($src$%4 !== 0 && $src$ < $stop$) {
            ; HEAP8[$dest$++] = HEAP8[$src$++];
          }
          $src$ >>= 2;
          $dest$ >>= 2;
          $stop4$ = $stop$ >> 2;
          while ($src$ < $stop4$) {
        ;;;    HEAP32[$dest$++] = HEAP32[$src$++];
          }
          $src$ <<= 2;
          $dest$ <<= 2;
        }while ($src$ < $stop$) {
          ; HEAP8[$dest$++] = HEAP8[$src$++];
        }; //@line 972 "tr_languages.cpp"
        __label__ = 80;break $sw_default$$sw_bb$$sw_bb9$$sw_bb18$$sw_bb24$$sw_bb39$$sw_bb52$$sw_bb64$$sw_bb72$$sw_bb92$$sw_bb109$$sw_bb123$$sw_bb148$$sw_bb161$$sw_bb195$$sw_bb200$$sw_bb207$$sw_bb209$$sw_bb222$$sw_bb238$$sw_bb257$$sw_bb280$$sw_bb287$$sw_bb312$$sw_bb322$$sw_bb331$$sw_bb347$$sw_bb381$$sw_bb391$$sw_bb403$$sw_bb414$$sw_bb422$$sw_bb439$$sw_bb449$$sw_bb461$$sw_bb481$$sw_bb486$$sw_bb491$$sw_bb509$$sw_bb523$$sw_bb533$$sw_bb534$$sw_bb547$$sw_bb576$$sw_bb599$$sw_bb614$$sw_bb623$$sw_bb630$$sw_bb645$$sw_bb692$$sw_bb702$$sw_bb711$$sw_bb729$$sw_bb734$6; //@line 974 "tr_languages.cpp"
      }
      else if (__label__ == 47) {
  
        var $322=$tr; //@line 980 "tr_languages.cpp"
        __Z15SetupTranslatorP10TranslatorPKsPKh($322, ((__ZZ16SelectTranslatorPKcE17stress_lengths_no)&4294967295), 0); //@line 980 "tr_languages.cpp"
        var $323=$tr; //@line 981 "tr_languages.cpp"
        var $langopts482=(($323)&4294967295); //@line 981 "tr_languages.cpp"
        var $stress_rule483=(($langopts482+8)&4294967295); //@line 981 "tr_languages.cpp"
        HEAP32[(($stress_rule483)>>2)]=0; //@line 981 "tr_languages.cpp"
        var $324=$tr; //@line 982 "tr_languages.cpp"
        __ZL14SetLetterVowelP10Translatori($324, 121); //@line 982 "tr_languages.cpp"
        var $325=$tr; //@line 983 "tr_languages.cpp"
        var $langopts484=(($325)&4294967295); //@line 983 "tr_languages.cpp"
        var $numbers485=(($langopts484+184)&4294967295); //@line 983 "tr_languages.cpp"
        HEAP32[(($numbers485)>>2)]=71752; //@line 983 "tr_languages.cpp"
        __label__ = 80;break $sw_default$$sw_bb$$sw_bb9$$sw_bb18$$sw_bb24$$sw_bb39$$sw_bb52$$sw_bb64$$sw_bb72$$sw_bb92$$sw_bb109$$sw_bb123$$sw_bb148$$sw_bb161$$sw_bb195$$sw_bb200$$sw_bb207$$sw_bb209$$sw_bb222$$sw_bb238$$sw_bb257$$sw_bb280$$sw_bb287$$sw_bb312$$sw_bb322$$sw_bb331$$sw_bb347$$sw_bb381$$sw_bb391$$sw_bb403$$sw_bb414$$sw_bb422$$sw_bb439$$sw_bb449$$sw_bb461$$sw_bb481$$sw_bb486$$sw_bb491$$sw_bb509$$sw_bb523$$sw_bb533$$sw_bb534$$sw_bb547$$sw_bb576$$sw_bb599$$sw_bb614$$sw_bb623$$sw_bb630$$sw_bb645$$sw_bb692$$sw_bb702$$sw_bb711$$sw_bb729$$sw_bb734$6; //@line 985 "tr_languages.cpp"
      }
      else if (__label__ == 48) {
  
        var $326=$tr; //@line 992 "tr_languages.cpp"
        __Z15SetupTranslatorP10TranslatorPKsPKh($326, ((__ZZ16SelectTranslatorPKcE17stress_lengths_om)&4294967295), ((__ZZ16SelectTranslatorPKcE14stress_amps_om)&4294967295)); //@line 992 "tr_languages.cpp"
        var $327=$tr; //@line 993 "tr_languages.cpp"
        var $langopts487=(($327)&4294967295); //@line 993 "tr_languages.cpp"
        var $stress_rule488=(($langopts487+8)&4294967295); //@line 993 "tr_languages.cpp"
        HEAP32[(($stress_rule488)>>2)]=2; //@line 993 "tr_languages.cpp"
        var $328=$tr; //@line 994 "tr_languages.cpp"
        var $langopts489=(($328)&4294967295); //@line 994 "tr_languages.cpp"
        var $stress_flags490=(($langopts489+12)&4294967295); //@line 994 "tr_languages.cpp"
        HEAP32[(($stress_flags490)>>2)]=524310; //@line 994 "tr_languages.cpp"
        __label__ = 80;break $sw_default$$sw_bb$$sw_bb9$$sw_bb18$$sw_bb24$$sw_bb39$$sw_bb52$$sw_bb64$$sw_bb72$$sw_bb92$$sw_bb109$$sw_bb123$$sw_bb148$$sw_bb161$$sw_bb195$$sw_bb200$$sw_bb207$$sw_bb209$$sw_bb222$$sw_bb238$$sw_bb257$$sw_bb280$$sw_bb287$$sw_bb312$$sw_bb322$$sw_bb331$$sw_bb347$$sw_bb381$$sw_bb391$$sw_bb403$$sw_bb414$$sw_bb422$$sw_bb439$$sw_bb449$$sw_bb461$$sw_bb481$$sw_bb486$$sw_bb491$$sw_bb509$$sw_bb523$$sw_bb533$$sw_bb534$$sw_bb547$$sw_bb576$$sw_bb599$$sw_bb614$$sw_bb623$$sw_bb630$$sw_bb645$$sw_bb692$$sw_bb702$$sw_bb711$$sw_bb729$$sw_bb734$6; //@line 996 "tr_languages.cpp"
      }
      else if (__label__ == 49) {
  
        var $329=$tr; //@line 1003 "tr_languages.cpp"
        __Z15SetupTranslatorP10TranslatorPKsPKh($329, ((__ZZ16SelectTranslatorPKcE17stress_lengths_pl)&4294967295), ((__ZZ16SelectTranslatorPKcE14stress_amps_pl)&4294967295)); //@line 1003 "tr_languages.cpp"
        var $330=HEAP32[((((_charsets+8)&4294967295))>>2)]; //@line 1005 "tr_languages.cpp"
        var $331=$tr; //@line 1005 "tr_languages.cpp"
        var $charset_a0492=(($331+696)&4294967295); //@line 1005 "tr_languages.cpp"
        HEAP32[(($charset_a0492)>>2)]=$330; //@line 1005 "tr_languages.cpp"
        var $332=$tr; //@line 1006 "tr_languages.cpp"
        var $langopts493=(($332)&4294967295); //@line 1006 "tr_languages.cpp"
        var $stress_rule494=(($langopts493+8)&4294967295); //@line 1006 "tr_languages.cpp"
        HEAP32[(($stress_rule494)>>2)]=2; //@line 1006 "tr_languages.cpp"
        var $333=$tr; //@line 1007 "tr_languages.cpp"
        var $langopts495=(($333)&4294967295); //@line 1007 "tr_languages.cpp"
        var $stress_flags496=(($langopts495+12)&4294967295); //@line 1007 "tr_languages.cpp"
        HEAP32[(($stress_flags496)>>2)]=6; //@line 1007 "tr_languages.cpp"
        var $334=$tr; //@line 1008 "tr_languages.cpp"
        var $langopts497=(($334)&4294967295); //@line 1008 "tr_languages.cpp"
        var $param498=(($langopts497+24)&4294967295); //@line 1008 "tr_languages.cpp"
        var $arrayidx499=(($param498+16)&4294967295); //@line 1008 "tr_languages.cpp"
        HEAP32[(($arrayidx499)>>2)]=8; //@line 1008 "tr_languages.cpp"
        var $335=$tr; //@line 1009 "tr_languages.cpp"
        var $langopts500=(($335)&4294967295); //@line 1009 "tr_languages.cpp"
        var $max_initial_consonants501=(($langopts500+248)&4294967295); //@line 1009 "tr_languages.cpp"
        HEAP8[($max_initial_consonants501)]=7; //@line 1009 "tr_languages.cpp"
        var $336=$tr; //@line 1010 "tr_languages.cpp"
        var $langopts502=(($336)&4294967295); //@line 1010 "tr_languages.cpp"
        var $numbers503=(($langopts502+184)&4294967295); //@line 1010 "tr_languages.cpp"
        HEAP32[(($numbers503)>>2)]=20488; //@line 1010 "tr_languages.cpp"
        var $337=$tr; //@line 1011 "tr_languages.cpp"
        var $langopts504=(($337)&4294967295); //@line 1011 "tr_languages.cpp"
        var $numbers2505=(($langopts504+188)&4294967295); //@line 1011 "tr_languages.cpp"
        HEAP32[(($numbers2505)>>2)]=64; //@line 1011 "tr_languages.cpp"
        var $338=$tr; //@line 1012 "tr_languages.cpp"
        var $langopts506=(($338)&4294967295); //@line 1012 "tr_languages.cpp"
        var $param507=(($langopts506+24)&4294967295); //@line 1012 "tr_languages.cpp"
        var $arrayidx508=(($param507+44)&4294967295); //@line 1012 "tr_languages.cpp"
        HEAP32[(($arrayidx508)>>2)]=260; //@line 1012 "tr_languages.cpp"
        var $339=$tr; //@line 1013 "tr_languages.cpp"
        __ZL14SetLetterVowelP10Translatori($339, 121); //@line 1013 "tr_languages.cpp"
        __label__ = 80;break $sw_default$$sw_bb$$sw_bb9$$sw_bb18$$sw_bb24$$sw_bb39$$sw_bb52$$sw_bb64$$sw_bb72$$sw_bb92$$sw_bb109$$sw_bb123$$sw_bb148$$sw_bb161$$sw_bb195$$sw_bb200$$sw_bb207$$sw_bb209$$sw_bb222$$sw_bb238$$sw_bb257$$sw_bb280$$sw_bb287$$sw_bb312$$sw_bb322$$sw_bb331$$sw_bb347$$sw_bb381$$sw_bb391$$sw_bb403$$sw_bb414$$sw_bb422$$sw_bb439$$sw_bb449$$sw_bb461$$sw_bb481$$sw_bb486$$sw_bb491$$sw_bb509$$sw_bb523$$sw_bb533$$sw_bb534$$sw_bb547$$sw_bb576$$sw_bb599$$sw_bb614$$sw_bb623$$sw_bb630$$sw_bb645$$sw_bb692$$sw_bb702$$sw_bb711$$sw_bb729$$sw_bb734$6; //@line 1015 "tr_languages.cpp"
      }
      else if (__label__ == 50) {
  
        var $340=$tr; //@line 1022 "tr_languages.cpp"
        __Z15SetupTranslatorP10TranslatorPKsPKh($340, ((__ZZ16SelectTranslatorPKcE17stress_lengths_pt)&4294967295), ((__ZZ16SelectTranslatorPKcE14stress_amps_pt)&4294967295)); //@line 1022 "tr_languages.cpp"
        var $341=$tr; //@line 1023 "tr_languages.cpp"
        var $langopts510=(($341)&4294967295); //@line 1023 "tr_languages.cpp"
        var $length_mods511=(($langopts510+176)&4294967295); //@line 1023 "tr_languages.cpp"
        var $342=HEAP32[(($length_mods511)>>2)]; //@line 1023 "tr_languages.cpp"
        var $343=$tr; //@line 1023 "tr_languages.cpp"
        var $langopts512=(($343)&4294967295); //@line 1023 "tr_languages.cpp"
        var $length_mods0513=(($langopts512+180)&4294967295); //@line 1023 "tr_languages.cpp"
        HEAP32[(($length_mods0513)>>2)]=$342; //@line 1023 "tr_languages.cpp"
        var $344=$tr; //@line 1025 "tr_languages.cpp"
        var $langopts514=(($344)&4294967295); //@line 1025 "tr_languages.cpp"
        var $stress_rule515=(($langopts514+8)&4294967295); //@line 1025 "tr_languages.cpp"
        HEAP32[(($stress_rule515)>>2)]=3; //@line 1025 "tr_languages.cpp"
        var $345=$tr; //@line 1026 "tr_languages.cpp"
        var $langopts516=(($345)&4294967295); //@line 1026 "tr_languages.cpp"
        var $stress_flags517=(($langopts516+12)&4294967295); //@line 1026 "tr_languages.cpp"
        HEAP32[(($stress_flags517)>>2)]=139286; //@line 1026 "tr_languages.cpp"
        var $346=$tr; //@line 1027 "tr_languages.cpp"
        var $langopts518=(($346)&4294967295); //@line 1027 "tr_languages.cpp"
        var $numbers519=(($langopts518+184)&4294967295); //@line 1027 "tr_languages.cpp"
        HEAP32[(($numbers519)>>2)]=16793704; //@line 1027 "tr_languages.cpp"
        var $347=$tr; //@line 1028 "tr_languages.cpp"
        __ZL14SetLetterVowelP10Translatori($347, 121); //@line 1028 "tr_languages.cpp"
        var $348=$tr; //@line 1029 "tr_languages.cpp"
        __ZL15ResetLetterBitsP10Translatori($348, 2); //@line 1029 "tr_languages.cpp"
        var $349=$tr; //@line 1030 "tr_languages.cpp"
        __ZL13SetLetterBitsP10TranslatoriPKc($349, 1, ((__str4634)&4294967295)); //@line 1030 "tr_languages.cpp"
        var $350=$tr; //@line 1031 "tr_languages.cpp"
        var $langopts520=(($350)&4294967295); //@line 1031 "tr_languages.cpp"
        var $param521=(($langopts520+24)&4294967295); //@line 1031 "tr_languages.cpp"
        var $arrayidx522=(($param521+60)&4294967295); //@line 1031 "tr_languages.cpp"
        HEAP32[(($arrayidx522)>>2)]=2; //@line 1031 "tr_languages.cpp"
        __label__ = 80;break $sw_default$$sw_bb$$sw_bb9$$sw_bb18$$sw_bb24$$sw_bb39$$sw_bb52$$sw_bb64$$sw_bb72$$sw_bb92$$sw_bb109$$sw_bb123$$sw_bb148$$sw_bb161$$sw_bb195$$sw_bb200$$sw_bb207$$sw_bb209$$sw_bb222$$sw_bb238$$sw_bb257$$sw_bb280$$sw_bb287$$sw_bb312$$sw_bb322$$sw_bb331$$sw_bb347$$sw_bb381$$sw_bb391$$sw_bb403$$sw_bb414$$sw_bb422$$sw_bb439$$sw_bb449$$sw_bb461$$sw_bb481$$sw_bb486$$sw_bb491$$sw_bb509$$sw_bb523$$sw_bb533$$sw_bb534$$sw_bb547$$sw_bb576$$sw_bb599$$sw_bb614$$sw_bb623$$sw_bb630$$sw_bb645$$sw_bb692$$sw_bb702$$sw_bb711$$sw_bb729$$sw_bb734$6; //@line 1033 "tr_languages.cpp"
      }
      else if (__label__ == 51) {
  
        var $351=$tr; //@line 1040 "tr_languages.cpp"
        __Z15SetupTranslatorP10TranslatorPKsPKh($351, ((__ZZ16SelectTranslatorPKcE17stress_lengths_ro)&4294967295), ((__ZZ16SelectTranslatorPKcE14stress_amps_ro)&4294967295)); //@line 1040 "tr_languages.cpp"
        var $352=$tr; //@line 1042 "tr_languages.cpp"
        var $langopts524=(($352)&4294967295); //@line 1042 "tr_languages.cpp"
        var $stress_rule525=(($langopts524+8)&4294967295); //@line 1042 "tr_languages.cpp"
        HEAP32[(($stress_rule525)>>2)]=2; //@line 1042 "tr_languages.cpp"
        var $353=$tr; //@line 1043 "tr_languages.cpp"
        var $langopts526=(($353)&4294967295); //@line 1043 "tr_languages.cpp"
        var $stress_flags527=(($langopts526+12)&4294967295); //@line 1043 "tr_languages.cpp"
        HEAP32[(($stress_flags527)>>2)]=262; //@line 1043 "tr_languages.cpp"
        var $354=HEAP32[((((_charsets+8)&4294967295))>>2)]; //@line 1045 "tr_languages.cpp"
        var $355=$tr; //@line 1045 "tr_languages.cpp"
        var $charset_a0528=(($355+696)&4294967295); //@line 1045 "tr_languages.cpp"
        HEAP32[(($charset_a0528)>>2)]=$354; //@line 1045 "tr_languages.cpp"
        var $356=$tr; //@line 1046 "tr_languages.cpp"
        var $langopts529=(($356)&4294967295); //@line 1046 "tr_languages.cpp"
        var $numbers530=(($langopts529+184)&4294967295); //@line 1046 "tr_languages.cpp"
        HEAP32[(($numbers530)>>2)]=16805928; //@line 1046 "tr_languages.cpp"
        var $357=$tr; //@line 1047 "tr_languages.cpp"
        var $langopts531=(($357)&4294967295); //@line 1047 "tr_languages.cpp"
        var $numbers2532=(($langopts531+188)&4294967295); //@line 1047 "tr_languages.cpp"
        HEAP32[(($numbers2532)>>2)]=30; //@line 1047 "tr_languages.cpp"
        __label__ = 80;break $sw_default$$sw_bb$$sw_bb9$$sw_bb18$$sw_bb24$$sw_bb39$$sw_bb52$$sw_bb64$$sw_bb72$$sw_bb92$$sw_bb109$$sw_bb123$$sw_bb148$$sw_bb161$$sw_bb195$$sw_bb200$$sw_bb207$$sw_bb209$$sw_bb222$$sw_bb238$$sw_bb257$$sw_bb280$$sw_bb287$$sw_bb312$$sw_bb322$$sw_bb331$$sw_bb347$$sw_bb381$$sw_bb391$$sw_bb403$$sw_bb414$$sw_bb422$$sw_bb439$$sw_bb449$$sw_bb461$$sw_bb481$$sw_bb486$$sw_bb491$$sw_bb509$$sw_bb523$$sw_bb533$$sw_bb534$$sw_bb547$$sw_bb576$$sw_bb599$$sw_bb614$$sw_bb623$$sw_bb630$$sw_bb645$$sw_bb692$$sw_bb702$$sw_bb711$$sw_bb729$$sw_bb734$6; //@line 1049 "tr_languages.cpp"
      }
      else if (__label__ == 52) {
  
        var $358=$tr; //@line 1052 "tr_languages.cpp"
        __ZL18Translator_RussianP10Translator($358); //@line 1052 "tr_languages.cpp"
        __label__ = 80;break $sw_default$$sw_bb$$sw_bb9$$sw_bb18$$sw_bb24$$sw_bb39$$sw_bb52$$sw_bb64$$sw_bb72$$sw_bb92$$sw_bb109$$sw_bb123$$sw_bb148$$sw_bb161$$sw_bb195$$sw_bb200$$sw_bb207$$sw_bb209$$sw_bb222$$sw_bb238$$sw_bb257$$sw_bb280$$sw_bb287$$sw_bb312$$sw_bb322$$sw_bb331$$sw_bb347$$sw_bb381$$sw_bb391$$sw_bb403$$sw_bb414$$sw_bb422$$sw_bb439$$sw_bb449$$sw_bb461$$sw_bb481$$sw_bb486$$sw_bb491$$sw_bb509$$sw_bb523$$sw_bb533$$sw_bb534$$sw_bb547$$sw_bb576$$sw_bb599$$sw_bb614$$sw_bb623$$sw_bb630$$sw_bb645$$sw_bb692$$sw_bb702$$sw_bb711$$sw_bb729$$sw_bb734$6; //@line 1053 "tr_languages.cpp"
      }
      else if (__label__ == 53) {
  
        var $359=$tr; //@line 1057 "tr_languages.cpp"
        var $langopts535=(($359)&4294967295); //@line 1057 "tr_languages.cpp"
        var $stress_rule536=(($langopts535+8)&4294967295); //@line 1057 "tr_languages.cpp"
        HEAP32[(($stress_rule536)>>2)]=2; //@line 1057 "tr_languages.cpp"
        var $360=$tr; //@line 1058 "tr_languages.cpp"
        var $langopts537=(($360)&4294967295); //@line 1058 "tr_languages.cpp"
        var $stress_flags538=(($langopts537+12)&4294967295); //@line 1058 "tr_languages.cpp"
        HEAP32[(($stress_flags538)>>2)]=22; //@line 1058 "tr_languages.cpp"
        var $361=$tr; //@line 1059 "tr_languages.cpp"
        var $langopts539=(($361)&4294967295); //@line 1059 "tr_languages.cpp"
        var $length_mods540=(($langopts539+176)&4294967295); //@line 1059 "tr_languages.cpp"
        var $362=HEAP32[(($length_mods540)>>2)]; //@line 1059 "tr_languages.cpp"
        var $363=$tr; //@line 1059 "tr_languages.cpp"
        var $langopts541=(($363)&4294967295); //@line 1059 "tr_languages.cpp"
        var $length_mods0542=(($langopts541+180)&4294967295); //@line 1059 "tr_languages.cpp"
        HEAP32[(($length_mods0542)>>2)]=$362; //@line 1059 "tr_languages.cpp"
        var $364=$tr; //@line 1061 "tr_languages.cpp"
        var $langopts543=(($364)&4294967295); //@line 1061 "tr_languages.cpp"
        var $numbers544=(($langopts543+184)&4294967295); //@line 1061 "tr_languages.cpp"
        HEAP32[(($numbers544)>>2)]=278624; //@line 1061 "tr_languages.cpp"
        var $365=$tr; //@line 1062 "tr_languages.cpp"
        var $langopts545=(($365)&4294967295); //@line 1062 "tr_languages.cpp"
        var $numbers2546=(($langopts545+188)&4294967295); //@line 1062 "tr_languages.cpp"
        HEAP32[(($numbers2546)>>2)]=512; //@line 1062 "tr_languages.cpp"
        __label__ = 80;break $sw_default$$sw_bb$$sw_bb9$$sw_bb18$$sw_bb24$$sw_bb39$$sw_bb52$$sw_bb64$$sw_bb72$$sw_bb92$$sw_bb109$$sw_bb123$$sw_bb148$$sw_bb161$$sw_bb195$$sw_bb200$$sw_bb207$$sw_bb209$$sw_bb222$$sw_bb238$$sw_bb257$$sw_bb280$$sw_bb287$$sw_bb312$$sw_bb322$$sw_bb331$$sw_bb347$$sw_bb381$$sw_bb391$$sw_bb403$$sw_bb414$$sw_bb422$$sw_bb439$$sw_bb449$$sw_bb461$$sw_bb481$$sw_bb486$$sw_bb491$$sw_bb509$$sw_bb523$$sw_bb533$$sw_bb534$$sw_bb547$$sw_bb576$$sw_bb599$$sw_bb614$$sw_bb623$$sw_bb630$$sw_bb645$$sw_bb692$$sw_bb702$$sw_bb711$$sw_bb729$$sw_bb734$6; //@line 1064 "tr_languages.cpp"
      }
      else if (__label__ == 54) {
  
        var $366=$tr; //@line 1071 "tr_languages.cpp"
        __Z15SetupTranslatorP10TranslatorPKsPKh($366, ((__ZZ16SelectTranslatorPKcE17stress_lengths_sk)&4294967295), ((__ZZ16SelectTranslatorPKcE14stress_amps_sk)&4294967295)); //@line 1071 "tr_languages.cpp"
        var $367=HEAP32[((((_charsets+8)&4294967295))>>2)]; //@line 1072 "tr_languages.cpp"
        var $368=$tr; //@line 1072 "tr_languages.cpp"
        var $charset_a0548=(($368+696)&4294967295); //@line 1072 "tr_languages.cpp"
        HEAP32[(($charset_a0548)>>2)]=$367; //@line 1072 "tr_languages.cpp"
        var $369=$tr; //@line 1074 "tr_languages.cpp"
        var $langopts549=(($369)&4294967295); //@line 1074 "tr_languages.cpp"
        var $stress_rule550=(($langopts549+8)&4294967295); //@line 1074 "tr_languages.cpp"
        HEAP32[(($stress_rule550)>>2)]=0; //@line 1074 "tr_languages.cpp"
        var $370=$tr; //@line 1075 "tr_languages.cpp"
        var $langopts551=(($370)&4294967295); //@line 1075 "tr_languages.cpp"
        var $stress_flags552=(($langopts551+12)&4294967295); //@line 1075 "tr_languages.cpp"
        HEAP32[(($stress_flags552)>>2)]=22; //@line 1075 "tr_languages.cpp"
        var $371=$tr; //@line 1076 "tr_languages.cpp"
        var $langopts553=(($371)&4294967295); //@line 1076 "tr_languages.cpp"
        var $param554=(($langopts553+24)&4294967295); //@line 1076 "tr_languages.cpp"
        var $arrayidx555=(($param554+16)&4294967295); //@line 1076 "tr_languages.cpp"
        HEAP32[(($arrayidx555)>>2)]=3; //@line 1076 "tr_languages.cpp"
        var $372=$tr; //@line 1077 "tr_languages.cpp"
        var $langopts556=(($372)&4294967295); //@line 1077 "tr_languages.cpp"
        var $max_initial_consonants557=(($langopts556+248)&4294967295); //@line 1077 "tr_languages.cpp"
        HEAP8[($max_initial_consonants557)]=5; //@line 1077 "tr_languages.cpp"
        var $373=$tr; //@line 1078 "tr_languages.cpp"
        var $langopts558=(($373)&4294967295); //@line 1078 "tr_languages.cpp"
        var $spelling_stress559=(($langopts558+249)&4294967295); //@line 1078 "tr_languages.cpp"
        HEAP8[($spelling_stress559)]=1; //@line 1078 "tr_languages.cpp"
        var $374=$tr; //@line 1079 "tr_languages.cpp"
        var $langopts560=(($374)&4294967295); //@line 1079 "tr_languages.cpp"
        var $param561=(($langopts560+24)&4294967295); //@line 1079 "tr_languages.cpp"
        var $arrayidx562=(($param561+44)&4294967295); //@line 1079 "tr_languages.cpp"
        HEAP32[(($arrayidx562)>>2)]=4; //@line 1079 "tr_languages.cpp"
        var $375=$tr; //@line 1081 "tr_languages.cpp"
        var $langopts563=(($375)&4294967295); //@line 1081 "tr_languages.cpp"
        var $numbers564=(($langopts563+184)&4294967295); //@line 1081 "tr_languages.cpp"
        HEAP32[(($numbers564)>>2)]=16794624; //@line 1081 "tr_languages.cpp"
        var $376=$tr; //@line 1082 "tr_languages.cpp"
        var $langopts565=(($376)&4294967295); //@line 1082 "tr_languages.cpp"
        var $numbers2566=(($langopts565+188)&4294967295); //@line 1082 "tr_languages.cpp"
        HEAP32[(($numbers2566)>>2)]=256; //@line 1082 "tr_languages.cpp"
        var $377=$tr; //@line 1083 "tr_languages.cpp"
        var $langopts567=(($377)&4294967295); //@line 1083 "tr_languages.cpp"
        var $thousands_sep568=(($langopts567+204)&4294967295); //@line 1083 "tr_languages.cpp"
        HEAP32[(($thousands_sep568)>>2)]=0; //@line 1083 "tr_languages.cpp"
        var $378=$tr; //@line 1084 "tr_languages.cpp"
        var $langopts569=(($378)&4294967295); //@line 1084 "tr_languages.cpp"
        var $decimal_sep570=(($langopts569+208)&4294967295); //@line 1084 "tr_languages.cpp"
        HEAP32[(($decimal_sep570)>>2)]=44; //@line 1084 "tr_languages.cpp"
        var $379=$name2; //@line 1086 "tr_languages.cpp"
        var $cmp571=((($379))|0)==25459; //@line 1086 "tr_languages.cpp"
        if ($cmp571) { __label__ = 55;; } else { __label__ = 56;; } //@line 1086 "tr_languages.cpp"
        if (__label__ == 55) {
  
          var $380=$tr; //@line 1088 "tr_languages.cpp"
          var $langopts573=(($380)&4294967295); //@line 1088 "tr_languages.cpp"
          var $numbers2574=(($langopts573+188)&4294967295); //@line 1088 "tr_languages.cpp"
          HEAP32[(($numbers2574)>>2)]=264; //@line 1088 "tr_languages.cpp"
          ; //@line 1089 "tr_languages.cpp"
        }
  
        var $381=$tr; //@line 1091 "tr_languages.cpp"
        __ZL14SetLetterVowelP10Translatori($381, 121); //@line 1091 "tr_languages.cpp"
        var $382=$tr; //@line 1092 "tr_languages.cpp"
        __ZL14SetLetterVowelP10Translatori($382, 114); //@line 1092 "tr_languages.cpp"
        var $383=$tr; //@line 1093 "tr_languages.cpp"
        __ZL15ResetLetterBitsP10Translatori($383, 32); //@line 1093 "tr_languages.cpp"
        var $384=$tr; //@line 1094 "tr_languages.cpp"
        __ZL13SetLetterBitsP10TranslatoriPKc($384, 5, ((__str5635)&4294967295)); //@line 1094 "tr_languages.cpp"
        __label__ = 80;break $sw_default$$sw_bb$$sw_bb9$$sw_bb18$$sw_bb24$$sw_bb39$$sw_bb52$$sw_bb64$$sw_bb72$$sw_bb92$$sw_bb109$$sw_bb123$$sw_bb148$$sw_bb161$$sw_bb195$$sw_bb200$$sw_bb207$$sw_bb209$$sw_bb222$$sw_bb238$$sw_bb257$$sw_bb280$$sw_bb287$$sw_bb312$$sw_bb322$$sw_bb331$$sw_bb347$$sw_bb381$$sw_bb391$$sw_bb403$$sw_bb414$$sw_bb422$$sw_bb439$$sw_bb449$$sw_bb461$$sw_bb481$$sw_bb486$$sw_bb491$$sw_bb509$$sw_bb523$$sw_bb533$$sw_bb534$$sw_bb547$$sw_bb576$$sw_bb599$$sw_bb614$$sw_bb623$$sw_bb630$$sw_bb645$$sw_bb692$$sw_bb702$$sw_bb711$$sw_bb729$$sw_bb734$6; //@line 1096 "tr_languages.cpp"
      }
      else if (__label__ == 57) {
  
        var $385=$tr; //@line 1100 "tr_languages.cpp"
        __Z15SetupTranslatorP10TranslatorPKsPKh($385, ((__ZZ16SelectTranslatorPKcE17stress_lengths_ta)&4294967295), ((__ZZ16SelectTranslatorPKcE14stress_amps_ta)&4294967295)); //@line 1100 "tr_languages.cpp"
        var $386=$tr; //@line 1101 "tr_languages.cpp"
        var $langopts577=(($386)&4294967295); //@line 1101 "tr_languages.cpp"
        var $length_mods578=(($langopts577+176)&4294967295); //@line 1101 "tr_languages.cpp"
        var $387=HEAP32[(($length_mods578)>>2)]; //@line 1101 "tr_languages.cpp"
        var $388=$tr; //@line 1101 "tr_languages.cpp"
        var $langopts579=(($388)&4294967295); //@line 1101 "tr_languages.cpp"
        var $length_mods0580=(($langopts579+180)&4294967295); //@line 1101 "tr_languages.cpp"
        HEAP32[(($length_mods0580)>>2)]=$387; //@line 1101 "tr_languages.cpp"
        var $389=$tr; //@line 1103 "tr_languages.cpp"
        var $langopts581=(($389)&4294967295); //@line 1103 "tr_languages.cpp"
        var $stress_rule582=(($langopts581+8)&4294967295); //@line 1103 "tr_languages.cpp"
        HEAP32[(($stress_rule582)>>2)]=0; //@line 1103 "tr_languages.cpp"
        var $390=$tr; //@line 1104 "tr_languages.cpp"
        var $langopts583=(($390)&4294967295); //@line 1104 "tr_languages.cpp"
        var $stress_flags584=(($langopts583+12)&4294967295); //@line 1104 "tr_languages.cpp"
        HEAP32[(($stress_flags584)>>2)]=65556; //@line 1104 "tr_languages.cpp"
        var $391=$tr; //@line 1105 "tr_languages.cpp"
        var $langopts585=(($391)&4294967295); //@line 1105 "tr_languages.cpp"
        var $spelling_stress586=(($langopts585+249)&4294967295); //@line 1105 "tr_languages.cpp"
        HEAP8[($spelling_stress586)]=1; //@line 1105 "tr_languages.cpp"
        var $392=$tr; //@line 1107 "tr_languages.cpp"
        var $letter_bits_offset587=(($392+964)&4294967295); //@line 1107 "tr_languages.cpp"
        HEAP32[(($letter_bits_offset587)>>2)]=3456; //@line 1107 "tr_languages.cpp"
        var $393=$tr; //@line 1108 "tr_languages.cpp"
        var $letter_bits588=(($393+708)&4294967295); //@line 1108 "tr_languages.cpp"
        var $arraydecay589=(($letter_bits588)&4294967295); //@line 1108 "tr_languages.cpp"
        var $dest$, $stop$, $stop4$, $fast$, $value4$;
        $dest$ = $arraydecay589;
        $stop$ = $dest$ + 256;
        $value4$ = 0;
        if ($value4$ < 0) $value4$ += 256;
        $value4$ = $value4$ + ($value4$<<8) + ($value4$<<16) + ($value4$*16777216);
        while ($dest$%4 !== 0 && $dest$ < $stop$) {
          ;; HEAP8[$dest$++] = 0;
        }
        $dest$ >>= 2;
        $stop4$ = $stop$ >> 2;
        while ($dest$ < $stop4$) {
        ;;;;;;;  HEAP32[$dest$++] = $value4$;
        }
        $dest$ <<= 2;
        while ($dest$ < $stop$) {
          ;; HEAP8[$dest$++] = 0;
        }; //@line 1108 "tr_languages.cpp"
        var $394=$tr; //@line 1109 "tr_languages.cpp"
        __ZL18SetLetterBitsRangeP10Translatoriii($394, 0, 5, 22); //@line 1109 "tr_languages.cpp"
        var $395=$tr; //@line 1110 "tr_languages.cpp"
        __ZL18SetLetterBitsRangeP10Translatoriii($395, 0, 74, 115); //@line 1110 "tr_languages.cpp"
        var $396=$tr; //@line 1112 "tr_languages.cpp"
        __ZL18SetLetterBitsRangeP10Translatoriii($396, 1, 74, 115); //@line 1112 "tr_languages.cpp"
        var $397=$tr; //@line 1114 "tr_languages.cpp"
        __ZL18SetLetterBitsRangeP10Translatoriii($397, 2, 26, 70); //@line 1114 "tr_languages.cpp"
        var $398=$tr; //@line 1116 "tr_languages.cpp"
        var $langopts590=(($398)&4294967295); //@line 1116 "tr_languages.cpp"
        var $param591=(($langopts590+24)&4294967295); //@line 1116 "tr_languages.cpp"
        var $arrayidx592=(($param591+20)&4294967295); //@line 1116 "tr_languages.cpp"
        HEAP32[(($arrayidx592)>>2)]=1; //@line 1116 "tr_languages.cpp"
        var $399=$tr; //@line 1117 "tr_languages.cpp"
        var $letter_bits_offset593=(($399+964)&4294967295); //@line 1117 "tr_languages.cpp"
        var $400=HEAP32[(($letter_bits_offset593)>>2)]; //@line 1117 "tr_languages.cpp"
        var $add594=((($400)+74)&4294967295); //@line 1117 "tr_languages.cpp"
        var $401=$tr; //@line 1117 "tr_languages.cpp"
        var $langopts595=(($401)&4294967295); //@line 1117 "tr_languages.cpp"
        var $suffix_add_e596=(($langopts595+276)&4294967295); //@line 1117 "tr_languages.cpp"
        HEAP32[(($suffix_add_e596)>>2)]=$add594; //@line 1117 "tr_languages.cpp"
        var $402=$tr; //@line 1118 "tr_languages.cpp"
        var $langopts597=(($402)&4294967295); //@line 1118 "tr_languages.cpp"
        var $numbers598=(($langopts597+184)&4294967295); //@line 1118 "tr_languages.cpp"
        HEAP32[(($numbers598)>>2)]=2097152; //@line 1118 "tr_languages.cpp"
        __label__ = 80;break $sw_default$$sw_bb$$sw_bb9$$sw_bb18$$sw_bb24$$sw_bb39$$sw_bb52$$sw_bb64$$sw_bb72$$sw_bb92$$sw_bb109$$sw_bb123$$sw_bb148$$sw_bb161$$sw_bb195$$sw_bb200$$sw_bb207$$sw_bb209$$sw_bb222$$sw_bb238$$sw_bb257$$sw_bb280$$sw_bb287$$sw_bb312$$sw_bb322$$sw_bb331$$sw_bb347$$sw_bb381$$sw_bb391$$sw_bb403$$sw_bb414$$sw_bb422$$sw_bb439$$sw_bb449$$sw_bb461$$sw_bb481$$sw_bb486$$sw_bb491$$sw_bb509$$sw_bb523$$sw_bb533$$sw_bb534$$sw_bb547$$sw_bb576$$sw_bb599$$sw_bb614$$sw_bb623$$sw_bb630$$sw_bb645$$sw_bb692$$sw_bb702$$sw_bb711$$sw_bb729$$sw_bb734$6; //@line 1120 "tr_languages.cpp"
      }
      else if (__label__ == 58) {
  
        var $403=HEAP32[((((_charsets+8)&4294967295))>>2)]; //@line 1123 "tr_languages.cpp"
        var $404=$tr; //@line 1123 "tr_languages.cpp"
        var $charset_a0600=(($404+696)&4294967295); //@line 1123 "tr_languages.cpp"
        HEAP32[(($charset_a0600)>>2)]=$403; //@line 1123 "tr_languages.cpp"
        var $405=$tr; //@line 1124 "tr_languages.cpp"
        var $langopts601=(($405)&4294967295); //@line 1124 "tr_languages.cpp"
        var $stress_rule602=(($langopts601+8)&4294967295); //@line 1124 "tr_languages.cpp"
        HEAP32[(($stress_rule602)>>2)]=2; //@line 1124 "tr_languages.cpp"
        var $406=$tr; //@line 1125 "tr_languages.cpp"
        var $langopts603=(($406)&4294967295); //@line 1125 "tr_languages.cpp"
        var $stress_flags604=(($langopts603+12)&4294967295); //@line 1125 "tr_languages.cpp"
        HEAP32[(($stress_flags604)>>2)]=32; //@line 1125 "tr_languages.cpp"
        var $407=$tr; //@line 1126 "tr_languages.cpp"
        var $langopts605=(($407)&4294967295); //@line 1126 "tr_languages.cpp"
        var $param606=(($langopts605+24)&4294967295); //@line 1126 "tr_languages.cpp"
        var $arrayidx607=(($param606+16)&4294967295); //@line 1126 "tr_languages.cpp"
        HEAP32[(($arrayidx607)>>2)]=19; //@line 1126 "tr_languages.cpp"
        var $408=$tr; //@line 1127 "tr_languages.cpp"
        var $langopts608=(($408)&4294967295); //@line 1127 "tr_languages.cpp"
        var $numbers609=(($langopts608+184)&4294967295); //@line 1127 "tr_languages.cpp"
        HEAP32[(($numbers609)>>2)]=16864280; //@line 1127 "tr_languages.cpp"
        var $409=$tr; //@line 1128 "tr_languages.cpp"
        var $langopts610=(($409)&4294967295); //@line 1128 "tr_languages.cpp"
        var $numbers2611=(($langopts610+188)&4294967295); //@line 1128 "tr_languages.cpp"
        HEAP32[(($numbers2611)>>2)]=256; //@line 1128 "tr_languages.cpp"
        var $410=$tr; //@line 1129 "tr_languages.cpp"
        var $langopts612=(($410)&4294967295); //@line 1129 "tr_languages.cpp"
        var $thousands_sep613=(($langopts612+204)&4294967295); //@line 1129 "tr_languages.cpp"
        HEAP32[(($thousands_sep613)>>2)]=32; //@line 1129 "tr_languages.cpp"
        var $411=$tr; //@line 1130 "tr_languages.cpp"
        __ZL14SetLetterVowelP10Translatori($411, 114); //@line 1130 "tr_languages.cpp"
        __label__ = 80;break $sw_default$$sw_bb$$sw_bb9$$sw_bb18$$sw_bb24$$sw_bb39$$sw_bb52$$sw_bb64$$sw_bb72$$sw_bb92$$sw_bb109$$sw_bb123$$sw_bb148$$sw_bb161$$sw_bb195$$sw_bb200$$sw_bb207$$sw_bb209$$sw_bb222$$sw_bb238$$sw_bb257$$sw_bb280$$sw_bb287$$sw_bb312$$sw_bb322$$sw_bb331$$sw_bb347$$sw_bb381$$sw_bb391$$sw_bb403$$sw_bb414$$sw_bb422$$sw_bb439$$sw_bb449$$sw_bb461$$sw_bb481$$sw_bb486$$sw_bb491$$sw_bb509$$sw_bb523$$sw_bb533$$sw_bb534$$sw_bb547$$sw_bb576$$sw_bb599$$sw_bb614$$sw_bb623$$sw_bb630$$sw_bb645$$sw_bb692$$sw_bb702$$sw_bb711$$sw_bb729$$sw_bb734$6; //@line 1131 "tr_languages.cpp"
      }
      else if (__label__ == 59) {
  
        var $412=$tr; //@line 1138 "tr_languages.cpp"
        __Z15SetupTranslatorP10TranslatorPKsPKh($412, ((__ZZ16SelectTranslatorPKcE17stress_lengths_sq)&4294967295), ((__ZZ16SelectTranslatorPKcE14stress_amps_sq)&4294967295)); //@line 1138 "tr_languages.cpp"
        var $413=$tr; //@line 1140 "tr_languages.cpp"
        var $langopts615=(($413)&4294967295); //@line 1140 "tr_languages.cpp"
        var $stress_rule616=(($langopts615+8)&4294967295); //@line 1140 "tr_languages.cpp"
        HEAP32[(($stress_rule616)>>2)]=2; //@line 1140 "tr_languages.cpp"
        var $414=$tr; //@line 1141 "tr_languages.cpp"
        var $langopts617=(($414)&4294967295); //@line 1141 "tr_languages.cpp"
        var $stress_flags618=(($langopts617+12)&4294967295); //@line 1141 "tr_languages.cpp"
        HEAP32[(($stress_flags618)>>2)]=278; //@line 1141 "tr_languages.cpp"
        var $415=$tr; //@line 1142 "tr_languages.cpp"
        __ZL14SetLetterVowelP10Translatori($415, 121); //@line 1142 "tr_languages.cpp"
        var $416=$tr; //@line 1143 "tr_languages.cpp"
        var $langopts619=(($416)&4294967295); //@line 1143 "tr_languages.cpp"
        var $numbers620=(($langopts619+184)&4294967295); //@line 1143 "tr_languages.cpp"
        HEAP32[(($numbers620)>>2)]=32872; //@line 1143 "tr_languages.cpp"
        var $417=$tr; //@line 1144 "tr_languages.cpp"
        var $langopts621=(($417)&4294967295); //@line 1144 "tr_languages.cpp"
        var $accents622=(($langopts621+220)&4294967295); //@line 1144 "tr_languages.cpp"
        HEAP32[(($accents622)>>2)]=2; //@line 1144 "tr_languages.cpp"
        __label__ = 80;break $sw_default$$sw_bb$$sw_bb9$$sw_bb18$$sw_bb24$$sw_bb39$$sw_bb52$$sw_bb64$$sw_bb72$$sw_bb92$$sw_bb109$$sw_bb123$$sw_bb148$$sw_bb161$$sw_bb195$$sw_bb200$$sw_bb207$$sw_bb209$$sw_bb222$$sw_bb238$$sw_bb257$$sw_bb280$$sw_bb287$$sw_bb312$$sw_bb322$$sw_bb331$$sw_bb347$$sw_bb381$$sw_bb391$$sw_bb403$$sw_bb414$$sw_bb422$$sw_bb439$$sw_bb449$$sw_bb461$$sw_bb481$$sw_bb486$$sw_bb491$$sw_bb509$$sw_bb523$$sw_bb533$$sw_bb534$$sw_bb547$$sw_bb576$$sw_bb599$$sw_bb614$$sw_bb623$$sw_bb630$$sw_bb645$$sw_bb692$$sw_bb702$$sw_bb711$$sw_bb729$$sw_bb734$6; //@line 1146 "tr_languages.cpp"
      }
      else if (__label__ == 60) {
  
        var $418=$tr; //@line 1153 "tr_languages.cpp"
        __Z15SetupTranslatorP10TranslatorPKsPKh($418, ((__ZZ16SelectTranslatorPKcE17stress_lengths_sv)&4294967295), ((__ZZ16SelectTranslatorPKcE14stress_amps_sv)&4294967295)); //@line 1153 "tr_languages.cpp"
        var $419=$tr; //@line 1155 "tr_languages.cpp"
        var $langopts624=(($419)&4294967295); //@line 1155 "tr_languages.cpp"
        var $stress_rule625=(($langopts624+8)&4294967295); //@line 1155 "tr_languages.cpp"
        HEAP32[(($stress_rule625)>>2)]=0; //@line 1155 "tr_languages.cpp"
        var $420=$tr; //@line 1156 "tr_languages.cpp"
        __ZL14SetLetterVowelP10Translatori($420, 121); //@line 1156 "tr_languages.cpp"
        var $421=$tr; //@line 1157 "tr_languages.cpp"
        var $langopts626=(($421)&4294967295); //@line 1157 "tr_languages.cpp"
        var $numbers627=(($langopts626+184)&4294967295); //@line 1157 "tr_languages.cpp"
        HEAP32[(($numbers627)>>2)]=6408; //@line 1157 "tr_languages.cpp"
        var $422=$tr; //@line 1158 "tr_languages.cpp"
        var $langopts628=(($422)&4294967295); //@line 1158 "tr_languages.cpp"
        var $accents629=(($langopts628+220)&4294967295); //@line 1158 "tr_languages.cpp"
        HEAP32[(($accents629)>>2)]=1; //@line 1158 "tr_languages.cpp"
        __label__ = 80;break $sw_default$$sw_bb$$sw_bb9$$sw_bb18$$sw_bb24$$sw_bb39$$sw_bb52$$sw_bb64$$sw_bb72$$sw_bb92$$sw_bb109$$sw_bb123$$sw_bb148$$sw_bb161$$sw_bb195$$sw_bb200$$sw_bb207$$sw_bb209$$sw_bb222$$sw_bb238$$sw_bb257$$sw_bb280$$sw_bb287$$sw_bb312$$sw_bb322$$sw_bb331$$sw_bb347$$sw_bb381$$sw_bb391$$sw_bb403$$sw_bb414$$sw_bb422$$sw_bb439$$sw_bb449$$sw_bb461$$sw_bb481$$sw_bb486$$sw_bb491$$sw_bb509$$sw_bb523$$sw_bb533$$sw_bb534$$sw_bb547$$sw_bb576$$sw_bb599$$sw_bb614$$sw_bb623$$sw_bb630$$sw_bb645$$sw_bb692$$sw_bb702$$sw_bb711$$sw_bb729$$sw_bb734$6; //@line 1160 "tr_languages.cpp"
      }
      else if (__label__ == 61) {
  
        var $423=$tr; //@line 1168 "tr_languages.cpp"
        __Z15SetupTranslatorP10TranslatorPKsPKh($423, ((__ZZ16SelectTranslatorPKcE17stress_lengths_sw)&4294967295), ((__ZZ16SelectTranslatorPKcE14stress_amps_sw)&4294967295)); //@line 1168 "tr_languages.cpp"
        var $424=$tr; //@line 1169 "tr_languages.cpp"
        var $langopts631=(($424)&4294967295); //@line 1169 "tr_languages.cpp"
        var $length_mods632=(($langopts631+176)&4294967295); //@line 1169 "tr_languages.cpp"
        var $425=HEAP32[(($length_mods632)>>2)]; //@line 1169 "tr_languages.cpp"
        var $426=$tr; //@line 1169 "tr_languages.cpp"
        var $langopts633=(($426)&4294967295); //@line 1169 "tr_languages.cpp"
        var $length_mods0634=(($langopts633+180)&4294967295); //@line 1169 "tr_languages.cpp"
        HEAP32[(($length_mods0634)>>2)]=$425; //@line 1169 "tr_languages.cpp"
        var $427=$tr; //@line 1171 "tr_languages.cpp"
        var $langopts635=(($427)&4294967295); //@line 1171 "tr_languages.cpp"
        var $vowel_pause636=(($langopts635+4)&4294967295); //@line 1171 "tr_languages.cpp"
        HEAP32[(($vowel_pause636)>>2)]=1; //@line 1171 "tr_languages.cpp"
        var $428=$tr; //@line 1172 "tr_languages.cpp"
        var $langopts637=(($428)&4294967295); //@line 1172 "tr_languages.cpp"
        var $stress_rule638=(($langopts637+8)&4294967295); //@line 1172 "tr_languages.cpp"
        HEAP32[(($stress_rule638)>>2)]=2; //@line 1172 "tr_languages.cpp"
        var $429=$tr; //@line 1173 "tr_languages.cpp"
        var $langopts639=(($429)&4294967295); //@line 1173 "tr_languages.cpp"
        var $stress_flags640=(($langopts639+12)&4294967295); //@line 1173 "tr_languages.cpp"
        HEAP32[(($stress_flags640)>>2)]=22; //@line 1173 "tr_languages.cpp"
        var $430=$tr; //@line 1175 "tr_languages.cpp"
        var $langopts641=(($430)&4294967295); //@line 1175 "tr_languages.cpp"
        var $numbers642=(($langopts641+184)&4294967295); //@line 1175 "tr_languages.cpp"
        HEAP32[(($numbers642)>>2)]=1248; //@line 1175 "tr_languages.cpp"
        var $431=$tr; //@line 1176 "tr_languages.cpp"
        var $langopts643=(($431)&4294967295); //@line 1176 "tr_languages.cpp"
        var $break_numbers644=(($langopts643+192)&4294967295); //@line 1176 "tr_languages.cpp"
        HEAP32[(($break_numbers644)>>2)]=1227133544; //@line 1176 "tr_languages.cpp"
        __label__ = 80;break $sw_default$$sw_bb$$sw_bb9$$sw_bb18$$sw_bb24$$sw_bb39$$sw_bb52$$sw_bb64$$sw_bb72$$sw_bb92$$sw_bb109$$sw_bb123$$sw_bb148$$sw_bb161$$sw_bb195$$sw_bb200$$sw_bb207$$sw_bb209$$sw_bb222$$sw_bb238$$sw_bb257$$sw_bb280$$sw_bb287$$sw_bb312$$sw_bb322$$sw_bb331$$sw_bb347$$sw_bb381$$sw_bb391$$sw_bb403$$sw_bb414$$sw_bb422$$sw_bb439$$sw_bb449$$sw_bb461$$sw_bb481$$sw_bb486$$sw_bb491$$sw_bb509$$sw_bb523$$sw_bb533$$sw_bb534$$sw_bb547$$sw_bb576$$sw_bb599$$sw_bb614$$sw_bb623$$sw_bb630$$sw_bb645$$sw_bb692$$sw_bb702$$sw_bb711$$sw_bb729$$sw_bb734$6; //@line 1178 "tr_languages.cpp"
      }
      else if (__label__ == 62) {
  
        var $432=$tr; //@line 1186 "tr_languages.cpp"
        __Z15SetupTranslatorP10TranslatorPKsPKh($432, ((__ZZ16SelectTranslatorPKcE17stress_lengths_ta)&4294967295), ((__ZZ16SelectTranslatorPKcE14stress_amps_ta)&4294967295)); //@line 1186 "tr_languages.cpp"
        var $433=$tr; //@line 1187 "tr_languages.cpp"
        var $langopts646=(($433)&4294967295); //@line 1187 "tr_languages.cpp"
        var $length_mods647=(($langopts646+176)&4294967295); //@line 1187 "tr_languages.cpp"
        var $434=HEAP32[(($length_mods647)>>2)]; //@line 1187 "tr_languages.cpp"
        var $435=$tr; //@line 1187 "tr_languages.cpp"
        var $langopts648=(($435)&4294967295); //@line 1187 "tr_languages.cpp"
        var $length_mods0649=(($langopts648+180)&4294967295); //@line 1187 "tr_languages.cpp"
        HEAP32[(($length_mods0649)>>2)]=$434; //@line 1187 "tr_languages.cpp"
        var $436=$tr; //@line 1189 "tr_languages.cpp"
        var $langopts650=(($436)&4294967295); //@line 1189 "tr_languages.cpp"
        var $stress_rule651=(($langopts650+8)&4294967295); //@line 1189 "tr_languages.cpp"
        HEAP32[(($stress_rule651)>>2)]=0; //@line 1189 "tr_languages.cpp"
        var $437=$tr; //@line 1190 "tr_languages.cpp"
        var $langopts652=(($437)&4294967295); //@line 1190 "tr_languages.cpp"
        var $stress_flags653=(($langopts652+12)&4294967295); //@line 1190 "tr_languages.cpp"
        HEAP32[(($stress_flags653)>>2)]=65540; //@line 1190 "tr_languages.cpp"
        var $438=$tr; //@line 1191 "tr_languages.cpp"
        var $langopts654=(($438)&4294967295); //@line 1191 "tr_languages.cpp"
        var $spelling_stress655=(($langopts654+249)&4294967295); //@line 1191 "tr_languages.cpp"
        HEAP8[($spelling_stress655)]=1; //@line 1191 "tr_languages.cpp"
        var $439=$tr; //@line 1192 "tr_languages.cpp"
        var $langopts656=(($439)&4294967295); //@line 1192 "tr_languages.cpp"
        var $break_numbers657=(($langopts656+192)&4294967295); //@line 1192 "tr_languages.cpp"
        HEAP32[(($break_numbers657)>>2)]=5288; //@line 1192 "tr_languages.cpp"
        var $440=$name2; //@line 1194 "tr_languages.cpp"
        var $cmp658=((($440))|0)==29793; //@line 1194 "tr_languages.cpp"
        if ($cmp658) { __lastLabel__ = 62; __label__ = 63;; } else { __lastLabel__ = 62; __label__ = 64;; } //@line 1194 "tr_languages.cpp"
        if (__label__ == 63) {
  
          var $441=$tr; //@line 1196 "tr_languages.cpp"
          var $letter_bits_offset660=(($441+964)&4294967295); //@line 1196 "tr_languages.cpp"
          HEAP32[(($letter_bits_offset660)>>2)]=2944; //@line 1196 "tr_languages.cpp"
          var $442=$tr; //@line 1197 "tr_languages.cpp"
          var $langopts661=(($442)&4294967295); //@line 1197 "tr_languages.cpp"
          var $numbers662=(($langopts661+184)&4294967295); //@line 1197 "tr_languages.cpp"
          HEAP32[(($numbers662)>>2)]=2097152; //@line 1197 "tr_languages.cpp"
          var $_pr=$name2; //@line 1199 "tr_languages.cpp"
          __lastLabel__ = 63; ; //@line 1198 "tr_languages.cpp"
        }
  
        var $443=__lastLabel__ == 63 ? $_pr : ($440); //@line 1199 "tr_languages.cpp"
        var $cmp664=((($443))|0)==28018; //@line 1199 "tr_languages.cpp"
        if ($cmp664) { __label__ = 65;; } else { __label__ = 66;; } //@line 1199 "tr_languages.cpp"
        $if_then665$$if_else667$75: do { 
          if (__label__ == 65) {
  
            var $444=$tr; //@line 1201 "tr_languages.cpp"
            var $letter_bits_offset666=(($444+964)&4294967295); //@line 1201 "tr_languages.cpp"
            HEAP32[(($letter_bits_offset666)>>2)]=2304; //@line 1201 "tr_languages.cpp"
            ; //@line 1202 "tr_languages.cpp"
          }
          else if (__label__ == 66) {
  
            var $445=$name2; //@line 1204 "tr_languages.cpp"
            var $cmp668=((($445))|0)==28012; //@line 1204 "tr_languages.cpp"
            if ($cmp668) { __label__ = 67;; } else { __label__ = 68;; } //@line 1204 "tr_languages.cpp"
            if (__label__ == 67) {
  
              var $446=$tr; //@line 1206 "tr_languages.cpp"
              var $letter_bits_offset670=(($446+964)&4294967295); //@line 1206 "tr_languages.cpp"
              HEAP32[(($letter_bits_offset670)>>2)]=3328; //@line 1206 "tr_languages.cpp"
              var $447=$tr; //@line 1207 "tr_languages.cpp"
              var $langopts671=(($447)&4294967295); //@line 1207 "tr_languages.cpp"
              var $numbers672=(($langopts671+184)&4294967295); //@line 1207 "tr_languages.cpp"
              HEAP32[(($numbers672)>>2)]=2097152; //@line 1207 "tr_languages.cpp"
              ; //@line 1208 "tr_languages.cpp"
            }
            else if (__label__ == 68) {
  
              var $448=$name2; //@line 1210 "tr_languages.cpp"
              var $cmp674=((($448))|0)==27502; //@line 1210 "tr_languages.cpp"
              if ($cmp674) { __label__ = 69;; } else { __label__ = 70;; } //@line 1210 "tr_languages.cpp"
              if (__label__ == 69) {
  
                var $449=$tr; //@line 1212 "tr_languages.cpp"
                var $letter_bits_offset676=(($449+964)&4294967295); //@line 1212 "tr_languages.cpp"
                HEAP32[(($letter_bits_offset676)>>2)]=3200; //@line 1212 "tr_languages.cpp"
                var $450=$tr; //@line 1213 "tr_languages.cpp"
                var $langopts677=(($450)&4294967295); //@line 1213 "tr_languages.cpp"
                var $numbers678=(($langopts677+184)&4294967295); //@line 1213 "tr_languages.cpp"
                HEAP32[(($numbers678)>>2)]=1; //@line 1213 "tr_languages.cpp"
                ; //@line 1214 "tr_languages.cpp"
              }
              else if (__label__ == 70) {
  
                var $451=$name2; //@line 1216 "tr_languages.cpp"
                var $cmp680=((($451))|0)==29797; //@line 1216 "tr_languages.cpp"
                if (!($cmp680)) { __label__ = 72;break $if_then665$$if_else667$75; } //@line 1216 "tr_languages.cpp"
  
                var $452=$tr; //@line 1218 "tr_languages.cpp"
                var $letter_bits_offset682=(($452+964)&4294967295); //@line 1218 "tr_languages.cpp"
                HEAP32[(($letter_bits_offset682)>>2)]=3072; //@line 1218 "tr_languages.cpp"
                var $453=$tr; //@line 1219 "tr_languages.cpp"
                var $langopts683=(($453)&4294967295); //@line 1219 "tr_languages.cpp"
                var $numbers684=(($langopts683+184)&4294967295); //@line 1219 "tr_languages.cpp"
                HEAP32[(($numbers684)>>2)]=1; //@line 1219 "tr_languages.cpp"
                ; //@line 1220 "tr_languages.cpp"
              }
            }
          }
        } while(0);
  
        var $454=$tr; //@line 1221 "tr_languages.cpp"
        var $langopts689=(($454)&4294967295); //@line 1221 "tr_languages.cpp"
        var $param690=(($langopts689+24)&4294967295); //@line 1221 "tr_languages.cpp"
        var $arrayidx691=(($param690+32)&4294967295); //@line 1221 "tr_languages.cpp"
        HEAP32[(($arrayidx691)>>2)]=1; //@line 1221 "tr_languages.cpp"
        var $455=$tr; //@line 1222 "tr_languages.cpp"
        __Z15SetIndicLettersP10Translator($455); //@line 1222 "tr_languages.cpp"
        var $456=$tr; //@line 1223 "tr_languages.cpp"
        __ZL18SetLetterBitsRangeP10Translatoriii($456, 1, 78, 78); //@line 1223 "tr_languages.cpp"
        __label__ = 80;break $sw_default$$sw_bb$$sw_bb9$$sw_bb18$$sw_bb24$$sw_bb39$$sw_bb52$$sw_bb64$$sw_bb72$$sw_bb92$$sw_bb109$$sw_bb123$$sw_bb148$$sw_bb161$$sw_bb195$$sw_bb200$$sw_bb207$$sw_bb209$$sw_bb222$$sw_bb238$$sw_bb257$$sw_bb280$$sw_bb287$$sw_bb312$$sw_bb322$$sw_bb331$$sw_bb347$$sw_bb381$$sw_bb391$$sw_bb403$$sw_bb414$$sw_bb422$$sw_bb439$$sw_bb449$$sw_bb461$$sw_bb481$$sw_bb486$$sw_bb491$$sw_bb509$$sw_bb523$$sw_bb533$$sw_bb534$$sw_bb547$$sw_bb576$$sw_bb599$$sw_bb614$$sw_bb623$$sw_bb630$$sw_bb645$$sw_bb692$$sw_bb702$$sw_bb711$$sw_bb729$$sw_bb734$6; //@line 1225 "tr_languages.cpp"
      }
      else if (__label__ == 73) {
  
        var $457=$tr; //@line 1250 "tr_languages.cpp"
        __Z15SetupTranslatorP10TranslatorPKsPKh($457, ((__ZZ16SelectTranslatorPKcE17stress_lengths_tr)&4294967295), ((__ZZ16SelectTranslatorPKcE14stress_amps_tr)&4294967295)); //@line 1250 "tr_languages.cpp"
        var $458=HEAP32[((((_charsets+36)&4294967295))>>2)]; //@line 1251 "tr_languages.cpp"
        var $459=$tr; //@line 1251 "tr_languages.cpp"
        var $charset_a0693=(($459+696)&4294967295); //@line 1251 "tr_languages.cpp"
        HEAP32[(($charset_a0693)>>2)]=$458; //@line 1251 "tr_languages.cpp"
        var $460=$tr; //@line 1253 "tr_languages.cpp"
        var $langopts694=(($460)&4294967295); //@line 1253 "tr_languages.cpp"
        var $stress_rule695=(($langopts694+8)&4294967295); //@line 1253 "tr_languages.cpp"
        HEAP32[(($stress_rule695)>>2)]=7; //@line 1253 "tr_languages.cpp"
        var $461=$tr; //@line 1254 "tr_languages.cpp"
        var $langopts696=(($461)&4294967295); //@line 1254 "tr_languages.cpp"
        var $stress_flags697=(($langopts696+12)&4294967295); //@line 1254 "tr_languages.cpp"
        HEAP32[(($stress_flags697)>>2)]=32; //@line 1254 "tr_languages.cpp"
        var $462=$tr; //@line 1256 "tr_languages.cpp"
        var $langopts698=(($462)&4294967295); //@line 1256 "tr_languages.cpp"
        var $numbers699=(($langopts698+184)&4294967295); //@line 1256 "tr_languages.cpp"
        HEAP32[(($numbers699)>>2)]=2114824; //@line 1256 "tr_languages.cpp"
        var $463=$tr; //@line 1257 "tr_languages.cpp"
        var $langopts700=(($463)&4294967295); //@line 1257 "tr_languages.cpp"
        var $max_initial_consonants701=(($langopts700+248)&4294967295); //@line 1257 "tr_languages.cpp"
        HEAP8[($max_initial_consonants701)]=2; //@line 1257 "tr_languages.cpp"
        __label__ = 80;break $sw_default$$sw_bb$$sw_bb9$$sw_bb18$$sw_bb24$$sw_bb39$$sw_bb52$$sw_bb64$$sw_bb72$$sw_bb92$$sw_bb109$$sw_bb123$$sw_bb148$$sw_bb161$$sw_bb195$$sw_bb200$$sw_bb207$$sw_bb209$$sw_bb222$$sw_bb238$$sw_bb257$$sw_bb280$$sw_bb287$$sw_bb312$$sw_bb322$$sw_bb331$$sw_bb347$$sw_bb381$$sw_bb391$$sw_bb403$$sw_bb414$$sw_bb422$$sw_bb439$$sw_bb449$$sw_bb461$$sw_bb481$$sw_bb486$$sw_bb491$$sw_bb509$$sw_bb523$$sw_bb533$$sw_bb534$$sw_bb547$$sw_bb576$$sw_bb599$$sw_bb614$$sw_bb623$$sw_bb630$$sw_bb645$$sw_bb692$$sw_bb702$$sw_bb711$$sw_bb729$$sw_bb734$6; //@line 1259 "tr_languages.cpp"
      }
      else if (__label__ == 74) {
  
        var $464=$tr; //@line 1262 "tr_languages.cpp"
        var $letter_bits_offset703=(($464+964)&4294967295); //@line 1262 "tr_languages.cpp"
        HEAP32[(($letter_bits_offset703)>>2)]=1536; //@line 1262 "tr_languages.cpp"
        var $465=$tr; //@line 1263 "tr_languages.cpp"
        var $langopts704=(($465)&4294967295); //@line 1263 "tr_languages.cpp"
        var $param705=(($langopts704+24)&4294967295); //@line 1263 "tr_languages.cpp"
        var $arrayidx706=(($param705+20)&4294967295); //@line 1263 "tr_languages.cpp"
        HEAP32[(($arrayidx706)>>2)]=1; //@line 1263 "tr_languages.cpp"
        var $466=$tr; //@line 1264 "tr_languages.cpp"
        var $langopts707=(($466)&4294967295); //@line 1264 "tr_languages.cpp"
        var $numbers708=(($langopts707+184)&4294967295); //@line 1264 "tr_languages.cpp"
        HEAP32[(($numbers708)>>2)]=16; //@line 1264 "tr_languages.cpp"
        var $467=$tr; //@line 1265 "tr_languages.cpp"
        var $langopts709=(($467)&4294967295); //@line 1265 "tr_languages.cpp"
        var $break_numbers710=(($langopts709+192)&4294967295); //@line 1265 "tr_languages.cpp"
        HEAP32[(($break_numbers710)>>2)]=21160; //@line 1265 "tr_languages.cpp"
        __label__ = 80;break $sw_default$$sw_bb$$sw_bb9$$sw_bb18$$sw_bb24$$sw_bb39$$sw_bb52$$sw_bb64$$sw_bb72$$sw_bb92$$sw_bb109$$sw_bb123$$sw_bb148$$sw_bb161$$sw_bb195$$sw_bb200$$sw_bb207$$sw_bb209$$sw_bb222$$sw_bb238$$sw_bb257$$sw_bb280$$sw_bb287$$sw_bb312$$sw_bb322$$sw_bb331$$sw_bb347$$sw_bb381$$sw_bb391$$sw_bb403$$sw_bb414$$sw_bb422$$sw_bb439$$sw_bb449$$sw_bb461$$sw_bb481$$sw_bb486$$sw_bb491$$sw_bb509$$sw_bb523$$sw_bb533$$sw_bb534$$sw_bb547$$sw_bb576$$sw_bb599$$sw_bb614$$sw_bb623$$sw_bb630$$sw_bb645$$sw_bb692$$sw_bb702$$sw_bb711$$sw_bb729$$sw_bb734$6; //@line 1266 "tr_languages.cpp"
      }
      else if (__label__ == 75) {
  
        var $468=$tr; //@line 1286 "tr_languages.cpp"
        __Z15SetupTranslatorP10TranslatorPKsPKh($468, ((__ZZ16SelectTranslatorPKcE17stress_lengths_vi)&4294967295), ((__ZZ16SelectTranslatorPKcE14stress_amps_vi)&4294967295)); //@line 1286 "tr_languages.cpp"
        var $469=$tr; //@line 1287 "tr_languages.cpp"
        var $langopts712=(($469)&4294967295); //@line 1287 "tr_languages.cpp"
        var $length_mods713=(($langopts712+176)&4294967295); //@line 1287 "tr_languages.cpp"
        var $470=HEAP32[(($length_mods713)>>2)]; //@line 1287 "tr_languages.cpp"
        var $471=$tr; //@line 1287 "tr_languages.cpp"
        var $langopts714=(($471)&4294967295); //@line 1287 "tr_languages.cpp"
        var $length_mods0715=(($langopts714+180)&4294967295); //@line 1287 "tr_languages.cpp"
        HEAP32[(($length_mods0715)>>2)]=$470; //@line 1287 "tr_languages.cpp"
        var $472=$tr; //@line 1289 "tr_languages.cpp"
        var $langopts716=(($472)&4294967295); //@line 1289 "tr_languages.cpp"
        var $stress_rule717=(($langopts716+8)&4294967295); //@line 1289 "tr_languages.cpp"
        HEAP32[(($stress_rule717)>>2)]=0; //@line 1289 "tr_languages.cpp"
        var $473=$tr; //@line 1290 "tr_languages.cpp"
        var $langopts718=(($473)&4294967295); //@line 1290 "tr_languages.cpp"
        var $word_gap719=(($langopts718)&4294967295); //@line 1290 "tr_languages.cpp"
        HEAP32[(($word_gap719)>>2)]=33; //@line 1290 "tr_languages.cpp"
        var $474=$tr; //@line 1292 "tr_languages.cpp"
        var $letter_groups720=(($474+968)&4294967295); //@line 1292 "tr_languages.cpp"
        var $arrayidx721=(($letter_groups720+28)&4294967295); //@line 1292 "tr_languages.cpp"
        HEAP32[(($arrayidx721)>>2)]=((__ZZ16SelectTranslatorPKcE9vowels_vi)&4294967295); //@line 1292 "tr_languages.cpp"
        var $475=$tr; //@line 1292 "tr_languages.cpp"
        var $letter_groups722=(($475+968)&4294967295); //@line 1292 "tr_languages.cpp"
        var $arrayidx723=(($letter_groups722)&4294967295); //@line 1292 "tr_languages.cpp"
        HEAP32[(($arrayidx723)>>2)]=((__ZZ16SelectTranslatorPKcE9vowels_vi)&4294967295); //@line 1292 "tr_languages.cpp"
        var $476=$tr; //@line 1293 "tr_languages.cpp"
        var $langopts724=(($476)&4294967295); //@line 1293 "tr_languages.cpp"
        var $tone_language=(($langopts724+224)&4294967295); //@line 1293 "tr_languages.cpp"
        HEAP32[(($tone_language)>>2)]=1; //@line 1293 "tr_languages.cpp"
        var $477=$tr; //@line 1294 "tr_languages.cpp"
        var $langopts725=(($477)&4294967295); //@line 1294 "tr_languages.cpp"
        var $unstressed_wd1726=(($langopts725+16)&4294967295); //@line 1294 "tr_languages.cpp"
        HEAP32[(($unstressed_wd1726)>>2)]=2; //@line 1294 "tr_languages.cpp"
        var $478=$tr; //@line 1295 "tr_languages.cpp"
        var $langopts727=(($478)&4294967295); //@line 1295 "tr_languages.cpp"
        var $numbers728=(($langopts727+184)&4294967295); //@line 1295 "tr_languages.cpp"
        HEAP32[(($numbers728)>>2)]=12615688; //@line 1295 "tr_languages.cpp"
        __label__ = 80;break $sw_default$$sw_bb$$sw_bb9$$sw_bb18$$sw_bb24$$sw_bb39$$sw_bb52$$sw_bb64$$sw_bb72$$sw_bb92$$sw_bb109$$sw_bb123$$sw_bb148$$sw_bb161$$sw_bb195$$sw_bb200$$sw_bb207$$sw_bb209$$sw_bb222$$sw_bb238$$sw_bb257$$sw_bb280$$sw_bb287$$sw_bb312$$sw_bb322$$sw_bb331$$sw_bb347$$sw_bb381$$sw_bb391$$sw_bb403$$sw_bb414$$sw_bb422$$sw_bb439$$sw_bb449$$sw_bb461$$sw_bb481$$sw_bb486$$sw_bb491$$sw_bb509$$sw_bb523$$sw_bb533$$sw_bb534$$sw_bb547$$sw_bb576$$sw_bb599$$sw_bb614$$sw_bb623$$sw_bb630$$sw_bb645$$sw_bb692$$sw_bb702$$sw_bb711$$sw_bb729$$sw_bb734$6; //@line 1298 "tr_languages.cpp"
      }
      else if (__label__ == 76) {
  
        var $479=$tr; //@line 1301 "tr_languages.cpp"
        var $langopts730=(($479)&4294967295); //@line 1301 "tr_languages.cpp"
        var $stress_rule731=(($langopts730+8)&4294967295); //@line 1301 "tr_languages.cpp"
        HEAP32[(($stress_rule731)>>2)]=0; //@line 1301 "tr_languages.cpp"
        var $480=$tr; //@line 1302 "tr_languages.cpp"
        var $langopts732=(($480)&4294967295); //@line 1302 "tr_languages.cpp"
        var $numbers733=(($langopts732+184)&4294967295); //@line 1302 "tr_languages.cpp"
        HEAP32[(($numbers733)>>2)]=2098528; //@line 1302 "tr_languages.cpp"
        __label__ = 80;break $sw_default$$sw_bb$$sw_bb9$$sw_bb18$$sw_bb24$$sw_bb39$$sw_bb52$$sw_bb64$$sw_bb72$$sw_bb92$$sw_bb109$$sw_bb123$$sw_bb148$$sw_bb161$$sw_bb195$$sw_bb200$$sw_bb207$$sw_bb209$$sw_bb222$$sw_bb238$$sw_bb257$$sw_bb280$$sw_bb287$$sw_bb312$$sw_bb322$$sw_bb331$$sw_bb347$$sw_bb381$$sw_bb391$$sw_bb403$$sw_bb414$$sw_bb422$$sw_bb439$$sw_bb449$$sw_bb461$$sw_bb481$$sw_bb486$$sw_bb491$$sw_bb509$$sw_bb523$$sw_bb533$$sw_bb534$$sw_bb547$$sw_bb576$$sw_bb599$$sw_bb614$$sw_bb623$$sw_bb630$$sw_bb645$$sw_bb692$$sw_bb702$$sw_bb711$$sw_bb729$$sw_bb734$6; //@line 1303 "tr_languages.cpp"
      }
      else if (__label__ == 77) {
  
        var $481=$tr; //@line 1311 "tr_languages.cpp"
        __Z15SetupTranslatorP10TranslatorPKsPKh($481, ((__ZZ16SelectTranslatorPKcE17stress_lengths_zh)&4294967295), ((__ZZ16SelectTranslatorPKcE14stress_amps_zh)&4294967295)); //@line 1311 "tr_languages.cpp"
        var $482=$tr; //@line 1313 "tr_languages.cpp"
        var $langopts735=(($482)&4294967295); //@line 1313 "tr_languages.cpp"
        var $stress_rule736=(($langopts735+8)&4294967295); //@line 1313 "tr_languages.cpp"
        HEAP32[(($stress_rule736)>>2)]=3; //@line 1313 "tr_languages.cpp"
        var $483=$tr; //@line 1314 "tr_languages.cpp"
        var $langopts737=(($483)&4294967295); //@line 1314 "tr_languages.cpp"
        var $stress_flags738=(($langopts737+12)&4294967295); //@line 1314 "tr_languages.cpp"
        HEAP32[(($stress_flags738)>>2)]=2; //@line 1314 "tr_languages.cpp"
        var $484=$tr; //@line 1315 "tr_languages.cpp"
        var $langopts739=(($484)&4294967295); //@line 1315 "tr_languages.cpp"
        var $vowel_pause740=(($langopts739+4)&4294967295); //@line 1315 "tr_languages.cpp"
        HEAP32[(($vowel_pause740)>>2)]=0; //@line 1315 "tr_languages.cpp"
        var $485=$tr; //@line 1316 "tr_languages.cpp"
        var $langopts741=(($485)&4294967295); //@line 1316 "tr_languages.cpp"
        var $tone_language742=(($langopts741+224)&4294967295); //@line 1316 "tr_languages.cpp"
        HEAP32[(($tone_language742)>>2)]=1; //@line 1316 "tr_languages.cpp"
        var $486=$tr; //@line 1317 "tr_languages.cpp"
        var $langopts743=(($486)&4294967295); //@line 1317 "tr_languages.cpp"
        var $length_mods744=(($langopts743+176)&4294967295); //@line 1317 "tr_languages.cpp"
        var $487=HEAP32[(($length_mods744)>>2)]; //@line 1317 "tr_languages.cpp"
        var $488=$tr; //@line 1317 "tr_languages.cpp"
        var $langopts745=(($488)&4294967295); //@line 1317 "tr_languages.cpp"
        var $length_mods0746=(($langopts745+180)&4294967295); //@line 1317 "tr_languages.cpp"
        HEAP32[(($length_mods0746)>>2)]=$487; //@line 1317 "tr_languages.cpp"
        var $489=$tr; //@line 1318 "tr_languages.cpp"
        var $langopts747=(($489)&4294967295); //@line 1318 "tr_languages.cpp"
        var $tone_numbers=(($langopts747+250)&4294967295); //@line 1318 "tr_languages.cpp"
        HEAP8[($tone_numbers)]=1; //@line 1318 "tr_languages.cpp"
        var $490=$tr; //@line 1319 "tr_languages.cpp"
        var $langopts748=(($490)&4294967295); //@line 1319 "tr_languages.cpp"
        var $ideographs=(($langopts748+251)&4294967295); //@line 1319 "tr_languages.cpp"
        HEAP8[($ideographs)]=1; //@line 1319 "tr_languages.cpp"
        var $491=$tr; //@line 1320 "tr_languages.cpp"
        var $langopts749=(($491)&4294967295); //@line 1320 "tr_languages.cpp"
        var $word_gap750=(($langopts749)&4294967295); //@line 1320 "tr_languages.cpp"
        HEAP32[(($word_gap750)>>2)]=33; //@line 1320 "tr_languages.cpp"
        var $492=$name2; //@line 1321 "tr_languages.cpp"
        var $cmp751=((($492))|0)==31336; //@line 1321 "tr_languages.cpp"
        if (!($cmp751)) { __label__ = 80;break $sw_default$$sw_bb$$sw_bb9$$sw_bb18$$sw_bb24$$sw_bb39$$sw_bb52$$sw_bb64$$sw_bb72$$sw_bb92$$sw_bb109$$sw_bb123$$sw_bb148$$sw_bb161$$sw_bb195$$sw_bb200$$sw_bb207$$sw_bb209$$sw_bb222$$sw_bb238$$sw_bb257$$sw_bb280$$sw_bb287$$sw_bb312$$sw_bb322$$sw_bb331$$sw_bb347$$sw_bb381$$sw_bb391$$sw_bb403$$sw_bb414$$sw_bb422$$sw_bb439$$sw_bb449$$sw_bb461$$sw_bb481$$sw_bb486$$sw_bb491$$sw_bb509$$sw_bb523$$sw_bb533$$sw_bb534$$sw_bb547$$sw_bb576$$sw_bb599$$sw_bb614$$sw_bb623$$sw_bb630$$sw_bb645$$sw_bb692$$sw_bb702$$sw_bb711$$sw_bb729$$sw_bb734$6; } //@line 1321 "tr_languages.cpp"
  
        var $493=$tr; //@line 1323 "tr_languages.cpp"
        var $langopts753=(($493)&4294967295); //@line 1323 "tr_languages.cpp"
        var $textmode=(($langopts753+252)&4294967295); //@line 1323 "tr_languages.cpp"
        HEAP8[($textmode)]=1; //@line 1323 "tr_languages.cpp"
        var $494=$tr; //@line 1324 "tr_languages.cpp"
        var $langopts754=(($494)&4294967295); //@line 1324 "tr_languages.cpp"
        var $listx=(($langopts754+260)&4294967295); //@line 1324 "tr_languages.cpp"
        HEAP32[(($listx)>>2)]=1; //@line 1324 "tr_languages.cpp"
        __label__ = 80;break $sw_default$$sw_bb$$sw_bb9$$sw_bb18$$sw_bb24$$sw_bb39$$sw_bb52$$sw_bb64$$sw_bb72$$sw_bb92$$sw_bb109$$sw_bb123$$sw_bb148$$sw_bb161$$sw_bb195$$sw_bb200$$sw_bb207$$sw_bb209$$sw_bb222$$sw_bb238$$sw_bb257$$sw_bb280$$sw_bb287$$sw_bb312$$sw_bb322$$sw_bb331$$sw_bb347$$sw_bb381$$sw_bb391$$sw_bb403$$sw_bb414$$sw_bb422$$sw_bb439$$sw_bb449$$sw_bb461$$sw_bb481$$sw_bb486$$sw_bb491$$sw_bb509$$sw_bb523$$sw_bb533$$sw_bb534$$sw_bb547$$sw_bb576$$sw_bb599$$sw_bb614$$sw_bb623$$sw_bb630$$sw_bb645$$sw_bb692$$sw_bb702$$sw_bb711$$sw_bb729$$sw_bb734$6; //@line 1325 "tr_languages.cpp"
      }
    } while(0);
    if (__label__ == 24) {
  
      var $146=$tr; //@line 645 "tr_languages.cpp"
      __Z15SetupTranslatorP10TranslatorPKsPKh($146, ((__ZZ16SelectTranslatorPKcE17stress_lengths_fi)&4294967295), ((__ZZ16SelectTranslatorPKcE14stress_amps_fi)&4294967295)); //@line 645 "tr_languages.cpp"
      var $147=$tr; //@line 647 "tr_languages.cpp"
      var $langopts210=(($147)&4294967295); //@line 647 "tr_languages.cpp"
      var $stress_rule211=(($langopts210+8)&4294967295); //@line 647 "tr_languages.cpp"
      HEAP32[(($stress_rule211)>>2)]=0; //@line 647 "tr_languages.cpp"
      var $148=$tr; //@line 648 "tr_languages.cpp"
      var $langopts212=(($148)&4294967295); //@line 648 "tr_languages.cpp"
      var $stress_flags213=(($langopts212+12)&4294967295); //@line 648 "tr_languages.cpp"
      HEAP32[(($stress_flags213)>>2)]=86; //@line 648 "tr_languages.cpp"
      var $149=$tr; //@line 649 "tr_languages.cpp"
      var $langopts214=(($149)&4294967295); //@line 649 "tr_languages.cpp"
      var $param215=(($langopts214+24)&4294967295); //@line 649 "tr_languages.cpp"
      var $arrayidx216=(($param215+56)&4294967295); //@line 649 "tr_languages.cpp"
      HEAP32[(($arrayidx216)>>2)]=1; //@line 649 "tr_languages.cpp"
      var $150=$tr; //@line 650 "tr_languages.cpp"
      var $langopts217=(($150)&4294967295); //@line 650 "tr_languages.cpp"
      var $long_stop=(($langopts217+240)&4294967295); //@line 650 "tr_languages.cpp"
      HEAP32[(($long_stop)>>2)]=130; //@line 650 "tr_languages.cpp"
      var $151=$tr; //@line 652 "tr_languages.cpp"
      var $langopts218=(($151)&4294967295); //@line 652 "tr_languages.cpp"
      var $numbers219=(($langopts218+184)&4294967295); //@line 652 "tr_languages.cpp"
      HEAP32[(($numbers219)>>2)]=4104; //@line 652 "tr_languages.cpp"
      var $152=$tr; //@line 653 "tr_languages.cpp"
      __ZL14SetLetterVowelP10Translatori($152, 121); //@line 653 "tr_languages.cpp"
      var $153=$tr; //@line 655 "tr_languages.cpp"
      var $langopts220=(($153)&4294967295); //@line 655 "tr_languages.cpp"
      var $spelling_stress=(($langopts220+249)&4294967295); //@line 655 "tr_languages.cpp"
      HEAP8[($spelling_stress)]=1; //@line 655 "tr_languages.cpp"
      var $154=$tr; //@line 656 "tr_languages.cpp"
      var $langopts221=(($154)&4294967295); //@line 656 "tr_languages.cpp"
      var $intonation_group=(($langopts221+228)&4294967295); //@line 656 "tr_languages.cpp"
      HEAP32[(($intonation_group)>>2)]=3; //@line 656 "tr_languages.cpp"
      ; //@line 658 "tr_languages.cpp"
    }
  
    var $496=$name2; //@line 1334 "tr_languages.cpp"
    var $497=$tr; //@line 1334 "tr_languages.cpp"
    var $translator_name=(($497+280)&4294967295); //@line 1334 "tr_languages.cpp"
    HEAP32[(($translator_name)>>2)]=$496; //@line 1334 "tr_languages.cpp"
    var $498=$tr; //@line 1336 "tr_languages.cpp"
    var $langopts759=(($498)&4294967295); //@line 1336 "tr_languages.cpp"
    var $numbers760=(($langopts759+184)&4294967295); //@line 1336 "tr_languages.cpp"
    var $499=HEAP32[(($numbers760)>>2)]; //@line 1336 "tr_languages.cpp"
    var $and=($499) & 8; //@line 1336 "tr_languages.cpp"
    var $tobool=((($and))|0)!=0; //@line 1336 "tr_languages.cpp"
    if ($tobool) { __label__ = 81;; } else { __label__ = 82;; } //@line 1336 "tr_languages.cpp"
    if (__label__ == 81) {
  
      var $500=$tr; //@line 1339 "tr_languages.cpp"
      var $langopts762=(($500)&4294967295); //@line 1339 "tr_languages.cpp"
      var $thousands_sep763=(($langopts762+204)&4294967295); //@line 1339 "tr_languages.cpp"
      HEAP32[(($thousands_sep763)>>2)]=46; //@line 1339 "tr_languages.cpp"
      var $501=$tr; //@line 1340 "tr_languages.cpp"
      var $langopts764=(($501)&4294967295); //@line 1340 "tr_languages.cpp"
      var $decimal_sep765=(($langopts764+208)&4294967295); //@line 1340 "tr_languages.cpp"
      HEAP32[(($decimal_sep765)>>2)]=44; //@line 1340 "tr_languages.cpp"
      ; //@line 1341 "tr_languages.cpp"
    }
  
    var $502=$tr; //@line 1342 "tr_languages.cpp"
    var $langopts767=(($502)&4294967295); //@line 1342 "tr_languages.cpp"
    var $numbers768=(($langopts767+184)&4294967295); //@line 1342 "tr_languages.cpp"
    var $503=HEAP32[(($numbers768)>>2)]; //@line 1342 "tr_languages.cpp"
    var $and769=($503) & 4; //@line 1342 "tr_languages.cpp"
    var $tobool770=((($and769))|0)!=0; //@line 1342 "tr_languages.cpp"
    if ($tobool770) { __label__ = 83;; } else { __label__ = 84;; } //@line 1342 "tr_languages.cpp"
    if (__label__ == 83) {
  
      var $504=$tr; //@line 1344 "tr_languages.cpp"
      var $langopts772=(($504)&4294967295); //@line 1344 "tr_languages.cpp"
      var $thousands_sep773=(($langopts772+204)&4294967295); //@line 1344 "tr_languages.cpp"
      HEAP32[(($thousands_sep773)>>2)]=0; //@line 1344 "tr_languages.cpp"
      ; //@line 1345 "tr_languages.cpp"
    }
  
    var $505=$tr; //@line 1347 "tr_languages.cpp"
    ;
    return $505; //@line 1347 "tr_languages.cpp"
    return null;
  }
  

  function __ZL13NewTranslatorv() {
    ;
    var __label__;
  
    var $retval;
    var $tr;
    var $ix;
    var $call=__Z5Alloci(8328); //@line 123 "tr_languages.cpp"
    var $0=$call; //@line 123 "tr_languages.cpp"
    $tr=$0; //@line 123 "tr_languages.cpp"
    var $1=$tr; //@line 124 "tr_languages.cpp"
    var $cmp=((($1))|0)==0; //@line 124 "tr_languages.cpp"
    if ($cmp) { __label__ = 1;; } else { __label__ = 2;; } //@line 124 "tr_languages.cpp"
    if (__label__ == 1) {
  
      $retval=0; //@line 125 "tr_languages.cpp"
      ; //@line 125 "tr_languages.cpp"
    }
    else if (__label__ == 2) {
  
      var $2=HEAP32[((((_charsets+4)&4294967295))>>2)]; //@line 127 "tr_languages.cpp"
      var $3=$tr; //@line 127 "tr_languages.cpp"
      var $charset_a0=(($3+696)&4294967295); //@line 127 "tr_languages.cpp"
      HEAP32[(($charset_a0)>>2)]=$2; //@line 127 "tr_languages.cpp"
      HEAP8[(((_dictionary_name)&4294967295))]=0; //@line 128 "tr_languages.cpp"
      var $4=$tr; //@line 129 "tr_languages.cpp"
      var $dict_condition=(($4+692)&4294967295); //@line 129 "tr_languages.cpp"
      HEAP32[(($dict_condition)>>2)]=0; //@line 129 "tr_languages.cpp"
      var $5=$tr; //@line 130 "tr_languages.cpp"
      var $data_dictrules=(($5+1048)&4294967295); //@line 130 "tr_languages.cpp"
      HEAP32[(($data_dictrules)>>2)]=0; //@line 130 "tr_languages.cpp"
      var $6=$tr; //@line 131 "tr_languages.cpp"
      var $data_dictlist=(($6+1052)&4294967295); //@line 131 "tr_languages.cpp"
      HEAP32[(($data_dictlist)>>2)]=0; //@line 131 "tr_languages.cpp"
      var $7=$tr; //@line 133 "tr_languages.cpp"
      var $transpose_min=(($7+288)&4294967295); //@line 133 "tr_languages.cpp"
      HEAP32[(($transpose_min)>>2)]=97; //@line 133 "tr_languages.cpp"
      var $8=$tr; //@line 134 "tr_languages.cpp"
      var $transpose_max=(($8+284)&4294967295); //@line 134 "tr_languages.cpp"
      HEAP32[(($transpose_max)>>2)]=122; //@line 134 "tr_languages.cpp"
      var $9=$tr; //@line 135 "tr_languages.cpp"
      var $frequent_pairs=(($9+8268)&4294967295); //@line 135 "tr_languages.cpp"
      HEAP32[(($frequent_pairs)>>2)]=0; //@line 135 "tr_languages.cpp"
      var $10=$tr; //@line 138 "tr_languages.cpp"
      var $letter_bits_offset=(($10+964)&4294967295); //@line 138 "tr_languages.cpp"
      HEAP32[(($letter_bits_offset)>>2)]=0; //@line 138 "tr_languages.cpp"
      var $11=$tr; //@line 139 "tr_languages.cpp"
      var $letter_bits=(($11+708)&4294967295); //@line 139 "tr_languages.cpp"
      var $arraydecay=(($letter_bits)&4294967295); //@line 139 "tr_languages.cpp"
      var $dest$, $stop$, $stop4$, $fast$, $value4$;
      $dest$ = $arraydecay;
      $stop$ = $dest$ + 256;
      $value4$ = 0;
      if ($value4$ < 0) $value4$ += 256;
      $value4$ = $value4$ + ($value4$<<8) + ($value4$<<16) + ($value4$*16777216);
      while ($dest$%4 !== 0 && $dest$ < $stop$) {
        ;; HEAP8[$dest$++] = 0;
      }
      $dest$ >>= 2;
      $stop4$ = $stop$ >> 2;
      while ($dest$ < $stop4$) {
      ;;;;;;;  HEAP32[$dest$++] = $value4$;
      }
      $dest$ <<= 2;
      while ($dest$ < $stop$) {
        ;; HEAP8[$dest$++] = 0;
      }; //@line 139 "tr_languages.cpp"
      var $12=$tr; //@line 140 "tr_languages.cpp"
      var $letter_groups=(($12+968)&4294967295); //@line 140 "tr_languages.cpp"
      var $arraydecay1=(($letter_groups)&4294967295); //@line 140 "tr_languages.cpp"
      var $13=$arraydecay1; //@line 140 "tr_languages.cpp"
      var $dest$, $stop$, $stop4$, $fast$, $value4$;
      $dest$ = $13;
      $stop$ = $dest$ + 32;
      $value4$ = 0;
      if ($value4$ < 0) $value4$ += 256;
      $value4$ = $value4$ + ($value4$<<8) + ($value4$<<16) + ($value4$*16777216);
      while ($dest$%4 !== 0 && $dest$ < $stop$) {
        ;; HEAP8[$dest$++] = 0;
      }
      $dest$ >>= 2;
      $stop4$ = $stop$ >> 2;
      while ($dest$ < $stop4$) {
      ;;;;;;;  HEAP32[$dest$++] = $value4$;
      }
      $dest$ <<= 2;
      while ($dest$ < $stop$) {
        ;; HEAP8[$dest$++] = 0;
      }; //@line 140 "tr_languages.cpp"
      var $14=$tr; //@line 144 "tr_languages.cpp"
      __ZL13SetLetterBitsP10TranslatoriPKc($14, 0, ((__str6636)&4294967295)); //@line 144 "tr_languages.cpp"
      var $15=$tr; //@line 145 "tr_languages.cpp"
      __ZL13SetLetterBitsP10TranslatoriPKc($15, 1, ((__str7637)&4294967295)); //@line 145 "tr_languages.cpp"
      var $16=$tr; //@line 146 "tr_languages.cpp"
      __ZL13SetLetterBitsP10TranslatoriPKc($16, 2, ((__str8638)&4294967295)); //@line 146 "tr_languages.cpp"
      var $17=$tr; //@line 147 "tr_languages.cpp"
      __ZL13SetLetterBitsP10TranslatoriPKc($17, 3, ((__str9639)&4294967295)); //@line 147 "tr_languages.cpp"
      var $18=$tr; //@line 148 "tr_languages.cpp"
      __ZL13SetLetterBitsP10TranslatoriPKc($18, 4, ((__str10640)&4294967295)); //@line 148 "tr_languages.cpp"
      var $19=$tr; //@line 149 "tr_languages.cpp"
      __ZL13SetLetterBitsP10TranslatoriPKc($19, 5, ((__str11641)&4294967295)); //@line 149 "tr_languages.cpp"
      var $20=$tr; //@line 150 "tr_languages.cpp"
      __ZL13SetLetterBitsP10TranslatoriPKc($20, 6, ((__str12642)&4294967295)); //@line 150 "tr_languages.cpp"
      var $21=$tr; //@line 151 "tr_languages.cpp"
      __ZL13SetLetterBitsP10TranslatoriPKc($21, 7, ((__str630)&4294967295)); //@line 151 "tr_languages.cpp"
      var $22=$tr; //@line 154 "tr_languages.cpp"
      var $char_plus_apostrophe=(($22+700)&4294967295); //@line 154 "tr_languages.cpp"
      HEAP32[(($char_plus_apostrophe)>>2)]=((__ZZL13NewTranslatorvE13empty_wstring)&4294967295); //@line 154 "tr_languages.cpp"
      var $23=$tr; //@line 155 "tr_languages.cpp"
      var $punct_within_word=(($23+704)&4294967295); //@line 155 "tr_languages.cpp"
      HEAP32[(($punct_within_word)>>2)]=((__ZZL13NewTranslatorvE13punct_in_word)&4294967295); //@line 155 "tr_languages.cpp"
      $ix=0; //@line 157 "tr_languages.cpp"
      ; //@line 157 "tr_languages.cpp"
      while(1) { 
  
        var $24=$ix; //@line 159 "tr_languages.cpp"
        var $arrayidx=((__ZZL13NewTranslatorvE12stress_amps2+$24)&4294967295); //@line 159 "tr_languages.cpp"
        var $25=HEAP8[($arrayidx)]; //@line 159 "tr_languages.cpp"
        var $26=$ix; //@line 159 "tr_languages.cpp"
        var $27=$tr; //@line 159 "tr_languages.cpp"
        var $stress_amps=(($27+660)&4294967295); //@line 159 "tr_languages.cpp"
        var $arrayidx3=(($stress_amps+$26)&4294967295); //@line 159 "tr_languages.cpp"
        HEAP8[($arrayidx3)]=$25; //@line 159 "tr_languages.cpp"
        var $28=$ix; //@line 160 "tr_languages.cpp"
        var $arrayidx4=((__ZZL13NewTranslatorvE12stress_amps2+$28)&4294967295); //@line 160 "tr_languages.cpp"
        var $29=HEAPU8[($arrayidx4)]; //@line 160 "tr_languages.cpp"
        var $conv=((($29))&255); //@line 160 "tr_languages.cpp"
        var $sub=((($conv)-1)&4294967295); //@line 160 "tr_languages.cpp"
        var $conv5=((($sub)) & 255); //@line 160 "tr_languages.cpp"
        var $30=$ix; //@line 160 "tr_languages.cpp"
        var $31=$tr; //@line 160 "tr_languages.cpp"
        var $stress_amps_r=(($31+668)&4294967295); //@line 160 "tr_languages.cpp"
        var $arrayidx6=(($stress_amps_r+$30)&4294967295); //@line 160 "tr_languages.cpp"
        HEAP8[($arrayidx6)]=$conv5; //@line 160 "tr_languages.cpp"
        var $32=$ix; //@line 161 "tr_languages.cpp"
        var $arrayidx7=((__ZZL13NewTranslatorvE15stress_lengths2+$32*2)&4294967295); //@line 161 "tr_languages.cpp"
        var $33=HEAP16[(($arrayidx7)>>1)]; //@line 161 "tr_languages.cpp"
        var $34=$ix; //@line 161 "tr_languages.cpp"
        var $35=$tr; //@line 161 "tr_languages.cpp"
        var $stress_lengths=(($35+676)&4294967295); //@line 161 "tr_languages.cpp"
        var $arrayidx8=(($stress_lengths+$34*2)&4294967295); //@line 161 "tr_languages.cpp"
        HEAP16[(($arrayidx8)>>1)]=$33; //@line 161 "tr_languages.cpp"
        var $36=$ix; //@line 157 "tr_languages.cpp"
        var $inc=((($36)+1)&4294967295); //@line 157 "tr_languages.cpp"
        $ix=$inc; //@line 157 "tr_languages.cpp"
        var $cmp2=((($inc))|0) < 8; //@line 157 "tr_languages.cpp"
        if ($cmp2) { __label__ = 3;continue ; } else { __label__ = 4;break ; } //@line 157 "tr_languages.cpp"
      }
  
      var $37=$tr; //@line 163 "tr_languages.cpp"
      var $langopts=(($37)&4294967295); //@line 163 "tr_languages.cpp"
      var $38=$langopts; //@line 163 "tr_languages.cpp"
      var $dest$, $stop$, $stop4$, $fast$, $value4$;
      $dest$ = $38;
      $stop$ = $dest$ + 280;
      $value4$ = 0;
      if ($value4$ < 0) $value4$ += 256;
      $value4$ = $value4$ + ($value4$<<8) + ($value4$<<16) + ($value4$*16777216);
      while ($dest$%4 !== 0 && $dest$ < $stop$) {
        ;; HEAP8[$dest$++] = 0;
      }
      $dest$ >>= 2;
      $stop4$ = $stop$ >> 2;
      while ($dest$ < $stop4$) {
      ;;;;;;;  HEAP32[$dest$++] = $value4$;
      }
      $dest$ <<= 2;
      while ($dest$ < $stop$) {
        ;; HEAP8[$dest$++] = 0;
      }; //@line 163 "tr_languages.cpp"
      var $39=$tr; //@line 164 "tr_languages.cpp"
      var $langopts9=(($39)&4294967295); //@line 164 "tr_languages.cpp"
      var $max_lengthmod=(($langopts9+272)&4294967295); //@line 164 "tr_languages.cpp"
      HEAP32[(($max_lengthmod)>>2)]=500; //@line 164 "tr_languages.cpp"
      var $40=$tr; //@line 166 "tr_languages.cpp"
      var $langopts10=(($40)&4294967295); //@line 166 "tr_languages.cpp"
      var $stress_rule=(($langopts10+8)&4294967295); //@line 166 "tr_languages.cpp"
      HEAP32[(($stress_rule)>>2)]=2; //@line 166 "tr_languages.cpp"
      var $41=$tr; //@line 167 "tr_languages.cpp"
      var $langopts11=(($41)&4294967295); //@line 167 "tr_languages.cpp"
      var $unstressed_wd1=(($langopts11+16)&4294967295); //@line 167 "tr_languages.cpp"
      HEAP32[(($unstressed_wd1)>>2)]=1; //@line 167 "tr_languages.cpp"
      var $42=$tr; //@line 168 "tr_languages.cpp"
      var $langopts12=(($42)&4294967295); //@line 168 "tr_languages.cpp"
      var $unstressed_wd2=(($langopts12+20)&4294967295); //@line 168 "tr_languages.cpp"
      HEAP32[(($unstressed_wd2)>>2)]=3; //@line 168 "tr_languages.cpp"
      var $43=$tr; //@line 169 "tr_languages.cpp"
      var $langopts13=(($43)&4294967295); //@line 169 "tr_languages.cpp"
      var $param=(($langopts13+24)&4294967295); //@line 169 "tr_languages.cpp"
      var $arrayidx14=(($param+28)&4294967295); //@line 169 "tr_languages.cpp"
      HEAP32[(($arrayidx14)>>2)]=95; //@line 169 "tr_languages.cpp"
      var $44=$tr; //@line 170 "tr_languages.cpp"
      var $langopts15=(($44)&4294967295); //@line 170 "tr_languages.cpp"
      var $param16=(($langopts15+24)&4294967295); //@line 170 "tr_languages.cpp"
      var $arrayidx17=(($param16+72)&4294967295); //@line 170 "tr_languages.cpp"
      HEAP32[(($arrayidx17)>>2)]=95; //@line 170 "tr_languages.cpp"
      var $45=$tr; //@line 171 "tr_languages.cpp"
      var $langopts18=(($45)&4294967295); //@line 171 "tr_languages.cpp"
      var $param19=(($langopts18+24)&4294967295); //@line 171 "tr_languages.cpp"
      var $arrayidx20=(($param19+36)&4294967295); //@line 171 "tr_languages.cpp"
      HEAP32[(($arrayidx20)>>2)]=19; //@line 171 "tr_languages.cpp"
      var $46=$tr; //@line 172 "tr_languages.cpp"
      var $langopts21=(($46)&4294967295); //@line 172 "tr_languages.cpp"
      var $param22=(($langopts21+24)&4294967295); //@line 172 "tr_languages.cpp"
      var $arrayidx23=(($param22+20)&4294967295); //@line 172 "tr_languages.cpp"
      HEAP32[(($arrayidx23)>>2)]=115; //@line 172 "tr_languages.cpp"
      var $47=$tr; //@line 173 "tr_languages.cpp"
      var $langopts24=(($47)&4294967295); //@line 173 "tr_languages.cpp"
      var $param25=(($langopts24+24)&4294967295); //@line 173 "tr_languages.cpp"
      var $arrayidx26=(($param25+64)&4294967295); //@line 173 "tr_languages.cpp"
      HEAP32[(($arrayidx26)>>2)]=4; //@line 173 "tr_languages.cpp"
      var $48=$tr; //@line 174 "tr_languages.cpp"
      var $langopts27=(($48)&4294967295); //@line 174 "tr_languages.cpp"
      var $param2=(($langopts27+100)&4294967295); //@line 174 "tr_languages.cpp"
      var $arrayidx28=(($param2+64)&4294967295); //@line 174 "tr_languages.cpp"
      HEAP32[(($arrayidx28)>>2)]=2; //@line 174 "tr_languages.cpp"
      var $49=$tr; //@line 175 "tr_languages.cpp"
      var $langopts29=(($49)&4294967295); //@line 175 "tr_languages.cpp"
      var $max_initial_consonants=(($langopts29+248)&4294967295); //@line 175 "tr_languages.cpp"
      HEAP8[($max_initial_consonants)]=3; //@line 175 "tr_languages.cpp"
      var $50=$tr; //@line 176 "tr_languages.cpp"
      var $langopts30=(($50)&4294967295); //@line 176 "tr_languages.cpp"
      var $replace_chars=(($langopts30+264)&4294967295); //@line 176 "tr_languages.cpp"
      HEAP32[(($replace_chars)>>2)]=0; //@line 176 "tr_languages.cpp"
      var $51=$tr; //@line 177 "tr_languages.cpp"
      var $langopts31=(($51)&4294967295); //@line 177 "tr_languages.cpp"
      var $ascii_language=(($langopts31+268)&4294967295); //@line 177 "tr_languages.cpp"
      HEAP32[(($ascii_language)>>2)]=((__str13643)&4294967295); //@line 177 "tr_languages.cpp"
      var $52=$tr; //@line 180 "tr_languages.cpp"
      __Z13SetLengthModsP10Translatori($52, 201); //@line 180 "tr_languages.cpp"
      var $53=$tr; //@line 184 "tr_languages.cpp"
      var $langopts32=(($53)&4294967295); //@line 184 "tr_languages.cpp"
      var $long_stop=(($langopts32+240)&4294967295); //@line 184 "tr_languages.cpp"
      HEAP32[(($long_stop)>>2)]=100; //@line 184 "tr_languages.cpp"
      var $54=$tr; //@line 186 "tr_languages.cpp"
      var $langopts33=(($54)&4294967295); //@line 186 "tr_languages.cpp"
      var $max_roman=(($langopts33+196)&4294967295); //@line 186 "tr_languages.cpp"
      HEAP32[(($max_roman)>>2)]=49; //@line 186 "tr_languages.cpp"
      var $55=$tr; //@line 187 "tr_languages.cpp"
      var $langopts34=(($55)&4294967295); //@line 187 "tr_languages.cpp"
      var $min_roman=(($langopts34+200)&4294967295); //@line 187 "tr_languages.cpp"
      HEAP32[(($min_roman)>>2)]=2; //@line 187 "tr_languages.cpp"
      var $56=$tr; //@line 188 "tr_languages.cpp"
      var $langopts35=(($56)&4294967295); //@line 188 "tr_languages.cpp"
      var $thousands_sep=(($langopts35+204)&4294967295); //@line 188 "tr_languages.cpp"
      HEAP32[(($thousands_sep)>>2)]=44; //@line 188 "tr_languages.cpp"
      var $57=$tr; //@line 189 "tr_languages.cpp"
      var $langopts36=(($57)&4294967295); //@line 189 "tr_languages.cpp"
      var $decimal_sep=(($langopts36+208)&4294967295); //@line 189 "tr_languages.cpp"
      HEAP32[(($decimal_sep)>>2)]=46; //@line 189 "tr_languages.cpp"
      var $58=$tr; //@line 190 "tr_languages.cpp"
      var $langopts37=(($58)&4294967295); //@line 190 "tr_languages.cpp"
      var $break_numbers=(($langopts37+192)&4294967295); //@line 190 "tr_languages.cpp"
      HEAP32[(($break_numbers)>>2)]=1227133512; //@line 190 "tr_languages.cpp"
      var $59=$tr; //@line 191 "tr_languages.cpp"
      var $langopts38=(($59)&4294967295); //@line 191 "tr_languages.cpp"
      var $max_digits=(($langopts38+212)&4294967295); //@line 191 "tr_languages.cpp"
      HEAP32[(($max_digits)>>2)]=14; //@line 191 "tr_languages.cpp"
      var $60=$tr; //@line 193 "tr_languages.cpp"
      var $punct_to_tone=(($60+1000)&4294967295); //@line 193 "tr_languages.cpp"
      var $arraydecay39=(($punct_to_tone)&4294967295); //@line 193 "tr_languages.cpp"
      var $61=$arraydecay39; //@line 193 "tr_languages.cpp"
      var $src$, $dest$, $stop$, $stop4$, $fast$;
      $src$ = ((_punctuation_to_tone)&4294967295);
      $dest$ = $61;
      $stop$ = $src$ + 48;
      if (($dest$%4) == ($src$%4) && 48 > 8) {
        while ($src$%4 !== 0 && $src$ < $stop$) {
          ; HEAP8[$dest$++] = HEAP8[$src$++];
        }
        $src$ >>= 2;
        $dest$ >>= 2;
        $stop4$ = $stop$ >> 2;
        while ($src$ < $stop4$) {
      ;;;    HEAP32[$dest$++] = HEAP32[$src$++];
        }
        $src$ <<= 2;
        $dest$ <<= 2;
      }while ($src$ < $stop$) {
        ; HEAP8[$dest$++] = HEAP8[$src$++];
      }; //@line 193 "tr_languages.cpp"
      var $62=$tr; //@line 195 "tr_languages.cpp"
      var $langopts40=(($62)&4294967295); //@line 195 "tr_languages.cpp"
      var $tunes=(($langopts40+232)&4294967295); //@line 195 "tr_languages.cpp"
      var $arraydecay41=(($tunes)&4294967295); //@line 195 "tr_languages.cpp"
      var $src$, $dest$, $stop$, $stop4$, $fast$;
      $src$ = ((__ZZL13NewTranslatorvE13default_tunes)&4294967295);
      $dest$ = $arraydecay41;
      $stop$ = $src$ + 6;
      if (($dest$%4) == ($src$%4) && 6 > 8) {
        while ($src$%4 !== 0 && $src$ < $stop$) {
          ; HEAP8[$dest$++] = HEAP8[$src$++];
        }
        $src$ >>= 2;
        $dest$ >>= 2;
        $stop4$ = $stop$ >> 2;
        while ($src$ < $stop4$) {
      ;;;    HEAP32[$dest$++] = HEAP32[$src$++];
        }
        $src$ <<= 2;
        $dest$ <<= 2;
      }while ($src$ < $stop$) {
        ; HEAP8[$dest$++] = HEAP8[$src$++];
      }; //@line 195 "tr_languages.cpp"
      var $63=$tr; //@line 197 "tr_languages.cpp"
      $retval=$63; //@line 197 "tr_languages.cpp"
      ; //@line 197 "tr_languages.cpp"
    }
  
    var $64=$retval; //@line 198 "tr_languages.cpp"
    ;
    return $64; //@line 198 "tr_languages.cpp"
    return null;
  }
  

  function __ZL14SetLetterVowelP10Translatori($tr, $c) {
    ;
    var __label__;
  
    var $tr_addr;
    var $c_addr;
    $tr_addr=$tr;
    $c_addr=$c;
    var $0=$c_addr; //@line 73 "tr_languages.cpp"
    var $1=$tr_addr; //@line 73 "tr_languages.cpp"
    var $letter_bits=(($1+708)&4294967295); //@line 73 "tr_languages.cpp"
    var $arrayidx=(($letter_bits+$0)&4294967295); //@line 73 "tr_languages.cpp"
    var $2=HEAPU8[($arrayidx)]; //@line 73 "tr_languages.cpp"
    var $conv=((($2))&255); //@line 73 "tr_languages.cpp"
    var $and=($conv) & 64; //@line 73 "tr_languages.cpp"
    var $or=($and) | 129; //@line 73 "tr_languages.cpp"
    var $conv1=((($or)) & 255); //@line 73 "tr_languages.cpp"
    var $3=$c_addr; //@line 73 "tr_languages.cpp"
    var $4=$tr_addr; //@line 73 "tr_languages.cpp"
    var $letter_bits2=(($4+708)&4294967295); //@line 73 "tr_languages.cpp"
    var $arrayidx3=(($letter_bits2+$3)&4294967295); //@line 73 "tr_languages.cpp"
    HEAP8[($arrayidx3)]=$conv1; //@line 73 "tr_languages.cpp"
    ;
    return; //@line 74 "tr_languages.cpp"
    return;
  }
  

  function __ZL18SetCyrillicLettersP10Translator($tr) {
    ;
    var __label__;
  
    var $tr_addr;
    $tr_addr=$tr;
    var $0=HEAP32[((((_charsets+72)&4294967295))>>2)]; //@line 285 "tr_languages.cpp"
    var $1=$tr_addr; //@line 285 "tr_languages.cpp"
    var $charset_a0=(($1+696)&4294967295); //@line 285 "tr_languages.cpp"
    HEAP32[(($charset_a0)>>2)]=$0; //@line 285 "tr_languages.cpp"
    var $2=$tr_addr; //@line 286 "tr_languages.cpp"
    var $transpose_min=(($2+288)&4294967295); //@line 286 "tr_languages.cpp"
    HEAP32[(($transpose_min)>>2)]=1072; //@line 286 "tr_languages.cpp"
    var $3=$tr_addr; //@line 287 "tr_languages.cpp"
    var $transpose_max=(($3+284)&4294967295); //@line 287 "tr_languages.cpp"
    HEAP32[(($transpose_max)>>2)]=1105; //@line 287 "tr_languages.cpp"
    var $4=$tr_addr; //@line 288 "tr_languages.cpp"
    var $frequent_pairs=(($4+8268)&4294967295); //@line 288 "tr_languages.cpp"
    HEAP32[(($frequent_pairs)>>2)]=((__ZL8pairs_ru)&4294967295); //@line 288 "tr_languages.cpp"
    var $5=$tr_addr; //@line 290 "tr_languages.cpp"
    var $letter_bits_offset=(($5+964)&4294967295); //@line 290 "tr_languages.cpp"
    HEAP32[(($letter_bits_offset)>>2)]=1056; //@line 290 "tr_languages.cpp"
    var $6=$tr_addr; //@line 291 "tr_languages.cpp"
    var $letter_bits=(($6+708)&4294967295); //@line 291 "tr_languages.cpp"
    var $arraydecay=(($letter_bits)&4294967295); //@line 291 "tr_languages.cpp"
    var $dest$, $stop$, $stop4$, $fast$, $value4$;
    $dest$ = $arraydecay;
    $stop$ = $dest$ + 256;
    $value4$ = 0;
    if ($value4$ < 0) $value4$ += 256;
    $value4$ = $value4$ + ($value4$<<8) + ($value4$<<16) + ($value4$*16777216);
    while ($dest$%4 !== 0 && $dest$ < $stop$) {
      ;; HEAP8[$dest$++] = 0;
    }
    $dest$ >>= 2;
    $stop4$ = $stop$ >> 2;
    while ($dest$ < $stop4$) {
    ;;;;;;;  HEAP32[$dest$++] = $value4$;
    }
    $dest$ <<= 2;
    while ($dest$ < $stop$) {
      ;; HEAP8[$dest$++] = 0;
    }; //@line 291 "tr_languages.cpp"
    var $7=$tr_addr; //@line 292 "tr_languages.cpp"
    __ZL13SetLetterBitsP10TranslatoriPKc($7, 0, ((__ZZL18SetCyrillicLettersP10TranslatorE9ru_vowels)&4294967295)); //@line 292 "tr_languages.cpp"
    var $8=$tr_addr; //@line 293 "tr_languages.cpp"
    __ZL13SetLetterBitsP10TranslatoriPKc($8, 1, ((__ZZL18SetCyrillicLettersP10TranslatorE7ru_soft)&4294967295)); //@line 293 "tr_languages.cpp"
    var $9=$tr_addr; //@line 294 "tr_languages.cpp"
    __ZL13SetLetterBitsP10TranslatoriPKc($9, 2, ((__ZZL18SetCyrillicLettersP10TranslatorE13ru_consonants)&4294967295)); //@line 294 "tr_languages.cpp"
    var $10=$tr_addr; //@line 295 "tr_languages.cpp"
    __ZL13SetLetterBitsP10TranslatoriPKc($10, 3, ((__ZZL18SetCyrillicLettersP10TranslatorE7ru_hard)&4294967295)); //@line 295 "tr_languages.cpp"
    var $11=$tr_addr; //@line 296 "tr_languages.cpp"
    __ZL13SetLetterBitsP10TranslatoriPKc($11, 4, ((__ZZL18SetCyrillicLettersP10TranslatorE10ru_nothard)&4294967295)); //@line 296 "tr_languages.cpp"
    var $12=$tr_addr; //@line 297 "tr_languages.cpp"
    __ZL13SetLetterBitsP10TranslatoriPKc($12, 5, ((__ZZL18SetCyrillicLettersP10TranslatorE9ru_voiced)&4294967295)); //@line 297 "tr_languages.cpp"
    var $13=$tr_addr; //@line 298 "tr_languages.cpp"
    __ZL13SetLetterBitsP10TranslatoriPKc($13, 6, ((__ZZL18SetCyrillicLettersP10TranslatorE10ru_ivowels)&4294967295)); //@line 298 "tr_languages.cpp"
    var $14=$tr_addr; //@line 299 "tr_languages.cpp"
    __ZL13SetLetterBitsP10TranslatoriPKc($14, 7, ((__ZZL18SetCyrillicLettersP10TranslatorE9ru_vowels)&4294967295)); //@line 299 "tr_languages.cpp"
    ;
    return; //@line 300 "tr_languages.cpp"
    return;
  }
  

  function __ZL15ResetLetterBitsP10Translatori($tr, $groups) {
    ;
    var __label__;
  
    var $tr_addr;
    var $groups_addr;
    var $ix;
    var $mask;
    $tr_addr=$tr;
    $groups_addr=$groups;
    var $0=$groups_addr; //@line 82 "tr_languages.cpp"
    var $neg=($0) ^ -1; //@line 82 "tr_languages.cpp"
    $mask=$neg; //@line 82 "tr_languages.cpp"
    $ix=0; //@line 84 "tr_languages.cpp"
    ; //@line 84 "tr_languages.cpp"
    while(1) { 
  
      var $1=$mask; //@line 86 "tr_languages.cpp"
      var $2=$ix; //@line 86 "tr_languages.cpp"
      var $3=$tr_addr; //@line 86 "tr_languages.cpp"
      var $letter_bits=(($3+708)&4294967295); //@line 86 "tr_languages.cpp"
      var $arrayidx=(($letter_bits+$2)&4294967295); //@line 86 "tr_languages.cpp"
      var $4=HEAPU8[($arrayidx)]; //@line 86 "tr_languages.cpp"
      var $conv=((($4))&255); //@line 86 "tr_languages.cpp"
      var $and=($conv) & ($1); //@line 86 "tr_languages.cpp"
      var $conv1=((($and)) & 255); //@line 86 "tr_languages.cpp"
      HEAP8[($arrayidx)]=$conv1; //@line 86 "tr_languages.cpp"
      var $5=$ix; //@line 84 "tr_languages.cpp"
      var $inc=((($5)+1)&4294967295); //@line 84 "tr_languages.cpp"
      $ix=$inc; //@line 84 "tr_languages.cpp"
      var $cmp=((($inc))>>>0) < 256; //@line 84 "tr_languages.cpp"
      if ($cmp) { __label__ = 1;continue ; } else { __label__ = 2;break ; } //@line 84 "tr_languages.cpp"
    }
  
    ;
    return; //@line 88 "tr_languages.cpp"
    return;
  }
  

  function __ZL18Translator_RussianP10Translator($tr) {
    ;
    var __label__;
  
    var $tr_addr;
    $tr_addr=$tr;
    var $0=$tr_addr; //@line 1363 "tr_languages.cpp"
    __Z15SetupTranslatorP10TranslatorPKsPKh($0, ((__ZZL18Translator_RussianP10TranslatorE17stress_lengths_ru)&4294967295), ((__ZZL18Translator_RussianP10TranslatorE14stress_amps_ru)&4294967295)); //@line 1363 "tr_languages.cpp"
    var $1=$tr_addr; //@line 1364 "tr_languages.cpp"
    __ZL18SetCyrillicLettersP10Translator($1); //@line 1364 "tr_languages.cpp"
    var $2=$tr_addr; //@line 1365 "tr_languages.cpp"
    __ZL13SetLetterBitsP10TranslatoriPKc($2, 6, ((__ZZL18Translator_RussianP10TranslatorE11ru_ivowels2)&4294967295)); //@line 1365 "tr_languages.cpp"
    var $3=$tr_addr; //@line 1367 "tr_languages.cpp"
    var $langopts=(($3)&4294967295); //@line 1367 "tr_languages.cpp"
    var $param=(($langopts+24)&4294967295); //@line 1367 "tr_languages.cpp"
    var $arrayidx=(($param+20)&4294967295); //@line 1367 "tr_languages.cpp"
    HEAP32[(($arrayidx)>>2)]=1074; //@line 1367 "tr_languages.cpp"
    var $4=$tr_addr; //@line 1368 "tr_languages.cpp"
    var $langopts1=(($4)&4294967295); //@line 1368 "tr_languages.cpp"
    var $param2=(($langopts1+24)&4294967295); //@line 1368 "tr_languages.cpp"
    var $arrayidx3=(($param2+16)&4294967295); //@line 1368 "tr_languages.cpp"
    HEAP32[(($arrayidx3)>>2)]=1; //@line 1368 "tr_languages.cpp"
    var $5=$tr_addr; //@line 1369 "tr_languages.cpp"
    var $langopts4=(($5)&4294967295); //@line 1369 "tr_languages.cpp"
    var $param5=(($langopts4+24)&4294967295); //@line 1369 "tr_languages.cpp"
    var $arrayidx6=(($param5+40)&4294967295); //@line 1369 "tr_languages.cpp"
    HEAP32[(($arrayidx6)>>2)]=2; //@line 1369 "tr_languages.cpp"
    var $6=$tr_addr; //@line 1370 "tr_languages.cpp"
    var $langopts7=(($6)&4294967295); //@line 1370 "tr_languages.cpp"
    var $stress_rule=(($langopts7+8)&4294967295); //@line 1370 "tr_languages.cpp"
    HEAP32[(($stress_rule)>>2)]=5; //@line 1370 "tr_languages.cpp"
    var $7=$tr_addr; //@line 1371 "tr_languages.cpp"
    var $langopts8=(($7)&4294967295); //@line 1371 "tr_languages.cpp"
    var $stress_flags=(($langopts8+12)&4294967295); //@line 1371 "tr_languages.cpp"
    HEAP32[(($stress_flags)>>2)]=32; //@line 1371 "tr_languages.cpp"
    var $8=$tr_addr; //@line 1373 "tr_languages.cpp"
    var $langopts9=(($8)&4294967295); //@line 1373 "tr_languages.cpp"
    var $numbers=(($langopts9+184)&4294967295); //@line 1373 "tr_languages.cpp"
    HEAP32[(($numbers)>>2)]=1032; //@line 1373 "tr_languages.cpp"
    var $9=$tr_addr; //@line 1374 "tr_languages.cpp"
    var $langopts10=(($9)&4294967295); //@line 1374 "tr_languages.cpp"
    var $numbers2=(($langopts10+188)&4294967295); //@line 1374 "tr_languages.cpp"
    HEAP32[(($numbers2)>>2)]=194; //@line 1374 "tr_languages.cpp"
    var $10=$tr_addr; //@line 1375 "tr_languages.cpp"
    var $langopts11=(($10)&4294967295); //@line 1375 "tr_languages.cpp"
    var $phoneme_change=(($langopts11+244)&4294967295); //@line 1375 "tr_languages.cpp"
    HEAP32[(($phoneme_change)>>2)]=1; //@line 1375 "tr_languages.cpp"
    var $11=$tr_addr; //@line 1376 "tr_languages.cpp"
    var $langopts12=(($11)&4294967295); //@line 1376 "tr_languages.cpp"
    var $testing=(($langopts12+256)&4294967295); //@line 1376 "tr_languages.cpp"
    HEAP32[(($testing)>>2)]=2; //@line 1376 "tr_languages.cpp"
    ;
    return; //@line 1378 "tr_languages.cpp"
    return;
  }
  

  function __Z14ReadTonePointsPcPi($string, $tone_pts) {
    ;
    var __label__;
  
    var $string_addr;
    var $tone_pts_addr;
    var $ix;
    $string_addr=$string;
    $tone_pts_addr=$tone_pts;
    $ix=0; //@line 256 "voices.cpp"
    ; //@line 256 "voices.cpp"
    while(1) { 
  
      var $0=$ix; //@line 257 "voices.cpp"
      var $1=$tone_pts_addr; //@line 257 "voices.cpp"
      var $arrayidx=(($1+4*$0)&4294967295); //@line 257 "voices.cpp"
      HEAP32[(($arrayidx)>>2)]=-1; //@line 257 "voices.cpp"
      var $2=$ix; //@line 256 "voices.cpp"
      var $inc=((($2)+1)&4294967295); //@line 256 "voices.cpp"
      $ix=$inc; //@line 256 "voices.cpp"
      var $cmp=((($inc))|0) < 12; //@line 256 "voices.cpp"
      if ($cmp) { __label__ = 1;continue ; } else { __label__ = 2;break ; } //@line 256 "voices.cpp"
    }
  
    var $3=$string_addr; //@line 259 "voices.cpp"
    var $4=$tone_pts_addr; //@line 259 "voices.cpp"
    var $arrayidx1=(($4)&4294967295); //@line 259 "voices.cpp"
    var $5=$tone_pts_addr; //@line 259 "voices.cpp"
    var $arrayidx2=(($5+4)&4294967295); //@line 259 "voices.cpp"
    var $6=$tone_pts_addr; //@line 259 "voices.cpp"
    var $arrayidx3=(($6+8)&4294967295); //@line 259 "voices.cpp"
    var $7=$tone_pts_addr; //@line 259 "voices.cpp"
    var $arrayidx4=(($7+12)&4294967295); //@line 259 "voices.cpp"
    var $8=$tone_pts_addr; //@line 259 "voices.cpp"
    var $arrayidx5=(($8+16)&4294967295); //@line 259 "voices.cpp"
    var $9=$tone_pts_addr; //@line 259 "voices.cpp"
    var $arrayidx6=(($9+20)&4294967295); //@line 259 "voices.cpp"
    var $10=$tone_pts_addr; //@line 259 "voices.cpp"
    var $arrayidx7=(($10+24)&4294967295); //@line 259 "voices.cpp"
    var $11=$tone_pts_addr; //@line 259 "voices.cpp"
    var $arrayidx8=(($11+28)&4294967295); //@line 259 "voices.cpp"
    var $12=$tone_pts_addr; //@line 259 "voices.cpp"
    var $arrayidx9=(($12+32)&4294967295); //@line 259 "voices.cpp"
    var $13=$tone_pts_addr; //@line 259 "voices.cpp"
    var $arrayidx10=(($13+36)&4294967295); //@line 259 "voices.cpp"
    var $call=_sscanf($3, ((__str44696)&4294967295), allocate([$arrayidx1,0,0,0,$arrayidx2,0,0,0,$arrayidx3,0,0,0,$arrayidx4,0,0,0,$arrayidx5,0,0,0,$arrayidx6,0,0,0,$arrayidx7,0,0,0,$arrayidx8,0,0,0,$arrayidx9,0,0,0,$arrayidx10,0,0,0], ["i32*",0,0,0,"i32*",0,0,0,"i32*",0,0,0,"i32*",0,0,0,"i32*",0,0,0,"i32*",0,0,0,"i32*",0,0,0,"i32*",0,0,0,"i32*",0,0,0,"i32*",0,0,0], ALLOC_STACK)); //@line 259 "voices.cpp"
    ;
    return; //@line 263 "voices.cpp"
    return;
  }
  

  function __Z10VoiceReseti($tone_only) {
    ;
    var __label__;
  
    var $tone_only_addr;
    var $pk;
    $tone_only_addr=$tone_only;
    var $0=HEAP32[((_voice)>>2)]; //@line 393 "voices.cpp"
    var $pitch_base=(($0+44)&4294967295); //@line 393 "voices.cpp"
    HEAP32[(($pitch_base)>>2)]=290816; //@line 393 "voices.cpp"
    var $1=HEAP32[((_voice)>>2)]; //@line 394 "voices.cpp"
    var $pitch_range=(($1+48)&4294967295); //@line 394 "voices.cpp"
    HEAP32[(($pitch_range)>>2)]=4104; //@line 394 "voices.cpp"
    var $2=HEAP32[((_voice)>>2)]; //@line 400 "voices.cpp"
    var $formant_factor=(($2+96)&4294967295); //@line 400 "voices.cpp"
    HEAP32[(($formant_factor)>>2)]=256; //@line 400 "voices.cpp"
    var $3=HEAP32[((_voice)>>2)]; //@line 402 "voices.cpp"
    var $speed_percent=(($3+64)&4294967295); //@line 402 "voices.cpp"
    HEAP32[(($speed_percent)>>2)]=100; //@line 402 "voices.cpp"
    var $4=HEAP32[((_voice)>>2)]; //@line 403 "voices.cpp"
    var $echo_delay=(($4+76)&4294967295); //@line 403 "voices.cpp"
    HEAP32[(($echo_delay)>>2)]=0; //@line 403 "voices.cpp"
    var $5=HEAP32[((_voice)>>2)]; //@line 404 "voices.cpp"
    var $echo_amp=(($5+80)&4294967295); //@line 404 "voices.cpp"
    HEAP32[(($echo_amp)>>2)]=0; //@line 404 "voices.cpp"
    var $6=HEAP32[((_voice)>>2)]; //@line 405 "voices.cpp"
    var $flutter=(($6+68)&4294967295); //@line 405 "voices.cpp"
    HEAP32[(($flutter)>>2)]=64; //@line 405 "voices.cpp"
    var $7=HEAP32[((_voice)>>2)]; //@line 406 "voices.cpp"
    var $n_harmonic_peaks=(($7+84)&4294967295); //@line 406 "voices.cpp"
    HEAP32[(($n_harmonic_peaks)>>2)]=5; //@line 406 "voices.cpp"
    var $8=HEAP32[((_voice)>>2)]; //@line 407 "voices.cpp"
    var $peak_shape=(($8+88)&4294967295); //@line 407 "voices.cpp"
    HEAP32[(($peak_shape)>>2)]=0; //@line 407 "voices.cpp"
    var $9=HEAP32[((_voice)>>2)]; //@line 408 "voices.cpp"
    var $voicing=(($9+92)&4294967295); //@line 408 "voices.cpp"
    HEAP32[(($voicing)>>2)]=64; //@line 408 "voices.cpp"
    var $10=HEAP32[((_voice)>>2)]; //@line 409 "voices.cpp"
    var $consonant_amp=(($10+100)&4294967295); //@line 409 "voices.cpp"
    HEAP32[(($consonant_amp)>>2)]=100; //@line 409 "voices.cpp"
    var $11=HEAP32[((_voice)>>2)]; //@line 410 "voices.cpp"
    var $consonant_ampv=(($11+104)&4294967295); //@line 410 "voices.cpp"
    HEAP32[(($consonant_ampv)>>2)]=100; //@line 410 "voices.cpp"
    var $12=HEAP32[((_voice)>>2)]; //@line 411 "voices.cpp"
    var $samplerate=(($12+108)&4294967295); //@line 411 "voices.cpp"
    HEAP32[(($samplerate)>>2)]=22050; //@line 411 "voices.cpp"
    var $13=HEAP32[((_voice)>>2)]; //@line 412 "voices.cpp"
    var $klattv=(($13+112)&4294967295); //@line 412 "voices.cpp"
    var $arraydecay=(($klattv)&4294967295); //@line 412 "voices.cpp"
    var $14=$arraydecay; //@line 412 "voices.cpp"
    var $dest$, $stop$, $stop4$, $fast$, $value4$;
    $dest$ = $14;
    $stop$ = $dest$ + 32;
    $value4$ = 0;
    if ($value4$ < 0) $value4$ += 256;
    $value4$ = $value4$ + ($value4$<<8) + ($value4$<<16) + ($value4$*16777216);
    while ($dest$%4 !== 0 && $dest$ < $stop$) {
      ;; HEAP8[$dest$++] = 0;
    }
    $dest$ >>= 2;
    $stop4$ = $stop$ >> 2;
    while ($dest$ < $stop4$) {
    ;;;;;;;  HEAP32[$dest$++] = $value4$;
    }
    $dest$ <<= 2;
    while ($dest$ < $stop$) {
      ;; HEAP8[$dest$++] = 0;
    }; //@line 412 "voices.cpp"
    HEAP32[((((_speed+32)&4294967295))>>2)]=450; //@line 414 "voices.cpp"
    HEAP32[((((_speed+32+4)&4294967295))>>2)]=800; //@line 415 "voices.cpp"
    HEAP32[((((_speed+32+8)&4294967295))>>2)]=175; //@line 416 "voices.cpp"
    var $15=HEAP32[((_voice)>>2)]; //@line 421 "voices.cpp"
    var $roughness=(($15+72)&4294967295); //@line 421 "voices.cpp"
    HEAP32[(($roughness)>>2)]=2; //@line 421 "voices.cpp"
    __Z10InitBreathv(); //@line 424 "voices.cpp"
    $pk=0; //@line 425 "voices.cpp"
    ; //@line 425 "voices.cpp"
    while(1) { 
  
      var $16=$pk; //@line 427 "voices.cpp"
      var $17=HEAP32[((_voice)>>2)]; //@line 427 "voices.cpp"
      var $freq=(($17+144)&4294967295); //@line 427 "voices.cpp"
      var $arrayidx=(($freq+$16*2)&4294967295); //@line 427 "voices.cpp"
      HEAP16[(($arrayidx)>>1)]=256; //@line 427 "voices.cpp"
      var $18=$pk; //@line 428 "voices.cpp"
      var $arrayidx1=((__ZZ10VoiceResetiE15default_heights+$18)&4294967295); //@line 428 "voices.cpp"
      var $19=HEAPU8[($arrayidx1)]; //@line 428 "voices.cpp"
      var $conv=((($19))&255); //@line 428 "voices.cpp"
      var $mul=((($conv)*2)&4294967295); //@line 428 "voices.cpp"
      var $conv2=((($mul)) & 65535); //@line 428 "voices.cpp"
      var $20=$pk; //@line 428 "voices.cpp"
      var $21=HEAP32[((_voice)>>2)]; //@line 428 "voices.cpp"
      var $height=(($21+162)&4294967295); //@line 428 "voices.cpp"
      var $arrayidx3=(($height+$20*2)&4294967295); //@line 428 "voices.cpp"
      HEAP16[(($arrayidx3)>>1)]=$conv2; //@line 428 "voices.cpp"
      var $22=$pk; //@line 429 "voices.cpp"
      var $arrayidx4=((__ZZ10VoiceResetiE14default_widths+$22)&4294967295); //@line 429 "voices.cpp"
      var $23=HEAPU8[($arrayidx4)]; //@line 429 "voices.cpp"
      var $conv5=((($23))&255); //@line 429 "voices.cpp"
      var $mul6=((($conv5)*2)&4294967295); //@line 429 "voices.cpp"
      var $conv7=((($mul6)) & 65535); //@line 429 "voices.cpp"
      var $24=$pk; //@line 429 "voices.cpp"
      var $25=HEAP32[((_voice)>>2)]; //@line 429 "voices.cpp"
      var $width=(($25+180)&4294967295); //@line 429 "voices.cpp"
      var $arrayidx8=(($width+$24*2)&4294967295); //@line 429 "voices.cpp"
      HEAP16[(($arrayidx8)>>1)]=$conv7; //@line 429 "voices.cpp"
      var $26=$pk; //@line 430 "voices.cpp"
      var $27=HEAP32[((_voice)>>2)]; //@line 430 "voices.cpp"
      var $breath=(($27+272)&4294967295); //@line 430 "voices.cpp"
      var $arrayidx9=(($breath+$26*4)&4294967295); //@line 430 "voices.cpp"
      HEAP32[(($arrayidx9)>>2)]=0; //@line 430 "voices.cpp"
      var $28=$pk; //@line 431 "voices.cpp"
      var $arrayidx10=((__ZZ10VoiceResetiE13breath_widths+$28*4)&4294967295); //@line 431 "voices.cpp"
      var $29=HEAP32[(($arrayidx10)>>2)]; //@line 431 "voices.cpp"
      var $30=$pk; //@line 431 "voices.cpp"
      var $31=HEAP32[((_voice)>>2)]; //@line 431 "voices.cpp"
      var $breathw=(($31+308)&4294967295); //@line 431 "voices.cpp"
      var $arrayidx11=(($breathw+$30*4)&4294967295); //@line 431 "voices.cpp"
      HEAP32[(($arrayidx11)>>2)]=$29; //@line 431 "voices.cpp"
      var $32=$pk; //@line 432 "voices.cpp"
      var $33=HEAP32[((_voice)>>2)]; //@line 432 "voices.cpp"
      var $freqadd=(($33+198)&4294967295); //@line 432 "voices.cpp"
      var $arrayidx12=(($freqadd+$32*2)&4294967295); //@line 432 "voices.cpp"
      HEAP16[(($arrayidx12)>>1)]=0; //@line 432 "voices.cpp"
      var $34=$pk; //@line 435 "voices.cpp"
      var $arrayidx13=((__ZL18formant_rate_22050+$34*4)&4294967295); //@line 435 "voices.cpp"
      var $35=HEAP32[(($arrayidx13)>>2)]; //@line 435 "voices.cpp"
      var $mul14=((($35)*22050)&4294967295); //@line 435 "voices.cpp"
      var $36=HEAP32[((_samplerate)>>2)]; //@line 435 "voices.cpp"
      var $div=((((($mul14))|0)/((($36))|0))|0); //@line 435 "voices.cpp"
      var $37=$pk; //@line 435 "voices.cpp"
      var $arrayidx15=((_formant_rate+$37*4)&4294967295); //@line 435 "voices.cpp"
      HEAP32[(($arrayidx15)>>2)]=$div; //@line 435 "voices.cpp"
      var $38=$pk; //@line 425 "voices.cpp"
      var $inc=((($38)+1)&4294967295); //@line 425 "voices.cpp"
      $pk=$inc; //@line 425 "voices.cpp"
      var $cmp=((($inc))|0) < 9; //@line 425 "voices.cpp"
      if ($cmp) { __label__ = 1;continue ; } else { __label__ = 2;break ; } //@line 425 "voices.cpp"
    }
  
    var $39=HEAP32[((_voice)>>2)]; //@line 437 "voices.cpp"
    var $height16=(($39+162)&4294967295); //@line 437 "voices.cpp"
    var $arrayidx17=(($height16+4)&4294967295); //@line 437 "voices.cpp"
    HEAP16[(($arrayidx17)>>1)]=240; //@line 437 "voices.cpp"
    var $40=HEAP32[((_voice)>>2)]; //@line 443 "voices.cpp"
    __ZL13SetToneAdjustP7voice_tPi($40, ((_tone_points)&4294967295)); //@line 443 "voices.cpp"
    var $41=HEAP32[((_voice)>>2)]; //@line 446 "voices.cpp"
    var $speedf1=(($41+52)&4294967295); //@line 446 "voices.cpp"
    HEAP32[(($speedf1)>>2)]=256; //@line 446 "voices.cpp"
    var $42=HEAP32[((_voice)>>2)]; //@line 447 "voices.cpp"
    var $speedf2=(($42+56)&4294967295); //@line 447 "voices.cpp"
    HEAP32[(($speedf2)>>2)]=238; //@line 447 "voices.cpp"
    var $43=HEAP32[((_voice)>>2)]; //@line 448 "voices.cpp"
    var $speedf3=(($43+60)&4294967295); //@line 448 "voices.cpp"
    HEAP32[(($speedf3)>>2)]=232; //@line 448 "voices.cpp"
    var $44=$tone_only_addr; //@line 450 "voices.cpp"
    var $cmp18=((($44))|0)==0; //@line 450 "voices.cpp"
    if ($cmp18) { __label__ = 3;; } else { __label__ = 4;; } //@line 450 "voices.cpp"
    if (__label__ == 3) {
  
      HEAP32[((_n_replace_phonemes)>>2)]=0; //@line 452 "voices.cpp"
      HEAP32[((_option_quiet)>>2)]=0; //@line 453 "voices.cpp"
      var $call=__Z15LoadMbrolaTablePKcS0_i(0, 0, 0); //@line 454 "voices.cpp"
      ; //@line 455 "voices.cpp"
    }
  
    ;
    return; //@line 456 "voices.cpp"
    return;
  }
  

  function __ZL13SetToneAdjustP7voice_tPi($voice, $tone_pts) {
    ;
    var __label__;
  
    var $voice_addr;
    var $tone_pts_addr;
    var $ix;
    var $pt;
    var $y;
    var $freq1;
    var $freq2;
    var $height1;
    var $height2;
    var $rate;
    $voice_addr=$voice;
    $tone_pts_addr=$tone_pts;
    $freq1=0; //@line 217 "voices.cpp"
    var $0=$tone_pts_addr; //@line 219 "voices.cpp"
    var $arrayidx=(($0+4)&4294967295); //@line 219 "voices.cpp"
    var $1=HEAP32[(($arrayidx)>>2)]; //@line 219 "voices.cpp"
    $height1=$1; //@line 219 "voices.cpp"
    $pt=0; //@line 223 "voices.cpp"
    ; //@line 223 "voices.cpp"
    $for_body$2: while(1) { 
  
      var $2=$pt; //@line 225 "voices.cpp"
      var $3=$tone_pts_addr; //@line 225 "voices.cpp"
      var $arrayidx1=(($3+4*$2)&4294967295); //@line 225 "voices.cpp"
      var $4=HEAP32[(($arrayidx1)>>2)]; //@line 225 "voices.cpp"
      var $cmp2=((($4))|0)==-1; //@line 225 "voices.cpp"
      if ($cmp2) { __label__ = 2;; } else { __label__ = 4;; } //@line 225 "voices.cpp"
      $if_then$$if_end8$4: do { 
        if (__label__ == 2) {
  
          var $5=$pt; //@line 227 "voices.cpp"
          var $6=$tone_pts_addr; //@line 227 "voices.cpp"
          var $arrayidx3=(($6+4*$5)&4294967295); //@line 227 "voices.cpp"
          HEAP32[(($arrayidx3)>>2)]=8000; //@line 227 "voices.cpp"
          var $7=$pt; //@line 228 "voices.cpp"
          var $cmp4=((($7))|0) > 0; //@line 228 "voices.cpp"
          if (!($cmp4)) { __label__ = 4;break $if_then$$if_end8$4; } //@line 228 "voices.cpp"
  
          var $8=$pt; //@line 229 "voices.cpp"
          var $sub=((($8)-1)&4294967295); //@line 229 "voices.cpp"
          var $9=$tone_pts_addr; //@line 229 "voices.cpp"
          var $arrayidx6=(($9+4*$sub)&4294967295); //@line 229 "voices.cpp"
          var $10=HEAP32[(($arrayidx6)>>2)]; //@line 229 "voices.cpp"
          var $11=$pt; //@line 229 "voices.cpp"
          var $add=((($11)+1)&4294967295); //@line 229 "voices.cpp"
          var $12=$tone_pts_addr; //@line 229 "voices.cpp"
          var $arrayidx7=(($12+4*$add)&4294967295); //@line 229 "voices.cpp"
          HEAP32[(($arrayidx7)>>2)]=$10; //@line 229 "voices.cpp"
          ; //@line 229 "voices.cpp"
        }
      } while(0);
  
      var $13=$pt; //@line 231 "voices.cpp"
      var $14=$tone_pts_addr; //@line 231 "voices.cpp"
      var $arrayidx9=(($14+4*$13)&4294967295); //@line 231 "voices.cpp"
      var $15=HEAP32[(($arrayidx9)>>2)]; //@line 231 "voices.cpp"
      var $div=((((($15))|0)/8)|0); //@line 231 "voices.cpp"
      $freq2=$div; //@line 231 "voices.cpp"
      var $16=$pt; //@line 232 "voices.cpp"
      var $add10=((($16)+1)&4294967295); //@line 232 "voices.cpp"
      var $17=$tone_pts_addr; //@line 232 "voices.cpp"
      var $arrayidx11=(($17+4*$add10)&4294967295); //@line 232 "voices.cpp"
      var $18=HEAP32[(($arrayidx11)>>2)]; //@line 232 "voices.cpp"
      $height2=$18; //@line 232 "voices.cpp"
      var $19=$freq2; //@line 233 "voices.cpp"
      var $20=$freq1; //@line 233 "voices.cpp"
      var $sub12=((($19)-($20))&4294967295); //@line 233 "voices.cpp"
      var $cmp13=((($sub12))|0) > 0; //@line 233 "voices.cpp"
      if ($cmp13) { __label__ = 5;; } else { __label__ = 9;; } //@line 233 "voices.cpp"
      $if_then14$$if_end31$8: do { 
        if (__label__ == 5) {
  
          var $21=$height2; //@line 235 "voices.cpp"
          var $22=$height1; //@line 235 "voices.cpp"
          var $sub15=((($21)-($22))&4294967295); //@line 235 "voices.cpp"
          var $conv=((($sub15))|0); //@line 235 "voices.cpp"
          var $23=$freq2; //@line 235 "voices.cpp"
          var $24=$freq1; //@line 235 "voices.cpp"
          var $sub16=((($23)-($24))&4294967295); //@line 235 "voices.cpp"
          var $conv17=((($sub16))|0); //@line 235 "voices.cpp"
          var $div18=($conv)/($conv17); //@line 235 "voices.cpp"
          $rate=$div18; //@line 235 "voices.cpp"
          var $25=$freq1; //@line 237 "voices.cpp"
          $ix=$25; //@line 237 "voices.cpp"
          var $26=$ix; //@line 237 "voices.cpp"
          var $27=$freq2; //@line 237 "voices.cpp"
          var $cmp201=((($26))|0) < ((($27))|0); //@line 237 "voices.cpp"
          if (!($cmp201)) { __label__ = 9;break $if_then14$$if_end31$8; } //@line 237 "voices.cpp"
          while(1) { 
  
            var $28=$height1; //@line 239 "voices.cpp"
            var $29=$rate; //@line 239 "voices.cpp"
            var $30=$ix; //@line 239 "voices.cpp"
            var $31=$freq1; //@line 239 "voices.cpp"
            var $sub22=((($30)-($31))&4294967295); //@line 239 "voices.cpp"
            var $conv23=((($sub22))|0); //@line 239 "voices.cpp"
            var $mul=($29)*($conv23); //@line 239 "voices.cpp"
            var $conv24=((($mul))|0); //@line 239 "voices.cpp"
            var $add25=((($conv24)+($28))&4294967295); //@line 239 "voices.cpp"
            $y=$add25; //@line 239 "voices.cpp"
            var $cmp26=((($add25))|0) > 255; //@line 240 "voices.cpp"
            if ($cmp26) { __label__ = 7;; } else { __label__ = 8;; } //@line 240 "voices.cpp"
            if (__label__ == 7) {
  
              $y=255; //@line 241 "voices.cpp"
              ; //@line 241 "voices.cpp"
            }
  
            var $32=$y; //@line 242 "voices.cpp"
            var $conv29=((($32)) & 255); //@line 242 "voices.cpp"
            var $33=$ix; //@line 242 "voices.cpp"
            var $34=$voice_addr; //@line 242 "voices.cpp"
            var $tone_adjust=(($34+344)&4294967295); //@line 242 "voices.cpp"
            var $arrayidx30=(($tone_adjust+$33)&4294967295); //@line 242 "voices.cpp"
            HEAP8[($arrayidx30)]=$conv29; //@line 242 "voices.cpp"
            var $35=$ix; //@line 237 "voices.cpp"
            var $inc=((($35)+1)&4294967295); //@line 237 "voices.cpp"
            $ix=$inc; //@line 237 "voices.cpp"
            var $36=$ix; //@line 237 "voices.cpp"
            var $37=$freq2; //@line 237 "voices.cpp"
            var $cmp20=((($36))|0) < ((($37))|0); //@line 237 "voices.cpp"
            if ($cmp20) { __label__ = 6;continue ; } else { __label__ = 9;break $if_then14$$if_end31$8; } //@line 237 "voices.cpp"
          }
        }
      } while(0);
  
      var $38=$freq2; //@line 245 "voices.cpp"
      $freq1=$38; //@line 245 "voices.cpp"
      var $39=$height2; //@line 246 "voices.cpp"
      $height1=$39; //@line 246 "voices.cpp"
      var $40=$pt; //@line 223 "voices.cpp"
      var $add33=((($40)+2)&4294967295); //@line 223 "voices.cpp"
      $pt=$add33; //@line 223 "voices.cpp"
      var $cmp=((($add33))|0) < 12; //@line 223 "voices.cpp"
      if ($cmp) { __label__ = 1;continue $for_body$2; } else { __label__ = 10;break $for_body$2; } //@line 223 "voices.cpp"
    }
  
    ;
    return; //@line 248 "voices.cpp"
    return;
  }
  

  function __Z9LoadVoicePKci($vname, $control) {
    var __stackBase__  = STACKTOP; STACKTOP += 1260; _memset(__stackBase__, 0, 1260);
    var __label__;
    var __lastLabel__ = null;
  
    var $retval;
    var $vname_addr;
    var $control_addr;
    var $f_voice;
    var $p;
    var $key;
    var $ix;
    var $n;
    var $value=__stackBase__;
    var $value2=__stackBase__+4;
    var $error;
    var $langix;
    var $tone_only;
    var $language_set;
    var $phonemes_set;
    var $stress_amps_set;
    var $stress_lengths_set;
    var $stress_add_set;
    var $conditional_rules;
    var $langopts;
    var $new_translator;
    var $voicename=__stackBase__+8;
    var $language_name=__stackBase__+48;
    var $translator_name=__stackBase__+88;
    var $new_dictionary=__stackBase__+128;
    var $phonemes_name=__stackBase__+168;
    var $option_name=__stackBase__+208;
    var $language_type;
    var $buf=__stackBase__+248;
    var $path_voices=__stackBase__+448;
    var $langname=__stackBase__+612;
    var $stress_amps=__stackBase__+616;
    var $stress_lengths=__stackBase__+648;
    var $stress_add=__stackBase__+680;
    var $names=__stackBase__+712;
    var $pitch1=__stackBase__+1032;
    var $pitch2=__stackBase__+1036;
    var $len;
    var $priority=__stackBase__+1040;
    var $age=__stackBase__+1044;
    var $vgender=__stackBase__+1048;
    var $factor;
    var $tone_data=__stackBase__+1128;
    var $srate=__stackBase__+1176;
    var $name=__stackBase__+1180;
    var $phtrans=__stackBase__+1220;
    $vname_addr=$vname;
    $control_addr=$control;
    $f_voice=0; //@line 528 "voices.cpp"
    $error=0; //@line 535 "voices.cpp"
    $langix=0; //@line 536 "voices.cpp"
    var $0=$control_addr; //@line 537 "voices.cpp"
    var $and=($0) & 2; //@line 537 "voices.cpp"
    $tone_only=$and; //@line 537 "voices.cpp"
    $language_set=0; //@line 538 "voices.cpp"
    $phonemes_set=0; //@line 539 "voices.cpp"
    $stress_amps_set=0; //@line 540 "voices.cpp"
    $stress_lengths_set=0; //@line 541 "voices.cpp"
    $stress_add_set=0; //@line 542 "voices.cpp"
    $conditional_rules=0; //@line 543 "voices.cpp"
    $langopts=0; //@line 544 "voices.cpp"
    $new_translator=0; //@line 546 "voices.cpp"
    var $arraydecay=(($voicename)&4294967295); //@line 571 "voices.cpp"
    var $1=$vname_addr; //@line 571 "voices.cpp"
    var $call=_strcpy($arraydecay, $1); //@line 571 "voices.cpp"
    var $arrayidx=(($voicename)&4294967295); //@line 572 "voices.cpp"
    var $2=HEAP8[($arrayidx)]; //@line 572 "voices.cpp"
    var $conv=(tempInt=(($2)),(tempInt>=128?tempInt-256:tempInt)); //@line 572 "voices.cpp"
    var $cmp=((($conv))|0)==0; //@line 572 "voices.cpp"
    if ($cmp) { __label__ = 1;; } else { __label__ = 2;; } //@line 572 "voices.cpp"
    if (__label__ == 1) {
  
      var $arraydecay1=(($voicename)&4294967295); //@line 573 "voices.cpp"
      var $src$, $dest$, $stop$, $stop4$, $fast$;
      $src$ = ((__str45697)&4294967295);
      $dest$ = $arraydecay1;
      $stop$ = $src$ + 8;
      if (($dest$%4) == ($src$%4) && 8 > 8) {
        while ($src$%4 !== 0 && $src$ < $stop$) {
          ; HEAP8[$dest$++] = HEAP8[$src$++];
        }
        $src$ >>= 2;
        $dest$ >>= 2;
        $stop4$ = $stop$ >> 2;
        while ($src$ < $stop4$) {
      ;;;    HEAP32[$dest$++] = HEAP32[$src$++];
        }
        $src$ <<= 2;
        $dest$ <<= 2;
      }while ($src$ < $stop$) {
        ; HEAP8[$dest$++] = HEAP8[$src$++];
      }; //@line 573 "voices.cpp"
      ; //@line 573 "voices.cpp"
    }
  
    var $3=$control_addr; //@line 575 "voices.cpp"
    var $and3=($3) & 16; //@line 575 "voices.cpp"
    var $tobool=((($and3))|0)!=0; //@line 575 "voices.cpp"
    if ($tobool) { __label__ = 3;; } else { __label__ = 5;; } //@line 575 "voices.cpp"
    $if_then4$$if_else$5: do { 
      if (__label__ == 3) {
  
        var $arraydecay5=(($buf)&4294967295); //@line 577 "voices.cpp"
        var $4=$vname_addr; //@line 577 "voices.cpp"
        var $call6=_strcpy($arraydecay5, $4); //@line 577 "voices.cpp"
        var $arraydecay7=(($buf)&4294967295); //@line 578 "voices.cpp"
        var $call8=__Z13GetFileLengthPKc($arraydecay7); //@line 578 "voices.cpp"
        var $cmp9=((($call8))|0) <= 0; //@line 578 "voices.cpp"
        if (!($cmp9)) { __label__ = 9;break $if_then4$$if_else$5; } //@line 578 "voices.cpp"
  
        $retval=0; //@line 579 "voices.cpp"
        __label__ = 137;break $if_then4$$if_else$5; //@line 579 "voices.cpp"
      }
      else if (__label__ == 5) {
  
        var $arraydecay12=(($path_voices)&4294967295); //@line 583 "voices.cpp"
        var $call13=_sprintf($arraydecay12, ((__str46698)&4294967295), allocate([((_path_home)&4294967295),0,0,0,47,0,0,0,47,0,0,0], ["i8*",0,0,0,"i32",0,0,0,"i32",0,0,0], ALLOC_STACK)); //@line 583 "voices.cpp"
        var $arraydecay14=(($buf)&4294967295); //@line 584 "voices.cpp"
        var $arraydecay15=(($path_voices)&4294967295); //@line 584 "voices.cpp"
        var $arraydecay16=(($voicename)&4294967295); //@line 584 "voices.cpp"
        var $call17=_sprintf($arraydecay14, ((__str47699)&4294967295), allocate([$arraydecay15,0,0,0,$arraydecay16,0,0,0], ["i8*",0,0,0,"i8*",0,0,0], ALLOC_STACK)); //@line 584 "voices.cpp"
        var $arraydecay18=(($buf)&4294967295); //@line 586 "voices.cpp"
        var $call19=__Z13GetFileLengthPKc($arraydecay18); //@line 586 "voices.cpp"
        var $cmp20=((($call19))|0) <= 0; //@line 586 "voices.cpp"
        if (!($cmp20)) { __label__ = 9;break $if_then4$$if_else$5; } //@line 586 "voices.cpp"
  
        var $arrayidx22=(($voicename)&4294967295); //@line 589 "voices.cpp"
        var $5=HEAP8[($arrayidx22)]; //@line 589 "voices.cpp"
        var $arrayidx23=(($langname)&4294967295); //@line 589 "voices.cpp"
        HEAP8[($arrayidx23)]=$5; //@line 589 "voices.cpp"
        var $arrayidx24=(($voicename+1)&4294967295); //@line 590 "voices.cpp"
        var $6=HEAP8[($arrayidx24)]; //@line 590 "voices.cpp"
        var $arrayidx25=(($langname+1)&4294967295); //@line 590 "voices.cpp"
        HEAP8[($arrayidx25)]=$6; //@line 590 "voices.cpp"
        var $arrayidx26=(($langname+2)&4294967295); //@line 591 "voices.cpp"
        HEAP8[($arrayidx26)]=0; //@line 591 "voices.cpp"
        var $arraydecay27=(($buf)&4294967295); //@line 592 "voices.cpp"
        var $arraydecay28=(($path_voices)&4294967295); //@line 592 "voices.cpp"
        var $arraydecay29=(($langname)&4294967295); //@line 592 "voices.cpp"
        var $arraydecay30=(($voicename)&4294967295); //@line 592 "voices.cpp"
        var $call31=_sprintf($arraydecay27, ((__str48700)&4294967295), allocate([$arraydecay28,0,0,0,$arraydecay29,0,0,0,47,0,0,0,$arraydecay30,0,0,0], ["i8*",0,0,0,"i8*",0,0,0,"i32",0,0,0,"i8*",0,0,0], ALLOC_STACK)); //@line 592 "voices.cpp"
        var $arraydecay32=(($buf)&4294967295); //@line 594 "voices.cpp"
        var $call33=__Z13GetFileLengthPKc($arraydecay32); //@line 594 "voices.cpp"
        var $cmp34=((($call33))|0) <= 0; //@line 594 "voices.cpp"
        if (!($cmp34)) { __label__ = 9;break $if_then4$$if_else$5; } //@line 594 "voices.cpp"
  
        var $arraydecay36=(($buf)&4294967295); //@line 597 "voices.cpp"
        var $arraydecay37=(($path_voices)&4294967295); //@line 597 "voices.cpp"
        var $arraydecay38=(($voicename)&4294967295); //@line 597 "voices.cpp"
        var $call39=_sprintf($arraydecay36, ((__str49701)&4294967295), allocate([$arraydecay37,0,0,0,47,0,0,0,$arraydecay38,0,0,0], ["i8*",0,0,0,"i32",0,0,0,"i8*",0,0,0], ALLOC_STACK)); //@line 597 "voices.cpp"
        var $arraydecay40=(($buf)&4294967295); //@line 599 "voices.cpp"
        var $call41=__Z13GetFileLengthPKc($arraydecay40); //@line 599 "voices.cpp"
        var $cmp42=((($call41))|0) <= 0; //@line 599 "voices.cpp"
        if (!($cmp42)) { __label__ = 9;break $if_then4$$if_else$5; } //@line 599 "voices.cpp"
  
        var $arraydecay44=(($buf)&4294967295); //@line 602 "voices.cpp"
        var $arraydecay45=(($path_voices)&4294967295); //@line 602 "voices.cpp"
        var $arraydecay46=(($voicename)&4294967295); //@line 602 "voices.cpp"
        var $call47=_sprintf($arraydecay44, ((__str50702)&4294967295), allocate([$arraydecay45,0,0,0,47,0,0,0,$arraydecay46,0,0,0], ["i8*",0,0,0,"i32",0,0,0,"i8*",0,0,0], ALLOC_STACK)); //@line 602 "voices.cpp"
        __label__ = 9;break $if_then4$$if_else$5; //@line 603 "voices.cpp"
      }
    } while(0);
    $return$$if_end51$12: do { 
      if (__label__ == 9) {
  
        var $arraydecay52=(($buf)&4294967295); //@line 608 "voices.cpp"
        var $call53=_fopen($arraydecay52, ((__str51703)&4294967295)); //@line 608 "voices.cpp"
        $f_voice=$call53; //@line 608 "voices.cpp"
        $language_type=((__str52704)&4294967295); //@line 610 "voices.cpp"
        var $cmp54=((($call53))|0)==0; //@line 611 "voices.cpp"
        if ($cmp54) { __label__ = 10;; } else { __label__ = 14;; } //@line 611 "voices.cpp"
        $if_then55$$if_end66$14: do { 
          if (__label__ == 10) {
  
            var $7=$control_addr; //@line 613 "voices.cpp"
            var $and56=($7) & 3; //@line 613 "voices.cpp"
            var $tobool57=((($and56))|0)!=0; //@line 613 "voices.cpp"
            if ($tobool57) { __label__ = 11;; } else { __label__ = 12;; } //@line 613 "voices.cpp"
            if (__label__ == 11) {
  
              $retval=0; //@line 614 "voices.cpp"
              __label__ = 137;break $return$$if_end51$12; //@line 614 "voices.cpp"
            }
            else if (__label__ == 12) {
  
              var $arraydecay60=(($voicename)&4294967295); //@line 616 "voices.cpp"
              var $call61=__Z22SelectPhonemeTableNamePKc($arraydecay60); //@line 616 "voices.cpp"
              var $cmp62=((($call61))|0) >= 0; //@line 616 "voices.cpp"
              if (!($cmp62)) { __label__ = 14;break $if_then55$$if_end66$14; } //@line 616 "voices.cpp"
  
              var $arraydecay64=(($voicename)&4294967295); //@line 617 "voices.cpp"
              $language_type=$arraydecay64; //@line 617 "voices.cpp"
              ; //@line 617 "voices.cpp"
            }
          }
        } while(0);
  
        var $8=$tone_only; //@line 620 "voices.cpp"
        var $tobool67=((($8))|0)==0; //@line 620 "voices.cpp"
        var $9=HEAP32[((_translator)>>2)]; //@line 620 "voices.cpp"
        var $cmp68=((($9))|0)!=0; //@line 620 "voices.cpp"
        var $or_cond=($tobool67) & ($cmp68); //@line 620 "voices.cpp"
        if ($or_cond) { __label__ = 15;; } else { __label__ = 16;; } //@line 620 "voices.cpp"
        if (__label__ == 15) {
  
          var $10=HEAP32[((_translator)>>2)]; //@line 622 "voices.cpp"
          __Z16DeleteTranslatorP10Translator($10); //@line 622 "voices.cpp"
          HEAP32[((_translator)>>2)]=0; //@line 623 "voices.cpp"
          ; //@line 624 "voices.cpp"
        }
  
        var $arraydecay71=(($translator_name)&4294967295); //@line 626 "voices.cpp"
        var $11=$language_type; //@line 626 "voices.cpp"
        var $call72=_strcpy($arraydecay71, $11); //@line 626 "voices.cpp"
        var $arraydecay73=(($new_dictionary)&4294967295); //@line 627 "voices.cpp"
        var $12=$language_type; //@line 627 "voices.cpp"
        var $call74=_strcpy($arraydecay73, $12); //@line 627 "voices.cpp"
        var $arraydecay75=(($phonemes_name)&4294967295); //@line 628 "voices.cpp"
        var $13=$language_type; //@line 628 "voices.cpp"
        var $call76=_strcpy($arraydecay75, $13); //@line 628 "voices.cpp"
        var $14=$tone_only; //@line 631 "voices.cpp"
        var $tobool77=((($14))|0)!=0; //@line 631 "voices.cpp"
        if ($tobool77) { __label__ = 18;; } else { __label__ = 17;; } //@line 631 "voices.cpp"
        if (__label__ == 18) {
  
          var $call80=_strchr(((__ZZ9LoadVoicePKciE16voice_identifier)&4294967295), 43); //@line 645 "voices.cpp"
          $p=$call80; //@line 645 "voices.cpp"
          var $cmp81=((($call80))|0)!=0; //@line 645 "voices.cpp"
          if ($cmp81) { __label__ = 19;; } else { __label__ = 20;; } //@line 645 "voices.cpp"
          if (__label__ == 19) {
  
            var $16=$p; //@line 646 "voices.cpp"
            HEAP8[($16)]=0; //@line 646 "voices.cpp"
            ; //@line 646 "voices.cpp"
          }
  
          var $arraydecay84=(($buf)&4294967295); //@line 647 "voices.cpp"
          var $17=$vname_addr; //@line 647 "voices.cpp"
          var $arrayidx85=(($17+3)&4294967295); //@line 647 "voices.cpp"
          var $call86=_sprintf($arraydecay84, ((__str53705)&4294967295), allocate([$arrayidx85,0,0,0], ["i8*",0,0,0], ALLOC_STACK)); //@line 647 "voices.cpp"
          var $arraydecay87=(($buf)&4294967295); //@line 648 "voices.cpp"
          var $call88=_strcat(((__ZZ9LoadVoicePKciE16voice_identifier)&4294967295), $arraydecay87); //@line 648 "voices.cpp"
          var $18=HEAP32[((_translator)>>2)]; //@line 649 "voices.cpp"
          var $langopts89=(($18)&4294967295); //@line 649 "voices.cpp"
          $langopts=$langopts89; //@line 649 "voices.cpp"
          ;
        }
        else if (__label__ == 17) {
  
          HEAP32[((_voice)>>2)]=__ZL9voicedata; //@line 633 "voices.cpp"
          var $15=$vname_addr; //@line 634 "voices.cpp"
          __Z8strncpy0PcPKci(((__ZZ9LoadVoicePKciE16voice_identifier)&4294967295), $15, 40); //@line 634 "voices.cpp"
          HEAP8[(((__ZZ9LoadVoicePKciE10voice_name)&4294967295))]=0; //@line 635 "voices.cpp"
          HEAP8[(((__ZZ9LoadVoicePKciE15voice_languages)&4294967295))]=0; //@line 636 "voices.cpp"
          HEAP32[((((_voice_selected+8)&4294967295))>>2)]=((__ZZ9LoadVoicePKciE16voice_identifier)&4294967295); //@line 638 "voices.cpp"
          HEAP32[((((_voice_selected)&4294967295))>>2)]=((__ZZ9LoadVoicePKciE10voice_name)&4294967295); //@line 639 "voices.cpp"
          HEAP32[((((_voice_selected+4)&4294967295))>>2)]=((__ZZ9LoadVoicePKciE15voice_languages)&4294967295); //@line 640 "voices.cpp"
          ; //@line 641 "voices.cpp"
        }
  
        var $19=$tone_only; //@line 651 "voices.cpp"
        __Z10VoiceReseti($19); //@line 651 "voices.cpp"
        var $20=$tone_only; //@line 653 "voices.cpp"
        var $tobool91=((($20))|0)!=0; //@line 653 "voices.cpp"
        if ($tobool91) { __label__ = 23;; } else { __label__ = 22;; } //@line 653 "voices.cpp"
        if (__label__ == 22) {
  
          var $arraydecay93=(($phonemes_name)&4294967295); //@line 654 "voices.cpp"
          var $call94=__Z22SelectPhonemeTableNamePKc($arraydecay93); //@line 654 "voices.cpp"
          ; //@line 654 "voices.cpp"
        }
  
        var $arraydecay97=(($buf)&4294967295); //@line 657 "voices.cpp"
        var $arraydecay100=(($buf)&4294967295); //@line 660 "voices.cpp"
        var $arrayidx109=(($buf)&4294967295); //@line 663 "voices.cpp"
        var $arraydecay114=(($buf)&4294967295); //@line 665 "voices.cpp"
        var $arraydecay416=(($buf)&4294967295); //@line 954 "voices.cpp"
        var $arrayidx119=(($language_name)&4294967295); //@line 678 "voices.cpp"
        var $arraydecay120=(($language_name)&4294967295); //@line 680 "voices.cpp"
        var $arraydecay122=(($language_name)&4294967295); //@line 681 "voices.cpp"
        var $arraydecay127=(($language_name)&4294967295); //@line 684 "voices.cpp"
        var $arraydecay136=(($language_name)&4294967295); //@line 690 "voices.cpp"
        var $arraydecay142=(($language_name)&4294967295); //@line 697 "voices.cpp"
        var $arraydecay144=(($translator_name)&4294967295); //@line 699 "voices.cpp"
        var $arraydecay146=(($new_dictionary)&4294967295); //@line 700 "voices.cpp"
        var $arraydecay148=(($phonemes_name)&4294967295); //@line 701 "voices.cpp"
        var $arraydecay150=(($phonemes_name)&4294967295); //@line 702 "voices.cpp"
        var $arraydecay155=(($translator_name)&4294967295); //@line 707 "voices.cpp"
        var $arraydecay170=(($vgender)&4294967295); //@line 725 "voices.cpp"
        var $arraydecay172=(($vgender)&4294967295); //@line 726 "voices.cpp"
        var $arraydecay180=(($translator_name)&4294967295); //@line 734 "voices.cpp"
        var $arraydecay185=(($translator_name)&4294967295); //@line 739 "voices.cpp"
        var $arraydecay189=(($new_dictionary)&4294967295); //@line 744 "voices.cpp"
        var $arraydecay192=(($phonemes_name)&4294967295); //@line 748 "voices.cpp"
        var $arraydecay206=(($stress_lengths)&4294967295); //@line 768 "voices.cpp"
        var $arraydecay209=(($stress_amps)&4294967295); //@line 772 "voices.cpp"
        var $arraydecay212=(($stress_add)&4294967295); //@line 776 "voices.cpp"
        var $arrayidx222=(($names)&4294967295); //@line 786 "voices.cpp"
        var $arraydecay223=(($arrayidx222)&4294967295); //@line 786 "voices.cpp"
        var $arrayidx224=(($names+40)&4294967295); //@line 786 "voices.cpp"
        var $arraydecay225=(($arrayidx224)&4294967295); //@line 786 "voices.cpp"
        var $arrayidx226=(($names+80)&4294967295); //@line 786 "voices.cpp"
        var $arraydecay227=(($arrayidx226)&4294967295); //@line 786 "voices.cpp"
        var $arrayidx228=(($names+120)&4294967295); //@line 786 "voices.cpp"
        var $arraydecay229=(($arrayidx228)&4294967295); //@line 786 "voices.cpp"
        var $arrayidx230=(($names+160)&4294967295); //@line 786 "voices.cpp"
        var $arraydecay231=(($arrayidx230)&4294967295); //@line 786 "voices.cpp"
        var $arrayidx232=(($names+200)&4294967295); //@line 786 "voices.cpp"
        var $arraydecay233=(($arrayidx232)&4294967295); //@line 786 "voices.cpp"
        var $arraydecay290=(($phonemes_name)&4294967295); //@line 818 "voices.cpp"
        var $arraydecay308=(($option_name)&4294967295); //@line 846 "voices.cpp"
        var $arraydecay312=(($option_name)&4294967295); //@line 846 "voices.cpp"
        var $arraydecay320=(($buf)&4294967295); //@line 853 "voices.cpp"
        var $arraydecay347=(($tone_data)&4294967295); //@line 889 "voices.cpp"
        var $arraydecay348=(($tone_data)&4294967295); //@line 890 "voices.cpp"
        var $arrayidx385=(($phtrans)&4294967295); //@line 926 "voices.cpp"
        var $arraydecay386=(($name)&4294967295); //@line 927 "voices.cpp"
        var $arraydecay387=(($phtrans)&4294967295); //@line 927 "voices.cpp"
        var $arraydecay389=(($name)&4294967295); //@line 928 "voices.cpp"
        var $arraydecay390=(($phtrans)&4294967295); //@line 928 "voices.cpp"
        ; //@line 657 "voices.cpp"
        $while_cond$34: while(1) { 
  
          var $21=$f_voice; //@line 657 "voices.cpp"
          var $cmp96=((($21))|0)!=0; //@line 657 "voices.cpp"
          if (!($cmp96)) { __label__ = 114;break $while_cond$34; } //@line 657 "voices.cpp"
  
          var $22=$f_voice; //@line 657 "voices.cpp"
          var $call98=__ZL11fgets_stripPciP7__sFILE($arraydecay97, 200, $22); //@line 657 "voices.cpp"
          var $cmp99=((($call98))|0)!=0; //@line 657 "voices.cpp"
          if (!($cmp99)) { __label__ = 112;break $while_cond$34; }
  
          $p=$arraydecay100; //@line 660 "voices.cpp"
          ; //@line 660 "voices.cpp"
          $for_cond$38: while(1) { 
  
            var $23=$p; //@line 660 "voices.cpp"
            var $24=HEAP8[($23)]; //@line 660 "voices.cpp"
            var $conv101=(tempInt=(($24)),(tempInt>=128?tempInt-256:tempInt)); //@line 660 "voices.cpp"
            var $cmp102=((($conv101))|0)!=0; //@line 660 "voices.cpp"
            if (!($cmp102)) { __label__ = 30;break $for_cond$38; } //@line 660 "voices.cpp"
  
            var $25=$p; //@line 660 "voices.cpp"
            var $26=HEAP8[($25)]; //@line 660 "voices.cpp"
            var $conv104=(tempInt=(($26)),(tempInt>=128?tempInt-256:tempInt)); //@line 660 "voices.cpp"
            var $call105=_isspace($conv104); //@line 660 "voices.cpp"
            var $tobool106=((($call105))|0)!=0; //@line 660 "voices.cpp"
            var $lnot=($tobool106) ^ 1; //@line 660 "voices.cpp"
            if (!($lnot)) { __label__ = 30;break $for_cond$38; }
  
            var $27=$p; //@line 660 "voices.cpp"
            var $incdec_ptr=(($27+1)&4294967295); //@line 660 "voices.cpp"
            $p=$incdec_ptr; //@line 660 "voices.cpp"
            __label__ = 27;continue $for_cond$38; //@line 660 "voices.cpp"
          }
  
          var $28=$p; //@line 661 "voices.cpp"
          var $incdec_ptr108=(($28+1)&4294967295); //@line 661 "voices.cpp"
          $p=$incdec_ptr108; //@line 661 "voices.cpp"
          HEAP8[($28)]=0; //@line 661 "voices.cpp"
          var $29=HEAP8[($arrayidx109)]; //@line 663 "voices.cpp"
          var $conv110=(tempInt=(($29)),(tempInt>=128?tempInt-256:tempInt)); //@line 663 "voices.cpp"
          var $cmp111=((($conv110))|0)==0; //@line 663 "voices.cpp"
          if ($cmp111) { __label__ = 24;continue $while_cond$34; } //@line 663 "voices.cpp"
  
          var $call115=__Z10LookupMnemP8MNEM_TABPKc(((__ZL11keyword_tab)&4294967295), $arraydecay114); //@line 665 "voices.cpp"
          $key=$call115; //@line 665 "voices.cpp"
          var $30=$key; //@line 667 "voices.cpp"
          if ($30 == 2) {
            __label__ = 33;;
          }
          else if ($30 == 1) {
            __label__ = 41;;
          }
          else if ($30 == 3) {
            __label__ = 45;;
          }
          else if ($30 == 4) {
            __label__ = 46;;
          }
          else if ($30 == 6) {
            __label__ = 50;;
          }
          else if ($30 == 5) {
            __label__ = 51;;
          }
          else if ($30 == 7) {
            __label__ = 52;;
          }
          else if ($30 == 8) {
            __label__ = 53;;
          }
          else if ($30 == 20) {
            __label__ = 54;;
          }
          else if ($30 == 21) {
            __label__ = 55;;
          }
          else if ($30 == 22) {
            __label__ = 56;;
          }
          else if ($30 == 18) {
            __label__ = 57;;
          }
          else if ($30 == 19) {
            __label__ = 59;;
          }
          else if ($30 == 23) {
            __label__ = 32;;
          }
          else if ($30 == 32) {
            __label__ = 73;;
          }
          else if ($30 == 17) {
            __label__ = 76;;
          }
          else if ($30 == 24) {
            __label__ = 77;;
          }
          else if ($30 == 25) {
            __label__ = 78;;
          }
          else if ($30 == 26) {
            __label__ = 80;;
          }
          else if ($30 == 27) {
            __label__ = 81;;
          }
          else if ($30 == 9) {
            __label__ = 85;;
          }
          else if ($30 == 10) {
            __label__ = 86;;
          }
          else if ($30 == 11) {
            __label__ = 88;;
          }
          else if ($30 == 12) {
            __label__ = 90;;
          }
          else if ($30 == 13) {
            __label__ = 94;;
          }
          else if ($30 == 14) {
            __label__ = 95;;
          }
          else if ($30 == 15) {
            __label__ = 97;;
          }
          else if ($30 == 16) {
            __label__ = 101;;
          }
          else if ($30 == 33) {
            __label__ = 102;;
          }
          else if ($30 == 31) {
            __label__ = 103;;
          }
          else if ($30 == 28) {
            __label__ = 104;;
          }
          else if ($30 == 29) {
            __label__ = 107;;
          }
          else if ($30 == 30) {
            __label__ = 108;;
          }
          else {
          __label__ = 109;;
          }
          
          if (__label__ == 109) {
  
            var $195=$key; //@line 948 "voices.cpp"
            var $and407=($195) & 65280; //@line 948 "voices.cpp"
            var $cmp408=((($and407))|0)==256; //@line 948 "voices.cpp"
            if ($cmp408) { __label__ = 110;; } else { __label__ = 111;; } //@line 948 "voices.cpp"
            if (__label__ == 110) {
  
              var $196=$p; //@line 950 "voices.cpp"
              var $197=$key; //@line 950 "voices.cpp"
              var $and410=($197) & 255; //@line 950 "voices.cpp"
              var $198=$langopts; //@line 950 "voices.cpp"
              var $param411=(($198+24)&4294967295); //@line 950 "voices.cpp"
              var $arrayidx412=(($param411+$and410*4)&4294967295); //@line 950 "voices.cpp"
              var $call413=_sscanf($196, ((__str63715)&4294967295), allocate([$arrayidx412,0,0,0], ["i32*",0,0,0], ALLOC_STACK)); //@line 950 "voices.cpp"
              __label__ = 24;continue $while_cond$34; //@line 951 "voices.cpp"
            }
            else if (__label__ == 111) {
  
              var $199=HEAP32[((__impure_ptr)>>2)]; //@line 954 "voices.cpp"
              var $_stderr415=(($199+12)&4294967295); //@line 954 "voices.cpp"
              var $200=HEAP32[(($_stderr415)>>2)]; //@line 954 "voices.cpp"
              var $call417=_fprintf($200, ((__str68720)&4294967295), allocate([$arraydecay416,0,0,0], ["i8*",0,0,0], ALLOC_STACK)); //@line 954 "voices.cpp"
              __label__ = 24;continue $while_cond$34;
            }
          }
          else if (__label__ == 33) {
  
            var $33=$tone_only; //@line 674 "voices.cpp"
            var $tobool116=((($33))|0)!=0; //@line 674 "voices.cpp"
            if ($tobool116) { __label__ = 24;continue $while_cond$34; } //@line 674 "voices.cpp"
  
            HEAP32[(($priority)>>2)]=5; //@line 677 "voices.cpp"
            HEAP8[($arrayidx119)]=0; //@line 678 "voices.cpp"
            var $34=$p; //@line 680 "voices.cpp"
            var $call121=_sscanf($34, ((__str54706)&4294967295), allocate([$arraydecay120,0,0,0,$priority,0,0,0], ["i8*",0,0,0,"i32*",0,0,0], ALLOC_STACK)); //@line 680 "voices.cpp"
            var $call123=_strcmp($arraydecay122, ((__str55707)&4294967295)); //@line 681 "voices.cpp"
            var $cmp124=((($call123))|0)==0; //@line 681 "voices.cpp"
            if ($cmp124) { __label__ = 24;continue $while_cond$34; } //@line 681 "voices.cpp"
  
            var $call128=_strlen($arraydecay127); //@line 684 "voices.cpp"
            var $add=((($call128)+2)&4294967295); //@line 684 "voices.cpp"
            $len=$add; //@line 684 "voices.cpp"
            var $35=$len; //@line 686 "voices.cpp"
            var $36=$langix; //@line 686 "voices.cpp"
            var $_neg=(((-($36)))&4294967295);
            var $sub129=((($_neg)+99)&4294967295); //@line 686 "voices.cpp"
            var $cmp130=((($35))>>>0) < ((($sub129))>>>0); //@line 686 "voices.cpp"
            if ($cmp130) { __label__ = 36;; } else { __label__ = 37;; } //@line 686 "voices.cpp"
            if (__label__ == 36) {
  
              var $37=HEAP32[(($priority)>>2)]; //@line 688 "voices.cpp"
              var $conv132=((($37)) & 255); //@line 688 "voices.cpp"
              var $38=$langix; //@line 688 "voices.cpp"
              var $arrayidx133=((__ZZ9LoadVoicePKciE15voice_languages+$38)&4294967295); //@line 688 "voices.cpp"
              HEAP8[($arrayidx133)]=$conv132; //@line 688 "voices.cpp"
              var $39=$langix; //@line 690 "voices.cpp"
              var $add134=((($39)+1)&4294967295); //@line 690 "voices.cpp"
              var $arrayidx135=((__ZZ9LoadVoicePKciE15voice_languages+$add134)&4294967295); //@line 690 "voices.cpp"
              var $call137=_strcpy($arrayidx135, $arraydecay136); //@line 690 "voices.cpp"
              var $40=$len; //@line 691 "voices.cpp"
              var $41=$langix; //@line 691 "voices.cpp"
              var $add138=((($41)+($40))&4294967295); //@line 691 "voices.cpp"
              $langix=$add138; //@line 691 "voices.cpp"
              ; //@line 692 "voices.cpp"
            }
  
            var $42=$language_set; //@line 695 "voices.cpp"
            var $cmp140=((($42))|0)==0; //@line 695 "voices.cpp"
            if (!($cmp140)) { __label__ = 24;continue $while_cond$34; } //@line 695 "voices.cpp"
  
            var $call143=_strtok($arraydecay142, ((__str56708)&4294967295)); //@line 697 "voices.cpp"
            $language_type=$call143; //@line 697 "voices.cpp"
            $language_set=1; //@line 698 "voices.cpp"
            var $43=$language_type; //@line 699 "voices.cpp"
            var $call145=_strcpy($arraydecay144, $43); //@line 699 "voices.cpp"
            var $44=$language_type; //@line 700 "voices.cpp"
            var $call147=_strcpy($arraydecay146, $44); //@line 700 "voices.cpp"
            var $45=$language_type; //@line 701 "voices.cpp"
            var $call149=_strcpy($arraydecay148, $45); //@line 701 "voices.cpp"
            var $call151=__Z22SelectPhonemeTableNamePKc($arraydecay150); //@line 702 "voices.cpp"
            var $46=$new_translator; //@line 704 "voices.cpp"
            var $cmp152=((($46))|0)!=0; //@line 704 "voices.cpp"
            if ($cmp152) { __label__ = 39;; } else { __label__ = 40;; } //@line 704 "voices.cpp"
            if (__label__ == 39) {
  
              var $47=$new_translator; //@line 705 "voices.cpp"
              __Z16DeleteTranslatorP10Translator($47); //@line 705 "voices.cpp"
              ; //@line 705 "voices.cpp"
            }
  
            var $call156=__Z16SelectTranslatorPKc($arraydecay155); //@line 707 "voices.cpp"
            $new_translator=$call156; //@line 707 "voices.cpp"
            var $48=$new_translator; //@line 708 "voices.cpp"
            var $langopts157=(($48)&4294967295); //@line 708 "voices.cpp"
            $langopts=$langopts157; //@line 708 "voices.cpp"
            __label__ = 24;continue $while_cond$34; //@line 709 "voices.cpp"
          }
          else if (__label__ == 41) {
  
            var $49=$tone_only; //@line 714 "voices.cpp"
            var $cmp160=((($49))|0)==0; //@line 714 "voices.cpp"
            if (!($cmp160)) { __label__ = 24;continue $while_cond$34; } //@line 714 "voices.cpp"
  
            var $50=$p; //@line 716 "voices.cpp"
            var $51=HEAP8[($50)]; //@line 716 "voices.cpp"
            var $conv1636=(tempInt=(($51)),(tempInt>=128?tempInt-256:tempInt)); //@line 716 "voices.cpp"
            var $call1647=_isspace($conv1636); //@line 716 "voices.cpp"
            var $tobool1658=((($call1647))|0)!=0; //@line 716 "voices.cpp"
            var $52=$p; //@line 716 "voices.cpp"
            if ($tobool1658) { __lastLabel__ = 42; __label__ = 43;; } else { __lastLabel__ = 42; __label__ = 44;; } //@line 716 "voices.cpp"
            $while_body166$$while_end$61: do { 
              if (__label__ == 43) {
                while(1) { 
  
                  var $53=__lastLabel__ == 43 ? $56 : ($52);
                  var $incdec_ptr167=(($53+1)&4294967295); //@line 716 "voices.cpp"
                  $p=$incdec_ptr167; //@line 716 "voices.cpp"
                  var $54=$p; //@line 716 "voices.cpp"
                  var $55=HEAP8[($54)]; //@line 716 "voices.cpp"
                  var $conv163=(tempInt=(($55)),(tempInt>=128?tempInt-256:tempInt)); //@line 716 "voices.cpp"
                  var $call164=_isspace($conv163); //@line 716 "voices.cpp"
                  var $tobool165=((($call164))|0)!=0; //@line 716 "voices.cpp"
                  var $56=$p; //@line 716 "voices.cpp"
                  if ($tobool165) { __lastLabel__ = 43; __label__ = 43;continue ; } else { __lastLabel__ = 43; __label__ = 44;break $while_body166$$while_end$61; } //@line 716 "voices.cpp"
                }
              }
            } while(0);
  
            var $_lcssa=__lastLabel__ == 42 ? $52 : ($56);
            __Z8strncpy0PcPKci(((__ZZ9LoadVoicePKciE10voice_name)&4294967295), $_lcssa, 40); //@line 717 "voices.cpp"
            __label__ = 24;continue $while_cond$34; //@line 718 "voices.cpp"
          }
          else if (__label__ == 45) {
  
            HEAP32[(($age)>>2)]=0; //@line 723 "voices.cpp"
            var $57=$p; //@line 725 "voices.cpp"
            var $call171=_sscanf($57, ((__str54706)&4294967295), allocate([$arraydecay170,0,0,0,$age,0,0,0], ["i8*",0,0,0,"i32*",0,0,0], ALLOC_STACK)); //@line 725 "voices.cpp"
            var $call173=__Z10LookupMnemP8MNEM_TABPKc(((_genders)&4294967295), $arraydecay172); //@line 726 "voices.cpp"
            var $conv174=((($call173)) & 255); //@line 726 "voices.cpp"
            HEAP8[(((_voice_selected+12)&4294967295))]=$conv174; //@line 726 "voices.cpp"
            var $58=HEAP32[(($age)>>2)]; //@line 727 "voices.cpp"
            var $conv175=((($58)) & 255); //@line 727 "voices.cpp"
            HEAP8[(((_voice_selected+13)&4294967295))]=$conv175; //@line 727 "voices.cpp"
            __label__ = 24;continue $while_cond$34; //@line 729 "voices.cpp"
          }
          else if (__label__ == 46) {
  
            var $59=$tone_only; //@line 732 "voices.cpp"
            var $tobool177=((($59))|0)!=0; //@line 732 "voices.cpp"
            if ($tobool177) { __label__ = 24;continue $while_cond$34; } //@line 732 "voices.cpp"
  
            var $60=$p; //@line 734 "voices.cpp"
            var $call181=_sscanf($60, ((__str57709)&4294967295), allocate([$arraydecay180,0,0,0], ["i8*",0,0,0], ALLOC_STACK)); //@line 734 "voices.cpp"
            var $61=$new_translator; //@line 736 "voices.cpp"
            var $cmp182=((($61))|0)!=0; //@line 736 "voices.cpp"
            if ($cmp182) { __label__ = 48;; } else { __label__ = 49;; } //@line 736 "voices.cpp"
            if (__label__ == 48) {
  
              var $62=$new_translator; //@line 737 "voices.cpp"
              __Z16DeleteTranslatorP10Translator($62); //@line 737 "voices.cpp"
              ; //@line 737 "voices.cpp"
            }
  
            var $call186=__Z16SelectTranslatorPKc($arraydecay185); //@line 739 "voices.cpp"
            $new_translator=$call186; //@line 739 "voices.cpp"
            var $63=$new_translator; //@line 740 "voices.cpp"
            var $langopts187=(($63)&4294967295); //@line 740 "voices.cpp"
            $langopts=$langopts187; //@line 740 "voices.cpp"
            __label__ = 24;continue $while_cond$34; //@line 741 "voices.cpp"
          }
          else if (__label__ == 50) {
  
            var $64=$p; //@line 744 "voices.cpp"
            var $call190=_sscanf($64, ((__str57709)&4294967295), allocate([$arraydecay189,0,0,0], ["i8*",0,0,0], ALLOC_STACK)); //@line 744 "voices.cpp"
            __label__ = 24;continue $while_cond$34; //@line 745 "voices.cpp"
          }
          else if (__label__ == 51) {
  
            var $65=$p; //@line 748 "voices.cpp"
            var $call193=_sscanf($65, ((__str57709)&4294967295), allocate([$arraydecay192,0,0,0], ["i8*",0,0,0], ALLOC_STACK)); //@line 748 "voices.cpp"
            __label__ = 24;continue $while_cond$34; //@line 749 "voices.cpp"
          }
          else if (__label__ == 52) {
  
            var $66=$p; //@line 752 "voices.cpp"
            __ZL12VoiceFormantPc($66); //@line 752 "voices.cpp"
            __label__ = 24;continue $while_cond$34; //@line 753 "voices.cpp"
          }
          else if (__label__ == 53) {
  
            var $67=$p; //@line 759 "voices.cpp"
            var $call196=_sscanf($67, ((__str58710)&4294967295), allocate([$pitch1,0,0,0,$pitch2,0,0,0], ["i32*",0,0,0,"i32*",0,0,0], ALLOC_STACK)); //@line 759 "voices.cpp"
            $n=$call196; //@line 759 "voices.cpp"
            var $68=HEAP32[(($pitch1)>>2)]; //@line 760 "voices.cpp"
            var $sub197=((($68)-9)&4294967295); //@line 760 "voices.cpp"
            var $shl=($sub197) << 12; //@line 760 "voices.cpp"
            var $69=HEAP32[((_voice)>>2)]; //@line 760 "voices.cpp"
            var $pitch_base=(($69+44)&4294967295); //@line 760 "voices.cpp"
            HEAP32[(($pitch_base)>>2)]=$shl; //@line 760 "voices.cpp"
            var $70=HEAP32[(($pitch2)>>2)]; //@line 761 "voices.cpp"
            var $71=HEAP32[(($pitch1)>>2)]; //@line 761 "voices.cpp"
            var $sub198=((($70)-($71))&4294967295); //@line 761 "voices.cpp"
            var $mul=((($sub198)*108)&4294967295); //@line 761 "voices.cpp"
            var $72=HEAP32[((_voice)>>2)]; //@line 761 "voices.cpp"
            var $pitch_range=(($72+48)&4294967295); //@line 761 "voices.cpp"
            HEAP32[(($pitch_range)>>2)]=$mul; //@line 761 "voices.cpp"
            var $73=HEAP32[(($pitch1)>>2)]; //@line 762 "voices.cpp"
            var $sub199=((($73)-82)&4294967295); //@line 762 "voices.cpp"
            var $conv200=((($sub199))|0); //@line 762 "voices.cpp"
            var $div=($conv200)/82; //@line 762 "voices.cpp"
            $factor=$div; //@line 762 "voices.cpp"
            var $74=$factor; //@line 763 "voices.cpp"
            var $div201=($74)/4; //@line 763 "voices.cpp"
            var $add202=1+($div201); //@line 763 "voices.cpp"
            var $mul203=($add202)*256; //@line 763 "voices.cpp"
            var $conv204=((($mul203))|0); //@line 763 "voices.cpp"
            var $75=HEAP32[((_voice)>>2)]; //@line 763 "voices.cpp"
            var $formant_factor=(($75+96)&4294967295); //@line 763 "voices.cpp"
            HEAP32[(($formant_factor)>>2)]=$conv204; //@line 763 "voices.cpp"
            __label__ = 24;continue $while_cond$34; //@line 765 "voices.cpp"
          }
          else if (__label__ == 54) {
  
            var $76=$p; //@line 768 "voices.cpp"
            var $call207=__ZL12Read8NumbersPcPi($76, $arraydecay206); //@line 768 "voices.cpp"
            $stress_lengths_set=$call207; //@line 768 "voices.cpp"
            __label__ = 24;continue $while_cond$34; //@line 769 "voices.cpp"
          }
          else if (__label__ == 55) {
  
            var $77=$p; //@line 772 "voices.cpp"
            var $call210=__ZL12Read8NumbersPcPi($77, $arraydecay209); //@line 772 "voices.cpp"
            $stress_amps_set=$call210; //@line 772 "voices.cpp"
            __label__ = 24;continue $while_cond$34; //@line 773 "voices.cpp"
          }
          else if (__label__ == 56) {
  
            var $78=$p; //@line 776 "voices.cpp"
            var $call213=__ZL12Read8NumbersPcPi($78, $arraydecay212); //@line 776 "voices.cpp"
            $stress_add_set=$call213; //@line 776 "voices.cpp"
            __label__ = 24;continue $while_cond$34; //@line 777 "voices.cpp"
          }
          else if (__label__ == 57) {
  
            var $79=$p; //@line 780 "voices.cpp"
            var $call215=_sscanf($79, ((__str58710)&4294967295), allocate([_option_tone_flags,0,0,0,_option_tone2,0,0,0], ["i32*",0,0,0,"i32*",0,0,0], ALLOC_STACK)); //@line 780 "voices.cpp"
            var $80=HEAP32[((_option_tone_flags)>>2)]; //@line 781 "voices.cpp"
            var $and216=($80) & 255; //@line 781 "voices.cpp"
            var $cmp217=((($and216))|0)!=0; //@line 781 "voices.cpp"
            if (!($cmp217)) { __label__ = 24;continue $while_cond$34; } //@line 781 "voices.cpp"
  
            var $81=HEAP32[((_option_tone_flags)>>2)]; //@line 782 "voices.cpp"
            var $and219=($81) & 255; //@line 782 "voices.cpp"
            var $82=$langopts; //@line 782 "voices.cpp"
            var $intonation_group=(($82+228)&4294967295); //@line 782 "voices.cpp"
            HEAP32[(($intonation_group)>>2)]=$and219; //@line 782 "voices.cpp"
            __label__ = 24;continue $while_cond$34; //@line 782 "voices.cpp"
          }
          else if (__label__ == 59) {
  
            var $83=$p; //@line 786 "voices.cpp"
            var $call234=_sscanf($83, ((__str59711)&4294967295), allocate([$arraydecay223,0,0,0,$arraydecay225,0,0,0,$arraydecay227,0,0,0,$arraydecay229,0,0,0,$arraydecay231,0,0,0,$arraydecay233,0,0,0], ["i8*",0,0,0,"i8*",0,0,0,"i8*",0,0,0,"i8*",0,0,0,"i8*",0,0,0,"i8*",0,0,0], ALLOC_STACK)); //@line 786 "voices.cpp"
            $n=$call234; //@line 786 "voices.cpp"
            var $84=$langopts; //@line 787 "voices.cpp"
            var $intonation_group235=(($84+228)&4294967295); //@line 787 "voices.cpp"
            HEAP32[(($intonation_group235)>>2)]=0; //@line 787 "voices.cpp"
            $ix=0; //@line 788 "voices.cpp"
            var $85=$ix; //@line 788 "voices.cpp"
            var $86=$n; //@line 788 "voices.cpp"
            var $cmp2379=((($85))|0) < ((($86))|0); //@line 788 "voices.cpp"
            if (!($cmp2379)) { __label__ = 24;continue $while_cond$34; } //@line 788 "voices.cpp"
            while(1) { 
  
              var $87=$ix; //@line 790 "voices.cpp"
              var $arrayidx239=(($names+$87*40)&4294967295); //@line 790 "voices.cpp"
              var $arraydecay240=(($arrayidx239)&4294967295); //@line 790 "voices.cpp"
              var $call241=_strcmp($arraydecay240, ((__str60712)&4294967295)); //@line 790 "voices.cpp"
              var $cmp242=((($call241))|0)==0; //@line 790 "voices.cpp"
              if ($cmp242) { __label__ = 64;; } else { __label__ = 61;; } //@line 790 "voices.cpp"
              if (__label__ == 61) {
  
                var $88=$ix; //@line 793 "voices.cpp"
                var $arrayidx245=(($names+$88*40)&4294967295); //@line 793 "voices.cpp"
                var $arraydecay246=(($arrayidx245)&4294967295); //@line 793 "voices.cpp"
                var $call247=__ZL10LookupTunePKc($arraydecay246); //@line 793 "voices.cpp"
                HEAP32[(($value)>>2)]=$call247; //@line 793 "voices.cpp"
                var $cmp248=((($call247))|0) < 0; //@line 793 "voices.cpp"
                if ($cmp248) { __label__ = 62;; } else { __label__ = 63;; } //@line 793 "voices.cpp"
                if (__label__ == 62) {
  
                  var $89=HEAP32[((__impure_ptr)>>2)]; //@line 794 "voices.cpp"
                  var $_stderr=(($89+12)&4294967295); //@line 794 "voices.cpp"
                  var $90=HEAP32[(($_stderr)>>2)]; //@line 794 "voices.cpp"
                  var $91=$ix; //@line 794 "voices.cpp"
                  var $arrayidx250=(($names+$91*40)&4294967295); //@line 794 "voices.cpp"
                  var $arraydecay251=(($arrayidx250)&4294967295); //@line 794 "voices.cpp"
                  var $call252=_fprintf($90, ((__str61713)&4294967295), allocate([$arraydecay251,0,0,0], ["i8*",0,0,0], ALLOC_STACK)); //@line 794 "voices.cpp"
                  ; //@line 794 "voices.cpp"
                }
                else if (__label__ == 63) {
  
                  var $92=HEAP32[(($value)>>2)]; //@line 796 "voices.cpp"
                  var $conv254=((($92)) & 255); //@line 796 "voices.cpp"
                  var $93=$ix; //@line 796 "voices.cpp"
                  var $94=$langopts; //@line 796 "voices.cpp"
                  var $tunes=(($94+232)&4294967295); //@line 796 "voices.cpp"
                  var $arrayidx255=(($tunes+$93)&4294967295); //@line 796 "voices.cpp"
                  HEAP8[($arrayidx255)]=$conv254; //@line 796 "voices.cpp"
                  ;
                }
              }
  
              var $95=$ix; //@line 788 "voices.cpp"
              var $inc=((($95)+1)&4294967295); //@line 788 "voices.cpp"
              $ix=$inc; //@line 788 "voices.cpp"
              var $96=$ix; //@line 788 "voices.cpp"
              var $97=$n; //@line 788 "voices.cpp"
              var $cmp237=((($96))|0) < ((($97))|0); //@line 788 "voices.cpp"
              if ($cmp237) { __label__ = 60;continue ; } else { __label__ = 24;continue $while_cond$34; } //@line 788 "voices.cpp"
            }
          }
          else if (__label__ == 32) {
  
            var $31=$p; //@line 801 "voices.cpp"
            var $32=HEAP8[($31)]; //@line 801 "voices.cpp"
            var $conv26117=(tempInt=(($32)),(tempInt>=128?tempInt-256:tempInt)); //@line 801 "voices.cpp"
            var $cmp26218=((($conv26117))|0)!=0; //@line 801 "voices.cpp"
            if (!($cmp26218)) { __label__ = 24;continue $while_cond$34; } //@line 801 "voices.cpp"
            $while_cond264_preheader$90: while(1) { 
  
              var $100=$p; //@line 803 "voices.cpp"
              var $101=HEAP8[($100)]; //@line 803 "voices.cpp"
              var $conv26510=(tempInt=(($101)),(tempInt>=128?tempInt-256:tempInt)); //@line 803 "voices.cpp"
              var $call26611=_isspace($conv26510); //@line 803 "voices.cpp"
              var $tobool26712=((($call26611))|0)!=0; //@line 803 "voices.cpp"
              if ($tobool26712) { __label__ = 67;; } else { __label__ = 68;; } //@line 803 "voices.cpp"
              $while_body268$$while_end270$92: do { 
                if (__label__ == 67) {
                  while(1) { 
  
                    var $102=$p; //@line 803 "voices.cpp"
                    var $incdec_ptr269=(($102+1)&4294967295); //@line 803 "voices.cpp"
                    $p=$incdec_ptr269; //@line 803 "voices.cpp"
                    var $103=$p; //@line 803 "voices.cpp"
                    var $104=HEAP8[($103)]; //@line 803 "voices.cpp"
                    var $conv265=(tempInt=(($104)),(tempInt>=128?tempInt-256:tempInt)); //@line 803 "voices.cpp"
                    var $call266=_isspace($conv265); //@line 803 "voices.cpp"
                    var $tobool267=((($call266))|0)!=0; //@line 803 "voices.cpp"
                    if ($tobool267) { __label__ = 67;continue ; } else { __label__ = 68;break $while_body268$$while_end270$92; } //@line 803 "voices.cpp"
                  }
                }
              } while(0);
  
              $n=-1; //@line 804 "voices.cpp"
              var $105=$p; //@line 805 "voices.cpp"
              var $call271=_atoi($105); //@line 805 "voices.cpp"
              $n=$call271; //@line 805 "voices.cpp"
              var $cmp272=((($call271))|0) > 0; //@line 805 "voices.cpp"
              if ($cmp272) { __label__ = 69;; } else { __label__ = 71;; } //@line 805 "voices.cpp"
              $land_lhs_true273$$while_cond279_preheader$96: do { 
                if (__label__ == 69) {
  
                  var $106=$n; //@line 805 "voices.cpp"
                  var $cmp274=((($106))|0) < 32; //@line 805 "voices.cpp"
                  if (!($cmp274)) { __label__ = 71;break $land_lhs_true273$$while_cond279_preheader$96; } //@line 805 "voices.cpp"
  
                  var $107=$p; //@line 807 "voices.cpp"
                  var $incdec_ptr276=(($107+1)&4294967295); //@line 807 "voices.cpp"
                  $p=$incdec_ptr276; //@line 807 "voices.cpp"
                  var $108=$n; //@line 808 "voices.cpp"
                  var $shl277=1 << ($108); //@line 808 "voices.cpp"
                  var $109=$conditional_rules; //@line 808 "voices.cpp"
                  var $or=($109) | ($shl277); //@line 808 "voices.cpp"
                  $conditional_rules=$or; //@line 808 "voices.cpp"
                  ; //@line 809 "voices.cpp"
                }
              } while(0);
  
              var $110=$p; //@line 810 "voices.cpp"
              var $111=HEAP8[($110)]; //@line 810 "voices.cpp"
              var $conv28013=(tempInt=(($111)),(tempInt>=128?tempInt-256:tempInt)); //@line 810 "voices.cpp"
              var $call28114=_isalnum($conv28013); //@line 810 "voices.cpp"
              var $tobool28215=((($call28114))|0)!=0; //@line 810 "voices.cpp"
              if ($tobool28215) { __label__ = 72;; } else { __label__ = 65;; } //@line 810 "voices.cpp"
              $while_body283$$while_cond260_loopexit$100: do { 
                if (__label__ == 72) {
                  while(1) { 
  
                    var $112=$p; //@line 810 "voices.cpp"
                    var $incdec_ptr284=(($112+1)&4294967295); //@line 810 "voices.cpp"
                    $p=$incdec_ptr284; //@line 810 "voices.cpp"
                    var $113=$p; //@line 810 "voices.cpp"
                    var $114=HEAP8[($113)]; //@line 810 "voices.cpp"
                    var $conv280=(tempInt=(($114)),(tempInt>=128?tempInt-256:tempInt)); //@line 810 "voices.cpp"
                    var $call281=_isalnum($conv280); //@line 810 "voices.cpp"
                    var $tobool282=((($call281))|0)!=0; //@line 810 "voices.cpp"
                    if ($tobool282) { __label__ = 72;continue ; } else { __label__ = 65;break $while_body283$$while_cond260_loopexit$100; } //@line 810 "voices.cpp"
                  }
                }
              } while(0);
  
              var $98=$p; //@line 801 "voices.cpp"
              var $99=HEAP8[($98)]; //@line 801 "voices.cpp"
              var $conv261=(tempInt=(($99)),(tempInt>=128?tempInt-256:tempInt)); //@line 801 "voices.cpp"
              var $cmp262=((($conv261))|0)!=0; //@line 801 "voices.cpp"
              if ($cmp262) { __label__ = 66;continue $while_cond264_preheader$90; } else { __label__ = 24;continue $while_cond$34; } //@line 801 "voices.cpp"
            }
          }
          else if (__label__ == 73) {
  
            var $115=$phonemes_set; //@line 815 "voices.cpp"
            var $cmp288=((($115))|0)==0; //@line 815 "voices.cpp"
            if ($cmp288) { __label__ = 74;; } else { __label__ = 75;; } //@line 815 "voices.cpp"
            if (__label__ == 74) {
  
              var $call291=__Z22SelectPhonemeTableNamePKc($arraydecay290); //@line 818 "voices.cpp"
              $phonemes_set=1; //@line 819 "voices.cpp"
              ; //@line 820 "voices.cpp"
            }
  
            var $116=$key; //@line 821 "voices.cpp"
            var $117=$p; //@line 821 "voices.cpp"
            __ZL18PhonemeReplacementiPc($116, $117); //@line 821 "voices.cpp"
            __label__ = 24;continue $while_cond$34; //@line 822 "voices.cpp"
          }
          else if (__label__ == 76) {
  
            var $118=$p; //@line 825 "voices.cpp"
            var $119=$langopts; //@line 825 "voices.cpp"
            var $word_gap=(($119)&4294967295); //@line 825 "voices.cpp"
            var $120=$langopts; //@line 825 "voices.cpp"
            var $vowel_pause=(($120+4)&4294967295); //@line 825 "voices.cpp"
            var $call294=_sscanf($118, ((__str58710)&4294967295), allocate([$word_gap,0,0,0,$vowel_pause,0,0,0], ["i32*",0,0,0,"i32*",0,0,0], ALLOC_STACK)); //@line 825 "voices.cpp"
            __label__ = 24;continue $while_cond$34; //@line 826 "voices.cpp"
          }
          else if (__label__ == 77) {
  
            var $121=$p; //@line 829 "voices.cpp"
            var $122=$langopts; //@line 829 "voices.cpp"
            var $stress_rule=(($122+8)&4294967295); //@line 829 "voices.cpp"
            var $123=$langopts; //@line 829 "voices.cpp"
            var $stress_flags=(($123+12)&4294967295); //@line 829 "voices.cpp"
            var $124=$langopts; //@line 829 "voices.cpp"
            var $unstressed_wd1=(($124+16)&4294967295); //@line 829 "voices.cpp"
            var $125=$langopts; //@line 829 "voices.cpp"
            var $unstressed_wd2=(($125+20)&4294967295); //@line 829 "voices.cpp"
            var $call296=_sscanf($121, ((__str62714)&4294967295), allocate([$stress_rule,0,0,0,$stress_flags,0,0,0,$unstressed_wd1,0,0,0,$unstressed_wd2,0,0,0], ["i32*",0,0,0,"i32*",0,0,0,"i32*",0,0,0,"i32*",0,0,0], ALLOC_STACK)); //@line 829 "voices.cpp"
            __label__ = 24;continue $while_cond$34; //@line 833 "voices.cpp"
          }
          else if (__label__ == 78) {
  
            var $126=$p; //@line 836 "voices.cpp"
            var $call298=_sscanf($126, ((__str63715)&4294967295), allocate([$value,0,0,0], ["i32*",0,0,0], ALLOC_STACK)); //@line 836 "voices.cpp"
            var $cmp299=((($call298))|0)==1; //@line 836 "voices.cpp"
            var $127=HEAP32[(($value)>>2)]; //@line 836 "voices.cpp"
            var $cmp301=((($127))|0) < 20; //@line 836 "voices.cpp"
            var $or_cond1=($cmp299) & ($cmp301); //@line 836 "voices.cpp"
            if (!($or_cond1)) { __label__ = 24;continue $while_cond$34; } //@line 836 "voices.cpp"
  
            var $128=HEAP32[(($value)>>2)]; //@line 837 "voices.cpp"
            var $arrayidx303=((_charsets+$128*4)&4294967295); //@line 837 "voices.cpp"
            var $129=HEAP32[(($arrayidx303)>>2)]; //@line 837 "voices.cpp"
            var $130=$new_translator; //@line 837 "voices.cpp"
            var $charset_a0=(($130+696)&4294967295); //@line 837 "voices.cpp"
            HEAP32[(($charset_a0)>>2)]=$129; //@line 837 "voices.cpp"
            __label__ = 24;continue $while_cond$34; //@line 837 "voices.cpp"
          }
          else if (__label__ == 80) {
  
            var $131=$p; //@line 841 "voices.cpp"
            var $132=$langopts; //@line 841 "voices.cpp"
            var $numbers=(($132+184)&4294967295); //@line 841 "voices.cpp"
            var $133=$langopts; //@line 841 "voices.cpp"
            var $numbers2=(($133+188)&4294967295); //@line 841 "voices.cpp"
            var $call306=_sscanf($131, ((__str58710)&4294967295), allocate([$numbers,0,0,0,$numbers2,0,0,0], ["i32*",0,0,0,"i32*",0,0,0], ALLOC_STACK)); //@line 841 "voices.cpp"
            __label__ = 24;continue $while_cond$34; //@line 842 "voices.cpp"
          }
          else if (__label__ == 81) {
  
            HEAP32[(($value2)>>2)]=0; //@line 845 "voices.cpp"
            var $134=$p; //@line 846 "voices.cpp"
            var $call309=_sscanf($134, ((__str64716)&4294967295), allocate([$arraydecay308,0,0,0,$value,0,0,0,$value2,0,0,0], ["i8*",0,0,0,"i32*",0,0,0,"i32*",0,0,0], ALLOC_STACK)); //@line 846 "voices.cpp"
            var $cmp310=((($call309))|0) >= 2; //@line 846 "voices.cpp"
            if ($cmp310) { __label__ = 82;; } else { __label__ = 84;; } //@line 846 "voices.cpp"
            $land_lhs_true311$$if_else318$114: do { 
              if (__label__ == 82) {
  
                var $call313=__Z10LookupMnemP8MNEM_TABPKc(((__ZL11options_tab)&4294967295), $arraydecay312); //@line 846 "voices.cpp"
                $ix=$call313; //@line 846 "voices.cpp"
                var $cmp314=((($call313))|0) >= 0; //@line 846 "voices.cpp"
                if (!($cmp314)) { __label__ = 84;break $land_lhs_true311$$if_else318$114; } //@line 846 "voices.cpp"
  
                var $135=HEAP32[(($value)>>2)]; //@line 848 "voices.cpp"
                var $136=$ix; //@line 848 "voices.cpp"
                var $137=$langopts; //@line 848 "voices.cpp"
                var $param=(($137+24)&4294967295); //@line 848 "voices.cpp"
                var $arrayidx316=(($param+$136*4)&4294967295); //@line 848 "voices.cpp"
                HEAP32[(($arrayidx316)>>2)]=$135; //@line 848 "voices.cpp"
                var $138=HEAP32[(($value2)>>2)]; //@line 849 "voices.cpp"
                var $139=$ix; //@line 849 "voices.cpp"
                var $140=$langopts; //@line 849 "voices.cpp"
                var $param2=(($140+100)&4294967295); //@line 849 "voices.cpp"
                var $arrayidx317=(($param2+$139*4)&4294967295); //@line 849 "voices.cpp"
                HEAP32[(($arrayidx317)>>2)]=$138; //@line 849 "voices.cpp"
                __label__ = 24;continue $while_cond$34; //@line 850 "voices.cpp"
              }
            } while(0);
  
            var $141=HEAP32[((__impure_ptr)>>2)]; //@line 853 "voices.cpp"
            var $_stderr319=(($141+12)&4294967295); //@line 853 "voices.cpp"
            var $142=HEAP32[(($_stderr319)>>2)]; //@line 853 "voices.cpp"
            var $143=$p; //@line 853 "voices.cpp"
            var $call321=_fprintf($142, ((__str65717)&4294967295), allocate([$arraydecay320,0,0,0,$143,0,0,0], ["i8*",0,0,0,"i8*",0,0,0], ALLOC_STACK)); //@line 853 "voices.cpp"
            __label__ = 24;continue $while_cond$34;
          }
          else if (__label__ == 85) {
  
            HEAP32[(($value)>>2)]=0; //@line 859 "voices.cpp"
            var $144=HEAP32[((_voice)>>2)]; //@line 860 "voices.cpp"
            var $echo_amp=(($144+80)&4294967295); //@line 860 "voices.cpp"
            HEAP32[(($echo_amp)>>2)]=0; //@line 860 "voices.cpp"
            var $145=$p; //@line 861 "voices.cpp"
            var $146=HEAP32[((_voice)>>2)]; //@line 861 "voices.cpp"
            var $echo_delay=(($146+76)&4294967295); //@line 861 "voices.cpp"
            var $147=HEAP32[((_voice)>>2)]; //@line 861 "voices.cpp"
            var $echo_amp324=(($147+80)&4294967295); //@line 861 "voices.cpp"
            var $call325=_sscanf($145, ((__str58710)&4294967295), allocate([$echo_delay,0,0,0,$echo_amp324,0,0,0], ["i32*",0,0,0,"i32*",0,0,0], ALLOC_STACK)); //@line 861 "voices.cpp"
            __label__ = 24;continue $while_cond$34; //@line 862 "voices.cpp"
          }
          else if (__label__ == 86) {
  
            var $148=$p; //@line 865 "voices.cpp"
            var $call327=_sscanf($148, ((__str63715)&4294967295), allocate([$value,0,0,0], ["i32*",0,0,0], ALLOC_STACK)); //@line 865 "voices.cpp"
            var $cmp328=((($call327))|0)==1; //@line 865 "voices.cpp"
            if (!($cmp328)) { __label__ = 24;continue $while_cond$34; } //@line 865 "voices.cpp"
  
            var $149=HEAP32[(($value)>>2)]; //@line 866 "voices.cpp"
            var $mul330=((($149)*32)&4294967295); //@line 866 "voices.cpp"
            var $150=HEAP32[((_voice)>>2)]; //@line 866 "voices.cpp"
            var $flutter=(($150+68)&4294967295); //@line 866 "voices.cpp"
            HEAP32[(($flutter)>>2)]=$mul330; //@line 866 "voices.cpp"
            __label__ = 24;continue $while_cond$34; //@line 866 "voices.cpp"
          }
          else if (__label__ == 88) {
  
            var $151=$p; //@line 870 "voices.cpp"
            var $call333=_sscanf($151, ((__str63715)&4294967295), allocate([$value,0,0,0], ["i32*",0,0,0], ALLOC_STACK)); //@line 870 "voices.cpp"
            var $cmp334=((($call333))|0)==1; //@line 870 "voices.cpp"
            if (!($cmp334)) { __label__ = 24;continue $while_cond$34; } //@line 870 "voices.cpp"
  
            var $152=HEAP32[(($value)>>2)]; //@line 871 "voices.cpp"
            var $153=HEAP32[((_voice)>>2)]; //@line 871 "voices.cpp"
            var $roughness=(($153+72)&4294967295); //@line 871 "voices.cpp"
            HEAP32[(($roughness)>>2)]=$152; //@line 871 "voices.cpp"
            __label__ = 24;continue $while_cond$34; //@line 871 "voices.cpp"
          }
          else if (__label__ == 90) {
  
            var $154=$p; //@line 875 "voices.cpp"
            var $call338=_sscanf($154, ((__str63715)&4294967295), allocate([$value,0,0,0], ["i32*",0,0,0], ALLOC_STACK)); //@line 875 "voices.cpp"
            var $cmp339=((($call338))|0)==1; //@line 875 "voices.cpp"
            if (!($cmp339)) { __label__ = 24;continue $while_cond$34; } //@line 875 "voices.cpp"
  
            var $155=HEAP32[(($value)>>2)]; //@line 877 "voices.cpp"
            var $cmp341=((($155))|0) > 4; //@line 877 "voices.cpp"
            if ($cmp341) { __label__ = 92;; } else { __label__ = 93;; } //@line 877 "voices.cpp"
            if (__label__ == 92) {
  
              var $156=HEAP32[((_voice)>>2)]; //@line 879 "voices.cpp"
              var $peak_shape=(($156+88)&4294967295); //@line 879 "voices.cpp"
              HEAP32[(($peak_shape)>>2)]=1; //@line 879 "voices.cpp"
              HEAP32[(($value)>>2)]=4; //@line 880 "voices.cpp"
              ; //@line 881 "voices.cpp"
            }
  
            var $157=HEAP32[(($value)>>2)]; //@line 882 "voices.cpp"
            var $add344=((($157)+1)&4294967295); //@line 882 "voices.cpp"
            var $158=HEAP32[((_voice)>>2)]; //@line 882 "voices.cpp"
            var $n_harmonic_peaks=(($158+84)&4294967295); //@line 882 "voices.cpp"
            HEAP32[(($n_harmonic_peaks)>>2)]=$add344; //@line 882 "voices.cpp"
            __label__ = 24;continue $while_cond$34; //@line 883 "voices.cpp"
          }
          else if (__label__ == 94) {
  
            var $159=$p; //@line 889 "voices.cpp"
            __Z14ReadTonePointsPcPi($159, $arraydecay347); //@line 889 "voices.cpp"
            var $160=HEAP32[((_voice)>>2)]; //@line 890 "voices.cpp"
            __ZL13SetToneAdjustP7voice_tPi($160, $arraydecay348); //@line 890 "voices.cpp"
            __label__ = 24;continue $while_cond$34; //@line 892 "voices.cpp"
          }
          else if (__label__ == 95) {
  
            var $161=$p; //@line 895 "voices.cpp"
            var $call350=_sscanf($161, ((__str63715)&4294967295), allocate([$value,0,0,0], ["i32*",0,0,0], ALLOC_STACK)); //@line 895 "voices.cpp"
            var $cmp351=((($call350))|0)==1; //@line 895 "voices.cpp"
            if (!($cmp351)) { __label__ = 24;continue $while_cond$34; } //@line 895 "voices.cpp"
  
            var $162=HEAP32[(($value)>>2)]; //@line 896 "voices.cpp"
            var $mul353=((($162)*64)&4294967295); //@line 896 "voices.cpp"
            var $div354=((((($mul353))|0)/100)|0); //@line 896 "voices.cpp"
            var $163=HEAP32[((_voice)>>2)]; //@line 896 "voices.cpp"
            var $voicing=(($163+92)&4294967295); //@line 896 "voices.cpp"
            HEAP32[(($voicing)>>2)]=$div354; //@line 896 "voices.cpp"
            __label__ = 24;continue $while_cond$34; //@line 896 "voices.cpp"
          }
          else if (__label__ == 97) {
  
            var $164=$p; //@line 900 "voices.cpp"
            var $165=HEAP32[((_voice)>>2)]; //@line 900 "voices.cpp"
            var $breath=(($165+272)&4294967295); //@line 900 "voices.cpp"
            var $arrayidx357=(($breath+4)&4294967295); //@line 900 "voices.cpp"
            var $call358=__ZL12Read8NumbersPcPi($164, $arrayidx357); //@line 900 "voices.cpp"
            var $166=HEAP32[((_voice)>>2)]; //@line 900 "voices.cpp"
            var $breath359=(($166+272)&4294967295); //@line 900 "voices.cpp"
            var $arrayidx360=(($breath359)&4294967295); //@line 900 "voices.cpp"
            HEAP32[(($arrayidx360)>>2)]=$call358; //@line 900 "voices.cpp"
            $ix=1; //@line 901 "voices.cpp"
            ; //@line 901 "voices.cpp"
            while(1) { 
  
              var $167=$ix; //@line 903 "voices.cpp"
              var $rem=((($167))|0)%2; //@line 903 "voices.cpp"
              var $tobool364=((($rem))|0)!=0; //@line 903 "voices.cpp"
              if ($tobool364) { __label__ = 99;; } else { __label__ = 100;; } //@line 903 "voices.cpp"
              if (__label__ == 99) {
  
                var $168=$ix; //@line 904 "voices.cpp"
                var $169=HEAP32[((_voice)>>2)]; //@line 904 "voices.cpp"
                var $breath366=(($169+272)&4294967295); //@line 904 "voices.cpp"
                var $arrayidx367=(($breath366+$168*4)&4294967295); //@line 904 "voices.cpp"
                var $170=HEAP32[(($arrayidx367)>>2)]; //@line 904 "voices.cpp"
                var $sub368=(((-($170)))&4294967295); //@line 904 "voices.cpp"
                var $171=$ix; //@line 904 "voices.cpp"
                var $172=HEAP32[((_voice)>>2)]; //@line 904 "voices.cpp"
                var $breath369=(($172+272)&4294967295); //@line 904 "voices.cpp"
                var $arrayidx370=(($breath369+$171*4)&4294967295); //@line 904 "voices.cpp"
                HEAP32[(($arrayidx370)>>2)]=$sub368; //@line 904 "voices.cpp"
                ; //@line 904 "voices.cpp"
              }
  
              var $173=$ix; //@line 901 "voices.cpp"
              var $inc373=((($173)+1)&4294967295); //@line 901 "voices.cpp"
              $ix=$inc373; //@line 901 "voices.cpp"
              var $cmp362=((($inc373))|0) < 8; //@line 901 "voices.cpp"
              if ($cmp362) { __label__ = 98;continue ; } else { __label__ = 24;continue $while_cond$34; } //@line 901 "voices.cpp"
            }
          }
          else if (__label__ == 101) {
  
            var $174=$p; //@line 909 "voices.cpp"
            var $175=HEAP32[((_voice)>>2)]; //@line 909 "voices.cpp"
            var $breathw=(($175+308)&4294967295); //@line 909 "voices.cpp"
            var $arrayidx376=(($breathw+4)&4294967295); //@line 909 "voices.cpp"
            var $call377=__ZL12Read8NumbersPcPi($174, $arrayidx376); //@line 909 "voices.cpp"
            var $176=HEAP32[((_voice)>>2)]; //@line 909 "voices.cpp"
            var $breathw378=(($176+308)&4294967295); //@line 909 "voices.cpp"
            var $arrayidx379=(($breathw378)&4294967295); //@line 909 "voices.cpp"
            HEAP32[(($arrayidx379)>>2)]=$call377; //@line 909 "voices.cpp"
            __label__ = 24;continue $while_cond$34; //@line 910 "voices.cpp"
          }
          else if (__label__ == 102) {
  
            var $177=$p; //@line 913 "voices.cpp"
            var $178=HEAP32[((_voice)>>2)]; //@line 913 "voices.cpp"
            var $consonant_amp=(($178+100)&4294967295); //@line 913 "voices.cpp"
            var $179=HEAP32[((_voice)>>2)]; //@line 913 "voices.cpp"
            var $consonant_ampv=(($179+104)&4294967295); //@line 913 "voices.cpp"
            var $call381=_sscanf($177, ((__str58710)&4294967295), allocate([$consonant_amp,0,0,0,$consonant_ampv,0,0,0], ["i32*",0,0,0,"i32*",0,0,0], ALLOC_STACK)); //@line 913 "voices.cpp"
            HEAP32[(($value)>>2)]=$call381; //@line 913 "voices.cpp"
            __label__ = 24;continue $while_cond$34; //@line 914 "voices.cpp"
          }
          else if (__label__ == 103) {
  
            var $180=$p; //@line 917 "voices.cpp"
            var $181=HEAP32[((_voice)>>2)]; //@line 917 "voices.cpp"
            var $speed_percent=(($181+64)&4294967295); //@line 917 "voices.cpp"
            var $call383=_sscanf($180, ((__str63715)&4294967295), allocate([$speed_percent,0,0,0], ["i32*",0,0,0], ALLOC_STACK)); //@line 917 "voices.cpp"
            __label__ = 24;continue $while_cond$34; //@line 918 "voices.cpp"
          }
          else if (__label__ == 104) {
  
            HEAP32[(($srate)>>2)]=16000; //@line 922 "voices.cpp"
            HEAP8[($arrayidx385)]=0; //@line 926 "voices.cpp"
            var $182=$p; //@line 927 "voices.cpp"
            var $call388=_sscanf($182, ((__str66718)&4294967295), allocate([$arraydecay386,0,0,0,$arraydecay387,0,0,0,$srate,0,0,0], ["i8*",0,0,0,"i8*",0,0,0,"i32*",0,0,0], ALLOC_STACK)); //@line 927 "voices.cpp"
            var $183=HEAP32[(($srate)>>2)]; //@line 928 "voices.cpp"
            var $call391=__Z15LoadMbrolaTablePKcS0_i($arraydecay389, $arraydecay390, $183); //@line 928 "voices.cpp"
            var $cmp392=((($call391))|0)!=0; //@line 928 "voices.cpp"
            if ($cmp392) { __label__ = 105;; } else { __label__ = 106;; } //@line 928 "voices.cpp"
            if (__label__ == 105) {
  
              var $184=HEAP32[((__impure_ptr)>>2)]; //@line 930 "voices.cpp"
              var $_stderr394=(($184+12)&4294967295); //@line 930 "voices.cpp"
              var $185=HEAP32[(($_stderr394)>>2)]; //@line 930 "voices.cpp"
              var $186=_fwrite(((__str67719)&4294967295), 23, 1, $185); //@line 930 "voices.cpp"
              ; //@line 931 "voices.cpp"
            }
  
            var $187=HEAP32[(($srate)>>2)]; //@line 932 "voices.cpp"
            var $188=HEAP32[((_voice)>>2)]; //@line 932 "voices.cpp"
            var $samplerate=(($188+108)&4294967295); //@line 932 "voices.cpp"
            HEAP32[(($samplerate)>>2)]=$187; //@line 932 "voices.cpp"
            __label__ = 24;continue $while_cond$34; //@line 934 "voices.cpp"
          }
          else if (__label__ == 107) {
  
            var $189=HEAP32[((_voice)>>2)]; //@line 937 "voices.cpp"
            var $klattv=(($189+112)&4294967295); //@line 937 "voices.cpp"
            var $arrayidx398=(($klattv)&4294967295); //@line 937 "voices.cpp"
            HEAP32[(($arrayidx398)>>2)]=1; //@line 937 "voices.cpp"
            var $190=$p; //@line 938 "voices.cpp"
            var $191=HEAP32[((_voice)>>2)]; //@line 938 "voices.cpp"
            var $klattv399=(($191+112)&4294967295); //@line 938 "voices.cpp"
            var $arraydecay400=(($klattv399)&4294967295); //@line 938 "voices.cpp"
            var $call401=__ZL12Read8NumbersPcPi($190, $arraydecay400); //@line 938 "voices.cpp"
            var $192=HEAP32[((_voice)>>2)]; //@line 939 "voices.cpp"
            var $klattv402=(($192+112)&4294967295); //@line 939 "voices.cpp"
            var $arrayidx403=(($klattv402+20)&4294967295); //@line 939 "voices.cpp"
            var $193=HEAP32[(($arrayidx403)>>2)]; //@line 939 "voices.cpp"
            var $sub404=((($193)-40)&4294967295); //@line 939 "voices.cpp"
            HEAP32[(($arrayidx403)>>2)]=$sub404; //@line 939 "voices.cpp"
            __label__ = 24;continue $while_cond$34; //@line 940 "voices.cpp"
          }
          else if (__label__ == 108) {
  
            var $194=$p; //@line 943 "voices.cpp"
            var $call406=__ZL12Read8NumbersPcPi($194, ((_speed+32)&4294967295)); //@line 943 "voices.cpp"
            __Z8SetSpeedi(3); //@line 944 "voices.cpp"
            __label__ = 24;continue $while_cond$34; //@line 945 "voices.cpp"
          }
        }
        $if_end423$$while_end419$146: do { 
          if (__label__ == 112) {
  
            var $_pr=$f_voice; //@line 959 "voices.cpp"
            var $cmp420=((($_pr))|0)!=0; //@line 959 "voices.cpp"
            if (!($cmp420)) { __label__ = 114;break $if_end423$$while_end419$146; } //@line 959 "voices.cpp"
  
            var $201=$f_voice; //@line 960 "voices.cpp"
            var $call422=_fclose($201); //@line 960 "voices.cpp"
            ; //@line 960 "voices.cpp"
          }
        } while(0);
  
        var $202=$new_translator; //@line 962 "voices.cpp"
        var $cmp424=((($202))|0)==0; //@line 962 "voices.cpp"
        if ($cmp424) { __label__ = 115;; } else { __label__ = 117;; } //@line 962 "voices.cpp"
        $land_lhs_true425$$if_end430$150: do { 
          if (__label__ == 115) {
  
            var $203=$tone_only; //@line 962 "voices.cpp"
            var $tobool426=((($203))|0)!=0; //@line 962 "voices.cpp"
            if ($tobool426) { __label__ = 117;break $land_lhs_true425$$if_end430$150; } //@line 962 "voices.cpp"
  
            var $arraydecay428=(($translator_name)&4294967295); //@line 965 "voices.cpp"
            var $call429=__Z16SelectTranslatorPKc($arraydecay428); //@line 965 "voices.cpp"
            $new_translator=$call429; //@line 965 "voices.cpp"
            ; //@line 966 "voices.cpp"
          }
        } while(0);
  
        __Z8SetSpeedi(3); //@line 968 "voices.cpp"
        $ix=0; //@line 970 "voices.cpp"
        ; //@line 970 "voices.cpp"
        $for_body433$154: while(1) { 
  
          var $204=$ix; //@line 972 "voices.cpp"
          var $205=HEAP32[((_voice)>>2)]; //@line 972 "voices.cpp"
          var $freq=(($205+144)&4294967295); //@line 972 "voices.cpp"
          var $arrayidx434=(($freq+$204*2)&4294967295); //@line 972 "voices.cpp"
          var $206=HEAP16[(($arrayidx434)>>1)]; //@line 972 "voices.cpp"
          var $207=$ix; //@line 972 "voices.cpp"
          var $208=HEAP32[((_voice)>>2)]; //@line 972 "voices.cpp"
          var $freq2=(($208+216)&4294967295); //@line 972 "voices.cpp"
          var $arrayidx435=(($freq2+$207*2)&4294967295); //@line 972 "voices.cpp"
          HEAP16[(($arrayidx435)>>1)]=$206; //@line 972 "voices.cpp"
          var $209=$ix; //@line 973 "voices.cpp"
          var $210=HEAP32[((_voice)>>2)]; //@line 973 "voices.cpp"
          var $height=(($210+162)&4294967295); //@line 973 "voices.cpp"
          var $arrayidx436=(($height+$209*2)&4294967295); //@line 973 "voices.cpp"
          var $211=HEAP16[(($arrayidx436)>>1)]; //@line 973 "voices.cpp"
          var $212=$ix; //@line 973 "voices.cpp"
          var $213=HEAP32[((_voice)>>2)]; //@line 973 "voices.cpp"
          var $height2=(($213+234)&4294967295); //@line 973 "voices.cpp"
          var $arrayidx437=(($height2+$212*2)&4294967295); //@line 973 "voices.cpp"
          HEAP16[(($arrayidx437)>>1)]=$211; //@line 973 "voices.cpp"
          var $214=$ix; //@line 974 "voices.cpp"
          var $215=HEAP32[((_voice)>>2)]; //@line 974 "voices.cpp"
          var $width=(($215+180)&4294967295); //@line 974 "voices.cpp"
          var $arrayidx438=(($width+$214*2)&4294967295); //@line 974 "voices.cpp"
          var $216=HEAP16[(($arrayidx438)>>1)]; //@line 974 "voices.cpp"
          var $217=$ix; //@line 974 "voices.cpp"
          var $218=HEAP32[((_voice)>>2)]; //@line 974 "voices.cpp"
          var $width2=(($218+252)&4294967295); //@line 974 "voices.cpp"
          var $arrayidx439=(($width2+$217*2)&4294967295); //@line 974 "voices.cpp"
          HEAP16[(($arrayidx439)>>1)]=$216; //@line 974 "voices.cpp"
          var $219=$ix; //@line 970 "voices.cpp"
          var $inc441=((($219)+1)&4294967295); //@line 970 "voices.cpp"
          $ix=$inc441; //@line 970 "voices.cpp"
          var $cmp432=((($inc441))|0) < 9; //@line 970 "voices.cpp"
          if ($cmp432) { __label__ = 118;continue $for_body433$154; } else { __label__ = 119;break $for_body433$154; } //@line 970 "voices.cpp"
        }
  
        var $220=$tone_only; //@line 977 "voices.cpp"
        var $tobool443=((($220))|0)!=0; //@line 977 "voices.cpp"
        if ($tobool443) { __label__ = 120;; } else { __label__ = 121;; } //@line 977 "voices.cpp"
        if (__label__ == 120) {
  
          var $221=HEAP32[((_translator)>>2)]; //@line 979 "voices.cpp"
          $new_translator=$221; //@line 979 "voices.cpp"
          ; //@line 980 "voices.cpp"
        }
        else if (__label__ == 121) {
  
          var $arraydecay446=(($phonemes_name)&4294967295); //@line 983 "voices.cpp"
          var $call447=__Z22SelectPhonemeTableNamePKc($arraydecay446); //@line 983 "voices.cpp"
          $ix=$call447; //@line 983 "voices.cpp"
          var $cmp448=((($call447))|0) < 0; //@line 983 "voices.cpp"
          if ($cmp448) { __label__ = 122;; } else { __label__ = 123;; } //@line 983 "voices.cpp"
          if (__label__ == 122) {
  
            var $222=HEAP32[((__impure_ptr)>>2)]; //@line 985 "voices.cpp"
            var $_stderr450=(($222+12)&4294967295); //@line 985 "voices.cpp"
            var $223=HEAP32[(($_stderr450)>>2)]; //@line 985 "voices.cpp"
            var $arraydecay451=(($phonemes_name)&4294967295); //@line 985 "voices.cpp"
            var $call452=_fprintf($223, ((__str69721)&4294967295), allocate([$arraydecay451,0,0,0], ["i8*",0,0,0], ALLOC_STACK)); //@line 985 "voices.cpp"
            ; //@line 986 "voices.cpp"
          }
  
          var $224=$ix; //@line 987 "voices.cpp"
          var $225=HEAP32[((_voice)>>2)]; //@line 987 "voices.cpp"
          var $phoneme_tab_ix=(($225+40)&4294967295); //@line 987 "voices.cpp"
          HEAP32[(($phoneme_tab_ix)>>2)]=$224; //@line 987 "voices.cpp"
          var $226=$ix; //@line 988 "voices.cpp"
          var $227=$new_translator; //@line 988 "voices.cpp"
          var $phoneme_tab_ix454=(($227+656)&4294967295); //@line 988 "voices.cpp"
          HEAP32[(($phoneme_tab_ix454)>>2)]=$226; //@line 988 "voices.cpp"
          var $228=$new_translator; //@line 989 "voices.cpp"
          var $arraydecay455=(($new_dictionary)&4294967295); //@line 989 "voices.cpp"
          var $229=$control_addr; //@line 989 "voices.cpp"
          var $and456=($229) & 4; //@line 989 "voices.cpp"
          var $call457=__Z14LoadDictionaryP10TranslatorPKci($228, $arraydecay455, $and456); //@line 989 "voices.cpp"
          $error=$call457; //@line 989 "voices.cpp"
          var $230=HEAP8[(((_dictionary_name)&4294967295))]; //@line 990 "voices.cpp"
          var $conv458=(tempInt=(($230)),(tempInt>=128?tempInt-256:tempInt)); //@line 990 "voices.cpp"
          var $cmp459=((($conv458))|0)==0; //@line 990 "voices.cpp"
          if ($cmp459) { __label__ = 124;; } else { __label__ = 125;; } //@line 990 "voices.cpp"
          if (__label__ == 124) {
  
            $retval=0; //@line 991 "voices.cpp"
            __label__ = 137;break $return$$if_end51$12; //@line 991 "voices.cpp"
          }
          else if (__label__ == 125) {
  
            var $231=$conditional_rules; //@line 993 "voices.cpp"
            var $232=$new_translator; //@line 993 "voices.cpp"
            var $dict_condition=(($232+692)&4294967295); //@line 993 "voices.cpp"
            HEAP32[(($dict_condition)>>2)]=$231; //@line 993 "voices.cpp"
            var $233=$langix; //@line 995 "voices.cpp"
            var $arrayidx462=((__ZZ9LoadVoicePKciE15voice_languages+$233)&4294967295); //@line 995 "voices.cpp"
            HEAP8[($arrayidx462)]=0; //@line 995 "voices.cpp"
            ;
          }
        }
  
        var $234=$new_translator; //@line 998 "voices.cpp"
        var $langopts464=(($234)&4294967295); //@line 998 "voices.cpp"
        $langopts=$langopts464; //@line 998 "voices.cpp"
        var $235=$langopts; //@line 1001 "voices.cpp"
        var $param465=(($235+24)&4294967295); //@line 1001 "voices.cpp"
        var $arrayidx466=(($param465+24)&4294967295); //@line 1001 "voices.cpp"
        var $236=HEAP32[(($arrayidx466)>>2)]; //@line 1001 "voices.cpp"
        HEAP32[(($value)>>2)]=$236; //@line 1001 "voices.cpp"
        var $cmp467=((($236))|0)!=0; //@line 1001 "voices.cpp"
        if ($cmp467) { __label__ = 127;; } else { __label__ = 128;; } //@line 1001 "voices.cpp"
        if (__label__ == 127) {
  
          var $237=$new_translator; //@line 1003 "voices.cpp"
          var $238=HEAP32[(($value)>>2)]; //@line 1003 "voices.cpp"
          __Z13SetLengthModsP10Translatori($237, $238); //@line 1003 "voices.cpp"
          ; //@line 1004 "voices.cpp"
        }
  
        var $239=HEAP32[((_voice)>>2)]; //@line 1006 "voices.cpp"
        var $width470=(($239+180)&4294967295); //@line 1006 "voices.cpp"
        var $arrayidx471=(($width470)&4294967295); //@line 1006 "voices.cpp"
        var $240=HEAP16[(($arrayidx471)>>1)]; //@line 1006 "voices.cpp"
        var $conv472=(tempInt=(($240)),(tempInt>=32768?tempInt-65536:tempInt)); //@line 1006 "voices.cpp"
        var $mul473=((($conv472)*105)&4294967295); //@line 1006 "voices.cpp"
        var $div474=((((($mul473))|0)/100)|0); //@line 1006 "voices.cpp"
        var $conv475=((($div474)) & 65535); //@line 1006 "voices.cpp"
        var $241=HEAP32[((_voice)>>2)]; //@line 1006 "voices.cpp"
        var $width476=(($241+180)&4294967295); //@line 1006 "voices.cpp"
        var $arrayidx477=(($width476)&4294967295); //@line 1006 "voices.cpp"
        HEAP16[(($arrayidx477)>>1)]=$conv475; //@line 1006 "voices.cpp"
        var $242=$tone_only; //@line 1008 "voices.cpp"
        var $tobool478=((($242))|0)!=0; //@line 1008 "voices.cpp"
        if ($tobool478) { __label__ = 130;; } else { __label__ = 129;; } //@line 1008 "voices.cpp"
        if (__label__ == 129) {
  
          var $243=$new_translator; //@line 1010 "voices.cpp"
          HEAP32[((_translator)>>2)]=$243; //@line 1010 "voices.cpp"
          ; //@line 1011 "voices.cpp"
        }
  
        $ix=0; //@line 1014 "voices.cpp"
        var $244=$ix; //@line 1014 "voices.cpp"
        var $245=$stress_lengths_set; //@line 1014 "voices.cpp"
        var $cmp4825=((($244))|0) < ((($245))|0); //@line 1014 "voices.cpp"
        if ($cmp4825) { __label__ = 131;; } else { __label__ = 132;; } //@line 1014 "voices.cpp"
        $for_body483$$for_end490$173: do { 
          if (__label__ == 131) {
            while(1) { 
  
              var $246=$ix; //@line 1016 "voices.cpp"
              var $arrayidx484=(($stress_lengths+$246*4)&4294967295); //@line 1016 "voices.cpp"
              var $247=HEAP32[(($arrayidx484)>>2)]; //@line 1016 "voices.cpp"
              var $conv485=((($247)) & 65535); //@line 1016 "voices.cpp"
              var $248=$ix; //@line 1016 "voices.cpp"
              var $249=HEAP32[((_translator)>>2)]; //@line 1016 "voices.cpp"
              var $stress_lengths486=(($249+676)&4294967295); //@line 1016 "voices.cpp"
              var $arrayidx487=(($stress_lengths486+$248*2)&4294967295); //@line 1016 "voices.cpp"
              HEAP16[(($arrayidx487)>>1)]=$conv485; //@line 1016 "voices.cpp"
              var $250=$ix; //@line 1014 "voices.cpp"
              var $inc489=((($250)+1)&4294967295); //@line 1014 "voices.cpp"
              $ix=$inc489; //@line 1014 "voices.cpp"
              var $251=$ix; //@line 1014 "voices.cpp"
              var $252=$stress_lengths_set; //@line 1014 "voices.cpp"
              var $cmp482=((($251))|0) < ((($252))|0); //@line 1014 "voices.cpp"
              if ($cmp482) { __label__ = 131;continue ; } else { __label__ = 132;break $for_body483$$for_end490$173; } //@line 1014 "voices.cpp"
            }
          }
        } while(0);
  
        $ix=0; //@line 1018 "voices.cpp"
        var $253=$ix; //@line 1018 "voices.cpp"
        var $254=$stress_add_set; //@line 1018 "voices.cpp"
        var $cmp4924=((($253))|0) < ((($254))|0); //@line 1018 "voices.cpp"
        if ($cmp4924) { __label__ = 133;; } else { __label__ = 134;; } //@line 1018 "voices.cpp"
        $for_body493$$for_end502$177: do { 
          if (__label__ == 133) {
            while(1) { 
  
              var $255=$ix; //@line 1020 "voices.cpp"
              var $arrayidx494=(($stress_add+$255*4)&4294967295); //@line 1020 "voices.cpp"
              var $256=HEAP32[(($arrayidx494)>>2)]; //@line 1020 "voices.cpp"
              var $257=$ix; //@line 1020 "voices.cpp"
              var $258=HEAP32[((_translator)>>2)]; //@line 1020 "voices.cpp"
              var $stress_lengths495=(($258+676)&4294967295); //@line 1020 "voices.cpp"
              var $arrayidx496=(($stress_lengths495+$257*2)&4294967295); //@line 1020 "voices.cpp"
              var $259=HEAP16[(($arrayidx496)>>1)]; //@line 1020 "voices.cpp"
              var $conv497=(tempInt=(($259)),(tempInt>=32768?tempInt-65536:tempInt)); //@line 1020 "voices.cpp"
              var $add498=((($conv497)+($256))&4294967295); //@line 1020 "voices.cpp"
              var $conv499=((($add498)) & 65535); //@line 1020 "voices.cpp"
              HEAP16[(($arrayidx496)>>1)]=$conv499; //@line 1020 "voices.cpp"
              var $260=$ix; //@line 1018 "voices.cpp"
              var $inc501=((($260)+1)&4294967295); //@line 1018 "voices.cpp"
              $ix=$inc501; //@line 1018 "voices.cpp"
              var $261=$ix; //@line 1018 "voices.cpp"
              var $262=$stress_add_set; //@line 1018 "voices.cpp"
              var $cmp492=((($261))|0) < ((($262))|0); //@line 1018 "voices.cpp"
              if ($cmp492) { __label__ = 133;continue ; } else { __label__ = 134;break $for_body493$$for_end502$177; } //@line 1018 "voices.cpp"
            }
          }
        } while(0);
  
        $ix=0; //@line 1022 "voices.cpp"
        var $263=$ix; //@line 1022 "voices.cpp"
        var $264=$stress_amps_set; //@line 1022 "voices.cpp"
        var $cmp5043=((($263))|0) < ((($264))|0); //@line 1022 "voices.cpp"
        if ($cmp5043) { __label__ = 135;; } else { __label__ = 136;; } //@line 1022 "voices.cpp"
        $for_body505$$for_end516$181: do { 
          if (__label__ == 135) {
            while(1) { 
  
              var $265=$ix; //@line 1024 "voices.cpp"
              var $arrayidx506=(($stress_amps+$265*4)&4294967295); //@line 1024 "voices.cpp"
              var $266=HEAP32[(($arrayidx506)>>2)]; //@line 1024 "voices.cpp"
              var $conv507=((($266)) & 255); //@line 1024 "voices.cpp"
              var $267=$ix; //@line 1024 "voices.cpp"
              var $268=HEAP32[((_translator)>>2)]; //@line 1024 "voices.cpp"
              var $stress_amps508=(($268+660)&4294967295); //@line 1024 "voices.cpp"
              var $arrayidx509=(($stress_amps508+$267)&4294967295); //@line 1024 "voices.cpp"
              HEAP8[($arrayidx509)]=$conv507; //@line 1024 "voices.cpp"
              var $269=$ix; //@line 1025 "voices.cpp"
              var $arrayidx510=(($stress_amps+$269*4)&4294967295); //@line 1025 "voices.cpp"
              var $270=HEAP32[(($arrayidx510)>>2)]; //@line 1025 "voices.cpp"
              var $sub511=((($270)-1)&4294967295); //@line 1025 "voices.cpp"
              var $conv512=((($sub511)) & 255); //@line 1025 "voices.cpp"
              var $271=$ix; //@line 1025 "voices.cpp"
              var $272=HEAP32[((_translator)>>2)]; //@line 1025 "voices.cpp"
              var $stress_amps_r=(($272+668)&4294967295); //@line 1025 "voices.cpp"
              var $arrayidx513=(($stress_amps_r+$271)&4294967295); //@line 1025 "voices.cpp"
              HEAP8[($arrayidx513)]=$conv512; //@line 1025 "voices.cpp"
              var $273=$ix; //@line 1022 "voices.cpp"
              var $inc515=((($273)+1)&4294967295); //@line 1022 "voices.cpp"
              $ix=$inc515; //@line 1022 "voices.cpp"
              var $274=$ix; //@line 1022 "voices.cpp"
              var $275=$stress_amps_set; //@line 1022 "voices.cpp"
              var $cmp504=((($274))|0) < ((($275))|0); //@line 1022 "voices.cpp"
              if ($cmp504) { __label__ = 135;continue ; } else { __label__ = 136;break $for_body505$$for_end516$181; } //@line 1022 "voices.cpp"
            }
          }
        } while(0);
  
        var $276=HEAP32[((_voice)>>2)]; //@line 1028 "voices.cpp"
        $retval=$276; //@line 1028 "voices.cpp"
        ; //@line 1028 "voices.cpp"
      }
    } while(0);
  
    var $277=$retval; //@line 1029 "voices.cpp"
    STACKTOP = __stackBase__;
    return $277; //@line 1029 "voices.cpp"
    return null;
  }
  

  function __ZL11fgets_stripPciP7__sFILE($buf, $size, $f_in) {
    ;
    var __label__;
  
    var $retval;
    var $buf_addr;
    var $size_addr;
    var $f_in_addr;
    var $len;
    var $p;
    $buf_addr=$buf;
    $size_addr=$size;
    $f_in_addr=$f_in;
    var $0=$buf_addr; //@line 184 "voices.cpp"
    var $1=$size_addr; //@line 184 "voices.cpp"
    var $2=$f_in_addr; //@line 184 "voices.cpp"
    var $call=_fgets($0, $1, $2); //@line 184 "voices.cpp"
    var $cmp=((($call))|0)==0; //@line 184 "voices.cpp"
    if ($cmp) { __label__ = 1;; } else { __label__ = 2;; } //@line 184 "voices.cpp"
    if (__label__ == 1) {
  
      $retval=0; //@line 185 "voices.cpp"
      ; //@line 185 "voices.cpp"
    }
    else if (__label__ == 2) {
  
      var $3=$buf_addr; //@line 187 "voices.cpp"
      var $call1=_strlen($3); //@line 187 "voices.cpp"
      $len=$call1; //@line 187 "voices.cpp"
      ; //@line 188 "voices.cpp"
      while(1) { 
  
        var $4=$len; //@line 188 "voices.cpp"
        var $dec=((($4)-1)&4294967295); //@line 188 "voices.cpp"
        $len=$dec; //@line 188 "voices.cpp"
        var $cmp2=((($dec))|0) > 0; //@line 188 "voices.cpp"
        if (!($cmp2)) { __label__ = 6;break ; } //@line 188 "voices.cpp"
  
        var $5=$len; //@line 188 "voices.cpp"
        var $6=$buf_addr; //@line 188 "voices.cpp"
        var $arrayidx=(($6+$5)&4294967295); //@line 188 "voices.cpp"
        var $7=HEAP8[($arrayidx)]; //@line 188 "voices.cpp"
        var $conv=(tempInt=(($7)),(tempInt>=128?tempInt-256:tempInt)); //@line 188 "voices.cpp"
        var $call3=_isspace($conv); //@line 188 "voices.cpp"
        var $tobool=((($call3))|0)!=0; //@line 188 "voices.cpp"
        if (!($tobool)) { __label__ = 6;break ; }
  
        var $8=$len; //@line 189 "voices.cpp"
        var $9=$buf_addr; //@line 189 "voices.cpp"
        var $arrayidx4=(($9+$8)&4294967295); //@line 189 "voices.cpp"
        HEAP8[($arrayidx4)]=0; //@line 189 "voices.cpp"
        __label__ = 3;continue ; //@line 189 "voices.cpp"
      }
  
      var $10=$buf_addr; //@line 191 "voices.cpp"
      var $call5=_strstr($10, ((__str82734)&4294967295)); //@line 191 "voices.cpp"
      $p=$call5; //@line 191 "voices.cpp"
      var $cmp6=((($call5))|0)!=0; //@line 191 "voices.cpp"
      if ($cmp6) { __label__ = 7;; } else { __label__ = 8;; } //@line 191 "voices.cpp"
      if (__label__ == 7) {
  
        var $11=$p; //@line 192 "voices.cpp"
        HEAP8[($11)]=0; //@line 192 "voices.cpp"
        ; //@line 192 "voices.cpp"
      }
  
      var $12=$buf_addr; //@line 194 "voices.cpp"
      $retval=$12; //@line 194 "voices.cpp"
      ; //@line 194 "voices.cpp"
    }
  
    var $13=$retval; //@line 195 "voices.cpp"
    ;
    return $13; //@line 195 "voices.cpp"
    return null;
  }
  

  function __ZL12VoiceFormantPc($p) {
    var __stackBase__  = STACKTOP; STACKTOP += 20; _memset(__stackBase__, 0, 20);
    var __label__;
  
    var $p_addr;
    var $ix;
    var $formant=__stackBase__;
    var $freq=__stackBase__+4;
    var $height=__stackBase__+8;
    var $width=__stackBase__+12;
    var $freqadd=__stackBase__+16;
    $p_addr=$p;
    HEAP32[(($freq)>>2)]=100; //@line 464 "voices.cpp"
    HEAP32[(($height)>>2)]=100; //@line 465 "voices.cpp"
    HEAP32[(($width)>>2)]=100; //@line 466 "voices.cpp"
    HEAP32[(($freqadd)>>2)]=0; //@line 467 "voices.cpp"
    var $0=$p_addr; //@line 469 "voices.cpp"
    var $call=_sscanf($0, ((__str81733)&4294967295), allocate([$formant,0,0,0,$freq,0,0,0,$height,0,0,0,$width,0,0,0,$freqadd,0,0,0], ["i32*",0,0,0,"i32*",0,0,0,"i32*",0,0,0,"i32*",0,0,0,"i32*",0,0,0], ALLOC_STACK)); //@line 469 "voices.cpp"
    $ix=$call; //@line 469 "voices.cpp"
    var $1=$ix; //@line 470 "voices.cpp"
    var $cmp=((($1))|0) < 2; //@line 470 "voices.cpp"
    if ($cmp) { __label__ = 9;; } else { __label__ = 1;; } //@line 470 "voices.cpp"
    $return$$if_end$2: do { 
      if (__label__ == 1) {
  
        var $2=HEAP32[(($formant)>>2)]; //@line 473 "voices.cpp"
        var $cmp1=((($2))|0) < 0; //@line 473 "voices.cpp"
        var $3=HEAP32[(($formant)>>2)]; //@line 473 "voices.cpp"
        var $cmp2=((($3))|0) > 8; //@line 473 "voices.cpp"
        var $or_cond=($cmp1) | ($cmp2); //@line 473 "voices.cpp"
        if ($or_cond) { __label__ = 9;break $return$$if_end$2; } //@line 473 "voices.cpp"
  
        var $4=HEAP32[(($freq)>>2)]; //@line 476 "voices.cpp"
        var $cmp5=((($4))|0) >= 0; //@line 476 "voices.cpp"
        if ($cmp5) { __label__ = 3;; } else { __label__ = 4;; } //@line 476 "voices.cpp"
        if (__label__ == 3) {
  
          var $5=HEAP32[(($freq)>>2)]; //@line 477 "voices.cpp"
          var $conv=((($5))|0); //@line 477 "voices.cpp"
          var $mul=($conv)*2.56001; //@line 477 "voices.cpp"
          var $conv7=((($mul))|0); //@line 477 "voices.cpp"
          var $conv8=((($conv7)) & 65535); //@line 477 "voices.cpp"
          var $6=HEAP32[(($formant)>>2)]; //@line 477 "voices.cpp"
          var $7=HEAP32[((_voice)>>2)]; //@line 477 "voices.cpp"
          var $freq9=(($7+144)&4294967295); //@line 477 "voices.cpp"
          var $arrayidx=(($freq9+$6*2)&4294967295); //@line 477 "voices.cpp"
          HEAP16[(($arrayidx)>>1)]=$conv8; //@line 477 "voices.cpp"
          ; //@line 477 "voices.cpp"
        }
  
        var $8=HEAP32[(($height)>>2)]; //@line 478 "voices.cpp"
        var $cmp11=((($8))|0) >= 0; //@line 478 "voices.cpp"
        if ($cmp11) { __label__ = 5;; } else { __label__ = 6;; } //@line 478 "voices.cpp"
        if (__label__ == 5) {
  
          var $9=HEAP32[(($height)>>2)]; //@line 479 "voices.cpp"
          var $conv13=((($9))|0); //@line 479 "voices.cpp"
          var $mul14=($conv13)*2.56001; //@line 479 "voices.cpp"
          var $conv15=((($mul14))|0); //@line 479 "voices.cpp"
          var $conv16=((($conv15)) & 65535); //@line 479 "voices.cpp"
          var $10=HEAP32[(($formant)>>2)]; //@line 479 "voices.cpp"
          var $11=HEAP32[((_voice)>>2)]; //@line 479 "voices.cpp"
          var $height17=(($11+162)&4294967295); //@line 479 "voices.cpp"
          var $arrayidx18=(($height17+$10*2)&4294967295); //@line 479 "voices.cpp"
          HEAP16[(($arrayidx18)>>1)]=$conv16; //@line 479 "voices.cpp"
          ; //@line 479 "voices.cpp"
        }
  
        var $12=HEAP32[(($width)>>2)]; //@line 480 "voices.cpp"
        var $cmp20=((($12))|0) >= 0; //@line 480 "voices.cpp"
        if ($cmp20) { __label__ = 7;; } else { __label__ = 8;; } //@line 480 "voices.cpp"
        if (__label__ == 7) {
  
          var $13=HEAP32[(($width)>>2)]; //@line 481 "voices.cpp"
          var $conv22=((($13))|0); //@line 481 "voices.cpp"
          var $mul23=($conv22)*2.56001; //@line 481 "voices.cpp"
          var $conv24=((($mul23))|0); //@line 481 "voices.cpp"
          var $conv25=((($conv24)) & 65535); //@line 481 "voices.cpp"
          var $14=HEAP32[(($formant)>>2)]; //@line 481 "voices.cpp"
          var $15=HEAP32[((_voice)>>2)]; //@line 481 "voices.cpp"
          var $width26=(($15+180)&4294967295); //@line 481 "voices.cpp"
          var $arrayidx27=(($width26+$14*2)&4294967295); //@line 481 "voices.cpp"
          HEAP16[(($arrayidx27)>>1)]=$conv25; //@line 481 "voices.cpp"
          ; //@line 481 "voices.cpp"
        }
  
        var $16=HEAP32[(($freqadd)>>2)]; //@line 482 "voices.cpp"
        var $conv29=((($16)) & 65535); //@line 482 "voices.cpp"
        var $17=HEAP32[(($formant)>>2)]; //@line 482 "voices.cpp"
        var $18=HEAP32[((_voice)>>2)]; //@line 482 "voices.cpp"
        var $freqadd30=(($18+198)&4294967295); //@line 482 "voices.cpp"
        var $arrayidx31=(($freqadd30+$17*2)&4294967295); //@line 482 "voices.cpp"
        HEAP16[(($arrayidx31)>>1)]=$conv29; //@line 482 "voices.cpp"
        ; //@line 483 "voices.cpp"
      }
    } while(0);
  
    STACKTOP = __stackBase__;
    return; //@line 483 "voices.cpp"
    return;
  }
  

  function __ZL12Read8NumbersPcPi($data_in, $data) {
    ;
    var __label__;
  
    var $data_in_addr;
    var $data_addr;
    $data_in_addr=$data_in;
    $data_addr=$data;
    var $0=$data_addr; //@line 515 "voices.cpp"
    var $1=$0; //@line 515 "voices.cpp"
    var $dest$, $stop$, $stop4$, $fast$, $value4$;
    $dest$ = $1;
    $stop$ = $dest$ + 12;
    $value4$ = 0;
    if ($value4$ < 0) $value4$ += 256;
    $value4$ = $value4$ + ($value4$<<8) + ($value4$<<16) + ($value4$*16777216);
    while ($dest$%4 !== 0 && $dest$ < $stop$) {
      ;; HEAP8[$dest$++] = 0;
    }
    $dest$ >>= 2;
    $stop4$ = $stop$ >> 2;
    while ($dest$ < $stop4$) {
    ;;;;;;;  HEAP32[$dest$++] = $value4$;
    }
    $dest$ <<= 2;
    while ($dest$ < $stop$) {
      ;; HEAP8[$dest$++] = 0;
    }; //@line 515 "voices.cpp"
    var $2=$data_in_addr; //@line 516 "voices.cpp"
    var $3=$data_addr; //@line 516 "voices.cpp"
    var $arrayidx=(($3)&4294967295); //@line 516 "voices.cpp"
    var $4=$data_addr; //@line 516 "voices.cpp"
    var $arrayidx1=(($4+4)&4294967295); //@line 516 "voices.cpp"
    var $5=$data_addr; //@line 516 "voices.cpp"
    var $arrayidx2=(($5+8)&4294967295); //@line 516 "voices.cpp"
    var $6=$data_addr; //@line 516 "voices.cpp"
    var $arrayidx3=(($6+12)&4294967295); //@line 516 "voices.cpp"
    var $7=$data_addr; //@line 516 "voices.cpp"
    var $arrayidx4=(($7+16)&4294967295); //@line 516 "voices.cpp"
    var $8=$data_addr; //@line 516 "voices.cpp"
    var $arrayidx5=(($8+20)&4294967295); //@line 516 "voices.cpp"
    var $9=$data_addr; //@line 516 "voices.cpp"
    var $arrayidx6=(($9+24)&4294967295); //@line 516 "voices.cpp"
    var $10=$data_addr; //@line 516 "voices.cpp"
    var $arrayidx7=(($10+28)&4294967295); //@line 516 "voices.cpp"
    var $call=_sscanf($2, ((__str80732)&4294967295), allocate([$arrayidx,0,0,0,$arrayidx1,0,0,0,$arrayidx2,0,0,0,$arrayidx3,0,0,0,$arrayidx4,0,0,0,$arrayidx5,0,0,0,$arrayidx6,0,0,0,$arrayidx7,0,0,0], ["i32*",0,0,0,"i32*",0,0,0,"i32*",0,0,0,"i32*",0,0,0,"i32*",0,0,0,"i32*",0,0,0,"i32*",0,0,0,"i32*",0,0,0], ALLOC_STACK)); //@line 516 "voices.cpp"
    ;
    return $call; //@line 516 "voices.cpp"
    return null;
  }
  

  function __ZL10LookupTunePKc($name) {
    ;
    var __label__;
  
    var $retval;
    var $name_addr;
    var $ix;
    $name_addr=$name;
    $ix=0; //@line 202 "voices.cpp"
    ; //@line 202 "voices.cpp"
    while(1) { 
  
      var $0=$ix; //@line 202 "voices.cpp"
      var $1=HEAP32[((_n_tunes)>>2)]; //@line 202 "voices.cpp"
      var $cmp=((($0))|0) < ((($1))|0); //@line 202 "voices.cpp"
      if (!($cmp)) { __label__ = 5;break ; } //@line 202 "voices.cpp"
  
      var $2=$name_addr; //@line 204 "voices.cpp"
      var $3=$ix; //@line 204 "voices.cpp"
      var $4=HEAP32[((_tunes)>>2)]; //@line 204 "voices.cpp"
      var $arrayidx=(($4+68*$3)&4294967295); //@line 204 "voices.cpp"
      var $name1=(($arrayidx)&4294967295); //@line 204 "voices.cpp"
      var $arraydecay=(($name1)&4294967295); //@line 204 "voices.cpp"
      var $call=_strcmp($2, $arraydecay); //@line 204 "voices.cpp"
      var $cmp2=((($call))|0)==0; //@line 204 "voices.cpp"
      var $5=$ix; //@line 205 "voices.cpp"
      if ($cmp2) { __label__ = 3;break ; } //@line 204 "voices.cpp"
  
      var $inc=((($5)+1)&4294967295); //@line 202 "voices.cpp"
      $ix=$inc; //@line 202 "voices.cpp"
      __label__ = 1;continue ; //@line 202 "voices.cpp"
    }
    if (__label__ == 5) {
  
      $retval=-1; //@line 207 "voices.cpp"
      ; //@line 207 "voices.cpp"
    }
    else if (__label__ == 3) {
  
      $retval=$5; //@line 205 "voices.cpp"
      ; //@line 205 "voices.cpp"
    }
  
    var $6=$retval; //@line 208 "voices.cpp"
    ;
    return $6; //@line 208 "voices.cpp"
    return null;
  }
  

  function __ZL18PhonemeReplacementiPc($type, $p) {
    var __stackBase__  = STACKTOP; STACKTOP += 28; _memset(__stackBase__, 0, 28);
    var __label__;
  
    var $type_addr;
    var $p_addr;
    var $n;
    var $phon;
    var $flags=__stackBase__;
    var $phon_string1=__stackBase__+4;
    var $phon_string2=__stackBase__+16;
    $type_addr=$type;
    $p_addr=$p;
    HEAP32[(($flags)>>2)]=0; //@line 493 "voices.cpp"
    var $arraydecay=(($phon_string2)&4294967295); //@line 497 "voices.cpp"
    var $src$, $dest$, $stop$, $stop4$, $fast$;
    $src$ = ((__str60712)&4294967295);
    $dest$ = $arraydecay;
    $stop$ = $src$ + 5;
    if (($dest$%4) == ($src$%4) && 5 > 8) {
      while ($src$%4 !== 0 && $src$ < $stop$) {
        ; HEAP8[$dest$++] = HEAP8[$src$++];
      }
      $src$ >>= 2;
      $dest$ >>= 2;
      $stop4$ = $stop$ >> 2;
      while ($src$ < $stop4$) {
    ;;;    HEAP32[$dest$++] = HEAP32[$src$++];
      }
      $src$ <<= 2;
      $dest$ <<= 2;
    }while ($src$ < $stop$) {
      ; HEAP8[$dest$++] = HEAP8[$src$++];
    }; //@line 497 "voices.cpp"
    var $0=$p_addr; //@line 498 "voices.cpp"
    var $arraydecay1=(($phon_string1)&4294967295); //@line 498 "voices.cpp"
    var $arraydecay2=(($phon_string2)&4294967295); //@line 498 "voices.cpp"
    var $call3=_sscanf($0, ((__str79731)&4294967295), allocate([$flags,0,0,0,$arraydecay1,0,0,0,$arraydecay2,0,0,0], ["i32*",0,0,0,"i8*",0,0,0,"i8*",0,0,0], ALLOC_STACK)); //@line 498 "voices.cpp"
    $n=$call3; //@line 498 "voices.cpp"
    var $1=$n; //@line 499 "voices.cpp"
    var $cmp=((($1))|0) < 2; //@line 499 "voices.cpp"
    var $2=HEAP32[((_n_replace_phonemes)>>2)]; //@line 499 "voices.cpp"
    var $cmp4=((($2))|0) >= 60; //@line 499 "voices.cpp"
    var $or_cond=($cmp) | ($cmp4); //@line 499 "voices.cpp"
    if ($or_cond) { __label__ = 3;; } else { __label__ = 1;; } //@line 499 "voices.cpp"
    $return$$if_end$2: do { 
      if (__label__ == 1) {
  
        var $arraydecay5=(($phon_string1)&4294967295); //@line 502 "voices.cpp"
        var $call6=__Z19LookupPhonemeStringPKc($arraydecay5); //@line 502 "voices.cpp"
        $phon=$call6; //@line 502 "voices.cpp"
        var $cmp7=((($call6))|0)==0; //@line 502 "voices.cpp"
        if ($cmp7) { __label__ = 3;break $return$$if_end$2; } //@line 502 "voices.cpp"
  
        var $3=$phon; //@line 505 "voices.cpp"
        var $conv=((($3)) & 255); //@line 505 "voices.cpp"
        var $4=HEAP32[((_n_replace_phonemes)>>2)]; //@line 505 "voices.cpp"
        var $arrayidx=((_replace_phonemes+$4*3)&4294967295); //@line 505 "voices.cpp"
        var $old_ph=(($arrayidx)&4294967295); //@line 505 "voices.cpp"
        HEAP8[($old_ph)]=$conv; //@line 505 "voices.cpp"
        var $arraydecay10=(($phon_string2)&4294967295); //@line 506 "voices.cpp"
        var $call11=__Z19LookupPhonemeStringPKc($arraydecay10); //@line 506 "voices.cpp"
        var $conv12=((($call11)) & 255); //@line 506 "voices.cpp"
        var $5=HEAP32[((_n_replace_phonemes)>>2)]; //@line 506 "voices.cpp"
        var $arrayidx13=((_replace_phonemes+$5*3)&4294967295); //@line 506 "voices.cpp"
        var $new_ph=(($arrayidx13+1)&4294967295); //@line 506 "voices.cpp"
        HEAP8[($new_ph)]=$conv12; //@line 506 "voices.cpp"
        var $6=HEAP32[(($flags)>>2)]; //@line 507 "voices.cpp"
        var $conv14=((($6)) & 255); //@line 507 "voices.cpp"
        var $7=HEAP32[((_n_replace_phonemes)>>2)]; //@line 507 "voices.cpp"
        var $inc=((($7)+1)&4294967295); //@line 507 "voices.cpp"
        HEAP32[((_n_replace_phonemes)>>2)]=$inc; //@line 507 "voices.cpp"
        var $arrayidx15=((_replace_phonemes+$7*3)&4294967295); //@line 507 "voices.cpp"
        var $type16=(($arrayidx15+2)&4294967295); //@line 507 "voices.cpp"
        HEAP8[($type16)]=$conv14; //@line 507 "voices.cpp"
        ; //@line 508 "voices.cpp"
      }
    } while(0);
  
    STACKTOP = __stackBase__;
    return; //@line 508 "voices.cpp"
    return;
  }
  

  function __Z16LoadVoiceVariantPKci($vname, $variant_num) {
    var __stackBase__  = STACKTOP; STACKTOP += 60; _memset(__stackBase__, 0, 60);
    var __label__;
  
    var $retval;
    var $vname_addr;
    var $variant_num_addr;
    var $v;
    var $variant_name;
    var $buf=__stackBase__;
    $vname_addr=$vname;
    $variant_num_addr=$variant_num;
    var $arraydecay=(($buf)&4294967295); //@line 1085 "voices.cpp"
    var $0=$vname_addr; //@line 1085 "voices.cpp"
    __Z8strncpy0PcPKci($arraydecay, $0, 60); //@line 1085 "voices.cpp"
    var $arraydecay1=(($buf)&4294967295); //@line 1086 "voices.cpp"
    var $1=$variant_num_addr; //@line 1086 "voices.cpp"
    __ZL23ExtractVoiceVariantNamePci($arraydecay1, $1); //@line 1086 "voices.cpp"
    $variant_name=((__ZZL23ExtractVoiceVariantNamePciE12variant_name)&4294967295); //@line 1086 "voices.cpp"
    var $arraydecay2=(($buf)&4294967295); //@line 1088 "voices.cpp"
    var $call3=__Z9LoadVoicePKci($arraydecay2, 0); //@line 1088 "voices.cpp"
    $v=$call3; //@line 1088 "voices.cpp"
    var $cmp=((($call3))|0)==0; //@line 1088 "voices.cpp"
    if ($cmp) { __label__ = 1;; } else { __label__ = 2;; } //@line 1088 "voices.cpp"
    if (__label__ == 1) {
  
      $retval=0; //@line 1089 "voices.cpp"
      ; //@line 1089 "voices.cpp"
    }
    else if (__label__ == 2) {
  
      var $2=$variant_name; //@line 1091 "voices.cpp"
      var $arrayidx=(($2)&4294967295); //@line 1091 "voices.cpp"
      var $3=HEAP8[($arrayidx)]; //@line 1091 "voices.cpp"
      var $conv=(tempInt=(($3)),(tempInt>=128?tempInt-256:tempInt)); //@line 1091 "voices.cpp"
      var $cmp4=((($conv))|0)!=0; //@line 1091 "voices.cpp"
      if ($cmp4) { __label__ = 3;; } else { __label__ = 4;; } //@line 1091 "voices.cpp"
      if (__label__ == 3) {
  
        var $4=$variant_name; //@line 1093 "voices.cpp"
        var $call6=__Z9LoadVoicePKci($4, 2); //@line 1093 "voices.cpp"
        $v=$call6; //@line 1093 "voices.cpp"
        ; //@line 1094 "voices.cpp"
      }
  
      var $5=$v; //@line 1095 "voices.cpp"
      $retval=$5; //@line 1095 "voices.cpp"
      ; //@line 1095 "voices.cpp"
    }
  
    var $6=$retval; //@line 1096 "voices.cpp"
    STACKTOP = __stackBase__;
    return $6; //@line 1096 "voices.cpp"
    return null;
  }
  

  function __ZL23ExtractVoiceVariantNamePci($vname, $variant_num) {
    var __stackBase__  = STACKTOP; STACKTOP += 8; _memset(__stackBase__, 0, 8);
    var __label__;
    var __lastLabel__ = null;
  
    var $vname_addr;
    var $variant_num_addr;
    var $p;
    var $variant_prefix=__stackBase__;
    $vname_addr=$vname;
    $variant_num_addr=$variant_num;
    HEAP8[(((__ZZL23ExtractVoiceVariantNamePciE12variant_name)&4294967295))]=0; //@line 1041 "voices.cpp"
    var $arraydecay=(($variant_prefix)&4294967295); //@line 1042 "voices.cpp"
    var $call=_sprintf($arraydecay, ((__str76728)&4294967295), allocate([47,0,0,0], ["i32",0,0,0], ALLOC_STACK)); //@line 1042 "voices.cpp"
    var $0=$vname_addr; //@line 1044 "voices.cpp"
    var $cmp=((($0))|0)!=0; //@line 1044 "voices.cpp"
    if ($cmp) { __label__ = 1;; } else { __label__ = 5;; } //@line 1044 "voices.cpp"
    $if_then$$if_end10thread_pre_split$2: do { 
      if (__label__ == 1) {
  
        var $1=$vname_addr; //@line 1046 "voices.cpp"
        var $call1=_strchr($1, 43); //@line 1046 "voices.cpp"
        $p=$call1; //@line 1046 "voices.cpp"
        var $cmp2=((($call1))|0)!=0; //@line 1046 "voices.cpp"
        if (!($cmp2)) { __label__ = 5;break $if_then$$if_end10thread_pre_split$2; } //@line 1046 "voices.cpp"
  
        var $2=$p; //@line 1049 "voices.cpp"
        var $incdec_ptr=(($2+1)&4294967295); //@line 1049 "voices.cpp"
        $p=$incdec_ptr; //@line 1049 "voices.cpp"
        HEAP8[($2)]=0; //@line 1049 "voices.cpp"
        var $3=$p; //@line 1050 "voices.cpp"
        var $4=HEAP8[($3)]; //@line 1050 "voices.cpp"
        var $conv=(tempInt=(($4)),(tempInt>=128?tempInt-256:tempInt)); //@line 1050 "voices.cpp"
        var $isdigittmp=((($conv)-48)&4294967295); //@line 1050 "voices.cpp"
        var $isdigit=((($isdigittmp))>>>0) < 10; //@line 1050 "voices.cpp"
        var $call4=((($isdigit))&1); //@line 1050 "voices.cpp"
        var $tobool=((($call4))|0)!=0; //@line 1050 "voices.cpp"
        if ($tobool) { __label__ = 3;; } else { __label__ = 4;; } //@line 1050 "voices.cpp"
        if (__label__ == 3) {
  
          var $5=$p; //@line 1052 "voices.cpp"
          var $call6=_atoi($5); //@line 1052 "voices.cpp"
          $variant_num_addr=$call6; //@line 1052 "voices.cpp"
          __lastLabel__ = 3; __label__ = 6;break $if_then$$if_end10thread_pre_split$2; //@line 1053 "voices.cpp"
        }
        else if (__label__ == 4) {
  
          var $arraydecay7=(($variant_prefix)&4294967295); //@line 1057 "voices.cpp"
          var $call8=_strcpy(((__ZZL23ExtractVoiceVariantNamePciE12variant_name)&4294967295), $arraydecay7); //@line 1057 "voices.cpp"
          var $6=$p; //@line 1058 "voices.cpp"
          __Z8strncpy0PcPKci(((__ZZL23ExtractVoiceVariantNamePciE12variant_name+3)&4294967295), $6, 17); //@line 1058 "voices.cpp"
          __label__ = 5;break $if_then$$if_end10thread_pre_split$2;
        }
      }
    } while(0);
    if (__label__ == 5) {
  
      var $_pr=$variant_num_addr; //@line 1063 "voices.cpp"
      __lastLabel__ = 5; ;
    }
  
    var $7=__lastLabel__ == 5 ? $_pr : ($call6); //@line 1063 "voices.cpp"
    var $cmp11=((($7))|0) > 0; //@line 1063 "voices.cpp"
    if ($cmp11) { __label__ = 7;; } else { __label__ = 10;; } //@line 1063 "voices.cpp"
    if (__label__ == 7) {
  
      var $8=$variant_num_addr; //@line 1065 "voices.cpp"
      var $cmp13=((($8))|0) < 10; //@line 1065 "voices.cpp"
      var $arraydecay15=(($variant_prefix)&4294967295); //@line 1066 "voices.cpp"
      var $9=$variant_num_addr; //@line 1066 "voices.cpp"
      if ($cmp13) { __label__ = 8;; } else { __label__ = 9;; } //@line 1065 "voices.cpp"
      if (__label__ == 8) {
  
        var $call16=_sprintf(((__ZZL23ExtractVoiceVariantNamePciE12variant_name)&4294967295), ((__str77729)&4294967295), allocate([$arraydecay15,0,0,0,$9,0,0,0], ["i8*",0,0,0,"i32",0,0,0], ALLOC_STACK)); //@line 1066 "voices.cpp"
        ; //@line 1066 "voices.cpp"
      }
      else if (__label__ == 9) {
  
        var $sub=((($9)-10)&4294967295); //@line 1068 "voices.cpp"
        var $call19=_sprintf(((__ZZL23ExtractVoiceVariantNamePciE12variant_name)&4294967295), ((__str78730)&4294967295), allocate([$arraydecay15,0,0,0,$sub,0,0,0], ["i8*",0,0,0,"i32",0,0,0], ALLOC_STACK)); //@line 1068 "voices.cpp"
        ;
      }
    }
  
    STACKTOP = __stackBase__;
    return;
    return;
  }
  

  function __Z17SelectVoiceByNamePP12espeak_VOICEPKc($voices, $name) {
    var __stackBase__  = STACKTOP; STACKTOP += 44; _memset(__stackBase__, 0, 44);
    var __label__;
    var __lastLabel__ = null;
  
    var $retval;
    var $voices_addr;
    var $name_addr;
    var $ix;
    var $match_fname;
    var $match_fname2;
    var $match_name;
    var $id;
    var $last_part_len;
    var $last_part=__stackBase__;
    $voices_addr=$voices;
    $name_addr=$name;
    $match_fname=-1; //@line 1325 "voices.cpp"
    $match_fname2=-1; //@line 1326 "voices.cpp"
    $match_name=-1; //@line 1327 "voices.cpp"
    var $0=$voices_addr; //@line 1332 "voices.cpp"
    var $cmp=((($0))|0)==0; //@line 1332 "voices.cpp"
    if ($cmp) { __label__ = 1;; } else { __label__ = 4;; } //@line 1332 "voices.cpp"
    if (__label__ == 1) {
  
      var $1=HEAP32[((__ZL13n_voices_list)>>2)]; //@line 1334 "voices.cpp"
      var $cmp1=((($1))|0)==0; //@line 1334 "voices.cpp"
      if ($cmp1) { __label__ = 2;; } else { __label__ = 3;; } //@line 1334 "voices.cpp"
      if (__label__ == 2) {
  
        var $call=_espeak_ListVoices(0); //@line 1335 "voices.cpp"
        ; //@line 1335 "voices.cpp"
      }
  
      $voices_addr=((__ZL11voices_list)&4294967295); //@line 1336 "voices.cpp"
      ; //@line 1337 "voices.cpp"
    }
  
    var $arraydecay=(($last_part)&4294967295); //@line 1339 "voices.cpp"
    var $2=$name_addr; //@line 1339 "voices.cpp"
    var $call4=_sprintf($arraydecay, ((__str70722)&4294967295), allocate([47,0,0,0,$2,0,0,0], ["i32",0,0,0,"i8*",0,0,0], ALLOC_STACK)); //@line 1339 "voices.cpp"
    var $arraydecay5=(($last_part)&4294967295); //@line 1340 "voices.cpp"
    var $call6=_strlen($arraydecay5); //@line 1340 "voices.cpp"
    $last_part_len=$call6; //@line 1340 "voices.cpp"
    $ix=0; //@line 1342 "voices.cpp"
    var $arraydecay18=(($last_part)&4294967295); //@line 1355 "voices.cpp"
    ; //@line 1342 "voices.cpp"
    $for_cond$8: while(1) { 
  
      var $3=$ix; //@line 1342 "voices.cpp"
      var $4=$voices_addr; //@line 1342 "voices.cpp"
      var $arrayidx=(($4+4*$3)&4294967295); //@line 1342 "voices.cpp"
      var $5=HEAP32[(($arrayidx)>>2)]; //@line 1342 "voices.cpp"
      var $cmp7=((($5))|0)!=0; //@line 1342 "voices.cpp"
      if (!($cmp7)) { __label__ = 13;break $for_cond$8; } //@line 1342 "voices.cpp"
  
      var $6=$name_addr; //@line 1344 "voices.cpp"
      var $7=$ix; //@line 1344 "voices.cpp"
      var $8=$voices_addr; //@line 1344 "voices.cpp"
      var $arrayidx8=(($8+4*$7)&4294967295); //@line 1344 "voices.cpp"
      var $9=HEAP32[(($arrayidx8)>>2)]; //@line 1344 "voices.cpp"
      var $name9=(($9)&4294967295); //@line 1344 "voices.cpp"
      var $10=HEAP32[(($name9)>>2)]; //@line 1344 "voices.cpp"
      var $call10=_strcmp($6, $10); //@line 1344 "voices.cpp"
      var $cmp11=((($call10))|0)==0; //@line 1344 "voices.cpp"
      if ($cmp11) { __label__ = 7;break $for_cond$8; } //@line 1344 "voices.cpp"
  
      var $12=$name_addr; //@line 1350 "voices.cpp"
      var $13=$ix; //@line 1350 "voices.cpp"
      var $14=$voices_addr; //@line 1350 "voices.cpp"
      var $arrayidx13=(($14+4*$13)&4294967295); //@line 1350 "voices.cpp"
      var $15=HEAP32[(($arrayidx13)>>2)]; //@line 1350 "voices.cpp"
      var $identifier=(($15+8)&4294967295); //@line 1350 "voices.cpp"
      var $16=HEAP32[(($identifier)>>2)]; //@line 1350 "voices.cpp"
      $id=$16; //@line 1350 "voices.cpp"
      var $call14=_strcmp($12, $16); //@line 1350 "voices.cpp"
      var $cmp15=((($call14))|0)==0; //@line 1350 "voices.cpp"
      if ($cmp15) { __label__ = 9;; } else { __label__ = 10;; } //@line 1350 "voices.cpp"
      $if_then16$$if_else17$12: do { 
        if (__label__ == 9) {
  
          var $17=$ix; //@line 1352 "voices.cpp"
          $match_fname=$17; //@line 1352 "voices.cpp"
          ; //@line 1353 "voices.cpp"
        }
        else if (__label__ == 10) {
  
          var $18=$id; //@line 1355 "voices.cpp"
          var $call19=_strlen($18); //@line 1355 "voices.cpp"
          var $19=$last_part_len; //@line 1355 "voices.cpp"
          var $sub=((($call19)-($19))&4294967295); //@line 1355 "voices.cpp"
          var $20=$id; //@line 1355 "voices.cpp"
          var $arrayidx20=(($20+$sub)&4294967295); //@line 1355 "voices.cpp"
          var $call21=_strcmp($arraydecay18, $arrayidx20); //@line 1355 "voices.cpp"
          var $cmp22=((($call21))|0)==0; //@line 1355 "voices.cpp"
          if (!($cmp22)) { __label__ = 12;break $if_then16$$if_else17$12; } //@line 1355 "voices.cpp"
  
          var $21=$ix; //@line 1357 "voices.cpp"
          $match_fname2=$21; //@line 1357 "voices.cpp"
          ; //@line 1358 "voices.cpp"
        }
      } while(0);
  
      var $22=$ix; //@line 1342 "voices.cpp"
      var $inc=((($22)+1)&4294967295); //@line 1342 "voices.cpp"
      $ix=$inc; //@line 1342 "voices.cpp"
      __label__ = 5;continue $for_cond$8; //@line 1342 "voices.cpp"
    }
    if (__label__ == 13) {
  
      var $_pr=$match_name; //@line 1361 "voices.cpp"
      __lastLabel__ = 13; ;
    }
    else if (__label__ == 7) {
  
      var $11=$ix; //@line 1346 "voices.cpp"
      $match_name=$11; //@line 1346 "voices.cpp"
      __lastLabel__ = 7; ; //@line 1347 "voices.cpp"
    }
  
    var $23=__lastLabel__ == 13 ? $_pr : ($11); //@line 1361 "voices.cpp"
    var $cmp27=((($23))|0) < 0; //@line 1361 "voices.cpp"
    if ($cmp27) { __label__ = 15;; } else { __label__ = 17;; } //@line 1361 "voices.cpp"
    $if_then28$$if_end32thread_pre_split$21: do { 
      if (__label__ == 15) {
  
        var $24=$match_fname; //@line 1363 "voices.cpp"
        $match_name=$24; //@line 1363 "voices.cpp"
        var $25=$match_name; //@line 1364 "voices.cpp"
        var $cmp29=((($25))|0) < 0; //@line 1364 "voices.cpp"
        if (!($cmp29)) { __label__ = 20;break $if_then28$$if_end32thread_pre_split$21; } //@line 1364 "voices.cpp"
  
        var $26=$match_fname2; //@line 1365 "voices.cpp"
        $match_name=$26; //@line 1365 "voices.cpp"
        __lastLabel__ = 16; __label__ = 18;break $if_then28$$if_end32thread_pre_split$21; //@line 1365 "voices.cpp"
      }
      else if (__label__ == 17) {
  
        var $_pr1=$match_name; //@line 1368 "voices.cpp"
        __lastLabel__ = 17; __label__ = 18;break $if_then28$$if_end32thread_pre_split$21;
      }
    } while(0);
    $if_end32$$if_end35$25: do { 
      if (__label__ == 18) {
  
        var $27=__lastLabel__ == 17 ? $_pr1 : ($26); //@line 1368 "voices.cpp"
        var $cmp33=((($27))|0) < 0; //@line 1368 "voices.cpp"
        if (!($cmp33)) { __label__ = 20;break $if_end32$$if_end35$25; } //@line 1368 "voices.cpp"
  
        $retval=0; //@line 1369 "voices.cpp"
        __label__ = 21;break $if_end32$$if_end35$25; //@line 1369 "voices.cpp"
      }
    } while(0);
    if (__label__ == 20) {
  
      var $28=$match_name; //@line 1371 "voices.cpp"
      var $29=$voices_addr; //@line 1371 "voices.cpp"
      var $arrayidx36=(($29+4*$28)&4294967295); //@line 1371 "voices.cpp"
      var $30=HEAP32[(($arrayidx36)>>2)]; //@line 1371 "voices.cpp"
      $retval=$30; //@line 1371 "voices.cpp"
      ; //@line 1371 "voices.cpp"
    }
  
    var $31=$retval; //@line 1372 "voices.cpp"
    STACKTOP = __stackBase__;
    return $31; //@line 1372 "voices.cpp"
    return null;
  }
  

  function _espeak_ListVoices($voice_spec) {
    var __stackBase__  = STACKTOP; STACKTOP += 164; _memset(__stackBase__, 0, 164);
    var __label__;
  
    var $voice_spec_addr;
    var $ix;
    var $j;
    var $v;
    var $path_voices=__stackBase__;
    $voice_spec_addr=$voice_spec;
    $ix=0; //@line 1753 "voices.cpp"
    var $0=$ix; //@line 1753 "voices.cpp"
    var $1=HEAP32[((__ZL13n_voices_list)>>2)]; //@line 1753 "voices.cpp"
    var $cmp3=((($0))|0) < ((($1))|0); //@line 1753 "voices.cpp"
    if ($cmp3) { __label__ = 1;; } else { __label__ = 4;; } //@line 1753 "voices.cpp"
    $for_body$$for_end$2: do { 
      if (__label__ == 1) {
        while(1) { 
  
          var $2=$ix; //@line 1755 "voices.cpp"
          var $arrayidx=((__ZL11voices_list+$2*4)&4294967295); //@line 1755 "voices.cpp"
          var $3=HEAP32[(($arrayidx)>>2)]; //@line 1755 "voices.cpp"
          var $cmp1=((($3))|0)!=0; //@line 1755 "voices.cpp"
          if ($cmp1) { __label__ = 2;; } else { __label__ = 3;; } //@line 1755 "voices.cpp"
          if (__label__ == 2) {
  
            var $4=$ix; //@line 1756 "voices.cpp"
            var $arrayidx2=((__ZL11voices_list+$4*4)&4294967295); //@line 1756 "voices.cpp"
            var $5=HEAP32[(($arrayidx2)>>2)]; //@line 1756 "voices.cpp"
            var $6=$5; //@line 1756 "voices.cpp"
            ; //@line 1756 "voices.cpp"
            ; //@line 1756 "voices.cpp"
          }
  
          var $7=$ix; //@line 1753 "voices.cpp"
          var $inc=((($7)+1)&4294967295); //@line 1753 "voices.cpp"
          $ix=$inc; //@line 1753 "voices.cpp"
          var $8=$ix; //@line 1753 "voices.cpp"
          var $9=HEAP32[((__ZL13n_voices_list)>>2)]; //@line 1753 "voices.cpp"
          var $cmp=((($8))|0) < ((($9))|0); //@line 1753 "voices.cpp"
          if ($cmp) { __label__ = 1;continue ; } else { __label__ = 4;break $for_body$$for_end$2; } //@line 1753 "voices.cpp"
        }
      }
    } while(0);
  
    HEAP32[((__ZL13n_voices_list)>>2)]=0; //@line 1758 "voices.cpp"
    var $arraydecay=(($path_voices)&4294967295); //@line 1760 "voices.cpp"
    var $call=_sprintf($arraydecay, ((__str72724)&4294967295), allocate([((_path_home)&4294967295),0,0,0,47,0,0,0], ["i8*",0,0,0,"i32",0,0,0], ALLOC_STACK)); //@line 1760 "voices.cpp"
    var $arraydecay3=(($path_voices)&4294967295); //@line 1761 "voices.cpp"
    var $call4=_strlen($arraydecay3); //@line 1761 "voices.cpp"
    var $add=((($call4)+1)&4294967295); //@line 1761 "voices.cpp"
    HEAP32[((__ZL15len_path_voices)>>2)]=$add; //@line 1761 "voices.cpp"
    var $arraydecay5=(($path_voices)&4294967295); //@line 1763 "voices.cpp"
    __ZL9GetVoicesPKc($arraydecay5); //@line 1763 "voices.cpp"
    var $10=HEAP32[((__ZL13n_voices_list)>>2)]; //@line 1764 "voices.cpp"
    var $arrayidx6=((__ZL11voices_list+$10*4)&4294967295); //@line 1764 "voices.cpp"
    HEAP32[(($arrayidx6)>>2)]=0; //@line 1764 "voices.cpp"
    var $11=HEAP32[((__ZL13n_voices_list)>>2)]; //@line 1767 "voices.cpp"
    _qsort(__ZL11voices_list, $11, 4, 6); //@line 1767 "voices.cpp"
    var $12=$voice_spec_addr; //@line 1771 "voices.cpp"
    var $tobool=((($12))|0)!=0; //@line 1771 "voices.cpp"
    if ($tobool) { __label__ = 5;; } else { __label__ = 6;; } //@line 1771 "voices.cpp"
    if (__label__ == 5) {
  
      var $13=$voice_spec_addr; //@line 1774 "voices.cpp"
      var $call8=__ZL14SetVoiceScoresP12espeak_VOICEPS0_i($13, ((__ZZ17espeak_ListVoicesE6voices)&4294967295), 1); //@line 1774 "voices.cpp"
      ; //@line 1775 "voices.cpp"
    }
    else if (__label__ == 6) {
  
      $j=0; //@line 1779 "voices.cpp"
      $ix=0; //@line 1780 "voices.cpp"
      var $14=$ix; //@line 1780 "voices.cpp"
      var $arrayidx101=((__ZL11voices_list+$14*4)&4294967295); //@line 1780 "voices.cpp"
      var $15=HEAP32[(($arrayidx101)>>2)]; //@line 1780 "voices.cpp"
      $v=$15; //@line 1780 "voices.cpp"
      var $cmp112=((($15))|0)!=0; //@line 1780 "voices.cpp"
      if ($cmp112) { __label__ = 7;; } else { __label__ = 12;; } //@line 1780 "voices.cpp"
      $for_body12$$for_end28$12: do { 
        if (__label__ == 7) {
          $for_body12$13: while(1) { 
  
            var $16=$v; //@line 1782 "voices.cpp"
            var $languages=(($16+4)&4294967295); //@line 1782 "voices.cpp"
            var $17=HEAP32[(($languages)>>2)]; //@line 1782 "voices.cpp"
            var $arrayidx13=(($17)&4294967295); //@line 1782 "voices.cpp"
            var $18=HEAP8[($arrayidx13)]; //@line 1782 "voices.cpp"
            var $conv=(tempInt=(($18)),(tempInt>=128?tempInt-256:tempInt)); //@line 1782 "voices.cpp"
            var $cmp14=((($conv))|0)!=0; //@line 1782 "voices.cpp"
            if ($cmp14) { __label__ = 8;; } else { __label__ = 11;; } //@line 1782 "voices.cpp"
            $land_lhs_true$$for_inc26$15: do { 
              if (__label__ == 8) {
  
                var $19=$v; //@line 1782 "voices.cpp"
                var $languages15=(($19+4)&4294967295); //@line 1782 "voices.cpp"
                var $20=HEAP32[(($languages15)>>2)]; //@line 1782 "voices.cpp"
                var $arrayidx16=(($20+1)&4294967295); //@line 1782 "voices.cpp"
                var $call17=_strcmp($arrayidx16, ((__str55707)&4294967295)); //@line 1782 "voices.cpp"
                var $cmp18=((($call17))|0)!=0; //@line 1782 "voices.cpp"
                if (!($cmp18)) { __label__ = 11;break $land_lhs_true$$for_inc26$15; } //@line 1782 "voices.cpp"
  
                var $21=$v; //@line 1782 "voices.cpp"
                var $identifier=(($21+8)&4294967295); //@line 1782 "voices.cpp"
                var $22=HEAP32[(($identifier)>>2)]; //@line 1782 "voices.cpp"
                var $call20=_memcmp($22, ((__str73725)&4294967295), 3); //@line 1782 "voices.cpp"
                var $cmp21=((($call20))|0)!=0; //@line 1782 "voices.cpp"
                if (!($cmp21)) { __label__ = 11;break $land_lhs_true$$for_inc26$15; } //@line 1782 "voices.cpp"
  
                var $23=$v; //@line 1784 "voices.cpp"
                var $24=$j; //@line 1784 "voices.cpp"
                var $inc23=((($24)+1)&4294967295); //@line 1784 "voices.cpp"
                $j=$inc23; //@line 1784 "voices.cpp"
                var $arrayidx24=((__ZZ17espeak_ListVoicesE6voices+$24*4)&4294967295); //@line 1784 "voices.cpp"
                HEAP32[(($arrayidx24)>>2)]=$23; //@line 1784 "voices.cpp"
                ; //@line 1785 "voices.cpp"
              }
            } while(0);
  
            var $25=$ix; //@line 1780 "voices.cpp"
            var $inc27=((($25)+1)&4294967295); //@line 1780 "voices.cpp"
            $ix=$inc27; //@line 1780 "voices.cpp"
            var $26=$ix; //@line 1780 "voices.cpp"
            var $arrayidx10=((__ZL11voices_list+$26*4)&4294967295); //@line 1780 "voices.cpp"
            var $27=HEAP32[(($arrayidx10)>>2)]; //@line 1780 "voices.cpp"
            $v=$27; //@line 1780 "voices.cpp"
            var $cmp11=((($27))|0)!=0; //@line 1780 "voices.cpp"
            if ($cmp11) { __label__ = 7;continue $for_body12$13; } else { __label__ = 12;break $for_body12$$for_end28$12; } //@line 1780 "voices.cpp"
          }
        }
      } while(0);
  
      var $28=$j; //@line 1787 "voices.cpp"
      var $arrayidx29=((__ZZ17espeak_ListVoicesE6voices+$28*4)&4294967295); //@line 1787 "voices.cpp"
      HEAP32[(($arrayidx29)>>2)]=0; //@line 1787 "voices.cpp"
      ;
    }
  
    STACKTOP = __stackBase__;
    return ((__ZZ17espeak_ListVoicesE6voices)&4294967295); //@line 1789 "voices.cpp"
    return null;
  }
  

  function __Z11SelectVoiceP12espeak_VOICEPi($voice_select, $found) {
    var __stackBase__  = STACKTOP; STACKTOP += 1272; _memset(__stackBase__, 0, 1272);
    var __label__;
  
    var $retval;
    var $voice_select_addr;
    var $found_addr;
    var $nv;
    var $ix;
    var $ix2;
    var $j;
    var $n_variants;
    var $variant_number;
    var $gender;
    var $skip;
    var $aged;
    var $variant_name;
    var $p;
    var $p_start;
    var $vp;
    var $vp2;
    var $voice_select2=__stackBase__;
    var $voices=__stackBase__+24;
    var $voices2=__stackBase__+624;
    $voice_select_addr=$voice_select;
    $found_addr=$found;
    $aged=1; //@line 1388 "voices.cpp"
    $vp=0; //@line 1391 "voices.cpp"
    var $0=$found_addr; //@line 1399 "voices.cpp"
    HEAP32[(($0)>>2)]=1; //@line 1399 "voices.cpp"
    var $1=$voice_select2; //@line 1400 "voices.cpp"
    var $2=$voice_select_addr; //@line 1400 "voices.cpp"
    var $3=$2; //@line 1400 "voices.cpp"
    var $src$, $dest$, $stop$, $stop4$, $fast$;
    $src$ = $3;
    $dest$ = $1;
    $stop$ = $src$ + 24;
    if (($dest$%4) == ($src$%4) && 24 > 8) {
      while ($src$%4 !== 0 && $src$ < $stop$) {
        ; HEAP8[$dest$++] = HEAP8[$src$++];
      }
      $src$ >>= 2;
      $dest$ >>= 2;
      $stop4$ = $stop$ >> 2;
      while ($src$ < $stop4$) {
    ;;;    HEAP32[$dest$++] = HEAP32[$src$++];
      }
      $src$ <<= 2;
      $dest$ <<= 2;
    }while ($src$ < $stop$) {
      ; HEAP8[$dest$++] = HEAP8[$src$++];
    }; //@line 1400 "voices.cpp"
    var $4=HEAP32[((__ZL13n_voices_list)>>2)]; //@line 1402 "voices.cpp"
    var $cmp=((($4))|0)==0; //@line 1402 "voices.cpp"
    if ($cmp) { __label__ = 1;; } else { __label__ = 2;; } //@line 1402 "voices.cpp"
    if (__label__ == 1) {
  
      var $call=_espeak_ListVoices(0); //@line 1403 "voices.cpp"
      ; //@line 1403 "voices.cpp"
    }
  
    var $languages=(($voice_select2+4)&4294967295); //@line 1405 "voices.cpp"
    var $5=HEAP32[(($languages)>>2)]; //@line 1405 "voices.cpp"
    var $cmp1=((($5))|0)==0; //@line 1405 "voices.cpp"
    if ($cmp1) { __label__ = 4;; } else { __label__ = 3;; } //@line 1405 "voices.cpp"
    $if_then4$$lor_lhs_false$5: do { 
      if (__label__ == 3) {
  
        var $languages2=(($voice_select2+4)&4294967295); //@line 1405 "voices.cpp"
        var $6=HEAP32[(($languages2)>>2)]; //@line 1405 "voices.cpp"
        var $arrayidx=(($6)&4294967295); //@line 1405 "voices.cpp"
        var $7=HEAP8[($arrayidx)]; //@line 1405 "voices.cpp"
        var $conv=(tempInt=(($7)),(tempInt>=128?tempInt-256:tempInt)); //@line 1405 "voices.cpp"
        var $cmp3=((($conv))|0)==0; //@line 1405 "voices.cpp"
        if ($cmp3) { __label__ = 4;break $if_then4$$lor_lhs_false$5; } else { __label__ = 14;break $if_then4$$lor_lhs_false$5; } //@line 1405 "voices.cpp"
      }
    } while(0);
    $if_then4$$if_end41$7: do { 
      if (__label__ == 4) {
  
        var $name=(($voice_select2)&4294967295); //@line 1410 "voices.cpp"
        var $8=HEAP32[(($name)>>2)]; //@line 1410 "voices.cpp"
        var $cmp5=((($8))|0)==0; //@line 1410 "voices.cpp"
        if ($cmp5) { __label__ = 5;; } else { __label__ = 7;; } //@line 1410 "voices.cpp"
        $if_then6$$if_end12$9: do { 
          if (__label__ == 5) {
  
            var $identifier=(($voice_select2+8)&4294967295); //@line 1412 "voices.cpp"
            var $9=HEAP32[(($identifier)>>2)]; //@line 1412 "voices.cpp"
            var $name7=(($voice_select2)&4294967295); //@line 1412 "voices.cpp"
            HEAP32[(($name7)>>2)]=$9; //@line 1412 "voices.cpp"
            var $cmp8=((($9))|0)==0; //@line 1412 "voices.cpp"
            if (!($cmp8)) { __label__ = 7;break $if_then6$$if_end12$9; } //@line 1412 "voices.cpp"
  
            var $name10=(($voice_select2)&4294967295); //@line 1413 "voices.cpp"
            HEAP32[(($name10)>>2)]=((__str45697)&4294967295); //@line 1413 "voices.cpp"
            ; //@line 1413 "voices.cpp"
          }
        } while(0);
  
        var $name13=(($voice_select2)&4294967295); //@line 1416 "voices.cpp"
        var $10=HEAP32[(($name13)>>2)]; //@line 1416 "voices.cpp"
        __Z8strncpy0PcPKci(((__ZZ11SelectVoiceP12espeak_VOICEPiE3buf)&4294967295), $10, 60); //@line 1416 "voices.cpp"
        __ZL23ExtractVoiceVariantNamePci(((__ZZ11SelectVoiceP12espeak_VOICEPiE3buf)&4294967295), 0); //@line 1417 "voices.cpp"
        $variant_name=((__ZZL23ExtractVoiceVariantNamePciE12variant_name)&4294967295); //@line 1417 "voices.cpp"
        var $call15=__Z17SelectVoiceByNamePP12espeak_VOICEPKc(((__ZL11voices_list)&4294967295), ((__ZZ11SelectVoiceP12espeak_VOICEPiE3buf)&4294967295)); //@line 1419 "voices.cpp"
        $vp=$call15; //@line 1419 "voices.cpp"
        var $cmp16=((($call15))|0)!=0; //@line 1420 "voices.cpp"
        if (!($cmp16)) { __label__ = 14;break $if_then4$$if_end41$7; } //@line 1420 "voices.cpp"
  
        var $11=$vp; //@line 1422 "voices.cpp"
        var $languages18=(($11+4)&4294967295); //@line 1422 "voices.cpp"
        var $12=HEAP32[(($languages18)>>2)]; //@line 1422 "voices.cpp"
        var $arrayidx19=(($12+1)&4294967295); //@line 1422 "voices.cpp"
        var $languages20=(($voice_select2+4)&4294967295); //@line 1422 "voices.cpp"
        HEAP32[(($languages20)>>2)]=$arrayidx19; //@line 1422 "voices.cpp"
        var $gender21=(($voice_select2+12)&4294967295); //@line 1424 "voices.cpp"
        var $13=HEAPU8[($gender21)]; //@line 1424 "voices.cpp"
        var $conv22=((($13))&255); //@line 1424 "voices.cpp"
        var $cmp23=((($conv22))|0)==0; //@line 1424 "voices.cpp"
        if (!($cmp23)) { __label__ = 14;break $if_then4$$if_end41$7; } //@line 1424 "voices.cpp"
  
        var $age=(($voice_select2+13)&4294967295); //@line 1424 "voices.cpp"
        var $14=HEAPU8[($age)]; //@line 1424 "voices.cpp"
        var $conv24=((($14))&255); //@line 1424 "voices.cpp"
        var $cmp25=((($conv24))|0)==0; //@line 1424 "voices.cpp"
        if (!($cmp25)) { __label__ = 14;break $if_then4$$if_end41$7; } //@line 1424 "voices.cpp"
  
        var $variant=(($voice_select2+14)&4294967295); //@line 1424 "voices.cpp"
        var $15=HEAPU8[($variant)]; //@line 1424 "voices.cpp"
        var $conv27=((($15))&255); //@line 1424 "voices.cpp"
        var $cmp28=((($conv27))|0)==0; //@line 1424 "voices.cpp"
        if (!($cmp28)) { __label__ = 14;break $if_then4$$if_end41$7; } //@line 1424 "voices.cpp"
  
        var $16=$variant_name; //@line 1426 "voices.cpp"
        var $arrayidx30=(($16)&4294967295); //@line 1426 "voices.cpp"
        var $17=HEAP8[($arrayidx30)]; //@line 1426 "voices.cpp"
        var $conv31=(tempInt=(($17)),(tempInt>=128?tempInt-256:tempInt)); //@line 1426 "voices.cpp"
        var $cmp32=((($conv31))|0)!=0; //@line 1426 "voices.cpp"
        var $18=$vp; //@line 1428 "voices.cpp"
        var $identifier34=(($18+8)&4294967295); //@line 1428 "voices.cpp"
        var $19=HEAP32[(($identifier34)>>2)]; //@line 1428 "voices.cpp"
        if ($cmp32) { __label__ = 12;; } else { __label__ = 13;; } //@line 1426 "voices.cpp"
        if (__label__ == 12) {
  
          var $20=$variant_name; //@line 1428 "voices.cpp"
          var $arrayidx35=(($20+3)&4294967295); //@line 1428 "voices.cpp"
          var $call36=_sprintf(((__ZZ11SelectVoiceP12espeak_VOICEPiE8voice_id)&4294967295), ((__str71723)&4294967295), allocate([$19,0,0,0,$arrayidx35,0,0,0], ["i8*",0,0,0,"i8*",0,0,0], ALLOC_STACK)); //@line 1428 "voices.cpp"
          $retval=((__ZZ11SelectVoiceP12espeak_VOICEPiE8voice_id)&4294967295); //@line 1429 "voices.cpp"
          __label__ = 53;break $if_then4$$if_end41$7; //@line 1429 "voices.cpp"
        }
        else if (__label__ == 13) {
  
          $retval=$19; //@line 1432 "voices.cpp"
          __label__ = 53;break $if_then4$$if_end41$7; //@line 1432 "voices.cpp"
        }
      }
    } while(0);
    if (__label__ == 14) {
  
      var $arraydecay=(($voices)&4294967295); //@line 1438 "voices.cpp"
      var $call42=__ZL14SetVoiceScoresP12espeak_VOICEPS0_i($voice_select2, $arraydecay, 0); //@line 1438 "voices.cpp"
      $nv=$call42; //@line 1438 "voices.cpp"
      var $cmp43=((($call42))|0)==0; //@line 1440 "voices.cpp"
      if ($cmp43) { __label__ = 15;; } else { __label__ = 17;; } //@line 1440 "voices.cpp"
      $if_then44$$if_end50$22: do { 
        if (__label__ == 15) {
  
          var $21=$found_addr; //@line 1443 "voices.cpp"
          HEAP32[(($21)>>2)]=0; //@line 1443 "voices.cpp"
          var $call45=__Z17SelectVoiceByNamePP12espeak_VOICEPKc(((__ZL11voices_list)&4294967295), ((__str45697)&4294967295)); //@line 1444 "voices.cpp"
          var $arrayidx46=(($voices)&4294967295); //@line 1444 "voices.cpp"
          HEAP32[(($arrayidx46)>>2)]=$call45; //@line 1444 "voices.cpp"
          var $cmp47=((($call45))|0)!=0; //@line 1444 "voices.cpp"
          if (!($cmp47)) { __label__ = 17;break $if_then44$$if_end50$22; } //@line 1444 "voices.cpp"
  
          $nv=1; //@line 1445 "voices.cpp"
          ; //@line 1445 "voices.cpp"
        }
      } while(0);
  
      $gender=0; //@line 1448 "voices.cpp"
      var $gender51=(($voice_select2+12)&4294967295); //@line 1449 "voices.cpp"
      var $22=HEAPU8[($gender51)]; //@line 1449 "voices.cpp"
      var $conv52=((($22))&255); //@line 1449 "voices.cpp"
      var $cmp53=((($conv52))|0)==2; //@line 1449 "voices.cpp"
      if ($cmp53) { __label__ = 20;; } else { __label__ = 18;; } //@line 1449 "voices.cpp"
      $if_then62$$lor_lhs_false54$26: do { 
        if (__label__ == 18) {
  
          var $age55=(($voice_select2+13)&4294967295); //@line 1449 "voices.cpp"
          var $23=HEAPU8[($age55)]; //@line 1449 "voices.cpp"
          var $conv56=((($23))&255); //@line 1449 "voices.cpp"
          var $cmp57=((($conv56))|0) > 0; //@line 1449 "voices.cpp"
          if ($cmp57) { __label__ = 19;; } else { __label__ = 21;; } //@line 1449 "voices.cpp"
          if (__label__ == 19) {
  
            var $age59=(($voice_select2+13)&4294967295); //@line 1449 "voices.cpp"
            var $24=HEAPU8[($age59)]; //@line 1449 "voices.cpp"
            var $conv60=((($24))&255); //@line 1449 "voices.cpp"
            var $cmp61=((($conv60))|0) < 13; //@line 1449 "voices.cpp"
            if ($cmp61) { __label__ = 20;break $if_then62$$lor_lhs_false54$26; } //@line 1449 "voices.cpp"
          }
  
          var $gender63=(($voice_select2+12)&4294967295); //@line 1452 "voices.cpp"
          var $25=HEAPU8[($gender63)]; //@line 1452 "voices.cpp"
          var $conv64=((($25))&255); //@line 1452 "voices.cpp"
          var $cmp65=((($conv64))|0)==1; //@line 1452 "voices.cpp"
          if (!($cmp65)) { __label__ = 23;break $if_then62$$lor_lhs_false54$26; } //@line 1452 "voices.cpp"
  
          $gender=1; //@line 1453 "voices.cpp"
          __label__ = 23;break $if_then62$$lor_lhs_false54$26; //@line 1453 "voices.cpp"
        }
      } while(0);
      if (__label__ == 20) {
  
        $gender=2; //@line 1450 "voices.cpp"
        ; //@line 1450 "voices.cpp"
      }
  
      var $age69=(($voice_select2+13)&4294967295); //@line 1456 "voices.cpp"
      var $26=HEAPU8[($age69)]; //@line 1456 "voices.cpp"
      var $conv70=((($26))&255); //@line 1456 "voices.cpp"
      var $cmp71=((($conv70))|0) < 60; //@line 1456 "voices.cpp"
      if ($cmp71) { __label__ = 24;; } else { __label__ = 25;; } //@line 1456 "voices.cpp"
      $if_end73_thread$$if_end73$35: do { 
        if (__label__ == 24) {
  
          $aged=0; //@line 1457 "voices.cpp"
          var $27=$gender; //@line 1459 "voices.cpp"
          var $arrayidx742=((_variant_lists+$27*4)&4294967295); //@line 1459 "voices.cpp"
          var $28=HEAP32[(($arrayidx742)>>2)]; //@line 1459 "voices.cpp"
          $p_start=$28; //@line 1459 "voices.cpp"
          $p=$28; //@line 1459 "voices.cpp"
          __label__ = 26;break $if_end73_thread$$if_end73$35; //@line 1460 "voices.cpp"
        }
        else if (__label__ == 25) {
  
          var $_pr=$aged; //@line 1460 "voices.cpp"
          var $29=$gender; //@line 1459 "voices.cpp"
          var $arrayidx74=((_variant_lists+$29*4)&4294967295); //@line 1459 "voices.cpp"
          var $30=HEAP32[(($arrayidx74)>>2)]; //@line 1459 "voices.cpp"
          $p_start=$30; //@line 1459 "voices.cpp"
          $p=$30; //@line 1459 "voices.cpp"
          var $cmp75=((($_pr))|0)==0; //@line 1460 "voices.cpp"
          if ($cmp75) { __label__ = 26;break $if_end73_thread$$if_end73$35; } else { __label__ = 27;break $if_end73_thread$$if_end73$35; } //@line 1460 "voices.cpp"
        }
      } while(0);
      if (__label__ == 26) {
  
        var $31=$p; //@line 1461 "voices.cpp"
        var $incdec_ptr=(($31+1)&4294967295); //@line 1461 "voices.cpp"
        $p=$incdec_ptr; //@line 1461 "voices.cpp"
        ; //@line 1461 "voices.cpp"
      }
  
      $n_variants=0; //@line 1464 "voices.cpp"
      $ix=0; //@line 1465 "voices.cpp"
      $ix2=0; //@line 1465 "voices.cpp"
      var $32=$ix; //@line 1465 "voices.cpp"
      var $33=$nv; //@line 1465 "voices.cpp"
      var $cmp786=((($32))|0) < ((($33))|0); //@line 1465 "voices.cpp"
      if ($cmp786) { __label__ = 28;; } else { __label__ = 44;; } //@line 1465 "voices.cpp"
      $for_body$$while_cond$41: do { 
        if (__label__ == 28) {
          $for_body$42: while(1) { 
  
            var $34=$ix; //@line 1467 "voices.cpp"
            var $arrayidx79=(($voices+$34*4)&4294967295); //@line 1467 "voices.cpp"
            var $35=HEAP32[(($arrayidx79)>>2)]; //@line 1467 "voices.cpp"
            $vp=$35; //@line 1467 "voices.cpp"
            $skip=0; //@line 1469 "voices.cpp"
            var $36=$gender; //@line 1470 "voices.cpp"
            var $cmp80=((($36))|0)!=0; //@line 1470 "voices.cpp"
            if ($cmp80) { __label__ = 29;; } else { __label__ = 31;; } //@line 1470 "voices.cpp"
            $land_lhs_true81$$if_end86$44: do { 
              if (__label__ == 29) {
  
                var $37=$vp; //@line 1470 "voices.cpp"
                var $gender82=(($37+12)&4294967295); //@line 1470 "voices.cpp"
                var $38=HEAPU8[($gender82)]; //@line 1470 "voices.cpp"
                var $conv83=((($38))&255); //@line 1470 "voices.cpp"
                var $39=$gender; //@line 1470 "voices.cpp"
                var $cmp84=((($conv83))|0)!=((($39))|0); //@line 1470 "voices.cpp"
                if (!($cmp84)) { __label__ = 31;break $land_lhs_true81$$if_end86$44; } //@line 1470 "voices.cpp"
  
                $skip=1; //@line 1472 "voices.cpp"
                ; //@line 1473 "voices.cpp"
              }
            } while(0);
  
            var $40=$ix2; //@line 1474 "voices.cpp"
            var $cmp87=((($40))|0)==0; //@line 1474 "voices.cpp"
            if ($cmp87) { __label__ = 32;; } else { __label__ = 35;; } //@line 1474 "voices.cpp"
            $land_lhs_true88$$if_end94$48: do { 
              if (__label__ == 32) {
  
                var $41=$aged; //@line 1474 "voices.cpp"
                var $tobool=((($41))|0)!=0; //@line 1474 "voices.cpp"
                if (!($tobool)) { __label__ = 35;break $land_lhs_true88$$if_end94$48; } //@line 1474 "voices.cpp"
  
                var $42=$vp; //@line 1474 "voices.cpp"
                var $age90=(($42+13)&4294967295); //@line 1474 "voices.cpp"
                var $43=HEAPU8[($age90)]; //@line 1474 "voices.cpp"
                var $conv91=((($43))&255); //@line 1474 "voices.cpp"
                var $cmp92=((($conv91))|0) < 60; //@line 1474 "voices.cpp"
                if (!($cmp92)) { __label__ = 35;break $land_lhs_true88$$if_end94$48; } //@line 1474 "voices.cpp"
  
                $skip=1; //@line 1476 "voices.cpp"
                __label__ = 37;break $land_lhs_true88$$if_end94$48; //@line 1478 "voices.cpp"
              }
            } while(0);
            $if_end98$$if_end94$52: do { 
              if (__label__ == 35) {
  
                var $_pr4=$skip; //@line 1478 "voices.cpp"
                var $cmp95=((($_pr4))|0)==0; //@line 1478 "voices.cpp"
                if (!($cmp95)) { __label__ = 37;break $if_end98$$if_end94$52; } //@line 1478 "voices.cpp"
  
                var $44=$vp; //@line 1480 "voices.cpp"
                var $45=$ix2; //@line 1480 "voices.cpp"
                var $inc=((($45)+1)&4294967295); //@line 1480 "voices.cpp"
                $ix2=$inc; //@line 1480 "voices.cpp"
                var $arrayidx97=(($voices2+$45*4)&4294967295); //@line 1480 "voices.cpp"
                HEAP32[(($arrayidx97)>>2)]=$44; //@line 1480 "voices.cpp"
                ; //@line 1481 "voices.cpp"
              }
            } while(0);
  
            $j=0; //@line 1483 "voices.cpp"
            ; //@line 1483 "voices.cpp"
            while(1) { 
  
              var $46=$j; //@line 1483 "voices.cpp"
              var $47=$vp; //@line 1483 "voices.cpp"
              var $xx1=(($47+15)&4294967295); //@line 1483 "voices.cpp"
              var $48=HEAPU8[($xx1)]; //@line 1483 "voices.cpp"
              var $conv100=((($48))&255); //@line 1483 "voices.cpp"
              var $cmp101=((($46))|0) < ((($conv100))|0); //@line 1483 "voices.cpp"
              if (!($cmp101)) { __label__ = 43;break ; } //@line 1483 "voices.cpp"
  
              var $49=$n_variants; //@line 1483 "voices.cpp"
              var $cmp102=((($49))|0) < 12; //@line 1483 "voices.cpp"
              if (!($cmp102)) { __label__ = 43;break ; }
  
              var $50=$p; //@line 1485 "voices.cpp"
              var $51=HEAP8[($50)]; //@line 1485 "voices.cpp"
              var $conv104=(tempInt=(($51)),(tempInt>=128?tempInt-256:tempInt)); //@line 1485 "voices.cpp"
              $variant_number=$conv104; //@line 1485 "voices.cpp"
              var $cmp105=((($conv104))|0)==0; //@line 1485 "voices.cpp"
              if ($cmp105) { __label__ = 41;; } else { __label__ = 42;; } //@line 1485 "voices.cpp"
              if (__label__ == 41) {
  
                var $52=$p_start; //@line 1487 "voices.cpp"
                $p=$52; //@line 1487 "voices.cpp"
                __label__ = 38;continue ; //@line 1488 "voices.cpp"
              }
              else if (__label__ == 42) {
  
                var $53=$n_variants; //@line 1491 "voices.cpp"
                var $inc108=((($53)+1)&4294967295); //@line 1491 "voices.cpp"
                $n_variants=$inc108; //@line 1491 "voices.cpp"
                var $arrayidx109=((__ZZ11SelectVoiceP12espeak_VOICEPiE14voice_variants+$53*24)&4294967295); //@line 1491 "voices.cpp"
                $vp2=$arrayidx109; //@line 1491 "voices.cpp"
                var $54=$vp2; //@line 1492 "voices.cpp"
                var $55=$54; //@line 1492 "voices.cpp"
                var $56=$vp; //@line 1492 "voices.cpp"
                var $57=$56; //@line 1492 "voices.cpp"
                var $src$, $dest$, $stop$, $stop4$, $fast$;
                $src$ = $57;
                $dest$ = $55;
                $stop$ = $src$ + 24;
                if (($dest$%4) == ($src$%4) && 24 > 8) {
                  while ($src$%4 !== 0 && $src$ < $stop$) {
                    ; HEAP8[$dest$++] = HEAP8[$src$++];
                  }
                  $src$ >>= 2;
                  $dest$ >>= 2;
                  $stop4$ = $stop$ >> 2;
                  while ($src$ < $stop4$) {
                ;;;    HEAP32[$dest$++] = HEAP32[$src$++];
                  }
                  $src$ <<= 2;
                  $dest$ <<= 2;
                }while ($src$ < $stop$) {
                  ; HEAP8[$dest$++] = HEAP8[$src$++];
                }; //@line 1492 "voices.cpp"
                var $58=$variant_number; //@line 1493 "voices.cpp"
                var $conv110=((($58)) & 255); //@line 1493 "voices.cpp"
                var $59=$vp2; //@line 1493 "voices.cpp"
                var $variant111=(($59+14)&4294967295); //@line 1493 "voices.cpp"
                HEAP8[($variant111)]=$conv110; //@line 1493 "voices.cpp"
                var $60=$vp2; //@line 1494 "voices.cpp"
                var $61=$ix2; //@line 1494 "voices.cpp"
                var $inc112=((($61)+1)&4294967295); //@line 1494 "voices.cpp"
                $ix2=$inc112; //@line 1494 "voices.cpp"
                var $arrayidx113=(($voices2+$61*4)&4294967295); //@line 1494 "voices.cpp"
                HEAP32[(($arrayidx113)>>2)]=$60; //@line 1494 "voices.cpp"
                var $62=$p; //@line 1495 "voices.cpp"
                var $incdec_ptr114=(($62+1)&4294967295); //@line 1495 "voices.cpp"
                $p=$incdec_ptr114; //@line 1495 "voices.cpp"
                var $63=$j; //@line 1496 "voices.cpp"
                var $inc115=((($63)+1)&4294967295); //@line 1496 "voices.cpp"
                $j=$inc115; //@line 1496 "voices.cpp"
                __label__ = 38;continue ; //@line 1497 "voices.cpp"
              }
            }
  
            var $64=$ix; //@line 1465 "voices.cpp"
            var $inc116=((($64)+1)&4294967295); //@line 1465 "voices.cpp"
            $ix=$inc116; //@line 1465 "voices.cpp"
            var $65=$ix; //@line 1465 "voices.cpp"
            var $66=$nv; //@line 1465 "voices.cpp"
            var $cmp78=((($65))|0) < ((($66))|0); //@line 1465 "voices.cpp"
            if ($cmp78) { __label__ = 28;continue $for_body$42; } else { __label__ = 44;break $for_body$$while_cond$41; } //@line 1465 "voices.cpp"
          }
        }
      } while(0);
      while(1) { 
  
        var $67=$vp; //@line 1500 "voices.cpp"
        var $cmp118=((($67))|0)!=0; //@line 1500 "voices.cpp"
        if (!($cmp118)) { __label__ = 48;break ; } //@line 1500 "voices.cpp"
  
        var $68=$p; //@line 1500 "voices.cpp"
        var $incdec_ptr120=(($68+1)&4294967295); //@line 1500 "voices.cpp"
        $p=$incdec_ptr120; //@line 1500 "voices.cpp"
        var $69=HEAP8[($68)]; //@line 1500 "voices.cpp"
        var $conv121=(tempInt=(($69)),(tempInt>=128?tempInt-256:tempInt)); //@line 1500 "voices.cpp"
        $variant_number=$conv121; //@line 1500 "voices.cpp"
        var $cmp122=((($conv121))|0)!=0; //@line 1500 "voices.cpp"
        if (!($cmp122)) { __label__ = 48;break ; } //@line 1500 "voices.cpp"
  
        var $70=$n_variants; //@line 1500 "voices.cpp"
        var $cmp124=((($70))|0) < 12; //@line 1500 "voices.cpp"
        if (!($cmp124)) { __label__ = 48;break ; }
  
        var $71=$n_variants; //@line 1502 "voices.cpp"
        var $inc126=((($71)+1)&4294967295); //@line 1502 "voices.cpp"
        $n_variants=$inc126; //@line 1502 "voices.cpp"
        var $arrayidx127=((__ZZ11SelectVoiceP12espeak_VOICEPiE14voice_variants+$71*24)&4294967295); //@line 1502 "voices.cpp"
        $vp2=$arrayidx127; //@line 1502 "voices.cpp"
        var $72=$vp2; //@line 1503 "voices.cpp"
        var $73=$72; //@line 1503 "voices.cpp"
        var $74=$vp; //@line 1503 "voices.cpp"
        var $75=$74; //@line 1503 "voices.cpp"
        var $src$, $dest$, $stop$, $stop4$, $fast$;
        $src$ = $75;
        $dest$ = $73;
        $stop$ = $src$ + 24;
        if (($dest$%4) == ($src$%4) && 24 > 8) {
          while ($src$%4 !== 0 && $src$ < $stop$) {
            ; HEAP8[$dest$++] = HEAP8[$src$++];
          }
          $src$ >>= 2;
          $dest$ >>= 2;
          $stop4$ = $stop$ >> 2;
          while ($src$ < $stop4$) {
        ;;;    HEAP32[$dest$++] = HEAP32[$src$++];
          }
          $src$ <<= 2;
          $dest$ <<= 2;
        }while ($src$ < $stop$) {
          ; HEAP8[$dest$++] = HEAP8[$src$++];
        }; //@line 1503 "voices.cpp"
        var $76=$variant_number; //@line 1504 "voices.cpp"
        var $conv128=((($76)) & 255); //@line 1504 "voices.cpp"
        var $77=$vp2; //@line 1504 "voices.cpp"
        var $variant129=(($77+14)&4294967295); //@line 1504 "voices.cpp"
        HEAP8[($variant129)]=$conv128; //@line 1504 "voices.cpp"
        var $78=$vp2; //@line 1505 "voices.cpp"
        var $79=$ix2; //@line 1505 "voices.cpp"
        var $inc130=((($79)+1)&4294967295); //@line 1505 "voices.cpp"
        $ix2=$inc130; //@line 1505 "voices.cpp"
        var $arrayidx131=(($voices2+$79*4)&4294967295); //@line 1505 "voices.cpp"
        HEAP32[(($arrayidx131)>>2)]=$78; //@line 1505 "voices.cpp"
        __label__ = 44;continue ; //@line 1506 "voices.cpp"
      }
  
      var $80=$ix2; //@line 1509 "voices.cpp"
      var $cmp132=((($80))|0)==0; //@line 1509 "voices.cpp"
      if ($cmp132) { __label__ = 49;; } else { __label__ = 50;; } //@line 1509 "voices.cpp"
      if (__label__ == 49) {
  
        $retval=0; //@line 1510 "voices.cpp"
        ; //@line 1510 "voices.cpp"
      }
      else if (__label__ == 50) {
  
        var $variant135=(($voice_select2+14)&4294967295); //@line 1511 "voices.cpp"
        var $81=HEAPU8[($variant135)]; //@line 1511 "voices.cpp"
        var $conv136=((($81))&255); //@line 1511 "voices.cpp"
        var $82=$ix2; //@line 1511 "voices.cpp"
        var $rem=((($conv136))|0)%((($82))|0); //@line 1511 "voices.cpp"
        var $arrayidx137=(($voices2+$rem*4)&4294967295); //@line 1511 "voices.cpp"
        var $83=HEAP32[(($arrayidx137)>>2)]; //@line 1511 "voices.cpp"
        $vp=$83; //@line 1511 "voices.cpp"
        var $84=$vp; //@line 1513 "voices.cpp"
        var $variant138=(($84+14)&4294967295); //@line 1513 "voices.cpp"
        var $85=HEAPU8[($variant138)]; //@line 1513 "voices.cpp"
        var $conv139=((($85))&255); //@line 1513 "voices.cpp"
        var $cmp140=((($conv139))|0)!=0; //@line 1513 "voices.cpp"
        var $86=$vp; //@line 1515 "voices.cpp"
        if ($cmp140) { __label__ = 51;; } else { __label__ = 52;; } //@line 1513 "voices.cpp"
        if (__label__ == 51) {
  
          var $variant142=(($86+14)&4294967295); //@line 1515 "voices.cpp"
          var $87=HEAPU8[($variant142)]; //@line 1515 "voices.cpp"
          var $conv143=((($87))&255); //@line 1515 "voices.cpp"
          __ZL23ExtractVoiceVariantNamePci(0, $conv143); //@line 1515 "voices.cpp"
          $variant_name=((__ZZL23ExtractVoiceVariantNamePciE12variant_name)&4294967295); //@line 1515 "voices.cpp"
          var $88=$vp; //@line 1516 "voices.cpp"
          var $identifier145=(($88+8)&4294967295); //@line 1516 "voices.cpp"
          var $89=HEAP32[(($identifier145)>>2)]; //@line 1516 "voices.cpp"
          var $90=$variant_name; //@line 1516 "voices.cpp"
          var $arrayidx146=(($90+3)&4294967295); //@line 1516 "voices.cpp"
          var $call147=_sprintf(((__ZZ11SelectVoiceP12espeak_VOICEPiE8voice_id)&4294967295), ((__str71723)&4294967295), allocate([$89,0,0,0,$arrayidx146,0,0,0], ["i8*",0,0,0,"i8*",0,0,0], ALLOC_STACK)); //@line 1516 "voices.cpp"
          $retval=((__ZZ11SelectVoiceP12espeak_VOICEPiE8voice_id)&4294967295); //@line 1517 "voices.cpp"
          ; //@line 1517 "voices.cpp"
        }
        else if (__label__ == 52) {
  
          var $identifier149=(($86+8)&4294967295); //@line 1520 "voices.cpp"
          var $91=HEAP32[(($identifier149)>>2)]; //@line 1520 "voices.cpp"
          $retval=$91; //@line 1520 "voices.cpp"
          ; //@line 1520 "voices.cpp"
        }
      }
    }
  
    var $92=$retval; //@line 1521 "voices.cpp"
    STACKTOP = __stackBase__;
    return $92; //@line 1521 "voices.cpp"
    return null;
  }
  

  function __ZL14SetVoiceScoresP12espeak_VOICEPS0_i($voice_select, $voices, $control) {
    var __stackBase__  = STACKTOP; STACKTOP += 80; _memset(__stackBase__, 0, 80);
    var __label__;
  
    var $retval;
    var $voice_select_addr;
    var $voices_addr;
    var $control_addr;
    var $ix;
    var $score;
    var $nv;
    var $n_parts;
    var $lang_len;
    var $vp;
    var $language=__stackBase__;
    $voice_select_addr=$voice_select;
    $voices_addr=$voices;
    $control_addr=$control;
    $n_parts=0; //@line 1277 "voices.cpp"
    $lang_len=0; //@line 1278 "voices.cpp"
    var $0=$voice_select_addr; //@line 1283 "voices.cpp"
    var $languages=(($0+4)&4294967295); //@line 1283 "voices.cpp"
    var $1=HEAP32[(($languages)>>2)]; //@line 1283 "voices.cpp"
    var $cmp=((($1))|0)!=0; //@line 1283 "voices.cpp"
    if ($cmp) { __label__ = 1;; } else { __label__ = 6;; } //@line 1283 "voices.cpp"
    $land_lhs_true$$if_end16$2: do { 
      if (__label__ == 1) {
  
        var $2=$voice_select_addr; //@line 1283 "voices.cpp"
        var $languages1=(($2+4)&4294967295); //@line 1283 "voices.cpp"
        var $3=HEAP32[(($languages1)>>2)]; //@line 1283 "voices.cpp"
        var $arrayidx=(($3)&4294967295); //@line 1283 "voices.cpp"
        var $4=HEAP8[($arrayidx)]; //@line 1283 "voices.cpp"
        var $conv=(tempInt=(($4)),(tempInt>=128?tempInt-256:tempInt)); //@line 1283 "voices.cpp"
        var $cmp2=((($conv))|0)!=0; //@line 1283 "voices.cpp"
        if (!($cmp2)) { __label__ = 6;break $land_lhs_true$$if_end16$2; } //@line 1283 "voices.cpp"
  
        $n_parts=1; //@line 1285 "voices.cpp"
        var $5=$voice_select_addr; //@line 1286 "voices.cpp"
        var $languages3=(($5+4)&4294967295); //@line 1286 "voices.cpp"
        var $6=HEAP32[(($languages3)>>2)]; //@line 1286 "voices.cpp"
        var $call=_strlen($6); //@line 1286 "voices.cpp"
        $lang_len=$call; //@line 1286 "voices.cpp"
        $ix=0; //@line 1287 "voices.cpp"
        var $7=$ix; //@line 1287 "voices.cpp"
        var $8=$lang_len; //@line 1287 "voices.cpp"
        var $cmp42=((($7))|0) <= ((($8))|0); //@line 1287 "voices.cpp"
        var $9=$ix; //@line 1287 "voices.cpp"
        var $cmp53=((($9))>>>0) < 80; //@line 1287 "voices.cpp"
        var $or_cond4=($cmp42) & ($cmp53); //@line 1287 "voices.cpp"
        if (!($or_cond4)) { __label__ = 6;break $land_lhs_true$$if_end16$2; } //@line 1287 "voices.cpp"
        while(1) { 
  
          var $10=$ix; //@line 1289 "voices.cpp"
          var $11=$voice_select_addr; //@line 1289 "voices.cpp"
          var $languages6=(($11+4)&4294967295); //@line 1289 "voices.cpp"
          var $12=HEAP32[(($languages6)>>2)]; //@line 1289 "voices.cpp"
          var $arrayidx7=(($12+$10)&4294967295); //@line 1289 "voices.cpp"
          var $13=HEAP8[($arrayidx7)]; //@line 1289 "voices.cpp"
          var $conv8=(tempInt=(($13)),(tempInt>=128?tempInt-256:tempInt)); //@line 1289 "voices.cpp"
          var $call9=_tolower($conv8); //@line 1289 "voices.cpp"
          var $conv10=((($call9)) & 255); //@line 1289 "voices.cpp"
          var $14=$ix; //@line 1289 "voices.cpp"
          var $arrayidx11=(($language+$14)&4294967295); //@line 1289 "voices.cpp"
          HEAP8[($arrayidx11)]=$conv10; //@line 1289 "voices.cpp"
          var $conv12=(tempInt=(($conv10)),(tempInt>=128?tempInt-256:tempInt)); //@line 1289 "voices.cpp"
          var $cmp13=((($conv12))|0)==45; //@line 1289 "voices.cpp"
          if ($cmp13) { __label__ = 4;; } else { __label__ = 5;; } //@line 1289 "voices.cpp"
          if (__label__ == 4) {
  
            var $15=$n_parts; //@line 1290 "voices.cpp"
            var $inc=((($15)+1)&4294967295); //@line 1290 "voices.cpp"
            $n_parts=$inc; //@line 1290 "voices.cpp"
            ; //@line 1290 "voices.cpp"
          }
  
          var $16=$ix; //@line 1287 "voices.cpp"
          var $inc15=((($16)+1)&4294967295); //@line 1287 "voices.cpp"
          $ix=$inc15; //@line 1287 "voices.cpp"
          var $17=$ix; //@line 1287 "voices.cpp"
          var $18=$lang_len; //@line 1287 "voices.cpp"
          var $cmp4=((($17))|0) <= ((($18))|0); //@line 1287 "voices.cpp"
          var $19=$ix; //@line 1287 "voices.cpp"
          var $cmp5=((($19))>>>0) < 80; //@line 1287 "voices.cpp"
          var $or_cond=($cmp4) & ($cmp5); //@line 1287 "voices.cpp"
          if ($or_cond) { __label__ = 3;continue ; } else { __label__ = 6;break $land_lhs_true$$if_end16$2; } //@line 1287 "voices.cpp"
        }
      }
    } while(0);
  
    $nv=0; //@line 1294 "voices.cpp"
    $ix=0; //@line 1295 "voices.cpp"
    var $20=$ix; //@line 1295 "voices.cpp"
    var $21=HEAP32[((__ZL13n_voices_list)>>2)]; //@line 1295 "voices.cpp"
    var $cmp181=((($20))|0) < ((($21))|0); //@line 1295 "voices.cpp"
    if ($cmp181) { __label__ = 7;; } else { __label__ = 13;; } //@line 1295 "voices.cpp"
    $for_body19_lr_ph$$for_end37$11: do { 
      if (__label__ == 7) {
  
        var $arraydecay=(($language)&4294967295); //@line 1302 "voices.cpp"
        ; //@line 1295 "voices.cpp"
        $for_body19$13: while(1) { 
  
          var $22=$ix; //@line 1297 "voices.cpp"
          var $arrayidx20=((__ZL11voices_list+$22*4)&4294967295); //@line 1297 "voices.cpp"
          var $23=HEAP32[(($arrayidx20)>>2)]; //@line 1297 "voices.cpp"
          $vp=$23; //@line 1297 "voices.cpp"
          var $24=$control_addr; //@line 1299 "voices.cpp"
          var $and=($24) & 1; //@line 1299 "voices.cpp"
          var $cmp21=((($and))|0)==0; //@line 1299 "voices.cpp"
          if ($cmp21) { __label__ = 9;; } else { __label__ = 10;; } //@line 1299 "voices.cpp"
          $land_lhs_true22$$if_end26$15: do { 
            if (__label__ == 9) {
  
              var $25=$vp; //@line 1299 "voices.cpp"
              var $identifier=(($25+8)&4294967295); //@line 1299 "voices.cpp"
              var $26=HEAP32[(($identifier)>>2)]; //@line 1299 "voices.cpp"
              var $call23=_memcmp($26, ((__str73725)&4294967295), 3); //@line 1299 "voices.cpp"
              var $cmp24=((($call23))|0)==0; //@line 1299 "voices.cpp"
              if ($cmp24) { __label__ = 12;break $land_lhs_true22$$if_end26$15; } else { __label__ = 10;break $land_lhs_true22$$if_end26$15; } //@line 1299 "voices.cpp"
            }
          } while(0);
          $for_inc35$$if_end26$17: do { 
            if (__label__ == 10) {
  
              var $27=$voice_select_addr; //@line 1302 "voices.cpp"
              var $28=$n_parts; //@line 1302 "voices.cpp"
              var $29=$lang_len; //@line 1302 "voices.cpp"
              var $30=$ix; //@line 1302 "voices.cpp"
              var $arrayidx27=((__ZL11voices_list+$30*4)&4294967295); //@line 1302 "voices.cpp"
              var $31=HEAP32[(($arrayidx27)>>2)]; //@line 1302 "voices.cpp"
              var $call28=__ZL10ScoreVoiceP12espeak_VOICEPKciiS0_($27, $arraydecay, $28, $29, $31); //@line 1302 "voices.cpp"
              $score=$call28; //@line 1302 "voices.cpp"
              var $cmp29=((($call28))|0) > 0; //@line 1302 "voices.cpp"
              if (!($cmp29)) { __label__ = 12;break $for_inc35$$if_end26$17; } //@line 1302 "voices.cpp"
  
              var $32=$vp; //@line 1304 "voices.cpp"
              var $33=$nv; //@line 1304 "voices.cpp"
              var $inc31=((($33)+1)&4294967295); //@line 1304 "voices.cpp"
              $nv=$inc31; //@line 1304 "voices.cpp"
              var $34=$voices_addr; //@line 1304 "voices.cpp"
              var $arrayidx32=(($34+4*$33)&4294967295); //@line 1304 "voices.cpp"
              HEAP32[(($arrayidx32)>>2)]=$32; //@line 1304 "voices.cpp"
              var $35=$score; //@line 1305 "voices.cpp"
              var $36=$vp; //@line 1305 "voices.cpp"
              var $score33=(($36+16)&4294967295); //@line 1305 "voices.cpp"
              HEAP32[(($score33)>>2)]=$35; //@line 1305 "voices.cpp"
              ; //@line 1306 "voices.cpp"
            }
          } while(0);
  
          var $37=$ix; //@line 1295 "voices.cpp"
          var $inc36=((($37)+1)&4294967295); //@line 1295 "voices.cpp"
          $ix=$inc36; //@line 1295 "voices.cpp"
          var $38=$ix; //@line 1295 "voices.cpp"
          var $39=HEAP32[((__ZL13n_voices_list)>>2)]; //@line 1295 "voices.cpp"
          var $cmp18=((($38))|0) < ((($39))|0); //@line 1295 "voices.cpp"
          if ($cmp18) { __label__ = 8;continue $for_body19$13; } else { __label__ = 13;break $for_body19_lr_ph$$for_end37$11; } //@line 1295 "voices.cpp"
        }
      }
    } while(0);
  
    var $40=$nv; //@line 1308 "voices.cpp"
    var $41=$voices_addr; //@line 1308 "voices.cpp"
    var $arrayidx38=(($41+4*$40)&4294967295); //@line 1308 "voices.cpp"
    HEAP32[(($arrayidx38)>>2)]=0; //@line 1308 "voices.cpp"
    var $42=$nv; //@line 1310 "voices.cpp"
    var $cmp39=((($42))|0)==0; //@line 1310 "voices.cpp"
    if ($cmp39) { __label__ = 14;; } else { __label__ = 15;; } //@line 1310 "voices.cpp"
    if (__label__ == 14) {
  
      $retval=0; //@line 1311 "voices.cpp"
      ; //@line 1311 "voices.cpp"
    }
    else if (__label__ == 15) {
  
      var $43=$voices_addr; //@line 1314 "voices.cpp"
      var $44=$43; //@line 1314 "voices.cpp"
      var $45=$nv; //@line 1314 "voices.cpp"
      _qsort($44, $45, 4, 8); //@line 1314 "voices.cpp"
      var $46=$nv; //@line 1316 "voices.cpp"
      $retval=$46; //@line 1316 "voices.cpp"
      ; //@line 1316 "voices.cpp"
    }
  
    var $47=$retval; //@line 1317 "voices.cpp"
    STACKTOP = __stackBase__;
    return $47; //@line 1317 "voices.cpp"
    return null;
  }
  

  function __Z14SetVoiceByNamePKc($name) {
    var __stackBase__  = STACKTOP; STACKTOP += 24; _memset(__stackBase__, 0, 24);
    var __label__;
  
    var $retval;
    var $name_addr;
    var $v;
    var $voice_selector=__stackBase__;
    var $variant_name;
    $name_addr=$name;
    var $0=$name_addr; //@line 1673 "voices.cpp"
    __Z8strncpy0PcPKci(((__ZZ14SetVoiceByNamePKcE3buf)&4294967295), $0, 60); //@line 1673 "voices.cpp"
    __ZL23ExtractVoiceVariantNamePci(((__ZZ14SetVoiceByNamePKcE3buf)&4294967295), 0); //@line 1674 "voices.cpp"
    $variant_name=((__ZZL23ExtractVoiceVariantNamePciE12variant_name)&4294967295); //@line 1674 "voices.cpp"
    var $1=$voice_selector; //@line 1676 "voices.cpp"
    var $dest$, $stop$, $stop4$, $fast$, $value4$;
    $dest$ = $1;
    $stop$ = $dest$ + 24;
    $value4$ = 0;
    if ($value4$ < 0) $value4$ += 256;
    $value4$ = $value4$ + ($value4$<<8) + ($value4$<<16) + ($value4$*16777216);
    while ($dest$%4 !== 0 && $dest$ < $stop$) {
      ;; HEAP8[$dest$++] = 0;
    }
    $dest$ >>= 2;
    $stop4$ = $stop$ >> 2;
    while ($dest$ < $stop4$) {
    ;;;;;;;  HEAP32[$dest$++] = $value4$;
    }
    $dest$ <<= 2;
    while ($dest$ < $stop$) {
      ;; HEAP8[$dest$++] = 0;
    }; //@line 1676 "voices.cpp"
    var $2=$name_addr; //@line 1678 "voices.cpp"
    var $name1=(($voice_selector)&4294967295); //@line 1678 "voices.cpp"
    HEAP32[(($name1)>>2)]=$2; //@line 1678 "voices.cpp"
    var $call2=__Z9LoadVoicePKci(((__ZZ14SetVoiceByNamePKcE3buf)&4294967295), 1); //@line 1683 "voices.cpp"
    var $cmp=((($call2))|0)!=0; //@line 1683 "voices.cpp"
    if ($cmp) { __label__ = 1;; } else { __label__ = 4;; } //@line 1683 "voices.cpp"
    $if_then$$if_end6$2: do { 
      if (__label__ == 1) {
  
        var $3=$variant_name; //@line 1685 "voices.cpp"
        var $arrayidx=(($3)&4294967295); //@line 1685 "voices.cpp"
        var $4=HEAP8[($arrayidx)]; //@line 1685 "voices.cpp"
        var $conv=(tempInt=(($4)),(tempInt>=128?tempInt-256:tempInt)); //@line 1685 "voices.cpp"
        var $cmp3=((($conv))|0)!=0; //@line 1685 "voices.cpp"
        if ($cmp3) { __label__ = 2;; } else { __label__ = 3;; } //@line 1685 "voices.cpp"
        if (__label__ == 2) {
  
          var $5=$variant_name; //@line 1687 "voices.cpp"
          var $call5=__Z9LoadVoicePKci($5, 2); //@line 1687 "voices.cpp"
          ; //@line 1688 "voices.cpp"
        }
  
        var $6=HEAP32[((_voice)>>2)]; //@line 1690 "voices.cpp"
        __Z13DoVoiceChangeP7voice_t($6); //@line 1690 "voices.cpp"
        __Z13SetVoiceStackP12espeak_VOICE($voice_selector); //@line 1691 "voices.cpp"
        $retval=0; //@line 1692 "voices.cpp"
        ; //@line 1692 "voices.cpp"
      }
      else if (__label__ == 4) {
  
        var $7=HEAP32[((__ZL13n_voices_list)>>2)]; //@line 1695 "voices.cpp"
        var $cmp7=((($7))|0)==0; //@line 1695 "voices.cpp"
        if ($cmp7) { __label__ = 5;; } else { __label__ = 6;; } //@line 1695 "voices.cpp"
        if (__label__ == 5) {
  
          var $call9=_espeak_ListVoices(0); //@line 1696 "voices.cpp"
          ; //@line 1696 "voices.cpp"
        }
  
        var $call11=__Z17SelectVoiceByNamePP12espeak_VOICEPKc(((__ZL11voices_list)&4294967295), ((__ZZ14SetVoiceByNamePKcE3buf)&4294967295)); //@line 1698 "voices.cpp"
        $v=$call11; //@line 1698 "voices.cpp"
        var $cmp12=((($call11))|0)!=0; //@line 1698 "voices.cpp"
        if ($cmp12) { __label__ = 7;; } else { __label__ = 11;; } //@line 1698 "voices.cpp"
        $if_then13$$if_end24$11: do { 
          if (__label__ == 7) {
  
            var $8=$v; //@line 1700 "voices.cpp"
            var $identifier=(($8+8)&4294967295); //@line 1700 "voices.cpp"
            var $9=HEAP32[(($identifier)>>2)]; //@line 1700 "voices.cpp"
            var $call14=__Z9LoadVoicePKci($9, 0); //@line 1700 "voices.cpp"
            var $cmp15=((($call14))|0)!=0; //@line 1700 "voices.cpp"
            if (!($cmp15)) { __label__ = 11;break $if_then13$$if_end24$11; } //@line 1700 "voices.cpp"
  
            var $10=$variant_name; //@line 1702 "voices.cpp"
            var $arrayidx17=(($10)&4294967295); //@line 1702 "voices.cpp"
            var $11=HEAP8[($arrayidx17)]; //@line 1702 "voices.cpp"
            var $conv18=(tempInt=(($11)),(tempInt>=128?tempInt-256:tempInt)); //@line 1702 "voices.cpp"
            var $cmp19=((($conv18))|0)!=0; //@line 1702 "voices.cpp"
            if ($cmp19) { __label__ = 9;; } else { __label__ = 10;; } //@line 1702 "voices.cpp"
            if (__label__ == 9) {
  
              var $12=$variant_name; //@line 1704 "voices.cpp"
              var $call21=__Z9LoadVoicePKci($12, 2); //@line 1704 "voices.cpp"
              ; //@line 1705 "voices.cpp"
            }
  
            var $13=HEAP32[((_voice)>>2)]; //@line 1706 "voices.cpp"
            __Z13DoVoiceChangeP7voice_t($13); //@line 1706 "voices.cpp"
            __Z13SetVoiceStackP12espeak_VOICE($voice_selector); //@line 1707 "voices.cpp"
            $retval=0; //@line 1708 "voices.cpp"
            __label__ = 12;break $if_then$$if_end6$2; //@line 1708 "voices.cpp"
          }
        } while(0);
  
        $retval=-1; //@line 1711 "voices.cpp"
        ; //@line 1711 "voices.cpp"
      }
    } while(0);
  
    var $14=$retval; //@line 1712 "voices.cpp"
    STACKTOP = __stackBase__;
    return $14; //@line 1712 "voices.cpp"
    return null;
  }
  

  function __Z20SetVoiceByPropertiesP12espeak_VOICE($voice_selector) {
    var __stackBase__  = STACKTOP; STACKTOP += 4; _memset(__stackBase__, 0, 4);
    var __label__;
  
    var $retval;
    var $voice_selector_addr;
    var $voice_id;
    var $voice_found=__stackBase__;
    $voice_selector_addr=$voice_selector;
    var $0=$voice_selector_addr; //@line 1721 "voices.cpp"
    var $call=__Z11SelectVoiceP12espeak_VOICEPi($0, $voice_found); //@line 1721 "voices.cpp"
    $voice_id=$call; //@line 1721 "voices.cpp"
    var $1=HEAP32[(($voice_found)>>2)]; //@line 1723 "voices.cpp"
    var $cmp=((($1))|0)==0; //@line 1723 "voices.cpp"
    if ($cmp) { __label__ = 1;; } else { __label__ = 2;; } //@line 1723 "voices.cpp"
    if (__label__ == 1) {
  
      $retval=2; //@line 1724 "voices.cpp"
      ; //@line 1724 "voices.cpp"
    }
    else if (__label__ == 2) {
  
      var $2=$voice_id; //@line 1726 "voices.cpp"
      var $call1=__Z16LoadVoiceVariantPKci($2, 0); //@line 1726 "voices.cpp"
      var $3=HEAP32[((_voice)>>2)]; //@line 1727 "voices.cpp"
      __Z13DoVoiceChangeP7voice_t($3); //@line 1727 "voices.cpp"
      var $4=$voice_selector_addr; //@line 1728 "voices.cpp"
      __Z13SetVoiceStackP12espeak_VOICE($4); //@line 1728 "voices.cpp"
      $retval=0; //@line 1730 "voices.cpp"
      ; //@line 1730 "voices.cpp"
    }
  
    var $5=$retval; //@line 1731 "voices.cpp"
    STACKTOP = __stackBase__;
    return $5; //@line 1731 "voices.cpp"
    return null;
  }
  

  function __ZL9GetVoicesPKc($path) {
    var __stackBase__  = STACKTOP; STACKTOP += 252; _memset(__stackBase__, 0, 252);
    var __label__;
    var __lastLabel__ = null;
  
    var $path_addr;
    var $f_voice;
    var $voice_data;
    var $ftype;
    var $fname=__stackBase__;
    var $dir;
    var $ent;
    $path_addr=$path;
    var $0=$path_addr; //@line 1625 "voices.cpp"
    var $call=_opendir($0); //@line 1625 "voices.cpp"
    $dir=$call; //@line 1625 "voices.cpp"
    var $cmp=((($call))|0)==0; //@line 1625 "voices.cpp"
    if ($cmp) { __label__ = 13;; } else { __label__ = 1;; } //@line 1625 "voices.cpp"
    if (__label__ == 1) {
  
      var $1=$dir; //@line 1628 "voices.cpp"
      var $call11=_readdir($1); //@line 1628 "voices.cpp"
      $ent=$call11; //@line 1628 "voices.cpp"
      var $cmp22=((($call11))|0)==0; //@line 1628 "voices.cpp"
      var $2=HEAP32[((__ZL13n_voices_list)>>2)]; //@line 1630 "voices.cpp"
      var $cmp33=((($2))|0) >= 148; //@line 1630 "voices.cpp"
      var $or_cond4=($cmp22) | ($cmp33); //@line 1628 "voices.cpp"
      if ($or_cond4) { __label__ = 12;; } else { __label__ = 2;; } //@line 1628 "voices.cpp"
      $while_end$$if_end5_lr_ph$4: do { 
        if (__label__ == 2) {
  
          var $arraydecay=(($fname)&4294967295); //@line 1633 "voices.cpp"
          var $arraydecay8=(($fname)&4294967295); //@line 1635 "voices.cpp"
          var $arraydecay14=(($fname)&4294967295); //@line 1640 "voices.cpp"
          var $arraydecay17=(($fname)&4294967295); //@line 1646 "voices.cpp"
          var $arraydecay22=(($fname)&4294967295); //@line 1650 "voices.cpp"
          ; //@line 1628 "voices.cpp"
          $if_end5$6: while(1) { 
  
            var $3=$path_addr; //@line 1633 "voices.cpp"
            var $4=$ent; //@line 1633 "voices.cpp"
            var $d_name=(($4+4)&4294967295); //@line 1633 "voices.cpp"
            var $arraydecay6=(($d_name)&4294967295); //@line 1633 "voices.cpp"
            var $call7=_sprintf($arraydecay, ((__str74726)&4294967295), allocate([$3,0,0,0,47,0,0,0,$arraydecay6,0,0,0], ["i8*",0,0,0,"i32",0,0,0,"i8*",0,0,0], ALLOC_STACK)); //@line 1633 "voices.cpp"
            var $call9=__Z13GetFileLengthPKc($arraydecay8); //@line 1635 "voices.cpp"
            $ftype=$call9; //@line 1635 "voices.cpp"
            var $cmp10=((($call9))|0)==-2; //@line 1637 "voices.cpp"
            if ($cmp10) { __lastLabel__ = 3; __label__ = 4;; } else { __lastLabel__ = 3; __label__ = 7;; } //@line 1637 "voices.cpp"
            $land_lhs_true$$if_else$8: do { 
              if (__label__ == 4) {
  
                var $5=$ent; //@line 1637 "voices.cpp"
                var $d_name11=(($5+4)&4294967295); //@line 1637 "voices.cpp"
                var $arrayidx=(($d_name11)&4294967295); //@line 1637 "voices.cpp"
                var $6=HEAP8[($arrayidx)]; //@line 1637 "voices.cpp"
                var $conv=(tempInt=(($6)),(tempInt>=128?tempInt-256:tempInt)); //@line 1637 "voices.cpp"
                var $cmp12=((($conv))|0)!=46; //@line 1637 "voices.cpp"
                if ($cmp12) { __label__ = 5;; } else { __label__ = 6;; } //@line 1637 "voices.cpp"
                if (__label__ == 5) {
  
                  __ZL9GetVoicesPKc($arraydecay14); //@line 1640 "voices.cpp"
                  __label__ = 9;break $land_lhs_true$$if_else$8; //@line 1641 "voices.cpp"
                }
                else if (__label__ == 6) {
  
                  var $_pr=$ftype; //@line 1643 "voices.cpp"
                  __lastLabel__ = 6; __label__ = 7;break $land_lhs_true$$if_else$8;
                }
              }
            } while(0);
            $while_cond_backedge$$if_else$13: do { 
              if (__label__ == 7) {
  
                var $7=__lastLabel__ == 6 ? $_pr : ($call9); //@line 1643 "voices.cpp"
                var $cmp15=((($7))|0) > 0; //@line 1643 "voices.cpp"
                if (!($cmp15)) { __label__ = 9;break $while_cond_backedge$$if_else$13; } //@line 1643 "voices.cpp"
  
                var $call18=_fopen($arraydecay17, ((__str51703)&4294967295)); //@line 1646 "voices.cpp"
                $f_voice=$call18; //@line 1646 "voices.cpp"
                var $cmp19=((($call18))|0)==0; //@line 1646 "voices.cpp"
                if ($cmp19) { __label__ = 9;break $while_cond_backedge$$if_else$13; } //@line 1646 "voices.cpp"
  
                var $10=$f_voice; //@line 1650 "voices.cpp"
                var $11=HEAP32[((__ZL15len_path_voices)>>2)]; //@line 1650 "voices.cpp"
                var $add_ptr=(($arraydecay22+$11)&4294967295); //@line 1650 "voices.cpp"
                var $12=$ent; //@line 1650 "voices.cpp"
                var $d_name23=(($12+4)&4294967295); //@line 1650 "voices.cpp"
                var $arraydecay24=(($d_name23)&4294967295); //@line 1650 "voices.cpp"
                var $call25=__ZL13ReadVoiceFileP7__sFILEPKcS2_($10, $add_ptr, $arraydecay24); //@line 1650 "voices.cpp"
                $voice_data=$call25; //@line 1650 "voices.cpp"
                var $13=$f_voice; //@line 1651 "voices.cpp"
                var $call26=_fclose($13); //@line 1651 "voices.cpp"
                var $14=$voice_data; //@line 1653 "voices.cpp"
                var $cmp27=((($14))|0)!=0; //@line 1653 "voices.cpp"
                if (!($cmp27)) { __label__ = 9;break $while_cond_backedge$$if_else$13; } //@line 1653 "voices.cpp"
  
                var $15=$voice_data; //@line 1655 "voices.cpp"
                var $16=HEAP32[((__ZL13n_voices_list)>>2)]; //@line 1655 "voices.cpp"
                var $inc=((($16)+1)&4294967295); //@line 1655 "voices.cpp"
                HEAP32[((__ZL13n_voices_list)>>2)]=$inc; //@line 1655 "voices.cpp"
                var $arrayidx29=((__ZL11voices_list+$16*4)&4294967295); //@line 1655 "voices.cpp"
                HEAP32[(($arrayidx29)>>2)]=$15; //@line 1655 "voices.cpp"
                ; //@line 1656 "voices.cpp"
              }
            } while(0);
  
            var $8=$dir; //@line 1628 "voices.cpp"
            var $call1=_readdir($8); //@line 1628 "voices.cpp"
            $ent=$call1; //@line 1628 "voices.cpp"
            var $cmp2=((($call1))|0)==0; //@line 1628 "voices.cpp"
            var $9=HEAP32[((__ZL13n_voices_list)>>2)]; //@line 1630 "voices.cpp"
            var $cmp3=((($9))|0) >= 148; //@line 1630 "voices.cpp"
            var $or_cond=($cmp2) | ($cmp3); //@line 1628 "voices.cpp"
            if ($or_cond) { __label__ = 12;break $while_end$$if_end5_lr_ph$4; } else { __label__ = 3;continue $if_end5$6; } //@line 1628 "voices.cpp"
          }
        }
      } while(0);
  
      var $17=$dir; //@line 1659 "voices.cpp"
      var $call33=_closedir($17); //@line 1659 "voices.cpp"
      ; //@line 1662 "voices.cpp"
    }
  
    STACKTOP = __stackBase__;
    return; //@line 1662 "voices.cpp"
    return;
  }
  

  function __ZL15VoiceNameSorterPKvS0_($p1, $p2) {
    ;
    var __label__;
  
    var $retval;
    var $p1_addr;
    var $p2_addr;
    var $ix;
    var $v1;
    var $v2;
    $p1_addr=$p1;
    $p2_addr=$p2;
    var $0=$p1_addr; //@line 1103 "voices.cpp"
    var $1=$0; //@line 1103 "voices.cpp"
    var $2=HEAP32[(($1)>>2)]; //@line 1103 "voices.cpp"
    $v1=$2; //@line 1103 "voices.cpp"
    var $3=$p2_addr; //@line 1104 "voices.cpp"
    var $4=$3; //@line 1104 "voices.cpp"
    var $5=HEAP32[(($4)>>2)]; //@line 1104 "voices.cpp"
    $v2=$5; //@line 1104 "voices.cpp"
    var $6=$v1; //@line 1107 "voices.cpp"
    var $languages=(($6+4)&4294967295); //@line 1107 "voices.cpp"
    var $7=HEAP32[(($languages)>>2)]; //@line 1107 "voices.cpp"
    var $arrayidx=(($7+1)&4294967295); //@line 1107 "voices.cpp"
    var $8=$v2; //@line 1107 "voices.cpp"
    var $languages1=(($8+4)&4294967295); //@line 1107 "voices.cpp"
    var $9=HEAP32[(($languages1)>>2)]; //@line 1107 "voices.cpp"
    var $arrayidx2=(($9+1)&4294967295); //@line 1107 "voices.cpp"
    var $call=_strcmp($arrayidx, $arrayidx2); //@line 1107 "voices.cpp"
    $ix=$call; //@line 1107 "voices.cpp"
    var $cmp=((($call))|0)!=0; //@line 1107 "voices.cpp"
    if ($cmp) { __label__ = 1;; } else { __label__ = 2;; } //@line 1107 "voices.cpp"
    if (__label__ == 1) {
  
      var $10=$ix; //@line 1108 "voices.cpp"
      $retval=$10; //@line 1108 "voices.cpp"
      ; //@line 1108 "voices.cpp"
    }
    else if (__label__ == 2) {
  
      var $11=$v1; //@line 1109 "voices.cpp"
      var $languages3=(($11+4)&4294967295); //@line 1109 "voices.cpp"
      var $12=HEAP32[(($languages3)>>2)]; //@line 1109 "voices.cpp"
      var $arrayidx4=(($12)&4294967295); //@line 1109 "voices.cpp"
      var $13=HEAP8[($arrayidx4)]; //@line 1109 "voices.cpp"
      var $conv=(tempInt=(($13)),(tempInt>=128?tempInt-256:tempInt)); //@line 1109 "voices.cpp"
      var $14=$v2; //@line 1109 "voices.cpp"
      var $languages5=(($14+4)&4294967295); //@line 1109 "voices.cpp"
      var $15=HEAP32[(($languages5)>>2)]; //@line 1109 "voices.cpp"
      var $arrayidx6=(($15)&4294967295); //@line 1109 "voices.cpp"
      var $16=HEAP8[($arrayidx6)]; //@line 1109 "voices.cpp"
      var $conv7=(tempInt=(($16)),(tempInt>=128?tempInt-256:tempInt)); //@line 1109 "voices.cpp"
      var $sub=((($conv)-($conv7))&4294967295); //@line 1109 "voices.cpp"
      $ix=$sub; //@line 1109 "voices.cpp"
      var $cmp8=((($sub))|0)!=0; //@line 1109 "voices.cpp"
      if ($cmp8) { __label__ = 3;; } else { __label__ = 4;; } //@line 1109 "voices.cpp"
      if (__label__ == 3) {
  
        var $17=$ix; //@line 1110 "voices.cpp"
        $retval=$17; //@line 1110 "voices.cpp"
        ; //@line 1110 "voices.cpp"
      }
      else if (__label__ == 4) {
  
        var $18=$v1; //@line 1111 "voices.cpp"
        var $name=(($18)&4294967295); //@line 1111 "voices.cpp"
        var $19=HEAP32[(($name)>>2)]; //@line 1111 "voices.cpp"
        var $20=$v2; //@line 1111 "voices.cpp"
        var $name11=(($20)&4294967295); //@line 1111 "voices.cpp"
        var $21=HEAP32[(($name11)>>2)]; //@line 1111 "voices.cpp"
        var $call12=_strcmp($19, $21); //@line 1111 "voices.cpp"
        $retval=$call12; //@line 1111 "voices.cpp"
        ; //@line 1111 "voices.cpp"
      }
    }
  
    var $22=$retval; //@line 1112 "voices.cpp"
    ;
    return $22; //@line 1112 "voices.cpp"
    return null;
  }
  

  function _espeak_GetCurrentVoice() {
    ;
    var __label__;
  
    ;
    return _voice_selected; //@line 1798 "voices.cpp"
    return null;
  }
  

  function __ZL13ReadVoiceFileP7__sFILEPKcS2_($f_in, $fname, $leafname) {
    var __stackBase__  = STACKTOP; STACKTOP += 672; _memset(__stackBase__, 0, 672);
    var __label__;
  
    var $retval;
    var $f_in_addr;
    var $fname_addr;
    var $leafname_addr;
    var $linebuf=__stackBase__;
    var $vname=__stackBase__+120;
    var $vgender=__stackBase__+200;
    var $vlanguage=__stackBase__+280;
    var $languages=__stackBase__+360;
    var $len;
    var $langix;
    var $n_languages;
    var $p;
    var $voice_data;
    var $priority=__stackBase__+660;
    var $age=__stackBase__+664;
    var $n_variants=__stackBase__+668;
    var $gender;
    $f_in_addr=$f_in;
    $fname_addr=$fname;
    $leafname_addr=$leafname;
    $langix=0; //@line 281 "voices.cpp"
    $n_languages=0; //@line 282 "voices.cpp"
    HEAP32[(($n_variants)>>2)]=3; //@line 287 "voices.cpp"
    var $arrayidx=(($vname)&4294967295); //@line 304 "voices.cpp"
    HEAP8[($arrayidx)]=0; //@line 304 "voices.cpp"
    var $arrayidx1=(($vgender)&4294967295); //@line 305 "voices.cpp"
    HEAP8[($arrayidx1)]=0; //@line 305 "voices.cpp"
    HEAP32[(($age)>>2)]=0; //@line 306 "voices.cpp"
    var $arraydecay=(($linebuf)&4294967295); //@line 308 "voices.cpp"
    var $0=$f_in_addr; //@line 308 "voices.cpp"
    var $call4=__ZL11fgets_stripPciP7__sFILE($arraydecay, 120, $0); //@line 308 "voices.cpp"
    var $cmp5=((($call4))|0)!=0; //@line 308 "voices.cpp"
    if ($cmp5) { __label__ = 1;; } else { __label__ = 14;; } //@line 308 "voices.cpp"
    $while_body_lr_ph$$while_end49$2: do { 
      if (__label__ == 1) {
  
        var $arraydecay2=(($linebuf)&4294967295); //@line 310 "voices.cpp"
        var $arrayidx5=(($linebuf+4)&4294967295); //@line 312 "voices.cpp"
        var $arraydecay9=(($vname)&4294967295); //@line 314 "voices.cpp"
        var $arraydecay10=(($linebuf)&4294967295); //@line 317 "voices.cpp"
        var $arrayidx14=(($vlanguage)&4294967295); //@line 320 "voices.cpp"
        var $arrayidx15=(($linebuf+8)&4294967295); //@line 322 "voices.cpp"
        var $arraydecay16=(($vlanguage)&4294967295); //@line 322 "voices.cpp"
        var $arraydecay18=(($vlanguage)&4294967295); //@line 323 "voices.cpp"
        var $arraydecay27=(($vlanguage)&4294967295); //@line 329 "voices.cpp"
        var $arraydecay31=(($linebuf)&4294967295); //@line 335 "voices.cpp"
        var $arrayidx35=(($linebuf+6)&4294967295); //@line 337 "voices.cpp"
        var $arraydecay36=(($vgender)&4294967295); //@line 337 "voices.cpp"
        var $arraydecay39=(($linebuf)&4294967295); //@line 340 "voices.cpp"
        var $arrayidx43=(($linebuf+8)&4294967295); //@line 342 "voices.cpp"
        ; //@line 308 "voices.cpp"
        $while_body$4: while(1) { 
  
          var $call3=_memcmp($arraydecay2, ((__str5655)&4294967295), 4); //@line 310 "voices.cpp"
          var $cmp4=((($call3))|0)==0; //@line 310 "voices.cpp"
          if ($cmp4) { __label__ = 3;; } else { __label__ = 6;; } //@line 310 "voices.cpp"
          $if_then$$if_else$6: do { 
            if (__label__ == 3) {
  
              $p=$arrayidx5; //@line 312 "voices.cpp"
              var $1=$p; //@line 313 "voices.cpp"
              var $2=HEAP8[($1)]; //@line 313 "voices.cpp"
              var $conv1=(tempInt=(($2)),(tempInt>=128?tempInt-256:tempInt)); //@line 313 "voices.cpp"
              var $call72=_isspace($conv1); //@line 313 "voices.cpp"
              var $tobool3=((($call72))|0)!=0; //@line 313 "voices.cpp"
              if ($tobool3) { __label__ = 4;; } else { __label__ = 5;; } //@line 313 "voices.cpp"
              $while_body8$$while_end$8: do { 
                if (__label__ == 4) {
                  while(1) { 
  
                    var $3=$p; //@line 313 "voices.cpp"
                    var $incdec_ptr=(($3+1)&4294967295); //@line 313 "voices.cpp"
                    $p=$incdec_ptr; //@line 313 "voices.cpp"
                    var $4=$p; //@line 313 "voices.cpp"
                    var $5=HEAP8[($4)]; //@line 313 "voices.cpp"
                    var $conv=(tempInt=(($5)),(tempInt>=128?tempInt-256:tempInt)); //@line 313 "voices.cpp"
                    var $call7=_isspace($conv); //@line 313 "voices.cpp"
                    var $tobool=((($call7))|0)!=0; //@line 313 "voices.cpp"
                    if ($tobool) { __label__ = 4;continue ; } else { __label__ = 5;break $while_body8$$while_end$8; } //@line 313 "voices.cpp"
                  }
                }
              } while(0);
  
              var $6=$p; //@line 314 "voices.cpp"
              __Z8strncpy0PcPKci($arraydecay9, $6, 80); //@line 314 "voices.cpp"
              ; //@line 315 "voices.cpp"
            }
            else if (__label__ == 6) {
  
              var $call11=_memcmp($arraydecay10, ((__str6656)&4294967295), 8); //@line 317 "voices.cpp"
              var $cmp12=((($call11))|0)==0; //@line 317 "voices.cpp"
              if ($cmp12) { __label__ = 7;; } else { __label__ = 10;; } //@line 317 "voices.cpp"
              if (__label__ == 7) {
  
                HEAP32[(($priority)>>2)]=5; //@line 319 "voices.cpp"
                HEAP8[($arrayidx14)]=0; //@line 320 "voices.cpp"
                var $call17=_sscanf($arrayidx15, ((__str54706)&4294967295), allocate([$arraydecay16,0,0,0,$priority,0,0,0], ["i8*",0,0,0,"i32*",0,0,0], ALLOC_STACK)); //@line 322 "voices.cpp"
                var $call19=_strlen($arraydecay18); //@line 323 "voices.cpp"
                var $add=((($call19)+2)&4294967295); //@line 323 "voices.cpp"
                $len=$add; //@line 323 "voices.cpp"
                var $7=$len; //@line 325 "voices.cpp"
                var $8=$langix; //@line 325 "voices.cpp"
                var $_neg=(((-($8)))&4294967295);
                var $sub20=((($_neg)+299)&4294967295); //@line 325 "voices.cpp"
                var $cmp21=((($7))>>>0) < ((($sub20))>>>0); //@line 325 "voices.cpp"
                if (!($cmp21)) { __label__ = 9;break $if_then$$if_else$6; } //@line 325 "voices.cpp"
  
                var $9=HEAP32[(($priority)>>2)]; //@line 327 "voices.cpp"
                var $conv23=((($9)) & 255); //@line 327 "voices.cpp"
                var $10=$langix; //@line 327 "voices.cpp"
                var $arrayidx24=(($languages+$10)&4294967295); //@line 327 "voices.cpp"
                HEAP8[($arrayidx24)]=$conv23; //@line 327 "voices.cpp"
                var $11=$langix; //@line 329 "voices.cpp"
                var $add25=((($11)+1)&4294967295); //@line 329 "voices.cpp"
                var $arrayidx26=(($languages+$add25)&4294967295); //@line 329 "voices.cpp"
                var $call28=_strcpy($arrayidx26, $arraydecay27); //@line 329 "voices.cpp"
                var $12=$len; //@line 330 "voices.cpp"
                var $13=$langix; //@line 330 "voices.cpp"
                var $add29=((($13)+($12))&4294967295); //@line 330 "voices.cpp"
                $langix=$add29; //@line 330 "voices.cpp"
                var $14=$n_languages; //@line 331 "voices.cpp"
                var $inc=((($14)+1)&4294967295); //@line 331 "voices.cpp"
                $n_languages=$inc; //@line 331 "voices.cpp"
                ; //@line 332 "voices.cpp"
              }
              else if (__label__ == 10) {
  
                var $call32=_memcmp($arraydecay31, ((__str7657)&4294967295), 6); //@line 335 "voices.cpp"
                var $cmp33=((($call32))|0)==0; //@line 335 "voices.cpp"
                if ($cmp33) { __label__ = 11;; } else { __label__ = 12;; } //@line 335 "voices.cpp"
                if (__label__ == 11) {
  
                  var $call37=_sscanf($arrayidx35, ((__str54706)&4294967295), allocate([$arraydecay36,0,0,0,$age,0,0,0], ["i8*",0,0,0,"i32*",0,0,0], ALLOC_STACK)); //@line 337 "voices.cpp"
                  ; //@line 338 "voices.cpp"
                }
                else if (__label__ == 12) {
  
                  var $call40=_memcmp($arraydecay39, ((__str75727)&4294967295), 8); //@line 340 "voices.cpp"
                  var $cmp41=((($call40))|0)==0; //@line 340 "voices.cpp"
                  if (!($cmp41)) { __label__ = 9;break $if_then$$if_else$6; } //@line 340 "voices.cpp"
  
                  var $call44=_sscanf($arrayidx43, ((__str63715)&4294967295), allocate([$n_variants,0,0,0], ["i32*",0,0,0], ALLOC_STACK)); //@line 342 "voices.cpp"
                  ; //@line 343 "voices.cpp"
                }
              }
            }
          } while(0);
  
          var $15=$f_in_addr; //@line 308 "voices.cpp"
          var $call=__ZL11fgets_stripPciP7__sFILE($arraydecay, 120, $15); //@line 308 "voices.cpp"
          var $cmp=((($call))|0)!=0; //@line 308 "voices.cpp"
          if ($cmp) { __label__ = 2;continue $while_body$4; } else { __label__ = 14;break $while_body_lr_ph$$while_end49$2; } //@line 308 "voices.cpp"
        }
      }
    } while(0);
  
    var $16=$langix; //@line 345 "voices.cpp"
    var $inc50=((($16)+1)&4294967295); //@line 345 "voices.cpp"
    $langix=$inc50; //@line 345 "voices.cpp"
    var $arrayidx51=(($languages+$16)&4294967295); //@line 345 "voices.cpp"
    HEAP8[($arrayidx51)]=0; //@line 345 "voices.cpp"
    var $arraydecay52=(($vgender)&4294967295); //@line 347 "voices.cpp"
    var $call53=__Z10LookupMnemP8MNEM_TABPKc(((_genders)&4294967295), $arraydecay52); //@line 347 "voices.cpp"
    $gender=$call53; //@line 347 "voices.cpp"
    var $17=$n_languages; //@line 349 "voices.cpp"
    var $cmp54=((($17))|0)==0; //@line 349 "voices.cpp"
    if ($cmp54) { __label__ = 15;; } else { __label__ = 16;; } //@line 349 "voices.cpp"
    if (__label__ == 15) {
  
      $retval=0; //@line 351 "voices.cpp"
      ; //@line 351 "voices.cpp"
    }
    else if (__label__ == 16) {
  
      var $18=$langix; //@line 354 "voices.cpp"
      var $19=$fname_addr; //@line 354 "voices.cpp"
      var $call58=_strlen($19); //@line 354 "voices.cpp"
      var $arraydecay60=(($vname)&4294967295); //@line 354 "voices.cpp"
      var $call61=_strlen($arraydecay60); //@line 354 "voices.cpp"
      var $add59=((($18)+27)&4294967295); //@line 354 "voices.cpp"
      var $add62=((($add59)+($call58))&4294967295); //@line 354 "voices.cpp"
      var $add63=((($add62)+($call61))&4294967295); //@line 354 "voices.cpp"
      var $call64=_calloc($add63, 1); //@line 354 "voices.cpp"
      $p=$call64; //@line 354 "voices.cpp"
      var $20=$p; //@line 355 "voices.cpp"
      var $21=$20; //@line 355 "voices.cpp"
      $voice_data=$21; //@line 355 "voices.cpp"
      var $22=$p; //@line 356 "voices.cpp"
      var $arrayidx65=(($22+24)&4294967295); //@line 356 "voices.cpp"
      $p=$arrayidx65; //@line 356 "voices.cpp"
      var $23=$p; //@line 358 "voices.cpp"
      var $arraydecay66=(($languages)&4294967295); //@line 358 "voices.cpp"
      var $24=$langix; //@line 358 "voices.cpp"
      var $src$, $dest$, $stop$, $stop4$, $fast$;
      $src$ = $arraydecay66;
      $dest$ = $23;
      $stop$ = $src$ + $24;
      if (($dest$%4) == ($src$%4) && $24 > 8) {
        while ($src$%4 !== 0 && $src$ < $stop$) {
          ; HEAP8[$dest$++] = HEAP8[$src$++];
        }
        $src$ >>= 2;
        $dest$ >>= 2;
        $stop4$ = $stop$ >> 2;
        while ($src$ < $stop4$) {
      ;;;    HEAP32[$dest$++] = HEAP32[$src$++];
        }
        $src$ <<= 2;
        $dest$ <<= 2;
      }while ($src$ < $stop$) {
        ; HEAP8[$dest$++] = HEAP8[$src$++];
      }; //@line 358 "voices.cpp"
      var $25=$p; //@line 359 "voices.cpp"
      var $26=$voice_data; //@line 359 "voices.cpp"
      var $languages67=(($26+4)&4294967295); //@line 359 "voices.cpp"
      HEAP32[(($languages67)>>2)]=$25; //@line 359 "voices.cpp"
      var $27=$langix; //@line 361 "voices.cpp"
      var $28=$p; //@line 361 "voices.cpp"
      var $arrayidx68=(($28+$27)&4294967295); //@line 361 "voices.cpp"
      var $29=$fname_addr; //@line 361 "voices.cpp"
      var $call69=_strcpy($arrayidx68, $29); //@line 361 "voices.cpp"
      var $30=$langix; //@line 362 "voices.cpp"
      var $31=$p; //@line 362 "voices.cpp"
      var $arrayidx70=(($31+$30)&4294967295); //@line 362 "voices.cpp"
      var $32=$voice_data; //@line 362 "voices.cpp"
      var $identifier=(($32+8)&4294967295); //@line 362 "voices.cpp"
      HEAP32[(($identifier)>>2)]=$arrayidx70; //@line 362 "voices.cpp"
      var $33=$langix; //@line 363 "voices.cpp"
      var $34=$p; //@line 363 "voices.cpp"
      var $arrayidx71=(($34+$33)&4294967295); //@line 363 "voices.cpp"
      var $35=$voice_data; //@line 363 "voices.cpp"
      var $name=(($35)&4294967295); //@line 363 "voices.cpp"
      HEAP32[(($name)>>2)]=$arrayidx71; //@line 363 "voices.cpp"
      var $arrayidx72=(($vname)&4294967295); //@line 365 "voices.cpp"
      var $36=HEAP8[($arrayidx72)]; //@line 365 "voices.cpp"
      var $conv73=(tempInt=(($36)),(tempInt>=128?tempInt-256:tempInt)); //@line 365 "voices.cpp"
      var $cmp74=((($conv73))|0)!=0; //@line 365 "voices.cpp"
      if ($cmp74) { __label__ = 17;; } else { __label__ = 18;; } //@line 365 "voices.cpp"
      if (__label__ == 17) {
  
        var $37=$fname_addr; //@line 367 "voices.cpp"
        var $call76=_strlen($37); //@line 367 "voices.cpp"
        var $38=$langix; //@line 367 "voices.cpp"
        var $add77=((($call76)+1)&4294967295); //@line 367 "voices.cpp"
        var $add78=((($add77)+($38))&4294967295); //@line 367 "voices.cpp"
        $langix=$add78; //@line 367 "voices.cpp"
        var $39=$langix; //@line 368 "voices.cpp"
        var $40=$p; //@line 368 "voices.cpp"
        var $arrayidx79=(($40+$39)&4294967295); //@line 368 "voices.cpp"
        var $arraydecay80=(($vname)&4294967295); //@line 368 "voices.cpp"
        var $call81=_strcpy($arrayidx79, $arraydecay80); //@line 368 "voices.cpp"
        var $41=$langix; //@line 369 "voices.cpp"
        var $42=$p; //@line 369 "voices.cpp"
        var $arrayidx82=(($42+$41)&4294967295); //@line 369 "voices.cpp"
        var $43=$voice_data; //@line 369 "voices.cpp"
        var $name83=(($43)&4294967295); //@line 369 "voices.cpp"
        HEAP32[(($name83)>>2)]=$arrayidx82; //@line 369 "voices.cpp"
        ; //@line 370 "voices.cpp"
      }
  
      var $44=HEAP32[(($age)>>2)]; //@line 372 "voices.cpp"
      var $conv85=((($44)) & 255); //@line 372 "voices.cpp"
      var $45=$voice_data; //@line 372 "voices.cpp"
      var $age86=(($45+13)&4294967295); //@line 372 "voices.cpp"
      HEAP8[($age86)]=$conv85; //@line 372 "voices.cpp"
      var $46=$gender; //@line 373 "voices.cpp"
      var $conv87=((($46)) & 255); //@line 373 "voices.cpp"
      var $47=$voice_data; //@line 373 "voices.cpp"
      var $gender88=(($47+12)&4294967295); //@line 373 "voices.cpp"
      HEAP8[($gender88)]=$conv87; //@line 373 "voices.cpp"
      var $48=$voice_data; //@line 374 "voices.cpp"
      var $variant=(($48+14)&4294967295); //@line 374 "voices.cpp"
      HEAP8[($variant)]=0; //@line 374 "voices.cpp"
      var $49=HEAP32[(($n_variants)>>2)]; //@line 375 "voices.cpp"
      var $conv89=((($49)) & 255); //@line 375 "voices.cpp"
      var $50=$voice_data; //@line 375 "voices.cpp"
      var $xx1=(($50+15)&4294967295); //@line 375 "voices.cpp"
      HEAP8[($xx1)]=$conv89; //@line 375 "voices.cpp"
      var $51=$voice_data; //@line 376 "voices.cpp"
      $retval=$51; //@line 376 "voices.cpp"
      ; //@line 376 "voices.cpp"
    }
  
    var $52=$retval; //@line 377 "voices.cpp"
    STACKTOP = __stackBase__;
    return $52; //@line 377 "voices.cpp"
    return null;
  }
  

  function __ZL10ScoreVoiceP12espeak_VOICEPKciiS0_($voice_spec, $spec_language, $spec_n_parts, $spec_lang_len, $voice) {
    ;
    var __label__;
    var __lastLabel__ = null;
  
    var $retval;
    var $voice_spec_addr;
    var $spec_language_addr;
    var $spec_n_parts_addr;
    var $spec_lang_len_addr;
    var $voice_addr;
    var $ix;
    var $p;
    var $c1;
    var $c2;
    var $language_priority;
    var $n_parts;
    var $matching;
    var $matching_parts;
    var $score;
    var $x;
    var $ratio;
    var $required_age;
    var $diff;
    $voice_spec_addr=$voice_spec;
    $spec_language_addr=$spec_language;
    $spec_n_parts_addr=$spec_n_parts;
    $spec_lang_len_addr=$spec_lang_len;
    $voice_addr=$voice;
    $score=0; //@line 1136 "voices.cpp"
    var $0=$voice_addr; //@line 1142 "voices.cpp"
    var $languages=(($0+4)&4294967295); //@line 1142 "voices.cpp"
    var $1=HEAP32[(($languages)>>2)]; //@line 1142 "voices.cpp"
    $p=$1; //@line 1142 "voices.cpp"
    var $2=$spec_language_addr; //@line 1144 "voices.cpp"
    var $call=_strcmp($2, ((__str33683)&4294967295)); //@line 1144 "voices.cpp"
    var $cmp=((($call))|0)==0; //@line 1144 "voices.cpp"
    if ($cmp) { __label__ = 1;; } else { __label__ = 4;; } //@line 1144 "voices.cpp"
    $if_then$$if_end4$2: do { 
      if (__label__ == 1) {
  
        var $3=$voice_addr; //@line 1147 "voices.cpp"
        var $identifier=(($3+8)&4294967295); //@line 1147 "voices.cpp"
        var $4=HEAP32[(($identifier)>>2)]; //@line 1147 "voices.cpp"
        var $call1=_memcmp($4, ((__str73725)&4294967295), 3); //@line 1147 "voices.cpp"
        var $cmp2=((($call1))|0)==0; //@line 1147 "voices.cpp"
        if ($cmp2) { __label__ = 2;; } else { __label__ = 3;; } //@line 1147 "voices.cpp"
        if (__label__ == 2) {
  
          $retval=100; //@line 1148 "voices.cpp"
          ; //@line 1148 "voices.cpp"
        }
        else if (__label__ == 3) {
  
          $retval=0; //@line 1149 "voices.cpp"
          ; //@line 1149 "voices.cpp"
        }
      }
      else if (__label__ == 4) {
  
        var $5=$spec_n_parts_addr; //@line 1152 "voices.cpp"
        var $cmp5=((($5))|0)==0; //@line 1152 "voices.cpp"
        if ($cmp5) { __label__ = 5;; } else { __label__ = 6;; } //@line 1152 "voices.cpp"
        $if_end60_thread$$if_else$8: do { 
          if (__label__ == 5) {
  
            $score=100; //@line 1154 "voices.cpp"
            ; //@line 1215 "voices.cpp"
          }
          else if (__label__ == 6) {
  
            var $6=$p; //@line 1158 "voices.cpp"
            var $7=HEAP8[($6)]; //@line 1158 "voices.cpp"
            var $conv=(tempInt=(($7)),(tempInt>=128?tempInt-256:tempInt)); //@line 1158 "voices.cpp"
            var $cmp7=((($conv))|0)==0; //@line 1158 "voices.cpp"
            if ($cmp7) { __label__ = 7;; } else { __label__ = 9;; } //@line 1158 "voices.cpp"
            $land_lhs_true$$while_cond_preheader$11: do { 
              if (__label__ == 7) {
  
                var $8=$spec_language_addr; //@line 1158 "voices.cpp"
                var $call8=_strcmp($8, ((__str75727)&4294967295)); //@line 1158 "voices.cpp"
                var $cmp9=((($call8))|0)==0; //@line 1158 "voices.cpp"
                if (!($cmp9)) { __label__ = 9;break $land_lhs_true$$while_cond_preheader$11; } //@line 1158 "voices.cpp"
  
                $score=100; //@line 1161 "voices.cpp"
                ; //@line 1162 "voices.cpp"
              }
            } while(0);
  
            var $9=$p; //@line 1165 "voices.cpp"
            var $10=HEAP8[($9)]; //@line 1165 "voices.cpp"
            var $conv123=(tempInt=(($10)),(tempInt>=128?tempInt-256:tempInt)); //@line 1165 "voices.cpp"
            var $cmp134=((($conv123))|0)!=0; //@line 1165 "voices.cpp"
            if ($cmp134) { __label__ = 10;; } else { __label__ = 31;; } //@line 1165 "voices.cpp"
            $while_body$$if_end60$15: do { 
              if (__label__ == 10) {
                $while_body$16: while(1) { 
  
                  var $11=$p; //@line 1167 "voices.cpp"
                  var $incdec_ptr=(($11+1)&4294967295); //@line 1167 "voices.cpp"
                  $p=$incdec_ptr; //@line 1167 "voices.cpp"
                  var $12=HEAP8[($11)]; //@line 1167 "voices.cpp"
                  var $conv14=(tempInt=(($12)),(tempInt>=128?tempInt-256:tempInt)); //@line 1167 "voices.cpp"
                  $language_priority=$conv14; //@line 1167 "voices.cpp"
                  $matching=1; //@line 1169 "voices.cpp"
                  $matching_parts=0; //@line 1170 "voices.cpp"
                  $n_parts=1; //@line 1171 "voices.cpp"
                  $ix=0; //@line 1173 "voices.cpp"
                  ; //@line 1173 "voices.cpp"
                  $for_cond$18: while(1) { 
  
                    var $13=$ix; //@line 1175 "voices.cpp"
                    var $14=$spec_lang_len_addr; //@line 1175 "voices.cpp"
                    var $cmp15=((($13))|0) >= ((($14))|0); //@line 1175 "voices.cpp"
                    if ($cmp15) { __label__ = 13;; } else { __label__ = 12;; } //@line 1175 "voices.cpp"
                    $if_then18$$lor_lhs_false$20: do { 
                      if (__label__ == 12) {
  
                        var $15=$ix; //@line 1175 "voices.cpp"
                        var $16=$spec_language_addr; //@line 1175 "voices.cpp"
                        var $arrayidx=(($16+$15)&4294967295); //@line 1175 "voices.cpp"
                        var $17=HEAP8[($arrayidx)]; //@line 1175 "voices.cpp"
                        var $conv16=(tempInt=(($17)),(tempInt>=128?tempInt-256:tempInt)); //@line 1175 "voices.cpp"
                        $c1=$conv16; //@line 1175 "voices.cpp"
                        var $cmp17=((($conv16))|0)==45; //@line 1175 "voices.cpp"
                        if ($cmp17) { __label__ = 13;break $if_then18$$lor_lhs_false$20; } else { __label__ = 14;break $if_then18$$lor_lhs_false$20; } //@line 1175 "voices.cpp"
                      }
                    } while(0);
                    if (__label__ == 13) {
  
                      $c1=0; //@line 1176 "voices.cpp"
                      ; //@line 1176 "voices.cpp"
                    }
  
                    var $18=$ix; //@line 1177 "voices.cpp"
                    var $19=$p; //@line 1177 "voices.cpp"
                    var $arrayidx20=(($19+$18)&4294967295); //@line 1177 "voices.cpp"
                    var $20=HEAP8[($arrayidx20)]; //@line 1177 "voices.cpp"
                    var $conv21=(tempInt=(($20)),(tempInt>=128?tempInt-256:tempInt)); //@line 1177 "voices.cpp"
                    $c2=$conv21; //@line 1177 "voices.cpp"
                    var $cmp22=((($conv21))|0)==45; //@line 1177 "voices.cpp"
                    if ($cmp22) { __label__ = 15;; } else { __label__ = 16;; } //@line 1177 "voices.cpp"
                    if (__label__ == 15) {
  
                      $c2=0; //@line 1178 "voices.cpp"
                      ; //@line 1178 "voices.cpp"
                    }
  
                    var $21=$c1; //@line 1180 "voices.cpp"
                    var $22=$c2; //@line 1180 "voices.cpp"
                    var $cmp25=((($21))|0)!=((($22))|0); //@line 1180 "voices.cpp"
                    if ($cmp25) { __label__ = 17;; } else { __label__ = 18;; } //@line 1180 "voices.cpp"
                    if (__label__ == 17) {
  
                      $matching=0; //@line 1182 "voices.cpp"
                      ; //@line 1183 "voices.cpp"
                    }
  
                    var $23=$ix; //@line 1185 "voices.cpp"
                    var $24=$p; //@line 1185 "voices.cpp"
                    var $arrayidx28=(($24+$23)&4294967295); //@line 1185 "voices.cpp"
                    var $25=HEAP8[($arrayidx28)]; //@line 1185 "voices.cpp"
                    var $conv29=(tempInt=(($25)),(tempInt>=128?tempInt-256:tempInt)); //@line 1185 "voices.cpp"
                    var $cmp30=((($conv29))|0)==45; //@line 1185 "voices.cpp"
                    if ($cmp30) { __label__ = 19;; } else { __label__ = 21;; } //@line 1185 "voices.cpp"
                    $if_then31$$if_end35$31: do { 
                      if (__label__ == 19) {
  
                        var $26=$n_parts; //@line 1187 "voices.cpp"
                        var $inc=((($26)+1)&4294967295); //@line 1187 "voices.cpp"
                        $n_parts=$inc; //@line 1187 "voices.cpp"
                        var $27=$matching; //@line 1188 "voices.cpp"
                        var $tobool=((($27))|0)!=0; //@line 1188 "voices.cpp"
                        if (!($tobool)) { __label__ = 21;break $if_then31$$if_end35$31; } //@line 1188 "voices.cpp"
  
                        var $28=$matching_parts; //@line 1189 "voices.cpp"
                        var $inc33=((($28)+1)&4294967295); //@line 1189 "voices.cpp"
                        $matching_parts=$inc33; //@line 1189 "voices.cpp"
                        ; //@line 1189 "voices.cpp"
                      }
                    } while(0);
  
                    var $29=$ix; //@line 1191 "voices.cpp"
                    var $30=$p; //@line 1191 "voices.cpp"
                    var $arrayidx36=(($30+$29)&4294967295); //@line 1191 "voices.cpp"
                    var $31=HEAP8[($arrayidx36)]; //@line 1191 "voices.cpp"
                    var $conv37=(tempInt=(($31)),(tempInt>=128?tempInt-256:tempInt)); //@line 1191 "voices.cpp"
                    var $cmp38=((($conv37))|0)==0; //@line 1191 "voices.cpp"
                    var $32=$ix; //@line 1194 "voices.cpp"
                    var $add=((($32)+1)&4294967295); //@line 1194 "voices.cpp"
                    if ($cmp38) { __label__ = 23;break $for_cond$18; } //@line 1191 "voices.cpp"
  
                    $ix=$add; //@line 1173 "voices.cpp"
                    __label__ = 11;continue $for_cond$18; //@line 1173 "voices.cpp"
                  }
  
                  var $33=$p; //@line 1194 "voices.cpp"
                  var $add_ptr=(($33+$add)&4294967295); //@line 1194 "voices.cpp"
                  $p=$add_ptr; //@line 1194 "voices.cpp"
                  var $34=$matching; //@line 1195 "voices.cpp"
                  var $35=$matching_parts; //@line 1195 "voices.cpp"
                  var $add42=((($35)+($34))&4294967295); //@line 1195 "voices.cpp"
                  $matching_parts=$add42; //@line 1195 "voices.cpp"
                  var $36=$matching_parts; //@line 1197 "voices.cpp"
                  var $cmp43=((($36))|0)==0; //@line 1197 "voices.cpp"
                  if ($cmp43) { __label__ = 24;; } else { __label__ = 25;; } //@line 1197 "voices.cpp"
                  $while_cond_backedge$$if_end45$37: do { 
                    if (__label__ == 25) {
  
                      $x=5; //@line 1200 "voices.cpp"
                      var $39=$spec_n_parts_addr; //@line 1202 "voices.cpp"
                      var $40=$matching_parts; //@line 1202 "voices.cpp"
                      var $sub=((($39)-($40))&4294967295); //@line 1202 "voices.cpp"
                      $diff=$sub; //@line 1202 "voices.cpp"
                      var $cmp46=((($sub))|0) > 0; //@line 1202 "voices.cpp"
                      if ($cmp46) { __label__ = 26;; } else { __label__ = 27;; } //@line 1202 "voices.cpp"
                      if (__label__ == 26) {
  
                        var $41=$diff; //@line 1203 "voices.cpp"
                        var $42=$x; //@line 1203 "voices.cpp"
                        var $sub48=((($42)-($41))&4294967295); //@line 1203 "voices.cpp"
                        $x=$sub48; //@line 1203 "voices.cpp"
                        ; //@line 1203 "voices.cpp"
                      }
  
                      var $43=$n_parts; //@line 1206 "voices.cpp"
                      var $44=$matching_parts; //@line 1206 "voices.cpp"
                      var $sub50=((($43)-($44))&4294967295); //@line 1206 "voices.cpp"
                      $diff=$sub50; //@line 1206 "voices.cpp"
                      var $cmp51=((($sub50))|0) > 0; //@line 1206 "voices.cpp"
                      if ($cmp51) { __label__ = 28;; } else { __label__ = 29;; } //@line 1206 "voices.cpp"
                      if (__label__ == 28) {
  
                        var $45=$diff; //@line 1207 "voices.cpp"
                        var $46=$x; //@line 1207 "voices.cpp"
                        var $sub53=((($46)-($45))&4294967295); //@line 1207 "voices.cpp"
                        $x=$sub53; //@line 1207 "voices.cpp"
                        ; //@line 1207 "voices.cpp"
                      }
  
                      var $47=$x; //@line 1209 "voices.cpp"
                      var $mul=((($47)*100)&4294967295); //@line 1209 "voices.cpp"
                      var $48=$language_priority; //@line 1209 "voices.cpp"
                      var $mul55=((($48)*2)&4294967295); //@line 1209 "voices.cpp"
                      var $sub56=((($mul)-($mul55))&4294967295); //@line 1209 "voices.cpp"
                      $x=$sub56; //@line 1209 "voices.cpp"
                      var $49=$x; //@line 1211 "voices.cpp"
                      var $50=$score; //@line 1211 "voices.cpp"
                      var $cmp57=((($49))|0) > ((($50))|0); //@line 1211 "voices.cpp"
                      if (!($cmp57)) { __label__ = 24;break $while_cond_backedge$$if_end45$37; } //@line 1211 "voices.cpp"
  
                      var $51=$x; //@line 1212 "voices.cpp"
                      $score=$51; //@line 1212 "voices.cpp"
                      ; //@line 1212 "voices.cpp"
                    }
                  } while(0);
  
                  var $37=$p; //@line 1165 "voices.cpp"
                  var $38=HEAP8[($37)]; //@line 1165 "voices.cpp"
                  var $conv12=(tempInt=(($38)),(tempInt>=128?tempInt-256:tempInt)); //@line 1165 "voices.cpp"
                  var $cmp13=((($conv12))|0)!=0; //@line 1165 "voices.cpp"
                  if ($cmp13) { __label__ = 10;continue $while_body$16; } else { __label__ = 31;break $while_body$$if_end60$15; } //@line 1165 "voices.cpp"
                }
              }
            } while(0);
  
            var $_pr=$score; //@line 1215 "voices.cpp"
            var $cmp61=((($_pr))|0)==0; //@line 1215 "voices.cpp"
            if (!($cmp61)) { __label__ = 33;break $if_end60_thread$$if_else$8; } //@line 1215 "voices.cpp"
  
            $retval=0; //@line 1216 "voices.cpp"
            __label__ = 63;break $if_then$$if_end4$2; //@line 1216 "voices.cpp"
          }
        } while(0);
  
        var $52=$voice_spec_addr; //@line 1218 "voices.cpp"
        var $name=(($52)&4294967295); //@line 1218 "voices.cpp"
        var $53=HEAP32[(($name)>>2)]; //@line 1218 "voices.cpp"
        var $cmp64=((($53))|0)!=0; //@line 1218 "voices.cpp"
        if ($cmp64) { __label__ = 34;; } else { __label__ = 38;; } //@line 1218 "voices.cpp"
        $if_then65$$if_end81$50: do { 
          if (__label__ == 34) {
  
            var $54=$voice_spec_addr; //@line 1220 "voices.cpp"
            var $name66=(($54)&4294967295); //@line 1220 "voices.cpp"
            var $55=HEAP32[(($name66)>>2)]; //@line 1220 "voices.cpp"
            var $56=$voice_addr; //@line 1220 "voices.cpp"
            var $name67=(($56)&4294967295); //@line 1220 "voices.cpp"
            var $57=HEAP32[(($name67)>>2)]; //@line 1220 "voices.cpp"
            var $call68=_strcmp($55, $57); //@line 1220 "voices.cpp"
            var $cmp69=((($call68))|0)==0; //@line 1220 "voices.cpp"
            if ($cmp69) { __label__ = 35;; } else { __label__ = 36;; } //@line 1220 "voices.cpp"
            if (__label__ == 35) {
  
              var $58=$score; //@line 1223 "voices.cpp"
              var $add71=((($58)+500)&4294967295); //@line 1223 "voices.cpp"
              $score=$add71; //@line 1223 "voices.cpp"
              ; //@line 1224 "voices.cpp"
            }
            else if (__label__ == 36) {
  
              var $59=$voice_spec_addr; //@line 1226 "voices.cpp"
              var $name73=(($59)&4294967295); //@line 1226 "voices.cpp"
              var $60=HEAP32[(($name73)>>2)]; //@line 1226 "voices.cpp"
              var $61=$voice_addr; //@line 1226 "voices.cpp"
              var $identifier74=(($61+8)&4294967295); //@line 1226 "voices.cpp"
              var $62=HEAP32[(($identifier74)>>2)]; //@line 1226 "voices.cpp"
              var $call75=_strcmp($60, $62); //@line 1226 "voices.cpp"
              var $cmp76=((($call75))|0)==0; //@line 1226 "voices.cpp"
              if (!($cmp76)) { __label__ = 38;break $if_then65$$if_end81$50; } //@line 1226 "voices.cpp"
  
              var $63=$score; //@line 1228 "voices.cpp"
              var $add78=((($63)+400)&4294967295); //@line 1228 "voices.cpp"
              $score=$add78; //@line 1228 "voices.cpp"
              ; //@line 1229 "voices.cpp"
            }
          }
        } while(0);
  
        var $64=$voice_spec_addr; //@line 1232 "voices.cpp"
        var $gender=(($64+12)&4294967295); //@line 1232 "voices.cpp"
        var $65=HEAPU8[($gender)]; //@line 1232 "voices.cpp"
        var $conv82=((($65))&255); //@line 1232 "voices.cpp"
        var $cmp83=((($conv82))|0)==1; //@line 1232 "voices.cpp"
        if ($cmp83) { __label__ = 40;; } else { __label__ = 39;; } //@line 1232 "voices.cpp"
        $land_lhs_true88$$lor_lhs_false84$57: do { 
          if (__label__ == 39) {
  
            var $66=$voice_spec_addr; //@line 1232 "voices.cpp"
            var $gender85=(($66+12)&4294967295); //@line 1232 "voices.cpp"
            var $67=HEAPU8[($gender85)]; //@line 1232 "voices.cpp"
            var $conv86=((($67))&255); //@line 1232 "voices.cpp"
            var $cmp87=((($conv86))|0)==2; //@line 1232 "voices.cpp"
            if ($cmp87) { __label__ = 40;break $land_lhs_true88$$lor_lhs_false84$57; } else { __label__ = 45;break $land_lhs_true88$$lor_lhs_false84$57; } //@line 1232 "voices.cpp"
          }
        } while(0);
        $land_lhs_true88$$if_end107$59: do { 
          if (__label__ == 40) {
  
            var $68=$voice_addr; //@line 1232 "voices.cpp"
            var $gender89=(($68+12)&4294967295); //@line 1232 "voices.cpp"
            var $69=HEAPU8[($gender89)]; //@line 1232 "voices.cpp"
            var $conv90=((($69))&255); //@line 1232 "voices.cpp"
            var $cmp91=((($conv90))|0)==1; //@line 1232 "voices.cpp"
            if ($cmp91) { __label__ = 42;; } else { __label__ = 41;; } //@line 1232 "voices.cpp"
            if (__label__ == 41) {
  
              var $70=$voice_addr; //@line 1232 "voices.cpp"
              var $gender93=(($70+12)&4294967295); //@line 1232 "voices.cpp"
              var $71=HEAPU8[($gender93)]; //@line 1232 "voices.cpp"
              var $conv94=((($71))&255); //@line 1232 "voices.cpp"
              var $cmp95=((($conv94))|0)==2; //@line 1232 "voices.cpp"
              if (!($cmp95)) { __label__ = 45;break $land_lhs_true88$$if_end107$59; } //@line 1232 "voices.cpp"
            }
  
            var $72=$voice_spec_addr; //@line 1235 "voices.cpp"
            var $gender97=(($72+12)&4294967295); //@line 1235 "voices.cpp"
            var $73=HEAPU8[($gender97)]; //@line 1235 "voices.cpp"
            var $conv98=((($73))&255); //@line 1235 "voices.cpp"
            var $74=$voice_addr; //@line 1235 "voices.cpp"
            var $gender99=(($74+12)&4294967295); //@line 1235 "voices.cpp"
            var $75=HEAPU8[($gender99)]; //@line 1235 "voices.cpp"
            var $conv100=((($75))&255); //@line 1235 "voices.cpp"
            var $cmp101=((($conv98))|0)==((($conv100))|0); //@line 1235 "voices.cpp"
            var $76=$score; //@line 1236 "voices.cpp"
            if ($cmp101) { __label__ = 43;; } else { __label__ = 44;; } //@line 1235 "voices.cpp"
            if (__label__ == 43) {
  
              var $add103=((($76)+50)&4294967295); //@line 1236 "voices.cpp"
              $score=$add103; //@line 1236 "voices.cpp"
              ; //@line 1236 "voices.cpp"
            }
            else if (__label__ == 44) {
  
              var $sub105=((($76)-50)&4294967295); //@line 1238 "voices.cpp"
              $score=$sub105; //@line 1238 "voices.cpp"
              ;
            }
          }
        } while(0);
  
        var $77=$voice_spec_addr; //@line 1241 "voices.cpp"
        var $age=(($77+13)&4294967295); //@line 1241 "voices.cpp"
        var $78=HEAPU8[($age)]; //@line 1241 "voices.cpp"
        var $conv108=((($78))&255); //@line 1241 "voices.cpp"
        var $cmp109=((($conv108))|0) <= 12; //@line 1241 "voices.cpp"
        if ($cmp109) { __label__ = 46;; } else { __label__ = 49;; } //@line 1241 "voices.cpp"
        $land_lhs_true110$$if_end120$68: do { 
          if (__label__ == 46) {
  
            var $79=$voice_addr; //@line 1241 "voices.cpp"
            var $gender111=(($79+12)&4294967295); //@line 1241 "voices.cpp"
            var $80=HEAPU8[($gender111)]; //@line 1241 "voices.cpp"
            var $conv112=((($80))&255); //@line 1241 "voices.cpp"
            var $cmp113=((($conv112))|0)==2; //@line 1241 "voices.cpp"
            if (!($cmp113)) { __label__ = 49;break $land_lhs_true110$$if_end120$68; } //@line 1241 "voices.cpp"
  
            var $81=$voice_addr; //@line 1241 "voices.cpp"
            var $age115=(($81+13)&4294967295); //@line 1241 "voices.cpp"
            var $82=HEAPU8[($age115)]; //@line 1241 "voices.cpp"
            var $conv116=((($82))&255); //@line 1241 "voices.cpp"
            var $cmp117=((($conv116))|0) > 12; //@line 1241 "voices.cpp"
            if (!($cmp117)) { __label__ = 49;break $land_lhs_true110$$if_end120$68; } //@line 1241 "voices.cpp"
  
            var $83=$score; //@line 1243 "voices.cpp"
            var $add119=((($83)+5)&4294967295); //@line 1243 "voices.cpp"
            $score=$add119; //@line 1243 "voices.cpp"
            ; //@line 1244 "voices.cpp"
          }
        } while(0);
  
        var $84=$voice_addr; //@line 1246 "voices.cpp"
        var $age121=(($84+13)&4294967295); //@line 1246 "voices.cpp"
        var $85=HEAPU8[($age121)]; //@line 1246 "voices.cpp"
        var $conv122=((($85))&255); //@line 1246 "voices.cpp"
        var $cmp123=((($conv122))|0)!=0; //@line 1246 "voices.cpp"
        if ($cmp123) { __label__ = 50;; } else { __label__ = 59;; } //@line 1246 "voices.cpp"
        $if_then124$$if_end153thread_pre_split$73: do { 
          if (__label__ == 50) {
  
            var $86=$voice_spec_addr; //@line 1248 "voices.cpp"
            var $age125=(($86+13)&4294967295); //@line 1248 "voices.cpp"
            var $87=HEAPU8[($age125)]; //@line 1248 "voices.cpp"
            var $conv126=((($87))&255); //@line 1248 "voices.cpp"
            var $cmp127=((($conv126))|0)==0; //@line 1248 "voices.cpp"
            if ($cmp127) { __label__ = 51;; } else { __label__ = 52;; } //@line 1248 "voices.cpp"
            if (__label__ == 51) {
  
              $required_age=30; //@line 1249 "voices.cpp"
              ; //@line 1249 "voices.cpp"
            }
            else if (__label__ == 52) {
  
              var $88=$voice_spec_addr; //@line 1251 "voices.cpp"
              var $age130=(($88+13)&4294967295); //@line 1251 "voices.cpp"
              var $89=HEAPU8[($age130)]; //@line 1251 "voices.cpp"
              var $conv131=((($89))&255); //@line 1251 "voices.cpp"
              $required_age=$conv131; //@line 1251 "voices.cpp"
              ;
            }
  
            var $90=$required_age; //@line 1253 "voices.cpp"
            var $mul133=((($90)*100)&4294967295); //@line 1253 "voices.cpp"
            var $91=$voice_addr; //@line 1253 "voices.cpp"
            var $age134=(($91+13)&4294967295); //@line 1253 "voices.cpp"
            var $92=HEAPU8[($age134)]; //@line 1253 "voices.cpp"
            var $conv135=((($92))&255); //@line 1253 "voices.cpp"
            var $div=((((($mul133))|0)/((($conv135))|0))|0); //@line 1253 "voices.cpp"
            $ratio=$div; //@line 1253 "voices.cpp"
            var $cmp136=((($div))|0) < 100; //@line 1254 "voices.cpp"
            if ($cmp136) { __label__ = 54;; } else { __label__ = 55;; } //@line 1254 "voices.cpp"
            if (__label__ == 54) {
  
              var $93=$ratio; //@line 1255 "voices.cpp"
              var $div138=((10000/((($93))|0))|0); //@line 1255 "voices.cpp"
              $ratio=$div138; //@line 1255 "voices.cpp"
              ; //@line 1255 "voices.cpp"
            }
  
            var $94=$ratio; //@line 1256 "voices.cpp"
            var $sub140=((($94)-100)&4294967295); //@line 1256 "voices.cpp"
            var $div141=((((($sub140))|0)/10)|0); //@line 1256 "voices.cpp"
            $ratio=$div141; //@line 1256 "voices.cpp"
            var $95=$ratio; //@line 1257 "voices.cpp"
            var $sub142=((5-($95))&4294967295); //@line 1257 "voices.cpp"
            $x=$sub142; //@line 1257 "voices.cpp"
            var $cmp143=((($sub142))|0) > 0; //@line 1258 "voices.cpp"
            if ($cmp143) { __label__ = 56;; } else { __label__ = 57;; } //@line 1258 "voices.cpp"
            if (__label__ == 56) {
  
              $x=0; //@line 1258 "voices.cpp"
              ; //@line 1258 "voices.cpp"
            }
  
            var $96=$score; //@line 1260 "voices.cpp"
            var $97=$x; //@line 1260 "voices.cpp"
            var $add146=((($97)+($96))&4294967295); //@line 1260 "voices.cpp"
            $score=$add146; //@line 1260 "voices.cpp"
            var $98=$voice_spec_addr; //@line 1262 "voices.cpp"
            var $age147=(($98+13)&4294967295); //@line 1262 "voices.cpp"
            var $99=HEAPU8[($age147)]; //@line 1262 "voices.cpp"
            var $conv148=((($99))&255); //@line 1262 "voices.cpp"
            var $cmp149=((($conv148))|0) > 0; //@line 1262 "voices.cpp"
            if (!($cmp149)) { __label__ = 59;break $if_then124$$if_end153thread_pre_split$73; } //@line 1262 "voices.cpp"
  
            var $100=$score; //@line 1263 "voices.cpp"
            var $add151=((($100)+10)&4294967295); //@line 1263 "voices.cpp"
            $score=$add151; //@line 1263 "voices.cpp"
            __lastLabel__ = 58; __label__ = 60;break $if_then124$$if_end153thread_pre_split$73; //@line 1263 "voices.cpp"
          }
        } while(0);
        if (__label__ == 59) {
  
          var $_pr2=$score; //@line 1265 "voices.cpp"
          __lastLabel__ = 59; ;
        }
  
        var $101=__lastLabel__ == 59 ? $_pr2 : ($add151); //@line 1265 "voices.cpp"
        var $cmp154=((($101))|0) < 1; //@line 1265 "voices.cpp"
        if ($cmp154) { __label__ = 61;; } else { __label__ = 62;; } //@line 1265 "voices.cpp"
        if (__label__ == 61) {
  
          $score=1; //@line 1266 "voices.cpp"
          ; //@line 1266 "voices.cpp"
        }
  
        var $102=$score; //@line 1267 "voices.cpp"
        $retval=$102; //@line 1267 "voices.cpp"
        ; //@line 1267 "voices.cpp"
      }
    } while(0);
  
    var $103=$retval; //@line 1268 "voices.cpp"
    ;
    return $103; //@line 1268 "voices.cpp"
    return null;
  }
  

  function __ZL16VoiceScoreSorterPKvS0_($p1, $p2) {
    ;
    var __label__;
  
    var $retval;
    var $p1_addr;
    var $p2_addr;
    var $ix;
    var $v1;
    var $v2;
    $p1_addr=$p1;
    $p2_addr=$p2;
    var $0=$p1_addr; //@line 1118 "voices.cpp"
    var $1=$0; //@line 1118 "voices.cpp"
    var $2=HEAP32[(($1)>>2)]; //@line 1118 "voices.cpp"
    $v1=$2; //@line 1118 "voices.cpp"
    var $3=$p2_addr; //@line 1119 "voices.cpp"
    var $4=$3; //@line 1119 "voices.cpp"
    var $5=HEAP32[(($4)>>2)]; //@line 1119 "voices.cpp"
    $v2=$5; //@line 1119 "voices.cpp"
    var $6=$v2; //@line 1121 "voices.cpp"
    var $score=(($6+16)&4294967295); //@line 1121 "voices.cpp"
    var $7=HEAP32[(($score)>>2)]; //@line 1121 "voices.cpp"
    var $8=$v1; //@line 1121 "voices.cpp"
    var $score1=(($8+16)&4294967295); //@line 1121 "voices.cpp"
    var $9=HEAP32[(($score1)>>2)]; //@line 1121 "voices.cpp"
    var $sub=((($7)-($9))&4294967295); //@line 1121 "voices.cpp"
    $ix=$sub; //@line 1121 "voices.cpp"
    var $cmp=((($sub))|0)!=0; //@line 1121 "voices.cpp"
    if ($cmp) { __label__ = 1;; } else { __label__ = 2;; } //@line 1121 "voices.cpp"
    if (__label__ == 1) {
  
      var $10=$ix; //@line 1122 "voices.cpp"
      $retval=$10; //@line 1122 "voices.cpp"
      ; //@line 1122 "voices.cpp"
    }
    else if (__label__ == 2) {
  
      var $11=$v1; //@line 1123 "voices.cpp"
      var $name=(($11)&4294967295); //@line 1123 "voices.cpp"
      var $12=HEAP32[(($name)>>2)]; //@line 1123 "voices.cpp"
      var $13=$v2; //@line 1123 "voices.cpp"
      var $name2=(($13)&4294967295); //@line 1123 "voices.cpp"
      var $14=HEAP32[(($name2)>>2)]; //@line 1123 "voices.cpp"
      var $call=_strcmp($12, $14); //@line 1123 "voices.cpp"
      $retval=$call; //@line 1123 "voices.cpp"
      ; //@line 1123 "voices.cpp"
    }
  
    var $15=$retval; //@line 1124 "voices.cpp"
    ;
    return $15; //@line 1124 "voices.cpp"
    return null;
  }
  

  function __Z9WcmdqStopv() {
    ;
    var __label__;
  
    HEAP32[((_wcmdq_head)>>2)]=0; //@line 287 "wavegen.cpp"
    HEAP32[((_wcmdq_tail)>>2)]=0; //@line 288 "wavegen.cpp"
    var $0=HEAP32[((__ZL18sonicSpeedupStream)>>2)]; //@line 289 "wavegen.cpp"
    var $cmp=((($0))|0)!=0; //@line 289 "wavegen.cpp"
    if ($cmp) { __label__ = 1;; } else { __label__ = 2;; } //@line 289 "wavegen.cpp"
    if (__label__ == 1) {
  
      var $1=HEAP32[((__ZL18sonicSpeedupStream)>>2)]; //@line 291 "wavegen.cpp"
      _sonicDestroyStream($1); //@line 291 "wavegen.cpp"
      HEAP32[((__ZL18sonicSpeedupStream)>>2)]=0; //@line 292 "wavegen.cpp"
      ; //@line 293 "wavegen.cpp"
    }
  
    var $2=HEAP8[(((_mbrola_name)&4294967295))]; //@line 297 "wavegen.cpp"
    var $conv=(tempInt=(($2)),(tempInt>=128?tempInt-256:tempInt)); //@line 297 "wavegen.cpp"
    var $cmp1=((($conv))|0)!=0; //@line 297 "wavegen.cpp"
    if ($cmp1) { __label__ = 3;; } else { __label__ = 4;; } //@line 297 "wavegen.cpp"
    if (__label__ == 3) {
  
      __Z11MbrolaResetv(); //@line 298 "wavegen.cpp"
      ; //@line 298 "wavegen.cpp"
    }
  
    ;
    return; //@line 299 "wavegen.cpp"
    return;
  }
  

  function __Z9WcmdqFreev() {
    ;
    var __label__;
  
    var $i;
    var $0=HEAP32[((_wcmdq_head)>>2)]; //@line 305 "wavegen.cpp"
    var $1=HEAP32[((_wcmdq_tail)>>2)]; //@line 305 "wavegen.cpp"
    var $sub=((($0)-($1))&4294967295); //@line 305 "wavegen.cpp"
    $i=$sub; //@line 305 "wavegen.cpp"
    var $2=$i; //@line 306 "wavegen.cpp"
    var $cmp=((($2))|0) <= 0; //@line 306 "wavegen.cpp"
    if ($cmp) { __label__ = 1;; } else { __label__ = 2;; } //@line 306 "wavegen.cpp"
    if (__label__ == 1) {
  
      var $3=$i; //@line 306 "wavegen.cpp"
      var $add=((($3)+160)&4294967295); //@line 306 "wavegen.cpp"
      $i=$add; //@line 306 "wavegen.cpp"
      ; //@line 306 "wavegen.cpp"
    }
  
    var $4=$i; //@line 307 "wavegen.cpp"
    ;
    return $4; //@line 307 "wavegen.cpp"
    return null;
  }
  

  function __Z9WcmdqUsedv() {
    ;
    var __label__;
  
    var $call=__Z9WcmdqFreev(); //@line 312 "wavegen.cpp"
    var $sub=((160-($call))&4294967295); //@line 312 "wavegen.cpp"
    ;
    return $sub; //@line 312 "wavegen.cpp"
    return null;
  }
  

  function __Z8WcmdqIncv() {
    ;
    var __label__;
  
    var $0=HEAP32[((_wcmdq_tail)>>2)]; //@line 318 "wavegen.cpp"
    var $inc=((($0)+1)&4294967295); //@line 318 "wavegen.cpp"
    HEAP32[((_wcmdq_tail)>>2)]=$inc; //@line 318 "wavegen.cpp"
    var $cmp=((($inc))|0) >= 160; //@line 319 "wavegen.cpp"
    if ($cmp) { __label__ = 1;; } else { __label__ = 2;; } //@line 319 "wavegen.cpp"
    if (__label__ == 1) {
  
      HEAP32[((_wcmdq_tail)>>2)]=0; //@line 319 "wavegen.cpp"
      ; //@line 319 "wavegen.cpp"
    }
  
    ;
    return; //@line 320 "wavegen.cpp"
    return;
  }
  

  function __Z16WavegenOpenSoundv() {
    ;
    var __label__;
  
    ;
    return 0; //@line 715 "wavegen.cpp"
    return null;
  }
  

  function __Z17WavegenCloseSoundv() {
    ;
    var __label__;
  
    ;
    return 0; //@line 719 "wavegen.cpp"
    return null;
  }
  

  function __Z16WavegenInitSoundv() {
    ;
    var __label__;
  
    ;
    return 0; //@line 723 "wavegen.cpp"
    return null;
  }
  

  function __Z11WavegenInitii($rate, $wavemult_fact) {
    ;
    var __label__;
  
    var $rate_addr;
    var $wavemult_fact_addr;
    var $ix;
    var $x;
    $rate_addr=$rate;
    $wavemult_fact_addr=$wavemult_fact;
    var $0=$wavemult_fact_addr; //@line 733 "wavegen.cpp"
    var $cmp=((($0))|0)==0; //@line 733 "wavegen.cpp"
    if ($cmp) { __label__ = 1;; } else { __label__ = 2;; } //@line 733 "wavegen.cpp"
    if (__label__ == 1) {
  
      $wavemult_fact_addr=60; //@line 734 "wavegen.cpp"
      ; //@line 734 "wavegen.cpp"
    }
  
    HEAP32[((_wvoice)>>2)]=0; //@line 736 "wavegen.cpp"
    var $1=$rate_addr; //@line 737 "wavegen.cpp"
    HEAP32[((_samplerate_native)>>2)]=$1; //@line 737 "wavegen.cpp"
    HEAP32[((_samplerate)>>2)]=$1; //@line 737 "wavegen.cpp"
    var $2=HEAP32[((_samplerate)>>2)]; //@line 738 "wavegen.cpp"
    var $div=((134217728/((($2))|0))|0); //@line 738 "wavegen.cpp"
    HEAP32[((__ZL16PHASE_INC_FACTOR)>>2)]=$div; //@line 738 "wavegen.cpp"
    var $3=HEAP32[((_samplerate)>>2)]; //@line 739 "wavegen.cpp"
    var $mul=((($3)*64)&4294967295); //@line 739 "wavegen.cpp"
    var $4=$rate_addr; //@line 739 "wavegen.cpp"
    var $div1=((((($mul))|0)/((($4))|0))|0); //@line 739 "wavegen.cpp"
    HEAP32[((__ZL11Flutter_inc)>>2)]=$div1; //@line 739 "wavegen.cpp"
    HEAP32[((__ZL11samplecount)>>2)]=0; //@line 740 "wavegen.cpp"
    HEAP32[((__ZL8nsamples)>>2)]=0; //@line 741 "wavegen.cpp"
    HEAP32[((__ZL9wavephase)>>2)]=2147483647; //@line 742 "wavegen.cpp"
    HEAP32[((((_wdata+52)&4294967295))>>2)]=32; //@line 745 "wavegen.cpp"
    HEAP32[((((_wdata+60)&4294967295))>>2)]=100; //@line 746 "wavegen.cpp"
    $ix=0; //@line 748 "wavegen.cpp"
    ; //@line 748 "wavegen.cpp"
    $for_body$5: while(1) { 
  
      var $5=$ix; //@line 749 "wavegen.cpp"
      var $arrayidx=((_embedded_default+$5*4)&4294967295); //@line 749 "wavegen.cpp"
      var $6=HEAP32[(($arrayidx)>>2)]; //@line 749 "wavegen.cpp"
      var $7=$ix; //@line 749 "wavegen.cpp"
      var $arrayidx3=((_embedded_value+$7*4)&4294967295); //@line 749 "wavegen.cpp"
      HEAP32[(($arrayidx3)>>2)]=$6; //@line 749 "wavegen.cpp"
      var $8=$ix; //@line 748 "wavegen.cpp"
      var $inc=((($8)+1)&4294967295); //@line 748 "wavegen.cpp"
      $ix=$inc; //@line 748 "wavegen.cpp"
      var $cmp2=((($inc))|0) < 15; //@line 748 "wavegen.cpp"
      if ($cmp2) { __label__ = 3;continue $for_body$5; } else { __label__ = 4;break $for_body$5; } //@line 748 "wavegen.cpp"
    }
  
    var $9=HEAP32[((_samplerate)>>2)]; //@line 754 "wavegen.cpp"
    var $10=$wavemult_fact_addr; //@line 754 "wavegen.cpp"
    var $mul4=((($10)*($9))&4294967295); //@line 754 "wavegen.cpp"
    var $div5=((((($mul4))|0)/12800)|0); //@line 754 "wavegen.cpp"
    HEAP32[((__ZL12wavemult_max)>>2)]=$div5; //@line 754 "wavegen.cpp"
    var $11=HEAP32[((__ZL12wavemult_max)>>2)]; //@line 755 "wavegen.cpp"
    var $cmp6=((($11))|0) > 128; //@line 755 "wavegen.cpp"
    if ($cmp6) { __label__ = 5;; } else { __label__ = 6;; } //@line 755 "wavegen.cpp"
    if (__label__ == 5) {
  
      HEAP32[((__ZL12wavemult_max)>>2)]=128; //@line 755 "wavegen.cpp"
      ; //@line 755 "wavegen.cpp"
    }
  
    var $12=HEAP32[((__ZL12wavemult_max)>>2)]; //@line 757 "wavegen.cpp"
    var $div9=((((($12))|0)/2)|0); //@line 757 "wavegen.cpp"
    HEAP32[((__ZL15wavemult_offset)>>2)]=$div9; //@line 757 "wavegen.cpp"
    var $13=HEAP32[((_samplerate)>>2)]; //@line 759 "wavegen.cpp"
    var $cmp10=((($13))|0)!=22050; //@line 759 "wavegen.cpp"
    if ($cmp10) { __label__ = 7;; } else { __label__ = 9;; } //@line 759 "wavegen.cpp"
    $if_then11$$if_end25$11: do { 
      if (__label__ == 7) {
  
        $ix=0; //@line 763 "wavegen.cpp"
        var $14=$ix; //@line 763 "wavegen.cpp"
        var $15=HEAP32[((__ZL12wavemult_max)>>2)]; //@line 763 "wavegen.cpp"
        var $cmp131=((($14))|0) < ((($15))|0); //@line 763 "wavegen.cpp"
        if (!($cmp131)) { __label__ = 9;break $if_then11$$if_end25$11; } //@line 763 "wavegen.cpp"
        while(1) { 
  
          var $16=$ix; //@line 765 "wavegen.cpp"
          var $conv=((($16))|0); //@line 765 "wavegen.cpp"
          var $mul15=6.283185307*($conv); //@line 765 "wavegen.cpp"
          var $17=HEAP32[((__ZL12wavemult_max)>>2)]; //@line 765 "wavegen.cpp"
          var $conv16=((($17))|0); //@line 765 "wavegen.cpp"
          var $div17=($mul15)/($conv16); //@line 765 "wavegen.cpp"
          var $call=_cos($div17); //@line 765 "wavegen.cpp"
          var $sub=1-($call); //@line 765 "wavegen.cpp"
          var $mul18=127*($sub); //@line 765 "wavegen.cpp"
          $x=$mul18; //@line 765 "wavegen.cpp"
          var $18=$x; //@line 766 "wavegen.cpp"
          var $conv19=((($18))|0); //@line 766 "wavegen.cpp"
          var $conv20=((($conv19)) & 255); //@line 766 "wavegen.cpp"
          var $19=$ix; //@line 766 "wavegen.cpp"
          var $arrayidx21=((__ZL8wavemult+$19)&4294967295); //@line 766 "wavegen.cpp"
          HEAP8[($arrayidx21)]=$conv20; //@line 766 "wavegen.cpp"
          var $20=$ix; //@line 763 "wavegen.cpp"
          var $inc23=((($20)+1)&4294967295); //@line 763 "wavegen.cpp"
          $ix=$inc23; //@line 763 "wavegen.cpp"
          var $21=$ix; //@line 763 "wavegen.cpp"
          var $22=HEAP32[((__ZL12wavemult_max)>>2)]; //@line 763 "wavegen.cpp"
          var $cmp13=((($21))|0) < ((($22))|0); //@line 763 "wavegen.cpp"
          if ($cmp13) { __label__ = 8;continue ; } else { __label__ = 9;break $if_then11$$if_end25$11; } //@line 763 "wavegen.cpp"
        }
      }
    } while(0);
  
    __ZL17WavegenInitPkDatai(1); //@line 770 "wavegen.cpp"
    __ZL17WavegenInitPkDatai(0); //@line 771 "wavegen.cpp"
    HEAP32[((__ZL8pk_shape)>>2)]=((__ZL9pk_shape2)&4294967295); //@line 772 "wavegen.cpp"
    __Z9KlattInitv(); //@line 775 "wavegen.cpp"
    ;
    return; //@line 782 "wavegen.cpp"
    return;
  }
  

  function __ZL17WavegenInitPkDatai($which) {
    ;
    var __label__;
  
    ;
    return; //@line 411 "wavegen.cpp"
    return;
  }
  

  function __Z12GetAmplitudev() {
    ;
    var __label__;
  
    var $amp;
    var $0=HEAP32[((((_embedded_value+12)&4294967295))>>2)]; //@line 792 "wavegen.cpp"
    var $mul=((($0)*55)&4294967295); //@line 792 "wavegen.cpp"
    var $div=((((($mul))|0)/100)|0); //@line 792 "wavegen.cpp"
    $amp=$div; //@line 792 "wavegen.cpp"
    var $1=$amp; //@line 793 "wavegen.cpp"
    var $2=HEAP32[((((_embedded_value+52)&4294967295))>>2)]; //@line 793 "wavegen.cpp"
    var $arrayidx=((__ZZ12GetAmplitudevE12amp_emphasis+$2)&4294967295); //@line 793 "wavegen.cpp"
    var $3=HEAPU8[($arrayidx)]; //@line 793 "wavegen.cpp"
    var $conv=((($3))&255); //@line 793 "wavegen.cpp"
    var $mul1=((($conv)*($1))&4294967295); //@line 793 "wavegen.cpp"
    var $div2=((((($mul1))|0)/16)|0); //@line 793 "wavegen.cpp"
    HEAP32[((__ZL17general_amplitude)>>2)]=$div2; //@line 793 "wavegen.cpp"
    var $4=HEAP32[((__ZL17general_amplitude)>>2)]; //@line 794 "wavegen.cpp"
    ;
    return $4; //@line 794 "wavegen.cpp"
    return null;
  }
  

  function __Z16PeaksToHarmspectP15wavegen_peaks_tiPii($peaks, $pitch, $htab, $control) {
    ;
    var __label__;
  
    var $retval;
    var $peaks_addr;
    var $pitch_addr;
    var $htab_addr;
    var $control_addr;
    var $f;
    var $p;
    var $fp;
    var $fhi;
    var $h;
    var $pk;
    var $hmax;
    var $hmax_samplerate;
    var $x;
    var $ix;
    var $h1;
    var $y;
    var $h2;
    $peaks_addr=$peaks;
    $pitch_addr=$pitch;
    $htab_addr=$htab;
    $control_addr=$control;
    var $0=HEAP32[((_wvoice)>>2)]; //@line 877 "wavegen.cpp"
    var $cmp=((($0))|0)==0; //@line 877 "wavegen.cpp"
    if ($cmp) { __label__ = 1;; } else { __label__ = 2;; } //@line 877 "wavegen.cpp"
    if (__label__ == 1) {
  
      $retval=1; //@line 878 "wavegen.cpp"
      ; //@line 878 "wavegen.cpp"
    }
    else if (__label__ == 2) {
  
      var $1=HEAP32[((_wvoice)>>2)]; //@line 879 "wavegen.cpp"
      var $n_harmonic_peaks=(($1+84)&4294967295); //@line 879 "wavegen.cpp"
      var $2=HEAP32[(($n_harmonic_peaks)>>2)]; //@line 879 "wavegen.cpp"
      var $3=$peaks_addr; //@line 879 "wavegen.cpp"
      var $arrayidx=(($3+80*$2)&4294967295); //@line 879 "wavegen.cpp"
      var $freq=(($arrayidx)&4294967295); //@line 879 "wavegen.cpp"
      var $4=HEAP32[(($freq)>>2)]; //@line 879 "wavegen.cpp"
      var $5=HEAP32[((_wvoice)>>2)]; //@line 879 "wavegen.cpp"
      var $n_harmonic_peaks1=(($5+84)&4294967295); //@line 879 "wavegen.cpp"
      var $6=HEAP32[(($n_harmonic_peaks1)>>2)]; //@line 879 "wavegen.cpp"
      var $7=$peaks_addr; //@line 879 "wavegen.cpp"
      var $arrayidx2=(($7+80*$6)&4294967295); //@line 879 "wavegen.cpp"
      var $right=(($arrayidx2+12)&4294967295); //@line 879 "wavegen.cpp"
      var $8=HEAP32[(($right)>>2)]; //@line 879 "wavegen.cpp"
      var $add=((($8)+($4))&4294967295); //@line 879 "wavegen.cpp"
      var $9=$pitch_addr; //@line 879 "wavegen.cpp"
      var $div=((((($add))|0)/((($9))|0))|0); //@line 879 "wavegen.cpp"
      $hmax=$div; //@line 879 "wavegen.cpp"
      var $10=$hmax; //@line 880 "wavegen.cpp"
      var $cmp3=((($10))|0) >= 400; //@line 880 "wavegen.cpp"
      if ($cmp3) { __label__ = 3;; } else { __label__ = 4;; } //@line 880 "wavegen.cpp"
      if (__label__ == 3) {
  
        $hmax=399; //@line 881 "wavegen.cpp"
        ; //@line 881 "wavegen.cpp"
      }
  
      var $11=HEAP32[((_samplerate)>>2)]; //@line 884 "wavegen.cpp"
      var $mul=((($11)*19)&4294967295); //@line 884 "wavegen.cpp"
      var $div6=((((($mul))|0)/40)|0); //@line 884 "wavegen.cpp"
      var $shl=($div6) << 16; //@line 884 "wavegen.cpp"
      var $12=$pitch_addr; //@line 884 "wavegen.cpp"
      var $div7=((((($shl))|0)/((($12))|0))|0); //@line 884 "wavegen.cpp"
      $hmax_samplerate=$div7; //@line 884 "wavegen.cpp"
      var $13=$hmax; //@line 887 "wavegen.cpp"
      var $14=$hmax_samplerate; //@line 887 "wavegen.cpp"
      var $cmp8=((($13))|0) > ((($14))|0); //@line 887 "wavegen.cpp"
      if ($cmp8) { __label__ = 5;; } else { __label__ = 6;; } //@line 887 "wavegen.cpp"
      if (__label__ == 5) {
  
        var $15=$hmax_samplerate; //@line 888 "wavegen.cpp"
        $hmax=$15; //@line 888 "wavegen.cpp"
        ; //@line 888 "wavegen.cpp"
      }
  
      $h=0; //@line 890 "wavegen.cpp"
      var $16=$h; //@line 890 "wavegen.cpp"
      var $17=$hmax; //@line 890 "wavegen.cpp"
      var $cmp118=((($16))|0) <= ((($17))|0); //@line 890 "wavegen.cpp"
      if ($cmp118) { __label__ = 7;; } else { __label__ = 8;; } //@line 890 "wavegen.cpp"
      $for_body$$for_end$11: do { 
        if (__label__ == 7) {
          while(1) { 
  
            var $18=$h; //@line 891 "wavegen.cpp"
            var $19=$htab_addr; //@line 891 "wavegen.cpp"
            var $arrayidx12=(($19+4*$18)&4294967295); //@line 891 "wavegen.cpp"
            HEAP32[(($arrayidx12)>>2)]=0; //@line 891 "wavegen.cpp"
            var $20=$h; //@line 890 "wavegen.cpp"
            var $inc=((($20)+1)&4294967295); //@line 890 "wavegen.cpp"
            $h=$inc; //@line 890 "wavegen.cpp"
            var $21=$h; //@line 890 "wavegen.cpp"
            var $22=$hmax; //@line 890 "wavegen.cpp"
            var $cmp11=((($21))|0) <= ((($22))|0); //@line 890 "wavegen.cpp"
            if ($cmp11) { __label__ = 7;continue ; } else { __label__ = 8;break $for_body$$for_end$11; } //@line 890 "wavegen.cpp"
          }
        }
      } while(0);
  
      $h=0; //@line 893 "wavegen.cpp"
      $pk=0; //@line 894 "wavegen.cpp"
      var $23=$pk; //@line 894 "wavegen.cpp"
      var $24=HEAP32[((_wvoice)>>2)]; //@line 894 "wavegen.cpp"
      var $n_harmonic_peaks146=(($24+84)&4294967295); //@line 894 "wavegen.cpp"
      var $25=HEAP32[(($n_harmonic_peaks146)>>2)]; //@line 894 "wavegen.cpp"
      var $cmp157=((($23))|0) <= ((($25))|0); //@line 894 "wavegen.cpp"
      if ($cmp157) { __label__ = 9;; } else { __label__ = 18;; } //@line 894 "wavegen.cpp"
      $for_body16$$for_end67$15: do { 
        if (__label__ == 9) {
          $for_body16$16: while(1) { 
  
            var $26=$pk; //@line 896 "wavegen.cpp"
            var $27=$peaks_addr; //@line 896 "wavegen.cpp"
            var $arrayidx17=(($27+80*$26)&4294967295); //@line 896 "wavegen.cpp"
            $p=$arrayidx17; //@line 896 "wavegen.cpp"
            var $28=$p; //@line 897 "wavegen.cpp"
            var $height=(($28+4)&4294967295); //@line 897 "wavegen.cpp"
            var $29=HEAP32[(($height)>>2)]; //@line 897 "wavegen.cpp"
            var $cmp18=((($29))|0)==0; //@line 897 "wavegen.cpp"
            if ($cmp18) { __label__ = 17;; } else { __label__ = 10;; } //@line 897 "wavegen.cpp"
            $for_inc65$$lor_lhs_false$18: do { 
              if (__label__ == 10) {
  
                var $30=$p; //@line 897 "wavegen.cpp"
                var $freq19=(($30)&4294967295); //@line 897 "wavegen.cpp"
                var $31=HEAP32[(($freq19)>>2)]; //@line 897 "wavegen.cpp"
                $fp=$31; //@line 897 "wavegen.cpp"
                var $cmp20=((($31))|0)==0; //@line 897 "wavegen.cpp"
                if ($cmp20) { __label__ = 17;break $for_inc65$$lor_lhs_false$18; } //@line 897 "wavegen.cpp"
  
                var $32=$p; //@line 900 "wavegen.cpp"
                var $freq23=(($32)&4294967295); //@line 900 "wavegen.cpp"
                var $33=HEAP32[(($freq23)>>2)]; //@line 900 "wavegen.cpp"
                var $34=$p; //@line 900 "wavegen.cpp"
                var $right24=(($34+12)&4294967295); //@line 900 "wavegen.cpp"
                var $35=HEAP32[(($right24)>>2)]; //@line 900 "wavegen.cpp"
                var $add25=((($35)+($33))&4294967295); //@line 900 "wavegen.cpp"
                $fhi=$add25; //@line 900 "wavegen.cpp"
                var $36=$p; //@line 901 "wavegen.cpp"
                var $freq26=(($36)&4294967295); //@line 901 "wavegen.cpp"
                var $37=HEAP32[(($freq26)>>2)]; //@line 901 "wavegen.cpp"
                var $38=$p; //@line 901 "wavegen.cpp"
                var $left=(($38+8)&4294967295); //@line 901 "wavegen.cpp"
                var $39=HEAP32[(($left)>>2)]; //@line 901 "wavegen.cpp"
                var $sub=((($37)-($39))&4294967295); //@line 901 "wavegen.cpp"
                var $40=$pitch_addr; //@line 901 "wavegen.cpp"
                var $div27=((((($sub))|0)/((($40))|0))|0); //@line 901 "wavegen.cpp"
                var $add28=((($div27)+1)&4294967295); //@line 901 "wavegen.cpp"
                $h=$add28; //@line 901 "wavegen.cpp"
                var $41=$h; //@line 902 "wavegen.cpp"
                var $cmp29=((($41))|0) <= 0; //@line 902 "wavegen.cpp"
                if ($cmp29) { __label__ = 12;; } else { __label__ = 13;; } //@line 902 "wavegen.cpp"
                if (__label__ == 12) {
  
                  $h=1; //@line 902 "wavegen.cpp"
                  ; //@line 902 "wavegen.cpp"
                }
  
                var $42=$pitch_addr; //@line 904 "wavegen.cpp"
                var $43=$h; //@line 904 "wavegen.cpp"
                var $mul32=((($43)*($42))&4294967295); //@line 904 "wavegen.cpp"
                $f=$mul32; //@line 904 "wavegen.cpp"
                var $44=$f; //@line 904 "wavegen.cpp"
                var $45=$fp; //@line 904 "wavegen.cpp"
                var $cmp344=((($44))|0) < ((($45))|0); //@line 904 "wavegen.cpp"
                if ($cmp344) { __label__ = 14;; } else { __label__ = 15;; } //@line 904 "wavegen.cpp"
                $for_body35$$for_cond48_loopexit$24: do { 
                  if (__label__ == 14) {
                    while(1) { 
  
                      var $46=$fp; //@line 906 "wavegen.cpp"
                      var $47=$f; //@line 906 "wavegen.cpp"
                      var $sub36=((($46)-($47))&4294967295); //@line 906 "wavegen.cpp"
                      var $48=$p; //@line 906 "wavegen.cpp"
                      var $left37=(($48+8)&4294967295); //@line 906 "wavegen.cpp"
                      var $49=HEAP32[(($left37)>>2)]; //@line 906 "wavegen.cpp"
                      var $shr=($49) >> 8; //@line 906 "wavegen.cpp"
                      var $div38=((((($sub36))|0)/((($shr))|0))|0); //@line 906 "wavegen.cpp"
                      var $50=HEAP32[((__ZL8pk_shape)>>2)]; //@line 906 "wavegen.cpp"
                      var $arrayidx39=(($50+$div38)&4294967295); //@line 906 "wavegen.cpp"
                      var $51=HEAPU8[($arrayidx39)]; //@line 906 "wavegen.cpp"
                      var $conv=((($51))&255); //@line 906 "wavegen.cpp"
                      var $52=$p; //@line 906 "wavegen.cpp"
                      var $height40=(($52+4)&4294967295); //@line 906 "wavegen.cpp"
                      var $53=HEAP32[(($height40)>>2)]; //@line 906 "wavegen.cpp"
                      var $mul41=((($53)*($conv))&4294967295); //@line 906 "wavegen.cpp"
                      var $54=$h; //@line 906 "wavegen.cpp"
                      var $inc42=((($54)+1)&4294967295); //@line 906 "wavegen.cpp"
                      $h=$inc42; //@line 906 "wavegen.cpp"
                      var $55=$htab_addr; //@line 906 "wavegen.cpp"
                      var $arrayidx43=(($55+4*$54)&4294967295); //@line 906 "wavegen.cpp"
                      var $56=HEAP32[(($arrayidx43)>>2)]; //@line 906 "wavegen.cpp"
                      var $add44=((($56)+($mul41))&4294967295); //@line 906 "wavegen.cpp"
                      HEAP32[(($arrayidx43)>>2)]=$add44; //@line 906 "wavegen.cpp"
                      var $57=$pitch_addr; //@line 904 "wavegen.cpp"
                      var $58=$f; //@line 904 "wavegen.cpp"
                      var $add46=((($58)+($57))&4294967295); //@line 904 "wavegen.cpp"
                      $f=$add46; //@line 904 "wavegen.cpp"
                      var $59=$f; //@line 904 "wavegen.cpp"
                      var $60=$fp; //@line 904 "wavegen.cpp"
                      var $cmp34=((($59))|0) < ((($60))|0); //@line 904 "wavegen.cpp"
                      if ($cmp34) { __label__ = 14;continue ; } else { __label__ = 15;break $for_body35$$for_cond48_loopexit$24; } //@line 904 "wavegen.cpp"
                    }
                  }
                } while(0);
  
                var $61=$f; //@line 908 "wavegen.cpp"
                var $62=$fhi; //@line 908 "wavegen.cpp"
                var $cmp495=((($61))|0) < ((($62))|0); //@line 908 "wavegen.cpp"
                if (!($cmp495)) { __label__ = 17;break $for_inc65$$lor_lhs_false$18; } //@line 908 "wavegen.cpp"
                while(1) { 
  
                  var $63=$f; //@line 910 "wavegen.cpp"
                  var $64=$fp; //@line 910 "wavegen.cpp"
                  var $sub51=((($63)-($64))&4294967295); //@line 910 "wavegen.cpp"
                  var $65=$p; //@line 910 "wavegen.cpp"
                  var $right52=(($65+12)&4294967295); //@line 910 "wavegen.cpp"
                  var $66=HEAP32[(($right52)>>2)]; //@line 910 "wavegen.cpp"
                  var $shr53=($66) >> 8; //@line 910 "wavegen.cpp"
                  var $div54=((((($sub51))|0)/((($shr53))|0))|0); //@line 910 "wavegen.cpp"
                  var $67=HEAP32[((__ZL8pk_shape)>>2)]; //@line 910 "wavegen.cpp"
                  var $arrayidx55=(($67+$div54)&4294967295); //@line 910 "wavegen.cpp"
                  var $68=HEAPU8[($arrayidx55)]; //@line 910 "wavegen.cpp"
                  var $conv56=((($68))&255); //@line 910 "wavegen.cpp"
                  var $69=$p; //@line 910 "wavegen.cpp"
                  var $height57=(($69+4)&4294967295); //@line 910 "wavegen.cpp"
                  var $70=HEAP32[(($height57)>>2)]; //@line 910 "wavegen.cpp"
                  var $mul58=((($70)*($conv56))&4294967295); //@line 910 "wavegen.cpp"
                  var $71=$h; //@line 910 "wavegen.cpp"
                  var $inc59=((($71)+1)&4294967295); //@line 910 "wavegen.cpp"
                  $h=$inc59; //@line 910 "wavegen.cpp"
                  var $72=$htab_addr; //@line 910 "wavegen.cpp"
                  var $arrayidx60=(($72+4*$71)&4294967295); //@line 910 "wavegen.cpp"
                  var $73=HEAP32[(($arrayidx60)>>2)]; //@line 910 "wavegen.cpp"
                  var $add61=((($73)+($mul58))&4294967295); //@line 910 "wavegen.cpp"
                  HEAP32[(($arrayidx60)>>2)]=$add61; //@line 910 "wavegen.cpp"
                  var $74=$pitch_addr; //@line 908 "wavegen.cpp"
                  var $75=$f; //@line 908 "wavegen.cpp"
                  var $add63=((($75)+($74))&4294967295); //@line 908 "wavegen.cpp"
                  $f=$add63; //@line 908 "wavegen.cpp"
                  var $76=$f; //@line 908 "wavegen.cpp"
                  var $77=$fhi; //@line 908 "wavegen.cpp"
                  var $cmp49=((($76))|0) < ((($77))|0); //@line 908 "wavegen.cpp"
                  if ($cmp49) { __label__ = 16;continue ; } else { __label__ = 17;break $for_inc65$$lor_lhs_false$18; } //@line 908 "wavegen.cpp"
                }
              }
            } while(0);
  
            var $78=$pk; //@line 894 "wavegen.cpp"
            var $inc66=((($78)+1)&4294967295); //@line 894 "wavegen.cpp"
            $pk=$inc66; //@line 894 "wavegen.cpp"
            var $79=$pk; //@line 894 "wavegen.cpp"
            var $80=HEAP32[((_wvoice)>>2)]; //@line 894 "wavegen.cpp"
            var $n_harmonic_peaks14=(($80+84)&4294967295); //@line 894 "wavegen.cpp"
            var $81=HEAP32[(($n_harmonic_peaks14)>>2)]; //@line 894 "wavegen.cpp"
            var $cmp15=((($79))|0) <= ((($81))|0); //@line 894 "wavegen.cpp"
            if ($cmp15) { __label__ = 9;continue $for_body16$16; } else { __label__ = 18;break $for_body16$$for_end67$15; } //@line 894 "wavegen.cpp"
          }
        }
      } while(0);
  
      var $82=$peaks_addr; //@line 918 "wavegen.cpp"
      var $arrayidx68=(($82+80)&4294967295); //@line 918 "wavegen.cpp"
      var $height69=(($arrayidx68+4)&4294967295); //@line 918 "wavegen.cpp"
      var $83=HEAP32[(($height69)>>2)]; //@line 918 "wavegen.cpp"
      var $mul70=((($83)*10)&4294967295); //@line 918 "wavegen.cpp"
      $y=$mul70; //@line 918 "wavegen.cpp"
      var $84=$pitch_addr; //@line 919 "wavegen.cpp"
      var $div71=((65536000/((($84))|0))|0); //@line 919 "wavegen.cpp"
      $h2=$div71; //@line 919 "wavegen.cpp"
      var $cmp72=((($div71))|0) > 0; //@line 920 "wavegen.cpp"
      if ($cmp72) { __label__ = 19;; } else { __label__ = 21;; } //@line 920 "wavegen.cpp"
      $if_then73$$for_cond81thread_pre_split$32: do { 
        if (__label__ == 19) {
  
          var $85=$y; //@line 922 "wavegen.cpp"
          var $86=$h2; //@line 922 "wavegen.cpp"
          var $div74=((((($85))|0)/((($86))|0))|0); //@line 922 "wavegen.cpp"
          $x=$div74; //@line 922 "wavegen.cpp"
          $h=1; //@line 923 "wavegen.cpp"
          var $cmp753=((($85))|0) > 0; //@line 924 "wavegen.cpp"
          if (!($cmp753)) { __label__ = 21;break $if_then73$$for_cond81thread_pre_split$32; } //@line 924 "wavegen.cpp"
          while(1) { 
  
            var $87=$y; //@line 926 "wavegen.cpp"
            var $88=$h; //@line 926 "wavegen.cpp"
            var $inc76=((($88)+1)&4294967295); //@line 926 "wavegen.cpp"
            $h=$inc76; //@line 926 "wavegen.cpp"
            var $89=$htab_addr; //@line 926 "wavegen.cpp"
            var $arrayidx77=(($89+4*$88)&4294967295); //@line 926 "wavegen.cpp"
            var $90=HEAP32[(($arrayidx77)>>2)]; //@line 926 "wavegen.cpp"
            var $add78=((($90)+($87))&4294967295); //@line 926 "wavegen.cpp"
            HEAP32[(($arrayidx77)>>2)]=$add78; //@line 926 "wavegen.cpp"
            var $91=$x; //@line 927 "wavegen.cpp"
            var $92=$y; //@line 927 "wavegen.cpp"
            var $sub79=((($92)-($91))&4294967295); //@line 927 "wavegen.cpp"
            $y=$sub79; //@line 927 "wavegen.cpp"
            var $cmp75=((($sub79))|0) > 0; //@line 924 "wavegen.cpp"
            if ($cmp75) { __label__ = 20;continue ; } else { __label__ = 21;break $if_then73$$for_cond81thread_pre_split$32; } //@line 924 "wavegen.cpp"
          }
        }
      } while(0);
  
      var $_pr=$pk; //@line 933 "wavegen.cpp"
      var $cmp822=((($_pr))|0) < 9; //@line 933 "wavegen.cpp"
      if ($cmp822) { __label__ = 22;; } else { __label__ = 27;; } //@line 933 "wavegen.cpp"
      $for_body83$$for_end105$37: do { 
        if (__label__ == 22) {
          while(1) { 
  
            var $93=$pk; //@line 935 "wavegen.cpp"
            var $94=$peaks_addr; //@line 935 "wavegen.cpp"
            var $arrayidx84=(($94+80*$93)&4294967295); //@line 935 "wavegen.cpp"
            var $height85=(($arrayidx84+4)&4294967295); //@line 935 "wavegen.cpp"
            var $95=HEAP32[(($height85)>>2)]; //@line 935 "wavegen.cpp"
            var $shr86=($95) >> 14; //@line 935 "wavegen.cpp"
            $x=$shr86; //@line 935 "wavegen.cpp"
            var $96=$x; //@line 936 "wavegen.cpp"
            var $97=$x; //@line 936 "wavegen.cpp"
            var $mul87=((($96)*5)&4294967295); //@line 936 "wavegen.cpp"
            var $mul88=((($mul87)*($97))&4294967295); //@line 936 "wavegen.cpp"
            var $div89=((((($mul88))|0)/2)|0); //@line 936 "wavegen.cpp"
            var $98=$pk; //@line 936 "wavegen.cpp"
            var $arrayidx90=((__ZL11peak_height+$98*4)&4294967295); //@line 936 "wavegen.cpp"
            HEAP32[(($arrayidx90)>>2)]=$div89; //@line 936 "wavegen.cpp"
            var $99=$control_addr; //@line 939 "wavegen.cpp"
            var $cmp91=((($99))|0)==0; //@line 939 "wavegen.cpp"
            if ($cmp91) { __label__ = 23;; } else { __label__ = 24;; } //@line 939 "wavegen.cpp"
            if (__label__ == 23) {
  
              var $100=$pk; //@line 942 "wavegen.cpp"
              var $101=$peaks_addr; //@line 942 "wavegen.cpp"
              var $arrayidx93=(($101+80*$100)&4294967295); //@line 942 "wavegen.cpp"
              var $freq94=(($arrayidx93)&4294967295); //@line 942 "wavegen.cpp"
              var $102=HEAP32[(($freq94)>>2)]; //@line 942 "wavegen.cpp"
              var $103=$pitch_addr; //@line 942 "wavegen.cpp"
              var $div95=((((($102))|0)/((($103))|0))|0); //@line 942 "wavegen.cpp"
              var $104=$pk; //@line 942 "wavegen.cpp"
              var $arrayidx96=((__ZL13peak_harmonic+$104*4)&4294967295); //@line 942 "wavegen.cpp"
              HEAP32[(($arrayidx96)>>2)]=$div95; //@line 942 "wavegen.cpp"
              ; //@line 943 "wavegen.cpp"
            }
  
            var $105=$pk; //@line 945 "wavegen.cpp"
            var $arrayidx98=((__ZL13peak_harmonic+$105*4)&4294967295); //@line 945 "wavegen.cpp"
            var $106=HEAP32[(($arrayidx98)>>2)]; //@line 945 "wavegen.cpp"
            var $107=$hmax_samplerate; //@line 945 "wavegen.cpp"
            var $cmp99=((($106))|0) >= ((($107))|0); //@line 945 "wavegen.cpp"
            if ($cmp99) { __label__ = 25;; } else { __label__ = 26;; } //@line 945 "wavegen.cpp"
            if (__label__ == 25) {
  
              var $108=$pk; //@line 946 "wavegen.cpp"
              var $arrayidx101=((__ZL11peak_height+$108*4)&4294967295); //@line 946 "wavegen.cpp"
              HEAP32[(($arrayidx101)>>2)]=0; //@line 946 "wavegen.cpp"
              ; //@line 946 "wavegen.cpp"
            }
  
            var $109=$pk; //@line 933 "wavegen.cpp"
            var $inc104=((($109)+1)&4294967295); //@line 933 "wavegen.cpp"
            $pk=$inc104; //@line 933 "wavegen.cpp"
            var $cmp82=((($inc104))|0) < 9; //@line 933 "wavegen.cpp"
            if ($cmp82) { __label__ = 22;continue ; } else { __label__ = 27;break $for_body83$$for_end105$37; } //@line 933 "wavegen.cpp"
          }
        }
      } while(0);
  
      $f=0; //@line 950 "wavegen.cpp"
      $h=0; //@line 951 "wavegen.cpp"
      var $110=$h; //@line 951 "wavegen.cpp"
      var $111=$hmax; //@line 951 "wavegen.cpp"
      var $cmp1071=((($110))|0) <= ((($111))|0); //@line 951 "wavegen.cpp"
      if ($cmp1071) { __label__ = 28;; } else { __label__ = 31;; } //@line 951 "wavegen.cpp"
      $for_body108$$for_end127$47: do { 
        if (__label__ == 28) {
          while(1) { 
  
            var $112=$h; //@line 953 "wavegen.cpp"
            var $113=$htab_addr; //@line 953 "wavegen.cpp"
            var $arrayidx109=(($113+4*$112)&4294967295); //@line 953 "wavegen.cpp"
            var $114=HEAP32[(($arrayidx109)>>2)]; //@line 953 "wavegen.cpp"
            var $shr110=($114) >> 15; //@line 953 "wavegen.cpp"
            $x=$shr110; //@line 953 "wavegen.cpp"
            var $115=$x; //@line 954 "wavegen.cpp"
            var $116=$x; //@line 954 "wavegen.cpp"
            var $mul111=((($116)*($115))&4294967295); //@line 954 "wavegen.cpp"
            var $shr112=($mul111) >> 8; //@line 954 "wavegen.cpp"
            var $117=$h; //@line 954 "wavegen.cpp"
            var $118=$htab_addr; //@line 954 "wavegen.cpp"
            var $arrayidx113=(($118+4*$117)&4294967295); //@line 954 "wavegen.cpp"
            HEAP32[(($arrayidx113)>>2)]=$shr112; //@line 954 "wavegen.cpp"
            var $119=$f; //@line 956 "wavegen.cpp"
            var $shr114=($119) >> 19; //@line 956 "wavegen.cpp"
            $ix=$shr114; //@line 956 "wavegen.cpp"
            var $cmp115=((($shr114))|0) < 1000; //@line 956 "wavegen.cpp"
            if ($cmp115) { __label__ = 29;; } else { __label__ = 30;; } //@line 956 "wavegen.cpp"
            if (__label__ == 29) {
  
              var $120=$h; //@line 958 "wavegen.cpp"
              var $121=$htab_addr; //@line 958 "wavegen.cpp"
              var $arrayidx117=(($121+4*$120)&4294967295); //@line 958 "wavegen.cpp"
              var $122=HEAP32[(($arrayidx117)>>2)]; //@line 958 "wavegen.cpp"
              var $123=$ix; //@line 958 "wavegen.cpp"
              var $124=HEAP32[((_wvoice)>>2)]; //@line 958 "wavegen.cpp"
              var $tone_adjust=(($124+344)&4294967295); //@line 958 "wavegen.cpp"
              var $arrayidx118=(($tone_adjust+$123)&4294967295); //@line 958 "wavegen.cpp"
              var $125=HEAPU8[($arrayidx118)]; //@line 958 "wavegen.cpp"
              var $conv119=((($125))&255); //@line 958 "wavegen.cpp"
              var $mul120=((($conv119)*($122))&4294967295); //@line 958 "wavegen.cpp"
              var $shr121=($mul120) >> 13; //@line 958 "wavegen.cpp"
              var $126=$h; //@line 958 "wavegen.cpp"
              var $127=$htab_addr; //@line 958 "wavegen.cpp"
              var $arrayidx122=(($127+4*$126)&4294967295); //@line 958 "wavegen.cpp"
              HEAP32[(($arrayidx122)>>2)]=$shr121; //@line 958 "wavegen.cpp"
              ; //@line 959 "wavegen.cpp"
            }
  
            var $128=$h; //@line 951 "wavegen.cpp"
            var $inc125=((($128)+1)&4294967295); //@line 951 "wavegen.cpp"
            $h=$inc125; //@line 951 "wavegen.cpp"
            var $129=$pitch_addr; //@line 951 "wavegen.cpp"
            var $130=$f; //@line 951 "wavegen.cpp"
            var $add126=((($130)+($129))&4294967295); //@line 951 "wavegen.cpp"
            $f=$add126; //@line 951 "wavegen.cpp"
            var $131=$h; //@line 951 "wavegen.cpp"
            var $132=$hmax; //@line 951 "wavegen.cpp"
            var $cmp107=((($131))|0) <= ((($132))|0); //@line 951 "wavegen.cpp"
            if ($cmp107) { __label__ = 28;continue ; } else { __label__ = 31;break $for_body108$$for_end127$47; } //@line 951 "wavegen.cpp"
          }
        }
      } while(0);
  
      var $133=$htab_addr; //@line 963 "wavegen.cpp"
      var $arrayidx128=(($133+4)&4294967295); //@line 963 "wavegen.cpp"
      var $134=HEAP32[(($arrayidx128)>>2)]; //@line 963 "wavegen.cpp"
      var $_b=HEAPU8[(__ZL16option_harmonic1_b)];
      var $135=($_b) ? 6 : 10;
      var $mul129=((($135)*($134))&4294967295); //@line 963 "wavegen.cpp"
      $h1=$mul129; //@line 963 "wavegen.cpp"
      var $136=$h1; //@line 964 "wavegen.cpp"
      var $div130=((((($136))|0)/8)|0); //@line 964 "wavegen.cpp"
      var $137=$htab_addr; //@line 964 "wavegen.cpp"
      var $arrayidx131=(($137+4)&4294967295); //@line 964 "wavegen.cpp"
      HEAP32[(($arrayidx131)>>2)]=$div130; //@line 964 "wavegen.cpp"
      var $138=$control_addr; //@line 968 "wavegen.cpp"
      var $and=($138) & 1; //@line 968 "wavegen.cpp"
      var $tobool=((($and))|0)!=0; //@line 968 "wavegen.cpp"
      if ($tobool) { __label__ = 32;; } else { __label__ = 34;; } //@line 968 "wavegen.cpp"
      $if_then132$$if_end144$54: do { 
        if (__label__ == 32) {
  
          $h=1; //@line 970 "wavegen.cpp"
          ; //@line 970 "wavegen.cpp"
          while(1) { 
  
            var $139=$h; //@line 972 "wavegen.cpp"
            var $140=$htab_addr; //@line 972 "wavegen.cpp"
            var $arrayidx136=(($140+4*$139)&4294967295); //@line 972 "wavegen.cpp"
            var $141=HEAP32[(($arrayidx136)>>2)]; //@line 972 "wavegen.cpp"
            var $142=$h; //@line 972 "wavegen.cpp"
            var $143=HEAP32[((__ZL9harmspect)>>2)]; //@line 972 "wavegen.cpp"
            var $arrayidx137=(($143+4*$142)&4294967295); //@line 972 "wavegen.cpp"
            var $144=HEAP32[(($arrayidx137)>>2)]; //@line 972 "wavegen.cpp"
            var $sub138=((($141)-($144))&4294967295); //@line 972 "wavegen.cpp"
            var $shr139=($sub138) >> 3; //@line 972 "wavegen.cpp"
            var $145=$h; //@line 972 "wavegen.cpp"
            var $arrayidx140=((__ZL8harm_inc+$145*4)&4294967295); //@line 972 "wavegen.cpp"
            HEAP32[(($arrayidx140)>>2)]=$shr139; //@line 972 "wavegen.cpp"
            var $146=$h; //@line 970 "wavegen.cpp"
            var $inc142=((($146)+1)&4294967295); //@line 970 "wavegen.cpp"
            $h=$inc142; //@line 970 "wavegen.cpp"
            var $cmp134=((($inc142))|0) < 30; //@line 970 "wavegen.cpp"
            if ($cmp134) { __label__ = 33;continue ; } else { __label__ = 34;break $if_then132$$if_end144$54; } //@line 970 "wavegen.cpp"
          }
        }
      } while(0);
  
      var $147=$hmax; //@line 976 "wavegen.cpp"
      $retval=$147; //@line 976 "wavegen.cpp"
      ; //@line 976 "wavegen.cpp"
    }
  
    var $148=$retval; //@line 977 "wavegen.cpp"
    ;
    return $148; //@line 977 "wavegen.cpp"
    return null;
  }
  

  function __Z10InitBreathv() {
    ;
    var __label__;
  
    var $ix;
    var $0=HEAP32[((_samplerate)>>2)]; //@line 1104 "wavegen.cpp"
    var $conv=((($0))|0); //@line 1104 "wavegen.cpp"
    var $div=-3.1415927/($conv); //@line 1104 "wavegen.cpp"
    HEAPF32[((__ZL10minus_pi_t)>>2)]=$div; //@line 1104 "wavegen.cpp"
    var $1=HEAPF32[((__ZL10minus_pi_t)>>2)]; //@line 1105 "wavegen.cpp"
    var $mul=-2*($1); //@line 1105 "wavegen.cpp"
    HEAPF32[((__ZL8two_pi_t)>>2)]=$mul; //@line 1105 "wavegen.cpp"
    $ix=0; //@line 1107 "wavegen.cpp"
    ; //@line 1107 "wavegen.cpp"
    while(1) { 
  
      var $2=$ix; //@line 1109 "wavegen.cpp"
      var $arrayidx=((__ZL7rbreath+$2*40)&4294967295); //@line 1109 "wavegen.cpp"
      __ZL12setresonatorP9RESONATORiii($arrayidx, 2000, 200, 1); //@line 1109 "wavegen.cpp"
      var $3=$ix; //@line 1107 "wavegen.cpp"
      var $inc=((($3)+1)&4294967295); //@line 1107 "wavegen.cpp"
      $ix=$inc; //@line 1107 "wavegen.cpp"
      var $cmp=((($inc))|0) < 9; //@line 1107 "wavegen.cpp"
      if ($cmp) { __label__ = 1;continue ; } else { __label__ = 2;break ; } //@line 1107 "wavegen.cpp"
    }
  
    ;
    return; //@line 1112 "wavegen.cpp"
    return;
  }
  

  function __ZL12setresonatorP9RESONATORiii($rp, $freq, $bwidth, $init) {
    ;
    var __label__;
  
    var $rp_addr;
    var $freq_addr;
    var $bwidth_addr;
    var $init_addr;
    var $x;
    var $arg;
    $rp_addr=$rp;
    $freq_addr=$freq;
    $bwidth_addr=$bwidth;
    $init_addr=$init;
    var $0=$init_addr; //@line 1075 "wavegen.cpp"
    var $tobool=((($0))|0)!=0; //@line 1075 "wavegen.cpp"
    if ($tobool) { __label__ = 1;; } else { __label__ = 2;; } //@line 1075 "wavegen.cpp"
    if (__label__ == 1) {
  
      var $1=$rp_addr; //@line 1077 "wavegen.cpp"
      var $x1=(($1+24)&4294967295); //@line 1077 "wavegen.cpp"
      HEAPF32[(($x1)>>2)]=0; //@line 1077 "wavegen.cpp"
      var $2=$rp_addr; //@line 1078 "wavegen.cpp"
      var $x2=(($2+32)&4294967295); //@line 1078 "wavegen.cpp"
      HEAPF32[(($x2)>>2)]=0; //@line 1078 "wavegen.cpp"
      ; //@line 1079 "wavegen.cpp"
    }
  
    var $3=HEAPF32[((__ZL10minus_pi_t)>>2)]; //@line 1082 "wavegen.cpp"
    var $4=$bwidth_addr; //@line 1082 "wavegen.cpp"
    var $conv=((($4))|0); //@line 1082 "wavegen.cpp"
    var $mul=($3)*($conv); //@line 1082 "wavegen.cpp"
    $arg=$mul; //@line 1082 "wavegen.cpp"
    var $5=$arg; //@line 1083 "wavegen.cpp"
    var $call=_exp($5); //@line 1083 "wavegen.cpp"
    $x=$call; //@line 1083 "wavegen.cpp"
    var $6=$x; //@line 1086 "wavegen.cpp"
    var $7=$x; //@line 1086 "wavegen.cpp"
    var $mul1=($6)*($7); //@line 1086 "wavegen.cpp"
    var $sub=(-($mul1)); //@line 1086 "wavegen.cpp"
    var $8=$rp_addr; //@line 1086 "wavegen.cpp"
    var $c=(($8+16)&4294967295); //@line 1086 "wavegen.cpp"
    HEAPF32[(($c)>>2)]=$sub; //@line 1086 "wavegen.cpp"
    var $9=HEAPF32[((__ZL8two_pi_t)>>2)]; //@line 1090 "wavegen.cpp"
    var $10=$freq_addr; //@line 1090 "wavegen.cpp"
    var $conv2=((($10))|0); //@line 1090 "wavegen.cpp"
    var $mul3=($9)*($conv2); //@line 1090 "wavegen.cpp"
    $arg=$mul3; //@line 1090 "wavegen.cpp"
    var $11=$x; //@line 1091 "wavegen.cpp"
    var $12=$arg; //@line 1091 "wavegen.cpp"
    var $call4=_cos($12); //@line 1091 "wavegen.cpp"
    var $mul5=($11)*($call4); //@line 1091 "wavegen.cpp"
    var $mul6=($mul5)*2; //@line 1091 "wavegen.cpp"
    var $13=$rp_addr; //@line 1091 "wavegen.cpp"
    var $b=(($13+8)&4294967295); //@line 1091 "wavegen.cpp"
    HEAPF32[(($b)>>2)]=$mul6; //@line 1091 "wavegen.cpp"
    var $14=$rp_addr; //@line 1094 "wavegen.cpp"
    var $b7=(($14+8)&4294967295); //@line 1094 "wavegen.cpp"
    var $15=HEAPF32[(($b7)>>2)]; //@line 1094 "wavegen.cpp"
    var $sub8=1-($15); //@line 1094 "wavegen.cpp"
    var $16=$rp_addr; //@line 1094 "wavegen.cpp"
    var $c9=(($16+16)&4294967295); //@line 1094 "wavegen.cpp"
    var $17=HEAPF32[(($c9)>>2)]; //@line 1094 "wavegen.cpp"
    var $sub10=($sub8)-($17); //@line 1094 "wavegen.cpp"
    var $18=$rp_addr; //@line 1094 "wavegen.cpp"
    var $a=(($18)&4294967295); //@line 1094 "wavegen.cpp"
    HEAPF32[(($a)>>2)]=$sub10; //@line 1094 "wavegen.cpp"
    ;
    return; //@line 1095 "wavegen.cpp"
    return;
  }
  

  function __Z7Wavegenv() {
    ;
    var __label__;
    var __lastLabel__ = null;
  
    var $retval;
    var $waveph;
    var $theta;
    var $total;
    var $h;
    var $ix;
    var $z;
    var $z1;
    var $z2;
    var $echo;
    var $ov;
    var $pk;
    var $c;
    var $sample;
    var $amp;
    var $modn_amp;
    var $modn_period;
    ; //@line 1186 "wavegen.cpp"
    $for_cond$2: while(1) { 
  
      var $_b=HEAPU8[(__ZL8end_wave_b)];
      var $0=((($_b))&1);
      var $cmp=((($0))|0)==0; //@line 1188 "wavegen.cpp"
      var $1=HEAP32[((__ZL11samplecount)>>2)]; //@line 1188 "wavegen.cpp"
      var $2=HEAP32[((__ZL8nsamples)>>2)]; //@line 1188 "wavegen.cpp"
      var $cmp1=((($1))|0)==((($2))|0); //@line 1188 "wavegen.cpp"
      var $or_cond3=($cmp) & ($cmp1); //@line 1188 "wavegen.cpp"
      if ($or_cond3) { __label__ = 2;break $for_cond$2; } //@line 1188 "wavegen.cpp"
  
      var $and=($1) & 63; //@line 1191 "wavegen.cpp"
      var $cmp2=((($and))|0)==0; //@line 1191 "wavegen.cpp"
      if ($cmp2) { __label__ = 4;; } else { __label__ = 8;; } //@line 1191 "wavegen.cpp"
      $if_then3$$if_else19$5: do { 
        if (__label__ == 4) {
  
          var $cmp4=((($1))|0)==0; //@line 1194 "wavegen.cpp"
          if ($cmp4) { __label__ = 5;; } else { __label__ = 6;; } //@line 1194 "wavegen.cpp"
          if (__label__ == 5) {
  
            HEAP32[((__ZL7hswitch)>>2)]=0; //@line 1196 "wavegen.cpp"
            HEAP32[((__ZL9harmspect)>>2)]=((__ZL6hspect)&4294967295); //@line 1197 "wavegen.cpp"
            var $3=HEAP32[((((_wdata+4)&4294967295))>>2)]; //@line 1198 "wavegen.cpp"
            var $shl=($3) << 4; //@line 1198 "wavegen.cpp"
            var $call=__Z16PeaksToHarmspectP15wavegen_peaks_tiPii(((__ZL5peaks)&4294967295), $shl, ((__ZL6hspect)&4294967295), 0); //@line 1198 "wavegen.cpp"
            HEAP32[((__ZZ7WavegenvE5maxh2)>>2)]=$call; //@line 1198 "wavegen.cpp"
            var $4=HEAP32[((((_wdata+52)&4294967295))>>2)]; //@line 1202 "wavegen.cpp"
            var $5=HEAP32[((((_wdata+4)&4294967295))>>2)]; //@line 1202 "wavegen.cpp"
            var $shr=($5) >> 8; //@line 1202 "wavegen.cpp"
            var $6=HEAP32[((((_wdata+60)&4294967295))>>2)]; //@line 1202 "wavegen.cpp"
            var $mul=((($6)*($4))&4294967295); //@line 1202 "wavegen.cpp"
            var $mul6=((($mul)*($shr))&4294967295); //@line 1202 "wavegen.cpp"
            var $div=((((($mul6))|0)/80000)|0); //@line 1202 "wavegen.cpp"
            HEAP32[((__ZZ7WavegenvE10amplitude2)>>2)]=$div; //@line 1202 "wavegen.cpp"
            var $7=HEAP32[((((_wdata+4)&4294967295))>>2)]; //@line 1205 "wavegen.cpp"
            var $shr7=($7) >> 12; //@line 1205 "wavegen.cpp"
            var $div8=((890/((($shr7))|0))|0); //@line 1205 "wavegen.cpp"
            HEAP32[((__ZZ7WavegenvE13h_switch_sign)>>2)]=$div8; //@line 1205 "wavegen.cpp"
            ; //@line 1206 "wavegen.cpp"
          }
          else if (__label__ == 6) {
  
            __ZL17AdvanceParametersv(); //@line 1208 "wavegen.cpp"
            ;
          }
  
          var $8=HEAP32[((((_wdata+4)&4294967295))>>2)]; //@line 1211 "wavegen.cpp"
          var $shr10=($8) >> 7; //@line 1211 "wavegen.cpp"
          var $9=HEAP32[((__ZL16PHASE_INC_FACTOR)>>2)]; //@line 1211 "wavegen.cpp"
          var $mul11=((($shr10)*($9))&4294967295); //@line 1211 "wavegen.cpp"
          HEAP32[((__ZL8phaseinc)>>2)]=$mul11; //@line 1211 "wavegen.cpp"
          var $10=HEAP32[((_samplerate)>>2)]; //@line 1212 "wavegen.cpp"
          var $11=HEAP32[((((_wdata+4)&4294967295))>>2)]; //@line 1212 "wavegen.cpp"
          var $shr12=($11) >> 12; //@line 1212 "wavegen.cpp"
          var $div13=((((($10))|0)/((($shr12))|0))|0); //@line 1212 "wavegen.cpp"
          HEAP32[((__ZL13cycle_samples)>>2)]=$div13; //@line 1212 "wavegen.cpp"
          var $12=HEAP32[((((_wdata+4)&4294967295))>>2)]; //@line 1213 "wavegen.cpp"
          var $shr14=($12) >> 11; //@line 1213 "wavegen.cpp"
          HEAP32[((__ZL9hf_factor)>>2)]=$shr14; //@line 1213 "wavegen.cpp"
          var $13=HEAP32[((__ZZ7WavegenvE5maxh2)>>2)]; //@line 1215 "wavegen.cpp"
          HEAP32[((__ZZ7WavegenvE4maxh)>>2)]=$13; //@line 1215 "wavegen.cpp"
          var $14=HEAP32[((__ZL7hswitch)>>2)]; //@line 1216 "wavegen.cpp"
          var $arrayidx=((__ZL6hspect+$14*1600)&4294967295); //@line 1216 "wavegen.cpp"
          var $arraydecay=(($arrayidx)&4294967295); //@line 1216 "wavegen.cpp"
          HEAP32[((__ZL9harmspect)>>2)]=$arraydecay; //@line 1216 "wavegen.cpp"
          var $15=HEAP32[((__ZL7hswitch)>>2)]; //@line 1217 "wavegen.cpp"
          var $xor=($15) ^ 1; //@line 1217 "wavegen.cpp"
          HEAP32[((__ZL7hswitch)>>2)]=$xor; //@line 1217 "wavegen.cpp"
          var $16=HEAP32[((((_wdata+4)&4294967295))>>2)]; //@line 1218 "wavegen.cpp"
          var $shl15=($16) << 4; //@line 1218 "wavegen.cpp"
          var $17=HEAP32[((__ZL7hswitch)>>2)]; //@line 1218 "wavegen.cpp"
          var $arrayidx16=((__ZL6hspect+$17*1600)&4294967295); //@line 1218 "wavegen.cpp"
          var $arraydecay17=(($arrayidx16)&4294967295); //@line 1218 "wavegen.cpp"
          var $call18=__Z16PeaksToHarmspectP15wavegen_peaks_tiPii(((__ZL5peaks)&4294967295), $shl15, $arraydecay17, 1); //@line 1218 "wavegen.cpp"
          HEAP32[((__ZZ7WavegenvE5maxh2)>>2)]=$call18; //@line 1218 "wavegen.cpp"
          __ZL9SetBreathv(); //@line 1220 "wavegen.cpp"
          ; //@line 1221 "wavegen.cpp"
        }
        else if (__label__ == 8) {
  
          var $and20=($1) & 7; //@line 1223 "wavegen.cpp"
          var $cmp21=((($and20))|0)==0; //@line 1223 "wavegen.cpp"
          if (!($cmp21)) { __label__ = 16;break $if_then3$$if_else19$5; } //@line 1223 "wavegen.cpp"
  
          $h=1; //@line 1225 "wavegen.cpp"
          __lastLabel__ = 9; ; //@line 1225 "wavegen.cpp"
          while(1) { 
  
            var $18=__lastLabel__ == 13 ? $inc : (1); //@line 1225 "wavegen.cpp"
            var $cmp24=((($18))|0) < 30; //@line 1225 "wavegen.cpp"
            if (!($cmp24)) { __label__ = 14;break ; } //@line 1225 "wavegen.cpp"
  
            var $19=$h; //@line 1225 "wavegen.cpp"
            var $20=HEAP32[((__ZZ7WavegenvE5maxh2)>>2)]; //@line 1225 "wavegen.cpp"
            var $cmp26=((($19))|0) <= ((($20))|0); //@line 1225 "wavegen.cpp"
            if (!($cmp26)) { __label__ = 14;break ; } //@line 1225 "wavegen.cpp"
  
            var $21=$h; //@line 1225 "wavegen.cpp"
            var $22=HEAP32[((__ZZ7WavegenvE4maxh)>>2)]; //@line 1225 "wavegen.cpp"
            var $cmp27=((($21))|0) <= ((($22))|0); //@line 1225 "wavegen.cpp"
            if (!($cmp27)) { __label__ = 14;break ; }
  
            var $23=$h; //@line 1227 "wavegen.cpp"
            var $arrayidx28=((__ZL8harm_inc+$23*4)&4294967295); //@line 1227 "wavegen.cpp"
            var $24=HEAP32[(($arrayidx28)>>2)]; //@line 1227 "wavegen.cpp"
            var $25=$h; //@line 1227 "wavegen.cpp"
            var $26=HEAP32[((__ZL9harmspect)>>2)]; //@line 1227 "wavegen.cpp"
            var $arrayidx29=(($26+4*$25)&4294967295); //@line 1227 "wavegen.cpp"
            var $27=HEAP32[(($arrayidx29)>>2)]; //@line 1227 "wavegen.cpp"
            var $add=((($27)+($24))&4294967295); //@line 1227 "wavegen.cpp"
            HEAP32[(($arrayidx29)>>2)]=$add; //@line 1227 "wavegen.cpp"
            var $28=$h; //@line 1225 "wavegen.cpp"
            var $inc=((($28)+1)&4294967295); //@line 1225 "wavegen.cpp"
            $h=$inc; //@line 1225 "wavegen.cpp"
            __lastLabel__ = 13; __label__ = 10;continue ; //@line 1225 "wavegen.cpp"
          }
  
          var $29=HEAP32[((__ZZ7WavegenvE3agc)>>2)]; //@line 1231 "wavegen.cpp"
          var $cmp30=((($29))|0) < 256; //@line 1231 "wavegen.cpp"
          if (!($cmp30)) { __label__ = 16;break $if_then3$$if_else19$5; } //@line 1231 "wavegen.cpp"
  
          var $30=HEAP32[((__ZZ7WavegenvE3agc)>>2)]; //@line 1231 "wavegen.cpp"
          var $inc32=((($30)+1)&4294967295); //@line 1231 "wavegen.cpp"
          HEAP32[((__ZZ7WavegenvE3agc)>>2)]=$inc32; //@line 1231 "wavegen.cpp"
          ; //@line 1231 "wavegen.cpp"
        }
      } while(0);
  
      var $31=HEAP32[((__ZL11samplecount)>>2)]; //@line 1234 "wavegen.cpp"
      var $inc36=((($31)+1)&4294967295); //@line 1234 "wavegen.cpp"
      HEAP32[((__ZL11samplecount)>>2)]=$inc36; //@line 1234 "wavegen.cpp"
      var $32=HEAP32[((__ZL9wavephase)>>2)]; //@line 1236 "wavegen.cpp"
      var $cmp37=((($32))|0) > 0; //@line 1236 "wavegen.cpp"
      var $33=HEAP32[((__ZL8phaseinc)>>2)]; //@line 1238 "wavegen.cpp"
      var $34=HEAP32[((__ZL9wavephase)>>2)]; //@line 1238 "wavegen.cpp"
      var $add39=((($34)+($33))&4294967295); //@line 1238 "wavegen.cpp"
      HEAP32[((__ZL9wavephase)>>2)]=$add39; //@line 1238 "wavegen.cpp"
      var $35=HEAP32[((__ZL9wavephase)>>2)]; //@line 1239 "wavegen.cpp"
      var $cmp40=((($35))|0) < 0; //@line 1239 "wavegen.cpp"
      var $or_cond=($cmp37) & ($cmp40); //@line 1236 "wavegen.cpp"
      if ($or_cond) { __label__ = 17;; } else { __label__ = 38;; } //@line 1236 "wavegen.cpp"
      $if_then41$$if_end118$21: do { 
        if (__label__ == 17) {
  
          var $36=HEAP32[((__ZL15wavemult_offset)>>2)]; //@line 1242 "wavegen.cpp"
          var $37=HEAP32[((__ZL13cycle_samples)>>2)]; //@line 1242 "wavegen.cpp"
          var $div42=((((($37))|0)/2)|0); //@line 1242 "wavegen.cpp"
          var $sub=((($36)-($div42))&4294967295); //@line 1242 "wavegen.cpp"
          HEAP32[((__ZL6cbytes)>>2)]=$sub; //@line 1242 "wavegen.cpp"
          var $38=HEAP32[((__ZL11samplecount)>>2)]; //@line 1243 "wavegen.cpp"
          var $39=HEAP32[((__ZL8nsamples)>>2)]; //@line 1243 "wavegen.cpp"
          var $cmp43=((($38))|0) > ((($39))|0); //@line 1243 "wavegen.cpp"
          if ($cmp43) { __label__ = 18;break $for_cond$2; } //@line 1243 "wavegen.cpp"
  
          var $40=HEAP32[((__ZZ7WavegenvE11cycle_count)>>2)]; //@line 1246 "wavegen.cpp"
          var $inc46=((($40)+1)&4294967295); //@line 1246 "wavegen.cpp"
          HEAP32[((__ZZ7WavegenvE11cycle_count)>>2)]=$inc46; //@line 1246 "wavegen.cpp"
          var $41=HEAP32[((_wvoice)>>2)]; //@line 1248 "wavegen.cpp"
          var $n_harmonic_peaks=(($41+84)&4294967295); //@line 1248 "wavegen.cpp"
          var $42=HEAP32[(($n_harmonic_peaks)>>2)]; //@line 1248 "wavegen.cpp"
          var $add47=((($42)+1)&4294967295); //@line 1248 "wavegen.cpp"
          $pk=$add47; //@line 1248 "wavegen.cpp"
          var $cmp495=((($add47))|0) < 9; //@line 1248 "wavegen.cpp"
          if ($cmp495) { __label__ = 20;; } else { __label__ = 21;; } //@line 1248 "wavegen.cpp"
          $for_body50$$for_end57$24: do { 
            if (__label__ == 20) {
              while(1) { 
  
                var $43=$pk; //@line 1251 "wavegen.cpp"
                var $arrayidx51=((__ZL5peaks+$43*80)&4294967295); //@line 1251 "wavegen.cpp"
                var $freq=(($arrayidx51)&4294967295); //@line 1251 "wavegen.cpp"
                var $44=HEAP32[(($freq)>>2)]; //@line 1251 "wavegen.cpp"
                var $45=HEAP32[((((_wdata+4)&4294967295))>>2)]; //@line 1251 "wavegen.cpp"
                var $mul52=((($45)*16)&4294967295); //@line 1251 "wavegen.cpp"
                var $div53=((((($44))|0)/((($mul52))|0))|0); //@line 1251 "wavegen.cpp"
                var $46=$pk; //@line 1251 "wavegen.cpp"
                var $arrayidx54=((__ZL13peak_harmonic+$46*4)&4294967295); //@line 1251 "wavegen.cpp"
                HEAP32[(($arrayidx54)>>2)]=$div53; //@line 1251 "wavegen.cpp"
                var $47=$pk; //@line 1248 "wavegen.cpp"
                var $inc56=((($47)+1)&4294967295); //@line 1248 "wavegen.cpp"
                $pk=$inc56; //@line 1248 "wavegen.cpp"
                var $cmp49=((($inc56))|0) < 9; //@line 1248 "wavegen.cpp"
                if ($cmp49) { __label__ = 20;continue ; } else { __label__ = 21;break $for_body50$$for_end57$24; } //@line 1248 "wavegen.cpp"
              }
            }
          } while(0);
  
          var $48=HEAP32[((((_wdata+52)&4294967295))>>2)]; //@line 1256 "wavegen.cpp"
          var $49=HEAP32[((((_wdata+4)&4294967295))>>2)]; //@line 1256 "wavegen.cpp"
          var $shr58=($49) >> 8; //@line 1256 "wavegen.cpp"
          var $50=HEAP32[((((_wdata+60)&4294967295))>>2)]; //@line 1256 "wavegen.cpp"
          var $mul59=((($50)*($48))&4294967295); //@line 1256 "wavegen.cpp"
          var $mul60=((($mul59)*($shr58))&4294967295); //@line 1256 "wavegen.cpp"
          var $div61=((((($mul60))|0)/80000)|0); //@line 1256 "wavegen.cpp"
          HEAP32[((__ZZ7WavegenvE10amplitude2)>>2)]=$div61; //@line 1256 "wavegen.cpp"
          var $51=HEAP32[((__ZL12glottal_flag)>>2)]; //@line 1258 "wavegen.cpp"
          var $cmp62=((($51))|0) > 0; //@line 1258 "wavegen.cpp"
          if ($cmp62) { __label__ = 22;; } else { __label__ = 28;; } //@line 1258 "wavegen.cpp"
          $if_then63$$if_end81$28: do { 
            if (__label__ == 22) {
  
              var $52=HEAP32[((__ZL12glottal_flag)>>2)]; //@line 1260 "wavegen.cpp"
              var $cmp64=((($52))|0)==3; //@line 1260 "wavegen.cpp"
              if ($cmp64) { __label__ = 23;; } else { __label__ = 25;; } //@line 1260 "wavegen.cpp"
              if (__label__ == 23) {
  
                var $53=HEAP32[((__ZL8nsamples)>>2)]; //@line 1262 "wavegen.cpp"
                var $54=HEAP32[((__ZL11samplecount)>>2)]; //@line 1262 "wavegen.cpp"
                var $sub66=((($53)-($54))&4294967295); //@line 1262 "wavegen.cpp"
                var $55=HEAP32[((__ZL13cycle_samples)>>2)]; //@line 1262 "wavegen.cpp"
                var $mul67=((($55)*2)&4294967295); //@line 1262 "wavegen.cpp"
                var $cmp68=((($sub66))|0) < ((($mul67))|0); //@line 1262 "wavegen.cpp"
                if (!($cmp68)) { __label__ = 28;break $if_then63$$if_end81$28; } //@line 1262 "wavegen.cpp"
  
                HEAP32[((__ZL12glottal_flag)>>2)]=2; //@line 1266 "wavegen.cpp"
                var $56=HEAP32[((__ZZ7WavegenvE10amplitude2)>>2)]; //@line 1267 "wavegen.cpp"
                var $57=HEAP32[((__ZL14glottal_reduce)>>2)]; //@line 1267 "wavegen.cpp"
                var $mul70=((($57)*($56))&4294967295); //@line 1267 "wavegen.cpp"
                var $div71=((((($mul70))|0)/256)|0); //@line 1267 "wavegen.cpp"
                HEAP32[((__ZZ7WavegenvE10amplitude2)>>2)]=$div71; //@line 1267 "wavegen.cpp"
                ; //@line 1268 "wavegen.cpp"
              }
              else if (__label__ == 25) {
  
                var $58=HEAP32[((__ZL12glottal_flag)>>2)]; //@line 1271 "wavegen.cpp"
                var $cmp74=((($58))|0)==4; //@line 1271 "wavegen.cpp"
                if ($cmp74) { __label__ = 26;; } else { __label__ = 27;; } //@line 1271 "wavegen.cpp"
                if (__label__ == 26) {
  
                  HEAP32[((__ZL12glottal_flag)>>2)]=2; //@line 1275 "wavegen.cpp"
                  var $59=HEAP32[((__ZZ7WavegenvE10amplitude2)>>2)]; //@line 1276 "wavegen.cpp"
                  var $60=HEAP32[((__ZL14glottal_reduce)>>2)]; //@line 1276 "wavegen.cpp"
                  var $mul76=((($60)*($59))&4294967295); //@line 1276 "wavegen.cpp"
                  var $div77=((((($mul76))|0)/256)|0); //@line 1276 "wavegen.cpp"
                  HEAP32[((__ZZ7WavegenvE10amplitude2)>>2)]=$div77; //@line 1276 "wavegen.cpp"
                  ; //@line 1277 "wavegen.cpp"
                }
                else if (__label__ == 27) {
  
                  var $61=HEAP32[((__ZL12glottal_flag)>>2)]; //@line 1280 "wavegen.cpp"
                  var $dec=((($61)-1)&4294967295); //@line 1280 "wavegen.cpp"
                  HEAP32[((__ZL12glottal_flag)>>2)]=$dec; //@line 1280 "wavegen.cpp"
                  ;
                }
              }
            }
          } while(0);
  
          var $62=HEAP32[((__ZL13amplitude_env)>>2)]; //@line 1284 "wavegen.cpp"
          var $cmp82=((($62))|0)!=0; //@line 1284 "wavegen.cpp"
          if ($cmp82) { __label__ = 29;; } else { __label__ = 32;; } //@line 1284 "wavegen.cpp"
          if (__label__ == 29) {
  
            var $63=HEAP32[((__ZL6amp_ix)>>2)]; //@line 1287 "wavegen.cpp"
            var $shr84=($63) >> 8; //@line 1287 "wavegen.cpp"
            $ix=$shr84; //@line 1287 "wavegen.cpp"
            var $cmp85=((($shr84))|0) > 127; //@line 1287 "wavegen.cpp"
            if ($cmp85) { __label__ = 30;; } else { __label__ = 31;; } //@line 1287 "wavegen.cpp"
            if (__label__ == 30) {
  
              $ix=127; //@line 1287 "wavegen.cpp"
              ; //@line 1287 "wavegen.cpp"
            }
  
            var $64=$ix; //@line 1288 "wavegen.cpp"
            var $65=HEAP32[((__ZL13amplitude_env)>>2)]; //@line 1288 "wavegen.cpp"
            var $arrayidx88=(($65+$64)&4294967295); //@line 1288 "wavegen.cpp"
            var $66=HEAPU8[($arrayidx88)]; //@line 1288 "wavegen.cpp"
            var $conv=((($66))&255); //@line 1288 "wavegen.cpp"
            $amp=$conv; //@line 1288 "wavegen.cpp"
            var $67=HEAP32[((__ZZ7WavegenvE10amplitude2)>>2)]; //@line 1289 "wavegen.cpp"
            var $68=$amp; //@line 1289 "wavegen.cpp"
            var $mul89=((($68)*($67))&4294967295); //@line 1289 "wavegen.cpp"
            var $div90=((((($mul89))|0)/128)|0); //@line 1289 "wavegen.cpp"
            HEAP32[((__ZZ7WavegenvE10amplitude2)>>2)]=$div90; //@line 1289 "wavegen.cpp"
            ; //@line 1292 "wavegen.cpp"
          }
  
          $modn_period=0; //@line 1295 "wavegen.cpp"
          var $69=HEAP32[((_voice)>>2)]; //@line 1296 "wavegen.cpp"
          var $roughness=(($69+72)&4294967295); //@line 1296 "wavegen.cpp"
          var $70=HEAP32[(($roughness)>>2)]; //@line 1296 "wavegen.cpp"
          var $cmp92=((($70))|0) < 8; //@line 1296 "wavegen.cpp"
          if (!($cmp92)) { __label__ = 38;break $if_then41$$if_end118$21; } //@line 1296 "wavegen.cpp"
  
          var $71=HEAP32[((__ZL15modulation_type)>>2)]; //@line 1298 "wavegen.cpp"
          var $72=HEAP32[((_voice)>>2)]; //@line 1298 "wavegen.cpp"
          var $roughness94=(($72+72)&4294967295); //@line 1298 "wavegen.cpp"
          var $73=HEAP32[(($roughness94)>>2)]; //@line 1298 "wavegen.cpp"
          var $arrayidx95=((__ZL14modulation_tab+$73*8)&4294967295); //@line 1298 "wavegen.cpp"
          var $arrayidx96=(($arrayidx95+$71)&4294967295); //@line 1298 "wavegen.cpp"
          var $74=HEAPU8[($arrayidx96)]; //@line 1298 "wavegen.cpp"
          var $conv97=((($74))&255); //@line 1298 "wavegen.cpp"
          $modn_period=$conv97; //@line 1298 "wavegen.cpp"
          var $75=$modn_period; //@line 1299 "wavegen.cpp"
          var $and98=($75) & 15; //@line 1299 "wavegen.cpp"
          $modn_amp=$and98; //@line 1299 "wavegen.cpp"
          var $76=$modn_period; //@line 1300 "wavegen.cpp"
          var $shr99=($76) >> 4; //@line 1300 "wavegen.cpp"
          $modn_period=$shr99; //@line 1300 "wavegen.cpp"
          var $cmp101=((($shr99))|0)!=0; //@line 1303 "wavegen.cpp"
          if (!($cmp101)) { __label__ = 38;break $if_then41$$if_end118$21; } //@line 1303 "wavegen.cpp"
  
          var $77=$modn_period; //@line 1305 "wavegen.cpp"
          var $cmp103=((($77))|0)==15; //@line 1305 "wavegen.cpp"
          if ($cmp103) { __label__ = 35;; } else { __label__ = 36;; } //@line 1305 "wavegen.cpp"
          if (__label__ == 35) {
  
            var $78=HEAP32[((__ZZ7WavegenvE10amplitude2)>>2)]; //@line 1308 "wavegen.cpp"
            var $79=$modn_amp; //@line 1308 "wavegen.cpp"
            var $mul105=((($79)*($78))&4294967295); //@line 1308 "wavegen.cpp"
            var $div106=((((($mul105))|0)/16)|0); //@line 1308 "wavegen.cpp"
            HEAP32[((__ZZ7WavegenvE10amplitude2)>>2)]=$div106; //@line 1308 "wavegen.cpp"
            HEAP32[((__ZL15modulation_type)>>2)]=0; //@line 1309 "wavegen.cpp"
            ; //@line 1310 "wavegen.cpp"
          }
          else if (__label__ == 36) {
  
            var $80=HEAP32[((__ZZ7WavegenvE11cycle_count)>>2)]; //@line 1314 "wavegen.cpp"
            var $81=$modn_period; //@line 1314 "wavegen.cpp"
            var $rem=((($80))|0)%((($81))|0); //@line 1314 "wavegen.cpp"
            var $cmp108=((($rem))|0)==0; //@line 1314 "wavegen.cpp"
            if (!($cmp108)) { __label__ = 38;break $if_then41$$if_end118$21; } //@line 1314 "wavegen.cpp"
  
            var $82=HEAP32[((__ZZ7WavegenvE10amplitude2)>>2)]; //@line 1315 "wavegen.cpp"
            var $83=$modn_amp; //@line 1315 "wavegen.cpp"
            var $mul110=((($83)*($82))&4294967295); //@line 1315 "wavegen.cpp"
            var $div111=((((($mul110))|0)/16)|0); //@line 1315 "wavegen.cpp"
            HEAP32[((__ZZ7WavegenvE10amplitude2)>>2)]=$div111; //@line 1315 "wavegen.cpp"
            ; //@line 1315 "wavegen.cpp"
          }
        }
      } while(0);
  
      var $84=HEAP32[((__ZL9wavephase)>>2)]; //@line 1324 "wavegen.cpp"
      var $shr119=($84) >> 16; //@line 1324 "wavegen.cpp"
      var $conv120=((($shr119)) & 65535); //@line 1324 "wavegen.cpp"
      $waveph=$conv120; //@line 1324 "wavegen.cpp"
      $total=0; //@line 1325 "wavegen.cpp"
      var $85=HEAP32[((__ZL6cbytes)>>2)]; //@line 1331 "wavegen.cpp"
      var $inc121=((($85)+1)&4294967295); //@line 1331 "wavegen.cpp"
      HEAP32[((__ZL6cbytes)>>2)]=$inc121; //@line 1331 "wavegen.cpp"
      var $cmp122=((($inc121))|0) >= 0; //@line 1332 "wavegen.cpp"
      if ($cmp122) { __label__ = 39;; } else { __label__ = 43;; } //@line 1332 "wavegen.cpp"
      $land_lhs_true123$$if_end149$51: do { 
        if (__label__ == 39) {
  
          var $86=HEAP32[((__ZL6cbytes)>>2)]; //@line 1332 "wavegen.cpp"
          var $87=HEAP32[((__ZL12wavemult_max)>>2)]; //@line 1332 "wavegen.cpp"
          var $cmp124=((($86))|0) < ((($87))|0); //@line 1332 "wavegen.cpp"
          if (!($cmp124)) { __label__ = 43;break $land_lhs_true123$$if_end149$51; } //@line 1332 "wavegen.cpp"
  
          var $88=HEAP32[((_wvoice)>>2)]; //@line 1334 "wavegen.cpp"
          var $n_harmonic_peaks126=(($88+84)&4294967295); //@line 1334 "wavegen.cpp"
          var $89=HEAP32[(($n_harmonic_peaks126)>>2)]; //@line 1334 "wavegen.cpp"
          var $add127=((($89)+1)&4294967295); //@line 1334 "wavegen.cpp"
          $pk=$add127; //@line 1334 "wavegen.cpp"
          var $cmp1294=((($add127))|0) < 9; //@line 1334 "wavegen.cpp"
          if ($cmp1294) { __label__ = 41;; } else { __label__ = 42;; } //@line 1334 "wavegen.cpp"
          $for_body130$$for_end144$54: do { 
            if (__label__ == 41) {
              while(1) { 
  
                var $90=$pk; //@line 1336 "wavegen.cpp"
                var $arrayidx131=((__ZL13peak_harmonic+$90*4)&4294967295); //@line 1336 "wavegen.cpp"
                var $91=HEAP32[(($arrayidx131)>>2)]; //@line 1336 "wavegen.cpp"
                var $92=$waveph; //@line 1336 "wavegen.cpp"
                var $conv132=((($92))&65535); //@line 1336 "wavegen.cpp"
                var $mul133=((($conv132)*($91))&4294967295); //@line 1336 "wavegen.cpp"
                var $conv134=((($mul133)) & 65535); //@line 1336 "wavegen.cpp"
                $theta=$conv134; //@line 1336 "wavegen.cpp"
                var $93=$theta; //@line 1337 "wavegen.cpp"
                var $conv135=((($93))&65535); //@line 1337 "wavegen.cpp"
                var $shr136=($conv135) >> 5; //@line 1337 "wavegen.cpp"
                var $arrayidx137=((_sin_tab+$shr136*2)&4294967295); //@line 1337 "wavegen.cpp"
                var $94=HEAP16[(($arrayidx137)>>1)]; //@line 1337 "wavegen.cpp"
                var $conv138=(tempInt=(($94)),(tempInt>=32768?tempInt-65536:tempInt)); //@line 1337 "wavegen.cpp"
                var $95=$pk; //@line 1337 "wavegen.cpp"
                var $arrayidx139=((__ZL11peak_height+$95*4)&4294967295); //@line 1337 "wavegen.cpp"
                var $96=HEAP32[(($arrayidx139)>>2)]; //@line 1337 "wavegen.cpp"
                var $mul140=((($96)*($conv138))&4294967295); //@line 1337 "wavegen.cpp"
                var $97=$total; //@line 1337 "wavegen.cpp"
                var $add141=((($97)+($mul140))&4294967295); //@line 1337 "wavegen.cpp"
                $total=$add141; //@line 1337 "wavegen.cpp"
                var $98=$pk; //@line 1334 "wavegen.cpp"
                var $inc143=((($98)+1)&4294967295); //@line 1334 "wavegen.cpp"
                $pk=$inc143; //@line 1334 "wavegen.cpp"
                var $cmp129=((($inc143))|0) < 9; //@line 1334 "wavegen.cpp"
                if ($cmp129) { __label__ = 41;continue ; } else { __label__ = 42;break $for_body130$$for_end144$54; } //@line 1334 "wavegen.cpp"
              }
            }
          } while(0);
  
          var $99=$total; //@line 1341 "wavegen.cpp"
          var $100=HEAP32[((__ZL9hf_factor)>>2)]; //@line 1341 "wavegen.cpp"
          var $div145=((((($99))|0)/((($100))|0))|0); //@line 1341 "wavegen.cpp"
          var $101=HEAP32[((__ZL6cbytes)>>2)]; //@line 1341 "wavegen.cpp"
          var $arrayidx146=((__ZL8wavemult+$101)&4294967295); //@line 1341 "wavegen.cpp"
          var $102=HEAPU8[($arrayidx146)]; //@line 1341 "wavegen.cpp"
          var $conv147=((($102))&255); //@line 1341 "wavegen.cpp"
          var $mul148=((($conv147)*($div145))&4294967295); //@line 1341 "wavegen.cpp"
          $total=$mul148; //@line 1341 "wavegen.cpp"
          ; //@line 1342 "wavegen.cpp"
        }
      } while(0);
  
      var $103=$waveph; //@line 1349 "wavegen.cpp"
      $theta=$103; //@line 1349 "wavegen.cpp"
      $h=1; //@line 1351 "wavegen.cpp"
      var $104=$h; //@line 1351 "wavegen.cpp"
      var $105=HEAP32[((__ZZ7WavegenvE13h_switch_sign)>>2)]; //@line 1351 "wavegen.cpp"
      var $cmp1516=((($104))|0) <= ((($105))|0); //@line 1351 "wavegen.cpp"
      if ($cmp1516) { __label__ = 44;; } else { __label__ = 45;; } //@line 1351 "wavegen.cpp"
      $for_body152$$while_cond_loopexit$59: do { 
        if (__label__ == 44) {
          while(1) { 
  
            var $106=$theta; //@line 1353 "wavegen.cpp"
            var $conv153=((($106))&65535); //@line 1353 "wavegen.cpp"
            var $shr154=($conv153) >> 5; //@line 1353 "wavegen.cpp"
            var $arrayidx155=((_sin_tab+$shr154*2)&4294967295); //@line 1353 "wavegen.cpp"
            var $107=HEAP16[(($arrayidx155)>>1)]; //@line 1353 "wavegen.cpp"
            var $conv156=(tempInt=(($107)),(tempInt>=32768?tempInt-65536:tempInt)); //@line 1353 "wavegen.cpp"
            var $108=$h; //@line 1353 "wavegen.cpp"
            var $109=HEAP32[((__ZL9harmspect)>>2)]; //@line 1353 "wavegen.cpp"
            var $arrayidx157=(($109+4*$108)&4294967295); //@line 1353 "wavegen.cpp"
            var $110=HEAP32[(($arrayidx157)>>2)]; //@line 1353 "wavegen.cpp"
            var $mul158=((($110)*($conv156))&4294967295); //@line 1353 "wavegen.cpp"
            var $111=$total; //@line 1353 "wavegen.cpp"
            var $add159=((($111)+($mul158))&4294967295); //@line 1353 "wavegen.cpp"
            $total=$add159; //@line 1353 "wavegen.cpp"
            var $112=$waveph; //@line 1354 "wavegen.cpp"
            var $conv160=((($112))&65535); //@line 1354 "wavegen.cpp"
            var $113=$theta; //@line 1354 "wavegen.cpp"
            var $conv161=((($113))&65535); //@line 1354 "wavegen.cpp"
            var $add162=((($conv161)+($conv160))&4294967295); //@line 1354 "wavegen.cpp"
            var $conv163=((($add162)) & 65535); //@line 1354 "wavegen.cpp"
            $theta=$conv163; //@line 1354 "wavegen.cpp"
            var $114=$h; //@line 1351 "wavegen.cpp"
            var $inc165=((($114)+1)&4294967295); //@line 1351 "wavegen.cpp"
            $h=$inc165; //@line 1351 "wavegen.cpp"
            var $115=$h; //@line 1351 "wavegen.cpp"
            var $116=HEAP32[((__ZZ7WavegenvE13h_switch_sign)>>2)]; //@line 1351 "wavegen.cpp"
            var $cmp151=((($115))|0) <= ((($116))|0); //@line 1351 "wavegen.cpp"
            if ($cmp151) { __label__ = 44;continue ; } else { __label__ = 45;break $for_body152$$while_cond_loopexit$59; } //@line 1351 "wavegen.cpp"
          }
        }
      } while(0);
  
      var $117=$h; //@line 1356 "wavegen.cpp"
      var $118=HEAP32[((__ZZ7WavegenvE4maxh)>>2)]; //@line 1356 "wavegen.cpp"
      var $cmp1677=((($117))|0) <= ((($118))|0); //@line 1356 "wavegen.cpp"
      if ($cmp1677) { __label__ = 46;; } else { __label__ = 47;; } //@line 1356 "wavegen.cpp"
      $while_body$$while_end$63: do { 
        if (__label__ == 46) {
          while(1) { 
  
            var $119=$theta; //@line 1358 "wavegen.cpp"
            var $conv168=((($119))&65535); //@line 1358 "wavegen.cpp"
            var $shr169=($conv168) >> 5; //@line 1358 "wavegen.cpp"
            var $arrayidx170=((_sin_tab+$shr169*2)&4294967295); //@line 1358 "wavegen.cpp"
            var $120=HEAP16[(($arrayidx170)>>1)]; //@line 1358 "wavegen.cpp"
            var $conv171=(tempInt=(($120)),(tempInt>=32768?tempInt-65536:tempInt)); //@line 1358 "wavegen.cpp"
            var $121=$h; //@line 1358 "wavegen.cpp"
            var $122=HEAP32[((__ZL9harmspect)>>2)]; //@line 1358 "wavegen.cpp"
            var $arrayidx172=(($122+4*$121)&4294967295); //@line 1358 "wavegen.cpp"
            var $123=HEAP32[(($arrayidx172)>>2)]; //@line 1358 "wavegen.cpp"
            var $mul173=((($123)*($conv171))&4294967295); //@line 1358 "wavegen.cpp"
            var $124=$total; //@line 1358 "wavegen.cpp"
            var $sub174=((($124)-($mul173))&4294967295); //@line 1358 "wavegen.cpp"
            $total=$sub174; //@line 1358 "wavegen.cpp"
            var $125=$waveph; //@line 1359 "wavegen.cpp"
            var $conv175=((($125))&65535); //@line 1359 "wavegen.cpp"
            var $126=$theta; //@line 1359 "wavegen.cpp"
            var $conv176=((($126))&65535); //@line 1359 "wavegen.cpp"
            var $add177=((($conv176)+($conv175))&4294967295); //@line 1359 "wavegen.cpp"
            var $conv178=((($add177)) & 65535); //@line 1359 "wavegen.cpp"
            $theta=$conv178; //@line 1359 "wavegen.cpp"
            var $127=$h; //@line 1360 "wavegen.cpp"
            var $inc179=((($127)+1)&4294967295); //@line 1360 "wavegen.cpp"
            $h=$inc179; //@line 1360 "wavegen.cpp"
            var $128=$h; //@line 1356 "wavegen.cpp"
            var $129=HEAP32[((__ZZ7WavegenvE4maxh)>>2)]; //@line 1356 "wavegen.cpp"
            var $cmp167=((($128))|0) <= ((($129))|0); //@line 1356 "wavegen.cpp"
            if ($cmp167) { __label__ = 46;continue ; } else { __label__ = 47;break $while_body$$while_end$63; } //@line 1356 "wavegen.cpp"
          }
        }
      } while(0);
  
      var $130=HEAP32[((__ZL7voicing)>>2)]; //@line 1364 "wavegen.cpp"
      var $cmp180=((($130))|0)!=64; //@line 1364 "wavegen.cpp"
      if ($cmp180) { __label__ = 48;; } else { __label__ = 49;; } //@line 1364 "wavegen.cpp"
      if (__label__ == 48) {
  
        var $131=$total; //@line 1366 "wavegen.cpp"
        var $shr182=($131) >> 6; //@line 1366 "wavegen.cpp"
        var $132=HEAP32[((__ZL7voicing)>>2)]; //@line 1366 "wavegen.cpp"
        var $mul183=((($shr182)*($132))&4294967295); //@line 1366 "wavegen.cpp"
        $total=$mul183; //@line 1366 "wavegen.cpp"
        ; //@line 1367 "wavegen.cpp"
      }
  
      var $133=HEAP32[((_wvoice)>>2)]; //@line 1370 "wavegen.cpp"
      var $breath=(($133+272)&4294967295); //@line 1370 "wavegen.cpp"
      var $arrayidx185=(($breath)&4294967295); //@line 1370 "wavegen.cpp"
      var $134=HEAP32[(($arrayidx185)>>2)]; //@line 1370 "wavegen.cpp"
      var $tobool=((($134))|0)!=0; //@line 1370 "wavegen.cpp"
      if ($tobool) { __label__ = 50;; } else { __label__ = 51;; } //@line 1370 "wavegen.cpp"
      if (__label__ == 50) {
  
        var $call187=__ZL11ApplyBreathv(); //@line 1372 "wavegen.cpp"
        var $135=$total; //@line 1372 "wavegen.cpp"
        var $add188=((($135)+($call187))&4294967295); //@line 1372 "wavegen.cpp"
        $total=$add188; //@line 1372 "wavegen.cpp"
        ; //@line 1373 "wavegen.cpp"
      }
  
      $z2=0; //@line 1377 "wavegen.cpp"
      var $136=HEAP32[((((_wdata+40)&4294967295))>>2)]; //@line 1378 "wavegen.cpp"
      var $137=HEAP32[((((_wdata+28)&4294967295))>>2)]; //@line 1378 "wavegen.cpp"
      var $cmp190=((($136))|0) < ((($137))|0); //@line 1378 "wavegen.cpp"
      if ($cmp190) { __label__ = 52;; } else { __label__ = 57;; } //@line 1378 "wavegen.cpp"
      $if_then191$$if_end222$73: do { 
        if (__label__ == 52) {
  
          var $138=HEAP32[((((_wdata+32)&4294967295))>>2)]; //@line 1380 "wavegen.cpp"
          var $cmp192=((($138))|0)==0; //@line 1380 "wavegen.cpp"
          if ($cmp192) { __label__ = 53;; } else { __label__ = 54;; } //@line 1380 "wavegen.cpp"
          if (__label__ == 53) {
  
            var $139=HEAP32[((((_wdata+40)&4294967295))>>2)]; //@line 1383 "wavegen.cpp"
            var $140=HEAP32[((((_wdata+48)&4294967295))>>2)]; //@line 1383 "wavegen.cpp"
            var $add194=((($139)+1)&4294967295); //@line 1383 "wavegen.cpp"
            var $add195=((($add194)+($140))&4294967295); //@line 1383 "wavegen.cpp"
            var $141=HEAP32[((((_wdata+24)&4294967295))>>2)]; //@line 1383 "wavegen.cpp"
            var $arrayidx196=(($141+$add195)&4294967295); //@line 1383 "wavegen.cpp"
            var $142=HEAP8[($arrayidx196)]; //@line 1383 "wavegen.cpp"
            $c=$142; //@line 1383 "wavegen.cpp"
            var $143=HEAP32[((((_wdata+40)&4294967295))>>2)]; //@line 1384 "wavegen.cpp"
            var $144=HEAP32[((((_wdata+48)&4294967295))>>2)]; //@line 1384 "wavegen.cpp"
            var $add197=((($144)+($143))&4294967295); //@line 1384 "wavegen.cpp"
            var $145=HEAP32[((((_wdata+24)&4294967295))>>2)]; //@line 1384 "wavegen.cpp"
            var $arrayidx198=(($145+$add197)&4294967295); //@line 1384 "wavegen.cpp"
            var $146=HEAPU8[($arrayidx198)]; //@line 1384 "wavegen.cpp"
            var $conv199=((($146))&255); //@line 1384 "wavegen.cpp"
            var $147=$c; //@line 1384 "wavegen.cpp"
            var $conv200=(tempInt=(($147)),(tempInt>=128?tempInt-256:tempInt)); //@line 1384 "wavegen.cpp"
            var $mul201=((($conv200)*256)&4294967295); //@line 1384 "wavegen.cpp"
            var $add202=((($mul201)+($conv199))&4294967295); //@line 1384 "wavegen.cpp"
            $sample=$add202; //@line 1384 "wavegen.cpp"
            var $148=HEAP32[((((_wdata+40)&4294967295))>>2)]; //@line 1385 "wavegen.cpp"
            var $add203=((($148)+2)&4294967295); //@line 1385 "wavegen.cpp"
            HEAP32[((((_wdata+40)&4294967295))>>2)]=$add203; //@line 1385 "wavegen.cpp"
            ; //@line 1386 "wavegen.cpp"
          }
          else if (__label__ == 54) {
  
            var $149=HEAP32[((((_wdata+48)&4294967295))>>2)]; //@line 1390 "wavegen.cpp"
            var $150=HEAP32[((((_wdata+40)&4294967295))>>2)]; //@line 1390 "wavegen.cpp"
            var $inc205=((($150)+1)&4294967295); //@line 1390 "wavegen.cpp"
            HEAP32[((((_wdata+40)&4294967295))>>2)]=$inc205; //@line 1390 "wavegen.cpp"
            var $add206=((($150)+($149))&4294967295); //@line 1390 "wavegen.cpp"
            var $151=HEAP32[((((_wdata+24)&4294967295))>>2)]; //@line 1390 "wavegen.cpp"
            var $arrayidx207=(($151+$add206)&4294967295); //@line 1390 "wavegen.cpp"
            var $152=HEAP8[($arrayidx207)]; //@line 1390 "wavegen.cpp"
            var $conv208=(tempInt=(($152)),(tempInt>=128?tempInt-256:tempInt)); //@line 1390 "wavegen.cpp"
            var $153=HEAP32[((((_wdata+32)&4294967295))>>2)]; //@line 1390 "wavegen.cpp"
            var $mul209=((($conv208)*($153))&4294967295); //@line 1390 "wavegen.cpp"
            $sample=$mul209; //@line 1390 "wavegen.cpp"
            ;
          }
  
          var $154=$sample; //@line 1392 "wavegen.cpp"
          var $155=HEAP32[((((_wdata+56)&4294967295))>>2)]; //@line 1392 "wavegen.cpp"
          var $mul211=((($155)*($154))&4294967295); //@line 1392 "wavegen.cpp"
          var $shr212=($mul211) >> 10; //@line 1392 "wavegen.cpp"
          $z2=$shr212; //@line 1392 "wavegen.cpp"
          var $156=$z2; //@line 1393 "wavegen.cpp"
          var $157=HEAP32[((((_wdata+36)&4294967295))>>2)]; //@line 1393 "wavegen.cpp"
          var $mul213=((($157)*($156))&4294967295); //@line 1393 "wavegen.cpp"
          var $div214=((((($mul213))|0)/32)|0); //@line 1393 "wavegen.cpp"
          $z2=$div214; //@line 1393 "wavegen.cpp"
          var $158=HEAP32[((((_wdata+40)&4294967295))>>2)]; //@line 1395 "wavegen.cpp"
          var $159=HEAP32[((((_wdata+48)&4294967295))>>2)]; //@line 1395 "wavegen.cpp"
          var $add215=((($159)+($158))&4294967295); //@line 1395 "wavegen.cpp"
          var $160=HEAP32[((((_wdata+44)&4294967295))>>2)]; //@line 1395 "wavegen.cpp"
          var $cmp216=((($add215))|0) >= ((($160))|0); //@line 1395 "wavegen.cpp"
          if (!($cmp216)) { __label__ = 57;break $if_then191$$if_end222$73; } //@line 1395 "wavegen.cpp"
  
          var $161=HEAP32[((((_wdata+44)&4294967295))>>2)]; //@line 1396 "wavegen.cpp"
          var $mul218=((($161)*3)&4294967295); //@line 1396 "wavegen.cpp"
          var $div219=((((($mul218))|0)/4)|0); //@line 1396 "wavegen.cpp"
          var $162=HEAP32[((((_wdata+48)&4294967295))>>2)]; //@line 1396 "wavegen.cpp"
          var $sub220=((($162)-($div219))&4294967295); //@line 1396 "wavegen.cpp"
          HEAP32[((((_wdata+48)&4294967295))>>2)]=$sub220; //@line 1396 "wavegen.cpp"
          ; //@line 1396 "wavegen.cpp"
        }
      } while(0);
  
      var $163=$z2; //@line 1399 "wavegen.cpp"
      var $164=$total; //@line 1399 "wavegen.cpp"
      var $shr223=($164) >> 8; //@line 1399 "wavegen.cpp"
      var $165=HEAP32[((__ZZ7WavegenvE10amplitude2)>>2)]; //@line 1399 "wavegen.cpp"
      var $mul224=((($shr223)*($165))&4294967295); //@line 1399 "wavegen.cpp"
      var $shr225=($mul224) >> 13; //@line 1399 "wavegen.cpp"
      var $add226=((($shr225)+($163))&4294967295); //@line 1399 "wavegen.cpp"
      $z1=$add226; //@line 1399 "wavegen.cpp"
      var $166=HEAP32[((_echo_tail)>>2)]; //@line 1401 "wavegen.cpp"
      var $inc227=((($166)+1)&4294967295); //@line 1401 "wavegen.cpp"
      HEAP32[((_echo_tail)>>2)]=$inc227; //@line 1401 "wavegen.cpp"
      var $arrayidx228=((_echo_buf+$166*2)&4294967295); //@line 1401 "wavegen.cpp"
      var $167=HEAP16[(($arrayidx228)>>1)]; //@line 1401 "wavegen.cpp"
      var $conv229=(tempInt=(($167)),(tempInt>=32768?tempInt-65536:tempInt)); //@line 1401 "wavegen.cpp"
      var $168=HEAP32[((_echo_amp)>>2)]; //@line 1401 "wavegen.cpp"
      var $mul230=((($conv229)*($168))&4294967295); //@line 1401 "wavegen.cpp"
      $echo=$mul230; //@line 1401 "wavegen.cpp"
      var $169=$echo; //@line 1402 "wavegen.cpp"
      var $shr231=($169) >> 8; //@line 1402 "wavegen.cpp"
      var $170=$z1; //@line 1402 "wavegen.cpp"
      var $add232=((($170)+($shr231))&4294967295); //@line 1402 "wavegen.cpp"
      $z1=$add232; //@line 1402 "wavegen.cpp"
      var $171=HEAP32[((_echo_tail)>>2)]; //@line 1403 "wavegen.cpp"
      var $cmp233=((($171))|0) >= 5500; //@line 1403 "wavegen.cpp"
      if ($cmp233) { __label__ = 58;; } else { __label__ = 59;; } //@line 1403 "wavegen.cpp"
      if (__label__ == 58) {
  
        HEAP32[((_echo_tail)>>2)]=0; //@line 1404 "wavegen.cpp"
        ; //@line 1404 "wavegen.cpp"
      }
  
      var $172=$z1; //@line 1406 "wavegen.cpp"
      var $173=HEAP32[((__ZZ7WavegenvE3agc)>>2)]; //@line 1406 "wavegen.cpp"
      var $mul236=((($173)*($172))&4294967295); //@line 1406 "wavegen.cpp"
      var $shr237=($mul236) >> 8; //@line 1406 "wavegen.cpp"
      $z=$shr237; //@line 1406 "wavegen.cpp"
      var $cmp238=((($shr237))|0) >= 32768; //@line 1409 "wavegen.cpp"
      if ($cmp238) { __label__ = 60;; } else { __label__ = 63;; } //@line 1409 "wavegen.cpp"
      $if_then239$$if_else247$84: do { 
        if (__label__ == 60) {
  
          var $174=$z1; //@line 1411 "wavegen.cpp"
          var $div240=((8388608/((($174))|0))|0); //@line 1411 "wavegen.cpp"
          var $sub241=((($div240)-1)&4294967295); //@line 1411 "wavegen.cpp"
          $ov=$sub241; //@line 1411 "wavegen.cpp"
          var $175=$ov; //@line 1412 "wavegen.cpp"
          var $176=HEAP32[((__ZZ7WavegenvE3agc)>>2)]; //@line 1412 "wavegen.cpp"
          var $cmp242=((($175))|0) < ((($176))|0); //@line 1412 "wavegen.cpp"
          if ($cmp242) { __label__ = 61;; } else { __label__ = 62;; } //@line 1412 "wavegen.cpp"
          if (__label__ == 61) {
  
            var $177=$ov; //@line 1412 "wavegen.cpp"
            HEAP32[((__ZZ7WavegenvE3agc)>>2)]=$177; //@line 1412 "wavegen.cpp"
            ; //@line 1412 "wavegen.cpp"
          }
  
          var $178=$z1; //@line 1413 "wavegen.cpp"
          var $179=HEAP32[((__ZZ7WavegenvE3agc)>>2)]; //@line 1413 "wavegen.cpp"
          var $mul245=((($179)*($178))&4294967295); //@line 1413 "wavegen.cpp"
          var $shr246=($mul245) >> 8; //@line 1413 "wavegen.cpp"
          $z=$shr246; //@line 1413 "wavegen.cpp"
          ; //@line 1414 "wavegen.cpp"
        }
        else if (__label__ == 63) {
  
          var $180=$z; //@line 1416 "wavegen.cpp"
          var $cmp248=((($180))|0) <= -32768; //@line 1416 "wavegen.cpp"
          if (!($cmp248)) { __label__ = 67;break $if_then239$$if_else247$84; } //@line 1416 "wavegen.cpp"
  
          var $181=$z1; //@line 1418 "wavegen.cpp"
          var $div250=((-8388608/((($181))|0))|0); //@line 1418 "wavegen.cpp"
          var $sub251=((($div250)-1)&4294967295); //@line 1418 "wavegen.cpp"
          $ov=$sub251; //@line 1418 "wavegen.cpp"
          var $182=$ov; //@line 1419 "wavegen.cpp"
          var $183=HEAP32[((__ZZ7WavegenvE3agc)>>2)]; //@line 1419 "wavegen.cpp"
          var $cmp252=((($182))|0) < ((($183))|0); //@line 1419 "wavegen.cpp"
          if ($cmp252) { __label__ = 65;; } else { __label__ = 66;; } //@line 1419 "wavegen.cpp"
          if (__label__ == 65) {
  
            var $184=$ov; //@line 1419 "wavegen.cpp"
            HEAP32[((__ZZ7WavegenvE3agc)>>2)]=$184; //@line 1419 "wavegen.cpp"
            ; //@line 1419 "wavegen.cpp"
          }
  
          var $185=$z1; //@line 1420 "wavegen.cpp"
          var $186=HEAP32[((__ZZ7WavegenvE3agc)>>2)]; //@line 1420 "wavegen.cpp"
          var $mul255=((($186)*($185))&4294967295); //@line 1420 "wavegen.cpp"
          var $shr256=($mul255) >> 8; //@line 1420 "wavegen.cpp"
          $z=$shr256; //@line 1420 "wavegen.cpp"
          ; //@line 1421 "wavegen.cpp"
        }
      } while(0);
  
      var $187=$z; //@line 1422 "wavegen.cpp"
      var $conv259=((($187)) & 255); //@line 1422 "wavegen.cpp"
      var $188=HEAP32[((_out_ptr)>>2)]; //@line 1422 "wavegen.cpp"
      var $incdec_ptr=(($188+1)&4294967295); //@line 1422 "wavegen.cpp"
      HEAP32[((_out_ptr)>>2)]=$incdec_ptr; //@line 1422 "wavegen.cpp"
      HEAP8[($188)]=$conv259; //@line 1422 "wavegen.cpp"
      var $189=$z; //@line 1423 "wavegen.cpp"
      var $shr260=($189) >> 8; //@line 1423 "wavegen.cpp"
      var $conv261=((($shr260)) & 255); //@line 1423 "wavegen.cpp"
      var $190=HEAP32[((_out_ptr)>>2)]; //@line 1423 "wavegen.cpp"
      var $incdec_ptr262=(($190+1)&4294967295); //@line 1423 "wavegen.cpp"
      HEAP32[((_out_ptr)>>2)]=$incdec_ptr262; //@line 1423 "wavegen.cpp"
      HEAP8[($190)]=$conv261; //@line 1423 "wavegen.cpp"
      var $191=$z; //@line 1425 "wavegen.cpp"
      var $conv263=((($191)) & 65535); //@line 1425 "wavegen.cpp"
      var $192=HEAP32[((_echo_head)>>2)]; //@line 1425 "wavegen.cpp"
      var $inc264=((($192)+1)&4294967295); //@line 1425 "wavegen.cpp"
      HEAP32[((_echo_head)>>2)]=$inc264; //@line 1425 "wavegen.cpp"
      var $arrayidx265=((_echo_buf+$192*2)&4294967295); //@line 1425 "wavegen.cpp"
      HEAP16[(($arrayidx265)>>1)]=$conv263; //@line 1425 "wavegen.cpp"
      var $193=HEAP32[((_echo_head)>>2)]; //@line 1426 "wavegen.cpp"
      var $cmp266=((($193))|0) >= 5500; //@line 1426 "wavegen.cpp"
      if ($cmp266) { __label__ = 68;; } else { __label__ = 69;; } //@line 1426 "wavegen.cpp"
      if (__label__ == 68) {
  
        HEAP32[((_echo_head)>>2)]=0; //@line 1427 "wavegen.cpp"
        ; //@line 1427 "wavegen.cpp"
      }
  
      var $194=HEAPU32[((_out_ptr)>>2)]; //@line 1429 "wavegen.cpp"
      var $195=HEAPU32[((_out_end)>>2)]; //@line 1429 "wavegen.cpp"
      var $cmp269=((($194))>>>0) >= ((($195))>>>0); //@line 1429 "wavegen.cpp"
      if ($cmp269) { __label__ = 70;break $for_cond$2; } else { __label__ = 1;continue $for_cond$2; } //@line 1429 "wavegen.cpp"
    }
    if (__label__ == 2) {
  
      $retval=0; //@line 1189 "wavegen.cpp"
      ; //@line 1189 "wavegen.cpp"
    }
    else if (__label__ == 18) {
  
      $retval=0; //@line 1244 "wavegen.cpp"
      ; //@line 1244 "wavegen.cpp"
    }
    else if (__label__ == 70) {
  
      $retval=1; //@line 1430 "wavegen.cpp"
      ; //@line 1430 "wavegen.cpp"
    }
  
    var $196=$retval; //@line 1433 "wavegen.cpp"
    ;
    return $196; //@line 1433 "wavegen.cpp"
    return null;
  }
  

  function __ZL17AdvanceParametersv() {
    ;
    var __label__;
    var __lastLabel__ = null;
  
    var $x;
    var $ix;
    var $0=HEAP32[((((_wdata+12)&4294967295))>>2)]; //@line 990 "wavegen.cpp"
    var $1=HEAP32[((((_wdata+8)&4294967295))>>2)]; //@line 990 "wavegen.cpp"
    var $add=((($1)+($0))&4294967295); //@line 990 "wavegen.cpp"
    HEAP32[((((_wdata+8)&4294967295))>>2)]=$add; //@line 990 "wavegen.cpp"
    var $2=HEAP32[((((_wdata+8)&4294967295))>>2)]; //@line 991 "wavegen.cpp"
    var $shr=($2) >> 8; //@line 991 "wavegen.cpp"
    $ix=$shr; //@line 991 "wavegen.cpp"
    var $cmp=((($shr))|0) > 127; //@line 991 "wavegen.cpp"
    if ($cmp) { __label__ = 1;; } else { __label__ = 2;; } //@line 991 "wavegen.cpp"
    if (__label__ == 1) {
  
      $ix=127; //@line 991 "wavegen.cpp"
      ; //@line 991 "wavegen.cpp"
    }
  
    var $3=$ix; //@line 992 "wavegen.cpp"
    var $4=HEAP32[((((_wdata)&4294967295))>>2)]; //@line 992 "wavegen.cpp"
    var $arrayidx=(($4+$3)&4294967295); //@line 992 "wavegen.cpp"
    var $5=HEAPU8[($arrayidx)]; //@line 992 "wavegen.cpp"
    var $conv=((($5))&255); //@line 992 "wavegen.cpp"
    var $6=HEAP32[((((_wdata+20)&4294967295))>>2)]; //@line 992 "wavegen.cpp"
    var $mul=((($conv)*($6))&4294967295); //@line 992 "wavegen.cpp"
    $x=$mul; //@line 992 "wavegen.cpp"
    var $7=$x; //@line 993 "wavegen.cpp"
    var $shr1=($7) >> 8; //@line 993 "wavegen.cpp"
    var $8=HEAP32[((((_wdata+16)&4294967295))>>2)]; //@line 993 "wavegen.cpp"
    var $add2=((($shr1)+($8))&4294967295); //@line 993 "wavegen.cpp"
    HEAP32[((((_wdata+4)&4294967295))>>2)]=$add2; //@line 993 "wavegen.cpp"
    var $9=HEAP32[((__ZL7amp_inc)>>2)]; //@line 995 "wavegen.cpp"
    var $10=HEAP32[((__ZL6amp_ix)>>2)]; //@line 995 "wavegen.cpp"
    var $add3=((($10)+($9))&4294967295); //@line 995 "wavegen.cpp"
    HEAP32[((__ZL6amp_ix)>>2)]=$add3; //@line 995 "wavegen.cpp"
    var $11=HEAP32[((__ZZL17AdvanceParametersvE10Flutter_ix)>>2)]; //@line 998 "wavegen.cpp"
    var $cmp4=((($11))|0) >= 23552; //@line 998 "wavegen.cpp"
    if ($cmp4) { __label__ = 3;; } else { __label__ = 4;; } //@line 998 "wavegen.cpp"
    if (__label__ == 3) {
  
      HEAP32[((__ZZL17AdvanceParametersvE10Flutter_ix)>>2)]=0; //@line 999 "wavegen.cpp"
      ; //@line 999 "wavegen.cpp"
    }
  
    var $12=HEAP32[((__ZZL17AdvanceParametersvE10Flutter_ix)>>2)]; //@line 1000 "wavegen.cpp"
    var $shr7=($12) >> 6; //@line 1000 "wavegen.cpp"
    var $arrayidx8=((__ZL11Flutter_tab+$shr7)&4294967295); //@line 1000 "wavegen.cpp"
    var $13=HEAPU8[($arrayidx8)]; //@line 1000 "wavegen.cpp"
    var $conv9=((($13))&255); //@line 1000 "wavegen.cpp"
    var $sub=((($conv9)-128)&4294967295); //@line 1000 "wavegen.cpp"
    var $14=HEAP32[((__ZL11flutter_amp)>>2)]; //@line 1000 "wavegen.cpp"
    var $mul10=((($sub)*($14))&4294967295); //@line 1000 "wavegen.cpp"
    $x=$mul10; //@line 1000 "wavegen.cpp"
    var $15=HEAP32[((__ZL11Flutter_inc)>>2)]; //@line 1001 "wavegen.cpp"
    var $16=HEAP32[((__ZZL17AdvanceParametersvE10Flutter_ix)>>2)]; //@line 1001 "wavegen.cpp"
    var $add11=((($16)+($15))&4294967295); //@line 1001 "wavegen.cpp"
    HEAP32[((__ZZL17AdvanceParametersvE10Flutter_ix)>>2)]=$add11; //@line 1001 "wavegen.cpp"
    var $17=$x; //@line 1002 "wavegen.cpp"
    var $18=HEAP32[((((_wdata+4)&4294967295))>>2)]; //@line 1002 "wavegen.cpp"
    var $add12=((($18)+($17))&4294967295); //@line 1002 "wavegen.cpp"
    HEAP32[((((_wdata+4)&4294967295))>>2)]=$add12; //@line 1002 "wavegen.cpp"
    var $cmp13=((($add12))|0) < 102400; //@line 1003 "wavegen.cpp"
    if ($cmp13) { __label__ = 5;; } else { __label__ = 6;; } //@line 1003 "wavegen.cpp"
    if (__label__ == 5) {
  
      HEAP32[((((_wdata+4)&4294967295))>>2)]=102400; //@line 1004 "wavegen.cpp"
      ; //@line 1004 "wavegen.cpp"
    }
  
    var $19=HEAP32[((__ZL11samplecount)>>2)]; //@line 1006 "wavegen.cpp"
    var $20=HEAP32[((__ZL17samplecount_start)>>2)]; //@line 1006 "wavegen.cpp"
    var $cmp16=((($19))|0)==((($20))|0); //@line 1006 "wavegen.cpp"
    if ($cmp16) { __label__ = 21;; } else { __label__ = 7;; } //@line 1006 "wavegen.cpp"
    $for_end93$$if_end18$11: do { 
      if (__label__ == 7) {
  
        $ix=0; //@line 1009 "wavegen.cpp"
        var $21=$ix; //@line 1009 "wavegen.cpp"
        var $22=HEAP32[((_wvoice)>>2)]; //@line 1009 "wavegen.cpp"
        var $n_harmonic_peaks2=(($22+84)&4294967295); //@line 1009 "wavegen.cpp"
        var $23=HEAP32[(($n_harmonic_peaks2)>>2)]; //@line 1009 "wavegen.cpp"
        var $cmp193=((($21))|0) <= ((($23))|0); //@line 1009 "wavegen.cpp"
        if ($cmp193) { __lastLabel__ = 7; __label__ = 9;; } else { __lastLabel__ = 7; __label__ = 8;; } //@line 1009 "wavegen.cpp"
        $for_body$$for_cond60_preheader$13: do { 
          if (__label__ == 9) {
            while(1) { 
  
              var $24=$ix; //@line 1011 "wavegen.cpp"
              var $arrayidx20=((__ZL5peaks+$24*80)&4294967295); //@line 1011 "wavegen.cpp"
              var $freq_inc=(($arrayidx20+48)&4294967295); //@line 1011 "wavegen.cpp"
              var $25=HEAPF32[(($freq_inc)>>2)]; //@line 1011 "wavegen.cpp"
              var $26=$ix; //@line 1011 "wavegen.cpp"
              var $arrayidx21=((__ZL5peaks+$26*80)&4294967295); //@line 1011 "wavegen.cpp"
              var $freq1=(($arrayidx21+16)&4294967295); //@line 1011 "wavegen.cpp"
              var $27=HEAPF32[(($freq1)>>2)]; //@line 1011 "wavegen.cpp"
              var $add22=($27)+($25); //@line 1011 "wavegen.cpp"
              HEAPF32[(($freq1)>>2)]=$add22; //@line 1011 "wavegen.cpp"
              var $28=$ix; //@line 1012 "wavegen.cpp"
              var $arrayidx23=((__ZL5peaks+$28*80)&4294967295); //@line 1012 "wavegen.cpp"
              var $freq124=(($arrayidx23+16)&4294967295); //@line 1012 "wavegen.cpp"
              var $29=HEAPF32[(($freq124)>>2)]; //@line 1012 "wavegen.cpp"
              var $conv25=((($29))|0); //@line 1012 "wavegen.cpp"
              var $30=$ix; //@line 1012 "wavegen.cpp"
              var $arrayidx26=((__ZL5peaks+$30*80)&4294967295); //@line 1012 "wavegen.cpp"
              var $freq=(($arrayidx26)&4294967295); //@line 1012 "wavegen.cpp"
              HEAP32[(($freq)>>2)]=$conv25; //@line 1012 "wavegen.cpp"
              var $31=$ix; //@line 1013 "wavegen.cpp"
              var $arrayidx27=((__ZL5peaks+$31*80)&4294967295); //@line 1013 "wavegen.cpp"
              var $height_inc=(($arrayidx27+56)&4294967295); //@line 1013 "wavegen.cpp"
              var $32=HEAPF32[(($height_inc)>>2)]; //@line 1013 "wavegen.cpp"
              var $33=$ix; //@line 1013 "wavegen.cpp"
              var $arrayidx28=((__ZL5peaks+$33*80)&4294967295); //@line 1013 "wavegen.cpp"
              var $height1=(($arrayidx28+24)&4294967295); //@line 1013 "wavegen.cpp"
              var $34=HEAPF32[(($height1)>>2)]; //@line 1013 "wavegen.cpp"
              var $add29=($34)+($32); //@line 1013 "wavegen.cpp"
              HEAPF32[(($height1)>>2)]=$add29; //@line 1013 "wavegen.cpp"
              var $35=$ix; //@line 1014 "wavegen.cpp"
              var $arrayidx30=((__ZL5peaks+$35*80)&4294967295); //@line 1014 "wavegen.cpp"
              var $height131=(($arrayidx30+24)&4294967295); //@line 1014 "wavegen.cpp"
              var $36=HEAPF32[(($height131)>>2)]; //@line 1014 "wavegen.cpp"
              var $conv32=((($36))|0); //@line 1014 "wavegen.cpp"
              var $37=$ix; //@line 1014 "wavegen.cpp"
              var $arrayidx33=((__ZL5peaks+$37*80)&4294967295); //@line 1014 "wavegen.cpp"
              var $height=(($arrayidx33+4)&4294967295); //@line 1014 "wavegen.cpp"
              HEAP32[(($height)>>2)]=$conv32; //@line 1014 "wavegen.cpp"
              var $cmp34=((($conv32))|0) < 0; //@line 1014 "wavegen.cpp"
              if ($cmp34) { __label__ = 10;; } else { __label__ = 11;; } //@line 1014 "wavegen.cpp"
              if (__label__ == 10) {
  
                var $38=$ix; //@line 1015 "wavegen.cpp"
                var $arrayidx36=((__ZL5peaks+$38*80)&4294967295); //@line 1015 "wavegen.cpp"
                var $height37=(($arrayidx36+4)&4294967295); //@line 1015 "wavegen.cpp"
                HEAP32[(($height37)>>2)]=0; //@line 1015 "wavegen.cpp"
                ; //@line 1015 "wavegen.cpp"
              }
  
              var $39=$ix; //@line 1016 "wavegen.cpp"
              var $arrayidx39=((__ZL5peaks+$39*80)&4294967295); //@line 1016 "wavegen.cpp"
              var $left_inc=(($arrayidx39+64)&4294967295); //@line 1016 "wavegen.cpp"
              var $40=HEAPF32[(($left_inc)>>2)]; //@line 1016 "wavegen.cpp"
              var $41=$ix; //@line 1016 "wavegen.cpp"
              var $arrayidx40=((__ZL5peaks+$41*80)&4294967295); //@line 1016 "wavegen.cpp"
              var $left1=(($arrayidx40+32)&4294967295); //@line 1016 "wavegen.cpp"
              var $42=HEAPF32[(($left1)>>2)]; //@line 1016 "wavegen.cpp"
              var $add41=($42)+($40); //@line 1016 "wavegen.cpp"
              HEAPF32[(($left1)>>2)]=$add41; //@line 1016 "wavegen.cpp"
              var $43=$ix; //@line 1017 "wavegen.cpp"
              var $arrayidx42=((__ZL5peaks+$43*80)&4294967295); //@line 1017 "wavegen.cpp"
              var $left143=(($arrayidx42+32)&4294967295); //@line 1017 "wavegen.cpp"
              var $44=HEAPF32[(($left143)>>2)]; //@line 1017 "wavegen.cpp"
              var $conv44=((($44))|0); //@line 1017 "wavegen.cpp"
              var $45=$ix; //@line 1017 "wavegen.cpp"
              var $arrayidx45=((__ZL5peaks+$45*80)&4294967295); //@line 1017 "wavegen.cpp"
              var $left=(($arrayidx45+8)&4294967295); //@line 1017 "wavegen.cpp"
              HEAP32[(($left)>>2)]=$conv44; //@line 1017 "wavegen.cpp"
              var $46=$ix; //@line 1018 "wavegen.cpp"
              var $cmp46=((($46))|0) < 3; //@line 1018 "wavegen.cpp"
              var $47=$ix; //@line 1020 "wavegen.cpp"
              var $arrayidx48=((__ZL5peaks+$47*80)&4294967295); //@line 1020 "wavegen.cpp"
              if ($cmp46) { __label__ = 12;; } else { __label__ = 13;; } //@line 1018 "wavegen.cpp"
              if (__label__ == 12) {
  
                var $right_inc=(($arrayidx48+72)&4294967295); //@line 1020 "wavegen.cpp"
                var $48=HEAPF32[(($right_inc)>>2)]; //@line 1020 "wavegen.cpp"
                var $49=$ix; //@line 1020 "wavegen.cpp"
                var $arrayidx49=((__ZL5peaks+$49*80)&4294967295); //@line 1020 "wavegen.cpp"
                var $right1=(($arrayidx49+40)&4294967295); //@line 1020 "wavegen.cpp"
                var $50=HEAPF32[(($right1)>>2)]; //@line 1020 "wavegen.cpp"
                var $add50=($50)+($48); //@line 1020 "wavegen.cpp"
                HEAPF32[(($right1)>>2)]=$add50; //@line 1020 "wavegen.cpp"
                var $51=$ix; //@line 1021 "wavegen.cpp"
                var $arrayidx51=((__ZL5peaks+$51*80)&4294967295); //@line 1021 "wavegen.cpp"
                var $right152=(($arrayidx51+40)&4294967295); //@line 1021 "wavegen.cpp"
                var $52=HEAPF32[(($right152)>>2)]; //@line 1021 "wavegen.cpp"
                var $conv53=((($52))|0); //@line 1021 "wavegen.cpp"
                var $53=$ix; //@line 1021 "wavegen.cpp"
                var $arrayidx54=((__ZL5peaks+$53*80)&4294967295); //@line 1021 "wavegen.cpp"
                var $right=(($arrayidx54+12)&4294967295); //@line 1021 "wavegen.cpp"
                HEAP32[(($right)>>2)]=$conv53; //@line 1021 "wavegen.cpp"
                ; //@line 1022 "wavegen.cpp"
              }
              else if (__label__ == 13) {
  
                var $left56=(($arrayidx48+8)&4294967295); //@line 1025 "wavegen.cpp"
                var $54=HEAP32[(($left56)>>2)]; //@line 1025 "wavegen.cpp"
                var $55=$ix; //@line 1025 "wavegen.cpp"
                var $arrayidx57=((__ZL5peaks+$55*80)&4294967295); //@line 1025 "wavegen.cpp"
                var $right58=(($arrayidx57+12)&4294967295); //@line 1025 "wavegen.cpp"
                HEAP32[(($right58)>>2)]=$54; //@line 1025 "wavegen.cpp"
                ;
              }
  
              var $56=$ix; //@line 1009 "wavegen.cpp"
              var $inc=((($56)+1)&4294967295); //@line 1009 "wavegen.cpp"
              $ix=$inc; //@line 1009 "wavegen.cpp"
              var $57=$ix; //@line 1009 "wavegen.cpp"
              var $58=HEAP32[((_wvoice)>>2)]; //@line 1009 "wavegen.cpp"
              var $n_harmonic_peaks=(($58+84)&4294967295); //@line 1009 "wavegen.cpp"
              var $59=HEAP32[(($n_harmonic_peaks)>>2)]; //@line 1009 "wavegen.cpp"
              var $cmp19=((($57))|0) <= ((($59))|0); //@line 1009 "wavegen.cpp"
              if ($cmp19) { __lastLabel__ = 14; __label__ = 9;continue ; } else { __lastLabel__ = 14; __label__ = 8;break $for_body$$for_cond60_preheader$13; } //@line 1009 "wavegen.cpp"
            }
          }
        } while(0);
  
        var $_lcssa=__lastLabel__ == 7 ? $21 : ($57);
        var $cmp611=((($_lcssa))|0) < 8; //@line 1028 "wavegen.cpp"
        if (!($cmp611)) { __label__ = 21;break $for_end93$$if_end18$11; } //@line 1028 "wavegen.cpp"
  
        var $_pr=$ix; //@line 1031 "wavegen.cpp"
        __lastLabel__ = 15; ;
        while(1) { 
  
          var $60=__lastLabel__ == 15 ? $_pr : ($inc92); //@line 1031 "wavegen.cpp"
          var $cmp63=((($60))|0) < 7; //@line 1031 "wavegen.cpp"
          if ($cmp63) { __label__ = 17;; } else { __label__ = 18;; } //@line 1031 "wavegen.cpp"
          if (__label__ == 17) {
  
            var $61=$ix; //@line 1033 "wavegen.cpp"
            var $arrayidx65=((__ZL5peaks+$61*80)&4294967295); //@line 1033 "wavegen.cpp"
            var $freq_inc66=(($arrayidx65+48)&4294967295); //@line 1033 "wavegen.cpp"
            var $62=HEAPF32[(($freq_inc66)>>2)]; //@line 1033 "wavegen.cpp"
            var $63=$ix; //@line 1033 "wavegen.cpp"
            var $arrayidx67=((__ZL5peaks+$63*80)&4294967295); //@line 1033 "wavegen.cpp"
            var $freq168=(($arrayidx67+16)&4294967295); //@line 1033 "wavegen.cpp"
            var $64=HEAPF32[(($freq168)>>2)]; //@line 1033 "wavegen.cpp"
            var $add69=($64)+($62); //@line 1033 "wavegen.cpp"
            HEAPF32[(($freq168)>>2)]=$add69; //@line 1033 "wavegen.cpp"
            var $65=$ix; //@line 1034 "wavegen.cpp"
            var $arrayidx70=((__ZL5peaks+$65*80)&4294967295); //@line 1034 "wavegen.cpp"
            var $freq171=(($arrayidx70+16)&4294967295); //@line 1034 "wavegen.cpp"
            var $66=HEAPF32[(($freq171)>>2)]; //@line 1034 "wavegen.cpp"
            var $conv72=((($66))|0); //@line 1034 "wavegen.cpp"
            var $67=$ix; //@line 1034 "wavegen.cpp"
            var $arrayidx73=((__ZL5peaks+$67*80)&4294967295); //@line 1034 "wavegen.cpp"
            var $freq74=(($arrayidx73)&4294967295); //@line 1034 "wavegen.cpp"
            HEAP32[(($freq74)>>2)]=$conv72; //@line 1034 "wavegen.cpp"
            ; //@line 1035 "wavegen.cpp"
          }
  
          var $68=$ix; //@line 1036 "wavegen.cpp"
          var $arrayidx76=((__ZL5peaks+$68*80)&4294967295); //@line 1036 "wavegen.cpp"
          var $height_inc77=(($arrayidx76+56)&4294967295); //@line 1036 "wavegen.cpp"
          var $69=HEAPF32[(($height_inc77)>>2)]; //@line 1036 "wavegen.cpp"
          var $70=$ix; //@line 1036 "wavegen.cpp"
          var $arrayidx78=((__ZL5peaks+$70*80)&4294967295); //@line 1036 "wavegen.cpp"
          var $height179=(($arrayidx78+24)&4294967295); //@line 1036 "wavegen.cpp"
          var $71=HEAPF32[(($height179)>>2)]; //@line 1036 "wavegen.cpp"
          var $add80=($71)+($69); //@line 1036 "wavegen.cpp"
          HEAPF32[(($height179)>>2)]=$add80; //@line 1036 "wavegen.cpp"
          var $72=$ix; //@line 1037 "wavegen.cpp"
          var $arrayidx81=((__ZL5peaks+$72*80)&4294967295); //@line 1037 "wavegen.cpp"
          var $height182=(($arrayidx81+24)&4294967295); //@line 1037 "wavegen.cpp"
          var $73=HEAPF32[(($height182)>>2)]; //@line 1037 "wavegen.cpp"
          var $conv83=((($73))|0); //@line 1037 "wavegen.cpp"
          var $74=$ix; //@line 1037 "wavegen.cpp"
          var $arrayidx84=((__ZL5peaks+$74*80)&4294967295); //@line 1037 "wavegen.cpp"
          var $height85=(($arrayidx84+4)&4294967295); //@line 1037 "wavegen.cpp"
          HEAP32[(($height85)>>2)]=$conv83; //@line 1037 "wavegen.cpp"
          var $cmp86=((($conv83))|0) < 0; //@line 1037 "wavegen.cpp"
          if ($cmp86) { __label__ = 19;; } else { __label__ = 20;; } //@line 1037 "wavegen.cpp"
          if (__label__ == 19) {
  
            var $75=$ix; //@line 1038 "wavegen.cpp"
            var $arrayidx88=((__ZL5peaks+$75*80)&4294967295); //@line 1038 "wavegen.cpp"
            var $height89=(($arrayidx88+4)&4294967295); //@line 1038 "wavegen.cpp"
            HEAP32[(($height89)>>2)]=0; //@line 1038 "wavegen.cpp"
            ; //@line 1038 "wavegen.cpp"
          }
  
          var $76=$ix; //@line 1028 "wavegen.cpp"
          var $inc92=((($76)+1)&4294967295); //@line 1028 "wavegen.cpp"
          $ix=$inc92; //@line 1028 "wavegen.cpp"
          var $cmp61=((($inc92))|0) < 8; //@line 1028 "wavegen.cpp"
          if ($cmp61) { __lastLabel__ = 20; __label__ = 16;continue ; } else { __lastLabel__ = 20; __label__ = 21;break $for_end93$$if_end18$11; } //@line 1028 "wavegen.cpp"
        }
      }
    } while(0);
  
    ;
    return; //@line 1049 "wavegen.cpp"
    return;
  }
  

  function __ZL9SetBreathv() {
    ;
    var __label__;
  
    var $pk;
    var $0=HEAP32[((_wvoice)>>2)]; //@line 1121 "wavegen.cpp"
    var $breath=(($0+272)&4294967295); //@line 1121 "wavegen.cpp"
    var $arrayidx=(($breath)&4294967295); //@line 1121 "wavegen.cpp"
    var $1=HEAP32[(($arrayidx)>>2)]; //@line 1121 "wavegen.cpp"
    var $cmp=((($1))|0)==0; //@line 1121 "wavegen.cpp"
    if ($cmp) { __label__ = 5;; } else { __label__ = 1;; } //@line 1121 "wavegen.cpp"
    $for_end$$if_end$2: do { 
      if (__label__ == 1) {
  
        $pk=1; //@line 1124 "wavegen.cpp"
        ; //@line 1124 "wavegen.cpp"
        while(1) { 
  
          var $2=$pk; //@line 1126 "wavegen.cpp"
          var $3=HEAP32[((_wvoice)>>2)]; //@line 1126 "wavegen.cpp"
          var $breath2=(($3+272)&4294967295); //@line 1126 "wavegen.cpp"
          var $arrayidx3=(($breath2+$2*4)&4294967295); //@line 1126 "wavegen.cpp"
          var $4=HEAP32[(($arrayidx3)>>2)]; //@line 1126 "wavegen.cpp"
          var $cmp4=((($4))|0)!=0; //@line 1126 "wavegen.cpp"
          if ($cmp4) { __label__ = 3;; } else { __label__ = 4;; } //@line 1126 "wavegen.cpp"
          if (__label__ == 3) {
  
            var $5=$pk; //@line 1130 "wavegen.cpp"
            var $arrayidx6=((__ZL7rbreath+$5*40)&4294967295); //@line 1130 "wavegen.cpp"
            var $6=$pk; //@line 1130 "wavegen.cpp"
            var $arrayidx7=((__ZL5peaks+$6*80)&4294967295); //@line 1130 "wavegen.cpp"
            var $freq=(($arrayidx7)&4294967295); //@line 1130 "wavegen.cpp"
            var $7=HEAP32[(($freq)>>2)]; //@line 1130 "wavegen.cpp"
            var $shr=($7) >> 16; //@line 1130 "wavegen.cpp"
            var $8=$pk; //@line 1130 "wavegen.cpp"
            var $9=HEAP32[((_wvoice)>>2)]; //@line 1130 "wavegen.cpp"
            var $breathw=(($9+308)&4294967295); //@line 1130 "wavegen.cpp"
            var $arrayidx8=(($breathw+$8*4)&4294967295); //@line 1130 "wavegen.cpp"
            var $10=HEAP32[(($arrayidx8)>>2)]; //@line 1130 "wavegen.cpp"
            __ZL12setresonatorP9RESONATORiii($arrayidx6, $shr, $10, 0); //@line 1130 "wavegen.cpp"
            ; //@line 1131 "wavegen.cpp"
          }
  
          var $11=$pk; //@line 1124 "wavegen.cpp"
          var $inc=((($11)+1)&4294967295); //@line 1124 "wavegen.cpp"
          $pk=$inc; //@line 1124 "wavegen.cpp"
          var $cmp1=((($inc))|0) < 9; //@line 1124 "wavegen.cpp"
          if ($cmp1) { __label__ = 2;continue ; } else { __label__ = 5;break $for_end$$if_end$2; } //@line 1124 "wavegen.cpp"
        }
      }
    } while(0);
  
    ;
    return; //@line 1134 "wavegen.cpp"
    return;
  }
  

  function __ZL11ApplyBreathv() {
    ;
    var __label__;
  
    var $value;
    var $noise;
    var $ix;
    var $amp;
    $value=0; //@line 1139 "wavegen.cpp"
    var $call=_rand(); //@line 1146 "wavegen.cpp"
    var $and=($call) & 16383; //@line 1146 "wavegen.cpp"
    var $sub=((($and)-8192)&4294967295); //@line 1146 "wavegen.cpp"
    $noise=$sub; //@line 1146 "wavegen.cpp"
    $ix=1; //@line 1148 "wavegen.cpp"
    ; //@line 1148 "wavegen.cpp"
    while(1) { 
  
      var $0=$ix; //@line 1150 "wavegen.cpp"
      var $1=HEAP32[((_wvoice)>>2)]; //@line 1150 "wavegen.cpp"
      var $breath=(($1+272)&4294967295); //@line 1150 "wavegen.cpp"
      var $arrayidx=(($breath+$0*4)&4294967295); //@line 1150 "wavegen.cpp"
      var $2=HEAP32[(($arrayidx)>>2)]; //@line 1150 "wavegen.cpp"
      $amp=$2; //@line 1150 "wavegen.cpp"
      var $cmp1=((($2))|0)!=0; //@line 1150 "wavegen.cpp"
      if ($cmp1) { __label__ = 2;; } else { __label__ = 3;; } //@line 1150 "wavegen.cpp"
      if (__label__ == 2) {
  
        var $3=$ix; //@line 1152 "wavegen.cpp"
        var $arrayidx2=((__ZL5peaks+$3*80)&4294967295); //@line 1152 "wavegen.cpp"
        var $height=(($arrayidx2+4)&4294967295); //@line 1152 "wavegen.cpp"
        var $4=HEAP32[(($height)>>2)]; //@line 1152 "wavegen.cpp"
        var $shr=($4) >> 14; //@line 1152 "wavegen.cpp"
        var $5=$amp; //@line 1152 "wavegen.cpp"
        var $mul=((($5)*($shr))&4294967295); //@line 1152 "wavegen.cpp"
        $amp=$mul; //@line 1152 "wavegen.cpp"
        var $6=$ix; //@line 1153 "wavegen.cpp"
        var $arrayidx3=((__ZL7rbreath+$6*40)&4294967295); //@line 1153 "wavegen.cpp"
        var $7=$noise; //@line 1153 "wavegen.cpp"
        var $conv=((($7))|0); //@line 1153 "wavegen.cpp"
        var $call4=__ZL9resonatorP9RESONATORd($arrayidx3, $conv); //@line 1153 "wavegen.cpp"
        var $8=$amp; //@line 1153 "wavegen.cpp"
        var $conv5=((($8))|0); //@line 1153 "wavegen.cpp"
        var $mul6=($call4)*($conv5); //@line 1153 "wavegen.cpp"
        var $conv7=((($mul6))|0); //@line 1153 "wavegen.cpp"
        var $9=$value; //@line 1153 "wavegen.cpp"
        var $add=((($conv7)+($9))&4294967295); //@line 1153 "wavegen.cpp"
        $value=$add; //@line 1153 "wavegen.cpp"
        ; //@line 1154 "wavegen.cpp"
      }
  
      var $10=$ix; //@line 1148 "wavegen.cpp"
      var $inc=((($10)+1)&4294967295); //@line 1148 "wavegen.cpp"
      $ix=$inc; //@line 1148 "wavegen.cpp"
      var $cmp=((($inc))|0) < 9; //@line 1148 "wavegen.cpp"
      if ($cmp) { __label__ = 1;continue ; } else { __label__ = 4;break ; } //@line 1148 "wavegen.cpp"
    }
  
    var $11=$value; //@line 1157 "wavegen.cpp"
    ;
    return $11; //@line 1157 "wavegen.cpp"
    return null;
  }
  

  function __Z11SetEmbeddedii($control, $value) {
    ;
    var __label__;
    var __lastLabel__ = null;
  
    var $control_addr;
    var $value_addr;
    var $sign;
    var $command;
    $control_addr=$control;
    $value_addr=$value;
    $sign=0; //@line 1567 "wavegen.cpp"
    var $0=$control_addr; //@line 1570 "wavegen.cpp"
    var $and=($0) & 31; //@line 1570 "wavegen.cpp"
    $command=$and; //@line 1570 "wavegen.cpp"
    var $1=$control_addr; //@line 1571 "wavegen.cpp"
    var $and1=($1) & 96; //@line 1571 "wavegen.cpp"
    var $cmp=((($and1))|0)==96; //@line 1571 "wavegen.cpp"
    if ($cmp) { __label__ = 1;; } else { __label__ = 2;; } //@line 1571 "wavegen.cpp"
    $if_then$$if_else$2: do { 
      if (__label__ == 1) {
  
        $sign=-1; //@line 1572 "wavegen.cpp"
        ; //@line 1572 "wavegen.cpp"
      }
      else if (__label__ == 2) {
  
        var $2=$control_addr; //@line 1574 "wavegen.cpp"
        var $and2=($2) & 96; //@line 1574 "wavegen.cpp"
        var $cmp3=((($and2))|0)==64; //@line 1574 "wavegen.cpp"
        if (!($cmp3)) { __label__ = 4;break $if_then$$if_else$2; } //@line 1574 "wavegen.cpp"
  
        $sign=1; //@line 1575 "wavegen.cpp"
        ; //@line 1575 "wavegen.cpp"
      }
    } while(0);
  
    var $3=$command; //@line 1577 "wavegen.cpp"
    var $cmp6=((($3))|0) < 15; //@line 1577 "wavegen.cpp"
    if ($cmp6) { __lastLabel__ = 4; __label__ = 5;; } else { __lastLabel__ = 4; __label__ = 9;; } //@line 1577 "wavegen.cpp"
    if (__label__ == 5) {
  
      var $4=$sign; //@line 1579 "wavegen.cpp"
      var $cmp8=((($4))|0)==0; //@line 1579 "wavegen.cpp"
      var $5=$value_addr; //@line 1580 "wavegen.cpp"
      if ($cmp8) { __label__ = 6;; } else { __label__ = 7;; } //@line 1579 "wavegen.cpp"
      if (__label__ == 6) {
  
        var $6=$command; //@line 1580 "wavegen.cpp"
        var $arrayidx=((_embedded_value+$6*4)&4294967295); //@line 1580 "wavegen.cpp"
        HEAP32[(($arrayidx)>>2)]=$5; //@line 1580 "wavegen.cpp"
        ; //@line 1580 "wavegen.cpp"
      }
      else if (__label__ == 7) {
  
        var $7=$sign; //@line 1582 "wavegen.cpp"
        var $mul=((($7)*($5))&4294967295); //@line 1582 "wavegen.cpp"
        var $8=$command; //@line 1582 "wavegen.cpp"
        var $arrayidx11=((_embedded_value+$8*4)&4294967295); //@line 1582 "wavegen.cpp"
        var $9=HEAP32[(($arrayidx11)>>2)]; //@line 1582 "wavegen.cpp"
        var $add=((($9)+($mul))&4294967295); //@line 1582 "wavegen.cpp"
        HEAP32[(($arrayidx11)>>2)]=$add; //@line 1582 "wavegen.cpp"
        ;
      }
  
      var $10=$command; //@line 1583 "wavegen.cpp"
      var $arrayidx13=((_embedded_value+$10*4)&4294967295); //@line 1583 "wavegen.cpp"
      var $11=HEAP32[(($arrayidx13)>>2)]; //@line 1583 "wavegen.cpp"
      var $12=$command; //@line 1583 "wavegen.cpp"
      var $arrayidx14=((__ZL12embedded_max+$12*4)&4294967295); //@line 1583 "wavegen.cpp"
      var $13=HEAP32[(($arrayidx14)>>2)]; //@line 1583 "wavegen.cpp"
      var $call=__ZL13SetWithRange0ii($11, $13); //@line 1583 "wavegen.cpp"
      var $14=$command; //@line 1583 "wavegen.cpp"
      var $arrayidx15=((_embedded_value+$14*4)&4294967295); //@line 1583 "wavegen.cpp"
      HEAP32[(($arrayidx15)>>2)]=$call; //@line 1583 "wavegen.cpp"
      var $_pr=$command; //@line 1586 "wavegen.cpp"
      __lastLabel__ = 8; ; //@line 1584 "wavegen.cpp"
    }
  
    var $15=__lastLabel__ == 8 ? $_pr : ($3); //@line 1586 "wavegen.cpp"
    if ($15 == 6) {
      __label__ = 10;;
    }
    else if ($15 == 1) {
      __label__ = 11;;
    }
    else if ($15 == 3) {
      __label__ = 12;;
    }
    else if ($15 == 13) {
      __label__ = 13;;
    }
    else if ($15 == 5) {
      __label__ = 14;;
    }
    else {
    __label__ = 15;;
    }
    
    $sw_epilog$$sw_bb$$sw_bb17$$sw_bb18$$sw_bb20$$sw_bb22$14: do { 
      if (__label__ == 10) {
  
        __ZL14WavegenSetEchov(); //@line 1589 "wavegen.cpp"
        __label__ = 11;break $sw_epilog$$sw_bb$$sw_bb17$$sw_bb18$$sw_bb20$$sw_bb22$14; //@line 1589 "wavegen.cpp"
      }
      else if (__label__ == 12) {
  
        var $call19=__Z12GetAmplitudev(); //@line 1595 "wavegen.cpp"
        HEAP32[((__ZL17general_amplitude)>>2)]=$call19; //@line 1595 "wavegen.cpp"
        __label__ = 15;break $sw_epilog$$sw_bb$$sw_bb17$$sw_bb18$$sw_bb20$$sw_bb22$14; //@line 1596 "wavegen.cpp"
      }
      else if (__label__ == 13) {
  
        var $call21=__Z12GetAmplitudev(); //@line 1599 "wavegen.cpp"
        HEAP32[((__ZL17general_amplitude)>>2)]=$call21; //@line 1599 "wavegen.cpp"
        __label__ = 15;break $sw_epilog$$sw_bb$$sw_bb17$$sw_bb18$$sw_bb20$$sw_bb22$14; //@line 1600 "wavegen.cpp"
      }
      else if (__label__ == 14) {
  
        __ZL14WavegenSetEchov(); //@line 1603 "wavegen.cpp"
        __label__ = 15;break $sw_epilog$$sw_bb$$sw_bb17$$sw_bb18$$sw_bb20$$sw_bb22$14; //@line 1604 "wavegen.cpp"
      }
    } while(0);
    if (__label__ == 11) {
  
      __ZL16SetPitchFormantsv(); //@line 1591 "wavegen.cpp"
      ; //@line 1592 "wavegen.cpp"
    }
  
    ;
    return; //@line 1606 "wavegen.cpp"
    return;
  }
  

  function __ZL13SetWithRange0ii($value, $max) {
    ;
    var __label__;
  
    var $retval;
    var $value_addr;
    var $max_addr;
    $value_addr=$value;
    $max_addr=$max;
    var $0=$value_addr; //@line 1535 "wavegen.cpp"
    var $cmp=((($0))|0) < 0; //@line 1535 "wavegen.cpp"
    if ($cmp) { __label__ = 1;; } else { __label__ = 2;; } //@line 1535 "wavegen.cpp"
    if (__label__ == 1) {
  
      $retval=0; //@line 1536 "wavegen.cpp"
      ; //@line 1536 "wavegen.cpp"
    }
    else if (__label__ == 2) {
  
      var $1=$value_addr; //@line 1537 "wavegen.cpp"
      var $2=$max_addr; //@line 1537 "wavegen.cpp"
      var $cmp1=((($1))|0) > ((($2))|0); //@line 1537 "wavegen.cpp"
      if ($cmp1) { __label__ = 3;; } else { __label__ = 4;; } //@line 1537 "wavegen.cpp"
      if (__label__ == 3) {
  
        var $3=$max_addr; //@line 1538 "wavegen.cpp"
        $retval=$3; //@line 1538 "wavegen.cpp"
        ; //@line 1538 "wavegen.cpp"
      }
      else if (__label__ == 4) {
  
        var $4=$value_addr; //@line 1539 "wavegen.cpp"
        $retval=$4; //@line 1539 "wavegen.cpp"
        ; //@line 1539 "wavegen.cpp"
      }
    }
  
    var $5=$retval; //@line 1540 "wavegen.cpp"
    ;
    return $5; //@line 1540 "wavegen.cpp"
    return null;
  }
  

  function __ZL14WavegenSetEchov() {
    ;
    var __label__;
    var __lastLabel__ = null;
  
    var $delay;
    var $amp;
    var $0=HEAP32[((_wvoice)>>2)]; //@line 803 "wavegen.cpp"
    var $voicing=(($0+92)&4294967295); //@line 803 "wavegen.cpp"
    var $1=HEAP32[(($voicing)>>2)]; //@line 803 "wavegen.cpp"
    HEAP32[((__ZL7voicing)>>2)]=$1; //@line 803 "wavegen.cpp"
    var $2=HEAP32[((_wvoice)>>2)]; //@line 804 "wavegen.cpp"
    var $echo_delay=(($2+76)&4294967295); //@line 804 "wavegen.cpp"
    var $3=HEAP32[(($echo_delay)>>2)]; //@line 804 "wavegen.cpp"
    $delay=$3; //@line 804 "wavegen.cpp"
    var $4=HEAP32[((_wvoice)>>2)]; //@line 805 "wavegen.cpp"
    var $echo_amp=(($4+80)&4294967295); //@line 805 "wavegen.cpp"
    var $5=HEAP32[(($echo_amp)>>2)]; //@line 805 "wavegen.cpp"
    $amp=$5; //@line 805 "wavegen.cpp"
    var $6=$delay; //@line 807 "wavegen.cpp"
    var $cmp=((($6))|0) >= 5500; //@line 807 "wavegen.cpp"
    if ($cmp) { __lastLabel__ = 0; __label__ = 1;; } else { __lastLabel__ = 0; __label__ = 2;; } //@line 807 "wavegen.cpp"
    if (__label__ == 1) {
  
      $delay=5499; //@line 808 "wavegen.cpp"
      var $_pr=$amp; //@line 809 "wavegen.cpp"
      __lastLabel__ = 1; ; //@line 808 "wavegen.cpp"
    }
  
    var $7=__lastLabel__ == 1 ? $_pr : ($5); //@line 809 "wavegen.cpp"
    var $cmp1=((($7))|0) > 100; //@line 809 "wavegen.cpp"
    if ($cmp1) { __label__ = 3;; } else { __label__ = 4;; } //@line 809 "wavegen.cpp"
    if (__label__ == 3) {
  
      $amp=100; //@line 810 "wavegen.cpp"
      ; //@line 810 "wavegen.cpp"
    }
  
    var $dest$, $stop$, $stop4$, $fast$, $value4$;
    $dest$ = _echo_buf;
    $stop$ = $dest$ + 11000;
    $value4$ = 0;
    if ($value4$ < 0) $value4$ += 256;
    $value4$ = $value4$ + ($value4$<<8) + ($value4$<<16) + ($value4$*16777216);
    while ($dest$%4 !== 0 && $dest$ < $stop$) {
      ;; HEAP8[$dest$++] = 0;
    }
    $dest$ >>= 2;
    $stop4$ = $stop$ >> 2;
    while ($dest$ < $stop4$) {
    ;;;;;;;  HEAP32[$dest$++] = $value4$;
    }
    $dest$ <<= 2;
    while ($dest$ < $stop$) {
      ;; HEAP8[$dest$++] = 0;
    }; //@line 812 "wavegen.cpp"
    HEAP32[((_echo_tail)>>2)]=0; //@line 813 "wavegen.cpp"
    var $8=HEAP32[((((_embedded_value+20)&4294967295))>>2)]; //@line 815 "wavegen.cpp"
    var $cmp4=((($8))|0) > 0; //@line 815 "wavegen.cpp"
    if ($cmp4) { __label__ = 5;; } else { __label__ = 6;; } //@line 815 "wavegen.cpp"
    $if_end6_thread$$if_end6$8: do { 
      if (__label__ == 5) {
  
        var $9=HEAP32[((((_embedded_value+20)&4294967295))>>2)]; //@line 818 "wavegen.cpp"
        $amp=$9; //@line 818 "wavegen.cpp"
        $delay=130; //@line 819 "wavegen.cpp"
        ; //@line 831 "wavegen.cpp"
      }
      else if (__label__ == 6) {
  
        var $_pr1=$delay; //@line 831 "wavegen.cpp"
        var $cmp7=((($_pr1))|0)==0; //@line 831 "wavegen.cpp"
        if (!($cmp7)) { __label__ = 8;break $if_end6_thread$$if_end6$8; } //@line 831 "wavegen.cpp"
  
        $amp=0; //@line 832 "wavegen.cpp"
        ; //@line 832 "wavegen.cpp"
      }
    } while(0);
  
    var $10=$delay; //@line 834 "wavegen.cpp"
    var $11=HEAP32[((_samplerate)>>2)]; //@line 834 "wavegen.cpp"
    var $mul=((($11)*($10))&4294967295); //@line 834 "wavegen.cpp"
    var $div=((((($mul))|0)/1000)|0); //@line 834 "wavegen.cpp"
    HEAP32[((_echo_head)>>2)]=$div; //@line 834 "wavegen.cpp"
    var $12=HEAP32[((_echo_head)>>2)]; //@line 835 "wavegen.cpp"
    HEAP32[((__ZL11echo_length)>>2)]=$12; //@line 835 "wavegen.cpp"
    var $13=$amp; //@line 836 "wavegen.cpp"
    var $cmp10=((($13))|0)==0; //@line 836 "wavegen.cpp"
    if ($cmp10) { __lastLabel__ = 8; __label__ = 9;; } else { __lastLabel__ = 8; __label__ = 10;; } //@line 836 "wavegen.cpp"
    if (__label__ == 9) {
  
      HEAP32[((__ZL11echo_length)>>2)]=0; //@line 837 "wavegen.cpp"
      var $_pr3=$amp; //@line 838 "wavegen.cpp"
      __lastLabel__ = 9; ; //@line 837 "wavegen.cpp"
    }
  
    var $14=__lastLabel__ == 9 ? $_pr3 : ($13); //@line 838 "wavegen.cpp"
    var $cmp13=((($14))|0) > 20; //@line 838 "wavegen.cpp"
    if ($cmp13) { __label__ = 11;; } else { __label__ = 12;; } //@line 838 "wavegen.cpp"
    if (__label__ == 11) {
  
      var $15=HEAP32[((_echo_head)>>2)]; //@line 839 "wavegen.cpp"
      var $mul15=((($15)*2)&4294967295); //@line 839 "wavegen.cpp"
      HEAP32[((__ZL11echo_length)>>2)]=$mul15; //@line 839 "wavegen.cpp"
      ; //@line 839 "wavegen.cpp"
    }
  
    var $16=$amp; //@line 842 "wavegen.cpp"
    HEAP32[((_echo_amp)>>2)]=$16; //@line 842 "wavegen.cpp"
    var $call=__Z12GetAmplitudev(); //@line 844 "wavegen.cpp"
    HEAP32[((__ZL17general_amplitude)>>2)]=$call; //@line 844 "wavegen.cpp"
    var $17=HEAP32[((__ZL17general_amplitude)>>2)]; //@line 845 "wavegen.cpp"
    var $18=$amp; //@line 845 "wavegen.cpp"
    var $sub=((500-($18))&4294967295); //@line 845 "wavegen.cpp"
    var $mul17=((($sub)*($17))&4294967295); //@line 845 "wavegen.cpp"
    var $div18=((((($mul17))|0)/500)|0); //@line 845 "wavegen.cpp"
    HEAP32[((__ZL17general_amplitude)>>2)]=$div18; //@line 845 "wavegen.cpp"
    ;
    return; //@line 846 "wavegen.cpp"
    return;
  }
  

  function __ZL16SetPitchFormantsv() {
    ;
    var __label__;
  
    var $ix;
    var $factor;
    var $pitch_value;
    var $0=HEAP32[((((_embedded_value+4)&4294967295))>>2)]; //@line 1550 "wavegen.cpp"
    $pitch_value=$0; //@line 1550 "wavegen.cpp"
    var $cmp=((($0))|0) > 101; //@line 1550 "wavegen.cpp"
    if ($cmp) { __label__ = 1;; } else { __label__ = 2;; } //@line 1550 "wavegen.cpp"
    if (__label__ == 1) {
  
      $pitch_value=101; //@line 1551 "wavegen.cpp"
      ; //@line 1551 "wavegen.cpp"
    }
  
    var $1=$pitch_value; //@line 1553 "wavegen.cpp"
    var $sub=((($1)-50)&4294967295); //@line 1553 "wavegen.cpp"
    var $mul=((($sub)*25)&4294967295); //@line 1553 "wavegen.cpp"
    var $div=((((($mul))|0)/50)|0); //@line 1553 "wavegen.cpp"
    var $add=((($div)+256)&4294967295); //@line 1553 "wavegen.cpp"
    $factor=$add; //@line 1553 "wavegen.cpp"
    $ix=0; //@line 1554 "wavegen.cpp"
    ; //@line 1554 "wavegen.cpp"
    while(1) { 
  
      var $2=$ix; //@line 1556 "wavegen.cpp"
      var $3=HEAP32[((_wvoice)>>2)]; //@line 1556 "wavegen.cpp"
      var $freq2=(($3+216)&4294967295); //@line 1556 "wavegen.cpp"
      var $arrayidx=(($freq2+$2*2)&4294967295); //@line 1556 "wavegen.cpp"
      var $4=HEAP16[(($arrayidx)>>1)]; //@line 1556 "wavegen.cpp"
      var $conv=(tempInt=(($4)),(tempInt>=32768?tempInt-65536:tempInt)); //@line 1556 "wavegen.cpp"
      var $5=$factor; //@line 1556 "wavegen.cpp"
      var $mul2=((($conv)*($5))&4294967295); //@line 1556 "wavegen.cpp"
      var $div3=((((($mul2))|0)/256)|0); //@line 1556 "wavegen.cpp"
      var $conv4=((($div3)) & 65535); //@line 1556 "wavegen.cpp"
      var $6=$ix; //@line 1556 "wavegen.cpp"
      var $7=HEAP32[((_wvoice)>>2)]; //@line 1556 "wavegen.cpp"
      var $freq=(($7+144)&4294967295); //@line 1556 "wavegen.cpp"
      var $arrayidx5=(($freq+$6*2)&4294967295); //@line 1556 "wavegen.cpp"
      HEAP16[(($arrayidx5)>>1)]=$conv4; //@line 1556 "wavegen.cpp"
      var $8=$ix; //@line 1554 "wavegen.cpp"
      var $inc=((($8)+1)&4294967295); //@line 1554 "wavegen.cpp"
      $ix=$inc; //@line 1554 "wavegen.cpp"
      var $cmp1=((($inc))|0) <= 5; //@line 1554 "wavegen.cpp"
      if ($cmp1) { __label__ = 3;continue ; } else { __label__ = 4;break ; } //@line 1554 "wavegen.cpp"
    }
  
    var $9=HEAP32[((((_embedded_value+24)&4294967295))>>2)]; //@line 1558 "wavegen.cpp"
    var $mul6=((($9)*3)&4294967295); //@line 1558 "wavegen.cpp"
    $factor=$mul6; //@line 1558 "wavegen.cpp"
    var $10=HEAP32[((_wvoice)>>2)]; //@line 1559 "wavegen.cpp"
    var $height2=(($10+234)&4294967295); //@line 1559 "wavegen.cpp"
    var $arrayidx7=(($height2)&4294967295); //@line 1559 "wavegen.cpp"
    var $11=HEAP16[(($arrayidx7)>>1)]; //@line 1559 "wavegen.cpp"
    var $conv8=(tempInt=(($11)),(tempInt>=32768?tempInt-65536:tempInt)); //@line 1559 "wavegen.cpp"
    var $12=$factor; //@line 1559 "wavegen.cpp"
    var $mul9=((($12)*2)&4294967295); //@line 1559 "wavegen.cpp"
    var $sub10=((256-($mul9))&4294967295); //@line 1559 "wavegen.cpp"
    var $mul11=((($sub10)*($conv8))&4294967295); //@line 1559 "wavegen.cpp"
    var $div12=((((($mul11))|0)/256)|0); //@line 1559 "wavegen.cpp"
    var $conv13=((($div12)) & 65535); //@line 1559 "wavegen.cpp"
    var $13=HEAP32[((_wvoice)>>2)]; //@line 1559 "wavegen.cpp"
    var $height=(($13+162)&4294967295); //@line 1559 "wavegen.cpp"
    var $arrayidx14=(($height)&4294967295); //@line 1559 "wavegen.cpp"
    HEAP16[(($arrayidx14)>>1)]=$conv13; //@line 1559 "wavegen.cpp"
    var $14=HEAP32[((_wvoice)>>2)]; //@line 1560 "wavegen.cpp"
    var $height215=(($14+234)&4294967295); //@line 1560 "wavegen.cpp"
    var $arrayidx16=(($height215+2)&4294967295); //@line 1560 "wavegen.cpp"
    var $15=HEAP16[(($arrayidx16)>>1)]; //@line 1560 "wavegen.cpp"
    var $conv17=(tempInt=(($15)),(tempInt>=32768?tempInt-65536:tempInt)); //@line 1560 "wavegen.cpp"
    var $16=$factor; //@line 1560 "wavegen.cpp"
    var $sub18=((256-($16))&4294967295); //@line 1560 "wavegen.cpp"
    var $mul19=((($sub18)*($conv17))&4294967295); //@line 1560 "wavegen.cpp"
    var $div20=((((($mul19))|0)/256)|0); //@line 1560 "wavegen.cpp"
    var $conv21=((($div20)) & 65535); //@line 1560 "wavegen.cpp"
    var $17=HEAP32[((_wvoice)>>2)]; //@line 1560 "wavegen.cpp"
    var $height22=(($17+162)&4294967295); //@line 1560 "wavegen.cpp"
    var $arrayidx23=(($height22+2)&4294967295); //@line 1560 "wavegen.cpp"
    HEAP16[(($arrayidx23)>>1)]=$conv21; //@line 1560 "wavegen.cpp"
    ;
    return; //@line 1561 "wavegen.cpp"
    return;
  }
  

  function __Z15WavegenSetVoiceP7voice_t($v) {
    ;
    var __label__;
  
    var $v_addr;
    $v_addr=$v;
    var $0=$v_addr; //@line 1613 "wavegen.cpp"
    var $1=$0; //@line 1613 "wavegen.cpp"
    var $src$, $dest$, $stop$, $stop4$, $fast$;
    $src$ = $1;
    $dest$ = ((__ZZ15WavegenSetVoiceP7voice_tE2v2)&4294967295);
    $stop$ = $src$ + 1344;
    if (($dest$%4) == ($src$%4) && 1344 > 8) {
      while ($src$%4 !== 0 && $src$ < $stop$) {
        ; HEAP8[$dest$++] = HEAP8[$src$++];
      }
      $src$ >>= 2;
      $dest$ >>= 2;
      $stop4$ = $stop$ >> 2;
      while ($src$ < $stop4$) {
    ;;;    HEAP32[$dest$++] = HEAP32[$src$++];
      }
      $src$ <<= 2;
      $dest$ <<= 2;
    }while ($src$ < $stop$) {
      ; HEAP8[$dest$++] = HEAP8[$src$++];
    }; //@line 1613 "wavegen.cpp"
    HEAP32[((_wvoice)>>2)]=__ZZ15WavegenSetVoiceP7voice_tE2v2; //@line 1614 "wavegen.cpp"
    var $2=$v_addr; //@line 1616 "wavegen.cpp"
    var $peak_shape=(($2+88)&4294967295); //@line 1616 "wavegen.cpp"
    var $3=HEAP32[(($peak_shape)>>2)]; //@line 1616 "wavegen.cpp"
    var $cmp=((($3))|0)==0; //@line 1616 "wavegen.cpp"
    if ($cmp) { __label__ = 1;; } else { __label__ = 2;; } //@line 1616 "wavegen.cpp"
    if (__label__ == 1) {
  
      HEAP32[((__ZL8pk_shape)>>2)]=((_pk_shape1)&4294967295); //@line 1617 "wavegen.cpp"
      ; //@line 1617 "wavegen.cpp"
    }
    else if (__label__ == 2) {
  
      HEAP32[((__ZL8pk_shape)>>2)]=((__ZL9pk_shape2)&4294967295); //@line 1619 "wavegen.cpp"
      ;
    }
  
    var $4=$v_addr; //@line 1621 "wavegen.cpp"
    var $consonant_amp=(($4+100)&4294967295); //@line 1621 "wavegen.cpp"
    var $5=HEAP32[(($consonant_amp)>>2)]; //@line 1621 "wavegen.cpp"
    var $mul=((($5)*26)&4294967295); //@line 1621 "wavegen.cpp"
    var $div=((((($mul))|0)/100)|0); //@line 1621 "wavegen.cpp"
    HEAP32[((__ZL13consonant_amp)>>2)]=$div; //@line 1621 "wavegen.cpp"
    var $6=HEAP32[((_samplerate)>>2)]; //@line 1622 "wavegen.cpp"
    var $cmp1=((($6))|0) <= 11000; //@line 1622 "wavegen.cpp"
    if ($cmp1) { __label__ = 4;; } else { __label__ = 5;; } //@line 1622 "wavegen.cpp"
    if (__label__ == 4) {
  
      var $7=HEAP32[((__ZL13consonant_amp)>>2)]; //@line 1624 "wavegen.cpp"
      var $mul3=((($7)*2)&4294967295); //@line 1624 "wavegen.cpp"
      HEAP32[((__ZL13consonant_amp)>>2)]=$mul3; //@line 1624 "wavegen.cpp"
      HEAP8[(__ZL16option_harmonic1_b)]=1;
      ; //@line 1626 "wavegen.cpp"
    }
  
    __ZL14WavegenSetEchov(); //@line 1627 "wavegen.cpp"
    __ZL16SetPitchFormantsv(); //@line 1628 "wavegen.cpp"
    var $8=HEAP32[((_wvoice)>>2)]; //@line 1629 "wavegen.cpp"
    var $samplerate=(($8+108)&4294967295); //@line 1629 "wavegen.cpp"
    var $9=HEAP32[(($samplerate)>>2)]; //@line 1629 "wavegen.cpp"
    var $10=HEAP32[((_out_ptr)>>2)]; //@line 1629 "wavegen.cpp"
    __Z11MarkerEventijiPh(8, 0, $9, $10); //@line 1629 "wavegen.cpp"
    ;
    return; //@line 1631 "wavegen.cpp"
    return;
  }
  

  function __Z9SetPitch2P7voice_tiiPiS1_($voice, $pitch1, $pitch2, $pitch_base, $pitch_range) {
    ;
    var __label__;
  
    var $voice_addr;
    var $pitch1_addr;
    var $pitch2_addr;
    var $pitch_base_addr;
    var $pitch_range_addr;
    var $x;
    var $base;
    var $range;
    var $pitch_value;
    $voice_addr=$voice;
    $pitch1_addr=$pitch1;
    $pitch2_addr=$pitch2;
    $pitch_base_addr=$pitch_base;
    $pitch_range_addr=$pitch_range;
    var $0=$pitch1_addr; //@line 1656 "wavegen.cpp"
    var $1=$pitch2_addr; //@line 1656 "wavegen.cpp"
    var $cmp=((($0))|0) > ((($1))|0); //@line 1656 "wavegen.cpp"
    if ($cmp) { __label__ = 1;; } else { __label__ = 2;; } //@line 1656 "wavegen.cpp"
    if (__label__ == 1) {
  
      var $2=$pitch1_addr; //@line 1658 "wavegen.cpp"
      $x=$2; //@line 1658 "wavegen.cpp"
      var $3=$pitch2_addr; //@line 1659 "wavegen.cpp"
      $pitch1_addr=$3; //@line 1659 "wavegen.cpp"
      var $4=$x; //@line 1660 "wavegen.cpp"
      $pitch2_addr=$4; //@line 1660 "wavegen.cpp"
      ; //@line 1661 "wavegen.cpp"
    }
  
    var $5=HEAP32[((((_embedded_value+4)&4294967295))>>2)]; //@line 1663 "wavegen.cpp"
    $pitch_value=$5; //@line 1663 "wavegen.cpp"
    var $cmp1=((($5))|0) > 101; //@line 1663 "wavegen.cpp"
    if ($cmp1) { __label__ = 3;; } else { __label__ = 4;; } //@line 1663 "wavegen.cpp"
    if (__label__ == 3) {
  
      $pitch_value=101; //@line 1664 "wavegen.cpp"
      ; //@line 1664 "wavegen.cpp"
    }
  
    var $6=HEAP32[((((_embedded_value+24)&4294967295))>>2)]; //@line 1665 "wavegen.cpp"
    var $7=$pitch_value; //@line 1665 "wavegen.cpp"
    var $sub=((($7)-($6))&4294967295); //@line 1665 "wavegen.cpp"
    $pitch_value=$sub; //@line 1665 "wavegen.cpp"
    var $cmp4=((($sub))|0) < 0; //@line 1666 "wavegen.cpp"
    if ($cmp4) { __label__ = 5;; } else { __label__ = 6;; } //@line 1666 "wavegen.cpp"
    if (__label__ == 5) {
  
      $pitch_value=0; //@line 1667 "wavegen.cpp"
      ; //@line 1667 "wavegen.cpp"
    }
  
    var $8=$voice_addr; //@line 1669 "wavegen.cpp"
    var $pitch_base7=(($8+44)&4294967295); //@line 1669 "wavegen.cpp"
    var $9=HEAP32[(($pitch_base7)>>2)]; //@line 1669 "wavegen.cpp"
    var $10=$pitch_value; //@line 1669 "wavegen.cpp"
    var $arrayidx=((_pitch_adjust_tab+$10)&4294967295); //@line 1669 "wavegen.cpp"
    var $11=HEAPU8[($arrayidx)]; //@line 1669 "wavegen.cpp"
    var $conv=((($11))&255); //@line 1669 "wavegen.cpp"
    var $mul=((($conv)*($9))&4294967295); //@line 1669 "wavegen.cpp"
    var $div=((((($mul))|0)/128)|0); //@line 1669 "wavegen.cpp"
    $base=$div; //@line 1669 "wavegen.cpp"
    var $12=$voice_addr; //@line 1670 "wavegen.cpp"
    var $pitch_range8=(($12+48)&4294967295); //@line 1670 "wavegen.cpp"
    var $13=HEAP32[(($pitch_range8)>>2)]; //@line 1670 "wavegen.cpp"
    var $14=HEAP32[((((_embedded_value+16)&4294967295))>>2)]; //@line 1670 "wavegen.cpp"
    var $mul9=((($14)*($13))&4294967295); //@line 1670 "wavegen.cpp"
    var $div10=((((($mul9))|0)/50)|0); //@line 1670 "wavegen.cpp"
    $range=$div10; //@line 1670 "wavegen.cpp"
    var $15=$range; //@line 1673 "wavegen.cpp"
    var $16=$voice_addr; //@line 1673 "wavegen.cpp"
    var $pitch_range11=(($16+48)&4294967295); //@line 1673 "wavegen.cpp"
    var $17=HEAP32[(($pitch_range11)>>2)]; //@line 1673 "wavegen.cpp"
    var $sub12=((($15)-($17))&4294967295); //@line 1673 "wavegen.cpp"
    var $mul13=((($sub12)*18)&4294967295); //@line 1673 "wavegen.cpp"
    var $18=$base; //@line 1673 "wavegen.cpp"
    var $sub14=((($18)-($mul13))&4294967295); //@line 1673 "wavegen.cpp"
    $base=$sub14; //@line 1673 "wavegen.cpp"
    var $19=$base; //@line 1675 "wavegen.cpp"
    var $20=$pitch1_addr; //@line 1675 "wavegen.cpp"
    var $21=$range; //@line 1675 "wavegen.cpp"
    var $mul15=((($21)*($20))&4294967295); //@line 1675 "wavegen.cpp"
    var $div16=((((($mul15))|0)/2)|0); //@line 1675 "wavegen.cpp"
    var $add=((($div16)+($19))&4294967295); //@line 1675 "wavegen.cpp"
    var $22=$pitch_base_addr; //@line 1675 "wavegen.cpp"
    HEAP32[(($22)>>2)]=$add; //@line 1675 "wavegen.cpp"
    var $23=$base; //@line 1676 "wavegen.cpp"
    var $24=$pitch2_addr; //@line 1676 "wavegen.cpp"
    var $25=$range; //@line 1676 "wavegen.cpp"
    var $mul17=((($25)*($24))&4294967295); //@line 1676 "wavegen.cpp"
    var $div18=((((($mul17))|0)/2)|0); //@line 1676 "wavegen.cpp"
    var $26=$pitch_base_addr; //@line 1676 "wavegen.cpp"
    var $27=HEAP32[(($26)>>2)]; //@line 1676 "wavegen.cpp"
    var $_neg=(((-($27)))&4294967295);
    var $add19=((($div18)+($23))&4294967295); //@line 1676 "wavegen.cpp"
    var $sub20=((($add19)+($_neg))&4294967295); //@line 1676 "wavegen.cpp"
    var $28=$pitch_range_addr; //@line 1676 "wavegen.cpp"
    HEAP32[(($28)>>2)]=$sub20; //@line 1676 "wavegen.cpp"
    ;
    return; //@line 1677 "wavegen.cpp"
    return;
  }
  

  function __Z8SetPitchiPhii($length, $env, $pitch1, $pitch2) {
    ;
    var __label__;
  
    var $length_addr;
    var $env_addr;
    var $pitch1_addr;
    var $pitch2_addr;
    $length_addr=$length;
    $env_addr=$env;
    $pitch1_addr=$pitch1;
    $pitch2_addr=$pitch2;
    var $0=$env_addr; //@line 1696 "wavegen.cpp"
    HEAP32[((((_wdata)&4294967295))>>2)]=$0; //@line 1696 "wavegen.cpp"
    var $cmp=((($0))|0)==0; //@line 1696 "wavegen.cpp"
    if ($cmp) { __label__ = 1;; } else { __label__ = 2;; } //@line 1696 "wavegen.cpp"
    if (__label__ == 1) {
  
      HEAP32[((((_wdata)&4294967295))>>2)]=((_env_fall)&4294967295); //@line 1697 "wavegen.cpp"
      ; //@line 1697 "wavegen.cpp"
    }
  
    HEAP32[((((_wdata+8)&4294967295))>>2)]=0; //@line 1699 "wavegen.cpp"
    var $1=$length_addr; //@line 1700 "wavegen.cpp"
    var $cmp1=((($1))|0)==0; //@line 1700 "wavegen.cpp"
    if ($cmp1) { __label__ = 3;; } else { __label__ = 4;; } //@line 1700 "wavegen.cpp"
    if (__label__ == 3) {
  
      HEAP32[((((_wdata+12)&4294967295))>>2)]=0; //@line 1701 "wavegen.cpp"
      ; //@line 1701 "wavegen.cpp"
    }
    else if (__label__ == 4) {
  
      var $2=$length_addr; //@line 1703 "wavegen.cpp"
      var $div=((2097152/((($2))|0))|0); //@line 1703 "wavegen.cpp"
      HEAP32[((((_wdata+12)&4294967295))>>2)]=$div; //@line 1703 "wavegen.cpp"
      ;
    }
  
    var $3=HEAP32[((_wvoice)>>2)]; //@line 1705 "wavegen.cpp"
    var $4=$pitch1_addr; //@line 1705 "wavegen.cpp"
    var $5=$pitch2_addr; //@line 1705 "wavegen.cpp"
    __Z9SetPitch2P7voice_tiiPiS1_($3, $4, $5, ((_wdata+16)&4294967295), ((_wdata+20)&4294967295)); //@line 1705 "wavegen.cpp"
    var $6=HEAP32[((((_wdata)&4294967295))>>2)]; //@line 1707 "wavegen.cpp"
    var $arrayidx=(($6)&4294967295); //@line 1707 "wavegen.cpp"
    var $7=HEAPU8[($arrayidx)]; //@line 1707 "wavegen.cpp"
    var $conv=((($7))&255); //@line 1707 "wavegen.cpp"
    var $8=HEAP32[((((_wdata+20)&4294967295))>>2)]; //@line 1707 "wavegen.cpp"
    var $mul=((($conv)*($8))&4294967295); //@line 1707 "wavegen.cpp"
    var $shr=($mul) >> 8; //@line 1707 "wavegen.cpp"
    var $9=HEAP32[((((_wdata+16)&4294967295))>>2)]; //@line 1707 "wavegen.cpp"
    var $add=((($shr)+($9))&4294967295); //@line 1707 "wavegen.cpp"
    HEAP32[((((_wdata+4)&4294967295))>>2)]=$add; //@line 1707 "wavegen.cpp"
    var $10=HEAP32[((_wvoice)>>2)]; //@line 1709 "wavegen.cpp"
    var $flutter=(($10+68)&4294967295); //@line 1709 "wavegen.cpp"
    var $11=HEAP32[(($flutter)>>2)]; //@line 1709 "wavegen.cpp"
    HEAP32[((__ZL11flutter_amp)>>2)]=$11; //@line 1709 "wavegen.cpp"
    ;
    return; //@line 1711 "wavegen.cpp"
    return;
  }
  

  function __Z8SetSynthiiP7frame_tS0_P7voice_t($length, $modn, $fr1, $fr2, $v) {
    ;
    var __label__;
    var __lastLabel__ = null;
  
    var $length_addr;
    var $modn_addr;
    var $fr1_addr;
    var $fr2_addr;
    var $v_addr;
    var $ix;
    var $next;
    var $length2;
    var $length4;
    var $qix;
    var $cmd;
    $length_addr=$length;
    $modn_addr=$modn;
    $fr1_addr=$fr1;
    $fr2_addr=$fr2;
    $v_addr=$v;
    HEAP8[(__ZL8end_wave_b)]=1;
    var $0=$modn_addr; //@line 1749 "wavegen.cpp"
    var $and=($0) & 255; //@line 1749 "wavegen.cpp"
    HEAP32[((__ZL15modulation_type)>>2)]=$and; //@line 1749 "wavegen.cpp"
    HEAP32[((__ZL12glottal_flag)>>2)]=0; //@line 1751 "wavegen.cpp"
    var $1=$modn_addr; //@line 1752 "wavegen.cpp"
    var $and1=($1) & 1024; //@line 1752 "wavegen.cpp"
    var $tobool=((($and1))|0)!=0; //@line 1752 "wavegen.cpp"
    if ($tobool) { __label__ = 1;; } else { __label__ = 2;; } //@line 1752 "wavegen.cpp"
    if (__label__ == 1) {
  
      HEAP32[((__ZL12glottal_flag)>>2)]=3; //@line 1754 "wavegen.cpp"
      var $2=$modn_addr; //@line 1755 "wavegen.cpp"
      var $shr=($2) >> 8; //@line 1755 "wavegen.cpp"
      var $and2=($shr) & 3; //@line 1755 "wavegen.cpp"
      var $arrayidx=((__ZZ8SetSynthiiP7frame_tS0_P7voice_tE19glottal_reduce_tab1+$and2*4)&4294967295); //@line 1755 "wavegen.cpp"
      var $3=HEAP32[(($arrayidx)>>2)]; //@line 1755 "wavegen.cpp"
      HEAP32[((__ZL14glottal_reduce)>>2)]=$3; //@line 1755 "wavegen.cpp"
      ; //@line 1756 "wavegen.cpp"
    }
  
    var $4=$modn_addr; //@line 1757 "wavegen.cpp"
    var $and3=($4) & 2048; //@line 1757 "wavegen.cpp"
    var $tobool4=((($and3))|0)!=0; //@line 1757 "wavegen.cpp"
    if ($tobool4) { __label__ = 3;; } else { __label__ = 4;; } //@line 1757 "wavegen.cpp"
    if (__label__ == 3) {
  
      HEAP32[((__ZL12glottal_flag)>>2)]=4; //@line 1759 "wavegen.cpp"
      var $5=$modn_addr; //@line 1760 "wavegen.cpp"
      var $shr6=($5) >> 8; //@line 1760 "wavegen.cpp"
      var $and7=($shr6) & 3; //@line 1760 "wavegen.cpp"
      var $arrayidx8=((__ZZ8SetSynthiiP7frame_tS0_P7voice_tE19glottal_reduce_tab2+$and7*4)&4294967295); //@line 1760 "wavegen.cpp"
      var $6=HEAP32[(($arrayidx8)>>2)]; //@line 1760 "wavegen.cpp"
      HEAP32[((__ZL14glottal_reduce)>>2)]=$6; //@line 1760 "wavegen.cpp"
      ; //@line 1761 "wavegen.cpp"
    }
  
    var $7=HEAP32[((_wcmdq_head)>>2)]; //@line 1763 "wavegen.cpp"
    var $add=((($7)+1)&4294967295); //@line 1763 "wavegen.cpp"
    $qix=$add; //@line 1763 "wavegen.cpp"
    __lastLabel__ = 4; ; //@line 1763 "wavegen.cpp"
    $for_cond$8: while(1) { 
  
      var $8=__lastLabel__ == 11 ? $inc : ($add); //@line 1765 "wavegen.cpp"
      var $cmp=((($8))|0) >= 160; //@line 1765 "wavegen.cpp"
      if ($cmp) { __label__ = 6;; } else { __label__ = 7;; } //@line 1765 "wavegen.cpp"
      if (__label__ == 6) {
  
        $qix=0; //@line 1765 "wavegen.cpp"
        ; //@line 1765 "wavegen.cpp"
      }
  
      var $9=$qix; //@line 1766 "wavegen.cpp"
      var $10=HEAP32[((_wcmdq_tail)>>2)]; //@line 1766 "wavegen.cpp"
      var $cmp12=((($9))|0)==((($10))|0); //@line 1766 "wavegen.cpp"
      if ($cmp12) { __label__ = 12;break $for_cond$8; } //@line 1766 "wavegen.cpp"
  
      var $11=$qix; //@line 1768 "wavegen.cpp"
      var $arrayidx15=((_wcmdq+$11*16)&4294967295); //@line 1768 "wavegen.cpp"
      var $arrayidx16=(($arrayidx15)&4294967295); //@line 1768 "wavegen.cpp"
      var $12=HEAP32[(($arrayidx16)>>2)]; //@line 1768 "wavegen.cpp"
      $cmd=$12; //@line 1768 "wavegen.cpp"
      var $13=$cmd; //@line 1769 "wavegen.cpp"
      var $cmp17=((($13))|0)==3; //@line 1769 "wavegen.cpp"
      if ($cmp17) { __label__ = 9;break $for_cond$8; } //@line 1769 "wavegen.cpp"
  
      var $14=$cmd; //@line 1774 "wavegen.cpp"
      var $cmp20=((($14))|0)==6; //@line 1774 "wavegen.cpp"
      var $15=$cmd; //@line 1774 "wavegen.cpp"
      var $cmp21=((($15))|0)==5; //@line 1774 "wavegen.cpp"
      var $or_cond=($cmp20) | ($cmp21); //@line 1774 "wavegen.cpp"
      if ($or_cond) { __label__ = 12;break $for_cond$8; } //@line 1774 "wavegen.cpp"
  
      var $16=$qix; //@line 1763 "wavegen.cpp"
      var $inc=((($16)+1)&4294967295); //@line 1763 "wavegen.cpp"
      $qix=$inc; //@line 1763 "wavegen.cpp"
      __lastLabel__ = 11; __label__ = 5;continue $for_cond$8; //@line 1763 "wavegen.cpp"
    }
    if (__label__ == 9) {
  
      HEAP8[(__ZL8end_wave_b)]=0;
      ; //@line 1772 "wavegen.cpp"
    }
  
    var $17=$length_addr; //@line 1779 "wavegen.cpp"
    var $add24=((($17)+32)&4294967295); //@line 1779 "wavegen.cpp"
    var $and25=($add24) & -64; //@line 1779 "wavegen.cpp"
    $length2=$and25; //@line 1779 "wavegen.cpp"
    var $cmp26=((($and25))|0)==0; //@line 1780 "wavegen.cpp"
    if ($cmp26) { __label__ = 13;; } else { __label__ = 14;; } //@line 1780 "wavegen.cpp"
    if (__label__ == 13) {
  
      $length2=64; //@line 1781 "wavegen.cpp"
      ; //@line 1781 "wavegen.cpp"
    }
  
    var $18=HEAP32[((__ZL11samplecount)>>2)]; //@line 1784 "wavegen.cpp"
    HEAP32[((__ZL17samplecount_start)>>2)]=$18; //@line 1784 "wavegen.cpp"
    var $19=$length2; //@line 1785 "wavegen.cpp"
    var $20=HEAP32[((__ZL8nsamples)>>2)]; //@line 1785 "wavegen.cpp"
    var $add29=((($20)+($19))&4294967295); //@line 1785 "wavegen.cpp"
    HEAP32[((__ZL8nsamples)>>2)]=$add29; //@line 1785 "wavegen.cpp"
    var $21=$length2; //@line 1787 "wavegen.cpp"
    var $div=((((($21))|0)/4)|0); //@line 1787 "wavegen.cpp"
    $length4=$div; //@line 1787 "wavegen.cpp"
    var $22=$v_addr; //@line 1789 "wavegen.cpp"
    var $freq=(($22+144)&4294967295); //@line 1789 "wavegen.cpp"
    var $arrayidx30=(($freq+14)&4294967295); //@line 1789 "wavegen.cpp"
    var $23=HEAP16[(($arrayidx30)>>1)]; //@line 1789 "wavegen.cpp"
    var $conv=(tempInt=(($23)),(tempInt>=32768?tempInt-65536:tempInt)); //@line 1789 "wavegen.cpp"
    var $mul=((($conv)*7800)&4294967295); //@line 1789 "wavegen.cpp"
    var $24=$v_addr; //@line 1789 "wavegen.cpp"
    var $freqadd=(($24+198)&4294967295); //@line 1789 "wavegen.cpp"
    var $arrayidx31=(($freqadd+14)&4294967295); //@line 1789 "wavegen.cpp"
    var $25=HEAP16[(($arrayidx31)>>1)]; //@line 1789 "wavegen.cpp"
    var $conv32=(tempInt=(($25)),(tempInt>=32768?tempInt-65536:tempInt)); //@line 1789 "wavegen.cpp"
    var $mul33=((($conv32)*256)&4294967295); //@line 1789 "wavegen.cpp"
    var $add34=((($mul33)+($mul))&4294967295); //@line 1789 "wavegen.cpp"
    var $shl=($add34) << 8; //@line 1789 "wavegen.cpp"
    HEAP32[((((__ZL5peaks+560)&4294967295))>>2)]=$shl; //@line 1789 "wavegen.cpp"
    var $26=$v_addr; //@line 1790 "wavegen.cpp"
    var $freq35=(($26+144)&4294967295); //@line 1790 "wavegen.cpp"
    var $arrayidx36=(($freq35+16)&4294967295); //@line 1790 "wavegen.cpp"
    var $27=HEAP16[(($arrayidx36)>>1)]; //@line 1790 "wavegen.cpp"
    var $conv37=(tempInt=(($27)),(tempInt>=32768?tempInt-65536:tempInt)); //@line 1790 "wavegen.cpp"
    var $mul38=((($conv37)*9000)&4294967295); //@line 1790 "wavegen.cpp"
    var $28=$v_addr; //@line 1790 "wavegen.cpp"
    var $freqadd39=(($28+198)&4294967295); //@line 1790 "wavegen.cpp"
    var $arrayidx40=(($freqadd39+16)&4294967295); //@line 1790 "wavegen.cpp"
    var $29=HEAP16[(($arrayidx40)>>1)]; //@line 1790 "wavegen.cpp"
    var $conv41=(tempInt=(($29)),(tempInt>=32768?tempInt-65536:tempInt)); //@line 1790 "wavegen.cpp"
    var $mul42=((($conv41)*256)&4294967295); //@line 1790 "wavegen.cpp"
    var $add43=((($mul42)+($mul38))&4294967295); //@line 1790 "wavegen.cpp"
    var $shl44=($add43) << 8; //@line 1790 "wavegen.cpp"
    HEAP32[((((__ZL5peaks+640)&4294967295))>>2)]=$shl44; //@line 1790 "wavegen.cpp"
    $ix=0; //@line 1792 "wavegen.cpp"
    __lastLabel__ = 14; ; //@line 1792 "wavegen.cpp"
    $for_body$22: while(1) { 
  
      var $30=__lastLabel__ == 14 ? 0 : ($inc187); //@line 1794 "wavegen.cpp"
      var $cmp47=((($30))|0) < 7; //@line 1794 "wavegen.cpp"
      if ($cmp47) { __label__ = 16;; } else { __label__ = 17;; } //@line 1794 "wavegen.cpp"
      if (__label__ == 16) {
  
        var $31=$ix; //@line 1796 "wavegen.cpp"
        var $32=$fr1_addr; //@line 1796 "wavegen.cpp"
        var $ffreq=(($32+2)&4294967295); //@line 1796 "wavegen.cpp"
        var $arrayidx49=(($ffreq+$31*2)&4294967295); //@line 1796 "wavegen.cpp"
        var $33=HEAP16[(($arrayidx49)>>1)]; //@line 1796 "wavegen.cpp"
        var $conv50=(tempInt=(($33)),(tempInt>=32768?tempInt-65536:tempInt)); //@line 1796 "wavegen.cpp"
        var $34=$ix; //@line 1796 "wavegen.cpp"
        var $35=$v_addr; //@line 1796 "wavegen.cpp"
        var $freq51=(($35+144)&4294967295); //@line 1796 "wavegen.cpp"
        var $arrayidx52=(($freq51+$34*2)&4294967295); //@line 1796 "wavegen.cpp"
        var $36=HEAP16[(($arrayidx52)>>1)]; //@line 1796 "wavegen.cpp"
        var $conv53=(tempInt=(($36)),(tempInt>=32768?tempInt-65536:tempInt)); //@line 1796 "wavegen.cpp"
        var $mul54=((($conv53)*($conv50))&4294967295); //@line 1796 "wavegen.cpp"
        var $37=$ix; //@line 1796 "wavegen.cpp"
        var $38=$v_addr; //@line 1796 "wavegen.cpp"
        var $freqadd55=(($38+198)&4294967295); //@line 1796 "wavegen.cpp"
        var $arrayidx56=(($freqadd55+$37*2)&4294967295); //@line 1796 "wavegen.cpp"
        var $39=HEAP16[(($arrayidx56)>>1)]; //@line 1796 "wavegen.cpp"
        var $conv57=(tempInt=(($39)),(tempInt>=32768?tempInt-65536:tempInt)); //@line 1796 "wavegen.cpp"
        var $mul58=((($conv57)*256)&4294967295); //@line 1796 "wavegen.cpp"
        var $add59=((($mul58)+($mul54))&4294967295); //@line 1796 "wavegen.cpp"
        var $shl60=($add59) << 8; //@line 1796 "wavegen.cpp"
        var $conv61=((($shl60))|0); //@line 1796 "wavegen.cpp"
        var $40=$ix; //@line 1796 "wavegen.cpp"
        var $arrayidx62=((__ZL5peaks+$40*80)&4294967295); //@line 1796 "wavegen.cpp"
        var $freq1=(($arrayidx62+16)&4294967295); //@line 1796 "wavegen.cpp"
        HEAPF32[(($freq1)>>2)]=$conv61; //@line 1796 "wavegen.cpp"
        var $41=$ix; //@line 1797 "wavegen.cpp"
        var $arrayidx63=((__ZL5peaks+$41*80)&4294967295); //@line 1797 "wavegen.cpp"
        var $freq164=(($arrayidx63+16)&4294967295); //@line 1797 "wavegen.cpp"
        var $42=HEAPF32[(($freq164)>>2)]; //@line 1797 "wavegen.cpp"
        var $conv65=((($42))|0); //@line 1797 "wavegen.cpp"
        var $43=$ix; //@line 1797 "wavegen.cpp"
        var $arrayidx66=((__ZL5peaks+$43*80)&4294967295); //@line 1797 "wavegen.cpp"
        var $freq67=(($arrayidx66)&4294967295); //@line 1797 "wavegen.cpp"
        HEAP32[(($freq67)>>2)]=$conv65; //@line 1797 "wavegen.cpp"
        var $44=$ix; //@line 1798 "wavegen.cpp"
        var $45=$fr2_addr; //@line 1798 "wavegen.cpp"
        var $ffreq68=(($45+2)&4294967295); //@line 1798 "wavegen.cpp"
        var $arrayidx69=(($ffreq68+$44*2)&4294967295); //@line 1798 "wavegen.cpp"
        var $46=HEAP16[(($arrayidx69)>>1)]; //@line 1798 "wavegen.cpp"
        var $conv70=(tempInt=(($46)),(tempInt>=32768?tempInt-65536:tempInt)); //@line 1798 "wavegen.cpp"
        var $47=$ix; //@line 1798 "wavegen.cpp"
        var $48=$v_addr; //@line 1798 "wavegen.cpp"
        var $freq71=(($48+144)&4294967295); //@line 1798 "wavegen.cpp"
        var $arrayidx72=(($freq71+$47*2)&4294967295); //@line 1798 "wavegen.cpp"
        var $49=HEAP16[(($arrayidx72)>>1)]; //@line 1798 "wavegen.cpp"
        var $conv73=(tempInt=(($49)),(tempInt>=32768?tempInt-65536:tempInt)); //@line 1798 "wavegen.cpp"
        var $mul74=((($conv73)*($conv70))&4294967295); //@line 1798 "wavegen.cpp"
        var $50=$ix; //@line 1798 "wavegen.cpp"
        var $51=$v_addr; //@line 1798 "wavegen.cpp"
        var $freqadd75=(($51+198)&4294967295); //@line 1798 "wavegen.cpp"
        var $arrayidx76=(($freqadd75+$50*2)&4294967295); //@line 1798 "wavegen.cpp"
        var $52=HEAP16[(($arrayidx76)>>1)]; //@line 1798 "wavegen.cpp"
        var $conv77=(tempInt=(($52)),(tempInt>=32768?tempInt-65536:tempInt)); //@line 1798 "wavegen.cpp"
        var $mul78=((($conv77)*256)&4294967295); //@line 1798 "wavegen.cpp"
        var $add79=((($mul78)+($mul74))&4294967295); //@line 1798 "wavegen.cpp"
        var $shl80=($add79) << 8; //@line 1798 "wavegen.cpp"
        var $conv81=((($shl80))|0); //@line 1798 "wavegen.cpp"
        $next=$conv81; //@line 1798 "wavegen.cpp"
        var $53=$next; //@line 1799 "wavegen.cpp"
        var $54=$ix; //@line 1799 "wavegen.cpp"
        var $arrayidx82=((__ZL5peaks+$54*80)&4294967295); //@line 1799 "wavegen.cpp"
        var $freq183=(($arrayidx82+16)&4294967295); //@line 1799 "wavegen.cpp"
        var $55=HEAPF32[(($freq183)>>2)]; //@line 1799 "wavegen.cpp"
        var $sub=($53)-($55); //@line 1799 "wavegen.cpp"
        var $mul84=($sub)*16; //@line 1799 "wavegen.cpp"
        var $56=$length4; //@line 1799 "wavegen.cpp"
        var $conv85=((($56))|0); //@line 1799 "wavegen.cpp"
        var $div86=($mul84)/($conv85); //@line 1799 "wavegen.cpp"
        var $57=$ix; //@line 1799 "wavegen.cpp"
        var $arrayidx87=((__ZL5peaks+$57*80)&4294967295); //@line 1799 "wavegen.cpp"
        var $freq_inc=(($arrayidx87+48)&4294967295); //@line 1799 "wavegen.cpp"
        HEAPF32[(($freq_inc)>>2)]=$div86; //@line 1799 "wavegen.cpp"
        ; //@line 1800 "wavegen.cpp"
      }
  
      var $58=$ix; //@line 1802 "wavegen.cpp"
      var $59=$fr1_addr; //@line 1802 "wavegen.cpp"
      var $fheight=(($59+18)&4294967295); //@line 1802 "wavegen.cpp"
      var $arrayidx89=(($fheight+$58)&4294967295); //@line 1802 "wavegen.cpp"
      var $60=HEAPU8[($arrayidx89)]; //@line 1802 "wavegen.cpp"
      var $conv90=((($60))&255); //@line 1802 "wavegen.cpp"
      var $61=$ix; //@line 1802 "wavegen.cpp"
      var $62=$v_addr; //@line 1802 "wavegen.cpp"
      var $height=(($62+162)&4294967295); //@line 1802 "wavegen.cpp"
      var $arrayidx91=(($height+$61*2)&4294967295); //@line 1802 "wavegen.cpp"
      var $63=HEAP16[(($arrayidx91)>>1)]; //@line 1802 "wavegen.cpp"
      var $conv92=(tempInt=(($63)),(tempInt>=32768?tempInt-65536:tempInt)); //@line 1802 "wavegen.cpp"
      var $mul93=((($conv90)*64)&4294967295); //@line 1802 "wavegen.cpp"
      var $shl94=((($mul93)*($conv92))&4294967295); //@line 1802 "wavegen.cpp"
      var $conv95=((($shl94))|0); //@line 1802 "wavegen.cpp"
      var $64=$ix; //@line 1802 "wavegen.cpp"
      var $arrayidx96=((__ZL5peaks+$64*80)&4294967295); //@line 1802 "wavegen.cpp"
      var $height1=(($arrayidx96+24)&4294967295); //@line 1802 "wavegen.cpp"
      HEAPF32[(($height1)>>2)]=$conv95; //@line 1802 "wavegen.cpp"
      var $65=$ix; //@line 1803 "wavegen.cpp"
      var $arrayidx97=((__ZL5peaks+$65*80)&4294967295); //@line 1803 "wavegen.cpp"
      var $height198=(($arrayidx97+24)&4294967295); //@line 1803 "wavegen.cpp"
      var $66=HEAPF32[(($height198)>>2)]; //@line 1803 "wavegen.cpp"
      var $conv99=((($66))|0); //@line 1803 "wavegen.cpp"
      var $67=$ix; //@line 1803 "wavegen.cpp"
      var $arrayidx100=((__ZL5peaks+$67*80)&4294967295); //@line 1803 "wavegen.cpp"
      var $height101=(($arrayidx100+4)&4294967295); //@line 1803 "wavegen.cpp"
      HEAP32[(($height101)>>2)]=$conv99; //@line 1803 "wavegen.cpp"
      var $68=$ix; //@line 1804 "wavegen.cpp"
      var $69=$fr2_addr; //@line 1804 "wavegen.cpp"
      var $fheight102=(($69+18)&4294967295); //@line 1804 "wavegen.cpp"
      var $arrayidx103=(($fheight102+$68)&4294967295); //@line 1804 "wavegen.cpp"
      var $70=HEAPU8[($arrayidx103)]; //@line 1804 "wavegen.cpp"
      var $conv104=((($70))&255); //@line 1804 "wavegen.cpp"
      var $71=$ix; //@line 1804 "wavegen.cpp"
      var $72=$v_addr; //@line 1804 "wavegen.cpp"
      var $height105=(($72+162)&4294967295); //@line 1804 "wavegen.cpp"
      var $arrayidx106=(($height105+$71*2)&4294967295); //@line 1804 "wavegen.cpp"
      var $73=HEAP16[(($arrayidx106)>>1)]; //@line 1804 "wavegen.cpp"
      var $conv107=(tempInt=(($73)),(tempInt>=32768?tempInt-65536:tempInt)); //@line 1804 "wavegen.cpp"
      var $mul108=((($conv104)*64)&4294967295); //@line 1804 "wavegen.cpp"
      var $shl109=((($mul108)*($conv107))&4294967295); //@line 1804 "wavegen.cpp"
      var $conv110=((($shl109))|0); //@line 1804 "wavegen.cpp"
      $next=$conv110; //@line 1804 "wavegen.cpp"
      var $74=$next; //@line 1805 "wavegen.cpp"
      var $75=$ix; //@line 1805 "wavegen.cpp"
      var $arrayidx111=((__ZL5peaks+$75*80)&4294967295); //@line 1805 "wavegen.cpp"
      var $height1112=(($arrayidx111+24)&4294967295); //@line 1805 "wavegen.cpp"
      var $76=HEAPF32[(($height1112)>>2)]; //@line 1805 "wavegen.cpp"
      var $sub113=($74)-($76); //@line 1805 "wavegen.cpp"
      var $mul114=($sub113)*64; //@line 1805 "wavegen.cpp"
      var $77=$length2; //@line 1805 "wavegen.cpp"
      var $conv115=((($77))|0); //@line 1805 "wavegen.cpp"
      var $div116=($mul114)/($conv115); //@line 1805 "wavegen.cpp"
      var $78=$ix; //@line 1805 "wavegen.cpp"
      var $arrayidx117=((__ZL5peaks+$78*80)&4294967295); //@line 1805 "wavegen.cpp"
      var $height_inc=(($arrayidx117+56)&4294967295); //@line 1805 "wavegen.cpp"
      HEAPF32[(($height_inc)>>2)]=$div116; //@line 1805 "wavegen.cpp"
      var $79=$ix; //@line 1807 "wavegen.cpp"
      var $cmp118=((($79))|0) <= 5; //@line 1807 "wavegen.cpp"
      if ($cmp118) { __label__ = 18;; } else { __label__ = 22;; } //@line 1807 "wavegen.cpp"
      $land_lhs_true$$for_inc186$27: do { 
        if (__label__ == 18) {
  
          var $80=$ix; //@line 1807 "wavegen.cpp"
          var $81=HEAP32[((_wvoice)>>2)]; //@line 1807 "wavegen.cpp"
          var $n_harmonic_peaks=(($81+84)&4294967295); //@line 1807 "wavegen.cpp"
          var $82=HEAP32[(($n_harmonic_peaks)>>2)]; //@line 1807 "wavegen.cpp"
          var $cmp119=((($80))|0) <= ((($82))|0); //@line 1807 "wavegen.cpp"
          if (!($cmp119)) { __label__ = 22;break $land_lhs_true$$for_inc186$27; } //@line 1807 "wavegen.cpp"
  
          var $83=$ix; //@line 1809 "wavegen.cpp"
          var $84=$fr1_addr; //@line 1809 "wavegen.cpp"
          var $fwidth=(($84+26)&4294967295); //@line 1809 "wavegen.cpp"
          var $arrayidx121=(($fwidth+$83)&4294967295); //@line 1809 "wavegen.cpp"
          var $85=HEAPU8[($arrayidx121)]; //@line 1809 "wavegen.cpp"
          var $conv122=((($85))&255); //@line 1809 "wavegen.cpp"
          var $86=$ix; //@line 1809 "wavegen.cpp"
          var $87=$v_addr; //@line 1809 "wavegen.cpp"
          var $width=(($87+180)&4294967295); //@line 1809 "wavegen.cpp"
          var $arrayidx123=(($width+$86*2)&4294967295); //@line 1809 "wavegen.cpp"
          var $88=HEAP16[(($arrayidx123)>>1)]; //@line 1809 "wavegen.cpp"
          var $conv124=(tempInt=(($88)),(tempInt>=32768?tempInt-65536:tempInt)); //@line 1809 "wavegen.cpp"
          var $mul125=((($conv122)*1024)&4294967295); //@line 1809 "wavegen.cpp"
          var $shl126=((($mul125)*($conv124))&4294967295); //@line 1809 "wavegen.cpp"
          var $conv127=((($shl126))|0); //@line 1809 "wavegen.cpp"
          var $89=$ix; //@line 1809 "wavegen.cpp"
          var $arrayidx128=((__ZL5peaks+$89*80)&4294967295); //@line 1809 "wavegen.cpp"
          var $left1=(($arrayidx128+32)&4294967295); //@line 1809 "wavegen.cpp"
          HEAPF32[(($left1)>>2)]=$conv127; //@line 1809 "wavegen.cpp"
          var $90=$ix; //@line 1810 "wavegen.cpp"
          var $arrayidx129=((__ZL5peaks+$90*80)&4294967295); //@line 1810 "wavegen.cpp"
          var $left1130=(($arrayidx129+32)&4294967295); //@line 1810 "wavegen.cpp"
          var $91=HEAPF32[(($left1130)>>2)]; //@line 1810 "wavegen.cpp"
          var $conv131=((($91))|0); //@line 1810 "wavegen.cpp"
          var $92=$ix; //@line 1810 "wavegen.cpp"
          var $arrayidx132=((__ZL5peaks+$92*80)&4294967295); //@line 1810 "wavegen.cpp"
          var $left=(($arrayidx132+8)&4294967295); //@line 1810 "wavegen.cpp"
          HEAP32[(($left)>>2)]=$conv131; //@line 1810 "wavegen.cpp"
          var $93=$ix; //@line 1811 "wavegen.cpp"
          var $94=$fr2_addr; //@line 1811 "wavegen.cpp"
          var $fwidth133=(($94+26)&4294967295); //@line 1811 "wavegen.cpp"
          var $arrayidx134=(($fwidth133+$93)&4294967295); //@line 1811 "wavegen.cpp"
          var $95=HEAPU8[($arrayidx134)]; //@line 1811 "wavegen.cpp"
          var $conv135=((($95))&255); //@line 1811 "wavegen.cpp"
          var $96=$ix; //@line 1811 "wavegen.cpp"
          var $97=$v_addr; //@line 1811 "wavegen.cpp"
          var $width136=(($97+180)&4294967295); //@line 1811 "wavegen.cpp"
          var $arrayidx137=(($width136+$96*2)&4294967295); //@line 1811 "wavegen.cpp"
          var $98=HEAP16[(($arrayidx137)>>1)]; //@line 1811 "wavegen.cpp"
          var $conv138=(tempInt=(($98)),(tempInt>=32768?tempInt-65536:tempInt)); //@line 1811 "wavegen.cpp"
          var $mul139=((($conv135)*1024)&4294967295); //@line 1811 "wavegen.cpp"
          var $shl140=((($mul139)*($conv138))&4294967295); //@line 1811 "wavegen.cpp"
          var $conv141=((($shl140))|0); //@line 1811 "wavegen.cpp"
          $next=$conv141; //@line 1811 "wavegen.cpp"
          var $99=$next; //@line 1812 "wavegen.cpp"
          var $100=$ix; //@line 1812 "wavegen.cpp"
          var $arrayidx142=((__ZL5peaks+$100*80)&4294967295); //@line 1812 "wavegen.cpp"
          var $left1143=(($arrayidx142+32)&4294967295); //@line 1812 "wavegen.cpp"
          var $101=HEAPF32[(($left1143)>>2)]; //@line 1812 "wavegen.cpp"
          var $sub144=($99)-($101); //@line 1812 "wavegen.cpp"
          var $mul145=($sub144)*64; //@line 1812 "wavegen.cpp"
          var $102=$length2; //@line 1812 "wavegen.cpp"
          var $conv146=((($102))|0); //@line 1812 "wavegen.cpp"
          var $div147=($mul145)/($conv146); //@line 1812 "wavegen.cpp"
          var $103=$ix; //@line 1812 "wavegen.cpp"
          var $arrayidx148=((__ZL5peaks+$103*80)&4294967295); //@line 1812 "wavegen.cpp"
          var $left_inc=(($arrayidx148+64)&4294967295); //@line 1812 "wavegen.cpp"
          HEAPF32[(($left_inc)>>2)]=$div147; //@line 1812 "wavegen.cpp"
          var $104=$ix; //@line 1814 "wavegen.cpp"
          var $cmp149=((($104))|0) < 3; //@line 1814 "wavegen.cpp"
          var $105=$ix; //@line 1816 "wavegen.cpp"
          if ($cmp149) { __label__ = 20;; } else { __label__ = 21;; } //@line 1814 "wavegen.cpp"
          if (__label__ == 20) {
  
            var $106=$fr1_addr; //@line 1816 "wavegen.cpp"
            var $fright=(($106+32)&4294967295); //@line 1816 "wavegen.cpp"
            var $arrayidx151=(($fright+$105)&4294967295); //@line 1816 "wavegen.cpp"
            var $107=HEAPU8[($arrayidx151)]; //@line 1816 "wavegen.cpp"
            var $conv152=((($107))&255); //@line 1816 "wavegen.cpp"
            var $108=$ix; //@line 1816 "wavegen.cpp"
            var $109=$v_addr; //@line 1816 "wavegen.cpp"
            var $width153=(($109+180)&4294967295); //@line 1816 "wavegen.cpp"
            var $arrayidx154=(($width153+$108*2)&4294967295); //@line 1816 "wavegen.cpp"
            var $110=HEAP16[(($arrayidx154)>>1)]; //@line 1816 "wavegen.cpp"
            var $conv155=(tempInt=(($110)),(tempInt>=32768?tempInt-65536:tempInt)); //@line 1816 "wavegen.cpp"
            var $mul156=((($conv152)*1024)&4294967295); //@line 1816 "wavegen.cpp"
            var $shl157=((($mul156)*($conv155))&4294967295); //@line 1816 "wavegen.cpp"
            var $conv158=((($shl157))|0); //@line 1816 "wavegen.cpp"
            var $111=$ix; //@line 1816 "wavegen.cpp"
            var $arrayidx159=((__ZL5peaks+$111*80)&4294967295); //@line 1816 "wavegen.cpp"
            var $right1=(($arrayidx159+40)&4294967295); //@line 1816 "wavegen.cpp"
            HEAPF32[(($right1)>>2)]=$conv158; //@line 1816 "wavegen.cpp"
            var $112=$ix; //@line 1817 "wavegen.cpp"
            var $arrayidx160=((__ZL5peaks+$112*80)&4294967295); //@line 1817 "wavegen.cpp"
            var $right1161=(($arrayidx160+40)&4294967295); //@line 1817 "wavegen.cpp"
            var $113=HEAPF32[(($right1161)>>2)]; //@line 1817 "wavegen.cpp"
            var $conv162=((($113))|0); //@line 1817 "wavegen.cpp"
            var $114=$ix; //@line 1817 "wavegen.cpp"
            var $arrayidx163=((__ZL5peaks+$114*80)&4294967295); //@line 1817 "wavegen.cpp"
            var $right=(($arrayidx163+12)&4294967295); //@line 1817 "wavegen.cpp"
            HEAP32[(($right)>>2)]=$conv162; //@line 1817 "wavegen.cpp"
            var $115=$ix; //@line 1818 "wavegen.cpp"
            var $116=$fr2_addr; //@line 1818 "wavegen.cpp"
            var $fright164=(($116+32)&4294967295); //@line 1818 "wavegen.cpp"
            var $arrayidx165=(($fright164+$115)&4294967295); //@line 1818 "wavegen.cpp"
            var $117=HEAPU8[($arrayidx165)]; //@line 1818 "wavegen.cpp"
            var $conv166=((($117))&255); //@line 1818 "wavegen.cpp"
            var $118=$ix; //@line 1818 "wavegen.cpp"
            var $119=$v_addr; //@line 1818 "wavegen.cpp"
            var $width167=(($119+180)&4294967295); //@line 1818 "wavegen.cpp"
            var $arrayidx168=(($width167+$118*2)&4294967295); //@line 1818 "wavegen.cpp"
            var $120=HEAP16[(($arrayidx168)>>1)]; //@line 1818 "wavegen.cpp"
            var $conv169=(tempInt=(($120)),(tempInt>=32768?tempInt-65536:tempInt)); //@line 1818 "wavegen.cpp"
            var $mul170=((($conv166)*1024)&4294967295); //@line 1818 "wavegen.cpp"
            var $shl171=((($mul170)*($conv169))&4294967295); //@line 1818 "wavegen.cpp"
            var $conv172=((($shl171))|0); //@line 1818 "wavegen.cpp"
            $next=$conv172; //@line 1818 "wavegen.cpp"
            var $121=$next; //@line 1819 "wavegen.cpp"
            var $122=$ix; //@line 1819 "wavegen.cpp"
            var $arrayidx173=((__ZL5peaks+$122*80)&4294967295); //@line 1819 "wavegen.cpp"
            var $right1174=(($arrayidx173+40)&4294967295); //@line 1819 "wavegen.cpp"
            var $123=HEAPF32[(($right1174)>>2)]; //@line 1819 "wavegen.cpp"
            var $sub175=($121)-($123); //@line 1819 "wavegen.cpp"
            var $mul176=($sub175)*64; //@line 1819 "wavegen.cpp"
            var $124=$length2; //@line 1819 "wavegen.cpp"
            var $conv177=((($124))|0); //@line 1819 "wavegen.cpp"
            var $div178=($mul176)/($conv177); //@line 1819 "wavegen.cpp"
            var $125=$ix; //@line 1819 "wavegen.cpp"
            var $arrayidx179=((__ZL5peaks+$125*80)&4294967295); //@line 1819 "wavegen.cpp"
            var $right_inc=(($arrayidx179+72)&4294967295); //@line 1819 "wavegen.cpp"
            HEAPF32[(($right_inc)>>2)]=$div178; //@line 1819 "wavegen.cpp"
            ; //@line 1820 "wavegen.cpp"
          }
          else if (__label__ == 21) {
  
            var $arrayidx180=((__ZL5peaks+$105*80)&4294967295); //@line 1823 "wavegen.cpp"
            var $left181=(($arrayidx180+8)&4294967295); //@line 1823 "wavegen.cpp"
            var $126=HEAP32[(($left181)>>2)]; //@line 1823 "wavegen.cpp"
            var $127=$ix; //@line 1823 "wavegen.cpp"
            var $arrayidx182=((__ZL5peaks+$127*80)&4294967295); //@line 1823 "wavegen.cpp"
            var $right183=(($arrayidx182+12)&4294967295); //@line 1823 "wavegen.cpp"
            HEAP32[(($right183)>>2)]=$126; //@line 1823 "wavegen.cpp"
            ;
          }
        }
      } while(0);
  
      var $128=$ix; //@line 1792 "wavegen.cpp"
      var $inc187=((($128)+1)&4294967295); //@line 1792 "wavegen.cpp"
      $ix=$inc187; //@line 1792 "wavegen.cpp"
      var $cmp46=((($inc187))|0) < 8; //@line 1792 "wavegen.cpp"
      if ($cmp46) { __lastLabel__ = 22; __label__ = 15;continue $for_body$22; } else { __lastLabel__ = 22; __label__ = 23;break $for_body$22; } //@line 1792 "wavegen.cpp"
    }
  
    ;
    return; //@line 1827 "wavegen.cpp"
    return;
  }
  

  function __Z11Write4BytesP7__sFILEi($f, $value) {
    ;
    var __label__;
  
    var $f_addr;
    var $value_addr;
    var $ix;
    $f_addr=$f;
    $value_addr=$value;
    $ix=0; //@line 1843 "wavegen.cpp"
    ; //@line 1843 "wavegen.cpp"
    while(1) { 
  
      var $0=$value_addr; //@line 1845 "wavegen.cpp"
      var $and=($0) & 255; //@line 1845 "wavegen.cpp"
      var $1=$f_addr; //@line 1845 "wavegen.cpp"
      var $call=_fputc($and, $1); //@line 1845 "wavegen.cpp"
      var $2=$value_addr; //@line 1846 "wavegen.cpp"
      var $shr=($2) >> 8; //@line 1846 "wavegen.cpp"
      $value_addr=$shr; //@line 1846 "wavegen.cpp"
      var $3=$ix; //@line 1843 "wavegen.cpp"
      var $inc=((($3)+1)&4294967295); //@line 1843 "wavegen.cpp"
      $ix=$inc; //@line 1843 "wavegen.cpp"
      var $cmp=((($inc))|0) < 4; //@line 1843 "wavegen.cpp"
      if ($cmp) { __label__ = 1;continue ; } else { __label__ = 2;break ; } //@line 1843 "wavegen.cpp"
    }
  
    ;
    return; //@line 1848 "wavegen.cpp"
    return;
  }
  

  function __Z12WavegenFill2i($fill_zeros) {
    ;
    var __label__;
    var __lastLabel__ = null;
  
    var $retval;
    var $fill_zeros_addr;
    var $q;
    var $length;
    var $result;
    $fill_zeros_addr=$fill_zeros;
    ; //@line 1864 "wavegen.cpp"
    $while_cond$2: while(1) { 
  
      var $0=HEAPU32[((_out_ptr)>>2)]; //@line 1864 "wavegen.cpp"
      var $1=HEAPU32[((_out_end)>>2)]; //@line 1864 "wavegen.cpp"
      var $cmp=((($0))>>>0) < ((($1))>>>0); //@line 1864 "wavegen.cpp"
      if (!($cmp)) { __label__ = 36;break $while_cond$2; } //@line 1864 "wavegen.cpp"
  
      var $call=__Z9WcmdqUsedv(); //@line 1866 "wavegen.cpp"
      var $cmp1=((($call))|0) <= 0; //@line 1866 "wavegen.cpp"
      if ($cmp1) { __label__ = 3;break $while_cond$2; } //@line 1866 "wavegen.cpp"
  
      $result=0; //@line 1884 "wavegen.cpp"
      var $12=HEAP32[((_wcmdq_head)>>2)]; //@line 1885 "wavegen.cpp"
      var $arrayidx=((_wcmdq+$12*16)&4294967295); //@line 1885 "wavegen.cpp"
      var $arraydecay=(($arrayidx)&4294967295); //@line 1885 "wavegen.cpp"
      $q=$arraydecay; //@line 1885 "wavegen.cpp"
      var $13=$q; //@line 1886 "wavegen.cpp"
      var $arrayidx14=(($13+4)&4294967295); //@line 1886 "wavegen.cpp"
      var $14=HEAP32[(($arrayidx14)>>2)]; //@line 1886 "wavegen.cpp"
      $length=$14; //@line 1886 "wavegen.cpp"
      var $15=$q; //@line 1888 "wavegen.cpp"
      var $arrayidx15=(($15)&4294967295); //@line 1888 "wavegen.cpp"
      var $16=HEAP32[(($arrayidx15)>>2)]; //@line 1888 "wavegen.cpp"
      if ($16 == 9) {
        __label__ = 11;;
      }
      else if ($16 == 5) {
        __label__ = 12;;
      }
      else if ($16 == 6) {
        __label__ = 15;;
      }
      else if ($16 == 7) {
        __label__ = 16;;
      }
      else if ($16 == 4) {
        __label__ = 19;;
      }
      else if ($16 == 3) {
        __label__ = 20;;
      }
      else if ($16 == 2) {
        __label__ = 21;;
      }
      else if ($16 == 1) {
        __label__ = 22;;
      }
      else if ($16 == 10) {
        __label__ = 23;;
      }
      else if ($16 == 8) {
        __label__ = 25;;
      }
      else if ($16 == 11) {
        __label__ = 26;;
      }
      else if ($16 == 12) {
        __label__ = 27;;
      }
      else if ($16 == 13) {
        __label__ = 28;;
      }
      else if ($16 == 14) {
        __label__ = 29;;
      }
      else if ($16 == 15) {
        __label__ = 31;;
      }
      else {
      __label__ = 32;;
      }
      
      $sw_epilogthread_pre_split$$sw_bb$$sw_bb19$$sw_bb24$$sw_bb31$$sw_bb44$$sw_bb45$$sw_bb52$$sw_bb53$$sw_bb60$$sw_bb70$$sw_bb73$$sw_bb76$$sw_bb79$$sw_bb81$$sw_bb86$6: do { 
        if (__label__ == 11) {
  
          var $17=$length; //@line 1891 "wavegen.cpp"
          var $18=$q; //@line 1891 "wavegen.cpp"
          var $arrayidx16=(($18+8)&4294967295); //@line 1891 "wavegen.cpp"
          var $19=HEAP32[(($arrayidx16)>>2)]; //@line 1891 "wavegen.cpp"
          var $20=($19); //@line 1891 "wavegen.cpp"
          var $21=$q; //@line 1891 "wavegen.cpp"
          var $arrayidx17=(($21+12)&4294967295); //@line 1891 "wavegen.cpp"
          var $22=HEAP32[(($arrayidx17)>>2)]; //@line 1891 "wavegen.cpp"
          var $shr=($22) >> 16; //@line 1891 "wavegen.cpp"
          var $23=$q; //@line 1891 "wavegen.cpp"
          var $arrayidx18=(($23+12)&4294967295); //@line 1891 "wavegen.cpp"
          var $24=HEAP32[(($arrayidx18)>>2)]; //@line 1891 "wavegen.cpp"
          var $and=($24) & 65535; //@line 1891 "wavegen.cpp"
          __Z8SetPitchiPhii($17, $20, $shr, $and); //@line 1891 "wavegen.cpp"
          __label__ = 32;break $sw_epilogthread_pre_split$$sw_bb$$sw_bb19$$sw_bb24$$sw_bb31$$sw_bb44$$sw_bb45$$sw_bb52$$sw_bb53$$sw_bb60$$sw_bb70$$sw_bb73$$sw_bb76$$sw_bb79$$sw_bb81$$sw_bb86$6; //@line 1892 "wavegen.cpp"
        }
        else if (__label__ == 12) {
  
          var $25=HEAP32[((__ZZ12WavegenFill2iE6resume)>>2)]; //@line 1895 "wavegen.cpp"
          var $cmp20=((($25))|0)==0; //@line 1895 "wavegen.cpp"
          if ($cmp20) { __label__ = 13;; } else { __label__ = 14;; } //@line 1895 "wavegen.cpp"
          if (__label__ == 13) {
  
            var $26=$length; //@line 1897 "wavegen.cpp"
            var $27=HEAP32[((__ZZ12WavegenFill2iE13echo_complete)>>2)]; //@line 1897 "wavegen.cpp"
            var $sub=((($27)-($26))&4294967295); //@line 1897 "wavegen.cpp"
            HEAP32[((__ZZ12WavegenFill2iE13echo_complete)>>2)]=$sub; //@line 1897 "wavegen.cpp"
            ; //@line 1898 "wavegen.cpp"
          }
  
          HEAP32[((((_wdata+28)&4294967295))>>2)]=0; //@line 1899 "wavegen.cpp"
          HEAP32[((((_wdata+60)&4294967295))>>2)]=100; //@line 1900 "wavegen.cpp"
          __Z10KlattReseti(1); //@line 1901 "wavegen.cpp"
          var $28=$length; //@line 1902 "wavegen.cpp"
          var $29=HEAP32[((__ZZ12WavegenFill2iE6resume)>>2)]; //@line 1902 "wavegen.cpp"
          var $call23=__ZL11PlaySilenceii($28, $29); //@line 1902 "wavegen.cpp"
          $result=$call23; //@line 1902 "wavegen.cpp"
          __lastLabel__ = 14; __label__ = 33;break $sw_epilogthread_pre_split$$sw_bb$$sw_bb19$$sw_bb24$$sw_bb31$$sw_bb44$$sw_bb45$$sw_bb52$$sw_bb53$$sw_bb60$$sw_bb70$$sw_bb73$$sw_bb76$$sw_bb79$$sw_bb81$$sw_bb86$6; //@line 1903 "wavegen.cpp"
        }
        else if (__label__ == 15) {
  
          var $30=HEAP32[((__ZL11echo_length)>>2)]; //@line 1906 "wavegen.cpp"
          HEAP32[((__ZZ12WavegenFill2iE13echo_complete)>>2)]=$30; //@line 1906 "wavegen.cpp"
          HEAP32[((((_wdata+28)&4294967295))>>2)]=0; //@line 1907 "wavegen.cpp"
          __Z10KlattReseti(1); //@line 1908 "wavegen.cpp"
          var $31=$length; //@line 1909 "wavegen.cpp"
          var $32=HEAP32[((__ZZ12WavegenFill2iE6resume)>>2)]; //@line 1909 "wavegen.cpp"
          var $33=$q; //@line 1909 "wavegen.cpp"
          var $arrayidx25=(($33+8)&4294967295); //@line 1909 "wavegen.cpp"
          var $34=HEAP32[(($arrayidx25)>>2)]; //@line 1909 "wavegen.cpp"
          var $35=($34); //@line 1909 "wavegen.cpp"
          var $36=$q; //@line 1909 "wavegen.cpp"
          var $arrayidx26=(($36+12)&4294967295); //@line 1909 "wavegen.cpp"
          var $37=HEAP32[(($arrayidx26)>>2)]; //@line 1909 "wavegen.cpp"
          var $and27=($37) & 255; //@line 1909 "wavegen.cpp"
          var $38=$q; //@line 1909 "wavegen.cpp"
          var $arrayidx28=(($38+12)&4294967295); //@line 1909 "wavegen.cpp"
          var $39=HEAP32[(($arrayidx28)>>2)]; //@line 1909 "wavegen.cpp"
          var $shr29=($39) >> 8; //@line 1909 "wavegen.cpp"
          var $call30=__ZL8PlayWaveiiPhii($31, $32, $35, $and27, $shr29); //@line 1909 "wavegen.cpp"
          $result=$call30; //@line 1909 "wavegen.cpp"
          __lastLabel__ = 15; __label__ = 33;break $sw_epilogthread_pre_split$$sw_bb$$sw_bb19$$sw_bb24$$sw_bb31$$sw_bb44$$sw_bb45$$sw_bb52$$sw_bb53$$sw_bb60$$sw_bb70$$sw_bb73$$sw_bb76$$sw_bb79$$sw_bb81$$sw_bb86$6; //@line 1910 "wavegen.cpp"
        }
        else if (__label__ == 16) {
  
          var $40=$q; //@line 1914 "wavegen.cpp"
          var $arrayidx32=(($40+12)&4294967295); //@line 1914 "wavegen.cpp"
          var $41=HEAP32[(($arrayidx32)>>2)]; //@line 1914 "wavegen.cpp"
          var $shr33=($41) >> 8; //@line 1914 "wavegen.cpp"
          HEAP32[((((_wdata+36)&4294967295))>>2)]=$shr33; //@line 1914 "wavegen.cpp"
          var $42=$q; //@line 1915 "wavegen.cpp"
          var $arrayidx34=(($42+12)&4294967295); //@line 1915 "wavegen.cpp"
          var $43=HEAP32[(($arrayidx34)>>2)]; //@line 1915 "wavegen.cpp"
          var $and35=($43) & 255; //@line 1915 "wavegen.cpp"
          HEAP32[((((_wdata+32)&4294967295))>>2)]=$and35; //@line 1915 "wavegen.cpp"
          var $44=$length; //@line 1916 "wavegen.cpp"
          var $and36=($44) & 65535; //@line 1916 "wavegen.cpp"
          HEAP32[((((_wdata+28)&4294967295))>>2)]=$and36; //@line 1916 "wavegen.cpp"
          var $45=$length; //@line 1917 "wavegen.cpp"
          var $shr37=($45) >> 16; //@line 1917 "wavegen.cpp"
          var $and38=($shr37) & 65535; //@line 1917 "wavegen.cpp"
          HEAP32[((((_wdata+44)&4294967295))>>2)]=$and38; //@line 1917 "wavegen.cpp"
          var $46=HEAP32[((((_wdata+32)&4294967295))>>2)]; //@line 1918 "wavegen.cpp"
          var $cmp39=((($46))|0)==0; //@line 1918 "wavegen.cpp"
          if ($cmp39) { __label__ = 17;; } else { __label__ = 18;; } //@line 1918 "wavegen.cpp"
          if (__label__ == 17) {
  
            var $47=HEAP32[((((_wdata+28)&4294967295))>>2)]; //@line 1920 "wavegen.cpp"
            var $mul=((($47)*2)&4294967295); //@line 1920 "wavegen.cpp"
            HEAP32[((((_wdata+28)&4294967295))>>2)]=$mul; //@line 1920 "wavegen.cpp"
            var $48=HEAP32[((((_wdata+44)&4294967295))>>2)]; //@line 1921 "wavegen.cpp"
            var $mul41=((($48)*2)&4294967295); //@line 1921 "wavegen.cpp"
            HEAP32[((((_wdata+44)&4294967295))>>2)]=$mul41; //@line 1921 "wavegen.cpp"
            ; //@line 1922 "wavegen.cpp"
          }
  
          HEAP32[((((_wdata+40)&4294967295))>>2)]=0; //@line 1923 "wavegen.cpp"
          HEAP32[((((_wdata+48)&4294967295))>>2)]=0; //@line 1924 "wavegen.cpp"
          var $49=$q; //@line 1925 "wavegen.cpp"
          var $arrayidx43=(($49+8)&4294967295); //@line 1925 "wavegen.cpp"
          var $50=HEAP32[(($arrayidx43)>>2)]; //@line 1925 "wavegen.cpp"
          var $51=($50); //@line 1925 "wavegen.cpp"
          HEAP32[((((_wdata+24)&4294967295))>>2)]=$51; //@line 1925 "wavegen.cpp"
          __label__ = 32;break $sw_epilogthread_pre_split$$sw_bb$$sw_bb19$$sw_bb24$$sw_bb31$$sw_bb44$$sw_bb45$$sw_bb52$$sw_bb53$$sw_bb60$$sw_bb70$$sw_bb73$$sw_bb76$$sw_bb79$$sw_bb81$$sw_bb86$6; //@line 1926 "wavegen.cpp"
        }
        else if (__label__ == 19) {
  
          HEAP32[((((_wdata+28)&4294967295))>>2)]=0; //@line 1929 "wavegen.cpp"
          __label__ = 20;break $sw_epilogthread_pre_split$$sw_bb$$sw_bb19$$sw_bb24$$sw_bb31$$sw_bb44$$sw_bb45$$sw_bb52$$sw_bb53$$sw_bb60$$sw_bb70$$sw_bb73$$sw_bb76$$sw_bb79$$sw_bb81$$sw_bb86$6; //@line 1929 "wavegen.cpp"
        }
        else if (__label__ == 21) {
  
          HEAP32[((((_wdata+28)&4294967295))>>2)]=0; //@line 1937 "wavegen.cpp"
          __label__ = 22;break $sw_epilogthread_pre_split$$sw_bb$$sw_bb19$$sw_bb24$$sw_bb31$$sw_bb44$$sw_bb45$$sw_bb52$$sw_bb53$$sw_bb60$$sw_bb70$$sw_bb73$$sw_bb76$$sw_bb79$$sw_bb81$$sw_bb86$6; //@line 1937 "wavegen.cpp"
        }
        else if (__label__ == 23) {
  
          var $74=$q; //@line 1945 "wavegen.cpp"
          var $arrayidx61=(($74+4)&4294967295); //@line 1945 "wavegen.cpp"
          var $75=HEAP32[(($arrayidx61)>>2)]; //@line 1945 "wavegen.cpp"
          var $76=$q; //@line 1945 "wavegen.cpp"
          var $arrayidx62=(($76+8)&4294967295); //@line 1945 "wavegen.cpp"
          var $77=HEAP32[(($arrayidx62)>>2)]; //@line 1945 "wavegen.cpp"
          var $78=$q; //@line 1945 "wavegen.cpp"
          var $arrayidx63=(($78+12)&4294967295); //@line 1945 "wavegen.cpp"
          var $79=HEAP32[(($arrayidx63)>>2)]; //@line 1945 "wavegen.cpp"
          var $80=HEAP32[((_out_ptr)>>2)]; //@line 1945 "wavegen.cpp"
          __Z11MarkerEventijiPh($75, $77, $79, $80); //@line 1945 "wavegen.cpp"
          var $81=$q; //@line 1949 "wavegen.cpp"
          var $arrayidx64=(($81+4)&4294967295); //@line 1949 "wavegen.cpp"
          var $82=HEAP32[(($arrayidx64)>>2)]; //@line 1949 "wavegen.cpp"
          var $cmp65=((($82))|0)==1; //@line 1949 "wavegen.cpp"
          if (!($cmp65)) { __label__ = 32;break $sw_epilogthread_pre_split$$sw_bb$$sw_bb19$$sw_bb24$$sw_bb31$$sw_bb44$$sw_bb45$$sw_bb52$$sw_bb53$$sw_bb60$$sw_bb70$$sw_bb73$$sw_bb76$$sw_bb79$$sw_bb81$$sw_bb86$6; } //@line 1949 "wavegen.cpp"
  
          var $83=$q; //@line 1951 "wavegen.cpp"
          var $arrayidx67=(($83+8)&4294967295); //@line 1951 "wavegen.cpp"
          var $84=HEAP32[(($arrayidx67)>>2)]; //@line 1951 "wavegen.cpp"
          var $and68=($84) & 16777215; //@line 1951 "wavegen.cpp"
          HEAP32[((_current_source_index)>>2)]=$and68; //@line 1951 "wavegen.cpp"
          __label__ = 32;break $sw_epilogthread_pre_split$$sw_bb$$sw_bb19$$sw_bb24$$sw_bb31$$sw_bb44$$sw_bb45$$sw_bb52$$sw_bb53$$sw_bb60$$sw_bb70$$sw_bb73$$sw_bb76$$sw_bb79$$sw_bb81$$sw_bb86$6; //@line 1952 "wavegen.cpp"
        }
        else if (__label__ == 25) {
  
          var $85=$length; //@line 1956 "wavegen.cpp"
          var $86=$q; //@line 1956 "wavegen.cpp"
          var $arrayidx71=(($86+8)&4294967295); //@line 1956 "wavegen.cpp"
          var $87=HEAP32[(($arrayidx71)>>2)]; //@line 1956 "wavegen.cpp"
          var $88=($87); //@line 1956 "wavegen.cpp"
          var $89=$q; //@line 1956 "wavegen.cpp"
          var $arrayidx72=(($89+12)&4294967295); //@line 1956 "wavegen.cpp"
          var $90=HEAP32[(($arrayidx72)>>2)]; //@line 1956 "wavegen.cpp"
          __ZL12SetAmplitudeiPhi($85, $88, $90); //@line 1956 "wavegen.cpp"
          __label__ = 32;break $sw_epilogthread_pre_split$$sw_bb$$sw_bb19$$sw_bb24$$sw_bb31$$sw_bb44$$sw_bb45$$sw_bb52$$sw_bb53$$sw_bb60$$sw_bb70$$sw_bb73$$sw_bb76$$sw_bb79$$sw_bb81$$sw_bb86$6; //@line 1957 "wavegen.cpp"
        }
        else if (__label__ == 26) {
  
          var $91=$q; //@line 1960 "wavegen.cpp"
          var $arrayidx74=(($91+4)&4294967295); //@line 1960 "wavegen.cpp"
          var $92=HEAP32[(($arrayidx74)>>2)]; //@line 1960 "wavegen.cpp"
          var $93=($92); //@line 1960 "wavegen.cpp"
          __Z15WavegenSetVoiceP7voice_t($93); //@line 1960 "wavegen.cpp"
          var $94=$q; //@line 1961 "wavegen.cpp"
          var $arrayidx75=(($94+4)&4294967295); //@line 1961 "wavegen.cpp"
          var $95=HEAP32[(($arrayidx75)>>2)]; //@line 1961 "wavegen.cpp"
          var $96=($95); //@line 1961 "wavegen.cpp"
          var $97=$96; //@line 1961 "wavegen.cpp"
          ; //@line 1961 "wavegen.cpp"
          __label__ = 32;break $sw_epilogthread_pre_split$$sw_bb$$sw_bb19$$sw_bb24$$sw_bb31$$sw_bb44$$sw_bb45$$sw_bb52$$sw_bb53$$sw_bb60$$sw_bb70$$sw_bb73$$sw_bb76$$sw_bb79$$sw_bb81$$sw_bb86$6; //@line 1962 "wavegen.cpp"
        }
        else if (__label__ == 27) {
  
          var $98=$q; //@line 1965 "wavegen.cpp"
          var $arrayidx77=(($98+4)&4294967295); //@line 1965 "wavegen.cpp"
          var $99=HEAP32[(($arrayidx77)>>2)]; //@line 1965 "wavegen.cpp"
          var $100=$q; //@line 1965 "wavegen.cpp"
          var $arrayidx78=(($100+8)&4294967295); //@line 1965 "wavegen.cpp"
          var $101=HEAP32[(($arrayidx78)>>2)]; //@line 1965 "wavegen.cpp"
          __Z11SetEmbeddedii($99, $101); //@line 1965 "wavegen.cpp"
          __label__ = 32;break $sw_epilogthread_pre_split$$sw_bb$$sw_bb19$$sw_bb24$$sw_bb31$$sw_bb44$$sw_bb45$$sw_bb52$$sw_bb53$$sw_bb60$$sw_bb70$$sw_bb73$$sw_bb76$$sw_bb79$$sw_bb81$$sw_bb86$6; //@line 1966 "wavegen.cpp"
        }
        else if (__label__ == 28) {
  
          var $102=$length; //@line 1969 "wavegen.cpp"
          var $103=HEAP32[((__ZZ12WavegenFill2iE6resume)>>2)]; //@line 1969 "wavegen.cpp"
          var $call80=__Z10MbrolaFillii($102, $103); //@line 1969 "wavegen.cpp"
          $result=$call80; //@line 1969 "wavegen.cpp"
          __lastLabel__ = 28; __label__ = 33;break $sw_epilogthread_pre_split$$sw_bb$$sw_bb19$$sw_bb24$$sw_bb31$$sw_bb44$$sw_bb45$$sw_bb52$$sw_bb53$$sw_bb60$$sw_bb70$$sw_bb73$$sw_bb76$$sw_bb79$$sw_bb81$$sw_bb86$6; //@line 1970 "wavegen.cpp"
        }
        else if (__label__ == 29) {
  
          var $104=$q; //@line 1973 "wavegen.cpp"
          var $arrayidx82=(($104+4)&4294967295); //@line 1973 "wavegen.cpp"
          var $105=HEAP32[(($arrayidx82)>>2)]; //@line 1973 "wavegen.cpp"
          HEAP32[((((_wdata+60)&4294967295))>>2)]=$105; //@line 1973 "wavegen.cpp"
          var $cmp83=((($105))|0)==0; //@line 1973 "wavegen.cpp"
          if (!($cmp83)) { __label__ = 32;break $sw_epilogthread_pre_split$$sw_bb$$sw_bb19$$sw_bb24$$sw_bb31$$sw_bb44$$sw_bb45$$sw_bb52$$sw_bb53$$sw_bb60$$sw_bb70$$sw_bb73$$sw_bb76$$sw_bb79$$sw_bb81$$sw_bb86$6; } //@line 1973 "wavegen.cpp"
  
          HEAP32[((((_wdata+60)&4294967295))>>2)]=100; //@line 1974 "wavegen.cpp"
          __label__ = 32;break $sw_epilogthread_pre_split$$sw_bb$$sw_bb19$$sw_bb24$$sw_bb31$$sw_bb44$$sw_bb45$$sw_bb52$$sw_bb53$$sw_bb60$$sw_bb70$$sw_bb73$$sw_bb76$$sw_bb79$$sw_bb81$$sw_bb86$6; //@line 1974 "wavegen.cpp"
        }
        else if (__label__ == 31) {
  
          var $106=$q; //@line 1978 "wavegen.cpp"
          var $arrayidx87=(($106+4)&4294967295); //@line 1978 "wavegen.cpp"
          var $107=HEAP32[(($arrayidx87)>>2)]; //@line 1978 "wavegen.cpp"
          var $conv=((($107))|0); //@line 1978 "wavegen.cpp"
          var $div=($conv)/1024; //@line 1978 "wavegen.cpp"
          HEAPF32[((_sonicSpeed)>>2)]=$div; //@line 1978 "wavegen.cpp"
          __label__ = 32;break $sw_epilogthread_pre_split$$sw_bb$$sw_bb19$$sw_bb24$$sw_bb31$$sw_bb44$$sw_bb45$$sw_bb52$$sw_bb53$$sw_bb60$$sw_bb70$$sw_bb73$$sw_bb76$$sw_bb79$$sw_bb81$$sw_bb86$6; //@line 1979 "wavegen.cpp"
        }
      } while(0);
      if (__label__ == 32) {
  
        var $_pr=$result; //@line 1982 "wavegen.cpp"
        __lastLabel__ = 32; ;
      }
      else if (__label__ == 20) {
  
        var $52=HEAP32[((__ZL11echo_length)>>2)]; //@line 1931 "wavegen.cpp"
        HEAP32[((__ZZ12WavegenFill2iE13echo_complete)>>2)]=$52; //@line 1931 "wavegen.cpp"
        var $53=$length; //@line 1932 "wavegen.cpp"
        var $and46=($53) & 65535; //@line 1932 "wavegen.cpp"
        var $54=$q; //@line 1932 "wavegen.cpp"
        var $arrayidx47=(($54+4)&4294967295); //@line 1932 "wavegen.cpp"
        var $55=HEAP32[(($arrayidx47)>>2)]; //@line 1932 "wavegen.cpp"
        var $shr48=($55) >> 16; //@line 1932 "wavegen.cpp"
        var $56=HEAP32[((__ZZ12WavegenFill2iE6resume)>>2)]; //@line 1932 "wavegen.cpp"
        var $57=$q; //@line 1932 "wavegen.cpp"
        var $arrayidx49=(($57+8)&4294967295); //@line 1932 "wavegen.cpp"
        var $58=HEAP32[(($arrayidx49)>>2)]; //@line 1932 "wavegen.cpp"
        var $59=($58); //@line 1932 "wavegen.cpp"
        var $60=$q; //@line 1932 "wavegen.cpp"
        var $arrayidx50=(($60+12)&4294967295); //@line 1932 "wavegen.cpp"
        var $61=HEAP32[(($arrayidx50)>>2)]; //@line 1932 "wavegen.cpp"
        var $62=($61); //@line 1932 "wavegen.cpp"
        var $call51=__ZL8Wavegen2iiiP7frame_tS0_($and46, $shr48, $56, $59, $62); //@line 1932 "wavegen.cpp"
        $result=$call51; //@line 1932 "wavegen.cpp"
        __lastLabel__ = 20; ; //@line 1933 "wavegen.cpp"
      }
      else if (__label__ == 22) {
  
        var $63=HEAP32[((__ZL11echo_length)>>2)]; //@line 1939 "wavegen.cpp"
        HEAP32[((__ZZ12WavegenFill2iE13echo_complete)>>2)]=$63; //@line 1939 "wavegen.cpp"
        var $64=$length; //@line 1940 "wavegen.cpp"
        var $and54=($64) & 65535; //@line 1940 "wavegen.cpp"
        var $65=$q; //@line 1940 "wavegen.cpp"
        var $arrayidx55=(($65+4)&4294967295); //@line 1940 "wavegen.cpp"
        var $66=HEAP32[(($arrayidx55)>>2)]; //@line 1940 "wavegen.cpp"
        var $shr56=($66) >> 16; //@line 1940 "wavegen.cpp"
        var $67=HEAP32[((__ZZ12WavegenFill2iE6resume)>>2)]; //@line 1940 "wavegen.cpp"
        var $68=$q; //@line 1940 "wavegen.cpp"
        var $arrayidx57=(($68+8)&4294967295); //@line 1940 "wavegen.cpp"
        var $69=HEAP32[(($arrayidx57)>>2)]; //@line 1940 "wavegen.cpp"
        var $70=($69); //@line 1940 "wavegen.cpp"
        var $71=$q; //@line 1940 "wavegen.cpp"
        var $arrayidx58=(($71+12)&4294967295); //@line 1940 "wavegen.cpp"
        var $72=HEAP32[(($arrayidx58)>>2)]; //@line 1940 "wavegen.cpp"
        var $73=($72); //@line 1940 "wavegen.cpp"
        var $call59=__Z14Wavegen_Klatt2iiiP7frame_tS0_($and54, $shr56, $67, $70, $73); //@line 1940 "wavegen.cpp"
        $result=$call59; //@line 1940 "wavegen.cpp"
        __lastLabel__ = 22; ; //@line 1941 "wavegen.cpp"
      }
  
      var $108=__lastLabel__ == 32 ? $_pr : (__lastLabel__ == 28 ? $call80 : (__lastLabel__ == 22 ? $call59 : (__lastLabel__ == 20 ? $call51 : (__lastLabel__ == 15 ? $call30 : ($call23))))); //@line 1982 "wavegen.cpp"
      var $cmp88=((($108))|0)==0; //@line 1982 "wavegen.cpp"
      if ($cmp88) { __label__ = 34;; } else { __label__ = 35;; } //@line 1982 "wavegen.cpp"
      if (__label__ == 34) {
  
        __ZL12WcmdqIncHeadv(); //@line 1984 "wavegen.cpp"
        HEAP32[((__ZZ12WavegenFill2iE6resume)>>2)]=0; //@line 1985 "wavegen.cpp"
        __label__ = 1;continue $while_cond$2; //@line 1986 "wavegen.cpp"
      }
      else if (__label__ == 35) {
  
        HEAP32[((__ZZ12WavegenFill2iE6resume)>>2)]=1; //@line 1989 "wavegen.cpp"
        __label__ = 1;continue $while_cond$2;
      }
    }
    $while_end91$$if_then$36: do { 
      if (__label__ == 36) {
  
        $retval=0; //@line 1993 "wavegen.cpp"
        ; //@line 1993 "wavegen.cpp"
      }
      else if (__label__ == 3) {
  
        var $2=HEAP32[((__ZZ12WavegenFill2iE13echo_complete)>>2)]; //@line 1868 "wavegen.cpp"
        var $cmp2=((($2))|0) > 0; //@line 1868 "wavegen.cpp"
        if ($cmp2) { __label__ = 4;; } else { __label__ = 6;; } //@line 1868 "wavegen.cpp"
        $if_then3$$if_end7$39: do { 
          if (__label__ == 4) {
  
            var $3=HEAP32[((__ZZ12WavegenFill2iE13echo_complete)>>2)]; //@line 1871 "wavegen.cpp"
            var $4=HEAP32[((__ZZ12WavegenFill2iE6resume)>>2)]; //@line 1871 "wavegen.cpp"
            var $call4=__ZL11PlaySilenceii($3, $4); //@line 1871 "wavegen.cpp"
            HEAP32[((__ZZ12WavegenFill2iE6resume)>>2)]=$call4; //@line 1871 "wavegen.cpp"
            var $5=HEAP32[((__ZZ12WavegenFill2iE6resume)>>2)]; //@line 1872 "wavegen.cpp"
            var $cmp5=((($5))|0)==1; //@line 1872 "wavegen.cpp"
            if (!($cmp5)) { __label__ = 6;break $if_then3$$if_end7$39; } //@line 1872 "wavegen.cpp"
  
            $retval=0; //@line 1873 "wavegen.cpp"
            __label__ = 37;break $while_end91$$if_then$36; //@line 1873 "wavegen.cpp"
          }
        } while(0);
  
        var $6=$fill_zeros_addr; //@line 1876 "wavegen.cpp"
        var $tobool=((($6))|0)!=0; //@line 1876 "wavegen.cpp"
        if ($tobool) { __label__ = 7;; } else { __label__ = 9;; } //@line 1876 "wavegen.cpp"
        $while_cond9_preheader$$if_end12$43: do { 
          if (__label__ == 7) {
  
            var $7=HEAPU32[((_out_ptr)>>2)]; //@line 1878 "wavegen.cpp"
            var $8=HEAPU32[((_out_end)>>2)]; //@line 1878 "wavegen.cpp"
            var $cmp101=((($7))>>>0) < ((($8))>>>0); //@line 1878 "wavegen.cpp"
            if (!($cmp101)) { __label__ = 9;break $while_cond9_preheader$$if_end12$43; } //@line 1878 "wavegen.cpp"
            while(1) { 
  
              var $9=HEAP32[((_out_ptr)>>2)]; //@line 1879 "wavegen.cpp"
              var $incdec_ptr=(($9+1)&4294967295); //@line 1879 "wavegen.cpp"
              HEAP32[((_out_ptr)>>2)]=$incdec_ptr; //@line 1879 "wavegen.cpp"
              HEAP8[($9)]=0; //@line 1879 "wavegen.cpp"
              var $10=HEAPU32[((_out_ptr)>>2)]; //@line 1878 "wavegen.cpp"
              var $11=HEAPU32[((_out_end)>>2)]; //@line 1878 "wavegen.cpp"
              var $cmp10=((($10))>>>0) < ((($11))>>>0); //@line 1878 "wavegen.cpp"
              if ($cmp10) { __label__ = 8;continue ; } else { __label__ = 9;break $while_cond9_preheader$$if_end12$43; } //@line 1878 "wavegen.cpp"
            }
          }
        } while(0);
  
        $retval=1; //@line 1881 "wavegen.cpp"
        ; //@line 1881 "wavegen.cpp"
      }
    } while(0);
  
    var $109=$retval; //@line 1994 "wavegen.cpp"
    ;
    return $109; //@line 1994 "wavegen.cpp"
    return null;
  }
  

  function __ZL11PlaySilenceii($length, $resume) {
    ;
    var __label__;
    var __lastLabel__ = null;
  
    var $retval;
    var $length_addr;
    var $resume_addr;
    var $value;
    $length_addr=$length;
    $resume_addr=$resume;
    $value=0; //@line 1439 "wavegen.cpp"
    HEAP32[((__ZL8nsamples)>>2)]=0; //@line 1441 "wavegen.cpp"
    HEAP32[((__ZL11samplecount)>>2)]=0; //@line 1442 "wavegen.cpp"
    HEAP32[((__ZL9wavephase)>>2)]=2147483647; //@line 1443 "wavegen.cpp"
    var $0=$length_addr; //@line 1445 "wavegen.cpp"
    var $cmp=((($0))|0)==0; //@line 1445 "wavegen.cpp"
    if ($cmp) { __label__ = 1;; } else { __label__ = 2;; } //@line 1445 "wavegen.cpp"
    if (__label__ == 1) {
  
      $retval=0; //@line 1446 "wavegen.cpp"
      ; //@line 1446 "wavegen.cpp"
    }
    else if (__label__ == 2) {
  
      var $1=$resume_addr; //@line 1448 "wavegen.cpp"
      var $cmp1=((($1))|0)==0; //@line 1448 "wavegen.cpp"
      if ($cmp1) { __label__ = 3;; } else { __label__ = 4;; } //@line 1448 "wavegen.cpp"
      $if_then2$$while_condthread_pre_split$5: do { 
        if (__label__ == 3) {
  
          var $2=$length_addr; //@line 1449 "wavegen.cpp"
          HEAP32[((__ZZL11PlaySilenceiiE9n_samples)>>2)]=$2; //@line 1449 "wavegen.cpp"
          __lastLabel__ = 3; __label__ = 5;break $if_then2$$while_condthread_pre_split$5; //@line 1449 "wavegen.cpp"
        }
      } while(0);
      while(1) { 
        if (__label__ == 5) {
  
          var $3=__lastLabel__ == 4 ? $_pr : ($2); //@line 1451 "wavegen.cpp"
          var $dec=((($3)-1)&4294967295); //@line 1451 "wavegen.cpp"
          HEAP32[((__ZZL11PlaySilenceiiE9n_samples)>>2)]=$dec; //@line 1451 "wavegen.cpp"
          var $cmp4=((($3))|0) > 0; //@line 1451 "wavegen.cpp"
          if (!($cmp4)) { __label__ = 12;break ; } //@line 1451 "wavegen.cpp"
  
          var $4=HEAP32[((_echo_tail)>>2)]; //@line 1453 "wavegen.cpp"
          var $inc=((($4)+1)&4294967295); //@line 1453 "wavegen.cpp"
          HEAP32[((_echo_tail)>>2)]=$inc; //@line 1453 "wavegen.cpp"
          var $arrayidx=((_echo_buf+$4*2)&4294967295); //@line 1453 "wavegen.cpp"
          var $5=HEAP16[(($arrayidx)>>1)]; //@line 1453 "wavegen.cpp"
          var $conv=(tempInt=(($5)),(tempInt>=32768?tempInt-65536:tempInt)); //@line 1453 "wavegen.cpp"
          var $6=HEAP32[((_echo_amp)>>2)]; //@line 1453 "wavegen.cpp"
          var $mul=((($conv)*($6))&4294967295); //@line 1453 "wavegen.cpp"
          var $shr=($mul) >> 8; //@line 1453 "wavegen.cpp"
          $value=$shr; //@line 1453 "wavegen.cpp"
          var $7=HEAP32[((_echo_tail)>>2)]; //@line 1455 "wavegen.cpp"
          var $cmp5=((($7))|0) >= 5500; //@line 1455 "wavegen.cpp"
          if ($cmp5) { __label__ = 7;; } else { __label__ = 8;; } //@line 1455 "wavegen.cpp"
          if (__label__ == 7) {
  
            HEAP32[((_echo_tail)>>2)]=0; //@line 1456 "wavegen.cpp"
            ; //@line 1456 "wavegen.cpp"
          }
  
          var $8=$value; //@line 1458 "wavegen.cpp"
          var $conv8=((($8)) & 255); //@line 1458 "wavegen.cpp"
          var $9=HEAP32[((_out_ptr)>>2)]; //@line 1458 "wavegen.cpp"
          var $incdec_ptr=(($9+1)&4294967295); //@line 1458 "wavegen.cpp"
          HEAP32[((_out_ptr)>>2)]=$incdec_ptr; //@line 1458 "wavegen.cpp"
          HEAP8[($9)]=$conv8; //@line 1458 "wavegen.cpp"
          var $10=$value; //@line 1459 "wavegen.cpp"
          var $shr9=($10) >> 8; //@line 1459 "wavegen.cpp"
          var $conv10=((($shr9)) & 255); //@line 1459 "wavegen.cpp"
          var $11=HEAP32[((_out_ptr)>>2)]; //@line 1459 "wavegen.cpp"
          var $incdec_ptr11=(($11+1)&4294967295); //@line 1459 "wavegen.cpp"
          HEAP32[((_out_ptr)>>2)]=$incdec_ptr11; //@line 1459 "wavegen.cpp"
          HEAP8[($11)]=$conv10; //@line 1459 "wavegen.cpp"
          var $12=$value; //@line 1461 "wavegen.cpp"
          var $conv12=((($12)) & 65535); //@line 1461 "wavegen.cpp"
          var $13=HEAP32[((_echo_head)>>2)]; //@line 1461 "wavegen.cpp"
          var $inc13=((($13)+1)&4294967295); //@line 1461 "wavegen.cpp"
          HEAP32[((_echo_head)>>2)]=$inc13; //@line 1461 "wavegen.cpp"
          var $arrayidx14=((_echo_buf+$13*2)&4294967295); //@line 1461 "wavegen.cpp"
          HEAP16[(($arrayidx14)>>1)]=$conv12; //@line 1461 "wavegen.cpp"
          var $14=HEAP32[((_echo_head)>>2)]; //@line 1462 "wavegen.cpp"
          var $cmp15=((($14))|0) >= 5500; //@line 1462 "wavegen.cpp"
          if ($cmp15) { __label__ = 9;; } else { __label__ = 10;; } //@line 1462 "wavegen.cpp"
          if (__label__ == 9) {
  
            HEAP32[((_echo_head)>>2)]=0; //@line 1463 "wavegen.cpp"
            ; //@line 1463 "wavegen.cpp"
          }
  
          var $15=HEAPU32[((_out_ptr)>>2)]; //@line 1465 "wavegen.cpp"
          var $16=HEAPU32[((_out_end)>>2)]; //@line 1465 "wavegen.cpp"
          var $cmp18=((($15))>>>0) >= ((($16))>>>0); //@line 1465 "wavegen.cpp"
          if ($cmp18) { __label__ = 11;break ; } else { __label__ = 4;continue ; } //@line 1465 "wavegen.cpp"
        }
        else if (__label__ == 4) {
  
          var $_pr=HEAP32[((__ZZL11PlaySilenceiiE9n_samples)>>2)]; //@line 1451 "wavegen.cpp"
          __lastLabel__ = 4; __label__ = 5;continue ;
        }
      }
      if (__label__ == 12) {
  
        $retval=0; //@line 1468 "wavegen.cpp"
        ; //@line 1468 "wavegen.cpp"
      }
      else if (__label__ == 11) {
  
        $retval=1; //@line 1466 "wavegen.cpp"
        ; //@line 1466 "wavegen.cpp"
      }
    }
  
    var $17=$retval; //@line 1469 "wavegen.cpp"
    ;
    return $17; //@line 1469 "wavegen.cpp"
    return null;
  }
  

  function __ZL8PlayWaveiiPhii($length, $resume, $data, $scale, $amp) {
    ;
    var __label__;
  
    var $retval;
    var $length_addr;
    var $resume_addr;
    var $data_addr;
    var $scale_addr;
    var $amp_addr;
    var $value;
    var $c;
    $length_addr=$length;
    $resume_addr=$resume;
    $data_addr=$data;
    $scale_addr=$scale;
    $amp_addr=$amp;
    var $0=$resume_addr; //@line 1480 "wavegen.cpp"
    var $cmp=((($0))|0)==0; //@line 1480 "wavegen.cpp"
    if ($cmp) { __label__ = 1;; } else { __label__ = 2;; } //@line 1480 "wavegen.cpp"
    if (__label__ == 1) {
  
      var $1=$length_addr; //@line 1482 "wavegen.cpp"
      HEAP32[((__ZZL8PlayWaveiiPhiiE9n_samples)>>2)]=$1; //@line 1482 "wavegen.cpp"
      HEAP32[((__ZZL8PlayWaveiiPhiiE2ix)>>2)]=0; //@line 1483 "wavegen.cpp"
      ; //@line 1484 "wavegen.cpp"
    }
  
    HEAP32[((__ZL8nsamples)>>2)]=0; //@line 1486 "wavegen.cpp"
    HEAP32[((__ZL11samplecount)>>2)]=0; //@line 1487 "wavegen.cpp"
    ; //@line 1489 "wavegen.cpp"
    $while_cond$5: while(1) { 
  
      var $2=HEAP32[((__ZZL8PlayWaveiiPhiiE9n_samples)>>2)]; //@line 1489 "wavegen.cpp"
      var $dec=((($2)-1)&4294967295); //@line 1489 "wavegen.cpp"
      HEAP32[((__ZZL8PlayWaveiiPhiiE9n_samples)>>2)]=$dec; //@line 1489 "wavegen.cpp"
      var $cmp1=((($2))|0) > 0; //@line 1489 "wavegen.cpp"
      if (!($cmp1)) { __label__ = 17;break $while_cond$5; } //@line 1489 "wavegen.cpp"
  
      var $3=$scale_addr; //@line 1491 "wavegen.cpp"
      var $cmp2=((($3))|0)==0; //@line 1491 "wavegen.cpp"
      var $4=HEAPU32[((__ZZL8PlayWaveiiPhiiE2ix)>>2)]; //@line 1494 "wavegen.cpp"
      var $add=((($4)+1)&4294967295); //@line 1494 "wavegen.cpp"
      if ($cmp2) { __label__ = 5;; } else { __label__ = 6;; } //@line 1491 "wavegen.cpp"
      if (__label__ == 5) {
  
        var $5=$data_addr; //@line 1494 "wavegen.cpp"
        var $arrayidx=(($5+$add)&4294967295); //@line 1494 "wavegen.cpp"
        var $6=HEAP8[($arrayidx)]; //@line 1494 "wavegen.cpp"
        $c=$6; //@line 1494 "wavegen.cpp"
        var $7=HEAP32[((__ZZL8PlayWaveiiPhiiE2ix)>>2)]; //@line 1495 "wavegen.cpp"
        var $8=$data_addr; //@line 1495 "wavegen.cpp"
        var $arrayidx4=(($8+$7)&4294967295); //@line 1495 "wavegen.cpp"
        var $9=HEAPU8[($arrayidx4)]; //@line 1495 "wavegen.cpp"
        var $conv=((($9))&255); //@line 1495 "wavegen.cpp"
        var $10=$c; //@line 1495 "wavegen.cpp"
        var $conv5=(tempInt=(($10)),(tempInt>=128?tempInt-256:tempInt)); //@line 1495 "wavegen.cpp"
        var $mul=((($conv5)*256)&4294967295); //@line 1495 "wavegen.cpp"
        var $add6=((($mul)+($conv))&4294967295); //@line 1495 "wavegen.cpp"
        $value=$add6; //@line 1495 "wavegen.cpp"
        var $11=HEAP32[((__ZZL8PlayWaveiiPhiiE2ix)>>2)]; //@line 1496 "wavegen.cpp"
        var $add7=((($11)+2)&4294967295); //@line 1496 "wavegen.cpp"
        HEAP32[((__ZZL8PlayWaveiiPhiiE2ix)>>2)]=$add7; //@line 1496 "wavegen.cpp"
        ; //@line 1497 "wavegen.cpp"
      }
      else if (__label__ == 6) {
  
        HEAP32[((__ZZL8PlayWaveiiPhiiE2ix)>>2)]=$add; //@line 1501 "wavegen.cpp"
        var $12=$data_addr; //@line 1501 "wavegen.cpp"
        var $arrayidx8=(($12+$4)&4294967295); //@line 1501 "wavegen.cpp"
        var $13=HEAP8[($arrayidx8)]; //@line 1501 "wavegen.cpp"
        var $conv9=(tempInt=(($13)),(tempInt>=128?tempInt-256:tempInt)); //@line 1501 "wavegen.cpp"
        var $14=$scale_addr; //@line 1501 "wavegen.cpp"
        var $mul10=((($conv9)*($14))&4294967295); //@line 1501 "wavegen.cpp"
        $value=$mul10; //@line 1501 "wavegen.cpp"
        ;
      }
  
      var $15=HEAP32[((__ZL13consonant_amp)>>2)]; //@line 1503 "wavegen.cpp"
      var $16=HEAP32[((__ZL17general_amplitude)>>2)]; //@line 1503 "wavegen.cpp"
      var $17=$value; //@line 1503 "wavegen.cpp"
      var $mul12=((($16)*($15))&4294967295); //@line 1503 "wavegen.cpp"
      var $mul13=((($mul12)*($17))&4294967295); //@line 1503 "wavegen.cpp"
      $value=$mul13; //@line 1503 "wavegen.cpp"
      var $18=$value; //@line 1504 "wavegen.cpp"
      var $shr=($18) >> 10; //@line 1504 "wavegen.cpp"
      $value=$shr; //@line 1504 "wavegen.cpp"
      var $19=$value; //@line 1505 "wavegen.cpp"
      var $20=$amp_addr; //@line 1505 "wavegen.cpp"
      var $mul14=((($20)*($19))&4294967295); //@line 1505 "wavegen.cpp"
      var $div=((((($mul14))|0)/32)|0); //@line 1505 "wavegen.cpp"
      $value=$div; //@line 1505 "wavegen.cpp"
      var $21=HEAP32[((_echo_tail)>>2)]; //@line 1507 "wavegen.cpp"
      var $inc15=((($21)+1)&4294967295); //@line 1507 "wavegen.cpp"
      HEAP32[((_echo_tail)>>2)]=$inc15; //@line 1507 "wavegen.cpp"
      var $arrayidx16=((_echo_buf+$21*2)&4294967295); //@line 1507 "wavegen.cpp"
      var $22=HEAP16[(($arrayidx16)>>1)]; //@line 1507 "wavegen.cpp"
      var $conv17=(tempInt=(($22)),(tempInt>=32768?tempInt-65536:tempInt)); //@line 1507 "wavegen.cpp"
      var $23=HEAP32[((_echo_amp)>>2)]; //@line 1507 "wavegen.cpp"
      var $mul18=((($conv17)*($23))&4294967295); //@line 1507 "wavegen.cpp"
      var $shr19=($mul18) >> 8; //@line 1507 "wavegen.cpp"
      var $24=$value; //@line 1507 "wavegen.cpp"
      var $add20=((($shr19)+($24))&4294967295); //@line 1507 "wavegen.cpp"
      $value=$add20; //@line 1507 "wavegen.cpp"
      var $cmp21=((($add20))|0) > 32767; //@line 1509 "wavegen.cpp"
      if ($cmp21) { __label__ = 8;; } else { __label__ = 9;; } //@line 1509 "wavegen.cpp"
      $if_then22$$if_else23$12: do { 
        if (__label__ == 8) {
  
          $value=32768; //@line 1510 "wavegen.cpp"
          ; //@line 1510 "wavegen.cpp"
        }
        else if (__label__ == 9) {
  
          var $25=$value; //@line 1512 "wavegen.cpp"
          var $cmp24=((($25))|0) < -32768; //@line 1512 "wavegen.cpp"
          if (!($cmp24)) { __label__ = 11;break $if_then22$$if_else23$12; } //@line 1512 "wavegen.cpp"
  
          $value=-32768; //@line 1513 "wavegen.cpp"
          ; //@line 1513 "wavegen.cpp"
        }
      } while(0);
  
      var $26=HEAP32[((_echo_tail)>>2)]; //@line 1515 "wavegen.cpp"
      var $cmp28=((($26))|0) >= 5500; //@line 1515 "wavegen.cpp"
      if ($cmp28) { __label__ = 12;; } else { __label__ = 13;; } //@line 1515 "wavegen.cpp"
      if (__label__ == 12) {
  
        HEAP32[((_echo_tail)>>2)]=0; //@line 1516 "wavegen.cpp"
        ; //@line 1516 "wavegen.cpp"
      }
  
      var $27=$value; //@line 1518 "wavegen.cpp"
      var $conv31=((($27)) & 255); //@line 1518 "wavegen.cpp"
      var $28=HEAP32[((_out_ptr)>>2)]; //@line 1518 "wavegen.cpp"
      var $arrayidx32=(($28)&4294967295); //@line 1518 "wavegen.cpp"
      HEAP8[($arrayidx32)]=$conv31; //@line 1518 "wavegen.cpp"
      var $29=$value; //@line 1519 "wavegen.cpp"
      var $shr33=($29) >> 8; //@line 1519 "wavegen.cpp"
      var $conv34=((($shr33)) & 255); //@line 1519 "wavegen.cpp"
      var $30=HEAP32[((_out_ptr)>>2)]; //@line 1519 "wavegen.cpp"
      var $arrayidx35=(($30+1)&4294967295); //@line 1519 "wavegen.cpp"
      HEAP8[($arrayidx35)]=$conv34; //@line 1519 "wavegen.cpp"
      var $31=HEAP32[((_out_ptr)>>2)]; //@line 1520 "wavegen.cpp"
      var $add_ptr=(($31+2)&4294967295); //@line 1520 "wavegen.cpp"
      HEAP32[((_out_ptr)>>2)]=$add_ptr; //@line 1520 "wavegen.cpp"
      var $32=$value; //@line 1522 "wavegen.cpp"
      var $mul36=((($32)*3)&4294967295); //@line 1522 "wavegen.cpp"
      var $div37=((((($mul36))|0)/4)|0); //@line 1522 "wavegen.cpp"
      var $conv38=((($div37)) & 65535); //@line 1522 "wavegen.cpp"
      var $33=HEAP32[((_echo_head)>>2)]; //@line 1522 "wavegen.cpp"
      var $inc39=((($33)+1)&4294967295); //@line 1522 "wavegen.cpp"
      HEAP32[((_echo_head)>>2)]=$inc39; //@line 1522 "wavegen.cpp"
      var $arrayidx40=((_echo_buf+$33*2)&4294967295); //@line 1522 "wavegen.cpp"
      HEAP16[(($arrayidx40)>>1)]=$conv38; //@line 1522 "wavegen.cpp"
      var $34=HEAP32[((_echo_head)>>2)]; //@line 1523 "wavegen.cpp"
      var $cmp41=((($34))|0) >= 5500; //@line 1523 "wavegen.cpp"
      if ($cmp41) { __label__ = 14;; } else { __label__ = 15;; } //@line 1523 "wavegen.cpp"
      if (__label__ == 14) {
  
        HEAP32[((_echo_head)>>2)]=0; //@line 1524 "wavegen.cpp"
        ; //@line 1524 "wavegen.cpp"
      }
  
      var $35=HEAPU32[((_out_ptr)>>2)]; //@line 1526 "wavegen.cpp"
      var $36=HEAPU32[((_out_end)>>2)]; //@line 1526 "wavegen.cpp"
      var $cmp44=((($35))>>>0) >= ((($36))>>>0); //@line 1526 "wavegen.cpp"
      if ($cmp44) { __label__ = 16;break $while_cond$5; } else { __label__ = 3;continue $while_cond$5; } //@line 1526 "wavegen.cpp"
    }
    if (__label__ == 17) {
  
      $retval=0; //@line 1529 "wavegen.cpp"
      ; //@line 1529 "wavegen.cpp"
    }
    else if (__label__ == 16) {
  
      $retval=1; //@line 1527 "wavegen.cpp"
      ; //@line 1527 "wavegen.cpp"
    }
  
    var $37=$retval; //@line 1530 "wavegen.cpp"
    ;
    return $37; //@line 1530 "wavegen.cpp"
    return null;
  }
  

  function __ZL8Wavegen2iiiP7frame_tS0_($length, $modulation, $resume, $fr1, $fr2) {
    ;
    var __label__;
  
    var $length_addr;
    var $modulation_addr;
    var $resume_addr;
    var $fr1_addr;
    var $fr2_addr;
    $length_addr=$length;
    $modulation_addr=$modulation;
    $resume_addr=$resume;
    $fr1_addr=$fr1;
    $fr2_addr=$fr2;
    var $0=$resume_addr; //@line 1832 "wavegen.cpp"
    var $cmp=((($0))|0)==0; //@line 1832 "wavegen.cpp"
    if ($cmp) { __label__ = 1;; } else { __label__ = 2;; } //@line 1832 "wavegen.cpp"
    if (__label__ == 1) {
  
      var $1=$length_addr; //@line 1833 "wavegen.cpp"
      var $2=$modulation_addr; //@line 1833 "wavegen.cpp"
      var $3=$fr1_addr; //@line 1833 "wavegen.cpp"
      var $4=$fr2_addr; //@line 1833 "wavegen.cpp"
      var $5=HEAP32[((_wvoice)>>2)]; //@line 1833 "wavegen.cpp"
      __Z8SetSynthiiP7frame_tS0_P7voice_t($1, $2, $3, $4, $5); //@line 1833 "wavegen.cpp"
      ; //@line 1833 "wavegen.cpp"
    }
  
    var $call=__Z7Wavegenv(); //@line 1835 "wavegen.cpp"
    ;
    return $call; //@line 1835 "wavegen.cpp"
    return null;
  }
  

  function __ZL12SetAmplitudeiPhi($length, $amp_env, $value) {
    ;
    var __label__;
  
    var $length_addr;
    var $amp_env_addr;
    var $value_addr;
    $length_addr=$length;
    $amp_env_addr=$amp_env;
    $value_addr=$value;
    HEAP32[((__ZL6amp_ix)>>2)]=0; //@line 1636 "wavegen.cpp"
    var $0=$length_addr; //@line 1637 "wavegen.cpp"
    var $cmp=((($0))|0)==0; //@line 1637 "wavegen.cpp"
    if ($cmp) { __label__ = 1;; } else { __label__ = 2;; } //@line 1637 "wavegen.cpp"
    if (__label__ == 1) {
  
      HEAP32[((__ZL7amp_inc)>>2)]=0; //@line 1638 "wavegen.cpp"
      ; //@line 1638 "wavegen.cpp"
    }
    else if (__label__ == 2) {
  
      var $1=$length_addr; //@line 1640 "wavegen.cpp"
      var $div=((2097152/((($1))|0))|0); //@line 1640 "wavegen.cpp"
      HEAP32[((__ZL7amp_inc)>>2)]=$div; //@line 1640 "wavegen.cpp"
      ;
    }
  
    var $2=$value_addr; //@line 1642 "wavegen.cpp"
    var $3=HEAP32[((__ZL17general_amplitude)>>2)]; //@line 1642 "wavegen.cpp"
    var $mul=((($3)*($2))&4294967295); //@line 1642 "wavegen.cpp"
    var $div1=((((($mul))|0)/16)|0); //@line 1642 "wavegen.cpp"
    HEAP32[((((_wdata+52)&4294967295))>>2)]=$div1; //@line 1642 "wavegen.cpp"
    var $4=HEAP32[((((_wdata+52)&4294967295))>>2)]; //@line 1643 "wavegen.cpp"
    var $5=HEAP32[((_wvoice)>>2)]; //@line 1643 "wavegen.cpp"
    var $consonant_ampv=(($5+104)&4294967295); //@line 1643 "wavegen.cpp"
    var $6=HEAP32[(($consonant_ampv)>>2)]; //@line 1643 "wavegen.cpp"
    var $mul2=((($4)*15)&4294967295); //@line 1643 "wavegen.cpp"
    var $mul3=((($mul2)*($6))&4294967295); //@line 1643 "wavegen.cpp"
    var $div4=((((($mul3))|0)/100)|0); //@line 1643 "wavegen.cpp"
    HEAP32[((((_wdata+56)&4294967295))>>2)]=$div4; //@line 1643 "wavegen.cpp"
    var $7=$amp_env_addr; //@line 1645 "wavegen.cpp"
    HEAP32[((__ZL13amplitude_env)>>2)]=$7; //@line 1645 "wavegen.cpp"
    ;
    return; //@line 1646 "wavegen.cpp"
    return;
  }
  

  function __ZL12WcmdqIncHeadv() {
    ;
    var __label__;
  
    var $0=HEAP32[((_wcmdq_head)>>2)]; //@line 324 "wavegen.cpp"
    var $inc=((($0)+1)&4294967295); //@line 324 "wavegen.cpp"
    HEAP32[((_wcmdq_head)>>2)]=$inc; //@line 324 "wavegen.cpp"
    var $cmp=((($inc))|0) >= 160; //@line 325 "wavegen.cpp"
    if ($cmp) { __label__ = 1;; } else { __label__ = 2;; } //@line 325 "wavegen.cpp"
    if (__label__ == 1) {
  
      HEAP32[((_wcmdq_head)>>2)]=0; //@line 325 "wavegen.cpp"
      ; //@line 325 "wavegen.cpp"
    }
  
    ;
    return; //@line 326 "wavegen.cpp"
    return;
  }
  

  function __Z11WavegenFilli($fill_zeros) {
    ;
    var __label__;
  
    var $fill_zeros_addr;
    var $finished;
    var $p_start;
    var $length;
    var $max_length;
    $fill_zeros_addr=$fill_zeros;
    var $0=HEAP32[((_out_ptr)>>2)]; //@line 2033 "wavegen.cpp"
    $p_start=$0; //@line 2033 "wavegen.cpp"
    var $call=__Z12WavegenFill2i(0); //@line 2036 "wavegen.cpp"
    $finished=$call; //@line 2036 "wavegen.cpp"
    var $1=HEAPF32[((_sonicSpeed)>>2)]; //@line 2038 "wavegen.cpp"
    var $cmp=($1) > 1; //@line 2038 "wavegen.cpp"
    if ($cmp) { __label__ = 1;; } else { __label__ = 3;; } //@line 2038 "wavegen.cpp"
    $if_then$$if_end8$2: do { 
      if (__label__ == 1) {
  
        var $2=HEAP32[((_out_end)>>2)]; //@line 2040 "wavegen.cpp"
        var $3=$p_start; //@line 2040 "wavegen.cpp"
        var $sub_ptr_lhs_cast=($2); //@line 2040 "wavegen.cpp"
        var $sub_ptr_rhs_cast=($3); //@line 2040 "wavegen.cpp"
        var $sub_ptr_sub=((($sub_ptr_lhs_cast)-($sub_ptr_rhs_cast))&4294967295); //@line 2040 "wavegen.cpp"
        $max_length=$sub_ptr_sub; //@line 2040 "wavegen.cpp"
        var $4=$p_start; //@line 2041 "wavegen.cpp"
        var $5=$4; //@line 2041 "wavegen.cpp"
        var $6=HEAP32[((_out_ptr)>>2)]; //@line 2041 "wavegen.cpp"
        var $7=$p_start; //@line 2041 "wavegen.cpp"
        var $sub_ptr_lhs_cast1=($6); //@line 2041 "wavegen.cpp"
        var $sub_ptr_rhs_cast2=($7); //@line 2041 "wavegen.cpp"
        var $sub_ptr_sub3=((($sub_ptr_lhs_cast1)-($sub_ptr_rhs_cast2))&4294967295); //@line 2041 "wavegen.cpp"
        var $div=((((($sub_ptr_sub3))|0)/2)|0); //@line 2041 "wavegen.cpp"
        var $8=$max_length; //@line 2041 "wavegen.cpp"
        var $div4=((((($8))|0)/2)|0); //@line 2041 "wavegen.cpp"
        var $9=$finished; //@line 2041 "wavegen.cpp"
        var $call5=__ZL7SpeedUpPsiii($5, $div, $div4, $9); //@line 2041 "wavegen.cpp"
        var $mul=((($call5)*2)&4294967295); //@line 2041 "wavegen.cpp"
        $length=$mul; //@line 2041 "wavegen.cpp"
        var $10=$p_start; //@line 2042 "wavegen.cpp"
        var $11=$length; //@line 2042 "wavegen.cpp"
        var $add_ptr=(($10+$11)&4294967295); //@line 2042 "wavegen.cpp"
        HEAP32[((_out_ptr)>>2)]=$add_ptr; //@line 2042 "wavegen.cpp"
        var $12=$length; //@line 2044 "wavegen.cpp"
        var $13=$max_length; //@line 2044 "wavegen.cpp"
        var $cmp6=((($12))|0) >= ((($13))|0); //@line 2044 "wavegen.cpp"
        if (!($cmp6)) { __label__ = 3;break $if_then$$if_end8$2; } //@line 2044 "wavegen.cpp"
  
        $finished=0; //@line 2045 "wavegen.cpp"
        ; //@line 2045 "wavegen.cpp"
      }
    } while(0);
  
    var $14=$finished; //@line 2047 "wavegen.cpp"
    ;
    return $14; //@line 2047 "wavegen.cpp"
    return null;
  }
  

  function __ZL7SpeedUpPsiii($outbuf, $length_in, $length_out, $end_of_text) {
    ;
    var __label__;
  
    var $retval;
    var $outbuf_addr;
    var $length_in_addr;
    var $length_out_addr;
    var $end_of_text_addr;
    $outbuf_addr=$outbuf;
    $length_in_addr=$length_in;
    $length_out_addr=$length_out;
    $end_of_text_addr=$end_of_text;
    var $0=$length_in_addr; //@line 2000 "wavegen.cpp"
    var $cmp=((($0))|0) > 0; //@line 2000 "wavegen.cpp"
    if ($cmp) { __label__ = 1;; } else { __label__ = 6;; } //@line 2000 "wavegen.cpp"
    if (__label__ == 1) {
  
      var $1=HEAP32[((__ZL18sonicSpeedupStream)>>2)]; //@line 2002 "wavegen.cpp"
      var $cmp1=((($1))|0)==0; //@line 2002 "wavegen.cpp"
      if ($cmp1) { __label__ = 2;; } else { __label__ = 3;; } //@line 2002 "wavegen.cpp"
      if (__label__ == 2) {
  
        var $call=_sonicCreateStream(22050, 1); //@line 2004 "wavegen.cpp"
        HEAP32[((__ZL18sonicSpeedupStream)>>2)]=$call; //@line 2004 "wavegen.cpp"
        ; //@line 2005 "wavegen.cpp"
      }
  
      var $2=HEAP32[((__ZL18sonicSpeedupStream)>>2)]; //@line 2006 "wavegen.cpp"
      var $call3=_sonicGetSpeed($2); //@line 2006 "wavegen.cpp"
      var $conv=($call3); //@line 2006 "wavegen.cpp"
      var $3=HEAPF32[((_sonicSpeed)>>2)]; //@line 2006 "wavegen.cpp"
      var $cmp4=($conv) != ($3); //@line 2006 "wavegen.cpp"
      if ($cmp4) { __label__ = 4;; } else { __label__ = 5;; } //@line 2006 "wavegen.cpp"
      if (__label__ == 4) {
  
        var $4=HEAP32[((__ZL18sonicSpeedupStream)>>2)]; //@line 2008 "wavegen.cpp"
        var $5=HEAPF32[((_sonicSpeed)>>2)]; //@line 2008 "wavegen.cpp"
        var $conv6=($5); //@line 2008 "wavegen.cpp"
        _sonicSetSpeed($4, $conv6); //@line 2008 "wavegen.cpp"
        ; //@line 2009 "wavegen.cpp"
      }
  
      var $6=HEAP32[((__ZL18sonicSpeedupStream)>>2)]; //@line 2011 "wavegen.cpp"
      var $7=$outbuf_addr; //@line 2011 "wavegen.cpp"
      var $8=$length_in_addr; //@line 2011 "wavegen.cpp"
      var $call8=_sonicWriteShortToStream($6, $7, $8); //@line 2011 "wavegen.cpp"
      ; //@line 2012 "wavegen.cpp"
    }
  
    var $9=HEAP32[((__ZL18sonicSpeedupStream)>>2)]; //@line 2014 "wavegen.cpp"
    var $cmp10=((($9))|0)==0; //@line 2014 "wavegen.cpp"
    if ($cmp10) { __label__ = 7;; } else { __label__ = 8;; } //@line 2014 "wavegen.cpp"
    if (__label__ == 7) {
  
      $retval=0; //@line 2015 "wavegen.cpp"
      ; //@line 2015 "wavegen.cpp"
    }
    else if (__label__ == 8) {
  
      var $10=$end_of_text_addr; //@line 2017 "wavegen.cpp"
      var $tobool=((($10))|0)!=0; //@line 2017 "wavegen.cpp"
      if ($tobool) { __label__ = 9;; } else { __label__ = 10;; } //@line 2017 "wavegen.cpp"
      if (__label__ == 9) {
  
        var $11=HEAP32[((__ZL18sonicSpeedupStream)>>2)]; //@line 2019 "wavegen.cpp"
        var $call14=_sonicFlushStream($11); //@line 2019 "wavegen.cpp"
        ; //@line 2020 "wavegen.cpp"
      }
  
      var $12=HEAP32[((__ZL18sonicSpeedupStream)>>2)]; //@line 2021 "wavegen.cpp"
      var $13=$outbuf_addr; //@line 2021 "wavegen.cpp"
      var $14=$length_out_addr; //@line 2021 "wavegen.cpp"
      var $call16=_sonicReadShortFromStream($12, $13, $14); //@line 2021 "wavegen.cpp"
      $retval=$call16; //@line 2021 "wavegen.cpp"
      ; //@line 2021 "wavegen.cpp"
    }
  
    var $15=$retval; //@line 2022 "wavegen.cpp"
    ;
    return $15; //@line 2022 "wavegen.cpp"
    return null;
  }
  

  function __ZL9resonatorP9RESONATORd($r, $input) {
    ;
    var __label__;
  
    var $r_addr;
    var $input_addr;
    var $x;
    $r_addr=$r;
    $input_addr=$input;
    var $0=$r_addr; //@line 1057 "wavegen.cpp"
    var $a=(($0)&4294967295); //@line 1057 "wavegen.cpp"
    var $1=HEAPF32[(($a)>>2)]; //@line 1057 "wavegen.cpp"
    var $2=$input_addr; //@line 1057 "wavegen.cpp"
    var $mul=($1)*($2); //@line 1057 "wavegen.cpp"
    var $3=$r_addr; //@line 1057 "wavegen.cpp"
    var $b=(($3+8)&4294967295); //@line 1057 "wavegen.cpp"
    var $4=HEAPF32[(($b)>>2)]; //@line 1057 "wavegen.cpp"
    var $5=$r_addr; //@line 1057 "wavegen.cpp"
    var $x1=(($5+24)&4294967295); //@line 1057 "wavegen.cpp"
    var $6=HEAPF32[(($x1)>>2)]; //@line 1057 "wavegen.cpp"
    var $mul1=($4)*($6); //@line 1057 "wavegen.cpp"
    var $add=($mul)+($mul1); //@line 1057 "wavegen.cpp"
    var $7=$r_addr; //@line 1057 "wavegen.cpp"
    var $c=(($7+16)&4294967295); //@line 1057 "wavegen.cpp"
    var $8=HEAPF32[(($c)>>2)]; //@line 1057 "wavegen.cpp"
    var $9=$r_addr; //@line 1057 "wavegen.cpp"
    var $x2=(($9+32)&4294967295); //@line 1057 "wavegen.cpp"
    var $10=HEAPF32[(($x2)>>2)]; //@line 1057 "wavegen.cpp"
    var $mul2=($8)*($10); //@line 1057 "wavegen.cpp"
    var $add3=($add)+($mul2); //@line 1057 "wavegen.cpp"
    $x=$add3; //@line 1057 "wavegen.cpp"
    var $11=$r_addr; //@line 1058 "wavegen.cpp"
    var $x14=(($11+24)&4294967295); //@line 1058 "wavegen.cpp"
    var $12=HEAPF32[(($x14)>>2)]; //@line 1058 "wavegen.cpp"
    var $13=$r_addr; //@line 1058 "wavegen.cpp"
    var $x25=(($13+32)&4294967295); //@line 1058 "wavegen.cpp"
    HEAPF32[(($x25)>>2)]=$12; //@line 1058 "wavegen.cpp"
    var $14=$x; //@line 1059 "wavegen.cpp"
    var $15=$r_addr; //@line 1059 "wavegen.cpp"
    var $x16=(($15+24)&4294967295); //@line 1059 "wavegen.cpp"
    HEAPF32[(($x16)>>2)]=$14; //@line 1059 "wavegen.cpp"
    var $16=$x; //@line 1061 "wavegen.cpp"
    ;
    return $16; //@line 1061 "wavegen.cpp"
    return null;
  }
  

  function __Z15MakePhonemeListP10Translatorii($tr, $post_pause, $start_sentence) {
    var __stackBase__  = STACKTOP; STACKTOP += 24152; _memset(__stackBase__, 0, 24152);
    var __label__;
    var __lastLabel__ = null;
  
    var $tr_addr;
    var $post_pause_addr;
    var $start_sentence_addr;
    var $ix;
    var $j;
    var $insert_ph;
    var $phlist;
    var $ph;
    var $prev;
    var $next;
    var $next2;
    var $unstress_count;
    var $word_stress;
    var $switched_language;
    var $max_stress;
    var $voicing;
    var $regression;
    var $end_sourceix;
    var $alternative;
    var $first_vowel;
    var $phdata=__stackBase__;
    var $n_ph_list3;
    var $plist3;
    var $plist3_inserted;
    var $ph_list3=__stackBase__+152;
    var $plist2;
    var $type;
    var $word_end_devoice;
    var $k;
    var $nextw;
    var $ph2;
    var $ph2266;
    var $p;
    var $x;
    $tr_addr=$tr;
    $post_pause_addr=$post_pause;
    $start_sentence_addr=$start_sentence;
    $ix=0; //@line 112 "phonemelist.cpp"
    $insert_ph=0; //@line 114 "phonemelist.cpp"
    $unstress_count=0; //@line 118 "phonemelist.cpp"
    $word_stress=0; //@line 119 "phonemelist.cpp"
    $switched_language=0; //@line 120 "phonemelist.cpp"
    $first_vowel=0; //@line 126 "phonemelist.cpp"
    $plist3_inserted=0; //@line 131 "phonemelist.cpp"
    $plist2=((_ph_list2)&4294967295); //@line 137 "phonemelist.cpp"
    $phlist=((_phoneme_list)&4294967295); //@line 138 "phonemelist.cpp"
    var $0=HEAP32[((_n_ph_list2)>>2)]; //@line 139 "phonemelist.cpp"
    var $sub=((($0)-1)&4294967295); //@line 139 "phonemelist.cpp"
    var $1=$plist2; //@line 139 "phonemelist.cpp"
    var $arrayidx=(($1+8*$sub)&4294967295); //@line 139 "phonemelist.cpp"
    var $sourceix=(($arrayidx+6)&4294967295); //@line 139 "phonemelist.cpp"
    var $2=HEAPU16[(($sourceix)>>1)]; //@line 139 "phonemelist.cpp"
    var $conv=((($2))&65535); //@line 139 "phonemelist.cpp"
    $end_sourceix=$conv; //@line 139 "phonemelist.cpp"
    $max_stress=0; //@line 142 "phonemelist.cpp"
    var $3=HEAP32[((_n_ph_list2)>>2)]; //@line 143 "phonemelist.cpp"
    var $sub1=((($3)-3)&4294967295); //@line 143 "phonemelist.cpp"
    $j=$sub1; //@line 143 "phonemelist.cpp"
    __lastLabel__ = 0; ; //@line 143 "phonemelist.cpp"
    $for_cond$2: while(1) { 
  
      var $4=__lastLabel__ == 5 ? $dec : ($sub1); //@line 143 "phonemelist.cpp"
      var $cmp=((($4))|0) >= 0; //@line 143 "phonemelist.cpp"
      if (!($cmp)) { __label__ = 6;break $for_cond$2; } //@line 143 "phonemelist.cpp"
  
      var $5=$j; //@line 146 "phonemelist.cpp"
      var $6=$plist2; //@line 146 "phonemelist.cpp"
      var $arrayidx2=(($6+8*$5)&4294967295); //@line 146 "phonemelist.cpp"
      var $stresslevel=(($arrayidx2+1)&4294967295); //@line 146 "phonemelist.cpp"
      var $7=HEAPU8[($stresslevel)]; //@line 146 "phonemelist.cpp"
      var $conv3=((($7))&255); //@line 146 "phonemelist.cpp"
      var $and=($conv3) & 127; //@line 146 "phonemelist.cpp"
      var $8=$max_stress; //@line 146 "phonemelist.cpp"
      var $cmp4=((($and))|0) > ((($8))|0); //@line 146 "phonemelist.cpp"
      if ($cmp4) { __label__ = 3;; } else { __label__ = 4;; } //@line 146 "phonemelist.cpp"
      if (__label__ == 3) {
  
        var $9=$j; //@line 147 "phonemelist.cpp"
        var $10=$plist2; //@line 147 "phonemelist.cpp"
        var $arrayidx5=(($10+8*$9)&4294967295); //@line 147 "phonemelist.cpp"
        var $stresslevel6=(($arrayidx5+1)&4294967295); //@line 147 "phonemelist.cpp"
        var $11=HEAPU8[($stresslevel6)]; //@line 147 "phonemelist.cpp"
        var $conv7=((($11))&255); //@line 147 "phonemelist.cpp"
        var $and8=($conv7) & 127; //@line 147 "phonemelist.cpp"
        $max_stress=$and8; //@line 147 "phonemelist.cpp"
        ; //@line 147 "phonemelist.cpp"
      }
  
      var $12=$j; //@line 148 "phonemelist.cpp"
      var $13=$plist2; //@line 148 "phonemelist.cpp"
      var $arrayidx9=(($13+8*$12)&4294967295); //@line 148 "phonemelist.cpp"
      var $sourceix10=(($arrayidx9+6)&4294967295); //@line 148 "phonemelist.cpp"
      var $14=HEAPU16[(($sourceix10)>>1)]; //@line 148 "phonemelist.cpp"
      var $conv11=((($14))&65535); //@line 148 "phonemelist.cpp"
      var $cmp12=((($conv11))|0)!=0; //@line 148 "phonemelist.cpp"
      if ($cmp12) { __label__ = 6;break $for_cond$2; } //@line 148 "phonemelist.cpp"
  
      var $15=$j; //@line 143 "phonemelist.cpp"
      var $dec=((($15)-1)&4294967295); //@line 143 "phonemelist.cpp"
      $j=$dec; //@line 143 "phonemelist.cpp"
      __lastLabel__ = 5; __label__ = 1;continue $for_cond$2; //@line 143 "phonemelist.cpp"
    }
  
    var $16=$max_stress; //@line 151 "phonemelist.cpp"
    var $cmp15=((($16))|0) < 4; //@line 151 "phonemelist.cpp"
    if ($cmp15) { __label__ = 7;; } else { __label__ = 11;; } //@line 151 "phonemelist.cpp"
    $while_cond$$if_end32$10: do { 
      if (__label__ == 7) {
        while(1) { 
  
          var $17=$j; //@line 154 "phonemelist.cpp"
          var $dec17=((($17)-1)&4294967295); //@line 154 "phonemelist.cpp"
          $j=$dec17; //@line 154 "phonemelist.cpp"
          var $cmp18=((($dec17))|0) >= 0; //@line 154 "phonemelist.cpp"
          if (!($cmp18)) { __label__ = 11;break $while_cond$$if_end32$10; } //@line 154 "phonemelist.cpp"
  
          var $18=$j; //@line 156 "phonemelist.cpp"
          var $19=$plist2; //@line 156 "phonemelist.cpp"
          var $arrayidx19=(($19+8*$18)&4294967295); //@line 156 "phonemelist.cpp"
          var $synthflags=(($arrayidx19+4)&4294967295); //@line 156 "phonemelist.cpp"
          var $20=HEAPU16[(($synthflags)>>1)]; //@line 156 "phonemelist.cpp"
          var $conv20=((($20))&65535); //@line 156 "phonemelist.cpp"
          var $and21=($conv20) & 64; //@line 156 "phonemelist.cpp"
          var $tobool=((($and21))|0)!=0; //@line 156 "phonemelist.cpp"
          var $21=$j; //@line 158 "phonemelist.cpp"
          var $22=$plist2; //@line 158 "phonemelist.cpp"
          var $arrayidx23=(($22+8*$21)&4294967295); //@line 158 "phonemelist.cpp"
          var $stresslevel24=(($arrayidx23+1)&4294967295); //@line 158 "phonemelist.cpp"
          if ($tobool) { __label__ = 9;break ; } //@line 156 "phonemelist.cpp"
  
          var $23=HEAPU8[($stresslevel24)]; //@line 161 "phonemelist.cpp"
          var $conv28=((($23))&255); //@line 161 "phonemelist.cpp"
          var $cmp29=((($conv28))|0) >= 4; //@line 161 "phonemelist.cpp"
          if ($cmp29) { __label__ = 11;break $while_cond$$if_end32$10; } else { __label__ = 7;continue ; } //@line 161 "phonemelist.cpp"
        }
  
        HEAP8[($stresslevel24)]=4; //@line 158 "phonemelist.cpp"
        ; //@line 159 "phonemelist.cpp"
      }
    } while(0);
  
    var $24=$tr_addr; //@line 169 "phonemelist.cpp"
    var $langopts=(($24)&4294967295); //@line 169 "phonemelist.cpp"
    var $param=(($langopts+24)&4294967295); //@line 169 "phonemelist.cpp"
    var $arrayidx33=(($param+16)&4294967295); //@line 169 "phonemelist.cpp"
    var $25=HEAP32[(($arrayidx33)>>2)]; //@line 169 "phonemelist.cpp"
    $regression=$25; //@line 169 "phonemelist.cpp"
    var $cmp34=((($25))|0)!=0; //@line 169 "phonemelist.cpp"
    if ($cmp34) { __label__ = 12;; } else { __label__ = 51;; } //@line 169 "phonemelist.cpp"
    $if_then35$$if_end143$17: do { 
      if (__label__ == 12) {
  
        $word_end_devoice=0; //@line 174 "phonemelist.cpp"
        $voicing=0; //@line 175 "phonemelist.cpp"
        var $26=HEAP32[((_n_ph_list2)>>2)]; //@line 177 "phonemelist.cpp"
        var $sub36=((($26)-1)&4294967295); //@line 177 "phonemelist.cpp"
        $j=$sub36; //@line 177 "phonemelist.cpp"
        var $cmp389=((($sub36))|0) >= 0; //@line 177 "phonemelist.cpp"
        if (!($cmp389)) { __label__ = 51;break $if_then35$$if_end143$17; } //@line 177 "phonemelist.cpp"
        $for_body39$19: while(1) { 
  
          var $27=$j; //@line 179 "phonemelist.cpp"
          var $28=$plist2; //@line 179 "phonemelist.cpp"
          var $arrayidx40=(($28+8*$27)&4294967295); //@line 179 "phonemelist.cpp"
          var $phcode=(($arrayidx40)&4294967295); //@line 179 "phonemelist.cpp"
          var $29=HEAPU8[($phcode)]; //@line 179 "phonemelist.cpp"
          var $idxprom=((($29))&255); //@line 179 "phonemelist.cpp"
          var $arrayidx41=((_phoneme_tab+$idxprom*4)&4294967295); //@line 179 "phonemelist.cpp"
          var $30=HEAPU32[(($arrayidx41)>>2)]; //@line 179 "phonemelist.cpp"
          $ph=$30; //@line 179 "phonemelist.cpp"
          var $cmp42=((($30))|0)==0; //@line 180 "phonemelist.cpp"
          if ($cmp42) { __label__ = 50;; } else { __label__ = 14;; } //@line 180 "phonemelist.cpp"
          $for_inc140$$if_end44$21: do { 
            if (__label__ == 14) {
  
              var $31=$ph; //@line 183 "phonemelist.cpp"
              var $code=(($31+10)&4294967295); //@line 183 "phonemelist.cpp"
              var $32=HEAPU8[($code)]; //@line 183 "phonemelist.cpp"
              var $conv45=((($32))&255); //@line 183 "phonemelist.cpp"
              var $cmp46=((($conv45))|0)==21; //@line 183 "phonemelist.cpp"
              var $33=$switched_language; //@line 184 "phonemelist.cpp"
              if ($cmp46) { __lastLabel__ = 14; __label__ = 15;; } else { __lastLabel__ = 14; __label__ = 16;; } //@line 183 "phonemelist.cpp"
              if (__label__ == 15) {
  
                var $xor=($33) ^ 1; //@line 184 "phonemelist.cpp"
                $switched_language=$xor; //@line 184 "phonemelist.cpp"
                __lastLabel__ = 15; ; //@line 184 "phonemelist.cpp"
              }
  
              var $34=__lastLabel__ == 15 ? $xor : ($33); //@line 185 "phonemelist.cpp"
              var $tobool49=((($34))|0)!=0; //@line 185 "phonemelist.cpp"
              if ($tobool49) { __label__ = 50;break $for_inc140$$if_end44$21; } //@line 185 "phonemelist.cpp"
  
              var $35=$ph; //@line 188 "phonemelist.cpp"
              var $type52=(($35+11)&4294967295); //@line 188 "phonemelist.cpp"
              var $36=HEAPU8[($type52)]; //@line 188 "phonemelist.cpp"
              var $conv53=((($36))&255); //@line 188 "phonemelist.cpp"
              $type=$conv53; //@line 188 "phonemelist.cpp"
              var $37=$regression; //@line 190 "phonemelist.cpp"
              var $and54=($37) & 2; //@line 190 "phonemelist.cpp"
              var $tobool55=((($and54))|0)!=0; //@line 190 "phonemelist.cpp"
              if ($tobool55) { __label__ = 18;; } else { __label__ = 24;; } //@line 190 "phonemelist.cpp"
              $if_then56$$if_end69$27: do { 
                if (__label__ == 18) {
  
                  var $38=$ph; //@line 193 "phonemelist.cpp"
                  var $mnemonic=(($38)&4294967295); //@line 193 "phonemelist.cpp"
                  var $39=HEAP32[(($mnemonic)>>2)]; //@line 193 "phonemelist.cpp"
                  var $cmp57=((($39))|0)==118; //@line 193 "phonemelist.cpp"
                  if ($cmp57) { __label__ = 21;; } else { __label__ = 19;; } //@line 193 "phonemelist.cpp"
                  $if_then64$$lor_lhs_false$29: do { 
                    if (__label__ == 19) {
  
                      var $40=$ph; //@line 193 "phonemelist.cpp"
                      var $mnemonic58=(($40)&4294967295); //@line 193 "phonemelist.cpp"
                      var $41=HEAP32[(($mnemonic58)>>2)]; //@line 193 "phonemelist.cpp"
                      var $cmp59=((($41))|0)==15222; //@line 193 "phonemelist.cpp"
                      if ($cmp59) { __label__ = 21;break $if_then64$$lor_lhs_false$29; } //@line 193 "phonemelist.cpp"
  
                      var $42=$ph; //@line 193 "phonemelist.cpp"
                      var $mnemonic61=(($42)&4294967295); //@line 193 "phonemelist.cpp"
                      var $43=HEAP32[(($mnemonic61)>>2)]; //@line 193 "phonemelist.cpp"
                      var $and62=($43) & 255; //@line 193 "phonemelist.cpp"
                      var $cmp63=((($and62))|0)==82; //@line 193 "phonemelist.cpp"
                      if (!($cmp63)) { __label__ = 24;break $if_then56$$if_end69$27; } //@line 193 "phonemelist.cpp"
                    }
                  } while(0);
  
                  var $44=$word_end_devoice; //@line 195 "phonemelist.cpp"
                  var $cmp65=((($44))|0)==1; //@line 195 "phonemelist.cpp"
                  if ($cmp65) { __label__ = 22;; } else { __label__ = 23;; } //@line 195 "phonemelist.cpp"
                  if (__label__ == 22) {
  
                    $voicing=0; //@line 196 "phonemelist.cpp"
                    ; //@line 196 "phonemelist.cpp"
                  }
                  else if (__label__ == 23) {
  
                    $type=3; //@line 198 "phonemelist.cpp"
                    ;
                  }
                }
              } while(0);
  
              var $45=$type; //@line 202 "phonemelist.cpp"
              var $cmp70=((($45))|0)==4; //@line 202 "phonemelist.cpp"
              var $46=$type; //@line 202 "phonemelist.cpp"
              var $cmp72=((($46))|0)==6; //@line 202 "phonemelist.cpp"
              var $or_cond=($cmp70) | ($cmp72); //@line 202 "phonemelist.cpp"
              if ($or_cond) { __label__ = 25;; } else { __label__ = 32;; } //@line 202 "phonemelist.cpp"
              $if_then73$$if_else89$37: do { 
                if (__label__ == 25) {
  
                  var $47=$voicing; //@line 204 "phonemelist.cpp"
                  var $cmp74=((($47))|0)==0; //@line 204 "phonemelist.cpp"
                  if ($cmp74) { __lastLabel__ = 25; __label__ = 26;; } else { __lastLabel__ = 25; __label__ = 29;; } //@line 204 "phonemelist.cpp"
                  if (__label__ == 26) {
  
                    var $48=$regression; //@line 204 "phonemelist.cpp"
                    var $and75=($48) & 15; //@line 204 "phonemelist.cpp"
                    var $tobool76=((($and75))|0)!=0; //@line 204 "phonemelist.cpp"
                    if ($tobool76) { __label__ = 27;; } else { __label__ = 28;; } //@line 204 "phonemelist.cpp"
                    if (__label__ == 27) {
  
                      $voicing=1; //@line 206 "phonemelist.cpp"
                      __label__ = 44;break $if_then73$$if_else89$37; //@line 207 "phonemelist.cpp"
                    }
                    else if (__label__ == 28) {
  
                      var $_pr5=$voicing; //@line 209 "phonemelist.cpp"
                      __lastLabel__ = 28; ;
                    }
                  }
  
                  var $49=__lastLabel__ == 28 ? $_pr5 : ($47); //@line 209 "phonemelist.cpp"
                  var $cmp79=((($49))|0)==2; //@line 209 "phonemelist.cpp"
                  if (!($cmp79)) { __label__ = 44;break $if_then73$$if_else89$37; } //@line 209 "phonemelist.cpp"
  
                  var $50=$ph; //@line 209 "phonemelist.cpp"
                  var $end_type=(($50+13)&4294967295); //@line 209 "phonemelist.cpp"
                  var $51=HEAPU8[($end_type)]; //@line 209 "phonemelist.cpp"
                  var $conv81=((($51))&255); //@line 209 "phonemelist.cpp"
                  var $cmp82=((($conv81))|0)!=0; //@line 209 "phonemelist.cpp"
                  if (!($cmp82)) { __label__ = 44;break $if_then73$$if_else89$37; } //@line 209 "phonemelist.cpp"
  
                  var $52=$ph; //@line 211 "phonemelist.cpp"
                  var $end_type84=(($52+13)&4294967295); //@line 211 "phonemelist.cpp"
                  var $53=HEAP8[($end_type84)]; //@line 211 "phonemelist.cpp"
                  var $54=$j; //@line 211 "phonemelist.cpp"
                  var $55=$plist2; //@line 211 "phonemelist.cpp"
                  var $arrayidx85=(($55+8*$54)&4294967295); //@line 211 "phonemelist.cpp"
                  var $phcode86=(($arrayidx85)&4294967295); //@line 211 "phonemelist.cpp"
                  HEAP8[($phcode86)]=$53; //@line 211 "phonemelist.cpp"
                  ; //@line 212 "phonemelist.cpp"
                }
                else if (__label__ == 32) {
  
                  var $56=$type; //@line 215 "phonemelist.cpp"
                  var $cmp90=((($56))|0)==5; //@line 215 "phonemelist.cpp"
                  var $57=$type; //@line 215 "phonemelist.cpp"
                  var $cmp92=((($57))|0)==7; //@line 215 "phonemelist.cpp"
                  var $or_cond1=($cmp90) | ($cmp92); //@line 215 "phonemelist.cpp"
                  if ($or_cond1) { __label__ = 33;; } else { __label__ = 40;; } //@line 215 "phonemelist.cpp"
                  if (__label__ == 33) {
  
                    var $58=$voicing; //@line 217 "phonemelist.cpp"
                    var $cmp94=((($58))|0)==0; //@line 217 "phonemelist.cpp"
                    if ($cmp94) { __lastLabel__ = 33; __label__ = 34;; } else { __lastLabel__ = 33; __label__ = 37;; } //@line 217 "phonemelist.cpp"
                    if (__label__ == 34) {
  
                      var $59=$regression; //@line 217 "phonemelist.cpp"
                      var $and96=($59) & 15; //@line 217 "phonemelist.cpp"
                      var $tobool97=((($and96))|0)!=0; //@line 217 "phonemelist.cpp"
                      if ($tobool97) { __label__ = 35;; } else { __label__ = 36;; } //@line 217 "phonemelist.cpp"
                      if (__label__ == 35) {
  
                        $voicing=2; //@line 219 "phonemelist.cpp"
                        __label__ = 44;break $if_then73$$if_else89$37; //@line 220 "phonemelist.cpp"
                      }
                      else if (__label__ == 36) {
  
                        var $_pr6=$voicing; //@line 222 "phonemelist.cpp"
                        __lastLabel__ = 36; ;
                      }
                    }
  
                    var $60=__lastLabel__ == 36 ? $_pr6 : ($58); //@line 222 "phonemelist.cpp"
                    var $cmp100=((($60))|0)==1; //@line 222 "phonemelist.cpp"
                    if (!($cmp100)) { __label__ = 44;break $if_then73$$if_else89$37; } //@line 222 "phonemelist.cpp"
  
                    var $61=$ph; //@line 222 "phonemelist.cpp"
                    var $end_type102=(($61+13)&4294967295); //@line 222 "phonemelist.cpp"
                    var $62=HEAPU8[($end_type102)]; //@line 222 "phonemelist.cpp"
                    var $conv103=((($62))&255); //@line 222 "phonemelist.cpp"
                    var $cmp104=((($conv103))|0)!=0; //@line 222 "phonemelist.cpp"
                    if (!($cmp104)) { __label__ = 44;break $if_then73$$if_else89$37; } //@line 222 "phonemelist.cpp"
  
                    var $63=$ph; //@line 224 "phonemelist.cpp"
                    var $end_type106=(($63+13)&4294967295); //@line 224 "phonemelist.cpp"
                    var $64=HEAP8[($end_type106)]; //@line 224 "phonemelist.cpp"
                    var $65=$j; //@line 224 "phonemelist.cpp"
                    var $66=$plist2; //@line 224 "phonemelist.cpp"
                    var $arrayidx107=(($66+8*$65)&4294967295); //@line 224 "phonemelist.cpp"
                    var $phcode108=(($arrayidx107)&4294967295); //@line 224 "phonemelist.cpp"
                    HEAP8[($phcode108)]=$64; //@line 224 "phonemelist.cpp"
                    ; //@line 225 "phonemelist.cpp"
                  }
                  else if (__label__ == 40) {
  
                    var $67=$regression; //@line 229 "phonemelist.cpp"
                    var $and112=($67) & 8; //@line 229 "phonemelist.cpp"
                    var $tobool113=((($and112))|0)!=0; //@line 229 "phonemelist.cpp"
                    if ($tobool113) { __label__ = 41;; } else { __label__ = 43;; } //@line 229 "phonemelist.cpp"
                    if (__label__ == 41) {
  
                      var $68=$type; //@line 232 "phonemelist.cpp"
                      var $cmp115=((($68))|0)==0; //@line 232 "phonemelist.cpp"
                      var $69=$type; //@line 232 "phonemelist.cpp"
                      var $cmp117=((($69))|0)==2; //@line 232 "phonemelist.cpp"
                      var $or_cond2=($cmp115) | ($cmp117); //@line 232 "phonemelist.cpp"
                      if (!($or_cond2)) { __label__ = 44;break $if_then73$$if_else89$37; } //@line 232 "phonemelist.cpp"
  
                      $voicing=0; //@line 233 "phonemelist.cpp"
                      ; //@line 233 "phonemelist.cpp"
                    }
                    else if (__label__ == 43) {
  
                      $voicing=0; //@line 237 "phonemelist.cpp"
                      ;
                    }
                  }
                }
              } while(0);
  
              $word_end_devoice=0; //@line 241 "phonemelist.cpp"
              var $70=$j; //@line 242 "phonemelist.cpp"
              var $71=$plist2; //@line 242 "phonemelist.cpp"
              var $arrayidx124=(($71+8*$70)&4294967295); //@line 242 "phonemelist.cpp"
              var $sourceix125=(($arrayidx124+6)&4294967295); //@line 242 "phonemelist.cpp"
              var $72=HEAP16[(($sourceix125)>>1)]; //@line 242 "phonemelist.cpp"
              var $tobool126=(tempInt=(($72)),(tempInt>=32768?tempInt-65536:tempInt))!=0; //@line 242 "phonemelist.cpp"
              if (!($tobool126)) { __label__ = 50;break $for_inc140$$if_end44$21; } //@line 242 "phonemelist.cpp"
  
              var $73=$regression; //@line 244 "phonemelist.cpp"
              var $and128=($73) & 4; //@line 244 "phonemelist.cpp"
              var $tobool129=((($and128))|0)!=0; //@line 244 "phonemelist.cpp"
              if ($tobool129) { __label__ = 46;; } else { __label__ = 47;; } //@line 244 "phonemelist.cpp"
              if (__label__ == 46) {
  
                $voicing=0; //@line 247 "phonemelist.cpp"
                ; //@line 248 "phonemelist.cpp"
              }
  
              var $74=$regression; //@line 249 "phonemelist.cpp"
              var $and132=($74) & 16; //@line 249 "phonemelist.cpp"
              var $tobool133=((($and132))|0)!=0; //@line 249 "phonemelist.cpp"
              if (!($tobool133)) { __label__ = 50;break $for_inc140$$if_end44$21; } //@line 249 "phonemelist.cpp"
  
              var $75=$voicing; //@line 252 "phonemelist.cpp"
              var $cmp135=((($75))|0)==0; //@line 252 "phonemelist.cpp"
              if (!($cmp135)) { __label__ = 50;break $for_inc140$$if_end44$21; } //@line 252 "phonemelist.cpp"
  
              $voicing=1; //@line 254 "phonemelist.cpp"
              $word_end_devoice=1; //@line 255 "phonemelist.cpp"
              ; //@line 256 "phonemelist.cpp"
            }
          } while(0);
  
          var $76=$j; //@line 177 "phonemelist.cpp"
          var $dec141=((($76)-1)&4294967295); //@line 177 "phonemelist.cpp"
          $j=$dec141; //@line 177 "phonemelist.cpp"
          var $cmp38=((($dec141))|0) >= 0; //@line 177 "phonemelist.cpp"
          if ($cmp38) { __label__ = 13;continue $for_body39$19; } else { __label__ = 51;break $if_then35$$if_end143$17; } //@line 177 "phonemelist.cpp"
        }
      }
    } while(0);
  
    var $77=$tr_addr; //@line 262 "phonemelist.cpp"
    var $arraydecay=(($ph_list3)&4294967295); //@line 262 "phonemelist.cpp"
    var $call=__ZL18SubstitutePhonemesP10TranslatorP12PHONEME_LIST($77, $arraydecay); //@line 262 "phonemelist.cpp"
    var $sub144=((($call)-2)&4294967295); //@line 262 "phonemelist.cpp"
    $n_ph_list3=$sub144; //@line 262 "phonemelist.cpp"
    $j=0; //@line 264 "phonemelist.cpp"
    ; //@line 264 "phonemelist.cpp"
    $for_cond145$72: while(1) { 
  
      var $78=$j; //@line 264 "phonemelist.cpp"
      var $79=$n_ph_list3; //@line 264 "phonemelist.cpp"
      var $cmp146=((($78))|0) < ((($79))|0); //@line 264 "phonemelist.cpp"
      if (!($cmp146)) { __label__ = 64;break $for_cond145$72; } //@line 264 "phonemelist.cpp"
  
      var $80=$ix; //@line 264 "phonemelist.cpp"
      var $cmp147=((($80))|0) < 997; //@line 264 "phonemelist.cpp"
      if (!($cmp147)) { __label__ = 64;break $for_cond145$72; }
  
      var $81=$j; //@line 266 "phonemelist.cpp"
      var $arrayidx149=(($ph_list3+$81*24)&4294967295); //@line 266 "phonemelist.cpp"
      var $sourceix150=(($arrayidx149+6)&4294967295); //@line 266 "phonemelist.cpp"
      var $82=HEAP16[(($sourceix150)>>1)]; //@line 266 "phonemelist.cpp"
      var $tobool151=(tempInt=(($82)),(tempInt>=32768?tempInt-65536:tempInt))!=0; //@line 266 "phonemelist.cpp"
      if ($tobool151) { __label__ = 55;; } else { __label__ = 63;; } //@line 266 "phonemelist.cpp"
      if (__label__ == 55) {
  
        $word_stress=0; //@line 271 "phonemelist.cpp"
        var $83=$j; //@line 274 "phonemelist.cpp"
        $nextw=$83; //@line 274 "phonemelist.cpp"
        ; //@line 274 "phonemelist.cpp"
        $for_cond153$78: while(1) { 
  
          var $84=$nextw; //@line 274 "phonemelist.cpp"
          var $85=$n_ph_list3; //@line 274 "phonemelist.cpp"
          var $cmp154=((($84))|0) < ((($85))|0); //@line 274 "phonemelist.cpp"
          if (!($cmp154)) { __label__ = 60;break $for_cond153$78; } //@line 274 "phonemelist.cpp"
  
          var $86=$nextw; //@line 276 "phonemelist.cpp"
          var $arrayidx156=(($ph_list3+$86*24)&4294967295); //@line 276 "phonemelist.cpp"
          var $stresslevel157=(($arrayidx156+1)&4294967295); //@line 276 "phonemelist.cpp"
          var $87=HEAPU8[($stresslevel157)]; //@line 276 "phonemelist.cpp"
          var $conv158=((($87))&255); //@line 276 "phonemelist.cpp"
          var $88=$word_stress; //@line 276 "phonemelist.cpp"
          var $cmp159=((($conv158))|0) > ((($88))|0); //@line 276 "phonemelist.cpp"
          if ($cmp159) { __label__ = 58;; } else { __label__ = 59;; } //@line 276 "phonemelist.cpp"
          if (__label__ == 58) {
  
            var $89=$nextw; //@line 277 "phonemelist.cpp"
            var $arrayidx161=(($ph_list3+$89*24)&4294967295); //@line 277 "phonemelist.cpp"
            var $stresslevel162=(($arrayidx161+1)&4294967295); //@line 277 "phonemelist.cpp"
            var $90=HEAPU8[($stresslevel162)]; //@line 277 "phonemelist.cpp"
            var $conv163=((($90))&255); //@line 277 "phonemelist.cpp"
            $word_stress=$conv163; //@line 277 "phonemelist.cpp"
            ; //@line 277 "phonemelist.cpp"
          }
  
          var $91=$nextw; //@line 279 "phonemelist.cpp"
          var $inc=((($91)+1)&4294967295); //@line 279 "phonemelist.cpp"
          $nextw=$inc; //@line 279 "phonemelist.cpp"
          var $92=$nextw; //@line 280 "phonemelist.cpp"
          var $arrayidx165=(($ph_list3+$92*24)&4294967295); //@line 280 "phonemelist.cpp"
          var $sourceix166=(($arrayidx165+6)&4294967295); //@line 280 "phonemelist.cpp"
          var $93=HEAP16[(($sourceix166)>>1)]; //@line 280 "phonemelist.cpp"
          var $tobool167=(tempInt=(($93)),(tempInt>=32768?tempInt-65536:tempInt))!=0; //@line 280 "phonemelist.cpp"
          if ($tobool167) { __label__ = 60;break $for_cond153$78; } else { __label__ = 56;continue $for_cond153$78; } //@line 280 "phonemelist.cpp"
        }
  
        var $94=$j; //@line 283 "phonemelist.cpp"
        $k=$94; //@line 283 "phonemelist.cpp"
        var $95=$k; //@line 283 "phonemelist.cpp"
        var $96=$nextw; //@line 283 "phonemelist.cpp"
        var $cmp1728=((($95))|0) < ((($96))|0); //@line 283 "phonemelist.cpp"
        if ($cmp1728) { __label__ = 61;; } else { __label__ = 62;; } //@line 283 "phonemelist.cpp"
        $for_body173$$for_end178$85: do { 
          if (__label__ == 61) {
            while(1) { 
  
              var $97=$word_stress; //@line 285 "phonemelist.cpp"
              var $conv174=((($97)) & 255); //@line 285 "phonemelist.cpp"
              var $98=$k; //@line 285 "phonemelist.cpp"
              var $arrayidx175=(($ph_list3+$98*24)&4294967295); //@line 285 "phonemelist.cpp"
              var $wordstress=(($arrayidx175+2)&4294967295); //@line 285 "phonemelist.cpp"
              HEAP8[($wordstress)]=$conv174; //@line 285 "phonemelist.cpp"
              var $99=$k; //@line 283 "phonemelist.cpp"
              var $inc177=((($99)+1)&4294967295); //@line 283 "phonemelist.cpp"
              $k=$inc177; //@line 283 "phonemelist.cpp"
              var $100=$k; //@line 283 "phonemelist.cpp"
              var $101=$nextw; //@line 283 "phonemelist.cpp"
              var $cmp172=((($100))|0) < ((($101))|0); //@line 283 "phonemelist.cpp"
              if ($cmp172) { __label__ = 61;continue ; } else { __label__ = 62;break $for_body173$$for_end178$85; } //@line 283 "phonemelist.cpp"
            }
          }
        } while(0);
  
        var $102=$nextw; //@line 287 "phonemelist.cpp"
        $j=$102; //@line 287 "phonemelist.cpp"
        __label__ = 52;continue $for_cond145$72; //@line 288 "phonemelist.cpp"
      }
      else if (__label__ == 63) {
  
        var $103=$j; //@line 291 "phonemelist.cpp"
        var $inc180=((($103)+1)&4294967295); //@line 291 "phonemelist.cpp"
        $j=$inc180; //@line 291 "phonemelist.cpp"
        __label__ = 52;continue $for_cond145$72;
      }
    }
  
    var $104=HEAP32[((((_phoneme_tab+36)&4294967295))>>2)]; //@line 296 "phonemelist.cpp"
    $ph=$104; //@line 296 "phonemelist.cpp"
    var $105=$ph; //@line 297 "phonemelist.cpp"
    var $arrayidx183=(($ph_list3)&4294967295); //@line 297 "phonemelist.cpp"
    var $ph184=(($arrayidx183+8)&4294967295); //@line 297 "phonemelist.cpp"
    HEAP32[(($ph184)>>2)]=$105; //@line 297 "phonemelist.cpp"
    $switched_language=0; //@line 299 "phonemelist.cpp"
    $j=0; //@line 301 "phonemelist.cpp"
    var $pd_param=(($phdata+4)&4294967295); //@line 346 "phonemelist.cpp"
    var $arrayidx233=(($pd_param+12)&4294967295); //@line 346 "phonemelist.cpp"
    var $pd_param262=(($phdata+4)&4294967295); //@line 371 "phonemelist.cpp"
    var $arrayidx263=(($pd_param262+4)&4294967295); //@line 371 "phonemelist.cpp"
    var $pd_param480=(($phdata+4)&4294967295); //@line 568 "phonemelist.cpp"
    var $arrayidx481=(($pd_param480+16)&4294967295); //@line 568 "phonemelist.cpp"
    var $pd_param484=(($phdata+4)&4294967295); //@line 570 "phonemelist.cpp"
    var $arrayidx485=(($pd_param484+16)&4294967295); //@line 570 "phonemelist.cpp"
    var $pd_param537=(($phdata+4)&4294967295); //@line 608 "phonemelist.cpp"
    var $arrayidx538=(($pd_param537+40)&4294967295); //@line 608 "phonemelist.cpp"
    ; //@line 301 "phonemelist.cpp"
    $for_cond185$91: while(1) { 
  
      var $106=$insert_ph; //@line 301 "phonemelist.cpp"
      var $tobool186=((($106))|0)!=0; //@line 301 "phonemelist.cpp"
      if ($tobool186) { __lastLabel__ = 65; __label__ = 69;; } else { __lastLabel__ = 65; __label__ = 66;; } //@line 301 "phonemelist.cpp"
      if (__label__ == 66) {
  
        var $107=$j; //@line 301 "phonemelist.cpp"
        var $108=$n_ph_list3; //@line 301 "phonemelist.cpp"
        var $cmp187=((($107))|0) < ((($108))|0); //@line 301 "phonemelist.cpp"
        if (!($cmp187)) { __label__ = 158;break $for_cond185$91; } //@line 301 "phonemelist.cpp"
  
        var $109=$ix; //@line 301 "phonemelist.cpp"
        var $cmp189=((($109))|0) < 997; //@line 301 "phonemelist.cpp"
        if (!($cmp189)) { __label__ = 158;break $for_cond185$91; }
  
        var $_pr7=$insert_ph; //@line 307 "phonemelist.cpp"
        __lastLabel__ = 68; ;
      }
  
      var $110=__lastLabel__ == 68 ? $_pr7 : ($106); //@line 307 "phonemelist.cpp"
      var $111=$ph; //@line 303 "phonemelist.cpp"
      $prev=$111; //@line 303 "phonemelist.cpp"
      var $112=$j; //@line 305 "phonemelist.cpp"
      var $arrayidx192=(($ph_list3+$112*24)&4294967295); //@line 305 "phonemelist.cpp"
      $plist3=$arrayidx192; //@line 305 "phonemelist.cpp"
      var $cmp193=((($110))|0)!=0; //@line 307 "phonemelist.cpp"
      var $113=$plist3; //@line 310 "phonemelist.cpp"
      var $phcode195=(($113)&4294967295); //@line 310 "phonemelist.cpp"
      var $114=HEAPU8[($phcode195)]; //@line 310 "phonemelist.cpp"
      var $idxprom196=((($114))&255); //@line 310 "phonemelist.cpp"
      var $arrayidx197=((_phoneme_tab+$idxprom196*4)&4294967295); //@line 310 "phonemelist.cpp"
      var $115=HEAP32[(($arrayidx197)>>2)]; //@line 310 "phonemelist.cpp"
      if ($cmp193) { __label__ = 70;; } else { __label__ = 73;; } //@line 307 "phonemelist.cpp"
      if (__label__ == 70) {
  
        $next=$115; //@line 310 "phonemelist.cpp"
        var $116=$j; //@line 314 "phonemelist.cpp"
        var $dec198=((($116)-1)&4294967295); //@line 314 "phonemelist.cpp"
        $j=$dec198; //@line 314 "phonemelist.cpp"
        var $117=$j; //@line 315 "phonemelist.cpp"
        var $arrayidx199=(($ph_list3+$117*24)&4294967295); //@line 315 "phonemelist.cpp"
        $plist3_inserted=$arrayidx199; //@line 315 "phonemelist.cpp"
        $plist3=$arrayidx199; //@line 315 "phonemelist.cpp"
        var $118=$j; //@line 316 "phonemelist.cpp"
        var $cmp200=((($118))|0) > 0; //@line 316 "phonemelist.cpp"
        if ($cmp200) { __label__ = 71;; } else { __label__ = 72;; } //@line 316 "phonemelist.cpp"
        if (__label__ == 71) {
  
          var $119=$plist3; //@line 318 "phonemelist.cpp"
          var $arrayidx202=(($119+-24)&4294967295); //@line 318 "phonemelist.cpp"
          var $120=$arrayidx202; //@line 318 "phonemelist.cpp"
          var $121=$plist3; //@line 318 "phonemelist.cpp"
          var $arrayidx203=(($121)&4294967295); //@line 318 "phonemelist.cpp"
          var $122=$arrayidx203; //@line 318 "phonemelist.cpp"
          var $src$, $dest$, $stop$, $stop4$, $fast$;
          $src$ = $122;
          $dest$ = $120;
          $stop$ = $src$ + 24;
          if (($dest$%4) == ($src$%4) && 24 > 8) {
            while ($src$%4 !== 0 && $src$ < $stop$) {
              ; HEAP8[$dest$++] = HEAP8[$src$++];
            }
            $src$ >>= 2;
            $dest$ >>= 2;
            $stop4$ = $stop$ >> 2;
            while ($src$ < $stop4$) {
          ;;;    HEAP32[$dest$++] = HEAP32[$src$++];
            }
            $src$ <<= 2;
            $dest$ <<= 2;
          }while ($src$ < $stop$) {
            ; HEAP8[$dest$++] = HEAP8[$src$++];
          }; //@line 318 "phonemelist.cpp"
          ; //@line 319 "phonemelist.cpp"
        }
  
        var $123=$plist3; //@line 320 "phonemelist.cpp"
        var $arrayidx205=(($123)&4294967295); //@line 320 "phonemelist.cpp"
        var $124=$arrayidx205; //@line 320 "phonemelist.cpp"
        var $dest$, $stop$, $stop4$, $fast$, $value4$;
        $dest$ = $124;
        $stop$ = $dest$ + 24;
        $value4$ = 0;
        if ($value4$ < 0) $value4$ += 256;
        $value4$ = $value4$ + ($value4$<<8) + ($value4$<<16) + ($value4$*16777216);
        while ($dest$%4 !== 0 && $dest$ < $stop$) {
          ;; HEAP8[$dest$++] = 0;
        }
        $dest$ >>= 2;
        $stop4$ = $stop$ >> 2;
        while ($dest$ < $stop4$) {
        ;;;;;;;  HEAP32[$dest$++] = $value4$;
        }
        $dest$ <<= 2;
        while ($dest$ < $stop$) {
          ;; HEAP8[$dest$++] = 0;
        }; //@line 320 "phonemelist.cpp"
        var $125=$insert_ph; //@line 321 "phonemelist.cpp"
        var $conv206=((($125)) & 255); //@line 321 "phonemelist.cpp"
        var $126=$plist3; //@line 321 "phonemelist.cpp"
        var $phcode207=(($126)&4294967295); //@line 321 "phonemelist.cpp"
        HEAP8[($phcode207)]=$conv206; //@line 321 "phonemelist.cpp"
        var $127=$insert_ph; //@line 322 "phonemelist.cpp"
        var $arrayidx208=((_phoneme_tab+$127*4)&4294967295); //@line 322 "phonemelist.cpp"
        var $128=HEAP32[(($arrayidx208)>>2)]; //@line 322 "phonemelist.cpp"
        $ph=$128; //@line 322 "phonemelist.cpp"
        var $129=$ph; //@line 323 "phonemelist.cpp"
        var $130=$plist3; //@line 323 "phonemelist.cpp"
        var $ph209=(($130+8)&4294967295); //@line 323 "phonemelist.cpp"
        HEAP32[(($ph209)>>2)]=$129; //@line 323 "phonemelist.cpp"
        $insert_ph=0; //@line 324 "phonemelist.cpp"
        ; //@line 325 "phonemelist.cpp"
      }
      else if (__label__ == 73) {
  
        $ph=$115; //@line 329 "phonemelist.cpp"
        var $131=$ph; //@line 330 "phonemelist.cpp"
        var $132=$plist3; //@line 330 "phonemelist.cpp"
        var $arrayidx214=(($132)&4294967295); //@line 330 "phonemelist.cpp"
        var $ph215=(($arrayidx214+8)&4294967295); //@line 330 "phonemelist.cpp"
        HEAP32[(($ph215)>>2)]=$131; //@line 330 "phonemelist.cpp"
        var $133=$plist3; //@line 332 "phonemelist.cpp"
        var $phcode216=(($133)&4294967295); //@line 332 "phonemelist.cpp"
        var $134=HEAPU8[($phcode216)]; //@line 332 "phonemelist.cpp"
        var $conv217=((($134))&255); //@line 332 "phonemelist.cpp"
        var $cmp218=((($conv217))|0)==21; //@line 332 "phonemelist.cpp"
        if ($cmp218) { __label__ = 74;; } else { __label__ = 75;; } //@line 332 "phonemelist.cpp"
        if (__label__ == 74) {
  
          var $135=$plist3; //@line 335 "phonemelist.cpp"
          var $tone_ph=(($135+3)&4294967295); //@line 335 "phonemelist.cpp"
          var $136=HEAPU8[($tone_ph)]; //@line 335 "phonemelist.cpp"
          var $conv220=((($136))&255); //@line 335 "phonemelist.cpp"
          __Z18SelectPhonemeTablei($conv220); //@line 335 "phonemelist.cpp"
          var $137=$switched_language; //@line 336 "phonemelist.cpp"
          var $xor221=($137) ^ 32; //@line 336 "phonemelist.cpp"
          $switched_language=$xor221; //@line 336 "phonemelist.cpp"
          ; //@line 337 "phonemelist.cpp"
        }
  
        var $138=$plist3; //@line 338 "phonemelist.cpp"
        var $arrayidx223=(($138+24)&4294967295); //@line 338 "phonemelist.cpp"
        var $phcode224=(($arrayidx223)&4294967295); //@line 338 "phonemelist.cpp"
        var $139=HEAPU8[($phcode224)]; //@line 338 "phonemelist.cpp"
        var $idxprom225=((($139))&255); //@line 338 "phonemelist.cpp"
        var $arrayidx226=((_phoneme_tab+$idxprom225*4)&4294967295); //@line 338 "phonemelist.cpp"
        var $140=HEAP32[(($arrayidx226)>>2)]; //@line 338 "phonemelist.cpp"
        $next=$140; //@line 338 "phonemelist.cpp"
        var $141=$next; //@line 339 "phonemelist.cpp"
        var $142=$plist3; //@line 339 "phonemelist.cpp"
        var $arrayidx227=(($142+24)&4294967295); //@line 339 "phonemelist.cpp"
        var $ph228=(($arrayidx227+8)&4294967295); //@line 339 "phonemelist.cpp"
        HEAP32[(($ph228)>>2)]=$141; //@line 339 "phonemelist.cpp"
        ;
      }
  
      var $143=$ph; //@line 342 "phonemelist.cpp"
      var $cmp230=((($143))|0)==0; //@line 342 "phonemelist.cpp"
      if ($cmp230) { __label__ = 157;; } else { __label__ = 77;; } //@line 342 "phonemelist.cpp"
      $for_inc584$$if_end232$108: do { 
        if (__label__ == 77) {
  
          var $144=$tr_addr; //@line 344 "phonemelist.cpp"
          var $145=$plist3; //@line 344 "phonemelist.cpp"
          __Z16InterpretPhonemeP10TranslatoriP12PHONEME_LISTP12PHONEME_DATA($144, 256, $145, $phdata); //@line 344 "phonemelist.cpp"
          var $146=HEAP32[(($arrayidx233)>>2)]; //@line 346 "phonemelist.cpp"
          $alternative=$146; //@line 346 "phonemelist.cpp"
          var $cmp234=((($146))|0) > 0; //@line 346 "phonemelist.cpp"
          if ($cmp234) { __label__ = 78;; } else { __label__ = 83;; } //@line 346 "phonemelist.cpp"
          if (__label__ == 78) {
  
            var $147=$ph; //@line 350 "phonemelist.cpp"
            $ph2=$147; //@line 350 "phonemelist.cpp"
            var $148=$plist3; //@line 352 "phonemelist.cpp"
            var $phcode236=(($148)&4294967295); //@line 352 "phonemelist.cpp"
            var $149=HEAPU8[($phcode236)]; //@line 352 "phonemelist.cpp"
            var $conv237=((($149))&255); //@line 352 "phonemelist.cpp"
            $insert_ph=$conv237; //@line 352 "phonemelist.cpp"
            var $150=$alternative; //@line 353 "phonemelist.cpp"
            var $arrayidx238=((_phoneme_tab+$150*4)&4294967295); //@line 353 "phonemelist.cpp"
            var $151=HEAP32[(($arrayidx238)>>2)]; //@line 353 "phonemelist.cpp"
            $ph=$151; //@line 353 "phonemelist.cpp"
            var $152=$ph; //@line 354 "phonemelist.cpp"
            var $153=$plist3; //@line 354 "phonemelist.cpp"
            var $ph239=(($153+8)&4294967295); //@line 354 "phonemelist.cpp"
            HEAP32[(($ph239)>>2)]=$152; //@line 354 "phonemelist.cpp"
            var $154=$alternative; //@line 355 "phonemelist.cpp"
            var $conv240=((($154)) & 255); //@line 355 "phonemelist.cpp"
            var $155=$plist3; //@line 355 "phonemelist.cpp"
            var $phcode241=(($155)&4294967295); //@line 355 "phonemelist.cpp"
            HEAP8[($phcode241)]=$conv240; //@line 355 "phonemelist.cpp"
            var $156=$ph; //@line 357 "phonemelist.cpp"
            var $type242=(($156+11)&4294967295); //@line 357 "phonemelist.cpp"
            var $157=HEAPU8[($type242)]; //@line 357 "phonemelist.cpp"
            var $conv243=((($157))&255); //@line 357 "phonemelist.cpp"
            var $cmp244=((($conv243))|0)==2; //@line 357 "phonemelist.cpp"
            var $158=$plist3; //@line 359 "phonemelist.cpp"
            var $synthflags246=(($158+4)&4294967295); //@line 359 "phonemelist.cpp"
            var $159=HEAPU16[(($synthflags246)>>1)]; //@line 359 "phonemelist.cpp"
            var $conv247=((($159))&65535); //@line 359 "phonemelist.cpp"
            if ($cmp244) { __label__ = 79;; } else { __label__ = 81;; } //@line 357 "phonemelist.cpp"
            $if_then245$$if_else255$112: do { 
              if (__label__ == 79) {
  
                var $or=($conv247) | 4; //@line 359 "phonemelist.cpp"
                var $conv248=((($or)) & 65535); //@line 359 "phonemelist.cpp"
                HEAP16[(($synthflags246)>>1)]=$conv248; //@line 359 "phonemelist.cpp"
                var $160=$ph2; //@line 360 "phonemelist.cpp"
                var $type249=(($160+11)&4294967295); //@line 360 "phonemelist.cpp"
                var $161=HEAPU8[($type249)]; //@line 360 "phonemelist.cpp"
                var $conv250=((($161))&255); //@line 360 "phonemelist.cpp"
                var $cmp251=((($conv250))|0)!=2; //@line 360 "phonemelist.cpp"
                if (!($cmp251)) { __label__ = 82;break $if_then245$$if_else255$112; } //@line 360 "phonemelist.cpp"
  
                var $162=$plist3; //@line 361 "phonemelist.cpp"
                var $stresslevel253=(($162+1)&4294967295); //@line 361 "phonemelist.cpp"
                HEAP8[($stresslevel253)]=0; //@line 361 "phonemelist.cpp"
                ; //@line 361 "phonemelist.cpp"
              }
              else if (__label__ == 81) {
  
                var $and258=($conv247) & -5; //@line 364 "phonemelist.cpp"
                var $conv259=((($and258)) & 65535); //@line 364 "phonemelist.cpp"
                HEAP16[(($synthflags246)>>1)]=$conv259; //@line 364 "phonemelist.cpp"
                ;
              }
            } while(0);
  
            var $163=$tr_addr; //@line 368 "phonemelist.cpp"
            var $164=$plist3; //@line 368 "phonemelist.cpp"
            __Z16InterpretPhonemeP10TranslatoriP12PHONEME_LISTP12PHONEME_DATA($163, 256, $164, $phdata); //@line 368 "phonemelist.cpp"
            ; //@line 369 "phonemelist.cpp"
          }
  
          var $165=HEAP32[(($arrayidx263)>>2)]; //@line 371 "phonemelist.cpp"
          $alternative=$165; //@line 371 "phonemelist.cpp"
          var $cmp264=((($165))|0) > 0; //@line 371 "phonemelist.cpp"
          if ($cmp264) { __label__ = 84;; } else { __label__ = 90;; } //@line 371 "phonemelist.cpp"
          if (__label__ == 84) {
  
            var $166=$ph; //@line 374 "phonemelist.cpp"
            $ph2266=$166; //@line 374 "phonemelist.cpp"
            var $167=$alternative; //@line 375 "phonemelist.cpp"
            var $arrayidx267=((_phoneme_tab+$167*4)&4294967295); //@line 375 "phonemelist.cpp"
            var $168=HEAP32[(($arrayidx267)>>2)]; //@line 375 "phonemelist.cpp"
            $ph=$168; //@line 375 "phonemelist.cpp"
            var $169=$ph; //@line 376 "phonemelist.cpp"
            var $170=$plist3; //@line 376 "phonemelist.cpp"
            var $ph268=(($170+8)&4294967295); //@line 376 "phonemelist.cpp"
            HEAP32[(($ph268)>>2)]=$169; //@line 376 "phonemelist.cpp"
            var $171=$alternative; //@line 377 "phonemelist.cpp"
            var $conv269=((($171)) & 255); //@line 377 "phonemelist.cpp"
            var $172=$plist3; //@line 377 "phonemelist.cpp"
            var $phcode270=(($172)&4294967295); //@line 377 "phonemelist.cpp"
            HEAP8[($phcode270)]=$conv269; //@line 377 "phonemelist.cpp"
            var $173=$alternative; //@line 379 "phonemelist.cpp"
            var $cmp271=((($173))|0)==1; //@line 379 "phonemelist.cpp"
            if ($cmp271) { __label__ = 157;break $for_inc584$$if_end232$108; } //@line 379 "phonemelist.cpp"
  
            var $174=$ph; //@line 382 "phonemelist.cpp"
            var $type274=(($174+11)&4294967295); //@line 382 "phonemelist.cpp"
            var $175=HEAPU8[($type274)]; //@line 382 "phonemelist.cpp"
            var $conv275=((($175))&255); //@line 382 "phonemelist.cpp"
            var $cmp276=((($conv275))|0)==2; //@line 382 "phonemelist.cpp"
            var $176=$plist3; //@line 384 "phonemelist.cpp"
            var $synthflags278=(($176+4)&4294967295); //@line 384 "phonemelist.cpp"
            var $177=HEAPU16[(($synthflags278)>>1)]; //@line 384 "phonemelist.cpp"
            var $conv279=((($177))&65535); //@line 384 "phonemelist.cpp"
            if ($cmp276) { __label__ = 86;; } else { __label__ = 88;; } //@line 382 "phonemelist.cpp"
            $if_then277$$if_else288$121: do { 
              if (__label__ == 86) {
  
                var $or280=($conv279) | 4; //@line 384 "phonemelist.cpp"
                var $conv281=((($or280)) & 65535); //@line 384 "phonemelist.cpp"
                HEAP16[(($synthflags278)>>1)]=$conv281; //@line 384 "phonemelist.cpp"
                var $178=$ph2266; //@line 385 "phonemelist.cpp"
                var $type282=(($178+11)&4294967295); //@line 385 "phonemelist.cpp"
                var $179=HEAPU8[($type282)]; //@line 385 "phonemelist.cpp"
                var $conv283=((($179))&255); //@line 385 "phonemelist.cpp"
                var $cmp284=((($conv283))|0)!=2; //@line 385 "phonemelist.cpp"
                if (!($cmp284)) { __label__ = 89;break $if_then277$$if_else288$121; } //@line 385 "phonemelist.cpp"
  
                var $180=$plist3; //@line 386 "phonemelist.cpp"
                var $stresslevel286=(($180+1)&4294967295); //@line 386 "phonemelist.cpp"
                HEAP8[($stresslevel286)]=0; //@line 386 "phonemelist.cpp"
                ; //@line 386 "phonemelist.cpp"
              }
              else if (__label__ == 88) {
  
                var $and291=($conv279) & -5; //@line 389 "phonemelist.cpp"
                var $conv292=((($and291)) & 65535); //@line 389 "phonemelist.cpp"
                HEAP16[(($synthflags278)>>1)]=$conv292; //@line 389 "phonemelist.cpp"
                ;
              }
            } while(0);
  
            var $181=$tr_addr; //@line 393 "phonemelist.cpp"
            var $182=$plist3; //@line 393 "phonemelist.cpp"
            __Z16InterpretPhonemeP10TranslatoriP12PHONEME_LISTP12PHONEME_DATA($181, 256, $182, $phdata); //@line 393 "phonemelist.cpp"
            ; //@line 394 "phonemelist.cpp"
          }
  
          var $183=$ph; //@line 396 "phonemelist.cpp"
          var $type295=(($183+11)&4294967295); //@line 396 "phonemelist.cpp"
          var $184=HEAPU8[($type295)]; //@line 396 "phonemelist.cpp"
          var $conv296=((($184))&255); //@line 396 "phonemelist.cpp"
          var $cmp297=((($conv296))|0)==2; //@line 396 "phonemelist.cpp"
          if ($cmp297) { __label__ = 91;; } else { __label__ = 110;; } //@line 396 "phonemelist.cpp"
          $if_then298$$if_end362$127: do { 
            if (__label__ == 91) {
  
              var $185=$plist3; //@line 402 "phonemelist.cpp"
              var $stresslevel299=(($185+1)&4294967295); //@line 402 "phonemelist.cpp"
              var $186=HEAPU8[($stresslevel299)]; //@line 402 "phonemelist.cpp"
              var $conv300=((($186))&255); //@line 402 "phonemelist.cpp"
              var $cmp301=((($conv300))|0) <= 1; //@line 402 "phonemelist.cpp"
              if ($cmp301) { __label__ = 92;; } else { __label__ = 109;; } //@line 402 "phonemelist.cpp"
              if (__label__ == 92) {
  
                var $187=$unstress_count; //@line 405 "phonemelist.cpp"
                var $inc303=((($187)+1)&4294967295); //@line 405 "phonemelist.cpp"
                $unstress_count=$inc303; //@line 405 "phonemelist.cpp"
                var $188=$tr_addr; //@line 407 "phonemelist.cpp"
                var $langopts304=(($188)&4294967295); //@line 407 "phonemelist.cpp"
                var $stress_flags=(($langopts304+12)&4294967295); //@line 407 "phonemelist.cpp"
                var $189=HEAP32[(($stress_flags)>>2)]; //@line 407 "phonemelist.cpp"
                var $and305=($189) & 8; //@line 407 "phonemelist.cpp"
                var $tobool306=((($and305))|0)!=0; //@line 407 "phonemelist.cpp"
                if ($tobool306) { __label__ = 93;; } else { __label__ = 102;; } //@line 407 "phonemelist.cpp"
                if (__label__ == 93) {
  
                  var $190=$plist3; //@line 410 "phonemelist.cpp"
                  var $add_ptr=(($190+24)&4294967295); //@line 410 "phonemelist.cpp"
                  $p=$add_ptr; //@line 410 "phonemelist.cpp"
                  ; //@line 410 "phonemelist.cpp"
                  while(1) { 
  
                    var $191=$p; //@line 410 "phonemelist.cpp"
                    var $type309=(($191+15)&4294967295); //@line 410 "phonemelist.cpp"
                    var $192=HEAPU8[($type309)]; //@line 410 "phonemelist.cpp"
                    var $conv310=((($192))&255); //@line 410 "phonemelist.cpp"
                    var $cmp311=((($conv310))|0)!=0; //@line 410 "phonemelist.cpp"
                    if (!($cmp311)) { __label__ = 110;break $if_then298$$if_end362$127; } //@line 410 "phonemelist.cpp"
  
                    var $193=$p; //@line 412 "phonemelist.cpp"
                    var $type313=(($193+15)&4294967295); //@line 412 "phonemelist.cpp"
                    var $194=HEAPU8[($type313)]; //@line 412 "phonemelist.cpp"
                    var $conv314=((($194))&255); //@line 412 "phonemelist.cpp"
                    var $cmp315=((($conv314))|0)==2; //@line 412 "phonemelist.cpp"
                    var $195=$p; //@line 414 "phonemelist.cpp"
                    if ($cmp315) { __label__ = 96;break ; } //@line 412 "phonemelist.cpp"
  
                    var $incdec_ptr=(($195+24)&4294967295); //@line 410 "phonemelist.cpp"
                    $p=$incdec_ptr; //@line 410 "phonemelist.cpp"
                    __label__ = 94;continue ; //@line 410 "phonemelist.cpp"
                  }
  
                  var $stresslevel317=(($195+1)&4294967295); //@line 414 "phonemelist.cpp"
                  var $196=HEAPU8[($stresslevel317)]; //@line 414 "phonemelist.cpp"
                  var $conv318=((($196))&255); //@line 414 "phonemelist.cpp"
                  var $cmp319=((($conv318))|0) <= 1; //@line 414 "phonemelist.cpp"
                  if (!($cmp319)) { __label__ = 110;break $if_then298$$if_end362$127; } //@line 414 "phonemelist.cpp"
  
                  var $197=$plist3; //@line 416 "phonemelist.cpp"
                  var $wordstress321=(($197+2)&4294967295); //@line 416 "phonemelist.cpp"
                  var $198=HEAPU8[($wordstress321)]; //@line 416 "phonemelist.cpp"
                  var $conv322=((($198))&255); //@line 416 "phonemelist.cpp"
                  var $cmp323=((($conv322))|0) < 4; //@line 416 "phonemelist.cpp"
                  if ($cmp323) { __label__ = 98;; } else { __label__ = 99;; } //@line 416 "phonemelist.cpp"
                  if (__label__ == 98) {
  
                    var $199=$plist3; //@line 417 "phonemelist.cpp"
                    var $stresslevel325=(($199+1)&4294967295); //@line 417 "phonemelist.cpp"
                    HEAP8[($stresslevel325)]=0; //@line 417 "phonemelist.cpp"
                    ; //@line 417 "phonemelist.cpp"
                  }
  
                  var $200=$p; //@line 418 "phonemelist.cpp"
                  var $wordstress327=(($200+2)&4294967295); //@line 418 "phonemelist.cpp"
                  var $201=HEAPU8[($wordstress327)]; //@line 418 "phonemelist.cpp"
                  var $conv328=((($201))&255); //@line 418 "phonemelist.cpp"
                  var $cmp329=((($conv328))|0) < 4; //@line 418 "phonemelist.cpp"
                  if (!($cmp329)) { __label__ = 110;break $if_then298$$if_end362$127; } //@line 418 "phonemelist.cpp"
  
                  var $202=$p; //@line 419 "phonemelist.cpp"
                  var $stresslevel331=(($202+1)&4294967295); //@line 419 "phonemelist.cpp"
                  HEAP8[($stresslevel331)]=0; //@line 419 "phonemelist.cpp"
                  ; //@line 419 "phonemelist.cpp"
                }
                else if (__label__ == 102) {
  
                  var $203=$unstress_count; //@line 427 "phonemelist.cpp"
                  var $cmp338=((($203))|0) > 1; //@line 427 "phonemelist.cpp"
                  if (!($cmp338)) { __label__ = 110;break $if_then298$$if_end362$127; } //@line 427 "phonemelist.cpp"
  
                  var $204=$unstress_count; //@line 427 "phonemelist.cpp"
                  var $and340=($204) & 1; //@line 427 "phonemelist.cpp"
                  var $cmp341=((($and340))|0)==0; //@line 427 "phonemelist.cpp"
                  if (!($cmp341)) { __label__ = 110;break $if_then298$$if_end362$127; } //@line 427 "phonemelist.cpp"
  
                  var $205=$tr_addr; //@line 431 "phonemelist.cpp"
                  var $langopts343=(($205)&4294967295); //@line 431 "phonemelist.cpp"
                  var $stress_flags344=(($langopts343+12)&4294967295); //@line 431 "phonemelist.cpp"
                  var $206=HEAP32[(($stress_flags344)>>2)]; //@line 431 "phonemelist.cpp"
                  var $and345=($206) & 2; //@line 431 "phonemelist.cpp"
                  var $tobool346=((($and345))|0)!=0; //@line 431 "phonemelist.cpp"
                  if ($tobool346) { __label__ = 107;; } else { __label__ = 105;; } //@line 431 "phonemelist.cpp"
                  $if_then354$$lor_lhs_false347$146: do { 
                    if (__label__ == 105) {
  
                      var $207=$word_stress; //@line 431 "phonemelist.cpp"
                      var $cmp348=((($207))|0) > 3; //@line 431 "phonemelist.cpp"
                      if ($cmp348) { __label__ = 106;; } else { __label__ = 108;; } //@line 431 "phonemelist.cpp"
                      if (__label__ == 106) {
  
                        var $208=$plist3; //@line 431 "phonemelist.cpp"
                        var $add_ptr350=(($208+24)&4294967295); //@line 431 "phonemelist.cpp"
                        var $sourceix351=(($add_ptr350+6)&4294967295); //@line 431 "phonemelist.cpp"
                        var $209=HEAPU16[(($sourceix351)>>1)]; //@line 431 "phonemelist.cpp"
                        var $conv352=((($209))&65535); //@line 431 "phonemelist.cpp"
                        var $cmp353=((($conv352))|0)!=0; //@line 431 "phonemelist.cpp"
                        if ($cmp353) { __label__ = 107;break $if_then354$$lor_lhs_false347$146; } //@line 431 "phonemelist.cpp"
                      }
  
                      var $210=$plist3; //@line 438 "phonemelist.cpp"
                      var $stresslevel356=(($210+1)&4294967295); //@line 438 "phonemelist.cpp"
                      HEAP8[($stresslevel356)]=0; //@line 438 "phonemelist.cpp"
                      __label__ = 110;break $if_then298$$if_end362$127;
                    }
                  } while(0);
  
                  $unstress_count=1; //@line 434 "phonemelist.cpp"
                  ; //@line 435 "phonemelist.cpp"
                }
              }
              else if (__label__ == 109) {
  
                $unstress_count=0; //@line 445 "phonemelist.cpp"
                ;
              }
            }
          } while(0);
  
          var $211=$ph; //@line 493 "phonemelist.cpp"
          var $type363=(($211+11)&4294967295); //@line 493 "phonemelist.cpp"
          var $212=HEAPU8[($type363)]; //@line 493 "phonemelist.cpp"
          var $conv364=((($212))&255); //@line 493 "phonemelist.cpp"
          var $cmp365=((($conv364))|0)==2; //@line 493 "phonemelist.cpp"
          if ($cmp365) { __label__ = 111;; } else { __label__ = 112;; } //@line 493 "phonemelist.cpp"
          if (__label__ == 111) {
  
            $first_vowel=0; //@line 494 "phonemelist.cpp"
            ; //@line 494 "phonemelist.cpp"
          }
  
          var $213=$plist3; //@line 496 "phonemelist.cpp"
          var $add_ptr368=(($213+24)&4294967295); //@line 496 "phonemelist.cpp"
          var $synthflags369=(($add_ptr368+4)&4294967295); //@line 496 "phonemelist.cpp"
          var $214=HEAPU16[(($synthflags369)>>1)]; //@line 496 "phonemelist.cpp"
          var $conv370=((($214))&65535); //@line 496 "phonemelist.cpp"
          var $and371=($conv370) & 8; //@line 496 "phonemelist.cpp"
          var $tobool372=((($and371))|0)!=0; //@line 496 "phonemelist.cpp"
          if ($tobool372) { __label__ = 113;; } else { __label__ = 115;; } //@line 496 "phonemelist.cpp"
          $if_then373$$if_end387$157: do { 
            if (__label__ == 113) {
  
              var $215=$next; //@line 499 "phonemelist.cpp"
              var $type374=(($215+11)&4294967295); //@line 499 "phonemelist.cpp"
              var $216=HEAPU8[($type374)]; //@line 499 "phonemelist.cpp"
              var $conv375=((($216))&255); //@line 499 "phonemelist.cpp"
              var $call376=_strchr(((__ZZ15MakePhonemeListP10TranslatoriiE12types_double)&4294967295), $conv375); //@line 499 "phonemelist.cpp"
              var $tobool377=((($call376))|0)!=0; //@line 499 "phonemelist.cpp"
              if (!($tobool377)) { __label__ = 115;break $if_then373$$if_end387$157; } //@line 499 "phonemelist.cpp"
  
              var $217=$next; //@line 502 "phonemelist.cpp"
              var $code379=(($217+10)&4294967295); //@line 502 "phonemelist.cpp"
              var $218=HEAPU8[($code379)]; //@line 502 "phonemelist.cpp"
              var $conv380=((($218))&255); //@line 502 "phonemelist.cpp"
              $insert_ph=$conv380; //@line 502 "phonemelist.cpp"
              var $219=$plist3; //@line 503 "phonemelist.cpp"
              var $add_ptr381=(($219+24)&4294967295); //@line 503 "phonemelist.cpp"
              var $synthflags382=(($add_ptr381+4)&4294967295); //@line 503 "phonemelist.cpp"
              var $220=HEAPU16[(($synthflags382)>>1)]; //@line 503 "phonemelist.cpp"
              var $conv383=((($220))&65535); //@line 503 "phonemelist.cpp"
              var $xor384=($conv383) ^ 8; //@line 503 "phonemelist.cpp"
              var $conv385=((($xor384)) & 65535); //@line 503 "phonemelist.cpp"
              HEAP16[(($synthflags382)>>1)]=$conv385; //@line 503 "phonemelist.cpp"
              ; //@line 504 "phonemelist.cpp"
            }
          } while(0);
  
          var $221=$plist3; //@line 507 "phonemelist.cpp"
          var $add_ptr388=(($221+24)&4294967295); //@line 507 "phonemelist.cpp"
          var $sourceix389=(($add_ptr388+6)&4294967295); //@line 507 "phonemelist.cpp"
          var $222=HEAPU16[(($sourceix389)>>1)]; //@line 507 "phonemelist.cpp"
          var $conv390=((($222))&65535); //@line 507 "phonemelist.cpp"
          var $cmp391=((($conv390))|0)!=0; //@line 507 "phonemelist.cpp"
          if ($cmp391) { __label__ = 116;; } else { __label__ = 141;; } //@line 507 "phonemelist.cpp"
          $if_then392$$if_end471$161: do { 
            if (__label__ == 116) {
  
              var $223=$tr_addr; //@line 511 "phonemelist.cpp"
              var $langopts393=(($223)&4294967295); //@line 511 "phonemelist.cpp"
              var $vowel_pause=(($langopts393+4)&4294967295); //@line 511 "phonemelist.cpp"
              var $224=HEAP32[(($vowel_pause)>>2)]; //@line 511 "phonemelist.cpp"
              var $tobool394=((($224))|0)!=0; //@line 511 "phonemelist.cpp"
              if ($tobool394) { __label__ = 117;; } else { __label__ = 134;; } //@line 511 "phonemelist.cpp"
              $land_lhs_true395$$if_end450$163: do { 
                if (__label__ == 117) {
  
                  var $225=$ph; //@line 511 "phonemelist.cpp"
                  var $type396=(($225+11)&4294967295); //@line 511 "phonemelist.cpp"
                  var $226=HEAPU8[($type396)]; //@line 511 "phonemelist.cpp"
                  var $conv397=((($226))&255); //@line 511 "phonemelist.cpp"
                  var $cmp398=((($conv397))|0)!=0; //@line 511 "phonemelist.cpp"
                  if (!($cmp398)) { __label__ = 134;break $land_lhs_true395$$if_end450$163; } //@line 511 "phonemelist.cpp"
  
                  var $227=$ph; //@line 514 "phonemelist.cpp"
                  var $type400=(($227+11)&4294967295); //@line 514 "phonemelist.cpp"
                  var $228=HEAPU8[($type400)]; //@line 514 "phonemelist.cpp"
                  var $conv401=((($228))&255); //@line 514 "phonemelist.cpp"
                  var $cmp402=((($conv401))|0)!=2; //@line 514 "phonemelist.cpp"
                  if ($cmp402) { __label__ = 119;; } else { __label__ = 121;; } //@line 514 "phonemelist.cpp"
                  $land_lhs_true403$$if_end409$166: do { 
                    if (__label__ == 119) {
  
                      var $229=$tr_addr; //@line 514 "phonemelist.cpp"
                      var $langopts404=(($229)&4294967295); //@line 514 "phonemelist.cpp"
                      var $vowel_pause405=(($langopts404+4)&4294967295); //@line 514 "phonemelist.cpp"
                      var $230=HEAP32[(($vowel_pause405)>>2)]; //@line 514 "phonemelist.cpp"
                      var $and406=($230) & 512; //@line 514 "phonemelist.cpp"
                      var $tobool407=((($and406))|0)!=0; //@line 514 "phonemelist.cpp"
                      if (!($tobool407)) { __label__ = 121;break $land_lhs_true403$$if_end409$166; } //@line 514 "phonemelist.cpp"
  
                      $insert_ph=11; //@line 517 "phonemelist.cpp"
                      ; //@line 518 "phonemelist.cpp"
                    }
                  } while(0);
  
                  var $231=$next; //@line 520 "phonemelist.cpp"
                  var $type410=(($231+11)&4294967295); //@line 520 "phonemelist.cpp"
                  var $232=HEAPU8[($type410)]; //@line 520 "phonemelist.cpp"
                  var $conv411=((($232))&255); //@line 520 "phonemelist.cpp"
                  var $cmp412=((($conv411))|0)==2; //@line 520 "phonemelist.cpp"
                  if (!($cmp412)) { __label__ = 134;break $land_lhs_true395$$if_end450$163; } //@line 520 "phonemelist.cpp"
  
                  var $233=$tr_addr; //@line 522 "phonemelist.cpp"
                  var $langopts414=(($233)&4294967295); //@line 522 "phonemelist.cpp"
                  var $vowel_pause415=(($langopts414+4)&4294967295); //@line 522 "phonemelist.cpp"
                  var $234=HEAP32[(($vowel_pause415)>>2)]; //@line 522 "phonemelist.cpp"
                  var $and416=($234) & 12; //@line 522 "phonemelist.cpp"
                  $x=$and416; //@line 522 "phonemelist.cpp"
                  var $cmp417=((($and416))|0)!=0; //@line 522 "phonemelist.cpp"
                  if ($cmp417) { __label__ = 123;; } else { __label__ = 126;; } //@line 522 "phonemelist.cpp"
                  if (__label__ == 123) {
  
                    var $235=$x; //@line 525 "phonemelist.cpp"
                    var $cmp419=((($235))|0)==12; //@line 525 "phonemelist.cpp"
                    if ($cmp419) { __label__ = 124;; } else { __label__ = 125;; } //@line 525 "phonemelist.cpp"
                    if (__label__ == 124) {
  
                      $insert_ph=11; //@line 526 "phonemelist.cpp"
                      ; //@line 526 "phonemelist.cpp"
                    }
                    else if (__label__ == 125) {
  
                      $insert_ph=23; //@line 528 "phonemelist.cpp"
                      ;
                    }
                  }
  
                  var $236=$ph; //@line 531 "phonemelist.cpp"
                  var $type424=(($236+11)&4294967295); //@line 531 "phonemelist.cpp"
                  var $237=HEAPU8[($type424)]; //@line 531 "phonemelist.cpp"
                  var $conv425=((($237))&255); //@line 531 "phonemelist.cpp"
                  var $cmp426=((($conv425))|0)==2; //@line 531 "phonemelist.cpp"
                  if ($cmp426) { __label__ = 127;; } else { __label__ = 131;; } //@line 531 "phonemelist.cpp"
                  $land_lhs_true427$$if_end437$177: do { 
                    if (__label__ == 127) {
  
                      var $238=$tr_addr; //@line 531 "phonemelist.cpp"
                      var $langopts428=(($238)&4294967295); //@line 531 "phonemelist.cpp"
                      var $vowel_pause429=(($langopts428+4)&4294967295); //@line 531 "phonemelist.cpp"
                      var $239=HEAP32[(($vowel_pause429)>>2)]; //@line 531 "phonemelist.cpp"
                      var $and430=($239) & 3; //@line 531 "phonemelist.cpp"
                      $x=$and430; //@line 531 "phonemelist.cpp"
                      var $cmp431=((($and430))|0)!=0; //@line 531 "phonemelist.cpp"
                      if (!($cmp431)) { __label__ = 131;break $land_lhs_true427$$if_end437$177; } //@line 531 "phonemelist.cpp"
  
                      var $240=$x; //@line 534 "phonemelist.cpp"
                      var $cmp433=((($240))|0)==2; //@line 534 "phonemelist.cpp"
                      if ($cmp433) { __label__ = 129;; } else { __label__ = 130;; } //@line 534 "phonemelist.cpp"
                      if (__label__ == 129) {
  
                        $insert_ph=10; //@line 535 "phonemelist.cpp"
                        ; //@line 535 "phonemelist.cpp"
                      }
                      else if (__label__ == 130) {
  
                        $insert_ph=23; //@line 537 "phonemelist.cpp"
                        ;
                      }
                    }
                  } while(0);
  
                  var $241=$plist3; //@line 540 "phonemelist.cpp"
                  var $add_ptr438=(($241+24)&4294967295); //@line 540 "phonemelist.cpp"
                  var $stresslevel439=(($add_ptr438+1)&4294967295); //@line 540 "phonemelist.cpp"
                  var $242=HEAPU8[($stresslevel439)]; //@line 540 "phonemelist.cpp"
                  var $conv440=((($242))&255); //@line 540 "phonemelist.cpp"
                  var $cmp441=((($conv440))|0) >= 4; //@line 540 "phonemelist.cpp"
                  if (!($cmp441)) { __label__ = 134;break $land_lhs_true395$$if_end450$163; } //@line 540 "phonemelist.cpp"
  
                  var $243=$tr_addr; //@line 540 "phonemelist.cpp"
                  var $langopts443=(($243)&4294967295); //@line 540 "phonemelist.cpp"
                  var $vowel_pause444=(($langopts443+4)&4294967295); //@line 540 "phonemelist.cpp"
                  var $244=HEAP32[(($vowel_pause444)>>2)]; //@line 540 "phonemelist.cpp"
                  var $and445=($244) & 256; //@line 540 "phonemelist.cpp"
                  var $tobool446=((($and445))|0)!=0; //@line 540 "phonemelist.cpp"
                  if (!($tobool446)) { __label__ = 134;break $land_lhs_true395$$if_end450$163; } //@line 540 "phonemelist.cpp"
  
                  $insert_ph=10; //@line 543 "phonemelist.cpp"
                  ; //@line 544 "phonemelist.cpp"
                }
              } while(0);
  
              var $245=$plist3; //@line 548 "phonemelist.cpp"
              var $246=$plist3_inserted; //@line 548 "phonemelist.cpp"
              var $cmp451=((($245))|0)!=((($246))|0); //@line 548 "phonemelist.cpp"
              if (!($cmp451)) { __label__ = 141;break $if_then392$$if_end471$161; } //@line 548 "phonemelist.cpp"
  
              var $247=$tr_addr; //@line 550 "phonemelist.cpp"
              var $langopts453=(($247)&4294967295); //@line 550 "phonemelist.cpp"
              var $word_gap=(($langopts453)&4294967295); //@line 550 "phonemelist.cpp"
              var $248=HEAP32[(($word_gap)>>2)]; //@line 550 "phonemelist.cpp"
              var $and454=($248) & 7; //@line 550 "phonemelist.cpp"
              $x=$and454; //@line 550 "phonemelist.cpp"
              var $cmp455=((($and454))|0)!=0; //@line 550 "phonemelist.cpp"
              if ($cmp455) { __label__ = 136;; } else { __label__ = 139;; } //@line 550 "phonemelist.cpp"
              $if_then456$$if_end466$188: do { 
                if (__label__ == 136) {
  
                  var $249=$x; //@line 552 "phonemelist.cpp"
                  var $cmp457=((($249))|0) > 1; //@line 552 "phonemelist.cpp"
                  if ($cmp457) { __label__ = 138;; } else { __label__ = 137;; } //@line 552 "phonemelist.cpp"
                  if (__label__ == 137) {
  
                    var $250=$insert_ph; //@line 552 "phonemelist.cpp"
                    var $cmp459=((($250))|0)!=10; //@line 552 "phonemelist.cpp"
                    var $251=$insert_ph; //@line 552 "phonemelist.cpp"
                    var $cmp461=((($251))|0)!=11; //@line 552 "phonemelist.cpp"
                    var $or_cond3=($cmp459) & ($cmp461); //@line 552 "phonemelist.cpp"
                    if (!($or_cond3)) { __label__ = 139;break $if_then456$$if_end466$188; } //@line 552 "phonemelist.cpp"
                  }
  
                  var $252=$x; //@line 555 "phonemelist.cpp"
                  var $arrayidx463=((_pause_phonemes+$252)&4294967295); //@line 555 "phonemelist.cpp"
                  var $253=HEAPU8[($arrayidx463)]; //@line 555 "phonemelist.cpp"
                  var $conv464=((($253))&255); //@line 555 "phonemelist.cpp"
                  $insert_ph=$conv464; //@line 555 "phonemelist.cpp"
                  ; //@line 556 "phonemelist.cpp"
                }
              } while(0);
  
              var $254=HEAP32[((_option_wordgap)>>2)]; //@line 558 "phonemelist.cpp"
              var $cmp467=((($254))|0) > 0; //@line 558 "phonemelist.cpp"
              if (!($cmp467)) { __label__ = 141;break $if_then392$$if_end471$161; } //@line 558 "phonemelist.cpp"
  
              $insert_ph=24; //@line 560 "phonemelist.cpp"
              ; //@line 561 "phonemelist.cpp"
            }
          } while(0);
  
          var $255=$plist3; //@line 565 "phonemelist.cpp"
          var $arrayidx472=(($255+48)&4294967295); //@line 565 "phonemelist.cpp"
          var $phcode473=(($arrayidx472)&4294967295); //@line 565 "phonemelist.cpp"
          var $256=HEAPU8[($phcode473)]; //@line 565 "phonemelist.cpp"
          var $idxprom474=((($256))&255); //@line 565 "phonemelist.cpp"
          var $arrayidx475=((_phoneme_tab+$idxprom474*4)&4294967295); //@line 565 "phonemelist.cpp"
          var $257=HEAP32[(($arrayidx475)>>2)]; //@line 565 "phonemelist.cpp"
          $next2=$257; //@line 565 "phonemelist.cpp"
          var $258=$next2; //@line 566 "phonemelist.cpp"
          var $259=$plist3; //@line 566 "phonemelist.cpp"
          var $arrayidx476=(($259+48)&4294967295); //@line 566 "phonemelist.cpp"
          var $ph477=(($arrayidx476+8)&4294967295); //@line 566 "phonemelist.cpp"
          HEAP32[(($ph477)>>2)]=$258; //@line 566 "phonemelist.cpp"
          var $260=$insert_ph; //@line 568 "phonemelist.cpp"
          var $cmp478=((($260))|0)==0; //@line 568 "phonemelist.cpp"
          if ($cmp478) { __label__ = 142;; } else { __label__ = 144;; } //@line 568 "phonemelist.cpp"
          $land_lhs_true479$$if_end486$196: do { 
            if (__label__ == 142) {
  
              var $261=HEAP32[(($arrayidx481)>>2)]; //@line 568 "phonemelist.cpp"
              var $cmp482=((($261))|0)!=0; //@line 568 "phonemelist.cpp"
              if (!($cmp482)) { __label__ = 144;break $land_lhs_true479$$if_end486$196; } //@line 568 "phonemelist.cpp"
  
              var $262=HEAP32[(($arrayidx485)>>2)]; //@line 570 "phonemelist.cpp"
              $insert_ph=$262; //@line 570 "phonemelist.cpp"
              ; //@line 571 "phonemelist.cpp"
            }
          } while(0);
  
          var $263=$ph; //@line 581 "phonemelist.cpp"
          var $264=$ix; //@line 581 "phonemelist.cpp"
          var $265=$phlist; //@line 581 "phonemelist.cpp"
          var $arrayidx491=(($265+24*$264)&4294967295); //@line 581 "phonemelist.cpp"
          var $ph492=(($arrayidx491+8)&4294967295); //@line 581 "phonemelist.cpp"
          HEAP32[(($ph492)>>2)]=$263; //@line 581 "phonemelist.cpp"
          var $266=$ph; //@line 582 "phonemelist.cpp"
          var $type493=(($266+11)&4294967295); //@line 582 "phonemelist.cpp"
          var $267=HEAP8[($type493)]; //@line 582 "phonemelist.cpp"
          var $268=$ix; //@line 582 "phonemelist.cpp"
          var $269=$phlist; //@line 582 "phonemelist.cpp"
          var $arrayidx494=(($269+24*$268)&4294967295); //@line 582 "phonemelist.cpp"
          var $type495=(($arrayidx494+15)&4294967295); //@line 582 "phonemelist.cpp"
          HEAP8[($type495)]=$267; //@line 582 "phonemelist.cpp"
          var $270=$ix; //@line 583 "phonemelist.cpp"
          var $271=$phlist; //@line 583 "phonemelist.cpp"
          var $arrayidx496=(($271+24*$270)&4294967295); //@line 583 "phonemelist.cpp"
          var $env=(($arrayidx496+14)&4294967295); //@line 583 "phonemelist.cpp"
          HEAP8[($env)]=0; //@line 583 "phonemelist.cpp"
          var $272=$plist3; //@line 584 "phonemelist.cpp"
          var $synthflags497=(($272+4)&4294967295); //@line 584 "phonemelist.cpp"
          var $273=HEAPU16[(($synthflags497)>>1)]; //@line 584 "phonemelist.cpp"
          var $conv498=((($273))&65535); //@line 584 "phonemelist.cpp"
          var $274=$switched_language; //@line 584 "phonemelist.cpp"
          var $or499=($conv498) | ($274); //@line 584 "phonemelist.cpp"
          var $conv500=((($or499)) & 65535); //@line 584 "phonemelist.cpp"
          var $275=$ix; //@line 584 "phonemelist.cpp"
          var $276=$phlist; //@line 584 "phonemelist.cpp"
          var $arrayidx501=(($276+24*$275)&4294967295); //@line 584 "phonemelist.cpp"
          var $synthflags502=(($arrayidx501+4)&4294967295); //@line 584 "phonemelist.cpp"
          HEAP16[(($synthflags502)>>1)]=$conv500; //@line 584 "phonemelist.cpp"
          var $277=$plist3; //@line 585 "phonemelist.cpp"
          var $stresslevel503=(($277+1)&4294967295); //@line 585 "phonemelist.cpp"
          var $278=HEAPU8[($stresslevel503)]; //@line 585 "phonemelist.cpp"
          var $conv504=((($278))&255); //@line 585 "phonemelist.cpp"
          var $and505=($conv504) & 15; //@line 585 "phonemelist.cpp"
          var $conv506=((($and505)) & 255); //@line 585 "phonemelist.cpp"
          var $279=$ix; //@line 585 "phonemelist.cpp"
          var $280=$phlist; //@line 585 "phonemelist.cpp"
          var $arrayidx507=(($280+24*$279)&4294967295); //@line 585 "phonemelist.cpp"
          var $stresslevel508=(($arrayidx507+1)&4294967295); //@line 585 "phonemelist.cpp"
          HEAP8[($stresslevel508)]=$conv506; //@line 585 "phonemelist.cpp"
          var $281=$plist3; //@line 586 "phonemelist.cpp"
          var $wordstress509=(($281+2)&4294967295); //@line 586 "phonemelist.cpp"
          var $282=HEAP8[($wordstress509)]; //@line 586 "phonemelist.cpp"
          var $283=$ix; //@line 586 "phonemelist.cpp"
          var $284=$phlist; //@line 586 "phonemelist.cpp"
          var $arrayidx510=(($284+24*$283)&4294967295); //@line 586 "phonemelist.cpp"
          var $wordstress511=(($arrayidx510+2)&4294967295); //@line 586 "phonemelist.cpp"
          HEAP8[($wordstress511)]=$282; //@line 586 "phonemelist.cpp"
          var $285=$plist3; //@line 587 "phonemelist.cpp"
          var $tone_ph512=(($285+3)&4294967295); //@line 587 "phonemelist.cpp"
          var $286=HEAP8[($tone_ph512)]; //@line 587 "phonemelist.cpp"
          var $287=$ix; //@line 587 "phonemelist.cpp"
          var $288=$phlist; //@line 587 "phonemelist.cpp"
          var $arrayidx513=(($288+24*$287)&4294967295); //@line 587 "phonemelist.cpp"
          var $tone_ph514=(($arrayidx513+3)&4294967295); //@line 587 "phonemelist.cpp"
          HEAP8[($tone_ph514)]=$286; //@line 587 "phonemelist.cpp"
          var $289=$ix; //@line 588 "phonemelist.cpp"
          var $290=$phlist; //@line 588 "phonemelist.cpp"
          var $arrayidx515=(($290+24*$289)&4294967295); //@line 588 "phonemelist.cpp"
          var $sourceix516=(($arrayidx515+6)&4294967295); //@line 588 "phonemelist.cpp"
          HEAP16[(($sourceix516)>>1)]=0; //@line 588 "phonemelist.cpp"
          var $291=$ph; //@line 589 "phonemelist.cpp"
          var $code517=(($291+10)&4294967295); //@line 589 "phonemelist.cpp"
          var $292=HEAP8[($code517)]; //@line 589 "phonemelist.cpp"
          var $293=$ix; //@line 589 "phonemelist.cpp"
          var $294=$phlist; //@line 589 "phonemelist.cpp"
          var $arrayidx518=(($294+24*$293)&4294967295); //@line 589 "phonemelist.cpp"
          var $phcode519=(($arrayidx518)&4294967295); //@line 589 "phonemelist.cpp"
          HEAP8[($phcode519)]=$292; //@line 589 "phonemelist.cpp"
          var $295=$plist3; //@line 591 "phonemelist.cpp"
          var $sourceix520=(($295+6)&4294967295); //@line 591 "phonemelist.cpp"
          var $296=HEAPU16[(($sourceix520)>>1)]; //@line 591 "phonemelist.cpp"
          var $conv521=((($296))&65535); //@line 591 "phonemelist.cpp"
          var $cmp522=((($conv521))|0)!=0; //@line 591 "phonemelist.cpp"
          if ($cmp522) { __label__ = 145;; } else { __label__ = 147;; } //@line 591 "phonemelist.cpp"
          $if_then523$$if_else533$200: do { 
            if (__label__ == 145) {
  
              var $297=$plist3; //@line 593 "phonemelist.cpp"
              var $sourceix524=(($297+6)&4294967295); //@line 593 "phonemelist.cpp"
              var $298=HEAP16[(($sourceix524)>>1)]; //@line 593 "phonemelist.cpp"
              var $299=$ix; //@line 593 "phonemelist.cpp"
              var $300=$phlist; //@line 593 "phonemelist.cpp"
              var $arrayidx525=(($300+24*$299)&4294967295); //@line 593 "phonemelist.cpp"
              var $sourceix526=(($arrayidx525+6)&4294967295); //@line 593 "phonemelist.cpp"
              HEAP16[(($sourceix526)>>1)]=$298; //@line 593 "phonemelist.cpp"
              var $301=$ix; //@line 594 "phonemelist.cpp"
              var $302=$phlist; //@line 594 "phonemelist.cpp"
              var $arrayidx527=(($302+24*$301)&4294967295); //@line 594 "phonemelist.cpp"
              var $newword=(($arrayidx527+19)&4294967295); //@line 594 "phonemelist.cpp"
              HEAP8[($newword)]=1; //@line 594 "phonemelist.cpp"
              var $303=$start_sentence_addr; //@line 596 "phonemelist.cpp"
              var $tobool528=((($303))|0)!=0; //@line 596 "phonemelist.cpp"
              if (!($tobool528)) { __label__ = 148;break $if_then523$$if_else533$200; } //@line 596 "phonemelist.cpp"
  
              var $304=$ix; //@line 598 "phonemelist.cpp"
              var $305=$phlist; //@line 598 "phonemelist.cpp"
              var $arrayidx530=(($305+24*$304)&4294967295); //@line 598 "phonemelist.cpp"
              var $newword531=(($arrayidx530+19)&4294967295); //@line 598 "phonemelist.cpp"
              HEAP8[($newword531)]=5; //@line 598 "phonemelist.cpp"
              $start_sentence_addr=0; //@line 599 "phonemelist.cpp"
              ; //@line 600 "phonemelist.cpp"
            }
            else if (__label__ == 147) {
  
              var $306=$ix; //@line 604 "phonemelist.cpp"
              var $307=$phlist; //@line 604 "phonemelist.cpp"
              var $arrayidx534=(($307+24*$306)&4294967295); //@line 604 "phonemelist.cpp"
              var $newword535=(($arrayidx534+19)&4294967295); //@line 604 "phonemelist.cpp"
              HEAP8[($newword535)]=0; //@line 604 "phonemelist.cpp"
              ;
            }
          } while(0);
  
          var $308=HEAP32[(($arrayidx538)>>2)]; //@line 608 "phonemelist.cpp"
          var $mul=((($308)*2)&4294967295); //@line 608 "phonemelist.cpp"
          var $conv539=((($mul)) & 65535); //@line 608 "phonemelist.cpp"
          var $309=$ix; //@line 608 "phonemelist.cpp"
          var $310=$phlist; //@line 608 "phonemelist.cpp"
          var $arrayidx540=(($310+24*$309)&4294967295); //@line 608 "phonemelist.cpp"
          var $length=(($arrayidx540+12)&4294967295); //@line 608 "phonemelist.cpp"
          HEAP16[(($length)>>1)]=$conv539; //@line 608 "phonemelist.cpp"
          var $311=$ph; //@line 609 "phonemelist.cpp"
          var $code541=(($311+10)&4294967295); //@line 609 "phonemelist.cpp"
          var $312=HEAPU8[($code541)]; //@line 609 "phonemelist.cpp"
          var $conv542=((($312))&255); //@line 609 "phonemelist.cpp"
          var $cmp543=((($conv542))|0)==24; //@line 609 "phonemelist.cpp"
          var $313=HEAP32[((_option_wordgap)>>2)]; //@line 609 "phonemelist.cpp"
          var $cmp545=((($313))|0) > 0; //@line 609 "phonemelist.cpp"
          var $or_cond4=($cmp543) & ($cmp545); //@line 609 "phonemelist.cpp"
          if ($or_cond4) { __label__ = 149;; } else { __label__ = 150;; } //@line 609 "phonemelist.cpp"
          if (__label__ == 149) {
  
            var $314=HEAP32[((((_phoneme_tab+40)&4294967295))>>2)]; //@line 611 "phonemelist.cpp"
            var $315=$ix; //@line 611 "phonemelist.cpp"
            var $316=$phlist; //@line 611 "phonemelist.cpp"
            var $arrayidx547=(($316+24*$315)&4294967295); //@line 611 "phonemelist.cpp"
            var $ph548=(($arrayidx547+8)&4294967295); //@line 611 "phonemelist.cpp"
            HEAP32[(($ph548)>>2)]=$314; //@line 611 "phonemelist.cpp"
            var $317=HEAP32[((_option_wordgap)>>2)]; //@line 612 "phonemelist.cpp"
            var $mul549=((($317)*14)&4294967295); //@line 612 "phonemelist.cpp"
            var $conv550=((($mul549)) & 65535); //@line 612 "phonemelist.cpp"
            var $318=$ix; //@line 612 "phonemelist.cpp"
            var $319=$phlist; //@line 612 "phonemelist.cpp"
            var $arrayidx551=(($319+24*$318)&4294967295); //@line 612 "phonemelist.cpp"
            var $length552=(($arrayidx551+12)&4294967295); //@line 612 "phonemelist.cpp"
            HEAP16[(($length552)>>1)]=$conv550; //@line 612 "phonemelist.cpp"
            ; //@line 613 "phonemelist.cpp"
          }
  
          var $320=$ph; //@line 615 "phonemelist.cpp"
          var $type554=(($320+11)&4294967295); //@line 615 "phonemelist.cpp"
          var $321=HEAPU8[($type554)]; //@line 615 "phonemelist.cpp"
          var $conv555=((($321))&255); //@line 615 "phonemelist.cpp"
          var $cmp556=((($conv555))|0)==2; //@line 615 "phonemelist.cpp"
          if ($cmp556) { __label__ = 155;; } else { __label__ = 151;; } //@line 615 "phonemelist.cpp"
          $if_then573$$lor_lhs_false557$208: do { 
            if (__label__ == 151) {
  
              var $322=$ph; //@line 615 "phonemelist.cpp"
              var $type558=(($322+11)&4294967295); //@line 615 "phonemelist.cpp"
              var $323=HEAPU8[($type558)]; //@line 615 "phonemelist.cpp"
              var $conv559=((($323))&255); //@line 615 "phonemelist.cpp"
              var $cmp560=((($conv559))|0)==3; //@line 615 "phonemelist.cpp"
              if ($cmp560) { __label__ = 155;break $if_then573$$lor_lhs_false557$208; } //@line 615 "phonemelist.cpp"
  
              var $324=$ph; //@line 615 "phonemelist.cpp"
              var $type562=(($324+11)&4294967295); //@line 615 "phonemelist.cpp"
              var $325=HEAPU8[($type562)]; //@line 615 "phonemelist.cpp"
              var $conv563=((($325))&255); //@line 615 "phonemelist.cpp"
              var $cmp564=((($conv563))|0)==8; //@line 615 "phonemelist.cpp"
              if ($cmp564) { __label__ = 155;break $if_then573$$lor_lhs_false557$208; } //@line 615 "phonemelist.cpp"
  
              var $326=$ph; //@line 615 "phonemelist.cpp"
              var $type566=(($326+11)&4294967295); //@line 615 "phonemelist.cpp"
              var $327=HEAPU8[($type566)]; //@line 615 "phonemelist.cpp"
              var $conv567=((($327))&255); //@line 615 "phonemelist.cpp"
              var $cmp568=((($conv567))|0)==5; //@line 615 "phonemelist.cpp"
              if ($cmp568) { __label__ = 155;break $if_then573$$lor_lhs_false557$208; } //@line 615 "phonemelist.cpp"
  
              var $328=$ph; //@line 615 "phonemelist.cpp"
              var $type570=(($328+11)&4294967295); //@line 615 "phonemelist.cpp"
              var $329=HEAPU8[($type570)]; //@line 615 "phonemelist.cpp"
              var $conv571=((($329))&255); //@line 615 "phonemelist.cpp"
              var $cmp572=((($conv571))|0)==7; //@line 615 "phonemelist.cpp"
              if ($cmp572) { __label__ = 155;break $if_then573$$lor_lhs_false557$208; } else { __label__ = 156;break $if_then573$$lor_lhs_false557$208; } //@line 615 "phonemelist.cpp"
            }
          } while(0);
          if (__label__ == 155) {
  
            var $330=$ix; //@line 617 "phonemelist.cpp"
            var $331=$phlist; //@line 617 "phonemelist.cpp"
            var $arrayidx574=(($331+24*$330)&4294967295); //@line 617 "phonemelist.cpp"
            var $length575=(($arrayidx574+12)&4294967295); //@line 617 "phonemelist.cpp"
            HEAP16[(($length575)>>1)]=128; //@line 617 "phonemelist.cpp"
            var $332=$ix; //@line 618 "phonemelist.cpp"
            var $333=$phlist; //@line 618 "phonemelist.cpp"
            var $arrayidx576=(($333+24*$332)&4294967295); //@line 618 "phonemelist.cpp"
            var $env577=(($arrayidx576+14)&4294967295); //@line 618 "phonemelist.cpp"
            HEAP8[($env577)]=0; //@line 618 "phonemelist.cpp"
            ; //@line 619 "phonemelist.cpp"
          }
  
          var $334=$ix; //@line 621 "phonemelist.cpp"
          var $335=$phlist; //@line 621 "phonemelist.cpp"
          var $arrayidx579=(($335+24*$334)&4294967295); //@line 621 "phonemelist.cpp"
          var $prepause=(($arrayidx579+16)&4294967295); //@line 621 "phonemelist.cpp"
          HEAP8[($prepause)]=0; //@line 621 "phonemelist.cpp"
          var $336=$ix; //@line 622 "phonemelist.cpp"
          var $337=$phlist; //@line 622 "phonemelist.cpp"
          var $arrayidx580=(($337+24*$336)&4294967295); //@line 622 "phonemelist.cpp"
          var $amp=(($arrayidx580+18)&4294967295); //@line 622 "phonemelist.cpp"
          HEAP8[($amp)]=20; //@line 622 "phonemelist.cpp"
          var $338=$ix; //@line 623 "phonemelist.cpp"
          var $339=$phlist; //@line 623 "phonemelist.cpp"
          var $arrayidx581=(($339+24*$338)&4294967295); //@line 623 "phonemelist.cpp"
          var $pitch1=(($arrayidx581+20)&4294967295); //@line 623 "phonemelist.cpp"
          HEAP8[($pitch1)]=-1; //@line 623 "phonemelist.cpp"
          var $340=$ix; //@line 624 "phonemelist.cpp"
          var $341=$phlist; //@line 624 "phonemelist.cpp"
          var $arrayidx582=(($341+24*$340)&4294967295); //@line 624 "phonemelist.cpp"
          var $pitch2=(($arrayidx582+21)&4294967295); //@line 624 "phonemelist.cpp"
          HEAP8[($pitch2)]=-1; //@line 624 "phonemelist.cpp"
          var $342=$ix; //@line 625 "phonemelist.cpp"
          var $inc583=((($342)+1)&4294967295); //@line 625 "phonemelist.cpp"
          $ix=$inc583; //@line 625 "phonemelist.cpp"
          ; //@line 626 "phonemelist.cpp"
        }
      } while(0);
  
      var $343=$j; //@line 301 "phonemelist.cpp"
      var $inc585=((($343)+1)&4294967295); //@line 301 "phonemelist.cpp"
      $j=$inc585; //@line 301 "phonemelist.cpp"
      __label__ = 65;continue $for_cond185$91; //@line 301 "phonemelist.cpp"
    }
  
    var $344=$ix; //@line 627 "phonemelist.cpp"
    var $345=$phlist; //@line 627 "phonemelist.cpp"
    var $arrayidx587=(($345+24*$344)&4294967295); //@line 627 "phonemelist.cpp"
    var $newword588=(($arrayidx587+19)&4294967295); //@line 627 "phonemelist.cpp"
    HEAP8[($newword588)]=2; //@line 627 "phonemelist.cpp"
    var $346=$ix; //@line 629 "phonemelist.cpp"
    var $347=$phlist; //@line 629 "phonemelist.cpp"
    var $arrayidx589=(($347+24*$346)&4294967295); //@line 629 "phonemelist.cpp"
    var $phcode590=(($arrayidx589)&4294967295); //@line 629 "phonemelist.cpp"
    HEAP8[($phcode590)]=9; //@line 629 "phonemelist.cpp"
    var $348=$ix; //@line 630 "phonemelist.cpp"
    var $349=$phlist; //@line 630 "phonemelist.cpp"
    var $arrayidx591=(($349+24*$348)&4294967295); //@line 630 "phonemelist.cpp"
    var $type592=(($arrayidx591+15)&4294967295); //@line 630 "phonemelist.cpp"
    HEAP8[($type592)]=0; //@line 630 "phonemelist.cpp"
    var $350=$post_pause_addr; //@line 631 "phonemelist.cpp"
    var $conv593=((($350)) & 65535); //@line 631 "phonemelist.cpp"
    var $351=$ix; //@line 631 "phonemelist.cpp"
    var $352=$phlist; //@line 631 "phonemelist.cpp"
    var $arrayidx594=(($352+24*$351)&4294967295); //@line 631 "phonemelist.cpp"
    var $length595=(($arrayidx594+12)&4294967295); //@line 631 "phonemelist.cpp"
    HEAP16[(($length595)>>1)]=$conv593; //@line 631 "phonemelist.cpp"
    var $353=$end_sourceix; //@line 632 "phonemelist.cpp"
    var $conv596=((($353)) & 65535); //@line 632 "phonemelist.cpp"
    var $354=$ix; //@line 632 "phonemelist.cpp"
    var $355=$phlist; //@line 632 "phonemelist.cpp"
    var $arrayidx597=(($355+24*$354)&4294967295); //@line 632 "phonemelist.cpp"
    var $sourceix598=(($arrayidx597+6)&4294967295); //@line 632 "phonemelist.cpp"
    HEAP16[(($sourceix598)>>1)]=$conv596; //@line 632 "phonemelist.cpp"
    var $356=$ix; //@line 633 "phonemelist.cpp"
    var $357=$phlist; //@line 633 "phonemelist.cpp"
    var $arrayidx599=(($357+24*$356)&4294967295); //@line 633 "phonemelist.cpp"
    var $synthflags600=(($arrayidx599+4)&4294967295); //@line 633 "phonemelist.cpp"
    HEAP16[(($synthflags600)>>1)]=0; //@line 633 "phonemelist.cpp"
    var $358=HEAP32[((((_phoneme_tab+36)&4294967295))>>2)]; //@line 634 "phonemelist.cpp"
    var $359=$ix; //@line 634 "phonemelist.cpp"
    var $inc601=((($359)+1)&4294967295); //@line 634 "phonemelist.cpp"
    $ix=$inc601; //@line 634 "phonemelist.cpp"
    var $360=$phlist; //@line 634 "phonemelist.cpp"
    var $arrayidx602=(($360+24*$359)&4294967295); //@line 634 "phonemelist.cpp"
    var $ph603=(($arrayidx602+8)&4294967295); //@line 634 "phonemelist.cpp"
    HEAP32[(($ph603)>>2)]=$358; //@line 634 "phonemelist.cpp"
    var $361=$ix; //@line 636 "phonemelist.cpp"
    var $362=$phlist; //@line 636 "phonemelist.cpp"
    var $arrayidx604=(($362+24*$361)&4294967295); //@line 636 "phonemelist.cpp"
    var $phcode605=(($arrayidx604)&4294967295); //@line 636 "phonemelist.cpp"
    HEAP8[($phcode605)]=9; //@line 636 "phonemelist.cpp"
    var $363=$ix; //@line 637 "phonemelist.cpp"
    var $364=$phlist; //@line 637 "phonemelist.cpp"
    var $arrayidx606=(($364+24*$363)&4294967295); //@line 637 "phonemelist.cpp"
    var $type607=(($arrayidx606+15)&4294967295); //@line 637 "phonemelist.cpp"
    HEAP8[($type607)]=0; //@line 637 "phonemelist.cpp"
    var $365=$ix; //@line 638 "phonemelist.cpp"
    var $366=$phlist; //@line 638 "phonemelist.cpp"
    var $arrayidx608=(($366+24*$365)&4294967295); //@line 638 "phonemelist.cpp"
    var $length609=(($arrayidx608+12)&4294967295); //@line 638 "phonemelist.cpp"
    HEAP16[(($length609)>>1)]=0; //@line 638 "phonemelist.cpp"
    var $367=$ix; //@line 639 "phonemelist.cpp"
    var $368=$phlist; //@line 639 "phonemelist.cpp"
    var $arrayidx610=(($368+24*$367)&4294967295); //@line 639 "phonemelist.cpp"
    var $sourceix611=(($arrayidx610+6)&4294967295); //@line 639 "phonemelist.cpp"
    HEAP16[(($sourceix611)>>1)]=0; //@line 639 "phonemelist.cpp"
    var $369=$ix; //@line 640 "phonemelist.cpp"
    var $370=$phlist; //@line 640 "phonemelist.cpp"
    var $arrayidx612=(($370+24*$369)&4294967295); //@line 640 "phonemelist.cpp"
    var $synthflags613=(($arrayidx612+4)&4294967295); //@line 640 "phonemelist.cpp"
    HEAP16[(($synthflags613)>>1)]=0; //@line 640 "phonemelist.cpp"
    var $371=HEAP32[((((_phoneme_tab+40)&4294967295))>>2)]; //@line 641 "phonemelist.cpp"
    var $372=$ix; //@line 641 "phonemelist.cpp"
    var $inc614=((($372)+1)&4294967295); //@line 641 "phonemelist.cpp"
    $ix=$inc614; //@line 641 "phonemelist.cpp"
    var $373=$phlist; //@line 641 "phonemelist.cpp"
    var $arrayidx615=(($373+24*$372)&4294967295); //@line 641 "phonemelist.cpp"
    var $ph616=(($arrayidx615+8)&4294967295); //@line 641 "phonemelist.cpp"
    HEAP32[(($ph616)>>2)]=$371; //@line 641 "phonemelist.cpp"
    var $374=$ix; //@line 643 "phonemelist.cpp"
    HEAP32[((_n_phoneme_list)>>2)]=$374; //@line 643 "phonemelist.cpp"
    STACKTOP = __stackBase__;
    return; //@line 644 "phonemelist.cpp"
    return;
  }
  

  function __ZL18SubstitutePhonemesP10TranslatorP12PHONEME_LIST($tr, $plist_out) {
    ;
    var __label__;
    var __lastLabel__ = null;
  
    var $tr_addr;
    var $plist_out_addr;
    var $ix;
    var $k;
    var $replace_flags;
    var $n_plist_out;
    var $word_end;
    var $switched_language;
    var $plist2;
    var $next;
    $tr_addr=$tr;
    $plist_out_addr=$plist_out;
    $n_plist_out=0; //@line 48 "phonemelist.cpp"
    $switched_language=0; //@line 50 "phonemelist.cpp"
    $next=0; //@line 52 "phonemelist.cpp"
    $ix=0; //@line 54 "phonemelist.cpp"
    ; //@line 54 "phonemelist.cpp"
    $for_cond$2: while(1) { 
  
      var $0=$ix; //@line 54 "phonemelist.cpp"
      var $1=HEAP32[((_n_ph_list2)>>2)]; //@line 54 "phonemelist.cpp"
      var $cmp=((($0))|0) < ((($1))|0); //@line 54 "phonemelist.cpp"
      if (!($cmp)) { __label__ = 26;break $for_cond$2; } //@line 54 "phonemelist.cpp"
  
      var $2=$n_plist_out; //@line 54 "phonemelist.cpp"
      var $cmp1=((($2))|0) < 1000; //@line 54 "phonemelist.cpp"
      if (!($cmp1)) { __label__ = 26;break $for_cond$2; }
  
      var $3=$ix; //@line 56 "phonemelist.cpp"
      var $arrayidx=((_ph_list2+$3*8)&4294967295); //@line 56 "phonemelist.cpp"
      $plist2=$arrayidx; //@line 56 "phonemelist.cpp"
      var $4=$plist2; //@line 58 "phonemelist.cpp"
      var $phcode=(($4)&4294967295); //@line 58 "phonemelist.cpp"
      var $5=HEAPU8[($phcode)]; //@line 58 "phonemelist.cpp"
      var $conv=((($5))&255); //@line 58 "phonemelist.cpp"
      var $cmp2=((($conv))|0)==21; //@line 58 "phonemelist.cpp"
      var $6=$switched_language; //@line 59 "phonemelist.cpp"
      if ($cmp2) { __lastLabel__ = 3; __label__ = 4;; } else { __lastLabel__ = 3; __label__ = 5;; } //@line 58 "phonemelist.cpp"
      if (__label__ == 4) {
  
        var $xor=($6) ^ 1; //@line 59 "phonemelist.cpp"
        $switched_language=$xor; //@line 59 "phonemelist.cpp"
        __lastLabel__ = 4; ; //@line 59 "phonemelist.cpp"
      }
  
      var $7=__lastLabel__ == 4 ? $xor : ($6); //@line 62 "phonemelist.cpp"
      var $cmp3=((($7))|0)==0; //@line 62 "phonemelist.cpp"
      if ($cmp3) { __label__ = 6;; } else { __label__ = 24;; } //@line 62 "phonemelist.cpp"
      $if_then4$$if_end61$9: do { 
        if (__label__ == 6) {
  
          var $8=$ix; //@line 64 "phonemelist.cpp"
          var $9=HEAP32[((_n_ph_list2)>>2)]; //@line 64 "phonemelist.cpp"
          var $sub=((($9)-1)&4294967295); //@line 64 "phonemelist.cpp"
          var $cmp5=((($8))|0) < ((($sub))|0); //@line 64 "phonemelist.cpp"
          if ($cmp5) { __label__ = 7;; } else { __label__ = 8;; } //@line 64 "phonemelist.cpp"
          if (__label__ == 7) {
  
            var $10=$ix; //@line 65 "phonemelist.cpp"
            var $add=((($10)+1)&4294967295); //@line 65 "phonemelist.cpp"
            var $arrayidx7=((_ph_list2+$add*8)&4294967295); //@line 65 "phonemelist.cpp"
            var $phcode8=(($arrayidx7)&4294967295); //@line 65 "phonemelist.cpp"
            var $11=HEAPU8[($phcode8)]; //@line 65 "phonemelist.cpp"
            var $idxprom=((($11))&255); //@line 65 "phonemelist.cpp"
            var $arrayidx9=((_phoneme_tab+$idxprom*4)&4294967295); //@line 65 "phonemelist.cpp"
            var $12=HEAP32[(($arrayidx9)>>2)]; //@line 65 "phonemelist.cpp"
            $next=$12; //@line 65 "phonemelist.cpp"
            ; //@line 65 "phonemelist.cpp"
          }
  
          $word_end=0; //@line 67 "phonemelist.cpp"
          var $13=$plist2; //@line 68 "phonemelist.cpp"
          var $add_ptr=(($13+8)&4294967295); //@line 68 "phonemelist.cpp"
          var $sourceix=(($add_ptr+6)&4294967295); //@line 68 "phonemelist.cpp"
          var $14=HEAP16[(($sourceix)>>1)]; //@line 68 "phonemelist.cpp"
          var $tobool=(tempInt=(($14)),(tempInt>=32768?tempInt-65536:tempInt))!=0; //@line 68 "phonemelist.cpp"
          if ($tobool) { __label__ = 11;; } else { __label__ = 9;; } //@line 68 "phonemelist.cpp"
          $if_then14$$lor_lhs_false$14: do { 
            if (__label__ == 9) {
  
              var $15=$next; //@line 68 "phonemelist.cpp"
              var $cmp11=((($15))|0)!=0; //@line 68 "phonemelist.cpp"
              if (!($cmp11)) { __label__ = 12;break $if_then14$$lor_lhs_false$14; } //@line 68 "phonemelist.cpp"
  
              var $16=$next; //@line 68 "phonemelist.cpp"
              var $type=(($16+11)&4294967295); //@line 68 "phonemelist.cpp"
              var $17=HEAPU8[($type)]; //@line 68 "phonemelist.cpp"
              var $conv12=((($17))&255); //@line 68 "phonemelist.cpp"
              var $cmp13=((($conv12))|0)==0; //@line 68 "phonemelist.cpp"
              if ($cmp13) { __label__ = 11;break $if_then14$$lor_lhs_false$14; } else { __label__ = 12;break $if_then14$$lor_lhs_false$14; } //@line 68 "phonemelist.cpp"
            }
          } while(0);
          if (__label__ == 11) {
  
            $word_end=1; //@line 69 "phonemelist.cpp"
            ; //@line 69 "phonemelist.cpp"
          }
  
          $k=0; //@line 72 "phonemelist.cpp"
          ; //@line 72 "phonemelist.cpp"
          $for_cond16$20: while(1) { 
  
            var $18=$k; //@line 72 "phonemelist.cpp"
            var $19=HEAP32[((_n_replace_phonemes)>>2)]; //@line 72 "phonemelist.cpp"
            var $cmp17=((($18))|0) < ((($19))|0); //@line 72 "phonemelist.cpp"
            if (!($cmp17)) { __label__ = 23;break $for_cond16$20; } //@line 72 "phonemelist.cpp"
  
            var $20=$plist2; //@line 74 "phonemelist.cpp"
            var $phcode19=(($20)&4294967295); //@line 74 "phonemelist.cpp"
            var $21=HEAPU8[($phcode19)]; //@line 74 "phonemelist.cpp"
            var $conv20=((($21))&255); //@line 74 "phonemelist.cpp"
            var $22=$k; //@line 74 "phonemelist.cpp"
            var $arrayidx21=((_replace_phonemes+$22*3)&4294967295); //@line 74 "phonemelist.cpp"
            var $old_ph=(($arrayidx21)&4294967295); //@line 74 "phonemelist.cpp"
            var $23=HEAPU8[($old_ph)]; //@line 74 "phonemelist.cpp"
            var $conv22=((($23))&255); //@line 74 "phonemelist.cpp"
            var $cmp23=((($conv20))|0)==((($conv22))|0); //@line 74 "phonemelist.cpp"
            if ($cmp23) { __label__ = 15;; } else { __label__ = 22;; } //@line 74 "phonemelist.cpp"
            $if_then24$$for_inc$23: do { 
              if (__label__ == 15) {
  
                var $24=$k; //@line 76 "phonemelist.cpp"
                var $arrayidx25=((_replace_phonemes+$24*3)&4294967295); //@line 76 "phonemelist.cpp"
                var $type26=(($arrayidx25+2)&4294967295); //@line 76 "phonemelist.cpp"
                var $25=HEAP8[($type26)]; //@line 76 "phonemelist.cpp"
                var $conv27=(tempInt=(($25)),(tempInt>=128?tempInt-256:tempInt)); //@line 76 "phonemelist.cpp"
                $replace_flags=$conv27; //@line 76 "phonemelist.cpp"
                var $26=$replace_flags; //@line 78 "phonemelist.cpp"
                var $and=($26) & 1; //@line 78 "phonemelist.cpp"
                var $tobool28=((($and))|0)!=0; //@line 78 "phonemelist.cpp"
                if ($tobool28) { __label__ = 16;; } else { __label__ = 17;; } //@line 78 "phonemelist.cpp"
                if (__label__ == 16) {
  
                  var $27=$word_end; //@line 78 "phonemelist.cpp"
                  var $cmp30=((($27))|0)==0; //@line 78 "phonemelist.cpp"
                  if ($cmp30) { __label__ = 22;break $if_then24$$for_inc$23; } //@line 78 "phonemelist.cpp"
                }
  
                var $28=$replace_flags; //@line 81 "phonemelist.cpp"
                var $and33=($28) & 2; //@line 81 "phonemelist.cpp"
                var $tobool34=((($and33))|0)!=0; //@line 81 "phonemelist.cpp"
                if (!($tobool34)) { __label__ = 19;break $for_cond16$20; } //@line 81 "phonemelist.cpp"
  
                var $29=$plist2; //@line 81 "phonemelist.cpp"
                var $stresslevel=(($29+1)&4294967295); //@line 81 "phonemelist.cpp"
                var $30=HEAPU8[($stresslevel)]; //@line 81 "phonemelist.cpp"
                var $conv36=((($30))&255); //@line 81 "phonemelist.cpp"
                var $and37=($conv36) & 7; //@line 81 "phonemelist.cpp"
                var $cmp38=((($and37))|0) > 3; //@line 81 "phonemelist.cpp"
                if (!($cmp38)) { __label__ = 19;break $for_cond16$20; } //@line 81 "phonemelist.cpp"
              }
            } while(0);
  
            var $41=$k; //@line 72 "phonemelist.cpp"
            var $inc=((($41)+1)&4294967295); //@line 72 "phonemelist.cpp"
            $k=$inc; //@line 72 "phonemelist.cpp"
            __label__ = 13;continue $for_cond16$20; //@line 72 "phonemelist.cpp"
          }
          $for_end$$if_end40$30: do { 
            if (__label__ == 19) {
  
              var $31=$k; //@line 85 "phonemelist.cpp"
              var $arrayidx41=((_replace_phonemes+$31*3)&4294967295); //@line 85 "phonemelist.cpp"
              var $new_ph=(($arrayidx41+1)&4294967295); //@line 85 "phonemelist.cpp"
              var $32=HEAP8[($new_ph)]; //@line 85 "phonemelist.cpp"
              var $33=$plist2; //@line 85 "phonemelist.cpp"
              var $phcode42=(($33)&4294967295); //@line 85 "phonemelist.cpp"
              HEAP8[($phcode42)]=$32; //@line 85 "phonemelist.cpp"
              var $34=$plist2; //@line 86 "phonemelist.cpp"
              var $stresslevel43=(($34+1)&4294967295); //@line 86 "phonemelist.cpp"
              var $35=HEAPU8[($stresslevel43)]; //@line 86 "phonemelist.cpp"
              var $conv44=((($35))&255); //@line 86 "phonemelist.cpp"
              var $cmp45=((($conv44))|0) > 1; //@line 86 "phonemelist.cpp"
              if (!($cmp45)) { __label__ = 23;break $for_end$$if_end40$30; } //@line 86 "phonemelist.cpp"
  
              var $36=$plist2; //@line 86 "phonemelist.cpp"
              var $phcode47=(($36)&4294967295); //@line 86 "phonemelist.cpp"
              var $37=HEAPU8[($phcode47)]; //@line 86 "phonemelist.cpp"
              var $idxprom48=((($37))&255); //@line 86 "phonemelist.cpp"
              var $arrayidx49=((_phoneme_tab+$idxprom48*4)&4294967295); //@line 86 "phonemelist.cpp"
              var $38=HEAP32[(($arrayidx49)>>2)]; //@line 86 "phonemelist.cpp"
              var $phflags=(($38+4)&4294967295); //@line 86 "phonemelist.cpp"
              var $39=HEAP32[(($phflags)>>2)]; //@line 86 "phonemelist.cpp"
              var $and50=($39) & 2; //@line 86 "phonemelist.cpp"
              var $tobool51=((($and50))|0)!=0; //@line 86 "phonemelist.cpp"
              if (!($tobool51)) { __label__ = 23;break $for_end$$if_end40$30; } //@line 86 "phonemelist.cpp"
  
              var $40=$plist2; //@line 87 "phonemelist.cpp"
              var $stresslevel53=(($40+1)&4294967295); //@line 87 "phonemelist.cpp"
              HEAP8[($stresslevel53)]=0; //@line 87 "phonemelist.cpp"
              ; //@line 87 "phonemelist.cpp"
            }
          } while(0);
  
          var $42=$plist2; //@line 92 "phonemelist.cpp"
          var $phcode56=(($42)&4294967295); //@line 92 "phonemelist.cpp"
          var $43=HEAPU8[($phcode56)]; //@line 92 "phonemelist.cpp"
          var $conv57=((($43))&255); //@line 92 "phonemelist.cpp"
          var $cmp58=((($conv57))|0)==0; //@line 92 "phonemelist.cpp"
          if ($cmp58) { __label__ = 25;break $if_then4$$if_end61$9; } else { __label__ = 24;break $if_then4$$if_end61$9; } //@line 92 "phonemelist.cpp"
        }
      } while(0);
      if (__label__ == 24) {
  
        var $44=$n_plist_out; //@line 99 "phonemelist.cpp"
        var $45=$plist_out_addr; //@line 99 "phonemelist.cpp"
        var $arrayidx62=(($45+24*$44)&4294967295); //@line 99 "phonemelist.cpp"
        var $46=$arrayidx62; //@line 99 "phonemelist.cpp"
        var $47=$plist2; //@line 99 "phonemelist.cpp"
        var $48=$47; //@line 99 "phonemelist.cpp"
        var $src$, $dest$, $stop$, $stop4$, $fast$;
        $src$ = $48;
        $dest$ = $46;
        $stop$ = $src$ + 8;
        if (($dest$%4) == ($src$%4) && 8 > 8) {
          while ($src$%4 !== 0 && $src$ < $stop$) {
            ; HEAP8[$dest$++] = HEAP8[$src$++];
          }
          $src$ >>= 2;
          $dest$ >>= 2;
          $stop4$ = $stop$ >> 2;
          while ($src$ < $stop4$) {
        ;;;    HEAP32[$dest$++] = HEAP32[$src$++];
          }
          $src$ <<= 2;
          $dest$ <<= 2;
        }while ($src$ < $stop$) {
          ; HEAP8[$dest$++] = HEAP8[$src$++];
        }; //@line 99 "phonemelist.cpp"
        var $49=$plist2; //@line 100 "phonemelist.cpp"
        var $phcode63=(($49)&4294967295); //@line 100 "phonemelist.cpp"
        var $50=HEAPU8[($phcode63)]; //@line 100 "phonemelist.cpp"
        var $idxprom64=((($50))&255); //@line 100 "phonemelist.cpp"
        var $arrayidx65=((_phoneme_tab+$idxprom64*4)&4294967295); //@line 100 "phonemelist.cpp"
        var $51=HEAP32[(($arrayidx65)>>2)]; //@line 100 "phonemelist.cpp"
        var $52=$n_plist_out; //@line 100 "phonemelist.cpp"
        var $53=$plist_out_addr; //@line 100 "phonemelist.cpp"
        var $arrayidx66=(($53+24*$52)&4294967295); //@line 100 "phonemelist.cpp"
        var $ph=(($arrayidx66+8)&4294967295); //@line 100 "phonemelist.cpp"
        HEAP32[(($ph)>>2)]=$51; //@line 100 "phonemelist.cpp"
        var $54=$n_plist_out; //@line 101 "phonemelist.cpp"
        var $55=$plist_out_addr; //@line 101 "phonemelist.cpp"
        var $arrayidx67=(($55+24*$54)&4294967295); //@line 101 "phonemelist.cpp"
        var $ph68=(($arrayidx67+8)&4294967295); //@line 101 "phonemelist.cpp"
        var $56=HEAP32[(($ph68)>>2)]; //@line 101 "phonemelist.cpp"
        var $type69=(($56+11)&4294967295); //@line 101 "phonemelist.cpp"
        var $57=HEAP8[($type69)]; //@line 101 "phonemelist.cpp"
        var $58=$n_plist_out; //@line 101 "phonemelist.cpp"
        var $59=$plist_out_addr; //@line 101 "phonemelist.cpp"
        var $arrayidx70=(($59+24*$58)&4294967295); //@line 101 "phonemelist.cpp"
        var $type71=(($arrayidx70+15)&4294967295); //@line 101 "phonemelist.cpp"
        HEAP8[($type71)]=$57; //@line 101 "phonemelist.cpp"
        var $60=$n_plist_out; //@line 102 "phonemelist.cpp"
        var $inc72=((($60)+1)&4294967295); //@line 102 "phonemelist.cpp"
        $n_plist_out=$inc72; //@line 102 "phonemelist.cpp"
        ; //@line 103 "phonemelist.cpp"
      }
  
      var $61=$ix; //@line 54 "phonemelist.cpp"
      var $inc74=((($61)+1)&4294967295); //@line 54 "phonemelist.cpp"
      $ix=$inc74; //@line 54 "phonemelist.cpp"
      __label__ = 1;continue $for_cond$2; //@line 54 "phonemelist.cpp"
    }
  
    var $62=$n_plist_out; //@line 104 "phonemelist.cpp"
    ;
    return $62; //@line 104 "phonemelist.cpp"
    return null;
  }
  

  function __Z10KlattReseti($control) {
    ;
    var __label__;
    var __lastLabel__ = null;
  
    var $control_addr;
    var $r_ix;
    $control_addr=$control;
    var $0=$control_addr; //@line 499 "klatt.cpp"
    var $cmp=((($0))|0)==2; //@line 499 "klatt.cpp"
    if ($cmp) { __lastLabel__ = 0; __label__ = 1;; } else { __lastLabel__ = 0; __label__ = 2;; } //@line 499 "klatt.cpp"
    if (__label__ == 1) {
  
      var $1=HEAP32[((((__ZL10kt_globals+8)&4294967295))>>2)]; //@line 502 "klatt.cpp"
      var $mul=((($1)*950)&4294967295); //@line 502 "klatt.cpp"
      var $div=((((($mul))|0)/10000)|0); //@line 502 "klatt.cpp"
      HEAP32[((((__ZL10kt_globals+12)&4294967295))>>2)]=$div; //@line 502 "klatt.cpp"
      var $2=HEAP32[((((__ZL10kt_globals+8)&4294967295))>>2)]; //@line 503 "klatt.cpp"
      var $mul1=((($2)*630)&4294967295); //@line 503 "klatt.cpp"
      var $div2=((((($mul1))|0)/10000)|0); //@line 503 "klatt.cpp"
      HEAP32[((((__ZL10kt_globals+16)&4294967295))>>2)]=$div2; //@line 503 "klatt.cpp"
      var $3=HEAP32[((((__ZL10kt_globals+8)&4294967295))>>2)]; //@line 504 "klatt.cpp"
      var $conv=((($3))|0); //@line 504 "klatt.cpp"
      var $div3=-3.1415927/($conv); //@line 504 "klatt.cpp"
      HEAPF32[((((__ZL10kt_globals+72)&4294967295))>>2)]=$div3; //@line 504 "klatt.cpp"
      var $4=HEAPF32[((((__ZL10kt_globals+72)&4294967295))>>2)]; //@line 505 "klatt.cpp"
      var $mul4=-2*($4); //@line 505 "klatt.cpp"
      HEAPF32[((((__ZL10kt_globals+80)&4294967295))>>2)]=$mul4; //@line 505 "klatt.cpp"
      var $5=HEAP32[((((__ZL10kt_globals+12)&4294967295))>>2)]; //@line 506 "klatt.cpp"
      var $6=HEAP32[((((__ZL10kt_globals+16)&4294967295))>>2)]; //@line 506 "klatt.cpp"
      __ZL6setabcllP11resonator_t($5, $6, ((__ZL10kt_globals+188+1152)&4294967295)); //@line 506 "klatt.cpp"
      var $_pr=$control_addr; //@line 510 "klatt.cpp"
      __lastLabel__ = 1; ; //@line 508 "klatt.cpp"
    }
  
    var $7=__lastLabel__ == 1 ? $_pr : ($0); //@line 510 "klatt.cpp"
    var $cmp5=((($7))|0) > 0; //@line 510 "klatt.cpp"
    if ($cmp5) { __label__ = 3;; } else { __label__ = 5;; } //@line 510 "klatt.cpp"
    $if_then6$$if_end9$5: do { 
      if (__label__ == 3) {
  
        HEAP32[((((__ZL10kt_globals+32)&4294967295))>>2)]=0; //@line 512 "klatt.cpp"
        HEAP32[((((__ZL10kt_globals+40)&4294967295))>>2)]=0; //@line 513 "klatt.cpp"
        HEAP32[((((__ZL10kt_globals+44)&4294967295))>>2)]=0; //@line 514 "klatt.cpp"
        HEAP32[((((__ZL10kt_globals+48)&4294967295))>>2)]=0; //@line 515 "klatt.cpp"
        $r_ix=17; //@line 517 "klatt.cpp"
        ; //@line 517 "klatt.cpp"
        while(1) { 
  
          var $8=$r_ix; //@line 519 "klatt.cpp"
          var $arrayidx=((((__ZL10kt_globals+188)&4294967295)+$8*64)&4294967295); //@line 519 "klatt.cpp"
          var $p1=(($arrayidx+24)&4294967295); //@line 519 "klatt.cpp"
          HEAPF32[(($p1)>>2)]=0; //@line 519 "klatt.cpp"
          var $9=$r_ix; //@line 520 "klatt.cpp"
          var $arrayidx8=((((__ZL10kt_globals+188)&4294967295)+$9*64)&4294967295); //@line 520 "klatt.cpp"
          var $p2=(($arrayidx8+32)&4294967295); //@line 520 "klatt.cpp"
          HEAPF32[(($p2)>>2)]=0; //@line 520 "klatt.cpp"
          var $10=$r_ix; //@line 517 "klatt.cpp"
          var $inc=((($10)+1)&4294967295); //@line 517 "klatt.cpp"
          $r_ix=$inc; //@line 517 "klatt.cpp"
          var $cmp7=((($inc))|0) < 20; //@line 517 "klatt.cpp"
          if ($cmp7) { __label__ = 4;continue ; } else { __label__ = 5;break $if_then6$$if_end9$5; } //@line 517 "klatt.cpp"
        }
      }
    } while(0);
  
    $r_ix=0; //@line 525 "klatt.cpp"
    ; //@line 525 "klatt.cpp"
    while(1) { 
  
      var $11=$r_ix; //@line 527 "klatt.cpp"
      var $arrayidx13=((((__ZL10kt_globals+188)&4294967295)+$11*64)&4294967295); //@line 527 "klatt.cpp"
      var $p114=(($arrayidx13+24)&4294967295); //@line 527 "klatt.cpp"
      HEAPF32[(($p114)>>2)]=0; //@line 527 "klatt.cpp"
      var $12=$r_ix; //@line 528 "klatt.cpp"
      var $arrayidx15=((((__ZL10kt_globals+188)&4294967295)+$12*64)&4294967295); //@line 528 "klatt.cpp"
      var $p216=(($arrayidx15+32)&4294967295); //@line 528 "klatt.cpp"
      HEAPF32[(($p216)>>2)]=0; //@line 528 "klatt.cpp"
      var $13=$r_ix; //@line 525 "klatt.cpp"
      var $inc18=((($13)+1)&4294967295); //@line 525 "klatt.cpp"
      $r_ix=$inc18; //@line 525 "klatt.cpp"
      var $cmp11=((($inc18))|0) <= 16; //@line 525 "klatt.cpp"
      if ($cmp11) { __label__ = 6;continue ; } else { __label__ = 7;break ; } //@line 525 "klatt.cpp"
    }
  
    ;
    return; //@line 530 "klatt.cpp"
    return;
  }
  

  function __ZL6setabcllP11resonator_t($f, $bw, $rp) {
    ;
    var __label__;
  
    var $f_addr;
    var $bw_addr;
    var $rp_addr;
    var $r;
    var $arg;
    $f_addr=$f;
    $bw_addr=$bw;
    $rp_addr=$rp;
    var $0=HEAPF32[((((__ZL10kt_globals+72)&4294967295))>>2)]; //@line 858 "klatt.cpp"
    var $1=$bw_addr; //@line 858 "klatt.cpp"
    var $conv=((($1))|0); //@line 858 "klatt.cpp"
    var $mul=($0)*($conv); //@line 858 "klatt.cpp"
    $arg=$mul; //@line 858 "klatt.cpp"
    var $2=$arg; //@line 859 "klatt.cpp"
    var $call=_exp($2); //@line 859 "klatt.cpp"
    $r=$call; //@line 859 "klatt.cpp"
    var $3=$r; //@line 862 "klatt.cpp"
    var $4=$r; //@line 862 "klatt.cpp"
    var $mul1=($3)*($4); //@line 862 "klatt.cpp"
    var $sub=(-($mul1)); //@line 862 "klatt.cpp"
    var $5=$rp_addr; //@line 862 "klatt.cpp"
    var $c=(($5+16)&4294967295); //@line 862 "klatt.cpp"
    HEAPF32[(($c)>>2)]=$sub; //@line 862 "klatt.cpp"
    var $6=HEAPF32[((((__ZL10kt_globals+80)&4294967295))>>2)]; //@line 865 "klatt.cpp"
    var $7=$f_addr; //@line 865 "klatt.cpp"
    var $conv2=((($7))|0); //@line 865 "klatt.cpp"
    var $mul3=($6)*($conv2); //@line 865 "klatt.cpp"
    $arg=$mul3; //@line 865 "klatt.cpp"
    var $8=$r; //@line 866 "klatt.cpp"
    var $9=$arg; //@line 866 "klatt.cpp"
    var $call4=_cos($9); //@line 866 "klatt.cpp"
    var $mul5=($8)*($call4); //@line 866 "klatt.cpp"
    var $mul6=($mul5)*2; //@line 866 "klatt.cpp"
    var $10=$rp_addr; //@line 866 "klatt.cpp"
    var $b=(($10+8)&4294967295); //@line 866 "klatt.cpp"
    HEAPF32[(($b)>>2)]=$mul6; //@line 866 "klatt.cpp"
    var $11=$rp_addr; //@line 869 "klatt.cpp"
    var $b7=(($11+8)&4294967295); //@line 869 "klatt.cpp"
    var $12=HEAPF32[(($b7)>>2)]; //@line 869 "klatt.cpp"
    var $sub8=1-($12); //@line 869 "klatt.cpp"
    var $13=$rp_addr; //@line 869 "klatt.cpp"
    var $c9=(($13+16)&4294967295); //@line 869 "klatt.cpp"
    var $14=HEAPF32[(($c9)>>2)]; //@line 869 "klatt.cpp"
    var $sub10=($sub8)-($14); //@line 869 "klatt.cpp"
    var $15=$rp_addr; //@line 869 "klatt.cpp"
    var $a=(($15)&4294967295); //@line 869 "klatt.cpp"
    HEAPF32[(($a)>>2)]=$sub10; //@line 869 "klatt.cpp"
    ;
    return; //@line 870 "klatt.cpp"
    return;
  }
  

  function __Z13Wavegen_Klatti($resume) {
    ;
    var __label__;
  
    var $retval;
    var $resume_addr;
    var $pk;
    var $x;
    var $ix;
    var $fade;
    $resume_addr=$resume;
    var $0=$resume_addr; //@line 1001 "klatt.cpp"
    var $cmp=((($0))|0)==0; //@line 1001 "klatt.cpp"
    if ($cmp) { __label__ = 1;; } else { __label__ = 2;; } //@line 1001 "klatt.cpp"
    if (__label__ == 1) {
  
      HEAP32[((__ZL12sample_count)>>2)]=0; //@line 1003 "klatt.cpp"
      ; //@line 1004 "klatt.cpp"
    }
    $while_cond$4: while(1) { 
  
      var $1=HEAP32[((__ZL12sample_count)>>2)]; //@line 1006 "klatt.cpp"
      var $2=HEAP32[((__ZL8nsamples802)>>2)]; //@line 1006 "klatt.cpp"
      var $cmp1=((($1))|0) < ((($2))|0); //@line 1006 "klatt.cpp"
      if (!($cmp1)) { __label__ = 23;break $while_cond$4; } //@line 1006 "klatt.cpp"
  
      var $3=HEAP32[((((_wdata+4)&4294967295))>>2)]; //@line 1008 "klatt.cpp"
      var $mul=((($3)*10)&4294967295); //@line 1008 "klatt.cpp"
      var $div=((((($mul))|0)/4096)|0); //@line 1008 "klatt.cpp"
      HEAP32[((((__ZL8kt_frame)&4294967295))>>2)]=$div; //@line 1008 "klatt.cpp"
      $ix=0; //@line 1013 "klatt.cpp"
      ; //@line 1013 "klatt.cpp"
      $for_body$7: while(1) { 
  
        var $4=$ix; //@line 1015 "klatt.cpp"
        var $arrayidx=((__ZL5peaks801+$4*80)&4294967295); //@line 1015 "klatt.cpp"
        var $freq=(($arrayidx)&4294967295); //@line 1015 "klatt.cpp"
        var $5=HEAP32[(($freq)>>2)]; //@line 1015 "klatt.cpp"
        var $6=$ix; //@line 1015 "klatt.cpp"
        var $arrayidx3=((((__ZL8kt_frame+8)&4294967295)+$6*4)&4294967295); //@line 1015 "klatt.cpp"
        HEAP32[(($arrayidx3)>>2)]=$5; //@line 1015 "klatt.cpp"
        var $7=$ix; //@line 1016 "klatt.cpp"
        var $cmp4=((($7))|0) < 4; //@line 1016 "klatt.cpp"
        if ($cmp4) { __label__ = 5;; } else { __label__ = 6;; } //@line 1016 "klatt.cpp"
        if (__label__ == 5) {
  
          var $8=$ix; //@line 1018 "klatt.cpp"
          var $arrayidx6=((__ZL5peaks801+$8*80)&4294967295); //@line 1018 "klatt.cpp"
          var $bw=(($arrayidx6+4)&4294967295); //@line 1018 "klatt.cpp"
          var $9=HEAP32[(($bw)>>2)]; //@line 1018 "klatt.cpp"
          var $10=$ix; //@line 1018 "klatt.cpp"
          var $arrayidx7=((((__ZL8kt_frame+48)&4294967295)+$10*4)&4294967295); //@line 1018 "klatt.cpp"
          HEAP32[(($arrayidx7)>>2)]=$9; //@line 1018 "klatt.cpp"
          ; //@line 1019 "klatt.cpp"
        }
  
        var $11=$ix; //@line 1013 "klatt.cpp"
        var $inc=((($11)+1)&4294967295); //@line 1013 "klatt.cpp"
        $ix=$inc; //@line 1013 "klatt.cpp"
        var $cmp2=((($inc))|0) < 6; //@line 1013 "klatt.cpp"
        if ($cmp2) { __label__ = 4;continue $for_body$7; } else { __label__ = 7;break $for_body$7; } //@line 1013 "klatt.cpp"
      }
  
      $ix=1; //@line 1021 "klatt.cpp"
      ; //@line 1021 "klatt.cpp"
      $for_body11$13: while(1) { 
  
        var $12=$ix; //@line 1023 "klatt.cpp"
        var $arrayidx12=((((__ZL8kt_frame+88)&4294967295)+$12*4)&4294967295); //@line 1023 "klatt.cpp"
        HEAP32[(($arrayidx12)>>2)]=0; //@line 1023 "klatt.cpp"
        var $13=$ix; //@line 1021 "klatt.cpp"
        var $inc14=((($13)+1)&4294967295); //@line 1021 "klatt.cpp"
        $ix=$inc14; //@line 1021 "klatt.cpp"
        var $cmp10=((($inc14))|0) < 7; //@line 1021 "klatt.cpp"
        if ($cmp10) { __label__ = 8;continue $for_body11$13; } else { __label__ = 9;break $for_body11$13; } //@line 1021 "klatt.cpp"
      }
  
      var $14=HEAP32[((((__ZL6klattp)&4294967295))>>2)]; //@line 1026 "klatt.cpp"
      HEAP32[((((__ZL8kt_frame+4)&4294967295))>>2)]=$14; //@line 1026 "klatt.cpp"
      var $15=HEAP32[((((__ZL6klattp+24)&4294967295))>>2)]; //@line 1027 "klatt.cpp"
      HEAP32[((((__ZL8kt_frame+196)&4294967295))>>2)]=$15; //@line 1027 "klatt.cpp"
      var $16=HEAP32[((((__ZL6klattp+28)&4294967295))>>2)]; //@line 1028 "klatt.cpp"
      HEAP32[((((__ZL8kt_frame+184)&4294967295))>>2)]=$16; //@line 1028 "klatt.cpp"
      var $17=HEAP32[((((__ZL6klattp+32)&4294967295))>>2)]; //@line 1029 "klatt.cpp"
      HEAP32[((((__ZL8kt_frame+192)&4294967295))>>2)]=$17; //@line 1029 "klatt.cpp"
      var $18=HEAP32[((((__ZL6klattp+12)&4294967295))>>2)]; //@line 1030 "klatt.cpp"
      HEAP32[((((__ZL8kt_frame+168)&4294967295))>>2)]=$18; //@line 1030 "klatt.cpp"
      var $19=HEAP32[((((__ZL6klattp+36)&4294967295))>>2)]; //@line 1031 "klatt.cpp"
      HEAP32[((((__ZL8kt_frame+176)&4294967295))>>2)]=$19; //@line 1031 "klatt.cpp"
      var $20=HEAP32[((((__ZL6klattp+16)&4294967295))>>2)]; //@line 1032 "klatt.cpp"
      HEAP32[((((__ZL8kt_frame+188)&4294967295))>>2)]=$20; //@line 1032 "klatt.cpp"
      var $21=HEAP32[((((__ZL6klattp+8)&4294967295))>>2)]; //@line 1033 "klatt.cpp"
      HEAP32[((((__ZL8kt_frame+180)&4294967295))>>2)]=$21; //@line 1033 "klatt.cpp"
      var $22=HEAP32[((((__ZL6klattp+20)&4294967295))>>2)]; //@line 1034 "klatt.cpp"
      HEAP32[((((__ZL8kt_frame+172)&4294967295))>>2)]=$22; //@line 1034 "klatt.cpp"
      $pk=0; //@line 1037 "klatt.cpp"
      ; //@line 1037 "klatt.cpp"
      $for_body18$16: while(1) { 
  
        var $23=$pk; //@line 1039 "klatt.cpp"
        var $arrayidx19=((__ZL5peaks801+$23*80)&4294967295); //@line 1039 "klatt.cpp"
        var $freq_inc=(($arrayidx19+48)&4294967295); //@line 1039 "klatt.cpp"
        var $24=HEAPF32[(($freq_inc)>>2)]; //@line 1039 "klatt.cpp"
        var $25=$pk; //@line 1039 "klatt.cpp"
        var $arrayidx20=((__ZL5peaks801+$25*80)&4294967295); //@line 1039 "klatt.cpp"
        var $freq1=(($arrayidx20+16)&4294967295); //@line 1039 "klatt.cpp"
        var $26=HEAPF32[(($freq1)>>2)]; //@line 1039 "klatt.cpp"
        var $add=($26)+($24); //@line 1039 "klatt.cpp"
        HEAPF32[(($freq1)>>2)]=$add; //@line 1039 "klatt.cpp"
        var $27=$pk; //@line 1040 "klatt.cpp"
        var $arrayidx21=((__ZL5peaks801+$27*80)&4294967295); //@line 1040 "klatt.cpp"
        var $freq122=(($arrayidx21+16)&4294967295); //@line 1040 "klatt.cpp"
        var $28=HEAPF32[(($freq122)>>2)]; //@line 1040 "klatt.cpp"
        var $conv=((($28))|0); //@line 1040 "klatt.cpp"
        var $29=$pk; //@line 1040 "klatt.cpp"
        var $arrayidx23=((__ZL5peaks801+$29*80)&4294967295); //@line 1040 "klatt.cpp"
        var $freq24=(($arrayidx23)&4294967295); //@line 1040 "klatt.cpp"
        HEAP32[(($freq24)>>2)]=$conv; //@line 1040 "klatt.cpp"
        var $30=$pk; //@line 1041 "klatt.cpp"
        var $arrayidx25=((__ZL5peaks801+$30*80)&4294967295); //@line 1041 "klatt.cpp"
        var $bw_inc=(($arrayidx25+56)&4294967295); //@line 1041 "klatt.cpp"
        var $31=HEAPF32[(($bw_inc)>>2)]; //@line 1041 "klatt.cpp"
        var $32=$pk; //@line 1041 "klatt.cpp"
        var $arrayidx26=((__ZL5peaks801+$32*80)&4294967295); //@line 1041 "klatt.cpp"
        var $bw1=(($arrayidx26+24)&4294967295); //@line 1041 "klatt.cpp"
        var $33=HEAPF32[(($bw1)>>2)]; //@line 1041 "klatt.cpp"
        var $add27=($33)+($31); //@line 1041 "klatt.cpp"
        HEAPF32[(($bw1)>>2)]=$add27; //@line 1041 "klatt.cpp"
        var $34=$pk; //@line 1042 "klatt.cpp"
        var $arrayidx28=((__ZL5peaks801+$34*80)&4294967295); //@line 1042 "klatt.cpp"
        var $bw129=(($arrayidx28+24)&4294967295); //@line 1042 "klatt.cpp"
        var $35=HEAPF32[(($bw129)>>2)]; //@line 1042 "klatt.cpp"
        var $conv30=((($35))|0); //@line 1042 "klatt.cpp"
        var $36=$pk; //@line 1042 "klatt.cpp"
        var $arrayidx31=((__ZL5peaks801+$36*80)&4294967295); //@line 1042 "klatt.cpp"
        var $bw32=(($arrayidx31+4)&4294967295); //@line 1042 "klatt.cpp"
        HEAP32[(($bw32)>>2)]=$conv30; //@line 1042 "klatt.cpp"
        var $37=$pk; //@line 1043 "klatt.cpp"
        var $arrayidx33=((__ZL5peaks801+$37*80)&4294967295); //@line 1043 "klatt.cpp"
        var $bp_inc=(($arrayidx33+72)&4294967295); //@line 1043 "klatt.cpp"
        var $38=HEAPF32[(($bp_inc)>>2)]; //@line 1043 "klatt.cpp"
        var $39=$pk; //@line 1043 "klatt.cpp"
        var $arrayidx34=((__ZL5peaks801+$39*80)&4294967295); //@line 1043 "klatt.cpp"
        var $bp1=(($arrayidx34+40)&4294967295); //@line 1043 "klatt.cpp"
        var $40=HEAPF32[(($bp1)>>2)]; //@line 1043 "klatt.cpp"
        var $add35=($40)+($38); //@line 1043 "klatt.cpp"
        HEAPF32[(($bp1)>>2)]=$add35; //@line 1043 "klatt.cpp"
        var $41=$pk; //@line 1044 "klatt.cpp"
        var $arrayidx36=((__ZL5peaks801+$41*80)&4294967295); //@line 1044 "klatt.cpp"
        var $bp137=(($arrayidx36+40)&4294967295); //@line 1044 "klatt.cpp"
        var $42=HEAPF32[(($bp137)>>2)]; //@line 1044 "klatt.cpp"
        var $conv38=((($42))|0); //@line 1044 "klatt.cpp"
        var $43=$pk; //@line 1044 "klatt.cpp"
        var $arrayidx39=((__ZL5peaks801+$43*80)&4294967295); //@line 1044 "klatt.cpp"
        var $bp=(($arrayidx39+12)&4294967295); //@line 1044 "klatt.cpp"
        HEAP32[(($bp)>>2)]=$conv38; //@line 1044 "klatt.cpp"
        var $44=$pk; //@line 1045 "klatt.cpp"
        var $arrayidx40=((__ZL5peaks801+$44*80)&4294967295); //@line 1045 "klatt.cpp"
        var $ap_inc=(($arrayidx40+64)&4294967295); //@line 1045 "klatt.cpp"
        var $45=HEAPF32[(($ap_inc)>>2)]; //@line 1045 "klatt.cpp"
        var $46=$pk; //@line 1045 "klatt.cpp"
        var $arrayidx41=((__ZL5peaks801+$46*80)&4294967295); //@line 1045 "klatt.cpp"
        var $ap1=(($arrayidx41+32)&4294967295); //@line 1045 "klatt.cpp"
        var $47=HEAPF32[(($ap1)>>2)]; //@line 1045 "klatt.cpp"
        var $add42=($47)+($45); //@line 1045 "klatt.cpp"
        HEAPF32[(($ap1)>>2)]=$add42; //@line 1045 "klatt.cpp"
        var $48=$pk; //@line 1046 "klatt.cpp"
        var $arrayidx43=((__ZL5peaks801+$48*80)&4294967295); //@line 1046 "klatt.cpp"
        var $ap144=(($arrayidx43+32)&4294967295); //@line 1046 "klatt.cpp"
        var $49=HEAPF32[(($ap144)>>2)]; //@line 1046 "klatt.cpp"
        var $conv45=((($49))|0); //@line 1046 "klatt.cpp"
        var $50=$pk; //@line 1046 "klatt.cpp"
        var $arrayidx46=((__ZL5peaks801+$50*80)&4294967295); //@line 1046 "klatt.cpp"
        var $ap=(($arrayidx46+8)&4294967295); //@line 1046 "klatt.cpp"
        HEAP32[(($ap)>>2)]=$conv45; //@line 1046 "klatt.cpp"
        var $51=$pk; //@line 1037 "klatt.cpp"
        var $inc48=((($51)+1)&4294967295); //@line 1037 "klatt.cpp"
        $pk=$inc48; //@line 1037 "klatt.cpp"
        var $cmp17=((($inc48))|0) < 9; //@line 1037 "klatt.cpp"
        if ($cmp17) { __label__ = 10;continue $for_body18$16; } else { __label__ = 11;break $for_body18$16; } //@line 1037 "klatt.cpp"
      }
  
      $ix=0; //@line 1050 "klatt.cpp"
      ; //@line 1050 "klatt.cpp"
      $for_body52$19: while(1) { 
  
        var $52=$ix; //@line 1052 "klatt.cpp"
        var $arrayidx53=((__ZL10klattp_inc+$52*8)&4294967295); //@line 1052 "klatt.cpp"
        var $53=HEAPF32[(($arrayidx53)>>2)]; //@line 1052 "klatt.cpp"
        var $54=$ix; //@line 1052 "klatt.cpp"
        var $arrayidx54=((__ZL7klattp1+$54*8)&4294967295); //@line 1052 "klatt.cpp"
        var $55=HEAPF32[(($arrayidx54)>>2)]; //@line 1052 "klatt.cpp"
        var $add55=($55)+($53); //@line 1052 "klatt.cpp"
        HEAPF32[(($arrayidx54)>>2)]=$add55; //@line 1052 "klatt.cpp"
        var $56=$ix; //@line 1053 "klatt.cpp"
        var $arrayidx56=((__ZL7klattp1+$56*8)&4294967295); //@line 1053 "klatt.cpp"
        var $57=HEAPF32[(($arrayidx56)>>2)]; //@line 1053 "klatt.cpp"
        var $conv57=((($57))|0); //@line 1053 "klatt.cpp"
        var $58=$ix; //@line 1053 "klatt.cpp"
        var $arrayidx58=((__ZL6klattp+$58*4)&4294967295); //@line 1053 "klatt.cpp"
        HEAP32[(($arrayidx58)>>2)]=$conv57; //@line 1053 "klatt.cpp"
        var $59=$ix; //@line 1050 "klatt.cpp"
        var $inc60=((($59)+1)&4294967295); //@line 1050 "klatt.cpp"
        $ix=$inc60; //@line 1050 "klatt.cpp"
        var $cmp51=((($inc60))|0) < 10; //@line 1050 "klatt.cpp"
        if ($cmp51) { __label__ = 12;continue $for_body52$19; } else { __label__ = 13;break $for_body52$19; } //@line 1050 "klatt.cpp"
      }
  
      $ix=0; //@line 1056 "klatt.cpp"
      ; //@line 1056 "klatt.cpp"
      while(1) { 
  
        var $60=$ix; //@line 1058 "klatt.cpp"
        var $arrayidx65=((__ZL5peaks801+$60*80)&4294967295); //@line 1058 "klatt.cpp"
        var $freq66=(($arrayidx65)&4294967295); //@line 1058 "klatt.cpp"
        var $61=HEAP32[(($freq66)>>2)]; //@line 1058 "klatt.cpp"
        var $62=$ix; //@line 1058 "klatt.cpp"
        var $arrayidx67=((((__ZL8kt_frame+208)&4294967295)+$62*4)&4294967295); //@line 1058 "klatt.cpp"
        HEAP32[(($arrayidx67)>>2)]=$61; //@line 1058 "klatt.cpp"
        var $63=$ix; //@line 1059 "klatt.cpp"
        var $cmp68=((($63))|0) < 4; //@line 1059 "klatt.cpp"
        if ($cmp68) { __label__ = 15;; } else { __label__ = 16;; } //@line 1059 "klatt.cpp"
        if (__label__ == 15) {
  
          var $64=$ix; //@line 1061 "klatt.cpp"
          var $arrayidx70=((__ZL5peaks801+$64*80)&4294967295); //@line 1061 "klatt.cpp"
          var $bw71=(($arrayidx70+4)&4294967295); //@line 1061 "klatt.cpp"
          var $65=HEAP32[(($bw71)>>2)]; //@line 1061 "klatt.cpp"
          var $66=$ix; //@line 1061 "klatt.cpp"
          var $arrayidx72=((((__ZL8kt_frame+248)&4294967295)+$66*4)&4294967295); //@line 1061 "klatt.cpp"
          HEAP32[(($arrayidx72)>>2)]=$65; //@line 1061 "klatt.cpp"
          ; //@line 1062 "klatt.cpp"
        }
  
        var $67=$ix; //@line 1056 "klatt.cpp"
        var $inc75=((($67)+1)&4294967295); //@line 1056 "klatt.cpp"
        $ix=$inc75; //@line 1056 "klatt.cpp"
        var $cmp63=((($inc75))|0) <= 6; //@line 1056 "klatt.cpp"
        if ($cmp63) { __label__ = 14;continue ; } else { __label__ = 17;break ; } //@line 1056 "klatt.cpp"
      }
  
      var $68=HEAP32[((((_wdata+12)&4294967295))>>2)]; //@line 1066 "klatt.cpp"
      var $69=HEAP32[((((_wdata+8)&4294967295))>>2)]; //@line 1066 "klatt.cpp"
      var $add77=((($69)+($68))&4294967295); //@line 1066 "klatt.cpp"
      HEAP32[((((_wdata+8)&4294967295))>>2)]=$add77; //@line 1066 "klatt.cpp"
      var $70=HEAP32[((((_wdata+8)&4294967295))>>2)]; //@line 1067 "klatt.cpp"
      var $shr=($70) >> 8; //@line 1067 "klatt.cpp"
      $ix=$shr; //@line 1067 "klatt.cpp"
      var $cmp78=((($shr))|0) > 127; //@line 1067 "klatt.cpp"
      if ($cmp78) { __label__ = 18;; } else { __label__ = 19;; } //@line 1067 "klatt.cpp"
      if (__label__ == 18) {
  
        $ix=127; //@line 1067 "klatt.cpp"
        ; //@line 1067 "klatt.cpp"
      }
  
      var $71=$ix; //@line 1068 "klatt.cpp"
      var $72=HEAP32[((((_wdata)&4294967295))>>2)]; //@line 1068 "klatt.cpp"
      var $arrayidx81=(($72+$71)&4294967295); //@line 1068 "klatt.cpp"
      var $73=HEAPU8[($arrayidx81)]; //@line 1068 "klatt.cpp"
      var $conv82=((($73))&255); //@line 1068 "klatt.cpp"
      var $74=HEAP32[((((_wdata+20)&4294967295))>>2)]; //@line 1068 "klatt.cpp"
      var $mul83=((($conv82)*($74))&4294967295); //@line 1068 "klatt.cpp"
      $x=$mul83; //@line 1068 "klatt.cpp"
      var $75=$x; //@line 1069 "klatt.cpp"
      var $shr84=($75) >> 8; //@line 1069 "klatt.cpp"
      var $76=HEAP32[((((_wdata+16)&4294967295))>>2)]; //@line 1069 "klatt.cpp"
      var $add85=((($shr84)+($76))&4294967295); //@line 1069 "klatt.cpp"
      HEAP32[((((_wdata+4)&4294967295))>>2)]=$add85; //@line 1069 "klatt.cpp"
      var $77=HEAP32[((__ZL8nsamples802)>>2)]; //@line 1071 "klatt.cpp"
      var $78=HEAP32[((__ZL12sample_count)>>2)]; //@line 1071 "klatt.cpp"
      var $sub=((($77)-($78))&4294967295); //@line 1071 "klatt.cpp"
      HEAP32[((((__ZL10kt_globals+28)&4294967295))>>2)]=$sub; //@line 1071 "klatt.cpp"
      var $cmp86=((($sub))|0) > 64; //@line 1072 "klatt.cpp"
      if ($cmp86) { __label__ = 20;; } else { __label__ = 21;; } //@line 1072 "klatt.cpp"
      if (__label__ == 20) {
  
        HEAP32[((((__ZL10kt_globals+28)&4294967295))>>2)]=64; //@line 1073 "klatt.cpp"
        ; //@line 1073 "klatt.cpp"
      }
  
      __ZL10frame_initP13klatt_frame_t(); //@line 1075 "klatt.cpp"
      var $call=__ZL7parwaveP13klatt_frame_t(); //@line 1077 "klatt.cpp"
      var $cmp89=((($call))|0)==1; //@line 1077 "klatt.cpp"
      if ($cmp89) { __label__ = 22;break $while_cond$4; } else { __label__ = 2;continue $while_cond$4; } //@line 1077 "klatt.cpp"
    }
    $while_end$$if_then90$34: do { 
      if (__label__ == 23) {
  
        var $79=HEAP32[((__ZL8end_wave803)>>2)]; //@line 1083 "klatt.cpp"
        var $cmp92=((($79))|0) > 0; //@line 1083 "klatt.cpp"
        if ($cmp92) { __label__ = 24;; } else { __label__ = 26;; } //@line 1083 "klatt.cpp"
        $if_then93$$if_end99$36: do { 
          if (__label__ == 24) {
  
            $fade=64; //@line 1095 "klatt.cpp"
            var $80=$fade; //@line 1099 "klatt.cpp"
            HEAP32[((((__ZL10kt_globals+180)&4294967295))>>2)]=$80; //@line 1099 "klatt.cpp"
            HEAP32[((__ZL8end_wave803)>>2)]=0; //@line 1100 "klatt.cpp"
            var $81=$fade; //@line 1101 "klatt.cpp"
            var $82=HEAP32[((__ZL12sample_count)>>2)]; //@line 1101 "klatt.cpp"
            var $sub94=((($82)-($81))&4294967295); //@line 1101 "klatt.cpp"
            HEAP32[((__ZL12sample_count)>>2)]=$sub94; //@line 1101 "klatt.cpp"
            var $83=$fade; //@line 1102 "klatt.cpp"
            HEAP32[((((__ZL10kt_globals+28)&4294967295))>>2)]=$83; //@line 1102 "klatt.cpp"
            var $call95=__ZL7parwaveP13klatt_frame_t(); //@line 1103 "klatt.cpp"
            var $cmp96=((($call95))|0)==1; //@line 1103 "klatt.cpp"
            if (!($cmp96)) { __label__ = 26;break $if_then93$$if_end99$36; } //@line 1103 "klatt.cpp"
  
            $retval=1; //@line 1105 "klatt.cpp"
            __label__ = 27;break $while_end$$if_then90$34; //@line 1105 "klatt.cpp"
          }
        } while(0);
  
        $retval=0; //@line 1109 "klatt.cpp"
        ; //@line 1109 "klatt.cpp"
      }
      else if (__label__ == 22) {
  
        $retval=1; //@line 1079 "klatt.cpp"
        ; //@line 1079 "klatt.cpp"
      }
    } while(0);
  
    var $84=$retval; //@line 1110 "klatt.cpp"
    ;
    return $84; //@line 1110 "klatt.cpp"
    return null;
  }
  

  function __ZL10frame_initP13klatt_frame_t() {
    var __stackBase__  = STACKTOP; STACKTOP += 56; _memset(__stackBase__, 0, 56);
    var __label__;
  
    var $frame_addr;
    var $amp_par=__stackBase__;
    var $Gain0_tmp;
    var $ix;
    $frame_addr=__ZL8kt_frame;
    var $0=$frame_addr; //@line 546 "klatt.cpp"
    var $F0hz10=(($0)&4294967295); //@line 546 "klatt.cpp"
    var $1=HEAP32[(($F0hz10)>>2)]; //@line 546 "klatt.cpp"
    var $div=((((($1))|0)/10)|0); //@line 546 "klatt.cpp"
    HEAP32[((((__ZL10kt_globals+176)&4294967295))>>2)]=$div; //@line 546 "klatt.cpp"
    var $2=$frame_addr; //@line 548 "klatt.cpp"
    var $AVdb=(($2+4)&4294967295); //@line 548 "klatt.cpp"
    var $3=HEAP32[(($AVdb)>>2)]; //@line 548 "klatt.cpp"
    var $sub=((($3)-7)&4294967295); //@line 548 "klatt.cpp"
    var $4=$frame_addr; //@line 548 "klatt.cpp"
    var $AVdb_tmp=(($4+204)&4294967295); //@line 548 "klatt.cpp"
    HEAP32[(($AVdb_tmp)>>2)]=$sub; //@line 548 "klatt.cpp"
    var $5=$frame_addr; //@line 549 "klatt.cpp"
    var $AVdb_tmp1=(($5+204)&4294967295); //@line 549 "klatt.cpp"
    var $6=HEAP32[(($AVdb_tmp1)>>2)]; //@line 549 "klatt.cpp"
    var $cmp=((($6))|0) < 0; //@line 549 "klatt.cpp"
    if ($cmp) { __label__ = 1;; } else { __label__ = 2;; } //@line 549 "klatt.cpp"
    if (__label__ == 1) {
  
      var $7=$frame_addr; //@line 551 "klatt.cpp"
      var $AVdb_tmp2=(($7+204)&4294967295); //@line 551 "klatt.cpp"
      HEAP32[(($AVdb_tmp2)>>2)]=0; //@line 551 "klatt.cpp"
      ; //@line 552 "klatt.cpp"
    }
  
    var $8=$frame_addr; //@line 554 "klatt.cpp"
    var $ASP=(($8+168)&4294967295); //@line 554 "klatt.cpp"
    var $9=HEAP32[(($ASP)>>2)]; //@line 554 "klatt.cpp"
    var $call=__ZL7DBtoLINl($9); //@line 554 "klatt.cpp"
    var $mul=($call)*0.05; //@line 554 "klatt.cpp"
    HEAPF32[((((__ZL10kt_globals+128)&4294967295))>>2)]=$mul; //@line 554 "klatt.cpp"
    var $10=$frame_addr; //@line 555 "klatt.cpp"
    var $AF=(($10+184)&4294967295); //@line 555 "klatt.cpp"
    var $11=HEAP32[(($AF)>>2)]; //@line 555 "klatt.cpp"
    var $call3=__ZL7DBtoLINl($11); //@line 555 "klatt.cpp"
    var $mul4=($call3)*0.25; //@line 555 "klatt.cpp"
    HEAPF32[((((__ZL10kt_globals+136)&4294967295))>>2)]=$mul4; //@line 555 "klatt.cpp"
    var $12=$frame_addr; //@line 556 "klatt.cpp"
    var $AVpdb=(($12+196)&4294967295); //@line 556 "klatt.cpp"
    var $13=HEAP32[(($AVpdb)>>2)]; //@line 556 "klatt.cpp"
    var $call5=__ZL7DBtoLINl($13); //@line 556 "klatt.cpp"
    HEAPF32[((((__ZL10kt_globals+120)&4294967295))>>2)]=$call5; //@line 556 "klatt.cpp"
    var $14=$frame_addr; //@line 557 "klatt.cpp"
    var $AB=(($14+192)&4294967295); //@line 557 "klatt.cpp"
    var $15=HEAP32[(($AB)>>2)]; //@line 557 "klatt.cpp"
    var $call6=__ZL7DBtoLINl($15); //@line 557 "klatt.cpp"
    var $mul7=($call6)*0.05; //@line 557 "klatt.cpp"
    HEAPF32[((((__ZL10kt_globals+104)&4294967295))>>2)]=$mul7; //@line 557 "klatt.cpp"
    $ix=0; //@line 559 "klatt.cpp"
    ; //@line 559 "klatt.cpp"
    $for_body$5: while(1) { 
  
      var $16=$ix; //@line 562 "klatt.cpp"
      var $17=$frame_addr; //@line 562 "klatt.cpp"
      var $Ap=(($17+88)&4294967295); //@line 562 "klatt.cpp"
      var $arrayidx=(($Ap+$16*4)&4294967295); //@line 562 "klatt.cpp"
      var $18=HEAP32[(($arrayidx)>>2)]; //@line 562 "klatt.cpp"
      var $call9=__ZL7DBtoLINl($18); //@line 562 "klatt.cpp"
      var $19=$ix; //@line 562 "klatt.cpp"
      var $arrayidx10=((__ZZL10frame_initP13klatt_frame_tE14amp_par_factor+$19*8)&4294967295); //@line 562 "klatt.cpp"
      var $20=HEAPF32[(($arrayidx10)>>2)]; //@line 562 "klatt.cpp"
      var $mul11=($call9)*($20); //@line 562 "klatt.cpp"
      var $21=$ix; //@line 562 "klatt.cpp"
      var $arrayidx12=(($amp_par+$21*8)&4294967295); //@line 562 "klatt.cpp"
      HEAPF32[(($arrayidx12)>>2)]=$mul11; //@line 562 "klatt.cpp"
      var $22=$ix; //@line 559 "klatt.cpp"
      var $inc=((($22)+1)&4294967295); //@line 559 "klatt.cpp"
      $ix=$inc; //@line 559 "klatt.cpp"
      var $cmp8=((($inc))|0) <= 6; //@line 559 "klatt.cpp"
      if ($cmp8) { __label__ = 3;continue $for_body$5; } else { __label__ = 4;break $for_body$5; } //@line 559 "klatt.cpp"
    }
  
    var $23=$frame_addr; //@line 565 "klatt.cpp"
    var $Gain0=(($23+200)&4294967295); //@line 565 "klatt.cpp"
    var $24=HEAP32[(($Gain0)>>2)]; //@line 565 "klatt.cpp"
    var $sub13=((($24)-3)&4294967295); //@line 565 "klatt.cpp"
    $Gain0_tmp=$sub13; //@line 565 "klatt.cpp"
    var $25=$Gain0_tmp; //@line 566 "klatt.cpp"
    var $cmp14=((($25))|0) <= 0; //@line 566 "klatt.cpp"
    if ($cmp14) { __label__ = 5;; } else { __label__ = 6;; } //@line 566 "klatt.cpp"
    if (__label__ == 5) {
  
      $Gain0_tmp=57; //@line 568 "klatt.cpp"
      ; //@line 569 "klatt.cpp"
    }
  
    var $26=$Gain0_tmp; //@line 570 "klatt.cpp"
    var $call17=__ZL7DBtoLINl($26); //@line 570 "klatt.cpp"
    var $27=HEAP32[((((__ZL10kt_globals+184)&4294967295))>>2)]; //@line 570 "klatt.cpp"
    var $conv=((($27))|0); //@line 570 "klatt.cpp"
    var $div18=($call17)/($conv); //@line 570 "klatt.cpp"
    HEAPF32[((((__ZL10kt_globals+152)&4294967295))>>2)]=$div18; //@line 570 "klatt.cpp"
    $ix=1; //@line 573 "klatt.cpp"
    ; //@line 573 "klatt.cpp"
    $for_body21$11: while(1) { 
  
      var $28=$ix; //@line 576 "klatt.cpp"
      var $29=$frame_addr; //@line 576 "klatt.cpp"
      var $Fhz=(($29+8)&4294967295); //@line 576 "klatt.cpp"
      var $arrayidx22=(($Fhz+$28*4)&4294967295); //@line 576 "klatt.cpp"
      var $30=HEAP32[(($arrayidx22)>>2)]; //@line 576 "klatt.cpp"
      var $31=$ix; //@line 576 "klatt.cpp"
      var $32=$frame_addr; //@line 576 "klatt.cpp"
      var $Bhz=(($32+48)&4294967295); //@line 576 "klatt.cpp"
      var $arrayidx23=(($Bhz+$31*4)&4294967295); //@line 576 "klatt.cpp"
      var $33=HEAP32[(($arrayidx23)>>2)]; //@line 576 "klatt.cpp"
      var $34=$ix; //@line 576 "klatt.cpp"
      var $arrayidx24=((((__ZL10kt_globals+188)&4294967295)+$34*64)&4294967295); //@line 576 "klatt.cpp"
      __ZL6setabcllP11resonator_t($30, $33, $arrayidx24); //@line 576 "klatt.cpp"
      var $35=$ix; //@line 578 "klatt.cpp"
      var $cmp25=((($35))|0) <= 5; //@line 578 "klatt.cpp"
      if ($cmp25) { __label__ = 8;; } else { __label__ = 9;; } //@line 578 "klatt.cpp"
      if (__label__ == 8) {
  
        var $36=$ix; //@line 580 "klatt.cpp"
        var $37=$frame_addr; //@line 580 "klatt.cpp"
        var $Fhz_next=(($37+208)&4294967295); //@line 580 "klatt.cpp"
        var $arrayidx27=(($Fhz_next+$36*4)&4294967295); //@line 580 "klatt.cpp"
        var $38=HEAP32[(($arrayidx27)>>2)]; //@line 580 "klatt.cpp"
        var $39=$ix; //@line 580 "klatt.cpp"
        var $40=$frame_addr; //@line 580 "klatt.cpp"
        var $Bhz_next=(($40+248)&4294967295); //@line 580 "klatt.cpp"
        var $arrayidx28=(($Bhz_next+$39*4)&4294967295); //@line 580 "klatt.cpp"
        var $41=HEAP32[(($arrayidx28)>>2)]; //@line 580 "klatt.cpp"
        var $42=$ix; //@line 580 "klatt.cpp"
        var $arrayidx29=((((__ZL10kt_globals+1468)&4294967295)+$42*64)&4294967295); //@line 580 "klatt.cpp"
        __ZL6setabcllP11resonator_t($38, $41, $arrayidx29); //@line 580 "klatt.cpp"
        var $43=$ix; //@line 582 "klatt.cpp"
        var $arrayidx30=((((__ZL10kt_globals+1468)&4294967295)+$43*64)&4294967295); //@line 582 "klatt.cpp"
        var $a=(($arrayidx30)&4294967295); //@line 582 "klatt.cpp"
        var $44=HEAPF32[(($a)>>2)]; //@line 582 "klatt.cpp"
        var $45=$ix; //@line 582 "klatt.cpp"
        var $arrayidx31=((((__ZL10kt_globals+188)&4294967295)+$45*64)&4294967295); //@line 582 "klatt.cpp"
        var $a32=(($arrayidx31)&4294967295); //@line 582 "klatt.cpp"
        var $46=HEAPF32[(($a32)>>2)]; //@line 582 "klatt.cpp"
        var $sub33=($44)-($46); //@line 582 "klatt.cpp"
        var $div34=($sub33)/64; //@line 582 "klatt.cpp"
        var $47=$ix; //@line 582 "klatt.cpp"
        var $arrayidx35=((((__ZL10kt_globals+188)&4294967295)+$47*64)&4294967295); //@line 582 "klatt.cpp"
        var $a_inc=(($arrayidx35+40)&4294967295); //@line 582 "klatt.cpp"
        HEAPF32[(($a_inc)>>2)]=$div34; //@line 582 "klatt.cpp"
        var $48=$ix; //@line 583 "klatt.cpp"
        var $arrayidx36=((((__ZL10kt_globals+1468)&4294967295)+$48*64)&4294967295); //@line 583 "klatt.cpp"
        var $b=(($arrayidx36+8)&4294967295); //@line 583 "klatt.cpp"
        var $49=HEAPF32[(($b)>>2)]; //@line 583 "klatt.cpp"
        var $50=$ix; //@line 583 "klatt.cpp"
        var $arrayidx37=((((__ZL10kt_globals+188)&4294967295)+$50*64)&4294967295); //@line 583 "klatt.cpp"
        var $b38=(($arrayidx37+8)&4294967295); //@line 583 "klatt.cpp"
        var $51=HEAPF32[(($b38)>>2)]; //@line 583 "klatt.cpp"
        var $sub39=($49)-($51); //@line 583 "klatt.cpp"
        var $div40=($sub39)/64; //@line 583 "klatt.cpp"
        var $52=$ix; //@line 583 "klatt.cpp"
        var $arrayidx41=((((__ZL10kt_globals+188)&4294967295)+$52*64)&4294967295); //@line 583 "klatt.cpp"
        var $b_inc=(($arrayidx41+48)&4294967295); //@line 583 "klatt.cpp"
        HEAPF32[(($b_inc)>>2)]=$div40; //@line 583 "klatt.cpp"
        var $53=$ix; //@line 584 "klatt.cpp"
        var $arrayidx42=((((__ZL10kt_globals+1468)&4294967295)+$53*64)&4294967295); //@line 584 "klatt.cpp"
        var $c=(($arrayidx42+16)&4294967295); //@line 584 "klatt.cpp"
        var $54=HEAPF32[(($c)>>2)]; //@line 584 "klatt.cpp"
        var $55=$ix; //@line 584 "klatt.cpp"
        var $arrayidx43=((((__ZL10kt_globals+188)&4294967295)+$55*64)&4294967295); //@line 584 "klatt.cpp"
        var $c44=(($arrayidx43+16)&4294967295); //@line 584 "klatt.cpp"
        var $56=HEAPF32[(($c44)>>2)]; //@line 584 "klatt.cpp"
        var $sub45=($54)-($56); //@line 584 "klatt.cpp"
        var $div46=($sub45)/64; //@line 584 "klatt.cpp"
        var $57=$ix; //@line 584 "klatt.cpp"
        var $arrayidx47=((((__ZL10kt_globals+188)&4294967295)+$57*64)&4294967295); //@line 584 "klatt.cpp"
        var $c_inc=(($arrayidx47+56)&4294967295); //@line 584 "klatt.cpp"
        HEAPF32[(($c_inc)>>2)]=$div46; //@line 584 "klatt.cpp"
        ; //@line 585 "klatt.cpp"
      }
  
      var $58=$ix; //@line 573 "klatt.cpp"
      var $inc50=((($58)+1)&4294967295); //@line 573 "klatt.cpp"
      $ix=$inc50; //@line 573 "klatt.cpp"
      var $cmp20=((($inc50))|0) <= 9; //@line 573 "klatt.cpp"
      if ($cmp20) { __label__ = 7;continue $for_body21$11; } else { __label__ = 10;break $for_body21$11; } //@line 573 "klatt.cpp"
    }
  
    var $59=$frame_addr; //@line 589 "klatt.cpp"
    var $Fhz52=(($59+8)&4294967295); //@line 589 "klatt.cpp"
    var $arrayidx53=(($Fhz52)&4294967295); //@line 589 "klatt.cpp"
    var $60=HEAP32[(($arrayidx53)>>2)]; //@line 589 "klatt.cpp"
    var $61=$frame_addr; //@line 589 "klatt.cpp"
    var $Bhz54=(($61+48)&4294967295); //@line 589 "klatt.cpp"
    var $arrayidx55=(($Bhz54)&4294967295); //@line 589 "klatt.cpp"
    var $62=HEAP32[(($arrayidx55)>>2)]; //@line 589 "klatt.cpp"
    __ZL10setzeroabcllP11resonator_t($60, $62, ((__ZL10kt_globals+188)&4294967295)); //@line 589 "klatt.cpp"
    var $63=$frame_addr; //@line 590 "klatt.cpp"
    var $Fhz_next56=(($63+208)&4294967295); //@line 590 "klatt.cpp"
    var $arrayidx57=(($Fhz_next56)&4294967295); //@line 590 "klatt.cpp"
    var $64=HEAP32[(($arrayidx57)>>2)]; //@line 590 "klatt.cpp"
    var $65=$frame_addr; //@line 590 "klatt.cpp"
    var $Bhz_next58=(($65+248)&4294967295); //@line 590 "klatt.cpp"
    var $arrayidx59=(($Bhz_next58)&4294967295); //@line 590 "klatt.cpp"
    var $66=HEAP32[(($arrayidx59)>>2)]; //@line 590 "klatt.cpp"
    __ZL10setzeroabcllP11resonator_t($64, $66, ((__ZL10kt_globals+1468)&4294967295)); //@line 590 "klatt.cpp"
    var $67=HEAPF32[((((__ZL10kt_globals+1468)&4294967295))>>2)]; //@line 591 "klatt.cpp"
    var $68=HEAPF32[((((__ZL10kt_globals+188)&4294967295))>>2)]; //@line 591 "klatt.cpp"
    var $sub60=($67)-($68); //@line 591 "klatt.cpp"
    var $div61=($sub60)/64; //@line 591 "klatt.cpp"
    HEAPF32[((((__ZL10kt_globals+188+40)&4294967295))>>2)]=$div61; //@line 591 "klatt.cpp"
    var $69=HEAPF32[((((__ZL10kt_globals+1468+8)&4294967295))>>2)]; //@line 592 "klatt.cpp"
    var $70=HEAPF32[((((__ZL10kt_globals+188+8)&4294967295))>>2)]; //@line 592 "klatt.cpp"
    var $sub62=($69)-($70); //@line 592 "klatt.cpp"
    var $div63=($sub62)/64; //@line 592 "klatt.cpp"
    HEAPF32[((((__ZL10kt_globals+188+48)&4294967295))>>2)]=$div63; //@line 592 "klatt.cpp"
    var $71=HEAPF32[((((__ZL10kt_globals+1468+16)&4294967295))>>2)]; //@line 593 "klatt.cpp"
    var $72=HEAPF32[((((__ZL10kt_globals+188+16)&4294967295))>>2)]; //@line 593 "klatt.cpp"
    var $sub64=($71)-($72); //@line 593 "klatt.cpp"
    var $div65=($sub64)/64; //@line 593 "klatt.cpp"
    HEAPF32[((((__ZL10kt_globals+188+56)&4294967295))>>2)]=$div65; //@line 593 "klatt.cpp"
    $ix=0; //@line 598 "klatt.cpp"
    ; //@line 598 "klatt.cpp"
    while(1) { 
  
      var $73=$ix; //@line 600 "klatt.cpp"
      var $74=$frame_addr; //@line 600 "klatt.cpp"
      var $Fhz69=(($74+8)&4294967295); //@line 600 "klatt.cpp"
      var $arrayidx70=(($Fhz69+$73*4)&4294967295); //@line 600 "klatt.cpp"
      var $75=HEAP32[(($arrayidx70)>>2)]; //@line 600 "klatt.cpp"
      var $76=$ix; //@line 600 "klatt.cpp"
      var $77=$frame_addr; //@line 600 "klatt.cpp"
      var $Bphz=(($77+128)&4294967295); //@line 600 "klatt.cpp"
      var $arrayidx71=(($Bphz+$76*4)&4294967295); //@line 600 "klatt.cpp"
      var $78=HEAP32[(($arrayidx71)>>2)]; //@line 600 "klatt.cpp"
      var $79=$ix; //@line 600 "klatt.cpp"
      var $add=((($79)+10)&4294967295); //@line 600 "klatt.cpp"
      var $arrayidx72=((((__ZL10kt_globals+188)&4294967295)+$add*64)&4294967295); //@line 600 "klatt.cpp"
      __ZL6setabcllP11resonator_t($75, $78, $arrayidx72); //@line 600 "klatt.cpp"
      var $80=$ix; //@line 601 "klatt.cpp"
      var $arrayidx73=(($amp_par+$80*8)&4294967295); //@line 601 "klatt.cpp"
      var $81=HEAPF32[(($arrayidx73)>>2)]; //@line 601 "klatt.cpp"
      var $82=$ix; //@line 601 "klatt.cpp"
      var $add74=((($82)+10)&4294967295); //@line 601 "klatt.cpp"
      var $arrayidx75=((((__ZL10kt_globals+188)&4294967295)+$add74*64)&4294967295); //@line 601 "klatt.cpp"
      var $a76=(($arrayidx75)&4294967295); //@line 601 "klatt.cpp"
      var $83=HEAPF32[(($a76)>>2)]; //@line 601 "klatt.cpp"
      var $mul77=($83)*($81); //@line 601 "klatt.cpp"
      HEAPF32[(($a76)>>2)]=$mul77; //@line 601 "klatt.cpp"
      var $84=$ix; //@line 598 "klatt.cpp"
      var $inc79=((($84)+1)&4294967295); //@line 598 "klatt.cpp"
      $ix=$inc79; //@line 598 "klatt.cpp"
      var $cmp67=((($inc79))|0) <= 6; //@line 598 "klatt.cpp"
      if ($cmp67) { __label__ = 11;continue ; } else { __label__ = 12;break ; } //@line 598 "klatt.cpp"
    }
  
    var $85=HEAP32[((((__ZL10kt_globals+8)&4294967295))>>2)]; //@line 606 "klatt.cpp"
    var $div81=((((($85))|0)/2)|0); //@line 606 "klatt.cpp"
    __ZL6setabcllP11resonator_t(0, $div81, ((__ZL10kt_globals+188+1216)&4294967295)); //@line 606 "klatt.cpp"
    STACKTOP = __stackBase__;
    return; //@line 608 "klatt.cpp"
    return;
  }
  

  function __ZL7parwaveP13klatt_frame_t() {
    ;
    var __label__;
    var __lastLabel__ = null;
  
    var $retval;
    var $frame_addr;
    var $temp;
    var $value;
    var $outbypas;
    var $out;
    var $n4;
    var $frics;
    var $glotout;
    var $aspiration;
    var $casc_next_in;
    var $par_glotout;
    var $ix;
    var $z2;
    var $c;
    var $sample;
    $frame_addr=__ZL8kt_frame;
    var $0=$frame_addr; //@line 245 "klatt.cpp"
    __ZL7flutterP13klatt_frame_t($0); //@line 245 "klatt.cpp"
    HEAP32[((((__ZL10kt_globals+36)&4294967295))>>2)]=0; //@line 260 "klatt.cpp"
    ; //@line 260 "klatt.cpp"
    $for_cond$2: while(1) { 
  
      var $1=HEAP32[((((__ZL10kt_globals+36)&4294967295))>>2)]; //@line 260 "klatt.cpp"
      var $2=HEAP32[((((__ZL10kt_globals+28)&4294967295))>>2)]; //@line 260 "klatt.cpp"
      var $cmp=((($1))|0) < ((($2))|0); //@line 260 "klatt.cpp"
      if (!($cmp)) { __label__ = 36;break $for_cond$2; } //@line 260 "klatt.cpp"
  
      var $3=HEAPF32[((__ZZL7parwaveP13klatt_frame_tE5noise)>>2)]; //@line 263 "klatt.cpp"
      var $call=__ZL9gen_noised($3); //@line 263 "klatt.cpp"
      HEAPF32[((__ZZL7parwaveP13klatt_frame_tE5noise)>>2)]=$call; //@line 263 "klatt.cpp"
      var $4=HEAP32[((((__ZL10kt_globals+32)&4294967295))>>2)]; //@line 270 "klatt.cpp"
      var $5=HEAP32[((((__ZL10kt_globals+48)&4294967295))>>2)]; //@line 270 "klatt.cpp"
      var $cmp1=((($4))|0) > ((($5))|0); //@line 270 "klatt.cpp"
      if ($cmp1) { __label__ = 3;; } else { __label__ = 4;; } //@line 270 "klatt.cpp"
      if (__label__ == 3) {
  
        var $6=HEAPF32[((__ZZL7parwaveP13klatt_frame_tE5noise)>>2)]; //@line 272 "klatt.cpp"
        var $mul=($6)*0.5; //@line 272 "klatt.cpp"
        HEAPF32[((__ZZL7parwaveP13klatt_frame_tE5noise)>>2)]=$mul; //@line 272 "klatt.cpp"
        ; //@line 273 "klatt.cpp"
      }
  
      var $7=HEAPF32[((((__ZL10kt_globals+136)&4294967295))>>2)]; //@line 276 "klatt.cpp"
      var $8=HEAPF32[((__ZZL7parwaveP13klatt_frame_tE5noise)>>2)]; //@line 276 "klatt.cpp"
      var $mul2=($7)*($8); //@line 276 "klatt.cpp"
      $frics=$mul2; //@line 276 "klatt.cpp"
      $n4=0; //@line 284 "klatt.cpp"
      ; //@line 284 "klatt.cpp"
      $for_body5$8: while(1) { 
  
        var $9=HEAP32[((((__ZL10kt_globals+20)&4294967295))>>2)]; //@line 286 "klatt.cpp"
        if ($9 == 1) {
          __label__ = 6;;
        }
        else if ($9 == 2) {
          __label__ = 7;;
        }
        else if ($9 == 3) {
          __label__ = 8;;
        }
        else {
        __label__ = 9;;
        }
        
        if (__label__ == 6) {
  
          var $call6=__ZL16impulsive_sourcev(); //@line 289 "klatt.cpp"
          HEAPF32[((__ZZL7parwaveP13klatt_frame_tE5voice)>>2)]=$call6; //@line 289 "klatt.cpp"
          ; //@line 290 "klatt.cpp"
        }
        else if (__label__ == 7) {
  
          var $call8=__ZL14natural_sourcev(); //@line 292 "klatt.cpp"
          HEAPF32[((__ZZL7parwaveP13klatt_frame_tE5voice)>>2)]=$call8; //@line 292 "klatt.cpp"
          ; //@line 293 "klatt.cpp"
        }
        else if (__label__ == 8) {
  
          var $call10=__ZL14sampled_sourcev(); //@line 295 "klatt.cpp"
          HEAPF32[((__ZZL7parwaveP13klatt_frame_tE5voice)>>2)]=$call10; //@line 295 "klatt.cpp"
          ; //@line 296 "klatt.cpp"
        }
  
        var $10=HEAP32[((((__ZL10kt_globals+32)&4294967295))>>2)]; //@line 300 "klatt.cpp"
        var $11=HEAP32[((((__ZL10kt_globals+40)&4294967295))>>2)]; //@line 300 "klatt.cpp"
        var $cmp11=((($10))|0) >= ((($11))|0); //@line 300 "klatt.cpp"
        if ($cmp11) { __label__ = 10;; } else { __label__ = 11;; } //@line 300 "klatt.cpp"
        if (__label__ == 10) {
  
          HEAP32[((((__ZL10kt_globals+32)&4294967295))>>2)]=0; //@line 302 "klatt.cpp"
          var $12=$frame_addr; //@line 303 "klatt.cpp"
          __ZL21pitch_synch_par_resetP13klatt_frame_t($12); //@line 303 "klatt.cpp"
          ; //@line 304 "klatt.cpp"
        }
  
        var $13=HEAPF32[((__ZZL7parwaveP13klatt_frame_tE5voice)>>2)]; //@line 311 "klatt.cpp"
        var $call14=__ZL9resonatorP11resonator_td(((__ZL10kt_globals+188+1152)&4294967295), $13); //@line 311 "klatt.cpp"
        HEAPF32[((__ZZL7parwaveP13klatt_frame_tE5voice)>>2)]=$call14; //@line 311 "klatt.cpp"
        var $14=HEAP32[((((__ZL10kt_globals+32)&4294967295))>>2)]; //@line 314 "klatt.cpp"
        var $inc=((($14)+1)&4294967295); //@line 314 "klatt.cpp"
        HEAP32[((((__ZL10kt_globals+32)&4294967295))>>2)]=$inc; //@line 314 "klatt.cpp"
        var $15=$n4; //@line 284 "klatt.cpp"
        var $inc15=((($15)+1)&4294967295); //@line 284 "klatt.cpp"
        $n4=$inc15; //@line 284 "klatt.cpp"
        var $cmp4=((($inc15))|0) < 4; //@line 284 "klatt.cpp"
        if ($cmp4) { __label__ = 5;continue $for_body5$8; } else { __label__ = 12;break $for_body5$8; } //@line 284 "klatt.cpp"
      }
  
      var $16=HEAPF32[((__ZZL7parwaveP13klatt_frame_tE5voice)>>2)]; //@line 322 "klatt.cpp"
      var $17=HEAPF32[((((__ZL10kt_globals+88)&4294967295))>>2)]; //@line 322 "klatt.cpp"
      var $mul16=($16)*($17); //@line 322 "klatt.cpp"
      var $18=HEAPF32[((__ZZL7parwaveP13klatt_frame_tE5vlast)>>2)]; //@line 322 "klatt.cpp"
      var $19=HEAPF32[((((__ZL10kt_globals+96)&4294967295))>>2)]; //@line 322 "klatt.cpp"
      var $mul17=($18)*($19); //@line 322 "klatt.cpp"
      var $add=($mul16)+($mul17); //@line 322 "klatt.cpp"
      HEAPF32[((__ZZL7parwaveP13klatt_frame_tE5voice)>>2)]=$add; //@line 322 "klatt.cpp"
      var $20=HEAPF32[((__ZZL7parwaveP13klatt_frame_tE5voice)>>2)]; //@line 323 "klatt.cpp"
      HEAPF32[((__ZZL7parwaveP13klatt_frame_tE5vlast)>>2)]=$20; //@line 323 "klatt.cpp"
      var $21=HEAP32[((((__ZL10kt_globals+32)&4294967295))>>2)]; //@line 332 "klatt.cpp"
      var $22=HEAP32[((((__ZL10kt_globals+44)&4294967295))>>2)]; //@line 332 "klatt.cpp"
      var $cmp18=((($21))|0) < ((($22))|0); //@line 332 "klatt.cpp"
      if ($cmp18) { __label__ = 13;; } else { __label__ = 14;; } //@line 332 "klatt.cpp"
      if (__label__ == 13) {
  
        var $23=HEAPF32[((((__ZL10kt_globals+144)&4294967295))>>2)]; //@line 334 "klatt.cpp"
        var $24=HEAP32[((((__ZL10kt_globals+52)&4294967295))>>2)]; //@line 334 "klatt.cpp"
        var $conv=((($24))|0); //@line 334 "klatt.cpp"
        var $mul20=($23)*($conv); //@line 334 "klatt.cpp"
        var $25=HEAPF32[((__ZZL7parwaveP13klatt_frame_tE5voice)>>2)]; //@line 334 "klatt.cpp"
        var $add21=($25)+($mul20); //@line 334 "klatt.cpp"
        HEAPF32[((__ZZL7parwaveP13klatt_frame_tE5voice)>>2)]=$add21; //@line 334 "klatt.cpp"
        ; //@line 335 "klatt.cpp"
      }
  
      var $26=HEAPF32[((((__ZL10kt_globals+112)&4294967295))>>2)]; //@line 338 "klatt.cpp"
      var $27=HEAPF32[((__ZZL7parwaveP13klatt_frame_tE5voice)>>2)]; //@line 338 "klatt.cpp"
      var $mul23=($26)*($27); //@line 338 "klatt.cpp"
      $glotout=$mul23; //@line 338 "klatt.cpp"
      var $28=HEAPF32[((((__ZL10kt_globals+120)&4294967295))>>2)]; //@line 339 "klatt.cpp"
      var $29=HEAPF32[((__ZZL7parwaveP13klatt_frame_tE5voice)>>2)]; //@line 339 "klatt.cpp"
      var $mul24=($28)*($29); //@line 339 "klatt.cpp"
      $par_glotout=$mul24; //@line 339 "klatt.cpp"
      var $30=HEAPF32[((((__ZL10kt_globals+128)&4294967295))>>2)]; //@line 342 "klatt.cpp"
      var $31=HEAPF32[((__ZZL7parwaveP13klatt_frame_tE5noise)>>2)]; //@line 342 "klatt.cpp"
      var $mul25=($30)*($31); //@line 342 "klatt.cpp"
      $aspiration=$mul25; //@line 342 "klatt.cpp"
      var $32=$aspiration; //@line 343 "klatt.cpp"
      var $33=$glotout; //@line 343 "klatt.cpp"
      var $add26=($33)+($32); //@line 343 "klatt.cpp"
      $glotout=$add26; //@line 343 "klatt.cpp"
      var $34=$aspiration; //@line 345 "klatt.cpp"
      var $35=$par_glotout; //@line 345 "klatt.cpp"
      var $add27=($35)+($34); //@line 345 "klatt.cpp"
      $par_glotout=$add27; //@line 345 "klatt.cpp"
      $out=0; //@line 352 "klatt.cpp"
      var $36=HEAP32[((((__ZL10kt_globals)&4294967295))>>2)]; //@line 353 "klatt.cpp"
      var $cmp28=((($36))|0)!=2; //@line 353 "klatt.cpp"
      if ($cmp28) { __label__ = 15;; } else { __label__ = 16;; } //@line 353 "klatt.cpp"
      if (__label__ == 15) {
  
        var $37=$glotout; //@line 355 "klatt.cpp"
        var $call30=__ZL14antiresonator2P11resonator_td($37); //@line 355 "klatt.cpp"
        $casc_next_in=$call30; //@line 355 "klatt.cpp"
        var $38=$casc_next_in; //@line 356 "klatt.cpp"
        var $call31=__ZL9resonatorP11resonator_td(((__ZL10kt_globals+188+576)&4294967295), $38); //@line 356 "klatt.cpp"
        $casc_next_in=$call31; //@line 356 "klatt.cpp"
        var $39=$casc_next_in; //@line 357 "klatt.cpp"
        var $call32=__ZL9resonatorP11resonator_td(((__ZL10kt_globals+188+512)&4294967295), $39); //@line 357 "klatt.cpp"
        $casc_next_in=$call32; //@line 357 "klatt.cpp"
        var $40=$casc_next_in; //@line 358 "klatt.cpp"
        var $call33=__ZL9resonatorP11resonator_td(((__ZL10kt_globals+188+448)&4294967295), $40); //@line 358 "klatt.cpp"
        $casc_next_in=$call33; //@line 358 "klatt.cpp"
        var $41=$casc_next_in; //@line 359 "klatt.cpp"
        var $call34=__ZL9resonatorP11resonator_td(((__ZL10kt_globals+188+384)&4294967295), $41); //@line 359 "klatt.cpp"
        $casc_next_in=$call34; //@line 359 "klatt.cpp"
        var $42=$casc_next_in; //@line 360 "klatt.cpp"
        var $call35=__ZL10resonator2P11resonator_td(((__ZL10kt_globals+188+320)&4294967295), $42); //@line 360 "klatt.cpp"
        $casc_next_in=$call35; //@line 360 "klatt.cpp"
        var $43=$casc_next_in; //@line 361 "klatt.cpp"
        var $call36=__ZL10resonator2P11resonator_td(((__ZL10kt_globals+188+256)&4294967295), $43); //@line 361 "klatt.cpp"
        $casc_next_in=$call36; //@line 361 "klatt.cpp"
        var $44=$casc_next_in; //@line 362 "klatt.cpp"
        var $call37=__ZL10resonator2P11resonator_td(((__ZL10kt_globals+188+192)&4294967295), $44); //@line 362 "klatt.cpp"
        $casc_next_in=$call37; //@line 362 "klatt.cpp"
        var $45=$casc_next_in; //@line 363 "klatt.cpp"
        var $call38=__ZL10resonator2P11resonator_td(((__ZL10kt_globals+188+128)&4294967295), $45); //@line 363 "klatt.cpp"
        $casc_next_in=$call38; //@line 363 "klatt.cpp"
        var $46=$casc_next_in; //@line 364 "klatt.cpp"
        var $call39=__ZL10resonator2P11resonator_td(((__ZL10kt_globals+188+64)&4294967295), $46); //@line 364 "klatt.cpp"
        $out=$call39; //@line 364 "klatt.cpp"
        ; //@line 365 "klatt.cpp"
      }
  
      var $47=$par_glotout; //@line 368 "klatt.cpp"
      HEAPF32[((__ZZL7parwaveP13klatt_frame_tE5sourc)>>2)]=$47; //@line 368 "klatt.cpp"
      var $48=HEAPF32[((__ZZL7parwaveP13klatt_frame_tE5sourc)>>2)]; //@line 377 "klatt.cpp"
      var $call41=__ZL9resonatorP11resonator_td(((__ZL10kt_globals+188+704)&4294967295), $48); //@line 377 "klatt.cpp"
      var $49=$out; //@line 377 "klatt.cpp"
      var $add42=($49)+($call41); //@line 377 "klatt.cpp"
      $out=$add42; //@line 377 "klatt.cpp"
      var $50=HEAPF32[((__ZZL7parwaveP13klatt_frame_tE5sourc)>>2)]; //@line 378 "klatt.cpp"
      var $call43=__ZL9resonatorP11resonator_td(((__ZL10kt_globals+188+640)&4294967295), $50); //@line 378 "klatt.cpp"
      var $51=$out; //@line 378 "klatt.cpp"
      var $add44=($51)+($call43); //@line 378 "klatt.cpp"
      $out=$add44; //@line 378 "klatt.cpp"
      var $52=$frics; //@line 380 "klatt.cpp"
      var $53=$par_glotout; //@line 380 "klatt.cpp"
      var $add45=($52)+($53); //@line 380 "klatt.cpp"
      var $54=HEAPF32[((__ZZL7parwaveP13klatt_frame_tE8glotlast)>>2)]; //@line 380 "klatt.cpp"
      var $sub=($add45)-($54); //@line 380 "klatt.cpp"
      HEAPF32[((__ZZL7parwaveP13klatt_frame_tE5sourc)>>2)]=$sub; //@line 380 "klatt.cpp"
      var $55=$par_glotout; //@line 381 "klatt.cpp"
      HEAPF32[((__ZZL7parwaveP13klatt_frame_tE8glotlast)>>2)]=$55; //@line 381 "klatt.cpp"
      $ix=12; //@line 383 "klatt.cpp"
      ; //@line 383 "klatt.cpp"
      $for_body48$25: while(1) { 
  
        var $56=$ix; //@line 385 "klatt.cpp"
        var $arrayidx=((((__ZL10kt_globals+188)&4294967295)+$56*64)&4294967295); //@line 385 "klatt.cpp"
        var $57=HEAPF32[((__ZZL7parwaveP13klatt_frame_tE5sourc)>>2)]; //@line 385 "klatt.cpp"
        var $call49=__ZL9resonatorP11resonator_td($arrayidx, $57); //@line 385 "klatt.cpp"
        var $58=$out; //@line 385 "klatt.cpp"
        var $sub50=($call49)-($58); //@line 385 "klatt.cpp"
        $out=$sub50; //@line 385 "klatt.cpp"
        var $59=$ix; //@line 383 "klatt.cpp"
        var $inc52=((($59)+1)&4294967295); //@line 383 "klatt.cpp"
        $ix=$inc52; //@line 383 "klatt.cpp"
        var $cmp47=((($inc52))|0) <= 16; //@line 383 "klatt.cpp"
        if ($cmp47) { __label__ = 17;continue $for_body48$25; } else { __label__ = 18;break $for_body48$25; } //@line 383 "klatt.cpp"
      }
  
      var $60=HEAPF32[((((__ZL10kt_globals+104)&4294967295))>>2)]; //@line 388 "klatt.cpp"
      var $61=HEAPF32[((__ZZL7parwaveP13klatt_frame_tE5sourc)>>2)]; //@line 388 "klatt.cpp"
      var $mul54=($60)*($61); //@line 388 "klatt.cpp"
      $outbypas=$mul54; //@line 388 "klatt.cpp"
      var $62=$outbypas; //@line 390 "klatt.cpp"
      var $63=$out; //@line 390 "klatt.cpp"
      var $sub55=($62)-($63); //@line 390 "klatt.cpp"
      $out=$sub55; //@line 390 "klatt.cpp"
      var $64=$out; //@line 423 "klatt.cpp"
      var $call56=__ZL9resonatorP11resonator_td(((__ZL10kt_globals+188+1216)&4294967295), $64); //@line 423 "klatt.cpp"
      $out=$call56; //@line 423 "klatt.cpp"
      var $65=$out; //@line 424 "klatt.cpp"
      var $66=HEAP32[((((_wdata+52)&4294967295))>>2)]; //@line 424 "klatt.cpp"
      var $conv57=((($66))|0); //@line 424 "klatt.cpp"
      var $mul58=($65)*($conv57); //@line 424 "klatt.cpp"
      var $67=HEAPF32[((((__ZL10kt_globals+152)&4294967295))>>2)]; //@line 424 "klatt.cpp"
      var $mul59=($mul58)*($67); //@line 424 "klatt.cpp"
      var $conv60=((($mul59))|0); //@line 424 "klatt.cpp"
      var $conv61=((($conv60))|0); //@line 424 "klatt.cpp"
      $temp=$conv61; //@line 424 "klatt.cpp"
      $z2=0; //@line 433 "klatt.cpp"
      var $68=HEAP32[((((_wdata+40)&4294967295))>>2)]; //@line 434 "klatt.cpp"
      var $69=HEAP32[((((_wdata+28)&4294967295))>>2)]; //@line 434 "klatt.cpp"
      var $cmp62=((($68))|0) < ((($69))|0); //@line 434 "klatt.cpp"
      if ($cmp62) { __label__ = 19;; } else { __label__ = 23;; } //@line 434 "klatt.cpp"
      if (__label__ == 19) {
  
        var $70=HEAP32[((((_wdata+32)&4294967295))>>2)]; //@line 436 "klatt.cpp"
        var $cmp64=((($70))|0)==0; //@line 436 "klatt.cpp"
        var $71=HEAPU32[((((_wdata+40)&4294967295))>>2)]; //@line 439 "klatt.cpp"
        var $add66=((($71)+1)&4294967295); //@line 439 "klatt.cpp"
        if ($cmp64) { __label__ = 20;; } else { __label__ = 21;; } //@line 436 "klatt.cpp"
        if (__label__ == 20) {
  
          var $72=HEAP32[((((_wdata+24)&4294967295))>>2)]; //@line 439 "klatt.cpp"
          var $arrayidx67=(($72+$add66)&4294967295); //@line 439 "klatt.cpp"
          var $73=HEAP8[($arrayidx67)]; //@line 439 "klatt.cpp"
          $c=$73; //@line 439 "klatt.cpp"
          var $74=HEAP32[((((_wdata+40)&4294967295))>>2)]; //@line 440 "klatt.cpp"
          var $75=HEAP32[((((_wdata+24)&4294967295))>>2)]; //@line 440 "klatt.cpp"
          var $arrayidx68=(($75+$74)&4294967295); //@line 440 "klatt.cpp"
          var $76=HEAPU8[($arrayidx68)]; //@line 440 "klatt.cpp"
          var $conv69=((($76))&255); //@line 440 "klatt.cpp"
          var $77=$c; //@line 440 "klatt.cpp"
          var $conv70=(tempInt=(($77)),(tempInt>=128?tempInt-256:tempInt)); //@line 440 "klatt.cpp"
          var $mul71=((($conv70)*256)&4294967295); //@line 440 "klatt.cpp"
          var $add72=((($mul71)+($conv69))&4294967295); //@line 440 "klatt.cpp"
          $sample=$add72; //@line 440 "klatt.cpp"
          var $78=HEAP32[((((_wdata+40)&4294967295))>>2)]; //@line 441 "klatt.cpp"
          var $add73=((($78)+2)&4294967295); //@line 441 "klatt.cpp"
          HEAP32[((((_wdata+40)&4294967295))>>2)]=$add73; //@line 441 "klatt.cpp"
          ; //@line 442 "klatt.cpp"
        }
        else if (__label__ == 21) {
  
          HEAP32[((((_wdata+40)&4294967295))>>2)]=$add66; //@line 446 "klatt.cpp"
          var $79=HEAP32[((((_wdata+24)&4294967295))>>2)]; //@line 446 "klatt.cpp"
          var $arrayidx75=(($79+$71)&4294967295); //@line 446 "klatt.cpp"
          var $80=HEAP8[($arrayidx75)]; //@line 446 "klatt.cpp"
          var $conv76=(tempInt=(($80)),(tempInt>=128?tempInt-256:tempInt)); //@line 446 "klatt.cpp"
          var $81=HEAP32[((((_wdata+32)&4294967295))>>2)]; //@line 446 "klatt.cpp"
          var $mul77=((($conv76)*($81))&4294967295); //@line 446 "klatt.cpp"
          $sample=$mul77; //@line 446 "klatt.cpp"
          ;
        }
  
        var $82=$sample; //@line 448 "klatt.cpp"
        var $83=HEAP32[((((_wdata+56)&4294967295))>>2)]; //@line 448 "klatt.cpp"
        var $mul79=((($83)*($82))&4294967295); //@line 448 "klatt.cpp"
        var $div=((((($mul79))|0)/1024)|0); //@line 448 "klatt.cpp"
        $z2=$div; //@line 448 "klatt.cpp"
        var $84=$z2; //@line 449 "klatt.cpp"
        var $85=HEAP32[((((_wdata+36)&4294967295))>>2)]; //@line 449 "klatt.cpp"
        var $mul80=((($85)*($84))&4294967295); //@line 449 "klatt.cpp"
        var $div81=((((($mul80))|0)/40)|0); //@line 449 "klatt.cpp"
        $z2=$div81; //@line 449 "klatt.cpp"
        var $86=$z2; //@line 450 "klatt.cpp"
        var $conv82=((($86))|0); //@line 450 "klatt.cpp"
        var $87=$temp; //@line 450 "klatt.cpp"
        var $add83=($87)+($conv82); //@line 450 "klatt.cpp"
        $temp=$add83; //@line 450 "klatt.cpp"
        ; //@line 451 "klatt.cpp"
      }
  
      var $88=HEAP32[((((__ZL10kt_globals+180)&4294967295))>>2)]; //@line 455 "klatt.cpp"
      var $cmp85=((($88))|0) > 0; //@line 455 "klatt.cpp"
      if ($cmp85) { __label__ = 24;; } else { __label__ = 25;; } //@line 455 "klatt.cpp"
      if (__label__ == 24) {
  
        var $89=HEAP32[((((__ZL10kt_globals+180)&4294967295))>>2)]; //@line 457 "klatt.cpp"
        var $dec=((($89)-1)&4294967295); //@line 457 "klatt.cpp"
        HEAP32[((((__ZL10kt_globals+180)&4294967295))>>2)]=$dec; //@line 457 "klatt.cpp"
        var $90=$temp; //@line 458 "klatt.cpp"
        var $91=HEAP32[((((__ZL10kt_globals+180)&4294967295))>>2)]; //@line 458 "klatt.cpp"
        var $conv87=((($91))|0); //@line 458 "klatt.cpp"
        var $mul88=($90)*($conv87); //@line 458 "klatt.cpp"
        var $div89=($mul88)/64; //@line 458 "klatt.cpp"
        $temp=$div89; //@line 458 "klatt.cpp"
        ; //@line 459 "klatt.cpp"
      }
  
      var $92=$temp; //@line 461 "klatt.cpp"
      var $conv91=((($92))|0); //@line 461 "klatt.cpp"
      var $93=HEAP32[((_echo_tail)>>2)]; //@line 461 "klatt.cpp"
      var $inc92=((($93)+1)&4294967295); //@line 461 "klatt.cpp"
      HEAP32[((_echo_tail)>>2)]=$inc92; //@line 461 "klatt.cpp"
      var $arrayidx93=((_echo_buf+$93*2)&4294967295); //@line 461 "klatt.cpp"
      var $94=HEAP16[(($arrayidx93)>>1)]; //@line 461 "klatt.cpp"
      var $conv94=(tempInt=(($94)),(tempInt>=32768?tempInt-65536:tempInt)); //@line 461 "klatt.cpp"
      var $95=HEAP32[((_echo_amp)>>2)]; //@line 461 "klatt.cpp"
      var $mul95=((($conv94)*($95))&4294967295); //@line 461 "klatt.cpp"
      var $shr=($mul95) >> 8; //@line 461 "klatt.cpp"
      var $add96=((($shr)+($conv91))&4294967295); //@line 461 "klatt.cpp"
      $value=$add96; //@line 461 "klatt.cpp"
      var $96=HEAP32[((_echo_tail)>>2)]; //@line 462 "klatt.cpp"
      var $cmp97=((($96))|0) >= 5500; //@line 462 "klatt.cpp"
      if ($cmp97) { __lastLabel__ = 25; __label__ = 26;; } else { __lastLabel__ = 25; __label__ = 27;; } //@line 462 "klatt.cpp"
      if (__label__ == 26) {
  
        HEAP32[((_echo_tail)>>2)]=0; //@line 463 "klatt.cpp"
        var $_pr=$value; //@line 465 "klatt.cpp"
        __lastLabel__ = 26; ; //@line 463 "klatt.cpp"
      }
  
      var $97=__lastLabel__ == 26 ? $_pr : ($add96); //@line 465 "klatt.cpp"
      var $cmp100=((($97))|0) < -32768; //@line 465 "klatt.cpp"
      if ($cmp100) { __label__ = 28;; } else { __label__ = 29;; } //@line 465 "klatt.cpp"
      $if_end102_thread$$if_end102$41: do { 
        if (__label__ == 28) {
  
          $value=-32768; //@line 467 "klatt.cpp"
          ; //@line 470 "klatt.cpp"
        }
        else if (__label__ == 29) {
  
          var $_pr1=$value; //@line 470 "klatt.cpp"
          var $cmp103=((($_pr1))|0) > 32767; //@line 470 "klatt.cpp"
          if (!($cmp103)) { __label__ = 31;break $if_end102_thread$$if_end102$41; } //@line 470 "klatt.cpp"
  
          $value=32767; //@line 472 "klatt.cpp"
          ; //@line 473 "klatt.cpp"
        }
      } while(0);
  
      var $98=$value; //@line 475 "klatt.cpp"
      var $conv106=((($98)) & 255); //@line 475 "klatt.cpp"
      var $99=HEAP32[((_out_ptr)>>2)]; //@line 475 "klatt.cpp"
      var $incdec_ptr=(($99+1)&4294967295); //@line 475 "klatt.cpp"
      HEAP32[((_out_ptr)>>2)]=$incdec_ptr; //@line 475 "klatt.cpp"
      HEAP8[($99)]=$conv106; //@line 475 "klatt.cpp"
      var $100=$value; //@line 476 "klatt.cpp"
      var $shr107=($100) >> 8; //@line 476 "klatt.cpp"
      var $conv108=((($shr107)) & 255); //@line 476 "klatt.cpp"
      var $101=HEAP32[((_out_ptr)>>2)]; //@line 476 "klatt.cpp"
      var $incdec_ptr109=(($101+1)&4294967295); //@line 476 "klatt.cpp"
      HEAP32[((_out_ptr)>>2)]=$incdec_ptr109; //@line 476 "klatt.cpp"
      HEAP8[($101)]=$conv108; //@line 476 "klatt.cpp"
      var $102=$value; //@line 478 "klatt.cpp"
      var $conv110=((($102)) & 65535); //@line 478 "klatt.cpp"
      var $103=HEAP32[((_echo_head)>>2)]; //@line 478 "klatt.cpp"
      var $inc111=((($103)+1)&4294967295); //@line 478 "klatt.cpp"
      HEAP32[((_echo_head)>>2)]=$inc111; //@line 478 "klatt.cpp"
      var $arrayidx112=((_echo_buf+$103*2)&4294967295); //@line 478 "klatt.cpp"
      HEAP16[(($arrayidx112)>>1)]=$conv110; //@line 478 "klatt.cpp"
      var $104=HEAP32[((_echo_head)>>2)]; //@line 479 "klatt.cpp"
      var $cmp113=((($104))|0) >= 5500; //@line 479 "klatt.cpp"
      if ($cmp113) { __label__ = 32;; } else { __label__ = 33;; } //@line 479 "klatt.cpp"
      if (__label__ == 32) {
  
        HEAP32[((_echo_head)>>2)]=0; //@line 480 "klatt.cpp"
        ; //@line 480 "klatt.cpp"
      }
  
      var $105=HEAP32[((__ZL12sample_count)>>2)]; //@line 482 "klatt.cpp"
      var $inc116=((($105)+1)&4294967295); //@line 482 "klatt.cpp"
      HEAP32[((__ZL12sample_count)>>2)]=$inc116; //@line 482 "klatt.cpp"
      var $106=HEAPU32[((_out_ptr)>>2)]; //@line 483 "klatt.cpp"
      var $107=HEAPU32[((_out_end)>>2)]; //@line 483 "klatt.cpp"
      var $cmp117=((($106))>>>0) >= ((($107))>>>0); //@line 483 "klatt.cpp"
      if ($cmp117) { __label__ = 34;break $for_cond$2; } //@line 483 "klatt.cpp"
  
      var $108=HEAP32[((((__ZL10kt_globals+36)&4294967295))>>2)]; //@line 260 "klatt.cpp"
      var $inc121=((($108)+1)&4294967295); //@line 260 "klatt.cpp"
      HEAP32[((((__ZL10kt_globals+36)&4294967295))>>2)]=$inc121; //@line 260 "klatt.cpp"
      __label__ = 1;continue $for_cond$2; //@line 260 "klatt.cpp"
    }
    if (__label__ == 36) {
  
      $retval=0; //@line 488 "klatt.cpp"
      ; //@line 488 "klatt.cpp"
    }
    else if (__label__ == 34) {
  
      $retval=1; //@line 485 "klatt.cpp"
      ; //@line 485 "klatt.cpp"
    }
  
    var $109=$retval; //@line 489 "klatt.cpp"
    ;
    return $109; //@line 489 "klatt.cpp"
    return null;
  }
  

  function __Z14SetSynth_KlattiiP7frame_tS0_P7voice_ti($length, $modn, $fr1, $fr2, $v, $control) {
    ;
    var __label__;
    var __lastLabel__ = null;
  
    var $length_addr;
    var $modn_addr;
    var $fr1_addr;
    var $fr2_addr;
    var $v_addr;
    var $control_addr;
    var $ix;
    var $next;
    var $qix;
    var $cmd;
    var $fr3;
    $length_addr=$length;
    $modn_addr=$modn;
    $fr1_addr=$fr1;
    $fr2_addr=$fr2;
    $v_addr=$v;
    $control_addr=$control;
    var $0=HEAP32[((_wvoice)>>2)]; //@line 1122 "klatt.cpp"
    var $cmp=((($0))|0)!=0; //@line 1122 "klatt.cpp"
    if ($cmp) { __label__ = 1;; } else { __label__ = 5;; } //@line 1122 "klatt.cpp"
    if (__label__ == 1) {
  
      var $1=HEAP32[((_wvoice)>>2)]; //@line 1124 "klatt.cpp"
      var $klattv=(($1+112)&4294967295); //@line 1124 "klatt.cpp"
      var $arrayidx=(($klattv)&4294967295); //@line 1124 "klatt.cpp"
      var $2=HEAP32[(($arrayidx)>>2)]; //@line 1124 "klatt.cpp"
      var $cmp1=((($2))|0) > 0; //@line 1124 "klatt.cpp"
      if ($cmp1) { __label__ = 2;; } else { __label__ = 4;; } //@line 1124 "klatt.cpp"
      $land_lhs_true$$if_end$4: do { 
        if (__label__ == 2) {
  
          var $3=HEAP32[((_wvoice)>>2)]; //@line 1124 "klatt.cpp"
          var $klattv2=(($3+112)&4294967295); //@line 1124 "klatt.cpp"
          var $arrayidx3=(($klattv2)&4294967295); //@line 1124 "klatt.cpp"
          var $4=HEAP32[(($arrayidx3)>>2)]; //@line 1124 "klatt.cpp"
          var $cmp4=((($4))|0) <= 3; //@line 1124 "klatt.cpp"
          if (!($cmp4)) { __label__ = 4;break $land_lhs_true$$if_end$4; } //@line 1124 "klatt.cpp"
  
          var $5=HEAP32[((_wvoice)>>2)]; //@line 1126 "klatt.cpp"
          var $klattv6=(($5+112)&4294967295); //@line 1126 "klatt.cpp"
          var $arrayidx7=(($klattv6)&4294967295); //@line 1126 "klatt.cpp"
          var $6=HEAP32[(($arrayidx7)>>2)]; //@line 1126 "klatt.cpp"
          HEAP32[((((__ZL10kt_globals+20)&4294967295))>>2)]=$6; //@line 1126 "klatt.cpp"
          var $7=HEAP32[((((__ZL10kt_globals+20)&4294967295))>>2)]; //@line 1127 "klatt.cpp"
          var $arrayidx8=((__ZL13scale_wav_tab+$7*4)&4294967295); //@line 1127 "klatt.cpp"
          var $8=HEAP32[(($arrayidx8)>>2)]; //@line 1127 "klatt.cpp"
          HEAP32[((((__ZL10kt_globals+184)&4294967295))>>2)]=$8; //@line 1127 "klatt.cpp"
          ; //@line 1128 "klatt.cpp"
        }
      } while(0);
  
      var $9=HEAP32[((_wvoice)>>2)]; //@line 1129 "klatt.cpp"
      var $flutter=(($9+68)&4294967295); //@line 1129 "klatt.cpp"
      var $10=HEAP32[(($flutter)>>2)]; //@line 1129 "klatt.cpp"
      var $div=((((($10))|0)/32)|0); //@line 1129 "klatt.cpp"
      HEAP32[((((__ZL10kt_globals+24)&4294967295))>>2)]=$div; //@line 1129 "klatt.cpp"
      ; //@line 1130 "klatt.cpp"
    }
  
    HEAP32[((__ZL8end_wave803)>>2)]=0; //@line 1132 "klatt.cpp"
    var $11=$control_addr; //@line 1133 "klatt.cpp"
    var $and=($11) & 2; //@line 1133 "klatt.cpp"
    var $tobool=((($and))|0)!=0; //@line 1133 "klatt.cpp"
    if ($tobool) { __label__ = 6;; } else { __label__ = 7;; } //@line 1133 "klatt.cpp"
    if (__label__ == 6) {
  
      HEAP32[((__ZL8end_wave803)>>2)]=1; //@line 1135 "klatt.cpp"
      ; //@line 1136 "klatt.cpp"
    }
  
    var $12=$control_addr; //@line 1137 "klatt.cpp"
    var $and12=($12) & 1; //@line 1137 "klatt.cpp"
    var $tobool13=((($and12))|0)!=0; //@line 1137 "klatt.cpp"
    if ($tobool13) { __label__ = 8;; } else { __label__ = 20;; } //@line 1137 "klatt.cpp"
    $if_then14$$if_end44$12: do { 
      if (__label__ == 8) {
  
        HEAP32[((__ZL8end_wave803)>>2)]=1; //@line 1139 "klatt.cpp"
        var $13=HEAP32[((_wcmdq_head)>>2)]; //@line 1140 "klatt.cpp"
        var $add=((($13)+1)&4294967295); //@line 1140 "klatt.cpp"
        $qix=$add; //@line 1140 "klatt.cpp"
        __lastLabel__ = 8; ; //@line 1140 "klatt.cpp"
        $for_cond$14: while(1) { 
  
          var $14=__lastLabel__ == 19 ? $inc42 : ($add); //@line 1142 "klatt.cpp"
          var $cmp15=((($14))|0) >= 160; //@line 1142 "klatt.cpp"
          if ($cmp15) { __label__ = 10;; } else { __label__ = 11;; } //@line 1142 "klatt.cpp"
          if (__label__ == 10) {
  
            $qix=0; //@line 1142 "klatt.cpp"
            ; //@line 1142 "klatt.cpp"
          }
  
          var $15=$qix; //@line 1143 "klatt.cpp"
          var $16=HEAP32[((_wcmdq_tail)>>2)]; //@line 1143 "klatt.cpp"
          var $cmp18=((($15))|0)==((($16))|0); //@line 1143 "klatt.cpp"
          if ($cmp18) { __label__ = 20;break $if_then14$$if_end44$12; } //@line 1143 "klatt.cpp"
  
          var $17=$qix; //@line 1145 "klatt.cpp"
          var $arrayidx21=((_wcmdq+$17*16)&4294967295); //@line 1145 "klatt.cpp"
          var $arrayidx22=(($arrayidx21)&4294967295); //@line 1145 "klatt.cpp"
          var $18=HEAP32[(($arrayidx22)>>2)]; //@line 1145 "klatt.cpp"
          $cmd=$18; //@line 1145 "klatt.cpp"
          var $19=$cmd; //@line 1146 "klatt.cpp"
          var $cmp23=((($19))|0)==1; //@line 1146 "klatt.cpp"
          if ($cmp23) { __label__ = 13;break $for_cond$14; } //@line 1146 "klatt.cpp"
  
          var $31=$cmd; //@line 1162 "klatt.cpp"
          var $cmp37=((($31))|0)==6; //@line 1162 "klatt.cpp"
          var $32=$cmd; //@line 1162 "klatt.cpp"
          var $cmp38=((($32))|0)==5; //@line 1162 "klatt.cpp"
          var $or_cond=($cmp37) | ($cmp38); //@line 1162 "klatt.cpp"
          if ($or_cond) { __label__ = 20;break $if_then14$$if_end44$12; } //@line 1162 "klatt.cpp"
  
          var $33=$qix; //@line 1140 "klatt.cpp"
          var $inc42=((($33)+1)&4294967295); //@line 1140 "klatt.cpp"
          $qix=$inc42; //@line 1140 "klatt.cpp"
          __lastLabel__ = 19; __label__ = 9;continue $for_cond$14; //@line 1140 "klatt.cpp"
        }
  
        HEAP32[((__ZL8end_wave803)>>2)]=0; //@line 1148 "klatt.cpp"
        var $20=$qix; //@line 1150 "klatt.cpp"
        var $arrayidx25=((_wcmdq+$20*16)&4294967295); //@line 1150 "klatt.cpp"
        var $arrayidx26=(($arrayidx25+8)&4294967295); //@line 1150 "klatt.cpp"
        var $21=HEAP32[(($arrayidx26)>>2)]; //@line 1150 "klatt.cpp"
        var $22=($21); //@line 1150 "klatt.cpp"
        $fr3=$22; //@line 1150 "klatt.cpp"
        $ix=1; //@line 1151 "klatt.cpp"
        __lastLabel__ = 13; ; //@line 1151 "klatt.cpp"
        while(1) { 
  
          var $23=__lastLabel__ == 17 ? $inc : (1); //@line 1151 "klatt.cpp"
          var $cmp28=((($23))|0) < 6; //@line 1151 "klatt.cpp"
          if (!($cmp28)) { __label__ = 20;break $if_then14$$if_end44$12; } //@line 1151 "klatt.cpp"
  
          var $24=$ix; //@line 1153 "klatt.cpp"
          var $25=$fr3; //@line 1153 "klatt.cpp"
          var $ffreq=(($25+2)&4294967295); //@line 1153 "klatt.cpp"
          var $arrayidx29=(($ffreq+$24*2)&4294967295); //@line 1153 "klatt.cpp"
          var $26=HEAP16[(($arrayidx29)>>1)]; //@line 1153 "klatt.cpp"
          var $conv=(tempInt=(($26)),(tempInt>=32768?tempInt-65536:tempInt)); //@line 1153 "klatt.cpp"
          var $27=$ix; //@line 1153 "klatt.cpp"
          var $28=$fr2_addr; //@line 1153 "klatt.cpp"
          var $ffreq30=(($28+2)&4294967295); //@line 1153 "klatt.cpp"
          var $arrayidx31=(($ffreq30+$27*2)&4294967295); //@line 1153 "klatt.cpp"
          var $29=HEAP16[(($arrayidx31)>>1)]; //@line 1153 "klatt.cpp"
          var $conv32=(tempInt=(($29)),(tempInt>=32768?tempInt-65536:tempInt)); //@line 1153 "klatt.cpp"
          var $cmp33=((($conv))|0)!=((($conv32))|0); //@line 1153 "klatt.cpp"
          if ($cmp33) { __label__ = 16;break ; } //@line 1153 "klatt.cpp"
  
          var $30=$ix; //@line 1151 "klatt.cpp"
          var $inc=((($30)+1)&4294967295); //@line 1151 "klatt.cpp"
          $ix=$inc; //@line 1151 "klatt.cpp"
          __lastLabel__ = 17; __label__ = 14;continue ; //@line 1151 "klatt.cpp"
        }
  
        HEAP32[((__ZL8end_wave803)>>2)]=2; //@line 1156 "klatt.cpp"
        ; //@line 1157 "klatt.cpp"
      }
    } while(0);
  
    var $34=$control_addr; //@line 1191 "klatt.cpp"
    var $and45=($34) & 1; //@line 1191 "klatt.cpp"
    var $tobool46=((($and45))|0)!=0; //@line 1191 "klatt.cpp"
    if ($tobool46) { __label__ = 21;; } else { __label__ = 27;; } //@line 1191 "klatt.cpp"
    if (__label__ == 21) {
  
      $ix=1; //@line 1193 "klatt.cpp"
      __lastLabel__ = 21; ; //@line 1193 "klatt.cpp"
      while(1) { 
  
        var $35=__lastLabel__ == 25 ? $inc60 : (1); //@line 1193 "klatt.cpp"
        var $cmp49=((($35))|0) < 6; //@line 1193 "klatt.cpp"
        if (!($cmp49)) { __label__ = 26;break ; } //@line 1193 "klatt.cpp"
  
        var $36=$ix; //@line 1195 "klatt.cpp"
        var $arrayidx51=((((__ZZ14SetSynth_KlattiiP7frame_tS0_P7voice_tiE7prev_fr+2)&4294967295)+$36*2)&4294967295); //@line 1195 "klatt.cpp"
        var $37=HEAP16[(($arrayidx51)>>1)]; //@line 1195 "klatt.cpp"
        var $conv52=(tempInt=(($37)),(tempInt>=32768?tempInt-65536:tempInt)); //@line 1195 "klatt.cpp"
        var $38=$ix; //@line 1195 "klatt.cpp"
        var $39=$fr1_addr; //@line 1195 "klatt.cpp"
        var $ffreq53=(($39+2)&4294967295); //@line 1195 "klatt.cpp"
        var $arrayidx54=(($ffreq53+$38*2)&4294967295); //@line 1195 "klatt.cpp"
        var $40=HEAP16[(($arrayidx54)>>1)]; //@line 1195 "klatt.cpp"
        var $conv55=(tempInt=(($40)),(tempInt>=32768?tempInt-65536:tempInt)); //@line 1195 "klatt.cpp"
        var $cmp56=((($conv52))|0)!=((($conv55))|0); //@line 1195 "klatt.cpp"
        if ($cmp56) { __label__ = 24;break ; } //@line 1195 "klatt.cpp"
  
        var $41=$ix; //@line 1193 "klatt.cpp"
        var $inc60=((($41)+1)&4294967295); //@line 1193 "klatt.cpp"
        $ix=$inc60; //@line 1193 "klatt.cpp"
        __lastLabel__ = 25; __label__ = 22;continue ; //@line 1193 "klatt.cpp"
      }
      if (__label__ == 24) {
  
        __Z10KlattReseti(0); //@line 1199 "klatt.cpp"
        ; //@line 1200 "klatt.cpp"
      }
  
      var $42=$fr2_addr; //@line 1203 "klatt.cpp"
      var $43=$42; //@line 1203 "klatt.cpp"
      var $src$, $dest$, $stop$, $stop4$, $fast$;
      $src$ = $43;
      $dest$ = __ZZ14SetSynth_KlattiiP7frame_tS0_P7voice_tiE7prev_fr;
      $stop$ = $src$ + 64;
      if (($dest$%4) == ($src$%4) && 64 > 8) {
        while ($src$%4 !== 0 && $src$ < $stop$) {
          ; HEAP8[$dest$++] = HEAP8[$src$++];
        }
        $src$ >>= 2;
        $dest$ >>= 2;
        $stop4$ = $stop$ >> 2;
        while ($src$ < $stop4$) {
      ;;;    HEAP32[$dest$++] = HEAP32[$src$++];
        }
        $src$ <<= 2;
        $dest$ <<= 2;
      }while ($src$ < $stop$) {
        ; HEAP8[$dest$++] = HEAP8[$src$++];
      }; //@line 1203 "klatt.cpp"
      ; //@line 1204 "klatt.cpp"
    }
  
    $ix=0; //@line 1206 "klatt.cpp"
    __lastLabel__ = 27; ; //@line 1206 "klatt.cpp"
    $for_body65$39: while(1) { 
  
      var $44=__lastLabel__ == 27 ? 0 : ($inc90); //@line 1208 "klatt.cpp"
      var $cmp66=((($44))|0) >= 5; //@line 1208 "klatt.cpp"
      if ($cmp66) { __label__ = 29;; } else { __label__ = 31;; } //@line 1208 "klatt.cpp"
      $land_lhs_true67$$if_else$41: do { 
        if (__label__ == 29) {
  
          var $45=$fr1_addr; //@line 1208 "klatt.cpp"
          var $frflags=(($45)&4294967295); //@line 1208 "klatt.cpp"
          var $46=HEAP16[(($frflags)>>1)]; //@line 1208 "klatt.cpp"
          var $conv68=(tempInt=(($46)),(tempInt>=32768?tempInt-65536:tempInt)); //@line 1208 "klatt.cpp"
          var $and69=($conv68) & 1; //@line 1208 "klatt.cpp"
          var $cmp70=((($and69))|0)==0; //@line 1208 "klatt.cpp"
          if (!($cmp70)) { __label__ = 31;break $land_lhs_true67$$if_else$41; } //@line 1208 "klatt.cpp"
  
          var $47=$ix; //@line 1210 "klatt.cpp"
          var $arrayidx72=((__ZL6klattp+$47*4)&4294967295); //@line 1210 "klatt.cpp"
          HEAP32[(($arrayidx72)>>2)]=0; //@line 1210 "klatt.cpp"
          var $48=$ix; //@line 1210 "klatt.cpp"
          var $arrayidx73=((__ZL7klattp1+$48*8)&4294967295); //@line 1210 "klatt.cpp"
          HEAPF32[(($arrayidx73)>>2)]=0; //@line 1210 "klatt.cpp"
          var $49=$ix; //@line 1211 "klatt.cpp"
          var $arrayidx74=((__ZL10klattp_inc+$49*8)&4294967295); //@line 1211 "klatt.cpp"
          HEAPF32[(($arrayidx74)>>2)]=0; //@line 1211 "klatt.cpp"
          __label__ = 32;break $land_lhs_true67$$if_else$41; //@line 1212 "klatt.cpp"
        }
      } while(0);
      if (__label__ == 31) {
  
        var $50=$ix; //@line 1215 "klatt.cpp"
        var $51=$fr1_addr; //@line 1215 "klatt.cpp"
        var $klattp=(($51+39)&4294967295); //@line 1215 "klatt.cpp"
        var $arrayidx75=(($klattp+$50)&4294967295); //@line 1215 "klatt.cpp"
        var $52=HEAPU8[($arrayidx75)]; //@line 1215 "klatt.cpp"
        var $conv76=((($52))&255); //@line 1215 "klatt.cpp"
        var $53=$ix; //@line 1215 "klatt.cpp"
        var $arrayidx77=((__ZL6klattp+$53*4)&4294967295); //@line 1215 "klatt.cpp"
        HEAP32[(($arrayidx77)>>2)]=$conv76; //@line 1215 "klatt.cpp"
        var $conv78=((($conv76))|0); //@line 1215 "klatt.cpp"
        var $54=$ix; //@line 1215 "klatt.cpp"
        var $arrayidx79=((__ZL7klattp1+$54*8)&4294967295); //@line 1215 "klatt.cpp"
        HEAPF32[(($arrayidx79)>>2)]=$conv78; //@line 1215 "klatt.cpp"
        var $55=$ix; //@line 1216 "klatt.cpp"
        var $56=$fr2_addr; //@line 1216 "klatt.cpp"
        var $klattp80=(($56+39)&4294967295); //@line 1216 "klatt.cpp"
        var $arrayidx81=(($klattp80+$55)&4294967295); //@line 1216 "klatt.cpp"
        var $57=HEAPU8[($arrayidx81)]; //@line 1216 "klatt.cpp"
        var $conv82=((($57))&255); //@line 1216 "klatt.cpp"
        var $58=$ix; //@line 1216 "klatt.cpp"
        var $arrayidx83=((__ZL6klattp+$58*4)&4294967295); //@line 1216 "klatt.cpp"
        var $59=HEAP32[(($arrayidx83)>>2)]; //@line 1216 "klatt.cpp"
        var $sub=((($conv82)-($59))&4294967295); //@line 1216 "klatt.cpp"
        var $mul=((($sub)*64)&4294967295); //@line 1216 "klatt.cpp"
        var $conv84=((($mul))|0); //@line 1216 "klatt.cpp"
        var $60=$length_addr; //@line 1216 "klatt.cpp"
        var $conv85=((($60))|0); //@line 1216 "klatt.cpp"
        var $div86=($conv84)/($conv85); //@line 1216 "klatt.cpp"
        var $61=$ix; //@line 1216 "klatt.cpp"
        var $arrayidx87=((__ZL10klattp_inc+$61*8)&4294967295); //@line 1216 "klatt.cpp"
        HEAPF32[(($arrayidx87)>>2)]=$div86; //@line 1216 "klatt.cpp"
        ;
      }
  
      var $62=$ix; //@line 1206 "klatt.cpp"
      var $inc90=((($62)+1)&4294967295); //@line 1206 "klatt.cpp"
      $ix=$inc90; //@line 1206 "klatt.cpp"
      var $cmp64=((($inc90))|0) < 10; //@line 1206 "klatt.cpp"
      if ($cmp64) { __lastLabel__ = 32; __label__ = 28;continue $for_body65$39; } else { __lastLabel__ = 32; __label__ = 33;break $for_body65$39; } //@line 1206 "klatt.cpp"
    }
  
    var $63=$length_addr; //@line 1224 "klatt.cpp"
    HEAP32[((__ZL8nsamples802)>>2)]=$63; //@line 1224 "klatt.cpp"
    $ix=1; //@line 1226 "klatt.cpp"
    ; //@line 1226 "klatt.cpp"
    $for_body94$48: while(1) { 
  
      var $64=$ix; //@line 1228 "klatt.cpp"
      var $65=$fr1_addr; //@line 1228 "klatt.cpp"
      var $ffreq95=(($65+2)&4294967295); //@line 1228 "klatt.cpp"
      var $arrayidx96=(($ffreq95+$64*2)&4294967295); //@line 1228 "klatt.cpp"
      var $66=HEAP16[(($arrayidx96)>>1)]; //@line 1228 "klatt.cpp"
      var $conv97=(tempInt=(($66)),(tempInt>=32768?tempInt-65536:tempInt)); //@line 1228 "klatt.cpp"
      var $67=$ix; //@line 1228 "klatt.cpp"
      var $68=$v_addr; //@line 1228 "klatt.cpp"
      var $freq=(($68+144)&4294967295); //@line 1228 "klatt.cpp"
      var $arrayidx98=(($freq+$67*2)&4294967295); //@line 1228 "klatt.cpp"
      var $69=HEAP16[(($arrayidx98)>>1)]; //@line 1228 "klatt.cpp"
      var $conv99=(tempInt=(($69)),(tempInt>=32768?tempInt-65536:tempInt)); //@line 1228 "klatt.cpp"
      var $mul100=((($conv99)*($conv97))&4294967295); //@line 1228 "klatt.cpp"
      var $conv101=((($mul100))|0); //@line 1228 "klatt.cpp"
      var $div102=($conv101)/256; //@line 1228 "klatt.cpp"
      var $70=$ix; //@line 1228 "klatt.cpp"
      var $71=$v_addr; //@line 1228 "klatt.cpp"
      var $freqadd=(($71+198)&4294967295); //@line 1228 "klatt.cpp"
      var $arrayidx103=(($freqadd+$70*2)&4294967295); //@line 1228 "klatt.cpp"
      var $72=HEAP16[(($arrayidx103)>>1)]; //@line 1228 "klatt.cpp"
      var $conv104=(tempInt=(($72)),(tempInt>=32768?tempInt-65536:tempInt)); //@line 1228 "klatt.cpp"
      var $conv105=((($conv104))|0); //@line 1228 "klatt.cpp"
      var $add106=($div102)+($conv105); //@line 1228 "klatt.cpp"
      var $73=$ix; //@line 1228 "klatt.cpp"
      var $arrayidx107=((__ZL5peaks801+$73*80)&4294967295); //@line 1228 "klatt.cpp"
      var $freq1=(($arrayidx107+16)&4294967295); //@line 1228 "klatt.cpp"
      HEAPF32[(($freq1)>>2)]=$add106; //@line 1228 "klatt.cpp"
      var $74=$ix; //@line 1229 "klatt.cpp"
      var $arrayidx108=((__ZL5peaks801+$74*80)&4294967295); //@line 1229 "klatt.cpp"
      var $freq1109=(($arrayidx108+16)&4294967295); //@line 1229 "klatt.cpp"
      var $75=HEAPF32[(($freq1109)>>2)]; //@line 1229 "klatt.cpp"
      var $conv110=((($75))|0); //@line 1229 "klatt.cpp"
      var $76=$ix; //@line 1229 "klatt.cpp"
      var $arrayidx111=((__ZL5peaks801+$76*80)&4294967295); //@line 1229 "klatt.cpp"
      var $freq112=(($arrayidx111)&4294967295); //@line 1229 "klatt.cpp"
      HEAP32[(($freq112)>>2)]=$conv110; //@line 1229 "klatt.cpp"
      var $77=$ix; //@line 1230 "klatt.cpp"
      var $78=$fr2_addr; //@line 1230 "klatt.cpp"
      var $ffreq113=(($78+2)&4294967295); //@line 1230 "klatt.cpp"
      var $arrayidx114=(($ffreq113+$77*2)&4294967295); //@line 1230 "klatt.cpp"
      var $79=HEAP16[(($arrayidx114)>>1)]; //@line 1230 "klatt.cpp"
      var $conv115=(tempInt=(($79)),(tempInt>=32768?tempInt-65536:tempInt)); //@line 1230 "klatt.cpp"
      var $80=$ix; //@line 1230 "klatt.cpp"
      var $81=$v_addr; //@line 1230 "klatt.cpp"
      var $freq116=(($81+144)&4294967295); //@line 1230 "klatt.cpp"
      var $arrayidx117=(($freq116+$80*2)&4294967295); //@line 1230 "klatt.cpp"
      var $82=HEAP16[(($arrayidx117)>>1)]; //@line 1230 "klatt.cpp"
      var $conv118=(tempInt=(($82)),(tempInt>=32768?tempInt-65536:tempInt)); //@line 1230 "klatt.cpp"
      var $mul119=((($conv118)*($conv115))&4294967295); //@line 1230 "klatt.cpp"
      var $conv120=((($mul119))|0); //@line 1230 "klatt.cpp"
      var $div121=($conv120)/256; //@line 1230 "klatt.cpp"
      var $83=$ix; //@line 1230 "klatt.cpp"
      var $84=$v_addr; //@line 1230 "klatt.cpp"
      var $freqadd122=(($84+198)&4294967295); //@line 1230 "klatt.cpp"
      var $arrayidx123=(($freqadd122+$83*2)&4294967295); //@line 1230 "klatt.cpp"
      var $85=HEAP16[(($arrayidx123)>>1)]; //@line 1230 "klatt.cpp"
      var $conv124=(tempInt=(($85)),(tempInt>=32768?tempInt-65536:tempInt)); //@line 1230 "klatt.cpp"
      var $conv125=((($conv124))|0); //@line 1230 "klatt.cpp"
      var $add126=($div121)+($conv125); //@line 1230 "klatt.cpp"
      $next=$add126; //@line 1230 "klatt.cpp"
      var $86=$next; //@line 1231 "klatt.cpp"
      var $87=$ix; //@line 1231 "klatt.cpp"
      var $arrayidx127=((__ZL5peaks801+$87*80)&4294967295); //@line 1231 "klatt.cpp"
      var $freq1128=(($arrayidx127+16)&4294967295); //@line 1231 "klatt.cpp"
      var $88=HEAPF32[(($freq1128)>>2)]; //@line 1231 "klatt.cpp"
      var $sub129=($86)-($88); //@line 1231 "klatt.cpp"
      var $mul130=($sub129)*64; //@line 1231 "klatt.cpp"
      var $89=$length_addr; //@line 1231 "klatt.cpp"
      var $conv131=((($89))|0); //@line 1231 "klatt.cpp"
      var $div132=($mul130)/($conv131); //@line 1231 "klatt.cpp"
      var $90=$ix; //@line 1231 "klatt.cpp"
      var $arrayidx133=((__ZL5peaks801+$90*80)&4294967295); //@line 1231 "klatt.cpp"
      var $freq_inc=(($arrayidx133+48)&4294967295); //@line 1231 "klatt.cpp"
      HEAPF32[(($freq_inc)>>2)]=$div132; //@line 1231 "klatt.cpp"
      var $91=$ix; //@line 1233 "klatt.cpp"
      var $cmp134=((($91))|0) < 4; //@line 1233 "klatt.cpp"
      if ($cmp134) { __label__ = 35;; } else { __label__ = 36;; } //@line 1233 "klatt.cpp"
      if (__label__ == 35) {
  
        var $92=$ix; //@line 1236 "klatt.cpp"
        var $93=$fr1_addr; //@line 1236 "klatt.cpp"
        var $bw=(($93+35)&4294967295); //@line 1236 "klatt.cpp"
        var $arrayidx136=(($bw+$92)&4294967295); //@line 1236 "klatt.cpp"
        var $94=HEAPU8[($arrayidx136)]; //@line 1236 "klatt.cpp"
        var $conv137=((($94))&255); //@line 1236 "klatt.cpp"
        var $mul138=((($conv137)*2)&4294967295); //@line 1236 "klatt.cpp"
        var $conv139=((($mul138))|0); //@line 1236 "klatt.cpp"
        var $95=$ix; //@line 1236 "klatt.cpp"
        var $arrayidx140=((__ZL5peaks801+$95*80)&4294967295); //@line 1236 "klatt.cpp"
        var $bw1=(($arrayidx140+24)&4294967295); //@line 1236 "klatt.cpp"
        HEAPF32[(($bw1)>>2)]=$conv139; //@line 1236 "klatt.cpp"
        var $96=$ix; //@line 1237 "klatt.cpp"
        var $arrayidx141=((__ZL5peaks801+$96*80)&4294967295); //@line 1237 "klatt.cpp"
        var $bw1142=(($arrayidx141+24)&4294967295); //@line 1237 "klatt.cpp"
        var $97=HEAPF32[(($bw1142)>>2)]; //@line 1237 "klatt.cpp"
        var $conv143=((($97))|0); //@line 1237 "klatt.cpp"
        var $98=$ix; //@line 1237 "klatt.cpp"
        var $arrayidx144=((__ZL5peaks801+$98*80)&4294967295); //@line 1237 "klatt.cpp"
        var $bw145=(($arrayidx144+4)&4294967295); //@line 1237 "klatt.cpp"
        HEAP32[(($bw145)>>2)]=$conv143; //@line 1237 "klatt.cpp"
        var $99=$ix; //@line 1238 "klatt.cpp"
        var $100=$fr2_addr; //@line 1238 "klatt.cpp"
        var $bw146=(($100+35)&4294967295); //@line 1238 "klatt.cpp"
        var $arrayidx147=(($bw146+$99)&4294967295); //@line 1238 "klatt.cpp"
        var $101=HEAPU8[($arrayidx147)]; //@line 1238 "klatt.cpp"
        var $conv148=((($101))&255); //@line 1238 "klatt.cpp"
        var $mul149=((($conv148)*2)&4294967295); //@line 1238 "klatt.cpp"
        var $conv150=((($mul149))|0); //@line 1238 "klatt.cpp"
        $next=$conv150; //@line 1238 "klatt.cpp"
        var $102=$next; //@line 1239 "klatt.cpp"
        var $103=$ix; //@line 1239 "klatt.cpp"
        var $arrayidx151=((__ZL5peaks801+$103*80)&4294967295); //@line 1239 "klatt.cpp"
        var $bw1152=(($arrayidx151+24)&4294967295); //@line 1239 "klatt.cpp"
        var $104=HEAPF32[(($bw1152)>>2)]; //@line 1239 "klatt.cpp"
        var $sub153=($102)-($104); //@line 1239 "klatt.cpp"
        var $mul154=($sub153)*64; //@line 1239 "klatt.cpp"
        var $105=$length_addr; //@line 1239 "klatt.cpp"
        var $conv155=((($105))|0); //@line 1239 "klatt.cpp"
        var $div156=($mul154)/($conv155); //@line 1239 "klatt.cpp"
        var $106=$ix; //@line 1239 "klatt.cpp"
        var $arrayidx157=((__ZL5peaks801+$106*80)&4294967295); //@line 1239 "klatt.cpp"
        var $bw_inc=(($arrayidx157+56)&4294967295); //@line 1239 "klatt.cpp"
        HEAPF32[(($bw_inc)>>2)]=$div156; //@line 1239 "klatt.cpp"
        ; //@line 1240 "klatt.cpp"
      }
  
      var $107=$ix; //@line 1226 "klatt.cpp"
      var $inc160=((($107)+1)&4294967295); //@line 1226 "klatt.cpp"
      $ix=$inc160; //@line 1226 "klatt.cpp"
      var $cmp93=((($inc160))|0) < 6; //@line 1226 "klatt.cpp"
      if ($cmp93) { __label__ = 34;continue $for_body94$48; } else { __label__ = 37;break $for_body94$48; } //@line 1226 "klatt.cpp"
    }
  
    var $108=$fr1_addr; //@line 1244 "klatt.cpp"
    var $klattp162=(($108+39)&4294967295); //@line 1244 "klatt.cpp"
    var $arrayidx163=(($klattp162+1)&4294967295); //@line 1244 "klatt.cpp"
    var $109=HEAPU8[($arrayidx163)]; //@line 1244 "klatt.cpp"
    var $conv164=((($109))&255); //@line 1244 "klatt.cpp"
    var $mul165=((($conv164)*2)&4294967295); //@line 1244 "klatt.cpp"
    var $conv166=((($mul165))|0); //@line 1244 "klatt.cpp"
    HEAPF32[((((__ZL5peaks801+16)&4294967295))>>2)]=$conv166; //@line 1244 "klatt.cpp"
    var $110=HEAPF32[((((__ZL5peaks801+16)&4294967295))>>2)]; //@line 1245 "klatt.cpp"
    var $cmp167=($110) == 0; //@line 1245 "klatt.cpp"
    if ($cmp167) { __label__ = 38;; } else { __label__ = 39;; } //@line 1245 "klatt.cpp"
    if (__label__ == 38) {
  
      var $111=HEAP32[((((__ZL8kt_frame+8+36)&4294967295))>>2)]; //@line 1246 "klatt.cpp"
      var $conv169=((($111))|0); //@line 1246 "klatt.cpp"
      HEAPF32[((((__ZL5peaks801+16)&4294967295))>>2)]=$conv169; //@line 1246 "klatt.cpp"
      ; //@line 1246 "klatt.cpp"
    }
  
    var $112=HEAPF32[((((__ZL5peaks801+16)&4294967295))>>2)]; //@line 1248 "klatt.cpp"
    var $conv171=((($112))|0); //@line 1248 "klatt.cpp"
    HEAP32[((((__ZL5peaks801)&4294967295))>>2)]=$conv171; //@line 1248 "klatt.cpp"
    var $113=$fr2_addr; //@line 1249 "klatt.cpp"
    var $klattp172=(($113+39)&4294967295); //@line 1249 "klatt.cpp"
    var $arrayidx173=(($klattp172+1)&4294967295); //@line 1249 "klatt.cpp"
    var $114=HEAPU8[($arrayidx173)]; //@line 1249 "klatt.cpp"
    var $conv174=((($114))&255); //@line 1249 "klatt.cpp"
    var $mul175=((($conv174)*2)&4294967295); //@line 1249 "klatt.cpp"
    var $conv176=((($mul175))|0); //@line 1249 "klatt.cpp"
    $next=$conv176; //@line 1249 "klatt.cpp"
    var $cmp177=($conv176) == 0; //@line 1250 "klatt.cpp"
    if ($cmp177) { __label__ = 40;; } else { __label__ = 41;; } //@line 1250 "klatt.cpp"
    if (__label__ == 40) {
  
      var $115=HEAP32[((((__ZL8kt_frame+8+36)&4294967295))>>2)]; //@line 1251 "klatt.cpp"
      var $conv179=((($115))|0); //@line 1251 "klatt.cpp"
      $next=$conv179; //@line 1251 "klatt.cpp"
      ; //@line 1251 "klatt.cpp"
    }
  
    var $116=$next; //@line 1253 "klatt.cpp"
    var $117=HEAPF32[((((__ZL5peaks801+16)&4294967295))>>2)]; //@line 1253 "klatt.cpp"
    var $sub181=($116)-($117); //@line 1253 "klatt.cpp"
    var $mul182=($sub181)*64; //@line 1253 "klatt.cpp"
    var $118=$length_addr; //@line 1253 "klatt.cpp"
    var $conv183=((($118))|0); //@line 1253 "klatt.cpp"
    var $div184=($mul182)/($conv183); //@line 1253 "klatt.cpp"
    HEAPF32[((((__ZL5peaks801+48)&4294967295))>>2)]=$div184; //@line 1253 "klatt.cpp"
    HEAPF32[((((__ZL5peaks801+24)&4294967295))>>2)]=89; //@line 1255 "klatt.cpp"
    HEAP32[((((__ZL5peaks801+4)&4294967295))>>2)]=89; //@line 1256 "klatt.cpp"
    HEAPF32[((((__ZL5peaks801+56)&4294967295))>>2)]=0; //@line 1257 "klatt.cpp"
    var $119=$fr1_addr; //@line 1259 "klatt.cpp"
    var $frflags185=(($119)&4294967295); //@line 1259 "klatt.cpp"
    var $120=HEAP16[(($frflags185)>>1)]; //@line 1259 "klatt.cpp"
    var $conv186=(tempInt=(($120)),(tempInt>=32768?tempInt-65536:tempInt)); //@line 1259 "klatt.cpp"
    var $and187=($conv186) & 1; //@line 1259 "klatt.cpp"
    var $tobool188=((($and187))|0)!=0; //@line 1259 "klatt.cpp"
    if ($tobool188) { __label__ = 42;; } else { __label__ = 44;; } //@line 1259 "klatt.cpp"
    $if_then189$$if_end236$60: do { 
      if (__label__ == 42) {
  
        $ix=1; //@line 1262 "klatt.cpp"
        ; //@line 1262 "klatt.cpp"
        while(1) { 
  
          var $121=$ix; //@line 1264 "klatt.cpp"
          var $122=$fr1_addr; //@line 1264 "klatt.cpp"
          var $klatt_bp=(($122+56)&4294967295); //@line 1264 "klatt.cpp"
          var $arrayidx193=(($klatt_bp+$121)&4294967295); //@line 1264 "klatt.cpp"
          var $123=HEAPU8[($arrayidx193)]; //@line 1264 "klatt.cpp"
          var $conv194=((($123))&255); //@line 1264 "klatt.cpp"
          var $mul195=((($conv194)*4)&4294967295); //@line 1264 "klatt.cpp"
          var $conv196=((($mul195))|0); //@line 1264 "klatt.cpp"
          var $124=$ix; //@line 1264 "klatt.cpp"
          var $arrayidx197=((__ZL5peaks801+$124*80)&4294967295); //@line 1264 "klatt.cpp"
          var $bp1=(($arrayidx197+40)&4294967295); //@line 1264 "klatt.cpp"
          HEAPF32[(($bp1)>>2)]=$conv196; //@line 1264 "klatt.cpp"
          var $125=$ix; //@line 1265 "klatt.cpp"
          var $arrayidx198=((__ZL5peaks801+$125*80)&4294967295); //@line 1265 "klatt.cpp"
          var $bp1199=(($arrayidx198+40)&4294967295); //@line 1265 "klatt.cpp"
          var $126=HEAPF32[(($bp1199)>>2)]; //@line 1265 "klatt.cpp"
          var $conv200=((($126))|0); //@line 1265 "klatt.cpp"
          var $127=$ix; //@line 1265 "klatt.cpp"
          var $arrayidx201=((__ZL5peaks801+$127*80)&4294967295); //@line 1265 "klatt.cpp"
          var $bp=(($arrayidx201+12)&4294967295); //@line 1265 "klatt.cpp"
          HEAP32[(($bp)>>2)]=$conv200; //@line 1265 "klatt.cpp"
          var $128=$ix; //@line 1266 "klatt.cpp"
          var $129=$fr2_addr; //@line 1266 "klatt.cpp"
          var $klatt_bp202=(($129+56)&4294967295); //@line 1266 "klatt.cpp"
          var $arrayidx203=(($klatt_bp202+$128)&4294967295); //@line 1266 "klatt.cpp"
          var $130=HEAPU8[($arrayidx203)]; //@line 1266 "klatt.cpp"
          var $conv204=((($130))&255); //@line 1266 "klatt.cpp"
          var $mul205=((($conv204)*2)&4294967295); //@line 1266 "klatt.cpp"
          var $conv206=((($mul205))|0); //@line 1266 "klatt.cpp"
          $next=$conv206; //@line 1266 "klatt.cpp"
          var $131=$next; //@line 1267 "klatt.cpp"
          var $132=$ix; //@line 1267 "klatt.cpp"
          var $arrayidx207=((__ZL5peaks801+$132*80)&4294967295); //@line 1267 "klatt.cpp"
          var $bp1208=(($arrayidx207+40)&4294967295); //@line 1267 "klatt.cpp"
          var $133=HEAPF32[(($bp1208)>>2)]; //@line 1267 "klatt.cpp"
          var $sub209=($131)-($133); //@line 1267 "klatt.cpp"
          var $mul210=($sub209)*64; //@line 1267 "klatt.cpp"
          var $134=$length_addr; //@line 1267 "klatt.cpp"
          var $conv211=((($134))|0); //@line 1267 "klatt.cpp"
          var $div212=($mul210)/($conv211); //@line 1267 "klatt.cpp"
          var $135=$ix; //@line 1267 "klatt.cpp"
          var $arrayidx213=((__ZL5peaks801+$135*80)&4294967295); //@line 1267 "klatt.cpp"
          var $bp_inc=(($arrayidx213+72)&4294967295); //@line 1267 "klatt.cpp"
          HEAPF32[(($bp_inc)>>2)]=$div212; //@line 1267 "klatt.cpp"
          var $136=$ix; //@line 1269 "klatt.cpp"
          var $137=$fr1_addr; //@line 1269 "klatt.cpp"
          var $klatt_ap=(($137+49)&4294967295); //@line 1269 "klatt.cpp"
          var $arrayidx214=(($klatt_ap+$136)&4294967295); //@line 1269 "klatt.cpp"
          var $138=HEAPU8[($arrayidx214)]; //@line 1269 "klatt.cpp"
          var $conv215=((($138))&255); //@line 1269 "klatt.cpp"
          var $139=$ix; //@line 1269 "klatt.cpp"
          var $arrayidx216=((__ZL5peaks801+$139*80)&4294967295); //@line 1269 "klatt.cpp"
          var $ap1=(($arrayidx216+32)&4294967295); //@line 1269 "klatt.cpp"
          HEAPF32[(($ap1)>>2)]=$conv215; //@line 1269 "klatt.cpp"
          var $140=$ix; //@line 1270 "klatt.cpp"
          var $arrayidx217=((__ZL5peaks801+$140*80)&4294967295); //@line 1270 "klatt.cpp"
          var $ap1218=(($arrayidx217+32)&4294967295); //@line 1270 "klatt.cpp"
          var $141=HEAPF32[(($ap1218)>>2)]; //@line 1270 "klatt.cpp"
          var $conv219=((($141))|0); //@line 1270 "klatt.cpp"
          var $142=$ix; //@line 1270 "klatt.cpp"
          var $arrayidx220=((__ZL5peaks801+$142*80)&4294967295); //@line 1270 "klatt.cpp"
          var $ap=(($arrayidx220+8)&4294967295); //@line 1270 "klatt.cpp"
          HEAP32[(($ap)>>2)]=$conv219; //@line 1270 "klatt.cpp"
          var $143=$ix; //@line 1271 "klatt.cpp"
          var $144=$fr2_addr; //@line 1271 "klatt.cpp"
          var $klatt_ap221=(($144+49)&4294967295); //@line 1271 "klatt.cpp"
          var $arrayidx222=(($klatt_ap221+$143)&4294967295); //@line 1271 "klatt.cpp"
          var $145=HEAPU8[($arrayidx222)]; //@line 1271 "klatt.cpp"
          var $conv223=((($145))&255); //@line 1271 "klatt.cpp"
          var $mul224=((($conv223)*2)&4294967295); //@line 1271 "klatt.cpp"
          var $conv225=((($mul224))|0); //@line 1271 "klatt.cpp"
          $next=$conv225; //@line 1271 "klatt.cpp"
          var $146=$next; //@line 1272 "klatt.cpp"
          var $147=$ix; //@line 1272 "klatt.cpp"
          var $arrayidx226=((__ZL5peaks801+$147*80)&4294967295); //@line 1272 "klatt.cpp"
          var $ap1227=(($arrayidx226+32)&4294967295); //@line 1272 "klatt.cpp"
          var $148=HEAPF32[(($ap1227)>>2)]; //@line 1272 "klatt.cpp"
          var $sub228=($146)-($148); //@line 1272 "klatt.cpp"
          var $mul229=($sub228)*64; //@line 1272 "klatt.cpp"
          var $149=$length_addr; //@line 1272 "klatt.cpp"
          var $conv230=((($149))|0); //@line 1272 "klatt.cpp"
          var $div231=($mul229)/($conv230); //@line 1272 "klatt.cpp"
          var $150=$ix; //@line 1272 "klatt.cpp"
          var $arrayidx232=((__ZL5peaks801+$150*80)&4294967295); //@line 1272 "klatt.cpp"
          var $ap_inc=(($arrayidx232+64)&4294967295); //@line 1272 "klatt.cpp"
          HEAPF32[(($ap_inc)>>2)]=$div231; //@line 1272 "klatt.cpp"
          var $151=$ix; //@line 1262 "klatt.cpp"
          var $inc234=((($151)+1)&4294967295); //@line 1262 "klatt.cpp"
          $ix=$inc234; //@line 1262 "klatt.cpp"
          var $cmp191=((($inc234))|0) < 7; //@line 1262 "klatt.cpp"
          if ($cmp191) { __label__ = 43;continue ; } else { __label__ = 44;break $if_then189$$if_end236$60; } //@line 1262 "klatt.cpp"
        }
      }
    } while(0);
  
    ;
    return; //@line 1275 "klatt.cpp"
    return;
  }
  

  function __Z14Wavegen_Klatt2iiiP7frame_tS0_($length, $modulation, $resume, $fr1, $fr2) {
    ;
    var __label__;
  
    var $length_addr;
    var $modulation_addr;
    var $resume_addr;
    var $fr1_addr;
    var $fr2_addr;
    $length_addr=$length;
    $modulation_addr=$modulation;
    $resume_addr=$resume;
    $fr1_addr=$fr1;
    $fr2_addr=$fr2;
    var $0=$resume_addr; //@line 1280 "klatt.cpp"
    var $cmp=((($0))|0)==0; //@line 1280 "klatt.cpp"
    if ($cmp) { __label__ = 1;; } else { __label__ = 2;; } //@line 1280 "klatt.cpp"
    if (__label__ == 1) {
  
      var $1=$length_addr; //@line 1281 "klatt.cpp"
      var $2=$modulation_addr; //@line 1281 "klatt.cpp"
      var $3=$fr1_addr; //@line 1281 "klatt.cpp"
      var $4=$fr2_addr; //@line 1281 "klatt.cpp"
      var $5=HEAP32[((_wvoice)>>2)]; //@line 1281 "klatt.cpp"
      __Z14SetSynth_KlattiiP7frame_tS0_P7voice_ti($1, $2, $3, $4, $5, 1); //@line 1281 "klatt.cpp"
      ; //@line 1281 "klatt.cpp"
    }
  
    var $6=$resume_addr; //@line 1283 "klatt.cpp"
    var $call=__Z13Wavegen_Klatti($6); //@line 1283 "klatt.cpp"
    ;
    return $call; //@line 1283 "klatt.cpp"
    return null;
  }
  

  function __Z9KlattInitv() {
    ;
    var __label__;
  
    var $ix;
    HEAP32[((__ZL12sample_count)>>2)]=0; //@line 1309 "klatt.cpp"
    HEAP32[((((__ZL10kt_globals)&4294967295))>>2)]=1; //@line 1311 "klatt.cpp"
    HEAP32[((((__ZL10kt_globals+8)&4294967295))>>2)]=22050; //@line 1312 "klatt.cpp"
    HEAP32[((((__ZL10kt_globals+20)&4294967295))>>2)]=1; //@line 1314 "klatt.cpp"
    var $0=HEAP32[((((__ZL10kt_globals+20)&4294967295))>>2)]; //@line 1315 "klatt.cpp"
    var $arrayidx=((__ZL13scale_wav_tab+$0*4)&4294967295); //@line 1315 "klatt.cpp"
    var $1=HEAP32[(($arrayidx)>>2)]; //@line 1315 "klatt.cpp"
    HEAP32[((((__ZL10kt_globals+184)&4294967295))>>2)]=$1; //@line 1315 "klatt.cpp"
    HEAP32[((((__ZL10kt_globals+172)&4294967295))>>2)]=((__ZZ9KlattInitvE15natural_samples)&4294967295); //@line 1316 "klatt.cpp"
    HEAP32[((((__ZL10kt_globals+160)&4294967295))>>2)]=100; //@line 1317 "klatt.cpp"
    HEAPF32[((((__ZL10kt_globals+164)&4294967295))>>2)]=3; //@line 1318 "klatt.cpp"
    var $2=HEAP32[((((__ZL10kt_globals+8)&4294967295))>>2)]; //@line 1319 "klatt.cpp"
    var $mul=((($2)*10)&4294967295); //@line 1319 "klatt.cpp"
    var $div=((((($mul))|0)/1000)|0); //@line 1319 "klatt.cpp"
    HEAP32[((((__ZL10kt_globals+28)&4294967295))>>2)]=$div; //@line 1319 "klatt.cpp"
    HEAP32[((((__ZL10kt_globals+4)&4294967295))>>2)]=0; //@line 1320 "klatt.cpp"
    HEAP32[((((__ZL10kt_globals+24)&4294967295))>>2)]=20; //@line 1321 "klatt.cpp"
    __Z10KlattReseti(2); //@line 1323 "klatt.cpp"
    $ix=0; //@line 1326 "klatt.cpp"
    ; //@line 1326 "klatt.cpp"
    while(1) { 
  
      var $3=$ix; //@line 1328 "klatt.cpp"
      var $arrayidx1=((__ZZ9KlattInitvE10formant_hz+$3*2)&4294967295); //@line 1328 "klatt.cpp"
      var $4=HEAP16[(($arrayidx1)>>1)]; //@line 1328 "klatt.cpp"
      var $conv=(tempInt=(($4)),(tempInt>=32768?tempInt-65536:tempInt)); //@line 1328 "klatt.cpp"
      var $5=$ix; //@line 1328 "klatt.cpp"
      var $arrayidx2=((((__ZL8kt_frame+8)&4294967295)+$5*4)&4294967295); //@line 1328 "klatt.cpp"
      HEAP32[(($arrayidx2)>>2)]=$conv; //@line 1328 "klatt.cpp"
      var $6=$ix; //@line 1329 "klatt.cpp"
      var $arrayidx3=((__ZZ9KlattInitvE9bandwidth+$6*2)&4294967295); //@line 1329 "klatt.cpp"
      var $7=HEAP16[(($arrayidx3)>>1)]; //@line 1329 "klatt.cpp"
      var $conv4=(tempInt=(($7)),(tempInt>=32768?tempInt-65536:tempInt)); //@line 1329 "klatt.cpp"
      var $8=$ix; //@line 1329 "klatt.cpp"
      var $arrayidx5=((((__ZL8kt_frame+48)&4294967295)+$8*4)&4294967295); //@line 1329 "klatt.cpp"
      HEAP32[(($arrayidx5)>>2)]=$conv4; //@line 1329 "klatt.cpp"
      var $9=$ix; //@line 1330 "klatt.cpp"
      var $arrayidx6=((__ZZ9KlattInitvE12parallel_amp+$9*2)&4294967295); //@line 1330 "klatt.cpp"
      var $10=HEAP16[(($arrayidx6)>>1)]; //@line 1330 "klatt.cpp"
      var $conv7=(tempInt=(($10)),(tempInt>=32768?tempInt-65536:tempInt)); //@line 1330 "klatt.cpp"
      var $11=$ix; //@line 1330 "klatt.cpp"
      var $arrayidx8=((((__ZL8kt_frame+88)&4294967295)+$11*4)&4294967295); //@line 1330 "klatt.cpp"
      HEAP32[(($arrayidx8)>>2)]=$conv7; //@line 1330 "klatt.cpp"
      var $12=$ix; //@line 1331 "klatt.cpp"
      var $arrayidx9=((__ZZ9KlattInitvE11parallel_bw+$12*2)&4294967295); //@line 1331 "klatt.cpp"
      var $13=HEAP16[(($arrayidx9)>>1)]; //@line 1331 "klatt.cpp"
      var $conv10=(tempInt=(($13)),(tempInt>=32768?tempInt-65536:tempInt)); //@line 1331 "klatt.cpp"
      var $14=$ix; //@line 1331 "klatt.cpp"
      var $arrayidx11=((((__ZL8kt_frame+128)&4294967295)+$14*4)&4294967295); //@line 1331 "klatt.cpp"
      HEAP32[(($arrayidx11)>>2)]=$conv10; //@line 1331 "klatt.cpp"
      var $15=$ix; //@line 1326 "klatt.cpp"
      var $inc=((($15)+1)&4294967295); //@line 1326 "klatt.cpp"
      $ix=$inc; //@line 1326 "klatt.cpp"
      var $cmp=((($inc))|0) <= 9; //@line 1326 "klatt.cpp"
      if ($cmp) { __label__ = 1;continue ; } else { __label__ = 2;break ; } //@line 1326 "klatt.cpp"
    }
  
    HEAP32[((((__ZL8kt_frame+248)&4294967295))>>2)]=89; //@line 1333 "klatt.cpp"
    HEAP32[((((__ZL8kt_frame)&4294967295))>>2)]=1000; //@line 1335 "klatt.cpp"
    HEAP32[((((__ZL8kt_frame+4)&4294967295))>>2)]=59; //@line 1336 "klatt.cpp"
    HEAP32[((((__ZL8kt_frame+168)&4294967295))>>2)]=0; //@line 1337 "klatt.cpp"
    HEAP32[((((__ZL8kt_frame+172)&4294967295))>>2)]=40; //@line 1338 "klatt.cpp"
    HEAP32[((((__ZL8kt_frame+176)&4294967295))>>2)]=0; //@line 1339 "klatt.cpp"
    HEAP32[((((__ZL8kt_frame+180)&4294967295))>>2)]=0; //@line 1340 "klatt.cpp"
    HEAP32[((((__ZL8kt_frame+184)&4294967295))>>2)]=50; //@line 1341 "klatt.cpp"
    HEAP32[((((__ZL8kt_frame+188)&4294967295))>>2)]=0; //@line 1342 "klatt.cpp"
    HEAP32[((((__ZL8kt_frame+192)&4294967295))>>2)]=0; //@line 1343 "klatt.cpp"
    HEAP32[((((__ZL8kt_frame+196)&4294967295))>>2)]=0; //@line 1344 "klatt.cpp"
    HEAP32[((((__ZL8kt_frame+200)&4294967295))>>2)]=62; //@line 1345 "klatt.cpp"
    ;
    return; //@line 1346 "klatt.cpp"
    return;
  }
  

  function __ZL7flutterP13klatt_frame_t($frame) {
    ;
    var __label__;
  
    var $frame_addr;
    var $delta_f0;
    var $fla;
    var $flb;
    var $flc;
    var $fld;
    var $fle;
    $frame_addr=$frame;
    var $0=HEAP32[((((__ZL10kt_globals+24)&4294967295))>>2)]; //@line 159 "klatt.cpp"
    var $conv=((($0))|0); //@line 159 "klatt.cpp"
    var $div=($conv)/50; //@line 159 "klatt.cpp"
    $fla=$div; //@line 159 "klatt.cpp"
    var $1=HEAP32[((((__ZL10kt_globals+176)&4294967295))>>2)]; //@line 160 "klatt.cpp"
    var $conv1=((($1))|0); //@line 160 "klatt.cpp"
    var $div2=($conv1)/100; //@line 160 "klatt.cpp"
    $flb=$div2; //@line 160 "klatt.cpp"
    var $2=HEAP32[((__ZZL7flutterP13klatt_frame_tE10time_count)>>2)]; //@line 164 "klatt.cpp"
    var $conv3=((($2))|0); //@line 164 "klatt.cpp"
    var $mul=39.898227289999994*($conv3); //@line 164 "klatt.cpp"
    var $call=_sin($mul); //@line 164 "klatt.cpp"
    $flc=$call; //@line 164 "klatt.cpp"
    var $3=HEAP32[((__ZZL7flutterP13klatt_frame_tE10time_count)>>2)]; //@line 165 "klatt.cpp"
    var $conv4=((($3))|0); //@line 165 "klatt.cpp"
    var $mul5=22.305308169999996*($conv4); //@line 165 "klatt.cpp"
    var $call6=_sin($mul5); //@line 165 "klatt.cpp"
    $fld=$call6; //@line 165 "klatt.cpp"
    var $4=HEAP32[((__ZZL7flutterP13klatt_frame_tE10time_count)>>2)]; //@line 166 "klatt.cpp"
    var $conv7=((($4))|0); //@line 166 "klatt.cpp"
    var $mul8=14.76548569*($conv7); //@line 166 "klatt.cpp"
    var $call9=_sin($mul8); //@line 166 "klatt.cpp"
    $fle=$call9; //@line 166 "klatt.cpp"
    var $5=$fla; //@line 167 "klatt.cpp"
    var $6=$flb; //@line 167 "klatt.cpp"
    var $mul10=($5)*($6); //@line 167 "klatt.cpp"
    var $7=$flc; //@line 167 "klatt.cpp"
    var $8=$fld; //@line 167 "klatt.cpp"
    var $add=($7)+($8); //@line 167 "klatt.cpp"
    var $9=$fle; //@line 167 "klatt.cpp"
    var $add11=($add)+($9); //@line 167 "klatt.cpp"
    var $mul12=($mul10)*($add11); //@line 167 "klatt.cpp"
    var $mul13=($mul12)*10; //@line 167 "klatt.cpp"
    $delta_f0=$mul13; //@line 167 "klatt.cpp"
    var $10=$frame_addr; //@line 168 "klatt.cpp"
    var $F0hz10=(($10)&4294967295); //@line 168 "klatt.cpp"
    var $11=HEAP32[(($F0hz10)>>2)]; //@line 168 "klatt.cpp"
    var $12=$delta_f0; //@line 168 "klatt.cpp"
    var $conv14=((($12))|0); //@line 168 "klatt.cpp"
    var $add15=((($conv14)+($11))&4294967295); //@line 168 "klatt.cpp"
    var $13=$frame_addr; //@line 168 "klatt.cpp"
    var $F0hz1016=(($13)&4294967295); //@line 168 "klatt.cpp"
    HEAP32[(($F0hz1016)>>2)]=$add15; //@line 168 "klatt.cpp"
    var $14=HEAP32[((__ZZL7flutterP13klatt_frame_tE10time_count)>>2)]; //@line 169 "klatt.cpp"
    var $inc=((($14)+1)&4294967295); //@line 169 "klatt.cpp"
    HEAP32[((__ZZL7flutterP13klatt_frame_tE10time_count)>>2)]=$inc; //@line 169 "klatt.cpp"
    ;
    return; //@line 170 "klatt.cpp"
    return;
  }
  

  function __ZL9gen_noised($noise) {
    ;
    var __label__;
  
    var $noise_addr;
    var $temp;
    $noise_addr=$noise;
    var $call=_rand(); //@line 928 "klatt.cpp"
    var $rem=((($call))|0)%16383; //@line 928 "klatt.cpp"
    var $add=((($rem)-8191)&4294967295); //@line 928 "klatt.cpp"
    $temp=$add; //@line 928 "klatt.cpp"
    var $0=$temp; //@line 929 "klatt.cpp"
    HEAP32[((((__ZL10kt_globals+52)&4294967295))>>2)]=$0; //@line 929 "klatt.cpp"
    var $1=HEAP32[((((__ZL10kt_globals+52)&4294967295))>>2)]; //@line 931 "klatt.cpp"
    var $conv=((($1))|0); //@line 931 "klatt.cpp"
    var $2=HEAPF32[((__ZZL9gen_noisedE5nlast)>>2)]; //@line 931 "klatt.cpp"
    var $mul=0.75*($2); //@line 931 "klatt.cpp"
    var $add1=($conv)+($mul); //@line 931 "klatt.cpp"
    $noise_addr=$add1; //@line 931 "klatt.cpp"
    var $3=$noise_addr; //@line 932 "klatt.cpp"
    HEAPF32[((__ZZL9gen_noisedE5nlast)>>2)]=$3; //@line 932 "klatt.cpp"
    var $4=$noise_addr; //@line 934 "klatt.cpp"
    ;
    return $4; //@line 934 "klatt.cpp"
    return null;
  }
  

  function __ZL16impulsive_sourcev() {
    ;
    var __label__;
  
    var $0=HEAP32[((((__ZL10kt_globals+32)&4294967295))>>2)]; //@line 627 "klatt.cpp"
    var $cmp=((($0))|0) < 3; //@line 627 "klatt.cpp"
    if ($cmp) { __label__ = 1;; } else { __label__ = 2;; } //@line 627 "klatt.cpp"
    if (__label__ == 1) {
  
      var $1=HEAP32[((((__ZL10kt_globals+32)&4294967295))>>2)]; //@line 629 "klatt.cpp"
      var $arrayidx=((__ZZL16impulsive_sourcevE7doublet+$1*8)&4294967295); //@line 629 "klatt.cpp"
      var $2=HEAPF32[(($arrayidx)>>2)]; //@line 629 "klatt.cpp"
      HEAPF32[((__ZZL16impulsive_sourcevE5vwave)>>2)]=$2; //@line 629 "klatt.cpp"
      ; //@line 630 "klatt.cpp"
    }
    else if (__label__ == 2) {
  
      HEAPF32[((__ZZL16impulsive_sourcevE5vwave)>>2)]=0; //@line 633 "klatt.cpp"
      ;
    }
  
    var $3=HEAPF32[((__ZZL16impulsive_sourcevE5vwave)>>2)]; //@line 636 "klatt.cpp"
    var $call=__ZL9resonatorP11resonator_td(((__ZL10kt_globals+188+1088)&4294967295), $3); //@line 636 "klatt.cpp"
    ;
    return $call; //@line 636 "klatt.cpp"
    return null;
  }
  

  function __ZL14natural_sourcev() {
    ;
    var __label__;
  
    var $retval;
    var $lgtemp;
    var $0=HEAP32[((((__ZL10kt_globals+32)&4294967295))>>2)]; //@line 653 "klatt.cpp"
    var $1=HEAP32[((((__ZL10kt_globals+44)&4294967295))>>2)]; //@line 653 "klatt.cpp"
    var $cmp=((($0))|0) < ((($1))|0); //@line 653 "klatt.cpp"
    if ($cmp) { __label__ = 1;; } else { __label__ = 2;; } //@line 653 "klatt.cpp"
    if (__label__ == 1) {
  
      var $2=HEAPF32[((((__ZL10kt_globals+64)&4294967295))>>2)]; //@line 655 "klatt.cpp"
      var $3=HEAPF32[((((__ZL10kt_globals+56)&4294967295))>>2)]; //@line 655 "klatt.cpp"
      var $sub=($3)-($2); //@line 655 "klatt.cpp"
      HEAPF32[((((__ZL10kt_globals+56)&4294967295))>>2)]=$sub; //@line 655 "klatt.cpp"
      var $4=HEAPF32[((((__ZL10kt_globals+56)&4294967295))>>2)]; //@line 656 "klatt.cpp"
      var $5=HEAPF32[((__ZZL14natural_sourcevE5vwave)>>2)]; //@line 656 "klatt.cpp"
      var $add=($5)+($4); //@line 656 "klatt.cpp"
      HEAPF32[((__ZZL14natural_sourcevE5vwave)>>2)]=$add; //@line 656 "klatt.cpp"
      var $6=HEAPF32[((__ZZL14natural_sourcevE5vwave)>>2)]; //@line 657 "klatt.cpp"
      var $mul=($6)*0.028; //@line 657 "klatt.cpp"
      $lgtemp=$mul; //@line 657 "klatt.cpp"
      var $7=$lgtemp; //@line 659 "klatt.cpp"
      $retval=$7; //@line 659 "klatt.cpp"
      ; //@line 659 "klatt.cpp"
    }
    else if (__label__ == 2) {
  
      HEAPF32[((__ZZL14natural_sourcevE5vwave)>>2)]=0; //@line 663 "klatt.cpp"
      $retval=0; //@line 664 "klatt.cpp"
      ; //@line 664 "klatt.cpp"
    }
  
    var $8=$retval; //@line 666 "klatt.cpp"
    ;
    return $8; //@line 666 "klatt.cpp"
    return null;
  }
  

  function __ZL14sampled_sourcev() {
    ;
    var __label__;
  
    var $itemp;
    var $ftemp;
    var $result;
    var $diff_value;
    var $current_value;
    var $next_value;
    var $temp_diff;
    var $0=HEAP32[((((__ZL10kt_globals+40)&4294967295))>>2)]; //@line 191 "klatt.cpp"
    var $cmp=((($0))|0)!=0; //@line 191 "klatt.cpp"
    if ($cmp) { __label__ = 1;; } else { __label__ = 2;; } //@line 191 "klatt.cpp"
    if (__label__ == 1) {
  
      var $1=HEAP32[((((__ZL10kt_globals+32)&4294967295))>>2)]; //@line 193 "klatt.cpp"
      var $conv=((($1))|0); //@line 193 "klatt.cpp"
      $ftemp=$conv; //@line 193 "klatt.cpp"
      var $2=$ftemp; //@line 194 "klatt.cpp"
      var $3=HEAP32[((((__ZL10kt_globals+40)&4294967295))>>2)]; //@line 194 "klatt.cpp"
      var $conv1=((($3))|0); //@line 194 "klatt.cpp"
      var $div=($2)/($conv1); //@line 194 "klatt.cpp"
      $ftemp=$div; //@line 194 "klatt.cpp"
      var $4=$ftemp; //@line 195 "klatt.cpp"
      var $5=HEAP32[((((__ZL10kt_globals+160)&4294967295))>>2)]; //@line 195 "klatt.cpp"
      var $conv2=((($5))|0); //@line 195 "klatt.cpp"
      var $mul=($4)*($conv2); //@line 195 "klatt.cpp"
      $ftemp=$mul; //@line 195 "klatt.cpp"
      var $6=$ftemp; //@line 196 "klatt.cpp"
      var $conv3=((($6))|0); //@line 196 "klatt.cpp"
      $itemp=$conv3; //@line 196 "klatt.cpp"
      var $7=$ftemp; //@line 198 "klatt.cpp"
      var $8=$itemp; //@line 198 "klatt.cpp"
      var $conv4=((($8))|0); //@line 198 "klatt.cpp"
      var $sub=($7)-($conv4); //@line 198 "klatt.cpp"
      $temp_diff=$sub; //@line 198 "klatt.cpp"
      var $9=$itemp; //@line 200 "klatt.cpp"
      var $10=HEAP32[((((__ZL10kt_globals+172)&4294967295))>>2)]; //@line 200 "klatt.cpp"
      var $arrayidx=(($10+2*$9)&4294967295); //@line 200 "klatt.cpp"
      var $11=HEAP16[(($arrayidx)>>1)]; //@line 200 "klatt.cpp"
      var $conv5=(tempInt=(($11)),(tempInt>=32768?tempInt-65536:tempInt)); //@line 200 "klatt.cpp"
      $current_value=$conv5; //@line 200 "klatt.cpp"
      var $12=$itemp; //@line 201 "klatt.cpp"
      var $add=((($12)+1)&4294967295); //@line 201 "klatt.cpp"
      var $13=HEAP32[((((__ZL10kt_globals+172)&4294967295))>>2)]; //@line 201 "klatt.cpp"
      var $arrayidx6=(($13+2*$add)&4294967295); //@line 201 "klatt.cpp"
      var $14=HEAP16[(($arrayidx6)>>1)]; //@line 201 "klatt.cpp"
      var $conv7=(tempInt=(($14)),(tempInt>=32768?tempInt-65536:tempInt)); //@line 201 "klatt.cpp"
      $next_value=$conv7; //@line 201 "klatt.cpp"
      var $15=$next_value; //@line 203 "klatt.cpp"
      var $conv8=((($15))|0); //@line 203 "klatt.cpp"
      var $16=$current_value; //@line 203 "klatt.cpp"
      var $conv9=((($16))|0); //@line 203 "klatt.cpp"
      var $sub10=($conv8)-($conv9); //@line 203 "klatt.cpp"
      $diff_value=$sub10; //@line 203 "klatt.cpp"
      var $17=$diff_value; //@line 204 "klatt.cpp"
      var $18=$temp_diff; //@line 204 "klatt.cpp"
      var $mul11=($17)*($18); //@line 204 "klatt.cpp"
      $diff_value=$mul11; //@line 204 "klatt.cpp"
      var $19=$itemp; //@line 206 "klatt.cpp"
      var $20=HEAP32[((((__ZL10kt_globals+172)&4294967295))>>2)]; //@line 206 "klatt.cpp"
      var $arrayidx12=(($20+2*$19)&4294967295); //@line 206 "klatt.cpp"
      var $21=HEAP16[(($arrayidx12)>>1)]; //@line 206 "klatt.cpp"
      var $conv13=(tempInt=(($21)),(tempInt>=32768?tempInt-65536:tempInt)); //@line 206 "klatt.cpp"
      var $conv14=((($conv13))|0); //@line 206 "klatt.cpp"
      var $22=$diff_value; //@line 206 "klatt.cpp"
      var $add15=($conv14)+($22); //@line 206 "klatt.cpp"
      $result=$add15; //@line 206 "klatt.cpp"
      var $23=$result; //@line 207 "klatt.cpp"
      var $24=HEAPF32[((((__ZL10kt_globals+164)&4294967295))>>2)]; //@line 207 "klatt.cpp"
      var $mul16=($23)*($24); //@line 207 "klatt.cpp"
      $result=$mul16; //@line 207 "klatt.cpp"
      ; //@line 208 "klatt.cpp"
    }
    else if (__label__ == 2) {
  
      $result=0; //@line 211 "klatt.cpp"
      ;
    }
  
    var $25=$result; //@line 213 "klatt.cpp"
    ;
    return $25; //@line 213 "klatt.cpp"
    return null;
  }
  

  function __ZL21pitch_synch_par_resetP13klatt_frame_t($frame) {
    ;
    var __label__;
    var __lastLabel__ = null;
  
    var $frame_addr;
    var $temp;
    var $temp1;
    $frame_addr=$frame;
    var $0=$frame_addr; //@line 727 "klatt.cpp"
    var $F0hz10=(($0)&4294967295); //@line 727 "klatt.cpp"
    var $1=HEAP32[(($F0hz10)>>2)]; //@line 727 "klatt.cpp"
    var $cmp=((($1))|0) > 0; //@line 727 "klatt.cpp"
    if ($cmp) { __label__ = 1;; } else { __label__ = 15;; } //@line 727 "klatt.cpp"
    if (__label__ == 1) {
  
      var $2=HEAP32[((((__ZL10kt_globals+8)&4294967295))>>2)]; //@line 731 "klatt.cpp"
      var $mul=((($2)*40)&4294967295); //@line 731 "klatt.cpp"
      var $3=$frame_addr; //@line 731 "klatt.cpp"
      var $F0hz101=(($3)&4294967295); //@line 731 "klatt.cpp"
      var $4=HEAP32[(($F0hz101)>>2)]; //@line 731 "klatt.cpp"
      var $div=((((($mul))|0)/((($4))|0))|0); //@line 731 "klatt.cpp"
      HEAP32[((((__ZL10kt_globals+40)&4294967295))>>2)]=$div; //@line 731 "klatt.cpp"
      var $5=$frame_addr; //@line 734 "klatt.cpp"
      var $AVdb_tmp=(($5+204)&4294967295); //@line 734 "klatt.cpp"
      var $6=HEAP32[(($AVdb_tmp)>>2)]; //@line 734 "klatt.cpp"
      var $call=__ZL7DBtoLINl($6); //@line 734 "klatt.cpp"
      HEAPF32[((((__ZL10kt_globals+112)&4294967295))>>2)]=$call; //@line 734 "klatt.cpp"
      var $7=HEAP32[((((__ZL10kt_globals+40)&4294967295))>>2)]; //@line 738 "klatt.cpp"
      HEAP32[((((__ZL10kt_globals+48)&4294967295))>>2)]=$7; //@line 738 "klatt.cpp"
      var $8=$frame_addr; //@line 739 "klatt.cpp"
      var $AVdb_tmp2=(($8+204)&4294967295); //@line 739 "klatt.cpp"
      var $9=HEAP32[(($AVdb_tmp2)>>2)]; //@line 739 "klatt.cpp"
      var $cmp3=((($9))|0) > 0; //@line 739 "klatt.cpp"
      if ($cmp3) { __label__ = 2;; } else { __label__ = 3;; } //@line 739 "klatt.cpp"
      if (__label__ == 2) {
  
        var $10=HEAP32[((((__ZL10kt_globals+48)&4294967295))>>2)]; //@line 741 "klatt.cpp"
        var $shr=($10) >> 1; //@line 741 "klatt.cpp"
        HEAP32[((((__ZL10kt_globals+48)&4294967295))>>2)]=$shr; //@line 741 "klatt.cpp"
        ; //@line 742 "klatt.cpp"
      }
  
      var $11=$frame_addr; //@line 746 "klatt.cpp"
      var $Aturb=(($11+176)&4294967295); //@line 746 "klatt.cpp"
      var $12=HEAP32[(($Aturb)>>2)]; //@line 746 "klatt.cpp"
      var $call5=__ZL7DBtoLINl($12); //@line 746 "klatt.cpp"
      var $mul6=($call5)*0.1; //@line 746 "klatt.cpp"
      HEAPF32[((((__ZL10kt_globals+144)&4294967295))>>2)]=$mul6; //@line 746 "klatt.cpp"
      var $13=$frame_addr; //@line 750 "klatt.cpp"
      var $Kopen=(($13+172)&4294967295); //@line 750 "klatt.cpp"
      var $14=HEAP32[(($Kopen)>>2)]; //@line 750 "klatt.cpp"
      var $mul7=((($14)*4)&4294967295); //@line 750 "klatt.cpp"
      HEAP32[((((__ZL10kt_globals+44)&4294967295))>>2)]=$mul7; //@line 750 "klatt.cpp"
      var $15=HEAP32[((((__ZL10kt_globals+20)&4294967295))>>2)]; //@line 752 "klatt.cpp"
      var $cmp8=((($15))|0)==1; //@line 752 "klatt.cpp"
      var $16=HEAP32[((((__ZL10kt_globals+44)&4294967295))>>2)]; //@line 752 "klatt.cpp"
      var $cmp9=((($16))|0) > 263; //@line 752 "klatt.cpp"
      var $or_cond=($cmp8) & ($cmp9); //@line 752 "klatt.cpp"
      if ($or_cond) { __label__ = 4;; } else { __label__ = 5;; } //@line 752 "klatt.cpp"
      if (__label__ == 4) {
  
        HEAP32[((((__ZL10kt_globals+44)&4294967295))>>2)]=263; //@line 754 "klatt.cpp"
        ; //@line 755 "klatt.cpp"
      }
  
      var $17=HEAP32[((((__ZL10kt_globals+44)&4294967295))>>2)]; //@line 757 "klatt.cpp"
      var $18=HEAP32[((((__ZL10kt_globals+40)&4294967295))>>2)]; //@line 757 "klatt.cpp"
      var $sub=((($18)-1)&4294967295); //@line 757 "klatt.cpp"
      var $cmp12=((($17))|0) >= ((($sub))|0); //@line 757 "klatt.cpp"
      if ($cmp12) { __lastLabel__ = 5; __label__ = 6;; } else { __lastLabel__ = 5; __label__ = 7;; } //@line 757 "klatt.cpp"
      if (__label__ == 6) {
  
        var $19=HEAP32[((((__ZL10kt_globals+40)&4294967295))>>2)]; //@line 760 "klatt.cpp"
        var $sub14=((($19)-2)&4294967295); //@line 760 "klatt.cpp"
        HEAP32[((((__ZL10kt_globals+44)&4294967295))>>2)]=$sub14; //@line 760 "klatt.cpp"
        __lastLabel__ = 6; ; //@line 761 "klatt.cpp"
      }
  
      var $20=__lastLabel__ == 6 ? $sub14 : ($17); //@line 763 "klatt.cpp"
      var $cmp16=((($20))|0) < 40; //@line 763 "klatt.cpp"
      if ($cmp16) { __label__ = 8;; } else { __label__ = 9;; } //@line 763 "klatt.cpp"
      if (__label__ == 8) {
  
        HEAP32[((((__ZL10kt_globals+44)&4294967295))>>2)]=40; //@line 768 "klatt.cpp"
        ; //@line 769 "klatt.cpp"
      }
  
      var $21=HEAP32[((((__ZL10kt_globals+44)&4294967295))>>2)]; //@line 774 "klatt.cpp"
      var $sub19=((($21)-40)&4294967295); //@line 774 "klatt.cpp"
      var $arrayidx=((__ZZL21pitch_synch_par_resetP13klatt_frame_tE2B0+$sub19*2)&4294967295); //@line 774 "klatt.cpp"
      var $22=HEAP16[(($arrayidx)>>1)]; //@line 774 "klatt.cpp"
      var $conv=(tempInt=(($22)),(tempInt>=32768?tempInt-65536:tempInt)); //@line 774 "klatt.cpp"
      HEAPF32[((((__ZL10kt_globals+64)&4294967295))>>2)]=$conv; //@line 774 "klatt.cpp"
      var $23=HEAPF32[((((__ZL10kt_globals+64)&4294967295))>>2)]; //@line 775 "klatt.cpp"
      var $24=HEAP32[((((__ZL10kt_globals+44)&4294967295))>>2)]; //@line 775 "klatt.cpp"
      var $conv20=((($24))|0); //@line 775 "klatt.cpp"
      var $mul21=($23)*($conv20); //@line 775 "klatt.cpp"
      var $mul22=($mul21)*0.333; //@line 775 "klatt.cpp"
      HEAPF32[((((__ZL10kt_globals+56)&4294967295))>>2)]=$mul22; //@line 775 "klatt.cpp"
      var $25=HEAP32[((((__ZL10kt_globals+8)&4294967295))>>2)]; //@line 779 "klatt.cpp"
      var $26=HEAP32[((((__ZL10kt_globals+44)&4294967295))>>2)]; //@line 779 "klatt.cpp"
      var $div23=((((($25))|0)/((($26))|0))|0); //@line 779 "klatt.cpp"
      $temp=$div23; //@line 779 "klatt.cpp"
      var $27=$temp; //@line 781 "klatt.cpp"
      __ZL6setabcllP11resonator_t(0, $27, ((__ZL10kt_globals+188+1088)&4294967295)); //@line 781 "klatt.cpp"
      var $28=HEAP32[((((__ZL10kt_globals+44)&4294967295))>>2)]; //@line 785 "klatt.cpp"
      var $conv24=((($28))|0); //@line 785 "klatt.cpp"
      var $mul25=($conv24)*0.00833; //@line 785 "klatt.cpp"
      $temp1=$mul25; //@line 785 "klatt.cpp"
      var $29=$temp1; //@line 786 "klatt.cpp"
      var $30=$temp1; //@line 786 "klatt.cpp"
      var $mul26=($29)*($30); //@line 786 "klatt.cpp"
      var $31=HEAPF32[((((__ZL10kt_globals+188+1088)&4294967295))>>2)]; //@line 786 "klatt.cpp"
      var $mul27=($31)*($mul26); //@line 786 "klatt.cpp"
      HEAPF32[((((__ZL10kt_globals+188+1088)&4294967295))>>2)]=$mul27; //@line 786 "klatt.cpp"
      var $32=HEAP32[((((__ZL10kt_globals+40)&4294967295))>>2)]; //@line 794 "klatt.cpp"
      var $33=HEAP32[((((__ZL10kt_globals+44)&4294967295))>>2)]; //@line 794 "klatt.cpp"
      var $sub28=((($32)-($33))&4294967295); //@line 794 "klatt.cpp"
      $temp=$sub28; //@line 794 "klatt.cpp"
      var $34=$frame_addr; //@line 795 "klatt.cpp"
      var $Kskew=(($34+188)&4294967295); //@line 795 "klatt.cpp"
      var $35=HEAP32[(($Kskew)>>2)]; //@line 795 "klatt.cpp"
      var $36=$temp; //@line 795 "klatt.cpp"
      var $cmp29=((($35))|0) > ((($36))|0); //@line 795 "klatt.cpp"
      if ($cmp29) { __label__ = 10;; } else { __label__ = 11;; } //@line 795 "klatt.cpp"
      if (__label__ == 10) {
  
        var $37=$temp; //@line 798 "klatt.cpp"
        var $38=$frame_addr; //@line 798 "klatt.cpp"
        var $Kskew31=(($38+188)&4294967295); //@line 798 "klatt.cpp"
        HEAP32[(($Kskew31)>>2)]=$37; //@line 798 "klatt.cpp"
        ; //@line 799 "klatt.cpp"
      }
  
      var $39=HEAP32[((__ZZL21pitch_synch_par_resetP13klatt_frame_tE4skew)>>2)]; //@line 800 "klatt.cpp"
      var $cmp33=((($39))|0) >= 0; //@line 800 "klatt.cpp"
      var $40=$frame_addr; //@line 802 "klatt.cpp"
      var $Kskew35=(($40+188)&4294967295); //@line 802 "klatt.cpp"
      var $41=HEAP32[(($Kskew35)>>2)]; //@line 802 "klatt.cpp"
      if ($cmp33) { __label__ = 12;; } else { __label__ = 13;; } //@line 800 "klatt.cpp"
      if (__label__ == 12) {
  
        HEAP32[((__ZZL21pitch_synch_par_resetP13klatt_frame_tE4skew)>>2)]=$41; //@line 802 "klatt.cpp"
        ; //@line 803 "klatt.cpp"
      }
      else if (__label__ == 13) {
  
        var $sub37=(((-($41)))&4294967295); //@line 806 "klatt.cpp"
        HEAP32[((__ZZL21pitch_synch_par_resetP13klatt_frame_tE4skew)>>2)]=$sub37; //@line 806 "klatt.cpp"
        ;
      }
  
      var $42=HEAP32[((((__ZL10kt_globals+40)&4294967295))>>2)]; //@line 810 "klatt.cpp"
      var $43=HEAP32[((__ZZL21pitch_synch_par_resetP13klatt_frame_tE4skew)>>2)]; //@line 810 "klatt.cpp"
      var $add=((($43)+($42))&4294967295); //@line 810 "klatt.cpp"
      HEAP32[((((__ZL10kt_globals+40)&4294967295))>>2)]=$add; //@line 810 "klatt.cpp"
      var $44=HEAP32[((__ZZL21pitch_synch_par_resetP13klatt_frame_tE4skew)>>2)]; //@line 811 "klatt.cpp"
      var $sub39=(((-($44)))&4294967295); //@line 811 "klatt.cpp"
      HEAP32[((__ZZL21pitch_synch_par_resetP13klatt_frame_tE4skew)>>2)]=$sub39; //@line 811 "klatt.cpp"
      ; //@line 812 "klatt.cpp"
    }
    else if (__label__ == 15) {
  
      HEAP32[((((__ZL10kt_globals+40)&4294967295))>>2)]=4; //@line 815 "klatt.cpp"
      HEAPF32[((((__ZL10kt_globals+112)&4294967295))>>2)]=0; //@line 816 "klatt.cpp"
      var $45=HEAP32[((((__ZL10kt_globals+40)&4294967295))>>2)]; //@line 817 "klatt.cpp"
      HEAP32[((((__ZL10kt_globals+48)&4294967295))>>2)]=$45; //@line 817 "klatt.cpp"
      HEAPF32[((((__ZL10kt_globals+144)&4294967295))>>2)]=0; //@line 818 "klatt.cpp"
      HEAPF32[((((__ZL10kt_globals+56)&4294967295))>>2)]=0; //@line 819 "klatt.cpp"
      HEAPF32[((((__ZL10kt_globals+64)&4294967295))>>2)]=0; //@line 820 "klatt.cpp"
      ;
    }
  
    var $46=HEAP32[((((__ZL10kt_globals+40)&4294967295))>>2)]; //@line 825 "klatt.cpp"
    var $cmp42=((($46))|0)!=4; //@line 825 "klatt.cpp"
    var $47=HEAP32[((((__ZL10kt_globals+36)&4294967295))>>2)]; //@line 825 "klatt.cpp"
    var $cmp43=((($47))|0)==0; //@line 825 "klatt.cpp"
    var $or_cond1=($cmp42) | ($cmp43); //@line 825 "klatt.cpp"
    if ($or_cond1) { __label__ = 17;; } else { __label__ = 20;; } //@line 825 "klatt.cpp"
    if (__label__ == 17) {
  
      var $48=$frame_addr; //@line 829 "klatt.cpp"
      var $TLTdb=(($48+180)&4294967295); //@line 829 "klatt.cpp"
      var $49=HEAP32[(($TLTdb)>>2)]; //@line 829 "klatt.cpp"
      var $conv45=((($49))|0); //@line 829 "klatt.cpp"
      var $mul46=0.033*($conv45); //@line 829 "klatt.cpp"
      HEAPF32[((((__ZL10kt_globals+96)&4294967295))>>2)]=$mul46; //@line 829 "klatt.cpp"
      var $50=HEAPF32[((((__ZL10kt_globals+96)&4294967295))>>2)]; //@line 831 "klatt.cpp"
      var $cmp47=($50) > 0; //@line 831 "klatt.cpp"
      if ($cmp47) { __label__ = 18;; } else { __label__ = 19;; } //@line 831 "klatt.cpp"
      if (__label__ == 18) {
  
        var $51=HEAPF32[((((__ZL10kt_globals+96)&4294967295))>>2)]; //@line 833 "klatt.cpp"
        var $sub49=1-($51); //@line 833 "klatt.cpp"
        HEAPF32[((((__ZL10kt_globals+88)&4294967295))>>2)]=$sub49; //@line 833 "klatt.cpp"
        ; //@line 834 "klatt.cpp"
      }
      else if (__label__ == 19) {
  
        HEAPF32[((((__ZL10kt_globals+88)&4294967295))>>2)]=1; //@line 837 "klatt.cpp"
        ;
      }
    }
  
    ;
    return; //@line 840 "klatt.cpp"
    return;
  }
  

  function __ZL9resonatorP11resonator_td($r, $input) {
    ;
    var __label__;
  
    var $r_addr;
    var $input_addr;
    var $x;
    $r_addr=$r;
    $input_addr=$input;
    var $0=$r_addr; //@line 85 "klatt.cpp"
    var $a=(($0)&4294967295); //@line 85 "klatt.cpp"
    var $1=HEAPF32[(($a)>>2)]; //@line 85 "klatt.cpp"
    var $2=$input_addr; //@line 85 "klatt.cpp"
    var $mul=($1)*($2); //@line 85 "klatt.cpp"
    var $3=$r_addr; //@line 85 "klatt.cpp"
    var $b=(($3+8)&4294967295); //@line 85 "klatt.cpp"
    var $4=HEAPF32[(($b)>>2)]; //@line 85 "klatt.cpp"
    var $5=$r_addr; //@line 85 "klatt.cpp"
    var $p1=(($5+24)&4294967295); //@line 85 "klatt.cpp"
    var $6=HEAPF32[(($p1)>>2)]; //@line 85 "klatt.cpp"
    var $mul1=($4)*($6); //@line 85 "klatt.cpp"
    var $add=($mul)+($mul1); //@line 85 "klatt.cpp"
    var $7=$r_addr; //@line 85 "klatt.cpp"
    var $c=(($7+16)&4294967295); //@line 85 "klatt.cpp"
    var $8=HEAPF32[(($c)>>2)]; //@line 85 "klatt.cpp"
    var $9=$r_addr; //@line 85 "klatt.cpp"
    var $p2=(($9+32)&4294967295); //@line 85 "klatt.cpp"
    var $10=HEAPF32[(($p2)>>2)]; //@line 85 "klatt.cpp"
    var $mul2=($8)*($10); //@line 85 "klatt.cpp"
    var $add3=($add)+($mul2); //@line 85 "klatt.cpp"
    $x=$add3; //@line 85 "klatt.cpp"
    var $11=$r_addr; //@line 86 "klatt.cpp"
    var $p14=(($11+24)&4294967295); //@line 86 "klatt.cpp"
    var $12=HEAPF32[(($p14)>>2)]; //@line 86 "klatt.cpp"
    var $13=$r_addr; //@line 86 "klatt.cpp"
    var $p25=(($13+32)&4294967295); //@line 86 "klatt.cpp"
    HEAPF32[(($p25)>>2)]=$12; //@line 86 "klatt.cpp"
    var $14=$x; //@line 87 "klatt.cpp"
    var $15=$r_addr; //@line 87 "klatt.cpp"
    var $p16=(($15+24)&4294967295); //@line 87 "klatt.cpp"
    HEAPF32[(($p16)>>2)]=$14; //@line 87 "klatt.cpp"
    var $16=$x; //@line 89 "klatt.cpp"
    ;
    return $16; //@line 89 "klatt.cpp"
    return null;
  }
  

  function __ZL14antiresonator2P11resonator_td($input) {
    ;
    var __label__;
  
    var $r_addr;
    var $input_addr;
    var $x;
    $r_addr=((__ZL10kt_globals+188)&4294967295);
    $input_addr=$input;
    var $0=$r_addr; //@line 129 "klatt.cpp"
    var $a=(($0)&4294967295); //@line 129 "klatt.cpp"
    var $1=HEAPF32[(($a)>>2)]; //@line 129 "klatt.cpp"
    var $2=$input_addr; //@line 129 "klatt.cpp"
    var $mul=($1)*($2); //@line 129 "klatt.cpp"
    var $3=$r_addr; //@line 129 "klatt.cpp"
    var $b=(($3+8)&4294967295); //@line 129 "klatt.cpp"
    var $4=HEAPF32[(($b)>>2)]; //@line 129 "klatt.cpp"
    var $5=$r_addr; //@line 129 "klatt.cpp"
    var $p1=(($5+24)&4294967295); //@line 129 "klatt.cpp"
    var $6=HEAPF32[(($p1)>>2)]; //@line 129 "klatt.cpp"
    var $mul1=($4)*($6); //@line 129 "klatt.cpp"
    var $add=($mul)+($mul1); //@line 129 "klatt.cpp"
    var $7=$r_addr; //@line 129 "klatt.cpp"
    var $c=(($7+16)&4294967295); //@line 129 "klatt.cpp"
    var $8=HEAPF32[(($c)>>2)]; //@line 129 "klatt.cpp"
    var $9=$r_addr; //@line 129 "klatt.cpp"
    var $p2=(($9+32)&4294967295); //@line 129 "klatt.cpp"
    var $10=HEAPF32[(($p2)>>2)]; //@line 129 "klatt.cpp"
    var $mul2=($8)*($10); //@line 129 "klatt.cpp"
    var $add3=($add)+($mul2); //@line 129 "klatt.cpp"
    $x=$add3; //@line 129 "klatt.cpp"
    var $11=$r_addr; //@line 130 "klatt.cpp"
    var $p14=(($11+24)&4294967295); //@line 130 "klatt.cpp"
    var $12=HEAPF32[(($p14)>>2)]; //@line 130 "klatt.cpp"
    var $13=$r_addr; //@line 130 "klatt.cpp"
    var $p25=(($13+32)&4294967295); //@line 130 "klatt.cpp"
    HEAPF32[(($p25)>>2)]=$12; //@line 130 "klatt.cpp"
    var $14=$input_addr; //@line 131 "klatt.cpp"
    var $15=$r_addr; //@line 131 "klatt.cpp"
    var $p16=(($15+24)&4294967295); //@line 131 "klatt.cpp"
    HEAPF32[(($p16)>>2)]=$14; //@line 131 "klatt.cpp"
    var $16=$r_addr; //@line 133 "klatt.cpp"
    var $a_inc=(($16+40)&4294967295); //@line 133 "klatt.cpp"
    var $17=HEAPF32[(($a_inc)>>2)]; //@line 133 "klatt.cpp"
    var $18=$r_addr; //@line 133 "klatt.cpp"
    var $a7=(($18)&4294967295); //@line 133 "klatt.cpp"
    var $19=HEAPF32[(($a7)>>2)]; //@line 133 "klatt.cpp"
    var $add8=($19)+($17); //@line 133 "klatt.cpp"
    HEAPF32[(($a7)>>2)]=$add8; //@line 133 "klatt.cpp"
    var $20=$r_addr; //@line 134 "klatt.cpp"
    var $b_inc=(($20+48)&4294967295); //@line 134 "klatt.cpp"
    var $21=HEAPF32[(($b_inc)>>2)]; //@line 134 "klatt.cpp"
    var $22=$r_addr; //@line 134 "klatt.cpp"
    var $b9=(($22+8)&4294967295); //@line 134 "klatt.cpp"
    var $23=HEAPF32[(($b9)>>2)]; //@line 134 "klatt.cpp"
    var $add10=($23)+($21); //@line 134 "klatt.cpp"
    HEAPF32[(($b9)>>2)]=$add10; //@line 134 "klatt.cpp"
    var $24=$r_addr; //@line 135 "klatt.cpp"
    var $c_inc=(($24+56)&4294967295); //@line 135 "klatt.cpp"
    var $25=HEAPF32[(($c_inc)>>2)]; //@line 135 "klatt.cpp"
    var $26=$r_addr; //@line 135 "klatt.cpp"
    var $c11=(($26+16)&4294967295); //@line 135 "klatt.cpp"
    var $27=HEAPF32[(($c11)>>2)]; //@line 135 "klatt.cpp"
    var $add12=($27)+($25); //@line 135 "klatt.cpp"
    HEAPF32[(($c11)>>2)]=$add12; //@line 135 "klatt.cpp"
    var $28=$x; //@line 136 "klatt.cpp"
    ;
    return $28; //@line 136 "klatt.cpp"
    return null;
  }
  

  function __ZL10resonator2P11resonator_td($r, $input) {
    ;
    var __label__;
  
    var $r_addr;
    var $input_addr;
    var $x;
    $r_addr=$r;
    $input_addr=$input;
    var $0=$r_addr; //@line 96 "klatt.cpp"
    var $a=(($0)&4294967295); //@line 96 "klatt.cpp"
    var $1=HEAPF32[(($a)>>2)]; //@line 96 "klatt.cpp"
    var $2=$input_addr; //@line 96 "klatt.cpp"
    var $mul=($1)*($2); //@line 96 "klatt.cpp"
    var $3=$r_addr; //@line 96 "klatt.cpp"
    var $b=(($3+8)&4294967295); //@line 96 "klatt.cpp"
    var $4=HEAPF32[(($b)>>2)]; //@line 96 "klatt.cpp"
    var $5=$r_addr; //@line 96 "klatt.cpp"
    var $p1=(($5+24)&4294967295); //@line 96 "klatt.cpp"
    var $6=HEAPF32[(($p1)>>2)]; //@line 96 "klatt.cpp"
    var $mul1=($4)*($6); //@line 96 "klatt.cpp"
    var $add=($mul)+($mul1); //@line 96 "klatt.cpp"
    var $7=$r_addr; //@line 96 "klatt.cpp"
    var $c=(($7+16)&4294967295); //@line 96 "klatt.cpp"
    var $8=HEAPF32[(($c)>>2)]; //@line 96 "klatt.cpp"
    var $9=$r_addr; //@line 96 "klatt.cpp"
    var $p2=(($9+32)&4294967295); //@line 96 "klatt.cpp"
    var $10=HEAPF32[(($p2)>>2)]; //@line 96 "klatt.cpp"
    var $mul2=($8)*($10); //@line 96 "klatt.cpp"
    var $add3=($add)+($mul2); //@line 96 "klatt.cpp"
    $x=$add3; //@line 96 "klatt.cpp"
    var $11=$r_addr; //@line 97 "klatt.cpp"
    var $p14=(($11+24)&4294967295); //@line 97 "klatt.cpp"
    var $12=HEAPF32[(($p14)>>2)]; //@line 97 "klatt.cpp"
    var $13=$r_addr; //@line 97 "klatt.cpp"
    var $p25=(($13+32)&4294967295); //@line 97 "klatt.cpp"
    HEAPF32[(($p25)>>2)]=$12; //@line 97 "klatt.cpp"
    var $14=$x; //@line 98 "klatt.cpp"
    var $15=$r_addr; //@line 98 "klatt.cpp"
    var $p16=(($15+24)&4294967295); //@line 98 "klatt.cpp"
    HEAPF32[(($p16)>>2)]=$14; //@line 98 "klatt.cpp"
    var $16=$r_addr; //@line 100 "klatt.cpp"
    var $a_inc=(($16+40)&4294967295); //@line 100 "klatt.cpp"
    var $17=HEAPF32[(($a_inc)>>2)]; //@line 100 "klatt.cpp"
    var $18=$r_addr; //@line 100 "klatt.cpp"
    var $a7=(($18)&4294967295); //@line 100 "klatt.cpp"
    var $19=HEAPF32[(($a7)>>2)]; //@line 100 "klatt.cpp"
    var $add8=($19)+($17); //@line 100 "klatt.cpp"
    HEAPF32[(($a7)>>2)]=$add8; //@line 100 "klatt.cpp"
    var $20=$r_addr; //@line 101 "klatt.cpp"
    var $b_inc=(($20+48)&4294967295); //@line 101 "klatt.cpp"
    var $21=HEAPF32[(($b_inc)>>2)]; //@line 101 "klatt.cpp"
    var $22=$r_addr; //@line 101 "klatt.cpp"
    var $b9=(($22+8)&4294967295); //@line 101 "klatt.cpp"
    var $23=HEAPF32[(($b9)>>2)]; //@line 101 "klatt.cpp"
    var $add10=($23)+($21); //@line 101 "klatt.cpp"
    HEAPF32[(($b9)>>2)]=$add10; //@line 101 "klatt.cpp"
    var $24=$r_addr; //@line 102 "klatt.cpp"
    var $c_inc=(($24+56)&4294967295); //@line 102 "klatt.cpp"
    var $25=HEAPF32[(($c_inc)>>2)]; //@line 102 "klatt.cpp"
    var $26=$r_addr; //@line 102 "klatt.cpp"
    var $c11=(($26+16)&4294967295); //@line 102 "klatt.cpp"
    var $27=HEAPF32[(($c11)>>2)]; //@line 102 "klatt.cpp"
    var $add12=($27)+($25); //@line 102 "klatt.cpp"
    HEAPF32[(($c11)>>2)]=$add12; //@line 102 "klatt.cpp"
    var $28=$x; //@line 103 "klatt.cpp"
    ;
    return $28; //@line 103 "klatt.cpp"
    return null;
  }
  

  function __ZL7DBtoLINl($dB) {
    ;
    var __label__;
  
    var $retval;
    var $dB_addr;
    $dB_addr=$dB;
    var $0=$dB_addr; //@line 971 "klatt.cpp"
    var $cmp=((($0))|0) < 0; //@line 971 "klatt.cpp"
    var $1=$dB_addr; //@line 971 "klatt.cpp"
    var $cmp1=((($1))|0) > 87; //@line 971 "klatt.cpp"
    var $or_cond=($cmp) | ($cmp1); //@line 971 "klatt.cpp"
    if ($or_cond) { __label__ = 1;; } else { __label__ = 2;; } //@line 971 "klatt.cpp"
    if (__label__ == 1) {
  
      $retval=0; //@line 973 "klatt.cpp"
      ; //@line 973 "klatt.cpp"
    }
    else if (__label__ == 2) {
  
      var $2=$dB_addr; //@line 976 "klatt.cpp"
      var $arrayidx=((__ZZL7DBtoLINlE8amptable+$2*2)&4294967295); //@line 976 "klatt.cpp"
      var $3=HEAP16[(($arrayidx)>>1)]; //@line 976 "klatt.cpp"
      var $conv=(tempInt=(($3)),(tempInt>=32768?tempInt-65536:tempInt)); //@line 976 "klatt.cpp"
      var $mul=($conv)*0.001; //@line 976 "klatt.cpp"
      $retval=$mul; //@line 976 "klatt.cpp"
      ; //@line 976 "klatt.cpp"
    }
  
    var $4=$retval; //@line 977 "klatt.cpp"
    ;
    return $4; //@line 977 "klatt.cpp"
    return null;
  }
  

  function __ZL10setzeroabcllP11resonator_t($f, $bw, $rp) {
    ;
    var __label__;
  
    var $f_addr;
    var $bw_addr;
    var $rp_addr;
    var $r;
    var $arg;
    $f_addr=$f;
    $bw_addr=$bw;
    $rp_addr=$rp;
    var $0=$f_addr; //@line 885 "klatt.cpp"
    var $sub=(((-($0)))&4294967295); //@line 885 "klatt.cpp"
    $f_addr=$sub; //@line 885 "klatt.cpp"
    var $1=$f_addr; //@line 887 "klatt.cpp"
    var $cmp=((($1))|0) >= 0; //@line 887 "klatt.cpp"
    if ($cmp) { __label__ = 1;; } else { __label__ = 2;; } //@line 887 "klatt.cpp"
    if (__label__ == 1) {
  
      $f_addr=-1; //@line 889 "klatt.cpp"
      ; //@line 890 "klatt.cpp"
    }
  
    var $2=HEAPF32[((((__ZL10kt_globals+72)&4294967295))>>2)]; //@line 894 "klatt.cpp"
    var $3=$bw_addr; //@line 894 "klatt.cpp"
    var $conv=((($3))|0); //@line 894 "klatt.cpp"
    var $mul=($2)*($conv); //@line 894 "klatt.cpp"
    $arg=$mul; //@line 894 "klatt.cpp"
    var $4=$arg; //@line 895 "klatt.cpp"
    var $call=_exp($4); //@line 895 "klatt.cpp"
    $r=$call; //@line 895 "klatt.cpp"
    var $5=$r; //@line 898 "klatt.cpp"
    var $6=$r; //@line 898 "klatt.cpp"
    var $mul1=($5)*($6); //@line 898 "klatt.cpp"
    var $sub2=(-($mul1)); //@line 898 "klatt.cpp"
    var $7=$rp_addr; //@line 898 "klatt.cpp"
    var $c=(($7+16)&4294967295); //@line 898 "klatt.cpp"
    HEAPF32[(($c)>>2)]=$sub2; //@line 898 "klatt.cpp"
    var $8=HEAPF32[((((__ZL10kt_globals+80)&4294967295))>>2)]; //@line 901 "klatt.cpp"
    var $9=$f_addr; //@line 901 "klatt.cpp"
    var $conv3=((($9))|0); //@line 901 "klatt.cpp"
    var $mul4=($8)*($conv3); //@line 901 "klatt.cpp"
    $arg=$mul4; //@line 901 "klatt.cpp"
    var $10=$r; //@line 902 "klatt.cpp"
    var $11=$arg; //@line 902 "klatt.cpp"
    var $call5=_cos($11); //@line 902 "klatt.cpp"
    var $mul6=($10)*($call5); //@line 902 "klatt.cpp"
    var $mul7=($mul6)*2; //@line 902 "klatt.cpp"
    var $12=$rp_addr; //@line 902 "klatt.cpp"
    var $b=(($12+8)&4294967295); //@line 902 "klatt.cpp"
    HEAPF32[(($b)>>2)]=$mul7; //@line 902 "klatt.cpp"
    var $13=$rp_addr; //@line 905 "klatt.cpp"
    var $b8=(($13+8)&4294967295); //@line 905 "klatt.cpp"
    var $14=HEAPF32[(($b8)>>2)]; //@line 905 "klatt.cpp"
    var $sub9=1-($14); //@line 905 "klatt.cpp"
    var $15=$rp_addr; //@line 905 "klatt.cpp"
    var $c10=(($15+16)&4294967295); //@line 905 "klatt.cpp"
    var $16=HEAPF32[(($c10)>>2)]; //@line 905 "klatt.cpp"
    var $sub11=($sub9)-($16); //@line 905 "klatt.cpp"
    var $17=$rp_addr; //@line 905 "klatt.cpp"
    var $a=(($17)&4294967295); //@line 905 "klatt.cpp"
    HEAPF32[(($a)>>2)]=$sub11; //@line 905 "klatt.cpp"
    var $18=$rp_addr; //@line 908 "klatt.cpp"
    var $a12=(($18)&4294967295); //@line 908 "klatt.cpp"
    var $19=HEAPF32[(($a12)>>2)]; //@line 908 "klatt.cpp"
    var $div=1/($19); //@line 908 "klatt.cpp"
    var $20=$rp_addr; //@line 908 "klatt.cpp"
    var $a13=(($20)&4294967295); //@line 908 "klatt.cpp"
    HEAPF32[(($a13)>>2)]=$div; //@line 908 "klatt.cpp"
    var $21=$rp_addr; //@line 909 "klatt.cpp"
    var $a14=(($21)&4294967295); //@line 909 "klatt.cpp"
    var $22=HEAPF32[(($a14)>>2)]; //@line 909 "klatt.cpp"
    var $sub15=(-($22)); //@line 909 "klatt.cpp"
    var $23=$rp_addr; //@line 909 "klatt.cpp"
    var $c16=(($23+16)&4294967295); //@line 909 "klatt.cpp"
    var $24=HEAPF32[(($c16)>>2)]; //@line 909 "klatt.cpp"
    var $mul17=($24)*($sub15); //@line 909 "klatt.cpp"
    HEAPF32[(($c16)>>2)]=$mul17; //@line 909 "klatt.cpp"
    var $25=$rp_addr; //@line 910 "klatt.cpp"
    var $a18=(($25)&4294967295); //@line 910 "klatt.cpp"
    var $26=HEAPF32[(($a18)>>2)]; //@line 910 "klatt.cpp"
    var $sub19=(-($26)); //@line 910 "klatt.cpp"
    var $27=$rp_addr; //@line 910 "klatt.cpp"
    var $b20=(($27+8)&4294967295); //@line 910 "klatt.cpp"
    var $28=HEAPF32[(($b20)>>2)]; //@line 910 "klatt.cpp"
    var $mul21=($28)*($sub19); //@line 910 "klatt.cpp"
    HEAPF32[(($b20)>>2)]=$mul21; //@line 910 "klatt.cpp"
    ;
    return; //@line 911 "klatt.cpp"
    return;
  }
  

  function __Z8sonicMSGPcz($format) {
    var __stackBase__  = STACKTOP; STACKTOP += 4100; _memset(__stackBase__, 0, 4100);
    var __label__;
  
    var $format_addr;
    var $buffer=__stackBase__;
    var $ap=__stackBase__+4096;
    var $file;
    $format_addr=$format;
    var $ap1=$ap; //@line 61 "sonic.cpp"
    HEAP32[(($ap1)>>2)]=arguments[__Z8sonicMSGPcz.length];; //@line 61 "sonic.cpp"
    var $arraydecay=(($buffer)&4294967295); //@line 62 "sonic.cpp"
    var $0=$format_addr; //@line 62 "sonic.cpp"
    var $1=HEAP32[(($ap)>>2)]; //@line 62 "sonic.cpp"
    var $call=_vsprintf($arraydecay, $0, $1); //@line 62 "sonic.cpp"
    var $ap2=$ap; //@line 63 "sonic.cpp"
    ; //@line 63 "sonic.cpp"
    var $call3=_fopen(((__str810)&4294967295), ((__str1811)&4294967295)); //@line 64 "sonic.cpp"
    $file=$call3; //@line 64 "sonic.cpp"
    var $2=$file; //@line 65 "sonic.cpp"
    var $arraydecay4=(($buffer)&4294967295); //@line 65 "sonic.cpp"
    var $fputs=_fputs($arraydecay4, $2); //@line 65 "sonic.cpp"
    var $3=$file; //@line 66 "sonic.cpp"
    var $call6=_fclose($3); //@line 66 "sonic.cpp"
    STACKTOP = __stackBase__;
    return; //@line 67 "sonic.cpp"
    return;
  }
  

  function _sonicGetSpeed($stream) {
    ;
    var __label__;
  
    var $stream_addr;
    $stream_addr=$stream;
    var $0=$stream_addr; //@line 93 "sonic.cpp"
    var $speed=(($0+16)&4294967295); //@line 93 "sonic.cpp"
    var $1=HEAPF32[(($speed)>>2)]; //@line 93 "sonic.cpp"
    ;
    return $1; //@line 93 "sonic.cpp"
    return null;
  }
  

  function _sonicSetSpeed($stream, $speed) {
    ;
    var __label__;
  
    var $stream_addr;
    var $speed_addr;
    $stream_addr=$stream;
    $speed_addr=$speed;
    var $0=$speed_addr; //@line 101 "sonic.cpp"
    var $1=$stream_addr; //@line 101 "sonic.cpp"
    var $speed1=(($1+16)&4294967295); //@line 101 "sonic.cpp"
    HEAPF32[(($speed1)>>2)]=$0; //@line 101 "sonic.cpp"
    ;
    return; //@line 102 "sonic.cpp"
    return;
  }
  

  function _sonicGetPitch($stream) {
    ;
    var __label__;
  
    var $stream_addr;
    $stream_addr=$stream;
    var $0=$stream_addr; //@line 108 "sonic.cpp"
    var $pitch=(($0+24)&4294967295); //@line 108 "sonic.cpp"
    var $1=HEAPF32[(($pitch)>>2)]; //@line 108 "sonic.cpp"
    ;
    return $1; //@line 108 "sonic.cpp"
    return null;
  }
  

  function _sonicSetPitch($stream, $pitch) {
    ;
    var __label__;
  
    var $stream_addr;
    var $pitch_addr;
    $stream_addr=$stream;
    $pitch_addr=$pitch;
    var $0=$pitch_addr; //@line 116 "sonic.cpp"
    var $1=$stream_addr; //@line 116 "sonic.cpp"
    var $pitch1=(($1+24)&4294967295); //@line 116 "sonic.cpp"
    HEAPF32[(($pitch1)>>2)]=$0; //@line 116 "sonic.cpp"
    ;
    return; //@line 117 "sonic.cpp"
    return;
  }
  

  function _sonicGetVolume($stream) {
    ;
    var __label__;
  
    var $stream_addr;
    $stream_addr=$stream;
    var $0=$stream_addr; //@line 123 "sonic.cpp"
    var $volume=(($0+20)&4294967295); //@line 123 "sonic.cpp"
    var $1=HEAPF32[(($volume)>>2)]; //@line 123 "sonic.cpp"
    ;
    return $1; //@line 123 "sonic.cpp"
    return null;
  }
  

  function _sonicSetVolume($stream, $volume) {
    ;
    var __label__;
  
    var $stream_addr;
    var $volume_addr;
    $stream_addr=$stream;
    $volume_addr=$volume;
    var $0=$volume_addr; //@line 131 "sonic.cpp"
    var $1=$stream_addr; //@line 131 "sonic.cpp"
    var $volume1=(($1+20)&4294967295); //@line 131 "sonic.cpp"
    HEAPF32[(($volume1)>>2)]=$0; //@line 131 "sonic.cpp"
    ;
    return; //@line 132 "sonic.cpp"
    return;
  }
  

  function _sonicGetSampleRate($stream) {
    ;
    var __label__;
  
    var $stream_addr;
    $stream_addr=$stream;
    var $0=$stream_addr; //@line 138 "sonic.cpp"
    var $sampleRate=(($0+72)&4294967295); //@line 138 "sonic.cpp"
    var $1=HEAP32[(($sampleRate)>>2)]; //@line 138 "sonic.cpp"
    ;
    return $1; //@line 138 "sonic.cpp"
    return null;
  }
  

  function _sonicGetNumChannels($stream) {
    ;
    var __label__;
  
    var $stream_addr;
    $stream_addr=$stream;
    var $0=$stream_addr; //@line 145 "sonic.cpp"
    var $numChannels=(($0+28)&4294967295); //@line 145 "sonic.cpp"
    var $1=HEAP32[(($numChannels)>>2)]; //@line 145 "sonic.cpp"
    ;
    return $1; //@line 145 "sonic.cpp"
    return null;
  }
  

  function _sonicDestroyStream($stream) {
    ;
    var __label__;
  
    var $stream_addr;
    $stream_addr=$stream;
    var $0=$stream_addr; //@line 152 "sonic.cpp"
    var $inputBuffer=(($0)&4294967295); //@line 152 "sonic.cpp"
    var $1=HEAP32[(($inputBuffer)>>2)]; //@line 152 "sonic.cpp"
    var $cmp=((($1))|0)!=0; //@line 152 "sonic.cpp"
    if ($cmp) { __label__ = 1;; } else { __label__ = 2;; } //@line 152 "sonic.cpp"
    if (__label__ == 1) {
  
      var $2=$stream_addr; //@line 153 "sonic.cpp"
      var $inputBuffer1=(($2)&4294967295); //@line 153 "sonic.cpp"
      var $3=HEAP32[(($inputBuffer1)>>2)]; //@line 153 "sonic.cpp"
      var $4=$3; //@line 153 "sonic.cpp"
      ; //@line 153 "sonic.cpp"
      ; //@line 154 "sonic.cpp"
    }
  
    var $5=$stream_addr; //@line 155 "sonic.cpp"
    var $outputBuffer=(($5+4)&4294967295); //@line 155 "sonic.cpp"
    var $6=HEAP32[(($outputBuffer)>>2)]; //@line 155 "sonic.cpp"
    var $cmp2=((($6))|0)!=0; //@line 155 "sonic.cpp"
    if ($cmp2) { __label__ = 3;; } else { __label__ = 4;; } //@line 155 "sonic.cpp"
    if (__label__ == 3) {
  
      var $7=$stream_addr; //@line 156 "sonic.cpp"
      var $outputBuffer4=(($7+4)&4294967295); //@line 156 "sonic.cpp"
      var $8=HEAP32[(($outputBuffer4)>>2)]; //@line 156 "sonic.cpp"
      var $9=$8; //@line 156 "sonic.cpp"
      ; //@line 156 "sonic.cpp"
      ; //@line 157 "sonic.cpp"
    }
  
    var $10=$stream_addr; //@line 158 "sonic.cpp"
    var $pitchBuffer=(($10+8)&4294967295); //@line 158 "sonic.cpp"
    var $11=HEAP32[(($pitchBuffer)>>2)]; //@line 158 "sonic.cpp"
    var $cmp6=((($11))|0)!=0; //@line 158 "sonic.cpp"
    if ($cmp6) { __label__ = 5;; } else { __label__ = 6;; } //@line 158 "sonic.cpp"
    if (__label__ == 5) {
  
      var $12=$stream_addr; //@line 159 "sonic.cpp"
      var $pitchBuffer8=(($12+8)&4294967295); //@line 159 "sonic.cpp"
      var $13=HEAP32[(($pitchBuffer8)>>2)]; //@line 159 "sonic.cpp"
      var $14=$13; //@line 159 "sonic.cpp"
      ; //@line 159 "sonic.cpp"
      ; //@line 160 "sonic.cpp"
    }
  
    var $15=$stream_addr; //@line 161 "sonic.cpp"
    var $downSampleBuffer=(($15+12)&4294967295); //@line 161 "sonic.cpp"
    var $16=HEAP32[(($downSampleBuffer)>>2)]; //@line 161 "sonic.cpp"
    var $cmp10=((($16))|0)!=0; //@line 161 "sonic.cpp"
    if ($cmp10) { __label__ = 7;; } else { __label__ = 8;; } //@line 161 "sonic.cpp"
    if (__label__ == 7) {
  
      var $17=$stream_addr; //@line 162 "sonic.cpp"
      var $downSampleBuffer12=(($17+12)&4294967295); //@line 162 "sonic.cpp"
      var $18=HEAP32[(($downSampleBuffer12)>>2)]; //@line 162 "sonic.cpp"
      var $19=$18; //@line 162 "sonic.cpp"
      ; //@line 162 "sonic.cpp"
      ; //@line 163 "sonic.cpp"
    }
  
    var $20=$stream_addr; //@line 164 "sonic.cpp"
    var $21=$20; //@line 164 "sonic.cpp"
    ; //@line 164 "sonic.cpp"
    ;
    return; //@line 165 "sonic.cpp"
    return;
  }
  

  function _sonicCreateStream($sampleRate, $numChannels) {
    ;
    var __label__;
  
    var $retval;
    var $sampleRate_addr;
    var $numChannels_addr;
    var $stream;
    var $minPeriod;
    var $maxPeriod;
    var $maxRequired;
    $sampleRate_addr=$sampleRate;
    $numChannels_addr=$numChannels;
    var $call=_calloc(1, 88); //@line 173 "sonic.cpp"
    var $0=$call; //@line 173 "sonic.cpp"
    $stream=$0; //@line 173 "sonic.cpp"
    var $1=$sampleRate_addr; //@line 174 "sonic.cpp"
    var $div=((((($1))|0)/400)|0); //@line 174 "sonic.cpp"
    $minPeriod=$div; //@line 174 "sonic.cpp"
    var $2=$sampleRate_addr; //@line 175 "sonic.cpp"
    var $div1=((((($2))|0)/65)|0); //@line 175 "sonic.cpp"
    $maxPeriod=$div1; //@line 175 "sonic.cpp"
    var $3=$maxPeriod; //@line 176 "sonic.cpp"
    var $mul=((($3)*2)&4294967295); //@line 176 "sonic.cpp"
    $maxRequired=$mul; //@line 176 "sonic.cpp"
    var $4=$stream; //@line 178 "sonic.cpp"
    var $cmp=((($4))|0)==0; //@line 178 "sonic.cpp"
    if ($cmp) { __label__ = 1;; } else { __label__ = 2;; } //@line 178 "sonic.cpp"
    if (__label__ == 1) {
  
      $retval=0; //@line 179 "sonic.cpp"
      ; //@line 179 "sonic.cpp"
    }
    else if (__label__ == 2) {
  
      var $5=$maxRequired; //@line 181 "sonic.cpp"
      var $6=$stream; //@line 181 "sonic.cpp"
      var $inputBufferSize=(($6+32)&4294967295); //@line 181 "sonic.cpp"
      HEAP32[(($inputBufferSize)>>2)]=$5; //@line 181 "sonic.cpp"
      var $7=$maxRequired; //@line 182 "sonic.cpp"
      var $8=$numChannels_addr; //@line 182 "sonic.cpp"
      var $mul2=((($8)*2)&4294967295); //@line 182 "sonic.cpp"
      var $call3=_calloc($7, $mul2); //@line 182 "sonic.cpp"
      var $9=$call3; //@line 182 "sonic.cpp"
      var $10=$stream; //@line 182 "sonic.cpp"
      var $inputBuffer=(($10)&4294967295); //@line 182 "sonic.cpp"
      HEAP32[(($inputBuffer)>>2)]=$9; //@line 182 "sonic.cpp"
      var $11=$stream; //@line 183 "sonic.cpp"
      var $inputBuffer4=(($11)&4294967295); //@line 183 "sonic.cpp"
      var $12=HEAP32[(($inputBuffer4)>>2)]; //@line 183 "sonic.cpp"
      var $cmp5=((($12))|0)==0; //@line 183 "sonic.cpp"
      if ($cmp5) { __label__ = 3;; } else { __label__ = 4;; } //@line 183 "sonic.cpp"
      if (__label__ == 3) {
  
        var $13=$stream; //@line 184 "sonic.cpp"
        _sonicDestroyStream($13); //@line 184 "sonic.cpp"
        $retval=0; //@line 185 "sonic.cpp"
        ; //@line 185 "sonic.cpp"
      }
      else if (__label__ == 4) {
  
        var $14=$maxRequired; //@line 187 "sonic.cpp"
        var $15=$stream; //@line 187 "sonic.cpp"
        var $outputBufferSize=(($15+40)&4294967295); //@line 187 "sonic.cpp"
        HEAP32[(($outputBufferSize)>>2)]=$14; //@line 187 "sonic.cpp"
        var $16=$maxRequired; //@line 188 "sonic.cpp"
        var $17=$numChannels_addr; //@line 188 "sonic.cpp"
        var $mul8=((($17)*2)&4294967295); //@line 188 "sonic.cpp"
        var $call9=_calloc($16, $mul8); //@line 188 "sonic.cpp"
        var $18=$call9; //@line 188 "sonic.cpp"
        var $19=$stream; //@line 188 "sonic.cpp"
        var $outputBuffer=(($19+4)&4294967295); //@line 188 "sonic.cpp"
        HEAP32[(($outputBuffer)>>2)]=$18; //@line 188 "sonic.cpp"
        var $20=$stream; //@line 189 "sonic.cpp"
        var $outputBuffer10=(($20+4)&4294967295); //@line 189 "sonic.cpp"
        var $21=HEAP32[(($outputBuffer10)>>2)]; //@line 189 "sonic.cpp"
        var $cmp11=((($21))|0)==0; //@line 189 "sonic.cpp"
        if ($cmp11) { __label__ = 5;; } else { __label__ = 6;; } //@line 189 "sonic.cpp"
        if (__label__ == 5) {
  
          var $22=$stream; //@line 190 "sonic.cpp"
          _sonicDestroyStream($22); //@line 190 "sonic.cpp"
          $retval=0; //@line 191 "sonic.cpp"
          ; //@line 191 "sonic.cpp"
        }
        else if (__label__ == 6) {
  
          var $23=$maxRequired; //@line 193 "sonic.cpp"
          var $24=$stream; //@line 193 "sonic.cpp"
          var $pitchBufferSize=(($24+36)&4294967295); //@line 193 "sonic.cpp"
          HEAP32[(($pitchBufferSize)>>2)]=$23; //@line 193 "sonic.cpp"
          var $25=$maxRequired; //@line 194 "sonic.cpp"
          var $26=$numChannels_addr; //@line 194 "sonic.cpp"
          var $mul14=((($26)*2)&4294967295); //@line 194 "sonic.cpp"
          var $call15=_calloc($25, $mul14); //@line 194 "sonic.cpp"
          var $27=$call15; //@line 194 "sonic.cpp"
          var $28=$stream; //@line 194 "sonic.cpp"
          var $pitchBuffer=(($28+8)&4294967295); //@line 194 "sonic.cpp"
          HEAP32[(($pitchBuffer)>>2)]=$27; //@line 194 "sonic.cpp"
          var $29=$stream; //@line 195 "sonic.cpp"
          var $pitchBuffer16=(($29+8)&4294967295); //@line 195 "sonic.cpp"
          var $30=HEAP32[(($pitchBuffer16)>>2)]; //@line 195 "sonic.cpp"
          var $cmp17=((($30))|0)==0; //@line 195 "sonic.cpp"
          if ($cmp17) { __label__ = 7;; } else { __label__ = 8;; } //@line 195 "sonic.cpp"
          if (__label__ == 7) {
  
            var $31=$stream; //@line 196 "sonic.cpp"
            _sonicDestroyStream($31); //@line 196 "sonic.cpp"
            $retval=0; //@line 197 "sonic.cpp"
            ; //@line 197 "sonic.cpp"
          }
          else if (__label__ == 8) {
  
            var $32=$maxRequired; //@line 199 "sonic.cpp"
            var $call20=_calloc($32, 2); //@line 199 "sonic.cpp"
            var $33=$call20; //@line 199 "sonic.cpp"
            var $34=$stream; //@line 199 "sonic.cpp"
            var $downSampleBuffer=(($34+12)&4294967295); //@line 199 "sonic.cpp"
            HEAP32[(($downSampleBuffer)>>2)]=$33; //@line 199 "sonic.cpp"
            var $35=$stream; //@line 200 "sonic.cpp"
            var $speed=(($35+16)&4294967295); //@line 200 "sonic.cpp"
            HEAPF32[(($speed)>>2)]=1; //@line 200 "sonic.cpp"
            var $36=$stream; //@line 201 "sonic.cpp"
            var $pitch=(($36+24)&4294967295); //@line 201 "sonic.cpp"
            HEAPF32[(($pitch)>>2)]=1; //@line 201 "sonic.cpp"
            var $37=$stream; //@line 202 "sonic.cpp"
            var $volume=(($37+20)&4294967295); //@line 202 "sonic.cpp"
            HEAPF32[(($volume)>>2)]=1; //@line 202 "sonic.cpp"
            var $38=$sampleRate_addr; //@line 203 "sonic.cpp"
            var $39=$stream; //@line 203 "sonic.cpp"
            var $sampleRate21=(($39+72)&4294967295); //@line 203 "sonic.cpp"
            HEAP32[(($sampleRate21)>>2)]=$38; //@line 203 "sonic.cpp"
            var $40=$numChannels_addr; //@line 204 "sonic.cpp"
            var $41=$stream; //@line 204 "sonic.cpp"
            var $numChannels22=(($41+28)&4294967295); //@line 204 "sonic.cpp"
            HEAP32[(($numChannels22)>>2)]=$40; //@line 204 "sonic.cpp"
            var $42=$minPeriod; //@line 205 "sonic.cpp"
            var $43=$stream; //@line 205 "sonic.cpp"
            var $minPeriod23=(($43+56)&4294967295); //@line 205 "sonic.cpp"
            HEAP32[(($minPeriod23)>>2)]=$42; //@line 205 "sonic.cpp"
            var $44=$maxPeriod; //@line 206 "sonic.cpp"
            var $45=$stream; //@line 206 "sonic.cpp"
            var $maxPeriod24=(($45+60)&4294967295); //@line 206 "sonic.cpp"
            HEAP32[(($maxPeriod24)>>2)]=$44; //@line 206 "sonic.cpp"
            var $46=$maxRequired; //@line 207 "sonic.cpp"
            var $47=$stream; //@line 207 "sonic.cpp"
            var $maxRequired25=(($47+64)&4294967295); //@line 207 "sonic.cpp"
            HEAP32[(($maxRequired25)>>2)]=$46; //@line 207 "sonic.cpp"
            var $48=$stream; //@line 208 "sonic.cpp"
            $retval=$48; //@line 208 "sonic.cpp"
            ; //@line 208 "sonic.cpp"
          }
        }
      }
    }
  
    var $49=$retval; //@line 209 "sonic.cpp"
    ;
    return $49; //@line 209 "sonic.cpp"
    return null;
  }
  

  function _sonicReadFloatFromStream($stream, $samples, $maxSamples) {
    ;
    var __label__;
  
    var $retval;
    var $stream_addr;
    var $samples_addr;
    var $maxSamples_addr;
    var $numSamples;
    var $remainingSamples;
    var $buffer;
    var $count;
    $stream_addr=$stream;
    $samples_addr=$samples;
    $maxSamples_addr=$maxSamples;
    var $0=$stream_addr; //@line 362 "sonic.cpp"
    var $numOutputSamples=(($0+48)&4294967295); //@line 362 "sonic.cpp"
    var $1=HEAP32[(($numOutputSamples)>>2)]; //@line 362 "sonic.cpp"
    $numSamples=$1; //@line 362 "sonic.cpp"
    $remainingSamples=0; //@line 363 "sonic.cpp"
    var $2=$numSamples; //@line 367 "sonic.cpp"
    var $cmp=((($2))|0)==0; //@line 367 "sonic.cpp"
    if ($cmp) { __label__ = 1;; } else { __label__ = 2;; } //@line 367 "sonic.cpp"
    if (__label__ == 1) {
  
      $retval=0; //@line 368 "sonic.cpp"
      ; //@line 368 "sonic.cpp"
    }
    else if (__label__ == 2) {
  
      var $3=$numSamples; //@line 370 "sonic.cpp"
      var $4=$maxSamples_addr; //@line 370 "sonic.cpp"
      var $cmp1=((($3))|0) > ((($4))|0); //@line 370 "sonic.cpp"
      if ($cmp1) { __label__ = 3;; } else { __label__ = 4;; } //@line 370 "sonic.cpp"
      if (__label__ == 3) {
  
        var $5=$numSamples; //@line 371 "sonic.cpp"
        var $6=$maxSamples_addr; //@line 371 "sonic.cpp"
        var $sub=((($5)-($6))&4294967295); //@line 371 "sonic.cpp"
        $remainingSamples=$sub; //@line 371 "sonic.cpp"
        var $7=$maxSamples_addr; //@line 372 "sonic.cpp"
        $numSamples=$7; //@line 372 "sonic.cpp"
        ; //@line 373 "sonic.cpp"
      }
  
      var $8=$stream_addr; //@line 374 "sonic.cpp"
      var $outputBuffer=(($8+4)&4294967295); //@line 374 "sonic.cpp"
      var $9=HEAP32[(($outputBuffer)>>2)]; //@line 374 "sonic.cpp"
      $buffer=$9; //@line 374 "sonic.cpp"
      var $10=$numSamples; //@line 375 "sonic.cpp"
      var $11=$stream_addr; //@line 375 "sonic.cpp"
      var $numChannels=(($11+28)&4294967295); //@line 375 "sonic.cpp"
      var $12=HEAP32[(($numChannels)>>2)]; //@line 375 "sonic.cpp"
      var $mul=((($12)*($10))&4294967295); //@line 375 "sonic.cpp"
      var $dec1=((($mul)-1)&4294967295); //@line 376 "sonic.cpp"
      $count=$dec1; //@line 376 "sonic.cpp"
      var $tobool2=((($mul))|0)!=0; //@line 376 "sonic.cpp"
      if ($tobool2) { __label__ = 5;; } else { __label__ = 6;; } //@line 376 "sonic.cpp"
      $while_body$$while_end$8: do { 
        if (__label__ == 5) {
          while(1) { 
  
            var $13=$buffer; //@line 377 "sonic.cpp"
            var $incdec_ptr=(($13+2)&4294967295); //@line 377 "sonic.cpp"
            $buffer=$incdec_ptr; //@line 377 "sonic.cpp"
            var $14=HEAP16[(($13)>>1)]; //@line 377 "sonic.cpp"
            var $conv=(tempInt=(($14)),(tempInt>=32768?tempInt-65536:tempInt)); //@line 377 "sonic.cpp"
            var $conv4=((($conv))|0); //@line 377 "sonic.cpp"
            var $div=($conv4)/32767; //@line 377 "sonic.cpp"
            var $15=$samples_addr; //@line 377 "sonic.cpp"
            var $incdec_ptr5=(($15+4)&4294967295); //@line 377 "sonic.cpp"
            $samples_addr=$incdec_ptr5; //@line 377 "sonic.cpp"
            HEAPF32[(($15)>>2)]=$div; //@line 377 "sonic.cpp"
            var $_pr=$count; //@line 376 "sonic.cpp"
            var $dec=((($_pr)-1)&4294967295); //@line 376 "sonic.cpp"
            $count=$dec; //@line 376 "sonic.cpp"
            var $tobool=((($_pr))|0)!=0; //@line 376 "sonic.cpp"
            if ($tobool) { __label__ = 5;continue ; } else { __label__ = 6;break $while_body$$while_end$8; } //@line 376 "sonic.cpp"
          }
        }
      } while(0);
  
      var $16=$remainingSamples; //@line 379 "sonic.cpp"
      var $cmp6=((($16))|0) > 0; //@line 379 "sonic.cpp"
      if ($cmp6) { __label__ = 7;; } else { __label__ = 8;; } //@line 379 "sonic.cpp"
      if (__label__ == 7) {
  
        var $17=$stream_addr; //@line 380 "sonic.cpp"
        var $outputBuffer8=(($17+4)&4294967295); //@line 380 "sonic.cpp"
        var $18=HEAP32[(($outputBuffer8)>>2)]; //@line 380 "sonic.cpp"
        var $19=$18; //@line 380 "sonic.cpp"
        var $20=$stream_addr; //@line 380 "sonic.cpp"
        var $outputBuffer9=(($20+4)&4294967295); //@line 380 "sonic.cpp"
        var $21=HEAP32[(($outputBuffer9)>>2)]; //@line 380 "sonic.cpp"
        var $22=$numSamples; //@line 380 "sonic.cpp"
        var $23=$stream_addr; //@line 380 "sonic.cpp"
        var $numChannels10=(($23+28)&4294967295); //@line 380 "sonic.cpp"
        var $24=HEAP32[(($numChannels10)>>2)]; //@line 380 "sonic.cpp"
        var $mul11=((($24)*($22))&4294967295); //@line 380 "sonic.cpp"
        var $add_ptr=(($21+2*$mul11)&4294967295); //@line 380 "sonic.cpp"
        var $25=$add_ptr; //@line 380 "sonic.cpp"
        var $26=$remainingSamples; //@line 380 "sonic.cpp"
        var $27=$stream_addr; //@line 380 "sonic.cpp"
        var $numChannels13=(($27+28)&4294967295); //@line 380 "sonic.cpp"
        var $28=HEAP32[(($numChannels13)>>2)]; //@line 380 "sonic.cpp"
        var $mul12=((($26)*2)&4294967295); //@line 380 "sonic.cpp"
        var $mul14=((($mul12)*($28))&4294967295); //@line 380 "sonic.cpp"
        _llvm_memmove_p0i8_p0i8_i32($19, $25, $mul14, 1, 0); //@line 380 "sonic.cpp"
        ; //@line 382 "sonic.cpp"
      }
  
      var $29=$remainingSamples; //@line 383 "sonic.cpp"
      var $30=$stream_addr; //@line 383 "sonic.cpp"
      var $numOutputSamples16=(($30+48)&4294967295); //@line 383 "sonic.cpp"
      HEAP32[(($numOutputSamples16)>>2)]=$29; //@line 383 "sonic.cpp"
      var $31=$numSamples; //@line 384 "sonic.cpp"
      $retval=$31; //@line 384 "sonic.cpp"
      ; //@line 384 "sonic.cpp"
    }
  
    var $32=$retval; //@line 385 "sonic.cpp"
    ;
    return $32; //@line 385 "sonic.cpp"
    return null;
  }
  

  function _sonicReadShortFromStream($stream, $samples, $maxSamples) {
    ;
    var __label__;
  
    var $retval;
    var $stream_addr;
    var $samples_addr;
    var $maxSamples_addr;
    var $numSamples;
    var $remainingSamples;
    $stream_addr=$stream;
    $samples_addr=$samples;
    $maxSamples_addr=$maxSamples;
    var $0=$stream_addr; //@line 394 "sonic.cpp"
    var $numOutputSamples=(($0+48)&4294967295); //@line 394 "sonic.cpp"
    var $1=HEAP32[(($numOutputSamples)>>2)]; //@line 394 "sonic.cpp"
    $numSamples=$1; //@line 394 "sonic.cpp"
    $remainingSamples=0; //@line 395 "sonic.cpp"
    var $2=$numSamples; //@line 397 "sonic.cpp"
    var $cmp=((($2))|0)==0; //@line 397 "sonic.cpp"
    if ($cmp) { __label__ = 1;; } else { __label__ = 2;; } //@line 397 "sonic.cpp"
    if (__label__ == 1) {
  
      $retval=0; //@line 398 "sonic.cpp"
      ; //@line 398 "sonic.cpp"
    }
    else if (__label__ == 2) {
  
      var $3=$numSamples; //@line 400 "sonic.cpp"
      var $4=$maxSamples_addr; //@line 400 "sonic.cpp"
      var $cmp1=((($3))|0) > ((($4))|0); //@line 400 "sonic.cpp"
      if ($cmp1) { __label__ = 3;; } else { __label__ = 4;; } //@line 400 "sonic.cpp"
      if (__label__ == 3) {
  
        var $5=$numSamples; //@line 401 "sonic.cpp"
        var $6=$maxSamples_addr; //@line 401 "sonic.cpp"
        var $sub=((($5)-($6))&4294967295); //@line 401 "sonic.cpp"
        $remainingSamples=$sub; //@line 401 "sonic.cpp"
        var $7=$maxSamples_addr; //@line 402 "sonic.cpp"
        $numSamples=$7; //@line 402 "sonic.cpp"
        ; //@line 403 "sonic.cpp"
      }
  
      var $8=$samples_addr; //@line 404 "sonic.cpp"
      var $9=$8; //@line 404 "sonic.cpp"
      var $10=$stream_addr; //@line 404 "sonic.cpp"
      var $outputBuffer=(($10+4)&4294967295); //@line 404 "sonic.cpp"
      var $11=HEAP32[(($outputBuffer)>>2)]; //@line 404 "sonic.cpp"
      var $12=$11; //@line 404 "sonic.cpp"
      var $13=$numSamples; //@line 404 "sonic.cpp"
      var $14=$stream_addr; //@line 404 "sonic.cpp"
      var $numChannels=(($14+28)&4294967295); //@line 404 "sonic.cpp"
      var $15=HEAP32[(($numChannels)>>2)]; //@line 404 "sonic.cpp"
      var $mul=((($13)*2)&4294967295); //@line 404 "sonic.cpp"
      var $mul4=((($mul)*($15))&4294967295); //@line 404 "sonic.cpp"
      var $src$, $dest$, $stop$, $stop4$, $fast$;
      $src$ = $12;
      $dest$ = $9;
      $stop$ = $src$ + $mul4;
      if (($dest$%4) == ($src$%4) && $mul4 > 8) {
        while ($src$%4 !== 0 && $src$ < $stop$) {
          ; HEAP8[$dest$++] = HEAP8[$src$++];
        }
        $src$ >>= 2;
        $dest$ >>= 2;
        $stop4$ = $stop$ >> 2;
        while ($src$ < $stop4$) {
      ;;;    HEAP32[$dest$++] = HEAP32[$src$++];
        }
        $src$ <<= 2;
        $dest$ <<= 2;
      }while ($src$ < $stop$) {
        ; HEAP8[$dest$++] = HEAP8[$src$++];
      }; //@line 404 "sonic.cpp"
      var $16=$remainingSamples; //@line 405 "sonic.cpp"
      var $cmp5=((($16))|0) > 0; //@line 405 "sonic.cpp"
      if ($cmp5) { __label__ = 5;; } else { __label__ = 6;; } //@line 405 "sonic.cpp"
      if (__label__ == 5) {
  
        var $17=$stream_addr; //@line 406 "sonic.cpp"
        var $outputBuffer7=(($17+4)&4294967295); //@line 406 "sonic.cpp"
        var $18=HEAP32[(($outputBuffer7)>>2)]; //@line 406 "sonic.cpp"
        var $19=$18; //@line 406 "sonic.cpp"
        var $20=$stream_addr; //@line 406 "sonic.cpp"
        var $outputBuffer8=(($20+4)&4294967295); //@line 406 "sonic.cpp"
        var $21=HEAP32[(($outputBuffer8)>>2)]; //@line 406 "sonic.cpp"
        var $22=$numSamples; //@line 406 "sonic.cpp"
        var $23=$stream_addr; //@line 406 "sonic.cpp"
        var $numChannels9=(($23+28)&4294967295); //@line 406 "sonic.cpp"
        var $24=HEAP32[(($numChannels9)>>2)]; //@line 406 "sonic.cpp"
        var $mul10=((($24)*($22))&4294967295); //@line 406 "sonic.cpp"
        var $add_ptr=(($21+2*$mul10)&4294967295); //@line 406 "sonic.cpp"
        var $25=$add_ptr; //@line 406 "sonic.cpp"
        var $26=$remainingSamples; //@line 406 "sonic.cpp"
        var $27=$stream_addr; //@line 406 "sonic.cpp"
        var $numChannels12=(($27+28)&4294967295); //@line 406 "sonic.cpp"
        var $28=HEAP32[(($numChannels12)>>2)]; //@line 406 "sonic.cpp"
        var $mul11=((($26)*2)&4294967295); //@line 406 "sonic.cpp"
        var $mul13=((($mul11)*($28))&4294967295); //@line 406 "sonic.cpp"
        _llvm_memmove_p0i8_p0i8_i32($19, $25, $mul13, 1, 0); //@line 406 "sonic.cpp"
        ; //@line 408 "sonic.cpp"
      }
  
      var $29=$remainingSamples; //@line 409 "sonic.cpp"
      var $30=$stream_addr; //@line 409 "sonic.cpp"
      var $numOutputSamples15=(($30+48)&4294967295); //@line 409 "sonic.cpp"
      HEAP32[(($numOutputSamples15)>>2)]=$29; //@line 409 "sonic.cpp"
      var $31=$numSamples; //@line 410 "sonic.cpp"
      $retval=$31; //@line 410 "sonic.cpp"
      ; //@line 410 "sonic.cpp"
    }
  
    var $32=$retval; //@line 411 "sonic.cpp"
    ;
    return $32; //@line 411 "sonic.cpp"
    return null;
  }
  

  function _sonicReadUnsignedCharFromStream($stream, $samples, $maxSamples) {
    ;
    var __label__;
  
    var $retval;
    var $stream_addr;
    var $samples_addr;
    var $maxSamples_addr;
    var $numSamples;
    var $remainingSamples;
    var $buffer;
    var $count;
    $stream_addr=$stream;
    $samples_addr=$samples;
    $maxSamples_addr=$maxSamples;
    var $0=$stream_addr; //@line 420 "sonic.cpp"
    var $numOutputSamples=(($0+48)&4294967295); //@line 420 "sonic.cpp"
    var $1=HEAP32[(($numOutputSamples)>>2)]; //@line 420 "sonic.cpp"
    $numSamples=$1; //@line 420 "sonic.cpp"
    $remainingSamples=0; //@line 421 "sonic.cpp"
    var $2=$numSamples; //@line 425 "sonic.cpp"
    var $cmp=((($2))|0)==0; //@line 425 "sonic.cpp"
    if ($cmp) { __label__ = 1;; } else { __label__ = 2;; } //@line 425 "sonic.cpp"
    if (__label__ == 1) {
  
      $retval=0; //@line 426 "sonic.cpp"
      ; //@line 426 "sonic.cpp"
    }
    else if (__label__ == 2) {
  
      var $3=$numSamples; //@line 428 "sonic.cpp"
      var $4=$maxSamples_addr; //@line 428 "sonic.cpp"
      var $cmp1=((($3))|0) > ((($4))|0); //@line 428 "sonic.cpp"
      if ($cmp1) { __label__ = 3;; } else { __label__ = 4;; } //@line 428 "sonic.cpp"
      if (__label__ == 3) {
  
        var $5=$numSamples; //@line 429 "sonic.cpp"
        var $6=$maxSamples_addr; //@line 429 "sonic.cpp"
        var $sub=((($5)-($6))&4294967295); //@line 429 "sonic.cpp"
        $remainingSamples=$sub; //@line 429 "sonic.cpp"
        var $7=$maxSamples_addr; //@line 430 "sonic.cpp"
        $numSamples=$7; //@line 430 "sonic.cpp"
        ; //@line 431 "sonic.cpp"
      }
  
      var $8=$stream_addr; //@line 432 "sonic.cpp"
      var $outputBuffer=(($8+4)&4294967295); //@line 432 "sonic.cpp"
      var $9=HEAP32[(($outputBuffer)>>2)]; //@line 432 "sonic.cpp"
      $buffer=$9; //@line 432 "sonic.cpp"
      var $10=$numSamples; //@line 433 "sonic.cpp"
      var $11=$stream_addr; //@line 433 "sonic.cpp"
      var $numChannels=(($11+28)&4294967295); //@line 433 "sonic.cpp"
      var $12=HEAP32[(($numChannels)>>2)]; //@line 433 "sonic.cpp"
      var $mul=((($12)*($10))&4294967295); //@line 433 "sonic.cpp"
      var $dec1=((($mul)-1)&4294967295); //@line 434 "sonic.cpp"
      $count=$dec1; //@line 434 "sonic.cpp"
      var $tobool2=((($mul))|0)!=0; //@line 434 "sonic.cpp"
      if ($tobool2) { __label__ = 5;; } else { __label__ = 6;; } //@line 434 "sonic.cpp"
      $while_body$$while_end$8: do { 
        if (__label__ == 5) {
          while(1) { 
  
            var $13=$buffer; //@line 435 "sonic.cpp"
            var $incdec_ptr=(($13+2)&4294967295); //@line 435 "sonic.cpp"
            $buffer=$incdec_ptr; //@line 435 "sonic.cpp"
            var $14=HEAP16[(($13)>>1)]; //@line 435 "sonic.cpp"
            var $conv=(tempInt=(($14)),(tempInt>=32768?tempInt-65536:tempInt)); //@line 435 "sonic.cpp"
            var $shr=($conv) >> 8; //@line 435 "sonic.cpp"
            var $conv4=((($shr)) & 255); //@line 435 "sonic.cpp"
            var $conv5=(tempInt=(($conv4)),(tempInt>=128?tempInt-256:tempInt)); //@line 435 "sonic.cpp"
            var $add=((($conv5)+128)&4294967295); //@line 435 "sonic.cpp"
            var $conv6=((($add)) & 255); //@line 435 "sonic.cpp"
            var $15=$samples_addr; //@line 435 "sonic.cpp"
            var $incdec_ptr7=(($15+1)&4294967295); //@line 435 "sonic.cpp"
            $samples_addr=$incdec_ptr7; //@line 435 "sonic.cpp"
            HEAP8[($15)]=$conv6; //@line 435 "sonic.cpp"
            var $_pr=$count; //@line 434 "sonic.cpp"
            var $dec=((($_pr)-1)&4294967295); //@line 434 "sonic.cpp"
            $count=$dec; //@line 434 "sonic.cpp"
            var $tobool=((($_pr))|0)!=0; //@line 434 "sonic.cpp"
            if ($tobool) { __label__ = 5;continue ; } else { __label__ = 6;break $while_body$$while_end$8; } //@line 434 "sonic.cpp"
          }
        }
      } while(0);
  
      var $16=$remainingSamples; //@line 437 "sonic.cpp"
      var $cmp8=((($16))|0) > 0; //@line 437 "sonic.cpp"
      if ($cmp8) { __label__ = 7;; } else { __label__ = 8;; } //@line 437 "sonic.cpp"
      if (__label__ == 7) {
  
        var $17=$stream_addr; //@line 438 "sonic.cpp"
        var $outputBuffer10=(($17+4)&4294967295); //@line 438 "sonic.cpp"
        var $18=HEAP32[(($outputBuffer10)>>2)]; //@line 438 "sonic.cpp"
        var $19=$18; //@line 438 "sonic.cpp"
        var $20=$stream_addr; //@line 438 "sonic.cpp"
        var $outputBuffer11=(($20+4)&4294967295); //@line 438 "sonic.cpp"
        var $21=HEAP32[(($outputBuffer11)>>2)]; //@line 438 "sonic.cpp"
        var $22=$numSamples; //@line 438 "sonic.cpp"
        var $23=$stream_addr; //@line 438 "sonic.cpp"
        var $numChannels12=(($23+28)&4294967295); //@line 438 "sonic.cpp"
        var $24=HEAP32[(($numChannels12)>>2)]; //@line 438 "sonic.cpp"
        var $mul13=((($24)*($22))&4294967295); //@line 438 "sonic.cpp"
        var $add_ptr=(($21+2*$mul13)&4294967295); //@line 438 "sonic.cpp"
        var $25=$add_ptr; //@line 438 "sonic.cpp"
        var $26=$remainingSamples; //@line 438 "sonic.cpp"
        var $27=$stream_addr; //@line 438 "sonic.cpp"
        var $numChannels15=(($27+28)&4294967295); //@line 438 "sonic.cpp"
        var $28=HEAP32[(($numChannels15)>>2)]; //@line 438 "sonic.cpp"
        var $mul14=((($26)*2)&4294967295); //@line 438 "sonic.cpp"
        var $mul16=((($mul14)*($28))&4294967295); //@line 438 "sonic.cpp"
        _llvm_memmove_p0i8_p0i8_i32($19, $25, $mul16, 1, 0); //@line 438 "sonic.cpp"
        ; //@line 440 "sonic.cpp"
      }
  
      var $29=$remainingSamples; //@line 441 "sonic.cpp"
      var $30=$stream_addr; //@line 441 "sonic.cpp"
      var $numOutputSamples18=(($30+48)&4294967295); //@line 441 "sonic.cpp"
      HEAP32[(($numOutputSamples18)>>2)]=$29; //@line 441 "sonic.cpp"
      var $31=$numSamples; //@line 442 "sonic.cpp"
      $retval=$31; //@line 442 "sonic.cpp"
      ; //@line 442 "sonic.cpp"
    }
  
    var $32=$retval; //@line 443 "sonic.cpp"
    ;
    return $32; //@line 443 "sonic.cpp"
    return null;
  }
  

  function _sonicFlushStream($stream) {
    ;
    var __label__;
  
    var $retval;
    var $stream_addr;
    var $maxRequired;
    var $numSamples;
    var $remainingSpace;
    var $numOutputSamples;
    var $expectedSamples;
    $stream_addr=$stream;
    var $0=$stream_addr; //@line 451 "sonic.cpp"
    var $maxRequired1=(($0+64)&4294967295); //@line 451 "sonic.cpp"
    var $1=HEAP32[(($maxRequired1)>>2)]; //@line 451 "sonic.cpp"
    $maxRequired=$1; //@line 451 "sonic.cpp"
    var $2=$stream_addr; //@line 452 "sonic.cpp"
    var $numInputSamples=(($2+44)&4294967295); //@line 452 "sonic.cpp"
    var $3=HEAP32[(($numInputSamples)>>2)]; //@line 452 "sonic.cpp"
    $numSamples=$3; //@line 452 "sonic.cpp"
    var $4=$numSamples; //@line 455 "sonic.cpp"
    var $cmp=((($4))|0)==0; //@line 455 "sonic.cpp"
    if ($cmp) { __label__ = 1;; } else { __label__ = 2;; } //@line 455 "sonic.cpp"
    $if_then$$if_end$2: do { 
      if (__label__ == 1) {
  
        $retval=1; //@line 456 "sonic.cpp"
        ; //@line 456 "sonic.cpp"
      }
      else if (__label__ == 2) {
  
        var $5=$numSamples; //@line 458 "sonic.cpp"
        var $6=$maxRequired; //@line 458 "sonic.cpp"
        var $cmp2=((($5))|0) >= ((($6))|0); //@line 458 "sonic.cpp"
        if ($cmp2) { __label__ = 3;; } else { __label__ = 5;; } //@line 458 "sonic.cpp"
        $land_lhs_true$$if_end4$5: do { 
          if (__label__ == 3) {
  
            var $7=$stream_addr; //@line 458 "sonic.cpp"
            var $call=_sonicWriteShortToStream($7, 0, 0); //@line 458 "sonic.cpp"
            var $tobool=((($call))|0)!=0; //@line 458 "sonic.cpp"
            if ($tobool) { __label__ = 5;break $land_lhs_true$$if_end4$5; } //@line 458 "sonic.cpp"
  
            $retval=0; //@line 459 "sonic.cpp"
            __label__ = 12;break $if_then$$if_end$2; //@line 459 "sonic.cpp"
          }
        } while(0);
  
        var $8=$stream_addr; //@line 461 "sonic.cpp"
        var $numInputSamples5=(($8+44)&4294967295); //@line 461 "sonic.cpp"
        var $9=HEAP32[(($numInputSamples5)>>2)]; //@line 461 "sonic.cpp"
        $numSamples=$9; //@line 461 "sonic.cpp"
        var $cmp6=((($9))|0)==0; //@line 462 "sonic.cpp"
        if ($cmp6) { __label__ = 6;; } else { __label__ = 7;; } //@line 462 "sonic.cpp"
        if (__label__ == 6) {
  
          $retval=1; //@line 463 "sonic.cpp"
          ; //@line 463 "sonic.cpp"
        }
        else if (__label__ == 7) {
  
          var $10=$maxRequired; //@line 465 "sonic.cpp"
          var $11=$numSamples; //@line 465 "sonic.cpp"
          var $sub=((($10)-($11))&4294967295); //@line 465 "sonic.cpp"
          $remainingSpace=$sub; //@line 465 "sonic.cpp"
          var $12=$stream_addr; //@line 466 "sonic.cpp"
          var $inputBuffer=(($12)&4294967295); //@line 466 "sonic.cpp"
          var $13=HEAP32[(($inputBuffer)>>2)]; //@line 466 "sonic.cpp"
          var $14=$numSamples; //@line 466 "sonic.cpp"
          var $15=$stream_addr; //@line 466 "sonic.cpp"
          var $numChannels=(($15+28)&4294967295); //@line 466 "sonic.cpp"
          var $16=HEAP32[(($numChannels)>>2)]; //@line 466 "sonic.cpp"
          var $mul=((($16)*($14))&4294967295); //@line 466 "sonic.cpp"
          var $add_ptr=(($13+2*$mul)&4294967295); //@line 466 "sonic.cpp"
          var $17=$add_ptr; //@line 466 "sonic.cpp"
          var $18=$remainingSpace; //@line 466 "sonic.cpp"
          var $19=$stream_addr; //@line 466 "sonic.cpp"
          var $numChannels10=(($19+28)&4294967295); //@line 466 "sonic.cpp"
          var $20=HEAP32[(($numChannels10)>>2)]; //@line 466 "sonic.cpp"
          var $mul9=((($18)*2)&4294967295); //@line 466 "sonic.cpp"
          var $mul11=((($mul9)*($20))&4294967295); //@line 466 "sonic.cpp"
          var $dest$, $stop$, $stop4$, $fast$, $value4$;
          $dest$ = $17;
          $stop$ = $dest$ + $mul11;
          $value4$ = 0;
          if ($value4$ < 0) $value4$ += 256;
          $value4$ = $value4$ + ($value4$<<8) + ($value4$<<16) + ($value4$*16777216);
          while ($dest$%4 !== 0 && $dest$ < $stop$) {
            ;; HEAP8[$dest$++] = 0;
          }
          $dest$ >>= 2;
          $stop4$ = $stop$ >> 2;
          while ($dest$ < $stop4$) {
          ;;;;;;;  HEAP32[$dest$++] = $value4$;
          }
          $dest$ <<= 2;
          while ($dest$ < $stop$) {
            ;; HEAP8[$dest$++] = 0;
          }; //@line 466 "sonic.cpp"
          var $21=$maxRequired; //@line 468 "sonic.cpp"
          var $22=$stream_addr; //@line 468 "sonic.cpp"
          var $numInputSamples12=(($22+44)&4294967295); //@line 468 "sonic.cpp"
          HEAP32[(($numInputSamples12)>>2)]=$21; //@line 468 "sonic.cpp"
          var $23=$stream_addr; //@line 469 "sonic.cpp"
          var $numOutputSamples13=(($23+48)&4294967295); //@line 469 "sonic.cpp"
          var $24=HEAP32[(($numOutputSamples13)>>2)]; //@line 469 "sonic.cpp"
          $numOutputSamples=$24; //@line 469 "sonic.cpp"
          var $25=$stream_addr; //@line 470 "sonic.cpp"
          var $call14=_sonicWriteShortToStream($25, 0, 0); //@line 470 "sonic.cpp"
          var $tobool15=((($call14))|0)!=0; //@line 470 "sonic.cpp"
          if ($tobool15) { __label__ = 9;; } else { __label__ = 8;; } //@line 470 "sonic.cpp"
          if (__label__ == 9) {
  
            var $26=$numSamples; //@line 474 "sonic.cpp"
            var $conv=((($26))|0); //@line 474 "sonic.cpp"
            var $27=$stream_addr; //@line 474 "sonic.cpp"
            var $speed=(($27+16)&4294967295); //@line 474 "sonic.cpp"
            var $28=HEAPF32[(($speed)>>2)]; //@line 474 "sonic.cpp"
            var $mul18=($conv)*($28); //@line 474 "sonic.cpp"
            var $conv19=($mul18); //@line 474 "sonic.cpp"
            var $add=($conv19)+0.5; //@line 474 "sonic.cpp"
            var $conv20=((($add))|0); //@line 474 "sonic.cpp"
            $expectedSamples=$conv20; //@line 474 "sonic.cpp"
            var $29=$stream_addr; //@line 475 "sonic.cpp"
            var $numOutputSamples21=(($29+48)&4294967295); //@line 475 "sonic.cpp"
            var $30=HEAP32[(($numOutputSamples21)>>2)]; //@line 475 "sonic.cpp"
            var $31=$numOutputSamples; //@line 475 "sonic.cpp"
            var $32=$expectedSamples; //@line 475 "sonic.cpp"
            var $add22=((($32)+($31))&4294967295); //@line 475 "sonic.cpp"
            var $cmp23=((($30))|0) > ((($add22))|0); //@line 475 "sonic.cpp"
            if ($cmp23) { __label__ = 10;; } else { __label__ = 11;; } //@line 475 "sonic.cpp"
            if (__label__ == 10) {
  
              var $33=$numOutputSamples; //@line 476 "sonic.cpp"
              var $34=$expectedSamples; //@line 476 "sonic.cpp"
              var $add25=((($34)+($33))&4294967295); //@line 476 "sonic.cpp"
              var $35=$stream_addr; //@line 476 "sonic.cpp"
              var $numOutputSamples26=(($35+48)&4294967295); //@line 476 "sonic.cpp"
              HEAP32[(($numOutputSamples26)>>2)]=$add25; //@line 476 "sonic.cpp"
              ; //@line 477 "sonic.cpp"
            }
  
            $retval=1; //@line 478 "sonic.cpp"
            ; //@line 478 "sonic.cpp"
          }
          else if (__label__ == 8) {
  
            $retval=0; //@line 471 "sonic.cpp"
            ; //@line 471 "sonic.cpp"
          }
        }
      }
    } while(0);
  
    var $36=$retval; //@line 479 "sonic.cpp"
    ;
    return $36; //@line 479 "sonic.cpp"
    return null;
  }
  

  function _sonicWriteShortToStream($stream, $samples, $numSamples) {
    ;
    var __label__;
  
    var $retval;
    var $stream_addr;
    var $samples_addr;
    var $numSamples_addr;
    $stream_addr=$stream;
    $samples_addr=$samples;
    $numSamples_addr=$numSamples;
    var $0=$stream_addr; //@line 904 "sonic.cpp"
    var $1=$samples_addr; //@line 904 "sonic.cpp"
    var $2=$numSamples_addr; //@line 904 "sonic.cpp"
    var $call=__ZL28addShortSamplesToInputBufferP17sonicStreamStructPsi($0, $1, $2); //@line 904 "sonic.cpp"
    var $tobool=((($call))|0)!=0; //@line 904 "sonic.cpp"
    if ($tobool) { __label__ = 2;; } else { __label__ = 1;; } //@line 904 "sonic.cpp"
    if (__label__ == 2) {
  
      var $3=$stream_addr; //@line 907 "sonic.cpp"
      var $call1=__ZL18processStreamInputP17sonicStreamStruct($3); //@line 907 "sonic.cpp"
      $retval=$call1; //@line 907 "sonic.cpp"
      ; //@line 907 "sonic.cpp"
    }
    else if (__label__ == 1) {
  
      $retval=0; //@line 905 "sonic.cpp"
      ; //@line 905 "sonic.cpp"
    }
  
    var $4=$retval; //@line 908 "sonic.cpp"
    ;
    return $4; //@line 908 "sonic.cpp"
    return null;
  }
  

  function _sonicSamplesAvailable($stream) {
    ;
    var __label__;
  
    var $stream_addr;
    $stream_addr=$stream;
    var $0=$stream_addr; //@line 485 "sonic.cpp"
    var $numOutputSamples=(($0+48)&4294967295); //@line 485 "sonic.cpp"
    var $1=HEAP32[(($numOutputSamples)>>2)]; //@line 485 "sonic.cpp"
    ;
    return $1; //@line 485 "sonic.cpp"
    return null;
  }
  

  function _sonicWriteFloatToStream($stream, $samples, $numSamples) {
    ;
    var __label__;
  
    var $retval;
    var $stream_addr;
    var $samples_addr;
    var $numSamples_addr;
    $stream_addr=$stream;
    $samples_addr=$samples;
    $numSamples_addr=$numSamples;
    var $0=$stream_addr; //@line 891 "sonic.cpp"
    var $1=$samples_addr; //@line 891 "sonic.cpp"
    var $2=$numSamples_addr; //@line 891 "sonic.cpp"
    var $call=__ZL28addFloatSamplesToInputBufferP17sonicStreamStructPfi($0, $1, $2); //@line 891 "sonic.cpp"
    var $tobool=((($call))|0)!=0; //@line 891 "sonic.cpp"
    if ($tobool) { __label__ = 2;; } else { __label__ = 1;; } //@line 891 "sonic.cpp"
    if (__label__ == 2) {
  
      var $3=$stream_addr; //@line 894 "sonic.cpp"
      var $call1=__ZL18processStreamInputP17sonicStreamStruct($3); //@line 894 "sonic.cpp"
      $retval=$call1; //@line 894 "sonic.cpp"
      ; //@line 894 "sonic.cpp"
    }
    else if (__label__ == 1) {
  
      $retval=0; //@line 892 "sonic.cpp"
      ; //@line 892 "sonic.cpp"
    }
  
    var $4=$retval; //@line 895 "sonic.cpp"
    ;
    return $4; //@line 895 "sonic.cpp"
    return null;
  }
  

  function __ZL28addFloatSamplesToInputBufferP17sonicStreamStructPfi($stream, $samples, $numSamples) {
    ;
    var __label__;
  
    var $retval;
    var $stream_addr;
    var $samples_addr;
    var $numSamples_addr;
    var $buffer;
    var $count;
    $stream_addr=$stream;
    $samples_addr=$samples;
    $numSamples_addr=$numSamples;
    var $0=$numSamples_addr; //@line 250 "sonic.cpp"
    var $1=$stream_addr; //@line 250 "sonic.cpp"
    var $numChannels=(($1+28)&4294967295); //@line 250 "sonic.cpp"
    var $2=HEAP32[(($numChannels)>>2)]; //@line 250 "sonic.cpp"
    var $mul=((($2)*($0))&4294967295); //@line 250 "sonic.cpp"
    $count=$mul; //@line 250 "sonic.cpp"
    var $3=$numSamples_addr; //@line 252 "sonic.cpp"
    var $cmp=((($3))|0)==0; //@line 252 "sonic.cpp"
    if ($cmp) { __label__ = 1;; } else { __label__ = 2;; } //@line 252 "sonic.cpp"
    if (__label__ == 1) {
  
      $retval=1; //@line 253 "sonic.cpp"
      ; //@line 253 "sonic.cpp"
    }
    else if (__label__ == 2) {
  
      var $4=$stream_addr; //@line 255 "sonic.cpp"
      var $5=$numSamples_addr; //@line 255 "sonic.cpp"
      var $call=__ZL26enlargeInputBufferIfNeededP17sonicStreamStructi($4, $5); //@line 255 "sonic.cpp"
      var $tobool=((($call))|0)!=0; //@line 255 "sonic.cpp"
      if ($tobool) { __label__ = 4;; } else { __label__ = 3;; } //@line 255 "sonic.cpp"
      if (__label__ == 4) {
  
        var $6=$stream_addr; //@line 258 "sonic.cpp"
        var $inputBuffer=(($6)&4294967295); //@line 258 "sonic.cpp"
        var $7=HEAP32[(($inputBuffer)>>2)]; //@line 258 "sonic.cpp"
        var $8=$stream_addr; //@line 258 "sonic.cpp"
        var $numInputSamples=(($8+44)&4294967295); //@line 258 "sonic.cpp"
        var $9=HEAP32[(($numInputSamples)>>2)]; //@line 258 "sonic.cpp"
        var $10=$stream_addr; //@line 258 "sonic.cpp"
        var $numChannels3=(($10+28)&4294967295); //@line 258 "sonic.cpp"
        var $11=HEAP32[(($numChannels3)>>2)]; //@line 258 "sonic.cpp"
        var $mul4=((($11)*($9))&4294967295); //@line 258 "sonic.cpp"
        var $add_ptr=(($7+2*$mul4)&4294967295); //@line 258 "sonic.cpp"
        $buffer=$add_ptr; //@line 258 "sonic.cpp"
        var $12=$count; //@line 259 "sonic.cpp"
        var $dec1=((($12)-1)&4294967295); //@line 259 "sonic.cpp"
        $count=$dec1; //@line 259 "sonic.cpp"
        var $tobool52=((($12))|0)!=0; //@line 259 "sonic.cpp"
        if ($tobool52) { __label__ = 5;; } else { __label__ = 6;; } //@line 259 "sonic.cpp"
        $while_body$$while_end$7: do { 
          if (__label__ == 5) {
            while(1) { 
  
              var $13=$samples_addr; //@line 260 "sonic.cpp"
              var $incdec_ptr=(($13+4)&4294967295); //@line 260 "sonic.cpp"
              $samples_addr=$incdec_ptr; //@line 260 "sonic.cpp"
              var $14=HEAPF32[(($13)>>2)]; //@line 260 "sonic.cpp"
              var $mul6=($14)*32767; //@line 260 "sonic.cpp"
              var $conv=((($mul6))|0); //@line 260 "sonic.cpp"
              var $15=$buffer; //@line 260 "sonic.cpp"
              var $incdec_ptr7=(($15+2)&4294967295); //@line 260 "sonic.cpp"
              $buffer=$incdec_ptr7; //@line 260 "sonic.cpp"
              HEAP16[(($15)>>1)]=$conv; //@line 260 "sonic.cpp"
              var $16=$count; //@line 259 "sonic.cpp"
              var $dec=((($16)-1)&4294967295); //@line 259 "sonic.cpp"
              $count=$dec; //@line 259 "sonic.cpp"
              var $tobool5=((($16))|0)!=0; //@line 259 "sonic.cpp"
              if ($tobool5) { __label__ = 5;continue ; } else { __label__ = 6;break $while_body$$while_end$7; } //@line 259 "sonic.cpp"
            }
          }
        } while(0);
  
        var $17=$numSamples_addr; //@line 262 "sonic.cpp"
        var $18=$stream_addr; //@line 262 "sonic.cpp"
        var $numInputSamples8=(($18+44)&4294967295); //@line 262 "sonic.cpp"
        var $19=HEAP32[(($numInputSamples8)>>2)]; //@line 262 "sonic.cpp"
        var $add=((($19)+($17))&4294967295); //@line 262 "sonic.cpp"
        HEAP32[(($numInputSamples8)>>2)]=$add; //@line 262 "sonic.cpp"
        $retval=1; //@line 263 "sonic.cpp"
        ; //@line 263 "sonic.cpp"
      }
      else if (__label__ == 3) {
  
        $retval=0; //@line 256 "sonic.cpp"
        ; //@line 256 "sonic.cpp"
      }
    }
  
    var $20=$retval; //@line 264 "sonic.cpp"
    ;
    return $20; //@line 264 "sonic.cpp"
    return null;
  }
  

  function __ZL18processStreamInputP17sonicStreamStruct($stream) {
    ;
    var __label__;
  
    var $retval;
    var $stream_addr;
    var $originalNumOutputSamples;
    var $speed;
    $stream_addr=$stream;
    var $0=$stream_addr; //@line 860 "sonic.cpp"
    var $numOutputSamples=(($0+48)&4294967295); //@line 860 "sonic.cpp"
    var $1=HEAP32[(($numOutputSamples)>>2)]; //@line 860 "sonic.cpp"
    $originalNumOutputSamples=$1; //@line 860 "sonic.cpp"
    var $2=$stream_addr; //@line 861 "sonic.cpp"
    var $speed1=(($2+16)&4294967295); //@line 861 "sonic.cpp"
    var $3=HEAPF32[(($speed1)>>2)]; //@line 861 "sonic.cpp"
    var $4=$stream_addr; //@line 861 "sonic.cpp"
    var $pitch=(($4+24)&4294967295); //@line 861 "sonic.cpp"
    var $5=HEAPF32[(($pitch)>>2)]; //@line 861 "sonic.cpp"
    var $div=($3)/($5); //@line 861 "sonic.cpp"
    $speed=$div; //@line 861 "sonic.cpp"
    var $6=$speed; //@line 863 "sonic.cpp"
    var $conv=($6); //@line 863 "sonic.cpp"
    var $cmp=($conv) > 1.00001; //@line 863 "sonic.cpp"
    if ($cmp) { __label__ = 2;; } else { __label__ = 1;; } //@line 863 "sonic.cpp"
    $if_then$$lor_lhs_false$2: do { 
      if (__label__ == 1) {
  
        var $7=$speed; //@line 863 "sonic.cpp"
        var $conv2=($7); //@line 863 "sonic.cpp"
        var $cmp3=($conv2) < 0.99999; //@line 863 "sonic.cpp"
        if ($cmp3) { __label__ = 2;break $if_then$$lor_lhs_false$2; } //@line 863 "sonic.cpp"
  
        var $10=$stream_addr; //@line 866 "sonic.cpp"
        var $11=$stream_addr; //@line 866 "sonic.cpp"
        var $inputBuffer=(($11)&4294967295); //@line 866 "sonic.cpp"
        var $12=HEAP32[(($inputBuffer)>>2)]; //@line 866 "sonic.cpp"
        var $13=$stream_addr; //@line 866 "sonic.cpp"
        var $numInputSamples=(($13+44)&4294967295); //@line 866 "sonic.cpp"
        var $14=HEAP32[(($numInputSamples)>>2)]; //@line 866 "sonic.cpp"
        var $call4=__ZL12copyToOutputP17sonicStreamStructPsi($10, $12, $14); //@line 866 "sonic.cpp"
        var $tobool=((($call4))|0)!=0; //@line 866 "sonic.cpp"
        if ($tobool) { __label__ = 5;; } else { __label__ = 4;; } //@line 866 "sonic.cpp"
        if (__label__ == 5) {
  
          var $15=$stream_addr; //@line 869 "sonic.cpp"
          var $numInputSamples6=(($15+44)&4294967295); //@line 869 "sonic.cpp"
          HEAP32[(($numInputSamples6)>>2)]=0; //@line 869 "sonic.cpp"
          __label__ = 6;break $if_then$$lor_lhs_false$2;
        }
        else if (__label__ == 4) {
  
          $retval=0; //@line 867 "sonic.cpp"
          __label__ = 12;break $if_then$$lor_lhs_false$2; //@line 867 "sonic.cpp"
        }
      }
    } while(0);
    $if_then$$if_end7$$return$8: do { 
      if (__label__ == 2) {
  
        var $8=$stream_addr; //@line 864 "sonic.cpp"
        var $9=$speed; //@line 864 "sonic.cpp"
        __ZL11changeSpeedP17sonicStreamStructf($8, $9); //@line 864 "sonic.cpp"
        __label__ = 6;break $if_then$$if_end7$$return$8; //@line 865 "sonic.cpp"
      }
    } while(0);
    $if_end7$$return$10: do { 
      if (__label__ == 6) {
  
        var $16=$stream_addr; //@line 871 "sonic.cpp"
        var $pitch8=(($16+24)&4294967295); //@line 871 "sonic.cpp"
        var $17=HEAPF32[(($pitch8)>>2)]; //@line 871 "sonic.cpp"
        var $cmp9=($17) != 1; //@line 871 "sonic.cpp"
        if ($cmp9) { __label__ = 7;; } else { __label__ = 9;; } //@line 871 "sonic.cpp"
        $if_then10$$if_end15$12: do { 
          if (__label__ == 7) {
  
            var $18=$stream_addr; //@line 872 "sonic.cpp"
            var $19=$originalNumOutputSamples; //@line 872 "sonic.cpp"
            var $call11=__ZL11adjustPitchP17sonicStreamStructi($18, $19); //@line 872 "sonic.cpp"
            var $tobool12=((($call11))|0)!=0; //@line 872 "sonic.cpp"
            if ($tobool12) { __label__ = 9;break $if_then10$$if_end15$12; } //@line 872 "sonic.cpp"
  
            $retval=0; //@line 873 "sonic.cpp"
            __label__ = 12;break $if_end7$$return$10; //@line 873 "sonic.cpp"
          }
        } while(0);
  
        var $20=$stream_addr; //@line 876 "sonic.cpp"
        var $volume=(($20+20)&4294967295); //@line 876 "sonic.cpp"
        var $21=HEAPF32[(($volume)>>2)]; //@line 876 "sonic.cpp"
        var $cmp16=($21) != 1; //@line 876 "sonic.cpp"
        if ($cmp16) { __label__ = 10;; } else { __label__ = 11;; } //@line 876 "sonic.cpp"
        if (__label__ == 10) {
  
          var $22=$stream_addr; //@line 878 "sonic.cpp"
          var $outputBuffer=(($22+4)&4294967295); //@line 878 "sonic.cpp"
          var $23=HEAP32[(($outputBuffer)>>2)]; //@line 878 "sonic.cpp"
          var $24=$originalNumOutputSamples; //@line 878 "sonic.cpp"
          var $25=$stream_addr; //@line 878 "sonic.cpp"
          var $numChannels=(($25+28)&4294967295); //@line 878 "sonic.cpp"
          var $26=HEAP32[(($numChannels)>>2)]; //@line 878 "sonic.cpp"
          var $mul=((($26)*($24))&4294967295); //@line 878 "sonic.cpp"
          var $add_ptr=(($23+2*$mul)&4294967295); //@line 878 "sonic.cpp"
          var $27=$stream_addr; //@line 878 "sonic.cpp"
          var $numOutputSamples18=(($27+48)&4294967295); //@line 878 "sonic.cpp"
          var $28=HEAP32[(($numOutputSamples18)>>2)]; //@line 878 "sonic.cpp"
          var $29=$originalNumOutputSamples; //@line 878 "sonic.cpp"
          var $sub=((($28)-($29))&4294967295); //@line 878 "sonic.cpp"
          var $30=$stream_addr; //@line 878 "sonic.cpp"
          var $numChannels19=(($30+28)&4294967295); //@line 878 "sonic.cpp"
          var $31=HEAP32[(($numChannels19)>>2)]; //@line 878 "sonic.cpp"
          var $mul20=((($31)*($sub))&4294967295); //@line 878 "sonic.cpp"
          var $32=$stream_addr; //@line 878 "sonic.cpp"
          var $volume21=(($32+20)&4294967295); //@line 878 "sonic.cpp"
          var $33=HEAPF32[(($volume21)>>2)]; //@line 878 "sonic.cpp"
          __ZL12scaleSamplesPsif($add_ptr, $mul20, $33); //@line 878 "sonic.cpp"
          ; //@line 881 "sonic.cpp"
        }
  
        $retval=1; //@line 882 "sonic.cpp"
        ; //@line 882 "sonic.cpp"
      }
    } while(0);
  
    var $34=$retval; //@line 883 "sonic.cpp"
    ;
    return $34; //@line 883 "sonic.cpp"
    return null;
  }
  

  function __ZL28addShortSamplesToInputBufferP17sonicStreamStructPsi($stream, $samples, $numSamples) {
    ;
    var __label__;
  
    var $retval;
    var $stream_addr;
    var $samples_addr;
    var $numSamples_addr;
    $stream_addr=$stream;
    $samples_addr=$samples;
    $numSamples_addr=$numSamples;
    var $0=$numSamples_addr; //@line 272 "sonic.cpp"
    var $cmp=((($0))|0)==0; //@line 272 "sonic.cpp"
    if ($cmp) { __label__ = 1;; } else { __label__ = 2;; } //@line 272 "sonic.cpp"
    if (__label__ == 1) {
  
      $retval=1; //@line 273 "sonic.cpp"
      ; //@line 273 "sonic.cpp"
    }
    else if (__label__ == 2) {
  
      var $1=$stream_addr; //@line 275 "sonic.cpp"
      var $2=$numSamples_addr; //@line 275 "sonic.cpp"
      var $call=__ZL26enlargeInputBufferIfNeededP17sonicStreamStructi($1, $2); //@line 275 "sonic.cpp"
      var $tobool=((($call))|0)!=0; //@line 275 "sonic.cpp"
      if ($tobool) { __label__ = 4;; } else { __label__ = 3;; } //@line 275 "sonic.cpp"
      if (__label__ == 4) {
  
        var $3=$stream_addr; //@line 278 "sonic.cpp"
        var $inputBuffer=(($3)&4294967295); //@line 278 "sonic.cpp"
        var $4=HEAP32[(($inputBuffer)>>2)]; //@line 278 "sonic.cpp"
        var $5=$stream_addr; //@line 278 "sonic.cpp"
        var $numInputSamples=(($5+44)&4294967295); //@line 278 "sonic.cpp"
        var $6=HEAP32[(($numInputSamples)>>2)]; //@line 278 "sonic.cpp"
        var $7=$stream_addr; //@line 278 "sonic.cpp"
        var $numChannels=(($7+28)&4294967295); //@line 278 "sonic.cpp"
        var $8=HEAP32[(($numChannels)>>2)]; //@line 278 "sonic.cpp"
        var $mul=((($8)*($6))&4294967295); //@line 278 "sonic.cpp"
        var $add_ptr=(($4+2*$mul)&4294967295); //@line 278 "sonic.cpp"
        var $9=$add_ptr; //@line 278 "sonic.cpp"
        var $10=$samples_addr; //@line 278 "sonic.cpp"
        var $11=$10; //@line 278 "sonic.cpp"
        var $12=$numSamples_addr; //@line 278 "sonic.cpp"
        var $13=$stream_addr; //@line 278 "sonic.cpp"
        var $numChannels4=(($13+28)&4294967295); //@line 278 "sonic.cpp"
        var $14=HEAP32[(($numChannels4)>>2)]; //@line 278 "sonic.cpp"
        var $mul3=((($12)*2)&4294967295); //@line 278 "sonic.cpp"
        var $mul5=((($mul3)*($14))&4294967295); //@line 278 "sonic.cpp"
        var $src$, $dest$, $stop$, $stop4$, $fast$;
        $src$ = $11;
        $dest$ = $9;
        $stop$ = $src$ + $mul5;
        if (($dest$%4) == ($src$%4) && $mul5 > 8) {
          while ($src$%4 !== 0 && $src$ < $stop$) {
            ; HEAP8[$dest$++] = HEAP8[$src$++];
          }
          $src$ >>= 2;
          $dest$ >>= 2;
          $stop4$ = $stop$ >> 2;
          while ($src$ < $stop4$) {
        ;;;    HEAP32[$dest$++] = HEAP32[$src$++];
          }
          $src$ <<= 2;
          $dest$ <<= 2;
        }while ($src$ < $stop$) {
          ; HEAP8[$dest$++] = HEAP8[$src$++];
        }; //@line 278 "sonic.cpp"
        var $15=$numSamples_addr; //@line 280 "sonic.cpp"
        var $16=$stream_addr; //@line 280 "sonic.cpp"
        var $numInputSamples6=(($16+44)&4294967295); //@line 280 "sonic.cpp"
        var $17=HEAP32[(($numInputSamples6)>>2)]; //@line 280 "sonic.cpp"
        var $add=((($17)+($15))&4294967295); //@line 280 "sonic.cpp"
        HEAP32[(($numInputSamples6)>>2)]=$add; //@line 280 "sonic.cpp"
        $retval=1; //@line 281 "sonic.cpp"
        ; //@line 281 "sonic.cpp"
      }
      else if (__label__ == 3) {
  
        $retval=0; //@line 276 "sonic.cpp"
        ; //@line 276 "sonic.cpp"
      }
    }
  
    var $18=$retval; //@line 282 "sonic.cpp"
    ;
    return $18; //@line 282 "sonic.cpp"
    return null;
  }
  

  function _sonicWriteUnsignedCharToStream($stream, $samples, $numSamples) {
    ;
    var __label__;
  
    var $retval;
    var $stream_addr;
    var $samples_addr;
    var $numSamples_addr;
    $stream_addr=$stream;
    $samples_addr=$samples;
    $numSamples_addr=$numSamples;
    var $0=$stream_addr; //@line 917 "sonic.cpp"
    var $1=$samples_addr; //@line 917 "sonic.cpp"
    var $2=$numSamples_addr; //@line 917 "sonic.cpp"
    var $call=__ZL35addUnsignedCharSamplesToInputBufferP17sonicStreamStructPhi($0, $1, $2); //@line 917 "sonic.cpp"
    var $tobool=((($call))|0)!=0; //@line 917 "sonic.cpp"
    if ($tobool) { __label__ = 2;; } else { __label__ = 1;; } //@line 917 "sonic.cpp"
    if (__label__ == 2) {
  
      var $3=$stream_addr; //@line 920 "sonic.cpp"
      var $call1=__ZL18processStreamInputP17sonicStreamStruct($3); //@line 920 "sonic.cpp"
      $retval=$call1; //@line 920 "sonic.cpp"
      ; //@line 920 "sonic.cpp"
    }
    else if (__label__ == 1) {
  
      $retval=0; //@line 918 "sonic.cpp"
      ; //@line 918 "sonic.cpp"
    }
  
    var $4=$retval; //@line 921 "sonic.cpp"
    ;
    return $4; //@line 921 "sonic.cpp"
    return null;
  }
  

  function __ZL35addUnsignedCharSamplesToInputBufferP17sonicStreamStructPhi($stream, $samples, $numSamples) {
    ;
    var __label__;
  
    var $retval;
    var $stream_addr;
    var $samples_addr;
    var $numSamples_addr;
    var $buffer;
    var $count;
    $stream_addr=$stream;
    $samples_addr=$samples;
    $numSamples_addr=$numSamples;
    var $0=$numSamples_addr; //@line 291 "sonic.cpp"
    var $1=$stream_addr; //@line 291 "sonic.cpp"
    var $numChannels=(($1+28)&4294967295); //@line 291 "sonic.cpp"
    var $2=HEAP32[(($numChannels)>>2)]; //@line 291 "sonic.cpp"
    var $mul=((($2)*($0))&4294967295); //@line 291 "sonic.cpp"
    $count=$mul; //@line 291 "sonic.cpp"
    var $3=$numSamples_addr; //@line 293 "sonic.cpp"
    var $cmp=((($3))|0)==0; //@line 293 "sonic.cpp"
    if ($cmp) { __label__ = 1;; } else { __label__ = 2;; } //@line 293 "sonic.cpp"
    if (__label__ == 1) {
  
      $retval=1; //@line 294 "sonic.cpp"
      ; //@line 294 "sonic.cpp"
    }
    else if (__label__ == 2) {
  
      var $4=$stream_addr; //@line 296 "sonic.cpp"
      var $5=$numSamples_addr; //@line 296 "sonic.cpp"
      var $call=__ZL26enlargeInputBufferIfNeededP17sonicStreamStructi($4, $5); //@line 296 "sonic.cpp"
      var $tobool=((($call))|0)!=0; //@line 296 "sonic.cpp"
      if ($tobool) { __label__ = 4;; } else { __label__ = 3;; } //@line 296 "sonic.cpp"
      if (__label__ == 4) {
  
        var $6=$stream_addr; //@line 299 "sonic.cpp"
        var $inputBuffer=(($6)&4294967295); //@line 299 "sonic.cpp"
        var $7=HEAP32[(($inputBuffer)>>2)]; //@line 299 "sonic.cpp"
        var $8=$stream_addr; //@line 299 "sonic.cpp"
        var $numInputSamples=(($8+44)&4294967295); //@line 299 "sonic.cpp"
        var $9=HEAP32[(($numInputSamples)>>2)]; //@line 299 "sonic.cpp"
        var $10=$stream_addr; //@line 299 "sonic.cpp"
        var $numChannels3=(($10+28)&4294967295); //@line 299 "sonic.cpp"
        var $11=HEAP32[(($numChannels3)>>2)]; //@line 299 "sonic.cpp"
        var $mul4=((($11)*($9))&4294967295); //@line 299 "sonic.cpp"
        var $add_ptr=(($7+2*$mul4)&4294967295); //@line 299 "sonic.cpp"
        $buffer=$add_ptr; //@line 299 "sonic.cpp"
        var $12=$count; //@line 300 "sonic.cpp"
        var $dec1=((($12)-1)&4294967295); //@line 300 "sonic.cpp"
        $count=$dec1; //@line 300 "sonic.cpp"
        var $tobool52=((($12))|0)!=0; //@line 300 "sonic.cpp"
        if ($tobool52) { __label__ = 5;; } else { __label__ = 6;; } //@line 300 "sonic.cpp"
        $while_body$$while_end$7: do { 
          if (__label__ == 5) {
            while(1) { 
  
              var $13=$samples_addr; //@line 301 "sonic.cpp"
              var $incdec_ptr=(($13+1)&4294967295); //@line 301 "sonic.cpp"
              $samples_addr=$incdec_ptr; //@line 301 "sonic.cpp"
              var $14=HEAPU8[($13)]; //@line 301 "sonic.cpp"
              var $conv=((($14))&255); //@line 301 "sonic.cpp"
              var $sub=((($conv)-128)&4294967295); //@line 301 "sonic.cpp"
              var $shl=($sub) << 8; //@line 301 "sonic.cpp"
              var $conv6=((($shl)) & 65535); //@line 301 "sonic.cpp"
              var $15=$buffer; //@line 301 "sonic.cpp"
              var $incdec_ptr7=(($15+2)&4294967295); //@line 301 "sonic.cpp"
              $buffer=$incdec_ptr7; //@line 301 "sonic.cpp"
              HEAP16[(($15)>>1)]=$conv6; //@line 301 "sonic.cpp"
              var $16=$count; //@line 300 "sonic.cpp"
              var $dec=((($16)-1)&4294967295); //@line 300 "sonic.cpp"
              $count=$dec; //@line 300 "sonic.cpp"
              var $tobool5=((($16))|0)!=0; //@line 300 "sonic.cpp"
              if ($tobool5) { __label__ = 5;continue ; } else { __label__ = 6;break $while_body$$while_end$7; } //@line 300 "sonic.cpp"
            }
          }
        } while(0);
  
        var $17=$numSamples_addr; //@line 303 "sonic.cpp"
        var $18=$stream_addr; //@line 303 "sonic.cpp"
        var $numInputSamples8=(($18+44)&4294967295); //@line 303 "sonic.cpp"
        var $19=HEAP32[(($numInputSamples8)>>2)]; //@line 303 "sonic.cpp"
        var $add=((($19)+($17))&4294967295); //@line 303 "sonic.cpp"
        HEAP32[(($numInputSamples8)>>2)]=$add; //@line 303 "sonic.cpp"
        $retval=1; //@line 304 "sonic.cpp"
        ; //@line 304 "sonic.cpp"
      }
      else if (__label__ == 3) {
  
        $retval=0; //@line 297 "sonic.cpp"
        ; //@line 297 "sonic.cpp"
      }
    }
  
    var $20=$retval; //@line 305 "sonic.cpp"
    ;
    return $20; //@line 305 "sonic.cpp"
    return null;
  }
  

  function _sonicChangeFloatSpeed($samples, $numSamples, $speed, $pitch, $volume, $sampleRate, $numChannels) {
    ;
    var __label__;
  
    var $samples_addr;
    var $numSamples_addr;
    var $speed_addr;
    var $pitch_addr;
    var $volume_addr;
    var $sampleRate_addr;
    var $numChannels_addr;
    var $stream;
    $samples_addr=$samples;
    $numSamples_addr=$numSamples;
    $speed_addr=$speed;
    $pitch_addr=$pitch;
    $volume_addr=$volume;
    $sampleRate_addr=$sampleRate;
    $numChannels_addr=$numChannels;
    var $0=$sampleRate_addr; //@line 933 "sonic.cpp"
    var $1=$numChannels_addr; //@line 933 "sonic.cpp"
    var $call=_sonicCreateStream($0, $1); //@line 933 "sonic.cpp"
    $stream=$call; //@line 933 "sonic.cpp"
    var $2=$stream; //@line 935 "sonic.cpp"
    var $3=$speed_addr; //@line 935 "sonic.cpp"
    _sonicSetSpeed($2, $3); //@line 935 "sonic.cpp"
    var $4=$stream; //@line 936 "sonic.cpp"
    var $5=$pitch_addr; //@line 936 "sonic.cpp"
    _sonicSetPitch($4, $5); //@line 936 "sonic.cpp"
    var $6=$stream; //@line 937 "sonic.cpp"
    var $7=$volume_addr; //@line 937 "sonic.cpp"
    _sonicSetVolume($6, $7); //@line 937 "sonic.cpp"
    var $8=$stream; //@line 938 "sonic.cpp"
    var $9=$samples_addr; //@line 938 "sonic.cpp"
    var $10=$numSamples_addr; //@line 938 "sonic.cpp"
    var $call1=_sonicWriteFloatToStream($8, $9, $10); //@line 938 "sonic.cpp"
    var $11=$stream; //@line 939 "sonic.cpp"
    var $call2=_sonicFlushStream($11); //@line 939 "sonic.cpp"
    var $12=$stream; //@line 940 "sonic.cpp"
    var $call3=_sonicSamplesAvailable($12); //@line 940 "sonic.cpp"
    $numSamples_addr=$call3; //@line 940 "sonic.cpp"
    var $13=$stream; //@line 941 "sonic.cpp"
    var $14=$samples_addr; //@line 941 "sonic.cpp"
    var $15=$numSamples_addr; //@line 941 "sonic.cpp"
    var $call4=_sonicReadFloatFromStream($13, $14, $15); //@line 941 "sonic.cpp"
    var $16=$stream; //@line 942 "sonic.cpp"
    _sonicDestroyStream($16); //@line 942 "sonic.cpp"
    var $17=$numSamples_addr; //@line 943 "sonic.cpp"
    ;
    return $17; //@line 943 "sonic.cpp"
    return null;
  }
  

  function _sonicChangeShortSpeed($samples, $numSamples, $speed, $pitch, $volume, $sampleRate, $numChannels) {
    ;
    var __label__;
  
    var $samples_addr;
    var $numSamples_addr;
    var $speed_addr;
    var $pitch_addr;
    var $volume_addr;
    var $sampleRate_addr;
    var $numChannels_addr;
    var $stream;
    $samples_addr=$samples;
    $numSamples_addr=$numSamples;
    $speed_addr=$speed;
    $pitch_addr=$pitch;
    $volume_addr=$volume;
    $sampleRate_addr=$sampleRate;
    $numChannels_addr=$numChannels;
    var $0=$sampleRate_addr; //@line 956 "sonic.cpp"
    var $1=$numChannels_addr; //@line 956 "sonic.cpp"
    var $call=_sonicCreateStream($0, $1); //@line 956 "sonic.cpp"
    $stream=$call; //@line 956 "sonic.cpp"
    var $2=$stream; //@line 958 "sonic.cpp"
    var $3=$speed_addr; //@line 958 "sonic.cpp"
    _sonicSetSpeed($2, $3); //@line 958 "sonic.cpp"
    var $4=$stream; //@line 959 "sonic.cpp"
    var $5=$pitch_addr; //@line 959 "sonic.cpp"
    _sonicSetPitch($4, $5); //@line 959 "sonic.cpp"
    var $6=$stream; //@line 960 "sonic.cpp"
    var $7=$volume_addr; //@line 960 "sonic.cpp"
    _sonicSetVolume($6, $7); //@line 960 "sonic.cpp"
    var $8=$stream; //@line 961 "sonic.cpp"
    var $9=$samples_addr; //@line 961 "sonic.cpp"
    var $10=$numSamples_addr; //@line 961 "sonic.cpp"
    var $call1=_sonicWriteShortToStream($8, $9, $10); //@line 961 "sonic.cpp"
    var $11=$stream; //@line 962 "sonic.cpp"
    var $call2=_sonicFlushStream($11); //@line 962 "sonic.cpp"
    var $12=$stream; //@line 963 "sonic.cpp"
    var $call3=_sonicSamplesAvailable($12); //@line 963 "sonic.cpp"
    $numSamples_addr=$call3; //@line 963 "sonic.cpp"
    var $13=$stream; //@line 964 "sonic.cpp"
    var $14=$samples_addr; //@line 964 "sonic.cpp"
    var $15=$numSamples_addr; //@line 964 "sonic.cpp"
    var $call4=_sonicReadShortFromStream($13, $14, $15); //@line 964 "sonic.cpp"
    var $16=$stream; //@line 965 "sonic.cpp"
    _sonicDestroyStream($16); //@line 965 "sonic.cpp"
    var $17=$numSamples_addr; //@line 966 "sonic.cpp"
    ;
    return $17; //@line 966 "sonic.cpp"
    return null;
  }
  

  function __ZL26enlargeInputBufferIfNeededP17sonicStreamStructi($stream, $numSamples) {
    ;
    var __label__;
  
    var $retval;
    var $stream_addr;
    var $numSamples_addr;
    $stream_addr=$stream;
    $numSamples_addr=$numSamples;
    var $0=$stream_addr; //@line 232 "sonic.cpp"
    var $numInputSamples=(($0+44)&4294967295); //@line 232 "sonic.cpp"
    var $1=HEAP32[(($numInputSamples)>>2)]; //@line 232 "sonic.cpp"
    var $2=$numSamples_addr; //@line 232 "sonic.cpp"
    var $add=((($2)+($1))&4294967295); //@line 232 "sonic.cpp"
    var $3=$stream_addr; //@line 232 "sonic.cpp"
    var $inputBufferSize=(($3+32)&4294967295); //@line 232 "sonic.cpp"
    var $4=HEAP32[(($inputBufferSize)>>2)]; //@line 232 "sonic.cpp"
    var $cmp=((($add))|0) > ((($4))|0); //@line 232 "sonic.cpp"
    if ($cmp) { __label__ = 1;; } else { __label__ = 3;; } //@line 232 "sonic.cpp"
    $if_then$$if_end11$2: do { 
      if (__label__ == 1) {
  
        var $5=$stream_addr; //@line 233 "sonic.cpp"
        var $inputBufferSize1=(($5+32)&4294967295); //@line 233 "sonic.cpp"
        var $6=HEAP32[(($inputBufferSize1)>>2)]; //@line 233 "sonic.cpp"
        var $shr=($6) >> 1; //@line 233 "sonic.cpp"
        var $7=$numSamples_addr; //@line 233 "sonic.cpp"
        var $8=$stream_addr; //@line 233 "sonic.cpp"
        var $inputBufferSize3=(($8+32)&4294967295); //@line 233 "sonic.cpp"
        var $9=HEAP32[(($inputBufferSize3)>>2)]; //@line 233 "sonic.cpp"
        var $add2=((($shr)+($7))&4294967295); //@line 233 "sonic.cpp"
        var $add4=((($add2)+($9))&4294967295); //@line 233 "sonic.cpp"
        HEAP32[(($inputBufferSize3)>>2)]=$add4; //@line 233 "sonic.cpp"
        var $10=$stream_addr; //@line 234 "sonic.cpp"
        var $inputBuffer=(($10)&4294967295); //@line 234 "sonic.cpp"
        var $11=HEAP32[(($inputBuffer)>>2)]; //@line 234 "sonic.cpp"
        var $12=$11; //@line 234 "sonic.cpp"
        var $13=$stream_addr; //@line 234 "sonic.cpp"
        var $inputBufferSize5=(($13+32)&4294967295); //@line 234 "sonic.cpp"
        var $14=HEAP32[(($inputBufferSize5)>>2)]; //@line 234 "sonic.cpp"
        var $15=$stream_addr; //@line 234 "sonic.cpp"
        var $numChannels=(($15+28)&4294967295); //@line 234 "sonic.cpp"
        var $16=HEAP32[(($numChannels)>>2)]; //@line 234 "sonic.cpp"
        var $mul=((($14)*2)&4294967295); //@line 234 "sonic.cpp"
        var $mul6=((($mul)*($16))&4294967295); //@line 234 "sonic.cpp"
        var $call=_realloc($12, $mul6); //@line 234 "sonic.cpp"
        var $17=$call; //@line 234 "sonic.cpp"
        var $18=$stream_addr; //@line 234 "sonic.cpp"
        var $inputBuffer7=(($18)&4294967295); //@line 234 "sonic.cpp"
        HEAP32[(($inputBuffer7)>>2)]=$17; //@line 234 "sonic.cpp"
        var $19=$stream_addr; //@line 236 "sonic.cpp"
        var $inputBuffer8=(($19)&4294967295); //@line 236 "sonic.cpp"
        var $20=HEAP32[(($inputBuffer8)>>2)]; //@line 236 "sonic.cpp"
        var $cmp9=((($20))|0)==0; //@line 236 "sonic.cpp"
        if (!($cmp9)) { __label__ = 3;break $if_then$$if_end11$2; } //@line 236 "sonic.cpp"
  
        $retval=0; //@line 237 "sonic.cpp"
        __label__ = 4;break $if_then$$if_end11$2; //@line 237 "sonic.cpp"
      }
    } while(0);
    if (__label__ == 3) {
  
      $retval=1; //@line 240 "sonic.cpp"
      ; //@line 240 "sonic.cpp"
    }
  
    var $21=$retval; //@line 241 "sonic.cpp"
    ;
    return $21; //@line 241 "sonic.cpp"
    return null;
  }
  

  function __ZL11changeSpeedP17sonicStreamStructf($stream, $speed) {
    ;
    var __label__;
    var __lastLabel__ = null;
  
    var $retval;
    var $stream_addr;
    var $speed_addr;
    var $samples;
    var $numSamples;
    var $position;
    var $period;
    var $newSamples;
    var $maxRequired;
    $stream_addr=$stream;
    $speed_addr=$speed;
    var $0=$stream_addr; //@line 825 "sonic.cpp"
    var $numInputSamples=(($0+44)&4294967295); //@line 825 "sonic.cpp"
    var $1=HEAP32[(($numInputSamples)>>2)]; //@line 825 "sonic.cpp"
    $numSamples=$1; //@line 825 "sonic.cpp"
    $position=0; //@line 826 "sonic.cpp"
    var $2=$stream_addr; //@line 827 "sonic.cpp"
    var $maxRequired1=(($2+64)&4294967295); //@line 827 "sonic.cpp"
    var $3=HEAP32[(($maxRequired1)>>2)]; //@line 827 "sonic.cpp"
    $maxRequired=$3; //@line 827 "sonic.cpp"
    var $4=$stream_addr; //@line 829 "sonic.cpp"
    var $numInputSamples2=(($4+44)&4294967295); //@line 829 "sonic.cpp"
    var $5=HEAP32[(($numInputSamples2)>>2)]; //@line 829 "sonic.cpp"
    var $6=$maxRequired; //@line 829 "sonic.cpp"
    var $cmp=((($5))|0) < ((($6))|0); //@line 829 "sonic.cpp"
    if ($cmp) { __label__ = 1;; } else { __label__ = 2;; } //@line 829 "sonic.cpp"
    if (__label__ == 1) {
  
      $retval=1; //@line 830 "sonic.cpp"
      ; //@line 830 "sonic.cpp"
    }
    else if (__label__ == 2) {
      while(1) { 
  
        var $7=$stream_addr; //@line 833 "sonic.cpp"
        var $remainingInputToCopy=(($7+68)&4294967295); //@line 833 "sonic.cpp"
        var $8=HEAP32[(($remainingInputToCopy)>>2)]; //@line 833 "sonic.cpp"
        var $cmp3=((($8))|0) > 0; //@line 833 "sonic.cpp"
        var $9=$stream_addr; //@line 834 "sonic.cpp"
        if ($cmp3) { __label__ = 3;; } else { __label__ = 4;; } //@line 833 "sonic.cpp"
        if (__label__ == 3) {
  
          var $10=$position; //@line 834 "sonic.cpp"
          var $call=__ZL17copyInputToOutputP17sonicStreamStructi($9, $10); //@line 834 "sonic.cpp"
          $newSamples=$call; //@line 834 "sonic.cpp"
          var $11=$newSamples; //@line 835 "sonic.cpp"
          var $12=$position; //@line 835 "sonic.cpp"
          var $add=((($12)+($11))&4294967295); //@line 835 "sonic.cpp"
          $position=$add; //@line 835 "sonic.cpp"
          __lastLabel__ = 3; ; //@line 836 "sonic.cpp"
        }
        else if (__label__ == 4) {
  
          var $inputBuffer=(($9)&4294967295); //@line 837 "sonic.cpp"
          var $13=HEAP32[(($inputBuffer)>>2)]; //@line 837 "sonic.cpp"
          var $14=$position; //@line 837 "sonic.cpp"
          var $15=$stream_addr; //@line 837 "sonic.cpp"
          var $numChannels=(($15+28)&4294967295); //@line 837 "sonic.cpp"
          var $16=HEAP32[(($numChannels)>>2)]; //@line 837 "sonic.cpp"
          var $mul=((($16)*($14))&4294967295); //@line 837 "sonic.cpp"
          var $add_ptr=(($13+2*$mul)&4294967295); //@line 837 "sonic.cpp"
          $samples=$add_ptr; //@line 837 "sonic.cpp"
          var $17=$stream_addr; //@line 838 "sonic.cpp"
          var $18=$samples; //@line 838 "sonic.cpp"
          var $call5=__ZL15findPitchPeriodP17sonicStreamStructPs($17, $18); //@line 838 "sonic.cpp"
          $period=$call5; //@line 838 "sonic.cpp"
          var $19=$speed_addr; //@line 839 "sonic.cpp"
          var $conv=($19); //@line 839 "sonic.cpp"
          var $cmp6=($conv) > 1; //@line 839 "sonic.cpp"
          var $20=$stream_addr; //@line 840 "sonic.cpp"
          var $21=$samples; //@line 840 "sonic.cpp"
          var $22=$speed_addr; //@line 840 "sonic.cpp"
          var $23=$period; //@line 840 "sonic.cpp"
          if ($cmp6) { __label__ = 5;; } else { __label__ = 6;; } //@line 839 "sonic.cpp"
          if (__label__ == 5) {
  
            var $call8=__ZL15skipPitchPeriodP17sonicStreamStructPsfi($20, $21, $22, $23); //@line 840 "sonic.cpp"
            $newSamples=$call8; //@line 840 "sonic.cpp"
            var $24=$period; //@line 841 "sonic.cpp"
            var $25=$newSamples; //@line 841 "sonic.cpp"
            var $26=$position; //@line 841 "sonic.cpp"
            var $add9=((($25)+($24))&4294967295); //@line 841 "sonic.cpp"
            var $add10=((($add9)+($26))&4294967295); //@line 841 "sonic.cpp"
            $position=$add10; //@line 841 "sonic.cpp"
            __lastLabel__ = 5; ; //@line 842 "sonic.cpp"
          }
          else if (__label__ == 6) {
  
            var $call12=__ZL17insertPitchPeriodP17sonicStreamStructPsfi($20, $21, $22, $23); //@line 843 "sonic.cpp"
            $newSamples=$call12; //@line 843 "sonic.cpp"
            var $27=$newSamples; //@line 844 "sonic.cpp"
            var $28=$position; //@line 844 "sonic.cpp"
            var $add13=((($28)+($27))&4294967295); //@line 844 "sonic.cpp"
            $position=$add13; //@line 844 "sonic.cpp"
            __lastLabel__ = 6; ;
          }
        }
  
        var $29=__lastLabel__ == 5 ? $25 : (__lastLabel__ == 6 ? $27 : ($11)); //@line 847 "sonic.cpp"
        var $cmp16=((($29))|0)==0; //@line 847 "sonic.cpp"
        if ($cmp16) { __label__ = 8;break ; } //@line 847 "sonic.cpp"
  
        var $30=$position; //@line 850 "sonic.cpp"
        var $31=$maxRequired; //@line 850 "sonic.cpp"
        var $add19=((($31)+($30))&4294967295); //@line 850 "sonic.cpp"
        var $32=$numSamples; //@line 850 "sonic.cpp"
        var $cmp20=((($add19))|0) <= ((($32))|0); //@line 850 "sonic.cpp"
        if ($cmp20) { __label__ = 2;continue ; } else { __label__ = 10;break ; } //@line 850 "sonic.cpp"
      }
      if (__label__ == 8) {
  
        $retval=0; //@line 848 "sonic.cpp"
        ; //@line 848 "sonic.cpp"
      }
      else if (__label__ == 10) {
  
        var $33=$stream_addr; //@line 851 "sonic.cpp"
        var $34=$position; //@line 851 "sonic.cpp"
        __ZL18removeInputSamplesP17sonicStreamStructi($33, $34); //@line 851 "sonic.cpp"
        $retval=1; //@line 852 "sonic.cpp"
        ; //@line 852 "sonic.cpp"
      }
    }
  
    ;
    return;
    return;
  }
  

  function __ZL12copyToOutputP17sonicStreamStructPsi($stream, $samples, $numSamples) {
    ;
    var __label__;
  
    var $retval;
    var $stream_addr;
    var $samples_addr;
    var $numSamples_addr;
    $stream_addr=$stream;
    $samples_addr=$samples;
    $numSamples_addr=$numSamples;
    var $0=$stream_addr; //@line 327 "sonic.cpp"
    var $1=$numSamples_addr; //@line 327 "sonic.cpp"
    var $call=__ZL27enlargeOutputBufferIfNeededP17sonicStreamStructi($0, $1); //@line 327 "sonic.cpp"
    var $tobool=((($call))|0)!=0; //@line 327 "sonic.cpp"
    if ($tobool) { __label__ = 2;; } else { __label__ = 1;; } //@line 327 "sonic.cpp"
    if (__label__ == 2) {
  
      var $2=$stream_addr; //@line 330 "sonic.cpp"
      var $outputBuffer=(($2+4)&4294967295); //@line 330 "sonic.cpp"
      var $3=HEAP32[(($outputBuffer)>>2)]; //@line 330 "sonic.cpp"
      var $4=$stream_addr; //@line 330 "sonic.cpp"
      var $numOutputSamples=(($4+48)&4294967295); //@line 330 "sonic.cpp"
      var $5=HEAP32[(($numOutputSamples)>>2)]; //@line 330 "sonic.cpp"
      var $6=$stream_addr; //@line 330 "sonic.cpp"
      var $numChannels=(($6+28)&4294967295); //@line 330 "sonic.cpp"
      var $7=HEAP32[(($numChannels)>>2)]; //@line 330 "sonic.cpp"
      var $mul=((($7)*($5))&4294967295); //@line 330 "sonic.cpp"
      var $add_ptr=(($3+2*$mul)&4294967295); //@line 330 "sonic.cpp"
      var $8=$add_ptr; //@line 330 "sonic.cpp"
      var $9=$samples_addr; //@line 330 "sonic.cpp"
      var $10=$9; //@line 330 "sonic.cpp"
      var $11=$numSamples_addr; //@line 330 "sonic.cpp"
      var $12=$stream_addr; //@line 330 "sonic.cpp"
      var $numChannels2=(($12+28)&4294967295); //@line 330 "sonic.cpp"
      var $13=HEAP32[(($numChannels2)>>2)]; //@line 330 "sonic.cpp"
      var $mul1=((($11)*2)&4294967295); //@line 330 "sonic.cpp"
      var $mul3=((($mul1)*($13))&4294967295); //@line 330 "sonic.cpp"
      var $src$, $dest$, $stop$, $stop4$, $fast$;
      $src$ = $10;
      $dest$ = $8;
      $stop$ = $src$ + $mul3;
      if (($dest$%4) == ($src$%4) && $mul3 > 8) {
        while ($src$%4 !== 0 && $src$ < $stop$) {
          ; HEAP8[$dest$++] = HEAP8[$src$++];
        }
        $src$ >>= 2;
        $dest$ >>= 2;
        $stop4$ = $stop$ >> 2;
        while ($src$ < $stop4$) {
      ;;;    HEAP32[$dest$++] = HEAP32[$src$++];
        }
        $src$ <<= 2;
        $dest$ <<= 2;
      }while ($src$ < $stop$) {
        ; HEAP8[$dest$++] = HEAP8[$src$++];
      }; //@line 330 "sonic.cpp"
      var $14=$numSamples_addr; //@line 332 "sonic.cpp"
      var $15=$stream_addr; //@line 332 "sonic.cpp"
      var $numOutputSamples4=(($15+48)&4294967295); //@line 332 "sonic.cpp"
      var $16=HEAP32[(($numOutputSamples4)>>2)]; //@line 332 "sonic.cpp"
      var $add=((($16)+($14))&4294967295); //@line 332 "sonic.cpp"
      HEAP32[(($numOutputSamples4)>>2)]=$add; //@line 332 "sonic.cpp"
      var $17=$numSamples_addr; //@line 333 "sonic.cpp"
      $retval=$17; //@line 333 "sonic.cpp"
      ; //@line 333 "sonic.cpp"
    }
    else if (__label__ == 1) {
  
      $retval=0; //@line 328 "sonic.cpp"
      ; //@line 328 "sonic.cpp"
    }
  
    var $18=$retval; //@line 334 "sonic.cpp"
    ;
    return $18; //@line 334 "sonic.cpp"
    return null;
  }
  

  function __ZL11adjustPitchP17sonicStreamStructi($stream, $originalNumOutputSamples) {
    ;
    var __label__;
  
    var $retval;
    var $stream_addr;
    var $originalNumOutputSamples_addr;
    var $pitch;
    var $numChannels;
    var $period;
    var $newPeriod;
    var $separation;
    var $position;
    var $out;
    var $rampDown;
    var $rampUp;
    $stream_addr=$stream;
    $originalNumOutputSamples_addr=$originalNumOutputSamples;
    var $0=$stream_addr; //@line 729 "sonic.cpp"
    var $pitch1=(($0+24)&4294967295); //@line 729 "sonic.cpp"
    var $1=HEAPF32[(($pitch1)>>2)]; //@line 729 "sonic.cpp"
    $pitch=$1; //@line 729 "sonic.cpp"
    var $2=$stream_addr; //@line 730 "sonic.cpp"
    var $numChannels2=(($2+28)&4294967295); //@line 730 "sonic.cpp"
    var $3=HEAP32[(($numChannels2)>>2)]; //@line 730 "sonic.cpp"
    $numChannels=$3; //@line 730 "sonic.cpp"
    $position=0; //@line 732 "sonic.cpp"
    var $4=$stream_addr; //@line 735 "sonic.cpp"
    var $numOutputSamples=(($4+48)&4294967295); //@line 735 "sonic.cpp"
    var $5=HEAP32[(($numOutputSamples)>>2)]; //@line 735 "sonic.cpp"
    var $6=$originalNumOutputSamples_addr; //@line 735 "sonic.cpp"
    var $cmp=((($5))|0)==((($6))|0); //@line 735 "sonic.cpp"
    if ($cmp) { __label__ = 1;; } else { __label__ = 2;; } //@line 735 "sonic.cpp"
    if (__label__ == 1) {
  
      $retval=1; //@line 736 "sonic.cpp"
      ; //@line 736 "sonic.cpp"
    }
    else if (__label__ == 2) {
  
      var $7=$stream_addr; //@line 738 "sonic.cpp"
      var $8=$originalNumOutputSamples_addr; //@line 738 "sonic.cpp"
      var $call=__ZL27moveNewSamplesToPitchBufferP17sonicStreamStructi($7, $8); //@line 738 "sonic.cpp"
      var $tobool=((($call))|0)!=0; //@line 738 "sonic.cpp"
      if ($tobool) { __label__ = 4;; } else { __label__ = 3;; } //@line 738 "sonic.cpp"
      if (__label__ == 4) {
        while(1) { 
  
          var $9=$stream_addr; //@line 741 "sonic.cpp"
          var $numPitchSamples=(($9+52)&4294967295); //@line 741 "sonic.cpp"
          var $10=HEAP32[(($numPitchSamples)>>2)]; //@line 741 "sonic.cpp"
          var $11=$position; //@line 741 "sonic.cpp"
          var $sub=((($10)-($11))&4294967295); //@line 741 "sonic.cpp"
          var $12=$stream_addr; //@line 741 "sonic.cpp"
          var $maxRequired=(($12+64)&4294967295); //@line 741 "sonic.cpp"
          var $13=HEAP32[(($maxRequired)>>2)]; //@line 741 "sonic.cpp"
          var $cmp5=((($sub))|0) >= ((($13))|0); //@line 741 "sonic.cpp"
          var $14=$stream_addr; //@line 742 "sonic.cpp"
          if (!($cmp5)) { __label__ = 11;break ; } //@line 741 "sonic.cpp"
  
          var $15=$stream_addr; //@line 742 "sonic.cpp"
          var $pitchBuffer=(($15+8)&4294967295); //@line 742 "sonic.cpp"
          var $16=HEAP32[(($pitchBuffer)>>2)]; //@line 742 "sonic.cpp"
          var $17=$position; //@line 742 "sonic.cpp"
          var $18=$numChannels; //@line 742 "sonic.cpp"
          var $mul=((($18)*($17))&4294967295); //@line 742 "sonic.cpp"
          var $add_ptr=(($16+2*$mul)&4294967295); //@line 742 "sonic.cpp"
          var $call6=__ZL15findPitchPeriodP17sonicStreamStructPs($14, $add_ptr); //@line 742 "sonic.cpp"
          $period=$call6; //@line 742 "sonic.cpp"
          var $19=$period; //@line 743 "sonic.cpp"
          var $conv=((($19))|0); //@line 743 "sonic.cpp"
          var $20=$pitch; //@line 743 "sonic.cpp"
          var $div=($conv)/($20); //@line 743 "sonic.cpp"
          var $conv7=((($div))|0); //@line 743 "sonic.cpp"
          $newPeriod=$conv7; //@line 743 "sonic.cpp"
          var $21=$stream_addr; //@line 744 "sonic.cpp"
          var $22=$newPeriod; //@line 744 "sonic.cpp"
          var $call8=__ZL27enlargeOutputBufferIfNeededP17sonicStreamStructi($21, $22); //@line 744 "sonic.cpp"
          var $tobool9=((($call8))|0)!=0; //@line 744 "sonic.cpp"
          if (!($tobool9)) { __label__ = 6;break ; } //@line 744 "sonic.cpp"
  
          var $23=$stream_addr; //@line 747 "sonic.cpp"
          var $outputBuffer=(($23+4)&4294967295); //@line 747 "sonic.cpp"
          var $24=HEAP32[(($outputBuffer)>>2)]; //@line 747 "sonic.cpp"
          var $25=$stream_addr; //@line 747 "sonic.cpp"
          var $numOutputSamples12=(($25+48)&4294967295); //@line 747 "sonic.cpp"
          var $26=HEAP32[(($numOutputSamples12)>>2)]; //@line 747 "sonic.cpp"
          var $27=$numChannels; //@line 747 "sonic.cpp"
          var $mul13=((($27)*($26))&4294967295); //@line 747 "sonic.cpp"
          var $add_ptr14=(($24+2*$mul13)&4294967295); //@line 747 "sonic.cpp"
          $out=$add_ptr14; //@line 747 "sonic.cpp"
          var $28=$pitch; //@line 748 "sonic.cpp"
          var $cmp15=($28) >= 1; //@line 748 "sonic.cpp"
          var $29=$stream_addr; //@line 749 "sonic.cpp"
          var $pitchBuffer17=(($29+8)&4294967295); //@line 749 "sonic.cpp"
          var $30=HEAP32[(($pitchBuffer17)>>2)]; //@line 749 "sonic.cpp"
          var $31=$position; //@line 749 "sonic.cpp"
          var $32=$numChannels; //@line 749 "sonic.cpp"
          var $mul18=((($32)*($31))&4294967295); //@line 749 "sonic.cpp"
          var $add_ptr19=(($30+2*$mul18)&4294967295); //@line 749 "sonic.cpp"
          $rampDown=$add_ptr19; //@line 749 "sonic.cpp"
          var $33=$stream_addr; //@line 750 "sonic.cpp"
          var $pitchBuffer20=(($33+8)&4294967295); //@line 750 "sonic.cpp"
          var $34=HEAP32[(($pitchBuffer20)>>2)]; //@line 750 "sonic.cpp"
          var $35=$position; //@line 750 "sonic.cpp"
          if ($cmp15) { __label__ = 8;; } else { __label__ = 9;; } //@line 748 "sonic.cpp"
          if (__label__ == 8) {
  
            var $36=$period; //@line 750 "sonic.cpp"
            var $37=$newPeriod; //@line 750 "sonic.cpp"
            var $_neg=(((-($37)))&4294967295);
            var $add=((($36)+($35))&4294967295); //@line 750 "sonic.cpp"
            var $sub21=((($add)+($_neg))&4294967295); //@line 750 "sonic.cpp"
            var $38=$numChannels; //@line 750 "sonic.cpp"
            var $mul22=((($sub21)*($38))&4294967295); //@line 750 "sonic.cpp"
            var $add_ptr23=(($34+2*$mul22)&4294967295); //@line 750 "sonic.cpp"
            $rampUp=$add_ptr23; //@line 750 "sonic.cpp"
            var $39=$newPeriod; //@line 751 "sonic.cpp"
            var $40=$numChannels; //@line 751 "sonic.cpp"
            var $41=$out; //@line 751 "sonic.cpp"
            var $42=$rampDown; //@line 751 "sonic.cpp"
            var $43=$rampUp; //@line 751 "sonic.cpp"
            __ZL10overlapAddiiPsS_S_($39, $40, $41, $42, $43); //@line 751 "sonic.cpp"
            ; //@line 752 "sonic.cpp"
          }
          else if (__label__ == 9) {
  
            var $44=$numChannels; //@line 754 "sonic.cpp"
            var $mul28=((($44)*($35))&4294967295); //@line 754 "sonic.cpp"
            var $add_ptr29=(($34+2*$mul28)&4294967295); //@line 754 "sonic.cpp"
            $rampUp=$add_ptr29; //@line 754 "sonic.cpp"
            var $45=$newPeriod; //@line 755 "sonic.cpp"
            var $46=$period; //@line 755 "sonic.cpp"
            var $sub30=((($45)-($46))&4294967295); //@line 755 "sonic.cpp"
            $separation=$sub30; //@line 755 "sonic.cpp"
            var $47=$period; //@line 756 "sonic.cpp"
            var $48=$numChannels; //@line 756 "sonic.cpp"
            var $49=$separation; //@line 756 "sonic.cpp"
            var $50=$out; //@line 756 "sonic.cpp"
            var $51=$rampDown; //@line 756 "sonic.cpp"
            var $52=$rampUp; //@line 756 "sonic.cpp"
            __ZL24overlapAddWithSeparationiiiPsS_S_($47, $48, $49, $50, $51, $52); //@line 756 "sonic.cpp"
            ;
          }
  
          var $53=$newPeriod; //@line 758 "sonic.cpp"
          var $54=$stream_addr; //@line 758 "sonic.cpp"
          var $numOutputSamples32=(($54+48)&4294967295); //@line 758 "sonic.cpp"
          var $55=HEAP32[(($numOutputSamples32)>>2)]; //@line 758 "sonic.cpp"
          var $add33=((($55)+($53))&4294967295); //@line 758 "sonic.cpp"
          HEAP32[(($numOutputSamples32)>>2)]=$add33; //@line 758 "sonic.cpp"
          var $56=$period; //@line 759 "sonic.cpp"
          var $57=$position; //@line 759 "sonic.cpp"
          var $add34=((($57)+($56))&4294967295); //@line 759 "sonic.cpp"
          $position=$add34; //@line 759 "sonic.cpp"
          __label__ = 4;continue ; //@line 760 "sonic.cpp"
        }
        if (__label__ == 11) {
  
          var $58=$position; //@line 761 "sonic.cpp"
          __ZL18removePitchSamplesP17sonicStreamStructi($14, $58); //@line 761 "sonic.cpp"
          $retval=1; //@line 762 "sonic.cpp"
          ; //@line 762 "sonic.cpp"
        }
        else if (__label__ == 6) {
  
          $retval=0; //@line 745 "sonic.cpp"
          ; //@line 745 "sonic.cpp"
        }
      }
      else if (__label__ == 3) {
  
        $retval=0; //@line 739 "sonic.cpp"
        ; //@line 739 "sonic.cpp"
      }
    }
  
    var $59=$retval; //@line 763 "sonic.cpp"
    ;
    return $59; //@line 763 "sonic.cpp"
    return null;
  }
  

  function __ZL12scaleSamplesPsif($samples, $numSamples, $volume) {
    ;
    var __label__;
  
    var $samples_addr;
    var $numSamples_addr;
    var $volume_addr;
    var $fixedPointVolume;
    var $value;
    $samples_addr=$samples;
    $numSamples_addr=$numSamples;
    $volume_addr=$volume;
    var $0=$volume_addr; //@line 75 "sonic.cpp"
    var $mul=($0)*4096; //@line 75 "sonic.cpp"
    var $conv=((($mul))|0); //@line 75 "sonic.cpp"
    $fixedPointVolume=$conv; //@line 75 "sonic.cpp"
    var $1=$numSamples_addr; //@line 78 "sonic.cpp"
    var $dec1=((($1)-1)&4294967295); //@line 78 "sonic.cpp"
    $numSamples_addr=$dec1; //@line 78 "sonic.cpp"
    var $tobool2=((($1))|0)!=0; //@line 78 "sonic.cpp"
    if ($tobool2) { __label__ = 1;; } else { __label__ = 6;; } //@line 78 "sonic.cpp"
    $while_body$$while_end$2: do { 
      if (__label__ == 1) {
        $while_body$3: while(1) { 
  
          var $2=$samples_addr; //@line 79 "sonic.cpp"
          var $3=HEAP16[(($2)>>1)]; //@line 79 "sonic.cpp"
          var $conv1=(tempInt=(($3)),(tempInt>=32768?tempInt-65536:tempInt)); //@line 79 "sonic.cpp"
          var $4=$fixedPointVolume; //@line 79 "sonic.cpp"
          var $mul2=((($conv1)*($4))&4294967295); //@line 79 "sonic.cpp"
          var $shr=($mul2) >> 12; //@line 79 "sonic.cpp"
          $value=$shr; //@line 79 "sonic.cpp"
          var $cmp=((($shr))|0) > 32767; //@line 80 "sonic.cpp"
          if ($cmp) { __label__ = 2;; } else { __label__ = 3;; } //@line 80 "sonic.cpp"
          $if_then$$if_else$5: do { 
            if (__label__ == 2) {
  
              $value=32767; //@line 81 "sonic.cpp"
              ; //@line 82 "sonic.cpp"
            }
            else if (__label__ == 3) {
  
              var $5=$value; //@line 82 "sonic.cpp"
              var $cmp3=((($5))|0) < -32767; //@line 82 "sonic.cpp"
              if (!($cmp3)) { __label__ = 5;break $if_then$$if_else$5; } //@line 82 "sonic.cpp"
  
              $value=-32767; //@line 83 "sonic.cpp"
              ; //@line 84 "sonic.cpp"
            }
          } while(0);
  
          var $6=$value; //@line 85 "sonic.cpp"
          var $conv6=((($6)) & 65535); //@line 85 "sonic.cpp"
          var $7=$samples_addr; //@line 85 "sonic.cpp"
          var $incdec_ptr=(($7+2)&4294967295); //@line 85 "sonic.cpp"
          $samples_addr=$incdec_ptr; //@line 85 "sonic.cpp"
          HEAP16[(($7)>>1)]=$conv6; //@line 85 "sonic.cpp"
          var $8=$numSamples_addr; //@line 78 "sonic.cpp"
          var $dec=((($8)-1)&4294967295); //@line 78 "sonic.cpp"
          $numSamples_addr=$dec; //@line 78 "sonic.cpp"
          var $tobool=((($8))|0)!=0; //@line 78 "sonic.cpp"
          if ($tobool) { __label__ = 1;continue $while_body$3; } else { __label__ = 6;break $while_body$$while_end$2; } //@line 78 "sonic.cpp"
        }
      }
    } while(0);
  
    ;
    return; //@line 87 "sonic.cpp"
    return;
  }
  

  function __ZL27moveNewSamplesToPitchBufferP17sonicStreamStructi($stream, $originalNumOutputSamples) {
    ;
    var __label__;
  
    var $retval;
    var $stream_addr;
    var $originalNumOutputSamples_addr;
    var $numSamples;
    var $numChannels;
    $stream_addr=$stream;
    $originalNumOutputSamples_addr=$originalNumOutputSamples;
    var $0=$stream_addr; //@line 686 "sonic.cpp"
    var $numOutputSamples=(($0+48)&4294967295); //@line 686 "sonic.cpp"
    var $1=HEAP32[(($numOutputSamples)>>2)]; //@line 686 "sonic.cpp"
    var $2=$originalNumOutputSamples_addr; //@line 686 "sonic.cpp"
    var $sub=((($1)-($2))&4294967295); //@line 686 "sonic.cpp"
    $numSamples=$sub; //@line 686 "sonic.cpp"
    var $3=$stream_addr; //@line 687 "sonic.cpp"
    var $numChannels1=(($3+28)&4294967295); //@line 687 "sonic.cpp"
    var $4=HEAP32[(($numChannels1)>>2)]; //@line 687 "sonic.cpp"
    $numChannels=$4; //@line 687 "sonic.cpp"
    var $5=$stream_addr; //@line 689 "sonic.cpp"
    var $numPitchSamples=(($5+52)&4294967295); //@line 689 "sonic.cpp"
    var $6=HEAP32[(($numPitchSamples)>>2)]; //@line 689 "sonic.cpp"
    var $7=$numSamples; //@line 689 "sonic.cpp"
    var $add=((($7)+($6))&4294967295); //@line 689 "sonic.cpp"
    var $8=$stream_addr; //@line 689 "sonic.cpp"
    var $pitchBufferSize=(($8+36)&4294967295); //@line 689 "sonic.cpp"
    var $9=HEAP32[(($pitchBufferSize)>>2)]; //@line 689 "sonic.cpp"
    var $cmp=((($add))|0) > ((($9))|0); //@line 689 "sonic.cpp"
    if ($cmp) { __label__ = 1;; } else { __label__ = 3;; } //@line 689 "sonic.cpp"
    $if_then$$if_end12$2: do { 
      if (__label__ == 1) {
  
        var $10=$stream_addr; //@line 690 "sonic.cpp"
        var $pitchBufferSize2=(($10+36)&4294967295); //@line 690 "sonic.cpp"
        var $11=HEAP32[(($pitchBufferSize2)>>2)]; //@line 690 "sonic.cpp"
        var $shr=($11) >> 1; //@line 690 "sonic.cpp"
        var $12=$numSamples; //@line 690 "sonic.cpp"
        var $13=$stream_addr; //@line 690 "sonic.cpp"
        var $pitchBufferSize4=(($13+36)&4294967295); //@line 690 "sonic.cpp"
        var $14=HEAP32[(($pitchBufferSize4)>>2)]; //@line 690 "sonic.cpp"
        var $add3=((($shr)+($12))&4294967295); //@line 690 "sonic.cpp"
        var $add5=((($add3)+($14))&4294967295); //@line 690 "sonic.cpp"
        HEAP32[(($pitchBufferSize4)>>2)]=$add5; //@line 690 "sonic.cpp"
        var $15=$stream_addr; //@line 691 "sonic.cpp"
        var $pitchBuffer=(($15+8)&4294967295); //@line 691 "sonic.cpp"
        var $16=HEAP32[(($pitchBuffer)>>2)]; //@line 691 "sonic.cpp"
        var $17=$16; //@line 691 "sonic.cpp"
        var $18=$stream_addr; //@line 691 "sonic.cpp"
        var $pitchBufferSize6=(($18+36)&4294967295); //@line 691 "sonic.cpp"
        var $19=HEAP32[(($pitchBufferSize6)>>2)]; //@line 691 "sonic.cpp"
        var $20=$numChannels; //@line 691 "sonic.cpp"
        var $mul=((($19)*2)&4294967295); //@line 691 "sonic.cpp"
        var $mul7=((($mul)*($20))&4294967295); //@line 691 "sonic.cpp"
        var $call=_realloc($17, $mul7); //@line 691 "sonic.cpp"
        var $21=$call; //@line 691 "sonic.cpp"
        var $22=$stream_addr; //@line 691 "sonic.cpp"
        var $pitchBuffer8=(($22+8)&4294967295); //@line 691 "sonic.cpp"
        HEAP32[(($pitchBuffer8)>>2)]=$21; //@line 691 "sonic.cpp"
        var $23=$stream_addr; //@line 693 "sonic.cpp"
        var $pitchBuffer9=(($23+8)&4294967295); //@line 693 "sonic.cpp"
        var $24=HEAP32[(($pitchBuffer9)>>2)]; //@line 693 "sonic.cpp"
        var $cmp10=((($24))|0)==0; //@line 693 "sonic.cpp"
        if (!($cmp10)) { __label__ = 3;break $if_then$$if_end12$2; } //@line 693 "sonic.cpp"
  
        $retval=0; //@line 694 "sonic.cpp"
        __label__ = 4;break $if_then$$if_end12$2; //@line 694 "sonic.cpp"
      }
    } while(0);
    if (__label__ == 3) {
  
      var $25=$stream_addr; //@line 697 "sonic.cpp"
      var $pitchBuffer13=(($25+8)&4294967295); //@line 697 "sonic.cpp"
      var $26=HEAP32[(($pitchBuffer13)>>2)]; //@line 697 "sonic.cpp"
      var $27=$stream_addr; //@line 697 "sonic.cpp"
      var $numPitchSamples14=(($27+52)&4294967295); //@line 697 "sonic.cpp"
      var $28=HEAP32[(($numPitchSamples14)>>2)]; //@line 697 "sonic.cpp"
      var $29=$numChannels; //@line 697 "sonic.cpp"
      var $mul15=((($29)*($28))&4294967295); //@line 697 "sonic.cpp"
      var $add_ptr=(($26+2*$mul15)&4294967295); //@line 697 "sonic.cpp"
      var $30=$add_ptr; //@line 697 "sonic.cpp"
      var $31=$stream_addr; //@line 697 "sonic.cpp"
      var $outputBuffer=(($31+4)&4294967295); //@line 697 "sonic.cpp"
      var $32=HEAP32[(($outputBuffer)>>2)]; //@line 697 "sonic.cpp"
      var $33=$originalNumOutputSamples_addr; //@line 697 "sonic.cpp"
      var $34=$numChannels; //@line 697 "sonic.cpp"
      var $mul16=((($34)*($33))&4294967295); //@line 697 "sonic.cpp"
      var $add_ptr17=(($32+2*$mul16)&4294967295); //@line 697 "sonic.cpp"
      var $35=$add_ptr17; //@line 697 "sonic.cpp"
      var $36=$numSamples; //@line 697 "sonic.cpp"
      var $37=$numChannels; //@line 697 "sonic.cpp"
      var $mul18=((($36)*2)&4294967295); //@line 697 "sonic.cpp"
      var $mul19=((($mul18)*($37))&4294967295); //@line 697 "sonic.cpp"
      var $src$, $dest$, $stop$, $stop4$, $fast$;
      $src$ = $35;
      $dest$ = $30;
      $stop$ = $src$ + $mul19;
      if (($dest$%4) == ($src$%4) && $mul19 > 8) {
        while ($src$%4 !== 0 && $src$ < $stop$) {
          ; HEAP8[$dest$++] = HEAP8[$src$++];
        }
        $src$ >>= 2;
        $dest$ >>= 2;
        $stop4$ = $stop$ >> 2;
        while ($src$ < $stop4$) {
      ;;;    HEAP32[$dest$++] = HEAP32[$src$++];
        }
        $src$ <<= 2;
        $dest$ <<= 2;
      }while ($src$ < $stop$) {
        ; HEAP8[$dest$++] = HEAP8[$src$++];
      }; //@line 697 "sonic.cpp"
      var $38=$originalNumOutputSamples_addr; //@line 700 "sonic.cpp"
      var $39=$stream_addr; //@line 700 "sonic.cpp"
      var $numOutputSamples20=(($39+48)&4294967295); //@line 700 "sonic.cpp"
      HEAP32[(($numOutputSamples20)>>2)]=$38; //@line 700 "sonic.cpp"
      var $40=$numSamples; //@line 701 "sonic.cpp"
      var $41=$stream_addr; //@line 701 "sonic.cpp"
      var $numPitchSamples21=(($41+52)&4294967295); //@line 701 "sonic.cpp"
      var $42=HEAP32[(($numPitchSamples21)>>2)]; //@line 701 "sonic.cpp"
      var $add22=((($42)+($40))&4294967295); //@line 701 "sonic.cpp"
      HEAP32[(($numPitchSamples21)>>2)]=$add22; //@line 701 "sonic.cpp"
      $retval=1; //@line 702 "sonic.cpp"
      ; //@line 702 "sonic.cpp"
    }
  
    var $43=$retval; //@line 703 "sonic.cpp"
    ;
    return $43; //@line 703 "sonic.cpp"
    return null;
  }
  

  function __ZL15findPitchPeriodP17sonicStreamStructPs($stream, $samples) {
    var __stackBase__  = STACKTOP; STACKTOP += 8; _memset(__stackBase__, 0, 8);
    var __label__;
  
    var $stream_addr;
    var $samples_addr;
    var $minPeriod;
    var $maxPeriod;
    var $sampleRate;
    var $minDiff=__stackBase__;
    var $maxDiff=__stackBase__+4;
    var $retPeriod;
    var $skip;
    var $period;
    $stream_addr=$stream;
    $samples_addr=$samples;
    var $0=$stream_addr; //@line 575 "sonic.cpp"
    var $minPeriod1=(($0+56)&4294967295); //@line 575 "sonic.cpp"
    var $1=HEAP32[(($minPeriod1)>>2)]; //@line 575 "sonic.cpp"
    $minPeriod=$1; //@line 575 "sonic.cpp"
    var $2=$stream_addr; //@line 576 "sonic.cpp"
    var $maxPeriod2=(($2+60)&4294967295); //@line 576 "sonic.cpp"
    var $3=HEAP32[(($maxPeriod2)>>2)]; //@line 576 "sonic.cpp"
    $maxPeriod=$3; //@line 576 "sonic.cpp"
    var $4=$stream_addr; //@line 577 "sonic.cpp"
    var $sampleRate3=(($4+72)&4294967295); //@line 577 "sonic.cpp"
    var $5=HEAP32[(($sampleRate3)>>2)]; //@line 577 "sonic.cpp"
    $sampleRate=$5; //@line 577 "sonic.cpp"
    $skip=1; //@line 579 "sonic.cpp"
    var $6=$sampleRate; //@line 582 "sonic.cpp"
    var $cmp=((($6))|0) > 4000; //@line 582 "sonic.cpp"
    if ($cmp) { __label__ = 1;; } else { __label__ = 2;; } //@line 582 "sonic.cpp"
    if (__label__ == 1) {
  
      var $7=$sampleRate; //@line 583 "sonic.cpp"
      var $div=((((($7))|0)/4000)|0); //@line 583 "sonic.cpp"
      $skip=$div; //@line 583 "sonic.cpp"
      ; //@line 584 "sonic.cpp"
    }
  
    var $8=$stream_addr; //@line 585 "sonic.cpp"
    var $numChannels=(($8+28)&4294967295); //@line 585 "sonic.cpp"
    var $9=HEAP32[(($numChannels)>>2)]; //@line 585 "sonic.cpp"
    var $cmp4=((($9))|0)==1; //@line 585 "sonic.cpp"
    if ($cmp4) { __label__ = 3;; } else { __label__ = 5;; } //@line 585 "sonic.cpp"
    $land_lhs_true$$if_else$5: do { 
      if (__label__ == 3) {
  
        var $10=$skip; //@line 585 "sonic.cpp"
        var $cmp5=((($10))|0)==1; //@line 585 "sonic.cpp"
        if (!($cmp5)) { __label__ = 5;break $land_lhs_true$$if_else$5; } //@line 585 "sonic.cpp"
  
        var $11=$samples_addr; //@line 586 "sonic.cpp"
        var $12=$minPeriod; //@line 586 "sonic.cpp"
        var $13=$maxPeriod; //@line 586 "sonic.cpp"
        var $call=__ZL22findPitchPeriodInRangePsiiPiS0_($11, $12, $13, $minDiff, $maxDiff); //@line 586 "sonic.cpp"
        $period=$call; //@line 586 "sonic.cpp"
        __label__ = 13;break $land_lhs_true$$if_else$5; //@line 587 "sonic.cpp"
      }
    } while(0);
    $if_end32$$if_else$8: do { 
      if (__label__ == 5) {
  
        var $14=$stream_addr; //@line 588 "sonic.cpp"
        var $15=$samples_addr; //@line 588 "sonic.cpp"
        var $16=$skip; //@line 588 "sonic.cpp"
        __ZL15downSampleInputP17sonicStreamStructPsi($14, $15, $16); //@line 588 "sonic.cpp"
        var $17=$stream_addr; //@line 589 "sonic.cpp"
        var $downSampleBuffer=(($17+12)&4294967295); //@line 589 "sonic.cpp"
        var $18=HEAP32[(($downSampleBuffer)>>2)]; //@line 589 "sonic.cpp"
        var $19=$minPeriod; //@line 589 "sonic.cpp"
        var $20=$skip; //@line 589 "sonic.cpp"
        var $div7=((((($19))|0)/((($20))|0))|0); //@line 589 "sonic.cpp"
        var $21=$maxPeriod; //@line 589 "sonic.cpp"
        var $22=$skip; //@line 589 "sonic.cpp"
        var $div8=((((($21))|0)/((($22))|0))|0); //@line 589 "sonic.cpp"
        var $call9=__ZL22findPitchPeriodInRangePsiiPiS0_($18, $div7, $div8, $minDiff, $maxDiff); //@line 589 "sonic.cpp"
        $period=$call9; //@line 589 "sonic.cpp"
        var $23=$skip; //@line 591 "sonic.cpp"
        var $cmp10=((($23))|0)!=1; //@line 591 "sonic.cpp"
        if (!($cmp10)) { __label__ = 13;break $if_end32$$if_else$8; } //@line 591 "sonic.cpp"
  
        var $24=$skip; //@line 592 "sonic.cpp"
        var $25=$period; //@line 592 "sonic.cpp"
        var $mul=((($25)*($24))&4294967295); //@line 592 "sonic.cpp"
        $period=$mul; //@line 592 "sonic.cpp"
        var $26=$period; //@line 593 "sonic.cpp"
        var $27=$skip; //@line 593 "sonic.cpp"
        var $shl=($27) << 2; //@line 593 "sonic.cpp"
        var $sub=((($26)-($shl))&4294967295); //@line 593 "sonic.cpp"
        $minPeriod=$sub; //@line 593 "sonic.cpp"
        var $28=$period; //@line 594 "sonic.cpp"
        var $29=$skip; //@line 594 "sonic.cpp"
        var $shl12=((($29)*4)&4294967295); //@line 594 "sonic.cpp"
        var $add=((($shl12)+($28))&4294967295); //@line 594 "sonic.cpp"
        $maxPeriod=$add; //@line 594 "sonic.cpp"
        var $30=$minPeriod; //@line 595 "sonic.cpp"
        var $31=$stream_addr; //@line 595 "sonic.cpp"
        var $minPeriod13=(($31+56)&4294967295); //@line 595 "sonic.cpp"
        var $32=HEAP32[(($minPeriod13)>>2)]; //@line 595 "sonic.cpp"
        var $cmp14=((($30))|0) < ((($32))|0); //@line 595 "sonic.cpp"
        if ($cmp14) { __label__ = 7;; } else { __label__ = 8;; } //@line 595 "sonic.cpp"
        if (__label__ == 7) {
  
          var $33=$stream_addr; //@line 596 "sonic.cpp"
          var $minPeriod16=(($33+56)&4294967295); //@line 596 "sonic.cpp"
          var $34=HEAP32[(($minPeriod16)>>2)]; //@line 596 "sonic.cpp"
          $minPeriod=$34; //@line 596 "sonic.cpp"
          ; //@line 597 "sonic.cpp"
        }
  
        var $35=$maxPeriod; //@line 598 "sonic.cpp"
        var $36=$stream_addr; //@line 598 "sonic.cpp"
        var $maxPeriod18=(($36+60)&4294967295); //@line 598 "sonic.cpp"
        var $37=HEAP32[(($maxPeriod18)>>2)]; //@line 598 "sonic.cpp"
        var $cmp19=((($35))|0) > ((($37))|0); //@line 598 "sonic.cpp"
        if ($cmp19) { __label__ = 9;; } else { __label__ = 10;; } //@line 598 "sonic.cpp"
        if (__label__ == 9) {
  
          var $38=$stream_addr; //@line 599 "sonic.cpp"
          var $maxPeriod21=(($38+60)&4294967295); //@line 599 "sonic.cpp"
          var $39=HEAP32[(($maxPeriod21)>>2)]; //@line 599 "sonic.cpp"
          $maxPeriod=$39; //@line 599 "sonic.cpp"
          ; //@line 600 "sonic.cpp"
        }
  
        var $40=$stream_addr; //@line 601 "sonic.cpp"
        var $numChannels23=(($40+28)&4294967295); //@line 601 "sonic.cpp"
        var $41=HEAP32[(($numChannels23)>>2)]; //@line 601 "sonic.cpp"
        var $cmp24=((($41))|0)==1; //@line 601 "sonic.cpp"
        if ($cmp24) { __label__ = 11;; } else { __label__ = 12;; } //@line 601 "sonic.cpp"
        if (__label__ == 11) {
  
          var $42=$samples_addr; //@line 602 "sonic.cpp"
          var $43=$minPeriod; //@line 602 "sonic.cpp"
          var $44=$maxPeriod; //@line 602 "sonic.cpp"
          var $call26=__ZL22findPitchPeriodInRangePsiiPiS0_($42, $43, $44, $minDiff, $maxDiff); //@line 602 "sonic.cpp"
          $period=$call26; //@line 602 "sonic.cpp"
          ; //@line 604 "sonic.cpp"
        }
        else if (__label__ == 12) {
  
          var $45=$stream_addr; //@line 605 "sonic.cpp"
          var $46=$samples_addr; //@line 605 "sonic.cpp"
          __ZL15downSampleInputP17sonicStreamStructPsi($45, $46, 1); //@line 605 "sonic.cpp"
          var $47=$stream_addr; //@line 606 "sonic.cpp"
          var $downSampleBuffer28=(($47+12)&4294967295); //@line 606 "sonic.cpp"
          var $48=HEAP32[(($downSampleBuffer28)>>2)]; //@line 606 "sonic.cpp"
          var $49=$minPeriod; //@line 606 "sonic.cpp"
          var $50=$maxPeriod; //@line 606 "sonic.cpp"
          var $call29=__ZL22findPitchPeriodInRangePsiiPiS0_($48, $49, $50, $minDiff, $maxDiff); //@line 606 "sonic.cpp"
          $period=$call29; //@line 606 "sonic.cpp"
          ;
        }
      }
    } while(0);
  
    var $51=$stream_addr; //@line 611 "sonic.cpp"
    var $52=$period; //@line 611 "sonic.cpp"
    var $53=HEAP32[(($minDiff)>>2)]; //@line 611 "sonic.cpp"
    var $54=HEAP32[(($maxDiff)>>2)]; //@line 611 "sonic.cpp"
    var $call33=__ZL16prevPeriodBetterP17sonicStreamStructiii($51, $52, $53, $54); //@line 611 "sonic.cpp"
    var $tobool=((($call33))|0)!=0; //@line 611 "sonic.cpp"
    if ($tobool) { __label__ = 14;; } else { __label__ = 15;; } //@line 611 "sonic.cpp"
    if (__label__ == 14) {
  
      var $55=$stream_addr; //@line 612 "sonic.cpp"
      var $prevPeriod=(($55+76)&4294967295); //@line 612 "sonic.cpp"
      var $56=HEAP32[(($prevPeriod)>>2)]; //@line 612 "sonic.cpp"
      $retPeriod=$56; //@line 612 "sonic.cpp"
      ; //@line 613 "sonic.cpp"
    }
    else if (__label__ == 15) {
  
      var $57=$period; //@line 614 "sonic.cpp"
      $retPeriod=$57; //@line 614 "sonic.cpp"
      ;
    }
  
    var $58=HEAP32[(($minDiff)>>2)]; //@line 616 "sonic.cpp"
    var $59=$stream_addr; //@line 616 "sonic.cpp"
    var $prevMinDiff=(($59+84)&4294967295); //@line 616 "sonic.cpp"
    HEAP32[(($prevMinDiff)>>2)]=$58; //@line 616 "sonic.cpp"
    var $60=HEAP32[(($maxDiff)>>2)]; //@line 617 "sonic.cpp"
    var $61=$stream_addr; //@line 617 "sonic.cpp"
    var $prevMaxDiff=(($61+80)&4294967295); //@line 617 "sonic.cpp"
    HEAP32[(($prevMaxDiff)>>2)]=$60; //@line 617 "sonic.cpp"
    var $62=$period; //@line 618 "sonic.cpp"
    var $63=$stream_addr; //@line 618 "sonic.cpp"
    var $prevPeriod37=(($63+76)&4294967295); //@line 618 "sonic.cpp"
    HEAP32[(($prevPeriod37)>>2)]=$62; //@line 618 "sonic.cpp"
    var $64=$retPeriod; //@line 619 "sonic.cpp"
    STACKTOP = __stackBase__;
    return $64; //@line 619 "sonic.cpp"
    return null;
  }
  

  function __ZL27enlargeOutputBufferIfNeededP17sonicStreamStructi($stream, $numSamples) {
    ;
    var __label__;
  
    var $retval;
    var $stream_addr;
    var $numSamples_addr;
    $stream_addr=$stream;
    $numSamples_addr=$numSamples;
    var $0=$stream_addr; //@line 216 "sonic.cpp"
    var $numOutputSamples=(($0+48)&4294967295); //@line 216 "sonic.cpp"
    var $1=HEAP32[(($numOutputSamples)>>2)]; //@line 216 "sonic.cpp"
    var $2=$numSamples_addr; //@line 216 "sonic.cpp"
    var $add=((($2)+($1))&4294967295); //@line 216 "sonic.cpp"
    var $3=$stream_addr; //@line 216 "sonic.cpp"
    var $outputBufferSize=(($3+40)&4294967295); //@line 216 "sonic.cpp"
    var $4=HEAP32[(($outputBufferSize)>>2)]; //@line 216 "sonic.cpp"
    var $cmp=((($add))|0) > ((($4))|0); //@line 216 "sonic.cpp"
    if ($cmp) { __label__ = 1;; } else { __label__ = 3;; } //@line 216 "sonic.cpp"
    $if_then$$if_end11$2: do { 
      if (__label__ == 1) {
  
        var $5=$stream_addr; //@line 217 "sonic.cpp"
        var $outputBufferSize1=(($5+40)&4294967295); //@line 217 "sonic.cpp"
        var $6=HEAP32[(($outputBufferSize1)>>2)]; //@line 217 "sonic.cpp"
        var $shr=($6) >> 1; //@line 217 "sonic.cpp"
        var $7=$numSamples_addr; //@line 217 "sonic.cpp"
        var $8=$stream_addr; //@line 217 "sonic.cpp"
        var $outputBufferSize3=(($8+40)&4294967295); //@line 217 "sonic.cpp"
        var $9=HEAP32[(($outputBufferSize3)>>2)]; //@line 217 "sonic.cpp"
        var $add2=((($shr)+($7))&4294967295); //@line 217 "sonic.cpp"
        var $add4=((($add2)+($9))&4294967295); //@line 217 "sonic.cpp"
        HEAP32[(($outputBufferSize3)>>2)]=$add4; //@line 217 "sonic.cpp"
        var $10=$stream_addr; //@line 218 "sonic.cpp"
        var $outputBuffer=(($10+4)&4294967295); //@line 218 "sonic.cpp"
        var $11=HEAP32[(($outputBuffer)>>2)]; //@line 218 "sonic.cpp"
        var $12=$11; //@line 218 "sonic.cpp"
        var $13=$stream_addr; //@line 218 "sonic.cpp"
        var $outputBufferSize5=(($13+40)&4294967295); //@line 218 "sonic.cpp"
        var $14=HEAP32[(($outputBufferSize5)>>2)]; //@line 218 "sonic.cpp"
        var $15=$stream_addr; //@line 218 "sonic.cpp"
        var $numChannels=(($15+28)&4294967295); //@line 218 "sonic.cpp"
        var $16=HEAP32[(($numChannels)>>2)]; //@line 218 "sonic.cpp"
        var $mul=((($14)*2)&4294967295); //@line 218 "sonic.cpp"
        var $mul6=((($mul)*($16))&4294967295); //@line 218 "sonic.cpp"
        var $call=_realloc($12, $mul6); //@line 218 "sonic.cpp"
        var $17=$call; //@line 218 "sonic.cpp"
        var $18=$stream_addr; //@line 218 "sonic.cpp"
        var $outputBuffer7=(($18+4)&4294967295); //@line 218 "sonic.cpp"
        HEAP32[(($outputBuffer7)>>2)]=$17; //@line 218 "sonic.cpp"
        var $19=$stream_addr; //@line 220 "sonic.cpp"
        var $outputBuffer8=(($19+4)&4294967295); //@line 220 "sonic.cpp"
        var $20=HEAP32[(($outputBuffer8)>>2)]; //@line 220 "sonic.cpp"
        var $cmp9=((($20))|0)==0; //@line 220 "sonic.cpp"
        if (!($cmp9)) { __label__ = 3;break $if_then$$if_end11$2; } //@line 220 "sonic.cpp"
  
        $retval=0; //@line 221 "sonic.cpp"
        __label__ = 4;break $if_then$$if_end11$2; //@line 221 "sonic.cpp"
      }
    } while(0);
    if (__label__ == 3) {
  
      $retval=1; //@line 224 "sonic.cpp"
      ; //@line 224 "sonic.cpp"
    }
  
    var $21=$retval; //@line 225 "sonic.cpp"
    ;
    return $21; //@line 225 "sonic.cpp"
    return null;
  }
  

  function __ZL10overlapAddiiPsS_S_($numSamples, $numChannels, $out, $rampDown, $rampUp) {
    ;
    var __label__;
  
    var $numSamples_addr;
    var $numChannels_addr;
    var $out_addr;
    var $rampDown_addr;
    var $rampUp_addr;
    var $o;
    var $u;
    var $d;
    var $i;
    var $t;
    $numSamples_addr=$numSamples;
    $numChannels_addr=$numChannels;
    $out_addr=$out;
    $rampDown_addr=$rampDown;
    $rampUp_addr=$rampUp;
    $i=0; //@line 634 "sonic.cpp"
    var $0=$i; //@line 634 "sonic.cpp"
    var $1=$numChannels_addr; //@line 634 "sonic.cpp"
    var $cmp2=((($0))|0) < ((($1))|0); //@line 634 "sonic.cpp"
    if ($cmp2) { __label__ = 1;; } else { __label__ = 4;; } //@line 634 "sonic.cpp"
    $for_body$$for_end14$2: do { 
      if (__label__ == 1) {
        $for_body$3: while(1) { 
  
          var $2=$out_addr; //@line 635 "sonic.cpp"
          var $3=$i; //@line 635 "sonic.cpp"
          var $add_ptr=(($2+2*$3)&4294967295); //@line 635 "sonic.cpp"
          $o=$add_ptr; //@line 635 "sonic.cpp"
          var $4=$rampUp_addr; //@line 636 "sonic.cpp"
          var $5=$i; //@line 636 "sonic.cpp"
          var $add_ptr1=(($4+2*$5)&4294967295); //@line 636 "sonic.cpp"
          $u=$add_ptr1; //@line 636 "sonic.cpp"
          var $6=$rampDown_addr; //@line 637 "sonic.cpp"
          var $7=$i; //@line 637 "sonic.cpp"
          var $add_ptr2=(($6+2*$7)&4294967295); //@line 637 "sonic.cpp"
          $d=$add_ptr2; //@line 637 "sonic.cpp"
          $t=0; //@line 638 "sonic.cpp"
          var $8=$t; //@line 638 "sonic.cpp"
          var $9=$numSamples_addr; //@line 638 "sonic.cpp"
          var $cmp41=((($8))|0) < ((($9))|0); //@line 638 "sonic.cpp"
          if ($cmp41) { __label__ = 2;; } else { __label__ = 3;; } //@line 638 "sonic.cpp"
          $for_body5$$for_inc12$5: do { 
            if (__label__ == 2) {
              while(1) { 
  
                var $10=$d; //@line 639 "sonic.cpp"
                var $11=HEAP16[(($10)>>1)]; //@line 639 "sonic.cpp"
                var $conv=(tempInt=(($11)),(tempInt>=32768?tempInt-65536:tempInt)); //@line 639 "sonic.cpp"
                var $12=$numSamples_addr; //@line 639 "sonic.cpp"
                var $13=$t; //@line 639 "sonic.cpp"
                var $sub=((($12)-($13))&4294967295); //@line 639 "sonic.cpp"
                var $mul=((($sub)*($conv))&4294967295); //@line 639 "sonic.cpp"
                var $14=$u; //@line 639 "sonic.cpp"
                var $15=HEAP16[(($14)>>1)]; //@line 639 "sonic.cpp"
                var $conv6=(tempInt=(($15)),(tempInt>=32768?tempInt-65536:tempInt)); //@line 639 "sonic.cpp"
                var $16=$t; //@line 639 "sonic.cpp"
                var $mul7=((($conv6)*($16))&4294967295); //@line 639 "sonic.cpp"
                var $add=((($mul7)+($mul))&4294967295); //@line 639 "sonic.cpp"
                var $17=$numSamples_addr; //@line 639 "sonic.cpp"
                var $div=((((($add))|0)/((($17))|0))|0); //@line 639 "sonic.cpp"
                var $conv8=((($div)) & 65535); //@line 639 "sonic.cpp"
                var $18=$o; //@line 639 "sonic.cpp"
                HEAP16[(($18)>>1)]=$conv8; //@line 639 "sonic.cpp"
                var $19=$numChannels_addr; //@line 640 "sonic.cpp"
                var $20=$o; //@line 640 "sonic.cpp"
                var $add_ptr9=(($20+2*$19)&4294967295); //@line 640 "sonic.cpp"
                $o=$add_ptr9; //@line 640 "sonic.cpp"
                var $21=$numChannels_addr; //@line 641 "sonic.cpp"
                var $22=$d; //@line 641 "sonic.cpp"
                var $add_ptr10=(($22+2*$21)&4294967295); //@line 641 "sonic.cpp"
                $d=$add_ptr10; //@line 641 "sonic.cpp"
                var $23=$numChannels_addr; //@line 642 "sonic.cpp"
                var $24=$u; //@line 642 "sonic.cpp"
                var $add_ptr11=(($24+2*$23)&4294967295); //@line 642 "sonic.cpp"
                $u=$add_ptr11; //@line 642 "sonic.cpp"
                var $25=$t; //@line 638 "sonic.cpp"
                var $inc=((($25)+1)&4294967295); //@line 638 "sonic.cpp"
                $t=$inc; //@line 638 "sonic.cpp"
                var $26=$t; //@line 638 "sonic.cpp"
                var $27=$numSamples_addr; //@line 638 "sonic.cpp"
                var $cmp4=((($26))|0) < ((($27))|0); //@line 638 "sonic.cpp"
                if ($cmp4) { __label__ = 2;continue ; } else { __label__ = 3;break $for_body5$$for_inc12$5; } //@line 638 "sonic.cpp"
              }
            }
          } while(0);
  
          var $28=$i; //@line 634 "sonic.cpp"
          var $inc13=((($28)+1)&4294967295); //@line 634 "sonic.cpp"
          $i=$inc13; //@line 634 "sonic.cpp"
          var $29=$i; //@line 634 "sonic.cpp"
          var $30=$numChannels_addr; //@line 634 "sonic.cpp"
          var $cmp=((($29))|0) < ((($30))|0); //@line 634 "sonic.cpp"
          if ($cmp) { __label__ = 1;continue $for_body$3; } else { __label__ = 4;break $for_body$$for_end14$2; } //@line 634 "sonic.cpp"
        }
      }
    } while(0);
  
    ;
    return; //@line 645 "sonic.cpp"
    return;
  }
  

  function __ZL24overlapAddWithSeparationiiiPsS_S_($numSamples, $numChannels, $separation, $out, $rampDown, $rampUp) {
    ;
    var __label__;
  
    var $numSamples_addr;
    var $numChannels_addr;
    var $separation_addr;
    var $out_addr;
    var $rampDown_addr;
    var $rampUp_addr;
    var $o;
    var $u;
    var $d;
    var $i;
    var $t;
    $numSamples_addr=$numSamples;
    $numChannels_addr=$numChannels;
    $separation_addr=$separation;
    $out_addr=$out;
    $rampDown_addr=$rampDown;
    $rampUp_addr=$rampUp;
    $i=0; //@line 660 "sonic.cpp"
    var $0=$i; //@line 660 "sonic.cpp"
    var $1=$numChannels_addr; //@line 660 "sonic.cpp"
    var $cmp3=((($0))|0) < ((($1))|0); //@line 660 "sonic.cpp"
    if ($cmp3) { __label__ = 1;; } else { __label__ = 9;; } //@line 660 "sonic.cpp"
    $for_body$$for_end33$2: do { 
      if (__label__ == 1) {
        $for_body$3: while(1) { 
  
          var $2=$out_addr; //@line 661 "sonic.cpp"
          var $3=$i; //@line 661 "sonic.cpp"
          var $add_ptr=(($2+2*$3)&4294967295); //@line 661 "sonic.cpp"
          $o=$add_ptr; //@line 661 "sonic.cpp"
          var $4=$rampUp_addr; //@line 662 "sonic.cpp"
          var $5=$i; //@line 662 "sonic.cpp"
          var $add_ptr1=(($4+2*$5)&4294967295); //@line 662 "sonic.cpp"
          $u=$add_ptr1; //@line 662 "sonic.cpp"
          var $6=$rampDown_addr; //@line 663 "sonic.cpp"
          var $7=$i; //@line 663 "sonic.cpp"
          var $add_ptr2=(($6+2*$7)&4294967295); //@line 663 "sonic.cpp"
          $d=$add_ptr2; //@line 663 "sonic.cpp"
          $t=0; //@line 664 "sonic.cpp"
          var $8=$t; //@line 664 "sonic.cpp"
          var $9=$numSamples_addr; //@line 664 "sonic.cpp"
          var $10=$separation_addr; //@line 664 "sonic.cpp"
          var $add1=((($10)+($9))&4294967295); //@line 664 "sonic.cpp"
          var $cmp42=((($8))|0) < ((($add1))|0); //@line 664 "sonic.cpp"
          if ($cmp42) { __label__ = 2;; } else { __label__ = 8;; } //@line 664 "sonic.cpp"
          $for_body5$$for_inc31$5: do { 
            if (__label__ == 2) {
              while(1) { 
  
                var $11=$t; //@line 665 "sonic.cpp"
                var $12=$separation_addr; //@line 665 "sonic.cpp"
                var $cmp6=((($11))|0) < ((($12))|0); //@line 665 "sonic.cpp"
                if ($cmp6) { __label__ = 3;; } else { __label__ = 4;; } //@line 665 "sonic.cpp"
                if (__label__ == 3) {
  
                  var $13=$d; //@line 666 "sonic.cpp"
                  var $14=HEAP16[(($13)>>1)]; //@line 666 "sonic.cpp"
                  var $conv=(tempInt=(($14)),(tempInt>=32768?tempInt-65536:tempInt)); //@line 666 "sonic.cpp"
                  var $15=$numSamples_addr; //@line 666 "sonic.cpp"
                  var $16=$t; //@line 666 "sonic.cpp"
                  var $sub=((($15)-($16))&4294967295); //@line 666 "sonic.cpp"
                  var $mul=((($sub)*($conv))&4294967295); //@line 666 "sonic.cpp"
                  var $17=$numSamples_addr; //@line 666 "sonic.cpp"
                  var $div=((((($mul))|0)/((($17))|0))|0); //@line 666 "sonic.cpp"
                  var $conv7=((($div)) & 65535); //@line 666 "sonic.cpp"
                  var $18=$o; //@line 666 "sonic.cpp"
                  HEAP16[(($18)>>1)]=$conv7; //@line 666 "sonic.cpp"
                  var $19=$numChannels_addr; //@line 667 "sonic.cpp"
                  var $20=$d; //@line 667 "sonic.cpp"
                  var $add_ptr8=(($20+2*$19)&4294967295); //@line 667 "sonic.cpp"
                  $d=$add_ptr8; //@line 667 "sonic.cpp"
                  ; //@line 668 "sonic.cpp"
                }
                else if (__label__ == 4) {
  
                  var $21=$t; //@line 668 "sonic.cpp"
                  var $22=$numSamples_addr; //@line 668 "sonic.cpp"
                  var $cmp9=((($21))|0) < ((($22))|0); //@line 668 "sonic.cpp"
                  if ($cmp9) { __label__ = 5;; } else { __label__ = 6;; } //@line 668 "sonic.cpp"
                  if (__label__ == 5) {
  
                    var $23=$d; //@line 669 "sonic.cpp"
                    var $24=HEAP16[(($23)>>1)]; //@line 669 "sonic.cpp"
                    var $conv11=(tempInt=(($24)),(tempInt>=32768?tempInt-65536:tempInt)); //@line 669 "sonic.cpp"
                    var $25=$numSamples_addr; //@line 669 "sonic.cpp"
                    var $26=$t; //@line 669 "sonic.cpp"
                    var $sub12=((($25)-($26))&4294967295); //@line 669 "sonic.cpp"
                    var $mul13=((($sub12)*($conv11))&4294967295); //@line 669 "sonic.cpp"
                    var $27=$u; //@line 669 "sonic.cpp"
                    var $28=HEAP16[(($27)>>1)]; //@line 669 "sonic.cpp"
                    var $conv14=(tempInt=(($28)),(tempInt>=32768?tempInt-65536:tempInt)); //@line 669 "sonic.cpp"
                    var $29=$t; //@line 669 "sonic.cpp"
                    var $30=$separation_addr; //@line 669 "sonic.cpp"
                    var $sub15=((($29)-($30))&4294967295); //@line 669 "sonic.cpp"
                    var $mul16=((($sub15)*($conv14))&4294967295); //@line 669 "sonic.cpp"
                    var $add17=((($mul16)+($mul13))&4294967295); //@line 669 "sonic.cpp"
                    var $31=$numSamples_addr; //@line 669 "sonic.cpp"
                    var $div18=((((($add17))|0)/((($31))|0))|0); //@line 669 "sonic.cpp"
                    var $conv19=((($div18)) & 65535); //@line 669 "sonic.cpp"
                    var $32=$o; //@line 669 "sonic.cpp"
                    HEAP16[(($32)>>1)]=$conv19; //@line 669 "sonic.cpp"
                    var $33=$numChannels_addr; //@line 670 "sonic.cpp"
                    var $34=$d; //@line 670 "sonic.cpp"
                    var $add_ptr20=(($34+2*$33)&4294967295); //@line 670 "sonic.cpp"
                    $d=$add_ptr20; //@line 670 "sonic.cpp"
                    var $35=$numChannels_addr; //@line 671 "sonic.cpp"
                    var $36=$u; //@line 671 "sonic.cpp"
                    var $add_ptr21=(($36+2*$35)&4294967295); //@line 671 "sonic.cpp"
                    $u=$add_ptr21; //@line 671 "sonic.cpp"
                    ; //@line 672 "sonic.cpp"
                  }
                  else if (__label__ == 6) {
  
                    var $37=$u; //@line 673 "sonic.cpp"
                    var $38=HEAP16[(($37)>>1)]; //@line 673 "sonic.cpp"
                    var $conv23=(tempInt=(($38)),(tempInt>=32768?tempInt-65536:tempInt)); //@line 673 "sonic.cpp"
                    var $39=$t; //@line 673 "sonic.cpp"
                    var $40=$separation_addr; //@line 673 "sonic.cpp"
                    var $sub24=((($39)-($40))&4294967295); //@line 673 "sonic.cpp"
                    var $mul25=((($sub24)*($conv23))&4294967295); //@line 673 "sonic.cpp"
                    var $41=$numSamples_addr; //@line 673 "sonic.cpp"
                    var $div26=((((($mul25))|0)/((($41))|0))|0); //@line 673 "sonic.cpp"
                    var $conv27=((($div26)) & 65535); //@line 673 "sonic.cpp"
                    var $42=$o; //@line 673 "sonic.cpp"
                    HEAP16[(($42)>>1)]=$conv27; //@line 673 "sonic.cpp"
                    var $43=$numChannels_addr; //@line 674 "sonic.cpp"
                    var $44=$u; //@line 674 "sonic.cpp"
                    var $add_ptr28=(($44+2*$43)&4294967295); //@line 674 "sonic.cpp"
                    $u=$add_ptr28; //@line 674 "sonic.cpp"
                    ;
                  }
                }
  
                var $45=$numChannels_addr; //@line 676 "sonic.cpp"
                var $46=$o; //@line 676 "sonic.cpp"
                var $add_ptr30=(($46+2*$45)&4294967295); //@line 676 "sonic.cpp"
                $o=$add_ptr30; //@line 676 "sonic.cpp"
                var $47=$t; //@line 664 "sonic.cpp"
                var $inc=((($47)+1)&4294967295); //@line 664 "sonic.cpp"
                $t=$inc; //@line 664 "sonic.cpp"
                var $48=$t; //@line 664 "sonic.cpp"
                var $49=$numSamples_addr; //@line 664 "sonic.cpp"
                var $50=$separation_addr; //@line 664 "sonic.cpp"
                var $add=((($50)+($49))&4294967295); //@line 664 "sonic.cpp"
                var $cmp4=((($48))|0) < ((($add))|0); //@line 664 "sonic.cpp"
                if ($cmp4) { __label__ = 2;continue ; } else { __label__ = 8;break $for_body5$$for_inc31$5; } //@line 664 "sonic.cpp"
              }
            }
          } while(0);
  
          var $51=$i; //@line 660 "sonic.cpp"
          var $inc32=((($51)+1)&4294967295); //@line 660 "sonic.cpp"
          $i=$inc32; //@line 660 "sonic.cpp"
          var $52=$i; //@line 660 "sonic.cpp"
          var $53=$numChannels_addr; //@line 660 "sonic.cpp"
          var $cmp=((($52))|0) < ((($53))|0); //@line 660 "sonic.cpp"
          if ($cmp) { __label__ = 1;continue $for_body$3; } else { __label__ = 9;break $for_body$$for_end33$2; } //@line 660 "sonic.cpp"
        }
      }
    } while(0);
  
    ;
    return; //@line 679 "sonic.cpp"
    return;
  }
  

  function __ZL18removePitchSamplesP17sonicStreamStructi($stream, $numSamples) {
    ;
    var __label__;
  
    var $stream_addr;
    var $numSamples_addr;
    var $numChannels;
    var $source;
    $stream_addr=$stream;
    $numSamples_addr=$numSamples;
    var $0=$stream_addr; //@line 710 "sonic.cpp"
    var $numChannels1=(($0+28)&4294967295); //@line 710 "sonic.cpp"
    var $1=HEAP32[(($numChannels1)>>2)]; //@line 710 "sonic.cpp"
    $numChannels=$1; //@line 710 "sonic.cpp"
    var $2=$stream_addr; //@line 711 "sonic.cpp"
    var $pitchBuffer=(($2+8)&4294967295); //@line 711 "sonic.cpp"
    var $3=HEAP32[(($pitchBuffer)>>2)]; //@line 711 "sonic.cpp"
    var $4=$numSamples_addr; //@line 711 "sonic.cpp"
    var $5=$numChannels; //@line 711 "sonic.cpp"
    var $mul=((($5)*($4))&4294967295); //@line 711 "sonic.cpp"
    var $add_ptr=(($3+2*$mul)&4294967295); //@line 711 "sonic.cpp"
    $source=$add_ptr; //@line 711 "sonic.cpp"
    var $6=$numSamples_addr; //@line 713 "sonic.cpp"
    var $cmp=((($6))|0)==0; //@line 713 "sonic.cpp"
    if ($cmp) { __label__ = 4;; } else { __label__ = 1;; } //@line 713 "sonic.cpp"
    if (__label__ == 1) {
  
      var $7=$numSamples_addr; //@line 716 "sonic.cpp"
      var $8=$stream_addr; //@line 716 "sonic.cpp"
      var $numPitchSamples=(($8+52)&4294967295); //@line 716 "sonic.cpp"
      var $9=HEAP32[(($numPitchSamples)>>2)]; //@line 716 "sonic.cpp"
      var $cmp2=((($7))|0)!=((($9))|0); //@line 716 "sonic.cpp"
      if ($cmp2) { __label__ = 2;; } else { __label__ = 3;; } //@line 716 "sonic.cpp"
      if (__label__ == 2) {
  
        var $10=$stream_addr; //@line 717 "sonic.cpp"
        var $pitchBuffer4=(($10+8)&4294967295); //@line 717 "sonic.cpp"
        var $11=HEAP32[(($pitchBuffer4)>>2)]; //@line 717 "sonic.cpp"
        var $12=$11; //@line 717 "sonic.cpp"
        var $13=$source; //@line 717 "sonic.cpp"
        var $14=$13; //@line 717 "sonic.cpp"
        var $15=$stream_addr; //@line 717 "sonic.cpp"
        var $numPitchSamples5=(($15+52)&4294967295); //@line 717 "sonic.cpp"
        var $16=HEAP32[(($numPitchSamples5)>>2)]; //@line 717 "sonic.cpp"
        var $17=$numSamples_addr; //@line 717 "sonic.cpp"
        var $sub=((($16)-($17))&4294967295); //@line 717 "sonic.cpp"
        var $18=$numChannels; //@line 717 "sonic.cpp"
        var $mul6=((($18)*2)&4294967295); //@line 717 "sonic.cpp"
        var $mul7=((($mul6)*($sub))&4294967295); //@line 717 "sonic.cpp"
        _llvm_memmove_p0i8_p0i8_i32($12, $14, $mul7, 1, 0); //@line 717 "sonic.cpp"
        ; //@line 719 "sonic.cpp"
      }
  
      var $19=$numSamples_addr; //@line 720 "sonic.cpp"
      var $20=$stream_addr; //@line 720 "sonic.cpp"
      var $numPitchSamples9=(($20+52)&4294967295); //@line 720 "sonic.cpp"
      var $21=HEAP32[(($numPitchSamples9)>>2)]; //@line 720 "sonic.cpp"
      var $sub10=((($21)-($19))&4294967295); //@line 720 "sonic.cpp"
      HEAP32[(($numPitchSamples9)>>2)]=$sub10; //@line 720 "sonic.cpp"
      ; //@line 721 "sonic.cpp"
    }
  
    ;
    return; //@line 721 "sonic.cpp"
    return;
  }
  

  function __ZL22findPitchPeriodInRangePsiiPiS0_($samples, $minPeriod, $maxPeriod, $retMinDiff, $retMaxDiff) {
    ;
    var __label__;
    var __lastLabel__ = null;
  
    var $samples_addr;
    var $minPeriod_addr;
    var $maxPeriod_addr;
    var $retMinDiff_addr;
    var $retMaxDiff_addr;
    var $period;
    var $bestPeriod;
    var $s;
    var $p;
    var $sVal;
    var $pVal;
    var $diff;
    var $minDiff;
    var $maxDiff;
    var $i;
    $samples_addr=$samples;
    $minPeriod_addr=$minPeriod;
    $maxPeriod_addr=$maxPeriod;
    $retMinDiff_addr=$retMinDiff;
    $retMaxDiff_addr=$retMaxDiff;
    $bestPeriod=0; //@line 521 "sonic.cpp"
    $minDiff=1; //@line 523 "sonic.cpp"
    $maxDiff=0; //@line 523 "sonic.cpp"
    var $0=$minPeriod_addr; //@line 526 "sonic.cpp"
    $period=$0; //@line 526 "sonic.cpp"
    var $1=$period; //@line 526 "sonic.cpp"
    var $2=$maxPeriod_addr; //@line 526 "sonic.cpp"
    var $cmp3=((($1))|0) <= ((($2))|0); //@line 526 "sonic.cpp"
    if ($cmp3) { __label__ = 1;; } else { __label__ = 11;; } //@line 526 "sonic.cpp"
    $for_body$$for_end24$2: do { 
      if (__label__ == 1) {
        $for_body$3: while(1) { 
  
          $diff=0; //@line 527 "sonic.cpp"
          var $3=$samples_addr; //@line 528 "sonic.cpp"
          $s=$3; //@line 528 "sonic.cpp"
          var $4=$samples_addr; //@line 529 "sonic.cpp"
          var $5=$period; //@line 529 "sonic.cpp"
          var $add_ptr=(($4+2*$5)&4294967295); //@line 529 "sonic.cpp"
          $p=$add_ptr; //@line 529 "sonic.cpp"
          $i=0; //@line 530 "sonic.cpp"
          var $6=$i; //@line 530 "sonic.cpp"
          var $7=$period; //@line 530 "sonic.cpp"
          var $cmp21=((($6))|0) < ((($7))|0); //@line 530 "sonic.cpp"
          if ($cmp21) { __label__ = 2;; } else { __label__ = 6;; } //@line 530 "sonic.cpp"
          $for_body3$$for_end$5: do { 
            if (__label__ == 2) {
              while(1) { 
  
                var $8=$s; //@line 531 "sonic.cpp"
                var $incdec_ptr=(($8+2)&4294967295); //@line 531 "sonic.cpp"
                $s=$incdec_ptr; //@line 531 "sonic.cpp"
                var $9=HEAP16[(($8)>>1)]; //@line 531 "sonic.cpp"
                $sVal=$9; //@line 531 "sonic.cpp"
                var $10=$p; //@line 532 "sonic.cpp"
                var $incdec_ptr4=(($10+2)&4294967295); //@line 532 "sonic.cpp"
                $p=$incdec_ptr4; //@line 532 "sonic.cpp"
                var $11=HEAP16[(($10)>>1)]; //@line 532 "sonic.cpp"
                $pVal=$11; //@line 532 "sonic.cpp"
                var $12=$sVal; //@line 533 "sonic.cpp"
                var $conv=(tempInt=(($12)),(tempInt>=32768?tempInt-65536:tempInt)); //@line 533 "sonic.cpp"
                var $13=$pVal; //@line 533 "sonic.cpp"
                var $conv5=(tempInt=(($13)),(tempInt>=32768?tempInt-65536:tempInt)); //@line 533 "sonic.cpp"
                var $cmp6=((($conv))|0) >= ((($conv5))|0); //@line 533 "sonic.cpp"
                if ($cmp6) { __label__ = 3;; } else { __label__ = 4;; } //@line 533 "sonic.cpp"
                if (__label__ == 3) {
  
                  var $14=$sVal; //@line 533 "sonic.cpp"
                  var $conv7=(tempInt=(($14)),(tempInt>=32768?tempInt-65536:tempInt)); //@line 533 "sonic.cpp"
                  var $15=$pVal; //@line 533 "sonic.cpp"
                  var $conv8=(tempInt=(($15)),(tempInt>=32768?tempInt-65536:tempInt)); //@line 533 "sonic.cpp"
                  var $sub=((($conv7)-($conv8))&4294967295); //@line 533 "sonic.cpp"
                  var $conv9=((($sub)) & 65535); //@line 533 "sonic.cpp"
                  __lastLabel__ = 3; ; //@line 533 "sonic.cpp"
                }
                else if (__label__ == 4) {
  
                  var $16=$pVal; //@line 533 "sonic.cpp"
                  var $conv10=(tempInt=(($16)),(tempInt>=32768?tempInt-65536:tempInt)); //@line 533 "sonic.cpp"
                  var $17=$sVal; //@line 533 "sonic.cpp"
                  var $conv11=(tempInt=(($17)),(tempInt>=32768?tempInt-65536:tempInt)); //@line 533 "sonic.cpp"
                  var $sub12=((($conv10)-($conv11))&4294967295); //@line 533 "sonic.cpp"
                  var $conv13=((($sub12)) & 65535); //@line 533 "sonic.cpp"
                  __lastLabel__ = 4; ; //@line 533 "sonic.cpp"
                }
  
                var $cond=__lastLabel__ == 3 ? $conv9 : ($conv13); //@line 533 "sonic.cpp"
                var $conv14=((($cond))&65535); //@line 533 "sonic.cpp"
                var $18=$diff; //@line 533 "sonic.cpp"
                var $add=((($18)+($conv14))&4294967295); //@line 533 "sonic.cpp"
                $diff=$add; //@line 533 "sonic.cpp"
                var $19=$i; //@line 530 "sonic.cpp"
                var $inc=((($19)+1)&4294967295); //@line 530 "sonic.cpp"
                $i=$inc; //@line 530 "sonic.cpp"
                var $20=$i; //@line 530 "sonic.cpp"
                var $21=$period; //@line 530 "sonic.cpp"
                var $cmp2=((($20))|0) < ((($21))|0); //@line 530 "sonic.cpp"
                if ($cmp2) { __label__ = 2;continue ; } else { __label__ = 6;break $for_body3$$for_end$5; } //@line 530 "sonic.cpp"
              }
            }
          } while(0);
  
          var $22=$diff; //@line 539 "sonic.cpp"
          var $23=$bestPeriod; //@line 539 "sonic.cpp"
          var $mul=((($23)*($22))&4294967295); //@line 539 "sonic.cpp"
          var $24=$minDiff; //@line 539 "sonic.cpp"
          var $25=$period; //@line 539 "sonic.cpp"
          var $mul15=((($25)*($24))&4294967295); //@line 539 "sonic.cpp"
          var $cmp16=((($mul))>>>0) < ((($mul15))>>>0); //@line 539 "sonic.cpp"
          if ($cmp16) { __label__ = 7;; } else { __label__ = 8;; } //@line 539 "sonic.cpp"
          if (__label__ == 7) {
  
            var $26=$diff; //@line 540 "sonic.cpp"
            $minDiff=$26; //@line 540 "sonic.cpp"
            var $27=$period; //@line 541 "sonic.cpp"
            $bestPeriod=$27; //@line 541 "sonic.cpp"
            ; //@line 542 "sonic.cpp"
          }
  
          var $28=$diff; //@line 543 "sonic.cpp"
          var $29=$bestPeriod; //@line 543 "sonic.cpp"
          var $mul17=((($29)*($28))&4294967295); //@line 543 "sonic.cpp"
          var $30=$maxDiff; //@line 543 "sonic.cpp"
          var $31=$period; //@line 543 "sonic.cpp"
          var $mul18=((($31)*($30))&4294967295); //@line 543 "sonic.cpp"
          var $cmp19=((($mul17))>>>0) > ((($mul18))>>>0); //@line 543 "sonic.cpp"
          if ($cmp19) { __label__ = 9;; } else { __label__ = 10;; } //@line 543 "sonic.cpp"
          if (__label__ == 9) {
  
            var $32=$diff; //@line 544 "sonic.cpp"
            $maxDiff=$32; //@line 544 "sonic.cpp"
            ; //@line 545 "sonic.cpp"
          }
  
          var $33=$period; //@line 526 "sonic.cpp"
          var $inc23=((($33)+1)&4294967295); //@line 526 "sonic.cpp"
          $period=$inc23; //@line 526 "sonic.cpp"
          var $34=$period; //@line 526 "sonic.cpp"
          var $35=$maxPeriod_addr; //@line 526 "sonic.cpp"
          var $cmp=((($34))|0) <= ((($35))|0); //@line 526 "sonic.cpp"
          if ($cmp) { __label__ = 1;continue $for_body$3; } else { __label__ = 11;break $for_body$$for_end24$2; } //@line 526 "sonic.cpp"
        }
      }
    } while(0);
  
    var $36=$minDiff; //@line 547 "sonic.cpp"
    var $37=$retMinDiff_addr; //@line 547 "sonic.cpp"
    HEAP32[(($37)>>2)]=$36; //@line 547 "sonic.cpp"
    var $38=$maxDiff; //@line 548 "sonic.cpp"
    var $39=$retMaxDiff_addr; //@line 548 "sonic.cpp"
    HEAP32[(($39)>>2)]=$38; //@line 548 "sonic.cpp"
    var $40=$bestPeriod; //@line 549 "sonic.cpp"
    ;
    return $40; //@line 549 "sonic.cpp"
    return null;
  }
  

  function __ZL15downSampleInputP17sonicStreamStructPsi($stream, $samples, $skip) {
    ;
    var __label__;
  
    var $stream_addr;
    var $samples_addr;
    var $skip_addr;
    var $numSamples;
    var $samplesPerValue;
    var $i;
    var $j;
    var $value;
    var $downSamples;
    $stream_addr=$stream;
    $samples_addr=$samples;
    $skip_addr=$skip;
    var $0=$stream_addr; //@line 496 "sonic.cpp"
    var $maxRequired=(($0+64)&4294967295); //@line 496 "sonic.cpp"
    var $1=HEAP32[(($maxRequired)>>2)]; //@line 496 "sonic.cpp"
    var $2=$skip_addr; //@line 496 "sonic.cpp"
    var $div=((((($1))|0)/((($2))|0))|0); //@line 496 "sonic.cpp"
    $numSamples=$div; //@line 496 "sonic.cpp"
    var $3=$stream_addr; //@line 497 "sonic.cpp"
    var $numChannels=(($3+28)&4294967295); //@line 497 "sonic.cpp"
    var $4=HEAP32[(($numChannels)>>2)]; //@line 497 "sonic.cpp"
    var $5=$skip_addr; //@line 497 "sonic.cpp"
    var $mul=((($5)*($4))&4294967295); //@line 497 "sonic.cpp"
    $samplesPerValue=$mul; //@line 497 "sonic.cpp"
    var $6=$stream_addr; //@line 500 "sonic.cpp"
    var $downSampleBuffer=(($6+12)&4294967295); //@line 500 "sonic.cpp"
    var $7=HEAP32[(($downSampleBuffer)>>2)]; //@line 500 "sonic.cpp"
    $downSamples=$7; //@line 500 "sonic.cpp"
    $i=0; //@line 502 "sonic.cpp"
    var $8=$i; //@line 502 "sonic.cpp"
    var $9=$numSamples; //@line 502 "sonic.cpp"
    var $cmp3=((($8))|0) < ((($9))|0); //@line 502 "sonic.cpp"
    if ($cmp3) { __label__ = 1;; } else { __label__ = 4;; } //@line 502 "sonic.cpp"
    $for_body$$for_end9$2: do { 
      if (__label__ == 1) {
        $for_body$3: while(1) { 
  
          $value=0; //@line 503 "sonic.cpp"
          $j=0; //@line 504 "sonic.cpp"
          var $10=$j; //@line 504 "sonic.cpp"
          var $11=$samplesPerValue; //@line 504 "sonic.cpp"
          var $cmp21=((($10))|0) < ((($11))|0); //@line 504 "sonic.cpp"
          if ($cmp21) { __label__ = 2;; } else { __label__ = 3;; } //@line 504 "sonic.cpp"
          $for_body3$$for_end$5: do { 
            if (__label__ == 2) {
              while(1) { 
  
                var $12=$samples_addr; //@line 505 "sonic.cpp"
                var $incdec_ptr=(($12+2)&4294967295); //@line 505 "sonic.cpp"
                $samples_addr=$incdec_ptr; //@line 505 "sonic.cpp"
                var $13=HEAP16[(($12)>>1)]; //@line 505 "sonic.cpp"
                var $conv=(tempInt=(($13)),(tempInt>=32768?tempInt-65536:tempInt)); //@line 505 "sonic.cpp"
                var $14=$value; //@line 505 "sonic.cpp"
                var $add=((($14)+($conv))&4294967295); //@line 505 "sonic.cpp"
                $value=$add; //@line 505 "sonic.cpp"
                var $15=$j; //@line 504 "sonic.cpp"
                var $inc=((($15)+1)&4294967295); //@line 504 "sonic.cpp"
                $j=$inc; //@line 504 "sonic.cpp"
                var $16=$j; //@line 504 "sonic.cpp"
                var $17=$samplesPerValue; //@line 504 "sonic.cpp"
                var $cmp2=((($16))|0) < ((($17))|0); //@line 504 "sonic.cpp"
                if ($cmp2) { __label__ = 2;continue ; } else { __label__ = 3;break $for_body3$$for_end$5; } //@line 504 "sonic.cpp"
              }
            }
          } while(0);
  
          var $18=$samplesPerValue; //@line 507 "sonic.cpp"
          var $19=$value; //@line 507 "sonic.cpp"
          var $div4=((((($19))|0)/((($18))|0))|0); //@line 507 "sonic.cpp"
          $value=$div4; //@line 507 "sonic.cpp"
          var $20=$value; //@line 508 "sonic.cpp"
          var $conv5=((($20)) & 65535); //@line 508 "sonic.cpp"
          var $21=$downSamples; //@line 508 "sonic.cpp"
          var $incdec_ptr6=(($21+2)&4294967295); //@line 508 "sonic.cpp"
          $downSamples=$incdec_ptr6; //@line 508 "sonic.cpp"
          HEAP16[(($21)>>1)]=$conv5; //@line 508 "sonic.cpp"
          var $22=$i; //@line 502 "sonic.cpp"
          var $inc8=((($22)+1)&4294967295); //@line 502 "sonic.cpp"
          $i=$inc8; //@line 502 "sonic.cpp"
          var $23=$i; //@line 502 "sonic.cpp"
          var $24=$numSamples; //@line 502 "sonic.cpp"
          var $cmp=((($23))|0) < ((($24))|0); //@line 502 "sonic.cpp"
          if ($cmp) { __label__ = 1;continue $for_body$3; } else { __label__ = 4;break $for_body$$for_end9$2; } //@line 502 "sonic.cpp"
        }
      }
    } while(0);
  
    ;
    return; //@line 510 "sonic.cpp"
    return;
  }
  

  function __ZL16prevPeriodBetterP17sonicStreamStructiii($stream, $period, $minDiff, $maxDiff) {
    ;
    var __label__;
  
    var $retval;
    var $stream_addr;
    var $period_addr;
    var $minDiff_addr;
    var $maxDiff_addr;
    $stream_addr=$stream;
    $period_addr=$period;
    $minDiff_addr=$minDiff;
    $maxDiff_addr=$maxDiff;
    var $0=$maxDiff_addr; //@line 560 "sonic.cpp"
    var $mul=((($0)*3)&4294967295); //@line 560 "sonic.cpp"
    var $div=((((($mul))|0)/2)|0); //@line 560 "sonic.cpp"
    var $1=$stream_addr; //@line 560 "sonic.cpp"
    var $prevMaxDiff=(($1+80)&4294967295); //@line 560 "sonic.cpp"
    var $2=HEAP32[(($prevMaxDiff)>>2)]; //@line 560 "sonic.cpp"
    var $cmp=((($div))|0) < ((($2))|0); //@line 560 "sonic.cpp"
    if ($cmp) { __label__ = 1;; } else { __label__ = 3;; } //@line 560 "sonic.cpp"
    $land_lhs_true$$if_end$2: do { 
      if (__label__ == 1) {
  
        var $3=$maxDiff_addr; //@line 560 "sonic.cpp"
        var $conv=((($3))|0); //@line 560 "sonic.cpp"
        var $mul1=($conv)*3; //@line 560 "sonic.cpp"
        var $4=$stream_addr; //@line 560 "sonic.cpp"
        var $prevMinDiff=(($4+84)&4294967295); //@line 560 "sonic.cpp"
        var $5=HEAP32[(($prevMinDiff)>>2)]; //@line 560 "sonic.cpp"
        var $conv2=((($5))|0); //@line 560 "sonic.cpp"
        var $mul3=($mul1)*($conv2); //@line 560 "sonic.cpp"
        var $6=$stream_addr; //@line 560 "sonic.cpp"
        var $prevMaxDiff4=(($6+80)&4294967295); //@line 560 "sonic.cpp"
        var $7=HEAP32[(($prevMaxDiff4)>>2)]; //@line 560 "sonic.cpp"
        var $conv5=((($7))|0); //@line 560 "sonic.cpp"
        var $8=$minDiff_addr; //@line 560 "sonic.cpp"
        var $conv6=((($8))|0); //@line 560 "sonic.cpp"
        var $mul7=($conv5)*($conv6); //@line 560 "sonic.cpp"
        var $mul8=($mul7)*2; //@line 560 "sonic.cpp"
        var $cmp9=($mul3) < ($mul8); //@line 560 "sonic.cpp"
        if (!($cmp9)) { __label__ = 3;break $land_lhs_true$$if_end$2; } //@line 560 "sonic.cpp"
  
        $retval=1; //@line 562 "sonic.cpp"
        __label__ = 4;break $land_lhs_true$$if_end$2; //@line 562 "sonic.cpp"
      }
    } while(0);
    if (__label__ == 3) {
  
      $retval=0; //@line 564 "sonic.cpp"
      ; //@line 564 "sonic.cpp"
    }
  
    var $9=$retval; //@line 565 "sonic.cpp"
    ;
    return $9; //@line 565 "sonic.cpp"
    return null;
  }
  

  function __ZL17copyInputToOutputP17sonicStreamStructi($stream, $position) {
    ;
    var __label__;
  
    var $retval;
    var $stream_addr;
    var $position_addr;
    var $numSamples;
    $stream_addr=$stream;
    $position_addr=$position;
    var $0=$stream_addr; //@line 342 "sonic.cpp"
    var $remainingInputToCopy=(($0+68)&4294967295); //@line 342 "sonic.cpp"
    var $1=HEAP32[(($remainingInputToCopy)>>2)]; //@line 342 "sonic.cpp"
    $numSamples=$1; //@line 342 "sonic.cpp"
    var $2=$numSamples; //@line 344 "sonic.cpp"
    var $3=$stream_addr; //@line 344 "sonic.cpp"
    var $maxRequired=(($3+64)&4294967295); //@line 344 "sonic.cpp"
    var $4=HEAP32[(($maxRequired)>>2)]; //@line 344 "sonic.cpp"
    var $cmp=((($2))|0) > ((($4))|0); //@line 344 "sonic.cpp"
    if ($cmp) { __label__ = 1;; } else { __label__ = 2;; } //@line 344 "sonic.cpp"
    if (__label__ == 1) {
  
      var $5=$stream_addr; //@line 345 "sonic.cpp"
      var $maxRequired1=(($5+64)&4294967295); //@line 345 "sonic.cpp"
      var $6=HEAP32[(($maxRequired1)>>2)]; //@line 345 "sonic.cpp"
      $numSamples=$6; //@line 345 "sonic.cpp"
      ; //@line 346 "sonic.cpp"
    }
  
    var $7=$stream_addr; //@line 347 "sonic.cpp"
    var $8=$stream_addr; //@line 347 "sonic.cpp"
    var $inputBuffer=(($8)&4294967295); //@line 347 "sonic.cpp"
    var $9=HEAP32[(($inputBuffer)>>2)]; //@line 347 "sonic.cpp"
    var $10=$position_addr; //@line 347 "sonic.cpp"
    var $11=$stream_addr; //@line 347 "sonic.cpp"
    var $numChannels=(($11+28)&4294967295); //@line 347 "sonic.cpp"
    var $12=HEAP32[(($numChannels)>>2)]; //@line 347 "sonic.cpp"
    var $mul=((($12)*($10))&4294967295); //@line 347 "sonic.cpp"
    var $add_ptr=(($9+2*$mul)&4294967295); //@line 347 "sonic.cpp"
    var $13=$numSamples; //@line 347 "sonic.cpp"
    var $call=__ZL12copyToOutputP17sonicStreamStructPsi($7, $add_ptr, $13); //@line 347 "sonic.cpp"
    var $tobool=((($call))|0)!=0; //@line 347 "sonic.cpp"
    if ($tobool) { __label__ = 4;; } else { __label__ = 3;; } //@line 347 "sonic.cpp"
    if (__label__ == 4) {
  
      var $14=$numSamples; //@line 351 "sonic.cpp"
      var $15=$stream_addr; //@line 351 "sonic.cpp"
      var $remainingInputToCopy4=(($15+68)&4294967295); //@line 351 "sonic.cpp"
      var $16=HEAP32[(($remainingInputToCopy4)>>2)]; //@line 351 "sonic.cpp"
      var $sub=((($16)-($14))&4294967295); //@line 351 "sonic.cpp"
      HEAP32[(($remainingInputToCopy4)>>2)]=$sub; //@line 351 "sonic.cpp"
      var $17=$numSamples; //@line 352 "sonic.cpp"
      $retval=$17; //@line 352 "sonic.cpp"
      ; //@line 352 "sonic.cpp"
    }
    else if (__label__ == 3) {
  
      $retval=0; //@line 349 "sonic.cpp"
      ; //@line 349 "sonic.cpp"
    }
  
    var $18=$retval; //@line 353 "sonic.cpp"
    ;
    return $18; //@line 353 "sonic.cpp"
    return null;
  }
  

  function __ZL15skipPitchPeriodP17sonicStreamStructPsfi($stream, $samples, $speed, $period) {
    ;
    var __label__;
  
    var $retval;
    var $stream_addr;
    var $samples_addr;
    var $speed_addr;
    var $period_addr;
    var $newSamples;
    var $numChannels;
    $stream_addr=$stream;
    $samples_addr=$samples;
    $speed_addr=$speed;
    $period_addr=$period;
    var $0=$stream_addr; //@line 773 "sonic.cpp"
    var $numChannels1=(($0+28)&4294967295); //@line 773 "sonic.cpp"
    var $1=HEAP32[(($numChannels1)>>2)]; //@line 773 "sonic.cpp"
    $numChannels=$1; //@line 773 "sonic.cpp"
    var $2=$speed_addr; //@line 775 "sonic.cpp"
    var $cmp=($2) >= 2; //@line 775 "sonic.cpp"
    if ($cmp) { __label__ = 1;; } else { __label__ = 2;; } //@line 775 "sonic.cpp"
    $if_then$$if_else$2: do { 
      if (__label__ == 1) {
  
        var $3=$period_addr; //@line 776 "sonic.cpp"
        var $conv=((($3))|0); //@line 776 "sonic.cpp"
        var $4=$speed_addr; //@line 776 "sonic.cpp"
        var $sub=($4)-1; //@line 776 "sonic.cpp"
        var $div=($conv)/($sub); //@line 776 "sonic.cpp"
        var $conv2=((($div))|0); //@line 776 "sonic.cpp"
        $newSamples=$conv2; //@line 776 "sonic.cpp"
        ; //@line 777 "sonic.cpp"
      }
      else if (__label__ == 2) {
  
        var $5=$speed_addr; //@line 777 "sonic.cpp"
        var $cmp3=($5) > 1; //@line 777 "sonic.cpp"
        if (!($cmp3)) { __label__ = 4;break $if_then$$if_else$2; } //@line 777 "sonic.cpp"
  
        var $6=$period_addr; //@line 778 "sonic.cpp"
        $newSamples=$6; //@line 778 "sonic.cpp"
        var $7=$period_addr; //@line 779 "sonic.cpp"
        var $conv5=((($7))|0); //@line 779 "sonic.cpp"
        var $8=$speed_addr; //@line 779 "sonic.cpp"
        var $sub6=2-($8); //@line 779 "sonic.cpp"
        var $mul=($conv5)*($sub6); //@line 779 "sonic.cpp"
        var $9=$speed_addr; //@line 779 "sonic.cpp"
        var $sub7=($9)-1; //@line 779 "sonic.cpp"
        var $div8=($mul)/($sub7); //@line 779 "sonic.cpp"
        var $conv9=((($div8))|0); //@line 779 "sonic.cpp"
        var $10=$stream_addr; //@line 779 "sonic.cpp"
        var $remainingInputToCopy=(($10+68)&4294967295); //@line 779 "sonic.cpp"
        HEAP32[(($remainingInputToCopy)>>2)]=$conv9; //@line 779 "sonic.cpp"
        ; //@line 780 "sonic.cpp"
      }
    } while(0);
  
    var $11=$stream_addr; //@line 781 "sonic.cpp"
    var $12=$newSamples; //@line 781 "sonic.cpp"
    var $call=__ZL27enlargeOutputBufferIfNeededP17sonicStreamStructi($11, $12); //@line 781 "sonic.cpp"
    var $tobool=((($call))|0)!=0; //@line 781 "sonic.cpp"
    if ($tobool) { __label__ = 6;; } else { __label__ = 5;; } //@line 781 "sonic.cpp"
    if (__label__ == 6) {
  
      var $13=$newSamples; //@line 784 "sonic.cpp"
      var $14=$numChannels; //@line 784 "sonic.cpp"
      var $15=$stream_addr; //@line 784 "sonic.cpp"
      var $outputBuffer=(($15+4)&4294967295); //@line 784 "sonic.cpp"
      var $16=HEAP32[(($outputBuffer)>>2)]; //@line 784 "sonic.cpp"
      var $17=$stream_addr; //@line 784 "sonic.cpp"
      var $numOutputSamples=(($17+48)&4294967295); //@line 784 "sonic.cpp"
      var $18=HEAP32[(($numOutputSamples)>>2)]; //@line 784 "sonic.cpp"
      var $19=$numChannels; //@line 784 "sonic.cpp"
      var $mul13=((($19)*($18))&4294967295); //@line 784 "sonic.cpp"
      var $add_ptr=(($16+2*$mul13)&4294967295); //@line 784 "sonic.cpp"
      var $20=$samples_addr; //@line 784 "sonic.cpp"
      var $21=$samples_addr; //@line 784 "sonic.cpp"
      var $22=$period_addr; //@line 784 "sonic.cpp"
      var $23=$numChannels; //@line 784 "sonic.cpp"
      var $mul14=((($23)*($22))&4294967295); //@line 784 "sonic.cpp"
      var $add_ptr15=(($21+2*$mul14)&4294967295); //@line 784 "sonic.cpp"
      __ZL10overlapAddiiPsS_S_($13, $14, $add_ptr, $20, $add_ptr15); //@line 784 "sonic.cpp"
      var $24=$newSamples; //@line 786 "sonic.cpp"
      var $25=$stream_addr; //@line 786 "sonic.cpp"
      var $numOutputSamples16=(($25+48)&4294967295); //@line 786 "sonic.cpp"
      var $26=HEAP32[(($numOutputSamples16)>>2)]; //@line 786 "sonic.cpp"
      var $add=((($26)+($24))&4294967295); //@line 786 "sonic.cpp"
      HEAP32[(($numOutputSamples16)>>2)]=$add; //@line 786 "sonic.cpp"
      var $27=$newSamples; //@line 787 "sonic.cpp"
      $retval=$27; //@line 787 "sonic.cpp"
      ; //@line 787 "sonic.cpp"
    }
    else if (__label__ == 5) {
  
      $retval=0; //@line 782 "sonic.cpp"
      ; //@line 782 "sonic.cpp"
    }
  
    var $28=$retval; //@line 788 "sonic.cpp"
    ;
    return $28; //@line 788 "sonic.cpp"
    return null;
  }
  

  function __ZL17insertPitchPeriodP17sonicStreamStructPsfi($stream, $samples, $speed, $period) {
    ;
    var __label__;
  
    var $retval;
    var $stream_addr;
    var $samples_addr;
    var $speed_addr;
    var $period_addr;
    var $newSamples;
    var $out;
    var $numChannels;
    $stream_addr=$stream;
    $samples_addr=$samples;
    $speed_addr=$speed;
    $period_addr=$period;
    var $0=$stream_addr; //@line 799 "sonic.cpp"
    var $numChannels1=(($0+28)&4294967295); //@line 799 "sonic.cpp"
    var $1=HEAP32[(($numChannels1)>>2)]; //@line 799 "sonic.cpp"
    $numChannels=$1; //@line 799 "sonic.cpp"
    var $2=$speed_addr; //@line 801 "sonic.cpp"
    var $cmp=($2) < 0.5; //@line 801 "sonic.cpp"
    var $3=$period_addr; //@line 802 "sonic.cpp"
    if ($cmp) { __label__ = 1;; } else { __label__ = 2;; } //@line 801 "sonic.cpp"
    if (__label__ == 1) {
  
      var $conv=((($3))|0); //@line 802 "sonic.cpp"
      var $4=$speed_addr; //@line 802 "sonic.cpp"
      var $mul=($conv)*($4); //@line 802 "sonic.cpp"
      var $5=$speed_addr; //@line 802 "sonic.cpp"
      var $sub=1-($5); //@line 802 "sonic.cpp"
      var $div=($mul)/($sub); //@line 802 "sonic.cpp"
      var $conv2=((($div))|0); //@line 802 "sonic.cpp"
      $newSamples=$conv2; //@line 802 "sonic.cpp"
      ; //@line 803 "sonic.cpp"
    }
    else if (__label__ == 2) {
  
      $newSamples=$3; //@line 804 "sonic.cpp"
      var $6=$period_addr; //@line 805 "sonic.cpp"
      var $conv3=((($6))|0); //@line 805 "sonic.cpp"
      var $7=$speed_addr; //@line 805 "sonic.cpp"
      var $mul4=2*($7); //@line 805 "sonic.cpp"
      var $sub5=($mul4)-1; //@line 805 "sonic.cpp"
      var $mul6=($conv3)*($sub5); //@line 805 "sonic.cpp"
      var $8=$speed_addr; //@line 805 "sonic.cpp"
      var $sub7=1-($8); //@line 805 "sonic.cpp"
      var $div8=($mul6)/($sub7); //@line 805 "sonic.cpp"
      var $conv9=((($div8))|0); //@line 805 "sonic.cpp"
      var $9=$stream_addr; //@line 805 "sonic.cpp"
      var $remainingInputToCopy=(($9+68)&4294967295); //@line 805 "sonic.cpp"
      HEAP32[(($remainingInputToCopy)>>2)]=$conv9; //@line 805 "sonic.cpp"
      ;
    }
  
    var $10=$stream_addr; //@line 807 "sonic.cpp"
    var $11=$period_addr; //@line 807 "sonic.cpp"
    var $12=$newSamples; //@line 807 "sonic.cpp"
    var $add=((($12)+($11))&4294967295); //@line 807 "sonic.cpp"
    var $call=__ZL27enlargeOutputBufferIfNeededP17sonicStreamStructi($10, $add); //@line 807 "sonic.cpp"
    var $tobool=((($call))|0)!=0; //@line 807 "sonic.cpp"
    if ($tobool) { __label__ = 5;; } else { __label__ = 4;; } //@line 807 "sonic.cpp"
    if (__label__ == 5) {
  
      var $13=$stream_addr; //@line 810 "sonic.cpp"
      var $outputBuffer=(($13+4)&4294967295); //@line 810 "sonic.cpp"
      var $14=HEAP32[(($outputBuffer)>>2)]; //@line 810 "sonic.cpp"
      var $15=$stream_addr; //@line 810 "sonic.cpp"
      var $numOutputSamples=(($15+48)&4294967295); //@line 810 "sonic.cpp"
      var $16=HEAP32[(($numOutputSamples)>>2)]; //@line 810 "sonic.cpp"
      var $17=$numChannels; //@line 810 "sonic.cpp"
      var $mul12=((($17)*($16))&4294967295); //@line 810 "sonic.cpp"
      var $add_ptr=(($14+2*$mul12)&4294967295); //@line 810 "sonic.cpp"
      $out=$add_ptr; //@line 810 "sonic.cpp"
      var $18=$out; //@line 811 "sonic.cpp"
      var $19=$18; //@line 811 "sonic.cpp"
      var $20=$samples_addr; //@line 811 "sonic.cpp"
      var $21=$20; //@line 811 "sonic.cpp"
      var $22=$period_addr; //@line 811 "sonic.cpp"
      var $23=$numChannels; //@line 811 "sonic.cpp"
      var $mul13=((($22)*2)&4294967295); //@line 811 "sonic.cpp"
      var $mul14=((($mul13)*($23))&4294967295); //@line 811 "sonic.cpp"
      var $src$, $dest$, $stop$, $stop4$, $fast$;
      $src$ = $21;
      $dest$ = $19;
      $stop$ = $src$ + $mul14;
      if (($dest$%4) == ($src$%4) && $mul14 > 8) {
        while ($src$%4 !== 0 && $src$ < $stop$) {
          ; HEAP8[$dest$++] = HEAP8[$src$++];
        }
        $src$ >>= 2;
        $dest$ >>= 2;
        $stop4$ = $stop$ >> 2;
        while ($src$ < $stop4$) {
      ;;;    HEAP32[$dest$++] = HEAP32[$src$++];
        }
        $src$ <<= 2;
        $dest$ <<= 2;
      }while ($src$ < $stop$) {
        ; HEAP8[$dest$++] = HEAP8[$src$++];
      }; //@line 811 "sonic.cpp"
      var $24=$stream_addr; //@line 812 "sonic.cpp"
      var $outputBuffer15=(($24+4)&4294967295); //@line 812 "sonic.cpp"
      var $25=HEAP32[(($outputBuffer15)>>2)]; //@line 812 "sonic.cpp"
      var $26=$stream_addr; //@line 812 "sonic.cpp"
      var $numOutputSamples16=(($26+48)&4294967295); //@line 812 "sonic.cpp"
      var $27=HEAP32[(($numOutputSamples16)>>2)]; //@line 812 "sonic.cpp"
      var $28=$period_addr; //@line 812 "sonic.cpp"
      var $add17=((($28)+($27))&4294967295); //@line 812 "sonic.cpp"
      var $29=$numChannels; //@line 812 "sonic.cpp"
      var $mul18=((($add17)*($29))&4294967295); //@line 812 "sonic.cpp"
      var $add_ptr19=(($25+2*$mul18)&4294967295); //@line 812 "sonic.cpp"
      $out=$add_ptr19; //@line 812 "sonic.cpp"
      var $30=$newSamples; //@line 813 "sonic.cpp"
      var $31=$numChannels; //@line 813 "sonic.cpp"
      var $32=$out; //@line 813 "sonic.cpp"
      var $33=$samples_addr; //@line 813 "sonic.cpp"
      var $34=$period_addr; //@line 813 "sonic.cpp"
      var $35=$numChannels; //@line 813 "sonic.cpp"
      var $mul20=((($35)*($34))&4294967295); //@line 813 "sonic.cpp"
      var $add_ptr21=(($33+2*$mul20)&4294967295); //@line 813 "sonic.cpp"
      var $36=$samples_addr; //@line 813 "sonic.cpp"
      __ZL10overlapAddiiPsS_S_($30, $31, $32, $add_ptr21, $36); //@line 813 "sonic.cpp"
      var $37=$period_addr; //@line 814 "sonic.cpp"
      var $38=$newSamples; //@line 814 "sonic.cpp"
      var $39=$stream_addr; //@line 814 "sonic.cpp"
      var $numOutputSamples23=(($39+48)&4294967295); //@line 814 "sonic.cpp"
      var $40=HEAP32[(($numOutputSamples23)>>2)]; //@line 814 "sonic.cpp"
      var $add22=((($38)+($37))&4294967295); //@line 814 "sonic.cpp"
      var $add24=((($add22)+($40))&4294967295); //@line 814 "sonic.cpp"
      HEAP32[(($numOutputSamples23)>>2)]=$add24; //@line 814 "sonic.cpp"
      var $41=$newSamples; //@line 815 "sonic.cpp"
      $retval=$41; //@line 815 "sonic.cpp"
      ; //@line 815 "sonic.cpp"
    }
    else if (__label__ == 4) {
  
      $retval=0; //@line 808 "sonic.cpp"
      ; //@line 808 "sonic.cpp"
    }
  
    var $42=$retval; //@line 816 "sonic.cpp"
    ;
    return $42; //@line 816 "sonic.cpp"
    return null;
  }
  

  function __ZL18removeInputSamplesP17sonicStreamStructi($stream, $position) {
    ;
    var __label__;
  
    var $stream_addr;
    var $position_addr;
    var $remainingSamples;
    $stream_addr=$stream;
    $position_addr=$position;
    var $0=$stream_addr; //@line 312 "sonic.cpp"
    var $numInputSamples=(($0+44)&4294967295); //@line 312 "sonic.cpp"
    var $1=HEAP32[(($numInputSamples)>>2)]; //@line 312 "sonic.cpp"
    var $2=$position_addr; //@line 312 "sonic.cpp"
    var $sub=((($1)-($2))&4294967295); //@line 312 "sonic.cpp"
    $remainingSamples=$sub; //@line 312 "sonic.cpp"
    var $3=$remainingSamples; //@line 314 "sonic.cpp"
    var $cmp=((($3))|0) > 0; //@line 314 "sonic.cpp"
    if ($cmp) { __label__ = 1;; } else { __label__ = 2;; } //@line 314 "sonic.cpp"
    if (__label__ == 1) {
  
      var $4=$stream_addr; //@line 315 "sonic.cpp"
      var $inputBuffer=(($4)&4294967295); //@line 315 "sonic.cpp"
      var $5=HEAP32[(($inputBuffer)>>2)]; //@line 315 "sonic.cpp"
      var $6=$5; //@line 315 "sonic.cpp"
      var $7=$stream_addr; //@line 315 "sonic.cpp"
      var $inputBuffer1=(($7)&4294967295); //@line 315 "sonic.cpp"
      var $8=HEAP32[(($inputBuffer1)>>2)]; //@line 315 "sonic.cpp"
      var $9=$position_addr; //@line 315 "sonic.cpp"
      var $10=$stream_addr; //@line 315 "sonic.cpp"
      var $numChannels=(($10+28)&4294967295); //@line 315 "sonic.cpp"
      var $11=HEAP32[(($numChannels)>>2)]; //@line 315 "sonic.cpp"
      var $mul=((($11)*($9))&4294967295); //@line 315 "sonic.cpp"
      var $add_ptr=(($8+2*$mul)&4294967295); //@line 315 "sonic.cpp"
      var $12=$add_ptr; //@line 315 "sonic.cpp"
      var $13=$remainingSamples; //@line 315 "sonic.cpp"
      var $14=$stream_addr; //@line 315 "sonic.cpp"
      var $numChannels3=(($14+28)&4294967295); //@line 315 "sonic.cpp"
      var $15=HEAP32[(($numChannels3)>>2)]; //@line 315 "sonic.cpp"
      var $mul2=((($13)*2)&4294967295); //@line 315 "sonic.cpp"
      var $mul4=((($mul2)*($15))&4294967295); //@line 315 "sonic.cpp"
      _llvm_memmove_p0i8_p0i8_i32($6, $12, $mul4, 1, 0); //@line 315 "sonic.cpp"
      ; //@line 317 "sonic.cpp"
    }
  
    var $16=$remainingSamples; //@line 318 "sonic.cpp"
    var $17=$stream_addr; //@line 318 "sonic.cpp"
    var $numInputSamples5=(($17+44)&4294967295); //@line 318 "sonic.cpp"
    HEAP32[(($numInputSamples5)>>2)]=$16; //@line 318 "sonic.cpp"
    ;
    return; //@line 319 "sonic.cpp"
    return;
  }
  

  function _getopt($argc, $argv, $optstring) {
    var __stackBase__  = STACKTOP; STACKTOP += 20; _memset(__stackBase__, 0, 20);
    var __label__;
  
    var $argc_addr;
    var $argv_addr;
    var $optstring_addr;
    var $data=__stackBase__;
    var $r;
    $argc_addr=$argc;
    $argv_addr=$argv;
    $optstring_addr=$optstring;
    __ZL12read_globalsP11getopt_data($data); //@line 423 "getopt.cpp"
    var $0=$argc_addr; //@line 424 "getopt.cpp"
    var $1=$argv_addr; //@line 424 "getopt.cpp"
    var $2=$optstring_addr; //@line 424 "getopt.cpp"
    var $call=__ZL15getopt_internaliPKPcPKcPK6optionPiiP11getopt_data($0, $1, $2, 0, 0, 0, $data); //@line 424 "getopt.cpp"
    $r=$call; //@line 424 "getopt.cpp"
    __ZL13write_globalsP11getopt_data($data); //@line 425 "getopt.cpp"
    var $3=$r; //@line 426 "getopt.cpp"
    STACKTOP = __stackBase__;
    return $3; //@line 426 "getopt.cpp"
    return null;
  }
  

  function __ZL12read_globalsP11getopt_data($data) {
    ;
    var __label__;
  
    var $data_addr;
    $data_addr=$data;
    var $0=HEAP32[((_optarg)>>2)]; //@line 154 "getopt.cpp"
    var $1=$data_addr; //@line 154 "getopt.cpp"
    var $optarg=(($1)&4294967295); //@line 154 "getopt.cpp"
    HEAP32[(($optarg)>>2)]=$0; //@line 154 "getopt.cpp"
    var $2=HEAP32[((_optind)>>2)]; //@line 155 "getopt.cpp"
    var $3=$data_addr; //@line 155 "getopt.cpp"
    var $optind=(($3+4)&4294967295); //@line 155 "getopt.cpp"
    HEAP32[(($optind)>>2)]=$2; //@line 155 "getopt.cpp"
    var $4=HEAP32[((_opterr)>>2)]; //@line 156 "getopt.cpp"
    var $5=$data_addr; //@line 156 "getopt.cpp"
    var $opterr=(($5+8)&4294967295); //@line 156 "getopt.cpp"
    HEAP32[(($opterr)>>2)]=$4; //@line 156 "getopt.cpp"
    var $6=HEAP32[((_optopt)>>2)]; //@line 157 "getopt.cpp"
    var $7=$data_addr; //@line 157 "getopt.cpp"
    var $optopt=(($7+12)&4294967295); //@line 157 "getopt.cpp"
    HEAP32[(($optopt)>>2)]=$6; //@line 157 "getopt.cpp"
    var $8=HEAP32[((__ZL8optwhere)>>2)]; //@line 158 "getopt.cpp"
    var $9=$data_addr; //@line 158 "getopt.cpp"
    var $optwhere=(($9+16)&4294967295); //@line 158 "getopt.cpp"
    HEAP32[(($optwhere)>>2)]=$8; //@line 158 "getopt.cpp"
    ;
    return; //@line 159 "getopt.cpp"
    return;
  }
  

  function __ZL15getopt_internaliPKPcPKcPK6optionPiiP11getopt_data($argc, $argv, $shortopts, $longopts, $longind, $only, $data) {
    ;
    var __label__;
    var __lastLabel__ = null;
  
    var $retval;
    var $argc_addr;
    var $argv_addr;
    var $shortopts_addr;
    var $longopts_addr;
    var $longind_addr;
    var $only_addr;
    var $data_addr;
    var $ordering;
    var $permute_from;
    var $num_nonopts;
    var $optindex;
    var $match_chars;
    var $possible_arg;
    var $longopt_match;
    var $has_arg;
    var $cp;
    var $arg_next;
    $argc_addr=$argc;
    $argv_addr=$argv;
    $shortopts_addr=$shortopts;
    $longopts_addr=$longopts;
    $longind_addr=$longind;
    $only_addr=$only;
    $data_addr=$data;
    $ordering=0; //@line 179 "getopt.cpp"
    $permute_from=0; //@line 180 "getopt.cpp"
    $num_nonopts=0; //@line 181 "getopt.cpp"
    $optindex=0; //@line 182 "getopt.cpp"
    $match_chars=0; //@line 183 "getopt.cpp"
    $possible_arg=0; //@line 184 "getopt.cpp"
    $longopt_match=-1; //@line 185 "getopt.cpp"
    $has_arg=-1; //@line 186 "getopt.cpp"
    $cp=0; //@line 187 "getopt.cpp"
    $arg_next=0; //@line 188 "getopt.cpp"
    var $0=$argc_addr; //@line 191 "getopt.cpp"
    var $cmp=((($0))|0)==0; //@line 191 "getopt.cpp"
    if ($cmp) { __label__ = 6;; } else { __label__ = 1;; } //@line 191 "getopt.cpp"
    $if_then$$lor_lhs_false$2: do { 
      if (__label__ == 1) {
  
        var $1=$argv_addr; //@line 191 "getopt.cpp"
        var $cmp1=((($1))|0)==0; //@line 191 "getopt.cpp"
        if ($cmp1) { __label__ = 6;break $if_then$$lor_lhs_false$2; } //@line 191 "getopt.cpp"
  
        var $2=$shortopts_addr; //@line 191 "getopt.cpp"
        var $cmp3=((($2))|0)==0; //@line 191 "getopt.cpp"
        if ($cmp3) { __label__ = 3;; } else { __label__ = 4;; } //@line 191 "getopt.cpp"
        if (__label__ == 3) {
  
          var $3=$longopts_addr; //@line 191 "getopt.cpp"
          var $cmp4=((($3))|0)==0; //@line 191 "getopt.cpp"
          if ($cmp4) { __label__ = 6;break $if_then$$lor_lhs_false$2; } //@line 191 "getopt.cpp"
        }
  
        var $4=$data_addr; //@line 191 "getopt.cpp"
        var $optind=(($4+4)&4294967295); //@line 191 "getopt.cpp"
        var $5=HEAP32[(($optind)>>2)]; //@line 191 "getopt.cpp"
        var $6=$argc_addr; //@line 191 "getopt.cpp"
        var $cmp6=((($5))|0) >= ((($6))|0); //@line 191 "getopt.cpp"
        if ($cmp6) { __label__ = 6;break $if_then$$lor_lhs_false$2; } //@line 191 "getopt.cpp"
  
        var $7=$data_addr; //@line 191 "getopt.cpp"
        var $optind8=(($7+4)&4294967295); //@line 191 "getopt.cpp"
        var $8=HEAP32[(($optind8)>>2)]; //@line 191 "getopt.cpp"
        var $9=$argv_addr; //@line 191 "getopt.cpp"
        var $arrayidx=(($9+4*$8)&4294967295); //@line 191 "getopt.cpp"
        var $10=HEAP32[(($arrayidx)>>2)]; //@line 191 "getopt.cpp"
        var $cmp9=((($10))|0)==0; //@line 191 "getopt.cpp"
        if ($cmp9) { __label__ = 6;break $if_then$$lor_lhs_false$2; } //@line 191 "getopt.cpp"
  
        var $11=$data_addr; //@line 194 "getopt.cpp"
        var $optind10=(($11+4)&4294967295); //@line 194 "getopt.cpp"
        var $12=HEAP32[(($optind10)>>2)]; //@line 194 "getopt.cpp"
        var $13=$argv_addr; //@line 194 "getopt.cpp"
        var $arrayidx11=(($13+4*$12)&4294967295); //@line 194 "getopt.cpp"
        var $14=HEAP32[(($arrayidx11)>>2)]; //@line 194 "getopt.cpp"
        var $call=_strcmp($14, ((__str831)&4294967295)); //@line 194 "getopt.cpp"
        var $cmp12=((($call))|0)==0; //@line 194 "getopt.cpp"
        var $15=$data_addr; //@line 196 "getopt.cpp"
        var $optind14=(($15+4)&4294967295); //@line 196 "getopt.cpp"
        var $16=HEAP32[(($optind14)>>2)]; //@line 196 "getopt.cpp"
        if ($cmp12) { __label__ = 8;; } else { __label__ = 9;; } //@line 194 "getopt.cpp"
        if (__label__ == 8) {
  
          var $inc=((($16)+1)&4294967295); //@line 196 "getopt.cpp"
          HEAP32[(($optind14)>>2)]=$inc; //@line 196 "getopt.cpp"
          $retval=-1; //@line 197 "getopt.cpp"
          __label__ = 96;break $if_then$$lor_lhs_false$2; //@line 197 "getopt.cpp"
        }
        else if (__label__ == 9) {
  
          var $cmp17=((($16))|0)==0; //@line 201 "getopt.cpp"
          if ($cmp17) { __label__ = 10;; } else { __label__ = 11;; } //@line 201 "getopt.cpp"
          if (__label__ == 10) {
  
            var $17=$data_addr; //@line 202 "getopt.cpp"
            var $optwhere=(($17+16)&4294967295); //@line 202 "getopt.cpp"
            HEAP32[(($optwhere)>>2)]=1; //@line 202 "getopt.cpp"
            var $18=$data_addr; //@line 202 "getopt.cpp"
            var $optind19=(($18+4)&4294967295); //@line 202 "getopt.cpp"
            HEAP32[(($optind19)>>2)]=1; //@line 202 "getopt.cpp"
            ; //@line 202 "getopt.cpp"
          }
  
          var $19=$shortopts_addr; //@line 205 "getopt.cpp"
          var $cmp21=((($19))|0)!=0; //@line 205 "getopt.cpp"
          if ($cmp21) { __label__ = 12;; } else { __label__ = 15;; } //@line 205 "getopt.cpp"
          $land_lhs_true22$$if_else$16: do { 
            if (__label__ == 12) {
  
              var $20=$shortopts_addr; //@line 205 "getopt.cpp"
              var $21=HEAP8[($20)]; //@line 205 "getopt.cpp"
              var $conv=(tempInt=(($21)),(tempInt>=128?tempInt-256:tempInt)); //@line 205 "getopt.cpp"
              var $cmp23=((($conv))|0)==45; //@line 205 "getopt.cpp"
              if ($cmp23) { __label__ = 14;; } else { __label__ = 13;; } //@line 205 "getopt.cpp"
              if (__label__ == 13) {
  
                var $22=$shortopts_addr; //@line 205 "getopt.cpp"
                var $23=HEAP8[($22)]; //@line 205 "getopt.cpp"
                var $conv25=(tempInt=(($23)),(tempInt>=128?tempInt-256:tempInt)); //@line 205 "getopt.cpp"
                var $cmp26=((($conv25))|0)==43; //@line 205 "getopt.cpp"
                if (!($cmp26)) { __label__ = 15;break $land_lhs_true22$$if_else$16; } //@line 205 "getopt.cpp"
              }
  
              var $24=$shortopts_addr; //@line 207 "getopt.cpp"
              var $25=HEAP8[($24)]; //@line 207 "getopt.cpp"
              var $conv28=(tempInt=(($25)),(tempInt>=128?tempInt-256:tempInt)); //@line 207 "getopt.cpp"
              var $cmp29=((($conv28))|0)==45; //@line 207 "getopt.cpp"
              var $cond=($cmp29) ? 1 : 2; //@line 207 "getopt.cpp"
              $ordering=$cond; //@line 207 "getopt.cpp"
              var $26=$shortopts_addr; //@line 208 "getopt.cpp"
              var $incdec_ptr=(($26+1)&4294967295); //@line 208 "getopt.cpp"
              $shortopts_addr=$incdec_ptr; //@line 208 "getopt.cpp"
              __label__ = 16;break $land_lhs_true22$$if_else$16; //@line 209 "getopt.cpp"
            }
          } while(0);
          if (__label__ == 15) {
  
            var $call30=_getenv(((__str1832)&4294967295)); //@line 211 "getopt.cpp"
            var $cmp31=((($call30))|0)!=0; //@line 211 "getopt.cpp"
            var $cond32=($cmp31) ? 2 : 0; //@line 211 "getopt.cpp"
            $ordering=$cond32; //@line 211 "getopt.cpp"
            ;
          }
  
          var $27=$data_addr; //@line 217 "getopt.cpp"
          var $optwhere34=(($27+16)&4294967295); //@line 217 "getopt.cpp"
          var $28=HEAP32[(($optwhere34)>>2)]; //@line 217 "getopt.cpp"
          var $cmp35=((($28))|0)==1; //@line 217 "getopt.cpp"
          if ($cmp35) { __label__ = 17;; } else { __label__ = 28;; } //@line 217 "getopt.cpp"
          $if_then36$$if_end75$24: do { 
            if (__label__ == 17) {
  
              var $29=$ordering; //@line 219 "getopt.cpp"
              if ($29 == 2) {
                __label__ = 26;;
              }
              else if ($29 == 1) {
                __label__ = 24;;
              }
              else {
              __label__ = 18;;
              }
              
              if (__label__ == 18) {
  
                var $30=$data_addr; //@line 223 "getopt.cpp"
                var $optind37=(($30+4)&4294967295); //@line 223 "getopt.cpp"
                var $31=HEAP32[(($optind37)>>2)]; //@line 223 "getopt.cpp"
                $permute_from=$31; //@line 223 "getopt.cpp"
                $num_nonopts=0; //@line 224 "getopt.cpp"
                var $32=$data_addr; //@line 225 "getopt.cpp"
                var $optind381=(($32+4)&4294967295); //@line 225 "getopt.cpp"
                var $33=HEAP32[(($optind381)>>2)]; //@line 225 "getopt.cpp"
                var $34=$argv_addr; //@line 225 "getopt.cpp"
                var $arrayidx392=(($34+4*$33)&4294967295); //@line 225 "getopt.cpp"
                var $35=HEAP32[(($arrayidx392)>>2)]; //@line 225 "getopt.cpp"
                var $36=$only_addr; //@line 225 "getopt.cpp"
                var $call403=__ZL9is_optionPci($35, $36); //@line 225 "getopt.cpp"
                var $tobool4=((($call403))|0)!=0; //@line 225 "getopt.cpp"
                var $lnot5=($tobool4) ^ 1; //@line 225 "getopt.cpp"
                var $37=$data_addr; //@line 227 "getopt.cpp"
                var $optind416=(($37+4)&4294967295); //@line 227 "getopt.cpp"
                var $38=HEAP32[(($optind416)>>2)]; //@line 227 "getopt.cpp"
                if ($lnot5) { __lastLabel__ = 18; __label__ = 19;; } else { __lastLabel__ = 18; __label__ = 20;; } //@line 225 "getopt.cpp"
                $while_body$$while_end$28: do { 
                  if (__label__ == 19) {
                    while(1) { 
  
                      var $39=__lastLabel__ == 19 ? $47 : ($38);
                      var $optind417=__lastLabel__ == 19 ? $optind41 : ($optind416);
                      var $inc42=((($39)+1)&4294967295); //@line 227 "getopt.cpp"
                      HEAP32[(($optind417)>>2)]=$inc42; //@line 227 "getopt.cpp"
                      var $40=$num_nonopts; //@line 228 "getopt.cpp"
                      var $inc43=((($40)+1)&4294967295); //@line 228 "getopt.cpp"
                      $num_nonopts=$inc43; //@line 228 "getopt.cpp"
                      var $41=$data_addr; //@line 225 "getopt.cpp"
                      var $optind38=(($41+4)&4294967295); //@line 225 "getopt.cpp"
                      var $42=HEAP32[(($optind38)>>2)]; //@line 225 "getopt.cpp"
                      var $43=$argv_addr; //@line 225 "getopt.cpp"
                      var $arrayidx39=(($43+4*$42)&4294967295); //@line 225 "getopt.cpp"
                      var $44=HEAP32[(($arrayidx39)>>2)]; //@line 225 "getopt.cpp"
                      var $45=$only_addr; //@line 225 "getopt.cpp"
                      var $call40=__ZL9is_optionPci($44, $45); //@line 225 "getopt.cpp"
                      var $tobool=((($call40))|0)!=0; //@line 225 "getopt.cpp"
                      var $lnot=($tobool) ^ 1; //@line 225 "getopt.cpp"
                      var $46=$data_addr; //@line 227 "getopt.cpp"
                      var $optind41=(($46+4)&4294967295); //@line 227 "getopt.cpp"
                      var $47=HEAP32[(($optind41)>>2)]; //@line 227 "getopt.cpp"
                      if ($lnot) { __lastLabel__ = 19; __label__ = 19;continue ; } else { __lastLabel__ = 19; __label__ = 20;break $while_body$$while_end$28; } //@line 225 "getopt.cpp"
                    }
                  }
                } while(0);
  
                var $_lcssa=__lastLabel__ == 18 ? $38 : ($47);
                var $48=$argv_addr; //@line 230 "getopt.cpp"
                var $arrayidx45=(($48+4*$_lcssa)&4294967295); //@line 230 "getopt.cpp"
                var $49=HEAP32[(($arrayidx45)>>2)]; //@line 230 "getopt.cpp"
                var $cmp46=((($49))|0)==0; //@line 230 "getopt.cpp"
                if ($cmp46) { __label__ = 21;; } else { __label__ = 22;; } //@line 230 "getopt.cpp"
                if (__label__ == 21) {
  
                  var $50=$permute_from; //@line 233 "getopt.cpp"
                  var $51=$data_addr; //@line 233 "getopt.cpp"
                  var $optind48=(($51+4)&4294967295); //@line 233 "getopt.cpp"
                  HEAP32[(($optind48)>>2)]=$50; //@line 233 "getopt.cpp"
                  $retval=-1; //@line 234 "getopt.cpp"
                  __label__ = 96;break $if_then$$lor_lhs_false$2; //@line 234 "getopt.cpp"
                }
                else if (__label__ == 22) {
  
                  var $52=$data_addr; //@line 236 "getopt.cpp"
                  var $optind50=(($52+4)&4294967295); //@line 236 "getopt.cpp"
                  var $53=HEAP32[(($optind50)>>2)]; //@line 236 "getopt.cpp"
                  var $54=$argv_addr; //@line 236 "getopt.cpp"
                  var $arrayidx51=(($54+4*$53)&4294967295); //@line 236 "getopt.cpp"
                  var $55=HEAP32[(($arrayidx51)>>2)]; //@line 236 "getopt.cpp"
                  var $call52=_strcmp($55, ((__str831)&4294967295)); //@line 236 "getopt.cpp"
                  var $cmp53=((($call52))|0)==0; //@line 236 "getopt.cpp"
                  if (!($cmp53)) { __label__ = 28;break $if_then36$$if_end75$24; } //@line 236 "getopt.cpp"
  
                  var $56=$argv_addr; //@line 239 "getopt.cpp"
                  var $57=$permute_from; //@line 239 "getopt.cpp"
                  var $add_ptr=(($56+4*$57)&4294967295); //@line 239 "getopt.cpp"
                  var $58=$num_nonopts; //@line 239 "getopt.cpp"
                  __ZL7permutePKPcii($add_ptr, $58, 1); //@line 239 "getopt.cpp"
                  var $59=$permute_from; //@line 240 "getopt.cpp"
                  var $add=((($59)+1)&4294967295); //@line 240 "getopt.cpp"
                  var $60=$data_addr; //@line 240 "getopt.cpp"
                  var $optind55=(($60+4)&4294967295); //@line 240 "getopt.cpp"
                  HEAP32[(($optind55)>>2)]=$add; //@line 240 "getopt.cpp"
                  $retval=-1; //@line 241 "getopt.cpp"
                  __label__ = 96;break $if_then$$lor_lhs_false$2; //@line 241 "getopt.cpp"
                }
              }
              else if (__label__ == 26) {
  
                var $72=$data_addr; //@line 252 "getopt.cpp"
                var $optind69=(($72+4)&4294967295); //@line 252 "getopt.cpp"
                var $73=HEAP32[(($optind69)>>2)]; //@line 252 "getopt.cpp"
                var $74=$argv_addr; //@line 252 "getopt.cpp"
                var $arrayidx70=(($74+4*$73)&4294967295); //@line 252 "getopt.cpp"
                var $75=HEAP32[(($arrayidx70)>>2)]; //@line 252 "getopt.cpp"
                var $76=$only_addr; //@line 252 "getopt.cpp"
                var $call71=__ZL9is_optionPci($75, $76); //@line 252 "getopt.cpp"
                var $tobool72=((($call71))|0)!=0; //@line 252 "getopt.cpp"
                if ($tobool72) { __label__ = 28;break $if_then36$$if_end75$24; } //@line 252 "getopt.cpp"
  
                $retval=-1; //@line 253 "getopt.cpp"
                __label__ = 96;break $if_then$$lor_lhs_false$2; //@line 253 "getopt.cpp"
              }
              else if (__label__ == 24) {
  
                var $61=$data_addr; //@line 245 "getopt.cpp"
                var $optind59=(($61+4)&4294967295); //@line 245 "getopt.cpp"
                var $62=HEAP32[(($optind59)>>2)]; //@line 245 "getopt.cpp"
                var $63=$argv_addr; //@line 245 "getopt.cpp"
                var $arrayidx60=(($63+4*$62)&4294967295); //@line 245 "getopt.cpp"
                var $64=HEAP32[(($arrayidx60)>>2)]; //@line 245 "getopt.cpp"
                var $65=$only_addr; //@line 245 "getopt.cpp"
                var $call61=__ZL9is_optionPci($64, $65); //@line 245 "getopt.cpp"
                var $tobool62=((($call61))|0)!=0; //@line 245 "getopt.cpp"
                if ($tobool62) { __label__ = 28;break $if_then36$$if_end75$24; } //@line 245 "getopt.cpp"
  
                var $66=$data_addr; //@line 247 "getopt.cpp"
                var $optind64=(($66+4)&4294967295); //@line 247 "getopt.cpp"
                var $67=HEAP32[(($optind64)>>2)]; //@line 247 "getopt.cpp"
                var $inc65=((($67)+1)&4294967295); //@line 247 "getopt.cpp"
                HEAP32[(($optind64)>>2)]=$inc65; //@line 247 "getopt.cpp"
                var $68=$argv_addr; //@line 247 "getopt.cpp"
                var $arrayidx66=(($68+4*$67)&4294967295); //@line 247 "getopt.cpp"
                var $69=HEAP32[(($arrayidx66)>>2)]; //@line 247 "getopt.cpp"
                var $70=$data_addr; //@line 247 "getopt.cpp"
                var $optarg=(($70)&4294967295); //@line 247 "getopt.cpp"
                HEAP32[(($optarg)>>2)]=$69; //@line 247 "getopt.cpp"
                var $71=$data_addr; //@line 248 "getopt.cpp"
                var $optopt=(($71+12)&4294967295); //@line 248 "getopt.cpp"
                HEAP32[(($optopt)>>2)]=1; //@line 248 "getopt.cpp"
                $retval=1; //@line 248 "getopt.cpp"
                __label__ = 96;break $if_then$$lor_lhs_false$2; //@line 248 "getopt.cpp"
              }
            }
          } while(0);
  
          var $77=$longopts_addr; //@line 260 "getopt.cpp"
          var $cmp76=((($77))|0)!=0; //@line 260 "getopt.cpp"
          if ($cmp76) { __label__ = 29;; } else { __label__ = 52;; } //@line 260 "getopt.cpp"
          $land_lhs_true77$$if_end162$41: do { 
            if (__label__ == 29) {
  
              var $78=$data_addr; //@line 261 "getopt.cpp"
              var $optind78=(($78+4)&4294967295); //@line 261 "getopt.cpp"
              var $79=HEAP32[(($optind78)>>2)]; //@line 261 "getopt.cpp"
              var $80=$argv_addr; //@line 261 "getopt.cpp"
              var $arrayidx79=(($80+4*$79)&4294967295); //@line 261 "getopt.cpp"
              var $81=HEAP32[(($arrayidx79)>>2)]; //@line 261 "getopt.cpp"
              var $call80=_memcmp($81, ((__str831)&4294967295), 2); //@line 261 "getopt.cpp"
              var $cmp81=((($call80))|0)==0; //@line 261 "getopt.cpp"
              if ($cmp81) { __label__ = 32;; } else { __label__ = 30;; } //@line 261 "getopt.cpp"
              if (__label__ == 30) {
  
                var $82=$only_addr; //@line 261 "getopt.cpp"
                var $tobool83=((($82))|0)!=0; //@line 261 "getopt.cpp"
                if (!($tobool83)) { __label__ = 52;break $land_lhs_true77$$if_end162$41; } //@line 261 "getopt.cpp"
  
                var $83=$data_addr; //@line 261 "getopt.cpp"
                var $optind85=(($83+4)&4294967295); //@line 261 "getopt.cpp"
                var $84=HEAP32[(($optind85)>>2)]; //@line 261 "getopt.cpp"
                var $85=$argv_addr; //@line 261 "getopt.cpp"
                var $arrayidx86=(($85+4*$84)&4294967295); //@line 261 "getopt.cpp"
                var $86=HEAP32[(($arrayidx86)>>2)]; //@line 261 "getopt.cpp"
                var $arrayidx87=(($86)&4294967295); //@line 261 "getopt.cpp"
                var $87=HEAP8[($arrayidx87)]; //@line 261 "getopt.cpp"
                var $conv88=(tempInt=(($87)),(tempInt>=128?tempInt-256:tempInt)); //@line 261 "getopt.cpp"
                var $cmp89=((($conv88))|0)==43; //@line 261 "getopt.cpp"
                if (!($cmp89)) { __label__ = 52;break $land_lhs_true77$$if_end162$41; } //@line 261 "getopt.cpp"
              }
  
              var $88=$data_addr; //@line 261 "getopt.cpp"
              var $optwhere91=(($88+16)&4294967295); //@line 261 "getopt.cpp"
              var $89=HEAP32[(($optwhere91)>>2)]; //@line 261 "getopt.cpp"
              var $cmp92=((($89))|0)==1; //@line 261 "getopt.cpp"
              if (!($cmp92)) { __label__ = 52;break $land_lhs_true77$$if_end162$41; } //@line 261 "getopt.cpp"
  
              var $90=$data_addr; //@line 265 "getopt.cpp"
              var $optind94=(($90+4)&4294967295); //@line 265 "getopt.cpp"
              var $91=HEAP32[(($optind94)>>2)]; //@line 265 "getopt.cpp"
              var $92=$argv_addr; //@line 265 "getopt.cpp"
              var $arrayidx95=(($92+4*$91)&4294967295); //@line 265 "getopt.cpp"
              var $93=HEAP32[(($arrayidx95)>>2)]; //@line 265 "getopt.cpp"
              var $call96=_memcmp($93, ((__str831)&4294967295), 2); //@line 265 "getopt.cpp"
              var $cmp97=((($call96))|0)==0; //@line 265 "getopt.cpp"
              if ($cmp97) { __label__ = 34;; } else { __label__ = 35;; } //@line 265 "getopt.cpp"
              if (__label__ == 34) {
  
                var $94=$data_addr; //@line 266 "getopt.cpp"
                var $optwhere99=(($94+16)&4294967295); //@line 266 "getopt.cpp"
                HEAP32[(($optwhere99)>>2)]=2; //@line 266 "getopt.cpp"
                ; //@line 266 "getopt.cpp"
              }
  
              $longopt_match=-1; //@line 267 "getopt.cpp"
              var $95=$data_addr; //@line 268 "getopt.cpp"
              var $optind101=(($95+4)&4294967295); //@line 268 "getopt.cpp"
              var $96=HEAP32[(($optind101)>>2)]; //@line 268 "getopt.cpp"
              var $97=$argv_addr; //@line 268 "getopt.cpp"
              var $arrayidx102=(($97+4*$96)&4294967295); //@line 268 "getopt.cpp"
              var $98=HEAP32[(($arrayidx102)>>2)]; //@line 268 "getopt.cpp"
              var $99=$data_addr; //@line 268 "getopt.cpp"
              var $optwhere103=(($99+16)&4294967295); //@line 268 "getopt.cpp"
              var $100=HEAP32[(($optwhere103)>>2)]; //@line 268 "getopt.cpp"
              var $add_ptr104=(($98+$100)&4294967295); //@line 268 "getopt.cpp"
              var $call105=_strchr($add_ptr104, 61); //@line 268 "getopt.cpp"
              $possible_arg=$call105; //@line 268 "getopt.cpp"
              var $cmp106=((($call105))|0)==0; //@line 269 "getopt.cpp"
              if ($cmp106) { __label__ = 36;; } else { __label__ = 37;; } //@line 269 "getopt.cpp"
              if (__label__ == 36) {
  
                var $101=$data_addr; //@line 272 "getopt.cpp"
                var $optind108=(($101+4)&4294967295); //@line 272 "getopt.cpp"
                var $102=HEAP32[(($optind108)>>2)]; //@line 272 "getopt.cpp"
                var $103=$argv_addr; //@line 272 "getopt.cpp"
                var $arrayidx109=(($103+4*$102)&4294967295); //@line 272 "getopt.cpp"
                var $104=HEAP32[(($arrayidx109)>>2)]; //@line 272 "getopt.cpp"
                var $call110=_strlen($104); //@line 272 "getopt.cpp"
                $match_chars=$call110; //@line 272 "getopt.cpp"
                var $105=$data_addr; //@line 273 "getopt.cpp"
                var $optind111=(($105+4)&4294967295); //@line 273 "getopt.cpp"
                var $106=HEAP32[(($optind111)>>2)]; //@line 273 "getopt.cpp"
                var $107=$argv_addr; //@line 273 "getopt.cpp"
                var $arrayidx112=(($107+4*$106)&4294967295); //@line 273 "getopt.cpp"
                var $108=HEAP32[(($arrayidx112)>>2)]; //@line 273 "getopt.cpp"
                var $109=$match_chars; //@line 273 "getopt.cpp"
                var $add_ptr113=(($108+$109)&4294967295); //@line 273 "getopt.cpp"
                $possible_arg=$add_ptr113; //@line 273 "getopt.cpp"
                var $110=$match_chars; //@line 274 "getopt.cpp"
                var $111=$data_addr; //@line 274 "getopt.cpp"
                var $optwhere114=(($111+16)&4294967295); //@line 274 "getopt.cpp"
                var $112=HEAP32[(($optwhere114)>>2)]; //@line 274 "getopt.cpp"
                var $sub=((($110)-($112))&4294967295); //@line 274 "getopt.cpp"
                $match_chars=$sub; //@line 274 "getopt.cpp"
                ; //@line 275 "getopt.cpp"
              }
              else if (__label__ == 37) {
  
                var $113=$possible_arg; //@line 277 "getopt.cpp"
                var $114=$data_addr; //@line 277 "getopt.cpp"
                var $optind116=(($114+4)&4294967295); //@line 277 "getopt.cpp"
                var $115=HEAP32[(($optind116)>>2)]; //@line 277 "getopt.cpp"
                var $116=$argv_addr; //@line 277 "getopt.cpp"
                var $arrayidx117=(($116+4*$115)&4294967295); //@line 277 "getopt.cpp"
                var $117=HEAP32[(($arrayidx117)>>2)]; //@line 277 "getopt.cpp"
                var $sub_ptr_lhs_cast=($113); //@line 277 "getopt.cpp"
                var $sub_ptr_rhs_cast=($117); //@line 277 "getopt.cpp"
                var $sub_ptr_rhs_cast_neg=(((-($sub_ptr_rhs_cast)))&4294967295);
                var $118=$data_addr; //@line 277 "getopt.cpp"
                var $optwhere118=(($118+16)&4294967295); //@line 277 "getopt.cpp"
                var $119=HEAP32[(($optwhere118)>>2)]; //@line 277 "getopt.cpp"
                var $_neg=(((-($119)))&4294967295);
                var $sub_ptr_sub=((($sub_ptr_rhs_cast_neg)+($sub_ptr_lhs_cast))&4294967295); //@line 277 "getopt.cpp"
                var $sub119=((($sub_ptr_sub)+($_neg))&4294967295); //@line 277 "getopt.cpp"
                $match_chars=$sub119; //@line 277 "getopt.cpp"
                ;
              }
  
              $optindex=0; //@line 278 "getopt.cpp"
              ; //@line 278 "getopt.cpp"
              while(1) { 
  
                var $120=$optindex; //@line 278 "getopt.cpp"
                var $121=$longopts_addr; //@line 278 "getopt.cpp"
                var $arrayidx121=(($121+16*$120)&4294967295); //@line 278 "getopt.cpp"
                var $name=(($arrayidx121)&4294967295); //@line 278 "getopt.cpp"
                var $122=HEAP32[(($name)>>2)]; //@line 278 "getopt.cpp"
                var $cmp122=((($122))|0)!=0; //@line 278 "getopt.cpp"
                if (!($cmp122)) { __label__ = 49;break ; } //@line 278 "getopt.cpp"
  
                var $123=$data_addr; //@line 280 "getopt.cpp"
                var $optind123=(($123+4)&4294967295); //@line 280 "getopt.cpp"
                var $124=HEAP32[(($optind123)>>2)]; //@line 280 "getopt.cpp"
                var $125=$argv_addr; //@line 280 "getopt.cpp"
                var $arrayidx124=(($125+4*$124)&4294967295); //@line 280 "getopt.cpp"
                var $126=HEAP32[(($arrayidx124)>>2)]; //@line 280 "getopt.cpp"
                var $127=$data_addr; //@line 280 "getopt.cpp"
                var $optwhere125=(($127+16)&4294967295); //@line 280 "getopt.cpp"
                var $128=HEAP32[(($optwhere125)>>2)]; //@line 280 "getopt.cpp"
                var $add_ptr126=(($126+$128)&4294967295); //@line 280 "getopt.cpp"
                var $129=$optindex; //@line 280 "getopt.cpp"
                var $130=$longopts_addr; //@line 280 "getopt.cpp"
                var $arrayidx127=(($130+16*$129)&4294967295); //@line 280 "getopt.cpp"
                var $name128=(($arrayidx127)&4294967295); //@line 280 "getopt.cpp"
                var $131=HEAP32[(($name128)>>2)]; //@line 280 "getopt.cpp"
                var $132=$match_chars; //@line 280 "getopt.cpp"
                var $call129=_memcmp($add_ptr126, $131, $132); //@line 280 "getopt.cpp"
                var $cmp130=((($call129))|0)==0; //@line 280 "getopt.cpp"
                if ($cmp130) { __label__ = 41;; } else { __label__ = 48;; } //@line 280 "getopt.cpp"
                if (__label__ == 41) {
  
                  var $133=$match_chars; //@line 285 "getopt.cpp"
                  var $134=$optindex; //@line 285 "getopt.cpp"
                  var $135=$longopts_addr; //@line 285 "getopt.cpp"
                  var $arrayidx132=(($135+16*$134)&4294967295); //@line 285 "getopt.cpp"
                  var $name133=(($arrayidx132)&4294967295); //@line 285 "getopt.cpp"
                  var $136=HEAP32[(($name133)>>2)]; //@line 285 "getopt.cpp"
                  var $call134=_strlen($136); //@line 285 "getopt.cpp"
                  var $cmp135=((($133))|0)==((($call134))|0); //@line 285 "getopt.cpp"
                  if ($cmp135) { __label__ = 42;break ; } //@line 285 "getopt.cpp"
  
                  var $138=$longopt_match; //@line 293 "getopt.cpp"
                  var $cmp138=((($138))|0) < 0; //@line 293 "getopt.cpp"
                  if (!($cmp138)) { __label__ = 45;break ; } //@line 293 "getopt.cpp"
  
                  var $139=$optindex; //@line 294 "getopt.cpp"
                  $longopt_match=$139; //@line 294 "getopt.cpp"
                  ; //@line 308 "getopt.cpp"
                }
  
                var $157=$optindex; //@line 278 "getopt.cpp"
                var $inc156=((($157)+1)&4294967295); //@line 278 "getopt.cpp"
                $optindex=$inc156; //@line 278 "getopt.cpp"
                __label__ = 39;continue ; //@line 278 "getopt.cpp"
              }
              if (__label__ == 49) {
  
                var $_pr=$longopt_match; //@line 310 "getopt.cpp"
                __lastLabel__ = 49; ;
              }
              else if (__label__ == 42) {
  
                var $137=$optindex; //@line 287 "getopt.cpp"
                $longopt_match=$137; //@line 287 "getopt.cpp"
                __lastLabel__ = 42; ; //@line 288 "getopt.cpp"
              }
              else if (__label__ == 45) {
  
                var $140=$data_addr; //@line 298 "getopt.cpp"
                var $opterr=(($140+8)&4294967295); //@line 298 "getopt.cpp"
                var $141=HEAP32[(($opterr)>>2)]; //@line 298 "getopt.cpp"
                var $tobool141=((($141))|0)!=0; //@line 298 "getopt.cpp"
                if ($tobool141) { __label__ = 46;; } else { __label__ = 47;; } //@line 298 "getopt.cpp"
                if (__label__ == 46) {
  
                  var $142=HEAP32[((__impure_ptr)>>2)]; //@line 299 "getopt.cpp"
                  var $_stderr=(($142+12)&4294967295); //@line 299 "getopt.cpp"
                  var $143=HEAP32[(($_stderr)>>2)]; //@line 299 "getopt.cpp"
                  var $144=$argv_addr; //@line 299 "getopt.cpp"
                  var $arrayidx143=(($144)&4294967295); //@line 299 "getopt.cpp"
                  var $145=HEAP32[(($arrayidx143)>>2)]; //@line 299 "getopt.cpp"
                  var $146=$data_addr; //@line 299 "getopt.cpp"
                  var $optind144=(($146+4)&4294967295); //@line 299 "getopt.cpp"
                  var $147=HEAP32[(($optind144)>>2)]; //@line 299 "getopt.cpp"
                  var $148=$argv_addr; //@line 299 "getopt.cpp"
                  var $arrayidx145=(($148+4*$147)&4294967295); //@line 299 "getopt.cpp"
                  var $149=HEAP32[(($arrayidx145)>>2)]; //@line 299 "getopt.cpp"
                  var $150=$longopt_match; //@line 299 "getopt.cpp"
                  var $151=$longopts_addr; //@line 299 "getopt.cpp"
                  var $arrayidx146=(($151+16*$150)&4294967295); //@line 299 "getopt.cpp"
                  var $name147=(($arrayidx146)&4294967295); //@line 299 "getopt.cpp"
                  var $152=HEAP32[(($name147)>>2)]; //@line 299 "getopt.cpp"
                  var $153=$optindex; //@line 299 "getopt.cpp"
                  var $154=$longopts_addr; //@line 299 "getopt.cpp"
                  var $arrayidx148=(($154+16*$153)&4294967295); //@line 299 "getopt.cpp"
                  var $name149=(($arrayidx148)&4294967295); //@line 299 "getopt.cpp"
                  var $155=HEAP32[(($name149)>>2)]; //@line 299 "getopt.cpp"
                  var $call150=_fprintf($143, ((__str2833)&4294967295), allocate([$145,0,0,0,$149,0,0,0,$152,0,0,0,$155,0,0,0], ["i8*",0,0,0,"i8*",0,0,0,"i8*",0,0,0,"i8*",0,0,0], ALLOC_STACK)); //@line 299 "getopt.cpp"
                  ; //@line 299 "getopt.cpp"
                }
  
                var $156=$data_addr; //@line 305 "getopt.cpp"
                var $optopt152=(($156+12)&4294967295); //@line 305 "getopt.cpp"
                HEAP32[(($optopt152)>>2)]=63; //@line 305 "getopt.cpp"
                $retval=63; //@line 305 "getopt.cpp"
                __label__ = 96;break $if_then$$lor_lhs_false$2; //@line 305 "getopt.cpp"
              }
  
              var $158=__lastLabel__ == 49 ? $_pr : ($137); //@line 310 "getopt.cpp"
              var $cmp157=((($158))|0) >= 0; //@line 310 "getopt.cpp"
              if (!($cmp157)) { __label__ = 52;break $land_lhs_true77$$if_end162$41; } //@line 310 "getopt.cpp"
  
              var $159=$longopt_match; //@line 311 "getopt.cpp"
              var $160=$longopts_addr; //@line 311 "getopt.cpp"
              var $arrayidx159=(($160+16*$159)&4294967295); //@line 311 "getopt.cpp"
              var $has_arg160=(($arrayidx159+4)&4294967295); //@line 311 "getopt.cpp"
              var $161=HEAP32[(($has_arg160)>>2)]; //@line 311 "getopt.cpp"
              $has_arg=$161; //@line 311 "getopt.cpp"
              ; //@line 311 "getopt.cpp"
            }
          } while(0);
  
          var $162=$longopt_match; //@line 315 "getopt.cpp"
          var $cmp163=((($162))|0) < 0; //@line 315 "getopt.cpp"
          if ($cmp163) { __label__ = 53;; } else { __label__ = 63;; } //@line 315 "getopt.cpp"
          $land_lhs_true164$$if_end217$73: do { 
            if (__label__ == 53) {
  
              var $163=$shortopts_addr; //@line 315 "getopt.cpp"
              var $cmp165=((($163))|0)!=0; //@line 315 "getopt.cpp"
              if (!($cmp165)) { __label__ = 63;break $land_lhs_true164$$if_end217$73; } //@line 315 "getopt.cpp"
  
              var $164=$shortopts_addr; //@line 317 "getopt.cpp"
              var $165=$data_addr; //@line 317 "getopt.cpp"
              var $optwhere167=(($165+16)&4294967295); //@line 317 "getopt.cpp"
              var $166=HEAP32[(($optwhere167)>>2)]; //@line 317 "getopt.cpp"
              var $167=$data_addr; //@line 317 "getopt.cpp"
              var $optind168=(($167+4)&4294967295); //@line 317 "getopt.cpp"
              var $168=HEAP32[(($optind168)>>2)]; //@line 317 "getopt.cpp"
              var $169=$argv_addr; //@line 317 "getopt.cpp"
              var $arrayidx169=(($169+4*$168)&4294967295); //@line 317 "getopt.cpp"
              var $170=HEAP32[(($arrayidx169)>>2)]; //@line 317 "getopt.cpp"
              var $arrayidx170=(($170+$166)&4294967295); //@line 317 "getopt.cpp"
              var $171=HEAP8[($arrayidx170)]; //@line 317 "getopt.cpp"
              var $conv171=(tempInt=(($171)),(tempInt>=128?tempInt-256:tempInt)); //@line 317 "getopt.cpp"
              var $call172=_strchr($164, $conv171); //@line 317 "getopt.cpp"
              $cp=$call172; //@line 317 "getopt.cpp"
              var $172=$cp; //@line 318 "getopt.cpp"
              var $cmp173=((($172))|0)==0; //@line 318 "getopt.cpp"
              if ($cmp173) { __label__ = 55;; } else { __label__ = 60;; } //@line 318 "getopt.cpp"
              if (__label__ == 55) {
  
                var $173=$data_addr; //@line 321 "getopt.cpp"
                var $opterr175=(($173+8)&4294967295); //@line 321 "getopt.cpp"
                var $174=HEAP32[(($opterr175)>>2)]; //@line 321 "getopt.cpp"
                var $tobool176=((($174))|0)!=0; //@line 321 "getopt.cpp"
                if ($tobool176) { __label__ = 56;; } else { __label__ = 57;; } //@line 321 "getopt.cpp"
                if (__label__ == 56) {
  
                  var $175=HEAP32[((__impure_ptr)>>2)]; //@line 322 "getopt.cpp"
                  var $_stderr178=(($175+12)&4294967295); //@line 322 "getopt.cpp"
                  var $176=HEAP32[(($_stderr178)>>2)]; //@line 322 "getopt.cpp"
                  var $177=$argv_addr; //@line 322 "getopt.cpp"
                  var $arrayidx179=(($177)&4294967295); //@line 322 "getopt.cpp"
                  var $178=HEAP32[(($arrayidx179)>>2)]; //@line 322 "getopt.cpp"
                  var $179=$data_addr; //@line 322 "getopt.cpp"
                  var $optwhere180=(($179+16)&4294967295); //@line 322 "getopt.cpp"
                  var $180=HEAP32[(($optwhere180)>>2)]; //@line 322 "getopt.cpp"
                  var $181=$data_addr; //@line 322 "getopt.cpp"
                  var $optind181=(($181+4)&4294967295); //@line 322 "getopt.cpp"
                  var $182=HEAP32[(($optind181)>>2)]; //@line 322 "getopt.cpp"
                  var $183=$argv_addr; //@line 322 "getopt.cpp"
                  var $arrayidx182=(($183+4*$182)&4294967295); //@line 322 "getopt.cpp"
                  var $184=HEAP32[(($arrayidx182)>>2)]; //@line 322 "getopt.cpp"
                  var $arrayidx183=(($184+$180)&4294967295); //@line 322 "getopt.cpp"
                  var $185=HEAP8[($arrayidx183)]; //@line 322 "getopt.cpp"
                  var $conv184=(tempInt=(($185)),(tempInt>=128?tempInt-256:tempInt)); //@line 322 "getopt.cpp"
                  var $call185=_fprintf($176, ((__str3834)&4294967295), allocate([$178,0,0,0,$conv184,0,0,0], ["i8*",0,0,0,"i32",0,0,0], ALLOC_STACK)); //@line 322 "getopt.cpp"
                  ; //@line 322 "getopt.cpp"
                }
  
                var $186=$data_addr; //@line 325 "getopt.cpp"
                var $optwhere187=(($186+16)&4294967295); //@line 325 "getopt.cpp"
                var $187=HEAP32[(($optwhere187)>>2)]; //@line 325 "getopt.cpp"
                var $inc188=((($187)+1)&4294967295); //@line 325 "getopt.cpp"
                HEAP32[(($optwhere187)>>2)]=$inc188; //@line 325 "getopt.cpp"
                var $188=$data_addr; //@line 326 "getopt.cpp"
                var $optwhere189=(($188+16)&4294967295); //@line 326 "getopt.cpp"
                var $189=HEAP32[(($optwhere189)>>2)]; //@line 326 "getopt.cpp"
                var $190=$data_addr; //@line 326 "getopt.cpp"
                var $optind190=(($190+4)&4294967295); //@line 326 "getopt.cpp"
                var $191=HEAP32[(($optind190)>>2)]; //@line 326 "getopt.cpp"
                var $192=$argv_addr; //@line 326 "getopt.cpp"
                var $arrayidx191=(($192+4*$191)&4294967295); //@line 326 "getopt.cpp"
                var $193=HEAP32[(($arrayidx191)>>2)]; //@line 326 "getopt.cpp"
                var $arrayidx192=(($193+$189)&4294967295); //@line 326 "getopt.cpp"
                var $194=HEAP8[($arrayidx192)]; //@line 326 "getopt.cpp"
                var $conv193=(tempInt=(($194)),(tempInt>=128?tempInt-256:tempInt)); //@line 326 "getopt.cpp"
                var $cmp194=((($conv193))|0)==0; //@line 326 "getopt.cpp"
                if ($cmp194) { __label__ = 58;; } else { __label__ = 59;; } //@line 326 "getopt.cpp"
                if (__label__ == 58) {
  
                  var $195=$data_addr; //@line 328 "getopt.cpp"
                  var $optind196=(($195+4)&4294967295); //@line 328 "getopt.cpp"
                  var $196=HEAP32[(($optind196)>>2)]; //@line 328 "getopt.cpp"
                  var $inc197=((($196)+1)&4294967295); //@line 328 "getopt.cpp"
                  HEAP32[(($optind196)>>2)]=$inc197; //@line 328 "getopt.cpp"
                  var $197=$data_addr; //@line 329 "getopt.cpp"
                  var $optwhere198=(($197+16)&4294967295); //@line 329 "getopt.cpp"
                  HEAP32[(($optwhere198)>>2)]=1; //@line 329 "getopt.cpp"
                  ; //@line 330 "getopt.cpp"
                }
  
                var $198=$data_addr; //@line 331 "getopt.cpp"
                var $optopt200=(($198+12)&4294967295); //@line 331 "getopt.cpp"
                HEAP32[(($optopt200)>>2)]=63; //@line 331 "getopt.cpp"
                $retval=63; //@line 331 "getopt.cpp"
                __label__ = 96;break $if_then$$lor_lhs_false$2; //@line 331 "getopt.cpp"
              }
              else if (__label__ == 60) {
  
                var $199=$cp; //@line 333 "getopt.cpp"
                var $arrayidx202=(($199+1)&4294967295); //@line 333 "getopt.cpp"
                var $200=HEAP8[($arrayidx202)]; //@line 333 "getopt.cpp"
                var $conv203=(tempInt=(($200)),(tempInt>=128?tempInt-256:tempInt)); //@line 333 "getopt.cpp"
                var $cmp204=((($conv203))|0)==58; //@line 333 "getopt.cpp"
                if ($cmp204) { __lastLabel__ = 60; __label__ = 61;; } else { __lastLabel__ = 60; __label__ = 62;; } //@line 333 "getopt.cpp"
                if (__label__ == 61) {
  
                  var $201=$cp; //@line 333 "getopt.cpp"
                  var $arrayidx205=(($201+2)&4294967295); //@line 333 "getopt.cpp"
                  var $202=HEAP8[($arrayidx205)]; //@line 333 "getopt.cpp"
                  var $conv206=(tempInt=(($202)),(tempInt>=128?tempInt-256:tempInt)); //@line 333 "getopt.cpp"
                  var $cmp207=((($conv206))|0)==58; //@line 333 "getopt.cpp"
                  var $cond208=($cmp207) ? 2 : 1; //@line 333 "getopt.cpp"
                  __lastLabel__ = 61; ; //@line 333 "getopt.cpp"
                }
  
                var $cond209=__lastLabel__ == 61 ? $cond208 : (0); //@line 333 "getopt.cpp"
                $has_arg=$cond209; //@line 333 "getopt.cpp"
                var $203=$data_addr; //@line 335 "getopt.cpp"
                var $optind210=(($203+4)&4294967295); //@line 335 "getopt.cpp"
                var $204=HEAP32[(($optind210)>>2)]; //@line 335 "getopt.cpp"
                var $205=$argv_addr; //@line 335 "getopt.cpp"
                var $arrayidx211=(($205+4*$204)&4294967295); //@line 335 "getopt.cpp"
                var $206=HEAP32[(($arrayidx211)>>2)]; //@line 335 "getopt.cpp"
                var $207=$data_addr; //@line 335 "getopt.cpp"
                var $optwhere212=(($207+16)&4294967295); //@line 335 "getopt.cpp"
                var $208=HEAP32[(($optwhere212)>>2)]; //@line 335 "getopt.cpp"
                var $add_ptr213=(($206+$208)&4294967295); //@line 335 "getopt.cpp"
                var $add_ptr214=(($add_ptr213+1)&4294967295); //@line 335 "getopt.cpp"
                $possible_arg=$add_ptr214; //@line 335 "getopt.cpp"
                var $209=$cp; //@line 336 "getopt.cpp"
                var $210=HEAP8[($209)]; //@line 336 "getopt.cpp"
                var $conv215=(tempInt=(($210)),(tempInt>=128?tempInt-256:tempInt)); //@line 336 "getopt.cpp"
                var $211=$data_addr; //@line 336 "getopt.cpp"
                var $optopt216=(($211+12)&4294967295); //@line 336 "getopt.cpp"
                HEAP32[(($optopt216)>>2)]=$conv215; //@line 336 "getopt.cpp"
                ; //@line 337 "getopt.cpp"
              }
            }
          } while(0);
  
          $arg_next=0; //@line 340 "getopt.cpp"
          var $212=$has_arg; //@line 341 "getopt.cpp"
          if ($212 == 2) {
            __label__ = 64;;
          }
          else if ($212 == 1) {
            __label__ = 67;;
          }
          else {
          __label__ = 78;;
          }
          
          if (__label__ == 78) {
  
            var $254=$longopt_match; //@line 379 "getopt.cpp"
            var $cmp279=((($254))|0) < 0; //@line 379 "getopt.cpp"
            var $255=$data_addr; //@line 381 "getopt.cpp"
            var $optwhere281=(($255+16)&4294967295); //@line 381 "getopt.cpp"
            if ($cmp279) { __label__ = 79;; } else { __label__ = 81;; } //@line 379 "getopt.cpp"
            $if_then280$$if_else292$91: do { 
              if (__label__ == 79) {
  
                var $256=HEAP32[(($optwhere281)>>2)]; //@line 381 "getopt.cpp"
                var $inc282=((($256)+1)&4294967295); //@line 381 "getopt.cpp"
                HEAP32[(($optwhere281)>>2)]=$inc282; //@line 381 "getopt.cpp"
                var $257=$data_addr; //@line 382 "getopt.cpp"
                var $optwhere283=(($257+16)&4294967295); //@line 382 "getopt.cpp"
                var $258=HEAP32[(($optwhere283)>>2)]; //@line 382 "getopt.cpp"
                var $259=$data_addr; //@line 382 "getopt.cpp"
                var $optind284=(($259+4)&4294967295); //@line 382 "getopt.cpp"
                var $260=HEAP32[(($optind284)>>2)]; //@line 382 "getopt.cpp"
                var $261=$argv_addr; //@line 382 "getopt.cpp"
                var $arrayidx285=(($261+4*$260)&4294967295); //@line 382 "getopt.cpp"
                var $262=HEAP32[(($arrayidx285)>>2)]; //@line 382 "getopt.cpp"
                var $arrayidx286=(($262+$258)&4294967295); //@line 382 "getopt.cpp"
                var $263=HEAP8[($arrayidx286)]; //@line 382 "getopt.cpp"
                var $conv287=(tempInt=(($263)),(tempInt>=128?tempInt-256:tempInt)); //@line 382 "getopt.cpp"
                var $cmp288=((($conv287))|0)==0; //@line 382 "getopt.cpp"
                if (!($cmp288)) { __label__ = 82;break $if_then280$$if_else292$91; } //@line 382 "getopt.cpp"
  
                var $264=$data_addr; //@line 383 "getopt.cpp"
                var $optwhere290=(($264+16)&4294967295); //@line 383 "getopt.cpp"
                HEAP32[(($optwhere290)>>2)]=1; //@line 383 "getopt.cpp"
                ; //@line 383 "getopt.cpp"
              }
              else if (__label__ == 81) {
  
                HEAP32[(($optwhere281)>>2)]=1; //@line 386 "getopt.cpp"
                ;
              }
            } while(0);
  
            var $265=$data_addr; //@line 387 "getopt.cpp"
            var $optarg295=(($265)&4294967295); //@line 387 "getopt.cpp"
            HEAP32[(($optarg295)>>2)]=0; //@line 387 "getopt.cpp"
            ; //@line 388 "getopt.cpp"
          }
          else if (__label__ == 64) {
  
            var $213=$possible_arg; //@line 344 "getopt.cpp"
            var $214=HEAP8[($213)]; //@line 344 "getopt.cpp"
            var $conv219=(tempInt=(($214)),(tempInt>=128?tempInt-256:tempInt)); //@line 344 "getopt.cpp"
            var $cmp220=((($conv219))|0)==61; //@line 344 "getopt.cpp"
            if ($cmp220) { __label__ = 65;; } else { __label__ = 66;; } //@line 344 "getopt.cpp"
            if (__label__ == 65) {
  
              var $215=$possible_arg; //@line 345 "getopt.cpp"
              var $incdec_ptr222=(($215+1)&4294967295); //@line 345 "getopt.cpp"
              $possible_arg=$incdec_ptr222; //@line 345 "getopt.cpp"
              ; //@line 345 "getopt.cpp"
            }
  
            var $216=$possible_arg; //@line 346 "getopt.cpp"
            var $217=HEAP8[($216)]; //@line 346 "getopt.cpp"
            var $conv224=(tempInt=(($217)),(tempInt>=128?tempInt-256:tempInt)); //@line 346 "getopt.cpp"
            var $cmp225=((($conv224))|0)!=0; //@line 346 "getopt.cpp"
            var $218=$possible_arg; //@line 346 "getopt.cpp"
            var $cond229=($cmp225) ? ($218) : 0; //@line 346 "getopt.cpp"
            var $219=$data_addr; //@line 346 "getopt.cpp"
            var $optarg230=(($219)&4294967295); //@line 346 "getopt.cpp"
            HEAP32[(($optarg230)>>2)]=$cond229; //@line 346 "getopt.cpp"
            var $220=$data_addr; //@line 347 "getopt.cpp"
            var $optwhere231=(($220+16)&4294967295); //@line 347 "getopt.cpp"
            HEAP32[(($optwhere231)>>2)]=1; //@line 347 "getopt.cpp"
            ; //@line 348 "getopt.cpp"
          }
          else if (__label__ == 67) {
  
            var $221=$possible_arg; //@line 350 "getopt.cpp"
            var $222=HEAP8[($221)]; //@line 350 "getopt.cpp"
            var $conv233=(tempInt=(($222)),(tempInt>=128?tempInt-256:tempInt)); //@line 350 "getopt.cpp"
            var $cmp234=((($conv233))|0)==61; //@line 350 "getopt.cpp"
            if ($cmp234) { __label__ = 68;; } else { __label__ = 69;; } //@line 350 "getopt.cpp"
            if (__label__ == 68) {
  
              var $223=$possible_arg; //@line 351 "getopt.cpp"
              var $incdec_ptr236=(($223+1)&4294967295); //@line 351 "getopt.cpp"
              $possible_arg=$incdec_ptr236; //@line 351 "getopt.cpp"
              ; //@line 351 "getopt.cpp"
            }
  
            var $224=$possible_arg; //@line 352 "getopt.cpp"
            var $225=HEAP8[($224)]; //@line 352 "getopt.cpp"
            var $conv238=(tempInt=(($225)),(tempInt>=128?tempInt-256:tempInt)); //@line 352 "getopt.cpp"
            var $cmp239=((($conv238))|0)!=0; //@line 352 "getopt.cpp"
            if ($cmp239) { __label__ = 70;; } else { __label__ = 71;; } //@line 352 "getopt.cpp"
            if (__label__ == 70) {
  
              var $226=$possible_arg; //@line 354 "getopt.cpp"
              var $227=$data_addr; //@line 354 "getopt.cpp"
              var $optarg241=(($227)&4294967295); //@line 354 "getopt.cpp"
              HEAP32[(($optarg241)>>2)]=$226; //@line 354 "getopt.cpp"
              var $228=$data_addr; //@line 355 "getopt.cpp"
              var $optwhere242=(($228+16)&4294967295); //@line 355 "getopt.cpp"
              HEAP32[(($optwhere242)>>2)]=1; //@line 355 "getopt.cpp"
              ; //@line 356 "getopt.cpp"
            }
            else if (__label__ == 71) {
  
              var $229=$data_addr; //@line 357 "getopt.cpp"
              var $optind244=(($229+4)&4294967295); //@line 357 "getopt.cpp"
              var $230=HEAP32[(($optind244)>>2)]; //@line 357 "getopt.cpp"
              var $add245=((($230)+1)&4294967295); //@line 357 "getopt.cpp"
              var $231=$argc_addr; //@line 357 "getopt.cpp"
              var $cmp246=((($add245))|0) >= ((($231))|0); //@line 357 "getopt.cpp"
              var $232=$data_addr; //@line 359 "getopt.cpp"
              if ($cmp246) { __label__ = 72;; } else { __label__ = 77;; } //@line 357 "getopt.cpp"
              if (__label__ == 72) {
  
                var $opterr248=(($232+8)&4294967295); //@line 359 "getopt.cpp"
                var $233=HEAP32[(($opterr248)>>2)]; //@line 359 "getopt.cpp"
                var $tobool249=((($233))|0)!=0; //@line 359 "getopt.cpp"
                if ($tobool249) { __label__ = 73;; } else { __label__ = 76;; } //@line 359 "getopt.cpp"
                if (__label__ == 73) {
  
                  var $234=HEAP32[((__impure_ptr)>>2)]; //@line 361 "getopt.cpp"
                  var $_stderr251=(($234+12)&4294967295); //@line 361 "getopt.cpp"
                  var $235=HEAP32[(($_stderr251)>>2)]; //@line 361 "getopt.cpp"
                  var $236=$argv_addr; //@line 361 "getopt.cpp"
                  var $arrayidx252=(($236)&4294967295); //@line 361 "getopt.cpp"
                  var $237=HEAP32[(($arrayidx252)>>2)]; //@line 361 "getopt.cpp"
                  var $call253=_fprintf($235, ((__str4835)&4294967295), allocate([$237,0,0,0], ["i8*",0,0,0], ALLOC_STACK)); //@line 361 "getopt.cpp"
                  var $238=$longopt_match; //@line 362 "getopt.cpp"
                  var $cmp254=((($238))|0) >= 0; //@line 362 "getopt.cpp"
                  var $239=HEAP32[((__impure_ptr)>>2)]; //@line 363 "getopt.cpp"
                  var $_stderr256=(($239+12)&4294967295); //@line 363 "getopt.cpp"
                  var $240=HEAP32[(($_stderr256)>>2)]; //@line 363 "getopt.cpp"
                  if ($cmp254) { __label__ = 74;; } else { __label__ = 75;; } //@line 362 "getopt.cpp"
                  if (__label__ == 74) {
  
                    var $241=$longopt_match; //@line 363 "getopt.cpp"
                    var $242=$longopts_addr; //@line 363 "getopt.cpp"
                    var $arrayidx257=(($242+16*$241)&4294967295); //@line 363 "getopt.cpp"
                    var $name258=(($arrayidx257)&4294967295); //@line 363 "getopt.cpp"
                    var $243=HEAP32[(($name258)>>2)]; //@line 363 "getopt.cpp"
                    var $call259=_fprintf($240, ((__str5836)&4294967295), allocate([$243,0,0,0], ["i8*",0,0,0], ALLOC_STACK)); //@line 363 "getopt.cpp"
                    ; //@line 363 "getopt.cpp"
                  }
                  else if (__label__ == 75) {
  
                    var $244=$cp; //@line 365 "getopt.cpp"
                    var $245=HEAP8[($244)]; //@line 365 "getopt.cpp"
                    var $conv262=(tempInt=(($245)),(tempInt>=128?tempInt-256:tempInt)); //@line 365 "getopt.cpp"
                    var $call263=_fprintf($240, ((__str6837)&4294967295), allocate([$conv262,0,0,0], ["i32",0,0,0], ALLOC_STACK)); //@line 365 "getopt.cpp"
                    ;
                  }
                }
  
                var $246=$data_addr; //@line 367 "getopt.cpp"
                var $optind266=(($246+4)&4294967295); //@line 367 "getopt.cpp"
                var $247=HEAP32[(($optind266)>>2)]; //@line 367 "getopt.cpp"
                var $inc267=((($247)+1)&4294967295); //@line 367 "getopt.cpp"
                HEAP32[(($optind266)>>2)]=$inc267; //@line 367 "getopt.cpp"
                var $248=$data_addr; //@line 368 "getopt.cpp"
                var $optopt268=(($248+12)&4294967295); //@line 368 "getopt.cpp"
                HEAP32[(($optopt268)>>2)]=58; //@line 368 "getopt.cpp"
                $retval=58; //@line 368 "getopt.cpp"
                __label__ = 96;break $if_then$$lor_lhs_false$2; //@line 368 "getopt.cpp"
              }
              else if (__label__ == 77) {
  
                var $optind270=(($232+4)&4294967295); //@line 372 "getopt.cpp"
                var $249=HEAP32[(($optind270)>>2)]; //@line 372 "getopt.cpp"
                var $add271=((($249)+1)&4294967295); //@line 372 "getopt.cpp"
                var $250=$argv_addr; //@line 372 "getopt.cpp"
                var $arrayidx272=(($250+4*$add271)&4294967295); //@line 372 "getopt.cpp"
                var $251=HEAP32[(($arrayidx272)>>2)]; //@line 372 "getopt.cpp"
                var $252=$data_addr; //@line 372 "getopt.cpp"
                var $optarg273=(($252)&4294967295); //@line 372 "getopt.cpp"
                HEAP32[(($optarg273)>>2)]=$251; //@line 372 "getopt.cpp"
                $arg_next=1; //@line 373 "getopt.cpp"
                var $253=$data_addr; //@line 374 "getopt.cpp"
                var $optwhere274=(($253+16)&4294967295); //@line 374 "getopt.cpp"
                HEAP32[(($optwhere274)>>2)]=1; //@line 374 "getopt.cpp"
                ;
              }
            }
          }
  
          var $266=$ordering; //@line 392 "getopt.cpp"
          var $cmp297=((($266))|0)==0; //@line 392 "getopt.cpp"
          if ($cmp297) { __label__ = 84;; } else { __label__ = 87;; } //@line 392 "getopt.cpp"
          $land_lhs_true298$$if_else309$117: do { 
            if (__label__ == 84) {
  
              var $267=$data_addr; //@line 392 "getopt.cpp"
              var $optwhere299=(($267+16)&4294967295); //@line 392 "getopt.cpp"
              var $268=HEAP32[(($optwhere299)>>2)]; //@line 392 "getopt.cpp"
              var $cmp300=((($268))|0)==1; //@line 392 "getopt.cpp"
              if (!($cmp300)) { __label__ = 87;break $land_lhs_true298$$if_else309$117; } //@line 392 "getopt.cpp"
  
              var $269=$num_nonopts; //@line 392 "getopt.cpp"
              var $cmp302=((($269))|0)!=0; //@line 392 "getopt.cpp"
              if (!($cmp302)) { __label__ = 87;break $land_lhs_true298$$if_else309$117; } //@line 392 "getopt.cpp"
  
              var $270=$argv_addr; //@line 394 "getopt.cpp"
              var $271=$permute_from; //@line 394 "getopt.cpp"
              var $add_ptr304=(($270+4*$271)&4294967295); //@line 394 "getopt.cpp"
              var $272=$num_nonopts; //@line 394 "getopt.cpp"
              var $273=$arg_next; //@line 394 "getopt.cpp"
              var $add305=((($273)+1)&4294967295); //@line 394 "getopt.cpp"
              __ZL7permutePKPcii($add_ptr304, $272, $add305); //@line 394 "getopt.cpp"
              var $274=$permute_from; //@line 395 "getopt.cpp"
              var $275=$arg_next; //@line 395 "getopt.cpp"
              var $add306=((($274)+1)&4294967295); //@line 395 "getopt.cpp"
              var $add307=((($add306)+($275))&4294967295); //@line 395 "getopt.cpp"
              var $276=$data_addr; //@line 395 "getopt.cpp"
              var $optind308=(($276+4)&4294967295); //@line 395 "getopt.cpp"
              HEAP32[(($optind308)>>2)]=$add307; //@line 395 "getopt.cpp"
              __label__ = 89;break $land_lhs_true298$$if_else309$117; //@line 396 "getopt.cpp"
            }
          } while(0);
          $if_end318$$if_else309$121: do { 
            if (__label__ == 87) {
  
              var $277=$data_addr; //@line 397 "getopt.cpp"
              var $optwhere310=(($277+16)&4294967295); //@line 397 "getopt.cpp"
              var $278=HEAP32[(($optwhere310)>>2)]; //@line 397 "getopt.cpp"
              var $cmp311=((($278))|0)==1; //@line 397 "getopt.cpp"
              if (!($cmp311)) { __label__ = 89;break $if_end318$$if_else309$121; } //@line 397 "getopt.cpp"
  
              var $279=$data_addr; //@line 398 "getopt.cpp"
              var $optind313=(($279+4)&4294967295); //@line 398 "getopt.cpp"
              var $280=HEAP32[(($optind313)>>2)]; //@line 398 "getopt.cpp"
              var $281=$arg_next; //@line 398 "getopt.cpp"
              var $add314=((($280)+1)&4294967295); //@line 398 "getopt.cpp"
              var $add315=((($add314)+($281))&4294967295); //@line 398 "getopt.cpp"
              var $282=$data_addr; //@line 398 "getopt.cpp"
              var $optind316=(($282+4)&4294967295); //@line 398 "getopt.cpp"
              HEAP32[(($optind316)>>2)]=$add315; //@line 398 "getopt.cpp"
              ; //@line 398 "getopt.cpp"
            }
          } while(0);
  
          var $283=$longopt_match; //@line 401 "getopt.cpp"
          var $cmp319=((($283))|0) >= 0; //@line 401 "getopt.cpp"
          if ($cmp319) { __label__ = 90;; } else { __label__ = 95;; } //@line 401 "getopt.cpp"
          if (__label__ == 90) {
  
            var $284=$longind_addr; //@line 403 "getopt.cpp"
            var $cmp321=((($284))|0)!=0; //@line 403 "getopt.cpp"
            if ($cmp321) { __label__ = 91;; } else { __label__ = 92;; } //@line 403 "getopt.cpp"
            if (__label__ == 91) {
  
              var $285=$longopt_match; //@line 404 "getopt.cpp"
              var $286=$longind_addr; //@line 404 "getopt.cpp"
              HEAP32[(($286)>>2)]=$285; //@line 404 "getopt.cpp"
              ; //@line 404 "getopt.cpp"
            }
  
            var $287=$longopt_match; //@line 405 "getopt.cpp"
            var $288=$longopts_addr; //@line 405 "getopt.cpp"
            var $arrayidx324=(($288+16*$287)&4294967295); //@line 405 "getopt.cpp"
            var $flag=(($arrayidx324+8)&4294967295); //@line 405 "getopt.cpp"
            var $289=HEAP32[(($flag)>>2)]; //@line 405 "getopt.cpp"
            var $cmp325=((($289))|0)!=0; //@line 405 "getopt.cpp"
            var $290=$longopt_match; //@line 407 "getopt.cpp"
            var $291=$longopts_addr; //@line 407 "getopt.cpp"
            var $arrayidx327=(($291+16*$290)&4294967295); //@line 407 "getopt.cpp"
            var $val=(($arrayidx327+12)&4294967295); //@line 407 "getopt.cpp"
            var $292=HEAP32[(($val)>>2)]; //@line 407 "getopt.cpp"
            if ($cmp325) { __label__ = 93;; } else { __label__ = 94;; } //@line 405 "getopt.cpp"
            if (__label__ == 93) {
  
              var $293=$longopt_match; //@line 407 "getopt.cpp"
              var $294=$longopts_addr; //@line 407 "getopt.cpp"
              var $arrayidx328=(($294+16*$293)&4294967295); //@line 407 "getopt.cpp"
              var $flag329=(($arrayidx328+8)&4294967295); //@line 407 "getopt.cpp"
              var $295=HEAP32[(($flag329)>>2)]; //@line 407 "getopt.cpp"
              HEAP32[(($295)>>2)]=$292; //@line 407 "getopt.cpp"
              $retval=0; //@line 408 "getopt.cpp"
              __label__ = 96;break $if_then$$lor_lhs_false$2; //@line 408 "getopt.cpp"
            }
            else if (__label__ == 94) {
  
              $retval=$292; //@line 411 "getopt.cpp"
              __label__ = 96;break $if_then$$lor_lhs_false$2; //@line 411 "getopt.cpp"
            }
          }
          else if (__label__ == 95) {
  
            var $296=$data_addr; //@line 414 "getopt.cpp"
            var $optopt334=(($296+12)&4294967295); //@line 414 "getopt.cpp"
            var $297=HEAP32[(($optopt334)>>2)]; //@line 414 "getopt.cpp"
            $retval=$297; //@line 414 "getopt.cpp"
            __label__ = 96;break $if_then$$lor_lhs_false$2; //@line 414 "getopt.cpp"
          }
        }
      }
    } while(0);
    if (__label__ == 6) {
  
      $retval=-1; //@line 193 "getopt.cpp"
      ; //@line 193 "getopt.cpp"
    }
  
    var $298=$retval; //@line 415 "getopt.cpp"
    ;
    return $298; //@line 415 "getopt.cpp"
    return null;
  }
  

  function __ZL13write_globalsP11getopt_data($data) {
    ;
    var __label__;
  
    var $data_addr;
    $data_addr=$data;
    var $0=$data_addr; //@line 166 "getopt.cpp"
    var $optarg=(($0)&4294967295); //@line 166 "getopt.cpp"
    var $1=HEAP32[(($optarg)>>2)]; //@line 166 "getopt.cpp"
    HEAP32[((_optarg)>>2)]=$1; //@line 166 "getopt.cpp"
    var $2=$data_addr; //@line 167 "getopt.cpp"
    var $optind=(($2+4)&4294967295); //@line 167 "getopt.cpp"
    var $3=HEAP32[(($optind)>>2)]; //@line 167 "getopt.cpp"
    HEAP32[((_optind)>>2)]=$3; //@line 167 "getopt.cpp"
    var $4=$data_addr; //@line 168 "getopt.cpp"
    var $opterr=(($4+8)&4294967295); //@line 168 "getopt.cpp"
    var $5=HEAP32[(($opterr)>>2)]; //@line 168 "getopt.cpp"
    HEAP32[((_opterr)>>2)]=$5; //@line 168 "getopt.cpp"
    var $6=$data_addr; //@line 169 "getopt.cpp"
    var $optopt=(($6+12)&4294967295); //@line 169 "getopt.cpp"
    var $7=HEAP32[(($optopt)>>2)]; //@line 169 "getopt.cpp"
    HEAP32[((_optopt)>>2)]=$7; //@line 169 "getopt.cpp"
    var $8=$data_addr; //@line 170 "getopt.cpp"
    var $optwhere=(($8+16)&4294967295); //@line 170 "getopt.cpp"
    var $9=HEAP32[(($optwhere)>>2)]; //@line 170 "getopt.cpp"
    HEAP32[((__ZL8optwhere)>>2)]=$9; //@line 170 "getopt.cpp"
    ;
    return; //@line 171 "getopt.cpp"
    return;
  }
  

  function _getopt_long($argc, $argv, $shortopts, $longopts, $longind) {
    var __stackBase__  = STACKTOP; STACKTOP += 20; _memset(__stackBase__, 0, 20);
    var __label__;
  
    var $argc_addr;
    var $argv_addr;
    var $shortopts_addr;
    var $longopts_addr;
    var $longind_addr;
    var $data=__stackBase__;
    var $r;
    $argc_addr=$argc;
    $argv_addr=$argv;
    $shortopts_addr=$shortopts;
    $longopts_addr=$longopts;
    $longind_addr=$longind;
    __ZL12read_globalsP11getopt_data($data); //@line 436 "getopt.cpp"
    var $0=$argc_addr; //@line 437 "getopt.cpp"
    var $1=$argv_addr; //@line 437 "getopt.cpp"
    var $2=$shortopts_addr; //@line 437 "getopt.cpp"
    var $3=$longopts_addr; //@line 437 "getopt.cpp"
    var $4=$longind_addr; //@line 437 "getopt.cpp"
    var $call=__ZL15getopt_internaliPKPcPKcPK6optionPiiP11getopt_data($0, $1, $2, $3, $4, 0, $data); //@line 437 "getopt.cpp"
    $r=$call; //@line 437 "getopt.cpp"
    __ZL13write_globalsP11getopt_data($data); //@line 438 "getopt.cpp"
    var $5=$r; //@line 439 "getopt.cpp"
    STACKTOP = __stackBase__;
    return $5; //@line 439 "getopt.cpp"
    return null;
  }
  

  function _getopt_long_only($argc, $argv, $shortopts, $longopts, $longind) {
    var __stackBase__  = STACKTOP; STACKTOP += 20; _memset(__stackBase__, 0, 20);
    var __label__;
  
    var $argc_addr;
    var $argv_addr;
    var $shortopts_addr;
    var $longopts_addr;
    var $longind_addr;
    var $data=__stackBase__;
    var $r;
    $argc_addr=$argc;
    $argv_addr=$argv;
    $shortopts_addr=$shortopts;
    $longopts_addr=$longopts;
    $longind_addr=$longind;
    __ZL12read_globalsP11getopt_data($data); //@line 449 "getopt.cpp"
    var $0=$argc_addr; //@line 450 "getopt.cpp"
    var $1=$argv_addr; //@line 450 "getopt.cpp"
    var $2=$shortopts_addr; //@line 450 "getopt.cpp"
    var $3=$longopts_addr; //@line 450 "getopt.cpp"
    var $4=$longind_addr; //@line 450 "getopt.cpp"
    var $call=__ZL15getopt_internaliPKPcPKcPK6optionPiiP11getopt_data($0, $1, $2, $3, $4, 1, $data); //@line 450 "getopt.cpp"
    $r=$call; //@line 450 "getopt.cpp"
    __ZL13write_globalsP11getopt_data($data); //@line 451 "getopt.cpp"
    var $5=$r; //@line 452 "getopt.cpp"
    STACKTOP = __stackBase__;
    return $5; //@line 452 "getopt.cpp"
    return null;
  }
  

  function ___getopt_r($argc, $argv, $optstring, $data) {
    ;
    var __label__;
  
    var $argc_addr;
    var $argv_addr;
    var $optstring_addr;
    var $data_addr;
    $argc_addr=$argc;
    $argv_addr=$argv;
    $optstring_addr=$optstring;
    $data_addr=$data;
    var $0=$argc_addr; //@line 459 "getopt.cpp"
    var $1=$argv_addr; //@line 459 "getopt.cpp"
    var $2=$optstring_addr; //@line 459 "getopt.cpp"
    var $3=$data_addr; //@line 459 "getopt.cpp"
    var $call=__ZL15getopt_internaliPKPcPKcPK6optionPiiP11getopt_data($0, $1, $2, 0, 0, 0, $3); //@line 459 "getopt.cpp"
    ;
    return $call; //@line 459 "getopt.cpp"
    return null;
  }
  

  function ___getopt_long_r($argc, $argv, $shortopts, $longopts, $longind, $data) {
    ;
    var __label__;
  
    var $argc_addr;
    var $argv_addr;
    var $shortopts_addr;
    var $longopts_addr;
    var $longind_addr;
    var $data_addr;
    $argc_addr=$argc;
    $argv_addr=$argv;
    $shortopts_addr=$shortopts;
    $longopts_addr=$longopts;
    $longind_addr=$longind;
    $data_addr=$data;
    var $0=$argc_addr; //@line 467 "getopt.cpp"
    var $1=$argv_addr; //@line 467 "getopt.cpp"
    var $2=$shortopts_addr; //@line 467 "getopt.cpp"
    var $3=$longopts_addr; //@line 467 "getopt.cpp"
    var $4=$longind_addr; //@line 467 "getopt.cpp"
    var $5=$data_addr; //@line 467 "getopt.cpp"
    var $call=__ZL15getopt_internaliPKPcPKcPK6optionPiiP11getopt_data($0, $1, $2, $3, $4, 0, $5); //@line 467 "getopt.cpp"
    ;
    return $call; //@line 467 "getopt.cpp"
    return null;
  }
  

  function ___getopt_long_only_r($argc, $argv, $shortopts, $longopts, $longind, $data) {
    ;
    var __label__;
  
    var $argc_addr;
    var $argv_addr;
    var $shortopts_addr;
    var $longopts_addr;
    var $longind_addr;
    var $data_addr;
    $argc_addr=$argc;
    $argv_addr=$argv;
    $shortopts_addr=$shortopts;
    $longopts_addr=$longopts;
    $longind_addr=$longind;
    $data_addr=$data;
    var $0=$argc_addr; //@line 475 "getopt.cpp"
    var $1=$argv_addr; //@line 475 "getopt.cpp"
    var $2=$shortopts_addr; //@line 475 "getopt.cpp"
    var $3=$longopts_addr; //@line 475 "getopt.cpp"
    var $4=$longind_addr; //@line 475 "getopt.cpp"
    var $5=$data_addr; //@line 475 "getopt.cpp"
    var $call=__ZL15getopt_internaliPKPcPKcPK6optionPiiP11getopt_data($0, $1, $2, $3, $4, 1, $5); //@line 475 "getopt.cpp"
    ;
    return $call; //@line 475 "getopt.cpp"
    return null;
  }
  

  function __ZL9is_optionPci($argv_element, $only) {
    ;
    var __label__;
    var __lastLabel__ = null;
  
    var $argv_element_addr;
    var $only_addr;
    $argv_element_addr=$argv_element;
    $only_addr=$only;
    var $0=$argv_element_addr; //@line 145 "getopt.cpp"
    var $cmp=((($0))|0)==0; //@line 145 "getopt.cpp"
    if ($cmp) { __lastLabel__ = 0; __label__ = 4;; } else { __lastLabel__ = 0; __label__ = 1;; } //@line 145 "getopt.cpp"
    $lor_end$$lor_lhs_false$2: do { 
      if (__label__ == 1) {
  
        var $1=$argv_element_addr; //@line 145 "getopt.cpp"
        var $arrayidx=(($1)&4294967295); //@line 145 "getopt.cpp"
        var $2=HEAP8[($arrayidx)]; //@line 145 "getopt.cpp"
        var $conv=(tempInt=(($2)),(tempInt>=128?tempInt-256:tempInt)); //@line 145 "getopt.cpp"
        var $cmp1=((($conv))|0)==45; //@line 145 "getopt.cpp"
        if ($cmp1) { __lastLabel__ = 1; __label__ = 4;break $lor_end$$lor_lhs_false$2; } else { __lastLabel__ = 1; ; } //@line 145 "getopt.cpp"
  
        var $3=$only_addr; //@line 145 "getopt.cpp"
        var $tobool=((($3))|0)!=0; //@line 145 "getopt.cpp"
        if ($tobool) { __lastLabel__ = 2; ; } else { __lastLabel__ = 2; __label__ = 4;break $lor_end$$lor_lhs_false$2; } //@line 145 "getopt.cpp"
  
        var $4=$argv_element_addr; //@line 145 "getopt.cpp"
        var $arrayidx2=(($4)&4294967295); //@line 145 "getopt.cpp"
        var $5=HEAP8[($arrayidx2)]; //@line 145 "getopt.cpp"
        var $conv3=(tempInt=(($5)),(tempInt>=128?tempInt-256:tempInt)); //@line 145 "getopt.cpp"
        var $cmp4=((($conv3))|0)==43; //@line 145 "getopt.cpp"
        __lastLabel__ = 3; ;
      }
    } while(0);
  
    var $6=__lastLabel__ == 1 ? 1 : (__lastLabel__ == 0 ? 1 : (__lastLabel__ == 2 ? 0 : ($cmp4)));
    var $conv5=((($6))&1);
    ;
    return $conv5; //@line 147 "getopt.cpp"
    return null;
  }
  

  function __ZL7permutePKPcii($argv, $len1, $len2) {
    ;
    var __label__;
  
    var $argv_addr;
    var $len1_addr;
    var $len2_addr;
    $argv_addr=$argv;
    $len1_addr=$len1;
    $len2_addr=$len2;
    var $0=$argv_addr; //@line 136 "getopt.cpp"
    var $1=$len1_addr; //@line 136 "getopt.cpp"
    __ZL21reverse_argv_elementsPPci($0, $1); //@line 136 "getopt.cpp"
    var $2=$argv_addr; //@line 137 "getopt.cpp"
    var $3=$len1_addr; //@line 137 "getopt.cpp"
    var $4=$len2_addr; //@line 137 "getopt.cpp"
    var $add=((($4)+($3))&4294967295); //@line 137 "getopt.cpp"
    __ZL21reverse_argv_elementsPPci($2, $add); //@line 137 "getopt.cpp"
    var $5=$argv_addr; //@line 138 "getopt.cpp"
    var $6=$len2_addr; //@line 138 "getopt.cpp"
    __ZL21reverse_argv_elementsPPci($5, $6); //@line 138 "getopt.cpp"
    ;
    return; //@line 139 "getopt.cpp"
    return;
  }
  

  function __ZL21reverse_argv_elementsPPci($argv, $num) {
    ;
    var __label__;
  
    var $argv_addr;
    var $num_addr;
    var $i;
    var $tmp;
    $argv_addr=$argv;
    $num_addr=$num;
    $i=0; //@line 124 "getopt.cpp"
    var $0=$i; //@line 124 "getopt.cpp"
    var $1=$num_addr; //@line 124 "getopt.cpp"
    var $shr2=($1) >> 1; //@line 124 "getopt.cpp"
    var $cmp3=((($0))|0) < ((($shr2))|0); //@line 124 "getopt.cpp"
    if ($cmp3) { __label__ = 1;; } else { __label__ = 2;; } //@line 124 "getopt.cpp"
    $for_body$$for_end$2: do { 
      if (__label__ == 1) {
        while(1) { 
  
          var $2=$i; //@line 126 "getopt.cpp"
          var $3=$argv_addr; //@line 126 "getopt.cpp"
          var $arrayidx=(($3+4*$2)&4294967295); //@line 126 "getopt.cpp"
          var $4=HEAP32[(($arrayidx)>>2)]; //@line 126 "getopt.cpp"
          $tmp=$4; //@line 126 "getopt.cpp"
          var $5=$num_addr; //@line 127 "getopt.cpp"
          var $6=$i; //@line 127 "getopt.cpp"
          var $_neg=(((-($6)))&4294967295);
          var $sub=((($5)-1)&4294967295); //@line 127 "getopt.cpp"
          var $sub1=((($sub)+($_neg))&4294967295); //@line 127 "getopt.cpp"
          var $7=$argv_addr; //@line 127 "getopt.cpp"
          var $arrayidx2=(($7+4*$sub1)&4294967295); //@line 127 "getopt.cpp"
          var $8=HEAP32[(($arrayidx2)>>2)]; //@line 127 "getopt.cpp"
          var $9=$i; //@line 127 "getopt.cpp"
          var $10=$argv_addr; //@line 127 "getopt.cpp"
          var $arrayidx3=(($10+4*$9)&4294967295); //@line 127 "getopt.cpp"
          HEAP32[(($arrayidx3)>>2)]=$8; //@line 127 "getopt.cpp"
          var $11=$tmp; //@line 128 "getopt.cpp"
          var $12=$num_addr; //@line 128 "getopt.cpp"
          var $13=$i; //@line 128 "getopt.cpp"
          var $_neg1=(((-($13)))&4294967295);
          var $sub4=((($12)-1)&4294967295); //@line 128 "getopt.cpp"
          var $sub5=((($sub4)+($_neg1))&4294967295); //@line 128 "getopt.cpp"
          var $14=$argv_addr; //@line 128 "getopt.cpp"
          var $arrayidx6=(($14+4*$sub5)&4294967295); //@line 128 "getopt.cpp"
          HEAP32[(($arrayidx6)>>2)]=$11; //@line 128 "getopt.cpp"
          var $15=$i; //@line 124 "getopt.cpp"
          var $inc=((($15)+1)&4294967295); //@line 124 "getopt.cpp"
          $i=$inc; //@line 124 "getopt.cpp"
          var $16=$i; //@line 124 "getopt.cpp"
          var $17=$num_addr; //@line 124 "getopt.cpp"
          var $shr=($17) >> 1; //@line 124 "getopt.cpp"
          var $cmp=((($16))|0) < ((($shr))|0); //@line 124 "getopt.cpp"
          if ($cmp) { __label__ = 1;continue ; } else { __label__ = 2;break $for_body$$for_end$2; } //@line 124 "getopt.cpp"
        }
      }
    } while(0);
  
    ;
    return; //@line 130 "getopt.cpp"
    return;
  }
  

  function ___strtok_r($s, $delim, $lasts, $skip_leading_delim) {
    ;
    var __label__;
  
    var $retval;
    var $s_addr;
    var $delim_addr;
    var $lasts_addr;
    var $skip_leading_delim_addr;
    var $spanp;
    var $c;
    var $sc;
    var $tok;
    $s_addr=$s;
    $delim_addr=$delim;
    $lasts_addr=$lasts;
    $skip_leading_delim_addr=$skip_leading_delim;
    var $0=$s_addr; //@line 48 "strtok.cpp"
    var $cmp=((($0))|0)==0; //@line 48 "strtok.cpp"
    if ($cmp) { __label__ = 1;; } else { __label__ = 3;; } //@line 48 "strtok.cpp"
    $land_lhs_true$$cont$2: do { 
      if (__label__ == 1) {
  
        var $1=$lasts_addr; //@line 48 "strtok.cpp"
        var $2=HEAP32[(($1)>>2)]; //@line 48 "strtok.cpp"
        $s_addr=$2; //@line 48 "strtok.cpp"
        var $cmp1=((($2))|0)==0; //@line 48 "strtok.cpp"
        if (!($cmp1)) { __label__ = 3;break $land_lhs_true$$cont$2; } //@line 48 "strtok.cpp"
  
        $retval=0; //@line 49 "strtok.cpp"
        __label__ = 18;break $land_lhs_true$$cont$2; //@line 49 "strtok.cpp"
      }
    } while(0);
    if (__label__ == 3) {
      $cont$6: while(1) { 
  
        var $3=$s_addr; //@line 55 "strtok.cpp"
        var $incdec_ptr=(($3+1)&4294967295); //@line 55 "strtok.cpp"
        $s_addr=$incdec_ptr; //@line 55 "strtok.cpp"
        var $4=HEAP8[($3)]; //@line 55 "strtok.cpp"
        var $conv=(tempInt=(($4)),(tempInt>=128?tempInt-256:tempInt)); //@line 55 "strtok.cpp"
        $c=$conv; //@line 55 "strtok.cpp"
        var $5=$delim_addr; //@line 56 "strtok.cpp"
        $spanp=$5; //@line 56 "strtok.cpp"
        ; //@line 56 "strtok.cpp"
        while(1) { 
  
          var $6=$spanp; //@line 56 "strtok.cpp"
          var $incdec_ptr2=(($6+1)&4294967295); //@line 56 "strtok.cpp"
          $spanp=$incdec_ptr2; //@line 56 "strtok.cpp"
          var $7=HEAP8[($6)]; //@line 56 "strtok.cpp"
          var $conv3=(tempInt=(($7)),(tempInt>=128?tempInt-256:tempInt)); //@line 56 "strtok.cpp"
          $sc=$conv3; //@line 56 "strtok.cpp"
          var $cmp4=((($conv3))|0)!=0; //@line 56 "strtok.cpp"
          var $8=$c; //@line 57 "strtok.cpp"
          if (!($cmp4)) { __label__ = 8;break $cont$6; } //@line 56 "strtok.cpp"
  
          var $9=$sc; //@line 57 "strtok.cpp"
          var $cmp5=((($8))|0)==((($9))|0); //@line 57 "strtok.cpp"
          if ($cmp5) { __label__ = 6;break ; } else { __label__ = 4;continue ; } //@line 57 "strtok.cpp"
        }
  
        var $10=$skip_leading_delim_addr; //@line 58 "strtok.cpp"
        var $tobool=((($10))|0)!=0; //@line 58 "strtok.cpp"
        if ($tobool) { __label__ = 3;continue $cont$6; } else { __label__ = 7;break $cont$6; } //@line 58 "strtok.cpp"
      }
      if (__label__ == 8) {
  
        var $cmp9=((($8))|0)==0; //@line 69 "strtok.cpp"
        if ($cmp9) { __label__ = 9;; } else { __label__ = 10;; } //@line 69 "strtok.cpp"
        if (__label__ == 9) {
  
          var $15=$lasts_addr; //@line 70 "strtok.cpp"
          HEAP32[(($15)>>2)]=0; //@line 70 "strtok.cpp"
          $retval=0; //@line 71 "strtok.cpp"
          ; //@line 71 "strtok.cpp"
        }
        else if (__label__ == 10) {
  
          var $16=$s_addr; //@line 73 "strtok.cpp"
          var $add_ptr12=(($16+-1)&4294967295); //@line 73 "strtok.cpp"
          $tok=$add_ptr12; //@line 73 "strtok.cpp"
          ; //@line 79 "strtok.cpp"
          $for_cond13$17: while(1) { 
  
            var $17=$s_addr; //@line 80 "strtok.cpp"
            var $incdec_ptr14=(($17+1)&4294967295); //@line 80 "strtok.cpp"
            $s_addr=$incdec_ptr14; //@line 80 "strtok.cpp"
            var $18=HEAP8[($17)]; //@line 80 "strtok.cpp"
            var $conv15=(tempInt=(($18)),(tempInt>=128?tempInt-256:tempInt)); //@line 80 "strtok.cpp"
            $c=$conv15; //@line 80 "strtok.cpp"
            var $19=$delim_addr; //@line 81 "strtok.cpp"
            $spanp=$19; //@line 81 "strtok.cpp"
            ; //@line 82 "strtok.cpp"
            while(1) { 
  
              var $20=$spanp; //@line 83 "strtok.cpp"
              var $incdec_ptr16=(($20+1)&4294967295); //@line 83 "strtok.cpp"
              $spanp=$incdec_ptr16; //@line 83 "strtok.cpp"
              var $21=HEAP8[($20)]; //@line 83 "strtok.cpp"
              var $conv17=(tempInt=(($21)),(tempInt>=128?tempInt-256:tempInt)); //@line 83 "strtok.cpp"
              $sc=$conv17; //@line 83 "strtok.cpp"
              var $22=$c; //@line 83 "strtok.cpp"
              var $cmp18=((($conv17))|0)==((($22))|0); //@line 83 "strtok.cpp"
              if ($cmp18) { __label__ = 13;break $for_cond13$17; } //@line 83 "strtok.cpp"
  
              var $28=$sc; //@line 91 "strtok.cpp"
              var $cmp26=((($28))|0)!=0; //@line 91 "strtok.cpp"
              if ($cmp26) { __label__ = 12;continue ; } else { __label__ = 11;continue $for_cond13$17; } //@line 91 "strtok.cpp"
            }
          }
  
          var $23=$c; //@line 84 "strtok.cpp"
          var $cmp20=((($23))|0)==0; //@line 84 "strtok.cpp"
          if ($cmp20) { __label__ = 14;; } else { __label__ = 15;; } //@line 84 "strtok.cpp"
          if (__label__ == 14) {
  
            $s_addr=0; //@line 85 "strtok.cpp"
            ; //@line 85 "strtok.cpp"
          }
          else if (__label__ == 15) {
  
            var $24=$s_addr; //@line 87 "strtok.cpp"
            var $arrayidx23=(($24+-1)&4294967295); //@line 87 "strtok.cpp"
            HEAP8[($arrayidx23)]=0; //@line 87 "strtok.cpp"
            ;
          }
  
          var $25=$s_addr; //@line 88 "strtok.cpp"
          var $26=$lasts_addr; //@line 88 "strtok.cpp"
          HEAP32[(($26)>>2)]=$25; //@line 88 "strtok.cpp"
          var $27=$tok; //@line 89 "strtok.cpp"
          $retval=$27; //@line 89 "strtok.cpp"
          ; //@line 89 "strtok.cpp"
        }
      }
      else if (__label__ == 7) {
  
        var $11=$s_addr; //@line 62 "strtok.cpp"
        var $12=$lasts_addr; //@line 62 "strtok.cpp"
        HEAP32[(($12)>>2)]=$11; //@line 62 "strtok.cpp"
        var $13=$s_addr; //@line 63 "strtok.cpp"
        var $arrayidx=(($13+-1)&4294967295); //@line 63 "strtok.cpp"
        HEAP8[($arrayidx)]=0; //@line 63 "strtok.cpp"
        var $14=$s_addr; //@line 64 "strtok.cpp"
        var $add_ptr=(($14+-1)&4294967295); //@line 64 "strtok.cpp"
        $retval=$add_ptr; //@line 64 "strtok.cpp"
        ; //@line 64 "strtok.cpp"
      }
    }
  
    var $29=$retval; //@line 94 "strtok.cpp"
    ;
    return $29; //@line 94 "strtok.cpp"
    return null;
  }
  

  function _strtok_r($s, $delim, $lasts) {
    ;
    var __label__;
  
    var $s_addr;
    var $delim_addr;
    var $lasts_addr;
    $s_addr=$s;
    $delim_addr=$delim;
    $lasts_addr=$lasts;
    var $0=$s_addr; //@line 101 "strtok.cpp"
    var $1=$delim_addr; //@line 101 "strtok.cpp"
    var $2=$lasts_addr; //@line 101 "strtok.cpp"
    var $call=___strtok_r($0, $1, $2, 1); //@line 101 "strtok.cpp"
    ;
    return $call; //@line 101 "strtok.cpp"
    return null;
  }
  

  function _strtok($s, $delim) {
    var __stackBase__  = STACKTOP; STACKTOP += 4; _memset(__stackBase__, 0, 4);
    var __label__;
  
    var $s_addr;
    var $delim_addr;
    var $chr=__stackBase__;
    $s_addr=$s;
    $delim_addr=$delim;
    var $0=$s_addr; //@line 197 "strtok.cpp"
    var $1=$delim_addr; //@line 197 "strtok.cpp"
    var $call=___strtok_r($0, $1, $chr, 1); //@line 197 "strtok.cpp"
    STACKTOP = __stackBase__;
    return $call; //@line 197 "strtok.cpp"
    return null;
  }
  

// === Auto-generated postamble setup entry stuff ===

Module.callMain = function callMain(args) {
  var argc = args.length+1;
  function pad() {
    for (var i = 0; i < 4-1; i++) {
      argv.push(0);
    }
  }
  var argv = [allocate(intArrayFromString("/bin/this.program"), 'i8', ALLOC_STATIC) ];
  pad();
  for (var i = 0; i < argc-1; i = i + 1) {
    argv.push(allocate(intArrayFromString(args[i]), 'i8', ALLOC_STATIC));
    pad();
  }
  argv.push(0);
  argv = allocate(argv, 'i32', ALLOC_STATIC);

  return _main(argc, argv, 0);
}

function run(args) {
  args = args || Module.arguments;


_path_home=allocate(150, "i8", ALLOC_STATIC);
_filetype=allocate(5, "i8", ALLOC_STATIC);
_wavefile=allocate(200, "i8", ALLOC_STATIC);
_uri_callback=allocate(1, "i32 (i32, i8*, i8*)*", ALLOC_STATIC);
_phoneme_callback=allocate(1, "i32 (i8*)*", ALLOC_STATIC);
_f_wave=allocate(1, "%struct.__sFILE*", ALLOC_STATIC);
_quiet=allocate(1, "i32", ALLOC_STATIC);
_samples_total=allocate(1, "i32", ALLOC_STATIC);
_samples_split=allocate(1, "i32", ALLOC_STATIC);
_wavefile_count=allocate(1, "i32", ALLOC_STATIC);
_end_of_sentence=allocate(1, "i32", ALLOC_STATIC);
_voice_pcnt=allocate(60, "i16", ALLOC_STATIC);
__ZZ13DisplayVoicesP7__sFILEPcE7genders=allocate([32,77,70,32] /*  MF  */, "i8", ALLOC_STATIC);
__str1=allocate([80,116,121,32,76,97,110,103,117,97,103,101,32,65,103,101,47,71,101,110,100,101,114,32,86,111,105,99,101,78,97,109,101,32,32,32,32,32,32,32,70,105,108,101,32,32,32,32,32,32,32,32,79,116,104,101,114,32,76,97,110,103,115,10,0] /* Pty Language Age/Gen */, "i8", ALLOC_STATIC);
__str3=allocate([37,51,100,0] /* %3d\00 */, "i8", ALLOC_STATIC);
__str4=allocate([37,50,100,32,32,37,45,49,50,115,37,115,37,99,32,32,37,45,49,55,115,32,37,45,49,49,115,32,0] /* %2d  %-12s%s%c  %-17 */, "i8", ALLOC_STATIC);
__str5=allocate([40,37,115,32,37,100,41,0] /* (%s %d)\00 */, "i8", ALLOC_STATIC);
__ZZ4mainE12long_options=allocate([0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 104, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 256, 0, 0, 0, 0, 0, 0, 0, 2, 0, 0, 0, 0, 0, 0, 0, 257, 0, 0, 0, 0, 0, 0, 0, 2, 0, 0, 0, 0, 0, 0, 0, 258, 0, 0, 0, 0, 0, 0, 0, 2, 0, 0, 0, 0, 0, 0, 0, 259, 0, 0, 0, 0, 0, 0, 0, 2, 0, 0, 0, 0, 0, 0, 0, 260, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 261, 0, 0, 0, 0, 0, 0, 0, 2, 0, 0, 0, 0, 0, 0, 0, 262, 0, 0, 0, 0, 0, 0, 0, 1, 0, 0, 0, 0, 0, 0, 0, 263, 0, 0, 0, 0, 0, 0, 0, 1, 0, 0, 0, 0, 0, 0, 0, 264, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 265, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 266, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0], ["i8*",0,0,0,"i32",0,0,0,"i32*",0,0,0,"i32",0,0,0,"i8*",0,0,0,"i32",0,0,0,"i32*",0,0,0,"i32",0,0,0,"i8*",0,0,0,"i32",0,0,0,"i32*",0,0,0,"i32",0,0,0,"i8*",0,0,0,"i32",0,0,0,"i32*",0,0,0,"i32",0,0,0,"i8*",0,0,0,"i32",0,0,0,"i32*",0,0,0,"i32",0,0,0,"i8*",0,0,0,"i32",0,0,0,"i32*",0,0,0,"i32",0,0,0,"i8*",0,0,0,"i32",0,0,0,"i32*",0,0,0,"i32",0,0,0,"i8*",0,0,0,"i32",0,0,0,"i32*",0,0,0,"i32",0,0,0,"i8*",0,0,0,"i32",0,0,0,"i32*",0,0,0,"i32",0,0,0,"i8*",0,0,0,"i32",0,0,0,"i32*",0,0,0,"i32",0,0,0,"i8*",0,0,0,"i32",0,0,0,"i32*",0,0,0,"i32",0,0,0,"i8*",0,0,0,"i32",0,0,0,"i32*",0,0,0,"i32",0,0,0,"i8*",0,0,0,"i32",0,0,0,"i32*",0,0,0,"i32",0,0,0], ALLOC_STATIC);
__str6=allocate([104,101,108,112,0] /* help\00 */, "i8", ALLOC_STATIC);
__str7=allocate([115,116,100,105,110,0] /* stdin\00 */, "i8", ALLOC_STATIC);
__str8=allocate([99,111,109,112,105,108,101,45,100,101,98,117,103,0] /* compile-debug\00 */, "i8", ALLOC_STATIC);
__str9=allocate([99,111,109,112,105,108,101,0] /* compile\00 */, "i8", ALLOC_STATIC);
__str10=allocate([112,117,110,99,116,0] /* punct\00 */, "i8", ALLOC_STATIC);
__str11=allocate([118,111,105,99,101,115,0] /* voices\00 */, "i8", ALLOC_STATIC);
__str12=allocate([115,116,100,111,117,116,0] /* stdout\00 */, "i8", ALLOC_STATIC);
__str13=allocate([115,112,108,105,116,0] /* split\00 */, "i8", ALLOC_STATIC);
__str14=allocate([112,97,116,104,0] /* path\00 */, "i8", ALLOC_STATIC);
__str15=allocate([112,104,111,110,111,117,116,0] /* phonout\00 */, "i8", ALLOC_STATIC);
__str16=allocate([112,104,111,0] /* pho\00 */, "i8", ALLOC_STATIC);
__str17=allocate([105,112,97,0] /* ipa\00 */, "i8", ALLOC_STATIC);
__str18=allocate([70,97,105,108,101,100,32,116,111,32,114,101,97,100,32,0] /* Failed to read \00 */, "i8", ALLOC_STATIC);
__str19=allocate([97,58,98,58,102,58,103,58,104,107,58,108,58,112,58,113,115,58,118,58,119,58,120,88,109,122,0] /* a:b:f:g:hk:l:p:qs:v: */, "i8", ALLOC_STATIC);
__str21=allocate([10,115,112,101,97,107,32,116,101,120,116,45,116,111,45,115,112,101,101,99,104,58,32,37,115,32,32,32,68,97,116,97,32,97,116,58,32,37,115,10,37,115,0] /* \0Aspeak text-to-spe */, "i8", ALLOC_STATIC);
__str23=allocate([67,97,110,39,116,32,119,114,105,116,101,32,116,111,58,32,37,115,10,0] /* Can't write to: %s\0 */, "i8", ALLOC_STATIC);
__str25=allocate([37,115,118,111,105,99,101,32,39,37,115,39,10,0] /* %svoice '%s'\0A\00 */, "i8", ALLOC_STATIC);
__str27=allocate([37,115,102,105,108,101,32,39,37,115,39,10,0] /* %sfile '%s'\0A\00 */, "i8", ALLOC_STATIC);
__str28=allocate([67,97,110,39,116,32,119,114,105,116,101,32,116,111,32,111,117,116,112,117,116,32,102,105,108,101,32,39,37,115,39,10,39,0] /* Can't write to outpu */, "i8", ALLOC_STATIC);
__str29=allocate([45,119,32,111,112,116,105,111,110,32,109,117,115,116,32,98,101,32,117,115,101,100,32,98,101,99,97,117,115,101,32,116,104,101,32,112,114,111,103,114,97,109,32,119,97,115,32,98,117,105,108,116,32,119,105,116,104,111,117,116,32,97,32,115,111,117,110,100,32,105,110,116,101,114,102,97,99,101,10,0] /* -w option must be us */, "i8", ALLOC_STATIC);
__str30=allocate([37,115,95,37,46,50,100,37,115,0] /* %s_%.2d%s\00 */, "i8", ALLOC_STATIC);
__ZZL12OpenWaveFilePKciE8wave_hdr=allocate([82,73,70,70,36,240,255,127,87,65,86,69,102,109,116,32,16,0,0,0,1,0,1,0,9,61,0,0,18,122,0,0,2,0,16,0,100,97,116,97,0,240,255,127] /* RIFF$\F0\FF\7FWAVEfm */, "i8", ALLOC_STATIC);
__str32=allocate([101,110,95,85,83,46,85,84,70,45,56,0] /* en_US.UTF-8\00 */, "i8", ALLOC_STATIC);
__str33=allocate([85,84,70,45,56,0] /* UTF-8\00 */, "i8", ALLOC_STATIC);
__str35=allocate([70,97,105,108,101,100,32,116,111,32,108,111,97,100,32,101,115,112,101,97,107,45,100,97,116,97,10,0] /* Failed to load espea */, "i8", ALLOC_STATIC);
__str36=allocate([87,114,111,110,103,32,118,101,114,115,105,111,110,32,111,102,32,101,115,112,101,97,107,45,100,97,116,97,32,48,120,37,120,32,40,101,120,112,101,99,116,115,32,48,120,37,120,41,32,97,116,32,37,115,10,0] /* Wrong version of esp */, "i8", ALLOC_STATIC);
__str37=allocate([10,115,112,101,97,107,32,91,111,112,116,105,111,110,115,93,32,91,34,60,119,111,114,100,115,62,34,93,10,10,45,102,32,60,116,101,120,116,32,102,105,108,101,62,32,32,32,84,101,120,116,32,102,105,108,101,32,116,111,32,115,112,101,97,107,10,45,45,115,116,100,105,110,32,32,32,32,82,101,97,100,32,116,101,120,116,32,105,110,112,117,116,32,102,114,111,109,32,115,116,100,105,110,32,105,110,115,116,101,97,100,32,111,102,32,97,32,102,105,108,101,10,10,73,102,32,110,101,105,116,104,101,114,32,45,102,32,110,111,114,32,45,45,115,116,100,105,110,44,32,116,104,101,110,32,60,119,111,114,100,115,62,32,97,114,101,32,115,112,111,107,101,110,44,32,111,114,32,105,102,32,110,111,110,101,32,116,104,101,110,32,116,101,120,116,10,105,115,32,115,112,111,107,101,110,32,102,114,111,109,32,115,116,100,105,110,44,32,101,97,99,104,32,108,105,110,101,32,115,101,112,97,114,97,116,101,108,121,46,10,10,45,97,32,60,105,110,116,101,103,101,114,62,10,9,32,32,32,65,109,112,108,105,116,117,100,101,44,32,48,32,116,111,32,50,48,48,44,32,100,101,102,97,117,108,116,32,105,115,32,49,48,48,10,45,103,32,60,105,110,116,101,103,101,114,62,10,9,32,32,32,87,111,114,100,32,103,97,112,46,32,80,97,117,115,101,32,98,101,116,119,101,101,110,32,119,111,114,100,115,44,32,117,110,105,116,115,32,111,102,32,49,48,109,83,32,97,116,32,116,104,101,32,100,101,102,97,117,108,116,32,115,112,101,101,100,10,45,107,32,60,105,110,116,101,103,101,114,62,10,9,32,32,32,73,110,100,105,99,97,116,101,32,99,97,112,105,116,97,108,32,108,101,116,116,101,114,115,32,119,105,116,104,58,32,49,61,115,111,117,110,100,44,32,50,61,116,104,101,32,119,111,114,100,32,34,99,97,112,105,116,97,108,115,34,44,10,9,32,32,32,104,105,103,104,101,114,32,118,97,108,117,101,115,32,105,110,100,105,99,97,116,101,32,97,32,112,105,116,99,104,32,105,110,99,114,101,97,115,101,32,40,116,114,121,32,45,107,50,48,41,46,10,45,108,32,60,105,110,116,101,103,101,114,62,10,9,32,32,32,76,105,110,101,32,108,101,110,103,116,104,46,32,73,102,32,110,111,116,32,122,101,114,111,32,40,119,104,105,99,104,32,105,115,32,116,104,101,32,100,101,102,97,117,108,116,41,44,32,99,111,110,115,105,100,101,114,10,9,32,32,32,108,105,110,101,115,32,108,101,115,115,32,116,104,97,110,32,116,104,105,115,32,108,101,110,103,116,104,32,97,115,32,101,110,100,45,111,102,45,99,108,97,117,115,101,10,45,112,32,60,105,110,116,101,103,101,114,62,10,9,32,32,32,80,105,116,99,104,32,97,100,106,117,115,116,109,101,110,116,44,32,48,32,116,111,32,57,57,44,32,100,101,102,97,117,108,116,32,105,115,32,53,48,10,45,115,32,60,105,110,116,101,103,101,114,62,10,9,32,32,32,83,112,101,101,100,32,105,110,32,119,111,114,100,115,32,112,101,114,32,109,105,110,117,116,101,44,32,56,48,32,116,111,32,52,53,48,44,32,100,101,102,97,117,108,116,32,105,115,32,49,55,53,10,45,118,32,60,118,111,105,99,101,32,110,97,109,101,62,10,9,32,32,32,85,115,101,32,118,111,105,99,101,32,102,105,108,101,32,111,102,32,116,104,105,115,32,110,97,109,101,32,102,114,111,109,32,101,115,112,101,97,107,45,100,97,116,97,47,118,111,105,99,101,115,10,45,119,32,60,119,97,118,101,32,102,105,108,101,32,110,97,109,101,62,10,9,32,32,32,87,114,105,116,101,32,115,112,101,101,99,104,32,116,111,32,116,104,105,115,32,87,65,86,32,102,105,108,101,44,32,114,97,116,104,101,114,32,116,104,97,110,32,115,112,101,97,107,105,110,103,32,105,116,32,100,105,114,101,99,116,108,121,10,45,98,9,32,32,32,73,110,112,117,116,32,116,101,120,116,32,101,110,99,111,100,105,110,103,44,32,49,61,85,84,70,56,44,32,50,61,56,32,98,105,116,44,32,52,61,49,54,32,98,105,116,32,10,45,109,9,32,32,32,73,110,116,101,114,112,114,101,116,32,83,83,77,76,32,109,97,114,107,117,112,44,32,97,110,100,32,105,103,110,111,114,101,32,111,116,104,101,114,32,60,32,62,32,116,97,103,115,10,45,113,9,32,32,32,81,117,105,101,116,44,32,100,111,110,39,116,32,112,114,111,100,117,99,101,32,97,110,121,32,115,112,101,101,99,104,32,40,109,97,121,32,98,101,32,117,115,101,102,117,108,32,119,105,116,104,32,45,120,41,10,45,120,9,32,32,32,87,114,105,116,101,32,112,104,111,110,101,109,101,32,109,110,101,109,111,110,105,99,115,32,116,111,32,115,116,100,111,117,116,10,45,88,9,32,32,32,87,114,105,116,101,32,112,104,111,110,101,109,101,115,32,109,110,101,109,111,110,105,99,115,32,97,110,100,32,116,114,97,110,115,108,97,116,105,111,110,32,116,114,97,99,101,32,116,111,32,115,116,100,111,117,116,10,45,122,9,32,32,32,78,111,32,102,105,110,97,108,32,115,101,110,116,101,110,99,101,32,112,97,117,115,101,32,97,116,32,116,104,101,32,101,110,100,32,111,102,32,116,104,101,32,116,101,120,116,10,45,45,99,111,109,112,105,108,101,61,60,118,111,105,99,101,32,110,97,109,101,62,10,9,32,32,32,67,111,109,112,105,108,101,32,112,114,111,110,117,110,99,105,97,116,105,111,110,32,114,117,108,101,115,32,97,110,100,32,100,105,99,116,105,111,110,97,114,121,32,102,114,111,109,32,116,104,101,32,99,117,114,114,101,110,116,10,9,32,32,32,100,105,114,101,99,116,111,114,121,46,32,60,118,111,105,99,101,32,110,97,109,101,62,32,115,112,101,99,105,102,105,101,115,32,116,104,101,32,108,97,110,103,117,97,103,101,10,45,45,105,112,97,32,32,32,32,32,32,87,114,105,116,101,32,112,104,111,110,101,109,101,115,32,116,111,32,115,116,100,111,117,116,32,117,115,105,110,103,32,73,110,116,101,114,110,97,116,105,111,110,97,108,32,80,104,111,110,101,116,105,99,32,65,108,112,104,97,98,101,116,10,45,45,112,97,116,104,61,34,60,112,97,116,104,62,34,10,9,32,32,32,83,112,101,99,105,102,105,101,115,32,116,104,101,32,100,105,114,101,99,116,111,114,121,32,99,111,110,116,97,105,110,105,110,103,32,116,104,101,32,101,115,112,101,97,107,45,100,97,116,97,32,100,105,114,101,99,116,111,114,121,10,45,45,112,104,111,32,32,32,32,32,32,87,114,105,116,101,32,109,98,114,111,108,97,32,112,104,111,110,101,109,101,32,100,97,116,97,32,40,46,112,104,111,41,32,116,111,32,115,116,100,111,117,116,32,111,114,32,116,111,32,116,104,101,32,102,105,108,101,32,105,110,32,45,45,112,104,111,110,111,117,116,10,45,45,112,104,111,110,111,117,116,61,34,60,102,105,108,101,110,97,109,101,62,34,10,9,32,32,32,87,114,105,116,101,32,112,104,111,110,101,109,101,32,111,117,116,112,117,116,32,102,114,111,109,32,45,120,32,45,88,32,45,45,105,112,97,32,97,110,100,32,45,45,112,104,111,32,116,111,32,116,104,105,115,32,102,105,108,101,10,45,45,112,117,110,99,116,61,34,60,99,104,97,114,97,99,116,101,114,115,62,34,10,9,32,32,32,83,112,101,97,107,32,116,104,101,32,110,97,109,101,115,32,111,102,32,112,117,110,99,116,117,97,116,105,111,110,32,99,104,97,114,97,99,116,101,114,115,32,100,117,114,105,110,103,32,115,112,101,97,107,105,110,103,46,32,32,73,102,10,9,32,32,32,61,60,99,104,97,114,97,99,116,101,114,115,62,32,105,115,32,111,109,105,116,116,101,100,44,32,97,108,108,32,112,117,110,99,116,117,97,116,105,111,110,32,105,115,32,115,112,111,107,101,110,46,10,45,45,115,112,108,105,116,61,34,60,109,105,110,117,116,101,115,62,34,10,9,32,32,32,83,116,97,114,116,115,32,97,32,110,101,119,32,87,65,86,32,102,105,108,101,32,101,118,101,114,121,32,60,109,105,110,117,116,101,115,62,46,32,32,85,115,101,100,32,119,105,116,104,32,45,119,10,45,45,115,116,100,111,117,116,32,32,32,87,114,105,116,101,32,115,112,101,101,99,104,32,111,117,116,112,117,116,32,116,111,32,115,116,100,111,117,116,10,45,45,118,111,105,99,101,115,61,60,108,97,110,103,117,97,103,101,62,10,9,32,32,32,76,105,115,116,32,116,104,101,32,97,118,97,105,108,97,98,108,101,32,118,111,105,99,101,115,32,102,111,114,32,116,104,101,32,115,112,101,99,105,102,105,101,100,32,108,97,110,103,117,97,103,101,46,10,9,32,32,32,73,102,32,60,108,97,110,103,117,97,103,101,62,32,105,115,32,111,109,105,116,116,101,100,44,32,116,104,101,110,32,108,105,115,116,32,97,108,108,32,118,111,105,99,101,115,46,10,0] /* \0Aspeak [options] [ */, "i8", ALLOC_STATIC);
__str38=allocate([37,115,47,101,115,112,101,97,107,45,100,97,116,97,0] /* %s/espeak-data\00 */, "i8", ALLOC_STATIC);
__str39=allocate([69,83,80,69,65,75,95,68,65,84,65,95,80,65,84,72,0] /* ESPEAK_DATA_PATH\00 */, "i8", ALLOC_STATIC);
__str40=allocate([72,79,77,69,0] /* HOME\00 */, "i8", ALLOC_STATIC);
__str41=allocate([47,117,115,114,47,115,104,97,114,101,47,101,115,112,101,97,107,45,100,97,116,97,0] /* /usr/share/espeak-da */, "i8", ALLOC_STATIC);
__ZL5f_log=allocate(1, "%struct.__sFILE*", ALLOC_STATIC);
__str42=allocate([117,110,112,114,0] /* unpr\00 */, "i8", ALLOC_STATIC);
__str143=allocate([119,95,97,108,116,50,0] /* w_alt2\00 */, "i8", ALLOC_STATIC);
__str244=allocate([119,95,97,108,116,51,0] /* w_alt3\00 */, "i8", ALLOC_STATIC);
__str345=allocate([119,95,97,108,116,0] /* w_alt\00 */, "i8", ALLOC_STATIC);
__str446=allocate([112,95,97,108,116,50,0] /* p_alt2\00 */, "i8", ALLOC_STATIC);
__str547=allocate([112,95,97,108,116,51,0] /* p_alt3\00 */, "i8", ALLOC_STATIC);
__str648=allocate([112,95,97,108,116,0] /* p_alt\00 */, "i8", ALLOC_STATIC);
_mnem_rules=allocate([0, 0, 0, 0, 1, 0, 0, 0, 0, 0, 0, 0, 18, 0, 0, 0, 0, 0, 0, 0, 19, 0, 0, 0, 0, 0, 0, 0, 17, 0, 0, 0, 0, 0, 0, 0, 34, 0, 0, 0, 0, 0, 0, 0, 35, 0, 0, 0, 0, 0, 0, 0, 33, 0, 0, 0, 0, 0, 0, 0, -1, 0, 0, 0], ["i8*",0,0,0,"i32",0,0,0,"i8*",0,0,0,"i32",0,0,0,"i8*",0,0,0,"i32",0,0,0,"i8*",0,0,0,"i32",0,0,0,"i8*",0,0,0,"i32",0,0,0,"i8*",0,0,0,"i32",0,0,0,"i8*",0,0,0,"i32",0,0,0,"i8*",0,0,0,"i32",0,0,0], ALLOC_STATIC);
__str749=allocate([36,49,0] /* $1\00 */, "i8", ALLOC_STATIC);
__str850=allocate([36,50,0] /* $2\00 */, "i8", ALLOC_STATIC);
__str951=allocate([36,51,0] /* $3\00 */, "i8", ALLOC_STATIC);
__str1052=allocate([36,52,0] /* $4\00 */, "i8", ALLOC_STATIC);
__str1153=allocate([36,53,0] /* $5\00 */, "i8", ALLOC_STATIC);
__str1254=allocate([36,54,0] /* $6\00 */, "i8", ALLOC_STATIC);
__str1355=allocate([36,55,0] /* $7\00 */, "i8", ALLOC_STATIC);
__str1456=allocate([36,117,0] /* $u\00 */, "i8", ALLOC_STATIC);
__str1557=allocate([36,117,49,0] /* $u1\00 */, "i8", ALLOC_STATIC);
__str1658=allocate([36,117,50,0] /* $u2\00 */, "i8", ALLOC_STATIC);
__str1759=allocate([36,117,51,0] /* $u3\00 */, "i8", ALLOC_STATIC);
__str1860=allocate([36,117,43,0] /* $u+\00 */, "i8", ALLOC_STATIC);
__str1961=allocate([36,117,49,43,0] /* $u1+\00 */, "i8", ALLOC_STATIC);
__str2062=allocate([36,117,50,43,0] /* $u2+\00 */, "i8", ALLOC_STATIC);
__str2163=allocate([36,117,51,43,0] /* $u3+\00 */, "i8", ALLOC_STATIC);
__str2264=allocate([36,112,97,117,115,101,0] /* $pause\00 */, "i8", ALLOC_STATIC);
__str2365=allocate([36,111,110,108,121,0] /* $only\00 */, "i8", ALLOC_STATIC);
__str2466=allocate([36,111,110,108,121,115,0] /* $onlys\00 */, "i8", ALLOC_STATIC);
__str2567=allocate([36,115,116,114,101,110,100,0] /* $strend\00 */, "i8", ALLOC_STATIC);
__str2668=allocate([36,115,116,114,101,110,100,50,0] /* $strend2\00 */, "i8", ALLOC_STATIC);
__str2769=allocate([36,117,110,115,116,114,101,115,115,101,110,100,0] /* $unstressend\00 */, "i8", ALLOC_STATIC);
__str2870=allocate([36,97,116,101,110,100,0] /* $atend\00 */, "i8", ALLOC_STATIC);
__str2971=allocate([36,97,116,115,116,97,114,116,0] /* $atstart\00 */, "i8", ALLOC_STATIC);
__str3072=allocate([36,97,98,98,114,101,118,0] /* $abbrev\00 */, "i8", ALLOC_STATIC);
__str3173=allocate([36,115,116,101,109,0] /* $stem\00 */, "i8", ALLOC_STATIC);
__str3274=allocate([36,100,111,117,98,108,101,0] /* $double\00 */, "i8", ALLOC_STATIC);
__str3375=allocate([36,97,108,116,0] /* $alt\00 */, "i8", ALLOC_STATIC);
__str3476=allocate([36,97,108,116,49,0] /* $alt1\00 */, "i8", ALLOC_STATIC);
__str3577=allocate([36,97,108,116,50,0] /* $alt2\00 */, "i8", ALLOC_STATIC);
__str3678=allocate([36,97,108,116,51,0] /* $alt3\00 */, "i8", ALLOC_STATIC);
__str3779=allocate([36,99,111,109,98,105,110,101,0] /* $combine\00 */, "i8", ALLOC_STATIC);
__str3880=allocate([36,100,111,116,0] /* $dot\00 */, "i8", ALLOC_STATIC);
__str3981=allocate([36,104,97,115,100,111,116,0] /* $hasdot\00 */, "i8", ALLOC_STATIC);
__str4082=allocate([36,109,97,120,51,0] /* $max3\00 */, "i8", ALLOC_STATIC);
__str4183=allocate([36,98,114,107,0] /* $brk\00 */, "i8", ALLOC_STATIC);
__str4284=allocate([36,116,101,120,116,0] /* $text\00 */, "i8", ALLOC_STATIC);
__str43=allocate([36,118,101,114,98,102,0] /* $verbf\00 */, "i8", ALLOC_STATIC);
__str44=allocate([36,118,101,114,98,115,102,0] /* $verbsf\00 */, "i8", ALLOC_STATIC);
__str45=allocate([36,110,111,117,110,102,0] /* $nounf\00 */, "i8", ALLOC_STATIC);
__str46=allocate([36,112,97,115,116,102,0] /* $pastf\00 */, "i8", ALLOC_STATIC);
__str47=allocate([36,118,101,114,98,0] /* $verb\00 */, "i8", ALLOC_STATIC);
__str48=allocate([36,110,111,117,110,0] /* $noun\00 */, "i8", ALLOC_STATIC);
__str49=allocate([36,112,97,115,116,0] /* $past\00 */, "i8", ALLOC_STATIC);
__str50=allocate([36,118,101,114,98,101,120,116,101,110,100,0] /* $verbextend\00 */, "i8", ALLOC_STATIC);
__str51=allocate([36,99,97,112,105,116,97,108,0] /* $capital\00 */, "i8", ALLOC_STATIC);
__str52=allocate([36,97,108,108,99,97,112,115,0] /* $allcaps\00 */, "i8", ALLOC_STATIC);
__str53=allocate([36,97,99,99,101,110,116,0] /* $accent\00 */, "i8", ALLOC_STATIC);
__str54=allocate([36,115,101,110,116,101,110,99,101,0] /* $sentence\00 */, "i8", ALLOC_STATIC);
__str55=allocate([36,63,0] /* $?\00 */, "i8", ALLOC_STATIC);
__str56=allocate([36,116,101,120,116,109,111,100,101,0] /* $textmode\00 */, "i8", ALLOC_STATIC);
__str57=allocate([36,112,104,111,110,101,109,101,109,111,100,101,0] /* $phonememode\00 */, "i8", ALLOC_STATIC);
_mnem_flags=allocate([0, 0, 0, 0, 65, 0, 0, 0, 0, 0, 0, 0, 66, 0, 0, 0, 0, 0, 0, 0, 67, 0, 0, 0, 0, 0, 0, 0, 68, 0, 0, 0, 0, 0, 0, 0, 69, 0, 0, 0, 0, 0, 0, 0, 70, 0, 0, 0, 0, 0, 0, 0, 71, 0, 0, 0, 0, 0, 0, 0, 72, 0, 0, 0, 0, 0, 0, 0, 73, 0, 0, 0, 0, 0, 0, 0, 74, 0, 0, 0, 0, 0, 0, 0, 75, 0, 0, 0, 0, 0, 0, 0, 76, 0, 0, 0, 0, 0, 0, 0, 77, 0, 0, 0, 0, 0, 0, 0, 78, 0, 0, 0, 0, 0, 0, 0, 79, 0, 0, 0, 0, 0, 0, 0, 8, 0, 0, 0, 0, 0, 0, 0, 9, 0, 0, 0, 0, 0, 0, 0, 10, 0, 0, 0, 0, 0, 0, 0, 11, 0, 0, 0, 0, 0, 0, 0, 12, 0, 0, 0, 0, 0, 0, 0, 13, 0, 0, 0, 0, 0, 0, 0, 14, 0, 0, 0, 0, 0, 0, 0, 15, 0, 0, 0, 0, 0, 0, 0, 17, 0, 0, 0, 0, 0, 0, 0, 18, 0, 0, 0, 0, 0, 0, 0, 19, 0, 0, 0, 0, 0, 0, 0, 20, 0, 0, 0, 0, 0, 0, 0, 20, 0, 0, 0, 0, 0, 0, 0, 21, 0, 0, 0, 0, 0, 0, 0, 22, 0, 0, 0, 0, 0, 0, 0, 23, 0, 0, 0, 0, 0, 0, 0, 24, 0, 0, 0, 0, 0, 0, 0, 25, 0, 0, 0, 0, 0, 0, 0, 27, 0, 0, 0, 0, 0, 0, 0, 28, 0, 0, 0, 0, 0, 0, 0, 29, 0, 0, 0, 0, 0, 0, 0, 32, 0, 0, 0, 0, 0, 0, 0, 33, 0, 0, 0, 0, 0, 0, 0, 34, 0, 0, 0, 0, 0, 0, 0, 35, 0, 0, 0, 0, 0, 0, 0, 36, 0, 0, 0, 0, 0, 0, 0, 37, 0, 0, 0, 0, 0, 0, 0, 38, 0, 0, 0, 0, 0, 0, 0, 40, 0, 0, 0, 0, 0, 0, 0, 41, 0, 0, 0, 0, 0, 0, 0, 42, 0, 0, 0, 0, 0, 0, 0, 43, 0, 0, 0, 0, 0, 0, 0, 45, 0, 0, 0, 0, 0, 0, 0, 100, 0, 0, 0, 0, 0, 0, 0, 200, 0, 0, 0, 0, 0, 0, 0, 201, 0, 0, 0, 0, 0, 0, 0, -1, 0, 0, 0], ["i8*",0,0,0,"i32",0,0,0,"i8*",0,0,0,"i32",0,0,0,"i8*",0,0,0,"i32",0,0,0,"i8*",0,0,0,"i32",0,0,0,"i8*",0,0,0,"i32",0,0,0,"i8*",0,0,0,"i32",0,0,0,"i8*",0,0,0,"i32",0,0,0,"i8*",0,0,0,"i32",0,0,0,"i8*",0,0,0,"i32",0,0,0,"i8*",0,0,0,"i32",0,0,0,"i8*",0,0,0,"i32",0,0,0,"i8*",0,0,0,"i32",0,0,0,"i8*",0,0,0,"i32",0,0,0,"i8*",0,0,0,"i32",0,0,0,"i8*",0,0,0,"i32",0,0,0,"i8*",0,0,0,"i32",0,0,0,"i8*",0,0,0,"i32",0,0,0,"i8*",0,0,0,"i32",0,0,0,"i8*",0,0,0,"i32",0,0,0,"i8*",0,0,0,"i32",0,0,0,"i8*",0,0,0,"i32",0,0,0,"i8*",0,0,0,"i32",0,0,0,"i8*",0,0,0,"i32",0,0,0,"i8*",0,0,0,"i32",0,0,0,"i8*",0,0,0,"i32",0,0,0,"i8*",0,0,0,"i32",0,0,0,"i8*",0,0,0,"i32",0,0,0,"i8*",0,0,0,"i32",0,0,0,"i8*",0,0,0,"i32",0,0,0,"i8*",0,0,0,"i32",0,0,0,"i8*",0,0,0,"i32",0,0,0,"i8*",0,0,0,"i32",0,0,0,"i8*",0,0,0,"i32",0,0,0,"i8*",0,0,0,"i32",0,0,0,"i8*",0,0,0,"i32",0,0,0,"i8*",0,0,0,"i32",0,0,0,"i8*",0,0,0,"i32",0,0,0,"i8*",0,0,0,"i32",0,0,0,"i8*",0,0,0,"i32",0,0,0,"i8*",0,0,0,"i32",0,0,0,"i8*",0,0,0,"i32",0,0,0,"i8*",0,0,0,"i32",0,0,0,"i8*",0,0,0,"i32",0,0,0,"i8*",0,0,0,"i32",0,0,0,"i8*",0,0,0,"i32",0,0,0,"i8*",0,0,0,"i32",0,0,0,"i8*",0,0,0,"i32",0,0,0,"i8*",0,0,0,"i32",0,0,0,"i8*",0,0,0,"i32",0,0,0,"i8*",0,0,0,"i32",0,0,0,"i8*",0,0,0,"i32",0,0,0,"i8*",0,0,0,"i32",0,0,0], ALLOC_STATIC);
__ZZ22print_dictionary_flagsPjE3buf=allocate(20, "i8", ALLOC_STATIC);
__str59=allocate([37,115,32,32,48,120,37,120,47,37,120,0] /* %s  0x%x/%x\00 */, "i8", ALLOC_STATIC);
__ZZ10DecodeRulePKciPciE6output=allocate(60, "i8", ALLOC_STATIC);
__ZZ10DecodeRulePKciPciE7symbols=allocate([32,32,32,32,32,32,32,32,32,32,38,37,43,35,83,68,90,65,76,33,32,64,63,74,78,75,86,63,84,88,63,87] /*           &%+#SDZAL! */, "i8", ALLOC_STATIC);
__ZZ10DecodeRulePKciPciE10symbols_lg=allocate([65,66,67,72,70,71,89] /* ABCHFGY */, "i8", ALLOC_STATIC);
__str60=allocate([32,40,0] /*  (\00 */, "i8", ALLOC_STATIC);
__str61=allocate([101,105,112,118,100,102,113,32,116,98,97,32,0] /* eipvdfq tba \00 */, "i8", ALLOC_STATIC);
__str62=allocate([37,99,37,100,0] /* %c%d\00 */, "i8", ALLOC_STATIC);
__str64=allocate([37,53,100,58,9,0] /* %5d:\09\00 */, "i8", ALLOC_STATIC);
__str65=allocate([63,37,100,32,0] /* ?%d \00 */, "i8", ALLOC_STATIC);
__ZL11error_count=allocate(1, "i32", ALLOC_STATIC);
__ZL21error_need_dictionary=allocate(1, "i32", ALLOC_STATIC);
__ZL19letterGroupsDefined=allocate(26, "i8", ALLOC_STATIC);
__ZL10debug_flag=allocate(1, "i32", ALLOC_STATIC);
__str66=allocate([37,115,37,115,95,0] /* %s%s_\00 */, "i8", ALLOC_STATIC);
__str67=allocate([37,115,114,117,108,101,115,46,116,120,116,0] /* %srules.txt\00 */, "i8", ALLOC_STATIC);
__str69=allocate([37,115,114,117,108,101,115,0] /* %srules\00 */, "i8", ALLOC_STATIC);
__str71=allocate([119,98,43,0] /* wb+\00 */, "i8", ALLOC_STATIC);
__str72=allocate([37,115,37,99,116,101,109,112,0] /* %s%ctemp\00 */, "i8", ALLOC_STATIC);
__str73=allocate([85,115,105,110,103,32,112,104,111,110,101,109,101,116,97,98,108,101,58,32,39,37,115,39,10,0] /* Using phonemetable:  */, "i8", ALLOC_STATIC);
__str74=allocate([114,111,111,116,115,0] /* roots\00 */, "i8", ALLOC_STATIC);
__str75=allocate([108,105,115,116,0] /* list\00 */, "i8", ALLOC_STATIC);
__str76=allocate([108,105,115,116,120,0] /* listx\00 */, "i8", ALLOC_STATIC);
__str77=allocate([101,120,116,114,97,0] /* extra\00 */, "i8", ALLOC_STATIC);
__str78=allocate([67,111,109,112,105,108,105,110,103,58,32,39,37,115,39,10,0] /* Compiling: '%s'\0A\0 */, "i8", ALLOC_STATIC);
__ZL7linenum=allocate(1, "i32", ALLOC_STATIC);
__ZL10group_name=allocate(13, "i8", ALLOC_STATIC);
__str79=allocate([119,98,0] /* wb\00 */, "i8", ALLOC_STATIC);
__ZL9group3_ix=allocate(1, "i32", ALLOC_STATIC);
__str81=allocate([46,76,0] /* .L\00 */, "i8", ALLOC_STATIC);
__str82=allocate([46,114,101,112,108,97,99,101,0] /* .replace\00 */, "i8", ALLOC_STATIC);
__str83=allocate([46,103,114,111,117,112,0] /* .group\00 */, "i8", ALLOC_STATIC);
__str84=allocate([48,120,37,120,0] /* 0x%x\00 */, "i8", ALLOC_STATIC);
__str85=allocate([37,53,100,58,32,71,114,111,117,112,32,110,97,109,101,32,108,111,110,103,101,114,32,116,104,97,110,32,50,32,98,121,116,101,115,32,40,85,84,70,56,41,0] /* %5d: Group name long */, "i8", ALLOC_STATIC);
__str89=allocate([9,37,100,32,114,117,108,101,115,44,32,37,100,32,103,114,111,117,112,115,32,40,37,100,41,10,10,0] /* \09%d rules, %d grou */, "i8", ALLOC_STATIC);
__ZL9rule_cond=allocate(80, "i8", ALLOC_STATIC);
__ZL8rule_pre=allocate(80, "i8", ALLOC_STATIC);
__ZL9rule_post=allocate(80, "i8", ALLOC_STATIC);
__ZL10rule_match=allocate(80, "i8", ALLOC_STATIC);
__ZL13rule_phonemes=allocate(80, "i8", ALLOC_STATIC);
__str90=allocate([37,53,100,58,32,83,121,110,116,97,120,32,101,114,114,111,114,46,32,83,112,97,99,101,32,97,102,116,101,114,32,40,10,0] /* %5d: Syntax error. S */, "i8", ALLOC_STATIC);
__str91=allocate([36,103,114,111,117,112,0] /* $group\00 */, "i8", ALLOC_STATIC);
__str92=allocate([37,53,100,58,32,83,121,110,116,97,120,32,101,114,114,111,114,10,0] /* %5d: Syntax error\0A */, "i8", ALLOC_STATIC);
__str93=allocate([37,53,100,58,32,66,97,100,32,112,104,111,110,101,109,101,32,91,37,99,93,32,105,110,32,37,115,10,0] /* %5d: Bad phoneme [%c */, "i8", ALLOC_STATIC);
__str94=allocate([37,53,100,58,32,87,114,111,110,103,32,105,110,105,116,105,97,108,32,108,101,116,116,101,114,115,32,39,37,115,39,32,102,111,114,32,103,114,111,117,112,32,39,37,115,39,10,0] /* %5d: Wrong initial l */, "i8", ALLOC_STATIC);
__str95=allocate([37,53,100,58,32,98,97,100,32,99,111,110,100,105,116,105,111,110,32,110,117,109,98,101,114,32,63,37,100,10,0] /* %5d: bad condition n */, "i8", ALLOC_STATIC);
__ZZL16copy_rule_stringPcRiE6outbuf=allocate(20, "i8*", ALLOC_STATIC);
__ZZL16copy_rule_stringPcRiE10next_state=allocate([2, 0, 0, 0, 2, 0, 0, 0, 4, 0, 0, 0, 4, 0, 0, 0, 4, 0, 0, 0], ["i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0], ALLOC_STATIC);
__ZZL16copy_rule_stringPcRiE16lettergp_letters=allocate([0,1,2,0,0,4,5,3,6] /* \00\01\02\00\00\04\0 */, "i8", ALLOC_STATIC);
__str97=allocate([37,53,100,58,32,69,120,112,101,99,116,101,100,32,50,32,100,105,103,105,116,115,32,97,102,116,101,114,32,39,76,39,10,0] /* %5d: Expected 2 digi */, "i8", ALLOC_STATIC);
__str98=allocate([37,53,100,58,32,76,101,116,116,101,114,32,103,114,111,117,112,32,76,37,46,50,100,32,110,111,116,32,100,101,102,105,110,101,100,10,0] /* %5d: Letter group L% */, "i8", ALLOC_STATIC);
__str99=allocate([37,53,100,58,32,36,32,99,111,109,109,97,110,100,32,110,111,116,32,114,101,99,111,103,110,105,122,101,100,10,0] /* %5d: $ command not r */, "i8", ALLOC_STATIC);
__str100=allocate([37,53,100,58,32,69,120,112,101,99,116,101,100,32,50,32,100,105,103,105,116,115,32,97,102,116,101,114,32,39,46,76,39,10,0] /* %5d: Expected 2 digi */, "i8", ALLOC_STATIC);
__str101=allocate([37,53,100,58,32,108,101,116,116,101,114,103,114,111,117,112,32,111,117,116,32,111,102,32,114,97,110,103,101,32,40,48,49,45,37,46,50,100,41,10,0] /* %5d: lettergroup out */, "i8", ALLOC_STATIC);
__str102=allocate([37,53,100,58,32,108,101,116,116,101,114,103,114,111,117,112,32,76,37,46,50,100,32,105,115,32,97,108,114,101,97,100,121,32,100,101,102,105,110,101,100,10,0] /* %5d: lettergroup L%. */, "i8", ALLOC_STATIC);
__str103=allocate([57,0] /* 9\00 */, "i8", ALLOC_STATIC);
__ZL11hash_chains=allocate(4096, "i8*", ALLOC_STATIC);
__ZL11hash_counts=allocate(4096, "i32", ALLOC_STATIC);
__ZL9text_mode_b=allocate(1, "i1", ALLOC_STATIC);
__str104=allocate([37,115,37,115,46,116,120,116,0] /* %s%s.txt\00 */, "i8", ALLOC_STATIC);
__str106=allocate([67,97,110,39,116,32,97,108,108,111,99,97,116,101,32,109,101,109,111,114,121,10,0] /* Can't allocate memor */, "i8", ALLOC_STATIC);
__str107=allocate([9,37,100,32,101,110,116,114,105,101,115,10,0] /* \09%d entries\0A\00 */, "i8", ALLOC_STATIC);
__ZZL12compile_linePcS_PiE10nullstring=allocate(1, "i8", ALLOC_STATIC);
__str108=allocate([95,45,0] /* _-\00 */, "i8", ALLOC_STATIC);
__str109=allocate([37,53,100,58,32,85,110,107,110,111,119,110,32,107,101,121,119,111,114,100,58,32,37,115,10,0] /* %5d: Unknown keyword */, "i8", ALLOC_STATIC);
__str110=allocate([32,0] /*  \00 */, "i8", ALLOC_STATIC);
__str111=allocate([37,53,100,58,32,78,101,101,100,32,116,111,32,99,111,109,112,105,108,101,32,100,105,99,116,105,111,110,97,114,121,32,97,103,97,105,110,10,0] /* %5d: Need to compile */, "i8", ALLOC_STATIC);
__str112=allocate([37,53,100,58,32,66,97,100,32,112,104,111,110,101,109,101,32,91,37,99,93,32,40,48,120,37,120,41,32,105,110,58,32,37,115,32,32,37,115,10,0] /* %5d: Bad phoneme [%c */, "i8", ALLOC_STATIC);
__str113=allocate([85,43,37,120,0] /* U+%x\00 */, "i8", ALLOC_STATIC);
__str114=allocate([37,53,100,58,32,84,119,111,32,109,97,110,121,32,112,97,114,116,115,32,105,110,32,97,32,109,117,108,116,105,45,119,111,114,100,32,101,110,116,114,121,58,32,37,100,10,0] /* %5d: Two many parts  */, "i8", ALLOC_STATIC);
__str115=allocate([67,97,110,39,116,32,97,99,99,101,115,115,32,40,37,115,41,32,102,105,108,101,32,39,37,115,39,10,0] /* Can't access (%s) fi */, "i8", ALLOC_STATIC);
_dictionary_skipwords=allocate(1, "i32", ALLOC_STATIC);
_dictionary_name=allocate(40, "i8", ALLOC_STATIC);
__ZL13remove_accent=allocate([97,97,97,97,97,97,97,99,101,101,101,101,105,105,105,105,100,110,111,111,111,111,111,0,111,117,117,117,117,121,116,115,97,97,97,97,97,97,97,99,101,101,101,101,105,105,105,105,100,110,111,111,111,111,111,0,111,117,117,117,117,121,116,121,97,97,97,97,97,97,99,99,99,99,99,99,99,99,100,100,100,100,101,101,101,101,101,101,101,101,101,101,103,103,103,103,103,103,103,103,104,104,104,104,105,105,105,105,105,105,105,105,105,105,105,105,106,106,107,107,107,108,108,108,108,108,108,108,108,108,108,110,110,110,110,110,110,110,110,110,111,111,111,111,111,111,111,111,114,114,114,114,114,114,115,115,115,115,115,115,115,115,116,116,116,116,116,116,117,117,117,117,117,117,117,117,117,117,117,117,119,119,121,121,121,122,122,122,122,122,122,115,98,98,98,98,0,0,111,99,99,100,100,100,100,100,101,101,101,102,102,103,103,104,105,105,107,107,108,108,109,110,110,111,111,111,111,111,112,112,121,0,0,115,115,116,116,116,116,117,117,117,118,121,121,122,122,122,122,122,122,122,0,0,0,119,116,116,116,107,100,100,100,108,108,108,110,110,110,97,97,105,105,111,111,117,117,117,117,117,117,117,117,117,117,101,97,97,97,97,97,97,103,103,103,103,107,107,111,111,111,111,122,122,106,100,100,100,103,103,119,119,110,110,97,97,97,97,111,111,97,97,97,97,101,101,101,101,105,105,105,105,111,111,111,111,114,114,114,114,117,117,117,117,115,115,116,116,121,121,104,104,110,100,111,111,122,122,97,97,101,101,111,111,111,111,111,111,111,111,121,121,108,110,116,106,100,113,97,99,99,108,116,115,122,0,0,98,117,118,101,101,106,106,113,113,114,114,121,121,97,97,97,98,111,99,100,100,101,101,101,101,101,101,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0] /* aaaaaaaceeeeiiiidnoo */, "i8", ALLOC_STATIC);
__str116=allocate([37,115,37,99,37,115,95,100,105,99,116,0] /* %s%c%s_dict\00 */, "i8", ALLOC_STATIC);
__str2118=allocate([67,97,110,39,116,32,114,101,97,100,32,100,105,99,116,105,111,110,97,114,121,32,102,105,108,101,58,32,39,37,115,39,10,0] /* Can't read dictionar */, "i8", ALLOC_STATIC);
__str3119=allocate([69,109,112,116,121,32,95,100,105,99,116,32,102,105,108,101,58,32,39,37,115,10,0] /* Empty _dict file: '% */, "i8", ALLOC_STATIC);
__str4120=allocate([66,97,100,32,100,97,116,97,58,32,39,37,115,39,32,40,37,120,32,108,101,110,103,116,104,61,37,120,41,10,0] /* Bad data: '%s' (%x l */, "i8", ALLOC_STATIC);
__str5121=allocate([69,114,114,111,114,32,105,110,32,37,115,95,114,117,108,101,115,44,32,110,111,32,100,101,102,97,117,108,116,32,114,117,108,101,32,103,114,111,117,112,10,0] /* Error in %s_rules, n */, "i8", ALLOC_STATIC);
__str7123=allocate([61,61,44,44,39,42,32,32,0] /* ==,,'_  \00 */, "i8", ALLOC_STATIC);
_ipa1=allocate([32, 0, 33, 0, 34, 0, 688, 0, 36, 0, 37, 0, 230, 0, 712, 0, 40, 0, 638, 0, 42, 0, 43, 0, 716, 0, 45, 0, 46, 0, 47, 0, 594, 0, 49, 0, 50, 0, 604, 0, 52, 0, 53, 0, 54, 0, 55, 0, 629, 0, 57, 0, 720, 0, 690, 0, 60, 0, 61, 0, 62, 0, 660, 0, 601, 0, 593, 0, 946, 0, 231, 0, 240, 0, 603, 0, 70, 0, 610, 0, 295, 0, 618, 0, 607, 0, 75, 0, 76, 0, 625, 0, 331, 0, 596, 0, 934, 0, 611, 0, 640, 0, 643, 0, 952, 0, 650, 0, 652, 0, 339, 0, 967, 0, 248, 0, 658, 0, 810, 0, 92, 0, 93, 0, 94, 0, 95, 0, 96, 0, 97, 0, 98, 0, 99, 0, 100, 0, 101, 0, 102, 0, 609, 0, 104, 0, 105, 0, 106, 0, 107, 0, 108, 0, 109, 0, 110, 0, 111, 0, 112, 0, 113, 0, 114, 0, 115, 0, 116, 0, 117, 0, 118, 0, 119, 0, 120, 0, 121, 0, 122, 0, 123, 0, 124, 0, 125, 0, 771, 0, 127, 0], ["i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0], ALLOC_STATIC);
__str8124=allocate([61,61,44,44,39,39,0] /* ==,,''\00 */, "i8", ALLOC_STATIC);
__str9125=allocate([40,37,115,41,0] /* (%s)\00 */, "i8", ALLOC_STATIC);
__ZL15stress_phonemes=allocate([3,2,4,5,6,7,26] /* \03\02\04\05\06\07\1 */, "i8", ALLOC_STATIC);
__ZZ13SetWordStressP10TranslatorPcPjiiE15consonant_types=allocate([0,0,0,1,1,1,1,1,1,1,0,0,0,0,0,0] /* \00\00\00\01\01\01\0 */, "i8", ALLOC_STATIC);
__ZZ13SetWordStressP10TranslatorPcPjiiE8guess_ru=allocate([0,0,1,1,2,3,3,4,5,6,7,7,8,9,10,11] /* \00\00\01\01\02\03\0 */, "i8", ALLOC_STATIC);
__ZZ13SetWordStressP10TranslatorPcPjiiE10guess_ru_v=allocate([0,0,1,1,2,2,3,3,4,5,6,7,7,8,9,10] /* \00\00\01\01\02\02\0 */, "i8", ALLOC_STATIC);
__ZZ13SetWordStressP10TranslatorPcPjiiE10guess_ru_t=allocate([0,0,1,2,3,3,3,4,5,6,7,7,7,8,9,10] /* \00\00\01\02\03\03\0 */, "i8", ALLOC_STATIC);
__ZZ14TranslateRulesP10TranslatorPcS1_iS1_iPjE9str_pause=allocate([11,0] /* \0B\00 */, "i8", ALLOC_STATIC);
__str10126=allocate([85,110,112,114,111,110,111,117,110,99,97,98,108,101,63,32,39,37,115,39,10,0] /* Unpronouncable? '%s' */, "i8", ALLOC_STATIC);
__str11127=allocate([84,114,97,110,115,108,97,116,101,32,39,37,115,39,10,0] /* Translate '%s'\0A\00 */, "i8", ALLOC_STATIC);
__ZL13diereses_list=allocate([228, 0, 235, 0, 239, 0, 246, 0, 252, 0, 255, 0, 0, 0], ["i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0], ALLOC_STATIC);
__ZZ14LookupDictListP10TranslatorPPcS1_PjiP8WORD_TABE16word_replacement=allocate(160, "i8", ALLOC_STATIC);
__str15131=allocate([37,115,32,0] /* %s \00 */, "i8", ALLOC_STATIC);
__str16132=allocate([82,101,112,108,97,99,101,58,32,37,115,32,32,37,115,10,0] /* Replace: %s  %s\0A\0 */, "i8", ALLOC_STATIC);
__ZZ11LookupFlagsP10TranslatorPKcE5flags=allocate(8, "i32", ALLOC_STATIC);
__ZZ12RemoveEndingP10TranslatorPciS1_E6ending=allocate(12, "i8", ALLOC_STATIC);
__ZZ12RemoveEndingP10TranslatorPciS1_E16add_e_exceptions=allocate(8, "i8*", ALLOC_STATIC);
__str17133=allocate([105,111,110,0] /* ion\00 */, "i8", ALLOC_STATIC);
__ZZ12RemoveEndingP10TranslatorPciS1_E15add_e_additions=allocate(32, "i8*", ALLOC_STATIC);
__str18134=allocate([99,0] /* c\00 */, "i8", ALLOC_STATIC);
__str19135=allocate([114,115,0] /* rs\00 */, "i8", ALLOC_STATIC);
__str20136=allocate([105,114,0] /* ir\00 */, "i8", ALLOC_STATIC);
__str21137=allocate([117,114,0] /* ur\00 */, "i8", ALLOC_STATIC);
__str22138=allocate([97,116,104,0] /* ath\00 */, "i8", ALLOC_STATIC);
__str23139=allocate([110,115,0] /* ns\00 */, "i8", ALLOC_STATIC);
__str24140=allocate([117,0] /* u\00 */, "i8", ALLOC_STATIC);
__str25141=allocate([97,100,100,32,101,10,0] /* add e\0A\00 */, "i8", ALLOC_STATIC);
__str27143=allocate([101,115,0] /* es\00 */, "i8", ALLOC_STATIC);
__str28144=allocate([70,108,97,103,115,58,32,32,37,115,32,32,37,115,10,0] /* Flags:  %s  %s\0A\00 */, "i8", ALLOC_STATIC);
__str29145=allocate([70,111,117,110,100,58,32,39,37,115,32,37,115,0] /* Found: '%s %s\00 */, "i8", ALLOC_STATIC);
__str30146=allocate([70,111,117,110,100,58,32,39,37,115,0] /* Found: '%s\00 */, "i8", ALLOC_STATIC);
__str31147=allocate([39,32,91,37,115,93,32,32,37,115,10,0] /* ' [%s]  %s\0A\00 */, "i8", ALLOC_STATIC);
__ZZL9MatchRuleP10TranslatorPPcS1_iS1_P11MatchRecordiiE4best=allocate(16, ["i32",0,0,0,"i8*",0,0,0,"i32",0,0,0,"i8*",0,0,0], ALLOC_STATIC);
__str32148=allocate([37,51,100,9,37,115,32,91,37,115,93,10,0] /* %3d\09%s [%s]\0A\00 */, "i8", ALLOC_STATIC);
__str34150=allocate([66,97,100,32,114,117,108,101,115,32,100,97,116,97,32,105,110,32,39,37,115,95,100,105,99,116,39,32,97,116,32,48,120,37,120,10,0] /* Bad rules data in '% */, "i8", ALLOC_STATIC);
_env_fall=allocate([255,253,250,248,246,244,242,240,238,236,234,232,230,228,226,224,222,220,218,216,214,212,210,208,206,204,202,200,198,196,194,192,190,188,186,184,182,180,178,176,174,172,170,168,166,164,162,160,158,156,154,152,150,148,146,144,142,140,138,136,134,132,130,128,126,124,122,120,118,116,114,112,110,108,106,104,102,100,98,96,94,92,90,88,86,84,82,80,78,76,74,72,70,68,66,64,62,60,58,56,54,52,50,48,46,44,42,40,38,36,34,32,30,28,26,24,22,20,18,16,14,12,10,8,6,4,2,0] /* \FF\FD\FA\F8\F6\F4\F */, "i8", ALLOC_STATIC);
_env_rise=allocate([0,2,4,6,8,10,12,14,16,18,20,22,24,26,28,30,32,34,36,38,40,42,44,46,48,50,52,54,56,58,60,62,64,66,68,70,72,74,76,78,80,82,84,86,88,90,92,94,96,98,100,102,104,106,108,110,112,114,116,118,120,122,124,126,128,130,132,134,136,138,140,142,144,146,148,150,152,154,156,158,160,162,164,166,168,170,172,174,176,178,180,182,184,186,188,190,192,194,196,198,200,202,204,206,208,210,212,214,216,218,220,222,224,226,228,230,232,234,236,238,240,242,244,246,248,250,253,255] /* \00\02\04\06\08\0A\0 */, "i8", ALLOC_STATIC);
_env_frise=allocate([255,244,234,224,214,204,195,186,177,168,159,151,143,135,127,120,113,106,99,92,86,80,74,68,63,57,52,47,43,38,34,30,26,23,19,16,13,11,8,6,4,2,1,0,0,0,0,0,0,0,1,2,3,4,5,7,8,10,12,14,16,19,21,23,26,29,31,34,37,40,44,46,48,50,52,54,57,59,61,64,66,69,71,74,76,79,81,84,87,90,93,95,98,101,104,107,110,113,116,120,123,126,129,133,136,139,143,146,150,153,157,160,164,168,172,175,179,183,187,191,195,199,203,207,211,215,219,224] /* \FF\F4\EA\E0\D6\CC\C */, "i8", ALLOC_STATIC);
__ZL11env_r_frise=allocate([207,204,201,198,195,192,189,185,180,176,171,167,162,156,151,146,140,134,129,123,117,111,105,99,93,87,80,74,68,62,56,51,45,39,34,28,23,18,13,8,4,2,2,1,0,0,0,0,0,1,1,2,3,4,5,7,8,10,12,13,15,18,20,22,25,27,30,33,36,39,42,45,48,52,54,56,58,60,63,65,67,70,72,75,77,80,82,85,88,90,93,96,99,102,105,108,111,114,117,120,123,126,129,133,136,139,143,146,150,153,157,160,164,168,172,175,179,183,187,191,195,199,203,207,211,215,219,224] /* \CF\CC\C9\C6\C3\C0\B */, "i8", ALLOC_STATIC);
__ZL10env_frise2=allocate([255,249,244,238,233,228,223,218,213,208,203,198,193,189,184,179,175,170,166,161,157,153,149,144,140,136,132,128,125,121,117,113,110,106,103,99,96,93,89,86,83,80,77,74,71,68,65,62,60,57,55,52,50,47,45,43,40,38,36,34,32,30,28,26,25,23,21,20,18,17,15,14,13,12,10,9,8,7,6,5,5,4,3,2,2,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,1,2,2,3,4,4,5,6,7,8,9,10,11,12,14,15,16,18,19,21,23,24,26,28,30,32] /* \FF\F9\F4\EE\E9\E4\D */, "i8", ALLOC_STATIC);
__ZL12env_r_frise2=allocate([208,206,205,204,202,200,199,197,195,193,192,189,187,184,181,179,176,173,170,167,163,160,157,153,150,146,143,139,135,132,128,124,120,116,112,109,105,101,97,93,89,85,81,77,74,70,66,62,59,55,52,49,47,45,42,40,38,36,34,32,30,28,26,25,23,21,20,18,17,15,14,13,12,10,9,8,7,6,5,5,4,3,2,2,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,1,2,2,3,4,4,5,6,7,8,9,10,11,12,14,15,16,18,19,21,23,24,26,28,30,32] /* \D0\CE\CD\CC\CA\C8\C */, "i8", ALLOC_STATIC);
__ZL12env_risefall=allocate([152,153,153,154,156,157,159,161,164,167,169,172,176,179,182,186,190,193,197,201,205,209,212,216,220,223,227,230,234,237,240,242,245,247,249,251,252,253,254,254,255,255,255,255,255,255,254,253,251,250,248,246,243,241,238,236,233,230,228,224,221,218,215,211,208,204,200,196,192,188,184,180,176,172,167,163,159,154,150,145,141,136,132,127,123,118,114,109,105,101,96,92,88,84,80,76,72,68,64,60,57,53,50,47,43,40,38,35,32,29,26,23,21,18,15,13,10,8,7,5,3,2,1,0,0,0,0,0] /* \98\99\99\9A\9C\9D\9 */, "i8", ALLOC_STATIC);
__ZL13env_fallrise3=allocate([254,255,255,255,255,255,254,253,252,250,248,246,244,241,238,235,232,229,225,222,218,214,210,205,201,196,191,186,182,176,171,166,161,156,150,145,139,134,128,123,117,111,106,100,95,89,84,79,73,68,63,58,53,48,43,38,34,29,25,21,17,13,10,7,4,3,1,0,0,0,0,0,0,0,0,1,1,2,4,5,7,9,11,13,16,18,21,24,27,30,34,37,41,45,49,53,58,62,67,72,76,81,87,91,94,98,101,104,107,110,113,116,118,120,123,124,126,128,129,130,131,131,132,132,131,131,130,129] /* \FE\FF\FF\FF\FF\FF\F */, "i8", ALLOC_STATIC);
__ZL13env_fallrise4=allocate([114,114,113,113,112,111,109,108,106,104,102,100,97,95,92,90,87,84,81,78,75,72,69,66,63,59,56,53,50,47,44,41,38,35,32,29,27,24,22,20,18,16,14,12,11,10,9,8,7,7,7,7,7,6,5,5,5,5,5,5,5,5,5,6,7,7,8,9,10,12,13,15,16,18,20,22,24,27,29,32,35,38,41,44,47,51,55,59,63,67,71,76,81,86,91,96,101,106,111,116,121,127,132,137,143,149,155,161,167,173,179,186,192,199,206,213,220,227,234,241,245,247,250,252,253,254,255,255] /* rrqqpomljhfda_\5CZWT */, "i8", ALLOC_STATIC);
__ZL9env_fall2=allocate([254,254,253,253,252,251,251,250,250,249,248,248,247,247,246,246,245,244,244,243,243,242,242,241,240,240,239,238,238,237,236,235,234,234,233,232,231,230,229,228,227,226,225,224,222,221,220,219,217,216,214,213,211,210,208,206,204,203,201,199,197,195,192,190,188,185,183,181,178,175,173,170,167,164,161,158,154,151,148,144,141,137,133,129,125,121,117,113,109,104,100,97,94,91,87,84,81,77,74,70,67,64,60,57,53,50,46,42,39,35,31,28,24,20,17,13,11,9,7,5,4,3,2,1,0,0,0,0] /* \FE\FE\FD\FD\FC\FB\F */, "i8", ALLOC_STATIC);
__ZL9env_rise2=allocate([0,0,0,0,1,1,1,2,2,3,3,4,4,5,6,6,7,8,8,9,10,11,12,12,13,14,15,16,17,18,19,20,22,23,24,25,27,28,29,31,32,34,35,37,38,40,41,43,45,47,48,50,52,54,56,58,60,62,64,66,68,71,73,75,78,80,82,85,87,90,93,95,98,101,103,106,109,112,115,118,121,124,127,130,134,137,140,144,147,150,154,157,160,163,166,169,172,175,178,181,184,187,190,193,196,199,202,205,208,211,214,217,220,223,226,228,231,233,236,238,240,242,244,246,248,250,251,253] /* \00\00\00\00\01\01\0 */, "i8", ALLOC_STATIC);
__ZL16env_risefallrise=allocate([127,127,127,128,129,131,132,135,137,140,143,146,150,153,157,161,165,170,174,178,183,187,192,197,201,205,210,214,218,222,226,230,234,237,240,243,245,248,250,251,252,253,254,254,253,252,251,249,247,244,240,236,231,226,220,213,206,198,189,180,169,158,146,136,130,125,119,114,108,102,96,90,84,78,73,66,60,55,50,45,40,36,31,27,24,20,17,14,12,9,7,6,5,4,4,4,4,5,6,8,10,13,16,20,24,29,35,41,47,55,62,71,80,90,100,112,124,131,133,136,138,140,142,143,145,146,147,147] /* \7F\7F\7F\80\81\83\8 */, "i8", ALLOC_STATIC);
_envelope_data=allocate(80, "i8*", ALLOC_STATIC);
__ZL18tone_nucleus_table=allocate([0, 64, 8, 0, 70, 18, 0, 0, 0, 0, 0, 0, 24, 12, 0, undef, 4, 80, 18, 6, 78, 22, 0, 0, 0, 0, 0, 0, 34, 52, 0, undef, 4, 88, 22, 6, 82, 22, 0, 0, 0, 0, 0, 0, 34, 64, 0, undef, 0, 92, 8, 0, 92, 80, 0, 0, 0, 0, 0, 0, 76, 8, 1, undef, 0, 86, 4, 0, 94, 66, 0, 0, 0, 0, 0, 0, 34, 10, 0, undef, 0, 62, 10, 0, 62, 20, 0, 0, 0, 0, 0, 0, 28, 16, 0, undef, 4, 68, 18, 6, 68, 22, 0, 0, 0, 0, 0, 0, 30, 44, 0, undef, 6, 64, 16, 0, 66, 32, 0, 0, 0, 0, 0, 0, 32, 18, 0, undef, 2, 68, 46, 0, 42, 32, 0, 0, 0, 0, 0, 0, 46, 58, 0, undef, 4, 78, 24, 6, 72, 22, 0, 0, 0, 0, 0, 0, 42, 52, 0, undef, 4, 88, 34, 0, 64, 32, 0, 0, 0, 0, 0, 0, 46, 82, 0, undef, 0, 56, 12, 0, 56, 20, 0, 0, 0, 0, 0, 0, 24, 12, 0, undef, 0, 70, 18, 0, 70, 24, 0, 0, 0, 0, 0, 0, 32, 20, 0, undef], ["i8","i8","i8","i8","i8","i8",0,0,"i16*",0,0,0,"i8","i8","i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i16*",0,0,0,"i8","i8","i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i16*",0,0,0,"i8","i8","i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i16*",0,0,0,"i8","i8","i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i16*",0,0,0,"i8","i8","i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i16*",0,0,0,"i8","i8","i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i16*",0,0,0,"i8","i8","i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i16*",0,0,0,"i8","i8","i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i16*",0,0,0,"i8","i8","i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i16*",0,0,0,"i8","i8","i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i16*",0,0,0,"i8","i8","i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i16*",0,0,0,"i8","i8","i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i16*",0,0,0,"i8","i8","i8","i8"], ALLOC_STATIC);
_punctuation_to_tone=allocate([0, 1, 2, 3, 0, 4, 0, 1, 2, 3, 0, 4, 5, 6, 2, 3, 0, 4, 5, 7, 1, 3, 0, 4, 8, 9, 10, 3, 0, 0, 8, 8, 10, 3, 0, 0, 11, 11, 11, 11, 0, 0, 12, 12, 12, 12, 0, 0], "i8", ALLOC_STATIC);
_n_tunes=allocate(1, "i32", ALLOC_STATIC);
_tunes=allocate(1, "%struct.TUNE*", ALLOC_STATIC);
__ZL12syllable_tab=allocate(1, "%struct.SYLLABLE*", ALLOC_STATIC);
__ZL8no_tonic_b=allocate(1, "i1", ALLOC_STATIC);
__ZL15tone_head_table=allocate([46, 57, 78, 50, 0, 0, 0, 0, 3, 7, 5, 0, 0, 0, 0, 0, 46, 57, 78, 46, 0, 0, 0, 0, 3, 7, 5, 0, 0, 0, 0, 0, 46, 57, 78, 46, 0, 0, 0, 0, 3, 7, 5, 0, 0, 0, 0, 0, 46, 57, 90, 50, 0, 0, 0, 0, 3, 9, 5, 0, 0, 0, 0, 0, 46, 57, 78, 50, 0, 0, 0, 0, 3, 7, 5, 0, 0, 0, 0, 0, 46, 57, 74, 55, 0, 0, 0, 0, 4, 7, 5, 0, 0, 0, 0, 0, 46, 57, 74, 55, 0, 0, 0, 0, 4, 7, 5, 0, 0, 0, 0, 0, 46, 57, 74, 55, 0, 0, 0, 0, 4, 7, 5, 0, 0, 0, 0, 0, 46, 57, 78, 50, 0, 0, 0, 0, 3, 7, 5, 0, 0, 0, 0, 0, 46, 57, 78, 46, 0, 0, 0, 0, 3, 7, 5, 0, 0, 0, 0, 0, 46, 57, 78, 50, 0, 0, 0, 0, 3, 7, 5, 0, 0, 0, 0, 0, 34, 41, 41, 32, 0, 0, 0, 0, 3, 7, 5, 0, 0, 0, 0, 0, 46, 57, 55, 50, 0, 0, 0, 0, 3, 7, 5, 0, 0, 0, 0, 0], ["i8","i8","i8","i8","i32*",0,0,0,"i8","i8","i8",0,"i8*",0,0,0,"i8","i8","i8","i8","i32*",0,0,0,"i8","i8","i8",0,"i8*",0,0,0,"i8","i8","i8","i8","i32*",0,0,0,"i8","i8","i8",0,"i8*",0,0,0,"i8","i8","i8","i8","i32*",0,0,0,"i8","i8","i8",0,"i8*",0,0,0,"i8","i8","i8","i8","i32*",0,0,0,"i8","i8","i8",0,"i8*",0,0,0,"i8","i8","i8","i8","i32*",0,0,0,"i8","i8","i8",0,"i8*",0,0,0,"i8","i8","i8","i8","i32*",0,0,0,"i8","i8","i8",0,"i8*",0,0,0,"i8","i8","i8","i8","i32*",0,0,0,"i8","i8","i8",0,"i8*",0,0,0,"i8","i8","i8","i8","i32*",0,0,0,"i8","i8","i8",0,"i8*",0,0,0,"i8","i8","i8","i8","i32*",0,0,0,"i8","i8","i8",0,"i8*",0,0,0,"i8","i8","i8","i8","i32*",0,0,0,"i8","i8","i8",0,"i8*",0,0,0,"i8","i8","i8","i8","i32*",0,0,0,"i8","i8","i8",0,"i8*",0,0,0,"i8","i8","i8","i8","i32*",0,0,0,"i8","i8","i8",0,"i8*",0,0,0], ALLOC_STATIC);
__ZL10number_pre=allocate(1, "i32", ALLOC_STATIC);
__ZL10tone_posn2=allocate(1, "i32", ALLOC_STATIC);
__ZL9tone_posn=allocate(1, "i32", ALLOC_STATIC);
__ZL11number_tail=allocate(1, "i32", ALLOC_STATIC);
__ZL14tone_pitch_env=allocate(1, "i32", ALLOC_STATIC);
__ZZL18calc_pitch_segmentiiP9TONE_HEADP12TONE_NUCLEUSiiE12continue_tab=allocate([230,32,20,8,0] /* \E6 \14\08\00 */, "i8", ALLOC_STATIC);
__ZL8min_drop=allocate([6, 0, 0, 0, 7, 0, 0, 0, 9, 0, 0, 0, 9, 0, 0, 0, 20, 0, 0, 0, 20, 0, 0, 0, 20, 0, 0, 0, 25, 0, 0, 0], ["i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0], ALLOC_STATIC);
__ZL7drops_0=allocate([9, 0, 0, 0, 9, 0, 0, 0, 16, 0, 0, 0, 16, 0, 0, 0, 16, 0, 0, 0, 23, 0, 0, 0, 55, 0, 0, 0, 32, 0, 0, 0], ["i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0], ALLOC_STATIC);
__ZL5oflow=allocate([0,40,24,8,0] /* \00(\18\08\00 */, "i8", ALLOC_STATIC);
__ZL9oflow_emf=allocate([10,52,32,20,10] /* \0A4 \14\0A */, "i8", ALLOC_STATIC);
__ZL10oflow_less=allocate([6,38,24,14,4] /* \06&\18\0E\04 */, "i8", ALLOC_STATIC);
__ZL12last_primary=allocate(1, "i32", ALLOC_STATIC);
_namedata=allocate(1, "i8*", ALLOC_STATIC);
__ZL7f_input=allocate(1, "%struct.__sFILE*", ALLOC_STATIC);
_p_textinput=allocate(1, "i8*", ALLOC_STATIC);
_p_wchar_input=allocate(1, "i32*", ALLOC_STATIC);
__ZL10ungot_word=allocate(1, "i8*", ALLOC_STATIC);
_count_characters=allocate(1, "i32", ALLOC_STATIC);
__ZL10ssml_stack=allocate(1520, ["i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8"], ALLOC_STATIC);
__ZL16current_voice_id=allocate(40, "i8", ALLOC_STATIC);
_param_stack=allocate(1280, "i32", ALLOC_STATIC);
_param_defaults=allocate([0, 0, 0, 0, 175, 0, 0, 0, 100, 0, 0, 0, 50, 0, 0, 0, 50, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0], ["i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0], ALLOC_STATIC);
_walpha_tab=allocate([1,255,1,255,1,255,1,255,1,255,1,255,1,255,1,255,1,255,1,255,1,255,1,255,1,255,1,255,1,255,1,255,1,255,1,255,1,255,1,255,1,255,1,255,1,255,1,255,254,255,1,255,1,255,1,255,255,1,255,1,255,1,255,1,255,1,255,1,255,1,255,1,255,255,1,255,1,255,1,255,1,255,1,255,1,255,1,255,1,255,1,255,1,255,1,255,1,255,1,255,1,255,1,255,1,255,1,255,1,255,1,255,1,255,1,255,1,255,1,255,255,1,255,1,255,1,255,255,255,210,1,255,1,255,206,1,255,205,205,1,255,255,79,202,203,1,255,205,207,255,211,209,1,255,255,255,211,213,255,214,1,255,1,255,1,255,218,1,255,218,255,255,1,255,218,1,255,217,217,1,255,1,255,219,1,255,255,255,1,255,255,255,255,255,255,255,2,1,255,2,1,255,2,1,255,1,255,1,255,1,255,1,255,1,255,1,255,1,255,1,255,255,1,255,1,255,1,255,1,255,1,255,1,255,1,255,1,255,1,255,255,2,1,255,1,255,255,255,1,255,1,255,1,255,1,255,1,255,1,255,1,255,1,255,1,255,1,255,1,255,1,255,1,255,1,255,1,255,1,255,1,255,1,255,1,255,1,255,255,0,1,255,1,255,1,255,1,255,1,255,1,255,1,255,1,255,1,255] /* \01\FF\01\FF\01\FF\0 */, "i8", ALLOC_STATIC);
__ZL10ungot_char=allocate(1, "i32", ALLOC_STATIC);
__ZL12end_of_input_b=allocate(1, "i1", ALLOC_STATIC);
__str167=allocate([115,112,101,97,107,0] /* speak\00 */, "i8", ALLOC_STATIC);
__str1168=allocate([118,111,105,99,101,0] /* voice\00 */, "i8", ALLOC_STATIC);
__str2169=allocate([112,114,111,115,111,100,121,0] /* prosody\00 */, "i8", ALLOC_STATIC);
__str3170=allocate([115,97,121,45,97,115,0] /* say-as\00 */, "i8", ALLOC_STATIC);
__str4171=allocate([109,97,114,107,0] /* mark\00 */, "i8", ALLOC_STATIC);
__str5172=allocate([115,0] /* s\00 */, "i8", ALLOC_STATIC);
__str6173=allocate([112,0] /* p\00 */, "i8", ALLOC_STATIC);
__str7174=allocate([112,104,111,110,101,109,101,0] /* phoneme\00 */, "i8", ALLOC_STATIC);
__str8175=allocate([115,117,98,0] /* sub\00 */, "i8", ALLOC_STATIC);
__str9176=allocate([116,116,115,58,115,116,121,108,101,0] /* tts:style\00 */, "i8", ALLOC_STATIC);
__str10177=allocate([97,117,100,105,111,0] /* audio\00 */, "i8", ALLOC_STATIC);
__str11178=allocate([101,109,112,104,97,115,105,115,0] /* emphasis\00 */, "i8", ALLOC_STATIC);
__str12179=allocate([98,114,101,97,107,0] /* break\00 */, "i8", ALLOC_STATIC);
__str13180=allocate([109,101,116,97,100,97,116,97,0] /* metadata\00 */, "i8", ALLOC_STATIC);
__str14181=allocate([98,114,0] /* br\00 */, "i8", ALLOC_STATIC);
__str15182=allocate([108,105,0] /* li\00 */, "i8", ALLOC_STATIC);
__str16183=allocate([100,100,0] /* dd\00 */, "i8", ALLOC_STATIC);
__str17184=allocate([105,109,103,0] /* img\00 */, "i8", ALLOC_STATIC);
__str18185=allocate([116,100,0] /* td\00 */, "i8", ALLOC_STATIC);
__str19186=allocate([104,49,0] /* h1\00 */, "i8", ALLOC_STATIC);
__str20187=allocate([104,50,0] /* h2\00 */, "i8", ALLOC_STATIC);
__str21188=allocate([104,51,0] /* h3\00 */, "i8", ALLOC_STATIC);
__str22189=allocate([104,52,0] /* h4\00 */, "i8", ALLOC_STATIC);
__str23190=allocate([104,114,0] /* hr\00 */, "i8", ALLOC_STATIC);
__str24191=allocate([115,99,114,105,112,116,0] /* script\00 */, "i8", ALLOC_STATIC);
__str25192=allocate([115,116,121,108,101,0] /* style\00 */, "i8", ALLOC_STATIC);
__str26193=allocate([102,111,110,116,0] /* font\00 */, "i8", ALLOC_STATIC);
__str27194=allocate([98,0] /* b\00 */, "i8", ALLOC_STATIC);
__str28195=allocate([105,0] /* i\00 */, "i8", ALLOC_STATIC);
__str29196=allocate([115,116,114,111,110,103,0] /* strong\00 */, "i8", ALLOC_STATIC);
__str30197=allocate([101,109,0] /* em\00 */, "i8", ALLOC_STATIC);
__str31198=allocate([99,111,100,101,0] /* code\00 */, "i8", ALLOC_STATIC);
__ZL8ssmltags=allocate([0, 0, 0, 0, 1, 0, 0, 0, 0, 0, 0, 0, 2, 0, 0, 0, 0, 0, 0, 0, 3, 0, 0, 0, 0, 0, 0, 0, 4, 0, 0, 0, 0, 0, 0, 0, 5, 0, 0, 0, 0, 0, 0, 0, 6, 0, 0, 0, 0, 0, 0, 0, 7, 0, 0, 0, 0, 0, 0, 0, 8, 0, 0, 0, 0, 0, 0, 0, 9, 0, 0, 0, 0, 0, 0, 0, 10, 0, 0, 0, 0, 0, 0, 0, 11, 0, 0, 0, 0, 0, 0, 0, 12, 0, 0, 0, 0, 0, 0, 0, 13, 0, 0, 0, 0, 0, 0, 0, 14, 0, 0, 0, 0, 0, 0, 0, 15, 0, 0, 0, 0, 0, 0, 0, 15, 0, 0, 0, 0, 0, 0, 0, 15, 0, 0, 0, 0, 0, 0, 0, 15, 0, 0, 0, 0, 0, 0, 0, 15, 0, 0, 0, 0, 0, 0, 0, 7, 0, 0, 0, 0, 0, 0, 0, 7, 0, 0, 0, 0, 0, 0, 0, 7, 0, 0, 0, 0, 0, 0, 0, 7, 0, 0, 0, 0, 0, 0, 0, 7, 0, 0, 0, 0, 0, 0, 0, 14, 0, 0, 0, 0, 0, 0, 0, 14, 0, 0, 0, 0, 0, 0, 0, 16, 0, 0, 0, 0, 0, 0, 0, 16, 0, 0, 0, 0, 0, 0, 0, 16, 0, 0, 0, 0, 0, 0, 0, 16, 0, 0, 0, 0, 0, 0, 0, 16, 0, 0, 0, 0, 0, 0, 0, 16, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0], ["i8*",0,0,0,"i32",0,0,0,"i8*",0,0,0,"i32",0,0,0,"i8*",0,0,0,"i32",0,0,0,"i8*",0,0,0,"i32",0,0,0,"i8*",0,0,0,"i32",0,0,0,"i8*",0,0,0,"i32",0,0,0,"i8*",0,0,0,"i32",0,0,0,"i8*",0,0,0,"i32",0,0,0,"i8*",0,0,0,"i32",0,0,0,"i8*",0,0,0,"i32",0,0,0,"i8*",0,0,0,"i32",0,0,0,"i8*",0,0,0,"i32",0,0,0,"i8*",0,0,0,"i32",0,0,0,"i8*",0,0,0,"i32",0,0,0,"i8*",0,0,0,"i32",0,0,0,"i8*",0,0,0,"i32",0,0,0,"i8*",0,0,0,"i32",0,0,0,"i8*",0,0,0,"i32",0,0,0,"i8*",0,0,0,"i32",0,0,0,"i8*",0,0,0,"i32",0,0,0,"i8*",0,0,0,"i32",0,0,0,"i8*",0,0,0,"i32",0,0,0,"i8*",0,0,0,"i32",0,0,0,"i8*",0,0,0,"i32",0,0,0,"i8*",0,0,0,"i32",0,0,0,"i8*",0,0,0,"i32",0,0,0,"i8*",0,0,0,"i32",0,0,0,"i8*",0,0,0,"i32",0,0,0,"i8*",0,0,0,"i32",0,0,0,"i8*",0,0,0,"i32",0,0,0,"i8*",0,0,0,"i32",0,0,0,"i8*",0,0,0,"i32",0,0,0,"i8*",0,0,0,"i32",0,0,0], ALLOC_STATIC);
__ZL10n_namedata=allocate(1, "i32", ALLOC_STATIC);
__ZL11namedata_ix=allocate(1, "i32", ALLOC_STATIC);
__str32199=allocate([103,116,0] /* gt\00 */, "i8", ALLOC_STATIC);
__str33200=allocate([108,116,0] /* lt\00 */, "i8", ALLOC_STATIC);
__str34201=allocate([97,109,112,0] /* amp\00 */, "i8", ALLOC_STATIC);
__str35202=allocate([113,117,111,116,0] /* quot\00 */, "i8", ALLOC_STATIC);
__str36203=allocate([110,98,115,112,0] /* nbsp\00 */, "i8", ALLOC_STATIC);
__str37204=allocate([97,112,111,115,0] /* apos\00 */, "i8", ALLOC_STATIC);
__ZL18xml_char_mnemonics=allocate([0, 0, 0, 0, 62, 0, 0, 0, 0, 0, 0, 0, 57404, 0, 0, 0, 0, 0, 0, 0, 38, 0, 0, 0, 0, 0, 0, 0, 34, 0, 0, 0, 0, 0, 0, 0, 32, 0, 0, 0, 0, 0, 0, 0, 39, 0, 0, 0, 0, 0, 0, 0, -1, 0, 0, 0], ["i8*",0,0,0,"i32",0,0,0,"i8*",0,0,0,"i32",0,0,0,"i8*",0,0,0,"i32",0,0,0,"i8*",0,0,0,"i32",0,0,0,"i8*",0,0,0,"i32",0,0,0,"i8*",0,0,0,"i32",0,0,0,"i8*",0,0,0,"i32",0,0,0], ALLOC_STATIC);
__ZZ10ReadClauseP10TranslatorP7__sFILEPcPsPiiS5_S3_E12ungot_string=allocate(24, "i8", ALLOC_STATIC);
__ZZ10ReadClauseP10TranslatorP7__sFILEPcPsPiiS5_S3_E15ungot_string_ix=allocate([-1], ["i32",0,0,0,0], ALLOC_STATIC);
__ZL19clear_skipping_text_b=allocate(1, "i1", ALLOC_STATIC);
__ZL11ungot_char2=allocate(1, "i32", ALLOC_STATIC);
__ZL19ssml_ignore_l_angle=allocate(1, "i32", ALLOC_STATIC);
__str38205=allocate([37,115,37,99,37,99,0] /* %s%c%c\00 */, "i8", ALLOC_STATIC);
__ZL10sayas_mode=allocate(1, "i32", ALLOC_STATIC);
__ZL11ignore_text_b=allocate(1, "i1", ALLOC_STATIC);
__ZL11punct_chars=allocate([44, 0, 46, 0, 63, 0, 33, 0, 58, 0, 59, 0, 8211, 0, 8212, 0, 8230, 0, 894, 0, 903, 0, 2404, 0, 1417, 0, 1373, 0, 1372, 0, 1374, 0, 1371, 0, 2843, 0, 1567, 0, 4962, 0, 4963, 0, 4964, 0, 4965, 0, 4966, 0, 4967, 0, 4968, 0, 4347, 0, 12289, 0, 12290, 0, -255, 0, -244, 0, -242, 0, -230, 0, -229, 0, -225, 0, 0, 0], ["i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0], ALLOC_STATIC);
__ZL16punct_attributes=allocate([0, 0, 0, 0, 266260, 0, 0, 0, 524328, 0, 0, 0, 532520, 0, 0, 0, 536621, 0, 0, 0, 262174, 0, 0, 0, 266270, 0, 0, 0, 266270, 0, 0, 0, 266270, 0, 0, 0, 2396190, 0, 0, 0, 532520, 0, 0, 0, 266270, 0, 0, 0, 557096, 0, 0, 0, 557096, 0, 0, 0, 266260, 0, 0, 0, 1585197, 0, 0, 0, 1581096, 0, 0, 0, 1572904, 0, 0, 0, 266270, 0, 0, 0, 532520, 0, 0, 0, 524328, 0, 0, 0, 266260, 0, 0, 0, 266270, 0, 0, 0, 262174, 0, 0, 0, 262174, 0, 0, 0, 532520, 0, 0, 0, 524358, 0, 0, 0, 524358, 0, 0, 0, 299028, 0, 0, 0, 557096, 0, 0, 0, 569389, 0, 0, 0, 299028, 0, 0, 0, 557096, 0, 0, 0, 294942, 0, 0, 0, 299038, 0, 0, 0, 565288, 0, 0, 0, 266270, 0, 0, 0, 0, 0, 0, 0], ["i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0], ALLOC_STATIC);
__str41208=allocate([32,32,32,0] /*    \00 */, "i8", ALLOC_STATIC);
__ZL12chars_ignore=allocate([8204, 0, 8205, 0, 0, 0], ["i16",0,"i16",0,"i16",0], ALLOC_STATIC);
__str42209=allocate([105,32,0] /* i \00 */, "i8", ALLOC_STATIC);
__ZL10audio_text_b=allocate(1, "i1", ALLOC_STATIC);
__ZL17speech_parameters=allocate(60, "i32", ALLOC_STATIC);
__ZL12n_ssml_stack=allocate(1, "i32", ALLOC_STATIC);
__ZL13n_param_stack=allocate(1, "i32", ALLOC_STATIC);
__ZL7xmlbase=allocate(1, "i8*", ALLOC_STATIC);
__ZZL14LookupCharNameP10TranslatoriiE3buf=allocate(60, "i8", ALLOC_STATIC);
__str47214=allocate([91,2,95,94,95,37,115,32,37,115,32,95,94,95,37,115,93,93,0] /* [\02_^_%s %s _^_%s]] */, "i8", ALLOC_STATIC);
__str48215=allocate([91,2,37,115,93,93,32,0] /* [\02%s]] \00 */, "i8", ALLOC_STATIC);
__str49216=allocate([91,2,40,88,49,41,40,88,49,41,40,88,49,41,93,93,0] /* [\02(X1)(X1)(X1)]]\0 */, "i8", ALLOC_STATIC);
__ZZL13WordToString2jE3buf=allocate(5, "i8", ALLOC_STATIC);
__str50217=allocate([1,37,100,73,32,0] /* \01%dI \00 */, "i8", ALLOC_STATIC);
__str51218=allocate([32,37,115,0] /*  %s\00 */, "i8", ALLOC_STATIC);
__str52219=allocate([1,43,49,48,83,0] /* \01+10S\00 */, "i8", ALLOC_STATIC);
__str53220=allocate([32,1,45,49,48,83,0] /*  \01-10S\00 */, "i8", ALLOC_STATIC);
__str54221=allocate([32,37,115,32,37,100,32,37,115,0] /*  %s %d %s\00 */, "i8", ALLOC_STATIC);
__str55222=allocate([37,115,37,99,115,111,117,110,100,105,99,111,110,115,37,99,37,115,0] /* %s%csoundicons%c%s\0 */, "i8", ALLOC_STATIC);
__str57224=allocate([112,111,108,121,112,104,97,115,101,0] /* polyphase\00 */, "i8", ALLOC_STATIC);
__str58225=allocate([47,116,109,112,47,101,115,112,101,97,107,88,88,88,88,88,88,0] /* /tmp/espeakXXXXXX\00 */, "i8", ALLOC_STATIC);
__str59226=allocate([115,111,120,32,34,37,115,34,32,45,114,32,37,100,32,45,119,32,45,115,32,45,99,49,32,37,115,32,37,115,10,0] /* sox \22%s\22 -r %d - */, "i8", ALLOC_STATIC);
__str60227=allocate([91,2,37,115,93,93,0] /* [\02%s]]\00 */, "i8", ALLOC_STATIC);
__ZZL14ProcessSsmlTagPwPcRiiiE10mnem_punct=allocate([0, 0, 0, 0, 1, 0, 0, 0, 0, 0, 0, 0, 2, 0, 0, 0, 0, 0, 0, 0, 3, 0, 0, 0, 0, 0, 0, 0, -1, 0, 0, 0], ["i8*",0,0,0,"i32",0,0,0,"i8*",0,0,0,"i32",0,0,0,"i8*",0,0,0,"i32",0,0,0,"i8*",0,0,0,"i32",0,0,0], ALLOC_STATIC);
__str61228=allocate([110,111,110,101,0] /* none\00 */, "i8", ALLOC_STATIC);
__str62229=allocate([97,108,108,0] /* all\00 */, "i8", ALLOC_STATIC);
__str63230=allocate([115,111,109,101,0] /* some\00 */, "i8", ALLOC_STATIC);
__ZZL14ProcessSsmlTagPwPcRiiiE13mnem_capitals=allocate([0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 2, 0, 0, 0, 0, 0, 0, 0, 1, 0, 0, 0, 0, 0, 0, 0, 20, 0, 0, 0, 0, 0, 0, 0, -1, 0, 0, 0], ["i8*",0,0,0,"i32",0,0,0,"i8*",0,0,0,"i32",0,0,0,"i8*",0,0,0,"i32",0,0,0,"i8*",0,0,0,"i32",0,0,0,"i8*",0,0,0,"i32",0,0,0], ALLOC_STATIC);
__str64231=allocate([110,111,0] /* no\00 */, "i8", ALLOC_STATIC);
__str65232=allocate([115,112,101,108,108,105,110,103,0] /* spelling\00 */, "i8", ALLOC_STATIC);
__str66233=allocate([105,99,111,110,0] /* icon\00 */, "i8", ALLOC_STATIC);
__ZZL14ProcessSsmlTagPwPcRiiiE17mnem_interpret_as=allocate([0, 0, 0, 0, 18, 0, 0, 0, 0, 0, 0, 0, 20, 0, 0, 0, 0, 0, 0, 0, 36, 0, 0, 0, 0, 0, 0, 0, 64, 0, 0, 0, 0, 0, 0, 0, 193, 0, 0, 0, 0, 0, 0, 0, -1, 0, 0, 0], ["i8*",0,0,0,"i32",0,0,0,"i8*",0,0,0,"i32",0,0,0,"i8*",0,0,0,"i32",0,0,0,"i8*",0,0,0,"i32",0,0,0,"i8*",0,0,0,"i32",0,0,0,"i8*",0,0,0,"i32",0,0,0], ALLOC_STATIC);
__str68235=allocate([99,104,97,114,97,99,116,101,114,115,0] /* characters\00 */, "i8", ALLOC_STATIC);
__str69236=allocate([116,116,115,58,99,104,97,114,0] /* tts:char\00 */, "i8", ALLOC_STATIC);
__str70237=allocate([116,116,115,58,107,101,121,0] /* tts:key\00 */, "i8", ALLOC_STATIC);
__str71238=allocate([116,116,115,58,100,105,103,105,116,115,0] /* tts:digits\00 */, "i8", ALLOC_STATIC);
__str72239=allocate([116,101,108,101,112,104,111,110,101,0] /* telephone\00 */, "i8", ALLOC_STATIC);
__ZZL14ProcessSsmlTagPwPcRiiiE17mnem_sayas_format=allocate([0, 0, 0, 0, 1, 0, 0, 0, 0, 0, 0, 0, -1, 0, 0, 0], ["i8*",0,0,0,"i32",0,0,0,"i8*",0,0,0,"i32",0,0,0], ALLOC_STATIC);
__str73240=allocate([103,108,121,112,104,115,0] /* glyphs\00 */, "i8", ALLOC_STATIC);
__ZZL14ProcessSsmlTagPwPcRiiiE10mnem_break=allocate([0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 0, 0, 0, 0, 0, 0, 0, 2, 0, 0, 0, 0, 0, 0, 0, 3, 0, 0, 0, 0, 0, 0, 0, 4, 0, 0, 0, 0, 0, 0, 0, 5, 0, 0, 0, 0, 0, 0, 0, -1, 0, 0, 0], ["i8*",0,0,0,"i32",0,0,0,"i8*",0,0,0,"i32",0,0,0,"i8*",0,0,0,"i32",0,0,0,"i8*",0,0,0,"i32",0,0,0,"i8*",0,0,0,"i32",0,0,0,"i8*",0,0,0,"i32",0,0,0,"i8*",0,0,0,"i32",0,0,0], ALLOC_STATIC);
__str74241=allocate([120,45,119,101,97,107,0] /* x-weak\00 */, "i8", ALLOC_STATIC);
__str75242=allocate([119,101,97,107,0] /* weak\00 */, "i8", ALLOC_STATIC);
__str76243=allocate([109,101,100,105,117,109,0] /* medium\00 */, "i8", ALLOC_STATIC);
__str77244=allocate([120,45,115,116,114,111,110,103,0] /* x-strong\00 */, "i8", ALLOC_STATIC);
__ZZL14ProcessSsmlTagPwPcRiiiE13mnem_emphasis=allocate([0, 0, 0, 0, 1, 0, 0, 0, 0, 0, 0, 0, 2, 0, 0, 0, 0, 0, 0, 0, 3, 0, 0, 0, 0, 0, 0, 0, 4, 0, 0, 0, 0, 0, 0, 0, -1, 0, 0, 0], ["i8*",0,0,0,"i32",0,0,0,"i8*",0,0,0,"i32",0,0,0,"i8*",0,0,0,"i32",0,0,0,"i8*",0,0,0,"i32",0,0,0,"i8*",0,0,0,"i32",0,0,0], ALLOC_STATIC);
__str78245=allocate([114,101,100,117,99,101,100,0] /* reduced\00 */, "i8", ALLOC_STATIC);
__str79246=allocate([109,111,100,101,114,97,116,101,0] /* moderate\00 */, "i8", ALLOC_STATIC);
__ZZL14ProcessSsmlTagPwPcRiiiE12prosody_attr=allocate(20, "i8*", ALLOC_STATIC);
__str80247=allocate([114,97,116,101,0] /* rate\00 */, "i8", ALLOC_STATIC);
__str81248=allocate([118,111,108,117,109,101,0] /* volume\00 */, "i8", ALLOC_STATIC);
__str82249=allocate([114,97,110,103,101,0] /* range\00 */, "i8", ALLOC_STATIC);
__ZL22ignore_if_self_closing=allocate([0,1,1,1,1,0,0,0,0,1,1,0,1,0,1,0,0,0,0] /* \00\01\01\01\01\00\0 */, "i8", ALLOC_STATIC);
__str83250=allocate([102,105,101,108,100,0] /* field\00 */, "i8", ALLOC_STATIC);
__str84251=allocate([109,111,100,101,0] /* mode\00 */, "i8", ALLOC_STATIC);
__str85252=allocate([112,117,110,99,116,117,97,116,105,111,110,0] /* punctuation\00 */, "i8", ALLOC_STATIC);
__str86253=allocate([99,97,112,105,116,97,108,95,108,101,116,116,101,114,115,0] /* capital_letters\00 */, "i8", ALLOC_STATIC);
__str87254=allocate([108,101,118,101,108,0] /* level\00 */, "i8", ALLOC_STATIC);
__ZZL14ProcessSsmlTagPwPcRiiiE23emphasis_to_pitch_range=allocate([50,50,40,70,90,90] /* 22(FZZ */, "i8", ALLOC_STATIC);
__ZZL14ProcessSsmlTagPwPcRiiiE18emphasis_to_volume=allocate([100,100,70,110,140,140] /* ddFn\8C\8C */, "i8", ALLOC_STATIC);
__str88255=allocate([105,110,116,101,114,112,114,101,116,45,97,115,0] /* interpret-as\00 */, "i8", ALLOC_STATIC);
__str89256=allocate([102,111,114,109,97,116,0] /* format\00 */, "i8", ALLOC_STATIC);
__str90257=allocate([100,101,116,97,105,108,0] /* detail\00 */, "i8", ALLOC_STATIC);
__str91258=allocate([37,99,37,100,89,0] /* %c%dY\00 */, "i8", ALLOC_STATIC);
__ZL11sayas_start=allocate(1, "i32", ALLOC_STATIC);
__str92259=allocate([97,108,105,97,115,0] /* alias\00 */, "i8", ALLOC_STATIC);
__str94261=allocate([37,99,37,100,77,0] /* %c%dM\00 */, "i8", ALLOC_STATIC);
__str95262=allocate([115,114,99,0] /* src\00 */, "i8", ALLOC_STATIC);
__str96263=allocate([37,115,47,37,115,0] /* %s/%s\00 */, "i8", ALLOC_STATIC);
__str97264=allocate([37,99,37,100,73,0] /* %c%dI\00 */, "i8", ALLOC_STATIC);
__str98265=allocate([37,99,37,100,85,0] /* %c%dU\00 */, "i8", ALLOC_STATIC);
__str99266=allocate([115,116,114,101,110,103,116,104,0] /* strength\00 */, "i8", ALLOC_STATIC);
__ZZL14ProcessSsmlTagPwPcRiiiE11break_value=allocate([0, 0, 0, 0, 7, 0, 0, 0, 14, 0, 0, 0, 21, 0, 0, 0, 40, 0, 0, 0, 80, 0, 0, 0], ["i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0], ALLOC_STATIC);
__str100267=allocate([37,99,37,100,66,0] /* %c%dB\00 */, "i8", ALLOC_STATIC);
__str101268=allocate([116,105,109,101,0] /* time\00 */, "i8", ALLOC_STATIC);
__str102269=allocate([120,109,108,58,98,97,115,101,0] /* xml:base\00 */, "i8", ALLOC_STATIC);
__ZZL18GetVoiceAttributesPwiE11mnem_gender=allocate([0, 0, 0, 0, 1, 0, 0, 0, 0, 0, 0, 0, 2, 0, 0, 0, 0, 0, 0, 0, 3, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0], ["i8*",0,0,0,"i32",0,0,0,"i8*",0,0,0,"i32",0,0,0,"i8*",0,0,0,"i32",0,0,0,"i8*",0,0,0,"i32",0,0,0], ALLOC_STATIC);
__str105272=allocate([110,101,117,116,114,97,108,0] /* neutral\00 */, "i8", ALLOC_STATIC);
__str106273=allocate([120,109,108,58,108,97,110,103,0] /* xml:lang\00 */, "i8", ALLOC_STATIC);
__str108275=allocate([97,103,101,0] /* age\00 */, "i8", ALLOC_STATIC);
__ZZL14LoadSoundFile2PKcE4slot=allocate([-1], ["i32",0,0,0,0], ALLOC_STATIC);
__str111278=allocate([115,112,97,99,101,32,0] /* space \00 */, "i8", ALLOC_STATIC);
__str112279=allocate([116,97,98,32,0] /* tab \00 */, "i8", ALLOC_STATIC);
__str113280=allocate([117,110,100,101,114,115,99,111,114,101,32,0] /* underscore \00 */, "i8", ALLOC_STATIC);
__str114281=allocate([100,111,117,98,108,101,45,113,117,111,116,101,32,0] /* double-quote \00 */, "i8", ALLOC_STATIC);
__ZZL14ReplaceKeyNamePciRiE8keynames=allocate([0, 0, 0, 0, 57376, 0, 0, 0, 0, 0, 0, 0, 57353, 0, 0, 0, 0, 0, 0, 0, 57439, 0, 0, 0, 0, 0, 0, 0, 34, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0], ["i8*",0,0,0,"i32",0,0,0,"i8*",0,0,0,"i32",0,0,0,"i8*",0,0,0,"i32",0,0,0,"i8*",0,0,0,"i32",0,0,0,"i8*",0,0,0,"i32",0,0,0], ALLOC_STATIC);
__ZZL19SetProsodyParameteriPwP11PARAM_STACKE11mnem_volume=allocate([0, 0, 0, 0, 100, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 30, 0, 0, 0, 0, 0, 0, 0, 65, 0, 0, 0, 0, 0, 0, 0, 100, 0, 0, 0, 0, 0, 0, 0, 150, 0, 0, 0, 0, 0, 0, 0, 230, 0, 0, 0, 0, 0, 0, 0, -1, 0, 0, 0], ["i8*",0,0,0,"i32",0,0,0,"i8*",0,0,0,"i32",0,0,0,"i8*",0,0,0,"i32",0,0,0,"i8*",0,0,0,"i32",0,0,0,"i8*",0,0,0,"i32",0,0,0,"i8*",0,0,0,"i32",0,0,0,"i8*",0,0,0,"i32",0,0,0,"i8*",0,0,0,"i32",0,0,0], ALLOC_STATIC);
__str115282=allocate([115,105,108,101,110,116,0] /* silent\00 */, "i8", ALLOC_STATIC);
__str116283=allocate([120,45,115,111,102,116,0] /* x-soft\00 */, "i8", ALLOC_STATIC);
__str117=allocate([115,111,102,116,0] /* soft\00 */, "i8", ALLOC_STATIC);
__str118=allocate([108,111,117,100,0] /* loud\00 */, "i8", ALLOC_STATIC);
__str119=allocate([120,45,108,111,117,100,0] /* x-loud\00 */, "i8", ALLOC_STATIC);
__ZZL19SetProsodyParameteriPwP11PARAM_STACKE9mnem_rate=allocate([0, 0, 0, 0, 100, 0, 0, 0, 0, 0, 0, 0, 60, 0, 0, 0, 0, 0, 0, 0, 80, 0, 0, 0, 0, 0, 0, 0, 100, 0, 0, 0, 0, 0, 0, 0, 125, 0, 0, 0, 0, 0, 0, 0, 160, 0, 0, 0, 0, 0, 0, 0, -1, 0, 0, 0], ["i8*",0,0,0,"i32",0,0,0,"i8*",0,0,0,"i32",0,0,0,"i8*",0,0,0,"i32",0,0,0,"i8*",0,0,0,"i32",0,0,0,"i8*",0,0,0,"i32",0,0,0,"i8*",0,0,0,"i32",0,0,0,"i8*",0,0,0,"i32",0,0,0], ALLOC_STATIC);
__str120=allocate([120,45,115,108,111,119,0] /* x-slow\00 */, "i8", ALLOC_STATIC);
__str121=allocate([115,108,111,119,0] /* slow\00 */, "i8", ALLOC_STATIC);
__str122=allocate([102,97,115,116,0] /* fast\00 */, "i8", ALLOC_STATIC);
__str123=allocate([120,45,102,97,115,116,0] /* x-fast\00 */, "i8", ALLOC_STATIC);
__ZZL19SetProsodyParameteriPwP11PARAM_STACKE10mnem_pitch=allocate([0, 0, 0, 0, 100, 0, 0, 0, 0, 0, 0, 0, 70, 0, 0, 0, 0, 0, 0, 0, 85, 0, 0, 0, 0, 0, 0, 0, 100, 0, 0, 0, 0, 0, 0, 0, 110, 0, 0, 0, 0, 0, 0, 0, 120, 0, 0, 0, 0, 0, 0, 0, -1, 0, 0, 0], ["i8*",0,0,0,"i32",0,0,0,"i8*",0,0,0,"i32",0,0,0,"i8*",0,0,0,"i32",0,0,0,"i8*",0,0,0,"i32",0,0,0,"i8*",0,0,0,"i32",0,0,0,"i8*",0,0,0,"i32",0,0,0,"i8*",0,0,0,"i32",0,0,0], ALLOC_STATIC);
__str124=allocate([120,45,108,111,119,0] /* x-low\00 */, "i8", ALLOC_STATIC);
__str125=allocate([108,111,119,0] /* low\00 */, "i8", ALLOC_STATIC);
__str126=allocate([104,105,103,104,0] /* high\00 */, "i8", ALLOC_STATIC);
__str127=allocate([120,45,104,105,103,104,0] /* x-high\00 */, "i8", ALLOC_STATIC);
__ZZL19SetProsodyParameteriPwP11PARAM_STACKE10mnem_range=allocate([0, 0, 0, 0, 100, 0, 0, 0, 0, 0, 0, 0, 20, 0, 0, 0, 0, 0, 0, 0, 50, 0, 0, 0, 0, 0, 0, 0, 100, 0, 0, 0, 0, 0, 0, 0, 140, 0, 0, 0, 0, 0, 0, 0, 180, 0, 0, 0, 0, 0, 0, 0, -1, 0, 0, 0], ["i8*",0,0,0,"i32",0,0,0,"i8*",0,0,0,"i32",0,0,0,"i8*",0,0,0,"i32",0,0,0,"i8*",0,0,0,"i32",0,0,0,"i8*",0,0,0,"i32",0,0,0,"i8*",0,0,0,"i32",0,0,0,"i8*",0,0,0,"i32",0,0,0], ALLOC_STATIC);
__ZZL19SetProsodyParameteriPwP11PARAM_STACKE9mnem_tabs=allocate(20, "%struct.MNEM_TAB*", ALLOC_STATIC);
__ZZL17ProcessParamStackPcRiE10cmd_letter=allocate([0,83,65,80,82,0,67,0,0,0,0,0,70,0,0] /* \00SAPR\00C\00\00\00 */, "i8", ALLOC_STATIC);
__str128=allocate([37,99,37,100,37,99,0] /* %c%d%c\00 */, "i8", ALLOC_STATIC);
__ZZL16GetSsmlAttributePwPKcE5empty=allocate(4, "i32", ALLOC_STATIC);
__ZZL4GetCvE6ungot2=allocate(1, "i32", ALLOC_STATIC);
__ZZL4GetCvE5mask2=allocate([0,128,32,48] /* \00\80 0 */, "i8", ALLOC_STATIC);
__ZL6speed1=allocate([130], ["i32",0,0,0,0], ALLOC_STATIC);
__ZL6speed2=allocate([121], ["i32",0,0,0,0], ALLOC_STATIC);
__ZL6speed3=allocate([118], ["i32",0,0,0,0], ALLOC_STATIC);
__ZL12speed_lookup=allocate([255,255,255,255,255,253,249,245,242,238,235,232,228,225,222,218,216,213,210,207,204,201,198,196,193,191,188,186,183,181,179,176,174,172,169,168,165,163,161,159,158,155,153,152,150,148,146,145,143,141,139,137,136,135,133,131,130,129,127,126,124,123,122,120,119,118,117,115,114,113,112,111,110,109,107,106,105,104,103,102,101,100,99,98,97,96,95,94,93,92,91,90,89,89,88,87,86,85,84,83,82,82,81,80,80,79,78,77,76,76,75,75,74,73,72,71,71,70,69,69,68,67,67,66,66,65,64,64,63,62,62,61,61,60,59,59,58,58,57,57,56,56,55,54,54,53,53,52,52,52,51,50,50,49,49,48,48,47,47,46,46,46,45,45,44,44,44,43,43,42,41,40,40,40,39,39,39,38,38,38,37,37,37,36,36,35,35,35,35,34,34,34,33,33,33,32,32,31,31,31,30,30,30,29,29,29,29,28,28,27,27,27,27,26,26,26,26,25,25,25,24,24,24,24,23,23,23,23,22,22,22,21,21,21,21,20,20,20,20,19,19,19,18,18,17,17,17,16,16,16,16,16,16,15,15,15,15,14,14,14,13,13,13,12,12,12,12,11,11,11,11,10,10,10,9,9,9,8,8,8] /* \FF\FF\FF\FF\FF\FD\F */, "i8", ALLOC_STATIC);
__ZL14wav_factor_350=allocate([120,121,120,119,119,118,118,117,116,116,115,114,113,112,112,111,111,110,109,108,107,106,106,104,103,103,102,102,102,101,101,99,98,98,97,96,96,95,94,93,91,90,91,90,89,88,86,85,86,85,85,84,82,81,80,79,77,78,78,76,77,75,75,74,73,71,72,70,69,69,69,67,65,64,63,63,63,61,61,59,59,59,58,56,57,58,56,54,53,52,52,53,52,52,50,48,47,47,45,46,45] /* xyxwwvvuttsrqppoonml */, "i8", ALLOC_STATIC);
__ZL16pause_factor_350=allocate([22,22,22,22,22,22,22,21,21,21,21,20,20,19,19,18,17,16,15,15,15,15,15,15,15] /* \16\16\16\16\16\16\1 */, "i8", ALLOC_STATIC);
__ZZ11CalcLengthsP10TranslatorE14more_syllables=allocate(1, "i32", ALLOC_STATIC);
__str294=allocate([101,115,112,101,97,107,58,32,66,97,100,32,105,110,116,111,110,97,116,105,111,110,32,100,97,116,97,10,0] /* espeak: Bad intonati */, "i8", ALLOC_STATIC);
__ZL18letter_accents_0e0=allocate([870, 0, 358, 0, 614, 0, 1190, 0, 678, 0, 1062, 0, -30042, 0, 552, 0, 874, 0, 362, 0, 618, 0, 682, 0, 878, 0, 366, 0, 622, 0, 686, 0, 41, 0, 1203, 0, 884, 0, 372, 0, 628, 0, 1204, 0, 692, 0, 0, 0, 1140, 0, 890, 0, 378, 0, 634, 0, 698, 0, 382, 0, 57, 0, 702, 0, 0, 0, 934, 0, 0, 0, 422, 0, 0, 0, 998, 0, 0, 0, 360, 0, 0, 0, 616, 0, 0, 0, 808, 0, 0, 0, 488, 0, 0, 0, 489, 0, 0, 0, 1129, 0, 0, 0, 938, 0, 0, 0, 426, 0, 0, 0, 810, 0, 0, 0, 1002, 0, 0, 0, 490, 0, 0, 0, 620, 0, 0, 0, 428, 0, 0, 0, 812, 0, 0, 0, 556, 0, 0, 0, 621, 0, 0, 0, 1133, 0, 0, 0, 1198, 0, 0, 0, 942, 0, 0, 0, 430, 0, 0, 0, 1006, 0, 0, 0, 46, 0, 0, 0, -29714, 0, 0, 0, 623, 0, 0, 0, 560, 0, 48, 0, 0, 0, 369, 0, 0, 0, 561, 0, 0, 0, 497, 0, 0, 0, 817, 0, 0, 0, 1137, 0, 0, 0, 371, 0, 0, 0, 563, 0, 0, 0, 499, 0, 51, 0, 0, 0, 51, 0, 0, 0, 948, 0, 0, 0, 436, 0, 0, 0, 756, 0, 0, 0, -30028, 0, 0, 0, 375, 0, 0, 0, 567, 0, 0, 0, 503, 0, 0, 0, 376, 0, 0, 0, 632, 0, 0, 0, 568, 0, 0, 0, 504, 0, 0, 0, 569, 0, 0, 0, 505, 0, 0, 0, 1145, 0, 0, 0, 1210, 0, 0, 0, 954, 0, 0, 0, 442, 0, 0, 0, 1082, 0, 0, 0, 762, 0, 0, 0, 1018, 0, 0, 0, 636, 0, 0, 0, 638, 0, 0, 0, 0, 0, 383, 0, 0, 0, 831, 0, 0, 0, 511, 0, 56, 0], ["i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0], ALLOC_STATIC);
__ZL18letter_accents_250=allocate([166, 0, 1, 0, 129, 0, 1383, 0, 0, 0, 296, 0, 1321, 0, 1385, 0, 234, 0, 0, 0, 1346, 0, 0, 0, 195, 0, 7491, 0, 0, 0, 1263, 0, 1388, 0, 44, 0, 108, 0, 4, 0, 0, 0, 173, 0, 1389, 0, 0, 0, 1262, 0, 5, 0, 110, 0, 1201, 0, 1265, 0, 1329, 0, -28687, 0, 178, 0, 0, 0, 1394, 0, 0, 0, 1331, 0, 115, 0, 1268, 0, -25932, 0, 0, 0, 8, 0, 183, 0, 0, 0, 5431, 0, 0, 0, 1335, 0, 0, 0, 205, 0, 119, 0, 2231, 0, 1336, 0, 0, 0, 0, 0, 201, 0, 265, 0, 185, 0, 1337, 0, 1274, 0, 10, 0, 1403, 0, 187, 0, 188, 0, 190, 0, 126, 0, 1343, 0, 319, 0, 0, 0, 267, 0, 0, 0, 204, 0, 140, 0, 0, 0, 0, 0, 103, 0, 0, 0, 3436, 0, 109, 0, 303, 0, 176, 0, 113, 0, 1398, 0, 1100, 0, 7244, 0, -28695, 0, 0, 0, -12311, 0, -29127, 0, 0, 0, -12743, 0], ["i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0], ALLOC_STATIC);
__ZL13non_ascii_tab=allocate([0, 0, 945, 0, 601, 0, 603, 0, 947, 0, 953, 0, 339, 0, 969, 0, 966, 0, 643, 0, 965, 0, 658, 0, 660, 0, 638, 0], ["i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0], ALLOC_STATIC);
__ZL11accents_tab=allocate([0, 0, 0, 0, 1, 0, 0, 0, 0, 0, 0, 0, 1, 0, 0, 0, 0, 0, 0, 0, 1, 0, 0, 0, 0, 0, 0, 0, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0], ["i8*",0,0,0,"i32",0,0,0,"i8*",0,0,0,"i32",0,0,0,"i8*",0,0,0,"i32",0,0,0,"i8*",0,0,0,"i32",0,0,0,"i8*",0,0,0,"i32",0,0,0,"i8*",0,0,0,"i32",0,0,0,"i8*",0,0,0,"i32",0,0,0,"i8*",0,0,0,"i32",0,0,0,"i8*",0,0,0,"i32",0,0,0,"i8*",0,0,0,"i32",0,0,0,"i8*",0,0,0,"i32",0,0,0,"i8*",0,0,0,"i32",0,0,0,"i8*",0,0,0,"i32",0,0,0,"i8*",0,0,0,"i32",0,0,0,"i8*",0,0,0,"i32",0,0,0,"i8*",0,0,0,"i32",0,0,0,"i8*",0,0,0,"i32",0,0,0,"i8*",0,0,0,"i32",0,0,0,"i8*",0,0,0,"i32",0,0,0,"i8*",0,0,0,"i32",0,0,0,"i8*",0,0,0,"i32",0,0,0,"i8*",0,0,0,"i32",0,0,0], ALLOC_STATIC);
__str297=allocate([37,115,37,99,37,115,37,99,37,115,37,115,0] /* %s%c%s%c%s%s\00 */, "i8", ALLOC_STATIC);
__str1298=allocate([37,115,37,99,37,99,37,115,0] /* %s%c%c%s\00 */, "i8", ALLOC_STATIC);
__str2299=allocate([37,99,37,115,37,99,37,115,37,99,0] /* %c%s%c%s%c\00 */, "i8", ALLOC_STATIC);
__ZZ12LookupLetterP10TranslatorjiPciE13single_letter=allocate(10, "i8", ALLOC_STATIC);
__str5302=allocate([95,35,37,100,32,0] /* _#%d \00 */, "i8", ALLOC_STATIC);
__str6303=allocate([95,99,97,112,0] /* _cap\00 */, "i8", ALLOC_STATIC);
__str7304=allocate([37,99,101,110,0] /* %cen\00 */, "i8", ALLOC_STATIC);
__str8305=allocate([95,63,65,0] /* _?A\00 */, "i8", ALLOC_STATIC);
__str9306=allocate([95,63,63,0] /* _??\00 */, "i8", ALLOC_STATIC);
__str10307=allocate([37,120,0] /* %x\00 */, "i8", ALLOC_STATIC);
__str11308=allocate([37,99,37,115,37,115,0] /* %c%s%s\00 */, "i8", ALLOC_STATIC);
__str12309=allocate([105,120,99,109,118,108,100,0] /* ixcmvld\00 */, "i8", ALLOC_STATIC);
__ZZ14TranslateRomanP10TranslatorPcS1_P8WORD_TABE12roman_values=allocate([1, 0, 0, 0, 10, 0, 0, 0, 100, 0, 0, 0, 1000, 0, 0, 0, 5, 0, 0, 0, 50, 0, 0, 0, 500, 0, 0, 0], ["i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0], ALLOC_STATIC);
__str13310=allocate([95,114,111,109,97,110,0] /* _roman\00 */, "i8", ALLOC_STATIC);
__str14311=allocate([32,32,37,100,32,32,32,32,0] /*   %d    \00 */, "i8", ALLOC_STATIC);
__ZZL17TranslateNumber_1P10TranslatorPcS1_PjP8WORD_TABiE9str_pause=allocate([11,0] /* \0B\00 */, "i8", ALLOC_STATIC);
__ZL14n_digit_lookup=allocate(1, "i32", ALLOC_STATIC);
__ZL12digit_lookup=allocate(1, "i8*", ALLOC_STATIC);
__ZL14number_control=allocate(1, "i32", ALLOC_STATIC);
__ZL23speak_missing_thousands=allocate(1, "i32", ALLOC_STATIC);
__ZL11ph_ordinal2=allocate(12, "i8", ALLOC_STATIC);
__str15312=allocate([95,35,37,115,0] /* _#%s\00 */, "i8", ALLOC_STATIC);
__str16313=allocate([95,48,0] /* _0\00 */, "i8", ALLOC_STATIC);
__str17314=allocate([95,100,112,116,0] /* _dpt\00 */, "i8", ALLOC_STATIC);
__str18315=allocate([95,48,77,37,100,0] /* _0M%d\00 */, "i8", ALLOC_STATIC);
__str19316=allocate([95,46,0] /* _.\00 */, "i8", ALLOC_STATIC);
__str20317=allocate([37,115,37,115,37,115,37,115,0] /* %s%s%s%s\00 */, "i8", ALLOC_STATIC);
__str21318=allocate([95,48,90,37,100,0] /* _0Z%d\00 */, "i8", ALLOC_STATIC);
__str22319=allocate([95,100,112,116,50,0] /* _dpt2\00 */, "i8", ALLOC_STATIC);
__str23320=allocate([95,37,100,102,0] /* _%df\00 */, "i8", ALLOC_STATIC);
__str24321=allocate([95,37,100,37,99,120,0] /* _%d%cx\00 */, "i8", ALLOC_STATIC);
__str25322=allocate([95,37,100,37,99,0] /* _%d%c\00 */, "i8", ALLOC_STATIC);
__str26323=allocate([95,37,100,101,0] /* _%de\00 */, "i8", ALLOC_STATIC);
__str27324=allocate([95,37,100,97,0] /* _%da\00 */, "i8", ALLOC_STATIC);
__str28325=allocate([95,37,100,0] /* _%d\00 */, "i8", ALLOC_STATIC);
__str29326=allocate([95,37,100,88,37,99,0] /* _%dX%c\00 */, "i8", ALLOC_STATIC);
__str30327=allocate([95,37,100,88,0] /* _%dX\00 */, "i8", ALLOC_STATIC);
__str31328=allocate([95,111,114,100,50,48,0] /* _ord20\00 */, "i8", ALLOC_STATIC);
__str32329=allocate([95,111,114,100,0] /* _ord\00 */, "i8", ALLOC_STATIC);
__str33330=allocate([95,48,97,110,100,0] /* _0and\00 */, "i8", ALLOC_STATIC);
__str35332=allocate([95,48,67,111,0] /* _0Co\00 */, "i8", ALLOC_STATIC);
__str36333=allocate([95,48,67,48,0] /* _0C0\00 */, "i8", ALLOC_STATIC);
__str37334=allocate([95,48,67,0] /* _0C\00 */, "i8", ALLOC_STATIC);
__str39336=allocate([95,37,100,67,111,0] /* _%dCo\00 */, "i8", ALLOC_STATIC);
__str40337=allocate([95,37,100,67,48,0] /* _%dC0\00 */, "i8", ALLOC_STATIC);
__str41338=allocate([95,37,100,67,0] /* _%dC\00 */, "i8", ALLOC_STATIC);
__str42339=allocate([95,37,100,77,37,100,111,0] /* _%dM%do\00 */, "i8", ALLOC_STATIC);
__str43340=allocate([95,37,100,77,37,100,101,0] /* _%dM%de\00 */, "i8", ALLOC_STATIC);
__str44341=allocate([95,37,100,77,37,100,120,0] /* _%dM%dx\00 */, "i8", ALLOC_STATIC);
__str45342=allocate([95,37,100,77,37,100,0] /* _%dM%d\00 */, "i8", ALLOC_STATIC);
__str46343=allocate([95,48,111,102,0] /* _0of\00 */, "i8", ALLOC_STATIC);
__str47344=allocate([95,37,115,37,100,111,0] /* _%s%do\00 */, "i8", ALLOC_STATIC);
__str48345=allocate([95,37,115,37,100,101,0] /* _%s%de\00 */, "i8", ALLOC_STATIC);
__str49346=allocate([95,37,115,37,100,120,0] /* _%s%dx\00 */, "i8", ALLOC_STATIC);
__str50347=allocate([95,37,115,37,100,0] /* _%s%d\00 */, "i8", ALLOC_STATIC);
__str51348=allocate([95,48,77,50,0] /* _0M2\00 */, "i8", ALLOC_STATIC);
__str52349=allocate([95,37,100,77,49,0] /* _%dM1\00 */, "i8", ALLOC_STATIC);
__str53350=allocate([95,48,77,49,0] /* _0M1\00 */, "i8", ALLOC_STATIC);
__str54351=allocate([48,77,65,0] /* 0MA\00 */, "i8", ALLOC_STATIC);
__str55352=allocate([49,77,65,0] /* 1MA\00 */, "i8", ALLOC_STATIC);
__str56353=allocate([48,77,0] /* 0M\00 */, "i8", ALLOC_STATIC);
__str57354=allocate([95,108,105,103,0] /* _lig\00 */, "i8", ALLOC_STATIC);
__str58355=allocate([95,115,109,99,0] /* _smc\00 */, "i8", ALLOC_STATIC);
__str59356=allocate([95,116,117,114,0] /* _tur\00 */, "i8", ALLOC_STATIC);
__str60357=allocate([95,114,101,118,0] /* _rev\00 */, "i8", ALLOC_STATIC);
__str61358=allocate([95,99,114,108,0] /* _crl\00 */, "i8", ALLOC_STATIC);
__str62359=allocate([95,97,99,117,0] /* _acu\00 */, "i8", ALLOC_STATIC);
__str63360=allocate([95,98,114,118,0] /* _brv\00 */, "i8", ALLOC_STATIC);
__str64361=allocate([95,104,97,99,0] /* _hac\00 */, "i8", ALLOC_STATIC);
__str65362=allocate([95,99,101,100,0] /* _ced\00 */, "i8", ALLOC_STATIC);
__str66363=allocate([95,99,105,114,0] /* _cir\00 */, "i8", ALLOC_STATIC);
__str67364=allocate([95,100,105,97,0] /* _dia\00 */, "i8", ALLOC_STATIC);
__str68365=allocate([95,97,99,50,0] /* _ac2\00 */, "i8", ALLOC_STATIC);
__str69366=allocate([95,100,111,116,0] /* _dot\00 */, "i8", ALLOC_STATIC);
__str70367=allocate([95,103,114,118,0] /* _grv\00 */, "i8", ALLOC_STATIC);
__str71368=allocate([95,109,99,110,0] /* _mcn\00 */, "i8", ALLOC_STATIC);
__str72369=allocate([95,111,103,111,0] /* _ogo\00 */, "i8", ALLOC_STATIC);
__str73370=allocate([95,114,110,103,0] /* _rng\00 */, "i8", ALLOC_STATIC);
__str74371=allocate([95,115,116,107,0] /* _stk\00 */, "i8", ALLOC_STATIC);
__str75372=allocate([95,116,108,100,0] /* _tld\00 */, "i8", ALLOC_STATIC);
__str76373=allocate([95,98,97,114,0] /* _bar\00 */, "i8", ALLOC_STATIC);
__str77374=allocate([95,114,102,120,0] /* _rfx\00 */, "i8", ALLOC_STATIC);
__str78375=allocate([95,104,111,107,0] /* _hok\00 */, "i8", ALLOC_STATIC);
_option_mbrola_phonemes=allocate(1, "i32", ALLOC_STATIC);
__ZL10mbrola_tab=allocate(1, "%struct.MBROLA_TAB*", ALLOC_STATIC);
__ZL15mbr_name_prefix=allocate(1, "i32", ALLOC_STATIC);
__str382=allocate([37,115,47,109,98,114,111,108,97,47,37,115,0] /* %s/mbrola/%s\00 */, "i8", ALLOC_STATIC);
__str1383=allocate([47,117,115,114,47,115,104,97,114,101,47,109,98,114,111,108,97,47,37,115,0] /* /usr/share/mbrola/%s */, "i8", ALLOC_STATIC);
__str2384=allocate([47,117,115,114,47,115,104,97,114,101,47,109,98,114,111,108,97,47,37,115,47,37,115,0] /* /usr/share/mbrola/%s */, "i8", ALLOC_STATIC);
__str3385=allocate([47,117,115,114,47,115,104,97,114,101,47,109,98,114,111,108,97,47,118,111,105,99,101,115,47,37,115,0] /* /usr/share/mbrola/vo */, "i8", ALLOC_STATIC);
__str4386=allocate([37,115,47,109,98,114,111,108,97,95,112,104,47,37,115,0] /* %s/mbrola_ph/%s\00 */, "i8", ALLOC_STATIC);
__ZL14mbrola_control=allocate(1, "i32", ALLOC_STATIC);
__ZZ15MbrolaTranslateP12PHONEME_LISTiiP7__sFILEE4phix=allocate(1, "i32", ALLOC_STATIC);
__ZZ15MbrolaTranslateP12PHONEME_LISTiiP7__sFILEE11embedded_ix=allocate(1, "i32", ALLOC_STATIC);
__ZZ15MbrolaTranslateP12PHONEME_LISTiiP7__sFILEE10word_count=allocate(1, "i32", ALLOC_STATIC);
__str6388=allocate([37,115,9,0] /* %s\09\00 */, "i8", ALLOC_STATIC);
__str8390=allocate([37,100,9,37,115,0] /* %d\09%s\00 */, "i8", ALLOC_STATIC);
__str9391=allocate([37,115,9,37,100,9,37,115,0] /* %s\09%d\09%s\00 */, "i8", ALLOC_STATIC);
__str10392=allocate([37,100,10,37,115,9,0] /* %d\0A%s\09\00 */, "i8", ALLOC_STATIC);
__str11393=allocate([37,100,37,115,10,0] /* %d%s\0A\00 */, "i8", ALLOC_STATIC);
__str12394=allocate([95,32,9,37,100,10,0] /* _ \09%d\0A\00 */, "i8", ALLOC_STATIC);
__ZZ10MbrolaFilliiE9n_samples=allocate(1, "i32", ALLOC_STATIC);
__ZZL10WritePitchiiiiiE6output=allocate(50, "i8", ALLOC_STATIC);
__str13395=allocate([32,48,32,37,100,0] /*  0 %d\00 */, "i8", ALLOC_STATIC);
__str14396=allocate([32,37,100,32,37,100,0] /*  %d %d\00 */, "i8", ALLOC_STATIC);
__str15397=allocate([10,0] /* \0A\00 */, "i8", ALLOC_STATIC);
__str16398=allocate([9,49,48,48,32,37,100,10,0] /* \09100 %d\0A\00 */, "i8", ALLOC_STATIC);
__ZZL10GetMbrNameP12PHONEME_LISTP11PHONEME_TABS2_S2_PiS3_S3_E4mnem=allocate(1, "i32", ALLOC_STATIC);
__str399=allocate([49,46,52,53,46,48,52,32,32,50,53,46,65,112,114,46,49,49,0] /* 1.45.04  25.Apr.11\0 */, "i8", ALLOC_STATIC);
_version_string=allocate(1, "i8*", ALLOC_STATIC);
_version_phdata=allocate([83200], ["i32",0,0,0,0], ALLOC_STATIC);
_option_device_number=allocate([-1], ["i32",0,0,0,0], ALLOC_STATIC);
_f_logespeak=allocate(1, "%struct.__sFILE*", ALLOC_STATIC);
_logging_type=allocate(1, "i32", ALLOC_STATIC);
_n_phoneme_tab=allocate(1, "i32", ALLOC_STATIC);
_current_phoneme_table=allocate(1, "i32", ALLOC_STATIC);
_phoneme_tab=allocate(1024, "%struct.PHONEME_TAB*", ALLOC_STATIC);
_phoneme_tab_flags=allocate(256, "i8", ALLOC_STATIC);
_phoneme_index=allocate(1, "i16*", ALLOC_STATIC);
_phondata_ptr=allocate(1, "i8*", ALLOC_STATIC);
_wavefile_data=allocate(1, "i8*", ALLOC_STATIC);
__ZL16phoneme_tab_data=allocate(1, "i8*", ALLOC_STATIC);
_n_phoneme_tables=allocate(1, "i32", ALLOC_STATIC);
_phoneme_tab_list=allocate(4800, ["i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","%struct.PHONEME_TAB*",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","%struct.PHONEME_TAB*",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","%struct.PHONEME_TAB*",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","%struct.PHONEME_TAB*",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","%struct.PHONEME_TAB*",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","%struct.PHONEME_TAB*",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","%struct.PHONEME_TAB*",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","%struct.PHONEME_TAB*",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","%struct.PHONEME_TAB*",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","%struct.PHONEME_TAB*",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","%struct.PHONEME_TAB*",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","%struct.PHONEME_TAB*",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","%struct.PHONEME_TAB*",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","%struct.PHONEME_TAB*",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","%struct.PHONEME_TAB*",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","%struct.PHONEME_TAB*",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","%struct.PHONEME_TAB*",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","%struct.PHONEME_TAB*",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","%struct.PHONEME_TAB*",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","%struct.PHONEME_TAB*",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","%struct.PHONEME_TAB*",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","%struct.PHONEME_TAB*",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","%struct.PHONEME_TAB*",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","%struct.PHONEME_TAB*",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","%struct.PHONEME_TAB*",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","%struct.PHONEME_TAB*",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","%struct.PHONEME_TAB*",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","%struct.PHONEME_TAB*",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","%struct.PHONEME_TAB*",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","%struct.PHONEME_TAB*",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","%struct.PHONEME_TAB*",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","%struct.PHONEME_TAB*",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","%struct.PHONEME_TAB*",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","%struct.PHONEME_TAB*",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","%struct.PHONEME_TAB*",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","%struct.PHONEME_TAB*",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","%struct.PHONEME_TAB*",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","%struct.PHONEME_TAB*",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","%struct.PHONEME_TAB*",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","%struct.PHONEME_TAB*",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","%struct.PHONEME_TAB*",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","%struct.PHONEME_TAB*",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","%struct.PHONEME_TAB*",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","%struct.PHONEME_TAB*",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","%struct.PHONEME_TAB*",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","%struct.PHONEME_TAB*",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","%struct.PHONEME_TAB*",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","%struct.PHONEME_TAB*",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","%struct.PHONEME_TAB*",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","%struct.PHONEME_TAB*",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","%struct.PHONEME_TAB*",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","%struct.PHONEME_TAB*",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","%struct.PHONEME_TAB*",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","%struct.PHONEME_TAB*",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","%struct.PHONEME_TAB*",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","%struct.PHONEME_TAB*",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","%struct.PHONEME_TAB*",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","%struct.PHONEME_TAB*",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","%struct.PHONEME_TAB*",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","%struct.PHONEME_TAB*",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","%struct.PHONEME_TAB*",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","%struct.PHONEME_TAB*",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","%struct.PHONEME_TAB*",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","%struct.PHONEME_TAB*",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","%struct.PHONEME_TAB*",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","%struct.PHONEME_TAB*",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","%struct.PHONEME_TAB*",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","%struct.PHONEME_TAB*",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","%struct.PHONEME_TAB*",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","%struct.PHONEME_TAB*",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","%struct.PHONEME_TAB*",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","%struct.PHONEME_TAB*",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","%struct.PHONEME_TAB*",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","%struct.PHONEME_TAB*",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","%struct.PHONEME_TAB*",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","%struct.PHONEME_TAB*",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","%struct.PHONEME_TAB*",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","%struct.PHONEME_TAB*",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","%struct.PHONEME_TAB*",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","%struct.PHONEME_TAB*",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","%struct.PHONEME_TAB*",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","%struct.PHONEME_TAB*",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","%struct.PHONEME_TAB*",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","%struct.PHONEME_TAB*",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","%struct.PHONEME_TAB*",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","%struct.PHONEME_TAB*",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","%struct.PHONEME_TAB*",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","%struct.PHONEME_TAB*",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","%struct.PHONEME_TAB*",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","%struct.PHONEME_TAB*",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","%struct.PHONEME_TAB*",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","%struct.PHONEME_TAB*",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","%struct.PHONEME_TAB*",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","%struct.PHONEME_TAB*",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","%struct.PHONEME_TAB*",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","%struct.PHONEME_TAB*",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","%struct.PHONEME_TAB*",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","%struct.PHONEME_TAB*",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","%struct.PHONEME_TAB*",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","%struct.PHONEME_TAB*",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0], ALLOC_STATIC);
_phoneme_tab_number=allocate(1, "i32", ALLOC_STATIC);
_wavefile_ix=allocate(1, "i32", ALLOC_STATIC);
_wavefile_amp=allocate(1, "i32", ALLOC_STATIC);
_wavefile_ix2=allocate(1, "i32", ALLOC_STATIC);
_wavefile_amp2=allocate(1, "i32", ALLOC_STATIC);
_seq_len_adjust=allocate(1, "i32", ALLOC_STATIC);
_vowel_transition=allocate(16, "i32", ALLOC_STATIC);
_vowel_transition0=allocate(1, "i32", ALLOC_STATIC);
_vowel_transition1=allocate(1, "i32", ALLOC_STATIC);
__str1412=allocate([112,104,111,110,116,97,98,0] /* phontab\00 */, "i8", ALLOC_STATIC);
__str2413=allocate([112,104,111,110,105,110,100,101,120,0] /* phonindex\00 */, "i8", ALLOC_STATIC);
__str3414=allocate([112,104,111,110,100,97,116,97,0] /* phondata\00 */, "i8", ALLOC_STATIC);
__str4415=allocate([105,110,116,111,110,97,116,105,111,110,115,0] /* intonations\00 */, "i8", ALLOC_STATIC);
__ZZ11LookupSpectP11PHONEME_TABiP10FMT_PARAMSPiP12PHONEME_LISTE10frames_buf=allocate(200, ["i16",0,"i16",0,"%struct.frame_t*",0,0,0,"i16",0,"i16",0,"%struct.frame_t*",0,0,0,"i16",0,"i16",0,"%struct.frame_t*",0,0,0,"i16",0,"i16",0,"%struct.frame_t*",0,0,0,"i16",0,"i16",0,"%struct.frame_t*",0,0,0,"i16",0,"i16",0,"%struct.frame_t*",0,0,0,"i16",0,"i16",0,"%struct.frame_t*",0,0,0,"i16",0,"i16",0,"%struct.frame_t*",0,0,0,"i16",0,"i16",0,"%struct.frame_t*",0,0,0,"i16",0,"i16",0,"%struct.frame_t*",0,0,0,"i16",0,"i16",0,"%struct.frame_t*",0,0,0,"i16",0,"i16",0,"%struct.frame_t*",0,0,0,"i16",0,"i16",0,"%struct.frame_t*",0,0,0,"i16",0,"i16",0,"%struct.frame_t*",0,0,0,"i16",0,"i16",0,"%struct.frame_t*",0,0,0,"i16",0,"i16",0,"%struct.frame_t*",0,0,0,"i16",0,"i16",0,"%struct.frame_t*",0,0,0,"i16",0,"i16",0,"%struct.frame_t*",0,0,0,"i16",0,"i16",0,"%struct.frame_t*",0,0,0,"i16",0,"i16",0,"%struct.frame_t*",0,0,0,"i16",0,"i16",0,"%struct.frame_t*",0,0,0,"i16",0,"i16",0,"%struct.frame_t*",0,0,0,"i16",0,"i16",0,"%struct.frame_t*",0,0,0,"i16",0,"i16",0,"%struct.frame_t*",0,0,0,"i16",0,"i16",0,"%struct.frame_t*",0,0,0], ALLOC_STATIC);
__str5416=allocate([101,115,112,101,97,107,58,32,78,111,32,101,110,118,101,108,111,112,101,10,0] /* espeak: No envelope\ */, "i8", ALLOC_STATIC);
__str7418=allocate([99,111,110,102,105,103,0] /* config\00 */, "i8", ALLOC_STATIC);
__str9420=allocate([108,111,103,0] /* log\00 */, "i8", ALLOC_STATIC);
__str10421=allocate([37,100,32,37,115,0] /* %d %s\00 */, "i8", ALLOC_STATIC);
__str11422=allocate([119,0] /* w\00 */, "i8", ALLOC_STATIC);
__str13424=allocate([112,97,95,100,101,118,105,99,101,0] /* pa_device\00 */, "i8", ALLOC_STATIC);
__str15426=allocate([115,111,117,110,100,105,99,111,110,0] /* soundicon\00 */, "i8", ALLOC_STATIC);
__str16427=allocate([95,37,99,32,37,115,0] /* _%c %s\00 */, "i8", ALLOC_STATIC);
_this_ph_data=allocate(152, ["i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8",0,0], ALLOC_STATIC);
__ZZL13NumInstnWordsPtE7n_words=allocate([1,1,1,1,1,1,1,1,1,2,4] /* \01\01\01\01\01\01\0 */, "i8", ALLOC_STATIC);
__ZZL18InterpretConditionP10TranslatoriP12PHONEME_LISTiE11ph_position=allocate([0, 0, 0, 0, 1, 0, 0, 0, 2, 0, 0, 0, 3, 0, 0, 0, 2, 0, 0, 0, 0, 0, 0, 0, 1, 0, 0, 0, 3, 0, 0, 0], ["i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0], ALLOC_STATIC);
__ZZL15StressConditionP10TranslatorP12PHONEME_LISTiiE15condition_level=allocate([1, 0, 0, 0, 2, 0, 0, 0, 4, 0, 0, 0, 15, 0, 0, 0], ["i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0], ALLOC_STATIC);
__str17428=allocate([73,110,118,97,108,105,100,32,105,110,115,116,114,117,99,116,105,111,110,32,37,46,52,120,32,102,111,114,32,112,104,111,110,101,109,101,32,39,37,115,39,10,0] /* Invalid instruction  */, "i8", ALLOC_STATIC);
__str18429=allocate([114,98,0] /* rb\00 */, "i8", ALLOC_STATIC);
__str19430=allocate([67,97,110,39,116,32,114,101,97,100,32,100,97,116,97,32,102,105,108,101,58,32,39,37,115,39,10,0] /* Can't read data file */, "i8", ALLOC_STATIC);
_n_phoneme_list=allocate(1, "i32", ALLOC_STATIC);
_phoneme_list=allocate(24000, ["i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0,"i8","i8","i8","i8","i16",0,"i16",0,"%struct.PHONEME_TAB*",0,0,0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8",0,0], ALLOC_STATIC);
_mbrola_delay=allocate(1, "i32", ALLOC_STATIC);
_mbrola_name=allocate(20, "i8", ALLOC_STATIC);
_speed=allocate(64, "i32", ALLOC_STATIC);
__ZL9new_voice=allocate(1, "%struct.voice_t*", ALLOC_STATIC);
_n_soundicon_tab=allocate([4], ["i32",0,0,0,0], ALLOC_STATIC);
_soundicon_tab=allocate(1280, ["i32",0,0,0,"i32",0,0,0,"i8*",0,0,0,"i8*",0,0,0,"i32",0,0,0,"i32",0,0,0,"i8*",0,0,0,"i8*",0,0,0,"i32",0,0,0,"i32",0,0,0,"i8*",0,0,0,"i8*",0,0,0,"i32",0,0,0,"i32",0,0,0,"i8*",0,0,0,"i8*",0,0,0,"i32",0,0,0,"i32",0,0,0,"i8*",0,0,0,"i8*",0,0,0,"i32",0,0,0,"i32",0,0,0,"i8*",0,0,0,"i8*",0,0,0,"i32",0,0,0,"i32",0,0,0,"i8*",0,0,0,"i8*",0,0,0,"i32",0,0,0,"i32",0,0,0,"i8*",0,0,0,"i8*",0,0,0,"i32",0,0,0,"i32",0,0,0,"i8*",0,0,0,"i8*",0,0,0,"i32",0,0,0,"i32",0,0,0,"i8*",0,0,0,"i8*",0,0,0,"i32",0,0,0,"i32",0,0,0,"i8*",0,0,0,"i8*",0,0,0,"i32",0,0,0,"i32",0,0,0,"i8*",0,0,0,"i8*",0,0,0,"i32",0,0,0,"i32",0,0,0,"i8*",0,0,0,"i8*",0,0,0,"i32",0,0,0,"i32",0,0,0,"i8*",0,0,0,"i8*",0,0,0,"i32",0,0,0,"i32",0,0,0,"i8*",0,0,0,"i8*",0,0,0,"i32",0,0,0,"i32",0,0,0,"i8*",0,0,0,"i8*",0,0,0,"i32",0,0,0,"i32",0,0,0,"i8*",0,0,0,"i8*",0,0,0,"i32",0,0,0,"i32",0,0,0,"i8*",0,0,0,"i8*",0,0,0,"i32",0,0,0,"i32",0,0,0,"i8*",0,0,0,"i8*",0,0,0,"i32",0,0,0,"i32",0,0,0,"i8*",0,0,0,"i8*",0,0,0,"i32",0,0,0,"i32",0,0,0,"i8*",0,0,0,"i8*",0,0,0,"i32",0,0,0,"i32",0,0,0,"i8*",0,0,0,"i8*",0,0,0,"i32",0,0,0,"i32",0,0,0,"i8*",0,0,0,"i8*",0,0,0,"i32",0,0,0,"i32",0,0,0,"i8*",0,0,0,"i8*",0,0,0,"i32",0,0,0,"i32",0,0,0,"i8*",0,0,0,"i8*",0,0,0,"i32",0,0,0,"i32",0,0,0,"i8*",0,0,0,"i8*",0,0,0,"i32",0,0,0,"i32",0,0,0,"i8*",0,0,0,"i8*",0,0,0,"i32",0,0,0,"i32",0,0,0,"i8*",0,0,0,"i8*",0,0,0,"i32",0,0,0,"i32",0,0,0,"i8*",0,0,0,"i8*",0,0,0,"i32",0,0,0,"i32",0,0,0,"i8*",0,0,0,"i8*",0,0,0,"i32",0,0,0,"i32",0,0,0,"i8*",0,0,0,"i8*",0,0,0,"i32",0,0,0,"i32",0,0,0,"i8*",0,0,0,"i8*",0,0,0,"i32",0,0,0,"i32",0,0,0,"i8*",0,0,0,"i8*",0,0,0,"i32",0,0,0,"i32",0,0,0,"i8*",0,0,0,"i8*",0,0,0,"i32",0,0,0,"i32",0,0,0,"i8*",0,0,0,"i8*",0,0,0,"i32",0,0,0,"i32",0,0,0,"i8*",0,0,0,"i8*",0,0,0,"i32",0,0,0,"i32",0,0,0,"i8*",0,0,0,"i8*",0,0,0,"i32",0,0,0,"i32",0,0,0,"i8*",0,0,0,"i8*",0,0,0,"i32",0,0,0,"i32",0,0,0,"i8*",0,0,0,"i8*",0,0,0,"i32",0,0,0,"i32",0,0,0,"i8*",0,0,0,"i8*",0,0,0,"i32",0,0,0,"i32",0,0,0,"i8*",0,0,0,"i8*",0,0,0,"i32",0,0,0,"i32",0,0,0,"i8*",0,0,0,"i8*",0,0,0,"i32",0,0,0,"i32",0,0,0,"i8*",0,0,0,"i8*",0,0,0,"i32",0,0,0,"i32",0,0,0,"i8*",0,0,0,"i8*",0,0,0,"i32",0,0,0,"i32",0,0,0,"i8*",0,0,0,"i8*",0,0,0,"i32",0,0,0,"i32",0,0,0,"i8*",0,0,0,"i8*",0,0,0,"i32",0,0,0,"i32",0,0,0,"i8*",0,0,0,"i8*",0,0,0,"i32",0,0,0,"i32",0,0,0,"i8*",0,0,0,"i8*",0,0,0,"i32",0,0,0,"i32",0,0,0,"i8*",0,0,0,"i8*",0,0,0,"i32",0,0,0,"i32",0,0,0,"i8*",0,0,0,"i8*",0,0,0,"i32",0,0,0,"i32",0,0,0,"i8*",0,0,0,"i8*",0,0,0,"i32",0,0,0,"i32",0,0,0,"i8*",0,0,0,"i8*",0,0,0,"i32",0,0,0,"i32",0,0,0,"i8*",0,0,0,"i8*",0,0,0,"i32",0,0,0,"i32",0,0,0,"i8*",0,0,0,"i8*",0,0,0,"i32",0,0,0,"i32",0,0,0,"i8*",0,0,0,"i8*",0,0,0,"i32",0,0,0,"i32",0,0,0,"i8*",0,0,0,"i8*",0,0,0,"i32",0,0,0,"i32",0,0,0,"i8*",0,0,0,"i8*",0,0,0,"i32",0,0,0,"i32",0,0,0,"i8*",0,0,0,"i8*",0,0,0,"i32",0,0,0,"i32",0,0,0,"i8*",0,0,0,"i8*",0,0,0,"i32",0,0,0,"i32",0,0,0,"i8*",0,0,0,"i8*",0,0,0,"i32",0,0,0,"i32",0,0,0,"i8*",0,0,0,"i8*",0,0,0,"i32",0,0,0,"i32",0,0,0,"i8*",0,0,0,"i8*",0,0,0,"i32",0,0,0,"i32",0,0,0,"i8*",0,0,0,"i8*",0,0,0,"i32",0,0,0,"i32",0,0,0,"i8*",0,0,0,"i8*",0,0,0,"i32",0,0,0,"i32",0,0,0,"i8*",0,0,0,"i8*",0,0,0,"i32",0,0,0,"i32",0,0,0,"i8*",0,0,0,"i8*",0,0,0,"i32",0,0,0,"i32",0,0,0,"i8*",0,0,0,"i8*",0,0,0,"i32",0,0,0,"i32",0,0,0,"i8*",0,0,0,"i8*",0,0,0,"i32",0,0,0,"i32",0,0,0,"i8*",0,0,0,"i8*",0,0,0,"i32",0,0,0,"i32",0,0,0,"i8*",0,0,0,"i8*",0,0,0,"i32",0,0,0,"i32",0,0,0,"i8*",0,0,0,"i8*",0,0,0,"i32",0,0,0,"i32",0,0,0,"i8*",0,0,0,"i8*",0,0,0,"i32",0,0,0,"i32",0,0,0,"i8*",0,0,0,"i8*",0,0,0,"i32",0,0,0,"i32",0,0,0,"i8*",0,0,0,"i8*",0,0,0,"i32",0,0,0,"i32",0,0,0,"i8*",0,0,0,"i8*",0,0,0,"i32",0,0,0,"i32",0,0,0,"i8*",0,0,0,"i8*",0,0,0,"i32",0,0,0,"i32",0,0,0,"i8*",0,0,0,"i8*",0,0,0,"i32",0,0,0,"i32",0,0,0,"i8*",0,0,0,"i8*",0,0,0,"i32",0,0,0,"i32",0,0,0,"i8*",0,0,0,"i8*",0,0,0,"i32",0,0,0,"i32",0,0,0,"i8*",0,0,0,"i8*",0,0,0], ALLOC_STATIC);
__ZZ12WordToStringjE3buf=allocate(5, "i8", ALLOC_STATIC);
__ZL14last_pitch_cmd=allocate(1, "i32", ALLOC_STATIC);
__ZL12last_amp_cmd=allocate(1, "i32", ALLOC_STATIC);
__ZL10last_frame=allocate(1, "%struct.frame_t*", ALLOC_STATIC);
__ZL15syllable_centre=allocate(1, "i32", ALLOC_STATIC);
__ZZ18FormantTransition2P10frameref_tRijjP11PHONEME_TABiE10vcolouring=allocate([243, 0, 272, 0, 256, 0, 256, 0, 256, 0, 256, 0, 256, 0, 240, 0, 240, 0, 240, 0], ["i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0], ALLOC_STATIC);
__ZL10modn_flags=allocate(1, "i32", ALLOC_STATIC);
__ZZ8DoSpect2P11PHONEME_TABiP10FMT_PARAMSP12PHONEME_LISTiE9wave_flag_b=allocate(1, "i1", ALLOC_STATIC);
__ZL13fmt_amplitude=allocate(1, "i32", ALLOC_STATIC);
__ZL10last_wcmdq=allocate(1, "i32", ALLOC_STATIC);
__ZL12pitch_length=allocate(1, "i32", ALLOC_STATIC);
__ZL10amp_length=allocate(1, "i32", ALLOC_STATIC);
__ZZ8GenerateP12PHONEME_LISTPiiE2ix=allocate(1, "i32", ALLOC_STATIC);
__ZZ8GenerateP12PHONEME_LISTPiiE11embedded_ix=allocate(1, "i32", ALLOC_STATIC);
__ZZ8GenerateP12PHONEME_LISTPiiE10word_count=allocate(1, "i32", ALLOC_STATIC);
__ZZ8GenerateP12PHONEME_LISTPiiE8sourceix=allocate(1, "i32", ALLOC_STATIC);
__ZL14syllable_start=allocate(1, "i32", ALLOC_STATIC);
__ZL12syllable_end=allocate(1, "i32", ALLOC_STATIC);
__ZL8timer_on_b=allocate(1, "i1", ALLOC_STATIC);
__ZL6paused_b=allocate(1, "i1", ALLOC_STATIC);
__ZZ15SpeakNextClauseP7__sFILEPKviE6f_text=allocate(1, "%struct.__sFILE*", ALLOC_STATIC);
__ZZ15SpeakNextClauseP7__sFILEPKviE6p_text=allocate(1, "i8*", ALLOC_STATIC);
__str459=allocate([37,115,10,0] /* %s\0A\00 */, "i8", ALLOC_STATIC);
__str1460=allocate([87,97,114,110,105,110,103,58,32,65,99,99,101,110,116,101,100,32,108,101,116,116,101,114,115,32,97,114,101,32,110,111,116,32,114,101,99,111,103,110,105,122,101,100,44,32,101,103,58,32,85,43,48,49,48,68,10,83,101,116,32,76,67,95,67,84,89,80,69,32,116,111,32,97,32,85,84,70,45,56,32,108,111,99,97,108,101,10,0] /* Warning: Accented le */, "i8", ALLOC_STATIC);
__ZZL13set_frame_rmsP7frame_tiE8sqrt_tab=allocate([0, 0, 64, 0, 90, 0, 110, 0, 128, 0, 143, 0, 156, 0, 169, 0, 181, 0, 192, 0, 202, 0, 212, 0, 221, 0, 230, 0, 239, 0, 247, 0, 256, 0, 263, 0, 271, 0, 278, 0, 286, 0, 293, 0, 300, 0, 306, 0, 313, 0, 320, 0, 326, 0, 332, 0, 338, 0, 344, 0, 350, 0, 356, 0, 362, 0, 367, 0, 373, 0, 378, 0, 384, 0, 389, 0, 394, 0, 399, 0, 404, 0, 409, 0, 414, 0, 419, 0, 424, 0, 429, 0, 434, 0, 438, 0, 443, 0, 448, 0, 452, 0, 457, 0, 461, 0, 465, 0, 470, 0, 474, 0, 478, 0, 483, 0, 487, 0, 491, 0, 495, 0, 499, 0, 503, 0, 507, 0, 512, 0, 515, 0, 519, 0, 523, 0, 527, 0, 531, 0, 535, 0, 539, 0, 543, 0, 546, 0, 550, 0, 554, 0, 557, 0, 561, 0, 565, 0, 568, 0, 572, 0, 576, 0, 579, 0, 583, 0, 586, 0, 590, 0, 593, 0, 596, 0, 600, 0, 603, 0, 607, 0, 610, 0, 613, 0, 617, 0, 620, 0, 623, 0, 627, 0, 630, 0, 633, 0, 636, 0, 640, 0, 643, 0, 646, 0, 649, 0, 652, 0, 655, 0, 658, 0, 662, 0, 665, 0, 668, 0, 671, 0, 674, 0, 677, 0, 680, 0, 683, 0, 686, 0, 689, 0, 692, 0, 695, 0, 698, 0, 701, 0, 704, 0, 706, 0, 709, 0, 712, 0, 715, 0, 718, 0, 721, 0, 724, 0, 726, 0, 729, 0, 732, 0, 735, 0, 738, 0, 740, 0, 743, 0, 746, 0, 749, 0, 751, 0, 754, 0, 757, 0, 759, 0, 762, 0, 765, 0, 768, 0, 770, 0, 773, 0, 775, 0, 778, 0, 781, 0, 783, 0, 786, 0, 789, 0, 791, 0, 794, 0, 796, 0, 799, 0, 801, 0, 804, 0, 807, 0, 809, 0, 812, 0, 814, 0, 817, 0, 819, 0, 822, 0, 824, 0, 827, 0, 829, 0, 832, 0, 834, 0, 836, 0, 839, 0, 841, 0, 844, 0, 846, 0, 849, 0, 851, 0, 853, 0, 856, 0, 858, 0, 861, 0, 863, 0, 865, 0, 868, 0, 870, 0, 872, 0, 875, 0, 877, 0, 879, 0, 882, 0, 884, 0, 886, 0, 889, 0, 891, 0, 893, 0, 896, 0, 898, 0, 900, 0, 902, 0], ["i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0], ALLOC_STATIC);
__ZZL10AllocFramevE2ix=allocate(1, "i32", ALLOC_STATIC);
__ZZL10AllocFramevE10frame_pool=allocate(10240, ["i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8"], ALLOC_STATIC);
_translator=allocate(1, "%struct.Translator*", ALLOC_STATIC);
_translator2=allocate(1, "%struct.Translator*", ALLOC_STATIC);
__ZL20translator2_language=allocate(20, "i8", ALLOC_STATIC);
_f_trans=allocate(1, "%struct.__sFILE*", ALLOC_STATIC);
_option_tone2=allocate(1, "i32", ALLOC_STATIC);
_option_tone_flags=allocate(1, "i32", ALLOC_STATIC);
_option_phonemes=allocate(1, "i32", ALLOC_STATIC);
_option_phoneme_events=allocate(1, "i32", ALLOC_STATIC);
_option_quiet=allocate(1, "i32", ALLOC_STATIC);
_option_endpause=allocate(1, "i32", ALLOC_STATIC);
_option_capitals=allocate(1, "i32", ALLOC_STATIC);
_option_punctuation=allocate(1, "i32", ALLOC_STATIC);
_option_sayas=allocate(1, "i32", ALLOC_STATIC);
_option_ssml=allocate(1, "i32", ALLOC_STATIC);
_option_phoneme_input=allocate(1, "i32", ALLOC_STATIC);
_option_phoneme_variants=allocate(1, "i32", ALLOC_STATIC);
_option_wordgap=allocate(1, "i32", ALLOC_STATIC);
_skip_sentences=allocate(1, "i32", ALLOC_STATIC);
_skip_words=allocate(1, "i32", ALLOC_STATIC);
_skip_characters=allocate(1, "i32", ALLOC_STATIC);
_skip_marker=allocate(50, "i8", ALLOC_STATIC);
_skipping_text=allocate(1, "i32", ALLOC_STATIC);
_end_character_position=allocate(1, "i32", ALLOC_STATIC);
_count_sentences=allocate(1, "i32", ALLOC_STATIC);
_count_words=allocate(1, "i32", ALLOC_STATIC);
_clause_start_char=allocate(1, "i32", ALLOC_STATIC);
_clause_start_word=allocate(1, "i32", ALLOC_STATIC);
_new_sentence=allocate(1, "i32", ALLOC_STATIC);
_pre_pause=allocate(1, "i32", ALLOC_STATIC);
_word_phonemes=allocate(160, "i8", ALLOC_STATIC);
_n_ph_list2=allocate(1, "i32", ALLOC_STATIC);
_ph_list2=allocate(8000, ["i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0,"i8","i8","i8","i8","i16",0,"i16",0], ALLOC_STATIC);
_option_punctlist=allocate(240, "i32", ALLOC_STATIC);
_ctrl_embedded=allocate([1], ["i8",0,0,0,0], ALLOC_STATIC);
_option_multibyte=allocate(1, "i32", ALLOC_STATIC);
_option_linelength=allocate(1, "i32", ALLOC_STATIC);
_embedded_list=allocate(1000, "i32", ALLOC_STATIC);
_n_replace_phonemes=allocate(1, "i32", ALLOC_STATIC);
_replace_phonemes=allocate(180, "i8", ALLOC_STATIC);
__ZL10ISO_8859_1=allocate([160, 0, 161, 0, 162, 0, 163, 0, 164, 0, 165, 0, 166, 0, 167, 0, 168, 0, 169, 0, 170, 0, 171, 0, 172, 0, 173, 0, 174, 0, 175, 0, 176, 0, 177, 0, 178, 0, 179, 0, 180, 0, 181, 0, 182, 0, 183, 0, 184, 0, 185, 0, 186, 0, 187, 0, 188, 0, 189, 0, 190, 0, 191, 0, 192, 0, 193, 0, 194, 0, 195, 0, 196, 0, 197, 0, 198, 0, 199, 0, 200, 0, 201, 0, 202, 0, 203, 0, 204, 0, 205, 0, 206, 0, 207, 0, 208, 0, 209, 0, 210, 0, 211, 0, 212, 0, 213, 0, 214, 0, 215, 0, 216, 0, 217, 0, 218, 0, 219, 0, 220, 0, 221, 0, 222, 0, 223, 0, 224, 0, 225, 0, 226, 0, 227, 0, 228, 0, 229, 0, 230, 0, 231, 0, 232, 0, 233, 0, 234, 0, 235, 0, 236, 0, 237, 0, 238, 0, 239, 0, 240, 0, 241, 0, 242, 0, 243, 0, 244, 0, 245, 0, 246, 0, 247, 0, 248, 0, 249, 0, 250, 0, 251, 0, 252, 0, 253, 0, 254, 0, 255, 0], ["i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0], ALLOC_STATIC);
__ZL10ISO_8859_2=allocate([160, 0, 260, 0, 728, 0, 321, 0, 164, 0, 317, 0, 346, 0, 167, 0, 168, 0, 352, 0, 350, 0, 356, 0, 377, 0, 173, 0, 381, 0, 379, 0, 176, 0, 261, 0, 731, 0, 322, 0, 180, 0, 318, 0, 347, 0, 711, 0, 184, 0, 353, 0, 351, 0, 357, 0, 378, 0, 733, 0, 382, 0, 380, 0, 340, 0, 193, 0, 194, 0, 258, 0, 196, 0, 313, 0, 262, 0, 199, 0, 268, 0, 201, 0, 280, 0, 203, 0, 282, 0, 205, 0, 206, 0, 270, 0, 272, 0, 323, 0, 327, 0, 211, 0, 212, 0, 336, 0, 214, 0, 215, 0, 344, 0, 366, 0, 218, 0, 368, 0, 220, 0, 221, 0, 354, 0, 223, 0, 341, 0, 225, 0, 226, 0, 259, 0, 228, 0, 314, 0, 263, 0, 231, 0, 269, 0, 233, 0, 281, 0, 235, 0, 283, 0, 237, 0, 238, 0, 271, 0, 273, 0, 324, 0, 328, 0, 243, 0, 244, 0, 337, 0, 246, 0, 247, 0, 345, 0, 367, 0, 250, 0, 369, 0, 252, 0, 253, 0, 355, 0, 729, 0], ["i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0], ALLOC_STATIC);
__ZL10ISO_8859_3=allocate([160, 0, 294, 0, 728, 0, 163, 0, 164, 0, 0, 0, 292, 0, 167, 0, 168, 0, 304, 0, 350, 0, 286, 0, 308, 0, 173, 0, 0, 0, 379, 0, 176, 0, 295, 0, 178, 0, 179, 0, 180, 0, 181, 0, 293, 0, 183, 0, 184, 0, 305, 0, 351, 0, 287, 0, 309, 0, 189, 0, 0, 0, 380, 0, 192, 0, 193, 0, 194, 0, 0, 0, 196, 0, 266, 0, 264, 0, 199, 0, 200, 0, 201, 0, 202, 0, 203, 0, 204, 0, 205, 0, 206, 0, 207, 0, 0, 0, 209, 0, 210, 0, 211, 0, 212, 0, 288, 0, 214, 0, 215, 0, 284, 0, 217, 0, 218, 0, 219, 0, 220, 0, 364, 0, 348, 0, 223, 0, 224, 0, 225, 0, 226, 0, 0, 0, 228, 0, 267, 0, 265, 0, 231, 0, 232, 0, 233, 0, 234, 0, 235, 0, 236, 0, 237, 0, 238, 0, 239, 0, 0, 0, 241, 0, 242, 0, 243, 0, 244, 0, 289, 0, 246, 0, 247, 0, 285, 0, 249, 0, 250, 0, 251, 0, 252, 0, 365, 0, 349, 0, 729, 0], ["i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0], ALLOC_STATIC);
__ZL10ISO_8859_4=allocate([160, 0, 260, 0, 312, 0, 342, 0, 164, 0, 296, 0, 315, 0, 167, 0, 168, 0, 352, 0, 274, 0, 290, 0, 358, 0, 173, 0, 381, 0, 175, 0, 176, 0, 261, 0, 731, 0, 343, 0, 180, 0, 297, 0, 316, 0, 711, 0, 184, 0, 353, 0, 275, 0, 291, 0, 359, 0, 330, 0, 382, 0, 331, 0, 256, 0, 193, 0, 194, 0, 195, 0, 196, 0, 197, 0, 198, 0, 302, 0, 268, 0, 201, 0, 280, 0, 203, 0, 278, 0, 205, 0, 206, 0, 298, 0, 272, 0, 325, 0, 332, 0, 310, 0, 212, 0, 213, 0, 214, 0, 215, 0, 216, 0, 370, 0, 218, 0, 219, 0, 220, 0, 360, 0, 362, 0, 223, 0, 257, 0, 225, 0, 226, 0, 227, 0, 228, 0, 229, 0, 230, 0, 303, 0, 269, 0, 233, 0, 281, 0, 235, 0, 279, 0, 237, 0, 238, 0, 299, 0, 273, 0, 326, 0, 333, 0, 311, 0, 244, 0, 245, 0, 246, 0, 247, 0, 248, 0, 371, 0, 250, 0, 251, 0, 252, 0, 361, 0, 363, 0, 729, 0], ["i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0], ALLOC_STATIC);
__ZL10ISO_8859_5=allocate([160, 0, 1025, 0, 1026, 0, 1027, 0, 1028, 0, 1029, 0, 1030, 0, 1031, 0, 1032, 0, 1033, 0, 1034, 0, 1035, 0, 1036, 0, 173, 0, 1038, 0, 1039, 0, 1040, 0, 1041, 0, 1042, 0, 1043, 0, 1044, 0, 1045, 0, 1046, 0, 1047, 0, 1048, 0, 1049, 0, 1050, 0, 1051, 0, 1052, 0, 1053, 0, 1054, 0, 1055, 0, 1056, 0, 1057, 0, 1058, 0, 1059, 0, 1060, 0, 1061, 0, 1062, 0, 1063, 0, 1064, 0, 1065, 0, 1066, 0, 1067, 0, 1068, 0, 1069, 0, 1070, 0, 1071, 0, 1072, 0, 1073, 0, 1074, 0, 1075, 0, 1076, 0, 1077, 0, 1078, 0, 1079, 0, 1080, 0, 1081, 0, 1082, 0, 1083, 0, 1084, 0, 1085, 0, 1086, 0, 1087, 0, 1088, 0, 1089, 0, 1090, 0, 1091, 0, 1092, 0, 1093, 0, 1094, 0, 1095, 0, 1096, 0, 1097, 0, 1098, 0, 1099, 0, 1100, 0, 1101, 0, 1102, 0, 1103, 0, 8470, 0, 1105, 0, 1106, 0, 1107, 0, 1108, 0, 1109, 0, 1110, 0, 1111, 0, 1112, 0, 1113, 0, 1114, 0, 1115, 0, 1116, 0, 167, 0, 1118, 0, 1119, 0], ["i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0], ALLOC_STATIC);
__ZL10ISO_8859_7=allocate([160, 0, 8216, 0, 8217, 0, 163, 0, 8364, 0, 8367, 0, 166, 0, 167, 0, 168, 0, 169, 0, 890, 0, 171, 0, 172, 0, 173, 0, 0, 0, 8213, 0, 176, 0, 177, 0, 178, 0, 179, 0, 900, 0, 901, 0, 902, 0, 183, 0, 904, 0, 905, 0, 906, 0, 187, 0, 908, 0, 189, 0, 910, 0, 911, 0, 912, 0, 913, 0, 914, 0, 915, 0, 916, 0, 917, 0, 918, 0, 919, 0, 920, 0, 921, 0, 922, 0, 923, 0, 924, 0, 925, 0, 926, 0, 927, 0, 928, 0, 929, 0, 0, 0, 931, 0, 932, 0, 933, 0, 934, 0, 935, 0, 936, 0, 937, 0, 938, 0, 939, 0, 940, 0, 941, 0, 942, 0, 943, 0, 944, 0, 945, 0, 946, 0, 947, 0, 948, 0, 949, 0, 950, 0, 951, 0, 952, 0, 953, 0, 954, 0, 955, 0, 956, 0, 957, 0, 958, 0, 959, 0, 960, 0, 961, 0, 962, 0, 963, 0, 964, 0, 965, 0, 966, 0, 967, 0, 968, 0, 969, 0, 970, 0, 971, 0, 972, 0, 973, 0, 974, 0, 0, 0], ["i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0], ALLOC_STATIC);
__ZL10ISO_8859_9=allocate([160, 0, 161, 0, 162, 0, 163, 0, 164, 0, 165, 0, 166, 0, 167, 0, 168, 0, 169, 0, 170, 0, 171, 0, 172, 0, 173, 0, 174, 0, 175, 0, 176, 0, 177, 0, 178, 0, 179, 0, 180, 0, 181, 0, 182, 0, 183, 0, 184, 0, 185, 0, 186, 0, 187, 0, 188, 0, 189, 0, 190, 0, 191, 0, 192, 0, 193, 0, 194, 0, 195, 0, 196, 0, 197, 0, 198, 0, 199, 0, 200, 0, 201, 0, 202, 0, 203, 0, 204, 0, 205, 0, 206, 0, 207, 0, 286, 0, 209, 0, 210, 0, 211, 0, 212, 0, 213, 0, 214, 0, 215, 0, 216, 0, 217, 0, 218, 0, 219, 0, 220, 0, 304, 0, 350, 0, 223, 0, 224, 0, 225, 0, 226, 0, 227, 0, 228, 0, 229, 0, 230, 0, 231, 0, 232, 0, 233, 0, 234, 0, 235, 0, 236, 0, 237, 0, 238, 0, 239, 0, 287, 0, 241, 0, 242, 0, 243, 0, 244, 0, 245, 0, 246, 0, 247, 0, 248, 0, 249, 0, 250, 0, 251, 0, 252, 0, 305, 0, 351, 0, 255, 0], ["i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0], ALLOC_STATIC);
__ZL11ISO_8859_14=allocate([160, 0, 7682, 0, 7683, 0, 163, 0, 266, 0, 267, 0, 7690, 0, 167, 0, 7808, 0, 169, 0, 7810, 0, 7691, 0, 7922, 0, 173, 0, 174, 0, 376, 0, 7710, 0, 7711, 0, 288, 0, 289, 0, 7744, 0, 7745, 0, 182, 0, 7766, 0, 7809, 0, 7767, 0, 7811, 0, 7776, 0, 7923, 0, 7812, 0, 7813, 0, 7777, 0, 192, 0, 193, 0, 194, 0, 195, 0, 196, 0, 197, 0, 198, 0, 199, 0, 200, 0, 201, 0, 202, 0, 203, 0, 204, 0, 205, 0, 206, 0, 207, 0, 372, 0, 209, 0, 210, 0, 211, 0, 212, 0, 213, 0, 214, 0, 7786, 0, 216, 0, 217, 0, 218, 0, 219, 0, 220, 0, 221, 0, 374, 0, 223, 0, 224, 0, 225, 0, 226, 0, 227, 0, 228, 0, 229, 0, 230, 0, 231, 0, 232, 0, 233, 0, 234, 0, 235, 0, 236, 0, 237, 0, 238, 0, 239, 0, 373, 0, 241, 0, 242, 0, 243, 0, 244, 0, 245, 0, 246, 0, 7787, 0, 248, 0, 249, 0, 250, 0, 251, 0, 252, 0, 253, 0, 375, 0, 255, 0], ["i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0], ALLOC_STATIC);
__ZL6KOI8_R=allocate([9552, 0, 9553, 0, 9554, 0, 1105, 0, 9555, 0, 9556, 0, 9557, 0, 9558, 0, 9559, 0, 9560, 0, 9561, 0, 9562, 0, 9563, 0, 9564, 0, 9565, 0, 9566, 0, 9567, 0, 9568, 0, 9569, 0, 1025, 0, 9570, 0, 9571, 0, 9572, 0, 9573, 0, 9574, 0, 9575, 0, 9576, 0, 9577, 0, 9578, 0, 9579, 0, 9580, 0, 169, 0, 1102, 0, 1072, 0, 1073, 0, 1094, 0, 1076, 0, 1077, 0, 1092, 0, 1075, 0, 1093, 0, 1080, 0, 1081, 0, 1082, 0, 1083, 0, 1084, 0, 1085, 0, 1086, 0, 1087, 0, 1103, 0, 1088, 0, 1089, 0, 1090, 0, 1091, 0, 1078, 0, 1074, 0, 1100, 0, 1099, 0, 1079, 0, 1096, 0, 1101, 0, 1097, 0, 1095, 0, 1098, 0, 1070, 0, 1040, 0, 1041, 0, 1062, 0, 1044, 0, 1045, 0, 1060, 0, 1043, 0, 1061, 0, 1048, 0, 1049, 0, 1050, 0, 1051, 0, 1052, 0, 1053, 0, 1054, 0, 1055, 0, 1071, 0, 1056, 0, 1057, 0, 1058, 0, 1059, 0, 1046, 0, 1042, 0, 1068, 0, 1067, 0, 1047, 0, 1064, 0, 1069, 0, 1065, 0, 1063, 0, 1066, 0], ["i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0], ALLOC_STATIC);
__ZL5ISCII=allocate([32, 0, 2305, 0, 2306, 0, 2307, 0, 2309, 0, 2310, 0, 2311, 0, 2312, 0, 2313, 0, 2314, 0, 2315, 0, 2318, 0, 2319, 0, 2320, 0, 2317, 0, 2322, 0, 2323, 0, 2324, 0, 2321, 0, 2325, 0, 2326, 0, 2327, 0, 2328, 0, 2329, 0, 2330, 0, 2331, 0, 2332, 0, 2333, 0, 2334, 0, 2335, 0, 2336, 0, 2337, 0, 2338, 0, 2339, 0, 2340, 0, 2341, 0, 2342, 0, 2343, 0, 2344, 0, 2345, 0, 2346, 0, 2347, 0, 2348, 0, 2349, 0, 2350, 0, 2351, 0, 2399, 0, 2352, 0, 2353, 0, 2354, 0, 2355, 0, 2356, 0, 2357, 0, 2358, 0, 2359, 0, 2360, 0, 2361, 0, 32, 0, 2366, 0, 2367, 0, 2368, 0, 2369, 0, 2370, 0, 2371, 0, 2374, 0, 2375, 0, 2376, 0, 2373, 0, 2378, 0, 2379, 0, 2380, 0, 2377, 0, 2381, 0, 2364, 0, 2404, 0, 32, 0, 32, 0, 32, 0, 32, 0, 32, 0, 32, 0, 48, 0, 49, 0, 50, 0, 51, 0, 52, 0, 53, 0, 54, 0, 55, 0, 56, 0, 57, 0, 32, 0, 32, 0, 32, 0, 32, 0, 32, 0], ["i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0], ALLOC_STATIC);
_charsets=allocate(80, "i16*", ALLOC_STATIC);
__ZL15length_mod_tabs=allocate(24, "i8*", ALLOC_STATIC);
__ZZ7IsAlphajE18extra_indic_alphas=allocate([2672, 0, 2673, 0, 0, 0], ["i16",0,"i16",0,"i16",0], ALLOC_STATIC);
__ZL8brackets=allocate([40, 0, 41, 0, 91, 0, 93, 0, 123, 0, 125, 0, 60, 0, 62, 0, 34, 0, 39, 0, 96, 0, 171, 0, 187, 0, 12298, 0, 12299, 0, -8132, 0, 0, 0], ["i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0], ALLOC_STATIC);
__ZZ8utf8_outjPcE4code=allocate([0,192,224,240] /* \00\C0\E0\F0 */, "i8", ALLOC_STATIC);
__ZZ8utf8_in2PiPKciE4mask=allocate([255,31,15,7] /* \FF\1F\0F\07 */, "i8", ALLOC_STATIC);
__str543=allocate([40,37,115,41,32,37,115,32,32,45,62,32,40,37,115,41,32,0] /* (%s) %s  -> (%s) \00 */, "i8", ALLOC_STATIC);
__str1544=allocate([37,115,10,10,0] /* %s\0A\0A\00 */, "i8", ALLOC_STATIC);
__ZZ13TranslateWordP10TranslatorPciP8WORD_TABE7word_zz=allocate([0,122,122,0] /* \00zz\00 */, "i8", ALLOC_STATIC);
__ZZ13TranslateWordP10TranslatorPciP8WORD_TABE7word_iz=allocate([0,105,122,0] /* \00iz\00 */, "i8", ALLOC_STATIC);
__ZZ13TranslateWordP10TranslatorPciP8WORD_TABE7word_ss=allocate([0,115,115,0] /* \00ss\00 */, "i8", ALLOC_STATIC);
__str2545=allocate([95,48,108,97,110,103,0] /* _0lang\00 */, "i8", ALLOC_STATIC);
__str4547=allocate([32,32,115,117,102,102,105,120,32,91,37,115,93,10,10,0] /*   suffix [%s]\0A\0A\ */, "i8", ALLOC_STATIC);
__str5548=allocate([104,115,120,0] /* hsx\00 */, "i8", ALLOC_STATIC);
__str6549=allocate([37,115,37,115,37,115,0] /* %s%s%s\00 */, "i8", ALLOC_STATIC);
__ZL18any_stressed_words_b=allocate(1, "i1", ALLOC_STATIC);
__ZZ15TranslateClauseP10TranslatorP7__sFILEPKvPiPPcE17voice_change_name=allocate(40, "i8", ALLOC_STATIC);
__ZL11embedded_ix=allocate(1, "i32", ALLOC_STATIC);
__ZL13embedded_read=allocate(1, "i32", ALLOC_STATIC);
__ZL6source=allocate(740, "i8", ALLOC_STATIC);
__str7550=allocate([67,76,65,85,83,69,32,37,120,58,10,0] /* CLAUSE %x:\0A\00 */, "i8", ALLOC_STATIC);
__str8551=allocate([69,78,68,67,76,65,85,83,69,10,0] /* ENDCLAUSE\0A\00 */, "i8", ALLOC_STATIC);
__ZL16max_clause_pause=allocate(1, "i32", ALLOC_STATIC);
__ZL13option_sayas2=allocate(1, "i32", ALLOC_STATIC);
__ZL18count_sayas_digits=allocate(1, "i32", ALLOC_STATIC);
__ZL6breaks=allocate([95, 0, 0, 0], ["i16",0,"i16",0], ALLOC_STATIC);
__ZL13word_emphasis_b=allocate(1, "i1", ALLOC_STATIC);
__ZL15option_emphasis=allocate(1, "i32", ALLOC_STATIC);
__ZL13embedded_flag_b=allocate(1, "i1", ALLOC_STATIC);
__str9552=allocate([87,79,82,68,58,32,102,108,103,61,37,46,53,120,32,108,101,110,61,37,100,32,32,39,0] /* WORD: flg=%.5x len=% */, "i8", ALLOC_STATIC);
__str10553=allocate([39,10,0] /* '\0A\00 */, "i8", ALLOC_STATIC);
__str11554=allocate([95,94,95,0] /* _^_\00 */, "i8", ALLOC_STATIC);
__ZZL13TranslateCharP10TranslatorPcijjPiE20hangul_compatibility=allocate([0,0,1,170,2,172,173,3,4,5,176,177,178,179,180,180,182,6,7,8,185,9,10,188,12,13,14,15,16,17,18,97,98,99,100,101,102,103,104,105,106,107,108,109,110,111,112,113,114,115,116,117] /* \00\00\01\AA\02\AC\A */, "i8", ALLOC_STATIC);
__ZZL14SubstituteCharP10TranslatorjjPiE11ignore_next_b=allocate(1, "i1", ALLOC_STATIC);
__str12555=allocate([80,83,65,82,72,84,73,86,89,77,85,66,70,0] /* PSARHTIVYMUBF\00 */, "i8", ALLOC_STATIC);
__ZL14length_mods_en=allocate([100,120,100,105,100,110,110,100,95,100,105,120,105,110,125,130,135,115,125,100,105,120,75,100,75,105,120,85,75,100,105,120,85,105,95,115,120,100,95,100,110,120,95,105,100,115,120,100,100,100,105,120,100,105,95,115,120,110,95,100,105,120,100,105,105,122,125,110,105,100,105,120,100,105,105,122,125,110,105,100,105,120,95,105,100,115,120,110,100,100,100,120,100,100,100,100,100,100,100,100] /* dxdidnnd_dixin}\82\8 */, "i8", ALLOC_STATIC);
__ZL15length_mods_en0=allocate([100,150,100,105,110,115,110,110,110,100,105,150,105,110,125,135,140,115,135,100,105,150,90,105,90,122,135,100,90,100,105,150,100,105,100,122,135,100,100,100,105,150,100,105,105,115,135,110,105,100,105,150,100,105,105,122,130,120,125,100,105,150,100,105,110,122,125,115,110,100,105,150,100,105,105,122,135,120,105,100,105,150,100,105,105,115,135,110,105,100,100,100,100,100,100,100,100,100,100,100] /* d\96dinsnnndi\96in}\ */, "i8", ALLOC_STATIC);
__ZL17length_mods_equal=allocate([110,110,110,110,110,110,110,110,110,110,110,110,110,110,110,110,110,110,110,110,110,110,110,110,110,110,110,110,110,110,110,110,110,110,110,110,110,110,110,110,110,110,110,110,110,110,110,110,110,110,110,110,110,110,110,110,110,110,110,110,110,110,110,110,110,110,110,110,110,110,110,110,110,110,110,110,110,110,110,110,110,110,110,110,110,110,110,110,110,110,110,110,110,110,110,110,110,110,110,110] /* nnnnnnnnnnnnnnnnnnnn */, "i8", ALLOC_STATIC);
__str580=allocate([35,10,0] /* #\0A\00 */, "i8", ALLOC_STATIC);
__str1581=allocate([117,110,97,98,108,101,32,116,111,32,103,101,116,32,46,119,97,118,32,104,101,97,100,101,114,32,102,114,111,109,32,109,98,114,111,108,97,0] /* unable to get .wav h */, "i8", ALLOC_STATIC);
__str2582=allocate([82,73,70,70,0] /* RIFF\00 */, "i8", ALLOC_STATIC);
__str3583=allocate([87,65,86,69,102,109,116,32,0] /* WAVEfmt \00 */, "i8", ALLOC_STATIC);
__str4584=allocate([109,98,114,111,108,97,32,100,105,100,32,110,111,116,32,114,101,116,117,114,110,32,97,32,46,119,97,118,32,104,101,97,100,101,114,0] /* mbrola did not retur */, "i8", ALLOC_STATIC);
__ZL14mbr_samplerate=allocate(1, "i32", ALLOC_STATIC);
__ZL14mbr_voice_path=allocate(1, "i8*", ALLOC_STATIC);
__ZL10mbr_volume=allocate([1], ["float",0,0,0,0], ALLOC_STATIC);
__ZL9mbr_state=allocate(1, "i32", ALLOC_STATIC);
__ZL7mbr_pid=allocate(1, "i32", ALLOC_STATIC);
__ZL10mbr_cmd_fd=allocate(1, "i32", ALLOC_STATIC);
__str5585=allocate([10,35,10,0] /* \0A#\0A\00 */, "i8", ALLOC_STATIC);
__ZL12mbr_audio_fd=allocate(1, "i32", ALLOC_STATIC);
__ZL12mbr_errorbuf=allocate(160, "i8", ALLOC_STATIC);
__ZL12mbr_error_fd=allocate(1, "i32", ALLOC_STATIC);
__str7587=allocate([114,101,97,100,40,101,114,114,111,114,41,58,32,37,115,0] /* read(error): %s\00 */, "i8", ALLOC_STATIC);
__str8588=allocate([71,111,116,32,97,32,114,101,115,101,116,32,115,105,103,110,97,108,0] /* Got a reset signal\0 */, "i8", ALLOC_STATIC);
__str9589=allocate([73,110,112,117,116,32,70,108,117,115,104,32,83,105,103,110,97,108,0] /* Input Flush Signal\0 */, "i8", ALLOC_STATIC);
__str10590=allocate([109,98,114,111,108,97,58,32,37,115,0] /* mbrola: %s\00 */, "i8", ALLOC_STATIC);
__str11591=allocate([109,98,114,111,108,97,32,99,108,111,115,101,100,32,115,116,100,101,114,114,32,97,110,100,32,100,105,100,32,110,111,116,32,101,120,105,116,0] /* mbrola closed stderr */, "i8", ALLOC_STATIC);
__str12592=allocate([119,97,105,116,112,105,100,40,41,32,105,115,32,99,111,110,102,117,115,101,100,0] /* waitpid() is confuse */, "i8", ALLOC_STATIC);
__str13593=allocate([109,98,114,111,108,97,32,100,105,101,100,32,98,121,32,115,105,103,110,97,108,32,37,100,0] /* mbrola died by signa */, "i8", ALLOC_STATIC);
__str14594=allocate([109,98,114,111,108,97,32,101,120,105,116,101,100,32,119,105,116,104,32,115,116,97,116,117,115,32,37,100,0] /* mbrola exited with s */, "i8", ALLOC_STATIC);
__str15595=allocate([109,98,114,111,108,97,32,100,105,101,100,32,97,110,100,32,119,97,105,116,32,115,116,97,116,117,115,32,105,115,32,119,101,105,114,100,0] /* mbrola died and wait */, "i8", ALLOC_STATIC);
__str16596=allocate([109,98,114,111,119,114,97,112,32,101,114,114,111,114,58,32,37,115,0] /* mbrowrap error: %s\0 */, "i8", ALLOC_STATIC);
__str17597=allocate([44,32,40,37,115,41,0] /* , (%s)\00 */, "i8", ALLOC_STATIC);
__ZL21mbr_pending_data_head=allocate(1, "%struct.datablock*", ALLOC_STATIC);
__ZL21mbr_pending_data_tail=allocate(1, "%struct.datablock*", ALLOC_STATIC);
__str18598=allocate([112,111,108,108,40,41,58,32,37,115,0] /* poll(): %s\00 */, "i8", ALLOC_STATIC);
__str19599=allocate([109,98,114,111,108,97,32,112,114,111,99,101,115,115,32,105,115,32,115,116,97,108,108,101,100,0] /* mbrola process is st */, "i8", ALLOC_STATIC);
__str20600=allocate([119,114,105,116,101,40,41,58,32,37,115,0] /* write(): %s\00 */, "i8", ALLOC_STATIC);
__str21601=allocate([114,101,97,100,40,41,58,32,37,115,0] /* read(): %s\00 */, "i8", ALLOC_STATIC);
__ZL13mbr_proc_stat=allocate(1, "i32", ALLOC_STATIC);
__str22602=allocate([109,98,114,111,108,97,32,105,110,105,116,32,114,101,113,117,101,115,116,32,119,104,101,110,32,97,108,114,101,97,100,121,32,105,110,105,116,105,97,108,105,122,101,100,0] /* mbrola init request  */, "i8", ALLOC_STATIC);
__str23603=allocate([102,111,114,107,40,41,58,32,37,115,0] /* fork(): %s\00 */, "i8", ALLOC_STATIC);
__str24604=allocate([100,117,112,50,40,41,58,32,37,115,10,0] /* dup2(): %s\0A\00 */, "i8", ALLOC_STATIC);
__str25605=allocate([37,103,0] /* %g\00 */, "i8", ALLOC_STATIC);
__str27607=allocate([45,101,0] /* -e\00 */, "i8", ALLOC_STATIC);
__str28608=allocate([45,118,0] /* -v\00 */, "i8", ALLOC_STATIC);
__str30610=allocate([45,46,119,97,118,0] /* -.wav\00 */, "i8", ALLOC_STATIC);
__str31611=allocate([109,98,114,111,108,97,58,32,37,115,10,0] /* mbrola: %s\0A\00 */, "i8", ALLOC_STATIC);
__str32612=allocate([47,112,114,111,99,47,37,100,47,115,116,97,116,0] /* /proc/%d/stat\00 */, "i8", ALLOC_STATIC);
__str33613=allocate([47,112,114,111,99,32,105,115,32,117,110,97,99,99,101,115,115,105,98,108,101,58,32,37,115,0] /* /proc is unaccessibl */, "i8", ALLOC_STATIC);
__str34614=allocate([102,99,110,116,108,40,41,58,32,37,115,0] /* fcntl(): %s\00 */, "i8", ALLOC_STATIC);
__str35615=allocate([112,105,112,101,40,41,58,32,37,115,0] /* pipe(): %s\00 */, "i8", ALLOC_STATIC);
__ZZ15SetIndicLettersP10TranslatorE15dev_consonants2=allocate([2,3,88,89,90,91,92,93,94,95] /* \02\03XYZ[\5C]^_ */, "i8", ALLOC_STATIC);
__ZZ16SelectTranslatorPKcE17stress_lengths_fr=allocate([190, 0, 170, 0, 190, 0, 200, 0, 0, 0, 0, 0, 190, 0, 240, 0], ["i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0], ALLOC_STATIC);
__ZZ16SelectTranslatorPKcE14stress_amps_fr=allocate([18,16,18,18,18,18,18,18] /* \12\10\12\12\12\12\1 */, "i8", ALLOC_STATIC);
__ZZ16SelectTranslatorPKcE14stress_amps_sk=allocate([17,17,20,20,20,22,22,21] /* \11\11\14\14\14\16\1 */, "i8", ALLOC_STATIC);
__ZZ16SelectTranslatorPKcE17stress_lengths_sk=allocate([190, 0, 190, 0, 210, 0, 210, 0, 0, 0, 0, 0, 210, 0, 210, 0], ["i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0], ALLOC_STATIC);
__ZZ16SelectTranslatorPKcE17stress_lengths_ta=allocate([200, 0, 200, 0, 210, 0, 210, 0, 0, 0, 0, 0, 230, 0, 230, 0], ["i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0], ALLOC_STATIC);
__ZZ16SelectTranslatorPKcE14stress_amps_ta=allocate([18,18,18,18,20,20,22,22] /* \12\12\12\12\14\14\1 */, "i8", ALLOC_STATIC);
__ZZ16SelectTranslatorPKcE17stress_lengths_af=allocate([170, 0, 140, 0, 220, 0, 220, 0, 0, 0, 0, 0, 250, 0, 270, 0], ["i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0], ALLOC_STATIC);
__ZZ16SelectTranslatorPKcE17stress_lengths_bn=allocate([180, 0, 180, 0, 210, 0, 210, 0, 0, 0, 0, 0, 230, 0, 240, 0], ["i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0], ALLOC_STATIC);
__ZZ16SelectTranslatorPKcE14stress_amps_bn=allocate([18,18,18,18,20,20,22,22] /* \12\12\12\12\14\14\1 */, "i8", ALLOC_STATIC);
__ZZ16SelectTranslatorPKcE17stress_lengths_cy=allocate([170, 0, 220, 0, 180, 0, 180, 0, 0, 0, 0, 0, 250, 0, 270, 0], ["i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0], ALLOC_STATIC);
__ZZ16SelectTranslatorPKcE14stress_amps_cy=allocate([17,15,18,18,0,0,22,20] /* \11\0F\12\12\00\00\1 */, "i8", ALLOC_STATIC);
__ZZ16SelectTranslatorPKcE17stress_lengths_da=allocate([160, 0, 140, 0, 200, 0, 200, 0, 0, 0, 0, 0, 220, 0, 230, 0], ["i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0], ALLOC_STATIC);
__ZZ16SelectTranslatorPKcE17stress_lengths_de=allocate([150, 0, 130, 0, 200, 0, 200, 0, 0, 0, 0, 0, 270, 0, 270, 0], ["i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0], ALLOC_STATIC);
__ZZ16SelectTranslatorPKcE14stress_amps_de=allocate([20,20,20,20,20,22,22,20] /* \14\14\14\14\14\16\1 */, "i8", ALLOC_STATIC);
__ZZ16SelectTranslatorPKcE17stress_lengths_en=allocate([182, 0, 140, 0, 220, 0, 220, 0, 0, 0, 0, 0, 248, 0, 275, 0], ["i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0], ALLOC_STATIC);
__str630=allocate([97,101,105,111,117,121,0] /* aeiouy\00 */, "i8", ALLOC_STATIC);
__ZZ16SelectTranslatorPKcE17stress_lengths_el=allocate([155, 0, 180, 0, 210, 0, 210, 0, 0, 0, 0, 0, 270, 0, 300, 0], ["i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0], ALLOC_STATIC);
__ZZ16SelectTranslatorPKcE14stress_amps_el=allocate([15,12,20,20,20,22,22,21] /* \0F\0C\14\14\14\16\1 */, "i8", ALLOC_STATIC);
__ZZ16SelectTranslatorPKcE9el_vowels=allocate([16,44,45,46,47,48,49,53,55,57,63,69,73,74,75,76,77,78,79,0] /* \10,-./01579?EIJKLMN */, "i8", ALLOC_STATIC);
__ZZ16SelectTranslatorPKcE10el_fvowels=allocate([45,46,47,53,55,57,69,77,0] /* -./579EM\00 */, "i8", ALLOC_STATIC);
__ZZ16SelectTranslatorPKcE12el_voiceless=allocate([56,58,62,64,66,67,68,70,71,0] /* 8:>@BCDFG\00 */, "i8", ALLOC_STATIC);
__ZZ16SelectTranslatorPKcE13el_consonants=allocate([50,51,52,54,56,58,59,60,61,62,64,65,66,67,68,70,71,72,0] /* 23468:;<=>@ABCDFGH\0 */, "i8", ALLOC_STATIC);
__ZZ16SelectTranslatorPKcE18el_char_apostrophe=allocate([963, 0, 0, 0, 0, 0, 0, 0], ["i32",0,0,0,"i32",0,0,0], ALLOC_STATIC);
__ZZ16SelectTranslatorPKcE17stress_lengths_eo=allocate([150, 0, 150, 0, 230, 0, 180, 0, 0, 0, 0, 0, 300, 0, 320, 0], ["i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0], ALLOC_STATIC);
__ZZ16SelectTranslatorPKcE14stress_amps_eo=allocate([16,14,20,20,20,22,22,21] /* \10\0E\14\14\14\16\1 */, "i8", ALLOC_STATIC);
__ZZ16SelectTranslatorPKcE18eo_char_apostrophe=allocate([108, 0, 0, 0, 0, 0, 0, 0], ["i32",0,0,0,"i32",0,0,0], ALLOC_STATIC);
__ZZ16SelectTranslatorPKcE17stress_lengths_es=allocate([180, 0, 210, 0, 190, 0, 190, 0, 0, 0, 0, 0, 230, 0, 260, 0], ["i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0], ALLOC_STATIC);
__ZZ16SelectTranslatorPKcE14stress_amps_es=allocate([16,12,18,18,20,20,20,20] /* \10\0C\12\12\14\14\1 */, "i8", ALLOC_STATIC);
__ZZ16SelectTranslatorPKcE20ca_punct_within_word=allocate([39, 0, 0, 0, 183, 0, 0, 0, 0, 0, 0, 0], ["i32",0,0,0,"i32",0,0,0,"i32",0,0,0], ALLOC_STATIC);
__ZZ16SelectTranslatorPKcE17stress_lengths_eu=allocate([200, 0, 200, 0, 200, 0, 200, 0, 0, 0, 0, 0, 210, 0, 230, 0], ["i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0], ALLOC_STATIC);
__ZZ16SelectTranslatorPKcE14stress_amps_eu=allocate([16,16,18,18,18,18,18,18] /* \10\10\12\12\12\12\1 */, "i8", ALLOC_STATIC);
__ZZ16SelectTranslatorPKcE14stress_amps_fi=allocate([18,16,22,22,20,22,22,22] /* \12\10\16\16\14\16\1 */, "i8", ALLOC_STATIC);
__ZZ16SelectTranslatorPKcE17stress_lengths_fi=allocate([150, 0, 180, 0, 200, 0, 200, 0, 0, 0, 0, 0, 210, 0, 250, 0], ["i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0], ALLOC_STATIC);
__ZZ16SelectTranslatorPKcE17stress_lengths_hi=allocate([190, 0, 190, 0, 210, 0, 210, 0, 0, 0, 0, 0, 230, 0, 250, 0], ["i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0], ALLOC_STATIC);
__ZZ16SelectTranslatorPKcE14stress_amps_hi=allocate([17,14,20,19,20,22,22,21] /* \11\0E\14\13\14\16\1 */, "i8", ALLOC_STATIC);
__ZZ16SelectTranslatorPKcE14stress_amps_hr=allocate([17,17,20,20,20,22,22,21] /* \11\11\14\14\14\16\1 */, "i8", ALLOC_STATIC);
__ZZ16SelectTranslatorPKcE17stress_lengths_hr=allocate([180, 0, 160, 0, 200, 0, 200, 0, 0, 0, 0, 0, 220, 0, 230, 0], ["i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0], ALLOC_STATIC);
__ZZ16SelectTranslatorPKcE17stress_lengths_sr=allocate([160, 0, 150, 0, 200, 0, 200, 0, 0, 0, 0, 0, 250, 0, 260, 0], ["i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0], ALLOC_STATIC);
__ZL22replace_cyrillic_latin=allocate([1072, 0, 0, 0, 97, 0, 0, 0, 1073, 0, 0, 0, 98, 0, 0, 0, 1094, 0, 0, 0, 99, 0, 0, 0, 1115, 0, 0, 0, 263, 0, 0, 0, 1095, 0, 0, 0, 269, 0, 0, 0, 1119, 0, 0, 0, 25034852, 0, 0, 0, 1109, 0, 0, 0, 7995492, 0, 0, 0, 1076, 0, 0, 0, 100, 0, 0, 0, 1106, 0, 0, 0, 273, 0, 0, 0, 1077, 0, 0, 0, 101, 0, 0, 0, 1092, 0, 0, 0, 102, 0, 0, 0, 1075, 0, 0, 0, 103, 0, 0, 0, 1093, 0, 0, 0, 104, 0, 0, 0, 1080, 0, 0, 0, 105, 0, 0, 0, 1112, 0, 0, 0, 106, 0, 0, 0, 1082, 0, 0, 0, 107, 0, 0, 0, 1113, 0, 0, 0, 6946924, 0, 0, 0, 1083, 0, 0, 0, 108, 0, 0, 0, 1084, 0, 0, 0, 109, 0, 0, 0, 1114, 0, 0, 0, 6946926, 0, 0, 0, 1085, 0, 0, 0, 110, 0, 0, 0, 1086, 0, 0, 0, 111, 0, 0, 0, 1087, 0, 0, 0, 112, 0, 0, 0, 1088, 0, 0, 0, 114, 0, 0, 0, 1089, 0, 0, 0, 115, 0, 0, 0, 1096, 0, 0, 0, 353, 0, 0, 0, 1090, 0, 0, 0, 116, 0, 0, 0, 1091, 0, 0, 0, 117, 0, 0, 0, 1074, 0, 0, 0, 118, 0, 0, 0, 1079, 0, 0, 0, 122, 0, 0, 0, 1078, 0, 0, 0, 382, 0, 0, 0, 1107, 0, 0, 0, 273, 0, 0, 0, 1116, 0, 0, 0, 263, 0, 0, 0, 0, 0, 0, 0], ["i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0], ALLOC_STATIC);
__ZZ16SelectTranslatorPKcE14stress_amps_hu=allocate([17,17,19,19,20,22,22,21] /* \11\11\13\13\14\16\1 */, "i8", ALLOC_STATIC);
__ZZ16SelectTranslatorPKcE17stress_lengths_hu=allocate([185, 0, 195, 0, 195, 0, 190, 0, 0, 0, 0, 0, 210, 0, 220, 0], ["i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0], ALLOC_STATIC);
__ZZ16SelectTranslatorPKcE17stress_lengths_hy=allocate([250, 0, 200, 0, 250, 0, 250, 0, 0, 0, 0, 0, 250, 0, 250, 0], ["i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0], ALLOC_STATIC);
__ZZ16SelectTranslatorPKcE9hy_vowels=allocate([49,53,55,56,59,72,85,0] /* 1578;HU\00 */, "i8", ALLOC_STATIC);
__ZZ16SelectTranslatorPKcE13hy_consonants=allocate([50,51,52,54,57,58,60,61,62,63,64,65,66,67,68,69,70,71,73,74,75,76,77,78,79,80,81,82,83,84,86,0] /* 23469:<=>?@ABCDEFGIJ */, "i8", ALLOC_STATIC);
__ZZ16SelectTranslatorPKcE17stress_lengths_id=allocate([160, 0, 200, 0, 180, 0, 180, 0, 0, 0, 0, 0, 220, 0, 240, 0], ["i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0], ALLOC_STATIC);
__ZZ16SelectTranslatorPKcE14stress_amps_id=allocate([16,18,18,18,20,22,22,21] /* \10\12\12\12\14\16\1 */, "i8", ALLOC_STATIC);
__ZZ16SelectTranslatorPKcE17stress_lengths_is=allocate([180, 0, 160, 0, 200, 0, 200, 0, 0, 0, 0, 0, 240, 0, 250, 0], ["i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0], ALLOC_STATIC);
__ZZ16SelectTranslatorPKcE16is_lettergroup_B=allocate([99, 0, 0, 0, 102, 0, 0, 0, 104, 0, 0, 0, 107, 0, 0, 0, 112, 0, 0, 0, 116, 0, 0, 0, 120, 0, 0, 0, 254, 0, 0, 0, 0, 0, 0, 0], ["i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0], ALLOC_STATIC);
__str1631=allocate([107,112,115,116,0] /* kpst\00 */, "i8", ALLOC_STATIC);
__str2632=allocate([106,118,114,0] /* jvr\00 */, "i8", ALLOC_STATIC);
__ZZ16SelectTranslatorPKcE17stress_lengths_it=allocate([150, 0, 140, 0, 170, 0, 170, 0, 0, 0, 0, 0, 300, 0, 330, 0], ["i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0], ALLOC_STATIC);
__ZZ16SelectTranslatorPKcE14stress_amps_it=allocate([15,14,19,19,20,22,22,20] /* \0F\0E\13\13\14\16\1 */, "i8", ALLOC_STATIC);
__ZZ16SelectTranslatorPKcE18stress_lengths_jbo=allocate([145, 0, 145, 0, 170, 0, 160, 0, 0, 0, 0, 0, 330, 0, 350, 0], ["i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0], ALLOC_STATIC);
__ZZ16SelectTranslatorPKcE21jbo_punct_within_word=allocate([46, 0, 0, 0, 44, 0, 0, 0, 39, 0, 0, 0, 712, 0, 0, 0, 0, 0, 0, 0], ["i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0], ALLOC_STATIC);
__ZZ16SelectTranslatorPKcE9ka_vowels=allocate([80,84,88,93,99,117,119,0] /* PTX]cuw\00 */, "i8", ALLOC_STATIC);
__ZZ16SelectTranslatorPKcE13ka_consonants=allocate([81,82,83,85,86,87,89,90,91,92,94,95,96,97,98,100,101,102,103,104,105,106,107,108,109,110,111,112,113,114,115,116,118,0] /* QRSUVWYZ[\5C^_`abdef */, "i8", ALLOC_STATIC);
__ZZ16SelectTranslatorPKcE10ko_ivowels=allocate([99,100,103,104,109,114,116,117,0] /* cdghmrtu\00 */, "i8", ALLOC_STATIC);
__ZZ16SelectTranslatorPKcE9ko_voiced=allocate([2,5,6,171,175,183,188,0] /* \02\05\06\AB\AF\B7\B */, "i8", ALLOC_STATIC);
__ZZ16SelectTranslatorPKcE14stress_amps_ku=allocate([18,18,20,20,20,22,22,21] /* \12\12\14\14\14\16\1 */, "i8", ALLOC_STATIC);
__ZZ16SelectTranslatorPKcE17stress_lengths_ku=allocate([180, 0, 180, 0, 190, 0, 180, 0, 0, 0, 0, 0, 230, 0, 240, 0], ["i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0], ALLOC_STATIC);
__ZZ16SelectTranslatorPKcE14stress_amps_lv=allocate([17,13,20,20,20,22,22,21] /* \11\0D\14\14\14\16\1 */, "i8", ALLOC_STATIC);
__ZZ16SelectTranslatorPKcE17stress_lengths_lv=allocate([180, 0, 130, 0, 210, 0, 210, 0, 0, 0, 0, 0, 210, 0, 210, 0], ["i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0], ALLOC_STATIC);
__ZZ16SelectTranslatorPKcE15vowels_cyrillic=allocate([1088, 0, 0, 0, 1072, 0, 0, 0, 1077, 0, 0, 0, 1080, 0, 0, 0, 1081, 0, 0, 0, 1086, 0, 0, 0, 1091, 0, 0, 0, 1099, 0, 0, 0, 1101, 0, 0, 0, 1102, 0, 0, 0, 1103, 0, 0, 0, 1104, 0, 0, 0, 1105, 0, 0, 0, 1110, 0, 0, 0, 1111, 0, 0, 0, 1117, 0, 0, 0, 1118, 0, 0, 0, 0, 0, 0, 0], ["i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0], ALLOC_STATIC);
__ZZ16SelectTranslatorPKcE14stress_amps_mk=allocate([17,17,20,20,20,22,22,21] /* \11\11\14\14\14\16\1 */, "i8", ALLOC_STATIC);
__ZZ16SelectTranslatorPKcE17stress_lengths_mk=allocate([180, 0, 160, 0, 200, 0, 200, 0, 0, 0, 0, 0, 220, 0, 230, 0], ["i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0], ALLOC_STATIC);
__ZZ16SelectTranslatorPKcE17stress_lengths_nl=allocate([160, 0, 135, 0, 210, 0, 210, 0, 0, 0, 0, 0, 260, 0, 280, 0], ["i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0], ALLOC_STATIC);
__str3633=allocate([101,0] /* e\00 */, "i8", ALLOC_STATIC);
__ZZ16SelectTranslatorPKcE17stress_lengths_no=allocate([160, 0, 140, 0, 200, 0, 200, 0, 0, 0, 0, 0, 220, 0, 230, 0], ["i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0], ALLOC_STATIC);
__ZZ16SelectTranslatorPKcE14stress_amps_om=allocate([18,15,20,20,20,22,22,22] /* \12\0F\14\14\14\16\1 */, "i8", ALLOC_STATIC);
__ZZ16SelectTranslatorPKcE17stress_lengths_om=allocate([200, 0, 200, 0, 200, 0, 200, 0, 0, 0, 0, 0, 200, 0, 200, 0], ["i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0], ALLOC_STATIC);
__ZZ16SelectTranslatorPKcE17stress_lengths_pl=allocate([160, 0, 190, 0, 175, 0, 175, 0, 0, 0, 0, 0, 200, 0, 210, 0], ["i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0], ALLOC_STATIC);
__ZZ16SelectTranslatorPKcE14stress_amps_pl=allocate([17,13,19,19,20,22,22,21] /* \11\0D\13\13\14\16\1 */, "i8", ALLOC_STATIC);
__ZZ16SelectTranslatorPKcE17stress_lengths_pt=allocate([170, 0, 115, 0, 210, 0, 240, 0, 0, 0, 0, 0, 260, 0, 280, 0], ["i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0], ALLOC_STATIC);
__ZZ16SelectTranslatorPKcE14stress_amps_pt=allocate([16,11,19,21,20,22,22,21] /* \10\0B\13\15\14\16\1 */, "i8", ALLOC_STATIC);
__str4634=allocate([98,99,100,102,103,106,107,109,110,112,113,115,116,118,120,122,0] /* bcdfgjkmnpqstvxz\00 */, "i8", ALLOC_STATIC);
__ZZ16SelectTranslatorPKcE17stress_lengths_ro=allocate([170, 0, 170, 0, 180, 0, 180, 0, 0, 0, 0, 0, 240, 0, 260, 0], ["i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0], ALLOC_STATIC);
__ZZ16SelectTranslatorPKcE14stress_amps_ro=allocate([15,13,18,18,20,22,22,21] /* \0F\0D\12\12\14\16\1 */, "i8", ALLOC_STATIC);
__str5635=allocate([98,100,103,106,108,109,110,114,118,119,122,97,101,105,111,117,121,0] /* bdgjlmnrvwzaeiouy\00 */, "i8", ALLOC_STATIC);
__ZZ16SelectTranslatorPKcE17stress_lengths_sq=allocate([150, 0, 150, 0, 180, 0, 180, 0, 0, 0, 0, 0, 300, 0, 300, 0], ["i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0], ALLOC_STATIC);
__ZZ16SelectTranslatorPKcE14stress_amps_sq=allocate([16,12,16,16,20,20,21,19] /* \10\0C\10\10\14\14\1 */, "i8", ALLOC_STATIC);
__ZZ16SelectTranslatorPKcE14stress_amps_sv=allocate([16,16,20,20,20,22,22,21] /* \10\10\14\14\14\16\1 */, "i8", ALLOC_STATIC);
__ZZ16SelectTranslatorPKcE17stress_lengths_sv=allocate([160, 0, 135, 0, 220, 0, 220, 0, 0, 0, 0, 0, 250, 0, 280, 0], ["i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0], ALLOC_STATIC);
__ZZ16SelectTranslatorPKcE17stress_lengths_sw=allocate([160, 0, 170, 0, 200, 0, 200, 0, 0, 0, 0, 0, 320, 0, 340, 0], ["i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0], ALLOC_STATIC);
__ZZ16SelectTranslatorPKcE14stress_amps_sw=allocate([16,12,19,19,20,22,22,21] /* \10\0C\13\13\14\16\1 */, "i8", ALLOC_STATIC);
__ZZ16SelectTranslatorPKcE14stress_amps_tr=allocate([18,16,20,21,20,21,21,20] /* \12\10\14\15\14\15\1 */, "i8", ALLOC_STATIC);
__ZZ16SelectTranslatorPKcE17stress_lengths_tr=allocate([190, 0, 180, 0, 200, 0, 230, 0, 0, 0, 0, 0, 240, 0, 250, 0], ["i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0], ALLOC_STATIC);
__ZZ16SelectTranslatorPKcE17stress_lengths_vi=allocate([150, 0, 150, 0, 180, 0, 180, 0, 210, 0, 230, 0, 230, 0, 240, 0], ["i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0], ALLOC_STATIC);
__ZZ16SelectTranslatorPKcE14stress_amps_vi=allocate([16,16,16,16,22,22,22,22] /* \10\10\10\10\16\16\1 */, "i8", ALLOC_STATIC);
__ZZ16SelectTranslatorPKcE9vowels_vi=allocate([97, 0, 0, 0, 224, 0, 0, 0, 225, 0, 0, 0, 7843, 0, 0, 0, 227, 0, 0, 0, 7841, 0, 0, 0, 259, 0, 0, 0, 7857, 0, 0, 0, 7855, 0, 0, 0, 7859, 0, 0, 0, 7861, 0, 0, 0, 7863, 0, 0, 0, 226, 0, 0, 0, 7847, 0, 0, 0, 7845, 0, 0, 0, 7849, 0, 0, 0, 7851, 0, 0, 0, 7853, 0, 0, 0, 101, 0, 0, 0, 232, 0, 0, 0, 233, 0, 0, 0, 7867, 0, 0, 0, 7869, 0, 0, 0, 7865, 0, 0, 0, 234, 0, 0, 0, 7873, 0, 0, 0, 7871, 0, 0, 0, 7875, 0, 0, 0, 7877, 0, 0, 0, 7879, 0, 0, 0, 105, 0, 0, 0, 236, 0, 0, 0, 237, 0, 0, 0, 7881, 0, 0, 0, 297, 0, 0, 0, 7883, 0, 0, 0, 111, 0, 0, 0, 242, 0, 0, 0, 243, 0, 0, 0, 7887, 0, 0, 0, 245, 0, 0, 0, 7885, 0, 0, 0, 244, 0, 0, 0, 7891, 0, 0, 0, 7889, 0, 0, 0, 7893, 0, 0, 0, 7895, 0, 0, 0, 7897, 0, 0, 0, 417, 0, 0, 0, 7901, 0, 0, 0, 7899, 0, 0, 0, 7903, 0, 0, 0, 7905, 0, 0, 0, 7907, 0, 0, 0, 117, 0, 0, 0, 249, 0, 0, 0, 250, 0, 0, 0, 7911, 0, 0, 0, 361, 0, 0, 0, 7909, 0, 0, 0, 432, 0, 0, 0, 7915, 0, 0, 0, 7913, 0, 0, 0, 7917, 0, 0, 0, 7919, 0, 0, 0, 7921, 0, 0, 0, 121, 0, 0, 0, 7923, 0, 0, 0, 253, 0, 0, 0, 7927, 0, 0, 0, 7929, 0, 0, 0, 30, 0, 0, 0, 0, 0, 0, 0], ["i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0], ALLOC_STATIC);
__ZZ16SelectTranslatorPKcE17stress_lengths_zh=allocate([230, 0, 150, 0, 230, 0, 230, 0, 230, 0, 0, 0, 240, 0, 250, 0], ["i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0], ALLOC_STATIC);
__ZZ16SelectTranslatorPKcE14stress_amps_zh=allocate([22,16,22,22,22,22,22,22] /* \16\10\16\16\16\16\1 */, "i8", ALLOC_STATIC);
__ZZL18Translator_RussianP10TranslatorE14stress_amps_ru=allocate([16,16,18,18,20,24,24,22] /* \10\10\12\12\14\18\1 */, "i8", ALLOC_STATIC);
__ZZL18Translator_RussianP10TranslatorE17stress_lengths_ru=allocate([150, 0, 140, 0, 220, 0, 220, 0, 0, 0, 0, 0, 260, 0, 280, 0], ["i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0], ALLOC_STATIC);
__ZZL18Translator_RussianP10TranslatorE11ru_ivowels2=allocate([44,21,24,46,47,0] /* ,\15\18./\00 */, "i8", ALLOC_STATIC);
__ZZL18SetCyrillicLettersP10TranslatorE9ru_vowels=allocate([16,21,49,24,30,35,43,45,46,47,0] /* \10\151\18\1E#+-./\0 */, "i8", ALLOC_STATIC);
__ZZL18SetCyrillicLettersP10TranslatorE13ru_consonants=allocate([17,18,19,20,22,23,25,26,27,28,29,31,32,33,34,36,37,38,39,40,41,42,44,0] /* \11\12\13\14\16\17\1 */, "i8", ALLOC_STATIC);
__ZZL18SetCyrillicLettersP10TranslatorE7ru_soft=allocate([44,25,39,41,0] /* ,\19')\00 */, "i8", ALLOC_STATIC);
__ZZL18SetCyrillicLettersP10TranslatorE7ru_hard=allocate([42,22,38,40,0] /* _\16&(\00 */, "i8", ALLOC_STATIC);
__ZZL18SetCyrillicLettersP10TranslatorE10ru_nothard=allocate([17,18,19,20,23,25,26,27,28,29,31,32,33,34,36,37,39,41,44,0] /* \11\12\13\14\17\19\1 */, "i8", ALLOC_STATIC);
__ZZL18SetCyrillicLettersP10TranslatorE9ru_voiced=allocate([17,18,19,20,22,23,0] /* \11\12\13\14\16\17\0 */, "i8", ALLOC_STATIC);
__ZZL18SetCyrillicLettersP10TranslatorE10ru_ivowels=allocate([44,46,47,49,0] /* ,./1\00 */, "i8", ALLOC_STATIC);
__ZL8pairs_ru=allocate([268, 0, 270, 0, 275, 0, 769, 0, 783, 0, 1550, 0, 1553, 0, 2307, 0, 2817, 0, 2831, 0, 3073, 0, 3081, 0, 3585, 0, 3590, 0, 3593, 0, 3598, 0, 3599, 0, 3612, 0, 3843, 0, 3857, 0, 3858, 0, 4111, 0, 4113, 0, 4353, 0, 4358, 0, 4361, 0, 4367, 0, 4627, 0, 4640, 0, 32767, 0], ["i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0], ALLOC_STATIC);
__ZZL13NewTranslatorvE12stress_amps2=allocate([18,18,20,20,20,22,22,20] /* \12\12\14\14\14\16\1 */, "i8", ALLOC_STATIC);
__ZZL13NewTranslatorvE15stress_lengths2=allocate([182, 0, 140, 0, 220, 0, 220, 0, 220, 0, 240, 0, 260, 0, 280, 0], ["i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0], ALLOC_STATIC);
__ZZL13NewTranslatorvE13empty_wstring=allocate(4, "i32", ALLOC_STATIC);
__ZZL13NewTranslatorvE13punct_in_word=allocate([39, 0, 0, 0, 0, 0, 0, 0], ["i32",0,0,0,"i32",0,0,0], ALLOC_STATIC);
__ZZL13NewTranslatorvE13default_tunes=allocate([0,1,2,3,0,0] /* \00\01\02\03\00\00 */, "i8", ALLOC_STATIC);
__str6636=allocate([97,101,105,111,117,0] /* aeiou\00 */, "i8", ALLOC_STATIC);
__str7637=allocate([98,99,100,102,103,106,107,108,109,110,112,113,115,116,118,120,122,0] /* bcdfgjklmnpqstvxz\00 */, "i8", ALLOC_STATIC);
__str8638=allocate([98,99,100,102,103,104,106,107,108,109,110,112,113,114,115,116,118,119,120,122,0] /* bcdfghjklmnpqrstvwxz */, "i8", ALLOC_STATIC);
__str9639=allocate([104,108,109,110,114,0] /* hlmnr\00 */, "i8", ALLOC_STATIC);
__str10640=allocate([99,102,104,107,112,113,115,116,120,0] /* cfhkpqstx\00 */, "i8", ALLOC_STATIC);
__str11641=allocate([98,100,103,106,108,109,110,114,118,119,121,122,0] /* bdgjlmnrvwyz\00 */, "i8", ALLOC_STATIC);
__str12642=allocate([101,105,121,0] /* eiy\00 */, "i8", ALLOC_STATIC);
__str13643=allocate(1, "i8", ALLOC_STATIC);
__str646=allocate([117,110,107,110,111,119,110,0] /* unknown\00 */, "i8", ALLOC_STATIC);
__str1647=allocate([109,97,108,101,0] /* male\00 */, "i8", ALLOC_STATIC);
__str2648=allocate([102,101,109,97,108,101,0] /* female\00 */, "i8", ALLOC_STATIC);
_genders=allocate([0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 0, 0, 0, 0, 0, 0, 0, 2, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0], ["i8*",0,0,0,"i32",0,0,0,"i8*",0,0,0,"i32",0,0,0,"i8*",0,0,0,"i32",0,0,0,"i8*",0,0,0,"i32",0,0,0], ALLOC_STATIC);
_tone_points=allocate([600, 0, 0, 0, 170, 0, 0, 0, 1200, 0, 0, 0, 135, 0, 0, 0, 2000, 0, 0, 0, 110, 0, 0, 0, 3000, 0, 0, 0, 110, 0, 0, 0, -1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0], ["i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0], ALLOC_STATIC);
_formant_rate=allocate(36, "i32", ALLOC_STATIC);
_voice_selected=allocate(24, ["i8*",0,0,0,"i8*",0,0,0,"i8*",0,0,0,"i8","i8","i8","i8","i32",0,0,0,"i8*",0,0,0], ALLOC_STATIC);
__str3653=allocate([114,101,100,117,99,101,95,116,0] /* reduce_t\00 */, "i8", ALLOC_STATIC);
__str4654=allocate([98,114,97,99,107,101,116,0] /* bracket\00 */, "i8", ALLOC_STATIC);
__ZL11options_tab=allocate([0, 0, 0, 0, 12, 0, 0, 0, 0, 0, 0, 0, 16, 0, 0, 0, 0, 0, 0, 0, -1, 0, 0, 0], ["i8*",0,0,0,"i32",0,0,0,"i8*",0,0,0,"i32",0,0,0,"i8*",0,0,0,"i32",0,0,0], ALLOC_STATIC);
__str5655=allocate([110,97,109,101,0] /* name\00 */, "i8", ALLOC_STATIC);
__str6656=allocate([108,97,110,103,117,97,103,101,0] /* language\00 */, "i8", ALLOC_STATIC);
__str7657=allocate([103,101,110,100,101,114,0] /* gender\00 */, "i8", ALLOC_STATIC);
__str8658=allocate([102,111,114,109,97,110,116,0] /* formant\00 */, "i8", ALLOC_STATIC);
__str9659=allocate([112,105,116,99,104,0] /* pitch\00 */, "i8", ALLOC_STATIC);
__str10660=allocate([112,104,111,110,101,109,101,115,0] /* phonemes\00 */, "i8", ALLOC_STATIC);
__str11661=allocate([116,114,97,110,115,108,97,116,111,114,0] /* translator\00 */, "i8", ALLOC_STATIC);
__str12662=allocate([100,105,99,116,105,111,110,97,114,121,0] /* dictionary\00 */, "i8", ALLOC_STATIC);
__str13663=allocate([115,116,114,101,115,115,76,101,110,103,116,104,0] /* stressLength\00 */, "i8", ALLOC_STATIC);
__str14664=allocate([115,116,114,101,115,115,65,109,112,0] /* stressAmp\00 */, "i8", ALLOC_STATIC);
__str15665=allocate([115,116,114,101,115,115,65,100,100,0] /* stressAdd\00 */, "i8", ALLOC_STATIC);
__str16666=allocate([105,110,116,111,110,97,116,105,111,110,0] /* intonation\00 */, "i8", ALLOC_STATIC);
__str17667=allocate([116,117,110,101,115,0] /* tunes\00 */, "i8", ALLOC_STATIC);
__str18668=allocate([100,105,99,116,114,117,108,101,115,0] /* dictrules\00 */, "i8", ALLOC_STATIC);
__str19669=allocate([115,116,114,101,115,115,114,117,108,101,0] /* stressrule\00 */, "i8", ALLOC_STATIC);
__str20670=allocate([99,104,97,114,115,101,116,0] /* charset\00 */, "i8", ALLOC_STATIC);
__str21671=allocate([114,101,112,108,97,99,101,0] /* replace\00 */, "i8", ALLOC_STATIC);
__str22672=allocate([119,111,114,100,115,0] /* words\00 */, "i8", ALLOC_STATIC);
__str23673=allocate([101,99,104,111,0] /* echo\00 */, "i8", ALLOC_STATIC);
__str24674=allocate([102,108,117,116,116,101,114,0] /* flutter\00 */, "i8", ALLOC_STATIC);
__str25675=allocate([114,111,117,103,104,110,101,115,115,0] /* roughness\00 */, "i8", ALLOC_STATIC);
__str26676=allocate([99,108,97,114,105,116,121,0] /* clarity\00 */, "i8", ALLOC_STATIC);
__str27677=allocate([116,111,110,101,0] /* tone\00 */, "i8", ALLOC_STATIC);
__str28678=allocate([118,111,105,99,105,110,103,0] /* voicing\00 */, "i8", ALLOC_STATIC);
__str29679=allocate([98,114,101,97,116,104,0] /* breath\00 */, "i8", ALLOC_STATIC);
__str30680=allocate([98,114,101,97,116,104,119,0] /* breathw\00 */, "i8", ALLOC_STATIC);
__str31681=allocate([110,117,109,98,101,114,115,0] /* numbers\00 */, "i8", ALLOC_STATIC);
__str32682=allocate([111,112,116,105,111,110,0] /* option\00 */, "i8", ALLOC_STATIC);
__str33683=allocate([109,98,114,111,108,97,0] /* mbrola\00 */, "i8", ALLOC_STATIC);
__str34684=allocate([99,111,110,115,111,110,97,110,116,115,0] /* consonants\00 */, "i8", ALLOC_STATIC);
__str35685=allocate([107,108,97,116,116,0] /* klatt\00 */, "i8", ALLOC_STATIC);
__str36686=allocate([102,97,115,116,95,116,101,115,116,50,0] /* fast_test2\00 */, "i8", ALLOC_STATIC);
__str37687=allocate([115,112,101,101,100,0] /* speed\00 */, "i8", ALLOC_STATIC);
__str38688=allocate([108,95,100,105,101,114,101,115,105,115,0] /* l_dieresis\00 */, "i8", ALLOC_STATIC);
__str39689=allocate([108,95,112,114,101,102,105,120,0] /* l_prefix\00 */, "i8", ALLOC_STATIC);
__str40690=allocate([108,95,114,101,103,114,101,115,115,105,118,101,95,118,0] /* l_regressive_v\00 */, "i8", ALLOC_STATIC);
__str41691=allocate([108,95,117,110,112,114,111,110,111,117,110,99,97,98,108,101,0] /* l_unpronouncable\00 */, "i8", ALLOC_STATIC);
__str42692=allocate([108,95,115,111,110,111,114,97,110,116,95,109,105,110,0] /* l_sonorant_min\00 */, "i8", ALLOC_STATIC);
__str43693=allocate([108,95,108,101,110,103,116,104,95,109,111,100,115,0] /* l_length_mods\00 */, "i8", ALLOC_STATIC);
__ZL11keyword_tab=allocate([0, 0, 0, 0, 1, 0, 0, 0, 0, 0, 0, 0, 2, 0, 0, 0, 0, 0, 0, 0, 3, 0, 0, 0, 0, 0, 0, 0, 7, 0, 0, 0, 0, 0, 0, 0, 8, 0, 0, 0, 0, 0, 0, 0, 5, 0, 0, 0, 0, 0, 0, 0, 4, 0, 0, 0, 0, 0, 0, 0, 6, 0, 0, 0, 0, 0, 0, 0, 20, 0, 0, 0, 0, 0, 0, 0, 21, 0, 0, 0, 0, 0, 0, 0, 22, 0, 0, 0, 0, 0, 0, 0, 18, 0, 0, 0, 0, 0, 0, 0, 19, 0, 0, 0, 0, 0, 0, 0, 23, 0, 0, 0, 0, 0, 0, 0, 24, 0, 0, 0, 0, 0, 0, 0, 25, 0, 0, 0, 0, 0, 0, 0, 32, 0, 0, 0, 0, 0, 0, 0, 17, 0, 0, 0, 0, 0, 0, 0, 9, 0, 0, 0, 0, 0, 0, 0, 10, 0, 0, 0, 0, 0, 0, 0, 11, 0, 0, 0, 0, 0, 0, 0, 12, 0, 0, 0, 0, 0, 0, 0, 13, 0, 0, 0, 0, 0, 0, 0, 14, 0, 0, 0, 0, 0, 0, 0, 15, 0, 0, 0, 0, 0, 0, 0, 16, 0, 0, 0, 0, 0, 0, 0, 26, 0, 0, 0, 0, 0, 0, 0, 27, 0, 0, 0, 0, 0, 0, 0, 28, 0, 0, 0, 0, 0, 0, 0, 33, 0, 0, 0, 0, 0, 0, 0, 29, 0, 0, 0, 0, 0, 0, 0, 30, 0, 0, 0, 0, 0, 0, 0, 31, 0, 0, 0, 0, 0, 0, 0, 257, 0, 0, 0, 0, 0, 0, 0, 259, 0, 0, 0, 0, 0, 0, 0, 260, 0, 0, 0, 0, 0, 0, 0, 261, 0, 0, 0, 0, 0, 0, 0, 263, 0, 0, 0, 0, 0, 0, 0, 262, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0], ["i8*",0,0,0,"i32",0,0,0,"i8*",0,0,0,"i32",0,0,0,"i8*",0,0,0,"i32",0,0,0,"i8*",0,0,0,"i32",0,0,0,"i8*",0,0,0,"i32",0,0,0,"i8*",0,0,0,"i32",0,0,0,"i8*",0,0,0,"i32",0,0,0,"i8*",0,0,0,"i32",0,0,0,"i8*",0,0,0,"i32",0,0,0,"i8*",0,0,0,"i32",0,0,0,"i8*",0,0,0,"i32",0,0,0,"i8*",0,0,0,"i32",0,0,0,"i8*",0,0,0,"i32",0,0,0,"i8*",0,0,0,"i32",0,0,0,"i8*",0,0,0,"i32",0,0,0,"i8*",0,0,0,"i32",0,0,0,"i8*",0,0,0,"i32",0,0,0,"i8*",0,0,0,"i32",0,0,0,"i8*",0,0,0,"i32",0,0,0,"i8*",0,0,0,"i32",0,0,0,"i8*",0,0,0,"i32",0,0,0,"i8*",0,0,0,"i32",0,0,0,"i8*",0,0,0,"i32",0,0,0,"i8*",0,0,0,"i32",0,0,0,"i8*",0,0,0,"i32",0,0,0,"i8*",0,0,0,"i32",0,0,0,"i8*",0,0,0,"i32",0,0,0,"i8*",0,0,0,"i32",0,0,0,"i8*",0,0,0,"i32",0,0,0,"i8*",0,0,0,"i32",0,0,0,"i8*",0,0,0,"i32",0,0,0,"i8*",0,0,0,"i32",0,0,0,"i8*",0,0,0,"i32",0,0,0,"i8*",0,0,0,"i32",0,0,0,"i8*",0,0,0,"i32",0,0,0,"i8*",0,0,0,"i32",0,0,0,"i8*",0,0,0,"i32",0,0,0,"i8*",0,0,0,"i32",0,0,0,"i8*",0,0,0,"i32",0,0,0,"i8*",0,0,0,"i32",0,0,0], ALLOC_STATIC);
__ZL15variants_either=allocate([1,2,12,3,13,4,14,5,11,0,0,0] /* \01\02\0C\03\0D\04\0 */, "i8", ALLOC_STATIC);
__ZL13variants_male=allocate([1,2,3,4,5,0,0,0,0,0,0,0] /* \01\02\03\04\05\00\0 */, "i8", ALLOC_STATIC);
__ZL15variants_female=allocate([11,12,13,14,0,0,0,0,0,0,0,0] /* \0B\0C\0D\0E\00\00\0 */, "i8", ALLOC_STATIC);
_variant_lists=allocate(12, "i8*", ALLOC_STATIC);
__ZL9voicedata=allocate(1344, ["i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8"], ALLOC_STATIC);
_voice=allocate(1, "%struct.voice_t*", ALLOC_STATIC);
__str44696=allocate([37,100,32,37,100,32,37,100,32,37,100,32,37,100,32,37,100,32,37,100,32,37,100,32,37,100,32,37,100,0] /* %d %d %d %d %d %d %d */, "i8", ALLOC_STATIC);
__ZZ10VoiceResetiE15default_heights=allocate([128,128,120,120,110,110,128,128,128] /* \80\80xxnn\80\80\80 */, "i8", ALLOC_STATIC);
__ZZ10VoiceResetiE14default_widths=allocate([128,128,128,160,171,171,128,128,128] /* \80\80\80\A0\AB\AB\8 */, "i8", ALLOC_STATIC);
__ZZ10VoiceResetiE13breath_widths=allocate([0, 0, 0, 0, 200, 0, 0, 0, 200, 0, 0, 0, 400, 0, 0, 0, 400, 0, 0, 0, 400, 0, 0, 0, 600, 0, 0, 0, 600, 0, 0, 0, 600, 0, 0, 0], ["i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0], ALLOC_STATIC);
__ZL18formant_rate_22050=allocate([240, 0, 0, 0, 170, 0, 0, 0, 170, 0, 0, 0, 170, 0, 0, 0, 170, 0, 0, 0, 170, 0, 0, 0, 170, 0, 0, 0, 170, 0, 0, 0, 170, 0, 0, 0], ["i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0], ALLOC_STATIC);
__ZZ9LoadVoicePKciE16voice_identifier=allocate(40, "i8", ALLOC_STATIC);
__ZZ9LoadVoicePKciE10voice_name=allocate(40, "i8", ALLOC_STATIC);
__ZZ9LoadVoicePKciE15voice_languages=allocate(100, "i8", ALLOC_STATIC);
__str45697=allocate([100,101,102,97,117,108,116,0] /* default\00 */, "i8", ALLOC_STATIC);
__str46698=allocate([37,115,37,99,118,111,105,99,101,115,37,99,0] /* %s%cvoices%c\00 */, "i8", ALLOC_STATIC);
__str47699=allocate([37,115,37,115,0] /* %s%s\00 */, "i8", ALLOC_STATIC);
__str48700=allocate([37,115,37,115,37,99,37,115,0] /* %s%s%c%s\00 */, "i8", ALLOC_STATIC);
__str49701=allocate([37,115,101,120,116,114,97,37,99,37,115,0] /* %sextra%c%s\00 */, "i8", ALLOC_STATIC);
__str50702=allocate([37,115,116,101,115,116,37,99,37,115,0] /* %stest%c%s\00 */, "i8", ALLOC_STATIC);
__str51703=allocate([114,0] /* r\00 */, "i8", ALLOC_STATIC);
__str52704=allocate([101,110,0] /* en\00 */, "i8", ALLOC_STATIC);
__str53705=allocate([43,37,115,0] /* +%s\00 */, "i8", ALLOC_STATIC);
__str54706=allocate([37,115,32,37,100,0] /* %s %d\00 */, "i8", ALLOC_STATIC);
__str55707=allocate([118,97,114,105,97,110,116,0] /* variant\00 */, "i8", ALLOC_STATIC);
__str56708=allocate([45,0] /* -\00 */, "i8", ALLOC_STATIC);
__str57709=allocate([37,115,0] /* %s\00 */, "i8", ALLOC_STATIC);
__str58710=allocate([37,100,32,37,100,0] /* %d %d\00 */, "i8", ALLOC_STATIC);
__str59711=allocate([37,115,32,37,115,32,37,115,32,37,115,32,37,115,32,37,115,0] /* %s %s %s %s %s %s\00 */, "i8", ALLOC_STATIC);
__str60712=allocate([78,85,76,76,0] /* NULL\00 */, "i8", ALLOC_STATIC);
__str61713=allocate([85,110,107,110,111,119,110,32,116,117,110,101,32,39,37,115,39,10,0] /* Unknown tune '%s'\0A */, "i8", ALLOC_STATIC);
__str62714=allocate([37,100,32,37,100,32,37,100,32,37,100,0] /* %d %d %d %d\00 */, "i8", ALLOC_STATIC);
__str63715=allocate([37,100,0] /* %d\00 */, "i8", ALLOC_STATIC);
__str64716=allocate([37,115,32,37,100,32,37,100,0] /* %s %d %d\00 */, "i8", ALLOC_STATIC);
__str65717=allocate([66,97,100,32,118,111,105,99,101,32,111,112,116,105,111,110,58,32,37,115,32,37,115,10,0] /* Bad voice option: %s */, "i8", ALLOC_STATIC);
__str66718=allocate([37,115,32,37,115,32,37,100,0] /* %s %s %d\00 */, "i8", ALLOC_STATIC);
__str67719=allocate([109,98,114,111,108,97,32,118,111,105,99,101,32,110,111,116,32,102,111,117,110,100,10,0] /* mbrola voice not fou */, "i8", ALLOC_STATIC);
__str68720=allocate([66,97,100,32,118,111,105,99,101,32,97,116,116,114,105,98,117,116,101,58,32,37,115,10,0] /* Bad voice attribute: */, "i8", ALLOC_STATIC);
__str69721=allocate([85,110,107,110,111,119,110,32,112,104,111,110,101,109,101,32,116,97,98,108,101,58,32,39,37,115,39,10,0] /* Unknown phoneme tabl */, "i8", ALLOC_STATIC);
__ZL13n_voices_list=allocate(1, "i32", ALLOC_STATIC);
__ZL11voices_list=allocate(600, "%struct.espeak_VOICE*", ALLOC_STATIC);
__str70722=allocate([37,99,37,115,0] /* %c%s\00 */, "i8", ALLOC_STATIC);
__ZZ11SelectVoiceP12espeak_VOICEPiE14voice_variants=allocate(288, ["i8*",0,0,0,"i8*",0,0,0,"i8*",0,0,0,"i8","i8","i8","i8","i32",0,0,0,"i8*",0,0,0,"i8*",0,0,0,"i8*",0,0,0,"i8*",0,0,0,"i8","i8","i8","i8","i32",0,0,0,"i8*",0,0,0,"i8*",0,0,0,"i8*",0,0,0,"i8*",0,0,0,"i8","i8","i8","i8","i32",0,0,0,"i8*",0,0,0,"i8*",0,0,0,"i8*",0,0,0,"i8*",0,0,0,"i8","i8","i8","i8","i32",0,0,0,"i8*",0,0,0,"i8*",0,0,0,"i8*",0,0,0,"i8*",0,0,0,"i8","i8","i8","i8","i32",0,0,0,"i8*",0,0,0,"i8*",0,0,0,"i8*",0,0,0,"i8*",0,0,0,"i8","i8","i8","i8","i32",0,0,0,"i8*",0,0,0,"i8*",0,0,0,"i8*",0,0,0,"i8*",0,0,0,"i8","i8","i8","i8","i32",0,0,0,"i8*",0,0,0,"i8*",0,0,0,"i8*",0,0,0,"i8*",0,0,0,"i8","i8","i8","i8","i32",0,0,0,"i8*",0,0,0,"i8*",0,0,0,"i8*",0,0,0,"i8*",0,0,0,"i8","i8","i8","i8","i32",0,0,0,"i8*",0,0,0,"i8*",0,0,0,"i8*",0,0,0,"i8*",0,0,0,"i8","i8","i8","i8","i32",0,0,0,"i8*",0,0,0,"i8*",0,0,0,"i8*",0,0,0,"i8*",0,0,0,"i8","i8","i8","i8","i32",0,0,0,"i8*",0,0,0,"i8*",0,0,0,"i8*",0,0,0,"i8*",0,0,0,"i8","i8","i8","i8","i32",0,0,0,"i8*",0,0,0], ALLOC_STATIC);
__ZZ11SelectVoiceP12espeak_VOICEPiE8voice_id=allocate(50, "i8", ALLOC_STATIC);
__ZZ11SelectVoiceP12espeak_VOICEPiE3buf=allocate(60, "i8", ALLOC_STATIC);
__str71723=allocate([37,115,43,37,115,0] /* %s+%s\00 */, "i8", ALLOC_STATIC);
__ZZ14SetVoiceByNamePKcE3buf=allocate(60, "i8", ALLOC_STATIC);
__ZZ17espeak_ListVoicesE6voices=allocate(600, "%struct.espeak_VOICE*", ALLOC_STATIC);
__str72724=allocate([37,115,37,99,118,111,105,99,101,115,0] /* %s%cvoices\00 */, "i8", ALLOC_STATIC);
__ZL15len_path_voices=allocate(1, "i32", ALLOC_STATIC);
__str73725=allocate([109,98,47,0] /* mb/\00 */, "i8", ALLOC_STATIC);
__str74726=allocate([37,115,37,99,37,115,0] /* %s%c%s\00 */, "i8", ALLOC_STATIC);
__str75727=allocate([118,97,114,105,97,110,116,115,0] /* variants\00 */, "i8", ALLOC_STATIC);
__ZZL23ExtractVoiceVariantNamePciE12variant_name=allocate(20, "i8", ALLOC_STATIC);
__str76728=allocate([33,118,37,99,0] /* !v%c\00 */, "i8", ALLOC_STATIC);
__str77729=allocate([37,115,109,37,100,0] /* %sm%d\00 */, "i8", ALLOC_STATIC);
__str78730=allocate([37,115,102,37,100,0] /* %sf%d\00 */, "i8", ALLOC_STATIC);
__str79731=allocate([37,100,32,37,115,32,37,115,0] /* %d %s %s\00 */, "i8", ALLOC_STATIC);
__str80732=allocate([37,100,32,37,100,32,37,100,32,37,100,32,37,100,32,37,100,32,37,100,32,37,100,0] /* %d %d %d %d %d %d %d */, "i8", ALLOC_STATIC);
__str81733=allocate([37,100,32,37,100,32,37,100,32,37,100,32,37,100,0] /* %d %d %d %d %d\00 */, "i8", ALLOC_STATIC);
__str82734=allocate([47,47,0] /* //\00 */, "i8", ALLOC_STATIC);
_sin_tab=allocate([0, 0, -25, 0, -50, 0, -75, 0, -100, 0, -125, 0, -150, 0, -175, 0, -201, 0, -226, 0, -251, 0, -276, 0, -301, 0, -326, 0, -351, 0, -376, 0, -401, 0, -427, 0, -452, 0, -477, 0, -502, 0, -527, 0, -552, 0, -577, 0, -602, 0, -627, 0, -652, 0, -677, 0, -702, 0, -727, 0, -752, 0, -777, 0, -802, 0, -827, 0, -852, 0, -877, 0, -902, 0, -927, 0, -952, 0, -977, 0, -1002, 0, -1027, 0, -1052, 0, -1077, 0, -1102, 0, -1127, 0, -1152, 0, -1177, 0, -1201, 0, -1226, 0, -1251, 0, -1276, 0, -1301, 0, -1326, 0, -1350, 0, -1375, 0, -1400, 0, -1425, 0, -1449, 0, -1474, 0, -1499, 0, -1523, 0, -1548, 0, -1573, 0, -1597, 0, -1622, 0, -1647, 0, -1671, 0, -1696, 0, -1721, 0, -1745, 0, -1770, 0, -1794, 0, -1819, 0, -1843, 0, -1868, 0, -1892, 0, -1917, 0, -1941, 0, -1965, 0, -1990, 0, -2014, 0, -2038, 0, -2063, 0, -2087, 0, -2111, 0, -2136, 0, -2160, 0, -2184, 0, -2208, 0, -2233, 0, -2257, 0, -2281, 0, -2305, 0, -2329, 0, -2353, 0, -2377, 0, -2401, 0, -2425, 0, -2449, 0, -2473, 0, -2497, 0, -2521, 0, -2545, 0, -2569, 0, -2593, 0, -2617, 0, -2640, 0, -2664, 0, -2688, 0, -2712, 0, -2735, 0, -2759, 0, -2783, 0, -2806, 0, -2830, 0, -2853, 0, -2877, 0, -2900, 0, -2924, 0, -2947, 0, -2971, 0, -2994, 0, -3018, 0, -3041, 0, -3064, 0, -3088, 0, -3111, 0, -3134, 0, -3157, 0, -3180, 0, -3204, 0, -3227, 0, -3250, 0, -3273, 0, -3296, 0, -3319, 0, -3342, 0, -3365, 0, -3388, 0, -3410, 0, -3433, 0, -3456, 0, -3479, 0, -3502, 0, -3524, 0, -3547, 0, -3570, 0, -3592, 0, -3615, 0, -3637, 0, -3660, 0, -3682, 0, -3705, 0, -3727, 0, -3749, 0, -3772, 0, -3794, 0, -3816, 0, -3839, 0, -3861, 0, -3883, 0, -3905, 0, -3927, 0, -3949, 0, -3971, 0, -3993, 0, -4015, 0, -4037, 0, -4059, 0, -4080, 0, -4102, 0, -4124, 0, -4146, 0, -4167, 0, -4189, 0, -4211, 0, -4232, 0, -4254, 0, -4275, 0, -4296, 0, -4318, 0, -4339, 0, -4360, 0, -4382, 0, -4403, 0, -4424, 0, -4445, 0, -4466, 0, -4487, 0, -4508, 0, -4529, 0, -4550, 0, -4571, 0, -4592, 0, -4613, 0, -4633, 0, -4654, 0, -4675, 0, -4695, 0, -4716, 0, -4736, 0, -4757, 0, -4777, 0, -4798, 0, -4818, 0, -4838, 0, -4859, 0, -4879, 0, -4899, 0, -4919, 0, -4939, 0, -4959, 0, -4979, 0, -4999, 0, -5019, 0, -5039, 0, -5059, 0, -5078, 0, -5098, 0, -5118, 0, -5137, 0, -5157, 0, -5176, 0, -5196, 0, -5215, 0, -5235, 0, -5254, 0, -5273, 0, -5292, 0, -5311, 0, -5331, 0, -5350, 0, -5369, 0, -5388, 0, -5406, 0, -5425, 0, -5444, 0, -5463, 0, -5482, 0, -5500, 0, -5519, 0, -5537, 0, -5556, 0, -5574, 0, -5593, 0, -5611, 0, -5629, 0, -5648, 0, -5666, 0, -5684, 0, -5702, 0, -5720, 0, -5738, 0, -5756, 0, -5774, 0, -5791, 0, -5809, 0, -5827, 0, -5844, 0, -5862, 0, -5880, 0, -5897, 0, -5914, 0, -5932, 0, -5949, 0, -5966, 0, -5984, 0, -6001, 0, -6018, 0, -6035, 0, -6052, 0, -6069, 0, -6085, 0, -6102, 0, -6119, 0, -6136, 0, -6152, 0, -6169, 0, -6185, 0, -6202, 0, -6218, 0, -6235, 0, -6251, 0, -6267, 0, -6283, 0, -6299, 0, -6315, 0, -6331, 0, -6347, 0, -6363, 0, -6379, 0, -6395, 0, -6410, 0, -6426, 0, -6441, 0, -6457, 0, -6472, 0, -6488, 0, -6503, 0, -6518, 0, -6533, 0, -6549, 0, -6564, 0, -6579, 0, -6594, 0, -6608, 0, -6623, 0, -6638, 0, -6653, 0, -6667, 0, -6682, 0, -6696, 0, -6711, 0, -6725, 0, -6739, 0, -6754, 0, -6768, 0, -6782, 0, -6796, 0, -6810, 0, -6824, 0, -6838, 0, -6852, 0, -6865, 0, -6879, 0, -6893, 0, -6906, 0, -6920, 0, -6933, 0, -6946, 0, -6960, 0, -6973, 0, -6986, 0, -6999, 0, -7012, 0, -7025, 0, -7038, 0, -7051, 0, -7064, 0, -7076, 0, -7089, 0, -7101, 0, -7114, 0, -7126, 0, -7139, 0, -7151, 0, -7163, 0, -7175, 0, -7187, 0, -7199, 0, -7211, 0, -7223, 0, -7235, 0, -7247, 0, -7259, 0, -7270, 0, -7282, 0, -7293, 0, -7305, 0, -7316, 0, -7327, 0, -7338, 0, -7349, 0, -7361, 0, -7372, 0, -7382, 0, -7393, 0, -7404, 0, -7415, 0, -7425, 0, -7436, 0, -7446, 0, -7457, 0, -7467, 0, -7478, 0, -7488, 0, -7498, 0, -7508, 0, -7518, 0, -7528, 0, -7538, 0, -7548, 0, -7557, 0, -7567, 0, -7577, 0, -7586, 0, -7596, 0, -7605, 0, -7614, 0, -7623, 0, -7633, 0, -7642, 0, -7651, 0, -7660, 0, -7668, 0, -7677, 0, -7686, 0, -7695, 0, -7703, 0, -7712, 0, -7720, 0, -7728, 0, -7737, 0, -7745, 0, -7753, 0, -7761, 0, -7769, 0, -7777, 0, -7785, 0, -7793, 0, -7800, 0, -7808, 0, -7816, 0, -7823, 0, -7830, 0, -7838, 0, -7845, 0, -7852, 0, -7859, 0, -7866, 0, -7873, 0, -7880, 0, -7887, 0, -7894, 0, -7900, 0, -7907, 0, -7914, 0, -7920, 0, -7926, 0, -7933, 0, -7939, 0, -7945, 0, -7951, 0, -7957, 0, -7963, 0, -7969, 0, -7975, 0, -7980, 0, -7986, 0, -7991, 0, -7997, 0, -8002, 0, -8008, 0, -8013, 0, -8018, 0, -8023, 0, -8028, 0, -8033, 0, -8038, 0, -8043, 0, -8047, 0, -8052, 0, -8057, 0, -8061, 0, -8066, 0, -8070, 0, -8074, 0, -8078, 0, -8082, 0, -8086, 0, -8090, 0, -8094, 0, -8098, 0, -8102, 0, -8105, 0, -8109, 0, -8113, 0, -8116, 0, -8119, 0, -8123, 0, -8126, 0, -8129, 0, -8132, 0, -8135, 0, -8138, 0, -8141, 0, -8143, 0, -8146, 0, -8149, 0, -8151, 0, -8153, 0, -8156, 0, -8158, 0, -8160, 0, -8162, 0, -8164, 0, -8166, 0, -8168, 0, -8170, 0, -8172, 0, -8174, 0, -8175, 0, -8177, 0, -8178, 0, -8179, 0, -8181, 0, -8182, 0, -8183, 0, -8184, 0, -8185, 0, -8186, 0, -8187, 0, -8187, 0, -8188, 0, -8189, 0, -8189, 0, -8190, 0, -8190, 0, -8190, 0, -8190, 0, -8190, 0, -8191, 0, -8190, 0, -8190, 0, -8190, 0, -8190, 0, -8190, 0, -8189, 0, -8189, 0, -8188, 0, -8187, 0, -8187, 0, -8186, 0, -8185, 0, -8184, 0, -8183, 0, -8182, 0, -8181, 0, -8179, 0, -8178, 0, -8177, 0, -8175, 0, -8174, 0, -8172, 0, -8170, 0, -8168, 0, -8166, 0, -8164, 0, -8162, 0, -8160, 0, -8158, 0, -8156, 0, -8153, 0, -8151, 0, -8149, 0, -8146, 0, -8143, 0, -8141, 0, -8138, 0, -8135, 0, -8132, 0, -8129, 0, -8126, 0, -8123, 0, -8119, 0, -8116, 0, -8113, 0, -8109, 0, -8105, 0, -8102, 0, -8098, 0, -8094, 0, -8090, 0, -8086, 0, -8082, 0, -8078, 0, -8074, 0, -8070, 0, -8066, 0, -8061, 0, -8057, 0, -8052, 0, -8047, 0, -8043, 0, -8038, 0, -8033, 0, -8028, 0, -8023, 0, -8018, 0, -8013, 0, -8008, 0, -8002, 0, -7997, 0, -7991, 0, -7986, 0, -7980, 0, -7975, 0, -7969, 0, -7963, 0, -7957, 0, -7951, 0, -7945, 0, -7939, 0, -7933, 0, -7926, 0, -7920, 0, -7914, 0, -7907, 0, -7900, 0, -7894, 0, -7887, 0, -7880, 0, -7873, 0, -7866, 0, -7859, 0, -7852, 0, -7845, 0, -7838, 0, -7830, 0, -7823, 0, -7816, 0, -7808, 0, -7800, 0, -7793, 0, -7785, 0, -7777, 0, -7769, 0, -7761, 0, -7753, 0, -7745, 0, -7737, 0, -7728, 0, -7720, 0, -7712, 0, -7703, 0, -7695, 0, -7686, 0, -7677, 0, -7668, 0, -7660, 0, -7651, 0, -7642, 0, -7633, 0, -7623, 0, -7614, 0, -7605, 0, -7596, 0, -7586, 0, -7577, 0, -7567, 0, -7557, 0, -7548, 0, -7538, 0, -7528, 0, -7518, 0, -7508, 0, -7498, 0, -7488, 0, -7478, 0, -7467, 0, -7457, 0, -7446, 0, -7436, 0, -7425, 0, -7415, 0, -7404, 0, -7393, 0, -7382, 0, -7372, 0, -7361, 0, -7349, 0, -7338, 0, -7327, 0, -7316, 0, -7305, 0, -7293, 0, -7282, 0, -7270, 0, -7259, 0, -7247, 0, -7235, 0, -7223, 0, -7211, 0, -7199, 0, -7187, 0, -7175, 0, -7163, 0, -7151, 0, -7139, 0, -7126, 0, -7114, 0, -7101, 0, -7089, 0, -7076, 0, -7064, 0, -7051, 0, -7038, 0, -7025, 0, -7012, 0, -6999, 0, -6986, 0, -6973, 0, -6960, 0, -6946, 0, -6933, 0, -6920, 0, -6906, 0, -6893, 0, -6879, 0, -6865, 0, -6852, 0, -6838, 0, -6824, 0, -6810, 0, -6796, 0, -6782, 0, -6768, 0, -6754, 0, -6739, 0, -6725, 0, -6711, 0, -6696, 0, -6682, 0, -6667, 0, -6653, 0, -6638, 0, -6623, 0, -6608, 0, -6594, 0, -6579, 0, -6564, 0, -6549, 0, -6533, 0, -6518, 0, -6503, 0, -6488, 0, -6472, 0, -6457, 0, -6441, 0, -6426, 0, -6410, 0, -6395, 0, -6379, 0, -6363, 0, -6347, 0, -6331, 0, -6315, 0, -6299, 0, -6283, 0, -6267, 0, -6251, 0, -6235, 0, -6218, 0, -6202, 0, -6185, 0, -6169, 0, -6152, 0, -6136, 0, -6119, 0, -6102, 0, -6085, 0, -6069, 0, -6052, 0, -6035, 0, -6018, 0, -6001, 0, -5984, 0, -5966, 0, -5949, 0, -5932, 0, -5914, 0, -5897, 0, -5880, 0, -5862, 0, -5844, 0, -5827, 0, -5809, 0, -5791, 0, -5774, 0, -5756, 0, -5738, 0, -5720, 0, -5702, 0, -5684, 0, -5666, 0, -5648, 0, -5629, 0, -5611, 0, -5593, 0, -5574, 0, -5556, 0, -5537, 0, -5519, 0, -5500, 0, -5482, 0, -5463, 0, -5444, 0, -5425, 0, -5406, 0, -5388, 0, -5369, 0, -5350, 0, -5331, 0, -5311, 0, -5292, 0, -5273, 0, -5254, 0, -5235, 0, -5215, 0, -5196, 0, -5176, 0, -5157, 0, -5137, 0, -5118, 0, -5098, 0, -5078, 0, -5059, 0, -5039, 0, -5019, 0, -4999, 0, -4979, 0, -4959, 0, -4939, 0, -4919, 0, -4899, 0, -4879, 0, -4859, 0, -4838, 0, -4818, 0, -4798, 0, -4777, 0, -4757, 0, -4736, 0, -4716, 0, -4695, 0, -4675, 0, -4654, 0, -4633, 0, -4613, 0, -4592, 0, -4571, 0, -4550, 0, -4529, 0, -4508, 0, -4487, 0, -4466, 0, -4445, 0, -4424, 0, -4403, 0, -4382, 0, -4360, 0, -4339, 0, -4318, 0, -4296, 0, -4275, 0, -4254, 0, -4232, 0, -4211, 0, -4189, 0, -4167, 0, -4146, 0, -4124, 0, -4102, 0, -4080, 0, -4059, 0, -4037, 0, -4015, 0, -3993, 0, -3971, 0, -3949, 0, -3927, 0, -3905, 0, -3883, 0, -3861, 0, -3839, 0, -3816, 0, -3794, 0, -3772, 0, -3749, 0, -3727, 0, -3705, 0, -3682, 0, -3660, 0, -3637, 0, -3615, 0, -3592, 0, -3570, 0, -3547, 0, -3524, 0, -3502, 0, -3479, 0, -3456, 0, -3433, 0, -3410, 0, -3388, 0, -3365, 0, -3342, 0, -3319, 0, -3296, 0, -3273, 0, -3250, 0, -3227, 0, -3204, 0, -3180, 0, -3157, 0, -3134, 0, -3111, 0, -3088, 0, -3064, 0, -3041, 0, -3018, 0, -2994, 0, -2971, 0, -2947, 0, -2924, 0, -2900, 0, -2877, 0, -2853, 0, -2830, 0, -2806, 0, -2783, 0, -2759, 0, -2735, 0, -2712, 0, -2688, 0, -2664, 0, -2640, 0, -2617, 0, -2593, 0, -2569, 0, -2545, 0, -2521, 0, -2497, 0, -2473, 0, -2449, 0, -2425, 0, -2401, 0, -2377, 0, -2353, 0, -2329, 0, -2305, 0, -2281, 0, -2257, 0, -2233, 0, -2208, 0, -2184, 0, -2160, 0, -2136, 0, -2111, 0, -2087, 0, -2063, 0, -2038, 0, -2014, 0, -1990, 0, -1965, 0, -1941, 0, -1917, 0, -1892, 0, -1868, 0, -1843, 0, -1819, 0, -1794, 0, -1770, 0, -1745, 0, -1721, 0, -1696, 0, -1671, 0, -1647, 0, -1622, 0, -1597, 0, -1573, 0, -1548, 0, -1523, 0, -1499, 0, -1474, 0, -1449, 0, -1425, 0, -1400, 0, -1375, 0, -1350, 0, -1326, 0, -1301, 0, -1276, 0, -1251, 0, -1226, 0, -1201, 0, -1177, 0, -1152, 0, -1127, 0, -1102, 0, -1077, 0, -1052, 0, -1027, 0, -1002, 0, -977, 0, -952, 0, -927, 0, -902, 0, -877, 0, -852, 0, -827, 0, -802, 0, -777, 0, -752, 0, -727, 0, -702, 0, -677, 0, -652, 0, -627, 0, -602, 0, -577, 0, -552, 0, -527, 0, -502, 0, -477, 0, -452, 0, -427, 0, -401, 0, -376, 0, -351, 0, -326, 0, -301, 0, -276, 0, -251, 0, -226, 0, -201, 0, -175, 0, -150, 0, -125, 0, -100, 0, -75, 0, -50, 0, -25, 0, 0, 0, 25, 0, 50, 0, 75, 0, 100, 0, 125, 0, 150, 0, 175, 0, 201, 0, 226, 0, 251, 0, 276, 0, 301, 0, 326, 0, 351, 0, 376, 0, 401, 0, 427, 0, 452, 0, 477, 0, 502, 0, 527, 0, 552, 0, 577, 0, 602, 0, 627, 0, 652, 0, 677, 0, 702, 0, 727, 0, 752, 0, 777, 0, 802, 0, 827, 0, 852, 0, 877, 0, 902, 0, 927, 0, 952, 0, 977, 0, 1002, 0, 1027, 0, 1052, 0, 1077, 0, 1102, 0, 1127, 0, 1152, 0, 1177, 0, 1201, 0, 1226, 0, 1251, 0, 1276, 0, 1301, 0, 1326, 0, 1350, 0, 1375, 0, 1400, 0, 1425, 0, 1449, 0, 1474, 0, 1499, 0, 1523, 0, 1548, 0, 1573, 0, 1597, 0, 1622, 0, 1647, 0, 1671, 0, 1696, 0, 1721, 0, 1745, 0, 1770, 0, 1794, 0, 1819, 0, 1843, 0, 1868, 0, 1892, 0, 1917, 0, 1941, 0, 1965, 0, 1990, 0, 2014, 0, 2038, 0, 2063, 0, 2087, 0, 2111, 0, 2136, 0, 2160, 0, 2184, 0, 2208, 0, 2233, 0, 2257, 0, 2281, 0, 2305, 0, 2329, 0, 2353, 0, 2377, 0, 2401, 0, 2425, 0, 2449, 0, 2473, 0, 2497, 0, 2521, 0, 2545, 0, 2569, 0, 2593, 0, 2617, 0, 2640, 0, 2664, 0, 2688, 0, 2712, 0, 2735, 0, 2759, 0, 2783, 0, 2806, 0, 2830, 0, 2853, 0, 2877, 0, 2900, 0, 2924, 0, 2947, 0, 2971, 0, 2994, 0, 3018, 0, 3041, 0, 3064, 0, 3088, 0, 3111, 0, 3134, 0, 3157, 0, 3180, 0, 3204, 0, 3227, 0, 3250, 0, 3273, 0, 3296, 0, 3319, 0, 3342, 0, 3365, 0, 3388, 0, 3410, 0, 3433, 0, 3456, 0, 3479, 0, 3502, 0, 3524, 0, 3547, 0, 3570, 0, 3592, 0, 3615, 0, 3637, 0, 3660, 0, 3682, 0, 3705, 0, 3727, 0, 3749, 0, 3772, 0, 3794, 0, 3816, 0, 3839, 0, 3861, 0, 3883, 0, 3905, 0, 3927, 0, 3949, 0, 3971, 0, 3993, 0, 4015, 0, 4037, 0, 4059, 0, 4080, 0, 4102, 0, 4124, 0, 4146, 0, 4167, 0, 4189, 0, 4211, 0, 4232, 0, 4254, 0, 4275, 0, 4296, 0, 4318, 0, 4339, 0, 4360, 0, 4382, 0, 4403, 0, 4424, 0, 4445, 0, 4466, 0, 4487, 0, 4508, 0, 4529, 0, 4550, 0, 4571, 0, 4592, 0, 4613, 0, 4633, 0, 4654, 0, 4675, 0, 4695, 0, 4716, 0, 4736, 0, 4757, 0, 4777, 0, 4798, 0, 4818, 0, 4838, 0, 4859, 0, 4879, 0, 4899, 0, 4919, 0, 4939, 0, 4959, 0, 4979, 0, 4999, 0, 5019, 0, 5039, 0, 5059, 0, 5078, 0, 5098, 0, 5118, 0, 5137, 0, 5157, 0, 5176, 0, 5196, 0, 5215, 0, 5235, 0, 5254, 0, 5273, 0, 5292, 0, 5311, 0, 5331, 0, 5350, 0, 5369, 0, 5388, 0, 5406, 0, 5425, 0, 5444, 0, 5463, 0, 5482, 0, 5500, 0, 5519, 0, 5537, 0, 5556, 0, 5574, 0, 5593, 0, 5611, 0, 5629, 0, 5648, 0, 5666, 0, 5684, 0, 5702, 0, 5720, 0, 5738, 0, 5756, 0, 5774, 0, 5791, 0, 5809, 0, 5827, 0, 5844, 0, 5862, 0, 5880, 0, 5897, 0, 5914, 0, 5932, 0, 5949, 0, 5966, 0, 5984, 0, 6001, 0, 6018, 0, 6035, 0, 6052, 0, 6069, 0, 6085, 0, 6102, 0, 6119, 0, 6136, 0, 6152, 0, 6169, 0, 6185, 0, 6202, 0, 6218, 0, 6235, 0, 6251, 0, 6267, 0, 6283, 0, 6299, 0, 6315, 0, 6331, 0, 6347, 0, 6363, 0, 6379, 0, 6395, 0, 6410, 0, 6426, 0, 6441, 0, 6457, 0, 6472, 0, 6488, 0, 6503, 0, 6518, 0, 6533, 0, 6549, 0, 6564, 0, 6579, 0, 6594, 0, 6608, 0, 6623, 0, 6638, 0, 6653, 0, 6667, 0, 6682, 0, 6696, 0, 6711, 0, 6725, 0, 6739, 0, 6754, 0, 6768, 0, 6782, 0, 6796, 0, 6810, 0, 6824, 0, 6838, 0, 6852, 0, 6865, 0, 6879, 0, 6893, 0, 6906, 0, 6920, 0, 6933, 0, 6946, 0, 6960, 0, 6973, 0, 6986, 0, 6999, 0, 7012, 0, 7025, 0, 7038, 0, 7051, 0, 7064, 0, 7076, 0, 7089, 0, 7101, 0, 7114, 0, 7126, 0, 7139, 0, 7151, 0, 7163, 0, 7175, 0, 7187, 0, 7199, 0, 7211, 0, 7223, 0, 7235, 0, 7247, 0, 7259, 0, 7270, 0, 7282, 0, 7293, 0, 7305, 0, 7316, 0, 7327, 0, 7338, 0, 7349, 0, 7361, 0, 7372, 0, 7382, 0, 7393, 0, 7404, 0, 7415, 0, 7425, 0, 7436, 0, 7446, 0, 7457, 0, 7467, 0, 7478, 0, 7488, 0, 7498, 0, 7508, 0, 7518, 0, 7528, 0, 7538, 0, 7548, 0, 7557, 0, 7567, 0, 7577, 0, 7586, 0, 7596, 0, 7605, 0, 7614, 0, 7623, 0, 7633, 0, 7642, 0, 7651, 0, 7660, 0, 7668, 0, 7677, 0, 7686, 0, 7695, 0, 7703, 0, 7712, 0, 7720, 0, 7728, 0, 7737, 0, 7745, 0, 7753, 0, 7761, 0, 7769, 0, 7777, 0, 7785, 0, 7793, 0, 7800, 0, 7808, 0, 7816, 0, 7823, 0, 7830, 0, 7838, 0, 7845, 0, 7852, 0, 7859, 0, 7866, 0, 7873, 0, 7880, 0, 7887, 0, 7894, 0, 7900, 0, 7907, 0, 7914, 0, 7920, 0, 7926, 0, 7933, 0, 7939, 0, 7945, 0, 7951, 0, 7957, 0, 7963, 0, 7969, 0, 7975, 0, 7980, 0, 7986, 0, 7991, 0, 7997, 0, 8002, 0, 8008, 0, 8013, 0, 8018, 0, 8023, 0, 8028, 0, 8033, 0, 8038, 0, 8043, 0, 8047, 0, 8052, 0, 8057, 0, 8061, 0, 8066, 0, 8070, 0, 8074, 0, 8078, 0, 8082, 0, 8086, 0, 8090, 0, 8094, 0, 8098, 0, 8102, 0, 8105, 0, 8109, 0, 8113, 0, 8116, 0, 8119, 0, 8123, 0, 8126, 0, 8129, 0, 8132, 0, 8135, 0, 8138, 0, 8141, 0, 8143, 0, 8146, 0, 8149, 0, 8151, 0, 8153, 0, 8156, 0, 8158, 0, 8160, 0, 8162, 0, 8164, 0, 8166, 0, 8168, 0, 8170, 0, 8172, 0, 8174, 0, 8175, 0, 8177, 0, 8178, 0, 8179, 0, 8181, 0, 8182, 0, 8183, 0, 8184, 0, 8185, 0, 8186, 0, 8187, 0, 8187, 0, 8188, 0, 8189, 0, 8189, 0, 8190, 0, 8190, 0, 8190, 0, 8190, 0, 8190, 0, 8191, 0, 8190, 0, 8190, 0, 8190, 0, 8190, 0, 8190, 0, 8189, 0, 8189, 0, 8188, 0, 8187, 0, 8187, 0, 8186, 0, 8185, 0, 8184, 0, 8183, 0, 8182, 0, 8181, 0, 8179, 0, 8178, 0, 8177, 0, 8175, 0, 8174, 0, 8172, 0, 8170, 0, 8168, 0, 8166, 0, 8164, 0, 8162, 0, 8160, 0, 8158, 0, 8156, 0, 8153, 0, 8151, 0, 8149, 0, 8146, 0, 8143, 0, 8141, 0, 8138, 0, 8135, 0, 8132, 0, 8129, 0, 8126, 0, 8123, 0, 8119, 0, 8116, 0, 8113, 0, 8109, 0, 8105, 0, 8102, 0, 8098, 0, 8094, 0, 8090, 0, 8086, 0, 8082, 0, 8078, 0, 8074, 0, 8070, 0, 8066, 0, 8061, 0, 8057, 0, 8052, 0, 8047, 0, 8043, 0, 8038, 0, 8033, 0, 8028, 0, 8023, 0, 8018, 0, 8013, 0, 8008, 0, 8002, 0, 7997, 0, 7991, 0, 7986, 0, 7980, 0, 7975, 0, 7969, 0, 7963, 0, 7957, 0, 7951, 0, 7945, 0, 7939, 0, 7933, 0, 7926, 0, 7920, 0, 7914, 0, 7907, 0, 7900, 0, 7894, 0, 7887, 0, 7880, 0, 7873, 0, 7866, 0, 7859, 0, 7852, 0, 7845, 0, 7838, 0, 7830, 0, 7823, 0, 7816, 0, 7808, 0, 7800, 0, 7793, 0, 7785, 0, 7777, 0, 7769, 0, 7761, 0, 7753, 0, 7745, 0, 7737, 0, 7728, 0, 7720, 0, 7712, 0, 7703, 0, 7695, 0, 7686, 0, 7677, 0, 7668, 0, 7660, 0, 7651, 0, 7642, 0, 7633, 0, 7623, 0, 7614, 0, 7605, 0, 7596, 0, 7586, 0, 7577, 0, 7567, 0, 7557, 0, 7548, 0, 7538, 0, 7528, 0, 7518, 0, 7508, 0, 7498, 0, 7488, 0, 7478, 0, 7467, 0, 7457, 0, 7446, 0, 7436, 0, 7425, 0, 7415, 0, 7404, 0, 7393, 0, 7382, 0, 7372, 0, 7361, 0, 7349, 0, 7338, 0, 7327, 0, 7316, 0, 7305, 0, 7293, 0, 7282, 0, 7270, 0, 7259, 0, 7247, 0, 7235, 0, 7223, 0, 7211, 0, 7199, 0, 7187, 0, 7175, 0, 7163, 0, 7151, 0, 7139, 0, 7126, 0, 7114, 0, 7101, 0, 7089, 0, 7076, 0, 7064, 0, 7051, 0, 7038, 0, 7025, 0, 7012, 0, 6999, 0, 6986, 0, 6973, 0, 6960, 0, 6946, 0, 6933, 0, 6920, 0, 6906, 0, 6893, 0, 6879, 0, 6865, 0, 6852, 0, 6838, 0, 6824, 0, 6810, 0, 6796, 0, 6782, 0, 6768, 0, 6754, 0, 6739, 0, 6725, 0, 6711, 0, 6696, 0, 6682, 0, 6667, 0, 6653, 0, 6638, 0, 6623, 0, 6608, 0, 6594, 0, 6579, 0, 6564, 0, 6549, 0, 6533, 0, 6518, 0, 6503, 0, 6488, 0, 6472, 0, 6457, 0, 6441, 0, 6426, 0, 6410, 0, 6395, 0, 6379, 0, 6363, 0, 6347, 0, 6331, 0, 6315, 0, 6299, 0, 6283, 0, 6267, 0, 6251, 0, 6235, 0, 6218, 0, 6202, 0, 6185, 0, 6169, 0, 6152, 0, 6136, 0, 6119, 0, 6102, 0, 6085, 0, 6069, 0, 6052, 0, 6035, 0, 6018, 0, 6001, 0, 5984, 0, 5966, 0, 5949, 0, 5932, 0, 5914, 0, 5897, 0, 5880, 0, 5862, 0, 5844, 0, 5827, 0, 5809, 0, 5791, 0, 5774, 0, 5756, 0, 5738, 0, 5720, 0, 5702, 0, 5684, 0, 5666, 0, 5648, 0, 5629, 0, 5611, 0, 5593, 0, 5574, 0, 5556, 0, 5537, 0, 5519, 0, 5500, 0, 5482, 0, 5463, 0, 5444, 0, 5425, 0, 5406, 0, 5388, 0, 5369, 0, 5350, 0, 5331, 0, 5311, 0, 5292, 0, 5273, 0, 5254, 0, 5235, 0, 5215, 0, 5196, 0, 5176, 0, 5157, 0, 5137, 0, 5118, 0, 5098, 0, 5078, 0, 5059, 0, 5039, 0, 5019, 0, 4999, 0, 4979, 0, 4959, 0, 4939, 0, 4919, 0, 4899, 0, 4879, 0, 4859, 0, 4838, 0, 4818, 0, 4798, 0, 4777, 0, 4757, 0, 4736, 0, 4716, 0, 4695, 0, 4675, 0, 4654, 0, 4633, 0, 4613, 0, 4592, 0, 4571, 0, 4550, 0, 4529, 0, 4508, 0, 4487, 0, 4466, 0, 4445, 0, 4424, 0, 4403, 0, 4382, 0, 4360, 0, 4339, 0, 4318, 0, 4296, 0, 4275, 0, 4254, 0, 4232, 0, 4211, 0, 4189, 0, 4167, 0, 4146, 0, 4124, 0, 4102, 0, 4080, 0, 4059, 0, 4037, 0, 4015, 0, 3993, 0, 3971, 0, 3949, 0, 3927, 0, 3905, 0, 3883, 0, 3861, 0, 3839, 0, 3816, 0, 3794, 0, 3772, 0, 3749, 0, 3727, 0, 3705, 0, 3682, 0, 3660, 0, 3637, 0, 3615, 0, 3592, 0, 3570, 0, 3547, 0, 3524, 0, 3502, 0, 3479, 0, 3456, 0, 3433, 0, 3410, 0, 3388, 0, 3365, 0, 3342, 0, 3319, 0, 3296, 0, 3273, 0, 3250, 0, 3227, 0, 3204, 0, 3180, 0, 3157, 0, 3134, 0, 3111, 0, 3088, 0, 3064, 0, 3041, 0, 3018, 0, 2994, 0, 2971, 0, 2947, 0, 2924, 0, 2900, 0, 2877, 0, 2853, 0, 2830, 0, 2806, 0, 2783, 0, 2759, 0, 2735, 0, 2712, 0, 2688, 0, 2664, 0, 2640, 0, 2617, 0, 2593, 0, 2569, 0, 2545, 0, 2521, 0, 2497, 0, 2473, 0, 2449, 0, 2425, 0, 2401, 0, 2377, 0, 2353, 0, 2329, 0, 2305, 0, 2281, 0, 2257, 0, 2233, 0, 2208, 0, 2184, 0, 2160, 0, 2136, 0, 2111, 0, 2087, 0, 2063, 0, 2038, 0, 2014, 0, 1990, 0, 1965, 0, 1941, 0, 1917, 0, 1892, 0, 1868, 0, 1843, 0, 1819, 0, 1794, 0, 1770, 0, 1745, 0, 1721, 0, 1696, 0, 1671, 0, 1647, 0, 1622, 0, 1597, 0, 1573, 0, 1548, 0, 1523, 0, 1499, 0, 1474, 0, 1449, 0, 1425, 0, 1400, 0, 1375, 0, 1350, 0, 1326, 0, 1301, 0, 1276, 0, 1251, 0, 1226, 0, 1201, 0, 1177, 0, 1152, 0, 1127, 0, 1102, 0, 1077, 0, 1052, 0, 1027, 0, 1002, 0, 977, 0, 952, 0, 927, 0, 902, 0, 877, 0, 852, 0, 827, 0, 802, 0, 777, 0, 752, 0, 727, 0, 702, 0, 677, 0, 652, 0, 627, 0, 602, 0, 577, 0, 552, 0, 527, 0, 502, 0, 477, 0, 452, 0, 427, 0, 401, 0, 376, 0, 351, 0, 326, 0, 301, 0, 276, 0, 251, 0, 226, 0, 201, 0, 175, 0, 150, 0, 125, 0, 100, 0, 75, 0, 50, 0, 25, 0], ["i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0], ALLOC_STATIC);
_wvoice=allocate(1, "%struct.voice_t*", ALLOC_STATIC);
_f_log=allocate(1, "%struct.__sFILE*", ALLOC_STATIC);
_option_waveout=allocate(1, "i32", ALLOC_STATIC);
_option_log_frames=allocate(1, "i32", ALLOC_STATIC);
_embedded_value=allocate(60, "i32", ALLOC_STATIC);
_samplerate=allocate(1, "i32", ALLOC_STATIC);
_samplerate_native=allocate(1, "i32", ALLOC_STATIC);
__ZL5peaks=allocate(720, ["i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"double",0,0,0,0,0,0,0,"double",0,0,0,0,0,0,0,"double",0,0,0,0,0,0,0,"double",0,0,0,0,0,0,0,"double",0,0,0,0,0,0,0,"double",0,0,0,0,0,0,0,"double",0,0,0,0,0,0,0,"double",0,0,0,0,0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"double",0,0,0,0,0,0,0,"double",0,0,0,0,0,0,0,"double",0,0,0,0,0,0,0,"double",0,0,0,0,0,0,0,"double",0,0,0,0,0,0,0,"double",0,0,0,0,0,0,0,"double",0,0,0,0,0,0,0,"double",0,0,0,0,0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"double",0,0,0,0,0,0,0,"double",0,0,0,0,0,0,0,"double",0,0,0,0,0,0,0,"double",0,0,0,0,0,0,0,"double",0,0,0,0,0,0,0,"double",0,0,0,0,0,0,0,"double",0,0,0,0,0,0,0,"double",0,0,0,0,0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"double",0,0,0,0,0,0,0,"double",0,0,0,0,0,0,0,"double",0,0,0,0,0,0,0,"double",0,0,0,0,0,0,0,"double",0,0,0,0,0,0,0,"double",0,0,0,0,0,0,0,"double",0,0,0,0,0,0,0,"double",0,0,0,0,0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"double",0,0,0,0,0,0,0,"double",0,0,0,0,0,0,0,"double",0,0,0,0,0,0,0,"double",0,0,0,0,0,0,0,"double",0,0,0,0,0,0,0,"double",0,0,0,0,0,0,0,"double",0,0,0,0,0,0,0,"double",0,0,0,0,0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"double",0,0,0,0,0,0,0,"double",0,0,0,0,0,0,0,"double",0,0,0,0,0,0,0,"double",0,0,0,0,0,0,0,"double",0,0,0,0,0,0,0,"double",0,0,0,0,0,0,0,"double",0,0,0,0,0,0,0,"double",0,0,0,0,0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"double",0,0,0,0,0,0,0,"double",0,0,0,0,0,0,0,"double",0,0,0,0,0,0,0,"double",0,0,0,0,0,0,0,"double",0,0,0,0,0,0,0,"double",0,0,0,0,0,0,0,"double",0,0,0,0,0,0,0,"double",0,0,0,0,0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"double",0,0,0,0,0,0,0,"double",0,0,0,0,0,0,0,"double",0,0,0,0,0,0,0,"double",0,0,0,0,0,0,0,"double",0,0,0,0,0,0,0,"double",0,0,0,0,0,0,0,"double",0,0,0,0,0,0,0,"double",0,0,0,0,0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"double",0,0,0,0,0,0,0,"double",0,0,0,0,0,0,0,"double",0,0,0,0,0,0,0,"double",0,0,0,0,0,0,0,"double",0,0,0,0,0,0,0,"double",0,0,0,0,0,0,0,"double",0,0,0,0,0,0,0,"double",0,0,0,0,0,0,0], ALLOC_STATIC);
_echo_head=allocate(1, "i32", ALLOC_STATIC);
_echo_tail=allocate(1, "i32", ALLOC_STATIC);
_echo_amp=allocate(1, "i32", ALLOC_STATIC);
_echo_buf=allocate(11000, "i16", ALLOC_STATIC);
__ZL7rbreath=allocate(360, "double", ALLOC_STATIC);
_wdata=allocate(64, ["i8*",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i8*",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0], ALLOC_STATIC);
__ZL13amplitude_env=allocate(1, "i8*", ALLOC_STATIC);
_out_ptr=allocate(1, "i8*", ALLOC_STATIC);
_out_start=allocate(1, "i8*", ALLOC_STATIC);
_out_end=allocate(1, "i8*", ALLOC_STATIC);
_outbuf_size=allocate(1, "i32", ALLOC_STATIC);
_wcmdq=allocate(2560, "i32", ALLOC_STATIC);
_wcmdq_head=allocate(1, "i32", ALLOC_STATIC);
_wcmdq_tail=allocate(1, "i32", ALLOC_STATIC);
_embedded_default=allocate([0, 0, 0, 0, 50, 0, 0, 0, 175, 0, 0, 0, 100, 0, 0, 0, 50, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 175, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0], ["i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0], ALLOC_STATIC);
_current_source_index=allocate(1, "i32", ALLOC_STATIC);
__ZL18sonicSpeedupStream=allocate(1, "%struct.sonicStreamStruct*", ALLOC_STATIC);
_sonicSpeed=allocate([1], ["double",0,0,0,0,0,0,0,0], ALLOC_STATIC);
_pitch_adjust_tab=allocate([64,65,66,67,68,69,70,71,72,73,74,75,76,77,78,79,80,81,82,83,84,86,87,88,89,91,92,93,94,96,97,98,100,101,103,104,105,107,108,110,111,113,115,116,118,119,121,123,124,126,128,130,132,133,135,137,139,141,143,145,147,149,151,153,155,158,160,162,164,167,169,171,174,176,179,181,184,186,189,191,194,197,199,202,205,208,211,214,217,220,223,226,229,232,236,239,242,246,249,252,254,255] /* @ABCDEFGHIJKLMNOPQRS */, "i8", ALLOC_STATIC);
_pk_shape1=allocate([255,254,254,254,254,254,253,253,252,251,251,250,249,248,247,246,245,244,242,241,239,238,236,234,233,231,229,227,225,223,220,218,216,213,211,209,207,205,203,201,199,197,195,193,191,189,187,185,183,180,178,176,173,171,169,166,164,161,159,156,154,151,148,146,143,140,138,135,132,129,126,123,120,118,115,112,108,105,102,99,96,95,93,91,90,88,86,85,83,82,80,79,77,76,74,73,72,70,69,68,67,66,64,63,62,61,60,59,58,57,56,55,55,54,53,52,52,51,50,50,49,48,48,47,47,46,46,46,45,45,45,44,44,44,44,44,44,44,43,43,43,43,44,43,42,42,41,40,40,39,38,38,37,36,36,35,35,34,33,33,32,32,31,30,30,29,29,28,28,27,26,26,25,25,24,24,23,23,22,22,21,21,20,20,19,19,18,18,18,17,17,16,16,15,15,15,14,14,13,13,13,12,12,11,11,11,10,10,10,9,9,9,8,8,8,7,7,7,7,6,6,6,5,5,5,5,4,4,4,4,4,3,3,3,3,2,2,2,2,2,2,1,1,1,1,1,1,0,0,0,0,0,0,0,0,0,0] /* \FF\FE\FE\FE\FE\FE\F */, "i8", ALLOC_STATIC);
__ZL16PHASE_INC_FACTOR=allocate(1, "i32", ALLOC_STATIC);
__ZL11Flutter_inc=allocate(1, "i32", ALLOC_STATIC);
__ZL11samplecount=allocate(1, "i32", ALLOC_STATIC);
__ZL8nsamples=allocate(1, "i32", ALLOC_STATIC);
__ZL9wavephase=allocate(1, "i32", ALLOC_STATIC);
__ZL12wavemult_max=allocate(1, "i32", ALLOC_STATIC);
__ZL15wavemult_offset=allocate(1, "i32", ALLOC_STATIC);
__ZL8wavemult=allocate([0,0,0,2,3,5,8,11,14,18,22,27,32,37,43,49,55,62,69,76,83,90,98,105,113,121,128,136,144,152,159,166,174,181,188,194,201,207,213,218,224,228,233,237,240,244,246,249,251,252,253,253,253,253,252,251,249,246,244,240,237,233,228,224,218,213,207,201,194,188,181,174,166,159,152,144,136,128,121,113,105,98,90,83,76,69,62,55,49,43,37,32,27,22,18,14,11,8,5,3,2,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0] /* \00\00\00\02\03\05\0 */, "i8", ALLOC_STATIC);
__ZL9pk_shape2=allocate([255,254,254,254,254,254,254,254,254,254,253,253,253,253,252,252,252,251,251,251,250,250,249,249,248,248,247,247,246,245,245,244,243,243,242,241,239,237,235,233,231,229,227,225,223,221,218,216,213,211,208,205,203,200,197,194,191,187,184,181,178,174,171,167,163,160,156,152,148,144,140,136,132,127,123,119,114,110,105,100,96,94,91,88,86,83,81,78,76,74,71,69,66,64,62,60,57,55,53,51,49,47,44,42,40,38,36,34,32,30,29,27,25,23,21,19,18,16,14,12,11,9,7,6,4,3,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0] /* \FF\FE\FE\FE\FE\FE\F */, "i8", ALLOC_STATIC);
__ZL8pk_shape=allocate(1, "i8*", ALLOC_STATIC);
__ZZ12GetAmplitudevE12amp_emphasis=allocate([16,16,10,16,22] /* \10\10\0A\10\16 */, "i8", ALLOC_STATIC);
__ZL17general_amplitude=allocate([60], ["i32",0,0,0,0], ALLOC_STATIC);
__ZL11peak_height=allocate(36, "i32", ALLOC_STATIC);
__ZL13peak_harmonic=allocate(36, "i32", ALLOC_STATIC);
__ZL16option_harmonic1_b=allocate(1, "i1", ALLOC_STATIC);
__ZL9harmspect=allocate(1, "i32*", ALLOC_STATIC);
__ZL8harm_inc=allocate(120, "i32", ALLOC_STATIC);
__ZL10minus_pi_t=allocate(1, "double", ALLOC_STATIC);
__ZL8two_pi_t=allocate(1, "double", ALLOC_STATIC);
__ZZ7WavegenvE4maxh=allocate(1, "i32", ALLOC_STATIC);
__ZZ7WavegenvE5maxh2=allocate(1, "i32", ALLOC_STATIC);
__ZZ7WavegenvE3agc=allocate([256], ["i32",0,0,0,0], ALLOC_STATIC);
__ZZ7WavegenvE13h_switch_sign=allocate(1, "i32", ALLOC_STATIC);
__ZZ7WavegenvE11cycle_count=allocate(1, "i32", ALLOC_STATIC);
__ZZ7WavegenvE10amplitude2=allocate(1, "i32", ALLOC_STATIC);
__ZL8end_wave_b=allocate(1, "i1", ALLOC_STATIC);
__ZL7hswitch=allocate(1, "i32", ALLOC_STATIC);
__ZL6hspect=allocate(3200, "i32", ALLOC_STATIC);
__ZL8phaseinc=allocate(1, "i32", ALLOC_STATIC);
__ZL13cycle_samples=allocate(1, "i32", ALLOC_STATIC);
__ZL9hf_factor=allocate(1, "i32", ALLOC_STATIC);
__ZL6cbytes=allocate(1, "i32", ALLOC_STATIC);
__ZL12glottal_flag=allocate(1, "i32", ALLOC_STATIC);
__ZL14glottal_reduce=allocate(1, "i32", ALLOC_STATIC);
__ZL6amp_ix=allocate(1, "i32", ALLOC_STATIC);
__ZL15modulation_type=allocate(1, "i32", ALLOC_STATIC);
__ZL14modulation_tab=allocate([0, 0, 0, 0, 0, 70, 242, 41, 0, 47, 0, 47, 0, 69, 242, 41, 0, 47, 0, 46, 0, 69, 242, 40, 0, 46, 0, 45, 0, 52, 242, 40, 0, 45, 45, 44, 0, 52, 242, 40, 0, 43, 43, 43, 0, 52, 242, 40, 0, 42, 42, 42, 0, 52, 242, 40, 0, 41, 41, 41, 0, 52, 242, 40], "i8", ALLOC_STATIC);
__ZL7voicing=allocate(1, "i32", ALLOC_STATIC);
__ZL12embedded_max=allocate([0, 0, 0, 0, 32767, 0, 0, 0, 750, 0, 0, 0, 300, 0, 0, 0, 99, 0, 0, 0, 99, 0, 0, 0, 99, 0, 0, 0, 0, 0, 0, 0, 750, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 4, 0, 0, 0, 0, 0, 0, 0], ["i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0], ALLOC_STATIC);
__ZZ15WavegenSetVoiceP7voice_tE2v2=allocate(1344, ["i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8"], ALLOC_STATIC);
__ZL13consonant_amp=allocate([26], ["i32",0,0,0,0], ALLOC_STATIC);
__ZL11flutter_amp=allocate([64], ["i32",0,0,0,0], ALLOC_STATIC);
__ZZ8SetSynthiiP7frame_tS0_P7voice_tE19glottal_reduce_tab1=allocate([48, 0, 0, 0, 48, 0, 0, 0, 64, 0, 0, 0, 80, 0, 0, 0], ["i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0], ALLOC_STATIC);
__ZZ8SetSynthiiP7frame_tS0_P7voice_tE19glottal_reduce_tab2=allocate([144, 0, 0, 0, 160, 0, 0, 0, 176, 0, 0, 0, 192, 0, 0, 0], ["i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0], ALLOC_STATIC);
__ZL17samplecount_start=allocate(1, "i32", ALLOC_STATIC);
__ZZ12WavegenFill2iE6resume=allocate(1, "i32", ALLOC_STATIC);
__ZZ12WavegenFill2iE13echo_complete=allocate(1, "i32", ALLOC_STATIC);
__ZL11echo_length=allocate(1, "i32", ALLOC_STATIC);
__ZL7amp_inc=allocate(1, "i32", ALLOC_STATIC);
__ZZL8PlayWaveiiPhiiE9n_samples=allocate(1, "i32", ALLOC_STATIC);
__ZZL8PlayWaveiiPhiiE2ix=allocate(1, "i32", ALLOC_STATIC);
__ZZL11PlaySilenceiiE9n_samples=allocate(1, "i32", ALLOC_STATIC);
__ZZL17AdvanceParametersvE10Flutter_ix=allocate(1, "i32", ALLOC_STATIC);
__ZL11Flutter_tab=allocate([128,155,181,203,220,232,237,236,230,220,206,191,176,163,152,144,140,139,140,143,146,148,149,146,140,131,120,105,89,73,60,49,42,41,45,54,68,86,105,125,143,159,170,177,178,173,164,150,135,120,105,92,83,79,79,85,94,107,122,136,150,162,171,176,177,174,168,160,152,145,139,136,137,141,148,157,168,178,187,192,193,189,180,165,146,124,99,74,50,30,14,5,2,5,15,30,48,68,89,109,127,140,150,156,159,159,157,155,153,153,156,161,169,179,191,202,213,220,224,222,216,204,187,166,143,119,96,75,58,46,40,41,47,58,72,89,106,122,134,144,148,149,145,137,128,117,107,98,92,90,92,97,105,116,128,138,148,154,158,157,152,144,134,124,113,104,98,96,99,107,120,136,155,175,194,210,223,230,231,226,215,198,178,156,132,111,91,75,64,57,55,56,61,67,74,80,84,86,85,82,77,72,66,63,62,65,73,86,103,124,147,171,195,217,234,246,252,251,244,231,213,192,170,148,128,113,100,93,90,92,97,104,112,119,125,127,127,123,116,107,97,87,78,72,70,72,78,89,102,117,132,147,159,167,171,170,164,153,139,123,106,91,78,70,67,69,77,90,107,127,146,166,184,197,207,211,210,205,196,185,173,161,150,142,137,135,135,138,141,145,146,145,140,132,120,104,85,65,46,28,14,5,1,5,15,31,52,77,104,129,154,176,193,205,211,211,208,200,191,181,171,164,159,156,157,160,165,170,174,177,176,171,163,150,135,118,99,81,66,54,47,45,49,58,72,89,107,126,142,156,166,170,169,163,152,138,123,108,93,82,74,72,74,80,90,103,117,130] /* \80\9B\B5\CB\DC\E8\E */, "i8", ALLOC_STATIC);
_pause_phonemes=allocate([0,23,10,9,24,19,24,24] /* \00\17\0A\09\18\13\1 */, "i8", ALLOC_STATIC);
__ZZ15MakePhonemeListP10TranslatoriiE12types_double=allocate([6,7,8,3,0] /* \06\07\08\03\00 */, "i8", ALLOC_STATIC);
__ZL8kt_frame=allocate(288, "i32", ALLOC_STATIC);
__ZL10kt_globals=allocate(2748, ["i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"double",0,0,0,0,0,0,0,"double",0,0,0,0,0,0,0,"double",0,0,0,0,0,0,0,"double",0,0,0,0,0,0,0,"double",0,0,0,0,0,0,0,"double",0,0,0,0,0,0,0,"double",0,0,0,0,0,0,0,"double",0,0,0,0,0,0,0,"double",0,0,0,0,0,0,0,"double",0,0,0,0,0,0,0,"double",0,0,0,0,0,0,0,"double",0,0,0,0,0,0,0,"double",0,0,0,0,0,0,0,"i32",0,0,0,"double",0,0,0,0,0,0,0,"i16*",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"double",0,0,0,0,0,0,0,"double",0,0,0,0,0,0,0,"double",0,0,0,0,0,0,0,"double",0,0,0,0,0,0,0,"double",0,0,0,0,0,0,0,"double",0,0,0,0,0,0,0,"double",0,0,0,0,0,0,0,"double",0,0,0,0,0,0,0,"double",0,0,0,0,0,0,0,"double",0,0,0,0,0,0,0,"double",0,0,0,0,0,0,0,"double",0,0,0,0,0,0,0,"double",0,0,0,0,0,0,0,"double",0,0,0,0,0,0,0,"double",0,0,0,0,0,0,0,"double",0,0,0,0,0,0,0,"double",0,0,0,0,0,0,0,"double",0,0,0,0,0,0,0,"double",0,0,0,0,0,0,0,"double",0,0,0,0,0,0,0,"double",0,0,0,0,0,0,0,"double",0,0,0,0,0,0,0,"double",0,0,0,0,0,0,0,"double",0,0,0,0,0,0,0,"double",0,0,0,0,0,0,0,"double",0,0,0,0,0,0,0,"double",0,0,0,0,0,0,0,"double",0,0,0,0,0,0,0,"double",0,0,0,0,0,0,0,"double",0,0,0,0,0,0,0,"double",0,0,0,0,0,0,0,"double",0,0,0,0,0,0,0,"double",0,0,0,0,0,0,0,"double",0,0,0,0,0,0,0,"double",0,0,0,0,0,0,0,"double",0,0,0,0,0,0,0,"double",0,0,0,0,0,0,0,"double",0,0,0,0,0,0,0,"double",0,0,0,0,0,0,0,"double",0,0,0,0,0,0,0,"double",0,0,0,0,0,0,0,"double",0,0,0,0,0,0,0,"double",0,0,0,0,0,0,0,"double",0,0,0,0,0,0,0,"double",0,0,0,0,0,0,0,"double",0,0,0,0,0,0,0,"double",0,0,0,0,0,0,0,"double",0,0,0,0,0,0,0,"double",0,0,0,0,0,0,0,"double",0,0,0,0,0,0,0,"double",0,0,0,0,0,0,0,"double",0,0,0,0,0,0,0,"double",0,0,0,0,0,0,0,"double",0,0,0,0,0,0,0,"double",0,0,0,0,0,0,0,"double",0,0,0,0,0,0,0,"double",0,0,0,0,0,0,0,"double",0,0,0,0,0,0,0,"double",0,0,0,0,0,0,0,"double",0,0,0,0,0,0,0,"double",0,0,0,0,0,0,0,"double",0,0,0,0,0,0,0,"double",0,0,0,0,0,0,0,"double",0,0,0,0,0,0,0,"double",0,0,0,0,0,0,0,"double",0,0,0,0,0,0,0,"double",0,0,0,0,0,0,0,"double",0,0,0,0,0,0,0,"double",0,0,0,0,0,0,0,"double",0,0,0,0,0,0,0,"double",0,0,0,0,0,0,0,"double",0,0,0,0,0,0,0,"double",0,0,0,0,0,0,0,"double",0,0,0,0,0,0,0,"double",0,0,0,0,0,0,0,"double",0,0,0,0,0,0,0,"double",0,0,0,0,0,0,0,"double",0,0,0,0,0,0,0,"double",0,0,0,0,0,0,0,"double",0,0,0,0,0,0,0,"double",0,0,0,0,0,0,0,"double",0,0,0,0,0,0,0,"double",0,0,0,0,0,0,0,"double",0,0,0,0,0,0,0,"double",0,0,0,0,0,0,0,"double",0,0,0,0,0,0,0,"double",0,0,0,0,0,0,0,"double",0,0,0,0,0,0,0,"double",0,0,0,0,0,0,0,"double",0,0,0,0,0,0,0,"double",0,0,0,0,0,0,0,"double",0,0,0,0,0,0,0,"double",0,0,0,0,0,0,0,"double",0,0,0,0,0,0,0,"double",0,0,0,0,0,0,0,"double",0,0,0,0,0,0,0,"double",0,0,0,0,0,0,0,"double",0,0,0,0,0,0,0,"double",0,0,0,0,0,0,0,"double",0,0,0,0,0,0,0,"double",0,0,0,0,0,0,0,"double",0,0,0,0,0,0,0,"double",0,0,0,0,0,0,0,"double",0,0,0,0,0,0,0,"double",0,0,0,0,0,0,0,"double",0,0,0,0,0,0,0,"double",0,0,0,0,0,0,0,"double",0,0,0,0,0,0,0,"double",0,0,0,0,0,0,0,"double",0,0,0,0,0,0,0,"double",0,0,0,0,0,0,0,"double",0,0,0,0,0,0,0,"double",0,0,0,0,0,0,0,"double",0,0,0,0,0,0,0,"double",0,0,0,0,0,0,0,"double",0,0,0,0,0,0,0,"double",0,0,0,0,0,0,0,"double",0,0,0,0,0,0,0,"double",0,0,0,0,0,0,0,"double",0,0,0,0,0,0,0,"double",0,0,0,0,0,0,0,"double",0,0,0,0,0,0,0,"double",0,0,0,0,0,0,0,"double",0,0,0,0,0,0,0,"double",0,0,0,0,0,0,0,"double",0,0,0,0,0,0,0,"double",0,0,0,0,0,0,0,"double",0,0,0,0,0,0,0,"double",0,0,0,0,0,0,0,"double",0,0,0,0,0,0,0,"double",0,0,0,0,0,0,0,"double",0,0,0,0,0,0,0,"double",0,0,0,0,0,0,0,"double",0,0,0,0,0,0,0,"double",0,0,0,0,0,0,0,"double",0,0,0,0,0,0,0,"double",0,0,0,0,0,0,0,"double",0,0,0,0,0,0,0,"double",0,0,0,0,0,0,0,"double",0,0,0,0,0,0,0,"double",0,0,0,0,0,0,0,"double",0,0,0,0,0,0,0,"double",0,0,0,0,0,0,0,"double",0,0,0,0,0,0,0,"double",0,0,0,0,0,0,0,"double",0,0,0,0,0,0,0,"double",0,0,0,0,0,0,0,"double",0,0,0,0,0,0,0,"double",0,0,0,0,0,0,0,"double",0,0,0,0,0,0,0,"double",0,0,0,0,0,0,0,"double",0,0,0,0,0,0,0,"double",0,0,0,0,0,0,0,"double",0,0,0,0,0,0,0,"double",0,0,0,0,0,0,0,"double",0,0,0,0,0,0,0,"double",0,0,0,0,0,0,0,"double",0,0,0,0,0,0,0,"double",0,0,0,0,0,0,0,"double",0,0,0,0,0,0,0,"double",0,0,0,0,0,0,0,"double",0,0,0,0,0,0,0,"double",0,0,0,0,0,0,0,"double",0,0,0,0,0,0,0,"double",0,0,0,0,0,0,0,"double",0,0,0,0,0,0,0,"double",0,0,0,0,0,0,0,"double",0,0,0,0,0,0,0,"double",0,0,0,0,0,0,0,"double",0,0,0,0,0,0,0,"double",0,0,0,0,0,0,0,"double",0,0,0,0,0,0,0,"double",0,0,0,0,0,0,0,"double",0,0,0,0,0,0,0,"double",0,0,0,0,0,0,0,"double",0,0,0,0,0,0,0,"double",0,0,0,0,0,0,0,"double",0,0,0,0,0,0,0,"double",0,0,0,0,0,0,0,"double",0,0,0,0,0,0,0,"double",0,0,0,0,0,0,0,"double",0,0,0,0,0,0,0,"double",0,0,0,0,0,0,0,"double",0,0,0,0,0,0,0,"double",0,0,0,0,0,0,0,"double",0,0,0,0,0,0,0,"double",0,0,0,0,0,0,0,"double",0,0,0,0,0,0,0,"double",0,0,0,0,0,0,0,"double",0,0,0,0,0,0,0,"double",0,0,0,0,0,0,0,"double",0,0,0,0,0,0,0,"double",0,0,0,0,0,0,0,"double",0,0,0,0,0,0,0,"double",0,0,0,0,0,0,0,"double",0,0,0,0,0,0,0,"double",0,0,0,0,0,0,0,"double",0,0,0,0,0,0,0,"double",0,0,0,0,0,0,0,"double",0,0,0,0,0,0,0,"double",0,0,0,0,0,0,0,"double",0,0,0,0,0,0,0,"double",0,0,0,0,0,0,0,"double",0,0,0,0,0,0,0,"double",0,0,0,0,0,0,0,"double",0,0,0,0,0,0,0,"double",0,0,0,0,0,0,0,"double",0,0,0,0,0,0,0,"double",0,0,0,0,0,0,0,"double",0,0,0,0,0,0,0,"double",0,0,0,0,0,0,0,"double",0,0,0,0,0,0,0,"double",0,0,0,0,0,0,0,"double",0,0,0,0,0,0,0,"double",0,0,0,0,0,0,0,"double",0,0,0,0,0,0,0,"double",0,0,0,0,0,0,0,"double",0,0,0,0,0,0,0,"double",0,0,0,0,0,0,0,"double",0,0,0,0,0,0,0,"double",0,0,0,0,0,0,0,"double",0,0,0,0,0,0,0,"double",0,0,0,0,0,0,0,"double",0,0,0,0,0,0,0,"double",0,0,0,0,0,0,0,"double",0,0,0,0,0,0,0,"double",0,0,0,0,0,0,0,"double",0,0,0,0,0,0,0,"double",0,0,0,0,0,0,0,"double",0,0,0,0,0,0,0,"double",0,0,0,0,0,0,0,"double",0,0,0,0,0,0,0,"double",0,0,0,0,0,0,0,"double",0,0,0,0,0,0,0,"double",0,0,0,0,0,0,0,"double",0,0,0,0,0,0,0,"double",0,0,0,0,0,0,0,"double",0,0,0,0,0,0,0,"double",0,0,0,0,0,0,0,"double",0,0,0,0,0,0,0,"double",0,0,0,0,0,0,0,"double",0,0,0,0,0,0,0,"double",0,0,0,0,0,0,0,"double",0,0,0,0,0,0,0,"double",0,0,0,0,0,0,0,"double",0,0,0,0,0,0,0,"double",0,0,0,0,0,0,0,"double",0,0,0,0,0,0,0,"double",0,0,0,0,0,0,0,"double",0,0,0,0,0,0,0,"double",0,0,0,0,0,0,0,"double",0,0,0,0,0,0,0,"double",0,0,0,0,0,0,0,"double",0,0,0,0,0,0,0,"double",0,0,0,0,0,0,0,"double",0,0,0,0,0,0,0,"double",0,0,0,0,0,0,0,"double",0,0,0,0,0,0,0,"double",0,0,0,0,0,0,0,"double",0,0,0,0,0,0,0,"double",0,0,0,0,0,0,0,"double",0,0,0,0,0,0,0,"double",0,0,0,0,0,0,0,"double",0,0,0,0,0,0,0,"double",0,0,0,0,0,0,0,"double",0,0,0,0,0,0,0,"double",0,0,0,0,0,0,0,"double",0,0,0,0,0,0,0,"double",0,0,0,0,0,0,0,"double",0,0,0,0,0,0,0,"double",0,0,0,0,0,0,0,"double",0,0,0,0,0,0,0,"double",0,0,0,0,0,0,0,"double",0,0,0,0,0,0,0,"double",0,0,0,0,0,0,0,"double",0,0,0,0,0,0,0,"double",0,0,0,0,0,0,0,"double",0,0,0,0,0,0,0,"double",0,0,0,0,0,0,0,"double",0,0,0,0,0,0,0,"double",0,0,0,0,0,0,0,"double",0,0,0,0,0,0,0,"double",0,0,0,0,0,0,0,"double",0,0,0,0,0,0,0,"double",0,0,0,0,0,0,0,"double",0,0,0,0,0,0,0,"double",0,0,0,0,0,0,0,"double",0,0,0,0,0,0,0,"double",0,0,0,0,0,0,0,"double",0,0,0,0,0,0,0,"double",0,0,0,0,0,0,0,"double",0,0,0,0,0,0,0,"double",0,0,0,0,0,0,0,"double",0,0,0,0,0,0,0,"double",0,0,0,0,0,0,0,"double",0,0,0,0,0,0,0,"double",0,0,0,0,0,0,0,"double",0,0,0,0,0,0,0,"double",0,0,0,0,0,0,0,"double",0,0,0,0,0,0,0,"double",0,0,0,0,0,0,0,"double",0,0,0,0,0,0,0,"double",0,0,0,0,0,0,0,"double",0,0,0,0,0,0,0,"double",0,0,0,0,0,0,0,"double",0,0,0,0,0,0,0,"double",0,0,0,0,0,0,0,"double",0,0,0,0,0,0,0,"double",0,0,0,0,0,0,0,"double",0,0,0,0,0,0,0,"double",0,0,0,0,0,0,0,"double",0,0,0,0,0,0,0,"double",0,0,0,0,0,0,0,"double",0,0,0,0,0,0,0,"double",0,0,0,0,0,0,0,"double",0,0,0,0,0,0,0,"double",0,0,0,0,0,0,0,"double",0,0,0,0,0,0,0,"double",0,0,0,0,0,0,0,"double",0,0,0,0,0,0,0], ALLOC_STATIC);
__ZL5peaks801=allocate(720, ["i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"double",0,0,0,0,0,0,0,"double",0,0,0,0,0,0,0,"double",0,0,0,0,0,0,0,"double",0,0,0,0,0,0,0,"double",0,0,0,0,0,0,0,"double",0,0,0,0,0,0,0,"double",0,0,0,0,0,0,0,"double",0,0,0,0,0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"double",0,0,0,0,0,0,0,"double",0,0,0,0,0,0,0,"double",0,0,0,0,0,0,0,"double",0,0,0,0,0,0,0,"double",0,0,0,0,0,0,0,"double",0,0,0,0,0,0,0,"double",0,0,0,0,0,0,0,"double",0,0,0,0,0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"double",0,0,0,0,0,0,0,"double",0,0,0,0,0,0,0,"double",0,0,0,0,0,0,0,"double",0,0,0,0,0,0,0,"double",0,0,0,0,0,0,0,"double",0,0,0,0,0,0,0,"double",0,0,0,0,0,0,0,"double",0,0,0,0,0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"double",0,0,0,0,0,0,0,"double",0,0,0,0,0,0,0,"double",0,0,0,0,0,0,0,"double",0,0,0,0,0,0,0,"double",0,0,0,0,0,0,0,"double",0,0,0,0,0,0,0,"double",0,0,0,0,0,0,0,"double",0,0,0,0,0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"double",0,0,0,0,0,0,0,"double",0,0,0,0,0,0,0,"double",0,0,0,0,0,0,0,"double",0,0,0,0,0,0,0,"double",0,0,0,0,0,0,0,"double",0,0,0,0,0,0,0,"double",0,0,0,0,0,0,0,"double",0,0,0,0,0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"double",0,0,0,0,0,0,0,"double",0,0,0,0,0,0,0,"double",0,0,0,0,0,0,0,"double",0,0,0,0,0,0,0,"double",0,0,0,0,0,0,0,"double",0,0,0,0,0,0,0,"double",0,0,0,0,0,0,0,"double",0,0,0,0,0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"double",0,0,0,0,0,0,0,"double",0,0,0,0,0,0,0,"double",0,0,0,0,0,0,0,"double",0,0,0,0,0,0,0,"double",0,0,0,0,0,0,0,"double",0,0,0,0,0,0,0,"double",0,0,0,0,0,0,0,"double",0,0,0,0,0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"double",0,0,0,0,0,0,0,"double",0,0,0,0,0,0,0,"double",0,0,0,0,0,0,0,"double",0,0,0,0,0,0,0,"double",0,0,0,0,0,0,0,"double",0,0,0,0,0,0,0,"double",0,0,0,0,0,0,0,"double",0,0,0,0,0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"double",0,0,0,0,0,0,0,"double",0,0,0,0,0,0,0,"double",0,0,0,0,0,0,0,"double",0,0,0,0,0,0,0,"double",0,0,0,0,0,0,0,"double",0,0,0,0,0,0,0,"double",0,0,0,0,0,0,0,"double",0,0,0,0,0,0,0], ALLOC_STATIC);
__ZL12sample_count=allocate(1, "i32", ALLOC_STATIC);
__ZL8nsamples802=allocate(1, "i32", ALLOC_STATIC);
__ZL6klattp=allocate(40, "i32", ALLOC_STATIC);
__ZL10klattp_inc=allocate(80, "double", ALLOC_STATIC);
__ZL7klattp1=allocate(80, "double", ALLOC_STATIC);
__ZL8end_wave803=allocate(1, "i32", ALLOC_STATIC);
__ZZ14SetSynth_KlattiiP7frame_tS0_P7voice_tiE7prev_fr=allocate(64, ["i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8","i8"], ALLOC_STATIC);
__ZL13scale_wav_tab=allocate([45, 0, 0, 0, 38, 0, 0, 0, 45, 0, 0, 0, 45, 0, 0, 0], ["i32",0,0,0,"i32",0,0,0,"i32",0,0,0,"i32",0,0,0], ALLOC_STATIC);
__ZZ9KlattInitvE15natural_samples=allocate([-310, 0, -400, 0, 530, 0, 356, 0, 224, 0, 89, 0, 23, 0, -10, 0, -58, 0, -16, 0, 461, 0, 599, 0, 536, 0, 701, 0, 770, 0, 605, 0, 497, 0, 461, 0, 560, 0, 404, 0, 110, 0, 224, 0, 131, 0, 104, 0, -97, 0, 155, 0, 278, 0, -154, 0, -1165, 0, -598, 0, 737, 0, 125, 0, -592, 0, 41, 0, 11, 0, -247, 0, -10, 0, 65, 0, 92, 0, 80, 0, -304, 0, 71, 0, 167, 0, -1, 0, 122, 0, 233, 0, 161, 0, -43, 0, 278, 0, 479, 0, 485, 0, 407, 0, 266, 0, 650, 0, 134, 0, 80, 0, 236, 0, 68, 0, 260, 0, 269, 0, 179, 0, 53, 0, 140, 0, 275, 0, 293, 0, 296, 0, 104, 0, 257, 0, 152, 0, 311, 0, 182, 0, 263, 0, 245, 0, 125, 0, 314, 0, 140, 0, 44, 0, 203, 0, 230, 0, -235, 0, -286, 0, 23, 0, 107, 0, 92, 0, -91, 0, 38, 0, 464, 0, 443, 0, 176, 0, 98, 0, -784, 0, -2449, 0, -1891, 0, -1045, 0, -1600, 0, -1462, 0, -1384, 0, -1261, 0, -949, 0, -730, 0], ["i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0], ALLOC_STATIC);
__ZZ9KlattInitvE10formant_hz=allocate([280, 0, 688, 0, 1064, 0, 2806, 0, 3260, 0, 3700, 0, 6500, 0, 7000, 0, 8000, 0, 280, 0], ["i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0], ALLOC_STATIC);
__ZZ9KlattInitvE9bandwidth=allocate([89, 0, 160, 0, 70, 0, 160, 0, 200, 0, 200, 0, 500, 0, 500, 0, 500, 0, 89, 0], ["i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0], ALLOC_STATIC);
__ZZ9KlattInitvE12parallel_amp=allocate([0, 0, 59, 0, 59, 0, 59, 0, 59, 0, 59, 0, 59, 0, 0, 0, 0, 0, 0, 0], ["i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0], ALLOC_STATIC);
__ZZ9KlattInitvE11parallel_bw=allocate([59, 0, 59, 0, 89, 0, 149, 0, 200, 0, 200, 0, 500, 0, 0, 0, 0, 0, 0, 0], ["i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0], ALLOC_STATIC);
__ZZL7parwaveP13klatt_frame_tE5noise=allocate(1, "double", ALLOC_STATIC);
__ZZL7parwaveP13klatt_frame_tE5voice=allocate(1, "double", ALLOC_STATIC);
__ZZL7parwaveP13klatt_frame_tE5vlast=allocate(1, "double", ALLOC_STATIC);
__ZZL7parwaveP13klatt_frame_tE8glotlast=allocate(1, "double", ALLOC_STATIC);
__ZZL7parwaveP13klatt_frame_tE5sourc=allocate(1, "double", ALLOC_STATIC);
__ZZL21pitch_synch_par_resetP13klatt_frame_tE4skew=allocate(1, "i32", ALLOC_STATIC);
__ZZL21pitch_synch_par_resetP13klatt_frame_tE2B0=allocate([1200, 0, 1142, 0, 1088, 0, 1038, 0, 991, 0, 948, 0, 907, 0, 869, 0, 833, 0, 799, 0, 768, 0, 738, 0, 710, 0, 683, 0, 658, 0, 634, 0, 612, 0, 590, 0, 570, 0, 551, 0, 533, 0, 515, 0, 499, 0, 483, 0, 468, 0, 454, 0, 440, 0, 427, 0, 415, 0, 403, 0, 391, 0, 380, 0, 370, 0, 360, 0, 350, 0, 341, 0, 332, 0, 323, 0, 315, 0, 307, 0, 300, 0, 292, 0, 285, 0, 278, 0, 272, 0, 265, 0, 259, 0, 253, 0, 247, 0, 242, 0, 237, 0, 231, 0, 226, 0, 221, 0, 217, 0, 212, 0, 208, 0, 204, 0, 199, 0, 195, 0, 192, 0, 188, 0, 184, 0, 180, 0, 177, 0, 174, 0, 170, 0, 167, 0, 164, 0, 161, 0, 158, 0, 155, 0, 153, 0, 150, 0, 147, 0, 145, 0, 142, 0, 140, 0, 137, 0, 135, 0, 133, 0, 131, 0, 128, 0, 126, 0, 124, 0, 122, 0, 120, 0, 119, 0, 117, 0, 115, 0, 113, 0, 111, 0, 110, 0, 108, 0, 106, 0, 105, 0, 103, 0, 102, 0, 100, 0, 99, 0, 97, 0, 96, 0, 95, 0, 93, 0, 92, 0, 91, 0, 90, 0, 88, 0, 87, 0, 86, 0, 85, 0, 84, 0, 83, 0, 82, 0, 80, 0, 79, 0, 78, 0, 77, 0, 76, 0, 75, 0, 75, 0, 74, 0, 73, 0, 72, 0, 71, 0, 70, 0, 69, 0, 68, 0, 68, 0, 67, 0, 66, 0, 65, 0, 64, 0, 64, 0, 63, 0, 62, 0, 61, 0, 61, 0, 60, 0, 59, 0, 59, 0, 58, 0, 57, 0, 57, 0, 56, 0, 56, 0, 55, 0, 55, 0, 54, 0, 54, 0, 53, 0, 53, 0, 52, 0, 52, 0, 51, 0, 51, 0, 50, 0, 50, 0, 49, 0, 49, 0, 48, 0, 48, 0, 47, 0, 47, 0, 46, 0, 46, 0, 45, 0, 45, 0, 44, 0, 44, 0, 43, 0, 43, 0, 42, 0, 42, 0, 41, 0, 41, 0, 41, 0, 41, 0, 40, 0, 40, 0, 39, 0, 39, 0, 38, 0, 38, 0, 38, 0, 38, 0, 37, 0, 37, 0, 36, 0, 36, 0, 36, 0, 36, 0, 35, 0, 35, 0, 35, 0, 35, 0, 34, 0, 34, 0, 33, 0, 33, 0, 33, 0, 33, 0, 32, 0, 32, 0, 32, 0, 32, 0, 31, 0, 31, 0, 31, 0, 31, 0, 30, 0, 30, 0, 30, 0, 30, 0, 29, 0, 29, 0, 29, 0, 29, 0, 28, 0, 28, 0, 28, 0, 28, 0, 27, 0, 27, 0], ["i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0], ALLOC_STATIC);
__ZZL7DBtoLINlE8amptable=allocate([0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 6, 0, 7, 0, 8, 0, 9, 0, 10, 0, 11, 0, 13, 0, 14, 0, 16, 0, 18, 0, 20, 0, 22, 0, 25, 0, 28, 0, 32, 0, 35, 0, 40, 0, 45, 0, 51, 0, 57, 0, 64, 0, 71, 0, 80, 0, 90, 0, 101, 0, 114, 0, 128, 0, 142, 0, 159, 0, 179, 0, 202, 0, 227, 0, 256, 0, 284, 0, 318, 0, 359, 0, 405, 0, 455, 0, 512, 0, 568, 0, 638, 0, 719, 0, 881, 0, 911, 0, 1024, 0, 1137, 0, 1276, 0, 1438, 0, 1622, 0, 1823, 0, 2048, 0, 2273, 0, 2552, 0, 2875, 0, 3244, 0, 3645, 0, 4096, 0, 4547, 0, 5104, 0, 5751, 0, 6488, 0, 7291, 0, 8192, 0, 9093, 0, 10207, 0, 11502, 0, 12976, 0, 14582, 0, 16384, 0, 18350, 0, 20644, 0, 23429, 0, 26214, 0, 29491, 0, 32767, 0], ["i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0,"i16",0], ALLOC_STATIC);
__ZZL14natural_sourcevE5vwave=allocate(1, "double", ALLOC_STATIC);
__ZZL16impulsive_sourcevE7doublet=allocate([0, 0, 0, 0, 0, 0, 0, 0, 13000000, 0, 0, 0, 0, 0, 0, 0, -13000000, 0, 0, 0, 0, 0, 0, 0], ["double",0,0,0,0,0,0,0,"double",0,0,0,0,0,0,0,"double",0,0,0,0,0,0,0], ALLOC_STATIC);
__ZZL16impulsive_sourcevE5vwave=allocate(1, "double", ALLOC_STATIC);
__ZZL9gen_noisedE5nlast=allocate(1, "double", ALLOC_STATIC);
__ZZL7flutterP13klatt_frame_tE10time_count=allocate(1, "i32", ALLOC_STATIC);
__ZZL10frame_initP13klatt_frame_tE14amp_par_factor=allocate([0.6, 0, 0, 0, 0, 0, 0, 0, 0.4, 0, 0, 0, 0, 0, 0, 0, 0.15, 0, 0, 0, 0, 0, 0, 0, 0.06, 0, 0, 0, 0, 0, 0, 0, 0.04, 0, 0, 0, 0, 0, 0, 0, 0.022, 0, 0, 0, 0, 0, 0, 0, 0.03, 0, 0, 0, 0, 0, 0, 0], ["double",0,0,0,0,0,0,0,"double",0,0,0,0,0,0,0,"double",0,0,0,0,0,0,0,"double",0,0,0,0,0,0,0,"double",0,0,0,0,0,0,0,"double",0,0,0,0,0,0,0,"double",0,0,0,0,0,0,0], ALLOC_STATIC);
__str810=allocate([47,116,109,112,47,115,111,110,105,99,46,108,111,103,0] /* /tmp/sonic.log\00 */, "i8", ALLOC_STATIC);
__str1811=allocate([97,0] /* a\00 */, "i8", ALLOC_STATIC);
_optarg=allocate(1, "i8*", ALLOC_STATIC);
_optind=allocate(1, "i32", ALLOC_STATIC);
_opterr=allocate([1], ["i32",0,0,0,0], ALLOC_STATIC);
_optopt=allocate([63], ["i32",0,0,0,0], ALLOC_STATIC);
__ZL8optwhere=allocate(1, "i32", ALLOC_STATIC);
__str831=allocate([45,45,0] /* --\00 */, "i8", ALLOC_STATIC);
__str1832=allocate([80,79,83,73,88,76,89,95,67,79,82,82,69,67,84,0] /* POSIXLY_CORRECT\00 */, "i8", ALLOC_STATIC);
__str2833=allocate([37,115,58,32,111,112,116,105,111,110,32,96,37,115,39,32,105,115,32,97,109,98,105,103,117,111,117,115,32,40,99,111,117,108,100,32,98,101,32,96,45,45,37,115,39,32,111,114,32,96,45,45,37,115,39,41,10,0] /* %s: option `%s' is a */, "i8", ALLOC_STATIC);
__str3834=allocate([37,115,58,32,105,110,118,97,108,105,100,32,111,112,116,105,111,110,32,45,45,32,96,45,37,99,39,10,0] /* %s: invalid option - */, "i8", ALLOC_STATIC);
__str4835=allocate([37,115,58,32,97,114,103,117,109,101,110,116,32,114,101,113,117,105,114,101,100,32,102,111,114,32,111,112,116,105,111,110,32,96,0] /* %s: argument require */, "i8", ALLOC_STATIC);
__str5836=allocate([45,45,37,115,39,10,0] /* --%s'\0A\00 */, "i8", ALLOC_STATIC);
__str6837=allocate([45,37,99,39,10,0] /* -%c'\0A\00 */, "i8", ALLOC_STATIC);
HEAP32[((__ZZ4mainE12long_options)>>2)]=((__str6)&4294967295);
HEAP32[((__ZZ4mainE12long_options+16)>>2)]=((__str7)&4294967295);
HEAP32[((__ZZ4mainE12long_options+32)>>2)]=((__str8)&4294967295);
HEAP32[((__ZZ4mainE12long_options+48)>>2)]=((__str9)&4294967295);
HEAP32[((__ZZ4mainE12long_options+64)>>2)]=((__str10)&4294967295);
HEAP32[((__ZZ4mainE12long_options+80)>>2)]=((__str11)&4294967295);
HEAP32[((__ZZ4mainE12long_options+96)>>2)]=((__str12)&4294967295);
HEAP32[((__ZZ4mainE12long_options+112)>>2)]=((__str13)&4294967295);
HEAP32[((__ZZ4mainE12long_options+128)>>2)]=((__str14)&4294967295);
HEAP32[((__ZZ4mainE12long_options+144)>>2)]=((__str15)&4294967295);
HEAP32[((__ZZ4mainE12long_options+160)>>2)]=((__str16)&4294967295);
HEAP32[((__ZZ4mainE12long_options+176)>>2)]=((__str17)&4294967295);
HEAP32[((_mnem_rules)>>2)]=((__str42)&4294967295);
HEAP32[((_mnem_rules+8)>>2)]=((__str143)&4294967295);
HEAP32[((_mnem_rules+16)>>2)]=((__str244)&4294967295);
HEAP32[((_mnem_rules+24)>>2)]=((__str345)&4294967295);
HEAP32[((_mnem_rules+32)>>2)]=((__str446)&4294967295);
HEAP32[((_mnem_rules+40)>>2)]=((__str547)&4294967295);
HEAP32[((_mnem_rules+48)>>2)]=((__str648)&4294967295);
HEAP32[((_mnem_flags)>>2)]=((__str749)&4294967295);
HEAP32[((_mnem_flags+8)>>2)]=((__str850)&4294967295);
HEAP32[((_mnem_flags+16)>>2)]=((__str951)&4294967295);
HEAP32[((_mnem_flags+24)>>2)]=((__str1052)&4294967295);
HEAP32[((_mnem_flags+32)>>2)]=((__str1153)&4294967295);
HEAP32[((_mnem_flags+40)>>2)]=((__str1254)&4294967295);
HEAP32[((_mnem_flags+48)>>2)]=((__str1355)&4294967295);
HEAP32[((_mnem_flags+56)>>2)]=((__str1456)&4294967295);
HEAP32[((_mnem_flags+64)>>2)]=((__str1557)&4294967295);
HEAP32[((_mnem_flags+72)>>2)]=((__str1658)&4294967295);
HEAP32[((_mnem_flags+80)>>2)]=((__str1759)&4294967295);
HEAP32[((_mnem_flags+88)>>2)]=((__str1860)&4294967295);
HEAP32[((_mnem_flags+96)>>2)]=((__str1961)&4294967295);
HEAP32[((_mnem_flags+104)>>2)]=((__str2062)&4294967295);
HEAP32[((_mnem_flags+112)>>2)]=((__str2163)&4294967295);
HEAP32[((_mnem_flags+120)>>2)]=((__str2264)&4294967295);
HEAP32[((_mnem_flags+128)>>2)]=((__str2365)&4294967295);
HEAP32[((_mnem_flags+136)>>2)]=((__str2466)&4294967295);
HEAP32[((_mnem_flags+144)>>2)]=((__str2567)&4294967295);
HEAP32[((_mnem_flags+152)>>2)]=((__str2668)&4294967295);
HEAP32[((_mnem_flags+160)>>2)]=((__str2769)&4294967295);
HEAP32[((_mnem_flags+168)>>2)]=((__str2870)&4294967295);
HEAP32[((_mnem_flags+176)>>2)]=((__str2971)&4294967295);
HEAP32[((_mnem_flags+184)>>2)]=((__str3072)&4294967295);
HEAP32[((_mnem_flags+192)>>2)]=((__str3173)&4294967295);
HEAP32[((_mnem_flags+200)>>2)]=((__str3274)&4294967295);
HEAP32[((_mnem_flags+208)>>2)]=((__str3375)&4294967295);
HEAP32[((_mnem_flags+216)>>2)]=((__str3476)&4294967295);
HEAP32[((_mnem_flags+224)>>2)]=((__str3577)&4294967295);
HEAP32[((_mnem_flags+232)>>2)]=((__str3678)&4294967295);
HEAP32[((_mnem_flags+240)>>2)]=((__str3779)&4294967295);
HEAP32[((_mnem_flags+248)>>2)]=((__str3880)&4294967295);
HEAP32[((_mnem_flags+256)>>2)]=((__str3981)&4294967295);
HEAP32[((_mnem_flags+264)>>2)]=((__str4082)&4294967295);
HEAP32[((_mnem_flags+272)>>2)]=((__str4183)&4294967295);
HEAP32[((_mnem_flags+280)>>2)]=((__str4284)&4294967295);
HEAP32[((_mnem_flags+288)>>2)]=((__str43)&4294967295);
HEAP32[((_mnem_flags+296)>>2)]=((__str44)&4294967295);
HEAP32[((_mnem_flags+304)>>2)]=((__str45)&4294967295);
HEAP32[((_mnem_flags+312)>>2)]=((__str46)&4294967295);
HEAP32[((_mnem_flags+320)>>2)]=((__str47)&4294967295);
HEAP32[((_mnem_flags+328)>>2)]=((__str48)&4294967295);
HEAP32[((_mnem_flags+336)>>2)]=((__str49)&4294967295);
HEAP32[((_mnem_flags+344)>>2)]=((__str50)&4294967295);
HEAP32[((_mnem_flags+352)>>2)]=((__str51)&4294967295);
HEAP32[((_mnem_flags+360)>>2)]=((__str52)&4294967295);
HEAP32[((_mnem_flags+368)>>2)]=((__str53)&4294967295);
HEAP32[((_mnem_flags+376)>>2)]=((__str54)&4294967295);
HEAP32[((_mnem_flags+384)>>2)]=((__str55)&4294967295);
HEAP32[((_mnem_flags+392)>>2)]=((__str56)&4294967295);
HEAP32[((_mnem_flags+400)>>2)]=((__str57)&4294967295);
HEAP32[((__ZZL16copy_rule_stringPcRiE6outbuf)>>2)]=((__ZL9rule_cond)&4294967295);
HEAP32[((__ZZL16copy_rule_stringPcRiE6outbuf+4)>>2)]=((__ZL8rule_pre)&4294967295);
HEAP32[((__ZZL16copy_rule_stringPcRiE6outbuf+8)>>2)]=((__ZL10rule_match)&4294967295);
HEAP32[((__ZZL16copy_rule_stringPcRiE6outbuf+12)>>2)]=((__ZL9rule_post)&4294967295);
HEAP32[((__ZZL16copy_rule_stringPcRiE6outbuf+16)>>2)]=((__ZL13rule_phonemes)&4294967295);
HEAP32[((__ZZ12RemoveEndingP10TranslatorPciS1_E16add_e_exceptions)>>2)]=((__str17133)&4294967295);
HEAP32[((__ZZ12RemoveEndingP10TranslatorPciS1_E15add_e_additions)>>2)]=((__str18134)&4294967295);
HEAP32[((__ZZ12RemoveEndingP10TranslatorPciS1_E15add_e_additions+4)>>2)]=((__str19135)&4294967295);
HEAP32[((__ZZ12RemoveEndingP10TranslatorPciS1_E15add_e_additions+8)>>2)]=((__str20136)&4294967295);
HEAP32[((__ZZ12RemoveEndingP10TranslatorPciS1_E15add_e_additions+12)>>2)]=((__str21137)&4294967295);
HEAP32[((__ZZ12RemoveEndingP10TranslatorPciS1_E15add_e_additions+16)>>2)]=((__str22138)&4294967295);
HEAP32[((__ZZ12RemoveEndingP10TranslatorPciS1_E15add_e_additions+20)>>2)]=((__str23139)&4294967295);
HEAP32[((__ZZ12RemoveEndingP10TranslatorPciS1_E15add_e_additions+24)>>2)]=((__str24140)&4294967295);
HEAP32[((_envelope_data)>>2)]=((_env_fall)&4294967295);
HEAP32[((_envelope_data+4)>>2)]=((_env_fall)&4294967295);
HEAP32[((_envelope_data+8)>>2)]=((_env_rise)&4294967295);
HEAP32[((_envelope_data+12)>>2)]=((_env_rise)&4294967295);
HEAP32[((_envelope_data+16)>>2)]=((_env_frise)&4294967295);
HEAP32[((_envelope_data+20)>>2)]=((__ZL11env_r_frise)&4294967295);
HEAP32[((_envelope_data+24)>>2)]=((__ZL10env_frise2)&4294967295);
HEAP32[((_envelope_data+28)>>2)]=((__ZL12env_r_frise2)&4294967295);
HEAP32[((_envelope_data+32)>>2)]=((__ZL12env_risefall)&4294967295);
HEAP32[((_envelope_data+36)>>2)]=((__ZL12env_risefall)&4294967295);
HEAP32[((_envelope_data+40)>>2)]=((__ZL13env_fallrise3)&4294967295);
HEAP32[((_envelope_data+44)>>2)]=((__ZL13env_fallrise3)&4294967295);
HEAP32[((_envelope_data+48)>>2)]=((__ZL13env_fallrise4)&4294967295);
HEAP32[((_envelope_data+52)>>2)]=((__ZL13env_fallrise4)&4294967295);
HEAP32[((_envelope_data+56)>>2)]=((__ZL9env_fall2)&4294967295);
HEAP32[((_envelope_data+60)>>2)]=((__ZL9env_fall2)&4294967295);
HEAP32[((_envelope_data+64)>>2)]=((__ZL9env_rise2)&4294967295);
HEAP32[((_envelope_data+68)>>2)]=((__ZL9env_rise2)&4294967295);
HEAP32[((_envelope_data+72)>>2)]=((__ZL16env_risefallrise)&4294967295);
HEAP32[((_envelope_data+76)>>2)]=((__ZL16env_risefallrise)&4294967295);
HEAP32[((__ZL15tone_head_table+4)>>2)]=((__ZL7drops_0)&4294967295);
HEAP32[((__ZL15tone_head_table+12)>>2)]=((__ZL5oflow)&4294967295);
HEAP32[((__ZL15tone_head_table+20)>>2)]=((__ZL7drops_0)&4294967295);
HEAP32[((__ZL15tone_head_table+28)>>2)]=((__ZL5oflow)&4294967295);
HEAP32[((__ZL15tone_head_table+36)>>2)]=((__ZL7drops_0)&4294967295);
HEAP32[((__ZL15tone_head_table+44)>>2)]=((__ZL5oflow)&4294967295);
HEAP32[((__ZL15tone_head_table+52)>>2)]=((__ZL7drops_0)&4294967295);
HEAP32[((__ZL15tone_head_table+60)>>2)]=((__ZL9oflow_emf)&4294967295);
HEAP32[((__ZL15tone_head_table+68)>>2)]=((__ZL7drops_0)&4294967295);
HEAP32[((__ZL15tone_head_table+76)>>2)]=((__ZL5oflow)&4294967295);
HEAP32[((__ZL15tone_head_table+84)>>2)]=((__ZL7drops_0)&4294967295);
HEAP32[((__ZL15tone_head_table+92)>>2)]=((__ZL10oflow_less)&4294967295);
HEAP32[((__ZL15tone_head_table+100)>>2)]=((__ZL7drops_0)&4294967295);
HEAP32[((__ZL15tone_head_table+108)>>2)]=((__ZL10oflow_less)&4294967295);
HEAP32[((__ZL15tone_head_table+116)>>2)]=((__ZL7drops_0)&4294967295);
HEAP32[((__ZL15tone_head_table+124)>>2)]=((__ZL10oflow_less)&4294967295);
HEAP32[((__ZL15tone_head_table+132)>>2)]=((__ZL7drops_0)&4294967295);
HEAP32[((__ZL15tone_head_table+140)>>2)]=((__ZL5oflow)&4294967295);
HEAP32[((__ZL15tone_head_table+148)>>2)]=((__ZL7drops_0)&4294967295);
HEAP32[((__ZL15tone_head_table+156)>>2)]=((__ZL5oflow)&4294967295);
HEAP32[((__ZL15tone_head_table+164)>>2)]=((__ZL7drops_0)&4294967295);
HEAP32[((__ZL15tone_head_table+172)>>2)]=((__ZL5oflow)&4294967295);
HEAP32[((__ZL15tone_head_table+180)>>2)]=((__ZL7drops_0)&4294967295);
HEAP32[((__ZL15tone_head_table+188)>>2)]=((__ZL10oflow_less)&4294967295);
HEAP32[((__ZL15tone_head_table+196)>>2)]=((__ZL7drops_0)&4294967295);
HEAP32[((__ZL15tone_head_table+204)>>2)]=((__ZL10oflow_less)&4294967295);
HEAP32[((__ZL8ssmltags)>>2)]=((__str167)&4294967295);
HEAP32[((__ZL8ssmltags+8)>>2)]=((__str1168)&4294967295);
HEAP32[((__ZL8ssmltags+16)>>2)]=((__str2169)&4294967295);
HEAP32[((__ZL8ssmltags+24)>>2)]=((__str3170)&4294967295);
HEAP32[((__ZL8ssmltags+32)>>2)]=((__str4171)&4294967295);
HEAP32[((__ZL8ssmltags+40)>>2)]=((__str5172)&4294967295);
HEAP32[((__ZL8ssmltags+48)>>2)]=((__str6173)&4294967295);
HEAP32[((__ZL8ssmltags+56)>>2)]=((__str7174)&4294967295);
HEAP32[((__ZL8ssmltags+64)>>2)]=((__str8175)&4294967295);
HEAP32[((__ZL8ssmltags+72)>>2)]=((__str9176)&4294967295);
HEAP32[((__ZL8ssmltags+80)>>2)]=((__str10177)&4294967295);
HEAP32[((__ZL8ssmltags+88)>>2)]=((__str11178)&4294967295);
HEAP32[((__ZL8ssmltags+96)>>2)]=((__str12179)&4294967295);
HEAP32[((__ZL8ssmltags+104)>>2)]=((__str13180)&4294967295);
HEAP32[((__ZL8ssmltags+112)>>2)]=((__str14181)&4294967295);
HEAP32[((__ZL8ssmltags+120)>>2)]=((__str15182)&4294967295);
HEAP32[((__ZL8ssmltags+128)>>2)]=((__str16183)&4294967295);
HEAP32[((__ZL8ssmltags+136)>>2)]=((__str17184)&4294967295);
HEAP32[((__ZL8ssmltags+144)>>2)]=((__str18185)&4294967295);
HEAP32[((__ZL8ssmltags+152)>>2)]=((__str19186)&4294967295);
HEAP32[((__ZL8ssmltags+160)>>2)]=((__str20187)&4294967295);
HEAP32[((__ZL8ssmltags+168)>>2)]=((__str21188)&4294967295);
HEAP32[((__ZL8ssmltags+176)>>2)]=((__str22189)&4294967295);
HEAP32[((__ZL8ssmltags+184)>>2)]=((__str23190)&4294967295);
HEAP32[((__ZL8ssmltags+192)>>2)]=((__str24191)&4294967295);
HEAP32[((__ZL8ssmltags+200)>>2)]=((__str25192)&4294967295);
HEAP32[((__ZL8ssmltags+208)>>2)]=((__str26193)&4294967295);
HEAP32[((__ZL8ssmltags+216)>>2)]=((__str27194)&4294967295);
HEAP32[((__ZL8ssmltags+224)>>2)]=((__str28195)&4294967295);
HEAP32[((__ZL8ssmltags+232)>>2)]=((__str29196)&4294967295);
HEAP32[((__ZL8ssmltags+240)>>2)]=((__str30197)&4294967295);
HEAP32[((__ZL8ssmltags+248)>>2)]=((__str31198)&4294967295);
HEAP32[((__ZL18xml_char_mnemonics)>>2)]=((__str32199)&4294967295);
HEAP32[((__ZL18xml_char_mnemonics+8)>>2)]=((__str33200)&4294967295);
HEAP32[((__ZL18xml_char_mnemonics+16)>>2)]=((__str34201)&4294967295);
HEAP32[((__ZL18xml_char_mnemonics+24)>>2)]=((__str35202)&4294967295);
HEAP32[((__ZL18xml_char_mnemonics+32)>>2)]=((__str36203)&4294967295);
HEAP32[((__ZL18xml_char_mnemonics+40)>>2)]=((__str37204)&4294967295);
HEAP32[((__ZL7xmlbase)>>2)]=((__str13643)&4294967295);
HEAP32[((__ZZL14ProcessSsmlTagPwPcRiiiE10mnem_punct)>>2)]=((__str61228)&4294967295);
HEAP32[((__ZZL14ProcessSsmlTagPwPcRiiiE10mnem_punct+8)>>2)]=((__str62229)&4294967295);
HEAP32[((__ZZL14ProcessSsmlTagPwPcRiiiE10mnem_punct+16)>>2)]=((__str63230)&4294967295);
HEAP32[((__ZZL14ProcessSsmlTagPwPcRiiiE13mnem_capitals)>>2)]=((__str64231)&4294967295);
HEAP32[((__ZZL14ProcessSsmlTagPwPcRiiiE13mnem_capitals+8)>>2)]=((__str65232)&4294967295);
HEAP32[((__ZZL14ProcessSsmlTagPwPcRiiiE13mnem_capitals+16)>>2)]=((__str66233)&4294967295);
HEAP32[((__ZZL14ProcessSsmlTagPwPcRiiiE13mnem_capitals+24)>>2)]=((__str9659)&4294967295);
HEAP32[((__ZZL14ProcessSsmlTagPwPcRiiiE17mnem_interpret_as)>>2)]=((__str68235)&4294967295);
HEAP32[((__ZZL14ProcessSsmlTagPwPcRiiiE17mnem_interpret_as+8)>>2)]=((__str69236)&4294967295);
HEAP32[((__ZZL14ProcessSsmlTagPwPcRiiiE17mnem_interpret_as+16)>>2)]=((__str70237)&4294967295);
HEAP32[((__ZZL14ProcessSsmlTagPwPcRiiiE17mnem_interpret_as+24)>>2)]=((__str71238)&4294967295);
HEAP32[((__ZZL14ProcessSsmlTagPwPcRiiiE17mnem_interpret_as+32)>>2)]=((__str72239)&4294967295);
HEAP32[((__ZZL14ProcessSsmlTagPwPcRiiiE17mnem_sayas_format)>>2)]=((__str73240)&4294967295);
HEAP32[((__ZZL14ProcessSsmlTagPwPcRiiiE10mnem_break)>>2)]=((__str61228)&4294967295);
HEAP32[((__ZZL14ProcessSsmlTagPwPcRiiiE10mnem_break+8)>>2)]=((__str74241)&4294967295);
HEAP32[((__ZZL14ProcessSsmlTagPwPcRiiiE10mnem_break+16)>>2)]=((__str75242)&4294967295);
HEAP32[((__ZZL14ProcessSsmlTagPwPcRiiiE10mnem_break+24)>>2)]=((__str76243)&4294967295);
HEAP32[((__ZZL14ProcessSsmlTagPwPcRiiiE10mnem_break+32)>>2)]=((__str29196)&4294967295);
HEAP32[((__ZZL14ProcessSsmlTagPwPcRiiiE10mnem_break+40)>>2)]=((__str77244)&4294967295);
HEAP32[((__ZZL14ProcessSsmlTagPwPcRiiiE13mnem_emphasis)>>2)]=((__str61228)&4294967295);
HEAP32[((__ZZL14ProcessSsmlTagPwPcRiiiE13mnem_emphasis+8)>>2)]=((__str78245)&4294967295);
HEAP32[((__ZZL14ProcessSsmlTagPwPcRiiiE13mnem_emphasis+16)>>2)]=((__str79246)&4294967295);
HEAP32[((__ZZL14ProcessSsmlTagPwPcRiiiE13mnem_emphasis+24)>>2)]=((__str29196)&4294967295);
HEAP32[((__ZZL14ProcessSsmlTagPwPcRiiiE12prosody_attr+4)>>2)]=((__str80247)&4294967295);
HEAP32[((__ZZL14ProcessSsmlTagPwPcRiiiE12prosody_attr+8)>>2)]=((__str81248)&4294967295);
HEAP32[((__ZZL14ProcessSsmlTagPwPcRiiiE12prosody_attr+12)>>2)]=((__str9659)&4294967295);
HEAP32[((__ZZL14ProcessSsmlTagPwPcRiiiE12prosody_attr+16)>>2)]=((__str82249)&4294967295);
HEAP32[((__ZZL18GetVoiceAttributesPwiE11mnem_gender)>>2)]=((__str1647)&4294967295);
HEAP32[((__ZZL18GetVoiceAttributesPwiE11mnem_gender+8)>>2)]=((__str2648)&4294967295);
HEAP32[((__ZZL18GetVoiceAttributesPwiE11mnem_gender+16)>>2)]=((__str105272)&4294967295);
HEAP32[((__ZZL14ReplaceKeyNamePciRiE8keynames)>>2)]=((__str111278)&4294967295);
HEAP32[((__ZZL14ReplaceKeyNamePciRiE8keynames+8)>>2)]=((__str112279)&4294967295);
HEAP32[((__ZZL14ReplaceKeyNamePciRiE8keynames+16)>>2)]=((__str113280)&4294967295);
HEAP32[((__ZZL14ReplaceKeyNamePciRiE8keynames+24)>>2)]=((__str114281)&4294967295);
HEAP32[((__ZZL19SetProsodyParameteriPwP11PARAM_STACKE11mnem_volume)>>2)]=((__str45697)&4294967295);
HEAP32[((__ZZL19SetProsodyParameteriPwP11PARAM_STACKE11mnem_volume+8)>>2)]=((__str115282)&4294967295);
HEAP32[((__ZZL19SetProsodyParameteriPwP11PARAM_STACKE11mnem_volume+16)>>2)]=((__str116283)&4294967295);
HEAP32[((__ZZL19SetProsodyParameteriPwP11PARAM_STACKE11mnem_volume+24)>>2)]=((__str117)&4294967295);
HEAP32[((__ZZL19SetProsodyParameteriPwP11PARAM_STACKE11mnem_volume+32)>>2)]=((__str76243)&4294967295);
HEAP32[((__ZZL19SetProsodyParameteriPwP11PARAM_STACKE11mnem_volume+40)>>2)]=((__str118)&4294967295);
HEAP32[((__ZZL19SetProsodyParameteriPwP11PARAM_STACKE11mnem_volume+48)>>2)]=((__str119)&4294967295);
HEAP32[((__ZZL19SetProsodyParameteriPwP11PARAM_STACKE9mnem_rate)>>2)]=((__str45697)&4294967295);
HEAP32[((__ZZL19SetProsodyParameteriPwP11PARAM_STACKE9mnem_rate+8)>>2)]=((__str120)&4294967295);
HEAP32[((__ZZL19SetProsodyParameteriPwP11PARAM_STACKE9mnem_rate+16)>>2)]=((__str121)&4294967295);
HEAP32[((__ZZL19SetProsodyParameteriPwP11PARAM_STACKE9mnem_rate+24)>>2)]=((__str76243)&4294967295);
HEAP32[((__ZZL19SetProsodyParameteriPwP11PARAM_STACKE9mnem_rate+32)>>2)]=((__str122)&4294967295);
HEAP32[((__ZZL19SetProsodyParameteriPwP11PARAM_STACKE9mnem_rate+40)>>2)]=((__str123)&4294967295);
HEAP32[((__ZZL19SetProsodyParameteriPwP11PARAM_STACKE10mnem_pitch)>>2)]=((__str45697)&4294967295);
HEAP32[((__ZZL19SetProsodyParameteriPwP11PARAM_STACKE10mnem_pitch+8)>>2)]=((__str124)&4294967295);
HEAP32[((__ZZL19SetProsodyParameteriPwP11PARAM_STACKE10mnem_pitch+16)>>2)]=((__str125)&4294967295);
HEAP32[((__ZZL19SetProsodyParameteriPwP11PARAM_STACKE10mnem_pitch+24)>>2)]=((__str76243)&4294967295);
HEAP32[((__ZZL19SetProsodyParameteriPwP11PARAM_STACKE10mnem_pitch+32)>>2)]=((__str126)&4294967295);
HEAP32[((__ZZL19SetProsodyParameteriPwP11PARAM_STACKE10mnem_pitch+40)>>2)]=((__str127)&4294967295);
HEAP32[((__ZZL19SetProsodyParameteriPwP11PARAM_STACKE10mnem_range)>>2)]=((__str45697)&4294967295);
HEAP32[((__ZZL19SetProsodyParameteriPwP11PARAM_STACKE10mnem_range+8)>>2)]=((__str124)&4294967295);
HEAP32[((__ZZL19SetProsodyParameteriPwP11PARAM_STACKE10mnem_range+16)>>2)]=((__str125)&4294967295);
HEAP32[((__ZZL19SetProsodyParameteriPwP11PARAM_STACKE10mnem_range+24)>>2)]=((__str76243)&4294967295);
HEAP32[((__ZZL19SetProsodyParameteriPwP11PARAM_STACKE10mnem_range+32)>>2)]=((__str126)&4294967295);
HEAP32[((__ZZL19SetProsodyParameteriPwP11PARAM_STACKE10mnem_range+40)>>2)]=((__str127)&4294967295);
HEAP32[((__ZZL19SetProsodyParameteriPwP11PARAM_STACKE9mnem_tabs+4)>>2)]=((__ZZL19SetProsodyParameteriPwP11PARAM_STACKE9mnem_rate)&4294967295);
HEAP32[((__ZZL19SetProsodyParameteriPwP11PARAM_STACKE9mnem_tabs+8)>>2)]=((__ZZL19SetProsodyParameteriPwP11PARAM_STACKE11mnem_volume)&4294967295);
HEAP32[((__ZZL19SetProsodyParameteriPwP11PARAM_STACKE9mnem_tabs+12)>>2)]=((__ZZL19SetProsodyParameteriPwP11PARAM_STACKE10mnem_pitch)&4294967295);
HEAP32[((__ZZL19SetProsodyParameteriPwP11PARAM_STACKE9mnem_tabs+16)>>2)]=((__ZZL19SetProsodyParameteriPwP11PARAM_STACKE10mnem_range)&4294967295);
HEAP32[((__ZL11accents_tab)>>2)]=((__str57354)&4294967295);
HEAP32[((__ZL11accents_tab+8)>>2)]=((__str58355)&4294967295);
HEAP32[((__ZL11accents_tab+16)>>2)]=((__str59356)&4294967295);
HEAP32[((__ZL11accents_tab+24)>>2)]=((__str60357)&4294967295);
HEAP32[((__ZL11accents_tab+32)>>2)]=((__str61358)&4294967295);
HEAP32[((__ZL11accents_tab+40)>>2)]=((__str62359)&4294967295);
HEAP32[((__ZL11accents_tab+48)>>2)]=((__str63360)&4294967295);
HEAP32[((__ZL11accents_tab+56)>>2)]=((__str64361)&4294967295);
HEAP32[((__ZL11accents_tab+64)>>2)]=((__str65362)&4294967295);
HEAP32[((__ZL11accents_tab+72)>>2)]=((__str66363)&4294967295);
HEAP32[((__ZL11accents_tab+80)>>2)]=((__str67364)&4294967295);
HEAP32[((__ZL11accents_tab+88)>>2)]=((__str68365)&4294967295);
HEAP32[((__ZL11accents_tab+96)>>2)]=((__str69366)&4294967295);
HEAP32[((__ZL11accents_tab+104)>>2)]=((__str70367)&4294967295);
HEAP32[((__ZL11accents_tab+112)>>2)]=((__str71368)&4294967295);
HEAP32[((__ZL11accents_tab+120)>>2)]=((__str72369)&4294967295);
HEAP32[((__ZL11accents_tab+128)>>2)]=((__str73370)&4294967295);
HEAP32[((__ZL11accents_tab+136)>>2)]=((__str74371)&4294967295);
HEAP32[((__ZL11accents_tab+144)>>2)]=((__str75372)&4294967295);
HEAP32[((__ZL11accents_tab+152)>>2)]=((__str76373)&4294967295);
HEAP32[((__ZL11accents_tab+160)>>2)]=((__str77374)&4294967295);
HEAP32[((__ZL11accents_tab+168)>>2)]=((__str78375)&4294967295);
HEAP32[((_version_string)>>2)]=((__str399)&4294967295);
HEAP32[((_charsets)>>2)]=((__ZL10ISO_8859_1)&4294967295);
HEAP32[((_charsets+4)>>2)]=((__ZL10ISO_8859_1)&4294967295);
HEAP32[((_charsets+8)>>2)]=((__ZL10ISO_8859_2)&4294967295);
HEAP32[((_charsets+12)>>2)]=((__ZL10ISO_8859_3)&4294967295);
HEAP32[((_charsets+16)>>2)]=((__ZL10ISO_8859_4)&4294967295);
HEAP32[((_charsets+20)>>2)]=((__ZL10ISO_8859_5)&4294967295);
HEAP32[((_charsets+24)>>2)]=((__ZL10ISO_8859_1)&4294967295);
HEAP32[((_charsets+28)>>2)]=((__ZL10ISO_8859_7)&4294967295);
HEAP32[((_charsets+32)>>2)]=((__ZL10ISO_8859_1)&4294967295);
HEAP32[((_charsets+36)>>2)]=((__ZL10ISO_8859_9)&4294967295);
HEAP32[((_charsets+40)>>2)]=((__ZL10ISO_8859_1)&4294967295);
HEAP32[((_charsets+44)>>2)]=((__ZL10ISO_8859_1)&4294967295);
HEAP32[((_charsets+48)>>2)]=((__ZL10ISO_8859_1)&4294967295);
HEAP32[((_charsets+52)>>2)]=((__ZL10ISO_8859_1)&4294967295);
HEAP32[((_charsets+56)>>2)]=((__ZL11ISO_8859_14)&4294967295);
HEAP32[((_charsets+60)>>2)]=((__ZL10ISO_8859_1)&4294967295);
HEAP32[((_charsets+64)>>2)]=((__ZL10ISO_8859_1)&4294967295);
HEAP32[((_charsets+68)>>2)]=((__ZL10ISO_8859_1)&4294967295);
HEAP32[((_charsets+72)>>2)]=((__ZL6KOI8_R)&4294967295);
HEAP32[((_charsets+76)>>2)]=((__ZL5ISCII)&4294967295);
HEAP32[((__ZL15length_mod_tabs)>>2)]=((__ZL14length_mods_en)&4294967295);
HEAP32[((__ZL15length_mod_tabs+4)>>2)]=((__ZL14length_mods_en)&4294967295);
HEAP32[((__ZL15length_mod_tabs+8)>>2)]=((__ZL15length_mods_en0)&4294967295);
HEAP32[((__ZL15length_mod_tabs+12)>>2)]=((__ZL17length_mods_equal)&4294967295);
HEAP32[((__ZL15length_mod_tabs+16)>>2)]=((__ZL17length_mods_equal)&4294967295);
HEAP32[((__ZL15length_mod_tabs+20)>>2)]=((__ZL17length_mods_equal)&4294967295);
HEAP32[((_genders)>>2)]=((__str646)&4294967295);
HEAP32[((_genders+8)>>2)]=((__str1647)&4294967295);
HEAP32[((_genders+16)>>2)]=((__str2648)&4294967295);
HEAP32[((__ZL11options_tab)>>2)]=((__str3653)&4294967295);
HEAP32[((__ZL11options_tab+8)>>2)]=((__str4654)&4294967295);
HEAP32[((__ZL11keyword_tab)>>2)]=((__str5655)&4294967295);
HEAP32[((__ZL11keyword_tab+8)>>2)]=((__str6656)&4294967295);
HEAP32[((__ZL11keyword_tab+16)>>2)]=((__str7657)&4294967295);
HEAP32[((__ZL11keyword_tab+24)>>2)]=((__str8658)&4294967295);
HEAP32[((__ZL11keyword_tab+32)>>2)]=((__str9659)&4294967295);
HEAP32[((__ZL11keyword_tab+40)>>2)]=((__str10660)&4294967295);
HEAP32[((__ZL11keyword_tab+48)>>2)]=((__str11661)&4294967295);
HEAP32[((__ZL11keyword_tab+56)>>2)]=((__str12662)&4294967295);
HEAP32[((__ZL11keyword_tab+64)>>2)]=((__str13663)&4294967295);
HEAP32[((__ZL11keyword_tab+72)>>2)]=((__str14664)&4294967295);
HEAP32[((__ZL11keyword_tab+80)>>2)]=((__str15665)&4294967295);
HEAP32[((__ZL11keyword_tab+88)>>2)]=((__str16666)&4294967295);
HEAP32[((__ZL11keyword_tab+96)>>2)]=((__str17667)&4294967295);
HEAP32[((__ZL11keyword_tab+104)>>2)]=((__str18668)&4294967295);
HEAP32[((__ZL11keyword_tab+112)>>2)]=((__str19669)&4294967295);
HEAP32[((__ZL11keyword_tab+120)>>2)]=((__str20670)&4294967295);
HEAP32[((__ZL11keyword_tab+128)>>2)]=((__str21671)&4294967295);
HEAP32[((__ZL11keyword_tab+136)>>2)]=((__str22672)&4294967295);
HEAP32[((__ZL11keyword_tab+144)>>2)]=((__str23673)&4294967295);
HEAP32[((__ZL11keyword_tab+152)>>2)]=((__str24674)&4294967295);
HEAP32[((__ZL11keyword_tab+160)>>2)]=((__str25675)&4294967295);
HEAP32[((__ZL11keyword_tab+168)>>2)]=((__str26676)&4294967295);
HEAP32[((__ZL11keyword_tab+176)>>2)]=((__str27677)&4294967295);
HEAP32[((__ZL11keyword_tab+184)>>2)]=((__str28678)&4294967295);
HEAP32[((__ZL11keyword_tab+192)>>2)]=((__str29679)&4294967295);
HEAP32[((__ZL11keyword_tab+200)>>2)]=((__str30680)&4294967295);
HEAP32[((__ZL11keyword_tab+208)>>2)]=((__str31681)&4294967295);
HEAP32[((__ZL11keyword_tab+216)>>2)]=((__str32682)&4294967295);
HEAP32[((__ZL11keyword_tab+224)>>2)]=((__str33683)&4294967295);
HEAP32[((__ZL11keyword_tab+232)>>2)]=((__str34684)&4294967295);
HEAP32[((__ZL11keyword_tab+240)>>2)]=((__str35685)&4294967295);
HEAP32[((__ZL11keyword_tab+248)>>2)]=((__str36686)&4294967295);
HEAP32[((__ZL11keyword_tab+256)>>2)]=((__str37687)&4294967295);
HEAP32[((__ZL11keyword_tab+264)>>2)]=((__str38688)&4294967295);
HEAP32[((__ZL11keyword_tab+272)>>2)]=((__str39689)&4294967295);
HEAP32[((__ZL11keyword_tab+280)>>2)]=((__str40690)&4294967295);
HEAP32[((__ZL11keyword_tab+288)>>2)]=((__str41691)&4294967295);
HEAP32[((__ZL11keyword_tab+296)>>2)]=((__str42692)&4294967295);
HEAP32[((__ZL11keyword_tab+304)>>2)]=((__str43693)&4294967295);
HEAP32[((_variant_lists)>>2)]=((__ZL15variants_either)&4294967295);
HEAP32[((_variant_lists+4)>>2)]=((__ZL13variants_male)&4294967295);
HEAP32[((_variant_lists+8)>>2)]=((__ZL15variants_female)&4294967295);
HEAP32[((_voice)>>2)]=__ZL9voicedata;
FS.init(); __ATEXIT__.push({ func: function() { FS.quit() } });
___setErrNo(0);
_fputc.ret = allocate([0], "i8", ALLOC_STATIC);
___buildEnvironment(ENV);
_fgetc.ret = allocate([0], "i8", ALLOC_STATIC);
FUNCTION_TABLE = [0,0,__ZL13rgroup_sorterP6RGROUPS0_,0,__Z13string_sorterPPcS0_,0,__ZL15VoiceNameSorterPKvS0_,0,__ZL16VoiceScoreSorterPKvS0_,0]; Module["FUNCTION_TABLE"] = FUNCTION_TABLE;


  __globalConstructor__();

  var ret = null;
  if (Module._main) {
    ret = Module.callMain(args);
    __shutdownRuntime__();
  }
  return ret;
}
Module.run = run;

// {{PRE_RUN_ADDITIONS}}

if (!Module.noInitialRun) {
  run();
}

// {{POST_RUN_ADDITIONS}}


  // {{MODULE_ADDITIONS}}

/*
  return Module;
}).call(this, {}, arguments); // Replace parameters as needed
*/


// EMSCRIPTEN_GENERATED_FUNCTIONS: ["__Z13GetFileLengthPKc","__Z5Alloci","__Z4FreePv","__Z13DisplayVoicesP7__sFILEPc","__Z13WVoiceChangedP7voice_t","__Z11MarkerEventijiPh","_main","__ZL9init_pathPcS_","__ZL10initialisev","__ZL12OpenWaveFilePKci","__ZL11WavegenFilev","__ZL13CloseWaveFilev","__Z8isspace2j","__Z14LookupMnemNameP8MNEM_TABi","__Z22print_dictionary_flagsPj","__Z10DecodeRulePKciPci","__Z13string_sorterPPcS0_","__Z17CompileDictionaryPKcS0_P7__sFILEPci","__ZL9fopen_logPKcS0_","__ZL22compile_dictlist_startv","__ZL21compile_dictlist_filePKcS0_","__ZL20compile_dictlist_endP7__sFILE","__ZL17compile_dictrulesP7__sFILES0_Pc","__ZL17output_rule_groupP7__sFILEiPPcS1_","__ZL19compile_lettergroupPcP7__sFILE","__ZL12compile_rulePc","__ZL13rgroup_sorterP6RGROUPS0_","__ZL16copy_rule_stringPcRi","__ZL12compile_linePcS_Pi","__Z8strncpy0PcPKci","__Z13Reverse4Bytesi","__Z10LookupMnemP8MNEM_TABPKc","__Z14LoadDictionaryP10TranslatorPKci","__ZL10InitGroupsP10Translator","__Z14HashDictionaryPKc","__Z14EncodePhonemesPcS_Ph","__Z14DecodePhonemesPKcPc","__Z26GetTranslatedPhonemeStringPci","__ZL15WritePhMnemonicPcPiP11PHONEME_TABP12PHONEME_LIST","__Z7IsVowelP10Translatori","__ZL8IsLetterP10Translatorii","__Z14UnpronouncableP10TranslatorPci","__ZL15Unpronouncable2P10TranslatorPc","__Z16ChangeWordStressP10TranslatorPci","__ZL14GetVowelStressP10TranslatorPhPaRiS3_i","__Z13SetWordStressP10TranslatorPcPjii","__Z14AppendPhonemesP10TranslatorPciPKc","__Z14TranslateRulesP10TranslatorPcS1_iS1_iPj","__Z6LookupP10TranslatorPKcPc","__ZL9MatchRuleP10TranslatorPPcS1_iS1_P11MatchRecordii","__Z21ApplySpecialAttributeP10TranslatorPci","__Z22ApplySpecialAttribute2P10TranslatorPci","__Z17TransposeAlphabetP10TranslatorPc","__Z14LookupDictListP10TranslatorPPcS1_PjiP8WORD_TAB","__ZL11LookupDict2P10TranslatorPKcS2_PcPjiP8WORD_TAB","__Z11LookupFlagsP10TranslatorPKc","__Z12RemoveEndingP10TranslatorPciS1_","__ZL13IsLetterGroupP10TranslatorPcii","__Z11CalcPitchesP10Translatori","__ZL16CalcPitches_ToneP10Translatori","__ZL18count_pitch_vowelsiii","__ZL12calc_pitchesiiii","__ZL13calc_pitches2iii","__ZL16SetPitchGradientiiii","__ZL18calc_pitch_segmentiiP9TONE_HEADP12TONE_NUCLEUSii","__ZL9set_pitchP8SYLLABLEii","__ZL16count_incrementsiii","__ZL17SetHeadIntonationP4TUNEiii","__ZL15CountUnstressediii","_iswalpha","_iswdigit","_iswalnum","_towlower","_towupper","_iswupper","_iswlower","_iswspace","_iswpunct","_wcschr","_wcslen","_wcstod","__Z9towlower2j","__Z3Eofv","__Z10Read4BytesP7__sFILE","__Z11AddNameDataPKci","__Z13SetVoiceStackP12espeak_VOICE","__Z10ReadClauseP10TranslatorP7__sFILEPcPsPiiS5_S3_","__ZL4GetCv","__ZL6UngetCi","__ZL14ProcessSsmlTagPwPcRiii","__ZL13LookupSpecialP10TranslatorPKcPc","__ZL10RemoveCharPc","__ZL19AnnouncePunctuationP10TranslatoriPiPcS1_i","__ZL14LookupCharNameP10Translatorii","__ZL8IsRomanUj","__Z12InitNamedatav","__Z9InitText2v","__ZL13WordToString2j","__ZL15LookupSoundiconi","__ZL13LoadSoundFilePKci","__ZL14PushParamStacki","__ZL16GetSsmlAttributePwPKc","__ZL7attrcmpPKwPKc","__ZL10attrlookupPKwPK8MNEM_TAB","__ZL17ProcessParamStackPcRi","__ZL19SetProsodyParameteriPwP11PARAM_STACK","__ZL13PopParamStackiPcRi","__ZL10attrnumberPKwii","__ZL14ReplaceKeyNamePciRi","__ZL13attrcopy_utf8PcPKwi","__ZL14LoadSoundFile2PKc","__ZL18GetVoiceAttributesPwi","__ZL14VoiceFromStackv","__ZL18attr_prosody_valueiPKwPi","__ZL8GetC_getv","__ZL10GetC_ungeti","__Z8SetSpeedi","__Z12SetParameteriii","__Z11CalcLengthsP10Translator","__ZL11DoEmbedded2Pi","__Z20LookupAccentedLetterP10TranslatorjPc","__ZL13LookupLetter2P10TranslatorjPc","__Z12LookupLetterP10TranslatorjiPci","__Z15TranslateLetterP10TranslatorPcS1_i","__Z17SetSpellingStressP10TranslatorPcii","__Z14TranslateRomanP10TranslatorPcS1_P8WORD_TAB","__ZL15CheckDotOrdinalP10TranslatorPcS1_P8WORD_TABi","__ZL11hu_number_ePKcii","__Z15TranslateNumberP10TranslatorPcS1_PjP8WORD_TABi","__ZL17TranslateNumber_1P10TranslatorPcS1_PjP8WORD_TABi","__ZL15LookupThousandsP10TranslatoriiiPc","__ZL10LookupNum3P10TranslatoriPciii","__ZL10LookupNum2P10TranslatoriiPc","__ZL9M_Varianti","__Z15LoadMbrolaTablePKcS0_i","_setNoError_MBR","__Z15MbrolaTranslateP12PHONEME_LISTiiP7__sFILE","__ZL10GetMbrNameP12PHONEME_LISTP11PHONEME_TABS2_S2_PiS3_S3_","__ZL10WritePitchiiiii","__Z14MbrolaGenerateP12PHONEME_LISTPii","__Z10MbrolaFillii","__Z11MbrolaResetv","__Z10LoadPhDatav","__ZL10ReadPhFilePvPKcPi","__Z10FreePhDatav","__Z11PhonemeCodej","__Z19LookupPhonemeStringPKc","__Z11LookupSpectP11PHONEME_TABiP10FMT_PARAMSPiP12PHONEME_LIST","__Z11GetEnvelopei","__Z18SelectPhonemeTablei","__ZL17SetUpPhonemeTableii","__Z18LookupPhonemeTablePKc","__Z22SelectPhonemeTableNamePKc","__Z10LoadConfigv","__Z16InterpretPhonemeP10TranslatoriP12PHONEME_LISTP12PHONEME_DATA","__ZL12InvalidInstnP11PHONEME_TABi","__ZL15StressConditionP10TranslatorP12PHONEME_LISTii","__ZL18InterpretConditionP10TranslatoriP12PHONEME_LISTi","__ZL13NumInstnWordsPt","__ZL15SwitchVowelTypeP12PHONEME_LISTP12PHONEME_DATAPPti","__Z17InterpretPhoneme2iP12PHONEME_DATA","__Z12WordToStringj","__Z14SynthesizeInitv","__Z11PauseLengthii","__Z9DoSample3P12PHONEME_DATAii","__ZL8EndPitchi","__ZL9DoSample2iiiiii","__Z18FormantTransition2P10frameref_tRijjP11PHONEME_TABi","__ZL9CopyFrameP7frame_ti","__ZL13set_frame_rmsP7frame_ti","__ZL14AdjustFormantsP7frame_tiiiiiii","__ZL14VowelClosenessP7frame_t","__ZL18DuplicateLastFrameP10frameref_tii","__ZL7DoPauseii","__Z8DoSpect2P11PHONEME_TABiP10FMT_PARAMSP12PHONEME_LISTi","__ZL11SmoothSpectv","__Z8DoMarkeriiii","__Z12DoSonicSpeedi","__Z13DoVoiceChangeP7voice_t","__Z10DoEmbeddedPii","__Z8GenerateP12PHONEME_LISTPii","__ZL12EndAmplitudev","__ZL11DoAmplitudeiPh","__ZL7DoPitchPhii","__ZL13StartSyllablev","__Z12SynthOnTimerv","__Z15SpeakNextClauseP7__sFILEPKvi","__Z11SynthStatusv","__ZL18formants_reduce_hfP7frame_ti","__ZL10AllocFramev","__Z13SetLengthModsP10Translatori","__Z7IsAlphaj","__Z11lookupwcharPKti","__Z7IsDigitj","__Z7IsSpacej","__Z16DeleteTranslatorP10Translator","__Z9IsBracketi","__Z8utf8_outjPc","__Z11utf8_nbytesPKc","__Z8utf8_in2PiPKci","__Z7utf8_inPiPKc","__Z8strchr_wPKci","__Z10IsAllUpperPKc","__Z24ChangeEquivalentPhonemesP10TranslatoriPc","__Z13TranslateWordP10TranslatorPciP8WORD_TAB","__ZL17CheckDottedAbbrevPcP8WORD_TAB","__ZL22SpeakIndividualLettersP10TranslatorPcS1_i","__Z14SetTranslator2PKc","__Z15TranslateClauseP10TranslatorP7__sFILEPKvPiPPc","__ZL9IsDigit09j","__ZL15EmbeddedCommandRj","__ZL13TranslateCharP10TranslatorPcijjPi","__ZL14TranslateWord2P10TranslatorPcP8WORD_TABii","__Z8InitTexti","__ZL14CountSyllablesPh","__ZL9SetPlist2P13PHONEME_LIST2h","__ZL14SubstituteCharP10TranslatorjjPi","_init_MBR","__ZL12start_mbrolaPKc","__ZL14send_to_mbrolaPKc","__ZL11stop_mbrolav","__ZL19receive_from_mbrolaPvj","__ZL3errPKcz","_close_MBR","__ZL17free_pending_datav","_reset_MBR","__ZL17mbrola_has_errorsv","_read_MBR","_write_MBR","_flush_MBR","_getFreq_MBR","_setVolumeRatio_MBR","_lastErrorStr_MBR","_resetError_MBR","__ZL11mbrola_diedv","__ZL3logPKcz","__ZL14mbrola_is_idlev","__ZL12create_pipesPiS_S_","__ZL11close_pipesPiS_S_","__Z15SetIndicLettersP10Translator","__ZL18SetLetterBitsRangeP10Translatoriii","__ZL13SetLetterBitsP10TranslatoriPKc","__Z15SetupTranslatorP10TranslatorPKsPKh","__Z16SelectTranslatorPKc","__ZL13NewTranslatorv","__ZL14SetLetterVowelP10Translatori","__ZL18SetCyrillicLettersP10Translator","__ZL15ResetLetterBitsP10Translatori","__ZL18Translator_RussianP10Translator","__Z14ReadTonePointsPcPi","__Z10VoiceReseti","__ZL13SetToneAdjustP7voice_tPi","__Z9LoadVoicePKci","__ZL11fgets_stripPciP7__sFILE","__ZL12VoiceFormantPc","__ZL12Read8NumbersPcPi","__ZL10LookupTunePKc","__ZL18PhonemeReplacementiPc","__Z16LoadVoiceVariantPKci","__ZL23ExtractVoiceVariantNamePci","__Z17SelectVoiceByNamePP12espeak_VOICEPKc","_espeak_ListVoices","__Z11SelectVoiceP12espeak_VOICEPi","__ZL14SetVoiceScoresP12espeak_VOICEPS0_i","__Z14SetVoiceByNamePKc","__Z20SetVoiceByPropertiesP12espeak_VOICE","__ZL9GetVoicesPKc","__ZL15VoiceNameSorterPKvS0_","_espeak_GetCurrentVoice","__ZL13ReadVoiceFileP7__sFILEPKcS2_","__ZL10ScoreVoiceP12espeak_VOICEPKciiS0_","__ZL16VoiceScoreSorterPKvS0_","__Z9WcmdqStopv","__Z9WcmdqFreev","__Z9WcmdqUsedv","__Z8WcmdqIncv","__Z16WavegenOpenSoundv","__Z17WavegenCloseSoundv","__Z16WavegenInitSoundv","__Z11WavegenInitii","__ZL17WavegenInitPkDatai","__Z12GetAmplitudev","__Z16PeaksToHarmspectP15wavegen_peaks_tiPii","__Z10InitBreathv","__ZL12setresonatorP9RESONATORiii","__Z7Wavegenv","__ZL17AdvanceParametersv","__ZL9SetBreathv","__ZL11ApplyBreathv","__Z11SetEmbeddedii","__ZL13SetWithRange0ii","__ZL14WavegenSetEchov","__ZL16SetPitchFormantsv","__Z15WavegenSetVoiceP7voice_t","__Z9SetPitch2P7voice_tiiPiS1_","__Z8SetPitchiPhii","__Z8SetSynthiiP7frame_tS0_P7voice_t","__Z11Write4BytesP7__sFILEi","__Z12WavegenFill2i","__ZL11PlaySilenceii","__ZL8PlayWaveiiPhii","__ZL8Wavegen2iiiP7frame_tS0_","__ZL12SetAmplitudeiPhi","__ZL12WcmdqIncHeadv","__Z11WavegenFilli","__ZL7SpeedUpPsiii","__ZL9resonatorP9RESONATORd","__Z15MakePhonemeListP10Translatorii","__ZL18SubstitutePhonemesP10TranslatorP12PHONEME_LIST","__Z10KlattReseti","__ZL6setabcllP11resonator_t","__Z13Wavegen_Klatti","__ZL10frame_initP13klatt_frame_t","__ZL7parwaveP13klatt_frame_t","__Z14SetSynth_KlattiiP7frame_tS0_P7voice_ti","__Z14Wavegen_Klatt2iiiP7frame_tS0_","__Z9KlattInitv","__ZL7flutterP13klatt_frame_t","__ZL9gen_noised","__ZL16impulsive_sourcev","__ZL14natural_sourcev","__ZL14sampled_sourcev","__ZL21pitch_synch_par_resetP13klatt_frame_t","__ZL9resonatorP11resonator_td","__ZL14antiresonator2P11resonator_td","__ZL10resonator2P11resonator_td","__ZL7DBtoLINl","__ZL10setzeroabcllP11resonator_t","__Z8sonicMSGPcz","_sonicGetSpeed","_sonicSetSpeed","_sonicGetPitch","_sonicSetPitch","_sonicGetVolume","_sonicSetVolume","_sonicGetSampleRate","_sonicGetNumChannels","_sonicDestroyStream","_sonicCreateStream","_sonicReadFloatFromStream","_sonicReadShortFromStream","_sonicReadUnsignedCharFromStream","_sonicFlushStream","_sonicWriteShortToStream","_sonicSamplesAvailable","_sonicWriteFloatToStream","__ZL28addFloatSamplesToInputBufferP17sonicStreamStructPfi","__ZL18processStreamInputP17sonicStreamStruct","__ZL28addShortSamplesToInputBufferP17sonicStreamStructPsi","_sonicWriteUnsignedCharToStream","__ZL35addUnsignedCharSamplesToInputBufferP17sonicStreamStructPhi","_sonicChangeFloatSpeed","_sonicChangeShortSpeed","__ZL26enlargeInputBufferIfNeededP17sonicStreamStructi","__ZL11changeSpeedP17sonicStreamStructf","__ZL12copyToOutputP17sonicStreamStructPsi","__ZL11adjustPitchP17sonicStreamStructi","__ZL12scaleSamplesPsif","__ZL27moveNewSamplesToPitchBufferP17sonicStreamStructi","__ZL15findPitchPeriodP17sonicStreamStructPs","__ZL27enlargeOutputBufferIfNeededP17sonicStreamStructi","__ZL10overlapAddiiPsS_S_","__ZL24overlapAddWithSeparationiiiPsS_S_","__ZL18removePitchSamplesP17sonicStreamStructi","__ZL22findPitchPeriodInRangePsiiPiS0_","__ZL15downSampleInputP17sonicStreamStructPsi","__ZL16prevPeriodBetterP17sonicStreamStructiii","__ZL17copyInputToOutputP17sonicStreamStructi","__ZL15skipPitchPeriodP17sonicStreamStructPsfi","__ZL17insertPitchPeriodP17sonicStreamStructPsfi","__ZL18removeInputSamplesP17sonicStreamStructi","_getopt","__ZL12read_globalsP11getopt_data","__ZL15getopt_internaliPKPcPKcPK6optionPiiP11getopt_data","__ZL13write_globalsP11getopt_data","_getopt_long","_getopt_long_only","___getopt_r","___getopt_long_r","___getopt_long_only_r","__ZL9is_optionPci","__ZL7permutePKPcii","__ZL21reverse_argv_elementsPPci","___strtok_r","_strtok_r","_strtok"]


return {
	Module: Module,
	FS: FS,
	run: run
};
}

/* meScript.js essential changes start here */
  
  var eSpeakDir =     '/espeak',
  	eSpeakDataDir =   '/espeak/espeak-data',
  	eSpeakVoicesDir = '/espeak/espeak-data/voices',
  	eSpeak = new ESpeak(),
  	fileRegistry = [],
  	fsErrorDetected = false,
  	configDataLoaded=false,
  	unloading=false,
  	speakQueue = [],
  	voicesLoaded={},
  	defaultVoice='',
  	AudioAPI=null,
  	canPlay=false,
  	playbackVolume=1,
  	masterGain=null,
  	audioContext=null,
  	audioPool={},
  	webSoundPool={},
  	wsUseTimeout=false,
  	wsStartFromCallback=false;
  

  if (typeof navigator !== 'undefined' && navigator.userAgent) {
    if (navigator.userAgent.indexOf('Chrome')!==-1) {
      if (parseInt(navigator.userAgent.replace(/^.*?\bChrome\/([0-9]+).*$/, '$1'),10)>=32) {
        wsUseTimeout=wsStartFromCallback=true;
      }
    }
    else if (navigator.userAgent.match(/\bVersion\/[0-9]+\.[0-9\.]+ (Mobile\/\w+ )?Safari\//)) {
      if (parseInt(navigator.userAgent.replace(/^.*?\bVersion\/([0-9]+).*$/, '$1'),10)>=9) {
        wsStartFromCallback=true;
      }
    }
  }

  
  function fsCreateDataFile(path, fname, data, doNotRegister) {
	if (!eSpeak.FS.findObject(path)) eSpeak.FS.createPath('/', path.substring(1), true, true);
	eSpeak.FS.createDataFile(path, fname, data, true, false);
	if (!doNotRegister) fileRegistry.push({path: path, fname: fname});
  }
  
  // work around a emscripten FS-bug causing the engine to stop at some iteration (80th call)
  // simply create a new instance and reload any files
  function recoverFromFSError(message) {
    console.log('meSpeak -- recovering from error:', message);
    // save any loaded files from broken instance
    var files = [], f, i, l=fileRegistry.length;
    for (i=0; i<l; i++) {
      f=fileRegistry[i];
      files.push(eSpeak.FS.findObject(f.path+'/'+f.fname).contents);
    }
    // reboot to a new instance and insert any loaded files
    eSpeak = new ESpeak();
    setUpVFS();
    for (i=0; i<l; i++) {
      f=fileRegistry[i];
      fsCreateDataFile(f.path, f.fname, files[i], true);
    }
    fsErrorDetected=true;
  }

  // basic FS setup, for config data see setConfigData(), for voices see setVoiceData()
  function setUpVFS() {
	  var optionFiles = {
	    'croak':'language variant\nname croak\ngender male 70\npitch 85 117\nflutter 20\nformant 0 100 80 110\n',
	    'f1':'language variant\nname female1\ngender female 70\npitch 140 200\nflutter 8\nroughness 4\nformant 0 115 80 150\nformant 1 120 80 180\nformant 2 100 70 150 150\nformant 3 115 70 150\nformant 4 110 80 150\nformant 5 110 90 150\nformant 6 105 80 150\nformant 7 110 70 150\nformant 8 110 70 150\nstressAdd -10 -10 -20 -20 0 0 40 60\n',
	    'f2':'language variant\nname female2\ngender female\npitch 142 220\nroughness 3\nformant 0 105 80 150\nformant 1 110 80 160\nformant 2 110 70 150\nformant 3 110 70 150\nformant 4 115 80 150\nformant 5 115 80 150\nformant 6 110 70 150\nformant 7 110 70 150\nformant 8 110 70 150\nstressAdd 0 0 -10 -10 0 0 10 40\nbreath 0 2 3 3 3 3 3 2\necho 140 10\nconsonants 125 125\n',
	    'f3':'language variant\nname female3\ngender female\npitch 140 240\nformant 0 105 80 150\nformant 1 120 75 150 -50\nformant 2 135 70 150 -250\nformant 3 125 80 150\nformant 4 125 80 150\nformant 5 125 80 150\nformant 6 120 70 150\nformant 7 110 70 150\nformant 8 110 70 150\nstressAmp 18 18 20 20 20 20 20 20\n//breath 0 2 4 4 4 4 4 4\nbreath 0 2 3 3 3 3 3 2\necho 120 10\nroughness 4\n',
	    'f4':'language variant\nname female4\ngender female\necho 130 15\npitch 142 200\nformant 0 120 80 150\nformant 1 115 80 160 -20\nformant 2 130 75 150 -200\nformant 3 123 75 150\nformant 4 125 80 150\nformant 5 125 80 150\nformant 6 110 80 150\nformant 7 110 75 150\nformant 8 110 75 150\nstressAdd -20 -20 -20 -20 0 0 20 120\nstressAmp 18 16 20 20 20 20 20 20\n',
	    'f5':'language variant\nname female5\ngender female\npitch 160 228\nroughness 0\nformant 0 105 80 150\nformant 1 110 80 160\nformant 2 110 70 150\nformant 3 110 70 150\nformant 4 115 80 200\nformant 5 115 80 100\nformant 6 110 70 150\nformant 7 110 70 100\nformant 8 110 70 150\nstressAdd 0 0 -10 -10 0 0 10 40\nbreath 0 4 6   6 6   6 0 10\necho 140 10\nvoicing 75\nconsonants 150 150\nbreathw 150 150 200 200 400 400\n',
	    'klatt':'language variant\nname klatt\nklatt 1\n',
	    'klatt2':'language variant\nname klatt2\nklatt 2\n',
	    'klatt3':'language variant\nname klatt3\nklatt 3\n',
	    'm1':'language variant\nname male1\ngender male 70\npitch 75 109\nflutter 5\nroughness 4\nconsonants 80 100\nformant 0 98 100 100\nformant 1 97 100 100\nformant 2 97 95 100\nformant 3 97 95 100\nformant 4 97 85 100\nformant 5 105 80 100\nformant 6 95 80 100\nformant 7 100 100 100\nformant 8 100 100 100\n//stressAdd -10 -10 -20 -20 0 0 40 70\n',
	    'm2':'language variant\nname male2\ngender male\npitch 88 115\necho 130 15\nformant 0 100 80 120\nformant 1 90 85 120\nformant 2 110 85 120\nformant 3 105 90 120\nformant 4 100 90 120\nformant 5 100 90 120\nformant 6 100 90 120\nformant 7 100 90 120\nformant 8 100 90 120\n',
	    'm3':'language variant\nname male3\ngender male\npitch 80 122\nformant 0 100 100 100\nformant 1 96 97 100\nformant 2 96 97 100\nformant 3 96 103 100\nformant 4 95 103 100\nformant 5 95 103 100\nformant 6 100 100 100\nformant 7 100 100 100\nformant 8 100 100 100\nstressAdd 10 10 0 0 0 0 -30 -30\n',
	    'm4':'language variant\nname male4\ngender male\npitch 70 110\nformant 0 103 100 100\nformant 1 103 100 100\nformant 2 103 100 100\nformant 3 103 100 100\nformant 4 106 100 100\nformant 5 106 100 100\nformant 6 106 100 100\nformant 7 103 100 100\nformant 8 103 100 100\nstressAdd -10 -10 -30 -30 0 0 60 90\n',
	    'm5':'language variant\nname male5\ngender male\nformant 0 100 85 130\nformant 1 90 85 130 40\nformant 2 80 85 130 310\nformant 3 105 85 130\nformant 4 105 85 130\nformant 5 105 85 130\nformant 6 105 85 150\nformant 7 105 85 150\nformant 8 105 85 150\nintonation 2\n',
	    'm6':'language variant\nname male6\ngender male\npitch 82 117\nformant 0 100 90 120\nformant 1 100 90 140\nformant 2 100 70 140\nformant 3 100 75 140\nformant 4 100 80 140\nformant 5 100 80 140\n',
	    'm7':'language variant\nname male7\npitch 75 125\nformant 0 100 125 100\nformant 1 100 90 80\nformant 2 100 70 90\nformant 3 100 60 90\nformant 4 100 60 90\nformant 5 75 50 90\nformant 6 90 50 100\nformant 7 100 50 100\nformant 8 100 50 100\nvoicing 155\n',
	    'whisper':'language variant\nname whisper\npitch 75 125\nformant 0 100 125 100\nformant 1 100 90 80\nformant 2 100 70 90\nformant 3 100 60 90\nformant 4 100 60 90\nformant 5 75 50 90\nformant 6 90 50 100\nformant 7 100 50 100\nformant 8 100 50 100\nvoicing 155\n',
	    'whisperf':'language variant\nname female whisper\ngender female\npitch 160 220\nroughness 3\nformant 0 105 0 150\nformant 1 110 40 160\nformant 2 110 70 150\nformant 3 110 70 150\nformant 4 115 80 150\nformant 5 115 80 150\nformant 6 110 70 150\nformant 7 110 70 150\nformant 8 110 70 150\nstressAdd 0 0 -10 -10 0 0 10 40\n// whisper\nvoicing 20\nbreath 75 75 50 40 15 10\nbreathw 150 150 200 200 400 400\n'
	  };
	  var dir=eSpeakVoicesDir+'/!v';
	  eSpeak.FS.createPath('/', dir.substring(1), true, true);
	  eSpeak.FS.root.write = true;
	  for (var fn in optionFiles) {
	    eSpeak.FS.createDataFile(dir, fn, decodeStringToArray(optionFiles[fn]), true, true);
	  }
  }
  if (eSpeak && eSpeak.FS) setUpVFS();
  
  function setConfigData(data) {
    [
      ['config', decodeBase64ToArray(data.config)],
      ['phontab', decodeBase64ToArray(data.phontab)],
      ['phonindex', decodeBase64ToArray(data.phonindex)],
      ['phondata', decodeBase64ToArray(data.phondata)],
      ['intonations', decodeBase64ToArray(data.intonations)]
    ].forEach(function(pair) {
      var id = pair[0];
      var data = pair[1];
      fsCreateDataFile(eSpeakDataDir, id, data);
    });
    configDataLoaded=true;
    // inspect data for an optional default-voice
    var voice=data.voice;
    if (typeof voice == 'object' && typeof voice.voice_id =='string' && typeof voice.dict_id == 'string' && typeof voice.dict == 'string' && typeof voice.voice == 'string') {
		setVoiceData(voice);
	}
	executeQueuedCalls();
  }

  function setVoiceData(data) {
    var parts, path, fname, f;
    if (!voicesLoaded[data.voice_id]) {
      if (data.dict_id && !eSpeak.FS.findObject(eSpeakDataDir+'/'+data.dict_id)) fsCreateDataFile(eSpeakDataDir, data.dict_id, decodeBase64ToArray(data.dict));
      parts=data.voice_id.split('/');
      path=eSpeakVoicesDir;
      if (parts.length>1) {
        path+='/'+parts.slice(0, parts.length-1).join('/');
      }
      fname=parts[parts.length-1];
      if (data.voice_encoding=='text') {
        fsCreateDataFile(path, fname, decodeStringToArray(data.voice));
      }
      else {
        fsCreateDataFile(path, fname, decodeBase64ToArray(data.voice));
      }
      if (data.files && Array.isArray(data.files)) {
        for (var i=0, l=data.files.length; i<l; i++) {
           f=data.files[i];
           if (typeof f == 'object' && typeof f.path == 'string' && f.path.length && typeof f.data == 'string' && f.data.length) {
             parts=f.path.split('/');
             if (parts.length>1) {
               path=eSpeakDataDir+'/'+parts.slice(0, parts.length-1).join('/');
             }
             else {
               path=eSpeakDataDir;
             }
             fname=parts[parts.length-1];
             if (f.encoding=='text') {
               fsCreateDataFile(path, fname, decodeStringToArray(f.data));
             }
             else {
               fsCreateDataFile(path, fname, decodeBase64ToArray(f.data));
             }
           }
        }
      }
      voicesLoaded[data.voice_id]=true;
    }
    setDefaultVoice(data.voice_id);
    executeQueuedCalls();
  }
  
  function decodeBase64ToArray(str) {
    function f(c) {
      if (c == 43) {
        return 62; // "+"
      }
      else if (c == 47) {
        return 63; // "/"
      }
      else if (c == 61) {
        return 64; // "="
      }
      else if (c <= 57) {
        return c+4; // "0".."9"
      }
      else if (c <= 90) {
        return c-65; // "A".."Z"
      }
      else {
        return c-71; // "a".."z"
      }
    }
    var h1, h2, h3, h4, out=[], i=0, l=str.length;
    while (i < l) {
      h1 = f(str.charCodeAt(i++));
      h2 = f(str.charCodeAt(i++));
      h3 = f(str.charCodeAt(i++));
      h4 = f(str.charCodeAt(i++));
      out.push( (h1 << 2) | (h2 >> 4) );
      if (h3 != 64) out.push( ((h2 & 15) << 4) | (h3 >> 2) );
      if (h4 != 64) out.push( ((h3 & 3) << 6) | h4 );
    }
    return out;
  }

  function encode64(data) {
    var BASE = 'ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+/';
    var PAD = '=';
    var ret = '';
    var leftchar = 0;
    var leftbits = 0;
    for (var i = 0, l = data.length; i < l; i++) {
      leftchar = (leftchar << 8) | data[i];
      leftbits += 8;
      while (leftbits >= 6) {
        var curr = (leftchar >> (leftbits-6)) & 0x3f;
        leftbits -= 6;
        ret += BASE[curr];
      }
    }
    if (leftbits == 2) {
      ret += BASE[(leftchar&3) << 4];
      ret += PAD + PAD;
    } else if (leftbits == 4) {
      ret += BASE[(leftchar&0xf) << 2];
      ret += PAD;
    }
    return ret;
  }

  function decodeStringToArray(str) {
    var out=[], l=str.length, i;
    for (i=0; i<l; i++) out.push(str.charCodeAt(i));
    return out;
  }

  function executeQueuedCalls() {
    // execute any defered calls to speak()
  	if (configDataLoaded && defaultVoice) {
      for (var i=0, l=speakQueue.length; i<l; i++) {
    	var args=speakQueue[i];
    	if (args[3].multipart) {
    	  speakMultipart( args[0], args[1], args[2], args[3] );
    	}
    	else {
    	  speak( args[0], args[1], args[2], args[3] );
    	}
      }
      speakQueue.length=0;
    }
  }
  
  function resetQueue() {
    // public
    speakQueue.length=0;
  }

  function speak(text, args, callback, _id) {
    // public
    // returns:
    // with option "rawdata": stream-object in specified format (or null on failure)
    // default: returns numeric 32bit id (or 0 on failure)
    if (typeof args !== 'object') args = {};
  	if (!canPlay && !args.rawdata) {
  	  if (self.console) console.log('meSpeak: Can\'t play; No audio support.');
  	  return 0;
  	}
  	if (!_id || !isValidJobId(_id)) _id=getJobId();
  	if (!configDataLoaded || !defaultVoice) {
  		if (self.console) console.log(((!configDataLoaded)? 'No config-data loaded': 'No voice module loaded')+', deferring call.');
  		if (args.rawdata) {
  			return null;
  		}
  		else {
  			speakQueue.push( [text, args, callback, _id] );
  			return _id.number;
  		}
  	}
  	if (args && args.voice && !voicesLoaded[args.voice]) {
  		if (self.console) console.log('Voice '+args.voice+' not available. Using default voice: '+defaultVoice);
  		args.voice=defaultVoice;
  	}
  	var varg=(args.voice)? String(args.voice) : (args.v)? String(args.v) : defaultVoice;
  	if (varg.indexOf('mb/mb-')==0) varg=varg.substring(3);
  	if (args.variant) varg+='+'+String(args.variant).replace(/\+/g, '');
  	var argstack = [
      '-w', 'wav.wav',
      '-a', (typeof args.amplitude !== 'undefined')? String(args.amplitude) : (typeof args.a !== 'undefined')? String(args.a) : '100',
      '-g', (typeof args.wordgap !== 'undefined')? String(args.wordgap) : (typeof args.g !== 'undefined')? String(args.g) : '0',
      '-p', (typeof args.pitch !== 'undefined')? String(args.pitch) : (typeof args.p !== 'undefined')? String(args.p) : '50',
      '-s', (typeof args.speed !== 'undefined')? String(args.speed) : (typeof args.s !== 'undefined')? String(args.s) : '175',
      '-b', args.utf16 ? '4' : (typeof args.b !== 'undefined')? String(args.b) : '1',
      '-v', varg
    ];
    var a = args.linebreak || args.l;
    if (a) argstack.push('-l', String(a));
    a = args.capitals || args.k;
    if (a) argstack.push('-k', String(a));
    a = args.nostop || args.z;
    if (a) argstack.push('-z');
    if (typeof args.punct !== 'undefined') {
    	if (typeof args.punct == 'string') {
    		if (args.punct.length) {
    			argstack.push('--punct="' + String(args.punct).replace(/([\\"'])/g, '\\$1')+'"');
    		}
    		else {
    			argstack.push('--punct');
    		}
    	}
    	else if (args.punct) {
    		argstack.push('--punct');
    	}
    }
    a = args.ssml || args.m || args.markup;
    if (a) argstack.push('-m');
    argstack.push('--path='+eSpeakDir, String(text));
    if (args.log && self.console) console.log('executing espeak '+argstack.join(' '));
    eSpeak.Module.arguments = argstack;
    
    if (typeof callback !== 'function') callback=null;
    if (!callback && typeof args.callback == 'function') callback=args.callback;

    fsErrorDetected=false;
    eSpeak.run();
    if (fsErrorDetected) {
      // eSpeak breaks sometimes on flushing streams, approx. every 80th call
      fsErrorDetected=false;
      eSpeak.Module.arguments = argstack;
      eSpeak.run();
    }

    var outfile = eSpeak.FS.root.contents['wav.wav'].contents, wav = unsignedStream(outfile);
    outfile.length=0;
    argstack=null;

	return (args.rawdata)? exportStream(wav, args.rawdata) : playSound(wav, args.volume, callback, _id);
  }

  function exportStream(stream, format) {
    // private
    switch (String(format).toLowerCase()) {
      case 'array':
        return stream;
      case 'base64':
        return encode64(stream);
      case 'data-url':
      case 'data-uri':
      case 'dataurl':
      case 'datauri':
      case 'mime':
        return 'data:audio/x-wav;base64,'+encode64(stream);
      default:
	    var buffer=new ArrayBuffer(stream.length);
	    new Uint8Array(buffer).set(stream);
        return buffer;
    }
  }

  function speakMultipart(parts, args, callback, _id) {
    // public
    // returns:
    // with option "rawdata": stream-object in specified format (or null on failure)
    // default: returns numeric 32bit id (or 0 on failure)
    if (typeof args !== 'object') args = {};
  	if (!canPlay && !args.rawdata) {
  	  if (self.console) console.log('meSpeak: Can\'t play; No audio support.');
  	  return 0;
  	}
  	var failed=false;
  	if (typeof callback !== 'function') callback=null;
  	if (!callback && typeof args.callback == 'function') callback=args.callback;
  	if (Object.prototype.toString.call(parts) != '[object Array]' || !parts.length) {
  	  if (self.console) console.log('meSpeak.speakMultipart: First argument must be an array of objects.');
  	  failed=true;
  	}
  	if (!failed) {
      for (var i=0, l=parts.length; i<l; i++) {
    	if (typeof parts[i] != 'object') {
    	  if (self.console) console.log('meSpeak.speakMultipart: First argument must be an array of objects (part #'+i+').');
    	  failed=true;
    	  break;
    	}
      }
  	}
  	if (failed) {
      if (args.rawdata) {
        return null;
      }
      else {
        if (callback) callback(false);
        return 0;
      }
  	}
  	if (!_id || !isValidJobId(_id)) _id=getJobId();
  	if (!configDataLoaded || !defaultVoice) {
  		if (self.console) console.log(((!configDataLoaded)? 'No config-data loaded': 'No voice module loaded')+', deferring call.');
  		if (args.rawdata) {
  			return null;
  		}
  		else {
  			_id.multipart=true;
  			speakQueue.push( [parts, args, callback, _id] );
  			return _id.number;
  		}
  	}
  	// everything ok, now actually do it
    var wav, sampleLength;
    // loop over the parts
    for (var i=0, l=parts.length; i<l; i++) {
      var n, part = parts[i], opts={};
      for (n in args) opts[n]=args[n];
      for (n in part) opts[n]=part[n];
      opts.rawdata='array';
      opts.callback=null;
      var buffer = meSpeak.speak(part.text, opts);
      if (!buffer) {
        if (args.rawdata) {
          return null;
        }
        else {
          if (callback) callback(false);
          return 0;
        }
      }
      if (i==0) {
        // first: store it and extract the sample length
        wav = buffer;
        sampleLength = readBytes(buffer, 40, 4);
      }
      else {
        // extract the sample length and add it
        sampleLength += readBytes(buffer, 40, 4);
        // strip header and append samples
        wav = wav.concat(buffer.slice(44));
      }
      buffer=null;
    }
    // fix up the sample length (pos 4 and pos 40 in header)
    writeBytes(wav, 4, 4, sampleLength);
    writeBytes(wav, 40, 4, sampleLength);
    // export/play it
	return (args.rawdata)? exportStream(wav, args.rawdata) : playSound(wav, args.volume, callback, _id);
  }

  function writeBytes(f, p, n, value) {
    // private: write bytes to a file, least significant first
    while (n) {
      f[p++]=value & 0xff;
      value = value >> 8;
      n--;
    }
  }
  
  function readBytes(f, p, n) {
    // private: read bytes from a file, least significant first
    var value=0, shft=0;
    while(n) {
      value |= f[p++] << shft;
      shft += 8;
      n--;
    }
    return value;
  }
  
  function unsignedStream(wav) {
    // private
	var i, l= wav.length, v, out=new Array(l);
	for (i = 0; i < l; i++) {
	  v= wav[i];
	  out[i] = (v >= 0)? v : 256 + v;
	}
  	return out;
  }
  
  function getJobId() {
    // private
  	var n, s;
  	while (!n || audioPool[s] || webSoundPool[s]) {
  		n=Math.floor(Math.random()*Math.pow(2,32));
  		s=n.toString(16);
  	}
  	return {
  		number: n,
  		string: s
  	}
  }
  
  function isValidJobId(id) {
    // private
  	return !(Object.prototype.toString.call(id) !== '[object Object]' || Object.prototype.toString.call(id.number) !== '[object Number]' || Object.prototype.toString.call(id.string) !== '[object String]' || !id.number || id.string !== id.number.toString(16));
  }

  function playSound(stream, relVolume, callback, _id) {
    // public (play): play a wav-stream or an ArrayBuffer generated using option 'rawdata'
    // returns numeric 32bit id (or 0 on failure)
    if (callback && (typeof callback !== 'function')) callback=null;
    if (!_id || !isValidJobId(_id)) _id=getJobId();
    var streamType=Object.prototype.toString.call(stream);
    if (!(streamType=='[object Array]' || streamType=='[object ArrayBuffer]' || streamType=='[object String]')) {
      if (self.console) console.log('meSpeak: Can\'t play, not an Array, or ArrayBuffer, or base64-String: '+streamType);
      if (callback) callback(false);
      return 0;
    }
    if (typeof relVolume !== 'undefined') {
      relVolume=parseFloat(relVolume);
      if (isNaN(relVolume) || relVolume<0 || relVolume>1) relVolume=undefined;
    }
	if (AudioAPI) {
      if (streamType=='[object String]') {
        if (stream.indexOf('data:audio/x-wav;base64,', 0)==0) stream=stream.substring(24);
        stream=stream.replace(/=+$/,'');
        if (stream.match(/[^A-Za-z0-9\+\/]/)) {
          if (self.console) console.log('meSpeak: Can\'t play, not a proper base64-String.');
          if (callback) callback(false);
          return 0;
        }
        stream=decodeBase64ToArray(stream);
        if (!stream.length) {
          if (self.console) console.log('meSpeak: Can\'t play, empty sound data.');
          if (callback) callback(false);
          return 0;
        }
        streamType='[object Array]';
      }
	  var buffer;
	  if (streamType=='[object Array]') {
        var buffer= new ArrayBuffer(stream.length);
        new Uint8Array(buffer).set(stream);
      }
      else {
        buffer=stream;
      }
      return playWebSound(buffer, relVolume, callback, _id);
	}
	else if (canPlay) {
	  var isDataUrl=false;
      if (streamType=='[object String]') {
      	if (stream.indexOf('data:audio/x-wav;base64,', 0)<0) {
          if (stream.match(/[^A-Za-z0-9\+\/]/)) {
            if (self.console) console.log('meSpeak: Can\'t play, not a proper base64-String.');
            if (callback) callback(false);
            return 0;
          }
          stream='data:audio/x-wav;base64,'+stream;
      	}
      	isDataUrl=true;
      }
	  else if (streamType=='[object ArrayBuffer]') {
	     stream=new Uint8Array(stream);
	  }
	  return (new AudioPlayback(stream, relVolume, isDataUrl, callback, _id).started)? _id.number:0;
    }
    else {
      if (self.console) console.log('meSpeak: Can\'t play; No audio support.');
      if (callback) callback(false);
      return 0;
    }
  }

  // public meSpeak.stop()
  // takes IDs returned by meSpeak.speak() and/or meSpeak.play as arguments
  // if called whitout an argument, all sounds will be stopped.
  // returns the number of sounds stopped
  function stopSound() {
  	var id, i, k, kl, l, item, n, stopped=0;
  	if (arguments.length>0) {
  	    // stop specific sounds (by numeric 32bit id)
  		for (i=0, l=arguments.length; i<l; i++) {
  			n=parseInt(arguments[i]);
  			if (n && !isNaN(n)) {
  			    id=n.toString(16);
  				if (audioPool[id]) {
  					audioPool[id].stop();
  					stopped++;
  				}
  				else if (webSoundPool[id]) {
  					stopWebSound(webSoundPool[id]);
  					stopped++;
  				}
  				else {
  					for (k=0, kl=speakQueue.length; k<kl; k++) {
  					  item=speakQueue[i];
  					  if (item[3].string===id) {
  					    if (!unloading && typeof item[2]=='function') item[2](false);
  					    speakQueue.splice(k, 1);
  					    stopped++;
  					    break;
  					  }
  					}
  				}
  			}
  		}
  	}
  	else {
  	    // stop all sounds
  		for (id in audioPool) {
  		  audioPool[id].stop();
  		  stopped++;
  		}
  		for (id in webSoundPool) {
  		  stopWebSound(webSoundPool[id]);
  		  stopped++;
  		}
  		for (i=0, l=speakQueue.length; i<l; i++) {
  			item=speakQueue[i];
  			if (!unloading && typeof item[2] == 'function') item[2](false);
  			stopped++;
  		}
  		speakQueue.length=0;
  	}
  	return stopped;
  }
  
  function setVolume(vol) {
    // public, sets master volume (no arguments) or rel volume for provided ids
    var id, i, l, n, v;
    v=parseFloat(vol);
    if (!isNaN(v) && v>=0 && v<=1 && v!=playbackVolume) {
      if (arguments.length==1) playbackVolume=v;
      if (AudioAPI) {
      	if (arguments.length>1) {
      	  for (i=0, l=arguments.length; i<l; i++) {
      	    n=parseInt(arguments[i]);
      	    if (n && !isNaN(n)) {
      	      id=n.toString(16);
      	      if (webSoundPool[id]) setWebSoundVolume(webSoundPool[id], v);
      	    }
      	  }
      	}
        else if (masterGain) {
          masterGain.gain.value=playbackVolume;
        }
      }
      else if (canPlay) {
      	if (arguments.length>1) {
      	  for (i=0, l=arguments.length; i<l; i++) {
      	    n=parseInt(arguments[i]);
      	    if (n && !isNaN(n)) {
      	      id=n.toString(16);
      	      if (audioPool[id]) audioPool[id].setVolume(v);
      	    }
      	  }
      	}
      	else {
          for (id in audioPool) audioPool[id].adjustVolume();
        }
      }
    }
    return vol;
  }

  function getVolume() {
    // public, getter for master volume (no argument) or rel volume for first id provided
    if (arguments.length) {
      var n=parseInt(arguments[0]);
      if (n && !isNaN(n)) {
        var id=n.toString(16);
        if (webSoundPool[id]) return webSoundPool[id].relVolume;
        if (audioPool[id]) return audioPool[id].relVolume;
      }
      return undefined;
    }
    else {
      return playbackVolume;
    }
  }
  
  function resolveAudioApi() {
    if (self.AudioContext) {
      AudioAPI=self.AudioContext;
      canPlay=true;
      return;
    }
    else {
      var vendors=['webkit', 'moz', 'o', 'ms'];
      for (var i=0; i<vendors.length; i++) {
        var api=self[vendors[i]+'AudioContext'];
        if (api) {
          AudioAPI=api;
          canPlay=true;
          return;
        }
      }
    }
    if (!AudioAPI) {
	  var audioElement=document.createElement('audio');
      if (audioElement && audioElement.canPlayType && (audioElement.canPlayType('audio/wav') || audioElement.canPlayType('audio/x-wav'))) {
        canPlay=true;
      }
      else {
        canPlay=false;
        if (self.console) console.log('meSpeak: Muted. No support for HTMLAudioElement with MIME "audio/x-wav" dected.');
      }
    }
  }
  resolveAudioApi();
  
  function canPlaybackWav() {
    // public (canPlay)
    return canPlay;
  }

  // Controller for HTMLAudioElements (collected in pool to provide access to volume of playing instances)
  // (external dependencies: var playbackVolume; input: stream Array (uint8), relVolume: Number 0..1, callback: function)
  // provides external method adjustVolume()
  function AudioPlayback(stream, relVolume, isDataUrl, callback, id) {
    this.relVolume=relVolume;
    this.audio=null;
    this.id=id.string;
    this.callback=(typeof callback == 'function')? callback:null;
    this.playing=false;
    this.started=false;
    this.play(stream, isDataUrl);
  }
  AudioPlayback.prototype= {
    play: function(stream, isDataUrl) {
      try {
        audioPool[this.id]=this;
        this.audio=document.createElement('audio');
        this.adjustVolume();
        var that=this;
        this.audio.addEventListener('ended', function() {that.remove();}, false);
        this.audio.addEventListener('canplaythrough', function() {that.audio.play(); that.playing=true;}, false);
        if (isDataUrl) {
          this.audio.src=stream;
        }
        else {
          this.audio.src='data:audio/x-wav;base64,'+encode64(stream);
        }
        this.audio.load();
        this.started=true;
      }
      catch(e) {
        if (self.console) console.log('meSpeak: HTMLAudioElement Exception: '+e.message);
        this.started=false;
        this.remove();
      }
    },
    adjustVolume: function() {
      this.audio.volume= (typeof this.relVolume!=='undefined')? this.relVolume*playbackVolume:playbackVolume;
    },
    setVolume: function(v) {
      this.relVolume=v;
      this.adjustVolume();
    },
    remove: function(stopped) {
      if (this.id) delete audioPool[this.id];
      if (this.callback) {
		  var f=this.callback;
		  this.callback=null;
		  if (!unloading) f(!stopped);
      }
    },
    stop: function() {
      try {
        if (this.playing) this.audio.pause();
      }
      catch(e) {}
      this.remove(true);
    }
  };

  // play a sound via Web Audio API
  // (external dependencies: vars masterGain, playbackVolume; input: stream ArrayBuffer (Uint8Array), relVolume: Number 0..1, callback: function)
  function playWebSound(stream, volume, callback, id) {
  	try {
      var source, gainNode, timer, relVolume;
      if (!audioContext) audioContext=new AudioAPI();
      if (!masterGain) {
        masterGain= (audioContext.createGain)? audioContext.createGain():audioContext.createGainNode();
        masterGain.connect(audioContext.destination);
        masterGain.gain.value=playbackVolume;
      }
      source= audioContext.createBufferSource();
      relVolume=parseFloat(volume);
      if (typeof relVolume==='undefined' || isNaN(relVolume) || relVolume<0 || relVolume>1) relVolume=1;
      gainNode= (audioContext.createGain)? audioContext.createGain():audioContext.createGainNode();
      gainNode.connect(masterGain);
      gainNode.gain.value=relVolume;
      source.connect(gainNode);
      audioContext.decodeAudioData(stream, function(audioData) {
        var f=function() { webSoundEndHandler(source, gainNode, callback, true, id.string); };
      	if (wsUseTimeout || typeof source.onended === 'undefined') {
			var duration=audioData.duration;
			timer=setTimeout(f, duration? Math.ceil(duration*1000):10);
      	}
      	else {
      		source.onended=f;
        }
        source.buffer=audioData;
        if (wsStartFromCallback) {
          if (source.start) {
            source.start(0);
          }
          else {
            source.noteOn(0);
          }
        }
      },
      function(err) {
        console.log('meSpeak: Web Audio Decoding Error: '+ ((typeof err == 'object')? err.message:err));
        if (timer) clearTimeout(timer);
        if (source) source.disconnect();
        if (gainNode) gainNode.disconnect();
        if (webSoundPool[id.string]) delete webSoundPool[id.string];
        if (!unloading && typeof callback == 'function') callback(false);
        return 0;
      });
      webSoundPool[id.string]={
        'source': source,
        'gainNode': gainNode,
        'callback': callback,
        'id': id.string,
        'relVolume': relVolume,
        'timer': timer
      };
      if (!wsStartFromCallback) {
	    if (source.start) {
	      source.start(0);
	    }
	    else {
	      source.noteOn(0);
        }
      }
      return id.number;
    }
    catch (e) {
      if (self.console) console.log('meSpeak: Web Audio Exception: '+e.message);
      if (timer) { clearTimeout(timer); timer=0; }
      webSoundEndHandler(source, gainNode, callback, false, id.string);
      return 0;
    }
  }
  
  function webSoundEndHandler(source, gainNode, callback, success, id) {
    if (id && webSoundPool[id] && webSoundPool[id].timer) clearTimeout(webSoundPool[id].timer);
    if (!unloading && typeof callback == 'function') callback(success);
    // finnaly clean up with a bit of delay (work around a Chrome duration bug/issue)
    var f=function() {
		if (source) {
		  if (typeof source.onended!=='undefined') source.onended=null;
		  source.disconnect();
		}
		if (gainNode) gainNode.disconnect();
		if (id && webSoundPool[id]) delete webSoundPool[id];
    };
    if (!success) {
    	f();
    }
    else {
      setTimeout(f, 500);
    }
  }
  
  function stopWebSound(obj) {
    // private
    try {
  	  if (obj.source.stop) {
  	    obj.source.stop(0);
  	  }
  	  else {
  	    obj.sourcesource.noteOff(0);
  	  }
  	}
  	catch(e) {}
  	webSoundEndHandler(obj.source, obj.gainNode, obj.callback, false, obj.id);
  }
  
  function setWebSoundVolume(obj, v) {
    // private
    obj.gainNode.gain.value=v;
    obj.relVolume=v;
  }
    
    function setDefaultVoice(voice) {
    	// public
    	if (voice && voicesLoaded[voice]) defaultVoice=voice;
    }
    function getDefaultVoice() {
    	// public
    	return defaultVoice;
    }
    
    function loadVoice(url, callback) {
    	// public
    	if (url) new HttpRequest(url, voiceRequestCallback, callback);
    }
    
    function loadConfig(url) {
    	// public
    	if (url) new HttpRequest(url, configDataRequestCallback, null);
    }
    
    function isVoiceLoaded(voice) {
    	// public
    	return (voicesLoaded[voice])? true:false;
    }
    
    function isConfigLoaded() {
    	// public
    	return (configDataLoaded)? true:false;
    }
    
    function HttpRequest(url, loadCallback, userCallback) {
    	// userCallback is an optional callback-handler.
    	var req=this.request=new XMLHttpRequest();
    	this.handler=loadCallback;
    	this.url=url;
    	this.localmode=Boolean(self.location.href.search(/^file:/i)==0);
    	this.userCallback=userCallback;
    	var objref=this;
    	try {
    		req.open('GET', url);
    		req.onreadystatechange=function() { objref.handler(); };
    		req.send('');
    	}
    	catch(e) {
    		if (self.console) console.log('Failed to load resource from '+url+': Network error.');
    		if (typeof userCallback == 'function') userCallback(false, 'network error');
    		this.request=this.handler=this.userCallback=null;
    	}
    }
    
    function voiceRequestCallback() {
    	var req=this.request;
		if (req.readyState==4) {
			if (this.localmode || req.status==200) {
				var data = JSON.parse(req.responseText);
				if (typeof data == 'object' && typeof data.voice_id =='string' && typeof data.voice == 'string'
					&& (
						((!data.dict_id && !data.dict_id) || (typeof data.dict_id == 'string' && typeof data.dict == 'string'))
						|| (!data.files || (typeof data.files == 'object' && Array.isArray(data.files)))
					)) {
					setVoiceData(data);
					if (typeof this.userCallback == 'function') this.userCallback(true, data.voice_id);
				}
				else {
					if (self.console) console.log('Failed to load voice from '+this.url+': Not a voice module.');
					if (typeof this.userCallback == 'function') this.userCallback(false, 'data error');
				}
			}
			else {
				if (self.console) console.log('Failed to load voice from '+this.url+': Received status '+req.status+'.');
				if (typeof this.userCallback == 'function') this.userCallback(false, 'file error');
			}
			this.request=this.handler=this.userCallback=null;
		}
    }
    
    function configDataRequestCallback() {
    	var req=this.request;
		if (req.readyState==4) {
			if (this.localmode || req.status==200) {
				var data = JSON.parse(req.responseText);
				if (typeof data == 'object' && typeof data.config == 'string' && typeof data.phontab == 'string' && typeof data.phonindex == 'string' && typeof data.phondata == 'string' && typeof data.intonations == 'string') {
					setConfigData(data);
				}
				else {
					if (self.console) console.log('Failed to load config-data from '+this.url+': No valid data.');
				}
			}
			else {
				if (self.console) console.log('Failed to load config-data from '+this.url+': Received status '+req.status+'.');
			}
			this.request=this.handler=this.userCallback=null;
		}
    }
    
    function unloadHandler(event) {
    	unloading=true;
    	stopSound();
    	audioPool={};
    	webSoundPool={};
    }
    
    window.addEventListener('unload', unloadHandler, false);
	
  // public interface
  return {
  	speak: speak,
  	speakMultipart: speakMultipart,
  	loadConfig: loadConfig,
  	loadVoice: loadVoice,
  	setDefaultVoice: setDefaultVoice,
  	getDefaultVoice: getDefaultVoice,
  	setVolume: setVolume,
  	getVolume: getVolume,
  	play: playSound,
  	isConfigLoaded: isConfigLoaded,
  	isVoiceLoaded: isVoiceLoaded,
  	resetQueue: resetQueue,
  	canPlay: canPlaybackWav,
  	stop: stopSound
  }
})();